// Copyright (C) 2024 The Qt Company Ltd.
// Copyright © 2004-2023 Unicode, Inc.
// SPDX-License-Identifier: Unicode-3.0

#ifndef QTIMEZONELOCALE_DATA_P_H
#define QTIMEZONELOCALE_DATA_P_H

//
//  W A R N I N G
//  -------------
//
// This file is not a normal header file.
// It defines data tables for internal use by QTimeZone's L10n code.
// It may change incompatibly between versions, or even be removed.
//
// We mean it.
//

// Only qtimezonelocale.cpp should #include this (after other things it needs),
// and even that only when feature icu is disabled.

QT_REQUIRE_CONFIG(timezone_locale);
#if QT_CONFIG(icu)
#  error "This file should only be needed (or seen) when ICU is not in use"
#endif

QT_BEGIN_NAMESPACE

#ifdef QT_CLDR_ZONE_DEBUG
#  define LOCALE_TAGS(lang, script, land) lang, script, land,
#else
#  define LOCALE_TAGS(lang, script, land)
#endif

namespace QtTimeZoneLocale {

/*
    Locale-specific data for timezone naming.
    https://www.unicode.org/reports/tr35/tr35-68/tr35-dates.html#Time_Zone_Names

    Available data, per locale, from LocaleScanner.timeZoneNames():

        formats: {
            'hour': ('+HH:mm', '-HH:mm'),
            'GMT': 'GMT%0',
            'region': ("%0 Time", std, dst),
            'fallback': "%1 (%0)" }
        zones: { ianaid: {
                'exemplarCity': name,
                'long': (gen, std, dst), 'short': (gen, std, dst) } }
        metazones: { meta: {'long': (gen, std, dst), 'short': (gen, std, dst) } }

    Mapped to C++ data-structures below (defined in QTZL_p.h) and used by
    QTZL.cpp, in conjunction with QTZP_data_p.h's locale-independent data
    conditioned on timezone_locale.
*/

// GENERATED PART STARTS HERE

/*
    This part of the file was generated on 2025-06-17 from the
    Common Locale Data Repository v47

    http://www.unicode.org/cldr/

    Do not edit this section: instead regenerate it using
    cldr2qlocalexml.py and qlocalexml2cpp.py on updated (or
    edited) CLDR data; see qtbase/util/locale_database/.
*/

// Sorted by locale index, then iana name
static inline constexpr LocaleZoneExemplar localeZoneExemplarTable[] = {
    // locInd, ianaInd, xcty{ind, sz}
    {    0,   14,       0,  6, }, // Africa/Asmara C/AnyScript/AnyTerritory
    {    0, 4435,       6,  8, }, // Africa/Sao_Tome
    {    0, 4252,      14,  8, }, // America/Asuncion
    {    0,  237,      22,  8, }, // America/Atikokan
    {    0, 7623,      30, 17, }, // America/Bahia_Banderas
    {    0, 3868,      47,  6, }, // America/Cancun
    {    0, 7774,      53, 13, }, // America/Ciudad_Juarez
    {    0, 5723,      66,  7, }, // America/Curacao
    {    0,  481,      73, 13, }, // America/Indiana/Knox
    {    0, 7961,      86, 16, }, // America/Indiana/Marengo
    {    0, 7985,     102, 19, }, // America/Indiana/Petersburg
    {    0, 8012,     121, 18, }, // America/Indiana/Tell_City
    {    0, 8038,     139, 14, }, // America/Indiana/Vevay
    {    0, 8060,     153, 18, }, // America/Indiana/Vincennes
    {    0, 8086,     171, 16, }, // America/Indiana/Winamac
    {    0, 8125,     187, 20, }, // America/Kentucky/Monticello
    {    0, 5932,     207, 22, }, // America/Lower_Princes
    {    0, 8220,     229,  6, }, // America/Merida
    {    0, 1850,     235, 19, }, // America/Noronha
    {    0, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {    0, 8329,     274, 20, }, // America/North_Dakota/Center
    {    0, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {    0,  393,     317,  4, }, // America/Nuuk
    {    0, 6899,     321, 16, }, // America/Scoresbysund
    {    0, 8508,     337, 14, }, // America/St_Barthelemy
    {    0, 2061,     351, 10, }, // America/St_Johns
    {    0, 5863,     361,  9, }, // America/St_Kitts
    {    0, 5880,     370,  9, }, // America/St_Lucia
    {    0,  855,     379, 10, }, // America/St_Thomas
    {    0, 5913,     389, 11, }, // America/St_Vincent
    {    0, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {    0, 8568,     416, 16, }, // Antarctica/Macquarie
    {    0, 1259,     432, 16, }, // Asia/Ho_Chi_Minh
    {    0, 1185,     448,  9, }, // Asia/Kathmandu
    {    0,  992,     457,  7, }, // Asia/Kolkata
    {    0, 1211,     464,  5, }, // Asia/Macau
    {    0, 8708,     469,  8, }, // Asia/Qostanay
    {    0, 1235,     477,  6, }, // Asia/Yangon
    {    0, 1395,     483,  5, }, // Atlantic/Faroe
    {    0, 6821,     488, 10, }, // Atlantic/St_Helena
    {    0, 1547,     498, 16, }, // Australia/Lord_Howe
    {    0, 2527,     514,  4, }, // Europe/Kyiv
    {    0, 6046,     518, 16, }, // Indian/Christmas
    {    0, 6063,     534, 13, }, // Indian/Cocos
    {    0, 7211,     547,  7, }, // Indian/Reunion
    {    0, 3157,     554,  5, }, // Pacific/Chuuk
    {    0, 2237,     559, 13, }, // Pacific/Easter
    {    0, 4168,     572, 14, }, // Pacific/Norfolk
    {    0, 3096,     586,  7, }, // Pacific/Pohnpei
    {    0, 7392,     593, 11, }, // Pacific/Wake
    {    1, 6788,     604,  6, }, // Africa/Abidjan Abkhazian/Cyrillic/Georgia
    {    1, 6760,     610,  5, }, // Africa/Accra
    {    1, 5059,     615, 11, }, // Africa/Addis_Ababa
    {    1, 7420,     626,  5, }, // Africa/Algiers
    {    1,   14,     631,  6, }, // Africa/Asmara
    {    1,   44,     637,  6, }, // Africa/Bamako
    {    1, 5257,     643,  5, }, // Africa/Bangui
    {    1, 6746,     648,  6, }, // Africa/Banjul
    {    1, 7435,     654,  5, }, // Africa/Bissau
    {    1, 4957,     659,  8, }, // Africa/Blantyre
    {    1, 5287,     667,  9, }, // Africa/Brazzaville
    {    1, 4922,     676,  9, }, // Africa/Bujumbura
    {    1, 2320,     685,  4, }, // Africa/Cairo
    {    1, 4117,     689, 10, }, // Africa/Casablanca
    {    1, 7449,     699,  5, }, // Africa/Ceuta
    {    1, 6773,     704,  7, }, // Africa/Conakry
    {    1, 6840,     711,  5, }, // Africa/Dakar
    {    1, 5130,     716, 12, }, // Africa/Dar_es_Salaam
    {    1, 5043,     728,  6, }, // Africa/Djibouti
    {    1, 5243,     734,  5, }, // Africa/Douala
    {    1, 5166,     739,  7, }, // Africa/El_Aaiun
    {    1, 6853,     746,  7, }, // Africa/Freetown
    {    1, 4906,     753,  8, }, // Africa/Gaborone
    {    1, 5015,     761,  6, }, // Africa/Harare
    {    1, 4479,     767, 12, }, // Africa/Johannesburg
    {    1, 4499,     779,  4, }, // Africa/Juba
    {    1, 5151,     783,  7, }, // Africa/Kampala
    {    1, 4524,     790,  6, }, // Africa/Khartoum
    {    1, 4973,     796,  6, }, // Africa/Kigali
    {    1, 5306,     802,  7, }, // Africa/Kinshasa
    {    1, 4744,     809,  5, }, // Africa/Lagos
    {    1, 5336,     814,  8, }, // Africa/Libreville
    {    1, 6869,     822,  4, }, // Africa/Lome
    {    1, 5211,     826,  6, }, // Africa/Luanda
    {    1, 4939,     832,  9, }, // Africa/Lubumbashi
    {    1, 5001,     841,  6, }, // Africa/Lusaka
    {    1, 5322,     847,  6, }, // Africa/Malabo
    {    1, 4987,     853,  6, }, // Africa/Maputo
    {    1, 5197,     859,  6, }, // Africa/Maseru
    {    1, 5182,     865,  7, }, // Africa/Mbabane
    {    1, 5113,     872,  8, }, // Africa/Mogadishu
    {    1, 7034,     880,  8, }, // Africa/Monrovia
    {    1, 3853,     888,  7, }, // Africa/Nairobi
    {    1, 5271,     895,  7, }, // Africa/Ndjamena
    {    1, 5354,     902,  6, }, // Africa/Niamey
    {    1, 6803,     908,  7, }, // Africa/Nouakchott
    {    1, 6727,     915,  8, }, // Africa/Ouagadougou
    {    1, 5225,     923, 10, }, // Africa/Porto-Novo
    {    1, 4435,     933,  8, }, // Africa/Sao_Tome
    {    1, 2866,     941,  7, }, // Africa/Tripoli
    {    1, 6554,     948,  5, }, // Africa/Tunis
    {    1, 4152,     953,  7, }, // Africa/Windhoek
    {    1,  132,     960,  4, }, // America/Adak
    {    1, 3346,     964,  7, }, // America/Anchorage
    {    1, 5607,     971,  7, }, // America/Anguilla
    {    1, 5624,     978,  7, }, // America/Antigua
    {    1, 4554,     985,  9, }, // America/Araguaina
    {    1,  166,     994, 12, }, // America/Argentina/Buenos_Aires
    {    1,   91,    1006,  9, }, // America/Argentina/Catamarca
    {    1,  270,    1015,  7, }, // America/Argentina/Cordoba
    {    1,  441,    1022,  5, }, // America/Argentina/Jujuy
    {    1, 7462,    1027,  8, }, // America/Argentina/La_Rioja
    {    1,  565,    1035,  7, }, // America/Argentina/Mendoza
    {    1, 7489,    1042, 11, }, // America/Argentina/Rio_Gallegos
    {    1, 7520,    1053,  5, }, // America/Argentina/Salta
    {    1, 7544,    1058,  8, }, // America/Argentina/San_Juan
    {    1, 5580,    1066,  8, }, // America/Argentina/San_Luis
    {    1, 7571,    1074,  7, }, // America/Argentina/Tucuman
    {    1, 7597,    1081,  6, }, // America/Argentina/Ushuaia
    {    1, 5640,    1087,  5, }, // America/Aruba
    {    1, 4252,    1092,  7, }, // America/Asuncion
    {    1,  237,    1099, 12, }, // America/Atikokan
    {    1, 3679,    1111,  4, }, // America/Bahia
    {    1, 7623,    1115, 16, }, // America/Bahia_Banderas
    {    1, 5654,    1131,  8, }, // America/Barbados
    {    1, 7646,    1139,  5, }, // America/Belem
    {    1, 5407,    1144,  5, }, // America/Belize
    {    1, 7660,    1149, 12, }, // America/Blanc-Sablon
    {    1, 7681,    1161,  9, }, // America/Boa_Vista
    {    1, 4361,    1170,  6, }, // America/Bogota
    {    1, 7699,    1176,  5, }, // America/Boise
    {    1, 7713,    1181, 12, }, // America/Cambridge_Bay
    {    1, 7735,    1193, 12, }, // America/Campo_Grande
    {    1, 3868,    1205,  6, }, // America/Cancun
    {    1, 4694,    1211,  7, }, // America/Caracas
    {    1, 4345,    1218,  7, }, // America/Cayenne
    {    1, 5496,    1225, 22, }, // America/Cayman
    {    1, 2260,    1247,  6, }, // America/Chicago
    {    1, 7756,    1253,  6, }, // America/Chihuahua
    {    1, 7774,      53, 13, }, // America/Ciudad_Juarez
    {    1, 5422,    1259, 10, }, // America/Costa_Rica
    {    1, 7796,    1269,  7, }, // America/Creston
    {    1, 3791,    1276,  6, }, // America/Cuiaba
    {    1, 5723,    1282,  8, }, // America/Curacao
    {    1, 7812,    1290, 12, }, // America/Danmarkshavn
    {    1, 7833,    1302,  6, }, // America/Dawson
    {    1, 7848,    1308, 11, }, // America/Dawson_Creek
    {    1,  805,    1319,  6, }, // America/Denver
    {    1, 3465,    1325,  7, }, // America/Detroit
    {    1, 5739,    1332,  8, }, // America/Dominica
    {    1,  893,    1340,  8, }, // America/Edmonton
    {    1, 7869,    1348,  8, }, // America/Eirunepe
    {    1, 5441,    1356,  8, }, // America/El_Salvador
    {    1, 7886,    1364, 11, }, // America/Fort_Nelson
    {    1, 7906,    1375,  9, }, // America/Fortaleza
    {    1, 7924,    1384,  9, }, // America/Glace_Bay
    {    1, 6881,    1393,  7, }, // America/Goose_Bay
    {    1, 4612,    1400, 10, }, // America/Grand_Turk
    {    1, 5770,    1410,  7, }, // America/Grenada
    {    1, 5786,    1417,  9, }, // America/Guadeloupe
    {    1, 3760,    1426,  9, }, // America/Guatemala
    {    1, 4841,    1435,  8, }, // America/Guayaquil
    {    1, 6932,    1443,  6, }, // America/Guyana
    {    1, 1939,    1449,  8, }, // America/Halifax
    {    1, 2281,    1457,  6, }, // America/Havana
    {    1, 7942,    1463,  8, }, // America/Hermosillo
    {    1,  348,    1471, 12, }, // America/Indiana/Indianapolis
    {    1,  481,    1483, 13, }, // America/Indiana/Knox
    {    1, 7961,    1496, 16, }, // America/Indiana/Marengo
    {    1, 7985,    1512, 19, }, // America/Indiana/Petersburg
    {    1, 8012,    1531,  9, }, // America/Indiana/Tell_City
    {    1, 8038,    1540, 14, }, // America/Indiana/Vevay
    {    1, 8060,    1554,  9, }, // America/Indiana/Vincennes
    {    1, 8086,    1563,  7, }, // America/Indiana/Winamac
    {    1, 8110,    1570,  6, }, // America/Inuvik
    {    1,  660,    1576,  7, }, // America/Iqaluit
    {    1, 2799,    1583,  7, }, // America/Jamaica
    {    1, 5380,    1590,  4, }, // America/Juneau
    {    1,  521,    1594,  8, }, // America/Kentucky/Louisville
    {    1, 8125,    1602, 20, }, // America/Kentucky/Monticello
    {    1, 5704,    1622, 10, }, // America/Kralendijk
    {    1, 4376,    1632,  6, }, // America/La_Paz
    {    1, 7169,    1638,  4, }, // America/Lima
    {    1, 3239,    1642, 11, }, // America/Los_Angeles
    {    1, 5932,    1653, 18, }, // America/Lower_Princes
    {    1, 8153,    1671,  6, }, // America/Maceio
    {    1, 8168,    1677,  7, }, // America/Managua
    {    1, 1908,    1684,  6, }, // America/Manaus
    {    1, 5897,    1690,  6, }, // America/Marigot
    {    1, 5805,    1696,  9, }, // America/Martinique
    {    1, 8184,    1705,  9, }, // America/Matamoros
    {    1, 2917,    1714,  8, }, // America/Mazatlan
    {    1, 8202,    1722,  8, }, // America/Menominee
    {    1, 8220,    1730,  6, }, // America/Merida
    {    1, 8235,    1736, 10, }, // America/Metlakatla
    {    1, 2949,    1746,  6, }, // America/Mexico_City
    {    1, 4391,    1752,  7, }, // America/Miquelon
    {    1, 8254,    1759,  7, }, // America/Moncton
    {    1, 8270,    1766,  9, }, // America/Monterrey
    {    1, 4098,    1775, 10, }, // America/Montevideo
    {    1, 5824,    1785, 10, }, // America/Montserrat
    {    1, 5481,    1795,  6, }, // America/Nassau
    {    1, 2379,    1801,  8, }, // America/New_York
    {    1, 8288,    1809,  3, }, // America/Nome
    {    1, 1850,    1812,  7, }, // America/Noronha
    {    1, 8301,    1819, 22, }, // America/North_Dakota/Beulah
    {    1, 8329,    1841, 22, }, // America/North_Dakota/Center
    {    1, 8357,    1863, 27, }, // America/North_Dakota/New_Salem
    {    1,  393,    1890,  4, }, // America/Nuuk
    {    1, 8388,    1894,  7, }, // America/Ojinaga
    {    1, 2356,    1901,  6, }, // America/Panama
    {    1, 6173,    1907, 10, }, // America/Paramaribo
    {    1, 2973,    1917,  6, }, // America/Phoenix
    {    1, 3945,    1923, 12, }, // America/Port-au-Prince
    {    1, 5954,    1935, 13, }, // America/Port_of_Spain
    {    1, 8404,    1948, 11, }, // America/Porto_Velho
    {    1, 5843,    1959, 11, }, // America/Puerto_Rico
    {    1, 4030,    1970, 12, }, // America/Punta_Arenas
    {    1, 8424,    1982, 12, }, // America/Rankin_Inlet
    {    1, 8445,    1994,  6, }, // America/Recife
    {    1, 1995,    2000,  7, }, // America/Regina
    {    1, 8460,    2007,  8, }, // America/Resolute
    {    1,  695,    2015, 10, }, // America/Rio_Branco
    {    1, 8477,    2025,  8, }, // America/Santarem
    {    1, 2201,    2033,  8, }, // America/Santiago
    {    1, 6111,    2041, 13, }, // America/Santo_Domingo
    {    1, 1878,    2054,  9, }, // America/Sao_Paulo
    {    1, 6899,    2063, 11, }, // America/Scoresbysund
    {    1, 8494,    2074,  5, }, // America/Sitka
    {    1, 8508,    2079, 13, }, // America/St_Barthelemy
    {    1, 2061,    2092,  9, }, // America/St_Johns
    {    1, 5863,    2101,  9, }, // America/St_Kitts
    {    1, 5880,    2110, 11, }, // America/St_Lucia
    {    1,  855,    2121, 10, }, // America/St_Thomas
    {    1, 5913,    2131, 12, }, // America/St_Vincent
    {    1, 8530,    2143, 13, }, // America/Swift_Current
    {    1, 5461,    2156, 11, }, // America/Tegucigalpa
    {    1, 5756,    2167,  4, }, // America/Thule
    {    1,  313,    2171,  7, }, // America/Tijuana
    {    1,  608,    2178,  7, }, // America/Toronto
    {    1, 5688,    2185,  7, }, // America/Tortola
    {    1, 2093,    2192,  8, }, // America/Vancouver
    {    1, 2144,    2200,  8, }, // America/Whitehorse
    {    1,  734,    2208,  8, }, // America/Winnipeg
    {    1, 8552,    2216,  7, }, // America/Yakutat
    {    1, 6001,    2223,  5, }, // Antarctica/Casey
    {    1, 6094,    2228,  6, }, // Antarctica/Davis
    {    1, 6133,    2234, 15, }, // Antarctica/DumontDUrville
    {    1, 8568,    2249,  8, }, // Antarctica/Macquarie
    {    1, 7066,    2257,  6, }, // Antarctica/Mawson
    {    1, 7113,    2263,  9, }, // Antarctica/McMurdo
    {    1, 8589,    2272,  6, }, // Antarctica/Palmer
    {    1, 7226,    2278,  6, }, // Antarctica/Rothera
    {    1, 7295,    2284,  5, }, // Antarctica/Syowa
    {    1, 8607,    2289,  5, }, // Antarctica/Troll
    {    1, 7374,    2294, 10, }, // Antarctica/Vostok
    {    1, 1429,    2304,  6, }, // Arctic/Longyearbyen
    {    1, 5570,    2310,  4, }, // Asia/Aden
    {    1, 5395,    2314,  6, }, // Asia/Almaty
    {    1, 3968,    2320,  5, }, // Asia/Amman
    {    1, 5511,    2325,  6, }, // Asia/Anadyr
    {    1, 5523,    2331,  5, }, // Asia/Aqtau
    {    1, 5368,    2336,  6, }, // Asia/Aqtobe
    {    1,  964,    2342,  7, }, // Asia/Ashgabat
    {    1, 8624,    2349,  6, }, // Asia/Atyrau
    {    1, 3607,    2355,  6, }, // Asia/Baghdad
    {    1, 5534,    2361,  7, }, // Asia/Bahrain
    {    1, 3653,    2368,  4, }, // Asia/Baku
    {    1, 4466,    2372,  7, }, // Asia/Bangkok
    {    1, 3571,    2379,  7, }, // Asia/Barnaul
    {    1, 4086,    2386,  6, }, // Asia/Beirut
    {    1, 3778,    2392,  6, }, // Asia/Bishkek
    {    1, 5989,    2398,  6, }, // Asia/Brunei
    {    1, 4601,    2404,  4, }, // Asia/Chita
    {    1, 4511,    2408,  7, }, // Asia/Colombo
    {    1, 4540,    2415,  6, }, // Asia/Damascus
    {    1, 1093,    2421,  5, }, // Asia/Dhaka
    {    1, 6192,    2426,  4, }, // Asia/Dili
    {    1, 3596,    2430,  5, }, // Asia/Dubai
    {    1, 6159,    2435,  7, }, // Asia/Dushanbe
    {    1, 8636,    2442,  9, }, // Asia/Famagusta
    {    1, 8651,    2451,  4, }, // Asia/Gaza
    {    1, 4795,    2455,  6, }, // Asia/Hebron
    {    1, 1259,    2461,  7, }, // Asia/Ho_Chi_Minh
    {    1, 2704,    2468,  7, }, // Asia/Hong_Kong
    {    1, 4771,    2475,  4, }, // Asia/Hovd
    {    1, 4184,    2479,  7, }, // Asia/Irkutsk
    {    1, 7006,    2486,  7, }, // Asia/Jakarta
    {    1, 6992,    2493,  8, }, // Asia/Jayapura
    {    1, 1290,    2501,  9, }, // Asia/Jerusalem
    {    1, 3560,    2510,  5, }, // Asia/Kabul
    {    1, 4316,    2515, 25, }, // Asia/Kamchatka
    {    1, 4239,    2540,  6, }, // Asia/Karachi
    {    1, 1185,    2546,  8, }, // Asia/Kathmandu
    {    1, 8661,    2554,  7, }, // Asia/Khandyga
    {    1,  992,    2561,  8, }, // Asia/Kolkata
    {    1, 4197,    2569, 11, }, // Asia/Krasnoyarsk
    {    1, 4859,    2580, 12, }, // Asia/Kuala_Lumpur
    {    1, 5976,    2592,  6, }, // Asia/Kuching
    {    1, 5547,    2598,  6, }, // Asia/Kuwait
    {    1, 1211,    2604,  5, }, // Asia/Macau
    {    1, 4017,    2609,  7, }, // Asia/Magadan
    {    1, 1349,    2616,  7, }, // Asia/Makassar
    {    1, 7182,    2623,  6, }, // Asia/Manila
    {    1, 6920,    2629,  6, }, // Asia/Muscat
    {    1, 2554,    2635,  7, }, // Asia/Nicosia
    {    1, 8675,    2642, 11, }, // Asia/Novokuznetsk
    {    1, 4135,    2653, 11, }, // Asia/Novosibirsk
    {    1, 4229,    2664,  4, }, // Asia/Omsk
    {    1, 7145,    2668,  6, }, // Asia/Oral
    {    1, 6961,    2674,  7, }, // Asia/Phnom_Penh
    {    1, 8693,    2681,  9, }, // Asia/Pontianak
    {    1, 4214,    2690,  7, }, // Asia/Pyongyang
    {    1, 5559,    2697,  5, }, // Asia/Qatar
    {    1, 8708,    2702,  8, }, // Asia/Qostanay
    {    1, 4269,    2710,  9, }, // Asia/Qyzylorda
    {    1, 3584,    2719,  7, }, // Asia/Riyadh
    {    1, 4408,    2726, 13, }, // Asia/Sakhalin
    {    1, 7245,    2739,  9, }, // Asia/Samarkand
    {    1, 3279,    2748,  4, }, // Asia/Seoul
    {    1, 1053,    2752,  6, }, // Asia/Shanghai
    {    1, 3300,    2758,  8, }, // Asia/Singapore
    {    1, 4297,    2766, 13, }, // Asia/Srednekolymsk
    {    1, 3263,    2779,  6, }, // Asia/Taipei
    {    1, 4781,    2785,  7, }, // Asia/Tashkent
    {    1, 3915,    2792,  4, }, // Asia/Tbilisi
    {    1, 2772,    2796,  7, }, // Asia/Tehran
    {    1, 1317,    2803,  7, }, // Asia/Thimphu
    {    1, 2821,    2810,  5, }, // Asia/Tokyo
    {    1, 4572,    2815,  5, }, // Asia/Tomsk
    {    1, 1021,    2820, 10, }, // Asia/Ulaanbaatar
    {    1, 1159,    2830,  6, }, // Asia/Urumqi
    {    1, 8722,    2836,  8, }, // Asia/Ust-Nera
    {    1, 6977,    2844,  7, }, // Asia/Vientiane
    {    1, 4710,    2851, 11, }, // Asia/Vladivostok
    {    1, 4828,    2862,  7, }, // Asia/Yakutsk
    {    1, 1235,    2869,  6, }, // Asia/Yangon
    {    1, 3883,    2875, 12, }, // Asia/Yekaterinburg
    {    1, 3747,    2887,  6, }, // Asia/Yerevan
    {    1, 3663,    2893, 16, }, // Atlantic/Azores
    {    1, 5671,    2909, 18, }, // Atlantic/Bermuda
    {    1, 6628,    2927, 17, }, // Atlantic/Canary
    {    1, 3727,    2944, 10, }, // Atlantic/Cape_Verde
    {    1, 1395,    2954, 17, }, // Atlantic/Faroe
    {    1, 8736,    2971,  7, }, // Atlantic/Madeira
    {    1, 2748,    2978,  9, }, // Atlantic/Reykjavik
    {    1, 7272,    2987, 16, }, // Atlantic/South_Georgia
    {    1, 6821,    3003, 26, }, // Atlantic/St_Helena
    {    1, 6644,    3029,  6, }, // Atlantic/Stanley
    {    1, 1670,    3035,  8, }, // Australia/Adelaide
    {    1, 1635,    3043,  7, }, // Australia/Brisbane
    {    1, 1799,    3050, 11, }, // Australia/Broken_Hill
    {    1, 1583,    3061,  6, }, // Australia/Darwin
    {    1, 3637,    3067,  5, }, // Australia/Eucla
    {    1, 1516,    3072,  6, }, // Australia/Hobart
    {    1, 8753,    3078,  8, }, // Australia/Lindeman
    {    1, 1547,    3086,  8, }, // Australia/Lord_Howe
    {    1, 1727,    3094,  7, }, // Australia/Melbourne
    {    1, 1762,    3101,  4, }, // Australia/Perth
    {    1, 1463,    3105,  6, }, // Australia/Sydney
    {    1, 6441,    3111,  9, }, // Europe/Amsterdam
    {    1, 6216,    3120,  7, }, // Europe/Andorra
    {    1, 3620,    3127,  8, }, // Europe/Astrakhan
    {    1, 2300,    3135,  4, }, // Europe/Athens
    {    1, 6336,    3139,  7, }, // Europe/Belgrade
    {    1, 4757,    3146,  6, }, // Europe/Berlin
    {    1, 6488,    3152, 10, }, // Europe/Bratislava
    {    1, 2167,    3162,  8, }, // Europe/Brussels
    {    1, 3928,    3170,  8, }, // Europe/Bucharest
    {    1, 3806,    3178,  8, }, // Europe/Budapest
    {    1, 8772,    3186, 23, }, // Europe/Busingen
    {    1, 2583,    3209,  7, }, // Europe/Chisinau
    {    1, 6289,    3216, 10, }, // Europe/Copenhagen
    {    1, 2338,    3226,  6, }, // Europe/Dublin
    {    1, 6307,    3232,  9, }, // Europe/Gibraltar
    {    1, 8788,    3241,  6, }, // Europe/Guernsey
    {    1, 6612,    3247,  8, }, // Europe/Helsinki
    {    1, 8804,    3255,  9, }, // Europe/Isle_of_Man
    {    1, 1130,    3264,  7, }, // Europe/Istanbul
    {    1, 8823,    3271,  5, }, // Europe/Jersey
    {    1, 3979,    3276, 11, }, // Europe/Kaliningrad
    {    1, 8929,    3287,  5, }, // Europe/Kirov
    {    1, 2527,    3292,  4, }, // Europe/Kyiv
    {    1, 3213,    3296,  8, }, // Europe/Lisbon
    {    1, 6506,    3304,  9, }, // Europe/Ljubljana
    {    1, 2501,    3313,  6, }, // Europe/London
    {    1, 6365,    3319, 11, }, // Europe/Luxembourg
    {    1, 4892,    3330,  6, }, // Europe/Madrid
    {    1, 6397,    3336,  5, }, // Europe/Malta
    {    1, 6582,    3341,  9, }, // Europe/Mariehamn
    {    1, 3693,    3350,  5, }, // Europe/Minsk
    {    1, 6410,    3355,  6, }, // Europe/Monaco
    {    1, 3537,    3361,  6, }, // Europe/Moscow
    {    1, 6458,    3367,  4, }, // Europe/Oslo
    {    1, 4284,    3371,  5, }, // Europe/Paris
    {    1, 6424,    3376,  9, }, // Europe/Podgorica
    {    1, 6275,    3385,  5, }, // Europe/Prague
    {    1, 8837,    3390,  4, }, // Europe/Riga
    {    1, 6324,    3394,  3, }, // Europe/Rome
    {    1, 4331,    3397,  6, }, // Europe/Samara
    {    1, 6470,    3403, 10, }, // Europe/San_Marino
    {    1, 6245,    3413,  7, }, // Europe/Sarajevo
    {    1, 4451,    3420,  7, }, // Europe/Saratov
    {    1, 8849,    3427, 10, }, // Europe/Simferopol
    {    1, 6383,    3437,  5, }, // Europe/Skopje
    {    1, 6599,    3442,  5, }, // Europe/Sofia
    {    1, 6523,    3447,  8, }, // Europe/Stockholm
    {    1, 8867,    3455,  6, }, // Europe/Tallinn
    {    1, 6202,    3461,  6, }, // Europe/Tirane
    {    1, 8882,    3467,  9, }, // Europe/Ulyanovsk
    {    1, 6352,    3476,  5, }, // Europe/Vaduz
    {    1, 6567,    3481,  7, }, // Europe/Vatican
    {    1, 6231,    3488,  4, }, // Europe/Vienna
    {    1, 8899,    3492,  7, }, // Europe/Vilnius
    {    1, 4727,    3499,  9, }, // Europe/Volgograd
    {    1, 3190,    3508,  7, }, // Europe/Warsaw
    {    1, 6261,    3515,  6, }, // Europe/Zagreb
    {    1, 6540,    3521,  6, }, // Europe/Zurich
    {    1, 5078,    3527, 12, }, // Indian/Antananarivo
    {    1, 6947,    3539,  5, }, // Indian/Chagos
    {    1, 6046,    3544, 12, }, // Indian/Christmas
    {    1, 6063,    3556, 17, }, // Indian/Cocos
    {    1, 5029,    3573,  5, }, // Indian/Comoro
    {    1, 6661,    3578,  8, }, // Indian/Kerguelen
    {    1, 7260,    3586,  3, }, // Indian/Mahe
    {    1, 7050,    3589,  6, }, // Indian/Maldives
    {    1, 4069,    3595,  7, }, // Indian/Mauritius
    {    1, 5098,    3602,  7, }, // Indian/Mayotte
    {    1, 7211,    3609,  7, }, // Indian/Reunion
    {    1, 4422,    3616,  4, }, // Pacific/Apia
    {    1,  932,    3620,  6, }, // Pacific/Auckland
    {    1, 3706,    3626,  8, }, // Pacific/Bougainville
    {    1, 3015,    3634,  5, }, // Pacific/Chatham
    {    1, 3157,    3639,  4, }, // Pacific/Chuuk
    {    1, 2237,    3643, 11, }, // Pacific/Easter
    {    1, 7360,    3654,  5, }, // Pacific/Efate
    {    1, 7327,    3659,  7, }, // Pacific/Fakaofo
    {    1, 3902,    3666,  4, }, // Pacific/Fiji
    {    1, 7343,    3670,  8, }, // Pacific/Funafuti
    {    1, 6678,    3678, 21, }, // Pacific/Galapagos
    {    1, 6696,    3699, 14, }, // Pacific/Gambier
    {    1, 3822,    3713, 11, }, // Pacific/Guadalcanal
    {    1, 6018,    3724,  4, }, // Pacific/Guam
    {    1, 3049,    3728,  6, }, // Pacific/Kanton
    {    1, 3998,    3734, 10, }, // Pacific/Kiritimati
    {    1, 7019,    3744,  6, }, // Pacific/Kosrae
    {    1, 2842,    3750,  9, }, // Pacific/Kwajalein
    {    1, 4877,    3759,  6, }, // Pacific/Majuro
    {    1, 4051,    3765, 18, }, // Pacific/Marquesas
    {    1, 8914,    3783, 15, }, // Pacific/Midway
    {    1, 7084,    3798,  5, }, // Pacific/Nauru
    {    1, 7132,    3803,  4, }, // Pacific/Niue
    {    1, 4168,    3807,  7, }, // Pacific/Norfolk
    {    1, 7098,    3814,  5, }, // Pacific/Noumea
    {    1, 3126,    3819,  9, }, // Pacific/Pago_Pago
    {    1, 7155,    3828,  5, }, // Pacific/Palau
    {    1, 7194,    3833,  7, }, // Pacific/Pitcairn
    {    1, 3096,    3840,  6, }, // Pacific/Pohnpei
    {    1, 4807,    3846, 11, }, // Pacific/Port_Moresby
    {    1, 6076,    3857,  9, }, // Pacific/Rarotonga
    {    1, 6031,    3866,  6, }, // Pacific/Saipan
    {    1, 7312,    3872,  5, }, // Pacific/Tahiti
    {    1, 6712,    3877,  6, }, // Pacific/Tarawa
    {    1, 4583,    3883,  9, }, // Pacific/Tongatapu
    {    1, 7392,    3892,  4, }, // Pacific/Wake
    {    1, 7405,    3896,  6, }, // Pacific/Wallis
    {    2,   14,       0,  6, }, // Africa/Asmara Afar/Latin/Ethiopia
    {    2, 4435,       6,  8, }, // Africa/Sao_Tome
    {    2, 4252,      14,  8, }, // America/Asuncion
    {    2,  237,      22,  8, }, // America/Atikokan
    {    2, 7623,      30, 17, }, // America/Bahia_Banderas
    {    2, 3868,      47,  6, }, // America/Cancun
    {    2, 7774,      53, 13, }, // America/Ciudad_Juarez
    {    2, 5723,      66,  7, }, // America/Curacao
    {    2,  481,      73, 13, }, // America/Indiana/Knox
    {    2, 7961,      86, 16, }, // America/Indiana/Marengo
    {    2, 7985,     102, 19, }, // America/Indiana/Petersburg
    {    2, 8012,     121, 18, }, // America/Indiana/Tell_City
    {    2, 8038,     139, 14, }, // America/Indiana/Vevay
    {    2, 8060,     153, 18, }, // America/Indiana/Vincennes
    {    2, 8086,     171, 16, }, // America/Indiana/Winamac
    {    2, 8125,     187, 20, }, // America/Kentucky/Monticello
    {    2, 5932,     207, 22, }, // America/Lower_Princes
    {    2, 8220,     229,  6, }, // America/Merida
    {    2, 1850,     235, 19, }, // America/Noronha
    {    2, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {    2, 8329,     274, 20, }, // America/North_Dakota/Center
    {    2, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {    2,  393,     317,  4, }, // America/Nuuk
    {    2, 6899,     321, 16, }, // America/Scoresbysund
    {    2, 8508,     337, 14, }, // America/St_Barthelemy
    {    2, 2061,     351, 10, }, // America/St_Johns
    {    2, 5863,     361,  9, }, // America/St_Kitts
    {    2, 5880,     370,  9, }, // America/St_Lucia
    {    2,  855,     379, 10, }, // America/St_Thomas
    {    2, 5913,     389, 11, }, // America/St_Vincent
    {    2, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {    2, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {    2, 1185,     448,  9, }, // Asia/Kathmandu
    {    2,  992,     457,  7, }, // Asia/Kolkata
    {    2, 1211,     464,  5, }, // Asia/Macau
    {    2, 1235,     477,  6, }, // Asia/Yangon
    {    2, 1395,     483,  5, }, // Atlantic/Faroe
    {    2, 6821,     488, 10, }, // Atlantic/St_Helena
    {    2, 2527,     514,  4, }, // Europe/Kyiv
    {    2, 7211,     547,  7, }, // Indian/Reunion
    {    2, 3157,     554,  5, }, // Pacific/Chuuk
    {    2, 3096,     586,  7, }, // Pacific/Pohnpei
    {    5, 5059,    3902, 11, }, // Africa/Addis_Ababa Afrikaans/Latin/South Africa
    {    5,   14,       0,  6, }, // Africa/Asmara
    {    5, 2320,    3913,  5, }, // Africa/Cairo
    {    5, 5043,    3918,  8, }, // Africa/Djibouti
    {    5, 4524,    3926,  7, }, // Africa/Khartoum
    {    5, 5113,    3933, 10, }, // Africa/Mogadishu
    {    5, 4435,       6,  8, }, // Africa/Sao_Tome
    {    5, 4252,      14,  8, }, // America/Asuncion
    {    5,  237,      22,  8, }, // America/Atikokan
    {    5, 7623,    3943, 14, }, // America/Bahia_Banderas
    {    5, 7713,    3957, 13, }, // America/Cambridge_Bay
    {    5, 3868,      47,  6, }, // America/Cancun
    {    5, 5496,    3970,  7, }, // America/Cayman
    {    5, 7774,      53, 13, }, // America/Ciudad_Juarez
    {    5, 5723,      66,  7, }, // America/Curacao
    {    5, 7924,    3977,  9, }, // America/Glace_Bay
    {    5, 6881,    3986,  9, }, // America/Goose_Bay
    {    5,  481,      73, 13, }, // America/Indiana/Knox
    {    5, 7961,      86, 16, }, // America/Indiana/Marengo
    {    5, 7985,     102, 19, }, // America/Indiana/Petersburg
    {    5, 8012,     121, 18, }, // America/Indiana/Tell_City
    {    5, 8038,     139, 14, }, // America/Indiana/Vevay
    {    5, 8060,     153, 18, }, // America/Indiana/Vincennes
    {    5, 8086,     171, 16, }, // America/Indiana/Winamac
    {    5, 2799,    3995,  7, }, // America/Jamaica
    {    5, 8125,     187, 20, }, // America/Kentucky/Monticello
    {    5, 5932,     207, 22, }, // America/Lower_Princes
    {    5, 8220,     229,  6, }, // America/Merida
    {    5, 2949,    4002, 11, }, // America/Mexico_City
    {    5, 1850,     235, 19, }, // America/Noronha
    {    5, 8301,    4013, 20, }, // America/North_Dakota/Beulah
    {    5, 8329,    4033, 20, }, // America/North_Dakota/Center
    {    5, 8357,    4053, 23, }, // America/North_Dakota/New_Salem
    {    5,  393,     317,  4, }, // America/Nuuk
    {    5, 6899,     321, 16, }, // America/Scoresbysund
    {    5, 8508,    4076, 17, }, // America/St_Barthelemy
    {    5, 2061,    4093, 11, }, // America/St_Johns
    {    5, 5863,    4104, 10, }, // America/St_Kitts
    {    5, 5880,    4114, 10, }, // America/St_Lucia
    {    5,  855,    4124, 11, }, // America/St_Thomas
    {    5, 5913,    4135, 12, }, // America/St_Vincent
    {    5, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {    5, 7374,    4147,  6, }, // Antarctica/Vostok
    {    5,  964,    4153,  9, }, // Asia/Ashgabat
    {    5, 3607,    4162,  6, }, // Asia/Baghdad
    {    5, 5534,    4168,  7, }, // Asia/Bahrain
    {    5, 3653,    4175,  5, }, // Asia/Baku
    {    5, 4086,    4180,  7, }, // Asia/Beirut
    {    5, 3778,    4187,  7, }, // Asia/Bishkek
    {    5, 5989,    4194,  7, }, // Asia/Brunei
    {    5, 4540,    4201,  8, }, // Asia/Damascus
    {    5, 3596,    4209,  6, }, // Asia/Dubai
    {    5, 1259,    4215, 16, }, // Asia/Ho_Chi_Minh
    {    5, 2704,    4231,  8, }, // Asia/Hong_Kong
    {    5, 4184,    4239,  8, }, // Asia/Irkutsk
    {    5, 7006,    4247,  8, }, // Asia/Jakarta
    {    5, 3560,    4255,  6, }, // Asia/Kabul
    {    5, 4316,    4261, 10, }, // Asia/Kamchatka
    {    5, 4239,    4271,  8, }, // Asia/Karachi
    {    5, 1185,    4279,  9, }, // Asia/Kathmandu
    {    5,  992,     457,  7, }, // Asia/Kolkata
    {    5, 4197,    4288, 11, }, // Asia/Krasnoyarsk
    {    5, 4859,    4299, 15, }, // Asia/Kuala_Lumpur
    {    5, 5547,    4314,  7, }, // Asia/Kuwait
    {    5, 1211,    4321,  5, }, // Asia/Macau
    {    5, 6920,    4326,  6, }, // Asia/Muscat
    {    5, 5559,    4332,  5, }, // Asia/Qatar
    {    5, 8708,     469,  8, }, // Asia/Qostanay
    {    5, 3584,    4337,  5, }, // Asia/Riyadh
    {    5, 3279,    4342,  5, }, // Asia/Seoul
    {    5, 3300,    4347,  9, }, // Asia/Singapore
    {    5, 2821,    4356,  5, }, // Asia/Tokyo
    {    5, 4710,    4361, 11, }, // Asia/Vladivostok
    {    5, 4828,    4372,  8, }, // Asia/Yakutsk
    {    5, 1235,     477,  6, }, // Asia/Yangon
    {    5, 3883,    4380, 13, }, // Asia/Yekaterinburg
    {    5, 3663,    4393,  5, }, // Atlantic/Azores
    {    5, 6628,    4398,  7, }, // Atlantic/Canary
    {    5, 3727,    4405, 10, }, // Atlantic/Cape_Verde
    {    5, 1395,     483,  5, }, // Atlantic/Faroe
    {    5, 7272,    4415, 12, }, // Atlantic/South_Georgia
    {    5, 6821,    4427, 11, }, // Atlantic/St_Helena
    {    5, 2300,    4438,  6, }, // Europe/Athens
    {    5, 4757,    4444,  6, }, // Europe/Berlin
    {    5, 2167,    4450,  7, }, // Europe/Brussels
    {    5, 3928,    4457,  9, }, // Europe/Bucharest
    {    5, 3806,    4466,  9, }, // Europe/Budapest
    {    5, 6289,    4475, 10, }, // Europe/Copenhagen
    {    5, 8804,    4485, 10, }, // Europe/Isle_of_Man
    {    5, 2527,    4495,  4, }, // Europe/Kyiv
    {    5, 3213,    4499,  8, }, // Europe/Lisbon
    {    5, 2501,    4507,  6, }, // Europe/London
    {    5, 6365,    4513,  9, }, // Europe/Luxembourg
    {    5, 3537,    4522,  6, }, // Europe/Moscow
    {    5, 4284,    4528,  5, }, // Europe/Paris
    {    5, 6275,    4533,  5, }, // Europe/Prague
    {    5, 4451,    4538,  7, }, // Europe/Saratov
    {    5, 6567,    4545, 12, }, // Europe/Vatican
    {    5, 6231,    4557,  4, }, // Europe/Vienna
    {    5, 4727,    4561,  9, }, // Europe/Volgograd
    {    5, 3190,    4570,  7, }, // Europe/Warsaw
    {    5, 6540,    4577,  6, }, // Europe/Zurich
    {    5, 6063,    4583,  5, }, // Indian/Cocos
    {    5, 5029,    4588,  6, }, // Indian/Comoro
    {    5, 7050,    4594,  8, }, // Indian/Maldives
    {    5, 7211,     547,  7, }, // Indian/Reunion
    {    5, 3157,     554,  5, }, // Pacific/Chuuk
    {    5, 2237,    4602,  4, }, // Pacific/Easter
    {    5, 3902,    4606,  5, }, // Pacific/Fiji
    {    5, 2723,    4611,  8, }, // Pacific/Honolulu
    {    5, 3049,    4619,  9, }, // Pacific/Kanton
    {    5, 7098,    4628,  6, }, // Pacific/Noumea
    {    5, 3096,     586,  7, }, // Pacific/Pohnpei
    {    5, 7405,    4634,  8, }, // Pacific/Wallis
    {    7,   14,       0,  6, }, // Africa/Asmara Aghem/Latin/Cameroon
    {    7, 4435,       6,  8, }, // Africa/Sao_Tome
    {    7, 4252,      14,  8, }, // America/Asuncion
    {    7,  237,      22,  8, }, // America/Atikokan
    {    7, 7623,      30, 17, }, // America/Bahia_Banderas
    {    7, 3868,      47,  6, }, // America/Cancun
    {    7, 7774,      53, 13, }, // America/Ciudad_Juarez
    {    7, 5723,      66,  7, }, // America/Curacao
    {    7,  481,      73, 13, }, // America/Indiana/Knox
    {    7, 7961,      86, 16, }, // America/Indiana/Marengo
    {    7, 7985,     102, 19, }, // America/Indiana/Petersburg
    {    7, 8012,     121, 18, }, // America/Indiana/Tell_City
    {    7, 8038,     139, 14, }, // America/Indiana/Vevay
    {    7, 8060,     153, 18, }, // America/Indiana/Vincennes
    {    7, 8086,     171, 16, }, // America/Indiana/Winamac
    {    7, 8125,     187, 20, }, // America/Kentucky/Monticello
    {    7, 5932,     207, 22, }, // America/Lower_Princes
    {    7, 8220,     229,  6, }, // America/Merida
    {    7, 1850,     235, 19, }, // America/Noronha
    {    7, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {    7, 8329,     274, 20, }, // America/North_Dakota/Center
    {    7, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {    7,  393,     317,  4, }, // America/Nuuk
    {    7, 6899,     321, 16, }, // America/Scoresbysund
    {    7, 8508,     337, 14, }, // America/St_Barthelemy
    {    7, 2061,     351, 10, }, // America/St_Johns
    {    7, 5863,     361,  9, }, // America/St_Kitts
    {    7, 5880,     370,  9, }, // America/St_Lucia
    {    7,  855,     379, 10, }, // America/St_Thomas
    {    7, 5913,     389, 11, }, // America/St_Vincent
    {    7, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {    7, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {    7, 1185,     448,  9, }, // Asia/Kathmandu
    {    7,  992,     457,  7, }, // Asia/Kolkata
    {    7, 1211,     464,  5, }, // Asia/Macau
    {    7, 1235,     477,  6, }, // Asia/Yangon
    {    7, 1395,     483,  5, }, // Atlantic/Faroe
    {    7, 6821,     488, 10, }, // Atlantic/St_Helena
    {    7, 2527,     514,  4, }, // Europe/Kyiv
    {    7, 7211,     547,  7, }, // Indian/Reunion
    {    7, 3157,     554,  5, }, // Pacific/Chuuk
    {    7, 3096,     586,  7, }, // Pacific/Pohnpei
    {    8, 6788,    4642,  7, }, // Africa/Abidjan Akan/Latin/Ghana
    {    8, 6760,    4649,  5, }, // Africa/Accra
    {    8, 7420,    4654,  7, }, // Africa/Algiers
    {    8,   14,       0,  6, }, // Africa/Asmara
    {    8, 6746,    4661,  7, }, // Africa/Banjul
    {    8, 7435,    4668,  5, }, // Africa/Bissau
    {    8, 4957,    4673,  7, }, // Africa/Blantyre
    {    8, 4922,    4680, 10, }, // Africa/Bujumbura
    {    8, 2320,    4690,  5, }, // Africa/Cairo
    {    8, 4117,    4695, 10, }, // Africa/Casablanca
    {    8, 7449,    4705,  5, }, // Africa/Ceuta
    {    8, 6773,    4710,  7, }, // Africa/Conakry
    {    8, 6840,    4717,  5, }, // Africa/Dakar
    {    8, 5043,    4722,  8, }, // Africa/Djibouti
    {    8, 4499,    4730,  5, }, // Africa/Juba
    {    8, 5306,    4735, 10, }, // Africa/Kinshasa
    {    8, 4744,    4745,  5, }, // Africa/Lagos
    {    8, 5271,    4750,  8, }, // Africa/Ndjamena
    {    8, 6727,    4758,  8, }, // Africa/Ouagadougou
    {    8, 4435,       6,  8, }, // Africa/Sao_Tome
    {    8, 3346,    4766,  9, }, // America/Anchorage
    {    8, 5607,    4775,  7, }, // America/Anguilla
    {    8,   91,    4782,  9, }, // America/Argentina/Catamarca
    {    8,  270,    4791,  7, }, // America/Argentina/Cordoba
    {    8,  441,    4798,  7, }, // America/Argentina/Jujuy
    {    8, 7462,    4805,  9, }, // America/Argentina/La_Rioja
    {    8,  565,    4814,  7, }, // America/Argentina/Mendoza
    {    8, 7489,    4821, 12, }, // America/Argentina/Rio_Gallegos
    {    8, 7544,    4833,  9, }, // America/Argentina/San_Juan
    {    8, 7571,    4842,  7, }, // America/Argentina/Tucuman
    {    8, 4252,      14,  8, }, // America/Asuncion
    {    8,  237,      22,  8, }, // America/Atikokan
    {    8, 7623,      30, 17, }, // America/Bahia_Banderas
    {    8, 5654,    4849,  8, }, // America/Barbados
    {    8, 7646,    4857,  5, }, // America/Belem
    {    8, 5407,    4862,  6, }, // America/Belize
    {    8, 7660,    4868, 12, }, // America/Blanc-Sablon
    {    8, 7699,    4880,  5, }, // America/Boise
    {    8, 7713,    4885, 13, }, // America/Cambridge_Bay
    {    8, 7735,    4898, 12, }, // America/Campo_Grande
    {    8, 3868,      47,  6, }, // America/Cancun
    {    8, 4694,    4910,  7, }, // America/Caracas
    {    8, 4345,    4917,  7, }, // America/Cayenne
    {    8, 5496,    4924,  7, }, // America/Cayman
    {    8, 2260,    4931,  7, }, // America/Chicago
    {    8, 7756,    4938,  9, }, // America/Chihuahua
    {    8, 7774,      53, 13, }, // America/Ciudad_Juarez
    {    8, 5422,    4947, 10, }, // America/Costa_Rica
    {    8, 7796,    4957,  7, }, // America/Creston
    {    8, 3791,    4964,  6, }, // America/Cuiaba
    {    8, 5723,    4970,  7, }, // America/Curacao
    {    8, 7833,    4977,  6, }, // America/Dawson
    {    8, 7848,    4983, 12, }, // America/Dawson_Creek
    {    8,  805,    4995,  5, }, // America/Denver
    {    8, 3465,    5000,  7, }, // America/Detroit
    {    8, 5739,    5007,  8, }, // America/Dominica
    {    8,  893,    5015,  8, }, // America/Edmonton
    {    8, 5441,    5023, 11, }, // America/El_Salvador
    {    8, 7886,    5034, 10, }, // America/Fort_Nelson
    {    8, 7906,    5044,  9, }, // America/Fortaleza
    {    8, 6881,    5053,  8, }, // America/Goose_Bay
    {    8, 4612,    5061,  9, }, // America/Grand_Turk
    {    8, 5786,    5070,  9, }, // America/Guadeloupe
    {    8, 3760,    5079, 10, }, // America/Guatemala
    {    8, 4841,    5089,  9, }, // America/Guayaquil
    {    8, 6932,    5098,  6, }, // America/Guyana
    {    8, 7942,    5104,  8, }, // America/Hermosillo
    {    8,  348,    5112, 12, }, // America/Indiana/Indianapolis
    {    8,  481,      73, 13, }, // America/Indiana/Knox
    {    8, 7961,      86, 16, }, // America/Indiana/Marengo
    {    8, 7985,    5124, 19, }, // America/Indiana/Petersburg
    {    8, 8012,    5143, 18, }, // America/Indiana/Tell_City
    {    8, 8038,     139, 14, }, // America/Indiana/Vevay
    {    8, 8060,     153, 18, }, // America/Indiana/Vincennes
    {    8, 8086,    5161, 16, }, // America/Indiana/Winamac
    {    8,  660,    5177,  7, }, // America/Iqaluit
    {    8, 2799,    5184,  7, }, // America/Jamaica
    {    8, 8125,    5191, 18, }, // America/Kentucky/Monticello
    {    8, 5704,    5209, 10, }, // America/Kralendijk
    {    8, 3239,    5219, 12, }, // America/Los_Angeles
    {    8, 5932,    5231, 16, }, // America/Lower_Princes
    {    8, 8153,    5247,  6, }, // America/Maceio
    {    8, 5897,    5253,  7, }, // America/Marigot
    {    8, 5805,    5260,  9, }, // America/Martinique
    {    8, 8184,    5269, 10, }, // America/Matamoros
    {    8, 2917,    5279,  8, }, // America/Mazatlan
    {    8, 8202,    5287,  9, }, // America/Menominee
    {    8, 8220,     229,  6, }, // America/Merida
    {    8, 8235,    5296, 10, }, // America/Metlakatla
    {    8, 2949,    5306, 12, }, // America/Mexico_City
    {    8, 8254,    5318,  7, }, // America/Moncton
    {    8, 8270,    5325,  8, }, // America/Monterrey
    {    8, 5824,    5333,  9, }, // America/Montserrat
    {    8, 2379,    5342,  7, }, // America/New_York
    {    8, 1850,     235, 19, }, // America/Noronha
    {    8, 8301,    5349, 17, }, // America/North_Dakota/Beulah
    {    8, 8329,    5366, 17, }, // America/North_Dakota/Center
    {    8, 8357,    5383, 21, }, // America/North_Dakota/New_Salem
    {    8,  393,     317,  4, }, // America/Nuuk
    {    8, 8388,    5404,  8, }, // America/Ojinaga
    {    8, 6173,    5412, 10, }, // America/Paramaribo
    {    8, 2973,    5422,  6, }, // America/Phoenix
    {    8, 5954,    5428, 11, }, // America/Port_of_Spain
    {    8, 8404,    5439, 11, }, // America/Porto_Velho
    {    8, 5843,    5450, 10, }, // America/Puerto_Rico
    {    8, 8445,    5460,  6, }, // America/Recife
    {    8, 1995,    5466,  7, }, // America/Regina
    {    8, 8460,    5473,  7, }, // America/Resolute
    {    8,  695,    5480, 10, }, // America/Rio_Branco
    {    8, 6899,    5490, 13, }, // America/Scoresbysund
    {    8, 8508,    5503, 13, }, // America/St_Barthelemy
    {    8, 2061,     351, 10, }, // America/St_Johns
    {    8, 5863,     361,  9, }, // America/St_Kitts
    {    8, 5880,     370,  9, }, // America/St_Lucia
    {    8,  855,     379, 10, }, // America/St_Thomas
    {    8, 5913,     389, 11, }, // America/St_Vincent
    {    8, 8530,    5516, 12, }, // America/Swift_Current
    {    8, 5461,    5528, 11, }, // America/Tegucigalpa
    {    8,  313,    5539,  8, }, // America/Tijuana
    {    8, 5688,    5547,  7, }, // America/Tortola
    {    8,  734,    5554,  7, }, // America/Winnipeg
    {    8, 6001,    5561,  4, }, // Antarctica/Casey
    {    8, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {    8, 8568,    5565,  7, }, // Antarctica/Macquarie
    {    8, 7066,    5572,  6, }, // Antarctica/Mawson
    {    8, 8589,    5578,  5, }, // Antarctica/Palmer
    {    8, 7226,    5583,  6, }, // Antarctica/Rothera
    {    8, 8607,    5589,  4, }, // Antarctica/Troll
    {    8, 7374,    5593,  6, }, // Antarctica/Vostok
    {    8, 5395,    5599,  6, }, // Asia/Almaty
    {    8, 3968,    5605,  4, }, // Asia/Amman
    {    8, 5523,    5609,  5, }, // Asia/Aqtau
    {    8, 5368,    5614,  6, }, // Asia/Aqtobe
    {    8, 4466,    5620,  6, }, // Asia/Bangkok
    {    8, 4086,    5626,  6, }, // Asia/Beirut
    {    8, 4601,    5632,  5, }, // Asia/Chita
    {    8, 4511,    5637,  7, }, // Asia/Colombo
    {    8, 4540,    5644,  9, }, // Asia/Damascus
    {    8, 1093,    4717,  4, }, // Asia/Dhaka
    {    8, 4795,    5653,  6, }, // Asia/Hebron
    {    8, 1259,    5659, 10, }, // Asia/Ho_Chi_Minh
    {    8, 7006,    5669,  8, }, // Asia/Jakarta
    {    8, 6992,    5677,  9, }, // Asia/Jayapura
    {    8, 1290,    5686,  9, }, // Asia/Jerusalem
    {    8, 4316,    5695,  9, }, // Asia/Kamchatka
    {    8, 4239,    5704,  7, }, // Asia/Karachi
    {    8, 1185,    5711,  8, }, // Asia/Kathmandu
    {    8,  992,    5719,  7, }, // Asia/Kolkata
    {    8, 5976,    5726,  6, }, // Asia/Kuching
    {    8, 1211,     464,  5, }, // Asia/Macau
    {    8, 1349,    5732,  6, }, // Asia/Makassar
    {    8, 6920,    4326,  6, }, // Asia/Muscat
    {    8, 2554,    5738,  7, }, // Asia/Nicosia
    {    8, 5559,    4332,  4, }, // Asia/Qatar
    {    8, 8708,     469,  8, }, // Asia/Qostanay
    {    8, 3300,    5745,  7, }, // Asia/Singapore
    {    8, 2772,    5752,  6, }, // Asia/Tehran
    {    8, 1317,    5758,  6, }, // Asia/Thimphu
    {    8, 1021,    5764,  9, }, // Asia/Ulaanbaatar
    {    8, 1159,    5773,  7, }, // Asia/Urumqi
    {    8, 6977,    5780, 10, }, // Asia/Vientiane
    {    8, 1235,     477,  6, }, // Asia/Yangon
    {    8, 3883,    5790, 12, }, // Asia/Yekaterinburg
    {    8, 5671,    5802,  6, }, // Atlantic/Bermuda
    {    8, 6628,    4398,  6, }, // Atlantic/Canary
    {    8, 3727,    5808,  7, }, // Atlantic/Cape_Verde
    {    8, 1395,     483,  5, }, // Atlantic/Faroe
    {    8, 2748,    5815, 10, }, // Atlantic/Reykjavik
    {    8, 7272,    5825, 16, }, // Atlantic/South_Georgia
    {    8, 6821,     488, 10, }, // Atlantic/St_Helena
    {    8, 6644,    5841,  6, }, // Atlantic/Stanley
    {    8, 1799,    5847, 11, }, // Australia/Broken_Hill
    {    8, 1583,    5858,  6, }, // Australia/Darwin
    {    8, 3637,    5864,  5, }, // Australia/Eucla
    {    8, 1516,    5869,  5, }, // Australia/Hobart
    {    8, 1547,    5874,  8, }, // Australia/Lord_Howe
    {    8, 1727,    5882,  6, }, // Australia/Melbourne
    {    8, 1762,    5888,  4, }, // Australia/Perth
    {    8, 1463,    5892,  5, }, // Australia/Sydney
    {    8, 6441,    5897,  8, }, // Europe/Amsterdam
    {    8, 6216,    5905,  6, }, // Europe/Andorra
    {    8, 2300,    5911,  5, }, // Europe/Athens
    {    8, 6336,    5916,  8, }, // Europe/Belgrade
    {    8, 4757,    5924,  6, }, // Europe/Berlin
    {    8, 2167,    5930,  8, }, // Europe/Brussels
    {    8, 3928,    5938,  9, }, // Europe/Bucharest
    {    8, 3806,    5947,  8, }, // Europe/Budapest
    {    8, 8772,    5955,  8, }, // Europe/Busingen
    {    8, 2583,    5963,  8, }, // Europe/Chisinau
    {    8, 6289,    5971, 11, }, // Europe/Copenhagen
    {    8, 2338,    5982,  6, }, // Europe/Dublin
    {    8, 6307,    5988,  8, }, // Europe/Gibraltar
    {    8, 6612,    5996,  8, }, // Europe/Helsinki
    {    8, 8823,    6004,  5, }, // Europe/Jersey
    {    8, 2527,     514,  4, }, // Europe/Kyiv
    {    8, 3213,    6009,  6, }, // Europe/Lisbon
    {    8, 6506,    6015, 10, }, // Europe/Ljubljana
    {    8, 2501,    6025,  6, }, // Europe/London
    {    8, 6365,    6031,  8, }, // Europe/Luxembourg
    {    8, 6397,    6039,  5, }, // Europe/Malta
    {    8, 6410,    6044,  6, }, // Europe/Monaco
    {    8, 3537,    6050,  5, }, // Europe/Moscow
    {    8, 6424,    6055,  9, }, // Europe/Podgorica
    {    8, 6324,    6064,  4, }, // Europe/Rome
    {    8, 6245,    6068,  9, }, // Europe/Sarajevo
    {    8, 6383,    6077,  7, }, // Europe/Skopje
    {    8, 6599,    6084,  5, }, // Europe/Sofia
    {    8, 6523,    6089,  7, }, // Europe/Stockholm
    {    8, 6567,    6096,  7, }, // Europe/Vatican
    {    8, 6231,    6103,  5, }, // Europe/Vienna
    {    8, 3190,    6108,  6, }, // Europe/Warsaw
    {    8, 6540,    6114,  7, }, // Europe/Zurich
    {    8, 6947,    6121,  6, }, // Indian/Chagos
    {    8, 6046,    6127,  7, }, // Indian/Christmas
    {    8, 6063,    6134,  6, }, // Indian/Cocos
    {    8, 5029,    6140,  6, }, // Indian/Comoro
    {    8, 6661,    6146,  8, }, // Indian/Kerguelen
    {    8, 4069,    6154, 10, }, // Indian/Mauritius
    {    8, 5098,    6164,  6, }, // Indian/Mayotte
    {    8, 7211,     547,  7, }, // Indian/Reunion
    {    8,  932,    6170,  7, }, // Pacific/Auckland
    {    8, 3015,    6177,  6, }, // Pacific/Chatham
    {    8, 3157,    6183,  5, }, // Pacific/Chuuk
    {    8, 2237,    6188,  5, }, // Pacific/Easter
    {    8, 3902,    6193,  5, }, // Pacific/Fiji
    {    8, 6678,    6198,  9, }, // Pacific/Galapagos
    {    8, 3822,    6207, 11, }, // Pacific/Guadalcanal
    {    8, 2842,    6218, 11, }, // Pacific/Kwajalein
    {    8, 4877,    6229,  7, }, // Pacific/Majuro
    {    8, 4051,    6236,  9, }, // Pacific/Marquesas
    {    8, 4168,    6245,  6, }, // Pacific/Norfolk
    {    8, 7194,    6251,  9, }, // Pacific/Pitcairn
    {    8, 3096,    6260,  6, }, // Pacific/Pohnpei
    {    8, 4807,    6266, 11, }, // Pacific/Port_Moresby
    {    9,   14,       0,  6, }, // Africa/Asmara Akoose/Latin/Cameroon
    {    9, 4435,       6,  8, }, // Africa/Sao_Tome
    {    9, 4252,      14,  8, }, // America/Asuncion
    {    9,  237,      22,  8, }, // America/Atikokan
    {    9, 7623,      30, 17, }, // America/Bahia_Banderas
    {    9, 3868,      47,  6, }, // America/Cancun
    {    9, 7774,      53, 13, }, // America/Ciudad_Juarez
    {    9, 5723,      66,  7, }, // America/Curacao
    {    9,  481,      73, 13, }, // America/Indiana/Knox
    {    9, 7961,      86, 16, }, // America/Indiana/Marengo
    {    9, 7985,     102, 19, }, // America/Indiana/Petersburg
    {    9, 8012,     121, 18, }, // America/Indiana/Tell_City
    {    9, 8038,     139, 14, }, // America/Indiana/Vevay
    {    9, 8060,     153, 18, }, // America/Indiana/Vincennes
    {    9, 8086,     171, 16, }, // America/Indiana/Winamac
    {    9, 8125,     187, 20, }, // America/Kentucky/Monticello
    {    9, 5932,     207, 22, }, // America/Lower_Princes
    {    9, 8220,     229,  6, }, // America/Merida
    {    9, 1850,     235, 19, }, // America/Noronha
    {    9, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {    9, 8329,     274, 20, }, // America/North_Dakota/Center
    {    9, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {    9,  393,     317,  4, }, // America/Nuuk
    {    9, 6899,     321, 16, }, // America/Scoresbysund
    {    9, 8508,     337, 14, }, // America/St_Barthelemy
    {    9, 2061,     351, 10, }, // America/St_Johns
    {    9, 5863,     361,  9, }, // America/St_Kitts
    {    9, 5880,     370,  9, }, // America/St_Lucia
    {    9,  855,     379, 10, }, // America/St_Thomas
    {    9, 5913,     389, 11, }, // America/St_Vincent
    {    9, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {    9, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {    9, 1185,     448,  9, }, // Asia/Kathmandu
    {    9,  992,     457,  7, }, // Asia/Kolkata
    {    9, 1211,     464,  5, }, // Asia/Macau
    {    9, 1235,     477,  6, }, // Asia/Yangon
    {    9, 1395,     483,  5, }, // Atlantic/Faroe
    {    9, 6821,     488, 10, }, // Atlantic/St_Helena
    {    9, 2527,     514,  4, }, // Europe/Kyiv
    {    9, 7211,     547,  7, }, // Indian/Reunion
    {    9, 3157,     554,  5, }, // Pacific/Chuuk
    {    9, 3096,     586,  7, }, // Pacific/Pohnpei
    {   10, 6788,    6277,  7, }, // Africa/Abidjan Albanian/Latin/Albania
    {   10, 6760,    4649,  4, }, // Africa/Accra
    {   10, 5059,    6284, 10, }, // Africa/Addis_Ababa
    {   10, 7420,    6294,  6, }, // Africa/Algiers
    {   10,   14,    6300,  6, }, // Africa/Asmara
    {   10, 6746,    6306,  7, }, // Africa/Banjul
    {   10, 7435,    6313,  5, }, // Africa/Bissau
    {   10, 4957,    6318,  8, }, // Africa/Blantyre
    {   10, 5287,    6326, 10, }, // Africa/Brazzaville
    {   10, 4922,    6336, 10, }, // Africa/Bujumbura
    {   10, 2320,    6346,  5, }, // Africa/Cairo
    {   10, 4117,    6351, 10, }, // Africa/Casablanca
    {   10, 7449,    6361,  6, }, // Africa/Ceuta
    {   10, 6773,    6367,  7, }, // Africa/Conakry
    {   10, 5130,    6374, 12, }, // Africa/Dar_es_Salaam
    {   10, 5043,    6386,  7, }, // Africa/Djibouti
    {   10, 5166,    6393,  7, }, // Africa/El_Aaiun
    {   10, 6853,    6400,  7, }, // Africa/Freetown
    {   10, 4479,    6407, 11, }, // Africa/Johannesburg
    {   10, 4499,    6418,  5, }, // Africa/Juba
    {   10, 4524,    6423,  6, }, // Africa/Khartoum
    {   10, 5336,    6429,  9, }, // Africa/Libreville
    {   10, 5354,    6438,  6, }, // Africa/Niamey
    {   10, 6803,    6444,  7, }, // Africa/Nouakchott
    {   10, 6727,    6451,  8, }, // Africa/Ouagadougou
    {   10, 4435,    6459,  8, }, // Africa/Sao_Tome
    {   10, 6554,    6467,  5, }, // Africa/Tunis
    {   10, 4152,    6472,  8, }, // Africa/Windhoek
    {   10, 3346,    6480,  9, }, // America/Anchorage
    {   10, 5607,    6489,  7, }, // America/Anguilla
    {   10, 4554,    6496, 10, }, // America/Araguaina
    {   10,  166,    6506, 12, }, // America/Argentina/Buenos_Aires
    {   10,   91,    6518,  9, }, // America/Argentina/Catamarca
    {   10,  270,    6527,  7, }, // America/Argentina/Cordoba
    {   10,  441,    6534,  5, }, // America/Argentina/Jujuy
    {   10, 7462,    6539,  8, }, // America/Argentina/La_Rioja
    {   10, 7489,    6547, 11, }, // America/Argentina/Rio_Gallegos
    {   10, 7520,    6558,  5, }, // America/Argentina/Salta
    {   10, 7544,    6563,  8, }, // America/Argentina/San_Juan
    {   10, 5580,    6571,  9, }, // America/Argentina/San_Luis
    {   10, 7571,    4842,  7, }, // America/Argentina/Tucuman
    {   10, 7597,    6580,  7, }, // America/Argentina/Ushuaia
    {   10, 5640,    6587,  5, }, // America/Aruba
    {   10, 4252,    6592,  8, }, // America/Asuncion
    {   10,  237,      22,  8, }, // America/Atikokan
    {   10, 7623,    6600, 14, }, // America/Bahia_Banderas
    {   10, 5407,    6614,  6, }, // America/Belize
    {   10, 7660,    6620, 12, }, // America/Blanc-Sablon
    {   10, 7681,    6632,  9, }, // America/Boa_Vista
    {   10, 4361,    6641,  6, }, // America/Bogota
    {   10, 7699,    6647,  5, }, // America/Boise
    {   10, 7713,    6652, 18, }, // America/Cambridge_Bay
    {   10, 7735,    6670, 12, }, // America/Campo_Grande
    {   10, 3868,    6682,  6, }, // America/Cancun
    {   10, 4694,    4910,  7, }, // America/Caracas
    {   10, 4345,    6688,  6, }, // America/Cayenne
    {   10, 5496,    6694,  6, }, // America/Cayman
    {   10, 2260,    6700,  6, }, // America/Chicago
    {   10, 7756,    6706,  7, }, // America/Chihuahua
    {   10, 7774,    6713, 13, }, // America/Ciudad_Juarez
    {   10, 5422,    6726, 10, }, // America/Costa_Rica
    {   10, 7796,    6736,  7, }, // America/Creston
    {   10, 3791,    6743,  6, }, // America/Cuiaba
    {   10, 5723,    6749,  7, }, // America/Curacao
    {   10, 7812,    6756, 13, }, // America/Danmarkshavn
    {   10, 7833,    6769,  6, }, // America/Dawson
    {   10, 7848,    6775, 16, }, // America/Dawson_Creek
    {   10, 5739,    6791,  8, }, // America/Dominica
    {   10, 7869,    6799,  7, }, // America/Eirunepe
    {   10, 5441,    6806,  8, }, // America/El_Salvador
    {   10, 7886,    6814, 11, }, // America/Fort_Nelson
    {   10, 7924,    6825, 14, }, // America/Glace_Bay
    {   10, 6881,    6839, 13, }, // America/Goose_Bay
    {   10, 4612,    6852, 12, }, // America/Grand_Turk
    {   10, 5770,    6864,  7, }, // America/Grenada
    {   10, 5786,    6871,  9, }, // America/Guadeloupe
    {   10, 3760,    6880,  9, }, // America/Guatemala
    {   10, 4841,    6889,  9, }, // America/Guayaquil
    {   10, 6932,    6898,  7, }, // America/Guyana
    {   10, 1939,    6905,  8, }, // America/Halifax
    {   10, 2281,    6913,  6, }, // America/Havana
    {   10, 7942,    6919,  9, }, // America/Hermosillo
    {   10,  481,    6928, 14, }, // America/Indiana/Knox
    {   10, 7961,      86, 16, }, // America/Indiana/Marengo
    {   10, 7985,    6942, 19, }, // America/Indiana/Petersburg
    {   10, 8012,    6961, 18, }, // America/Indiana/Tell_City
    {   10, 8038,    6979, 14, }, // America/Indiana/Vevay
    {   10, 8060,    6993, 17, }, // America/Indiana/Vincennes
    {   10, 8086,    7010, 16, }, // America/Indiana/Winamac
    {   10,  660,    5177,  7, }, // America/Iqaluit
    {   10, 2799,    7026,  8, }, // America/Jamaica
    {   10, 5380,    7034,  6, }, // America/Juneau
    {   10,  521,    7040,  8, }, // America/Kentucky/Louisville
    {   10, 8125,    7048, 18, }, // America/Kentucky/Monticello
    {   10, 7169,    7066,  4, }, // America/Lima
    {   10, 3239,    7070, 12, }, // America/Los_Angeles
    {   10, 5932,    7082, 21, }, // America/Lower_Princes
    {   10, 8153,    7103,  6, }, // America/Maceio
    {   10, 5805,    7109,  9, }, // America/Martinique
    {   10, 8202,    7118,  8, }, // America/Menominee
    {   10, 8220,    7126,  6, }, // America/Merida
    {   10, 2949,    7132, 17, }, // America/Mexico_City
    {   10, 4391,    7149,  7, }, // America/Miquelon
    {   10, 8254,    7156,  7, }, // America/Moncton
    {   10, 8270,    7163,  9, }, // America/Monterrey
    {   10, 5824,    7172,  9, }, // America/Montserrat
    {   10, 5481,    7181,  5, }, // America/Nassau
    {   10, 2379,    7186,  8, }, // America/New_York
    {   10, 1850,    7194,  7, }, // America/Noronha
    {   10, 8301,    7201, 22, }, // America/North_Dakota/Beulah
    {   10, 8329,    7223, 23, }, // America/North_Dakota/Center
    {   10, 8357,    7246, 26, }, // America/North_Dakota/New_Salem
    {   10,  393,    7272,  4, }, // America/Nuuk
    {   10, 8388,    7276,  7, }, // America/Ojinaga
    {   10, 2973,    7283,  6, }, // America/Phoenix
    {   10, 3945,    7289, 12, }, // America/Port-au-Prince
    {   10, 8404,    7301, 11, }, // America/Porto_Velho
    {   10, 5843,    7312, 10, }, // America/Puerto_Rico
    {   10, 4030,    7322, 12, }, // America/Punta_Arenas
    {   10, 8424,    7334, 11, }, // America/Rankin_Inlet
    {   10, 1995,    7345,  7, }, // America/Regina
    {   10,  695,    7352, 10, }, // America/Rio_Branco
    {   10, 6111,    7362, 13, }, // America/Santo_Domingo
    {   10, 1878,    7375,  9, }, // America/Sao_Paulo
    {   10, 6899,    7384, 12, }, // America/Scoresbysund
    {   10, 8508,    7396, 13, }, // America/St_Barthelemy
    {   10, 2061,    7409,  9, }, // America/St_Johns
    {   10, 5863,    7418,  9, }, // America/St_Kitts
    {   10, 5880,    7427, 10, }, // America/St_Lucia
    {   10,  855,    7437, 10, }, // America/St_Thomas
    {   10, 5913,    7447, 12, }, // America/St_Vincent
    {   10, 8530,    7459, 15, }, // America/Swift_Current
    {   10, 5461,    5528, 11, }, // America/Tegucigalpa
    {   10, 5756,    7474,  5, }, // America/Thule
    {   10,  313,    7479,  7, }, // America/Tijuana
    {   10, 5688,    7486,  7, }, // America/Tortola
    {   10, 2093,    7493,  8, }, // America/Vancouver
    {   10, 2144,    7501,  9, }, // America/Whitehorse
    {   10,  734,    7510,  7, }, // America/Winnipeg
    {   10, 8552,    7517,  7, }, // America/Yakutat
    {   10, 6001,    7524,  5, }, // Antarctica/Casey
    {   10, 6094,    7529,  6, }, // Antarctica/Davis
    {   10, 6133,    7535, 15, }, // Antarctica/DumontDUrville
    {   10, 8568,    7550,  7, }, // Antarctica/Macquarie
    {   10, 7066,    7557,  6, }, // Antarctica/Mawson
    {   10, 7113,    7563,  8, }, // Antarctica/McMurdo
    {   10, 7226,    7571,  7, }, // Antarctica/Rothera
    {   10, 7295,    7578,  5, }, // Antarctica/Syowa
    {   10, 1429,    7583, 11, }, // Arctic/Longyearbyen
    {   10, 5395,    7594,  6, }, // Asia/Almaty
    {   10, 3968,    5605,  4, }, // Asia/Amman
    {   10, 5511,    7600,  6, }, // Asia/Anadyr
    {   10, 5523,    5609,  5, }, // Asia/Aqtau
    {   10, 5368,    7606,  6, }, // Asia/Aqtobe
    {   10, 8624,    7612,  6, }, // Asia/Atyrau
    {   10, 3607,    4162,  6, }, // Asia/Baghdad
    {   10, 5534,    7618,  7, }, // Asia/Bahrain
    {   10, 4086,    7625,  6, }, // Asia/Beirut
    {   10, 4601,    7631,  4, }, // Asia/Chita
    {   10, 4511,    5637,  7, }, // Asia/Colombo
    {   10, 4540,    4201,  6, }, // Asia/Damascus
    {   10, 1093,    4717,  4, }, // Asia/Dhaka
    {   10, 8636,    7635,  9, }, // Asia/Famagusta
    {   10, 1259,    7644,  9, }, // Asia/Ho_Chi_Minh
    {   10, 2704,    7653,  9, }, // Asia/Hong_Kong
    {   10, 7006,    7662,  8, }, // Asia/Jakarta
    {   10, 6992,    7670,  9, }, // Asia/Jayapura
    {   10, 4316,    7679,  8, }, // Asia/Kamchatka
    {   10, 4239,    7687,  6, }, // Asia/Karachi
    {   10, 1185,    5711,  8, }, // Asia/Kathmandu
    {   10, 8661,    7693,  7, }, // Asia/Khandyga
    {   10,  992,    7700,  7, }, // Asia/Kolkata
    {   10, 4197,    4288, 11, }, // Asia/Krasnoyarsk
    {   10, 4859,    7707, 12, }, // Asia/Kuala_Lumpur
    {   10, 5976,    7719,  6, }, // Asia/Kuching
    {   10, 5547,    7725,  6, }, // Asia/Kuwait
    {   10, 1211,    7731,  5, }, // Asia/Macau
    {   10, 1349,    7736,  7, }, // Asia/Makassar
    {   10, 7182,    7743,  6, }, // Asia/Manila
    {   10, 6920,    4326,  6, }, // Asia/Muscat
    {   10, 2554,    7749,  7, }, // Asia/Nicosia
    {   10, 6961,    7756,  8, }, // Asia/Phnom_Penh
    {   10, 4214,    7764,  6, }, // Asia/Pyongyang
    {   10, 5559,    4332,  5, }, // Asia/Qatar
    {   10, 8708,    7770,  8, }, // Asia/Qostanay
    {   10, 4269,    7778,  9, }, // Asia/Qyzylorda
    {   10, 3584,    7787,  4, }, // Asia/Riyadh
    {   10, 4408,    7791,  7, }, // Asia/Sakhalin
    {   10, 3279,    7798,  4, }, // Asia/Seoul
    {   10, 1053,    7802,  7, }, // Asia/Shanghai
    {   10, 3300,    7809,  8, }, // Asia/Singapore
    {   10, 4297,    7817, 13, }, // Asia/Srednekolymsk
    {   10, 3263,    7830,  6, }, // Asia/Taipei
    {   10, 3915,    7836,  6, }, // Asia/Tbilisi
    {   10, 2772,    7842,  7, }, // Asia/Tehran
    {   10, 1317,    7849,  6, }, // Asia/Thimphu
    {   10, 2821,    4356,  5, }, // Asia/Tokyo
    {   10, 1021,    7855,  9, }, // Asia/Ulaanbaatar
    {   10, 6977,    7864,  9, }, // Asia/Vientiane
    {   10, 4828,    7873,  7, }, // Asia/Yakutsk
    {   10, 1235,    7880,  6, }, // Asia/Yangon
    {   10, 3883,    7886, 12, }, // Asia/Yekaterinburg
    {   10, 3747,    7898,  7, }, // Asia/Yerevan
    {   10, 3663,    7905,  5, }, // Atlantic/Azores
    {   10, 5671,    7910,  7, }, // Atlantic/Bermuda
    {   10, 6628,    4398,  6, }, // Atlantic/Canary
    {   10, 3727,    7917, 14, }, // Atlantic/Cape_Verde
    {   10, 1395,     483,  5, }, // Atlantic/Faroe
    {   10, 2748,    7931,  9, }, // Atlantic/Reykjavik
    {   10, 7272,    7940, 15, }, // Atlantic/South_Georgia
    {   10, 6821,    7955, 10, }, // Atlantic/St_Helena
    {   10, 6644,    7965,  6, }, // Atlantic/Stanley
    {   10, 1670,    7971,  8, }, // Australia/Adelaide
    {   10, 1635,    7979,  8, }, // Australia/Brisbane
    {   10, 1799,    7987, 11, }, // Australia/Broken_Hill
    {   10, 1583,    7998,  6, }, // Australia/Darwin
    {   10, 3637,    5864,  5, }, // Australia/Eucla
    {   10, 8753,    8004,  8, }, // Australia/Lindeman
    {   10, 1547,    8012,  9, }, // Australia/Lord_Howe
    {   10, 1727,    8021,  7, }, // Australia/Melbourne
    {   10, 1762,    8028,  5, }, // Australia/Perth
    {   10, 1463,    8033,  6, }, // Australia/Sydney
    {   10, 6216,    8039,  7, }, // Europe/Andorra
    {   10, 3620,    8046,  8, }, // Europe/Astrakhan
    {   10, 2300,    8054,  6, }, // Europe/Athens
    {   10, 6336,    8060,  7, }, // Europe/Belgrade
    {   10, 6488,    8067, 10, }, // Europe/Bratislava
    {   10, 2167,    8077,  7, }, // Europe/Brussels
    {   10, 3928,    8084,  9, }, // Europe/Bucharest
    {   10, 2583,    8093,  8, }, // Europe/Chisinau
    {   10, 6289,    4475, 10, }, // Europe/Copenhagen
    {   10, 6307,    8101, 10, }, // Europe/Gibraltar
    {   10, 8788,    8111,  7, }, // Europe/Guernsey
    {   10, 8804,    8118, 15, }, // Europe/Isle_of_Man
    {   10, 1130,    8133,  8, }, // Europe/Istanbul
    {   10, 8823,    8141,  7, }, // Europe/Jersey
    {   10, 2527,    8148,  4, }, // Europe/Kyiv
    {   10, 3213,    8152,  7, }, // Europe/Lisbon
    {   10, 6506,    8159,  7, }, // Europe/Ljubljana
    {   10, 2501,    8166,  6, }, // Europe/London
    {   10, 6365,    8172, 10, }, // Europe/Luxembourg
    {   10, 6397,    8182,  5, }, // Europe/Malta
    {   10, 6410,    6044,  6, }, // Europe/Monaco
    {   10, 3537,    8187,  5, }, // Europe/Moscow
    {   10, 6424,    8192,  9, }, // Europe/Podgorica
    {   10, 6275,    8201,  5, }, // Europe/Prague
    {   10, 8837,    8206,  4, }, // Europe/Riga
    {   10, 6324,    8210,  4, }, // Europe/Rome
    {   10, 6470,    8214, 10, }, // Europe/San_Marino
    {   10, 6245,    8224,  8, }, // Europe/Sarajevo
    {   10, 6383,    8232,  5, }, // Europe/Skopje
    {   10, 6599,    8237,  5, }, // Europe/Sofia
    {   10, 6523,    8242,  8, }, // Europe/Stockholm
    {   10, 8867,    8250,  5, }, // Europe/Tallinn
    {   10, 6202,    8255,  6, }, // Europe/Tirane
    {   10, 8882,    8261,  9, }, // Europe/Ulyanovsk
    {   10, 6567,    6096,  7, }, // Europe/Vatican
    {   10, 6231,    8270,  5, }, // Europe/Vienna
    {   10, 3190,    8275,  8, }, // Europe/Warsaw
    {   10, 6540,    8283,  5, }, // Europe/Zurich
    {   10, 6947,    8288,  5, }, // Indian/Chagos
    {   10, 6046,    8293, 12, }, // Indian/Christmas
    {   10, 6063,    4583,  5, }, // Indian/Cocos
    {   10, 5029,    8305,  6, }, // Indian/Comoro
    {   10, 7050,    8311,  7, }, // Indian/Maldives
    {   10, 5098,    8318,  6, }, // Indian/Mayotte
    {   10, 7211,     547,  7, }, // Indian/Reunion
    {   10,  932,    8324,  6, }, // Pacific/Auckland
    {   10, 3706,    8330, 10, }, // Pacific/Bougainville
    {   10, 3015,    8340,  6, }, // Pacific/Chatham
    {   10, 3157,    8346,  3, }, // Pacific/Chuuk
    {   10, 2237,    8349,  6, }, // Pacific/Easter
    {   10, 3902,    8355,  5, }, // Pacific/Fiji
    {   10, 3822,    8360, 11, }, // Pacific/Guadalcanal
    {   10, 2723,    4611,  8, }, // Pacific/Honolulu
    {   10, 3049,    4619,  8, }, // Pacific/Kanton
    {   10, 3998,    8371,  9, }, // Pacific/Kiritimati
    {   10, 7019,    8380,  5, }, // Pacific/Kosrae
    {   10, 2842,    8385, 10, }, // Pacific/Kwajalein
    {   10, 4877,    8395,  6, }, // Pacific/Majuro
    {   10, 4051,    8401,  6, }, // Pacific/Marquesas
    {   10, 8914,    8407,  6, }, // Pacific/Midway
    {   10, 3126,    8413,  9, }, // Pacific/Pago_Pago
    {   10, 7194,    8422,  7, }, // Pacific/Pitcairn
    {   10, 3096,    8429,  7, }, // Pacific/Pohnpei
    {   10, 4807,    8436, 12, }, // Pacific/Port_Moresby
    {   10, 6076,    8448,  9, }, // Pacific/Rarotonga
    {   10, 6712,    8457,  6, }, // Pacific/Tarawa
    {   10, 7392,    8463,  4, }, // Pacific/Wake
    {   10, 7405,    8467,  6, }, // Pacific/Wallis
    {   13, 6788,    8473,  4, }, // Africa/Abidjan Amharic/Ethiopic/Ethiopia
    {   13, 6760,    8477,  3, }, // Africa/Accra
    {   13, 5059,    8480,  7, }, // Africa/Addis_Ababa
    {   13, 7420,    8487,  5, }, // Africa/Algiers
    {   13,   14,    8492,  4, }, // Africa/Asmara
    {   13,   44,    8496,  3, }, // Africa/Bamako
    {   13, 5257,    8499,  4, }, // Africa/Bangui
    {   13, 6746,    8503,  4, }, // Africa/Banjul
    {   13, 7435,    8507,  3, }, // Africa/Bissau
    {   13, 4957,    8510,  6, }, // Africa/Blantyre
    {   13, 5287,    8516,  5, }, // Africa/Brazzaville
    {   13, 4922,    8521,  5, }, // Africa/Bujumbura
    {   13, 2320,    8526,  3, }, // Africa/Cairo
    {   13, 4117,    8529,  6, }, // Africa/Casablanca
    {   13, 7449,    8535,  3, }, // Africa/Ceuta
    {   13, 6773,    8538,  4, }, // Africa/Conakry
    {   13, 6840,    8542,  3, }, // Africa/Dakar
    {   13, 5130,    8545,  6, }, // Africa/Dar_es_Salaam
    {   13, 5043,    8551,  3, }, // Africa/Djibouti
    {   13, 5243,    8554,  3, }, // Africa/Douala
    {   13, 5166,    8557,  7, }, // Africa/El_Aaiun
    {   13, 6853,    8564,  5, }, // Africa/Freetown
    {   13, 4906,    8569,  4, }, // Africa/Gaborone
    {   13, 5015,    8573,  3, }, // Africa/Harare
    {   13, 4479,    8576,  7, }, // Africa/Johannesburg
    {   13, 4499,    8583,  2, }, // Africa/Juba
    {   13, 5151,    8585,  4, }, // Africa/Kampala
    {   13, 4524,    8589,  4, }, // Africa/Khartoum
    {   13, 4973,    8593,  3, }, // Africa/Kigali
    {   13, 5306,    8596,  4, }, // Africa/Kinshasa
    {   13, 4744,    8600,  3, }, // Africa/Lagos
    {   13, 5336,    8603,  5, }, // Africa/Libreville
    {   13, 6869,    8608,  2, }, // Africa/Lome
    {   13, 5211,    8610,  4, }, // Africa/Luanda
    {   13, 4939,    8614,  5, }, // Africa/Lubumbashi
    {   13, 5001,    8619,  3, }, // Africa/Lusaka
    {   13, 5322,    8622,  3, }, // Africa/Malabo
    {   13, 4987,    8625,  3, }, // Africa/Maputo
    {   13, 5197,    8628,  3, }, // Africa/Maseru
    {   13, 5182,    8631,  4, }, // Africa/Mbabane
    {   13, 5113,    8635,  4, }, // Africa/Mogadishu
    {   13, 7034,    8639,  5, }, // Africa/Monrovia
    {   13, 3853,    8644,  4, }, // Africa/Nairobi
    {   13, 5271,    8648,  4, }, // Africa/Ndjamena
    {   13, 5354,    8652,  4, }, // Africa/Niamey
    {   13, 6803,    8656,  5, }, // Africa/Nouakchott
    {   13, 6727,    8661,  4, }, // Africa/Ouagadougou
    {   13, 5225,    8665,  8, }, // Africa/Porto-Novo
    {   13, 4435,    8673,  5, }, // Africa/Sao_Tome
    {   13, 2866,    8678,  4, }, // Africa/Tripoli
    {   13, 6554,    8682,  3, }, // Africa/Tunis
    {   13, 4152,    8685,  5, }, // Africa/Windhoek
    {   13,  132,    8690,  3, }, // America/Adak
    {   13, 3346,    8693,  5, }, // America/Anchorage
    {   13, 5607,    8698,  5, }, // America/Anguilla
    {   13, 5624,    8703,  4, }, // America/Antigua
    {   13, 4554,    8707,  5, }, // America/Araguaina
    {   13,  166,    8712,  8, }, // America/Argentina/Buenos_Aires
    {   13,   91,    8720,  5, }, // America/Argentina/Catamarca
    {   13,  270,    8725,  4, }, // America/Argentina/Cordoba
    {   13,  441,    8729,  3, }, // America/Argentina/Jujuy
    {   13, 7462,    8732,  5, }, // America/Argentina/La_Rioja
    {   13,  565,    8737,  4, }, // America/Argentina/Mendoza
    {   13, 7489,    8741,  7, }, // America/Argentina/Rio_Gallegos
    {   13, 7520,    8748,  3, }, // America/Argentina/Salta
    {   13, 7544,    8751,  6, }, // America/Argentina/San_Juan
    {   13, 5580,    8757,  6, }, // America/Argentina/San_Luis
    {   13, 7571,    8763,  4, }, // America/Argentina/Tucuman
    {   13, 7597,    8767,  4, }, // America/Argentina/Ushuaia
    {   13, 5640,    8771,  3, }, // America/Aruba
    {   13, 4252,    8774,  6, }, // America/Asuncion
    {   13,  237,    8780,  5, }, // America/Atikokan
    {   13, 3679,    8785,  3, }, // America/Bahia
    {   13, 7623,    8788,  9, }, // America/Bahia_Banderas
    {   13, 5654,    8797,  5, }, // America/Barbados
    {   13, 7646,    8802,  3, }, // America/Belem
    {   13, 5407,    8805,  3, }, // America/Belize
    {   13, 7660,    8808, 10, }, // America/Blanc-Sablon
    {   13, 7681,    8818,  6, }, // America/Boa_Vista
    {   13, 4361,    8824,  3, }, // America/Bogota
    {   13, 7699,    8827,  3, }, // America/Boise
    {   13, 7713,    8830,  8, }, // America/Cambridge_Bay
    {   13, 7735,    8838,  8, }, // America/Campo_Grande
    {   13, 3868,    8846,  4, }, // America/Cancun
    {   13, 4694,    8850,  4, }, // America/Caracas
    {   13, 4345,    8854,  4, }, // America/Cayenne
    {   13, 5496,    8858,  4, }, // America/Cayman
    {   13, 2260,    8862,  3, }, // America/Chicago
    {   13, 7756,    8865,  5, }, // America/Chihuahua
    {   13, 7774,    8870,  9, }, // America/Ciudad_Juarez
    {   13, 5422,    8879,  5, }, // America/Costa_Rica
    {   13, 7796,    8884,  5, }, // America/Creston
    {   13, 3791,    8889,  4, }, // America/Cuiaba
    {   13, 5723,    8893,  4, }, // America/Curacao
    {   13, 7812,    8897,  8, }, // America/Danmarkshavn
    {   13, 7833,    8905,  4, }, // America/Dawson
    {   13, 7848,    8909,  8, }, // America/Dawson_Creek
    {   13,  805,    8917,  4, }, // America/Denver
    {   13, 3465,    8921,  5, }, // America/Detroit
    {   13, 5739,    8926,  4, }, // America/Dominica
    {   13,  893,    8930,  6, }, // America/Edmonton
    {   13, 7869,    8936,  4, }, // America/Eirunepe
    {   13, 5441,    8940,  7, }, // America/El_Salvador
    {   13, 7886,    8947,  8, }, // America/Fort_Nelson
    {   13, 7906,    8955,  5, }, // America/Fortaleza
    {   13, 7924,    8960,  6, }, // America/Glace_Bay
    {   13, 6881,    8966,  5, }, // America/Goose_Bay
    {   13, 4612,    8971,  8, }, // America/Grand_Turk
    {   13, 5770,    8979,  4, }, // America/Grenada
    {   13, 5786,    8983,  4, }, // America/Guadeloupe
    {   13, 3760,    8987,  4, }, // America/Guatemala
    {   13, 4841,    8991,  5, }, // America/Guayaquil
    {   13, 6932,    8996,  3, }, // America/Guyana
    {   13, 1939,    8999,  5, }, // America/Halifax
    {   13, 2281,    9004,  3, }, // America/Havana
    {   13, 7942,    9007,  5, }, // America/Hermosillo
    {   13,  348,    9012,  8, }, // America/Indiana/Indianapolis
    {   13,  481,    9020, 10, }, // America/Indiana/Knox
    {   13, 7961,    9030, 11, }, // America/Indiana/Marengo
    {   13, 7985,    9041, 14, }, // America/Indiana/Petersburg
    {   13, 8012,    9055, 13, }, // America/Indiana/Tell_City
    {   13, 8038,    9068, 10, }, // America/Indiana/Vevay
    {   13, 8060,    9078, 12, }, // America/Indiana/Vincennes
    {   13, 8086,    9090, 11, }, // America/Indiana/Winamac
    {   13, 8110,    9101,  4, }, // America/Inuvik
    {   13,  660,    9105,  5, }, // America/Iqaluit
    {   13, 2799,    9110,  4, }, // America/Jamaica
    {   13, 5380,    9114,  3, }, // America/Juneau
    {   13,  521,    9117,  5, }, // America/Kentucky/Louisville
    {   13, 8125,    9122, 11, }, // America/Kentucky/Monticello
    {   13, 5704,    9133,  7, }, // America/Kralendijk
    {   13, 4376,    9140,  4, }, // America/La_Paz
    {   13, 7169,    9144,  2, }, // America/Lima
    {   13, 3239,    9146,  8, }, // America/Los_Angeles
    {   13, 5932,    9154, 12, }, // America/Lower_Princes
    {   13, 8153,    9166,  3, }, // America/Maceio
    {   13, 8168,    9169,  4, }, // America/Managua
    {   13, 1908,    9173,  4, }, // America/Manaus
    {   13, 5897,    9177,  4, }, // America/Marigot
    {   13, 5805,    9181,  5, }, // America/Martinique
    {   13, 8184,    9186,  5, }, // America/Matamoros
    {   13, 2917,    9191,  5, }, // America/Mazatlan
    {   13, 8202,    9196,  4, }, // America/Menominee
    {   13, 8220,    9200,  3, }, // America/Merida
    {   13, 8235,    9203,  6, }, // America/Metlakatla
    {   13, 2949,    9209,  8, }, // America/Mexico_City
    {   13, 4391,    9217,  5, }, // America/Miquelon
    {   13, 8254,    9222,  5, }, // America/Moncton
    {   13, 8270,    9227,  6, }, // America/Monterrey
    {   13, 4098,    9233,  6, }, // America/Montevideo
    {   13, 5824,    9239,  6, }, // America/Montserrat
    {   13, 5481,    9245,  3, }, // America/Nassau
    {   13, 2379,    9248,  5, }, // America/New_York
    {   13, 8288,    9253,  2, }, // America/Nome
    {   13, 1850,    9255,  3, }, // America/Noronha
    {   13, 8301,    9258, 13, }, // America/North_Dakota/Beulah
    {   13, 8329,    9271, 14, }, // America/North_Dakota/Center
    {   13, 8357,    9285, 16, }, // America/North_Dakota/New_Salem
    {   13,  393,    9301,  4, }, // America/Nuuk
    {   13, 8388,    9305,  4, }, // America/Ojinaga
    {   13, 2356,    9309,  3, }, // America/Panama
    {   13, 6173,    9312,  5, }, // America/Paramaribo
    {   13, 2973,    9317,  4, }, // America/Phoenix
    {   13, 3945,    9321, 10, }, // America/Port-au-Prince
    {   13, 5954,    9331,  9, }, // America/Port_of_Spain
    {   13, 8404,    9340,  7, }, // America/Porto_Velho
    {   13, 5843,    9347,  5, }, // America/Puerto_Rico
    {   13, 4030,    9352,  8, }, // America/Punta_Arenas
    {   13, 8424,    9360,  9, }, // America/Rankin_Inlet
    {   13, 8445,    9369,  3, }, // America/Recife
    {   13, 1995,    9372,  3, }, // America/Regina
    {   13, 8460,    9375,  5, }, // America/Resolute
    {   13,  695,    9380,  7, }, // America/Rio_Branco
    {   13, 8477,    9387,  5, }, // America/Santarem
    {   13, 2201,    9392,  5, }, // America/Santiago
    {   13, 6111,    9397,  8, }, // America/Santo_Domingo
    {   13, 1878,    9405,  5, }, // America/Sao_Paulo
    {   13, 6899,    9410,  9, }, // America/Scoresbysund
    {   13, 8494,    9419,  3, }, // America/Sitka
    {   13, 8508,    9422, 10, }, // America/St_Barthelemy
    {   13, 2061,    9432,  8, }, // America/St_Johns
    {   13, 5863,    9440,  7, }, // America/St_Kitts
    {   13, 5880,    9447,  8, }, // America/St_Lucia
    {   13,  855,    9455,  7, }, // America/St_Thomas
    {   13, 5913,    9462,  9, }, // America/St_Vincent
    {   13, 8530,    9471,  9, }, // America/Swift_Current
    {   13, 5461,    9480,  6, }, // America/Tegucigalpa
    {   13, 5756,    9486,  2, }, // America/Thule
    {   13,  313,    9488,  4, }, // America/Tijuana
    {   13,  608,    9492,  4, }, // America/Toronto
    {   13, 5688,    9496,  4, }, // America/Tortola
    {   13, 2093,    9500,  5, }, // America/Vancouver
    {   13, 2144,    9505,  6, }, // America/Whitehorse
    {   13,  734,    9511,  4, }, // America/Winnipeg
    {   13, 8552,    9515,  4, }, // America/Yakutat
    {   13, 6001,    9519,  3, }, // Antarctica/Casey
    {   13, 6094,    9522,  3, }, // Antarctica/Davis
    {   13, 6133,    9525, 10, }, // Antarctica/DumontDUrville
    {   13, 8568,    9535,  3, }, // Antarctica/Macquarie
    {   13, 7066,    9538,  4, }, // Antarctica/Mawson
    {   13, 7113,    9542,  5, }, // Antarctica/McMurdo
    {   13, 8589,    9547,  4, }, // Antarctica/Palmer
    {   13, 7226,    9551,  3, }, // Antarctica/Rothera
    {   13, 7295,    9554,  3, }, // Antarctica/Syowa
    {   13, 8607,    9557,  3, }, // Antarctica/Troll
    {   13, 7374,    9560,  4, }, // Antarctica/Vostok
    {   13, 1429,    9564,  8, }, // Arctic/Longyearbyen
    {   13, 5570,    9572,  3, }, // Asia/Aden
    {   13, 5395,    9575,  5, }, // Asia/Almaty
    {   13, 3968,    9580,  3, }, // Asia/Amman
    {   13, 5511,    9583,  5, }, // Asia/Anadyr
    {   13, 5523,    9588,  4, }, // Asia/Aqtau
    {   13, 5368,    9592,  4, }, // Asia/Aqtobe
    {   13,  964,    9596,  5, }, // Asia/Ashgabat
    {   13, 8624,    9601,  5, }, // Asia/Atyrau
    {   13, 3607,    9606,  4, }, // Asia/Baghdad
    {   13, 5534,    9610,  4, }, // Asia/Bahrain
    {   13, 3653,    8892,  2, }, // Asia/Baku
    {   13, 4466,    9614,  4, }, // Asia/Bangkok
    {   13, 3571,    9618,  5, }, // Asia/Barnaul
    {   13, 4086,    9623,  3, }, // Asia/Beirut
    {   13, 3778,    9626,  4, }, // Asia/Bishkek
    {   13, 5989,    9630,  4, }, // Asia/Brunei
    {   13, 4601,    9634,  2, }, // Asia/Chita
    {   13, 4511,    9636,  4, }, // Asia/Colombo
    {   13, 4540,    9640,  4, }, // Asia/Damascus
    {   13, 1093,    8542,  2, }, // Asia/Dhaka
    {   13, 6192,    9644,  2, }, // Asia/Dili
    {   13, 3596,    9646,  3, }, // Asia/Dubai
    {   13, 6159,    9649,  4, }, // Asia/Dushanbe
    {   13, 8636,    9653,  5, }, // Asia/Famagusta
    {   13, 8651,    9658,  2, }, // Asia/Gaza
    {   13, 4795,    9660,  4, }, // Asia/Hebron
    {   13, 1259,    9664, 11, }, // Asia/Ho_Chi_Minh
    {   13, 2704,    9675,  7, }, // Asia/Hong_Kong
    {   13, 4771,    9682,  3, }, // Asia/Hovd
    {   13, 4184,    9685,  6, }, // Asia/Irkutsk
    {   13, 7006,    9691,  4, }, // Asia/Jakarta
    {   13, 6992,    9695,  4, }, // Asia/Jayapura
    {   13, 1290,    9699,  6, }, // Asia/Jerusalem
    {   13, 3560,    9705,  3, }, // Asia/Kabul
    {   13, 4316,    9708,  5, }, // Asia/Kamchatka
    {   13, 4239,    9713,  3, }, // Asia/Karachi
    {   13, 1185,    9716,  5, }, // Asia/Kathmandu
    {   13, 8661,    9721,  4, }, // Asia/Khandyga
    {   13,  992,    9725,  4, }, // Asia/Kolkata
    {   13, 4197,    9729,  8, }, // Asia/Krasnoyarsk
    {   13, 4859,    9737,  8, }, // Asia/Kuala_Lumpur
    {   13, 5976,    9745,  4, }, // Asia/Kuching
    {   13, 5547,    9749,  3, }, // Asia/Kuwait
    {   13, 1211,    9752,  3, }, // Asia/Macau
    {   13, 4017,    9755,  4, }, // Asia/Magadan
    {   13, 1349,    9759,  4, }, // Asia/Makassar
    {   13, 7182,    9763,  3, }, // Asia/Manila
    {   13, 6920,    9766,  4, }, // Asia/Muscat
    {   13, 2554,    9770,  4, }, // Asia/Nicosia
    {   13, 8675,    9774,  7, }, // Asia/Novokuznetsk
    {   13, 4135,    9781,  7, }, // Asia/Novosibirsk
    {   13, 4229,    9788,  4, }, // Asia/Omsk
    {   13, 7145,    9792,  3, }, // Asia/Oral
    {   13, 6961,    9795,  7, }, // Asia/Phnom_Penh
    {   13, 8693,    9802,  6, }, // Asia/Pontianak
    {   13, 4214,    9808,  7, }, // Asia/Pyongyang
    {   13, 5559,    9815,  3, }, // Asia/Qatar
    {   13, 8708,    9818,  5, }, // Asia/Qostanay
    {   13, 4269,    9823,  6, }, // Asia/Qyzylorda
    {   13, 3584,    9829,  3, }, // Asia/Riyadh
    {   13, 4408,    9832,  5, }, // Asia/Sakhalin
    {   13, 7245,    9837,  6, }, // Asia/Samarkand
    {   13, 3279,    9843,  3, }, // Asia/Seoul
    {   13, 1053,    9846,  4, }, // Asia/Shanghai
    {   13, 3300,    9850,  5, }, // Asia/Singapore
    {   13, 4297,    9855, 10, }, // Asia/Srednekolymsk
    {   13, 3263,    9865,  4, }, // Asia/Taipei
    {   13, 4781,    9869,  5, }, // Asia/Tashkent
    {   13, 3915,    9874,  4, }, // Asia/Tbilisi
    {   13, 2772,    9878,  4, }, // Asia/Tehran
    {   13, 1317,    9882,  3, }, // Asia/Thimphu
    {   13, 2821,    9885,  3, }, // Asia/Tokyo
    {   13, 4572,    9888,  4, }, // Asia/Tomsk
    {   13, 1021,    9892,  8, }, // Asia/Ulaanbaatar
    {   13, 1159,    9900,  4, }, // Asia/Urumqi
    {   13, 8722,    9904,  6, }, // Asia/Ust-Nera
    {   13, 6977,    9910,  5, }, // Asia/Vientiane
    {   13, 4710,    9915,  7, }, // Asia/Vladivostok
    {   13, 4828,    9922,  5, }, // Asia/Yakutsk
    {   13, 1235,    9927,  4, }, // Asia/Yangon
    {   13, 3883,    9931,  8, }, // Asia/Yekaterinburg
    {   13, 3747,    9939,  4, }, // Asia/Yerevan
    {   13, 3663,    9943,  4, }, // Atlantic/Azores
    {   13, 5671,    9947,  4, }, // Atlantic/Bermuda
    {   13, 6628,    9951,  3, }, // Atlantic/Canary
    {   13, 3727,    9954,  6, }, // Atlantic/Cape_Verde
    {   13, 1395,    9960,  3, }, // Atlantic/Faroe
    {   13, 8736,    9963,  3, }, // Atlantic/Madeira
    {   13, 2748,    9966,  6, }, // Atlantic/Reykjavik
    {   13, 7272,    9972,  8, }, // Atlantic/South_Georgia
    {   13, 6821,    9980,  8, }, // Atlantic/St_Helena
    {   13, 6644,    9988,  5, }, // Atlantic/Stanley
    {   13, 1670,    9993,  5, }, // Australia/Adelaide
    {   13, 1635,    9998,  5, }, // Australia/Brisbane
    {   13, 1799,   10003,  7, }, // Australia/Broken_Hill
    {   13, 1583,   10010,  4, }, // Australia/Darwin
    {   13, 3637,   10014,  3, }, // Australia/Eucla
    {   13, 1516,   10017,  4, }, // Australia/Hobart
    {   13, 8753,   10021,  5, }, // Australia/Lindeman
    {   13, 1547,   10026,  6, }, // Australia/Lord_Howe
    {   13, 1727,   10032,  5, }, // Australia/Melbourne
    {   13, 1762,   10037,  3, }, // Australia/Perth
    {   13, 1463,   10040,  3, }, // Australia/Sydney
    {   13, 6441,   10043,  7, }, // Europe/Amsterdam
    {   13, 6216,   10050,  4, }, // Europe/Andorra
    {   13, 3620,   10054,  7, }, // Europe/Astrakhan
    {   13, 2300,   10061,  4, }, // Europe/Athens
    {   13, 6336,   10065,  5, }, // Europe/Belgrade
    {   13, 4757,   10070,  4, }, // Europe/Berlin
    {   13, 6488,   10074,  6, }, // Europe/Bratislava
    {   13, 2167,   10080,  5, }, // Europe/Brussels
    {   13, 3928,   10085,  5, }, // Europe/Bucharest
    {   13, 3806,   10090,  5, }, // Europe/Budapest
    {   13, 8772,   10095,  5, }, // Europe/Busingen
    {   13, 2583,   10100,  4, }, // Europe/Chisinau
    {   13, 6289,   10104,  6, }, // Europe/Copenhagen
    {   13, 2338,   10110,  4, }, // Europe/Dublin
    {   13, 6307,   10114,  6, }, // Europe/Gibraltar
    {   13, 8788,   10120,  4, }, // Europe/Guernsey
    {   13, 6612,   10124,  5, }, // Europe/Helsinki
    {   13, 8804,   10129, 10, }, // Europe/Isle_of_Man
    {   13, 1130,   10139,  6, }, // Europe/Istanbul
    {   13, 8823,   10145,  3, }, // Europe/Jersey
    {   13, 3979,   10148,  7, }, // Europe/Kaliningrad
    {   13, 8929,   10155,  3, }, // Europe/Kirov
    {   13, 2527,   10158,  3, }, // Europe/Kyiv
    {   13, 3213,   10161,  4, }, // Europe/Lisbon
    {   13, 6506,   10165,  6, }, // Europe/Ljubljana
    {   13, 2501,   10171,  4, }, // Europe/London
    {   13, 6365,   10175,  7, }, // Europe/Luxembourg
    {   13, 4892,   10182,  4, }, // Europe/Madrid
    {   13, 6397,   10186,  3, }, // Europe/Malta
    {   13, 6582,   10189,  5, }, // Europe/Mariehamn
    {   13, 3693,   10194,  4, }, // Europe/Minsk
    {   13, 6410,   10198,  3, }, // Europe/Monaco
    {   13, 3537,   10201,  3, }, // Europe/Moscow
    {   13, 6458,   10204,  3, }, // Europe/Oslo
    {   13, 4284,   10207,  3, }, // Europe/Paris
    {   13, 6424,   10210,  5, }, // Europe/Podgorica
    {   13, 6275,   10215,  3, }, // Europe/Prague
    {   13, 8837,   10218,  2, }, // Europe/Riga
    {   13, 6324,   10220,  2, }, // Europe/Rome
    {   13, 4331,   10222,  3, }, // Europe/Samara
    {   13, 6470,   10225,  6, }, // Europe/San_Marino
    {   13, 6245,   10231,  4, }, // Europe/Sarajevo
    {   13, 4451,   10235,  4, }, // Europe/Saratov
    {   13, 8849,   10239,  6, }, // Europe/Simferopol
    {   13, 6383,   10245,  4, }, // Europe/Skopje
    {   13, 6599,   10249,  3, }, // Europe/Sofia
    {   13, 6523,   10252,  6, }, // Europe/Stockholm
    {   13, 8867,   10258,  3, }, // Europe/Tallinn
    {   13, 6202,   10261,  3, }, // Europe/Tirane
    {   13, 8882,   10264,  7, }, // Europe/Ulyanovsk
    {   13, 6352,   10271,  3, }, // Europe/Vaduz
    {   13, 6567,   10274,  4, }, // Europe/Vatican
    {   13, 6231,   10278,  3, }, // Europe/Vienna
    {   13, 8899,   10281,  4, }, // Europe/Vilnius
    {   13, 4727,   10285,  5, }, // Europe/Volgograd
    {   13, 3190,   10290,  4, }, // Europe/Warsaw
    {   13, 6261,   10294,  4, }, // Europe/Zagreb
    {   13, 6540,   10298,  3, }, // Europe/Zurich
    {   13, 5078,   10301,  7, }, // Indian/Antananarivo
    {   13, 6947,   10308,  3, }, // Indian/Chagos
    {   13, 6046,   10311,  2, }, // Indian/Christmas
    {   13, 6063,   10313,  3, }, // Indian/Cocos
    {   13, 5029,   10316,  3, }, // Indian/Comoro
    {   13, 6661,   10319,  5, }, // Indian/Kerguelen
    {   13, 7260,   10324,  2, }, // Indian/Mahe
    {   13, 7050,   10326,  4, }, // Indian/Maldives
    {   13, 4069,   10330,  5, }, // Indian/Mauritius
    {   13, 5098,   10335,  3, }, // Indian/Mayotte
    {   13, 7211,   10338,  5, }, // Indian/Reunion
    {   13, 4422,   10343,  3, }, // Pacific/Apia
    {   13,  932,   10346,  5, }, // Pacific/Auckland
    {   13, 3706,   10351,  6, }, // Pacific/Bougainville
    {   13, 3015,   10357,  3, }, // Pacific/Chatham
    {   13, 3157,   10360,  2, }, // Pacific/Chuuk
    {   13, 2237,   10362,  3, }, // Pacific/Easter
    {   13, 7360,   10365,  3, }, // Pacific/Efate
    {   13, 7327,   10368,  4, }, // Pacific/Fakaofo
    {   13, 3902,   10372,  2, }, // Pacific/Fiji
    {   13, 7343,   10374,  4, }, // Pacific/Funafuti
    {   13, 6678,   10378,  5, }, // Pacific/Galapagos
    {   13, 6696,   10383,  5, }, // Pacific/Gambier
    {   13, 3822,   10388,  7, }, // Pacific/Guadalcanal
    {   13, 6018,   10395,  3, }, // Pacific/Guam
    {   13, 2723,   10398,  4, }, // Pacific/Honolulu
    {   13, 3049,   10402,  6, }, // Pacific/Kanton
    {   13, 3998,   10408,  5, }, // Pacific/Kiritimati
    {   13, 7019,   10413,  3, }, // Pacific/Kosrae
    {   13, 2842,   10416,  5, }, // Pacific/Kwajalein
    {   13, 4877,   10421,  3, }, // Pacific/Majuro
    {   13, 4051,   10424,  6, }, // Pacific/Marquesas
    {   13, 8914,   10430,  4, }, // Pacific/Midway
    {   13, 7084,   10434,  3, }, // Pacific/Nauru
    {   13, 7132,   10437,  2, }, // Pacific/Niue
    {   13, 4168,   10439,  5, }, // Pacific/Norfolk
    {   13, 7098,   10444,  4, }, // Pacific/Noumea
    {   13, 3126,   10448,  5, }, // Pacific/Pago_Pago
    {   13, 7155,   10453,  3, }, // Pacific/Palau
    {   13, 7194,   10456,  6, }, // Pacific/Pitcairn
    {   13, 3096,   10462,  4, }, // Pacific/Pohnpei
    {   13, 4807,   10466,  9, }, // Pacific/Port_Moresby
    {   13, 6076,   10475,  5, }, // Pacific/Rarotonga
    {   13, 6031,   10480,  4, }, // Pacific/Saipan
    {   13, 7312,   10484,  3, }, // Pacific/Tahiti
    {   13, 6712,   10487,  3, }, // Pacific/Tarawa
    {   13, 4583,   10490,  5, }, // Pacific/Tongatapu
    {   13, 7392,   10495,  2, }, // Pacific/Wake
    {   13, 7405,   10497,  3, }, // Pacific/Wallis
    {   14, 6788,   10500,  7, }, // Africa/Abidjan Arabic/Arabic/Egypt
    {   14, 6760,   10507,  4, }, // Africa/Accra
    {   14, 5059,   10511, 10, }, // Africa/Addis_Ababa
    {   14, 7420,   10521,  7, }, // Africa/Algiers
    {   14,   14,   10528,  5, }, // Africa/Asmara
    {   14,   44,   10533,  6, }, // Africa/Bamako
    {   14, 5257,   10539,  6, }, // Africa/Bangui
    {   14, 6746,   10545,  6, }, // Africa/Banjul
    {   14, 7435,   10551,  5, }, // Africa/Bissau
    {   14, 4957,   10556,  8, }, // Africa/Blantyre
    {   14, 5287,   10564,  8, }, // Africa/Brazzaville
    {   14, 4922,   10572,  9, }, // Africa/Bujumbura
    {   14, 2320,   10581,  7, }, // Africa/Cairo
    {   14, 4117,   10588, 13, }, // Africa/Casablanca
    {   14, 7449,   10601,  4, }, // Africa/Ceuta
    {   14, 6773,   10605,  7, }, // Africa/Conakry
    {   14, 6840,   10612,  5, }, // Africa/Dakar
    {   14, 5130,   10617, 10, }, // Africa/Dar_es_Salaam
    {   14, 5043,   10627,  6, }, // Africa/Djibouti
    {   14, 5243,   10633,  5, }, // Africa/Douala
    {   14, 5166,   10638,  6, }, // Africa/El_Aaiun
    {   14, 6853,   10644,  8, }, // Africa/Freetown
    {   14, 4906,   10652,  7, }, // Africa/Gaborone
    {   14, 5015,   10659,  5, }, // Africa/Harare
    {   14, 4479,   10664,  9, }, // Africa/Johannesburg
    {   14, 4499,   10673,  4, }, // Africa/Juba
    {   14, 5151,   10677,  7, }, // Africa/Kampala
    {   14, 4524,   10684,  7, }, // Africa/Khartoum
    {   14, 4973,   10691,  6, }, // Africa/Kigali
    {   14, 5306,   10697,  7, }, // Africa/Kinshasa
    {   14, 4744,   10704,  5, }, // Africa/Lagos
    {   14, 5336,   10709,  7, }, // Africa/Libreville
    {   14, 6869,   10716,  4, }, // Africa/Lome
    {   14, 5211,   10720,  6, }, // Africa/Luanda
    {   14, 4939,   10726,  8, }, // Africa/Lubumbashi
    {   14, 5001,   10734,  6, }, // Africa/Lusaka
    {   14, 5322,   10740,  6, }, // Africa/Malabo
    {   14, 4987,   10746,  6, }, // Africa/Maputo
    {   14, 5197,   10752,  6, }, // Africa/Maseru
    {   14, 5182,   10758,  7, }, // Africa/Mbabane
    {   14, 5113,   10765,  7, }, // Africa/Mogadishu
    {   14, 7034,   10772,  8, }, // Africa/Monrovia
    {   14, 3853,   10780,  6, }, // Africa/Nairobi
    {   14, 5271,   10786,  7, }, // Africa/Ndjamena
    {   14, 5354,   10793,  5, }, // Africa/Niamey
    {   14, 6803,   10798,  7, }, // Africa/Nouakchott
    {   14, 6727,   10805,  8, }, // Africa/Ouagadougou
    {   14, 5225,   10813, 10, }, // Africa/Porto-Novo
    {   14, 4435,   10823,  8, }, // Africa/Sao_Tome
    {   14, 2866,   10831,  6, }, // Africa/Tripoli
    {   14, 6554,   10837,  4, }, // Africa/Tunis
    {   14, 4152,   10841,  7, }, // Africa/Windhoek
    {   14,  132,   10848,  4, }, // America/Adak
    {   14, 3346,   10852,  7, }, // America/Anchorage
    {   14, 5607,   10859,  7, }, // America/Anguilla
    {   14, 5624,   10866,  7, }, // America/Antigua
    {   14, 4554,   10873,  9, }, // America/Araguaina
    {   14,  166,   10882, 11, }, // America/Argentina/Buenos_Aires
    {   14,   91,   10893,  9, }, // America/Argentina/Catamarca
    {   14,  270,   10902,  7, }, // America/Argentina/Cordoba
    {   14,  441,   10909,  4, }, // America/Argentina/Jujuy
    {   14, 7462,   10913,  8, }, // America/Argentina/La_Rioja
    {   14,  565,   10921,  7, }, // America/Argentina/Mendoza
    {   14, 7489,   10928, 11, }, // America/Argentina/Rio_Gallegos
    {   14, 7520,   10939,  5, }, // America/Argentina/Salta
    {   14, 7544,   10944,  8, }, // America/Argentina/San_Juan
    {   14, 5580,   10952,  8, }, // America/Argentina/San_Luis
    {   14, 7571,   10960,  6, }, // America/Argentina/Tucuman
    {   14, 7597,   10966,  4, }, // America/Argentina/Ushuaia
    {   14, 5640,   10970,  5, }, // America/Aruba
    {   14, 4252,   10975,  8, }, // America/Asuncion
    {   14,  237,   10983, 11, }, // America/Atikokan
    {   14, 3679,   10994,  5, }, // America/Bahia
    {   14, 7623,   10999, 14, }, // America/Bahia_Banderas
    {   14, 5654,   11013,  7, }, // America/Barbados
    {   14, 7646,   11020,  3, }, // America/Belem
    {   14, 5407,   11023,  4, }, // America/Belize
    {   14, 7660,   11027, 12, }, // America/Blanc-Sablon
    {   14, 7681,   11039,  9, }, // America/Boa_Vista
    {   14, 4361,   11048,  6, }, // America/Bogota
    {   14, 7699,   11054,  4, }, // America/Boise
    {   14, 7713,   11058, 12, }, // America/Cambridge_Bay
    {   14, 7735,   11070, 11, }, // America/Campo_Grande
    {   14, 3868,   11081,  6, }, // America/Cancun
    {   14, 4694,   11087,  7, }, // America/Caracas
    {   14, 4345,   11094,  5, }, // America/Cayenne
    {   14, 5496,   11099,  6, }, // America/Cayman
    {   14, 2260,   11105,  6, }, // America/Chicago
    {   14, 7756,   11111,  7, }, // America/Chihuahua
    {   14, 7774,   11118, 13, }, // America/Ciudad_Juarez
    {   14, 5422,   11131,  9, }, // America/Costa_Rica
    {   14, 7796,   11140,  7, }, // America/Creston
    {   14, 3791,   11147,  5, }, // America/Cuiaba
    {   14, 5723,   11152,  7, }, // America/Curacao
    {   14, 7812,   11159, 11, }, // America/Danmarkshavn
    {   14, 7833,   11170,  6, }, // America/Dawson
    {   14, 7848,   11176, 10, }, // America/Dawson_Creek
    {   14,  805,   11186,  4, }, // America/Denver
    {   14, 3465,   11190,  7, }, // America/Detroit
    {   14, 5739,   11197,  8, }, // America/Dominica
    {   14,  893,   11205,  9, }, // America/Edmonton
    {   14, 7869,   11214,  7, }, // America/Eirunepe
    {   14, 5441,   11221,  9, }, // America/El_Salvador
    {   14, 7886,   11230, 11, }, // America/Fort_Nelson
    {   14, 7906,   11241,  9, }, // America/Fortaleza
    {   14, 7924,   11250,  8, }, // America/Glace_Bay
    {   14, 6881,   11258,  7, }, // America/Goose_Bay
    {   14, 4612,   11265,  9, }, // America/Grand_Turk
    {   14, 5770,   11274,  7, }, // America/Grenada
    {   14, 5786,   11281,  7, }, // America/Guadeloupe
    {   14, 3760,   11288,  9, }, // America/Guatemala
    {   14, 4841,   11297,  9, }, // America/Guayaquil
    {   14, 6932,   11306,  5, }, // America/Guyana
    {   14, 1939,   11311,  8, }, // America/Halifax
    {   14, 2281,   11319,  6, }, // America/Havana
    {   14, 7942,   11325,  9, }, // America/Hermosillo
    {   14,  348,   11334, 12, }, // America/Indiana/Indianapolis
    {   14,  481,   11346,  5, }, // America/Indiana/Knox
    {   14, 7961,   11351,  6, }, // America/Indiana/Marengo
    {   14, 7985,   11357,  8, }, // America/Indiana/Petersburg
    {   14, 8012,   11365, 17, }, // America/Indiana/Tell_City
    {   14, 8038,   11382,  5, }, // America/Indiana/Vevay
    {   14, 8060,   11387,  7, }, // America/Indiana/Vincennes
    {   14, 8086,   11394,  7, }, // America/Indiana/Winamac
    {   14, 8110,   11401,  7, }, // America/Inuvik
    {   14,  660,   11408,  6, }, // America/Iqaluit
    {   14, 2799,   11414,  7, }, // America/Jamaica
    {   14, 5380,   11421,  4, }, // America/Juneau
    {   14,  521,   11425,  8, }, // America/Kentucky/Louisville
    {   14, 8125,   11433,  9, }, // America/Kentucky/Monticello
    {   14, 5704,   11442,  8, }, // America/Kralendijk
    {   14, 4376,   11450,  6, }, // America/La_Paz
    {   14, 7169,   11456,  4, }, // America/Lima
    {   14, 3239,   11460, 10, }, // America/Los_Angeles
    {   14, 5932,   11470, 16, }, // America/Lower_Princes
    {   14, 8153,   11486,  5, }, // America/Maceio
    {   14, 8168,   11491,  7, }, // America/Managua
    {   14, 1908,   11498,  6, }, // America/Manaus
    {   14, 5897,   11504,  7, }, // America/Marigot
    {   14, 5805,   11511, 10, }, // America/Martinique
    {   14, 8184,   11521,  9, }, // America/Matamoros
    {   14, 2917,   11530,  8, }, // America/Mazatlan
    {   14, 8202,   11538,  8, }, // America/Menominee
    {   14, 8220,   11546,  6, }, // America/Merida
    {   14, 8235,   11552, 10, }, // America/Metlakatla
    {   14, 2949,   11562, 11, }, // America/Mexico_City
    {   14, 4391,   11573,  7, }, // America/Miquelon
    {   14, 8254,   11580,  7, }, // America/Moncton
    {   14, 8270,   11587,  7, }, // America/Monterrey
    {   14, 4098,   11594,  9, }, // America/Montevideo
    {   14, 5824,   11603,  9, }, // America/Montserrat
    {   14, 5481,   11612,  4, }, // America/Nassau
    {   14, 2379,   11616,  7, }, // America/New_York
    {   14, 8288,   11540,  3, }, // America/Nome
    {   14, 1850,   11623,  7, }, // America/Noronha
    {   14, 8301,   11630, 22, }, // America/North_Dakota/Beulah
    {   14, 8329,   11652,  4, }, // America/North_Dakota/Center
    {   14, 8357,   11656,  9, }, // America/North_Dakota/New_Salem
    {   14,  393,   11665,  6, }, // America/Nuuk
    {   14, 8388,   11671,  8, }, // America/Ojinaga
    {   14, 2356,   11679,  4, }, // America/Panama
    {   14, 6173,   11683, 10, }, // America/Paramaribo
    {   14, 2973,   11693,  5, }, // America/Phoenix
    {   14, 3945,   11698, 12, }, // America/Port-au-Prince
    {   14, 5954,   11710, 13, }, // America/Port_of_Spain
    {   14, 8404,   11723, 10, }, // America/Porto_Velho
    {   14, 5843,   11733,  9, }, // America/Puerto_Rico
    {   14, 4030,   11742, 12, }, // America/Punta_Arenas
    {   14, 8424,   11754, 10, }, // America/Rankin_Inlet
    {   14, 8445,   11764,  5, }, // America/Recife
    {   14, 1995,   11769,  6, }, // America/Regina
    {   14, 8460,   11775,  7, }, // America/Resolute
    {   14,  695,   11782,  9, }, // America/Rio_Branco
    {   14, 8477,   11791,  8, }, // America/Santarem
    {   14, 2201,   11799,  8, }, // America/Santiago
    {   14, 6111,   11807, 13, }, // America/Santo_Domingo
    {   14, 1878,   11820,  9, }, // America/Sao_Paulo
    {   14, 6899,   11829, 10, }, // America/Scoresbysund
    {   14, 8494,   11839,  5, }, // America/Sitka
    {   14, 8508,   11844, 14, }, // America/St_Barthelemy
    {   14, 2061,   11858,  9, }, // America/St_Johns
    {   14, 5863,   11867,  9, }, // America/St_Kitts
    {   14, 5880,   11876, 10, }, // America/St_Lucia
    {   14,  855,   11886, 10, }, // America/St_Thomas
    {   14, 5913,   11896, 10, }, // America/St_Vincent
    {   14, 8530,   11906, 10, }, // America/Swift_Current
    {   14, 5461,   11916, 11, }, // America/Tegucigalpa
    {   14, 5756,   11927,  3, }, // America/Thule
    {   14,  313,   11930,  7, }, // America/Tijuana
    {   14,  608,   11937,  7, }, // America/Toronto
    {   14, 5688,   11944,  7, }, // America/Tortola
    {   14, 2093,   11951,  7, }, // America/Vancouver
    {   14, 2144,   11958,  9, }, // America/Whitehorse
    {   14,  734,   11967,  7, }, // America/Winnipeg
    {   14, 8552,   11974,  7, }, // America/Yakutat
    {   14, 6001,   11981,  5, }, // Antarctica/Casey
    {   14, 6094,   11986,  5, }, // Antarctica/Davis
    {   14, 6133,   11991, 16, }, // Antarctica/DumontDUrville
    {   14, 8568,   12007,  7, }, // Antarctica/Macquarie
    {   14, 7066,   12014,  6, }, // Antarctica/Mawson
    {   14, 7113,   12020,  9, }, // Antarctica/McMurdo
    {   14, 8589,   12029,  6, }, // Antarctica/Palmer
    {   14, 7226,   12035,  6, }, // Antarctica/Rothera
    {   14, 7295,   12041,  6, }, // Antarctica/Syowa
    {   14, 8607,   12047,  4, }, // Antarctica/Troll
    {   14, 7374,   12051,  6, }, // Antarctica/Vostok
    {   14, 1429,   12057,  9, }, // Arctic/Longyearbyen
    {   14, 5570,   12066,  3, }, // Asia/Aden
    {   14, 5395,   12069,  6, }, // Asia/Almaty
    {   14, 3968,   12075,  5, }, // Asia/Amman
    {   14, 5511,   12080,  5, }, // Asia/Anadyr
    {   14, 5523,   12085,  5, }, // Asia/Aqtau
    {   14, 5368,   12090,  5, }, // Asia/Aqtobe
    {   14,  964,   12095,  8, }, // Asia/Ashgabat
    {   14, 8624,   12103,  6, }, // Asia/Atyrau
    {   14, 3607,   12109,  5, }, // Asia/Baghdad
    {   14, 5534,   12114,  7, }, // Asia/Bahrain
    {   14, 3653,   11150,  4, }, // Asia/Baku
    {   14, 4466,   12121,  6, }, // Asia/Bangkok
    {   14, 3571,   12127,  7, }, // Asia/Barnaul
    {   14, 4086,   12134,  5, }, // Asia/Beirut
    {   14, 3778,   12139,  5, }, // Asia/Bishkek
    {   14, 5989,   12144,  6, }, // Asia/Brunei
    {   14, 4601,   12150,  5, }, // Asia/Chita
    {   14, 4511,   12155,  7, }, // Asia/Colombo
    {   14, 4540,   12162,  4, }, // Asia/Damascus
    {   14, 1093,   11080,  3, }, // Asia/Dhaka
    {   14, 6192,   12166,  4, }, // Asia/Dili
    {   14, 3596,   12170,  3, }, // Asia/Dubai
    {   14, 6159,   12173,  7, }, // Asia/Dushanbe
    {   14, 8636,   12180,  9, }, // Asia/Famagusta
    {   14, 8651,   12189,  3, }, // Asia/Gaza
    {   14, 4795,   12192, 21, }, // Asia/Hebron
    {   14, 1259,   12213, 16, }, // Asia/Ho_Chi_Minh
    {   14, 2704,   12229,  9, }, // Asia/Hong_Kong
    {   14, 4771,   12238,  4, }, // Asia/Hovd
    {   14, 4184,   12242,  8, }, // Asia/Irkutsk
    {   14, 7006,   12250,  6, }, // Asia/Jakarta
    {   14, 6992,   12256,  9, }, // Asia/Jayapura
    {   14, 1290,   12265,  5, }, // Asia/Jerusalem
    {   14, 3560,   12270,  5, }, // Asia/Kabul
    {   14, 4316,   12275,  9, }, // Asia/Kamchatka
    {   14, 4239,   12284,  6, }, // Asia/Karachi
    {   14, 1185,   12290,  8, }, // Asia/Kathmandu
    {   14, 8661,   12298,  7, }, // Asia/Khandyga
    {   14,  992,   12305,  6, }, // Asia/Kolkata
    {   14, 4197,   12311, 11, }, // Asia/Krasnoyarsk
    {   14, 4859,   12322, 12, }, // Asia/Kuala_Lumpur
    {   14, 5976,   12334,  6, }, // Asia/Kuching
    {   14, 5547,   12340,  6, }, // Asia/Kuwait
    {   14, 1211,   12346,  5, }, // Asia/Macau
    {   14, 4017,   12351,  5, }, // Asia/Magadan
    {   14, 1349,   12356,  6, }, // Asia/Makassar
    {   14, 7182,   12362,  6, }, // Asia/Manila
    {   14, 6920,   12368,  4, }, // Asia/Muscat
    {   14, 2554,   12372,  7, }, // Asia/Nicosia
    {   14, 8675,   12379, 11, }, // Asia/Novokuznetsk
    {   14, 4135,   12390, 10, }, // Asia/Novosibirsk
    {   14, 4229,   12400,  5, }, // Asia/Omsk
    {   14, 7145,   12405,  5, }, // Asia/Oral
    {   14, 6961,   12410,  8, }, // Asia/Phnom_Penh
    {   14, 8693,   12418,  8, }, // Asia/Pontianak
    {   14, 4214,   12426, 10, }, // Asia/Pyongyang
    {   14, 5559,   12436,  3, }, // Asia/Qatar
    {   14, 8708,   12439,  8, }, // Asia/Qostanay
    {   14, 4269,   12447,  9, }, // Asia/Qyzylorda
    {   14, 3584,   12456,  6, }, // Asia/Riyadh
    {   14, 4408,   12462,  6, }, // Asia/Sakhalin
    {   14, 7245,   12468,  6, }, // Asia/Samarkand
    {   14, 3279,   12474,  3, }, // Asia/Seoul
    {   14, 1053,   12477,  6, }, // Asia/Shanghai
    {   14, 3300,   12483,  8, }, // Asia/Singapore
    {   14, 4297,   12491, 12, }, // Asia/Srednekolymsk
    {   14, 3263,   12503,  6, }, // Asia/Taipei
    {   14, 4781,   12509,  5, }, // Asia/Tashkent
    {   14, 3915,   12514,  6, }, // Asia/Tbilisi
    {   14, 2772,   12520,  5, }, // Asia/Tehran
    {   14, 1317,   12525,  5, }, // Asia/Thimphu
    {   14, 2821,   12530,  5, }, // Asia/Tokyo
    {   14, 4572,   12535,  5, }, // Asia/Tomsk
    {   14, 1021,   12540,  9, }, // Asia/Ulaanbaatar
    {   14, 1159,   12549,  6, }, // Asia/Urumqi
    {   14, 8722,   12555,  9, }, // Asia/Ust-Nera
    {   14, 6977,   12564,  8, }, // Asia/Vientiane
    {   14, 4710,   12572, 10, }, // Asia/Vladivostok
    {   14, 4828,   12582,  6, }, // Asia/Yakutsk
    {   14, 1235,   12588,  6, }, // Asia/Yangon
    {   14, 3883,   12594, 10, }, // Asia/Yekaterinburg
    {   14, 3747,   12604,  6, }, // Asia/Yerevan
    {   14, 3663,   12610,  5, }, // Atlantic/Azores
    {   14, 5671,   12615,  6, }, // Atlantic/Bermuda
    {   14, 6628,   12621,  5, }, // Atlantic/Canary
    {   14, 3727,   12626, 12, }, // Atlantic/Cape_Verde
    {   14, 1395,   12638,  4, }, // Atlantic/Faroe
    {   14, 8736,   12642,  6, }, // Atlantic/Madeira
    {   14, 2748,   12648,  8, }, // Atlantic/Reykjavik
    {   14, 7272,   12656, 15, }, // Atlantic/South_Georgia
    {   14, 6821,   12671, 11, }, // Atlantic/St_Helena
    {   14, 6644,   12682,  7, }, // Atlantic/Stanley
    {   14, 1670,   12689,  6, }, // Australia/Adelaide
    {   14, 1635,   12695,  7, }, // Australia/Brisbane
    {   14, 1799,   12702,  9, }, // Australia/Broken_Hill
    {   14, 1583,   12711,  5, }, // Australia/Darwin
    {   14, 3637,   12716,  5, }, // Australia/Eucla
    {   14, 1516,   12721,  6, }, // Australia/Hobart
    {   14, 8753,   12727,  7, }, // Australia/Lindeman
    {   14, 1547,   12734,  8, }, // Australia/Lord_Howe
    {   14, 1727,   12742,  7, }, // Australia/Melbourne
    {   14, 1762,   12749,  4, }, // Australia/Perth
    {   14, 1463,   12753,  5, }, // Australia/Sydney
    {   14, 6441,   12758,  8, }, // Europe/Amsterdam
    {   14, 6216,   12766,  6, }, // Europe/Andorra
    {   14, 3620,   12772,  8, }, // Europe/Astrakhan
    {   14, 2300,   12780,  5, }, // Europe/Athens
    {   14, 6336,   12785,  6, }, // Europe/Belgrade
    {   14, 4757,   12791,  5, }, // Europe/Berlin
    {   14, 6488,   12796, 10, }, // Europe/Bratislava
    {   14, 2167,   12806,  6, }, // Europe/Brussels
    {   14, 3928,   12812,  7, }, // Europe/Bucharest
    {   14, 3806,   12819,  7, }, // Europe/Budapest
    {   14, 8772,   12826,  6, }, // Europe/Busingen
    {   14, 2583,   12832,  8, }, // Europe/Chisinau
    {   14, 6289,   12840,  8, }, // Europe/Copenhagen
    {   14, 2338,   12848,  4, }, // Europe/Dublin
    {   14, 6307,   12852,  8, }, // Europe/Gibraltar
    {   14, 8788,   12860,  6, }, // Europe/Guernsey
    {   14, 6612,   12866,  6, }, // Europe/Helsinki
    {   14, 8804,   12872,  9, }, // Europe/Isle_of_Man
    {   14, 1130,   12881,  7, }, // Europe/Istanbul
    {   14, 8823,   12888,  5, }, // Europe/Jersey
    {   14, 3979,   12893,  9, }, // Europe/Kaliningrad
    {   14, 8929,   12902,  5, }, // Europe/Kirov
    {   14, 2527,   12907,  4, }, // Europe/Kyiv
    {   14, 3213,   12911,  6, }, // Europe/Lisbon
    {   14, 6506,   12917,  9, }, // Europe/Ljubljana
    {   14, 2501,   12926,  4, }, // Europe/London
    {   14, 6365,   12930,  9, }, // Europe/Luxembourg
    {   14, 4892,   12939,  5, }, // Europe/Madrid
    {   14, 6397,   12944,  5, }, // Europe/Malta
    {   14, 6582,   12949,  8, }, // Europe/Mariehamn
    {   14, 3693,   12957,  5, }, // Europe/Minsk
    {   14, 6410,   12962,  6, }, // Europe/Monaco
    {   14, 3537,   12968,  5, }, // Europe/Moscow
    {   14, 6458,   12973,  5, }, // Europe/Oslo
    {   14, 4284,   12978,  5, }, // Europe/Paris
    {   14, 6424,   12983,  9, }, // Europe/Podgorica
    {   14, 6275,   12992,  4, }, // Europe/Prague
    {   14, 8837,   12996,  4, }, // Europe/Riga
    {   14, 6324,   12640,  4, }, // Europe/Rome
    {   14, 4331,   13000,  5, }, // Europe/Samara
    {   14, 6470,   13005, 10, }, // Europe/San_Marino
    {   14, 6245,   13015,  7, }, // Europe/Sarajevo
    {   14, 4451,   13022,  7, }, // Europe/Saratov
    {   14, 8849,   13029,  9, }, // Europe/Simferopol
    {   14, 6383,   13038,  5, }, // Europe/Skopje
    {   14, 6599,   13043,  5, }, // Europe/Sofia
    {   14, 6523,   13048,  8, }, // Europe/Stockholm
    {   14, 8867,   13056,  5, }, // Europe/Tallinn
    {   14, 6202,   13061,  6, }, // Europe/Tirane
    {   14, 8882,   13067, 10, }, // Europe/Ulyanovsk
    {   14, 6352,   13077,  5, }, // Europe/Vaduz
    {   14, 6567,   13082,  9, }, // Europe/Vatican
    {   14, 6231,   13091,  5, }, // Europe/Vienna
    {   14, 8899,   13096,  7, }, // Europe/Vilnius
    {   14, 4727,   13103,  8, }, // Europe/Volgograd
    {   14, 3190,   13111,  5, }, // Europe/Warsaw
    {   14, 6261,   13116,  4, }, // Europe/Zagreb
    {   14, 6540,   13120,  5, }, // Europe/Zurich
    {   14, 5078,   13125, 12, }, // Indian/Antananarivo
    {   14, 6947,   13137,  6, }, // Indian/Chagos
    {   14, 6046,   13143,  7, }, // Indian/Christmas
    {   14, 6063,   13150,  5, }, // Indian/Cocos
    {   14, 5029,   13155,  9, }, // Indian/Comoro
    {   14, 6661,   13164,  9, }, // Indian/Kerguelen
    {   14, 7260,   13173,  4, }, // Indian/Mahe
    {   14, 7050,   13177,  8, }, // Indian/Maldives
    {   14, 4069,   13185,  8, }, // Indian/Mauritius
    {   14, 5098,   13193,  5, }, // Indian/Mayotte
    {   14, 7211,   13198,  7, }, // Indian/Reunion
    {   14, 4422,   13205,  4, }, // Pacific/Apia
    {   14,  932,   13209,  7, }, // Pacific/Auckland
    {   14, 3706,   13216,  8, }, // Pacific/Bougainville
    {   14, 3015,   13224,  6, }, // Pacific/Chatham
    {   14, 3157,   11271,  3, }, // Pacific/Chuuk
    {   14, 2237,   13230,  4, }, // Pacific/Easter
    {   14, 7360,   13234,  5, }, // Pacific/Efate
    {   14, 7327,   13239,  7, }, // Pacific/Fakaofo
    {   14, 3902,   13246,  4, }, // Pacific/Fiji
    {   14, 7343,   13250,  8, }, // Pacific/Funafuti
    {   14, 6678,   13258,  8, }, // Pacific/Galapagos
    {   14, 6696,   13266,  6, }, // Pacific/Gambier
    {   14, 3822,   13272, 11, }, // Pacific/Guadalcanal
    {   14, 6018,   11495,  4, }, // Pacific/Guam
    {   14, 2723,   13283,  8, }, // Pacific/Honolulu
    {   14, 3049,   13291,  8, }, // Pacific/Kanton
    {   14, 3998,   13299, 11, }, // Pacific/Kiritimati
    {   14, 7019,   13310,  5, }, // Pacific/Kosrae
    {   14, 2842,   13315,  8, }, // Pacific/Kwajalein
    {   14, 4877,   13323,  6, }, // Pacific/Majuro
    {   14, 4051,   13329,  8, }, // Pacific/Marquesas
    {   14, 8914,   13337,  6, }, // Pacific/Midway
    {   14, 7084,   13343,  5, }, // Pacific/Nauru
    {   14, 7132,   11616,  4, }, // Pacific/Niue
    {   14, 4168,   13348,  7, }, // Pacific/Norfolk
    {   14, 7098,   13355,  5, }, // Pacific/Noumea
    {   14, 3126,   13360,  9, }, // Pacific/Pago_Pago
    {   14, 7155,   13369,  5, }, // Pacific/Palau
    {   14, 7194,   13374,  7, }, // Pacific/Pitcairn
    {   14, 3096,   13381,  5, }, // Pacific/Pohnpei
    {   14, 4807,   13386, 10, }, // Pacific/Port_Moresby
    {   14, 6076,   13396,  9, }, // Pacific/Rarotonga
    {   14, 6031,   13405,  6, }, // Pacific/Saipan
    {   14, 7312,   13411,  6, }, // Pacific/Tahiti
    {   14, 6712,   13417,  6, }, // Pacific/Tarawa
    {   14, 4583,   13423,  9, }, // Pacific/Tongatapu
    {   14, 7392,   10799,  3, }, // Pacific/Wake
    {   14, 7405,   13432,  5, }, // Pacific/Wallis
    {   42,   14,       0,  6, }, // Africa/Asmara Aragonese/Latin/Spain
    {   42, 4435,       6,  8, }, // Africa/Sao_Tome
    {   42, 4252,      14,  8, }, // America/Asuncion
    {   42,  237,      22,  8, }, // America/Atikokan
    {   42, 7623,      30, 17, }, // America/Bahia_Banderas
    {   42, 3868,      47,  6, }, // America/Cancun
    {   42, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   42, 5723,      66,  7, }, // America/Curacao
    {   42,  481,      73, 13, }, // America/Indiana/Knox
    {   42, 7961,      86, 16, }, // America/Indiana/Marengo
    {   42, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   42, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   42, 8038,     139, 14, }, // America/Indiana/Vevay
    {   42, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   42, 8086,     171, 16, }, // America/Indiana/Winamac
    {   42, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   42, 5932,     207, 22, }, // America/Lower_Princes
    {   42, 8220,     229,  6, }, // America/Merida
    {   42, 1850,     235, 19, }, // America/Noronha
    {   42, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {   42, 8329,     274, 20, }, // America/North_Dakota/Center
    {   42, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {   42,  393,     317,  4, }, // America/Nuuk
    {   42, 6899,     321, 16, }, // America/Scoresbysund
    {   42, 8508,     337, 14, }, // America/St_Barthelemy
    {   42, 2061,     351, 10, }, // America/St_Johns
    {   42, 5863,     361,  9, }, // America/St_Kitts
    {   42, 5880,     370,  9, }, // America/St_Lucia
    {   42,  855,     379, 10, }, // America/St_Thomas
    {   42, 5913,     389, 11, }, // America/St_Vincent
    {   42, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {   42, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {   42, 1185,     448,  9, }, // Asia/Kathmandu
    {   42,  992,     457,  7, }, // Asia/Kolkata
    {   42, 1211,    4321,  5, }, // Asia/Macau
    {   42, 1235,     477,  6, }, // Asia/Yangon
    {   42, 1395,     483,  5, }, // Atlantic/Faroe
    {   42, 6821,     488, 10, }, // Atlantic/St_Helena
    {   42, 2527,     514,  4, }, // Europe/Kyiv
    {   42, 7211,     547,  7, }, // Indian/Reunion
    {   42, 3157,     554,  5, }, // Pacific/Chuuk
    {   42, 3096,     586,  7, }, // Pacific/Pohnpei
    {   43, 6788,   13437,  6, }, // Africa/Abidjan Armenian/Armenian/Armenia
    {   43, 6760,   13443,  4, }, // Africa/Accra
    {   43, 5059,   13447, 10, }, // Africa/Addis_Ababa
    {   43, 7420,   13457,  5, }, // Africa/Algiers
    {   43,   14,   13462,  6, }, // Africa/Asmara
    {   43,   44,   13468,  6, }, // Africa/Bamako
    {   43, 5257,   13474,  5, }, // Africa/Bangui
    {   43, 6746,   13479,  7, }, // Africa/Banjul
    {   43, 7435,   13486,  6, }, // Africa/Bissau
    {   43, 4957,   13492,  8, }, // Africa/Blantyre
    {   43, 5287,   13500,  8, }, // Africa/Brazzaville
    {   43, 4922,   13508, 12, }, // Africa/Bujumbura
    {   43, 2320,   13520,  6, }, // Africa/Cairo
    {   43, 4117,   13526, 10, }, // Africa/Casablanca
    {   43, 7449,   13536,  6, }, // Africa/Ceuta
    {   43, 6773,   13542,  7, }, // Africa/Conakry
    {   43, 6840,   13549,  5, }, // Africa/Dakar
    {   43, 5130,   13554, 12, }, // Africa/Dar_es_Salaam
    {   43, 5043,   13566,  7, }, // Africa/Djibouti
    {   43, 5243,   13573,  6, }, // Africa/Douala
    {   43, 5166,   13579,  8, }, // Africa/El_Aaiun
    {   43, 6853,   13587,  8, }, // Africa/Freetown
    {   43, 4906,   13595,  8, }, // Africa/Gaborone
    {   43, 5015,   13603,  6, }, // Africa/Harare
    {   43, 4479,   13609, 12, }, // Africa/Johannesburg
    {   43, 4499,   13621,  5, }, // Africa/Juba
    {   43, 5151,   13626,  7, }, // Africa/Kampala
    {   43, 4524,   13633,  7, }, // Africa/Khartoum
    {   43, 4973,   13640,  6, }, // Africa/Kigali
    {   43, 5306,   13646,  7, }, // Africa/Kinshasa
    {   43, 4744,   13653,  5, }, // Africa/Lagos
    {   43, 5336,   13658,  7, }, // Africa/Libreville
    {   43, 6869,   13665,  4, }, // Africa/Lome
    {   43, 5211,   13669,  7, }, // Africa/Luanda
    {   43, 4939,   13676, 11, }, // Africa/Lubumbashi
    {   43, 5001,   13687,  7, }, // Africa/Lusaka
    {   43, 5322,   13694,  6, }, // Africa/Malabo
    {   43, 4987,   13700,  8, }, // Africa/Maputo
    {   43, 5197,   13708,  7, }, // Africa/Maseru
    {   43, 5182,   13715,  7, }, // Africa/Mbabane
    {   43, 5113,   13722,  8, }, // Africa/Mogadishu
    {   43, 7034,   13730,  8, }, // Africa/Monrovia
    {   43, 3853,   13738,  7, }, // Africa/Nairobi
    {   43, 5271,   13745,  7, }, // Africa/Ndjamena
    {   43, 5354,   13752,  6, }, // Africa/Niamey
    {   43, 6803,   13758,  8, }, // Africa/Nouakchott
    {   43, 6727,   13766, 11, }, // Africa/Ouagadougou
    {   43, 5225,   13777, 10, }, // Africa/Porto-Novo
    {   43, 4435,   13787,  8, }, // Africa/Sao_Tome
    {   43, 2866,   13795,  7, }, // Africa/Tripoli
    {   43, 6554,   13802,  6, }, // Africa/Tunis
    {   43, 4152,   13808,  8, }, // Africa/Windhoek
    {   43,  132,   13816,  9, }, // America/Adak
    {   43, 3346,   13825,  7, }, // America/Anchorage
    {   43, 5607,   13832,  8, }, // America/Anguilla
    {   43, 5624,   13840,  8, }, // America/Antigua
    {   43, 4554,   13848, 10, }, // America/Araguaina
    {   43,  166,   13858, 13, }, // America/Argentina/Buenos_Aires
    {   43,   91,   13871,  9, }, // America/Argentina/Catamarca
    {   43,  270,   13880,  7, }, // America/Argentina/Cordoba
    {   43,  441,   13887,  7, }, // America/Argentina/Jujuy
    {   43, 7462,   13894,  8, }, // America/Argentina/La_Rioja
    {   43,  565,   13902,  7, }, // America/Argentina/Mendoza
    {   43, 7489,   13909, 12, }, // America/Argentina/Rio_Gallegos
    {   43, 7520,   13921,  5, }, // America/Argentina/Salta
    {   43, 7544,   13926,  9, }, // America/Argentina/San_Juan
    {   43, 5580,   13935,  9, }, // America/Argentina/San_Luis
    {   43, 7571,   13944,  9, }, // America/Argentina/Tucuman
    {   43, 7597,   13953,  8, }, // America/Argentina/Ushuaia
    {   43, 5640,   13961,  6, }, // America/Aruba
    {   43, 4252,   13967,  9, }, // America/Asuncion
    {   43,  237,   13976,  8, }, // America/Atikokan
    {   43, 3679,   13984,  5, }, // America/Bahia
    {   43, 7623,   13989, 14, }, // America/Bahia_Banderas
    {   43, 5654,   14003,  8, }, // America/Barbados
    {   43, 7646,   14011,  5, }, // America/Belem
    {   43, 5407,   14016,  5, }, // America/Belize
    {   43, 7660,   14021, 12, }, // America/Blanc-Sablon
    {   43, 7681,   14033,  9, }, // America/Boa_Vista
    {   43, 4361,   14042,  6, }, // America/Bogota
    {   43, 7699,   14048,  5, }, // America/Boise
    {   43, 7713,   14053, 11, }, // America/Cambridge_Bay
    {   43, 7735,   14064, 13, }, // America/Campo_Grande
    {   43, 3868,   14077,  7, }, // America/Cancun
    {   43, 4694,   14084,  7, }, // America/Caracas
    {   43, 4345,   14091,  5, }, // America/Cayenne
    {   43, 5496,   14096, 14, }, // America/Cayman
    {   43, 2260,   14110,  6, }, // America/Chicago
    {   43, 7756,   14116,  8, }, // America/Chihuahua
    {   43, 7774,   14124, 15, }, // America/Ciudad_Juarez
    {   43, 5422,   14139, 10, }, // America/Costa_Rica
    {   43, 7796,   14149,  7, }, // America/Creston
    {   43, 3791,   14156,  7, }, // America/Cuiaba
    {   43, 5723,   14163,  9, }, // America/Curacao
    {   43, 7812,   14172, 12, }, // America/Danmarkshavn
    {   43, 7833,   14184,  7, }, // America/Dawson
    {   43, 7848,   14191, 12, }, // America/Dawson_Creek
    {   43,  805,   14203,  6, }, // America/Denver
    {   43, 3465,   14209,  7, }, // America/Detroit
    {   43, 5739,   14216,  8, }, // America/Dominica
    {   43,  893,   14224,  8, }, // America/Edmonton
    {   43, 7869,   14232,  9, }, // America/Eirunepe
    {   43, 5441,   14241,  8, }, // America/El_Salvador
    {   43, 7886,   14249, 11, }, // America/Fort_Nelson
    {   43, 7906,   14260,  9, }, // America/Fortaleza
    {   43, 7924,   14269,  9, }, // America/Glace_Bay
    {   43, 6881,   14278,  8, }, // America/Goose_Bay
    {   43, 4612,   14286, 11, }, // America/Grand_Turk
    {   43, 5770,   14297,  7, }, // America/Grenada
    {   43, 5786,   14304, 10, }, // America/Guadeloupe
    {   43, 3760,   14314,  9, }, // America/Guatemala
    {   43, 4841,   14323,  9, }, // America/Guayaquil
    {   43, 6932,   14332,  6, }, // America/Guyana
    {   43, 1939,   14338,  8, }, // America/Halifax
    {   43, 2281,   14346,  6, }, // America/Havana
    {   43, 7942,   14352,  9, }, // America/Hermosillo
    {   43,  348,   14361, 12, }, // America/Indiana/Indianapolis
    {   43,  481,   14373, 13, }, // America/Indiana/Knox
    {   43, 7961,   14386, 16, }, // America/Indiana/Marengo
    {   43, 7985,   14402, 20, }, // America/Indiana/Petersburg
    {   43, 8012,   14422, 17, }, // America/Indiana/Tell_City
    {   43, 8038,   14439, 14, }, // America/Indiana/Vevay
    {   43, 8060,   14453, 16, }, // America/Indiana/Vincennes
    {   43, 8086,   14469, 16, }, // America/Indiana/Winamac
    {   43, 8110,   14485,  7, }, // America/Inuvik
    {   43,  660,   14492,  8, }, // America/Iqaluit
    {   43, 2799,   14500,  7, }, // America/Jamaica
    {   43, 5380,   14507,  5, }, // America/Juneau
    {   43,  521,   14512,  8, }, // America/Kentucky/Louisville
    {   43, 8125,   14520, 19, }, // America/Kentucky/Monticello
    {   43, 5704,   14539, 10, }, // America/Kralendijk
    {   43, 4376,   14549,  6, }, // America/La_Paz
    {   43, 7169,   14555,  4, }, // America/Lima
    {   43, 3239,   14559, 11, }, // America/Los_Angeles
    {   43, 5932,   14570, 22, }, // America/Lower_Princes
    {   43, 8153,   14592,  6, }, // America/Maceio
    {   43, 8168,   14598,  8, }, // America/Managua
    {   43, 1908,   14606,  7, }, // America/Manaus
    {   43, 5897,   14613,  6, }, // America/Marigot
    {   43, 5805,   14619,  9, }, // America/Martinique
    {   43, 8184,   14628,  9, }, // America/Matamoros
    {   43, 2917,   14637,  8, }, // America/Mazatlan
    {   43, 8202,   14645,  8, }, // America/Menominee
    {   43, 8220,   14653,  6, }, // America/Merida
    {   43, 8235,   14659, 10, }, // America/Metlakatla
    {   43, 2949,   14669,  6, }, // America/Mexico_City
    {   43, 4391,   14675,  7, }, // America/Miquelon
    {   43, 8254,   14682,  7, }, // America/Moncton
    {   43, 8270,   14689,  8, }, // America/Monterrey
    {   43, 4098,   14697, 10, }, // America/Montevideo
    {   43, 5824,   14707,  8, }, // America/Montserrat
    {   43, 5481,   14715,  6, }, // America/Nassau
    {   43, 2379,   14721,  9, }, // America/New_York
    {   43, 8288,   14730,  3, }, // America/Nome
    {   43, 1850,   14733,  7, }, // America/Noronha
    {   43, 8301,   14740, 25, }, // America/North_Dakota/Beulah
    {   43, 8329,   14765, 28, }, // America/North_Dakota/Center
    {   43, 8357,   14793, 30, }, // America/North_Dakota/New_Salem
    {   43,  393,   14823,  6, }, // America/Nuuk
    {   43, 8388,   14829,  7, }, // America/Ojinaga
    {   43, 2356,   14836,  6, }, // America/Panama
    {   43, 6173,   14842, 10, }, // America/Paramaribo
    {   43, 2973,   14852,  6, }, // America/Phoenix
    {   43, 3945,   14858, 12, }, // America/Port-au-Prince
    {   43, 5954,   14870, 13, }, // America/Port_of_Spain
    {   43, 8404,   14883, 13, }, // America/Porto_Velho
    {   43, 5843,   14896, 12, }, // America/Puerto_Rico
    {   43, 4030,   14908, 13, }, // America/Punta_Arenas
    {   43, 8424,   14921, 12, }, // America/Rankin_Inlet
    {   43, 8445,   14933,  6, }, // America/Recife
    {   43, 1995,   14939,  7, }, // America/Regina
    {   43, 8460,   14946,  9, }, // America/Resolute
    {   43,  695,   14955, 12, }, // America/Rio_Branco
    {   43, 8477,   14967,  8, }, // America/Santarem
    {   43, 2201,   14975,  8, }, // America/Santiago
    {   43, 6111,   14983, 13, }, // America/Santo_Domingo
    {   43, 1878,   14996, 11, }, // America/Sao_Paulo
    {   43, 6899,   15007, 11, }, // America/Scoresbysund
    {   43, 8494,   15018,  5, }, // America/Sitka
    {   43, 8508,   15023, 12, }, // America/St_Barthelemy
    {   43, 2061,   15035,  9, }, // America/St_Johns
    {   43, 5863,   15044,  9, }, // America/St_Kitts
    {   43, 5880,   15053, 12, }, // America/St_Lucia
    {   43,  855,   15065, 10, }, // America/St_Thomas
    {   43, 5913,   15075, 12, }, // America/St_Vincent
    {   43, 8530,   15087, 12, }, // America/Swift_Current
    {   43, 5461,   15099, 12, }, // America/Tegucigalpa
    {   43, 5756,   15111,  5, }, // America/Thule
    {   43,  313,   15116,  8, }, // America/Tijuana
    {   43,  608,   15124,  7, }, // America/Toronto
    {   43, 5688,   15131,  7, }, // America/Tortola
    {   43, 2093,   15138,  9, }, // America/Vancouver
    {   43, 2144,   15147,  9, }, // America/Whitehorse
    {   43,  734,   15156,  7, }, // America/Winnipeg
    {   43, 8552,   15163,  8, }, // America/Yakutat
    {   43, 6001,   15171,  5, }, // Antarctica/Casey
    {   43, 6094,   15176,  6, }, // Antarctica/Davis
    {   43, 6133,   15182, 17, }, // Antarctica/DumontDUrville
    {   43, 8568,   15199, 14, }, // Antarctica/Macquarie
    {   43, 7066,   15213,  7, }, // Antarctica/Mawson
    {   43, 7113,   15220,  9, }, // Antarctica/McMurdo
    {   43, 8589,   15229,  6, }, // Antarctica/Palmer
    {   43, 7226,   15235,  6, }, // Antarctica/Rothera
    {   43, 7295,   15241,  5, }, // Antarctica/Syowa
    {   43, 8607,   15246,  5, }, // Antarctica/Troll
    {   43, 7374,   15251,  6, }, // Antarctica/Vostok
    {   43, 1429,   15257,  7, }, // Arctic/Longyearbyen
    {   43, 5570,   15264,  4, }, // Asia/Aden
    {   43, 5395,   15268,  6, }, // Asia/Almaty
    {   43, 3968,   15274,  5, }, // Asia/Amman
    {   43, 5511,   15279,  6, }, // Asia/Anadyr
    {   43, 5523,   15285,  6, }, // Asia/Aqtau
    {   43, 5368,   15291,  6, }, // Asia/Aqtobe
    {   43,  964,   15297,  7, }, // Asia/Ashgabat
    {   43, 8624,   15304,  7, }, // Asia/Atyrau
    {   43, 3607,   15311,  6, }, // Asia/Baghdad
    {   43, 5534,   15317,  7, }, // Asia/Bahrain
    {   43, 3653,   15324,  5, }, // Asia/Baku
    {   43, 4466,   15329,  7, }, // Asia/Bangkok
    {   43, 3571,   15336,  8, }, // Asia/Barnaul
    {   43, 4086,   15344,  7, }, // Asia/Beirut
    {   43, 3778,   15351,  6, }, // Asia/Bishkek
    {   43, 5989,   15357,  7, }, // Asia/Brunei
    {   43, 4601,   15364,  4, }, // Asia/Chita
    {   43, 4511,   15368,  7, }, // Asia/Colombo
    {   43, 4540,   15375,  8, }, // Asia/Damascus
    {   43, 1093,   15383,  5, }, // Asia/Dhaka
    {   43, 6192,   15388,  4, }, // Asia/Dili
    {   43, 3596,   15392,  6, }, // Asia/Dubai
    {   43, 6159,   15398,  8, }, // Asia/Dushanbe
    {   43, 8636,   15406, 10, }, // Asia/Famagusta
    {   43, 8651,   15416,  4, }, // Asia/Gaza
    {   43, 4795,   15420,  6, }, // Asia/Hebron
    {   43, 1259,   15426,  7, }, // Asia/Ho_Chi_Minh
    {   43, 2704,   15433,  7, }, // Asia/Hong_Kong
    {   43, 4771,   15440,  4, }, // Asia/Hovd
    {   43, 4184,   15444,  8, }, // Asia/Irkutsk
    {   43, 7006,   15452,  7, }, // Asia/Jakarta
    {   43, 6992,   15459,  8, }, // Asia/Jayapura
    {   43, 1290,   15467,  9, }, // Asia/Jerusalem
    {   43, 3560,   15476,  6, }, // Asia/Kabul
    {   43, 4316,   15482,  8, }, // Asia/Kamchatka
    {   43, 4239,   15490,  6, }, // Asia/Karachi
    {   43, 1185,   15496,  9, }, // Asia/Kathmandu
    {   43, 8661,   15505,  7, }, // Asia/Khandyga
    {   43,  992,   15512,  8, }, // Asia/Kolkata
    {   43, 4197,   15520, 11, }, // Asia/Krasnoyarsk
    {   43, 4859,   15531, 15, }, // Asia/Kuala_Lumpur
    {   43, 5976,   15546,  7, }, // Asia/Kuching
    {   43, 5547,   15553,  7, }, // Asia/Kuwait
    {   43, 1211,   15560,  5, }, // Asia/Macau
    {   43, 4017,   15565,  7, }, // Asia/Magadan
    {   43, 1349,   15572,  7, }, // Asia/Makassar
    {   43, 7182,   15579,  6, }, // Asia/Manila
    {   43, 6920,   15585,  6, }, // Asia/Muscat
    {   43, 2554,   15591,  7, }, // Asia/Nicosia
    {   43, 8675,   15598, 12, }, // Asia/Novokuznetsk
    {   43, 4135,   15610, 11, }, // Asia/Novosibirsk
    {   43, 4229,   15621,  4, }, // Asia/Omsk
    {   43, 7145,   15625,  7, }, // Asia/Oral
    {   43, 6961,   15632,  7, }, // Asia/Phnom_Penh
    {   43, 8693,   15639,  9, }, // Asia/Pontianak
    {   43, 4214,   15648,  7, }, // Asia/Pyongyang
    {   43, 5559,   15655,  5, }, // Asia/Qatar
    {   43, 8708,   15660,  8, }, // Asia/Qostanay
    {   43, 4269,   15668,  9, }, // Asia/Qyzylorda
    {   43, 3584,   15677,  7, }, // Asia/Riyadh
    {   43, 4408,   15684,  7, }, // Asia/Sakhalin
    {   43, 7245,   15691,  9, }, // Asia/Samarkand
    {   43, 3279,   15700,  5, }, // Asia/Seoul
    {   43, 1053,   15705,  6, }, // Asia/Shanghai
    {   43, 3300,   15711,  9, }, // Asia/Singapore
    {   43, 4297,   15720, 13, }, // Asia/Srednekolymsk
    {   43, 3263,   15733,  6, }, // Asia/Taipei
    {   43, 4781,   15739,  7, }, // Asia/Tashkent
    {   43, 3915,   15746,  7, }, // Asia/Tbilisi
    {   43, 2772,   15753,  6, }, // Asia/Tehran
    {   43, 1317,   15759,  8, }, // Asia/Thimphu
    {   43, 2821,   15767,  5, }, // Asia/Tokyo
    {   43, 4572,   15772,  5, }, // Asia/Tomsk
    {   43, 1021,   15777, 11, }, // Asia/Ulaanbaatar
    {   43, 1159,   15788,  8, }, // Asia/Urumqi
    {   43, 8722,   15796,  9, }, // Asia/Ust-Nera
    {   43, 6977,   15805,  8, }, // Asia/Vientiane
    {   43, 4710,   15813, 11, }, // Asia/Vladivostok
    {   43, 4828,   15824,  8, }, // Asia/Yakutsk
    {   43, 1235,   15832,  7, }, // Asia/Yangon
    {   43, 3883,   15839, 13, }, // Asia/Yekaterinburg
    {   43, 3747,   15852,  5, }, // Asia/Yerevan
    {   43, 3663,   15857, 15, }, // Atlantic/Azores
    {   43, 5671,   15872, 10, }, // Atlantic/Bermuda
    {   43, 6628,   15882, 16, }, // Atlantic/Canary
    {   43, 3727,   15898, 10, }, // Atlantic/Cape_Verde
    {   43, 1395,   15908, 16, }, // Atlantic/Faroe
    {   43, 8736,   15924,  7, }, // Atlantic/Madeira
    {   43, 2748,   15931,  9, }, // Atlantic/Reykjavik
    {   43, 7272,   15940, 16, }, // Atlantic/South_Georgia
    {   43, 6821,   15956, 18, }, // Atlantic/St_Helena
    {   43, 6644,   15974,  6, }, // Atlantic/Stanley
    {   43, 1670,   15980,  8, }, // Australia/Adelaide
    {   43, 1635,   15988,  7, }, // Australia/Brisbane
    {   43, 1799,   15995, 10, }, // Australia/Broken_Hill
    {   43, 1583,   16005,  6, }, // Australia/Darwin
    {   43, 3637,   16011,  6, }, // Australia/Eucla
    {   43, 1516,   16017,  6, }, // Australia/Hobart
    {   43, 8753,   16023,  8, }, // Australia/Lindeman
    {   43, 1547,   16031, 14, }, // Australia/Lord_Howe
    {   43, 1727,   16045,  8, }, // Australia/Melbourne
    {   43, 1762,   16053,  4, }, // Australia/Perth
    {   43, 1463,   16057,  6, }, // Australia/Sydney
    {   43, 6441,   16063,  9, }, // Europe/Amsterdam
    {   43, 6216,   16072,  7, }, // Europe/Andorra
    {   43, 3620,   16079,  8, }, // Europe/Astrakhan
    {   43, 2300,   16087,  5, }, // Europe/Athens
    {   43, 6336,   16092,  7, }, // Europe/Belgrade
    {   43, 4757,   16099,  6, }, // Europe/Berlin
    {   43, 6488,   16105, 10, }, // Europe/Bratislava
    {   43, 2167,   16115,  8, }, // Europe/Brussels
    {   43, 3928,   16123,  9, }, // Europe/Bucharest
    {   43, 3806,   16132,  9, }, // Europe/Budapest
    {   43, 8772,   16141, 10, }, // Europe/Busingen
    {   43, 2583,   16151,  5, }, // Europe/Chisinau
    {   43, 6289,   16156, 10, }, // Europe/Copenhagen
    {   43, 2338,   16166,  7, }, // Europe/Dublin
    {   43, 6307,   16173,  9, }, // Europe/Gibraltar
    {   43, 8788,   16182,  6, }, // Europe/Guernsey
    {   43, 6612,   16188,  8, }, // Europe/Helsinki
    {   43, 8804,   16196,  8, }, // Europe/Isle_of_Man
    {   43, 1130,   16204,  8, }, // Europe/Istanbul
    {   43, 8823,   16212,  5, }, // Europe/Jersey
    {   43, 3979,   16217, 11, }, // Europe/Kaliningrad
    {   43, 8929,   16228,  5, }, // Europe/Kirov
    {   43, 2527,   16233,  3, }, // Europe/Kyiv
    {   43, 3213,   16236,  7, }, // Europe/Lisbon
    {   43, 6506,   16243, 10, }, // Europe/Ljubljana
    {   43, 2501,   16253,  6, }, // Europe/London
    {   43, 6365,   16259, 13, }, // Europe/Luxembourg
    {   43, 4892,   16272,  6, }, // Europe/Madrid
    {   43, 6397,   16278,  5, }, // Europe/Malta
    {   43, 6582,   16283,  9, }, // Europe/Mariehamn
    {   43, 3693,   16292,  5, }, // Europe/Minsk
    {   43, 6410,   16297,  6, }, // Europe/Monaco
    {   43, 3537,   16303,  6, }, // Europe/Moscow
    {   43, 6458,   16309,  4, }, // Europe/Oslo
    {   43, 4284,   16313,  5, }, // Europe/Paris
    {   43, 6424,   16318,  9, }, // Europe/Podgorica
    {   43, 6275,   16327,  5, }, // Europe/Prague
    {   43, 8837,   16332,  4, }, // Europe/Riga
    {   43, 6324,   16336,  4, }, // Europe/Rome
    {   43, 4331,   16340,  6, }, // Europe/Samara
    {   43, 6470,   16346, 10, }, // Europe/San_Marino
    {   43, 6245,   16356,  6, }, // Europe/Sarajevo
    {   43, 4451,   16362,  7, }, // Europe/Saratov
    {   43, 8849,   16369, 10, }, // Europe/Simferopol
    {   43, 6383,   16379,  6, }, // Europe/Skopje
    {   43, 6599,   16385,  5, }, // Europe/Sofia
    {   43, 6523,   16390,  8, }, // Europe/Stockholm
    {   43, 8867,   16398,  6, }, // Europe/Tallinn
    {   43, 6202,   16404,  6, }, // Europe/Tirane
    {   43, 8882,   16410, 10, }, // Europe/Ulyanovsk
    {   43, 6352,   16420,  6, }, // Europe/Vaduz
    {   43, 6567,   16426,  7, }, // Europe/Vatican
    {   43, 6231,   16433,  6, }, // Europe/Vienna
    {   43, 8899,   16439,  8, }, // Europe/Vilnius
    {   43, 4727,   16447,  9, }, // Europe/Volgograd
    {   43, 3190,   16456,  7, }, // Europe/Warsaw
    {   43, 6261,   16463,  6, }, // Europe/Zagreb
    {   43, 6540,   16469,  7, }, // Europe/Zurich
    {   43, 5078,   16476, 13, }, // Indian/Antananarivo
    {   43, 6947,   16489,  5, }, // Indian/Chagos
    {   43, 6046,   16494, 18, }, // Indian/Christmas
    {   43, 6063,   16512, 16, }, // Indian/Cocos
    {   43, 5029,   16528, 16, }, // Indian/Comoro
    {   43, 6661,   16544,  8, }, // Indian/Kerguelen
    {   43, 7260,   16552,  3, }, // Indian/Mahe
    {   43, 7050,   16555,  9, }, // Indian/Maldives
    {   43, 4069,   16564,  9, }, // Indian/Mauritius
    {   43, 5098,   16573,  5, }, // Indian/Mayotte
    {   43, 7211,   16578,  9, }, // Indian/Reunion
    {   43, 4422,   16587,  4, }, // Pacific/Apia
    {   43,  932,   16591,  6, }, // Pacific/Auckland
    {   43, 3706,   16597,  9, }, // Pacific/Bougainville
    {   43, 3015,   16606, 10, }, // Pacific/Chatham
    {   43, 3157,   16616,  6, }, // Pacific/Chuuk
    {   43, 2237,   16622, 10, }, // Pacific/Easter
    {   43, 7360,   16632,  5, }, // Pacific/Efate
    {   43, 7327,   16637,  7, }, // Pacific/Fakaofo
    {   43, 3902,   16644,  4, }, // Pacific/Fiji
    {   43, 7343,   16648, 10, }, // Pacific/Funafuti
    {   43, 6678,   16658, 20, }, // Pacific/Galapagos
    {   43, 6696,   16678, 14, }, // Pacific/Gambier
    {   43, 3822,   16692, 12, }, // Pacific/Guadalcanal
    {   43, 6018,   16704,  5, }, // Pacific/Guam
    {   43, 2723,   16709, 10, }, // Pacific/Honolulu
    {   43, 3049,   16719, 14, }, // Pacific/Kanton
    {   43, 3998,   16733, 10, }, // Pacific/Kiritimati
    {   43, 7019,   16743,  6, }, // Pacific/Kosrae
    {   43, 2842,   16749,  9, }, // Pacific/Kwajalein
    {   43, 4877,   16758,  7, }, // Pacific/Majuro
    {   43, 4051,   16765, 17, }, // Pacific/Marquesas
    {   43, 8914,   16782, 12, }, // Pacific/Midway
    {   43, 7084,   16794,  7, }, // Pacific/Nauru
    {   43, 7132,   16801,  5, }, // Pacific/Niue
    {   43, 4168,   16806,  7, }, // Pacific/Norfolk
    {   43, 7098,   16813,  6, }, // Pacific/Noumea
    {   43, 3126,   16819,  9, }, // Pacific/Pago_Pago
    {   43, 7155,   16828,  6, }, // Pacific/Palau
    {   43, 7194,   16834,  7, }, // Pacific/Pitcairn
    {   43, 3096,   16841,  6, }, // Pacific/Pohnpei
    {   43, 4807,   16847, 11, }, // Pacific/Port_Moresby
    {   43, 6076,   16858,  9, }, // Pacific/Rarotonga
    {   43, 6031,   16867,  6, }, // Pacific/Saipan
    {   43, 7312,   16873,  5, }, // Pacific/Tahiti
    {   43, 6712,   16878,  6, }, // Pacific/Tarawa
    {   43, 4583,   16884, 10, }, // Pacific/Tongatapu
    {   43, 7392,   16894, 10, }, // Pacific/Wake
    {   43, 7405,   16904,  6, }, // Pacific/Wallis
    {   44, 6788,   16910,  7, }, // Africa/Abidjan Assamese/Bangla/India
    {   44, 6760,   16917,  5, }, // Africa/Accra
    {   44, 5059,   16922, 12, }, // Africa/Addis_Ababa
    {   44, 7420,   16934, 10, }, // Africa/Algiers
    {   44,   14,   16944,  7, }, // Africa/Asmara
    {   44,   44,   16951,  6, }, // Africa/Bamako
    {   44, 5257,   16957,  6, }, // Africa/Bangui
    {   44, 6746,   16963,  7, }, // Africa/Banjul
    {   44, 7435,   16970,  5, }, // Africa/Bissau
    {   44, 4957,   16975, 12, }, // Africa/Blantyre
    {   44, 5287,   16987, 10, }, // Africa/Brazzaville
    {   44, 4922,   16997, 10, }, // Africa/Bujumbura
    {   44, 2320,   17007,  5, }, // Africa/Cairo
    {   44, 4117,   17012, 11, }, // Africa/Casablanca
    {   44, 7449,   17023,  5, }, // Africa/Ceuta
    {   44, 6773,   17028,  8, }, // Africa/Conakry
    {   44, 6840,   17036,  5, }, // Africa/Dakar
    {   44, 5130,   17041, 12, }, // Africa/Dar_es_Salaam
    {   44, 5043,   17053,  6, }, // Africa/Djibouti
    {   44, 5243,   17059,  6, }, // Africa/Douala
    {   44, 5166,   17065,  9, }, // Africa/El_Aaiun
    {   44, 6853,   17074,  8, }, // Africa/Freetown
    {   44, 4906,   17082,  7, }, // Africa/Gaborone
    {   44, 5015,   17089,  6, }, // Africa/Harare
    {   44, 4479,   17095, 12, }, // Africa/Johannesburg
    {   44, 4499,   17107,  4, }, // Africa/Juba
    {   44, 5151,   17111,  8, }, // Africa/Kampala
    {   44, 4524,   17119,  7, }, // Africa/Khartoum
    {   44, 4973,   17126,  6, }, // Africa/Kigali
    {   44, 5306,   17132,  7, }, // Africa/Kinshasa
    {   44, 4744,   17139,  5, }, // Africa/Lagos
    {   44, 5336,   17144, 10, }, // Africa/Libreville
    {   44, 6869,   17154,  3, }, // Africa/Lome
    {   44, 5211,   17157,  8, }, // Africa/Luanda
    {   44, 4939,   17165, 10, }, // Africa/Lubumbashi
    {   44, 5001,   17175,  6, }, // Africa/Lusaka
    {   44, 5322,   17181,  6, }, // Africa/Malabo
    {   44, 4987,   17187,  6, }, // Africa/Maputo
    {   44, 5197,   17193,  6, }, // Africa/Maseru
    {   44, 5182,   17199,  8, }, // Africa/Mbabane
    {   44, 5113,   17207,  8, }, // Africa/Mogadishu
    {   44, 7034,   17215, 10, }, // Africa/Monrovia
    {   44, 3853,   17225,  7, }, // Africa/Nairobi
    {   44, 5271,   17232,  9, }, // Africa/Ndjamena
    {   44, 5354,   17241,  7, }, // Africa/Niamey
    {   44, 6803,   17248,  7, }, // Africa/Nouakchott
    {   44, 6727,   17255,  7, }, // Africa/Ouagadougou
    {   44, 5225,   17262, 11, }, // Africa/Porto-Novo
    {   44, 4435,   17273,  8, }, // Africa/Sao_Tome
    {   44, 2866,   17281,  8, }, // Africa/Tripoli
    {   44, 6554,   17289,  5, }, // Africa/Tunis
    {   44, 4152,   17294,  8, }, // Africa/Windhoek
    {   44,  132,   17302,  4, }, // America/Adak
    {   44, 3346,   17306,  7, }, // America/Anchorage
    {   44, 5607,   17313,  7, }, // America/Anguilla
    {   44, 5624,   17320,  9, }, // America/Antigua
    {   44, 4554,   17329, 10, }, // America/Araguaina
    {   44,  166,   17339, 12, }, // America/Argentina/Buenos_Aires
    {   44,   91,   17351, 10, }, // America/Argentina/Catamarca
    {   44,  270,   17361,  8, }, // America/Argentina/Cordoba
    {   44,  441,   17369,  5, }, // America/Argentina/Jujuy
    {   44, 7462,   17374,  8, }, // America/Argentina/La_Rioja
    {   44,  565,   17382,  8, }, // America/Argentina/Mendoza
    {   44, 7489,   17390, 12, }, // America/Argentina/Rio_Gallegos
    {   44, 7520,   17402,  6, }, // America/Argentina/Salta
    {   44, 7544,   17408,  9, }, // America/Argentina/San_Juan
    {   44, 5580,   17417,  8, }, // America/Argentina/San_Luis
    {   44, 7571,   17425,  6, }, // America/Argentina/Tucuman
    {   44, 7597,   17431,  8, }, // America/Argentina/Ushuaia
    {   44, 5640,   17439,  5, }, // America/Aruba
    {   44, 4252,   17444,  9, }, // America/Asuncion
    {   44,  237,   17453,  8, }, // America/Atikokan
    {   44, 3679,   17461,  7, }, // America/Bahia
    {   44, 7623,   17468, 17, }, // America/Bahia_Banderas
    {   44, 5654,   17485,  9, }, // America/Barbados
    {   44, 7646,   17494,  5, }, // America/Belem
    {   44, 5407,   17499,  5, }, // America/Belize
    {   44, 7660,   17504, 13, }, // America/Blanc-Sablon
    {   44, 7681,   17517, 11, }, // America/Boa_Vista
    {   44, 4361,   17528,  6, }, // America/Bogota
    {   44, 7699,   17534,  5, }, // America/Boise
    {   44, 7713,   17539, 15, }, // America/Cambridge_Bay
    {   44, 7735,   17554, 15, }, // America/Campo_Grande
    {   44, 3868,   17569,  6, }, // America/Cancun
    {   44, 4694,   17575,  7, }, // America/Caracas
    {   44, 4345,   17582,  6, }, // America/Cayenne
    {   44, 5496,   17588,  6, }, // America/Cayman
    {   44, 2260,   17594,  6, }, // America/Chicago
    {   44, 7756,   17600,  9, }, // America/Chihuahua
    {   44, 7774,   17609, 12, }, // America/Ciudad_Juarez
    {   44, 5422,   17621, 11, }, // America/Costa_Rica
    {   44, 7796,   17632,  8, }, // America/Creston
    {   44, 3791,   17640,  6, }, // America/Cuiaba
    {   44, 5723,   17646,  7, }, // America/Curacao
    {   44, 7812,   17653, 12, }, // America/Danmarkshavn
    {   44, 7833,   17665,  5, }, // America/Dawson
    {   44, 7848,   17670, 11, }, // America/Dawson_Creek
    {   44,  805,   17681,  6, }, // America/Denver
    {   44, 3465,   17687,  8, }, // America/Detroit
    {   44, 5739,   17695,  7, }, // America/Dominica
    {   44,  893,   17702,  7, }, // America/Edmonton
    {   44, 7869,   17709,  7, }, // America/Eirunepe
    {   44, 5441,   17716, 11, }, // America/El_Salvador
    {   44, 7886,   17727, 11, }, // America/Fort_Nelson
    {   44, 7906,   17738, 10, }, // America/Fortaleza
    {   44, 7924,   17748, 12, }, // America/Glace_Bay
    {   44, 6881,   17760, 10, }, // America/Goose_Bay
    {   44, 4612,   17770, 13, }, // America/Grand_Turk
    {   44, 5770,   17783,  8, }, // America/Grenada
    {   44, 5786,   17791,  9, }, // America/Guadeloupe
    {   44, 3760,   17800, 10, }, // America/Guatemala
    {   44, 4841,   17810,  9, }, // America/Guayaquil
    {   44, 6932,   17819,  7, }, // America/Guyana
    {   44, 1939,   17826,  9, }, // America/Halifax
    {   44, 2281,   17835,  6, }, // America/Havana
    {   44, 7942,   17841, 12, }, // America/Hermosillo
    {   44,  348,   17853, 15, }, // America/Indiana/Indianapolis
    {   44,  481,   17868, 16, }, // America/Indiana/Knox
    {   44, 7961,   17884, 19, }, // America/Indiana/Marengo
    {   44, 7985,   17903, 24, }, // America/Indiana/Petersburg
    {   44, 8012,   17927, 20, }, // America/Indiana/Tell_City
    {   44, 8038,   17947, 16, }, // America/Indiana/Vevay
    {   44, 8060,   17963, 20, }, // America/Indiana/Vincennes
    {   44, 8086,   17983, 19, }, // America/Indiana/Winamac
    {   44, 8110,   18002,  6, }, // America/Inuvik
    {   44,  660,   18008,  7, }, // America/Iqaluit
    {   44, 2799,   18015,  7, }, // America/Jamaica
    {   44, 5380,   18022,  5, }, // America/Juneau
    {   44,  521,   18027,  8, }, // America/Kentucky/Louisville
    {   44, 8125,   18035, 19, }, // America/Kentucky/Monticello
    {   44, 5704,   18054, 13, }, // America/Kralendijk
    {   44, 4376,   18067,  6, }, // America/La_Paz
    {   44, 7169,   18073,  4, }, // America/Lima
    {   44, 3239,   18077, 11, }, // America/Los_Angeles
    {   44, 5932,   18088, 25, }, // America/Lower_Princes
    {   44, 8153,   18113,  5, }, // America/Maceio
    {   44, 8168,   18118,  8, }, // America/Managua
    {   44, 1908,   18126,  5, }, // America/Manaus
    {   44, 5897,   18131,  7, }, // America/Marigot
    {   44, 5805,   18138,  9, }, // America/Martinique
    {   44, 8184,   18147,  9, }, // America/Matamoros
    {   44, 2917,   18156, 10, }, // America/Mazatlan
    {   44, 8202,   18166,  8, }, // America/Menominee
    {   44, 8220,   18174,  6, }, // America/Merida
    {   44, 8235,   18180, 12, }, // America/Metlakatla
    {   44, 2949,   18192, 13, }, // America/Mexico_City
    {   44, 4391,   18205,  9, }, // America/Miquelon
    {   44, 8254,   18214,  7, }, // America/Moncton
    {   44, 8270,   18221,  9, }, // America/Monterrey
    {   44, 4098,   18230, 11, }, // America/Montevideo
    {   44, 5824,   18241,  9, }, // America/Montserrat
    {   44, 5481,   18250,  5, }, // America/Nassau
    {   44, 2379,   18255, 10, }, // America/New_York
    {   44, 8288,   18168,  3, }, // America/Nome
    {   44, 1850,   18265,  9, }, // America/Noronha
    {   44, 8301,   18274, 20, }, // America/North_Dakota/Beulah
    {   44, 8329,   18294, 21, }, // America/North_Dakota/Center
    {   44, 8357,   18315, 23, }, // America/North_Dakota/New_Salem
    {   44,  393,   18338,  3, }, // America/Nuuk
    {   44, 8388,   18341,  8, }, // America/Ojinaga
    {   44, 2356,   18349,  6, }, // America/Panama
    {   44, 6173,   18355, 10, }, // America/Paramaribo
    {   44, 2973,   18365,  7, }, // America/Phoenix
    {   44, 3945,   18372, 15, }, // America/Port-au-Prince
    {   44, 5954,   18387, 15, }, // America/Port_of_Spain
    {   44, 8404,   18402, 14, }, // America/Porto_Velho
    {   44, 5843,   18416, 13, }, // America/Puerto_Rico
    {   44, 4030,   18429, 13, }, // America/Punta_Arenas
    {   44, 8424,   18442, 12, }, // America/Rankin_Inlet
    {   44, 8445,   18454,  6, }, // America/Recife
    {   44, 1995,   18460,  6, }, // America/Regina
    {   44, 8460,   18466,  8, }, // America/Resolute
    {   44,  695,   18474, 12, }, // America/Rio_Branco
    {   44, 8477,   18486,  9, }, // America/Santarem
    {   44, 2201,   18495, 11, }, // America/Santiago
    {   44, 6111,   18506, 14, }, // America/Santo_Domingo
    {   44, 1878,   18520,  9, }, // America/Sao_Paulo
    {   44, 6899,   18529, 13, }, // America/Scoresbysund
    {   44, 8494,   18542,  8, }, // America/Sitka
    {   44, 8508,   18550, 16, }, // America/St_Barthelemy
    {   44, 2061,   18566, 11, }, // America/St_Johns
    {   44, 5863,   18577, 11, }, // America/St_Kitts
    {   44, 5880,   18588, 14, }, // America/St_Lucia
    {   44,  855,   18602, 11, }, // America/St_Thomas
    {   44, 5913,   18613, 15, }, // America/St_Vincent
    {   44, 8530,   18628, 14, }, // America/Swift_Current
    {   44, 5461,   18642, 12, }, // America/Tegucigalpa
    {   44, 5756,   18654,  6, }, // America/Thule
    {   44,  313,   18660,  8, }, // America/Tijuana
    {   44,  608,   18668,  8, }, // America/Toronto
    {   44, 5688,   18676,  8, }, // America/Tortola
    {   44, 2093,   18684,  8, }, // America/Vancouver
    {   44, 2144,   18692, 10, }, // America/Whitehorse
    {   44,  734,   18702,  7, }, // America/Winnipeg
    {   44, 8552,   18709,  8, }, // America/Yakutat
    {   44, 6001,   18717,  4, }, // Antarctica/Casey
    {   44, 6094,   18721,  5, }, // Antarctica/Davis
    {   44, 6133,   18726, 18, }, // Antarctica/DumontDUrville
    {   44, 8568,   18744, 11, }, // Antarctica/Macquarie
    {   44, 7066,   18755,  5, }, // Antarctica/Mawson
    {   44, 7113,   18760,  9, }, // Antarctica/McMurdo
    {   44, 8589,   18769,  4, }, // Antarctica/Palmer
    {   44, 7226,   18773,  6, }, // Antarctica/Rothera
    {   44, 7295,   18779,  6, }, // Antarctica/Syowa
    {   44, 8607,   18785,  5, }, // Antarctica/Troll
    {   44, 7374,   18790,  6, }, // Antarctica/Vostok
    {   44, 1429,   18796, 12, }, // Arctic/Longyearbyen
    {   44, 5570,   18808,  4, }, // Asia/Aden
    {   44, 5395,   18812,  6, }, // Asia/Almaty
    {   44, 3968,   18818,  6, }, // Asia/Amman
    {   44, 5511,   18824,  6, }, // Asia/Anadyr
    {   44, 5523,   18830, 10, }, // Asia/Aqtau
    {   44, 5368,   18840,  7, }, // Asia/Aqtobe
    {   44,  964,   18847, 11, }, // Asia/Ashgabat
    {   44, 8624,   18858,  5, }, // Asia/Atyrau
    {   44, 3607,   18863,  6, }, // Asia/Baghdad
    {   44, 5534,   18869,  7, }, // Asia/Bahrain
    {   44, 3653,   17644,  4, }, // Asia/Baku
    {   44, 4466,   18876,  5, }, // Asia/Bangkok
    {   44, 3571,   18881,  7, }, // Asia/Barnaul
    {   44, 4086,   18888,  6, }, // Asia/Beirut
    {   44, 3778,   18894, 10, }, // Asia/Bishkek
    {   44, 5989,   18904,  7, }, // Asia/Brunei
    {   44, 4601,   18911,  4, }, // Asia/Chita
    {   44, 4511,   18915,  6, }, // Asia/Colombo
    {   44, 4540,   18921,  9, }, // Asia/Damascus
    {   44, 1093,   18930,  4, }, // Asia/Dhaka
    {   44, 6192,   18934,  4, }, // Asia/Dili
    {   44, 3596,   18938,  5, }, // Asia/Dubai
    {   44, 6159,   18943,  7, }, // Asia/Dushanbe
    {   44, 8636,   18950, 10, }, // Asia/Famagusta
    {   44, 8651,   18960,  4, }, // Asia/Gaza
    {   44, 4795,   18964,  7, }, // Asia/Hebron
    {   44, 1259,   18971, 14, }, // Asia/Ho_Chi_Minh
    {   44, 2704,   18985,  5, }, // Asia/Hong_Kong
    {   44, 4771,   18990,  6, }, // Asia/Hovd
    {   44, 4184,   18996, 10, }, // Asia/Irkutsk
    {   44, 7006,   19006,  8, }, // Asia/Jakarta
    {   44, 6992,   19014,  9, }, // Asia/Jayapura
    {   44, 1290,   19023,  9, }, // Asia/Jerusalem
    {   44, 3560,   19032,  5, }, // Asia/Kabul
    {   44, 4316,   19037,  8, }, // Asia/Kamchatka
    {   44, 4239,   19045,  5, }, // Asia/Karachi
    {   44, 1185,   19050,  9, }, // Asia/Kathmandu
    {   44, 8661,   19059,  8, }, // Asia/Khandyga
    {   44,  992,   19067,  6, }, // Asia/Kolkata
    {   44, 4197,   19073, 15, }, // Asia/Krasnoyarsk
    {   44, 4859,   19088, 12, }, // Asia/Kuala_Lumpur
    {   44, 5976,   19100,  5, }, // Asia/Kuching
    {   44, 5547,   19105,  6, }, // Asia/Kuwait
    {   44, 1211,   19111,  5, }, // Asia/Macau
    {   44, 4017,   19116,  7, }, // Asia/Magadan
    {   44, 1349,   19123,  7, }, // Asia/Makassar
    {   44, 7182,   19130,  6, }, // Asia/Manila
    {   44, 6920,   19136,  7, }, // Asia/Muscat
    {   44, 2554,   19143,  9, }, // Asia/Nicosia
    {   44, 8675,   19152, 14, }, // Asia/Novokuznetsk
    {   44, 4135,   19166, 13, }, // Asia/Novosibirsk
    {   44, 4229,   19179,  5, }, // Asia/Omsk
    {   44, 7145,   19184,  5, }, // Asia/Oral
    {   44, 6961,   19189, 10, }, // Asia/Phnom_Penh
    {   44, 8693,   19199, 11, }, // Asia/Pontianak
    {   44, 4214,   19210,  8, }, // Asia/Pyongyang
    {   44, 5559,   19218,  5, }, // Asia/Qatar
    {   44, 8708,   19223,  8, }, // Asia/Qostanay
    {   44, 4269,   19231, 10, }, // Asia/Qyzylorda
    {   44, 3584,   19241,  6, }, // Asia/Riyadh
    {   44, 4408,   19247,  7, }, // Asia/Sakhalin
    {   44, 7245,   19254, 10, }, // Asia/Samarkand
    {   44, 3279,   19264,  4, }, // Asia/Seoul
    {   44, 1053,   19268,  6, }, // Asia/Shanghai
    {   44, 3300,   19274,  8, }, // Asia/Singapore
    {   44, 4297,   19282, 15, }, // Asia/Srednekolymsk
    {   44, 3263,   19297,  6, }, // Asia/Taipei
    {   44, 4781,   19303,  8, }, // Asia/Tashkent
    {   44, 3915,   19311,  7, }, // Asia/Tbilisi
    {   44, 2772,   19318,  6, }, // Asia/Tehran
    {   44, 1317,   19324,  6, }, // Asia/Thimphu
    {   44, 2821,   19330,  5, }, // Asia/Tokyo
    {   44, 4572,   19335,  8, }, // Asia/Tomsk
    {   44, 1021,   19343,  9, }, // Asia/Ulaanbaatar
    {   44, 1159,   19352,  8, }, // Asia/Urumqi
    {   44, 8722,   19360,  9, }, // Asia/Ust-Nera
    {   44, 6977,   19369, 13, }, // Asia/Vientiane
    {   44, 4710,   19382, 12, }, // Asia/Vladivostok
    {   44, 4828,   19394, 10, }, // Asia/Yakutsk
    {   44, 1235,   19404,  6, }, // Asia/Yangon
    {   44, 3883,   19410, 15, }, // Asia/Yekaterinburg
    {   44, 3747,   19425,  8, }, // Asia/Yerevan
    {   44, 3663,   19433,  6, }, // Atlantic/Azores
    {   44, 5671,   19439,  8, }, // Atlantic/Bermuda
    {   44, 6628,   19447,  6, }, // Atlantic/Canary
    {   44, 3727,   19453, 10, }, // Atlantic/Cape_Verde
    {   44, 1395,   19463,  4, }, // Atlantic/Faroe
    {   44, 8736,   19467,  6, }, // Atlantic/Madeira
    {   44, 2748,   19473,  9, }, // Atlantic/Reykjavik
    {   44, 7272,   19482, 15, }, // Atlantic/South_Georgia
    {   44, 6821,   19497, 13, }, // Atlantic/St_Helena
    {   44, 6644,   19510,  9, }, // Atlantic/Stanley
    {   44, 1670,   19519,  7, }, // Australia/Adelaide
    {   44, 1635,   19526,  8, }, // Australia/Brisbane
    {   44, 1799,   19534, 11, }, // Australia/Broken_Hill
    {   44, 1583,   19545,  6, }, // Australia/Darwin
    {   44, 3637,   19551,  6, }, // Australia/Eucla
    {   44, 1516,   19557,  7, }, // Australia/Hobart
    {   44, 8753,   19564,  8, }, // Australia/Lindeman
    {   44, 1547,   19572,  8, }, // Australia/Lord_Howe
    {   44, 1727,   19580,  8, }, // Australia/Melbourne
    {   44, 1762,   19588,  5, }, // Australia/Perth
    {   44, 1463,   19593,  5, }, // Australia/Sydney
    {   44, 6441,   19598, 10, }, // Europe/Amsterdam
    {   44, 6216,   19608,  7, }, // Europe/Andorra
    {   44, 3620,   19615, 10, }, // Europe/Astrakhan
    {   44, 2300,   19625,  6, }, // Europe/Athens
    {   44, 6336,   19631,  8, }, // Europe/Belgrade
    {   44, 4757,   19639,  7, }, // Europe/Berlin
    {   44, 6488,   19646, 11, }, // Europe/Bratislava
    {   44, 2167,   19657,  8, }, // Europe/Brussels
    {   44, 3928,   19665,  9, }, // Europe/Bucharest
    {   44, 3806,   19674,  9, }, // Europe/Budapest
    {   44, 8772,   19683,  8, }, // Europe/Busingen
    {   44, 2583,   19691,  7, }, // Europe/Chisinau
    {   44, 6289,   19698, 10, }, // Europe/Copenhagen
    {   44, 2338,   19708,  6, }, // Europe/Dublin
    {   44, 6307,   19714, 10, }, // Europe/Gibraltar
    {   44, 8788,   19724,  7, }, // Europe/Guernsey
    {   44, 6612,   19731,  8, }, // Europe/Helsinki
    {   44, 8804,   19739, 10, }, // Europe/Isle_of_Man
    {   44, 1130,   19749,  9, }, // Europe/Istanbul
    {   44, 8823,   19758,  6, }, // Europe/Jersey
    {   44, 3979,   19764, 12, }, // Europe/Kaliningrad
    {   44, 8929,   19776,  5, }, // Europe/Kirov
    {   44, 2527,   19781,  3, }, // Europe/Kyiv
    {   44, 3213,   19784,  5, }, // Europe/Lisbon
    {   44, 6506,   19789, 11, }, // Europe/Ljubljana
    {   44, 2501,   19800,  5, }, // Europe/London
    {   44, 6365,   19805, 12, }, // Europe/Luxembourg
    {   44, 4892,   19817,  7, }, // Europe/Madrid
    {   44, 6397,   19824,  6, }, // Europe/Malta
    {   44, 6582,   19830, 10, }, // Europe/Mariehamn
    {   44, 3693,   19840,  6, }, // Europe/Minsk
    {   44, 6410,   19846,  6, }, // Europe/Monaco
    {   44, 3537,   19852,  5, }, // Europe/Moscow
    {   44, 6458,   19857,  5, }, // Europe/Oslo
    {   44, 4284,   19862,  5, }, // Europe/Paris
    {   44, 6424,   19867,  9, }, // Europe/Podgorica
    {   44, 6275,   19876,  5, }, // Europe/Prague
    {   44, 8837,   19881,  4, }, // Europe/Riga
    {   44, 6324,   19465,  3, }, // Europe/Rome
    {   44, 4331,   19885,  6, }, // Europe/Samara
    {   44, 6470,   19891, 10, }, // Europe/San_Marino
    {   44, 6245,   19901,  8, }, // Europe/Sarajevo
    {   44, 4451,   19909,  7, }, // Europe/Saratov
    {   44, 8849,   19916, 11, }, // Europe/Simferopol
    {   44, 6383,   19927,  8, }, // Europe/Skopje
    {   44, 6599,   19935,  7, }, // Europe/Sofia
    {   44, 6523,   19942,  7, }, // Europe/Stockholm
    {   44, 8867,   19949,  5, }, // Europe/Tallinn
    {   44, 6202,   19954,  6, }, // Europe/Tirane
    {   44, 8882,   19960, 13, }, // Europe/Ulyanovsk
    {   44, 6352,   19973,  5, }, // Europe/Vaduz
    {   44, 6567,   19978,  7, }, // Europe/Vatican
    {   44, 6231,   19985,  7, }, // Europe/Vienna
    {   44, 8899,   19992,  9, }, // Europe/Vilnius
    {   44, 4727,   20001, 11, }, // Europe/Volgograd
    {   44, 3190,   20012,  6, }, // Europe/Warsaw
    {   44, 6261,   20018,  7, }, // Europe/Zagreb
    {   44, 6540,   20025,  5, }, // Europe/Zurich
    {   44, 5078,   20030, 13, }, // Indian/Antananarivo
    {   44, 6947,   20043,  5, }, // Indian/Chagos
    {   44, 6046,   20048, 10, }, // Indian/Christmas
    {   44, 6063,   20058,  5, }, // Indian/Cocos
    {   44, 5029,   20063,  6, }, // Indian/Comoro
    {   44, 6661,   20069,  8, }, // Indian/Kerguelen
    {   44, 7260,   20077,  4, }, // Indian/Mahe
    {   44, 7050,   20081,  8, }, // Indian/Maldives
    {   44, 4069,   20089,  6, }, // Indian/Mauritius
    {   44, 5098,   20095,  9, }, // Indian/Mayotte
    {   44, 7211,   20104,  9, }, // Indian/Reunion
    {   44, 4422,   20113,  6, }, // Pacific/Apia
    {   44,  932,   20119,  7, }, // Pacific/Auckland
    {   44, 3706,   20126, 10, }, // Pacific/Bougainville
    {   44, 3015,   20136,  5, }, // Pacific/Chatham
    {   44, 3157,   20141,  5, }, // Pacific/Chuuk
    {   44, 2237,   20146,  6, }, // Pacific/Easter
    {   44, 7360,   20152,  4, }, // Pacific/Efate
    {   44, 7327,   20156,  7, }, // Pacific/Fakaofo
    {   44, 3902,   20163,  4, }, // Pacific/Fiji
    {   44, 7343,   20167,  8, }, // Pacific/Funafuti
    {   44, 6678,   20175,  9, }, // Pacific/Galapagos
    {   44, 6696,   20184, 10, }, // Pacific/Gambier
    {   44, 3822,   20194, 11, }, // Pacific/Guadalcanal
    {   44, 6018,   18122,  5, }, // Pacific/Guam
    {   44, 3049,   20205,  9, }, // Pacific/Kanton
    {   44, 3998,   20214, 10, }, // Pacific/Kiritimati
    {   44, 7019,   20224,  8, }, // Pacific/Kosrae
    {   44, 2842,   20232,  9, }, // Pacific/Kwajalein
    {   44, 4877,   20241,  6, }, // Pacific/Majuro
    {   44, 4051,   20247,  8, }, // Pacific/Marquesas
    {   44, 8914,   20255,  5, }, // Pacific/Midway
    {   44, 7084,   20260,  4, }, // Pacific/Nauru
    {   44, 7132,   20264,  5, }, // Pacific/Niue
    {   44, 4168,   20269,  6, }, // Pacific/Norfolk
    {   44, 7098,   20275,  7, }, // Pacific/Noumea
    {   44, 3126,   20282,  9, }, // Pacific/Pago_Pago
    {   44, 7155,   20291,  5, }, // Pacific/Palau
    {   44, 7194,   20296,  9, }, // Pacific/Pitcairn
    {   44, 3096,   20305,  6, }, // Pacific/Pohnpei
    {   44, 4807,   20311, 15, }, // Pacific/Port_Moresby
    {   44, 6076,   20326, 10, }, // Pacific/Rarotonga
    {   44, 6031,   20336,  6, }, // Pacific/Saipan
    {   44, 7312,   20342,  6, }, // Pacific/Tahiti
    {   44, 6712,   20348,  6, }, // Pacific/Tarawa
    {   44, 4583,   20354,  8, }, // Pacific/Tongatapu
    {   44, 7392,   20362,  3, }, // Pacific/Wake
    {   44, 7405,   20365,  5, }, // Pacific/Wallis
    {   45, 5059,   20370, 10, }, // Africa/Addis_Ababa Asturian/Latin/Spain
    {   45, 7420,   20380,  5, }, // Africa/Algiers
    {   45,   14,       0,  6, }, // Africa/Asmara
    {   45, 2320,   20385,  8, }, // Africa/Cairo
    {   45, 5043,   20393,  6, }, // Africa/Djibouti
    {   45, 4479,   20399, 13, }, // Africa/Johannesburg
    {   45, 4524,   20412,  6, }, // Africa/Khartoum
    {   45, 6869,   20418,  4, }, // Africa/Lome
    {   45, 5113,   20422,  8, }, // Africa/Mogadishu
    {   45, 5271,   20430,  6, }, // Africa/Ndjamena
    {   45, 5354,   20436,  6, }, // Africa/Niamey
    {   45, 6803,   20442,  7, }, // Africa/Nouakchott
    {   45, 6727,   20449,  8, }, // Africa/Ouagadougou
    {   45, 4435,   20457, 10, }, // Africa/Sao_Tome
    {   45, 2866,   20467,  7, }, // Africa/Tripoli
    {   45, 6554,   20474,  5, }, // Africa/Tunis
    {   45, 5607,    4775,  7, }, // America/Anguilla
    {   45,  270,   20479,  7, }, // America/Argentina/Cordoba
    {   45, 7571,   20486,  7, }, // America/Argentina/Tucuman
    {   45, 4252,      14,  8, }, // America/Asuncion
    {   45,  237,      22,  8, }, // America/Atikokan
    {   45, 3679,      30,  5, }, // America/Bahia
    {   45, 7623,   20493, 14, }, // America/Bahia_Banderas
    {   45, 4361,   20507,  6, }, // America/Bogota
    {   45, 3868,      47,  6, }, // America/Cancun
    {   45, 5496,   20513,  6, }, // America/Cayman
    {   45, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   45, 5723,      66,  7, }, // America/Curacao
    {   45, 5770,   20519,  7, }, // America/Grenada
    {   45, 5786,   20526,  9, }, // America/Guadeloupe
    {   45, 2281,   20535,  9, }, // America/Havana
    {   45,  481,      73, 13, }, // America/Indiana/Knox
    {   45, 7961,      86, 16, }, // America/Indiana/Marengo
    {   45, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   45, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   45, 8038,     139, 14, }, // America/Indiana/Vevay
    {   45, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   45, 8086,     171, 16, }, // America/Indiana/Winamac
    {   45, 2799,   20544,  7, }, // America/Jamaica
    {   45, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   45, 5932,     207, 22, }, // America/Lower_Princes
    {   45, 5805,   20551, 12, }, // America/Martinique
    {   45, 8220,     229,  6, }, // America/Merida
    {   45, 2949,   20563, 15, }, // America/Mexico_City
    {   45, 2379,   20578, 10, }, // America/New_York
    {   45, 1850,     235, 19, }, // America/Noronha
    {   45, 8301,   20588, 24, }, // America/North_Dakota/Beulah
    {   45, 8329,   20612, 24, }, // America/North_Dakota/Center
    {   45, 8357,   20636, 29, }, // America/North_Dakota/New_Salem
    {   45,  393,     317,  4, }, // America/Nuuk
    {   45, 2356,   20665,  6, }, // America/Panama
    {   45, 3945,   20671, 15, }, // America/Port-au-Prince
    {   45, 5954,   20686, 13, }, // America/Port_of_Spain
    {   45, 5843,   20699, 11, }, // America/Puerto_Rico
    {   45, 6111,   20710, 13, }, // America/Santo_Domingo
    {   45, 6899,     321, 16, }, // America/Scoresbysund
    {   45, 8508,     337, 14, }, // America/St_Barthelemy
    {   45, 2061,     351, 10, }, // America/St_Johns
    {   45, 5863,     361,  9, }, // America/St_Kitts
    {   45, 5880,     370,  9, }, // America/St_Lucia
    {   45,  855,     379, 10, }, // America/St_Thomas
    {   45, 5913,     389, 11, }, // America/St_Vincent
    {   45, 5688,   20723,  7, }, // America/Tortola
    {   45, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {   45, 3968,   20730,  4, }, // Asia/Amman
    {   45, 3607,   20734,  6, }, // Asia/Baghdad
    {   45, 5534,   20740,  7, }, // Asia/Bahrain
    {   45, 3653,   20747,  4, }, // Asia/Baku
    {   45, 5989,   20751,  6, }, // Asia/Brunei
    {   45, 4540,   20757,  7, }, // Asia/Damascus
    {   45, 6159,   20764,  7, }, // Asia/Dushanbe
    {   45, 4795,   20771,  6, }, // Asia/Hebron
    {   45, 1259,   20777, 20, }, // Asia/Ho_Chi_Minh
    {   45, 2704,   20797,  9, }, // Asia/Hong_Kong
    {   45, 7006,   20806,  7, }, // Asia/Jakarta
    {   45, 6992,   20813,  8, }, // Asia/Jayapura
    {   45, 1290,   20821,  9, }, // Asia/Jerusalem
    {   45, 1185,   20830,  8, }, // Asia/Kathmandu
    {   45,  992,   20838,  7, }, // Asia/Kolkata
    {   45, 1211,   20845,  5, }, // Asia/Macau
    {   45, 4017,   20850,  7, }, // Asia/Magadan
    {   45, 6920,   20857,  7, }, // Asia/Muscat
    {   45, 4214,   20864, 10, }, // Asia/Pyongyang
    {   45, 3584,   20874,  5, }, // Asia/Riyadh
    {   45, 4408,   20879,  7, }, // Asia/Sakhalin
    {   45, 7245,   20886, 10, }, // Asia/Samarkand
    {   45, 3279,   20896,  4, }, // Asia/Seoul
    {   45, 3300,   20900,  8, }, // Asia/Singapore
    {   45, 2772,   20908,  6, }, // Asia/Tehran
    {   45, 2821,   20914,  5, }, // Asia/Tokyo
    {   45, 1021,   20919, 10, }, // Asia/Ulaanbaatar
    {   45, 6977,   20929,  8, }, // Asia/Vientiane
    {   45, 1235,   20937,  6, }, // Asia/Yangon
    {   45, 3883,   20943, 14, }, // Asia/Yekaterinburg
    {   45, 3663,   20957, 10, }, // Atlantic/Azores
    {   45, 6628,   20967,  8, }, // Atlantic/Canary
    {   45, 3727,   20975, 10, }, // Atlantic/Cape_Verde
    {   45, 1395,   20985, 12, }, // Atlantic/Faroe
    {   45, 2748,   20997,  9, }, // Atlantic/Reykjavik
    {   45, 7272,   21006, 14, }, // Atlantic/South_Georgia
    {   45, 6821,   21020, 10, }, // Atlantic/St_Helena
    {   45, 1670,   21030,  8, }, // Australia/Adelaide
    {   45, 3620,   21038,  8, }, // Europe/Astrakhan
    {   45, 2300,   21046,  6, }, // Europe/Athens
    {   45, 6336,   21052,  7, }, // Europe/Belgrade
    {   45, 4757,   21059,  6, }, // Europe/Berlin
    {   45, 2167,   21065,  8, }, // Europe/Brussels
    {   45, 3928,   21073,  8, }, // Europe/Bucharest
    {   45, 6289,   21081, 10, }, // Europe/Copenhagen
    {   45, 2338,   21091,  6, }, // Europe/Dublin
    {   45, 6307,   21097,  9, }, // Europe/Gibraltar
    {   45, 6612,   21106,  8, }, // Europe/Helsinki
    {   45, 8804,   21114, 12, }, // Europe/Isle_of_Man
    {   45, 3979,   21126, 11, }, // Europe/Kaliningrad
    {   45, 2527,    8148,  4, }, // Europe/Kyiv
    {   45, 3213,   21137,  6, }, // Europe/Lisbon
    {   45, 6506,   21143,  9, }, // Europe/Ljubljana
    {   45, 2501,   21152,  7, }, // Europe/London
    {   45, 6365,   21159, 10, }, // Europe/Luxembourg
    {   45, 6410,   21169,  6, }, // Europe/Monaco
    {   45, 3537,   21175,  5, }, // Europe/Moscow
    {   45, 4284,   21180,  5, }, // Europe/Paris
    {   45, 6275,   21185,  5, }, // Europe/Prague
    {   45, 6324,    6064,  4, }, // Europe/Rome
    {   45, 6470,   21190,  9, }, // Europe/San_Marino
    {   45, 6599,   21199,  5, }, // Europe/Sofia
    {   45, 6523,   21204,  9, }, // Europe/Stockholm
    {   45, 6202,   21213,  6, }, // Europe/Tirane
    {   45, 6567,   21219, 11, }, // Europe/Vatican
    {   45, 6231,   21230,  5, }, // Europe/Vienna
    {   45, 4727,   21235,  9, }, // Europe/Volgograd
    {   45, 3190,   21244,  8, }, // Europe/Warsaw
    {   45, 4069,   21252,  8, }, // Indian/Mauritius
    {   45, 7211,   21260,  7, }, // Indian/Reunion
    {   45, 3157,     554,  5, }, // Pacific/Chuuk
    {   45, 2237,   21267,  6, }, // Pacific/Easter
    {   45, 3902,   21273,  4, }, // Pacific/Fiji
    {   45, 6678,   21277,  9, }, // Pacific/Galapagos
    {   45, 2723,    4611,  8, }, // Pacific/Honolulu
    {   45, 3049,    4619,  9, }, // Pacific/Kanton
    {   45, 3096,     586,  7, }, // Pacific/Pohnpei
    {   46,   14,       0,  6, }, // Africa/Asmara Asu/Latin/Tanzania
    {   46, 4435,       6,  8, }, // Africa/Sao_Tome
    {   46, 4252,      14,  8, }, // America/Asuncion
    {   46,  237,      22,  8, }, // America/Atikokan
    {   46, 7623,      30, 17, }, // America/Bahia_Banderas
    {   46, 3868,      47,  6, }, // America/Cancun
    {   46, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   46, 5723,      66,  7, }, // America/Curacao
    {   46,  481,      73, 13, }, // America/Indiana/Knox
    {   46, 7961,      86, 16, }, // America/Indiana/Marengo
    {   46, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   46, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   46, 8038,     139, 14, }, // America/Indiana/Vevay
    {   46, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   46, 8086,     171, 16, }, // America/Indiana/Winamac
    {   46, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   46, 5932,     207, 22, }, // America/Lower_Princes
    {   46, 8220,     229,  6, }, // America/Merida
    {   46, 1850,     235, 19, }, // America/Noronha
    {   46, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {   46, 8329,     274, 20, }, // America/North_Dakota/Center
    {   46, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {   46,  393,     317,  4, }, // America/Nuuk
    {   46, 6899,     321, 16, }, // America/Scoresbysund
    {   46, 8508,     337, 14, }, // America/St_Barthelemy
    {   46, 2061,     351, 10, }, // America/St_Johns
    {   46, 5863,     361,  9, }, // America/St_Kitts
    {   46, 5880,     370,  9, }, // America/St_Lucia
    {   46,  855,     379, 10, }, // America/St_Thomas
    {   46, 5913,     389, 11, }, // America/St_Vincent
    {   46, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {   46, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {   46, 1185,     448,  9, }, // Asia/Kathmandu
    {   46,  992,     457,  7, }, // Asia/Kolkata
    {   46, 1211,     464,  5, }, // Asia/Macau
    {   46, 1235,     477,  6, }, // Asia/Yangon
    {   46, 1395,     483,  5, }, // Atlantic/Faroe
    {   46, 6821,     488, 10, }, // Atlantic/St_Helena
    {   46, 2527,     514,  4, }, // Europe/Kyiv
    {   46, 7211,     547,  7, }, // Indian/Reunion
    {   46, 3157,     554,  5, }, // Pacific/Chuuk
    {   46, 3096,     586,  7, }, // Pacific/Pohnpei
    {   47,   14,       0,  6, }, // Africa/Asmara Atsam/Latin/Nigeria
    {   47, 4435,       6,  8, }, // Africa/Sao_Tome
    {   47, 4252,      14,  8, }, // America/Asuncion
    {   47,  237,      22,  8, }, // America/Atikokan
    {   47, 7623,      30, 17, }, // America/Bahia_Banderas
    {   47, 3868,      47,  6, }, // America/Cancun
    {   47, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   47, 5723,      66,  7, }, // America/Curacao
    {   47,  481,      73, 13, }, // America/Indiana/Knox
    {   47, 7961,      86, 16, }, // America/Indiana/Marengo
    {   47, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   47, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   47, 8038,     139, 14, }, // America/Indiana/Vevay
    {   47, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   47, 8086,     171, 16, }, // America/Indiana/Winamac
    {   47, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   47, 5932,     207, 22, }, // America/Lower_Princes
    {   47, 8220,     229,  6, }, // America/Merida
    {   47, 1850,     235, 19, }, // America/Noronha
    {   47, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {   47, 8329,     274, 20, }, // America/North_Dakota/Center
    {   47, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {   47,  393,     317,  4, }, // America/Nuuk
    {   47, 6899,     321, 16, }, // America/Scoresbysund
    {   47, 8508,     337, 14, }, // America/St_Barthelemy
    {   47, 2061,     351, 10, }, // America/St_Johns
    {   47, 5863,     361,  9, }, // America/St_Kitts
    {   47, 5880,     370,  9, }, // America/St_Lucia
    {   47,  855,     379, 10, }, // America/St_Thomas
    {   47, 5913,     389, 11, }, // America/St_Vincent
    {   47, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {   47, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {   47, 1185,     448,  9, }, // Asia/Kathmandu
    {   47,  992,     457,  7, }, // Asia/Kolkata
    {   47, 1211,     464,  5, }, // Asia/Macau
    {   47, 1235,     477,  6, }, // Asia/Yangon
    {   47, 1395,     483,  5, }, // Atlantic/Faroe
    {   47, 6821,     488, 10, }, // Atlantic/St_Helena
    {   47, 2527,     514,  4, }, // Europe/Kyiv
    {   47, 7211,     547,  7, }, // Indian/Reunion
    {   47, 3157,     554,  5, }, // Pacific/Chuuk
    {   47, 3096,     586,  7, }, // Pacific/Pohnpei
    {   48, 6788,   21286,  6, }, // Africa/Abidjan Azerbaijani/Latin/Azerbaijan
    {   48, 6760,   21292,  5, }, // Africa/Accra
    {   48, 5059,   21297, 11, }, // Africa/Addis_Ababa
    {   48, 7420,   21308,  8, }, // Africa/Algiers
    {   48,   14,   21316,  6, }, // Africa/Asmara
    {   48, 5257,   21322,  6, }, // Africa/Bangui
    {   48, 6746,   21328,  6, }, // Africa/Banjul
    {   48, 4957,    6318,  7, }, // Africa/Blantyre
    {   48, 5287,   21334,  9, }, // Africa/Brazzaville
    {   48, 2320,   21343,  6, }, // Africa/Cairo
    {   48, 4117,    4695, 10, }, // Africa/Casablanca
    {   48, 7449,   21349,  5, }, // Africa/Ceuta
    {   48, 6773,    6367,  7, }, // Africa/Conakry
    {   48, 5130,   21354, 12, }, // Africa/Dar_es_Salaam
    {   48, 5043,   21366,  6, }, // Africa/Djibouti
    {   48, 5243,   21372,  5, }, // Africa/Douala
    {   48, 5166,   21377,  7, }, // Africa/El_Aaiun
    {   48, 6853,    6400,  7, }, // Africa/Freetown
    {   48, 4906,   21384,  7, }, // Africa/Gaborone
    {   48, 4479,   21391, 11, }, // Africa/Johannesburg
    {   48, 4524,   21402,  6, }, // Africa/Khartoum
    {   48, 4973,   21408,  6, }, // Africa/Kigali
    {   48, 5306,   21414,  7, }, // Africa/Kinshasa
    {   48, 4744,   21421,  5, }, // Africa/Lagos
    {   48, 5336,    6429,  8, }, // Africa/Libreville
    {   48, 4939,   21426,  9, }, // Africa/Lubumbashi
    {   48, 5113,   21435,  8, }, // Africa/Mogadishu
    {   48, 5271,   21443,  7, }, // Africa/Ndjamena
    {   48, 6803,   21450,  8, }, // Africa/Nouakchott
    {   48, 6727,   21458,  7, }, // Africa/Ouagadougou
    {   48, 4435,   21465,  7, }, // Africa/Sao_Tome
    {   48, 4152,   21472,  7, }, // Africa/Windhoek
    {   48, 3346,   21479,  7, }, // America/Anchorage
    {   48, 5607,   21486,  7, }, // America/Anguilla
    {   48, 5624,   21493,  7, }, // America/Antigua
    {   48, 4554,   21500,  9, }, // America/Araguaina
    {   48,  166,   21509, 12, }, // America/Argentina/Buenos_Aires
    {   48,   91,    6518,  9, }, // America/Argentina/Catamarca
    {   48,  270,    6527,  7, }, // America/Argentina/Cordoba
    {   48, 7462,   21521,  5, }, // America/Argentina/La_Rioja
    {   48, 7489,   21526, 12, }, // America/Argentina/Rio_Gallegos
    {   48, 7544,   21538,  8, }, // America/Argentina/San_Juan
    {   48, 7571,    4842,  7, }, // America/Argentina/Tucuman
    {   48, 7597,   21546,  6, }, // America/Argentina/Ushuaia
    {   48, 4252,    6592,  8, }, // America/Asuncion
    {   48,  237,      22,  8, }, // America/Atikokan
    {   48, 3679,   21552,  6, }, // America/Bahia
    {   48, 7623,    3943, 14, }, // America/Bahia_Banderas
    {   48, 5407,    6614,  5, }, // America/Belize
    {   48, 7660,    6620, 12, }, // America/Blanc-Sablon
    {   48, 4361,   21558,  6, }, // America/Bogota
    {   48, 7699,   21564,  5, }, // America/Boise
    {   48, 7713,   21569, 15, }, // America/Cambridge_Bay
    {   48, 7735,   21584, 12, }, // America/Campo_Grande
    {   48, 3868,    6682,  6, }, // America/Cancun
    {   48, 4694,    4910,  7, }, // America/Caracas
    {   48, 4345,   21596,  5, }, // America/Cayenne
    {   48, 5496,   21601,  6, }, // America/Cayman
    {   48, 2260,   21607,  6, }, // America/Chicago
    {   48, 7756,   21613,  8, }, // America/Chihuahua
    {   48, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   48, 5422,   21621, 10, }, // America/Costa_Rica
    {   48, 7796,    6736,  7, }, // America/Creston
    {   48, 3791,   21631,  6, }, // America/Cuiaba
    {   48, 5723,    6749,  7, }, // America/Curacao
    {   48, 7812,   21637, 11, }, // America/Danmarkshavn
    {   48, 7833,    6769,  6, }, // America/Dawson
    {   48, 7848,   21648, 11, }, // America/Dawson_Creek
    {   48, 3465,   21659,  7, }, // America/Detroit
    {   48, 5739,   21666,  8, }, // America/Dominica
    {   48,  893,   21674,  9, }, // America/Edmonton
    {   48, 7869,   21683,  7, }, // America/Eirunepe
    {   48, 5441,    6806,  8, }, // America/El_Salvador
    {   48, 7924,   21690, 13, }, // America/Glace_Bay
    {   48, 6881,   21703, 11, }, // America/Goose_Bay
    {   48, 4612,   21714, 10, }, // America/Grand_Turk
    {   48, 5770,   21724,  7, }, // America/Grenada
    {   48, 5786,   21731,  9, }, // America/Guadeloupe
    {   48, 3760,   21740,  9, }, // America/Guatemala
    {   48, 4841,   21749,  8, }, // America/Guayaquil
    {   48, 6932,   21757,  6, }, // America/Guyana
    {   48, 1939,    6905,  8, }, // America/Halifax
    {   48, 7942,   21763,  9, }, // America/Hermosillo
    {   48,  348,   21772, 12, }, // America/Indiana/Indianapolis
    {   48,  481,   21784,  4, }, // America/Indiana/Knox
    {   48, 7961,   21788,  7, }, // America/Indiana/Marengo
    {   48, 7985,   21795, 10, }, // America/Indiana/Petersburg
    {   48, 8012,     121,  4, }, // America/Indiana/Tell_City
    {   48, 8038,   21805,  5, }, // America/Indiana/Vevay
    {   48, 8060,   21810,  6, }, // America/Indiana/Vincennes
    {   48, 8086,   21816,  7, }, // America/Indiana/Winamac
    {   48, 8110,   21823,  6, }, // America/Inuvik
    {   48,  660,   21829,  7, }, // America/Iqaluit
    {   48, 2799,   21836,  7, }, // America/Jamaica
    {   48, 5380,   21843,  4, }, // America/Juneau
    {   48,  521,   21847,  7, }, // America/Kentucky/Louisville
    {   48, 8125,   21854, 10, }, // America/Kentucky/Monticello
    {   48, 5704,   21864,  9, }, // America/Kralendijk
    {   48, 4376,   21873,  6, }, // America/La_Paz
    {   48, 3239,   21879, 11, }, // America/Los_Angeles
    {   48, 5932,   21890, 20, }, // America/Lower_Princes
    {   48, 8153,   21910,  6, }, // America/Maceio
    {   48, 8168,   21916,  7, }, // America/Managua
    {   48, 5897,   21923,  7, }, // America/Marigot
    {   48, 5805,    5260,  8, }, // America/Martinique
    {   48, 2917,   21930,  8, }, // America/Mazatlan
    {   48, 8202,    7118,  8, }, // America/Menominee
    {   48, 8220,    7126,  6, }, // America/Merida
    {   48, 2949,   21938,  6, }, // America/Mexico_City
    {   48, 4391,    7149,  7, }, // America/Miquelon
    {   48, 8254,    7156,  7, }, // America/Moncton
    {   48, 8270,   21944,  8, }, // America/Monterrey
    {   48, 5824,   21952,  8, }, // America/Montserrat
    {   48, 2379,   21960,  8, }, // America/New_York
    {   48, 8288,   21968,  3, }, // America/Nome
    {   48, 1850,   21971,  7, }, // America/Noronha
    {   48, 8301,   21978, 21, }, // America/North_Dakota/Beulah
    {   48, 8329,   21999, 20, }, // America/North_Dakota/Center
    {   48, 8357,   22019,  9, }, // America/North_Dakota/New_Salem
    {   48,  393,     317,  4, }, // America/Nuuk
    {   48, 8388,   22028,  7, }, // America/Ojinaga
    {   48, 2973,    7283,  6, }, // America/Phoenix
    {   48, 3945,    7289, 12, }, // America/Port-au-Prince
    {   48, 5954,   22035, 12, }, // America/Port_of_Spain
    {   48, 8404,   22047, 11, }, // America/Porto_Velho
    {   48, 5843,   22058, 11, }, // America/Puerto_Rico
    {   48, 8424,   22069, 13, }, // America/Rankin_Inlet
    {   48, 8445,   22082,  5, }, // America/Recife
    {   48, 1995,   22087,  6, }, // America/Regina
    {   48, 8460,   22093,  8, }, // America/Resolute
    {   48,  695,    5480, 10, }, // America/Rio_Branco
    {   48, 2201,   22101,  8, }, // America/Santiago
    {   48, 6111,   22109, 13, }, // America/Santo_Domingo
    {   48, 1878,   22122,  9, }, // America/Sao_Paulo
    {   48, 6899,   22131, 12, }, // America/Scoresbysund
    {   48, 8508,   22143, 14, }, // America/St_Barthelemy
    {   48, 2061,   22157,  9, }, // America/St_Johns
    {   48, 5863,   22166,  8, }, // America/St_Kitts
    {   48, 5880,   22174, 10, }, // America/St_Lucia
    {   48,  855,   22184,  9, }, // America/St_Thomas
    {   48, 5913,   22193, 11, }, // America/St_Vincent
    {   48, 8530,   22204, 12, }, // America/Swift_Current
    {   48, 5461,   22216, 11, }, // America/Tegucigalpa
    {   48, 5756,   22227,  3, }, // America/Thule
    {   48,  313,   22230,  7, }, // America/Tijuana
    {   48, 2093,    7493,  8, }, // America/Vancouver
    {   48, 2144,   22237,  8, }, // America/Whitehorse
    {   48,  734,   22245,  8, }, // America/Winnipeg
    {   48, 6001,   22253,  5, }, // Antarctica/Casey
    {   48, 6094,   22258,  6, }, // Antarctica/Davis
    {   48, 6133,   22264, 14, }, // Antarctica/DumontDUrville
    {   48, 8568,   22278,  7, }, // Antarctica/Macquarie
    {   48, 7066,   22285,  6, }, // Antarctica/Mawson
    {   48, 7113,   22291,  9, }, // Antarctica/McMurdo
    {   48, 7295,   22300,  5, }, // Antarctica/Syowa
    {   48, 1429,   22305,  7, }, // Arctic/Longyearbyen
    {   48, 5395,   22312,  6, }, // Asia/Almaty
    {   48, 5511,   22318,  6, }, // Asia/Anadyr
    {   48, 5523,    5609,  5, }, // Asia/Aqtau
    {   48,  964,   22324,  7, }, // Asia/Ashgabat
    {   48, 8624,   22331,  6, }, // Asia/Atyrau
    {   48, 3607,   22337,  6, }, // Asia/Baghdad
    {   48, 5534,   22343,  7, }, // Asia/Bahrain
    {   48, 3653,   22350,  4, }, // Asia/Baku
    {   48, 4466,   22354,  7, }, // Asia/Bangkok
    {   48, 4086,   22361,  6, }, // Asia/Beirut
    {   48, 3778,   22367,  6, }, // Asia/Bishkek
    {   48, 5989,   22373,  6, }, // Asia/Brunei
    {   48, 4601,    7631,  4, }, // Asia/Chita
    {   48, 4511,    5637,  7, }, // Asia/Colombo
    {   48, 4540,   22379,  6, }, // Asia/Damascus
    {   48, 1093,   22385,  5, }, // Asia/Dhaka
    {   48, 3596,   22390,  5, }, // Asia/Dubai
    {   48, 6159,   22395,  7, }, // Asia/Dushanbe
    {   48, 8636,   22402,  9, }, // Asia/Famagusta
    {   48, 8651,   22411,  4, }, // Asia/Gaza
    {   48, 1259,   22415,  9, }, // Asia/Ho_Chi_Minh
    {   48, 2704,   22424,  9, }, // Asia/Hong_Kong
    {   48, 4184,   22433,  7, }, // Asia/Irkutsk
    {   48, 7006,   22440,  7, }, // Asia/Jakarta
    {   48, 6992,   22447,  8, }, // Asia/Jayapura
    {   48, 1290,   22455,  9, }, // Asia/Jerusalem
    {   48, 3560,   22464,  5, }, // Asia/Kabul
    {   48, 4316,   22469,  8, }, // Asia/Kamchatka
    {   48, 4239,    7687,  6, }, // Asia/Karachi
    {   48, 1185,    5711,  8, }, // Asia/Kathmandu
    {   48, 8661,   22477,  7, }, // Asia/Khandyga
    {   48,  992,   22484,  7, }, // Asia/Kolkata
    {   48, 5976,   22491,  6, }, // Asia/Kuching
    {   48, 5547,   22497,  6, }, // Asia/Kuwait
    {   48, 1211,    7731,  5, }, // Asia/Macau
    {   48, 4017,   22503,  7, }, // Asia/Magadan
    {   48, 1349,    7736,  7, }, // Asia/Makassar
    {   48, 6920,    4326,  6, }, // Asia/Muscat
    {   48, 2554,   22510,  8, }, // Asia/Nicosia
    {   48, 6961,   22518,  8, }, // Asia/Phnom_Penh
    {   48, 4214,   22526,  7, }, // Asia/Pyongyang
    {   48, 4269,   22533,  9, }, // Asia/Qyzylorda
    {   48, 3584,   20874,  5, }, // Asia/Riyadh
    {   48, 4408,   22542,  7, }, // Asia/Sakhalin
    {   48, 7245,   22549,  9, }, // Asia/Samarkand
    {   48, 3279,    7798,  4, }, // Asia/Seoul
    {   48, 1053,   22558,  6, }, // Asia/Shanghai
    {   48, 3300,   22564,  8, }, // Asia/Singapore
    {   48, 4297,   22572, 13, }, // Asia/Srednekolymsk
    {   48, 3263,   22585,  6, }, // Asia/Taipei
    {   48, 4781,   22591,  7, }, // Asia/Tashkent
    {   48, 1021,   22598,  9, }, // Asia/Ulaanbaatar
    {   48, 1159,   22607,  6, }, // Asia/Urumqi
    {   48, 6977,   22613,  8, }, // Asia/Vientiane
    {   48, 1235,   22621,  6, }, // Asia/Yangon
    {   48, 3883,   22627, 13, }, // Asia/Yekaterinburg
    {   48, 3663,    7905,  4, }, // Atlantic/Azores
    {   48, 5671,   22640, 14, }, // Atlantic/Bermuda
    {   48, 6628,    4398,  5, }, // Atlantic/Canary
    {   48, 3727,   22654, 10, }, // Atlantic/Cape_Verde
    {   48, 1395,   22664,  5, }, // Atlantic/Faroe
    {   48, 8736,   22669,  7, }, // Atlantic/Madeira
    {   48, 2748,   22676,  9, }, // Atlantic/Reykjavik
    {   48, 7272,   22685, 14, }, // Atlantic/South_Georgia
    {   48, 6821,   22699, 15, }, // Atlantic/St_Helena
    {   48, 6644,    7965,  6, }, // Atlantic/Stanley
    {   48, 1670,   21030,  8, }, // Australia/Adelaide
    {   48, 1635,   22714,  8, }, // Australia/Brisbane
    {   48, 1583,    7998,  6, }, // Australia/Darwin
    {   48, 3637,   22722,  5, }, // Australia/Eucla
    {   48, 1547,   22727,  8, }, // Australia/Lord_Howe
    {   48, 1727,    8021,  7, }, // Australia/Melbourne
    {   48, 1762,   22735,  4, }, // Australia/Perth
    {   48, 1463,   22739,  6, }, // Australia/Sydney
    {   48, 3620,   22745,  9, }, // Europe/Astrakhan
    {   48, 2300,   22754,  5, }, // Europe/Athens
    {   48, 6336,   22759,  7, }, // Europe/Belgrade
    {   48, 2167,   22766,  7, }, // Europe/Brussels
    {   48, 3928,   22773,  8, }, // Europe/Bucharest
    {   48, 3806,   22781,  8, }, // Europe/Budapest
    {   48, 2583,   22789,  8, }, // Europe/Chisinau
    {   48, 6289,    4475, 10, }, // Europe/Copenhagen
    {   48, 6307,   22797, 12, }, // Europe/Gibraltar
    {   48, 8788,   22809,  7, }, // Europe/Guernsey
    {   48, 8804,   22816,  9, }, // Europe/Isle_of_Man
    {   48, 1130,   22825,  8, }, // Europe/Istanbul
    {   48, 8823,   22833,  5, }, // Europe/Jersey
    {   48, 3979,   22838, 11, }, // Europe/Kaliningrad
    {   48, 2527,   22849,  5, }, // Europe/Kyiv
    {   48, 3213,    4499,  8, }, // Europe/Lisbon
    {   48, 6506,   22854,  9, }, // Europe/Ljubljana
    {   48, 6365,   22863, 10, }, // Europe/Luxembourg
    {   48, 6582,   22873,  7, }, // Europe/Mariehamn
    {   48, 6410,    6044,  6, }, // Europe/Monaco
    {   48, 3537,   22880,  6, }, // Europe/Moscow
    {   48, 6424,   22886, 10, }, // Europe/Podgorica
    {   48, 6275,   22896,  5, }, // Europe/Prague
    {   48, 8837,   22901,  4, }, // Europe/Riga
    {   48, 6324,    6064,  4, }, // Europe/Rome
    {   48, 6245,   22905,  8, }, // Europe/Sarajevo
    {   48, 6383,   22913,  6, }, // Europe/Skopje
    {   48, 6599,   22919,  6, }, // Europe/Sofia
    {   48, 6523,    8242,  8, }, // Europe/Stockholm
    {   48, 8867,   22925,  6, }, // Europe/Tallinn
    {   48, 6202,   21213,  6, }, // Europe/Tirane
    {   48, 6352,   22931,  6, }, // Europe/Vaduz
    {   48, 6567,    6096,  7, }, // Europe/Vatican
    {   48, 6231,   22937,  5, }, // Europe/Vienna
    {   48, 8899,   22942,  6, }, // Europe/Vilnius
    {   48, 4727,   22948,  9, }, // Europe/Volgograd
    {   48, 3190,   22957,  7, }, // Europe/Warsaw
    {   48, 6261,   22964,  6, }, // Europe/Zagreb
    {   48, 6540,   22970,  5, }, // Europe/Zurich
    {   48, 6947,   22975,  5, }, // Indian/Chagos
    {   48, 6046,   22980,  5, }, // Indian/Christmas
    {   48, 6063,    4583,  5, }, // Indian/Cocos
    {   48, 5029,   22985,  6, }, // Indian/Comoro
    {   48, 6661,   22991,  8, }, // Indian/Kerguelen
    {   48, 7050,    8311,  6, }, // Indian/Maldives
    {   48, 4069,   22999,  7, }, // Indian/Mauritius
    {   48, 5098,    6164,  5, }, // Indian/Mayotte
    {   48, 7211,     547,  7, }, // Indian/Reunion
    {   48,  932,   23006,  6, }, // Pacific/Auckland
    {   48, 3706,   23012,  8, }, // Pacific/Bougainville
    {   48, 3015,   23020,  5, }, // Pacific/Chatham
    {   48, 3157,   23025,  4, }, // Pacific/Chuuk
    {   48, 2237,   23029,  5, }, // Pacific/Easter
    {   48, 3902,   23034,  4, }, // Pacific/Fiji
    {   48, 6678,   23038,  9, }, // Pacific/Galapagos
    {   48, 6696,   23047,  8, }, // Pacific/Gambier
    {   48, 3822,   23055, 11, }, // Pacific/Guadalcanal
    {   48, 6018,   23066,  4, }, // Pacific/Guam
    {   48, 2723,    4611,  8, }, // Pacific/Honolulu
    {   48, 3049,   23070,  9, }, // Pacific/Kanton
    {   48, 3998,   23079,  8, }, // Pacific/Kiritimati
    {   48, 7019,   23087,  7, }, // Pacific/Kosrae
    {   48, 2842,   23094,  9, }, // Pacific/Kwajalein
    {   48, 4877,   23103,  6, }, // Pacific/Majuro
    {   48, 4051,   23109,  8, }, // Pacific/Marquesas
    {   48, 8914,   23117,  6, }, // Pacific/Midway
    {   48, 3126,   23123,  9, }, // Pacific/Pago_Pago
    {   48, 7194,    8422,  7, }, // Pacific/Pitcairn
    {   48, 3096,     586,  7, }, // Pacific/Pohnpei
    {   48, 4807,   23132, 12, }, // Pacific/Port_Moresby
    {   48, 6076,   23144,  9, }, // Pacific/Rarotonga
    {   48, 6031,   23153,  6, }, // Pacific/Saipan
    {   48, 6712,   23159,  6, }, // Pacific/Tarawa
    {   48, 4583,   23165,  9, }, // Pacific/Tongatapu
    {   48, 7392,   23174,  4, }, // Pacific/Wake
    {   48, 7405,    8467,  6, }, // Pacific/Wallis
    {   49,   14,       0,  6, }, // Africa/Asmara Azerbaijani/Arabic/Iran
    {   49, 4435,       6,  8, }, // Africa/Sao_Tome
    {   49, 4252,      14,  8, }, // America/Asuncion
    {   49, 7623,      30, 17, }, // America/Bahia_Banderas
    {   49, 3868,      47,  6, }, // America/Cancun
    {   49, 5723,      66,  7, }, // America/Curacao
    {   49,  481,      73, 13, }, // America/Indiana/Knox
    {   49, 7961,      86, 16, }, // America/Indiana/Marengo
    {   49, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   49, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   49, 8038,     139, 14, }, // America/Indiana/Vevay
    {   49, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   49, 8086,     171, 16, }, // America/Indiana/Winamac
    {   49, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   49, 5932,     207, 22, }, // America/Lower_Princes
    {   49, 8220,     229,  6, }, // America/Merida
    {   49, 1850,     235, 19, }, // America/Noronha
    {   49, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {   49, 8329,     274, 20, }, // America/North_Dakota/Center
    {   49, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {   49, 6899,     321, 16, }, // America/Scoresbysund
    {   49, 8508,     337, 14, }, // America/St_Barthelemy
    {   49, 2061,     351, 10, }, // America/St_Johns
    {   49, 5863,     361,  9, }, // America/St_Kitts
    {   49, 5880,     370,  9, }, // America/St_Lucia
    {   49,  855,     379, 10, }, // America/St_Thomas
    {   49, 5913,     389, 11, }, // America/St_Vincent
    {   49, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {   49, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {   49, 1185,     448,  9, }, // Asia/Kathmandu
    {   49,  992,     457,  7, }, // Asia/Kolkata
    {   49, 1211,     464,  5, }, // Asia/Macau
    {   49, 1235,     477,  6, }, // Asia/Yangon
    {   49, 1395,     483,  5, }, // Atlantic/Faroe
    {   49, 6821,     488, 10, }, // Atlantic/St_Helena
    {   49, 2527,     514,  4, }, // Europe/Kyiv
    {   49, 3157,     554,  5, }, // Pacific/Chuuk
    {   52,   14,       0,  6, }, // Africa/Asmara Azerbaijani/Cyrillic/Azerbaijan
    {   52, 4435,       6,  8, }, // Africa/Sao_Tome
    {   52, 4252,      14,  8, }, // America/Asuncion
    {   52, 7623,      30, 17, }, // America/Bahia_Banderas
    {   52, 3868,      47,  6, }, // America/Cancun
    {   52, 5723,      66,  7, }, // America/Curacao
    {   52,  481,      73, 13, }, // America/Indiana/Knox
    {   52, 7961,      86, 16, }, // America/Indiana/Marengo
    {   52, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   52, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   52, 8038,     139, 14, }, // America/Indiana/Vevay
    {   52, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   52, 8086,     171, 16, }, // America/Indiana/Winamac
    {   52, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   52, 5932,     207, 22, }, // America/Lower_Princes
    {   52, 8220,     229,  6, }, // America/Merida
    {   52, 1850,     235, 19, }, // America/Noronha
    {   52, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {   52, 8329,     274, 20, }, // America/North_Dakota/Center
    {   52, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {   52, 6899,     321, 16, }, // America/Scoresbysund
    {   52, 8508,     337, 14, }, // America/St_Barthelemy
    {   52, 2061,     351, 10, }, // America/St_Johns
    {   52, 5863,     361,  9, }, // America/St_Kitts
    {   52, 5880,     370,  9, }, // America/St_Lucia
    {   52,  855,     379, 10, }, // America/St_Thomas
    {   52, 5913,     389, 11, }, // America/St_Vincent
    {   52, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {   52, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {   52, 1185,     448,  9, }, // Asia/Kathmandu
    {   52,  992,     457,  7, }, // Asia/Kolkata
    {   52, 1211,     464,  5, }, // Asia/Macau
    {   52, 1235,     477,  6, }, // Asia/Yangon
    {   52, 1395,     483,  5, }, // Atlantic/Faroe
    {   52, 6821,     488, 10, }, // Atlantic/St_Helena
    {   52, 2527,     514,  4, }, // Europe/Kyiv
    {   52, 3157,     554,  5, }, // Pacific/Chuuk
    {   53,   14,       0,  6, }, // Africa/Asmara Bafia/Latin/Cameroon
    {   53, 4435,       6,  8, }, // Africa/Sao_Tome
    {   53, 4252,      14,  8, }, // America/Asuncion
    {   53,  237,      22,  8, }, // America/Atikokan
    {   53, 7623,      30, 17, }, // America/Bahia_Banderas
    {   53, 3868,      47,  6, }, // America/Cancun
    {   53, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   53, 5723,      66,  7, }, // America/Curacao
    {   53,  481,      73, 13, }, // America/Indiana/Knox
    {   53, 7961,      86, 16, }, // America/Indiana/Marengo
    {   53, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   53, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   53, 8038,     139, 14, }, // America/Indiana/Vevay
    {   53, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   53, 8086,     171, 16, }, // America/Indiana/Winamac
    {   53, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   53, 5932,     207, 22, }, // America/Lower_Princes
    {   53, 8220,     229,  6, }, // America/Merida
    {   53, 1850,     235, 19, }, // America/Noronha
    {   53, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {   53, 8329,     274, 20, }, // America/North_Dakota/Center
    {   53, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {   53,  393,     317,  4, }, // America/Nuuk
    {   53, 6899,     321, 16, }, // America/Scoresbysund
    {   53, 8508,     337, 14, }, // America/St_Barthelemy
    {   53, 2061,     351, 10, }, // America/St_Johns
    {   53, 5863,     361,  9, }, // America/St_Kitts
    {   53, 5880,     370,  9, }, // America/St_Lucia
    {   53,  855,     379, 10, }, // America/St_Thomas
    {   53, 5913,     389, 11, }, // America/St_Vincent
    {   53, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {   53, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {   53, 1185,     448,  9, }, // Asia/Kathmandu
    {   53,  992,     457,  7, }, // Asia/Kolkata
    {   53, 1211,     464,  5, }, // Asia/Macau
    {   53, 1235,     477,  6, }, // Asia/Yangon
    {   53, 1395,     483,  5, }, // Atlantic/Faroe
    {   53, 6821,     488, 10, }, // Atlantic/St_Helena
    {   53, 2527,     514,  4, }, // Europe/Kyiv
    {   53, 7211,     547,  7, }, // Indian/Reunion
    {   53, 3157,     554,  5, }, // Pacific/Chuuk
    {   53, 3096,     586,  7, }, // Pacific/Pohnpei
    {   54,   14,       0,  6, }, // Africa/Asmara Bambara/Latin/Mali
    {   54, 4435,       6,  8, }, // Africa/Sao_Tome
    {   54, 4252,      14,  8, }, // America/Asuncion
    {   54,  237,      22,  8, }, // America/Atikokan
    {   54, 7623,      30, 17, }, // America/Bahia_Banderas
    {   54, 3868,      47,  6, }, // America/Cancun
    {   54, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   54, 5723,      66,  7, }, // America/Curacao
    {   54,  481,      73, 13, }, // America/Indiana/Knox
    {   54, 7961,      86, 16, }, // America/Indiana/Marengo
    {   54, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   54, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   54, 8038,     139, 14, }, // America/Indiana/Vevay
    {   54, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   54, 8086,     171, 16, }, // America/Indiana/Winamac
    {   54, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   54, 5932,     207, 22, }, // America/Lower_Princes
    {   54, 8220,     229,  6, }, // America/Merida
    {   54, 1850,     235, 19, }, // America/Noronha
    {   54, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {   54, 8329,     274, 20, }, // America/North_Dakota/Center
    {   54, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {   54,  393,     317,  4, }, // America/Nuuk
    {   54, 6899,     321, 16, }, // America/Scoresbysund
    {   54, 8508,     337, 14, }, // America/St_Barthelemy
    {   54, 2061,     351, 10, }, // America/St_Johns
    {   54, 5863,     361,  9, }, // America/St_Kitts
    {   54, 5880,     370,  9, }, // America/St_Lucia
    {   54,  855,     379, 10, }, // America/St_Thomas
    {   54, 5913,     389, 11, }, // America/St_Vincent
    {   54, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {   54, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {   54, 1185,     448,  9, }, // Asia/Kathmandu
    {   54,  992,     457,  7, }, // Asia/Kolkata
    {   54, 1211,     464,  5, }, // Asia/Macau
    {   54, 1235,     477,  6, }, // Asia/Yangon
    {   54, 1395,     483,  5, }, // Atlantic/Faroe
    {   54, 6821,     488, 10, }, // Atlantic/St_Helena
    {   54, 2527,     514,  4, }, // Europe/Kyiv
    {   54, 7211,     547,  7, }, // Indian/Reunion
    {   54, 3157,     554,  5, }, // Pacific/Chuuk
    {   54, 3096,     586,  7, }, // Pacific/Pohnpei
    {   56, 6788,   23178,  7, }, // Africa/Abidjan Bangla/Bangla/Bangladesh
    {   56, 6760,   23185,  5, }, // Africa/Accra
    {   56, 5059,   23190, 12, }, // Africa/Addis_Ababa
    {   56, 7420,   23202, 10, }, // Africa/Algiers
    {   56,   14,   23212, 11, }, // Africa/Asmara
    {   56,   44,   23223,  8, }, // Africa/Bamako
    {   56, 5257,   23231,  7, }, // Africa/Bangui
    {   56, 6746,   23238,  7, }, // Africa/Banjul
    {   56, 7435,   23245,  5, }, // Africa/Bissau
    {   56, 4957,   23250, 13, }, // Africa/Blantyre
    {   56, 5287,   23263, 10, }, // Africa/Brazzaville
    {   56, 4922,   23273, 10, }, // Africa/Bujumbura
    {   56, 2320,   23283,  6, }, // Africa/Cairo
    {   56, 4117,   23289, 12, }, // Africa/Casablanca
    {   56, 7449,   23301,  5, }, // Africa/Ceuta
    {   56, 6773,   23306,  7, }, // Africa/Conakry
    {   56, 6840,   23313,  5, }, // Africa/Dakar
    {   56, 5130,   23318, 12, }, // Africa/Dar_es_Salaam
    {   56, 5043,   17053,  6, }, // Africa/Djibouti
    {   56, 5243,   23330,  7, }, // Africa/Douala
    {   56, 5166,   23337,  8, }, // Africa/El_Aaiun
    {   56, 6853,   23345,  8, }, // Africa/Freetown
    {   56, 4906,   23353,  9, }, // Africa/Gaborone
    {   56, 5015,   23362,  6, }, // Africa/Harare
    {   56, 4479,   23368, 12, }, // Africa/Johannesburg
    {   56, 4499,   17107,  4, }, // Africa/Juba
    {   56, 5151,   23380,  7, }, // Africa/Kampala
    {   56, 4524,   23387,  7, }, // Africa/Khartoum
    {   56, 4973,   23394,  6, }, // Africa/Kigali
    {   56, 5306,   23400,  7, }, // Africa/Kinshasa
    {   56, 4744,   23407,  5, }, // Africa/Lagos
    {   56, 5336,   23412,  9, }, // Africa/Libreville
    {   56, 6869,   23421,  4, }, // Africa/Lome
    {   56, 5211,   23425,  9, }, // Africa/Luanda
    {   56, 4939,   23434, 10, }, // Africa/Lubumbashi
    {   56, 5001,   23444,  6, }, // Africa/Lusaka
    {   56, 5322,   17181,  6, }, // Africa/Malabo
    {   56, 4987,   23450,  6, }, // Africa/Maputo
    {   56, 5197,   23456,  7, }, // Africa/Maseru
    {   56, 5182,   23463,  7, }, // Africa/Mbabane
    {   56, 5113,   23470,  9, }, // Africa/Mogadishu
    {   56, 7034,   23479,  9, }, // Africa/Monrovia
    {   56, 3853,   23488,  7, }, // Africa/Nairobi
    {   56, 5271,   23495,  7, }, // Africa/Ndjamena
    {   56, 5354,   23502,  7, }, // Africa/Niamey
    {   56, 6803,   23509,  8, }, // Africa/Nouakchott
    {   56, 6727,   23517, 11, }, // Africa/Ouagadougou
    {   56, 5225,   23528, 11, }, // Africa/Porto-Novo
    {   56, 4435,   23539,  7, }, // Africa/Sao_Tome
    {   56, 2866,   23546,  8, }, // Africa/Tripoli
    {   56, 6554,   23554,  6, }, // Africa/Tunis
    {   56, 4152,   23560,  6, }, // Africa/Windhoek
    {   56,  132,   23566,  3, }, // America/Adak
    {   56, 3346,   23569,  7, }, // America/Anchorage
    {   56, 5607,   23576, 11, }, // America/Anguilla
    {   56, 5624,   23587, 13, }, // America/Antigua
    {   56, 4554,   23600, 11, }, // America/Araguaina
    {   56,  166,   23611, 16, }, // America/Argentina/Buenos_Aires
    {   56,   91,   23627, 12, }, // America/Argentina/Catamarca
    {   56,  270,   23639,  7, }, // America/Argentina/Cordoba
    {   56,  441,   17369,  5, }, // America/Argentina/Jujuy
    {   56, 7462,   23646,  8, }, // America/Argentina/La_Rioja
    {   56,  565,   23654,  8, }, // America/Argentina/Mendoza
    {   56, 7489,   23662, 12, }, // America/Argentina/Rio_Gallegos
    {   56, 7520,   23674,  7, }, // America/Argentina/Salta
    {   56, 7544,   23681, 10, }, // America/Argentina/San_Juan
    {   56, 5580,   23691,  8, }, // America/Argentina/San_Luis
    {   56, 7571,   23699,  7, }, // America/Argentina/Tucuman
    {   56, 7597,   23706, 10, }, // America/Argentina/Ushuaia
    {   56, 5640,   23716,  5, }, // America/Aruba
    {   56, 4252,   23721,  9, }, // America/Asuncion
    {   56,  237,   17453,  8, }, // America/Atikokan
    {   56, 3679,   17461,  7, }, // America/Bahia
    {   56, 7623,   23730, 17, }, // America/Bahia_Banderas
    {   56, 5654,   23747,  9, }, // America/Barbados
    {   56, 7646,   17494,  5, }, // America/Belem
    {   56, 5407,   17499,  5, }, // America/Belize
    {   56, 7660,   23756, 15, }, // America/Blanc-Sablon
    {   56, 7681,   23771, 12, }, // America/Boa_Vista
    {   56, 4361,   17528,  6, }, // America/Bogota
    {   56, 7699,   23783,  5, }, // America/Boise
    {   56, 7713,   23788, 11, }, // America/Cambridge_Bay
    {   56, 7735,   23799, 15, }, // America/Campo_Grande
    {   56, 3868,   23814,  8, }, // America/Cancun
    {   56, 4694,   23822,  9, }, // America/Caracas
    {   56, 4345,   23831,  5, }, // America/Cayenne
    {   56, 5496,   23836,  5, }, // America/Cayman
    {   56, 2260,   23841,  6, }, // America/Chicago
    {   56, 7756,   23847, 12, }, // America/Chihuahua
    {   56, 7774,   23859, 15, }, // America/Ciudad_Juarez
    {   56, 5422,   23874, 10, }, // America/Costa_Rica
    {   56, 7796,   23884,  9, }, // America/Creston
    {   56, 3791,   23893,  8, }, // America/Cuiaba
    {   56, 5723,   23901,  7, }, // America/Curacao
    {   56, 7812,   23908, 14, }, // America/Danmarkshavn
    {   56, 7833,   23922,  4, }, // America/Dawson
    {   56, 7848,   23926, 10, }, // America/Dawson_Creek
    {   56,  805,   23936,  6, }, // America/Denver
    {   56, 3465,   23942,  8, }, // America/Detroit
    {   56, 5739,   23950,  8, }, // America/Dominica
    {   56,  893,   23958,  8, }, // America/Edmonton
    {   56, 7869,   23966,  8, }, // America/Eirunepe
    {   56, 5441,   23974, 11, }, // America/El_Salvador
    {   56, 7886,   23985, 11, }, // America/Fort_Nelson
    {   56, 7906,   23996, 10, }, // America/Fortaleza
    {   56, 7924,   24006,  8, }, // America/Glace_Bay
    {   56, 6881,   24014,  6, }, // America/Goose_Bay
    {   56, 4612,   24020, 15, }, // America/Grand_Turk
    {   56, 5770,   24035,  8, }, // America/Grenada
    {   56, 5786,   24043, 10, }, // America/Guadeloupe
    {   56, 3760,   24053, 11, }, // America/Guatemala
    {   56, 4841,   24064, 12, }, // America/Guayaquil
    {   56, 6932,   17819,  7, }, // America/Guyana
    {   56, 1939,   24076, 13, }, // America/Halifax
    {   56, 2281,   17835,  6, }, // America/Havana
    {   56, 7942,   24089, 11, }, // America/Hermosillo
    {   56,  348,   24100, 15, }, // America/Indiana/Indianapolis
    {   56,  481,   24115, 16, }, // America/Indiana/Knox
    {   56, 7961,   24131, 20, }, // America/Indiana/Marengo
    {   56, 7985,   24151, 25, }, // America/Indiana/Petersburg
    {   56, 8012,   24176, 19, }, // America/Indiana/Tell_City
    {   56, 8038,   24195, 18, }, // America/Indiana/Vevay
    {   56, 8060,   24213, 22, }, // America/Indiana/Vincennes
    {   56, 8086,   24235, 21, }, // America/Indiana/Winamac
    {   56, 8110,   24256,  8, }, // America/Inuvik
    {   56,  660,   24264, 10, }, // America/Iqaluit
    {   56, 2799,   18015,  7, }, // America/Jamaica
    {   56, 5380,   24274,  4, }, // America/Juneau
    {   56,  521,   24278,  7, }, // America/Kentucky/Louisville
    {   56, 8125,   24285, 19, }, // America/Kentucky/Monticello
    {   56, 5704,   24304, 13, }, // America/Kralendijk
    {   56, 4376,   18067,  6, }, // America/La_Paz
    {   56, 7169,   18073,  4, }, // America/Lima
    {   56, 3239,   24317, 14, }, // America/Los_Angeles
    {   56, 5932,   24331, 27, }, // America/Lower_Princes
    {   56, 8153,   24358,  7, }, // America/Maceio
    {   56, 8168,   24365,  9, }, // America/Managua
    {   56, 1908,   24374,  6, }, // America/Manaus
    {   56, 5897,   24380,  6, }, // America/Marigot
    {   56, 5805,   24386,  8, }, // America/Martinique
    {   56, 8184,   24394,  8, }, // America/Matamoros
    {   56, 2917,   24402,  8, }, // America/Mazatlan
    {   56, 8202,   24410,  8, }, // America/Menominee
    {   56, 8220,   24418,  6, }, // America/Merida
    {   56, 8235,   24424, 10, }, // America/Metlakatla
    {   56, 2949,   24434, 13, }, // America/Mexico_City
    {   56, 4391,   24447,  6, }, // America/Miquelon
    {   56, 8254,   24453,  7, }, // America/Moncton
    {   56, 8270,   24460,  7, }, // America/Monterrey
    {   56, 4098,   24467, 10, }, // America/Montevideo
    {   56, 5824,   24477,  9, }, // America/Montserrat
    {   56, 5481,   24486,  5, }, // America/Nassau
    {   56, 2379,   24491,  9, }, // America/New_York
    {   56, 8288,   18168,  3, }, // America/Nome
    {   56, 1850,   24500,  8, }, // America/Noronha
    {   56, 8301,   24508, 19, }, // America/North_Dakota/Beulah
    {   56, 8329,   24527, 18, }, // America/North_Dakota/Center
    {   56, 8357,   24545, 23, }, // America/North_Dakota/New_Salem
    {   56,  393,   18338,  3, }, // America/Nuuk
    {   56, 8388,   24568,  7, }, // America/Ojinaga
    {   56, 2356,   18349,  6, }, // America/Panama
    {   56, 6173,   24575, 12, }, // America/Paramaribo
    {   56, 2973,   18365,  7, }, // America/Phoenix
    {   56, 3945,   24587, 16, }, // America/Port-au-Prince
    {   56, 5954,   24603, 14, }, // America/Port_of_Spain
    {   56, 8404,   24617, 14, }, // America/Porto_Velho
    {   56, 5843,   24631, 14, }, // America/Puerto_Rico
    {   56, 4030,   24645, 13, }, // America/Punta_Arenas
    {   56, 8424,   24658, 16, }, // America/Rankin_Inlet
    {   56, 8445,   24674,  6, }, // America/Recife
    {   56, 1995,   24680,  6, }, // America/Regina
    {   56, 8460,   24686,  7, }, // America/Resolute
    {   56,  695,   24693, 12, }, // America/Rio_Branco
    {   56, 8477,   24705,  8, }, // America/Santarem
    {   56, 2201,   24713, 11, }, // America/Santiago
    {   56, 6111,   24724, 16, }, // America/Santo_Domingo
    {   56, 1878,   24740,  9, }, // America/Sao_Paulo
    {   56, 6899,   24749, 16, }, // America/Scoresbysund
    {   56, 8494,   24765,  5, }, // America/Sitka
    {   56, 8508,   24770, 16, }, // America/St_Barthelemy
    {   56, 2061,   24786, 10, }, // America/St_Johns
    {   56, 5863,   24796, 11, }, // America/St_Kitts
    {   56, 5880,   24807, 14, }, // America/St_Lucia
    {   56,  855,   24821, 11, }, // America/St_Thomas
    {   56, 5913,   24832, 15, }, // America/St_Vincent
    {   56, 8530,   24847, 14, }, // America/Swift_Current
    {   56, 5461,   24861, 11, }, // America/Tegucigalpa
    {   56, 5756,   24872,  4, }, // America/Thule
    {   56,  313,   24876,  9, }, // America/Tijuana
    {   56,  608,   24885,  7, }, // America/Toronto
    {   56, 5688,   24892,  6, }, // America/Tortola
    {   56, 2093,   24898, 11, }, // America/Vancouver
    {   56, 2144,   24909, 11, }, // America/Whitehorse
    {   56,  734,   24920,  7, }, // America/Winnipeg
    {   56, 8552,   24927,  9, }, // America/Yakutat
    {   56, 6001,   24936,  5, }, // Antarctica/Casey
    {   56, 6094,   24941,  5, }, // Antarctica/Davis
    {   56, 6133,   24946, 15, }, // Antarctica/DumontDUrville
    {   56, 8568,   24961, 14, }, // Antarctica/Macquarie
    {   56, 7066,   24975,  4, }, // Antarctica/Mawson
    {   56, 7113,   24979, 11, }, // Antarctica/McMurdo
    {   56, 8589,   24990,  6, }, // Antarctica/Palmer
    {   56, 7226,   24996,  5, }, // Antarctica/Rothera
    {   56, 7295,   25001,  6, }, // Antarctica/Syowa
    {   56, 8607,   25007,  4, }, // Antarctica/Troll
    {   56, 7374,   25011,  6, }, // Antarctica/Vostok
    {   56, 1429,   25017, 14, }, // Arctic/Longyearbyen
    {   56, 5570,   25031,  5, }, // Asia/Aden
    {   56, 5395,   18812,  6, }, // Asia/Almaty
    {   56, 3968,   18818,  6, }, // Asia/Amman
    {   56, 5511,   25036,  9, }, // Asia/Anadyr
    {   56, 5523,   25045,  5, }, // Asia/Aqtau
    {   56, 5368,   25050,  6, }, // Asia/Aqtobe
    {   56,  964,   25056,  7, }, // Asia/Ashgabat
    {   56, 8624,   25063,  6, }, // Asia/Atyrau
    {   56, 3607,   18863,  6, }, // Asia/Baghdad
    {   56, 5534,   25069,  7, }, // Asia/Bahrain
    {   56, 3653,   17644,  4, }, // Asia/Baku
    {   56, 4466,   25076,  7, }, // Asia/Bangkok
    {   56, 3571,   25083,  7, }, // Asia/Barnaul
    {   56, 4086,   25090,  6, }, // Asia/Beirut
    {   56, 3778,   25096,  6, }, // Asia/Bishkek
    {   56, 5989,   25102,  7, }, // Asia/Brunei
    {   56, 4601,   25109,  4, }, // Asia/Chita
    {   56, 4511,   18915,  6, }, // Asia/Colombo
    {   56, 4540,   25113,  9, }, // Asia/Damascus
    {   56, 1093,   18930,  4, }, // Asia/Dhaka
    {   56, 6192,   25122,  4, }, // Asia/Dili
    {   56, 3596,   25126,  5, }, // Asia/Dubai
    {   56, 6159,   25131,  7, }, // Asia/Dushanbe
    {   56, 8636,   25138, 10, }, // Asia/Famagusta
    {   56, 8651,   18960,  4, }, // Asia/Gaza
    {   56, 4795,   25148,  7, }, // Asia/Hebron
    {   56, 1259,   25155, 13, }, // Asia/Ho_Chi_Minh
    {   56, 2704,   18985,  5, }, // Asia/Hong_Kong
    {   56, 4771,   25168,  5, }, // Asia/Hovd
    {   56, 4184,   25173,  8, }, // Asia/Irkutsk
    {   56, 7006,   25181,  8, }, // Asia/Jakarta
    {   56, 6992,   25189,  8, }, // Asia/Jayapura
    {   56, 1290,   25197,  9, }, // Asia/Jerusalem
    {   56, 3560,   19032,  5, }, // Asia/Kabul
    {   56, 4316,   25206,  8, }, // Asia/Kamchatka
    {   56, 4239,   25214,  5, }, // Asia/Karachi
    {   56, 1185,   25219,  9, }, // Asia/Kathmandu
    {   56, 8661,   25228, 10, }, // Asia/Khandyga
    {   56,  992,   25238,  7, }, // Asia/Kolkata
    {   56, 4197,   25245, 16, }, // Asia/Krasnoyarsk
    {   56, 4859,   25261, 13, }, // Asia/Kuala_Lumpur
    {   56, 5976,   19100,  5, }, // Asia/Kuching
    {   56, 5547,   25274,  6, }, // Asia/Kuwait
    {   56, 1211,   25280,  7, }, // Asia/Macau
    {   56, 4017,   25287,  9, }, // Asia/Magadan
    {   56, 1349,   25296,  8, }, // Asia/Makassar
    {   56, 7182,   25304,  8, }, // Asia/Manila
    {   56, 6920,   25312,  5, }, // Asia/Muscat
    {   56, 2554,   25317,  9, }, // Asia/Nicosia
    {   56, 8675,   25326, 12, }, // Asia/Novokuznetsk
    {   56, 4135,   25338, 12, }, // Asia/Novosibirsk
    {   56, 4229,   25350,  6, }, // Asia/Omsk
    {   56, 7145,   25356,  4, }, // Asia/Oral
    {   56, 6961,   25360,  6, }, // Asia/Phnom_Penh
    {   56, 8693,   25366, 13, }, // Asia/Pontianak
    {   56, 4214,   25379,  9, }, // Asia/Pyongyang
    {   56, 5559,   25388,  5, }, // Asia/Qatar
    {   56, 8708,   25393,  9, }, // Asia/Qostanay
    {   56, 4269,   25402,  9, }, // Asia/Qyzylorda
    {   56, 3584,   25411,  6, }, // Asia/Riyadh
    {   56, 4408,   25417,  7, }, // Asia/Sakhalin
    {   56, 7245,   25424,  7, }, // Asia/Samarkand
    {   56, 3279,   25431,  4, }, // Asia/Seoul
    {   56, 1053,   25435,  6, }, // Asia/Shanghai
    {   56, 3300,   25441,  9, }, // Asia/Singapore
    {   56, 4297,   25450, 16, }, // Asia/Srednekolymsk
    {   56, 3263,   25466,  6, }, // Asia/Taipei
    {   56, 4781,   25472,  7, }, // Asia/Tashkent
    {   56, 3915,   25479,  8, }, // Asia/Tbilisi
    {   56, 2772,   25487,  7, }, // Asia/Tehran
    {   56, 1317,   19324,  6, }, // Asia/Thimphu
    {   56, 2821,   25494,  5, }, // Asia/Tokyo
    {   56, 4572,   25499,  6, }, // Asia/Tomsk
    {   56, 1021,   25505,  9, }, // Asia/Ulaanbaatar
    {   56, 1159,   25514,  6, }, // Asia/Urumqi
    {   56, 8722,   25520, 10, }, // Asia/Ust-Nera
    {   56, 6977,   25530, 12, }, // Asia/Vientiane
    {   56, 4710,   25542, 12, }, // Asia/Vladivostok
    {   56, 4828,   25554, 10, }, // Asia/Yakutsk
    {   56, 1235,   25564,  7, }, // Asia/Yangon
    {   56, 3883,   25571, 16, }, // Asia/Yekaterinburg
    {   56, 3747,   25587,  9, }, // Asia/Yerevan
    {   56, 3663,   25596,  6, }, // Atlantic/Azores
    {   56, 5671,   25602,  7, }, // Atlantic/Bermuda
    {   56, 6628,   25609,  6, }, // Atlantic/Canary
    {   56, 3727,   25615,  9, }, // Atlantic/Cape_Verde
    {   56, 1395,   25624,  6, }, // Atlantic/Faroe
    {   56, 8736,   25630,  7, }, // Atlantic/Madeira
    {   56, 2748,   25637,  8, }, // Atlantic/Reykjavik
    {   56, 7272,   25645, 15, }, // Atlantic/South_Georgia
    {   56, 6821,   25660, 13, }, // Atlantic/St_Helena
    {   56, 6644,   25673,  7, }, // Atlantic/Stanley
    {   56, 1670,   25680, 10, }, // Australia/Adelaide
    {   56, 1635,   25690,  8, }, // Australia/Brisbane
    {   56, 1799,   25698, 11, }, // Australia/Broken_Hill
    {   56, 1583,   25709,  6, }, // Australia/Darwin
    {   56, 3637,   19551,  6, }, // Australia/Eucla
    {   56, 1516,   25715,  7, }, // Australia/Hobart
    {   56, 8753,   25722, 10, }, // Australia/Lindeman
    {   56, 1547,   25732,  8, }, // Australia/Lord_Howe
    {   56, 1727,   25740,  8, }, // Australia/Melbourne
    {   56, 1762,   25748,  5, }, // Australia/Perth
    {   56, 1463,   25753,  5, }, // Australia/Sydney
    {   56, 6441,   25758, 10, }, // Europe/Amsterdam
    {   56, 6216,   25768, 10, }, // Europe/Andorra
    {   56, 3620,   25778,  9, }, // Europe/Astrakhan
    {   56, 2300,   19625,  6, }, // Europe/Athens
    {   56, 6336,   25787,  8, }, // Europe/Belgrade
    {   56, 4757,   25795,  7, }, // Europe/Berlin
    {   56, 6488,   25802, 12, }, // Europe/Bratislava
    {   56, 2167,   25814,  8, }, // Europe/Brussels
    {   56, 3928,   25822,  9, }, // Europe/Bucharest
    {   56, 3806,   25831,  9, }, // Europe/Budapest
    {   56, 8772,   25840,  8, }, // Europe/Busingen
    {   56, 2583,   25848,  8, }, // Europe/Chisinau
    {   56, 6289,   25856, 10, }, // Europe/Copenhagen
    {   56, 2338,   19708,  6, }, // Europe/Dublin
    {   56, 6307,   25866, 11, }, // Europe/Gibraltar
    {   56, 8788,   25877, 10, }, // Europe/Guernsey
    {   56, 6612,   25887,  9, }, // Europe/Helsinki
    {   56, 8804,   25896, 12, }, // Europe/Isle_of_Man
    {   56, 1130,   19749,  9, }, // Europe/Istanbul
    {   56, 8823,   25908,  6, }, // Europe/Jersey
    {   56, 3979,   25914, 12, }, // Europe/Kaliningrad
    {   56, 8929,   25926,  5, }, // Europe/Kirov
    {   56, 2527,   25931,  6, }, // Europe/Kyiv
    {   56, 3213,   25937,  5, }, // Europe/Lisbon
    {   56, 6506,   25942, 10, }, // Europe/Ljubljana
    {   56, 2501,   25952,  5, }, // Europe/London
    {   56, 6365,   25957, 12, }, // Europe/Luxembourg
    {   56, 4892,   25969,  7, }, // Europe/Madrid
    {   56, 6397,   19824,  6, }, // Europe/Malta
    {   56, 6582,   25976, 11, }, // Europe/Mariehamn
    {   56, 3693,   25987,  6, }, // Europe/Minsk
    {   56, 6410,   19846,  6, }, // Europe/Monaco
    {   56, 3537,   19852,  5, }, // Europe/Moscow
    {   56, 6458,   25993,  4, }, // Europe/Oslo
    {   56, 4284,   25997,  7, }, // Europe/Paris
    {   56, 6424,   26004,  9, }, // Europe/Podgorica
    {   56, 6275,   26013,  5, }, // Europe/Prague
    {   56, 8837,   26018,  4, }, // Europe/Riga
    {   56, 6324,   25628,  3, }, // Europe/Rome
    {   56, 4331,   26022,  6, }, // Europe/Samara
    {   56, 6470,   26028, 10, }, // Europe/San_Marino
    {   56, 6245,   26038,  8, }, // Europe/Sarajevo
    {   56, 4451,   26046,  7, }, // Europe/Saratov
    {   56, 8849,   26053, 10, }, // Europe/Simferopol
    {   56, 6383,   26063,  7, }, // Europe/Skopje
    {   56, 6599,   26070,  7, }, // Europe/Sofia
    {   56, 6523,   26077,  7, }, // Europe/Stockholm
    {   56, 8867,   26084,  6, }, // Europe/Tallinn
    {   56, 6202,   26090,  6, }, // Europe/Tirane
    {   56, 8882,   26096, 12, }, // Europe/Ulyanovsk
    {   56, 6352,   19973,  5, }, // Europe/Vaduz
    {   56, 6567,   26108,  7, }, // Europe/Vatican
    {   56, 6231,   19985,  7, }, // Europe/Vienna
    {   56, 8899,   26115,  7, }, // Europe/Vilnius
    {   56, 4727,   26122, 11, }, // Europe/Volgograd
    {   56, 3190,   26133,  6, }, // Europe/Warsaw
    {   56, 6261,   26139,  7, }, // Europe/Zagreb
    {   56, 6540,   26146,  5, }, // Europe/Zurich
    {   56, 5078,   26151, 13, }, // Indian/Antananarivo
    {   56, 6947,   26164,  5, }, // Indian/Chagos
    {   56, 6046,   26169,  8, }, // Indian/Christmas
    {   56, 6063,   26177,  5, }, // Indian/Cocos
    {   56, 5029,   26182,  6, }, // Indian/Comoro
    {   56, 6661,   26188,  9, }, // Indian/Kerguelen
    {   56, 7260,   20077,  4, }, // Indian/Mahe
    {   56, 7050,   20081,  8, }, // Indian/Maldives
    {   56, 4069,   26197,  6, }, // Indian/Mauritius
    {   56, 5098,   26203,  7, }, // Indian/Mayotte
    {   56, 7211,   26210,  9, }, // Indian/Reunion
    {   56, 4422,   20113,  6, }, // Pacific/Apia
    {   56,  932,   26219,  9, }, // Pacific/Auckland
    {   56, 3706,   26228,  9, }, // Pacific/Bougainville
    {   56, 3015,   26237,  7, }, // Pacific/Chatham
    {   56, 3157,   20143,  3, }, // Pacific/Chuuk
    {   56, 2237,   26244,  6, }, // Pacific/Easter
    {   56, 7360,   26250,  5, }, // Pacific/Efate
    {   56, 7327,   26255,  9, }, // Pacific/Fakaofo
    {   56, 3902,   20163,  4, }, // Pacific/Fiji
    {   56, 7343,   20167,  8, }, // Pacific/Funafuti
    {   56, 6678,   26264, 13, }, // Pacific/Galapagos
    {   56, 6696,   26277, 10, }, // Pacific/Gambier
    {   56, 3822,   26287, 13, }, // Pacific/Guadalcanal
    {   56, 6018,   24369,  6, }, // Pacific/Guam
    {   56, 2723,   26300,  7, }, // Pacific/Honolulu
    {   56, 3049,   26307, 10, }, // Pacific/Kanton
    {   56, 3998,   26317, 10, }, // Pacific/Kiritimati
    {   56, 7019,   26327,  7, }, // Pacific/Kosrae
    {   56, 2842,   26334, 11, }, // Pacific/Kwajalein
    {   56, 4877,   26345,  6, }, // Pacific/Majuro
    {   56, 4051,   26351,  9, }, // Pacific/Marquesas
    {   56, 8914,   26360,  7, }, // Pacific/Midway
    {   56, 7084,   26367,  5, }, // Pacific/Nauru
    {   56, 7132,   26372,  6, }, // Pacific/Niue
    {   56, 4168,   26378,  4, }, // Pacific/Norfolk
    {   56, 7098,   26382,  7, }, // Pacific/Noumea
    {   56, 3126,   26389, 13, }, // Pacific/Pago_Pago
    {   56, 7155,   20291,  5, }, // Pacific/Palau
    {   56, 7194,   26402, 11, }, // Pacific/Pitcairn
    {   56, 3096,   26413,  6, }, // Pacific/Pohnpei
    {   56, 4807,   26419, 12, }, // Pacific/Port_Moresby
    {   56, 6076,   26431, 11, }, // Pacific/Rarotonga
    {   56, 6031,   26442,  6, }, // Pacific/Saipan
    {   56, 7312,   20342,  6, }, // Pacific/Tahiti
    {   56, 6712,   26448,  8, }, // Pacific/Tarawa
    {   56, 4583,   26456, 10, }, // Pacific/Tongatapu
    {   56, 7392,   26466,  5, }, // Pacific/Wake
    {   56, 7405,   26471,  4, }, // Pacific/Wallis
    {   58,   14,       0,  6, }, // Africa/Asmara Basaa/Latin/Cameroon
    {   58, 4435,       6,  8, }, // Africa/Sao_Tome
    {   58, 4252,      14,  8, }, // America/Asuncion
    {   58,  237,      22,  8, }, // America/Atikokan
    {   58, 7623,      30, 17, }, // America/Bahia_Banderas
    {   58, 3868,      47,  6, }, // America/Cancun
    {   58, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   58, 5723,      66,  7, }, // America/Curacao
    {   58,  481,      73, 13, }, // America/Indiana/Knox
    {   58, 7961,      86, 16, }, // America/Indiana/Marengo
    {   58, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   58, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   58, 8038,     139, 14, }, // America/Indiana/Vevay
    {   58, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   58, 8086,     171, 16, }, // America/Indiana/Winamac
    {   58, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   58, 5932,     207, 22, }, // America/Lower_Princes
    {   58, 8220,     229,  6, }, // America/Merida
    {   58, 1850,     235, 19, }, // America/Noronha
    {   58, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {   58, 8329,     274, 20, }, // America/North_Dakota/Center
    {   58, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {   58,  393,     317,  4, }, // America/Nuuk
    {   58, 6899,     321, 16, }, // America/Scoresbysund
    {   58, 8508,     337, 14, }, // America/St_Barthelemy
    {   58, 2061,     351, 10, }, // America/St_Johns
    {   58, 5863,     361,  9, }, // America/St_Kitts
    {   58, 5880,     370,  9, }, // America/St_Lucia
    {   58,  855,     379, 10, }, // America/St_Thomas
    {   58, 5913,     389, 11, }, // America/St_Vincent
    {   58, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {   58, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {   58, 1185,     448,  9, }, // Asia/Kathmandu
    {   58,  992,     457,  7, }, // Asia/Kolkata
    {   58, 1211,     464,  5, }, // Asia/Macau
    {   58, 1235,     477,  6, }, // Asia/Yangon
    {   58, 1395,     483,  5, }, // Atlantic/Faroe
    {   58, 6821,     488, 10, }, // Atlantic/St_Helena
    {   58, 2527,     514,  4, }, // Europe/Kyiv
    {   58, 7211,     547,  7, }, // Indian/Reunion
    {   58, 3157,     554,  5, }, // Pacific/Chuuk
    {   58, 3096,     586,  7, }, // Pacific/Pohnpei
    {   59,   14,       0,  6, }, // Africa/Asmara Bashkir/Cyrillic/Russia
    {   59, 4435,       6,  8, }, // Africa/Sao_Tome
    {   59, 4252,      14,  8, }, // America/Asuncion
    {   59,  237,      22,  8, }, // America/Atikokan
    {   59, 7623,      30, 17, }, // America/Bahia_Banderas
    {   59, 3868,      47,  6, }, // America/Cancun
    {   59, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   59, 5723,      66,  7, }, // America/Curacao
    {   59,  481,      73, 13, }, // America/Indiana/Knox
    {   59, 7961,      86, 16, }, // America/Indiana/Marengo
    {   59, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   59, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   59, 8038,     139, 14, }, // America/Indiana/Vevay
    {   59, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   59, 8086,     171, 16, }, // America/Indiana/Winamac
    {   59, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   59, 5932,     207, 22, }, // America/Lower_Princes
    {   59, 8220,     229,  6, }, // America/Merida
    {   59, 1850,     235, 19, }, // America/Noronha
    {   59, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {   59, 8329,     274, 20, }, // America/North_Dakota/Center
    {   59, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {   59,  393,     317,  4, }, // America/Nuuk
    {   59, 6899,     321, 16, }, // America/Scoresbysund
    {   59, 8508,     337, 14, }, // America/St_Barthelemy
    {   59, 2061,     351, 10, }, // America/St_Johns
    {   59, 5863,     361,  9, }, // America/St_Kitts
    {   59, 5880,     370,  9, }, // America/St_Lucia
    {   59,  855,     379, 10, }, // America/St_Thomas
    {   59, 5913,     389, 11, }, // America/St_Vincent
    {   59, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {   59, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {   59, 1185,     448,  9, }, // Asia/Kathmandu
    {   59,  992,     457,  7, }, // Asia/Kolkata
    {   59, 1211,     464,  5, }, // Asia/Macau
    {   59, 1235,     477,  6, }, // Asia/Yangon
    {   59, 1395,     483,  5, }, // Atlantic/Faroe
    {   59, 6821,     488, 10, }, // Atlantic/St_Helena
    {   59, 2527,     514,  4, }, // Europe/Kyiv
    {   59, 7211,     547,  7, }, // Indian/Reunion
    {   59, 3157,     554,  5, }, // Pacific/Chuuk
    {   59, 3096,     586,  7, }, // Pacific/Pohnpei
    {   60, 6760,   21292,  5, }, // Africa/Accra Basque/Latin/Spain
    {   60, 5059,    3902, 11, }, // Africa/Addis_Ababa
    {   60, 7420,   26475,  5, }, // Africa/Algiers
    {   60,   14,       0,  6, }, // Africa/Asmara
    {   60, 5257,   26480,  5, }, // Africa/Bangui
    {   60, 2320,    4690,  5, }, // Africa/Cairo
    {   60, 6773,   26485,  7, }, // Africa/Conakry
    {   60, 5043,   26492,  7, }, // Africa/Djibouti
    {   60, 5243,   21372,  5, }, // Africa/Douala
    {   60, 5166,   26499,  5, }, // Africa/El_Aaiun
    {   60, 4524,   26504,  7, }, // Africa/Khartoum
    {   60, 5113,   26511,  8, }, // Africa/Mogadishu
    {   60, 5271,   26519,  9, }, // Africa/Ndjamena
    {   60, 5354,   26528,  6, }, // Africa/Niamey
    {   60, 6803,   20442,  7, }, // Africa/Nouakchott
    {   60, 6727,   26534,  8, }, // Africa/Ouagadougou
    {   60, 5225,   26542, 10, }, // Africa/Porto-Novo
    {   60, 4435,       6,  8, }, // Africa/Sao_Tome
    {   60, 5607,   26552,  7, }, // America/Anguilla
    {   60, 4554,   26559,  9, }, // America/Araguaina
    {   60,  270,   20479,  7, }, // America/Argentina/Cordoba
    {   60, 7571,   20486,  7, }, // America/Argentina/Tucuman
    {   60, 4252,      14,  8, }, // America/Asuncion
    {   60,  237,      22,  8, }, // America/Atikokan
    {   60, 7623,      30, 17, }, // America/Bahia_Banderas
    {   60, 3868,      47,  6, }, // America/Cancun
    {   60, 5496,   26568,  6, }, // America/Cayman
    {   60, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   60, 3791,   26574,  6, }, // America/Cuiaba
    {   60, 5723,      66,  7, }, // America/Curacao
    {   60, 5739,   21666,  8, }, // America/Dominica
    {   60, 7869,   26580,  8, }, // America/Eirunepe
    {   60, 5786,   20526,  9, }, // America/Guadeloupe
    {   60, 2281,   20538,  6, }, // America/Havana
    {   60,  481,      73, 13, }, // America/Indiana/Knox
    {   60, 7961,      86, 16, }, // America/Indiana/Marengo
    {   60, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   60, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   60, 8038,     139, 14, }, // America/Indiana/Vevay
    {   60, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   60, 8086,     171, 16, }, // America/Indiana/Winamac
    {   60, 2799,    3995,  7, }, // America/Jamaica
    {   60, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   60, 5932,     207, 22, }, // America/Lower_Princes
    {   60, 8153,   26588,  6, }, // America/Maceio
    {   60, 5805,   26594,  9, }, // America/Martinique
    {   60, 2917,   26603,  8, }, // America/Mazatlan
    {   60, 8220,     229,  6, }, // America/Merida
    {   60, 2949,   26611, 12, }, // America/Mexico_City
    {   60, 4391,   26623,  8, }, // America/Miquelon
    {   60, 1850,     235, 19, }, // America/Noronha
    {   60, 8301,   26631, 19, }, // America/North_Dakota/Beulah
    {   60, 8329,   26650, 19, }, // America/North_Dakota/Center
    {   60, 8357,   26669, 22, }, // America/North_Dakota/New_Salem
    {   60,  393,     317,  4, }, // America/Nuuk
    {   60, 5954,   26691, 13, }, // America/Port_of_Spain
    {   60, 8477,   26704,  8, }, // America/Santarem
    {   60, 1878,   26712,  9, }, // America/Sao_Paulo
    {   60, 6899,     321, 16, }, // America/Scoresbysund
    {   60, 8508,   26721, 16, }, // America/St_Barthelemy
    {   60, 2061,   26737, 12, }, // America/St_Johns
    {   60, 5863,   26749, 11, }, // America/St_Kitts
    {   60, 5880,   26760, 11, }, // America/St_Lucia
    {   60,  855,   26771, 12, }, // America/St_Thomas
    {   60, 5913,   26783, 13, }, // America/St_Vincent
    {   60, 5756,   26796,  7, }, // America/Thule
    {   60, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {   60, 5523,    5609,  5, }, // Asia/Aqtau
    {   60, 5368,    7606,  6, }, // Asia/Aqtobe
    {   60,  964,   26803,  7, }, // Asia/Ashgabat
    {   60, 3607,    4162,  6, }, // Asia/Baghdad
    {   60, 3778,   26810,  6, }, // Asia/Bishkek
    {   60, 4601,   26816,  5, }, // Asia/Chita
    {   60, 4511,    5637,  7, }, // Asia/Colombo
    {   60, 4540,   26821,  7, }, // Asia/Damascus
    {   60, 6159,   26828,  7, }, // Asia/Dushanbe
    {   60, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {   60, 4771,   26835,  5, }, // Asia/Hovd
    {   60, 4316,   26840,  9, }, // Asia/Kamchatka
    {   60, 4239,   26849,  7, }, // Asia/Karachi
    {   60, 1185,    5711,  8, }, // Asia/Kathmandu
    {   60, 8661,   26856,  8, }, // Asia/Khandyga
    {   60,  992,   26864,  7, }, // Asia/Kolkata
    {   60, 4197,   26871, 11, }, // Asia/Krasnoyarsk
    {   60, 1211,    4321,  5, }, // Asia/Macau
    {   60, 6920,   26882,  6, }, // Asia/Muscat
    {   60, 2554,    5738,  7, }, // Asia/Nicosia
    {   60, 4214,   26888,  9, }, // Asia/Pyongyang
    {   60, 8708,     469,  8, }, // Asia/Qostanay
    {   60, 4269,   26897,  9, }, // Asia/Qyzylorda
    {   60, 3584,    7787,  4, }, // Asia/Riyadh
    {   60, 7245,   26906, 10, }, // Asia/Samarkand
    {   60, 3279,    7798,  4, }, // Asia/Seoul
    {   60, 3300,   20900,  8, }, // Asia/Singapore
    {   60, 4297,    7817, 13, }, // Asia/Srednekolymsk
    {   60, 4781,   26916,  7, }, // Asia/Tashkent
    {   60, 2772,    7842,  7, }, // Asia/Tehran
    {   60, 2821,    4356,  5, }, // Asia/Tokyo
    {   60, 1021,   26923, 10, }, // Asia/Ulaanbaatar
    {   60, 1159,   26933,  6, }, // Asia/Urumqi
    {   60, 6977,   26939,  8, }, // Asia/Vientiane
    {   60, 4828,    7873,  7, }, // Asia/Yakutsk
    {   60, 1235,     477,  6, }, // Asia/Yangon
    {   60, 3883,    4380, 13, }, // Asia/Yekaterinburg
    {   60, 3747,   26947,  6, }, // Asia/Yerevan
    {   60, 3663,   26953,  7, }, // Atlantic/Azores
    {   60, 6628,   26960,  8, }, // Atlantic/Canary
    {   60, 3727,   26968, 10, }, // Atlantic/Cape_Verde
    {   60, 1395,     483,  5, }, // Atlantic/Faroe
    {   60, 7272,   26978, 19, }, // Atlantic/South_Georgia
    {   60, 6821,   26997, 12, }, // Atlantic/St_Helena
    {   60, 2300,   27009,  6, }, // Europe/Athens
    {   60, 6336,   27015,  7, }, // Europe/Belgrade
    {   60, 2167,   27022,  7, }, // Europe/Brussels
    {   60, 3928,   27029,  8, }, // Europe/Bucharest
    {   60, 8772,   27037,  8, }, // Europe/Busingen
    {   60, 6289,    4475,  9, }, // Europe/Copenhagen
    {   60, 8788,   27045,  9, }, // Europe/Guernsey
    {   60, 8804,   27054, 11, }, // Europe/Isle_of_Man
    {   60, 2527,    8148,  4, }, // Europe/Kyiv
    {   60, 3213,   21137,  6, }, // Europe/Lisbon
    {   60, 2501,   21152,  7, }, // Europe/London
    {   60, 6365,   27065, 10, }, // Europe/Luxembourg
    {   60, 4892,   27075,  6, }, // Europe/Madrid
    {   60, 6410,    6044,  6, }, // Europe/Monaco
    {   60, 3537,   27081,  5, }, // Europe/Moscow
    {   60, 6275,   21185,  5, }, // Europe/Prague
    {   60, 6324,   27086,  6, }, // Europe/Rome
    {   60, 6202,   21213,  6, }, // Europe/Tirane
    {   60, 8882,   27092,  9, }, // Europe/Ulyanovsk
    {   60, 6567,   27101, 14, }, // Europe/Vatican
    {   60, 6231,   21230,  5, }, // Europe/Vienna
    {   60, 3190,   21244,  8, }, // Europe/Warsaw
    {   60, 6540,    4577,  6, }, // Europe/Zurich
    {   60, 7260,   27115,  4, }, // Indian/Mahe
    {   60, 7050,   27119,  8, }, // Indian/Maldives
    {   60, 4069,   27127,  8, }, // Indian/Mauritius
    {   60, 7211,     547,  7, }, // Indian/Reunion
    {   60, 3157,     554,  5, }, // Pacific/Chuuk
    {   60, 2237,   27135, 13, }, // Pacific/Easter
    {   60, 7360,   27148,  5, }, // Pacific/Efate
    {   60, 6678,   27153, 10, }, // Pacific/Galapagos
    {   60, 2723,    4611,  8, }, // Pacific/Honolulu
    {   60, 3049,    4619,  9, }, // Pacific/Kanton
    {   60, 4051,   27163,  8, }, // Pacific/Marquesas
    {   60, 7098,    4628,  6, }, // Pacific/Noumea
    {   60, 3096,     586,  7, }, // Pacific/Pohnpei
    {   61, 6788,   27171,  7, }, // Africa/Abidjan Belarusian/Cyrillic/Belarus
    {   61, 6760,   27178,  4, }, // Africa/Accra
    {   61, 5059,   27182, 10, }, // Africa/Addis_Ababa
    {   61, 7420,   27192,  5, }, // Africa/Algiers
    {   61,   14,   27197,  6, }, // Africa/Asmara
    {   61,   44,     637,  6, }, // Africa/Bamako
    {   61, 5257,   27203,  5, }, // Africa/Bangui
    {   61, 6746,   27208,  7, }, // Africa/Banjul
    {   61, 7435,   27215,  5, }, // Africa/Bissau
    {   61, 4957,   27220,  8, }, // Africa/Blantyre
    {   61, 5287,   27228,  9, }, // Africa/Brazzaville
    {   61, 4922,     676,  9, }, // Africa/Bujumbura
    {   61, 2320,   27237,  4, }, // Africa/Cairo
    {   61, 4117,     689, 10, }, // Africa/Casablanca
    {   61, 7449,     699,  5, }, // Africa/Ceuta
    {   61, 6773,   27241,  7, }, // Africa/Conakry
    {   61, 6840,     711,  5, }, // Africa/Dakar
    {   61, 5130,   27248, 12, }, // Africa/Dar_es_Salaam
    {   61, 5043,   27260,  7, }, // Africa/Djibouti
    {   61, 5243,     734,  5, }, // Africa/Douala
    {   61, 5166,   27267,  7, }, // Africa/El_Aaiun
    {   61, 6853,   27274,  7, }, // Africa/Freetown
    {   61, 4906,   27281,  8, }, // Africa/Gaborone
    {   61, 5015,   27289,  6, }, // Africa/Harare
    {   61, 4479,   27295, 10, }, // Africa/Johannesburg
    {   61, 4499,   27305,  5, }, // Africa/Juba
    {   61, 5151,     783,  7, }, // Africa/Kampala
    {   61, 4524,     790,  6, }, // Africa/Khartoum
    {   61, 4973,   27310,  6, }, // Africa/Kigali
    {   61, 5306,   27316,  7, }, // Africa/Kinshasa
    {   61, 4744,   27323,  5, }, // Africa/Lagos
    {   61, 5336,   27328,  9, }, // Africa/Libreville
    {   61, 6869,   27337,  4, }, // Africa/Lome
    {   61, 5211,     826,  6, }, // Africa/Luanda
    {   61, 4939,   27341,  9, }, // Africa/Lubumbashi
    {   61, 5001,     841,  6, }, // Africa/Lusaka
    {   61, 5322,   27350,  6, }, // Africa/Malabo
    {   61, 4987,     853,  6, }, // Africa/Maputo
    {   61, 5197,     859,  6, }, // Africa/Maseru
    {   61, 5182,   27356,  7, }, // Africa/Mbabane
    {   61, 5113,   27363,  8, }, // Africa/Mogadishu
    {   61, 7034,   27371,  8, }, // Africa/Monrovia
    {   61, 3853,   27379,  7, }, // Africa/Nairobi
    {   61, 5271,   27386,  8, }, // Africa/Ndjamena
    {   61, 5354,   27394,  6, }, // Africa/Niamey
    {   61, 6803,     908,  7, }, // Africa/Nouakchott
    {   61, 6727,     915,  8, }, // Africa/Ouagadougou
    {   61, 5225,   27400, 10, }, // Africa/Porto-Novo
    {   61, 4435,   27410,  8, }, // Africa/Sao_Tome
    {   61, 2866,   27418,  7, }, // Africa/Tripoli
    {   61, 6554,   27425,  5, }, // Africa/Tunis
    {   61, 4152,   27430,  7, }, // Africa/Windhoek
    {   61,  132,     960,  4, }, // America/Adak
    {   61, 3346,   27437,  8, }, // America/Anchorage
    {   61, 5607,   27445,  7, }, // America/Anguilla
    {   61, 5624,   27452,  7, }, // America/Antigua
    {   61, 4554,   27459,  9, }, // America/Araguaina
    {   61,  166,   27468, 12, }, // America/Argentina/Buenos_Aires
    {   61,   91,    1006,  9, }, // America/Argentina/Catamarca
    {   61,  270,   27480,  7, }, // America/Argentina/Cordoba
    {   61,  441,   27487,  5, }, // America/Argentina/Jujuy
    {   61, 7462,   27492,  8, }, // America/Argentina/La_Rioja
    {   61,  565,    1035,  7, }, // America/Argentina/Mendoza
    {   61, 7489,   27500, 12, }, // America/Argentina/Rio_Gallegos
    {   61, 7520,   27512,  6, }, // America/Argentina/Salta
    {   61, 7544,    1058,  8, }, // America/Argentina/San_Juan
    {   61, 5580,   27518,  8, }, // America/Argentina/San_Luis
    {   61, 7571,    1074,  7, }, // America/Argentina/Tucuman
    {   61, 7597,   27526,  5, }, // America/Argentina/Ushuaia
    {   61, 5640,    1087,  5, }, // America/Aruba
    {   61, 4252,   27531,  8, }, // America/Asuncion
    {   61,  237,   27539,  8, }, // America/Atikokan
    {   61, 3679,   27547,  4, }, // America/Bahia
    {   61, 7623,   27551, 16, }, // America/Bahia_Banderas
    {   61, 5654,   27567,  8, }, // America/Barbados
    {   61, 7646,    1139,  5, }, // America/Belem
    {   61, 5407,   27575,  5, }, // America/Belize
    {   61, 7660,    1149, 12, }, // America/Blanc-Sablon
    {   61, 7681,   27580,  9, }, // America/Boa_Vista
    {   61, 4361,   27589,  6, }, // America/Bogota
    {   61, 7699,   27595,  5, }, // America/Boise
    {   61, 7713,   27600, 12, }, // America/Cambridge_Bay
    {   61, 7735,   27612, 12, }, // America/Campo_Grande
    {   61, 3868,    1205,  6, }, // America/Cancun
    {   61, 4694,    1211,  7, }, // America/Caracas
    {   61, 4345,   27624,  5, }, // America/Cayenne
    {   61, 5496,   27629, 17, }, // America/Cayman
    {   61, 2260,   27646,  6, }, // America/Chicago
    {   61, 7756,   27652,  6, }, // America/Chihuahua
    {   61, 7774,   27658, 13, }, // America/Ciudad_Juarez
    {   61, 5422,   27671, 10, }, // America/Costa_Rica
    {   61, 7796,   27681,  7, }, // America/Creston
    {   61, 3791,   27688,  5, }, // America/Cuiaba
    {   61, 5723,   27693,  7, }, // America/Curacao
    {   61, 7812,   27700, 12, }, // America/Danmarkshavn
    {   61, 7833,   27712,  6, }, // America/Dawson
    {   61, 7848,   27718, 10, }, // America/Dawson_Creek
    {   61,  805,   27728,  6, }, // America/Denver
    {   61, 3465,   27734,  7, }, // America/Detroit
    {   61, 5739,   27741,  8, }, // America/Dominica
    {   61,  893,   27749,  8, }, // America/Edmonton
    {   61, 7869,   27757,  8, }, // America/Eirunepe
    {   61, 5441,   27765,  9, }, // America/El_Salvador
    {   61, 7886,   27774, 12, }, // America/Fort_Nelson
    {   61, 7906,   27786,  9, }, // America/Fortaleza
    {   61, 7924,   27795,  9, }, // America/Glace_Bay
    {   61, 6881,   27804,  7, }, // America/Goose_Bay
    {   61, 4612,   27811, 10, }, // America/Grand_Turk
    {   61, 5770,   27821,  7, }, // America/Grenada
    {   61, 5786,   27828,  9, }, // America/Guadeloupe
    {   61, 3760,   27837,  9, }, // America/Guatemala
    {   61, 4841,   27846,  8, }, // America/Guayaquil
    {   61, 6932,   27854,  5, }, // America/Guyana
    {   61, 1939,   27859,  8, }, // America/Halifax
    {   61, 2281,    1457,  6, }, // America/Havana
    {   61, 7942,   27867,  9, }, // America/Hermosillo
    {   61,  348,   27876, 12, }, // America/Indiana/Indianapolis
    {   61,  481,   27888, 13, }, // America/Indiana/Knox
    {   61, 7961,   27901, 16, }, // America/Indiana/Marengo
    {   61, 7985,   27917, 19, }, // America/Indiana/Petersburg
    {   61, 8012,   27936, 17, }, // America/Indiana/Tell_City
    {   61, 8038,   27953, 13, }, // America/Indiana/Vevay
    {   61, 8060,   27966, 16, }, // America/Indiana/Vincennes
    {   61, 8086,   27982, 16, }, // America/Indiana/Winamac
    {   61, 8110,   27998,  6, }, // America/Inuvik
    {   61,  660,   28004,  7, }, // America/Iqaluit
    {   61, 2799,   28011,  6, }, // America/Jamaica
    {   61, 5380,   28017,  5, }, // America/Juneau
    {   61,  521,   28022,  7, }, // America/Kentucky/Louisville
    {   61, 8125,   28029, 18, }, // America/Kentucky/Monticello
    {   61, 5704,   28047, 10, }, // America/Kralendijk
    {   61, 4376,    1632,  6, }, // America/La_Paz
    {   61, 7169,   28057,  4, }, // America/Lima
    {   61, 3239,   28061, 12, }, // America/Los_Angeles
    {   61, 5932,   28073, 20, }, // America/Lower_Princes
    {   61, 8153,   28093,  5, }, // America/Maceio
    {   61, 8168,    1677,  7, }, // America/Managua
    {   61, 1908,   28098,  6, }, // America/Manaus
    {   61, 5897,   28104,  6, }, // America/Marigot
    {   61, 5805,   28110,  9, }, // America/Martinique
    {   61, 8184,   28119,  9, }, // America/Matamoros
    {   61, 2917,    1714,  8, }, // America/Mazatlan
    {   61, 8202,   28128,  8, }, // America/Menominee
    {   61, 8220,   28136,  6, }, // America/Merida
    {   61, 8235,    1736, 10, }, // America/Metlakatla
    {   61, 2949,   28142,  6, }, // America/Mexico_City
    {   61, 4391,   28148,  7, }, // America/Miquelon
    {   61, 8254,   28155,  7, }, // America/Moncton
    {   61, 8270,   28162,  8, }, // America/Monterrey
    {   61, 4098,   28170, 10, }, // America/Montevideo
    {   61, 5824,   28180,  9, }, // America/Montserrat
    {   61, 5481,   28189,  5, }, // America/Nassau
    {   61, 2379,   28194,  7, }, // America/New_York
    {   61, 8288,    1809,  3, }, // America/Nome
    {   61, 1850,   28201,  7, }, // America/Noronha
    {   61, 8301,   28208, 23, }, // America/North_Dakota/Beulah
    {   61, 8329,   28231, 24, }, // America/North_Dakota/Center
    {   61, 8357,   28255, 28, }, // America/North_Dakota/New_Salem
    {   61,  393,   28283,  3, }, // America/Nuuk
    {   61, 8388,   28286,  7, }, // America/Ojinaga
    {   61, 2356,    1901,  6, }, // America/Panama
    {   61, 6173,   28293, 10, }, // America/Paramaribo
    {   61, 2973,   28303,  6, }, // America/Phoenix
    {   61, 3945,   28309, 12, }, // America/Port-au-Prince
    {   61, 5954,   28321, 13, }, // America/Port_of_Spain
    {   61, 8404,   28334, 11, }, // America/Porto_Velho
    {   61, 5843,   28345, 11, }, // America/Puerto_Rico
    {   61, 4030,   28356, 12, }, // America/Punta_Arenas
    {   61, 8424,   28368, 12, }, // America/Rankin_Inlet
    {   61, 8445,   28380,  6, }, // America/Recife
    {   61, 1995,   28386,  8, }, // America/Regina
    {   61, 8460,   28394,  8, }, // America/Resolute
    {   61,  695,   28402, 10, }, // America/Rio_Branco
    {   61, 8477,   28412,  8, }, // America/Santarem
    {   61, 2201,   28420,  8, }, // America/Santiago
    {   61, 6111,   28428, 13, }, // America/Santo_Domingo
    {   61, 1878,   28441,  9, }, // America/Sao_Paulo
    {   61, 6899,   28450, 13, }, // America/Scoresbysund
    {   61, 8494,   28463,  5, }, // America/Sitka
    {   61, 8508,   28468, 13, }, // America/St_Barthelemy
    {   61, 2061,   28481, 10, }, // America/St_Johns
    {   61, 5863,   28491,  9, }, // America/St_Kitts
    {   61, 5880,   28500, 10, }, // America/St_Lucia
    {   61,  855,    2121, 10, }, // America/St_Thomas
    {   61, 5913,   28510, 12, }, // America/St_Vincent
    {   61, 8530,   28522, 12, }, // America/Swift_Current
    {   61, 5461,   28534, 12, }, // America/Tegucigalpa
    {   61, 5756,    2167,  4, }, // America/Thule
    {   61,  313,   28546,  7, }, // America/Tijuana
    {   61,  608,   28553,  7, }, // America/Toronto
    {   61, 5688,   28560,  7, }, // America/Tortola
    {   61, 2093,    2192,  8, }, // America/Vancouver
    {   61, 2144,   28567,  8, }, // America/Whitehorse
    {   61,  734,   28575,  7, }, // America/Winnipeg
    {   61, 8552,   28582,  6, }, // America/Yakutat
    {   61, 6001,   28588,  5, }, // Antarctica/Casey
    {   61, 6094,   28593,  6, }, // Antarctica/Davis
    {   61, 6133,   28599, 15, }, // Antarctica/DumontDUrville
    {   61, 8568,   28614,  7, }, // Antarctica/Macquarie
    {   61, 7066,   28621,  6, }, // Antarctica/Mawson
    {   61, 7113,   28627,  9, }, // Antarctica/McMurdo
    {   61, 8589,    2272,  6, }, // Antarctica/Palmer
    {   61, 7226,   28636,  6, }, // Antarctica/Rothera
    {   61, 7295,   28642,  4, }, // Antarctica/Syowa
    {   61, 8607,    2289,  4, }, // Antarctica/Troll
    {   61, 7374,   28646,  6, }, // Antarctica/Vostok
    {   61, 1429,   28652,  7, }, // Arctic/Longyearbyen
    {   61, 5570,   28659,  4, }, // Asia/Aden
    {   61, 5395,    2314,  6, }, // Asia/Almaty
    {   61, 3968,   28663, 12, }, // Asia/Amman
    {   61, 5511,    2325,  6, }, // Asia/Anadyr
    {   61, 5523,    2331,  5, }, // Asia/Aqtau
    {   61, 5368,   28675,  6, }, // Asia/Aqtobe
    {   61,  964,   28681,  7, }, // Asia/Ashgabat
    {   61, 8624,    2349,  6, }, // Asia/Atyrau
    {   61, 3607,    2355,  6, }, // Asia/Baghdad
    {   61, 5534,   28688,  7, }, // Asia/Bahrain
    {   61, 3653,    2368,  4, }, // Asia/Baku
    {   61, 4466,    2372,  7, }, // Asia/Bangkok
    {   61, 3571,    2379,  7, }, // Asia/Barnaul
    {   61, 4086,   28695,  6, }, // Asia/Beirut
    {   61, 3778,   28701,  6, }, // Asia/Bishkek
    {   61, 5989,   28707,  6, }, // Asia/Brunei
    {   61, 4601,   28713,  4, }, // Asia/Chita
    {   61, 4511,   28717,  7, }, // Asia/Colombo
    {   61, 4540,    2415,  6, }, // Asia/Damascus
    {   61, 1093,     711,  4, }, // Asia/Dhaka
    {   61, 6192,   28724,  4, }, // Asia/Dili
    {   61, 3596,   28728,  5, }, // Asia/Dubai
    {   61, 6159,   28733,  7, }, // Asia/Dushanbe
    {   61, 8636,    2442,  9, }, // Asia/Famagusta
    {   61, 8651,    2451,  4, }, // Asia/Gaza
    {   61, 4795,   28740,  6, }, // Asia/Hebron
    {   61, 1259,   28746,  7, }, // Asia/Ho_Chi_Minh
    {   61, 2704,   28753,  7, }, // Asia/Hong_Kong
    {   61, 4771,   28760,  4, }, // Asia/Hovd
    {   61, 4184,   28764,  6, }, // Asia/Irkutsk
    {   61, 7006,   28770,  8, }, // Asia/Jakarta
    {   61, 6992,   28778,  8, }, // Asia/Jayapura
    {   61, 1290,   28786,  9, }, // Asia/Jerusalem
    {   61, 3560,    2510,  5, }, // Asia/Kabul
    {   61, 4316,    2529,  8, }, // Asia/Kamchatka
    {   61, 4239,   28795,  6, }, // Asia/Karachi
    {   61, 1185,    2546,  8, }, // Asia/Kathmandu
    {   61, 8661,    2554,  7, }, // Asia/Khandyga
    {   61,  992,   28801,  8, }, // Asia/Kolkata
    {   61, 4197,   28809, 10, }, // Asia/Krasnoyarsk
    {   61, 4859,    2580, 12, }, // Asia/Kuala_Lumpur
    {   61, 5976,   28819,  6, }, // Asia/Kuching
    {   61, 5547,   28825,  6, }, // Asia/Kuwait
    {   61, 1211,   28831,  5, }, // Asia/Macau
    {   61, 4017,    2609,  7, }, // Asia/Magadan
    {   61, 1349,    2616,  7, }, // Asia/Makassar
    {   61, 7182,   28836,  6, }, // Asia/Manila
    {   61, 6920,    2629,  6, }, // Asia/Muscat
    {   61, 2554,   28842,  7, }, // Asia/Nicosia
    {   61, 8675,   28849, 11, }, // Asia/Novokuznetsk
    {   61, 4135,   28860, 11, }, // Asia/Novosibirsk
    {   61, 4229,    2664,  4, }, // Asia/Omsk
    {   61, 7145,   28871,  7, }, // Asia/Oral
    {   61, 6961,   28878,  8, }, // Asia/Phnom_Penh
    {   61, 8693,   28886,  9, }, // Asia/Pontianak
    {   61, 4214,   28895,  7, }, // Asia/Pyongyang
    {   61, 5559,    2697,  5, }, // Asia/Qatar
    {   61, 8708,   28902,  8, }, // Asia/Qostanay
    {   61, 4269,   28910,  9, }, // Asia/Qyzylorda
    {   61, 3584,   28919,  7, }, // Asia/Riyadh
    {   61, 4408,   28926,  7, }, // Asia/Sakhalin
    {   61, 7245,    2739,  9, }, // Asia/Samarkand
    {   61, 3279,    2748,  4, }, // Asia/Seoul
    {   61, 1053,   28933,  6, }, // Asia/Shanghai
    {   61, 3300,   28939,  8, }, // Asia/Singapore
    {   61, 4297,   28947, 14, }, // Asia/Srednekolymsk
    {   61, 3263,   28961,  6, }, // Asia/Taipei
    {   61, 4781,    2785,  7, }, // Asia/Tashkent
    {   61, 3915,   28967,  7, }, // Asia/Tbilisi
    {   61, 2772,   28974,  7, }, // Asia/Tehran
    {   61, 1317,   28981,  7, }, // Asia/Thimphu
    {   61, 2821,   28988,  5, }, // Asia/Tokyo
    {   61, 4572,    2815,  5, }, // Asia/Tomsk
    {   61, 1021,   28993, 10, }, // Asia/Ulaanbaatar
    {   61, 1159,   29003,  6, }, // Asia/Urumqi
    {   61, 8722,   29009,  9, }, // Asia/Ust-Nera
    {   61, 6977,   29018,  8, }, // Asia/Vientiane
    {   61, 4710,   29026, 12, }, // Asia/Vladivostok
    {   61, 4828,   29038,  5, }, // Asia/Yakutsk
    {   61, 1235,   29043,  6, }, // Asia/Yangon
    {   61, 3883,   29049, 12, }, // Asia/Yekaterinburg
    {   61, 3747,   29061,  6, }, // Asia/Yerevan
    {   61, 3663,   29067, 16, }, // Atlantic/Azores
    {   61, 5671,   29083, 18, }, // Atlantic/Bermuda
    {   61, 6628,   29101, 17, }, // Atlantic/Canary
    {   61, 3727,   29118, 10, }, // Atlantic/Cape_Verde
    {   61, 1395,   29128, 17, }, // Atlantic/Faroe
    {   61, 8736,   29145,  7, }, // Atlantic/Madeira
    {   61, 2748,   29152,  9, }, // Atlantic/Reykjavik
    {   61, 7272,   29161, 17, }, // Atlantic/South_Georgia
    {   61, 6821,   29178, 20, }, // Atlantic/St_Helena
    {   61, 6644,   29198, 11, }, // Atlantic/Stanley
    {   61, 1670,   29209,  8, }, // Australia/Adelaide
    {   61, 1635,   29217,  7, }, // Australia/Brisbane
    {   61, 1799,   29224, 10, }, // Australia/Broken_Hill
    {   61, 1583,   29234,  6, }, // Australia/Darwin
    {   61, 3637,   29240,  4, }, // Australia/Eucla
    {   61, 1516,    3072,  6, }, // Australia/Hobart
    {   61, 8753,   29244,  8, }, // Australia/Lindeman
    {   61, 1547,    3086,  8, }, // Australia/Lord_Howe
    {   61, 1727,   29252,  8, }, // Australia/Melbourne
    {   61, 1762,    3101,  4, }, // Australia/Perth
    {   61, 1463,   29260,  6, }, // Australia/Sydney
    {   61, 6441,   29266,  9, }, // Europe/Amsterdam
    {   61, 6216,   29275,  6, }, // Europe/Andorra
    {   61, 3620,   29281,  9, }, // Europe/Astrakhan
    {   61, 2300,   29290,  5, }, // Europe/Athens
    {   61, 6336,   29295,  7, }, // Europe/Belgrade
    {   61, 4757,   29302,  6, }, // Europe/Berlin
    {   61, 6488,   29308, 10, }, // Europe/Bratislava
    {   61, 2167,   29318,  7, }, // Europe/Brussels
    {   61, 3928,   29325,  8, }, // Europe/Bucharest
    {   61, 3806,    3178,  8, }, // Europe/Budapest
    {   61, 8772,   29333,  8, }, // Europe/Busingen
    {   61, 2583,   29341,  7, }, // Europe/Chisinau
    {   61, 6289,   29348, 10, }, // Europe/Copenhagen
    {   61, 2338,   29358,  6, }, // Europe/Dublin
    {   61, 6307,   29364,  9, }, // Europe/Gibraltar
    {   61, 8788,   29373,  6, }, // Europe/Guernsey
    {   61, 6612,   29379,  9, }, // Europe/Helsinki
    {   61, 8804,   29388, 11, }, // Europe/Isle_of_Man
    {   61, 1130,    3264,  7, }, // Europe/Istanbul
    {   61, 8823,   29399,  6, }, // Europe/Jersey
    {   61, 3979,   29405, 11, }, // Europe/Kaliningrad
    {   61, 8929,   29416,  5, }, // Europe/Kirov
    {   61, 2527,   29421,  4, }, // Europe/Kyiv
    {   61, 3213,   29425,  7, }, // Europe/Lisbon
    {   61, 6506,   29432,  7, }, // Europe/Ljubljana
    {   61, 2501,   29439,  6, }, // Europe/London
    {   61, 6365,   29445, 10, }, // Europe/Luxembourg
    {   61, 4892,   29455,  6, }, // Europe/Madrid
    {   61, 6397,   29461,  6, }, // Europe/Malta
    {   61, 6582,   29467, 12, }, // Europe/Mariehamn
    {   61, 3693,   29479,  5, }, // Europe/Minsk
    {   61, 6410,   29484,  6, }, // Europe/Monaco
    {   61, 3537,   29490,  6, }, // Europe/Moscow
    {   61, 6458,   29496,  4, }, // Europe/Oslo
    {   61, 4284,   29500,  5, }, // Europe/Paris
    {   61, 6424,   29505,  9, }, // Europe/Podgorica
    {   61, 6275,    3385,  5, }, // Europe/Prague
    {   61, 8837,   29514,  4, }, // Europe/Riga
    {   61, 6324,   29518,  3, }, // Europe/Rome
    {   61, 4331,    3397,  6, }, // Europe/Samara
    {   61, 6470,   29521, 10, }, // Europe/San_Marino
    {   61, 6245,   29531,  7, }, // Europe/Sarajevo
    {   61, 4451,   29538,  7, }, // Europe/Saratov
    {   61, 8849,   29545, 11, }, // Europe/Simferopol
    {   61, 6383,   29556,  6, }, // Europe/Skopje
    {   61, 6599,   29562,  5, }, // Europe/Sofia
    {   61, 6523,   29567,  9, }, // Europe/Stockholm
    {   61, 8867,   29576,  5, }, // Europe/Tallinn
    {   61, 6202,   29581,  6, }, // Europe/Tirane
    {   61, 8882,   29587,  9, }, // Europe/Ulyanovsk
    {   61, 6352,    3476,  5, }, // Europe/Vaduz
    {   61, 6567,   29596,  7, }, // Europe/Vatican
    {   61, 6231,    3488,  4, }, // Europe/Vienna
    {   61, 8899,   29603,  7, }, // Europe/Vilnius
    {   61, 4727,   29610,  9, }, // Europe/Volgograd
    {   61, 3190,    3508,  7, }, // Europe/Warsaw
    {   61, 6261,   29619,  6, }, // Europe/Zagreb
    {   61, 6540,   29625,  5, }, // Europe/Zurich
    {   61, 5078,   29630, 12, }, // Indian/Antananarivo
    {   61, 6947,   29642,  5, }, // Indian/Chagos
    {   61, 6046,   29647, 13, }, // Indian/Christmas
    {   61, 6063,   29660, 17, }, // Indian/Cocos
    {   61, 5029,   29677, 17, }, // Indian/Comoro
    {   61, 6661,    3578,  8, }, // Indian/Kerguelen
    {   61, 7260,   29694,  3, }, // Indian/Mahe
    {   61, 7050,   29697,  8, }, // Indian/Maldives
    {   61, 4069,   29705,  8, }, // Indian/Mauritius
    {   61, 5098,   29713,  5, }, // Indian/Mayotte
    {   61, 7211,   29718,  7, }, // Indian/Reunion
    {   61, 4422,   29725,  4, }, // Pacific/Apia
    {   61,  932,    3620,  6, }, // Pacific/Auckland
    {   61, 3706,   29729,  9, }, // Pacific/Bougainville
    {   61, 3015,   29738,  5, }, // Pacific/Chatham
    {   61, 3157,    3639,  4, }, // Pacific/Chuuk
    {   61, 2237,   29743, 17, }, // Pacific/Easter
    {   61, 7360,   29760,  5, }, // Pacific/Efate
    {   61, 7327,   29765,  7, }, // Pacific/Fakaofo
    {   61, 3902,   29772,  5, }, // Pacific/Fiji
    {   61, 7343,   29777,  8, }, // Pacific/Funafuti
    {   61, 6678,   29785,  9, }, // Pacific/Galapagos
    {   61, 6696,   29794, 14, }, // Pacific/Gambier
    {   61, 3822,    3713, 11, }, // Pacific/Guadalcanal
    {   61, 6018,    3724,  4, }, // Pacific/Guam
    {   61, 2723,   29808,  8, }, // Pacific/Honolulu
    {   61, 3049,   29816,  9, }, // Pacific/Kanton
    {   61, 3998,   29825, 10, }, // Pacific/Kiritimati
    {   61, 7019,   29835,  6, }, // Pacific/Kosrae
    {   61, 2842,   29841, 10, }, // Pacific/Kwajalein
    {   61, 4877,   29851,  7, }, // Pacific/Majuro
    {   61, 4051,   29858, 18, }, // Pacific/Marquesas
    {   61, 8914,   29876,  6, }, // Pacific/Midway
    {   61, 7084,    3798,  5, }, // Pacific/Nauru
    {   61, 7132,   29882,  4, }, // Pacific/Niue
    {   61, 4168,   29886,  7, }, // Pacific/Norfolk
    {   61, 7098,    3814,  5, }, // Pacific/Noumea
    {   61, 3126,   29893,  9, }, // Pacific/Pago_Pago
    {   61, 7155,    3828,  5, }, // Pacific/Palau
    {   61, 7194,   29902,  7, }, // Pacific/Pitcairn
    {   61, 3096,   29909,  6, }, // Pacific/Pohnpei
    {   61, 4807,   29915, 11, }, // Pacific/Port_Moresby
    {   61, 6076,   29926,  9, }, // Pacific/Rarotonga
    {   61, 6031,   29935,  6, }, // Pacific/Saipan
    {   61, 7312,   29941,  5, }, // Pacific/Tahiti
    {   61, 6712,    3877,  6, }, // Pacific/Tarawa
    {   61, 4583,   29946,  9, }, // Pacific/Tongatapu
    {   61, 7392,   29955,  4, }, // Pacific/Wake
    {   61, 7405,   29959,  5, }, // Pacific/Wallis
    {   62,   14,       0,  6, }, // Africa/Asmara Bemba/Latin/Zambia
    {   62, 4435,       6,  8, }, // Africa/Sao_Tome
    {   62, 4252,      14,  8, }, // America/Asuncion
    {   62,  237,      22,  8, }, // America/Atikokan
    {   62, 7623,      30, 17, }, // America/Bahia_Banderas
    {   62, 3868,      47,  6, }, // America/Cancun
    {   62, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   62, 5723,      66,  7, }, // America/Curacao
    {   62,  481,      73, 13, }, // America/Indiana/Knox
    {   62, 7961,      86, 16, }, // America/Indiana/Marengo
    {   62, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   62, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   62, 8038,     139, 14, }, // America/Indiana/Vevay
    {   62, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   62, 8086,     171, 16, }, // America/Indiana/Winamac
    {   62, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   62, 5932,     207, 22, }, // America/Lower_Princes
    {   62, 8220,     229,  6, }, // America/Merida
    {   62, 1850,     235, 19, }, // America/Noronha
    {   62, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {   62, 8329,     274, 20, }, // America/North_Dakota/Center
    {   62, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {   62,  393,     317,  4, }, // America/Nuuk
    {   62, 6899,     321, 16, }, // America/Scoresbysund
    {   62, 8508,     337, 14, }, // America/St_Barthelemy
    {   62, 2061,     351, 10, }, // America/St_Johns
    {   62, 5863,     361,  9, }, // America/St_Kitts
    {   62, 5880,     370,  9, }, // America/St_Lucia
    {   62,  855,     379, 10, }, // America/St_Thomas
    {   62, 5913,     389, 11, }, // America/St_Vincent
    {   62, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {   62, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {   62, 1185,     448,  9, }, // Asia/Kathmandu
    {   62,  992,     457,  7, }, // Asia/Kolkata
    {   62, 1211,     464,  5, }, // Asia/Macau
    {   62, 1235,     477,  6, }, // Asia/Yangon
    {   62, 1395,     483,  5, }, // Atlantic/Faroe
    {   62, 6821,     488, 10, }, // Atlantic/St_Helena
    {   62, 2527,     514,  4, }, // Europe/Kyiv
    {   62, 7211,     547,  7, }, // Indian/Reunion
    {   62, 3157,     554,  5, }, // Pacific/Chuuk
    {   62, 3096,     586,  7, }, // Pacific/Pohnpei
    {   63,   14,       0,  6, }, // Africa/Asmara Bena/Latin/Tanzania
    {   63, 4435,       6,  8, }, // Africa/Sao_Tome
    {   63, 4252,      14,  8, }, // America/Asuncion
    {   63,  237,      22,  8, }, // America/Atikokan
    {   63, 7623,      30, 17, }, // America/Bahia_Banderas
    {   63, 3868,      47,  6, }, // America/Cancun
    {   63, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   63, 5723,      66,  7, }, // America/Curacao
    {   63,  481,      73, 13, }, // America/Indiana/Knox
    {   63, 7961,      86, 16, }, // America/Indiana/Marengo
    {   63, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   63, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   63, 8038,     139, 14, }, // America/Indiana/Vevay
    {   63, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   63, 8086,     171, 16, }, // America/Indiana/Winamac
    {   63, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   63, 5932,     207, 22, }, // America/Lower_Princes
    {   63, 8220,     229,  6, }, // America/Merida
    {   63, 1850,     235, 19, }, // America/Noronha
    {   63, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {   63, 8329,     274, 20, }, // America/North_Dakota/Center
    {   63, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {   63,  393,     317,  4, }, // America/Nuuk
    {   63, 6899,     321, 16, }, // America/Scoresbysund
    {   63, 8508,     337, 14, }, // America/St_Barthelemy
    {   63, 2061,     351, 10, }, // America/St_Johns
    {   63, 5863,     361,  9, }, // America/St_Kitts
    {   63, 5880,     370,  9, }, // America/St_Lucia
    {   63,  855,     379, 10, }, // America/St_Thomas
    {   63, 5913,     389, 11, }, // America/St_Vincent
    {   63, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {   63, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {   63, 1185,     448,  9, }, // Asia/Kathmandu
    {   63,  992,     457,  7, }, // Asia/Kolkata
    {   63, 1211,     464,  5, }, // Asia/Macau
    {   63, 1235,     477,  6, }, // Asia/Yangon
    {   63, 1395,     483,  5, }, // Atlantic/Faroe
    {   63, 6821,     488, 10, }, // Atlantic/St_Helena
    {   63, 2527,     514,  4, }, // Europe/Kyiv
    {   63, 7211,     547,  7, }, // Indian/Reunion
    {   63, 3157,     554,  5, }, // Pacific/Chuuk
    {   63, 3096,     586,  7, }, // Pacific/Pohnpei
    {   64,   14,       0,  6, }, // Africa/Asmara Bhojpuri/Devanagari/India
    {   64, 4435,       6,  8, }, // Africa/Sao_Tome
    {   64, 4252,      14,  8, }, // America/Asuncion
    {   64,  237,      22,  8, }, // America/Atikokan
    {   64, 7623,      30, 17, }, // America/Bahia_Banderas
    {   64, 3868,      47,  6, }, // America/Cancun
    {   64, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   64, 5723,      66,  7, }, // America/Curacao
    {   64,  481,      73, 13, }, // America/Indiana/Knox
    {   64, 7961,      86, 16, }, // America/Indiana/Marengo
    {   64, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   64, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   64, 8038,     139, 14, }, // America/Indiana/Vevay
    {   64, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   64, 8086,     171, 16, }, // America/Indiana/Winamac
    {   64, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   64, 5932,     207, 22, }, // America/Lower_Princes
    {   64, 8220,     229,  6, }, // America/Merida
    {   64, 1850,     235, 19, }, // America/Noronha
    {   64, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {   64, 8329,     274, 20, }, // America/North_Dakota/Center
    {   64, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {   64,  393,     317,  4, }, // America/Nuuk
    {   64, 6899,     321, 16, }, // America/Scoresbysund
    {   64, 8508,     337, 14, }, // America/St_Barthelemy
    {   64, 2061,     351, 10, }, // America/St_Johns
    {   64, 5863,     361,  9, }, // America/St_Kitts
    {   64, 5880,     370,  9, }, // America/St_Lucia
    {   64,  855,     379, 10, }, // America/St_Thomas
    {   64, 5913,     389, 11, }, // America/St_Vincent
    {   64, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {   64, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {   64, 1185,     448,  9, }, // Asia/Kathmandu
    {   64,  992,     457,  7, }, // Asia/Kolkata
    {   64, 1211,     464,  5, }, // Asia/Macau
    {   64, 1235,     477,  6, }, // Asia/Yangon
    {   64, 1395,     483,  5, }, // Atlantic/Faroe
    {   64, 6821,     488, 10, }, // Atlantic/St_Helena
    {   64, 2527,     514,  4, }, // Europe/Kyiv
    {   64, 7211,     547,  7, }, // Indian/Reunion
    {   64, 3157,     554,  5, }, // Pacific/Chuuk
    {   64, 3096,     586,  7, }, // Pacific/Pohnpei
    {   65,   14,       0,  6, }, // Africa/Asmara Blin/Ethiopic/Eritrea
    {   65, 4435,       6,  8, }, // Africa/Sao_Tome
    {   65, 4252,      14,  8, }, // America/Asuncion
    {   65,  237,      22,  8, }, // America/Atikokan
    {   65, 7623,      30, 17, }, // America/Bahia_Banderas
    {   65, 3868,      47,  6, }, // America/Cancun
    {   65, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   65, 5723,      66,  7, }, // America/Curacao
    {   65,  481,      73, 13, }, // America/Indiana/Knox
    {   65, 7961,      86, 16, }, // America/Indiana/Marengo
    {   65, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   65, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   65, 8038,     139, 14, }, // America/Indiana/Vevay
    {   65, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   65, 8086,     171, 16, }, // America/Indiana/Winamac
    {   65, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   65, 5932,     207, 22, }, // America/Lower_Princes
    {   65, 8220,     229,  6, }, // America/Merida
    {   65, 1850,     235, 19, }, // America/Noronha
    {   65, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {   65, 8329,     274, 20, }, // America/North_Dakota/Center
    {   65, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {   65,  393,     317,  4, }, // America/Nuuk
    {   65, 6899,     321, 16, }, // America/Scoresbysund
    {   65, 8508,     337, 14, }, // America/St_Barthelemy
    {   65, 2061,     351, 10, }, // America/St_Johns
    {   65, 5863,     361,  9, }, // America/St_Kitts
    {   65, 5880,     370,  9, }, // America/St_Lucia
    {   65,  855,     379, 10, }, // America/St_Thomas
    {   65, 5913,     389, 11, }, // America/St_Vincent
    {   65, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {   65, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {   65, 1185,     448,  9, }, // Asia/Kathmandu
    {   65,  992,     457,  7, }, // Asia/Kolkata
    {   65, 1211,     464,  5, }, // Asia/Macau
    {   65, 1235,     477,  6, }, // Asia/Yangon
    {   65, 1395,     483,  5, }, // Atlantic/Faroe
    {   65, 6821,     488, 10, }, // Atlantic/St_Helena
    {   65, 2527,     514,  4, }, // Europe/Kyiv
    {   65, 7211,     547,  7, }, // Indian/Reunion
    {   65, 3157,     554,  5, }, // Pacific/Chuuk
    {   65, 3096,     586,  7, }, // Pacific/Pohnpei
    {   66, 6788,   29964,  7, }, // Africa/Abidjan Bodo/Devanagari/India
    {   66, 6760,   29971,  5, }, // Africa/Accra
    {   66, 5059,   29976, 12, }, // Africa/Addis_Ababa
    {   66, 7420,   29988,  9, }, // Africa/Algiers
    {   66,   14,   29997,  7, }, // Africa/Asmara
    {   66,   44,   30004,  6, }, // Africa/Bamako
    {   66, 5257,   30010,  6, }, // Africa/Bangui
    {   66, 6746,   30016,  6, }, // Africa/Banjul
    {   66, 7435,   30022,  7, }, // Africa/Bissau
    {   66, 4957,   30029, 10, }, // Africa/Blantyre
    {   66, 5287,   30039, 11, }, // Africa/Brazzaville
    {   66, 4922,   30050, 10, }, // Africa/Bujumbura
    {   66, 2320,   30060,  5, }, // Africa/Cairo
    {   66, 4117,   30065, 11, }, // Africa/Casablanca
    {   66, 7449,   30076,  5, }, // Africa/Ceuta
    {   66, 6773,   30081,  7, }, // Africa/Conakry
    {   66, 6840,   30088,  5, }, // Africa/Dakar
    {   66, 5130,   30093, 11, }, // Africa/Dar_es_Salaam
    {   66, 5043,   30104,  6, }, // Africa/Djibouti
    {   66, 5243,   30110,  5, }, // Africa/Douala
    {   66, 5166,   30115,  7, }, // Africa/El_Aaiun
    {   66, 6853,   30122,  8, }, // Africa/Freetown
    {   66, 4906,   30130,  7, }, // Africa/Gaborone
    {   66, 5015,   30137,  6, }, // Africa/Harare
    {   66, 4479,   30143, 11, }, // Africa/Johannesburg
    {   66, 4499,   30154,  4, }, // Africa/Juba
    {   66, 5151,   30158,  8, }, // Africa/Kampala
    {   66, 4524,   30166,  7, }, // Africa/Khartoum
    {   66, 4973,   30173,  6, }, // Africa/Kigali
    {   66, 5306,   30179,  8, }, // Africa/Kinshasa
    {   66, 4744,   30187,  5, }, // Africa/Lagos
    {   66, 5336,   30192,  9, }, // Africa/Libreville
    {   66, 6869,   30201,  3, }, // Africa/Lome
    {   66, 5211,   30204,  7, }, // Africa/Luanda
    {   66, 4939,   30211, 10, }, // Africa/Lubumbashi
    {   66, 5001,   30221,  6, }, // Africa/Lusaka
    {   66, 5322,   30227,  6, }, // Africa/Malabo
    {   66, 4987,   30233,  6, }, // Africa/Maputo
    {   66, 5197,   30239,  6, }, // Africa/Maseru
    {   66, 5182,   30245,  8, }, // Africa/Mbabane
    {   66, 5113,   30253,  7, }, // Africa/Mogadishu
    {   66, 7034,   30260,  9, }, // Africa/Monrovia
    {   66, 3853,   30269,  7, }, // Africa/Nairobi
    {   66, 5271,   30276,  8, }, // Africa/Ndjamena
    {   66, 5354,   30284,  5, }, // Africa/Niamey
    {   66, 6803,   30289,  8, }, // Africa/Nouakchott
    {   66, 6727,   30297,  7, }, // Africa/Ouagadougou
    {   66, 5225,   30304,  9, }, // Africa/Porto-Novo
    {   66, 4435,   30313,  7, }, // Africa/Sao_Tome
    {   66, 2866,   30320,  8, }, // Africa/Tripoli
    {   66, 6554,   30328,  7, }, // Africa/Tunis
    {   66, 4152,   30335,  8, }, // Africa/Windhoek
    {   66,  132,   30343,  4, }, // America/Adak
    {   66, 3346,   30347,  6, }, // America/Anchorage
    {   66, 5607,   30353,  8, }, // America/Anguilla
    {   66, 5624,   30361,  9, }, // America/Antigua
    {   66, 4554,   30370,  9, }, // America/Araguaina
    {   66,  166,   30379, 11, }, // America/Argentina/Buenos_Aires
    {   66,   91,   30390, 10, }, // America/Argentina/Catamarca
    {   66,  270,   30400,  6, }, // America/Argentina/Cordoba
    {   66,  441,   30406,  5, }, // America/Argentina/Jujuy
    {   66, 7462,   30411,  8, }, // America/Argentina/La_Rioja
    {   66,  565,   30419,  8, }, // America/Argentina/Mendoza
    {   66, 7489,   30427, 10, }, // America/Argentina/Rio_Gallegos
    {   66, 7520,   30437,  6, }, // America/Argentina/Salta
    {   66, 7544,   30443,  9, }, // America/Argentina/San_Juan
    {   66, 5580,   30452,  8, }, // America/Argentina/San_Luis
    {   66, 7571,   30460,  7, }, // America/Argentina/Tucuman
    {   66, 7597,   30467,  8, }, // America/Argentina/Ushuaia
    {   66, 5640,   30475,  5, }, // America/Aruba
    {   66, 4252,   30480,  8, }, // America/Asuncion
    {   66,  237,   30488,  7, }, // America/Atikokan
    {   66, 3679,   30495,  6, }, // America/Bahia
    {   66, 7623,   30501, 16, }, // America/Bahia_Banderas
    {   66, 5654,   30517,  9, }, // America/Barbados
    {   66, 7646,   30526,  5, }, // America/Belem
    {   66, 5407,   30531,  5, }, // America/Belize
    {   66, 7660,   30536, 14, }, // America/Blanc-Sablon
    {   66, 7681,   30550,  9, }, // America/Boa_Vista
    {   66, 4361,   30559,  5, }, // America/Bogota
    {   66, 7699,   30564,  4, }, // America/Boise
    {   66, 7713,   30568, 15, }, // America/Cambridge_Bay
    {   66, 7735,   30583, 13, }, // America/Campo_Grande
    {   66, 3868,   30596,  6, }, // America/Cancun
    {   66, 4694,   30602,  7, }, // America/Caracas
    {   66, 4345,   30609,  5, }, // America/Cayenne
    {   66, 5496,   30614,  5, }, // America/Cayman
    {   66, 2260,   30619,  6, }, // America/Chicago
    {   66, 7756,   30625,  7, }, // America/Chihuahua
    {   66, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   66, 5422,   30632, 10, }, // America/Costa_Rica
    {   66, 7796,   30642,  8, }, // America/Creston
    {   66, 3791,   30650,  6, }, // America/Cuiaba
    {   66, 5723,   30656,  7, }, // America/Curacao
    {   66, 7812,   30663, 13, }, // America/Danmarkshavn
    {   66, 7833,   30676,  6, }, // America/Dawson
    {   66, 7848,   30682, 12, }, // America/Dawson_Creek
    {   66,  805,   30694,  6, }, // America/Denver
    {   66, 3465,   30700,  8, }, // America/Detroit
    {   66, 5739,   30708,  7, }, // America/Dominica
    {   66,  893,   30715,  7, }, // America/Edmonton
    {   66, 7869,   30722,  8, }, // America/Eirunepe
    {   66, 5441,   30730, 11, }, // America/El_Salvador
    {   66, 7886,   30741, 10, }, // America/Fort_Nelson
    {   66, 7906,   30751,  9, }, // America/Fortaleza
    {   66, 7924,   30760,  9, }, // America/Glace_Bay
    {   66, 6881,   30769,  7, }, // America/Goose_Bay
    {   66, 4612,   30776, 13, }, // America/Grand_Turk
    {   66, 5770,   30789,  8, }, // America/Grenada
    {   66, 5786,   30797,  9, }, // America/Guadeloupe
    {   66, 3760,   30806, 10, }, // America/Guatemala
    {   66, 4841,   30816,  8, }, // America/Guayaquil
    {   66, 6932,   30824,  6, }, // America/Guyana
    {   66, 1939,   30830,  9, }, // America/Halifax
    {   66, 2281,   30839,  6, }, // America/Havana
    {   66, 7942,   30845, 12, }, // America/Hermosillo
    {   66,  348,   30857, 13, }, // America/Indiana/Indianapolis
    {   66,  481,   30870, 15, }, // America/Indiana/Knox
    {   66, 7961,   30885, 17, }, // America/Indiana/Marengo
    {   66, 7985,   30902, 23, }, // America/Indiana/Petersburg
    {   66, 8012,   30925, 20, }, // America/Indiana/Tell_City
    {   66, 8038,   30945, 15, }, // America/Indiana/Vevay
    {   66, 8060,   30960, 21, }, // America/Indiana/Vincennes
    {   66, 8086,   30981, 18, }, // America/Indiana/Winamac
    {   66, 8110,   30999,  6, }, // America/Inuvik
    {   66,  660,   31005,  7, }, // America/Iqaluit
    {   66, 2799,   31012,  7, }, // America/Jamaica
    {   66, 5380,   31019,  4, }, // America/Juneau
    {   66,  521,   31023,  6, }, // America/Kentucky/Louisville
    {   66, 8125,   31029, 19, }, // America/Kentucky/Monticello
    {   66, 5704,   31048, 10, }, // America/Kralendijk
    {   66, 4376,   31058,  6, }, // America/La_Paz
    {   66, 7169,   31064,  4, }, // America/Lima
    {   66, 3239,   31068, 11, }, // America/Los_Angeles
    {   66, 5932,   31079, 24, }, // America/Lower_Princes
    {   66, 8153,   31103,  6, }, // America/Maceio
    {   66, 8168,   31109,  6, }, // America/Managua
    {   66, 1908,   31115,  5, }, // America/Manaus
    {   66, 5897,   31120,  6, }, // America/Marigot
    {   66, 5805,   31126,  9, }, // America/Martinique
    {   66, 8184,   31135,  7, }, // America/Matamoros
    {   66, 2917,   31142,  8, }, // America/Mazatlan
    {   66, 8202,   31150,  8, }, // America/Menominee
    {   66, 8220,   31158,  6, }, // America/Merida
    {   66, 8235,   31164, 12, }, // America/Metlakatla
    {   66, 2949,   31176, 13, }, // America/Mexico_City
    {   66, 4391,   31189,  9, }, // America/Miquelon
    {   66, 8254,   31198,  6, }, // America/Moncton
    {   66, 8270,   31204,  9, }, // America/Monterrey
    {   66, 4098,   31213, 10, }, // America/Montevideo
    {   66, 5824,   31223,  9, }, // America/Montserrat
    {   66, 5481,   31232,  7, }, // America/Nassau
    {   66, 2379,   31239,  8, }, // America/New_York
    {   66, 8288,   31152,  3, }, // America/Nome
    {   66, 1850,   31247,  7, }, // America/Noronha
    {   66, 8301,   31254, 19, }, // America/North_Dakota/Beulah
    {   66, 8329,   31273, 20, }, // America/North_Dakota/Center
    {   66, 8357,   31293, 22, }, // America/North_Dakota/New_Salem
    {   66,  393,   31315,  3, }, // America/Nuuk
    {   66, 8388,   31318,  7, }, // America/Ojinaga
    {   66, 2356,   31325,  6, }, // America/Panama
    {   66, 6173,   31331, 10, }, // America/Paramaribo
    {   66, 2973,   31341,  7, }, // America/Phoenix
    {   66, 3945,   31348, 15, }, // America/Port-au-Prince
    {   66, 5954,   31363, 13, }, // America/Port_of_Spain
    {   66, 8404,   31376, 12, }, // America/Porto_Velho
    {   66, 5843,   31388, 12, }, // America/Puerto_Rico
    {   66, 4030,   31400, 13, }, // America/Punta_Arenas
    {   66, 8424,   31413, 12, }, // America/Rankin_Inlet
    {   66, 8445,   31425,  6, }, // America/Recife
    {   66, 1995,   31431,  6, }, // America/Regina
    {   66, 8460,   31437,  8, }, // America/Resolute
    {   66,  695,   31445, 12, }, // America/Rio_Branco
    {   66, 8477,   31457,  9, }, // America/Santarem
    {   66, 2201,   31466,  9, }, // America/Santiago
    {   66, 6111,   31475, 12, }, // America/Santo_Domingo
    {   66, 1878,   31487,  9, }, // America/Sao_Paulo
    {   66, 6899,   31496, 14, }, // America/Scoresbysund
    {   66, 8494,   31510,  6, }, // America/Sitka
    {   66, 8508,   31516, 16, }, // America/St_Barthelemy
    {   66, 2061,   31532, 10, }, // America/St_Johns
    {   66, 5863,   31542, 11, }, // America/St_Kitts
    {   66, 5880,   31553, 12, }, // America/St_Lucia
    {   66,  855,   31565, 10, }, // America/St_Thomas
    {   66, 5913,   31575, 15, }, // America/St_Vincent
    {   66, 8530,   31590, 15, }, // America/Swift_Current
    {   66, 5461,   31605, 12, }, // America/Tegucigalpa
    {   66, 5756,   31617,  5, }, // America/Thule
    {   66,  313,   31622,  7, }, // America/Tijuana
    {   66,  608,   31629,  6, }, // America/Toronto
    {   66, 5688,   31635,  7, }, // America/Tortola
    {   66, 2093,   31642,  7, }, // America/Vancouver
    {   66, 2144,   31649, 10, }, // America/Whitehorse
    {   66,  734,   31659,  7, }, // America/Winnipeg
    {   66, 8552,   31666,  7, }, // America/Yakutat
    {   66, 6001,   31673,  4, }, // Antarctica/Casey
    {   66, 6094,   31677,  5, }, // Antarctica/Davis
    {   66, 6133,   31682, 15, }, // Antarctica/DumontDUrville
    {   66, 8568,   31697,  8, }, // Antarctica/Macquarie
    {   66, 7066,   31705,  4, }, // Antarctica/Mawson
    {   66, 7113,   31709,  9, }, // Antarctica/McMurdo
    {   66, 8589,   31718,  6, }, // Antarctica/Palmer
    {   66, 7226,   31724,  5, }, // Antarctica/Rothera
    {   66, 7295,   31729,  5, }, // Antarctica/Syowa
    {   66, 8607,   31734,  5, }, // Antarctica/Troll
    {   66, 7374,   31739,  6, }, // Antarctica/Vostok
    {   66, 1429,   31745, 10, }, // Arctic/Longyearbyen
    {   66, 5570,   31755,  4, }, // Asia/Aden
    {   66, 5395,   31759,  7, }, // Asia/Almaty
    {   66, 3968,   31766,  6, }, // Asia/Amman
    {   66, 5511,   31772,  6, }, // Asia/Anadyr
    {   66, 5523,   31778,  6, }, // Asia/Aqtau
    {   66, 5368,   31784,  7, }, // Asia/Aqtobe
    {   66,  964,   31791,  8, }, // Asia/Ashgabat
    {   66, 8624,   31799,  6, }, // Asia/Atyrau
    {   66, 3607,   31805,  6, }, // Asia/Baghdad
    {   66, 5534,   31811,  6, }, // Asia/Bahrain
    {   66, 3653,   30654,  4, }, // Asia/Baku
    {   66, 4466,   31817,  7, }, // Asia/Bangkok
    {   66, 3571,   31824,  7, }, // Asia/Barnaul
    {   66, 4086,   31831,  5, }, // Asia/Beirut
    {   66, 3778,   31836,  7, }, // Asia/Bishkek
    {   66, 5989,   31843,  6, }, // Asia/Brunei
    {   66, 4601,   31849,  4, }, // Asia/Chita
    {   66, 4511,   31853,  6, }, // Asia/Colombo
    {   66, 4540,   31859,  9, }, // Asia/Damascus
    {   66, 1093,   31868,  4, }, // Asia/Dhaka
    {   66, 6192,   31872,  4, }, // Asia/Dili
    {   66, 3596,   31876,  5, }, // Asia/Dubai
    {   66, 6159,   31881,  8, }, // Asia/Dushanbe
    {   66, 8636,   31889, 10, }, // Asia/Famagusta
    {   66, 8651,   31899,  4, }, // Asia/Gaza
    {   66, 4795,   31903,  7, }, // Asia/Hebron
    {   66, 1259,   31910,  6, }, // Asia/Ho_Chi_Minh
    {   66, 2704,   31916,  4, }, // Asia/Hong_Kong
    {   66, 4771,   31920,  5, }, // Asia/Hovd
    {   66, 4184,   31925, 10, }, // Asia/Irkutsk
    {   66, 7006,   31935,  8, }, // Asia/Jakarta
    {   66, 6992,   31943,  7, }, // Asia/Jayapura
    {   66, 1290,   31950,  9, }, // Asia/Jerusalem
    {   66, 3560,   31959,  5, }, // Asia/Kabul
    {   66, 4316,   31964,  8, }, // Asia/Kamchatka
    {   66, 4239,   31972,  5, }, // Asia/Karachi
    {   66, 1185,   31977,  9, }, // Asia/Kathmandu
    {   66, 8661,   31986,  8, }, // Asia/Khandyga
    {   66,  992,   31994,  6, }, // Asia/Kolkata
    {   66, 4197,   32000, 15, }, // Asia/Krasnoyarsk
    {   66, 4859,   32015, 13, }, // Asia/Kuala_Lumpur
    {   66, 5976,   32028,  6, }, // Asia/Kuching
    {   66, 5547,   32034,  5, }, // Asia/Kuwait
    {   66, 1211,   32039,  5, }, // Asia/Macau
    {   66, 4017,   32044,  7, }, // Asia/Magadan
    {   66, 1349,   32051,  9, }, // Asia/Makassar
    {   66, 7182,   32060,  5, }, // Asia/Manila
    {   66, 6920,   32065,  7, }, // Asia/Muscat
    {   66, 2554,   32072,  8, }, // Asia/Nicosia
    {   66, 8675,   32080, 14, }, // Asia/Novokuznetsk
    {   66, 4135,   32094, 12, }, // Asia/Novosibirsk
    {   66, 4229,   32106,  6, }, // Asia/Omsk
    {   66, 7145,   32112,  4, }, // Asia/Oral
    {   66, 6961,   32116,  7, }, // Asia/Phnom_Penh
    {   66, 8693,   32123,  9, }, // Asia/Pontianak
    {   66, 4214,   32132,  7, }, // Asia/Pyongyang
    {   66, 5559,   32139,  5, }, // Asia/Qatar
    {   66, 8708,   32144,  7, }, // Asia/Qostanay
    {   66, 4269,   32151,  9, }, // Asia/Qyzylorda
    {   66, 3584,   32160,  5, }, // Asia/Riyadh
    {   66, 4408,   32165,  7, }, // Asia/Sakhalin
    {   66, 7245,   32172,  8, }, // Asia/Samarkand
    {   66, 3279,   32180,  4, }, // Asia/Seoul
    {   66, 1053,   32184,  6, }, // Asia/Shanghai
    {   66, 3300,   32190,  8, }, // Asia/Singapore
    {   66, 4297,   32198, 16, }, // Asia/Srednekolymsk
    {   66, 3263,   32214,  6, }, // Asia/Taipei
    {   66, 4781,   32220,  8, }, // Asia/Tashkent
    {   66, 3915,   32228,  8, }, // Asia/Tbilisi
    {   66, 2772,   32236,  6, }, // Asia/Tehran
    {   66, 1317,   32242,  6, }, // Asia/Thimphu
    {   66, 2821,   32248,  6, }, // Asia/Tokyo
    {   66, 4572,   32254,  6, }, // Asia/Tomsk
    {   66, 1021,   32260,  9, }, // Asia/Ulaanbaatar
    {   66, 1159,   32269,  6, }, // Asia/Urumqi
    {   66, 8722,   32275,  9, }, // Asia/Ust-Nera
    {   66, 6977,   32284,  9, }, // Asia/Vientiane
    {   66, 4710,   32293, 12, }, // Asia/Vladivostok
    {   66, 4828,   32305,  9, }, // Asia/Yakutsk
    {   66, 1235,   32314,  5, }, // Asia/Yangon
    {   66, 3883,   32319, 15, }, // Asia/Yekaterinburg
    {   66, 3747,   32334,  7, }, // Asia/Yerevan
    {   66, 3663,   32341,  6, }, // Atlantic/Azores
    {   66, 5671,   32347,  8, }, // Atlantic/Bermuda
    {   66, 6628,   32355,  6, }, // Atlantic/Canary
    {   66, 3727,   32361, 10, }, // Atlantic/Cape_Verde
    {   66, 1395,   32371,  4, }, // Atlantic/Faroe
    {   66, 8736,   32375,  6, }, // Atlantic/Madeira
    {   66, 2748,   32381,  8, }, // Atlantic/Reykjavik
    {   66, 7272,   32389, 12, }, // Atlantic/South_Georgia
    {   66, 6821,   32401, 12, }, // Atlantic/St_Helena
    {   66, 6644,   32413,  7, }, // Atlantic/Stanley
    {   66, 1670,   32420,  6, }, // Australia/Adelaide
    {   66, 1635,   32426,  8, }, // Australia/Brisbane
    {   66, 1799,   32434, 11, }, // Australia/Broken_Hill
    {   66, 1583,   32445,  7, }, // Australia/Darwin
    {   66, 3637,   32452,  6, }, // Australia/Eucla
    {   66, 1516,   32458,  6, }, // Australia/Hobart
    {   66, 8753,   32464,  8, }, // Australia/Lindeman
    {   66, 1547,   32472,  9, }, // Australia/Lord_Howe
    {   66, 1727,   32481,  8, }, // Australia/Melbourne
    {   66, 1762,   32489,  5, }, // Australia/Perth
    {   66, 1463,   32494,  5, }, // Australia/Sydney
    {   66, 6441,   32499, 10, }, // Europe/Amsterdam
    {   66, 6216,   32509,  7, }, // Europe/Andorra
    {   66, 3620,   32516, 10, }, // Europe/Astrakhan
    {   66, 2300,   32526,  6, }, // Europe/Athens
    {   66, 6336,   32532,  8, }, // Europe/Belgrade
    {   66, 4757,   32540,  7, }, // Europe/Berlin
    {   66, 6488,   32547, 12, }, // Europe/Bratislava
    {   66, 2167,   32559, 11, }, // Europe/Brussels
    {   66, 3928,   32570,  9, }, // Europe/Bucharest
    {   66, 3806,   32579,  9, }, // Europe/Budapest
    {   66, 8772,   32588,  8, }, // Europe/Busingen
    {   66, 2583,   32596,  6, }, // Europe/Chisinau
    {   66, 6289,   32602,  9, }, // Europe/Copenhagen
    {   66, 2338,   32611,  6, }, // Europe/Dublin
    {   66, 6307,   32617, 11, }, // Europe/Gibraltar
    {   66, 8788,   32628,  7, }, // Europe/Guernsey
    {   66, 6612,   32635,  9, }, // Europe/Helsinki
    {   66, 8804,   32644, 11, }, // Europe/Isle_of_Man
    {   66, 1130,   32655,  9, }, // Europe/Istanbul
    {   66, 8823,   32664,  6, }, // Europe/Jersey
    {   66, 3979,   32670, 12, }, // Europe/Kaliningrad
    {   66, 8929,   32682,  5, }, // Europe/Kirov
    {   66, 2527,   32687,  3, }, // Europe/Kyiv
    {   66, 3213,   32690,  6, }, // Europe/Lisbon
    {   66, 6506,   32696, 10, }, // Europe/Ljubljana
    {   66, 2501,   32706,  5, }, // Europe/London
    {   66, 6365,   32711, 13, }, // Europe/Luxembourg
    {   66, 4892,   32724,  7, }, // Europe/Madrid
    {   66, 6397,   32731,  6, }, // Europe/Malta
    {   66, 6582,   32737,  7, }, // Europe/Mariehamn
    {   66, 3693,   32744,  7, }, // Europe/Minsk
    {   66, 6410,   32751,  5, }, // Europe/Monaco
    {   66, 3537,   32756,  5, }, // Europe/Moscow
    {   66, 6458,   32761,  5, }, // Europe/Oslo
    {   66, 4284,   32766,  5, }, // Europe/Paris
    {   66, 6424,   32771,  8, }, // Europe/Podgorica
    {   66, 6275,   32779,  5, }, // Europe/Prague
    {   66, 8837,   32784,  4, }, // Europe/Riga
    {   66, 6324,   32788,  3, }, // Europe/Rome
    {   66, 4331,   32791,  6, }, // Europe/Samara
    {   66, 6470,   32797, 10, }, // Europe/San_Marino
    {   66, 6245,   32807,  8, }, // Europe/Sarajevo
    {   66, 4451,   32815,  7, }, // Europe/Saratov
    {   66, 8849,   32822, 11, }, // Europe/Simferopol
    {   66, 6383,   32833,  8, }, // Europe/Skopje
    {   66, 6599,   32841,  5, }, // Europe/Sofia
    {   66, 6523,   32846,  8, }, // Europe/Stockholm
    {   66, 8867,   32854,  5, }, // Europe/Tallinn
    {   66, 6202,   32859,  5, }, // Europe/Tirane
    {   66, 8882,   32864, 12, }, // Europe/Ulyanovsk
    {   66, 6352,   32876,  5, }, // Europe/Vaduz
    {   66, 6567,   32881,  7, }, // Europe/Vatican
    {   66, 6231,   32888,  8, }, // Europe/Vienna
    {   66, 8899,   32896,  8, }, // Europe/Vilnius
    {   66, 4727,   32904, 10, }, // Europe/Volgograd
    {   66, 3190,   32914,  5, }, // Europe/Warsaw
    {   66, 6261,   32919,  7, }, // Europe/Zagreb
    {   66, 6540,   32926,  5, }, // Europe/Zurich
    {   66, 5078,   32931, 13, }, // Indian/Antananarivo
    {   66, 6947,   32944,  5, }, // Indian/Chagos
    {   66, 6046,   32949, 16, }, // Indian/Christmas
    {   66, 6063,   32965,  4, }, // Indian/Cocos
    {   66, 5029,   32969,  5, }, // Indian/Comoro
    {   66, 6661,   32974,  9, }, // Indian/Kerguelen
    {   66, 7260,   32983,  4, }, // Indian/Mahe
    {   66, 7050,   32987,  8, }, // Indian/Maldives
    {   66, 4069,   32995,  8, }, // Indian/Mauritius
    {   66, 5098,   33003,  8, }, // Indian/Mayotte
    {   66, 7211,   33011,  9, }, // Indian/Reunion
    {   66, 4422,   33020,  5, }, // Pacific/Apia
    {   66,  932,   33025,  7, }, // Pacific/Auckland
    {   66, 3706,   33032,  8, }, // Pacific/Bougainville
    {   66, 3015,   33040,  5, }, // Pacific/Chatham
    {   66, 3157,   33045,  3, }, // Pacific/Chuuk
    {   66, 2237,   33048,  9, }, // Pacific/Easter
    {   66, 7360,   33057,  5, }, // Pacific/Efate
    {   66, 7327,   33062,  7, }, // Pacific/Fakaofo
    {   66, 3902,   33069,  4, }, // Pacific/Fiji
    {   66, 7343,   33073,  8, }, // Pacific/Funafuti
    {   66, 6678,   33081,  9, }, // Pacific/Galapagos
    {   66, 6696,   33090,  9, }, // Pacific/Gambier
    {   66, 3822,   33099, 12, }, // Pacific/Guadalcanal
    {   66, 6018,   33111,  5, }, // Pacific/Guam
    {   66, 2723,   33116,  6, }, // Pacific/Honolulu
    {   66, 3049,   33122, 10, }, // Pacific/Kanton
    {   66, 3998,   33132, 10, }, // Pacific/Kiritimati
    {   66, 7019,   33142,  5, }, // Pacific/Kosrae
    {   66, 2842,   33147,  9, }, // Pacific/Kwajalein
    {   66, 4877,   33156,  6, }, // Pacific/Majuro
    {   66, 4051,   33162,  9, }, // Pacific/Marquesas
    {   66, 8914,   33171,  5, }, // Pacific/Midway
    {   66, 7084,   33176,  5, }, // Pacific/Nauru
    {   66, 7132,   33181,  4, }, // Pacific/Niue
    {   66, 4168,   33185,  5, }, // Pacific/Norfolk
    {   66, 7098,   33190,  6, }, // Pacific/Noumea
    {   66, 3126,   33196,  9, }, // Pacific/Pago_Pago
    {   66, 7155,   33205,  5, }, // Pacific/Palau
    {   66, 7194,   33210,  8, }, // Pacific/Pitcairn
    {   66, 3096,   33218,  5, }, // Pacific/Pohnpei
    {   66, 4807,   33223, 11, }, // Pacific/Port_Moresby
    {   66, 6076,   33234,  8, }, // Pacific/Rarotonga
    {   66, 6031,   33242,  6, }, // Pacific/Saipan
    {   66, 7312,   33248,  6, }, // Pacific/Tahiti
    {   66, 6712,   33254,  6, }, // Pacific/Tarawa
    {   66, 4583,   33260,  8, }, // Pacific/Tongatapu
    {   66, 7392,   33268,  3, }, // Pacific/Wake
    {   66, 7405,   33271,  5, }, // Pacific/Wallis
    {   67,   14,       0,  6, }, // Africa/Asmara Bosnian/Latin/Bosnia and Herzegovina
    {   67, 2320,    4690,  5, }, // Africa/Cairo
    {   67, 4117,   33276, 10, }, // Africa/Casablanca
    {   67, 5043,   33286,  7, }, // Africa/Djibouti
    {   67, 4524,    6423,  6, }, // Africa/Khartoum
    {   67, 5113,   33293,  7, }, // Africa/Mogadishu
    {   67, 4435,       6,  8, }, // Africa/Sao_Tome
    {   67, 5607,   33300,  7, }, // America/Anguilla
    {   67, 5624,   33307,  7, }, // America/Antigua
    {   67, 4252,      14,  8, }, // America/Asuncion
    {   67,  237,      22,  8, }, // America/Atikokan
    {   67, 7623,    3943, 14, }, // America/Bahia_Banderas
    {   67, 3868,   33314,  6, }, // America/Cancun
    {   67, 5496,    6694,  6, }, // America/Cayman
    {   67, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   67, 5422,   33320,  9, }, // America/Costa_Rica
    {   67, 5723,    6749,  7, }, // America/Curacao
    {   67, 5739,   21666,  8, }, // America/Dominica
    {   67, 5441,    6806,  8, }, // America/El_Salvador
    {   67, 5786,   33329,  9, }, // America/Guadeloupe
    {   67, 3760,   33338,  9, }, // America/Guatemala
    {   67,  481,      73, 13, }, // America/Indiana/Knox
    {   67, 7961,      86, 16, }, // America/Indiana/Marengo
    {   67, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   67, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   67, 8038,     139, 14, }, // America/Indiana/Vevay
    {   67, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   67, 8086,     171, 16, }, // America/Indiana/Winamac
    {   67, 2799,   33347,  7, }, // America/Jamaica
    {   67, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   67, 5932,     207, 22, }, // America/Lower_Princes
    {   67, 8220,    7126,  6, }, // America/Merida
    {   67, 1850,     235, 19, }, // America/Noronha
    {   67, 8301,   33354, 23, }, // America/North_Dakota/Beulah
    {   67, 8329,   33377, 23, }, // America/North_Dakota/Center
    {   67, 8357,   33400, 26, }, // America/North_Dakota/New_Salem
    {   67,  393,     317,  4, }, // America/Nuuk
    {   67, 5843,   33426,  9, }, // America/Puerto_Rico
    {   67, 6899,     321, 16, }, // America/Scoresbysund
    {   67, 8508,     337, 14, }, // America/St_Barthelemy
    {   67, 2061,     351, 10, }, // America/St_Johns
    {   67, 5863,     361,  9, }, // America/St_Kitts
    {   67, 5880,     370,  9, }, // America/St_Lucia
    {   67,  855,     379, 10, }, // America/St_Thomas
    {   67, 5913,     389, 11, }, // America/St_Vincent
    {   67, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {   67, 5395,    7594,  6, }, // Asia/Almaty
    {   67, 3968,    5605,  4, }, // Asia/Amman
    {   67, 5511,    7600,  6, }, // Asia/Anadyr
    {   67, 5523,    5609,  5, }, // Asia/Aqtau
    {   67, 5368,   33435,  7, }, // Asia/Aqtobe
    {   67,  964,   33442,  7, }, // Asia/Ashgabat
    {   67, 8624,   33449,  6, }, // Asia/Atyrau
    {   67, 3607,    4162,  6, }, // Asia/Baghdad
    {   67, 5534,    4168,  7, }, // Asia/Bahrain
    {   67, 4086,    7625,  6, }, // Asia/Beirut
    {   67, 3778,   33455,  6, }, // Asia/Bishkek
    {   67, 5989,   33461,  7, }, // Asia/Brunei
    {   67, 4511,    5637,  7, }, // Asia/Colombo
    {   67, 4540,    4201,  6, }, // Asia/Damascus
    {   67, 1093,    4717,  4, }, // Asia/Dhaka
    {   67, 6159,   33468,  7, }, // Asia/Dushanbe
    {   67, 1259,   33475,  9, }, // Asia/Ho_Chi_Minh
    {   67, 7006,   33484,  8, }, // Asia/Jakarta
    {   67, 6992,   33492,  9, }, // Asia/Jayapura
    {   67, 1290,   33501,  9, }, // Asia/Jerusalem
    {   67, 4316,   33510,  8, }, // Asia/Kamchatka
    {   67, 4239,   33518,  6, }, // Asia/Karachi
    {   67, 1185,    5711,  8, }, // Asia/Kathmandu
    {   67, 8661,   33524,  7, }, // Asia/Khandyga
    {   67,  992,     457,  7, }, // Asia/Kolkata
    {   67, 4197,    4288, 11, }, // Asia/Krasnoyarsk
    {   67, 5976,   33531,  6, }, // Asia/Kuching
    {   67, 5547,    7725,  6, }, // Asia/Kuwait
    {   67, 1211,   33537,  5, }, // Asia/Macau
    {   67, 1349,    7736,  7, }, // Asia/Makassar
    {   67, 6920,    4326,  6, }, // Asia/Muscat
    {   67, 2554,   33542,  8, }, // Asia/Nicosia
    {   67, 8675,   33550, 12, }, // Asia/Novokuznetsk
    {   67, 6961,   22518,  8, }, // Asia/Phnom_Penh
    {   67, 4214,   33562,  9, }, // Asia/Pyongyang
    {   67, 5559,    4332,  5, }, // Asia/Qatar
    {   67, 8708,    7770,  8, }, // Asia/Qostanay
    {   67, 4269,    7778,  9, }, // Asia/Qyzylorda
    {   67, 3584,   33571,  5, }, // Asia/Riyadh
    {   67, 4408,   33576,  7, }, // Asia/Sakhalin
    {   67, 3279,    7798,  4, }, // Asia/Seoul
    {   67, 1053,   33583,  6, }, // Asia/Shanghai
    {   67, 3300,   20900,  8, }, // Asia/Singapore
    {   67, 3263,    7830,  6, }, // Asia/Taipei
    {   67, 4781,   33589,  7, }, // Asia/Tashkent
    {   67, 2772,    7842,  7, }, // Asia/Tehran
    {   67, 2821,    4356,  5, }, // Asia/Tokyo
    {   67, 1021,   26923, 10, }, // Asia/Ulaanbaatar
    {   67, 1159,   33596,  6, }, // Asia/Urumqi
    {   67, 6977,   33602, 10, }, // Asia/Vientiane
    {   67, 4828,    7873,  7, }, // Asia/Yakutsk
    {   67, 1235,    7880,  6, }, // Asia/Yangon
    {   67, 3883,    4380, 13, }, // Asia/Yekaterinburg
    {   67, 3747,    7898,  7, }, // Asia/Yerevan
    {   67, 3663,   33612,  5, }, // Atlantic/Azores
    {   67, 6628,    4398,  6, }, // Atlantic/Canary
    {   67, 3727,   22654, 10, }, // Atlantic/Cape_Verde
    {   67, 1395,     483,  5, }, // Atlantic/Faroe
    {   67, 2748,   33617,  9, }, // Atlantic/Reykjavik
    {   67, 6821,   33626, 12, }, // Atlantic/St_Helena
    {   67, 1547,   22727,  8, }, // Australia/Lord_Howe
    {   67, 1727,    8021,  7, }, // Australia/Melbourne
    {   67, 1762,   22735,  4, }, // Australia/Perth
    {   67, 1463,    8033,  6, }, // Australia/Sydney
    {   67, 6216,   33638,  6, }, // Europe/Andorra
    {   67, 3620,   33644,  8, }, // Europe/Astrakhan
    {   67, 2300,   33652,  5, }, // Europe/Athens
    {   67, 6336,    8060,  7, }, // Europe/Belgrade
    {   67, 2167,   33657,  6, }, // Europe/Brussels
    {   67, 3928,   33663,  8, }, // Europe/Bucharest
    {   67, 3806,   33671, 10, }, // Europe/Budapest
    {   67, 2583,   33681,  8, }, // Europe/Chisinau
    {   67, 6289,    4475, 10, }, // Europe/Copenhagen
    {   67, 2338,   33689,  6, }, // Europe/Dublin
    {   67, 8788,   33695,  6, }, // Europe/Guernsey
    {   67, 8804,   33701, 10, }, // Europe/Isle_of_Man
    {   67, 3979,   33711, 12, }, // Europe/Kaliningrad
    {   67, 2527,   33723,  5, }, // Europe/Kyiv
    {   67, 3213,   33728,  7, }, // Europe/Lisbon
    {   67, 6365,    8172, 10, }, // Europe/Luxembourg
    {   67, 6410,    6044,  6, }, // Europe/Monaco
    {   67, 3537,   22880,  6, }, // Europe/Moscow
    {   67, 4284,   33735,  5, }, // Europe/Paris
    {   67, 6275,    8201,  4, }, // Europe/Prague
    {   67, 6324,   33740,  3, }, // Europe/Rome
    {   67, 8849,   33743, 11, }, // Europe/Simferopol
    {   67, 6383,   33754,  7, }, // Europe/Skopje
    {   67, 6599,   33761,  6, }, // Europe/Sofia
    {   67, 6523,   33767,  8, }, // Europe/Stockholm
    {   67, 8867,    8250,  5, }, // Europe/Tallinn
    {   67, 6202,   21213,  6, }, // Europe/Tirane
    {   67, 6567,    6096,  7, }, // Europe/Vatican
    {   67, 6231,   33775,  3, }, // Europe/Vienna
    {   67, 3190,   33778,  7, }, // Europe/Warsaw
    {   67, 6540,   33785,  5, }, // Europe/Zurich
    {   67, 6046,   33790, 14, }, // Indian/Christmas
    {   67, 6063,   33804, 15, }, // Indian/Cocos
    {   67, 7050,   33819,  7, }, // Indian/Maldives
    {   67, 4069,   33826, 10, }, // Indian/Mauritius
    {   67, 7211,     547,  7, }, // Indian/Reunion
    {   67, 3157,     554,  5, }, // Pacific/Chuuk
    {   67, 3902,   33836,  5, }, // Pacific/Fiji
    {   67, 2723,    4611,  8, }, // Pacific/Honolulu
    {   67, 3049,    4619,  9, }, // Pacific/Kanton
    {   67, 7194,    8422,  7, }, // Pacific/Pitcairn
    {   67, 3096,     586,  7, }, // Pacific/Pohnpei
    {   67, 7405,   33841,  5, }, // Pacific/Wallis
    {   68, 6788,     604,  6, }, // Africa/Abidjan Bosnian/Cyrillic/Bosnia and Herzegovina
    {   68, 6760,   27178,  4, }, // Africa/Accra
    {   68, 5059,   33846, 10, }, // Africa/Addis_Ababa
    {   68, 7420,     626,  5, }, // Africa/Algiers
    {   68,   14,     631,  6, }, // Africa/Asmara
    {   68,   44,     637,  6, }, // Africa/Bamako
    {   68, 5257,   33856,  6, }, // Africa/Bangui
    {   68, 6746,     648,  6, }, // Africa/Banjul
    {   68, 7435,   33862,  5, }, // Africa/Bissau
    {   68, 4957,   33867,  7, }, // Africa/Blantyre
    {   68, 5287,   33874,  8, }, // Africa/Brazzaville
    {   68, 4922,   33882,  9, }, // Africa/Bujumbura
    {   68, 2320,   33891,  5, }, // Africa/Cairo
    {   68, 4117,   33896, 10, }, // Africa/Casablanca
    {   68, 7449,     699,  5, }, // Africa/Ceuta
    {   68, 6773,     704,  7, }, // Africa/Conakry
    {   68, 6840,     711,  5, }, // Africa/Dakar
    {   68, 5130,     716, 12, }, // Africa/Dar_es_Salaam
    {   68, 5043,     728,  6, }, // Africa/Djibouti
    {   68, 5243,     734,  5, }, // Africa/Douala
    {   68, 5166,   33906,  7, }, // Africa/El_Aaiun
    {   68, 6853,     746,  7, }, // Africa/Freetown
    {   68, 4906,     753,  7, }, // Africa/Gaborone
    {   68, 5015,   33913,  6, }, // Africa/Harare
    {   68, 4479,   33919, 11, }, // Africa/Johannesburg
    {   68, 4499,     779,  4, }, // Africa/Juba
    {   68, 5151,     783,  7, }, // Africa/Kampala
    {   68, 4524,   33930,  6, }, // Africa/Khartoum
    {   68, 4973,     796,  6, }, // Africa/Kigali
    {   68, 5306,     802,  7, }, // Africa/Kinshasa
    {   68, 4744,     809,  5, }, // Africa/Lagos
    {   68, 5336,     814,  8, }, // Africa/Libreville
    {   68, 6869,     822,  4, }, // Africa/Lome
    {   68, 5211,     826,  6, }, // Africa/Luanda
    {   68, 4939,   33936,  9, }, // Africa/Lubumbashi
    {   68, 5001,     841,  6, }, // Africa/Lusaka
    {   68, 5322,     847,  6, }, // Africa/Malabo
    {   68, 4987,   33945,  6, }, // Africa/Maputo
    {   68, 5197,     859,  6, }, // Africa/Maseru
    {   68, 5182,     865,  7, }, // Africa/Mbabane
    {   68, 5113,     872,  7, }, // Africa/Mogadishu
    {   68, 7034,   33951,  9, }, // Africa/Monrovia
    {   68, 3853,   33960,  7, }, // Africa/Nairobi
    {   68, 5271,     895,  7, }, // Africa/Ndjamena
    {   68, 5354,   33967,  7, }, // Africa/Niamey
    {   68, 6803,     908,  7, }, // Africa/Nouakchott
    {   68, 6727,     915,  8, }, // Africa/Ouagadougou
    {   68, 5225,   33974, 10, }, // Africa/Porto-Novo
    {   68, 4435,   33984,  8, }, // Africa/Sao_Tome
    {   68, 2866,     941,  7, }, // Africa/Tripoli
    {   68, 6554,     948,  5, }, // Africa/Tunis
    {   68, 4152,     953,  7, }, // Africa/Windhoek
    {   68,  132,     960,  4, }, // America/Adak
    {   68, 3346,   33992,  7, }, // America/Anchorage
    {   68, 5607,   33999,  7, }, // America/Anguilla
    {   68, 5624,   34006,  7, }, // America/Antigua
    {   68, 4554,   34013, 10, }, // America/Araguaina
    {   68,  166,   34023, 12, }, // America/Argentina/Buenos_Aires
    {   68,   91,    1006,  9, }, // America/Argentina/Catamarca
    {   68,  270,   34035,  7, }, // America/Argentina/Cordoba
    {   68,  441,   34042,  5, }, // America/Argentina/Jujuy
    {   68, 7462,   34047,  8, }, // America/Argentina/La_Rioja
    {   68,  565,   34055,  7, }, // America/Argentina/Mendoza
    {   68, 7489,   34062, 11, }, // America/Argentina/Rio_Gallegos
    {   68, 7520,    1053,  5, }, // America/Argentina/Salta
    {   68, 7544,   34073,  8, }, // America/Argentina/San_Juan
    {   68, 5580,   34081,  7, }, // America/Argentina/San_Luis
    {   68, 7571,    1074,  7, }, // America/Argentina/Tucuman
    {   68, 7597,   34088,  7, }, // America/Argentina/Ushuaia
    {   68, 5640,    1087,  5, }, // America/Aruba
    {   68, 4252,   34095,  8, }, // America/Asuncion
    {   68,  237,   34103,  8, }, // America/Atikokan
    {   68, 3679,   34111,  5, }, // America/Bahia
    {   68, 7623,   34116, 14, }, // America/Bahia_Banderas
    {   68, 5654,    1131,  8, }, // America/Barbados
    {   68, 7646,   34130,  5, }, // America/Belem
    {   68, 5407,   34135,  6, }, // America/Belize
    {   68, 7660,   34141, 13, }, // America/Blanc-Sablon
    {   68, 7681,   34154,  9, }, // America/Boa_Vista
    {   68, 4361,    1170,  6, }, // America/Bogota
    {   68, 7699,   34163,  5, }, // America/Boise
    {   68, 7713,   34168, 11, }, // America/Cambridge_Bay
    {   68, 7735,   34179, 12, }, // America/Campo_Grande
    {   68, 3868,    1205,  6, }, // America/Cancun
    {   68, 4694,    1211,  7, }, // America/Caracas
    {   68, 4345,   34191,  5, }, // America/Cayenne
    {   68, 5496,   34196, 16, }, // America/Cayman
    {   68, 2260,    1247,  6, }, // America/Chicago
    {   68, 7756,   34212,  6, }, // America/Chihuahua
    {   68, 5422,   34218,  9, }, // America/Costa_Rica
    {   68, 7796,    1269,  7, }, // America/Creston
    {   68, 3791,    1276,  6, }, // America/Cuiaba
    {   68, 5723,   34227,  7, }, // America/Curacao
    {   68, 7812,   34234, 13, }, // America/Danmarkshavn
    {   68, 7833,   34247,  5, }, // America/Dawson
    {   68, 7848,   34252, 10, }, // America/Dawson_Creek
    {   68,  805,    1319,  6, }, // America/Denver
    {   68, 3465,    1325,  7, }, // America/Detroit
    {   68, 5739,    1332,  8, }, // America/Dominica
    {   68,  893,    1340,  8, }, // America/Edmonton
    {   68, 7869,    1348,  8, }, // America/Eirunepe
    {   68, 5441,    1356,  8, }, // America/El_Salvador
    {   68, 7886,    1364, 11, }, // America/Fort_Nelson
    {   68, 7906,    1375,  9, }, // America/Fortaleza
    {   68, 7924,   34262,  9, }, // America/Glace_Bay
    {   68, 6881,   34271,  7, }, // America/Goose_Bay
    {   68, 4612,   34278, 10, }, // America/Grand_Turk
    {   68, 5770,    1410,  7, }, // America/Grenada
    {   68, 5786,   34288,  9, }, // America/Guadeloupe
    {   68, 3760,    1426,  9, }, // America/Guatemala
    {   68, 4841,   34297,  8, }, // America/Guayaquil
    {   68, 6932,   34305,  5, }, // America/Guyana
    {   68, 1939,   34310,  8, }, // America/Halifax
    {   68, 2281,   34318,  6, }, // America/Havana
    {   68, 7942,   34324,  9, }, // America/Hermosillo
    {   68,  348,    1471, 12, }, // America/Indiana/Indianapolis
    {   68,  481,   34333, 15, }, // America/Indiana/Knox
    {   68, 7961,   34348, 17, }, // America/Indiana/Marengo
    {   68, 7985,   34365, 20, }, // America/Indiana/Petersburg
    {   68, 8012,   34385, 18, }, // America/Indiana/Tell_City
    {   68, 8038,   34403, 15, }, // America/Indiana/Vevay
    {   68, 8060,   34418, 18, }, // America/Indiana/Vincennes
    {   68, 8086,   34436, 17, }, // America/Indiana/Winamac
    {   68, 8110,    1570,  6, }, // America/Inuvik
    {   68,  660,   34453,  8, }, // America/Iqaluit
    {   68, 2799,   34461,  7, }, // America/Jamaica
    {   68, 5380,   34468,  4, }, // America/Juneau
    {   68,  521,    1594,  7, }, // America/Kentucky/Louisville
    {   68, 8125,   34472, 18, }, // America/Kentucky/Monticello
    {   68, 5704,   34490, 10, }, // America/Kralendijk
    {   68, 4376,   34500,  6, }, // America/La_Paz
    {   68, 7169,    1638,  4, }, // America/Lima
    {   68, 3239,   34506, 11, }, // America/Los_Angeles
    {   68, 5932,   34517, 19, }, // America/Lower_Princes
    {   68, 8153,   34536,  6, }, // America/Maceio
    {   68, 8168,   34542,  7, }, // America/Managua
    {   68, 1908,    1684,  6, }, // America/Manaus
    {   68, 5897,    1690,  6, }, // America/Marigot
    {   68, 5805,    1696,  8, }, // America/Martinique
    {   68, 8184,    1705,  9, }, // America/Matamoros
    {   68, 2917,   34549,  8, }, // America/Mazatlan
    {   68, 8202,    1722,  8, }, // America/Menominee
    {   68, 8220,    1730,  6, }, // America/Merida
    {   68, 8235,    1736, 10, }, // America/Metlakatla
    {   68, 2949,   34557, 12, }, // America/Mexico_City
    {   68, 4391,    1752,  7, }, // America/Miquelon
    {   68, 8254,    1759,  7, }, // America/Moncton
    {   68, 8270,   34569,  8, }, // America/Monterrey
    {   68, 4098,    1775, 10, }, // America/Montevideo
    {   68, 5824,   34577,  9, }, // America/Montserrat
    {   68, 5481,   34586,  5, }, // America/Nassau
    {   68, 2379,   34591,  6, }, // America/New_York
    {   68, 8288,    1809,  3, }, // America/Nome
    {   68, 1850,   34597,  6, }, // America/Noronha
    {   68, 8301,   34603, 22, }, // America/North_Dakota/Beulah
    {   68, 8329,   34625, 22, }, // America/North_Dakota/Center
    {   68, 8357,   34647, 26, }, // America/North_Dakota/New_Salem
    {   68,  393,   28283,  3, }, // America/Nuuk
    {   68, 8388,    1894,  7, }, // America/Ojinaga
    {   68, 2356,    1901,  6, }, // America/Panama
    {   68, 6173,    1907, 10, }, // America/Paramaribo
    {   68, 2973,   34673,  6, }, // America/Phoenix
    {   68, 3945,    1923, 12, }, // America/Port-au-Prince
    {   68, 5954,   34679, 13, }, // America/Port_of_Spain
    {   68, 8404,   34692, 10, }, // America/Porto_Velho
    {   68, 5843,   34702,  9, }, // America/Puerto_Rico
    {   68, 4030,   34711, 12, }, // America/Punta_Arenas
    {   68, 8424,   34723, 12, }, // America/Rankin_Inlet
    {   68, 8445,   34735,  6, }, // America/Recife
    {   68, 1995,   34741,  6, }, // America/Regina
    {   68, 8460,   34747,  7, }, // America/Resolute
    {   68,  695,   34754, 10, }, // America/Rio_Branco
    {   68, 8477,   34764,  8, }, // America/Santarem
    {   68, 2201,   34772,  9, }, // America/Santiago
    {   68, 6111,   34781, 13, }, // America/Santo_Domingo
    {   68, 1878,   34794,  9, }, // America/Sao_Paulo
    {   68, 6899,   34803, 12, }, // America/Scoresbysund
    {   68, 8494,    2074,  5, }, // America/Sitka
    {   68, 8508,   34815, 14, }, // America/St_Barthelemy
    {   68, 2061,   34829,  7, }, // America/St_Johns
    {   68, 5863,   34836,  9, }, // America/St_Kitts
    {   68, 5880,   34845, 10, }, // America/St_Lucia
    {   68,  855,   34855,  8, }, // America/St_Thomas
    {   68, 5913,   34863, 12, }, // America/St_Vincent
    {   68, 8530,   34875, 12, }, // America/Swift_Current
    {   68, 5461,    2156, 11, }, // America/Tegucigalpa
    {   68, 5756,    2167,  4, }, // America/Thule
    {   68,  313,    2171,  7, }, // America/Tijuana
    {   68,  608,    2178,  7, }, // America/Toronto
    {   68, 5688,    2185,  7, }, // America/Tortola
    {   68, 2093,    2192,  8, }, // America/Vancouver
    {   68, 2144,   34887,  8, }, // America/Whitehorse
    {   68,  734,   34895,  7, }, // America/Winnipeg
    {   68, 8552,   34902,  7, }, // America/Yakutat
    {   68, 6001,   34909,  5, }, // Antarctica/Casey
    {   68, 6094,   34914,  6, }, // Antarctica/Davis
    {   68, 6133,   34920, 13, }, // Antarctica/DumontDUrville
    {   68, 8568,   34933,  7, }, // Antarctica/Macquarie
    {   68, 7066,   34940,  5, }, // Antarctica/Mawson
    {   68, 7113,   34945,  8, }, // Antarctica/McMurdo
    {   68, 8589,    2272,  6, }, // Antarctica/Palmer
    {   68, 7226,    2278,  6, }, // Antarctica/Rothera
    {   68, 7295,   34953,  4, }, // Antarctica/Syowa
    {   68, 8607,    2289,  4, }, // Antarctica/Troll
    {   68, 7374,   34957,  6, }, // Antarctica/Vostok
    {   68, 1429,   34963, 11, }, // Arctic/Longyearbyen
    {   68, 5570,    2310,  4, }, // Asia/Aden
    {   68, 5395,   34974,  6, }, // Asia/Almaty
    {   68, 3968,   28663,  4, }, // Asia/Amman
    {   68, 5511,   34980,  6, }, // Asia/Anadyr
    {   68, 5523,    2331,  5, }, // Asia/Aqtau
    {   68, 5368,   34986,  7, }, // Asia/Aqtobe
    {   68,  964,    2342,  7, }, // Asia/Ashgabat
    {   68, 8624,   34993,  6, }, // Asia/Atyrau
    {   68, 3607,    2355,  6, }, // Asia/Baghdad
    {   68, 5534,    2361,  7, }, // Asia/Bahrain
    {   68, 3653,    2368,  4, }, // Asia/Baku
    {   68, 4466,   34999,  6, }, // Asia/Bangkok
    {   68, 3571,    2379,  7, }, // Asia/Barnaul
    {   68, 4086,   35005,  6, }, // Asia/Beirut
    {   68, 3778,    2392,  6, }, // Asia/Bishkek
    {   68, 5989,   35011,  7, }, // Asia/Brunei
    {   68, 4601,    2404,  4, }, // Asia/Chita
    {   68, 4511,    2408,  7, }, // Asia/Colombo
    {   68, 4540,    2415,  6, }, // Asia/Damascus
    {   68, 1093,     711,  4, }, // Asia/Dhaka
    {   68, 6192,    2426,  4, }, // Asia/Dili
    {   68, 3596,    2430,  5, }, // Asia/Dubai
    {   68, 6159,    2435,  7, }, // Asia/Dushanbe
    {   68, 8636,    2442,  9, }, // Asia/Famagusta
    {   68, 8651,    2451,  4, }, // Asia/Gaza
    {   68, 4795,   35018,  6, }, // Asia/Hebron
    {   68, 1259,   35024,  9, }, // Asia/Ho_Chi_Minh
    {   68, 2704,   35033,  9, }, // Asia/Hong_Kong
    {   68, 4771,    2475,  4, }, // Asia/Hovd
    {   68, 4184,   35042,  6, }, // Asia/Irkutsk
    {   68, 7006,    2486,  7, }, // Asia/Jakarta
    {   68, 6992,   35048,  8, }, // Asia/Jayapura
    {   68, 1290,   35056,  9, }, // Asia/Jerusalem
    {   68, 3560,    2510,  5, }, // Asia/Kabul
    {   68, 4316,    2529,  8, }, // Asia/Kamchatka
    {   68, 4239,    2540,  6, }, // Asia/Karachi
    {   68, 1185,    2546,  8, }, // Asia/Kathmandu
    {   68, 8661,   35065,  7, }, // Asia/Khandyga
    {   68,  992,   35072,  7, }, // Asia/Kolkata
    {   68, 4197,   35079, 11, }, // Asia/Krasnoyarsk
    {   68, 4859,   35090, 12, }, // Asia/Kuala_Lumpur
    {   68, 5976,    2592,  6, }, // Asia/Kuching
    {   68, 5547,   35102,  6, }, // Asia/Kuwait
    {   68, 1211,   35108,  5, }, // Asia/Macau
    {   68, 4017,    2609,  7, }, // Asia/Magadan
    {   68, 1349,    2616,  7, }, // Asia/Makassar
    {   68, 7182,    2623,  6, }, // Asia/Manila
    {   68, 6920,   35113,  6, }, // Asia/Muscat
    {   68, 2554,   35119,  8, }, // Asia/Nicosia
    {   68, 8675,   35127, 11, }, // Asia/Novokuznetsk
    {   68, 4135,    2653, 11, }, // Asia/Novosibirsk
    {   68, 4229,    2664,  4, }, // Asia/Omsk
    {   68, 7145,   35138,  4, }, // Asia/Oral
    {   68, 6961,   35142,  8, }, // Asia/Phnom_Penh
    {   68, 8693,    2681,  9, }, // Asia/Pontianak
    {   68, 4214,   35150,  9, }, // Asia/Pyongyang
    {   68, 5559,    2697,  5, }, // Asia/Qatar
    {   68, 8708,   35159,  8, }, // Asia/Qostanay
    {   68, 4269,   35167,  9, }, // Asia/Qyzylorda
    {   68, 3584,   35176,  5, }, // Asia/Riyadh
    {   68, 4408,    2732,  7, }, // Asia/Sakhalin
    {   68, 7245,    2739,  9, }, // Asia/Samarkand
    {   68, 3279,    2748,  4, }, // Asia/Seoul
    {   68, 1053,   35181,  6, }, // Asia/Shanghai
    {   68, 3300,    2758,  8, }, // Asia/Singapore
    {   68, 4297,   35187, 13, }, // Asia/Srednekolymsk
    {   68, 3263,   35200,  6, }, // Asia/Taipei
    {   68, 4781,    2785,  7, }, // Asia/Tashkent
    {   68, 3915,   35206,  7, }, // Asia/Tbilisi
    {   68, 2772,   35213,  7, }, // Asia/Tehran
    {   68, 1317,   35220,  5, }, // Asia/Thimphu
    {   68, 2821,    2810,  5, }, // Asia/Tokyo
    {   68, 4572,    2815,  5, }, // Asia/Tomsk
    {   68, 1021,   35225, 10, }, // Asia/Ulaanbaatar
    {   68, 1159,   35235,  7, }, // Asia/Urumqi
    {   68, 8722,    2836,  8, }, // Asia/Ust-Nera
    {   68, 6977,   35242, 10, }, // Asia/Vientiane
    {   68, 4710,    2851, 11, }, // Asia/Vladivostok
    {   68, 4828,   35252,  7, }, // Asia/Yakutsk
    {   68, 1235,   29043,  6, }, // Asia/Yangon
    {   68, 3883,   35259, 13, }, // Asia/Yekaterinburg
    {   68, 3747,   35272,  7, }, // Asia/Yerevan
    {   68, 3663,   35279,  5, }, // Atlantic/Azores
    {   68, 5671,   35284,  7, }, // Atlantic/Bermuda
    {   68, 6628,   35291, 15, }, // Atlantic/Canary
    {   68, 3727,   35306, 18, }, // Atlantic/Cape_Verde
    {   68, 1395,   35324, 13, }, // Atlantic/Faroe
    {   68, 8736,    2971,  7, }, // Atlantic/Madeira
    {   68, 2748,   35337,  9, }, // Atlantic/Reykjavik
    {   68, 7272,   35346, 13, }, // Atlantic/South_Georgia
    {   68, 6821,   35359, 10, }, // Atlantic/St_Helena
    {   68, 6644,    3029,  6, }, // Atlantic/Stanley
    {   68, 1670,    3035,  8, }, // Australia/Adelaide
    {   68, 1635,   35369,  8, }, // Australia/Brisbane
    {   68, 1799,   35377, 10, }, // Australia/Broken_Hill
    {   68, 1583,    3061,  6, }, // Australia/Darwin
    {   68, 3637,    3067,  5, }, // Australia/Eucla
    {   68, 1516,    3072,  6, }, // Australia/Hobart
    {   68, 8753,    3078,  8, }, // Australia/Lindeman
    {   68, 1547,   35387,  8, }, // Australia/Lord_Howe
    {   68, 1727,    3094,  7, }, // Australia/Melbourne
    {   68, 1762,    3101,  4, }, // Australia/Perth
    {   68, 1463,   35395,  6, }, // Australia/Sydney
    {   68, 6441,    3111,  9, }, // Europe/Amsterdam
    {   68, 6216,   29275,  6, }, // Europe/Andorra
    {   68, 3620,    3127,  8, }, // Europe/Astrakhan
    {   68, 2300,   35401,  5, }, // Europe/Athens
    {   68, 6336,   35406,  7, }, // Europe/Belgrade
    {   68, 4757,    3146,  6, }, // Europe/Berlin
    {   68, 6488,    3152, 10, }, // Europe/Bratislava
    {   68, 2167,   35413,  6, }, // Europe/Brussels
    {   68, 3928,   35419,  8, }, // Europe/Bucharest
    {   68, 3806,   35427, 10, }, // Europe/Budapest
    {   68, 8772,   35437,  8, }, // Europe/Busingen
    {   68, 2583,   35445,  7, }, // Europe/Chisinau
    {   68, 6289,   35452, 10, }, // Europe/Copenhagen
    {   68, 2338,   35462,  6, }, // Europe/Dublin
    {   68, 6307,    3232,  9, }, // Europe/Gibraltar
    {   68, 8788,   35468,  6, }, // Europe/Guernsey
    {   68, 6612,    3247,  8, }, // Europe/Helsinki
    {   68, 8804,   35474, 10, }, // Europe/Isle_of_Man
    {   68, 1130,   35484,  8, }, // Europe/Istanbul
    {   68, 8823,    3271,  5, }, // Europe/Jersey
    {   68, 3979,   35492, 11, }, // Europe/Kaliningrad
    {   68, 8929,    3287,  5, }, // Europe/Kirov
    {   68, 2527,   35503,  5, }, // Europe/Kyiv
    {   68, 3213,   35508,  7, }, // Europe/Lisbon
    {   68, 6506,   35515,  7, }, // Europe/Ljubljana
    {   68, 2501,    3313,  6, }, // Europe/London
    {   68, 6365,   35522, 10, }, // Europe/Luxembourg
    {   68, 4892,    3330,  6, }, // Europe/Madrid
    {   68, 6397,    3336,  5, }, // Europe/Malta
    {   68, 6582,   35532,  8, }, // Europe/Mariehamn
    {   68, 3693,    3350,  5, }, // Europe/Minsk
    {   68, 6410,    3355,  6, }, // Europe/Monaco
    {   68, 3537,    3361,  6, }, // Europe/Moscow
    {   68, 6458,    3367,  4, }, // Europe/Oslo
    {   68, 4284,   35540,  5, }, // Europe/Paris
    {   68, 6424,    3376,  9, }, // Europe/Podgorica
    {   68, 6275,    3385,  4, }, // Europe/Prague
    {   68, 8837,    3390,  4, }, // Europe/Riga
    {   68, 6324,    3394,  3, }, // Europe/Rome
    {   68, 4331,    3397,  6, }, // Europe/Samara
    {   68, 6470,   35545, 10, }, // Europe/San_Marino
    {   68, 6245,   35555,  8, }, // Europe/Sarajevo
    {   68, 4451,    3420,  7, }, // Europe/Saratov
    {   68, 8849,   35563, 10, }, // Europe/Simferopol
    {   68, 6383,   35573,  6, }, // Europe/Skopje
    {   68, 6599,   35579,  6, }, // Europe/Sofia
    {   68, 6523,   35585,  8, }, // Europe/Stockholm
    {   68, 8867,   35593,  5, }, // Europe/Tallinn
    {   68, 6202,    3461,  6, }, // Europe/Tirane
    {   68, 8882,   35598,  8, }, // Europe/Ulyanovsk
    {   68, 6352,   35606,  5, }, // Europe/Vaduz
    {   68, 6567,    3481,  7, }, // Europe/Vatican
    {   68, 6231,   35611,  3, }, // Europe/Vienna
    {   68, 8899,   35614,  6, }, // Europe/Vilnius
    {   68, 4727,    3499,  9, }, // Europe/Volgograd
    {   68, 3190,    3508,  7, }, // Europe/Warsaw
    {   68, 6261,    3515,  6, }, // Europe/Zagreb
    {   68, 6540,   35620,  5, }, // Europe/Zurich
    {   68, 5078,   35625, 12, }, // Indian/Antananarivo
    {   68, 6947,    3539,  5, }, // Indian/Chagos
    {   68, 6046,   35637,  5, }, // Indian/Christmas
    {   68, 6063,    3556,  5, }, // Indian/Cocos
    {   68, 5029,   35642,  6, }, // Indian/Comoro
    {   68, 6661,    3578,  8, }, // Indian/Kerguelen
    {   68, 7260,   35648,  4, }, // Indian/Mahe
    {   68, 7050,   35652,  7, }, // Indian/Maldives
    {   68, 4069,   35659, 10, }, // Indian/Mauritius
    {   68, 5098,   35669,  5, }, // Indian/Mayotte
    {   68, 7211,   35674,  7, }, // Indian/Reunion
    {   68, 4422,   35681,  5, }, // Pacific/Apia
    {   68,  932,   35686,  6, }, // Pacific/Auckland
    {   68, 3706,    3626,  8, }, // Pacific/Bougainville
    {   68, 3015,   35692,  6, }, // Pacific/Chatham
    {   68, 3157,    3639,  4, }, // Pacific/Chuuk
    {   68, 2237,   35698, 14, }, // Pacific/Easter
    {   68, 7360,    3654,  4, }, // Pacific/Efate
    {   68, 7327,    3659,  7, }, // Pacific/Fakaofo
    {   68, 3902,    3666,  4, }, // Pacific/Fiji
    {   68, 7343,    3670,  8, }, // Pacific/Funafuti
    {   68, 6678,    3678,  9, }, // Pacific/Galapagos
    {   68, 6696,   35712,  7, }, // Pacific/Gambier
    {   68, 3822,   35719, 11, }, // Pacific/Guadalcanal
    {   68, 6018,    3724,  4, }, // Pacific/Guam
    {   68, 2723,   35730,  8, }, // Pacific/Honolulu
    {   68, 3049,   35738,  9, }, // Pacific/Kanton
    {   68, 3998,    3734, 10, }, // Pacific/Kiritimati
    {   68, 7019,   35747,  5, }, // Pacific/Kosrae
    {   68, 2842,   35752, 10, }, // Pacific/Kwajalein
    {   68, 4877,   35762,  6, }, // Pacific/Majuro
    {   68, 4051,    3765,  6, }, // Pacific/Marquesas
    {   68, 8914,   35768,  6, }, // Pacific/Midway
    {   68, 7084,    3798,  5, }, // Pacific/Nauru
    {   68, 7132,    3803,  4, }, // Pacific/Niue
    {   68, 4168,    3807,  7, }, // Pacific/Norfolk
    {   68, 7098,    3814,  5, }, // Pacific/Noumea
    {   68, 3126,   35774,  9, }, // Pacific/Pago_Pago
    {   68, 7155,    3828,  5, }, // Pacific/Palau
    {   68, 7194,    3833,  7, }, // Pacific/Pitcairn
    {   68, 3096,   35783,  6, }, // Pacific/Pohnpei
    {   68, 4807,   35789, 11, }, // Pacific/Port_Moresby
    {   68, 6076,    3857,  9, }, // Pacific/Rarotonga
    {   68, 6031,   35800,  6, }, // Pacific/Saipan
    {   68, 7312,   35806,  6, }, // Pacific/Tahiti
    {   68, 6712,    3877,  6, }, // Pacific/Tarawa
    {   68, 4583,    3883,  9, }, // Pacific/Tongatapu
    {   68, 7392,   35812,  4, }, // Pacific/Wake
    {   68, 7405,   35816,  5, }, // Pacific/Wallis
    {   69, 5059,   35821, 10, }, // Africa/Addis_Ababa Breton/Latin/France
    {   69, 7420,   26475,  5, }, // Africa/Algiers
    {   69,   14,       0,  6, }, // Africa/Asmara
    {   69, 2320,   35831,  5, }, // Africa/Cairo
    {   69, 4117,   35836, 25, }, // Africa/Casablanca
    {   69, 5166,   35861,  7, }, // Africa/El_Aaiun
    {   69, 6869,   20418,  4, }, // Africa/Lome
    {   69, 5113,   26511,  8, }, // Africa/Mogadishu
    {   69, 5271,   35868,  9, }, // Africa/Ndjamena
    {   69, 4435,       6,  8, }, // Africa/Sao_Tome
    {   69, 2866,   35877, 19, }, // Africa/Tripoli
    {   69, 6554,    6467,  5, }, // Africa/Tunis
    {   69, 4252,      14,  8, }, // America/Asuncion
    {   69,  237,      22,  8, }, // America/Atikokan
    {   69, 7623,      30, 17, }, // America/Bahia_Banderas
    {   69, 7646,   35896,  5, }, // America/Belem
    {   69, 4361,   20507,  6, }, // America/Bogota
    {   69, 3868,      47,  6, }, // America/Cancun
    {   69, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   69, 5723,      66,  7, }, // America/Curacao
    {   69, 5441,    6806,  8, }, // America/El_Salvador
    {   69, 5786,   35901,  9, }, // America/Guadeloupe
    {   69, 2281,   20535,  9, }, // America/Havana
    {   69,  481,      73, 13, }, // America/Indiana/Knox
    {   69, 7961,      86, 16, }, // America/Indiana/Marengo
    {   69, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   69, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   69, 8038,     139, 14, }, // America/Indiana/Vevay
    {   69, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   69, 8086,     171, 16, }, // America/Indiana/Winamac
    {   69, 2799,    3995,  7, }, // America/Jamaica
    {   69, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   69, 5932,     207, 22, }, // America/Lower_Princes
    {   69, 5805,    5260,  8, }, // America/Martinique
    {   69, 8220,     229,  6, }, // America/Merida
    {   69, 2949,   35910, 12, }, // America/Mexico_City
    {   69, 4391,    7149,  7, }, // America/Miquelon
    {   69, 1850,     235, 19, }, // America/Noronha
    {   69, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {   69, 8329,     274, 20, }, // America/North_Dakota/Center
    {   69, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {   69,  393,   35922, 14, }, // America/Nuuk
    {   69, 2356,   20665,  6, }, // America/Panama
    {   69, 1878,   26712,  9, }, // America/Sao_Paulo
    {   69, 6899,     321, 16, }, // America/Scoresbysund
    {   69, 8508,   35936, 16, }, // America/St_Barthelemy
    {   69, 2061,   35952, 12, }, // America/St_Johns
    {   69, 5863,   26749, 11, }, // America/St_Kitts
    {   69, 5880,   35964, 11, }, // America/St_Lucia
    {   69,  855,     379, 10, }, // America/St_Thomas
    {   69, 5913,   35975, 11, }, // America/St_Vincent
    {   69, 5756,   35986,  5, }, // America/Thule
    {   69, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {   69, 5511,   35991,  7, }, // Asia/Anadyr
    {   69, 5534,    4168,  7, }, // Asia/Bahrain
    {   69, 3653,   35998,  5, }, // Asia/Baku
    {   69, 4086,   36003,  6, }, // Asia/Beirut
    {   69, 4601,   36009,  6, }, // Asia/Chita
    {   69, 4511,   36015,  7, }, // Asia/Colombo
    {   69, 4540,    4201,  6, }, // Asia/Damascus
    {   69, 1259,   36022, 15, }, // Asia/Ho_Chi_Minh
    {   69, 1290,   33501,  9, }, // Asia/Jerusalem
    {   69, 3560,   36037,  6, }, // Asia/Kabul
    {   69, 4316,   36043, 10, }, // Asia/Kamchatka
    {   69, 1185,     448,  9, }, // Asia/Kathmandu
    {   69,  992,   36053,  8, }, // Asia/Kolkata
    {   69, 5547,   36061,  6, }, // Asia/Kuwait
    {   69, 1211,    4321,  5, }, // Asia/Macau
    {   69, 6920,   36067,  6, }, // Asia/Muscat
    {   69, 2554,   36073,  8, }, // Asia/Nicosia
    {   69, 6961,   36081, 10, }, // Asia/Phnom_Penh
    {   69, 4214,   36091, 10, }, // Asia/Pyongyang
    {   69, 3300,   36101,  9, }, // Asia/Singapore
    {   69, 4781,   36110,  8, }, // Asia/Tashkent
    {   69, 1159,   26933,  6, }, // Asia/Urumqi
    {   69, 6977,   36118,  9, }, // Asia/Vientiane
    {   69, 1235,     477,  6, }, // Asia/Yangon
    {   69, 3883,   36127, 14, }, // Asia/Yekaterinburg
    {   69, 3663,   36141,  6, }, // Atlantic/Azores
    {   69, 5671,   36147,  8, }, // Atlantic/Bermuda
    {   69, 6628,   36155,  8, }, // Atlantic/Canary
    {   69, 3727,   36163,  8, }, // Atlantic/Cape_Verde
    {   69, 1395,   36171,  5, }, // Atlantic/Faroe
    {   69, 2748,   36176,  9, }, // Atlantic/Reykjavik
    {   69, 7272,   36185, 13, }, // Atlantic/South_Georgia
    {   69, 6821,   36198, 12, }, // Atlantic/St_Helena
    {   69, 2300,    5911,  4, }, // Europe/Athens
    {   69, 6336,    8060,  7, }, // Europe/Belgrade
    {   69, 2167,   27022,  6, }, // Europe/Brussels
    {   69, 3928,   27029,  8, }, // Europe/Bucharest
    {   69, 6289,    4475, 10, }, // Europe/Copenhagen
    {   69, 2338,   36210,  6, }, // Europe/Dublin
    {   69, 6307,   36216,  9, }, // Europe/Gibraltar
    {   69, 8788,   36225,  9, }, // Europe/Guernsey
    {   69, 8804,   36234,  5, }, // Europe/Isle_of_Man
    {   69, 8823,   36239,  8, }, // Europe/Jersey
    {   69, 2527,    8148,  4, }, // Europe/Kyiv
    {   69, 3213,   21137,  6, }, // Europe/Lisbon
    {   69, 2501,   36247,  7, }, // Europe/London
    {   69, 6365,   36254, 11, }, // Europe/Luxembourg
    {   69, 6582,   36265,  9, }, // Europe/Mariehamn
    {   69, 3693,   36274,  5, }, // Europe/Minsk
    {   69, 3537,   36279,  6, }, // Europe/Moscow
    {   69, 4284,   33735,  5, }, // Europe/Paris
    {   69, 6275,   36285,  5, }, // Europe/Prague
    {   69, 6324,    6064,  4, }, // Europe/Rome
    {   69, 6202,    8255,  6, }, // Europe/Tirane
    {   69, 6567,    6096,  7, }, // Europe/Vatican
    {   69, 3190,   21244,  8, }, // Europe/Warsaw
    {   69, 6063,   36290,  5, }, // Indian/Cocos
    {   69, 5029,   36295,  7, }, // Indian/Comoro
    {   69, 6661,   36302,  9, }, // Indian/Kerguelen
    {   69, 7050,   36311,  8, }, // Indian/Maldives
    {   69, 4069,   36319,  5, }, // Indian/Mauritius
    {   69, 7211,   36324,  7, }, // Indian/Reunion
    {   69, 3157,     554,  5, }, // Pacific/Chuuk
    {   69, 2237,   36331,  9, }, // Pacific/Easter
    {   69, 3902,    4606,  5, }, // Pacific/Fiji
    {   69, 6678,   21277,  9, }, // Pacific/Galapagos
    {   69, 2723,    4611,  8, }, // Pacific/Honolulu
    {   69, 4051,   36340,  6, }, // Pacific/Marquesas
    {   69, 3096,     586,  7, }, // Pacific/Pohnpei
    {   70, 6788,   36346,  7, }, // Africa/Abidjan Bulgarian/Cyrillic/Bulgaria
    {   70, 6760,   27178,  4, }, // Africa/Accra
    {   70, 5059,   33846, 10, }, // Africa/Addis_Ababa
    {   70, 7420,     626,  5, }, // Africa/Algiers
    {   70,   14,   27197,  6, }, // Africa/Asmara
    {   70,   44,     637,  6, }, // Africa/Bamako
    {   70, 5257,   33856,  6, }, // Africa/Bangui
    {   70, 6746,   27208,  7, }, // Africa/Banjul
    {   70, 7435,     654,  5, }, // Africa/Bissau
    {   70, 4957,   36353,  9, }, // Africa/Blantyre
    {   70, 5287,   33874,  8, }, // Africa/Brazzaville
    {   70, 4922,     676,  9, }, // Africa/Bujumbura
    {   70, 2320,   36362,  5, }, // Africa/Cairo
    {   70, 4117,   33896, 10, }, // Africa/Casablanca
    {   70, 7449,     699,  5, }, // Africa/Ceuta
    {   70, 6773,     704,  7, }, // Africa/Conakry
    {   70, 6840,     711,  5, }, // Africa/Dakar
    {   70, 5130,   36367, 12, }, // Africa/Dar_es_Salaam
    {   70, 5043,   36379,  7, }, // Africa/Djibouti
    {   70, 5243,     734,  5, }, // Africa/Douala
    {   70, 5166,   36386,  6, }, // Africa/El_Aaiun
    {   70, 6853,   36392,  8, }, // Africa/Freetown
    {   70, 4906,     753,  8, }, // Africa/Gaborone
    {   70, 5015,   33913,  6, }, // Africa/Harare
    {   70, 4479,   36400, 11, }, // Africa/Johannesburg
    {   70, 4499,   27305,  5, }, // Africa/Juba
    {   70, 5151,     783,  7, }, // Africa/Kampala
    {   70, 4524,     790,  6, }, // Africa/Khartoum
    {   70, 4973,     796,  6, }, // Africa/Kigali
    {   70, 5306,     802,  7, }, // Africa/Kinshasa
    {   70, 4744,     809,  5, }, // Africa/Lagos
    {   70, 5336,     814,  8, }, // Africa/Libreville
    {   70, 6869,     822,  4, }, // Africa/Lome
    {   70, 5211,     826,  6, }, // Africa/Luanda
    {   70, 4939,     832,  9, }, // Africa/Lubumbashi
    {   70, 5001,     841,  6, }, // Africa/Lusaka
    {   70, 5322,     847,  6, }, // Africa/Malabo
    {   70, 4987,   33945,  6, }, // Africa/Maputo
    {   70, 5197,     859,  6, }, // Africa/Maseru
    {   70, 5182,     865,  7, }, // Africa/Mbabane
    {   70, 5113,   36411,  8, }, // Africa/Mogadishu
    {   70, 7034,   36419,  8, }, // Africa/Monrovia
    {   70, 3853,   36427,  7, }, // Africa/Nairobi
    {   70, 5271,   27386,  8, }, // Africa/Ndjamena
    {   70, 5354,   36434,  6, }, // Africa/Niamey
    {   70, 6803,     908,  7, }, // Africa/Nouakchott
    {   70, 6727,     915,  8, }, // Africa/Ouagadougou
    {   70, 5225,   33974, 10, }, // Africa/Porto-Novo
    {   70, 4435,   33984,  8, }, // Africa/Sao_Tome
    {   70, 2866,     941,  7, }, // Africa/Tripoli
    {   70, 6554,     948,  5, }, // Africa/Tunis
    {   70, 4152,     953,  7, }, // Africa/Windhoek
    {   70,  132,     960,  4, }, // America/Adak
    {   70, 3346,   36440,  8, }, // America/Anchorage
    {   70, 5607,   36448,  7, }, // America/Anguilla
    {   70, 5624,     978,  7, }, // America/Antigua
    {   70, 4554,   36455,  9, }, // America/Araguaina
    {   70,  166,   36464, 12, }, // America/Argentina/Buenos_Aires
    {   70,   91,    1006,  9, }, // America/Argentina/Catamarca
    {   70,  270,   34035,  7, }, // America/Argentina/Cordoba
    {   70,  441,   36476,  5, }, // America/Argentina/Jujuy
    {   70, 7462,   34047,  8, }, // America/Argentina/La_Rioja
    {   70,  565,    1035,  7, }, // America/Argentina/Mendoza
    {   70, 7489,   36481, 11, }, // America/Argentina/Rio_Gallegos
    {   70, 7520,    1053,  5, }, // America/Argentina/Salta
    {   70, 7544,   34073,  8, }, // America/Argentina/San_Juan
    {   70, 5580,   36492,  8, }, // America/Argentina/San_Luis
    {   70, 7571,    1074,  7, }, // America/Argentina/Tucuman
    {   70, 7597,   27526,  5, }, // America/Argentina/Ushuaia
    {   70, 5640,    1087,  5, }, // America/Aruba
    {   70, 4252,   34095,  8, }, // America/Asuncion
    {   70,  237,   34103,  8, }, // America/Atikokan
    {   70, 3679,   36500,  4, }, // America/Bahia
    {   70, 7623,   36504, 16, }, // America/Bahia_Banderas
    {   70, 5654,    1131,  8, }, // America/Barbados
    {   70, 7646,   34130,  5, }, // America/Belem
    {   70, 5407,    1144,  5, }, // America/Belize
    {   70, 7660,   36520, 11, }, // America/Blanc-Sablon
    {   70, 7681,   34154,  9, }, // America/Boa_Vista
    {   70, 4361,    1170,  6, }, // America/Bogota
    {   70, 7699,   36531,  5, }, // America/Boise
    {   70, 7713,   36536, 13, }, // America/Cambridge_Bay
    {   70, 7735,   34179, 12, }, // America/Campo_Grande
    {   70, 3868,    1205,  6, }, // America/Cancun
    {   70, 4694,    1211,  7, }, // America/Caracas
    {   70, 4345,   36549,  5, }, // America/Cayenne
    {   70, 5496,   36554, 17, }, // America/Cayman
    {   70, 2260,    1247,  6, }, // America/Chicago
    {   70, 7756,    1253,  6, }, // America/Chihuahua
    {   70, 7774,   36571, 13, }, // America/Ciudad_Juarez
    {   70, 5422,   36584, 10, }, // America/Costa_Rica
    {   70, 7796,   36594,  7, }, // America/Creston
    {   70, 3791,   36601,  5, }, // America/Cuiaba
    {   70, 5723,   36606,  7, }, // America/Curacao
    {   70, 7812,   36613, 12, }, // America/Danmarkshavn
    {   70, 7833,   36625,  6, }, // America/Dawson
    {   70, 7848,   36631, 12, }, // America/Dawson_Creek
    {   70,  805,   36643,  6, }, // America/Denver
    {   70, 3465,   36649,  7, }, // America/Detroit
    {   70, 5739,    1332,  8, }, // America/Dominica
    {   70,  893,   36656,  8, }, // America/Edmonton
    {   70, 7869,   36664,  8, }, // America/Eirunepe
    {   70, 5441,    1356,  8, }, // America/El_Salvador
    {   70, 7886,   36672, 11, }, // America/Fort_Nelson
    {   70, 7906,    1375,  9, }, // America/Fortaleza
    {   70, 7924,   36683,  9, }, // America/Glace_Bay
    {   70, 6881,   36692,  7, }, // America/Goose_Bay
    {   70, 4612,   36699, 10, }, // America/Grand_Turk
    {   70, 5770,    1410,  7, }, // America/Grenada
    {   70, 5786,    1417,  9, }, // America/Guadeloupe
    {   70, 3760,    1426,  9, }, // America/Guatemala
    {   70, 4841,   36709,  7, }, // America/Guayaquil
    {   70, 6932,   27854,  5, }, // America/Guyana
    {   70, 1939,   34310,  8, }, // America/Halifax
    {   70, 2281,   34318,  6, }, // America/Havana
    {   70, 7942,   36716,  9, }, // America/Hermosillo
    {   70,  348,    1471, 12, }, // America/Indiana/Indianapolis
    {   70,  481,    1483,  4, }, // America/Indiana/Knox
    {   70, 7961,    1496,  7, }, // America/Indiana/Marengo
    {   70, 7985,   36725, 10, }, // America/Indiana/Petersburg
    {   70, 8012,   34385,  8, }, // America/Indiana/Tell_City
    {   70, 8038,   36735,  4, }, // America/Indiana/Vevay
    {   70, 8060,   36739,  7, }, // America/Indiana/Vincennes
    {   70, 8086,    1563,  7, }, // America/Indiana/Winamac
    {   70, 8110,    1570,  6, }, // America/Inuvik
    {   70,  660,   34453,  8, }, // America/Iqaluit
    {   70, 2799,   28011,  6, }, // America/Jamaica
    {   70, 5380,   36746,  5, }, // America/Juneau
    {   70,  521,    1594,  7, }, // America/Kentucky/Louisville
    {   70, 8125,   34472,  9, }, // America/Kentucky/Monticello
    {   70, 5704,   36751, 10, }, // America/Kralendijk
    {   70, 4376,   36761,  6, }, // America/La_Paz
    {   70, 7169,    1638,  4, }, // America/Lima
    {   70, 3239,   36767, 12, }, // America/Los_Angeles
    {   70, 5932,   36779, 20, }, // America/Lower_Princes
    {   70, 8153,   36799,  6, }, // America/Maceio
    {   70, 8168,    1677,  7, }, // America/Managua
    {   70, 1908,    1684,  6, }, // America/Manaus
    {   70, 5897,    1690,  6, }, // America/Marigot
    {   70, 5805,    1696,  9, }, // America/Martinique
    {   70, 8184,    1705,  9, }, // America/Matamoros
    {   70, 2917,    1714,  8, }, // America/Mazatlan
    {   70, 8202,    1722,  8, }, // America/Menominee
    {   70, 8220,    1730,  6, }, // America/Merida
    {   70, 8235,    1736, 10, }, // America/Metlakatla
    {   70, 2949,   34557, 12, }, // America/Mexico_City
    {   70, 4391,    1752,  7, }, // America/Miquelon
    {   70, 8254,    1759,  7, }, // America/Moncton
    {   70, 8270,   36805,  8, }, // America/Monterrey
    {   70, 4098,    1775, 10, }, // America/Montevideo
    {   70, 5824,   34577,  9, }, // America/Montserrat
    {   70, 5481,   34586,  5, }, // America/Nassau
    {   70, 2379,   36813,  7, }, // America/New_York
    {   70, 8288,   36820,  4, }, // America/Nome
    {   70, 1850,   36824,  6, }, // America/Noronha
    {   70, 8301,   36830,  4, }, // America/North_Dakota/Beulah
    {   70, 8329,   36834,  6, }, // America/North_Dakota/Center
    {   70, 8357,   36840,  9, }, // America/North_Dakota/New_Salem
    {   70,  393,    1890,  4, }, // America/Nuuk
    {   70, 8388,    1894,  7, }, // America/Ojinaga
    {   70, 2356,    1901,  6, }, // America/Panama
    {   70, 6173,    1907, 10, }, // America/Paramaribo
    {   70, 2973,    1917,  6, }, // America/Phoenix
    {   70, 3945,    1923, 12, }, // America/Port-au-Prince
    {   70, 5954,   36849, 13, }, // America/Port_of_Spain
    {   70, 8404,   36862, 11, }, // America/Porto_Velho
    {   70, 5843,   36873, 11, }, // America/Puerto_Rico
    {   70, 4030,   34711, 12, }, // America/Punta_Arenas
    {   70, 8424,   34723, 12, }, // America/Rankin_Inlet
    {   70, 8445,   34735,  6, }, // America/Recife
    {   70, 1995,   36884,  8, }, // America/Regina
    {   70, 8460,   36892,  7, }, // America/Resolute
    {   70,  695,   34754, 10, }, // America/Rio_Branco
    {   70, 8477,   34764,  8, }, // America/Santarem
    {   70, 2201,   36899,  7, }, // America/Santiago
    {   70, 6111,   34781, 13, }, // America/Santo_Domingo
    {   70, 1878,   36906,  9, }, // America/Sao_Paulo
    {   70, 6899,   36915, 10, }, // America/Scoresbysund
    {   70, 8494,    2074,  5, }, // America/Sitka
    {   70, 8508,   36925, 13, }, // America/St_Barthelemy
    {   70, 2061,   36938, 11, }, // America/St_Johns
    {   70, 5863,   36949, 10, }, // America/St_Kitts
    {   70, 5880,   36959, 11, }, // America/St_Lucia
    {   70,  855,   36970, 11, }, // America/St_Thomas
    {   70, 5913,   36981, 13, }, // America/St_Vincent
    {   70, 8530,   36994, 12, }, // America/Swift_Current
    {   70, 5461,    2156, 11, }, // America/Tegucigalpa
    {   70, 5756,    2167,  4, }, // America/Thule
    {   70,  313,    2171,  7, }, // America/Tijuana
    {   70,  608,    2178,  7, }, // America/Toronto
    {   70, 5688,    2185,  7, }, // America/Tortola
    {   70, 2093,   37006,  8, }, // America/Vancouver
    {   70, 2144,   28567,  8, }, // America/Whitehorse
    {   70,  734,   37014,  7, }, // America/Winnipeg
    {   70, 8552,   28582,  6, }, // America/Yakutat
    {   70, 6001,   37021,  5, }, // Antarctica/Casey
    {   70, 6094,   37026,  6, }, // Antarctica/Davis
    {   70, 6133,   37032, 12, }, // Antarctica/DumontDUrville
    {   70, 8568,    2249,  8, }, // Antarctica/Macquarie
    {   70, 7066,   37044,  6, }, // Antarctica/Mawson
    {   70, 7113,   37050,  8, }, // Antarctica/McMurdo
    {   70, 8589,   37058,  6, }, // Antarctica/Palmer
    {   70, 7226,    2278,  6, }, // Antarctica/Rothera
    {   70, 7295,   34953,  4, }, // Antarctica/Syowa
    {   70, 8607,    2289,  4, }, // Antarctica/Troll
    {   70, 7374,   34957,  6, }, // Antarctica/Vostok
    {   70, 1429,   37064, 10, }, // Arctic/Longyearbyen
    {   70, 5570,    2310,  4, }, // Asia/Aden
    {   70, 5395,   34974,  6, }, // Asia/Almaty
    {   70, 3968,   28663,  4, }, // Asia/Amman
    {   70, 5511,   34980,  6, }, // Asia/Anadyr
    {   70, 5523,    2331,  5, }, // Asia/Aqtau
    {   70, 5368,    2336,  6, }, // Asia/Aqtobe
    {   70,  964,    2342,  7, }, // Asia/Ashgabat
    {   70, 8624,   37074,  6, }, // Asia/Atyrau
    {   70, 3607,    2355,  6, }, // Asia/Baghdad
    {   70, 5534,   37080,  7, }, // Asia/Bahrain
    {   70, 3653,    2368,  4, }, // Asia/Baku
    {   70, 4466,   34999,  6, }, // Asia/Bangkok
    {   70, 3571,    2379,  7, }, // Asia/Barnaul
    {   70, 4086,   28695,  6, }, // Asia/Beirut
    {   70, 3778,    2392,  6, }, // Asia/Bishkek
    {   70, 5989,   28707,  6, }, // Asia/Brunei
    {   70, 4601,    2404,  4, }, // Asia/Chita
    {   70, 4511,    2408,  7, }, // Asia/Colombo
    {   70, 4540,    2415,  6, }, // Asia/Damascus
    {   70, 1093,     711,  4, }, // Asia/Dhaka
    {   70, 6192,    2426,  4, }, // Asia/Dili
    {   70, 3596,   28728,  5, }, // Asia/Dubai
    {   70, 6159,    2435,  7, }, // Asia/Dushanbe
    {   70, 8636,    2442,  9, }, // Asia/Famagusta
    {   70, 8651,    2451,  4, }, // Asia/Gaza
    {   70, 4795,   35018,  6, }, // Asia/Hebron
    {   70, 1259,    2461,  7, }, // Asia/Ho_Chi_Minh
    {   70, 2704,   37087,  7, }, // Asia/Hong_Kong
    {   70, 4771,    2475,  4, }, // Asia/Hovd
    {   70, 4184,    2479,  7, }, // Asia/Irkutsk
    {   70, 7006,   28770,  8, }, // Asia/Jakarta
    {   70, 6992,   28778,  8, }, // Asia/Jayapura
    {   70, 1290,   37094,  9, }, // Asia/Jerusalem
    {   70, 3560,    2510,  5, }, // Asia/Kabul
    {   70, 4316,    2529,  8, }, // Asia/Kamchatka
    {   70, 4239,    2540,  6, }, // Asia/Karachi
    {   70, 1185,    2546,  8, }, // Asia/Kathmandu
    {   70, 8661,   35065,  7, }, // Asia/Khandyga
    {   70,  992,   37103,  7, }, // Asia/Kolkata
    {   70, 4197,   37110, 10, }, // Asia/Krasnoyarsk
    {   70, 4859,   35090, 12, }, // Asia/Kuala_Lumpur
    {   70, 5976,    2592,  5, }, // Asia/Kuching
    {   70, 5547,   28825,  6, }, // Asia/Kuwait
    {   70, 1211,    2604,  5, }, // Asia/Macau
    {   70, 4017,    2609,  7, }, // Asia/Magadan
    {   70, 1349,    2616,  7, }, // Asia/Makassar
    {   70, 7182,    2623,  6, }, // Asia/Manila
    {   70, 6920,   35113,  6, }, // Asia/Muscat
    {   70, 2554,   37120,  7, }, // Asia/Nicosia
    {   70, 8675,    2642, 11, }, // Asia/Novokuznetsk
    {   70, 4135,    2653, 11, }, // Asia/Novosibirsk
    {   70, 4229,    2664,  4, }, // Asia/Omsk
    {   70, 7145,   37127,  4, }, // Asia/Oral
    {   70, 6961,   35142,  8, }, // Asia/Phnom_Penh
    {   70, 8693,    2681,  9, }, // Asia/Pontianak
    {   70, 4214,   37131,  6, }, // Asia/Pyongyang
    {   70, 5559,    2697,  5, }, // Asia/Qatar
    {   70, 8708,   37137,  8, }, // Asia/Qostanay
    {   70, 4269,   37145,  9, }, // Asia/Qyzylorda
    {   70, 3584,   37154,  4, }, // Asia/Riyadh
    {   70, 4408,    2732,  7, }, // Asia/Sakhalin
    {   70, 7245,    2739,  9, }, // Asia/Samarkand
    {   70, 3279,    2748,  4, }, // Asia/Seoul
    {   70, 1053,   28933,  6, }, // Asia/Shanghai
    {   70, 3300,    2758,  8, }, // Asia/Singapore
    {   70, 4297,   37158, 13, }, // Asia/Srednekolymsk
    {   70, 3263,   37171,  5, }, // Asia/Taipei
    {   70, 4781,    2785,  7, }, // Asia/Tashkent
    {   70, 3915,   35206,  7, }, // Asia/Tbilisi
    {   70, 2772,   35213,  7, }, // Asia/Tehran
    {   70, 1317,    2803,  7, }, // Asia/Thimphu
    {   70, 2821,    2810,  5, }, // Asia/Tokyo
    {   70, 4572,    2815,  5, }, // Asia/Tomsk
    {   70, 1021,   35225, 10, }, // Asia/Ulaanbaatar
    {   70, 1159,    2830,  6, }, // Asia/Urumqi
    {   70, 8722,    2836,  8, }, // Asia/Ust-Nera
    {   70, 6977,   37176,  7, }, // Asia/Vientiane
    {   70, 4710,    2851, 11, }, // Asia/Vladivostok
    {   70, 4828,   37183,  6, }, // Asia/Yakutsk
    {   70, 1235,   29043,  6, }, // Asia/Yangon
    {   70, 3883,    2875, 12, }, // Asia/Yekaterinburg
    {   70, 3747,    2887,  6, }, // Asia/Yerevan
    {   70, 3663,   37189, 15, }, // Atlantic/Azores
    {   70, 5671,   37204, 17, }, // Atlantic/Bermuda
    {   70, 6628,   37221, 16, }, // Atlantic/Canary
    {   70, 3727,   37237, 10, }, // Atlantic/Cape_Verde
    {   70, 1395,   37247, 17, }, // Atlantic/Faroe
    {   70, 8736,   37264,  7, }, // Atlantic/Madeira
    {   70, 2748,   37271,  8, }, // Atlantic/Reykjavik
    {   70, 7272,   37279, 13, }, // Atlantic/South_Georgia
    {   70, 6821,   37292, 11, }, // Atlantic/St_Helena
    {   70, 6644,    3029,  6, }, // Atlantic/Stanley
    {   70, 1670,    3035,  8, }, // Australia/Adelaide
    {   70, 1635,   37303,  8, }, // Australia/Brisbane
    {   70, 1799,   37311, 11, }, // Australia/Broken_Hill
    {   70, 1583,    3061,  6, }, // Australia/Darwin
    {   70, 3637,   29240,  4, }, // Australia/Eucla
    {   70, 1516,    3072,  6, }, // Australia/Hobart
    {   70, 8753,    3078,  8, }, // Australia/Lindeman
    {   70, 1547,   37322,  8, }, // Australia/Lord_Howe
    {   70, 1727,   37330,  7, }, // Australia/Melbourne
    {   70, 1762,   37337,  4, }, // Australia/Perth
    {   70, 1463,   37341,  5, }, // Australia/Sydney
    {   70, 6441,    3111,  9, }, // Europe/Amsterdam
    {   70, 6216,   29275,  6, }, // Europe/Andorra
    {   70, 3620,    3127,  8, }, // Europe/Astrakhan
    {   70, 2300,   35401,  5, }, // Europe/Athens
    {   70, 6336,    3139,  7, }, // Europe/Belgrade
    {   70, 4757,    3146,  6, }, // Europe/Berlin
    {   70, 6488,    3152, 10, }, // Europe/Bratislava
    {   70, 2167,   37346,  7, }, // Europe/Brussels
    {   70, 3928,   37353,  7, }, // Europe/Bucharest
    {   70, 3806,   37360,  8, }, // Europe/Budapest
    {   70, 8772,   37368,  8, }, // Europe/Busingen
    {   70, 2583,    3209,  7, }, // Europe/Chisinau
    {   70, 6289,   35452, 10, }, // Europe/Copenhagen
    {   70, 2338,   37376,  6, }, // Europe/Dublin
    {   70, 6307,    3232,  9, }, // Europe/Gibraltar
    {   70, 8788,   37382,  6, }, // Europe/Guernsey
    {   70, 6612,   37388,  8, }, // Europe/Helsinki
    {   70, 8804,   37396, 10, }, // Europe/Isle_of_Man
    {   70, 1130,   35484,  8, }, // Europe/Istanbul
    {   70, 8823,   37406,  6, }, // Europe/Jersey
    {   70, 3979,    3276, 11, }, // Europe/Kaliningrad
    {   70, 8929,    3287,  5, }, // Europe/Kirov
    {   70, 2527,    3292,  4, }, // Europe/Kyiv
    {   70, 3213,   35508,  7, }, // Europe/Lisbon
    {   70, 6506,   29432,  7, }, // Europe/Ljubljana
    {   70, 2501,    3313,  6, }, // Europe/London
    {   70, 6365,   29445, 10, }, // Europe/Luxembourg
    {   70, 4892,    3330,  6, }, // Europe/Madrid
    {   70, 6397,    3336,  5, }, // Europe/Malta
    {   70, 6582,    3341,  9, }, // Europe/Mariehamn
    {   70, 3693,    3350,  5, }, // Europe/Minsk
    {   70, 6410,    3355,  6, }, // Europe/Monaco
    {   70, 3537,    3361,  6, }, // Europe/Moscow
    {   70, 6458,    3367,  4, }, // Europe/Oslo
    {   70, 4284,    3371,  5, }, // Europe/Paris
    {   70, 6424,    3376,  9, }, // Europe/Podgorica
    {   70, 6275,    3385,  5, }, // Europe/Prague
    {   70, 8837,    3390,  4, }, // Europe/Riga
    {   70, 6324,    3394,  3, }, // Europe/Rome
    {   70, 4331,    3397,  6, }, // Europe/Samara
    {   70, 6470,   35545, 10, }, // Europe/San_Marino
    {   70, 6245,    3413,  7, }, // Europe/Sarajevo
    {   70, 4451,    3420,  7, }, // Europe/Saratov
    {   70, 8849,    3427, 10, }, // Europe/Simferopol
    {   70, 6383,   37412,  6, }, // Europe/Skopje
    {   70, 6599,   37418,  5, }, // Europe/Sofia
    {   70, 6523,   35585,  8, }, // Europe/Stockholm
    {   70, 8867,   35593,  5, }, // Europe/Tallinn
    {   70, 6202,    3461,  6, }, // Europe/Tirane
    {   70, 8882,   37423,  8, }, // Europe/Ulyanovsk
    {   70, 6352,    3476,  5, }, // Europe/Vaduz
    {   70, 6567,    3481,  7, }, // Europe/Vatican
    {   70, 6231,   37431,  5, }, // Europe/Vienna
    {   70, 8899,   37436,  6, }, // Europe/Vilnius
    {   70, 4727,    3499,  9, }, // Europe/Volgograd
    {   70, 3190,    3508,  7, }, // Europe/Warsaw
    {   70, 6261,    3515,  6, }, // Europe/Zagreb
    {   70, 6540,   37442,  5, }, // Europe/Zurich
    {   70, 5078,   35625, 12, }, // Indian/Antananarivo
    {   70, 6947,    3539,  5, }, // Indian/Chagos
    {   70, 6046,   37447,  9, }, // Indian/Christmas
    {   70, 6063,   37456, 16, }, // Indian/Cocos
    {   70, 5029,   37472, 16, }, // Indian/Comoro
    {   70, 6661,    3578,  8, }, // Indian/Kerguelen
    {   70, 7260,    3586,  3, }, // Indian/Mahe
    {   70, 7050,   35652,  7, }, // Indian/Maldives
    {   70, 4069,   37488,  8, }, // Indian/Mauritius
    {   70, 5098,   37496,  5, }, // Indian/Mayotte
    {   70, 7211,   37501,  7, }, // Indian/Reunion
    {   70, 4422,   37508,  4, }, // Pacific/Apia
    {   70,  932,   35686,  6, }, // Pacific/Auckland
    {   70, 3706,    3626,  8, }, // Pacific/Bougainville
    {   70, 3015,   37512,  5, }, // Pacific/Chatham
    {   70, 3157,   37517,  4, }, // Pacific/Chuuk
    {   70, 2237,   37521, 18, }, // Pacific/Easter
    {   70, 7360,    3654,  5, }, // Pacific/Efate
    {   70, 7327,    3659,  7, }, // Pacific/Fakaofo
    {   70, 3902,   37539,  5, }, // Pacific/Fiji
    {   70, 7343,    3670,  8, }, // Pacific/Funafuti
    {   70, 6678,    3678,  9, }, // Pacific/Galapagos
    {   70, 6696,   37544,  6, }, // Pacific/Gambier
    {   70, 3822,    3713, 11, }, // Pacific/Guadalcanal
    {   70, 6018,    3724,  4, }, // Pacific/Guam
    {   70, 2723,   35730,  8, }, // Pacific/Honolulu
    {   70, 3049,   37550,  9, }, // Pacific/Kanton
    {   70, 3998,    3734, 10, }, // Pacific/Kiritimati
    {   70, 7019,   37559,  6, }, // Pacific/Kosrae
    {   70, 2842,   37565,  9, }, // Pacific/Kwajalein
    {   70, 4877,   37574,  7, }, // Pacific/Majuro
    {   70, 4051,   37581, 16, }, // Pacific/Marquesas
    {   70, 8914,   37597,  6, }, // Pacific/Midway
    {   70, 7084,    3798,  5, }, // Pacific/Nauru
    {   70, 7132,    3803,  4, }, // Pacific/Niue
    {   70, 4168,    3807,  7, }, // Pacific/Norfolk
    {   70, 7098,    3814,  5, }, // Pacific/Noumea
    {   70, 3126,   35774,  9, }, // Pacific/Pago_Pago
    {   70, 7155,    3828,  5, }, // Pacific/Palau
    {   70, 7194,    3833,  7, }, // Pacific/Pitcairn
    {   70, 3096,   37603,  6, }, // Pacific/Pohnpei
    {   70, 4807,   37609, 11, }, // Pacific/Port_Moresby
    {   70, 6076,    3857,  9, }, // Pacific/Rarotonga
    {   70, 6031,   29935,  6, }, // Pacific/Saipan
    {   70, 7312,    3872,  5, }, // Pacific/Tahiti
    {   70, 6712,   37620,  6, }, // Pacific/Tarawa
    {   70, 4583,    3883,  9, }, // Pacific/Tongatapu
    {   70, 7392,   37626,  4, }, // Pacific/Wake
    {   70, 7405,   37630,  5, }, // Pacific/Wallis
    {   71, 6788,   37635,  8, }, // Africa/Abidjan Burmese/Myanmar/Myanmar
    {   71, 6760,   37643,  6, }, // Africa/Accra
    {   71, 5059,   37649, 12, }, // Africa/Addis_Ababa
    {   71, 7420,   37661, 15, }, // Africa/Algiers
    {   71,   14,   37676, 10, }, // Africa/Asmara
    {   71,   44,   37686,  7, }, // Africa/Bamako
    {   71, 5257,   37693,  7, }, // Africa/Bangui
    {   71, 6746,   37700,  9, }, // Africa/Banjul
    {   71, 7435,   37709,  7, }, // Africa/Bissau
    {   71, 4957,   37716, 14, }, // Africa/Blantyre
    {   71, 5287,   37730, 11, }, // Africa/Brazzaville
    {   71, 4922,   37741, 12, }, // Africa/Bujumbura
    {   71, 2320,   37753,  8, }, // Africa/Cairo
    {   71, 4117,   37761, 10, }, // Africa/Casablanca
    {   71, 7449,   37771,  7, }, // Africa/Ceuta
    {   71, 6773,   37778,  9, }, // Africa/Conakry
    {   71, 6840,   37787,  6, }, // Africa/Dakar
    {   71, 5130,   37793, 12, }, // Africa/Dar_es_Salaam
    {   71, 5043,   37805,  7, }, // Africa/Djibouti
    {   71, 5243,   37812,  9, }, // Africa/Douala
    {   71, 5166,   37821, 12, }, // Africa/El_Aaiun
    {   71, 6853,   37833, 12, }, // Africa/Freetown
    {   71, 4906,   37845, 11, }, // Africa/Gaborone
    {   71, 5015,   37856,  9, }, // Africa/Harare
    {   71, 4479,   37865, 15, }, // Africa/Johannesburg
    {   71, 4499,   37880,  5, }, // Africa/Juba
    {   71, 5151,   37885,  7, }, // Africa/Kampala
    {   71, 4524,   37892,  8, }, // Africa/Khartoum
    {   71, 4973,   37900,  7, }, // Africa/Kigali
    {   71, 5306,   37907,  8, }, // Africa/Kinshasa
    {   71, 4744,   37915,  7, }, // Africa/Lagos
    {   71, 5336,   37922, 10, }, // Africa/Libreville
    {   71, 6869,   37932,  5, }, // Africa/Lome
    {   71, 5211,   37937,  7, }, // Africa/Luanda
    {   71, 4939,   37944, 12, }, // Africa/Lubumbashi
    {   71, 5001,   37956,  6, }, // Africa/Lusaka
    {   71, 5322,   37962,  7, }, // Africa/Malabo
    {   71, 4987,   37969,  8, }, // Africa/Maputo
    {   71, 5197,   37977,  7, }, // Africa/Maseru
    {   71, 5182,   37984,  8, }, // Africa/Mbabane
    {   71, 5113,   37992, 13, }, // Africa/Mogadishu
    {   71, 7034,   38005, 14, }, // Africa/Monrovia
    {   71, 3853,   38019, 10, }, // Africa/Nairobi
    {   71, 5271,   38029, 10, }, // Africa/Ndjamena
    {   71, 5354,   38039,  6, }, // Africa/Niamey
    {   71, 6803,   38045, 15, }, // Africa/Nouakchott
    {   71, 6727,   38060,  9, }, // Africa/Ouagadougou
    {   71, 5225,   38069, 13, }, // Africa/Porto-Novo
    {   71, 4435,   38082, 10, }, // Africa/Sao_Tome
    {   71, 2866,   38092,  8, }, // Africa/Tripoli
    {   71, 6554,   38100,  6, }, // Africa/Tunis
    {   71, 4152,   38106,  9, }, // Africa/Windhoek
    {   71,  132,   38115,  5, }, // America/Adak
    {   71, 3346,   38120, 11, }, // America/Anchorage
    {   71, 5607,   38131,  7, }, // America/Anguilla
    {   71, 5624,   38138,  8, }, // America/Antigua
    {   71, 4554,   38146, 11, }, // America/Araguaina
    {   71,  166,   38157, 13, }, // America/Argentina/Buenos_Aires
    {   71,   91,   38170,  9, }, // America/Argentina/Catamarca
    {   71,  270,   38179,  8, }, // America/Argentina/Cordoba
    {   71,  441,   38187,  6, }, // America/Argentina/Jujuy
    {   71, 7462,   38193, 10, }, // America/Argentina/La_Rioja
    {   71,  565,   38203,  8, }, // America/Argentina/Mendoza
    {   71, 7489,   38211, 15, }, // America/Argentina/Rio_Gallegos
    {   71, 7520,   38226,  6, }, // America/Argentina/Salta
    {   71, 7544,   38232,  8, }, // America/Argentina/San_Juan
    {   71, 5580,   38240, 10, }, // America/Argentina/San_Luis
    {   71, 7571,   38250,  7, }, // America/Argentina/Tucuman
    {   71, 7597,   38257,  8, }, // America/Argentina/Ushuaia
    {   71, 5640,   38265,  7, }, // America/Aruba
    {   71, 4252,   38272, 11, }, // America/Asuncion
    {   71,  237,   38283, 10, }, // America/Atikokan
    {   71, 3679,   38293,  6, }, // America/Bahia
    {   71, 7623,   38299, 16, }, // America/Bahia_Banderas
    {   71, 5654,   38315, 11, }, // America/Barbados
    {   71, 7646,   38326,  7, }, // America/Belem
    {   71, 5407,   38333,  5, }, // America/Belize
    {   71, 7660,   38338, 15, }, // America/Blanc-Sablon
    {   71, 7681,   38353, 11, }, // America/Boa_Vista
    {   71, 4361,   38364,  8, }, // America/Bogota
    {   71, 7699,   38372,  8, }, // America/Boise
    {   71, 7713,   38380, 16, }, // America/Cambridge_Bay
    {   71, 7735,   38396, 14, }, // America/Campo_Grande
    {   71, 3868,   38410,  6, }, // America/Cancun
    {   71, 4694,   38416,  8, }, // America/Caracas
    {   71, 4345,   38424,  7, }, // America/Cayenne
    {   71, 5496,   38431,  5, }, // America/Cayman
    {   71, 2260,   38436,  8, }, // America/Chicago
    {   71, 7756,   38444, 11, }, // America/Chihuahua
    {   71, 7774,   38455, 18, }, // America/Ciudad_Juarez
    {   71, 5422,   38473, 11, }, // America/Costa_Rica
    {   71, 7796,   38484,  9, }, // America/Creston
    {   71, 3791,   38493,  7, }, // America/Cuiaba
    {   71, 5723,   38500, 10, }, // America/Curacao
    {   71, 7812,   38510, 13, }, // America/Danmarkshavn
    {   71, 7833,   38523,  7, }, // America/Dawson
    {   71, 7848,   38530, 13, }, // America/Dawson_Creek
    {   71,  805,   38543,  7, }, // America/Denver
    {   71, 3465,   38550, 10, }, // America/Detroit
    {   71, 5739,   38560,  9, }, // America/Dominica
    {   71,  893,   38569, 12, }, // America/Edmonton
    {   71, 7869,   38581,  8, }, // America/Eirunepe
    {   71, 5441,   38589, 12, }, // America/El_Salvador
    {   71, 7886,   38601, 15, }, // America/Fort_Nelson
    {   71, 7906,   38616, 12, }, // America/Fortaleza
    {   71, 7924,   38628,  8, }, // America/Glace_Bay
    {   71, 6881,   38636,  8, }, // America/Goose_Bay
    {   71, 4612,   38644, 10, }, // America/Grand_Turk
    {   71, 5770,   38654,  7, }, // America/Grenada
    {   71, 5786,   38661,  9, }, // America/Guadeloupe
    {   71, 3760,   38670,  9, }, // America/Guatemala
    {   71, 4841,   38679, 11, }, // America/Guayaquil
    {   71, 6932,   38690,  9, }, // America/Guyana
    {   71, 1939,   38699,  8, }, // America/Halifax
    {   71, 2281,   38707,  6, }, // America/Havana
    {   71, 7942,   38713, 12, }, // America/Hermosillo
    {   71,  348,   38725, 19, }, // America/Indiana/Indianapolis
    {   71,  481,   38744, 22, }, // America/Indiana/Knox
    {   71, 7961,   38766, 21, }, // America/Indiana/Marengo
    {   71, 7985,   38787, 24, }, // America/Indiana/Petersburg
    {   71, 8012,   38811, 23, }, // America/Indiana/Tell_City
    {   71, 8038,   38834, 17, }, // America/Indiana/Vevay
    {   71, 8060,   38851, 22, }, // America/Indiana/Vincennes
    {   71, 8086,   38873, 22, }, // America/Indiana/Winamac
    {   71, 8110,   38895,  8, }, // America/Inuvik
    {   71,  660,   38903, 10, }, // America/Iqaluit
    {   71, 2799,   38913,  6, }, // America/Jamaica
    {   71, 5380,   38919,  9, }, // America/Juneau
    {   71,  521,   38928,  9, }, // America/Kentucky/Louisville
    {   71, 8125,   38937, 23, }, // America/Kentucky/Monticello
    {   71, 5704,   38960, 14, }, // America/Kralendijk
    {   71, 4376,   38974,  6, }, // America/La_Paz
    {   71, 7169,   38980,  4, }, // America/Lima
    {   71, 3239,   38984, 18, }, // America/Los_Angeles
    {   71, 5932,   39002, 18, }, // America/Lower_Princes
    {   71, 8153,   39020,  7, }, // America/Maceio
    {   71, 8168,   39027,  7, }, // America/Managua
    {   71, 1908,   39034,  9, }, // America/Manaus
    {   71, 5897,   39043,  8, }, // America/Marigot
    {   71, 5805,   39051,  8, }, // America/Martinique
    {   71, 8184,   39059, 12, }, // America/Matamoros
    {   71, 2917,   39071,  8, }, // America/Mazatlan
    {   71, 8202,   39079, 10, }, // America/Menominee
    {   71, 8220,   39089,  6, }, // America/Merida
    {   71, 8235,   39095, 12, }, // America/Metlakatla
    {   71, 2949,   39107, 16, }, // America/Mexico_City
    {   71, 4391,   39123,  9, }, // America/Miquelon
    {   71, 8254,   39132, 10, }, // America/Moncton
    {   71, 8270,   39142,  9, }, // America/Monterrey
    {   71, 4098,   39151, 13, }, // America/Montevideo
    {   71, 5824,   39164, 15, }, // America/Montserrat
    {   71, 5481,   39179,  7, }, // America/Nassau
    {   71, 2379,   39186,  9, }, // America/New_York
    {   71, 8288,   39081,  5, }, // America/Nome
    {   71, 1850,   39195,  8, }, // America/Noronha
    {   71, 8301,   39203, 20, }, // America/North_Dakota/Beulah
    {   71, 8329,   39223, 20, }, // America/North_Dakota/Center
    {   71, 8357,   39243, 25, }, // America/North_Dakota/New_Salem
    {   71,  393,   39268,  4, }, // America/Nuuk
    {   71, 8388,   39272,  8, }, // America/Ojinaga
    {   71, 2356,   39280,  7, }, // America/Panama
    {   71, 6173,   39287, 11, }, // America/Paramaribo
    {   71, 2973,   39298,  6, }, // America/Phoenix
    {   71, 3945,   39304, 18, }, // America/Port-au-Prince
    {   71, 5954,   39322, 19, }, // America/Port_of_Spain
    {   71, 8404,   39341, 17, }, // America/Porto_Velho
    {   71, 5843,   39358, 12, }, // America/Puerto_Rico
    {   71, 4030,   39370, 14, }, // America/Punta_Arenas
    {   71, 8424,   39384, 14, }, // America/Rankin_Inlet
    {   71, 8445,   39398,  8, }, // America/Recife
    {   71, 1995,   39406,  8, }, // America/Regina
    {   71, 8460,   39414, 13, }, // America/Resolute
    {   71,  695,   39427, 13, }, // America/Rio_Branco
    {   71, 8477,   39440,  8, }, // America/Santarem
    {   71, 2201,   39448, 10, }, // America/Santiago
    {   71, 6111,   39458, 17, }, // America/Santo_Domingo
    {   71, 1878,   39475, 10, }, // America/Sao_Paulo
    {   71, 6899,   39485, 19, }, // America/Scoresbysund
    {   71, 8494,   39504,  5, }, // America/Sitka
    {   71, 8508,   39509, 13, }, // America/St_Barthelemy
    {   71, 2061,   39522, 11, }, // America/St_Johns
    {   71, 5863,   39533,  9, }, // America/St_Kitts
    {   71, 5880,   39542, 11, }, // America/St_Lucia
    {   71,  855,   39553, 13, }, // America/St_Thomas
    {   71, 5913,   39566, 13, }, // America/St_Vincent
    {   71, 8530,   39579, 16, }, // America/Swift_Current
    {   71, 5461,   39595, 11, }, // America/Tegucigalpa
    {   71, 5756,   39606,  4, }, // America/Thule
    {   71,  313,   39610,  7, }, // America/Tijuana
    {   71,  608,   39617,  9, }, // America/Toronto
    {   71, 5688,   39626,  8, }, // America/Tortola
    {   71, 2093,   39634,  9, }, // America/Vancouver
    {   71, 2144,   39643, 14, }, // America/Whitehorse
    {   71,  734,   39657,  9, }, // America/Winnipeg
    {   71, 8552,   39666,  7, }, // America/Yakutat
    {   71, 6001,   39673,  4, }, // Antarctica/Casey
    {   71, 6094,   39677,  6, }, // Antarctica/Davis
    {   71, 6133,   39683, 19, }, // Antarctica/DumontDUrville
    {   71, 8568,   39702,  9, }, // Antarctica/Macquarie
    {   71, 7066,   39711,  8, }, // Antarctica/Mawson
    {   71, 7113,   39719,  9, }, // Antarctica/McMurdo
    {   71, 8589,   39728,  6, }, // Antarctica/Palmer
    {   71, 7226,   39734,  7, }, // Antarctica/Rothera
    {   71, 7295,   39741,  7, }, // Antarctica/Syowa
    {   71, 8607,   39748,  6, }, // Antarctica/Troll
    {   71, 7374,   39754, 10, }, // Antarctica/Vostok
    {   71, 1429,   39764, 13, }, // Arctic/Longyearbyen
    {   71, 5570,   39777,  5, }, // Asia/Aden
    {   71, 5395,   39782,  8, }, // Asia/Almaty
    {   71, 3968,   39790,  7, }, // Asia/Amman
    {   71, 5511,   39797,  9, }, // Asia/Anadyr
    {   71, 5523,   39806,  7, }, // Asia/Aqtau
    {   71, 5368,   39813,  9, }, // Asia/Aqtobe
    {   71,  964,   39822, 11, }, // Asia/Ashgabat
    {   71, 8624,   39833,  7, }, // Asia/Atyrau
    {   71, 3607,   39840,  7, }, // Asia/Baghdad
    {   71, 5534,   39847,  7, }, // Asia/Bahrain
    {   71, 3653,   39854,  4, }, // Asia/Baku
    {   71, 4466,   39858,  8, }, // Asia/Bangkok
    {   71, 3571,   39866,  7, }, // Asia/Barnaul
    {   71, 4086,   39873,  6, }, // Asia/Beirut
    {   71, 3778,   39879,  8, }, // Asia/Bishkek
    {   71, 5989,   39887,  9, }, // Asia/Brunei
    {   71, 4601,   39896,  5, }, // Asia/Chita
    {   71, 4511,   39901,  8, }, // Asia/Colombo
    {   71, 4540,   39909,  8, }, // Asia/Damascus
    {   71, 1093,   39917,  5, }, // Asia/Dhaka
    {   71, 6192,   39922,  5, }, // Asia/Dili
    {   71, 3596,   39927,  8, }, // Asia/Dubai
    {   71, 6159,   39935,  9, }, // Asia/Dushanbe
    {   71, 8636,   39944,  9, }, // Asia/Famagusta
    {   71, 8651,   39953,  4, }, // Asia/Gaza
    {   71, 4795,   39957,  7, }, // Asia/Hebron
    {   71, 1259,   39964, 16, }, // Asia/Ho_Chi_Minh
    {   71, 2704,   39980, 10, }, // Asia/Hong_Kong
    {   71, 4771,   39990,  6, }, // Asia/Hovd
    {   71, 4184,   39996,  8, }, // Asia/Irkutsk
    {   71, 7006,   40004,  6, }, // Asia/Jakarta
    {   71, 6992,   40010,  9, }, // Asia/Jayapura
    {   71, 1290,   40019,  9, }, // Asia/Jerusalem
    {   71, 3560,   40028,  7, }, // Asia/Kabul
    {   71, 4316,   40035,  9, }, // Asia/Kamchatka
    {   71, 4239,   40044,  6, }, // Asia/Karachi
    {   71, 1185,   40050,  9, }, // Asia/Kathmandu
    {   71, 8661,   40059, 10, }, // Asia/Khandyga
    {   71,  992,   40069, 11, }, // Asia/Kolkata
    {   71, 4197,   40080, 13, }, // Asia/Krasnoyarsk
    {   71, 4859,   40093, 10, }, // Asia/Kuala_Lumpur
    {   71, 5976,   40103,  6, }, // Asia/Kuching
    {   71, 5547,   40109,  6, }, // Asia/Kuwait
    {   71, 1211,   40115,  6, }, // Asia/Macau
    {   71, 4017,   40121,  8, }, // Asia/Magadan
    {   71, 1349,   40129,  8, }, // Asia/Makassar
    {   71, 7182,   40137,  5, }, // Asia/Manila
    {   71, 6920,   40142, 10, }, // Asia/Muscat
    {   71, 2554,   40152,  9, }, // Asia/Nicosia
    {   71, 8675,   40161, 15, }, // Asia/Novokuznetsk
    {   71, 4135,   40176, 16, }, // Asia/Novosibirsk
    {   71, 4229,   40192,  9, }, // Asia/Omsk
    {   71, 7145,   40201,  8, }, // Asia/Oral
    {   71, 6961,   40209,  8, }, // Asia/Phnom_Penh
    {   71, 8693,   40217, 14, }, // Asia/Pontianak
    {   71, 4214,   40231,  8, }, // Asia/Pyongyang
    {   71, 5559,   38170,  4, }, // Asia/Qatar
    {   71, 8708,   40239,  9, }, // Asia/Qostanay
    {   71, 4269,   40248,  9, }, // Asia/Qyzylorda
    {   71, 3584,   40257,  7, }, // Asia/Riyadh
    {   71, 4408,   40264,  7, }, // Asia/Sakhalin
    {   71, 7245,   40271, 12, }, // Asia/Samarkand
    {   71, 3279,   40283,  6, }, // Asia/Seoul
    {   71, 1053,   40289, 10, }, // Asia/Shanghai
    {   71, 3300,   40299,  8, }, // Asia/Singapore
    {   71, 4297,   40307, 16, }, // Asia/Srednekolymsk
    {   71, 3263,   40323,  7, }, // Asia/Taipei
    {   71, 4781,   40330, 10, }, // Asia/Tashkent
    {   71, 3915,   40340,  8, }, // Asia/Tbilisi
    {   71, 2772,   40348,  7, }, // Asia/Tehran
    {   71, 1317,   40355,  6, }, // Asia/Thimphu
    {   71, 2821,   40361,  7, }, // Asia/Tokyo
    {   71, 4572,   40368, 10, }, // Asia/Tomsk
    {   71, 1021,   40378,  9, }, // Asia/Ulaanbaatar
    {   71, 1159,   40387,  9, }, // Asia/Urumqi
    {   71, 8722,   40396, 11, }, // Asia/Ust-Nera
    {   71, 6977,   40407, 10, }, // Asia/Vientiane
    {   71, 4710,   40417, 16, }, // Asia/Vladivostok
    {   71, 4828,   40433,  9, }, // Asia/Yakutsk
    {   71, 1235,   40442,  7, }, // Asia/Yangon
    {   71, 3883,   40449, 16, }, // Asia/Yekaterinburg
    {   71, 3747,   40465, 10, }, // Asia/Yerevan
    {   71, 3663,   40475, 11, }, // Atlantic/Azores
    {   71, 5671,   40486,  7, }, // Atlantic/Bermuda
    {   71, 6628,   40493,  5, }, // Atlantic/Canary
    {   71, 3727,   40498, 11, }, // Atlantic/Cape_Verde
    {   71, 1395,   40509,  5, }, // Atlantic/Faroe
    {   71, 8736,   40514,  6, }, // Atlantic/Madeira
    {   71, 2748,   40520, 10, }, // Atlantic/Reykjavik
    {   71, 7272,   40530, 15, }, // Atlantic/South_Georgia
    {   71, 6821,   40545, 13, }, // Atlantic/St_Helena
    {   71, 6644,   40558,  6, }, // Atlantic/Stanley
    {   71, 1670,   40564, 11, }, // Australia/Adelaide
    {   71, 1635,   40575, 10, }, // Australia/Brisbane
    {   71, 1799,   40585, 12, }, // Australia/Broken_Hill
    {   71, 1583,   40597,  5, }, // Australia/Darwin
    {   71, 3637,   40602,  6, }, // Australia/Eucla
    {   71, 1516,   40608,  9, }, // Australia/Hobart
    {   71, 8753,   40617, 10, }, // Australia/Lindeman
    {   71, 1547,   40627, 12, }, // Australia/Lord_Howe
    {   71, 1727,   40639,  9, }, // Australia/Melbourne
    {   71, 1762,   40648,  5, }, // Australia/Perth
    {   71, 1463,   40653,  6, }, // Australia/Sydney
    {   71, 6441,   40659,  9, }, // Europe/Amsterdam
    {   71, 6216,   40668,  8, }, // Europe/Andorra
    {   71, 3620,   40676, 11, }, // Europe/Astrakhan
    {   71, 2300,   40687,  5, }, // Europe/Athens
    {   71, 6336,   40692,  9, }, // Europe/Belgrade
    {   71, 4757,   40701,  5, }, // Europe/Berlin
    {   71, 6488,   40706, 13, }, // Europe/Bratislava
    {   71, 2167,   40719,  8, }, // Europe/Brussels
    {   71, 3928,   40727,  9, }, // Europe/Bucharest
    {   71, 3806,   40736,  9, }, // Europe/Budapest
    {   71, 8772,   40745, 10, }, // Europe/Busingen
    {   71, 2583,   40755, 11, }, // Europe/Chisinau
    {   71, 6289,   40766, 11, }, // Europe/Copenhagen
    {   71, 2338,   40777,  7, }, // Europe/Dublin
    {   71, 6307,   40784, 12, }, // Europe/Gibraltar
    {   71, 8788,   40796,  7, }, // Europe/Guernsey
    {   71, 6612,   40803,  9, }, // Europe/Helsinki
    {   71, 8804,   40812, 10, }, // Europe/Isle_of_Man
    {   71, 1130,   40822, 11, }, // Europe/Istanbul
    {   71, 8823,   40833,  5, }, // Europe/Jersey
    {   71, 3979,   40838, 11, }, // Europe/Kaliningrad
    {   71, 8929,   40849,  7, }, // Europe/Kirov
    {   71, 2527,   40856,  8, }, // Europe/Kyiv
    {   71, 3213,   40864,  9, }, // Europe/Lisbon
    {   71, 6506,   40873, 11, }, // Europe/Ljubljana
    {   71, 2501,   40884,  6, }, // Europe/London
    {   71, 6365,   40890,  8, }, // Europe/Luxembourg
    {   71, 4892,   40898,  5, }, // Europe/Madrid
    {   71, 6397,   40903,  8, }, // Europe/Malta
    {   71, 6582,   40911, 12, }, // Europe/Mariehamn
    {   71, 3693,   40923,  7, }, // Europe/Minsk
    {   71, 6410,   40930,  8, }, // Europe/Monaco
    {   71, 3537,   40938,  8, }, // Europe/Moscow
    {   71, 6458,   40946,  8, }, // Europe/Oslo
    {   71, 4284,   40954,  5, }, // Europe/Paris
    {   71, 6424,   40959, 11, }, // Europe/Podgorica
    {   71, 6275,   40970,  6, }, // Europe/Prague
    {   71, 8837,   40976,  4, }, // Europe/Riga
    {   71, 6324,   40980,  4, }, // Europe/Rome
    {   71, 4331,   40984,  7, }, // Europe/Samara
    {   71, 6470,   40991, 10, }, // Europe/San_Marino
    {   71, 6245,   41001,  9, }, // Europe/Sarajevo
    {   71, 4451,   41010, 10, }, // Europe/Saratov
    {   71, 8849,   41020, 12, }, // Europe/Simferopol
    {   71, 6383,   41032,  9, }, // Europe/Skopje
    {   71, 6599,   41041,  7, }, // Europe/Sofia
    {   71, 6523,   41048, 10, }, // Europe/Stockholm
    {   71, 8867,   41058,  6, }, // Europe/Tallinn
    {   71, 6202,   41064,  6, }, // Europe/Tirane
    {   71, 8882,   41070, 15, }, // Europe/Ulyanovsk
    {   71, 6352,   41085,  7, }, // Europe/Vaduz
    {   71, 6567,   41092,  7, }, // Europe/Vatican
    {   71, 6231,   41099,  7, }, // Europe/Vienna
    {   71, 8899,   41106, 13, }, // Europe/Vilnius
    {   71, 4727,   41119, 12, }, // Europe/Volgograd
    {   71, 3190,   41131,  5, }, // Europe/Warsaw
    {   71, 6261,   41136, 10, }, // Europe/Zagreb
    {   71, 6540,   41146,  9, }, // Europe/Zurich
    {   71, 5078,   41155, 15, }, // Indian/Antananarivo
    {   71, 6947,   41170,  8, }, // Indian/Chagos
    {   71, 6046,   41178,  8, }, // Indian/Christmas
    {   71, 6063,   41186,  9, }, // Indian/Cocos
    {   71, 5029,   41195,  9, }, // Indian/Comoro
    {   71, 6661,   41204,  7, }, // Indian/Kerguelen
    {   71, 7260,   41211,  4, }, // Indian/Mahe
    {   71, 7050,   41215, 10, }, // Indian/Maldives
    {   71, 4069,   41225,  8, }, // Indian/Mauritius
    {   71, 5098,   41233,  7, }, // Indian/Mayotte
    {   71, 7211,   41240,  9, }, // Indian/Reunion
    {   71, 4422,   41249,  8, }, // Pacific/Apia
    {   71,  932,   41257,  8, }, // Pacific/Auckland
    {   71, 3706,   41265, 14, }, // Pacific/Bougainville
    {   71, 3015,   41279,  7, }, // Pacific/Chatham
    {   71, 3157,   41286,  5, }, // Pacific/Chuuk
    {   71, 2237,   41291,  5, }, // Pacific/Easter
    {   71, 7360,   41296,  6, }, // Pacific/Efate
    {   71, 7327,   41302, 10, }, // Pacific/Fakaofo
    {   71, 3902,   41312,  5, }, // Pacific/Fiji
    {   71, 7343,   41317,  8, }, // Pacific/Funafuti
    {   71, 6678,   41325, 11, }, // Pacific/Galapagos
    {   71, 6696,   41336,  7, }, // Pacific/Gambier
    {   71, 3822,   41343, 10, }, // Pacific/Guadalcanal
    {   71, 6018,   41353,  5, }, // Pacific/Guam
    {   71, 2723,   41358, 10, }, // Pacific/Honolulu
    {   71, 3049,   41368,  9, }, // Pacific/Kanton
    {   71, 3998,   41377,  8, }, // Pacific/Kiritimati
    {   71, 7019,   41385, 10, }, // Pacific/Kosrae
    {   71, 2842,   41395, 10, }, // Pacific/Kwajalein
    {   71, 4877,   41405,  9, }, // Pacific/Majuro
    {   71, 4051,   41414, 10, }, // Pacific/Marquesas
    {   71, 8914,   41424,  8, }, // Pacific/Midway
    {   71, 7084,   41432,  6, }, // Pacific/Nauru
    {   71, 7132,   41438,  6, }, // Pacific/Niue
    {   71, 4168,   41444, 10, }, // Pacific/Norfolk
    {   71, 7098,   41454,  7, }, // Pacific/Noumea
    {   71, 3126,   41461, 10, }, // Pacific/Pago_Pago
    {   71, 7155,   41471,  6, }, // Pacific/Palau
    {   71, 7194,   41477, 10, }, // Pacific/Pitcairn
    {   71, 3096,   41487,  9, }, // Pacific/Pohnpei
    {   71, 4807,   41496, 17, }, // Pacific/Port_Moresby
    {   71, 6076,   41513, 12, }, // Pacific/Rarotonga
    {   71, 6031,   41525,  8, }, // Pacific/Saipan
    {   71, 7312,   41533,  5, }, // Pacific/Tahiti
    {   71, 6712,   41538,  6, }, // Pacific/Tarawa
    {   71, 4583,   41544, 10, }, // Pacific/Tongatapu
    {   71, 7392,   41554,  6, }, // Pacific/Wake
    {   71, 7405,   41560,  7, }, // Pacific/Wallis
    {   72, 6788,   41567,  3, }, // Africa/Abidjan Cantonese/Traditional Han/Hong Kong
    {   72, 6760,   41570,  3, }, // Africa/Accra
    {   72, 5059,   41573,  6, }, // Africa/Addis_Ababa
    {   72, 7420,   41579,  4, }, // Africa/Algiers
    {   72,   14,   41583,  4, }, // Africa/Asmara
    {   72,   44,   41587,  3, }, // Africa/Bamako
    {   72, 5257,   41590,  2, }, // Africa/Bangui
    {   72, 6746,   41592,  2, }, // Africa/Banjul
    {   72, 7435,   41594,  2, }, // Africa/Bissau
    {   72, 4957,   41596,  4, }, // Africa/Blantyre
    {   72, 5287,   41600,  5, }, // Africa/Brazzaville
    {   72, 4922,   41605,  4, }, // Africa/Bujumbura
    {   72, 2320,   41609,  2, }, // Africa/Cairo
    {   72, 4117,   41611,  5, }, // Africa/Casablanca
    {   72, 7449,   41616,  2, }, // Africa/Ceuta
    {   72, 6773,   41618,  4, }, // Africa/Conakry
    {   72, 6840,   41622,  3, }, // Africa/Dakar
    {   72, 5130,   41625,  3, }, // Africa/Dar_es_Salaam
    {   72, 5043,   41628,  3, }, // Africa/Djibouti
    {   72, 5243,   41631,  3, }, // Africa/Douala
    {   72, 5166,   41634,  3, }, // Africa/El_Aaiun
    {   72, 6853,   41637,  3, }, // Africa/Freetown
    {   72, 4906,   41640,  4, }, // Africa/Gaborone
    {   72, 5015,   41644,  3, }, // Africa/Harare
    {   72, 4479,   41647,  5, }, // Africa/Johannesburg
    {   72, 4499,   41652,  2, }, // Africa/Juba
    {   72, 5151,   41654,  3, }, // Africa/Kampala
    {   72, 4524,   41657,  3, }, // Africa/Khartoum
    {   72, 4973,   41660,  3, }, // Africa/Kigali
    {   72, 5306,   41663,  3, }, // Africa/Kinshasa
    {   72, 4744,   41666,  3, }, // Africa/Lagos
    {   72, 5336,   41669,  3, }, // Africa/Libreville
    {   72, 6869,   41672,  2, }, // Africa/Lome
    {   72, 5211,   41674,  3, }, // Africa/Luanda
    {   72, 4939,   41677,  4, }, // Africa/Lubumbashi
    {   72, 5001,   41681,  3, }, // Africa/Lusaka
    {   72, 5322,   41684,  3, }, // Africa/Malabo
    {   72, 4987,   41687,  3, }, // Africa/Maputo
    {   72, 5197,   41690,  3, }, // Africa/Maseru
    {   72, 5182,   41693,  3, }, // Africa/Mbabane
    {   72, 5113,   41696,  4, }, // Africa/Mogadishu
    {   72, 7034,   41700,  4, }, // Africa/Monrovia
    {   72, 3853,   41704,  3, }, // Africa/Nairobi
    {   72, 5271,   41707,  4, }, // Africa/Ndjamena
    {   72, 5354,   41711,  3, }, // Africa/Niamey
    {   72, 6803,   41714,  3, }, // Africa/Nouakchott
    {   72, 6727,   41717,  4, }, // Africa/Ouagadougou
    {   72, 5225,   41721,  4, }, // Africa/Porto-Novo
    {   72, 4435,   41725,  3, }, // Africa/Sao_Tome
    {   72, 2866,   41728,  4, }, // Africa/Tripoli
    {   72, 6554,   41732,  3, }, // Africa/Tunis
    {   72, 4152,   41735,  4, }, // Africa/Windhoek
    {   72,  132,   41739,  3, }, // America/Adak
    {   72, 3346,   41742,  4, }, // America/Anchorage
    {   72, 5607,   41746,  3, }, // America/Anguilla
    {   72, 5624,   41749,  3, }, // America/Antigua
    {   72, 4554,   41752,  4, }, // America/Araguaina
    {   72,  166,   41756,  7, }, // America/Argentina/Buenos_Aires
    {   72,   91,   41763,  4, }, // America/Argentina/Catamarca
    {   72,  270,   41767,  3, }, // America/Argentina/Cordoba
    {   72,  441,   41770,  3, }, // America/Argentina/Jujuy
    {   72, 7462,   41773,  3, }, // America/Argentina/La_Rioja
    {   72,  565,   41776,  3, }, // America/Argentina/Mendoza
    {   72, 7489,   41779,  6, }, // America/Argentina/Rio_Gallegos
    {   72, 7520,   41785,  3, }, // America/Argentina/Salta
    {   72, 7544,   41788,  3, }, // America/Argentina/San_Juan
    {   72, 5580,   41791,  3, }, // America/Argentina/San_Luis
    {   72, 7571,   41794,  3, }, // America/Argentina/Tucuman
    {   72, 7597,   41797,  4, }, // America/Argentina/Ushuaia
    {   72, 5640,   41801,  3, }, // America/Aruba
    {   72, 4252,   41804,  3, }, // America/Asuncion
    {   72,  237,   41807,  4, }, // America/Atikokan
    {   72, 3679,   41811,  3, }, // America/Bahia
    {   72, 7623,   41814,  7, }, // America/Bahia_Banderas
    {   72, 5654,   41821,  3, }, // America/Barbados
    {   72, 7646,   41824,  2, }, // America/Belem
    {   72, 5407,   41826,  3, }, // America/Belize
    {   72, 7660,   41829,  5, }, // America/Blanc-Sablon
    {   72, 7681,   41834,  4, }, // America/Boa_Vista
    {   72, 4361,   41838,  3, }, // America/Bogota
    {   72, 7699,   41841,  2, }, // America/Boise
    {   72, 7713,   41843,  3, }, // America/Cambridge_Bay
    {   72, 7735,   41846,  3, }, // America/Campo_Grande
    {   72, 3868,   41849,  2, }, // America/Cancun
    {   72, 4694,   41851,  4, }, // America/Caracas
    {   72, 4345,   41855,  2, }, // America/Cayenne
    {   72, 5496,   41857,  4, }, // America/Cayman
    {   72, 2260,   41861,  3, }, // America/Chicago
    {   72, 7756,   41864,  3, }, // America/Chihuahua
    {   72, 7774,   41867,  4, }, // America/Ciudad_Juarez
    {   72, 5422,   41871,  5, }, // America/Costa_Rica
    {   72, 7796,   41876,  4, }, // America/Creston
    {   72, 3791,   41880,  3, }, // America/Cuiaba
    {   72, 5723,   41883,  3, }, // America/Curacao
    {   72, 7812,   41886,  4, }, // America/Danmarkshavn
    {   72, 7833,   41890,  2, }, // America/Dawson
    {   72, 7848,   41892,  5, }, // America/Dawson_Creek
    {   72,  805,   41897,  2, }, // America/Denver
    {   72, 3465,   41899,  3, }, // America/Detroit
    {   72, 5739,   41902,  4, }, // America/Dominica
    {   72,  893,   41906,  4, }, // America/Edmonton
    {   72, 7869,   41910,  4, }, // America/Eirunepe
    {   72, 5441,   41914,  4, }, // America/El_Salvador
    {   72, 7886,   41918,  4, }, // America/Fort_Nelson
    {   72, 7906,   41922,  4, }, // America/Fortaleza
    {   72, 7924,   41926,  4, }, // America/Glace_Bay
    {   72, 6881,   41930,  2, }, // America/Goose_Bay
    {   72, 4612,   41932,  4, }, // America/Grand_Turk
    {   72, 5770,   41936,  4, }, // America/Grenada
    {   72, 5786,   41940,  4, }, // America/Guadeloupe
    {   72, 3760,   41944,  4, }, // America/Guatemala
    {   72, 4841,   41948,  4, }, // America/Guayaquil
    {   72, 6932,   41952,  3, }, // America/Guyana
    {   72, 1939,   41955,  5, }, // America/Halifax
    {   72, 2281,   41960,  3, }, // America/Havana
    {   72, 7942,   41963,  4, }, // America/Hermosillo
    {   72,  348,   41967,  7, }, // America/Indiana/Indianapolis
    {   72,  481,   41974,  8, }, // America/Indiana/Knox
    {   72, 7961,   41982,  8, }, // America/Indiana/Marengo
    {   72, 7985,   41990,  8, }, // America/Indiana/Petersburg
    {   72, 8012,   41998,  8, }, // America/Indiana/Tell_City
    {   72, 8038,   42006,  7, }, // America/Indiana/Vevay
    {   72, 8060,   42013,  8, }, // America/Indiana/Vincennes
    {   72, 8086,   42021,  9, }, // America/Indiana/Winamac
    {   72, 8110,   42030,  4, }, // America/Inuvik
    {   72,  660,   42034,  3, }, // America/Iqaluit
    {   72, 2799,   42037,  3, }, // America/Jamaica
    {   72, 5380,   42040,  2, }, // America/Juneau
    {   72,  521,   42042,  5, }, // America/Kentucky/Louisville
    {   72, 8125,   42047,  8, }, // America/Kentucky/Monticello
    {   72, 5704,   42055,  5, }, // America/Kralendijk
    {   72, 4376,   42060,  3, }, // America/La_Paz
    {   72, 7169,   42063,  2, }, // America/Lima
    {   72, 3239,   42065,  3, }, // America/Los_Angeles
    {   72, 5932,   42068,  4, }, // America/Lower_Princes
    {   72, 8153,   42072,  3, }, // America/Maceio
    {   72, 8168,   42075,  3, }, // America/Managua
    {   72, 1908,   42078,  3, }, // America/Manaus
    {   72, 5897,   42081,  4, }, // America/Marigot
    {   72, 5805,   42085,  4, }, // America/Martinique
    {   72, 8184,   42089,  5, }, // America/Matamoros
    {   72, 2917,   42094,  4, }, // America/Mazatlan
    {   72, 8202,   42098,  4, }, // America/Menominee
    {   72, 8220,   42102,  3, }, // America/Merida
    {   72, 8235,   42105,  6, }, // America/Metlakatla
    {   72, 2949,   42111,  4, }, // America/Mexico_City
    {   72, 4391,   42115,  3, }, // America/Miquelon
    {   72, 8254,   42118,  3, }, // America/Moncton
    {   72, 8270,   42121,  3, }, // America/Monterrey
    {   72, 4098,   42124,  4, }, // America/Montevideo
    {   72, 5824,   42128,  3, }, // America/Montserrat
    {   72, 5481,   42131,  2, }, // America/Nassau
    {   72, 2379,   42133,  2, }, // America/New_York
    {   72, 8288,   42135,  2, }, // America/Nome
    {   72, 1850,   42137,  3, }, // America/Noronha
    {   72, 8301,   42140,  8, }, // America/North_Dakota/Beulah
    {   72, 8329,   42148,  7, }, // America/North_Dakota/Center
    {   72, 8357,   42155,  8, }, // America/North_Dakota/New_Salem
    {   72,  393,   42163,  2, }, // America/Nuuk
    {   72, 8388,   42165,  4, }, // America/Ojinaga
    {   72, 2356,   42169,  3, }, // America/Panama
    {   72, 6173,   42172,  5, }, // America/Paramaribo
    {   72, 2973,   42177,  3, }, // America/Phoenix
    {   72, 3945,   42180,  3, }, // America/Port-au-Prince
    {   72, 5954,   42183,  4, }, // America/Port_of_Spain
    {   72, 8404,   42187,  3, }, // America/Porto_Velho
    {   72, 5843,   42190,  4, }, // America/Puerto_Rico
    {   72, 4030,   42194,  6, }, // America/Punta_Arenas
    {   72, 8424,   42200,  3, }, // America/Rankin_Inlet
    {   72, 8445,   42203,  3, }, // America/Recife
    {   72, 1995,   42206,  3, }, // America/Regina
    {   72, 8460,   42209,  4, }, // America/Resolute
    {   72,  695,   42213,  4, }, // America/Rio_Branco
    {   72, 8477,   42217,  3, }, // America/Santarem
    {   72, 2201,   42220,  4, }, // America/Santiago
    {   72, 6111,   42224,  4, }, // America/Santo_Domingo
    {   72, 1878,   42228,  3, }, // America/Sao_Paulo
    {   72, 6899,   42231,  7, }, // America/Scoresbysund
    {   72, 8494,   42238,  3, }, // America/Sitka
    {   72, 8508,   42241,  6, }, // America/St_Barthelemy
    {   72, 2061,   42247,  3, }, // America/St_Johns
    {   72, 5863,   42250,  3, }, // America/St_Kitts
    {   72, 5880,   42253,  4, }, // America/St_Lucia
    {   72,  855,   42257,  4, }, // America/St_Thomas
    {   72, 5913,   42261,  3, }, // America/St_Vincent
    {   72, 8530,   42264,  7, }, // America/Swift_Current
    {   72, 5461,   42271,  5, }, // America/Tegucigalpa
    {   72, 5756,   42276,  2, }, // America/Thule
    {   72,  313,   42278,  3, }, // America/Tijuana
    {   72,  608,   42281,  3, }, // America/Toronto
    {   72, 5688,   42284,  4, }, // America/Tortola
    {   72, 2093,   42288,  3, }, // America/Vancouver
    {   72, 2144,   42291,  4, }, // America/Whitehorse
    {   72,  734,   42295,  3, }, // America/Winnipeg
    {   72, 8552,   42298,  3, }, // America/Yakutat
    {   72, 6001,   42301,  2, }, // Antarctica/Casey
    {   72, 6094,   42303,  3, }, // Antarctica/Davis
    {   72, 6133,   42306,  5, }, // Antarctica/DumontDUrville
    {   72, 8568,   42311,  3, }, // Antarctica/Macquarie
    {   72, 7066,   42314,  2, }, // Antarctica/Mawson
    {   72, 7113,   42316,  4, }, // Antarctica/McMurdo
    {   72, 8589,   42320,  2, }, // Antarctica/Palmer
    {   72, 7226,   42322,  3, }, // Antarctica/Rothera
    {   72, 7295,   42325,  4, }, // Antarctica/Syowa
    {   72, 8607,   42329,  2, }, // Antarctica/Troll
    {   72, 7374,   42331,  4, }, // Antarctica/Vostok
    {   72, 1429,   42335,  5, }, // Arctic/Longyearbyen
    {   72, 5570,   42340,  2, }, // Asia/Aden
    {   72, 5395,   42342,  4, }, // Asia/Almaty
    {   72, 3968,   42346,  2, }, // Asia/Amman
    {   72, 5511,   42348,  3, }, // Asia/Anadyr
    {   72, 5523,   42351,  3, }, // Asia/Aqtau
    {   72, 5368,   42354,  4, }, // Asia/Aqtobe
    {   72,  964,   42358,  5, }, // Asia/Ashgabat
    {   72, 8624,   42363,  3, }, // Asia/Atyrau
    {   72, 3607,   42366,  3, }, // Asia/Baghdad
    {   72, 5534,   42369,  2, }, // Asia/Bahrain
    {   72, 3653,   41882,  2, }, // Asia/Baku
    {   72, 4466,   42371,  2, }, // Asia/Bangkok
    {   72, 3571,   42373,  4, }, // Asia/Barnaul
    {   72, 4086,   42377,  3, }, // Asia/Beirut
    {   72, 3778,   42380,  4, }, // Asia/Bishkek
    {   72, 5989,   42384,  2, }, // Asia/Brunei
    {   72, 4601,   42386,  2, }, // Asia/Chita
    {   72, 4511,   42388,  3, }, // Asia/Colombo
    {   72, 4540,   42391,  4, }, // Asia/Damascus
    {   72, 1093,   42395,  2, }, // Asia/Dhaka
    {   72, 6192,   42397,  2, }, // Asia/Dili
    {   72, 3596,   42399,  2, }, // Asia/Dubai
    {   72, 6159,   42401,  3, }, // Asia/Dushanbe
    {   72, 8636,   42404,  5, }, // Asia/Famagusta
    {   72, 8651,   42409,  2, }, // Asia/Gaza
    {   72, 4795,   42411,  3, }, // Asia/Hebron
    {   72, 1259,   42414,  4, }, // Asia/Ho_Chi_Minh
    {   72, 2704,   42418, 14, }, // Asia/Hong_Kong
    {   72, 4771,   42432,  3, }, // Asia/Hovd
    {   72, 4184,   42435,  5, }, // Asia/Irkutsk
    {   72, 7006,   42440,  3, }, // Asia/Jakarta
    {   72, 6992,   42443,  4, }, // Asia/Jayapura
    {   72, 1290,   42447,  4, }, // Asia/Jerusalem
    {   72, 3560,   42451,  3, }, // Asia/Kabul
    {   72, 4316,   42454,  3, }, // Asia/Kamchatka
    {   72, 4239,   42457,  3, }, // Asia/Karachi
    {   72, 1185,   42460,  4, }, // Asia/Kathmandu
    {   72, 8661,   42464,  3, }, // Asia/Khandyga
    {   72,  992,   42467,  4, }, // Asia/Kolkata
    {   72, 4197,   42471,  8, }, // Asia/Krasnoyarsk
    {   72, 4859,   42479,  3, }, // Asia/Kuala_Lumpur
    {   72, 5976,   42482,  2, }, // Asia/Kuching
    {   72, 5547,   42484,  3, }, // Asia/Kuwait
    {   72, 1211,   42487, 14, }, // Asia/Macau
    {   72, 4017,   42501,  3, }, // Asia/Magadan
    {   72, 1349,   42504,  4, }, // Asia/Makassar
    {   72, 7182,   42508,  3, }, // Asia/Manila
    {   72, 6920,   42511,  4, }, // Asia/Muscat
    {   72, 2554,   42515,  4, }, // Asia/Nicosia
    {   72, 8675,   42519,  6, }, // Asia/Novokuznetsk
    {   72, 4135,   42525,  5, }, // Asia/Novosibirsk
    {   72, 4229,   42530,  4, }, // Asia/Omsk
    {   72, 7145,   42534,  3, }, // Asia/Oral
    {   72, 6961,   42537,  2, }, // Asia/Phnom_Penh
    {   72, 8693,   42539,  2, }, // Asia/Pontianak
    {   72, 4214,   42541,  2, }, // Asia/Pyongyang
    {   72, 5559,   42543,  2, }, // Asia/Qatar
    {   72, 8708,   42545,  4, }, // Asia/Qostanay
    {   72, 4269,   42549,  6, }, // Asia/Qyzylorda
    {   72, 3584,   42555,  3, }, // Asia/Riyadh
    {   72, 4408,   42558,  3, }, // Asia/Sakhalin
    {   72, 7245,   42561,  4, }, // Asia/Samarkand
    {   72, 3279,   42565,  2, }, // Asia/Seoul
    {   72, 1053,   42567,  2, }, // Asia/Shanghai
    {   72, 3300,   42569,  3, }, // Asia/Singapore
    {   72, 4297,   42572,  6, }, // Asia/Srednekolymsk
    {   72, 3263,   42578,  2, }, // Asia/Taipei
    {   72, 4781,   42580,  3, }, // Asia/Tashkent
    {   72, 3915,   42583,  4, }, // Asia/Tbilisi
    {   72, 2772,   42587,  3, }, // Asia/Tehran
    {   72, 1317,   42590,  2, }, // Asia/Thimphu
    {   72, 2821,   42592,  2, }, // Asia/Tokyo
    {   72, 4572,   42594,  4, }, // Asia/Tomsk
    {   72, 1021,   42598,  4, }, // Asia/Ulaanbaatar
    {   72, 1159,   42602,  4, }, // Asia/Urumqi
    {   72, 8722,   42606,  4, }, // Asia/Ust-Nera
    {   72, 6977,   42610,  2, }, // Asia/Vientiane
    {   72, 4710,   42612,  3, }, // Asia/Vladivostok
    {   72, 4828,   42615,  4, }, // Asia/Yakutsk
    {   72, 1235,   42619,  2, }, // Asia/Yangon
    {   72, 3883,   42621,  5, }, // Asia/Yekaterinburg
    {   72, 3747,   42626,  3, }, // Asia/Yerevan
    {   72, 3663,   42629,  5, }, // Atlantic/Azores
    {   72, 5671,   42634,  3, }, // Atlantic/Bermuda
    {   72, 6628,   42637,  3, }, // Atlantic/Canary
    {   72, 3727,   42640,  3, }, // Atlantic/Cape_Verde
    {   72, 1395,   42643,  4, }, // Atlantic/Faroe
    {   72, 8736,   42647,  5, }, // Atlantic/Madeira
    {   72, 2748,   42652,  5, }, // Atlantic/Reykjavik
    {   72, 7272,   42657,  4, }, // Atlantic/South_Georgia
    {   72, 6821,   42661,  5, }, // Atlantic/St_Helena
    {   72, 6644,   42666,  3, }, // Atlantic/Stanley
    {   72, 1670,   42669,  4, }, // Australia/Adelaide
    {   72, 1635,   42673,  4, }, // Australia/Brisbane
    {   72, 1799,   42677,  5, }, // Australia/Broken_Hill
    {   72, 1583,   42682,  3, }, // Australia/Darwin
    {   72, 3637,   42685,  3, }, // Australia/Eucla
    {   72, 1516,   42688,  3, }, // Australia/Hobart
    {   72, 8753,   42691,  3, }, // Australia/Lindeman
    {   72, 1547,   42694,  4, }, // Australia/Lord_Howe
    {   72, 1727,   42698,  3, }, // Australia/Melbourne
    {   72, 1762,   42701,  2, }, // Australia/Perth
    {   72, 1463,   42703,  2, }, // Australia/Sydney
    {   72, 6441,   42705,  5, }, // Europe/Amsterdam
    {   72, 6216,   42710,  3, }, // Europe/Andorra
    {   72, 3620,   42713,  5, }, // Europe/Astrakhan
    {   72, 2300,   42718,  2, }, // Europe/Athens
    {   72, 6336,   42720,  4, }, // Europe/Belgrade
    {   72, 4757,   42724,  2, }, // Europe/Berlin
    {   72, 6488,   42726,  6, }, // Europe/Bratislava
    {   72, 2167,   42732,  4, }, // Europe/Brussels
    {   72, 3928,   42736,  5, }, // Europe/Bucharest
    {   72, 3806,   42741,  4, }, // Europe/Budapest
    {   72, 8772,   42745,  3, }, // Europe/Busingen
    {   72, 2583,   42748,  3, }, // Europe/Chisinau
    {   72, 6289,   42751,  4, }, // Europe/Copenhagen
    {   72, 2338,   42755,  3, }, // Europe/Dublin
    {   72, 6307,   42758,  4, }, // Europe/Gibraltar
    {   72, 8788,   42762,  3, }, // Europe/Guernsey
    {   72, 6612,   42765,  4, }, // Europe/Helsinki
    {   72, 8804,   42769,  2, }, // Europe/Isle_of_Man
    {   72, 1130,   42771,  4, }, // Europe/Istanbul
    {   72, 8823,   42775,  3, }, // Europe/Jersey
    {   72, 3979,   42778,  5, }, // Europe/Kaliningrad
    {   72, 8929,   42783,  3, }, // Europe/Kirov
    {   72, 2527,   42786,  2, }, // Europe/Kyiv
    {   72, 3213,   42788,  3, }, // Europe/Lisbon
    {   72, 6506,   42791,  4, }, // Europe/Ljubljana
    {   72, 2501,   42795,  2, }, // Europe/London
    {   72, 6365,   42797,  3, }, // Europe/Luxembourg
    {   72, 4892,   42800,  3, }, // Europe/Madrid
    {   72, 6397,   42803,  3, }, // Europe/Malta
    {   72, 6582,   42806,  3, }, // Europe/Mariehamn
    {   72, 3693,   42809,  3, }, // Europe/Minsk
    {   72, 6410,   42812,  3, }, // Europe/Monaco
    {   72, 3537,   42815,  3, }, // Europe/Moscow
    {   72, 6458,   42818,  3, }, // Europe/Oslo
    {   72, 4284,   42821,  2, }, // Europe/Paris
    {   72, 6424,   42823,  4, }, // Europe/Podgorica
    {   72, 6275,   42827,  3, }, // Europe/Prague
    {   72, 8837,   42830,  2, }, // Europe/Riga
    {   72, 6324,   42832,  2, }, // Europe/Rome
    {   72, 4331,   42834,  3, }, // Europe/Samara
    {   72, 6470,   42837,  4, }, // Europe/San_Marino
    {   72, 6245,   42841,  4, }, // Europe/Sarajevo
    {   72, 4451,   42845,  4, }, // Europe/Saratov
    {   72, 8849,   42849,  4, }, // Europe/Simferopol
    {   72, 6383,   42853,  4, }, // Europe/Skopje
    {   72, 6599,   42857,  3, }, // Europe/Sofia
    {   72, 6523,   42860,  5, }, // Europe/Stockholm
    {   72, 8867,   42865,  2, }, // Europe/Tallinn
    {   72, 6202,   42867,  3, }, // Europe/Tirane
    {   72, 8882,   42870,  7, }, // Europe/Ulyanovsk
    {   72, 6352,   42877,  3, }, // Europe/Vaduz
    {   72, 6567,   42880,  3, }, // Europe/Vatican
    {   72, 6231,   42883,  3, }, // Europe/Vienna
    {   72, 8899,   42886,  4, }, // Europe/Vilnius
    {   72, 4727,   42890,  5, }, // Europe/Volgograd
    {   72, 3190,   42895,  2, }, // Europe/Warsaw
    {   72, 6261,   42897,  4, }, // Europe/Zagreb
    {   72, 6540,   42901,  3, }, // Europe/Zurich
    {   72, 5078,   42904,  6, }, // Indian/Antananarivo
    {   72, 6947,   42910,  3, }, // Indian/Chagos
    {   72, 6046,   42913,  3, }, // Indian/Christmas
    {   72, 6063,   42916,  5, }, // Indian/Cocos
    {   72, 5029,   42921,  5, }, // Indian/Comoro
    {   72, 6661,   42926,  5, }, // Indian/Kerguelen
    {   72, 7260,   42931,  3, }, // Indian/Mahe
    {   72, 7050,   42934,  4, }, // Indian/Maldives
    {   72, 4069,   42938,  4, }, // Indian/Mauritius
    {   72, 5098,   42942,  4, }, // Indian/Mayotte
    {   72, 7211,   42946,  4, }, // Indian/Reunion
    {   72, 4422,   42950,  3, }, // Pacific/Apia
    {   72,  932,   42953,  3, }, // Pacific/Auckland
    {   72, 3706,   42956,  4, }, // Pacific/Bougainville
    {   72, 3015,   42960,  2, }, // Pacific/Chatham
    {   72, 3157,   42962,  2, }, // Pacific/Chuuk
    {   72, 2237,   42964,  3, }, // Pacific/Easter
    {   72, 7360,   42967,  3, }, // Pacific/Efate
    {   72, 7327,   42970,  3, }, // Pacific/Fakaofo
    {   72, 3902,   42973,  2, }, // Pacific/Fiji
    {   72, 7343,   42975,  4, }, // Pacific/Funafuti
    {   72, 6678,   42979,  6, }, // Pacific/Galapagos
    {   72, 6696,   42985,  5, }, // Pacific/Gambier
    {   72, 3822,   42990,  6, }, // Pacific/Guadalcanal
    {   72, 6018,   42996,  2, }, // Pacific/Guam
    {   72, 2723,   42998,  3, }, // Pacific/Honolulu
    {   72, 3049,   43001,  5, }, // Pacific/Kanton
    {   72, 3998,   43006,  6, }, // Pacific/Kiritimati
    {   72, 7019,   43012,  3, }, // Pacific/Kosrae
    {   72, 2842,   43015,  4, }, // Pacific/Kwajalein
    {   72, 4877,   43019,  3, }, // Pacific/Majuro
    {   72, 4051,   43022,  5, }, // Pacific/Marquesas
    {   72, 8914,   43027,  3, }, // Pacific/Midway
    {   72, 7084,   43030,  2, }, // Pacific/Nauru
    {   72, 7132,   43032,  3, }, // Pacific/Niue
    {   72, 4168,   43035,  3, }, // Pacific/Norfolk
    {   72, 7098,   43038,  3, }, // Pacific/Noumea
    {   72, 3126,   43041,  4, }, // Pacific/Pago_Pago
    {   72, 7155,   43045,  2, }, // Pacific/Palau
    {   72, 7194,   43047,  5, }, // Pacific/Pitcairn
    {   72, 3096,   43052,  3, }, // Pacific/Pohnpei
    {   72, 4807,   43055,  4, }, // Pacific/Port_Moresby
    {   72, 6076,   43059,  4, }, // Pacific/Rarotonga
    {   72, 6031,   43063,  2, }, // Pacific/Saipan
    {   72, 7312,   43065,  3, }, // Pacific/Tahiti
    {   72, 6712,   43068,  3, }, // Pacific/Tarawa
    {   72, 4583,   43071,  5, }, // Pacific/Tongatapu
    {   72, 7392,   43076,  2, }, // Pacific/Wake
    {   72, 7405,   43078,  3, }, // Pacific/Wallis
    {   73, 6788,   43081,  3, }, // Africa/Abidjan Cantonese/Simplified Han/China
    {   73, 5059,   43084,  6, }, // Africa/Addis_Ababa
    {   73, 7420,   43090,  4, }, // Africa/Algiers
    {   73,   14,   43094,  4, }, // Africa/Asmara
    {   73,   44,   43098,  3, }, // Africa/Bamako
    {   73, 7435,   43101,  2, }, // Africa/Bissau
    {   73, 4957,   43103,  4, }, // Africa/Blantyre
    {   73, 5287,   43107,  5, }, // Africa/Brazzaville
    {   73, 2320,   43112,  2, }, // Africa/Cairo
    {   73, 4117,   43114,  5, }, // Africa/Casablanca
    {   73, 7449,   43119,  2, }, // Africa/Ceuta
    {   73, 6840,   43121,  3, }, // Africa/Dakar
    {   73, 5130,   43124,  3, }, // Africa/Dar_es_Salaam
    {   73, 4479,   43127,  5, }, // Africa/Johannesburg
    {   73, 5211,   43132,  3, }, // Africa/Luanda
    {   73, 4939,   43135,  4, }, // Africa/Lubumbashi
    {   73, 5322,   43139,  3, }, // Africa/Malabo
    {   73, 4987,   43142,  3, }, // Africa/Maputo
    {   73, 5197,   43145,  3, }, // Africa/Maseru
    {   73, 7034,   43148,  4, }, // Africa/Monrovia
    {   73, 5271,   43152,  4, }, // Africa/Ndjamena
    {   73, 5354,   43156,  3, }, // Africa/Niamey
    {   73, 6803,   43159,  3, }, // Africa/Nouakchott
    {   73, 5225,   43162,  4, }, // Africa/Porto-Novo
    {   73, 4435,   43166,  3, }, // Africa/Sao_Tome
    {   73, 4152,   43169,  4, }, // Africa/Windhoek
    {   73,  132,   43173,  3, }, // America/Adak
    {   73,  166,   43176,  7, }, // America/Argentina/Buenos_Aires
    {   73,   91,   43183,  4, }, // America/Argentina/Catamarca
    {   73,  270,   43187,  3, }, // America/Argentina/Cordoba
    {   73,  565,   43190,  3, }, // America/Argentina/Mendoza
    {   73, 7489,   43193,  6, }, // America/Argentina/Rio_Gallegos
    {   73, 7520,   43199,  3, }, // America/Argentina/Salta
    {   73, 7544,   43202,  3, }, // America/Argentina/San_Juan
    {   73, 5580,   43205,  3, }, // America/Argentina/San_Luis
    {   73, 7571,   43208,  3, }, // America/Argentina/Tucuman
    {   73, 7597,   43211,  4, }, // America/Argentina/Ushuaia
    {   73, 4252,   43215,  3, }, // America/Asuncion
    {   73, 7623,   43218,  7, }, // America/Bahia_Banderas
    {   73, 5654,   43225,  3, }, // America/Barbados
    {   73, 7646,   43228,  2, }, // America/Belem
    {   73, 5407,   43230,  3, }, // America/Belize
    {   73, 7660,   43233,  5, }, // America/Blanc-Sablon
    {   73, 7681,   43238,  4, }, // America/Boa_Vista
    {   73, 7713,   43242,  3, }, // America/Cambridge_Bay
    {   73, 7735,   43245,  3, }, // America/Campo_Grande
    {   73, 4345,   43248,  2, }, // America/Cayenne
    {   73, 5496,   43250,  4, }, // America/Cayman
    {   73, 7756,   43254,  3, }, // America/Chihuahua
    {   73, 7774,   43257,  4, }, // America/Ciudad_Juarez
    {   73, 7796,   43261,  4, }, // America/Creston
    {   73, 5723,   43265,  3, }, // America/Curacao
    {   73, 7812,   43268,  4, }, // America/Danmarkshavn
    {   73, 7869,   43272,  4, }, // America/Eirunepe
    {   73, 5441,   43276,  4, }, // America/El_Salvador
    {   73, 7886,   43280,  4, }, // America/Fort_Nelson
    {   73, 7924,   43284,  4, }, // America/Glace_Bay
    {   73, 6881,   43288,  2, }, // America/Goose_Bay
    {   73, 4612,   43290,  4, }, // America/Grand_Turk
    {   73, 5770,   43294,  4, }, // America/Grenada
    {   73, 3760,   43298,  4, }, // America/Guatemala
    {   73, 4841,   43302,  4, }, // America/Guayaquil
    {   73, 6932,   43306,  3, }, // America/Guyana
    {   73, 7942,   43309,  4, }, // America/Hermosillo
    {   73,  481,   43313,  8, }, // America/Indiana/Knox
    {   73, 7961,   43321,  8, }, // America/Indiana/Marengo
    {   73, 8012,   43329,  8, }, // America/Indiana/Tell_City
    {   73, 8038,   43337,  7, }, // America/Indiana/Vevay
    {   73, 8060,   43344,  8, }, // America/Indiana/Vincennes
    {   73, 8086,   43352,  9, }, // America/Indiana/Winamac
    {   73, 8110,   43361,  4, }, // America/Inuvik
    {   73, 2799,   43365,  3, }, // America/Jamaica
    {   73, 5380,   43368,  2, }, // America/Juneau
    {   73,  521,   43370,  5, }, // America/Kentucky/Louisville
    {   73, 8125,   43375,  8, }, // America/Kentucky/Monticello
    {   73, 5704,   43383,  5, }, // America/Kralendijk
    {   73, 7169,   43388,  2, }, // America/Lima
    {   73, 3239,   43390,  3, }, // America/Los_Angeles
    {   73, 5932,   43393,  4, }, // America/Lower_Princes
    {   73, 8153,   43397,  3, }, // America/Maceio
    {   73, 8168,   43400,  3, }, // America/Managua
    {   73, 1908,   43403,  3, }, // America/Manaus
    {   73, 5897,   43406,  4, }, // America/Marigot
    {   73, 5805,   43410,  4, }, // America/Martinique
    {   73, 8184,   43414,  5, }, // America/Matamoros
    {   73, 2917,   43419,  4, }, // America/Mazatlan
    {   73, 8202,   43423,  4, }, // America/Menominee
    {   73, 8220,   43427,  3, }, // America/Merida
    {   73, 4391,   43430,  3, }, // America/Miquelon
    {   73, 8254,   43433,  3, }, // America/Moncton
    {   73, 4098,   43436,  4, }, // America/Montevideo
    {   73, 5824,   43440,  3, }, // America/Montserrat
    {   73, 2379,   43443,  2, }, // America/New_York
    {   73, 8288,   43445,  2, }, // America/Nome
    {   73, 1850,   43447,  3, }, // America/Noronha
    {   73, 8301,   43450,  8, }, // America/North_Dakota/Beulah
    {   73, 8329,   43458,  7, }, // America/North_Dakota/Center
    {   73, 8357,   43465,  8, }, // America/North_Dakota/New_Salem
    {   73, 8388,   43473,  4, }, // America/Ojinaga
    {   73, 2356,   43477,  3, }, // America/Panama
    {   73, 6173,   43480,  5, }, // America/Paramaribo
    {   73, 2973,   43485,  3, }, // America/Phoenix
    {   73, 8404,   43488,  3, }, // America/Porto_Velho
    {   73, 4030,   43491,  6, }, // America/Punta_Arenas
    {   73, 8424,   43497,  3, }, // America/Rankin_Inlet
    {   73, 1995,   43500,  3, }, // America/Regina
    {   73, 8460,   43503,  4, }, // America/Resolute
    {   73,  695,   43507,  4, }, // America/Rio_Branco
    {   73, 8477,   43511,  3, }, // America/Santarem
    {   73, 2201,   43514,  4, }, // America/Santiago
    {   73, 6111,   43518,  4, }, // America/Santo_Domingo
    {   73, 1878,   43522,  3, }, // America/Sao_Paulo
    {   73, 6899,   43525,  7, }, // America/Scoresbysund
    {   73, 8494,   43532,  3, }, // America/Sitka
    {   73, 8508,   43535,  6, }, // America/St_Barthelemy
    {   73, 2061,   43541,  3, }, // America/St_Johns
    {   73, 5863,   43544,  3, }, // America/St_Kitts
    {   73, 5880,   43547,  4, }, // America/St_Lucia
    {   73,  855,   43551,  4, }, // America/St_Thomas
    {   73, 5913,   43555,  3, }, // America/St_Vincent
    {   73, 8530,   43558,  7, }, // America/Swift_Current
    {   73,  313,   43565,  3, }, // America/Tijuana
    {   73,  608,   43568,  3, }, // America/Toronto
    {   73, 5688,   43571,  4, }, // America/Tortola
    {   73, 2093,   43575,  3, }, // America/Vancouver
    {   73, 2144,   43578,  4, }, // America/Whitehorse
    {   73,  734,   43582,  3, }, // America/Winnipeg
    {   73, 8552,   43585,  3, }, // America/Yakutat
    {   73, 6001,   43588,  2, }, // Antarctica/Casey
    {   73, 6094,   43590,  3, }, // Antarctica/Davis
    {   73, 6133,   43593,  5, }, // Antarctica/DumontDUrville
    {   73, 8568,   43598,  3, }, // Antarctica/Macquarie
    {   73, 7113,   43601,  4, }, // Antarctica/McMurdo
    {   73, 8589,   43605,  2, }, // Antarctica/Palmer
    {   73, 7226,   43607,  3, }, // Antarctica/Rothera
    {   73, 8607,   43610,  2, }, // Antarctica/Troll
    {   73, 5570,   43612,  2, }, // Asia/Aden
    {   73, 5395,   43614,  4, }, // Asia/Almaty
    {   73, 8624,   43618,  3, }, // Asia/Atyrau
    {   73, 3607,   43621,  3, }, // Asia/Baghdad
    {   73, 3653,   43624,  2, }, // Asia/Baku
    {   73, 3571,   43626,  4, }, // Asia/Barnaul
    {   73, 4086,   43630,  3, }, // Asia/Beirut
    {   73, 3778,   43633,  4, }, // Asia/Bishkek
    {   73, 5989,   43637,  2, }, // Asia/Brunei
    {   73, 4511,   43639,  3, }, // Asia/Colombo
    {   73, 4540,   43642,  4, }, // Asia/Damascus
    {   73, 1093,   43646,  2, }, // Asia/Dhaka
    {   73, 6159,   43648,  3, }, // Asia/Dushanbe
    {   73, 8636,   43651,  5, }, // Asia/Famagusta
    {   73, 8651,   43656,  2, }, // Asia/Gaza
    {   73, 2704,   43658, 14, }, // Asia/Hong_Kong
    {   73, 4184,   43672,  5, }, // Asia/Irkutsk
    {   73, 7006,   43677,  3, }, // Asia/Jakarta
    {   73, 6992,   43680,  4, }, // Asia/Jayapura
    {   73, 3560,   43684,  3, }, // Asia/Kabul
    {   73, 1185,   43687,  4, }, // Asia/Kathmandu
    {   73,  992,   43691,  4, }, // Asia/Kolkata
    {   73, 4197,   43695,  8, }, // Asia/Krasnoyarsk
    {   73, 5976,   43703,  2, }, // Asia/Kuching
    {   73, 1211,   43705, 14, }, // Asia/Macau
    {   73, 4017,   43719,  3, }, // Asia/Magadan
    {   73, 1349,   43722,  4, }, // Asia/Makassar
    {   73, 7182,   43726,  3, }, // Asia/Manila
    {   73, 6920,   43729,  4, }, // Asia/Muscat
    {   73, 2554,   43733,  4, }, // Asia/Nicosia
    {   73, 8675,   43737,  6, }, // Asia/Novokuznetsk
    {   73, 4135,   43743,  5, }, // Asia/Novosibirsk
    {   73, 7145,   43748,  3, }, // Asia/Oral
    {   73, 6961,   43751,  2, }, // Asia/Phnom_Penh
    {   73, 5559,   43753,  2, }, // Asia/Qatar
    {   73, 4269,   43755,  6, }, // Asia/Qyzylorda
    {   73, 4408,   43761,  3, }, // Asia/Sakhalin
    {   73, 7245,   43764,  4, }, // Asia/Samarkand
    {   73, 3279,   43768,  2, }, // Asia/Seoul
    {   73, 2772,   43770,  3, }, // Asia/Tehran
    {   73, 2821,   43773,  2, }, // Asia/Tokyo
    {   73, 1021,   43775,  4, }, // Asia/Ulaanbaatar
    {   73, 1159,   43779,  4, }, // Asia/Urumqi
    {   73, 8722,   43783,  4, }, // Asia/Ust-Nera
    {   73, 4710,   43787,  3, }, // Asia/Vladivostok
    {   73, 4828,   43790,  4, }, // Asia/Yakutsk
    {   73, 3883,   43794,  5, }, // Asia/Yekaterinburg
    {   73, 3747,   43799,  3, }, // Asia/Yerevan
    {   73, 3663,   43802,  5, }, // Atlantic/Azores
    {   73, 5671,   43807,  3, }, // Atlantic/Bermuda
    {   73, 6628,   43810,  3, }, // Atlantic/Canary
    {   73, 3727,   43813,  3, }, // Atlantic/Cape_Verde
    {   73, 1395,   43816,  4, }, // Atlantic/Faroe
    {   73, 8736,   43820,  5, }, // Atlantic/Madeira
    {   73, 2748,   43825,  5, }, // Atlantic/Reykjavik
    {   73, 7272,   43830,  4, }, // Atlantic/South_Georgia
    {   73, 6821,   43834,  5, }, // Atlantic/St_Helena
    {   73, 1799,   43839,  5, }, // Australia/Broken_Hill
    {   73, 1583,   43844,  3, }, // Australia/Darwin
    {   73, 1547,   43847,  4, }, // Australia/Lord_Howe
    {   73, 1727,   43851,  3, }, // Australia/Melbourne
    {   73, 6216,   43854,  3, }, // Europe/Andorra
    {   73, 6336,   43857,  4, }, // Europe/Belgrade
    {   73, 2167,   43861,  4, }, // Europe/Brussels
    {   73, 3806,   43865,  4, }, // Europe/Budapest
    {   73, 2338,   42755,  3, }, // Europe/Dublin
    {   73, 6307,   43869,  4, }, // Europe/Gibraltar
    {   73, 8788,   43873,  3, }, // Europe/Guernsey
    {   73, 6612,   43876,  4, }, // Europe/Helsinki
    {   73, 8804,   43880,  2, }, // Europe/Isle_of_Man
    {   73, 8823,   43882,  3, }, // Europe/Jersey
    {   73, 3979,   43885,  5, }, // Europe/Kaliningrad
    {   73, 2527,   43890,  2, }, // Europe/Kyiv
    {   73, 6506,   43892,  4, }, // Europe/Ljubljana
    {   73, 2501,   43896,  2, }, // Europe/London
    {   73, 6365,   43898,  3, }, // Europe/Luxembourg
    {   73, 4892,   43901,  3, }, // Europe/Madrid
    {   73, 6397,   43904,  3, }, // Europe/Malta
    {   73, 6582,   43907,  3, }, // Europe/Mariehamn
    {   73, 6410,   43910,  3, }, // Europe/Monaco
    {   73, 6458,   43913,  3, }, // Europe/Oslo
    {   73, 6324,   43916,  2, }, // Europe/Rome
    {   73, 4331,   43918,  3, }, // Europe/Samara
    {   73, 6470,   43921,  4, }, // Europe/San_Marino
    {   73, 4451,   43925,  4, }, // Europe/Saratov
    {   73, 6599,   43929,  3, }, // Europe/Sofia
    {   73, 6523,   43932,  5, }, // Europe/Stockholm
    {   73, 8882,   43937,  7, }, // Europe/Ulyanovsk
    {   73, 6352,   43944,  3, }, // Europe/Vaduz
    {   73, 6567,   43947,  3, }, // Europe/Vatican
    {   73, 6231,   43950,  3, }, // Europe/Vienna
    {   73, 8899,   43953,  4, }, // Europe/Vilnius
    {   73, 4727,   43957,  5, }, // Europe/Volgograd
    {   73, 3190,   43962,  2, }, // Europe/Warsaw
    {   73, 6540,   43964,  3, }, // Europe/Zurich
    {   73, 6046,   43967,  3, }, // Indian/Christmas
    {   73, 6063,   43970,  5, }, // Indian/Cocos
    {   73, 5029,   43975,  5, }, // Indian/Comoro
    {   73, 6661,   43980,  5, }, // Indian/Kerguelen
    {   73, 7260,   43985,  3, }, // Indian/Mahe
    {   73, 7050,   43988,  4, }, // Indian/Maldives
    {   73, 5098,   43992,  4, }, // Indian/Mayotte
    {   73, 7211,   43996,  4, }, // Indian/Reunion
    {   73, 4422,   44000,  3, }, // Pacific/Apia
    {   73,  932,   44003,  3, }, // Pacific/Auckland
    {   73, 3706,   44006,  4, }, // Pacific/Bougainville
    {   73, 2237,   44010,  3, }, // Pacific/Easter
    {   73, 3902,   44013,  2, }, // Pacific/Fiji
    {   73, 6678,   44015,  6, }, // Pacific/Galapagos
    {   73, 6696,   44021,  5, }, // Pacific/Gambier
    {   73, 3822,   44026,  6, }, // Pacific/Guadalcanal
    {   73, 6018,   44032,  2, }, // Pacific/Guam
    {   73, 3049,   44034,  5, }, // Pacific/Kanton
    {   73, 3998,   44039,  6, }, // Pacific/Kiritimati
    {   73, 2842,   44045,  4, }, // Pacific/Kwajalein
    {   73, 4877,   44049,  3, }, // Pacific/Majuro
    {   73, 4051,   44052,  5, }, // Pacific/Marquesas
    {   73, 8914,   44057,  3, }, // Pacific/Midway
    {   73, 7084,   44060,  2, }, // Pacific/Nauru
    {   73, 7132,   44062,  3, }, // Pacific/Niue
    {   73, 4168,   44065,  3, }, // Pacific/Norfolk
    {   73, 7098,   44068,  3, }, // Pacific/Noumea
    {   73, 7194,   44071,  5, }, // Pacific/Pitcairn
    {   73, 3096,   44076,  3, }, // Pacific/Pohnpei
    {   73, 6076,   44079,  4, }, // Pacific/Rarotonga
    {   73, 4583,   44083,  5, }, // Pacific/Tongatapu
    {   76, 5059,    3902, 11, }, // Africa/Addis_Ababa Catalan/Latin/Spain
    {   76, 7420,   44088,  5, }, // Africa/Algiers
    {   76,   14,       0,  6, }, // Africa/Asmara
    {   76, 2320,   44093,  9, }, // Africa/Cairo
    {   76, 5166,   44102,  8, }, // Africa/El_Aaiun
    {   76, 4524,   26504,  7, }, // Africa/Khartoum
    {   76, 5113,   44110, 10, }, // Africa/Mogadishu
    {   76, 7034,   44120,  8, }, // Africa/Monrovia
    {   76, 5271,   44128,  9, }, // Africa/Ndjamena
    {   76, 4435,       6,  8, }, // Africa/Sao_Tome
    {   76, 2866,   20467,  7, }, // Africa/Tripoli
    {   76, 4554,   26559,  9, }, // America/Araguaina
    {   76,  270,   20479,  7, }, // America/Argentina/Cordoba
    {   76, 7489,   44137, 12, }, // America/Argentina/Rio_Gallegos
    {   76, 7571,   20486,  7, }, // America/Argentina/Tucuman
    {   76, 4252,      14,  8, }, // America/Asuncion
    {   76,  237,      22,  8, }, // America/Atikokan
    {   76, 7623,      30, 17, }, // America/Bahia_Banderas
    {   76, 7646,   35896,  5, }, // America/Belem
    {   76, 7660,   44149, 12, }, // America/Blanc-Sablon
    {   76, 4361,   44161,  6, }, // America/Bogota
    {   76, 3868,   33314,  6, }, // America/Cancun
    {   76, 4345,   44167,  6, }, // America/Cayenne
    {   76, 5496,   44173,  6, }, // America/Cayman
    {   76, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   76, 3791,   26574,  6, }, // America/Cuiaba
    {   76, 5723,      66,  7, }, // America/Curacao
    {   76, 7869,   26580,  8, }, // America/Eirunepe
    {   76,  481,      73, 13, }, // America/Indiana/Knox
    {   76, 7961,      86, 16, }, // America/Indiana/Marengo
    {   76, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   76, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   76, 8038,     139, 14, }, // America/Indiana/Vevay
    {   76, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   76, 8086,     171, 16, }, // America/Indiana/Winamac
    {   76, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   76, 5932,     207, 22, }, // America/Lower_Princes
    {   76, 8153,   26588,  6, }, // America/Maceio
    {   76, 5805,   20554,  9, }, // America/Martinique
    {   76, 2917,   26603,  8, }, // America/Mazatlan
    {   76, 8220,     229,  6, }, // America/Merida
    {   76, 2949,   44179, 15, }, // America/Mexico_City
    {   76, 2379,   44194,  9, }, // America/New_York
    {   76, 1850,     235, 19, }, // America/Noronha
    {   76, 8301,   44203, 23, }, // America/North_Dakota/Beulah
    {   76, 8329,   44226, 23, }, // America/North_Dakota/Center
    {   76, 8357,   44249, 26, }, // America/North_Dakota/New_Salem
    {   76,  393,     317,  4, }, // America/Nuuk
    {   76, 2356,   44275,  6, }, // America/Panama
    {   76, 8477,   26704,  8, }, // America/Santarem
    {   76, 1878,   26712,  9, }, // America/Sao_Paulo
    {   76, 6899,   44281, 12, }, // America/Scoresbysund
    {   76, 8508,   35936, 16, }, // America/St_Barthelemy
    {   76, 2061,   26737, 12, }, // America/St_Johns
    {   76, 5863,   26749, 11, }, // America/St_Kitts
    {   76, 5880,   35964, 11, }, // America/St_Lucia
    {   76,  855,   44293, 12, }, // America/St_Thomas
    {   76, 5913,   26783, 13, }, // America/St_Vincent
    {   76, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {   76, 5511,   44305,  6, }, // Asia/Anadyr
    {   76, 5523,   44311,  5, }, // Asia/Aqtau
    {   76, 5368,   44316,  6, }, // Asia/Aqtobe
    {   76,  964,   44322,  7, }, // Asia/Ashgabat
    {   76, 8624,   44329,  6, }, // Asia/Atyrau
    {   76, 3607,    4162,  6, }, // Asia/Baghdad
    {   76, 3653,   20747,  4, }, // Asia/Baku
    {   76, 3571,   44335,  7, }, // Asia/Barnaul
    {   76, 4601,   44342,  5, }, // Asia/Chita
    {   76, 4540,   20757,  6, }, // Asia/Damascus
    {   76, 6159,   44347,  8, }, // Asia/Dushanbe
    {   76, 1259,   44355, 11, }, // Asia/Ho_Chi_Minh
    {   76, 4771,   26835,  5, }, // Asia/Hovd
    {   76, 6992,   44366,  6, }, // Asia/Jayapura
    {   76, 4316,   26840,  9, }, // Asia/Kamchatka
    {   76, 1185,   20830,  8, }, // Asia/Kathmandu
    {   76,  992,   20838,  7, }, // Asia/Kolkata
    {   76, 4197,   26871, 11, }, // Asia/Krasnoyarsk
    {   76, 1211,    4321,  5, }, // Asia/Macau
    {   76, 6920,   36067,  6, }, // Asia/Muscat
    {   76, 2554,   44372,  7, }, // Asia/Nicosia
    {   76, 4135,   44379, 12, }, // Asia/Novosibirsk
    {   76, 8708,   44391,  8, }, // Asia/Qostanay
    {   76, 3584,    7787,  4, }, // Asia/Riyadh
    {   76, 4408,   44399,  8, }, // Asia/Sakhalin
    {   76, 7245,   20886, 10, }, // Asia/Samarkand
    {   76, 3279,   44407,  4, }, // Asia/Seoul
    {   76, 3300,   20900,  8, }, // Asia/Singapore
    {   76, 4297,    7817, 13, }, // Asia/Srednekolymsk
    {   76, 4781,   44411,  8, }, // Asia/Tashkent
    {   76, 2772,    7842,  7, }, // Asia/Tehran
    {   76, 2821,   44419,  6, }, // Asia/Tokyo
    {   76, 1159,   44425,  7, }, // Asia/Urumqi
    {   76, 8722,   44432,  9, }, // Asia/Ust-Nera
    {   76, 4828,   44441,  7, }, // Asia/Yakutsk
    {   76, 1235,     477,  6, }, // Asia/Yangon
    {   76, 3883,   44448, 13, }, // Asia/Yekaterinburg
    {   76, 3663,   44461,  6, }, // Atlantic/Azores
    {   76, 5671,   44467,  8, }, // Atlantic/Bermuda
    {   76, 6628,   44475, 14, }, // Atlantic/Canary
    {   76, 3727,   44489,  8, }, // Atlantic/Cape_Verde
    {   76, 1395,   44497, 11, }, // Atlantic/Faroe
    {   76, 2748,   36176,  9, }, // Atlantic/Reykjavik
    {   76, 7272,   44508, 15, }, // Atlantic/South_Georgia
    {   76, 6821,   36198, 12, }, // Atlantic/St_Helena
    {   76, 3620,   44523,  8, }, // Europe/Astrakhan
    {   76, 2300,   21046,  6, }, // Europe/Athens
    {   76, 6336,   27015,  7, }, // Europe/Belgrade
    {   76, 4757,   21059,  6, }, // Europe/Berlin
    {   76, 2167,   44531, 11, }, // Europe/Brussels
    {   76, 3928,   21073,  8, }, // Europe/Bucharest
    {   76, 6289,   44542, 11, }, // Europe/Copenhagen
    {   76, 2338,   21091,  6, }, // Europe/Dublin
    {   76, 6612,   44553,  8, }, // Europe/Helsinki
    {   76, 8804,    4492,  3, }, // Europe/Isle_of_Man
    {   76, 8929,   44561,  5, }, // Europe/Kirov
    {   76, 2527,   44566,  4, }, // Europe/Kyiv
    {   76, 3213,   21137,  6, }, // Europe/Lisbon
    {   76, 2501,   21152,  7, }, // Europe/London
    {   76, 6365,    4513,  9, }, // Europe/Luxembourg
    {   76, 6410,   44570,  6, }, // Europe/Monaco
    {   76, 3537,   44576,  6, }, // Europe/Moscow
    {   76, 4284,   21180,  5, }, // Europe/Paris
    {   76, 6275,   21185,  5, }, // Europe/Prague
    {   76, 6324,    6064,  4, }, // Europe/Rome
    {   76, 4451,   44582,  7, }, // Europe/Saratov
    {   76, 8849,   44589, 10, }, // Europe/Simferopol
    {   76, 6523,   21204,  8, }, // Europe/Stockholm
    {   76, 6202,   21213,  6, }, // Europe/Tirane
    {   76, 8882,   44599,  9, }, // Europe/Ulyanovsk
    {   76, 6567,   44608,  6, }, // Europe/Vatican
    {   76, 6231,   21230,  5, }, // Europe/Vienna
    {   76, 8899,   44614,  7, }, // Europe/Vilnius
    {   76, 3190,   44621,  8, }, // Europe/Warsaw
    {   76, 6540,   44629,  5, }, // Europe/Zurich
    {   76, 4069,   21252,  7, }, // Indian/Mauritius
    {   76, 7211,   21260,  6, }, // Indian/Reunion
    {   76, 3157,     554,  5, }, // Pacific/Chuuk
    {   76, 2237,   44634, 14, }, // Pacific/Easter
    {   76, 6678,   21277,  9, }, // Pacific/Galapagos
    {   76, 2723,    4611,  8, }, // Pacific/Honolulu
    {   76, 3049,    4619,  9, }, // Pacific/Kanton
    {   76, 4051,   44648,  9, }, // Pacific/Marquesas
    {   76, 7098,    4628,  6, }, // Pacific/Noumea
    {   76, 3096,     586,  7, }, // Pacific/Pohnpei
    {   76, 7312,   44657,  6, }, // Pacific/Tahiti
    {   80,   14,       0,  6, }, // Africa/Asmara Cebuano/Latin/Philippines
    {   80, 4435,       6,  8, }, // Africa/Sao_Tome
    {   80, 4252,      14,  8, }, // America/Asuncion
    {   80,  237,      22,  8, }, // America/Atikokan
    {   80, 7623,      30, 17, }, // America/Bahia_Banderas
    {   80, 3868,      47,  6, }, // America/Cancun
    {   80, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   80, 5723,      66,  7, }, // America/Curacao
    {   80,  481,      73, 13, }, // America/Indiana/Knox
    {   80, 7961,      86, 16, }, // America/Indiana/Marengo
    {   80, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   80, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   80, 8038,     139, 14, }, // America/Indiana/Vevay
    {   80, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   80, 8086,     171, 16, }, // America/Indiana/Winamac
    {   80, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   80, 5932,     207, 22, }, // America/Lower_Princes
    {   80, 8220,     229,  6, }, // America/Merida
    {   80, 1850,     235, 19, }, // America/Noronha
    {   80, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {   80, 8329,     274, 20, }, // America/North_Dakota/Center
    {   80, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {   80,  393,     317,  4, }, // America/Nuuk
    {   80, 6899,     321, 16, }, // America/Scoresbysund
    {   80, 8508,     337, 14, }, // America/St_Barthelemy
    {   80, 2061,     351, 10, }, // America/St_Johns
    {   80, 5863,     361,  9, }, // America/St_Kitts
    {   80, 5880,     370,  9, }, // America/St_Lucia
    {   80,  855,     379, 10, }, // America/St_Thomas
    {   80, 5913,     389, 11, }, // America/St_Vincent
    {   80, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {   80, 1259,   44663, 22, }, // Asia/Ho_Chi_Minh
    {   80, 1185,     448,  9, }, // Asia/Kathmandu
    {   80,  992,     457,  7, }, // Asia/Kolkata
    {   80, 1211,     464,  5, }, // Asia/Macau
    {   80, 1235,     477,  6, }, // Asia/Yangon
    {   80, 1395,     483,  5, }, // Atlantic/Faroe
    {   80, 6821,     488, 10, }, // Atlantic/St_Helena
    {   80, 2527,     514,  4, }, // Europe/Kyiv
    {   80, 7211,     547,  7, }, // Indian/Reunion
    {   80, 3157,     554,  5, }, // Pacific/Chuuk
    {   80, 3049,    4619,  9, }, // Pacific/Kanton
    {   80, 3096,     586,  7, }, // Pacific/Pohnpei
    {   81,   14,       0,  6, }, // Africa/Asmara Central Atlas Tamazight/Latin/Morocco
    {   81, 4435,       6,  8, }, // Africa/Sao_Tome
    {   81, 4252,      14,  8, }, // America/Asuncion
    {   81,  237,      22,  8, }, // America/Atikokan
    {   81, 7623,      30, 17, }, // America/Bahia_Banderas
    {   81, 3868,      47,  6, }, // America/Cancun
    {   81, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   81, 5723,      66,  7, }, // America/Curacao
    {   81,  481,      73, 13, }, // America/Indiana/Knox
    {   81, 7961,      86, 16, }, // America/Indiana/Marengo
    {   81, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   81, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   81, 8038,     139, 14, }, // America/Indiana/Vevay
    {   81, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   81, 8086,     171, 16, }, // America/Indiana/Winamac
    {   81, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   81, 5932,     207, 22, }, // America/Lower_Princes
    {   81, 8220,     229,  6, }, // America/Merida
    {   81, 1850,     235, 19, }, // America/Noronha
    {   81, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {   81, 8329,     274, 20, }, // America/North_Dakota/Center
    {   81, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {   81,  393,     317,  4, }, // America/Nuuk
    {   81, 6899,     321, 16, }, // America/Scoresbysund
    {   81, 8508,     337, 14, }, // America/St_Barthelemy
    {   81, 2061,     351, 10, }, // America/St_Johns
    {   81, 5863,     361,  9, }, // America/St_Kitts
    {   81, 5880,     370,  9, }, // America/St_Lucia
    {   81,  855,     379, 10, }, // America/St_Thomas
    {   81, 5913,     389, 11, }, // America/St_Vincent
    {   81, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {   81, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {   81, 1185,     448,  9, }, // Asia/Kathmandu
    {   81,  992,     457,  7, }, // Asia/Kolkata
    {   81, 1211,     464,  5, }, // Asia/Macau
    {   81, 1235,     477,  6, }, // Asia/Yangon
    {   81, 1395,     483,  5, }, // Atlantic/Faroe
    {   81, 6821,     488, 10, }, // Atlantic/St_Helena
    {   81, 2527,     514,  4, }, // Europe/Kyiv
    {   81, 7211,     547,  7, }, // Indian/Reunion
    {   81, 3157,     554,  5, }, // Pacific/Chuuk
    {   81, 3096,     586,  7, }, // Pacific/Pohnpei
    {   82,   14,       0,  6, }, // Africa/Asmara Central Kurdish/Arabic/Iraq
    {   82, 4435,       6,  8, }, // Africa/Sao_Tome
    {   82, 4252,      14,  8, }, // America/Asuncion
    {   82,  237,      22,  8, }, // America/Atikokan
    {   82, 7623,      30, 17, }, // America/Bahia_Banderas
    {   82, 3868,      47,  6, }, // America/Cancun
    {   82, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   82, 5723,      66,  7, }, // America/Curacao
    {   82,  481,      73, 13, }, // America/Indiana/Knox
    {   82, 7961,      86, 16, }, // America/Indiana/Marengo
    {   82, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   82, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   82, 8038,     139, 14, }, // America/Indiana/Vevay
    {   82, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   82, 8086,     171, 16, }, // America/Indiana/Winamac
    {   82, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   82, 5932,     207, 22, }, // America/Lower_Princes
    {   82, 8220,     229,  6, }, // America/Merida
    {   82, 1850,     235, 19, }, // America/Noronha
    {   82, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {   82, 8329,     274, 20, }, // America/North_Dakota/Center
    {   82, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {   82,  393,     317,  4, }, // America/Nuuk
    {   82, 6899,     321, 16, }, // America/Scoresbysund
    {   82, 8508,     337, 14, }, // America/St_Barthelemy
    {   82, 2061,     351, 10, }, // America/St_Johns
    {   82, 5863,     361,  9, }, // America/St_Kitts
    {   82, 5880,     370,  9, }, // America/St_Lucia
    {   82,  855,     379, 10, }, // America/St_Thomas
    {   82, 5913,     389, 11, }, // America/St_Vincent
    {   82, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {   82, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {   82, 1185,     448,  9, }, // Asia/Kathmandu
    {   82,  992,     457,  7, }, // Asia/Kolkata
    {   82, 1211,     464,  5, }, // Asia/Macau
    {   82, 1235,     477,  6, }, // Asia/Yangon
    {   82, 1395,     483,  5, }, // Atlantic/Faroe
    {   82, 6821,     488, 10, }, // Atlantic/St_Helena
    {   82, 2527,     514,  4, }, // Europe/Kyiv
    {   82, 7211,     547,  7, }, // Indian/Reunion
    {   82, 3157,     554,  5, }, // Pacific/Chuuk
    {   82, 3096,     586,  7, }, // Pacific/Pohnpei
    {   84, 6788,   44685, 16, }, // Africa/Abidjan Chakma/Chakma/Bangladesh
    {   84, 6760,   44701,  8, }, // Africa/Accra
    {   84, 5059,   44709, 21, }, // Africa/Addis_Ababa
    {   84, 7420,   44730, 20, }, // Africa/Algiers
    {   84,   14,   44750, 16, }, // Africa/Asmara
    {   84,   44,   44766, 12, }, // Africa/Bamako
    {   84, 5257,   44778, 12, }, // Africa/Bangui
    {   84, 6746,   44790, 12, }, // Africa/Banjul
    {   84, 7435,   44802,  8, }, // Africa/Bissau
    {   84, 4957,   44810, 24, }, // Africa/Blantyre
    {   84, 5287,   44834, 18, }, // Africa/Brazzaville
    {   84, 4922,   44852, 24, }, // Africa/Bujumbura
    {   84, 2320,   44876, 10, }, // Africa/Cairo
    {   84, 4117,   44886, 14, }, // Africa/Casablanca
    {   84, 7449,   44900, 10, }, // Africa/Ceuta
    {   84, 6773,   44910, 14, }, // Africa/Conakry
    {   84, 6840,   44924,  8, }, // Africa/Dakar
    {   84, 5130,   44932, 24, }, // Africa/Dar_es_Salaam
    {   84, 5043,   44956, 12, }, // Africa/Djibouti
    {   84, 5243,   44968,  8, }, // Africa/Douala
    {   84, 5166,   44976, 27, }, // Africa/El_Aaiun
    {   84, 6853,   45003, 18, }, // Africa/Freetown
    {   84, 4906,   45021, 18, }, // Africa/Gaborone
    {   84, 5015,   45039,  8, }, // Africa/Harare
    {   84, 4479,   45047, 24, }, // Africa/Johannesburg
    {   84, 4499,   45071,  6, }, // Africa/Juba
    {   84, 5151,   45077, 10, }, // Africa/Kampala
    {   84, 4524,   45087, 14, }, // Africa/Khartoum
    {   84, 4973,   45101, 10, }, // Africa/Kigali
    {   84, 5306,   45111, 12, }, // Africa/Kinshasa
    {   84, 4744,   45123, 10, }, // Africa/Lagos
    {   84, 5336,   45133, 20, }, // Africa/Libreville
    {   84, 6869,   45153,  8, }, // Africa/Lome
    {   84, 5211,   45161, 12, }, // Africa/Luanda
    {   84, 4939,   45173, 18, }, // Africa/Lubumbashi
    {   84, 5001,   45191,  8, }, // Africa/Lusaka
    {   84, 5322,   45199,  8, }, // Africa/Malabo
    {   84, 4987,   45207, 10, }, // Africa/Maputo
    {   84, 5197,   45217, 14, }, // Africa/Maseru
    {   84, 5182,   45231, 16, }, // Africa/Mbabane
    {   84, 5113,   45247, 14, }, // Africa/Mogadishu
    {   84, 7034,   45261, 18, }, // Africa/Monrovia
    {   84, 3853,   45279, 12, }, // Africa/Nairobi
    {   84, 5271,   45291, 18, }, // Africa/Ndjamena
    {   84, 5354,   45309, 10, }, // Africa/Niamey
    {   84, 6803,   45319, 18, }, // Africa/Nouakchott
    {   84, 6727,   45337, 18, }, // Africa/Ouagadougou
    {   84, 5225,   45355, 21, }, // Africa/Porto-Novo
    {   84, 4435,   45376, 15, }, // Africa/Sao_Tome
    {   84, 2866,   45391, 16, }, // Africa/Tripoli
    {   84, 6554,   45407, 16, }, // Africa/Tunis
    {   84, 4152,   45423, 20, }, // Africa/Windhoek
    {   84,  132,   45443,  8, }, // America/Adak
    {   84, 3346,   45451, 20, }, // America/Anchorage
    {   84, 5607,   45471, 20, }, // America/Anguilla
    {   84, 5624,   45491, 22, }, // America/Antigua
    {   84, 4554,   45513, 14, }, // America/Araguaina
    {   84,  166,   45527, 31, }, // America/Argentina/Buenos_Aires
    {   84,   91,   45558, 16, }, // America/Argentina/Catamarca
    {   84,  270,   45574, 14, }, // America/Argentina/Cordoba
    {   84,  441,   45588, 12, }, // America/Argentina/Jujuy
    {   84, 7462,   45600, 13, }, // America/Argentina/La_Rioja
    {   84,  565,   45613, 14, }, // America/Argentina/Mendoza
    {   84, 7489,   45627, 23, }, // America/Argentina/Rio_Gallegos
    {   84, 7520,   45650, 12, }, // America/Argentina/Salta
    {   84, 7544,   45662, 17, }, // America/Argentina/San_Juan
    {   84, 5580,   45679, 17, }, // America/Argentina/San_Luis
    {   84, 7571,   45696, 14, }, // America/Argentina/Tucuman
    {   84, 7597,   45710, 12, }, // America/Argentina/Ushuaia
    {   84, 5640,   45722, 10, }, // America/Aruba
    {   84, 4252,   45732, 22, }, // America/Asuncion
    {   84,  237,   45754, 23, }, // America/Atikokan
    {   84, 3679,   45777,  8, }, // America/Bahia
    {   84, 7623,   45785, 25, }, // America/Bahia_Banderas
    {   84, 5654,   45810, 16, }, // America/Barbados
    {   84, 7646,   45826, 12, }, // America/Belem
    {   84, 5407,   45838, 12, }, // America/Belize
    {   84, 7660,   45850, 27, }, // America/Blanc-Sablon
    {   84, 7681,   45877, 15, }, // America/Boa_Vista
    {   84, 4361,   45892, 10, }, // America/Bogota
    {   84, 7699,   45902, 12, }, // America/Boise
    {   84, 7713,   45914, 25, }, // America/Cambridge_Bay
    {   84, 7735,   45939, 25, }, // America/Campo_Grande
    {   84, 3868,   45964, 18, }, // America/Cancun
    {   84, 4694,   45982, 14, }, // America/Caracas
    {   84, 4345,   45996, 10, }, // America/Cayenne
    {   84, 5496,   46006, 10, }, // America/Cayman
    {   84, 2260,   46016, 10, }, // America/Chicago
    {   84, 7756,   46026, 16, }, // America/Chihuahua
    {   84, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   84, 5422,   46042, 16, }, // America/Costa_Rica
    {   84, 7796,   46058, 18, }, // America/Creston
    {   84, 3791,   46076, 10, }, // America/Cuiaba
    {   84, 5723,   46086, 12, }, // America/Curacao
    {   84, 7812,   46098, 30, }, // America/Danmarkshavn
    {   84, 7833,   46128, 12, }, // America/Dawson
    {   84, 7848,   46140, 25, }, // America/Dawson_Creek
    {   84,  805,   46165, 14, }, // America/Denver
    {   84, 3465,   46179, 20, }, // America/Detroit
    {   84, 5739,   46199, 14, }, // America/Dominica
    {   84,  893,   46213, 24, }, // America/Edmonton
    {   84, 7869,   46237, 16, }, // America/Eirunepe
    {   84, 5441,   46253, 25, }, // America/El_Salvador
    {   84, 7886,   46278, 29, }, // America/Fort_Nelson
    {   84, 7906,   46307, 14, }, // America/Fortaleza
    {   84, 7924,   46321, 15, }, // America/Glace_Bay
    {   84, 6881,   46336, 13, }, // America/Goose_Bay
    {   84, 4612,   46349, 31, }, // America/Grand_Turk
    {   84, 5770,   46380, 12, }, // America/Grenada
    {   84, 5786,   46392, 16, }, // America/Guadeloupe
    {   84, 3760,   46408, 12, }, // America/Guatemala
    {   84, 4841,   46420, 20, }, // America/Guayaquil
    {   84, 6932,   46440,  6, }, // America/Guyana
    {   84, 1939,   46446, 24, }, // America/Halifax
    {   84, 2281,   46470,  6, }, // America/Havana
    {   84, 7942,   46476, 22, }, // America/Hermosillo
    {   84,  348,   46498, 28, }, // America/Indiana/Indianapolis
    {   84,  481,   46526, 30, }, // America/Indiana/Knox
    {   84, 7961,   46556, 32, }, // America/Indiana/Marengo
    {   84, 7985,   46588, 42, }, // America/Indiana/Petersburg
    {   84, 8012,   46630, 34, }, // America/Indiana/Tell_City
    {   84, 8038,   46664, 30, }, // America/Indiana/Vevay
    {   84, 8060,   46694, 42, }, // America/Indiana/Vincennes
    {   84, 8086,   46736, 38, }, // America/Indiana/Winamac
    {   84, 8110,   46774, 18, }, // America/Inuvik
    {   84,  660,   46792, 20, }, // America/Iqaluit
    {   84, 2799,   46812,  8, }, // America/Jamaica
    {   84, 5380,   46820,  8, }, // America/Juneau
    {   84,  521,   46828, 18, }, // America/Kentucky/Louisville
    {   84, 8125,   46846, 36, }, // America/Kentucky/Monticello
    {   84, 5704,   46882, 26, }, // America/Kralendijk
    {   84, 4376,   46908,  9, }, // America/La_Paz
    {   84, 7169,   46917,  6, }, // America/Lima
    {   84, 3239,   46923, 31, }, // America/Los_Angeles
    {   84, 5932,   46954, 48, }, // America/Lower_Princes
    {   84, 8153,   47002, 10, }, // America/Maceio
    {   84, 8168,   47012,  8, }, // America/Managua
    {   84, 1908,   47020, 12, }, // America/Manaus
    {   84, 5897,   47032, 10, }, // America/Marigot
    {   84, 5805,   47042, 18, }, // America/Martinique
    {   84, 8184,   47060, 16, }, // America/Matamoros
    {   84, 2917,   47076, 14, }, // America/Mazatlan
    {   84, 8202,   47090, 16, }, // America/Menominee
    {   84, 8220,   47106, 10, }, // America/Merida
    {   84, 8235,   47116, 16, }, // America/Metlakatla
    {   84, 2949,   47132, 29, }, // America/Mexico_City
    {   84, 4391,   47161, 16, }, // America/Miquelon
    {   84, 8254,   47177, 16, }, // America/Moncton
    {   84, 8270,   47193, 16, }, // America/Monterrey
    {   84, 4098,   47209, 24, }, // America/Montevideo
    {   84, 5824,   47233, 22, }, // America/Montserrat
    {   84, 5481,   47255,  8, }, // America/Nassau
    {   84, 2379,   47263, 24, }, // America/New_York
    {   84, 8288,   47287,  8, }, // America/Nome
    {   84, 1850,   47295, 14, }, // America/Noronha
    {   84, 8301,   47309, 37, }, // America/North_Dakota/Beulah
    {   84, 8329,   47346, 41, }, // America/North_Dakota/Center
    {   84, 8357,   47387, 46, }, // America/North_Dakota/New_Salem
    {   84,  393,   47433,  8, }, // America/Nuuk
    {   84, 8388,   47441, 14, }, // America/Ojinaga
    {   84, 2356,   47455,  6, }, // America/Panama
    {   84, 6173,   47461, 20, }, // America/Paramaribo
    {   84, 2973,   47481, 16, }, // America/Phoenix
    {   84, 3945,   47497, 34, }, // America/Port-au-Prince
    {   84, 5954,   47531, 34, }, // America/Port_of_Spain
    {   84, 8404,   47565, 25, }, // America/Porto_Velho
    {   84, 5843,   47590, 25, }, // America/Puerto_Rico
    {   84, 8424,   47615, 35, }, // America/Rankin_Inlet
    {   84, 8445,   47650, 12, }, // America/Recife
    {   84, 1995,   47662, 10, }, // America/Regina
    {   84, 8460,   47672, 16, }, // America/Resolute
    {   84,  695,   47688, 23, }, // America/Rio_Branco
    {   84, 8477,   47711, 20, }, // America/Santarem
    {   84, 2201,   47731, 16, }, // America/Santiago
    {   84, 6111,   47747, 29, }, // America/Santo_Domingo
    {   84, 1878,   47776, 17, }, // America/Sao_Paulo
    {   84, 6899,   47793, 36, }, // America/Scoresbysund
    {   84, 8494,   47829, 10, }, // America/Sitka
    {   84, 8508,   47839, 32, }, // America/St_Barthelemy
    {   84, 2061,   47871, 25, }, // America/St_Johns
    {   84, 5863,   47896, 26, }, // America/St_Kitts
    {   84, 5880,   47922, 24, }, // America/St_Lucia
    {   84,  855,   47946, 23, }, // America/St_Thomas
    {   84, 5913,   47969, 33, }, // America/St_Vincent
    {   84, 8530,   48002, 29, }, // America/Swift_Current
    {   84, 5461,   48031, 20, }, // America/Tegucigalpa
    {   84, 5756,   48051,  8, }, // America/Thule
    {   84,  313,   48059, 12, }, // America/Tijuana
    {   84,  608,   48071, 16, }, // America/Toronto
    {   84, 5688,   48087, 14, }, // America/Tortola
    {   84, 2093,   48101, 20, }, // America/Vancouver
    {   84, 2144,   48121, 22, }, // America/Whitehorse
    {   84,  734,   48143, 20, }, // America/Winnipeg
    {   84, 8552,   48163, 16, }, // America/Yakutat
    {   84, 6001,   48179, 12, }, // Antarctica/Casey
    {   84, 6094,   48191, 12, }, // Antarctica/Davis
    {   84, 6133,   48203, 37, }, // Antarctica/DumontDUrville
    {   84, 8568,   48240, 22, }, // Antarctica/Macquarie
    {   84, 7066,   48262, 12, }, // Antarctica/Mawson
    {   84, 7113,   48274, 22, }, // Antarctica/McMurdo
    {   84, 8589,   48296, 12, }, // Antarctica/Palmer
    {   84, 7226,   48308, 10, }, // Antarctica/Rothera
    {   84, 7295,   48318,  6, }, // Antarctica/Syowa
    {   84, 8607,   48324, 12, }, // Antarctica/Troll
    {   84, 7374,   48336, 16, }, // Antarctica/Vostok
    {   84, 1429,   48352, 28, }, // Arctic/Longyearbyen
    {   84, 5570,   48380, 14, }, // Asia/Aden
    {   84, 5395,   48394, 12, }, // Asia/Almaty
    {   84, 3968,   48406, 14, }, // Asia/Amman
    {   84, 5511,   48420, 16, }, // Asia/Anadyr
    {   84, 5523,   48436, 12, }, // Asia/Aqtau
    {   84, 5368,   48448, 14, }, // Asia/Aqtobe
    {   84,  964,   48462, 16, }, // Asia/Ashgabat
    {   84, 3607,   48478, 12, }, // Asia/Baghdad
    {   84, 5534,   48490, 14, }, // Asia/Bahrain
    {   84, 3653,   46084,  6, }, // Asia/Baku
    {   84, 4466,   48504, 16, }, // Asia/Bangkok
    {   84, 3571,   48520, 14, }, // Asia/Barnaul
    {   84, 4086,   48534, 14, }, // Asia/Beirut
    {   84, 3778,   48548, 16, }, // Asia/Bishkek
    {   84, 5989,   48564, 14, }, // Asia/Brunei
    {   84, 4601,   48578,  6, }, // Asia/Chita
    {   84, 4511,   48584, 22, }, // Asia/Colombo
    {   84, 4540,   48606, 14, }, // Asia/Damascus
    {   84, 1093,   48620,  4, }, // Asia/Dhaka
    {   84, 6192,   48624,  8, }, // Asia/Dili
    {   84, 3596,   48632,  8, }, // Asia/Dubai
    {   84, 6159,   48640, 14, }, // Asia/Dushanbe
    {   84, 8651,   48654,  4, }, // Asia/Gaza
    {   84, 4795,   48658, 16, }, // Asia/Hebron
    {   84, 1259,   48674, 31, }, // Asia/Ho_Chi_Minh
    {   84, 2704,   48705, 13, }, // Asia/Hong_Kong
    {   84, 4771,   48718, 12, }, // Asia/Hovd
    {   84, 4184,   48730, 24, }, // Asia/Irkutsk
    {   84, 7006,   48754, 10, }, // Asia/Jakarta
    {   84, 6992,   48764, 12, }, // Asia/Jayapura
    {   84, 1290,   48776, 18, }, // Asia/Jerusalem
    {   84, 3560,   48794, 10, }, // Asia/Kabul
    {   84, 4316,   48804, 14, }, // Asia/Kamchatka
    {   84, 4239,   48818, 10, }, // Asia/Karachi
    {   84, 1185,   48828, 18, }, // Asia/Kathmandu
    {   84, 8661,   48846, 14, }, // Asia/Khandyga
    {   84,  992,   48860, 12, }, // Asia/Kolkata
    {   84, 4197,   48872, 28, }, // Asia/Krasnoyarsk
    {   84, 4859,   48900, 22, }, // Asia/Kuala_Lumpur
    {   84, 5976,   48922, 10, }, // Asia/Kuching
    {   84, 5547,   48932, 12, }, // Asia/Kuwait
    {   84, 1211,   48944, 12, }, // Asia/Macau
    {   84, 4017,   48956, 14, }, // Asia/Magadan
    {   84, 1349,   48970, 14, }, // Asia/Makassar
    {   84, 7182,   48984, 12, }, // Asia/Manila
    {   84, 6920,   48996, 12, }, // Asia/Muscat
    {   84, 2554,   49008, 14, }, // Asia/Nicosia
    {   84, 8675,   49022, 32, }, // Asia/Novokuznetsk
    {   84, 4135,   49054, 28, }, // Asia/Novosibirsk
    {   84, 4229,   49082, 16, }, // Asia/Omsk
    {   84, 7145,   49098, 10, }, // Asia/Oral
    {   84, 6961,   49108, 17, }, // Asia/Phnom_Penh
    {   84, 8693,   49125, 24, }, // Asia/Pontianak
    {   84, 4214,   49149, 21, }, // Asia/Pyongyang
    {   84, 5559,   49170,  8, }, // Asia/Qatar
    {   84, 4269,   49178, 18, }, // Asia/Qyzylorda
    {   84, 3584,   49196, 10, }, // Asia/Riyadh
    {   84, 4408,   49206, 12, }, // Asia/Sakhalin
    {   84, 7245,   49218, 24, }, // Asia/Samarkand
    {   84, 3279,   49242, 12, }, // Asia/Seoul
    {   84, 1053,   49254,  8, }, // Asia/Shanghai
    {   84, 3300,   49262, 18, }, // Asia/Singapore
    {   84, 4297,   49280, 40, }, // Asia/Srednekolymsk
    {   84, 3263,   49320, 12, }, // Asia/Taipei
    {   84, 4781,   49332, 18, }, // Asia/Tashkent
    {   84, 3915,   49350, 16, }, // Asia/Tbilisi
    {   84, 2772,   49366, 14, }, // Asia/Tehran
    {   84, 1317,   49380, 12, }, // Asia/Thimphu
    {   84, 2821,   49392, 12, }, // Asia/Tokyo
    {   84, 4572,   49404, 16, }, // Asia/Tomsk
    {   84, 1021,   49420, 18, }, // Asia/Ulaanbaatar
    {   84, 1159,   49438, 16, }, // Asia/Urumqi
    {   84, 8722,   49454, 19, }, // Asia/Ust-Nera
    {   84, 6977,   49473, 24, }, // Asia/Vientiane
    {   84, 4710,   49497, 26, }, // Asia/Vladivostok
    {   84, 4828,   49523, 18, }, // Asia/Yakutsk
    {   84, 1235,   49541, 16, }, // Asia/Yangon
    {   84, 3883,   49557, 32, }, // Asia/Yekaterinburg
    {   84, 3747,   49589, 16, }, // Asia/Yerevan
    {   84, 3663,   49605, 14, }, // Atlantic/Azores
    {   84, 5671,   49619, 12, }, // Atlantic/Bermuda
    {   84, 6628,   49631, 10, }, // Atlantic/Canary
    {   84, 3727,   49641, 19, }, // Atlantic/Cape_Verde
    {   84, 1395,   49660, 10, }, // Atlantic/Faroe
    {   84, 8736,   49670, 10, }, // Atlantic/Madeira
    {   84, 2748,   49680, 18, }, // Atlantic/Reykjavik
    {   84, 7272,   49698, 27, }, // Atlantic/South_Georgia
    {   84, 6821,   49725, 23, }, // Atlantic/St_Helena
    {   84, 6644,   49748, 14, }, // Atlantic/Stanley
    {   84, 1670,   49762, 20, }, // Australia/Adelaide
    {   84, 1635,   49782, 20, }, // Australia/Brisbane
    {   84, 1799,   49802, 25, }, // Australia/Broken_Hill
    {   84, 1583,   49827, 18, }, // Australia/Darwin
    {   84, 3637,   49845, 14, }, // Australia/Eucla
    {   84, 1516,   49859, 14, }, // Australia/Hobart
    {   84, 8753,   49873, 22, }, // Australia/Lindeman
    {   84, 1547,   49895, 19, }, // Australia/Lord_Howe
    {   84, 1727,   49914, 18, }, // Australia/Melbourne
    {   84, 1762,   49932, 10, }, // Australia/Perth
    {   84, 1463,   49942, 12, }, // Australia/Sydney
    {   84, 6441,   49954, 22, }, // Europe/Amsterdam
    {   84, 6216,   49976, 16, }, // Europe/Andorra
    {   84, 3620,   49992, 18, }, // Europe/Astrakhan
    {   84, 2300,   50010, 16, }, // Europe/Athens
    {   84, 6336,   50026, 20, }, // Europe/Belgrade
    {   84, 4757,   50046, 14, }, // Europe/Berlin
    {   84, 6488,   50060, 18, }, // Europe/Bratislava
    {   84, 2167,   50078, 18, }, // Europe/Brussels
    {   84, 3928,   50096, 18, }, // Europe/Bucharest
    {   84, 3806,   50114, 18, }, // Europe/Budapest
    {   84, 8772,   50132, 20, }, // Europe/Busingen
    {   84, 2583,   50152, 14, }, // Europe/Chisinau
    {   84, 6289,   50166, 24, }, // Europe/Copenhagen
    {   84, 2338,   50190, 14, }, // Europe/Dublin
    {   84, 6307,   50204, 20, }, // Europe/Gibraltar
    {   84, 8788,   50224, 16, }, // Europe/Guernsey
    {   84, 6612,   50240, 18, }, // Europe/Helsinki
    {   84, 8804,   50258, 28, }, // Europe/Isle_of_Man
    {   84, 1130,   50286, 22, }, // Europe/Istanbul
    {   84, 8823,   50308, 10, }, // Europe/Jersey
    {   84, 3979,   50318, 24, }, // Europe/Kaliningrad
    {   84, 8929,   50342, 12, }, // Europe/Kirov
    {   84, 2527,   50354, 12, }, // Europe/Kyiv
    {   84, 3213,   50366, 16, }, // Europe/Lisbon
    {   84, 6506,   50382, 16, }, // Europe/Ljubljana
    {   84, 2501,   50398, 16, }, // Europe/London
    {   84, 6365,   50414, 26, }, // Europe/Luxembourg
    {   84, 4892,   50440, 14, }, // Europe/Madrid
    {   84, 6397,   50454, 10, }, // Europe/Malta
    {   84, 6582,   50464, 22, }, // Europe/Mariehamn
    {   84, 3693,   50486, 16, }, // Europe/Minsk
    {   84, 6410,   50502, 10, }, // Europe/Monaco
    {   84, 3537,   50512, 12, }, // Europe/Moscow
    {   84, 6458,   50524, 12, }, // Europe/Oslo
    {   84, 4284,   50536, 14, }, // Europe/Paris
    {   84, 6424,   50550, 18, }, // Europe/Podgorica
    {   84, 6275,   50568, 10, }, // Europe/Prague
    {   84, 8837,   47034,  6, }, // Europe/Riga
    {   84, 6324,   50578,  8, }, // Europe/Rome
    {   84, 4331,   50586,  6, }, // Europe/Samara
    {   84, 6470,   50592, 19, }, // Europe/San_Marino
    {   84, 6245,   50611, 12, }, // Europe/Sarajevo
    {   84, 8849,   50623, 24, }, // Europe/Simferopol
    {   84, 6383,   50647, 16, }, // Europe/Skopje
    {   84, 6599,   50663, 10, }, // Europe/Sofia
    {   84, 6523,   50673, 20, }, // Europe/Stockholm
    {   84, 8867,   50693, 10, }, // Europe/Tallinn
    {   84, 6202,   50703,  8, }, // Europe/Tirane
    {   84, 8882,   50711, 26, }, // Europe/Ulyanovsk
    {   84, 6352,   50737, 10, }, // Europe/Vaduz
    {   84, 6567,   50747, 12, }, // Europe/Vatican
    {   84, 6231,   49473, 10, }, // Europe/Vienna
    {   84, 8899,   50759, 18, }, // Europe/Vilnius
    {   84, 4727,   50777, 22, }, // Europe/Volgograd
    {   84, 3190,   50799, 10, }, // Europe/Warsaw
    {   84, 6261,   50809, 14, }, // Europe/Zagreb
    {   84, 6540,   50823, 12, }, // Europe/Zurich
    {   84, 5078,   50835, 20, }, // Indian/Antananarivo
    {   84, 6947,   50855, 10, }, // Indian/Chagos
    {   84, 6046,   50865, 33, }, // Indian/Christmas
    {   84, 6063,   50898, 12, }, // Indian/Cocos
    {   84, 5029,   50910, 12, }, // Indian/Comoro
    {   84, 6661,   50922, 18, }, // Indian/Kerguelen
    {   84, 7260,   50940,  6, }, // Indian/Mahe
    {   84, 7050,   50946, 14, }, // Indian/Maldives
    {   84, 4069,   50960, 14, }, // Indian/Mauritius
    {   84, 5098,   50974, 10, }, // Indian/Mayotte
    {   84, 7211,   50984, 24, }, // Indian/Reunion
    {   84, 4422,   51008,  8, }, // Pacific/Apia
    {   84,  932,   51016, 22, }, // Pacific/Auckland
    {   84, 3706,   51038, 20, }, // Pacific/Bougainville
    {   84, 3015,   51058, 12, }, // Pacific/Chatham
    {   84, 3157,   51070, 12, }, // Pacific/Chuuk
    {   84, 2237,   51082, 14, }, // Pacific/Easter
    {   84, 7360,   51096, 10, }, // Pacific/Efate
    {   84, 7327,   51106, 16, }, // Pacific/Fakaofo
    {   84, 3902,   51122,  8, }, // Pacific/Fiji
    {   84, 7343,   51130, 14, }, // Pacific/Funafuti
    {   84, 6678,   51144, 22, }, // Pacific/Galapagos
    {   84, 6696,   51166, 18, }, // Pacific/Gambier
    {   84, 3822,   51184, 20, }, // Pacific/Guadalcanal
    {   84, 6018,   51204,  8, }, // Pacific/Guam
    {   84, 2723,   51212, 16, }, // Pacific/Honolulu
    {   84, 3049,   51228, 20, }, // Pacific/Kanton
    {   84, 3998,   51248, 18, }, // Pacific/Kiritimati
    {   84, 7019,   51266, 14, }, // Pacific/Kosrae
    {   84, 2842,   51280, 16, }, // Pacific/Kwajalein
    {   84, 4877,   51296, 10, }, // Pacific/Majuro
    {   84, 4051,   51306, 16, }, // Pacific/Marquesas
    {   84, 8914,   51322, 16, }, // Pacific/Midway
    {   84, 7084,   51338, 10, }, // Pacific/Nauru
    {   84, 7132,   51348, 12, }, // Pacific/Niue
    {   84, 4168,   51360, 16, }, // Pacific/Norfolk
    {   84, 7098,   51376, 10, }, // Pacific/Noumea
    {   84, 3126,   51386, 21, }, // Pacific/Pago_Pago
    {   84, 7155,   51407,  8, }, // Pacific/Palau
    {   84, 7194,   51415, 22, }, // Pacific/Pitcairn
    {   84, 3096,   51437, 10, }, // Pacific/Pohnpei
    {   84, 4807,   51447, 29, }, // Pacific/Port_Moresby
    {   84, 6076,   51476, 14, }, // Pacific/Rarotonga
    {   84, 6031,   51490, 10, }, // Pacific/Saipan
    {   84, 7312,   51500, 10, }, // Pacific/Tahiti
    {   84, 6712,   51510,  6, }, // Pacific/Tarawa
    {   84, 4583,   51516, 14, }, // Pacific/Tongatapu
    {   84, 7392,   51530, 12, }, // Pacific/Wake
    {   84, 7405,   51542, 12, }, // Pacific/Wallis
    {   86, 6788,   36346,  7, }, // Africa/Abidjan Chechen/Cyrillic/Russia
    {   86, 6760,     610,  5, }, // Africa/Accra
    {   86, 5059,     615, 11, }, // Africa/Addis_Ababa
    {   86, 7420,     626,  5, }, // Africa/Algiers
    {   86,   14,     631,  6, }, // Africa/Asmara
    {   86,   44,     637,  6, }, // Africa/Bamako
    {   86, 5257,     643,  5, }, // Africa/Bangui
    {   86, 6746,     648,  6, }, // Africa/Banjul
    {   86, 7435,     654,  5, }, // Africa/Bissau
    {   86, 4957,   27220,  8, }, // Africa/Blantyre
    {   86, 5287,   51554, 10, }, // Africa/Brazzaville
    {   86, 4922,     676,  9, }, // Africa/Bujumbura
    {   86, 2320,   51564,  8, }, // Africa/Cairo
    {   86, 4117,     689, 10, }, // Africa/Casablanca
    {   86, 7449,     699,  5, }, // Africa/Ceuta
    {   86, 6773,     704,  7, }, // Africa/Conakry
    {   86, 6840,     711,  5, }, // Africa/Dakar
    {   86, 5130,   27248, 12, }, // Africa/Dar_es_Salaam
    {   86, 5043,   36379,  7, }, // Africa/Djibouti
    {   86, 5243,     734,  5, }, // Africa/Douala
    {   86, 5166,   27267,  7, }, // Africa/El_Aaiun
    {   86, 6853,     746,  7, }, // Africa/Freetown
    {   86, 4906,     753,  8, }, // Africa/Gaborone
    {   86, 5015,   33913,  6, }, // Africa/Harare
    {   86, 4479,   51572, 12, }, // Africa/Johannesburg
    {   86, 4499,   27305,  5, }, // Africa/Juba
    {   86, 5151,     783,  7, }, // Africa/Kampala
    {   86, 4524,     790,  6, }, // Africa/Khartoum
    {   86, 4973,     796,  6, }, // Africa/Kigali
    {   86, 5306,     802,  7, }, // Africa/Kinshasa
    {   86, 4744,     809,  5, }, // Africa/Lagos
    {   86, 5336,   51584,  9, }, // Africa/Libreville
    {   86, 6869,     822,  4, }, // Africa/Lome
    {   86, 5211,     826,  6, }, // Africa/Luanda
    {   86, 4939,     832,  9, }, // Africa/Lubumbashi
    {   86, 5001,     841,  6, }, // Africa/Lusaka
    {   86, 5322,     847,  6, }, // Africa/Malabo
    {   86, 4987,     853,  6, }, // Africa/Maputo
    {   86, 5197,     859,  6, }, // Africa/Maseru
    {   86, 5182,     865,  7, }, // Africa/Mbabane
    {   86, 5113,     872,  8, }, // Africa/Mogadishu
    {   86, 7034,     880,  7, }, // Africa/Monrovia
    {   86, 3853,   36427,  7, }, // Africa/Nairobi
    {   86, 5271,   27386,  8, }, // Africa/Ndjamena
    {   86, 5354,   36434,  6, }, // Africa/Niamey
    {   86, 6803,     908,  7, }, // Africa/Nouakchott
    {   86, 6727,     915,  8, }, // Africa/Ouagadougou
    {   86, 5225,     923, 10, }, // Africa/Porto-Novo
    {   86, 4435,     933,  8, }, // Africa/Sao_Tome
    {   86, 2866,     941,  7, }, // Africa/Tripoli
    {   86, 6554,     948,  5, }, // Africa/Tunis
    {   86, 4152,     953,  7, }, // Africa/Windhoek
    {   86,  132,     960,  4, }, // America/Adak
    {   86, 3346,   51593,  8, }, // America/Anchorage
    {   86, 5607,   51601,  7, }, // America/Anguilla
    {   86, 5624,     978,  7, }, // America/Antigua
    {   86, 4554,     985,  9, }, // America/Araguaina
    {   86,  166,   51608, 12, }, // America/Argentina/Buenos_Aires
    {   86,   91,    1006,  9, }, // America/Argentina/Catamarca
    {   86,  270,    1015,  7, }, // America/Argentina/Cordoba
    {   86,  441,   27487,  5, }, // America/Argentina/Jujuy
    {   86, 7462,    1027,  8, }, // America/Argentina/La_Rioja
    {   86,  565,    1035,  7, }, // America/Argentina/Mendoza
    {   86, 7489,   51620, 12, }, // America/Argentina/Rio_Gallegos
    {   86, 7520,   27512,  6, }, // America/Argentina/Salta
    {   86, 7544,    1058,  8, }, // America/Argentina/San_Juan
    {   86, 5580,    1066,  8, }, // America/Argentina/San_Luis
    {   86, 7571,    1074,  7, }, // America/Argentina/Tucuman
    {   86, 7597,   27526,  5, }, // America/Argentina/Ushuaia
    {   86, 5640,    1087,  5, }, // America/Aruba
    {   86, 4252,   51632,  8, }, // America/Asuncion
    {   86,  237,    1099, 12, }, // America/Atikokan
    {   86, 3679,    1111,  3, }, // America/Bahia
    {   86, 7623,   51640, 16, }, // America/Bahia_Banderas
    {   86, 5654,    1131,  8, }, // America/Barbados
    {   86, 7646,    1139,  5, }, // America/Belem
    {   86, 5407,    1144,  5, }, // America/Belize
    {   86, 7660,    1149, 12, }, // America/Blanc-Sablon
    {   86, 7681,    1161,  9, }, // America/Boa_Vista
    {   86, 4361,    1170,  6, }, // America/Bogota
    {   86, 7699,   51656,  5, }, // America/Boise
    {   86, 7713,   51661, 13, }, // America/Cambridge_Bay
    {   86, 7735,    1193, 12, }, // America/Campo_Grande
    {   86, 3868,    1205,  6, }, // America/Cancun
    {   86, 4694,    1211,  7, }, // America/Caracas
    {   86, 4345,   51674,  7, }, // America/Cayenne
    {   86, 5496,   27629,  6, }, // America/Cayman
    {   86, 2260,    1247,  6, }, // America/Chicago
    {   86, 7756,    1253,  6, }, // America/Chihuahua
    {   86, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   86, 5422,    1259, 10, }, // America/Costa_Rica
    {   86, 7796,    1269,  7, }, // America/Creston
    {   86, 3791,   27688,  5, }, // America/Cuiaba
    {   86, 5723,   36606,  7, }, // America/Curacao
    {   86, 7812,    1290, 12, }, // America/Danmarkshavn
    {   86, 7833,    1302,  6, }, // America/Dawson
    {   86, 7848,    1308, 11, }, // America/Dawson_Creek
    {   86,  805,    1319,  6, }, // America/Denver
    {   86, 3465,   36649,  7, }, // America/Detroit
    {   86, 5739,    1332,  8, }, // America/Dominica
    {   86,  893,   51681,  8, }, // America/Edmonton
    {   86, 7869,   51689,  8, }, // America/Eirunepe
    {   86, 5441,   27765,  9, }, // America/El_Salvador
    {   86, 7886,   51697, 12, }, // America/Fort_Nelson
    {   86, 7906,    1375,  9, }, // America/Fortaleza
    {   86, 7924,   51709,  9, }, // America/Glace_Bay
    {   86, 6881,   51718,  7, }, // America/Goose_Bay
    {   86, 4612,   34278, 10, }, // America/Grand_Turk
    {   86, 5770,    1410,  7, }, // America/Grenada
    {   86, 5786,    1417,  9, }, // America/Guadeloupe
    {   86, 3760,    1426,  9, }, // America/Guatemala
    {   86, 4841,   51725,  8, }, // America/Guayaquil
    {   86, 6932,   51733,  6, }, // America/Guyana
    {   86, 1939,   51739,  8, }, // America/Halifax
    {   86, 2281,    1457,  6, }, // America/Havana
    {   86, 7942,   51747,  9, }, // America/Hermosillo
    {   86,  348,    1471, 12, }, // America/Indiana/Indianapolis
    {   86,  481,    1483,  4, }, // America/Indiana/Knox
    {   86, 7961,    1496,  7, }, // America/Indiana/Marengo
    {   86, 7985,   34365, 10, }, // America/Indiana/Petersburg
    {   86, 8012,    1531,  9, }, // America/Indiana/Tell_City
    {   86, 8038,   51756,  5, }, // America/Indiana/Vevay
    {   86, 8060,   36739,  7, }, // America/Indiana/Vincennes
    {   86, 8086,   34436,  7, }, // America/Indiana/Winamac
    {   86, 8110,    1570,  6, }, // America/Inuvik
    {   86,  660,    1576,  7, }, // America/Iqaluit
    {   86, 2799,   28011,  6, }, // America/Jamaica
    {   86, 5380,   36746,  5, }, // America/Juneau
    {   86,  521,    1594,  8, }, // America/Kentucky/Louisville
    {   86, 8125,    1602, 10, }, // America/Kentucky/Monticello
    {   86, 5704,   36751, 10, }, // America/Kralendijk
    {   86, 4376,    1632,  6, }, // America/La_Paz
    {   86, 7169,    1638,  4, }, // America/Lima
    {   86, 3239,   51761, 12, }, // America/Los_Angeles
    {   86, 5932,   51773, 21, }, // America/Lower_Princes
    {   86, 8153,   36799,  6, }, // America/Maceio
    {   86, 8168,    1677,  7, }, // America/Managua
    {   86, 1908,    1684,  6, }, // America/Manaus
    {   86, 5897,    1690,  6, }, // America/Marigot
    {   86, 5805,    1696,  9, }, // America/Martinique
    {   86, 8184,    1705,  9, }, // America/Matamoros
    {   86, 2917,    1714,  8, }, // America/Mazatlan
    {   86, 8202,    1722,  8, }, // America/Menominee
    {   86, 8220,    1730,  6, }, // America/Merida
    {   86, 8235,    1736, 10, }, // America/Metlakatla
    {   86, 2949,    1746,  6, }, // America/Mexico_City
    {   86, 4391,    1752,  7, }, // America/Miquelon
    {   86, 8254,    1759,  7, }, // America/Moncton
    {   86, 8270,   51794,  9, }, // America/Monterrey
    {   86, 4098,    1775, 10, }, // America/Montevideo
    {   86, 5824,   51803,  9, }, // America/Montserrat
    {   86, 5481,    1795,  6, }, // America/Nassau
    {   86, 2379,   51812,  8, }, // America/New_York
    {   86, 8288,    1809,  3, }, // America/Nome
    {   86, 1850,   51820,  7, }, // America/Noronha
    {   86, 8301,   51827, 24, }, // America/North_Dakota/Beulah
    {   86, 8329,   51851, 22, }, // America/North_Dakota/Center
    {   86, 8357,   51873,  9, }, // America/North_Dakota/New_Salem
    {   86,  393,   51882,  6, }, // America/Nuuk
    {   86, 8388,    1894,  7, }, // America/Ojinaga
    {   86, 2356,    1901,  6, }, // America/Panama
    {   86, 6173,    1907, 10, }, // America/Paramaribo
    {   86, 2973,    1917,  6, }, // America/Phoenix
    {   86, 3945,    1923, 12, }, // America/Port-au-Prince
    {   86, 5954,   28321, 13, }, // America/Port_of_Spain
    {   86, 8404,   28334, 11, }, // America/Porto_Velho
    {   86, 5843,   51888, 11, }, // America/Puerto_Rico
    {   86, 4030,    1970, 12, }, // America/Punta_Arenas
    {   86, 8424,    1982, 12, }, // America/Rankin_Inlet
    {   86, 8445,    1994,  6, }, // America/Recife
    {   86, 1995,   51899,  8, }, // America/Regina
    {   86, 8460,   36892,  7, }, // America/Resolute
    {   86,  695,    2015, 10, }, // America/Rio_Branco
    {   86, 8477,    2025,  8, }, // America/Santarem
    {   86, 2201,   51907,  8, }, // America/Santiago
    {   86, 6111,    2041, 13, }, // America/Santo_Domingo
    {   86, 1878,    2054,  9, }, // America/Sao_Paulo
    {   86, 6899,    2063, 11, }, // America/Scoresbysund
    {   86, 8494,    2074,  5, }, // America/Sitka
    {   86, 8508,   51915, 13, }, // America/St_Barthelemy
    {   86, 2061,   28481, 10, }, // America/St_Johns
    {   86, 5863,    2101,  9, }, // America/St_Kitts
    {   86, 5880,   51928,  9, }, // America/St_Lucia
    {   86,  855,    2121, 10, }, // America/St_Thomas
    {   86, 5913,    2131, 12, }, // America/St_Vincent
    {   86, 8530,   51937, 12, }, // America/Swift_Current
    {   86, 5461,   51949, 12, }, // America/Tegucigalpa
    {   86, 5756,    2167,  4, }, // America/Thule
    {   86,  313,    2171,  7, }, // America/Tijuana
    {   86,  608,    2178,  7, }, // America/Toronto
    {   86, 5688,    2185,  7, }, // America/Tortola
    {   86, 2093,    2192,  8, }, // America/Vancouver
    {   86, 2144,   28567,  8, }, // America/Whitehorse
    {   86,  734,    2208,  8, }, // America/Winnipeg
    {   86, 8552,   28582,  6, }, // America/Yakutat
    {   86, 6001,   37021,  5, }, // Antarctica/Casey
    {   86, 6094,   37026,  6, }, // Antarctica/Davis
    {   86, 6133,   51961, 14, }, // Antarctica/DumontDUrville
    {   86, 8568,    2249,  8, }, // Antarctica/Macquarie
    {   86, 7066,    2257,  6, }, // Antarctica/Mawson
    {   86, 7113,    2263,  9, }, // Antarctica/McMurdo
    {   86, 8589,    2272,  6, }, // Antarctica/Palmer
    {   86, 7226,    2278,  6, }, // Antarctica/Rothera
    {   86, 7295,   28642,  4, }, // Antarctica/Syowa
    {   86, 8607,   51975,  6, }, // Antarctica/Troll
    {   86, 7374,   34957,  6, }, // Antarctica/Vostok
    {   86, 1429,   51981,  7, }, // Arctic/Longyearbyen
    {   86, 5570,    2310,  4, }, // Asia/Aden
    {   86, 5395,   51988,  8, }, // Asia/Almaty
    {   86, 3968,   51996,  6, }, // Asia/Amman
    {   86, 5511,   52002,  7, }, // Asia/Anadyr
    {   86, 5523,    2331,  5, }, // Asia/Aqtau
    {   86, 5368,    2336,  6, }, // Asia/Aqtobe
    {   86,  964,    2342,  7, }, // Asia/Ashgabat
    {   86, 8624,   52009,  6, }, // Asia/Atyrau
    {   86, 3607,   52015,  7, }, // Asia/Baghdad
    {   86, 5534,   37080,  7, }, // Asia/Bahrain
    {   86, 3653,   52022,  5, }, // Asia/Baku
    {   86, 4466,    2372,  7, }, // Asia/Bangkok
    {   86, 3571,    2379,  7, }, // Asia/Barnaul
    {   86, 4086,   28695,  6, }, // Asia/Beirut
    {   86, 3778,    2392,  6, }, // Asia/Bishkek
    {   86, 5989,   28707,  6, }, // Asia/Brunei
    {   86, 4601,    2404,  4, }, // Asia/Chita
    {   86, 4511,    2408,  7, }, // Asia/Colombo
    {   86, 4540,   52027,  7, }, // Asia/Damascus
    {   86, 1093,    2421,  5, }, // Asia/Dhaka
    {   86, 6192,    2426,  4, }, // Asia/Dili
    {   86, 3596,   28728,  5, }, // Asia/Dubai
    {   86, 6159,    2435,  7, }, // Asia/Dushanbe
    {   86, 8636,    2442,  9, }, // Asia/Famagusta
    {   86, 8651,    2451,  4, }, // Asia/Gaza
    {   86, 4795,    2455,  6, }, // Asia/Hebron
    {   86, 1259,    2461,  7, }, // Asia/Ho_Chi_Minh
    {   86, 2704,   52034,  7, }, // Asia/Hong_Kong
    {   86, 4771,    2475,  4, }, // Asia/Hovd
    {   86, 4184,    2479,  7, }, // Asia/Irkutsk
    {   86, 7006,   28770,  8, }, // Asia/Jakarta
    {   86, 6992,   52041,  7, }, // Asia/Jayapura
    {   86, 1290,   52048, 11, }, // Asia/Jerusalem
    {   86, 3560,    2510,  5, }, // Asia/Kabul
    {   86, 4316,   52059, 23, }, // Asia/Kamchatka
    {   86, 4239,    2540,  6, }, // Asia/Karachi
    {   86, 1185,    2546,  8, }, // Asia/Kathmandu
    {   86, 8661,    2554,  7, }, // Asia/Khandyga
    {   86,  992,   52082,  9, }, // Asia/Kolkata
    {   86, 4197,   37110, 10, }, // Asia/Krasnoyarsk
    {   86, 4859,    2580, 12, }, // Asia/Kuala_Lumpur
    {   86, 5976,    2592,  6, }, // Asia/Kuching
    {   86, 5547,   28825,  6, }, // Asia/Kuwait
    {   86, 1211,    2604,  5, }, // Asia/Macau
    {   86, 4017,    2609,  7, }, // Asia/Magadan
    {   86, 1349,    2616,  7, }, // Asia/Makassar
    {   86, 7182,    2623,  6, }, // Asia/Manila
    {   86, 6920,    2629,  6, }, // Asia/Muscat
    {   86, 2554,    2635,  6, }, // Asia/Nicosia
    {   86, 8675,    2642, 11, }, // Asia/Novokuznetsk
    {   86, 4135,    2653, 11, }, // Asia/Novosibirsk
    {   86, 4229,    2664,  4, }, // Asia/Omsk
    {   86, 7145,   35138,  4, }, // Asia/Oral
    {   86, 6961,   52091,  8, }, // Asia/Phnom_Penh
    {   86, 8693,    2681,  9, }, // Asia/Pontianak
    {   86, 4214,   28895,  7, }, // Asia/Pyongyang
    {   86, 5559,    2697,  5, }, // Asia/Qatar
    {   86, 4269,    2710,  9, }, // Asia/Qyzylorda
    {   86, 3584,   52099,  7, }, // Asia/Riyadh
    {   86, 4408,   52106, 12, }, // Asia/Sakhalin
    {   86, 7245,    2739,  9, }, // Asia/Samarkand
    {   86, 3279,    2748,  4, }, // Asia/Seoul
    {   86, 1053,   28933,  6, }, // Asia/Shanghai
    {   86, 3300,    2758,  8, }, // Asia/Singapore
    {   86, 4297,    2766, 13, }, // Asia/Srednekolymsk
    {   86, 3263,   28961,  6, }, // Asia/Taipei
    {   86, 4781,    2785,  7, }, // Asia/Tashkent
    {   86, 3915,   35206,  7, }, // Asia/Tbilisi
    {   86, 2772,   52118,  7, }, // Asia/Tehran
    {   86, 1317,   35220,  5, }, // Asia/Thimphu
    {   86, 2821,    2810,  5, }, // Asia/Tokyo
    {   86, 4572,    2815,  5, }, // Asia/Tomsk
    {   86, 1021,    2820, 10, }, // Asia/Ulaanbaatar
    {   86, 1159,    2830,  6, }, // Asia/Urumqi
    {   86, 8722,   52125,  9, }, // Asia/Ust-Nera
    {   86, 6977,   52134,  8, }, // Asia/Vientiane
    {   86, 4710,    2851, 11, }, // Asia/Vladivostok
    {   86, 4828,   37183,  6, }, // Asia/Yakutsk
    {   86, 1235,   29043,  6, }, // Asia/Yangon
    {   86, 3883,    2875, 12, }, // Asia/Yekaterinburg
    {   86, 3747,    2887,  6, }, // Asia/Yerevan
    {   86, 3663,   52142, 16, }, // Atlantic/Azores
    {   86, 5671,   52158,  8, }, // Atlantic/Bermuda
    {   86, 6628,   52166, 17, }, // Atlantic/Canary
    {   86, 3727,    2944, 10, }, // Atlantic/Cape_Verde
    {   86, 1395,   52183, 18, }, // Atlantic/Faroe
    {   86, 8736,   52201, 13, }, // Atlantic/Madeira
    {   86, 2748,   52214,  9, }, // Atlantic/Reykjavik
    {   86, 7272,   52223, 14, }, // Atlantic/South_Georgia
    {   86, 6821,   52237, 19, }, // Atlantic/St_Helena
    {   86, 6644,   52256,  6, }, // Atlantic/Stanley
    {   86, 1670,    3035,  8, }, // Australia/Adelaide
    {   86, 1635,    3043,  7, }, // Australia/Brisbane
    {   86, 1799,    3050, 11, }, // Australia/Broken_Hill
    {   86, 1583,    3061,  6, }, // Australia/Darwin
    {   86, 3637,   29240,  4, }, // Australia/Eucla
    {   86, 1516,    3072,  6, }, // Australia/Hobart
    {   86, 8753,    3078,  8, }, // Australia/Lindeman
    {   86, 1547,   52262, 14, }, // Australia/Lord_Howe
    {   86, 1727,   29252,  8, }, // Australia/Melbourne
    {   86, 1762,    3101,  4, }, // Australia/Perth
    {   86, 1463,   52276,  6, }, // Australia/Sydney
    {   86, 6441,    3111,  9, }, // Europe/Amsterdam
    {   86, 6216,    3120,  7, }, // Europe/Andorra
    {   86, 3620,   52282,  9, }, // Europe/Astrakhan
    {   86, 2300,   52291,  5, }, // Europe/Athens
    {   86, 6336,    3139,  7, }, // Europe/Belgrade
    {   86, 4757,    3146,  6, }, // Europe/Berlin
    {   86, 6488,    3152, 10, }, // Europe/Bratislava
    {   86, 2167,   52296,  8, }, // Europe/Brussels
    {   86, 3928,    3170,  8, }, // Europe/Bucharest
    {   86, 3806,    3178,  8, }, // Europe/Budapest
    {   86, 8772,   52304, 19, }, // Europe/Busingen
    {   86, 2583,    3209,  7, }, // Europe/Chisinau
    {   86, 6289,   52323, 10, }, // Europe/Copenhagen
    {   86, 2338,    3226,  6, }, // Europe/Dublin
    {   86, 6307,    3232,  9, }, // Europe/Gibraltar
    {   86, 8788,    3241,  6, }, // Europe/Guernsey
    {   86, 6612,   52333,  9, }, // Europe/Helsinki
    {   86, 8804,   52342,  9, }, // Europe/Isle_of_Man
    {   86, 1130,    3264,  7, }, // Europe/Istanbul
    {   86, 8823,   52351,  6, }, // Europe/Jersey
    {   86, 3979,    3276, 11, }, // Europe/Kaliningrad
    {   86, 8929,    3287,  5, }, // Europe/Kirov
    {   86, 2527,    3292,  4, }, // Europe/Kyiv
    {   86, 3213,    3296,  8, }, // Europe/Lisbon
    {   86, 6506,   29432,  7, }, // Europe/Ljubljana
    {   86, 2501,    3313,  6, }, // Europe/London
    {   86, 6365,   29445, 10, }, // Europe/Luxembourg
    {   86, 4892,    3330,  6, }, // Europe/Madrid
    {   86, 6397,   29461,  6, }, // Europe/Malta
    {   86, 6582,    3341,  9, }, // Europe/Mariehamn
    {   86, 3693,    3350,  5, }, // Europe/Minsk
    {   86, 6410,    3355,  6, }, // Europe/Monaco
    {   86, 3537,    3361,  6, }, // Europe/Moscow
    {   86, 6458,    3367,  4, }, // Europe/Oslo
    {   86, 4284,    3371,  5, }, // Europe/Paris
    {   86, 6424,    3376,  9, }, // Europe/Podgorica
    {   86, 6275,    3385,  5, }, // Europe/Prague
    {   86, 8837,    3390,  4, }, // Europe/Riga
    {   86, 6324,    3394,  3, }, // Europe/Rome
    {   86, 4331,    3397,  6, }, // Europe/Samara
    {   86, 6470,    3403, 10, }, // Europe/San_Marino
    {   86, 6245,    3413,  7, }, // Europe/Sarajevo
    {   86, 4451,    3420,  7, }, // Europe/Saratov
    {   86, 8849,   52357, 11, }, // Europe/Simferopol
    {   86, 6383,   52368,  6, }, // Europe/Skopje
    {   86, 6599,    3442,  4, }, // Europe/Sofia
    {   86, 6523,   52374,  9, }, // Europe/Stockholm
    {   86, 8867,    3455,  6, }, // Europe/Tallinn
    {   86, 6202,    3461,  6, }, // Europe/Tirane
    {   86, 8882,   52383,  9, }, // Europe/Ulyanovsk
    {   86, 6352,    3476,  5, }, // Europe/Vaduz
    {   86, 6567,    3481,  7, }, // Europe/Vatican
    {   86, 6231,    3488,  4, }, // Europe/Vienna
    {   86, 8899,   52392,  7, }, // Europe/Vilnius
    {   86, 4727,    3499,  9, }, // Europe/Volgograd
    {   86, 3190,    3508,  7, }, // Europe/Warsaw
    {   86, 6261,    3515,  6, }, // Europe/Zagreb
    {   86, 6540,   37442,  5, }, // Europe/Zurich
    {   86, 5078,    3527, 12, }, // Indian/Antananarivo
    {   86, 6947,    3539,  5, }, // Indian/Chagos
    {   86, 6046,   52399, 31, }, // Indian/Christmas
    {   86, 6063,   52430, 16, }, // Indian/Cocos
    {   86, 5029,   52446, 18, }, // Indian/Comoro
    {   86, 6661,    3578,  8, }, // Indian/Kerguelen
    {   86, 7260,   29694,  3, }, // Indian/Mahe
    {   86, 7050,   52464,  9, }, // Indian/Maldives
    {   86, 4069,    3595,  7, }, // Indian/Mauritius
    {   86, 5098,   52473,  7, }, // Indian/Mayotte
    {   86, 7211,   52480,  7, }, // Indian/Reunion
    {   86, 4422,    3616,  3, }, // Pacific/Apia
    {   86,  932,    3620,  6, }, // Pacific/Auckland
    {   86, 3706,   52487, 15, }, // Pacific/Bougainville
    {   86, 3015,   52502, 11, }, // Pacific/Chatham
    {   86, 3157,   52513, 12, }, // Pacific/Chuuk
    {   86, 2237,   52525, 16, }, // Pacific/Easter
    {   86, 7360,   52541,  5, }, // Pacific/Efate
    {   86, 7327,    3659,  7, }, // Pacific/Fakaofo
    {   86, 3902,   37539,  5, }, // Pacific/Fiji
    {   86, 7343,    3670,  8, }, // Pacific/Funafuti
    {   86, 6678,   52546, 21, }, // Pacific/Galapagos
    {   86, 6696,   52567, 15, }, // Pacific/Gambier
    {   86, 3822,   35719, 11, }, // Pacific/Guadalcanal
    {   86, 6018,    3724,  4, }, // Pacific/Guam
    {   86, 2723,   52582,  8, }, // Pacific/Honolulu
    {   86, 3049,   52590, 16, }, // Pacific/Kanton
    {   86, 3998,    3734, 10, }, // Pacific/Kiritimati
    {   86, 7019,    3744,  6, }, // Pacific/Kosrae
    {   86, 2842,   29841, 10, }, // Pacific/Kwajalein
    {   86, 4877,   37574,  7, }, // Pacific/Majuro
    {   86, 4051,   52606, 16, }, // Pacific/Marquesas
    {   86, 8914,   52622, 14, }, // Pacific/Midway
    {   86, 7084,    3798,  5, }, // Pacific/Nauru
    {   86, 7132,   52636,  4, }, // Pacific/Niue
    {   86, 4168,    3807,  7, }, // Pacific/Norfolk
    {   86, 7098,    3814,  5, }, // Pacific/Noumea
    {   86, 3126,    3819,  9, }, // Pacific/Pago_Pago
    {   86, 7155,    3828,  5, }, // Pacific/Palau
    {   86, 7194,    3833,  7, }, // Pacific/Pitcairn
    {   86, 3096,   52640, 14, }, // Pacific/Pohnpei
    {   86, 4807,    3846, 11, }, // Pacific/Port_Moresby
    {   86, 6076,    3857,  9, }, // Pacific/Rarotonga
    {   86, 6031,   29935,  6, }, // Pacific/Saipan
    {   86, 7312,   52654, 13, }, // Pacific/Tahiti
    {   86, 6712,    3877,  6, }, // Pacific/Tarawa
    {   86, 4583,    3883,  9, }, // Pacific/Tongatapu
    {   86, 7392,   52667, 10, }, // Pacific/Wake
    {   86, 7405,    3896,  6, }, // Pacific/Wallis
    {   87, 6788,   52677,  4, }, // Africa/Abidjan Cherokee/Cherokee/United States
    {   87, 6760,   52681,  2, }, // Africa/Accra
    {   87, 5059,   52683,  7, }, // Africa/Addis_Ababa
    {   87, 7420,   52690,  4, }, // Africa/Algiers
    {   87,   14,   52694,  4, }, // Africa/Asmara
    {   87,   44,   52698,  3, }, // Africa/Bamako
    {   87, 5257,   52701,  4, }, // Africa/Bangui
    {   87, 6746,   52705,  4, }, // Africa/Banjul
    {   87, 7435,   52709,  3, }, // Africa/Bissau
    {   87, 4957,   52712,  3, }, // Africa/Blantyre
    {   87, 5287,   52715,  4, }, // Africa/Brazzaville
    {   87, 4922,   52719,  5, }, // Africa/Bujumbura
    {   87, 2320,   52724,  3, }, // Africa/Cairo
    {   87, 4117,   52727,  8, }, // Africa/Casablanca
    {   87, 7449,   52735,  2, }, // Africa/Ceuta
    {   87, 6773,   52737,  3, }, // Africa/Conakry
    {   87, 6840,   52740,  3, }, // Africa/Dakar
    {   87, 5130,   52743,  8, }, // Africa/Dar_es_Salaam
    {   87, 5043,   52751,  3, }, // Africa/Djibouti
    {   87, 5243,   52754,  4, }, // Africa/Douala
    {   87, 5166,   52758,  7, }, // Africa/El_Aaiun
    {   87, 6853,   52765,  7, }, // Africa/Freetown
    {   87, 4906,   52772,  4, }, // Africa/Gaborone
    {   87, 5015,   52776,  3, }, // Africa/Harare
    {   87, 4479,   52779,  6, }, // Africa/Johannesburg
    {   87, 4499,   52785,  2, }, // Africa/Juba
    {   87, 5151,   52787,  4, }, // Africa/Kampala
    {   87, 4524,   52791,  3, }, // Africa/Khartoum
    {   87, 4973,   52794,  3, }, // Africa/Kigali
    {   87, 5306,   52797,  4, }, // Africa/Kinshasa
    {   87, 4744,   52801,  3, }, // Africa/Lagos
    {   87, 5336,   52804,  4, }, // Africa/Libreville
    {   87, 6869,   52808,  2, }, // Africa/Lome
    {   87, 5211,   52810,  4, }, // Africa/Luanda
    {   87, 4939,   52814,  4, }, // Africa/Lubumbashi
    {   87, 5001,   52818,  3, }, // Africa/Lusaka
    {   87, 5322,   52821,  3, }, // Africa/Malabo
    {   87, 4987,   52824,  3, }, // Africa/Maputo
    {   87, 5197,   52827,  3, }, // Africa/Maseru
    {   87, 5182,   52830,  3, }, // Africa/Mbabane
    {   87, 5113,   52833,  4, }, // Africa/Mogadishu
    {   87, 7034,   52837,  5, }, // Africa/Monrovia
    {   87, 3853,   52842,  4, }, // Africa/Nairobi
    {   87, 5271,   52846,  4, }, // Africa/Ndjamena
    {   87, 5354,   52850,  3, }, // Africa/Niamey
    {   87, 6803,   52853,  4, }, // Africa/Nouakchott
    {   87, 6727,   52857,  5, }, // Africa/Ouagadougou
    {   87, 5225,   52862, 11, }, // Africa/Porto-Novo
    {   87, 4435,   52873,  4, }, // Africa/Sao_Tome
    {   87, 2866,   52877,  3, }, // Africa/Tripoli
    {   87, 6554,   52880,  3, }, // Africa/Tunis
    {   87, 4152,   52883,  5, }, // Africa/Windhoek
    {   87,  132,   52888,  3, }, // America/Adak
    {   87, 3346,   52891,  5, }, // America/Anchorage
    {   87, 5607,   52896,  4, }, // America/Anguilla
    {   87, 5624,   52900,  3, }, // America/Antigua
    {   87, 4554,   52903,  6, }, // America/Araguaina
    {   87,  166,   52909,  7, }, // America/Argentina/Buenos_Aires
    {   87,   91,   52916,  4, }, // America/Argentina/Catamarca
    {   87,  270,   52920,  3, }, // America/Argentina/Cordoba
    {   87,  441,   52923,  3, }, // America/Argentina/Jujuy
    {   87, 7462,   52926,  5, }, // America/Argentina/La_Rioja
    {   87,  565,   52931,  4, }, // America/Argentina/Mendoza
    {   87, 7489,   52935,  8, }, // America/Argentina/Rio_Gallegos
    {   87, 7520,   52943,  3, }, // America/Argentina/Salta
    {   87, 7544,   52946,  5, }, // America/Argentina/San_Juan
    {   87, 5580,   52951,  8, }, // America/Argentina/San_Luis
    {   87, 7571,   52959,  4, }, // America/Argentina/Tucuman
    {   87, 7597,   52963,  4, }, // America/Argentina/Ushuaia
    {   87, 5640,   52967,  3, }, // America/Aruba
    {   87, 4252,   52970,  5, }, // America/Asuncion
    {   87,  237,   52975,  5, }, // America/Atikokan
    {   87, 3679,   52980,  3, }, // America/Bahia
    {   87, 7623,   52983,  9, }, // America/Bahia_Banderas
    {   87, 5654,   52992,  4, }, // America/Barbados
    {   87, 7646,   52996,  3, }, // America/Belem
    {   87, 5407,   52999,  3, }, // America/Belize
    {   87, 7660,   53002,  6, }, // America/Blanc-Sablon
    {   87, 7681,   53008,  9, }, // America/Boa_Vista
    {   87, 4361,   53017,  3, }, // America/Bogota
    {   87, 7699,   53020,  3, }, // America/Boise
    {   87, 7713,   53023, 10, }, // America/Cambridge_Bay
    {   87, 7735,   53033,  7, }, // America/Campo_Grande
    {   87, 3868,   53040,  4, }, // America/Cancun
    {   87, 4694,   53044,  4, }, // America/Caracas
    {   87, 4345,   53048,  3, }, // America/Cayenne
    {   87, 5496,   53051,  4, }, // America/Cayman
    {   87, 2260,   53055,  3, }, // America/Chicago
    {   87, 7756,   53058,  3, }, // America/Chihuahua
    {   87, 7774,   53061,  7, }, // America/Ciudad_Juarez
    {   87, 5422,   53068,  5, }, // America/Costa_Rica
    {   87, 7796,   53073,  4, }, // America/Creston
    {   87, 3791,   53077,  3, }, // America/Cuiaba
    {   87, 5723,   53080,  4, }, // America/Curacao
    {   87, 7812,   53084,  7, }, // America/Danmarkshavn
    {   87, 7833,   53091,  3, }, // America/Dawson
    {   87, 7848,   53094,  7, }, // America/Dawson_Creek
    {   87,  805,   53101,  8, }, // America/Denver
    {   87, 3465,   53109,  3, }, // America/Detroit
    {   87, 5739,   53112,  4, }, // America/Dominica
    {   87,  893,   53116,  6, }, // America/Edmonton
    {   87, 7869,   53122,  4, }, // America/Eirunepe
    {   87, 5441,   53126,  8, }, // America/El_Salvador
    {   87, 7886,   53134,  8, }, // America/Fort_Nelson
    {   87, 7906,   53142,  4, }, // America/Fortaleza
    {   87, 7924,   53146,  8, }, // America/Glace_Bay
    {   87, 6881,   53154,  8, }, // America/Goose_Bay
    {   87, 4612,   53162,  8, }, // America/Grand_Turk
    {   87, 5770,   53170,  3, }, // America/Grenada
    {   87, 5786,   53173,  4, }, // America/Guadeloupe
    {   87, 3760,   53177,  4, }, // America/Guatemala
    {   87, 4841,   53181,  4, }, // America/Guayaquil
    {   87, 6932,   53185,  3, }, // America/Guyana
    {   87, 1939,   53188,  5, }, // America/Halifax
    {   87, 2281,   53193,  3, }, // America/Havana
    {   87, 7942,   53196,  4, }, // America/Hermosillo
    {   87,  348,   53200,  9, }, // America/Indiana/Indianapolis
    {   87,  481,   53209, 10, }, // America/Indiana/Knox
    {   87, 7961,   53219, 11, }, // America/Indiana/Marengo
    {   87, 7985,   53230, 11, }, // America/Indiana/Petersburg
    {   87, 8012,   53241, 13, }, // America/Indiana/Tell_City
    {   87, 8038,   53254,  9, }, // America/Indiana/Vevay
    {   87, 8060,   53263, 12, }, // America/Indiana/Vincennes
    {   87, 8086,   53275, 11, }, // America/Indiana/Winamac
    {   87, 8110,   53286,  4, }, // America/Inuvik
    {   87,  660,   53290,  5, }, // America/Iqaluit
    {   87, 2799,   53295,  4, }, // America/Jamaica
    {   87, 5380,   53299,  2, }, // America/Juneau
    {   87,  521,   53301,  4, }, // America/Kentucky/Louisville
    {   87, 8125,   53305, 11, }, // America/Kentucky/Monticello
    {   87, 5704,   53316,  4, }, // America/Kralendijk
    {   87, 4376,   53320,  2, }, // America/La_Paz
    {   87, 7169,   53304,  2, }, // America/Lima
    {   87, 3239,   53322, 12, }, // America/Los_Angeles
    {   87, 5932,   53334, 17, }, // America/Lower_Princes
    {   87, 8153,   53351,  3, }, // America/Maceio
    {   87, 8168,   53354,  3, }, // America/Managua
    {   87, 1908,   53357,  4, }, // America/Manaus
    {   87, 5897,   53361,  4, }, // America/Marigot
    {   87, 5805,   53365,  4, }, // America/Martinique
    {   87, 8184,   53369,  5, }, // America/Matamoros
    {   87, 2917,   53374,  4, }, // America/Mazatlan
    {   87, 8202,   53378,  4, }, // America/Menominee
    {   87, 8220,   53382,  3, }, // America/Merida
    {   87, 8235,   53385,  4, }, // America/Metlakatla
    {   87, 2949,   53389,  9, }, // America/Mexico_City
    {   87, 4391,   53398,  4, }, // America/Miquelon
    {   87, 8254,   53402,  4, }, // America/Moncton
    {   87, 8270,   53406,  3, }, // America/Monterrey
    {   87, 4098,   53409,  6, }, // America/Montevideo
    {   87, 5824,   53415,  6, }, // America/Montserrat
    {   87, 5481,   53421,  3, }, // America/Nassau
    {   87, 2379,   53424,  3, }, // America/New_York
    {   87, 8288,   53427,  2, }, // America/Nome
    {   87, 1850,   53429,  4, }, // America/Noronha
    {   87, 8301,   53433, 12, }, // America/North_Dakota/Beulah
    {   87, 8329,   53445, 12, }, // America/North_Dakota/Center
    {   87, 8357,   53457, 15, }, // America/North_Dakota/New_Salem
    {   87,  393,   53472,  2, }, // America/Nuuk
    {   87, 8388,   53474,  4, }, // America/Ojinaga
    {   87, 2356,   53478,  3, }, // America/Panama
    {   87, 6173,   53481,  5, }, // America/Paramaribo
    {   87, 2973,   53486,  6, }, // America/Phoenix
    {   87, 3945,   53492, 17, }, // America/Port-au-Prince
    {   87, 5954,   53509, 15, }, // America/Port_of_Spain
    {   87, 8404,   53524, 12, }, // America/Porto_Velho
    {   87, 5843,   53536,  7, }, // America/Puerto_Rico
    {   87, 4030,   53543,  7, }, // America/Punta_Arenas
    {   87, 8424,   53550,  9, }, // America/Rankin_Inlet
    {   87, 8445,   53559,  3, }, // America/Recife
    {   87, 1995,   53562,  3, }, // America/Regina
    {   87, 8460,   53565,  4, }, // America/Resolute
    {   87,  695,   53569,  7, }, // America/Rio_Branco
    {   87, 8477,   53576,  5, }, // America/Santarem
    {   87, 2201,   53581,  5, }, // America/Santiago
    {   87, 6111,   53586,  7, }, // America/Santo_Domingo
    {   87, 1878,   53593,  5, }, // America/Sao_Paulo
    {   87, 6899,   53598,  6, }, // America/Scoresbysund
    {   87, 8494,   53604,  3, }, // America/Sitka
    {   87, 8508,   53607, 10, }, // America/St_Barthelemy
    {   87, 2061,   53617, 11, }, // America/St_Johns
    {   87, 5863,   53628,  8, }, // America/St_Kitts
    {   87, 5880,   53636,  8, }, // America/St_Lucia
    {   87,  855,   53644,  8, }, // America/St_Thomas
    {   87, 5913,   53652,  9, }, // America/St_Vincent
    {   87, 8530,   53661,  8, }, // America/Swift_Current
    {   87, 5461,   53669,  6, }, // America/Tegucigalpa
    {   87, 5756,   53675,  2, }, // America/Thule
    {   87,  313,   53677,  4, }, // America/Tijuana
    {   87,  608,   53681,  4, }, // America/Toronto
    {   87, 5688,   53685,  3, }, // America/Tortola
    {   87, 2093,   53688,  5, }, // America/Vancouver
    {   87, 2144,   53693,  7, }, // America/Whitehorse
    {   87,  734,   53700,  4, }, // America/Winnipeg
    {   87, 8552,   53704,  4, }, // America/Yakutat
    {   87, 6001,   53038,  2, }, // Antarctica/Casey
    {   87, 6094,   53708,  3, }, // Antarctica/Davis
    {   87, 6133,   53711, 11, }, // Antarctica/DumontDUrville
    {   87, 8568,   53722,  3, }, // Antarctica/Macquarie
    {   87, 7066,   53725,  3, }, // Antarctica/Mawson
    {   87, 7113,   53728,  4, }, // Antarctica/McMurdo
    {   87, 8589,   53732,  4, }, // Antarctica/Palmer
    {   87, 7226,   53736,  3, }, // Antarctica/Rothera
    {   87, 7295,   53739,  3, }, // Antarctica/Syowa
    {   87, 8607,   53742,  2, }, // Antarctica/Troll
    {   87, 7374,   53744,  4, }, // Antarctica/Vostok
    {   87, 1429,   53748, 17, }, // Arctic/Longyearbyen
    {   87, 5570,   53765,  3, }, // Asia/Aden
    {   87, 5395,   53768,  4, }, // Asia/Almaty
    {   87, 3968,   53772,  3, }, // Asia/Amman
    {   87, 5511,   53775,  4, }, // Asia/Anadyr
    {   87, 5523,   53779,  4, }, // Asia/Aqtau
    {   87, 5368,   53783,  4, }, // Asia/Aqtobe
    {   87,  964,   53787,  5, }, // Asia/Ashgabat
    {   87, 8624,   53792,  4, }, // Asia/Atyrau
    {   87, 3607,   53796,  4, }, // Asia/Baghdad
    {   87, 5534,   53800,  5, }, // Asia/Bahrain
    {   87, 3653,   53079,  2, }, // Asia/Baku
    {   87, 4466,   53805,  5, }, // Asia/Bangkok
    {   87, 3571,   53810,  4, }, // Asia/Barnaul
    {   87, 4086,   53814,  3, }, // Asia/Beirut
    {   87, 3778,   53817,  4, }, // Asia/Bishkek
    {   87, 5989,   53821,  3, }, // Asia/Brunei
    {   87, 4601,   53824,  2, }, // Asia/Chita
    {   87, 4511,   53826,  4, }, // Asia/Colombo
    {   87, 4540,   53830,  5, }, // Asia/Damascus
    {   87, 1093,   52740,  2, }, // Asia/Dhaka
    {   87, 6192,   53777,  2, }, // Asia/Dili
    {   87, 3596,   53835,  3, }, // Asia/Dubai
    {   87, 6159,   53838,  4, }, // Asia/Dushanbe
    {   87, 8636,   53842,  5, }, // Asia/Famagusta
    {   87, 8651,   53847,  2, }, // Asia/Gaza
    {   87, 4795,   53849,  3, }, // Asia/Hebron
    {   87, 1259,   53852, 10, }, // Asia/Ho_Chi_Minh
    {   87, 2704,   53862,  7, }, // Asia/Hong_Kong
    {   87, 4771,   53869,  3, }, // Asia/Hovd
    {   87, 4184,   53872,  4, }, // Asia/Irkutsk
    {   87, 7006,   53876,  3, }, // Asia/Jakarta
    {   87, 6992,   53879,  4, }, // Asia/Jayapura
    {   87, 1290,   53883,  5, }, // Asia/Jerusalem
    {   87, 3560,   53888,  3, }, // Asia/Kabul
    {   87, 4316,   53891,  4, }, // Asia/Kamchatka
    {   87, 4239,   53895,  3, }, // Asia/Karachi
    {   87, 1185,   53898,  5, }, // Asia/Kathmandu
    {   87, 8661,   53903,  4, }, // Asia/Khandyga
    {   87,  992,   53907,  4, }, // Asia/Kolkata
    {   87, 4197,   53911,  6, }, // Asia/Krasnoyarsk
    {   87, 4859,   53917,  7, }, // Asia/Kuala_Lumpur
    {   87, 5976,   53924,  4, }, // Asia/Kuching
    {   87, 5547,   53928,  3, }, // Asia/Kuwait
    {   87, 1211,   53931,  3, }, // Asia/Macau
    {   87, 4017,   53934,  4, }, // Asia/Magadan
    {   87, 1349,   53938,  4, }, // Asia/Makassar
    {   87, 7182,   53942,  3, }, // Asia/Manila
    {   87, 6920,   53945,  4, }, // Asia/Muscat
    {   87, 2554,   53949,  4, }, // Asia/Nicosia
    {   87, 8675,   53953,  7, }, // Asia/Novokuznetsk
    {   87, 4135,   53960,  6, }, // Asia/Novosibirsk
    {   87, 4229,   53966,  4, }, // Asia/Omsk
    {   87, 7145,   53970,  3, }, // Asia/Oral
    {   87, 6961,   53973,  5, }, // Asia/Phnom_Penh
    {   87, 8693,   53978,  6, }, // Asia/Pontianak
    {   87, 4214,   53984,  7, }, // Asia/Pyongyang
    {   87, 5559,   53991,  3, }, // Asia/Qatar
    {   87, 8708,   53994,  4, }, // Asia/Qostanay
    {   87, 4269,   53998,  5, }, // Asia/Qyzylorda
    {   87, 3584,   54003,  3, }, // Asia/Riyadh
    {   87, 4408,   54006,  4, }, // Asia/Sakhalin
    {   87, 7245,   54010,  5, }, // Asia/Samarkand
    {   87, 3279,   54015,  2, }, // Asia/Seoul
    {   87, 1053,   54017,  5, }, // Asia/Shanghai
    {   87, 3300,   54022,  5, }, // Asia/Singapore
    {   87, 4297,   54027,  9, }, // Asia/Srednekolymsk
    {   87, 3263,   54036,  3, }, // Asia/Taipei
    {   87, 4781,   54039,  5, }, // Asia/Tashkent
    {   87, 3915,   54044,  4, }, // Asia/Tbilisi
    {   87, 2772,   54048,  3, }, // Asia/Tehran
    {   87, 1317,   54051,  3, }, // Asia/Thimphu
    {   87, 2821,   54054,  3, }, // Asia/Tokyo
    {   87, 4572,   54057,  4, }, // Asia/Tomsk
    {   87, 1021,   54061,  5, }, // Asia/Ulaanbaatar
    {   87, 1159,   54066,  4, }, // Asia/Urumqi
    {   87, 8722,   54070,  6, }, // Asia/Ust-Nera
    {   87, 6977,   54076,  6, }, // Asia/Vientiane
    {   87, 4710,   54082,  7, }, // Asia/Vladivostok
    {   87, 4828,   54089,  4, }, // Asia/Yakutsk
    {   87, 1235,   54093,  4, }, // Asia/Yangon
    {   87, 3883,   54097,  7, }, // Asia/Yekaterinburg
    {   87, 3747,   54104,  4, }, // Asia/Yerevan
    {   87, 3663,   54108,  4, }, // Atlantic/Azores
    {   87, 5671,   54112,  3, }, // Atlantic/Bermuda
    {   87, 6628,   54115,  3, }, // Atlantic/Canary
    {   87, 3727,   54118, 11, }, // Atlantic/Cape_Verde
    {   87, 1395,   54129,  2, }, // Atlantic/Faroe
    {   87, 8736,   54131,  3, }, // Atlantic/Madeira
    {   87, 2748,   54134,  4, }, // Atlantic/Reykjavik
    {   87, 7272,   54138,  9, }, // Atlantic/South_Georgia
    {   87, 6821,   54147,  8, }, // Atlantic/St_Helena
    {   87, 6644,   54155,  4, }, // Atlantic/Stanley
    {   87, 1670,   54159,  4, }, // Australia/Adelaide
    {   87, 1635,   54163,  4, }, // Australia/Brisbane
    {   87, 1799,   54167,  8, }, // Australia/Broken_Hill
    {   87, 1583,   54175,  3, }, // Australia/Darwin
    {   87, 3637,   54178,  2, }, // Australia/Eucla
    {   87, 1516,   54180,  3, }, // Australia/Hobart
    {   87, 8753,   54183,  5, }, // Australia/Lindeman
    {   87, 1547,   54188,  8, }, // Australia/Lord_Howe
    {   87, 1727,   54196,  4, }, // Australia/Melbourne
    {   87, 1762,   54200,  3, }, // Australia/Perth
    {   87, 1463,   54203,  3, }, // Australia/Sydney
    {   87, 6441,   54206,  6, }, // Europe/Amsterdam
    {   87, 6216,   54212,  4, }, // Europe/Andorra
    {   87, 3620,   54216,  4, }, // Europe/Astrakhan
    {   87, 2300,   54220,  4, }, // Europe/Athens
    {   87, 6336,   54224,  4, }, // Europe/Belgrade
    {   87, 4757,   54228,  3, }, // Europe/Berlin
    {   87, 6488,   54231,  5, }, // Europe/Bratislava
    {   87, 2167,   54236,  4, }, // Europe/Brussels
    {   87, 3928,   54240,  5, }, // Europe/Bucharest
    {   87, 3806,   54245,  5, }, // Europe/Budapest
    {   87, 8772,   54250,  5, }, // Europe/Busingen
    {   87, 2583,   54255,  4, }, // Europe/Chisinau
    {   87, 6289,   54259,  6, }, // Europe/Copenhagen
    {   87, 2338,   54265,  3, }, // Europe/Dublin
    {   87, 6307,   54268,  4, }, // Europe/Gibraltar
    {   87, 8788,   54272,  3, }, // Europe/Guernsey
    {   87, 6612,   54275,  5, }, // Europe/Helsinki
    {   87, 8804,   54280, 15, }, // Europe/Isle_of_Man
    {   87, 1130,   54295,  6, }, // Europe/Istanbul
    {   87, 8823,   54301,  3, }, // Europe/Jersey
    {   87, 3979,   54304,  5, }, // Europe/Kaliningrad
    {   87, 8929,   54309,  3, }, // Europe/Kirov
    {   87, 2527,   54312,  2, }, // Europe/Kyiv
    {   87, 3213,   54314,  4, }, // Europe/Lisbon
    {   87, 6506,   54318,  3, }, // Europe/Ljubljana
    {   87, 2501,   54321,  4, }, // Europe/London
    {   87, 6365,   54325,  5, }, // Europe/Luxembourg
    {   87, 4892,   54330,  3, }, // Europe/Madrid
    {   87, 6397,   54333,  3, }, // Europe/Malta
    {   87, 6582,   54336,  4, }, // Europe/Mariehamn
    {   87, 3693,   54340,  4, }, // Europe/Minsk
    {   87, 6410,   54344,  3, }, // Europe/Monaco
    {   87, 3537,   54347,  4, }, // Europe/Moscow
    {   87, 6458,   54351,  3, }, // Europe/Oslo
    {   87, 4284,   54354,  3, }, // Europe/Paris
    {   87, 6424,   54357,  4, }, // Europe/Podgorica
    {   87, 6275,   52830,  2, }, // Europe/Prague
    {   87, 8837,   53184,  2, }, // Europe/Riga
    {   87, 6324,   53827,  2, }, // Europe/Rome
    {   87, 4331,   54361,  3, }, // Europe/Samara
    {   87, 6470,   54364,  3, }, // Europe/San_Marino
    {   87, 6245,   54367,  4, }, // Europe/Sarajevo
    {   87, 4451,   54371,  4, }, // Europe/Saratov
    {   87, 8849,   54375,  6, }, // Europe/Simferopol
    {   87, 6383,   54381,  3, }, // Europe/Skopje
    {   87, 6599,   54384,  3, }, // Europe/Sofia
    {   87, 6523,   54387,  5, }, // Europe/Stockholm
    {   87, 8867,   54392,  3, }, // Europe/Tallinn
    {   87, 6202,   54395,  3, }, // Europe/Tirane
    {   87, 8882,   54398,  7, }, // Europe/Ulyanovsk
    {   87, 6352,   54405,  3, }, // Europe/Vaduz
    {   87, 6567,   54408,  5, }, // Europe/Vatican
    {   87, 6231,   54413,  3, }, // Europe/Vienna
    {   87, 8899,   54416,  5, }, // Europe/Vilnius
    {   87, 4727,   54421,  5, }, // Europe/Volgograd
    {   87, 3190,   54426,  8, }, // Europe/Warsaw
    {   87, 6261,   54434,  3, }, // Europe/Zagreb
    {   87, 6540,   54437,  3, }, // Europe/Zurich
    {   87, 5078,   54440,  7, }, // Indian/Antananarivo
    {   87, 6947,   54447,  3, }, // Indian/Chagos
    {   87, 6046,   54450,  7, }, // Indian/Christmas
    {   87, 6063,   54457,  3, }, // Indian/Cocos
    {   87, 5029,   54460,  3, }, // Indian/Comoro
    {   87, 6661,   54463,  6, }, // Indian/Kerguelen
    {   87, 7260,   54469,  2, }, // Indian/Mahe
    {   87, 7050,   54471,  5, }, // Indian/Maldives
    {   87, 4069,   54476,  5, }, // Indian/Mauritius
    {   87, 5098,   54481,  3, }, // Indian/Mayotte
    {   87, 7211,   54484,  5, }, // Indian/Reunion
    {   87, 4422,   54489,  3, }, // Pacific/Apia
    {   87,  932,   54492,  5, }, // Pacific/Auckland
    {   87, 3706,   54497,  5, }, // Pacific/Bougainville
    {   87, 3015,   54502,  3, }, // Pacific/Chatham
    {   87, 3157,   54505,  2, }, // Pacific/Chuuk
    {   87, 2237,   54507,  7, }, // Pacific/Easter
    {   87, 7360,   54514,  3, }, // Pacific/Efate
    {   87, 7327,   54517,  4, }, // Pacific/Fakaofo
    {   87, 3902,   54521,  2, }, // Pacific/Fiji
    {   87, 7343,   54523,  4, }, // Pacific/Funafuti
    {   87, 6678,   54527, 12, }, // Pacific/Galapagos
    {   87, 6696,   54539,  4, }, // Pacific/Gambier
    {   87, 3822,   54543,  6, }, // Pacific/Guadalcanal
    {   87, 6018,   54549,  2, }, // Pacific/Guam
    {   87, 2723,   54551,  4, }, // Pacific/Honolulu
    {   87, 3049,   54555,  4, }, // Pacific/Kanton
    {   87, 3998,   54559,  5, }, // Pacific/Kiritimati
    {   87, 7019,   54564,  3, }, // Pacific/Kosrae
    {   87, 2842,   54567,  5, }, // Pacific/Kwajalein
    {   87, 4877,   54572,  3, }, // Pacific/Majuro
    {   87, 4051,   54575,  5, }, // Pacific/Marquesas
    {   87, 8914,   54580,  5, }, // Pacific/Midway
    {   87, 7084,   54585,  3, }, // Pacific/Nauru
    {   87, 7132,   54177,  2, }, // Pacific/Niue
    {   87, 4168,   54588,  5, }, // Pacific/Norfolk
    {   87, 7098,   54593,  4, }, // Pacific/Noumea
    {   87, 3126,   54597,  5, }, // Pacific/Pago_Pago
    {   87, 7155,   54602,  4, }, // Pacific/Palau
    {   87, 7194,   54606,  5, }, // Pacific/Pitcairn
    {   87, 3096,   54611,  3, }, // Pacific/Pohnpei
    {   87, 4807,   54614, 13, }, // Pacific/Port_Moresby
    {   87, 6076,   54627,  5, }, // Pacific/Rarotonga
    {   87, 6031,   54632,  4, }, // Pacific/Saipan
    {   87, 7312,   54636,  3, }, // Pacific/Tahiti
    {   87, 6712,   54639,  3, }, // Pacific/Tarawa
    {   87, 4583,   54642,  5, }, // Pacific/Tongatapu
    {   87, 7392,   54647,  3, }, // Pacific/Wake
    {   87, 7405,   54650,  3, }, // Pacific/Wallis
    {   88,   14,       0,  6, }, // Africa/Asmara Chickasaw/Latin/United States
    {   88, 4435,       6,  8, }, // Africa/Sao_Tome
    {   88, 4252,      14,  8, }, // America/Asuncion
    {   88,  237,      22,  8, }, // America/Atikokan
    {   88, 7623,      30, 17, }, // America/Bahia_Banderas
    {   88, 3868,      47,  6, }, // America/Cancun
    {   88, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   88, 5723,      66,  7, }, // America/Curacao
    {   88,  481,      73, 13, }, // America/Indiana/Knox
    {   88, 7961,      86, 16, }, // America/Indiana/Marengo
    {   88, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   88, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   88, 8038,     139, 14, }, // America/Indiana/Vevay
    {   88, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   88, 8086,     171, 16, }, // America/Indiana/Winamac
    {   88, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   88, 5932,     207, 22, }, // America/Lower_Princes
    {   88, 8220,     229,  6, }, // America/Merida
    {   88, 1850,     235, 19, }, // America/Noronha
    {   88, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {   88, 8329,     274, 20, }, // America/North_Dakota/Center
    {   88, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {   88,  393,     317,  4, }, // America/Nuuk
    {   88, 6899,     321, 16, }, // America/Scoresbysund
    {   88, 8508,     337, 14, }, // America/St_Barthelemy
    {   88, 2061,     351, 10, }, // America/St_Johns
    {   88, 5863,     361,  9, }, // America/St_Kitts
    {   88, 5880,     370,  9, }, // America/St_Lucia
    {   88,  855,     379, 10, }, // America/St_Thomas
    {   88, 5913,     389, 11, }, // America/St_Vincent
    {   88, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {   88, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {   88, 1185,     448,  9, }, // Asia/Kathmandu
    {   88,  992,     457,  7, }, // Asia/Kolkata
    {   88, 1211,     464,  5, }, // Asia/Macau
    {   88, 1235,     477,  6, }, // Asia/Yangon
    {   88, 1395,     483,  5, }, // Atlantic/Faroe
    {   88, 6821,     488, 10, }, // Atlantic/St_Helena
    {   88, 2527,     514,  4, }, // Europe/Kyiv
    {   88, 7211,     547,  7, }, // Indian/Reunion
    {   88, 3157,     554,  5, }, // Pacific/Chuuk
    {   88, 3096,     586,  7, }, // Pacific/Pohnpei
    {   89,   14,       0,  6, }, // Africa/Asmara Chiga/Latin/Uganda
    {   89, 4435,       6,  8, }, // Africa/Sao_Tome
    {   89, 4252,      14,  8, }, // America/Asuncion
    {   89,  237,      22,  8, }, // America/Atikokan
    {   89, 7623,      30, 17, }, // America/Bahia_Banderas
    {   89, 3868,      47,  6, }, // America/Cancun
    {   89, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   89, 5723,      66,  7, }, // America/Curacao
    {   89,  481,      73, 13, }, // America/Indiana/Knox
    {   89, 7961,      86, 16, }, // America/Indiana/Marengo
    {   89, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   89, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   89, 8038,     139, 14, }, // America/Indiana/Vevay
    {   89, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   89, 8086,     171, 16, }, // America/Indiana/Winamac
    {   89, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   89, 5932,     207, 22, }, // America/Lower_Princes
    {   89, 8220,     229,  6, }, // America/Merida
    {   89, 1850,     235, 19, }, // America/Noronha
    {   89, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {   89, 8329,     274, 20, }, // America/North_Dakota/Center
    {   89, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {   89,  393,     317,  4, }, // America/Nuuk
    {   89, 6899,     321, 16, }, // America/Scoresbysund
    {   89, 8508,     337, 14, }, // America/St_Barthelemy
    {   89, 2061,     351, 10, }, // America/St_Johns
    {   89, 5863,     361,  9, }, // America/St_Kitts
    {   89, 5880,     370,  9, }, // America/St_Lucia
    {   89,  855,     379, 10, }, // America/St_Thomas
    {   89, 5913,     389, 11, }, // America/St_Vincent
    {   89, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {   89, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {   89, 1185,     448,  9, }, // Asia/Kathmandu
    {   89,  992,     457,  7, }, // Asia/Kolkata
    {   89, 1211,     464,  5, }, // Asia/Macau
    {   89, 1235,     477,  6, }, // Asia/Yangon
    {   89, 1395,     483,  5, }, // Atlantic/Faroe
    {   89, 6821,     488, 10, }, // Atlantic/St_Helena
    {   89, 2527,     514,  4, }, // Europe/Kyiv
    {   89, 7211,     547,  7, }, // Indian/Reunion
    {   89, 3157,     554,  5, }, // Pacific/Chuuk
    {   89, 3096,     586,  7, }, // Pacific/Pohnpei
    {   90, 6788,   43081,  3, }, // Africa/Abidjan Chinese/Simplified Han/China
    {   90, 6760,   41570,  3, }, // Africa/Accra
    {   90, 5059,   54653,  6, }, // Africa/Addis_Ababa
    {   90, 7420,   43090,  4, }, // Africa/Algiers
    {   90,   14,   54659,  4, }, // Africa/Asmara
    {   90,   44,   43098,  3, }, // Africa/Bamako
    {   90, 5257,   41590,  2, }, // Africa/Bangui
    {   90, 6746,   54663,  3, }, // Africa/Banjul
    {   90, 7435,   43101,  2, }, // Africa/Bissau
    {   90, 4957,   43103,  4, }, // Africa/Blantyre
    {   90, 5287,   43107,  5, }, // Africa/Brazzaville
    {   90, 4922,   54666,  4, }, // Africa/Bujumbura
    {   90, 2320,   43112,  2, }, // Africa/Cairo
    {   90, 4117,   43114,  5, }, // Africa/Casablanca
    {   90, 7449,   43119,  2, }, // Africa/Ceuta
    {   90, 6773,   54670,  4, }, // Africa/Conakry
    {   90, 6840,   43121,  3, }, // Africa/Dakar
    {   90, 5130,   54674,  6, }, // Africa/Dar_es_Salaam
    {   90, 5043,   54680,  3, }, // Africa/Djibouti
    {   90, 5243,   41631,  3, }, // Africa/Douala
    {   90, 5166,   41634,  3, }, // Africa/El_Aaiun
    {   90, 6853,   54683,  3, }, // Africa/Freetown
    {   90, 4906,   54686,  4, }, // Africa/Gaborone
    {   90, 5015,   41644,  3, }, // Africa/Harare
    {   90, 4479,   54690,  5, }, // Africa/Johannesburg
    {   90, 4499,   41652,  2, }, // Africa/Juba
    {   90, 5151,   41654,  3, }, // Africa/Kampala
    {   90, 4524,   41657,  3, }, // Africa/Khartoum
    {   90, 4973,   41660,  3, }, // Africa/Kigali
    {   90, 5306,   54695,  3, }, // Africa/Kinshasa
    {   90, 4744,   54698,  3, }, // Africa/Lagos
    {   90, 5336,   54701,  4, }, // Africa/Libreville
    {   90, 6869,   41672,  2, }, // Africa/Lome
    {   90, 5211,   43132,  3, }, // Africa/Luanda
    {   90, 4939,   43135,  4, }, // Africa/Lubumbashi
    {   90, 5001,   54705,  3, }, // Africa/Lusaka
    {   90, 5322,   43139,  3, }, // Africa/Malabo
    {   90, 4987,   43142,  3, }, // Africa/Maputo
    {   90, 5197,   54708,  3, }, // Africa/Maseru
    {   90, 5182,   54711,  4, }, // Africa/Mbabane
    {   90, 5113,   54715,  4, }, // Africa/Mogadishu
    {   90, 7034,   43148,  4, }, // Africa/Monrovia
    {   90, 3853,   54719,  3, }, // Africa/Nairobi
    {   90, 5271,   43152,  4, }, // Africa/Ndjamena
    {   90, 5354,   43156,  3, }, // Africa/Niamey
    {   90, 6803,   54722,  5, }, // Africa/Nouakchott
    {   90, 6727,   41717,  4, }, // Africa/Ouagadougou
    {   90, 5225,   54727,  4, }, // Africa/Porto-Novo
    {   90, 4435,   43166,  3, }, // Africa/Sao_Tome
    {   90, 2866,   41728,  4, }, // Africa/Tripoli
    {   90, 6554,   41732,  3, }, // Africa/Tunis
    {   90, 4152,   43169,  4, }, // Africa/Windhoek
    {   90,  132,   54731,  3, }, // America/Adak
    {   90, 3346,   54734,  4, }, // America/Anchorage
    {   90, 5607,   54738,  3, }, // America/Anguilla
    {   90, 5624,   54741,  3, }, // America/Antigua
    {   90, 4554,   54744,  5, }, // America/Araguaina
    {   90,  166,   43176,  7, }, // America/Argentina/Buenos_Aires
    {   90,   91,   43183,  4, }, // America/Argentina/Catamarca
    {   90,  270,   54749,  4, }, // America/Argentina/Cordoba
    {   90,  441,   41770,  3, }, // America/Argentina/Jujuy
    {   90, 7462,   54753,  4, }, // America/Argentina/La_Rioja
    {   90,  565,   43190,  3, }, // America/Argentina/Mendoza
    {   90, 7489,   43193,  6, }, // America/Argentina/Rio_Gallegos
    {   90, 7520,   43199,  3, }, // America/Argentina/Salta
    {   90, 7544,   43202,  3, }, // America/Argentina/San_Juan
    {   90, 5580,   54757,  4, }, // America/Argentina/San_Luis
    {   90, 7571,   54761,  3, }, // America/Argentina/Tucuman
    {   90, 7597,   43211,  4, }, // America/Argentina/Ushuaia
    {   90, 5640,   54764,  3, }, // America/Aruba
    {   90, 4252,   43215,  3, }, // America/Asuncion
    {   90,  237,   41807,  4, }, // America/Atikokan
    {   90, 3679,   43218,  3, }, // America/Bahia
    {   90, 7623,   43218,  7, }, // America/Bahia_Banderas
    {   90, 5654,   54767,  4, }, // America/Barbados
    {   90, 7646,   43228,  2, }, // America/Belem
    {   90, 5407,   54771,  3, }, // America/Belize
    {   90, 7660,   54774,  6, }, // America/Blanc-Sablon
    {   90, 7681,   54780,  5, }, // America/Boa_Vista
    {   90, 4361,   41838,  3, }, // America/Bogota
    {   90, 7699,   54785,  3, }, // America/Boise
    {   90, 7713,   43242,  3, }, // America/Cambridge_Bay
    {   90, 7735,   54788,  3, }, // America/Campo_Grande
    {   90, 3868,   41849,  2, }, // America/Cancun
    {   90, 4694,   54791,  4, }, // America/Caracas
    {   90, 4345,   54795,  2, }, // America/Cayenne
    {   90, 5496,   43250,  2, }, // America/Cayman
    {   90, 2260,   41861,  3, }, // America/Chicago
    {   90, 7756,   54797,  3, }, // America/Chihuahua
    {   90, 7774,   43257,  4, }, // America/Ciudad_Juarez
    {   90, 5422,   54800,  5, }, // America/Costa_Rica
    {   90, 7796,   43261,  4, }, // America/Creston
    {   90, 3791,   54805,  3, }, // America/Cuiaba
    {   90, 5723,   43265,  3, }, // America/Curacao
    {   90, 7812,   43268,  4, }, // America/Danmarkshavn
    {   90, 7833,   41890,  2, }, // America/Dawson
    {   90, 7848,   41892,  5, }, // America/Dawson_Creek
    {   90,  805,   41897,  2, }, // America/Denver
    {   90, 3465,   41899,  3, }, // America/Detroit
    {   90, 5739,   54808,  4, }, // America/Dominica
    {   90,  893,   54812,  4, }, // America/Edmonton
    {   90, 7869,   54816,  4, }, // America/Eirunepe
    {   90, 5441,   43276,  4, }, // America/El_Salvador
    {   90, 7886,   43280,  4, }, // America/Fort_Nelson
    {   90, 7906,   54820,  4, }, // America/Fortaleza
    {   90, 7924,   54824,  4, }, // America/Glace_Bay
    {   90, 6881,   54828,  3, }, // America/Goose_Bay
    {   90, 4612,   41932,  3, }, // America/Grand_Turk
    {   90, 5770,   54831,  4, }, // America/Grenada
    {   90, 5786,   54835,  4, }, // America/Guadeloupe
    {   90, 3760,   54839,  4, }, // America/Guatemala
    {   90, 4841,   43302,  4, }, // America/Guayaquil
    {   90, 6932,   54843,  3, }, // America/Guyana
    {   90, 1939,   54846,  5, }, // America/Halifax
    {   90, 2281,   41960,  3, }, // America/Havana
    {   90, 7942,   43309,  4, }, // America/Hermosillo
    {   90,  348,   54851,  7, }, // America/Indiana/Indianapolis
    {   90,  481,   54858,  8, }, // America/Indiana/Knox
    {   90, 7961,   54866,  8, }, // America/Indiana/Marengo
    {   90, 7985,   54874,  9, }, // America/Indiana/Petersburg
    {   90, 8012,   54883,  8, }, // America/Indiana/Tell_City
    {   90, 8038,   54891,  8, }, // America/Indiana/Vevay
    {   90, 8060,   54899,  8, }, // America/Indiana/Vincennes
    {   90, 8086,   54907,  9, }, // America/Indiana/Winamac
    {   90, 8110,   54916,  4, }, // America/Inuvik
    {   90,  660,   42034,  3, }, // America/Iqaluit
    {   90, 2799,   43365,  3, }, // America/Jamaica
    {   90, 5380,   43368,  2, }, // America/Juneau
    {   90,  521,   43370,  5, }, // America/Kentucky/Louisville
    {   90, 8125,   54920,  8, }, // America/Kentucky/Monticello
    {   90, 5704,   43383,  5, }, // America/Kralendijk
    {   90, 4376,   42060,  3, }, // America/La_Paz
    {   90, 7169,   43388,  2, }, // America/Lima
    {   90, 3239,   43390,  3, }, // America/Los_Angeles
    {   90, 5932,   43393,  4, }, // America/Lower_Princes
    {   90, 8153,   54928,  3, }, // America/Maceio
    {   90, 8168,   54931,  3, }, // America/Managua
    {   90, 1908,   54934,  3, }, // America/Manaus
    {   90, 5897,   43406,  4, }, // America/Marigot
    {   90, 5805,   54937,  4, }, // America/Martinique
    {   90, 8184,   43414,  5, }, // America/Matamoros
    {   90, 2917,   43419,  4, }, // America/Mazatlan
    {   90, 8202,   54941,  4, }, // America/Menominee
    {   90, 8220,   43427,  3, }, // America/Merida
    {   90, 8235,   42105,  6, }, // America/Metlakatla
    {   90, 2949,   54945,  4, }, // America/Mexico_City
    {   90, 4391,   54949,  3, }, // America/Miquelon
    {   90, 8254,   43433,  3, }, // America/Moncton
    {   90, 8270,   54952,  3, }, // America/Monterrey
    {   90, 4098,   54955,  5, }, // America/Montevideo
    {   90, 5824,   54960,  5, }, // America/Montserrat
    {   90, 5481,   54965,  2, }, // America/Nassau
    {   90, 2379,   43443,  2, }, // America/New_York
    {   90, 8288,   43445,  2, }, // America/Nome
    {   90, 1850,   54967,  4, }, // America/Noronha
    {   90, 8301,   54971,  8, }, // America/North_Dakota/Beulah
    {   90, 8329,   54979,  7, }, // America/North_Dakota/Center
    {   90, 8357,   54986,  9, }, // America/North_Dakota/New_Salem
    {   90,  393,   42163,  2, }, // America/Nuuk
    {   90, 8388,   43473,  4, }, // America/Ojinaga
    {   90, 2356,   43477,  3, }, // America/Panama
    {   90, 6173,   54995,  5, }, // America/Paramaribo
    {   90, 2973,   43485,  3, }, // America/Phoenix
    {   90, 3945,   42180,  3, }, // America/Port-au-Prince
    {   90, 5954,   42183,  4, }, // America/Port_of_Spain
    {   90, 8404,   55000,  4, }, // America/Porto_Velho
    {   90, 5843,   42190,  4, }, // America/Puerto_Rico
    {   90, 4030,   43491,  6, }, // America/Punta_Arenas
    {   90, 8424,   43497,  3, }, // America/Rankin_Inlet
    {   90, 8445,   55004,  3, }, // America/Recife
    {   90, 1995,   43500,  3, }, // America/Regina
    {   90, 8460,   55007,  4, }, // America/Resolute
    {   90,  695,   55011,  5, }, // America/Rio_Branco
    {   90, 8477,   43511,  3, }, // America/Santarem
    {   90, 2201,   55016,  4, }, // America/Santiago
    {   90, 6111,   55020,  4, }, // America/Santo_Domingo
    {   90, 1878,   43522,  3, }, // America/Sao_Paulo
    {   90, 6899,   55024,  7, }, // America/Scoresbysund
    {   90, 8494,   43532,  3, }, // America/Sitka
    {   90, 8508,   55031,  6, }, // America/St_Barthelemy
    {   90, 2061,   55037,  4, }, // America/St_Johns
    {   90, 5863,   43544,  3, }, // America/St_Kitts
    {   90, 5880,   55041,  4, }, // America/St_Lucia
    {   90,  855,   43551,  4, }, // America/St_Thomas
    {   90, 5913,   55045,  4, }, // America/St_Vincent
    {   90, 8530,   43558,  7, }, // America/Swift_Current
    {   90, 5461,   55049,  6, }, // America/Tegucigalpa
    {   90, 5756,   55055,  2, }, // America/Thule
    {   90,  313,   55057,  3, }, // America/Tijuana
    {   90,  608,   43568,  3, }, // America/Toronto
    {   90, 5688,   43571,  4, }, // America/Tortola
    {   90, 2093,   43575,  3, }, // America/Vancouver
    {   90, 2144,   43578,  4, }, // America/Whitehorse
    {   90,  734,   43582,  3, }, // America/Winnipeg
    {   90, 8552,   55060,  4, }, // America/Yakutat
    {   90, 6001,   55064,  2, }, // Antarctica/Casey
    {   90, 6094,   43590,  3, }, // Antarctica/Davis
    {   90, 6133,   55066,  6, }, // Antarctica/DumontDUrville
    {   90, 8568,   55072,  3, }, // Antarctica/Macquarie
    {   90, 7066,   42314,  2, }, // Antarctica/Mawson
    {   90, 7113,   43601,  4, }, // Antarctica/McMurdo
    {   90, 8589,   55075,  3, }, // Antarctica/Palmer
    {   90, 7226,   43607,  3, }, // Antarctica/Rothera
    {   90, 7295,   42325,  2, }, // Antarctica/Syowa
    {   90, 8607,   55078,  3, }, // Antarctica/Troll
    {   90, 7374,   42331,  4, }, // Antarctica/Vostok
    {   90, 1429,   55081,  4, }, // Arctic/Longyearbyen
    {   90, 5570,   43612,  2, }, // Asia/Aden
    {   90, 5395,   43614,  4, }, // Asia/Almaty
    {   90, 3968,   42346,  2, }, // Asia/Amman
    {   90, 5511,   55085,  4, }, // Asia/Anadyr
    {   90, 5523,   42351,  3, }, // Asia/Aqtau
    {   90, 5368,   55089,  4, }, // Asia/Aqtobe
    {   90,  964,   55093,  5, }, // Asia/Ashgabat
    {   90, 8624,   43618,  3, }, // Asia/Atyrau
    {   90, 3607,   43621,  3, }, // Asia/Baghdad
    {   90, 5534,   42369,  2, }, // Asia/Bahrain
    {   90, 3653,   43624,  2, }, // Asia/Baku
    {   90, 4466,   42371,  2, }, // Asia/Bangkok
    {   90, 3571,   43626,  4, }, // Asia/Barnaul
    {   90, 4086,   43630,  3, }, // Asia/Beirut
    {   90, 3778,   43633,  4, }, // Asia/Bishkek
    {   90, 5989,   55098,  2, }, // Asia/Brunei
    {   90, 4601,   42386,  2, }, // Asia/Chita
    {   90, 4511,   55100,  3, }, // Asia/Colombo
    {   90, 4540,   43642,  4, }, // Asia/Damascus
    {   90, 1093,   43646,  2, }, // Asia/Dhaka
    {   90, 6192,   42397,  2, }, // Asia/Dili
    {   90, 3596,   55103,  2, }, // Asia/Dubai
    {   90, 6159,   55105,  3, }, // Asia/Dushanbe
    {   90, 8636,   43651,  5, }, // Asia/Famagusta
    {   90, 8651,   55108,  2, }, // Asia/Gaza
    {   90, 4795,   55110,  3, }, // Asia/Hebron
    {   90, 1259,   42414,  4, }, // Asia/Ho_Chi_Minh
    {   90, 2704,   42425,  2, }, // Asia/Hong_Kong
    {   90, 4771,   42432,  3, }, // Asia/Hovd
    {   90, 4184,   55113,  5, }, // Asia/Irkutsk
    {   90, 7006,   43677,  3, }, // Asia/Jakarta
    {   90, 6992,   55118,  4, }, // Asia/Jayapura
    {   90, 1290,   42447,  4, }, // Asia/Jerusalem
    {   90, 3560,   43684,  3, }, // Asia/Kabul
    {   90, 4316,   42454,  3, }, // Asia/Kamchatka
    {   90, 4239,   55122,  3, }, // Asia/Karachi
    {   90, 1185,   43687,  4, }, // Asia/Kathmandu
    {   90, 8661,   55125,  3, }, // Asia/Khandyga
    {   90,  992,   43691,  4, }, // Asia/Kolkata
    {   90, 4197,   43695,  8, }, // Asia/Krasnoyarsk
    {   90, 4859,   42479,  3, }, // Asia/Kuala_Lumpur
    {   90, 5976,   43703,  2, }, // Asia/Kuching
    {   90, 5547,   42484,  3, }, // Asia/Kuwait
    {   90, 1211,   43712,  2, }, // Asia/Macau
    {   90, 4017,   43719,  3, }, // Asia/Magadan
    {   90, 1349,   55128,  3, }, // Asia/Makassar
    {   90, 7182,   43726,  3, }, // Asia/Manila
    {   90, 6920,   55131,  4, }, // Asia/Muscat
    {   90, 2554,   55135,  4, }, // Asia/Nicosia
    {   90, 8675,   43737,  6, }, // Asia/Novokuznetsk
    {   90, 4135,   43743,  5, }, // Asia/Novosibirsk
    {   90, 4229,   42530,  4, }, // Asia/Omsk
    {   90, 7145,   43748,  3, }, // Asia/Oral
    {   90, 6961,   43751,  2, }, // Asia/Phnom_Penh
    {   90, 8693,   42539,  2, }, // Asia/Pontianak
    {   90, 4214,   42541,  2, }, // Asia/Pyongyang
    {   90, 5559,   55139,  3, }, // Asia/Qatar
    {   90, 8708,   55142,  4, }, // Asia/Qostanay
    {   90, 4269,   55146,  5, }, // Asia/Qyzylorda
    {   90, 3584,   55151,  3, }, // Asia/Riyadh
    {   90, 4408,   55154,  3, }, // Asia/Sakhalin
    {   90, 7245,   43764,  4, }, // Asia/Samarkand
    {   90, 3279,   43768,  2, }, // Asia/Seoul
    {   90, 1053,   42567,  2, }, // Asia/Shanghai
    {   90, 3300,   42569,  3, }, // Asia/Singapore
    {   90, 4297,   42572,  6, }, // Asia/Srednekolymsk
    {   90, 3263,   42578,  2, }, // Asia/Taipei
    {   90, 4781,   42580,  3, }, // Asia/Tashkent
    {   90, 3915,   42583,  4, }, // Asia/Tbilisi
    {   90, 2772,   43770,  3, }, // Asia/Tehran
    {   90, 1317,   42590,  2, }, // Asia/Thimphu
    {   90, 2821,   43773,  2, }, // Asia/Tokyo
    {   90, 4572,   42594,  4, }, // Asia/Tomsk
    {   90, 1021,   43775,  4, }, // Asia/Ulaanbaatar
    {   90, 1159,   43779,  4, }, // Asia/Urumqi
    {   90, 8722,   43783,  4, }, // Asia/Ust-Nera
    {   90, 6977,   55157,  2, }, // Asia/Vientiane
    {   90, 4710,   55159,  7, }, // Asia/Vladivostok
    {   90, 4828,   55166,  4, }, // Asia/Yakutsk
    {   90, 1235,   42619,  2, }, // Asia/Yangon
    {   90, 3883,   55170,  5, }, // Asia/Yekaterinburg
    {   90, 3747,   55175,  3, }, // Asia/Yerevan
    {   90, 3663,   43802,  5, }, // Atlantic/Azores
    {   90, 5671,   55178,  3, }, // Atlantic/Bermuda
    {   90, 6628,   55181,  3, }, // Atlantic/Canary
    {   90, 3727,   55184,  3, }, // Atlantic/Cape_Verde
    {   90, 1395,   43816,  2, }, // Atlantic/Faroe
    {   90, 8736,   55187,  3, }, // Atlantic/Madeira
    {   90, 2748,   55190,  5, }, // Atlantic/Reykjavik
    {   90, 7272,   43830,  4, }, // Atlantic/South_Georgia
    {   90, 6821,   43834,  4, }, // Atlantic/St_Helena
    {   90, 6644,   55195,  3, }, // Atlantic/Stanley
    {   90, 1670,   55198,  4, }, // Australia/Adelaide
    {   90, 1635,   55202,  4, }, // Australia/Brisbane
    {   90, 1799,   43839,  5, }, // Australia/Broken_Hill
    {   90, 1583,   43844,  3, }, // Australia/Darwin
    {   90, 3637,   42685,  3, }, // Australia/Eucla
    {   90, 1516,   55206,  3, }, // Australia/Hobart
    {   90, 8753,   42691,  3, }, // Australia/Lindeman
    {   90, 1547,   43847,  4, }, // Australia/Lord_Howe
    {   90, 1727,   43851,  3, }, // Australia/Melbourne
    {   90, 1762,   55209,  2, }, // Australia/Perth
    {   90, 1463,   55211,  2, }, // Australia/Sydney
    {   90, 6441,   42705,  5, }, // Europe/Amsterdam
    {   90, 6216,   43854,  3, }, // Europe/Andorra
    {   90, 3620,   42713,  5, }, // Europe/Astrakhan
    {   90, 2300,   42718,  2, }, // Europe/Athens
    {   90, 6336,   55213,  5, }, // Europe/Belgrade
    {   90, 4757,   42724,  2, }, // Europe/Berlin
    {   90, 6488,   55218,  6, }, // Europe/Bratislava
    {   90, 2167,   43861,  4, }, // Europe/Brussels
    {   90, 3928,   42736,  5, }, // Europe/Bucharest
    {   90, 3806,   43865,  4, }, // Europe/Budapest
    {   90, 8772,   42745,  3, }, // Europe/Busingen
    {   90, 2583,   55224,  4, }, // Europe/Chisinau
    {   90, 6289,   42751,  4, }, // Europe/Copenhagen
    {   90, 2338,   42755,  3, }, // Europe/Dublin
    {   90, 6307,   43869,  4, }, // Europe/Gibraltar
    {   90, 8788,   55228,  3, }, // Europe/Guernsey
    {   90, 6612,   43876,  4, }, // Europe/Helsinki
    {   90, 8804,   55231,  3, }, // Europe/Isle_of_Man
    {   90, 1130,   55234,  5, }, // Europe/Istanbul
    {   90, 8823,   43882,  3, }, // Europe/Jersey
    {   90, 3979,   43885,  5, }, // Europe/Kaliningrad
    {   90, 8929,   42783,  3, }, // Europe/Kirov
    {   90, 2527,   43890,  2, }, // Europe/Kyiv
    {   90, 3213,   42788,  3, }, // Europe/Lisbon
    {   90, 6506,   55239,  5, }, // Europe/Ljubljana
    {   90, 2501,   43896,  2, }, // Europe/London
    {   90, 6365,   43898,  3, }, // Europe/Luxembourg
    {   90, 4892,   43901,  3, }, // Europe/Madrid
    {   90, 6397,   55244,  3, }, // Europe/Malta
    {   90, 6582,   43907,  3, }, // Europe/Mariehamn
    {   90, 3693,   42809,  3, }, // Europe/Minsk
    {   90, 6410,   43910,  3, }, // Europe/Monaco
    {   90, 3537,   42815,  3, }, // Europe/Moscow
    {   90, 6458,   43913,  3, }, // Europe/Oslo
    {   90, 4284,   42821,  2, }, // Europe/Paris
    {   90, 6424,   55247,  5, }, // Europe/Podgorica
    {   90, 6275,   42827,  3, }, // Europe/Prague
    {   90, 8837,   42830,  2, }, // Europe/Riga
    {   90, 6324,   43916,  2, }, // Europe/Rome
    {   90, 4331,   55252,  3, }, // Europe/Samara
    {   90, 6470,   55255,  4, }, // Europe/San_Marino
    {   90, 6245,   55259,  4, }, // Europe/Sarajevo
    {   90, 4451,   43925,  4, }, // Europe/Saratov
    {   90, 8849,   55263,  5, }, // Europe/Simferopol
    {   90, 6383,   55268,  4, }, // Europe/Skopje
    {   90, 6599,   55272,  3, }, // Europe/Sofia
    {   90, 6523,   43932,  5, }, // Europe/Stockholm
    {   90, 8867,   42865,  2, }, // Europe/Tallinn
    {   90, 6202,   42867,  3, }, // Europe/Tirane
    {   90, 8882,   43937,  7, }, // Europe/Ulyanovsk
    {   90, 6352,   55275,  3, }, // Europe/Vaduz
    {   90, 6567,   43947,  3, }, // Europe/Vatican
    {   90, 6231,   43950,  3, }, // Europe/Vienna
    {   90, 8899,   43953,  4, }, // Europe/Vilnius
    {   90, 4727,   43957,  5, }, // Europe/Volgograd
    {   90, 3190,   43962,  2, }, // Europe/Warsaw
    {   90, 6261,   55278,  4, }, // Europe/Zagreb
    {   90, 6540,   43964,  3, }, // Europe/Zurich
    {   90, 5078,   42904,  6, }, // Indian/Antananarivo
    {   90, 6947,   42910,  3, }, // Indian/Chagos
    {   90, 6046,   43967,  3, }, // Indian/Christmas
    {   90, 6063,   55282,  3, }, // Indian/Cocos
    {   90, 5029,   43975,  3, }, // Indian/Comoro
    {   90, 6661,   43980,  4, }, // Indian/Kerguelen
    {   90, 7260,   43985,  3, }, // Indian/Mahe
    {   90, 7050,   55285,  4, }, // Indian/Maldives
    {   90, 4069,   55289,  4, }, // Indian/Mauritius
    {   90, 5098,   43992,  3, }, // Indian/Mayotte
    {   90, 7211,   55293,  3, }, // Indian/Reunion
    {   90, 4422,   44000,  3, }, // Pacific/Apia
    {   90,  932,   44003,  3, }, // Pacific/Auckland
    {   90, 3706,   44006,  4, }, // Pacific/Bougainville
    {   90, 3015,   55296,  3, }, // Pacific/Chatham
    {   90, 3157,   55299,  5, }, // Pacific/Chuuk
    {   90, 2237,   55304,  4, }, // Pacific/Easter
    {   90, 7360,   42967,  3, }, // Pacific/Efate
    {   90, 7327,   42970,  3, }, // Pacific/Fakaofo
    {   90, 3902,   44013,  2, }, // Pacific/Fiji
    {   90, 7343,   55308,  4, }, // Pacific/Funafuti
    {   90, 6678,   55312,  4, }, // Pacific/Galapagos
    {   90, 6696,   44021,  3, }, // Pacific/Gambier
    {   90, 3822,   55316,  6, }, // Pacific/Guadalcanal
    {   90, 6018,   44032,  2, }, // Pacific/Guam
    {   90, 2723,   42998,  3, }, // Pacific/Honolulu
    {   90, 3049,   55322,  4, }, // Pacific/Kanton
    {   90, 3998,   44039,  6, }, // Pacific/Kiritimati
    {   90, 7019,   55326,  3, }, // Pacific/Kosrae
    {   90, 2842,   55329,  3, }, // Pacific/Kwajalein
    {   90, 4877,   55332,  3, }, // Pacific/Majuro
    {   90, 4051,   55335,  4, }, // Pacific/Marquesas
    {   90, 8914,   44057,  3, }, // Pacific/Midway
    {   90, 7084,   55339,  2, }, // Pacific/Nauru
    {   90, 7132,   44062,  2, }, // Pacific/Niue
    {   90, 4168,   44065,  3, }, // Pacific/Norfolk
    {   90, 7098,   55341,  3, }, // Pacific/Noumea
    {   90, 3126,   55344,  4, }, // Pacific/Pago_Pago
    {   90, 7155,   55348,  2, }, // Pacific/Palau
    {   90, 7194,   55350,  4, }, // Pacific/Pitcairn
    {   90, 3096,   55354,  4, }, // Pacific/Pohnpei
    {   90, 4807,   55358,  5, }, // Pacific/Port_Moresby
    {   90, 6076,   44079,  4, }, // Pacific/Rarotonga
    {   90, 6031,   43063,  2, }, // Pacific/Saipan
    {   90, 7312,   55363,  3, }, // Pacific/Tahiti
    {   90, 6712,   43068,  3, }, // Pacific/Tarawa
    {   90, 4583,   44083,  4, }, // Pacific/Tongatapu
    {   90, 7392,   43076,  2, }, // Pacific/Wake
    {   90, 7405,   43078,  3, }, // Pacific/Wallis
    {   95, 6899,   55366,  6, }, // America/Scoresbysund Chinese/Simplified Han/Singapore
    {   96, 6788,   55372,  3, }, // Africa/Abidjan Chinese/Traditional Han/Hong Kong
    {   96,   14,   55375,  4, }, // Africa/Asmara
    {   96, 5257,   55379,  2, }, // Africa/Bangui
    {   96, 6773,   55381,  4, }, // Africa/Conakry
    {   96, 5130,   55385,  6, }, // Africa/Dar_es_Salaam
    {   96, 4479,   55391,  5, }, // Africa/Johannesburg
    {   96, 5306,   55396,  3, }, // Africa/Kinshasa
    {   96, 5336,   55399,  3, }, // Africa/Libreville
    {   96, 5211,   55402,  3, }, // Africa/Luanda
    {   96, 5001,   55405,  3, }, // Africa/Lusaka
    {   96, 5197,   55408,  3, }, // Africa/Maseru
    {   96, 3853,   55411,  3, }, // Africa/Nairobi
    {   96, 4152,   55414,  4, }, // Africa/Windhoek
    {   96,  132,   55418,  3, }, // America/Adak
    {   96, 4554,   55421,  5, }, // America/Araguaina
    {   96,  270,   55426,  4, }, // America/Argentina/Cordoba
    {   96, 7462,   55430,  4, }, // America/Argentina/La_Rioja
    {   96, 5640,   55434,  3, }, // America/Aruba
    {   96, 4252,   55437,  3, }, // America/Asuncion
    {   96, 3679,   55440,  3, }, // America/Bahia
    {   96, 7623,   55443,  8, }, // America/Bahia_Banderas
    {   96, 5407,   55451,  3, }, // America/Belize
    {   96, 7681,   55454,  5, }, // America/Boa_Vista
    {   96, 7756,   55459,  3, }, // America/Chihuahua
    {   96, 7774,   41867,  4, }, // America/Ciudad_Juarez
    {   96, 5422,   55462,  5, }, // America/Costa_Rica
    {   96, 3791,   55467,  3, }, // America/Cuiaba
    {   96, 7848,   55470,  3, }, // America/Dawson_Creek
    {   96,  893,   55473,  3, }, // America/Edmonton
    {   96, 7906,   55476,  4, }, // America/Fortaleza
    {   96, 7924,   55480,  4, }, // America/Glace_Bay
    {   96, 5770,   55484,  4, }, // America/Grenada
    {   96, 5786,   55488,  4, }, // America/Guadeloupe
    {   96, 3760,   55492,  4, }, // America/Guatemala
    {   96, 6932,   55496,  3, }, // America/Guyana
    {   96, 2281,   55499,  3, }, // America/Havana
    {   96,  348,   55502,  7, }, // America/Indiana/Indianapolis
    {   96,  481,   55509,  8, }, // America/Indiana/Knox
    {   96, 7961,   55517,  8, }, // America/Indiana/Marengo
    {   96, 7985,   55525,  8, }, // America/Indiana/Petersburg
    {   96, 8012,   55533,  8, }, // America/Indiana/Tell_City
    {   96, 8038,   55541,  7, }, // America/Indiana/Vevay
    {   96, 8060,   55548,  8, }, // America/Indiana/Vincennes
    {   96, 8086,   55556,  9, }, // America/Indiana/Winamac
    {   96, 8110,   55565,  4, }, // America/Inuvik
    {   96,  521,   55569,  4, }, // America/Kentucky/Louisville
    {   96, 8153,   55573,  3, }, // America/Maceio
    {   96, 8168,   55576,  3, }, // America/Managua
    {   96, 5805,   55579,  4, }, // America/Martinique
    {   96, 8202,   55583,  4, }, // America/Menominee
    {   96, 4098,   55587,  5, }, // America/Montevideo
    {   96, 5824,   55592,  5, }, // America/Montserrat
    {   96, 5481,   55597,  2, }, // America/Nassau
    {   96, 1850,   55599,  4, }, // America/Noronha
    {   96, 8301,   55603,  8, }, // America/North_Dakota/Beulah
    {   96, 8329,   55611,  8, }, // America/North_Dakota/Center
    {   96, 8357,   55619,  9, }, // America/North_Dakota/New_Salem
    {   96, 8404,   55628,  3, }, // America/Porto_Velho
    {   96,  695,   55631,  5, }, // America/Rio_Branco
    {   96, 2201,   55636,  4, }, // America/Santiago
    {   96, 6111,   55640,  4, }, // America/Santo_Domingo
    {   96, 8508,   55644,  5, }, // America/St_Barthelemy
    {   96, 2061,   55649,  4, }, // America/St_Johns
    {   96, 5880,   55653,  4, }, // America/St_Lucia
    {   96, 5913,   55657,  4, }, // America/St_Vincent
    {   96, 5461,   55661,  6, }, // America/Tegucigalpa
    {   96, 5756,   55667,  2, }, // America/Thule
    {   96,  313,   55669,  3, }, // America/Tijuana
    {   96, 2144,   55672,  3, }, // America/Whitehorse
    {   96, 8552,   55675,  4, }, // America/Yakutat
    {   96, 6001,   55679,  3, }, // Antarctica/Casey
    {   96, 6094,   55682,  4, }, // Antarctica/Davis
    {   96, 6133,   55686,  6, }, // Antarctica/DumontDUrville
    {   96, 8568,   55692,  3, }, // Antarctica/Macquarie
    {   96, 7066,   55695,  3, }, // Antarctica/Mawson
    {   96, 7113,   55698,  5, }, // Antarctica/McMurdo
    {   96, 8589,   55703,  3, }, // Antarctica/Palmer
    {   96, 7226,   55706,  4, }, // Antarctica/Rothera
    {   96, 7295,   55710,  3, }, // Antarctica/Syowa
    {   96, 8607,   55713,  4, }, // Antarctica/Troll
    {   96, 1429,   55717,  4, }, // Arctic/Longyearbyen
    {   96, 5511,   55721,  4, }, // Asia/Anadyr
    {   96, 3778,   55725,  4, }, // Asia/Bishkek
    {   96, 4511,   55729,  3, }, // Asia/Colombo
    {   96, 6159,   55732,  3, }, // Asia/Dushanbe
    {   96, 4795,   55735,  3, }, // Asia/Hebron
    {   96, 4184,   55738,  5, }, // Asia/Irkutsk
    {   96, 6992,   55743,  4, }, // Asia/Jayapura
    {   96, 1349,   55747,  2, }, // Asia/Makassar
    {   96, 6920,   55749,  4, }, // Asia/Muscat
    {   96, 2554,   55753,  4, }, // Asia/Nicosia
    {   96, 5559,   55757,  3, }, // Asia/Qatar
    {   96, 4828,   55760,  4, }, // Asia/Yakutsk
    {   96, 3883,   55764,  5, }, // Asia/Yekaterinburg
    {   96, 8736,   55769,  4, }, // Atlantic/Madeira
    {   96, 6644,   55773,  3, }, // Atlantic/Stanley
    {   96, 1670,   55776,  4, }, // Australia/Adelaide
    {   96, 1635,   55780,  4, }, // Australia/Brisbane
    {   96, 1516,   55784,  3, }, // Australia/Hobart
    {   96, 6336,   55787,  5, }, // Europe/Belgrade
    {   96, 6488,   55792,  6, }, // Europe/Bratislava
    {   96, 2583,   55798,  4, }, // Europe/Chisinau
    {   96, 8788,   55802,  3, }, // Europe/Guernsey
    {   96, 8804,   55805,  3, }, // Europe/Isle_of_Man
    {   96, 6506,   55808,  5, }, // Europe/Ljubljana
    {   96, 6397,   55813,  3, }, // Europe/Malta
    {   96, 6245,   55816,  4, }, // Europe/Sarajevo
    {   96, 6352,   55820,  3, }, // Europe/Vaduz
    {   96, 6261,   55823,  4, }, // Europe/Zagreb
    {   96, 6947,   55827,  5, }, // Indian/Chagos
    {   96, 6063,   55832,  5, }, // Indian/Cocos
    {   96, 5029,   42921,  3, }, // Indian/Comoro
    {   96, 6661,   55837,  6, }, // Indian/Kerguelen
    {   96, 7050,   55843,  4, }, // Indian/Maldives
    {   96, 4069,   55847,  4, }, // Indian/Mauritius
    {   96, 5098,   42942,  3, }, // Indian/Mayotte
    {   96, 7211,   42946,  3, }, // Indian/Reunion
    {   96, 3706,   55851,  5, }, // Pacific/Bougainville
    {   96, 7327,   55856,  5, }, // Pacific/Fakaofo
    {   96, 6696,   42985,  3, }, // Pacific/Gambier
    {   96, 3822,   55861,  7, }, // Pacific/Guadalcanal
    {   96, 3049,   55868,  5, }, // Pacific/Kanton
    {   96, 7019,   55873,  3, }, // Pacific/Kosrae
    {   96, 2842,   55876,  5, }, // Pacific/Kwajalein
    {   96, 4877,   55881,  3, }, // Pacific/Majuro
    {   96, 4051,   55884,  6, }, // Pacific/Marquesas
    {   96, 7084,   55890,  2, }, // Pacific/Nauru
    {   96, 7194,   55892,  5, }, // Pacific/Pitcairn
    {   96, 4583,   55897,  5, }, // Pacific/Tongatapu
    {   96, 7392,   55902,  3, }, // Pacific/Wake
    {   97, 6788,   55372,  3, }, // Africa/Abidjan Chinese/Traditional Han/Macao
    {   97,   14,   55375,  4, }, // Africa/Asmara
    {   97, 5257,   55379,  2, }, // Africa/Bangui
    {   97, 6773,   55381,  4, }, // Africa/Conakry
    {   97, 5130,   55385,  6, }, // Africa/Dar_es_Salaam
    {   97, 4479,   55391,  5, }, // Africa/Johannesburg
    {   97, 5306,   55396,  3, }, // Africa/Kinshasa
    {   97, 5336,   55399,  3, }, // Africa/Libreville
    {   97, 5211,   55402,  3, }, // Africa/Luanda
    {   97, 5001,   55405,  3, }, // Africa/Lusaka
    {   97, 5197,   55408,  3, }, // Africa/Maseru
    {   97, 3853,   55411,  3, }, // Africa/Nairobi
    {   97, 4152,   55414,  4, }, // Africa/Windhoek
    {   97,  132,   55418,  3, }, // America/Adak
    {   97, 4554,   55421,  5, }, // America/Araguaina
    {   97,  270,   55426,  4, }, // America/Argentina/Cordoba
    {   97, 7462,   55430,  4, }, // America/Argentina/La_Rioja
    {   97, 5640,   55434,  3, }, // America/Aruba
    {   97, 4252,   55437,  3, }, // America/Asuncion
    {   97, 3679,   55440,  3, }, // America/Bahia
    {   97, 7623,   55443,  8, }, // America/Bahia_Banderas
    {   97, 5407,   55451,  3, }, // America/Belize
    {   97, 7681,   55454,  5, }, // America/Boa_Vista
    {   97, 7756,   55459,  3, }, // America/Chihuahua
    {   97, 7774,   41867,  4, }, // America/Ciudad_Juarez
    {   97, 5422,   55462,  5, }, // America/Costa_Rica
    {   97, 3791,   55467,  3, }, // America/Cuiaba
    {   97, 7848,   55470,  3, }, // America/Dawson_Creek
    {   97,  893,   55473,  3, }, // America/Edmonton
    {   97, 7906,   55476,  4, }, // America/Fortaleza
    {   97, 7924,   55480,  4, }, // America/Glace_Bay
    {   97, 5770,   55484,  4, }, // America/Grenada
    {   97, 5786,   55488,  4, }, // America/Guadeloupe
    {   97, 3760,   55492,  4, }, // America/Guatemala
    {   97, 6932,   55496,  3, }, // America/Guyana
    {   97, 2281,   55499,  3, }, // America/Havana
    {   97,  348,   55502,  7, }, // America/Indiana/Indianapolis
    {   97,  481,   55509,  8, }, // America/Indiana/Knox
    {   97, 7961,   55517,  8, }, // America/Indiana/Marengo
    {   97, 7985,   55525,  8, }, // America/Indiana/Petersburg
    {   97, 8012,   55533,  8, }, // America/Indiana/Tell_City
    {   97, 8038,   55541,  7, }, // America/Indiana/Vevay
    {   97, 8060,   55548,  8, }, // America/Indiana/Vincennes
    {   97, 8086,   55556,  9, }, // America/Indiana/Winamac
    {   97, 8110,   55565,  4, }, // America/Inuvik
    {   97,  521,   55569,  4, }, // America/Kentucky/Louisville
    {   97, 8153,   55573,  3, }, // America/Maceio
    {   97, 8168,   55576,  3, }, // America/Managua
    {   97, 5805,   55579,  4, }, // America/Martinique
    {   97, 8202,   55583,  4, }, // America/Menominee
    {   97, 4098,   55587,  5, }, // America/Montevideo
    {   97, 5824,   55592,  5, }, // America/Montserrat
    {   97, 5481,   55597,  2, }, // America/Nassau
    {   97, 1850,   55599,  4, }, // America/Noronha
    {   97, 8301,   55603,  8, }, // America/North_Dakota/Beulah
    {   97, 8329,   55611,  8, }, // America/North_Dakota/Center
    {   97, 8357,   55619,  9, }, // America/North_Dakota/New_Salem
    {   97, 8404,   55628,  3, }, // America/Porto_Velho
    {   97,  695,   55631,  5, }, // America/Rio_Branco
    {   97, 2201,   55636,  4, }, // America/Santiago
    {   97, 6111,   55640,  4, }, // America/Santo_Domingo
    {   97, 8508,   55644,  5, }, // America/St_Barthelemy
    {   97, 2061,   55649,  4, }, // America/St_Johns
    {   97, 5880,   55653,  4, }, // America/St_Lucia
    {   97, 5913,   55657,  4, }, // America/St_Vincent
    {   97, 5461,   55661,  6, }, // America/Tegucigalpa
    {   97, 5756,   55667,  2, }, // America/Thule
    {   97,  313,   55669,  3, }, // America/Tijuana
    {   97, 2144,   55672,  3, }, // America/Whitehorse
    {   97, 8552,   55675,  4, }, // America/Yakutat
    {   97, 6001,   55679,  3, }, // Antarctica/Casey
    {   97, 6094,   55682,  4, }, // Antarctica/Davis
    {   97, 6133,   55686,  6, }, // Antarctica/DumontDUrville
    {   97, 8568,   55692,  3, }, // Antarctica/Macquarie
    {   97, 7066,   55695,  3, }, // Antarctica/Mawson
    {   97, 7113,   55698,  5, }, // Antarctica/McMurdo
    {   97, 8589,   55703,  3, }, // Antarctica/Palmer
    {   97, 7226,   55706,  4, }, // Antarctica/Rothera
    {   97, 7295,   55710,  3, }, // Antarctica/Syowa
    {   97, 8607,   55713,  4, }, // Antarctica/Troll
    {   97, 1429,   55717,  4, }, // Arctic/Longyearbyen
    {   97, 5511,   55721,  4, }, // Asia/Anadyr
    {   97, 3778,   55725,  4, }, // Asia/Bishkek
    {   97, 4511,   55729,  3, }, // Asia/Colombo
    {   97, 6159,   55732,  3, }, // Asia/Dushanbe
    {   97, 4795,   55735,  3, }, // Asia/Hebron
    {   97, 4184,   55738,  5, }, // Asia/Irkutsk
    {   97, 6992,   55743,  4, }, // Asia/Jayapura
    {   97, 1349,   55747,  2, }, // Asia/Makassar
    {   97, 6920,   55749,  4, }, // Asia/Muscat
    {   97, 2554,   55753,  4, }, // Asia/Nicosia
    {   97, 5559,   55757,  3, }, // Asia/Qatar
    {   97, 4828,   55760,  4, }, // Asia/Yakutsk
    {   97, 3883,   55764,  5, }, // Asia/Yekaterinburg
    {   97, 8736,   55769,  4, }, // Atlantic/Madeira
    {   97, 6644,   55773,  3, }, // Atlantic/Stanley
    {   97, 1670,   55776,  4, }, // Australia/Adelaide
    {   97, 1635,   55780,  4, }, // Australia/Brisbane
    {   97, 1516,   55784,  3, }, // Australia/Hobart
    {   97, 6336,   55787,  5, }, // Europe/Belgrade
    {   97, 6488,   55792,  6, }, // Europe/Bratislava
    {   97, 2583,   55798,  4, }, // Europe/Chisinau
    {   97, 8788,   55802,  3, }, // Europe/Guernsey
    {   97, 8804,   55805,  3, }, // Europe/Isle_of_Man
    {   97, 6506,   55808,  5, }, // Europe/Ljubljana
    {   97, 6397,   55813,  3, }, // Europe/Malta
    {   97, 6245,   55816,  4, }, // Europe/Sarajevo
    {   97, 6352,   55820,  3, }, // Europe/Vaduz
    {   97, 6261,   55823,  4, }, // Europe/Zagreb
    {   97, 6947,   55827,  5, }, // Indian/Chagos
    {   97, 6063,   55832,  5, }, // Indian/Cocos
    {   97, 5029,   42921,  3, }, // Indian/Comoro
    {   97, 6661,   55837,  6, }, // Indian/Kerguelen
    {   97, 7050,   55843,  4, }, // Indian/Maldives
    {   97, 4069,   55847,  4, }, // Indian/Mauritius
    {   97, 5098,   42942,  3, }, // Indian/Mayotte
    {   97, 7211,   42946,  3, }, // Indian/Reunion
    {   97, 3706,   55851,  5, }, // Pacific/Bougainville
    {   97, 7327,   55856,  5, }, // Pacific/Fakaofo
    {   97, 6696,   42985,  3, }, // Pacific/Gambier
    {   97, 3822,   55861,  7, }, // Pacific/Guadalcanal
    {   97, 3049,   55868,  5, }, // Pacific/Kanton
    {   97, 7019,   55873,  3, }, // Pacific/Kosrae
    {   97, 2842,   55876,  5, }, // Pacific/Kwajalein
    {   97, 4877,   55881,  3, }, // Pacific/Majuro
    {   97, 4051,   55884,  6, }, // Pacific/Marquesas
    {   97, 7084,   55890,  2, }, // Pacific/Nauru
    {   97, 7194,   55892,  5, }, // Pacific/Pitcairn
    {   97, 4583,   55897,  5, }, // Pacific/Tongatapu
    {   97, 7392,   55902,  3, }, // Pacific/Wake
    {   99, 6788,   41567,  3, }, // Africa/Abidjan Chinese/Traditional Han/Taiwan
    {   99, 5059,   41573,  6, }, // Africa/Addis_Ababa
    {   99, 7420,   41579,  4, }, // Africa/Algiers
    {   99,   14,   41583,  4, }, // Africa/Asmara
    {   99,   44,   41587,  3, }, // Africa/Bamako
    {   99, 6746,   41592,  2, }, // Africa/Banjul
    {   99, 7435,   41594,  2, }, // Africa/Bissau
    {   99, 4957,   41596,  4, }, // Africa/Blantyre
    {   99, 5287,   41600,  5, }, // Africa/Brazzaville
    {   99, 4922,   41605,  4, }, // Africa/Bujumbura
    {   99, 2320,   41609,  2, }, // Africa/Cairo
    {   99, 4117,   41611,  5, }, // Africa/Casablanca
    {   99, 7449,   41616,  2, }, // Africa/Ceuta
    {   99, 6773,   41618,  4, }, // Africa/Conakry
    {   99, 6840,   41622,  3, }, // Africa/Dakar
    {   99, 5130,   41625,  3, }, // Africa/Dar_es_Salaam
    {   99, 5043,   41628,  3, }, // Africa/Djibouti
    {   99, 6853,   41637,  3, }, // Africa/Freetown
    {   99, 4906,   41640,  4, }, // Africa/Gaborone
    {   99, 4479,   41647,  5, }, // Africa/Johannesburg
    {   99, 5306,   41663,  3, }, // Africa/Kinshasa
    {   99, 4744,   41666,  3, }, // Africa/Lagos
    {   99, 5336,   41669,  3, }, // Africa/Libreville
    {   99, 5211,   41674,  3, }, // Africa/Luanda
    {   99, 4939,   41677,  4, }, // Africa/Lubumbashi
    {   99, 5001,   41681,  3, }, // Africa/Lusaka
    {   99, 5322,   41684,  3, }, // Africa/Malabo
    {   99, 4987,   41687,  3, }, // Africa/Maputo
    {   99, 5197,   41690,  3, }, // Africa/Maseru
    {   99, 5182,   41693,  3, }, // Africa/Mbabane
    {   99, 5113,   41696,  4, }, // Africa/Mogadishu
    {   99, 7034,   41700,  4, }, // Africa/Monrovia
    {   99, 3853,   41704,  3, }, // Africa/Nairobi
    {   99, 5271,   41707,  4, }, // Africa/Ndjamena
    {   99, 5354,   41711,  3, }, // Africa/Niamey
    {   99, 6803,   41714,  3, }, // Africa/Nouakchott
    {   99, 5225,   41721,  4, }, // Africa/Porto-Novo
    {   99, 4435,   41725,  3, }, // Africa/Sao_Tome
    {   99, 4152,   41735,  4, }, // Africa/Windhoek
    {   99,  132,   41739,  3, }, // America/Adak
    {   99, 3346,   41742,  4, }, // America/Anchorage
    {   99, 5607,   55905,  3, }, // America/Anguilla
    {   99, 5624,   41749,  3, }, // America/Antigua
    {   99, 4554,   41752,  4, }, // America/Araguaina
    {   99,  166,   41756,  7, }, // America/Argentina/Buenos_Aires
    {   99,   91,   41763,  4, }, // America/Argentina/Catamarca
    {   99,  270,   41767,  3, }, // America/Argentina/Cordoba
    {   99, 7462,   41773,  3, }, // America/Argentina/La_Rioja
    {   99,  565,   41776,  3, }, // America/Argentina/Mendoza
    {   99, 7489,   41779,  6, }, // America/Argentina/Rio_Gallegos
    {   99, 7520,   41785,  3, }, // America/Argentina/Salta
    {   99, 7544,   41788,  3, }, // America/Argentina/San_Juan
    {   99, 5580,   41791,  3, }, // America/Argentina/San_Luis
    {   99, 7571,   41794,  3, }, // America/Argentina/Tucuman
    {   99, 7597,   41797,  4, }, // America/Argentina/Ushuaia
    {   99, 5640,   55908,  5, }, // America/Aruba
    {   99, 4252,   41804,  3, }, // America/Asuncion
    {   99, 3679,   41811,  3, }, // America/Bahia
    {   99, 7623,   41814,  7, }, // America/Bahia_Banderas
    {   99, 5654,   41821,  3, }, // America/Barbados
    {   99, 7646,   41824,  2, }, // America/Belem
    {   99, 5407,   41826,  3, }, // America/Belize
    {   99, 7660,   41829,  5, }, // America/Blanc-Sablon
    {   99, 7681,   41834,  4, }, // America/Boa_Vista
    {   99, 7699,   41841,  2, }, // America/Boise
    {   99, 7713,   41843,  3, }, // America/Cambridge_Bay
    {   99, 7735,   41846,  3, }, // America/Campo_Grande
    {   99, 4694,   41851,  4, }, // America/Caracas
    {   99, 4345,   41855,  2, }, // America/Cayenne
    {   99, 5496,   41857,  4, }, // America/Cayman
    {   99, 7756,   41864,  3, }, // America/Chihuahua
    {   99, 7774,   41867,  3, }, // America/Ciudad_Juarez
    {   99, 5422,   41871,  5, }, // America/Costa_Rica
    {   99, 7796,   41876,  4, }, // America/Creston
    {   99, 3791,   41880,  3, }, // America/Cuiaba
    {   99, 5723,   41883,  3, }, // America/Curacao
    {   99, 7812,   41886,  4, }, // America/Danmarkshavn
    {   99, 5739,   55913,  4, }, // America/Dominica
    {   99,  893,   41906,  4, }, // America/Edmonton
    {   99, 7869,   41910,  4, }, // America/Eirunepe
    {   99, 5441,   41914,  4, }, // America/El_Salvador
    {   99, 7886,   41918,  4, }, // America/Fort_Nelson
    {   99, 7906,   41922,  4, }, // America/Fortaleza
    {   99, 7924,   41926,  4, }, // America/Glace_Bay
    {   99, 6881,   41930,  2, }, // America/Goose_Bay
    {   99, 4612,   41932,  4, }, // America/Grand_Turk
    {   99, 5770,   41936,  4, }, // America/Grenada
    {   99, 5786,   41940,  4, }, // America/Guadeloupe
    {   99, 3760,   41944,  4, }, // America/Guatemala
    {   99, 4841,   41948,  4, }, // America/Guayaquil
    {   99, 6932,   41952,  3, }, // America/Guyana
    {   99, 1939,   41955,  5, }, // America/Halifax
    {   99, 7942,   41963,  4, }, // America/Hermosillo
    {   99,  348,   41967,  7, }, // America/Indiana/Indianapolis
    {   99,  481,   41974,  8, }, // America/Indiana/Knox
    {   99, 7961,   41982,  8, }, // America/Indiana/Marengo
    {   99, 7985,   41990,  8, }, // America/Indiana/Petersburg
    {   99, 8012,   41998,  8, }, // America/Indiana/Tell_City
    {   99, 8038,   42006,  7, }, // America/Indiana/Vevay
    {   99, 8060,   42013,  8, }, // America/Indiana/Vincennes
    {   99, 8086,   42021,  9, }, // America/Indiana/Winamac
    {   99, 8110,   42030,  4, }, // America/Inuvik
    {   99, 2799,   42037,  3, }, // America/Jamaica
    {   99, 5380,   42040,  2, }, // America/Juneau
    {   99,  521,   42042,  5, }, // America/Kentucky/Louisville
    {   99, 8125,   42047,  8, }, // America/Kentucky/Monticello
    {   99, 5704,   42055,  5, }, // America/Kralendijk
    {   99, 7169,   42063,  2, }, // America/Lima
    {   99, 3239,   42065,  3, }, // America/Los_Angeles
    {   99, 5932,   42068,  4, }, // America/Lower_Princes
    {   99, 8153,   42072,  3, }, // America/Maceio
    {   99, 8168,   42075,  3, }, // America/Managua
    {   99, 1908,   42078,  3, }, // America/Manaus
    {   99, 5897,   42081,  4, }, // America/Marigot
    {   99, 5805,   42085,  4, }, // America/Martinique
    {   99, 8184,   42089,  5, }, // America/Matamoros
    {   99, 2917,   42094,  4, }, // America/Mazatlan
    {   99, 8202,   42098,  4, }, // America/Menominee
    {   99, 8220,   42102,  3, }, // America/Merida
    {   99, 2949,   42111,  4, }, // America/Mexico_City
    {   99, 4391,   42115,  3, }, // America/Miquelon
    {   99, 8254,   42118,  3, }, // America/Moncton
    {   99, 8270,   42121,  3, }, // America/Monterrey
    {   99, 4098,   42124,  4, }, // America/Montevideo
    {   99, 5824,   42128,  3, }, // America/Montserrat
    {   99, 5481,   42131,  2, }, // America/Nassau
    {   99, 2379,   42133,  2, }, // America/New_York
    {   99, 8288,   42135,  2, }, // America/Nome
    {   99, 1850,   42137,  3, }, // America/Noronha
    {   99, 8301,   42140,  8, }, // America/North_Dakota/Beulah
    {   99, 8329,   42148,  7, }, // America/North_Dakota/Center
    {   99, 8357,   42155,  8, }, // America/North_Dakota/New_Salem
    {   99, 8388,   42165,  4, }, // America/Ojinaga
    {   99, 2356,   42169,  3, }, // America/Panama
    {   99, 6173,   42172,  5, }, // America/Paramaribo
    {   99, 2973,   42177,  3, }, // America/Phoenix
    {   99, 8404,   42187,  3, }, // America/Porto_Velho
    {   99, 4030,   42194,  6, }, // America/Punta_Arenas
    {   99, 8424,   42200,  3, }, // America/Rankin_Inlet
    {   99, 8445,   42203,  3, }, // America/Recife
    {   99, 1995,   42206,  3, }, // America/Regina
    {   99, 8460,   42209,  4, }, // America/Resolute
    {   99,  695,   42213,  4, }, // America/Rio_Branco
    {   99, 8477,   42217,  3, }, // America/Santarem
    {   99, 2201,   42220,  4, }, // America/Santiago
    {   99, 6111,   42224,  4, }, // America/Santo_Domingo
    {   99, 1878,   42228,  3, }, // America/Sao_Paulo
    {   99, 6899,   42231,  7, }, // America/Scoresbysund
    {   99, 8494,   42238,  3, }, // America/Sitka
    {   99, 8508,   42241,  6, }, // America/St_Barthelemy
    {   99, 2061,   42247,  3, }, // America/St_Johns
    {   99, 5863,   42250,  3, }, // America/St_Kitts
    {   99, 5880,   42253,  4, }, // America/St_Lucia
    {   99,  855,   42257,  4, }, // America/St_Thomas
    {   99, 5913,   42261,  3, }, // America/St_Vincent
    {   99, 8530,   42264,  7, }, // America/Swift_Current
    {   99, 5461,   42271,  5, }, // America/Tegucigalpa
    {   99, 5756,   42276,  2, }, // America/Thule
    {   99,  313,   42278,  3, }, // America/Tijuana
    {   99,  608,   42281,  3, }, // America/Toronto
    {   99, 5688,   42284,  4, }, // America/Tortola
    {   99, 2093,   42288,  3, }, // America/Vancouver
    {   99, 2144,   42291,  4, }, // America/Whitehorse
    {   99,  734,   42295,  3, }, // America/Winnipeg
    {   99, 8552,   42298,  3, }, // America/Yakutat
    {   99, 6001,   42301,  2, }, // Antarctica/Casey
    {   99, 6094,   42303,  3, }, // Antarctica/Davis
    {   99, 6133,   42306,  5, }, // Antarctica/DumontDUrville
    {   99, 8568,   42311,  3, }, // Antarctica/Macquarie
    {   99, 7113,   42316,  4, }, // Antarctica/McMurdo
    {   99, 8589,   42320,  2, }, // Antarctica/Palmer
    {   99, 7226,   42322,  3, }, // Antarctica/Rothera
    {   99, 7295,   42325,  4, }, // Antarctica/Syowa
    {   99, 8607,   42329,  2, }, // Antarctica/Troll
    {   99, 1429,   42335,  5, }, // Arctic/Longyearbyen
    {   99, 5570,   42340,  2, }, // Asia/Aden
    {   99, 5395,   42342,  4, }, // Asia/Almaty
    {   99, 5511,   42348,  3, }, // Asia/Anadyr
    {   99, 5368,   42354,  4, }, // Asia/Aqtobe
    {   99,  964,   42358,  5, }, // Asia/Ashgabat
    {   99, 8624,   42363,  3, }, // Asia/Atyrau
    {   99, 3607,   42366,  3, }, // Asia/Baghdad
    {   99, 3653,   41882,  2, }, // Asia/Baku
    {   99, 3571,   42373,  4, }, // Asia/Barnaul
    {   99, 4086,   42377,  3, }, // Asia/Beirut
    {   99, 3778,   42380,  4, }, // Asia/Bishkek
    {   99, 5989,   42384,  2, }, // Asia/Brunei
    {   99, 4511,   42388,  3, }, // Asia/Colombo
    {   99, 4540,   42391,  4, }, // Asia/Damascus
    {   99, 1093,   42395,  2, }, // Asia/Dhaka
    {   99, 3596,   42399,  2, }, // Asia/Dubai
    {   99, 6159,   42401,  3, }, // Asia/Dushanbe
    {   99, 8636,   42404,  5, }, // Asia/Famagusta
    {   99, 8651,   42409,  2, }, // Asia/Gaza
    {   99, 4795,   42411,  3, }, // Asia/Hebron
    {   99, 4184,   42435,  5, }, // Asia/Irkutsk
    {   99, 7006,   42440,  3, }, // Asia/Jakarta
    {   99, 6992,   42443,  4, }, // Asia/Jayapura
    {   99, 3560,   42451,  3, }, // Asia/Kabul
    {   99, 4239,   42457,  3, }, // Asia/Karachi
    {   99, 1185,   42460,  4, }, // Asia/Kathmandu
    {   99, 8661,   42464,  3, }, // Asia/Khandyga
    {   99,  992,   42467,  4, }, // Asia/Kolkata
    {   99, 4197,   42471,  8, }, // Asia/Krasnoyarsk
    {   99, 5976,   42482,  2, }, // Asia/Kuching
    {   99, 1211,   42494,  2, }, // Asia/Macau
    {   99, 4017,   42501,  3, }, // Asia/Magadan
    {   99, 1349,   42504,  4, }, // Asia/Makassar
    {   99, 7182,   42508,  3, }, // Asia/Manila
    {   99, 6920,   42511,  4, }, // Asia/Muscat
    {   99, 2554,   42515,  4, }, // Asia/Nicosia
    {   99, 8675,   42519,  6, }, // Asia/Novokuznetsk
    {   99, 4135,   42525,  5, }, // Asia/Novosibirsk
    {   99, 7145,   42534,  3, }, // Asia/Oral
    {   99, 6961,   42537,  2, }, // Asia/Phnom_Penh
    {   99, 5559,   42543,  2, }, // Asia/Qatar
    {   99, 8708,   55917,  4, }, // Asia/Qostanay
    {   99, 4269,   42549,  6, }, // Asia/Qyzylorda
    {   99, 3584,   42555,  3, }, // Asia/Riyadh
    {   99, 4408,   42558,  3, }, // Asia/Sakhalin
    {   99, 7245,   42561,  4, }, // Asia/Samarkand
    {   99, 3279,   42565,  2, }, // Asia/Seoul
    {   99, 2772,   42587,  3, }, // Asia/Tehran
    {   99, 2821,   42592,  2, }, // Asia/Tokyo
    {   99, 1021,   42598,  4, }, // Asia/Ulaanbaatar
    {   99, 1159,   42602,  4, }, // Asia/Urumqi
    {   99, 8722,   42606,  4, }, // Asia/Ust-Nera
    {   99, 6977,   42610,  2, }, // Asia/Vientiane
    {   99, 4710,   42612,  3, }, // Asia/Vladivostok
    {   99, 4828,   42615,  4, }, // Asia/Yakutsk
    {   99, 3883,   42621,  5, }, // Asia/Yekaterinburg
    {   99, 3747,   42626,  3, }, // Asia/Yerevan
    {   99, 3663,   42629,  5, }, // Atlantic/Azores
    {   99, 5671,   42634,  3, }, // Atlantic/Bermuda
    {   99, 6628,   42637,  3, }, // Atlantic/Canary
    {   99, 3727,   42640,  3, }, // Atlantic/Cape_Verde
    {   99, 1395,   42643,  4, }, // Atlantic/Faroe
    {   99, 8736,   42647,  5, }, // Atlantic/Madeira
    {   99, 2748,   42652,  5, }, // Atlantic/Reykjavik
    {   99, 7272,   42657,  4, }, // Atlantic/South_Georgia
    {   99, 6821,   42661,  5, }, // Atlantic/St_Helena
    {   99, 6644,   42666,  3, }, // Atlantic/Stanley
    {   99, 1670,   42669,  4, }, // Australia/Adelaide
    {   99, 1635,   42673,  4, }, // Australia/Brisbane
    {   99, 1799,   42677,  5, }, // Australia/Broken_Hill
    {   99, 1583,   42682,  3, }, // Australia/Darwin
    {   99, 1516,   42688,  3, }, // Australia/Hobart
    {   99, 1547,   42694,  4, }, // Australia/Lord_Howe
    {   99, 1727,   42698,  3, }, // Australia/Melbourne
    {   99, 1762,   42701,  2, }, // Australia/Perth
    {   99, 1463,   42703,  2, }, // Australia/Sydney
    {   99, 6216,   42710,  3, }, // Europe/Andorra
    {   99, 6336,   42720,  4, }, // Europe/Belgrade
    {   99, 6488,   42726,  6, }, // Europe/Bratislava
    {   99, 2167,   42732,  4, }, // Europe/Brussels
    {   99, 3806,   42741,  4, }, // Europe/Budapest
    {   99, 2583,   55921,  3, }, // Europe/Chisinau
    {   99, 2338,   42755,  3, }, // Europe/Dublin
    {   99, 6307,   42758,  4, }, // Europe/Gibraltar
    {   99, 8788,   42762,  3, }, // Europe/Guernsey
    {   99, 6612,   42765,  4, }, // Europe/Helsinki
    {   99, 8804,   42769,  2, }, // Europe/Isle_of_Man
    {   99, 1130,   42771,  4, }, // Europe/Istanbul
    {   99, 8823,   42775,  3, }, // Europe/Jersey
    {   99, 3979,   42778,  5, }, // Europe/Kaliningrad
    {   99, 2527,   42786,  2, }, // Europe/Kyiv
    {   99, 6506,   42791,  4, }, // Europe/Ljubljana
    {   99, 2501,   42795,  2, }, // Europe/London
    {   99, 6365,   42797,  3, }, // Europe/Luxembourg
    {   99, 4892,   42800,  3, }, // Europe/Madrid
    {   99, 6397,   42803,  3, }, // Europe/Malta
    {   99, 6582,   42806,  3, }, // Europe/Mariehamn
    {   99, 6410,   42812,  3, }, // Europe/Monaco
    {   99, 6458,   42818,  3, }, // Europe/Oslo
    {   99, 6424,   42823,  4, }, // Europe/Podgorica
    {   99, 6324,   42832,  2, }, // Europe/Rome
    {   99, 4331,   42834,  3, }, // Europe/Samara
    {   99, 6470,   42837,  4, }, // Europe/San_Marino
    {   99, 6245,   42841,  4, }, // Europe/Sarajevo
    {   99, 4451,   42845,  4, }, // Europe/Saratov
    {   99, 8849,   42849,  4, }, // Europe/Simferopol
    {   99, 6383,   42853,  4, }, // Europe/Skopje
    {   99, 6599,   42857,  3, }, // Europe/Sofia
    {   99, 6523,   42860,  5, }, // Europe/Stockholm
    {   99, 8882,   42870,  7, }, // Europe/Ulyanovsk
    {   99, 6352,   42877,  3, }, // Europe/Vaduz
    {   99, 6567,   42880,  3, }, // Europe/Vatican
    {   99, 6231,   42883,  3, }, // Europe/Vienna
    {   99, 8899,   42886,  4, }, // Europe/Vilnius
    {   99, 4727,   42890,  5, }, // Europe/Volgograd
    {   99, 3190,   42895,  2, }, // Europe/Warsaw
    {   99, 6261,   42897,  4, }, // Europe/Zagreb
    {   99, 6540,   42901,  3, }, // Europe/Zurich
    {   99, 5078,   55924,  6, }, // Indian/Antananarivo
    {   99, 6046,   42913,  3, }, // Indian/Christmas
    {   99, 6063,   42916,  5, }, // Indian/Cocos
    {   99, 5029,   42921,  5, }, // Indian/Comoro
    {   99, 6661,   42926,  5, }, // Indian/Kerguelen
    {   99, 7260,   42931,  3, }, // Indian/Mahe
    {   99, 7050,   42934,  4, }, // Indian/Maldives
    {   99, 4069,   42938,  4, }, // Indian/Mauritius
    {   99, 5098,   42942,  4, }, // Indian/Mayotte
    {   99, 7211,   42946,  4, }, // Indian/Reunion
    {   99, 4422,   42950,  3, }, // Pacific/Apia
    {   99,  932,   42953,  3, }, // Pacific/Auckland
    {   99, 3706,   42956,  4, }, // Pacific/Bougainville
    {   99, 3015,   42960,  2, }, // Pacific/Chatham
    {   99, 3157,   42962,  2, }, // Pacific/Chuuk
    {   99, 2237,   42964,  3, }, // Pacific/Easter
    {   99, 3902,   42973,  2, }, // Pacific/Fiji
    {   99, 7343,   42975,  4, }, // Pacific/Funafuti
    {   99, 6678,   42979,  6, }, // Pacific/Galapagos
    {   99, 6696,   42985,  5, }, // Pacific/Gambier
    {   99, 3822,   42990,  6, }, // Pacific/Guadalcanal
    {   99, 6018,   42996,  2, }, // Pacific/Guam
    {   99, 3049,   43001,  5, }, // Pacific/Kanton
    {   99, 3998,   43006,  6, }, // Pacific/Kiritimati
    {   99, 7019,   43012,  3, }, // Pacific/Kosrae
    {   99, 2842,   43015,  4, }, // Pacific/Kwajalein
    {   99, 4877,   43019,  3, }, // Pacific/Majuro
    {   99, 4051,   43022,  5, }, // Pacific/Marquesas
    {   99, 8914,   43027,  3, }, // Pacific/Midway
    {   99, 7084,   43030,  2, }, // Pacific/Nauru
    {   99, 7132,   43032,  3, }, // Pacific/Niue
    {   99, 4168,   43035,  3, }, // Pacific/Norfolk
    {   99, 7098,   43038,  3, }, // Pacific/Noumea
    {   99, 3126,   43041,  4, }, // Pacific/Pago_Pago
    {   99, 7155,   43045,  2, }, // Pacific/Palau
    {   99, 7194,   43047,  5, }, // Pacific/Pitcairn
    {   99, 3096,   43052,  3, }, // Pacific/Pohnpei
    {   99, 4807,   43055,  4, }, // Pacific/Port_Moresby
    {   99, 6076,   43059,  4, }, // Pacific/Rarotonga
    {   99, 7312,   43065,  3, }, // Pacific/Tahiti
    {   99, 4583,   43071,  5, }, // Pacific/Tongatapu
    {  100,   14,       0,  6, }, // Africa/Asmara Church/Cyrillic/Russia
    {  100, 4435,       6,  8, }, // Africa/Sao_Tome
    {  100, 4252,      14,  8, }, // America/Asuncion
    {  100,  237,      22,  8, }, // America/Atikokan
    {  100, 7623,      30, 17, }, // America/Bahia_Banderas
    {  100, 3868,      47,  6, }, // America/Cancun
    {  100, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  100, 5723,      66,  7, }, // America/Curacao
    {  100,  481,      73, 13, }, // America/Indiana/Knox
    {  100, 7961,      86, 16, }, // America/Indiana/Marengo
    {  100, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  100, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  100, 8038,     139, 14, }, // America/Indiana/Vevay
    {  100, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  100, 8086,     171, 16, }, // America/Indiana/Winamac
    {  100, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  100, 5932,     207, 22, }, // America/Lower_Princes
    {  100, 8220,     229,  6, }, // America/Merida
    {  100, 1850,     235, 19, }, // America/Noronha
    {  100, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  100, 8329,     274, 20, }, // America/North_Dakota/Center
    {  100, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  100,  393,     317,  4, }, // America/Nuuk
    {  100, 6899,     321, 16, }, // America/Scoresbysund
    {  100, 8508,     337, 14, }, // America/St_Barthelemy
    {  100, 2061,     351, 10, }, // America/St_Johns
    {  100, 5863,     361,  9, }, // America/St_Kitts
    {  100, 5880,     370,  9, }, // America/St_Lucia
    {  100,  855,     379, 10, }, // America/St_Thomas
    {  100, 5913,     389, 11, }, // America/St_Vincent
    {  100, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  100, 5395,   55930,  8, }, // Asia/Almaty
    {  100, 5511,   55938,  9, }, // Asia/Anadyr
    {  100, 5523,   55947,  7, }, // Asia/Aqtau
    {  100, 5368,   55954, 12, }, // Asia/Aqtobe
    {  100, 3778,   55966,  8, }, // Asia/Bishkek
    {  100, 4601,   55974,  5, }, // Asia/Chita
    {  100, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  100, 4184,   55979, 10, }, // Asia/Irkutsk
    {  100, 4316,   55989, 27, }, // Asia/Kamchatka
    {  100, 1185,     448,  9, }, // Asia/Kathmandu
    {  100, 8661,   56016,  8, }, // Asia/Khandyga
    {  100,  992,     457,  7, }, // Asia/Kolkata
    {  100, 4197,   56024, 12, }, // Asia/Krasnoyarsk
    {  100, 1211,     464,  5, }, // Asia/Macau
    {  100, 4017,   56036,  9, }, // Asia/Magadan
    {  100, 8675,   56045, 13, }, // Asia/Novokuznetsk
    {  100, 4135,   56058, 13, }, // Asia/Novosibirsk
    {  100, 4229,   56071,  7, }, // Asia/Omsk
    {  100, 7145,   56078, 11, }, // Asia/Oral
    {  100, 4269,   56089, 10, }, // Asia/Qyzylorda
    {  100, 4408,   56099,  9, }, // Asia/Sakhalin
    {  100, 4297,   56108, 15, }, // Asia/Srednekolymsk
    {  100, 8722,   56123, 13, }, // Asia/Ust-Nera
    {  100, 4710,   56136, 13, }, // Asia/Vladivostok
    {  100, 4828,   56149,  9, }, // Asia/Yakutsk
    {  100, 1235,     477,  6, }, // Asia/Yangon
    {  100, 3883,   56158, 15, }, // Asia/Yekaterinburg
    {  100, 1395,     483,  5, }, // Atlantic/Faroe
    {  100, 6821,     488, 10, }, // Atlantic/St_Helena
    {  100, 3979,   56173, 13, }, // Europe/Kaliningrad
    {  100, 2527,   56186,  6, }, // Europe/Kyiv
    {  100, 3693,   56192,  7, }, // Europe/Minsk
    {  100, 3537,   56199,  7, }, // Europe/Moscow
    {  100, 4331,   56206,  7, }, // Europe/Samara
    {  100, 8849,   56213, 12, }, // Europe/Simferopol
    {  100, 4727,   56225, 11, }, // Europe/Volgograd
    {  100, 7211,     547,  7, }, // Indian/Reunion
    {  100, 3157,     554,  5, }, // Pacific/Chuuk
    {  100, 3096,     586,  7, }, // Pacific/Pohnpei
    {  101, 6788,   36346,  7, }, // Africa/Abidjan Chuvash/Cyrillic/Russia
    {  101, 6760,     610,  5, }, // Africa/Accra
    {  101, 5059,     615, 11, }, // Africa/Addis_Ababa
    {  101, 7420,     626,  5, }, // Africa/Algiers
    {  101,   14,   56236,  6, }, // Africa/Asmara
    {  101,   44,     637,  6, }, // Africa/Bamako
    {  101, 5257,     643,  5, }, // Africa/Bangui
    {  101, 6746,     648,  6, }, // Africa/Banjul
    {  101, 7435,     654,  5, }, // Africa/Bissau
    {  101, 4957,   27220,  8, }, // Africa/Blantyre
    {  101, 5287,   51554, 10, }, // Africa/Brazzaville
    {  101, 4922,     676,  9, }, // Africa/Bujumbura
    {  101, 2320,     685,  4, }, // Africa/Cairo
    {  101, 4117,     689, 10, }, // Africa/Casablanca
    {  101, 7449,     699,  5, }, // Africa/Ceuta
    {  101, 6773,     704,  7, }, // Africa/Conakry
    {  101, 6840,     711,  5, }, // Africa/Dakar
    {  101, 5130,   27248, 12, }, // Africa/Dar_es_Salaam
    {  101, 5043,   36379,  7, }, // Africa/Djibouti
    {  101, 5243,     734,  5, }, // Africa/Douala
    {  101, 5166,   27267,  7, }, // Africa/El_Aaiun
    {  101, 6853,     746,  7, }, // Africa/Freetown
    {  101, 4906,     753,  8, }, // Africa/Gaborone
    {  101, 5015,   33913,  6, }, // Africa/Harare
    {  101, 4479,   51572, 12, }, // Africa/Johannesburg
    {  101, 4499,   27305,  5, }, // Africa/Juba
    {  101, 5151,     783,  7, }, // Africa/Kampala
    {  101, 4524,     790,  6, }, // Africa/Khartoum
    {  101, 4973,     796,  6, }, // Africa/Kigali
    {  101, 5306,     802,  7, }, // Africa/Kinshasa
    {  101, 4744,     809,  5, }, // Africa/Lagos
    {  101, 5336,   51584,  9, }, // Africa/Libreville
    {  101, 6869,     822,  4, }, // Africa/Lome
    {  101, 5211,     826,  6, }, // Africa/Luanda
    {  101, 4939,     832,  9, }, // Africa/Lubumbashi
    {  101, 5001,     841,  6, }, // Africa/Lusaka
    {  101, 5322,     847,  6, }, // Africa/Malabo
    {  101, 4987,     853,  6, }, // Africa/Maputo
    {  101, 5197,     859,  6, }, // Africa/Maseru
    {  101, 5182,     865,  7, }, // Africa/Mbabane
    {  101, 5113,     872,  8, }, // Africa/Mogadishu
    {  101, 7034,     880,  7, }, // Africa/Monrovia
    {  101, 3853,   36427,  7, }, // Africa/Nairobi
    {  101, 5271,   27386,  8, }, // Africa/Ndjamena
    {  101, 5354,   36434,  6, }, // Africa/Niamey
    {  101, 6803,     908,  7, }, // Africa/Nouakchott
    {  101, 6727,     915,  8, }, // Africa/Ouagadougou
    {  101, 5225,     923, 10, }, // Africa/Porto-Novo
    {  101, 4435,   56242, 17, }, // Africa/Sao_Tome
    {  101, 2866,     941,  7, }, // Africa/Tripoli
    {  101, 6554,     948,  5, }, // Africa/Tunis
    {  101, 4152,     953,  7, }, // Africa/Windhoek
    {  101,  132,     960,  4, }, // America/Adak
    {  101, 3346,   51593,  8, }, // America/Anchorage
    {  101, 5607,   51601,  7, }, // America/Anguilla
    {  101, 5624,     978,  7, }, // America/Antigua
    {  101, 4554,     985,  9, }, // America/Araguaina
    {  101,  166,   51608, 12, }, // America/Argentina/Buenos_Aires
    {  101,   91,    1006,  9, }, // America/Argentina/Catamarca
    {  101,  270,    1015,  7, }, // America/Argentina/Cordoba
    {  101,  441,   27487,  5, }, // America/Argentina/Jujuy
    {  101, 7462,    1027,  8, }, // America/Argentina/La_Rioja
    {  101,  565,    1035,  7, }, // America/Argentina/Mendoza
    {  101, 7489,   51620, 12, }, // America/Argentina/Rio_Gallegos
    {  101, 7520,   27512,  6, }, // America/Argentina/Salta
    {  101, 7544,    1058,  8, }, // America/Argentina/San_Juan
    {  101, 5580,   56259, 17, }, // America/Argentina/San_Luis
    {  101, 7571,    1074,  7, }, // America/Argentina/Tucuman
    {  101, 7597,   27526,  5, }, // America/Argentina/Ushuaia
    {  101, 5640,    1087,  5, }, // America/Aruba
    {  101, 4252,   51632,  8, }, // America/Asuncion
    {  101,  237,    1099, 12, }, // America/Atikokan
    {  101, 3679,   36500,  4, }, // America/Bahia
    {  101, 7623,   51640, 16, }, // America/Bahia_Banderas
    {  101, 5654,    1131,  8, }, // America/Barbados
    {  101, 7646,    1139,  5, }, // America/Belem
    {  101, 5407,    1144,  5, }, // America/Belize
    {  101, 7660,    1149, 12, }, // America/Blanc-Sablon
    {  101, 7681,    1161,  9, }, // America/Boa_Vista
    {  101, 4361,    1170,  6, }, // America/Bogota
    {  101, 7699,   51656,  5, }, // America/Boise
    {  101, 7713,   51661, 13, }, // America/Cambridge_Bay
    {  101, 7735,    1193, 12, }, // America/Campo_Grande
    {  101, 3868,    1205,  6, }, // America/Cancun
    {  101, 4694,    1211,  7, }, // America/Caracas
    {  101, 4345,   51674,  7, }, // America/Cayenne
    {  101, 5496,   56276, 16, }, // America/Cayman
    {  101, 2260,    1247,  6, }, // America/Chicago
    {  101, 7756,    1253,  6, }, // America/Chihuahua
    {  101, 7774,   56292, 13, }, // America/Ciudad_Juarez
    {  101, 5422,    1259, 10, }, // America/Costa_Rica
    {  101, 7796,    1269,  7, }, // America/Creston
    {  101, 3791,   27688,  5, }, // America/Cuiaba
    {  101, 5723,   36606,  7, }, // America/Curacao
    {  101, 7812,    1290, 12, }, // America/Danmarkshavn
    {  101, 7833,    1302,  6, }, // America/Dawson
    {  101, 7848,    1308, 11, }, // America/Dawson_Creek
    {  101,  805,    1319,  6, }, // America/Denver
    {  101, 3465,   36649,  7, }, // America/Detroit
    {  101, 5739,    1332,  8, }, // America/Dominica
    {  101,  893,   51681,  8, }, // America/Edmonton
    {  101, 7869,   51689,  8, }, // America/Eirunepe
    {  101, 5441,   27765,  9, }, // America/El_Salvador
    {  101, 7886,   51697, 12, }, // America/Fort_Nelson
    {  101, 7906,    1375,  9, }, // America/Fortaleza
    {  101, 7924,   51709,  9, }, // America/Glace_Bay
    {  101, 6881,   51718,  7, }, // America/Goose_Bay
    {  101, 4612,    1400, 10, }, // America/Grand_Turk
    {  101, 5770,    1410,  7, }, // America/Grenada
    {  101, 5786,    1417,  9, }, // America/Guadeloupe
    {  101, 3760,    1426,  9, }, // America/Guatemala
    {  101, 4841,   51725,  8, }, // America/Guayaquil
    {  101, 6932,   51733,  6, }, // America/Guyana
    {  101, 1939,   51739,  8, }, // America/Halifax
    {  101, 2281,    1457,  6, }, // America/Havana
    {  101, 7942,   51747,  9, }, // America/Hermosillo
    {  101,  348,    1471, 12, }, // America/Indiana/Indianapolis
    {  101,  481,    1483, 13, }, // America/Indiana/Knox
    {  101, 7961,    1496, 16, }, // America/Indiana/Marengo
    {  101, 7985,    1512, 19, }, // America/Indiana/Petersburg
    {  101, 8012,   56305, 18, }, // America/Indiana/Tell_City
    {  101, 8038,   56323, 14, }, // America/Indiana/Vevay
    {  101, 8060,   56337, 18, }, // America/Indiana/Vincennes
    {  101, 8086,   56355, 16, }, // America/Indiana/Winamac
    {  101, 8110,    1570,  6, }, // America/Inuvik
    {  101,  660,    1576,  7, }, // America/Iqaluit
    {  101, 2799,   28011,  6, }, // America/Jamaica
    {  101, 5380,   36746,  5, }, // America/Juneau
    {  101,  521,    1594,  8, }, // America/Kentucky/Louisville
    {  101, 8125,    1602, 20, }, // America/Kentucky/Monticello
    {  101, 5704,   36751, 10, }, // America/Kralendijk
    {  101, 4376,    1632,  6, }, // America/La_Paz
    {  101, 7169,    1638,  4, }, // America/Lima
    {  101, 3239,   51761, 12, }, // America/Los_Angeles
    {  101, 5932,   56371, 18, }, // America/Lower_Princes
    {  101, 8153,   36799,  6, }, // America/Maceio
    {  101, 8168,    1677,  7, }, // America/Managua
    {  101, 1908,    1684,  6, }, // America/Manaus
    {  101, 5897,    1690,  6, }, // America/Marigot
    {  101, 5805,    1696,  9, }, // America/Martinique
    {  101, 8184,    1705,  9, }, // America/Matamoros
    {  101, 2917,    1714,  8, }, // America/Mazatlan
    {  101, 8202,    1722,  8, }, // America/Menominee
    {  101, 8220,    1730,  6, }, // America/Merida
    {  101, 8235,    1736, 10, }, // America/Metlakatla
    {  101, 2949,    1746,  6, }, // America/Mexico_City
    {  101, 4391,    1752,  7, }, // America/Miquelon
    {  101, 8254,    1759,  7, }, // America/Moncton
    {  101, 8270,   51794,  9, }, // America/Monterrey
    {  101, 4098,    1775, 10, }, // America/Montevideo
    {  101, 5824,    1785, 10, }, // America/Montserrat
    {  101, 5481,    1795,  6, }, // America/Nassau
    {  101, 2379,   51812,  8, }, // America/New_York
    {  101, 8288,    1809,  3, }, // America/Nome
    {  101, 1850,   56389,  7, }, // America/Noronha
    {  101, 8301,   56396, 20, }, // America/North_Dakota/Beulah
    {  101, 8329,   56416, 20, }, // America/North_Dakota/Center
    {  101, 8357,   56436, 25, }, // America/North_Dakota/New_Salem
    {  101,  393,    1890,  4, }, // America/Nuuk
    {  101, 8388,    1894,  7, }, // America/Ojinaga
    {  101, 2356,    1901,  6, }, // America/Panama
    {  101, 6173,    1907, 10, }, // America/Paramaribo
    {  101, 2973,    1917,  6, }, // America/Phoenix
    {  101, 3945,    1923, 12, }, // America/Port-au-Prince
    {  101, 5954,   28321, 13, }, // America/Port_of_Spain
    {  101, 8404,   28334, 11, }, // America/Porto_Velho
    {  101, 5843,   51888, 11, }, // America/Puerto_Rico
    {  101, 4030,    1970, 12, }, // America/Punta_Arenas
    {  101, 8424,    1982, 12, }, // America/Rankin_Inlet
    {  101, 8445,    1994,  6, }, // America/Recife
    {  101, 1995,   51899,  8, }, // America/Regina
    {  101, 8460,   56461,  8, }, // America/Resolute
    {  101,  695,    2015, 10, }, // America/Rio_Branco
    {  101, 8477,    2025,  8, }, // America/Santarem
    {  101, 2201,   51907,  8, }, // America/Santiago
    {  101, 6111,    2041, 13, }, // America/Santo_Domingo
    {  101, 1878,   56469, 19, }, // America/Sao_Paulo
    {  101, 6899,    2063, 11, }, // America/Scoresbysund
    {  101, 8494,    2074,  5, }, // America/Sitka
    {  101, 8508,    2079, 13, }, // America/St_Barthelemy
    {  101, 2061,   28481, 10, }, // America/St_Johns
    {  101, 5863,    2101,  9, }, // America/St_Kitts
    {  101, 5880,   56488, 10, }, // America/St_Lucia
    {  101,  855,    2121, 10, }, // America/St_Thomas
    {  101, 5913,    2131, 12, }, // America/St_Vincent
    {  101, 8530,    2143, 13, }, // America/Swift_Current
    {  101, 5461,   51949, 12, }, // America/Tegucigalpa
    {  101, 5756,    2167,  4, }, // America/Thule
    {  101,  313,    2171,  7, }, // America/Tijuana
    {  101,  608,    2178,  7, }, // America/Toronto
    {  101, 5688,    2185,  7, }, // America/Tortola
    {  101, 2093,    2192,  8, }, // America/Vancouver
    {  101, 2144,   28567,  8, }, // America/Whitehorse
    {  101,  734,    2208,  8, }, // America/Winnipeg
    {  101, 8552,   28582,  6, }, // America/Yakutat
    {  101, 6001,   37021,  5, }, // Antarctica/Casey
    {  101, 6094,   37026,  6, }, // Antarctica/Davis
    {  101, 6133,   56498, 29, }, // Antarctica/DumontDUrville
    {  101, 8568,    2249,  8, }, // Antarctica/Macquarie
    {  101, 7066,    2257,  6, }, // Antarctica/Mawson
    {  101, 7113,    2263,  9, }, // Antarctica/McMurdo
    {  101, 8589,    2272,  6, }, // Antarctica/Palmer
    {  101, 7226,    2278,  6, }, // Antarctica/Rothera
    {  101, 7295,   28642,  4, }, // Antarctica/Syowa
    {  101, 8607,   51975,  6, }, // Antarctica/Troll
    {  101, 7374,   34957,  6, }, // Antarctica/Vostok
    {  101, 1429,   51981,  7, }, // Arctic/Longyearbyen
    {  101, 5570,    2310,  4, }, // Asia/Aden
    {  101, 5395,    2314,  6, }, // Asia/Almaty
    {  101, 3968,    2320,  5, }, // Asia/Amman
    {  101, 5511,   52002,  7, }, // Asia/Anadyr
    {  101, 5523,    2331,  5, }, // Asia/Aqtau
    {  101, 5368,    2336,  6, }, // Asia/Aqtobe
    {  101,  964,    2342,  7, }, // Asia/Ashgabat
    {  101, 8624,    2349,  6, }, // Asia/Atyrau
    {  101, 3607,    2355,  6, }, // Asia/Baghdad
    {  101, 5534,   37080,  7, }, // Asia/Bahrain
    {  101, 3653,    2368,  4, }, // Asia/Baku
    {  101, 4466,    2372,  7, }, // Asia/Bangkok
    {  101, 3571,    2379,  7, }, // Asia/Barnaul
    {  101, 4086,   28695,  6, }, // Asia/Beirut
    {  101, 3778,    2392,  6, }, // Asia/Bishkek
    {  101, 5989,   28707,  6, }, // Asia/Brunei
    {  101, 4601,    2404,  4, }, // Asia/Chita
    {  101, 4511,    2408,  7, }, // Asia/Colombo
    {  101, 4540,    2415,  6, }, // Asia/Damascus
    {  101, 1093,    2421,  5, }, // Asia/Dhaka
    {  101, 6192,    2426,  4, }, // Asia/Dili
    {  101, 3596,   28728,  5, }, // Asia/Dubai
    {  101, 6159,    2435,  7, }, // Asia/Dushanbe
    {  101, 8636,    2442,  9, }, // Asia/Famagusta
    {  101, 8651,    2451,  4, }, // Asia/Gaza
    {  101, 4795,    2455,  6, }, // Asia/Hebron
    {  101, 1259,    2461,  7, }, // Asia/Ho_Chi_Minh
    {  101, 2704,   52034,  7, }, // Asia/Hong_Kong
    {  101, 4771,    2475,  4, }, // Asia/Hovd
    {  101, 4184,    2479,  7, }, // Asia/Irkutsk
    {  101, 7006,   28770,  8, }, // Asia/Jakarta
    {  101, 6992,   28778,  8, }, // Asia/Jayapura
    {  101, 1290,    2501,  9, }, // Asia/Jerusalem
    {  101, 3560,    2510,  5, }, // Asia/Kabul
    {  101, 4316,   52059, 23, }, // Asia/Kamchatka
    {  101, 4239,    2540,  6, }, // Asia/Karachi
    {  101, 1185,    2546,  8, }, // Asia/Kathmandu
    {  101, 8661,    2554,  7, }, // Asia/Khandyga
    {  101,  992,   52082,  9, }, // Asia/Kolkata
    {  101, 4197,   37110, 10, }, // Asia/Krasnoyarsk
    {  101, 4859,    2580, 12, }, // Asia/Kuala_Lumpur
    {  101, 5976,    2592,  6, }, // Asia/Kuching
    {  101, 5547,   28825,  6, }, // Asia/Kuwait
    {  101, 1211,    2604,  5, }, // Asia/Macau
    {  101, 4017,    2609,  7, }, // Asia/Magadan
    {  101, 1349,    2616,  7, }, // Asia/Makassar
    {  101, 7182,    2623,  6, }, // Asia/Manila
    {  101, 6920,    2629,  6, }, // Asia/Muscat
    {  101, 2554,    2635,  6, }, // Asia/Nicosia
    {  101, 8675,    2642, 11, }, // Asia/Novokuznetsk
    {  101, 4135,    2653, 11, }, // Asia/Novosibirsk
    {  101, 4229,    2664,  4, }, // Asia/Omsk
    {  101, 7145,   28871,  7, }, // Asia/Oral
    {  101, 6961,   52091,  8, }, // Asia/Phnom_Penh
    {  101, 8693,    2681,  9, }, // Asia/Pontianak
    {  101, 4214,   28895,  7, }, // Asia/Pyongyang
    {  101, 5559,    2697,  5, }, // Asia/Qatar
    {  101, 8708,   37137,  8, }, // Asia/Qostanay
    {  101, 4269,    2710,  9, }, // Asia/Qyzylorda
    {  101, 3584,   52099,  7, }, // Asia/Riyadh
    {  101, 4408,   56527, 14, }, // Asia/Sakhalin
    {  101, 7245,    2739,  9, }, // Asia/Samarkand
    {  101, 3279,    2748,  4, }, // Asia/Seoul
    {  101, 1053,   28933,  6, }, // Asia/Shanghai
    {  101, 3300,    2758,  8, }, // Asia/Singapore
    {  101, 4297,    2766, 13, }, // Asia/Srednekolymsk
    {  101, 3263,   28961,  6, }, // Asia/Taipei
    {  101, 4781,    2785,  7, }, // Asia/Tashkent
    {  101, 3915,   35206,  7, }, // Asia/Tbilisi
    {  101, 2772,    2796,  7, }, // Asia/Tehran
    {  101, 1317,    2803,  7, }, // Asia/Thimphu
    {  101, 2821,    2810,  5, }, // Asia/Tokyo
    {  101, 4572,    2815,  5, }, // Asia/Tomsk
    {  101, 1021,    2820, 10, }, // Asia/Ulaanbaatar
    {  101, 1159,    2830,  6, }, // Asia/Urumqi
    {  101, 8722,   52125,  9, }, // Asia/Ust-Nera
    {  101, 6977,   52134,  8, }, // Asia/Vientiane
    {  101, 4710,    2851, 11, }, // Asia/Vladivostok
    {  101, 4828,   37183,  6, }, // Asia/Yakutsk
    {  101, 1235,   56541,  5, }, // Asia/Yangon
    {  101, 3883,    2875, 12, }, // Asia/Yekaterinburg
    {  101, 3747,    2887,  6, }, // Asia/Yerevan
    {  101, 3663,   56546, 14, }, // Atlantic/Azores
    {  101, 5671,   56560, 16, }, // Atlantic/Bermuda
    {  101, 6628,   56576, 15, }, // Atlantic/Canary
    {  101, 3727,    2944, 10, }, // Atlantic/Cape_Verde
    {  101, 1395,   56591, 15, }, // Atlantic/Faroe
    {  101, 8736,   37264,  7, }, // Atlantic/Madeira
    {  101, 2748,   52214,  9, }, // Atlantic/Reykjavik
    {  101, 7272,   56606, 13, }, // Atlantic/South_Georgia
    {  101, 6821,   56619, 20, }, // Atlantic/St_Helena
    {  101, 6644,   52256,  6, }, // Atlantic/Stanley
    {  101, 1670,    3035,  8, }, // Australia/Adelaide
    {  101, 1635,    3043,  7, }, // Australia/Brisbane
    {  101, 1799,    3050, 11, }, // Australia/Broken_Hill
    {  101, 1583,    3061,  6, }, // Australia/Darwin
    {  101, 3637,   29240,  4, }, // Australia/Eucla
    {  101, 1516,    3072,  6, }, // Australia/Hobart
    {  101, 8753,    3078,  8, }, // Australia/Lindeman
    {  101, 1547,    3086,  8, }, // Australia/Lord_Howe
    {  101, 1727,   29252,  8, }, // Australia/Melbourne
    {  101, 1762,    3101,  4, }, // Australia/Perth
    {  101, 1463,   52276,  6, }, // Australia/Sydney
    {  101, 6441,    3111,  9, }, // Europe/Amsterdam
    {  101, 6216,    3120,  7, }, // Europe/Andorra
    {  101, 3620,   29281,  9, }, // Europe/Astrakhan
    {  101, 2300,   56639,  7, }, // Europe/Athens
    {  101, 6336,    3139,  7, }, // Europe/Belgrade
    {  101, 4757,    3146,  6, }, // Europe/Berlin
    {  101, 6488,    3152, 10, }, // Europe/Bratislava
    {  101, 2167,   52296,  8, }, // Europe/Brussels
    {  101, 3928,    3170,  8, }, // Europe/Bucharest
    {  101, 3806,    3178,  8, }, // Europe/Budapest
    {  101, 8772,   37368,  8, }, // Europe/Busingen
    {  101, 2583,    3209,  7, }, // Europe/Chisinau
    {  101, 6289,   52323, 10, }, // Europe/Copenhagen
    {  101, 2338,    3226,  6, }, // Europe/Dublin
    {  101, 6307,    3232,  9, }, // Europe/Gibraltar
    {  101, 8788,    3241,  6, }, // Europe/Guernsey
    {  101, 6612,   52333,  9, }, // Europe/Helsinki
    {  101, 8804,   56646, 10, }, // Europe/Isle_of_Man
    {  101, 1130,    3264,  7, }, // Europe/Istanbul
    {  101, 8823,   52351,  6, }, // Europe/Jersey
    {  101, 3979,    3276, 11, }, // Europe/Kaliningrad
    {  101, 8929,    3287,  5, }, // Europe/Kirov
    {  101, 2527,    3292,  4, }, // Europe/Kyiv
    {  101, 3213,    3296,  8, }, // Europe/Lisbon
    {  101, 6506,   29432,  7, }, // Europe/Ljubljana
    {  101, 2501,    3313,  6, }, // Europe/London
    {  101, 6365,   29445, 10, }, // Europe/Luxembourg
    {  101, 4892,    3330,  6, }, // Europe/Madrid
    {  101, 6397,   29461,  6, }, // Europe/Malta
    {  101, 6582,    3341,  9, }, // Europe/Mariehamn
    {  101, 3693,    3350,  5, }, // Europe/Minsk
    {  101, 6410,    3355,  6, }, // Europe/Monaco
    {  101, 3537,   56656,  6, }, // Europe/Moscow
    {  101, 6458,    3367,  4, }, // Europe/Oslo
    {  101, 4284,    3371,  5, }, // Europe/Paris
    {  101, 6424,    3376,  9, }, // Europe/Podgorica
    {  101, 6275,    3385,  5, }, // Europe/Prague
    {  101, 8837,    3390,  4, }, // Europe/Riga
    {  101, 6324,    3394,  3, }, // Europe/Rome
    {  101, 4331,    3397,  6, }, // Europe/Samara
    {  101, 6470,    3403, 10, }, // Europe/San_Marino
    {  101, 6245,    3413,  7, }, // Europe/Sarajevo
    {  101, 4451,    3420,  7, }, // Europe/Saratov
    {  101, 8849,   52357, 11, }, // Europe/Simferopol
    {  101, 6383,   52368,  6, }, // Europe/Skopje
    {  101, 6599,    3442,  4, }, // Europe/Sofia
    {  101, 6523,   52374,  9, }, // Europe/Stockholm
    {  101, 8867,    3455,  6, }, // Europe/Tallinn
    {  101, 6202,    3461,  6, }, // Europe/Tirane
    {  101, 8882,   52383,  9, }, // Europe/Ulyanovsk
    {  101, 6352,    3476,  5, }, // Europe/Vaduz
    {  101, 6567,    3481,  7, }, // Europe/Vatican
    {  101, 6231,    3488,  4, }, // Europe/Vienna
    {  101, 8899,   52392,  7, }, // Europe/Vilnius
    {  101, 4727,    3499,  9, }, // Europe/Volgograd
    {  101, 3190,    3508,  7, }, // Europe/Warsaw
    {  101, 6261,    3515,  6, }, // Europe/Zagreb
    {  101, 6540,   37442,  5, }, // Europe/Zurich
    {  101, 5078,    3527, 12, }, // Indian/Antananarivo
    {  101, 6947,    3539,  5, }, // Indian/Chagos
    {  101, 6046,   56662, 13, }, // Indian/Christmas
    {  101, 6063,   56675, 15, }, // Indian/Cocos
    {  101, 5029,   56690,  8, }, // Indian/Comoro
    {  101, 6661,    3578,  8, }, // Indian/Kerguelen
    {  101, 7260,   29694,  3, }, // Indian/Mahe
    {  101, 7050,   56698, 10, }, // Indian/Maldives
    {  101, 4069,   56708,  8, }, // Indian/Mauritius
    {  101, 5098,   56716,  7, }, // Indian/Mayotte
    {  101, 7211,   52480,  7, }, // Indian/Reunion
    {  101, 4422,    3616,  4, }, // Pacific/Apia
    {  101,  932,    3620,  6, }, // Pacific/Auckland
    {  101, 3706,   52487,  9, }, // Pacific/Bougainville
    {  101, 3015,    3634,  5, }, // Pacific/Chatham
    {  101, 3157,    3639,  4, }, // Pacific/Chuuk
    {  101, 2237,   56723, 13, }, // Pacific/Easter
    {  101, 7360,   52541,  5, }, // Pacific/Efate
    {  101, 7327,    3659,  7, }, // Pacific/Fakaofo
    {  101, 3902,   37539,  5, }, // Pacific/Fiji
    {  101, 7343,    3670,  8, }, // Pacific/Funafuti
    {  101, 6678,   56736, 19, }, // Pacific/Galapagos
    {  101, 6696,   56755, 16, }, // Pacific/Gambier
    {  101, 3822,    3713, 11, }, // Pacific/Guadalcanal
    {  101, 6018,    3724,  4, }, // Pacific/Guam
    {  101, 3049,    3728,  6, }, // Pacific/Kanton
    {  101, 3998,    3734, 10, }, // Pacific/Kiritimati
    {  101, 7019,    3744,  6, }, // Pacific/Kosrae
    {  101, 2842,   29841, 10, }, // Pacific/Kwajalein
    {  101, 4877,   37574,  7, }, // Pacific/Majuro
    {  101, 4051,   56771, 18, }, // Pacific/Marquesas
    {  101, 8914,   52622,  6, }, // Pacific/Midway
    {  101, 7084,    3798,  5, }, // Pacific/Nauru
    {  101, 7132,   52636,  4, }, // Pacific/Niue
    {  101, 4168,    3807,  7, }, // Pacific/Norfolk
    {  101, 7098,    3814,  5, }, // Pacific/Noumea
    {  101, 3126,    3819,  9, }, // Pacific/Pago_Pago
    {  101, 7155,    3828,  5, }, // Pacific/Palau
    {  101, 7194,   56789,  7, }, // Pacific/Pitcairn
    {  101, 3096,    3840,  6, }, // Pacific/Pohnpei
    {  101, 4807,    3846, 11, }, // Pacific/Port_Moresby
    {  101, 6076,    3857,  9, }, // Pacific/Rarotonga
    {  101, 6031,   29935,  6, }, // Pacific/Saipan
    {  101, 7312,    3872,  5, }, // Pacific/Tahiti
    {  101, 6712,    3877,  6, }, // Pacific/Tarawa
    {  101, 4583,    3883,  9, }, // Pacific/Tongatapu
    {  101, 7392,   29955,  4, }, // Pacific/Wake
    {  101, 7405,    3896,  6, }, // Pacific/Wallis
    {  102,   14,       0,  6, }, // Africa/Asmara Colognian/Latin/Germany
    {  102, 2320,   56796,  6, }, // Africa/Cairo
    {  102, 4744,   56802,  6, }, // Africa/Lagos
    {  102, 4435,       6,  8, }, // Africa/Sao_Tome
    {  102, 4152,   56808,  7, }, // Africa/Windhoek
    {  102, 4252,      14,  8, }, // America/Asuncion
    {  102,  237,      22,  8, }, // America/Atikokan
    {  102, 7623,      30, 17, }, // America/Bahia_Banderas
    {  102, 3868,      47,  6, }, // America/Cancun
    {  102, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  102, 5422,   56815, 10, }, // America/Costa_Rica
    {  102, 5723,      66,  7, }, // America/Curacao
    {  102, 4841,   56825,  9, }, // America/Guayaquil
    {  102,  481,   56834, 15, }, // America/Indiana/Knox
    {  102, 7961,   56849, 18, }, // America/Indiana/Marengo
    {  102, 7985,   56867, 21, }, // America/Indiana/Petersburg
    {  102, 8012,   56888, 20, }, // America/Indiana/Tell_City
    {  102, 8038,   56908, 16, }, // America/Indiana/Vevay
    {  102, 8060,   56924, 20, }, // America/Indiana/Vincennes
    {  102, 8086,   56944, 18, }, // America/Indiana/Winamac
    {  102, 8125,   56962, 22, }, // America/Kentucky/Monticello
    {  102, 5932,     207, 22, }, // America/Lower_Princes
    {  102, 8220,     229,  6, }, // America/Merida
    {  102, 2949,   56984, 14, }, // America/Mexico_City
    {  102, 1850,     235, 19, }, // America/Noronha
    {  102, 8301,   56998, 21, }, // America/North_Dakota/Beulah
    {  102, 8329,   57019, 21, }, // America/North_Dakota/Center
    {  102, 8357,   57040, 24, }, // America/North_Dakota/New_Salem
    {  102,  393,     317,  4, }, // America/Nuuk
    {  102, 6899,     321, 16, }, // America/Scoresbysund
    {  102, 8508,   57064, 15, }, // America/St_Barthelemy
    {  102, 2061,   57079, 11, }, // America/St_Johns
    {  102, 5863,   57090, 10, }, // America/St_Kitts
    {  102, 5880,   57100, 16, }, // America/St_Lucia
    {  102,  855,   57116, 11, }, // America/St_Thomas
    {  102, 5913,   57127, 15, }, // America/St_Vincent
    {  102, 8552,    7517,  7, }, // America/Yakutat
    {  102, 6133,   57142, 27, }, // Antarctica/DumontDUrville
    {  102, 7295,   57169, 36, }, // Antarctica/Syowa
    {  102, 7374,    4147,  6, }, // Antarctica/Vostok
    {  102, 5368,   57205,  6, }, // Asia/Aqtobe
    {  102,  964,   57211,  9, }, // Asia/Ashgabat
    {  102, 3607,    4162,  6, }, // Asia/Baghdad
    {  102, 3778,   57220,  8, }, // Asia/Bishkek
    {  102, 4540,    4201,  8, }, // Asia/Damascus
    {  102, 6159,   57228,  9, }, // Asia/Dushanbe
    {  102, 8651,   57237,  5, }, // Asia/Gaza
    {  102, 1259,   57242, 30, }, // Asia/Ho_Chi_Minh
    {  102, 2704,    4231,  8, }, // Asia/Hong_Kong
    {  102, 4316,   57272, 11, }, // Asia/Kamchatka
    {  102, 1185,     448,  9, }, // Asia/Kathmandu
    {  102,  992,     457,  7, }, // Asia/Kolkata
    {  102, 4197,    4288, 11, }, // Asia/Krasnoyarsk
    {  102, 1211,     464,  5, }, // Asia/Macau
    {  102, 2554,   57283,  8, }, // Asia/Nicosia
    {  102, 6961,   57291,  9, }, // Asia/Phnom_Penh
    {  102, 4269,   57300,  9, }, // Asia/Qyzylorda
    {  102, 3584,   33571,  5, }, // Asia/Riyadh
    {  102, 1053,   57309,  9, }, // Asia/Shanghai
    {  102, 3300,   57318, 10, }, // Asia/Singapore
    {  102, 4781,   57328,  9, }, // Asia/Tashkent
    {  102, 3915,   57337,  6, }, // Asia/Tbilisi
    {  102, 1021,   26923, 10, }, // Asia/Ulaanbaatar
    {  102, 1159,   57343, 11, }, // Asia/Urumqi
    {  102, 4710,    4361, 11, }, // Asia/Vladivostok
    {  102, 4828,    7873,  7, }, // Asia/Yakutsk
    {  102, 1235,   57354,  6, }, // Asia/Yangon
    {  102, 3883,    4380, 13, }, // Asia/Yekaterinburg
    {  102, 3747,   57360,  6, }, // Asia/Yerevan
    {  102, 3663,   57366,  6, }, // Atlantic/Azores
    {  102, 6628,   57372,  8, }, // Atlantic/Canary
    {  102, 3727,   57380,  9, }, // Atlantic/Cape_Verde
    {  102, 1395,   57389,  5, }, // Atlantic/Faroe
    {  102, 6821,   57394, 11, }, // Atlantic/St_Helena
    {  102, 2300,    4438,  5, }, // Europe/Athens
    {  102, 6336,   57405,  7, }, // Europe/Belgrade
    {  102, 2167,   22766,  7, }, // Europe/Brussels
    {  102, 3928,   27029,  8, }, // Europe/Bucharest
    {  102, 6289,   57412, 10, }, // Europe/Copenhagen
    {  102, 6307,   57422,  9, }, // Europe/Gibraltar
    {  102, 8788,   57431,  7, }, // Europe/Guernsey
    {  102, 8804,   57438,  9, }, // Europe/Isle_of_Man
    {  102, 1130,   57447,  8, }, // Europe/Istanbul
    {  102, 8823,   57455,  6, }, // Europe/Jersey
    {  102, 3979,   57461, 13, }, // Europe/Kaliningrad
    {  102, 2527,     514,  4, }, // Europe/Kyiv
    {  102, 3213,    4499,  8, }, // Europe/Lisbon
    {  102, 6365,   57474, 11, }, // Europe/Luxembourg
    {  102, 3537,   57485,  6, }, // Europe/Moscow
    {  102, 6275,    8201,  4, }, // Europe/Prague
    {  102, 8837,   57491,  5, }, // Europe/Riga
    {  102, 6324,    6064,  3, }, // Europe/Rome
    {  102, 6202,   57496,  7, }, // Europe/Tirane
    {  102, 6567,   57503, 12, }, // Europe/Vatican
    {  102, 6231,   57515,  4, }, // Europe/Vienna
    {  102, 4727,   57519,  9, }, // Europe/Volgograd
    {  102, 3190,   57528,  8, }, // Europe/Warsaw
    {  102, 6261,   57536,  6, }, // Europe/Zagreb
    {  102, 6540,   57542,  7, }, // Europe/Zurich
    {  102, 7050,   57549,  8, }, // Indian/Maldives
    {  102, 7211,     547,  7, }, // Indian/Reunion
    {  102, 3157,     554,  5, }, // Pacific/Chuuk
    {  102, 6678,   57557, 10, }, // Pacific/Galapagos
    {  102, 6018,   57567,  4, }, // Pacific/Guam
    {  102, 2723,   57571,  8, }, // Pacific/Honolulu
    {  102, 7194,    8422,  7, }, // Pacific/Pitcairn
    {  102, 3096,     586,  7, }, // Pacific/Pohnpei
    {  103,   14,       0,  6, }, // Africa/Asmara Coptic/Coptic/Egypt
    {  103, 4435,       6,  8, }, // Africa/Sao_Tome
    {  103, 4252,      14,  8, }, // America/Asuncion
    {  103,  237,      22,  8, }, // America/Atikokan
    {  103, 7623,      30, 17, }, // America/Bahia_Banderas
    {  103, 3868,      47,  6, }, // America/Cancun
    {  103, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  103, 5723,      66,  7, }, // America/Curacao
    {  103,  481,      73, 13, }, // America/Indiana/Knox
    {  103, 7961,      86, 16, }, // America/Indiana/Marengo
    {  103, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  103, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  103, 8038,     139, 14, }, // America/Indiana/Vevay
    {  103, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  103, 8086,     171, 16, }, // America/Indiana/Winamac
    {  103, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  103, 5932,     207, 22, }, // America/Lower_Princes
    {  103, 8220,     229,  6, }, // America/Merida
    {  103, 1850,     235, 19, }, // America/Noronha
    {  103, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  103, 8329,     274, 20, }, // America/North_Dakota/Center
    {  103, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  103,  393,     317,  4, }, // America/Nuuk
    {  103, 6899,     321, 16, }, // America/Scoresbysund
    {  103, 8508,     337, 14, }, // America/St_Barthelemy
    {  103, 2061,     351, 10, }, // America/St_Johns
    {  103, 5863,     361,  9, }, // America/St_Kitts
    {  103, 5880,     370,  9, }, // America/St_Lucia
    {  103,  855,     379, 10, }, // America/St_Thomas
    {  103, 5913,     389, 11, }, // America/St_Vincent
    {  103, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  103, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  103, 1185,     448,  9, }, // Asia/Kathmandu
    {  103,  992,     457,  7, }, // Asia/Kolkata
    {  103, 1211,     464,  5, }, // Asia/Macau
    {  103, 1235,     477,  6, }, // Asia/Yangon
    {  103, 1395,     483,  5, }, // Atlantic/Faroe
    {  103, 6821,     488, 10, }, // Atlantic/St_Helena
    {  103, 2527,     514,  4, }, // Europe/Kyiv
    {  103, 7211,     547,  7, }, // Indian/Reunion
    {  103, 3157,     554,  5, }, // Pacific/Chuuk
    {  103, 3096,     586,  7, }, // Pacific/Pohnpei
    {  104,   14,       0,  6, }, // Africa/Asmara Cornish/Latin/United Kingdom
    {  104, 4435,       6,  8, }, // Africa/Sao_Tome
    {  104, 4252,      14,  8, }, // America/Asuncion
    {  104,  237,      22,  8, }, // America/Atikokan
    {  104, 7623,      30, 17, }, // America/Bahia_Banderas
    {  104, 3868,      47,  6, }, // America/Cancun
    {  104, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  104, 5723,      66,  7, }, // America/Curacao
    {  104,  481,      73, 13, }, // America/Indiana/Knox
    {  104, 7961,      86, 16, }, // America/Indiana/Marengo
    {  104, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  104, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  104, 8038,     139, 14, }, // America/Indiana/Vevay
    {  104, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  104, 8086,     171, 16, }, // America/Indiana/Winamac
    {  104, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  104, 5932,     207, 22, }, // America/Lower_Princes
    {  104, 8220,     229,  6, }, // America/Merida
    {  104, 1850,     235, 19, }, // America/Noronha
    {  104, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  104, 8329,     274, 20, }, // America/North_Dakota/Center
    {  104, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  104,  393,     317,  4, }, // America/Nuuk
    {  104, 6899,     321, 16, }, // America/Scoresbysund
    {  104, 8508,     337, 14, }, // America/St_Barthelemy
    {  104, 2061,     351, 10, }, // America/St_Johns
    {  104, 5863,     361,  9, }, // America/St_Kitts
    {  104, 5880,     370,  9, }, // America/St_Lucia
    {  104,  855,     379, 10, }, // America/St_Thomas
    {  104, 5913,     389, 11, }, // America/St_Vincent
    {  104, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  104, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  104, 1185,     448,  9, }, // Asia/Kathmandu
    {  104,  992,     457,  7, }, // Asia/Kolkata
    {  104, 1211,     464,  5, }, // Asia/Macau
    {  104, 1235,     477,  6, }, // Asia/Yangon
    {  104, 1395,     483,  5, }, // Atlantic/Faroe
    {  104, 6821,     488, 10, }, // Atlantic/St_Helena
    {  104, 2527,     514,  4, }, // Europe/Kyiv
    {  104, 7211,     547,  7, }, // Indian/Reunion
    {  104, 3157,     554,  5, }, // Pacific/Chuuk
    {  104, 3096,     586,  7, }, // Pacific/Pohnpei
    {  105,   14,       0,  6, }, // Africa/Asmara Corsican/Latin/France
    {  105, 4435,       6,  8, }, // Africa/Sao_Tome
    {  105, 4252,      14,  8, }, // America/Asuncion
    {  105,  237,      22,  8, }, // America/Atikokan
    {  105, 7623,      30, 17, }, // America/Bahia_Banderas
    {  105, 3868,      47,  6, }, // America/Cancun
    {  105, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  105, 5723,      66,  7, }, // America/Curacao
    {  105,  481,      73, 13, }, // America/Indiana/Knox
    {  105, 7961,      86, 16, }, // America/Indiana/Marengo
    {  105, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  105, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  105, 8038,     139, 14, }, // America/Indiana/Vevay
    {  105, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  105, 8086,     171, 16, }, // America/Indiana/Winamac
    {  105, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  105, 5932,     207, 22, }, // America/Lower_Princes
    {  105, 8220,     229,  6, }, // America/Merida
    {  105, 1850,     235, 19, }, // America/Noronha
    {  105, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  105, 8329,     274, 20, }, // America/North_Dakota/Center
    {  105, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  105,  393,     317,  4, }, // America/Nuuk
    {  105, 6899,     321, 16, }, // America/Scoresbysund
    {  105, 8508,     337, 14, }, // America/St_Barthelemy
    {  105, 2061,     351, 10, }, // America/St_Johns
    {  105, 5863,     361,  9, }, // America/St_Kitts
    {  105, 5880,     370,  9, }, // America/St_Lucia
    {  105,  855,     379, 10, }, // America/St_Thomas
    {  105, 5913,     389, 11, }, // America/St_Vincent
    {  105, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  105, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  105, 1185,     448,  9, }, // Asia/Kathmandu
    {  105,  992,     457,  7, }, // Asia/Kolkata
    {  105, 1211,     464,  5, }, // Asia/Macau
    {  105, 1235,     477,  6, }, // Asia/Yangon
    {  105, 1395,     483,  5, }, // Atlantic/Faroe
    {  105, 6821,     488, 10, }, // Atlantic/St_Helena
    {  105, 2527,     514,  4, }, // Europe/Kyiv
    {  105, 7211,     547,  7, }, // Indian/Reunion
    {  105, 3157,     554,  5, }, // Pacific/Chuuk
    {  105, 3096,     586,  7, }, // Pacific/Pohnpei
    {  106, 5059,    3902, 11, }, // Africa/Addis_Ababa Croatian/Latin/Croatia
    {  106, 7420,   57579,  5, }, // Africa/Algiers
    {  106,   14,       0,  6, }, // Africa/Asmara
    {  106, 2320,    4690,  5, }, // Africa/Cairo
    {  106, 5043,   33286,  7, }, // Africa/Djibouti
    {  106, 6869,   20418,  4, }, // Africa/Lome
    {  106, 4435,       6,  8, }, // Africa/Sao_Tome
    {  106, 5607,   33300,  7, }, // America/Anguilla
    {  106, 4252,      14,  8, }, // America/Asuncion
    {  106,  237,      22,  8, }, // America/Atikokan
    {  106, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  106, 3868,   33314,  6, }, // America/Cancun
    {  106, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  106, 5422,   33320,  9, }, // America/Costa_Rica
    {  106, 5723,      66,  7, }, // America/Curacao
    {  106, 5739,   21666,  8, }, // America/Dominica
    {  106, 5441,    6806,  8, }, // America/El_Salvador
    {  106, 5786,   20526,  9, }, // America/Guadeloupe
    {  106, 3760,   33338,  9, }, // America/Guatemala
    {  106, 6932,   57584,  7, }, // America/Guyana
    {  106,  481,      73, 13, }, // America/Indiana/Knox
    {  106, 7961,      86, 16, }, // America/Indiana/Marengo
    {  106, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  106, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  106, 8038,     139, 14, }, // America/Indiana/Vevay
    {  106, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  106, 8086,     171, 16, }, // America/Indiana/Winamac
    {  106, 2799,   33347,  7, }, // America/Jamaica
    {  106, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  106, 5932,     207, 22, }, // America/Lower_Princes
    {  106, 8220,    7126,  6, }, // America/Merida
    {  106, 2949,   57591, 16, }, // America/Mexico_City
    {  106, 1850,     235, 19, }, // America/Noronha
    {  106, 8301,   33354, 23, }, // America/North_Dakota/Beulah
    {  106, 8329,   33377, 23, }, // America/North_Dakota/Center
    {  106, 8357,   33400, 26, }, // America/North_Dakota/New_Salem
    {  106,  393,     317,  4, }, // America/Nuuk
    {  106, 5843,   33426,  9, }, // America/Puerto_Rico
    {  106, 6899,     321, 16, }, // America/Scoresbysund
    {  106, 8508,   35936, 16, }, // America/St_Barthelemy
    {  106, 2061,     351, 10, }, // America/St_Johns
    {  106, 5863,     361,  9, }, // America/St_Kitts
    {  106, 5880,     370,  9, }, // America/St_Lucia
    {  106,  855,     379, 10, }, // America/St_Thomas
    {  106, 5913,     389, 11, }, // America/St_Vincent
    {  106, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  106, 5395,   57607,  8, }, // Asia/Almaty
    {  106, 5511,    7600,  6, }, // Asia/Anadyr
    {  106, 5523,    5609,  5, }, // Asia/Aqtau
    {  106, 5368,    7606,  6, }, // Asia/Aqtobe
    {  106,  964,   57615,  7, }, // Asia/Ashgabat
    {  106, 3607,    4162,  6, }, // Asia/Baghdad
    {  106, 5534,    4168,  7, }, // Asia/Bahrain
    {  106, 3778,   33455,  6, }, // Asia/Bishkek
    {  106, 5989,   33461,  6, }, // Asia/Brunei
    {  106, 4601,   57622,  4, }, // Asia/Chita
    {  106, 4540,    4201,  6, }, // Asia/Damascus
    {  106, 6159,   33468,  7, }, // Asia/Dushanbe
    {  106, 1259,   33475,  9, }, // Asia/Ho_Chi_Minh
    {  106, 7006,   33484,  8, }, // Asia/Jakarta
    {  106, 1290,   33501,  9, }, // Asia/Jerusalem
    {  106, 4316,   33510,  8, }, // Asia/Kamchatka
    {  106, 4239,   33518,  6, }, // Asia/Karachi
    {  106, 1185,    5711,  8, }, // Asia/Kathmandu
    {  106, 8661,   33524,  7, }, // Asia/Khandyga
    {  106,  992,     457,  7, }, // Asia/Kolkata
    {  106, 4197,    4288, 11, }, // Asia/Krasnoyarsk
    {  106, 5547,    7725,  6, }, // Asia/Kuwait
    {  106, 1211,    7731,  5, }, // Asia/Macau
    {  106, 2554,   33542,  8, }, // Asia/Nicosia
    {  106, 8675,   33550, 12, }, // Asia/Novokuznetsk
    {  106, 4214,   33562,  9, }, // Asia/Pyongyang
    {  106, 5559,    4332,  5, }, // Asia/Qatar
    {  106, 8708,     469,  8, }, // Asia/Qostanay
    {  106, 4269,    7778,  9, }, // Asia/Qyzylorda
    {  106, 3584,   33571,  5, }, // Asia/Riyadh
    {  106, 4408,   33576,  7, }, // Asia/Sakhalin
    {  106, 3279,    7798,  4, }, // Asia/Seoul
    {  106, 1053,   33583,  6, }, // Asia/Shanghai
    {  106, 3300,   20900,  8, }, // Asia/Singapore
    {  106, 4297,    7817, 13, }, // Asia/Srednekolymsk
    {  106, 4781,   33589,  7, }, // Asia/Tashkent
    {  106, 2772,    7842,  7, }, // Asia/Tehran
    {  106, 2821,    4356,  5, }, // Asia/Tokyo
    {  106, 1021,   26923, 10, }, // Asia/Ulaanbaatar
    {  106, 1159,   33596,  6, }, // Asia/Urumqi
    {  106, 4828,    7873,  7, }, // Asia/Yakutsk
    {  106, 1235,    7880,  6, }, // Asia/Yangon
    {  106, 3883,    4380, 13, }, // Asia/Yekaterinburg
    {  106, 3747,   26947,  6, }, // Asia/Yerevan
    {  106, 3663,   33612,  5, }, // Atlantic/Azores
    {  106, 5671,   57626,  7, }, // Atlantic/Bermuda
    {  106, 6628,    4398,  6, }, // Atlantic/Canary
    {  106, 1395,   57633, 14, }, // Atlantic/Faroe
    {  106, 7272,   57647, 14, }, // Atlantic/South_Georgia
    {  106, 6821,   33626, 12, }, // Atlantic/St_Helena
    {  106, 6216,   33638,  6, }, // Europe/Andorra
    {  106, 3620,   33644,  8, }, // Europe/Astrakhan
    {  106, 2300,   27009,  5, }, // Europe/Athens
    {  106, 6336,    8060,  7, }, // Europe/Belgrade
    {  106, 2167,   57661,  9, }, // Europe/Brussels
    {  106, 3928,   33663,  8, }, // Europe/Bucharest
    {  106, 3806,   33671, 10, }, // Europe/Budapest
    {  106, 2583,   33681,  8, }, // Europe/Chisinau
    {  106, 6289,    4475, 10, }, // Europe/Copenhagen
    {  106, 8804,   57670,  8, }, // Europe/Isle_of_Man
    {  106, 3979,   33711, 12, }, // Europe/Kaliningrad
    {  106, 2527,   33723,  5, }, // Europe/Kyiv
    {  106, 3213,   33728,  7, }, // Europe/Lisbon
    {  106, 6365,    8172, 10, }, // Europe/Luxembourg
    {  106, 6410,    6044,  6, }, // Europe/Monaco
    {  106, 3537,   22880,  6, }, // Europe/Moscow
    {  106, 4284,   33735,  5, }, // Europe/Paris
    {  106, 6275,    8201,  4, }, // Europe/Prague
    {  106, 6324,   33740,  3, }, // Europe/Rome
    {  106, 8849,   33743, 11, }, // Europe/Simferopol
    {  106, 6383,   33754,  7, }, // Europe/Skopje
    {  106, 6599,   33761,  6, }, // Europe/Sofia
    {  106, 6202,   21213,  6, }, // Europe/Tirane
    {  106, 8882,    8261,  9, }, // Europe/Ulyanovsk
    {  106, 6567,    6096,  7, }, // Europe/Vatican
    {  106, 6231,   33775,  3, }, // Europe/Vienna
    {  106, 3190,   33778,  7, }, // Europe/Warsaw
    {  106, 6540,    4577,  6, }, // Europe/Zurich
    {  106, 7050,   33819,  7, }, // Indian/Maldives
    {  106, 4069,   33826, 10, }, // Indian/Mauritius
    {  106, 7211,     547,  7, }, // Indian/Reunion
    {  106, 3157,     554,  5, }, // Pacific/Chuuk
    {  106, 3902,   33836,  5, }, // Pacific/Fiji
    {  106, 2723,    4611,  8, }, // Pacific/Honolulu
    {  106, 3049,    4619,  9, }, // Pacific/Kanton
    {  106, 4051,   57678, 18, }, // Pacific/Marquesas
    {  106, 3096,     586,  7, }, // Pacific/Pohnpei
    {  108, 6788,   57696,  7, }, // Africa/Abidjan Czech/Latin/Czechia
    {  108, 5059,    3902, 11, }, // Africa/Addis_Ababa
    {  108, 7420,   57703,  5, }, // Africa/Algiers
    {  108,   14,       0,  6, }, // Africa/Asmara
    {  108, 2320,   57708,  6, }, // Africa/Cairo
    {  108, 5043,   33286,  7, }, // Africa/Djibouti
    {  108, 4524,   57714,  7, }, // Africa/Khartoum
    {  108, 6869,   20418,  4, }, // Africa/Lome
    {  108, 5113,   57721,  8, }, // Africa/Mogadishu
    {  108, 5271,   57729,  8, }, // Africa/Ndjamena
    {  108, 6803,   57737,  8, }, // Africa/Nouakchott
    {  108, 4435,   57745, 11, }, // Africa/Sao_Tome
    {  108, 2866,   57756,  8, }, // Africa/Tripoli
    {  108,  270,   20479,  7, }, // America/Argentina/Cordoba
    {  108, 4252,      14,  8, }, // America/Asuncion
    {  108,  237,      22,  8, }, // America/Atikokan
    {  108, 3679,      30,  5, }, // America/Bahia
    {  108, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  108, 7646,   35896,  5, }, // America/Belem
    {  108, 4361,   20507,  6, }, // America/Bogota
    {  108, 3868,      47,  6, }, // America/Cancun
    {  108, 5496,   57764, 17, }, // America/Cayman
    {  108, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  108, 5422,   33320,  9, }, // America/Costa_Rica
    {  108, 5723,      66,  7, }, // America/Curacao
    {  108, 5739,   21666,  8, }, // America/Dominica
    {  108, 5441,    6806,  8, }, // America/El_Salvador
    {  108,  481,      73, 13, }, // America/Indiana/Knox
    {  108, 7961,      86, 16, }, // America/Indiana/Marengo
    {  108, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  108, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  108, 8038,     139, 14, }, // America/Indiana/Vevay
    {  108, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  108, 8086,     171, 16, }, // America/Indiana/Winamac
    {  108, 2799,   33347,  7, }, // America/Jamaica
    {  108, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  108, 5932,     207, 22, }, // America/Lower_Princes
    {  108, 5805,    5260,  8, }, // America/Martinique
    {  108, 2917,   26603,  8, }, // America/Mazatlan
    {  108, 8220,    7126,  6, }, // America/Merida
    {  108, 2949,   57591, 16, }, // America/Mexico_City
    {  108, 1850,     235, 19, }, // America/Noronha
    {  108, 8301,   57781, 22, }, // America/North_Dakota/Beulah
    {  108, 8329,   57803, 22, }, // America/North_Dakota/Center
    {  108, 8357,   57825, 25, }, // America/North_Dakota/New_Salem
    {  108,  393,     317,  4, }, // America/Nuuk
    {  108, 5843,   33426,  9, }, // America/Puerto_Rico
    {  108, 8477,   26704,  8, }, // America/Santarem
    {  108, 1878,   26712,  9, }, // America/Sao_Paulo
    {  108, 6899,     321, 16, }, // America/Scoresbysund
    {  108, 8508,   57850, 16, }, // America/St_Barthelemy
    {  108, 2061,     351, 10, }, // America/St_Johns
    {  108, 5863,   57866, 13, }, // America/St_Kitts
    {  108, 5880,   57879, 11, }, // America/St_Lucia
    {  108,  855,   57890, 21, }, // America/St_Thomas
    {  108, 5913,   57911, 13, }, // America/St_Vincent
    {  108, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  108, 3968,   57924,  5, }, // Asia/Amman
    {  108, 5523,    5609,  5, }, // Asia/Aqtau
    {  108, 5368,    7606,  6, }, // Asia/Aqtobe
    {  108,  964,   57929,  8, }, // Asia/Ashgabat
    {  108, 3607,   57937,  6, }, // Asia/Baghdad
    {  108, 5534,   57943,  7, }, // Asia/Bahrain
    {  108, 4086,   57950,  6, }, // Asia/Beirut
    {  108, 3778,   33455,  6, }, // Asia/Bishkek
    {  108, 5989,   33461,  6, }, // Asia/Brunei
    {  108, 4601,   57622,  4, }, // Asia/Chita
    {  108, 4511,    5637,  7, }, // Asia/Colombo
    {  108, 4540,   57956,  7, }, // Asia/Damascus
    {  108, 1093,   57963,  5, }, // Asia/Dhaka
    {  108, 3596,   57968,  5, }, // Asia/Dubai
    {  108, 6159,   33468,  7, }, // Asia/Dushanbe
    {  108, 1259,   57973, 18, }, // Asia/Ho_Chi_Minh
    {  108, 2704,    4231,  8, }, // Asia/Hong_Kong
    {  108, 1290,   57991,  9, }, // Asia/Jerusalem
    {  108, 3560,   58000,  5, }, // Asia/Kabul
    {  108, 4316,   33510,  8, }, // Asia/Kamchatka
    {  108, 4239,   58005,  6, }, // Asia/Karachi
    {  108, 1185,   58011,  9, }, // Asia/Kathmandu
    {  108, 8661,   58020,  8, }, // Asia/Khandyga
    {  108,  992,   58028,  7, }, // Asia/Kolkata
    {  108, 4197,    4288, 11, }, // Asia/Krasnoyarsk
    {  108, 5976,   33531,  6, }, // Asia/Kuching
    {  108, 5547,    7725,  6, }, // Asia/Kuwait
    {  108, 1211,     464,  5, }, // Asia/Macau
    {  108, 6920,   26882,  6, }, // Asia/Muscat
    {  108, 2554,   58035,  7, }, // Asia/Nicosia
    {  108, 8675,   58042, 11, }, // Asia/Novokuznetsk
    {  108, 7145,   58053,  6, }, // Asia/Oral
    {  108, 6961,   58059,  9, }, // Asia/Phnom_Penh
    {  108, 4214,   58068, 11, }, // Asia/Pyongyang
    {  108, 5559,    4332,  5, }, // Asia/Qatar
    {  108, 8708,    7770,  8, }, // Asia/Qostanay
    {  108, 4269,   26897,  9, }, // Asia/Qyzylorda
    {  108, 3584,   58079,  5, }, // Asia/Riyadh
    {  108, 4408,   58084,  8, }, // Asia/Sakhalin
    {  108, 3279,   58092,  4, }, // Asia/Seoul
    {  108, 1053,   58096,  7, }, // Asia/Shanghai
    {  108, 3300,   20900,  8, }, // Asia/Singapore
    {  108, 4297,   58103, 13, }, // Asia/Srednekolymsk
    {  108, 3263,   58116,  9, }, // Asia/Taipei
    {  108, 4781,   33589,  7, }, // Asia/Tashkent
    {  108, 2772,   58125,  7, }, // Asia/Tehran
    {  108, 1317,   58132,  6, }, // Asia/Thimphu
    {  108, 2821,    4356,  5, }, // Asia/Tokyo
    {  108, 1021,   58138,  9, }, // Asia/Ulaanbaatar
    {  108, 1159,   33596,  6, }, // Asia/Urumqi
    {  108, 4828,    7873,  7, }, // Asia/Yakutsk
    {  108, 1235,   20937,  6, }, // Asia/Yangon
    {  108, 3883,   58147, 13, }, // Asia/Yekaterinburg
    {  108, 3747,    7898,  7, }, // Asia/Yerevan
    {  108, 3663,   58160, 15, }, // Atlantic/Azores
    {  108, 5671,   58175,  7, }, // Atlantic/Bermuda
    {  108, 6628,   58182, 16, }, // Atlantic/Canary
    {  108, 3727,   58198,  8, }, // Atlantic/Cape_Verde
    {  108, 1395,   58206, 15, }, // Atlantic/Faroe
    {  108, 2748,   36176,  9, }, // Atlantic/Reykjavik
    {  108, 7272,   58221, 13, }, // Atlantic/South_Georgia
    {  108, 6821,   58234, 12, }, // Atlantic/St_Helena
    {  108, 3620,   58246,  9, }, // Europe/Astrakhan
    {  108, 2300,   58255,  6, }, // Europe/Athens
    {  108, 6336,   58261,  8, }, // Europe/Belgrade
    {  108, 4757,   21059,  6, }, // Europe/Berlin
    {  108, 2167,   27022,  6, }, // Europe/Brussels
    {  108, 3928,   58269,  8, }, // Europe/Bucharest
    {  108, 3806,   58277,  8, }, // Europe/Budapest
    {  108, 2583,   58285,  7, }, // Europe/Chisinau
    {  108, 6289,   58292,  5, }, // Europe/Copenhagen
    {  108, 6612,   58297,  8, }, // Europe/Helsinki
    {  108, 8804,   58305, 10, }, // Europe/Isle_of_Man
    {  108, 2527,   58315,  5, }, // Europe/Kyiv
    {  108, 3213,   33728,  7, }, // Europe/Lisbon
    {  108, 6506,   58320,  6, }, // Europe/Ljubljana
    {  108, 2501,   58326,  6, }, // Europe/London
    {  108, 6365,   58332,  9, }, // Europe/Luxembourg
    {  108, 6410,    6044,  6, }, // Europe/Monaco
    {  108, 3537,   22880,  6, }, // Europe/Moscow
    {  108, 4284,   58341,  5, }, // Europe/Paris
    {  108, 6275,   36285,  5, }, // Europe/Prague
    {  108, 6324,   58346,  3, }, // Europe/Rome
    {  108, 6599,   58349,  5, }, // Europe/Sofia
    {  108, 6202,   21213,  6, }, // Europe/Tirane
    {  108, 8882,    8261,  9, }, // Europe/Ulyanovsk
    {  108, 6567,   58354,  7, }, // Europe/Vatican
    {  108, 6231,   58361,  5, }, // Europe/Vienna
    {  108, 3190,   33778,  7, }, // Europe/Warsaw
    {  108, 6261,   58366,  6, }, // Europe/Zagreb
    {  108, 6540,   58372,  6, }, // Europe/Zurich
    {  108, 6046,   58378, 14, }, // Indian/Christmas
    {  108, 6063,   58392, 16, }, // Indian/Cocos
    {  108, 5029,   58408,  6, }, // Indian/Comoro
    {  108, 6661,   58414, 20, }, // Indian/Kerguelen
    {  108, 7260,   27115,  4, }, // Indian/Mahe
    {  108, 7050,   58434,  8, }, // Indian/Maldives
    {  108, 4069,   58442,  9, }, // Indian/Mauritius
    {  108, 7211,     547,  7, }, // Indian/Reunion
    {  108, 3015,   58451, 18, }, // Pacific/Chatham
    {  108, 3157,   58469, 16, }, // Pacific/Chuuk
    {  108, 2237,   58485, 18, }, // Pacific/Easter
    {  108, 7360,   27148,  5, }, // Pacific/Efate
    {  108, 3902,   33836,  5, }, // Pacific/Fiji
    {  108, 6678,   58503,  8, }, // Pacific/Galapagos
    {  108, 6696,   58511, 18, }, // Pacific/Gambier
    {  108, 2723,    4611,  8, }, // Pacific/Honolulu
    {  108, 3049,    4619,  9, }, // Pacific/Kanton
    {  108, 4051,   58529,  7, }, // Pacific/Marquesas
    {  108, 7098,    4628,  6, }, // Pacific/Noumea
    {  108, 7194,   58536, 19, }, // Pacific/Pitcairn
    {  108, 3096,     586,  7, }, // Pacific/Pohnpei
    {  109, 5059,    3902, 11, }, // Africa/Addis_Ababa Danish/Latin/Denmark
    {  109, 7420,   58555,  6, }, // Africa/Algiers
    {  109,   14,       0,  6, }, // Africa/Asmara
    {  109, 4435,       6,  8, }, // Africa/Sao_Tome
    {  109, 4252,      14,  8, }, // America/Asuncion
    {  109,  237,      22,  8, }, // America/Atikokan
    {  109, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  109, 3868,   33314,  6, }, // America/Cancun
    {  109, 5496,   58561, 11, }, // America/Cayman
    {  109, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  109, 5723,      66,  7, }, // America/Curacao
    {  109,  481,      73, 13, }, // America/Indiana/Knox
    {  109, 7961,      86, 16, }, // America/Indiana/Marengo
    {  109, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  109, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  109, 8038,     139, 14, }, // America/Indiana/Vevay
    {  109, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  109, 8086,     171, 16, }, // America/Indiana/Winamac
    {  109, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  109, 5932,     207, 22, }, // America/Lower_Princes
    {  109, 8220,    7126,  6, }, // America/Merida
    {  109, 1850,     235, 19, }, // America/Noronha
    {  109, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  109, 8329,     274, 20, }, // America/North_Dakota/Center
    {  109, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  109,  393,     317,  4, }, // America/Nuuk
    {  109, 1878,   26712,  9, }, // America/Sao_Paulo
    {  109, 6899,     321, 16, }, // America/Scoresbysund
    {  109, 8508,   26721, 16, }, // America/St_Barthelemy
    {  109, 2061,     351, 10, }, // America/St_Johns
    {  109, 5863,     361,  9, }, // America/St_Kitts
    {  109, 5880,     370,  9, }, // America/St_Lucia
    {  109,  855,     379, 10, }, // America/St_Thomas
    {  109, 5913,     389, 11, }, // America/St_Vincent
    {  109, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  109, 5523,    5609,  5, }, // Asia/Aqtau
    {  109, 5368,    7606,  6, }, // Asia/Aqtobe
    {  109,  964,   58572,  9, }, // Asia/Ashgabat
    {  109, 3607,    4162,  6, }, // Asia/Baghdad
    {  109, 3778,    4187,  7, }, // Asia/Bishkek
    {  109, 4540,    4201,  8, }, // Asia/Damascus
    {  109, 6159,   58581,  8, }, // Asia/Dushanbe
    {  109, 1259,     432, 16, }, // Asia/Ho_Chi_Minh
    {  109, 2704,    4231,  8, }, // Asia/Hong_Kong
    {  109, 4316,   58589,  9, }, // Asia/Kamchatka
    {  109, 1185,    5711,  8, }, // Asia/Kathmandu
    {  109,  992,     457,  7, }, // Asia/Kolkata
    {  109, 4197,    4288, 11, }, // Asia/Krasnoyarsk
    {  109, 1211,     464,  5, }, // Asia/Macau
    {  109, 8675,   58598, 12, }, // Asia/Novokuznetsk
    {  109, 8708,     469,  8, }, // Asia/Qostanay
    {  109, 4269,   26897,  9, }, // Asia/Qyzylorda
    {  109, 4781,   58610,  8, }, // Asia/Tashkent
    {  109, 2772,    7842,  7, }, // Asia/Tehran
    {  109, 1021,   26923, 10, }, // Asia/Ulaanbaatar
    {  109, 1159,   26933,  6, }, // Asia/Urumqi
    {  109, 4828,    7873,  7, }, // Asia/Yakutsk
    {  109, 1235,   58618,  7, }, // Asia/Yangon
    {  109, 3883,    4380, 13, }, // Asia/Yekaterinburg
    {  109, 3747,    7898,  7, }, // Asia/Yerevan
    {  109, 3663,   58625,  8, }, // Atlantic/Azores
    {  109, 6628,   58633, 16, }, // Atlantic/Canary
    {  109, 3727,   57380,  9, }, // Atlantic/Cape_Verde
    {  109, 1395,   58649,  8, }, // Atlantic/Faroe
    {  109, 6821,     488, 10, }, // Atlantic/St_Helena
    {  109, 2300,    4438,  5, }, // Europe/Athens
    {  109, 6336,    8060,  7, }, // Europe/Belgrade
    {  109, 2167,   57661,  9, }, // Europe/Brussels
    {  109, 3928,   27029,  8, }, // Europe/Bucharest
    {  109, 6289,   58657,  9, }, // Europe/Copenhagen
    {  109, 2527,    8148,  4, }, // Europe/Kyiv
    {  109, 3213,    4499,  8, }, // Europe/Lisbon
    {  109, 3537,   22880,  6, }, // Europe/Moscow
    {  109, 6275,    8201,  4, }, // Europe/Prague
    {  109, 6324,    6064,  3, }, // Europe/Rome
    {  109, 6202,   21213,  6, }, // Europe/Tirane
    {  109, 8882,    8261,  9, }, // Europe/Ulyanovsk
    {  109, 6567,   58666,  9, }, // Europe/Vatican
    {  109, 6231,   57515,  4, }, // Europe/Vienna
    {  109, 3190,   58675,  8, }, // Europe/Warsaw
    {  109, 6540,    4577,  6, }, // Europe/Zurich
    {  109, 6046,   58683,  7, }, // Indian/Christmas
    {  109, 5029,   58690,  9, }, // Indian/Comoro
    {  109, 7050,   58699, 10, }, // Indian/Maldives
    {  109, 7211,     547,  7, }, // Indian/Reunion
    {  109, 3157,     554,  5, }, // Pacific/Chuuk
    {  109, 2237,   58709,  8, }, // Pacific/Easter
    {  109, 2723,    4611,  8, }, // Pacific/Honolulu
    {  109, 3049,    4619,  9, }, // Pacific/Kanton
    {  109, 3096,     586,  7, }, // Pacific/Pohnpei
    {  111,   14,       0,  6, }, // Africa/Asmara Divehi/Thaana/Maldives
    {  111, 4435,       6,  8, }, // Africa/Sao_Tome
    {  111, 4252,      14,  8, }, // America/Asuncion
    {  111,  237,      22,  8, }, // America/Atikokan
    {  111, 7623,      30, 17, }, // America/Bahia_Banderas
    {  111, 3868,      47,  6, }, // America/Cancun
    {  111, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  111, 5723,      66,  7, }, // America/Curacao
    {  111,  481,      73, 13, }, // America/Indiana/Knox
    {  111, 7961,      86, 16, }, // America/Indiana/Marengo
    {  111, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  111, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  111, 8038,     139, 14, }, // America/Indiana/Vevay
    {  111, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  111, 8086,     171, 16, }, // America/Indiana/Winamac
    {  111, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  111, 5932,     207, 22, }, // America/Lower_Princes
    {  111, 8220,     229,  6, }, // America/Merida
    {  111, 1850,     235, 19, }, // America/Noronha
    {  111, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  111, 8329,     274, 20, }, // America/North_Dakota/Center
    {  111, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  111,  393,     317,  4, }, // America/Nuuk
    {  111, 6899,     321, 16, }, // America/Scoresbysund
    {  111, 8508,     337, 14, }, // America/St_Barthelemy
    {  111, 2061,     351, 10, }, // America/St_Johns
    {  111, 5863,     361,  9, }, // America/St_Kitts
    {  111, 5880,     370,  9, }, // America/St_Lucia
    {  111,  855,     379, 10, }, // America/St_Thomas
    {  111, 5913,     389, 11, }, // America/St_Vincent
    {  111, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  111, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  111, 1185,     448,  9, }, // Asia/Kathmandu
    {  111,  992,     457,  7, }, // Asia/Kolkata
    {  111, 1211,     464,  5, }, // Asia/Macau
    {  111, 1235,     477,  6, }, // Asia/Yangon
    {  111, 1395,     483,  5, }, // Atlantic/Faroe
    {  111, 6821,     488, 10, }, // Atlantic/St_Helena
    {  111, 2527,     514,  4, }, // Europe/Kyiv
    {  111, 7211,     547,  7, }, // Indian/Reunion
    {  111, 3157,     554,  5, }, // Pacific/Chuuk
    {  111, 3096,     586,  7, }, // Pacific/Pohnpei
    {  112,   14,       0,  6, }, // Africa/Asmara Dogri/Devanagari/India
    {  112, 4435,       6,  8, }, // Africa/Sao_Tome
    {  112, 4252,      14,  8, }, // America/Asuncion
    {  112,  237,      22,  8, }, // America/Atikokan
    {  112, 7623,      30, 17, }, // America/Bahia_Banderas
    {  112, 3868,      47,  6, }, // America/Cancun
    {  112, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  112, 5723,      66,  7, }, // America/Curacao
    {  112,  481,      73, 13, }, // America/Indiana/Knox
    {  112, 7961,      86, 16, }, // America/Indiana/Marengo
    {  112, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  112, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  112, 8038,     139, 14, }, // America/Indiana/Vevay
    {  112, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  112, 8086,     171, 16, }, // America/Indiana/Winamac
    {  112, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  112, 5932,     207, 22, }, // America/Lower_Princes
    {  112, 8220,     229,  6, }, // America/Merida
    {  112, 1850,     235, 19, }, // America/Noronha
    {  112, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  112, 8329,     274, 20, }, // America/North_Dakota/Center
    {  112, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  112,  393,     317,  4, }, // America/Nuuk
    {  112, 6899,     321, 16, }, // America/Scoresbysund
    {  112, 8508,     337, 14, }, // America/St_Barthelemy
    {  112, 2061,     351, 10, }, // America/St_Johns
    {  112, 5863,     361,  9, }, // America/St_Kitts
    {  112, 5880,     370,  9, }, // America/St_Lucia
    {  112,  855,     379, 10, }, // America/St_Thomas
    {  112, 5913,     389, 11, }, // America/St_Vincent
    {  112, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  112, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  112, 1185,     448,  9, }, // Asia/Kathmandu
    {  112,  992,     457,  7, }, // Asia/Kolkata
    {  112, 1211,     464,  5, }, // Asia/Macau
    {  112, 1235,     477,  6, }, // Asia/Yangon
    {  112, 1395,     483,  5, }, // Atlantic/Faroe
    {  112, 6821,     488, 10, }, // Atlantic/St_Helena
    {  112, 2527,     514,  4, }, // Europe/Kyiv
    {  112, 7211,     547,  7, }, // Indian/Reunion
    {  112, 3157,     554,  5, }, // Pacific/Chuuk
    {  112, 3096,     586,  7, }, // Pacific/Pohnpei
    {  113,   14,       0,  6, }, // Africa/Asmara Duala/Latin/Cameroon
    {  113, 4435,       6,  8, }, // Africa/Sao_Tome
    {  113, 4252,      14,  8, }, // America/Asuncion
    {  113,  237,      22,  8, }, // America/Atikokan
    {  113, 7623,      30, 17, }, // America/Bahia_Banderas
    {  113, 3868,      47,  6, }, // America/Cancun
    {  113, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  113, 5723,      66,  7, }, // America/Curacao
    {  113,  481,      73, 13, }, // America/Indiana/Knox
    {  113, 7961,      86, 16, }, // America/Indiana/Marengo
    {  113, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  113, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  113, 8038,     139, 14, }, // America/Indiana/Vevay
    {  113, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  113, 8086,     171, 16, }, // America/Indiana/Winamac
    {  113, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  113, 5932,     207, 22, }, // America/Lower_Princes
    {  113, 8220,     229,  6, }, // America/Merida
    {  113, 1850,     235, 19, }, // America/Noronha
    {  113, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  113, 8329,     274, 20, }, // America/North_Dakota/Center
    {  113, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  113,  393,     317,  4, }, // America/Nuuk
    {  113, 6899,     321, 16, }, // America/Scoresbysund
    {  113, 8508,     337, 14, }, // America/St_Barthelemy
    {  113, 2061,     351, 10, }, // America/St_Johns
    {  113, 5863,     361,  9, }, // America/St_Kitts
    {  113, 5880,     370,  9, }, // America/St_Lucia
    {  113,  855,     379, 10, }, // America/St_Thomas
    {  113, 5913,     389, 11, }, // America/St_Vincent
    {  113, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  113, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  113, 1185,     448,  9, }, // Asia/Kathmandu
    {  113,  992,     457,  7, }, // Asia/Kolkata
    {  113, 1211,     464,  5, }, // Asia/Macau
    {  113, 1235,     477,  6, }, // Asia/Yangon
    {  113, 1395,     483,  5, }, // Atlantic/Faroe
    {  113, 6821,     488, 10, }, // Atlantic/St_Helena
    {  113, 2527,     514,  4, }, // Europe/Kyiv
    {  113, 7211,     547,  7, }, // Indian/Reunion
    {  113, 3157,     554,  5, }, // Pacific/Chuuk
    {  113, 3096,     586,  7, }, // Pacific/Pohnpei
    {  114, 5059,    3902, 11, }, // Africa/Addis_Ababa Dutch/Latin/Netherlands
    {  114,   14,       0,  6, }, // Africa/Asmara
    {  114, 2320,   58717,  5, }, // Africa/Cairo
    {  114, 4524,   58722,  8, }, // Africa/Khartoum
    {  114, 6869,   20418,  4, }, // Africa/Lome
    {  114, 4435,   58730,  8, }, // Africa/Sao_Tome
    {  114,  270,   20479,  7, }, // America/Argentina/Cordoba
    {  114, 7489,   44137, 12, }, // America/Argentina/Rio_Gallegos
    {  114, 7571,   20486,  7, }, // America/Argentina/Tucuman
    {  114, 4252,      14,  8, }, // America/Asuncion
    {  114,  237,      22,  8, }, // America/Atikokan
    {  114, 7623,      30, 17, }, // America/Bahia_Banderas
    {  114, 7646,   35896,  5, }, // America/Belem
    {  114, 3868,   33314,  6, }, // America/Cancun
    {  114, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  114, 3791,   26574,  6, }, // America/Cuiaba
    {  114, 5723,      66,  7, }, // America/Curacao
    {  114,  481,      73, 13, }, // America/Indiana/Knox
    {  114, 7961,      86, 16, }, // America/Indiana/Marengo
    {  114, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  114, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  114, 8038,     139, 14, }, // America/Indiana/Vevay
    {  114, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  114, 8086,     171, 16, }, // America/Indiana/Winamac
    {  114, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  114, 5932,   58738, 24, }, // America/Lower_Princes
    {  114, 8153,   26588,  6, }, // America/Maceio
    {  114, 2917,   26603,  8, }, // America/Mazatlan
    {  114, 8220,     229,  6, }, // America/Merida
    {  114, 2949,   58762, 11, }, // America/Mexico_City
    {  114, 1850,     235, 19, }, // America/Noronha
    {  114, 8301,    4013, 20, }, // America/North_Dakota/Beulah
    {  114, 8329,    4033, 20, }, // America/North_Dakota/Center
    {  114, 8357,    4053, 23, }, // America/North_Dakota/New_Salem
    {  114,  393,     317,  4, }, // America/Nuuk
    {  114, 1878,   26712,  9, }, // America/Sao_Paulo
    {  114, 6899,     321, 16, }, // America/Scoresbysund
    {  114, 8508,   26721, 16, }, // America/St_Barthelemy
    {  114, 2061,   26737, 12, }, // America/St_Johns
    {  114, 5863,   26749, 11, }, // America/St_Kitts
    {  114, 5880,   35964, 11, }, // America/St_Lucia
    {  114,  855,   44293, 12, }, // America/St_Thomas
    {  114, 5913,   26783, 13, }, // America/St_Vincent
    {  114, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  114, 5395,   58773,  8, }, // Asia/Almaty
    {  114, 5368,   57205,  6, }, // Asia/Aqtobe
    {  114,  964,    4153,  9, }, // Asia/Ashgabat
    {  114, 8624,   58781,  6, }, // Asia/Atyrau
    {  114, 3607,    4162,  6, }, // Asia/Baghdad
    {  114, 5534,    4168,  7, }, // Asia/Bahrain
    {  114, 3653,    4175,  5, }, // Asia/Baku
    {  114, 4086,    4180,  7, }, // Asia/Beirut
    {  114, 3778,    4187,  7, }, // Asia/Bishkek
    {  114, 6159,   58787,  9, }, // Asia/Dushanbe
    {  114, 1259,   58796, 15, }, // Asia/Ho_Chi_Minh
    {  114, 2704,    4231,  8, }, // Asia/Hong_Kong
    {  114, 4184,    4239,  8, }, // Asia/Irkutsk
    {  114, 1290,   33501,  9, }, // Asia/Jerusalem
    {  114, 4316,    4261, 10, }, // Asia/Kamchatka
    {  114, 1185,     448,  9, }, // Asia/Kathmandu
    {  114,  992,   36053,  8, }, // Asia/Kolkata
    {  114, 4197,    4288, 11, }, // Asia/Krasnoyarsk
    {  114, 5547,    4314,  7, }, // Asia/Kuwait
    {  114, 1211,    4321,  5, }, // Asia/Macau
    {  114, 7182,   58811,  7, }, // Asia/Manila
    {  114, 3584,   20874,  5, }, // Asia/Riyadh
    {  114, 4408,   58084,  8, }, // Asia/Sakhalin
    {  114, 1053,   58818,  8, }, // Asia/Shanghai
    {  114, 4781,   58610,  8, }, // Asia/Tashkent
    {  114, 2772,    7842,  7, }, // Asia/Tehran
    {  114, 2821,    4356,  5, }, // Asia/Tokyo
    {  114, 4828,    4372,  8, }, // Asia/Yakutsk
    {  114, 1235,   58618,  7, }, // Asia/Yangon
    {  114, 3883,   58826, 15, }, // Asia/Yekaterinburg
    {  114, 3747,    7898,  7, }, // Asia/Yerevan
    {  114, 3663,   58841,  6, }, // Atlantic/Azores
    {  114, 6628,   58847, 19, }, // Atlantic/Canary
    {  114, 3727,   58866, 10, }, // Atlantic/Cape_Verde
    {  114, 1395,   58876,  7, }, // Atlantic/Faroe
    {  114, 7272,   58883, 12, }, // Atlantic/South_Georgia
    {  114, 6821,   58895, 11, }, // Atlantic/St_Helena
    {  114, 2300,    4438,  6, }, // Europe/Athens
    {  114, 6336,   58906,  8, }, // Europe/Belgrade
    {  114, 4757,   58914,  7, }, // Europe/Berlin
    {  114, 2167,    4450,  7, }, // Europe/Brussels
    {  114, 3928,    4457,  9, }, // Europe/Bucharest
    {  114, 3806,    4466,  9, }, // Europe/Budapest
    {  114, 6289,    4475, 10, }, // Europe/Copenhagen
    {  114, 1130,   58921,  9, }, // Europe/Istanbul
    {  114, 2527,    8148,  4, }, // Europe/Kyiv
    {  114, 3213,    4499,  8, }, // Europe/Lisbon
    {  114, 2501,    4507,  6, }, // Europe/London
    {  114, 6365,    4513,  9, }, // Europe/Luxembourg
    {  114, 3537,    4522,  6, }, // Europe/Moscow
    {  114, 4284,   58930,  6, }, // Europe/Paris
    {  114, 6275,    4533,  5, }, // Europe/Prague
    {  114, 6202,   21213,  6, }, // Europe/Tirane
    {  114, 6567,   58936, 12, }, // Europe/Vatican
    {  114, 6231,   58948,  5, }, // Europe/Vienna
    {  114, 4727,    4561,  9, }, // Europe/Volgograd
    {  114, 3190,   57528,  8, }, // Europe/Warsaw
    {  114, 6540,    4577,  6, }, // Europe/Zurich
    {  114, 6947,   58953, 14, }, // Indian/Chagos
    {  114, 6046,   58967, 15, }, // Indian/Christmas
    {  114, 6063,   58982, 13, }, // Indian/Cocos
    {  114, 7260,   27115,  4, }, // Indian/Mahe
    {  114, 7050,   58995,  8, }, // Indian/Maldives
    {  114, 7211,     547,  7, }, // Indian/Reunion
    {  114, 3157,     554,  5, }, // Pacific/Chuuk
    {  114, 2237,   59003, 10, }, // Pacific/Easter
    {  114, 6696,   59013, 12, }, // Pacific/Gambier
    {  114, 2723,    4611,  8, }, // Pacific/Honolulu
    {  114, 3049,    4619,  9, }, // Pacific/Kanton
    {  114, 4051,   59025, 17, }, // Pacific/Marquesas
    {  114, 7098,    4628,  6, }, // Pacific/Noumea
    {  114, 3096,     586,  7, }, // Pacific/Pohnpei
    {  121, 6760,   59042,  6, }, // Africa/Accra Dzongkha/Tibetan/Bhutan
    {  121, 5059,   59048, 14, }, // Africa/Addis_Ababa
    {  121, 7420,   59062,  8, }, // Africa/Algiers
    {  121,   14,   59070,  7, }, // Africa/Asmara
    {  121,   44,   59077,  7, }, // Africa/Bamako
    {  121, 5257,   59084,  6, }, // Africa/Bangui
    {  121, 6746,   59090,  8, }, // Africa/Banjul
    {  121, 7435,   59098,  8, }, // Africa/Bissau
    {  121, 5287,   59106,  9, }, // Africa/Brazzaville
    {  121, 4922,   59115, 13, }, // Africa/Bujumbura
    {  121, 2320,   59128,  7, }, // Africa/Cairo
    {  121, 6773,   59135,  8, }, // Africa/Conakry
    {  121, 6840,   59143,  5, }, // Africa/Dakar
    {  121, 5043,   59148,  9, }, // Africa/Djibouti
    {  121, 4906,   59157, 12, }, // Africa/Gaborone
    {  121, 5015,   59169,  7, }, // Africa/Harare
    {  121, 5151,   59176,  7, }, // Africa/Kampala
    {  121, 4524,   59183,  7, }, // Africa/Khartoum
    {  121, 4973,   59190,  8, }, // Africa/Kigali
    {  121, 5306,   59198,  7, }, // Africa/Kinshasa
    {  121, 5336,   59205, 10, }, // Africa/Libreville
    {  121, 6869,   59215,  5, }, // Africa/Lome
    {  121, 5211,   59220,  8, }, // Africa/Luanda
    {  121, 5001,   59228,  7, }, // Africa/Lusaka
    {  121, 5322,   59235,  7, }, // Africa/Malabo
    {  121, 4987,   59242,  7, }, // Africa/Maputo
    {  121, 5197,   59249,  6, }, // Africa/Maseru
    {  121, 5182,   59255, 12, }, // Africa/Mbabane
    {  121, 5113,   59267, 11, }, // Africa/Mogadishu
    {  121, 7034,   59278, 11, }, // Africa/Monrovia
    {  121, 3853,   59289, 10, }, // Africa/Nairobi
    {  121, 5354,   59299,  7, }, // Africa/Niamey
    {  121, 6803,   59306, 10, }, // Africa/Nouakchott
    {  121, 6727,   59316, 10, }, // Africa/Ouagadougou
    {  121, 4435,   59326,  9, }, // Africa/Sao_Tome
    {  121, 2866,   59335,  9, }, // Africa/Tripoli
    {  121, 6554,   59344,  7, }, // Africa/Tunis
    {  121, 4152,   59351,  8, }, // Africa/Windhoek
    {  121, 5624,   59359, 11, }, // America/Antigua
    {  121,  166,   59370, 18, }, // America/Argentina/Buenos_Aires
    {  121, 4252,   59388, 12, }, // America/Asuncion
    {  121,  237,   59400, 11, }, // America/Atikokan
    {  121, 7623,      30, 17, }, // America/Bahia_Banderas
    {  121, 5654,   59411, 10, }, // America/Barbados
    {  121, 5407,   59421,  7, }, // America/Belize
    {  121, 4361,   59428,  7, }, // America/Bogota
    {  121, 7713,   59435, 15, }, // America/Cambridge_Bay
    {  121, 3868,      47,  6, }, // America/Cancun
    {  121, 4694,   59450,  7, }, // America/Caracas
    {  121, 2260,   59457,  8, }, // America/Chicago
    {  121, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  121, 5422,   59465, 10, }, // America/Costa_Rica
    {  121, 5723,      66,  7, }, // America/Curacao
    {  121, 7833,   59475,  8, }, // America/Dawson
    {  121, 7848,   59483, 15, }, // America/Dawson_Creek
    {  121, 5739,   59498, 10, }, // America/Dominica
    {  121,  893,   59508, 12, }, // America/Edmonton
    {  121, 5441,   59520, 13, }, // America/El_Salvador
    {  121, 6881,   59533, 10, }, // America/Goose_Bay
    {  121, 3760,   59543,  9, }, // America/Guatemala
    {  121, 6932,   59552,  6, }, // America/Guyana
    {  121, 1939,   59558, 10, }, // America/Halifax
    {  121,  481,      73, 13, }, // America/Indiana/Knox
    {  121, 7961,      86, 16, }, // America/Indiana/Marengo
    {  121, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  121, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  121, 8038,     139, 14, }, // America/Indiana/Vevay
    {  121, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  121, 8086,     171, 16, }, // America/Indiana/Winamac
    {  121, 8110,   59568,  9, }, // America/Inuvik
    {  121, 2799,   59577,  8, }, // America/Jamaica
    {  121, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  121, 4376,   59585,  6, }, // America/La_Paz
    {  121, 7169,   59591,  4, }, // America/Lima
    {  121, 5932,     207, 22, }, // America/Lower_Princes
    {  121, 8220,     229,  6, }, // America/Merida
    {  121, 2949,   59595, 16, }, // America/Mexico_City
    {  121, 8254,   59611,  9, }, // America/Moncton
    {  121, 4098,   59620, 15, }, // America/Montevideo
    {  121, 5481,   59635,  5, }, // America/Nassau
    {  121, 1850,     235, 19, }, // America/Noronha
    {  121, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  121, 8329,     274, 20, }, // America/North_Dakota/Center
    {  121, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  121,  393,     317,  4, }, // America/Nuuk
    {  121, 2356,   59640,  5, }, // America/Panama
    {  121, 6173,   59645, 12, }, // America/Paramaribo
    {  121, 8424,   59657, 16, }, // America/Rankin_Inlet
    {  121, 1995,   59673,  7, }, // America/Regina
    {  121, 8460,   59680, 11, }, // America/Resolute
    {  121, 2201,   59691, 12, }, // America/Santiago
    {  121, 6111,   59703, 18, }, // America/Santo_Domingo
    {  121, 6899,     321, 16, }, // America/Scoresbysund
    {  121, 8508,     337, 14, }, // America/St_Barthelemy
    {  121, 2061,   59721, 17, }, // America/St_Johns
    {  121, 5863,     361,  9, }, // America/St_Kitts
    {  121, 5880,     370,  9, }, // America/St_Lucia
    {  121,  855,     379, 10, }, // America/St_Thomas
    {  121, 5913,     389, 11, }, // America/St_Vincent
    {  121, 8530,   59738, 14, }, // America/Swift_Current
    {  121, 5461,   59752, 14, }, // America/Tegucigalpa
    {  121,  313,   59766,  8, }, // America/Tijuana
    {  121,  608,   59774,  9, }, // America/Toronto
    {  121, 8552,   59783,  7, }, // America/Yakutat
    {  121, 6094,   59790,  7, }, // Antarctica/Davis
    {  121, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  121, 7066,   59797,  8, }, // Antarctica/Mawson
    {  121, 8589,   59805,  6, }, // Antarctica/Palmer
    {  121, 7226,   59811,  8, }, // Antarctica/Rothera
    {  121, 7374,   59819,  8, }, // Antarctica/Vostok
    {  121, 5534,   59827,  7, }, // Asia/Bahrain
    {  121, 5989,   59834,  8, }, // Asia/Brunei
    {  121, 4511,   59842, 10, }, // Asia/Colombo
    {  121, 1093,   59143,  4, }, // Asia/Dhaka
    {  121, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  121, 7006,   59852,  7, }, // Asia/Jakarta
    {  121, 6992,   59859,  8, }, // Asia/Jayapura
    {  121, 3560,   59867,  6, }, // Asia/Kabul
    {  121, 4239,   59873,  6, }, // Asia/Karachi
    {  121, 1185,   59879, 10, }, // Asia/Kathmandu
    {  121,  992,   59889,  8, }, // Asia/Kolkata
    {  121, 4859,   59897, 12, }, // Asia/Kuala_Lumpur
    {  121, 1211,     464,  5, }, // Asia/Macau
    {  121, 8693,   59909, 11, }, // Asia/Pontianak
    {  121, 2772,   59920,  6, }, // Asia/Tehran
    {  121, 1317,   59926,  7, }, // Asia/Thimphu
    {  121, 6977,   59933, 13, }, // Asia/Vientiane
    {  121, 1235,   59946,  7, }, // Asia/Yangon
    {  121, 5671,   59953,  7, }, // Atlantic/Bermuda
    {  121, 3727,   59960,  7, }, // Atlantic/Cape_Verde
    {  121, 1395,   59967,  8, }, // Atlantic/Faroe
    {  121, 2748,   59975, 11, }, // Atlantic/Reykjavik
    {  121, 7272,   59986, 11, }, // Atlantic/South_Georgia
    {  121, 6821,     488, 10, }, // Atlantic/St_Helena
    {  121, 6441,   59997,  9, }, // Europe/Amsterdam
    {  121, 6216,   60006,  8, }, // Europe/Andorra
    {  121, 2300,   60014,  8, }, // Europe/Athens
    {  121, 4757,   60022,  7, }, // Europe/Berlin
    {  121, 2167,   60029,  7, }, // Europe/Brussels
    {  121, 3928,   60036, 10, }, // Europe/Bucharest
    {  121, 3806,   60046, 10, }, // Europe/Budapest
    {  121, 2583,   60056,  9, }, // Europe/Chisinau
    {  121, 6289,   60065, 13, }, // Europe/Copenhagen
    {  121, 2338,   60078,  6, }, // Europe/Dublin
    {  121, 6612,   60084, 10, }, // Europe/Helsinki
    {  121, 2527,     514,  4, }, // Europe/Kyiv
    {  121, 2501,   60094,  7, }, // Europe/London
    {  121, 6365,   60101,  9, }, // Europe/Luxembourg
    {  121, 4892,   60110,  8, }, // Europe/Madrid
    {  121, 3693,   60118,  5, }, // Europe/Minsk
    {  121, 6410,   60123,  7, }, // Europe/Monaco
    {  121, 3537,   60130,  7, }, // Europe/Moscow
    {  121, 6458,   60137,  6, }, // Europe/Oslo
    {  121, 4284,   60143,  6, }, // Europe/Paris
    {  121, 6275,   60149,  3, }, // Europe/Prague
    {  121, 8837,   60152,  5, }, // Europe/Riga
    {  121, 6245,   60157,  9, }, // Europe/Sarajevo
    {  121, 6599,   60166,  7, }, // Europe/Sofia
    {  121, 6523,   60173, 12, }, // Europe/Stockholm
    {  121, 8867,   60185,  7, }, // Europe/Tallinn
    {  121, 6202,   60192,  6, }, // Europe/Tirane
    {  121, 6352,   60198,  6, }, // Europe/Vaduz
    {  121, 6231,   60204,  7, }, // Europe/Vienna
    {  121, 8899,   60211, 10, }, // Europe/Vilnius
    {  121, 3190,   60221,  5, }, // Europe/Warsaw
    {  121, 6540,   60226,  6, }, // Europe/Zurich
    {  121, 5078,   60232, 15, }, // Indian/Antananarivo
    {  121, 6046,   60247, 10, }, // Indian/Christmas
    {  121, 6063,   60257,  7, }, // Indian/Cocos
    {  121, 5029,   60264,  8, }, // Indian/Comoro
    {  121, 6661,   60272, 10, }, // Indian/Kerguelen
    {  121, 7050,   60282,  8, }, // Indian/Maldives
    {  121, 4069,   60290,  9, }, // Indian/Mauritius
    {  121, 7211,     547,  7, }, // Indian/Reunion
    {  121, 4422,   60299,  6, }, // Pacific/Apia
    {  121, 3157,     554,  5, }, // Pacific/Chuuk
    {  121, 7327,   60305,  8, }, // Pacific/Fakaofo
    {  121, 7343,   60313, 10, }, // Pacific/Funafuti
    {  121, 6696,   60323,  8, }, // Pacific/Gambier
    {  121, 4877,   60331,  8, }, // Pacific/Majuro
    {  121, 8914,   60339,  6, }, // Pacific/Midway
    {  121, 7084,   60345,  6, }, // Pacific/Nauru
    {  121, 3126,   60351, 10, }, // Pacific/Pago_Pago
    {  121, 7155,   60361,  5, }, // Pacific/Palau
    {  121, 3096,     586,  7, }, // Pacific/Pohnpei
    {  121, 6076,   60366, 12, }, // Pacific/Rarotonga
    {  121, 6031,   60378,  8, }, // Pacific/Saipan
    {  121, 4583,   60386, 10, }, // Pacific/Tongatapu
    {  121, 7392,   60396,  3, }, // Pacific/Wake
    {  122,   14,       0,  6, }, // Africa/Asmara Embu/Latin/Kenya
    {  122, 4435,       6,  8, }, // Africa/Sao_Tome
    {  122, 4252,      14,  8, }, // America/Asuncion
    {  122,  237,      22,  8, }, // America/Atikokan
    {  122, 7623,      30, 17, }, // America/Bahia_Banderas
    {  122, 3868,      47,  6, }, // America/Cancun
    {  122, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  122, 5723,      66,  7, }, // America/Curacao
    {  122,  481,      73, 13, }, // America/Indiana/Knox
    {  122, 7961,      86, 16, }, // America/Indiana/Marengo
    {  122, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  122, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  122, 8038,     139, 14, }, // America/Indiana/Vevay
    {  122, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  122, 8086,     171, 16, }, // America/Indiana/Winamac
    {  122, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  122, 5932,     207, 22, }, // America/Lower_Princes
    {  122, 8220,     229,  6, }, // America/Merida
    {  122, 1850,     235, 19, }, // America/Noronha
    {  122, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  122, 8329,     274, 20, }, // America/North_Dakota/Center
    {  122, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  122,  393,     317,  4, }, // America/Nuuk
    {  122, 6899,     321, 16, }, // America/Scoresbysund
    {  122, 8508,     337, 14, }, // America/St_Barthelemy
    {  122, 2061,     351, 10, }, // America/St_Johns
    {  122, 5863,     361,  9, }, // America/St_Kitts
    {  122, 5880,     370,  9, }, // America/St_Lucia
    {  122,  855,     379, 10, }, // America/St_Thomas
    {  122, 5913,     389, 11, }, // America/St_Vincent
    {  122, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  122, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  122, 1185,     448,  9, }, // Asia/Kathmandu
    {  122,  992,     457,  7, }, // Asia/Kolkata
    {  122, 1211,     464,  5, }, // Asia/Macau
    {  122, 1235,     477,  6, }, // Asia/Yangon
    {  122, 1395,     483,  5, }, // Atlantic/Faroe
    {  122, 6821,     488, 10, }, // Atlantic/St_Helena
    {  122, 2527,     514,  4, }, // Europe/Kyiv
    {  122, 7211,     547,  7, }, // Indian/Reunion
    {  122, 3157,     554,  5, }, // Pacific/Chuuk
    {  122, 3096,     586,  7, }, // Pacific/Pohnpei
    {  123,   14,       0,  6, }, // Africa/Asmara English/Latin/United States
    {  123, 4435,       6,  8, }, // Africa/Sao_Tome
    {  123, 4252,      14,  8, }, // America/Asuncion
    {  123,  237,      22,  8, }, // America/Atikokan
    {  123, 7623,      30, 17, }, // America/Bahia_Banderas
    {  123, 3868,      47,  6, }, // America/Cancun
    {  123, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  123, 5723,      66,  7, }, // America/Curacao
    {  123,  481,      73, 13, }, // America/Indiana/Knox
    {  123, 7961,      86, 16, }, // America/Indiana/Marengo
    {  123, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  123, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  123, 8038,     139, 14, }, // America/Indiana/Vevay
    {  123, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  123, 8086,     171, 16, }, // America/Indiana/Winamac
    {  123, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  123, 5932,     207, 22, }, // America/Lower_Princes
    {  123, 8220,     229,  6, }, // America/Merida
    {  123, 1850,     235, 19, }, // America/Noronha
    {  123, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  123, 8329,     274, 20, }, // America/North_Dakota/Center
    {  123, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  123,  393,     317,  4, }, // America/Nuuk
    {  123, 6899,     321, 16, }, // America/Scoresbysund
    {  123, 8508,     337, 14, }, // America/St_Barthelemy
    {  123, 2061,     351, 10, }, // America/St_Johns
    {  123, 5863,     361,  9, }, // America/St_Kitts
    {  123, 5880,     370,  9, }, // America/St_Lucia
    {  123,  855,     379, 10, }, // America/St_Thomas
    {  123, 5913,     389, 11, }, // America/St_Vincent
    {  123, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  123, 8568,     416, 16, }, // Antarctica/Macquarie
    {  123, 1259,     432, 16, }, // Asia/Ho_Chi_Minh
    {  123, 1185,     448,  9, }, // Asia/Kathmandu
    {  123,  992,     457,  7, }, // Asia/Kolkata
    {  123, 1211,     464,  5, }, // Asia/Macau
    {  123, 8708,     469,  8, }, // Asia/Qostanay
    {  123, 1235,     477,  6, }, // Asia/Yangon
    {  123, 1395,     483,  5, }, // Atlantic/Faroe
    {  123, 6821,     488, 10, }, // Atlantic/St_Helena
    {  123, 1547,     498, 16, }, // Australia/Lord_Howe
    {  123, 2527,     514,  4, }, // Europe/Kyiv
    {  123, 6046,     518, 16, }, // Indian/Christmas
    {  123, 6063,     534, 13, }, // Indian/Cocos
    {  123, 7211,     547,  7, }, // Indian/Reunion
    {  123, 3157,     554,  5, }, // Pacific/Chuuk
    {  123, 2237,     559, 13, }, // Pacific/Easter
    {  123, 4168,     572, 14, }, // Pacific/Norfolk
    {  123, 3096,     586,  7, }, // Pacific/Pohnpei
    {  123, 7392,     593, 11, }, // Pacific/Wake
    {  124,  132,   60399,  8, }, // America/Adak English/Deseret/United States
    {  124, 3346,   60407, 14, }, // America/Anchorage
    {  124, 7699,   60421, 10, }, // America/Boise
    {  124, 2260,   60431, 12, }, // America/Chicago
    {  124,  805,   60443, 12, }, // America/Denver
    {  124, 3465,   60455, 14, }, // America/Detroit
    {  124,  348,   60469, 24, }, // America/Indiana/Indianapolis
    {  124,  481,   60493, 24, }, // America/Indiana/Knox
    {  124, 7961,   60517, 30, }, // America/Indiana/Marengo
    {  124, 7985,   60547, 36, }, // America/Indiana/Petersburg
    {  124, 8012,   60583, 31, }, // America/Indiana/Tell_City
    {  124, 8038,   60614, 24, }, // America/Indiana/Vevay
    {  124, 8060,   60638, 30, }, // America/Indiana/Vincennes
    {  124, 8086,   60668, 30, }, // America/Indiana/Winamac
    {  124, 5380,   60698,  8, }, // America/Juneau
    {  124,  521,   60706, 14, }, // America/Kentucky/Louisville
    {  124, 8125,   60720, 34, }, // America/Kentucky/Monticello
    {  124, 3239,   60754, 19, }, // America/Los_Angeles
    {  124, 8202,   60773, 16, }, // America/Menominee
    {  124, 2379,   60789, 13, }, // America/New_York
    {  124, 8288,   60802,  6, }, // America/Nome
    {  124, 8329,   60808, 35, }, // America/North_Dakota/Center
    {  124, 8357,   60843, 38, }, // America/North_Dakota/New_Salem
    {  124, 2973,   60881, 12, }, // America/Phoenix
    {  124, 8552,   60893, 14, }, // America/Yakutat
    {  124, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  124, 2723,   60907, 16, }, // Pacific/Honolulu
    {  124, 8914,   60923, 10, }, // Pacific/Midway
    {  124, 7392,   60933,  6, }, // Pacific/Wake
    {  126, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Anguilla
    {  126, 2061,   60952,  9, }, // America/St_Johns
    {  126, 5863,   60961,  8, }, // America/St_Kitts
    {  126, 5880,   60969,  8, }, // America/St_Lucia
    {  126,  855,   60977,  9, }, // America/St_Thomas
    {  126, 5913,   60986, 10, }, // America/St_Vincent
    {  126, 5523,    5609,  5, }, // Asia/Aqtau
    {  126, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  126, 3049,    4619,  9, }, // Pacific/Kanton
    {  127, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Antigua and Barbuda
    {  127, 2061,   60952,  9, }, // America/St_Johns
    {  127, 5863,   60961,  8, }, // America/St_Kitts
    {  127, 5880,   60969,  8, }, // America/St_Lucia
    {  127,  855,   60977,  9, }, // America/St_Thomas
    {  127, 5913,   60986, 10, }, // America/St_Vincent
    {  127, 5523,    5609,  5, }, // Asia/Aqtau
    {  127, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  127, 3049,    4619,  9, }, // Pacific/Kanton
    {  128, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Australia
    {  128, 2061,   60952,  9, }, // America/St_Johns
    {  128, 5863,   60961,  8, }, // America/St_Kitts
    {  128, 5880,   60969,  8, }, // America/St_Lucia
    {  128,  855,   60977,  9, }, // America/St_Thomas
    {  128, 5913,   60986, 10, }, // America/St_Vincent
    {  128, 5523,    5609,  5, }, // Asia/Aqtau
    {  128, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  128, 3049,    4619,  9, }, // Pacific/Kanton
    {  129, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Austria
    {  129, 2061,   60952,  9, }, // America/St_Johns
    {  129, 5863,   60961,  8, }, // America/St_Kitts
    {  129, 5880,   60969,  8, }, // America/St_Lucia
    {  129,  855,   60977,  9, }, // America/St_Thomas
    {  129, 5913,   60986, 10, }, // America/St_Vincent
    {  129, 5523,    5609,  5, }, // Asia/Aqtau
    {  129, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  129, 3049,    4619,  9, }, // Pacific/Kanton
    {  130, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Bahamas
    {  130, 2061,   60952,  9, }, // America/St_Johns
    {  130, 5863,   60961,  8, }, // America/St_Kitts
    {  130, 5880,   60969,  8, }, // America/St_Lucia
    {  130,  855,   60977,  9, }, // America/St_Thomas
    {  130, 5913,   60986, 10, }, // America/St_Vincent
    {  130, 5523,    5609,  5, }, // Asia/Aqtau
    {  130, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  130, 3049,    4619,  9, }, // Pacific/Kanton
    {  131, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Barbados
    {  131, 2061,   60952,  9, }, // America/St_Johns
    {  131, 5863,   60961,  8, }, // America/St_Kitts
    {  131, 5880,   60969,  8, }, // America/St_Lucia
    {  131,  855,   60977,  9, }, // America/St_Thomas
    {  131, 5913,   60986, 10, }, // America/St_Vincent
    {  131, 5523,    5609,  5, }, // Asia/Aqtau
    {  131, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  131, 3049,    4619,  9, }, // Pacific/Kanton
    {  132, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Belgium
    {  132, 2061,   60952,  9, }, // America/St_Johns
    {  132, 5863,   60961,  8, }, // America/St_Kitts
    {  132, 5880,   60969,  8, }, // America/St_Lucia
    {  132,  855,   60977,  9, }, // America/St_Thomas
    {  132, 5913,   60986, 10, }, // America/St_Vincent
    {  132, 5523,    5609,  5, }, // Asia/Aqtau
    {  132, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  132, 3049,    4619,  9, }, // Pacific/Kanton
    {  133, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Belize
    {  133, 2061,   60952,  9, }, // America/St_Johns
    {  133, 5863,   60961,  8, }, // America/St_Kitts
    {  133, 5880,   60969,  8, }, // America/St_Lucia
    {  133,  855,   60977,  9, }, // America/St_Thomas
    {  133, 5913,   60986, 10, }, // America/St_Vincent
    {  133, 5523,    5609,  5, }, // Asia/Aqtau
    {  133, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  133, 3049,    4619,  9, }, // Pacific/Kanton
    {  134, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Bermuda
    {  134, 2061,   60952,  9, }, // America/St_Johns
    {  134, 5863,   60961,  8, }, // America/St_Kitts
    {  134, 5880,   60969,  8, }, // America/St_Lucia
    {  134,  855,   60977,  9, }, // America/St_Thomas
    {  134, 5913,   60986, 10, }, // America/St_Vincent
    {  134, 5523,    5609,  5, }, // Asia/Aqtau
    {  134, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  134, 3049,    4619,  9, }, // Pacific/Kanton
    {  135, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Botswana
    {  135, 2061,   60952,  9, }, // America/St_Johns
    {  135, 5863,   60961,  8, }, // America/St_Kitts
    {  135, 5880,   60969,  8, }, // America/St_Lucia
    {  135,  855,   60977,  9, }, // America/St_Thomas
    {  135, 5913,   60986, 10, }, // America/St_Vincent
    {  135, 5523,    5609,  5, }, // Asia/Aqtau
    {  135, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  135, 3049,    4619,  9, }, // Pacific/Kanton
    {  136, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/British Indian Ocean Territory
    {  136, 2061,   60952,  9, }, // America/St_Johns
    {  136, 5863,   60961,  8, }, // America/St_Kitts
    {  136, 5880,   60969,  8, }, // America/St_Lucia
    {  136,  855,   60977,  9, }, // America/St_Thomas
    {  136, 5913,   60986, 10, }, // America/St_Vincent
    {  136, 5523,    5609,  5, }, // Asia/Aqtau
    {  136, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  136, 3049,    4619,  9, }, // Pacific/Kanton
    {  137, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/British Virgin Islands
    {  137, 2061,   60952,  9, }, // America/St_Johns
    {  137, 5863,   60961,  8, }, // America/St_Kitts
    {  137, 5880,   60969,  8, }, // America/St_Lucia
    {  137,  855,   60977,  9, }, // America/St_Thomas
    {  137, 5913,   60986, 10, }, // America/St_Vincent
    {  137, 5523,    5609,  5, }, // Asia/Aqtau
    {  137, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  137, 3049,    4619,  9, }, // Pacific/Kanton
    {  139, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Cameroon
    {  139, 2061,   60952,  9, }, // America/St_Johns
    {  139, 5863,   60961,  8, }, // America/St_Kitts
    {  139, 5880,   60969,  8, }, // America/St_Lucia
    {  139,  855,   60977,  9, }, // America/St_Thomas
    {  139, 5913,   60986, 10, }, // America/St_Vincent
    {  139, 5523,    5609,  5, }, // Asia/Aqtau
    {  139, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  139, 3049,    4619,  9, }, // Pacific/Kanton
    {  140, 8508,   26721, 16, }, // America/St_Barthelemy English/Latin/Canada
    {  140, 2061,   26737, 12, }, // America/St_Johns
    {  140, 5863,   26749, 11, }, // America/St_Kitts
    {  140, 5880,   35964, 11, }, // America/St_Lucia
    {  140,  855,   44293, 12, }, // America/St_Thomas
    {  140, 5913,   26783, 13, }, // America/St_Vincent
    {  140, 1235,   58618,  7, }, // Asia/Yangon
    {  140, 6821,   36198, 12, }, // Atlantic/St_Helena
    {  141, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Cayman Islands
    {  141, 2061,   60952,  9, }, // America/St_Johns
    {  141, 5863,   60961,  8, }, // America/St_Kitts
    {  141, 5880,   60969,  8, }, // America/St_Lucia
    {  141,  855,   60977,  9, }, // America/St_Thomas
    {  141, 5913,   60986, 10, }, // America/St_Vincent
    {  141, 5523,    5609,  5, }, // Asia/Aqtau
    {  141, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  141, 3049,    4619,  9, }, // Pacific/Kanton
    {  142, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Christmas Island
    {  142, 2061,   60952,  9, }, // America/St_Johns
    {  142, 5863,   60961,  8, }, // America/St_Kitts
    {  142, 5880,   60969,  8, }, // America/St_Lucia
    {  142,  855,   60977,  9, }, // America/St_Thomas
    {  142, 5913,   60986, 10, }, // America/St_Vincent
    {  142, 5523,    5609,  5, }, // Asia/Aqtau
    {  142, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  142, 3049,    4619,  9, }, // Pacific/Kanton
    {  143, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Cocos Islands
    {  143, 2061,   60952,  9, }, // America/St_Johns
    {  143, 5863,   60961,  8, }, // America/St_Kitts
    {  143, 5880,   60969,  8, }, // America/St_Lucia
    {  143,  855,   60977,  9, }, // America/St_Thomas
    {  143, 5913,   60986, 10, }, // America/St_Vincent
    {  143, 5523,    5609,  5, }, // Asia/Aqtau
    {  143, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  143, 3049,    4619,  9, }, // Pacific/Kanton
    {  144, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Cook Islands
    {  144, 2061,   60952,  9, }, // America/St_Johns
    {  144, 5863,   60961,  8, }, // America/St_Kitts
    {  144, 5880,   60969,  8, }, // America/St_Lucia
    {  144,  855,   60977,  9, }, // America/St_Thomas
    {  144, 5913,   60986, 10, }, // America/St_Vincent
    {  144, 5523,    5609,  5, }, // Asia/Aqtau
    {  144, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  144, 3049,    4619,  9, }, // Pacific/Kanton
    {  145, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Cyprus
    {  145, 2061,   60952,  9, }, // America/St_Johns
    {  145, 5863,   60961,  8, }, // America/St_Kitts
    {  145, 5880,   60969,  8, }, // America/St_Lucia
    {  145,  855,   60977,  9, }, // America/St_Thomas
    {  145, 5913,   60986, 10, }, // America/St_Vincent
    {  145, 5523,    5609,  5, }, // Asia/Aqtau
    {  145, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  145, 3049,    4619,  9, }, // Pacific/Kanton
    {  146, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Czechia
    {  146, 2061,   60952,  9, }, // America/St_Johns
    {  146, 5863,   60961,  8, }, // America/St_Kitts
    {  146, 5880,   60969,  8, }, // America/St_Lucia
    {  146,  855,   60977,  9, }, // America/St_Thomas
    {  146, 5913,   60986, 10, }, // America/St_Vincent
    {  146, 5523,    5609,  5, }, // Asia/Aqtau
    {  146, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  146, 3049,    4619,  9, }, // Pacific/Kanton
    {  147, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Denmark
    {  147, 2061,   60952,  9, }, // America/St_Johns
    {  147, 5863,   60961,  8, }, // America/St_Kitts
    {  147, 5880,   60969,  8, }, // America/St_Lucia
    {  147,  855,   60977,  9, }, // America/St_Thomas
    {  147, 5913,   60986, 10, }, // America/St_Vincent
    {  147, 5523,    5609,  5, }, // Asia/Aqtau
    {  147, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  147, 3049,    4619,  9, }, // Pacific/Kanton
    {  148, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Diego Garcia
    {  148, 2061,   60952,  9, }, // America/St_Johns
    {  148, 5863,   60961,  8, }, // America/St_Kitts
    {  148, 5880,   60969,  8, }, // America/St_Lucia
    {  148,  855,   60977,  9, }, // America/St_Thomas
    {  148, 5913,   60986, 10, }, // America/St_Vincent
    {  148, 5523,    5609,  5, }, // Asia/Aqtau
    {  148, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  148, 3049,    4619,  9, }, // Pacific/Kanton
    {  149, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Dominica
    {  149, 2061,   60952,  9, }, // America/St_Johns
    {  149, 5863,   60961,  8, }, // America/St_Kitts
    {  149, 5880,   60969,  8, }, // America/St_Lucia
    {  149,  855,   60977,  9, }, // America/St_Thomas
    {  149, 5913,   60986, 10, }, // America/St_Vincent
    {  149, 5523,    5609,  5, }, // Asia/Aqtau
    {  149, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  149, 3049,    4619,  9, }, // Pacific/Kanton
    {  150, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Eritrea
    {  150, 2061,   60952,  9, }, // America/St_Johns
    {  150, 5863,   60961,  8, }, // America/St_Kitts
    {  150, 5880,   60969,  8, }, // America/St_Lucia
    {  150,  855,   60977,  9, }, // America/St_Thomas
    {  150, 5913,   60986, 10, }, // America/St_Vincent
    {  150, 5523,    5609,  5, }, // Asia/Aqtau
    {  150, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  150, 3049,    4619,  9, }, // Pacific/Kanton
    {  151, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Eswatini
    {  151, 2061,   60952,  9, }, // America/St_Johns
    {  151, 5863,   60961,  8, }, // America/St_Kitts
    {  151, 5880,   60969,  8, }, // America/St_Lucia
    {  151,  855,   60977,  9, }, // America/St_Thomas
    {  151, 5913,   60986, 10, }, // America/St_Vincent
    {  151, 5523,    5609,  5, }, // Asia/Aqtau
    {  151, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  151, 3049,    4619,  9, }, // Pacific/Kanton
    {  152, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Europe
    {  152, 2061,   60952,  9, }, // America/St_Johns
    {  152, 5863,   60961,  8, }, // America/St_Kitts
    {  152, 5880,   60969,  8, }, // America/St_Lucia
    {  152,  855,   60977,  9, }, // America/St_Thomas
    {  152, 5913,   60986, 10, }, // America/St_Vincent
    {  152, 5523,    5609,  5, }, // Asia/Aqtau
    {  152, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  152, 3049,    4619,  9, }, // Pacific/Kanton
    {  153, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Falkland Islands
    {  153, 2061,   60952,  9, }, // America/St_Johns
    {  153, 5863,   60961,  8, }, // America/St_Kitts
    {  153, 5880,   60969,  8, }, // America/St_Lucia
    {  153,  855,   60977,  9, }, // America/St_Thomas
    {  153, 5913,   60986, 10, }, // America/St_Vincent
    {  153, 5523,    5609,  5, }, // Asia/Aqtau
    {  153, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  153, 3049,    4619,  9, }, // Pacific/Kanton
    {  154, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Fiji
    {  154, 2061,   60952,  9, }, // America/St_Johns
    {  154, 5863,   60961,  8, }, // America/St_Kitts
    {  154, 5880,   60969,  8, }, // America/St_Lucia
    {  154,  855,   60977,  9, }, // America/St_Thomas
    {  154, 5913,   60986, 10, }, // America/St_Vincent
    {  154, 5523,    5609,  5, }, // Asia/Aqtau
    {  154, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  154, 3049,    4619,  9, }, // Pacific/Kanton
    {  155, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Finland
    {  155, 2061,   60952,  9, }, // America/St_Johns
    {  155, 5863,   60961,  8, }, // America/St_Kitts
    {  155, 5880,   60969,  8, }, // America/St_Lucia
    {  155,  855,   60977,  9, }, // America/St_Thomas
    {  155, 5913,   60986, 10, }, // America/St_Vincent
    {  155, 5523,    5609,  5, }, // Asia/Aqtau
    {  155, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  155, 3049,    4619,  9, }, // Pacific/Kanton
    {  156, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/France
    {  156, 2061,   60952,  9, }, // America/St_Johns
    {  156, 5863,   60961,  8, }, // America/St_Kitts
    {  156, 5880,   60969,  8, }, // America/St_Lucia
    {  156,  855,   60977,  9, }, // America/St_Thomas
    {  156, 5913,   60986, 10, }, // America/St_Vincent
    {  156, 5523,    5609,  5, }, // Asia/Aqtau
    {  156, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  156, 3049,    4619,  9, }, // Pacific/Kanton
    {  157, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Gambia
    {  157, 2061,   60952,  9, }, // America/St_Johns
    {  157, 5863,   60961,  8, }, // America/St_Kitts
    {  157, 5880,   60969,  8, }, // America/St_Lucia
    {  157,  855,   60977,  9, }, // America/St_Thomas
    {  157, 5913,   60986, 10, }, // America/St_Vincent
    {  157, 5523,    5609,  5, }, // Asia/Aqtau
    {  157, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  157, 3049,    4619,  9, }, // Pacific/Kanton
    {  158, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Germany
    {  158, 2061,   60952,  9, }, // America/St_Johns
    {  158, 5863,   60961,  8, }, // America/St_Kitts
    {  158, 5880,   60969,  8, }, // America/St_Lucia
    {  158,  855,   60977,  9, }, // America/St_Thomas
    {  158, 5913,   60986, 10, }, // America/St_Vincent
    {  158, 5523,    5609,  5, }, // Asia/Aqtau
    {  158, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  158, 3049,    4619,  9, }, // Pacific/Kanton
    {  159, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Ghana
    {  159, 2061,   60952,  9, }, // America/St_Johns
    {  159, 5863,   60961,  8, }, // America/St_Kitts
    {  159, 5880,   60969,  8, }, // America/St_Lucia
    {  159,  855,   60977,  9, }, // America/St_Thomas
    {  159, 5913,   60986, 10, }, // America/St_Vincent
    {  159, 5523,    5609,  5, }, // Asia/Aqtau
    {  159, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  159, 3049,    4619,  9, }, // Pacific/Kanton
    {  160, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Gibraltar
    {  160, 2061,   60952,  9, }, // America/St_Johns
    {  160, 5863,   60961,  8, }, // America/St_Kitts
    {  160, 5880,   60969,  8, }, // America/St_Lucia
    {  160,  855,   60977,  9, }, // America/St_Thomas
    {  160, 5913,   60986, 10, }, // America/St_Vincent
    {  160, 5523,    5609,  5, }, // Asia/Aqtau
    {  160, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  160, 3049,    4619,  9, }, // Pacific/Kanton
    {  161, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Grenada
    {  161, 2061,   60952,  9, }, // America/St_Johns
    {  161, 5863,   60961,  8, }, // America/St_Kitts
    {  161, 5880,   60969,  8, }, // America/St_Lucia
    {  161,  855,   60977,  9, }, // America/St_Thomas
    {  161, 5913,   60986, 10, }, // America/St_Vincent
    {  161, 5523,    5609,  5, }, // Asia/Aqtau
    {  161, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  161, 3049,    4619,  9, }, // Pacific/Kanton
    {  163, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Guernsey
    {  163, 2061,   60952,  9, }, // America/St_Johns
    {  163, 5863,   60961,  8, }, // America/St_Kitts
    {  163, 5880,   60969,  8, }, // America/St_Lucia
    {  163,  855,   60977,  9, }, // America/St_Thomas
    {  163, 5913,   60986, 10, }, // America/St_Vincent
    {  163, 5523,    5609,  5, }, // Asia/Aqtau
    {  163, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  163, 3049,    4619,  9, }, // Pacific/Kanton
    {  164, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Guyana
    {  164, 2061,   60952,  9, }, // America/St_Johns
    {  164, 5863,   60961,  8, }, // America/St_Kitts
    {  164, 5880,   60969,  8, }, // America/St_Lucia
    {  164,  855,   60977,  9, }, // America/St_Thomas
    {  164, 5913,   60986, 10, }, // America/St_Vincent
    {  164, 5523,    5609,  5, }, // Asia/Aqtau
    {  164, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  164, 3049,    4619,  9, }, // Pacific/Kanton
    {  165, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Hong Kong
    {  165, 2061,   60952,  9, }, // America/St_Johns
    {  165, 5863,   60961,  8, }, // America/St_Kitts
    {  165, 5880,   60969,  8, }, // America/St_Lucia
    {  165,  855,   60977,  9, }, // America/St_Thomas
    {  165, 5913,   60986, 10, }, // America/St_Vincent
    {  165, 5523,    5609,  5, }, // Asia/Aqtau
    {  165, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  165, 3049,    4619,  9, }, // Pacific/Kanton
    {  166, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Hungary
    {  166, 2061,   60952,  9, }, // America/St_Johns
    {  166, 5863,   60961,  8, }, // America/St_Kitts
    {  166, 5880,   60969,  8, }, // America/St_Lucia
    {  166,  855,   60977,  9, }, // America/St_Thomas
    {  166, 5913,   60986, 10, }, // America/St_Vincent
    {  166, 5523,    5609,  5, }, // Asia/Aqtau
    {  166, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  166, 3049,    4619,  9, }, // Pacific/Kanton
    {  167, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/India
    {  167, 2061,   60952,  9, }, // America/St_Johns
    {  167, 5863,   60961,  8, }, // America/St_Kitts
    {  167, 5880,   60969,  8, }, // America/St_Lucia
    {  167,  855,   60977,  9, }, // America/St_Thomas
    {  167, 5913,   60986, 10, }, // America/St_Vincent
    {  167, 5523,    5609,  5, }, // Asia/Aqtau
    {  167, 4771,   26835,  5, }, // Asia/Hovd
    {  167, 4269,   26897,  9, }, // Asia/Qyzylorda
    {  167, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  167, 3049,    4619,  9, }, // Pacific/Kanton
    {  168, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Indonesia
    {  168, 2061,   60952,  9, }, // America/St_Johns
    {  168, 5863,   60961,  8, }, // America/St_Kitts
    {  168, 5880,   60969,  8, }, // America/St_Lucia
    {  168,  855,   60977,  9, }, // America/St_Thomas
    {  168, 5913,   60986, 10, }, // America/St_Vincent
    {  168, 5523,    5609,  5, }, // Asia/Aqtau
    {  168, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  168, 3049,    4619,  9, }, // Pacific/Kanton
    {  169, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Ireland
    {  169, 2061,   60952,  9, }, // America/St_Johns
    {  169, 5863,   60961,  8, }, // America/St_Kitts
    {  169, 5880,   60969,  8, }, // America/St_Lucia
    {  169,  855,   60977,  9, }, // America/St_Thomas
    {  169, 5913,   60986, 10, }, // America/St_Vincent
    {  169, 5523,    5609,  5, }, // Asia/Aqtau
    {  169, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  169, 3049,    4619,  9, }, // Pacific/Kanton
    {  170, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Isle of Man
    {  170, 2061,   60952,  9, }, // America/St_Johns
    {  170, 5863,   60961,  8, }, // America/St_Kitts
    {  170, 5880,   60969,  8, }, // America/St_Lucia
    {  170,  855,   60977,  9, }, // America/St_Thomas
    {  170, 5913,   60986, 10, }, // America/St_Vincent
    {  170, 5523,    5609,  5, }, // Asia/Aqtau
    {  170, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  170, 3049,    4619,  9, }, // Pacific/Kanton
    {  171, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Israel
    {  171, 2061,   60952,  9, }, // America/St_Johns
    {  171, 5863,   60961,  8, }, // America/St_Kitts
    {  171, 5880,   60969,  8, }, // America/St_Lucia
    {  171,  855,   60977,  9, }, // America/St_Thomas
    {  171, 5913,   60986, 10, }, // America/St_Vincent
    {  171, 5523,    5609,  5, }, // Asia/Aqtau
    {  171, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  171, 3049,    4619,  9, }, // Pacific/Kanton
    {  172, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Italy
    {  172, 2061,   60952,  9, }, // America/St_Johns
    {  172, 5863,   60961,  8, }, // America/St_Kitts
    {  172, 5880,   60969,  8, }, // America/St_Lucia
    {  172,  855,   60977,  9, }, // America/St_Thomas
    {  172, 5913,   60986, 10, }, // America/St_Vincent
    {  172, 5523,    5609,  5, }, // Asia/Aqtau
    {  172, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  172, 3049,    4619,  9, }, // Pacific/Kanton
    {  173, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Jamaica
    {  173, 2061,   60952,  9, }, // America/St_Johns
    {  173, 5863,   60961,  8, }, // America/St_Kitts
    {  173, 5880,   60969,  8, }, // America/St_Lucia
    {  173,  855,   60977,  9, }, // America/St_Thomas
    {  173, 5913,   60986, 10, }, // America/St_Vincent
    {  173, 5523,    5609,  5, }, // Asia/Aqtau
    {  173, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  173, 3049,    4619,  9, }, // Pacific/Kanton
    {  174, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Jersey
    {  174, 2061,   60952,  9, }, // America/St_Johns
    {  174, 5863,   60961,  8, }, // America/St_Kitts
    {  174, 5880,   60969,  8, }, // America/St_Lucia
    {  174,  855,   60977,  9, }, // America/St_Thomas
    {  174, 5913,   60986, 10, }, // America/St_Vincent
    {  174, 5523,    5609,  5, }, // Asia/Aqtau
    {  174, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  174, 3049,    4619,  9, }, // Pacific/Kanton
    {  175, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Kenya
    {  175, 2061,   60952,  9, }, // America/St_Johns
    {  175, 5863,   60961,  8, }, // America/St_Kitts
    {  175, 5880,   60969,  8, }, // America/St_Lucia
    {  175,  855,   60977,  9, }, // America/St_Thomas
    {  175, 5913,   60986, 10, }, // America/St_Vincent
    {  175, 5523,    5609,  5, }, // Asia/Aqtau
    {  175, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  175, 3049,    4619,  9, }, // Pacific/Kanton
    {  176, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Kiribati
    {  176, 2061,   60952,  9, }, // America/St_Johns
    {  176, 5863,   60961,  8, }, // America/St_Kitts
    {  176, 5880,   60969,  8, }, // America/St_Lucia
    {  176,  855,   60977,  9, }, // America/St_Thomas
    {  176, 5913,   60986, 10, }, // America/St_Vincent
    {  176, 5523,    5609,  5, }, // Asia/Aqtau
    {  176, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  176, 3049,    4619,  9, }, // Pacific/Kanton
    {  177, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Lesotho
    {  177, 2061,   60952,  9, }, // America/St_Johns
    {  177, 5863,   60961,  8, }, // America/St_Kitts
    {  177, 5880,   60969,  8, }, // America/St_Lucia
    {  177,  855,   60977,  9, }, // America/St_Thomas
    {  177, 5913,   60986, 10, }, // America/St_Vincent
    {  177, 5523,    5609,  5, }, // Asia/Aqtau
    {  177, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  177, 3049,    4619,  9, }, // Pacific/Kanton
    {  178, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Liberia
    {  178, 2061,   60952,  9, }, // America/St_Johns
    {  178, 5863,   60961,  8, }, // America/St_Kitts
    {  178, 5880,   60969,  8, }, // America/St_Lucia
    {  178,  855,   60977,  9, }, // America/St_Thomas
    {  178, 5913,   60986, 10, }, // America/St_Vincent
    {  178, 5523,    5609,  5, }, // Asia/Aqtau
    {  178, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  178, 3049,    4619,  9, }, // Pacific/Kanton
    {  179, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Macao
    {  179, 2061,   60952,  9, }, // America/St_Johns
    {  179, 5863,   60961,  8, }, // America/St_Kitts
    {  179, 5880,   60969,  8, }, // America/St_Lucia
    {  179,  855,   60977,  9, }, // America/St_Thomas
    {  179, 5913,   60986, 10, }, // America/St_Vincent
    {  179, 5523,    5609,  5, }, // Asia/Aqtau
    {  179, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  179, 3049,    4619,  9, }, // Pacific/Kanton
    {  180, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Madagascar
    {  180, 2061,   60952,  9, }, // America/St_Johns
    {  180, 5863,   60961,  8, }, // America/St_Kitts
    {  180, 5880,   60969,  8, }, // America/St_Lucia
    {  180,  855,   60977,  9, }, // America/St_Thomas
    {  180, 5913,   60986, 10, }, // America/St_Vincent
    {  180, 5523,    5609,  5, }, // Asia/Aqtau
    {  180, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  180, 3049,    4619,  9, }, // Pacific/Kanton
    {  181, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Malawi
    {  181, 2061,   60952,  9, }, // America/St_Johns
    {  181, 5863,   60961,  8, }, // America/St_Kitts
    {  181, 5880,   60969,  8, }, // America/St_Lucia
    {  181,  855,   60977,  9, }, // America/St_Thomas
    {  181, 5913,   60986, 10, }, // America/St_Vincent
    {  181, 5523,    5609,  5, }, // Asia/Aqtau
    {  181, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  181, 3049,    4619,  9, }, // Pacific/Kanton
    {  182, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Malaysia
    {  182, 2061,   60952,  9, }, // America/St_Johns
    {  182, 5863,   60961,  8, }, // America/St_Kitts
    {  182, 5880,   60969,  8, }, // America/St_Lucia
    {  182,  855,   60977,  9, }, // America/St_Thomas
    {  182, 5913,   60986, 10, }, // America/St_Vincent
    {  182, 5523,    5609,  5, }, // Asia/Aqtau
    {  182, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  182, 3049,    4619,  9, }, // Pacific/Kanton
    {  183, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Maldives
    {  183, 2061,   60952,  9, }, // America/St_Johns
    {  183, 5863,   60961,  8, }, // America/St_Kitts
    {  183, 5880,   60969,  8, }, // America/St_Lucia
    {  183,  855,   60977,  9, }, // America/St_Thomas
    {  183, 5913,   60986, 10, }, // America/St_Vincent
    {  183, 5523,    5609,  5, }, // Asia/Aqtau
    {  183, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  183, 3049,    4619,  9, }, // Pacific/Kanton
    {  184, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Malta
    {  184, 2061,   60952,  9, }, // America/St_Johns
    {  184, 5863,   60961,  8, }, // America/St_Kitts
    {  184, 5880,   60969,  8, }, // America/St_Lucia
    {  184,  855,   60977,  9, }, // America/St_Thomas
    {  184, 5913,   60986, 10, }, // America/St_Vincent
    {  184, 5523,    5609,  5, }, // Asia/Aqtau
    {  184, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  184, 3049,    4619,  9, }, // Pacific/Kanton
    {  186, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Mauritius
    {  186, 2061,   60952,  9, }, // America/St_Johns
    {  186, 5863,   60961,  8, }, // America/St_Kitts
    {  186, 5880,   60969,  8, }, // America/St_Lucia
    {  186,  855,   60977,  9, }, // America/St_Thomas
    {  186, 5913,   60986, 10, }, // America/St_Vincent
    {  186, 5523,    5609,  5, }, // Asia/Aqtau
    {  186, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  186, 3049,    4619,  9, }, // Pacific/Kanton
    {  187, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Micronesia
    {  187, 2061,   60952,  9, }, // America/St_Johns
    {  187, 5863,   60961,  8, }, // America/St_Kitts
    {  187, 5880,   60969,  8, }, // America/St_Lucia
    {  187,  855,   60977,  9, }, // America/St_Thomas
    {  187, 5913,   60986, 10, }, // America/St_Vincent
    {  187, 5523,    5609,  5, }, // Asia/Aqtau
    {  187, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  187, 3049,    4619,  9, }, // Pacific/Kanton
    {  188, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Montserrat
    {  188, 2061,   60952,  9, }, // America/St_Johns
    {  188, 5863,   60961,  8, }, // America/St_Kitts
    {  188, 5880,   60969,  8, }, // America/St_Lucia
    {  188,  855,   60977,  9, }, // America/St_Thomas
    {  188, 5913,   60986, 10, }, // America/St_Vincent
    {  188, 5523,    5609,  5, }, // Asia/Aqtau
    {  188, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  188, 3049,    4619,  9, }, // Pacific/Kanton
    {  189, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Namibia
    {  189, 2061,   60952,  9, }, // America/St_Johns
    {  189, 5863,   60961,  8, }, // America/St_Kitts
    {  189, 5880,   60969,  8, }, // America/St_Lucia
    {  189,  855,   60977,  9, }, // America/St_Thomas
    {  189, 5913,   60986, 10, }, // America/St_Vincent
    {  189, 5523,    5609,  5, }, // Asia/Aqtau
    {  189, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  189, 3049,    4619,  9, }, // Pacific/Kanton
    {  190, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Nauru
    {  190, 2061,   60952,  9, }, // America/St_Johns
    {  190, 5863,   60961,  8, }, // America/St_Kitts
    {  190, 5880,   60969,  8, }, // America/St_Lucia
    {  190,  855,   60977,  9, }, // America/St_Thomas
    {  190, 5913,   60986, 10, }, // America/St_Vincent
    {  190, 5523,    5609,  5, }, // Asia/Aqtau
    {  190, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  190, 3049,    4619,  9, }, // Pacific/Kanton
    {  191, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Netherlands
    {  191, 2061,   60952,  9, }, // America/St_Johns
    {  191, 5863,   60961,  8, }, // America/St_Kitts
    {  191, 5880,   60969,  8, }, // America/St_Lucia
    {  191,  855,   60977,  9, }, // America/St_Thomas
    {  191, 5913,   60986, 10, }, // America/St_Vincent
    {  191, 5523,    5609,  5, }, // Asia/Aqtau
    {  191, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  191, 3049,    4619,  9, }, // Pacific/Kanton
    {  192, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/New Zealand
    {  192, 2061,   60952,  9, }, // America/St_Johns
    {  192, 5863,   60961,  8, }, // America/St_Kitts
    {  192, 5880,   60969,  8, }, // America/St_Lucia
    {  192,  855,   60977,  9, }, // America/St_Thomas
    {  192, 5913,   60986, 10, }, // America/St_Vincent
    {  192, 5523,    5609,  5, }, // Asia/Aqtau
    {  192, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  192, 3049,    4619,  9, }, // Pacific/Kanton
    {  193, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Nigeria
    {  193, 2061,   60952,  9, }, // America/St_Johns
    {  193, 5863,   60961,  8, }, // America/St_Kitts
    {  193, 5880,   60969,  8, }, // America/St_Lucia
    {  193,  855,   60977,  9, }, // America/St_Thomas
    {  193, 5913,   60986, 10, }, // America/St_Vincent
    {  193, 5523,    5609,  5, }, // Asia/Aqtau
    {  193, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  193, 3049,    4619,  9, }, // Pacific/Kanton
    {  194, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Niue
    {  194, 2061,   60952,  9, }, // America/St_Johns
    {  194, 5863,   60961,  8, }, // America/St_Kitts
    {  194, 5880,   60969,  8, }, // America/St_Lucia
    {  194,  855,   60977,  9, }, // America/St_Thomas
    {  194, 5913,   60986, 10, }, // America/St_Vincent
    {  194, 5523,    5609,  5, }, // Asia/Aqtau
    {  194, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  194, 3049,    4619,  9, }, // Pacific/Kanton
    {  195, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Norfolk Island
    {  195, 2061,   60952,  9, }, // America/St_Johns
    {  195, 5863,   60961,  8, }, // America/St_Kitts
    {  195, 5880,   60969,  8, }, // America/St_Lucia
    {  195,  855,   60977,  9, }, // America/St_Thomas
    {  195, 5913,   60986, 10, }, // America/St_Vincent
    {  195, 5523,    5609,  5, }, // Asia/Aqtau
    {  195, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  195, 3049,    4619,  9, }, // Pacific/Kanton
    {  197, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Norway
    {  197, 2061,   60952,  9, }, // America/St_Johns
    {  197, 5863,   60961,  8, }, // America/St_Kitts
    {  197, 5880,   60969,  8, }, // America/St_Lucia
    {  197,  855,   60977,  9, }, // America/St_Thomas
    {  197, 5913,   60986, 10, }, // America/St_Vincent
    {  197, 5523,    5609,  5, }, // Asia/Aqtau
    {  197, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  197, 3049,    4619,  9, }, // Pacific/Kanton
    {  198, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Pakistan
    {  198, 2061,   60952,  9, }, // America/St_Johns
    {  198, 5863,   60961,  8, }, // America/St_Kitts
    {  198, 5880,   60969,  8, }, // America/St_Lucia
    {  198,  855,   60977,  9, }, // America/St_Thomas
    {  198, 5913,   60986, 10, }, // America/St_Vincent
    {  198, 5523,    5609,  5, }, // Asia/Aqtau
    {  198, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  198, 3049,    4619,  9, }, // Pacific/Kanton
    {  199, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Palau
    {  199, 2061,   60952,  9, }, // America/St_Johns
    {  199, 5863,   60961,  8, }, // America/St_Kitts
    {  199, 5880,   60969,  8, }, // America/St_Lucia
    {  199,  855,   60977,  9, }, // America/St_Thomas
    {  199, 5913,   60986, 10, }, // America/St_Vincent
    {  199, 5523,    5609,  5, }, // Asia/Aqtau
    {  199, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  199, 3049,    4619,  9, }, // Pacific/Kanton
    {  200, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Papua New Guinea
    {  200, 2061,   60952,  9, }, // America/St_Johns
    {  200, 5863,   60961,  8, }, // America/St_Kitts
    {  200, 5880,   60969,  8, }, // America/St_Lucia
    {  200,  855,   60977,  9, }, // America/St_Thomas
    {  200, 5913,   60986, 10, }, // America/St_Vincent
    {  200, 5523,    5609,  5, }, // Asia/Aqtau
    {  200, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  200, 3049,    4619,  9, }, // Pacific/Kanton
    {  202, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Pitcairn
    {  202, 2061,   60952,  9, }, // America/St_Johns
    {  202, 5863,   60961,  8, }, // America/St_Kitts
    {  202, 5880,   60969,  8, }, // America/St_Lucia
    {  202,  855,   60977,  9, }, // America/St_Thomas
    {  202, 5913,   60986, 10, }, // America/St_Vincent
    {  202, 5523,    5609,  5, }, // Asia/Aqtau
    {  202, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  202, 3049,    4619,  9, }, // Pacific/Kanton
    {  203, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Poland
    {  203, 2061,   60952,  9, }, // America/St_Johns
    {  203, 5863,   60961,  8, }, // America/St_Kitts
    {  203, 5880,   60969,  8, }, // America/St_Lucia
    {  203,  855,   60977,  9, }, // America/St_Thomas
    {  203, 5913,   60986, 10, }, // America/St_Vincent
    {  203, 5523,    5609,  5, }, // Asia/Aqtau
    {  203, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  203, 3049,    4619,  9, }, // Pacific/Kanton
    {  204, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Portugal
    {  204, 2061,   60952,  9, }, // America/St_Johns
    {  204, 5863,   60961,  8, }, // America/St_Kitts
    {  204, 5880,   60969,  8, }, // America/St_Lucia
    {  204,  855,   60977,  9, }, // America/St_Thomas
    {  204, 5913,   60986, 10, }, // America/St_Vincent
    {  204, 5523,    5609,  5, }, // Asia/Aqtau
    {  204, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  204, 3049,    4619,  9, }, // Pacific/Kanton
    {  206, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Romania
    {  206, 2061,   60952,  9, }, // America/St_Johns
    {  206, 5863,   60961,  8, }, // America/St_Kitts
    {  206, 5880,   60969,  8, }, // America/St_Lucia
    {  206,  855,   60977,  9, }, // America/St_Thomas
    {  206, 5913,   60986, 10, }, // America/St_Vincent
    {  206, 5523,    5609,  5, }, // Asia/Aqtau
    {  206, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  206, 3049,    4619,  9, }, // Pacific/Kanton
    {  207, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Rwanda
    {  207, 2061,   60952,  9, }, // America/St_Johns
    {  207, 5863,   60961,  8, }, // America/St_Kitts
    {  207, 5880,   60969,  8, }, // America/St_Lucia
    {  207,  855,   60977,  9, }, // America/St_Thomas
    {  207, 5913,   60986, 10, }, // America/St_Vincent
    {  207, 5523,    5609,  5, }, // Asia/Aqtau
    {  207, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  207, 3049,    4619,  9, }, // Pacific/Kanton
    {  208, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Saint Helena
    {  208, 2061,   60952,  9, }, // America/St_Johns
    {  208, 5863,   60961,  8, }, // America/St_Kitts
    {  208, 5880,   60969,  8, }, // America/St_Lucia
    {  208,  855,   60977,  9, }, // America/St_Thomas
    {  208, 5913,   60986, 10, }, // America/St_Vincent
    {  208, 5523,    5609,  5, }, // Asia/Aqtau
    {  208, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  208, 3049,    4619,  9, }, // Pacific/Kanton
    {  209, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Saint Kitts and Nevis
    {  209, 2061,   60952,  9, }, // America/St_Johns
    {  209, 5863,   60961,  8, }, // America/St_Kitts
    {  209, 5880,   60969,  8, }, // America/St_Lucia
    {  209,  855,   60977,  9, }, // America/St_Thomas
    {  209, 5913,   60986, 10, }, // America/St_Vincent
    {  209, 5523,    5609,  5, }, // Asia/Aqtau
    {  209, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  209, 3049,    4619,  9, }, // Pacific/Kanton
    {  210, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Saint Lucia
    {  210, 2061,   60952,  9, }, // America/St_Johns
    {  210, 5863,   60961,  8, }, // America/St_Kitts
    {  210, 5880,   60969,  8, }, // America/St_Lucia
    {  210,  855,   60977,  9, }, // America/St_Thomas
    {  210, 5913,   60986, 10, }, // America/St_Vincent
    {  210, 5523,    5609,  5, }, // Asia/Aqtau
    {  210, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  210, 3049,    4619,  9, }, // Pacific/Kanton
    {  211, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Saint Vincent and Grenadines
    {  211, 2061,   60952,  9, }, // America/St_Johns
    {  211, 5863,   60961,  8, }, // America/St_Kitts
    {  211, 5880,   60969,  8, }, // America/St_Lucia
    {  211,  855,   60977,  9, }, // America/St_Thomas
    {  211, 5913,   60986, 10, }, // America/St_Vincent
    {  211, 5523,    5609,  5, }, // Asia/Aqtau
    {  211, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  211, 3049,    4619,  9, }, // Pacific/Kanton
    {  212, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Samoa
    {  212, 2061,   60952,  9, }, // America/St_Johns
    {  212, 5863,   60961,  8, }, // America/St_Kitts
    {  212, 5880,   60969,  8, }, // America/St_Lucia
    {  212,  855,   60977,  9, }, // America/St_Thomas
    {  212, 5913,   60986, 10, }, // America/St_Vincent
    {  212, 5523,    5609,  5, }, // Asia/Aqtau
    {  212, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  212, 3049,    4619,  9, }, // Pacific/Kanton
    {  213, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Seychelles
    {  213, 2061,   60952,  9, }, // America/St_Johns
    {  213, 5863,   60961,  8, }, // America/St_Kitts
    {  213, 5880,   60969,  8, }, // America/St_Lucia
    {  213,  855,   60977,  9, }, // America/St_Thomas
    {  213, 5913,   60986, 10, }, // America/St_Vincent
    {  213, 5523,    5609,  5, }, // Asia/Aqtau
    {  213, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  213, 3049,    4619,  9, }, // Pacific/Kanton
    {  214, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Sierra Leone
    {  214, 2061,   60952,  9, }, // America/St_Johns
    {  214, 5863,   60961,  8, }, // America/St_Kitts
    {  214, 5880,   60969,  8, }, // America/St_Lucia
    {  214,  855,   60977,  9, }, // America/St_Thomas
    {  214, 5913,   60986, 10, }, // America/St_Vincent
    {  214, 5523,    5609,  5, }, // Asia/Aqtau
    {  214, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  214, 3049,    4619,  9, }, // Pacific/Kanton
    {  215, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Singapore
    {  215, 2061,   60952,  9, }, // America/St_Johns
    {  215, 5863,   60961,  8, }, // America/St_Kitts
    {  215, 5880,   60969,  8, }, // America/St_Lucia
    {  215,  855,   60977,  9, }, // America/St_Thomas
    {  215, 5913,   60986, 10, }, // America/St_Vincent
    {  215, 5523,    5609,  5, }, // Asia/Aqtau
    {  215, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  215, 3049,    4619,  9, }, // Pacific/Kanton
    {  216, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Sint Maarten
    {  216, 2061,   60952,  9, }, // America/St_Johns
    {  216, 5863,   60961,  8, }, // America/St_Kitts
    {  216, 5880,   60969,  8, }, // America/St_Lucia
    {  216,  855,   60977,  9, }, // America/St_Thomas
    {  216, 5913,   60986, 10, }, // America/St_Vincent
    {  216, 5523,    5609,  5, }, // Asia/Aqtau
    {  216, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  216, 3049,    4619,  9, }, // Pacific/Kanton
    {  217, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Slovakia
    {  217, 2061,   60952,  9, }, // America/St_Johns
    {  217, 5863,   60961,  8, }, // America/St_Kitts
    {  217, 5880,   60969,  8, }, // America/St_Lucia
    {  217,  855,   60977,  9, }, // America/St_Thomas
    {  217, 5913,   60986, 10, }, // America/St_Vincent
    {  217, 5523,    5609,  5, }, // Asia/Aqtau
    {  217, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  217, 3049,    4619,  9, }, // Pacific/Kanton
    {  218, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Slovenia
    {  218, 2061,   60952,  9, }, // America/St_Johns
    {  218, 5863,   60961,  8, }, // America/St_Kitts
    {  218, 5880,   60969,  8, }, // America/St_Lucia
    {  218,  855,   60977,  9, }, // America/St_Thomas
    {  218, 5913,   60986, 10, }, // America/St_Vincent
    {  218, 5523,    5609,  5, }, // Asia/Aqtau
    {  218, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  218, 3049,    4619,  9, }, // Pacific/Kanton
    {  219, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Solomon Islands
    {  219, 2061,   60952,  9, }, // America/St_Johns
    {  219, 5863,   60961,  8, }, // America/St_Kitts
    {  219, 5880,   60969,  8, }, // America/St_Lucia
    {  219,  855,   60977,  9, }, // America/St_Thomas
    {  219, 5913,   60986, 10, }, // America/St_Vincent
    {  219, 5523,    5609,  5, }, // Asia/Aqtau
    {  219, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  219, 3049,    4619,  9, }, // Pacific/Kanton
    {  220, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/South Africa
    {  220, 2061,   60952,  9, }, // America/St_Johns
    {  220, 5863,   60961,  8, }, // America/St_Kitts
    {  220, 5880,   60969,  8, }, // America/St_Lucia
    {  220,  855,   60977,  9, }, // America/St_Thomas
    {  220, 5913,   60986, 10, }, // America/St_Vincent
    {  220, 5523,    5609,  5, }, // Asia/Aqtau
    {  220, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  220, 3049,    4619,  9, }, // Pacific/Kanton
    {  221, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/South Georgia and South Sandwich Islands
    {  221, 2061,   60952,  9, }, // America/St_Johns
    {  221, 5863,   60961,  8, }, // America/St_Kitts
    {  221, 5880,   60969,  8, }, // America/St_Lucia
    {  221,  855,   60977,  9, }, // America/St_Thomas
    {  221, 5913,   60986, 10, }, // America/St_Vincent
    {  221, 5523,    5609,  5, }, // Asia/Aqtau
    {  221, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  221, 3049,    4619,  9, }, // Pacific/Kanton
    {  222, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/South Sudan
    {  222, 2061,   60952,  9, }, // America/St_Johns
    {  222, 5863,   60961,  8, }, // America/St_Kitts
    {  222, 5880,   60969,  8, }, // America/St_Lucia
    {  222,  855,   60977,  9, }, // America/St_Thomas
    {  222, 5913,   60986, 10, }, // America/St_Vincent
    {  222, 5523,    5609,  5, }, // Asia/Aqtau
    {  222, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  222, 3049,    4619,  9, }, // Pacific/Kanton
    {  223, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Spain
    {  223, 2061,   60952,  9, }, // America/St_Johns
    {  223, 5863,   60961,  8, }, // America/St_Kitts
    {  223, 5880,   60969,  8, }, // America/St_Lucia
    {  223,  855,   60977,  9, }, // America/St_Thomas
    {  223, 5913,   60986, 10, }, // America/St_Vincent
    {  223, 5523,    5609,  5, }, // Asia/Aqtau
    {  223, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  223, 3049,    4619,  9, }, // Pacific/Kanton
    {  224, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Sudan
    {  224, 2061,   60952,  9, }, // America/St_Johns
    {  224, 5863,   60961,  8, }, // America/St_Kitts
    {  224, 5880,   60969,  8, }, // America/St_Lucia
    {  224,  855,   60977,  9, }, // America/St_Thomas
    {  224, 5913,   60986, 10, }, // America/St_Vincent
    {  224, 5523,    5609,  5, }, // Asia/Aqtau
    {  224, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  224, 3049,    4619,  9, }, // Pacific/Kanton
    {  225, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Sweden
    {  225, 2061,   60952,  9, }, // America/St_Johns
    {  225, 5863,   60961,  8, }, // America/St_Kitts
    {  225, 5880,   60969,  8, }, // America/St_Lucia
    {  225,  855,   60977,  9, }, // America/St_Thomas
    {  225, 5913,   60986, 10, }, // America/St_Vincent
    {  225, 5523,    5609,  5, }, // Asia/Aqtau
    {  225, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  225, 3049,    4619,  9, }, // Pacific/Kanton
    {  226, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Switzerland
    {  226, 2061,   60952,  9, }, // America/St_Johns
    {  226, 5863,   60961,  8, }, // America/St_Kitts
    {  226, 5880,   60969,  8, }, // America/St_Lucia
    {  226,  855,   60977,  9, }, // America/St_Thomas
    {  226, 5913,   60986, 10, }, // America/St_Vincent
    {  226, 5523,    5609,  5, }, // Asia/Aqtau
    {  226, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  226, 3049,    4619,  9, }, // Pacific/Kanton
    {  227, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Tanzania
    {  227, 2061,   60952,  9, }, // America/St_Johns
    {  227, 5863,   60961,  8, }, // America/St_Kitts
    {  227, 5880,   60969,  8, }, // America/St_Lucia
    {  227,  855,   60977,  9, }, // America/St_Thomas
    {  227, 5913,   60986, 10, }, // America/St_Vincent
    {  227, 5523,    5609,  5, }, // Asia/Aqtau
    {  227, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  227, 3049,    4619,  9, }, // Pacific/Kanton
    {  228, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Tokelau
    {  228, 2061,   60952,  9, }, // America/St_Johns
    {  228, 5863,   60961,  8, }, // America/St_Kitts
    {  228, 5880,   60969,  8, }, // America/St_Lucia
    {  228,  855,   60977,  9, }, // America/St_Thomas
    {  228, 5913,   60986, 10, }, // America/St_Vincent
    {  228, 5523,    5609,  5, }, // Asia/Aqtau
    {  228, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  228, 3049,    4619,  9, }, // Pacific/Kanton
    {  229, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Tonga
    {  229, 2061,   60952,  9, }, // America/St_Johns
    {  229, 5863,   60961,  8, }, // America/St_Kitts
    {  229, 5880,   60969,  8, }, // America/St_Lucia
    {  229,  855,   60977,  9, }, // America/St_Thomas
    {  229, 5913,   60986, 10, }, // America/St_Vincent
    {  229, 5523,    5609,  5, }, // Asia/Aqtau
    {  229, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  229, 3049,    4619,  9, }, // Pacific/Kanton
    {  230, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Trinidad and Tobago
    {  230, 2061,   60952,  9, }, // America/St_Johns
    {  230, 5863,   60961,  8, }, // America/St_Kitts
    {  230, 5880,   60969,  8, }, // America/St_Lucia
    {  230,  855,   60977,  9, }, // America/St_Thomas
    {  230, 5913,   60986, 10, }, // America/St_Vincent
    {  230, 5523,    5609,  5, }, // Asia/Aqtau
    {  230, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  230, 3049,    4619,  9, }, // Pacific/Kanton
    {  231, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Turks and Caicos Islands
    {  231, 2061,   60952,  9, }, // America/St_Johns
    {  231, 5863,   60961,  8, }, // America/St_Kitts
    {  231, 5880,   60969,  8, }, // America/St_Lucia
    {  231,  855,   60977,  9, }, // America/St_Thomas
    {  231, 5913,   60986, 10, }, // America/St_Vincent
    {  231, 5523,    5609,  5, }, // Asia/Aqtau
    {  231, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  231, 3049,    4619,  9, }, // Pacific/Kanton
    {  232, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Tuvalu
    {  232, 2061,   60952,  9, }, // America/St_Johns
    {  232, 5863,   60961,  8, }, // America/St_Kitts
    {  232, 5880,   60969,  8, }, // America/St_Lucia
    {  232,  855,   60977,  9, }, // America/St_Thomas
    {  232, 5913,   60986, 10, }, // America/St_Vincent
    {  232, 5523,    5609,  5, }, // Asia/Aqtau
    {  232, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  232, 3049,    4619,  9, }, // Pacific/Kanton
    {  233, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Uganda
    {  233, 2061,   60952,  9, }, // America/St_Johns
    {  233, 5863,   60961,  8, }, // America/St_Kitts
    {  233, 5880,   60969,  8, }, // America/St_Lucia
    {  233,  855,   60977,  9, }, // America/St_Thomas
    {  233, 5913,   60986, 10, }, // America/St_Vincent
    {  233, 5523,    5609,  5, }, // Asia/Aqtau
    {  233, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  233, 3049,    4619,  9, }, // Pacific/Kanton
    {  235, 7623,    3943, 14, }, // America/Bahia_Banderas English/Latin/United Kingdom
    {  235, 3868,   33314,  6, }, // America/Cancun
    {  235, 8220,    7126,  6, }, // America/Merida
    {  235, 8508,   60939, 13, }, // America/St_Barthelemy
    {  235, 2061,   60952,  9, }, // America/St_Johns
    {  235, 5863,   60961,  8, }, // America/St_Kitts
    {  235, 5880,   60969,  8, }, // America/St_Lucia
    {  235,  855,   60977,  9, }, // America/St_Thomas
    {  235, 5913,   60986, 10, }, // America/St_Vincent
    {  235, 5523,    5609,  5, }, // Asia/Aqtau
    {  235, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  235, 3049,    4619,  9, }, // Pacific/Kanton
    {  238, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Vanuatu
    {  238, 2061,   60952,  9, }, // America/St_Johns
    {  238, 5863,   60961,  8, }, // America/St_Kitts
    {  238, 5880,   60969,  8, }, // America/St_Lucia
    {  238,  855,   60977,  9, }, // America/St_Thomas
    {  238, 5913,   60986, 10, }, // America/St_Vincent
    {  238, 5523,    5609,  5, }, // Asia/Aqtau
    {  238, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  238, 3049,    4619,  9, }, // Pacific/Kanton
    {  239, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/world
    {  239, 2061,   60952,  9, }, // America/St_Johns
    {  239, 5863,   60961,  8, }, // America/St_Kitts
    {  239, 5880,   60969,  8, }, // America/St_Lucia
    {  239,  855,   60977,  9, }, // America/St_Thomas
    {  239, 5913,   60986, 10, }, // America/St_Vincent
    {  239, 5523,    5609,  5, }, // Asia/Aqtau
    {  239, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  239, 3049,    4619,  9, }, // Pacific/Kanton
    {  240, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Zambia
    {  240, 2061,   60952,  9, }, // America/St_Johns
    {  240, 5863,   60961,  8, }, // America/St_Kitts
    {  240, 5880,   60969,  8, }, // America/St_Lucia
    {  240,  855,   60977,  9, }, // America/St_Thomas
    {  240, 5913,   60986, 10, }, // America/St_Vincent
    {  240, 5523,    5609,  5, }, // Asia/Aqtau
    {  240, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  240, 3049,    4619,  9, }, // Pacific/Kanton
    {  241, 8508,   60939, 13, }, // America/St_Barthelemy English/Latin/Zimbabwe
    {  241, 2061,   60952,  9, }, // America/St_Johns
    {  241, 5863,   60961,  8, }, // America/St_Kitts
    {  241, 5880,   60969,  8, }, // America/St_Lucia
    {  241,  855,   60977,  9, }, // America/St_Thomas
    {  241, 5913,   60986, 10, }, // America/St_Vincent
    {  241, 5523,    5609,  5, }, // Asia/Aqtau
    {  241, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  241, 3049,    4619,  9, }, // Pacific/Kanton
    {  242, 1259,     432, 11, }, // Asia/Ho_Chi_Minh English/Shavian/United Kingdom
    {  243,   14,       0,  6, }, // Africa/Asmara Erzya/Cyrillic/Russia
    {  243, 4435,       6,  8, }, // Africa/Sao_Tome
    {  243, 4252,      14,  8, }, // America/Asuncion
    {  243,  237,      22,  8, }, // America/Atikokan
    {  243, 7623,      30, 17, }, // America/Bahia_Banderas
    {  243, 7699,   36531,  5, }, // America/Boise
    {  243, 3868,      47,  6, }, // America/Cancun
    {  243, 2260,   61005,  6, }, // America/Chicago
    {  243, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  243, 5723,      66,  7, }, // America/Curacao
    {  243,  805,    1319,  6, }, // America/Denver
    {  243, 5770,    1410,  7, }, // America/Grenada
    {  243,  481,      73, 13, }, // America/Indiana/Knox
    {  243, 7961,      86, 16, }, // America/Indiana/Marengo
    {  243, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  243, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  243, 8038,     139, 14, }, // America/Indiana/Vevay
    {  243, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  243, 8086,     171, 16, }, // America/Indiana/Winamac
    {  243, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  243, 5932,     207, 22, }, // America/Lower_Princes
    {  243, 8220,     229,  6, }, // America/Merida
    {  243, 1850,     235, 19, }, // America/Noronha
    {  243, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  243, 8329,     274, 20, }, // America/North_Dakota/Center
    {  243, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  243,  393,     317,  4, }, // America/Nuuk
    {  243, 2356,    1901,  6, }, // America/Panama
    {  243, 6899,     321, 16, }, // America/Scoresbysund
    {  243, 8508,     337, 14, }, // America/St_Barthelemy
    {  243, 2061,     351, 10, }, // America/St_Johns
    {  243, 5863,     361,  9, }, // America/St_Kitts
    {  243, 5880,     370,  9, }, // America/St_Lucia
    {  243,  855,     379, 10, }, // America/St_Thomas
    {  243, 5913,     389, 11, }, // America/St_Vincent
    {  243, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  243, 1429,   61011, 11, }, // Arctic/Longyearbyen
    {  243, 3607,    2355,  6, }, // Asia/Baghdad
    {  243, 3653,    2368,  4, }, // Asia/Baku
    {  243, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  243, 2704,   52034,  7, }, // Asia/Hong_Kong
    {  243, 1185,     448,  9, }, // Asia/Kathmandu
    {  243,  992,     457,  7, }, // Asia/Kolkata
    {  243, 1211,     464,  5, }, // Asia/Macau
    {  243, 4229,   61022,  6, }, // Asia/Omsk
    {  243, 4408,    2732,  7, }, // Asia/Sakhalin
    {  243, 7245,    2739,  9, }, // Asia/Samarkand
    {  243, 3279,    2748,  4, }, // Asia/Seoul
    {  243, 1053,   28933,  6, }, // Asia/Shanghai
    {  243, 3263,   61028,  6, }, // Asia/Taipei
    {  243, 4781,    2785,  7, }, // Asia/Tashkent
    {  243, 2821,    2810,  5, }, // Asia/Tokyo
    {  243, 4572,   61034,  7, }, // Asia/Tomsk
    {  243, 1021,   61041, 10, }, // Asia/Ulaanbaatar
    {  243, 1235,     477,  6, }, // Asia/Yangon
    {  243, 5671,   35284,  7, }, // Atlantic/Bermuda
    {  243, 1395,     483,  5, }, // Atlantic/Faroe
    {  243, 8736,   37264,  7, }, // Atlantic/Madeira
    {  243, 2748,   52214,  9, }, // Atlantic/Reykjavik
    {  243, 6821,     488, 10, }, // Atlantic/St_Helena
    {  243, 6441,    3111,  9, }, // Europe/Amsterdam
    {  243, 6216,    3120,  7, }, // Europe/Andorra
    {  243, 2300,   61051,  5, }, // Europe/Athens
    {  243, 6336,    3139,  7, }, // Europe/Belgrade
    {  243, 4757,    3146,  6, }, // Europe/Berlin
    {  243, 6488,    3152, 10, }, // Europe/Bratislava
    {  243, 2167,   52296,  8, }, // Europe/Brussels
    {  243, 3928,    3170,  8, }, // Europe/Bucharest
    {  243, 3806,    3178,  8, }, // Europe/Budapest
    {  243, 6289,   52323, 10, }, // Europe/Copenhagen
    {  243, 2338,    3226,  6, }, // Europe/Dublin
    {  243, 6612,   52333,  9, }, // Europe/Helsinki
    {  243, 8929,    3287,  5, }, // Europe/Kirov
    {  243, 2527,    3292,  4, }, // Europe/Kyiv
    {  243, 3213,    3296,  8, }, // Europe/Lisbon
    {  243, 6506,   29432,  7, }, // Europe/Ljubljana
    {  243, 2501,    3313,  6, }, // Europe/London
    {  243, 6365,   29445, 10, }, // Europe/Luxembourg
    {  243, 4892,    3330,  6, }, // Europe/Madrid
    {  243, 6397,   29461,  6, }, // Europe/Malta
    {  243, 3693,   61056,  7, }, // Europe/Minsk
    {  243, 6410,    3355,  6, }, // Europe/Monaco
    {  243, 3537,   61063,  6, }, // Europe/Moscow
    {  243, 6458,    3367,  4, }, // Europe/Oslo
    {  243, 4284,    3371,  5, }, // Europe/Paris
    {  243, 6424,    3376,  9, }, // Europe/Podgorica
    {  243, 6275,    3385,  5, }, // Europe/Prague
    {  243, 8837,    3390,  4, }, // Europe/Riga
    {  243, 6324,    3394,  3, }, // Europe/Rome
    {  243, 4331,    3397,  6, }, // Europe/Samara
    {  243, 6470,    3403, 10, }, // Europe/San_Marino
    {  243, 6245,    3413,  7, }, // Europe/Sarajevo
    {  243, 4451,    3420,  7, }, // Europe/Saratov
    {  243, 8849,   52357, 11, }, // Europe/Simferopol
    {  243, 6383,   52368,  6, }, // Europe/Skopje
    {  243, 6599,   37418,  5, }, // Europe/Sofia
    {  243, 6523,   52374,  9, }, // Europe/Stockholm
    {  243, 8867,    3455,  6, }, // Europe/Tallinn
    {  243, 6202,    3461,  6, }, // Europe/Tirane
    {  243, 6352,    3476,  5, }, // Europe/Vaduz
    {  243, 6567,    3481,  7, }, // Europe/Vatican
    {  243, 6231,    3488,  4, }, // Europe/Vienna
    {  243, 8899,   52392,  7, }, // Europe/Vilnius
    {  243, 3190,    3508,  7, }, // Europe/Warsaw
    {  243, 6261,    3515,  6, }, // Europe/Zagreb
    {  243, 6540,   37442,  5, }, // Europe/Zurich
    {  243, 7211,     547,  7, }, // Indian/Reunion
    {  243, 3157,     554,  5, }, // Pacific/Chuuk
    {  243, 3096,     586,  7, }, // Pacific/Pohnpei
    {  244, 6788,   61069,  7, }, // Africa/Abidjan Esperanto/Latin/world
    {  244, 6760,   61076,  5, }, // Africa/Accra
    {  244, 5059,   61081,  9, }, // Africa/Addis_Ababa
    {  244, 7420,   61090,  6, }, // Africa/Algiers
    {  244,   14,   61096,  6, }, // Africa/Asmara
    {  244, 5257,   61102,  5, }, // Africa/Bangui
    {  244, 6746,   61107,  7, }, // Africa/Banjul
    {  244, 7435,   61114,  6, }, // Africa/Bissau
    {  244, 5287,   61120,  9, }, // Africa/Brazzaville
    {  244, 4922,   61129,  9, }, // Africa/Bujumbura
    {  244, 2320,    4690,  5, }, // Africa/Cairo
    {  244, 4117,   61138, 10, }, // Africa/Casablanca
    {  244, 7449,   61148,  5, }, // Africa/Ceuta
    {  244, 6773,   61153,  8, }, // Africa/Conakry
    {  244, 6840,   61161,  6, }, // Africa/Dakar
    {  244, 5130,   61167, 10, }, // Africa/Dar_es_Salaam
    {  244, 5043,   61177,  7, }, // Africa/Djibouti
    {  244, 5243,   61184,  6, }, // Africa/Douala
    {  244, 5166,   61190,  5, }, // Africa/El_Aaiun
    {  244, 6853,   61195,  8, }, // Africa/Freetown
    {  244, 4906,   61203,  8, }, // Africa/Gaborone
    {  244, 5015,   61211,  7, }, // Africa/Harare
    {  244, 4479,   61218, 12, }, // Africa/Johannesburg
    {  244, 4499,   61230,  5, }, // Africa/Juba
    {  244, 5151,   61235,  7, }, // Africa/Kampala
    {  244, 4524,   61242,  7, }, // Africa/Khartoum
    {  244, 4973,   61249,  6, }, // Africa/Kigali
    {  244, 5306,   61255,  7, }, // Africa/Kinshasa
    {  244, 4744,   61262,  6, }, // Africa/Lagos
    {  244, 5336,   61268,  9, }, // Africa/Libreville
    {  244, 6869,   61277,  5, }, // Africa/Lome
    {  244, 5211,   61282,  6, }, // Africa/Luanda
    {  244, 4939,   61288,  9, }, // Africa/Lubumbashi
    {  244, 5001,   61297,  6, }, // Africa/Lusaka
    {  244, 5197,   61303,  7, }, // Africa/Maseru
    {  244, 5182,   61310,  7, }, // Africa/Mbabane
    {  244, 5113,   61317,  8, }, // Africa/Mogadishu
    {  244, 7034,   61325,  8, }, // Africa/Monrovia
    {  244, 3853,   61333,  8, }, // Africa/Nairobi
    {  244, 5271,   61341,  8, }, // Africa/Ndjamena
    {  244, 5354,   61349,  7, }, // Africa/Niamey
    {  244, 6803,   61356,  8, }, // Africa/Nouakchott
    {  244, 6727,   61364,  9, }, // Africa/Ouagadougou
    {  244, 5225,   61373,  9, }, // Africa/Porto-Novo
    {  244, 4435,   61382,  8, }, // Africa/Sao_Tome
    {  244, 2866,   61390,  7, }, // Africa/Tripoli
    {  244, 6554,   61397,  6, }, // Africa/Tunis
    {  244, 4152,   61403,  8, }, // Africa/Windhoek
    {  244, 5607,   61411,  7, }, // America/Anguilla
    {  244, 5624,   61418,  7, }, // America/Antigua
    {  244, 4554,   26559,  9, }, // America/Araguaina
    {  244,  166,   61425,  7, }, // America/Argentina/Buenos_Aires
    {  244,  270,   61432,  7, }, // America/Argentina/Cordoba
    {  244, 7489,   44137, 12, }, // America/Argentina/Rio_Gallegos
    {  244, 7520,   61439,  9, }, // America/Argentina/Salta
    {  244, 7571,   20486,  7, }, // America/Argentina/Tucuman
    {  244, 5640,   61448,  5, }, // America/Aruba
    {  244, 4252,   61453,  9, }, // America/Asuncion
    {  244,  237,      22,  8, }, // America/Atikokan
    {  244, 3679,   61462,  5, }, // America/Bahia
    {  244, 7623,      30, 17, }, // America/Bahia_Banderas
    {  244, 5654,   61467,  7, }, // America/Barbados
    {  244, 7646,   61474,  6, }, // America/Belem
    {  244, 5407,   61480,  6, }, // America/Belize
    {  244, 4361,   61486,  6, }, // America/Bogota
    {  244, 7713,   61492, 14, }, // America/Cambridge_Bay
    {  244, 3868,   61506,  7, }, // America/Cancun
    {  244, 4694,   61513,  8, }, // America/Caracas
    {  244, 4345,   61521,  6, }, // America/Cayenne
    {  244, 5496,   61527,  8, }, // America/Cayman
    {  244, 2260,   61535,  6, }, // America/Chicago
    {  244, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  244, 5422,   61541,  9, }, // America/Costa_Rica
    {  244, 3791,   26574,  6, }, // America/Cuiaba
    {  244, 5723,   61550,  7, }, // America/Curacao
    {  244,  805,   61557,  7, }, // America/Denver
    {  244, 3465,   61564,  8, }, // America/Detroit
    {  244, 5739,   61572,  8, }, // America/Dominica
    {  244,  893,   61580,  9, }, // America/Edmonton
    {  244, 7869,   26580,  8, }, // America/Eirunepe
    {  244, 5441,   61589,  9, }, // America/El_Salvador
    {  244, 7906,   61598,  9, }, // America/Fortaleza
    {  244, 4612,   61607, 12, }, // America/Grand_Turk
    {  244, 5770,   61619,  7, }, // America/Grenada
    {  244, 5786,   61626,  9, }, // America/Guadeloupe
    {  244, 3760,   61635,  9, }, // America/Guatemala
    {  244, 6932,   61644,  6, }, // America/Guyana
    {  244, 1939,   61650,  9, }, // America/Halifax
    {  244, 2281,   61659,  6, }, // America/Havana
    {  244,  348,   61665, 21, }, // America/Indiana/Indianapolis
    {  244,  481,   61686, 14, }, // America/Indiana/Knox
    {  244, 7961,   61700, 17, }, // America/Indiana/Marengo
    {  244, 7985,   61717, 20, }, // America/Indiana/Petersburg
    {  244, 8012,   61737, 19, }, // America/Indiana/Tell_City
    {  244, 8038,   61756, 15, }, // America/Indiana/Vevay
    {  244, 8060,   61771, 19, }, // America/Indiana/Vincennes
    {  244, 8086,   61790, 17, }, // America/Indiana/Winamac
    {  244,  660,   61807,  8, }, // America/Iqaluit
    {  244, 2799,   61815,  7, }, // America/Jamaica
    {  244, 8125,   61822, 20, }, // America/Kentucky/Monticello
    {  244, 7169,   61842,  4, }, // America/Lima
    {  244, 3239,   61846, 11, }, // America/Los_Angeles
    {  244, 5932,     207, 22, }, // America/Lower_Princes
    {  244, 8153,   26588,  6, }, // America/Maceio
    {  244, 8168,   61857,  7, }, // America/Managua
    {  244, 1908,   61864,  7, }, // America/Manaus
    {  244, 5897,   61871,  8, }, // America/Marigot
    {  244, 5805,   61879,  9, }, // America/Martinique
    {  244, 2917,   26603,  8, }, // America/Mazatlan
    {  244, 8220,   61888,  6, }, // America/Merida
    {  244, 2949,   61894, 10, }, // America/Mexico_City
    {  244, 4391,   61904,  8, }, // America/Miquelon
    {  244, 8254,   61912,  8, }, // America/Moncton
    {  244, 8270,   61920,  9, }, // America/Monterrey
    {  244, 5824,   61929,  9, }, // America/Montserrat
    {  244, 5481,   61938,  6, }, // America/Nassau
    {  244, 2379,   61944,  8, }, // America/New_York
    {  244, 1850,     235, 19, }, // America/Noronha
    {  244, 8301,   61952, 20, }, // America/North_Dakota/Beulah
    {  244, 8329,   61972, 20, }, // America/North_Dakota/Center
    {  244, 8357,   61992, 23, }, // America/North_Dakota/New_Salem
    {  244,  393,   62015,  4, }, // America/Nuuk
    {  244, 2356,   62019,  6, }, // America/Panama
    {  244, 2973,   62025,  7, }, // America/Phoenix
    {  244, 3945,   62032, 11, }, // America/Port-au-Prince
    {  244, 5954,   62043, 10, }, // America/Port_of_Spain
    {  244, 5843,   62053, 10, }, // America/Puerto_Rico
    {  244, 8477,   26704,  8, }, // America/Santarem
    {  244, 6111,   62063, 14, }, // America/Santo_Domingo
    {  244, 1878,   62077,  8, }, // America/Sao_Paulo
    {  244, 6899,     321, 16, }, // America/Scoresbysund
    {  244, 8508,   62085, 17, }, // America/St_Barthelemy
    {  244, 2061,     351, 10, }, // America/St_Johns
    {  244, 5863,   62102, 17, }, // America/St_Kitts
    {  244, 5880,   62119, 12, }, // America/St_Lucia
    {  244,  855,   62131, 13, }, // America/St_Thomas
    {  244, 5913,   62144, 15, }, // America/St_Vincent
    {  244, 5461,   62159, 11, }, // America/Tegucigalpa
    {  244, 5756,   26796,  7, }, // America/Thule
    {  244, 2093,   62170,  9, }, // America/Vancouver
    {  244,  734,   62179,  8, }, // America/Winnipeg
    {  244, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  244, 8568,   62187, 14, }, // Antarctica/Macquarie
    {  244, 7295,   62201,  5, }, // Antarctica/Syowa
    {  244, 1429,   62206, 11, }, // Arctic/Longyearbyen
    {  244, 5570,   62217,  5, }, // Asia/Aden
    {  244, 5395,   62222,  6, }, // Asia/Almaty
    {  244, 3968,   62228,  5, }, // Asia/Amman
    {  244, 5511,    7600,  6, }, // Asia/Anadyr
    {  244, 5523,    5609,  5, }, // Asia/Aqtau
    {  244, 5368,    7606,  6, }, // Asia/Aqtobe
    {  244,  964,   62233,  8, }, // Asia/Ashgabat
    {  244, 8624,    7612,  6, }, // Asia/Atyrau
    {  244, 3607,   62241,  7, }, // Asia/Baghdad
    {  244, 5534,   62248,  7, }, // Asia/Bahrain
    {  244, 3653,   62255,  5, }, // Asia/Baku
    {  244, 4466,   62260,  7, }, // Asia/Bangkok
    {  244, 4086,   62267,  7, }, // Asia/Beirut
    {  244, 3778,   62274,  7, }, // Asia/Bishkek
    {  244, 5989,   62281,  7, }, // Asia/Brunei
    {  244, 4601,   62288,  4, }, // Asia/Chita
    {  244, 4511,    5637,  7, }, // Asia/Colombo
    {  244, 4540,   26821,  7, }, // Asia/Damascus
    {  244, 1093,     268,  4, }, // Asia/Dhaka
    {  244, 6192,   62292,  4, }, // Asia/Dili
    {  244, 3596,   62296,  9, }, // Asia/Dubai
    {  244, 6159,   62305,  8, }, // Asia/Dushanbe
    {  244, 8651,   62313,  5, }, // Asia/Gaza
    {  244, 4795,   62318,  7, }, // Asia/Hebron
    {  244, 1259,   62325,  8, }, // Asia/Ho_Chi_Minh
    {  244, 2704,   62333,  8, }, // Asia/Hong_Kong
    {  244, 4771,   62341,  4, }, // Asia/Hovd
    {  244, 4184,   62345,  8, }, // Asia/Irkutsk
    {  244, 7006,   62353,  7, }, // Asia/Jakarta
    {  244, 1290,   62360, 10, }, // Asia/Jerusalem
    {  244, 3560,   62370,  6, }, // Asia/Kabul
    {  244, 4316,   62376,  8, }, // Asia/Kamchatka
    {  244, 4239,   62384,  7, }, // Asia/Karachi
    {  244, 1185,   62391,  9, }, // Asia/Kathmandu
    {  244, 8661,   62400,  7, }, // Asia/Khandyga
    {  244,  992,   62407,  7, }, // Asia/Kolkata
    {  244, 4197,   62414, 12, }, // Asia/Krasnoyarsk
    {  244, 4859,   62426, 12, }, // Asia/Kuala_Lumpur
    {  244, 5547,   62438,  7, }, // Asia/Kuwait
    {  244, 1211,    7731,  5, }, // Asia/Macau
    {  244, 7182,   62445,  6, }, // Asia/Manila
    {  244, 6920,   62451,  7, }, // Asia/Muscat
    {  244, 2554,   62458,  7, }, // Asia/Nicosia
    {  244, 8675,   62465, 12, }, // Asia/Novokuznetsk
    {  244, 4135,   62477, 12, }, // Asia/Novosibirsk
    {  244, 4229,   62489,  5, }, // Asia/Omsk
    {  244, 7145,   62494,  5, }, // Asia/Oral
    {  244, 6961,   62499,  8, }, // Asia/Phnom_Penh
    {  244, 4214,   62507, 10, }, // Asia/Pyongyang
    {  244, 5559,   62517,  6, }, // Asia/Qatar
    {  244, 8708,    7770,  8, }, // Asia/Qostanay
    {  244, 4269,   62523,  9, }, // Asia/Qyzylorda
    {  244, 3584,   62532,  5, }, // Asia/Riyadh
    {  244, 4408,   62537,  8, }, // Asia/Sakhalin
    {  244, 7245,   62545, 10, }, // Asia/Samarkand
    {  244, 3279,   62555,  5, }, // Asia/Seoul
    {  244, 1053,   62560,  7, }, // Asia/Shanghai
    {  244, 3300,   62567,  9, }, // Asia/Singapore
    {  244, 4297,    7817, 13, }, // Asia/Srednekolymsk
    {  244, 3263,   62576,  6, }, // Asia/Taipei
    {  244, 4781,   62582,  8, }, // Asia/Tashkent
    {  244, 3915,   62590,  7, }, // Asia/Tbilisi
    {  244, 2772,   62597,  8, }, // Asia/Tehran
    {  244, 1317,   62605,  6, }, // Asia/Thimphu
    {  244, 2821,    4356,  5, }, // Asia/Tokyo
    {  244, 1021,   62611, 10, }, // Asia/Ulaanbaatar
    {  244, 1159,   62621,  7, }, // Asia/Urumqi
    {  244, 8722,   62628,  7, }, // Asia/Ust-Nera
    {  244, 6977,   62635,  9, }, // Asia/Vientiane
    {  244, 4710,   62644, 12, }, // Asia/Vladivostok
    {  244, 4828,   62656,  8, }, // Asia/Yakutsk
    {  244, 1235,   62664,  7, }, // Asia/Yangon
    {  244, 3883,   62671, 14, }, // Asia/Yekaterinburg
    {  244, 3747,   62685,  7, }, // Asia/Yerevan
    {  244, 3663,   62692,  6, }, // Atlantic/Azores
    {  244, 5671,   62698,  8, }, // Atlantic/Bermuda
    {  244, 6628,   62706,  8, }, // Atlantic/Canary
    {  244, 3727,   62714,  9, }, // Atlantic/Cape_Verde
    {  244, 1395,   62723,  6, }, // Atlantic/Faroe
    {  244, 8736,   62729,  7, }, // Atlantic/Madeira
    {  244, 2748,   62736, 10, }, // Atlantic/Reykjavik
    {  244, 7272,   62746, 11, }, // Atlantic/South_Georgia
    {  244, 6821,   62757, 13, }, // Atlantic/St_Helena
    {  244, 6644,   62770,  8, }, // Atlantic/Stanley
    {  244, 1670,   62778,  8, }, // Australia/Adelaide
    {  244, 1635,   62786,  8, }, // Australia/Brisbane
    {  244, 1583,   62794,  7, }, // Australia/Darwin
    {  244, 1516,   62801,  7, }, // Australia/Hobart
    {  244, 1727,   62808,  8, }, // Australia/Melbourne
    {  244, 1762,   62816,  5, }, // Australia/Perth
    {  244, 1463,   62821,  7, }, // Australia/Sydney
    {  244, 6441,   62828, 10, }, // Europe/Amsterdam
    {  244, 6216,   62838,  6, }, // Europe/Andorra
    {  244, 3620,   62844,  9, }, // Europe/Astrakhan
    {  244, 2300,   62853,  5, }, // Europe/Athens
    {  244, 6336,   62858,  8, }, // Europe/Belgrade
    {  244, 4757,   62866,  7, }, // Europe/Berlin
    {  244, 6488,   62873, 10, }, // Europe/Bratislava
    {  244, 2167,   62883,  7, }, // Europe/Brussels
    {  244, 3928,   62890,  9, }, // Europe/Bucharest
    {  244, 3806,   62899,  9, }, // Europe/Budapest
    {  244, 8772,   62908, 21, }, // Europe/Busingen
    {  244, 2583,   62929,  8, }, // Europe/Chisinau
    {  244, 6289,   62937,  9, }, // Europe/Copenhagen
    {  244, 2338,   62946,  7, }, // Europe/Dublin
    {  244, 6307,   62953, 10, }, // Europe/Gibraltar
    {  244, 8788,   62963,  9, }, // Europe/Guernsey
    {  244, 6612,   62972,  8, }, // Europe/Helsinki
    {  244, 8804,   62980, 10, }, // Europe/Isle_of_Man
    {  244, 1130,   62990,  9, }, // Europe/Istanbul
    {  244, 8823,   62999,  7, }, // Europe/Jersey
    {  244, 3979,   63006, 12, }, // Europe/Kaliningrad
    {  244, 2527,   63018,  5, }, // Europe/Kyiv
    {  244, 3213,   63023,  7, }, // Europe/Lisbon
    {  244, 6506,   63030,  9, }, // Europe/Ljubljana
    {  244, 2501,   63039,  7, }, // Europe/London
    {  244, 6365,   63046, 11, }, // Europe/Luxembourg
    {  244, 4892,   63057,  7, }, // Europe/Madrid
    {  244, 6397,   63064,  5, }, // Europe/Malta
    {  244, 3693,   63069,  6, }, // Europe/Minsk
    {  244, 6410,    6044,  6, }, // Europe/Monaco
    {  244, 3537,   63075,  6, }, // Europe/Moscow
    {  244, 4284,   63081,  6, }, // Europe/Paris
    {  244, 6424,   63087,  9, }, // Europe/Podgorica
    {  244, 6275,   63096,  5, }, // Europe/Prague
    {  244, 8837,   63101,  4, }, // Europe/Riga
    {  244, 6324,   63105,  4, }, // Europe/Rome
    {  244, 6470,   63109,  9, }, // Europe/San_Marino
    {  244, 8849,   63118, 11, }, // Europe/Simferopol
    {  244, 6383,   63129,  6, }, // Europe/Skopje
    {  244, 6599,   63135,  5, }, // Europe/Sofia
    {  244, 6523,   63140,  9, }, // Europe/Stockholm
    {  244, 8867,   63149,  6, }, // Europe/Tallinn
    {  244, 6202,   63155,  6, }, // Europe/Tirane
    {  244, 8882,   63161, 10, }, // Europe/Ulyanovsk
    {  244, 6352,   63171,  6, }, // Europe/Vaduz
    {  244, 6567,   27101,  8, }, // Europe/Vatican
    {  244, 6231,   63177,  5, }, // Europe/Vienna
    {  244, 8899,   63182,  5, }, // Europe/Vilnius
    {  244, 4727,   63187, 10, }, // Europe/Volgograd
    {  244, 3190,   63197,  8, }, // Europe/Warsaw
    {  244, 6261,   63205,  7, }, // Europe/Zagreb
    {  244, 6540,   63212,  6, }, // Europe/Zurich
    {  244, 6947,   63218,  7, }, // Indian/Chagos
    {  244, 6046,   63225, 15, }, // Indian/Christmas
    {  244, 6063,   63240, 12, }, // Indian/Cocos
    {  244, 5029,   63252,  7, }, // Indian/Comoro
    {  244, 6661,   63259, 10, }, // Indian/Kerguelen
    {  244, 7260,   27115,  4, }, // Indian/Mahe
    {  244, 7050,   63269,  8, }, // Indian/Maldives
    {  244, 4069,   63277,  8, }, // Indian/Mauritius
    {  244, 5098,   63285,  6, }, // Indian/Mayotte
    {  244, 7211,   63291,  6, }, // Indian/Reunion
    {  244, 4422,   63297,  4, }, // Pacific/Apia
    {  244,  932,   63301,  8, }, // Pacific/Auckland
    {  244, 3015,   63309,  7, }, // Pacific/Chatham
    {  244, 3157,   63316,  4, }, // Pacific/Chuuk
    {  244, 2237,   63320, 10, }, // Pacific/Easter
    {  244, 3902,   63330,  5, }, // Pacific/Fiji
    {  244, 7343,   63335,  9, }, // Pacific/Funafuti
    {  244, 6678,   63344,  9, }, // Pacific/Galapagos
    {  244, 6018,   63353,  5, }, // Pacific/Guam
    {  244, 4051,   63358, 13, }, // Pacific/Marquesas
    {  244, 8914,   63371, 13, }, // Pacific/Midway
    {  244, 7084,   63384,  5, }, // Pacific/Nauru
    {  244, 7132,   63389,  4, }, // Pacific/Niue
    {  244, 4168,   63393, 13, }, // Pacific/Norfolk
    {  244, 7098,   63406,  5, }, // Pacific/Noumea
    {  244, 3126,   63411,  8, }, // Pacific/Pago_Pago
    {  244, 7155,   63419,  6, }, // Pacific/Palau
    {  244, 7194,   63425, 15, }, // Pacific/Pitcairn
    {  244, 3096,    8429,  6, }, // Pacific/Pohnpei
    {  244, 6076,   63440,  9, }, // Pacific/Rarotonga
    {  244, 6031,   63449,  7, }, // Pacific/Saipan
    {  244, 7312,   63456,  7, }, // Pacific/Tahiti
    {  244, 6712,   63463,  6, }, // Pacific/Tarawa
    {  244, 7392,   63469, 10, }, // Pacific/Wake
    {  244, 7405,   63479,  6, }, // Pacific/Wallis
    {  245, 5059,    3902, 11, }, // Africa/Addis_Ababa Estonian/Latin/Estonia
    {  245, 7420,   63485,  6, }, // Africa/Algiers
    {  245,   14,       0,  6, }, // Africa/Asmara
    {  245, 2320,    4690,  5, }, // Africa/Cairo
    {  245, 5166,   63491,  8, }, // Africa/El_Aaiun
    {  245, 4524,   63499,  6, }, // Africa/Khartoum
    {  245, 6869,   20418,  4, }, // Africa/Lome
    {  245, 5113,   26511,  8, }, // Africa/Mogadishu
    {  245, 5271,   44128,  9, }, // Africa/Ndjamena
    {  245, 4435,       6,  8, }, // Africa/Sao_Tome
    {  245, 4554,   26559,  9, }, // America/Araguaina
    {  245,  270,   20479,  7, }, // America/Argentina/Cordoba
    {  245, 7489,   44137, 12, }, // America/Argentina/Rio_Gallegos
    {  245, 7571,   20486,  7, }, // America/Argentina/Tucuman
    {  245, 4252,      14,  8, }, // America/Asuncion
    {  245,  237,      22,  8, }, // America/Atikokan
    {  245, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  245, 7646,   35896,  5, }, // America/Belem
    {  245, 4361,   20507,  6, }, // America/Bogota
    {  245, 3868,      47,  6, }, // America/Cancun
    {  245, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  245, 3791,   26574,  6, }, // America/Cuiaba
    {  245, 5723,      66,  7, }, // America/Curacao
    {  245, 7869,   26580,  8, }, // America/Eirunepe
    {  245, 2281,   63505,  7, }, // America/Havana
    {  245,  481,      73, 13, }, // America/Indiana/Knox
    {  245, 7961,      86, 16, }, // America/Indiana/Marengo
    {  245, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  245, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  245, 8038,     139, 14, }, // America/Indiana/Vevay
    {  245, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  245, 8086,     171, 16, }, // America/Indiana/Winamac
    {  245, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  245, 5932,     207, 22, }, // America/Lower_Princes
    {  245, 8153,   26588,  6, }, // America/Maceio
    {  245, 2917,   26603,  8, }, // America/Mazatlan
    {  245, 8220,     229,  6, }, // America/Merida
    {  245, 2949,   57601,  6, }, // America/Mexico_City
    {  245, 1850,     235, 19, }, // America/Noronha
    {  245, 8301,   63512, 20, }, // America/North_Dakota/Beulah
    {  245, 8329,   63532, 20, }, // America/North_Dakota/Center
    {  245, 8357,   63552, 23, }, // America/North_Dakota/New_Salem
    {  245,  393,     317,  4, }, // America/Nuuk
    {  245, 8477,   26704,  8, }, // America/Santarem
    {  245, 1878,   26712,  9, }, // America/Sao_Paulo
    {  245, 6899,     321, 16, }, // America/Scoresbysund
    {  245, 8508,   26721, 16, }, // America/St_Barthelemy
    {  245, 2061,   26737, 12, }, // America/St_Johns
    {  245, 5863,   26749, 11, }, // America/St_Kitts
    {  245, 5880,   35964, 11, }, // America/St_Lucia
    {  245,  855,   44293, 12, }, // America/St_Thomas
    {  245, 5913,   26783, 13, }, // America/St_Vincent
    {  245, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  245, 5395,   63575,  6, }, // Asia/Almaty
    {  245, 5511,   63581,  6, }, // Asia/Anadyr
    {  245, 5523,    5609,  5, }, // Asia/Aqtau
    {  245, 5368,   63587,  6, }, // Asia/Aqtobe
    {  245,  964,   44322,  7, }, // Asia/Ashgabat
    {  245, 8624,   63593,  6, }, // Asia/Atyrau
    {  245, 3607,    4162,  6, }, // Asia/Baghdad
    {  245, 5534,    4168,  7, }, // Asia/Bahrain
    {  245, 3653,   63599,  5, }, // Asia/Baku
    {  245, 3778,   33455,  6, }, // Asia/Bishkek
    {  245, 4601,   63604,  5, }, // Asia/Chita
    {  245, 4540,    4201,  8, }, // Asia/Damascus
    {  245, 6159,   33468,  7, }, // Asia/Dushanbe
    {  245, 1259,   63609, 11, }, // Asia/Ho_Chi_Minh
    {  245, 2704,    4231,  8, }, // Asia/Hong_Kong
    {  245, 1290,   63620, 11, }, // Asia/Jerusalem
    {  245, 4316,   63631,  9, }, // Asia/Kamchatka
    {  245, 1185,    5711,  8, }, // Asia/Kathmandu
    {  245, 8661,   63640,  7, }, // Asia/Khandyga
    {  245,  992,     457,  7, }, // Asia/Kolkata
    {  245, 4197,    4288, 11, }, // Asia/Krasnoyarsk
    {  245, 5547,   63647,  6, }, // Asia/Kuwait
    {  245, 1211,    4321,  5, }, // Asia/Macau
    {  245, 6920,   63653,  6, }, // Asia/Muscat
    {  245, 2554,    5738,  7, }, // Asia/Nicosia
    {  245, 5559,    4332,  5, }, // Asia/Qatar
    {  245, 8708,    7770,  8, }, // Asia/Qostanay
    {  245, 4269,   63659,  9, }, // Asia/Qyzylorda
    {  245, 3584,   63668,  8, }, // Asia/Riyadh
    {  245, 4408,   63676,  8, }, // Asia/Sakhalin
    {  245, 3279,   58092,  4, }, // Asia/Seoul
    {  245, 3300,   20900,  8, }, // Asia/Singapore
    {  245, 4297,   63684, 13, }, // Asia/Srednekolymsk
    {  245, 4781,   33589,  7, }, // Asia/Tashkent
    {  245, 3915,   63697,  8, }, // Asia/Tbilisi
    {  245, 2772,    7842,  7, }, // Asia/Tehran
    {  245, 2821,   63705,  5, }, // Asia/Tokyo
    {  245, 1159,   26933,  6, }, // Asia/Urumqi
    {  245, 4828,    7873,  7, }, // Asia/Yakutsk
    {  245, 1235,     477,  6, }, // Asia/Yangon
    {  245, 3883,    4380, 13, }, // Asia/Yekaterinburg
    {  245, 3747,    7898,  7, }, // Asia/Yerevan
    {  245, 3663,   63710,  8, }, // Atlantic/Azores
    {  245, 6628,   63718, 14, }, // Atlantic/Canary
    {  245, 3727,   63732, 15, }, // Atlantic/Cape_Verde
    {  245, 1395,   63747, 12, }, // Atlantic/Faroe
    {  245, 2748,   36176,  9, }, // Atlantic/Reykjavik
    {  245, 7272,   63759, 13, }, // Atlantic/South_Georgia
    {  245, 6821,   36198, 12, }, // Atlantic/St_Helena
    {  245, 3620,   33644,  8, }, // Europe/Astrakhan
    {  245, 2300,   63772,  6, }, // Europe/Athens
    {  245, 6336,   27015,  7, }, // Europe/Belgrade
    {  245, 4757,   63778,  7, }, // Europe/Berlin
    {  245, 2167,   22766,  7, }, // Europe/Brussels
    {  245, 3928,   27029,  8, }, // Europe/Bucharest
    {  245, 8772,   27037,  8, }, // Europe/Busingen
    {  245, 2583,   63785,  8, }, // Europe/Chisinau
    {  245, 6289,   63793, 11, }, // Europe/Copenhagen
    {  245, 6612,   63804,  8, }, // Europe/Helsinki
    {  245, 8804,   63812,  9, }, // Europe/Isle_of_Man
    {  245, 1130,   22825,  8, }, // Europe/Istanbul
    {  245, 2527,   63821,  5, }, // Europe/Kyiv
    {  245, 3213,    4499,  8, }, // Europe/Lisbon
    {  245, 6582,   63826, 13, }, // Europe/Mariehamn
    {  245, 3537,   22880,  6, }, // Europe/Moscow
    {  245, 4284,   63839,  6, }, // Europe/Paris
    {  245, 6275,   36285,  5, }, // Europe/Prague
    {  245, 8837,   63845,  4, }, // Europe/Riga
    {  245, 6324,   63849,  5, }, // Europe/Rome
    {  245, 6202,   21213,  6, }, // Europe/Tirane
    {  245, 8882,    8261,  9, }, // Europe/Ulyanovsk
    {  245, 6567,    6096,  7, }, // Europe/Vatican
    {  245, 6231,   63854,  4, }, // Europe/Vienna
    {  245, 3190,   63858,  8, }, // Europe/Warsaw
    {  245, 6540,    4577,  6, }, // Europe/Zurich
    {  245, 6046,   63866,  9, }, // Indian/Christmas
    {  245, 6063,   63875, 12, }, // Indian/Cocos
    {  245, 7260,   27115,  4, }, // Indian/Mahe
    {  245, 7050,   63887,  9, }, // Indian/Maldives
    {  245, 7211,     547,  7, }, // Indian/Reunion
    {  245, 3157,     554,  5, }, // Pacific/Chuuk
    {  245, 2237,   63896, 13, }, // Pacific/Easter
    {  245, 3902,   33836,  5, }, // Pacific/Fiji
    {  245, 6678,   21277,  9, }, // Pacific/Galapagos
    {  245, 2723,    4611,  8, }, // Pacific/Honolulu
    {  245, 3049,    4619,  9, }, // Pacific/Kanton
    {  245, 4051,   63909, 12, }, // Pacific/Marquesas
    {  245, 7098,    4628,  6, }, // Pacific/Noumea
    {  245, 7155,   63921,  5, }, // Pacific/Palau
    {  245, 3096,     586,  7, }, // Pacific/Pohnpei
    {  246,   14,       0,  6, }, // Africa/Asmara Ewe/Latin/Ghana
    {  246, 4435,       6,  8, }, // Africa/Sao_Tome
    {  246, 4252,      14,  8, }, // America/Asuncion
    {  246,  237,      22,  8, }, // America/Atikokan
    {  246, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  246, 3868,   33314,  6, }, // America/Cancun
    {  246, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  246, 5723,      66,  7, }, // America/Curacao
    {  246,  481,      73, 13, }, // America/Indiana/Knox
    {  246, 7961,      86, 16, }, // America/Indiana/Marengo
    {  246, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  246, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  246, 8038,     139, 14, }, // America/Indiana/Vevay
    {  246, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  246, 8086,     171, 16, }, // America/Indiana/Winamac
    {  246, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  246, 5932,     207, 22, }, // America/Lower_Princes
    {  246, 8220,    7126,  6, }, // America/Merida
    {  246, 1850,     235, 19, }, // America/Noronha
    {  246, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  246, 8329,     274, 20, }, // America/North_Dakota/Center
    {  246, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  246,  393,     317,  4, }, // America/Nuuk
    {  246, 6899,     321, 16, }, // America/Scoresbysund
    {  246, 8508,     337, 14, }, // America/St_Barthelemy
    {  246, 2061,     351, 10, }, // America/St_Johns
    {  246, 5863,     361,  9, }, // America/St_Kitts
    {  246, 5880,     370,  9, }, // America/St_Lucia
    {  246,  855,     379, 10, }, // America/St_Thomas
    {  246, 5913,     389, 11, }, // America/St_Vincent
    {  246, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  246, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  246, 1185,   63926, 24, }, // Asia/Kathmandu
    {  246,  992,     457,  7, }, // Asia/Kolkata
    {  246, 1211,    4321,  5, }, // Asia/Macau
    {  246, 1235,     477,  6, }, // Asia/Yangon
    {  246, 1395,     483,  5, }, // Atlantic/Faroe
    {  246, 6821,     488, 10, }, // Atlantic/St_Helena
    {  246, 2527,    8148,  4, }, // Europe/Kyiv
    {  246, 7211,     547,  7, }, // Indian/Reunion
    {  246, 3157,     554,  5, }, // Pacific/Chuuk
    {  246, 2723,    4611,  8, }, // Pacific/Honolulu
    {  246, 3049,    4619,  9, }, // Pacific/Kanton
    {  246, 3096,     586,  7, }, // Pacific/Pohnpei
    {  248,   14,       0,  6, }, // Africa/Asmara Ewondo/Latin/Cameroon
    {  248, 4435,       6,  8, }, // Africa/Sao_Tome
    {  248, 4252,      14,  8, }, // America/Asuncion
    {  248,  237,      22,  8, }, // America/Atikokan
    {  248, 7623,      30, 17, }, // America/Bahia_Banderas
    {  248, 3868,      47,  6, }, // America/Cancun
    {  248, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  248, 5723,      66,  7, }, // America/Curacao
    {  248,  481,      73, 13, }, // America/Indiana/Knox
    {  248, 7961,      86, 16, }, // America/Indiana/Marengo
    {  248, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  248, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  248, 8038,     139, 14, }, // America/Indiana/Vevay
    {  248, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  248, 8086,     171, 16, }, // America/Indiana/Winamac
    {  248, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  248, 5932,     207, 22, }, // America/Lower_Princes
    {  248, 8220,     229,  6, }, // America/Merida
    {  248, 1850,     235, 19, }, // America/Noronha
    {  248, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  248, 8329,     274, 20, }, // America/North_Dakota/Center
    {  248, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  248,  393,     317,  4, }, // America/Nuuk
    {  248, 6899,     321, 16, }, // America/Scoresbysund
    {  248, 8508,     337, 14, }, // America/St_Barthelemy
    {  248, 2061,     351, 10, }, // America/St_Johns
    {  248, 5863,     361,  9, }, // America/St_Kitts
    {  248, 5880,     370,  9, }, // America/St_Lucia
    {  248,  855,     379, 10, }, // America/St_Thomas
    {  248, 5913,     389, 11, }, // America/St_Vincent
    {  248, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  248, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  248, 1185,     448,  9, }, // Asia/Kathmandu
    {  248,  992,     457,  7, }, // Asia/Kolkata
    {  248, 1211,     464,  5, }, // Asia/Macau
    {  248, 1235,     477,  6, }, // Asia/Yangon
    {  248, 1395,     483,  5, }, // Atlantic/Faroe
    {  248, 6821,     488, 10, }, // Atlantic/St_Helena
    {  248, 2527,     514,  4, }, // Europe/Kyiv
    {  248, 7211,     547,  7, }, // Indian/Reunion
    {  248, 3157,     554,  5, }, // Pacific/Chuuk
    {  248, 3096,     586,  7, }, // Pacific/Pohnpei
    {  249,   14,       0,  6, }, // Africa/Asmara Faroese/Latin/Faroe Islands
    {  249, 5043,   26492,  7, }, // Africa/Djibouti
    {  249, 4435,       6,  8, }, // Africa/Sao_Tome
    {  249, 4252,      14,  8, }, // America/Asuncion
    {  249,  237,      22,  8, }, // America/Atikokan
    {  249, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  249, 5407,   63950,  5, }, // America/Belize
    {  249, 3868,   33314,  6, }, // America/Cancun
    {  249, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  249, 5422,   21621, 10, }, // America/Costa_Rica
    {  249, 5723,      66,  7, }, // America/Curacao
    {  249,  481,      73, 13, }, // America/Indiana/Knox
    {  249, 7961,      86, 16, }, // America/Indiana/Marengo
    {  249, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  249, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  249, 8038,     139, 14, }, // America/Indiana/Vevay
    {  249, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  249, 8086,     171, 16, }, // America/Indiana/Winamac
    {  249, 2799,    3995,  7, }, // America/Jamaica
    {  249, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  249, 5932,     207, 22, }, // America/Lower_Princes
    {  249, 8220,    7126,  6, }, // America/Merida
    {  249, 1850,     235, 19, }, // America/Noronha
    {  249, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  249, 8329,     274, 20, }, // America/North_Dakota/Center
    {  249, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  249,  393,     317,  4, }, // America/Nuuk
    {  249, 5843,   22058, 11, }, // America/Puerto_Rico
    {  249, 6899,     321, 16, }, // America/Scoresbysund
    {  249, 8508,     337, 14, }, // America/St_Barthelemy
    {  249, 2061,     351, 10, }, // America/St_Johns
    {  249, 5863,     361,  9, }, // America/St_Kitts
    {  249, 5880,     370,  9, }, // America/St_Lucia
    {  249,  855,     379, 10, }, // America/St_Thomas
    {  249, 5913,     389, 11, }, // America/St_Vincent
    {  249, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  249, 5534,   63955,  6, }, // Asia/Bahrain
    {  249, 8651,   63961,  4, }, // Asia/Gaza
    {  249, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  249, 1185,     448,  9, }, // Asia/Kathmandu
    {  249,  992,     457,  7, }, // Asia/Kolkata
    {  249, 5547,   63965,  6, }, // Asia/Kuwait
    {  249, 1211,    7731,  5, }, // Asia/Macau
    {  249, 8708,     469,  8, }, // Asia/Qostanay
    {  249, 3300,    7809,  8, }, // Asia/Singapore
    {  249, 2772,    7842,  7, }, // Asia/Tehran
    {  249, 1235,   58618,  7, }, // Asia/Yangon
    {  249, 3663,   63971,  9, }, // Atlantic/Azores
    {  249, 3727,   63980, 16, }, // Atlantic/Cape_Verde
    {  249, 1395,   63996,  7, }, // Atlantic/Faroe
    {  249, 2748,   36176,  9, }, // Atlantic/Reykjavik
    {  249, 7272,   64003, 19, }, // Atlantic/South_Georgia
    {  249, 6821,     488, 10, }, // Atlantic/St_Helena
    {  249, 2300,    5911,  4, }, // Europe/Athens
    {  249, 6336,    8060,  7, }, // Europe/Belgrade
    {  249, 2167,   57661,  9, }, // Europe/Brussels
    {  249, 3928,   27029,  8, }, // Europe/Bucharest
    {  249, 6289,   64022, 13, }, // Europe/Copenhagen
    {  249, 2527,    8148,  4, }, // Europe/Kyiv
    {  249, 3213,    4499,  8, }, // Europe/Lisbon
    {  249, 6365,   64035, 10, }, // Europe/Luxembourg
    {  249, 6410,    6044,  6, }, // Europe/Monaco
    {  249, 3537,   22880,  6, }, // Europe/Moscow
    {  249, 6275,    8201,  4, }, // Europe/Prague
    {  249, 6324,    6064,  3, }, // Europe/Rome
    {  249, 6523,   64045,  9, }, // Europe/Stockholm
    {  249, 6567,   64054,  9, }, // Europe/Vatican
    {  249, 6231,   57515,  4, }, // Europe/Vienna
    {  249, 3190,   64063,  8, }, // Europe/Warsaw
    {  249, 6540,    4577,  6, }, // Europe/Zurich
    {  249, 7050,   64071, 13, }, // Indian/Maldives
    {  249, 4069,   64084,  8, }, // Indian/Mauritius
    {  249, 7211,     547,  7, }, // Indian/Reunion
    {  249, 3157,     554,  5, }, // Pacific/Chuuk
    {  249, 2723,    4611,  8, }, // Pacific/Honolulu
    {  249, 3049,    4619,  9, }, // Pacific/Kanton
    {  249, 3096,     586,  7, }, // Pacific/Pohnpei
    {  251,   14,       0,  6, }, // Africa/Asmara Filipino/Latin/Philippines
    {  251, 4435,       6,  8, }, // Africa/Sao_Tome
    {  251, 4252,      14,  8, }, // America/Asuncion
    {  251,  237,      22,  8, }, // America/Atikokan
    {  251, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  251, 3868,   33314,  6, }, // America/Cancun
    {  251, 7774,   64092, 17, }, // America/Ciudad_Juarez
    {  251, 5723,      66,  7, }, // America/Curacao
    {  251,  481,      73, 13, }, // America/Indiana/Knox
    {  251, 7961,      86, 16, }, // America/Indiana/Marengo
    {  251, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  251, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  251, 8038,     139, 14, }, // America/Indiana/Vevay
    {  251, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  251, 8086,     171, 16, }, // America/Indiana/Winamac
    {  251, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  251, 5932,     207, 22, }, // America/Lower_Princes
    {  251, 8220,    7126,  6, }, // America/Merida
    {  251, 2949,   64109, 17, }, // America/Mexico_City
    {  251, 1850,     235, 19, }, // America/Noronha
    {  251, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  251, 8329,     274, 20, }, // America/North_Dakota/Center
    {  251, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  251,  393,     317,  4, }, // America/Nuuk
    {  251, 5954,   64126, 18, }, // America/Port_of_Spain
    {  251, 8424,   64144, 25, }, // America/Rankin_Inlet
    {  251, 6899,     321, 16, }, // America/Scoresbysund
    {  251, 8508,     337, 14, }, // America/St_Barthelemy
    {  251, 2061,     351, 10, }, // America/St_Johns
    {  251, 5863,     361,  9, }, // America/St_Kitts
    {  251, 5880,     370,  9, }, // America/St_Lucia
    {  251,  855,     379, 10, }, // America/St_Thomas
    {  251, 5913,     389, 11, }, // America/St_Vincent
    {  251, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  251, 1259,   64169, 22, }, // Asia/Ho_Chi_Minh
    {  251, 1185,     448,  9, }, // Asia/Kathmandu
    {  251,  992,     457,  7, }, // Asia/Kolkata
    {  251, 1211,    4321,  5, }, // Asia/Macau
    {  251, 8708,     469,  8, }, // Asia/Qostanay
    {  251, 1235,     477,  6, }, // Asia/Yangon
    {  251, 1395,     483,  5, }, // Atlantic/Faroe
    {  251, 6821,     488, 10, }, // Atlantic/St_Helena
    {  251, 2527,    8148,  4, }, // Europe/Kyiv
    {  251, 7211,     547,  7, }, // Indian/Reunion
    {  251, 3157,     554,  5, }, // Pacific/Chuuk
    {  251, 2723,    4611,  8, }, // Pacific/Honolulu
    {  251, 3049,    4619,  9, }, // Pacific/Kanton
    {  251, 3096,     586,  7, }, // Pacific/Pohnpei
    {  252, 5059,    3902, 11, }, // Africa/Addis_Ababa Finnish/Latin/Finland
    {  252, 7420,   44088,  5, }, // Africa/Algiers
    {  252,   14,       0,  6, }, // Africa/Asmara
    {  252, 2320,    4690,  5, }, // Africa/Cairo
    {  252, 5166,   63491,  8, }, // Africa/El_Aaiun
    {  252, 4524,   26504,  7, }, // Africa/Khartoum
    {  252, 6869,   20418,  4, }, // Africa/Lome
    {  252, 5271,   44128,  9, }, // Africa/Ndjamena
    {  252, 4435,       6,  8, }, // Africa/Sao_Tome
    {  252, 4554,   26559,  9, }, // America/Araguaina
    {  252,  270,   20479,  7, }, // America/Argentina/Cordoba
    {  252, 7571,   20486,  7, }, // America/Argentina/Tucuman
    {  252, 4252,      14,  8, }, // America/Asuncion
    {  252,  237,      22,  8, }, // America/Atikokan
    {  252, 7623,      30, 17, }, // America/Bahia_Banderas
    {  252, 4361,   20507,  6, }, // America/Bogota
    {  252, 3868,      47,  6, }, // America/Cancun
    {  252, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  252, 3791,   26574,  6, }, // America/Cuiaba
    {  252, 5723,      66,  7, }, // America/Curacao
    {  252, 7869,   26580,  8, }, // America/Eirunepe
    {  252, 2281,   63505,  7, }, // America/Havana
    {  252,  481,      73, 13, }, // America/Indiana/Knox
    {  252, 7961,      86, 16, }, // America/Indiana/Marengo
    {  252, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  252, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  252, 8038,     139, 14, }, // America/Indiana/Vevay
    {  252, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  252, 8086,     171, 16, }, // America/Indiana/Winamac
    {  252, 2799,    3995,  7, }, // America/Jamaica
    {  252, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  252, 5932,     207, 22, }, // America/Lower_Princes
    {  252, 8153,   26588,  6, }, // America/Maceio
    {  252, 2917,   26603,  8, }, // America/Mazatlan
    {  252, 8220,     229,  6, }, // America/Merida
    {  252, 2949,   57591, 16, }, // America/Mexico_City
    {  252, 1850,     235, 19, }, // America/Noronha
    {  252, 8301,   64191, 22, }, // America/North_Dakota/Beulah
    {  252, 8329,   64213, 22, }, // America/North_Dakota/Center
    {  252, 8357,   64235, 25, }, // America/North_Dakota/New_Salem
    {  252,  393,   35928,  7, }, // America/Nuuk
    {  252, 8477,   26704,  8, }, // America/Santarem
    {  252, 2201,   64260, 17, }, // America/Santiago
    {  252, 1878,   26712,  9, }, // America/Sao_Paulo
    {  252, 6899,   44281, 12, }, // America/Scoresbysund
    {  252, 8508,   26721, 16, }, // America/St_Barthelemy
    {  252, 2061,     351, 10, }, // America/St_Johns
    {  252, 5863,   26749, 11, }, // America/St_Kitts
    {  252, 5880,   35964, 11, }, // America/St_Lucia
    {  252,  855,   44293, 12, }, // America/St_Thomas
    {  252, 5913,   26783, 13, }, // America/St_Vincent
    {  252, 5756,   26796,  7, }, // America/Thule
    {  252, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  252, 8568,   64277, 15, }, // Antarctica/Macquarie
    {  252, 5523,   64292,  5, }, // Asia/Aqtau
    {  252, 5368,   57205,  6, }, // Asia/Aqtobe
    {  252,  964,   57615,  7, }, // Asia/Ashgabat
    {  252, 8624,   58781,  6, }, // Asia/Atyrau
    {  252, 3607,    4162,  6, }, // Asia/Baghdad
    {  252, 3778,   33455,  6, }, // Asia/Bishkek
    {  252, 4601,   63604,  5, }, // Asia/Chita
    {  252, 4540,   64297,  8, }, // Asia/Damascus
    {  252, 6159,   33468,  7, }, // Asia/Dushanbe
    {  252, 1259,   64305, 22, }, // Asia/Ho_Chi_Minh
    {  252, 2704,    4231,  8, }, // Asia/Hong_Kong
    {  252, 4316,   63631,  9, }, // Asia/Kamchatka
    {  252, 1185,     448,  9, }, // Asia/Kathmandu
    {  252, 8661,   64327,  7, }, // Asia/Khandyga
    {  252,  992,   64334,  8, }, // Asia/Kolkata
    {  252, 4197,    4288, 11, }, // Asia/Krasnoyarsk
    {  252, 1211,     464,  5, }, // Asia/Macau
    {  252, 6920,   36067,  6, }, // Asia/Muscat
    {  252, 2554,    5738,  7, }, // Asia/Nicosia
    {  252, 7145,   58053,  6, }, // Asia/Oral
    {  252, 4214,   33562,  9, }, // Asia/Pyongyang
    {  252, 8708,   64342,  8, }, // Asia/Qostanay
    {  252, 4269,   22533,  9, }, // Asia/Qyzylorda
    {  252, 3584,    7787,  4, }, // Asia/Riyadh
    {  252, 4408,   33576,  7, }, // Asia/Sakhalin
    {  252, 3279,   58092,  4, }, // Asia/Seoul
    {  252, 4781,   33589,  7, }, // Asia/Tashkent
    {  252, 2772,    7842,  7, }, // Asia/Tehran
    {  252, 2821,    4356,  5, }, // Asia/Tokyo
    {  252, 1021,   26923, 10, }, // Asia/Ulaanbaatar
    {  252, 1159,   26933,  6, }, // Asia/Urumqi
    {  252, 4828,    7873,  7, }, // Asia/Yakutsk
    {  252, 1235,     477,  6, }, // Asia/Yangon
    {  252, 3883,    4380, 13, }, // Asia/Yekaterinburg
    {  252, 3747,    7898,  7, }, // Asia/Yerevan
    {  252, 3663,   64350,  6, }, // Atlantic/Azores
    {  252, 6628,   64356, 14, }, // Atlantic/Canary
    {  252, 3727,   57380,  9, }, // Atlantic/Cape_Verde
    {  252, 1395,   64370,  9, }, // Atlantic/Faroe
    {  252, 2748,   36176,  9, }, // Atlantic/Reykjavik
    {  252, 7272,   64379, 13, }, // Atlantic/South_Georgia
    {  252, 6821,   36198, 12, }, // Atlantic/St_Helena
    {  252, 3620,   33644,  8, }, // Europe/Astrakhan
    {  252, 2300,   63772,  6, }, // Europe/Athens
    {  252, 6336,   27015,  7, }, // Europe/Belgrade
    {  252, 4757,   64392,  8, }, // Europe/Berlin
    {  252, 2167,   64400,  7, }, // Europe/Brussels
    {  252, 3928,   27029,  8, }, // Europe/Bucharest
    {  252, 8772,   27037,  8, }, // Europe/Busingen
    {  252, 2583,   64407,  8, }, // Europe/Chisinau
    {  252, 6289,   64415, 12, }, // Europe/Copenhagen
    {  252, 8804,   64427,  8, }, // Europe/Isle_of_Man
    {  252, 2527,   64435,  5, }, // Europe/Kyiv
    {  252, 3213,    4499,  8, }, // Europe/Lisbon
    {  252, 2501,   64440,  6, }, // Europe/London
    {  252, 6365,    4513,  9, }, // Europe/Luxembourg
    {  252, 6582,   63826, 13, }, // Europe/Mariehamn
    {  252, 3537,   64446,  7, }, // Europe/Moscow
    {  252, 4284,   64453,  7, }, // Europe/Paris
    {  252, 6275,   36285,  5, }, // Europe/Prague
    {  252, 8837,   64460,  5, }, // Europe/Riga
    {  252, 6324,   63849,  5, }, // Europe/Rome
    {  252, 6523,   64465,  8, }, // Europe/Stockholm
    {  252, 8867,   64473,  8, }, // Europe/Tallinn
    {  252, 6202,   21213,  6, }, // Europe/Tirane
    {  252, 8882,    8261,  9, }, // Europe/Ulyanovsk
    {  252, 6567,   64481,  9, }, // Europe/Vatican
    {  252, 6231,   57515,  4, }, // Europe/Vienna
    {  252, 8899,   64490,  5, }, // Europe/Vilnius
    {  252, 3190,   64495,  7, }, // Europe/Warsaw
    {  252, 6540,    4577,  6, }, // Europe/Zurich
    {  252, 6046,   64502, 10, }, // Indian/Christmas
    {  252, 6063,   64512, 12, }, // Indian/Cocos
    {  252, 5029,   64524,  7, }, // Indian/Comoro
    {  252, 6661,   64531, 15, }, // Indian/Kerguelen
    {  252, 7260,   27115,  4, }, // Indian/Mahe
    {  252, 7050,   64546, 10, }, // Indian/Maldives
    {  252, 7211,     547,  7, }, // Indian/Reunion
    {  252, 3015,   64556, 13, }, // Pacific/Chatham
    {  252, 3157,     554,  5, }, // Pacific/Chuuk
    {  252, 2237,   64569, 13, }, // Pacific/Easter
    {  252, 3902,   33836,  5, }, // Pacific/Fiji
    {  252, 6696,   64582, 13, }, // Pacific/Gambier
    {  252, 2723,    4611,  8, }, // Pacific/Honolulu
    {  252, 3049,    4619,  9, }, // Pacific/Kanton
    {  252, 4051,   64595, 15, }, // Pacific/Marquesas
    {  252, 8914,   64610, 12, }, // Pacific/Midway
    {  252, 7098,    4628,  6, }, // Pacific/Noumea
    {  252, 3096,     586,  7, }, // Pacific/Pohnpei
    {  253, 5059,   64622, 11, }, // Africa/Addis_Ababa French/Latin/France
    {  253, 7420,   44088,  5, }, // Africa/Algiers
    {  253,   14,       0,  6, }, // Africa/Asmara
    {  253, 2320,   64633,  8, }, // Africa/Cairo
    {  253, 5166,   64641,  8, }, // Africa/El_Aaiun
    {  253, 6869,   20418,  4, }, // Africa/Lome
    {  253, 5113,   44110, 10, }, // Africa/Mogadishu
    {  253, 5271,   44128,  9, }, // Africa/Ndjamena
    {  253, 4435,       6,  8, }, // Africa/Sao_Tome
    {  253, 2866,   64649, 15, }, // Africa/Tripoli
    {  253, 4554,   26559,  9, }, // America/Araguaina
    {  253,  270,   20479,  7, }, // America/Argentina/Cordoba
    {  253, 7489,   44137, 12, }, // America/Argentina/Rio_Gallegos
    {  253, 7571,   20486,  7, }, // America/Argentina/Tucuman
    {  253, 7597,   64664,  7, }, // America/Argentina/Ushuaia
    {  253, 4252,      14,  8, }, // America/Asuncion
    {  253,  237,      22,  8, }, // America/Atikokan
    {  253, 7623,   64671, 17, }, // America/Bahia_Banderas
    {  253, 5654,   64688, 10, }, // America/Barbados
    {  253, 7646,   35896,  5, }, // America/Belem
    {  253, 3868,      47,  6, }, // America/Cancun
    {  253, 5496,   64698,  7, }, // America/Cayman
    {  253, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  253, 3791,   26574,  6, }, // America/Cuiaba
    {  253, 5723,      66,  7, }, // America/Curacao
    {  253, 3465,   64705,  7, }, // America/Detroit
    {  253, 5739,   64712,  9, }, // America/Dominica
    {  253, 7869,   26580,  8, }, // America/Eirunepe
    {  253, 5770,   64721,  7, }, // America/Grenada
    {  253, 2281,   64728,  9, }, // America/Havana
    {  253,  481,   64737, 14, }, // America/Indiana/Knox
    {  253, 7961,   64751, 17, }, // America/Indiana/Marengo
    {  253, 7985,   64768, 20, }, // America/Indiana/Petersburg
    {  253, 8012,   64788, 19, }, // America/Indiana/Tell_City
    {  253, 8038,   64807, 15, }, // America/Indiana/Vevay
    {  253, 8060,   64822, 19, }, // America/Indiana/Vincennes
    {  253, 8086,   64841, 17, }, // America/Indiana/Winamac
    {  253, 2799,   64858,  8, }, // America/Jamaica
    {  253, 8125,   64866, 21, }, // America/Kentucky/Monticello
    {  253, 5932,     207, 22, }, // America/Lower_Princes
    {  253, 8153,   26588,  6, }, // America/Maceio
    {  253, 1908,   64887,  6, }, // America/Manaus
    {  253, 2917,   26603,  8, }, // America/Mazatlan
    {  253, 8220,     229,  6, }, // America/Merida
    {  253, 2949,   58762,  6, }, // America/Mexico_City
    {  253, 1850,     235, 19, }, // America/Noronha
    {  253, 8301,   64893, 23, }, // America/North_Dakota/Beulah
    {  253, 8329,   64916, 23, }, // America/North_Dakota/Center
    {  253, 8357,   64939, 26, }, // America/North_Dakota/New_Salem
    {  253,  393,     317,  4, }, // America/Nuuk
    {  253, 5954,   64965, 14, }, // America/Port_of_Spain
    {  253, 5843,   64979, 10, }, // America/Puerto_Rico
    {  253, 8477,   26704,  8, }, // America/Santarem
    {  253, 6111,   64989, 14, }, // America/Santo_Domingo
    {  253, 1878,   26712,  9, }, // America/Sao_Paulo
    {  253, 6899,     321, 16, }, // America/Scoresbysund
    {  253, 8508,   26721, 16, }, // America/St_Barthelemy
    {  253, 2061,   65003, 25, }, // America/St_Johns
    {  253, 5863,   65028, 16, }, // America/St_Kitts
    {  253, 5880,   65044, 12, }, // America/St_Lucia
    {  253,  855,   26771, 12, }, // America/St_Thomas
    {  253, 5913,   65056, 13, }, // America/St_Vincent
    {  253, 5756,   65069,  5, }, // America/Thule
    {  253, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  253, 7295,   62201,  5, }, // Antarctica/Syowa
    {  253, 5395,   57607,  8, }, // Asia/Almaty
    {  253, 5523,   65074,  6, }, // Asia/Aqtau
    {  253, 5368,   65080, 11, }, // Asia/Aqtobe
    {  253,  964,   65091,  8, }, // Asia/Ashgabat
    {  253, 8624,   65099,  7, }, // Asia/Atyrau
    {  253, 3607,    4162,  6, }, // Asia/Baghdad
    {  253, 5534,   65106,  7, }, // Asia/Bahrain
    {  253, 3653,   35998,  5, }, // Asia/Baku
    {  253, 4086,   65113,  8, }, // Asia/Beirut
    {  253, 3778,   65121,  7, }, // Asia/Bishkek
    {  253, 4601,   36009,  6, }, // Asia/Chita
    {  253, 4540,    4201,  5, }, // Asia/Damascus
    {  253, 3596,   65128,  5, }, // Asia/Dubai
    {  253, 6159,   65133,  9, }, // Asia/Dushanbe
    {  253, 8636,   65142, 10, }, // Asia/Famagusta
    {  253, 4795,   65152,  6, }, // Asia/Hebron
    {  253, 1259,   65158, 17, }, // Asia/Ho_Chi_Minh
    {  253, 4184,   65175,  8, }, // Asia/Irkutsk
    {  253, 1290,   65183,  9, }, // Asia/Jerusalem
    {  253, 3560,   36037,  6, }, // Asia/Kabul
    {  253, 4316,   36043, 10, }, // Asia/Kamchatka
    {  253, 1185,   65192,  9, }, // Asia/Kathmandu
    {  253,  992,   36053,  8, }, // Asia/Kolkata
    {  253, 4197,   65201, 11, }, // Asia/Krasnoyarsk
    {  253, 5547,   65212,  6, }, // Asia/Kuwait
    {  253, 1211,     464,  5, }, // Asia/Macau
    {  253, 1349,   65218,  8, }, // Asia/Makassar
    {  253, 7182,   65226,  7, }, // Asia/Manila
    {  253, 6920,   20857,  7, }, // Asia/Muscat
    {  253, 2554,   65233,  7, }, // Asia/Nicosia
    {  253, 4135,   44379, 12, }, // Asia/Novosibirsk
    {  253, 7145,   65240,  7, }, // Asia/Oral
    {  253, 8708,   65247,  8, }, // Asia/Qostanay
    {  253, 4269,   65255,  9, }, // Asia/Qyzylorda
    {  253, 3584,   20874,  5, }, // Asia/Riyadh
    {  253, 4408,   65264,  9, }, // Asia/Sakhalin
    {  253, 7245,   65273, 10, }, // Asia/Samarkand
    {  253, 3279,   65283,  5, }, // Asia/Seoul
    {  253, 3300,   36101,  9, }, // Asia/Singapore
    {  253, 4781,   65288,  8, }, // Asia/Tashkent
    {  253, 3915,   65296,  8, }, // Asia/Tbilisi
    {  253, 2772,   65304,  7, }, // Asia/Tehran
    {  253, 1021,   65311, 11, }, // Asia/Ulaanbaatar
    {  253, 1159,   26933,  6, }, // Asia/Urumqi
    {  253, 4828,   65322,  8, }, // Asia/Yakutsk
    {  253, 1235,   65330,  7, }, // Asia/Yangon
    {  253, 3883,   65337, 13, }, // Asia/Yekaterinburg
    {  253, 3747,   26947,  6, }, // Asia/Yerevan
    {  253, 3663,   44461,  6, }, // Atlantic/Azores
    {  253, 5671,   44467,  8, }, // Atlantic/Bermuda
    {  253, 6628,   65350, 13, }, // Atlantic/Canary
    {  253, 3727,   65363,  8, }, // Atlantic/Cape_Verde
    {  253, 1395,   65371, 10, }, // Atlantic/Faroe
    {  253, 8736,   65381,  6, }, // Atlantic/Madeira
    {  253, 7272,   65387, 14, }, // Atlantic/South_Georgia
    {  253, 6821,   65401, 13, }, // Atlantic/St_Helena
    {  253, 1670,   65414,  8, }, // Australia/Adelaide
    {  253, 6216,   65422,  7, }, // Europe/Andorra
    {  253, 2300,   65429,  7, }, // Europe/Athens
    {  253, 2167,   57661,  9, }, // Europe/Brussels
    {  253, 3928,   21073,  8, }, // Europe/Bucharest
    {  253, 8772,   27037,  8, }, // Europe/Busingen
    {  253, 6289,   21081, 10, }, // Europe/Copenhagen
    {  253, 8788,   27045,  9, }, // Europe/Guernsey
    {  253, 8804,   65436, 10, }, // Europe/Isle_of_Man
    {  253, 2527,    8148,  4, }, // Europe/Kyiv
    {  253, 3213,   65446,  8, }, // Europe/Lisbon
    {  253, 2501,   21152,  7, }, // Europe/London
    {  253, 6397,   65454,  5, }, // Europe/Malta
    {  253, 3537,   44576,  6, }, // Europe/Moscow
    {  253, 6470,   65459, 11, }, // Europe/San_Marino
    {  253, 6202,   21213,  6, }, // Europe/Tirane
    {  253, 8882,   65470, 10, }, // Europe/Ulyanovsk
    {  253, 6567,   65480, 10, }, // Europe/Vatican
    {  253, 6231,   65490,  6, }, // Europe/Vienna
    {  253, 3190,   65496,  8, }, // Europe/Warsaw
    {  253, 5029,   65504,  7, }, // Indian/Comoro
    {  253, 7260,   27115,  4, }, // Indian/Mahe
    {  253, 4069,   65511,  7, }, // Indian/Mauritius
    {  253, 7211,   65518, 10, }, // Indian/Reunion
    {  253, 3157,     554,  5, }, // Pacific/Chuuk
    {  253, 2237,   65528, 13, }, // Pacific/Easter
    {  253, 7360,   27148,  5, }, // Pacific/Efate
    {  253, 3902,    4606,  5, }, // Pacific/Fiji
    {  253, 6678,   21277,  9, }, // Pacific/Galapagos
    {  253, 2723,    4611,  8, }, // Pacific/Honolulu
    {  253, 3049,    4619,  9, }, // Pacific/Kanton
    {  253, 4051,   65541,  9, }, // Pacific/Marquesas
    {  253, 7098,    4628,  6, }, // Pacific/Noumea
    {  253, 7155,   65550,  6, }, // Pacific/Palau
    {  253, 3096,     586,  7, }, // Pacific/Pohnpei
    {  260, 5271,   65556,  8, }, // Africa/Ndjamena French/Latin/Canada
    {  260, 2866,   65564, 15, }, // Africa/Tripoli
    {  260, 5654,   65579, 12, }, // America/Barbados
    {  260, 5496,   65591, 12, }, // America/Cayman
    {  260, 8301,   65603, 23, }, // America/North_Dakota/Beulah
    {  260, 8329,   65626, 23, }, // America/North_Dakota/Center
    {  260, 8357,   65649, 26, }, // America/North_Dakota/New_Salem
    {  260, 2061,     351, 10, }, // America/St_Johns
    {  260, 5863,   65675, 26, }, // America/St_Kitts
    {  260,  855,   44293, 12, }, // America/St_Thomas
    {  260, 6133,   65701, 16, }, // Antarctica/DumontDUrville
    {  260, 1093,   65717,  5, }, // Asia/Dhaka
    {  260, 1317,   65722,  8, }, // Asia/Thimphu
    {  260, 6628,   65730, 13, }, // Atlantic/Canary
    {  260, 1395,   65743, 10, }, // Atlantic/Faroe
    {  260, 8804,   65753, 10, }, // Europe/Isle_of_Man
    {  260, 2501,   21152,  7, }, // Europe/London
    {  260, 6567,   21222,  7, }, // Europe/Vatican
    {  260, 2237,   65763, 13, }, // Pacific/Easter
    {  260, 7155,   65776,  5, }, // Pacific/Palau
    {  299,   14,       0,  6, }, // Africa/Asmara Friulian/Latin/Italy
    {  299, 4435,       6,  8, }, // Africa/Sao_Tome
    {  299, 4252,      14,  8, }, // America/Asuncion
    {  299,  237,      22,  8, }, // America/Atikokan
    {  299, 7623,      30, 17, }, // America/Bahia_Banderas
    {  299, 3868,      47,  6, }, // America/Cancun
    {  299, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  299, 5723,      66,  7, }, // America/Curacao
    {  299,  481,      73, 13, }, // America/Indiana/Knox
    {  299, 7961,      86, 16, }, // America/Indiana/Marengo
    {  299, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  299, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  299, 8038,     139, 14, }, // America/Indiana/Vevay
    {  299, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  299, 8086,     171, 16, }, // America/Indiana/Winamac
    {  299, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  299, 5932,     207, 22, }, // America/Lower_Princes
    {  299, 8220,     229,  6, }, // America/Merida
    {  299, 2379,   65781, 10, }, // America/New_York
    {  299, 1850,     235, 19, }, // America/Noronha
    {  299, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  299, 8329,     274, 20, }, // America/North_Dakota/Center
    {  299, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  299,  393,     317,  4, }, // America/Nuuk
    {  299, 1878,   65791, 20, }, // America/Sao_Paulo
    {  299, 6899,     321, 16, }, // America/Scoresbysund
    {  299, 8508,     337, 14, }, // America/St_Barthelemy
    {  299, 2061,     351, 10, }, // America/St_Johns
    {  299, 5863,     361,  9, }, // America/St_Kitts
    {  299, 5880,     370,  9, }, // America/St_Lucia
    {  299,  855,     379, 10, }, // America/St_Thomas
    {  299, 5913,     389, 11, }, // America/St_Vincent
    {  299, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  299, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  299, 1185,     448,  9, }, // Asia/Kathmandu
    {  299,  992,     457,  7, }, // Asia/Kolkata
    {  299, 1211,     464,  5, }, // Asia/Macau
    {  299, 1235,     477,  6, }, // Asia/Yangon
    {  299, 3663,   65811,  6, }, // Atlantic/Azores
    {  299, 6628,   65817,  8, }, // Atlantic/Canary
    {  299, 1395,     483,  5, }, // Atlantic/Faroe
    {  299, 6821,     488, 10, }, // Atlantic/St_Helena
    {  299, 6336,   65825,  7, }, // Europe/Belgrade
    {  299, 2527,     514,  4, }, // Europe/Kyiv
    {  299, 3213,   65832,  7, }, // Europe/Lisbon
    {  299, 6506,   65839,  7, }, // Europe/Ljubljana
    {  299, 6365,   65846, 10, }, // Europe/Luxembourg
    {  299, 6397,   65454,  5, }, // Europe/Malta
    {  299, 3537,   65856,  6, }, // Europe/Moscow
    {  299, 6275,   65862,  6, }, // Europe/Prague
    {  299, 6470,   65868,  9, }, // Europe/San_Marino
    {  299, 6231,   65877,  5, }, // Europe/Vienna
    {  299, 7211,     547,  7, }, // Indian/Reunion
    {  299, 3157,     554,  5, }, // Pacific/Chuuk
    {  299, 3096,     586,  7, }, // Pacific/Pohnpei
    {  300,   14,       0,  6, }, // Africa/Asmara Fulah/Latin/Senegal
    {  300, 4435,       6,  8, }, // Africa/Sao_Tome
    {  300, 4252,      14,  8, }, // America/Asuncion
    {  300,  237,      22,  8, }, // America/Atikokan
    {  300, 7623,      30, 17, }, // America/Bahia_Banderas
    {  300, 3868,      47,  6, }, // America/Cancun
    {  300, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  300, 5723,      66,  7, }, // America/Curacao
    {  300,  481,      73, 13, }, // America/Indiana/Knox
    {  300, 7961,      86, 16, }, // America/Indiana/Marengo
    {  300, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  300, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  300, 8038,     139, 14, }, // America/Indiana/Vevay
    {  300, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  300, 8086,     171, 16, }, // America/Indiana/Winamac
    {  300, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  300, 5932,     207, 22, }, // America/Lower_Princes
    {  300, 8220,     229,  6, }, // America/Merida
    {  300, 1850,     235, 19, }, // America/Noronha
    {  300, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  300, 8329,     274, 20, }, // America/North_Dakota/Center
    {  300, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  300,  393,     317,  4, }, // America/Nuuk
    {  300, 6899,     321, 16, }, // America/Scoresbysund
    {  300, 8508,     337, 14, }, // America/St_Barthelemy
    {  300, 2061,     351, 10, }, // America/St_Johns
    {  300, 5863,     361,  9, }, // America/St_Kitts
    {  300, 5880,     370,  9, }, // America/St_Lucia
    {  300,  855,     379, 10, }, // America/St_Thomas
    {  300, 5913,     389, 11, }, // America/St_Vincent
    {  300, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  300, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  300, 1185,     448,  9, }, // Asia/Kathmandu
    {  300,  992,     457,  7, }, // Asia/Kolkata
    {  300, 1211,     464,  5, }, // Asia/Macau
    {  300, 1235,     477,  6, }, // Asia/Yangon
    {  300, 1395,     483,  5, }, // Atlantic/Faroe
    {  300, 6821,     488, 10, }, // Atlantic/St_Helena
    {  300, 2527,     514,  4, }, // Europe/Kyiv
    {  300, 7211,     547,  7, }, // Indian/Reunion
    {  300, 3157,     554,  5, }, // Pacific/Chuuk
    {  300, 3096,     586,  7, }, // Pacific/Pohnpei
    {  306, 6788,   65882, 12, }, // Africa/Abidjan Fulah/Adlam/Guinea
    {  306, 6760,   65894, 10, }, // Africa/Accra
    {  306, 5059,   65904, 18, }, // Africa/Addis_Ababa
    {  306, 7420,   65922, 14, }, // Africa/Algiers
    {  306,   14,   65936, 12, }, // Africa/Asmara
    {  306,   44,   65948, 14, }, // Africa/Bamako
    {  306, 5257,   65962, 10, }, // Africa/Bangui
    {  306, 6746,   65972, 12, }, // Africa/Banjul
    {  306, 7435,   65984, 12, }, // Africa/Bissau
    {  306, 4957,   65996, 18, }, // Africa/Blantyre
    {  306, 5287,   66014, 16, }, // Africa/Brazzaville
    {  306, 4922,   66030, 18, }, // Africa/Bujumbura
    {  306, 2320,   66048, 10, }, // Africa/Cairo
    {  306, 4117,   66058, 26, }, // Africa/Casablanca
    {  306, 7449,   66084, 10, }, // Africa/Ceuta
    {  306, 6773,   66094, 18, }, // Africa/Conakry
    {  306, 6840,   66112, 12, }, // Africa/Dakar
    {  306, 5130,   66124, 25, }, // Africa/Dar_es_Salaam
    {  306, 5043,   66149, 14, }, // Africa/Djibouti
    {  306, 5243,   66163, 12, }, // Africa/Douala
    {  306, 5166,   66175, 14, }, // Africa/El_Aaiun
    {  306, 6853,   66189, 12, }, // Africa/Freetown
    {  306, 4906,   66201, 16, }, // Africa/Gaborone
    {  306, 5015,   66217, 12, }, // Africa/Harare
    {  306, 4479,   66229, 22, }, // Africa/Johannesburg
    {  306, 4499,   66251,  8, }, // Africa/Juba
    {  306, 5151,   66259, 14, }, // Africa/Kampala
    {  306, 4524,   66273, 14, }, // Africa/Khartoum
    {  306, 4973,   66287, 12, }, // Africa/Kigali
    {  306, 5306,   66299, 14, }, // Africa/Kinshasa
    {  306, 4744,   66313, 12, }, // Africa/Lagos
    {  306, 5336,   66325, 20, }, // Africa/Libreville
    {  306, 6869,   66345,  8, }, // Africa/Lome
    {  306, 5211,   66353, 16, }, // Africa/Luanda
    {  306, 4939,   66369, 18, }, // Africa/Lubumbashi
    {  306, 5001,   66387, 12, }, // Africa/Lusaka
    {  306, 5322,   66399, 14, }, // Africa/Malabo
    {  306, 4987,   66413, 12, }, // Africa/Maputo
    {  306, 5197,   66425, 12, }, // Africa/Maseru
    {  306, 5182,   66437, 18, }, // Africa/Mbabane
    {  306, 5113,   66455, 16, }, // Africa/Mogadishu
    {  306, 7034,   66471, 16, }, // Africa/Monrovia
    {  306, 3853,   66487, 14, }, // Africa/Nairobi
    {  306, 5271,   66501, 16, }, // Africa/Ndjamena
    {  306, 5354,   66517, 12, }, // Africa/Niamey
    {  306, 6803,   66529, 20, }, // Africa/Nouakchott
    {  306, 6727,   66549, 16, }, // Africa/Ouagadougou
    {  306, 5225,   66565, 21, }, // Africa/Porto-Novo
    {  306, 4435,   66586, 19, }, // Africa/Sao_Tome
    {  306, 2866,   66605, 14, }, // Africa/Tripoli
    {  306, 6554,   66619, 12, }, // Africa/Tunis
    {  306, 4152,   66631, 14, }, // Africa/Windhoek
    {  306,  132,   66645,  8, }, // America/Adak
    {  306, 3346,   66653, 18, }, // America/Anchorage
    {  306, 5607,   66671, 14, }, // America/Anguilla
    {  306, 5624,   66685, 12, }, // America/Antigua
    {  306, 4554,   66697, 18, }, // America/Araguaina
    {  306,  166,   66715, 29, }, // America/Argentina/Buenos_Aires
    {  306,   91,   66744, 20, }, // America/Argentina/Catamarca
    {  306,  270,   66764, 16, }, // America/Argentina/Cordoba
    {  306,  441,   66780, 10, }, // America/Argentina/Jujuy
    {  306, 7462,   66790, 17, }, // America/Argentina/La_Rioja
    {  306,  565,   66807, 16, }, // America/Argentina/Mendoza
    {  306, 7489,   66823, 21, }, // America/Argentina/Rio_Gallegos
    {  306, 7520,   66844, 10, }, // America/Argentina/Salta
    {  306, 7544,   66854, 19, }, // America/Argentina/San_Juan
    {  306, 5580,   66873, 19, }, // America/Argentina/San_Luis
    {  306, 7571,   66892, 16, }, // America/Argentina/Tucuman
    {  306, 7597,   66908, 14, }, // America/Argentina/Ushuaia
    {  306, 5640,   66922, 12, }, // America/Aruba
    {  306, 4252,   66934, 18, }, // America/Asuncion
    {  306,  237,   66952, 18, }, // America/Atikokan
    {  306, 3679,   66970, 12, }, // America/Bahia
    {  306, 7623,   66982, 33, }, // America/Bahia_Banderas
    {  306, 5654,   67015, 18, }, // America/Barbados
    {  306, 7646,   67033, 10, }, // America/Belem
    {  306, 5407,   67043, 12, }, // America/Belize
    {  306, 7660,   67055, 25, }, // America/Blanc-Sablon
    {  306, 7681,   67080, 19, }, // America/Boa_Vista
    {  306, 4361,   67099, 12, }, // America/Bogota
    {  306, 7699,   67111, 12, }, // America/Boise
    {  306, 7713,   67123, 25, }, // America/Cambridge_Bay
    {  306, 7735,   67148, 23, }, // America/Campo_Grande
    {  306, 3868,   67171, 14, }, // America/Cancun
    {  306, 4694,   67185, 14, }, // America/Caracas
    {  306, 4345,   67199, 12, }, // America/Cayenne
    {  306, 5496,   67211, 14, }, // America/Cayman
    {  306, 2260,   67225, 16, }, // America/Chicago
    {  306, 7756,   67241, 16, }, // America/Chihuahua
    {  306, 5422,   67257, 21, }, // America/Costa_Rica
    {  306, 7796,   67278, 14, }, // America/Creston
    {  306, 3791,   67292, 14, }, // America/Cuiaba
    {  306, 5723,   67306, 18, }, // America/Curacao
    {  306, 7812,   67324, 26, }, // America/Danmarkshavn
    {  306, 7833,   67350, 12, }, // America/Dawson
    {  306, 7848,   67362, 21, }, // America/Dawson_Creek
    {  306,  805,   67383, 12, }, // America/Denver
    {  306, 3465,   67395, 14, }, // America/Detroit
    {  306, 5739,   67409, 18, }, // America/Dominica
    {  306,  893,   67427, 16, }, // America/Edmonton
    {  306, 7869,   67443, 14, }, // America/Eirunepe
    {  306, 5441,   67457, 23, }, // America/El_Salvador
    {  306, 7886,   67480, 22, }, // America/Fort_Nelson
    {  306, 7906,   67502, 20, }, // America/Fortaleza
    {  306, 7924,   67522, 15, }, // America/Glace_Bay
    {  306, 6881,   67537, 15, }, // America/Goose_Bay
    {  306, 4612,   67552, 19, }, // America/Grand_Turk
    {  306, 5770,   67571, 16, }, // America/Grenada
    {  306, 5786,   67587, 24, }, // America/Guadeloupe
    {  306, 3760,   67611, 20, }, // America/Guatemala
    {  306, 4841,   67631, 14, }, // America/Guayaquil
    {  306, 6932,   67645, 14, }, // America/Guyana
    {  306, 1939,   67659, 18, }, // America/Halifax
    {  306, 2281,   67677, 14, }, // America/Havana
    {  306, 7942,   67691, 24, }, // America/Hermosillo
    {  306,  348,   67715, 28, }, // America/Indiana/Indianapolis
    {  306,  481,   67743, 28, }, // America/Indiana/Knox
    {  306, 7961,   67771, 34, }, // America/Indiana/Marengo
    {  306, 7985,   67805, 40, }, // America/Indiana/Petersburg
    {  306, 8012,   67845, 37, }, // America/Indiana/Tell_City
    {  306, 8038,   67882, 32, }, // America/Indiana/Vevay
    {  306, 8060,   67914, 34, }, // America/Indiana/Vincennes
    {  306, 8086,   67948, 34, }, // America/Indiana/Winamac
    {  306, 8110,   67982, 12, }, // America/Inuvik
    {  306,  660,   67994, 16, }, // America/Iqaluit
    {  306, 2799,   68010, 16, }, // America/Jamaica
    {  306, 5380,   68026, 12, }, // America/Juneau
    {  306,  521,   68038, 12, }, // America/Kentucky/Louisville
    {  306, 8125,   68050, 40, }, // America/Kentucky/Monticello
    {  306, 5704,   68090, 20, }, // America/Kralendijk
    {  306, 4376,   68110, 13, }, // America/La_Paz
    {  306, 7169,   68123, 10, }, // America/Lima
    {  306, 3239,   68133, 21, }, // America/Los_Angeles
    {  306, 5932,   68154, 36, }, // America/Lower_Princes
    {  306, 8153,   68190, 12, }, // America/Maceio
    {  306, 8168,   68202, 14, }, // America/Managua
    {  306, 1908,   68216, 12, }, // America/Manaus
    {  306, 5897,   68228, 14, }, // America/Marigot
    {  306, 5805,   68242, 16, }, // America/Martinique
    {  306, 8184,   68258, 20, }, // America/Matamoros
    {  306, 2917,   68278, 16, }, // America/Mazatlan
    {  306, 8202,   68294, 16, }, // America/Menominee
    {  306, 8220,   68310, 12, }, // America/Merida
    {  306, 8235,   68322, 20, }, // America/Metlakatla
    {  306, 2949,   68342, 27, }, // America/Mexico_City
    {  306, 4391,   68369, 14, }, // America/Miquelon
    {  306, 8254,   68383, 16, }, // America/Moncton
    {  306, 8270,   68399, 18, }, // America/Monterrey
    {  306, 4098,   68417, 24, }, // America/Montevideo
    {  306, 5824,   68441, 20, }, // America/Montserrat
    {  306, 5481,   68461, 10, }, // America/Nassau
    {  306, 2379,   68471, 15, }, // America/New_York
    {  306, 8288,   68486, 10, }, // America/Nome
    {  306, 1850,   68496, 14, }, // America/Noronha
    {  306, 8301,   68510, 33, }, // America/North_Dakota/Beulah
    {  306, 8329,   68543, 37, }, // America/North_Dakota/Center
    {  306, 8357,   68580, 42, }, // America/North_Dakota/New_Salem
    {  306,  393,   68622,  8, }, // America/Nuuk
    {  306, 8388,   68630, 14, }, // America/Ojinaga
    {  306, 2356,   68644, 16, }, // America/Panama
    {  306, 6173,   68660, 20, }, // America/Paramaribo
    {  306, 2973,   68680, 12, }, // America/Phoenix
    {  306, 3945,   68692, 22, }, // America/Port-au-Prince
    {  306, 5954,   68714, 26, }, // America/Port_of_Spain
    {  306, 8404,   68740, 23, }, // America/Porto_Velho
    {  306, 5843,   68763, 19, }, // America/Puerto_Rico
    {  306, 4030,   68782, 23, }, // America/Punta_Arenas
    {  306, 8424,   68805, 23, }, // America/Rankin_Inlet
    {  306, 8445,   68828, 12, }, // America/Recife
    {  306, 1995,   68840, 14, }, // America/Regina
    {  306, 8460,   68854, 16, }, // America/Resolute
    {  306,  695,   68870, 21, }, // America/Rio_Branco
    {  306, 8477,   68891, 18, }, // America/Santarem
    {  306, 2201,   68909, 20, }, // America/Santiago
    {  306, 6111,   68929, 27, }, // America/Santo_Domingo
    {  306, 1878,   68956, 19, }, // America/Sao_Paulo
    {  306, 6899,   68975, 30, }, // America/Scoresbysund
    {  306, 8494,   69005, 10, }, // America/Sitka
    {  306, 8508,   69015, 27, }, // America/St_Barthelemy
    {  306, 2061,   69042, 19, }, // America/St_Johns
    {  306, 5863,   69061, 21, }, // America/St_Kitts
    {  306, 5880,   69082, 19, }, // America/St_Lucia
    {  306,  855,   69101, 21, }, // America/St_Thomas
    {  306, 5913,   69122, 23, }, // America/St_Vincent
    {  306, 8530,   69145, 23, }, // America/Swift_Current
    {  306, 5461,   69168, 22, }, // America/Tegucigalpa
    {  306, 5756,   69190,  8, }, // America/Thule
    {  306,  313,   69198, 18, }, // America/Tijuana
    {  306,  608,   69216, 16, }, // America/Toronto
    {  306, 5688,   69232, 16, }, // America/Tortola
    {  306, 2093,   69248, 16, }, // America/Vancouver
    {  306, 2144,   69264, 18, }, // America/Whitehorse
    {  306,  734,   69282, 16, }, // America/Winnipeg
    {  306, 8552,   69298, 14, }, // America/Yakutat
    {  306, 6001,   69312, 10, }, // Antarctica/Casey
    {  306, 6094,   69322, 12, }, // Antarctica/Davis
    {  306, 6133,   69334, 27, }, // Antarctica/DumontDUrville
    {  306, 8568,   69361, 14, }, // Antarctica/Macquarie
    {  306, 7066,   69375, 12, }, // Antarctica/Mawson
    {  306, 7113,   69387, 18, }, // Antarctica/McMurdo
    {  306, 8589,   69405, 14, }, // Antarctica/Palmer
    {  306, 7226,   69419, 12, }, // Antarctica/Rothera
    {  306, 7295,   69431, 14, }, // Antarctica/Syowa
    {  306, 8607,   69445, 12, }, // Antarctica/Troll
    {  306, 7374,   69457, 12, }, // Antarctica/Vostok
    {  306, 1429,   69469, 24, }, // Arctic/Longyearbyen
    {  306, 5570,   69493,  8, }, // Asia/Aden
    {  306, 5395,   69501, 14, }, // Asia/Almaty
    {  306, 3968,   69515, 12, }, // Asia/Amman
    {  306, 5511,   69527, 14, }, // Asia/Anadyr
    {  306, 5523,   69541, 14, }, // Asia/Aqtau
    {  306, 5368,   69555, 14, }, // Asia/Aqtobe
    {  306,  964,   69569, 16, }, // Asia/Ashgabat
    {  306, 8624,   69585, 14, }, // Asia/Atyrau
    {  306, 3607,   69599, 16, }, // Asia/Baghdad
    {  306, 5534,   69615, 16, }, // Asia/Bahrain
    {  306, 3653,   69631, 10, }, // Asia/Baku
    {  306, 4466,   69641, 16, }, // Asia/Bangkok
    {  306, 3571,   69657, 16, }, // Asia/Barnaul
    {  306, 4086,   69673, 16, }, // Asia/Beirut
    {  306, 3778,   69689, 16, }, // Asia/Bishkek
    {  306, 5989,   69705, 12, }, // Asia/Brunei
    {  306, 4601,   69717, 10, }, // Asia/Chita
    {  306, 4511,   69727, 14, }, // Asia/Colombo
    {  306, 4540,   69741, 14, }, // Asia/Damascus
    {  306, 1093,   66112, 10, }, // Asia/Dhaka
    {  306, 6192,   69755, 10, }, // Asia/Dili
    {  306, 3596,   69765, 10, }, // Asia/Dubai
    {  306, 6159,   69775, 14, }, // Asia/Dushanbe
    {  306, 8636,   69789, 18, }, // Asia/Famagusta
    {  306, 8651,   69807, 10, }, // Asia/Gaza
    {  306, 4795,   69817, 14, }, // Asia/Hebron
    {  306, 1259,   69831, 31, }, // Asia/Ho_Chi_Minh
    {  306, 2704,   69862, 17, }, // Asia/Hong_Kong
    {  306, 4771,   69879, 10, }, // Asia/Hovd
    {  306, 4184,   69889, 18, }, // Asia/Irkutsk
    {  306, 7006,   69907, 16, }, // Asia/Jakarta
    {  306, 6992,   69923, 16, }, // Asia/Jayapura
    {  306, 1290,   69939, 12, }, // Asia/Jerusalem
    {  306, 3560,   69951, 10, }, // Asia/Kabul
    {  306, 4316,   69961, 18, }, // Asia/Kamchatka
    {  306, 4239,   69979, 14, }, // Asia/Karachi
    {  306, 1185,   69993, 18, }, // Asia/Kathmandu
    {  306, 8661,   70011, 14, }, // Asia/Khandyga
    {  306,  992,   70025, 14, }, // Asia/Kolkata
    {  306, 4197,   70039, 28, }, // Asia/Krasnoyarsk
    {  306, 4859,   70067, 27, }, // Asia/Kuala_Lumpur
    {  306, 5976,   70094, 10, }, // Asia/Kuching
    {  306, 5547,   70104, 14, }, // Asia/Kuwait
    {  306, 1211,   70118, 12, }, // Asia/Macau
    {  306, 4017,   70130, 14, }, // Asia/Magadan
    {  306, 1349,   70144, 16, }, // Asia/Makassar
    {  306, 7182,   70160, 12, }, // Asia/Manila
    {  306, 6920,   70172, 14, }, // Asia/Muscat
    {  306, 2554,   70186, 16, }, // Asia/Nicosia
    {  306, 8675,   70202, 28, }, // Asia/Novokuznetsk
    {  306, 4135,   70230, 24, }, // Asia/Novosibirsk
    {  306, 4229,   70254, 12, }, // Asia/Omsk
    {  306, 7145,   70266,  8, }, // Asia/Oral
    {  306, 6961,   70274, 17, }, // Asia/Phnom_Penh
    {  306, 8693,   70291, 20, }, // Asia/Pontianak
    {  306, 4214,   70311, 16, }, // Asia/Pyongyang
    {  306, 5559,   70327, 10, }, // Asia/Qatar
    {  306, 8708,   70337, 16, }, // Asia/Qostanay
    {  306, 4269,   70353, 18, }, // Asia/Qyzylorda
    {  306, 3584,   70371, 12, }, // Asia/Riyadh
    {  306, 4408,   70383, 16, }, // Asia/Sakhalin
    {  306, 7245,   70399, 20, }, // Asia/Samarkand
    {  306, 3279,   70419, 10, }, // Asia/Seoul
    {  306, 1053,   70429, 12, }, // Asia/Shanghai
    {  306, 3300,   70441, 18, }, // Asia/Singapore
    {  306, 4297,   70459, 24, }, // Asia/Srednekolymsk
    {  306, 3263,   70483, 12, }, // Asia/Taipei
    {  306, 4781,   70495, 16, }, // Asia/Tashkent
    {  306, 3915,   70511, 18, }, // Asia/Tbilisi
    {  306, 2772,   70529, 16, }, // Asia/Tehran
    {  306, 1317,   70545, 10, }, // Asia/Thimphu
    {  306, 2821,   70555, 12, }, // Asia/Tokyo
    {  306, 4572,   70567, 14, }, // Asia/Tomsk
    {  306, 1021,   70581, 18, }, // Asia/Ulaanbaatar
    {  306, 1159,   70599, 12, }, // Asia/Urumqi
    {  306, 8722,   70611, 17, }, // Asia/Ust-Nera
    {  306, 6977,   70628, 18, }, // Asia/Vientiane
    {  306, 4710,   70646, 28, }, // Asia/Vladivostok
    {  306, 4828,   70674, 18, }, // Asia/Yakutsk
    {  306, 1235,   70692, 14, }, // Asia/Yangon
    {  306, 3883,   70706, 28, }, // Asia/Yekaterinburg
    {  306, 3747,   70734, 16, }, // Asia/Yerevan
    {  306, 3663,   70750, 14, }, // Atlantic/Azores
    {  306, 5671,   70764, 16, }, // Atlantic/Bermuda
    {  306, 6628,   70780, 12, }, // Atlantic/Canary
    {  306, 3727,   70792, 21, }, // Atlantic/Cape_Verde
    {  306, 1395,   70813, 10, }, // Atlantic/Faroe
    {  306, 8736,   70823, 14, }, // Atlantic/Madeira
    {  306, 2748,   70837, 18, }, // Atlantic/Reykjavik
    {  306, 7272,   70855, 25, }, // Atlantic/South_Georgia
    {  306, 6821,   70880, 23, }, // Atlantic/St_Helena
    {  306, 6644,   70903, 16, }, // Atlantic/Stanley
    {  306, 1670,   70919, 16, }, // Australia/Adelaide
    {  306, 1635,   70935, 20, }, // Australia/Brisbane
    {  306, 1799,   70955, 21, }, // Australia/Broken_Hill
    {  306, 1583,   70976, 14, }, // Australia/Darwin
    {  306, 3637,   70990, 12, }, // Australia/Eucla
    {  306, 1516,   71002, 16, }, // Australia/Hobart
    {  306, 8753,   71018, 20, }, // Australia/Lindeman
    {  306, 1547,   71038, 19, }, // Australia/Lord_Howe
    {  306, 1727,   71057, 18, }, // Australia/Melbourne
    {  306, 1762,   71075, 10, }, // Australia/Perth
    {  306, 1463,   71085, 12, }, // Australia/Sydney
    {  306, 6441,   71097, 22, }, // Europe/Amsterdam
    {  306, 6216,   71119, 14, }, // Europe/Andorra
    {  306, 3620,   71133, 18, }, // Europe/Astrakhan
    {  306, 2300,   71151, 12, }, // Europe/Athens
    {  306, 6336,   71163, 18, }, // Europe/Belgrade
    {  306, 4757,   71181, 12, }, // Europe/Berlin
    {  306, 6488,   71193, 22, }, // Europe/Bratislava
    {  306, 2167,   71215, 16, }, // Europe/Brussels
    {  306, 3928,   71231, 18, }, // Europe/Bucharest
    {  306, 3806,   71249, 16, }, // Europe/Budapest
    {  306, 8772,   71265, 16, }, // Europe/Busingen
    {  306, 2583,   71281, 18, }, // Europe/Chisinau
    {  306, 6289,   71299, 18, }, // Europe/Copenhagen
    {  306, 2338,   71317, 14, }, // Europe/Dublin
    {  306, 6307,   71331, 20, }, // Europe/Gibraltar
    {  306, 8788,   71351, 16, }, // Europe/Guernsey
    {  306, 6612,   71367, 16, }, // Europe/Helsinki
    {  306, 8804,   71383, 23, }, // Europe/Isle_of_Man
    {  306, 1130,   71406, 16, }, // Europe/Istanbul
    {  306, 8823,   71422, 12, }, // Europe/Jersey
    {  306, 3979,   71434, 18, }, // Europe/Kaliningrad
    {  306, 8929,   71452, 14, }, // Europe/Kirov
    {  306, 2527,   71466, 12, }, // Europe/Kyiv
    {  306, 3213,   71478, 14, }, // Europe/Lisbon
    {  306, 6506,   71492, 20, }, // Europe/Ljubljana
    {  306, 2501,   71512, 12, }, // Europe/London
    {  306, 6365,   71524, 22, }, // Europe/Luxembourg
    {  306, 4892,   71546, 14, }, // Europe/Madrid
    {  306, 6397,   71560, 10, }, // Europe/Malta
    {  306, 6582,   71570, 26, }, // Europe/Mariehamn
    {  306, 3693,   71596, 14, }, // Europe/Minsk
    {  306, 6410,   71610, 14, }, // Europe/Monaco
    {  306, 3537,   71624, 10, }, // Europe/Moscow
    {  306, 6458,   71634, 10, }, // Europe/Oslo
    {  306, 4284,   71644,  8, }, // Europe/Paris
    {  306, 6424,   71652, 20, }, // Europe/Podgorica
    {  306, 6275,   71672, 14, }, // Europe/Prague
    {  306, 8837,   71686,  8, }, // Europe/Riga
    {  306, 6324,   71694, 10, }, // Europe/Rome
    {  306, 4331,   71704, 12, }, // Europe/Samara
    {  306, 6470,   71716, 19, }, // Europe/San_Marino
    {  306, 6245,   71735, 18, }, // Europe/Sarajevo
    {  306, 4451,   71753, 16, }, // Europe/Saratov
    {  306, 8849,   71769, 20, }, // Europe/Simferopol
    {  306, 6383,   71789, 18, }, // Europe/Skopje
    {  306, 6599,   71807, 12, }, // Europe/Sofia
    {  306, 6523,   71819, 18, }, // Europe/Stockholm
    {  306, 8867,   71837, 14, }, // Europe/Tallinn
    {  306, 6202,   71851, 12, }, // Europe/Tirane
    {  306, 8882,   71863, 22, }, // Europe/Ulyanovsk
    {  306, 6352,   71885, 14, }, // Europe/Vaduz
    {  306, 6567,   71899, 14, }, // Europe/Vatican
    {  306, 6231,   71913, 14, }, // Europe/Vienna
    {  306, 8899,   71927, 14, }, // Europe/Vilnius
    {  306, 4727,   71941, 20, }, // Europe/Volgograd
    {  306, 3190,   71961, 10, }, // Europe/Warsaw
    {  306, 6261,   71971, 14, }, // Europe/Zagreb
    {  306, 6540,   71985, 10, }, // Europe/Zurich
    {  306, 5078,   71995, 28, }, // Indian/Antananarivo
    {  306, 6947,   72023, 10, }, // Indian/Chagos
    {  306, 6046,   72033, 18, }, // Indian/Christmas
    {  306, 6063,   72051, 12, }, // Indian/Cocos
    {  306, 5029,   72063, 14, }, // Indian/Comoro
    {  306, 6661,   72077, 16, }, // Indian/Kerguelen
    {  306, 7260,   72093, 10, }, // Indian/Mahe
    {  306, 7050,   72103, 18, }, // Indian/Maldives
    {  306, 4069,   72121, 14, }, // Indian/Mauritius
    {  306, 5098,   72135, 14, }, // Indian/Mayotte
    {  306, 7211,   72149, 16, }, // Indian/Reunion
    {  306, 4422,   72165, 12, }, // Pacific/Apia
    {  306,  932,   72177, 14, }, // Pacific/Auckland
    {  306, 3706,   72191, 18, }, // Pacific/Bougainville
    {  306, 3015,   72209, 12, }, // Pacific/Chatham
    {  306, 3157,   72221, 10, }, // Pacific/Chuuk
    {  306, 2237,   72231, 10, }, // Pacific/Easter
    {  306, 7360,   72241, 10, }, // Pacific/Efate
    {  306, 7327,   72251, 14, }, // Pacific/Fakaofo
    {  306, 3902,   72265, 10, }, // Pacific/Fiji
    {  306, 7343,   72275, 18, }, // Pacific/Funafuti
    {  306, 6678,   72293, 18, }, // Pacific/Galapagos
    {  306, 6696,   72311, 16, }, // Pacific/Gambier
    {  306, 3822,   72327, 24, }, // Pacific/Guadalcanal
    {  306, 6018,   72351, 12, }, // Pacific/Guam
    {  306, 3049,   72363, 20, }, // Pacific/Kanton
    {  306, 3998,   72383, 20, }, // Pacific/Kiritimati
    {  306, 7019,   72403, 12, }, // Pacific/Kosrae
    {  306, 2842,   72415, 20, }, // Pacific/Kwajalein
    {  306, 4877,   72435, 14, }, // Pacific/Majuro
    {  306, 4051,   72449, 18, }, // Pacific/Marquesas
    {  306, 8914,   72467, 12, }, // Pacific/Midway
    {  306, 7084,   72479, 10, }, // Pacific/Nauru
    {  306, 7132,   72489, 10, }, // Pacific/Niue
    {  306, 4168,   72499, 16, }, // Pacific/Norfolk
    {  306, 7098,   72515, 12, }, // Pacific/Noumea
    {  306, 3126,   72527, 17, }, // Pacific/Pago_Pago
    {  306, 7155,   72544, 10, }, // Pacific/Palau
    {  306, 7194,   72554, 18, }, // Pacific/Pitcairn
    {  306, 3096,   72572, 12, }, // Pacific/Pohnpei
    {  306, 4807,   72584, 25, }, // Pacific/Port_Moresby
    {  306, 6076,   72609, 18, }, // Pacific/Rarotonga
    {  306, 6031,   72627, 14, }, // Pacific/Saipan
    {  306, 7312,   72641, 14, }, // Pacific/Tahiti
    {  306, 6712,   72655, 14, }, // Pacific/Tarawa
    {  306, 4583,   72669, 20, }, // Pacific/Tongatapu
    {  306, 7392,   72689, 10, }, // Pacific/Wake
    {  306, 7405,   72699, 14, }, // Pacific/Wallis
    {  324, 5059,   72713, 11, }, // Africa/Addis_Ababa Gaelic/Latin/United Kingdom
    {  324,   14,   72724,  6, }, // Africa/Asmara
    {  324, 5130,   72730, 12, }, // Africa/Dar_es_Salaam
    {  324, 5043,   72742, 10, }, // Africa/Djibouti
    {  324, 5166,   63491,  8, }, // Africa/El_Aaiun
    {  324, 4479,   72752, 11, }, // Africa/Johannesburg
    {  324, 6869,   20418,  4, }, // Africa/Lome
    {  324, 5271,   72763,  9, }, // Africa/Ndjamena
    {  324, 4435,       6,  8, }, // Africa/Sao_Tome
    {  324, 5607,   72772,  9, }, // America/Anguilla
    {  324, 5624,   72781,  8, }, // America/Antigua
    {  324, 4554,   26559,  9, }, // America/Araguaina
    {  324,  270,   20479,  7, }, // America/Argentina/Cordoba
    {  324, 7489,   44137, 12, }, // America/Argentina/Rio_Gallegos
    {  324, 7571,   20486,  7, }, // America/Argentina/Tucuman
    {  324, 5640,   72789,  5, }, // America/Aruba
    {  324, 4252,      14,  8, }, // America/Asuncion
    {  324,  237,      22,  8, }, // America/Atikokan
    {  324, 7623,      30, 17, }, // America/Bahia_Banderas
    {  324, 7646,   35896,  5, }, // America/Belem
    {  324, 5407,   72794, 10, }, // America/Belize
    {  324, 4361,   20507,  6, }, // America/Bogota
    {  324, 3868,      47,  6, }, // America/Cancun
    {  324, 5496,   72804,  7, }, // America/Cayman
    {  324, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  324, 5422,   72811, 11, }, // America/Costa_Rica
    {  324, 3791,   26574,  6, }, // America/Cuiaba
    {  324, 5723,      66,  7, }, // America/Curacao
    {  324, 5739,   72822, 10, }, // America/Dominica
    {  324, 7869,   26580,  8, }, // America/Eirunepe
    {  324, 5441,   72832, 12, }, // America/El_Salvador
    {  324, 7924,   72844,  9, }, // America/Glace_Bay
    {  324, 4612,   72853, 12, }, // America/Grand_Turk
    {  324, 5770,   72865,  8, }, // America/Grenada
    {  324, 5786,   20526,  8, }, // America/Guadeloupe
    {  324, 3760,   72873,  9, }, // America/Guatemala
    {  324, 6932,   72882,  9, }, // America/Guyana
    {  324,  481,      73, 13, }, // America/Indiana/Knox
    {  324, 7961,      86, 16, }, // America/Indiana/Marengo
    {  324, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  324, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  324, 8038,     139, 14, }, // America/Indiana/Vevay
    {  324, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  324, 8086,     171, 16, }, // America/Indiana/Winamac
    {  324, 8110,   72891,  7, }, // America/Inuvik
    {  324, 2799,   72898,  8, }, // America/Jamaica
    {  324, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  324, 5932,     207, 22, }, // America/Lower_Princes
    {  324, 8153,   26588,  6, }, // America/Maceio
    {  324, 5805,   72906,  9, }, // America/Martinique
    {  324, 2917,   26603,  8, }, // America/Mazatlan
    {  324, 8220,     229,  6, }, // America/Merida
    {  324, 2949,   72915, 17, }, // America/Mexico_City
    {  324, 5824,   72932,  9, }, // America/Montserrat
    {  324, 2379,   72941, 13, }, // America/New_York
    {  324, 1850,     235, 19, }, // America/Noronha
    {  324, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  324, 8329,     274, 20, }, // America/North_Dakota/Center
    {  324, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  324,  393,     317,  4, }, // America/Nuuk
    {  324, 5954,   72954, 15, }, // America/Port_of_Spain
    {  324, 5843,   72969, 11, }, // America/Puerto_Rico
    {  324, 8424,   72980, 11, }, // America/Rankin_Inlet
    {  324, 8460,   72991, 10, }, // America/Resolute
    {  324, 8477,   26704,  8, }, // America/Santarem
    {  324, 1878,   26712,  9, }, // America/Sao_Paulo
    {  324, 6899,     321, 16, }, // America/Scoresbysund
    {  324, 8508,   35936, 16, }, // America/St_Barthelemy
    {  324, 2061,     351, 10, }, // America/St_Johns
    {  324, 5863,   73001, 14, }, // America/St_Kitts
    {  324, 5880,   73015, 12, }, // America/St_Lucia
    {  324,  855,     379, 10, }, // America/St_Thomas
    {  324, 5913,   73027, 15, }, // America/St_Vincent
    {  324, 5756,   26796,  7, }, // America/Thule
    {  324, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  324, 8568,   73042, 17, }, // Antarctica/Macquarie
    {  324, 3968,   73059,  5, }, // Asia/Amman
    {  324, 5368,   57205,  6, }, // Asia/Aqtobe
    {  324,  964,   44322,  7, }, // Asia/Ashgabat
    {  324, 3607,   73064,  7, }, // Asia/Baghdad
    {  324, 5534,   73071,  8, }, // Asia/Bahrain
    {  324, 3778,   33455,  6, }, // Asia/Bishkek
    {  324, 5989,   73079,  8, }, // Asia/Brunei
    {  324, 6159,   33468,  7, }, // Asia/Dushanbe
    {  324, 8651,   73087,  4, }, // Asia/Gaza
    {  324, 1259,   73091, 19, }, // Asia/Ho_Chi_Minh
    {  324, 4771,   26835,  5, }, // Asia/Hovd
    {  324, 1290,   73110,  9, }, // Asia/Jerusalem
    {  324, 4239,   73119,  6, }, // Asia/Karachi
    {  324, 1185,     448,  9, }, // Asia/Kathmandu
    {  324,  992,     457,  7, }, // Asia/Kolkata
    {  324, 5547,   73125,  8, }, // Asia/Kuwait
    {  324, 1211,   73133,  7, }, // Asia/Macau
    {  324, 4214,   73140, 10, }, // Asia/Pyongyang
    {  324, 5559,   73150,  5, }, // Asia/Qatar
    {  324, 8708,   73155,  8, }, // Asia/Qostanay
    {  324, 4269,   22533,  9, }, // Asia/Qyzylorda
    {  324, 3300,   73163,  9, }, // Asia/Singapore
    {  324, 4781,   73172,  7, }, // Asia/Tashkent
    {  324, 3915,   73179,  8, }, // Asia/Tbilisi
    {  324, 2772,   73187,  6, }, // Asia/Tehran
    {  324, 2821,   63705,  5, }, // Asia/Tokyo
    {  324, 1159,   26933,  6, }, // Asia/Urumqi
    {  324, 6977,   73193, 10, }, // Asia/Vientiane
    {  324, 1235,    7880,  6, }, // Asia/Yangon
    {  324, 3663,   73203, 13, }, // Atlantic/Azores
    {  324, 5671,   73216,  8, }, // Atlantic/Bermuda
    {  324, 6628,   73224, 22, }, // Atlantic/Canary
    {  324, 3727,   73246, 13, }, // Atlantic/Cape_Verde
    {  324, 1395,   73259,  4, }, // Atlantic/Faroe
    {  324, 2748,   36176,  9, }, // Atlantic/Reykjavik
    {  324, 7272,   73263, 14, }, // Atlantic/South_Georgia
    {  324, 6821,   73277, 19, }, // Atlantic/St_Helena
    {  324, 1463,    5892,  5, }, // Australia/Sydney
    {  324, 2300,   73296,  9, }, // Europe/Athens
    {  324, 2167,   73305, 12, }, // Europe/Brussels
    {  324, 2583,   63785,  8, }, // Europe/Chisinau
    {  324, 6289,   73317,  6, }, // Europe/Copenhagen
    {  324, 2338,   73323, 17, }, // Europe/Dublin
    {  324, 6307,   73340, 10, }, // Europe/Gibraltar
    {  324, 8788,   73350, 10, }, // Europe/Guernsey
    {  324, 8804,   73360, 15, }, // Europe/Isle_of_Man
    {  324, 8823,   73375,  9, }, // Europe/Jersey
    {  324, 2527,    8148,  4, }, // Europe/Kyiv
    {  324, 2501,   73384, 20, }, // Europe/London
    {  324, 6365,   73404, 10, }, // Europe/Luxembourg
    {  324, 3537,   73414,  5, }, // Europe/Moscow
    {  324, 4284,   73419,  5, }, // Europe/Paris
    {  324, 6275,   73424,  4, }, // Europe/Prague
    {  324, 8837,   73428,  4, }, // Europe/Riga
    {  324, 6324,   73432,  8, }, // Europe/Rome
    {  324, 6599,   22919,  6, }, // Europe/Sofia
    {  324, 6202,    8255,  6, }, // Europe/Tirane
    {  324, 6567,   73440, 11, }, // Europe/Vatican
    {  324, 6540,    4577,  6, }, // Europe/Zurich
    {  324, 6046,   73451,  7, }, // Indian/Christmas
    {  324, 6661,   36302,  9, }, // Indian/Kerguelen
    {  324, 7260,   27115,  4, }, // Indian/Mahe
    {  324, 7050,   73458, 24, }, // Indian/Maldives
    {  324, 4069,   73482, 24, }, // Indian/Mauritius
    {  324, 7211,     547,  7, }, // Indian/Reunion
    {  324, 3157,     554,  5, }, // Pacific/Chuuk
    {  324, 2237,   73506,  8, }, // Pacific/Easter
    {  324, 3902,   73514,  4, }, // Pacific/Fiji
    {  324, 6678,   21277,  9, }, // Pacific/Galapagos
    {  324, 6696,   73518,  9, }, // Pacific/Gambier
    {  324, 2723,    4611,  8, }, // Pacific/Honolulu
    {  324, 3049,    4619,  9, }, // Pacific/Kanton
    {  324, 4051,   73527, 20, }, // Pacific/Marquesas
    {  324, 7084,   73547,  6, }, // Pacific/Nauru
    {  324, 7155,   73553,  6, }, // Pacific/Palau
    {  324, 7194,   73559, 14, }, // Pacific/Pitcairn
    {  324, 3096,     586,  7, }, // Pacific/Pohnpei
    {  324, 7405,   73573,  6, }, // Pacific/Wallis
    {  325, 6760,    4825,  2, }, // Africa/Accra Ga/Latin/Ghana
    {  325,   14,       0,  6, }, // Africa/Asmara
    {  325, 2320,    4690,  5, }, // Africa/Cairo
    {  325, 4117,    4695, 10, }, // Africa/Casablanca
    {  325, 6773,   26485,  7, }, // Africa/Conakry
    {  325, 6803,   73579, 10, }, // Africa/Nouakchott
    {  325, 4435,       6,  8, }, // Africa/Sao_Tome
    {  325, 3346,   73589,  8, }, // America/Anchorage
    {  325, 4252,      14,  8, }, // America/Asuncion
    {  325,  237,      22,  8, }, // America/Atikokan
    {  325, 7623,      30, 17, }, // America/Bahia_Banderas
    {  325, 7660,    6620, 12, }, // America/Blanc-Sablon
    {  325, 7713,   73597, 25, }, // America/Cambridge_Bay
    {  325, 3868,    6682,  6, }, // America/Cancun
    {  325, 5496,   21601,  6, }, // America/Cayman
    {  325, 2260,   73622,  7, }, // America/Chicago
    {  325, 7756,   73629,  9, }, // America/Chihuahua
    {  325, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  325, 5422,   21621, 10, }, // America/Costa_Rica
    {  325, 7796,    6736,  7, }, // America/Creston
    {  325, 5723,    6749,  7, }, // America/Curacao
    {  325, 7848,   73638, 21, }, // America/Dawson_Creek
    {  325, 5739,   21666,  8, }, // America/Dominica
    {  325, 7886,   73659, 10, }, // America/Fort_Nelson
    {  325, 7924,   73669, 21, }, // America/Glace_Bay
    {  325, 6881,   73690, 21, }, // America/Goose_Bay
    {  325, 4612,   73711, 10, }, // America/Grand_Turk
    {  325, 1939,   73721,  7, }, // America/Halifax
    {  325,  481,   73728, 13, }, // America/Indiana/Knox
    {  325, 7961,      86, 16, }, // America/Indiana/Marengo
    {  325, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  325, 8012,   73741, 19, }, // America/Indiana/Tell_City
    {  325, 8038,     139, 14, }, // America/Indiana/Vevay
    {  325, 8060,   73760, 18, }, // America/Indiana/Vincennes
    {  325, 8086,    5161, 16, }, // America/Indiana/Winamac
    {  325,  660,    5177,  7, }, // America/Iqaluit
    {  325, 2799,    3995,  7, }, // America/Jamaica
    {  325, 8125,   73778, 19, }, // America/Kentucky/Monticello
    {  325, 5932,   73797, 26, }, // America/Lower_Princes
    {  325, 5805,    5260,  8, }, // America/Martinique
    {  325, 8220,     229,  6, }, // America/Merida
    {  325, 2949,   73823, 10, }, // America/Mexico_City
    {  325, 4391,    7149,  7, }, // America/Miquelon
    {  325, 8254,    7156,  7, }, // America/Moncton
    {  325, 1850,     235, 19, }, // America/Noronha
    {  325, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  325, 8329,   73833, 23, }, // America/North_Dakota/Center
    {  325, 8357,   73856, 26, }, // America/North_Dakota/New_Salem
    {  325,  393,     317,  4, }, // America/Nuuk
    {  325, 3945,   73882, 21, }, // America/Port-au-Prince
    {  325, 5954,   73903, 18, }, // America/Port_of_Spain
    {  325, 5843,   22058, 11, }, // America/Puerto_Rico
    {  325, 8424,   73921, 15, }, // America/Rankin_Inlet
    {  325, 6899,   73936, 15, }, // America/Scoresbysund
    {  325, 8508,     337, 14, }, // America/St_Barthelemy
    {  325, 2061,     351, 10, }, // America/St_Johns
    {  325, 5863,     361,  9, }, // America/St_Kitts
    {  325, 5880,   73951,  9, }, // America/St_Lucia
    {  325,  855,     379, 10, }, // America/St_Thomas
    {  325, 5913,   73960, 11, }, // America/St_Vincent
    {  325, 8530,   73971, 12, }, // America/Swift_Current
    {  325, 5461,    5528, 11, }, // America/Tegucigalpa
    {  325, 2093,   73983,  9, }, // America/Vancouver
    {  325, 2144,   73992, 10, }, // America/Whitehorse
    {  325, 6001,   74002,  5, }, // Antarctica/Casey
    {  325, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  325, 8568,   74007,  8, }, // Antarctica/Macquarie
    {  325, 7113,   74015,  8, }, // Antarctica/McMurdo
    {  325, 5523,    5609,  5, }, // Asia/Aqtau
    {  325, 5368,    7606,  6, }, // Asia/Aqtobe
    {  325, 4511,    5637,  7, }, // Asia/Colombo
    {  325, 4540,   26821,  7, }, // Asia/Damascus
    {  325, 1259,   74023, 20, }, // Asia/Ho_Chi_Minh
    {  325, 1290,    5686,  9, }, // Asia/Jerusalem
    {  325, 4239,   74043,  7, }, // Asia/Karachi
    {  325, 1185,     448,  9, }, // Asia/Kathmandu
    {  325,  992,     457,  7, }, // Asia/Kolkata
    {  325, 5976,   74050,  7, }, // Asia/Kuching
    {  325, 1211,    7731,  5, }, // Asia/Macau
    {  325, 6920,    4326,  6, }, // Asia/Muscat
    {  325, 2554,    5738,  7, }, // Asia/Nicosia
    {  325, 5559,    4332,  4, }, // Asia/Qatar
    {  325, 8708,     469,  8, }, // Asia/Qostanay
    {  325, 4269,   26897,  9, }, // Asia/Qyzylorda
    {  325, 1159,   74057,  6, }, // Asia/Urumqi
    {  325, 1235,     477,  6, }, // Asia/Yangon
    {  325, 3727,   22654, 10, }, // Atlantic/Cape_Verde
    {  325, 1395,     483,  5, }, // Atlantic/Faroe
    {  325, 6821,     488, 10, }, // Atlantic/St_Helena
    {  325, 1799,   74063, 15, }, // Australia/Broken_Hill
    {  325, 3637,   22722,  5, }, // Australia/Eucla
    {  325, 1547,   74078, 11, }, // Australia/Lord_Howe
    {  325, 2527,     514,  4, }, // Europe/Kyiv
    {  325, 6947,   74089,  6, }, // Indian/Chagos
    {  325, 6046,   74095,  7, }, // Indian/Christmas
    {  325, 6063,    4583,  5, }, // Indian/Cocos
    {  325, 5029,   22985,  6, }, // Indian/Comoro
    {  325, 7211,     547,  7, }, // Indian/Reunion
    {  325,  932,   74102,  6, }, // Pacific/Auckland
    {  325, 3015,   74108,  7, }, // Pacific/Chatham
    {  325, 3157,   74115,  5, }, // Pacific/Chuuk
    {  325, 3822,    8360, 11, }, // Pacific/Guadalcanal
    {  325, 4051,   74120,  9, }, // Pacific/Marquesas
    {  325, 7194,    6251,  8, }, // Pacific/Pitcairn
    {  325, 3096,     586,  7, }, // Pacific/Pohnpei
    {  326, 6760,   74129,  4, }, // Africa/Accra Galician/Latin/Spain
    {  326, 5059,   20370, 10, }, // Africa/Addis_Ababa
    {  326, 7420,   74133,  5, }, // Africa/Algiers
    {  326,   14,       0,  6, }, // Africa/Asmara
    {  326,   44,   74138,  6, }, // Africa/Bamako
    {  326, 2320,   74144,  7, }, // Africa/Cairo
    {  326, 5043,   26492,  7, }, // Africa/Djibouti
    {  326, 5166,   74151,  6, }, // Africa/El_Aaiun
    {  326, 4479,   74157, 12, }, // Africa/Johannesburg
    {  326, 4524,   74169,  7, }, // Africa/Khartoum
    {  326, 6869,   20418,  4, }, // Africa/Lome
    {  326, 5001,   74176,  6, }, // Africa/Lusaka
    {  326, 5113,   74182,  8, }, // Africa/Mogadishu
    {  326, 5271,   44128,  9, }, // Africa/Ndjamena
    {  326, 6727,   26534,  8, }, // Africa/Ouagadougou
    {  326, 4435,   74190,  8, }, // Africa/Sao_Tome
    {  326, 2866,   20467,  7, }, // Africa/Tripoli
    {  326, 6554,   74198,  5, }, // Africa/Tunis
    {  326, 5607,    4775,  7, }, // America/Anguilla
    {  326, 4554,   26559,  9, }, // America/Araguaina
    {  326,  270,   20479,  7, }, // America/Argentina/Cordoba
    {  326, 7489,   44137, 12, }, // America/Argentina/Rio_Gallegos
    {  326, 7571,   20486,  7, }, // America/Argentina/Tucuman
    {  326, 4252,      14,  8, }, // America/Asuncion
    {  326,  237,      22,  8, }, // America/Atikokan
    {  326, 3679,   74203,  4, }, // America/Bahia
    {  326, 7623,      30, 17, }, // America/Bahia_Banderas
    {  326, 7646,   35896,  5, }, // America/Belem
    {  326, 4361,   20507,  6, }, // America/Bogota
    {  326, 3868,      47,  6, }, // America/Cancun
    {  326, 4345,   44167,  6, }, // America/Cayenne
    {  326, 5496,   74207, 12, }, // America/Cayman
    {  326, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  326, 3791,   26574,  6, }, // America/Cuiaba
    {  326, 5723,      66,  7, }, // America/Curacao
    {  326, 7869,   26580,  8, }, // America/Eirunepe
    {  326, 5441,   74219, 10, }, // America/El_Salvador
    {  326, 5770,   20519,  7, }, // America/Grenada
    {  326, 5786,   20526,  9, }, // America/Guadeloupe
    {  326, 6932,   74229,  6, }, // America/Guyana
    {  326, 2281,   74235,  8, }, // America/Havana
    {  326,  348,   74243, 12, }, // America/Indiana/Indianapolis
    {  326,  481,      73, 13, }, // America/Indiana/Knox
    {  326, 7961,      86, 16, }, // America/Indiana/Marengo
    {  326, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  326, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  326, 8038,     139, 14, }, // America/Indiana/Vevay
    {  326, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  326, 8086,     171, 16, }, // America/Indiana/Winamac
    {  326, 2799,   20544,  7, }, // America/Jamaica
    {  326, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  326, 4376,   74255,  5, }, // America/La_Paz
    {  326, 3239,   74260, 10, }, // America/Los_Angeles
    {  326, 5932,     207, 22, }, // America/Lower_Princes
    {  326, 8153,   26588,  6, }, // America/Maceio
    {  326, 5805,   20554,  9, }, // America/Martinique
    {  326, 2917,   26603,  8, }, // America/Mazatlan
    {  326, 8220,     229,  6, }, // America/Merida
    {  326, 2949,   74270, 16, }, // America/Mexico_City
    {  326, 2379,   44194,  9, }, // America/New_York
    {  326, 1850,     235, 19, }, // America/Noronha
    {  326, 8301,   74286, 23, }, // America/North_Dakota/Beulah
    {  326, 8329,   74309, 23, }, // America/North_Dakota/Center
    {  326, 8357,   74332, 26, }, // America/North_Dakota/New_Salem
    {  326,  393,     317,  4, }, // America/Nuuk
    {  326, 2356,   20665,  6, }, // America/Panama
    {  326, 3945,   74358, 14, }, // America/Port-au-Prince
    {  326, 5954,   74372, 12, }, // America/Port_of_Spain
    {  326, 5843,   64979, 10, }, // America/Puerto_Rico
    {  326,  695,   74384, 10, }, // America/Rio_Branco
    {  326, 8477,   26704,  8, }, // America/Santarem
    {  326, 1878,   26712,  9, }, // America/Sao_Paulo
    {  326, 6899,     321, 16, }, // America/Scoresbysund
    {  326, 8508,   35936, 16, }, // America/St_Barthelemy
    {  326, 2061,   26737, 12, }, // America/St_Johns
    {  326, 5863,   26749, 11, }, // America/St_Kitts
    {  326, 5880,   74394, 11, }, // America/St_Lucia
    {  326,  855,   44293, 12, }, // America/St_Thomas
    {  326, 5913,   74405, 11, }, // America/St_Vincent
    {  326, 5688,   20723,  7, }, // America/Tortola
    {  326, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  326, 7295,   62201,  5, }, // Antarctica/Syowa
    {  326, 5570,   74416,  4, }, // Asia/Aden
    {  326, 5395,    7594,  6, }, // Asia/Almaty
    {  326, 3968,   20730,  4, }, // Asia/Amman
    {  326, 5523,    5609,  5, }, // Asia/Aqtau
    {  326, 5368,    7606,  6, }, // Asia/Aqtobe
    {  326,  964,   74420,  9, }, // Asia/Ashgabat
    {  326, 3607,    4162,  6, }, // Asia/Baghdad
    {  326, 3653,   74429,  4, }, // Asia/Baku
    {  326, 4601,   74433,  5, }, // Asia/Chita
    {  326, 4540,   74438,  7, }, // Asia/Damascus
    {  326, 4795,   20771,  6, }, // Asia/Hebron
    {  326, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  326, 7006,   74445,  7, }, // Asia/Jakarta
    {  326, 1290,   20821,  9, }, // Asia/Jerusalem
    {  326, 3560,   74452,  5, }, // Asia/Kabul
    {  326, 1185,   20830,  8, }, // Asia/Kathmandu
    {  326, 8661,   58020,  8, }, // Asia/Khandyga
    {  326,  992,   20838,  7, }, // Asia/Kolkata
    {  326, 1211,    4321,  5, }, // Asia/Macau
    {  326, 6920,   20857,  7, }, // Asia/Muscat
    {  326, 8708,   44391,  8, }, // Asia/Qostanay
    {  326, 4269,   26897,  9, }, // Asia/Qyzylorda
    {  326, 3584,    7787,  4, }, // Asia/Riyadh
    {  326, 4408,   44399,  8, }, // Asia/Sakhalin
    {  326, 7245,   20886, 10, }, // Asia/Samarkand
    {  326, 3279,   20896,  4, }, // Asia/Seoul
    {  326, 3300,   20900,  8, }, // Asia/Singapore
    {  326, 4297,    7817, 13, }, // Asia/Srednekolymsk
    {  326, 2772,   58125,  7, }, // Asia/Tehran
    {  326, 1159,   26933,  6, }, // Asia/Urumqi
    {  326, 4828,   44441,  7, }, // Asia/Yakutsk
    {  326, 1235,     477,  6, }, // Asia/Yangon
    {  326, 3883,   74457, 13, }, // Asia/Yekaterinburg
    {  326, 3747,   74470,  7, }, // Asia/Yerevan
    {  326, 5671,   74477, 14, }, // Atlantic/Bermuda
    {  326, 6628,   74491, 14, }, // Atlantic/Canary
    {  326, 3727,   26968, 10, }, // Atlantic/Cape_Verde
    {  326, 1395,   20992,  5, }, // Atlantic/Faroe
    {  326, 2748,   74505, 10, }, // Atlantic/Reykjavik
    {  326, 7272,   74515, 14, }, // Atlantic/South_Georgia
    {  326, 6821,   26997, 12, }, // Atlantic/St_Helena
    {  326, 1670,   21030,  8, }, // Australia/Adelaide
    {  326, 1463,   22739,  6, }, // Australia/Sydney
    {  326, 6441,   74529,  9, }, // Europe/Amsterdam
    {  326, 3620,   74538,  8, }, // Europe/Astrakhan
    {  326, 2300,   27009,  6, }, // Europe/Athens
    {  326, 6336,   58906,  8, }, // Europe/Belgrade
    {  326, 4757,   21059,  6, }, // Europe/Berlin
    {  326, 2167,   74546,  8, }, // Europe/Brussels
    {  326, 3928,   21073,  8, }, // Europe/Bucharest
    {  326, 6289,   21081, 10, }, // Europe/Copenhagen
    {  326, 2338,   21091,  6, }, // Europe/Dublin
    {  326, 6307,   21097,  9, }, // Europe/Gibraltar
    {  326, 6612,   74554,  9, }, // Europe/Helsinki
    {  326, 8804,   74563, 11, }, // Europe/Isle_of_Man
    {  326, 3979,   63006, 12, }, // Europe/Kaliningrad
    {  326, 2527,   44566,  4, }, // Europe/Kyiv
    {  326, 3213,   21137,  6, }, // Europe/Lisbon
    {  326, 6506,   21143,  9, }, // Europe/Ljubljana
    {  326, 2501,   21152,  7, }, // Europe/London
    {  326, 6365,   74574, 10, }, // Europe/Luxembourg
    {  326, 6410,   74584,  6, }, // Europe/Monaco
    {  326, 3537,   74590,  7, }, // Europe/Moscow
    {  326, 4284,   21180,  5, }, // Europe/Paris
    {  326, 6275,   21185,  5, }, // Europe/Prague
    {  326, 6324,    6064,  4, }, // Europe/Rome
    {  326, 6245,   74597,  8, }, // Europe/Sarajevo
    {  326, 8849,   74605, 10, }, // Europe/Simferopol
    {  326, 6599,   21199,  5, }, // Europe/Sofia
    {  326, 6523,   74615,  9, }, // Europe/Stockholm
    {  326, 6202,   21213,  6, }, // Europe/Tirane
    {  326, 8882,   27092,  9, }, // Europe/Ulyanovsk
    {  326, 6567,   74624,  8, }, // Europe/Vatican
    {  326, 6231,   21230,  5, }, // Europe/Vienna
    {  326, 4727,   63187, 10, }, // Europe/Volgograd
    {  326, 3190,   21244,  8, }, // Europe/Warsaw
    {  326, 6540,    4577,  6, }, // Europe/Zurich
    {  326, 6046,   74632, 14, }, // Indian/Christmas
    {  326, 5029,   65504,  7, }, // Indian/Comoro
    {  326, 7260,   27115,  4, }, // Indian/Mahe
    {  326, 7050,   74646,  8, }, // Indian/Maldives
    {  326, 4069,   74654,  8, }, // Indian/Mauritius
    {  326, 7211,   21260,  7, }, // Indian/Reunion
    {  326, 3157,     554,  5, }, // Pacific/Chuuk
    {  326, 2237,   74662, 14, }, // Pacific/Easter
    {  326, 3902,   21273,  4, }, // Pacific/Fiji
    {  326, 6678,   74676, 15, }, // Pacific/Galapagos
    {  326, 2723,   74691,  8, }, // Pacific/Honolulu
    {  326, 3049,    4619,  9, }, // Pacific/Kanton
    {  326, 3096,     586,  7, }, // Pacific/Pohnpei
    {  326, 7312,   44657,  6, }, // Pacific/Tahiti
    {  327,   14,       0,  6, }, // Africa/Asmara Ganda/Latin/Uganda
    {  327, 4435,       6,  8, }, // Africa/Sao_Tome
    {  327, 4252,      14,  8, }, // America/Asuncion
    {  327,  237,      22,  8, }, // America/Atikokan
    {  327, 7623,      30, 17, }, // America/Bahia_Banderas
    {  327, 3868,      47,  6, }, // America/Cancun
    {  327, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  327, 5723,      66,  7, }, // America/Curacao
    {  327,  481,      73, 13, }, // America/Indiana/Knox
    {  327, 7961,      86, 16, }, // America/Indiana/Marengo
    {  327, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  327, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  327, 8038,     139, 14, }, // America/Indiana/Vevay
    {  327, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  327, 8086,     171, 16, }, // America/Indiana/Winamac
    {  327, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  327, 5932,     207, 22, }, // America/Lower_Princes
    {  327, 8220,     229,  6, }, // America/Merida
    {  327, 1850,     235, 19, }, // America/Noronha
    {  327, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  327, 8329,     274, 20, }, // America/North_Dakota/Center
    {  327, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  327,  393,     317,  4, }, // America/Nuuk
    {  327, 6899,     321, 16, }, // America/Scoresbysund
    {  327, 8508,     337, 14, }, // America/St_Barthelemy
    {  327, 2061,     351, 10, }, // America/St_Johns
    {  327, 5863,     361,  9, }, // America/St_Kitts
    {  327, 5880,     370,  9, }, // America/St_Lucia
    {  327,  855,     379, 10, }, // America/St_Thomas
    {  327, 5913,     389, 11, }, // America/St_Vincent
    {  327, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  327, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  327, 1185,     448,  9, }, // Asia/Kathmandu
    {  327,  992,     457,  7, }, // Asia/Kolkata
    {  327, 1211,     464,  5, }, // Asia/Macau
    {  327, 1235,     477,  6, }, // Asia/Yangon
    {  327, 1395,     483,  5, }, // Atlantic/Faroe
    {  327, 6821,     488, 10, }, // Atlantic/St_Helena
    {  327, 2527,     514,  4, }, // Europe/Kyiv
    {  327, 7211,     547,  7, }, // Indian/Reunion
    {  327, 3157,     554,  5, }, // Pacific/Chuuk
    {  327, 3096,     586,  7, }, // Pacific/Pohnpei
    {  328,   14,       0,  6, }, // Africa/Asmara Geez/Ethiopic/Ethiopia
    {  328, 4435,       6,  8, }, // Africa/Sao_Tome
    {  328, 4252,      14,  8, }, // America/Asuncion
    {  328,  237,      22,  8, }, // America/Atikokan
    {  328, 7623,      30, 17, }, // America/Bahia_Banderas
    {  328, 3868,      47,  6, }, // America/Cancun
    {  328, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  328, 5723,      66,  7, }, // America/Curacao
    {  328,  481,      73, 13, }, // America/Indiana/Knox
    {  328, 7961,      86, 16, }, // America/Indiana/Marengo
    {  328, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  328, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  328, 8038,     139, 14, }, // America/Indiana/Vevay
    {  328, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  328, 8086,     171, 16, }, // America/Indiana/Winamac
    {  328, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  328, 5932,     207, 22, }, // America/Lower_Princes
    {  328, 8220,     229,  6, }, // America/Merida
    {  328, 1850,     235, 19, }, // America/Noronha
    {  328, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  328, 8329,     274, 20, }, // America/North_Dakota/Center
    {  328, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  328,  393,     317,  4, }, // America/Nuuk
    {  328, 6899,     321, 16, }, // America/Scoresbysund
    {  328, 8508,     337, 14, }, // America/St_Barthelemy
    {  328, 2061,     351, 10, }, // America/St_Johns
    {  328, 5863,     361,  9, }, // America/St_Kitts
    {  328, 5880,     370,  9, }, // America/St_Lucia
    {  328,  855,     379, 10, }, // America/St_Thomas
    {  328, 5913,     389, 11, }, // America/St_Vincent
    {  328, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  328, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  328, 1185,     448,  9, }, // Asia/Kathmandu
    {  328,  992,     457,  7, }, // Asia/Kolkata
    {  328, 1211,     464,  5, }, // Asia/Macau
    {  328, 1235,     477,  6, }, // Asia/Yangon
    {  328, 1395,     483,  5, }, // Atlantic/Faroe
    {  328, 6821,     488, 10, }, // Atlantic/St_Helena
    {  328, 2527,     514,  4, }, // Europe/Kyiv
    {  328, 7211,     547,  7, }, // Indian/Reunion
    {  328, 3157,     554,  5, }, // Pacific/Chuuk
    {  328, 3096,     586,  7, }, // Pacific/Pohnpei
    {  330, 6788,   74699,  7, }, // Africa/Abidjan Georgian/Georgian/Georgia
    {  330, 6760,   74706,  4, }, // Africa/Accra
    {  330, 5059,   74710, 10, }, // Africa/Addis_Ababa
    {  330, 7420,   74720,  6, }, // Africa/Algiers
    {  330,   14,   74726,  6, }, // Africa/Asmara
    {  330,   44,   74732,  6, }, // Africa/Bamako
    {  330, 5257,   74738,  5, }, // Africa/Bangui
    {  330, 6746,   74743,  7, }, // Africa/Banjul
    {  330, 7435,   74750,  5, }, // Africa/Bissau
    {  330, 4957,   74755,  8, }, // Africa/Blantyre
    {  330, 5287,   74763,  9, }, // Africa/Brazzaville
    {  330, 4922,   74772,  9, }, // Africa/Bujumbura
    {  330, 2320,   74781,  5, }, // Africa/Cairo
    {  330, 4117,   74786, 10, }, // Africa/Casablanca
    {  330, 7449,   74796,  5, }, // Africa/Ceuta
    {  330, 6773,   74801,  7, }, // Africa/Conakry
    {  330, 6840,   74808,  6, }, // Africa/Dakar
    {  330, 5130,   74814, 13, }, // Africa/Dar_es_Salaam
    {  330, 5043,   74827,  6, }, // Africa/Djibouti
    {  330, 5243,   74833,  5, }, // Africa/Douala
    {  330, 5166,   74838,  9, }, // Africa/El_Aaiun
    {  330, 6853,   74847,  8, }, // Africa/Freetown
    {  330, 4906,   74855,  8, }, // Africa/Gaborone
    {  330, 5015,   74863,  6, }, // Africa/Harare
    {  330, 4479,   74869, 12, }, // Africa/Johannesburg
    {  330, 4499,   74881,  4, }, // Africa/Juba
    {  330, 5151,   74885,  7, }, // Africa/Kampala
    {  330, 4524,   74892,  7, }, // Africa/Khartoum
    {  330, 4973,   74899,  6, }, // Africa/Kigali
    {  330, 5306,   74905,  7, }, // Africa/Kinshasa
    {  330, 4744,   74912,  6, }, // Africa/Lagos
    {  330, 5336,   74918,  9, }, // Africa/Libreville
    {  330, 6869,   74927,  4, }, // Africa/Lome
    {  330, 5211,   74931,  6, }, // Africa/Luanda
    {  330, 4939,   74937,  9, }, // Africa/Lubumbashi
    {  330, 5001,   74946,  6, }, // Africa/Lusaka
    {  330, 5322,   74952,  6, }, // Africa/Malabo
    {  330, 4987,   74958,  6, }, // Africa/Maputo
    {  330, 5197,   74964,  6, }, // Africa/Maseru
    {  330, 5182,   74970,  7, }, // Africa/Mbabane
    {  330, 5113,   74977,  8, }, // Africa/Mogadishu
    {  330, 7034,   74985,  8, }, // Africa/Monrovia
    {  330, 3853,   74993,  7, }, // Africa/Nairobi
    {  330, 5271,   75000,  7, }, // Africa/Ndjamena
    {  330, 5354,   75007,  6, }, // Africa/Niamey
    {  330, 6803,   75013,  8, }, // Africa/Nouakchott
    {  330, 6727,   75021,  8, }, // Africa/Ouagadougou
    {  330, 5225,   75029, 10, }, // Africa/Porto-Novo
    {  330, 4435,   75039,  8, }, // Africa/Sao_Tome
    {  330, 2866,   75047,  7, }, // Africa/Tripoli
    {  330, 6554,   75054,  6, }, // Africa/Tunis
    {  330, 4152,   75060,  8, }, // Africa/Windhoek
    {  330,  132,   75068,  5, }, // America/Adak
    {  330, 3346,   75073,  8, }, // America/Anchorage
    {  330, 5607,   75081,  7, }, // America/Anguilla
    {  330, 5624,   75088,  7, }, // America/Antigua
    {  330, 4554,   75095,  9, }, // America/Araguaina
    {  330,  166,   75104, 13, }, // America/Argentina/Buenos_Aires
    {  330,   91,   75117,  9, }, // America/Argentina/Catamarca
    {  330,  270,   75126,  7, }, // America/Argentina/Cordoba
    {  330,  441,   75133,  5, }, // America/Argentina/Jujuy
    {  330, 7462,   75138,  8, }, // America/Argentina/La_Rioja
    {  330,  565,   75146,  7, }, // America/Argentina/Mendoza
    {  330, 7489,   75153, 12, }, // America/Argentina/Rio_Gallegos
    {  330, 7520,   75165,  5, }, // America/Argentina/Salta
    {  330, 7544,   75170,  9, }, // America/Argentina/San_Juan
    {  330, 5580,   75179,  9, }, // America/Argentina/San_Luis
    {  330, 7571,   75188,  8, }, // America/Argentina/Tucuman
    {  330, 7597,   75196,  6, }, // America/Argentina/Ushuaia
    {  330, 5640,   75202,  5, }, // America/Aruba
    {  330, 4252,   75207,  9, }, // America/Asuncion
    {  330,  237,   75216,  9, }, // America/Atikokan
    {  330, 3679,   75225,  4, }, // America/Bahia
    {  330, 7623,   75229, 15, }, // America/Bahia_Banderas
    {  330, 5654,   75244,  9, }, // America/Barbados
    {  330, 7646,   75253,  6, }, // America/Belem
    {  330, 5407,   75259,  6, }, // America/Belize
    {  330, 7660,   75265, 12, }, // America/Blanc-Sablon
    {  330, 7681,   75277,  9, }, // America/Boa_Vista
    {  330, 4361,   75286,  6, }, // America/Bogota
    {  330, 7699,   75292,  5, }, // America/Boise
    {  330, 7713,   75297, 11, }, // America/Cambridge_Bay
    {  330, 7735,   75308, 12, }, // America/Campo_Grande
    {  330, 3868,   75320,  7, }, // America/Cancun
    {  330, 4694,   75327,  8, }, // America/Caracas
    {  330, 4345,   75335,  6, }, // America/Cayenne
    {  330, 5496,   75341,  7, }, // America/Cayman
    {  330, 2260,   75348,  6, }, // America/Chicago
    {  330, 7756,   75354,  8, }, // America/Chihuahua
    {  330, 7774,   75362, 14, }, // America/Ciudad_Juarez
    {  330, 5422,   75376, 10, }, // America/Costa_Rica
    {  330, 7796,   75386,  8, }, // America/Creston
    {  330, 3791,   75394,  6, }, // America/Cuiaba
    {  330, 5723,   75400,  8, }, // America/Curacao
    {  330, 7812,   75408, 12, }, // America/Danmarkshavn
    {  330, 7833,   75420,  7, }, // America/Dawson
    {  330, 7848,   75427, 12, }, // America/Dawson_Creek
    {  330,  805,   75439,  7, }, // America/Denver
    {  330, 3465,   75446,  8, }, // America/Detroit
    {  330, 5739,   75454,  8, }, // America/Dominica
    {  330,  893,   75462,  9, }, // America/Edmonton
    {  330, 7869,   75471,  8, }, // America/Eirunepe
    {  330, 5441,   75479,  9, }, // America/El_Salvador
    {  330, 7886,   75488, 12, }, // America/Fort_Nelson
    {  330, 7906,   75500,  9, }, // America/Fortaleza
    {  330, 7924,   75509,  9, }, // America/Glace_Bay
    {  330, 6881,   75518,  7, }, // America/Goose_Bay
    {  330, 4612,   75525, 11, }, // America/Grand_Turk
    {  330, 5770,   75536,  7, }, // America/Grenada
    {  330, 5786,   75543,  9, }, // America/Guadeloupe
    {  330, 3760,   75552,  9, }, // America/Guatemala
    {  330, 4841,   75561,  9, }, // America/Guayaquil
    {  330, 6932,   75570,  6, }, // America/Guyana
    {  330, 1939,   75576,  9, }, // America/Halifax
    {  330, 2281,   75585,  6, }, // America/Havana
    {  330, 7942,   75591,  9, }, // America/Hermosillo
    {  330,  348,   75600, 13, }, // America/Indiana/Indianapolis
    {  330,  481,   75613, 14, }, // America/Indiana/Knox
    {  330, 7961,   75627, 16, }, // America/Indiana/Marengo
    {  330, 7985,   75643, 20, }, // America/Indiana/Petersburg
    {  330, 8012,   75663, 17, }, // America/Indiana/Tell_City
    {  330, 8038,   75680, 14, }, // America/Indiana/Vevay
    {  330, 8060,   75694, 16, }, // America/Indiana/Vincennes
    {  330, 8086,   75710, 17, }, // America/Indiana/Winamac
    {  330, 8110,   75727,  7, }, // America/Inuvik
    {  330,  660,   75734,  8, }, // America/Iqaluit
    {  330, 2799,   75742,  7, }, // America/Jamaica
    {  330, 5380,   75749,  4, }, // America/Juneau
    {  330,  521,   75753,  8, }, // America/Kentucky/Louisville
    {  330, 8125,   75761, 18, }, // America/Kentucky/Monticello
    {  330, 5704,   75779, 11, }, // America/Kralendijk
    {  330, 4376,   75790,  7, }, // America/La_Paz
    {  330, 7169,   75797,  4, }, // America/Lima
    {  330, 3239,   75801, 12, }, // America/Los_Angeles
    {  330, 5932,   75813, 19, }, // America/Lower_Princes
    {  330, 8153,   75832,  6, }, // America/Maceio
    {  330, 8168,   75838,  7, }, // America/Managua
    {  330, 1908,   75845,  7, }, // America/Manaus
    {  330, 5897,   75852,  6, }, // America/Marigot
    {  330, 5805,   75858,  9, }, // America/Martinique
    {  330, 8184,   75867, 10, }, // America/Matamoros
    {  330, 2917,   75877,  9, }, // America/Mazatlan
    {  330, 8202,   75886,  8, }, // America/Menominee
    {  330, 8220,   75894,  6, }, // America/Merida
    {  330, 8235,   75900, 10, }, // America/Metlakatla
    {  330, 2949,   75910,  6, }, // America/Mexico_City
    {  330, 4391,   75916,  8, }, // America/Miquelon
    {  330, 8254,   75924,  8, }, // America/Moncton
    {  330, 8270,   75932,  8, }, // America/Monterrey
    {  330, 4098,   75940, 10, }, // America/Montevideo
    {  330, 5824,   75950,  9, }, // America/Montserrat
    {  330, 5481,   75959,  5, }, // America/Nassau
    {  330, 2379,   75964,  9, }, // America/New_York
    {  330, 8288,   75888,  4, }, // America/Nome
    {  330, 1850,   75973,  7, }, // America/Noronha
    {  330, 8301,   75980, 23, }, // America/North_Dakota/Beulah
    {  330, 8329,   76003, 24, }, // America/North_Dakota/Center
    {  330, 8357,   76027, 28, }, // America/North_Dakota/New_Salem
    {  330,  393,   76055,  7, }, // America/Nuuk
    {  330, 8388,   76062,  7, }, // America/Ojinaga
    {  330, 2356,   76069,  6, }, // America/Panama
    {  330, 6173,   76075, 10, }, // America/Paramaribo
    {  330, 2973,   76085,  7, }, // America/Phoenix
    {  330, 3945,   76092, 13, }, // America/Port-au-Prince
    {  330, 5954,   76105, 14, }, // America/Port_of_Spain
    {  330, 8404,   76119, 11, }, // America/Porto_Velho
    {  330, 5843,   76130, 11, }, // America/Puerto_Rico
    {  330, 4030,   76141, 13, }, // America/Punta_Arenas
    {  330, 8424,   76154, 13, }, // America/Rankin_Inlet
    {  330, 8445,   76167,  6, }, // America/Recife
    {  330, 1995,   76173,  6, }, // America/Regina
    {  330, 8460,   76179,  8, }, // America/Resolute
    {  330,  695,   76187, 10, }, // America/Rio_Branco
    {  330, 8477,   76197,  9, }, // America/Santarem
    {  330, 2201,   76206,  8, }, // America/Santiago
    {  330, 6111,   76214, 13, }, // America/Santo_Domingo
    {  330, 1878,   76227,  9, }, // America/Sao_Paulo
    {  330, 6899,   76236, 12, }, // America/Scoresbysund
    {  330, 8494,   76248,  5, }, // America/Sitka
    {  330, 8508,   76253, 13, }, // America/St_Barthelemy
    {  330, 2061,   76266, 10, }, // America/St_Johns
    {  330, 5863,   76276, 10, }, // America/St_Kitts
    {  330, 5880,   76286, 10, }, // America/St_Lucia
    {  330,  855,   76296, 10, }, // America/St_Thomas
    {  330, 5913,   76306, 12, }, // America/St_Vincent
    {  330, 8530,   76318, 13, }, // America/Swift_Current
    {  330, 5461,   76331, 11, }, // America/Tegucigalpa
    {  330, 5756,   76342,  4, }, // America/Thule
    {  330,  313,   76346,  7, }, // America/Tijuana
    {  330,  608,   76353,  7, }, // America/Toronto
    {  330, 5688,   76360,  7, }, // America/Tortola
    {  330, 2093,   76367,  9, }, // America/Vancouver
    {  330, 2144,   76376,  9, }, // America/Whitehorse
    {  330,  734,   76385,  8, }, // America/Winnipeg
    {  330, 8552,   76393,  8, }, // America/Yakutat
    {  330, 6001,   76401,  5, }, // Antarctica/Casey
    {  330, 6094,   76406,  6, }, // Antarctica/Davis
    {  330, 6133,   76412, 15, }, // Antarctica/DumontDUrville
    {  330, 8568,   76427,  7, }, // Antarctica/Macquarie
    {  330, 7066,   76434,  7, }, // Antarctica/Mawson
    {  330, 7113,   76441,  8, }, // Antarctica/McMurdo
    {  330, 8589,   76449,  7, }, // Antarctica/Palmer
    {  330, 7226,   76456,  6, }, // Antarctica/Rothera
    {  330, 7295,   76462,  5, }, // Antarctica/Syowa
    {  330, 8607,   76467,  5, }, // Antarctica/Troll
    {  330, 7374,   76472,  7, }, // Antarctica/Vostok
    {  330, 1429,   76479, 11, }, // Arctic/Longyearbyen
    {  330, 5570,   76490,  5, }, // Asia/Aden
    {  330, 5395,   76495,  6, }, // Asia/Almaty
    {  330, 3968,   76501,  5, }, // Asia/Amman
    {  330, 5511,   76506,  7, }, // Asia/Anadyr
    {  330, 5523,   76513,  5, }, // Asia/Aqtau
    {  330, 5368,   76518,  6, }, // Asia/Aqtobe
    {  330,  964,   76524,  8, }, // Asia/Ashgabat
    {  330, 8624,   76532,  6, }, // Asia/Atyrau
    {  330, 3607,   76538,  7, }, // Asia/Baghdad
    {  330, 5534,   76545,  8, }, // Asia/Bahrain
    {  330, 3653,   76553,  4, }, // Asia/Baku
    {  330, 4466,   76557,  8, }, // Asia/Bangkok
    {  330, 3571,   76565,  8, }, // Asia/Barnaul
    {  330, 4086,   76573,  7, }, // Asia/Beirut
    {  330, 3778,   76580,  7, }, // Asia/Bishkek
    {  330, 5989,   76587,  6, }, // Asia/Brunei
    {  330, 4601,   76593,  4, }, // Asia/Chita
    {  330, 4511,   76597,  7, }, // Asia/Colombo
    {  330, 4540,   76604,  7, }, // Asia/Damascus
    {  330, 1093,   74808,  4, }, // Asia/Dhaka
    {  330, 6192,   76611,  4, }, // Asia/Dili
    {  330, 3596,   76615,  5, }, // Asia/Dubai
    {  330, 6159,   76620,  7, }, // Asia/Dushanbe
    {  330, 8636,   76627,  9, }, // Asia/Famagusta
    {  330, 8651,   76636,  4, }, // Asia/Gaza
    {  330, 4795,   76640,  7, }, // Asia/Hebron
    {  330, 1259,   76647,  8, }, // Asia/Ho_Chi_Minh
    {  330, 2704,   76655,  8, }, // Asia/Hong_Kong
    {  330, 4771,   76663,  5, }, // Asia/Hovd
    {  330, 4184,   76668,  8, }, // Asia/Irkutsk
    {  330, 7006,   76676,  7, }, // Asia/Jakarta
    {  330, 6992,   76683,  8, }, // Asia/Jayapura
    {  330, 1290,   76691, 10, }, // Asia/Jerusalem
    {  330, 3560,   76701,  6, }, // Asia/Kabul
    {  330, 4316,   76707,  8, }, // Asia/Kamchatka
    {  330, 4239,   76715,  6, }, // Asia/Karachi
    {  330, 1185,   76721,  8, }, // Asia/Kathmandu
    {  330, 8661,   76729,  7, }, // Asia/Khandyga
    {  330,  992,   76736,  7, }, // Asia/Kolkata
    {  330, 4197,   76743, 12, }, // Asia/Krasnoyarsk
    {  330, 4859,   76755, 13, }, // Asia/Kuala_Lumpur
    {  330, 5976,   76768,  7, }, // Asia/Kuching
    {  330, 5547,   76775,  7, }, // Asia/Kuwait
    {  330, 1211,   76782,  5, }, // Asia/Macau
    {  330, 4017,   76787,  8, }, // Asia/Magadan
    {  330, 1349,   76795,  8, }, // Asia/Makassar
    {  330, 7182,   76803,  6, }, // Asia/Manila
    {  330, 6920,   76809,  7, }, // Asia/Muscat
    {  330, 2554,   76816,  7, }, // Asia/Nicosia
    {  330, 8675,   76823, 12, }, // Asia/Novokuznetsk
    {  330, 4135,   76835, 12, }, // Asia/Novosibirsk
    {  330, 4229,   76847,  5, }, // Asia/Omsk
    {  330, 7145,   76852,  5, }, // Asia/Oral
    {  330, 6961,   76857,  8, }, // Asia/Phnom_Penh
    {  330, 8693,   76865, 10, }, // Asia/Pontianak
    {  330, 4214,   76875,  8, }, // Asia/Pyongyang
    {  330, 5559,   76883,  6, }, // Asia/Qatar
    {  330, 8708,   76889,  8, }, // Asia/Qostanay
    {  330, 4269,   76897,  9, }, // Asia/Qyzylorda
    {  330, 3584,   76906,  8, }, // Asia/Riyadh
    {  330, 4408,   76914,  8, }, // Asia/Sakhalin
    {  330, 7245,   76922, 10, }, // Asia/Samarkand
    {  330, 3279,   76932,  5, }, // Asia/Seoul
    {  330, 1053,   76937,  6, }, // Asia/Shanghai
    {  330, 3300,   76943,  9, }, // Asia/Singapore
    {  330, 4297,   76952, 14, }, // Asia/Srednekolymsk
    {  330, 3263,   76966,  6, }, // Asia/Taipei
    {  330, 4781,   76972,  8, }, // Asia/Tashkent
    {  330, 3915,   76980,  7, }, // Asia/Tbilisi
    {  330, 2772,   76987,  7, }, // Asia/Tehran
    {  330, 1317,   76994,  7, }, // Asia/Thimphu
    {  330, 2821,   77001,  5, }, // Asia/Tokyo
    {  330, 4572,   77006,  6, }, // Asia/Tomsk
    {  330, 1021,   77012, 10, }, // Asia/Ulaanbaatar
    {  330, 1159,   77022,  6, }, // Asia/Urumqi
    {  330, 8722,   77028,  8, }, // Asia/Ust-Nera
    {  330, 6977,   77036,  9, }, // Asia/Vientiane
    {  330, 4710,   77045, 12, }, // Asia/Vladivostok
    {  330, 4828,   77057,  8, }, // Asia/Yakutsk
    {  330, 1235,   77065,  7, }, // Asia/Yangon
    {  330, 3883,   77072, 13, }, // Asia/Yekaterinburg
    {  330, 3747,   77085,  7, }, // Asia/Yerevan
    {  330, 3663,   77092, 16, }, // Atlantic/Azores
    {  330, 5671,   77108,  7, }, // Atlantic/Bermuda
    {  330, 6628,   77115,  6, }, // Atlantic/Canary
    {  330, 3727,   77121, 10, }, // Atlantic/Cape_Verde
    {  330, 1395,   77131, 17, }, // Atlantic/Faroe
    {  330, 8736,   77148,  7, }, // Atlantic/Madeira
    {  330, 2748,   77155, 10, }, // Atlantic/Reykjavik
    {  330, 7272,   77165, 14, }, // Atlantic/South_Georgia
    {  330, 6821,   77179, 18, }, // Atlantic/St_Helena
    {  330, 6644,   77197,  6, }, // Atlantic/Stanley
    {  330, 1670,   77203,  8, }, // Australia/Adelaide
    {  330, 1635,   77211,  9, }, // Australia/Brisbane
    {  330, 1799,   77220, 12, }, // Australia/Broken_Hill
    {  330, 1583,   77232,  7, }, // Australia/Darwin
    {  330, 3637,   77239,  5, }, // Australia/Eucla
    {  330, 1516,   77244,  7, }, // Australia/Hobart
    {  330, 8753,   77251,  8, }, // Australia/Lindeman
    {  330, 1547,   77259,  9, }, // Australia/Lord_Howe
    {  330, 1727,   77268,  8, }, // Australia/Melbourne
    {  330, 1762,   77276,  5, }, // Australia/Perth
    {  330, 1463,   77281,  6, }, // Australia/Sydney
    {  330, 6441,   77287, 10, }, // Europe/Amsterdam
    {  330, 6216,   77297,  6, }, // Europe/Andorra
    {  330, 3620,   77303,  9, }, // Europe/Astrakhan
    {  330, 2300,   77312,  5, }, // Europe/Athens
    {  330, 6336,   77317,  8, }, // Europe/Belgrade
    {  330, 4757,   77325,  7, }, // Europe/Berlin
    {  330, 6488,   77332, 10, }, // Europe/Bratislava
    {  330, 2167,   77342,  8, }, // Europe/Brussels
    {  330, 3928,   77350,  9, }, // Europe/Bucharest
    {  330, 3806,   77359,  9, }, // Europe/Budapest
    {  330, 8772,   77368,  9, }, // Europe/Busingen
    {  330, 2583,   77377,  9, }, // Europe/Chisinau
    {  330, 6289,   77386, 11, }, // Europe/Copenhagen
    {  330, 2338,   77397,  7, }, // Europe/Dublin
    {  330, 6307,   77404, 10, }, // Europe/Gibraltar
    {  330, 8788,   77414,  6, }, // Europe/Guernsey
    {  330, 6612,   77420,  8, }, // Europe/Helsinki
    {  330, 8804,   77428, 13, }, // Europe/Isle_of_Man
    {  330, 1130,   77441,  8, }, // Europe/Istanbul
    {  330, 8823,   77449,  5, }, // Europe/Jersey
    {  330, 3979,   77454, 12, }, // Europe/Kaliningrad
    {  330, 8929,   77466,  6, }, // Europe/Kirov
    {  330, 2527,   77472,  5, }, // Europe/Kyiv
    {  330, 3213,   77477,  8, }, // Europe/Lisbon
    {  330, 6506,   77485,  9, }, // Europe/Ljubljana
    {  330, 2501,   77494,  7, }, // Europe/London
    {  330, 6365,   77501, 11, }, // Europe/Luxembourg
    {  330, 4892,   77512,  7, }, // Europe/Madrid
    {  330, 6397,   77519,  5, }, // Europe/Malta
    {  330, 6582,   77524,  9, }, // Europe/Mariehamn
    {  330, 3693,   77533,  6, }, // Europe/Minsk
    {  330, 6410,   77539,  6, }, // Europe/Monaco
    {  330, 3537,   77545,  7, }, // Europe/Moscow
    {  330, 6458,   77552,  4, }, // Europe/Oslo
    {  330, 4284,   77556,  6, }, // Europe/Paris
    {  330, 6424,   77562,  9, }, // Europe/Podgorica
    {  330, 6275,   77571,  5, }, // Europe/Prague
    {  330, 8837,   77576,  4, }, // Europe/Riga
    {  330, 6324,   77580,  4, }, // Europe/Rome
    {  330, 4331,   77584,  6, }, // Europe/Samara
    {  330, 6470,   77590, 10, }, // Europe/San_Marino
    {  330, 6245,   77600,  7, }, // Europe/Sarajevo
    {  330, 4451,   77607,  8, }, // Europe/Saratov
    {  330, 8849,   77615, 11, }, // Europe/Simferopol
    {  330, 6383,   77626,  6, }, // Europe/Skopje
    {  330, 6599,   77632,  5, }, // Europe/Sofia
    {  330, 6523,   77637,  9, }, // Europe/Stockholm
    {  330, 8867,   77646,  6, }, // Europe/Tallinn
    {  330, 6202,   77652,  6, }, // Europe/Tirane
    {  330, 8882,   77658, 10, }, // Europe/Ulyanovsk
    {  330, 6352,   77668,  6, }, // Europe/Vaduz
    {  330, 6567,   77674,  8, }, // Europe/Vatican
    {  330, 6231,   77682,  4, }, // Europe/Vienna
    {  330, 8899,   77686,  8, }, // Europe/Vilnius
    {  330, 4727,   77694, 10, }, // Europe/Volgograd
    {  330, 3190,   77704,  7, }, // Europe/Warsaw
    {  330, 6261,   77711,  7, }, // Europe/Zagreb
    {  330, 6540,   77718,  7, }, // Europe/Zurich
    {  330, 5078,   77725, 12, }, // Indian/Antananarivo
    {  330, 6947,   77737,  6, }, // Indian/Chagos
    {  330, 6046,   77743, 13, }, // Indian/Christmas
    {  330, 6063,   77756,  6, }, // Indian/Cocos
    {  330, 5029,   77762,  6, }, // Indian/Comoro
    {  330, 6661,   77768,  9, }, // Indian/Kerguelen
    {  330, 7260,   77777,  4, }, // Indian/Mahe
    {  330, 7050,   77781,  9, }, // Indian/Maldives
    {  330, 4069,   77790,  7, }, // Indian/Mauritius
    {  330, 5098,   77797,  6, }, // Indian/Mayotte
    {  330, 7211,   77803,  9, }, // Indian/Reunion
    {  330, 4422,   77812,  4, }, // Pacific/Apia
    {  330,  932,   77816,  7, }, // Pacific/Auckland
    {  330, 3706,   77823,  9, }, // Pacific/Bougainville
    {  330, 3015,   77832,  6, }, // Pacific/Chatham
    {  330, 3157,   77838,  5, }, // Pacific/Chuuk
    {  330, 2237,   77843,  6, }, // Pacific/Easter
    {  330, 7360,   77849,  5, }, // Pacific/Efate
    {  330, 7327,   77854,  7, }, // Pacific/Fakaofo
    {  330, 3902,   77861,  4, }, // Pacific/Fiji
    {  330, 7343,   77865,  8, }, // Pacific/Funafuti
    {  330, 6678,   77873, 10, }, // Pacific/Galapagos
    {  330, 6696,   77883,  8, }, // Pacific/Gambier
    {  330, 3822,   77891, 12, }, // Pacific/Guadalcanal
    {  330, 6018,   77903,  5, }, // Pacific/Guam
    {  330, 2723,   77908,  8, }, // Pacific/Honolulu
    {  330, 3049,   77916,  9, }, // Pacific/Kanton
    {  330, 3998,   77925, 10, }, // Pacific/Kiritimati
    {  330, 7019,   77935,  6, }, // Pacific/Kosrae
    {  330, 2842,   77941, 10, }, // Pacific/Kwajalein
    {  330, 4877,   77951,  6, }, // Pacific/Majuro
    {  330, 4051,   77957,  9, }, // Pacific/Marquesas
    {  330, 8914,   77966,  6, }, // Pacific/Midway
    {  330, 7084,   77972,  5, }, // Pacific/Nauru
    {  330, 7132,   77977,  4, }, // Pacific/Niue
    {  330, 4168,   77981,  8, }, // Pacific/Norfolk
    {  330, 7098,   77989,  5, }, // Pacific/Noumea
    {  330, 3126,   77994,  9, }, // Pacific/Pago_Pago
    {  330, 7155,   78003,  5, }, // Pacific/Palau
    {  330, 7194,   78008,  8, }, // Pacific/Pitcairn
    {  330, 3096,   78016,  6, }, // Pacific/Pohnpei
    {  330, 4807,   78022, 11, }, // Pacific/Port_Moresby
    {  330, 6076,   78033,  9, }, // Pacific/Rarotonga
    {  330, 6031,   78042,  7, }, // Pacific/Saipan
    {  330, 7312,   78049,  5, }, // Pacific/Tahiti
    {  330, 6712,   78054,  6, }, // Pacific/Tarawa
    {  330, 4583,   78060,  9, }, // Pacific/Tongatapu
    {  330, 7392,   78069,  5, }, // Pacific/Wake
    {  330, 7405,   78074,  6, }, // Pacific/Wallis
    {  331, 5059,    3902, 11, }, // Africa/Addis_Ababa German/Latin/Germany
    {  331, 7420,   58555,  6, }, // Africa/Algiers
    {  331,   14,       0,  6, }, // Africa/Asmara
    {  331, 2320,    4690,  5, }, // Africa/Cairo
    {  331, 5130,   78080, 10, }, // Africa/Dar_es_Salaam
    {  331, 5043,   78090,  9, }, // Africa/Djibouti
    {  331, 5166,   63491,  8, }, // Africa/El_Aaiun
    {  331, 4524,   26504,  7, }, // Africa/Khartoum
    {  331, 6869,   20418,  4, }, // Africa/Lome
    {  331, 5113,   78099, 10, }, // Africa/Mogadishu
    {  331, 5271,   44128,  9, }, // Africa/Ndjamena
    {  331, 5225,   26542, 10, }, // Africa/Porto-Novo
    {  331, 4435,       6,  8, }, // Africa/Sao_Tome
    {  331, 2866,   57756,  8, }, // Africa/Tripoli
    {  331,  270,   20479,  7, }, // America/Argentina/Cordoba
    {  331, 4252,      14,  8, }, // America/Asuncion
    {  331,  237,      22,  8, }, // America/Atikokan
    {  331, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  331, 4361,   20507,  6, }, // America/Bogota
    {  331, 3868,      47,  6, }, // America/Cancun
    {  331, 5496,   78109, 12, }, // America/Cayman
    {  331, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  331, 5723,      66,  7, }, // America/Curacao
    {  331, 2281,   63505,  7, }, // America/Havana
    {  331,  481,      73, 13, }, // America/Indiana/Knox
    {  331, 7961,      86, 16, }, // America/Indiana/Marengo
    {  331, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  331, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  331, 8038,     139, 14, }, // America/Indiana/Vevay
    {  331, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  331, 8086,     171, 16, }, // America/Indiana/Winamac
    {  331, 2799,    3995,  7, }, // America/Jamaica
    {  331, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  331, 5932,     207, 22, }, // America/Lower_Princes
    {  331, 8220,    7126,  6, }, // America/Merida
    {  331, 2949,   78121, 12, }, // America/Mexico_City
    {  331, 1850,     235, 19, }, // America/Noronha
    {  331, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  331, 8329,     274, 20, }, // America/North_Dakota/Center
    {  331, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  331,  393,     317,  4, }, // America/Nuuk
    {  331, 1878,   26712,  9, }, // America/Sao_Paulo
    {  331, 6899,     321, 16, }, // America/Scoresbysund
    {  331, 8508,   26721, 16, }, // America/St_Barthelemy
    {  331, 2061,     351, 10, }, // America/St_Johns
    {  331, 5863,     361,  9, }, // America/St_Kitts
    {  331, 5880,     370,  9, }, // America/St_Lucia
    {  331,  855,     379, 10, }, // America/St_Thomas
    {  331, 5913,     389, 11, }, // America/St_Vincent
    {  331, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  331, 7374,    4147,  6, }, // Antarctica/Vostok
    {  331, 5368,    7606,  6, }, // Asia/Aqtobe
    {  331,  964,   44322,  7, }, // Asia/Ashgabat
    {  331, 3607,    4162,  6, }, // Asia/Baghdad
    {  331, 3778,   57220,  8, }, // Asia/Bishkek
    {  331, 5989,   78133, 17, }, // Asia/Brunei
    {  331, 4601,   78150,  7, }, // Asia/Chita
    {  331, 4540,    4201,  8, }, // Asia/Damascus
    {  331, 6159,   57228,  9, }, // Asia/Dushanbe
    {  331, 1259,   78157, 17, }, // Asia/Ho_Chi_Minh
    {  331, 2704,    4231,  8, }, // Asia/Hong_Kong
    {  331, 4771,   78174,  5, }, // Asia/Hovd
    {  331, 4316,   78179, 11, }, // Asia/Kamchatka
    {  331, 4239,   78190,  9, }, // Asia/Karachi
    {  331, 1185,     448,  9, }, // Asia/Kathmandu
    {  331, 8661,   58020,  8, }, // Asia/Khandyga
    {  331,  992,   64334,  8, }, // Asia/Kolkata
    {  331, 4197,    4288, 11, }, // Asia/Krasnoyarsk
    {  331, 1211,    4321,  5, }, // Asia/Macau
    {  331, 6920,   26882,  6, }, // Asia/Muscat
    {  331, 2554,    5738,  7, }, // Asia/Nicosia
    {  331, 8675,   78199, 12, }, // Asia/Novokuznetsk
    {  331, 4135,   78211, 11, }, // Asia/Novosibirsk
    {  331, 4214,   78222,  9, }, // Asia/Pyongyang
    {  331, 5559,    4332,  5, }, // Asia/Qatar
    {  331, 8708,   44391,  8, }, // Asia/Qostanay
    {  331, 4269,   57300,  9, }, // Asia/Qyzylorda
    {  331, 3584,    7787,  4, }, // Asia/Riyadh
    {  331, 4408,   58084,  8, }, // Asia/Sakhalin
    {  331, 3300,   20900,  8, }, // Asia/Singapore
    {  331, 3263,   78231,  6, }, // Asia/Taipei
    {  331, 4781,   57328,  9, }, // Asia/Tashkent
    {  331, 3915,   78237,  6, }, // Asia/Tbilisi
    {  331, 2772,    7842,  7, }, // Asia/Tehran
    {  331, 2821,    4356,  5, }, // Asia/Tokyo
    {  331, 1159,   26933,  6, }, // Asia/Urumqi
    {  331, 4710,    4361, 11, }, // Asia/Vladivostok
    {  331, 4828,    7873,  7, }, // Asia/Yakutsk
    {  331, 1235,    7880,  6, }, // Asia/Yangon
    {  331, 3883,    4380, 13, }, // Asia/Yekaterinburg
    {  331, 3747,   57360,  6, }, // Asia/Yerevan
    {  331, 3663,   58841,  6, }, // Atlantic/Azores
    {  331, 6628,   78243,  7, }, // Atlantic/Canary
    {  331, 3727,   26968, 10, }, // Atlantic/Cape_Verde
    {  331, 1395,   78250,  6, }, // Atlantic/Faroe
    {  331, 2748,   78256, 11, }, // Atlantic/Reykjavik
    {  331, 7272,   78267, 11, }, // Atlantic/South_Georgia
    {  331, 6821,     488, 10, }, // Atlantic/St_Helena
    {  331, 3620,   78278,  9, }, // Europe/Astrakhan
    {  331, 2300,    4438,  5, }, // Europe/Athens
    {  331, 6336,   27015,  7, }, // Europe/Belgrade
    {  331, 2167,   22766,  7, }, // Europe/Brussels
    {  331, 3928,   27029,  8, }, // Europe/Bucharest
    {  331, 8772,   27037,  8, }, // Europe/Busingen
    {  331, 6289,    4475, 10, }, // Europe/Copenhagen
    {  331, 8929,   78287,  5, }, // Europe/Kirov
    {  331, 2527,   78292,  4, }, // Europe/Kyiv
    {  331, 3213,    4499,  8, }, // Europe/Lisbon
    {  331, 6365,    4513,  9, }, // Europe/Luxembourg
    {  331, 3537,   57485,  6, }, // Europe/Moscow
    {  331, 6275,    8201,  4, }, // Europe/Prague
    {  331, 6324,    6064,  3, }, // Europe/Rome
    {  331, 4451,   78296,  7, }, // Europe/Saratov
    {  331, 6202,   21213,  6, }, // Europe/Tirane
    {  331, 8882,   78303,  9, }, // Europe/Ulyanovsk
    {  331, 6567,    6096,  7, }, // Europe/Vatican
    {  331, 6231,   57515,  4, }, // Europe/Vienna
    {  331, 4727,    4561,  9, }, // Europe/Volgograd
    {  331, 3190,   57528,  8, }, // Europe/Warsaw
    {  331, 6540,    4577,  6, }, // Europe/Zurich
    {  331, 6046,   78312, 15, }, // Indian/Christmas
    {  331, 5029,   78327,  7, }, // Indian/Comoro
    {  331, 7050,   78334,  9, }, // Indian/Maldives
    {  331, 7211,     547,  7, }, // Indian/Reunion
    {  331, 3157,     554,  5, }, // Pacific/Chuuk
    {  331, 2237,   78343, 10, }, // Pacific/Easter
    {  331, 3902,   78353,  7, }, // Pacific/Fiji
    {  331, 2723,    4611,  8, }, // Pacific/Honolulu
    {  331, 3049,    4619,  9, }, // Pacific/Kanton
    {  331, 3096,     586,  7, }, // Pacific/Pohnpei
    {  337, 5989,   78133,  6, }, // Asia/Brunei German/Latin/Switzerland
    {  337, 1211,     464,  5, }, // Asia/Macau
    {  337, 4451,   78360,  7, }, // Europe/Saratov
    {  338, 6788,   78367,  8, }, // Africa/Abidjan Greek/Greek/Greece
    {  338, 6760,   78375,  4, }, // Africa/Accra
    {  338, 5059,   78379, 13, }, // Africa/Addis_Ababa
    {  338, 7420,   78392,  6, }, // Africa/Algiers
    {  338,   14,   78398,  6, }, // Africa/Asmara
    {  338,   44,   78404,  7, }, // Africa/Bamako
    {  338, 5257,   78411,  7, }, // Africa/Bangui
    {  338, 6746,   78418,  8, }, // Africa/Banjul
    {  338, 7435,   78426,  7, }, // Africa/Bissau
    {  338, 4957,   78433,  9, }, // Africa/Blantyre
    {  338, 5287,   78442,  9, }, // Africa/Brazzaville
    {  338, 4922,   78451, 13, }, // Africa/Bujumbura
    {  338, 2320,   78464,  5, }, // Africa/Cairo
    {  338, 4117,   78469, 11, }, // Africa/Casablanca
    {  338, 7449,   78480,  6, }, // Africa/Ceuta
    {  338, 6773,   78486,  7, }, // Africa/Conakry
    {  338, 6840,   78493,  6, }, // Africa/Dakar
    {  338, 5130,   78499, 13, }, // Africa/Dar_es_Salaam
    {  338, 5043,   78512,  9, }, // Africa/Djibouti
    {  338, 5243,   78521,  7, }, // Africa/Douala
    {  338, 5166,   78528,  9, }, // Africa/El_Aaiun
    {  338, 6853,   78537,  8, }, // Africa/Freetown
    {  338, 4906,   78545, 10, }, // Africa/Gaborone
    {  338, 5015,   78555,  6, }, // Africa/Harare
    {  338, 4479,   78561, 15, }, // Africa/Johannesburg
    {  338, 4499,   78576,  7, }, // Africa/Juba
    {  338, 5151,   78583,  7, }, // Africa/Kampala
    {  338, 4524,   78590,  7, }, // Africa/Khartoum
    {  338, 4973,   78597,  7, }, // Africa/Kigali
    {  338, 5306,   78604,  7, }, // Africa/Kinshasa
    {  338, 4744,   78611,  6, }, // Africa/Lagos
    {  338, 5336,   78617,  9, }, // Africa/Libreville
    {  338, 6869,   78626,  4, }, // Africa/Lome
    {  338, 5211,   78630,  7, }, // Africa/Luanda
    {  338, 4939,   78637, 12, }, // Africa/Lubumbashi
    {  338, 5001,   78649,  7, }, // Africa/Lusaka
    {  338, 5322,   78656,  7, }, // Africa/Malabo
    {  338, 4987,   78663,  7, }, // Africa/Maputo
    {  338, 5197,   78670,  7, }, // Africa/Maseru
    {  338, 5182,   78677,  8, }, // Africa/Mbabane
    {  338, 5113,   78685, 11, }, // Africa/Mogadishu
    {  338, 7034,   78696,  8, }, // Africa/Monrovia
    {  338, 3853,   78704,  8, }, // Africa/Nairobi
    {  338, 5271,   78712,  8, }, // Africa/Ndjamena
    {  338, 5354,   78720,  6, }, // Africa/Niamey
    {  338, 6803,   78726,  8, }, // Africa/Nouakchott
    {  338, 6727,   78734, 14, }, // Africa/Ouagadougou
    {  338, 5225,   78748, 10, }, // Africa/Porto-Novo
    {  338, 4435,   78758,  8, }, // Africa/Sao_Tome
    {  338, 2866,   78766,  7, }, // Africa/Tripoli
    {  338, 6554,   78773,  6, }, // Africa/Tunis
    {  338, 4152,   78779,  8, }, // Africa/Windhoek
    {  338,  132,   78787,  5, }, // America/Adak
    {  338, 3346,   78792,  8, }, // America/Anchorage
    {  338, 5607,   78800,  9, }, // America/Anguilla
    {  338, 5624,   78809,  9, }, // America/Antigua
    {  338, 4554,   78818, 11, }, // America/Araguaina
    {  338,  166,   78829, 14, }, // America/Argentina/Buenos_Aires
    {  338,   91,   78843,  9, }, // America/Argentina/Catamarca
    {  338,  270,   78852,  7, }, // America/Argentina/Cordoba
    {  338,  441,   78859,  7, }, // America/Argentina/Jujuy
    {  338, 7462,   78866,  8, }, // America/Argentina/La_Rioja
    {  338,  565,   78874,  7, }, // America/Argentina/Mendoza
    {  338, 7489,   78881, 14, }, // America/Argentina/Rio_Gallegos
    {  338, 7520,   78895,  5, }, // America/Argentina/Salta
    {  338, 7544,   78900,  9, }, // America/Argentina/San_Juan
    {  338, 5580,   78909,  9, }, // America/Argentina/San_Luis
    {  338, 7571,   78918,  9, }, // America/Argentina/Tucuman
    {  338, 7597,   78927,  8, }, // America/Argentina/Ushuaia
    {  338, 5640,   78935,  7, }, // America/Aruba
    {  338, 4252,   78942,  9, }, // America/Asuncion
    {  338,  237,   78951,  8, }, // America/Atikokan
    {  338, 3679,   78959,  5, }, // America/Bahia
    {  338, 7623,   78964, 19, }, // America/Bahia_Banderas
    {  338, 5654,   78983, 12, }, // America/Barbados
    {  338, 7646,   78995,  6, }, // America/Belem
    {  338, 5407,   79001,  6, }, // America/Belize
    {  338, 7660,   79007, 13, }, // America/Blanc-Sablon
    {  338, 7681,   79020, 10, }, // America/Boa_Vista
    {  338, 4361,   79030,  8, }, // America/Bogota
    {  338, 7699,   79038,  6, }, // America/Boise
    {  338, 7713,   79044, 13, }, // America/Cambridge_Bay
    {  338, 7735,   79057, 13, }, // America/Campo_Grande
    {  338, 3868,   79070,  7, }, // America/Cancun
    {  338, 4694,   79077,  7, }, // America/Caracas
    {  338, 4345,   79084,  6, }, // America/Cayenne
    {  338, 5496,   79090,  6, }, // America/Cayman
    {  338, 2260,   79096,  6, }, // America/Chicago
    {  338, 7756,   79102,  9, }, // America/Chihuahua
    {  338, 7774,   79111, 15, }, // America/Ciudad_Juarez
    {  338, 5422,   79126, 10, }, // America/Costa_Rica
    {  338, 7796,   79136,  7, }, // America/Creston
    {  338, 3791,   79143,  8, }, // America/Cuiaba
    {  338, 5723,   79151,  8, }, // America/Curacao
    {  338, 7812,   79159, 12, }, // America/Danmarkshavn
    {  338, 7833,   79171,  6, }, // America/Dawson
    {  338, 7848,   79177, 11, }, // America/Dawson_Creek
    {  338,  805,   79188,  7, }, // America/Denver
    {  338, 3465,   79195,  8, }, // America/Detroit
    {  338, 5739,   79203,  9, }, // America/Dominica
    {  338,  893,   79212,  9, }, // America/Edmonton
    {  338, 7869,   79221,  9, }, // America/Eirunepe
    {  338, 5441,   79230, 11, }, // America/El_Salvador
    {  338, 7886,   79241, 11, }, // America/Fort_Nelson
    {  338, 7906,   79252,  9, }, // America/Fortaleza
    {  338, 7924,   79261, 11, }, // America/Glace_Bay
    {  338, 6881,   79272, 10, }, // America/Goose_Bay
    {  338, 4612,   79282, 12, }, // America/Grand_Turk
    {  338, 5770,   79294,  7, }, // America/Grenada
    {  338, 5786,   79301, 11, }, // America/Guadeloupe
    {  338, 3760,   79312, 10, }, // America/Guatemala
    {  338, 4841,   79322, 11, }, // America/Guayaquil
    {  338, 6932,   79333,  7, }, // America/Guyana
    {  338, 1939,   79340,  7, }, // America/Halifax
    {  338, 2281,   79347,  5, }, // America/Havana
    {  338, 7942,   79352,  9, }, // America/Hermosillo
    {  338,  348,   79361, 12, }, // America/Indiana/Indianapolis
    {  338,  481,   79373, 12, }, // America/Indiana/Knox
    {  338, 7961,   79385, 17, }, // America/Indiana/Marengo
    {  338, 7985,   79402, 21, }, // America/Indiana/Petersburg
    {  338, 8012,   79423, 17, }, // America/Indiana/Tell_City
    {  338, 8038,   79440, 14, }, // America/Indiana/Vevay
    {  338, 8060,   79454, 15, }, // America/Indiana/Vincennes
    {  338, 8086,   79469, 18, }, // America/Indiana/Winamac
    {  338, 8110,   79487,  7, }, // America/Inuvik
    {  338,  660,   79494,  8, }, // America/Iqaluit
    {  338, 2799,   79502,  8, }, // America/Jamaica
    {  338, 5380,   79510,  6, }, // America/Juneau
    {  338,  521,   79516,  7, }, // America/Kentucky/Louisville
    {  338, 8125,   79523, 19, }, // America/Kentucky/Monticello
    {  338, 5704,   79542,  9, }, // America/Kralendijk
    {  338, 4376,   79551,  6, }, // America/La_Paz
    {  338, 7169,   79557,  4, }, // America/Lima
    {  338, 3239,   79561, 12, }, // America/Los_Angeles
    {  338, 5932,     207, 22, }, // America/Lower_Princes
    {  338, 8153,   79573,  6, }, // America/Maceio
    {  338, 8168,   79579,  9, }, // America/Managua
    {  338, 1908,   79588,  7, }, // America/Manaus
    {  338, 5897,   79595,  7, }, // America/Marigot
    {  338, 5805,   79602,  9, }, // America/Martinique
    {  338, 8184,   79611,  9, }, // America/Matamoros
    {  338, 2917,   79620,  8, }, // America/Mazatlan
    {  338, 8202,   79628,  8, }, // America/Menominee
    {  338, 8220,   79636,  6, }, // America/Merida
    {  338, 8235,   79642, 10, }, // America/Metlakatla
    {  338, 2949,   79652, 16, }, // America/Mexico_City
    {  338, 4391,   79668,  7, }, // America/Miquelon
    {  338, 8254,   79675,  7, }, // America/Moncton
    {  338, 8270,   79682,  8, }, // America/Monterrey
    {  338, 4098,   79690, 10, }, // America/Montevideo
    {  338, 5824,   79700,  8, }, // America/Montserrat
    {  338, 5481,   79708,  6, }, // America/Nassau
    {  338, 2379,   79714,  9, }, // America/New_York
    {  338, 8288,   79723,  4, }, // America/Nome
    {  338, 1850,   79727,  7, }, // America/Noronha
    {  338, 8301,   79734, 23, }, // America/North_Dakota/Beulah
    {  338, 8329,   79757, 22, }, // America/North_Dakota/Center
    {  338, 8357,   79779, 27, }, // America/North_Dakota/New_Salem
    {  338,  393,   79806,  6, }, // America/Nuuk
    {  338, 8388,   79812,  8, }, // America/Ojinaga
    {  338, 2356,   79820,  7, }, // America/Panama
    {  338, 6173,   79827, 11, }, // America/Paramaribo
    {  338, 2973,   79838,  6, }, // America/Phoenix
    {  338, 3945,   79844, 12, }, // America/Port-au-Prince
    {  338, 5954,   79856, 13, }, // America/Port_of_Spain
    {  338, 8404,   79869, 11, }, // America/Porto_Velho
    {  338, 5843,   79880, 12, }, // America/Puerto_Rico
    {  338, 4030,   79892, 13, }, // America/Punta_Arenas
    {  338, 8424,   79905, 12, }, // America/Rankin_Inlet
    {  338, 8445,   79917,  6, }, // America/Recife
    {  338, 1995,   79923,  8, }, // America/Regina
    {  338, 8460,   79931,  8, }, // America/Resolute
    {  338,  695,   79939, 11, }, // America/Rio_Branco
    {  338, 8477,   79950,  8, }, // America/Santarem
    {  338, 2201,   79958,  9, }, // America/Santiago
    {  338, 6111,   79967, 15, }, // America/Santo_Domingo
    {  338, 1878,   79982,  9, }, // America/Sao_Paulo
    {  338, 6899,   79991, 14, }, // America/Scoresbysund
    {  338, 8494,   80005,  5, }, // America/Sitka
    {  338, 8508,   80010, 18, }, // America/St_Barthelemy
    {  338, 2061,   80028,  9, }, // America/St_Johns
    {  338, 5863,   80037,  8, }, // America/St_Kitts
    {  338, 5880,   80045, 11, }, // America/St_Lucia
    {  338,  855,   80056, 11, }, // America/St_Thomas
    {  338, 5913,   80067, 15, }, // America/St_Vincent
    {  338, 8530,   80082, 13, }, // America/Swift_Current
    {  338, 5461,   80095, 14, }, // America/Tegucigalpa
    {  338, 5756,   80109,  5, }, // America/Thule
    {  338,  313,   80114,  8, }, // America/Tijuana
    {  338,  608,   80122,  7, }, // America/Toronto
    {  338, 5688,   80129,  7, }, // America/Tortola
    {  338, 2093,   80136,  9, }, // America/Vancouver
    {  338, 2144,   80145, 10, }, // America/Whitehorse
    {  338,  734,   80155, 10, }, // America/Winnipeg
    {  338, 8552,   80165,  9, }, // America/Yakutat
    {  338, 6001,   80174,  5, }, // Antarctica/Casey
    {  338, 6094,   80179,  7, }, // Antarctica/Davis
    {  338, 6133,   80186, 17, }, // Antarctica/DumontDUrville
    {  338, 8568,   80203,  8, }, // Antarctica/Macquarie
    {  338, 7066,   80211,  5, }, // Antarctica/Mawson
    {  338, 7113,   80216,  9, }, // Antarctica/McMurdo
    {  338, 8589,   80225,  6, }, // Antarctica/Palmer
    {  338, 7226,   80231,  6, }, // Antarctica/Rothera
    {  338, 7295,   80237,  5, }, // Antarctica/Syowa
    {  338, 8607,   80242,  4, }, // Antarctica/Troll
    {  338, 7374,   80246,  6, }, // Antarctica/Vostok
    {  338, 1429,   80252, 14, }, // Arctic/Longyearbyen
    {  338, 5570,   80266,  5, }, // Asia/Aden
    {  338, 5395,   80271,  6, }, // Asia/Almaty
    {  338, 3968,   80277,  5, }, // Asia/Amman
    {  338, 5511,   80282,  7, }, // Asia/Anadyr
    {  338, 5523,   80289,  6, }, // Asia/Aqtau
    {  338, 5368,   80295,  7, }, // Asia/Aqtobe
    {  338,  964,   80302,  8, }, // Asia/Ashgabat
    {  338, 8624,   80310,  7, }, // Asia/Atyrau
    {  338, 3607,   80317,  7, }, // Asia/Baghdad
    {  338, 5534,   80324,  8, }, // Asia/Bahrain
    {  338, 3653,   80332,  6, }, // Asia/Baku
    {  338, 4466,   80338,  8, }, // Asia/Bangkok
    {  338, 3571,   80346,  9, }, // Asia/Barnaul
    {  338, 4086,   80355,  7, }, // Asia/Beirut
    {  338, 3778,   80362,  7, }, // Asia/Bishkek
    {  338, 5989,   80369,  8, }, // Asia/Brunei
    {  338, 4601,   80377,  5, }, // Asia/Chita
    {  338, 4511,   80382,  7, }, // Asia/Colombo
    {  338, 4540,   80389,  8, }, // Asia/Damascus
    {  338, 1093,   80397,  5, }, // Asia/Dhaka
    {  338, 6192,   80402,  5, }, // Asia/Dili
    {  338, 3596,   80407,  8, }, // Asia/Dubai
    {  338, 6159,   80415, 10, }, // Asia/Dushanbe
    {  338, 8636,   80425, 10, }, // Asia/Famagusta
    {  338, 8651,   80435,  4, }, // Asia/Gaza
    {  338, 4795,   80439,  7, }, // Asia/Hebron
    {  338, 1259,   80446, 16, }, // Asia/Ho_Chi_Minh
    {  338, 2704,   80462, 11, }, // Asia/Hong_Kong
    {  338, 4771,   80473,  5, }, // Asia/Hovd
    {  338, 4184,   80478,  8, }, // Asia/Irkutsk
    {  338, 7006,   80486,  8, }, // Asia/Jakarta
    {  338, 6992,   80494, 11, }, // Asia/Jayapura
    {  338, 1290,   80505, 10, }, // Asia/Jerusalem
    {  338, 3560,   80515,  7, }, // Asia/Kabul
    {  338, 4316,   80522,  9, }, // Asia/Kamchatka
    {  338, 4239,   80531,  7, }, // Asia/Karachi
    {  338, 1185,   80538,  9, }, // Asia/Kathmandu
    {  338, 8661,   80547,  8, }, // Asia/Khandyga
    {  338,  992,   80555,  8, }, // Asia/Kolkata
    {  338, 4197,   80563, 12, }, // Asia/Krasnoyarsk
    {  338, 4859,   80575, 15, }, // Asia/Kuala_Lumpur
    {  338, 5976,   80590,  9, }, // Asia/Kuching
    {  338, 5547,   80599,  7, }, // Asia/Kuwait
    {  338, 1211,   80606,  5, }, // Asia/Macau
    {  338, 4017,   80611,  9, }, // Asia/Magadan
    {  338, 1349,   80620,  7, }, // Asia/Makassar
    {  338, 7182,   80627,  6, }, // Asia/Manila
    {  338, 6920,   80633,  6, }, // Asia/Muscat
    {  338, 2554,   80639,  8, }, // Asia/Nicosia
    {  338, 8675,   80647, 13, }, // Asia/Novokuznetsk
    {  338, 4135,   80660, 12, }, // Asia/Novosibirsk
    {  338, 4229,   80672,  4, }, // Asia/Omsk
    {  338, 7145,   80676,  4, }, // Asia/Oral
    {  338, 6961,   80680,  9, }, // Asia/Phnom_Penh
    {  338, 8693,   80689,  9, }, // Asia/Pontianak
    {  338, 4214,   80698, 12, }, // Asia/Pyongyang
    {  338, 5559,   80710,  5, }, // Asia/Qatar
    {  338, 8708,   80715,  8, }, // Asia/Qostanay
    {  338, 4269,   80723, 10, }, // Asia/Qyzylorda
    {  338, 3584,   80733,  5, }, // Asia/Riyadh
    {  338, 4408,   80738,  8, }, // Asia/Sakhalin
    {  338, 7245,   80746, 10, }, // Asia/Samarkand
    {  338, 3279,   80756,  5, }, // Asia/Seoul
    {  338, 1053,   80761,  7, }, // Asia/Shanghai
    {  338, 3300,   80768, 10, }, // Asia/Singapore
    {  338, 4297,   80778, 14, }, // Asia/Srednekolymsk
    {  338, 3263,   80792,  6, }, // Asia/Taipei
    {  338, 4781,   80798,  8, }, // Asia/Tashkent
    {  338, 3915,   80806,  7, }, // Asia/Tbilisi
    {  338, 2772,   80813,  8, }, // Asia/Tehran
    {  338, 1317,   80821,  6, }, // Asia/Thimphu
    {  338, 2821,   80827,  5, }, // Asia/Tokyo
    {  338, 4572,   80832,  5, }, // Asia/Tomsk
    {  338, 1021,   80837, 12, }, // Asia/Ulaanbaatar
    {  338, 1159,   80849,  9, }, // Asia/Urumqi
    {  338, 8722,   80858,  9, }, // Asia/Ust-Nera
    {  338, 6977,   80867,  8, }, // Asia/Vientiane
    {  338, 4710,   80875, 11, }, // Asia/Vladivostok
    {  338, 4828,   80886,  9, }, // Asia/Yakutsk
    {  338, 1235,   80895,  8, }, // Asia/Yangon
    {  338, 3883,   80903, 15, }, // Asia/Yekaterinburg
    {  338, 3747,   80918,  6, }, // Asia/Yerevan
    {  338, 3663,   80924,  6, }, // Atlantic/Azores
    {  338, 5671,   80930,  9, }, // Atlantic/Bermuda
    {  338, 6628,   80939,  7, }, // Atlantic/Canary
    {  338, 3727,   80946, 17, }, // Atlantic/Cape_Verde
    {  338, 1395,   80963,  6, }, // Atlantic/Faroe
    {  338, 8736,   80969,  6, }, // Atlantic/Madeira
    {  338, 2748,   80975,  9, }, // Atlantic/Reykjavik
    {  338, 7272,   80984, 13, }, // Atlantic/South_Georgia
    {  338, 6821,   80997,  9, }, // Atlantic/St_Helena
    {  338, 6644,   81006,  7, }, // Atlantic/Stanley
    {  338, 1670,   81013,  8, }, // Australia/Adelaide
    {  338, 1635,   81021, 10, }, // Australia/Brisbane
    {  338, 1799,   81031, 11, }, // Australia/Broken_Hill
    {  338, 1583,   81042,  9, }, // Australia/Darwin
    {  338, 3637,   81051,  7, }, // Australia/Eucla
    {  338, 1516,   81058,  7, }, // Australia/Hobart
    {  338, 8753,   81065,  8, }, // Australia/Lindeman
    {  338, 1547,   81073, 10, }, // Australia/Lord_Howe
    {  338, 1727,   81083,  9, }, // Australia/Melbourne
    {  338, 1762,   81092,  4, }, // Australia/Perth
    {  338, 1463,   81096,  6, }, // Australia/Sydney
    {  338, 6441,   81102, 10, }, // Europe/Amsterdam
    {  338, 6216,   81112,  6, }, // Europe/Andorra
    {  338, 3620,   81118,  8, }, // Europe/Astrakhan
    {  338, 2300,   81126,  5, }, // Europe/Athens
    {  338, 6336,   81131,  9, }, // Europe/Belgrade
    {  338, 4757,   81140,  8, }, // Europe/Berlin
    {  338, 6488,   81148, 11, }, // Europe/Bratislava
    {  338, 2167,   81159,  9, }, // Europe/Brussels
    {  338, 3928,   81168, 11, }, // Europe/Bucharest
    {  338, 3806,   81179, 10, }, // Europe/Budapest
    {  338, 8772,   81189, 10, }, // Europe/Busingen
    {  338, 2583,   81199,  8, }, // Europe/Chisinau
    {  338, 6289,   81207,  9, }, // Europe/Copenhagen
    {  338, 2338,   81216,  8, }, // Europe/Dublin
    {  338, 6307,   81224,  9, }, // Europe/Gibraltar
    {  338, 8788,   81233,  7, }, // Europe/Guernsey
    {  338, 6612,   81240,  7, }, // Europe/Helsinki
    {  338, 8804,   81247, 13, }, // Europe/Isle_of_Man
    {  338, 1130,   81260, 16, }, // Europe/Istanbul
    {  338, 8823,   81276,  7, }, // Europe/Jersey
    {  338, 3979,   81283, 13, }, // Europe/Kaliningrad
    {  338, 8929,   81296,  5, }, // Europe/Kirov
    {  338, 2527,   81301,  5, }, // Europe/Kyiv
    {  338, 3213,   81306,  8, }, // Europe/Lisbon
    {  338, 6506,   81314, 11, }, // Europe/Ljubljana
    {  338, 2501,   81325,  7, }, // Europe/London
    {  338, 6365,   81332, 12, }, // Europe/Luxembourg
    {  338, 4892,   81344,  7, }, // Europe/Madrid
    {  338, 6397,   81351,  5, }, // Europe/Malta
    {  338, 6582,   81356,  9, }, // Europe/Mariehamn
    {  338, 3693,   81365,  5, }, // Europe/Minsk
    {  338, 6410,   81370,  6, }, // Europe/Monaco
    {  338, 3537,   81376,  5, }, // Europe/Moscow
    {  338, 6458,   81381,  4, }, // Europe/Oslo
    {  338, 4284,   81385,  6, }, // Europe/Paris
    {  338, 6424,   81391, 12, }, // Europe/Podgorica
    {  338, 6275,   81403,  5, }, // Europe/Prague
    {  338, 8837,   81408,  4, }, // Europe/Riga
    {  338, 6324,   81412,  4, }, // Europe/Rome
    {  338, 4331,   81416,  6, }, // Europe/Samara
    {  338, 6470,   81422, 13, }, // Europe/San_Marino
    {  338, 6245,   81435,  8, }, // Europe/Sarajevo
    {  338, 4451,   81443,  7, }, // Europe/Saratov
    {  338, 8849,   81450, 11, }, // Europe/Simferopol
    {  338, 6383,   81461,  6, }, // Europe/Skopje
    {  338, 6599,   81467,  5, }, // Europe/Sofia
    {  338, 6523,   81472,  9, }, // Europe/Stockholm
    {  338, 8867,   81481,  5, }, // Europe/Tallinn
    {  338, 6202,   81486,  6, }, // Europe/Tirane
    {  338, 8882,   81492, 10, }, // Europe/Ulyanovsk
    {  338, 6352,   81502,  7, }, // Europe/Vaduz
    {  338, 6567,   81509,  8, }, // Europe/Vatican
    {  338, 6231,   81517,  6, }, // Europe/Vienna
    {  338, 8899,   81523,  8, }, // Europe/Vilnius
    {  338, 4727,   81531, 11, }, // Europe/Volgograd
    {  338, 3190,   81542,  8, }, // Europe/Warsaw
    {  338, 6261,   81550,  8, }, // Europe/Zagreb
    {  338, 6540,   81558,  6, }, // Europe/Zurich
    {  338, 5078,   81564, 12, }, // Indian/Antananarivo
    {  338, 6947,   81576,  7, }, // Indian/Chagos
    {  338, 6046,   81583, 19, }, // Indian/Christmas
    {  338, 6063,   81602,  5, }, // Indian/Cocos
    {  338, 5029,   81607,  6, }, // Indian/Comoro
    {  338, 6661,   81613,  9, }, // Indian/Kerguelen
    {  338, 7260,   81622,  4, }, // Indian/Mahe
    {  338, 7050,   81626,  8, }, // Indian/Maldives
    {  338, 4069,   81634,  9, }, // Indian/Mauritius
    {  338, 5098,   81643,  6, }, // Indian/Mayotte
    {  338, 7211,   81649,  7, }, // Indian/Reunion
    {  338, 4422,   81656,  4, }, // Pacific/Apia
    {  338,  932,   81660,  6, }, // Pacific/Auckland
    {  338, 3706,   81666, 11, }, // Pacific/Bougainville
    {  338, 3015,   81677,  6, }, // Pacific/Chatham
    {  338, 3157,   81683,  5, }, // Pacific/Chuuk
    {  338, 2237,   81688, 11, }, // Pacific/Easter
    {  338, 7360,   81699,  5, }, // Pacific/Efate
    {  338, 7327,   81704,  7, }, // Pacific/Fakaofo
    {  338, 3902,   81711,  5, }, // Pacific/Fiji
    {  338, 7343,   81716, 10, }, // Pacific/Funafuti
    {  338, 6678,   81726, 11, }, // Pacific/Galapagos
    {  338, 6696,   81737,  8, }, // Pacific/Gambier
    {  338, 3822,   81745, 14, }, // Pacific/Guadalcanal
    {  338, 6018,   81759,  6, }, // Pacific/Guam
    {  338, 2723,   81765, 10, }, // Pacific/Honolulu
    {  338, 3049,   81775, 10, }, // Pacific/Kanton
    {  338, 3998,   81785, 10, }, // Pacific/Kiritimati
    {  338, 7019,   81795,  6, }, // Pacific/Kosrae
    {  338, 2842,   81801, 11, }, // Pacific/Kwajalein
    {  338, 4877,   81812,  8, }, // Pacific/Majuro
    {  338, 4051,   81820,  8, }, // Pacific/Marquesas
    {  338, 8914,   81828,  9, }, // Pacific/Midway
    {  338, 7084,   81837,  7, }, // Pacific/Nauru
    {  338, 7132,   81844,  5, }, // Pacific/Niue
    {  338, 4168,   81849,  7, }, // Pacific/Norfolk
    {  338, 7098,   81856,  6, }, // Pacific/Noumea
    {  338, 3126,   81862, 11, }, // Pacific/Pago_Pago
    {  338, 7155,   81873,  6, }, // Pacific/Palau
    {  338, 7194,   81879,  7, }, // Pacific/Pitcairn
    {  338, 3096,   81886,  6, }, // Pacific/Pohnpei
    {  338, 4807,   81892, 13, }, // Pacific/Port_Moresby
    {  338, 6076,   81905, 10, }, // Pacific/Rarotonga
    {  338, 6031,   81915,  6, }, // Pacific/Saipan
    {  338, 7312,   81921,  5, }, // Pacific/Tahiti
    {  338, 6712,   81926,  7, }, // Pacific/Tarawa
    {  338, 4583,   81933, 11, }, // Pacific/Tongatapu
    {  338, 7392,   81944,  6, }, // Pacific/Wake
    {  338, 7405,   81950,  7, }, // Pacific/Wallis
    {  340,   14,       0,  6, }, // Africa/Asmara Guarani/Latin/Paraguay
    {  340, 4435,       6,  8, }, // Africa/Sao_Tome
    {  340, 4252,      14,  8, }, // America/Asuncion
    {  340,  237,      22,  8, }, // America/Atikokan
    {  340, 7623,      30, 17, }, // America/Bahia_Banderas
    {  340, 3868,      47,  6, }, // America/Cancun
    {  340, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  340, 5723,      66,  7, }, // America/Curacao
    {  340,  481,      73, 13, }, // America/Indiana/Knox
    {  340, 7961,      86, 16, }, // America/Indiana/Marengo
    {  340, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  340, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  340, 8038,     139, 14, }, // America/Indiana/Vevay
    {  340, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  340, 8086,     171, 16, }, // America/Indiana/Winamac
    {  340, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  340, 5932,     207, 22, }, // America/Lower_Princes
    {  340, 8220,     229,  6, }, // America/Merida
    {  340, 1850,     235, 19, }, // America/Noronha
    {  340, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  340, 8329,     274, 20, }, // America/North_Dakota/Center
    {  340, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  340,  393,     317,  4, }, // America/Nuuk
    {  340, 6899,     321, 16, }, // America/Scoresbysund
    {  340, 8508,     337, 14, }, // America/St_Barthelemy
    {  340, 2061,     351, 10, }, // America/St_Johns
    {  340, 5863,     361,  9, }, // America/St_Kitts
    {  340, 5880,     370,  9, }, // America/St_Lucia
    {  340,  855,     379, 10, }, // America/St_Thomas
    {  340, 5913,     389, 11, }, // America/St_Vincent
    {  340, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  340, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  340, 1185,     448,  9, }, // Asia/Kathmandu
    {  340,  992,     457,  7, }, // Asia/Kolkata
    {  340, 1211,     464,  5, }, // Asia/Macau
    {  340, 1235,     477,  6, }, // Asia/Yangon
    {  340, 1395,     483,  5, }, // Atlantic/Faroe
    {  340, 6821,     488, 10, }, // Atlantic/St_Helena
    {  340, 2527,     514,  4, }, // Europe/Kyiv
    {  340, 7211,     547,  7, }, // Indian/Reunion
    {  340, 3157,     554,  5, }, // Pacific/Chuuk
    {  340, 3096,     586,  7, }, // Pacific/Pohnpei
    {  341, 6788,   81957,  8, }, // Africa/Abidjan Gujarati/Gujarati/India
    {  341, 6760,   81965,  4, }, // Africa/Accra
    {  341, 5059,   81969, 10, }, // Africa/Addis_Ababa
    {  341, 7420,   81979,  9, }, // Africa/Algiers
    {  341,   14,   81988,  7, }, // Africa/Asmara
    {  341,   44,   81995,  5, }, // Africa/Bamako
    {  341, 5257,   82000,  6, }, // Africa/Bangui
    {  341, 6746,   82006,  6, }, // Africa/Banjul
    {  341, 7435,   82012,  5, }, // Africa/Bissau
    {  341, 4957,   82017,  8, }, // Africa/Blantyre
    {  341, 5287,   82025, 10, }, // Africa/Brazzaville
    {  341, 4922,   82035,  9, }, // Africa/Bujumbura
    {  341, 2320,   82044,  4, }, // Africa/Cairo
    {  341, 4117,   82048, 11, }, // Africa/Casablanca
    {  341, 7449,   82059,  6, }, // Africa/Ceuta
    {  341, 6773,   82065,  8, }, // Africa/Conakry
    {  341, 6840,   82073,  4, }, // Africa/Dakar
    {  341, 5130,   82077, 11, }, // Africa/Dar_es_Salaam
    {  341, 5043,   82088,  6, }, // Africa/Djibouti
    {  341, 5243,   82094,  5, }, // Africa/Douala
    {  341, 5166,   82099,  6, }, // Africa/El_Aaiun
    {  341, 6853,   82105,  8, }, // Africa/Freetown
    {  341, 4906,   82113,  7, }, // Africa/Gaborone
    {  341, 5015,   82120,  5, }, // Africa/Harare
    {  341, 4479,   82125, 11, }, // Africa/Johannesburg
    {  341, 4499,   82136,  4, }, // Africa/Juba
    {  341, 5151,   82140,  7, }, // Africa/Kampala
    {  341, 4524,   82147,  7, }, // Africa/Khartoum
    {  341, 4973,   82154,  6, }, // Africa/Kigali
    {  341, 5306,   82160,  7, }, // Africa/Kinshasa
    {  341, 4744,   82167,  5, }, // Africa/Lagos
    {  341, 5336,   82172, 10, }, // Africa/Libreville
    {  341, 6869,   82182,  3, }, // Africa/Lome
    {  341, 5211,   82185,  9, }, // Africa/Luanda
    {  341, 4939,   82194, 10, }, // Africa/Lubumbashi
    {  341, 5001,   82204,  6, }, // Africa/Lusaka
    {  341, 5322,   82210,  5, }, // Africa/Malabo
    {  341, 4987,   82215,  6, }, // Africa/Maputo
    {  341, 5197,   82221,  5, }, // Africa/Maseru
    {  341, 5182,   82226,  7, }, // Africa/Mbabane
    {  341, 5113,   82233,  8, }, // Africa/Mogadishu
    {  341, 7034,   82241,  9, }, // Africa/Monrovia
    {  341, 3853,   82250,  6, }, // Africa/Nairobi
    {  341, 5271,   82256,  7, }, // Africa/Ndjamena
    {  341, 5354,   82263,  6, }, // Africa/Niamey
    {  341, 6803,   82269,  6, }, // Africa/Nouakchott
    {  341, 6727,   82275,  8, }, // Africa/Ouagadougou
    {  341, 5225,   82283, 11, }, // Africa/Porto-Novo
    {  341, 4435,   82294,  7, }, // Africa/Sao_Tome
    {  341, 2866,   82301,  8, }, // Africa/Tripoli
    {  341, 6554,   82309,  7, }, // Africa/Tunis
    {  341, 4152,   82316,  7, }, // Africa/Windhoek
    {  341,  132,   82323,  3, }, // America/Adak
    {  341, 3346,   82326,  8, }, // America/Anchorage
    {  341, 5607,   82334,  8, }, // America/Anguilla
    {  341, 5624,   82342,  8, }, // America/Antigua
    {  341, 4554,   82350,  8, }, // America/Araguaina
    {  341,  166,   82358, 12, }, // America/Argentina/Buenos_Aires
    {  341,   91,   82370, 10, }, // America/Argentina/Catamarca
    {  341,  270,   82380,  8, }, // America/Argentina/Cordoba
    {  341,  441,   82388,  5, }, // America/Argentina/Jujuy
    {  341, 7462,   82393,  8, }, // America/Argentina/La_Rioja
    {  341,  565,   82401,  8, }, // America/Argentina/Mendoza
    {  341, 7489,   82409, 11, }, // America/Argentina/Rio_Gallegos
    {  341, 7520,   82420,  6, }, // America/Argentina/Salta
    {  341, 7544,   82426,  8, }, // America/Argentina/San_Juan
    {  341, 5580,   82434,  8, }, // America/Argentina/San_Luis
    {  341, 7571,   82442,  7, }, // America/Argentina/Tucuman
    {  341, 7597,   82449,  5, }, // America/Argentina/Ushuaia
    {  341, 5640,   82454,  5, }, // America/Aruba
    {  341, 4252,   82459,  8, }, // America/Asuncion
    {  341,  237,   82467,  8, }, // America/Atikokan
    {  341, 3679,   82475,  6, }, // America/Bahia
    {  341, 7623,   82481, 14, }, // America/Bahia_Banderas
    {  341, 5654,   82495,  8, }, // America/Barbados
    {  341, 7646,   82503,  5, }, // America/Belem
    {  341, 5407,   82508,  5, }, // America/Belize
    {  341, 7660,   82513, 13, }, // America/Blanc-Sablon
    {  341, 7681,   82526, 10, }, // America/Boa_Vista
    {  341, 4361,   82536,  6, }, // America/Bogota
    {  341, 7699,   82542,  4, }, // America/Boise
    {  341, 7713,   82546, 12, }, // America/Cambridge_Bay
    {  341, 7735,   82558, 13, }, // America/Campo_Grande
    {  341, 3868,   82571,  7, }, // America/Cancun
    {  341, 4694,   82578,  6, }, // America/Caracas
    {  341, 4345,   82584,  6, }, // America/Cayenne
    {  341, 5496,   82590,  5, }, // America/Cayman
    {  341, 2260,   82595,  6, }, // America/Chicago
    {  341, 7756,   82601,  8, }, // America/Chihuahua
    {  341, 7774,   82609, 14, }, // America/Ciudad_Juarez
    {  341, 5422,   82623, 11, }, // America/Costa_Rica
    {  341, 7796,   82634,  8, }, // America/Creston
    {  341, 3791,   82642,  7, }, // America/Cuiaba
    {  341, 5723,   82649,  7, }, // America/Curacao
    {  341, 7812,   82656, 11, }, // America/Danmarkshavn
    {  341, 7833,   82667,  4, }, // America/Dawson
    {  341, 7848,   82671, 10, }, // America/Dawson_Creek
    {  341,  805,   82681,  6, }, // America/Denver
    {  341, 3465,   82687,  8, }, // America/Detroit
    {  341, 5739,   82695,  8, }, // America/Dominica
    {  341,  893,   82703,  8, }, // America/Edmonton
    {  341, 7869,   82711,  6, }, // America/Eirunepe
    {  341, 5441,   82717, 12, }, // America/El_Salvador
    {  341, 7886,   82729, 12, }, // America/Fort_Nelson
    {  341, 7906,   82741, 10, }, // America/Fortaleza
    {  341, 7924,   82751,  8, }, // America/Glace_Bay
    {  341, 6881,   82759,  6, }, // America/Goose_Bay
    {  341, 4612,   82765, 12, }, // America/Grand_Turk
    {  341, 5770,   82777,  8, }, // America/Grenada
    {  341, 5786,   82785,  9, }, // America/Guadeloupe
    {  341, 3760,   82794, 10, }, // America/Guatemala
    {  341, 4841,   82804,  9, }, // America/Guayaquil
    {  341, 6932,   82813,  5, }, // America/Guyana
    {  341, 1939,   82818,  9, }, // America/Halifax
    {  341, 2281,   82827,  5, }, // America/Havana
    {  341, 7942,   82832,  8, }, // America/Hermosillo
    {  341,  348,   82840, 14, }, // America/Indiana/Indianapolis
    {  341,  481,   82854, 16, }, // America/Indiana/Knox
    {  341, 7961,   82870, 18, }, // America/Indiana/Marengo
    {  341, 7985,   82888, 21, }, // America/Indiana/Petersburg
    {  341, 8012,   82909, 19, }, // America/Indiana/Tell_City
    {  341, 8038,   82928, 15, }, // America/Indiana/Vevay
    {  341, 8060,   82943, 20, }, // America/Indiana/Vincennes
    {  341, 8086,   82963, 18, }, // America/Indiana/Winamac
    {  341, 8110,   82981,  6, }, // America/Inuvik
    {  341,  660,   82987,  7, }, // America/Iqaluit
    {  341, 2799,   82994,  5, }, // America/Jamaica
    {  341, 5380,   82999,  5, }, // America/Juneau
    {  341,  521,   83004,  6, }, // America/Kentucky/Louisville
    {  341, 8125,   83010, 20, }, // America/Kentucky/Monticello
    {  341, 5704,   83030, 11, }, // America/Kralendijk
    {  341, 4376,   83041,  6, }, // America/La_Paz
    {  341, 7169,   83047,  4, }, // America/Lima
    {  341, 3239,   83051, 11, }, // America/Los_Angeles
    {  341, 5932,   83062, 24, }, // America/Lower_Princes
    {  341, 8153,   83086,  5, }, // America/Maceio
    {  341, 8168,   83091,  7, }, // America/Managua
    {  341, 1908,   83098,  4, }, // America/Manaus
    {  341, 5897,   83102,  7, }, // America/Marigot
    {  341, 5805,   83109,  9, }, // America/Martinique
    {  341, 8184,   83118,  9, }, // America/Matamoros
    {  341, 2917,   83127,  7, }, // America/Mazatlan
    {  341, 8202,   83134,  8, }, // America/Menominee
    {  341, 8220,   83142,  6, }, // America/Merida
    {  341, 8235,   83148, 12, }, // America/Metlakatla
    {  341, 2949,   83160, 13, }, // America/Mexico_City
    {  341, 4391,   83173,  9, }, // America/Miquelon
    {  341, 8254,   83182,  7, }, // America/Moncton
    {  341, 8270,   83189,  8, }, // America/Monterrey
    {  341, 4098,   83197, 10, }, // America/Montevideo
    {  341, 5824,   83207,  9, }, // America/Montserrat
    {  341, 5481,   83216,  5, }, // America/Nassau
    {  341, 2379,   83221,  9, }, // America/New_York
    {  341, 8288,   83136,  3, }, // America/Nome
    {  341, 1850,   83230,  8, }, // America/Noronha
    {  341, 8301,   83238, 21, }, // America/North_Dakota/Beulah
    {  341, 8329,   83259, 20, }, // America/North_Dakota/Center
    {  341, 8357,   83279, 23, }, // America/North_Dakota/New_Salem
    {  341,  393,   83302,  3, }, // America/Nuuk
    {  341, 8388,   83305,  7, }, // America/Ojinaga
    {  341, 2356,   83312,  5, }, // America/Panama
    {  341, 6173,   83317, 10, }, // America/Paramaribo
    {  341, 2973,   83327,  7, }, // America/Phoenix
    {  341, 3945,   83334, 15, }, // America/Port-au-Prince
    {  341, 5954,   83349, 14, }, // America/Port_of_Spain
    {  341, 8404,   83363, 13, }, // America/Porto_Velho
    {  341, 5843,   83376, 14, }, // America/Puerto_Rico
    {  341, 4030,   83390, 13, }, // America/Punta_Arenas
    {  341, 8424,   83403, 13, }, // America/Rankin_Inlet
    {  341, 8445,   83416,  5, }, // America/Recife
    {  341, 1995,   83421,  6, }, // America/Regina
    {  341, 8460,   83427,  7, }, // America/Resolute
    {  341,  695,   83434, 12, }, // America/Rio_Branco
    {  341, 8477,   83446,  9, }, // America/Santarem
    {  341, 2201,   83455,  9, }, // America/Santiago
    {  341, 6111,   83464, 13, }, // America/Santo_Domingo
    {  341, 1878,   83477,  9, }, // America/Sao_Paulo
    {  341, 6899,   83486, 14, }, // America/Scoresbysund
    {  341, 8494,   83500,  6, }, // America/Sitka
    {  341, 8508,   83506, 15, }, // America/St_Barthelemy
    {  341, 2061,   83521, 12, }, // America/St_Johns
    {  341, 5863,   83533, 10, }, // America/St_Kitts
    {  341, 5880,   83543, 11, }, // America/St_Lucia
    {  341,  855,   83554,  9, }, // America/St_Thomas
    {  341, 5913,   83563, 14, }, // America/St_Vincent
    {  341, 8530,   83577, 12, }, // America/Swift_Current
    {  341, 5461,   83589, 11, }, // America/Tegucigalpa
    {  341, 5756,   83600,  4, }, // America/Thule
    {  341,  313,   83604,  7, }, // America/Tijuana
    {  341,  608,   83611,  7, }, // America/Toronto
    {  341, 5688,   83618,  8, }, // America/Tortola
    {  341, 2093,   83626,  8, }, // America/Vancouver
    {  341, 2144,   83634, 11, }, // America/Whitehorse
    {  341,  734,   83645,  9, }, // America/Winnipeg
    {  341, 8552,   83654,  5, }, // America/Yakutat
    {  341, 6001,   83659,  4, }, // Antarctica/Casey
    {  341, 6094,   83663,  5, }, // Antarctica/Davis
    {  341, 6133,   83668, 17, }, // Antarctica/DumontDUrville
    {  341, 8568,   83685,  8, }, // Antarctica/Macquarie
    {  341, 7066,   83693,  4, }, // Antarctica/Mawson
    {  341, 7113,   83697,  8, }, // Antarctica/McMurdo
    {  341, 8589,   83705,  6, }, // Antarctica/Palmer
    {  341, 7226,   83711,  6, }, // Antarctica/Rothera
    {  341, 7295,   83717,  6, }, // Antarctica/Syowa
    {  341, 8607,   83723,  5, }, // Antarctica/Troll
    {  341, 7374,   83728,  7, }, // Antarctica/Vostok
    {  341, 1429,   83735, 12, }, // Arctic/Longyearbyen
    {  341, 5570,   83747,  4, }, // Asia/Aden
    {  341, 5395,   83751,  7, }, // Asia/Almaty
    {  341, 3968,   83758,  6, }, // Asia/Amman
    {  341, 5511,   83764,  5, }, // Asia/Anadyr
    {  341, 5523,   83769,  5, }, // Asia/Aqtau
    {  341, 5368,   83774,  6, }, // Asia/Aqtobe
    {  341,  964,   83780,  6, }, // Asia/Ashgabat
    {  341, 8624,   83786,  7, }, // Asia/Atyrau
    {  341, 3607,   83793,  5, }, // Asia/Baghdad
    {  341, 5534,   83798,  6, }, // Asia/Bahrain
    {  341, 3653,   82647,  4, }, // Asia/Baku
    {  341, 4466,   83804,  7, }, // Asia/Bangkok
    {  341, 3571,   83811,  6, }, // Asia/Barnaul
    {  341, 4086,   83817,  5, }, // Asia/Beirut
    {  341, 3778,   83822,  6, }, // Asia/Bishkek
    {  341, 5989,   83828,  7, }, // Asia/Brunei
    {  341, 4601,   83835,  4, }, // Asia/Chita
    {  341, 4511,   83839,  6, }, // Asia/Colombo
    {  341, 4540,   83845,  7, }, // Asia/Damascus
    {  341, 1093,   83852,  4, }, // Asia/Dhaka
    {  341, 6192,   83856,  4, }, // Asia/Dili
    {  341, 3596,   83860,  4, }, // Asia/Dubai
    {  341, 6159,   83864,  8, }, // Asia/Dushanbe
    {  341, 8636,   83872, 10, }, // Asia/Famagusta
    {  341, 8651,   83882,  4, }, // Asia/Gaza
    {  341, 4795,   83886,  7, }, // Asia/Hebron
    {  341, 1259,   83893, 14, }, // Asia/Ho_Chi_Minh
    {  341, 2704,   83907,  8, }, // Asia/Hong_Kong
    {  341, 4771,   83915,  5, }, // Asia/Hovd
    {  341, 4184,   83920, 10, }, // Asia/Irkutsk
    {  341, 7006,   83930,  7, }, // Asia/Jakarta
    {  341, 6992,   83937,  7, }, // Asia/Jayapura
    {  341, 1290,   83944,  8, }, // Asia/Jerusalem
    {  341, 3560,   83952,  5, }, // Asia/Kabul
    {  341, 4316,   83957,  8, }, // Asia/Kamchatka
    {  341, 4239,   83965,  5, }, // Asia/Karachi
    {  341, 1185,   83970,  7, }, // Asia/Kathmandu
    {  341, 8661,   83977,  6, }, // Asia/Khandyga
    {  341,  992,   83983,  7, }, // Asia/Kolkata
    {  341, 4197,   83990, 15, }, // Asia/Krasnoyarsk
    {  341, 4859,   84005, 13, }, // Asia/Kuala_Lumpur
    {  341, 5976,   84018,  6, }, // Asia/Kuching
    {  341, 5547,   84024,  5, }, // Asia/Kuwait
    {  341, 1211,   84029,  4, }, // Asia/Macau
    {  341, 4017,   84033,  6, }, // Asia/Magadan
    {  341, 1349,   84039,  7, }, // Asia/Makassar
    {  341, 7182,   84046,  5, }, // Asia/Manila
    {  341, 6920,   84051,  5, }, // Asia/Muscat
    {  341, 2554,   84056,  8, }, // Asia/Nicosia
    {  341, 8675,   84064, 15, }, // Asia/Novokuznetsk
    {  341, 4135,   84079, 15, }, // Asia/Novosibirsk
    {  341, 4229,   84094,  6, }, // Asia/Omsk
    {  341, 7145,   84100,  3, }, // Asia/Oral
    {  341, 6961,   84103, 11, }, // Asia/Phnom_Penh
    {  341, 8693,   84114, 10, }, // Asia/Pontianak
    {  341, 4214,   84124, 10, }, // Asia/Pyongyang
    {  341, 5559,   84134,  4, }, // Asia/Qatar
    {  341, 8708,   84138,  8, }, // Asia/Qostanay
    {  341, 4269,   84146, 10, }, // Asia/Qyzylorda
    {  341, 3584,   84156,  5, }, // Asia/Riyadh
    {  341, 4408,   84161,  6, }, // Asia/Sakhalin
    {  341, 7245,   84167,  7, }, // Asia/Samarkand
    {  341, 3279,   84174,  4, }, // Asia/Seoul
    {  341, 1053,   84178,  6, }, // Asia/Shanghai
    {  341, 3300,   84184,  8, }, // Asia/Singapore
    {  341, 4297,   84192, 17, }, // Asia/Srednekolymsk
    {  341, 3263,   84209,  6, }, // Asia/Taipei
    {  341, 4781,   84215,  8, }, // Asia/Tashkent
    {  341, 3915,   84223,  6, }, // Asia/Tbilisi
    {  341, 2772,   84229,  6, }, // Asia/Tehran
    {  341, 1317,   84235,  6, }, // Asia/Thimphu
    {  341, 2821,   84241,  6, }, // Asia/Tokyo
    {  341, 4572,   84247,  6, }, // Asia/Tomsk
    {  341, 1021,   84253,  8, }, // Asia/Ulaanbaatar
    {  341, 1159,   84261,  7, }, // Asia/Urumqi
    {  341, 8722,   84268,  9, }, // Asia/Ust-Nera
    {  341, 6977,   84277, 10, }, // Asia/Vientiane
    {  341, 4710,   84287, 13, }, // Asia/Vladivostok
    {  341, 4828,   84300,  7, }, // Asia/Yakutsk
    {  341, 1235,   84307,  5, }, // Asia/Yangon
    {  341, 3883,   84312, 13, }, // Asia/Yekaterinburg
    {  341, 3747,   84325,  7, }, // Asia/Yerevan
    {  341, 3663,   84332,  6, }, // Atlantic/Azores
    {  341, 5671,   84338,  7, }, // Atlantic/Bermuda
    {  341, 6628,   84345,  6, }, // Atlantic/Canary
    {  341, 3727,   84351,  9, }, // Atlantic/Cape_Verde
    {  341, 1395,   84360,  4, }, // Atlantic/Faroe
    {  341, 8736,   84364,  5, }, // Atlantic/Madeira
    {  341, 2748,   84369,  8, }, // Atlantic/Reykjavik
    {  341, 7272,   84377, 17, }, // Atlantic/South_Georgia
    {  341, 6821,   84394, 11, }, // Atlantic/St_Helena
    {  341, 6644,   84405,  7, }, // Atlantic/Stanley
    {  341, 1670,   84412,  7, }, // Australia/Adelaide
    {  341, 1635,   84419,  8, }, // Australia/Brisbane
    {  341, 1799,   84427, 10, }, // Australia/Broken_Hill
    {  341, 1583,   84437,  7, }, // Australia/Darwin
    {  341, 3637,   84444,  5, }, // Australia/Eucla
    {  341, 1516,   84449,  7, }, // Australia/Hobart
    {  341, 8753,   84456,  9, }, // Australia/Lindeman
    {  341, 1547,   84465,  9, }, // Australia/Lord_Howe
    {  341, 1727,   84474,  8, }, // Australia/Melbourne
    {  341, 1762,   84482,  4, }, // Australia/Perth
    {  341, 1463,   84486,  5, }, // Australia/Sydney
    {  341, 6441,   84491, 10, }, // Europe/Amsterdam
    {  341, 6216,   84501,  6, }, // Europe/Andorra
    {  341, 3620,   84507,  8, }, // Europe/Astrakhan
    {  341, 2300,   84515,  5, }, // Europe/Athens
    {  341, 6336,   84520,  8, }, // Europe/Belgrade
    {  341, 4757,   84528,  6, }, // Europe/Berlin
    {  341, 6488,   84534, 11, }, // Europe/Bratislava
    {  341, 2167,   84545,  8, }, // Europe/Brussels
    {  341, 3928,   84553,  8, }, // Europe/Bucharest
    {  341, 3806,   84561,  9, }, // Europe/Budapest
    {  341, 8772,   84570,  8, }, // Europe/Busingen
    {  341, 2583,   84578,  7, }, // Europe/Chisinau
    {  341, 6289,   84585,  8, }, // Europe/Copenhagen
    {  341, 2338,   84593,  5, }, // Europe/Dublin
    {  341, 6307,   84598, 10, }, // Europe/Gibraltar
    {  341, 8788,   84608,  6, }, // Europe/Guernsey
    {  341, 6612,   84614,  8, }, // Europe/Helsinki
    {  341, 8804,   84622, 10, }, // Europe/Isle_of_Man
    {  341, 1130,   84632,  9, }, // Europe/Istanbul
    {  341, 8823,   84641,  5, }, // Europe/Jersey
    {  341, 3979,   84646, 12, }, // Europe/Kaliningrad
    {  341, 8929,   84658,  5, }, // Europe/Kirov
    {  341, 2527,   84663,  3, }, // Europe/Kyiv
    {  341, 3213,   84666,  6, }, // Europe/Lisbon
    {  341, 6506,   84672, 10, }, // Europe/Ljubljana
    {  341, 2501,   84682,  4, }, // Europe/London
    {  341, 6365,   84686,  9, }, // Europe/Luxembourg
    {  341, 4892,   84695,  7, }, // Europe/Madrid
    {  341, 6397,   84702,  6, }, // Europe/Malta
    {  341, 6582,   84708,  7, }, // Europe/Mariehamn
    {  341, 3693,   84715,  6, }, // Europe/Minsk
    {  341, 6410,   84721,  6, }, // Europe/Monaco
    {  341, 3537,   84727,  6, }, // Europe/Moscow
    {  341, 6458,   84733,  5, }, // Europe/Oslo
    {  341, 4284,   84738,  5, }, // Europe/Paris
    {  341, 6424,   84743,  9, }, // Europe/Podgorica
    {  341, 6275,   84752,  5, }, // Europe/Prague
    {  341, 8837,   84757,  4, }, // Europe/Riga
    {  341, 6324,   83495,  3, }, // Europe/Rome
    {  341, 4331,   84761,  5, }, // Europe/Samara
    {  341, 6470,   84766, 10, }, // Europe/San_Marino
    {  341, 6245,   84776,  8, }, // Europe/Sarajevo
    {  341, 4451,   84784,  7, }, // Europe/Saratov
    {  341, 8849,   84791, 11, }, // Europe/Simferopol
    {  341, 6383,   84802,  7, }, // Europe/Skopje
    {  341, 6599,   84809,  6, }, // Europe/Sofia
    {  341, 6523,   84815,  8, }, // Europe/Stockholm
    {  341, 8867,   84823,  6, }, // Europe/Tallinn
    {  341, 6202,   84829,  6, }, // Europe/Tirane
    {  341, 8882,   84835, 10, }, // Europe/Ulyanovsk
    {  341, 6352,   84845,  5, }, // Europe/Vaduz
    {  341, 6567,   84850,  6, }, // Europe/Vatican
    {  341, 6231,   84856,  6, }, // Europe/Vienna
    {  341, 8899,   84862,  8, }, // Europe/Vilnius
    {  341, 4727,   84870, 11, }, // Europe/Volgograd
    {  341, 3190,   84881,  5, }, // Europe/Warsaw
    {  341, 6261,   84886,  6, }, // Europe/Zagreb
    {  341, 6540,   84892,  5, }, // Europe/Zurich
    {  341, 5078,   84897, 12, }, // Indian/Antananarivo
    {  341, 6947,   84909,  5, }, // Indian/Chagos
    {  341, 6046,   84914,  7, }, // Indian/Christmas
    {  341, 6063,   84921,  5, }, // Indian/Cocos
    {  341, 5029,   84926,  6, }, // Indian/Comoro
    {  341, 6661,   84932, 10, }, // Indian/Kerguelen
    {  341, 7260,   84942,  4, }, // Indian/Mahe
    {  341, 7050,   84946,  8, }, // Indian/Maldives
    {  341, 4069,   84954,  8, }, // Indian/Mauritius
    {  341, 5098,   84962,  5, }, // Indian/Mayotte
    {  341, 7211,   84967,  8, }, // Indian/Reunion
    {  341, 4422,   84975,  5, }, // Pacific/Apia
    {  341,  932,   84980,  7, }, // Pacific/Auckland
    {  341, 3706,   84987, 10, }, // Pacific/Bougainville
    {  341, 3015,   84997,  3, }, // Pacific/Chatham
    {  341, 3157,   85000,  3, }, // Pacific/Chuuk
    {  341, 2237,   85003,  5, }, // Pacific/Easter
    {  341, 7360,   85008,  4, }, // Pacific/Efate
    {  341, 7327,   85012,  7, }, // Pacific/Fakaofo
    {  341, 3902,   85019,  4, }, // Pacific/Fiji
    {  341, 7343,   85023,  8, }, // Pacific/Funafuti
    {  341, 6678,   85031,  9, }, // Pacific/Galapagos
    {  341, 6696,   85040,  8, }, // Pacific/Gambier
    {  341, 3822,   85048,  8, }, // Pacific/Guadalcanal
    {  341, 6018,   85056,  5, }, // Pacific/Guam
    {  341, 2723,   85061,  8, }, // Pacific/Honolulu
    {  341, 3049,   85069,  8, }, // Pacific/Kanton
    {  341, 3998,   85077, 10, }, // Pacific/Kiritimati
    {  341, 7019,   85087,  5, }, // Pacific/Kosrae
    {  341, 2842,   85092,  9, }, // Pacific/Kwajalein
    {  341, 4877,   85101,  6, }, // Pacific/Majuro
    {  341, 4051,   85107,  9, }, // Pacific/Marquesas
    {  341, 8914,   85116,  5, }, // Pacific/Midway
    {  341, 7084,   85121,  4, }, // Pacific/Nauru
    {  341, 7132,   85125,  4, }, // Pacific/Niue
    {  341, 4168,   85129,  6, }, // Pacific/Norfolk
    {  341, 7098,   85135,  5, }, // Pacific/Noumea
    {  341, 3126,   85140,  9, }, // Pacific/Pago_Pago
    {  341, 7155,   85149,  4, }, // Pacific/Palau
    {  341, 7194,   85153,  8, }, // Pacific/Pitcairn
    {  341, 3096,   85161,  6, }, // Pacific/Pohnpei
    {  341, 4807,   85167, 14, }, // Pacific/Port_Moresby
    {  341, 6076,   85181,  9, }, // Pacific/Rarotonga
    {  341, 6031,   85190,  5, }, // Pacific/Saipan
    {  341, 7312,   85195,  6, }, // Pacific/Tahiti
    {  341, 6712,   85201,  6, }, // Pacific/Tarawa
    {  341, 4583,   85207,  9, }, // Pacific/Tongatapu
    {  341, 7392,   85216,  3, }, // Pacific/Wake
    {  341, 7405,   85219,  5, }, // Pacific/Wallis
    {  342,   14,       0,  6, }, // Africa/Asmara Gusii/Latin/Kenya
    {  342, 4435,       6,  8, }, // Africa/Sao_Tome
    {  342, 4252,      14,  8, }, // America/Asuncion
    {  342,  237,      22,  8, }, // America/Atikokan
    {  342, 7623,      30, 17, }, // America/Bahia_Banderas
    {  342, 3868,      47,  6, }, // America/Cancun
    {  342, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  342, 5723,      66,  7, }, // America/Curacao
    {  342,  481,      73, 13, }, // America/Indiana/Knox
    {  342, 7961,      86, 16, }, // America/Indiana/Marengo
    {  342, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  342, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  342, 8038,     139, 14, }, // America/Indiana/Vevay
    {  342, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  342, 8086,     171, 16, }, // America/Indiana/Winamac
    {  342, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  342, 5932,     207, 22, }, // America/Lower_Princes
    {  342, 8220,     229,  6, }, // America/Merida
    {  342, 1850,     235, 19, }, // America/Noronha
    {  342, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  342, 8329,     274, 20, }, // America/North_Dakota/Center
    {  342, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  342,  393,     317,  4, }, // America/Nuuk
    {  342, 6899,     321, 16, }, // America/Scoresbysund
    {  342, 8508,     337, 14, }, // America/St_Barthelemy
    {  342, 2061,     351, 10, }, // America/St_Johns
    {  342, 5863,     361,  9, }, // America/St_Kitts
    {  342, 5880,     370,  9, }, // America/St_Lucia
    {  342,  855,     379, 10, }, // America/St_Thomas
    {  342, 5913,     389, 11, }, // America/St_Vincent
    {  342, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  342, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  342, 1185,     448,  9, }, // Asia/Kathmandu
    {  342,  992,     457,  7, }, // Asia/Kolkata
    {  342, 1211,     464,  5, }, // Asia/Macau
    {  342, 1235,     477,  6, }, // Asia/Yangon
    {  342, 1395,     483,  5, }, // Atlantic/Faroe
    {  342, 6821,     488, 10, }, // Atlantic/St_Helena
    {  342, 2527,     514,  4, }, // Europe/Kyiv
    {  342, 7211,     547,  7, }, // Indian/Reunion
    {  342, 3157,     554,  5, }, // Pacific/Chuuk
    {  342, 3096,     586,  7, }, // Pacific/Pohnpei
    {  343, 5059,   64622, 11, }, // Africa/Addis_Ababa Haitian/Latin/Haiti
    {  343, 7420,   44088,  5, }, // Africa/Algiers
    {  343,   14,       0,  6, }, // Africa/Asmara
    {  343, 2320,   64633,  8, }, // Africa/Cairo
    {  343, 5166,   64641,  8, }, // Africa/El_Aaiun
    {  343, 6869,   20418,  4, }, // Africa/Lome
    {  343, 5113,   44110, 10, }, // Africa/Mogadishu
    {  343, 5271,   44128,  9, }, // Africa/Ndjamena
    {  343, 4435,       6,  8, }, // Africa/Sao_Tome
    {  343, 2866,   64649, 15, }, // Africa/Tripoli
    {  343, 4554,   26559,  9, }, // America/Araguaina
    {  343,  270,   20479,  7, }, // America/Argentina/Cordoba
    {  343, 7489,   44137, 12, }, // America/Argentina/Rio_Gallegos
    {  343, 7571,   20486,  7, }, // America/Argentina/Tucuman
    {  343, 7597,   64664,  7, }, // America/Argentina/Ushuaia
    {  343, 4252,      14,  8, }, // America/Asuncion
    {  343,  237,      22,  8, }, // America/Atikokan
    {  343, 7623,   64671, 17, }, // America/Bahia_Banderas
    {  343, 5654,   64688, 10, }, // America/Barbados
    {  343, 7646,   35896,  5, }, // America/Belem
    {  343, 3868,      47,  6, }, // America/Cancun
    {  343, 5496,   64698,  7, }, // America/Cayman
    {  343, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  343, 3791,   26574,  6, }, // America/Cuiaba
    {  343, 5723,      66,  7, }, // America/Curacao
    {  343, 3465,   64705,  7, }, // America/Detroit
    {  343, 5739,   64712,  9, }, // America/Dominica
    {  343, 7869,   26580,  8, }, // America/Eirunepe
    {  343, 5770,   64721,  7, }, // America/Grenada
    {  343, 2281,   64728,  9, }, // America/Havana
    {  343,  481,   64737, 14, }, // America/Indiana/Knox
    {  343, 7961,   64751, 17, }, // America/Indiana/Marengo
    {  343, 7985,   64768, 20, }, // America/Indiana/Petersburg
    {  343, 8012,   64788, 19, }, // America/Indiana/Tell_City
    {  343, 8038,   64807, 15, }, // America/Indiana/Vevay
    {  343, 8060,   64822, 19, }, // America/Indiana/Vincennes
    {  343, 8086,   64841, 17, }, // America/Indiana/Winamac
    {  343, 2799,   64858,  8, }, // America/Jamaica
    {  343, 8125,   64866, 21, }, // America/Kentucky/Monticello
    {  343, 5932,     207, 22, }, // America/Lower_Princes
    {  343, 8153,   26588,  6, }, // America/Maceio
    {  343, 1908,   64887,  6, }, // America/Manaus
    {  343, 2917,   26603,  8, }, // America/Mazatlan
    {  343, 8220,     229,  6, }, // America/Merida
    {  343, 2949,   58762,  6, }, // America/Mexico_City
    {  343, 1850,     235, 19, }, // America/Noronha
    {  343, 8301,   64893, 23, }, // America/North_Dakota/Beulah
    {  343, 8329,   64916, 23, }, // America/North_Dakota/Center
    {  343, 8357,   64939, 26, }, // America/North_Dakota/New_Salem
    {  343,  393,     317,  4, }, // America/Nuuk
    {  343, 5954,   64965, 14, }, // America/Port_of_Spain
    {  343, 5843,   64979, 10, }, // America/Puerto_Rico
    {  343, 8477,   26704,  8, }, // America/Santarem
    {  343, 6111,   64989, 14, }, // America/Santo_Domingo
    {  343, 1878,   26712,  9, }, // America/Sao_Paulo
    {  343, 6899,     321, 16, }, // America/Scoresbysund
    {  343, 8508,   26721, 16, }, // America/St_Barthelemy
    {  343, 2061,   65003, 25, }, // America/St_Johns
    {  343, 5863,   65028, 16, }, // America/St_Kitts
    {  343, 5880,   65044, 12, }, // America/St_Lucia
    {  343,  855,   26771, 12, }, // America/St_Thomas
    {  343, 5913,   65056, 13, }, // America/St_Vincent
    {  343, 5756,   65069,  5, }, // America/Thule
    {  343, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  343, 7295,   62201,  5, }, // Antarctica/Syowa
    {  343, 5395,   57607,  8, }, // Asia/Almaty
    {  343, 5523,   65074,  6, }, // Asia/Aqtau
    {  343, 5368,   65080, 11, }, // Asia/Aqtobe
    {  343,  964,   65091,  8, }, // Asia/Ashgabat
    {  343, 8624,   65099,  7, }, // Asia/Atyrau
    {  343, 3607,    4162,  6, }, // Asia/Baghdad
    {  343, 5534,   65106,  7, }, // Asia/Bahrain
    {  343, 3653,   35998,  5, }, // Asia/Baku
    {  343, 4086,   65113,  8, }, // Asia/Beirut
    {  343, 3778,   65121,  7, }, // Asia/Bishkek
    {  343, 4601,   36009,  6, }, // Asia/Chita
    {  343, 4540,    4201,  5, }, // Asia/Damascus
    {  343, 3596,   65128,  5, }, // Asia/Dubai
    {  343, 6159,   65133,  9, }, // Asia/Dushanbe
    {  343, 8636,   65142, 10, }, // Asia/Famagusta
    {  343, 4795,   65152,  6, }, // Asia/Hebron
    {  343, 1259,   65158, 17, }, // Asia/Ho_Chi_Minh
    {  343, 4184,   65175,  8, }, // Asia/Irkutsk
    {  343, 1290,   65183,  9, }, // Asia/Jerusalem
    {  343, 3560,   36037,  6, }, // Asia/Kabul
    {  343, 4316,   36043, 10, }, // Asia/Kamchatka
    {  343, 1185,   65192,  9, }, // Asia/Kathmandu
    {  343,  992,   36053,  8, }, // Asia/Kolkata
    {  343, 4197,   65201, 11, }, // Asia/Krasnoyarsk
    {  343, 5547,   65212,  6, }, // Asia/Kuwait
    {  343, 1211,     464,  5, }, // Asia/Macau
    {  343, 1349,   65218,  8, }, // Asia/Makassar
    {  343, 7182,   65226,  7, }, // Asia/Manila
    {  343, 6920,   20857,  7, }, // Asia/Muscat
    {  343, 2554,   65233,  7, }, // Asia/Nicosia
    {  343, 4135,   44379, 12, }, // Asia/Novosibirsk
    {  343, 7145,   65240,  7, }, // Asia/Oral
    {  343, 8708,   65247,  8, }, // Asia/Qostanay
    {  343, 4269,   65255,  9, }, // Asia/Qyzylorda
    {  343, 3584,   20874,  5, }, // Asia/Riyadh
    {  343, 4408,   65264,  9, }, // Asia/Sakhalin
    {  343, 7245,   65273, 10, }, // Asia/Samarkand
    {  343, 3279,   65283,  5, }, // Asia/Seoul
    {  343, 3300,   36101,  9, }, // Asia/Singapore
    {  343, 4781,   65288,  8, }, // Asia/Tashkent
    {  343, 3915,   65296,  8, }, // Asia/Tbilisi
    {  343, 2772,   65304,  7, }, // Asia/Tehran
    {  343, 1021,   65311, 11, }, // Asia/Ulaanbaatar
    {  343, 1159,   26933,  6, }, // Asia/Urumqi
    {  343, 4828,   65322,  8, }, // Asia/Yakutsk
    {  343, 1235,   65330,  7, }, // Asia/Yangon
    {  343, 3883,   65337, 13, }, // Asia/Yekaterinburg
    {  343, 3747,   26947,  6, }, // Asia/Yerevan
    {  343, 3663,   44461,  6, }, // Atlantic/Azores
    {  343, 5671,   44467,  8, }, // Atlantic/Bermuda
    {  343, 6628,   65350, 13, }, // Atlantic/Canary
    {  343, 3727,   65363,  8, }, // Atlantic/Cape_Verde
    {  343, 1395,   65371, 10, }, // Atlantic/Faroe
    {  343, 8736,   65381,  6, }, // Atlantic/Madeira
    {  343, 7272,   65387, 14, }, // Atlantic/South_Georgia
    {  343, 6821,   65401, 13, }, // Atlantic/St_Helena
    {  343, 1670,   65414,  8, }, // Australia/Adelaide
    {  343, 6216,   65422,  7, }, // Europe/Andorra
    {  343, 2300,   65429,  7, }, // Europe/Athens
    {  343, 2167,   57661,  9, }, // Europe/Brussels
    {  343, 3928,   21073,  8, }, // Europe/Bucharest
    {  343, 8772,   27037,  8, }, // Europe/Busingen
    {  343, 6289,   21081, 10, }, // Europe/Copenhagen
    {  343, 8788,   27045,  9, }, // Europe/Guernsey
    {  343, 8804,   65436, 10, }, // Europe/Isle_of_Man
    {  343, 2527,    8148,  4, }, // Europe/Kyiv
    {  343, 3213,   65446,  8, }, // Europe/Lisbon
    {  343, 2501,   21152,  7, }, // Europe/London
    {  343, 6397,   65454,  5, }, // Europe/Malta
    {  343, 3537,   44576,  6, }, // Europe/Moscow
    {  343, 6470,   65459, 11, }, // Europe/San_Marino
    {  343, 6202,   21213,  6, }, // Europe/Tirane
    {  343, 8882,   65470, 10, }, // Europe/Ulyanovsk
    {  343, 6567,   65480, 10, }, // Europe/Vatican
    {  343, 6231,   65490,  6, }, // Europe/Vienna
    {  343, 3190,   65496,  8, }, // Europe/Warsaw
    {  343, 5029,   65504,  7, }, // Indian/Comoro
    {  343, 7260,   27115,  4, }, // Indian/Mahe
    {  343, 4069,   65511,  7, }, // Indian/Mauritius
    {  343, 7211,   65518, 10, }, // Indian/Reunion
    {  343, 3157,     554,  5, }, // Pacific/Chuuk
    {  343, 2237,   65528, 13, }, // Pacific/Easter
    {  343, 7360,   27148,  5, }, // Pacific/Efate
    {  343, 3902,    4606,  5, }, // Pacific/Fiji
    {  343, 6678,   21277,  9, }, // Pacific/Galapagos
    {  343, 2723,    4611,  8, }, // Pacific/Honolulu
    {  343, 3049,    4619,  9, }, // Pacific/Kanton
    {  343, 4051,   65541,  9, }, // Pacific/Marquesas
    {  343, 7098,    4628,  6, }, // Pacific/Noumea
    {  343, 7155,   65550,  6, }, // Pacific/Palau
    {  343, 3096,     586,  7, }, // Pacific/Pohnpei
    {  344,   14,       0,  6, }, // Africa/Asmara Hausa/Latin/Nigeria
    {  344, 4435,       6,  8, }, // Africa/Sao_Tome
    {  344, 4252,      14,  8, }, // America/Asuncion
    {  344,  237,      22,  8, }, // America/Atikokan
    {  344, 7623,      30, 17, }, // America/Bahia_Banderas
    {  344, 3868,      47,  6, }, // America/Cancun
    {  344, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  344, 5723,      66,  7, }, // America/Curacao
    {  344,  481,      73, 13, }, // America/Indiana/Knox
    {  344, 7961,      86, 16, }, // America/Indiana/Marengo
    {  344, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  344, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  344, 8038,     139, 14, }, // America/Indiana/Vevay
    {  344, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  344, 8086,     171, 16, }, // America/Indiana/Winamac
    {  344, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  344, 5932,     207, 22, }, // America/Lower_Princes
    {  344, 8220,     229,  6, }, // America/Merida
    {  344, 1850,     235, 19, }, // America/Noronha
    {  344, 8301,   85224, 23, }, // America/North_Dakota/Beulah
    {  344, 8329,   85247, 23, }, // America/North_Dakota/Center
    {  344, 8357,   85270, 26, }, // America/North_Dakota/New_Salem
    {  344,  393,     317,  4, }, // America/Nuuk
    {  344, 6899,     321, 16, }, // America/Scoresbysund
    {  344, 8508,     337, 14, }, // America/St_Barthelemy
    {  344, 2061,     351, 10, }, // America/St_Johns
    {  344, 5863,     361,  9, }, // America/St_Kitts
    {  344, 5880,     370,  9, }, // America/St_Lucia
    {  344,  855,     379, 10, }, // America/St_Thomas
    {  344, 5913,     389, 11, }, // America/St_Vincent
    {  344, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  344, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  344, 1185,     448,  9, }, // Asia/Kathmandu
    {  344,  992,     457,  7, }, // Asia/Kolkata
    {  344, 1211,     464,  5, }, // Asia/Macau
    {  344, 1235,     477,  6, }, // Asia/Yangon
    {  344, 1395,     483,  5, }, // Atlantic/Faroe
    {  344, 6821,     488, 10, }, // Atlantic/St_Helena
    {  344, 2527,     514,  4, }, // Europe/Kyiv
    {  344, 7211,     547,  7, }, // Indian/Reunion
    {  344, 3157,     554,  5, }, // Pacific/Chuuk
    {  344, 3096,     586,  7, }, // Pacific/Pohnpei
    {  345, 8301,     254, 20, }, // America/North_Dakota/Beulah Hausa/Arabic/Nigeria
    {  345, 8329,     274, 20, }, // America/North_Dakota/Center
    {  345, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  346, 8301,     254, 20, }, // America/North_Dakota/Beulah Hausa/Arabic/Sudan
    {  346, 8329,     274, 20, }, // America/North_Dakota/Center
    {  346, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  349,   14,       0,  6, }, // Africa/Asmara Hawaiian/Latin/United States
    {  349, 4435,       6,  8, }, // Africa/Sao_Tome
    {  349, 4252,      14,  8, }, // America/Asuncion
    {  349,  237,      22,  8, }, // America/Atikokan
    {  349, 7623,      30, 17, }, // America/Bahia_Banderas
    {  349, 3868,      47,  6, }, // America/Cancun
    {  349, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  349, 5723,      66,  7, }, // America/Curacao
    {  349,  481,      73, 13, }, // America/Indiana/Knox
    {  349, 7961,      86, 16, }, // America/Indiana/Marengo
    {  349, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  349, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  349, 8038,     139, 14, }, // America/Indiana/Vevay
    {  349, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  349, 8086,     171, 16, }, // America/Indiana/Winamac
    {  349, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  349, 5932,     207, 22, }, // America/Lower_Princes
    {  349, 8220,     229,  6, }, // America/Merida
    {  349, 1850,     235, 19, }, // America/Noronha
    {  349, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  349, 8329,     274, 20, }, // America/North_Dakota/Center
    {  349, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  349,  393,     317,  4, }, // America/Nuuk
    {  349, 6899,     321, 16, }, // America/Scoresbysund
    {  349, 8508,     337, 14, }, // America/St_Barthelemy
    {  349, 2061,     351, 10, }, // America/St_Johns
    {  349, 5863,     361,  9, }, // America/St_Kitts
    {  349, 5880,     370,  9, }, // America/St_Lucia
    {  349,  855,     379, 10, }, // America/St_Thomas
    {  349, 5913,     389, 11, }, // America/St_Vincent
    {  349, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  349, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  349, 1185,     448,  9, }, // Asia/Kathmandu
    {  349,  992,     457,  7, }, // Asia/Kolkata
    {  349, 1211,     464,  5, }, // Asia/Macau
    {  349, 1235,     477,  6, }, // Asia/Yangon
    {  349, 1395,     483,  5, }, // Atlantic/Faroe
    {  349, 6821,     488, 10, }, // Atlantic/St_Helena
    {  349, 2527,     514,  4, }, // Europe/Kyiv
    {  349, 7211,     547,  7, }, // Indian/Reunion
    {  349, 3157,     554,  5, }, // Pacific/Chuuk
    {  349, 3096,     586,  7, }, // Pacific/Pohnpei
    {  350, 6788,   85296,  7, }, // Africa/Abidjan Hebrew/Hebrew/Israel
    {  350, 6760,   85303,  4, }, // Africa/Accra
    {  350, 5059,   85307,  9, }, // Africa/Addis_Ababa
    {  350, 7420,   85316,  6, }, // Africa/Algiers
    {  350,   14,   85322,  5, }, // Africa/Asmara
    {  350,   44,   85327,  5, }, // Africa/Bamako
    {  350, 5257,   85332,  6, }, // Africa/Bangui
    {  350, 6746,   85338,  6, }, // Africa/Banjul
    {  350, 7435,   85344,  5, }, // Africa/Bissau
    {  350, 4957,   85349,  6, }, // Africa/Blantyre
    {  350, 5287,   85355,  7, }, // Africa/Brazzaville
    {  350, 4922,   85362, 10, }, // Africa/Bujumbura
    {  350, 2320,   85372,  4, }, // Africa/Cairo
    {  350, 4117,   85376,  7, }, // Africa/Casablanca
    {  350, 7449,   85383,  5, }, // Africa/Ceuta
    {  350, 6773,   85388,  7, }, // Africa/Conakry
    {  350, 6840,   85395,  4, }, // Africa/Dakar
    {  350, 5130,   85399, 10, }, // Africa/Dar_es_Salaam
    {  350, 5043,   85409,  7, }, // Africa/Djibouti
    {  350, 5243,   85416,  5, }, // Africa/Douala
    {  350, 5166,   85421,  7, }, // Africa/El_Aaiun
    {  350, 6853,   85428,  7, }, // Africa/Freetown
    {  350, 4906,   85435,  7, }, // Africa/Gaborone
    {  350, 5015,   85442,  5, }, // Africa/Harare
    {  350, 4479,   85447,  9, }, // Africa/Johannesburg
    {  350, 4499,   85456,  5, }, // Africa/Juba
    {  350, 5151,   85461,  6, }, // Africa/Kampala
    {  350, 4524,   85467,  5, }, // Africa/Khartoum
    {  350, 4973,   85472,  5, }, // Africa/Kigali
    {  350, 5306,   85477,  6, }, // Africa/Kinshasa
    {  350, 4744,   85483,  5, }, // Africa/Lagos
    {  350, 5336,   85488,  8, }, // Africa/Libreville
    {  350, 6869,   85496,  4, }, // Africa/Lome
    {  350, 5211,   85500,  6, }, // Africa/Luanda
    {  350, 4939,   85506,  9, }, // Africa/Lubumbashi
    {  350, 5001,   85515,  5, }, // Africa/Lusaka
    {  350, 5322,   85520,  4, }, // Africa/Malabo
    {  350, 4987,   85524,  6, }, // Africa/Maputo
    {  350, 5197,   85530,  4, }, // Africa/Maseru
    {  350, 5182,   85534,  5, }, // Africa/Mbabane
    {  350, 5113,   85539,  7, }, // Africa/Mogadishu
    {  350, 7034,   85546,  8, }, // Africa/Monrovia
    {  350, 3853,   85554,  7, }, // Africa/Nairobi
    {  350, 5271,   85561,  6, }, // Africa/Ndjamena
    {  350, 5354,   85567,  6, }, // Africa/Niamey
    {  350, 6803,   85573,  8, }, // Africa/Nouakchott
    {  350, 6727,   85581,  9, }, // Africa/Ouagadougou
    {  350, 5225,   85590, 10, }, // Africa/Porto-Novo
    {  350, 4435,   85600,  8, }, // Africa/Sao_Tome
    {  350, 2866,   85608,  7, }, // Africa/Tripoli
    {  350, 6554,   85615,  5, }, // Africa/Tunis
    {  350, 4152,   85620,  7, }, // Africa/Windhoek
    {  350,  132,   85627,  4, }, // America/Adak
    {  350, 3346,   85631,  7, }, // America/Anchorage
    {  350, 5607,   85638,  8, }, // America/Anguilla
    {  350, 5624,   85646,  8, }, // America/Antigua
    {  350, 4554,   85654,  9, }, // America/Araguaina
    {  350,  166,   85663, 12, }, // America/Argentina/Buenos_Aires
    {  350,   91,   85675,  6, }, // America/Argentina/Catamarca
    {  350,  270,   85681,  7, }, // America/Argentina/Cordoba
    {  350,  441,   85688,  5, }, // America/Argentina/Jujuy
    {  350, 7462,   85693,  8, }, // America/Argentina/La_Rioja
    {  350,  565,   85701,  6, }, // America/Argentina/Mendoza
    {  350, 7489,   85707, 11, }, // America/Argentina/Rio_Gallegos
    {  350, 7520,   85718,  4, }, // America/Argentina/Salta
    {  350, 7544,   85722,  7, }, // America/Argentina/San_Juan
    {  350, 5580,   85729,  8, }, // America/Argentina/San_Luis
    {  350, 7571,   85737,  6, }, // America/Argentina/Tucuman
    {  350, 7597,   85743,  8, }, // America/Argentina/Ushuaia
    {  350, 5640,   85751,  5, }, // America/Aruba
    {  350, 4252,   85756,  8, }, // America/Asuncion
    {  350,  237,   85764,  7, }, // America/Atikokan
    {  350, 3679,   85771,  5, }, // America/Bahia
    {  350, 7623,   85776, 11, }, // America/Bahia_Banderas
    {  350, 5654,   85787,  6, }, // America/Barbados
    {  350, 7646,   85793,  3, }, // America/Belem
    {  350, 5407,   85796,  4, }, // America/Belize
    {  350, 7660,   85800, 10, }, // America/Blanc-Sablon
    {  350, 7681,   85810, 10, }, // America/Boa_Vista
    {  350, 4361,   85820,  6, }, // America/Bogota
    {  350, 7699,   85826,  5, }, // America/Boise
    {  350, 7713,   85831, 14, }, // America/Cambridge_Bay
    {  350, 7735,   85845, 10, }, // America/Campo_Grande
    {  350, 3868,   85855,  5, }, // America/Cancun
    {  350, 4694,   85860,  5, }, // America/Caracas
    {  350, 4345,   85865,  5, }, // America/Cayenne
    {  350, 5496,   85870,  5, }, // America/Cayman
    {  350, 2260,   85875,  5, }, // America/Chicago
    {  350, 7756,   85880,  9, }, // America/Chihuahua
    {  350, 7774,   85889, 11, }, // America/Ciudad_Juarez
    {  350, 5422,   85900, 10, }, // America/Costa_Rica
    {  350, 7796,   85910,  6, }, // America/Creston
    {  350, 3791,   85916,  6, }, // America/Cuiaba
    {  350, 5723,   85922,  7, }, // America/Curacao
    {  350, 7812,   85929, 10, }, // America/Danmarkshavn
    {  350, 7833,   85939,  5, }, // America/Dawson
    {  350, 7848,   85944, 10, }, // America/Dawson_Creek
    {  350,  805,   85954,  5, }, // America/Denver
    {  350, 3465,   85959,  6, }, // America/Detroit
    {  350, 5739,   85965,  8, }, // America/Dominica
    {  350,  893,   85973,  8, }, // America/Edmonton
    {  350, 7869,   85981,  7, }, // America/Eirunepe
    {  350, 5441,   85988,  9, }, // America/El_Salvador
    {  350, 7886,   85997, 10, }, // America/Fort_Nelson
    {  350, 7906,   86007,  8, }, // America/Fortaleza
    {  350, 7924,   86015,  9, }, // America/Glace_Bay
    {  350, 6881,   86024,  7, }, // America/Goose_Bay
    {  350, 4612,   86031,  9, }, // America/Grand_Turk
    {  350, 5770,   85850,  5, }, // America/Grenada
    {  350, 5786,   86040,  7, }, // America/Guadeloupe
    {  350, 3760,   86047,  7, }, // America/Guatemala
    {  350, 4841,   86054,  8, }, // America/Guayaquil
    {  350, 6932,   86062,  5, }, // America/Guyana
    {  350, 1939,   86067,  6, }, // America/Halifax
    {  350, 2281,   86073,  6, }, // America/Havana
    {  350, 7942,   86079,  7, }, // America/Hermosillo
    {  350,  348,   86086, 12, }, // America/Indiana/Indianapolis
    {  350,  481,   86098, 14, }, // America/Indiana/Knox
    {  350, 7961,   86112, 15, }, // America/Indiana/Marengo
    {  350, 7985,   86127, 19, }, // America/Indiana/Petersburg
    {  350, 8012,   86146, 17, }, // America/Indiana/Tell_City
    {  350, 8038,   86163, 17, }, // America/Indiana/Vevay
    {  350, 8060,   86180, 16, }, // America/Indiana/Vincennes
    {  350, 8086,   86196, 16, }, // America/Indiana/Winamac
    {  350, 8110,   86212,  7, }, // America/Inuvik
    {  350,  660,   86219,  8, }, // America/Iqaluit
    {  350, 2799,   86227,  7, }, // America/Jamaica
    {  350, 5380,   86234,  5, }, // America/Juneau
    {  350,  521,   86239,  8, }, // America/Kentucky/Louisville
    {  350, 8125,   86247, 17, }, // America/Kentucky/Monticello
    {  350, 5704,   86264,  8, }, // America/Kralendijk
    {  350, 4376,   86272,  6, }, // America/La_Paz
    {  350, 7169,   86278,  4, }, // America/Lima
    {  350, 3239,   86282, 10, }, // America/Los_Angeles
    {  350, 5932,   86292, 20, }, // America/Lower_Princes
    {  350, 8153,   86312,  6, }, // America/Maceio
    {  350, 8168,   86318,  6, }, // America/Managua
    {  350, 1908,   86324,  5, }, // America/Manaus
    {  350, 5897,   86329,  5, }, // America/Marigot
    {  350, 5805,   86334,  7, }, // America/Martinique
    {  350, 8184,   86341,  7, }, // America/Matamoros
    {  350, 2917,   86348,  5, }, // America/Mazatlan
    {  350, 8202,   86353,  7, }, // America/Menominee
    {  350, 8220,   86360,  5, }, // America/Merida
    {  350, 8235,   86365,  7, }, // America/Metlakatla
    {  350, 2949,   86372, 11, }, // America/Mexico_City
    {  350, 4391,   86383,  6, }, // America/Miquelon
    {  350, 8254,   86389,  7, }, // America/Moncton
    {  350, 8270,   86396,  7, }, // America/Monterrey
    {  350, 4098,   86403, 10, }, // America/Montevideo
    {  350, 5824,   86413,  7, }, // America/Montserrat
    {  350, 5481,   86420,  4, }, // America/Nassau
    {  350, 2379,   86424,  8, }, // America/New_York
    {  350, 8288,   86432,  3, }, // America/Nome
    {  350, 1850,   86435,  7, }, // America/Noronha
    {  350, 8301,   86442, 20, }, // America/North_Dakota/Beulah
    {  350, 8329,   86462, 19, }, // America/North_Dakota/Center
    {  350, 8357,   86481, 24, }, // America/North_Dakota/New_Salem
    {  350,  393,   86505,  5, }, // America/Nuuk
    {  350, 8388,   86510,  9, }, // America/Ojinaga
    {  350, 2356,   86519,  4, }, // America/Panama
    {  350, 6173,   86523,  7, }, // America/Paramaribo
    {  350, 2973,   86530,  6, }, // America/Phoenix
    {  350, 3945,   86536, 13, }, // America/Port-au-Prince
    {  350, 5954,   86549, 14, }, // America/Port_of_Spain
    {  350, 8404,   86563, 10, }, // America/Porto_Velho
    {  350, 5843,   86573, 11, }, // America/Puerto_Rico
    {  350, 4030,   86584, 10, }, // America/Punta_Arenas
    {  350, 8424,   86594, 11, }, // America/Rankin_Inlet
    {  350, 8445,   86605,  5, }, // America/Recife
    {  350, 1995,   86610,  6, }, // America/Regina
    {  350, 8460,   86616,  6, }, // America/Resolute
    {  350,  695,   86622,  9, }, // America/Rio_Branco
    {  350, 8477,   86631,  5, }, // America/Santarem
    {  350, 2201,   86636,  7, }, // America/Santiago
    {  350, 6111,   86643, 12, }, // America/Santo_Domingo
    {  350, 1878,   86655,  9, }, // America/Sao_Paulo
    {  350, 6899,   86664, 11, }, // America/Scoresbysund
    {  350, 8494,   86675,  5, }, // America/Sitka
    {  350, 8508,   86680, 10, }, // America/St_Barthelemy
    {  350, 2061,   86690,  9, }, // America/St_Johns
    {  350, 5863,   86699,  8, }, // America/St_Kitts
    {  350, 5880,   86707,  9, }, // America/St_Lucia
    {  350,  855,   86716,  9, }, // America/St_Thomas
    {  350, 5913,   86725, 10, }, // America/St_Vincent
    {  350, 8530,   86735, 11, }, // America/Swift_Current
    {  350, 5461,   86746,  9, }, // America/Tegucigalpa
    {  350, 5756,   86755,  4, }, // America/Thule
    {  350,  313,   86759,  7, }, // America/Tijuana
    {  350,  608,   86766,  7, }, // America/Toronto
    {  350, 5688,   86773,  7, }, // America/Tortola
    {  350, 2093,   86780,  6, }, // America/Vancouver
    {  350, 2144,   86786,  9, }, // America/Whitehorse
    {  350,  734,   86795,  7, }, // America/Winnipeg
    {  350, 8552,   86802,  6, }, // America/Yakutat
    {  350, 6001,   86808,  5, }, // Antarctica/Casey
    {  350, 6094,   86813,  6, }, // Antarctica/Davis
    {  350, 6133,   86819, 15, }, // Antarctica/DumontDUrville
    {  350, 8568,   86834,  7, }, // Antarctica/Macquarie
    {  350, 7066,   86841,  5, }, // Antarctica/Mawson
    {  350, 7113,   86846,  7, }, // Antarctica/McMurdo
    {  350, 8589,   86853,  5, }, // Antarctica/Palmer
    {  350, 7226,   86858,  6, }, // Antarctica/Rothera
    {  350, 7295,   86864,  6, }, // Antarctica/Syowa
    {  350, 8607,   86870,  4, }, // Antarctica/Troll
    {  350, 7374,   86874,  6, }, // Antarctica/Vostok
    {  350, 1429,   86880, 10, }, // Arctic/Longyearbyen
    {  350, 5570,   86890,  3, }, // Asia/Aden
    {  350, 5395,   86893,  6, }, // Asia/Almaty
    {  350, 3968,   86899,  4, }, // Asia/Amman
    {  350, 5511,   86903,  5, }, // Asia/Anadyr
    {  350, 5523,   86908,  5, }, // Asia/Aqtau
    {  350, 5368,   86913,  6, }, // Asia/Aqtobe
    {  350,  964,   86919,  5, }, // Asia/Ashgabat
    {  350, 8624,   86924,  6, }, // Asia/Atyrau
    {  350, 3607,   86930,  4, }, // Asia/Baghdad
    {  350, 5534,   86934,  6, }, // Asia/Bahrain
    {  350, 3653,   86940,  4, }, // Asia/Baku
    {  350, 4466,   86944,  6, }, // Asia/Bangkok
    {  350, 3571,   86950,  6, }, // Asia/Barnaul
    {  350, 4086,   86956,  6, }, // Asia/Beirut
    {  350, 3778,   86962,  5, }, // Asia/Bishkek
    {  350, 5989,   86967,  6, }, // Asia/Brunei
    {  350, 4601,   86973,  5, }, // Asia/Chita
    {  350, 4511,   86978,  7, }, // Asia/Colombo
    {  350, 4540,   86985,  4, }, // Asia/Damascus
    {  350, 1093,   86989,  4, }, // Asia/Dhaka
    {  350, 6192,   86993,  4, }, // Asia/Dili
    {  350, 3596,   86997,  5, }, // Asia/Dubai
    {  350, 6159,   87002,  6, }, // Asia/Dushanbe
    {  350, 8636,   87008,  7, }, // Asia/Famagusta
    {  350, 8651,   87015,  3, }, // Asia/Gaza
    {  350, 4795,   87018,  5, }, // Asia/Hebron
    {  350, 1259,   87023, 15, }, // Asia/Ho_Chi_Minh
    {  350, 2704,   87038,  9, }, // Asia/Hong_Kong
    {  350, 4771,   87047,  4, }, // Asia/Hovd
    {  350, 4184,   87051,  8, }, // Asia/Irkutsk
    {  350, 7006,   87059,  6, }, // Asia/Jakarta
    {  350, 6992,   87065,  9, }, // Asia/Jayapura
    {  350, 1290,   87074,  7, }, // Asia/Jerusalem
    {  350, 3560,   87081,  5, }, // Asia/Kabul
    {  350, 4316,   87086,  7, }, // Asia/Kamchatka
    {  350, 4239,   87093,  6, }, // Asia/Karachi
    {  350, 1185,   87099,  6, }, // Asia/Kathmandu
    {  350, 8661,   87105,  6, }, // Asia/Khandyga
    {  350,  992,   87111,  6, }, // Asia/Kolkata
    {  350, 4197,   87117, 10, }, // Asia/Krasnoyarsk
    {  350, 4859,   87127, 12, }, // Asia/Kuala_Lumpur
    {  350, 5976,   87139,  7, }, // Asia/Kuching
    {  350, 5547,   87146,  5, }, // Asia/Kuwait
    {  350, 1211,   87151,  4, }, // Asia/Macau
    {  350, 4017,   87155,  4, }, // Asia/Magadan
    {  350, 1349,   87159,  7, }, // Asia/Makassar
    {  350, 7182,   87166,  5, }, // Asia/Manila
    {  350, 6920,   87171,  5, }, // Asia/Muscat
    {  350, 2554,   87176,  7, }, // Asia/Nicosia
    {  350, 8675,   87183, 11, }, // Asia/Novokuznetsk
    {  350, 4135,   87194, 11, }, // Asia/Novosibirsk
    {  350, 4229,   87205,  5, }, // Asia/Omsk
    {  350, 7145,   87210,  4, }, // Asia/Oral
    {  350, 6961,   87214,  7, }, // Asia/Phnom_Penh
    {  350, 8693,   87221,  8, }, // Asia/Pontianak
    {  350, 4214,   87229,  9, }, // Asia/Pyongyang
    {  350, 5559,   87238,  4, }, // Asia/Qatar
    {  350, 8708,   87242,  7, }, // Asia/Qostanay
    {  350, 4269,   87249,  9, }, // Asia/Qyzylorda
    {  350, 3584,   87258,  4, }, // Asia/Riyadh
    {  350, 4408,   87262,  5, }, // Asia/Sakhalin
    {  350, 7245,   87267,  6, }, // Asia/Samarkand
    {  350, 3279,   87273,  5, }, // Asia/Seoul
    {  350, 1053,   87278,  5, }, // Asia/Shanghai
    {  350, 3300,   87283,  7, }, // Asia/Singapore
    {  350, 4297,   87290, 13, }, // Asia/Srednekolymsk
    {  350, 3263,   87303,  6, }, // Asia/Taipei
    {  350, 4781,   87309,  5, }, // Asia/Tashkent
    {  350, 3915,   87314,  7, }, // Asia/Tbilisi
    {  350, 2772,   87321,  4, }, // Asia/Tehran
    {  350, 1317,   87325,  7, }, // Asia/Thimphu
    {  350, 2821,   87332,  5, }, // Asia/Tokyo
    {  350, 4572,   87337,  5, }, // Asia/Tomsk
    {  350, 1021,   87342, 11, }, // Asia/Ulaanbaatar
    {  350, 1159,   87353,  7, }, // Asia/Urumqi
    {  350, 8722,   87360,  8, }, // Asia/Ust-Nera
    {  350, 6977,   87368,  8, }, // Asia/Vientiane
    {  350, 4710,   87376, 10, }, // Asia/Vladivostok
    {  350, 4828,   87386,  6, }, // Asia/Yakutsk
    {  350, 1235,   87392,  5, }, // Asia/Yangon
    {  350, 3883,   87397, 10, }, // Asia/Yekaterinburg
    {  350, 3747,   87407,  6, }, // Asia/Yerevan
    {  350, 3663,   87413, 14, }, // Atlantic/Azores
    {  350, 5671,   87427,  6, }, // Atlantic/Bermuda
    {  350, 6628,   87433, 13, }, // Atlantic/Canary
    {  350, 3727,   87446,  7, }, // Atlantic/Cape_Verde
    {  350, 1395,   87453,  4, }, // Atlantic/Faroe
    {  350, 8736,   87457,  6, }, // Atlantic/Madeira
    {  350, 2748,   87463, 10, }, // Atlantic/Reykjavik
    {  350, 7272,   87473, 13, }, // Atlantic/South_Georgia
    {  350, 6821,   87486,  8, }, // Atlantic/St_Helena
    {  350, 6644,   87494,  5, }, // Atlantic/Stanley
    {  350, 1670,   87499,  6, }, // Australia/Adelaide
    {  350, 1635,   87505,  8, }, // Australia/Brisbane
    {  350, 1799,   87513,  9, }, // Australia/Broken_Hill
    {  350, 1583,   87522,  6, }, // Australia/Darwin
    {  350, 3637,   87528,  5, }, // Australia/Eucla
    {  350, 1516,   87533,  5, }, // Australia/Hobart
    {  350, 8753,   87538,  6, }, // Australia/Lindeman
    {  350, 1547,   87544, 12, }, // Australia/Lord_Howe
    {  350, 1727,   87556,  6, }, // Australia/Melbourne
    {  350, 1762,   87562,  4, }, // Australia/Perth
    {  350, 1463,   87566,  5, }, // Australia/Sydney
    {  350, 6441,   87571,  7, }, // Europe/Amsterdam
    {  350, 6216,   87578,  6, }, // Europe/Andorra
    {  350, 3620,   87584,  6, }, // Europe/Astrakhan
    {  350, 2300,   87590,  5, }, // Europe/Athens
    {  350, 6336,   87595,  5, }, // Europe/Belgrade
    {  350, 4757,   87600,  5, }, // Europe/Berlin
    {  350, 6488,   87605,  8, }, // Europe/Bratislava
    {  350, 2167,   87613,  5, }, // Europe/Brussels
    {  350, 3928,   87618,  6, }, // Europe/Bucharest
    {  350, 3806,   87624,  6, }, // Europe/Budapest
    {  350, 8772,   87630,  7, }, // Europe/Busingen
    {  350, 2583,   87637,  6, }, // Europe/Chisinau
    {  350, 6289,   87643,  7, }, // Europe/Copenhagen
    {  350, 2338,   87650,  5, }, // Europe/Dublin
    {  350, 6307,   87655,  7, }, // Europe/Gibraltar
    {  350, 8788,   87662,  5, }, // Europe/Guernsey
    {  350, 6612,   87667,  7, }, // Europe/Helsinki
    {  350, 8804,   87674,  7, }, // Europe/Isle_of_Man
    {  350, 1130,   87681,  8, }, // Europe/Istanbul
    {  350, 8823,   87689,  5, }, // Europe/Jersey
    {  350, 3979,   87694,  9, }, // Europe/Kaliningrad
    {  350, 8929,   87703,  5, }, // Europe/Kirov
    {  350, 2527,   87708,  4, }, // Europe/Kyiv
    {  350, 3213,   87712,  6, }, // Europe/Lisbon
    {  350, 6506,   87718,  8, }, // Europe/Ljubljana
    {  350, 2501,   87726,  6, }, // Europe/London
    {  350, 6365,   87732,  9, }, // Europe/Luxembourg
    {  350, 4892,   87741,  5, }, // Europe/Madrid
    {  350, 6397,   87746,  4, }, // Europe/Malta
    {  350, 6582,   87750,  8, }, // Europe/Mariehamn
    {  350, 3693,   87758,  5, }, // Europe/Minsk
    {  350, 6410,   87763,  5, }, // Europe/Monaco
    {  350, 3537,   87768,  6, }, // Europe/Moscow
    {  350, 6458,   87774,  5, }, // Europe/Oslo
    {  350, 4284,   87779,  4, }, // Europe/Paris
    {  350, 6424,   87783,  9, }, // Europe/Podgorica
    {  350, 6275,   87792,  4, }, // Europe/Prague
    {  350, 8837,   87796,  4, }, // Europe/Riga
    {  350, 6324,   87800,  4, }, // Europe/Rome
    {  350, 4331,   85323,  4, }, // Europe/Samara
    {  350, 6470,   87804,  8, }, // Europe/San_Marino
    {  350, 6245,   87812,  6, }, // Europe/Sarajevo
    {  350, 4451,   87818,  6, }, // Europe/Saratov
    {  350, 8849,   87824,  9, }, // Europe/Simferopol
    {  350, 6383,   87833,  6, }, // Europe/Skopje
    {  350, 6599,   87839,  5, }, // Europe/Sofia
    {  350, 6523,   87844,  8, }, // Europe/Stockholm
    {  350, 8867,   87852,  5, }, // Europe/Tallinn
    {  350, 6202,   87857,  5, }, // Europe/Tirane
    {  350, 8882,   87862, 10, }, // Europe/Ulyanovsk
    {  350, 6352,   87872,  5, }, // Europe/Vaduz
    {  350, 6567,   87877,  7, }, // Europe/Vatican
    {  350, 6231,   87884,  4, }, // Europe/Vienna
    {  350, 8899,   87888,  5, }, // Europe/Vilnius
    {  350, 4727,   87893,  8, }, // Europe/Volgograd
    {  350, 3190,   87901,  4, }, // Europe/Warsaw
    {  350, 6261,   87905,  5, }, // Europe/Zagreb
    {  350, 6540,   87910,  5, }, // Europe/Zurich
    {  350, 5078,   87915,  9, }, // Indian/Antananarivo
    {  350, 6947,   87924,  6, }, // Indian/Chagos
    {  350, 6046,   87930, 11, }, // Indian/Christmas
    {  350, 6063,   87941,  5, }, // Indian/Cocos
    {  350, 5029,   87946,  6, }, // Indian/Comoro
    {  350, 6661,   87952,  7, }, // Indian/Kerguelen
    {  350, 7260,   87959,  3, }, // Indian/Mahe
    {  350, 7050,   87962, 15, }, // Indian/Maldives
    {  350, 4069,   87977,  9, }, // Indian/Mauritius
    {  350, 5098,   87986,  5, }, // Indian/Mayotte
    {  350, 7211,   87991,  7, }, // Indian/Reunion
    {  350, 4422,   87998,  4, }, // Pacific/Apia
    {  350,  932,   88002,  6, }, // Pacific/Auckland
    {  350, 3706,   88008,  8, }, // Pacific/Bougainville
    {  350, 3015,   88016,  7, }, // Pacific/Chatham
    {  350, 3157,   88023,  4, }, // Pacific/Chuuk
    {  350, 2237,   88027,  8, }, // Pacific/Easter
    {  350, 7360,   88035,  4, }, // Pacific/Efate
    {  350, 7327,   88039,  6, }, // Pacific/Fakaofo
    {  350, 3902,   88045,  5, }, // Pacific/Fiji
    {  350, 7343,   88050,  7, }, // Pacific/Funafuti
    {  350, 6678,   88057,  7, }, // Pacific/Galapagos
    {  350, 6696,   88064, 10, }, // Pacific/Gambier
    {  350, 3822,   88074,  9, }, // Pacific/Guadalcanal
    {  350, 6018,   88083,  4, }, // Pacific/Guam
    {  350, 2723,   88087,  8, }, // Pacific/Honolulu
    {  350, 3049,   88095,  8, }, // Pacific/Kanton
    {  350, 3998,   88103, 10, }, // Pacific/Kiritimati
    {  350, 7019,   88113,  5, }, // Pacific/Kosrae
    {  350, 2842,   88118, 10, }, // Pacific/Kwajalein
    {  350, 4877,   88128,  7, }, // Pacific/Majuro
    {  350, 4051,   88135,  9, }, // Pacific/Marquesas
    {  350, 8914,   88144,  7, }, // Pacific/Midway
    {  350, 7084,   88151,  5, }, // Pacific/Nauru
    {  350, 7132,   88156,  5, }, // Pacific/Niue
    {  350, 4168,   88161,  6, }, // Pacific/Norfolk
    {  350, 7098,   88167,  5, }, // Pacific/Noumea
    {  350, 3126,   88172,  9, }, // Pacific/Pago_Pago
    {  350, 7155,   88181,  4, }, // Pacific/Palau
    {  350, 7194,   88185,  6, }, // Pacific/Pitcairn
    {  350, 3096,   88191,  6, }, // Pacific/Pohnpei
    {  350, 4807,   88197, 11, }, // Pacific/Port_Moresby
    {  350, 6076,   88208,  9, }, // Pacific/Rarotonga
    {  350, 6031,   88217,  6, }, // Pacific/Saipan
    {  350, 7312,   88223,  5, }, // Pacific/Tahiti
    {  350, 6712,   88228,  7, }, // Pacific/Tarawa
    {  350, 4583,   88235,  8, }, // Pacific/Tongatapu
    {  350, 7392,   88243,  4, }, // Pacific/Wake
    {  350, 7405,   88247,  5, }, // Pacific/Wallis
    {  351, 6788,   29964,  7, }, // Africa/Abidjan Hindi/Devanagari/India
    {  351, 6760,   88252,  5, }, // Africa/Accra
    {  351, 5059,   88257, 10, }, // Africa/Addis_Ababa
    {  351, 7420,   88267,  9, }, // Africa/Algiers
    {  351,   14,   88276,  7, }, // Africa/Asmara
    {  351,   44,   88283,  6, }, // Africa/Bamako
    {  351, 5257,   88289,  6, }, // Africa/Bangui
    {  351, 6746,   88295,  6, }, // Africa/Banjul
    {  351, 7435,   88301,  5, }, // Africa/Bissau
    {  351, 4957,   88306,  9, }, // Africa/Blantyre
    {  351, 5287,   88315, 11, }, // Africa/Brazzaville
    {  351, 4922,   88326,  9, }, // Africa/Bujumbura
    {  351, 2320,   88335,  5, }, // Africa/Cairo
    {  351, 4117,   30065, 11, }, // Africa/Casablanca
    {  351, 7449,   88340,  5, }, // Africa/Ceuta
    {  351, 6773,   88345,  8, }, // Africa/Conakry
    {  351, 6840,   88353,  4, }, // Africa/Dakar
    {  351, 5130,   88357, 11, }, // Africa/Dar_es_Salaam
    {  351, 5043,   88368,  6, }, // Africa/Djibouti
    {  351, 5243,   88374,  5, }, // Africa/Douala
    {  351, 5166,   88379,  8, }, // Africa/El_Aaiun
    {  351, 6853,   88387,  9, }, // Africa/Freetown
    {  351, 4906,   88396,  7, }, // Africa/Gaborone
    {  351, 5015,   88403,  5, }, // Africa/Harare
    {  351, 4479,   88408, 10, }, // Africa/Johannesburg
    {  351, 4499,   30154,  4, }, // Africa/Juba
    {  351, 5151,   88418,  6, }, // Africa/Kampala
    {  351, 4524,   30166,  7, }, // Africa/Khartoum
    {  351, 4973,   88424,  6, }, // Africa/Kigali
    {  351, 5306,   88430,  7, }, // Africa/Kinshasa
    {  351, 4744,   88437,  5, }, // Africa/Lagos
    {  351, 5336,   88442, 10, }, // Africa/Libreville
    {  351, 6869,   88452,  3, }, // Africa/Lome
    {  351, 5211,   88455,  6, }, // Africa/Luanda
    {  351, 4939,   88461,  9, }, // Africa/Lubumbashi
    {  351, 5001,   30221,  6, }, // Africa/Lusaka
    {  351, 5322,   88470,  5, }, // Africa/Malabo
    {  351, 4987,   88475,  6, }, // Africa/Maputo
    {  351, 5197,   30239,  6, }, // Africa/Maseru
    {  351, 5182,   88481,  7, }, // Africa/Mbabane
    {  351, 5113,   88488,  8, }, // Africa/Mogadishu
    {  351, 7034,   88496,  9, }, // Africa/Monrovia
    {  351, 3853,   88505,  6, }, // Africa/Nairobi
    {  351, 5271,   88511, 10, }, // Africa/Ndjamena
    {  351, 5354,   88521,  6, }, // Africa/Niamey
    {  351, 6803,   88527,  7, }, // Africa/Nouakchott
    {  351, 6727,   88534,  7, }, // Africa/Ouagadougou
    {  351, 5225,   88541, 11, }, // Africa/Porto-Novo
    {  351, 4435,   88552,  7, }, // Africa/Sao_Tome
    {  351, 2866,   88559,  8, }, // Africa/Tripoli
    {  351, 6554,   30328,  7, }, // Africa/Tunis
    {  351, 4152,   88567,  7, }, // Africa/Windhoek
    {  351,  132,   88574,  3, }, // America/Adak
    {  351, 3346,   30347,  6, }, // America/Anchorage
    {  351, 5607,   88577,  8, }, // America/Anguilla
    {  351, 5624,   88585,  7, }, // America/Antigua
    {  351, 4554,   88592,  9, }, // America/Araguaina
    {  351,  166,   88601, 11, }, // America/Argentina/Buenos_Aires
    {  351,   91,   30390, 10, }, // America/Argentina/Catamarca
    {  351,  270,   88612,  8, }, // America/Argentina/Cordoba
    {  351,  441,   88620,  5, }, // America/Argentina/Jujuy
    {  351, 7462,   88625,  8, }, // America/Argentina/La_Rioja
    {  351,  565,   88633,  8, }, // America/Argentina/Mendoza
    {  351, 7489,   88641, 12, }, // America/Argentina/Rio_Gallegos
    {  351, 7520,   30437,  6, }, // America/Argentina/Salta
    {  351, 7544,   88653,  9, }, // America/Argentina/San_Juan
    {  351, 5580,   88662,  7, }, // America/Argentina/San_Luis
    {  351, 7571,   88669,  6, }, // America/Argentina/Tucuman
    {  351, 7597,   88675,  7, }, // America/Argentina/Ushuaia
    {  351, 5640,   88682,  5, }, // America/Aruba
    {  351, 4252,   88687,  8, }, // America/Asuncion
    {  351,  237,   88695,  8, }, // America/Atikokan
    {  351, 3679,   88703,  5, }, // America/Bahia
    {  351, 7623,   88708, 15, }, // America/Bahia_Banderas
    {  351, 5654,   88723,  8, }, // America/Barbados
    {  351, 7646,   30526,  5, }, // America/Belem
    {  351, 5407,   88731,  6, }, // America/Belize
    {  351, 7660,   88737, 13, }, // America/Blanc-Sablon
    {  351, 7681,   88750, 10, }, // America/Boa_Vista
    {  351, 4361,   88760,  6, }, // America/Bogota
    {  351, 7699,   88766,  5, }, // America/Boise
    {  351, 7713,   88771, 15, }, // America/Cambridge_Bay
    {  351, 7735,   88786, 13, }, // America/Campo_Grande
    {  351, 3868,   88799,  6, }, // America/Cancun
    {  351, 4694,   88805,  6, }, // America/Caracas
    {  351, 4345,   30609,  5, }, // America/Cayenne
    {  351, 5496,   30614,  5, }, // America/Cayman
    {  351, 2260,   88811,  6, }, // America/Chicago
    {  351, 7756,   88817,  8, }, // America/Chihuahua
    {  351, 7774,   88825, 16, }, // America/Ciudad_Juarez
    {  351, 5422,   88841, 11, }, // America/Costa_Rica
    {  351, 7796,   30642,  8, }, // America/Creston
    {  351, 3791,   88852,  7, }, // America/Cuiaba
    {  351, 5723,   88859,  7, }, // America/Curacao
    {  351, 7812,   88866, 11, }, // America/Danmarkshavn
    {  351, 7833,   88877,  4, }, // America/Dawson
    {  351, 7848,   88881, 10, }, // America/Dawson_Creek
    {  351,  805,   88891,  5, }, // America/Denver
    {  351, 3465,   88896,  8, }, // America/Detroit
    {  351, 5739,   88904,  8, }, // America/Dominica
    {  351,  893,   88912,  6, }, // America/Edmonton
    {  351, 7869,   30723,  7, }, // America/Eirunepe
    {  351, 5441,   88918, 11, }, // America/El_Salvador
    {  351, 7886,   88929, 13, }, // America/Fort_Nelson
    {  351, 7906,   88942, 12, }, // America/Fortaleza
    {  351, 7924,   88954, 11, }, // America/Glace_Bay
    {  351, 6881,   88965,  9, }, // America/Goose_Bay
    {  351, 4612,   88974, 11, }, // America/Grand_Turk
    {  351, 5770,   30789,  8, }, // America/Grenada
    {  351, 5786,   88985,  9, }, // America/Guadeloupe
    {  351, 3760,   30806, 10, }, // America/Guatemala
    {  351, 4841,   88994,  9, }, // America/Guayaquil
    {  351, 6932,   89003,  5, }, // America/Guyana
    {  351, 1939,   89008, 10, }, // America/Halifax
    {  351, 2281,   89018,  5, }, // America/Havana
    {  351, 7942,   89023, 11, }, // America/Hermosillo
    {  351,  348,   89034, 13, }, // America/Indiana/Indianapolis
    {  351,  481,   89047, 15, }, // America/Indiana/Knox
    {  351, 7961,   89062, 17, }, // America/Indiana/Marengo
    {  351, 7985,   89079, 20, }, // America/Indiana/Petersburg
    {  351, 8012,   89099, 18, }, // America/Indiana/Tell_City
    {  351, 8038,   89117, 14, }, // America/Indiana/Vevay
    {  351, 8060,   89131, 18, }, // America/Indiana/Vincennes
    {  351, 8086,   89149, 17, }, // America/Indiana/Winamac
    {  351, 8110,   30999,  6, }, // America/Inuvik
    {  351,  660,   89166,  7, }, // America/Iqaluit
    {  351, 2799,   89173,  5, }, // America/Jamaica
    {  351, 5380,   89178,  7, }, // America/Juneau
    {  351,  521,   89185,  8, }, // America/Kentucky/Louisville
    {  351, 8125,   89193, 17, }, // America/Kentucky/Monticello
    {  351, 5704,   89210, 12, }, // America/Kralendijk
    {  351, 4376,   89222,  7, }, // America/La_Paz
    {  351, 7169,   31064,  4, }, // America/Lima
    {  351, 3239,   89229, 11, }, // America/Los_Angeles
    {  351, 5932,   89240, 22, }, // America/Lower_Princes
    {  351, 8153,   89262,  5, }, // America/Maceio
    {  351, 8168,   89267,  7, }, // America/Managua
    {  351, 1908,   89274,  4, }, // America/Manaus
    {  351, 5897,   89278,  7, }, // America/Marigot
    {  351, 5805,   89285,  9, }, // America/Martinique
    {  351, 8184,   89294,  9, }, // America/Matamoros
    {  351, 2917,   89303,  9, }, // America/Mazatlan
    {  351, 8202,   89312,  8, }, // America/Menominee
    {  351, 8220,   31158,  6, }, // America/Merida
    {  351, 8235,   89320, 12, }, // America/Metlakatla
    {  351, 2949,   89332, 13, }, // America/Mexico_City
    {  351, 4391,   89345,  7, }, // America/Miquelon
    {  351, 8254,   89352,  6, }, // America/Moncton
    {  351, 8270,   89358,  9, }, // America/Monterrey
    {  351, 4098,   89367, 11, }, // America/Montevideo
    {  351, 5824,   89378,  9, }, // America/Montserrat
    {  351, 5481,   89387,  5, }, // America/Nassau
    {  351, 2379,   89392,  9, }, // America/New_York
    {  351, 8288,   89314,  3, }, // America/Nome
    {  351, 1850,   89401,  8, }, // America/Noronha
    {  351, 8301,   89409, 20, }, // America/North_Dakota/Beulah
    {  351, 8329,   89429, 19, }, // America/North_Dakota/Center
    {  351, 8357,   89448, 24, }, // America/North_Dakota/New_Salem
    {  351,  393,   89472,  3, }, // America/Nuuk
    {  351, 8388,   89475,  9, }, // America/Ojinaga
    {  351, 2356,   89484,  5, }, // America/Panama
    {  351, 6173,   89489, 10, }, // America/Paramaribo
    {  351, 2973,   89499,  8, }, // America/Phoenix
    {  351, 3945,   89507, 14, }, // America/Port-au-Prince
    {  351, 5954,   89521, 15, }, // America/Port_of_Spain
    {  351, 8404,   89536, 13, }, // America/Porto_Velho
    {  351, 5843,   89549, 11, }, // America/Puerto_Rico
    {  351, 4030,   89560, 12, }, // America/Punta_Arenas
    {  351, 8424,   89572, 12, }, // America/Rankin_Inlet
    {  351, 8445,   89584,  7, }, // America/Recife
    {  351, 1995,   31431,  6, }, // America/Regina
    {  351, 8460,   89591,  9, }, // America/Resolute
    {  351,  695,   89600, 12, }, // America/Rio_Branco
    {  351, 8477,   89612,  8, }, // America/Santarem
    {  351, 2201,   89620,  9, }, // America/Santiago
    {  351, 6111,   89629, 13, }, // America/Santo_Domingo
    {  351, 1878,   89642,  9, }, // America/Sao_Paulo
    {  351, 6899,   89651, 13, }, // America/Scoresbysund
    {  351, 8494,   89664,  6, }, // America/Sitka
    {  351, 8508,   89670, 15, }, // America/St_Barthelemy
    {  351, 2061,   89685,  9, }, // America/St_Johns
    {  351, 5863,   89694, 10, }, // America/St_Kitts
    {  351, 5880,   89704, 11, }, // America/St_Lucia
    {  351,  855,   89715,  9, }, // America/St_Thomas
    {  351, 5913,   89724, 12, }, // America/St_Vincent
    {  351, 8530,   89736, 13, }, // America/Swift_Current
    {  351, 5461,   89749, 11, }, // America/Tegucigalpa
    {  351, 5756,   89760,  6, }, // America/Thule
    {  351,  313,   89766,  7, }, // America/Tijuana
    {  351,  608,   89773,  6, }, // America/Toronto
    {  351, 5688,   89779,  8, }, // America/Tortola
    {  351, 2093,   89787,  7, }, // America/Vancouver
    {  351, 2144,   89794, 11, }, // America/Whitehorse
    {  351,  734,   89805,  7, }, // America/Winnipeg
    {  351, 8552,   89812,  7, }, // America/Yakutat
    {  351, 6001,   31673,  4, }, // Antarctica/Casey
    {  351, 6094,   89819,  5, }, // Antarctica/Davis
    {  351, 6133,   89824, 19, }, // Antarctica/DumontDUrville
    {  351, 8568,   89843,  7, }, // Antarctica/Macquarie
    {  351, 7066,   89850,  4, }, // Antarctica/Mawson
    {  351, 7113,   89854,  9, }, // Antarctica/McMurdo
    {  351, 8589,   89863,  4, }, // Antarctica/Palmer
    {  351, 7226,   89867,  6, }, // Antarctica/Rothera
    {  351, 7295,   89873,  6, }, // Antarctica/Syowa
    {  351, 8607,   89879,  5, }, // Antarctica/Troll
    {  351, 7374,   89884,  7, }, // Antarctica/Vostok
    {  351, 1429,   89891, 13, }, // Arctic/Longyearbyen
    {  351, 5570,   89904,  4, }, // Asia/Aden
    {  351, 5395,   89908,  7, }, // Asia/Almaty
    {  351, 3968,   89915,  6, }, // Asia/Amman
    {  351, 5511,   89921,  6, }, // Asia/Anadyr
    {  351, 5523,   31778,  6, }, // Asia/Aqtau
    {  351, 5368,   89927,  6, }, // Asia/Aqtobe
    {  351,  964,   31791,  8, }, // Asia/Ashgabat
    {  351, 8624,   89933,  5, }, // Asia/Atyrau
    {  351, 3607,   89938,  5, }, // Asia/Baghdad
    {  351, 5534,   89943,  5, }, // Asia/Bahrain
    {  351, 3653,   88857,  4, }, // Asia/Baku
    {  351, 4466,   89948,  6, }, // Asia/Bangkok
    {  351, 3571,   89954,  6, }, // Asia/Barnaul
    {  351, 4086,   89960,  5, }, // Asia/Beirut
    {  351, 3778,   31836,  7, }, // Asia/Bishkek
    {  351, 5989,   89965,  7, }, // Asia/Brunei
    {  351, 4601,   89972,  6, }, // Asia/Chita
    {  351, 4511,   89978,  6, }, // Asia/Colombo
    {  351, 4540,   89984,  7, }, // Asia/Damascus
    {  351, 1093,   89991,  4, }, // Asia/Dhaka
    {  351, 6192,   89995,  4, }, // Asia/Dili
    {  351, 3596,   89999,  4, }, // Asia/Dubai
    {  351, 6159,   90003,  7, }, // Asia/Dushanbe
    {  351, 8636,   90010, 11, }, // Asia/Famagusta
    {  351, 8651,   90021,  5, }, // Asia/Gaza
    {  351, 4795,   90026,  7, }, // Asia/Hebron
    {  351, 1259,   90033, 16, }, // Asia/Ho_Chi_Minh
    {  351, 2704,   90049,  9, }, // Asia/Hong_Kong
    {  351, 4771,   90058,  5, }, // Asia/Hovd
    {  351, 4184,   90063, 10, }, // Asia/Irkutsk
    {  351, 7006,   90073,  7, }, // Asia/Jakarta
    {  351, 6992,   31943,  7, }, // Asia/Jayapura
    {  351, 1290,   90080,  6, }, // Asia/Jerusalem
    {  351, 3560,   90086,  5, }, // Asia/Kabul
    {  351, 4316,   90091,  7, }, // Asia/Kamchatka
    {  351, 4239,   31972,  5, }, // Asia/Karachi
    {  351, 1185,   90098,  8, }, // Asia/Kathmandu
    {  351, 8661,   90106,  7, }, // Asia/Khandyga
    {  351,  992,   90113,  7, }, // Asia/Kolkata
    {  351, 4197,   90120, 15, }, // Asia/Krasnoyarsk
    {  351, 4859,   90135, 10, }, // Asia/Kuala_Lumpur
    {  351, 5976,   32028,  6, }, // Asia/Kuching
    {  351, 5547,   32034,  5, }, // Asia/Kuwait
    {  351, 1211,   90145,  4, }, // Asia/Macau
    {  351, 4017,   32044,  7, }, // Asia/Magadan
    {  351, 1349,   90149,  6, }, // Asia/Makassar
    {  351, 7182,   32060,  5, }, // Asia/Manila
    {  351, 6920,   90155,  5, }, // Asia/Muscat
    {  351, 2554,   90160,  8, }, // Asia/Nicosia
    {  351, 8675,   90168, 16, }, // Asia/Novokuznetsk
    {  351, 4135,   90184, 13, }, // Asia/Novosibirsk
    {  351, 4229,   90197,  6, }, // Asia/Omsk
    {  351, 7145,   90203,  3, }, // Asia/Oral
    {  351, 6961,   90206,  9, }, // Asia/Phnom_Penh
    {  351, 8693,   90215,  9, }, // Asia/Pontianak
    {  351, 4214,   90224, 10, }, // Asia/Pyongyang
    {  351, 5559,   90234,  3, }, // Asia/Qatar
    {  351, 8708,   90237,  8, }, // Asia/Qostanay
    {  351, 4269,   90245, 11, }, // Asia/Qyzylorda
    {  351, 3584,   90256,  5, }, // Asia/Riyadh
    {  351, 4408,   90261,  6, }, // Asia/Sakhalin
    {  351, 7245,   90267,  6, }, // Asia/Samarkand
    {  351, 3279,   90273,  4, }, // Asia/Seoul
    {  351, 1053,   90277,  5, }, // Asia/Shanghai
    {  351, 3300,   32190,  8, }, // Asia/Singapore
    {  351, 4297,   90282, 16, }, // Asia/Srednekolymsk
    {  351, 3263,   90298,  6, }, // Asia/Taipei
    {  351, 4781,   90304,  6, }, // Asia/Tashkent
    {  351, 3915,   90310,  7, }, // Asia/Tbilisi
    {  351, 2772,   32236,  6, }, // Asia/Tehran
    {  351, 1317,   90317,  5, }, // Asia/Thimphu
    {  351, 2821,   90322,  6, }, // Asia/Tokyo
    {  351, 4572,   90328,  7, }, // Asia/Tomsk
    {  351, 1021,   90335,  8, }, // Asia/Ulaanbaatar
    {  351, 1159,   90343,  7, }, // Asia/Urumqi
    {  351, 8722,   90350, 10, }, // Asia/Ust-Nera
    {  351, 6977,   90360,  9, }, // Asia/Vientiane
    {  351, 4710,   90369, 13, }, // Asia/Vladivostok
    {  351, 4828,   32305,  9, }, // Asia/Yakutsk
    {  351, 1235,   32314,  5, }, // Asia/Yangon
    {  351, 3883,   90382, 13, }, // Asia/Yekaterinburg
    {  351, 3747,   90395,  7, }, // Asia/Yerevan
    {  351, 3663,   90402,  7, }, // Atlantic/Azores
    {  351, 5671,   90409,  6, }, // Atlantic/Bermuda
    {  351, 6628,   90415,  6, }, // Atlantic/Canary
    {  351, 3727,   90421,  8, }, // Atlantic/Cape_Verde
    {  351, 1395,   90429,  5, }, // Atlantic/Faroe
    {  351, 8736,   90434,  5, }, // Atlantic/Madeira
    {  351, 2748,   90439,  9, }, // Atlantic/Reykjavik
    {  351, 7272,   90448, 15, }, // Atlantic/South_Georgia
    {  351, 6821,   90463, 11, }, // Atlantic/St_Helena
    {  351, 6644,   90474,  7, }, // Atlantic/Stanley
    {  351, 1670,   90481,  6, }, // Australia/Adelaide
    {  351, 1635,   90487,  8, }, // Australia/Brisbane
    {  351, 1799,   90495, 10, }, // Australia/Broken_Hill
    {  351, 1583,   90505,  7, }, // Australia/Darwin
    {  351, 3637,   32452,  6, }, // Australia/Eucla
    {  351, 1516,   90512,  7, }, // Australia/Hobart
    {  351, 8753,   90519,  8, }, // Australia/Lindeman
    {  351, 1547,   90527, 10, }, // Australia/Lord_Howe
    {  351, 1727,   90537,  8, }, // Australia/Melbourne
    {  351, 1762,   90545,  4, }, // Australia/Perth
    {  351, 1463,   32494,  5, }, // Australia/Sydney
    {  351, 6441,   90549, 10, }, // Europe/Amsterdam
    {  351, 6216,   90559,  6, }, // Europe/Andorra
    {  351, 3620,   90565, 10, }, // Europe/Astrakhan
    {  351, 2300,   90575,  5, }, // Europe/Athens
    {  351, 6336,   32532,  8, }, // Europe/Belgrade
    {  351, 4757,   90580,  6, }, // Europe/Berlin
    {  351, 6488,   90586, 12, }, // Europe/Bratislava
    {  351, 2167,   90598,  9, }, // Europe/Brussels
    {  351, 3928,   90607, 10, }, // Europe/Bucharest
    {  351, 3806,   90617,  9, }, // Europe/Budapest
    {  351, 8772,   90626, 10, }, // Europe/Busingen
    {  351, 2583,   90636,  7, }, // Europe/Chisinau
    {  351, 6289,   90643,  9, }, // Europe/Copenhagen
    {  351, 2338,   90652,  5, }, // Europe/Dublin
    {  351, 6307,   90657, 10, }, // Europe/Gibraltar
    {  351, 8788,   90667,  6, }, // Europe/Guernsey
    {  351, 6612,   90673,  8, }, // Europe/Helsinki
    {  351, 8804,   90681, 11, }, // Europe/Isle_of_Man
    {  351, 1130,   90692,  9, }, // Europe/Istanbul
    {  351, 8823,   90701,  5, }, // Europe/Jersey
    {  351, 3979,   90706, 12, }, // Europe/Kaliningrad
    {  351, 8929,   90718,  5, }, // Europe/Kirov
    {  351, 2527,   90723,  3, }, // Europe/Kyiv
    {  351, 3213,   32690,  6, }, // Europe/Lisbon
    {  351, 6506,   90726, 11, }, // Europe/Ljubljana
    {  351, 2501,   90737,  4, }, // Europe/London
    {  351, 6365,   90741, 10, }, // Europe/Luxembourg
    {  351, 4892,   90751,  7, }, // Europe/Madrid
    {  351, 6397,   32731,  6, }, // Europe/Malta
    {  351, 6582,   90758,  9, }, // Europe/Mariehamn
    {  351, 3693,   90767,  6, }, // Europe/Minsk
    {  351, 6410,   90773,  6, }, // Europe/Monaco
    {  351, 3537,   90779,  6, }, // Europe/Moscow
    {  351, 6458,   90785,  5, }, // Europe/Oslo
    {  351, 4284,   32766,  5, }, // Europe/Paris
    {  351, 6424,   90790, 10, }, // Europe/Podgorica
    {  351, 6275,   90800,  5, }, // Europe/Prague
    {  351, 8837,   90805,  4, }, // Europe/Riga
    {  351, 6324,   90432,  3, }, // Europe/Rome
    {  351, 4331,   90809,  5, }, // Europe/Samara
    {  351, 6470,   90814, 10, }, // Europe/San_Marino
    {  351, 6245,   90824,  8, }, // Europe/Sarajevo
    {  351, 4451,   90832,  7, }, // Europe/Saratov
    {  351, 8849,   90839, 12, }, // Europe/Simferopol
    {  351, 6383,   90851,  8, }, // Europe/Skopje
    {  351, 6599,   90859,  7, }, // Europe/Sofia
    {  351, 6523,   90866,  8, }, // Europe/Stockholm
    {  351, 8867,   90874,  5, }, // Europe/Tallinn
    {  351, 6202,   90879,  6, }, // Europe/Tirane
    {  351, 8882,   90885, 12, }, // Europe/Ulyanovsk
    {  351, 6352,   90897,  6, }, // Europe/Vaduz
    {  351, 6567,   90903,  6, }, // Europe/Vatican
    {  351, 6231,   90909,  5, }, // Europe/Vienna
    {  351, 8899,   90914,  8, }, // Europe/Vilnius
    {  351, 4727,   90922, 11, }, // Europe/Volgograd
    {  351, 3190,   90933,  5, }, // Europe/Warsaw
    {  351, 6261,   90938,  8, }, // Europe/Zagreb
    {  351, 6540,   90946,  9, }, // Europe/Zurich
    {  351, 5078,   90955, 11, }, // Indian/Antananarivo
    {  351, 6947,   90966,  5, }, // Indian/Chagos
    {  351, 6046,   90971,  7, }, // Indian/Christmas
    {  351, 6063,   90978,  5, }, // Indian/Cocos
    {  351, 5029,   90983,  6, }, // Indian/Comoro
    {  351, 6661,   90989,  7, }, // Indian/Kerguelen
    {  351, 7260,   32983,  4, }, // Indian/Mahe
    {  351, 7050,   90996,  6, }, // Indian/Maldives
    {  351, 4069,   91002,  6, }, // Indian/Mauritius
    {  351, 5098,   91008,  8, }, // Indian/Mayotte
    {  351, 7211,   91016,  8, }, // Indian/Reunion
    {  351, 4422,   91024,  5, }, // Pacific/Apia
    {  351,  932,   91029,  6, }, // Pacific/Auckland
    {  351, 3706,   91035,  8, }, // Pacific/Bougainville
    {  351, 3015,   91043,  4, }, // Pacific/Chatham
    {  351, 3157,   91047,  2, }, // Pacific/Chuuk
    {  351, 2237,   91049,  5, }, // Pacific/Easter
    {  351, 7360,   91054,  5, }, // Pacific/Efate
    {  351, 7327,   91059,  9, }, // Pacific/Fakaofo
    {  351, 3902,   91068,  5, }, // Pacific/Fiji
    {  351, 7343,   91073, 12, }, // Pacific/Funafuti
    {  351, 6678,   91085,  9, }, // Pacific/Galapagos
    {  351, 6696,   91094,  7, }, // Pacific/Gambier
    {  351, 3822,   91101, 10, }, // Pacific/Guadalcanal
    {  351, 6018,   31112,  4, }, // Pacific/Guam
    {  351, 2723,   91111,  8, }, // Pacific/Honolulu
    {  351, 3049,   91119,  7, }, // Pacific/Kanton
    {  351, 3998,   33132, 10, }, // Pacific/Kiritimati
    {  351, 7019,   91126,  6, }, // Pacific/Kosrae
    {  351, 2842,   91132, 10, }, // Pacific/Kwajalein
    {  351, 4877,   91142,  6, }, // Pacific/Majuro
    {  351, 4051,   91148,  8, }, // Pacific/Marquesas
    {  351, 8914,   91156,  5, }, // Pacific/Midway
    {  351, 7084,   91161,  4, }, // Pacific/Nauru
    {  351, 7132,   91165,  4, }, // Pacific/Niue
    {  351, 4168,   91169,  7, }, // Pacific/Norfolk
    {  351, 7098,   91176,  6, }, // Pacific/Noumea
    {  351, 3126,   91182,  9, }, // Pacific/Pago_Pago
    {  351, 7155,   91191,  4, }, // Pacific/Palau
    {  351, 7194,   33210,  8, }, // Pacific/Pitcairn
    {  351, 3096,   91195,  6, }, // Pacific/Pohnpei
    {  351, 4807,   91201, 14, }, // Pacific/Port_Moresby
    {  351, 6076,   91215,  9, }, // Pacific/Rarotonga
    {  351, 6031,   91224,  6, }, // Pacific/Saipan
    {  351, 7312,   91230,  6, }, // Pacific/Tahiti
    {  351, 6712,   91236,  6, }, // Pacific/Tarawa
    {  351, 4583,   91242,  9, }, // Pacific/Tongatapu
    {  351, 7392,   33268,  3, }, // Pacific/Wake
    {  351, 7405,   33271,  5, }, // Pacific/Wallis
    {  352,   14,   91251,  6, }, // Africa/Asmara Hindi/Latin/India
    {  352, 5225,   26542, 10, }, // Africa/Porto-Novo
    {  352, 4435,   91257,  8, }, // Africa/Sao_Tome
    {  352, 4252,   61453,  8, }, // America/Asuncion
    {  352,  237,   91265, 13, }, // America/Atikokan
    {  352, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  352, 7660,   44149, 12, }, // America/Blanc-Sablon
    {  352, 3868,   33314,  6, }, // America/Cancun
    {  352, 7774,   91278, 13, }, // America/Ciudad_Juarez
    {  352, 5723,   91291,  7, }, // America/Curacao
    {  352,  481,      73, 13, }, // America/Indiana/Knox
    {  352, 7961,      86, 16, }, // America/Indiana/Marengo
    {  352, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  352, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  352, 8038,     139, 14, }, // America/Indiana/Vevay
    {  352, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  352, 8086,     171, 16, }, // America/Indiana/Winamac
    {  352, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  352, 5932,     207, 22, }, // America/Lower_Princes
    {  352, 8220,    7126,  6, }, // America/Merida
    {  352, 1850,     235, 19, }, // America/Noronha
    {  352, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  352, 8329,     274, 20, }, // America/North_Dakota/Center
    {  352, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  352,  393,     317,  4, }, // America/Nuuk
    {  352, 6899,     321, 16, }, // America/Scoresbysund
    {  352, 8508,   91298, 13, }, // America/St_Barthelemy
    {  352, 2061,   60952,  9, }, // America/St_Johns
    {  352, 5863,   60961,  8, }, // America/St_Kitts
    {  352, 5880,   60969,  8, }, // America/St_Lucia
    {  352,  855,   60977,  9, }, // America/St_Thomas
    {  352, 5913,   60986, 10, }, // America/St_Vincent
    {  352, 6133,   91311, 14, }, // Antarctica/DumontDUrville
    {  352, 8568,     416, 16, }, // Antarctica/Macquarie
    {  352, 5523,   91325,  5, }, // Asia/Aqtau
    {  352, 1259,   91330,  6, }, // Asia/Ho_Chi_Minh
    {  352, 4771,   26835,  5, }, // Asia/Hovd
    {  352, 1185,     448,  9, }, // Asia/Kathmandu
    {  352,  992,     457,  7, }, // Asia/Kolkata
    {  352, 1211,    4321,  5, }, // Asia/Macau
    {  352, 8708,   91336,  8, }, // Asia/Qostanay
    {  352, 4269,   26897,  9, }, // Asia/Qyzylorda
    {  352, 1235,     477,  6, }, // Asia/Yangon
    {  352, 1395,   91344,  6, }, // Atlantic/Faroe
    {  352, 6821,   60996,  9, }, // Atlantic/St_Helena
    {  352, 1547,     498, 16, }, // Australia/Lord_Howe
    {  352, 2527,     514,  4, }, // Europe/Kyiv
    {  352, 6046,     518, 16, }, // Indian/Christmas
    {  352, 6063,     534, 13, }, // Indian/Cocos
    {  352, 7211,   91350,  7, }, // Indian/Reunion
    {  352, 3157,   91357,  4, }, // Pacific/Chuuk
    {  352, 2237,     559, 13, }, // Pacific/Easter
    {  352, 2723,    4611,  8, }, // Pacific/Honolulu
    {  352, 3049,    4619,  9, }, // Pacific/Kanton
    {  352, 4168,     572, 14, }, // Pacific/Norfolk
    {  352, 3096,    8429,  6, }, // Pacific/Pohnpei
    {  352, 7392,     593, 11, }, // Pacific/Wake
    {  353, 5059,   91361, 12, }, // Africa/Addis_Ababa Hungarian/Latin/Hungary
    {  353, 7420,   91373,  5, }, // Africa/Algiers
    {  353,   14,   91378,  7, }, // Africa/Asmara
    {  353, 2320,   91385,  5, }, // Africa/Cairo
    {  353, 5130,   91390, 13, }, // Africa/Dar_es_Salaam
    {  353, 5043,   91403,  8, }, // Africa/Djibouti
    {  353, 5166,   91411,  7, }, // Africa/El_Aaiun
    {  353, 4524,   91418,  6, }, // Africa/Khartoum
    {  353, 5322,   91424,  6, }, // Africa/Malabo
    {  353, 4435,       6,  8, }, // Africa/Sao_Tome
    {  353, 6554,   91430,  6, }, // Africa/Tunis
    {  353, 4554,   26559,  9, }, // America/Araguaina
    {  353,  270,   20479,  7, }, // America/Argentina/Cordoba
    {  353, 7489,   44137, 12, }, // America/Argentina/Rio_Gallegos
    {  353, 7571,   20486,  7, }, // America/Argentina/Tucuman
    {  353, 4252,      14,  8, }, // America/Asuncion
    {  353,  237,      22,  8, }, // America/Atikokan
    {  353, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  353, 7646,   35896,  5, }, // America/Belem
    {  353, 4361,   20507,  6, }, // America/Bogota
    {  353, 3868,      47,  6, }, // America/Cancun
    {  353, 5496,   91436, 15, }, // America/Cayman
    {  353, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  353, 3791,   26574,  6, }, // America/Cuiaba
    {  353, 5723,      66,  7, }, // America/Curacao
    {  353, 5739,   21666,  8, }, // America/Dominica
    {  353, 7869,   26580,  8, }, // America/Eirunepe
    {  353, 5441,    6806,  8, }, // America/El_Salvador
    {  353, 2281,   63505,  7, }, // America/Havana
    {  353,  481,      73, 13, }, // America/Indiana/Knox
    {  353, 7961,      86, 16, }, // America/Indiana/Marengo
    {  353, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  353, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  353, 8038,     139, 14, }, // America/Indiana/Vevay
    {  353, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  353, 8086,     171, 16, }, // America/Indiana/Winamac
    {  353, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  353, 5932,     207, 22, }, // America/Lower_Princes
    {  353, 8153,   26588,  6, }, // America/Maceio
    {  353, 2917,   26603,  8, }, // America/Mazatlan
    {  353, 8220,     229,  6, }, // America/Merida
    {  353, 2949,   91451, 11, }, // America/Mexico_City
    {  353, 1850,     235, 19, }, // America/Noronha
    {  353, 8301,   91462, 20, }, // America/North_Dakota/Beulah
    {  353, 8329,   91482, 20, }, // America/North_Dakota/Center
    {  353, 8357,   91502, 23, }, // America/North_Dakota/New_Salem
    {  353,  393,     317,  4, }, // America/Nuuk
    {  353,  695,   74384, 10, }, // America/Rio_Branco
    {  353, 1878,   26712,  9, }, // America/Sao_Paulo
    {  353, 6899,     321, 16, }, // America/Scoresbysund
    {  353, 8508,   26721, 16, }, // America/St_Barthelemy
    {  353, 2061,     351, 10, }, // America/St_Johns
    {  353, 5863,     361,  9, }, // America/St_Kitts
    {  353, 5880,     370,  9, }, // America/St_Lucia
    {  353,  855,     379, 10, }, // America/St_Thomas
    {  353, 5913,     389, 11, }, // America/St_Vincent
    {  353, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  353, 7374,   91525,  7, }, // Antarctica/Vostok
    {  353, 5570,   91532,  4, }, // Asia/Aden
    {  353, 5395,   58773,  8, }, // Asia/Almaty
    {  353, 3968,   57924,  5, }, // Asia/Amman
    {  353, 5511,    7600,  6, }, // Asia/Anadyr
    {  353, 5523,    5609,  5, }, // Asia/Aqtau
    {  353, 5368,   63587,  6, }, // Asia/Aqtobe
    {  353,  964,   26803,  7, }, // Asia/Ashgabat
    {  353, 8624,    7612,  6, }, // Asia/Atyrau
    {  353, 3607,    4162,  6, }, // Asia/Baghdad
    {  353, 5534,    4168,  7, }, // Asia/Bahrain
    {  353, 4086,   57950,  6, }, // Asia/Beirut
    {  353, 3778,   91536,  6, }, // Asia/Bishkek
    {  353, 4601,   91542,  5, }, // Asia/Chita
    {  353, 4540,   91547, 10, }, // Asia/Damascus
    {  353, 1093,   91557,  5, }, // Asia/Dhaka
    {  353, 3596,   57968,  5, }, // Asia/Dubai
    {  353, 6159,   91562,  7, }, // Asia/Dushanbe
    {  353, 8651,   91569,  4, }, // Asia/Gaza
    {  353, 1259,   91573, 16, }, // Asia/Ho_Chi_Minh
    {  353, 2704,    4231,  8, }, // Asia/Hong_Kong
    {  353, 4184,   91589,  8, }, // Asia/Irkutsk
    {  353, 1290,   91597, 10, }, // Asia/Jerusalem
    {  353, 4316,   91607,  9, }, // Asia/Kamchatka
    {  353, 4239,   91616,  7, }, // Asia/Karachi
    {  353, 1185,    5711,  8, }, // Asia/Kathmandu
    {  353, 8661,   33524,  7, }, // Asia/Khandyga
    {  353,  992,   64334,  8, }, // Asia/Kolkata
    {  353, 4197,   91623, 13, }, // Asia/Krasnoyarsk
    {  353, 5976,   91636,  7, }, // Asia/Kuching
    {  353, 5547,   63965,  6, }, // Asia/Kuwait
    {  353, 1211,   91643,  5, }, // Asia/Macau
    {  353, 4017,   20850,  7, }, // Asia/Magadan
    {  353, 1349,    7736,  7, }, // Asia/Makassar
    {  353, 6920,   91648,  7, }, // Asia/Muscat
    {  353, 8675,   91655, 12, }, // Asia/Novokuznetsk
    {  353, 4135,   91667, 13, }, // Asia/Novosibirsk
    {  353, 4229,   91680,  5, }, // Asia/Omsk
    {  353, 4214,   91685,  7, }, // Asia/Pyongyang
    {  353, 5559,    4332,  5, }, // Asia/Qatar
    {  353, 8708,   91692,  9, }, // Asia/Qostanay
    {  353, 4269,    7778,  9, }, // Asia/Qyzylorda
    {  353, 3584,   58079,  5, }, // Asia/Riyadh
    {  353, 4408,   91701,  8, }, // Asia/Sakhalin
    {  353, 7245,   91709, 10, }, // Asia/Samarkand
    {  353, 3279,   91719,  5, }, // Asia/Seoul
    {  353, 1053,   91724,  7, }, // Asia/Shanghai
    {  353, 3300,   91731,  9, }, // Asia/Singapore
    {  353, 4297,   91740, 15, }, // Asia/Srednekolymsk
    {  353, 3263,    7830,  6, }, // Asia/Taipei
    {  353, 4781,   91755,  7, }, // Asia/Tashkent
    {  353, 3915,   91762,  8, }, // Asia/Tbilisi
    {  353, 2772,   58125,  7, }, // Asia/Tehran
    {  353, 1317,   91770,  5, }, // Asia/Thimphu
    {  353, 2821,   91775,  5, }, // Asia/Tokyo
    {  353, 4572,   91780,  6, }, // Asia/Tomsk
    {  353, 1021,   91786,  9, }, // Asia/Ulaanbaatar
    {  353, 1159,   91795,  7, }, // Asia/Urumqi
    {  353, 8722,   91802, 11, }, // Asia/Ust-Nera
    {  353, 6977,   20929,  8, }, // Asia/Vientiane
    {  353, 4710,   91813, 13, }, // Asia/Vladivostok
    {  353, 4828,   91826,  8, }, // Asia/Yakutsk
    {  353, 1235,     477,  6, }, // Asia/Yangon
    {  353, 3883,   91834, 14, }, // Asia/Yekaterinburg
    {  353, 3747,   91848,  7, }, // Asia/Yerevan
    {  353, 3663,   91855, 14, }, // Atlantic/Azores
    {  353, 6628,   91869, 15, }, // Atlantic/Canary
    {  353, 3727,   91884, 18, }, // Atlantic/Cape_Verde
    {  353, 1395,   91902,  6, }, // Atlantic/Faroe
    {  353, 2748,   36176,  9, }, // Atlantic/Reykjavik
    {  353, 7272,   91908, 12, }, // Atlantic/South_Georgia
    {  353, 6821,   91920, 11, }, // Atlantic/St_Helena
    {  353, 6441,   91931, 10, }, // Europe/Amsterdam
    {  353, 3620,   91941,  9, }, // Europe/Astrakhan
    {  353, 2300,   58255,  5, }, // Europe/Athens
    {  353, 6336,   91950,  7, }, // Europe/Belgrade
    {  353, 6488,   91957,  7, }, // Europe/Bratislava
    {  353, 2167,   91964,  8, }, // Europe/Brussels
    {  353, 3928,   27029,  8, }, // Europe/Bucharest
    {  353, 8772,   27037,  8, }, // Europe/Busingen
    {  353, 6289,   91972, 10, }, // Europe/Copenhagen
    {  353, 6307,   91982,  9, }, // Europe/Gibraltar
    {  353, 8804,   91991, 10, }, // Europe/Isle_of_Man
    {  353, 1130,   92001,  9, }, // Europe/Istanbul
    {  353, 3979,   92010, 12, }, // Europe/Kaliningrad
    {  353, 2527,   33723,  5, }, // Europe/Kyiv
    {  353, 3213,   92022,  9, }, // Europe/Lisbon
    {  353, 6365,    4513,  9, }, // Europe/Luxembourg
    {  353, 6397,   92031,  5, }, // Europe/Malta
    {  353, 3693,   92036,  6, }, // Europe/Minsk
    {  353, 3537,   92042,  7, }, // Europe/Moscow
    {  353, 4284,   92049,  6, }, // Europe/Paris
    {  353, 6275,   92055,  5, }, // Europe/Prague
    {  353, 6324,   92060,  4, }, // Europe/Rome
    {  353, 4331,   92064,  7, }, // Europe/Samara
    {  353, 6245,   92071,  9, }, // Europe/Sarajevo
    {  353, 4451,   92080,  8, }, // Europe/Saratov
    {  353, 8849,   92088, 11, }, // Europe/Simferopol
    {  353, 6383,   92099,  7, }, // Europe/Skopje
    {  353, 6599,   92106,  6, }, // Europe/Sofia
    {  353, 8867,   22925,  6, }, // Europe/Tallinn
    {  353, 6202,   21213,  6, }, // Europe/Tirane
    {  353, 8882,   92112, 10, }, // Europe/Ulyanovsk
    {  353, 6567,   58354,  7, }, // Europe/Vatican
    {  353, 6231,   92122,  4, }, // Europe/Vienna
    {  353, 4727,   92126,  9, }, // Europe/Volgograd
    {  353, 3190,   92135,  5, }, // Europe/Warsaw
    {  353, 6261,   92140,  6, }, // Europe/Zagreb
    {  353, 6540,    4577,  6, }, // Europe/Zurich
    {  353, 6046,   92146, 16, }, // Indian/Christmas
    {  353, 6063,   92162, 13, }, // Indian/Cocos
    {  353, 5029,   92175,  6, }, // Indian/Comoro
    {  353, 7050,   92181, 15, }, // Indian/Maldives
    {  353, 7211,     547,  7, }, // Indian/Reunion
    {  353, 3015,   92196, 16, }, // Pacific/Chatham
    {  353, 3157,   91357,  4, }, // Pacific/Chuuk
    {  353, 2237,   92212, 15, }, // Pacific/Easter
    {  353, 3902,   92227,  6, }, // Pacific/Fiji
    {  353, 6678,   92233, 18, }, // Pacific/Galapagos
    {  353, 6696,   92251, 16, }, // Pacific/Gambier
    {  353, 2723,    4611,  8, }, // Pacific/Honolulu
    {  353, 3049,    4619,  9, }, // Pacific/Kanton
    {  353, 3998,   92267, 17, }, // Pacific/Kiritimati
    {  353, 7019,   92284, 15, }, // Pacific/Kosrae
    {  353, 2842,   92299, 16, }, // Pacific/Kwajalein
    {  353, 4877,   92315, 13, }, // Pacific/Majuro
    {  353, 4051,   92328, 18, }, // Pacific/Marquesas
    {  353, 8914,   92346, 15, }, // Pacific/Midway
    {  353, 7194,   92361, 17, }, // Pacific/Pitcairn
    {  353, 3096,   92378, 15, }, // Pacific/Pohnpei
    {  353, 7392,   92393, 11, }, // Pacific/Wake
    {  354, 7420,   92404, 11, }, // Africa/Algiers Icelandic/Latin/Iceland
    {  354,   14,       0,  6, }, // Africa/Asmara
    {  354, 7435,   92415,  5, }, // Africa/Bissau
    {  354, 2320,   92420,  5, }, // Africa/Cairo
    {  354, 5043,   92425,  7, }, // Africa/Djibouti
    {  354, 4479,   92432, 14, }, // Africa/Johannesburg
    {  354, 4435,   92446,  8, }, // Africa/Sao_Tome
    {  354, 2866,   92454,  7, }, // Africa/Tripoli
    {  354, 6554,   92461,  9, }, // Africa/Tunis
    {  354, 5607,   92470,  8, }, // America/Anguilla
    {  354, 5624,   92478,  7, }, // America/Antigua
    {  354, 5640,   92485,  5, }, // America/Aruba
    {  354, 4252,      14,  8, }, // America/Asuncion
    {  354,  237,      22,  8, }, // America/Atikokan
    {  354, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  354, 5407,   92490,  5, }, // America/Belize
    {  354, 3868,   92495,  6, }, // America/Cancun
    {  354, 5496,   92501, 12, }, // America/Cayman
    {  354, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  354, 5422,   92513,  9, }, // America/Costa_Rica
    {  354, 5723,      66,  7, }, // America/Curacao
    {  354, 5739,   92522,  8, }, // America/Dominica
    {  354, 5786,   92530,  8, }, // America/Guadeloupe
    {  354, 3760,   33338,  9, }, // America/Guatemala
    {  354, 6932,   92538,  7, }, // America/Guyana
    {  354,  481,      73, 13, }, // America/Indiana/Knox
    {  354, 7961,      86, 16, }, // America/Indiana/Marengo
    {  354, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  354, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  354, 8038,     139, 14, }, // America/Indiana/Vevay
    {  354, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  354, 8086,     171, 16, }, // America/Indiana/Winamac
    {  354, 2799,   92545,  7, }, // America/Jamaica
    {  354, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  354, 5932,     207, 22, }, // America/Lower_Princes
    {  354, 5805,   92552,  8, }, // America/Martinique
    {  354, 8220,    7126,  6, }, // America/Merida
    {  354, 2949,   92560, 10, }, // America/Mexico_City
    {  354, 1850,     235, 19, }, // America/Noronha
    {  354, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  354, 8329,     274, 20, }, // America/North_Dakota/Center
    {  354, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  354,  393,     317,  4, }, // America/Nuuk
    {  354, 5843,   92570, 11, }, // America/Puerto_Rico
    {  354, 6899,     321, 16, }, // America/Scoresbysund
    {  354, 8508,   92581, 25, }, // America/St_Barthelemy
    {  354, 2061,     351, 10, }, // America/St_Johns
    {  354, 5863,   92606, 12, }, // America/St_Kitts
    {  354, 5880,   92618, 12, }, // America/St_Lucia
    {  354,  855,   92630, 13, }, // America/St_Thomas
    {  354, 5913,   92643, 14, }, // America/St_Vincent
    {  354, 5688,   92657,  7, }, // America/Tortola
    {  354, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  354, 3607,    4162,  6, }, // Asia/Baghdad
    {  354, 5534,   63955,  6, }, // Asia/Bahrain
    {  354, 3653,   20747,  4, }, // Asia/Baku
    {  354, 4086,   92664,  6, }, // Asia/Beirut
    {  354, 5989,   92670,  6, }, // Asia/Brunei
    {  354, 4511,   92676,  7, }, // Asia/Colombo
    {  354, 4540,    4201,  8, }, // Asia/Damascus
    {  354, 1093,   91557,  5, }, // Asia/Dhaka
    {  354, 1259,   92683, 16, }, // Asia/Ho_Chi_Minh
    {  354, 7006,    4247,  8, }, // Asia/Jakarta
    {  354, 1290,   92699,  9, }, // Asia/Jerusalem
    {  354, 3560,   92708,  5, }, // Asia/Kabul
    {  354, 4316,    4261, 10, }, // Asia/Kamchatka
    {  354, 1185,   20830,  8, }, // Asia/Kathmandu
    {  354,  992,   92713,  8, }, // Asia/Kolkata
    {  354, 4859,   92721, 12, }, // Asia/Kuala_Lumpur
    {  354, 5547,   92733,  6, }, // Asia/Kuwait
    {  354, 1211,   91643,  5, }, // Asia/Macau
    {  354, 2554,   92739,  7, }, // Asia/Nicosia
    {  354, 4214,   33562,  9, }, // Asia/Pyongyang
    {  354, 5559,    4332,  5, }, // Asia/Qatar
    {  354, 8708,     469,  8, }, // Asia/Qostanay
    {  354, 3584,   92746,  5, }, // Asia/Riyadh
    {  354, 3279,   20896,  4, }, // Asia/Seoul
    {  354, 1053,   92751,  7, }, // Asia/Shanghai
    {  354, 3300,   92758,  8, }, // Asia/Singapore
    {  354, 2772,    7842,  7, }, // Asia/Tehran
    {  354, 2821,   92766,  5, }, // Asia/Tokyo
    {  354, 1021,   92771, 10, }, // Asia/Ulaanbaatar
    {  354, 1235,   20937,  6, }, // Asia/Yangon
    {  354, 3747,    7898,  7, }, // Asia/Yerevan
    {  354, 3663,   92781,  9, }, // Atlantic/Azores
    {  354, 5671,   92790,  7, }, // Atlantic/Bermuda
    {  354, 6628,   92797, 11, }, // Atlantic/Canary
    {  354, 3727,   92808, 14, }, // Atlantic/Cape_Verde
    {  354, 1395,   92822,  8, }, // Atlantic/Faroe
    {  354, 2748,   36176,  9, }, // Atlantic/Reykjavik
    {  354, 7272,   92830, 13, }, // Atlantic/South_Georgia
    {  354, 6821,   92843, 13, }, // Atlantic/St_Helena
    {  354, 2300,   92856,  5, }, // Europe/Athens
    {  354, 6336,   27015,  7, }, // Europe/Belgrade
    {  354, 4757,   21059,  6, }, // Europe/Berlin
    {  354, 2167,    4450,  7, }, // Europe/Brussels
    {  354, 3928,   92861,  8, }, // Europe/Bucharest
    {  354, 3806,   92869,  8, }, // Europe/Budapest
    {  354, 6289,   92877, 13, }, // Europe/Copenhagen
    {  354, 6307,   92890,  9, }, // Europe/Gibraltar
    {  354, 8804,   92899,  3, }, // Europe/Isle_of_Man
    {  354, 1130,   92902,  8, }, // Europe/Istanbul
    {  354, 2527,   92910, 10, }, // Europe/Kyiv
    {  354, 3213,    4499,  8, }, // Europe/Lisbon
    {  354, 2501,   92920,  8, }, // Europe/London
    {  354, 6365,   92928,  9, }, // Europe/Luxembourg
    {  354, 4892,   92937,  6, }, // Europe/Madrid
    {  354, 6582,   92943,  9, }, // Europe/Mariehamn
    {  354, 6410,   92952,  6, }, // Europe/Monaco
    {  354, 3537,   22880,  6, }, // Europe/Moscow
    {  354, 6458,   92958,  4, }, // Europe/Oslo
    {  354, 4284,   21180,  5, }, // Europe/Paris
    {  354, 6275,    8201,  4, }, // Europe/Prague
    {  354, 6324,   92060,  3, }, // Europe/Rome
    {  354, 6470,   92962, 10, }, // Europe/San_Marino
    {  354, 6245,   92972,  8, }, // Europe/Sarajevo
    {  354, 6599,   92980,  5, }, // Europe/Sofia
    {  354, 6523,   92985, 11, }, // Europe/Stockholm
    {  354, 6202,   92996,  6, }, // Europe/Tirane
    {  354, 6567,   93002,  9, }, // Europe/Vatican
    {  354, 6231,   93011,  3, }, // Europe/Vienna
    {  354, 8899,   93014,  7, }, // Europe/Vilnius
    {  354, 3190,   93021,  6, }, // Europe/Warsaw
    {  354, 6046,   93027,  6, }, // Indian/Christmas
    {  354, 6063,   93033, 10, }, // Indian/Cocos
    {  354, 7050,   93043, 11, }, // Indian/Maldives
    {  354, 4069,   93054,  8, }, // Indian/Mauritius
    {  354, 7211,     547,  7, }, // Indian/Reunion
    {  354, 3157,     554,  5, }, // Pacific/Chuuk
    {  354, 2237,   93062,  9, }, // Pacific/Easter
    {  354, 3902,   93071,  5, }, // Pacific/Fiji
    {  354, 6018,   63353,  4, }, // Pacific/Guam
    {  354, 2723,    4611,  8, }, // Pacific/Honolulu
    {  354, 3049,    4619,  9, }, // Pacific/Kanton
    {  354, 4051,   93076, 15, }, // Pacific/Marquesas
    {  354, 7084,   93091,  4, }, // Pacific/Nauru
    {  354, 7155,   93095,  4, }, // Pacific/Palau
    {  354, 3096,     586,  7, }, // Pacific/Pohnpei
    {  354, 7312,   93099,  6, }, // Pacific/Tahiti
    {  355,   14,       0,  6, }, // Africa/Asmara Ido/Latin/world
    {  355, 4435,       6,  8, }, // Africa/Sao_Tome
    {  355, 4252,      14,  8, }, // America/Asuncion
    {  355,  237,      22,  8, }, // America/Atikokan
    {  355, 7623,      30, 17, }, // America/Bahia_Banderas
    {  355, 3868,      47,  6, }, // America/Cancun
    {  355, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  355, 5723,      66,  7, }, // America/Curacao
    {  355,  481,      73, 13, }, // America/Indiana/Knox
    {  355, 7961,      86, 16, }, // America/Indiana/Marengo
    {  355, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  355, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  355, 8038,     139, 14, }, // America/Indiana/Vevay
    {  355, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  355, 8086,     171, 16, }, // America/Indiana/Winamac
    {  355, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  355, 5932,     207, 22, }, // America/Lower_Princes
    {  355, 8220,     229,  6, }, // America/Merida
    {  355, 1850,     235, 19, }, // America/Noronha
    {  355, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  355, 8329,     274, 20, }, // America/North_Dakota/Center
    {  355, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  355,  393,     317,  4, }, // America/Nuuk
    {  355, 6899,     321, 16, }, // America/Scoresbysund
    {  355, 8508,     337, 14, }, // America/St_Barthelemy
    {  355, 2061,     351, 10, }, // America/St_Johns
    {  355, 5863,     361,  9, }, // America/St_Kitts
    {  355, 5880,     370,  9, }, // America/St_Lucia
    {  355,  855,     379, 10, }, // America/St_Thomas
    {  355, 5913,     389, 11, }, // America/St_Vincent
    {  355, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  355, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  355, 1185,     448,  9, }, // Asia/Kathmandu
    {  355,  992,     457,  7, }, // Asia/Kolkata
    {  355, 1211,     464,  5, }, // Asia/Macau
    {  355, 1235,     477,  6, }, // Asia/Yangon
    {  355, 1395,     483,  5, }, // Atlantic/Faroe
    {  355, 6821,     488, 10, }, // Atlantic/St_Helena
    {  355, 2527,     514,  4, }, // Europe/Kyiv
    {  355, 7211,     547,  7, }, // Indian/Reunion
    {  355, 3157,     554,  5, }, // Pacific/Chuuk
    {  355, 3096,     586,  7, }, // Pacific/Pohnpei
    {  356,   14,       0,  6, }, // Africa/Asmara Igbo/Latin/Nigeria
    {  356, 4435,       6,  8, }, // Africa/Sao_Tome
    {  356, 4252,      14,  8, }, // America/Asuncion
    {  356,  237,      22,  8, }, // America/Atikokan
    {  356, 7623,      30, 17, }, // America/Bahia_Banderas
    {  356, 3868,      47,  6, }, // America/Cancun
    {  356, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  356, 5723,      66,  7, }, // America/Curacao
    {  356,  481,      73, 13, }, // America/Indiana/Knox
    {  356, 7961,      86, 16, }, // America/Indiana/Marengo
    {  356, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  356, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  356, 8038,     139, 14, }, // America/Indiana/Vevay
    {  356, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  356, 8086,     171, 16, }, // America/Indiana/Winamac
    {  356, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  356, 5932,     207, 22, }, // America/Lower_Princes
    {  356, 8220,     229,  6, }, // America/Merida
    {  356, 1850,     235, 19, }, // America/Noronha
    {  356, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  356, 8329,     274, 20, }, // America/North_Dakota/Center
    {  356, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  356,  393,     317,  4, }, // America/Nuuk
    {  356, 6899,     321, 16, }, // America/Scoresbysund
    {  356, 8508,     337, 14, }, // America/St_Barthelemy
    {  356, 2061,     351, 10, }, // America/St_Johns
    {  356, 5863,     361,  9, }, // America/St_Kitts
    {  356, 5880,     370,  9, }, // America/St_Lucia
    {  356,  855,     379, 10, }, // America/St_Thomas
    {  356, 5913,     389, 11, }, // America/St_Vincent
    {  356, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  356, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  356, 1185,     448,  9, }, // Asia/Kathmandu
    {  356,  992,     457,  7, }, // Asia/Kolkata
    {  356, 1211,     464,  5, }, // Asia/Macau
    {  356, 1235,     477,  6, }, // Asia/Yangon
    {  356, 1395,     483,  5, }, // Atlantic/Faroe
    {  356, 6821,     488, 10, }, // Atlantic/St_Helena
    {  356, 2527,     514,  4, }, // Europe/Kyiv
    {  356, 7211,     547,  7, }, // Indian/Reunion
    {  356, 3157,     554,  5, }, // Pacific/Chuuk
    {  356, 3049,    4619,  9, }, // Pacific/Kanton
    {  356, 3096,     586,  7, }, // Pacific/Pohnpei
    {  357,   14,       0,  6, }, // Africa/Asmara Inari Sami/Latin/Finland
    {  357, 4435,       6,  8, }, // Africa/Sao_Tome
    {  357, 4252,      14,  8, }, // America/Asuncion
    {  357,  237,      22,  8, }, // America/Atikokan
    {  357, 7623,      30, 17, }, // America/Bahia_Banderas
    {  357, 3868,      47,  6, }, // America/Cancun
    {  357, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  357, 5723,      66,  7, }, // America/Curacao
    {  357,  481,      73, 13, }, // America/Indiana/Knox
    {  357, 7961,      86, 16, }, // America/Indiana/Marengo
    {  357, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  357, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  357, 8038,     139, 14, }, // America/Indiana/Vevay
    {  357, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  357, 8086,     171, 16, }, // America/Indiana/Winamac
    {  357, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  357, 5932,     207, 22, }, // America/Lower_Princes
    {  357, 8220,     229,  6, }, // America/Merida
    {  357, 1850,     235, 19, }, // America/Noronha
    {  357, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  357, 8329,     274, 20, }, // America/North_Dakota/Center
    {  357, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  357,  393,     317,  4, }, // America/Nuuk
    {  357, 6899,     321, 16, }, // America/Scoresbysund
    {  357, 8508,     337, 14, }, // America/St_Barthelemy
    {  357, 2061,     351, 10, }, // America/St_Johns
    {  357, 5863,     361,  9, }, // America/St_Kitts
    {  357, 5880,     370,  9, }, // America/St_Lucia
    {  357,  855,     379, 10, }, // America/St_Thomas
    {  357, 5913,     389, 11, }, // America/St_Vincent
    {  357, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  357, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  357, 1185,     448,  9, }, // Asia/Kathmandu
    {  357,  992,     457,  7, }, // Asia/Kolkata
    {  357, 1211,     464,  5, }, // Asia/Macau
    {  357, 1235,     477,  6, }, // Asia/Yangon
    {  357, 1395,     483,  5, }, // Atlantic/Faroe
    {  357, 6821,     488, 10, }, // Atlantic/St_Helena
    {  357, 2527,     514,  4, }, // Europe/Kyiv
    {  357, 7211,     547,  7, }, // Indian/Reunion
    {  357, 3157,     554,  5, }, // Pacific/Chuuk
    {  357, 3096,     586,  7, }, // Pacific/Pohnpei
    {  358, 7420,   93105,  5, }, // Africa/Algiers Indonesian/Latin/Indonesia
    {  358,   14,       0,  6, }, // Africa/Asmara
    {  358, 2320,    4690,  5, }, // Africa/Cairo
    {  358, 4435,       6,  8, }, // Africa/Sao_Tome
    {  358, 5607,    4775,  7, }, // America/Anguilla
    {  358, 4252,      14,  8, }, // America/Asuncion
    {  358,  237,      22,  8, }, // America/Atikokan
    {  358, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  358, 3868,   33314,  6, }, // America/Cancun
    {  358, 7774,   91278, 13, }, // America/Ciudad_Juarez
    {  358, 5422,   21621, 10, }, // America/Costa_Rica
    {  358, 5723,      66,  7, }, // America/Curacao
    {  358, 5739,   21666,  8, }, // America/Dominica
    {  358,  481,      73, 13, }, // America/Indiana/Knox
    {  358, 7961,      86, 16, }, // America/Indiana/Marengo
    {  358, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  358, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  358, 8038,     139, 14, }, // America/Indiana/Vevay
    {  358, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  358, 8086,     171, 16, }, // America/Indiana/Winamac
    {  358, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  358, 5932,     207, 22, }, // America/Lower_Princes
    {  358, 5805,    5260,  8, }, // America/Martinique
    {  358, 8220,    7126,  6, }, // America/Merida
    {  358, 1850,     235, 19, }, // America/Noronha
    {  358, 8301,   93110, 20, }, // America/North_Dakota/Beulah
    {  358, 8329,   93130, 20, }, // America/North_Dakota/Center
    {  358, 8357,   93150, 23, }, // America/North_Dakota/New_Salem
    {  358,  393,     317,  4, }, // America/Nuuk
    {  358, 6899,     321, 16, }, // America/Scoresbysund
    {  358, 8508,     337, 14, }, // America/St_Barthelemy
    {  358, 2061,     351, 10, }, // America/St_Johns
    {  358, 5863,     361,  9, }, // America/St_Kitts
    {  358, 5880,     370,  9, }, // America/St_Lucia
    {  358,  855,     379, 10, }, // America/St_Thomas
    {  358, 5913,     389, 11, }, // America/St_Vincent
    {  358, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  358, 5523,    5609,  5, }, // Asia/Aqtau
    {  358, 5368,    7606,  6, }, // Asia/Aqtobe
    {  358, 4511,    5637,  7, }, // Asia/Colombo
    {  358, 4540,    4201,  8, }, // Asia/Damascus
    {  358, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  358, 1290,    5686,  9, }, // Asia/Jerusalem
    {  358, 1185,     448,  9, }, // Asia/Kathmandu
    {  358,  992,     457,  7, }, // Asia/Kolkata
    {  358, 1211,   33537,  5, }, // Asia/Macau
    {  358, 6920,    4326,  6, }, // Asia/Muscat
    {  358, 2554,    5738,  7, }, // Asia/Nicosia
    {  358, 8708,     469,  8, }, // Asia/Qostanay
    {  358, 3300,   93173,  9, }, // Asia/Singapore
    {  358, 2772,    7842,  7, }, // Asia/Tehran
    {  358, 1235,   58618,  7, }, // Asia/Yangon
    {  358, 3727,   93182, 13, }, // Atlantic/Cape_Verde
    {  358, 1395,     483,  5, }, // Atlantic/Faroe
    {  358, 7272,   93195, 15, }, // Atlantic/South_Georgia
    {  358, 6821,     488, 10, }, // Atlantic/St_Helena
    {  358, 2300,   93210,  6, }, // Europe/Athens
    {  358, 6336,    8060,  7, }, // Europe/Belgrade
    {  358, 2583,   93216,  8, }, // Europe/Chisinau
    {  358, 6289,    4475, 10, }, // Europe/Copenhagen
    {  358, 8804,   93224,  9, }, // Europe/Isle_of_Man
    {  358, 2527,    8148,  4, }, // Europe/Kyiv
    {  358, 6365,    8172, 10, }, // Europe/Luxembourg
    {  358, 6410,    6044,  6, }, // Europe/Monaco
    {  358, 3537,   93233,  6, }, // Europe/Moscow
    {  358, 6275,   36285,  5, }, // Europe/Prague
    {  358, 6324,    6064,  4, }, // Europe/Rome
    {  358, 6567,    6096,  7, }, // Europe/Vatican
    {  358, 6231,     171,  4, }, // Europe/Vienna
    {  358, 3190,   93239,  7, }, // Europe/Warsaw
    {  358, 5029,   22985,  6, }, // Indian/Comoro
    {  358, 7050,   93246,  8, }, // Indian/Maldives
    {  358, 7211,     547,  7, }, // Indian/Reunion
    {  358, 3157,     554,  5, }, // Pacific/Chuuk
    {  358, 3822,    8360, 11, }, // Pacific/Guadalcanal
    {  358, 2723,    4611,  8, }, // Pacific/Honolulu
    {  358, 3049,    4619,  9, }, // Pacific/Kanton
    {  358, 3096,     586,  7, }, // Pacific/Pohnpei
    {  359,   14,       0,  6, }, // Africa/Asmara Interlingua/Latin/world
    {  359, 5043,   26492,  7, }, // Africa/Djibouti
    {  359, 4435,       6,  8, }, // Africa/Sao_Tome
    {  359, 4252,      14,  8, }, // America/Asuncion
    {  359,  237,      22,  8, }, // America/Atikokan
    {  359, 7623,   64671, 17, }, // America/Bahia_Banderas
    {  359, 3868,   33314,  6, }, // America/Cancun
    {  359, 5496,   44173,  6, }, // America/Cayman
    {  359, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  359, 5723,      66,  7, }, // America/Curacao
    {  359,  481,      73, 13, }, // America/Indiana/Knox
    {  359, 7961,      86, 16, }, // America/Indiana/Marengo
    {  359, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  359, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  359, 8038,     139, 14, }, // America/Indiana/Vevay
    {  359, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  359, 8086,     171, 16, }, // America/Indiana/Winamac
    {  359, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  359, 5932,     207, 22, }, // America/Lower_Princes
    {  359, 5805,   20554,  9, }, // America/Martinique
    {  359, 8220,    7126,  6, }, // America/Merida
    {  359, 2949,   93254, 16, }, // America/Mexico_City
    {  359, 2379,   93270,  9, }, // America/New_York
    {  359, 1850,     235, 19, }, // America/Noronha
    {  359, 8301,   44203, 23, }, // America/North_Dakota/Beulah
    {  359, 8329,   44226, 23, }, // America/North_Dakota/Center
    {  359, 8357,   44249, 26, }, // America/North_Dakota/New_Salem
    {  359,  393,     317,  4, }, // America/Nuuk
    {  359, 5843,   64979, 10, }, // America/Puerto_Rico
    {  359, 6899,     321, 16, }, // America/Scoresbysund
    {  359, 8508,   93279, 18, }, // America/St_Barthelemy
    {  359, 2061,   93297, 28, }, // America/St_Johns
    {  359, 5863,   93325, 19, }, // America/St_Kitts
    {  359, 5880,   93344, 12, }, // America/St_Lucia
    {  359,  855,   93356, 13, }, // America/St_Thomas
    {  359, 5913,   93369, 15, }, // America/St_Vincent
    {  359, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  359, 5534,    4168,  7, }, // Asia/Bahrain
    {  359, 4540,   74438,  7, }, // Asia/Damascus
    {  359, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  359, 2704,    4231,  8, }, // Asia/Hong_Kong
    {  359, 1185,     448,  9, }, // Asia/Kathmandu
    {  359, 8661,   58020,  8, }, // Asia/Khandyga
    {  359,  992,   36053,  8, }, // Asia/Kolkata
    {  359, 1211,     464,  5, }, // Asia/Macau
    {  359, 4408,   58084,  8, }, // Asia/Sakhalin
    {  359, 1021,   26923, 10, }, // Asia/Ulaanbaatar
    {  359, 1235,     477,  6, }, // Asia/Yangon
    {  359, 3883,    7886, 12, }, // Asia/Yekaterinburg
    {  359, 6628,   74497,  8, }, // Atlantic/Canary
    {  359, 3727,   93384, 10, }, // Atlantic/Cape_Verde
    {  359, 1395,   20992,  5, }, // Atlantic/Faroe
    {  359, 6821,     488, 10, }, // Atlantic/St_Helena
    {  359, 2300,   93394,  7, }, // Europe/Athens
    {  359, 6336,   58906,  8, }, // Europe/Belgrade
    {  359, 2167,   57661,  9, }, // Europe/Brussels
    {  359, 3928,   21073,  8, }, // Europe/Bucharest
    {  359, 8772,   27037,  8, }, // Europe/Busingen
    {  359, 8804,   93401, 13, }, // Europe/Isle_of_Man
    {  359, 2527,    8148,  4, }, // Europe/Kyiv
    {  359, 3213,   93414,  7, }, // Europe/Lisbon
    {  359, 6365,    4513,  9, }, // Europe/Luxembourg
    {  359, 3537,   74590,  7, }, // Europe/Moscow
    {  359, 6275,   21185,  5, }, // Europe/Prague
    {  359, 6324,    6064,  4, }, // Europe/Rome
    {  359, 6202,   21213,  6, }, // Europe/Tirane
    {  359, 6567,   74624,  8, }, // Europe/Vatican
    {  359, 3190,   21244,  8, }, // Europe/Warsaw
    {  359, 6046,   93421,  5, }, // Indian/Christmas
    {  359, 7050,   74646,  8, }, // Indian/Maldives
    {  359, 4069,   93426,  8, }, // Indian/Mauritius
    {  359, 5098,   93434,  7, }, // Indian/Mayotte
    {  359, 7211,     547,  7, }, // Indian/Reunion
    {  359, 3157,     554,  5, }, // Pacific/Chuuk
    {  359, 2237,   93441,  6, }, // Pacific/Easter
    {  359, 6678,   21277,  9, }, // Pacific/Galapagos
    {  359, 3096,     586,  7, }, // Pacific/Pohnpei
    {  360, 5059,   64622, 11, }, // Africa/Addis_Ababa Interlingue/Latin/Estonia
    {  360, 7420,   44088,  5, }, // Africa/Algiers
    {  360,   14,       0,  6, }, // Africa/Asmara
    {  360, 5130,   93447, 13, }, // Africa/Dar_es_Salaam
    {  360, 5043,   26492,  7, }, // Africa/Djibouti
    {  360, 5166,   93460,  8, }, // Africa/El_Aaiun
    {  360, 4524,   26504,  7, }, // Africa/Khartoum
    {  360, 6727,   26534,  8, }, // Africa/Ouagadougou
    {  360, 4435,   93468,  8, }, // Africa/Sao_Tome
    {  360,  166,   93476, 12, }, // America/Argentina/Buenos_Aires
    {  360, 7462,   93488,  8, }, // America/Argentina/La_Rioja
    {  360, 7489,   93496, 12, }, // America/Argentina/Rio_Gallegos
    {  360, 7544,   93508,  9, }, // America/Argentina/San_Juan
    {  360, 5580,   93517,  9, }, // America/Argentina/San_Luis
    {  360, 4252,      14,  8, }, // America/Asuncion
    {  360,  237,      22,  8, }, // America/Atikokan
    {  360, 7623,   93526, 15, }, // America/Bahia_Banderas
    {  360, 7681,    6632,  9, }, // America/Boa_Vista
    {  360, 7713,   93541, 13, }, // America/Cambridge_Bay
    {  360, 7735,   93554, 12, }, // America/Campo_Grande
    {  360, 3868,      47,  6, }, // America/Cancun
    {  360, 7774,   93566, 13, }, // America/Ciudad_Juarez
    {  360, 5422,   93579, 10, }, // America/Costa_Rica
    {  360, 5723,      66,  7, }, // America/Curacao
    {  360, 7848,   93589, 12, }, // America/Dawson_Creek
    {  360, 5441,   93601, 11, }, // America/El_Salvador
    {  360, 7886,    6814, 11, }, // America/Fort_Nelson
    {  360, 7924,   93612,  9, }, // America/Glace_Bay
    {  360, 6881,   93621,  9, }, // America/Goose_Bay
    {  360, 4612,   93630, 10, }, // America/Grand_Turk
    {  360, 5786,   93640,  9, }, // America/Guadeloupe
    {  360,  481,      73, 13, }, // America/Indiana/Knox
    {  360, 7961,      86, 16, }, // America/Indiana/Marengo
    {  360, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  360, 8012,   93649, 18, }, // America/Indiana/Tell_City
    {  360, 8038,     139, 14, }, // America/Indiana/Vevay
    {  360, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  360, 8086,     171, 16, }, // America/Indiana/Winamac
    {  360, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  360, 4376,   93667,  6, }, // America/La_Paz
    {  360, 3239,   93673, 11, }, // America/Los_Angeles
    {  360, 5932,   93684, 22, }, // America/Lower_Princes
    {  360, 5805,   20554,  9, }, // America/Martinique
    {  360, 8220,     229,  6, }, // America/Merida
    {  360, 2949,   93706, 12, }, // America/Mexico_City
    {  360, 1850,     235, 19, }, // America/Noronha
    {  360, 8301,   93718, 19, }, // America/North_Dakota/Beulah
    {  360, 8329,   93737, 19, }, // America/North_Dakota/Center
    {  360, 8357,   93756, 22, }, // America/North_Dakota/New_Salem
    {  360,  393,     317,  4, }, // America/Nuuk
    {  360, 5954,   26691, 13, }, // America/Port_of_Spain
    {  360, 8404,    7301, 11, }, // America/Porto_Velho
    {  360, 5843,   93778, 10, }, // America/Puerto_Rico
    {  360, 4030,    7322, 12, }, // America/Punta_Arenas
    {  360, 8424,   93788, 12, }, // America/Rankin_Inlet
    {  360,  695,   93800, 10, }, // America/Rio_Branco
    {  360, 6111,   93810, 11, }, // America/Santo_Domingo
    {  360, 1878,   93821,  9, }, // America/Sao_Paulo
    {  360, 6899,     321, 16, }, // America/Scoresbysund
    {  360, 8508,   93830, 15, }, // America/St_Barthelemy
    {  360, 2061,   93845, 10, }, // America/St_Johns
    {  360, 5863,   93855,  9, }, // America/St_Kitts
    {  360, 5880,   93864,  9, }, // America/St_Lucia
    {  360,  855,   93873, 10, }, // America/St_Thomas
    {  360, 5913,   93883, 11, }, // America/St_Vincent
    {  360, 8530,   93894, 13, }, // America/Swift_Current
    {  360, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  360, 7295,   62201,  5, }, // Antarctica/Syowa
    {  360, 5395,    7594,  6, }, // Asia/Almaty
    {  360, 5523,    5609,  5, }, // Asia/Aqtau
    {  360, 5368,    7606,  6, }, // Asia/Aqtobe
    {  360, 8624,    7612,  6, }, // Asia/Atyrau
    {  360, 4540,   74438,  7, }, // Asia/Damascus
    {  360, 1259,   78157, 11, }, // Asia/Ho_Chi_Minh
    {  360, 2704,    7653,  9, }, // Asia/Hong_Kong
    {  360, 1185,     448,  9, }, // Asia/Kathmandu
    {  360,  992,     457,  7, }, // Asia/Kolkata
    {  360, 4859,    7707, 12, }, // Asia/Kuala_Lumpur
    {  360, 1211,     464,  5, }, // Asia/Macau
    {  360, 6961,   93907, 10, }, // Asia/Phnom_Penh
    {  360, 8708,     469,  8, }, // Asia/Qostanay
    {  360, 4269,    7778,  9, }, // Asia/Qyzylorda
    {  360, 3300,   20900,  8, }, // Asia/Singapore
    {  360, 2772,    7842,  7, }, // Asia/Tehran
    {  360, 2821,    4356,  5, }, // Asia/Tokyo
    {  360, 1235,     477,  6, }, // Asia/Yangon
    {  360, 6628,   74497,  7, }, // Atlantic/Canary
    {  360, 3727,   93917,  9, }, // Atlantic/Cape_Verde
    {  360, 1395,   20992,  5, }, // Atlantic/Faroe
    {  360, 7272,   93926, 11, }, // Atlantic/South_Georgia
    {  360, 6821,   93937, 11, }, // Atlantic/St_Helena
    {  360, 1799,   93948, 11, }, // Australia/Broken_Hill
    {  360, 1547,   93959, 15, }, // Australia/Lord_Howe
    {  360, 1762,   22735,  4, }, // Australia/Perth
    {  360, 2300,    5911,  4, }, // Europe/Athens
    {  360, 6336,   27015,  7, }, // Europe/Belgrade
    {  360, 2167,   57661,  9, }, // Europe/Brussels
    {  360, 3928,   93974,  9, }, // Europe/Bucharest
    {  360, 6289,   58657,  9, }, // Europe/Copenhagen
    {  360, 8804,   93983, 12, }, // Europe/Isle_of_Man
    {  360, 2527,    8148,  4, }, // Europe/Kyiv
    {  360, 3213,   21137,  6, }, // Europe/Lisbon
    {  360, 6365,    4513,  9, }, // Europe/Luxembourg
    {  360, 6410,   74584,  6, }, // Europe/Monaco
    {  360, 3537,   93995,  6, }, // Europe/Moscow
    {  360, 6275,   36285,  5, }, // Europe/Prague
    {  360, 6324,    6064,  4, }, // Europe/Rome
    {  360, 6470,    8214, 10, }, // Europe/San_Marino
    {  360, 6202,   21213,  6, }, // Europe/Tirane
    {  360, 3190,   58675,  8, }, // Europe/Warsaw
    {  360, 6540,    4577,  6, }, // Europe/Zurich
    {  360, 4069,   74654,  8, }, // Indian/Mauritius
    {  360, 7211,     547,  7, }, // Indian/Reunion
    {  360, 3157,     554,  5, }, // Pacific/Chuuk
    {  360, 2237,   94001,  5, }, // Pacific/Easter
    {  360, 3902,    4606,  5, }, // Pacific/Fiji
    {  360, 3126,    8413,  9, }, // Pacific/Pago_Pago
    {  360, 3096,     586,  7, }, // Pacific/Pohnpei
    {  360, 4807,   94006, 12, }, // Pacific/Port_Moresby
    {  361,   14,       0,  6, }, // Africa/Asmara Inuktitut/Canadian Aboriginal/Canada
    {  361, 4435,       6,  8, }, // Africa/Sao_Tome
    {  361, 4252,      14,  8, }, // America/Asuncion
    {  361,  237,      22,  8, }, // America/Atikokan
    {  361, 7623,      30, 17, }, // America/Bahia_Banderas
    {  361, 3868,      47,  6, }, // America/Cancun
    {  361, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  361, 5723,      66,  7, }, // America/Curacao
    {  361,  481,      73, 13, }, // America/Indiana/Knox
    {  361, 7961,      86, 16, }, // America/Indiana/Marengo
    {  361, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  361, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  361, 8038,     139, 14, }, // America/Indiana/Vevay
    {  361, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  361, 8086,     171, 16, }, // America/Indiana/Winamac
    {  361, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  361, 5932,     207, 22, }, // America/Lower_Princes
    {  361, 8220,     229,  6, }, // America/Merida
    {  361, 1850,     235, 19, }, // America/Noronha
    {  361, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  361, 8329,     274, 20, }, // America/North_Dakota/Center
    {  361, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  361,  393,     317,  4, }, // America/Nuuk
    {  361, 6899,     321, 16, }, // America/Scoresbysund
    {  361, 8508,     337, 14, }, // America/St_Barthelemy
    {  361, 2061,     351, 10, }, // America/St_Johns
    {  361, 5863,     361,  9, }, // America/St_Kitts
    {  361, 5880,     370,  9, }, // America/St_Lucia
    {  361,  855,     379, 10, }, // America/St_Thomas
    {  361, 5913,     389, 11, }, // America/St_Vincent
    {  361, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  361, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  361, 1185,     448,  9, }, // Asia/Kathmandu
    {  361,  992,     457,  7, }, // Asia/Kolkata
    {  361, 1211,     464,  5, }, // Asia/Macau
    {  361, 1235,     477,  6, }, // Asia/Yangon
    {  361, 1395,     483,  5, }, // Atlantic/Faroe
    {  361, 6821,     488, 10, }, // Atlantic/St_Helena
    {  361, 2527,     514,  4, }, // Europe/Kyiv
    {  361, 7211,     547,  7, }, // Indian/Reunion
    {  361, 3157,     554,  5, }, // Pacific/Chuuk
    {  361, 3096,     586,  7, }, // Pacific/Pohnpei
    {  363, 7420,   94018, 20, }, // Africa/Algiers Irish/Latin/Ireland
    {  363,   14,       0,  6, }, // Africa/Asmara
    {  363, 2320,   94038,  6, }, // Africa/Cairo
    {  363, 6773,   94044,  8, }, // Africa/Conakry
    {  363, 6840,   94052,  5, }, // Africa/Dakar
    {  363, 5130,   94057,  9, }, // Africa/Dar_es_Salaam
    {  363, 5166,   94066,  6, }, // Africa/El_Aaiun
    {  363, 4524,   94072,  6, }, // Africa/Khartoum
    {  363, 6869,   20418,  4, }, // Africa/Lome
    {  363, 4987,   94078,  6, }, // Africa/Maputo
    {  363, 5113,   94084, 10, }, // Africa/Mogadishu
    {  363, 6803,   94094,  7, }, // Africa/Nouakchott
    {  363, 4435,       6,  8, }, // Africa/Sao_Tome
    {  363, 2866,   94101,  7, }, // Africa/Tripoli
    {  363, 6554,   94108,  6, }, // Africa/Tunis
    {  363, 5607,   94114,  7, }, // America/Anguilla
    {  363,  270,   20479,  7, }, // America/Argentina/Cordoba
    {  363, 5640,   92485,  5, }, // America/Aruba
    {  363, 4252,      14,  8, }, // America/Asuncion
    {  363,  237,      22,  8, }, // America/Atikokan
    {  363, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  363, 5654,   94121,  8, }, // America/Barbados
    {  363, 7646,   35896,  5, }, // America/Belem
    {  363, 5407,   94129, 10, }, // America/Belize
    {  363, 4361,   94139,  6, }, // America/Bogota
    {  363, 3868,      47,  6, }, // America/Cancun
    {  363, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  363, 5422,   94145, 10, }, // America/Costa_Rica
    {  363, 5723,   94155,  6, }, // America/Curacao
    {  363, 5739,   72822,  9, }, // America/Dominica
    {  363, 5441,   94161, 13, }, // America/El_Salvador
    {  363, 5770,   94174,  8, }, // America/Grenada
    {  363, 5786,   94182,  9, }, // America/Guadeloupe
    {  363, 3760,   72873,  9, }, // America/Guatemala
    {  363, 6932,   94191,  9, }, // America/Guyana
    {  363,  481,      73, 13, }, // America/Indiana/Knox
    {  363, 7961,      86, 16, }, // America/Indiana/Marengo
    {  363, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  363, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  363, 8038,     139, 14, }, // America/Indiana/Vevay
    {  363, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  363, 8086,     171, 16, }, // America/Indiana/Winamac
    {  363, 2799,   94200,  7, }, // America/Jamaica
    {  363, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  363, 7169,   94207,  5, }, // America/Lima
    {  363, 5932,     207, 22, }, // America/Lower_Princes
    {  363, 8220,    7126,  6, }, // America/Merida
    {  363, 2949,   94212, 18, }, // America/Mexico_City
    {  363, 5824,   72932,  9, }, // America/Montserrat
    {  363, 2379,   94230, 11, }, // America/New_York
    {  363, 1850,     235, 19, }, // America/Noronha
    {  363, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  363, 8329,     274, 20, }, // America/North_Dakota/Center
    {  363, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  363,  393,     317,  4, }, // America/Nuuk
    {  363, 5843,   94241, 10, }, // America/Puerto_Rico
    {  363, 1878,   26712,  9, }, // America/Sao_Paulo
    {  363, 6899,     321, 16, }, // America/Scoresbysund
    {  363, 8508,   35936, 16, }, // America/St_Barthelemy
    {  363, 2061,     351, 10, }, // America/St_Johns
    {  363, 5863,   94251, 13, }, // America/St_Kitts
    {  363, 5880,   35964, 11, }, // America/St_Lucia
    {  363,  855,     379, 10, }, // America/St_Thomas
    {  363, 5913,   94264, 12, }, // America/St_Vincent
    {  363, 5756,   94276, 10, }, // America/Thule
    {  363, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  363, 8568,   94286, 10, }, // Antarctica/Macquarie
    {  363, 5570,   94296,  5, }, // Asia/Aden
    {  363, 3607,    4162,  6, }, // Asia/Baghdad
    {  363, 5534,   94301,  7, }, // Asia/Bahrain
    {  363, 3653,   94308,  4, }, // Asia/Baku
    {  363, 4086,   94312,  7, }, // Asia/Beirut
    {  363, 5989,   94319,  6, }, // Asia/Brunei
    {  363, 4540,   94325, 10, }, // Asia/Damascus
    {  363, 4795,   94335,  7, }, // Asia/Hebron
    {  363, 1259,   73091, 19, }, // Asia/Ho_Chi_Minh
    {  363, 2704,   94342,  9, }, // Asia/Hong_Kong
    {  363, 7006,   94351,  7, }, // Asia/Jakarta
    {  363, 1290,   94358, 11, }, // Asia/Jerusalem
    {  363, 3560,   94369,  5, }, // Asia/Kabul
    {  363, 1185,     448,  9, }, // Asia/Kathmandu
    {  363,  992,   94374,  7, }, // Asia/Kolkata
    {  363, 5547,   94381,  5, }, // Asia/Kuwait
    {  363, 1211,     464,  5, }, // Asia/Macau
    {  363, 1349,   94386,  7, }, // Asia/Makassar
    {  363, 7182,   94393,  7, }, // Asia/Manila
    {  363, 2554,   94400, 10, }, // Asia/Nicosia
    {  363, 5559,   73150,  5, }, // Asia/Qatar
    {  363, 8708,     469,  8, }, // Asia/Qostanay
    {  363, 3279,   94410,  3, }, // Asia/Seoul
    {  363, 1053,   94413,  9, }, // Asia/Shanghai
    {  363, 3300,   94422,  9, }, // Asia/Singapore
    {  363, 2821,   94431,  6, }, // Asia/Tokyo
    {  363, 4828,   94437,  7, }, // Asia/Yakutsk
    {  363, 1235,   20937,  6, }, // Asia/Yangon
    {  363, 3747,   94444,  8, }, // Asia/Yerevan
    {  363, 3663,   94452,  9, }, // Atlantic/Azores
    {  363, 5671,   94461,  9, }, // Atlantic/Bermuda
    {  363, 6628,   94470, 22, }, // Atlantic/Canary
    {  363, 3727,   94492, 10, }, // Atlantic/Cape_Verde
    {  363, 1395,   94502, 13, }, // Atlantic/Faroe
    {  363, 8736,   94515,  8, }, // Atlantic/Madeira
    {  363, 2748,   94523,  8, }, // Atlantic/Reykjavik
    {  363, 7272,   94531, 18, }, // Atlantic/South_Georgia
    {  363, 6821,   94549, 10, }, // Atlantic/St_Helena
    {  363, 6441,   94559,  9, }, // Europe/Amsterdam
    {  363, 6216,   94568,  6, }, // Europe/Andorra
    {  363, 3620,   94574, 12, }, // Europe/Astrakhan
    {  363, 2300,   94586,  9, }, // Europe/Athens
    {  363, 6336,   94595,  8, }, // Europe/Belgrade
    {  363, 4757,   94603,  7, }, // Europe/Berlin
    {  363, 6488,   94610, 14, }, // Europe/Bratislava
    {  363, 2167,   94624, 12, }, // Europe/Brussels
    {  363, 3928,   94636,  9, }, // Europe/Bucharest
    {  363, 3806,   94645, 10, }, // Europe/Budapest
    {  363, 2583,   94655,  7, }, // Europe/Chisinau
    {  363, 6289,   94662, 10, }, // Europe/Copenhagen
    {  363, 2338,   94672, 17, }, // Europe/Dublin
    {  363, 6307,   94689, 10, }, // Europe/Gibraltar
    {  363, 8788,   94699,  7, }, // Europe/Guernsey
    {  363, 6612,   94706,  9, }, // Europe/Helsinki
    {  363, 8804,   94715, 14, }, // Europe/Isle_of_Man
    {  363, 1130,   94729,  9, }, // Europe/Istanbul
    {  363, 8823,   94738,  6, }, // Europe/Jersey
    {  363, 2527,   94744,  3, }, // Europe/Kyiv
    {  363, 3213,   94747,  8, }, // Europe/Lisbon
    {  363, 6506,   94755, 10, }, // Europe/Ljubljana
    {  363, 2501,   94765,  7, }, // Europe/London
    {  363, 6365,   94772, 10, }, // Europe/Luxembourg
    {  363, 4892,   94782,  7, }, // Europe/Madrid
    {  363, 6397,   92031,  5, }, // Europe/Malta
    {  363, 3693,   94789,  6, }, // Europe/Minsk
    {  363, 6410,   94795,  6, }, // Europe/Monaco
    {  363, 3537,   94801,  5, }, // Europe/Moscow
    {  363, 6458,   92958,  4, }, // Europe/Oslo
    {  363, 4284,   94806,  5, }, // Europe/Paris
    {  363, 6424,   94811, 11, }, // Europe/Podgorica
    {  363, 6275,   92055,  4, }, // Europe/Prague
    {  363, 8837,   94822,  4, }, // Europe/Riga
    {  363, 6324,   94826,  8, }, // Europe/Rome
    {  363, 6470,   94834, 11, }, // Europe/San_Marino
    {  363, 6245,   94845,  8, }, // Europe/Sarajevo
    {  363, 6383,   94853,  6, }, // Europe/Skopje
    {  363, 6599,   94859,  6, }, // Europe/Sofia
    {  363, 6523,   94865,  7, }, // Europe/Stockholm
    {  363, 8867,   94872,  8, }, // Europe/Tallinn
    {  363, 6202,   94880,  7, }, // Europe/Tirane
    {  363, 6352,   94887,  5, }, // Europe/Vaduz
    {  363, 6567,   94892, 11, }, // Europe/Vatican
    {  363, 6231,   93011,  3, }, // Europe/Vienna
    {  363, 8899,   94903,  7, }, // Europe/Vilnius
    {  363, 3190,   94910,  5, }, // Europe/Warsaw
    {  363, 6261,   94915,  6, }, // Europe/Zagreb
    {  363, 6540,    4577,  6, }, // Europe/Zurich
    {  363, 5078,   94921, 14, }, // Indian/Antananarivo
    {  363, 6046,   94935, 16, }, // Indian/Christmas
    {  363, 6063,   94951, 12, }, // Indian/Cocos
    {  363, 5029,   94963, 15, }, // Indian/Comoro
    {  363, 7050,   94978, 17, }, // Indian/Maldives
    {  363, 4069,   94995, 14, }, // Indian/Mauritius
    {  363, 7211,   65518, 10, }, // Indian/Reunion
    {  363, 3157,     554,  5, }, // Pacific/Chuuk
    {  363, 3902,   95009,  5, }, // Pacific/Fiji
    {  363, 2723,    4611,  8, }, // Pacific/Honolulu
    {  363, 3049,    4619,  9, }, // Pacific/Kanton
    {  363, 4051,   95014, 23, }, // Pacific/Marquesas
    {  363, 8914,   95037, 14, }, // Pacific/Midway
    {  363, 7084,   93091,  4, }, // Pacific/Nauru
    {  363, 3096,     586,  7, }, // Pacific/Pohnpei
    {  363, 7312,   95051,  7, }, // Pacific/Tahiti
    {  363, 7405,   95058,  6, }, // Pacific/Wallis
    {  365, 5059,    3902, 11, }, // Africa/Addis_Ababa Italian/Latin/Italy
    {  365, 7420,   95064,  6, }, // Africa/Algiers
    {  365,   14,       0,  6, }, // Africa/Asmara
    {  365, 2320,   95070,  8, }, // Africa/Cairo
    {  365, 5043,   95078,  6, }, // Africa/Djibouti
    {  365, 5166,   95084,  7, }, // Africa/El_Aaiun
    {  365, 4499,   95091,  5, }, // Africa/Juba
    {  365, 4524,   26504,  7, }, // Africa/Khartoum
    {  365, 6869,   20418,  4, }, // Africa/Lome
    {  365, 5113,   44110, 10, }, // Africa/Mogadishu
    {  365, 5271,   44128,  9, }, // Africa/Ndjamena
    {  365, 4435,       6,  8, }, // Africa/Sao_Tome
    {  365, 6554,   95096,  6, }, // Africa/Tunis
    {  365, 7571,   20486,  7, }, // America/Argentina/Tucuman
    {  365, 4252,      14,  8, }, // America/Asuncion
    {  365,  237,      22,  8, }, // America/Atikokan
    {  365, 7623,      30, 17, }, // America/Bahia_Banderas
    {  365, 4361,   20507,  6, }, // America/Bogota
    {  365, 3868,      47,  6, }, // America/Cancun
    {  365, 4345,   95102,  7, }, // America/Cayenne
    {  365, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  365, 5723,      66,  7, }, // America/Curacao
    {  365, 5786,   95109,  9, }, // America/Guadeloupe
    {  365, 2281,   95118,  7, }, // America/Havana
    {  365,  481,      73, 13, }, // America/Indiana/Knox
    {  365, 7961,      86, 16, }, // America/Indiana/Marengo
    {  365, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  365, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  365, 8038,     139, 14, }, // America/Indiana/Vevay
    {  365, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  365, 8086,     171, 16, }, // America/Indiana/Winamac
    {  365, 2799,   95125,  8, }, // America/Jamaica
    {  365, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  365, 5932,     207, 22, }, // America/Lower_Princes
    {  365, 5805,   20554,  9, }, // America/Martinique
    {  365, 8220,     229,  6, }, // America/Merida
    {  365, 2949,   95133, 17, }, // America/Mexico_City
    {  365, 1850,     235, 19, }, // America/Noronha
    {  365, 8301,   95150, 23, }, // America/North_Dakota/Beulah
    {  365, 8329,   95173, 23, }, // America/North_Dakota/Center
    {  365, 8357,   95196, 26, }, // America/North_Dakota/New_Salem
    {  365,  393,     317,  4, }, // America/Nuuk
    {  365, 5843,   95222,  9, }, // America/Puerto_Rico
    {  365, 8477,   26704,  8, }, // America/Santarem
    {  365, 1878,   95231,  9, }, // America/Sao_Paulo
    {  365, 6899,     321, 16, }, // America/Scoresbysund
    {  365, 8508,   26721, 16, }, // America/St_Barthelemy
    {  365, 2061,     351, 10, }, // America/St_Johns
    {  365, 5863,     361,  9, }, // America/St_Kitts
    {  365, 5880,   95240, 11, }, // America/St_Lucia
    {  365,  855,   44293, 12, }, // America/St_Thomas
    {  365, 5913,   26783, 13, }, // America/St_Vincent
    {  365, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  365, 5511,   95251,  7, }, // Asia/Anadyr
    {  365, 5368,   57205,  6, }, // Asia/Aqtobe
    {  365, 5534,    4168,  7, }, // Asia/Bahrain
    {  365, 4601,   57622,  4, }, // Asia/Chita
    {  365, 4540,   74438,  7, }, // Asia/Damascus
    {  365, 1093,   65717,  5, }, // Asia/Dhaka
    {  365, 8636,   95258,  9, }, // Asia/Famagusta
    {  365, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  365, 7006,   95267,  8, }, // Asia/Jakarta
    {  365, 1290,   95275, 11, }, // Asia/Jerusalem
    {  365, 1185,     448,  9, }, // Asia/Kathmandu
    {  365, 8661,   58020,  8, }, // Asia/Khandyga
    {  365,  992,   36053,  8, }, // Asia/Kolkata
    {  365, 4197,    4288, 11, }, // Asia/Krasnoyarsk
    {  365, 1211,     464,  5, }, // Asia/Macau
    {  365, 6920,   20857,  7, }, // Asia/Muscat
    {  365, 8675,   62465, 11, }, // Asia/Novokuznetsk
    {  365, 3584,   20874,  5, }, // Asia/Riyadh
    {  365, 4408,   58084,  8, }, // Asia/Sakhalin
    {  365, 7245,   20886, 10, }, // Asia/Samarkand
    {  365, 3279,    7798,  4, }, // Asia/Seoul
    {  365, 2772,    7842,  7, }, // Asia/Tehran
    {  365, 1021,   26923, 10, }, // Asia/Ulaanbaatar
    {  365, 8722,   44432,  9, }, // Asia/Ust-Nera
    {  365, 4828,    7873,  7, }, // Asia/Yakutsk
    {  365, 1235,   58618,  7, }, // Asia/Yangon
    {  365, 3883,    7886, 12, }, // Asia/Yekaterinburg
    {  365, 3663,   95286,  7, }, // Atlantic/Azores
    {  365, 6628,   20967,  7, }, // Atlantic/Canary
    {  365, 3727,   93384, 10, }, // Atlantic/Cape_Verde
    {  365, 1395,   95293, 13, }, // Atlantic/Faroe
    {  365, 2748,   36176,  9, }, // Atlantic/Reykjavik
    {  365, 7272,   95306, 15, }, // Atlantic/South_Georgia
    {  365, 6821,   95321, 10, }, // Atlantic/St_Helena
    {  365, 2300,    5911,  5, }, // Europe/Athens
    {  365, 6336,   58906,  8, }, // Europe/Belgrade
    {  365, 4757,   62866,  7, }, // Europe/Berlin
    {  365, 2167,   57661,  9, }, // Europe/Brussels
    {  365, 3928,   21073,  8, }, // Europe/Bucharest
    {  365, 8772,   27037,  8, }, // Europe/Busingen
    {  365, 6289,   95331, 10, }, // Europe/Copenhagen
    {  365, 2338,   62946,  7, }, // Europe/Dublin
    {  365, 6307,   95341, 10, }, // Europe/Gibraltar
    {  365, 8804,   95351, 12, }, // Europe/Isle_of_Man
    {  365, 2527,    8148,  4, }, // Europe/Kyiv
    {  365, 3213,   93414,  7, }, // Europe/Lisbon
    {  365, 6506,   95363,  7, }, // Europe/Ljubljana
    {  365, 2501,   95370,  6, }, // Europe/London
    {  365, 6365,   95376, 11, }, // Europe/Luxembourg
    {  365, 3537,   95387,  5, }, // Europe/Moscow
    {  365, 4284,   95392,  6, }, // Europe/Paris
    {  365, 6275,   21185,  5, }, // Europe/Prague
    {  365, 6324,    6064,  4, }, // Europe/Rome
    {  365, 8849,   95398, 11, }, // Europe/Simferopol
    {  365, 6523,   95409,  9, }, // Europe/Stockholm
    {  365, 6202,   21213,  6, }, // Europe/Tirane
    {  365, 6567,   95418, 18, }, // Europe/Vatican
    {  365, 3190,   95436,  8, }, // Europe/Warsaw
    {  365, 6261,   95444,  8, }, // Europe/Zagreb
    {  365, 6540,   95452,  6, }, // Europe/Zurich
    {  365, 5029,    4588,  6, }, // Indian/Comoro
    {  365, 7050,    8311,  7, }, // Indian/Maldives
    {  365, 7211,   95458, 11, }, // Indian/Reunion
    {  365, 3157,     554,  5, }, // Pacific/Chuuk
    {  365, 2237,   44642,  6, }, // Pacific/Easter
    {  365, 3902,   95469,  4, }, // Pacific/Fiji
    {  365, 2723,    4611,  8, }, // Pacific/Honolulu
    {  365, 3049,    4619,  9, }, // Pacific/Kanton
    {  365, 4051,   95473,  8, }, // Pacific/Marquesas
    {  365, 3096,     586,  7, }, // Pacific/Pohnpei
    {  369, 6788,   95481,  5, }, // Africa/Abidjan Japanese/Japanese/Japan
    {  369, 6760,   95486,  3, }, // Africa/Accra
    {  369, 5059,   95489,  6, }, // Africa/Addis_Ababa
    {  369, 7420,   95495,  4, }, // Africa/Algiers
    {  369,   14,   95499,  4, }, // Africa/Asmara
    {  369,   44,   95503,  3, }, // Africa/Bamako
    {  369, 5257,   95506,  3, }, // Africa/Bangui
    {  369, 6746,   95509,  6, }, // Africa/Banjul
    {  369, 7435,   95515,  3, }, // Africa/Bissau
    {  369, 4957,   95518,  6, }, // Africa/Blantyre
    {  369, 5287,   95524,  5, }, // Africa/Brazzaville
    {  369, 4922,   95529,  6, }, // Africa/Bujumbura
    {  369, 2320,   95535,  3, }, // Africa/Cairo
    {  369, 4117,   95538,  6, }, // Africa/Casablanca
    {  369, 7449,   95544,  3, }, // Africa/Ceuta
    {  369, 6773,   95547,  4, }, // Africa/Conakry
    {  369, 6840,   95551,  4, }, // Africa/Dakar
    {  369, 5130,   95555,  8, }, // Africa/Dar_es_Salaam
    {  369, 5043,   95563,  3, }, // Africa/Djibouti
    {  369, 5243,   95566,  4, }, // Africa/Douala
    {  369, 5166,   95570,  4, }, // Africa/El_Aaiun
    {  369, 6853,   95574,  6, }, // Africa/Freetown
    {  369, 4906,   95580,  5, }, // Africa/Gaborone
    {  369, 5015,   95585,  3, }, // Africa/Harare
    {  369, 4479,   95588,  7, }, // Africa/Johannesburg
    {  369, 4499,   95595,  3, }, // Africa/Juba
    {  369, 5151,   95598,  4, }, // Africa/Kampala
    {  369, 4524,   95602,  5, }, // Africa/Khartoum
    {  369, 4973,   95607,  3, }, // Africa/Kigali
    {  369, 5306,   95610,  5, }, // Africa/Kinshasa
    {  369, 4744,   95615,  3, }, // Africa/Lagos
    {  369, 5336,   95618,  7, }, // Africa/Libreville
    {  369, 6869,   95625,  2, }, // Africa/Lome
    {  369, 5211,   95627,  4, }, // Africa/Luanda
    {  369, 4939,   95631,  5, }, // Africa/Lubumbashi
    {  369, 5001,   95636,  3, }, // Africa/Lusaka
    {  369, 5322,   95639,  3, }, // Africa/Malabo
    {  369, 4987,   95642,  3, }, // Africa/Maputo
    {  369, 5197,   95645,  3, }, // Africa/Maseru
    {  369, 5182,   95648,  5, }, // Africa/Mbabane
    {  369, 5113,   95653,  6, }, // Africa/Mogadishu
    {  369, 7034,   95659,  5, }, // Africa/Monrovia
    {  369, 3853,   95664,  4, }, // Africa/Nairobi
    {  369, 5271,   95668,  5, }, // Africa/Ndjamena
    {  369, 5354,   95673,  3, }, // Africa/Niamey
    {  369, 6803,   95676,  7, }, // Africa/Nouakchott
    {  369, 6727,   95683,  6, }, // Africa/Ouagadougou
    {  369, 5225,   95689,  5, }, // Africa/Porto-Novo
    {  369, 4435,   95694,  4, }, // Africa/Sao_Tome
    {  369, 2866,   95698,  4, }, // Africa/Tripoli
    {  369, 6554,   95702,  4, }, // Africa/Tunis
    {  369, 4152,   95706,  7, }, // Africa/Windhoek
    {  369,  132,   95713,  4, }, // America/Adak
    {  369, 3346,   95717,  6, }, // America/Anchorage
    {  369, 5607,   95723,  4, }, // America/Anguilla
    {  369, 5624,   95727,  6, }, // America/Antigua
    {  369, 4554,   95733,  6, }, // America/Araguaina
    {  369,  166,   95739,  8, }, // America/Argentina/Buenos_Aires
    {  369,   91,   95747,  5, }, // America/Argentina/Catamarca
    {  369,  270,   95752,  4, }, // America/Argentina/Cordoba
    {  369,  441,   95756,  3, }, // America/Argentina/Jujuy
    {  369, 7462,   95759,  4, }, // America/Argentina/La_Rioja
    {  369,  565,   95763,  5, }, // America/Argentina/Mendoza
    {  369, 7489,   95768,  7, }, // America/Argentina/Rio_Gallegos
    {  369, 7520,   95775,  3, }, // America/Argentina/Salta
    {  369, 7544,   95778,  5, }, // America/Argentina/San_Juan
    {  369, 5580,   95783,  5, }, // America/Argentina/San_Luis
    {  369, 7571,   95788,  5, }, // America/Argentina/Tucuman
    {  369, 7597,   95793,  6, }, // America/Argentina/Ushuaia
    {  369, 5640,   95799,  3, }, // America/Aruba
    {  369, 4252,   95802,  6, }, // America/Asuncion
    {  369,  237,   95808,  6, }, // America/Atikokan
    {  369, 3679,   95814,  4, }, // America/Bahia
    {  369, 7623,   95818,  7, }, // America/Bahia_Banderas
    {  369, 5654,   95825,  5, }, // America/Barbados
    {  369, 7646,   95830,  3, }, // America/Belem
    {  369, 5407,   95833,  4, }, // America/Belize
    {  369, 7660,   95837,  8, }, // America/Blanc-Sablon
    {  369, 7681,   95845,  5, }, // America/Boa_Vista
    {  369, 4361,   95850,  3, }, // America/Bogota
    {  369, 7699,   95853,  3, }, // America/Boise
    {  369, 7713,   95856,  8, }, // America/Cambridge_Bay
    {  369, 7735,   95864,  7, }, // America/Campo_Grande
    {  369, 3868,   95871,  4, }, // America/Cancun
    {  369, 4694,   95875,  4, }, // America/Caracas
    {  369, 4345,   95879,  5, }, // America/Cayenne
    {  369, 5496,   95884,  4, }, // America/Cayman
    {  369, 2260,   95888,  3, }, // America/Chicago
    {  369, 7756,   95891,  3, }, // America/Chihuahua
    {  369, 7774,   95894,  9, }, // America/Ciudad_Juarez
    {  369, 5422,   95903,  5, }, // America/Costa_Rica
    {  369, 7796,   95908,  5, }, // America/Creston
    {  369, 3791,   95913,  4, }, // America/Cuiaba
    {  369, 5723,   95917,  5, }, // America/Curacao
    {  369, 7812,   95922,  9, }, // America/Danmarkshavn
    {  369, 7833,   95931,  4, }, // America/Dawson
    {  369, 7848,   95935,  8, }, // America/Dawson_Creek
    {  369,  805,   95943,  4, }, // America/Denver
    {  369, 3465,   95947,  5, }, // America/Detroit
    {  369, 5739,   95952,  4, }, // America/Dominica
    {  369,  893,   95956,  6, }, // America/Edmonton
    {  369, 7869,   95962,  5, }, // America/Eirunepe
    {  369, 5441,   95967,  7, }, // America/El_Salvador
    {  369, 7886,   95974,  8, }, // America/Fort_Nelson
    {  369, 7906,   95982,  6, }, // America/Fortaleza
    {  369, 7924,   95988,  6, }, // America/Glace_Bay
    {  369, 6881,   95994,  5, }, // America/Goose_Bay
    {  369, 4612,   95999,  7, }, // America/Grand_Turk
    {  369, 5770,   96006,  4, }, // America/Grenada
    {  369, 5786,   96010,  6, }, // America/Guadeloupe
    {  369, 3760,   96016,  5, }, // America/Guatemala
    {  369, 4841,   96021,  5, }, // America/Guayaquil
    {  369, 6932,   96026,  4, }, // America/Guyana
    {  369, 1939,   96030,  6, }, // America/Halifax
    {  369, 2281,   96036,  3, }, // America/Havana
    {  369, 7942,   96039,  5, }, // America/Hermosillo
    {  369,  348,   96044,  9, }, // America/Indiana/Indianapolis
    {  369,  481,   96053, 11, }, // America/Indiana/Knox
    {  369, 7961,   96064, 11, }, // America/Indiana/Marengo
    {  369, 7985,   96075, 15, }, // America/Indiana/Petersburg
    {  369, 8012,   96090, 12, }, // America/Indiana/Tell_City
    {  369, 8038,   96102, 10, }, // America/Indiana/Vevay
    {  369, 8060,   96112, 12, }, // America/Indiana/Vincennes
    {  369, 8086,   96124, 13, }, // America/Indiana/Winamac
    {  369, 8110,   96137,  5, }, // America/Inuvik
    {  369,  660,   96142,  6, }, // America/Iqaluit
    {  369, 2799,   96148,  5, }, // America/Jamaica
    {  369, 5380,   96153,  4, }, // America/Juneau
    {  369,  521,   96157,  4, }, // America/Kentucky/Louisville
    {  369, 8125,   96161, 14, }, // America/Kentucky/Monticello
    {  369, 5704,   96175,  7, }, // America/Kralendijk
    {  369, 4376,   96182,  3, }, // America/La_Paz
    {  369, 7169,   96185,  2, }, // America/Lima
    {  369, 3239,   96187,  6, }, // America/Los_Angeles
    {  369, 5932,   96193, 17, }, // America/Lower_Princes
    {  369, 8153,   96210,  4, }, // America/Maceio
    {  369, 8168,   96214,  4, }, // America/Managua
    {  369, 1908,   96218,  4, }, // America/Manaus
    {  369, 5897,   96222,  3, }, // America/Marigot
    {  369, 5805,   96225,  7, }, // America/Martinique
    {  369, 8184,   96232,  5, }, // America/Matamoros
    {  369, 2917,   96237,  5, }, // America/Mazatlan
    {  369, 8202,   96242,  5, }, // America/Menominee
    {  369, 8220,   96247,  3, }, // America/Merida
    {  369, 8235,   96250,  6, }, // America/Metlakatla
    {  369, 2949,   96256,  8, }, // America/Mexico_City
    {  369, 4391,   96264,  5, }, // America/Miquelon
    {  369, 8254,   96269,  5, }, // America/Moncton
    {  369, 8270,   96274,  5, }, // America/Monterrey
    {  369, 4098,   96279,  6, }, // America/Montevideo
    {  369, 5824,   96285,  6, }, // America/Montserrat
    {  369, 5481,   96291,  4, }, // America/Nassau
    {  369, 2379,   96295,  6, }, // America/New_York
    {  369, 8288,   96301,  3, }, // America/Nome
    {  369, 1850,   96304,  5, }, // America/Noronha
    {  369, 8301,   96309, 12, }, // America/North_Dakota/Beulah
    {  369, 8329,   96321, 11, }, // America/North_Dakota/Center
    {  369, 8357,   96332, 14, }, // America/North_Dakota/New_Salem
    {  369,  393,   96346,  3, }, // America/Nuuk
    {  369, 8388,   96349,  4, }, // America/Ojinaga
    {  369, 2356,   96353,  3, }, // America/Panama
    {  369, 6173,   96356,  5, }, // America/Paramaribo
    {  369, 2973,   96361,  6, }, // America/Phoenix
    {  369, 3945,   96367,  8, }, // America/Port-au-Prince
    {  369, 5954,   96375,  9, }, // America/Port_of_Spain
    {  369, 8404,   96384,  7, }, // America/Porto_Velho
    {  369, 5843,   96391,  6, }, // America/Puerto_Rico
    {  369, 4030,   96397,  7, }, // America/Punta_Arenas
    {  369, 8424,   96404,  9, }, // America/Rankin_Inlet
    {  369, 8445,   96413,  4, }, // America/Recife
    {  369, 1995,   96417,  5, }, // America/Regina
    {  369, 8460,   96422,  6, }, // America/Resolute
    {  369,  695,   96428,  6, }, // America/Rio_Branco
    {  369, 8477,   96434,  5, }, // America/Santarem
    {  369, 2201,   96439,  5, }, // America/Santiago
    {  369, 6111,   96444,  7, }, // America/Santo_Domingo
    {  369, 1878,   96451,  5, }, // America/Sao_Paulo
    {  369, 6899,   96456,  9, }, // America/Scoresbysund
    {  369, 8494,   96465,  3, }, // America/Sitka
    {  369, 8508,   96468,  9, }, // America/St_Barthelemy
    {  369, 2061,   96477,  7, }, // America/St_Johns
    {  369, 5863,   96484, 10, }, // America/St_Kitts
    {  369, 5880,   96494,  6, }, // America/St_Lucia
    {  369,  855,   96500,  7, }, // America/St_Thomas
    {  369, 5913,   96507,  8, }, // America/St_Vincent
    {  369, 8530,   96515,  9, }, // America/Swift_Current
    {  369, 5461,   96524,  6, }, // America/Tegucigalpa
    {  369, 5756,   96530,  4, }, // America/Thule
    {  369,  313,   96534,  5, }, // America/Tijuana
    {  369,  608,   96539,  4, }, // America/Toronto
    {  369, 5688,   96543,  5, }, // America/Tortola
    {  369, 2093,   96548,  6, }, // America/Vancouver
    {  369, 2144,   96554,  7, }, // America/Whitehorse
    {  369,  734,   96561,  5, }, // America/Winnipeg
    {  369, 8552,   96566,  5, }, // America/Yakutat
    {  369, 6001,   96571,  6, }, // Antarctica/Casey
    {  369, 6094,   96577,  6, }, // Antarctica/Davis
    {  369, 6133,   96583, 13, }, // Antarctica/DumontDUrville
    {  369, 8568,   96596,  5, }, // Antarctica/Macquarie
    {  369, 7066,   96601,  6, }, // Antarctica/Mawson
    {  369, 7113,   96607,  7, }, // Antarctica/McMurdo
    {  369, 8589,   96614,  6, }, // Antarctica/Palmer
    {  369, 7226,   96620,  5, }, // Antarctica/Rothera
    {  369, 7295,   42325,  4, }, // Antarctica/Syowa
    {  369, 8607,   96625,  5, }, // Antarctica/Troll
    {  369, 7374,   96630,  7, }, // Antarctica/Vostok
    {  369, 1429,   96637, 10, }, // Arctic/Longyearbyen
    {  369, 5570,   96647,  3, }, // Asia/Aden
    {  369, 5395,   96650,  5, }, // Asia/Almaty
    {  369, 3968,   96655,  4, }, // Asia/Amman
    {  369, 5511,   96659,  5, }, // Asia/Anadyr
    {  369, 5523,   96664,  4, }, // Asia/Aqtau
    {  369, 5368,   96668,  4, }, // Asia/Aqtobe
    {  369,  964,   96672,  6, }, // Asia/Ashgabat
    {  369, 8624,   96678,  5, }, // Asia/Atyrau
    {  369, 3607,   96683,  5, }, // Asia/Baghdad
    {  369, 5534,   96688,  5, }, // Asia/Bahrain
    {  369, 3653,   96693,  3, }, // Asia/Baku
    {  369, 4466,   96696,  4, }, // Asia/Bangkok
    {  369, 3571,   96700,  5, }, // Asia/Barnaul
    {  369, 4086,   96705,  5, }, // Asia/Beirut
    {  369, 3778,   96710,  5, }, // Asia/Bishkek
    {  369, 5989,   96715,  4, }, // Asia/Brunei
    {  369, 4601,   96719,  2, }, // Asia/Chita
    {  369, 4511,   96721,  4, }, // Asia/Colombo
    {  369, 4540,   96725,  5, }, // Asia/Damascus
    {  369, 1093,   96730,  3, }, // Asia/Dhaka
    {  369, 6192,   96661,  3, }, // Asia/Dili
    {  369, 3596,   96733,  3, }, // Asia/Dubai
    {  369, 6159,   96736,  6, }, // Asia/Dushanbe
    {  369, 8636,   96742,  6, }, // Asia/Famagusta
    {  369, 8651,   96748,  2, }, // Asia/Gaza
    {  369, 4795,   96750,  4, }, // Asia/Hebron
    {  369, 1259,   96754,  5, }, // Asia/Ho_Chi_Minh
    {  369, 2704,   42425,  2, }, // Asia/Hong_Kong
    {  369, 4771,   96759,  3, }, // Asia/Hovd
    {  369, 4184,   96762,  6, }, // Asia/Irkutsk
    {  369, 7006,   96768,  5, }, // Asia/Jakarta
    {  369, 6992,   96773,  5, }, // Asia/Jayapura
    {  369, 1290,   96778,  5, }, // Asia/Jerusalem
    {  369, 3560,   96783,  4, }, // Asia/Kabul
    {  369, 4316,   96787,  6, }, // Asia/Kamchatka
    {  369, 4239,   96793,  3, }, // Asia/Karachi
    {  369, 1185,   96796,  5, }, // Asia/Kathmandu
    {  369, 8661,   96801,  6, }, // Asia/Khandyga
    {  369,  992,   96807,  4, }, // Asia/Kolkata
    {  369, 4197,   96811,  8, }, // Asia/Krasnoyarsk
    {  369, 4859,   96819,  8, }, // Asia/Kuala_Lumpur
    {  369, 5976,   96827,  3, }, // Asia/Kuching
    {  369, 5547,   96830,  5, }, // Asia/Kuwait
    {  369, 1211,   96835,  3, }, // Asia/Macau
    {  369, 4017,   96838,  4, }, // Asia/Magadan
    {  369, 1349,   96842,  5, }, // Asia/Makassar
    {  369, 7182,   96847,  3, }, // Asia/Manila
    {  369, 6920,   96850,  5, }, // Asia/Muscat
    {  369, 2554,   96855,  4, }, // Asia/Nicosia
    {  369, 8675,   96859,  8, }, // Asia/Novokuznetsk
    {  369, 4135,   96867,  8, }, // Asia/Novosibirsk
    {  369, 4229,   96875,  4, }, // Asia/Omsk
    {  369, 7145,   96879,  3, }, // Asia/Oral
    {  369, 6961,   96882,  5, }, // Asia/Phnom_Penh
    {  369, 8693,   96887,  8, }, // Asia/Pontianak
    {  369, 4214,   96895,  2, }, // Asia/Pyongyang
    {  369, 5559,   96897,  4, }, // Asia/Qatar
    {  369, 8708,   96901,  5, }, // Asia/Qostanay
    {  369, 4269,   96906,  5, }, // Asia/Qyzylorda
    {  369, 3584,   96911,  3, }, // Asia/Riyadh
    {  369, 4408,   96914,  4, }, // Asia/Sakhalin
    {  369, 7245,   96918,  6, }, // Asia/Samarkand
    {  369, 3279,   96924,  3, }, // Asia/Seoul
    {  369, 1053,   42567,  2, }, // Asia/Shanghai
    {  369, 3300,   96927,  6, }, // Asia/Singapore
    {  369, 4297,   96933,  9, }, // Asia/Srednekolymsk
    {  369, 3263,   42578,  2, }, // Asia/Taipei
    {  369, 4781,   96942,  5, }, // Asia/Tashkent
    {  369, 3915,   96947,  4, }, // Asia/Tbilisi
    {  369, 2772,   96951,  4, }, // Asia/Tehran
    {  369, 1317,   96955,  5, }, // Asia/Thimphu
    {  369, 2821,   42592,  2, }, // Asia/Tokyo
    {  369, 4572,   96960,  4, }, // Asia/Tomsk
    {  369, 1021,   96964,  7, }, // Asia/Ulaanbaatar
    {  369, 1159,   96971,  4, }, // Asia/Urumqi
    {  369, 8722,   96975,  5, }, // Asia/Ust-Nera
    {  369, 6977,   96980,  6, }, // Asia/Vientiane
    {  369, 4710,   96986,  7, }, // Asia/Vladivostok
    {  369, 4828,   96993,  5, }, // Asia/Yakutsk
    {  369, 1235,   96998,  4, }, // Asia/Yangon
    {  369, 3883,   97002,  8, }, // Asia/Yekaterinburg
    {  369, 3747,   97010,  4, }, // Asia/Yerevan
    {  369, 3663,   97014,  4, }, // Atlantic/Azores
    {  369, 5671,   97018,  5, }, // Atlantic/Bermuda
    {  369, 6628,   97023,  4, }, // Atlantic/Canary
    {  369, 3727,   97027,  6, }, // Atlantic/Cape_Verde
    {  369, 1395,   97033,  4, }, // Atlantic/Faroe
    {  369, 8736,   97037,  4, }, // Atlantic/Madeira
    {  369, 2748,   97041,  6, }, // Atlantic/Reykjavik
    {  369, 7272,   97047,  8, }, // Atlantic/South_Georgia
    {  369, 6821,   97055,  6, }, // Atlantic/St_Helena
    {  369, 6644,   97061,  5, }, // Atlantic/Stanley
    {  369, 1670,   97066,  5, }, // Australia/Adelaide
    {  369, 1635,   97071,  5, }, // Australia/Brisbane
    {  369, 1799,   97076,  7, }, // Australia/Broken_Hill
    {  369, 1583,   97083,  5, }, // Australia/Darwin
    {  369, 3637,   97088,  4, }, // Australia/Eucla
    {  369, 1516,   97092,  4, }, // Australia/Hobart
    {  369, 8753,   97096,  5, }, // Australia/Lindeman
    {  369, 1547,   97101,  5, }, // Australia/Lord_Howe
    {  369, 1727,   97106,  5, }, // Australia/Melbourne
    {  369, 1762,   97111,  3, }, // Australia/Perth
    {  369, 1463,   97114,  4, }, // Australia/Sydney
    {  369, 6441,   97118,  7, }, // Europe/Amsterdam
    {  369, 6216,   97125,  4, }, // Europe/Andorra
    {  369, 3620,   97129,  6, }, // Europe/Astrakhan
    {  369, 2300,   97135,  3, }, // Europe/Athens
    {  369, 6336,   97138,  6, }, // Europe/Belgrade
    {  369, 4757,   97144,  4, }, // Europe/Berlin
    {  369, 6488,   97148,  6, }, // Europe/Bratislava
    {  369, 2167,   97154,  6, }, // Europe/Brussels
    {  369, 3928,   97160,  5, }, // Europe/Bucharest
    {  369, 3806,   97165,  5, }, // Europe/Budapest
    {  369, 8772,   97170,  7, }, // Europe/Busingen
    {  369, 2583,   97177,  4, }, // Europe/Chisinau
    {  369, 6289,   97181,  7, }, // Europe/Copenhagen
    {  369, 2338,   97188,  4, }, // Europe/Dublin
    {  369, 6307,   97192,  6, }, // Europe/Gibraltar
    {  369, 8788,   97198,  5, }, // Europe/Guernsey
    {  369, 6612,   97203,  5, }, // Europe/Helsinki
    {  369, 8804,   97208,  3, }, // Europe/Isle_of_Man
    {  369, 1130,   97211,  7, }, // Europe/Istanbul
    {  369, 8823,   97218,  5, }, // Europe/Jersey
    {  369, 3979,   97223,  9, }, // Europe/Kaliningrad
    {  369, 8929,   97232,  4, }, // Europe/Kirov
    {  369, 2527,   97236,  3, }, // Europe/Kyiv
    {  369, 3213,   97239,  4, }, // Europe/Lisbon
    {  369, 6506,   97243,  6, }, // Europe/Ljubljana
    {  369, 2501,   97249,  4, }, // Europe/London
    {  369, 6365,   97253,  7, }, // Europe/Luxembourg
    {  369, 4892,   97260,  5, }, // Europe/Madrid
    {  369, 6397,   97265,  3, }, // Europe/Malta
    {  369, 6582,   97268,  6, }, // Europe/Mariehamn
    {  369, 3693,   97274,  4, }, // Europe/Minsk
    {  369, 6410,   97278,  3, }, // Europe/Monaco
    {  369, 3537,   97281,  4, }, // Europe/Moscow
    {  369, 6458,   97285,  3, }, // Europe/Oslo
    {  369, 4284,   97288,  2, }, // Europe/Paris
    {  369, 6424,   97290,  6, }, // Europe/Podgorica
    {  369, 6275,   97296,  3, }, // Europe/Prague
    {  369, 8837,   97299,  2, }, // Europe/Riga
    {  369, 6324,   97035,  3, }, // Europe/Rome
    {  369, 4331,   97301,  3, }, // Europe/Samara
    {  369, 6470,   97304,  5, }, // Europe/San_Marino
    {  369, 6245,   97309,  4, }, // Europe/Sarajevo
    {  369, 4451,   97313,  4, }, // Europe/Saratov
    {  369, 8849,   97317,  7, }, // Europe/Simferopol
    {  369, 6383,   97324,  4, }, // Europe/Skopje
    {  369, 6599,   97328,  4, }, // Europe/Sofia
    {  369, 6523,   97332,  7, }, // Europe/Stockholm
    {  369, 8867,   97339,  3, }, // Europe/Tallinn
    {  369, 6202,   97342,  4, }, // Europe/Tirane
    {  369, 8882,   97346,  7, }, // Europe/Ulyanovsk
    {  369, 6352,   97353,  6, }, // Europe/Vaduz
    {  369, 6567,   97359,  4, }, // Europe/Vatican
    {  369, 6231,   97363,  4, }, // Europe/Vienna
    {  369, 8899,   97367,  6, }, // Europe/Vilnius
    {  369, 4727,   97373,  7, }, // Europe/Volgograd
    {  369, 3190,   97380,  5, }, // Europe/Warsaw
    {  369, 6261,   97385,  4, }, // Europe/Zagreb
    {  369, 6540,   97389,  6, }, // Europe/Zurich
    {  369, 5078,   97395,  7, }, // Indian/Antananarivo
    {  369, 6947,   97402,  4, }, // Indian/Chagos
    {  369, 6046,   97406,  6, }, // Indian/Christmas
    {  369, 6063,   97412,  5, }, // Indian/Cocos
    {  369, 5029,   97417,  3, }, // Indian/Comoro
    {  369, 6661,   97420,  7, }, // Indian/Kerguelen
    {  369, 7260,   97427,  2, }, // Indian/Mahe
    {  369, 7050,   97429,  5, }, // Indian/Maldives
    {  369, 4069,   97434,  6, }, // Indian/Mauritius
    {  369, 5098,   97440,  4, }, // Indian/Mayotte
    {  369, 7211,   97444,  5, }, // Indian/Reunion
    {  369, 4422,   97449,  3, }, // Pacific/Apia
    {  369,  932,   97452,  6, }, // Pacific/Auckland
    {  369, 3706,   97458,  6, }, // Pacific/Bougainville
    {  369, 3015,   97464,  4, }, // Pacific/Chatham
    {  369, 3157,   97468,  4, }, // Pacific/Chuuk
    {  369, 2237,   97472,  6, }, // Pacific/Easter
    {  369, 7360,   97478,  5, }, // Pacific/Efate
    {  369, 7327,   97483,  6, }, // Pacific/Fakaofo
    {  369, 3902,   97489,  4, }, // Pacific/Fiji
    {  369, 7343,   97493,  5, }, // Pacific/Funafuti
    {  369, 6678,   97498,  5, }, // Pacific/Galapagos
    {  369, 6696,   97503,  6, }, // Pacific/Gambier
    {  369, 3822,   97509,  6, }, // Pacific/Guadalcanal
    {  369, 6018,   97515,  3, }, // Pacific/Guam
    {  369, 2723,   97518,  4, }, // Pacific/Honolulu
    {  369, 3049,   97522,  8, }, // Pacific/Kanton
    {  369, 3998,   97530,  8, }, // Pacific/Kiritimati
    {  369, 7019,   97538,  4, }, // Pacific/Kosrae
    {  369, 2842,   97542,  5, }, // Pacific/Kwajalein
    {  369, 4877,   97547,  4, }, // Pacific/Majuro
    {  369, 4051,   97551,  5, }, // Pacific/Marquesas
    {  369, 8914,   97556,  7, }, // Pacific/Midway
    {  369, 7084,   96702,  3, }, // Pacific/Nauru
    {  369, 7132,   97563,  3, }, // Pacific/Niue
    {  369, 4168,   97566,  7, }, // Pacific/Norfolk
    {  369, 7098,   97573,  3, }, // Pacific/Noumea
    {  369, 3126,   97576,  4, }, // Pacific/Pago_Pago
    {  369, 7155,   97580,  3, }, // Pacific/Palau
    {  369, 7194,   97583,  7, }, // Pacific/Pitcairn
    {  369, 3096,   97590,  5, }, // Pacific/Pohnpei
    {  369, 4807,   97595,  8, }, // Pacific/Port_Moresby
    {  369, 6076,   97603,  5, }, // Pacific/Rarotonga
    {  369, 6031,   97608,  4, }, // Pacific/Saipan
    {  369, 7312,   97612,  3, }, // Pacific/Tahiti
    {  369, 6712,   97615,  3, }, // Pacific/Tarawa
    {  369, 4583,   97618,  5, }, // Pacific/Tongatapu
    {  369, 7392,   97623,  5, }, // Pacific/Wake
    {  369, 7405,   97628,  6, }, // Pacific/Wallis
    {  370,   14,       0,  6, }, // Africa/Asmara Javanese/Latin/Indonesia
    {  370, 2320,    4690,  5, }, // Africa/Cairo
    {  370, 4117,    4695, 10, }, // Africa/Casablanca
    {  370, 6773,    6367,  7, }, // Africa/Conakry
    {  370, 4435,       6,  8, }, // Africa/Sao_Tome
    {  370,   91,    6518,  9, }, // America/Argentina/Catamarca
    {  370,  270,    6527,  7, }, // America/Argentina/Cordoba
    {  370,  565,   97634,  7, }, // America/Argentina/Mendoza
    {  370, 4252,      14,  8, }, // America/Asuncion
    {  370,  237,      22,  8, }, // America/Atikokan
    {  370, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  370, 5407,   97641,  6, }, // America/Belize
    {  370, 7713,   97647, 15, }, // America/Cambridge_Bay
    {  370, 7735,    4898, 12, }, // America/Campo_Grande
    {  370, 3868,   33314,  6, }, // America/Cancun
    {  370, 4694,    4910,  7, }, // America/Caracas
    {  370, 4345,   97662,  7, }, // America/Cayenne
    {  370, 5496,   44173,  6, }, // America/Cayman
    {  370, 7774,   97669, 13, }, // America/Ciudad_Juarez
    {  370, 5422,   21621, 10, }, // America/Costa_Rica
    {  370, 3791,    4964,  6, }, // America/Cuiaba
    {  370, 5723,      66,  7, }, // America/Curacao
    {  370, 5739,   21666,  8, }, // America/Dominica
    {  370, 7886,   97682, 14, }, // America/Fort_Nelson
    {  370, 7924,   97696, 11, }, // America/Glace_Bay
    {  370, 6881,   97707, 11, }, // America/Goose_Bay
    {  370, 1939,    6905,  7, }, // America/Halifax
    {  370,  481,   64737, 14, }, // America/Indiana/Knox
    {  370, 7961,   64751, 17, }, // America/Indiana/Marengo
    {  370, 7985,   64768, 20, }, // America/Indiana/Petersburg
    {  370, 8012,   64788, 19, }, // America/Indiana/Tell_City
    {  370, 8038,   64807, 15, }, // America/Indiana/Vevay
    {  370, 8060,   64822, 19, }, // America/Indiana/Vincennes
    {  370, 8086,   64841, 17, }, // America/Indiana/Winamac
    {  370, 8125,   64866, 21, }, // America/Kentucky/Monticello
    {  370, 5932,     207, 22, }, // America/Lower_Princes
    {  370, 5805,    5260,  8, }, // America/Martinique
    {  370, 8220,    7126,  6, }, // America/Merida
    {  370, 2949,   97718, 13, }, // America/Mexico_City
    {  370, 1850,     235, 19, }, // America/Noronha
    {  370, 8301,   97731, 19, }, // America/North_Dakota/Beulah
    {  370, 8329,   97750, 19, }, // America/North_Dakota/Center
    {  370, 8357,   97769, 24, }, // America/North_Dakota/New_Salem
    {  370,  393,     317,  4, }, // America/Nuuk
    {  370, 5954,   97793, 17, }, // America/Port_of_Spain
    {  370, 5843,   22058, 11, }, // America/Puerto_Rico
    {  370, 6899,     321, 16, }, // America/Scoresbysund
    {  370, 8508,   97810, 16, }, // America/St_Barthelemy
    {  370, 2061,   97826, 10, }, // America/St_Johns
    {  370, 5863,   97836, 11, }, // America/St_Kitts
    {  370, 5880,   95240, 11, }, // America/St_Lucia
    {  370,  855,   97847, 12, }, // America/St_Thomas
    {  370, 5913,   97859, 13, }, // America/St_Vincent
    {  370, 8530,   97872, 11, }, // America/Swift_Current
    {  370, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  370, 4511,    5637,  7, }, // Asia/Colombo
    {  370, 4540,    4201,  8, }, // Asia/Damascus
    {  370, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  370, 1290,    5686,  9, }, // Asia/Jerusalem
    {  370, 1185,     448,  9, }, // Asia/Kathmandu
    {  370,  992,   26864,  7, }, // Asia/Kolkata
    {  370, 1211,    4321,  5, }, // Asia/Macau
    {  370, 1349,    7736,  7, }, // Asia/Makassar
    {  370, 8708,   64342,  8, }, // Asia/Qostanay
    {  370, 3300,   93173,  9, }, // Asia/Singapore
    {  370, 2772,    7842,  7, }, // Asia/Tehran
    {  370, 1235,     477,  6, }, // Asia/Yangon
    {  370, 6628,    4398,  6, }, // Atlantic/Canary
    {  370, 3727,   22654, 10, }, // Atlantic/Cape_Verde
    {  370, 1395,     483,  5, }, // Atlantic/Faroe
    {  370, 7272,   97883, 13, }, // Atlantic/South_Georgia
    {  370, 6821,   26997, 12, }, // Atlantic/St_Helena
    {  370, 2300,   93210,  6, }, // Europe/Athens
    {  370, 6289,    4475, 10, }, // Europe/Copenhagen
    {  370, 8804,   97896,  8, }, // Europe/Isle_of_Man
    {  370, 2527,    8148,  4, }, // Europe/Kyiv
    {  370, 6365,   97904, 10, }, // Europe/Luxembourg
    {  370, 6410,    6044,  6, }, // Europe/Monaco
    {  370, 6324,    6064,  4, }, // Europe/Rome
    {  370, 6567,    6096,  7, }, // Europe/Vatican
    {  370, 6947,   97914,  6, }, // Indian/Chagos
    {  370, 6046,   93421,  5, }, // Indian/Christmas
    {  370, 5029,   22985,  6, }, // Indian/Comoro
    {  370, 7050,   93246,  8, }, // Indian/Maldives
    {  370, 7211,     547,  7, }, // Indian/Reunion
    {  370, 3157,     554,  5, }, // Pacific/Chuuk
    {  370, 2237,   97920,  6, }, // Pacific/Easter
    {  370, 3049,    4619,  9, }, // Pacific/Kanton
    {  370, 3096,     586,  7, }, // Pacific/Pohnpei
    {  370, 4807,   97926, 17, }, // Pacific/Port_Moresby
    {  371,   14,       0,  6, }, // Africa/Asmara Jju/Latin/Nigeria
    {  371, 4435,       6,  8, }, // Africa/Sao_Tome
    {  371, 4252,      14,  8, }, // America/Asuncion
    {  371,  237,      22,  8, }, // America/Atikokan
    {  371, 7623,      30, 17, }, // America/Bahia_Banderas
    {  371, 3868,      47,  6, }, // America/Cancun
    {  371, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  371, 5723,      66,  7, }, // America/Curacao
    {  371,  481,      73, 13, }, // America/Indiana/Knox
    {  371, 7961,      86, 16, }, // America/Indiana/Marengo
    {  371, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  371, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  371, 8038,     139, 14, }, // America/Indiana/Vevay
    {  371, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  371, 8086,     171, 16, }, // America/Indiana/Winamac
    {  371, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  371, 5932,     207, 22, }, // America/Lower_Princes
    {  371, 8220,     229,  6, }, // America/Merida
    {  371, 1850,     235, 19, }, // America/Noronha
    {  371, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  371, 8329,     274, 20, }, // America/North_Dakota/Center
    {  371, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  371,  393,     317,  4, }, // America/Nuuk
    {  371, 6899,     321, 16, }, // America/Scoresbysund
    {  371, 8508,     337, 14, }, // America/St_Barthelemy
    {  371, 2061,     351, 10, }, // America/St_Johns
    {  371, 5863,     361,  9, }, // America/St_Kitts
    {  371, 5880,     370,  9, }, // America/St_Lucia
    {  371,  855,     379, 10, }, // America/St_Thomas
    {  371, 5913,     389, 11, }, // America/St_Vincent
    {  371, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  371, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  371, 1185,     448,  9, }, // Asia/Kathmandu
    {  371,  992,     457,  7, }, // Asia/Kolkata
    {  371, 1211,     464,  5, }, // Asia/Macau
    {  371, 1235,     477,  6, }, // Asia/Yangon
    {  371, 1395,     483,  5, }, // Atlantic/Faroe
    {  371, 6821,     488, 10, }, // Atlantic/St_Helena
    {  371, 2527,     514,  4, }, // Europe/Kyiv
    {  371, 7211,     547,  7, }, // Indian/Reunion
    {  371, 3157,     554,  5, }, // Pacific/Chuuk
    {  371, 3096,     586,  7, }, // Pacific/Pohnpei
    {  372,   14,       0,  6, }, // Africa/Asmara Jola-Fonyi/Latin/Senegal
    {  372, 4435,       6,  8, }, // Africa/Sao_Tome
    {  372, 4252,      14,  8, }, // America/Asuncion
    {  372,  237,      22,  8, }, // America/Atikokan
    {  372, 7623,      30, 17, }, // America/Bahia_Banderas
    {  372, 3868,      47,  6, }, // America/Cancun
    {  372, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  372, 5723,      66,  7, }, // America/Curacao
    {  372,  481,      73, 13, }, // America/Indiana/Knox
    {  372, 7961,      86, 16, }, // America/Indiana/Marengo
    {  372, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  372, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  372, 8038,     139, 14, }, // America/Indiana/Vevay
    {  372, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  372, 8086,     171, 16, }, // America/Indiana/Winamac
    {  372, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  372, 5932,     207, 22, }, // America/Lower_Princes
    {  372, 8220,     229,  6, }, // America/Merida
    {  372, 1850,     235, 19, }, // America/Noronha
    {  372, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  372, 8329,     274, 20, }, // America/North_Dakota/Center
    {  372, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  372,  393,     317,  4, }, // America/Nuuk
    {  372, 6899,     321, 16, }, // America/Scoresbysund
    {  372, 8508,     337, 14, }, // America/St_Barthelemy
    {  372, 2061,     351, 10, }, // America/St_Johns
    {  372, 5863,     361,  9, }, // America/St_Kitts
    {  372, 5880,     370,  9, }, // America/St_Lucia
    {  372,  855,     379, 10, }, // America/St_Thomas
    {  372, 5913,     389, 11, }, // America/St_Vincent
    {  372, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  372, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  372, 1185,     448,  9, }, // Asia/Kathmandu
    {  372,  992,     457,  7, }, // Asia/Kolkata
    {  372, 1211,     464,  5, }, // Asia/Macau
    {  372, 1235,     477,  6, }, // Asia/Yangon
    {  372, 1395,     483,  5, }, // Atlantic/Faroe
    {  372, 6821,     488, 10, }, // Atlantic/St_Helena
    {  372, 2527,     514,  4, }, // Europe/Kyiv
    {  372, 7211,     547,  7, }, // Indian/Reunion
    {  372, 3157,     554,  5, }, // Pacific/Chuuk
    {  372, 3096,     586,  7, }, // Pacific/Pohnpei
    {  373,   14,       0,  6, }, // Africa/Asmara Kabuverdianu/Latin/Cape Verde
    {  373, 4435,       6,  8, }, // Africa/Sao_Tome
    {  373,  132,   97943,  5, }, // America/Adak
    {  373, 3346,   97948,  8, }, // America/Anchorage
    {  373, 5607,   97956,  6, }, // America/Anguilla
    {  373, 4252,      14,  8, }, // America/Asuncion
    {  373,  237,      22,  8, }, // America/Atikokan
    {  373, 7623,   97962, 16, }, // America/Bahia_Banderas
    {  373, 5654,   97978,  8, }, // America/Barbados
    {  373, 5407,   97986,  6, }, // America/Belize
    {  373, 7660,    6620, 12, }, // America/Blanc-Sablon
    {  373, 7699,    6647,  4, }, // America/Boise
    {  373, 3868,    6682,  6, }, // America/Cancun
    {  373, 5496,   26568,  6, }, // America/Cayman
    {  373, 2260,   97992,  6, }, // America/Chicago
    {  373, 7756,   97998,  6, }, // America/Chihuahua
    {  373, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  373, 5422,   21621, 10, }, // America/Costa_Rica
    {  373, 7796,   98004,  7, }, // America/Creston
    {  373, 5723,   98011,  7, }, // America/Curacao
    {  373, 5739,   21666,  8, }, // America/Dominica
    {  373, 5770,   20519,  7, }, // America/Grenada
    {  373, 5786,   98018,  9, }, // America/Guadeloupe
    {  373, 3760,   98027,  9, }, // America/Guatemala
    {  373, 2281,   95120,  5, }, // America/Havana
    {  373,  348,   74243, 12, }, // America/Indiana/Indianapolis
    {  373,  481,      73, 13, }, // America/Indiana/Knox
    {  373, 7961,      86, 16, }, // America/Indiana/Marengo
    {  373, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  373, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  373, 8038,     139, 14, }, // America/Indiana/Vevay
    {  373, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  373, 8086,     171, 16, }, // America/Indiana/Winamac
    {  373, 2799,    3995,  7, }, // America/Jamaica
    {  373, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  373, 5932,     207, 22, }, // America/Lower_Princes
    {  373, 8168,   98036,  7, }, // America/Managua
    {  373, 5805,   26594,  9, }, // America/Martinique
    {  373, 8220,     229,  6, }, // America/Merida
    {  373, 2949,   98043, 16, }, // America/Mexico_City
    {  373, 4391,    7149,  7, }, // America/Miquelon
    {  373, 2379,   98059, 10, }, // America/New_York
    {  373, 1850,     235, 19, }, // America/Noronha
    {  373, 8301,   98069, 23, }, // America/North_Dakota/Beulah
    {  373, 8329,   98092, 23, }, // America/North_Dakota/Center
    {  373, 8357,   98115, 26, }, // America/North_Dakota/New_Salem
    {  373,  393,     317,  4, }, // America/Nuuk
    {  373, 2356,   20665,  6, }, // America/Panama
    {  373, 5954,   98141, 15, }, // America/Port_of_Spain
    {  373, 5843,   98156, 10, }, // America/Puerto_Rico
    {  373, 6111,   98166, 13, }, // America/Santo_Domingo
    {  373, 6899,     321, 16, }, // America/Scoresbysund
    {  373, 8508,   98179, 14, }, // America/St_Barthelemy
    {  373, 2061,     351, 10, }, // America/St_Johns
    {  373, 5863,   98193, 13, }, // America/St_Kitts
    {  373, 5880,   98206, 11, }, // America/St_Lucia
    {  373,  855,     379, 10, }, // America/St_Thomas
    {  373, 5913,   98217, 11, }, // America/St_Vincent
    {  373,  608,   98228,  7, }, // America/Toronto
    {  373, 2093,    7493,  8, }, // America/Vancouver
    {  373,  734,   98235,  7, }, // America/Winnipeg
    {  373, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  373, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  373, 1185,     448,  9, }, // Asia/Kathmandu
    {  373,  992,     457,  7, }, // Asia/Kolkata
    {  373, 1211,     464,  5, }, // Asia/Macau
    {  373, 1235,     477,  6, }, // Asia/Yangon
    {  373, 1395,     483,  5, }, // Atlantic/Faroe
    {  373, 6821,     488, 10, }, // Atlantic/St_Helena
    {  373, 2527,     514,  4, }, // Europe/Kyiv
    {  373, 7211,     547,  7, }, // Indian/Reunion
    {  373, 3157,     554,  5, }, // Pacific/Chuuk
    {  373, 3096,     586,  7, }, // Pacific/Pohnpei
    {  374, 6788,   98242,  6, }, // Africa/Abidjan Kabyle/Latin/Algeria
    {  374, 6760,   98248,  4, }, // Africa/Accra
    {  374, 5059,   98252, 10, }, // Africa/Addis_Ababa
    {  374, 7420,   98262,  8, }, // Africa/Algiers
    {  374,   14,       0,  6, }, // Africa/Asmara
    {  374,   44,   98270,  6, }, // Africa/Bamako
    {  374, 5257,   26480,  5, }, // Africa/Bangui
    {  374, 7435,    4668,  5, }, // Africa/Bissau
    {  374, 4957,    6318,  7, }, // Africa/Blantyre
    {  374, 5287,   98276,  8, }, // Africa/Brazzaville
    {  374, 2320,   98284,  7, }, // Africa/Cairo
    {  374, 4117,   98291, 10, }, // Africa/Casablanca
    {  374, 7449,   98301,  5, }, // Africa/Ceuta
    {  374, 6773,   98306,  7, }, // Africa/Conakry
    {  374, 5130,   98313,  9, }, // Africa/Dar_es_Salaam
    {  374, 5043,   98322,  6, }, // Africa/Djibouti
    {  374, 5243,   98328,  5, }, // Africa/Douala
    {  374, 5166,   98333,  6, }, // Africa/El_Aaiun
    {  374, 6853,   98339,  7, }, // Africa/Freetown
    {  374, 4906,   98346,  7, }, // Africa/Gaborone
    {  374, 5015,   98353,  6, }, // Africa/Harare
    {  374, 4479,   98359, 11, }, // Africa/Johannesburg
    {  374, 4524,   98370,  6, }, // Africa/Khartoum
    {  374, 5306,   98376,  7, }, // Africa/Kinshasa
    {  374, 4744,   98383,  5, }, // Africa/Lagos
    {  374, 5336,   98388,  8, }, // Africa/Libreville
    {  374, 6869,   98396,  4, }, // Africa/Lome
    {  374, 5211,   98400,  6, }, // Africa/Luanda
    {  374, 4939,   98406,  9, }, // Africa/Lubumbashi
    {  374, 5001,   98415,  6, }, // Africa/Lusaka
    {  374, 5322,   98421,  6, }, // Africa/Malabo
    {  374, 4987,   98427,  6, }, // Africa/Maputo
    {  374, 5197,   98433,  6, }, // Africa/Maseru
    {  374, 5182,   61310,  6, }, // Africa/Mbabane
    {  374, 5113,   98439,  8, }, // Africa/Mogadishu
    {  374, 7034,   98447,  8, }, // Africa/Monrovia
    {  374, 3853,   98455,  7, }, // Africa/Nairobi
    {  374, 5271,   98462,  7, }, // Africa/Ndjamena
    {  374, 5354,   98469,  6, }, // Africa/Niamey
    {  374, 6803,   98475,  7, }, // Africa/Nouakchott
    {  374, 6727,    4758,  8, }, // Africa/Ouagadougou
    {  374, 5225,   98482, 10, }, // Africa/Porto-Novo
    {  374, 4435,   98492,  8, }, // Africa/Sao_Tome
    {  374, 2866,   98500,  7, }, // Africa/Tripoli
    {  374, 6554,   74198,  5, }, // Africa/Tunis
    {  374, 4152,   98507,  7, }, // Africa/Windhoek
    {  374, 3346,   98514,  7, }, // America/Anchorage
    {  374, 5607,   97956,  6, }, // America/Anguilla
    {  374, 5624,   98521,  6, }, // America/Antigua
    {  374,   91,    6518,  9, }, // America/Argentina/Catamarca
    {  374,  270,   98527,  7, }, // America/Argentina/Cordoba
    {  374,  565,   98534,  7, }, // America/Argentina/Mendoza
    {  374, 7544,   98541,  8, }, // America/Argentina/San_Juan
    {  374, 5580,   98549,  8, }, // America/Argentina/San_Luis
    {  374, 7571,    4842,  7, }, // America/Argentina/Tucuman
    {  374, 7597,   98557,  6, }, // America/Argentina/Ushuaia
    {  374, 4252,   98563,  8, }, // America/Asuncion
    {  374,  237,   98571,  8, }, // America/Atikokan
    {  374, 3679,   98579,  5, }, // America/Bahia
    {  374, 7623,   98584, 14, }, // America/Bahia_Banderas
    {  374, 5654,   97978,  8, }, // America/Barbados
    {  374, 5407,   98598,  5, }, // America/Belize
    {  374, 7660,   98603, 11, }, // America/Blanc-Sablon
    {  374, 4361,   98614,  6, }, // America/Bogota
    {  374, 7699,   98620,  4, }, // America/Boise
    {  374, 7713,   98624, 11, }, // America/Cambridge_Bay
    {  374, 7735,   98635, 12, }, // America/Campo_Grande
    {  374, 3868,    6682,  6, }, // America/Cancun
    {  374, 4694,    4910,  7, }, // America/Caracas
    {  374, 4345,   98647,  5, }, // America/Cayenne
    {  374, 5496,   21601,  6, }, // America/Cayman
    {  374, 2260,   98652,  6, }, // America/Chicago
    {  374, 7756,   98658,  6, }, // America/Chihuahua
    {  374, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  374, 5422,   98664, 10, }, // America/Costa_Rica
    {  374, 7796,   98674,  7, }, // America/Creston
    {  374, 5723,   98681,  7, }, // America/Curacao
    {  374, 7812,   98688, 12, }, // America/Danmarkshavn
    {  374, 7833,   98700,  6, }, // America/Dawson
    {  374, 7848,   98706, 11, }, // America/Dawson_Creek
    {  374, 3465,   98717,  6, }, // America/Detroit
    {  374, 5739,   98723,  7, }, // America/Dominica
    {  374,  893,   98730,  8, }, // America/Edmonton
    {  374, 5441,   98738,  8, }, // America/El_Salvador
    {  374, 7886,   98746, 11, }, // America/Fort_Nelson
    {  374, 7906,   98757,  9, }, // America/Fortaleza
    {  374, 5770,   98766,  7, }, // America/Grenada
    {  374, 5786,   98773,  9, }, // America/Guadeloupe
    {  374, 3760,   98782,  9, }, // America/Guatemala
    {  374, 4841,   98791,  7, }, // America/Guayaquil
    {  374, 6932,   98798,  5, }, // America/Guyana
    {  374,  481,      73, 13, }, // America/Indiana/Knox
    {  374, 7961,      86, 16, }, // America/Indiana/Marengo
    {  374, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  374, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  374, 8038,     139, 14, }, // America/Indiana/Vevay
    {  374, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  374, 8086,     171, 16, }, // America/Indiana/Winamac
    {  374, 2799,   98803,  7, }, // America/Jamaica
    {  374, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  374, 5932,     207, 22, }, // America/Lower_Princes
    {  374, 8168,   98810,  6, }, // America/Managua
    {  374, 5897,   98816,  7, }, // America/Marigot
    {  374, 5805,    5260,  8, }, // America/Martinique
    {  374, 8220,    7126,  6, }, // America/Merida
    {  374, 1850,   98823,  7, }, // America/Noronha
    {  374, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  374, 8329,     274, 20, }, // America/North_Dakota/Center
    {  374, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  374,  393,   62015,  3, }, // America/Nuuk
    {  374, 6173,   98830, 10, }, // America/Paramaribo
    {  374, 8404,   98840, 11, }, // America/Porto_Velho
    {  374, 8445,   98851,  5, }, // America/Recife
    {  374,  695,   98856, 10, }, // America/Rio_Branco
    {  374, 2201,   98866,  8, }, // America/Santiago
    {  374, 1878,   98874,  9, }, // America/Sao_Paulo
    {  374, 6899,     321, 16, }, // America/Scoresbysund
    {  374, 8508,     337, 14, }, // America/St_Barthelemy
    {  374, 2061,     351, 10, }, // America/St_Johns
    {  374, 5863,     361,  9, }, // America/St_Kitts
    {  374, 5880,     370,  9, }, // America/St_Lucia
    {  374,  855,     379, 10, }, // America/St_Thomas
    {  374, 5913,     389, 11, }, // America/St_Vincent
    {  374, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  374, 8568,   98883,  6, }, // Antarctica/Macquarie
    {  374, 7374,   98889,  6, }, // Antarctica/Vostok
    {  374, 1429,   98895, 11, }, // Arctic/Longyearbyen
    {  374, 5570,   98906,  5, }, // Asia/Aden
    {  374, 5395,    7594,  6, }, // Asia/Almaty
    {  374, 3968,   98911,  6, }, // Asia/Amman
    {  374, 5511,    7600,  6, }, // Asia/Anadyr
    {  374, 3607,   98917,  6, }, // Asia/Baghdad
    {  374, 5534,   98923,  6, }, // Asia/Bahrain
    {  374, 4466,   98929,  7, }, // Asia/Bangkok
    {  374, 4086,   36003,  6, }, // Asia/Beirut
    {  374, 5989,   98936,  6, }, // Asia/Brunei
    {  374, 4511,   98942,  7, }, // Asia/Colombo
    {  374, 4540,    4201,  5, }, // Asia/Damascus
    {  374, 3596,   22390,  5, }, // Asia/Dubai
    {  374, 8651,   98949,  5, }, // Asia/Gaza
    {  374, 4795,   98954,  6, }, // Asia/Hebron
    {  374, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  374, 2704,   98960,  9, }, // Asia/Hong_Kong
    {  374, 4771,   98969,  4, }, // Asia/Hovd
    {  374, 1290,   98973,  9, }, // Asia/Jerusalem
    {  374, 4239,   98982,  6, }, // Asia/Karachi
    {  374, 1185,    5711,  8, }, // Asia/Kathmandu
    {  374,  992,   98988,  7, }, // Asia/Kolkata
    {  374, 4859,   98995, 12, }, // Asia/Kuala_Lumpur
    {  374, 5547,   99007,  4, }, // Asia/Kuwait
    {  374, 1211,   99011,  5, }, // Asia/Macau
    {  374, 6920,   99016,  6, }, // Asia/Muscat
    {  374, 2554,   99022,  7, }, // Asia/Nicosia
    {  374, 4214,   99029,  9, }, // Asia/Pyongyang
    {  374, 5559,   99038,  5, }, // Asia/Qatar
    {  374, 8708,   99043,  8, }, // Asia/Qostanay
    {  374, 3584,   99051,  5, }, // Asia/Riyadh
    {  374, 3279,   99056,  5, }, // Asia/Seoul
    {  374, 1053,   99061,  6, }, // Asia/Shanghai
    {  374, 3300,   99067,  8, }, // Asia/Singapore
    {  374, 3263,   99075,  6, }, // Asia/Taipei
    {  374, 2821,   99081,  5, }, // Asia/Tokyo
    {  374, 1021,    7855,  9, }, // Asia/Ulaanbaatar
    {  374, 1235,   58618,  7, }, // Asia/Yangon
    {  374, 3747,   99086,  6, }, // Asia/Yerevan
    {  374, 3663,   99092,  6, }, // Atlantic/Azores
    {  374, 5671,   99098,  6, }, // Atlantic/Bermuda
    {  374, 6628,   99104, 17, }, // Atlantic/Canary
    {  374, 3727,   99121,  7, }, // Atlantic/Cape_Verde
    {  374, 1395,   99128,  5, }, // Atlantic/Faroe
    {  374, 8736,   99133,  6, }, // Atlantic/Madeira
    {  374, 7272,   99139, 15, }, // Atlantic/South_Georgia
    {  374, 6821,   99154, 10, }, // Atlantic/St_Helena
    {  374, 1670,   99164,  8, }, // Australia/Adelaide
    {  374, 1635,   62786,  7, }, // Australia/Brisbane
    {  374, 1727,   99172,  7, }, // Australia/Melbourne
    {  374, 1463,    5892,  5, }, // Australia/Sydney
    {  374, 6441,   99179, 10, }, // Europe/Amsterdam
    {  374, 6216,   99189,  6, }, // Europe/Andorra
    {  374, 2300,   33652,  5, }, // Europe/Athens
    {  374, 6336,   99195,  7, }, // Europe/Belgrade
    {  374, 2167,    8077,  7, }, // Europe/Brussels
    {  374, 3928,   22773,  8, }, // Europe/Bucharest
    {  374, 6289,   99202, 10, }, // Europe/Copenhagen
    {  374, 2338,   99212,  6, }, // Europe/Dublin
    {  374, 6307,   99218,  9, }, // Europe/Gibraltar
    {  374, 8788,   99227,  8, }, // Europe/Guernsey
    {  374, 6612,   99235,  8, }, // Europe/Helsinki
    {  374, 8804,   99243, 13, }, // Europe/Isle_of_Man
    {  374, 1130,   99256,  8, }, // Europe/Istanbul
    {  374, 8823,   99264,  6, }, // Europe/Jersey
    {  374, 2527,   99270,  4, }, // Europe/Kyiv
    {  374, 3213,   99274,  6, }, // Europe/Lisbon
    {  374, 2501,   99280,  6, }, // Europe/London
    {  374, 6365,    8172, 10, }, // Europe/Luxembourg
    {  374, 6582,   99286,  8, }, // Europe/Mariehamn
    {  374, 6410,   99294,  6, }, // Europe/Monaco
    {  374, 3537,   99300,  5, }, // Europe/Moscow
    {  374, 6458,   99305,  4, }, // Europe/Oslo
    {  374, 6275,    8201,  4, }, // Europe/Prague
    {  374, 6324,   99309,  3, }, // Europe/Rome
    {  374, 6470,   99312, 10, }, // Europe/San_Marino
    {  374, 6245,   99322,  8, }, // Europe/Sarajevo
    {  374, 6599,   99330,  5, }, // Europe/Sofia
    {  374, 6523,   99335,  8, }, // Europe/Stockholm
    {  374, 8867,   22925,  6, }, // Europe/Tallinn
    {  374, 6202,    8255,  5, }, // Europe/Tirane
    {  374, 6567,    6096,  7, }, // Europe/Vatican
    {  374, 6231,   99343,  6, }, // Europe/Vienna
    {  374, 8899,   99349,  7, }, // Europe/Vilnius
    {  374, 6540,   99356,  5, }, // Europe/Zurich
    {  374, 5078,   99361, 12, }, // Indian/Antananarivo
    {  374, 6947,   99373,  5, }, // Indian/Chagos
    {  374, 6046,   74095,  7, }, // Indian/Christmas
    {  374, 6063,   99378,  4, }, // Indian/Cocos
    {  374, 5029,   99382,  5, }, // Indian/Comoro
    {  374, 6661,   99387,  7, }, // Indian/Kerguelen
    {  374, 7260,   99394,  4, }, // Indian/Mahe
    {  374, 7050,    8311,  6, }, // Indian/Maldives
    {  374, 4069,   99398,  5, }, // Indian/Mauritius
    {  374, 5098,   99403,  5, }, // Indian/Mayotte
    {  374, 7211,   99408,  8, }, // Indian/Reunion
    {  374, 4422,   99416,  4, }, // Pacific/Apia
    {  374,  932,   99420,  6, }, // Pacific/Auckland
    {  374, 3015,   99426,  6, }, // Pacific/Chatham
    {  374, 3157,     554,  5, }, // Pacific/Chuuk
    {  374, 6678,   99432,  9, }, // Pacific/Galapagos
    {  374, 6696,   99441,  7, }, // Pacific/Gambier
    {  374, 2723,    4611,  8, }, // Pacific/Honolulu
    {  374, 3049,    4619,  9, }, // Pacific/Kanton
    {  374, 4051,   36340,  6, }, // Pacific/Marquesas
    {  374, 7132,   99448,  4, }, // Pacific/Niue
    {  374, 4168,   99452,  7, }, // Pacific/Norfolk
    {  374, 3096,     586,  7, }, // Pacific/Pohnpei
    {  375,   14,       0,  6, }, // Africa/Asmara Kako/Latin/Cameroon
    {  375, 4435,       6,  8, }, // Africa/Sao_Tome
    {  375, 4252,      14,  8, }, // America/Asuncion
    {  375,  237,      22,  8, }, // America/Atikokan
    {  375, 7623,      30, 17, }, // America/Bahia_Banderas
    {  375, 3868,      47,  6, }, // America/Cancun
    {  375, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  375, 5723,      66,  7, }, // America/Curacao
    {  375,  481,      73, 13, }, // America/Indiana/Knox
    {  375, 7961,      86, 16, }, // America/Indiana/Marengo
    {  375, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  375, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  375, 8038,     139, 14, }, // America/Indiana/Vevay
    {  375, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  375, 8086,     171, 16, }, // America/Indiana/Winamac
    {  375, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  375, 5932,     207, 22, }, // America/Lower_Princes
    {  375, 8220,     229,  6, }, // America/Merida
    {  375, 1850,     235, 19, }, // America/Noronha
    {  375, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  375, 8329,     274, 20, }, // America/North_Dakota/Center
    {  375, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  375,  393,     317,  4, }, // America/Nuuk
    {  375, 6899,     321, 16, }, // America/Scoresbysund
    {  375, 8508,     337, 14, }, // America/St_Barthelemy
    {  375, 2061,     351, 10, }, // America/St_Johns
    {  375, 5863,     361,  9, }, // America/St_Kitts
    {  375, 5880,     370,  9, }, // America/St_Lucia
    {  375,  855,     379, 10, }, // America/St_Thomas
    {  375, 5913,     389, 11, }, // America/St_Vincent
    {  375, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  375, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  375, 1185,     448,  9, }, // Asia/Kathmandu
    {  375,  992,     457,  7, }, // Asia/Kolkata
    {  375, 1211,     464,  5, }, // Asia/Macau
    {  375, 1235,     477,  6, }, // Asia/Yangon
    {  375, 1395,     483,  5, }, // Atlantic/Faroe
    {  375, 6821,     488, 10, }, // Atlantic/St_Helena
    {  375, 2527,     514,  4, }, // Europe/Kyiv
    {  375, 7211,     547,  7, }, // Indian/Reunion
    {  375, 3157,     554,  5, }, // Pacific/Chuuk
    {  375, 3096,     586,  7, }, // Pacific/Pohnpei
    {  376,   14,       0,  6, }, // Africa/Asmara Kalaallisut/Latin/Greenland
    {  376, 4435,       6,  8, }, // Africa/Sao_Tome
    {  376, 4252,      14,  8, }, // America/Asuncion
    {  376,  237,      22,  8, }, // America/Atikokan
    {  376, 7623,      30, 17, }, // America/Bahia_Banderas
    {  376, 3868,      47,  6, }, // America/Cancun
    {  376, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  376, 5723,      66,  7, }, // America/Curacao
    {  376,  481,      73, 13, }, // America/Indiana/Knox
    {  376, 7961,      86, 16, }, // America/Indiana/Marengo
    {  376, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  376, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  376, 8038,     139, 14, }, // America/Indiana/Vevay
    {  376, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  376, 8086,     171, 16, }, // America/Indiana/Winamac
    {  376, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  376, 5932,     207, 22, }, // America/Lower_Princes
    {  376, 8220,     229,  6, }, // America/Merida
    {  376, 1850,     235, 19, }, // America/Noronha
    {  376, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  376, 8329,     274, 20, }, // America/North_Dakota/Center
    {  376, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  376,  393,     317,  4, }, // America/Nuuk
    {  376, 6899,     321, 16, }, // America/Scoresbysund
    {  376, 8508,     337, 14, }, // America/St_Barthelemy
    {  376, 2061,     351, 10, }, // America/St_Johns
    {  376, 5863,     361,  9, }, // America/St_Kitts
    {  376, 5880,     370,  9, }, // America/St_Lucia
    {  376,  855,     379, 10, }, // America/St_Thomas
    {  376, 5913,     389, 11, }, // America/St_Vincent
    {  376, 5756,   26796,  7, }, // America/Thule
    {  376, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  376, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  376, 1185,     448,  9, }, // Asia/Kathmandu
    {  376,  992,     457,  7, }, // Asia/Kolkata
    {  376, 1211,     464,  5, }, // Asia/Macau
    {  376, 1235,     477,  6, }, // Asia/Yangon
    {  376, 1395,     483,  5, }, // Atlantic/Faroe
    {  376, 6821,     488, 10, }, // Atlantic/St_Helena
    {  376, 2527,     514,  4, }, // Europe/Kyiv
    {  376, 7211,     547,  7, }, // Indian/Reunion
    {  376, 3157,     554,  5, }, // Pacific/Chuuk
    {  376, 3096,     586,  7, }, // Pacific/Pohnpei
    {  377,   14,       0,  6, }, // Africa/Asmara Kalenjin/Latin/Kenya
    {  377, 4435,       6,  8, }, // Africa/Sao_Tome
    {  377, 4252,      14,  8, }, // America/Asuncion
    {  377,  237,      22,  8, }, // America/Atikokan
    {  377, 7623,      30, 17, }, // America/Bahia_Banderas
    {  377, 3868,      47,  6, }, // America/Cancun
    {  377, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  377, 5723,      66,  7, }, // America/Curacao
    {  377,  481,      73, 13, }, // America/Indiana/Knox
    {  377, 7961,      86, 16, }, // America/Indiana/Marengo
    {  377, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  377, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  377, 8038,     139, 14, }, // America/Indiana/Vevay
    {  377, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  377, 8086,     171, 16, }, // America/Indiana/Winamac
    {  377, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  377, 5932,     207, 22, }, // America/Lower_Princes
    {  377, 8220,     229,  6, }, // America/Merida
    {  377, 1850,     235, 19, }, // America/Noronha
    {  377, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  377, 8329,     274, 20, }, // America/North_Dakota/Center
    {  377, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  377,  393,     317,  4, }, // America/Nuuk
    {  377, 6899,     321, 16, }, // America/Scoresbysund
    {  377, 8508,     337, 14, }, // America/St_Barthelemy
    {  377, 2061,     351, 10, }, // America/St_Johns
    {  377, 5863,     361,  9, }, // America/St_Kitts
    {  377, 5880,     370,  9, }, // America/St_Lucia
    {  377,  855,     379, 10, }, // America/St_Thomas
    {  377, 5913,     389, 11, }, // America/St_Vincent
    {  377, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  377, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  377, 1185,     448,  9, }, // Asia/Kathmandu
    {  377,  992,     457,  7, }, // Asia/Kolkata
    {  377, 1211,     464,  5, }, // Asia/Macau
    {  377, 1235,     477,  6, }, // Asia/Yangon
    {  377, 1395,     483,  5, }, // Atlantic/Faroe
    {  377, 6821,     488, 10, }, // Atlantic/St_Helena
    {  377, 2527,     514,  4, }, // Europe/Kyiv
    {  377, 7211,     547,  7, }, // Indian/Reunion
    {  377, 3157,     554,  5, }, // Pacific/Chuuk
    {  377, 3096,     586,  7, }, // Pacific/Pohnpei
    {  378,   14,       0,  6, }, // Africa/Asmara Kamba/Latin/Kenya
    {  378, 4435,       6,  8, }, // Africa/Sao_Tome
    {  378, 4252,      14,  8, }, // America/Asuncion
    {  378,  237,      22,  8, }, // America/Atikokan
    {  378, 7623,      30, 17, }, // America/Bahia_Banderas
    {  378, 3868,      47,  6, }, // America/Cancun
    {  378, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  378, 5723,      66,  7, }, // America/Curacao
    {  378,  481,      73, 13, }, // America/Indiana/Knox
    {  378, 7961,      86, 16, }, // America/Indiana/Marengo
    {  378, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  378, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  378, 8038,     139, 14, }, // America/Indiana/Vevay
    {  378, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  378, 8086,     171, 16, }, // America/Indiana/Winamac
    {  378, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  378, 5932,     207, 22, }, // America/Lower_Princes
    {  378, 8220,     229,  6, }, // America/Merida
    {  378, 1850,     235, 19, }, // America/Noronha
    {  378, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  378, 8329,     274, 20, }, // America/North_Dakota/Center
    {  378, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  378,  393,     317,  4, }, // America/Nuuk
    {  378, 6899,     321, 16, }, // America/Scoresbysund
    {  378, 8508,     337, 14, }, // America/St_Barthelemy
    {  378, 2061,     351, 10, }, // America/St_Johns
    {  378, 5863,     361,  9, }, // America/St_Kitts
    {  378, 5880,     370,  9, }, // America/St_Lucia
    {  378,  855,     379, 10, }, // America/St_Thomas
    {  378, 5913,     389, 11, }, // America/St_Vincent
    {  378, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  378, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  378, 1185,     448,  9, }, // Asia/Kathmandu
    {  378,  992,     457,  7, }, // Asia/Kolkata
    {  378, 1211,     464,  5, }, // Asia/Macau
    {  378, 1235,     477,  6, }, // Asia/Yangon
    {  378, 1395,     483,  5, }, // Atlantic/Faroe
    {  378, 6821,     488, 10, }, // Atlantic/St_Helena
    {  378, 2527,     514,  4, }, // Europe/Kyiv
    {  378, 7211,     547,  7, }, // Indian/Reunion
    {  378, 3157,     554,  5, }, // Pacific/Chuuk
    {  378, 3096,     586,  7, }, // Pacific/Pohnpei
    {  379, 6788,   99459,  9, }, // Africa/Abidjan Kannada/Kannada/India
    {  379, 6760,   99468,  5, }, // Africa/Accra
    {  379, 5059,   99473, 11, }, // Africa/Addis_Ababa
    {  379, 7420,   99484,  9, }, // Africa/Algiers
    {  379,   14,   99493,  7, }, // Africa/Asmara
    {  379,   44,   99500,  5, }, // Africa/Bamako
    {  379, 5257,   99505,  5, }, // Africa/Bangui
    {  379, 6746,   99510,  6, }, // Africa/Banjul
    {  379, 7435,   99516,  8, }, // Africa/Bissau
    {  379, 4957,   99524, 11, }, // Africa/Blantyre
    {  379, 5287,   99535, 10, }, // Africa/Brazzaville
    {  379, 4922,   99545,  9, }, // Africa/Bujumbura
    {  379, 2320,   99554,  4, }, // Africa/Cairo
    {  379, 4117,   99558, 13, }, // Africa/Casablanca
    {  379, 7449,   99571,  6, }, // Africa/Ceuta
    {  379, 6773,   99577,  8, }, // Africa/Conakry
    {  379, 6840,   99585,  5, }, // Africa/Dakar
    {  379, 5130,   99590, 14, }, // Africa/Dar_es_Salaam
    {  379, 5043,   99604,  6, }, // Africa/Djibouti
    {  379, 5243,   99610,  4, }, // Africa/Douala
    {  379, 5166,   99614,  7, }, // Africa/El_Aaiun
    {  379, 6853,   99621,  8, }, // Africa/Freetown
    {  379, 4906,   99629,  9, }, // Africa/Gaborone
    {  379, 5015,   99638,  5, }, // Africa/Harare
    {  379, 4479,   99643, 14, }, // Africa/Johannesburg
    {  379, 4499,   99657,  4, }, // Africa/Juba
    {  379, 5151,   99661,  6, }, // Africa/Kampala
    {  379, 4524,   99667,  7, }, // Africa/Khartoum
    {  379, 4973,   99674,  6, }, // Africa/Kigali
    {  379, 5306,   99680,  8, }, // Africa/Kinshasa
    {  379, 4744,   99688,  6, }, // Africa/Lagos
    {  379, 5336,   99694, 10, }, // Africa/Libreville
    {  379, 6869,   99704,  4, }, // Africa/Lome
    {  379, 5211,   99708,  7, }, // Africa/Luanda
    {  379, 4939,   99715,  9, }, // Africa/Lubumbashi
    {  379, 5001,   99724,  6, }, // Africa/Lusaka
    {  379, 5322,   99730,  5, }, // Africa/Malabo
    {  379, 4987,   99735,  8, }, // Africa/Maputo
    {  379, 5197,   99743,  5, }, // Africa/Maseru
    {  379, 5182,   99748,  7, }, // Africa/Mbabane
    {  379, 5113,   99755,  8, }, // Africa/Mogadishu
    {  379, 7034,   99763, 10, }, // Africa/Monrovia
    {  379, 3853,   99773,  6, }, // Africa/Nairobi
    {  379, 5271,   99779,  7, }, // Africa/Ndjamena
    {  379, 5354,   99786,  6, }, // Africa/Niamey
    {  379, 6803,   99792, 11, }, // Africa/Nouakchott
    {  379, 6727,   99803,  7, }, // Africa/Ouagadougou
    {  379, 5225,   99810, 11, }, // Africa/Porto-Novo
    {  379, 4435,   99821,  9, }, // Africa/Sao_Tome
    {  379, 2866,   99830,  8, }, // Africa/Tripoli
    {  379, 6554,   99838,  6, }, // Africa/Tunis
    {  379, 4152,   99844,  9, }, // Africa/Windhoek
    {  379,  132,   99853,  6, }, // America/Adak
    {  379, 3346,   99859,  7, }, // America/Anchorage
    {  379, 5607,   99866, 10, }, // America/Anguilla
    {  379, 5624,   99876,  8, }, // America/Antigua
    {  379, 4554,   99884,  9, }, // America/Araguaina
    {  379,  166,   99893, 13, }, // America/Argentina/Buenos_Aires
    {  379,   91,   99906,  7, }, // America/Argentina/Catamarca
    {  379,  270,   99913,  8, }, // America/Argentina/Cordoba
    {  379,  441,   99921,  4, }, // America/Argentina/Jujuy
    {  379, 7462,   99925,  9, }, // America/Argentina/La_Rioja
    {  379,  565,   99934,  7, }, // America/Argentina/Mendoza
    {  379, 7489,   99941, 14, }, // America/Argentina/Rio_Gallegos
    {  379, 7520,   99955,  6, }, // America/Argentina/Salta
    {  379, 7544,   99961, 13, }, // America/Argentina/San_Juan
    {  379, 5580,   99974, 13, }, // America/Argentina/San_Luis
    {  379, 7571,   99987,  8, }, // America/Argentina/Tucuman
    {  379, 7597,   99995,  6, }, // America/Argentina/Ushuaia
    {  379, 5640,  100001,  5, }, // America/Aruba
    {  379, 4252,  100006, 11, }, // America/Asuncion
    {  379,  237,  100017,  9, }, // America/Atikokan
    {  379, 3679,  100026,  5, }, // America/Bahia
    {  379, 7623,  100031, 15, }, // America/Bahia_Banderas
    {  379, 5654,  100046,  9, }, // America/Barbados
    {  379, 7646,  100055,  5, }, // America/Belem
    {  379, 5407,  100060,  6, }, // America/Belize
    {  379, 7660,  100066, 15, }, // America/Blanc-Sablon
    {  379, 7681,  100081, 10, }, // America/Boa_Vista
    {  379, 4361,  100091,  5, }, // America/Bogota
    {  379, 7699,  100096,  6, }, // America/Boise
    {  379, 7713,  100102, 16, }, // America/Cambridge_Bay
    {  379, 7735,  100118, 13, }, // America/Campo_Grande
    {  379, 3868,  100131,  8, }, // America/Cancun
    {  379, 4694,  100139,  9, }, // America/Caracas
    {  379, 4345,  100148,  6, }, // America/Cayenne
    {  379, 5496,  100154,  5, }, // America/Cayman
    {  379, 2260,  100159,  6, }, // America/Chicago
    {  379, 7756,  100165,  5, }, // America/Chihuahua
    {  379, 7774,  100170, 15, }, // America/Ciudad_Juarez
    {  379, 5422,  100185, 11, }, // America/Costa_Rica
    {  379, 7796,  100196,  9, }, // America/Creston
    {  379, 3791,  100205,  7, }, // America/Cuiaba
    {  379, 5723,  100212,  8, }, // America/Curacao
    {  379, 7812,  100220, 17, }, // America/Danmarkshavn
    {  379, 7833,  100237,  5, }, // America/Dawson
    {  379, 7848,  100242, 12, }, // America/Dawson_Creek
    {  379,  805,  100254,  7, }, // America/Denver
    {  379, 3465,  100261, 10, }, // America/Detroit
    {  379, 5739,  100271,  8, }, // America/Dominica
    {  379,  893,  100279,  9, }, // America/Edmonton
    {  379, 7869,  100288,  7, }, // America/Eirunepe
    {  379, 5441,  100295, 13, }, // America/El_Salvador
    {  379, 7886,  100308, 14, }, // America/Fort_Nelson
    {  379, 7906,  100322,  9, }, // America/Fortaleza
    {  379, 7924,  100331,  9, }, // America/Glace_Bay
    {  379, 6881,  100340,  7, }, // America/Goose_Bay
    {  379, 4612,  100347, 15, }, // America/Grand_Turk
    {  379, 5770,  100362,  7, }, // America/Grenada
    {  379, 5786,  100369, 10, }, // America/Guadeloupe
    {  379, 3760,  100379, 10, }, // America/Guatemala
    {  379, 4841,  100389, 10, }, // America/Guayaquil
    {  379, 6932,  100399,  5, }, // America/Guyana
    {  379, 1939,  100404, 12, }, // America/Halifax
    {  379, 2281,  100416,  5, }, // America/Havana
    {  379, 7942,  100421, 11, }, // America/Hermosillo
    {  379,  348,  100432, 14, }, // America/Indiana/Indianapolis
    {  379,  481,  100446, 16, }, // America/Indiana/Knox
    {  379, 7961,  100462, 16, }, // America/Indiana/Marengo
    {  379, 7985,  100478, 23, }, // America/Indiana/Petersburg
    {  379, 8012,  100501, 19, }, // America/Indiana/Tell_City
    {  379, 8038,  100520, 16, }, // America/Indiana/Vevay
    {  379, 8060,  100536, 23, }, // America/Indiana/Vincennes
    {  379, 8086,  100559, 20, }, // America/Indiana/Winamac
    {  379, 8110,  100579,  7, }, // America/Inuvik
    {  379,  660,  100586,  9, }, // America/Iqaluit
    {  379, 2799,  100595,  5, }, // America/Jamaica
    {  379, 5380,  100600,  6, }, // America/Juneau
    {  379,  521,  100606, 13, }, // America/Kentucky/Louisville
    {  379, 8125,  100619, 20, }, // America/Kentucky/Monticello
    {  379, 5704,  100639, 13, }, // America/Kralendijk
    {  379, 4376,  100652,  7, }, // America/La_Paz
    {  379, 7169,  100659,  4, }, // America/Lima
    {  379, 3239,  100663, 12, }, // America/Los_Angeles
    {  379, 5932,  100675, 25, }, // America/Lower_Princes
    {  379, 8153,  100700,  6, }, // America/Maceio
    {  379, 8168,  100706,  7, }, // America/Managua
    {  379, 1908,  100713,  7, }, // America/Manaus
    {  379, 5897,  100720,  5, }, // America/Marigot
    {  379, 5805,  100725, 10, }, // America/Martinique
    {  379, 8184,  100735,  8, }, // America/Matamoros
    {  379, 2917,  100743,  8, }, // America/Mazatlan
    {  379, 8202,  100751,  8, }, // America/Menominee
    {  379, 8220,  100759,  6, }, // America/Merida
    {  379, 8235,  100765, 12, }, // America/Metlakatla
    {  379, 2949,  100777, 12, }, // America/Mexico_City
    {  379, 4391,  100789,  8, }, // America/Miquelon
    {  379, 8254,  100797,  8, }, // America/Moncton
    {  379, 8270,  100805,  9, }, // America/Monterrey
    {  379, 4098,  100814, 11, }, // America/Montevideo
    {  379, 5824,  100825, 12, }, // America/Montserrat
    {  379, 5481,  100837,  7, }, // America/Nassau
    {  379, 2379,  100844, 10, }, // America/New_York
    {  379, 8288,  100854,  4, }, // America/Nome
    {  379, 1850,  100858,  8, }, // America/Noronha
    {  379, 8301,  100866, 18, }, // America/North_Dakota/Beulah
    {  379, 8329,  100884, 18, }, // America/North_Dakota/Center
    {  379, 8357,  100902, 21, }, // America/North_Dakota/New_Salem
    {  379,  393,  100923,  4, }, // America/Nuuk
    {  379, 8388,  100927,  7, }, // America/Ojinaga
    {  379, 2356,  100934,  5, }, // America/Panama
    {  379, 6173,  100939,  8, }, // America/Paramaribo
    {  379, 2973,  100947,  8, }, // America/Phoenix
    {  379, 3945,  100955, 17, }, // America/Port-au-Prince
    {  379, 5954,  100972, 17, }, // America/Port_of_Spain
    {  379, 8404,  100989, 13, }, // America/Porto_Velho
    {  379, 5843,  101002, 13, }, // America/Puerto_Rico
    {  379, 4030,  101015, 13, }, // America/Punta_Arenas
    {  379, 8424,  101028, 16, }, // America/Rankin_Inlet
    {  379, 8445,  101044,  6, }, // America/Recife
    {  379, 1995,  101050,  6, }, // America/Regina
    {  379, 8460,  101056, 10, }, // America/Resolute
    {  379,  695,  101066, 14, }, // America/Rio_Branco
    {  379, 8477,  101080,  8, }, // America/Santarem
    {  379, 2201,  101088, 11, }, // America/Santiago
    {  379, 6111,  101099, 15, }, // America/Santo_Domingo
    {  379, 1878,  101114,  9, }, // America/Sao_Paulo
    {  379, 6899,  101123, 19, }, // America/Scoresbysund
    {  379, 8494,  101142,  6, }, // America/Sitka
    {  379, 8508,  101148, 16, }, // America/St_Barthelemy
    {  379, 2061,  101164, 12, }, // America/St_Johns
    {  379, 5863,  101176, 12, }, // America/St_Kitts
    {  379, 5880,  101188, 12, }, // America/St_Lucia
    {  379,  855,  101200, 11, }, // America/St_Thomas
    {  379, 5913,  101211, 15, }, // America/St_Vincent
    {  379, 8530,  101226, 15, }, // America/Swift_Current
    {  379, 5461,  101241, 11, }, // America/Tegucigalpa
    {  379, 5756,  101252,  4, }, // America/Thule
    {  379,  313,  101256,  8, }, // America/Tijuana
    {  379,  608,  101264,  7, }, // America/Toronto
    {  379, 5688,  101271,  8, }, // America/Tortola
    {  379, 2093,  101279, 11, }, // America/Vancouver
    {  379, 2144,  101290, 11, }, // America/Whitehorse
    {  379,  734,  101301, 10, }, // America/Winnipeg
    {  379, 8552,  101311,  8, }, // America/Yakutat
    {  379, 6001,  101319,  4, }, // Antarctica/Casey
    {  379, 6094,  101323,  6, }, // Antarctica/Davis
    {  379, 6133,  101329, 17, }, // Antarctica/DumontDUrville
    {  379, 8568,  101346, 10, }, // Antarctica/Macquarie
    {  379, 7066,  101356,  7, }, // Antarctica/Mawson
    {  379, 7113,  101363, 13, }, // Antarctica/McMurdo
    {  379, 8589,  101376,  7, }, // Antarctica/Palmer
    {  379, 7226,  101383,  6, }, // Antarctica/Rothera
    {  379, 7295,  101389,  6, }, // Antarctica/Syowa
    {  379, 8607,  101395,  6, }, // Antarctica/Troll
    {  379, 7374,  101401,  8, }, // Antarctica/Vostok
    {  379, 1429,  101409, 13, }, // Arctic/Longyearbyen
    {  379, 5570,  101422,  4, }, // Asia/Aden
    {  379, 5395,  101426,  7, }, // Asia/Almaty
    {  379, 3968,  101433,  7, }, // Asia/Amman
    {  379, 5511,  101440,  7, }, // Asia/Anadyr
    {  379, 5523,  101447,  7, }, // Asia/Aqtau
    {  379, 5368,  101454,  7, }, // Asia/Aqtobe
    {  379,  964,  101461,  8, }, // Asia/Ashgabat
    {  379, 8624,  101469,  7, }, // Asia/Atyrau
    {  379, 3607,  101476,  8, }, // Asia/Baghdad
    {  379, 5534,  101484,  7, }, // Asia/Bahrain
    {  379, 3653,  101491,  3, }, // Asia/Baku
    {  379, 4466,  101494,  9, }, // Asia/Bangkok
    {  379, 3571,  101503,  8, }, // Asia/Barnaul
    {  379, 4086,  101511,  5, }, // Asia/Beirut
    {  379, 3778,  101516,  8, }, // Asia/Bishkek
    {  379, 5989,  101524,  6, }, // Asia/Brunei
    {  379, 4601,  101530,  3, }, // Asia/Chita
    {  379, 4511,  101533,  6, }, // Asia/Colombo
    {  379, 4540,  101539,  8, }, // Asia/Damascus
    {  379, 1093,  101547,  4, }, // Asia/Dhaka
    {  379, 6192,  101551,  4, }, // Asia/Dili
    {  379, 3596,  101555,  4, }, // Asia/Dubai
    {  379, 6159,  101559,  7, }, // Asia/Dushanbe
    {  379, 8636,  101566,  9, }, // Asia/Famagusta
    {  379, 8651,  101575,  4, }, // Asia/Gaza
    {  379, 4795,  101579,  8, }, // Asia/Hebron
    {  379, 1259,  101587, 15, }, // Asia/Ho_Chi_Minh
    {  379, 2704,  101602, 11, }, // Asia/Hong_Kong
    {  379, 4771,  101613,  6, }, // Asia/Hovd
    {  379, 4184,  101619, 10, }, // Asia/Irkutsk
    {  379, 7006,  101629,  7, }, // Asia/Jakarta
    {  379, 6992,  101636,  5, }, // Asia/Jayapura
    {  379, 1290,  101641,  9, }, // Asia/Jerusalem
    {  379, 3560,  101650,  6, }, // Asia/Kabul
    {  379, 4316,  101656,  7, }, // Asia/Kamchatka
    {  379, 4239,  101663,  5, }, // Asia/Karachi
    {  379, 1185,  101668,  7, }, // Asia/Kathmandu
    {  379, 8661,  101675,  8, }, // Asia/Khandyga
    {  379,  992,  101683,  9, }, // Asia/Kolkata
    {  379, 4197,  101692, 16, }, // Asia/Krasnoyarsk
    {  379, 4859,  101708, 10, }, // Asia/Kuala_Lumpur
    {  379, 5976,  101718,  7, }, // Asia/Kuching
    {  379, 5547,  101725,  6, }, // Asia/Kuwait
    {  379, 1211,  101731,  5, }, // Asia/Macau
    {  379, 4017,  101736,  6, }, // Asia/Magadan
    {  379, 1349,  101742,  9, }, // Asia/Makassar
    {  379, 7182,  101751,  5, }, // Asia/Manila
    {  379, 6920,  101756,  6, }, // Asia/Muscat
    {  379, 2554,  101762,  8, }, // Asia/Nicosia
    {  379, 8675,  101770,  8, }, // Asia/Novokuznetsk
    {  379, 4135,  101778, 12, }, // Asia/Novosibirsk
    {  379, 4229,  101790,  7, }, // Asia/Omsk
    {  379, 7145,  101797,  4, }, // Asia/Oral
    {  379, 6961,  101801,  9, }, // Asia/Phnom_Penh
    {  379, 8693,  101810, 10, }, // Asia/Pontianak
    {  379, 4214,  101820, 13, }, // Asia/Pyongyang
    {  379, 5559,  101833,  5, }, // Asia/Qatar
    {  379, 8708,  101838,  8, }, // Asia/Qostanay
    {  379, 4269,  101846,  9, }, // Asia/Qyzylorda
    {  379, 3584,  101855,  6, }, // Asia/Riyadh
    {  379, 4408,  101861,  7, }, // Asia/Sakhalin
    {  379, 7245,  101868,  6, }, // Asia/Samarkand
    {  379, 3279,  101874,  6, }, // Asia/Seoul
    {  379, 1053,  101880,  5, }, // Asia/Shanghai
    {  379, 3300,  101885,  8, }, // Asia/Singapore
    {  379, 4297,  101893, 16, }, // Asia/Srednekolymsk
    {  379, 3263,  101909,  4, }, // Asia/Taipei
    {  379, 4781,  101913,  9, }, // Asia/Tashkent
    {  379, 3915,  101922,  8, }, // Asia/Tbilisi
    {  379, 2772,  101930,  8, }, // Asia/Tehran
    {  379, 1317,  101938,  5, }, // Asia/Thimphu
    {  379, 2821,  101943,  6, }, // Asia/Tokyo
    {  379, 4572,  101949,  9, }, // Asia/Tomsk
    {  379, 1021,  101958, 11, }, // Asia/Ulaanbaatar
    {  379, 1159,  101969,  8, }, // Asia/Urumqi
    {  379, 8722,  101977, 10, }, // Asia/Ust-Nera
    {  379, 6977,  101987, 11, }, // Asia/Vientiane
    {  379, 4710,  101998, 15, }, // Asia/Vladivostok
    {  379, 4828,  102013,  9, }, // Asia/Yakutsk
    {  379, 1235,  102022,  7, }, // Asia/Yangon
    {  379, 3883,  102029, 16, }, // Asia/Yekaterinburg
    {  379, 3747,  102045,  8, }, // Asia/Yerevan
    {  379, 3663,  102053,  7, }, // Atlantic/Azores
    {  379, 5671,  102060,  7, }, // Atlantic/Bermuda
    {  379, 6628,  102067,  7, }, // Atlantic/Canary
    {  379, 3727,  102074, 11, }, // Atlantic/Cape_Verde
    {  379, 1395,  102085,  6, }, // Atlantic/Faroe
    {  379, 8736,  102091,  6, }, // Atlantic/Madeira
    {  379, 2748,  102097, 10, }, // Atlantic/Reykjavik
    {  379, 7272,  102107, 15, }, // Atlantic/South_Georgia
    {  379, 6821,  102122, 12, }, // Atlantic/St_Helena
    {  379, 6644,  102134, 10, }, // Atlantic/Stanley
    {  379, 1670,  102144,  7, }, // Australia/Adelaide
    {  379, 1635,  102151, 11, }, // Australia/Brisbane
    {  379, 1799,  102162, 12, }, // Australia/Broken_Hill
    {  379, 1583,  102174,  8, }, // Australia/Darwin
    {  379, 3637,  102182,  6, }, // Australia/Eucla
    {  379, 1516,  102188,  9, }, // Australia/Hobart
    {  379, 8753,  102197,  9, }, // Australia/Lindeman
    {  379, 1547,  102206, 11, }, // Australia/Lord_Howe
    {  379, 1727,  102217,  9, }, // Australia/Melbourne
    {  379, 1762,  102226,  5, }, // Australia/Perth
    {  379, 1463,  102231,  6, }, // Australia/Sydney
    {  379, 6441,  102237, 16, }, // Europe/Amsterdam
    {  379, 6216,  102253,  6, }, // Europe/Andorra
    {  379, 3620,  102259, 11, }, // Europe/Astrakhan
    {  379, 2300,  102270,  7, }, // Europe/Athens
    {  379, 6336,  102277, 11, }, // Europe/Belgrade
    {  379, 4757,  102288,  7, }, // Europe/Berlin
    {  379, 6488,  102295, 11, }, // Europe/Bratislava
    {  379, 2167,  102306, 11, }, // Europe/Brussels
    {  379, 3928,  102317, 10, }, // Europe/Bucharest
    {  379, 3806,  102327, 10, }, // Europe/Budapest
    {  379, 8772,  102337,  9, }, // Europe/Busingen
    {  379, 2583,  102346,  8, }, // Europe/Chisinau
    {  379, 6289,  102354, 12, }, // Europe/Copenhagen
    {  379, 2338,  102366,  8, }, // Europe/Dublin
    {  379, 6307,  102374, 11, }, // Europe/Gibraltar
    {  379, 8788,  102385, 11, }, // Europe/Guernsey
    {  379, 6612,  102396,  9, }, // Europe/Helsinki
    {  379, 8804,  102405, 14, }, // Europe/Isle_of_Man
    {  379, 1130,  102419, 12, }, // Europe/Istanbul
    {  379, 8823,  102431,  6, }, // Europe/Jersey
    {  379, 3979,  102437, 14, }, // Europe/Kaliningrad
    {  379, 8929,  102451,  6, }, // Europe/Kirov
    {  379, 2527,  102457,  4, }, // Europe/Kyiv
    {  379, 3213,  102461,  7, }, // Europe/Lisbon
    {  379, 6506,  102468, 12, }, // Europe/Ljubljana
    {  379, 2501,  102480,  5, }, // Europe/London
    {  379, 6365,  102485, 11, }, // Europe/Luxembourg
    {  379, 4892,  102496, 10, }, // Europe/Madrid
    {  379, 6397,  102506,  6, }, // Europe/Malta
    {  379, 6582,  102512,  8, }, // Europe/Mariehamn
    {  379, 3693,  102520,  9, }, // Europe/Minsk
    {  379, 6410,  102529,  6, }, // Europe/Monaco
    {  379, 3537,  102535,  6, }, // Europe/Moscow
    {  379, 6458,  102541,  5, }, // Europe/Oslo
    {  379, 4284,  102546,  8, }, // Europe/Paris
    {  379, 6424,  102554,  9, }, // Europe/Podgorica
    {  379, 6275,  102563,  8, }, // Europe/Prague
    {  379, 8837,  102571,  4, }, // Europe/Riga
    {  379, 6324,  102575,  4, }, // Europe/Rome
    {  379, 4331,  101868,  3, }, // Europe/Samara
    {  379, 6470,  102579, 12, }, // Europe/San_Marino
    {  379, 6245,  102591,  6, }, // Europe/Sarajevo
    {  379, 4451,  102597,  7, }, // Europe/Saratov
    {  379, 8849,  102604, 12, }, // Europe/Simferopol
    {  379, 6383,  102616,  8, }, // Europe/Skopje
    {  379, 6599,  102624,  6, }, // Europe/Sofia
    {  379, 6523,  102630, 11, }, // Europe/Stockholm
    {  379, 8867,  102641,  8, }, // Europe/Tallinn
    {  379, 6202,  102649,  6, }, // Europe/Tirane
    {  379, 8882,  102655, 12, }, // Europe/Ulyanovsk
    {  379, 6352,  102667,  5, }, // Europe/Vaduz
    {  379, 6567,  102672,  9, }, // Europe/Vatican
    {  379, 6231,  102681,  8, }, // Europe/Vienna
    {  379, 8899,  102689,  9, }, // Europe/Vilnius
    {  379, 4727,  102698, 12, }, // Europe/Volgograd
    {  379, 3190,  102710,  6, }, // Europe/Warsaw
    {  379, 6261,  102716,  9, }, // Europe/Zagreb
    {  379, 6540,  102725,  8, }, // Europe/Zurich
    {  379, 5078,  102733, 11, }, // Indian/Antananarivo
    {  379, 6947,  102744,  6, }, // Indian/Chagos
    {  379, 6046,  102750,  9, }, // Indian/Christmas
    {  379, 6063,  102759,  6, }, // Indian/Cocos
    {  379, 5029,  102765,  6, }, // Indian/Comoro
    {  379, 6661,  102771, 12, }, // Indian/Kerguelen
    {  379, 7260,  102783,  4, }, // Indian/Mahe
    {  379, 7050,  102787, 10, }, // Indian/Maldives
    {  379, 4069,  102797,  7, }, // Indian/Mauritius
    {  379, 5098,  102804,  7, }, // Indian/Mayotte
    {  379, 7211,  102811,  9, }, // Indian/Reunion
    {  379, 4422,  102820,  5, }, // Pacific/Apia
    {  379,  932,  102825, 11, }, // Pacific/Auckland
    {  379, 3706,  102836, 13, }, // Pacific/Bougainville
    {  379, 3015,  102849,  5, }, // Pacific/Chatham
    {  379, 3157,  102854,  4, }, // Pacific/Chuuk
    {  379, 2237,  102858,  6, }, // Pacific/Easter
    {  379, 7360,  102864,  5, }, // Pacific/Efate
    {  379, 7327,  102869,  5, }, // Pacific/Fakaofo
    {  379, 3902,  102874,  4, }, // Pacific/Fiji
    {  379, 7343,  102878,  8, }, // Pacific/Funafuti
    {  379, 6678,  102886,  8, }, // Pacific/Galapagos
    {  379, 6696,  102894, 10, }, // Pacific/Gambier
    {  379, 3822,  102904, 13, }, // Pacific/Guadalcanal
    {  379, 6018,  102917,  6, }, // Pacific/Guam
    {  379, 2723,  102923,  7, }, // Pacific/Honolulu
    {  379, 3049,  102930,  8, }, // Pacific/Kanton
    {  379, 3998,  102938,  9, }, // Pacific/Kiritimati
    {  379, 7019,  102947,  6, }, // Pacific/Kosrae
    {  379, 2842,  102953,  9, }, // Pacific/Kwajalein
    {  379, 4877,  102962,  5, }, // Pacific/Majuro
    {  379, 4051,  102967, 11, }, // Pacific/Marquesas
    {  379, 8914,  102978,  7, }, // Pacific/Midway
    {  379, 7084,  102985,  4, }, // Pacific/Nauru
    {  379, 7132,  102989,  4, }, // Pacific/Niue
    {  379, 4168,  102993,  8, }, // Pacific/Norfolk
    {  379, 7098,  103001,  8, }, // Pacific/Noumea
    {  379, 3126,  103009,  9, }, // Pacific/Pago_Pago
    {  379, 7155,  103018,  6, }, // Pacific/Palau
    {  379, 7194,  103024, 11, }, // Pacific/Pitcairn
    {  379, 3096,  103035,  7, }, // Pacific/Pohnpei
    {  379, 4807,  103042, 16, }, // Pacific/Port_Moresby
    {  379, 6076,  103058,  8, }, // Pacific/Rarotonga
    {  379, 6031,  103066,  5, }, // Pacific/Saipan
    {  379, 7312,  103071,  5, }, // Pacific/Tahiti
    {  379, 6712,  103076,  5, }, // Pacific/Tarawa
    {  379, 4583,  103081,  9, }, // Pacific/Tongatapu
    {  379, 7392,  103090,  4, }, // Pacific/Wake
    {  379, 7405,  103094,  8, }, // Pacific/Wallis
    {  380, 6788,  103102,  8, }, // Africa/Abidjan Kashmiri/Arabic/India
    {  380, 6760,  103110,  5, }, // Africa/Accra
    {  380, 5059,  103115, 10, }, // Africa/Addis_Ababa
    {  380, 7420,  103125, 10, }, // Africa/Algiers
    {  380,   14,  103135,  7, }, // Africa/Asmara
    {  380,   44,  103142,  5, }, // Africa/Bamako
    {  380, 5257,  103147,  8, }, // Africa/Bangui
    {  380, 6746,  103155,  7, }, // Africa/Banjul
    {  380, 7435,  103162,  6, }, // Africa/Bissau
    {  380, 4957,  103168,  8, }, // Africa/Blantyre
    {  380, 5287,  103176,  8, }, // Africa/Brazzaville
    {  380, 4922,  103184,  9, }, // Africa/Bujumbura
    {  380, 2320,  103193,  5, }, // Africa/Cairo
    {  380, 4117,  103198, 10, }, // Africa/Casablanca
    {  380, 7449,  103208,  6, }, // Africa/Ceuta
    {  380, 6773,  103214,  7, }, // Africa/Conakry
    {  380, 6840,  103221,  5, }, // Africa/Dakar
    {  380, 5130,  103226,  9, }, // Africa/Dar_es_Salaam
    {  380, 5043,  103235,  8, }, // Africa/Djibouti
    {  380, 5243,  103243,  5, }, // Africa/Douala
    {  380, 5166,  103248,  7, }, // Africa/El_Aaiun
    {  380, 6853,  103255,  9, }, // Africa/Freetown
    {  380, 4906,  103264,  7, }, // Africa/Gaborone
    {  380, 5015,  103271,  7, }, // Africa/Harare
    {  380, 4479,  103278,  9, }, // Africa/Johannesburg
    {  380, 4499,   10673,  4, }, // Africa/Juba
    {  380, 5151,  103287,  7, }, // Africa/Kampala
    {  380, 4524,  103294,  7, }, // Africa/Khartoum
    {  380, 4973,  103301,  6, }, // Africa/Kigali
    {  380, 5306,  103307,  7, }, // Africa/Kinshasa
    {  380, 4744,  103314,  5, }, // Africa/Lagos
    {  380, 5336,  103319,  8, }, // Africa/Libreville
    {  380, 6869,   10716,  3, }, // Africa/Lome
    {  380, 5211,  103327,  7, }, // Africa/Luanda
    {  380, 4939,  103334,  9, }, // Africa/Lubumbashi
    {  380, 5001,  103343,  6, }, // Africa/Lusaka
    {  380, 5322,   10740,  6, }, // Africa/Malabo
    {  380, 4987,  103349,  7, }, // Africa/Maputo
    {  380, 5197,  103356,  7, }, // Africa/Maseru
    {  380, 5182,  103363,  5, }, // Africa/Mbabane
    {  380, 5113,  103368,  9, }, // Africa/Mogadishu
    {  380, 7034,  103377,  9, }, // Africa/Monrovia
    {  380, 3853,  103386,  6, }, // Africa/Nairobi
    {  380, 5271,  103392,  7, }, // Africa/Ndjamena
    {  380, 5354,  103399,  6, }, // Africa/Niamey
    {  380, 6803,  103405,  9, }, // Africa/Nouakchott
    {  380, 6727,  103414,  8, }, // Africa/Ouagadougou
    {  380, 5225,  103422, 10, }, // Africa/Porto-Novo
    {  380, 4435,  103432,  7, }, // Africa/Sao_Tome
    {  380, 2866,  103439,  6, }, // Africa/Tripoli
    {  380, 6554,  103445,  6, }, // Africa/Tunis
    {  380, 4152,  103451,  7, }, // Africa/Windhoek
    {  380,  132,  103458,  3, }, // America/Adak
    {  380, 3346,  103461,  8, }, // America/Anchorage
    {  380, 5607,  103469,  8, }, // America/Anguilla
    {  380, 5624,  103477,  9, }, // America/Antigua
    {  380, 4554,  103486, 10, }, // America/Araguaina
    {  380,  166,  103496, 13, }, // America/Argentina/Buenos_Aires
    {  380,   91,  103509,  9, }, // America/Argentina/Catamarca
    {  380,  270,  103518,  7, }, // America/Argentina/Cordoba
    {  380,  441,  103525,  6, }, // America/Argentina/Jujuy
    {  380, 7462,  103531,  9, }, // America/Argentina/La_Rioja
    {  380,  565,  103540,  7, }, // America/Argentina/Mendoza
    {  380, 7489,  103547, 12, }, // America/Argentina/Rio_Gallegos
    {  380, 7520,  103559,  5, }, // America/Argentina/Salta
    {  380, 7544,  103564,  9, }, // America/Argentina/San_Juan
    {  380, 5580,  103573, 10, }, // America/Argentina/San_Luis
    {  380, 7571,  103583,  8, }, // America/Argentina/Tucuman
    {  380, 7597,  103591, 10, }, // America/Argentina/Ushuaia
    {  380, 5640,  103601,  7, }, // America/Aruba
    {  380, 4252,  103608, 10, }, // America/Asuncion
    {  380,  237,  103618,  9, }, // America/Atikokan
    {  380, 3679,  103627,  6, }, // America/Bahia
    {  380, 7623,  103633, 12, }, // America/Bahia_Banderas
    {  380, 5654,  103645,  8, }, // America/Barbados
    {  380, 7646,  103653,  5, }, // America/Belem
    {  380, 5407,  103658,  6, }, // America/Belize
    {  380, 7660,  103664, 12, }, // America/Blanc-Sablon
    {  380, 7681,  103676,  9, }, // America/Boa_Vista
    {  380, 4361,  103685,  6, }, // America/Bogota
    {  380, 7699,  103691,  5, }, // America/Boise
    {  380, 7713,  103696, 13, }, // America/Cambridge_Bay
    {  380, 7735,  103709, 11, }, // America/Campo_Grande
    {  380, 3868,  103720,  6, }, // America/Cancun
    {  380, 4694,  103726,  7, }, // America/Caracas
    {  380, 4345,  103733,  5, }, // America/Cayenne
    {  380, 5496,  103738,  5, }, // America/Cayman
    {  380, 2260,  103743,  6, }, // America/Chicago
    {  380, 7756,  103749, 11, }, // America/Chihuahua
    {  380, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  380, 5422,  103760, 11, }, // America/Costa_Rica
    {  380, 7796,  103771,  6, }, // America/Creston
    {  380, 3791,  103777,  7, }, // America/Cuiaba
    {  380, 5723,  103784, 10, }, // America/Curacao
    {  380, 7812,  103794, 11, }, // America/Danmarkshavn
    {  380, 7833,  103805,  5, }, // America/Dawson
    {  380, 7848,  103810, 12, }, // America/Dawson_Creek
    {  380,  805,  103822,  6, }, // America/Denver
    {  380, 3465,  103828,  8, }, // America/Detroit
    {  380, 5739,  103836,  8, }, // America/Dominica
    {  380,  893,  103844,  9, }, // America/Edmonton
    {  380, 7869,  103853,  9, }, // America/Eirunepe
    {  380, 5441,  103862, 12, }, // America/El_Salvador
    {  380, 7886,  103874, 10, }, // America/Fort_Nelson
    {  380, 7906,  103884,  9, }, // America/Fortaleza
    {  380, 7924,  103893, 10, }, // America/Glace_Bay
    {  380, 6881,  103903, 10, }, // America/Goose_Bay
    {  380, 4612,  103913, 10, }, // America/Grand_Turk
    {  380, 5770,  103923,  7, }, // America/Grenada
    {  380, 5786,  103930,  6, }, // America/Guadeloupe
    {  380, 3760,  103936,  9, }, // America/Guatemala
    {  380, 4841,  103945, 10, }, // America/Guayaquil
    {  380, 6932,  103955,  6, }, // America/Guyana
    {  380, 1939,  103961,  8, }, // America/Halifax
    {  380, 2281,  103969,  5, }, // America/Havana
    {  380, 7942,  103974,  8, }, // America/Hermosillo
    {  380,  348,  103982, 13, }, // America/Indiana/Indianapolis
    {  380,  481,  103995,  4, }, // America/Indiana/Knox
    {  380, 7961,  103999, 15, }, // America/Indiana/Marengo
    {  380, 7985,  104014, 18, }, // America/Indiana/Petersburg
    {  380, 8012,  104032, 17, }, // America/Indiana/Tell_City
    {  380, 8038,  104049, 14, }, // America/Indiana/Vevay
    {  380, 8060,  104063, 17, }, // America/Indiana/Vincennes
    {  380, 8086,  104080, 16, }, // America/Indiana/Winamac
    {  380, 8110,  104096,  8, }, // America/Inuvik
    {  380,  660,  104104, 11, }, // America/Iqaluit
    {  380, 2799,  104115,  7, }, // America/Jamaica
    {  380, 5380,  104122,  5, }, // America/Juneau
    {  380,  521,  104127, 10, }, // America/Kentucky/Louisville
    {  380, 8125,  104137, 18, }, // America/Kentucky/Monticello
    {  380, 5704,  104155,  9, }, // America/Kralendijk
    {  380, 4376,  104164,  6, }, // America/La_Paz
    {  380, 7169,  104170,  4, }, // America/Lima
    {  380, 3239,  104174, 11, }, // America/Los_Angeles
    {  380, 5932,  104185, 16, }, // America/Lower_Princes
    {  380, 8153,  104201,  7, }, // America/Maceio
    {  380, 8168,  104208,  7, }, // America/Managua
    {  380, 1908,  104215,  5, }, // America/Manaus
    {  380, 5897,  104220,  7, }, // America/Marigot
    {  380, 5805,  104227,  8, }, // America/Martinique
    {  380, 8184,  104235,  9, }, // America/Matamoros
    {  380, 2917,  104244,  8, }, // America/Mazatlan
    {  380, 8202,  104252,  8, }, // America/Menominee
    {  380, 8220,  104260,  6, }, // America/Merida
    {  380, 8235,  104266, 11, }, // America/Metlakatla
    {  380, 2949,  104277, 12, }, // America/Mexico_City
    {  380, 4391,  104289,  8, }, // America/Miquelon
    {  380, 8254,  104297,  7, }, // America/Moncton
    {  380, 8270,  104304,  6, }, // America/Monterrey
    {  380, 4098,  104310, 10, }, // America/Montevideo
    {  380, 5824,  104320,  8, }, // America/Montserrat
    {  380, 5481,  104328,  5, }, // America/Nassau
    {  380, 2379,  104333,  8, }, // America/New_York
    {  380, 8288,   11540,  3, }, // America/Nome
    {  380, 1850,  104341,  7, }, // America/Noronha
    {  380, 8301,  104348, 21, }, // America/North_Dakota/Beulah
    {  380, 8329,  104369, 19, }, // America/North_Dakota/Center
    {  380, 8357,  104388, 24, }, // America/North_Dakota/New_Salem
    {  380,  393,  104412,  4, }, // America/Nuuk
    {  380, 8388,  104416,  9, }, // America/Ojinaga
    {  380, 2356,  104425,  6, }, // America/Panama
    {  380, 6173,  104431, 10, }, // America/Paramaribo
    {  380, 2973,  104441,  7, }, // America/Phoenix
    {  380, 3945,  104448, 13, }, // America/Port-au-Prince
    {  380, 5954,  104461, 11, }, // America/Port_of_Spain
    {  380, 8404,  104472, 10, }, // America/Porto_Velho
    {  380, 5843,  104482, 10, }, // America/Puerto_Rico
    {  380, 4030,  104492, 12, }, // America/Punta_Arenas
    {  380, 8424,  104504, 13, }, // America/Rankin_Inlet
    {  380, 8445,  104517,  5, }, // America/Recife
    {  380, 1995,  104522,  7, }, // America/Regina
    {  380, 8460,  104529,  9, }, // America/Resolute
    {  380,  695,  104538, 11, }, // America/Rio_Branco
    {  380, 8477,  104549,  8, }, // America/Santarem
    {  380, 2201,  104557,  8, }, // America/Santiago
    {  380, 6111,  104565, 13, }, // America/Santo_Domingo
    {  380, 1878,  104578,  8, }, // America/Sao_Paulo
    {  380, 6899,  104586, 13, }, // America/Scoresbysund
    {  380, 8494,  104599,  5, }, // America/Sitka
    {  380, 8508,  104604, 14, }, // America/St_Barthelemy
    {  380, 2061,  104618,  9, }, // America/St_Johns
    {  380, 5863,  104627,  9, }, // America/St_Kitts
    {  380, 5880,  104636, 12, }, // America/St_Lucia
    {  380,  855,  104648, 11, }, // America/St_Thomas
    {  380, 5913,  104659, 12, }, // America/St_Vincent
    {  380, 8530,  104671, 13, }, // America/Swift_Current
    {  380, 5461,  104684, 12, }, // America/Tegucigalpa
    {  380, 5756,  104696,  7, }, // America/Thule
    {  380,  313,  104703,  8, }, // America/Tijuana
    {  380,  608,  104711,  7, }, // America/Toronto
    {  380, 5688,  104718,  7, }, // America/Tortola
    {  380, 2093,  104725,  8, }, // America/Vancouver
    {  380, 2144,  104733, 11, }, // America/Whitehorse
    {  380,  734,  104744,  7, }, // America/Winnipeg
    {  380, 8552,  104751,  7, }, // America/Yakutat
    {  380, 6001,  104758,  4, }, // Antarctica/Casey
    {  380, 6094,  104762,  5, }, // Antarctica/Davis
    {  380, 6133,  104767, 15, }, // Antarctica/DumontDUrville
    {  380, 7066,  104782,  5, }, // Antarctica/Mawson
    {  380, 7113,  104787,  8, }, // Antarctica/McMurdo
    {  380, 8589,  104795,  5, }, // Antarctica/Palmer
    {  380, 7226,  104800,  7, }, // Antarctica/Rothera
    {  380, 7295,  104807,  5, }, // Antarctica/Syowa
    {  380, 7374,  104812,  6, }, // Antarctica/Vostok
    {  380, 1429,  104818, 14, }, // Arctic/Longyearbyen
    {  380, 5570,  104832,  5, }, // Asia/Aden
    {  380, 5395,  104837,  7, }, // Asia/Almaty
    {  380, 3968,  104844,  5, }, // Asia/Amman
    {  380, 5511,  104849,  7, }, // Asia/Anadyr
    {  380, 5523,  104856,  4, }, // Asia/Aqtau
    {  380, 5368,  104860,  6, }, // Asia/Aqtobe
    {  380,  964,  104866,  8, }, // Asia/Ashgabat
    {  380, 8624,  104874,  5, }, // Asia/Atyrau
    {  380, 3607,   12109,  5, }, // Asia/Baghdad
    {  380, 5534,  104879,  7, }, // Asia/Bahrain
    {  380, 3653,  104886,  5, }, // Asia/Baku
    {  380, 4466,  104891,  8, }, // Asia/Bangkok
    {  380, 3571,  104899,  5, }, // Asia/Barnaul
    {  380, 4086,  104904,  4, }, // Asia/Beirut
    {  380, 3778,  104908,  6, }, // Asia/Bishkek
    {  380, 5989,  104914,  7, }, // Asia/Brunei
    {  380, 4601,  104921,  4, }, // Asia/Chita
    {  380, 4511,  104925,  7, }, // Asia/Colombo
    {  380, 4540,  104932,  8, }, // Asia/Damascus
    {  380, 1093,  104940,  5, }, // Asia/Dhaka
    {  380, 6192,  104945,  4, }, // Asia/Dili
    {  380, 3596,  104949,  5, }, // Asia/Dubai
    {  380, 6159,  104954,  8, }, // Asia/Dushanbe
    {  380, 8636,  104962, 10, }, // Asia/Famagusta
    {  380, 8651,  104972,  3, }, // Asia/Gaza
    {  380, 4795,  104975,  6, }, // Asia/Hebron
    {  380, 1259,  104981,  5, }, // Asia/Ho_Chi_Minh
    {  380, 2704,  104986,  9, }, // Asia/Hong_Kong
    {  380, 4771,  104995,  4, }, // Asia/Hovd
    {  380, 4184,  104999,  7, }, // Asia/Irkutsk
    {  380, 7006,  105006,  6, }, // Asia/Jakarta
    {  380, 6992,  105012,  9, }, // Asia/Jayapura
    {  380, 1290,  105021,  6, }, // Asia/Jerusalem
    {  380, 3560,  105027,  5, }, // Asia/Kabul
    {  380, 4316,  105032,  8, }, // Asia/Kamchatka
    {  380, 4239,  105040,  6, }, // Asia/Karachi
    {  380, 1185,  105046, 10, }, // Asia/Kathmandu
    {  380, 8661,  105056,  8, }, // Asia/Khandyga
    {  380,  992,  105064,  8, }, // Asia/Kolkata
    {  380, 4197,  105072, 11, }, // Asia/Krasnoyarsk
    {  380, 4859,  105083, 11, }, // Asia/Kuala_Lumpur
    {  380, 5976,  105094,  6, }, // Asia/Kuching
    {  380, 5547,  105100,  5, }, // Asia/Kuwait
    {  380, 1211,  105105,  3, }, // Asia/Macau
    {  380, 4017,  105108,  7, }, // Asia/Magadan
    {  380, 1349,  105115,  7, }, // Asia/Makassar
    {  380, 7182,  105122,  7, }, // Asia/Manila
    {  380, 6920,   12368,  4, }, // Asia/Muscat
    {  380, 2554,  105129,  8, }, // Asia/Nicosia
    {  380, 8675,  105137, 12, }, // Asia/Novokuznetsk
    {  380, 4135,  105149, 11, }, // Asia/Novosibirsk
    {  380, 4229,  105160,  5, }, // Asia/Omsk
    {  380, 7145,  105165,  5, }, // Asia/Oral
    {  380, 6961,  105170,  9, }, // Asia/Phnom_Penh
    {  380, 8693,  105179,  9, }, // Asia/Pontianak
    {  380, 4214,  105188,  9, }, // Asia/Pyongyang
    {  380, 5559,   12436,  3, }, // Asia/Qatar
    {  380, 8708,  105197,  7, }, // Asia/Qostanay
    {  380, 4269,  105204,  8, }, // Asia/Qyzylorda
    {  380, 3584,  105212,  4, }, // Asia/Riyadh
    {  380, 4408,  105216,  8, }, // Asia/Sakhalin
    {  380, 7245,  105224,  9, }, // Asia/Samarkand
    {  380, 3279,  105233,  4, }, // Asia/Seoul
    {  380, 1053,  105237,  5, }, // Asia/Shanghai
    {  380, 3300,  105242,  8, }, // Asia/Singapore
    {  380, 4297,  105250, 13, }, // Asia/Srednekolymsk
    {  380, 3263,  105263,  6, }, // Asia/Taipei
    {  380, 4781,  105269,  7, }, // Asia/Tashkent
    {  380, 3915,  105276,  6, }, // Asia/Tbilisi
    {  380, 2772,  105282,  6, }, // Asia/Tehran
    {  380, 1317,  105288,  7, }, // Asia/Thimphu
    {  380, 2821,  105295,  5, }, // Asia/Tokyo
    {  380, 4572,  105300,  5, }, // Asia/Tomsk
    {  380, 1021,  105305,  9, }, // Asia/Ulaanbaatar
    {  380, 1159,  105314,  7, }, // Asia/Urumqi
    {  380, 8722,  105321,  9, }, // Asia/Ust-Nera
    {  380, 6977,  105330, 10, }, // Asia/Vientiane
    {  380, 4710,  105340, 10, }, // Asia/Vladivostok
    {  380, 4828,  105350,  5, }, // Asia/Yakutsk
    {  380, 1235,  105355,  6, }, // Asia/Yangon
    {  380, 3883,  105361, 13, }, // Asia/Yekaterinburg
    {  380, 3747,  105374,  7, }, // Asia/Yerevan
    {  380, 3663,  105381,  7, }, // Atlantic/Azores
    {  380, 5671,  105388,  7, }, // Atlantic/Bermuda
    {  380, 6628,  105395,  6, }, // Atlantic/Canary
    {  380, 3727,  105401,  8, }, // Atlantic/Cape_Verde
    {  380, 1395,  105409,  4, }, // Atlantic/Faroe
    {  380, 8736,  105413,  7, }, // Atlantic/Madeira
    {  380, 2748,  105420,  8, }, // Atlantic/Reykjavik
    {  380, 7272,  105428, 12, }, // Atlantic/South_Georgia
    {  380, 6821,  105440, 11, }, // Atlantic/St_Helena
    {  380, 6644,  105451,  6, }, // Atlantic/Stanley
    {  380, 1670,  105457,  7, }, // Australia/Adelaide
    {  380, 1635,  105464,  6, }, // Australia/Brisbane
    {  380, 1799,  105470, 10, }, // Australia/Broken_Hill
    {  380, 1583,  105480,  6, }, // Australia/Darwin
    {  380, 3637,  105486,  6, }, // Australia/Eucla
    {  380, 1516,  105492,  5, }, // Australia/Hobart
    {  380, 8753,  105497,  8, }, // Australia/Lindeman
    {  380, 1547,  105505,  9, }, // Australia/Lord_Howe
    {  380, 1727,  105514,  8, }, // Australia/Melbourne
    {  380, 1762,  105522,  5, }, // Australia/Perth
    {  380, 1463,  105527,  5, }, // Australia/Sydney
    {  380, 6441,  105532, 10, }, // Europe/Amsterdam
    {  380, 6216,  105542,  7, }, // Europe/Andorra
    {  380, 3620,  105549, 11, }, // Europe/Astrakhan
    {  380, 2300,  105560,  7, }, // Europe/Athens
    {  380, 6336,  105567,  7, }, // Europe/Belgrade
    {  380, 4757,  105574,  6, }, // Europe/Berlin
    {  380, 6488,  105580, 10, }, // Europe/Bratislava
    {  380, 2167,  105590,  6, }, // Europe/Brussels
    {  380, 3928,  105596,  8, }, // Europe/Bucharest
    {  380, 3806,  105604,  8, }, // Europe/Budapest
    {  380, 8772,  105612,  8, }, // Europe/Busingen
    {  380, 2583,  105620,  7, }, // Europe/Chisinau
    {  380, 6289,  105627,  9, }, // Europe/Copenhagen
    {  380, 2338,  105636,  6, }, // Europe/Dublin
    {  380, 6307,  105642,  9, }, // Europe/Gibraltar
    {  380, 8788,  105651,  7, }, // Europe/Guernsey
    {  380, 6612,  105658,  8, }, // Europe/Helsinki
    {  380, 8804,  105666, 10, }, // Europe/Isle_of_Man
    {  380, 1130,  105676,  9, }, // Europe/Istanbul
    {  380, 8823,  105685,  4, }, // Europe/Jersey
    {  380, 3979,  105689, 12, }, // Europe/Kaliningrad
    {  380, 8929,  105701,  4, }, // Europe/Kirov
    {  380, 2527,  105705,  4, }, // Europe/Kyiv
    {  380, 3213,  105709,  6, }, // Europe/Lisbon
    {  380, 6506,  105715, 13, }, // Europe/Ljubljana
    {  380, 2501,  105728,  6, }, // Europe/London
    {  380, 6365,  105734, 10, }, // Europe/Luxembourg
    {  380, 4892,  105744,  6, }, // Europe/Madrid
    {  380, 6397,  105750,  5, }, // Europe/Malta
    {  380, 6582,  105755,  8, }, // Europe/Mariehamn
    {  380, 3693,  105763,  5, }, // Europe/Minsk
    {  380, 6410,  105768,  6, }, // Europe/Monaco
    {  380, 3537,  105774,  5, }, // Europe/Moscow
    {  380, 6458,  105779,  5, }, // Europe/Oslo
    {  380, 4284,  105784,  5, }, // Europe/Paris
    {  380, 6424,  105789,  9, }, // Europe/Podgorica
    {  380, 6275,  105798,  4, }, // Europe/Prague
    {  380, 8837,  105802,  4, }, // Europe/Riga
    {  380, 6324,   10756,  3, }, // Europe/Rome
    {  380, 4331,  105806,  6, }, // Europe/Samara
    {  380, 6470,  105812,  9, }, // Europe/San_Marino
    {  380, 6245,  105821,  9, }, // Europe/Sarajevo
    {  380, 4451,  105830,  6, }, // Europe/Saratov
    {  380, 8849,  105836, 10, }, // Europe/Simferopol
    {  380, 6383,  105846,  7, }, // Europe/Skopje
    {  380, 6599,  105853,  6, }, // Europe/Sofia
    {  380, 6523,  105859,  9, }, // Europe/Stockholm
    {  380, 8867,  105868,  5, }, // Europe/Tallinn
    {  380, 6202,  105873,  5, }, // Europe/Tirane
    {  380, 8882,  105878, 10, }, // Europe/Ulyanovsk
    {  380, 6352,  105888,  5, }, // Europe/Vaduz
    {  380, 6567,  105893,  6, }, // Europe/Vatican
    {  380, 6231,  105899,  7, }, // Europe/Vienna
    {  380, 8899,  105906,  9, }, // Europe/Vilnius
    {  380, 4727,  105915,  9, }, // Europe/Volgograd
    {  380, 3190,  105924,  5, }, // Europe/Warsaw
    {  380, 6261,  105929,  5, }, // Europe/Zagreb
    {  380, 6540,  105934,  7, }, // Europe/Zurich
    {  380, 5078,  105941, 11, }, // Indian/Antananarivo
    {  380, 6947,  105952,  5, }, // Indian/Chagos
    {  380, 6046,  105957,  7, }, // Indian/Christmas
    {  380, 6063,  105964,  5, }, // Indian/Cocos
    {  380, 5029,  105969,  6, }, // Indian/Comoro
    {  380, 6661,  105975,  8, }, // Indian/Kerguelen
    {  380, 7260,  105983,  5, }, // Indian/Mahe
    {  380, 7050,  105988,  8, }, // Indian/Maldives
    {  380, 4069,  105996,  7, }, // Indian/Mauritius
    {  380, 5098,  106003,  4, }, // Indian/Mayotte
    {  380, 7211,  106007,  9, }, // Indian/Reunion
    {  380, 4422,  106016,  5, }, // Pacific/Apia
    {  380,  932,  106021,  6, }, // Pacific/Auckland
    {  380, 3015,  106027,  6, }, // Pacific/Chatham
    {  380, 3157,  106033,  3, }, // Pacific/Chuuk
    {  380, 2237,  106036,  7, }, // Pacific/Easter
    {  380, 7360,  106043,  7, }, // Pacific/Efate
    {  380, 7327,  106050,  7, }, // Pacific/Fakaofo
    {  380, 3902,  106057,  4, }, // Pacific/Fiji
    {  380, 7343,  106061,  9, }, // Pacific/Funafuti
    {  380, 6678,  106070,  8, }, // Pacific/Galapagos
    {  380, 6696,  106078,  8, }, // Pacific/Gambier
    {  380, 3822,  106086, 12, }, // Pacific/Guadalcanal
    {  380, 6018,  104212,  4, }, // Pacific/Guam
    {  380, 2723,  106098,  9, }, // Pacific/Honolulu
    {  380, 3049,  106107,  9, }, // Pacific/Kanton
    {  380, 3998,  106116, 10, }, // Pacific/Kiritimati
    {  380, 7019,  106126,  6, }, // Pacific/Kosrae
    {  380, 2842,  106132,  7, }, // Pacific/Kwajalein
    {  380, 4877,  106139,  7, }, // Pacific/Majuro
    {  380, 4051,  106146,  8, }, // Pacific/Marquesas
    {  380, 8914,  106154,  6, }, // Pacific/Midway
    {  380, 7084,  106160,  6, }, // Pacific/Nauru
    {  380, 7132,  104333,  3, }, // Pacific/Niue
    {  380, 4168,  106166,  6, }, // Pacific/Norfolk
    {  380, 7098,  106172,  5, }, // Pacific/Noumea
    {  380, 3126,  106177,  9, }, // Pacific/Pago_Pago
    {  380, 7155,  106186,  5, }, // Pacific/Palau
    {  380, 7194,  106191,  8, }, // Pacific/Pitcairn
    {  380, 3096,  106199,  6, }, // Pacific/Pohnpei
    {  380, 4807,  106205, 10, }, // Pacific/Port_Moresby
    {  380, 6076,  106215,  9, }, // Pacific/Rarotonga
    {  380, 6031,  106224,  6, }, // Pacific/Saipan
    {  380, 7312,  106230,  6, }, // Pacific/Tahiti
    {  380, 6712,  106236,  6, }, // Pacific/Tarawa
    {  380, 4583,  106242, 10, }, // Pacific/Tongatapu
    {  380, 7392,  106252,  3, }, // Pacific/Wake
    {  380, 7405,  106255,  5, }, // Pacific/Wallis
    {  381,   14,       0,  6, }, // Africa/Asmara Kashmiri/Devanagari/India
    {  381, 4435,       6,  8, }, // Africa/Sao_Tome
    {  381, 4252,      14,  8, }, // America/Asuncion
    {  381,  237,      22,  8, }, // America/Atikokan
    {  381, 7623,      30, 17, }, // America/Bahia_Banderas
    {  381, 3868,      47,  6, }, // America/Cancun
    {  381, 5723,      66,  7, }, // America/Curacao
    {  381,  481,      73, 13, }, // America/Indiana/Knox
    {  381, 7961,      86, 16, }, // America/Indiana/Marengo
    {  381, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  381, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  381, 8038,     139, 14, }, // America/Indiana/Vevay
    {  381, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  381, 8086,     171, 16, }, // America/Indiana/Winamac
    {  381, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  381, 5932,     207, 22, }, // America/Lower_Princes
    {  381, 8220,     229,  6, }, // America/Merida
    {  381, 1850,     235, 19, }, // America/Noronha
    {  381, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  381, 8329,     274, 20, }, // America/North_Dakota/Center
    {  381, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  381,  393,     317,  4, }, // America/Nuuk
    {  381, 6899,     321, 16, }, // America/Scoresbysund
    {  381, 8508,     337, 14, }, // America/St_Barthelemy
    {  381, 2061,     351, 10, }, // America/St_Johns
    {  381, 5863,     361,  9, }, // America/St_Kitts
    {  381, 5880,     370,  9, }, // America/St_Lucia
    {  381,  855,     379, 10, }, // America/St_Thomas
    {  381, 5913,     389, 11, }, // America/St_Vincent
    {  381, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  381, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  381, 1185,     448,  9, }, // Asia/Kathmandu
    {  381,  992,     457,  7, }, // Asia/Kolkata
    {  381, 1211,     464,  5, }, // Asia/Macau
    {  381, 1235,     477,  6, }, // Asia/Yangon
    {  381, 1395,     483,  5, }, // Atlantic/Faroe
    {  381, 6821,     488, 10, }, // Atlantic/St_Helena
    {  381, 2527,     514,  4, }, // Europe/Kyiv
    {  381, 7211,     547,  7, }, // Indian/Reunion
    {  381, 3157,     554,  5, }, // Pacific/Chuuk
    {  381, 3096,     586,  7, }, // Pacific/Pohnpei
    {  382, 6788,   36346,  7, }, // Africa/Abidjan Kazakh/Cyrillic/Kazakhstan
    {  382, 6760,     610,  5, }, // Africa/Accra
    {  382, 5059,     615, 11, }, // Africa/Addis_Ababa
    {  382, 7420,     626,  5, }, // Africa/Algiers
    {  382,   14,   27197,  6, }, // Africa/Asmara
    {  382,   44,     637,  6, }, // Africa/Bamako
    {  382, 5257,     643,  5, }, // Africa/Bangui
    {  382, 6746,     648,  6, }, // Africa/Banjul
    {  382, 7435,     654,  5, }, // Africa/Bissau
    {  382, 4957,   27220,  8, }, // Africa/Blantyre
    {  382, 5287,   51554, 10, }, // Africa/Brazzaville
    {  382, 4922,     676,  9, }, // Africa/Bujumbura
    {  382, 2320,     685,  4, }, // Africa/Cairo
    {  382, 4117,     689, 10, }, // Africa/Casablanca
    {  382, 7449,     699,  5, }, // Africa/Ceuta
    {  382, 6773,     704,  7, }, // Africa/Conakry
    {  382, 6840,     711,  5, }, // Africa/Dakar
    {  382, 5130,   27248, 12, }, // Africa/Dar_es_Salaam
    {  382, 5043,   36379,  7, }, // Africa/Djibouti
    {  382, 5243,     734,  5, }, // Africa/Douala
    {  382, 5166,   27267,  7, }, // Africa/El_Aaiun
    {  382, 6853,     746,  7, }, // Africa/Freetown
    {  382, 4906,     753,  8, }, // Africa/Gaborone
    {  382, 5015,   33913,  6, }, // Africa/Harare
    {  382, 4479,   51572, 12, }, // Africa/Johannesburg
    {  382, 4499,   27305,  5, }, // Africa/Juba
    {  382, 5151,     783,  7, }, // Africa/Kampala
    {  382, 4524,     790,  6, }, // Africa/Khartoum
    {  382, 4973,     796,  6, }, // Africa/Kigali
    {  382, 5306,     802,  7, }, // Africa/Kinshasa
    {  382, 4744,     809,  5, }, // Africa/Lagos
    {  382, 5336,   51584,  9, }, // Africa/Libreville
    {  382, 6869,     822,  4, }, // Africa/Lome
    {  382, 5211,     826,  6, }, // Africa/Luanda
    {  382, 4939,     832,  9, }, // Africa/Lubumbashi
    {  382, 5001,     841,  6, }, // Africa/Lusaka
    {  382, 5322,     847,  6, }, // Africa/Malabo
    {  382, 4987,     853,  6, }, // Africa/Maputo
    {  382, 5197,     859,  6, }, // Africa/Maseru
    {  382, 5182,     865,  7, }, // Africa/Mbabane
    {  382, 5113,     872,  8, }, // Africa/Mogadishu
    {  382, 7034,   36419,  8, }, // Africa/Monrovia
    {  382, 3853,   36427,  7, }, // Africa/Nairobi
    {  382, 5271,   27386,  8, }, // Africa/Ndjamena
    {  382, 5354,   36434,  6, }, // Africa/Niamey
    {  382, 6803,     908,  7, }, // Africa/Nouakchott
    {  382, 6727,     915,  8, }, // Africa/Ouagadougou
    {  382, 5225,     923, 10, }, // Africa/Porto-Novo
    {  382, 4435,     933,  8, }, // Africa/Sao_Tome
    {  382, 2866,     941,  7, }, // Africa/Tripoli
    {  382, 6554,     948,  5, }, // Africa/Tunis
    {  382, 4152,     953,  7, }, // Africa/Windhoek
    {  382,  132,     960,  4, }, // America/Adak
    {  382, 3346,   51593,  8, }, // America/Anchorage
    {  382, 5607,   51601,  7, }, // America/Anguilla
    {  382, 5624,     978,  7, }, // America/Antigua
    {  382, 4554,     985,  9, }, // America/Araguaina
    {  382,  166,   51608, 12, }, // America/Argentina/Buenos_Aires
    {  382,   91,    1006,  9, }, // America/Argentina/Catamarca
    {  382,  270,    1015,  7, }, // America/Argentina/Cordoba
    {  382,  441,   27487,  5, }, // America/Argentina/Jujuy
    {  382, 7462,    1027,  8, }, // America/Argentina/La_Rioja
    {  382,  565,    1035,  7, }, // America/Argentina/Mendoza
    {  382, 7489,   51620, 12, }, // America/Argentina/Rio_Gallegos
    {  382, 7520,   27512,  6, }, // America/Argentina/Salta
    {  382, 7544,    1058,  8, }, // America/Argentina/San_Juan
    {  382, 5580,    1066,  8, }, // America/Argentina/San_Luis
    {  382, 7571,    1074,  7, }, // America/Argentina/Tucuman
    {  382, 7597,  106260,  6, }, // America/Argentina/Ushuaia
    {  382, 5640,    1087,  5, }, // America/Aruba
    {  382, 4252,   51632,  8, }, // America/Asuncion
    {  382,  237,   34103,  8, }, // America/Atikokan
    {  382, 3679,   36500,  4, }, // America/Bahia
    {  382, 7623,   51640, 16, }, // America/Bahia_Banderas
    {  382, 5654,    1131,  8, }, // America/Barbados
    {  382, 7646,    1139,  5, }, // America/Belem
    {  382, 5407,    1144,  5, }, // America/Belize
    {  382, 7660,    1149, 12, }, // America/Blanc-Sablon
    {  382, 7681,    1161,  9, }, // America/Boa_Vista
    {  382, 4361,    1170,  6, }, // America/Bogota
    {  382, 7699,   51656,  5, }, // America/Boise
    {  382, 7713,  106266, 12, }, // America/Cambridge_Bay
    {  382, 7735,    1193, 12, }, // America/Campo_Grande
    {  382, 3868,    1205,  6, }, // America/Cancun
    {  382, 4694,    1211,  7, }, // America/Caracas
    {  382, 4345,   51674,  7, }, // America/Cayenne
    {  382, 5496,  106278, 15, }, // America/Cayman
    {  382, 2260,    1247,  6, }, // America/Chicago
    {  382, 7756,    1253,  6, }, // America/Chihuahua
    {  382, 7774,   56292, 13, }, // America/Ciudad_Juarez
    {  382, 5422,    1259, 10, }, // America/Costa_Rica
    {  382, 7796,    1269,  7, }, // America/Creston
    {  382, 3791,   27688,  5, }, // America/Cuiaba
    {  382, 5723,   36606,  7, }, // America/Curacao
    {  382, 7812,   36613, 12, }, // America/Danmarkshavn
    {  382, 7833,    1302,  6, }, // America/Dawson
    {  382, 7848,  106293, 10, }, // America/Dawson_Creek
    {  382,  805,    1319,  6, }, // America/Denver
    {  382, 3465,   36649,  7, }, // America/Detroit
    {  382, 5739,    1332,  8, }, // America/Dominica
    {  382,  893,   51681,  8, }, // America/Edmonton
    {  382, 7869,   51689,  8, }, // America/Eirunepe
    {  382, 5441,   27765,  9, }, // America/El_Salvador
    {  382, 7886,  106303, 12, }, // America/Fort_Nelson
    {  382, 7906,    1375,  9, }, // America/Fortaleza
    {  382, 7924,   51709,  9, }, // America/Glace_Bay
    {  382, 6881,   51718,  7, }, // America/Goose_Bay
    {  382, 4612,    1400, 10, }, // America/Grand_Turk
    {  382, 5770,    1410,  7, }, // America/Grenada
    {  382, 5786,    1417,  9, }, // America/Guadeloupe
    {  382, 3760,    1426,  9, }, // America/Guatemala
    {  382, 4841,   51725,  8, }, // America/Guayaquil
    {  382, 6932,   51733,  6, }, // America/Guyana
    {  382, 1939,   51739,  8, }, // America/Halifax
    {  382, 2281,    1457,  6, }, // America/Havana
    {  382, 7942,   51747,  9, }, // America/Hermosillo
    {  382,  348,    1471, 12, }, // America/Indiana/Indianapolis
    {  382,  481,    1483, 13, }, // America/Indiana/Knox
    {  382, 7961,    1496, 16, }, // America/Indiana/Marengo
    {  382, 7985,    1512, 19, }, // America/Indiana/Petersburg
    {  382, 8012,   56305, 18, }, // America/Indiana/Tell_City
    {  382, 8038,  106315, 14, }, // America/Indiana/Vevay
    {  382, 8060,   56337, 18, }, // America/Indiana/Vincennes
    {  382, 8086,   56355, 16, }, // America/Indiana/Winamac
    {  382, 8110,    1570,  6, }, // America/Inuvik
    {  382,  660,    1576,  7, }, // America/Iqaluit
    {  382, 2799,   28011,  6, }, // America/Jamaica
    {  382, 5380,   36746,  5, }, // America/Juneau
    {  382,  521,    1594,  8, }, // America/Kentucky/Louisville
    {  382, 8125,    1602, 20, }, // America/Kentucky/Monticello
    {  382, 5704,   36751, 10, }, // America/Kralendijk
    {  382, 4376,    1632,  6, }, // America/La_Paz
    {  382, 7169,    1638,  4, }, // America/Lima
    {  382, 3239,   51761, 12, }, // America/Los_Angeles
    {  382, 5932,   56371, 18, }, // America/Lower_Princes
    {  382, 8153,   36799,  6, }, // America/Maceio
    {  382, 8168,    1677,  7, }, // America/Managua
    {  382, 1908,    1684,  6, }, // America/Manaus
    {  382, 5897,    1690,  6, }, // America/Marigot
    {  382, 5805,    1696,  9, }, // America/Martinique
    {  382, 8184,    1705,  9, }, // America/Matamoros
    {  382, 2917,    1714,  8, }, // America/Mazatlan
    {  382, 8202,    1722,  8, }, // America/Menominee
    {  382, 8220,    1730,  6, }, // America/Merida
    {  382, 8235,    1736, 10, }, // America/Metlakatla
    {  382, 2949,    1746,  6, }, // America/Mexico_City
    {  382, 4391,    1752,  7, }, // America/Miquelon
    {  382, 8254,    1759,  7, }, // America/Moncton
    {  382, 8270,   51794,  9, }, // America/Monterrey
    {  382, 4098,    1775, 10, }, // America/Montevideo
    {  382, 5824,    1785, 10, }, // America/Montserrat
    {  382, 5481,    1795,  6, }, // America/Nassau
    {  382, 2379,   51812,  8, }, // America/New_York
    {  382, 8288,    1809,  3, }, // America/Nome
    {  382, 1850,   56389,  7, }, // America/Noronha
    {  382, 8301,  106329, 23, }, // America/North_Dakota/Beulah
    {  382, 8329,  106352, 24, }, // America/North_Dakota/Center
    {  382, 8357,  106376, 28, }, // America/North_Dakota/New_Salem
    {  382,  393,    1890,  4, }, // America/Nuuk
    {  382, 8388,    1894,  7, }, // America/Ojinaga
    {  382, 2356,    1901,  6, }, // America/Panama
    {  382, 6173,    1907, 10, }, // America/Paramaribo
    {  382, 2973,    1917,  6, }, // America/Phoenix
    {  382, 3945,    1923, 12, }, // America/Port-au-Prince
    {  382, 5954,   28321, 13, }, // America/Port_of_Spain
    {  382, 8404,   28334, 11, }, // America/Porto_Velho
    {  382, 5843,   51888, 11, }, // America/Puerto_Rico
    {  382, 4030,    1970, 12, }, // America/Punta_Arenas
    {  382, 8424,    1982, 12, }, // America/Rankin_Inlet
    {  382, 8445,    1994,  6, }, // America/Recife
    {  382, 1995,   51899,  8, }, // America/Regina
    {  382, 8460,   56461,  8, }, // America/Resolute
    {  382,  695,    2015, 10, }, // America/Rio_Branco
    {  382, 8477,    2025,  8, }, // America/Santarem
    {  382, 2201,   51907,  8, }, // America/Santiago
    {  382, 6111,    2041, 13, }, // America/Santo_Domingo
    {  382, 1878,    2054,  9, }, // America/Sao_Paulo
    {  382, 6899,  106404, 16, }, // America/Scoresbysund
    {  382, 8494,    2074,  5, }, // America/Sitka
    {  382, 8508,    2079, 13, }, // America/St_Barthelemy
    {  382, 2061,   28481, 10, }, // America/St_Johns
    {  382, 5863,    2101,  9, }, // America/St_Kitts
    {  382, 5880,   56488, 10, }, // America/St_Lucia
    {  382,  855,    2121, 10, }, // America/St_Thomas
    {  382, 5913,    2131, 12, }, // America/St_Vincent
    {  382, 8530,  106420, 13, }, // America/Swift_Current
    {  382, 5461,   51949, 12, }, // America/Tegucigalpa
    {  382, 5756,    2167,  4, }, // America/Thule
    {  382,  313,    2171,  7, }, // America/Tijuana
    {  382,  608,    2178,  7, }, // America/Toronto
    {  382, 5688,    2185,  7, }, // America/Tortola
    {  382, 2093,    2192,  8, }, // America/Vancouver
    {  382, 2144,   28567,  8, }, // America/Whitehorse
    {  382,  734,    2208,  8, }, // America/Winnipeg
    {  382, 8552,   28582,  6, }, // America/Yakutat
    {  382, 6001,   37021,  5, }, // Antarctica/Casey
    {  382, 6094,  106433,  6, }, // Antarctica/Davis
    {  382, 6133,   51961, 14, }, // Antarctica/DumontDUrville
    {  382, 8568,    2249,  8, }, // Antarctica/Macquarie
    {  382, 7066,    2257,  6, }, // Antarctica/Mawson
    {  382, 7113,    2263,  9, }, // Antarctica/McMurdo
    {  382, 8589,    2272,  6, }, // Antarctica/Palmer
    {  382, 7226,    2278,  6, }, // Antarctica/Rothera
    {  382, 7295,   28642,  4, }, // Antarctica/Syowa
    {  382, 8607,   51975,  6, }, // Antarctica/Troll
    {  382, 7374,   34957,  6, }, // Antarctica/Vostok
    {  382, 1429,   51981,  7, }, // Arctic/Longyearbyen
    {  382, 5570,    2310,  4, }, // Asia/Aden
    {  382, 5395,    2314,  6, }, // Asia/Almaty
    {  382, 3968,    2320,  5, }, // Asia/Amman
    {  382, 5511,    2325,  6, }, // Asia/Anadyr
    {  382, 5523,  106439,  5, }, // Asia/Aqtau
    {  382, 5368,  106444,  6, }, // Asia/Aqtobe
    {  382,  964,    2342,  7, }, // Asia/Ashgabat
    {  382, 8624,    2349,  6, }, // Asia/Atyrau
    {  382, 3607,  106450,  6, }, // Asia/Baghdad
    {  382, 5534,   37080,  7, }, // Asia/Bahrain
    {  382, 3653,    2368,  4, }, // Asia/Baku
    {  382, 4466,    2372,  7, }, // Asia/Bangkok
    {  382, 3571,  106456,  8, }, // Asia/Barnaul
    {  382, 4086,   28695,  6, }, // Asia/Beirut
    {  382, 3778,   28701,  6, }, // Asia/Bishkek
    {  382, 5989,   28707,  6, }, // Asia/Brunei
    {  382, 4601,    2404,  4, }, // Asia/Chita
    {  382, 4511,    2408,  7, }, // Asia/Colombo
    {  382, 4540,    2415,  6, }, // Asia/Damascus
    {  382, 1093,    2421,  5, }, // Asia/Dhaka
    {  382, 6192,    2426,  4, }, // Asia/Dili
    {  382, 3596,   28728,  5, }, // Asia/Dubai
    {  382, 6159,    2435,  7, }, // Asia/Dushanbe
    {  382, 8636,    2442,  9, }, // Asia/Famagusta
    {  382, 8651,    2451,  4, }, // Asia/Gaza
    {  382, 4795,    2455,  6, }, // Asia/Hebron
    {  382, 1259,    2461,  7, }, // Asia/Ho_Chi_Minh
    {  382, 2704,   52034,  7, }, // Asia/Hong_Kong
    {  382, 4771,    2475,  4, }, // Asia/Hovd
    {  382, 4184,    2479,  7, }, // Asia/Irkutsk
    {  382, 7006,   28770,  8, }, // Asia/Jakarta
    {  382, 6992,   28778,  8, }, // Asia/Jayapura
    {  382, 1290,    2501,  9, }, // Asia/Jerusalem
    {  382, 3560,    2510,  5, }, // Asia/Kabul
    {  382, 4316,    2529,  8, }, // Asia/Kamchatka
    {  382, 4239,    2540,  6, }, // Asia/Karachi
    {  382, 1185,    2546,  8, }, // Asia/Kathmandu
    {  382, 8661,    2554,  7, }, // Asia/Khandyga
    {  382,  992,   52082,  9, }, // Asia/Kolkata
    {  382, 4197,   37110, 10, }, // Asia/Krasnoyarsk
    {  382, 4859,    2580, 12, }, // Asia/Kuala_Lumpur
    {  382, 5976,    2592,  6, }, // Asia/Kuching
    {  382, 5547,   28825,  6, }, // Asia/Kuwait
    {  382, 1211,    2604,  5, }, // Asia/Macau
    {  382, 4017,    2609,  7, }, // Asia/Magadan
    {  382, 1349,    2616,  7, }, // Asia/Makassar
    {  382, 7182,    2623,  6, }, // Asia/Manila
    {  382, 6920,    2629,  6, }, // Asia/Muscat
    {  382, 2554,  106464,  7, }, // Asia/Nicosia
    {  382, 8675,    2642, 11, }, // Asia/Novokuznetsk
    {  382, 4135,  106471,  9, }, // Asia/Novosibirsk
    {  382, 4229,  106480,  4, }, // Asia/Omsk
    {  382, 7145,   35138,  4, }, // Asia/Oral
    {  382, 6961,   52091,  8, }, // Asia/Phnom_Penh
    {  382, 8693,    2681,  9, }, // Asia/Pontianak
    {  382, 4214,   28895,  7, }, // Asia/Pyongyang
    {  382, 5559,    2697,  5, }, // Asia/Qatar
    {  382, 8708,  106484,  8, }, // Asia/Qostanay
    {  382, 4269,  106492,  9, }, // Asia/Qyzylorda
    {  382, 3584,   52099,  7, }, // Asia/Riyadh
    {  382, 4408,    2732,  7, }, // Asia/Sakhalin
    {  382, 7245,  106501,  9, }, // Asia/Samarkand
    {  382, 3279,    2748,  4, }, // Asia/Seoul
    {  382, 1053,   28933,  6, }, // Asia/Shanghai
    {  382, 3300,    2758,  8, }, // Asia/Singapore
    {  382, 4297,    2766, 13, }, // Asia/Srednekolymsk
    {  382, 3263,   28961,  6, }, // Asia/Taipei
    {  382, 4781,    2785,  7, }, // Asia/Tashkent
    {  382, 3915,   35206,  7, }, // Asia/Tbilisi
    {  382, 2772,    2796,  7, }, // Asia/Tehran
    {  382, 1317,    2803,  7, }, // Asia/Thimphu
    {  382, 2821,    2810,  5, }, // Asia/Tokyo
    {  382, 4572,    2815,  5, }, // Asia/Tomsk
    {  382, 1021,  106510,  9, }, // Asia/Ulaanbaatar
    {  382, 1159,  106519,  6, }, // Asia/Urumqi
    {  382, 8722,   52125,  9, }, // Asia/Ust-Nera
    {  382, 6977,   52134,  8, }, // Asia/Vientiane
    {  382, 4710,    2851, 11, }, // Asia/Vladivostok
    {  382, 4828,   37183,  6, }, // Asia/Yakutsk
    {  382, 1235,   56541,  5, }, // Asia/Yangon
    {  382, 3883,    2875, 12, }, // Asia/Yekaterinburg
    {  382, 3747,    2887,  6, }, // Asia/Yerevan
    {  382, 3663,  106525, 13, }, // Atlantic/Azores
    {  382, 5671,  106538, 15, }, // Atlantic/Bermuda
    {  382, 6628,  106553, 14, }, // Atlantic/Canary
    {  382, 3727,    2944, 10, }, // Atlantic/Cape_Verde
    {  382, 1395,  106567, 14, }, // Atlantic/Faroe
    {  382, 8736,   37264,  7, }, // Atlantic/Madeira
    {  382, 2748,   52214,  9, }, // Atlantic/Reykjavik
    {  382, 7272,  106581, 16, }, // Atlantic/South_Georgia
    {  382, 6821,  106597, 17, }, // Atlantic/St_Helena
    {  382, 6644,   52256,  6, }, // Atlantic/Stanley
    {  382, 1670,    3035,  8, }, // Australia/Adelaide
    {  382, 1635,    3043,  7, }, // Australia/Brisbane
    {  382, 1799,    3050, 11, }, // Australia/Broken_Hill
    {  382, 1583,    3061,  6, }, // Australia/Darwin
    {  382, 3637,   29240,  4, }, // Australia/Eucla
    {  382, 1516,    3072,  6, }, // Australia/Hobart
    {  382, 8753,    3078,  8, }, // Australia/Lindeman
    {  382, 1547,    3086,  8, }, // Australia/Lord_Howe
    {  382, 1727,   29252,  8, }, // Australia/Melbourne
    {  382, 1762,    3101,  4, }, // Australia/Perth
    {  382, 1463,   52276,  6, }, // Australia/Sydney
    {  382, 6441,    3111,  9, }, // Europe/Amsterdam
    {  382, 6216,    3120,  7, }, // Europe/Andorra
    {  382, 3620,    3127,  8, }, // Europe/Astrakhan
    {  382, 2300,   52291,  5, }, // Europe/Athens
    {  382, 6336,    3139,  7, }, // Europe/Belgrade
    {  382, 4757,    3146,  6, }, // Europe/Berlin
    {  382, 6488,    3152, 10, }, // Europe/Bratislava
    {  382, 2167,   52296,  8, }, // Europe/Brussels
    {  382, 3928,    3170,  8, }, // Europe/Bucharest
    {  382, 3806,    3178,  8, }, // Europe/Budapest
    {  382, 8772,   52304, 19, }, // Europe/Busingen
    {  382, 2583,    3209,  7, }, // Europe/Chisinau
    {  382, 6289,   52323, 10, }, // Europe/Copenhagen
    {  382, 2338,    3226,  6, }, // Europe/Dublin
    {  382, 6307,    3232,  9, }, // Europe/Gibraltar
    {  382, 8788,    3241,  6, }, // Europe/Guernsey
    {  382, 6612,   52333,  9, }, // Europe/Helsinki
    {  382, 8804,  106614,  9, }, // Europe/Isle_of_Man
    {  382, 1130,  106623,  7, }, // Europe/Istanbul
    {  382, 8823,   52351,  6, }, // Europe/Jersey
    {  382, 3979,    3276, 11, }, // Europe/Kaliningrad
    {  382, 8929,    3287,  5, }, // Europe/Kirov
    {  382, 2527,    3292,  4, }, // Europe/Kyiv
    {  382, 3213,    3296,  8, }, // Europe/Lisbon
    {  382, 6506,   29432,  7, }, // Europe/Ljubljana
    {  382, 2501,    3313,  6, }, // Europe/London
    {  382, 6365,   29445, 10, }, // Europe/Luxembourg
    {  382, 4892,    3330,  6, }, // Europe/Madrid
    {  382, 6397,   29461,  6, }, // Europe/Malta
    {  382, 6582,    3341,  9, }, // Europe/Mariehamn
    {  382, 3693,    3350,  5, }, // Europe/Minsk
    {  382, 6410,    3355,  6, }, // Europe/Monaco
    {  382, 3537,  106630,  6, }, // Europe/Moscow
    {  382, 6458,    3367,  4, }, // Europe/Oslo
    {  382, 4284,    3371,  5, }, // Europe/Paris
    {  382, 6424,    3376,  9, }, // Europe/Podgorica
    {  382, 6275,    3385,  5, }, // Europe/Prague
    {  382, 8837,    3390,  4, }, // Europe/Riga
    {  382, 6324,    3394,  3, }, // Europe/Rome
    {  382, 4331,    3397,  6, }, // Europe/Samara
    {  382, 6470,    3403, 10, }, // Europe/San_Marino
    {  382, 6245,    3413,  7, }, // Europe/Sarajevo
    {  382, 4451,    3420,  7, }, // Europe/Saratov
    {  382, 8849,   52357, 11, }, // Europe/Simferopol
    {  382, 6383,   52368,  6, }, // Europe/Skopje
    {  382, 6599,   37418,  5, }, // Europe/Sofia
    {  382, 6523,   52374,  9, }, // Europe/Stockholm
    {  382, 8867,    3455,  6, }, // Europe/Tallinn
    {  382, 6202,    3461,  6, }, // Europe/Tirane
    {  382, 8882,   52383,  9, }, // Europe/Ulyanovsk
    {  382, 6352,    3476,  5, }, // Europe/Vaduz
    {  382, 6567,    3481,  7, }, // Europe/Vatican
    {  382, 6231,    3488,  4, }, // Europe/Vienna
    {  382, 8899,   52392,  7, }, // Europe/Vilnius
    {  382, 4727,    3499,  9, }, // Europe/Volgograd
    {  382, 3190,    3508,  7, }, // Europe/Warsaw
    {  382, 6261,    3515,  6, }, // Europe/Zagreb
    {  382, 6540,   37442,  5, }, // Europe/Zurich
    {  382, 5078,    3527, 12, }, // Indian/Antananarivo
    {  382, 6947,    3539,  5, }, // Indian/Chagos
    {  382, 6046,  106636, 15, }, // Indian/Christmas
    {  382, 6063,  106651, 14, }, // Indian/Cocos
    {  382, 5029,  106665, 14, }, // Indian/Comoro
    {  382, 6661,    3578,  8, }, // Indian/Kerguelen
    {  382, 7260,   29694,  3, }, // Indian/Mahe
    {  382, 7050,  106679, 16, }, // Indian/Maldives
    {  382, 4069,   56708,  8, }, // Indian/Mauritius
    {  382, 5098,   56716,  7, }, // Indian/Mayotte
    {  382, 7211,   52480,  7, }, // Indian/Reunion
    {  382, 4422,    3616,  4, }, // Pacific/Apia
    {  382,  932,    3620,  6, }, // Pacific/Auckland
    {  382, 3706,   52487,  9, }, // Pacific/Bougainville
    {  382, 3015,    3634,  5, }, // Pacific/Chatham
    {  382, 3157,    3639,  4, }, // Pacific/Chuuk
    {  382, 2237,  106695, 11, }, // Pacific/Easter
    {  382, 7360,   52541,  5, }, // Pacific/Efate
    {  382, 7327,    3659,  7, }, // Pacific/Fakaofo
    {  382, 3902,   37539,  5, }, // Pacific/Fiji
    {  382, 7343,    3670,  8, }, // Pacific/Funafuti
    {  382, 6678,    3678,  9, }, // Pacific/Galapagos
    {  382, 6696,   52567,  6, }, // Pacific/Gambier
    {  382, 3822,    3713, 11, }, // Pacific/Guadalcanal
    {  382, 6018,    3724,  4, }, // Pacific/Guam
    {  382, 2723,   52582,  8, }, // Pacific/Honolulu
    {  382, 3049,   52590,  9, }, // Pacific/Kanton
    {  382, 3998,    3734, 10, }, // Pacific/Kiritimati
    {  382, 7019,  106706,  6, }, // Pacific/Kosrae
    {  382, 2842,   29841, 10, }, // Pacific/Kwajalein
    {  382, 4877,   37574,  7, }, // Pacific/Majuro
    {  382, 4051,  106712, 15, }, // Pacific/Marquesas
    {  382, 8914,   52622,  6, }, // Pacific/Midway
    {  382, 7084,    3798,  5, }, // Pacific/Nauru
    {  382, 7132,   52636,  4, }, // Pacific/Niue
    {  382, 4168,    3807,  7, }, // Pacific/Norfolk
    {  382, 7098,    3814,  5, }, // Pacific/Noumea
    {  382, 3126,    3819,  9, }, // Pacific/Pago_Pago
    {  382, 7155,    3828,  5, }, // Pacific/Palau
    {  382, 7194,   56789,  7, }, // Pacific/Pitcairn
    {  382, 3096,   35783,  6, }, // Pacific/Pohnpei
    {  382, 4807,    3846, 11, }, // Pacific/Port_Moresby
    {  382, 6076,    3857,  9, }, // Pacific/Rarotonga
    {  382, 6031,   29935,  6, }, // Pacific/Saipan
    {  382, 7312,    3872,  5, }, // Pacific/Tahiti
    {  382, 6712,    3877,  6, }, // Pacific/Tarawa
    {  382, 4583,    3883,  9, }, // Pacific/Tongatapu
    {  382, 7392,   29955,  4, }, // Pacific/Wake
    {  382, 7405,    3896,  6, }, // Pacific/Wallis
    {  383, 6788,  106727,  7, }, // Africa/Abidjan Kazakh/Arabic/China
    {  383, 6760,  106734,  5, }, // Africa/Accra
    {  383, 5059,  106739, 11, }, // Africa/Addis_Ababa
    {  383, 7420,  106750,  5, }, // Africa/Algiers
    {  383,   14,  106755,  6, }, // Africa/Asmara
    {  383,   44,   10533,  6, }, // Africa/Bamako
    {  383, 5257,  106761,  5, }, // Africa/Bangui
    {  383, 6746,  106766,  6, }, // Africa/Banjul
    {  383, 7435,  106772,  5, }, // Africa/Bissau
    {  383, 4957,   10556,  8, }, // Africa/Blantyre
    {  383, 5287,  106777,  9, }, // Africa/Brazzaville
    {  383, 4922,  106786,  9, }, // Africa/Bujumbura
    {  383, 2320,  106795,  4, }, // Africa/Cairo
    {  383, 4117,  106799, 10, }, // Africa/Casablanca
    {  383, 7449,  106809,  5, }, // Africa/Ceuta
    {  383, 6773,   10605,  7, }, // Africa/Conakry
    {  383, 6840,   10612,  5, }, // Africa/Dakar
    {  383, 5130,  106814, 12, }, // Africa/Dar_es_Salaam
    {  383, 5043,  106826,  6, }, // Africa/Djibouti
    {  383, 5243,  106832,  5, }, // Africa/Douala
    {  383, 5166,  106837,  7, }, // Africa/El_Aaiun
    {  383, 6853,  106844,  7, }, // Africa/Freetown
    {  383, 4906,  106851,  8, }, // Africa/Gaborone
    {  383, 5015,  106859,  6, }, // Africa/Harare
    {  383, 4479,  106865, 12, }, // Africa/Johannesburg
    {  383, 4499,  106877,  4, }, // Africa/Juba
    {  383, 5151,  106881,  7, }, // Africa/Kampala
    {  383, 4524,  106888,  6, }, // Africa/Khartoum
    {  383, 4973,  106894,  6, }, // Africa/Kigali
    {  383, 5306,   10697,  7, }, // Africa/Kinshasa
    {  383, 4744,  106900,  5, }, // Africa/Lagos
    {  383, 5336,  106905,  8, }, // Africa/Libreville
    {  383, 6869,  106913,  4, }, // Africa/Lome
    {  383, 5211,  106917,  6, }, // Africa/Luanda
    {  383, 4939,  106923,  9, }, // Africa/Lubumbashi
    {  383, 5001,  106932,  6, }, // Africa/Lusaka
    {  383, 5322,   10740,  6, }, // Africa/Malabo
    {  383, 4987,  106938,  6, }, // Africa/Maputo
    {  383, 5197,  106944,  6, }, // Africa/Maseru
    {  383, 5182,  106950,  7, }, // Africa/Mbabane
    {  383, 5113,  106957,  8, }, // Africa/Mogadishu
    {  383, 7034,  106965,  8, }, // Africa/Monrovia
    {  383, 3853,  106973,  7, }, // Africa/Nairobi
    {  383, 5271,  106980,  8, }, // Africa/Ndjamena
    {  383, 5354,  106988,  6, }, // Africa/Niamey
    {  383, 6803,  106994,  7, }, // Africa/Nouakchott
    {  383, 6727,  107001,  8, }, // Africa/Ouagadougou
    {  383, 5225,  107009, 10, }, // Africa/Porto-Novo
    {  383, 4435,  107019,  8, }, // Africa/Sao_Tome
    {  383, 2866,  107027,  7, }, // Africa/Tripoli
    {  383, 6554,  107034,  5, }, // Africa/Tunis
    {  383, 4152,  107039,  7, }, // Africa/Windhoek
    {  383,  132,  107046,  4, }, // America/Adak
    {  383, 3346,  107050,  7, }, // America/Anchorage
    {  383, 5607,  107057,  7, }, // America/Anguilla
    {  383, 5624,  107064,  7, }, // America/Antigua
    {  383, 4554,  107071,  9, }, // America/Araguaina
    {  383,  166,  107080, 12, }, // America/Argentina/Buenos_Aires
    {  383,   91,   10893,  9, }, // America/Argentina/Catamarca
    {  383,  270,   10902,  7, }, // America/Argentina/Cordoba
    {  383,  441,  107092,  5, }, // America/Argentina/Jujuy
    {  383, 7462,  107097,  8, }, // America/Argentina/La_Rioja
    {  383,  565,  107105,  7, }, // America/Argentina/Mendoza
    {  383, 7489,  107112, 11, }, // America/Argentina/Rio_Gallegos
    {  383, 7520,  107123,  5, }, // America/Argentina/Salta
    {  383, 7544,  107128,  8, }, // America/Argentina/San_Juan
    {  383, 5580,  107136,  8, }, // America/Argentina/San_Luis
    {  383, 7571,  107144,  7, }, // America/Argentina/Tucuman
    {  383, 7597,  107151,  6, }, // America/Argentina/Ushuaia
    {  383, 5640,  107157,  5, }, // America/Aruba
    {  383, 4252,  107162,  8, }, // America/Asuncion
    {  383,  237,  107170,  8, }, // America/Atikokan
    {  383, 3679,  107178,  4, }, // America/Bahia
    {  383, 7623,  107182, 16, }, // America/Bahia_Banderas
    {  383, 5654,  107198,  8, }, // America/Barbados
    {  383, 7646,  107206,  5, }, // America/Belem
    {  383, 5407,  107211,  5, }, // America/Belize
    {  383, 7660,   11027, 12, }, // America/Blanc-Sablon
    {  383, 7681,  107216,  9, }, // America/Boa_Vista
    {  383, 4361,  107225,  6, }, // America/Bogota
    {  383, 7699,  107231,  5, }, // America/Boise
    {  383, 7713,  107236, 11, }, // America/Cambridge_Bay
    {  383, 7735,  107247, 12, }, // America/Campo_Grande
    {  383, 3868,  107259,  6, }, // America/Cancun
    {  383, 4694,   11087,  7, }, // America/Caracas
    {  383, 4345,  107265,  7, }, // America/Cayenne
    {  383, 5496,  107272, 15, }, // America/Cayman
    {  383, 2260,  107287,  6, }, // America/Chicago
    {  383, 7756,  107293,  6, }, // America/Chihuahua
    {  383, 7774,  107299, 13, }, // America/Ciudad_Juarez
    {  383, 5422,  107312, 10, }, // America/Costa_Rica
    {  383, 7796,  107322,  7, }, // America/Creston
    {  383, 3791,  107329,  6, }, // America/Cuiaba
    {  383, 5723,  107335,  8, }, // America/Curacao
    {  383, 7812,  107343, 12, }, // America/Danmarkshavn
    {  383, 7833,  107355,  6, }, // America/Dawson
    {  383, 7848,  107361, 11, }, // America/Dawson_Creek
    {  383,  805,  107372,  6, }, // America/Denver
    {  383, 3465,  107378,  7, }, // America/Detroit
    {  383, 5739,   11197,  8, }, // America/Dominica
    {  383,  893,  107385,  8, }, // America/Edmonton
    {  383, 7869,  107393,  8, }, // America/Eirunepe
    {  383, 5441,  107401,  8, }, // America/El_Salvador
    {  383, 7886,  107409, 11, }, // America/Fort_Nelson
    {  383, 7906,  107420,  9, }, // America/Fortaleza
    {  383, 7924,  107429,  9, }, // America/Glace_Bay
    {  383, 6881,  107438,  7, }, // America/Goose_Bay
    {  383, 4612,  107445, 10, }, // America/Grand_Turk
    {  383, 5770,  107455,  7, }, // America/Grenada
    {  383, 5786,  107462,  9, }, // America/Guadeloupe
    {  383, 3760,  107471,  9, }, // America/Guatemala
    {  383, 4841,  107480,  8, }, // America/Guayaquil
    {  383, 6932,  107488,  6, }, // America/Guyana
    {  383, 1939,  107494,  8, }, // America/Halifax
    {  383, 2281,  107502,  6, }, // America/Havana
    {  383, 7942,  107508,  8, }, // America/Hermosillo
    {  383,  348,  107516, 12, }, // America/Indiana/Indianapolis
    {  383,  481,  107528, 13, }, // America/Indiana/Knox
    {  383, 7961,  107541, 16, }, // America/Indiana/Marengo
    {  383, 7985,  107557, 19, }, // America/Indiana/Petersburg
    {  383, 8012,  107576, 18, }, // America/Indiana/Tell_City
    {  383, 8038,  107594, 14, }, // America/Indiana/Vevay
    {  383, 8060,  107608, 18, }, // America/Indiana/Vincennes
    {  383, 8086,  107626, 16, }, // America/Indiana/Winamac
    {  383, 8110,  107642,  6, }, // America/Inuvik
    {  383,  660,  107648,  7, }, // America/Iqaluit
    {  383, 2799,  107655,  7, }, // America/Jamaica
    {  383, 5380,  107662,  4, }, // America/Juneau
    {  383,  521,  107666,  7, }, // America/Kentucky/Louisville
    {  383, 8125,  107673, 20, }, // America/Kentucky/Monticello
    {  383, 5704,  107693, 10, }, // America/Kralendijk
    {  383, 4376,  107703,  6, }, // America/La_Paz
    {  383, 7169,   11456,  4, }, // America/Lima
    {  383, 3239,  107709, 11, }, // America/Los_Angeles
    {  383, 5932,  107720, 18, }, // America/Lower_Princes
    {  383, 8153,  107738,  6, }, // America/Maceio
    {  383, 8168,  107744,  7, }, // America/Managua
    {  383, 1908,  107751,  6, }, // America/Manaus
    {  383, 5897,  107757,  6, }, // America/Marigot
    {  383, 5805,  107763,  9, }, // America/Martinique
    {  383, 8184,   11521,  9, }, // America/Matamoros
    {  383, 2917,  107772,  8, }, // America/Mazatlan
    {  383, 8202,  107780,  8, }, // America/Menominee
    {  383, 8220,  107788,  6, }, // America/Merida
    {  383, 8235,  107794, 10, }, // America/Metlakatla
    {  383, 2949,  107804,  6, }, // America/Mexico_City
    {  383, 4391,  107810,  7, }, // America/Miquelon
    {  383, 8254,  107817,  7, }, // America/Moncton
    {  383, 8270,  107824,  9, }, // America/Monterrey
    {  383, 4098,  107833, 10, }, // America/Montevideo
    {  383, 5824,  107843, 10, }, // America/Montserrat
    {  383, 5481,  107853,  6, }, // America/Nassau
    {  383, 2379,  107859,  8, }, // America/New_York
    {  383, 8288,   11540,  3, }, // America/Nome
    {  383, 1850,  107867,  7, }, // America/Noronha
    {  383, 8301,  107874, 23, }, // America/North_Dakota/Beulah
    {  383, 8329,  107897, 24, }, // America/North_Dakota/Center
    {  383, 8357,  107921, 28, }, // America/North_Dakota/New_Salem
    {  383,  393,  107949,  4, }, // America/Nuuk
    {  383, 8388,  107953,  7, }, // America/Ojinaga
    {  383, 2356,  107960,  6, }, // America/Panama
    {  383, 6173,  107966, 10, }, // America/Paramaribo
    {  383, 2973,  107976,  6, }, // America/Phoenix
    {  383, 3945,  107982, 13, }, // America/Port-au-Prince
    {  383, 5954,  107995, 13, }, // America/Port_of_Spain
    {  383, 8404,  108008, 11, }, // America/Porto_Velho
    {  383, 5843,  108019, 11, }, // America/Puerto_Rico
    {  383, 4030,  108030, 12, }, // America/Punta_Arenas
    {  383, 8424,  108042, 12, }, // America/Rankin_Inlet
    {  383, 8445,  108054,  6, }, // America/Recife
    {  383, 1995,  108060,  7, }, // America/Regina
    {  383, 8460,  108067,  8, }, // America/Resolute
    {  383,  695,  108075, 10, }, // America/Rio_Branco
    {  383, 8477,  108085,  8, }, // America/Santarem
    {  383, 2201,  108093,  8, }, // America/Santiago
    {  383, 6111,  108101, 13, }, // America/Santo_Domingo
    {  383, 1878,  108114,  9, }, // America/Sao_Paulo
    {  383, 6899,  108123, 16, }, // America/Scoresbysund
    {  383, 8494,   11839,  5, }, // America/Sitka
    {  383, 8508,  108139, 13, }, // America/St_Barthelemy
    {  383, 2061,  108152,  9, }, // America/St_Johns
    {  383, 5863,  108161,  9, }, // America/St_Kitts
    {  383, 5880,  108170, 11, }, // America/St_Lucia
    {  383,  855,  108181, 10, }, // America/St_Thomas
    {  383, 5913,  108191, 12, }, // America/St_Vincent
    {  383, 8530,  108203, 13, }, // America/Swift_Current
    {  383, 5461,  108216, 11, }, // America/Tegucigalpa
    {  383, 5756,  108227,  4, }, // America/Thule
    {  383,  313,  108231,  7, }, // America/Tijuana
    {  383,  608,   11937,  7, }, // America/Toronto
    {  383, 5688,   11944,  7, }, // America/Tortola
    {  383, 2093,  108238,  8, }, // America/Vancouver
    {  383, 2144,  108246,  8, }, // America/Whitehorse
    {  383,  734,  108254,  8, }, // America/Winnipeg
    {  383, 8552,  108262,  7, }, // America/Yakutat
    {  383, 6001,  108269,  5, }, // Antarctica/Casey
    {  383, 6094,  108274,  6, }, // Antarctica/Davis
    {  383, 6133,  108280, 14, }, // Antarctica/DumontDUrville
    {  383, 8568,  108294,  8, }, // Antarctica/Macquarie
    {  383, 7066,  108302,  6, }, // Antarctica/Mawson
    {  383, 7113,  108308,  9, }, // Antarctica/McMurdo
    {  383, 8589,  108317,  6, }, // Antarctica/Palmer
    {  383, 7226,  108323,  6, }, // Antarctica/Rothera
    {  383, 7295,  108329,  4, }, // Antarctica/Syowa
    {  383, 8607,   12047,  4, }, // Antarctica/Troll
    {  383, 7374,  108333,  6, }, // Antarctica/Vostok
    {  383, 1429,  108339,  6, }, // Arctic/Longyearbyen
    {  383, 5570,  108345,  4, }, // Asia/Aden
    {  383, 5395,  108349,  6, }, // Asia/Almaty
    {  383, 3968,  108355,  5, }, // Asia/Amman
    {  383, 5511,  108360,  6, }, // Asia/Anadyr
    {  383, 5523,  108366,  5, }, // Asia/Aqtau
    {  383, 5368,  108371,  6, }, // Asia/Aqtobe
    {  383,  964,  108377,  7, }, // Asia/Ashgabat
    {  383, 8624,  108384,  6, }, // Asia/Atyrau
    {  383, 3607,  108390,  6, }, // Asia/Baghdad
    {  383, 5534,  108396,  7, }, // Asia/Bahrain
    {  383, 3653,  108403,  4, }, // Asia/Baku
    {  383, 4466,  108407,  6, }, // Asia/Bangkok
    {  383, 3571,  108413,  8, }, // Asia/Barnaul
    {  383, 4086,  108421,  6, }, // Asia/Beirut
    {  383, 3778,  108427,  6, }, // Asia/Bishkek
    {  383, 5989,  108433,  6, }, // Asia/Brunei
    {  383, 4601,  108439,  4, }, // Asia/Chita
    {  383, 4511,   12155,  7, }, // Asia/Colombo
    {  383, 4540,  108443,  6, }, // Asia/Damascus
    {  383, 1093,  108449,  5, }, // Asia/Dhaka
    {  383, 6192,   12166,  4, }, // Asia/Dili
    {  383, 3596,  108454,  5, }, // Asia/Dubai
    {  383, 6159,  108459,  7, }, // Asia/Dushanbe
    {  383, 8636,  108466,  9, }, // Asia/Famagusta
    {  383, 8651,  108475,  4, }, // Asia/Gaza
    {  383, 4795,  108479,  6, }, // Asia/Hebron
    {  383, 1259,  108485,  7, }, // Asia/Ho_Chi_Minh
    {  383, 2704,  108492,  6, }, // Asia/Hong_Kong
    {  383, 4771,  108498,  4, }, // Asia/Hovd
    {  383, 4184,  108502,  7, }, // Asia/Irkutsk
    {  383, 7006,  108509,  7, }, // Asia/Jakarta
    {  383, 6992,  108516,  8, }, // Asia/Jayapura
    {  383, 1290,  108524,  9, }, // Asia/Jerusalem
    {  383, 3560,  108533,  5, }, // Asia/Kabul
    {  383, 4316,  108538,  8, }, // Asia/Kamchatka
    {  383, 4239,  108546,  6, }, // Asia/Karachi
    {  383, 1185,  108552,  8, }, // Asia/Kathmandu
    {  383, 8661,  108560,  7, }, // Asia/Khandyga
    {  383,  992,  108567,  7, }, // Asia/Kolkata
    {  383, 4197,   12311, 11, }, // Asia/Krasnoyarsk
    {  383, 4859,  108574, 12, }, // Asia/Kuala_Lumpur
    {  383, 5976,  108586,  5, }, // Asia/Kuching
    {  383, 5547,  108591,  6, }, // Asia/Kuwait
    {  383, 1211,   12346,  5, }, // Asia/Macau
    {  383, 4017,  108597,  7, }, // Asia/Magadan
    {  383, 1349,  108604,  7, }, // Asia/Makassar
    {  383, 7182,   12362,  6, }, // Asia/Manila
    {  383, 6920,  108611,  6, }, // Asia/Muscat
    {  383, 2554,  108617,  7, }, // Asia/Nicosia
    {  383, 8675,  108624, 12, }, // Asia/Novokuznetsk
    {  383, 4135,  108636,  9, }, // Asia/Novosibirsk
    {  383, 4229,  108645,  4, }, // Asia/Omsk
    {  383, 7145,   10984,  4, }, // Asia/Oral
    {  383, 6961,  108649,  7, }, // Asia/Phnom_Penh
    {  383, 8693,  108656,  9, }, // Asia/Pontianak
    {  383, 4214,  108665,  7, }, // Asia/Pyongyang
    {  383, 5559,  108672,  5, }, // Asia/Qatar
    {  383, 8708,   12439,  8, }, // Asia/Qostanay
    {  383, 4269,  108677,  9, }, // Asia/Qyzylorda
    {  383, 3584,  108686,  7, }, // Asia/Riyadh
    {  383, 4408,  108693,  7, }, // Asia/Sakhalin
    {  383, 7245,  108700,  9, }, // Asia/Samarkand
    {  383, 3279,  108709,  4, }, // Asia/Seoul
    {  383, 1053,  108713,  6, }, // Asia/Shanghai
    {  383, 3300,  108719,  8, }, // Asia/Singapore
    {  383, 4297,  108727, 13, }, // Asia/Srednekolymsk
    {  383, 3263,  108740,  6, }, // Asia/Taipei
    {  383, 4781,  108746,  7, }, // Asia/Tashkent
    {  383, 3915,  108753,  7, }, // Asia/Tbilisi
    {  383, 2772,  108760,  7, }, // Asia/Tehran
    {  383, 1317,  108767,  7, }, // Asia/Thimphu
    {  383, 2821,  108774,  5, }, // Asia/Tokyo
    {  383, 4572,  108779,  5, }, // Asia/Tomsk
    {  383, 1021,  108784,  9, }, // Asia/Ulaanbaatar
    {  383, 1159,  108793,  7, }, // Asia/Urumqi
    {  383, 8722,  108800,  8, }, // Asia/Ust-Nera
    {  383, 6977,  108808,  7, }, // Asia/Vientiane
    {  383, 4710,  108815, 11, }, // Asia/Vladivostok
    {  383, 4828,  108826,  7, }, // Asia/Yakutsk
    {  383, 1235,  108833,  6, }, // Asia/Yangon
    {  383, 3883,  108839, 12, }, // Asia/Yekaterinburg
    {  383, 3747,  108851,  7, }, // Asia/Yerevan
    {  383, 3663,  108858, 13, }, // Atlantic/Azores
    {  383, 5671,  108871, 15, }, // Atlantic/Bermuda
    {  383, 6628,  108886, 14, }, // Atlantic/Canary
    {  383, 3727,  108900, 10, }, // Atlantic/Cape_Verde
    {  383, 1395,  108910, 14, }, // Atlantic/Faroe
    {  383, 8736,  108924,  7, }, // Atlantic/Madeira
    {  383, 2748,  108931,  9, }, // Atlantic/Reykjavik
    {  383, 7272,  108940, 16, }, // Atlantic/South_Georgia
    {  383, 6821,  108956, 17, }, // Atlantic/St_Helena
    {  383, 6644,  108973,  6, }, // Atlantic/Stanley
    {  383, 1670,  108979,  8, }, // Australia/Adelaide
    {  383, 1635,  108987,  7, }, // Australia/Brisbane
    {  383, 1799,  108994, 10, }, // Australia/Broken_Hill
    {  383, 1583,  109004,  6, }, // Australia/Darwin
    {  383, 3637,  109010,  5, }, // Australia/Eucla
    {  383, 1516,  109015,  6, }, // Australia/Hobart
    {  383, 8753,  109021,  8, }, // Australia/Lindeman
    {  383, 1547,  109029,  8, }, // Australia/Lord_Howe
    {  383, 1727,  109037,  7, }, // Australia/Melbourne
    {  383, 1762,  109044,  4, }, // Australia/Perth
    {  383, 1463,  109048,  6, }, // Australia/Sydney
    {  383, 6441,  109054,  9, }, // Europe/Amsterdam
    {  383, 6216,  109063,  7, }, // Europe/Andorra
    {  383, 3620,  109070,  8, }, // Europe/Astrakhan
    {  383, 2300,  109078,  5, }, // Europe/Athens
    {  383, 6336,  109083,  7, }, // Europe/Belgrade
    {  383, 4757,  109090,  6, }, // Europe/Berlin
    {  383, 6488,  109096, 10, }, // Europe/Bratislava
    {  383, 2167,  109106,  7, }, // Europe/Brussels
    {  383, 3928,  109113,  8, }, // Europe/Bucharest
    {  383, 3806,  109121,  8, }, // Europe/Budapest
    {  383, 8772,  109129,  9, }, // Europe/Busingen
    {  383, 2583,  109138,  7, }, // Europe/Chisinau
    {  383, 6289,  109145, 10, }, // Europe/Copenhagen
    {  383, 2338,  109155,  6, }, // Europe/Dublin
    {  383, 6307,  109161,  9, }, // Europe/Gibraltar
    {  383, 8788,  109170,  6, }, // Europe/Guernsey
    {  383, 6612,  109176,  8, }, // Europe/Helsinki
    {  383, 8804,  109184,  9, }, // Europe/Isle_of_Man
    {  383, 1130,  109193,  7, }, // Europe/Istanbul
    {  383, 8823,  109200,  6, }, // Europe/Jersey
    {  383, 3979,  109206, 11, }, // Europe/Kaliningrad
    {  383, 8929,  109217,  5, }, // Europe/Kirov
    {  383, 2527,  109222,  4, }, // Europe/Kyiv
    {  383, 3213,  109226,  7, }, // Europe/Lisbon
    {  383, 6506,  109233,  9, }, // Europe/Ljubljana
    {  383, 2501,  109242,  6, }, // Europe/London
    {  383, 6365,  109248, 11, }, // Europe/Luxembourg
    {  383, 4892,  109259,  6, }, // Europe/Madrid
    {  383, 6397,  109265,  5, }, // Europe/Malta
    {  383, 6582,  109270,  9, }, // Europe/Mariehamn
    {  383, 3693,   12957,  5, }, // Europe/Minsk
    {  383, 6410,   12962,  6, }, // Europe/Monaco
    {  383, 3537,  109279,  6, }, // Europe/Moscow
    {  383, 6458,   12974,  4, }, // Europe/Oslo
    {  383, 4284,  109285,  5, }, // Europe/Paris
    {  383, 6424,  109290,  9, }, // Europe/Podgorica
    {  383, 6275,  109299,  5, }, // Europe/Prague
    {  383, 8837,  109304,  4, }, // Europe/Riga
    {  383, 6324,   11592,  3, }, // Europe/Rome
    {  383, 4331,  109308,  6, }, // Europe/Samara
    {  383, 6470,  109314, 10, }, // Europe/San_Marino
    {  383, 6245,  109324,  7, }, // Europe/Sarajevo
    {  383, 4451,  109331,  7, }, // Europe/Saratov
    {  383, 8849,  109338,  8, }, // Europe/Simferopol
    {  383, 6383,  109346,  6, }, // Europe/Skopje
    {  383, 6599,  109352,  5, }, // Europe/Sofia
    {  383, 6523,  109357,  9, }, // Europe/Stockholm
    {  383, 8867,  109366,  6, }, // Europe/Tallinn
    {  383, 6202,   13061,  6, }, // Europe/Tirane
    {  383, 8882,  109372,  9, }, // Europe/Ulyanovsk
    {  383, 6352,  109381,  5, }, // Europe/Vaduz
    {  383, 6567,  109386,  7, }, // Europe/Vatican
    {  383, 6231,  109393,  4, }, // Europe/Vienna
    {  383, 8899,  109397,  6, }, // Europe/Vilnius
    {  383, 4727,  109403,  9, }, // Europe/Volgograd
    {  383, 3190,  109412,  7, }, // Europe/Warsaw
    {  383, 6261,  109419,  6, }, // Europe/Zagreb
    {  383, 6540,  109425,  6, }, // Europe/Zurich
    {  383, 5078,  109431, 12, }, // Indian/Antananarivo
    {  383, 6947,  105952,  5, }, // Indian/Chagos
    {  383, 6046,  109443, 17, }, // Indian/Christmas
    {  383, 6063,  109460, 14, }, // Indian/Cocos
    {  383, 5029,  109474, 14, }, // Indian/Comoro
    {  383, 6661,  109488,  8, }, // Indian/Kerguelen
    {  383, 7260,  109496,  3, }, // Indian/Mahe
    {  383, 7050,  109499, 15, }, // Indian/Maldives
    {  383, 4069,  109514,  7, }, // Indian/Mauritius
    {  383, 5098,  109521,  7, }, // Indian/Mayotte
    {  383, 7211,  109528,  8, }, // Indian/Reunion
    {  383, 4422,  109536,  4, }, // Pacific/Apia
    {  383,  932,  109540,  6, }, // Pacific/Auckland
    {  383, 3706,  109546,  8, }, // Pacific/Bougainville
    {  383, 3015,  109554,  5, }, // Pacific/Chatham
    {  383, 3157,  109559,  4, }, // Pacific/Chuuk
    {  383, 2237,  109563, 11, }, // Pacific/Easter
    {  383, 7360,  109574,  5, }, // Pacific/Efate
    {  383, 7327,   13239,  7, }, // Pacific/Fakaofo
    {  383, 3902,   13246,  4, }, // Pacific/Fiji
    {  383, 7343,  109579,  8, }, // Pacific/Funafuti
    {  383, 6678,  109587,  9, }, // Pacific/Galapagos
    {  383, 6696,  109596,  6, }, // Pacific/Gambier
    {  383, 3822,  109602, 11, }, // Pacific/Guadalcanal
    {  383, 6018,  107748,  4, }, // Pacific/Guam
    {  383, 3049,  109613,  6, }, // Pacific/Kanton
    {  383, 3998,  109619, 10, }, // Pacific/Kiritimati
    {  383, 7019,  109629,  6, }, // Pacific/Kosrae
    {  383, 2842,  109635, 10, }, // Pacific/Kwajalein
    {  383, 4877,  109645,  6, }, // Pacific/Majuro
    {  383, 4051,  109651, 15, }, // Pacific/Marquesas
    {  383, 8914,  109666,  6, }, // Pacific/Midway
    {  383, 7084,  109672,  5, }, // Pacific/Nauru
    {  383, 7132,  109677,  4, }, // Pacific/Niue
    {  383, 4168,   13348,  7, }, // Pacific/Norfolk
    {  383, 7098,  109681,  5, }, // Pacific/Noumea
    {  383, 3126,  109686,  9, }, // Pacific/Pago_Pago
    {  383, 7155,  109695,  5, }, // Pacific/Palau
    {  383, 7194,  109700,  7, }, // Pacific/Pitcairn
    {  383, 3096,  109707,  6, }, // Pacific/Pohnpei
    {  383, 4807,  109713, 11, }, // Pacific/Port_Moresby
    {  383, 6076,  109724,  9, }, // Pacific/Rarotonga
    {  383, 6031,  109733,  6, }, // Pacific/Saipan
    {  383, 7312,  109739,  5, }, // Pacific/Tahiti
    {  383, 6712,  109744,  6, }, // Pacific/Tarawa
    {  383, 4583,  109750,  9, }, // Pacific/Tongatapu
    {  383, 7392,  109759,  4, }, // Pacific/Wake
    {  383, 7405,  109763,  6, }, // Pacific/Wallis
    {  384,   14,       0,  6, }, // Africa/Asmara Kenyang/Latin/Cameroon
    {  384, 4435,       6,  8, }, // Africa/Sao_Tome
    {  384, 4252,      14,  8, }, // America/Asuncion
    {  384,  237,      22,  8, }, // America/Atikokan
    {  384, 7623,      30, 17, }, // America/Bahia_Banderas
    {  384, 3868,      47,  6, }, // America/Cancun
    {  384, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  384, 5723,      66,  7, }, // America/Curacao
    {  384,  481,      73, 13, }, // America/Indiana/Knox
    {  384, 7961,      86, 16, }, // America/Indiana/Marengo
    {  384, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  384, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  384, 8038,     139, 14, }, // America/Indiana/Vevay
    {  384, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  384, 8086,     171, 16, }, // America/Indiana/Winamac
    {  384, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  384, 5932,     207, 22, }, // America/Lower_Princes
    {  384, 8220,     229,  6, }, // America/Merida
    {  384, 1850,     235, 19, }, // America/Noronha
    {  384, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  384, 8329,     274, 20, }, // America/North_Dakota/Center
    {  384, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  384,  393,     317,  4, }, // America/Nuuk
    {  384, 6899,     321, 16, }, // America/Scoresbysund
    {  384, 8508,     337, 14, }, // America/St_Barthelemy
    {  384, 2061,     351, 10, }, // America/St_Johns
    {  384, 5863,     361,  9, }, // America/St_Kitts
    {  384, 5880,     370,  9, }, // America/St_Lucia
    {  384,  855,     379, 10, }, // America/St_Thomas
    {  384, 5913,     389, 11, }, // America/St_Vincent
    {  384, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  384, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  384, 1185,     448,  9, }, // Asia/Kathmandu
    {  384,  992,     457,  7, }, // Asia/Kolkata
    {  384, 1211,     464,  5, }, // Asia/Macau
    {  384, 1235,     477,  6, }, // Asia/Yangon
    {  384, 1395,     483,  5, }, // Atlantic/Faroe
    {  384, 6821,     488, 10, }, // Atlantic/St_Helena
    {  384, 2527,     514,  4, }, // Europe/Kyiv
    {  384, 7211,     547,  7, }, // Indian/Reunion
    {  384, 3157,     554,  5, }, // Pacific/Chuuk
    {  384, 3096,     586,  7, }, // Pacific/Pohnpei
    {  385, 6788,  109769, 10, }, // Africa/Abidjan Khmer/Khmer/Cambodia
    {  385, 6760,  109779,  6, }, // Africa/Accra
    {  385, 5059,  109785, 11, }, // Africa/Addis_Ababa
    {  385, 7420,  109796,  8, }, // Africa/Algiers
    {  385,   14,  109804,  9, }, // Africa/Asmara
    {  385,   44,  109813,  7, }, // Africa/Bamako
    {  385, 5257,  109820,  7, }, // Africa/Bangui
    {  385, 6746,  109827,  9, }, // Africa/Banjul
    {  385, 7435,  109836,  7, }, // Africa/Bissau
    {  385, 4957,  109843, 11, }, // Africa/Blantyre
    {  385, 5287,  109854, 11, }, // Africa/Brazzaville
    {  385, 4922,  109865, 13, }, // Africa/Bujumbura
    {  385, 2320,  109878,  3, }, // Africa/Cairo
    {  385, 4117,  109881, 12, }, // Africa/Casablanca
    {  385, 7449,  109893,  4, }, // Africa/Ceuta
    {  385, 6773,  109897,  8, }, // Africa/Conakry
    {  385, 6840,  109905,  4, }, // Africa/Dakar
    {  385, 5130,  109909, 10, }, // Africa/Dar_es_Salaam
    {  385, 5043,  109919,  7, }, // Africa/Djibouti
    {  385, 5243,  109926,  7, }, // Africa/Douala
    {  385, 5166,  109933, 11, }, // Africa/El_Aaiun
    {  385, 6853,  109944,  9, }, // Africa/Freetown
    {  385, 4906,  109953, 10, }, // Africa/Gaborone
    {  385, 5015,  109963,  8, }, // Africa/Harare
    {  385, 4479,  109971, 13, }, // Africa/Johannesburg
    {  385, 4499,  109984,  4, }, // Africa/Juba
    {  385, 5151,  109988,  7, }, // Africa/Kampala
    {  385, 4524,  109995,  5, }, // Africa/Khartoum
    {  385, 4973,  110000,  8, }, // Africa/Kigali
    {  385, 5306,  110008,  9, }, // Africa/Kinshasa
    {  385, 4744,  110017,  7, }, // Africa/Lagos
    {  385, 5336,  110024,  9, }, // Africa/Libreville
    {  385, 6869,  110033,  5, }, // Africa/Lome
    {  385, 5211,  110038,  7, }, // Africa/Luanda
    {  385, 4939,  110045, 11, }, // Africa/Lubumbashi
    {  385, 5001,  110056,  6, }, // Africa/Lusaka
    {  385, 5322,  110062,  7, }, // Africa/Malabo
    {  385, 4987,  110069,  8, }, // Africa/Maputo
    {  385, 5197,  110077,  8, }, // Africa/Maseru
    {  385, 5182,  110085,  5, }, // Africa/Mbabane
    {  385, 5113,  110090, 13, }, // Africa/Mogadishu
    {  385, 7034,  110103, 12, }, // Africa/Monrovia
    {  385, 3853,  110115,  8, }, // Africa/Nairobi
    {  385, 5271,  110123,  8, }, // Africa/Ndjamena
    {  385, 5354,  110131,  8, }, // Africa/Niamey
    {  385, 6803,  110139,  6, }, // Africa/Nouakchott
    {  385, 6727,  110145, 12, }, // Africa/Ouagadougou
    {  385, 5225,  110157, 10, }, // Africa/Porto-Novo
    {  385, 4435,  110167,  7, }, // Africa/Sao_Tome
    {  385, 2866,  110174,  9, }, // Africa/Tripoli
    {  385, 6554,  110183,  6, }, // Africa/Tunis
    {  385, 4152,  110189,  6, }, // Africa/Windhoek
    {  385,  132,  110195,  5, }, // America/Adak
    {  385, 3346,  110200,  9, }, // America/Anchorage
    {  385, 5607,  110209,  9, }, // America/Anguilla
    {  385, 5624,  110218,  9, }, // America/Antigua
    {  385, 4554,  110227, 12, }, // America/Araguaina
    {  385,  166,  110239,  9, }, // America/Argentina/Buenos_Aires
    {  385,   91,  110248,  9, }, // America/Argentina/Catamarca
    {  385,  270,  110257,  6, }, // America/Argentina/Cordoba
    {  385,  441,  110263,  5, }, // America/Argentina/Jujuy
    {  385, 7462,  110268,  9, }, // America/Argentina/La_Rioja
    {  385,  565,  110277, 10, }, // America/Argentina/Mendoza
    {  385, 7489,  110287, 16, }, // America/Argentina/Rio_Gallegos
    {  385, 7520,  110303,  6, }, // America/Argentina/Salta
    {  385, 7544,  110309,  9, }, // America/Argentina/San_Juan
    {  385, 5580,  110318, 10, }, // America/Argentina/San_Luis
    {  385, 7571,  110328,  8, }, // America/Argentina/Tucuman
    {  385, 7597,  110336,  8, }, // America/Argentina/Ushuaia
    {  385, 5640,  110344,  7, }, // America/Aruba
    {  385, 4252,  110351, 10, }, // America/Asuncion
    {  385,  237,  110361,  9, }, // America/Atikokan
    {  385, 3679,  110370,  4, }, // America/Bahia
    {  385, 7623,  110374, 13, }, // America/Bahia_Banderas
    {  385, 5654,  110387,  7, }, // America/Barbados
    {  385, 7646,  110394,  5, }, // America/Belem
    {  385, 5407,  110399,  4, }, // America/Belize
    {  385, 7660,  110403, 13, }, // America/Blanc-Sablon
    {  385, 7681,  110416,  9, }, // America/Boa_Vista
    {  385, 4361,  110425,  8, }, // America/Bogota
    {  385, 7699,  110433,  6, }, // America/Boise
    {  385, 7713,  110439, 10, }, // America/Cambridge_Bay
    {  385, 7735,  110449, 14, }, // America/Campo_Grande
    {  385, 3868,  110463,  6, }, // America/Cancun
    {  385, 4694,  110469,  8, }, // America/Caracas
    {  385, 4345,  110477,  6, }, // America/Cayenne
    {  385, 5496,  110483,  5, }, // America/Cayman
    {  385, 2260,  110488,  9, }, // America/Chicago
    {  385, 7756,  110497,  6, }, // America/Chihuahua
    {  385, 7774,  110503, 19, }, // America/Ciudad_Juarez
    {  385, 5422,  110522, 10, }, // America/Costa_Rica
    {  385, 7796,  110532,  9, }, // America/Creston
    {  385, 3791,  110541,  7, }, // America/Cuiaba
    {  385, 5723,  110548,  7, }, // America/Curacao
    {  385, 7812,  110555, 14, }, // America/Danmarkshavn
    {  385, 7833,  110569,  6, }, // America/Dawson
    {  385, 7848,  110575, 12, }, // America/Dawson_Creek
    {  385,  805,  110587,  5, }, // America/Denver
    {  385, 3465,  110592, 10, }, // America/Detroit
    {  385, 5739,  110602,  7, }, // America/Dominica
    {  385,  893,  110609, 10, }, // America/Edmonton
    {  385, 7869,  110619, 11, }, // America/Eirunepe
    {  385, 5441,  110630, 13, }, // America/El_Salvador
    {  385, 7886,  110643, 11, }, // America/Fort_Nelson
    {  385, 7906,  110654, 14, }, // America/Fortaleza
    {  385, 7924,  110668,  8, }, // America/Glace_Bay
    {  385, 6881,  110676,  6, }, // America/Goose_Bay
    {  385, 4612,  110682, 11, }, // America/Grand_Turk
    {  385, 5770,  110693,  9, }, // America/Grenada
    {  385, 5786,  110702, 11, }, // America/Guadeloupe
    {  385, 3760,  110713, 11, }, // America/Guatemala
    {  385, 4841,  110724, 10, }, // America/Guayaquil
    {  385, 6932,  110734,  8, }, // America/Guyana
    {  385, 1939,  110742,  9, }, // America/Halifax
    {  385, 2281,  110751,  8, }, // America/Havana
    {  385, 7942,  110759, 11, }, // America/Hermosillo
    {  385,  348,  110770, 13, }, // America/Indiana/Indianapolis
    {  385,  481,  110783, 11, }, // America/Indiana/Knox
    {  385, 7961,  110794, 16, }, // America/Indiana/Marengo
    {  385, 7985,  110810, 17, }, // America/Indiana/Petersburg
    {  385, 8012,  110827, 18, }, // America/Indiana/Tell_City
    {  385, 8038,  110845, 16, }, // America/Indiana/Vevay
    {  385, 8060,  110861, 16, }, // America/Indiana/Vincennes
    {  385, 8086,  110877, 17, }, // America/Indiana/Winamac
    {  385, 8110,  110894,  8, }, // America/Inuvik
    {  385,  660,  110902, 11, }, // America/Iqaluit
    {  385, 2799,  110913, 11, }, // America/Jamaica
    {  385, 5380,  110924,  6, }, // America/Juneau
    {  385,  521,  110930,  8, }, // America/Kentucky/Louisville
    {  385, 8125,  110938, 17, }, // America/Kentucky/Monticello
    {  385, 5704,  110955, 10, }, // America/Kralendijk
    {  385, 4376,  110965,  6, }, // America/La_Paz
    {  385, 7169,  110971,  5, }, // America/Lima
    {  385, 3239,  110976, 12, }, // America/Los_Angeles
    {  385, 5932,  110988, 19, }, // America/Lower_Princes
    {  385, 8153,  111007,  8, }, // America/Maceio
    {  385, 8168,  111015,  9, }, // America/Managua
    {  385, 1908,  111024,  9, }, // America/Manaus
    {  385, 5897,  111033, 10, }, // America/Marigot
    {  385, 5805,  111043,  8, }, // America/Martinique
    {  385, 8184,  111051, 12, }, // America/Matamoros
    {  385, 2917,  111063, 11, }, // America/Mazatlan
    {  385, 8202,  111074,  9, }, // America/Menominee
    {  385, 8220,  111083,  7, }, // America/Merida
    {  385, 8235,  111090, 13, }, // America/Metlakatla
    {  385, 2949,  111103, 16, }, // America/Mexico_City
    {  385, 4391,  111119,  9, }, // America/Miquelon
    {  385, 8254,  111128,  7, }, // America/Moncton
    {  385, 8270,  111135,  9, }, // America/Monterrey
    {  385, 4098,  111144, 12, }, // America/Montevideo
    {  385, 5824,  111156, 12, }, // America/Montserrat
    {  385, 5481,  111168,  6, }, // America/Nassau
    {  385, 2379,  111174,  6, }, // America/New_York
    {  385, 8288,  111077,  3, }, // America/Nome
    {  385, 1850,  111180,  9, }, // America/Noronha
    {  385, 8301,  111189, 19, }, // America/North_Dakota/Beulah
    {  385, 8329,  111208, 20, }, // America/North_Dakota/Center
    {  385, 8357,  111228, 24, }, // America/North_Dakota/New_Salem
    {  385,  393,  111252,  3, }, // America/Nuuk
    {  385, 8388,  111255, 10, }, // America/Ojinaga
    {  385, 2356,  111265,  8, }, // America/Panama
    {  385, 6173,  111273, 13, }, // America/Paramaribo
    {  385, 2973,  111286,  5, }, // America/Phoenix
    {  385, 3945,  111291, 12, }, // America/Port-au-Prince
    {  385, 5954,  111303, 11, }, // America/Port_of_Spain
    {  385, 8404,  111314, 11, }, // America/Porto_Velho
    {  385, 5843,  111325,  9, }, // America/Puerto_Rico
    {  385, 4030,  111334, 13, }, // America/Punta_Arenas
    {  385, 8424,  111347, 16, }, // America/Rankin_Inlet
    {  385, 8445,  111363, 10, }, // America/Recife
    {  385, 1995,  111373,  9, }, // America/Regina
    {  385, 8460,  111382, 10, }, // America/Resolute
    {  385,  695,  111392, 12, }, // America/Rio_Branco
    {  385, 8477,  111404,  9, }, // America/Santarem
    {  385, 2201,  111413, 12, }, // America/Santiago
    {  385, 6111,  111425, 11, }, // America/Santo_Domingo
    {  385, 1878,  111436,  8, }, // America/Sao_Paulo
    {  385, 6899,  111444, 13, }, // America/Scoresbysund
    {  385, 8494,  111457,  6, }, // America/Sitka
    {  385, 8508,  111463, 14, }, // America/St_Barthelemy
    {  385, 2061,  111477,  7, }, // America/St_Johns
    {  385, 5863,  111484,  8, }, // America/St_Kitts
    {  385, 5880,  111492,  9, }, // America/St_Lucia
    {  385,  855,  111501, 12, }, // America/St_Thomas
    {  385, 5913,  111513, 11, }, // America/St_Vincent
    {  385, 8530,  111524, 11, }, // America/Swift_Current
    {  385, 5461,  111535, 18, }, // America/Tegucigalpa
    {  385, 5756,  111553,  4, }, // America/Thule
    {  385,  313,  111557, 10, }, // America/Tijuana
    {  385,  608,  111567,  8, }, // America/Toronto
    {  385, 5688,  111575,  6, }, // America/Tortola
    {  385, 2093,  111581,  9, }, // America/Vancouver
    {  385, 2144,  111590,  6, }, // America/Whitehorse
    {  385,  734,  111596,  7, }, // America/Winnipeg
    {  385, 8552,  111603,  8, }, // America/Yakutat
    {  385, 6001,  111611,  4, }, // Antarctica/Casey
    {  385, 6094,  111615,  5, }, // Antarctica/Davis
    {  385, 6133,  111620, 16, }, // Antarctica/DumontDUrville
    {  385, 8568,  111636, 10, }, // Antarctica/Macquarie
    {  385, 7066,  111646,  7, }, // Antarctica/Mawson
    {  385, 7113,  111653, 10, }, // Antarctica/McMurdo
    {  385, 8589,  111663,  7, }, // Antarctica/Palmer
    {  385, 7226,  111670,  8, }, // Antarctica/Rothera
    {  385, 7295,  111678,  9, }, // Antarctica/Syowa
    {  385, 8607,  111687,  5, }, // Antarctica/Troll
    {  385, 7374,  111692,  8, }, // Antarctica/Vostok
    {  385, 1429,  111700, 14, }, // Arctic/Longyearbyen
    {  385, 5570,  111714,  5, }, // Asia/Aden
    {  385, 5395,  111719,  9, }, // Asia/Almaty
    {  385, 3968,  111728,  7, }, // Asia/Amman
    {  385, 5511,  111735,  6, }, // Asia/Anadyr
    {  385, 5523,  111741,  5, }, // Asia/Aqtau
    {  385, 5368,  111746,  8, }, // Asia/Aqtobe
    {  385,  964,  111754, 10, }, // Asia/Ashgabat
    {  385, 8624,  111764,  7, }, // Asia/Atyrau
    {  385, 3607,  111771,  6, }, // Asia/Baghdad
    {  385, 5534,  111777,  6, }, // Asia/Bahrain
    {  385, 3653,  111783,  4, }, // Asia/Baku
    {  385, 4466,  111787,  5, }, // Asia/Bangkok
    {  385, 3571,  111792,  5, }, // Asia/Barnaul
    {  385, 4086,  111797,  5, }, // Asia/Beirut
    {  385, 3778,  111802,  8, }, // Asia/Bishkek
    {  385, 5989,  111810,  7, }, // Asia/Brunei
    {  385, 4601,  111817,  4, }, // Asia/Chita
    {  385, 4511,  111821,  7, }, // Asia/Colombo
    {  385, 4540,  111828,  6, }, // Asia/Damascus
    {  385, 1093,  111834,  6, }, // Asia/Dhaka
    {  385, 6192,  111840,  4, }, // Asia/Dili
    {  385, 3596,  111844,  5, }, // Asia/Dubai
    {  385, 6159,  111849, 10, }, // Asia/Dushanbe
    {  385, 8636,  111859, 16, }, // Asia/Famagusta
    {  385, 8651,  109881,  4, }, // Asia/Gaza
    {  385, 4795,  111875,  6, }, // Asia/Hebron
    {  385, 1259,  111881,  7, }, // Asia/Ho_Chi_Minh
    {  385, 2704,  111888,  6, }, // Asia/Hong_Kong
    {  385, 4771,  111894,  3, }, // Asia/Hovd
    {  385, 4184,  111897, 11, }, // Asia/Irkutsk
    {  385, 7006,  111908,  9, }, // Asia/Jakarta
    {  385, 6992,  111917, 10, }, // Asia/Jayapura
    {  385, 1290,  111927, 13, }, // Asia/Jerusalem
    {  385, 3560,  111940,  6, }, // Asia/Kabul
    {  385, 4316,  111946,  8, }, // Asia/Kamchatka
    {  385, 4239,  111954,  7, }, // Asia/Karachi
    {  385, 1185,  111961, 11, }, // Asia/Kathmandu
    {  385, 8661,  111972, 10, }, // Asia/Khandyga
    {  385,  992,  111982,  7, }, // Asia/Kolkata
    {  385, 4197,  111989, 11, }, // Asia/Krasnoyarsk
    {  385, 4859,  112000, 10, }, // Asia/Kuala_Lumpur
    {  385, 5976,  112010,  6, }, // Asia/Kuching
    {  385, 5547,  112016,  6, }, // Asia/Kuwait
    {  385, 1211,  112022,  6, }, // Asia/Macau
    {  385, 4017,  112028, 10, }, // Asia/Magadan
    {  385, 1349,  112038,  9, }, // Asia/Makassar
    {  385, 7182,  112047,  6, }, // Asia/Manila
    {  385, 6920,  112053,  8, }, // Asia/Muscat
    {  385, 2554,  112061,  7, }, // Asia/Nicosia
    {  385, 8675,  112068, 16, }, // Asia/Novokuznetsk
    {  385, 4135,  112084, 12, }, // Asia/Novosibirsk
    {  385, 4229,  112096,  8, }, // Asia/Omsk
    {  385, 7145,  112104,  7, }, // Asia/Oral
    {  385, 6961,  112111,  7, }, // Asia/Phnom_Penh
    {  385, 8693,  112118, 12, }, // Asia/Pontianak
    {  385, 4214,  112130,  9, }, // Asia/Pyongyang
    {  385, 5559,  110248,  4, }, // Asia/Qatar
    {  385, 8708,  112139,  8, }, // Asia/Qostanay
    {  385, 4269,  112147, 11, }, // Asia/Qyzylorda
    {  385, 3584,  112158,  6, }, // Asia/Riyadh
    {  385, 4408,  112164,  7, }, // Asia/Sakhalin
    {  385, 7245,  112171,  9, }, // Asia/Samarkand
    {  385, 3279,  112180,  6, }, // Asia/Seoul
    {  385, 1053,  112186,  5, }, // Asia/Shanghai
    {  385, 3300,  112191,  9, }, // Asia/Singapore
    {  385, 4297,  112200, 12, }, // Asia/Srednekolymsk
    {  385, 3263,  112212,  5, }, // Asia/Taipei
    {  385, 4781,  112217,  7, }, // Asia/Tashkent
    {  385, 3915,  112224,  8, }, // Asia/Tbilisi
    {  385, 2772,  112232,  8, }, // Asia/Tehran
    {  385, 1317,  112240,  5, }, // Asia/Thimphu
    {  385, 2821,  112245,  6, }, // Asia/Tokyo
    {  385, 4572,  112251,  7, }, // Asia/Tomsk
    {  385, 1021,  112258, 11, }, // Asia/Ulaanbaatar
    {  385, 1159,  112269, 10, }, // Asia/Urumqi
    {  385, 8722,  112279, 10, }, // Asia/Ust-Nera
    {  385, 6977,  112289,  9, }, // Asia/Vientiane
    {  385, 4710,  112298, 15, }, // Asia/Vladivostok
    {  385, 4828,  112313, 11, }, // Asia/Yakutsk
    {  385, 1235,  112324,  9, }, // Asia/Yangon
    {  385, 3883,  112333, 14, }, // Asia/Yekaterinburg
    {  385, 3747,  112347, 11, }, // Asia/Yerevan
    {  385, 3663,  112358,  8, }, // Atlantic/Azores
    {  385, 5671,  112366,  8, }, // Atlantic/Bermuda
    {  385, 6628,  112374,  6, }, // Atlantic/Canary
    {  385, 3727,  112380,  7, }, // Atlantic/Cape_Verde
    {  385, 1395,  112387,  7, }, // Atlantic/Faroe
    {  385, 8736,  112394,  8, }, // Atlantic/Madeira
    {  385, 2748,  112402, 10, }, // Atlantic/Reykjavik
    {  385, 7272,  112412, 19, }, // Atlantic/South_Georgia
    {  385, 6821,  112431, 10, }, // Atlantic/St_Helena
    {  385, 6644,  112441,  7, }, // Atlantic/Stanley
    {  385, 1670,  112448,  7, }, // Australia/Adelaide
    {  385, 1635,  112455,  9, }, // Australia/Brisbane
    {  385, 1799,  112464, 11, }, // Australia/Broken_Hill
    {  385, 1583,  112475,  5, }, // Australia/Darwin
    {  385, 3637,  112480,  8, }, // Australia/Eucla
    {  385, 1516,  112488,  5, }, // Australia/Hobart
    {  385, 8753,  112493,  9, }, // Australia/Lindeman
    {  385, 1547,  112502,  7, }, // Australia/Lord_Howe
    {  385, 1727,  112509,  7, }, // Australia/Melbourne
    {  385, 1762,  112516,  3, }, // Australia/Perth
    {  385, 1463,  112519,  6, }, // Australia/Sydney
    {  385, 6441,  112525, 10, }, // Europe/Amsterdam
    {  385, 6216,  112535,  8, }, // Europe/Andorra
    {  385, 3620,  112543, 12, }, // Europe/Astrakhan
    {  385, 2300,  112555,  5, }, // Europe/Athens
    {  385, 6336,  112560,  8, }, // Europe/Belgrade
    {  385, 4757,  112568,  7, }, // Europe/Berlin
    {  385, 6488,  112575, 13, }, // Europe/Bratislava
    {  385, 2167,  112588,  8, }, // Europe/Brussels
    {  385, 3928,  112596, 10, }, // Europe/Bucharest
    {  385, 3806,  112606, 10, }, // Europe/Budapest
    {  385, 8772,  112616, 12, }, // Europe/Busingen
    {  385, 2583,  112628,  7, }, // Europe/Chisinau
    {  385, 6289,  112635,  9, }, // Europe/Copenhagen
    {  385, 2338,  112644,  9, }, // Europe/Dublin
    {  385, 6307,  112653, 13, }, // Europe/Gibraltar
    {  385, 8788,  112666,  8, }, // Europe/Guernsey
    {  385, 6612,  112674,  9, }, // Europe/Helsinki
    {  385, 8804,  112683, 12, }, // Europe/Isle_of_Man
    {  385, 1130,  112695, 12, }, // Europe/Istanbul
    {  385, 8823,  112707,  5, }, // Europe/Jersey
    {  385, 3979,  112712, 14, }, // Europe/Kaliningrad
    {  385, 8929,  112726,  6, }, // Europe/Kirov
    {  385, 2527,  112732,  3, }, // Europe/Kyiv
    {  385, 3213,  112735,  6, }, // Europe/Lisbon
    {  385, 6506,  112741,  9, }, // Europe/Ljubljana
    {  385, 2501,  112750,  5, }, // Europe/London
    {  385, 6365,  112755,  7, }, // Europe/Luxembourg
    {  385, 4892,  112762,  8, }, // Europe/Madrid
    {  385, 6397,  112770,  7, }, // Europe/Malta
    {  385, 6582,  112777,  8, }, // Europe/Mariehamn
    {  385, 3693,  112785,  8, }, // Europe/Minsk
    {  385, 6410,  112793,  7, }, // Europe/Monaco
    {  385, 3537,  112800,  6, }, // Europe/Moscow
    {  385, 6458,  112806,  6, }, // Europe/Oslo
    {  385, 4284,  112812,  6, }, // Europe/Paris
    {  385, 6424,  112818, 12, }, // Europe/Podgorica
    {  385, 6275,  112830,  5, }, // Europe/Prague
    {  385, 8837,  112835,  6, }, // Europe/Riga
    {  385, 6324,  112391,  4, }, // Europe/Rome
    {  385, 4331,  112841,  8, }, // Europe/Samara
    {  385, 6470,  112849, 12, }, // Europe/San_Marino
    {  385, 6245,  112861, 11, }, // Europe/Sarajevo
    {  385, 4451,  112872,  8, }, // Europe/Saratov
    {  385, 8849,  112880, 15, }, // Europe/Simferopol
    {  385, 6383,  112895,  7, }, // Europe/Skopje
    {  385, 6599,  112902, 10, }, // Europe/Sofia
    {  385, 6523,  112912,  8, }, // Europe/Stockholm
    {  385, 8867,  112920,  5, }, // Europe/Tallinn
    {  385, 6202,  112925,  7, }, // Europe/Tirane
    {  385, 8882,  112932, 14, }, // Europe/Ulyanovsk
    {  385, 6352,  112946,  6, }, // Europe/Vaduz
    {  385, 6567,  112952,  8, }, // Europe/Vatican
    {  385, 6231,  112960,  5, }, // Europe/Vienna
    {  385, 8899,  112965,  8, }, // Europe/Vilnius
    {  385, 4727,  112973, 15, }, // Europe/Volgograd
    {  385, 3190,  112988,  7, }, // Europe/Warsaw
    {  385, 6261,  112995,  7, }, // Europe/Zagreb
    {  385, 6540,  113002,  8, }, // Europe/Zurich
    {  385, 5078,  113010, 12, }, // Indian/Antananarivo
    {  385, 6947,  110491,  6, }, // Indian/Chagos
    {  385, 6046,  113022,  9, }, // Indian/Christmas
    {  385, 6063,  113031,  5, }, // Indian/Cocos
    {  385, 5029,  113036,  5, }, // Indian/Comoro
    {  385, 6661,  113041, 10, }, // Indian/Kerguelen
    {  385, 7260,  113051,  5, }, // Indian/Mahe
    {  385, 7050,  113056,  8, }, // Indian/Maldives
    {  385, 4069,  113064,  6, }, // Indian/Mauritius
    {  385, 5098,  113070,  6, }, // Indian/Mayotte
    {  385, 7211,  113076,  9, }, // Indian/Reunion
    {  385, 4422,  113085,  4, }, // Pacific/Apia
    {  385,  932,  113089,  5, }, // Pacific/Auckland
    {  385, 3706,  113094, 10, }, // Pacific/Bougainville
    {  385, 3015,  113104,  5, }, // Pacific/Chatham
    {  385, 3157,  113109,  5, }, // Pacific/Chuuk
    {  385, 2237,  113114,  8, }, // Pacific/Easter
    {  385, 7360,  113122,  9, }, // Pacific/Efate
    {  385, 7327,  113131, 10, }, // Pacific/Fakaofo
    {  385, 3902,  113141,  6, }, // Pacific/Fiji
    {  385, 7343,  113147, 14, }, // Pacific/Funafuti
    {  385, 6678,  113161, 14, }, // Pacific/Galapagos
    {  385, 6696,  113175,  7, }, // Pacific/Gambier
    {  385, 3822,  113182, 14, }, // Pacific/Guadalcanal
    {  385, 6018,  113175,  5, }, // Pacific/Guam
    {  385, 2723,  113196,  8, }, // Pacific/Honolulu
    {  385, 3049,  113204, 11, }, // Pacific/Kanton
    {  385, 3998,  113215, 11, }, // Pacific/Kiritimati
    {  385, 7019,  113226,  6, }, // Pacific/Kosrae
    {  385, 2842,  113232, 11, }, // Pacific/Kwajalein
    {  385, 4877,  113243, 10, }, // Pacific/Majuro
    {  385, 4051,  113253, 10, }, // Pacific/Marquesas
    {  385, 8914,  113263,  6, }, // Pacific/Midway
    {  385, 7084,  113269,  4, }, // Pacific/Nauru
    {  385, 7132,  113273,  5, }, // Pacific/Niue
    {  385, 4168,  113278,  8, }, // Pacific/Norfolk
    {  385, 7098,  113286,  4, }, // Pacific/Noumea
    {  385, 3126,  113290, 15, }, // Pacific/Pago_Pago
    {  385, 7155,  113305,  5, }, // Pacific/Palau
    {  385, 7194,  113310,  9, }, // Pacific/Pitcairn
    {  385, 3096,  113319,  6, }, // Pacific/Pohnpei
    {  385, 4807,  113325, 12, }, // Pacific/Port_Moresby
    {  385, 6076,  113337, 13, }, // Pacific/Rarotonga
    {  385, 6031,  113350,  6, }, // Pacific/Saipan
    {  385, 7312,  113356,  6, }, // Pacific/Tahiti
    {  385, 6712,  113362,  8, }, // Pacific/Tarawa
    {  385, 4583,  113370, 11, }, // Pacific/Tongatapu
    {  385, 7392,  113381,  5, }, // Pacific/Wake
    {  385, 7405,  113386,  6, }, // Pacific/Wallis
    {  386,   14,       0,  6, }, // Africa/Asmara Kiche/Latin/Guatemala
    {  386, 4435,       6,  8, }, // Africa/Sao_Tome
    {  386, 4252,      14,  8, }, // America/Asuncion
    {  386,  237,      22,  8, }, // America/Atikokan
    {  386, 7623,      30, 17, }, // America/Bahia_Banderas
    {  386, 3868,      47,  6, }, // America/Cancun
    {  386, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  386, 5723,      66,  7, }, // America/Curacao
    {  386,  481,      73, 13, }, // America/Indiana/Knox
    {  386, 7961,      86, 16, }, // America/Indiana/Marengo
    {  386, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  386, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  386, 8038,     139, 14, }, // America/Indiana/Vevay
    {  386, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  386, 8086,     171, 16, }, // America/Indiana/Winamac
    {  386, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  386, 5932,     207, 22, }, // America/Lower_Princes
    {  386, 8220,     229,  6, }, // America/Merida
    {  386, 1850,     235, 19, }, // America/Noronha
    {  386, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  386, 8329,     274, 20, }, // America/North_Dakota/Center
    {  386, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  386,  393,     317,  4, }, // America/Nuuk
    {  386, 6899,     321, 16, }, // America/Scoresbysund
    {  386, 8508,     337, 14, }, // America/St_Barthelemy
    {  386, 2061,     351, 10, }, // America/St_Johns
    {  386, 5863,     361,  9, }, // America/St_Kitts
    {  386, 5880,     370,  9, }, // America/St_Lucia
    {  386,  855,     379, 10, }, // America/St_Thomas
    {  386, 5913,     389, 11, }, // America/St_Vincent
    {  386, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  386, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  386, 1185,     448,  9, }, // Asia/Kathmandu
    {  386,  992,     457,  7, }, // Asia/Kolkata
    {  386, 1211,     464,  5, }, // Asia/Macau
    {  386, 1235,     477,  6, }, // Asia/Yangon
    {  386, 1395,     483,  5, }, // Atlantic/Faroe
    {  386, 6821,     488, 10, }, // Atlantic/St_Helena
    {  386, 2527,     514,  4, }, // Europe/Kyiv
    {  386, 7211,     547,  7, }, // Indian/Reunion
    {  386, 3157,     554,  5, }, // Pacific/Chuuk
    {  386, 3096,     586,  7, }, // Pacific/Pohnpei
    {  387,   14,       0,  6, }, // Africa/Asmara Kikuyu/Latin/Kenya
    {  387, 4435,       6,  8, }, // Africa/Sao_Tome
    {  387, 4252,      14,  8, }, // America/Asuncion
    {  387,  237,      22,  8, }, // America/Atikokan
    {  387, 7623,      30, 17, }, // America/Bahia_Banderas
    {  387, 3868,      47,  6, }, // America/Cancun
    {  387, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  387, 5723,      66,  7, }, // America/Curacao
    {  387,  481,      73, 13, }, // America/Indiana/Knox
    {  387, 7961,      86, 16, }, // America/Indiana/Marengo
    {  387, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  387, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  387, 8038,     139, 14, }, // America/Indiana/Vevay
    {  387, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  387, 8086,     171, 16, }, // America/Indiana/Winamac
    {  387, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  387, 5932,     207, 22, }, // America/Lower_Princes
    {  387, 8220,     229,  6, }, // America/Merida
    {  387, 1850,     235, 19, }, // America/Noronha
    {  387, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  387, 8329,     274, 20, }, // America/North_Dakota/Center
    {  387, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  387,  393,     317,  4, }, // America/Nuuk
    {  387, 6899,     321, 16, }, // America/Scoresbysund
    {  387, 8508,     337, 14, }, // America/St_Barthelemy
    {  387, 2061,     351, 10, }, // America/St_Johns
    {  387, 5863,     361,  9, }, // America/St_Kitts
    {  387, 5880,     370,  9, }, // America/St_Lucia
    {  387,  855,     379, 10, }, // America/St_Thomas
    {  387, 5913,     389, 11, }, // America/St_Vincent
    {  387, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  387, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  387, 1185,     448,  9, }, // Asia/Kathmandu
    {  387,  992,     457,  7, }, // Asia/Kolkata
    {  387, 1211,     464,  5, }, // Asia/Macau
    {  387, 1235,     477,  6, }, // Asia/Yangon
    {  387, 1395,     483,  5, }, // Atlantic/Faroe
    {  387, 6821,     488, 10, }, // Atlantic/St_Helena
    {  387, 2527,     514,  4, }, // Europe/Kyiv
    {  387, 7211,     547,  7, }, // Indian/Reunion
    {  387, 3157,     554,  5, }, // Pacific/Chuuk
    {  387, 3096,     586,  7, }, // Pacific/Pohnpei
    {  388,   14,       0,  6, }, // Africa/Asmara Kinyarwanda/Latin/Rwanda
    {  388, 4435,       6,  8, }, // Africa/Sao_Tome
    {  388, 4252,      14,  8, }, // America/Asuncion
    {  388,  237,      22,  8, }, // America/Atikokan
    {  388, 7623,      30, 17, }, // America/Bahia_Banderas
    {  388, 3868,      47,  6, }, // America/Cancun
    {  388, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  388, 5723,      66,  7, }, // America/Curacao
    {  388,  481,      73, 13, }, // America/Indiana/Knox
    {  388, 7961,      86, 16, }, // America/Indiana/Marengo
    {  388, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  388, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  388, 8038,     139, 14, }, // America/Indiana/Vevay
    {  388, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  388, 8086,     171, 16, }, // America/Indiana/Winamac
    {  388, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  388, 5932,     207, 22, }, // America/Lower_Princes
    {  388, 8220,     229,  6, }, // America/Merida
    {  388, 1850,     235, 19, }, // America/Noronha
    {  388, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  388, 8329,     274, 20, }, // America/North_Dakota/Center
    {  388, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  388,  393,     317,  4, }, // America/Nuuk
    {  388, 6899,     321, 16, }, // America/Scoresbysund
    {  388, 8508,     337, 14, }, // America/St_Barthelemy
    {  388, 2061,     351, 10, }, // America/St_Johns
    {  388, 5863,     361,  9, }, // America/St_Kitts
    {  388, 5880,     370,  9, }, // America/St_Lucia
    {  388,  855,     379, 10, }, // America/St_Thomas
    {  388, 5913,     389, 11, }, // America/St_Vincent
    {  388, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  388, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  388, 1185,     448,  9, }, // Asia/Kathmandu
    {  388,  992,     457,  7, }, // Asia/Kolkata
    {  388, 1211,     464,  5, }, // Asia/Macau
    {  388, 1235,     477,  6, }, // Asia/Yangon
    {  388, 1395,     483,  5, }, // Atlantic/Faroe
    {  388, 6821,     488, 10, }, // Atlantic/St_Helena
    {  388, 2527,     514,  4, }, // Europe/Kyiv
    {  388, 7211,     547,  7, }, // Indian/Reunion
    {  388, 3157,     554,  5, }, // Pacific/Chuuk
    {  388, 3096,     586,  7, }, // Pacific/Pohnpei
    {  389, 6788,   29964,  7, }, // Africa/Abidjan Konkani/Devanagari/India
    {  389, 6760,  113392,  4, }, // Africa/Accra
    {  389, 5059,  113396, 10, }, // Africa/Addis_Ababa
    {  389, 7420,  113406, 10, }, // Africa/Algiers
    {  389,   14,   88276,  7, }, // Africa/Asmara
    {  389,   44,   88283,  6, }, // Africa/Bamako
    {  389, 5257,  113416,  5, }, // Africa/Bangui
    {  389, 6746,  113421,  6, }, // Africa/Banjul
    {  389, 7435,  113427,  5, }, // Africa/Bissau
    {  389, 4957,  113432,  9, }, // Africa/Blantyre
    {  389, 5287,  113441, 10, }, // Africa/Brazzaville
    {  389, 4922,  113451,  9, }, // Africa/Bujumbura
    {  389, 2320,   88335,  5, }, // Africa/Cairo
    {  389, 4117,   30065, 11, }, // Africa/Casablanca
    {  389, 7449,   88340,  5, }, // Africa/Ceuta
    {  389, 6773,  113460,  8, }, // Africa/Conakry
    {  389, 6840,   88353,  4, }, // Africa/Dakar
    {  389, 5130,  113468, 10, }, // Africa/Dar_es_Salaam
    {  389, 5043,   88368,  6, }, // Africa/Djibouti
    {  389, 5243,  113478,  5, }, // Africa/Douala
    {  389, 5166,  113483,  7, }, // Africa/El_Aaiun
    {  389, 6853,  113490,  8, }, // Africa/Freetown
    {  389, 4906,   88396,  7, }, // Africa/Gaborone
    {  389, 5015,   88403,  5, }, // Africa/Harare
    {  389, 4479,  113498, 11, }, // Africa/Johannesburg
    {  389, 4499,   30154,  4, }, // Africa/Juba
    {  389, 5151,   30158,  8, }, // Africa/Kampala
    {  389, 4524,  113509,  7, }, // Africa/Khartoum
    {  389, 4973,   88424,  6, }, // Africa/Kigali
    {  389, 5306,  113516,  7, }, // Africa/Kinshasa
    {  389, 4744,   88437,  5, }, // Africa/Lagos
    {  389, 5336,   88442, 10, }, // Africa/Libreville
    {  389, 6869,   88452,  3, }, // Africa/Lome
    {  389, 5211,   88455,  6, }, // Africa/Luanda
    {  389, 4939,   88461,  9, }, // Africa/Lubumbashi
    {  389, 5001,   30221,  6, }, // Africa/Lusaka
    {  389, 5322,   88470,  5, }, // Africa/Malabo
    {  389, 4987,   88475,  6, }, // Africa/Maputo
    {  389, 5197,  113523,  6, }, // Africa/Maseru
    {  389, 5182,   88481,  7, }, // Africa/Mbabane
    {  389, 5113,  113529,  8, }, // Africa/Mogadishu
    {  389, 7034,   88496,  9, }, // Africa/Monrovia
    {  389, 3853,  113537,  7, }, // Africa/Nairobi
    {  389, 5271,  113544,  8, }, // Africa/Ndjamena
    {  389, 5354,   88521,  6, }, // Africa/Niamey
    {  389, 6803,  113552,  7, }, // Africa/Nouakchott
    {  389, 6727,  113559,  7, }, // Africa/Ouagadougou
    {  389, 5225,   88541, 11, }, // Africa/Porto-Novo
    {  389, 4435,   88552,  7, }, // Africa/Sao_Tome
    {  389, 2866,   88559,  8, }, // Africa/Tripoli
    {  389, 6554,  113566,  5, }, // Africa/Tunis
    {  389, 4152,   88567,  7, }, // Africa/Windhoek
    {  389,  132,  113571,  4, }, // America/Adak
    {  389, 3346,  113575,  5, }, // America/Anchorage
    {  389, 5607,  113580,  6, }, // America/Anguilla
    {  389, 5624,  113586,  7, }, // America/Antigua
    {  389, 4554,  113593,  8, }, // America/Araguaina
    {  389,  166,  113601, 12, }, // America/Argentina/Buenos_Aires
    {  389,   91,  113613,  9, }, // America/Argentina/Catamarca
    {  389,  270,  113622,  7, }, // America/Argentina/Cordoba
    {  389,  441,  113629,  5, }, // America/Argentina/Jujuy
    {  389, 7462,  113634,  9, }, // America/Argentina/La_Rioja
    {  389,  565,  113643,  8, }, // America/Argentina/Mendoza
    {  389, 7489,  113651, 11, }, // America/Argentina/Rio_Gallegos
    {  389, 7520,   30437,  6, }, // America/Argentina/Salta
    {  389, 7544,  113662,  8, }, // America/Argentina/San_Juan
    {  389, 5580,   30452,  8, }, // America/Argentina/San_Luis
    {  389, 7571,  113670,  7, }, // America/Argentina/Tucuman
    {  389, 7597,   88675,  7, }, // America/Argentina/Ushuaia
    {  389, 5640,  113677,  5, }, // America/Aruba
    {  389, 4252,  113682,  7, }, // America/Asuncion
    {  389,  237,  113689,  8, }, // America/Atikokan
    {  389, 3679,   88703,  5, }, // America/Bahia
    {  389, 7623,  113697, 12, }, // America/Bahia_Banderas
    {  389, 5654,   88723,  8, }, // America/Barbados
    {  389, 7646,   30526,  5, }, // America/Belem
    {  389, 5407,  113709,  5, }, // America/Belize
    {  389, 7660,  113714, 14, }, // America/Blanc-Sablon
    {  389, 7681,  113728, 10, }, // America/Boa_Vista
    {  389, 4361,   88760,  6, }, // America/Bogota
    {  389, 7699,  113738,  4, }, // America/Boise
    {  389, 7713,  113742, 11, }, // America/Cambridge_Bay
    {  389, 7735,  113753, 13, }, // America/Campo_Grande
    {  389, 3868,  113766,  6, }, // America/Cancun
    {  389, 4694,  113772,  5, }, // America/Caracas
    {  389, 4345,  113777,  4, }, // America/Cayenne
    {  389, 5496,   30614,  5, }, // America/Cayman
    {  389, 2260,   88811,  6, }, // America/Chicago
    {  389, 7756,   88817,  8, }, // America/Chihuahua
    {  389, 7774,  113781, 13, }, // America/Ciudad_Juarez
    {  389, 5422,  113794, 11, }, // America/Costa_Rica
    {  389, 7796,  113805,  9, }, // America/Creston
    {  389, 3791,  113814,  6, }, // America/Cuiaba
    {  389, 5723,   88859,  7, }, // America/Curacao
    {  389, 7812,  113820, 12, }, // America/Danmarkshavn
    {  389, 7833,  113832,  4, }, // America/Dawson
    {  389, 7848,  113836, 10, }, // America/Dawson_Creek
    {  389,  805,  113846,  7, }, // America/Denver
    {  389, 3465,  113853,  8, }, // America/Detroit
    {  389, 5739,  113861,  8, }, // America/Dominica
    {  389,  893,  113869,  7, }, // America/Edmonton
    {  389, 7869,   30723,  7, }, // America/Eirunepe
    {  389, 5441,  113876, 12, }, // America/El_Salvador
    {  389, 7886,  113888, 12, }, // America/Fort_Nelson
    {  389, 7906,  113900, 10, }, // America/Fortaleza
    {  389, 7924,   30760,  8, }, // America/Glace_Bay
    {  389, 6881,  113910,  6, }, // America/Goose_Bay
    {  389, 4612,  113916, 11, }, // America/Grand_Turk
    {  389, 5770,  113927,  8, }, // America/Grenada
    {  389, 5786,   88985,  9, }, // America/Guadeloupe
    {  389, 3760,   30806, 10, }, // America/Guatemala
    {  389, 4841,  113935,  9, }, // America/Guayaquil
    {  389, 6932,   30824,  6, }, // America/Guyana
    {  389, 1939,  113944,  9, }, // America/Halifax
    {  389, 2281,   89018,  5, }, // America/Havana
    {  389, 7942,  113953,  9, }, // America/Hermosillo
    {  389,  348,  113962, 13, }, // America/Indiana/Indianapolis
    {  389,  481,  113975, 15, }, // America/Indiana/Knox
    {  389, 7961,   89062, 17, }, // America/Indiana/Marengo
    {  389, 7985,  113990, 20, }, // America/Indiana/Petersburg
    {  389, 8012,   89099, 18, }, // America/Indiana/Tell_City
    {  389, 8038,  114010, 14, }, // America/Indiana/Vevay
    {  389, 8060,  114024, 17, }, // America/Indiana/Vincennes
    {  389, 8086,  114041, 17, }, // America/Indiana/Winamac
    {  389, 8110,  114058,  6, }, // America/Inuvik
    {  389,  660,  114064,  8, }, // America/Iqaluit
    {  389, 2799,   89173,  5, }, // America/Jamaica
    {  389, 5380,  114072,  4, }, // America/Juneau
    {  389,  521,  114076,  8, }, // America/Kentucky/Louisville
    {  389, 8125,  114084, 19, }, // America/Kentucky/Monticello
    {  389, 5704,  114103, 10, }, // America/Kralendijk
    {  389, 4376,  114113,  6, }, // America/La_Paz
    {  389, 7169,  114119,  4, }, // America/Lima
    {  389, 3239,  114123, 10, }, // America/Los_Angeles
    {  389, 5932,  114133, 22, }, // America/Lower_Princes
    {  389, 8153,  114155,  6, }, // America/Maceio
    {  389, 8168,  114161,  8, }, // America/Managua
    {  389, 1908,   89274,  4, }, // America/Manaus
    {  389, 5897,  114169,  7, }, // America/Marigot
    {  389, 5805,   89285,  9, }, // America/Martinique
    {  389, 8184,   89294,  9, }, // America/Matamoros
    {  389, 2917,  114176,  9, }, // America/Mazatlan
    {  389, 8202,   89312,  8, }, // America/Menominee
    {  389, 8220,   31158,  6, }, // America/Merida
    {  389, 8235,  114185, 12, }, // America/Metlakatla
    {  389, 2949,  114197, 12, }, // America/Mexico_City
    {  389, 4391,  114209,  9, }, // America/Miquelon
    {  389, 8254,  114218,  8, }, // America/Moncton
    {  389, 8270,  114226,  7, }, // America/Monterrey
    {  389, 4098,  114233, 13, }, // America/Montevideo
    {  389, 5824,  114246, 10, }, // America/Montserrat
    {  389, 5481,  114256,  4, }, // America/Nassau
    {  389, 2379,  114260, 10, }, // America/New_York
    {  389, 8288,  114270,  4, }, // America/Nome
    {  389, 1850,   89401,  8, }, // America/Noronha
    {  389, 8301,  114274, 23, }, // America/North_Dakota/Beulah
    {  389, 8329,  114297, 18, }, // America/North_Dakota/Center
    {  389, 8357,  114315, 24, }, // America/North_Dakota/New_Salem
    {  389,  393,   31315,  3, }, // America/Nuuk
    {  389, 8388,  114339,  7, }, // America/Ojinaga
    {  389, 2356,   89484,  5, }, // America/Panama
    {  389, 6173,  114346,  9, }, // America/Paramaribo
    {  389, 2973,  114355,  7, }, // America/Phoenix
    {  389, 3945,  114362, 15, }, // America/Port-au-Prince
    {  389, 5954,  114377, 14, }, // America/Port_of_Spain
    {  389, 8404,   89536, 13, }, // America/Porto_Velho
    {  389, 5843,  114391, 11, }, // America/Puerto_Rico
    {  389, 4030,  114402, 12, }, // America/Punta_Arenas
    {  389, 8424,  114414, 11, }, // America/Rankin_Inlet
    {  389, 8445,  114425,  5, }, // America/Recife
    {  389, 1995,  114430,  6, }, // America/Regina
    {  389, 8460,  114436,  9, }, // America/Resolute
    {  389,  695,   89600, 12, }, // America/Rio_Branco
    {  389, 8477,  114445,  8, }, // America/Santarem
    {  389, 2201,  114453,  8, }, // America/Santiago
    {  389, 6111,  114461, 13, }, // America/Santo_Domingo
    {  389, 1878,  114474,  9, }, // America/Sao_Paulo
    {  389, 6899,  114483, 13, }, // America/Scoresbysund
    {  389, 8494,   89664,  6, }, // America/Sitka
    {  389, 8508,   89670, 15, }, // America/St_Barthelemy
    {  389, 2061,  114496, 10, }, // America/St_Johns
    {  389, 5863,   89694, 10, }, // America/St_Kitts
    {  389, 5880,  114506, 11, }, // America/St_Lucia
    {  389,  855,   89715,  9, }, // America/St_Thomas
    {  389, 5913,  114517, 13, }, // America/St_Vincent
    {  389, 8530,  114530, 12, }, // America/Swift_Current
    {  389, 5461,  114542, 11, }, // America/Tegucigalpa
    {  389, 5756,  114553,  4, }, // America/Thule
    {  389,  313,   89766,  7, }, // America/Tijuana
    {  389,  608,  114557,  6, }, // America/Toronto
    {  389, 5688,  114563,  7, }, // America/Tortola
    {  389, 2093,  114570, 10, }, // America/Vancouver
    {  389, 2144,  114580, 11, }, // America/Whitehorse
    {  389,  734,   89805,  7, }, // America/Winnipeg
    {  389, 8552,  114591,  7, }, // America/Yakutat
    {  389, 6001,   31673,  4, }, // Antarctica/Casey
    {  389, 6094,  114598,  7, }, // Antarctica/Davis
    {  389, 6133,  114605, 14, }, // Antarctica/DumontDUrville
    {  389, 8568,  114619,  8, }, // Antarctica/Macquarie
    {  389, 7066,   89850,  4, }, // Antarctica/Mawson
    {  389, 7113,  114627,  9, }, // Antarctica/McMurdo
    {  389, 8589,  114636,  7, }, // Antarctica/Palmer
    {  389, 7226,   89867,  6, }, // Antarctica/Rothera
    {  389, 7295,   89873,  6, }, // Antarctica/Syowa
    {  389, 8607,   89879,  5, }, // Antarctica/Troll
    {  389, 7374,   89884,  7, }, // Antarctica/Vostok
    {  389, 1429,  114643, 13, }, // Arctic/Longyearbyen
    {  389, 5570,  114656,  3, }, // Asia/Aden
    {  389, 5395,   89908,  7, }, // Asia/Almaty
    {  389, 3968,   89915,  6, }, // Asia/Amman
    {  389, 5511,  114659,  6, }, // Asia/Anadyr
    {  389, 5523,  114665,  5, }, // Asia/Aqtau
    {  389, 5368,  114670,  7, }, // Asia/Aqtobe
    {  389,  964,  114677,  7, }, // Asia/Ashgabat
    {  389, 8624,  114684,  7, }, // Asia/Atyrau
    {  389, 3607,   89938,  5, }, // Asia/Baghdad
    {  389, 5534,  114691,  6, }, // Asia/Bahrain
    {  389, 3653,   30654,  4, }, // Asia/Baku
    {  389, 4466,  114697,  5, }, // Asia/Bangkok
    {  389, 3571,  114702,  5, }, // Asia/Barnaul
    {  389, 4086,  114707,  5, }, // Asia/Beirut
    {  389, 3778,   31836,  7, }, // Asia/Bishkek
    {  389, 5989,   89965,  7, }, // Asia/Brunei
    {  389, 4601,  114712,  4, }, // Asia/Chita
    {  389, 4511,   89978,  6, }, // Asia/Colombo
    {  389, 4540,  114716,  6, }, // Asia/Damascus
    {  389, 1093,   89991,  4, }, // Asia/Dhaka
    {  389, 6192,  114722,  4, }, // Asia/Dili
    {  389, 3596,  114726,  4, }, // Asia/Dubai
    {  389, 6159,   31881,  8, }, // Asia/Dushanbe
    {  389, 8636,  114730,  9, }, // Asia/Famagusta
    {  389, 8651,  114739,  4, }, // Asia/Gaza
    {  389, 4795,  114743,  7, }, // Asia/Hebron
    {  389, 1259,   90033, 11, }, // Asia/Ho_Chi_Minh
    {  389, 2704,   90049,  9, }, // Asia/Hong_Kong
    {  389, 4771,  114750,  6, }, // Asia/Hovd
    {  389, 4184,  114756,  9, }, // Asia/Irkutsk
    {  389, 7006,   90073,  7, }, // Asia/Jakarta
    {  389, 6992,  114765,  8, }, // Asia/Jayapura
    {  389, 1290,  114773,  9, }, // Asia/Jerusalem
    {  389, 3560,   90086,  5, }, // Asia/Kabul
    {  389, 4316,  114782,  9, }, // Asia/Kamchatka
    {  389, 4239,   31972,  5, }, // Asia/Karachi
    {  389, 1185,   90098,  8, }, // Asia/Kathmandu
    {  389, 8661,  114791,  8, }, // Asia/Khandyga
    {  389,  992,   90113,  7, }, // Asia/Kolkata
    {  389, 4197,   90120, 15, }, // Asia/Krasnoyarsk
    {  389, 4859,  114799, 11, }, // Asia/Kuala_Lumpur
    {  389, 5976,  114810,  6, }, // Asia/Kuching
    {  389, 5547,  114816,  5, }, // Asia/Kuwait
    {  389, 1211,  114821,  4, }, // Asia/Macau
    {  389, 4017,  114825,  6, }, // Asia/Magadan
    {  389, 1349,  114831,  8, }, // Asia/Makassar
    {  389, 7182,  114839,  5, }, // Asia/Manila
    {  389, 6920,   90155,  5, }, // Asia/Muscat
    {  389, 2554,   90160,  8, }, // Asia/Nicosia
    {  389, 8675,  114844, 14, }, // Asia/Novokuznetsk
    {  389, 4135,   90184, 13, }, // Asia/Novosibirsk
    {  389, 4229,   90197,  6, }, // Asia/Omsk
    {  389, 7145,   90203,  3, }, // Asia/Oral
    {  389, 6961,  114858, 11, }, // Asia/Phnom_Penh
    {  389, 8693,  114869, 10, }, // Asia/Pontianak
    {  389, 4214,   90224, 10, }, // Asia/Pyongyang
    {  389, 5559,  114879,  4, }, // Asia/Qatar
    {  389, 8708,  114883,  9, }, // Asia/Qostanay
    {  389, 4269,  114892, 12, }, // Asia/Qyzylorda
    {  389, 3584,   32160,  5, }, // Asia/Riyadh
    {  389, 4408,  114904,  5, }, // Asia/Sakhalin
    {  389, 7245,   90267,  6, }, // Asia/Samarkand
    {  389, 3279,   89593,  3, }, // Asia/Seoul
    {  389, 1053,  114909,  6, }, // Asia/Shanghai
    {  389, 3300,  114915,  8, }, // Asia/Singapore
    {  389, 4297,  114923, 16, }, // Asia/Srednekolymsk
    {  389, 3263,  114939,  5, }, // Asia/Taipei
    {  389, 4781,  114944,  7, }, // Asia/Tashkent
    {  389, 3915,  114951,  7, }, // Asia/Tbilisi
    {  389, 2772,   32236,  6, }, // Asia/Tehran
    {  389, 1317,  114958,  6, }, // Asia/Thimphu
    {  389, 2821,  114964,  6, }, // Asia/Tokyo
    {  389, 4572,  114970,  6, }, // Asia/Tomsk
    {  389, 1021,  114976,  8, }, // Asia/Ulaanbaatar
    {  389, 1159,  114984,  7, }, // Asia/Urumqi
    {  389, 8722,  114991,  9, }, // Asia/Ust-Nera
    {  389, 6977,  115000, 11, }, // Asia/Vientiane
    {  389, 4710,   90369, 13, }, // Asia/Vladivostok
    {  389, 4828,  115011,  8, }, // Asia/Yakutsk
    {  389, 1235,  115019,  6, }, // Asia/Yangon
    {  389, 3883,   90382, 13, }, // Asia/Yekaterinburg
    {  389, 3747,   90395,  7, }, // Asia/Yerevan
    {  389, 3663,  115025,  6, }, // Atlantic/Azores
    {  389, 5671,  115031,  7, }, // Atlantic/Bermuda
    {  389, 6628,  115038,  5, }, // Atlantic/Canary
    {  389, 3727,  115043,  9, }, // Atlantic/Cape_Verde
    {  389, 1395,  115052,  4, }, // Atlantic/Faroe
    {  389, 8736,  115056,  5, }, // Atlantic/Madeira
    {  389, 2748,  115061,  9, }, // Atlantic/Reykjavik
    {  389, 7272,   90448, 15, }, // Atlantic/South_Georgia
    {  389, 6821,  115070, 11, }, // Atlantic/St_Helena
    {  389, 6644,  115081,  8, }, // Atlantic/Stanley
    {  389, 1670,   90481,  6, }, // Australia/Adelaide
    {  389, 1635,  115089,  9, }, // Australia/Brisbane
    {  389, 1799,   90495, 10, }, // Australia/Broken_Hill
    {  389, 1583,   90505,  7, }, // Australia/Darwin
    {  389, 3637,  115098,  5, }, // Australia/Eucla
    {  389, 1516,   90512,  7, }, // Australia/Hobart
    {  389, 8753,  115103,  7, }, // Australia/Lindeman
    {  389, 1547,   90527, 10, }, // Australia/Lord_Howe
    {  389, 1727,  115110,  7, }, // Australia/Melbourne
    {  389, 1762,   90545,  4, }, // Australia/Perth
    {  389, 1463,   32494,  5, }, // Australia/Sydney
    {  389, 6441,  115117, 10, }, // Europe/Amsterdam
    {  389, 6216,   90559,  6, }, // Europe/Andorra
    {  389, 3620,  115127,  8, }, // Europe/Astrakhan
    {  389, 2300,  115135,  6, }, // Europe/Athens
    {  389, 6336,   32532,  8, }, // Europe/Belgrade
    {  389, 4757,   90580,  6, }, // Europe/Berlin
    {  389, 6488,  115141, 14, }, // Europe/Bratislava
    {  389, 2167,  115155,  9, }, // Europe/Brussels
    {  389, 3928,  115164,  9, }, // Europe/Bucharest
    {  389, 3806,   90617,  9, }, // Europe/Budapest
    {  389, 8772,  115173,  8, }, // Europe/Busingen
    {  389, 2583,  115181,  7, }, // Europe/Chisinau
    {  389, 6289,  115188,  8, }, // Europe/Copenhagen
    {  389, 2338,  115196,  6, }, // Europe/Dublin
    {  389, 6307,   90657, 10, }, // Europe/Gibraltar
    {  389, 8788,   90667,  6, }, // Europe/Guernsey
    {  389, 6612,   90673,  8, }, // Europe/Helsinki
    {  389, 8804,  115202, 11, }, // Europe/Isle_of_Man
    {  389, 1130,  115213,  8, }, // Europe/Istanbul
    {  389, 8823,   90701,  5, }, // Europe/Jersey
    {  389, 3979,  115221, 12, }, // Europe/Kaliningrad
    {  389, 8929,   90718,  5, }, // Europe/Kirov
    {  389, 2527,   90723,  3, }, // Europe/Kyiv
    {  389, 3213,   32690,  6, }, // Europe/Lisbon
    {  389, 6506,  115233,  9, }, // Europe/Ljubljana
    {  389, 2501,  115242,  4, }, // Europe/London
    {  389, 6365,  115246, 10, }, // Europe/Luxembourg
    {  389, 4892,  115256,  7, }, // Europe/Madrid
    {  389, 6397,   32731,  6, }, // Europe/Malta
    {  389, 6582,  115263,  9, }, // Europe/Mariehamn
    {  389, 3693,   90767,  6, }, // Europe/Minsk
    {  389, 6410,  115272,  6, }, // Europe/Monaco
    {  389, 3537,   90779,  6, }, // Europe/Moscow
    {  389, 6458,   90785,  5, }, // Europe/Oslo
    {  389, 4284,  115278,  5, }, // Europe/Paris
    {  389, 6424,  115283,  9, }, // Europe/Podgorica
    {  389, 6275,   90800,  5, }, // Europe/Prague
    {  389, 8837,   90805,  4, }, // Europe/Riga
    {  389, 6324,   90432,  3, }, // Europe/Rome
    {  389, 4331,   90809,  5, }, // Europe/Samara
    {  389, 6470,  115292, 10, }, // Europe/San_Marino
    {  389, 6245,  115302,  8, }, // Europe/Sarajevo
    {  389, 4451,  115310,  7, }, // Europe/Saratov
    {  389, 8849,  115317,  9, }, // Europe/Simferopol
    {  389, 6383,   90851,  8, }, // Europe/Skopje
    {  389, 6599,  115326,  6, }, // Europe/Sofia
    {  389, 6523,   90866,  8, }, // Europe/Stockholm
    {  389, 8867,  115332,  5, }, // Europe/Tallinn
    {  389, 6202,  115337,  6, }, // Europe/Tirane
    {  389, 8882,   90885, 12, }, // Europe/Ulyanovsk
    {  389, 6352,  115343,  4, }, // Europe/Vaduz
    {  389, 6567,  115347,  6, }, // Europe/Vatican
    {  389, 6231,  115353,  6, }, // Europe/Vienna
    {  389, 8899,   90914,  8, }, // Europe/Vilnius
    {  389, 4727,  115359, 13, }, // Europe/Volgograd
    {  389, 3190,  115372,  5, }, // Europe/Warsaw
    {  389, 6261,  115377,  7, }, // Europe/Zagreb
    {  389, 6540,  115384,  5, }, // Europe/Zurich
    {  389, 5078,  115389, 12, }, // Indian/Antananarivo
    {  389, 6947,  115401,  5, }, // Indian/Chagos
    {  389, 6046,   90971,  7, }, // Indian/Christmas
    {  389, 6063,   90978,  5, }, // Indian/Cocos
    {  389, 5029,   90983,  6, }, // Indian/Comoro
    {  389, 6661,  115406,  8, }, // Indian/Kerguelen
    {  389, 7260,   32983,  4, }, // Indian/Mahe
    {  389, 7050,   90996,  6, }, // Indian/Maldives
    {  389, 4069,  115414,  6, }, // Indian/Mauritius
    {  389, 5098,  115420,  5, }, // Indian/Mayotte
    {  389, 7211,  115425,  8, }, // Indian/Reunion
    {  389, 4422,  115433,  5, }, // Pacific/Apia
    {  389,  932,  115438,  5, }, // Pacific/Auckland
    {  389, 3706,  115443,  9, }, // Pacific/Bougainville
    {  389, 3015,  115452,  4, }, // Pacific/Chatham
    {  389, 3157,  115456,  3, }, // Pacific/Chuuk
    {  389, 2237,   91049,  5, }, // Pacific/Easter
    {  389, 7360,  115459,  4, }, // Pacific/Efate
    {  389, 7327,  115463,  7, }, // Pacific/Fakaofo
    {  389, 3902,   33069,  4, }, // Pacific/Fiji
    {  389, 7343,  115470,  8, }, // Pacific/Funafuti
    {  389, 6678,  115478,  9, }, // Pacific/Galapagos
    {  389, 6696,  115487,  8, }, // Pacific/Gambier
    {  389, 3822,  115495, 12, }, // Pacific/Guadalcanal
    {  389, 6018,   31112,  4, }, // Pacific/Guam
    {  389, 3049,  115507,  7, }, // Pacific/Kanton
    {  389, 3998,  115514, 10, }, // Pacific/Kiritimati
    {  389, 7019,  115524,  5, }, // Pacific/Kosrae
    {  389, 2842,  115529,  8, }, // Pacific/Kwajalein
    {  389, 4877,   91142,  6, }, // Pacific/Majuro
    {  389, 4051,  115537,  8, }, // Pacific/Marquesas
    {  389, 8914,   91156,  5, }, // Pacific/Midway
    {  389, 7084,  115545,  5, }, // Pacific/Nauru
    {  389, 7132,   91165,  4, }, // Pacific/Niue
    {  389, 4168,  115550,  6, }, // Pacific/Norfolk
    {  389, 7098,  115556,  5, }, // Pacific/Noumea
    {  389, 3126,   91182,  9, }, // Pacific/Pago_Pago
    {  389, 7155,   91191,  4, }, // Pacific/Palau
    {  389, 7194,  115561,  7, }, // Pacific/Pitcairn
    {  389, 3096,  115568,  8, }, // Pacific/Pohnpei
    {  389, 4807,  115576, 14, }, // Pacific/Port_Moresby
    {  389, 6076,   91215,  9, }, // Pacific/Rarotonga
    {  389, 6031,   91224,  6, }, // Pacific/Saipan
    {  389, 7312,   91230,  6, }, // Pacific/Tahiti
    {  389, 6712,  115590,  5, }, // Pacific/Tarawa
    {  389, 4583,   91242,  9, }, // Pacific/Tongatapu
    {  389, 7392,  115595,  3, }, // Pacific/Wake
    {  389, 7405,   33271,  5, }, // Pacific/Wallis
    {  390,   14,       0,  6, }, // Africa/Asmara Konkani/Latin/India
    {  390, 4435,       6,  8, }, // Africa/Sao_Tome
    {  390, 4252,      14,  8, }, // America/Asuncion
    {  390,  237,      22,  8, }, // America/Atikokan
    {  390, 7623,      30, 17, }, // America/Bahia_Banderas
    {  390, 3868,      47,  6, }, // America/Cancun
    {  390, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  390, 5723,      66,  7, }, // America/Curacao
    {  390,  481,      73, 13, }, // America/Indiana/Knox
    {  390, 7961,      86, 16, }, // America/Indiana/Marengo
    {  390, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  390, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  390, 8038,     139, 14, }, // America/Indiana/Vevay
    {  390, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  390, 8086,     171, 16, }, // America/Indiana/Winamac
    {  390, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  390, 5932,     207, 22, }, // America/Lower_Princes
    {  390, 8220,     229,  6, }, // America/Merida
    {  390, 1850,     235, 19, }, // America/Noronha
    {  390, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  390, 8329,     274, 20, }, // America/North_Dakota/Center
    {  390, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  390,  393,     317,  4, }, // America/Nuuk
    {  390, 6899,     321, 16, }, // America/Scoresbysund
    {  390, 8508,     337, 14, }, // America/St_Barthelemy
    {  390, 2061,     351, 10, }, // America/St_Johns
    {  390, 5863,     361,  9, }, // America/St_Kitts
    {  390, 5880,     370,  9, }, // America/St_Lucia
    {  390,  855,     379, 10, }, // America/St_Thomas
    {  390, 5913,     389, 11, }, // America/St_Vincent
    {  390, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  390, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  390, 1185,     448,  9, }, // Asia/Kathmandu
    {  390,  992,     457,  7, }, // Asia/Kolkata
    {  390, 1211,     464,  5, }, // Asia/Macau
    {  390, 1235,     477,  6, }, // Asia/Yangon
    {  390, 1395,     483,  5, }, // Atlantic/Faroe
    {  390, 6821,     488, 10, }, // Atlantic/St_Helena
    {  390, 2527,     514,  4, }, // Europe/Kyiv
    {  390, 7211,     547,  7, }, // Indian/Reunion
    {  390, 3157,     554,  5, }, // Pacific/Chuuk
    {  390, 3096,     586,  7, }, // Pacific/Pohnpei
    {  391, 6788,  115598,  3, }, // Africa/Abidjan Korean/Korean/South Korea
    {  391, 6760,  115601,  3, }, // Africa/Accra
    {  391, 5059,  115604,  6, }, // Africa/Addis_Ababa
    {  391, 7420,  115610,  2, }, // Africa/Algiers
    {  391,   14,  115612,  4, }, // Africa/Asmara
    {  391,   44,  115616,  3, }, // Africa/Bamako
    {  391, 5257,  115619,  2, }, // Africa/Bangui
    {  391, 6746,  115621,  2, }, // Africa/Banjul
    {  391, 7435,  115623,  3, }, // Africa/Bissau
    {  391, 4957,  115626,  5, }, // Africa/Blantyre
    {  391, 5287,  115631,  4, }, // Africa/Brazzaville
    {  391, 4922,  115635,  4, }, // Africa/Bujumbura
    {  391, 2320,  115639,  3, }, // Africa/Cairo
    {  391, 4117,  115642,  5, }, // Africa/Casablanca
    {  391, 7449,  115647,  3, }, // Africa/Ceuta
    {  391, 6773,  115650,  4, }, // Africa/Conakry
    {  391, 6840,  115654,  3, }, // Africa/Dakar
    {  391, 5130,  115657,  6, }, // Africa/Dar_es_Salaam
    {  391, 5043,  115663,  3, }, // Africa/Djibouti
    {  391, 5243,  115666,  3, }, // Africa/Douala
    {  391, 5166,  115669,  4, }, // Africa/El_Aaiun
    {  391, 6853,  115673,  4, }, // Africa/Freetown
    {  391, 4906,  115677,  4, }, // Africa/Gaborone
    {  391, 5015,  115681,  3, }, // Africa/Harare
    {  391, 4479,  115684,  6, }, // Africa/Johannesburg
    {  391, 4499,  115690,  2, }, // Africa/Juba
    {  391, 5151,  115692,  3, }, // Africa/Kampala
    {  391, 4524,  115695,  3, }, // Africa/Khartoum
    {  391, 4973,  115698,  3, }, // Africa/Kigali
    {  391, 5306,  115701,  3, }, // Africa/Kinshasa
    {  391, 4744,  115704,  3, }, // Africa/Lagos
    {  391, 5336,  115707,  4, }, // Africa/Libreville
    {  391, 6869,  115711,  2, }, // Africa/Lome
    {  391, 5211,  115713,  3, }, // Africa/Luanda
    {  391, 4939,  115716,  4, }, // Africa/Lubumbashi
    {  391, 5001,  115720,  3, }, // Africa/Lusaka
    {  391, 5322,  115723,  3, }, // Africa/Malabo
    {  391, 4987,  115726,  3, }, // Africa/Maputo
    {  391, 5197,  115729,  3, }, // Africa/Maseru
    {  391, 5182,  115732,  4, }, // Africa/Mbabane
    {  391, 5113,  115736,  4, }, // Africa/Mogadishu
    {  391, 7034,  115740,  4, }, // Africa/Monrovia
    {  391, 3853,  115744,  4, }, // Africa/Nairobi
    {  391, 5271,  115748,  4, }, // Africa/Ndjamena
    {  391, 5354,  115752,  3, }, // Africa/Niamey
    {  391, 6803,  115755,  4, }, // Africa/Nouakchott
    {  391, 6727,  115759,  4, }, // Africa/Ouagadougou
    {  391, 5225,  115763,  5, }, // Africa/Porto-Novo
    {  391, 4435,  115768,  3, }, // Africa/Sao_Tome
    {  391, 2866,  115771,  4, }, // Africa/Tripoli
    {  391, 6554,  115775,  3, }, // Africa/Tunis
    {  391, 4152,  115778,  4, }, // Africa/Windhoek
    {  391,  132,  115782,  3, }, // America/Adak
    {  391, 3346,  115785,  4, }, // America/Anchorage
    {  391, 5607,  115789,  3, }, // America/Anguilla
    {  391, 5624,  115792,  3, }, // America/Antigua
    {  391, 4554,  115795,  5, }, // America/Araguaina
    {  391,  166,  115800,  9, }, // America/Argentina/Buenos_Aires
    {  391,   91,  115809,  5, }, // America/Argentina/Catamarca
    {  391,  270,  115814,  4, }, // America/Argentina/Cordoba
    {  391,  441,  115818,  3, }, // America/Argentina/Jujuy
    {  391, 7462,  115821,  5, }, // America/Argentina/La_Rioja
    {  391,  565,  115826,  3, }, // America/Argentina/Mendoza
    {  391, 7489,  115829,  7, }, // America/Argentina/Rio_Gallegos
    {  391, 7520,  115836,  2, }, // America/Argentina/Salta
    {  391, 7544,  115838,  3, }, // America/Argentina/San_Juan
    {  391, 5580,  115841,  4, }, // America/Argentina/San_Luis
    {  391, 7571,  115845,  3, }, // America/Argentina/Tucuman
    {  391, 7597,  115848,  5, }, // America/Argentina/Ushuaia
    {  391, 5640,  115853,  3, }, // America/Aruba
    {  391, 4252,  115856,  4, }, // America/Asuncion
    {  391,  237,  115860,  4, }, // America/Atikokan
    {  391, 3679,  115864,  3, }, // America/Bahia
    {  391, 7623,  115867,  8, }, // America/Bahia_Banderas
    {  391, 5654,  115875,  5, }, // America/Barbados
    {  391, 7646,  115880,  2, }, // America/Belem
    {  391, 5407,  115882,  3, }, // America/Belize
    {  391, 7660,  115885,  6, }, // America/Blanc-Sablon
    {  391, 7681,  115891,  6, }, // America/Boa_Vista
    {  391, 4361,  115897,  3, }, // America/Bogota
    {  391, 7699,  115900,  3, }, // America/Boise
    {  391, 7713,  115903,  8, }, // America/Cambridge_Bay
    {  391, 7735,  115911,  6, }, // America/Campo_Grande
    {  391, 3868,  115917,  2, }, // America/Cancun
    {  391, 4694,  115919,  4, }, // America/Caracas
    {  391, 4345,  115923,  2, }, // America/Cayenne
    {  391, 5496,  115925,  3, }, // America/Cayman
    {  391, 2260,  115928,  3, }, // America/Chicago
    {  391, 7756,  115931,  3, }, // America/Chihuahua
    {  391, 7774,  115934,  8, }, // America/Ciudad_Juarez
    {  391, 5422,  115942,  5, }, // America/Costa_Rica
    {  391, 7796,  115947,  4, }, // America/Creston
    {  391, 3791,  115951,  4, }, // America/Cuiaba
    {  391, 5723,  115955,  3, }, // America/Curacao
    {  391, 7812,  115958,  5, }, // America/Danmarkshavn
    {  391, 7833,  115963,  2, }, // America/Dawson
    {  391, 7848,  115965,  4, }, // America/Dawson_Creek
    {  391,  805,  115969,  2, }, // America/Denver
    {  391, 3465,  115971,  5, }, // America/Detroit
    {  391, 5739,  115976,  4, }, // America/Dominica
    {  391,  893,  115980,  4, }, // America/Edmonton
    {  391, 7869,  115984,  5, }, // America/Eirunepe
    {  391, 5441,  115989,  5, }, // America/El_Salvador
    {  391, 7886,  115994,  5, }, // America/Fort_Nelson
    {  391, 7906,  115999,  5, }, // America/Fortaleza
    {  391, 7924,  116004,  5, }, // America/Glace_Bay
    {  391, 6881,  116009,  4, }, // America/Goose_Bay
    {  391, 4612,  116013,  6, }, // America/Grand_Turk
    {  391, 5770,  116019,  4, }, // America/Grenada
    {  391, 5786,  116023,  4, }, // America/Guadeloupe
    {  391, 3760,  116027,  4, }, // America/Guatemala
    {  391, 4841,  116031,  3, }, // America/Guayaquil
    {  391, 6932,  116034,  4, }, // America/Guyana
    {  391, 1939,  116038,  4, }, // America/Halifax
    {  391, 2281,  116042,  3, }, // America/Havana
    {  391, 7942,  116045,  5, }, // America/Hermosillo
    {  391,  348,  116050,  7, }, // America/Indiana/Indianapolis
    {  391,  481,  116057,  9, }, // America/Indiana/Knox
    {  391, 7961,  116066, 10, }, // America/Indiana/Marengo
    {  391, 7985,  116076, 11, }, // America/Indiana/Petersburg
    {  391, 8012,  116087, 10, }, // America/Indiana/Tell_City
    {  391, 8038,  116097,  9, }, // America/Indiana/Vevay
    {  391, 8060,  116106, 10, }, // America/Indiana/Vincennes
    {  391, 8086,  116116, 10, }, // America/Indiana/Winamac
    {  391, 8110,  116126,  3, }, // America/Inuvik
    {  391,  660,  116129,  5, }, // America/Iqaluit
    {  391, 2799,  116134,  4, }, // America/Jamaica
    {  391, 5380,  116138,  2, }, // America/Juneau
    {  391,  521,  116140,  3, }, // America/Kentucky/Louisville
    {  391, 8125,  116143, 10, }, // America/Kentucky/Monticello
    {  391, 5704,  116153,  6, }, // America/Kralendijk
    {  391, 4376,  116159,  3, }, // America/La_Paz
    {  391, 7169,  116162,  2, }, // America/Lima
    {  391, 3239,  116164,  6, }, // America/Los_Angeles
    {  391, 5932,  116170,  9, }, // America/Lower_Princes
    {  391, 8153,  116179,  4, }, // America/Maceio
    {  391, 8168,  116183,  3, }, // America/Managua
    {  391, 1908,  116186,  4, }, // America/Manaus
    {  391, 5897,  116190,  3, }, // America/Marigot
    {  391, 5805,  116193,  4, }, // America/Martinique
    {  391, 8184,  116197,  5, }, // America/Matamoros
    {  391, 2917,  116202,  4, }, // America/Mazatlan
    {  391, 8202,  116206,  4, }, // America/Menominee
    {  391, 8220,  116210,  3, }, // America/Merida
    {  391, 8235,  116213,  6, }, // America/Metlakatla
    {  391, 2949,  116219,  6, }, // America/Mexico_City
    {  391, 4391,  116225,  3, }, // America/Miquelon
    {  391, 8254,  116228,  2, }, // America/Moncton
    {  391, 8270,  116230,  4, }, // America/Monterrey
    {  391, 4098,  116234,  5, }, // America/Montevideo
    {  391, 5824,  116239,  4, }, // America/Montserrat
    {  391, 5481,  116243,  2, }, // America/Nassau
    {  391, 2379,  116245,  2, }, // America/New_York
    {  391, 8288,  116247,  1, }, // America/Nome
    {  391, 1850,  116248,  3, }, // America/Noronha
    {  391, 8301,  116251, 10, }, // America/North_Dakota/Beulah
    {  391, 8329,  116261,  9, }, // America/North_Dakota/Center
    {  391, 8357,  116270, 11, }, // America/North_Dakota/New_Salem
    {  391,  393,  116281,  4, }, // America/Nuuk
    {  391, 8388,  116285,  4, }, // America/Ojinaga
    {  391, 2356,  116289,  3, }, // America/Panama
    {  391, 6173,  116292,  5, }, // America/Paramaribo
    {  391, 2973,  116297,  3, }, // America/Phoenix
    {  391, 3945,  116300,  6, }, // America/Port-au-Prince
    {  391, 5954,  116306,  7, }, // America/Port_of_Spain
    {  391, 8404,  116313,  5, }, // America/Porto_Velho
    {  391, 5843,  116318,  6, }, // America/Puerto_Rico
    {  391, 4030,  116324,  6, }, // America/Punta_Arenas
    {  391, 8424,  116330,  5, }, // America/Rankin_Inlet
    {  391, 8445,  116335,  3, }, // America/Recife
    {  391, 1995,  116338,  4, }, // America/Regina
    {  391, 8460,  116342,  4, }, // America/Resolute
    {  391,  695,  116346,  6, }, // America/Rio_Branco
    {  391, 8477,  116352,  3, }, // America/Santarem
    {  391, 2201,  116355,  4, }, // America/Santiago
    {  391, 6111,  116359,  5, }, // America/Santo_Domingo
    {  391, 1878,  116364,  4, }, // America/Sao_Paulo
    {  391, 6899,  116368,  8, }, // America/Scoresbysund
    {  391, 8494,  116376,  2, }, // America/Sitka
    {  391, 8508,  116378,  6, }, // America/St_Barthelemy
    {  391, 2061,  116384,  5, }, // America/St_Johns
    {  391, 5863,  116389,  5, }, // America/St_Kitts
    {  391, 5880,  116394,  6, }, // America/St_Lucia
    {  391,  855,  116400,  6, }, // America/St_Thomas
    {  391, 5913,  116406,  6, }, // America/St_Vincent
    {  391, 8530,  116412,  7, }, // America/Swift_Current
    {  391, 5461,  116419,  5, }, // America/Tegucigalpa
    {  391, 5756,  116424,  2, }, // America/Thule
    {  391,  313,  116426,  4, }, // America/Tijuana
    {  391,  608,  116430,  3, }, // America/Toronto
    {  391, 5688,  116433,  4, }, // America/Tortola
    {  391, 2093,  116437,  3, }, // America/Vancouver
    {  391, 2144,  116440,  5, }, // America/Whitehorse
    {  391,  734,  116445,  3, }, // America/Winnipeg
    {  391, 8552,  116448,  4, }, // America/Yakutat
    {  391, 6001,  116452,  3, }, // Antarctica/Casey
    {  391, 6094,  116455,  4, }, // Antarctica/Davis
    {  391, 6133,  116459,  6, }, // Antarctica/DumontDUrville
    {  391, 8568,  116465,  3, }, // Antarctica/Macquarie
    {  391, 7066,  116468,  2, }, // Antarctica/Mawson
    {  391, 7113,  116470,  3, }, // Antarctica/McMurdo
    {  391, 8589,  116473,  2, }, // Antarctica/Palmer
    {  391, 7226,  116475,  3, }, // Antarctica/Rothera
    {  391, 7295,  116478,  2, }, // Antarctica/Syowa
    {  391, 8607,  116480,  2, }, // Antarctica/Troll
    {  391, 7374,  116482,  4, }, // Antarctica/Vostok
    {  391, 1429,  116486,  5, }, // Arctic/Longyearbyen
    {  391, 5570,  116491,  2, }, // Asia/Aden
    {  391, 5395,  116493,  3, }, // Asia/Almaty
    {  391, 3968,  116496,  2, }, // Asia/Amman
    {  391, 5511,  116498,  4, }, // Asia/Anadyr
    {  391, 5523,  116502,  4, }, // Asia/Aqtau
    {  391, 5368,  116506,  3, }, // Asia/Aqtobe
    {  391,  964,  116509,  5, }, // Asia/Ashgabat
    {  391, 8624,  116514,  4, }, // Asia/Atyrau
    {  391, 3607,  116518,  4, }, // Asia/Baghdad
    {  391, 5534,  116522,  3, }, // Asia/Bahrain
    {  391, 3653,  116525,  2, }, // Asia/Baku
    {  391, 4466,  116527,  2, }, // Asia/Bangkok
    {  391, 3571,  116529,  4, }, // Asia/Barnaul
    {  391, 4086,  116533,  4, }, // Asia/Beirut
    {  391, 3778,  116537,  4, }, // Asia/Bishkek
    {  391, 5989,  116541,  4, }, // Asia/Brunei
    {  391, 4601,  116545,  2, }, // Asia/Chita
    {  391, 4511,  116547,  3, }, // Asia/Colombo
    {  391, 4540,  116550,  5, }, // Asia/Damascus
    {  391, 1093,  115654,  2, }, // Asia/Dhaka
    {  391, 6192,  116555,  2, }, // Asia/Dili
    {  391, 3596,  116557,  3, }, // Asia/Dubai
    {  391, 6159,  116560,  3, }, // Asia/Dushanbe
    {  391, 8636,  116563,  5, }, // Asia/Famagusta
    {  391, 8651,  116568,  2, }, // Asia/Gaza
    {  391, 4795,  116570,  3, }, // Asia/Hebron
    {  391, 1259,  116573,  3, }, // Asia/Ho_Chi_Minh
    {  391, 2704,  116576,  2, }, // Asia/Hong_Kong
    {  391, 4771,  116578,  3, }, // Asia/Hovd
    {  391, 4184,  116581,  5, }, // Asia/Irkutsk
    {  391, 7006,  116586,  4, }, // Asia/Jakarta
    {  391, 6992,  116590,  4, }, // Asia/Jayapura
    {  391, 1290,  116594,  4, }, // Asia/Jerusalem
    {  391, 3560,  116598,  2, }, // Asia/Kabul
    {  391, 4316,  116600,  3, }, // Asia/Kamchatka
    {  391, 4239,  116603,  3, }, // Asia/Karachi
    {  391, 1185,  116606,  4, }, // Asia/Kathmandu
    {  391, 8661,  116610,  3, }, // Asia/Khandyga
    {  391,  992,  116613,  3, }, // Asia/Kolkata
    {  391, 4197,  116616,  8, }, // Asia/Krasnoyarsk
    {  391, 4859,  116624,  6, }, // Asia/Kuala_Lumpur
    {  391, 5976,  116630,  2, }, // Asia/Kuching
    {  391, 5547,  116632,  4, }, // Asia/Kuwait
    {  391, 1211,  116636,  3, }, // Asia/Macau
    {  391, 4017,  116639,  3, }, // Asia/Magadan
    {  391, 1349,  116642,  4, }, // Asia/Makassar
    {  391, 7182,  116646,  3, }, // Asia/Manila
    {  391, 6920,  116649,  4, }, // Asia/Muscat
    {  391, 2554,  116653,  4, }, // Asia/Nicosia
    {  391, 8675,  116657,  7, }, // Asia/Novokuznetsk
    {  391, 4135,  116664,  7, }, // Asia/Novosibirsk
    {  391, 4229,  116671,  3, }, // Asia/Omsk
    {  391, 7145,  116674,  2, }, // Asia/Oral
    {  391, 6961,  116676,  3, }, // Asia/Phnom_Penh
    {  391, 8693,  116679,  5, }, // Asia/Pontianak
    {  391, 4214,  116684,  2, }, // Asia/Pyongyang
    {  391, 5559,  116686,  3, }, // Asia/Qatar
    {  391, 8708,  116689,  5, }, // Asia/Qostanay
    {  391, 4269,  116694,  5, }, // Asia/Qyzylorda
    {  391, 3584,  116699,  3, }, // Asia/Riyadh
    {  391, 4408,  116702,  3, }, // Asia/Sakhalin
    {  391, 7245,  116705,  5, }, // Asia/Samarkand
    {  391, 3279,  116710,  2, }, // Asia/Seoul
    {  391, 1053,  116712,  3, }, // Asia/Shanghai
    {  391, 3300,  116715,  4, }, // Asia/Singapore
    {  391, 4297,  116719,  8, }, // Asia/Srednekolymsk
    {  391, 3263,  116727,  4, }, // Asia/Taipei
    {  391, 4781,  116731,  4, }, // Asia/Tashkent
    {  391, 3915,  116735,  4, }, // Asia/Tbilisi
    {  391, 2772,  116739,  3, }, // Asia/Tehran
    {  391, 1317,  116742,  2, }, // Asia/Thimphu
    {  391, 2821,  116744,  2, }, // Asia/Tokyo
    {  391, 4572,  116746,  3, }, // Asia/Tomsk
    {  391, 1021,  116749,  5, }, // Asia/Ulaanbaatar
    {  391, 1159,  116754,  4, }, // Asia/Urumqi
    {  391, 8722,  116758,  5, }, // Asia/Ust-Nera
    {  391, 6977,  116763,  4, }, // Asia/Vientiane
    {  391, 4710,  116767,  7, }, // Asia/Vladivostok
    {  391, 4828,  116774,  4, }, // Asia/Yakutsk
    {  391, 1235,  116778,  2, }, // Asia/Yangon
    {  391, 3883,  116780,  7, }, // Asia/Yekaterinburg
    {  391, 3747,  116787,  3, }, // Asia/Yerevan
    {  391, 3663,  116790,  4, }, // Atlantic/Azores
    {  391, 5671,  116794,  3, }, // Atlantic/Bermuda
    {  391, 6628,  116797,  7, }, // Atlantic/Canary
    {  391, 3727,  116804,  6, }, // Atlantic/Cape_Verde
    {  391, 1395,  116810,  5, }, // Atlantic/Faroe
    {  391, 8736,  116815,  4, }, // Atlantic/Madeira
    {  391, 2748,  116819,  5, }, // Atlantic/Reykjavik
    {  391, 7272,  116824,  6, }, // Atlantic/South_Georgia
    {  391, 6821,  116830,  7, }, // Atlantic/St_Helena
    {  391, 6644,  116837,  3, }, // Atlantic/Stanley
    {  391, 1670,  116840,  5, }, // Australia/Adelaide
    {  391, 1635,  116845,  5, }, // Australia/Brisbane
    {  391, 1799,  116850,  4, }, // Australia/Broken_Hill
    {  391, 1583,  116854,  2, }, // Australia/Darwin
    {  391, 3637,  116856,  3, }, // Australia/Eucla
    {  391, 1516,  116859,  3, }, // Australia/Hobart
    {  391, 8753,  116862,  3, }, // Australia/Lindeman
    {  391, 1547,  116865,  5, }, // Australia/Lord_Howe
    {  391, 1727,  116870,  3, }, // Australia/Melbourne
    {  391, 1762,  116873,  2, }, // Australia/Perth
    {  391, 1463,  116875,  3, }, // Australia/Sydney
    {  391, 6441,  116878,  5, }, // Europe/Amsterdam
    {  391, 6216,  116883,  3, }, // Europe/Andorra
    {  391, 3620,  116886,  5, }, // Europe/Astrakhan
    {  391, 2300,  116891,  3, }, // Europe/Athens
    {  391, 6336,  116894,  5, }, // Europe/Belgrade
    {  391, 4757,  116899,  3, }, // Europe/Berlin
    {  391, 6488,  116902,  6, }, // Europe/Bratislava
    {  391, 2167,  116908,  3, }, // Europe/Brussels
    {  391, 3928,  116911,  5, }, // Europe/Bucharest
    {  391, 3806,  116916,  5, }, // Europe/Budapest
    {  391, 8772,  116921,  3, }, // Europe/Busingen
    {  391, 2583,  116924,  4, }, // Europe/Chisinau
    {  391, 6289,  116928,  4, }, // Europe/Copenhagen
    {  391, 2338,  116932,  3, }, // Europe/Dublin
    {  391, 6307,  116935,  4, }, // Europe/Gibraltar
    {  391, 8788,  116939,  2, }, // Europe/Guernsey
    {  391, 6612,  116941,  3, }, // Europe/Helsinki
    {  391, 8804,  116944,  2, }, // Europe/Isle_of_Man
    {  391, 1130,  116946,  4, }, // Europe/Istanbul
    {  391, 8823,  116950,  2, }, // Europe/Jersey
    {  391, 3979,  116952,  6, }, // Europe/Kaliningrad
    {  391, 8929,  116958,  3, }, // Europe/Kirov
    {  391, 2527,  116961,  3, }, // Europe/Kyiv
    {  391, 3213,  116964,  3, }, // Europe/Lisbon
    {  391, 6506,  116967,  4, }, // Europe/Ljubljana
    {  391, 2501,  116971,  2, }, // Europe/London
    {  391, 6365,  116973,  5, }, // Europe/Luxembourg
    {  391, 4892,  116978,  4, }, // Europe/Madrid
    {  391, 6397,  116982,  2, }, // Europe/Malta
    {  391, 6582,  116984,  4, }, // Europe/Mariehamn
    {  391, 3693,  116988,  3, }, // Europe/Minsk
    {  391, 6410,  116991,  3, }, // Europe/Monaco
    {  391, 3537,  116994,  4, }, // Europe/Moscow
    {  391, 6458,  116998,  3, }, // Europe/Oslo
    {  391, 4284,  117001,  2, }, // Europe/Paris
    {  391, 6424,  117003,  5, }, // Europe/Podgorica
    {  391, 6275,  117008,  3, }, // Europe/Prague
    {  391, 8837,  117011,  2, }, // Europe/Riga
    {  391, 6324,  117013,  2, }, // Europe/Rome
    {  391, 4331,  117015,  3, }, // Europe/Samara
    {  391, 6470,  117018,  4, }, // Europe/San_Marino
    {  391, 6245,  117022,  4, }, // Europe/Sarajevo
    {  391, 4451,  117026,  4, }, // Europe/Saratov
    {  391, 8849,  117030,  4, }, // Europe/Simferopol
    {  391, 6383,  117034,  3, }, // Europe/Skopje
    {  391, 6599,  117037,  3, }, // Europe/Sofia
    {  391, 6523,  117040,  4, }, // Europe/Stockholm
    {  391, 8867,  117044,  2, }, // Europe/Tallinn
    {  391, 6202,  117046,  3, }, // Europe/Tirane
    {  391, 8882,  117049,  7, }, // Europe/Ulyanovsk
    {  391, 6352,  117056,  3, }, // Europe/Vaduz
    {  391, 6567,  117059,  3, }, // Europe/Vatican
    {  391, 6231,  117062,  3, }, // Europe/Vienna
    {  391, 8899,  117065,  4, }, // Europe/Vilnius
    {  391, 4727,  117069,  5, }, // Europe/Volgograd
    {  391, 3190,  117074,  4, }, // Europe/Warsaw
    {  391, 6261,  117078,  4, }, // Europe/Zagreb
    {  391, 6540,  117082,  3, }, // Europe/Zurich
    {  391, 5078,  117085,  6, }, // Indian/Antananarivo
    {  391, 6947,  117091,  3, }, // Indian/Chagos
    {  391, 6046,  117094,  5, }, // Indian/Christmas
    {  391, 6063,  117099,  3, }, // Indian/Cocos
    {  391, 5029,  117102,  3, }, // Indian/Comoro
    {  391, 6661,  117105,  4, }, // Indian/Kerguelen
    {  391, 7260,  117109,  2, }, // Indian/Mahe
    {  391, 7050,  117111,  3, }, // Indian/Maldives
    {  391, 4069,  117114,  4, }, // Indian/Mauritius
    {  391, 5098,  117118,  3, }, // Indian/Mayotte
    {  391, 7211,  117121,  4, }, // Indian/Reunion
    {  391, 4422,  117125,  3, }, // Pacific/Apia
    {  391,  932,  117128,  4, }, // Pacific/Auckland
    {  391, 3706,  117132,  3, }, // Pacific/Bougainville
    {  391, 3015,  117135,  2, }, // Pacific/Chatham
    {  391, 3157,  117137,  3, }, // Pacific/Chuuk
    {  391, 2237,  117140,  5, }, // Pacific/Easter
    {  391, 7360,  117145,  3, }, // Pacific/Efate
    {  391, 7327,  117148,  4, }, // Pacific/Fakaofo
    {  391, 3902,  117152,  2, }, // Pacific/Fiji
    {  391, 7343,  117154,  4, }, // Pacific/Funafuti
    {  391, 6678,  117158,  5, }, // Pacific/Galapagos
    {  391, 6696,  117163,  3, }, // Pacific/Gambier
    {  391, 3822,  117166,  4, }, // Pacific/Guadalcanal
    {  391, 6018,  117170,  1, }, // Pacific/Guam
    {  391, 2723,  117171,  4, }, // Pacific/Honolulu
    {  391, 3049,  117175,  4, }, // Pacific/Kanton
    {  391, 3998,  117179,  5, }, // Pacific/Kiritimati
    {  391, 7019,  117184,  3, }, // Pacific/Kosrae
    {  391, 2842,  117187,  3, }, // Pacific/Kwajalein
    {  391, 4877,  117190,  3, }, // Pacific/Majuro
    {  391, 4051,  117193,  4, }, // Pacific/Marquesas
    {  391, 8914,  117197,  4, }, // Pacific/Midway
    {  391, 7084,  117201,  3, }, // Pacific/Nauru
    {  391, 7132,  117204,  3, }, // Pacific/Niue
    {  391, 4168,  117207,  2, }, // Pacific/Norfolk
    {  391, 7098,  117209,  3, }, // Pacific/Noumea
    {  391, 3126,  117212,  4, }, // Pacific/Pago_Pago
    {  391, 7155,  117216,  3, }, // Pacific/Palau
    {  391, 7194,  117219,  3, }, // Pacific/Pitcairn
    {  391, 3096,  117222,  3, }, // Pacific/Pohnpei
    {  391, 4807,  117225,  6, }, // Pacific/Port_Moresby
    {  391, 6076,  117231,  4, }, // Pacific/Rarotonga
    {  391, 6031,  117235,  3, }, // Pacific/Saipan
    {  391, 7312,  117238,  3, }, // Pacific/Tahiti
    {  391, 6712,  117241,  3, }, // Pacific/Tarawa
    {  391, 4583,  117244,  4, }, // Pacific/Tongatapu
    {  391, 7392,  117248,  3, }, // Pacific/Wake
    {  391, 7405,  117251,  3, }, // Pacific/Wallis
    {  394,   14,       0,  6, }, // Africa/Asmara Koyraboro Senni/Latin/Mali
    {  394, 4435,       6,  8, }, // Africa/Sao_Tome
    {  394, 4252,      14,  8, }, // America/Asuncion
    {  394,  237,      22,  8, }, // America/Atikokan
    {  394, 7623,      30, 17, }, // America/Bahia_Banderas
    {  394, 3868,      47,  6, }, // America/Cancun
    {  394, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  394, 5723,      66,  7, }, // America/Curacao
    {  394,  481,      73, 13, }, // America/Indiana/Knox
    {  394, 7961,      86, 16, }, // America/Indiana/Marengo
    {  394, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  394, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  394, 8038,     139, 14, }, // America/Indiana/Vevay
    {  394, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  394, 8086,     171, 16, }, // America/Indiana/Winamac
    {  394, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  394, 5932,     207, 22, }, // America/Lower_Princes
    {  394, 8220,     229,  6, }, // America/Merida
    {  394, 1850,     235, 19, }, // America/Noronha
    {  394, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  394, 8329,     274, 20, }, // America/North_Dakota/Center
    {  394, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  394,  393,     317,  4, }, // America/Nuuk
    {  394, 6899,     321, 16, }, // America/Scoresbysund
    {  394, 8508,     337, 14, }, // America/St_Barthelemy
    {  394, 2061,     351, 10, }, // America/St_Johns
    {  394, 5863,     361,  9, }, // America/St_Kitts
    {  394, 5880,     370,  9, }, // America/St_Lucia
    {  394,  855,     379, 10, }, // America/St_Thomas
    {  394, 5913,     389, 11, }, // America/St_Vincent
    {  394, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  394, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  394, 1185,     448,  9, }, // Asia/Kathmandu
    {  394,  992,     457,  7, }, // Asia/Kolkata
    {  394, 1211,     464,  5, }, // Asia/Macau
    {  394, 1235,     477,  6, }, // Asia/Yangon
    {  394, 1395,     483,  5, }, // Atlantic/Faroe
    {  394, 6821,     488, 10, }, // Atlantic/St_Helena
    {  394, 2527,     514,  4, }, // Europe/Kyiv
    {  394, 7211,     547,  7, }, // Indian/Reunion
    {  394, 3157,     554,  5, }, // Pacific/Chuuk
    {  394, 3096,     586,  7, }, // Pacific/Pohnpei
    {  395,   14,       0,  6, }, // Africa/Asmara Koyra Chiini/Latin/Mali
    {  395, 4435,       6,  8, }, // Africa/Sao_Tome
    {  395, 4252,      14,  8, }, // America/Asuncion
    {  395,  237,      22,  8, }, // America/Atikokan
    {  395, 7623,      30, 17, }, // America/Bahia_Banderas
    {  395, 3868,      47,  6, }, // America/Cancun
    {  395, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  395, 5723,      66,  7, }, // America/Curacao
    {  395,  481,      73, 13, }, // America/Indiana/Knox
    {  395, 7961,      86, 16, }, // America/Indiana/Marengo
    {  395, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  395, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  395, 8038,     139, 14, }, // America/Indiana/Vevay
    {  395, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  395, 8086,     171, 16, }, // America/Indiana/Winamac
    {  395, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  395, 5932,     207, 22, }, // America/Lower_Princes
    {  395, 8220,     229,  6, }, // America/Merida
    {  395, 1850,     235, 19, }, // America/Noronha
    {  395, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  395, 8329,     274, 20, }, // America/North_Dakota/Center
    {  395, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  395,  393,     317,  4, }, // America/Nuuk
    {  395, 6899,     321, 16, }, // America/Scoresbysund
    {  395, 8508,     337, 14, }, // America/St_Barthelemy
    {  395, 2061,     351, 10, }, // America/St_Johns
    {  395, 5863,     361,  9, }, // America/St_Kitts
    {  395, 5880,     370,  9, }, // America/St_Lucia
    {  395,  855,     379, 10, }, // America/St_Thomas
    {  395, 5913,     389, 11, }, // America/St_Vincent
    {  395, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  395, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  395, 1185,     448,  9, }, // Asia/Kathmandu
    {  395,  992,     457,  7, }, // Asia/Kolkata
    {  395, 1211,     464,  5, }, // Asia/Macau
    {  395, 1235,     477,  6, }, // Asia/Yangon
    {  395, 1395,     483,  5, }, // Atlantic/Faroe
    {  395, 6821,     488, 10, }, // Atlantic/St_Helena
    {  395, 2527,     514,  4, }, // Europe/Kyiv
    {  395, 7211,     547,  7, }, // Indian/Reunion
    {  395, 3157,     554,  5, }, // Pacific/Chuuk
    {  395, 3096,     586,  7, }, // Pacific/Pohnpei
    {  396,   14,       0,  6, }, // Africa/Asmara Kpelle/Latin/Liberia
    {  396, 4435,       6,  8, }, // Africa/Sao_Tome
    {  396, 4252,      14,  8, }, // America/Asuncion
    {  396,  237,      22,  8, }, // America/Atikokan
    {  396, 7623,      30, 17, }, // America/Bahia_Banderas
    {  396, 3868,      47,  6, }, // America/Cancun
    {  396, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  396, 5723,      66,  7, }, // America/Curacao
    {  396,  481,      73, 13, }, // America/Indiana/Knox
    {  396, 7961,      86, 16, }, // America/Indiana/Marengo
    {  396, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  396, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  396, 8038,     139, 14, }, // America/Indiana/Vevay
    {  396, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  396, 8086,     171, 16, }, // America/Indiana/Winamac
    {  396, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  396, 5932,     207, 22, }, // America/Lower_Princes
    {  396, 8220,     229,  6, }, // America/Merida
    {  396, 1850,     235, 19, }, // America/Noronha
    {  396, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  396, 8329,     274, 20, }, // America/North_Dakota/Center
    {  396, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  396,  393,     317,  4, }, // America/Nuuk
    {  396, 6899,     321, 16, }, // America/Scoresbysund
    {  396, 8508,     337, 14, }, // America/St_Barthelemy
    {  396, 2061,     351, 10, }, // America/St_Johns
    {  396, 5863,     361,  9, }, // America/St_Kitts
    {  396, 5880,     370,  9, }, // America/St_Lucia
    {  396,  855,     379, 10, }, // America/St_Thomas
    {  396, 5913,     389, 11, }, // America/St_Vincent
    {  396, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  396, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  396, 1185,     448,  9, }, // Asia/Kathmandu
    {  396,  992,     457,  7, }, // Asia/Kolkata
    {  396, 1211,     464,  5, }, // Asia/Macau
    {  396, 1235,     477,  6, }, // Asia/Yangon
    {  396, 1395,     483,  5, }, // Atlantic/Faroe
    {  396, 6821,     488, 10, }, // Atlantic/St_Helena
    {  396, 2527,     514,  4, }, // Europe/Kyiv
    {  396, 7211,     547,  7, }, // Indian/Reunion
    {  396, 3157,     554,  5, }, // Pacific/Chuuk
    {  396, 3096,     586,  7, }, // Pacific/Pohnpei
    {  398, 6760,    4649,  4, }, // Africa/Accra Kurdish/Latin/Turkey
    {  398, 7420,  117254,  7, }, // Africa/Algiers
    {  398,   14,       0,  6, }, // Africa/Asmara
    {  398, 2320,  117261,  6, }, // Africa/Cairo
    {  398, 4117,   33276, 10, }, // Africa/Casablanca
    {  398, 7449,  117267,  5, }, // Africa/Ceuta
    {  398, 6773,  117272,  7, }, // Africa/Conakry
    {  398, 5130,  117279,  9, }, // Africa/Dar_es_Salaam
    {  398, 5043,  117288,  6, }, // Africa/Djibouti
    {  398, 4524,  117294,  6, }, // Africa/Khartoum
    {  398, 4973,  117300,  6, }, // Africa/Kigali
    {  398, 5306,  117306,  7, }, // Africa/Kinshasa
    {  398, 5336,  117313,  8, }, // Africa/Libreville
    {  398, 5197,  117321,  6, }, // Africa/Maseru
    {  398, 5113,  117327,  8, }, // Africa/Mogadishu
    {  398, 3853,  117335,  7, }, // Africa/Nairobi
    {  398, 4435,       6,  8, }, // Africa/Sao_Tome
    {  398, 2866,  117342,  7, }, // Africa/Tripoli
    {  398, 6554,  117349,  5, }, // Africa/Tunis
    {  398, 5640,  117354,  5, }, // America/Aruba
    {  398, 4252,      14,  8, }, // America/Asuncion
    {  398,  237,      22,  8, }, // America/Atikokan
    {  398, 7623,  117359, 14, }, // America/Bahia_Banderas
    {  398, 5407,  117373,  6, }, // America/Belize
    {  398, 3868,  117379,  6, }, // America/Cancun
    {  398, 7774,  117385, 13, }, // America/Ciudad_Juarez
    {  398, 5422,  117398, 10, }, // America/Costa_Rica
    {  398, 5723,      66,  7, }, // America/Curacao
    {  398, 5739,  117408,  8, }, // America/Dominica
    {  398,  481,      73, 13, }, // America/Indiana/Knox
    {  398, 7961,      86, 16, }, // America/Indiana/Marengo
    {  398, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  398, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  398, 8038,     139, 14, }, // America/Indiana/Vevay
    {  398, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  398, 8086,     171, 16, }, // America/Indiana/Winamac
    {  398, 2799,  117416,  7, }, // America/Jamaica
    {  398, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  398, 5932,     207, 22, }, // America/Lower_Princes
    {  398, 8220,  117423,  6, }, // America/Merida
    {  398, 1850,     235, 19, }, // America/Noronha
    {  398, 8301,  117429, 22, }, // America/North_Dakota/Beulah
    {  398, 8329,  117451, 22, }, // America/North_Dakota/Center
    {  398, 8357,  117473, 25, }, // America/North_Dakota/New_Salem
    {  398,  393,     317,  4, }, // America/Nuuk
    {  398, 5843,  117498, 10, }, // America/Puerto_Rico
    {  398, 6899,     321, 16, }, // America/Scoresbysund
    {  398, 8508,  117508, 16, }, // America/St_Barthelemy
    {  398, 2061,     351, 10, }, // America/St_Johns
    {  398, 5863,     361,  9, }, // America/St_Kitts
    {  398, 5880,     370,  9, }, // America/St_Lucia
    {  398,  855,     379, 10, }, // America/St_Thomas
    {  398, 5913,     389, 11, }, // America/St_Vincent
    {  398, 6094,  117524,  5, }, // Antarctica/Davis
    {  398, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  398, 5395,  117529,  6, }, // Asia/Almaty
    {  398, 3968,  117535,  4, }, // Asia/Amman
    {  398, 5511,    7600,  6, }, // Asia/Anadyr
    {  398, 5523,   64292,  5, }, // Asia/Aqtau
    {  398,  964,  117539,  7, }, // Asia/Ashgabat
    {  398, 8624,  117546,  6, }, // Asia/Atyrau
    {  398, 3607,  117552,  5, }, // Asia/Baghdad
    {  398, 5534,  117557,  7, }, // Asia/Bahrain
    {  398, 3653,  117564,  4, }, // Asia/Baku
    {  398, 4086,  117568,  6, }, // Asia/Beirut
    {  398, 3778,  117574,  6, }, // Asia/Bishkek
    {  398, 5989,  117580,  6, }, // Asia/Brunei
    {  398, 4601,  117586,  4, }, // Asia/Chita
    {  398, 4511,    5637,  7, }, // Asia/Colombo
    {  398, 4540,  117590,  3, }, // Asia/Damascus
    {  398, 1093,    4717,  4, }, // Asia/Dhaka
    {  398, 6192,  117593,  4, }, // Asia/Dili
    {  398, 3596,  117597,  5, }, // Asia/Dubai
    {  398, 6159,  117602,  7, }, // Asia/Dushanbe
    {  398, 8651,  117609,  4, }, // Asia/Gaza
    {  398, 1259,  117613, 19, }, // Asia/Ho_Chi_Minh
    {  398, 7006,   22440,  7, }, // Asia/Jakarta
    {  398, 6992,  117632,  8, }, // Asia/Jayapura
    {  398, 1290,  117640,  4, }, // Asia/Jerusalem
    {  398, 3560,  117644,  5, }, // Asia/Kabul
    {  398, 4316,   22469,  8, }, // Asia/Kamchatka
    {  398, 4239,  117649,  6, }, // Asia/Karachi
    {  398, 1185,  117655,  8, }, // Asia/Kathmandu
    {  398, 8661,  117663,  7, }, // Asia/Khandyga
    {  398,  992,     457,  7, }, // Asia/Kolkata
    {  398, 4859,  117670, 12, }, // Asia/Kuala_Lumpur
    {  398, 5976,  117682,  6, }, // Asia/Kuching
    {  398, 5547,  117688,  6, }, // Asia/Kuwait
    {  398, 1211,    7731,  5, }, // Asia/Macau
    {  398, 1349,    7736,  7, }, // Asia/Makassar
    {  398, 7182,  117694,  6, }, // Asia/Manila
    {  398, 6920,    4326,  6, }, // Asia/Muscat
    {  398, 2554,  117700,  7, }, // Asia/Nicosia
    {  398, 5559,  117707,  5, }, // Asia/Qatar
    {  398, 4269,  117712,  9, }, // Asia/Qyzylorda
    {  398, 3584,   20874,  5, }, // Asia/Riyadh
    {  398, 4408,  117721,  7, }, // Asia/Sakhalin
    {  398, 7245,  117728,  9, }, // Asia/Samarkand
    {  398, 3279,  117737,  4, }, // Asia/Seoul
    {  398, 1053,  117741,  7, }, // Asia/Shanghai
    {  398, 3300,  117748,  8, }, // Asia/Singapore
    {  398, 3263,  117756,  6, }, // Asia/Taipei
    {  398, 4781,  117762,  7, }, // Asia/Tashkent
    {  398, 3915,  117769,  6, }, // Asia/Tbilisi
    {  398, 1021,  117775,  9, }, // Asia/Ulaanbaatar
    {  398, 1159,  117784,  6, }, // Asia/Urumqi
    {  398, 8722,  117790,  8, }, // Asia/Ust-Nera
    {  398, 1235,     477,  6, }, // Asia/Yangon
    {  398, 3747,  117798,  5, }, // Asia/Yerevan
    {  398, 3663,  117803, 13, }, // Atlantic/Azores
    {  398, 5671,  117816,  7, }, // Atlantic/Bermuda
    {  398, 6628,  117823, 17, }, // Atlantic/Canary
    {  398, 3727,   57380,  9, }, // Atlantic/Cape_Verde
    {  398, 1395,     483,  5, }, // Atlantic/Faroe
    {  398, 7272,  117840, 15, }, // Atlantic/South_Georgia
    {  398, 6821,     488, 10, }, // Atlantic/St_Helena
    {  398, 1463,  117855,  6, }, // Australia/Sydney
    {  398, 6216,   33638,  6, }, // Europe/Andorra
    {  398, 3620,  117861,  8, }, // Europe/Astrakhan
    {  398, 2300,  117869,  5, }, // Europe/Athens
    {  398, 6336,   27015,  7, }, // Europe/Belgrade
    {  398, 4757,  117874,  6, }, // Europe/Berlin
    {  398, 2167,    8077,  7, }, // Europe/Brussels
    {  398, 3928,  117880,  6, }, // Europe/Bucharest
    {  398, 3806,  117886,  8, }, // Europe/Budapest
    {  398, 8772,  117894,  8, }, // Europe/Busingen
    {  398, 2583,  117902,  7, }, // Europe/Chisinau
    {  398, 6289,    4475,  8, }, // Europe/Copenhagen
    {  398, 2338,  117909,  6, }, // Europe/Dublin
    {  398, 6307,  117915, 11, }, // Europe/Gibraltar
    {  398, 6612,  117926,  8, }, // Europe/Helsinki
    {  398, 8804,  117934, 11, }, // Europe/Isle_of_Man
    {  398, 1130,  117945,  7, }, // Europe/Istanbul
    {  398, 3979,  117952, 11, }, // Europe/Kaliningrad
    {  398, 8929,  117963,  5, }, // Europe/Kirov
    {  398, 2527,  117968,  4, }, // Europe/Kyiv
    {  398, 3213,  117972,  6, }, // Europe/Lisbon
    {  398, 2501,   95370,  6, }, // Europe/London
    {  398, 6365,  117978, 10, }, // Europe/Luxembourg
    {  398, 4892,  117988,  6, }, // Europe/Madrid
    {  398, 3693,  117994,  5, }, // Europe/Minsk
    {  398, 6410,    6044,  6, }, // Europe/Monaco
    {  398, 3537,   64446,  7, }, // Europe/Moscow
    {  398, 4284,  117999,  5, }, // Europe/Paris
    {  398, 6424,  118004,  9, }, // Europe/Podgorica
    {  398, 6275,    8201,  4, }, // Europe/Prague
    {  398, 8837,  118013,  4, }, // Europe/Riga
    {  398, 6324,    6064,  4, }, // Europe/Rome
    {  398, 6470,  118017, 10, }, // Europe/San_Marino
    {  398, 6245,  118027, 10, }, // Europe/Sarajevo
    {  398, 6383,  118037,  5, }, // Europe/Skopje
    {  398, 6599,   99330,  5, }, // Europe/Sofia
    {  398, 6523,    8242,  8, }, // Europe/Stockholm
    {  398, 8867,  118042,  5, }, // Europe/Tallinn
    {  398, 6202,  118047,  5, }, // Europe/Tirane
    {  398, 6567,  118052,  7, }, // Europe/Vatican
    {  398, 6231,  118059,  6, }, // Europe/Vienna
    {  398, 8899,  118065,  6, }, // Europe/Vilnius
    {  398, 3190,  118071,  7, }, // Europe/Warsaw
    {  398, 6540,  118078,  5, }, // Europe/Zurich
    {  398, 5029,    8305,  5, }, // Indian/Comoro
    {  398, 7050,  118083,  6, }, // Indian/Maldives
    {  398, 7211,     547,  7, }, // Indian/Reunion
    {  398, 3157,     554,  5, }, // Pacific/Chuuk
    {  398, 3902,  118089,  4, }, // Pacific/Fiji
    {  398, 3096,     586,  7, }, // Pacific/Pohnpei
    {  398, 7312,  118093,  6, }, // Pacific/Tahiti
    {  399,   14,       0,  6, }, // Africa/Asmara Kwasio/Latin/Cameroon
    {  399, 4435,       6,  8, }, // Africa/Sao_Tome
    {  399, 4252,      14,  8, }, // America/Asuncion
    {  399,  237,      22,  8, }, // America/Atikokan
    {  399, 7623,      30, 17, }, // America/Bahia_Banderas
    {  399, 3868,      47,  6, }, // America/Cancun
    {  399, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  399, 5723,      66,  7, }, // America/Curacao
    {  399,  481,      73, 13, }, // America/Indiana/Knox
    {  399, 7961,      86, 16, }, // America/Indiana/Marengo
    {  399, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  399, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  399, 8038,     139, 14, }, // America/Indiana/Vevay
    {  399, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  399, 8086,     171, 16, }, // America/Indiana/Winamac
    {  399, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  399, 5932,     207, 22, }, // America/Lower_Princes
    {  399, 8220,     229,  6, }, // America/Merida
    {  399, 1850,     235, 19, }, // America/Noronha
    {  399, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  399, 8329,     274, 20, }, // America/North_Dakota/Center
    {  399, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  399,  393,     317,  4, }, // America/Nuuk
    {  399, 6899,     321, 16, }, // America/Scoresbysund
    {  399, 8508,     337, 14, }, // America/St_Barthelemy
    {  399, 2061,     351, 10, }, // America/St_Johns
    {  399, 5863,     361,  9, }, // America/St_Kitts
    {  399, 5880,     370,  9, }, // America/St_Lucia
    {  399,  855,     379, 10, }, // America/St_Thomas
    {  399, 5913,     389, 11, }, // America/St_Vincent
    {  399, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  399, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  399, 1185,     448,  9, }, // Asia/Kathmandu
    {  399,  992,     457,  7, }, // Asia/Kolkata
    {  399, 1211,     464,  5, }, // Asia/Macau
    {  399, 1235,     477,  6, }, // Asia/Yangon
    {  399, 1395,     483,  5, }, // Atlantic/Faroe
    {  399, 6821,     488, 10, }, // Atlantic/St_Helena
    {  399, 2527,     514,  4, }, // Europe/Kyiv
    {  399, 7211,     547,  7, }, // Indian/Reunion
    {  399, 3157,     554,  5, }, // Pacific/Chuuk
    {  399, 3096,     586,  7, }, // Pacific/Pohnpei
    {  400, 6788,   36346,  7, }, // Africa/Abidjan Kyrgyz/Cyrillic/Kyrgyzstan
    {  400, 6760,     610,  5, }, // Africa/Accra
    {  400, 5059,     615, 11, }, // Africa/Addis_Ababa
    {  400, 7420,     626,  5, }, // Africa/Algiers
    {  400,   14,   27197,  6, }, // Africa/Asmara
    {  400,   44,     637,  6, }, // Africa/Bamako
    {  400, 5257,  118099,  4, }, // Africa/Bangui
    {  400, 6746,     648,  6, }, // Africa/Banjul
    {  400, 7435,     654,  5, }, // Africa/Bissau
    {  400, 4957,   27220,  8, }, // Africa/Blantyre
    {  400, 5287,  118103, 10, }, // Africa/Brazzaville
    {  400, 4922,     676,  9, }, // Africa/Bujumbura
    {  400, 2320,     685,  4, }, // Africa/Cairo
    {  400, 4117,     689, 10, }, // Africa/Casablanca
    {  400, 7449,     699,  5, }, // Africa/Ceuta
    {  400, 6773,     704,  7, }, // Africa/Conakry
    {  400, 6840,     711,  5, }, // Africa/Dakar
    {  400, 5130,  118113, 13, }, // Africa/Dar_es_Salaam
    {  400, 5043,  118126,  6, }, // Africa/Djibouti
    {  400, 5243,     734,  5, }, // Africa/Douala
    {  400, 5166,  118132,  7, }, // Africa/El_Aaiun
    {  400, 6853,     746,  7, }, // Africa/Freetown
    {  400, 4906,     753,  8, }, // Africa/Gaborone
    {  400, 5015,   33913,  6, }, // Africa/Harare
    {  400, 4479,   51572, 12, }, // Africa/Johannesburg
    {  400, 4499,  118139,  4, }, // Africa/Juba
    {  400, 5151,     783,  7, }, // Africa/Kampala
    {  400, 4524,   33930,  6, }, // Africa/Khartoum
    {  400, 4973,     796,  6, }, // Africa/Kigali
    {  400, 5306,     802,  7, }, // Africa/Kinshasa
    {  400, 4744,     809,  5, }, // Africa/Lagos
    {  400, 5336,  118143,  9, }, // Africa/Libreville
    {  400, 6869,     822,  4, }, // Africa/Lome
    {  400, 5211,     826,  6, }, // Africa/Luanda
    {  400, 4939,     832,  9, }, // Africa/Lubumbashi
    {  400, 5001,     841,  6, }, // Africa/Lusaka
    {  400, 5322,     847,  6, }, // Africa/Malabo
    {  400, 4987,   33945,  6, }, // Africa/Maputo
    {  400, 5197,     859,  6, }, // Africa/Maseru
    {  400, 5182,     865,  7, }, // Africa/Mbabane
    {  400, 5113,   36411,  8, }, // Africa/Mogadishu
    {  400, 7034,     880,  8, }, // Africa/Monrovia
    {  400, 3853,   36427,  7, }, // Africa/Nairobi
    {  400, 5271,  118152,  7, }, // Africa/Ndjamena
    {  400, 5354,  118159,  5, }, // Africa/Niamey
    {  400, 6803,     908,  7, }, // Africa/Nouakchott
    {  400, 6727,     915,  8, }, // Africa/Ouagadougou
    {  400, 5225,     923, 10, }, // Africa/Porto-Novo
    {  400, 4435,   33984,  8, }, // Africa/Sao_Tome
    {  400, 2866,     941,  7, }, // Africa/Tripoli
    {  400, 6554,     948,  5, }, // Africa/Tunis
    {  400, 4152,  118164,  7, }, // Africa/Windhoek
    {  400,  132,     960,  4, }, // America/Adak
    {  400, 3346,  118171,  7, }, // America/Anchorage
    {  400, 5607,   36448,  7, }, // America/Anguilla
    {  400, 5624,     978,  7, }, // America/Antigua
    {  400, 4554,     985,  9, }, // America/Araguaina
    {  400,  166,   51608, 12, }, // America/Argentina/Buenos_Aires
    {  400,   91,    1006,  9, }, // America/Argentina/Catamarca
    {  400,  270,    1015,  7, }, // America/Argentina/Cordoba
    {  400,  441,   27487,  5, }, // America/Argentina/Jujuy
    {  400, 7462,    1027,  8, }, // America/Argentina/La_Rioja
    {  400,  565,   34055,  7, }, // America/Argentina/Mendoza
    {  400, 7489,  118178, 12, }, // America/Argentina/Rio_Gallegos
    {  400, 7520,    1053,  5, }, // America/Argentina/Salta
    {  400, 7544,   34073,  8, }, // America/Argentina/San_Juan
    {  400, 5580,   34081,  7, }, // America/Argentina/San_Luis
    {  400, 7571,    1074,  7, }, // America/Argentina/Tucuman
    {  400, 7597,  118190,  6, }, // America/Argentina/Ushuaia
    {  400, 5640,    1087,  5, }, // America/Aruba
    {  400, 4252,   34095,  8, }, // America/Asuncion
    {  400,  237,   34103,  8, }, // America/Atikokan
    {  400, 3679,  118196,  5, }, // America/Bahia
    {  400, 7623,  118201, 14, }, // America/Bahia_Banderas
    {  400, 5654,    1131,  8, }, // America/Barbados
    {  400, 7646,   34130,  5, }, // America/Belem
    {  400, 5407,    1144,  5, }, // America/Belize
    {  400, 7660,    1149, 12, }, // America/Blanc-Sablon
    {  400, 7681,   34154,  9, }, // America/Boa_Vista
    {  400, 4361,    1170,  6, }, // America/Bogota
    {  400, 7699,   51656,  5, }, // America/Boise
    {  400, 7713,  118215, 11, }, // America/Cambridge_Bay
    {  400, 7735,   34179, 12, }, // America/Campo_Grande
    {  400, 3868,    1205,  6, }, // America/Cancun
    {  400, 4694,    1211,  7, }, // America/Caracas
    {  400, 4345,   36549,  5, }, // America/Cayenne
    {  400, 5496,   27629,  6, }, // America/Cayman
    {  400, 2260,    1247,  6, }, // America/Chicago
    {  400, 7756,  118226,  8, }, // America/Chihuahua
    {  400, 7774,   56292, 13, }, // America/Ciudad_Juarez
    {  400, 5422,    1259, 10, }, // America/Costa_Rica
    {  400, 7796,    1269,  7, }, // America/Creston
    {  400, 3791,  118234,  6, }, // America/Cuiaba
    {  400, 5723,   36606,  7, }, // America/Curacao
    {  400, 7812,   36613, 12, }, // America/Danmarkshavn
    {  400, 7833,    1302,  6, }, // America/Dawson
    {  400, 7848,  118240, 11, }, // America/Dawson_Creek
    {  400,  805,    1319,  6, }, // America/Denver
    {  400, 3465,   36649,  7, }, // America/Detroit
    {  400, 5739,    1332,  8, }, // America/Dominica
    {  400,  893,   51681,  8, }, // America/Edmonton
    {  400, 7869,   51689,  8, }, // America/Eirunepe
    {  400, 5441,  118251, 11, }, // America/El_Salvador
    {  400, 7886,   51697, 12, }, // America/Fort_Nelson
    {  400, 7906,    1375,  9, }, // America/Fortaleza
    {  400, 7924,   36683,  9, }, // America/Glace_Bay
    {  400, 6881,   36692,  7, }, // America/Goose_Bay
    {  400, 4612,  118262, 10, }, // America/Grand_Turk
    {  400, 5770,    1410,  7, }, // America/Grenada
    {  400, 5786,  118272,  9, }, // America/Guadeloupe
    {  400, 3760,    1426,  9, }, // America/Guatemala
    {  400, 4841,  118281,  8, }, // America/Guayaquil
    {  400, 6932,  118289,  6, }, // America/Guyana
    {  400, 1939,   51739,  8, }, // America/Halifax
    {  400, 2281,    1457,  6, }, // America/Havana
    {  400, 7942,  118295,  9, }, // America/Hermosillo
    {  400,  348,    1471, 12, }, // America/Indiana/Indianapolis
    {  400,  481,    1483, 13, }, // America/Indiana/Knox
    {  400, 7961,    1496, 16, }, // America/Indiana/Marengo
    {  400, 7985,  118304, 19, }, // America/Indiana/Petersburg
    {  400, 8012,  118323, 18, }, // America/Indiana/Tell_City
    {  400, 8038,  106315, 14, }, // America/Indiana/Vevay
    {  400, 8060,  118341, 15, }, // America/Indiana/Vincennes
    {  400, 8086,   56355, 16, }, // America/Indiana/Winamac
    {  400, 8110,    1570,  6, }, // America/Inuvik
    {  400,  660,    1576,  7, }, // America/Iqaluit
    {  400, 2799,   28011,  6, }, // America/Jamaica
    {  400, 5380,   34468,  4, }, // America/Juneau
    {  400,  521,  118356,  8, }, // America/Kentucky/Louisville
    {  400, 8125,  118364, 20, }, // America/Kentucky/Monticello
    {  400, 5704,   36751, 10, }, // America/Kralendijk
    {  400, 4376,   36761,  6, }, // America/La_Paz
    {  400, 7169,    1638,  4, }, // America/Lima
    {  400, 3239,  118384, 11, }, // America/Los_Angeles
    {  400, 5932,  118395, 19, }, // America/Lower_Princes
    {  400, 8153,   36799,  6, }, // America/Maceio
    {  400, 8168,    1677,  7, }, // America/Managua
    {  400, 1908,    1684,  6, }, // America/Manaus
    {  400, 5897,    1690,  6, }, // America/Marigot
    {  400, 5805,    1696,  8, }, // America/Martinique
    {  400, 8184,    1705,  9, }, // America/Matamoros
    {  400, 2917,   34549,  8, }, // America/Mazatlan
    {  400, 8202,    1722,  8, }, // America/Menominee
    {  400, 8220,    1730,  6, }, // America/Merida
    {  400, 8235,    1736, 10, }, // America/Metlakatla
    {  400, 2949,  118414, 12, }, // America/Mexico_City
    {  400, 4391,    1752,  7, }, // America/Miquelon
    {  400, 8254,    1759,  7, }, // America/Moncton
    {  400, 8270,   36805,  8, }, // America/Monterrey
    {  400, 4098,    1775, 10, }, // America/Montevideo
    {  400, 5824,  118426,  8, }, // America/Montserrat
    {  400, 5481,    1795,  6, }, // America/Nassau
    {  400, 2379,   51812,  8, }, // America/New_York
    {  400, 8288,    1809,  3, }, // America/Nome
    {  400, 1850,    1812,  7, }, // America/Noronha
    {  400, 8301,  118434, 20, }, // America/North_Dakota/Beulah
    {  400, 8329,  118454, 24, }, // America/North_Dakota/Center
    {  400, 8357,  118478, 24, }, // America/North_Dakota/New_Salem
    {  400,  393,    1890,  4, }, // America/Nuuk
    {  400, 8388,    1894,  7, }, // America/Ojinaga
    {  400, 2356,    1901,  6, }, // America/Panama
    {  400, 6173,    1907, 10, }, // America/Paramaribo
    {  400, 2973,   34673,  6, }, // America/Phoenix
    {  400, 3945,    1923, 12, }, // America/Port-au-Prince
    {  400, 5954,  118502, 13, }, // America/Port_of_Spain
    {  400, 8404,  118515, 11, }, // America/Porto_Velho
    {  400, 5843,   51888, 11, }, // America/Puerto_Rico
    {  400, 4030,    1970, 12, }, // America/Punta_Arenas
    {  400, 8424,  118526, 12, }, // America/Rankin_Inlet
    {  400, 8445,    1994,  6, }, // America/Recife
    {  400, 1995,   34741,  6, }, // America/Regina
    {  400, 8460,  118538,  7, }, // America/Resolute
    {  400,  695,   34754, 10, }, // America/Rio_Branco
    {  400, 8477,   34764,  8, }, // America/Santarem
    {  400, 2201,    2033,  8, }, // America/Santiago
    {  400, 6111,   34781, 13, }, // America/Santo_Domingo
    {  400, 1878,   36906,  9, }, // America/Sao_Paulo
    {  400, 6899,  118545, 16, }, // America/Scoresbysund
    {  400, 8494,    2074,  5, }, // America/Sitka
    {  400, 8508,  118561, 14, }, // America/St_Barthelemy
    {  400, 2061,  118575,  9, }, // America/St_Johns
    {  400, 5863,    2101,  9, }, // America/St_Kitts
    {  400, 5880,  118584, 11, }, // America/St_Lucia
    {  400,  855,    2121, 10, }, // America/St_Thomas
    {  400, 5913,    2131, 12, }, // America/St_Vincent
    {  400, 8530,  118595, 13, }, // America/Swift_Current
    {  400, 5461,   51949, 12, }, // America/Tegucigalpa
    {  400, 5756,    2167,  4, }, // America/Thule
    {  400,  313,    2171,  7, }, // America/Tijuana
    {  400,  608,    2178,  7, }, // America/Toronto
    {  400, 5688,    2185,  7, }, // America/Tortola
    {  400, 2093,    2192,  8, }, // America/Vancouver
    {  400, 2144,   28567,  8, }, // America/Whitehorse
    {  400,  734,  118608,  8, }, // America/Winnipeg
    {  400, 8552,   28582,  6, }, // America/Yakutat
    {  400, 6001,   37021,  5, }, // Antarctica/Casey
    {  400, 6094,  118616,  5, }, // Antarctica/Davis
    {  400, 6133,  118621, 13, }, // Antarctica/DumontDUrville
    {  400, 8568,    2249,  8, }, // Antarctica/Macquarie
    {  400, 7066,    2257,  6, }, // Antarctica/Mawson
    {  400, 7113,  118634,  8, }, // Antarctica/McMurdo
    {  400, 8589,    2272,  6, }, // Antarctica/Palmer
    {  400, 7226,    2278,  6, }, // Antarctica/Rothera
    {  400, 7295,  118642,  4, }, // Antarctica/Syowa
    {  400, 8607,    2289,  5, }, // Antarctica/Troll
    {  400, 7374,   34957,  6, }, // Antarctica/Vostok
    {  400, 1429,  118646, 11, }, // Arctic/Longyearbyen
    {  400, 5570,    2310,  4, }, // Asia/Aden
    {  400, 5395,    2314,  6, }, // Asia/Almaty
    {  400, 3968,    2320,  5, }, // Asia/Amman
    {  400, 5511,    2325,  6, }, // Asia/Anadyr
    {  400, 5523,    2331,  5, }, // Asia/Aqtau
    {  400, 5368,    2336,  6, }, // Asia/Aqtobe
    {  400,  964,    2342,  7, }, // Asia/Ashgabat
    {  400, 8624,    2349,  6, }, // Asia/Atyrau
    {  400, 3607,    2355,  6, }, // Asia/Baghdad
    {  400, 5534,   37080,  7, }, // Asia/Bahrain
    {  400, 3653,    2368,  4, }, // Asia/Baku
    {  400, 4466,    2372,  7, }, // Asia/Bangkok
    {  400, 3571,    2379,  7, }, // Asia/Barnaul
    {  400, 4086,   28695,  6, }, // Asia/Beirut
    {  400, 3778,    2392,  6, }, // Asia/Bishkek
    {  400, 5989,   28707,  6, }, // Asia/Brunei
    {  400, 4601,    2404,  4, }, // Asia/Chita
    {  400, 4511,    2408,  7, }, // Asia/Colombo
    {  400, 4540,    2415,  6, }, // Asia/Damascus
    {  400, 1093,    2421,  5, }, // Asia/Dhaka
    {  400, 6192,    2426,  4, }, // Asia/Dili
    {  400, 3596,   28728,  5, }, // Asia/Dubai
    {  400, 6159,    2435,  7, }, // Asia/Dushanbe
    {  400, 8636,    2442,  9, }, // Asia/Famagusta
    {  400, 8651,    2451,  4, }, // Asia/Gaza
    {  400, 4795,   35018,  6, }, // Asia/Hebron
    {  400, 1259,   35024,  9, }, // Asia/Ho_Chi_Minh
    {  400, 2704,   52034,  7, }, // Asia/Hong_Kong
    {  400, 4771,    2475,  4, }, // Asia/Hovd
    {  400, 4184,    2479,  7, }, // Asia/Irkutsk
    {  400, 7006,  118657,  7, }, // Asia/Jakarta
    {  400, 6992,  118664,  8, }, // Asia/Jayapura
    {  400, 1290,    2501,  9, }, // Asia/Jerusalem
    {  400, 3560,    2510,  5, }, // Asia/Kabul
    {  400, 4316,    2529,  8, }, // Asia/Kamchatka
    {  400, 4239,    2540,  6, }, // Asia/Karachi
    {  400, 1185,    2546,  8, }, // Asia/Kathmandu
    {  400, 8661,  118672,  7, }, // Asia/Khandyga
    {  400,  992,   52082,  9, }, // Asia/Kolkata
    {  400, 4197,   37110, 10, }, // Asia/Krasnoyarsk
    {  400, 4859,   35090, 12, }, // Asia/Kuala_Lumpur
    {  400, 5976,  118679,  5, }, // Asia/Kuching
    {  400, 5547,   28825,  6, }, // Asia/Kuwait
    {  400, 1211,   35108,  5, }, // Asia/Macau
    {  400, 4017,    2609,  7, }, // Asia/Magadan
    {  400, 1349,  118684,  8, }, // Asia/Makassar
    {  400, 7182,    2623,  6, }, // Asia/Manila
    {  400, 6920,   35113,  6, }, // Asia/Muscat
    {  400, 2554,  106464,  7, }, // Asia/Nicosia
    {  400, 8675,    2642, 11, }, // Asia/Novokuznetsk
    {  400, 4135,    2653, 11, }, // Asia/Novosibirsk
    {  400, 4229,    2664,  4, }, // Asia/Omsk
    {  400, 7145,   35138,  4, }, // Asia/Oral
    {  400, 6961,   52091,  8, }, // Asia/Phnom_Penh
    {  400, 8693,    2681,  9, }, // Asia/Pontianak
    {  400, 4214,   28895,  7, }, // Asia/Pyongyang
    {  400, 5559,    2697,  5, }, // Asia/Qatar
    {  400, 8708,   37137,  8, }, // Asia/Qostanay
    {  400, 4269,    2710,  9, }, // Asia/Qyzylorda
    {  400, 3584,  118692,  5, }, // Asia/Riyadh
    {  400, 4408,    2732,  7, }, // Asia/Sakhalin
    {  400, 7245,    2739,  9, }, // Asia/Samarkand
    {  400, 3279,    2748,  4, }, // Asia/Seoul
    {  400, 1053,   28933,  6, }, // Asia/Shanghai
    {  400, 3300,    2758,  8, }, // Asia/Singapore
    {  400, 4297,    2766, 13, }, // Asia/Srednekolymsk
    {  400, 3263,   61028,  6, }, // Asia/Taipei
    {  400, 4781,    2785,  7, }, // Asia/Tashkent
    {  400, 3915,   35206,  7, }, // Asia/Tbilisi
    {  400, 2772,    2796,  7, }, // Asia/Tehran
    {  400, 1317,   35220,  5, }, // Asia/Thimphu
    {  400, 2821,    2810,  5, }, // Asia/Tokyo
    {  400, 4572,    2815,  5, }, // Asia/Tomsk
    {  400, 1021,    2820, 10, }, // Asia/Ulaanbaatar
    {  400, 1159,  118697,  6, }, // Asia/Urumqi
    {  400, 8722,   52125,  9, }, // Asia/Ust-Nera
    {  400, 6977,  118703,  8, }, // Asia/Vientiane
    {  400, 4710,    2851, 11, }, // Asia/Vladivostok
    {  400, 4828,   37183,  6, }, // Asia/Yakutsk
    {  400, 1235,   29043,  6, }, // Asia/Yangon
    {  400, 3883,    2875, 12, }, // Asia/Yekaterinburg
    {  400, 3747,    2887,  6, }, // Asia/Yerevan
    {  400, 3663,    2893,  4, }, // Atlantic/Azores
    {  400, 5671,   35284,  7, }, // Atlantic/Bermuda
    {  400, 6628,  118711,  6, }, // Atlantic/Canary
    {  400, 3727,  118717, 10, }, // Atlantic/Cape_Verde
    {  400, 1395,    2954,  5, }, // Atlantic/Faroe
    {  400, 8736,   37264,  7, }, // Atlantic/Madeira
    {  400, 2748,   52214,  9, }, // Atlantic/Reykjavik
    {  400, 7272,  118727, 13, }, // Atlantic/South_Georgia
    {  400, 6821,  118740, 10, }, // Atlantic/St_Helena
    {  400, 6644,   52256,  6, }, // Atlantic/Stanley
    {  400, 1670,    3035,  8, }, // Australia/Adelaide
    {  400, 1635,    3043,  7, }, // Australia/Brisbane
    {  400, 1799,  118750, 11, }, // Australia/Broken_Hill
    {  400, 1583,    3061,  6, }, // Australia/Darwin
    {  400, 3637,   29240,  4, }, // Australia/Eucla
    {  400, 1516,    3072,  6, }, // Australia/Hobart
    {  400, 8753,    3078,  8, }, // Australia/Lindeman
    {  400, 1547,   37322,  8, }, // Australia/Lord_Howe
    {  400, 1727,   29252,  8, }, // Australia/Melbourne
    {  400, 1762,    3101,  4, }, // Australia/Perth
    {  400, 1463,   52276,  6, }, // Australia/Sydney
    {  400, 6441,    3111,  9, }, // Europe/Amsterdam
    {  400, 6216,    3120,  7, }, // Europe/Andorra
    {  400, 3620,   29281,  9, }, // Europe/Astrakhan
    {  400, 2300,   61051,  5, }, // Europe/Athens
    {  400, 6336,    3139,  7, }, // Europe/Belgrade
    {  400, 4757,    3146,  6, }, // Europe/Berlin
    {  400, 6488,    3152, 10, }, // Europe/Bratislava
    {  400, 2167,   52296,  8, }, // Europe/Brussels
    {  400, 3928,    3170,  8, }, // Europe/Bucharest
    {  400, 3806,    3178,  8, }, // Europe/Budapest
    {  400, 8772,  118761,  8, }, // Europe/Busingen
    {  400, 2583,    3209,  7, }, // Europe/Chisinau
    {  400, 6289,   52323, 10, }, // Europe/Copenhagen
    {  400, 2338,    3226,  6, }, // Europe/Dublin
    {  400, 6307,    3232,  9, }, // Europe/Gibraltar
    {  400, 8788,    3241,  6, }, // Europe/Guernsey
    {  400, 6612,   52333,  9, }, // Europe/Helsinki
    {  400, 8804,  106614,  9, }, // Europe/Isle_of_Man
    {  400, 1130,    3264,  7, }, // Europe/Istanbul
    {  400, 8823,  118769,  5, }, // Europe/Jersey
    {  400, 3979,    3276, 11, }, // Europe/Kaliningrad
    {  400, 8929,    3287,  5, }, // Europe/Kirov
    {  400, 2527,    3292,  4, }, // Europe/Kyiv
    {  400, 3213,    3296,  8, }, // Europe/Lisbon
    {  400, 6506,   29432,  7, }, // Europe/Ljubljana
    {  400, 2501,    3313,  6, }, // Europe/London
    {  400, 6365,   29445, 10, }, // Europe/Luxembourg
    {  400, 4892,    3330,  6, }, // Europe/Madrid
    {  400, 6397,   29461,  6, }, // Europe/Malta
    {  400, 6582,    3341,  9, }, // Europe/Mariehamn
    {  400, 3693,    3350,  5, }, // Europe/Minsk
    {  400, 6410,    3355,  6, }, // Europe/Monaco
    {  400, 3537,    3361,  6, }, // Europe/Moscow
    {  400, 6458,    3367,  4, }, // Europe/Oslo
    {  400, 4284,    3371,  5, }, // Europe/Paris
    {  400, 6424,    3376,  9, }, // Europe/Podgorica
    {  400, 6275,    3385,  5, }, // Europe/Prague
    {  400, 8837,    3390,  4, }, // Europe/Riga
    {  400, 6324,    3394,  3, }, // Europe/Rome
    {  400, 4331,    3397,  6, }, // Europe/Samara
    {  400, 6470,   35545, 10, }, // Europe/San_Marino
    {  400, 6245,    3413,  7, }, // Europe/Sarajevo
    {  400, 4451,    3420,  7, }, // Europe/Saratov
    {  400, 8849,   52357, 11, }, // Europe/Simferopol
    {  400, 6383,  118774,  6, }, // Europe/Skopje
    {  400, 6599,   37418,  5, }, // Europe/Sofia
    {  400, 6523,   52374,  9, }, // Europe/Stockholm
    {  400, 8867,    3455,  6, }, // Europe/Tallinn
    {  400, 6202,    3461,  6, }, // Europe/Tirane
    {  400, 8882,   52383,  9, }, // Europe/Ulyanovsk
    {  400, 6352,  118780,  5, }, // Europe/Vaduz
    {  400, 6567,    3481,  7, }, // Europe/Vatican
    {  400, 6231,    3488,  4, }, // Europe/Vienna
    {  400, 8899,   52392,  7, }, // Europe/Vilnius
    {  400, 4727,    3499,  9, }, // Europe/Volgograd
    {  400, 3190,    3508,  7, }, // Europe/Warsaw
    {  400, 6261,    3515,  6, }, // Europe/Zagreb
    {  400, 6540,   37442,  5, }, // Europe/Zurich
    {  400, 5078,   35625, 12, }, // Indian/Antananarivo
    {  400, 6947,    3539,  5, }, // Indian/Chagos
    {  400, 6046,  118785,  7, }, // Indian/Christmas
    {  400, 6063,    3556,  5, }, // Indian/Cocos
    {  400, 5029,   35642,  6, }, // Indian/Comoro
    {  400, 6661,    3578,  8, }, // Indian/Kerguelen
    {  400, 7260,   29694,  3, }, // Indian/Mahe
    {  400, 7050,  106679, 16, }, // Indian/Maldives
    {  400, 4069,   56708,  8, }, // Indian/Mauritius
    {  400, 5098,   56716,  6, }, // Indian/Mayotte
    {  400, 7211,   37501,  7, }, // Indian/Reunion
    {  400, 4422,    3616,  4, }, // Pacific/Apia
    {  400,  932,  118792,  6, }, // Pacific/Auckland
    {  400, 3706,   52487,  9, }, // Pacific/Bougainville
    {  400, 3015,    3634,  5, }, // Pacific/Chatham
    {  400, 3157,  118798,  4, }, // Pacific/Chuuk
    {  400, 2237,  118802,  5, }, // Pacific/Easter
    {  400, 7360,   29760,  4, }, // Pacific/Efate
    {  400, 7327,    3659,  7, }, // Pacific/Fakaofo
    {  400, 3902,  118807,  4, }, // Pacific/Fiji
    {  400, 7343,    3670,  8, }, // Pacific/Funafuti
    {  400, 6678,    3678,  9, }, // Pacific/Galapagos
    {  400, 6696,   37544,  6, }, // Pacific/Gambier
    {  400, 3822,    3713, 11, }, // Pacific/Guadalcanal
    {  400, 6018,    3724,  4, }, // Pacific/Guam
    {  400, 2723,   52582,  8, }, // Pacific/Honolulu
    {  400, 3049,   52590,  9, }, // Pacific/Kanton
    {  400, 3998,    3734, 10, }, // Pacific/Kiritimati
    {  400, 7019,  118811,  6, }, // Pacific/Kosrae
    {  400, 2842,  118817,  9, }, // Pacific/Kwajalein
    {  400, 4877,  118826,  6, }, // Pacific/Majuro
    {  400, 4051,  118832,  8, }, // Pacific/Marquesas
    {  400, 8914,   52622,  6, }, // Pacific/Midway
    {  400, 7084,    3798,  5, }, // Pacific/Nauru
    {  400, 7132,   52636,  4, }, // Pacific/Niue
    {  400, 4168,    3807,  7, }, // Pacific/Norfolk
    {  400, 7098,    3814,  5, }, // Pacific/Noumea
    {  400, 3126,   35774,  9, }, // Pacific/Pago_Pago
    {  400, 7155,    3828,  5, }, // Pacific/Palau
    {  400, 7194,   56789,  7, }, // Pacific/Pitcairn
    {  400, 3096,   37603,  6, }, // Pacific/Pohnpei
    {  400, 4807,  118840, 12, }, // Pacific/Port_Moresby
    {  400, 6076,  118852,  9, }, // Pacific/Rarotonga
    {  400, 6031,   29935,  6, }, // Pacific/Saipan
    {  400, 7312,    3872,  5, }, // Pacific/Tahiti
    {  400, 6712,    3877,  6, }, // Pacific/Tarawa
    {  400, 4583,    3883,  9, }, // Pacific/Tongatapu
    {  400, 7392,   37626,  4, }, // Pacific/Wake
    {  400, 7405,   37630,  5, }, // Pacific/Wallis
    {  401,   14,       0,  6, }, // Africa/Asmara Lakota/Latin/United States
    {  401, 4435,       6,  8, }, // Africa/Sao_Tome
    {  401, 4252,      14,  8, }, // America/Asuncion
    {  401,  237,      22,  8, }, // America/Atikokan
    {  401, 7623,      30, 17, }, // America/Bahia_Banderas
    {  401, 3868,      47,  6, }, // America/Cancun
    {  401, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  401, 5723,      66,  7, }, // America/Curacao
    {  401,  481,      73, 13, }, // America/Indiana/Knox
    {  401, 7961,      86, 16, }, // America/Indiana/Marengo
    {  401, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  401, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  401, 8038,     139, 14, }, // America/Indiana/Vevay
    {  401, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  401, 8086,     171, 16, }, // America/Indiana/Winamac
    {  401, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  401, 5932,     207, 22, }, // America/Lower_Princes
    {  401, 8220,     229,  6, }, // America/Merida
    {  401, 1850,     235, 19, }, // America/Noronha
    {  401, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  401, 8329,     274, 20, }, // America/North_Dakota/Center
    {  401, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  401,  393,     317,  4, }, // America/Nuuk
    {  401, 6899,     321, 16, }, // America/Scoresbysund
    {  401, 8508,     337, 14, }, // America/St_Barthelemy
    {  401, 2061,     351, 10, }, // America/St_Johns
    {  401, 5863,     361,  9, }, // America/St_Kitts
    {  401, 5880,     370,  9, }, // America/St_Lucia
    {  401,  855,     379, 10, }, // America/St_Thomas
    {  401, 5913,     389, 11, }, // America/St_Vincent
    {  401, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  401, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  401, 1185,     448,  9, }, // Asia/Kathmandu
    {  401,  992,     457,  7, }, // Asia/Kolkata
    {  401, 1211,     464,  5, }, // Asia/Macau
    {  401, 1235,     477,  6, }, // Asia/Yangon
    {  401, 1395,     483,  5, }, // Atlantic/Faroe
    {  401, 6821,     488, 10, }, // Atlantic/St_Helena
    {  401, 2527,     514,  4, }, // Europe/Kyiv
    {  401, 7211,     547,  7, }, // Indian/Reunion
    {  401, 3157,     554,  5, }, // Pacific/Chuuk
    {  401, 3096,     586,  7, }, // Pacific/Pohnpei
    {  402,   14,       0,  6, }, // Africa/Asmara Langi/Latin/Tanzania
    {  402, 4435,       6,  8, }, // Africa/Sao_Tome
    {  402, 4252,      14,  8, }, // America/Asuncion
    {  402,  237,      22,  8, }, // America/Atikokan
    {  402, 7623,      30, 17, }, // America/Bahia_Banderas
    {  402, 3868,      47,  6, }, // America/Cancun
    {  402, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  402, 5723,      66,  7, }, // America/Curacao
    {  402,  481,      73, 13, }, // America/Indiana/Knox
    {  402, 7961,      86, 16, }, // America/Indiana/Marengo
    {  402, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  402, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  402, 8038,     139, 14, }, // America/Indiana/Vevay
    {  402, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  402, 8086,     171, 16, }, // America/Indiana/Winamac
    {  402, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  402, 5932,     207, 22, }, // America/Lower_Princes
    {  402, 8220,     229,  6, }, // America/Merida
    {  402, 1850,     235, 19, }, // America/Noronha
    {  402, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  402, 8329,     274, 20, }, // America/North_Dakota/Center
    {  402, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  402,  393,     317,  4, }, // America/Nuuk
    {  402, 6899,     321, 16, }, // America/Scoresbysund
    {  402, 8508,     337, 14, }, // America/St_Barthelemy
    {  402, 2061,     351, 10, }, // America/St_Johns
    {  402, 5863,     361,  9, }, // America/St_Kitts
    {  402, 5880,     370,  9, }, // America/St_Lucia
    {  402,  855,     379, 10, }, // America/St_Thomas
    {  402, 5913,     389, 11, }, // America/St_Vincent
    {  402, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  402, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  402, 1185,     448,  9, }, // Asia/Kathmandu
    {  402,  992,     457,  7, }, // Asia/Kolkata
    {  402, 1211,     464,  5, }, // Asia/Macau
    {  402, 1235,     477,  6, }, // Asia/Yangon
    {  402, 1395,     483,  5, }, // Atlantic/Faroe
    {  402, 6821,     488, 10, }, // Atlantic/St_Helena
    {  402, 2527,     514,  4, }, // Europe/Kyiv
    {  402, 7211,     547,  7, }, // Indian/Reunion
    {  402, 3157,     554,  5, }, // Pacific/Chuuk
    {  402, 3096,     586,  7, }, // Pacific/Pohnpei
    {  403, 6788,  118861,  8, }, // Africa/Abidjan Lao/Lao/Laos
    {  403, 6760,  118869,  6, }, // Africa/Accra
    {  403, 5059,  118875, 13, }, // Africa/Addis_Ababa
    {  403, 7420,  118888,  6, }, // Africa/Algiers
    {  403,   14,  118894,  7, }, // Africa/Asmara
    {  403,   44,  118901,  6, }, // Africa/Bamako
    {  403, 5257,  118907,  6, }, // Africa/Bangui
    {  403, 6746,  118913,  5, }, // Africa/Banjul
    {  403, 7435,  118918,  7, }, // Africa/Bissau
    {  403, 4957,  118925,  8, }, // Africa/Blantyre
    {  403, 5287,  118933,  8, }, // Africa/Brazzaville
    {  403, 4922,  118941,  9, }, // Africa/Bujumbura
    {  403, 2320,  118950,  4, }, // Africa/Cairo
    {  403, 4117,  118954, 10, }, // Africa/Casablanca
    {  403, 7449,  118964,  5, }, // Africa/Ceuta
    {  403, 6773,  118969,  7, }, // Africa/Conakry
    {  403, 6840,  118976,  4, }, // Africa/Dakar
    {  403, 5130,  118980, 10, }, // Africa/Dar_es_Salaam
    {  403, 5043,  118990,  6, }, // Africa/Djibouti
    {  403, 5243,  118996,  6, }, // Africa/Douala
    {  403, 5166,  119002,  9, }, // Africa/El_Aaiun
    {  403, 6853,  119011,  6, }, // Africa/Freetown
    {  403, 4906,  119017,  7, }, // Africa/Gaborone
    {  403, 5015,  119024,  6, }, // Africa/Harare
    {  403, 4479,  119030, 13, }, // Africa/Johannesburg
    {  403, 4499,  119043,  4, }, // Africa/Juba
    {  403, 5151,  119047,  6, }, // Africa/Kampala
    {  403, 4524,  119053,  5, }, // Africa/Khartoum
    {  403, 4973,  119058,  6, }, // Africa/Kigali
    {  403, 5306,  119064,  7, }, // Africa/Kinshasa
    {  403, 4744,  119071,  5, }, // Africa/Lagos
    {  403, 5336,  119076,  9, }, // Africa/Libreville
    {  403, 6869,  119085,  4, }, // Africa/Lome
    {  403, 5211,  119089,  5, }, // Africa/Luanda
    {  403, 4939,  119094,  8, }, // Africa/Lubumbashi
    {  403, 5001,  119102,  6, }, // Africa/Lusaka
    {  403, 5322,  119108,  6, }, // Africa/Malabo
    {  403, 4987,  119114,  6, }, // Africa/Maputo
    {  403, 5197,  119120,  6, }, // Africa/Maseru
    {  403, 5182,  119126,  8, }, // Africa/Mbabane
    {  403, 5113,  119134,  8, }, // Africa/Mogadishu
    {  403, 7034,  119142,  8, }, // Africa/Monrovia
    {  403, 3853,  119150,  6, }, // Africa/Nairobi
    {  403, 5271,  119156, 10, }, // Africa/Ndjamena
    {  403, 5354,  119166,  6, }, // Africa/Niamey
    {  403, 6803,  119172,  8, }, // Africa/Nouakchott
    {  403, 6727,  119180,  8, }, // Africa/Ouagadougou
    {  403, 5225,  119188,  9, }, // Africa/Porto-Novo
    {  403, 4435,  119197,  7, }, // Africa/Sao_Tome
    {  403, 2866,  119204,  7, }, // Africa/Tripoli
    {  403, 6554,  119211,  5, }, // Africa/Tunis
    {  403, 4152,  119216,  6, }, // Africa/Windhoek
    {  403,  132,  119222,  5, }, // America/Adak
    {  403, 3346,  119227,  9, }, // America/Anchorage
    {  403, 5607,  119236,  8, }, // America/Anguilla
    {  403, 5624,  119244,  8, }, // America/Antigua
    {  403, 4554,  119252,  9, }, // America/Araguaina
    {  403,  166,  119261, 12, }, // America/Argentina/Buenos_Aires
    {  403,   91,  119273,  8, }, // America/Argentina/Catamarca
    {  403,  270,  119281,  6, }, // America/Argentina/Cordoba
    {  403,  441,  119287,  5, }, // America/Argentina/Jujuy
    {  403, 7462,  119292,  9, }, // America/Argentina/La_Rioja
    {  403,  565,  119301,  9, }, // America/Argentina/Mendoza
    {  403, 7489,  119310, 13, }, // America/Argentina/Rio_Gallegos
    {  403, 7520,  119323,  5, }, // America/Argentina/Salta
    {  403, 7544,  119328,  6, }, // America/Argentina/San_Juan
    {  403, 5580,  119334,  8, }, // America/Argentina/San_Luis
    {  403, 7571,  119342,  7, }, // America/Argentina/Tucuman
    {  403, 7597,  119349,  7, }, // America/Argentina/Ushuaia
    {  403, 5640,  119356,  6, }, // America/Aruba
    {  403, 4252,  119362, 10, }, // America/Asuncion
    {  403,  237,  119372,  9, }, // America/Atikokan
    {  403, 3679,  119381,  5, }, // America/Bahia
    {  403, 7623,  119386, 15, }, // America/Bahia_Banderas
    {  403, 5654,  119401,  7, }, // America/Barbados
    {  403, 7646,  119408,  5, }, // America/Belem
    {  403, 5407,  119413,  5, }, // America/Belize
    {  403, 7660,  119418, 11, }, // America/Blanc-Sablon
    {  403, 7681,  119429,  9, }, // America/Boa_Vista
    {  403, 4361,  119438,  6, }, // America/Bogota
    {  403, 7699,  119444,  5, }, // America/Boise
    {  403, 7713,  119449, 10, }, // America/Cambridge_Bay
    {  403, 7735,  119459, 10, }, // America/Campo_Grande
    {  403, 3868,  119469,  6, }, // America/Cancun
    {  403, 4694,  119475,  7, }, // America/Caracas
    {  403, 4345,  119482,  5, }, // America/Cayenne
    {  403, 5496,  119487,  5, }, // America/Cayman
    {  403, 2260,  119492,  6, }, // America/Chicago
    {  403, 7756,  119498,  6, }, // America/Chihuahua
    {  403, 7774,  119504, 14, }, // America/Ciudad_Juarez
    {  403, 5422,  119518,  9, }, // America/Costa_Rica
    {  403, 7796,  119527,  7, }, // America/Creston
    {  403, 3791,  119534,  6, }, // America/Cuiaba
    {  403, 5723,  119540,  8, }, // America/Curacao
    {  403, 7812,  119548,  9, }, // America/Danmarkshavn
    {  403, 7833,  119557,  5, }, // America/Dawson
    {  403, 7848,  119562, 10, }, // America/Dawson_Creek
    {  403,  805,  119572,  7, }, // America/Denver
    {  403, 3465,  119579,  6, }, // America/Detroit
    {  403, 5739,  119585,  8, }, // America/Dominica
    {  403,  893,  119593,  9, }, // America/Edmonton
    {  403, 7869,  119602,  8, }, // America/Eirunepe
    {  403, 5441,  119610, 10, }, // America/El_Salvador
    {  403, 7886,  119620, 10, }, // America/Fort_Nelson
    {  403, 7906,  119630,  8, }, // America/Fortaleza
    {  403, 7924,  119638,  6, }, // America/Glace_Bay
    {  403, 6881,  119644,  5, }, // America/Goose_Bay
    {  403, 4612,  119649,  9, }, // America/Grand_Turk
    {  403, 5770,  119658,  7, }, // America/Grenada
    {  403, 5786,  119665,  8, }, // America/Guadeloupe
    {  403, 3760,  119673,  9, }, // America/Guatemala
    {  403, 4841,  119682,  8, }, // America/Guayaquil
    {  403, 6932,  119690,  6, }, // America/Guyana
    {  403, 1939,  119696,  8, }, // America/Halifax
    {  403, 2281,  119704,  6, }, // America/Havana
    {  403, 7942,  119710,  8, }, // America/Hermosillo
    {  403,  348,  119718, 14, }, // America/Indiana/Indianapolis
    {  403,  481,  119732, 14, }, // America/Indiana/Knox
    {  403, 7961,  119746, 17, }, // America/Indiana/Marengo
    {  403, 7985,  119763, 20, }, // America/Indiana/Petersburg
    {  403, 8012,  119783, 18, }, // America/Indiana/Tell_City
    {  403, 8038,  119801, 14, }, // America/Indiana/Vevay
    {  403, 8060,  119815, 19, }, // America/Indiana/Vincennes
    {  403, 8086,  119834, 18, }, // America/Indiana/Winamac
    {  403, 8110,  119852,  7, }, // America/Inuvik
    {  403,  660,  119859,  8, }, // America/Iqaluit
    {  403, 2799,  119867,  6, }, // America/Jamaica
    {  403, 5380,  119873,  4, }, // America/Juneau
    {  403,  521,  119877,  7, }, // America/Kentucky/Louisville
    {  403, 8125,  119884, 20, }, // America/Kentucky/Monticello
    {  403, 5704,  119904,  9, }, // America/Kralendijk
    {  403, 4376,  119913,  5, }, // America/La_Paz
    {  403, 7169,  119918,  4, }, // America/Lima
    {  403, 3239,  119922, 11, }, // America/Los_Angeles
    {  403, 5932,  119933, 19, }, // America/Lower_Princes
    {  403, 8153,  119952,  6, }, // America/Maceio
    {  403, 8168,  119958,  7, }, // America/Managua
    {  403, 1908,  119965,  8, }, // America/Manaus
    {  403, 5897,  119973,  7, }, // America/Marigot
    {  403, 5805,  119980,  7, }, // America/Martinique
    {  403, 8184,  119987,  9, }, // America/Matamoros
    {  403, 2917,  119996,  8, }, // America/Mazatlan
    {  403, 8202,  120004,  8, }, // America/Menominee
    {  403, 8220,  120012,  7, }, // America/Merida
    {  403, 8235,  120019, 11, }, // America/Metlakatla
    {  403, 2949,  120030, 13, }, // America/Mexico_City
    {  403, 4391,  120043,  8, }, // America/Miquelon
    {  403, 8254,  120051,  6, }, // America/Moncton
    {  403, 8270,  120057,  9, }, // America/Monterrey
    {  403, 4098,  120066, 11, }, // America/Montevideo
    {  403, 5824,  120077,  9, }, // America/Montserrat
    {  403, 5481,  120086,  5, }, // America/Nassau
    {  403, 2379,  120091,  6, }, // America/New_York
    {  403, 8288,  120097,  3, }, // America/Nome
    {  403, 1850,  120100,  7, }, // America/Noronha
    {  403, 8301,  120107, 17, }, // America/North_Dakota/Beulah
    {  403, 8329,  120124, 19, }, // America/North_Dakota/Center
    {  403, 8357,  120143, 21, }, // America/North_Dakota/New_Salem
    {  403,  393,  120164,  3, }, // America/Nuuk
    {  403, 8388,  120167,  8, }, // America/Ojinaga
    {  403, 2356,  120175,  6, }, // America/Panama
    {  403, 6173,  120181, 10, }, // America/Paramaribo
    {  403, 2973,  120191,  5, }, // America/Phoenix
    {  403, 3945,  120196,  8, }, // America/Port-au-Prince
    {  403, 5954,  120204, 11, }, // America/Port_of_Spain
    {  403, 8404,  120215, 10, }, // America/Porto_Velho
    {  403, 5843,  120225,  9, }, // America/Puerto_Rico
    {  403, 4030,  120234, 14, }, // America/Punta_Arenas
    {  403, 8424,  120248, 14, }, // America/Rankin_Inlet
    {  403, 8445,  120262,  6, }, // America/Recife
    {  403, 1995,  120268,  6, }, // America/Regina
    {  403, 8460,  120274,  7, }, // America/Resolute
    {  403,  695,  120281, 11, }, // America/Rio_Branco
    {  403, 8477,  120292,  8, }, // America/Santarem
    {  403, 2201,  120300,  9, }, // America/Santiago
    {  403, 6111,  120309, 12, }, // America/Santo_Domingo
    {  403, 1878,  120321, 11, }, // America/Sao_Paulo
    {  403, 6899,  120332, 12, }, // America/Scoresbysund
    {  403, 8494,  120344,  5, }, // America/Sitka
    {  403, 8508,  120349, 11, }, // America/St_Barthelemy
    {  403, 2061,  120360,  7, }, // America/St_Johns
    {  403, 5863,  120367,  9, }, // America/St_Kitts
    {  403, 5880,  120376,  9, }, // America/St_Lucia
    {  403,  855,  120385,  9, }, // America/St_Thomas
    {  403, 5913,  120394, 10, }, // America/St_Vincent
    {  403, 8530,  120404, 12, }, // America/Swift_Current
    {  403, 5461,  120416, 11, }, // America/Tegucigalpa
    {  403, 5756,  120427,  4, }, // America/Thule
    {  403,  313,  120431,  7, }, // America/Tijuana
    {  403,  608,  120438,  7, }, // America/Toronto
    {  403, 5688,  120445,  6, }, // America/Tortola
    {  403, 2093,  120451,  8, }, // America/Vancouver
    {  403, 2144,  120459,  7, }, // America/Whitehorse
    {  403,  734,  120466,  8, }, // America/Winnipeg
    {  403, 8552,  120474,  7, }, // America/Yakutat
    {  403, 6001,  120481,  4, }, // Antarctica/Casey
    {  403, 6094,  120485,  5, }, // Antarctica/Davis
    {  403, 6133,  120490, 14, }, // Antarctica/DumontDUrville
    {  403, 8568,  120504,  9, }, // Antarctica/Macquarie
    {  403, 7066,  120513,  7, }, // Antarctica/Mawson
    {  403, 7113,  120520,  9, }, // Antarctica/McMurdo
    {  403, 8589,  120529,  6, }, // Antarctica/Palmer
    {  403, 7226,  120535,  6, }, // Antarctica/Rothera
    {  403, 7295,  120541,  5, }, // Antarctica/Syowa
    {  403, 8607,  120546,  5, }, // Antarctica/Troll
    {  403, 7374,  120551,  7, }, // Antarctica/Vostok
    {  403, 1429,  120558,  9, }, // Arctic/Longyearbyen
    {  403, 5570,  120567,  6, }, // Asia/Aden
    {  403, 5395,  120573,  6, }, // Asia/Almaty
    {  403, 3968,  120579,  5, }, // Asia/Amman
    {  403, 5511,  120584,  6, }, // Asia/Anadyr
    {  403, 5523,  120590,  7, }, // Asia/Aqtau
    {  403, 5368,  120597,  7, }, // Asia/Aqtobe
    {  403,  964,  120604, 10, }, // Asia/Ashgabat
    {  403, 8624,  120614,  9, }, // Asia/Atyrau
    {  403, 3607,  120623,  6, }, // Asia/Baghdad
    {  403, 5534,  120629,  5, }, // Asia/Bahrain
    {  403, 3653,  120634,  4, }, // Asia/Baku
    {  403, 4466,  120638,  6, }, // Asia/Bangkok
    {  403, 3571,  120644,  5, }, // Asia/Barnaul
    {  403, 4086,  120649,  5, }, // Asia/Beirut
    {  403, 3778,  120654,  8, }, // Asia/Bishkek
    {  403, 5989,  120662,  5, }, // Asia/Brunei
    {  403, 4601,  120667,  5, }, // Asia/Chita
    {  403, 4511,  120672,  6, }, // Asia/Colombo
    {  403, 4540,  120678,  8, }, // Asia/Damascus
    {  403, 1093,  120686,  6, }, // Asia/Dhaka
    {  403, 6192,  120692,  4, }, // Asia/Dili
    {  403, 3596,  120696,  4, }, // Asia/Dubai
    {  403, 6159,  120700,  7, }, // Asia/Dushanbe
    {  403, 8636,  120707,  9, }, // Asia/Famagusta
    {  403, 8651,  120716,  4, }, // Asia/Gaza
    {  403, 4795,  120720,  6, }, // Asia/Hebron
    {  403, 1259,  120726,  7, }, // Asia/Ho_Chi_Minh
    {  403, 2704,  120733,  7, }, // Asia/Hong_Kong
    {  403, 4771,  120740,  3, }, // Asia/Hovd
    {  403, 4184,  120743,  7, }, // Asia/Irkutsk
    {  403, 7006,  120750,  6, }, // Asia/Jakarta
    {  403, 6992,  120756,  8, }, // Asia/Jayapura
    {  403, 1290,  120764,  9, }, // Asia/Jerusalem
    {  403, 3560,  120773,  4, }, // Asia/Kabul
    {  403, 4316,  120777,  8, }, // Asia/Kamchatka
    {  403, 4239,  120785,  6, }, // Asia/Karachi
    {  403, 1185,  120791,  8, }, // Asia/Kathmandu
    {  403, 8661,  120799,  7, }, // Asia/Khandyga
    {  403,  992,  120806,  7, }, // Asia/Kolkata
    {  403, 4197,  120813, 11, }, // Asia/Krasnoyarsk
    {  403, 4859,  120824, 10, }, // Asia/Kuala_Lumpur
    {  403, 5976,  120834,  5, }, // Asia/Kuching
    {  403, 5547,  120839,  5, }, // Asia/Kuwait
    {  403, 1211,  120844,  6, }, // Asia/Macau
    {  403, 4017,  120850,  7, }, // Asia/Magadan
    {  403, 1349,  120857,  7, }, // Asia/Makassar
    {  403, 7182,  120864,  6, }, // Asia/Manila
    {  403, 6920,  120870,  6, }, // Asia/Muscat
    {  403, 2554,  120876,  7, }, // Asia/Nicosia
    {  403, 8675,  120883, 11, }, // Asia/Novokuznetsk
    {  403, 4135,  120894, 11, }, // Asia/Novosibirsk
    {  403, 4229,  120905,  6, }, // Asia/Omsk
    {  403, 7145,  120911,  4, }, // Asia/Oral
    {  403, 6961,  120915,  9, }, // Asia/Phnom_Penh
    {  403, 8693,  120924,  9, }, // Asia/Pontianak
    {  403, 4214,  120933,  6, }, // Asia/Pyongyang
    {  403, 5559,  120939,  6, }, // Asia/Qatar
    {  403, 8708,  120945,  7, }, // Asia/Qostanay
    {  403, 4269,  120952,  8, }, // Asia/Qyzylorda
    {  403, 3584,  120960,  5, }, // Asia/Riyadh
    {  403, 4408,  120965,  7, }, // Asia/Sakhalin
    {  403, 7245,  120972,  7, }, // Asia/Samarkand
    {  403, 3279,  120979,  4, }, // Asia/Seoul
    {  403, 1053,  120983,  7, }, // Asia/Shanghai
    {  403, 3300,  120990,  7, }, // Asia/Singapore
    {  403, 4297,  120997, 13, }, // Asia/Srednekolymsk
    {  403, 3263,  121010,  4, }, // Asia/Taipei
    {  403, 4781,  121014,  8, }, // Asia/Tashkent
    {  403, 3915,  121022,  8, }, // Asia/Tbilisi
    {  403, 2772,  121030,  5, }, // Asia/Tehran
    {  403, 1317,  121035,  5, }, // Asia/Thimphu
    {  403, 2821,  121040,  5, }, // Asia/Tokyo
    {  403, 4572,  121045,  6, }, // Asia/Tomsk
    {  403, 1021,  121051, 11, }, // Asia/Ulaanbaatar
    {  403, 1159,  121062,  7, }, // Asia/Urumqi
    {  403, 8722,  121069,  8, }, // Asia/Ust-Nera
    {  403, 6977,  121077,  6, }, // Asia/Vientiane
    {  403, 4710,  121083, 12, }, // Asia/Vladivostok
    {  403, 4828,  121095,  7, }, // Asia/Yakutsk
    {  403, 1235,  121102,  6, }, // Asia/Yangon
    {  403, 3883,  121108, 14, }, // Asia/Yekaterinburg
    {  403, 3747,  121122,  7, }, // Asia/Yerevan
    {  403, 3663,  121129, 10, }, // Atlantic/Azores
    {  403, 5671,  121139,  8, }, // Atlantic/Bermuda
    {  403, 6628,  121147,  6, }, // Atlantic/Canary
    {  403, 3727,  121153,  7, }, // Atlantic/Cape_Verde
    {  403, 1395,  121160,  4, }, // Atlantic/Faroe
    {  403, 8736,  121164,  6, }, // Atlantic/Madeira
    {  403, 2748,  121170,  8, }, // Atlantic/Reykjavik
    {  403, 7272,  121178,  9, }, // Atlantic/South_Georgia
    {  403, 6821,  121187, 10, }, // Atlantic/St_Helena
    {  403, 6644,  121197,  7, }, // Atlantic/Stanley
    {  403, 1670,  121204,  7, }, // Australia/Adelaide
    {  403, 1635,  121211,  7, }, // Australia/Brisbane
    {  403, 1799,  121218, 11, }, // Australia/Broken_Hill
    {  403, 1583,  121229,  5, }, // Australia/Darwin
    {  403, 3637,  121234,  5, }, // Australia/Eucla
    {  403, 1516,  121239,  5, }, // Australia/Hobart
    {  403, 8753,  121244,  8, }, // Australia/Lindeman
    {  403, 1547,  121252,  7, }, // Australia/Lord_Howe
    {  403, 1727,  121259,  7, }, // Australia/Melbourne
    {  403, 1762,  121266,  4, }, // Australia/Perth
    {  403, 1463,  121270,  5, }, // Australia/Sydney
    {  403, 6441,  121275,  9, }, // Europe/Amsterdam
    {  403, 6216,  121284,  7, }, // Europe/Andorra
    {  403, 3620,  121291,  9, }, // Europe/Astrakhan
    {  403, 2300,  121300,  7, }, // Europe/Athens
    {  403, 6336,  121307,  7, }, // Europe/Belgrade
    {  403, 4757,  121314,  6, }, // Europe/Berlin
    {  403, 6488,  121320, 10, }, // Europe/Bratislava
    {  403, 2167,  121330,  7, }, // Europe/Brussels
    {  403, 3928,  121337,  9, }, // Europe/Bucharest
    {  403, 3806,  121346,  9, }, // Europe/Budapest
    {  403, 8772,  121355,  9, }, // Europe/Busingen
    {  403, 2583,  121364,  8, }, // Europe/Chisinau
    {  403, 6289,  121372, 10, }, // Europe/Copenhagen
    {  403, 2338,  121382,  7, }, // Europe/Dublin
    {  403, 6307,  121389,  8, }, // Europe/Gibraltar
    {  403, 8788,  121397,  6, }, // Europe/Guernsey
    {  403, 6612,  121403,  8, }, // Europe/Helsinki
    {  403, 8804,  121411,  7, }, // Europe/Isle_of_Man
    {  403, 1130,  121418, 10, }, // Europe/Istanbul
    {  403, 8823,  121428,  5, }, // Europe/Jersey
    {  403, 3979,  121433, 11, }, // Europe/Kaliningrad
    {  403, 8929,  121444,  5, }, // Europe/Kirov
    {  403, 2527,  121449,  3, }, // Europe/Kyiv
    {  403, 3213,  121452,  6, }, // Europe/Lisbon
    {  403, 6506,  121458,  9, }, // Europe/Ljubljana
    {  403, 2501,  121467,  6, }, // Europe/London
    {  403, 6365,  121473, 11, }, // Europe/Luxembourg
    {  403, 4892,  121484,  6, }, // Europe/Madrid
    {  403, 6397,  121490,  5, }, // Europe/Malta
    {  403, 6582,  121495,  9, }, // Europe/Mariehamn
    {  403, 3693,  121504,  6, }, // Europe/Minsk
    {  403, 6410,  121510,  6, }, // Europe/Monaco
    {  403, 3537,  121516,  5, }, // Europe/Moscow
    {  403, 6458,  121521,  5, }, // Europe/Oslo
    {  403, 4284,  121526,  5, }, // Europe/Paris
    {  403, 6424,  121531,  9, }, // Europe/Podgorica
    {  403, 6275,  121540,  4, }, // Europe/Prague
    {  403, 8837,  119523,  4, }, // Europe/Riga
    {  403, 6324,  121162,  3, }, // Europe/Rome
    {  403, 4331,  121544,  6, }, // Europe/Samara
    {  403, 6470,  121550,  9, }, // Europe/San_Marino
    {  403, 6245,  121559,  8, }, // Europe/Sarajevo
    {  403, 4451,  121567,  7, }, // Europe/Saratov
    {  403, 8849,  121574,  9, }, // Europe/Simferopol
    {  403, 6383,  121583,  7, }, // Europe/Skopje
    {  403, 6599,  121590,  5, }, // Europe/Sofia
    {  403, 6523,  121595,  8, }, // Europe/Stockholm
    {  403, 8867,  121603,  8, }, // Europe/Tallinn
    {  403, 6202,  121611,  5, }, // Europe/Tirane
    {  403, 8882,  121616, 12, }, // Europe/Ulyanovsk
    {  403, 6352,  121628,  5, }, // Europe/Vaduz
    {  403, 6567,  121633,  7, }, // Europe/Vatican
    {  403, 6231,  121640,  5, }, // Europe/Vienna
    {  403, 8899,  121645,  8, }, // Europe/Vilnius
    {  403, 4727,  121653,  9, }, // Europe/Volgograd
    {  403, 3190,  121662,  4, }, // Europe/Warsaw
    {  403, 6261,  121666,  6, }, // Europe/Zagreb
    {  403, 6540,  121672,  5, }, // Europe/Zurich
    {  403, 5078,  121677, 13, }, // Indian/Antananarivo
    {  403, 6947,  121690,  5, }, // Indian/Chagos
    {  403, 6046,  121695,  9, }, // Indian/Christmas
    {  403, 6063,  121704,  5, }, // Indian/Cocos
    {  403, 5029,  121709,  6, }, // Indian/Comoro
    {  403, 6661,  121715,  8, }, // Indian/Kerguelen
    {  403, 7260,  121723,  4, }, // Indian/Mahe
    {  403, 7050,  121727,  6, }, // Indian/Maldives
    {  403, 4069,  121733,  9, }, // Indian/Mauritius
    {  403, 5098,  121742,  8, }, // Indian/Mayotte
    {  403, 7211,  121750,  9, }, // Indian/Reunion
    {  403, 4422,  121759,  6, }, // Pacific/Apia
    {  403,  932,  121765,  6, }, // Pacific/Auckland
    {  403, 3706,  121771, 19, }, // Pacific/Bougainville
    {  403, 3015,  121790,  5, }, // Pacific/Chatham
    {  403, 3157,  121795,  3, }, // Pacific/Chuuk
    {  403, 2237,  121798,  6, }, // Pacific/Easter
    {  403, 7360,  121804,  6, }, // Pacific/Efate
    {  403, 7327,  121810,  7, }, // Pacific/Fakaofo
    {  403, 3902,  121817,  4, }, // Pacific/Fiji
    {  403, 7343,  121821,  8, }, // Pacific/Funafuti
    {  403, 6678,  121829,  9, }, // Pacific/Galapagos
    {  403, 6696,  121838,  6, }, // Pacific/Gambier
    {  403, 3822,  121844, 11, }, // Pacific/Guadalcanal
    {  403, 6018,  121855,  3, }, // Pacific/Guam
    {  403, 2723,  121858,  8, }, // Pacific/Honolulu
    {  403, 3049,  121866, 12, }, // Pacific/Kanton
    {  403, 3998,  121878, 10, }, // Pacific/Kiritimati
    {  403, 7019,  121888,  5, }, // Pacific/Kosrae
    {  403, 2842,  121893,  8, }, // Pacific/Kwajalein
    {  403, 4877,  121901,  6, }, // Pacific/Majuro
    {  403, 4051,  121907,  8, }, // Pacific/Marquesas
    {  403, 8914,  121915,  5, }, // Pacific/Midway
    {  403, 7084,  121920,  6, }, // Pacific/Nauru
    {  403, 7132,  121926,  6, }, // Pacific/Niue
    {  403, 4168,  121932,  7, }, // Pacific/Norfolk
    {  403, 7098,  121939,  6, }, // Pacific/Noumea
    {  403, 3126,  121945,  8, }, // Pacific/Pago_Pago
    {  403, 7155,  121953,  6, }, // Pacific/Palau
    {  403, 7194,  121959,  6, }, // Pacific/Pitcairn
    {  403, 3096,  121965,  7, }, // Pacific/Pohnpei
    {  403, 4807,  121972, 11, }, // Pacific/Port_Moresby
    {  403, 6076,  121983,  9, }, // Pacific/Rarotonga
    {  403, 6031,  121992,  5, }, // Pacific/Saipan
    {  403, 7312,  121997,  6, }, // Pacific/Tahiti
    {  403, 6712,  122003,  6, }, // Pacific/Tarawa
    {  403, 4583,  122009,  9, }, // Pacific/Tongatapu
    {  403, 7392,  122018,  3, }, // Pacific/Wake
    {  403, 7405,  122021,  5, }, // Pacific/Wallis
    {  404,   14,       0,  6, }, // Africa/Asmara Latin/Latin/Vatican City
    {  404, 4435,       6,  8, }, // Africa/Sao_Tome
    {  404, 4252,      14,  8, }, // America/Asuncion
    {  404,  237,      22,  8, }, // America/Atikokan
    {  404, 7623,      30, 17, }, // America/Bahia_Banderas
    {  404, 3868,      47,  6, }, // America/Cancun
    {  404, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  404, 5723,      66,  7, }, // America/Curacao
    {  404,  481,      73, 13, }, // America/Indiana/Knox
    {  404, 7961,      86, 16, }, // America/Indiana/Marengo
    {  404, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  404, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  404, 8038,     139, 14, }, // America/Indiana/Vevay
    {  404, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  404, 8086,     171, 16, }, // America/Indiana/Winamac
    {  404, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  404, 5932,     207, 22, }, // America/Lower_Princes
    {  404, 8220,     229,  6, }, // America/Merida
    {  404, 1850,     235, 19, }, // America/Noronha
    {  404, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  404, 8329,     274, 20, }, // America/North_Dakota/Center
    {  404, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  404,  393,     317,  4, }, // America/Nuuk
    {  404, 6899,     321, 16, }, // America/Scoresbysund
    {  404, 8508,     337, 14, }, // America/St_Barthelemy
    {  404, 2061,     351, 10, }, // America/St_Johns
    {  404, 5863,     361,  9, }, // America/St_Kitts
    {  404, 5880,     370,  9, }, // America/St_Lucia
    {  404,  855,     379, 10, }, // America/St_Thomas
    {  404, 5913,     389, 11, }, // America/St_Vincent
    {  404, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  404, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  404, 1185,     448,  9, }, // Asia/Kathmandu
    {  404,  992,     457,  7, }, // Asia/Kolkata
    {  404, 1211,     464,  5, }, // Asia/Macau
    {  404, 1235,     477,  6, }, // Asia/Yangon
    {  404, 1395,     483,  5, }, // Atlantic/Faroe
    {  404, 6821,     488, 10, }, // Atlantic/St_Helena
    {  404, 2527,     514,  4, }, // Europe/Kyiv
    {  404, 7211,     547,  7, }, // Indian/Reunion
    {  404, 3157,     554,  5, }, // Pacific/Chuuk
    {  404, 3096,     586,  7, }, // Pacific/Pohnpei
    {  405, 6788,  122026,  8, }, // Africa/Abidjan Latvian/Latin/Latvia
    {  405, 6760,    4649,  4, }, // Africa/Accra
    {  405, 5059,  122034,  9, }, // Africa/Addis_Ababa
    {  405, 7420,  122043,  6, }, // Africa/Algiers
    {  405,   14,       0,  6, }, // Africa/Asmara
    {  405, 5257,   26480,  5, }, // Africa/Bangui
    {  405, 6746,  122049,  8, }, // Africa/Banjul
    {  405, 7435,  122057,  6, }, // Africa/Bissau
    {  405, 4957,  122063,  9, }, // Africa/Blantyre
    {  405, 5287,  122072,  9, }, // Africa/Brazzaville
    {  405, 4922,  122081,  9, }, // Africa/Bujumbura
    {  405, 2320,  122090,  5, }, // Africa/Cairo
    {  405, 4117,    4695, 10, }, // Africa/Casablanca
    {  405, 7449,   21349,  5, }, // Africa/Ceuta
    {  405, 6773,    6367,  7, }, // Africa/Conakry
    {  405, 6840,  122095,  6, }, // Africa/Dakar
    {  405, 5130,  122101, 10, }, // Africa/Dar_es_Salaam
    {  405, 5043,  122111,  9, }, // Africa/Djibouti
    {  405, 5243,   21372,  5, }, // Africa/Douala
    {  405, 5166,  122120,  5, }, // Africa/El_Aaiun
    {  405, 6853,  122125,  8, }, // Africa/Freetown
    {  405, 4479,  122133, 13, }, // Africa/Johannesburg
    {  405, 4499,  122146,  5, }, // Africa/Juba
    {  405, 4524,  122151,  7, }, // Africa/Khartoum
    {  405, 5306,  122158,  7, }, // Africa/Kinshasa
    {  405, 4744,  122165,  6, }, // Africa/Lagos
    {  405, 5336,  122171,  9, }, // Africa/Libreville
    {  405, 4939,  122180,  9, }, // Africa/Lubumbashi
    {  405, 4987,  122189,  6, }, // Africa/Maputo
    {  405, 5113,  122195,  8, }, // Africa/Mogadishu
    {  405, 7034,  122203,  9, }, // Africa/Monrovia
    {  405, 5271,   57729,  8, }, // Africa/Ndjamena
    {  405, 5354,  122212,  7, }, // Africa/Niamey
    {  405, 6803,  122219,  8, }, // Africa/Nouakchott
    {  405, 6727,   61364,  8, }, // Africa/Ouagadougou
    {  405, 5225,   61373,  9, }, // Africa/Porto-Novo
    {  405, 4435,   61382,  7, }, // Africa/Sao_Tome
    {  405, 2866,  122227,  7, }, // Africa/Tripoli
    {  405, 6554,  122234,  6, }, // Africa/Tunis
    {  405, 4152,  122240,  8, }, // Africa/Windhoek
    {  405,  132,  122248,  5, }, // America/Adak
    {  405, 3346,  122253,  8, }, // America/Anchorage
    {  405, 5607,  122261,  7, }, // America/Anguilla
    {  405, 5624,   33307,  7, }, // America/Antigua
    {  405, 4554,  122268,  9, }, // America/Araguaina
    {  405,  166,  122277, 12, }, // America/Argentina/Buenos_Aires
    {  405,   91,    6518,  9, }, // America/Argentina/Catamarca
    {  405,  270,  122289,  7, }, // America/Argentina/Cordoba
    {  405,  441,  122296,  6, }, // America/Argentina/Jujuy
    {  405, 7462,  122302,  7, }, // America/Argentina/La_Rioja
    {  405,  565,   97634,  7, }, // America/Argentina/Mendoza
    {  405, 7489,  122309, 12, }, // America/Argentina/Rio_Gallegos
    {  405, 7544,  122321,  8, }, // America/Argentina/San_Juan
    {  405, 5580,  122329,  8, }, // America/Argentina/San_Luis
    {  405, 7571,  122337,  8, }, // America/Argentina/Tucuman
    {  405, 7597,  122345,  6, }, // America/Argentina/Ushuaia
    {  405, 4252,  122351,  9, }, // America/Asuncion
    {  405,  237,  122360,  9, }, // America/Atikokan
    {  405, 3679,  122369,  5, }, // America/Bahia
    {  405, 7623,  122374, 13, }, // America/Bahia_Banderas
    {  405, 5654,  122387,  9, }, // America/Barbados
    {  405, 7646,  122396,  6, }, // America/Belem
    {  405, 5407,  122402,  6, }, // America/Belize
    {  405, 7660,  122408, 11, }, // America/Blanc-Sablon
    {  405, 7681,  122419,  8, }, // America/Boa_Vista
    {  405, 7699,  122427, 11, }, // America/Boise
    {  405, 7713,  122438, 12, }, // America/Cambridge_Bay
    {  405, 7735,  122450, 11, }, // America/Campo_Grande
    {  405, 3868,  122461,  7, }, // America/Cancun
    {  405, 4694,  122468,  8, }, // America/Caracas
    {  405, 4345,  122476,  7, }, // America/Cayenne
    {  405, 5496,  122483, 13, }, // America/Cayman
    {  405, 2260,  122496,  6, }, // America/Chicago
    {  405, 7756,  122502,  6, }, // America/Chihuahua
    {  405, 7774,  122508,  7, }, // America/Ciudad_Juarez
    {  405, 5422,   33320,  9, }, // America/Costa_Rica
    {  405, 7796,  122515,  8, }, // America/Creston
    {  405, 3791,    6743,  6, }, // America/Cuiaba
    {  405, 5723,  122523,  7, }, // America/Curacao
    {  405, 7812,  122530, 12, }, // America/Danmarkshavn
    {  405, 7833,  122542,  7, }, // America/Dawson
    {  405, 7848,  122549, 11, }, // America/Dawson_Creek
    {  405,  805,  122560,  7, }, // America/Denver
    {  405, 3465,  122567,  8, }, // America/Detroit
    {  405, 5739,   21666,  8, }, // America/Dominica
    {  405,  893,  122575,  9, }, // America/Edmonton
    {  405, 5441,  122584,  9, }, // America/El_Salvador
    {  405, 7886,  122593, 11, }, // America/Fort_Nelson
    {  405, 7924,  122604,  9, }, // America/Glace_Bay
    {  405, 6881,  122613,  7, }, // America/Goose_Bay
    {  405, 4612,  122620, 11, }, // America/Grand_Turk
    {  405, 5770,  122631,  7, }, // America/Grenada
    {  405, 5786,  122638,  9, }, // America/Guadeloupe
    {  405, 3760,   33338,  9, }, // America/Guatemala
    {  405, 4841,  122647,  9, }, // America/Guayaquil
    {  405, 6932,  122656,  6, }, // America/Guyana
    {  405, 1939,  122662,  9, }, // America/Halifax
    {  405, 7942,  122671,  9, }, // America/Hermosillo
    {  405,  348,  122680, 13, }, // America/Indiana/Indianapolis
    {  405,  481,  122693, 14, }, // America/Indiana/Knox
    {  405, 7961,  122707, 16, }, // America/Indiana/Marengo
    {  405, 7985,  122723, 20, }, // America/Indiana/Petersburg
    {  405, 8012,  122743, 18, }, // America/Indiana/Tell_City
    {  405, 8038,  122761, 15, }, // America/Indiana/Vevay
    {  405, 8060,  122776, 17, }, // America/Indiana/Vincennes
    {  405, 8086,  122793, 17, }, // America/Indiana/Winamac
    {  405, 8110,  122810,  7, }, // America/Inuvik
    {  405,  660,  122817,  8, }, // America/Iqaluit
    {  405, 2799,    3995,  7, }, // America/Jamaica
    {  405, 5380,  122825,  5, }, // America/Juneau
    {  405,  521,  122830,  7, }, // America/Kentucky/Louisville
    {  405, 8125,  122837, 18, }, // America/Kentucky/Monticello
    {  405, 5704,  122855, 11, }, // America/Kralendijk
    {  405, 4376,  122866,  6, }, // America/La_Paz
    {  405, 3239,  122872, 12, }, // America/Los_Angeles
    {  405, 5932,  122884, 19, }, // America/Lower_Princes
    {  405, 8153,  122903,  6, }, // America/Maceio
    {  405, 8168,  122909,  7, }, // America/Managua
    {  405, 1908,  122916,  7, }, // America/Manaus
    {  405, 5897,  122923,  8, }, // America/Marigot
    {  405, 5805,   26594,  9, }, // America/Martinique
    {  405, 8184,  122931, 10, }, // America/Matamoros
    {  405, 2917,  122941,  9, }, // America/Mazatlan
    {  405, 8202,  122950,  8, }, // America/Menominee
    {  405, 8220,    7126,  6, }, // America/Merida
    {  405, 2949,  122958,  6, }, // America/Mexico_City
    {  405, 4391,  122964,  8, }, // America/Miquelon
    {  405, 8254,  122972,  8, }, // America/Moncton
    {  405, 8270,  122980, 10, }, // America/Monterrey
    {  405, 5824,  122990, 11, }, // America/Montserrat
    {  405, 5481,  123001,  4, }, // America/Nassau
    {  405, 2379,  123005,  7, }, // America/New_York
    {  405, 8288,  123012,  4, }, // America/Nome
    {  405, 1850,  123016,  6, }, // America/Noronha
    {  405, 8301,  123022, 19, }, // America/North_Dakota/Beulah
    {  405, 8329,  123041, 21, }, // America/North_Dakota/Center
    {  405, 8357,  123062, 23, }, // America/North_Dakota/New_Salem
    {  405,  393,  123085,  4, }, // America/Nuuk
    {  405, 8388,    7276,  7, }, // America/Ojinaga
    {  405, 2973,  123089,  7, }, // America/Phoenix
    {  405, 3945,  123096, 11, }, // America/Port-au-Prince
    {  405, 5954,  123107, 12, }, // America/Port_of_Spain
    {  405, 8404,  123119,  9, }, // America/Porto_Velho
    {  405, 5843,   62053, 10, }, // America/Puerto_Rico
    {  405, 4030,  123128, 12, }, // America/Punta_Arenas
    {  405, 8424,  123140, 12, }, // America/Rankin_Inlet
    {  405, 8445,  123152,  6, }, // America/Recife
    {  405, 1995,  123158,  8, }, // America/Regina
    {  405, 8460,  123166,  8, }, // America/Resolute
    {  405,  695,  123174,  9, }, // America/Rio_Branco
    {  405, 8477,  123183,  9, }, // America/Santarem
    {  405, 2201,  123192,  8, }, // America/Santiago
    {  405, 6111,  123200, 12, }, // America/Santo_Domingo
    {  405, 1878,  123212,  8, }, // America/Sao_Paulo
    {  405, 6899,  123220, 13, }, // America/Scoresbysund
    {  405, 8508,  123233, 11, }, // America/St_Barthelemy
    {  405, 2061,  123244, 10, }, // America/St_Johns
    {  405, 5863,  123254,  9, }, // America/St_Kitts
    {  405, 5880,  123263, 10, }, // America/St_Lucia
    {  405,  855,  123273,  9, }, // America/St_Thomas
    {  405, 5913,  123282, 12, }, // America/St_Vincent
    {  405, 8530,  123294, 12, }, // America/Swift_Current
    {  405, 5461,    5528, 11, }, // America/Tegucigalpa
    {  405, 5756,  123306,  4, }, // America/Thule
    {  405,  313,    7479,  7, }, // America/Tijuana
    {  405, 2093,  123310,  9, }, // America/Vancouver
    {  405, 2144,  123319,  9, }, // America/Whitehorse
    {  405,  734,  123328,  8, }, // America/Winnipeg
    {  405, 8552,  123336,  8, }, // America/Yakutat
    {  405, 6001,  123344,  5, }, // Antarctica/Casey
    {  405, 6094,  123349,  7, }, // Antarctica/Davis
    {  405, 6133,  123356, 12, }, // Antarctica/DumontDUrville
    {  405, 8568,  123368,  7, }, // Antarctica/Macquarie
    {  405, 7066,  123375,  6, }, // Antarctica/Mawson
    {  405, 7113,  123381,  8, }, // Antarctica/McMurdo
    {  405, 8589,  123389,  7, }, // Antarctica/Palmer
    {  405, 7226,    5583,  6, }, // Antarctica/Rothera
    {  405, 7295,  123396,  4, }, // Antarctica/Syowa
    {  405, 8607,  123400,  6, }, // Antarctica/Troll
    {  405, 7374,  123406,  7, }, // Antarctica/Vostok
    {  405, 1429,  123413, 12, }, // Arctic/Longyearbyen
    {  405, 5570,  123425,  5, }, // Asia/Aden
    {  405, 5395,    7594,  6, }, // Asia/Almaty
    {  405, 3968,  123430,  6, }, // Asia/Amman
    {  405, 5511,  123436,  7, }, // Asia/Anadyr
    {  405, 5523,    5609,  5, }, // Asia/Aqtau
    {  405, 5368,  123443,  6, }, // Asia/Aqtobe
    {  405,  964,  123449,  8, }, // Asia/Ashgabat
    {  405, 8624,    7612,  6, }, // Asia/Atyrau
    {  405, 3607,  123457,  7, }, // Asia/Baghdad
    {  405, 5534,  123464,  8, }, // Asia/Bahrain
    {  405, 4466,  123472,  8, }, // Asia/Bangkok
    {  405, 3571,  123480,  8, }, // Asia/Barnaul
    {  405, 4086,  123488,  7, }, // Asia/Beirut
    {  405, 3778,  123495,  7, }, // Asia/Bishkek
    {  405, 5989,  123502,  7, }, // Asia/Brunei
    {  405, 4601,   57622,  4, }, // Asia/Chita
    {  405, 4511,    5637,  7, }, // Asia/Colombo
    {  405, 4540,  123509,  7, }, // Asia/Damascus
    {  405, 1093,    4717,  4, }, // Asia/Dhaka
    {  405, 3596,  123516,  7, }, // Asia/Dubai
    {  405, 6159,   33468,  7, }, // Asia/Dushanbe
    {  405, 4795,  123523,  7, }, // Asia/Hebron
    {  405, 1259,  123530,  8, }, // Asia/Ho_Chi_Minh
    {  405, 2704,  123538,  8, }, // Asia/Hong_Kong
    {  405, 4771,  123546,  5, }, // Asia/Hovd
    {  405, 4184,  123551,  8, }, // Asia/Irkutsk
    {  405, 7006,   33484,  8, }, // Asia/Jakarta
    {  405, 6992,   33492,  9, }, // Asia/Jayapura
    {  405, 1290,  123559, 10, }, // Asia/Jerusalem
    {  405, 3560,  123569,  6, }, // Asia/Kabul
    {  405, 4316,   33510,  8, }, // Asia/Kamchatka
    {  405, 4239,  123575,  6, }, // Asia/Karachi
    {  405, 1185,    5711,  8, }, // Asia/Kathmandu
    {  405, 8661,   33524,  7, }, // Asia/Khandyga
    {  405,  992,  123581,  7, }, // Asia/Kolkata
    {  405, 4197,  123588, 12, }, // Asia/Krasnoyarsk
    {  405, 4859,  123600, 12, }, // Asia/Kuala_Lumpur
    {  405, 5976,  123612,  6, }, // Asia/Kuching
    {  405, 5547,  123618,  7, }, // Asia/Kuwait
    {  405, 1211,    7731,  5, }, // Asia/Macau
    {  405, 4017,  123625,  8, }, // Asia/Magadan
    {  405, 1349,  123633,  8, }, // Asia/Makassar
    {  405, 6920,  123641,  7, }, // Asia/Muscat
    {  405, 2554,   57283,  8, }, // Asia/Nicosia
    {  405, 8675,  123648, 12, }, // Asia/Novokuznetsk
    {  405, 4135,  123660, 12, }, // Asia/Novosibirsk
    {  405, 4229,  123672,  5, }, // Asia/Omsk
    {  405, 7145,  123677,  5, }, // Asia/Oral
    {  405, 6961,  123682,  8, }, // Asia/Phnom_Penh
    {  405, 8693,  123690, 10, }, // Asia/Pontianak
    {  405, 4214,  123700,  8, }, // Asia/Pyongyang
    {  405, 5559,  123708,  6, }, // Asia/Qatar
    {  405, 8708,  123714,  9, }, // Asia/Qostanay
    {  405, 4269,    7778,  9, }, // Asia/Qyzylorda
    {  405, 3584,  123723,  6, }, // Asia/Riyadh
    {  405, 4408,  123729,  8, }, // Asia/Sakhalin
    {  405, 7245,   26906, 10, }, // Asia/Samarkand
    {  405, 3279,  123737,  5, }, // Asia/Seoul
    {  405, 1053,  123742,  7, }, // Asia/Shanghai
    {  405, 3300,  123749,  9, }, // Asia/Singapore
    {  405, 4297,  123758, 14, }, // Asia/Srednekolymsk
    {  405, 3263,  123772,  6, }, // Asia/Taipei
    {  405, 4781,  123778,  8, }, // Asia/Tashkent
    {  405, 2772,  123786,  8, }, // Asia/Tehran
    {  405, 2821,  123794,  6, }, // Asia/Tokyo
    {  405, 4572,  123800,  6, }, // Asia/Tomsk
    {  405, 1021,  123806, 10, }, // Asia/Ulaanbaatar
    {  405, 1159,   33596,  6, }, // Asia/Urumqi
    {  405, 8722,  123816,  8, }, // Asia/Ust-Nera
    {  405, 6977,  123824,  8, }, // Asia/Vientiane
    {  405, 4710,  123832, 12, }, // Asia/Vladivostok
    {  405, 4828,  123844,  8, }, // Asia/Yakutsk
    {  405, 1235,  123852,  7, }, // Asia/Yangon
    {  405, 3883,  123859, 14, }, // Asia/Yekaterinburg
    {  405, 3747,  123873,  7, }, // Asia/Yerevan
    {  405, 3663,  123880, 11, }, // Atlantic/Azores
    {  405, 6628,  123891, 14, }, // Atlantic/Canary
    {  405, 3727,  123905,  9, }, // Atlantic/Cape_Verde
    {  405, 1395,  123914, 10, }, // Atlantic/Faroe
    {  405, 2748,  123924, 10, }, // Atlantic/Reykjavik
    {  405, 7272,  123934, 16, }, // Atlantic/South_Georgia
    {  405, 6821,  123950, 15, }, // Atlantic/St_Helena
    {  405, 6644,    7965,  6, }, // Atlantic/Stanley
    {  405, 1670,   21030,  8, }, // Australia/Adelaide
    {  405, 1635,  123965,  8, }, // Australia/Brisbane
    {  405, 1799,  123973, 10, }, // Australia/Broken_Hill
    {  405, 1583,  123983,  7, }, // Australia/Darwin
    {  405, 3637,  123990,  5, }, // Australia/Eucla
    {  405, 1516,  123995,  7, }, // Australia/Hobart
    {  405, 8753,  124002,  9, }, // Australia/Lindeman
    {  405, 1547,  124011,  8, }, // Australia/Lord_Howe
    {  405, 1727,  124019,  8, }, // Australia/Melbourne
    {  405, 1762,  124027,  5, }, // Australia/Perth
    {  405, 1463,  124032,  7, }, // Australia/Sydney
    {  405, 6441,  124039, 10, }, // Europe/Amsterdam
    {  405, 6216,   33638,  6, }, // Europe/Andorra
    {  405, 3620,  124049,  9, }, // Europe/Astrakhan
    {  405, 2300,  124058,  6, }, // Europe/Athens
    {  405, 6336,  124064,  8, }, // Europe/Belgrade
    {  405, 4757,  124072,  7, }, // Europe/Berlin
    {  405, 2167,  124079,  7, }, // Europe/Brussels
    {  405, 3928,  124086,  9, }, // Europe/Bucharest
    {  405, 3806,  124095,  9, }, // Europe/Budapest
    {  405, 8772,  124104,  9, }, // Europe/Busingen
    {  405, 2583,  124113,  8, }, // Europe/Chisinau
    {  405, 6289,  124121, 11, }, // Europe/Copenhagen
    {  405, 2338,  124132,  7, }, // Europe/Dublin
    {  405, 6307,  124139, 10, }, // Europe/Gibraltar
    {  405, 8788,  124149,  8, }, // Europe/Guernsey
    {  405, 8804,  124157, 10, }, // Europe/Isle_of_Man
    {  405, 1130,  124167,  8, }, // Europe/Istanbul
    {  405, 8823,  124175,  8, }, // Europe/Jersey
    {  405, 3979,  124183, 12, }, // Europe/Kaliningrad
    {  405, 8929,  124195,  6, }, // Europe/Kirov
    {  405, 2527,  124201,  6, }, // Europe/Kyiv
    {  405, 3213,  124207,  8, }, // Europe/Lisbon
    {  405, 6506,  124215,  7, }, // Europe/Ljubljana
    {  405, 2501,  124222,  7, }, // Europe/London
    {  405, 6365,  124229, 11, }, // Europe/Luxembourg
    {  405, 4892,  124240,  7, }, // Europe/Madrid
    {  405, 6582,  124247, 10, }, // Europe/Mariehamn
    {  405, 3693,  124257,  6, }, // Europe/Minsk
    {  405, 6410,    6044,  6, }, // Europe/Monaco
    {  405, 3537,  124263,  7, }, // Europe/Moscow
    {  405, 4284,  124270,  6, }, // Europe/Paris
    {  405, 6275,  124276,  5, }, // Europe/Prague
    {  405, 8837,   73428,  4, }, // Europe/Riga
    {  405, 6324,    6064,  4, }, // Europe/Rome
    {  405, 6470,  124281,  9, }, // Europe/San_Marino
    {  405, 6245,  124290,  8, }, // Europe/Sarajevo
    {  405, 4451,  124298,  8, }, // Europe/Saratov
    {  405, 8849,  124306, 11, }, // Europe/Simferopol
    {  405, 6599,   33761,  6, }, // Europe/Sofia
    {  405, 6523,  124317,  9, }, // Europe/Stockholm
    {  405, 8867,  124326,  7, }, // Europe/Tallinn
    {  405, 6202,  124333,  6, }, // Europe/Tirane
    {  405, 8882,  124339, 10, }, // Europe/Ulyanovsk
    {  405, 6352,  124349,  6, }, // Europe/Vaduz
    {  405, 6567,  124355,  8, }, // Europe/Vatican
    {  405, 6231,  124363,  4, }, // Europe/Vienna
    {  405, 8899,  124367,  5, }, // Europe/Vilnius
    {  405, 4727,  124372, 10, }, // Europe/Volgograd
    {  405, 3190,   33778,  7, }, // Europe/Warsaw
    {  405, 6261,  124382,  7, }, // Europe/Zagreb
    {  405, 6540,  124389,  6, }, // Europe/Zurich
    {  405, 5078,   99361, 12, }, // Indian/Antananarivo
    {  405, 6947,  124395,  6, }, // Indian/Chagos
    {  405, 6046,  124401, 15, }, // Indian/Christmas
    {  405, 6063,  124416, 21, }, // Indian/Cocos
    {  405, 5029,  124437,  7, }, // Indian/Comoro
    {  405, 6661,  124444, 14, }, // Indian/Kerguelen
    {  405, 7260,  124458,  3, }, // Indian/Mahe
    {  405, 7050,  124461,  9, }, // Indian/Maldives
    {  405, 4069,  124470,  9, }, // Indian/Mauritius
    {  405, 5098,  124479,  6, }, // Indian/Mayotte
    {  405, 7211,  124485,  8, }, // Indian/Reunion
    {  405, 4422,  124493,  5, }, // Pacific/Apia
    {  405,  932,  124498,  7, }, // Pacific/Auckland
    {  405, 3706,  124505, 14, }, // Pacific/Bougainville
    {  405, 3015,  124519,  6, }, // Pacific/Chatham
    {  405, 3157,  124525,  4, }, // Pacific/Chuuk
    {  405, 2237,  124529, 14, }, // Pacific/Easter
    {  405, 3902,   33836,  5, }, // Pacific/Fiji
    {  405, 6678,  124543, 14, }, // Pacific/Galapagos
    {  405, 6696,  124557, 12, }, // Pacific/Gambier
    {  405, 3822,  124569, 12, }, // Pacific/Guadalcanal
    {  405, 6018,  124581,  5, }, // Pacific/Guam
    {  405, 2723,    4611,  8, }, // Pacific/Honolulu
    {  405, 3049,  124586, 11, }, // Pacific/Kanton
    {  405, 3998,  124597, 10, }, // Pacific/Kiritimati
    {  405, 7019,  124607,  7, }, // Pacific/Kosrae
    {  405, 2842,  124614, 11, }, // Pacific/Kwajalein
    {  405, 4877,  124625,  7, }, // Pacific/Majuro
    {  405, 4051,  124632, 13, }, // Pacific/Marquesas
    {  405, 8914,  124645,  7, }, // Pacific/Midway
    {  405, 4168,  124652,  8, }, // Pacific/Norfolk
    {  405, 7098,  124660,  5, }, // Pacific/Noumea
    {  405, 3126,   63411,  8, }, // Pacific/Pago_Pago
    {  405, 7194,  124665,  8, }, // Pacific/Pitcairn
    {  405, 3096,  124673,  7, }, // Pacific/Pohnpei
    {  405, 4807,  124680, 10, }, // Pacific/Port_Moresby
    {  405, 6031,  124690,  7, }, // Pacific/Saipan
    {  405, 7312,  124697,  5, }, // Pacific/Tahiti
    {  405, 6712,   23159,  6, }, // Pacific/Tarawa
    {  405, 7392,  124702, 10, }, // Pacific/Wake
    {  405, 7405,  124712,  6, }, // Pacific/Wallis
    {  406,   14,       0,  6, }, // Africa/Asmara Lingala/Latin/Congo - Kinshasa
    {  406, 4435,       6,  8, }, // Africa/Sao_Tome
    {  406, 4252,      14,  8, }, // America/Asuncion
    {  406,  237,      22,  8, }, // America/Atikokan
    {  406, 7623,      30, 17, }, // America/Bahia_Banderas
    {  406, 3868,      47,  6, }, // America/Cancun
    {  406, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  406, 5723,      66,  7, }, // America/Curacao
    {  406,  481,      73, 13, }, // America/Indiana/Knox
    {  406, 7961,      86, 16, }, // America/Indiana/Marengo
    {  406, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  406, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  406, 8038,     139, 14, }, // America/Indiana/Vevay
    {  406, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  406, 8086,     171, 16, }, // America/Indiana/Winamac
    {  406, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  406, 5932,     207, 22, }, // America/Lower_Princes
    {  406, 8220,     229,  6, }, // America/Merida
    {  406, 1850,     235, 19, }, // America/Noronha
    {  406, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  406, 8329,     274, 20, }, // America/North_Dakota/Center
    {  406, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  406,  393,     317,  4, }, // America/Nuuk
    {  406, 6899,     321, 16, }, // America/Scoresbysund
    {  406, 8508,     337, 14, }, // America/St_Barthelemy
    {  406, 2061,     351, 10, }, // America/St_Johns
    {  406, 5863,     361,  9, }, // America/St_Kitts
    {  406, 5880,     370,  9, }, // America/St_Lucia
    {  406,  855,     379, 10, }, // America/St_Thomas
    {  406, 5913,     389, 11, }, // America/St_Vincent
    {  406, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  406, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  406, 1185,     448,  9, }, // Asia/Kathmandu
    {  406,  992,     457,  7, }, // Asia/Kolkata
    {  406, 1211,     464,  5, }, // Asia/Macau
    {  406, 1235,     477,  6, }, // Asia/Yangon
    {  406, 1395,     483,  5, }, // Atlantic/Faroe
    {  406, 6821,     488, 10, }, // Atlantic/St_Helena
    {  406, 2527,     514,  4, }, // Europe/Kyiv
    {  406, 7211,     547,  7, }, // Indian/Reunion
    {  406, 3157,     554,  5, }, // Pacific/Chuuk
    {  406, 3096,     586,  7, }, // Pacific/Pohnpei
    {  410, 6788,  124718,  9, }, // Africa/Abidjan Lithuanian/Latin/Lithuania
    {  410, 6760,    4649,  4, }, // Africa/Accra
    {  410, 5059,   35821, 10, }, // Africa/Addis_Ababa
    {  410, 7420,  124727,  7, }, // Africa/Algiers
    {  410,   14,       0,  6, }, // Africa/Asmara
    {  410,   44,  124734,  7, }, // Africa/Bamako
    {  410, 5257,  124741,  6, }, // Africa/Bangui
    {  410, 6746,  124747,  9, }, // Africa/Banjul
    {  410, 7435,    6313,  5, }, // Africa/Bissau
    {  410, 4957,  124756, 10, }, // Africa/Blantyre
    {  410, 5287,  124766, 10, }, // Africa/Brazzaville
    {  410, 4922,  124776,  9, }, // Africa/Bujumbura
    {  410, 2320,  124785,  6, }, // Africa/Cairo
    {  410, 4117,    4695, 10, }, // Africa/Casablanca
    {  410, 7449,   21349,  5, }, // Africa/Ceuta
    {  410, 6773,  124791,  8, }, // Africa/Conakry
    {  410, 6840,  124799,  7, }, // Africa/Dakar
    {  410, 5130,  124806, 14, }, // Africa/Dar_es_Salaam
    {  410, 5043,  124820,  8, }, // Africa/Djibouti
    {  410, 5243,   21372,  5, }, // Africa/Douala
    {  410, 5166,  124828,  9, }, // Africa/El_Aaiun
    {  410, 6853,  124837,  9, }, // Africa/Freetown
    {  410, 4906,  124846,  9, }, // Africa/Gaborone
    {  410, 5015,  124855,  6, }, // Africa/Harare
    {  410, 4479,  124861, 13, }, // Africa/Johannesburg
    {  410, 4499,  124874,  5, }, // Africa/Juba
    {  410, 4524,  124879,  9, }, // Africa/Khartoum
    {  410, 4973,  124888,  7, }, // Africa/Kigali
    {  410, 5306,  122158,  7, }, // Africa/Kinshasa
    {  410, 4744,  124895,  7, }, // Africa/Lagos
    {  410, 5336,  124902, 10, }, // Africa/Libreville
    {  410, 6869,  124912,  4, }, // Africa/Lome
    {  410, 4939,  124916, 10, }, // Africa/Lubumbashi
    {  410, 5322,  124926,  7, }, // Africa/Malabo
    {  410, 4987,  124933,  7, }, // Africa/Maputo
    {  410, 5182,  124940,  7, }, // Africa/Mbabane
    {  410, 5113,  124947,  9, }, // Africa/Mogadishu
    {  410, 7034,  122203,  9, }, // Africa/Monrovia
    {  410, 3853,  124956,  8, }, // Africa/Nairobi
    {  410, 5271,   57729,  8, }, // Africa/Ndjamena
    {  410, 5354,  124964,  8, }, // Africa/Niamey
    {  410, 6803,  124972,  9, }, // Africa/Nouakchott
    {  410, 6727,   61364,  8, }, // Africa/Ouagadougou
    {  410, 5225,  124981, 11, }, // Africa/Porto-Novo
    {  410, 4435,  124992,  8, }, // Africa/Sao_Tome
    {  410, 2866,   57756,  8, }, // Africa/Tripoli
    {  410, 6554,  125000,  7, }, // Africa/Tunis
    {  410, 4152,  125007,  9, }, // Africa/Windhoek
    {  410,  132,  125016,  7, }, // America/Adak
    {  410, 3346,  125023, 10, }, // America/Anchorage
    {  410, 5607,  125033,  8, }, // America/Anguilla
    {  410, 5624,   33307,  7, }, // America/Antigua
    {  410, 4554,  125041, 10, }, // America/Araguaina
    {  410,  166,  125051, 12, }, // America/Argentina/Buenos_Aires
    {  410,   91,    6518,  9, }, // America/Argentina/Catamarca
    {  410,  270,    6527,  7, }, // America/Argentina/Cordoba
    {  410,  441,  125063,  9, }, // America/Argentina/Jujuy
    {  410, 7462,  125072,  9, }, // America/Argentina/La_Rioja
    {  410,  565,   97634,  7, }, // America/Argentina/Mendoza
    {  410, 7489,  125081, 14, }, // America/Argentina/Rio_Gallegos
    {  410, 7520,  125095,  6, }, // America/Argentina/Salta
    {  410, 7544,  125101, 11, }, // America/Argentina/San_Juan
    {  410, 5580,  125112, 11, }, // America/Argentina/San_Luis
    {  410, 7571,  125123,  9, }, // America/Argentina/Tucuman
    {  410, 7597,  122345,  6, }, // America/Argentina/Ushuaia
    {  410, 4252,  125132, 10, }, // America/Asuncion
    {  410,  237,  125142, 10, }, // America/Atikokan
    {  410, 3679,  122369,  5, }, // America/Bahia
    {  410, 7623,  125152, 16, }, // America/Bahia_Banderas
    {  410, 5654,  125168, 10, }, // America/Barbados
    {  410, 7646,  125178,  7, }, // America/Belem
    {  410, 5407,  125185,  7, }, // America/Belize
    {  410, 7660,  125192, 14, }, // America/Blanc-Sablon
    {  410, 7681,  125206,  9, }, // America/Boa_Vista
    {  410, 7699,  122427,  6, }, // America/Boise
    {  410, 7713,  125215, 15, }, // America/Cambridge_Bay
    {  410, 7735,  125230, 12, }, // America/Campo_Grande
    {  410, 3868,  125242,  8, }, // America/Cancun
    {  410, 4694,  125250,  9, }, // America/Caracas
    {  410, 4345,  125259,  7, }, // America/Cayenne
    {  410, 5496,  125266,  8, }, // America/Cayman
    {  410, 2260,  125274,  6, }, // America/Chicago
    {  410, 7756,  125280,  8, }, // America/Chihuahua
    {  410, 7774,  125288, 16, }, // America/Ciudad_Juarez
    {  410, 5422,   21621, 10, }, // America/Costa_Rica
    {  410, 7796,  125304,  9, }, // America/Creston
    {  410, 3791,    6743,  6, }, // America/Cuiaba
    {  410, 5723,  125313,  8, }, // America/Curacao
    {  410, 7812,  125321, 14, }, // America/Danmarkshavn
    {  410, 7833,  125335,  7, }, // America/Dawson
    {  410, 7848,  125342, 12, }, // America/Dawson_Creek
    {  410,  805,  125354,  8, }, // America/Denver
    {  410, 3465,  125362,  9, }, // America/Detroit
    {  410, 5739,   21666,  8, }, // America/Dominica
    {  410,  893,  125371, 10, }, // America/Edmonton
    {  410, 7869,  125381,  8, }, // America/Eirunepe
    {  410, 5441,  125389, 10, }, // America/El_Salvador
    {  410, 7886,  125399, 13, }, // America/Fort_Nelson
    {  410, 7924,  125412, 11, }, // America/Glace_Bay
    {  410, 6881,  125423,  9, }, // America/Goose_Bay
    {  410, 4612,  125432, 12, }, // America/Grand_Turk
    {  410, 5786,  125444,  9, }, // America/Guadeloupe
    {  410, 3760,   33338,  9, }, // America/Guatemala
    {  410, 4841,  125453, 10, }, // America/Guayaquil
    {  410, 6932,   57584,  7, }, // America/Guyana
    {  410, 1939,  125463, 10, }, // America/Halifax
    {  410, 7942,  125473, 11, }, // America/Hermosillo
    {  410,  481,  125484, 15, }, // America/Indiana/Knox
    {  410, 7961,  125499, 17, }, // America/Indiana/Marengo
    {  410, 7985,  125516, 21, }, // America/Indiana/Petersburg
    {  410, 8012,  125537, 18, }, // America/Indiana/Tell_City
    {  410, 8038,  125555, 14, }, // America/Indiana/Vevay
    {  410, 8060,  125569, 17, }, // America/Indiana/Vincennes
    {  410, 8086,  125586, 18, }, // America/Indiana/Winamac
    {  410, 8110,  125604,  8, }, // America/Inuvik
    {  410,  660,  125612,  9, }, // America/Iqaluit
    {  410, 2799,    3995,  7, }, // America/Jamaica
    {  410, 5380,  125621,  6, }, // America/Juneau
    {  410,  521,  125627,  9, }, // America/Kentucky/Louisville
    {  410, 8125,  125636, 20, }, // America/Kentucky/Monticello
    {  410, 5704,  125656, 12, }, // America/Kralendijk
    {  410, 4376,  125668,  8, }, // America/La_Paz
    {  410, 3239,  125676, 12, }, // America/Los_Angeles
    {  410, 5932,  125688, 27, }, // America/Lower_Princes
    {  410, 8153,  122903,  6, }, // America/Maceio
    {  410, 8168,  122909,  7, }, // America/Managua
    {  410, 1908,  125715,  8, }, // America/Manaus
    {  410, 5897,   61871,  6, }, // America/Marigot
    {  410, 5805,   26594,  9, }, // America/Martinique
    {  410, 8184,  125723, 11, }, // America/Matamoros
    {  410, 2917,  125734, 10, }, // America/Mazatlan
    {  410, 8202,  125744,  9, }, // America/Menominee
    {  410, 8220,    7126,  6, }, // America/Merida
    {  410, 2949,  125753,  8, }, // America/Mexico_City
    {  410, 4391,  125761,  9, }, // America/Miquelon
    {  410, 8254,  125770,  9, }, // America/Moncton
    {  410, 8270,  125779, 10, }, // America/Monterrey
    {  410, 4098,  125789, 12, }, // America/Montevideo
    {  410, 5824,  125801, 11, }, // America/Montserrat
    {  410, 5481,  125812,  5, }, // America/Nassau
    {  410, 2379,  125817,  9, }, // America/New_York
    {  410, 8288,  125826,  5, }, // America/Nome
    {  410, 1850,    7194,  7, }, // America/Noronha
    {  410, 8301,  125831, 21, }, // America/North_Dakota/Beulah
    {  410, 8329,  125852, 24, }, // America/North_Dakota/Center
    {  410, 8357,  125876, 28, }, // America/North_Dakota/New_Salem
    {  410,  393,  125904,  5, }, // America/Nuuk
    {  410, 8388,  125909,  8, }, // America/Ojinaga
    {  410, 6173,  125917, 11, }, // America/Paramaribo
    {  410, 2973,  125928,  8, }, // America/Phoenix
    {  410, 3945,  125936, 14, }, // America/Port-au-Prince
    {  410, 5954,  125950, 15, }, // America/Port_of_Spain
    {  410, 8404,  125965, 12, }, // America/Porto_Velho
    {  410, 5843,  125977, 12, }, // America/Puerto_Rico
    {  410, 4030,  125989, 14, }, // America/Punta_Arenas
    {  410, 8424,  126003, 14, }, // America/Rankin_Inlet
    {  410, 8445,  126017,  6, }, // America/Recife
    {  410, 1995,  126023,  7, }, // America/Regina
    {  410, 8460,  126030,  9, }, // America/Resolute
    {  410,  695,  126039, 11, }, // America/Rio_Branco
    {  410, 8477,  126050, 10, }, // America/Santarem
    {  410, 2201,  126060,  9, }, // America/Santiago
    {  410, 6111,  126069, 14, }, // America/Santo_Domingo
    {  410, 1878,  126083, 10, }, // America/Sao_Paulo
    {  410, 6899,  126093, 14, }, // America/Scoresbysund
    {  410, 8508,  126107, 13, }, // America/St_Barthelemy
    {  410, 2061,  126120, 12, }, // America/St_Johns
    {  410, 5863,  126132, 11, }, // America/St_Kitts
    {  410, 5880,  126143, 11, }, // America/St_Lucia
    {  410,  855,  126154, 12, }, // America/St_Thomas
    {  410, 5913,  126166, 14, }, // America/St_Vincent
    {  410, 8530,  126180, 14, }, // America/Swift_Current
    {  410, 5461,    5528, 11, }, // America/Tegucigalpa
    {  410, 5756,  126194,  7, }, // America/Thule
    {  410,  313,  126201,  8, }, // America/Tijuana
    {  410,  608,  126209,  8, }, // America/Toronto
    {  410, 2093,  126217, 10, }, // America/Vancouver
    {  410, 2144,  126227, 10, }, // America/Whitehorse
    {  410,  734,  126237,  9, }, // America/Winnipeg
    {  410, 8552,  126246,  9, }, // America/Yakutat
    {  410, 6001,  126255,  6, }, // Antarctica/Casey
    {  410, 6094,  126261,  8, }, // Antarctica/Davis
    {  410, 6133,  126269, 18, }, // Antarctica/DumontDUrville
    {  410, 8568,  126287,  8, }, // Antarctica/Macquarie
    {  410, 7066,  126295,  7, }, // Antarctica/Mawson
    {  410, 7113,  126302,  9, }, // Antarctica/McMurdo
    {  410, 8589,  126311,  8, }, // Antarctica/Palmer
    {  410, 7226,    5583,  6, }, // Antarctica/Rothera
    {  410, 7295,  126319,  5, }, // Antarctica/Syowa
    {  410, 8607,  126324,  6, }, // Antarctica/Troll
    {  410, 7374,  126330,  8, }, // Antarctica/Vostok
    {  410, 1429,  126338, 13, }, // Arctic/Longyearbyen
    {  410, 5570,  126351,  6, }, // Asia/Aden
    {  410, 5395,   57607,  8, }, // Asia/Almaty
    {  410, 3968,  126357,  6, }, // Asia/Amman
    {  410, 5511,  126363,  8, }, // Asia/Anadyr
    {  410, 5523,    5609,  5, }, // Asia/Aqtau
    {  410, 5368,  126371, 12, }, // Asia/Aqtobe
    {  410,  964,  126383, 10, }, // Asia/Ashgabat
    {  410, 3607,  126393,  8, }, // Asia/Baghdad
    {  410, 5534,  126401,  9, }, // Asia/Bahrain
    {  410, 4466,  126410,  8, }, // Asia/Bangkok
    {  410, 3571,  126418,  9, }, // Asia/Barnaul
    {  410, 4086,  126427,  8, }, // Asia/Beirut
    {  410, 3778,  126435,  8, }, // Asia/Bishkek
    {  410, 5989,  126443,  8, }, // Asia/Brunei
    {  410, 4601,   57622,  4, }, // Asia/Chita
    {  410, 4511,  126451,  8, }, // Asia/Colombo
    {  410, 4540,  126459,  8, }, // Asia/Damascus
    {  410, 1093,    4717,  4, }, // Asia/Dhaka
    {  410, 6192,  126467,  5, }, // Asia/Dili
    {  410, 3596,  126472,  7, }, // Asia/Dubai
    {  410, 6159,  126479,  7, }, // Asia/Dushanbe
    {  410, 8651,  126486, 12, }, // Asia/Gaza
    {  410, 4795,  126498,  8, }, // Asia/Hebron
    {  410, 1259,  126506,  9, }, // Asia/Ho_Chi_Minh
    {  410, 2704,  126515,  9, }, // Asia/Hong_Kong
    {  410, 4771,  126524,  6, }, // Asia/Hovd
    {  410, 4184,  126530,  9, }, // Asia/Irkutsk
    {  410, 7006,   33484,  8, }, // Asia/Jakarta
    {  410, 6992,   33492,  9, }, // Asia/Jayapura
    {  410, 1290,  126539,  8, }, // Asia/Jerusalem
    {  410, 3560,  126547,  7, }, // Asia/Kabul
    {  410, 4316,  126554,  9, }, // Asia/Kamchatka
    {  410, 4239,  126563,  7, }, // Asia/Karachi
    {  410, 1185,    5711,  8, }, // Asia/Kathmandu
    {  410, 8661,   58020,  8, }, // Asia/Khandyga
    {  410,  992,     457,  7, }, // Asia/Kolkata
    {  410, 4197,  126570, 13, }, // Asia/Krasnoyarsk
    {  410, 4859,  126583, 14, }, // Asia/Kuala_Lumpur
    {  410, 5976,  126597,  8, }, // Asia/Kuching
    {  410, 5547,  126605,  8, }, // Asia/Kuwait
    {  410, 1211,    7731,  5, }, // Asia/Macau
    {  410, 4017,  126613,  9, }, // Asia/Magadan
    {  410, 1349,  126622,  9, }, // Asia/Makassar
    {  410, 6920,  126631,  8, }, // Asia/Muscat
    {  410, 2554,   57283,  8, }, // Asia/Nicosia
    {  410, 8675,  126639, 13, }, // Asia/Novokuznetsk
    {  410, 4135,  126652, 13, }, // Asia/Novosibirsk
    {  410, 4229,  126665,  6, }, // Asia/Omsk
    {  410, 7145,  126671,  8, }, // Asia/Oral
    {  410, 6961,  126679,  9, }, // Asia/Phnom_Penh
    {  410, 8693,  126688, 11, }, // Asia/Pontianak
    {  410, 4214,  126699, 10, }, // Asia/Pyongyang
    {  410, 5559,  126709,  7, }, // Asia/Qatar
    {  410, 8708,  126716, 10, }, // Asia/Qostanay
    {  410, 4269,  126726,  9, }, // Asia/Qyzylorda
    {  410, 3584,  126735,  7, }, // Asia/Riyadh
    {  410, 4408,  126742, 10, }, // Asia/Sakhalin
    {  410, 7245,  126752, 11, }, // Asia/Samarkand
    {  410, 3279,  126763,  6, }, // Asia/Seoul
    {  410, 1053,  126769,  9, }, // Asia/Shanghai
    {  410, 3300,  126778, 10, }, // Asia/Singapore
    {  410, 4297,  126788, 15, }, // Asia/Srednekolymsk
    {  410, 3263,  126803,  8, }, // Asia/Taipei
    {  410, 4781,  126811,  9, }, // Asia/Tashkent
    {  410, 3915,  126820,  8, }, // Asia/Tbilisi
    {  410, 2772,  126828,  9, }, // Asia/Tehran
    {  410, 1317,   91770,  5, }, // Asia/Thimphu
    {  410, 2821,  126837,  7, }, // Asia/Tokyo
    {  410, 4572,  126844,  7, }, // Asia/Tomsk
    {  410, 1021,  126851, 12, }, // Asia/Ulaanbaatar
    {  410, 1159,   33596,  6, }, // Asia/Urumqi
    {  410, 8722,  126863,  8, }, // Asia/Ust-Nera
    {  410, 6977,  126871, 10, }, // Asia/Vientiane
    {  410, 4710,  126881, 13, }, // Asia/Vladivostok
    {  410, 4828,  126894,  9, }, // Asia/Yakutsk
    {  410, 1235,  126903,  8, }, // Asia/Yangon
    {  410, 3883,  126911, 15, }, // Asia/Yekaterinburg
    {  410, 3747,  126926,  9, }, // Asia/Yerevan
    {  410, 3663,  126935,  6, }, // Atlantic/Azores
    {  410, 6628,  126941, 12, }, // Atlantic/Canary
    {  410, 3727,  126953, 16, }, // Atlantic/Cape_Verde
    {  410, 1395,  126969,  7, }, // Atlantic/Faroe
    {  410, 2748,  126976, 11, }, // Atlantic/Reykjavik
    {  410, 7272,  126987, 15, }, // Atlantic/South_Georgia
    {  410, 6821,  127002, 11, }, // Atlantic/St_Helena
    {  410, 6644,  127013,  7, }, // Atlantic/Stanley
    {  410, 1670,  127020,  8, }, // Australia/Adelaide
    {  410, 1635,  127028,  9, }, // Australia/Brisbane
    {  410, 1799,  127037, 12, }, // Australia/Broken_Hill
    {  410, 1583,  127049,  8, }, // Australia/Darwin
    {  410, 3637,  123990,  5, }, // Australia/Eucla
    {  410, 1516,  127057,  8, }, // Australia/Hobart
    {  410, 8753,  127065, 10, }, // Australia/Lindeman
    {  410, 1547,  127075, 14, }, // Australia/Lord_Howe
    {  410, 1727,  127089,  9, }, // Australia/Melbourne
    {  410, 1762,  127098,  6, }, // Australia/Perth
    {  410, 1463,  127104,  8, }, // Australia/Sydney
    {  410, 6441,  127112, 11, }, // Europe/Amsterdam
    {  410, 6216,   33638,  6, }, // Europe/Andorra
    {  410, 3620,  127123, 10, }, // Europe/Astrakhan
    {  410, 2300,  127133,  6, }, // Europe/Athens
    {  410, 6336,  127139,  9, }, // Europe/Belgrade
    {  410, 4757,  127148,  8, }, // Europe/Berlin
    {  410, 2167,  127156,  9, }, // Europe/Brussels
    {  410, 3928,  127165, 10, }, // Europe/Bucharest
    {  410, 3806,  127175, 10, }, // Europe/Budapest
    {  410, 8772,  127185, 11, }, // Europe/Busingen
    {  410, 2583,  127196, 10, }, // Europe/Chisinau
    {  410, 6289,  127206,  9, }, // Europe/Copenhagen
    {  410, 2338,  127215,  8, }, // Europe/Dublin
    {  410, 6307,  127223, 11, }, // Europe/Gibraltar
    {  410, 8788,  127234,  7, }, // Europe/Guernsey
    {  410, 6612,  127241,  9, }, // Europe/Helsinki
    {  410, 8804,  127250,  9, }, // Europe/Isle_of_Man
    {  410, 1130,  127259,  9, }, // Europe/Istanbul
    {  410, 8823,  127268,  7, }, // Europe/Jersey
    {  410, 3979,  127275, 13, }, // Europe/Kaliningrad
    {  410, 8929,  127288,  7, }, // Europe/Kirov
    {  410, 2527,  127295,  7, }, // Europe/Kyiv
    {  410, 3213,  124207,  8, }, // Europe/Lisbon
    {  410, 6506,   21143,  9, }, // Europe/Ljubljana
    {  410, 2501,  127302,  8, }, // Europe/London
    {  410, 6365,  127310, 13, }, // Europe/Luxembourg
    {  410, 4892,  127323,  8, }, // Europe/Madrid
    {  410, 6582,  127331, 12, }, // Europe/Mariehamn
    {  410, 3693,  127343,  7, }, // Europe/Minsk
    {  410, 6410,  127350,  7, }, // Europe/Monaco
    {  410, 3537,  127357,  6, }, // Europe/Moscow
    {  410, 6458,  127363,  5, }, // Europe/Oslo
    {  410, 4284,  127368,  8, }, // Europe/Paris
    {  410, 6275,   36285,  5, }, // Europe/Prague
    {  410, 8837,  127376,  4, }, // Europe/Riga
    {  410, 6324,    6064,  4, }, // Europe/Rome
    {  410, 6470,  127380, 11, }, // Europe/San_Marino
    {  410, 6245,  127391,  9, }, // Europe/Sarajevo
    {  410, 4451,  127400,  9, }, // Europe/Saratov
    {  410, 8849,  127409, 12, }, // Europe/Simferopol
    {  410, 6383,  127421,  6, }, // Europe/Skopje
    {  410, 6599,   33761,  6, }, // Europe/Sofia
    {  410, 6523,  127427, 10, }, // Europe/Stockholm
    {  410, 8867,  127437,  7, }, // Europe/Tallinn
    {  410, 6202,   21213,  6, }, // Europe/Tirane
    {  410, 8882,  127444, 11, }, // Europe/Ulyanovsk
    {  410, 6352,  127455,  7, }, // Europe/Vaduz
    {  410, 6567,  127462,  9, }, // Europe/Vatican
    {  410, 6231,   21230,  5, }, // Europe/Vienna
    {  410, 4727,  127471, 11, }, // Europe/Volgograd
    {  410, 3190,  127482,  7, }, // Europe/Warsaw
    {  410, 6261,  127489,  8, }, // Europe/Zagreb
    {  410, 6540,  127497,  9, }, // Europe/Zurich
    {  410, 5078,  127506, 13, }, // Indian/Antananarivo
    {  410, 6947,  127519,  7, }, // Indian/Chagos
    {  410, 6046,  127526, 11, }, // Indian/Christmas
    {  410, 6063,  127537, 11, }, // Indian/Cocos
    {  410, 5029,  124437,  7, }, // Indian/Comoro
    {  410, 6661,  127548, 10, }, // Indian/Kerguelen
    {  410, 7260,  127558,  4, }, // Indian/Mahe
    {  410, 7050,  127562,  8, }, // Indian/Maldives
    {  410, 4069,   33826, 10, }, // Indian/Mauritius
    {  410, 5098,  127570,  7, }, // Indian/Mayotte
    {  410, 7211,  127577,  9, }, // Indian/Reunion
    {  410, 4422,  124493,  5, }, // Pacific/Apia
    {  410,  932,  127586,  8, }, // Pacific/Auckland
    {  410, 3706,  127594, 10, }, // Pacific/Bougainville
    {  410, 3015,  127604,  7, }, // Pacific/Chatham
    {  410, 3157,  127611,  5, }, // Pacific/Chuuk
    {  410, 2237,  127616, 11, }, // Pacific/Easter
    {  410, 7360,  127627,  5, }, // Pacific/Efate
    {  410, 7327,  127632,  8, }, // Pacific/Fakaofo
    {  410, 3902,  127640,  6, }, // Pacific/Fiji
    {  410, 7343,  127646,  9, }, // Pacific/Funafuti
    {  410, 6678,  127655,  9, }, // Pacific/Galapagos
    {  410, 6696,  127664,  8, }, // Pacific/Gambier
    {  410, 3822,  127672, 13, }, // Pacific/Guadalcanal
    {  410, 6018,  127685,  6, }, // Pacific/Guam
    {  410, 2723,    4611,  8, }, // Pacific/Honolulu
    {  410, 3049,  127691, 10, }, // Pacific/Kanton
    {  410, 3998,  127701, 11, }, // Pacific/Kiritimati
    {  410, 7019,  127712,  7, }, // Pacific/Kosrae
    {  410, 2842,  127719, 12, }, // Pacific/Kwajalein
    {  410, 4877,  127731,  8, }, // Pacific/Majuro
    {  410, 4051,  127739, 13, }, // Pacific/Marquesas
    {  410, 8914,  127752,  8, }, // Pacific/Midway
    {  410, 7132,  127760,  5, }, // Pacific/Niue
    {  410, 4168,  127765,  9, }, // Pacific/Norfolk
    {  410, 7098,  127774,  6, }, // Pacific/Noumea
    {  410, 3126,  127780, 10, }, // Pacific/Pago_Pago
    {  410, 7194,  127790, 13, }, // Pacific/Pitcairn
    {  410, 3096,  127803,  6, }, // Pacific/Pohnpei
    {  410, 4807,  127809, 12, }, // Pacific/Port_Moresby
    {  410, 6031,  127821,  8, }, // Pacific/Saipan
    {  410, 7312,  127829,  6, }, // Pacific/Tahiti
    {  410, 6712,   23159,  6, }, // Pacific/Tarawa
    {  410, 7392,  127835, 10, }, // Pacific/Wake
    {  410, 7405,  127845,  7, }, // Pacific/Wallis
    {  411,   14,       0,  6, }, // Africa/Asmara Lojban/Latin/world
    {  411, 4435,       6,  8, }, // Africa/Sao_Tome
    {  411, 4252,      14,  8, }, // America/Asuncion
    {  411,  237,      22,  8, }, // America/Atikokan
    {  411, 7623,      30, 17, }, // America/Bahia_Banderas
    {  411, 3868,      47,  6, }, // America/Cancun
    {  411, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  411, 5723,      66,  7, }, // America/Curacao
    {  411,  481,      73, 13, }, // America/Indiana/Knox
    {  411, 7961,      86, 16, }, // America/Indiana/Marengo
    {  411, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  411, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  411, 8038,     139, 14, }, // America/Indiana/Vevay
    {  411, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  411, 8086,     171, 16, }, // America/Indiana/Winamac
    {  411, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  411, 5932,     207, 22, }, // America/Lower_Princes
    {  411, 8220,     229,  6, }, // America/Merida
    {  411, 1850,     235, 19, }, // America/Noronha
    {  411, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  411, 8329,     274, 20, }, // America/North_Dakota/Center
    {  411, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  411,  393,     317,  4, }, // America/Nuuk
    {  411, 6899,     321, 16, }, // America/Scoresbysund
    {  411, 8508,     337, 14, }, // America/St_Barthelemy
    {  411, 2061,     351, 10, }, // America/St_Johns
    {  411, 5863,     361,  9, }, // America/St_Kitts
    {  411, 5880,     370,  9, }, // America/St_Lucia
    {  411,  855,     379, 10, }, // America/St_Thomas
    {  411, 5913,     389, 11, }, // America/St_Vincent
    {  411, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  411, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  411, 1185,     448,  9, }, // Asia/Kathmandu
    {  411,  992,     457,  7, }, // Asia/Kolkata
    {  411, 1211,     464,  5, }, // Asia/Macau
    {  411, 1235,     477,  6, }, // Asia/Yangon
    {  411, 1395,     483,  5, }, // Atlantic/Faroe
    {  411, 6821,     488, 10, }, // Atlantic/St_Helena
    {  411, 2527,     514,  4, }, // Europe/Kyiv
    {  411, 7211,     547,  7, }, // Indian/Reunion
    {  411, 3157,     554,  5, }, // Pacific/Chuuk
    {  411, 3096,     586,  7, }, // Pacific/Pohnpei
    {  412, 6760,   21292,  5, }, // Africa/Accra Lower Sorbian/Latin/Germany
    {  412, 5059,    3902, 11, }, // Africa/Addis_Ababa
    {  412, 7420,   58555,  6, }, // Africa/Algiers
    {  412,   14,       0,  6, }, // Africa/Asmara
    {  412, 2320,    4690,  5, }, // Africa/Cairo
    {  412, 5130,   78080, 10, }, // Africa/Dar_es_Salaam
    {  412, 5043,   33286,  7, }, // Africa/Djibouti
    {  412, 5166,   63491,  8, }, // Africa/El_Aaiun
    {  412, 4524,   26504,  7, }, // Africa/Khartoum
    {  412, 6869,   20418,  4, }, // Africa/Lome
    {  412, 5113,   57721,  8, }, // Africa/Mogadishu
    {  412, 6727,    4758,  8, }, // Africa/Ouagadougou
    {  412, 5225,   26542, 10, }, // Africa/Porto-Novo
    {  412, 4435,       6,  8, }, // Africa/Sao_Tome
    {  412, 2866,   57756,  8, }, // Africa/Tripoli
    {  412, 4252,      14,  8, }, // America/Asuncion
    {  412,  237,      22,  8, }, // America/Atikokan
    {  412, 7623,      30, 17, }, // America/Bahia_Banderas
    {  412, 4361,   20507,  6, }, // America/Bogota
    {  412, 3868,      47,  6, }, // America/Cancun
    {  412, 5496,  127852, 15, }, // America/Cayman
    {  412, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  412, 5723,      66,  7, }, // America/Curacao
    {  412, 2281,   63505,  7, }, // America/Havana
    {  412,  481,      73, 13, }, // America/Indiana/Knox
    {  412, 7961,      86, 16, }, // America/Indiana/Marengo
    {  412, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  412, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  412, 8038,     139, 14, }, // America/Indiana/Vevay
    {  412, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  412, 8086,     171, 16, }, // America/Indiana/Winamac
    {  412, 2799,    3995,  7, }, // America/Jamaica
    {  412, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  412, 5932,     207, 22, }, // America/Lower_Princes
    {  412, 8220,     229,  6, }, // America/Merida
    {  412, 2949,  127867, 12, }, // America/Mexico_City
    {  412, 1850,     235, 19, }, // America/Noronha
    {  412, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  412, 8329,     274, 20, }, // America/North_Dakota/Center
    {  412, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  412,  393,     317,  4, }, // America/Nuuk
    {  412, 5954,   26691, 13, }, // America/Port_of_Spain
    {  412, 1878,   26712,  9, }, // America/Sao_Paulo
    {  412, 6899,     321, 16, }, // America/Scoresbysund
    {  412, 8508,  127879, 13, }, // America/St_Barthelemy
    {  412, 2061,     351, 10, }, // America/St_Johns
    {  412, 5863,     361,  9, }, // America/St_Kitts
    {  412, 5880,     370,  9, }, // America/St_Lucia
    {  412,  855,     379, 10, }, // America/St_Thomas
    {  412, 5913,     389, 11, }, // America/St_Vincent
    {  412, 6133,  127892, 16, }, // Antarctica/DumontDUrville
    {  412, 7374,    4147,  6, }, // Antarctica/Vostok
    {  412, 5368,    7606,  6, }, // Asia/Aqtobe
    {  412, 3607,    4162,  6, }, // Asia/Baghdad
    {  412, 3778,   33455,  6, }, // Asia/Bishkek
    {  412, 4540,    4201,  8, }, // Asia/Damascus
    {  412, 6159,   33468,  7, }, // Asia/Dushanbe
    {  412, 1259,  127908, 17, }, // Asia/Ho_Chi_Minh
    {  412, 2704,    4231,  8, }, // Asia/Hong_Kong
    {  412, 6992,  127925, 11, }, // Asia/Jayapura
    {  412, 4316,   33510,  8, }, // Asia/Kamchatka
    {  412, 4239,   33518,  6, }, // Asia/Karachi
    {  412, 1185,     448,  9, }, // Asia/Kathmandu
    {  412,  992,   64334,  8, }, // Asia/Kolkata
    {  412, 4197,    4288, 11, }, // Asia/Krasnoyarsk
    {  412, 1211,     464,  5, }, // Asia/Macau
    {  412, 6920,    4326,  6, }, // Asia/Muscat
    {  412, 2554,    5738,  7, }, // Asia/Nicosia
    {  412, 8675,  127936, 13, }, // Asia/Novokuznetsk
    {  412, 4135,   78211, 11, }, // Asia/Novosibirsk
    {  412, 4214,   78222,  9, }, // Asia/Pyongyang
    {  412, 5559,    4332,  5, }, // Asia/Qatar
    {  412, 3584,    7787,  4, }, // Asia/Riyadh
    {  412, 4408,   58084,  8, }, // Asia/Sakhalin
    {  412, 3300,   20900,  8, }, // Asia/Singapore
    {  412, 3263,   78231,  6, }, // Asia/Taipei
    {  412, 4781,   33589,  7, }, // Asia/Tashkent
    {  412, 2772,    7842,  7, }, // Asia/Tehran
    {  412, 2821,    4356,  5, }, // Asia/Tokyo
    {  412, 4710,    4361, 11, }, // Asia/Vladivostok
    {  412, 4828,    7873,  7, }, // Asia/Yakutsk
    {  412, 1235,    7880,  6, }, // Asia/Yangon
    {  412, 3883,    4380, 13, }, // Asia/Yekaterinburg
    {  412, 3747,  127949,  7, }, // Asia/Yerevan
    {  412, 3663,  127956,  5, }, // Atlantic/Azores
    {  412, 5671,   58175,  7, }, // Atlantic/Bermuda
    {  412, 6628,  117831,  6, }, // Atlantic/Canary
    {  412, 3727,   57380,  9, }, // Atlantic/Cape_Verde
    {  412, 1395,  127961,  6, }, // Atlantic/Faroe
    {  412, 2748,   36176,  9, }, // Atlantic/Reykjavik
    {  412, 7272,  127967, 22, }, // Atlantic/South_Georgia
    {  412, 6821,     488, 10, }, // Atlantic/St_Helena
    {  412, 2300,    4438,  5, }, // Europe/Athens
    {  412, 6336,  127989,  8, }, // Europe/Belgrade
    {  412, 2167,   22766,  7, }, // Europe/Brussels
    {  412, 3928,   27029,  8, }, // Europe/Bucharest
    {  412, 8772,   27037,  8, }, // Europe/Busingen
    {  412, 2583,  127997,  7, }, // Europe/Chisinau
    {  412, 6289,    4475, 10, }, // Europe/Copenhagen
    {  412, 2527,   78292,  4, }, // Europe/Kyiv
    {  412, 3213,    4499,  8, }, // Europe/Lisbon
    {  412, 6365,    4513,  9, }, // Europe/Luxembourg
    {  412, 3537,   93233,  6, }, // Europe/Moscow
    {  412, 6275,   21185,  5, }, // Europe/Prague
    {  412, 6324,    6064,  3, }, // Europe/Rome
    {  412, 6202,   21213,  6, }, // Europe/Tirane
    {  412, 6567,    6096,  7, }, // Europe/Vatican
    {  412, 6231,   57515,  4, }, // Europe/Vienna
    {  412, 8899,  128004,  5, }, // Europe/Vilnius
    {  412, 4727,    4561,  9, }, // Europe/Volgograd
    {  412, 3190,  128009,  7, }, // Europe/Warsaw
    {  412, 6540,    4577,  6, }, // Europe/Zurich
    {  412, 6046,  128016, 12, }, // Indian/Christmas
    {  412, 5029,   58408,  6, }, // Indian/Comoro
    {  412, 7050,  128028,  8, }, // Indian/Maldives
    {  412, 7211,     547,  7, }, // Indian/Reunion
    {  412, 3157,     554,  5, }, // Pacific/Chuuk
    {  412, 2237,  128036, 13, }, // Pacific/Easter
    {  412, 3902,   33836,  5, }, // Pacific/Fiji
    {  412, 3049,    4619,  9, }, // Pacific/Kanton
    {  412, 3096,     586,  7, }, // Pacific/Pohnpei
    {  413,   14,       0,  6, }, // Africa/Asmara Low German/Latin/Germany
    {  413, 4435,       6,  8, }, // Africa/Sao_Tome
    {  413, 4252,      14,  8, }, // America/Asuncion
    {  413,  237,      22,  8, }, // America/Atikokan
    {  413, 7623,      30, 17, }, // America/Bahia_Banderas
    {  413, 3868,      47,  6, }, // America/Cancun
    {  413, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  413, 5723,      66,  7, }, // America/Curacao
    {  413,  481,      73, 13, }, // America/Indiana/Knox
    {  413, 7961,      86, 16, }, // America/Indiana/Marengo
    {  413, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  413, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  413, 8038,     139, 14, }, // America/Indiana/Vevay
    {  413, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  413, 8086,     171, 16, }, // America/Indiana/Winamac
    {  413, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  413, 5932,     207, 22, }, // America/Lower_Princes
    {  413, 8220,     229,  6, }, // America/Merida
    {  413, 2949,   78121, 12, }, // America/Mexico_City
    {  413, 1850,     235, 19, }, // America/Noronha
    {  413, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  413, 8329,    4033, 20, }, // America/North_Dakota/Center
    {  413, 8357,    4053, 23, }, // America/North_Dakota/New_Salem
    {  413,  393,     317,  4, }, // America/Nuuk
    {  413, 6899,     321, 16, }, // America/Scoresbysund
    {  413, 8508,     337, 14, }, // America/St_Barthelemy
    {  413, 2061,     351, 10, }, // America/St_Johns
    {  413, 5863,     361,  9, }, // America/St_Kitts
    {  413, 5880,     370,  9, }, // America/St_Lucia
    {  413,  855,     379, 10, }, // America/St_Thomas
    {  413, 5913,     389, 11, }, // America/St_Vincent
    {  413, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  413, 7374,    4147,  6, }, // Antarctica/Vostok
    {  413, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  413, 4316,  128049, 10, }, // Asia/Kamchatka
    {  413, 1185,     448,  9, }, // Asia/Kathmandu
    {  413,  992,     457,  7, }, // Asia/Kolkata
    {  413, 4197,    4288, 11, }, // Asia/Krasnoyarsk
    {  413, 1211,     464,  5, }, // Asia/Macau
    {  413, 4135,   78211, 11, }, // Asia/Novosibirsk
    {  413, 4408,   58084,  8, }, // Asia/Sakhalin
    {  413, 1021,   22598,  9, }, // Asia/Ulaanbaatar
    {  413, 4710,    4361, 11, }, // Asia/Vladivostok
    {  413, 4828,    7873,  7, }, // Asia/Yakutsk
    {  413, 1235,     477,  6, }, // Asia/Yangon
    {  413, 3883,  128059, 13, }, // Asia/Yekaterinburg
    {  413, 3663,   58841,  6, }, // Atlantic/Azores
    {  413, 6628,   78243,  7, }, // Atlantic/Canary
    {  413, 1395,     483,  5, }, // Atlantic/Faroe
    {  413, 6821,     488, 10, }, // Atlantic/St_Helena
    {  413, 2527,     514,  4, }, // Europe/Kyiv
    {  413, 3537,   57485,  6, }, // Europe/Moscow
    {  413, 4727,    4561,  9, }, // Europe/Volgograd
    {  413, 7211,     547,  7, }, // Indian/Reunion
    {  413, 3157,     554,  5, }, // Pacific/Chuuk
    {  413, 3096,     586,  7, }, // Pacific/Pohnpei
    {  415,   14,       0,  6, }, // Africa/Asmara Luba-Katanga/Latin/Congo - Kinshasa
    {  415, 4435,       6,  8, }, // Africa/Sao_Tome
    {  415, 4252,      14,  8, }, // America/Asuncion
    {  415,  237,      22,  8, }, // America/Atikokan
    {  415, 7623,      30, 17, }, // America/Bahia_Banderas
    {  415, 3868,      47,  6, }, // America/Cancun
    {  415, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  415, 5723,      66,  7, }, // America/Curacao
    {  415,  481,      73, 13, }, // America/Indiana/Knox
    {  415, 7961,      86, 16, }, // America/Indiana/Marengo
    {  415, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  415, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  415, 8038,     139, 14, }, // America/Indiana/Vevay
    {  415, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  415, 8086,     171, 16, }, // America/Indiana/Winamac
    {  415, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  415, 5932,     207, 22, }, // America/Lower_Princes
    {  415, 8220,     229,  6, }, // America/Merida
    {  415, 1850,     235, 19, }, // America/Noronha
    {  415, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  415, 8329,     274, 20, }, // America/North_Dakota/Center
    {  415, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  415,  393,     317,  4, }, // America/Nuuk
    {  415, 6899,     321, 16, }, // America/Scoresbysund
    {  415, 8508,     337, 14, }, // America/St_Barthelemy
    {  415, 2061,     351, 10, }, // America/St_Johns
    {  415, 5863,     361,  9, }, // America/St_Kitts
    {  415, 5880,     370,  9, }, // America/St_Lucia
    {  415,  855,     379, 10, }, // America/St_Thomas
    {  415, 5913,     389, 11, }, // America/St_Vincent
    {  415, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  415, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  415, 1185,     448,  9, }, // Asia/Kathmandu
    {  415,  992,     457,  7, }, // Asia/Kolkata
    {  415, 1211,     464,  5, }, // Asia/Macau
    {  415, 1235,     477,  6, }, // Asia/Yangon
    {  415, 1395,     483,  5, }, // Atlantic/Faroe
    {  415, 6821,     488, 10, }, // Atlantic/St_Helena
    {  415, 2527,     514,  4, }, // Europe/Kyiv
    {  415, 7211,     547,  7, }, // Indian/Reunion
    {  415, 3157,     554,  5, }, // Pacific/Chuuk
    {  415, 3096,     586,  7, }, // Pacific/Pohnpei
    {  416,   14,       0,  6, }, // Africa/Asmara Lule Sami/Latin/Sweden
    {  416, 4435,       6,  8, }, // Africa/Sao_Tome
    {  416, 4252,      14,  8, }, // America/Asuncion
    {  416,  237,      22,  8, }, // America/Atikokan
    {  416, 7623,      30, 17, }, // America/Bahia_Banderas
    {  416, 3868,      47,  6, }, // America/Cancun
    {  416, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  416, 5723,      66,  7, }, // America/Curacao
    {  416,  481,      73, 13, }, // America/Indiana/Knox
    {  416, 7961,      86, 16, }, // America/Indiana/Marengo
    {  416, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  416, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  416, 8038,     139, 14, }, // America/Indiana/Vevay
    {  416, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  416, 8086,     171, 16, }, // America/Indiana/Winamac
    {  416, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  416, 5932,     207, 22, }, // America/Lower_Princes
    {  416, 8220,     229,  6, }, // America/Merida
    {  416, 1850,     235, 19, }, // America/Noronha
    {  416, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  416, 8329,     274, 20, }, // America/North_Dakota/Center
    {  416, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  416,  393,     317,  4, }, // America/Nuuk
    {  416, 6899,     321, 16, }, // America/Scoresbysund
    {  416, 8508,     337, 14, }, // America/St_Barthelemy
    {  416, 2061,     351, 10, }, // America/St_Johns
    {  416, 5863,     361,  9, }, // America/St_Kitts
    {  416, 5880,     370,  9, }, // America/St_Lucia
    {  416,  855,     379, 10, }, // America/St_Thomas
    {  416, 5913,     389, 11, }, // America/St_Vincent
    {  416, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  416, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  416, 1185,     448,  9, }, // Asia/Kathmandu
    {  416,  992,     457,  7, }, // Asia/Kolkata
    {  416, 1211,     464,  5, }, // Asia/Macau
    {  416, 1235,     477,  6, }, // Asia/Yangon
    {  416, 1395,     483,  5, }, // Atlantic/Faroe
    {  416, 6821,     488, 10, }, // Atlantic/St_Helena
    {  416, 2527,     514,  4, }, // Europe/Kyiv
    {  416, 7211,     547,  7, }, // Indian/Reunion
    {  416, 3157,     554,  5, }, // Pacific/Chuuk
    {  416, 3096,     586,  7, }, // Pacific/Pohnpei
    {  418,   14,       0,  6, }, // Africa/Asmara Luo/Latin/Kenya
    {  418, 4435,       6,  8, }, // Africa/Sao_Tome
    {  418, 4252,      14,  8, }, // America/Asuncion
    {  418,  237,      22,  8, }, // America/Atikokan
    {  418, 7623,      30, 17, }, // America/Bahia_Banderas
    {  418, 3868,      47,  6, }, // America/Cancun
    {  418, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  418, 5723,      66,  7, }, // America/Curacao
    {  418,  481,      73, 13, }, // America/Indiana/Knox
    {  418, 7961,      86, 16, }, // America/Indiana/Marengo
    {  418, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  418, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  418, 8038,     139, 14, }, // America/Indiana/Vevay
    {  418, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  418, 8086,     171, 16, }, // America/Indiana/Winamac
    {  418, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  418, 5932,     207, 22, }, // America/Lower_Princes
    {  418, 8220,     229,  6, }, // America/Merida
    {  418, 1850,     235, 19, }, // America/Noronha
    {  418, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  418, 8329,     274, 20, }, // America/North_Dakota/Center
    {  418, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  418,  393,     317,  4, }, // America/Nuuk
    {  418, 6899,     321, 16, }, // America/Scoresbysund
    {  418, 8508,     337, 14, }, // America/St_Barthelemy
    {  418, 2061,     351, 10, }, // America/St_Johns
    {  418, 5863,     361,  9, }, // America/St_Kitts
    {  418, 5880,     370,  9, }, // America/St_Lucia
    {  418,  855,     379, 10, }, // America/St_Thomas
    {  418, 5913,     389, 11, }, // America/St_Vincent
    {  418, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  418, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  418, 1185,     448,  9, }, // Asia/Kathmandu
    {  418,  992,     457,  7, }, // Asia/Kolkata
    {  418, 1211,     464,  5, }, // Asia/Macau
    {  418, 1235,     477,  6, }, // Asia/Yangon
    {  418, 1395,     483,  5, }, // Atlantic/Faroe
    {  418, 6821,     488, 10, }, // Atlantic/St_Helena
    {  418, 2527,     514,  4, }, // Europe/Kyiv
    {  418, 7211,     547,  7, }, // Indian/Reunion
    {  418, 3157,     554,  5, }, // Pacific/Chuuk
    {  418, 3096,     586,  7, }, // Pacific/Pohnpei
    {  419, 5059,    3902, 11, }, // Africa/Addis_Ababa Luxembourgish/Latin/Luxembourg
    {  419, 7420,   44088,  5, }, // Africa/Algiers
    {  419,   14,       0,  6, }, // Africa/Asmara
    {  419, 2320,    4690,  5, }, // Africa/Cairo
    {  419, 5043,   78090,  9, }, // Africa/Djibouti
    {  419, 5166,   63491,  8, }, // Africa/El_Aaiun
    {  419, 4524,   26504,  7, }, // Africa/Khartoum
    {  419, 5113,   78099, 10, }, // Africa/Mogadishu
    {  419, 6727,    4758,  8, }, // Africa/Ouagadougou
    {  419, 4435,       6,  8, }, // Africa/Sao_Tome
    {  419, 4252,      14,  8, }, // America/Asuncion
    {  419,  237,      22,  8, }, // America/Atikokan
    {  419, 7623,      30, 17, }, // America/Bahia_Banderas
    {  419, 3868,      47,  6, }, // America/Cancun
    {  419, 5496,  128072, 13, }, // America/Cayman
    {  419, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  419, 5723,      66,  7, }, // America/Curacao
    {  419, 5441,    6806,  8, }, // America/El_Salvador
    {  419, 2281,   63505,  7, }, // America/Havana
    {  419,  481,      73, 13, }, // America/Indiana/Knox
    {  419, 7961,      86, 16, }, // America/Indiana/Marengo
    {  419, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  419, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  419, 8038,     139, 14, }, // America/Indiana/Vevay
    {  419, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  419, 8086,     171, 16, }, // America/Indiana/Winamac
    {  419, 2799,    3995,  7, }, // America/Jamaica
    {  419, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  419, 5932,     207, 22, }, // America/Lower_Princes
    {  419, 8220,     229,  6, }, // America/Merida
    {  419, 2949,   78121, 11, }, // America/Mexico_City
    {  419, 1850,     235, 19, }, // America/Noronha
    {  419, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  419, 8329,     274, 20, }, // America/North_Dakota/Center
    {  419, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  419,  393,     317,  4, }, // America/Nuuk
    {  419, 5954,   26691, 13, }, // America/Port_of_Spain
    {  419, 6899,     321, 16, }, // America/Scoresbysund
    {  419, 8508,   26721, 16, }, // America/St_Barthelemy
    {  419, 2061,     351, 10, }, // America/St_Johns
    {  419, 5863,     361,  9, }, // America/St_Kitts
    {  419, 5880,     370,  9, }, // America/St_Lucia
    {  419,  855,     379, 10, }, // America/St_Thomas
    {  419, 5913,     389, 11, }, // America/St_Vincent
    {  419, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  419, 7374,    4147,  6, }, // Antarctica/Vostok
    {  419, 5368,   57205,  6, }, // Asia/Aqtobe
    {  419, 3607,    4162,  6, }, // Asia/Baghdad
    {  419, 3778,   57220,  8, }, // Asia/Bishkek
    {  419, 4540,    4201,  8, }, // Asia/Damascus
    {  419, 6159,   57228,  9, }, // Asia/Dushanbe
    {  419, 1259,   78157, 16, }, // Asia/Ho_Chi_Minh
    {  419, 6992,  127925, 11, }, // Asia/Jayapura
    {  419, 4316,   78179, 11, }, // Asia/Kamchatka
    {  419, 1185,     448,  9, }, // Asia/Kathmandu
    {  419,  992,   64334,  8, }, // Asia/Kolkata
    {  419, 4197,    4288, 11, }, // Asia/Krasnoyarsk
    {  419, 1211,    4321,  5, }, // Asia/Macau
    {  419, 6920,    4326,  6, }, // Asia/Muscat
    {  419, 2554,    5738,  7, }, // Asia/Nicosia
    {  419, 4135,   78211, 11, }, // Asia/Novosibirsk
    {  419, 4214,   78222,  9, }, // Asia/Pyongyang
    {  419, 5559,    4332,  5, }, // Asia/Qatar
    {  419, 3584,    7787,  4, }, // Asia/Riyadh
    {  419, 4408,   58084,  8, }, // Asia/Sakhalin
    {  419, 3300,   20900,  8, }, // Asia/Singapore
    {  419, 3263,   78231,  6, }, // Asia/Taipei
    {  419, 4781,   57328,  9, }, // Asia/Tashkent
    {  419, 3915,   78237,  6, }, // Asia/Tbilisi
    {  419, 2772,    7842,  7, }, // Asia/Tehran
    {  419, 2821,    4356,  5, }, // Asia/Tokyo
    {  419, 4710,    4361, 11, }, // Asia/Vladivostok
    {  419, 4828,    7873,  7, }, // Asia/Yakutsk
    {  419, 1235,     477,  6, }, // Asia/Yangon
    {  419, 3883,  128085, 14, }, // Asia/Yekaterinburg
    {  419, 3747,  128099,  6, }, // Asia/Yerevan
    {  419, 3663,   58841,  6, }, // Atlantic/Azores
    {  419, 5671,   74483,  8, }, // Atlantic/Bermuda
    {  419, 6628,  128105, 17, }, // Atlantic/Canary
    {  419, 3727,   57380,  9, }, // Atlantic/Cape_Verde
    {  419, 1395,   78250,  6, }, // Atlantic/Faroe
    {  419, 7272,   78267, 11, }, // Atlantic/South_Georgia
    {  419, 6821,     488, 10, }, // Atlantic/St_Helena
    {  419, 2300,    4438,  5, }, // Europe/Athens
    {  419, 6336,   27015,  7, }, // Europe/Belgrade
    {  419, 2167,  128122,  8, }, // Europe/Brussels
    {  419, 3928,   27029,  8, }, // Europe/Bucharest
    {  419, 2583,  128130,  9, }, // Europe/Chisinau
    {  419, 6289,    4475, 10, }, // Europe/Copenhagen
    {  419, 2527,   78292,  4, }, // Europe/Kyiv
    {  419, 3213,    4499,  8, }, // Europe/Lisbon
    {  419, 6365,  128139, 10, }, // Europe/Luxembourg
    {  419, 3537,   57485,  6, }, // Europe/Moscow
    {  419, 6275,    8201,  4, }, // Europe/Prague
    {  419, 6324,  128149,  4, }, // Europe/Rome
    {  419, 6202,   21213,  6, }, // Europe/Tirane
    {  419, 6567,    6096,  7, }, // Europe/Vatican
    {  419, 6231,   57515,  4, }, // Europe/Vienna
    {  419, 8899,  128004,  5, }, // Europe/Vilnius
    {  419, 4727,    4561,  9, }, // Europe/Volgograd
    {  419, 3190,   57528,  8, }, // Europe/Warsaw
    {  419, 6540,  128153,  6, }, // Europe/Zurich
    {  419, 6046,  128159, 17, }, // Indian/Christmas
    {  419, 5029,   78327,  7, }, // Indian/Comoro
    {  419, 7050,   58995,  8, }, // Indian/Maldives
    {  419, 7211,     547,  7, }, // Indian/Reunion
    {  419, 3157,     554,  5, }, // Pacific/Chuuk
    {  419, 2237,  128176, 13, }, // Pacific/Easter
    {  419, 3902,   78353,  7, }, // Pacific/Fiji
    {  419, 3096,     586,  7, }, // Pacific/Pohnpei
    {  420,   14,       0,  6, }, // Africa/Asmara Luyia/Latin/Kenya
    {  420, 4435,       6,  8, }, // Africa/Sao_Tome
    {  420, 4252,      14,  8, }, // America/Asuncion
    {  420,  237,      22,  8, }, // America/Atikokan
    {  420, 7623,      30, 17, }, // America/Bahia_Banderas
    {  420, 3868,      47,  6, }, // America/Cancun
    {  420, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  420, 5723,      66,  7, }, // America/Curacao
    {  420,  481,      73, 13, }, // America/Indiana/Knox
    {  420, 7961,      86, 16, }, // America/Indiana/Marengo
    {  420, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  420, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  420, 8038,     139, 14, }, // America/Indiana/Vevay
    {  420, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  420, 8086,     171, 16, }, // America/Indiana/Winamac
    {  420, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  420, 5932,     207, 22, }, // America/Lower_Princes
    {  420, 8220,     229,  6, }, // America/Merida
    {  420, 1850,     235, 19, }, // America/Noronha
    {  420, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  420, 8329,     274, 20, }, // America/North_Dakota/Center
    {  420, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  420,  393,     317,  4, }, // America/Nuuk
    {  420, 6899,     321, 16, }, // America/Scoresbysund
    {  420, 8508,     337, 14, }, // America/St_Barthelemy
    {  420, 2061,     351, 10, }, // America/St_Johns
    {  420, 5863,     361,  9, }, // America/St_Kitts
    {  420, 5880,     370,  9, }, // America/St_Lucia
    {  420,  855,     379, 10, }, // America/St_Thomas
    {  420, 5913,     389, 11, }, // America/St_Vincent
    {  420, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  420, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  420, 1185,     448,  9, }, // Asia/Kathmandu
    {  420,  992,     457,  7, }, // Asia/Kolkata
    {  420, 1211,     464,  5, }, // Asia/Macau
    {  420, 1235,     477,  6, }, // Asia/Yangon
    {  420, 1395,     483,  5, }, // Atlantic/Faroe
    {  420, 6821,     488, 10, }, // Atlantic/St_Helena
    {  420, 2527,     514,  4, }, // Europe/Kyiv
    {  420, 7211,     547,  7, }, // Indian/Reunion
    {  420, 3157,     554,  5, }, // Pacific/Chuuk
    {  420, 3096,     586,  7, }, // Pacific/Pohnpei
    {  421, 6788,     604,  6, }, // Africa/Abidjan Macedonian/Cyrillic/Macedonia
    {  421, 6760,   27178,  4, }, // Africa/Accra
    {  421, 5059,   33846, 10, }, // Africa/Addis_Ababa
    {  421, 7420,     626,  5, }, // Africa/Algiers
    {  421,   14,   27197,  6, }, // Africa/Asmara
    {  421,   44,     637,  6, }, // Africa/Bamako
    {  421, 5257,     643,  5, }, // Africa/Bangui
    {  421, 6746,  128189,  6, }, // Africa/Banjul
    {  421, 7435,     654,  5, }, // Africa/Bissau
    {  421, 4957,  128195,  8, }, // Africa/Blantyre
    {  421, 5287,   33874,  8, }, // Africa/Brazzaville
    {  421, 4922,   33882,  9, }, // Africa/Bujumbura
    {  421, 2320,   33891,  5, }, // Africa/Cairo
    {  421, 4117,   33896, 10, }, // Africa/Casablanca
    {  421, 7449,     699,  5, }, // Africa/Ceuta
    {  421, 6773,     704,  7, }, // Africa/Conakry
    {  421, 6840,     711,  5, }, // Africa/Dakar
    {  421, 5130,   36367, 12, }, // Africa/Dar_es_Salaam
    {  421, 5043,     728,  6, }, // Africa/Djibouti
    {  421, 5243,     734,  5, }, // Africa/Douala
    {  421, 5166,   33906,  7, }, // Africa/El_Aaiun
    {  421, 6853,     746,  7, }, // Africa/Freetown
    {  421, 4906,     753,  8, }, // Africa/Gaborone
    {  421, 5015,   33913,  6, }, // Africa/Harare
    {  421, 4479,   33919, 11, }, // Africa/Johannesburg
    {  421, 4499,     779,  4, }, // Africa/Juba
    {  421, 5151,     783,  7, }, // Africa/Kampala
    {  421, 4524,   33930,  6, }, // Africa/Khartoum
    {  421, 4973,     796,  6, }, // Africa/Kigali
    {  421, 5306,     802,  7, }, // Africa/Kinshasa
    {  421, 4744,     809,  5, }, // Africa/Lagos
    {  421, 5336,  128203,  7, }, // Africa/Libreville
    {  421, 6869,     822,  4, }, // Africa/Lome
    {  421, 5211,     826,  6, }, // Africa/Luanda
    {  421, 4939,     832,  9, }, // Africa/Lubumbashi
    {  421, 5001,     841,  6, }, // Africa/Lusaka
    {  421, 5322,     847,  6, }, // Africa/Malabo
    {  421, 4987,   33945,  6, }, // Africa/Maputo
    {  421, 5197,     859,  6, }, // Africa/Maseru
    {  421, 5182,     865,  7, }, // Africa/Mbabane
    {  421, 5113,   36411,  8, }, // Africa/Mogadishu
    {  421, 7034,   33951,  9, }, // Africa/Monrovia
    {  421, 3853,   33960,  7, }, // Africa/Nairobi
    {  421, 5271,     895,  7, }, // Africa/Ndjamena
    {  421, 5354,   33967,  7, }, // Africa/Niamey
    {  421, 6803,     908,  7, }, // Africa/Nouakchott
    {  421, 6727,     915,  8, }, // Africa/Ouagadougou
    {  421, 5225,   33974, 10, }, // Africa/Porto-Novo
    {  421, 4435,  128210,  9, }, // Africa/Sao_Tome
    {  421, 2866,     941,  7, }, // Africa/Tripoli
    {  421, 6554,     948,  5, }, // Africa/Tunis
    {  421, 4152,     953,  7, }, // Africa/Windhoek
    {  421,  132,     960,  4, }, // America/Adak
    {  421, 3346,   33992,  7, }, // America/Anchorage
    {  421, 5607,   33999,  7, }, // America/Anguilla
    {  421, 5624,  128219,  6, }, // America/Antigua
    {  421, 4554,   34013, 10, }, // America/Araguaina
    {  421,  166,   34023, 12, }, // America/Argentina/Buenos_Aires
    {  421,   91,    1006,  9, }, // America/Argentina/Catamarca
    {  421,  270,   34035,  7, }, // America/Argentina/Cordoba
    {  421,  441,   34042,  5, }, // America/Argentina/Jujuy
    {  421, 7462,   34047,  8, }, // America/Argentina/La_Rioja
    {  421,  565,   34055,  7, }, // America/Argentina/Mendoza
    {  421, 7489,   36481, 11, }, // America/Argentina/Rio_Gallegos
    {  421, 7520,    1053,  5, }, // America/Argentina/Salta
    {  421, 7544,   34073,  8, }, // America/Argentina/San_Juan
    {  421, 5580,   36492,  8, }, // America/Argentina/San_Luis
    {  421, 7571,    1074,  7, }, // America/Argentina/Tucuman
    {  421, 7597,  128225,  6, }, // America/Argentina/Ushuaia
    {  421, 5640,    1087,  5, }, // America/Aruba
    {  421, 4252,   34095,  8, }, // America/Asuncion
    {  421,  237,  128231, 12, }, // America/Atikokan
    {  421, 3679,  128243,  5, }, // America/Bahia
    {  421, 7623,  128248, 18, }, // America/Bahia_Banderas
    {  421, 5654,    1131,  8, }, // America/Barbados
    {  421, 7646,   34130,  5, }, // America/Belem
    {  421, 5407,   34135,  6, }, // America/Belize
    {  421, 7660,   34141, 13, }, // America/Blanc-Sablon
    {  421, 7681,   34154,  9, }, // America/Boa_Vista
    {  421, 4361,    1170,  6, }, // America/Bogota
    {  421, 7699,   34163,  5, }, // America/Boise
    {  421, 7713,   34168, 11, }, // America/Cambridge_Bay
    {  421, 7735,   34179, 12, }, // America/Campo_Grande
    {  421, 3868,    1205,  6, }, // America/Cancun
    {  421, 4694,    1211,  7, }, // America/Caracas
    {  421, 4345,  128266,  6, }, // America/Cayenne
    {  421, 5496,  128272, 17, }, // America/Cayman
    {  421, 2260,    1247,  6, }, // America/Chicago
    {  421, 7756,   34212,  6, }, // America/Chihuahua
    {  421, 7774,  128289, 13, }, // America/Ciudad_Juarez
    {  421, 5422,   34218,  9, }, // America/Costa_Rica
    {  421, 7796,    1269,  7, }, // America/Creston
    {  421, 3791,  128302,  6, }, // America/Cuiaba
    {  421, 5723,   34227,  7, }, // America/Curacao
    {  421, 7812,  128308, 10, }, // America/Danmarkshavn
    {  421, 7833,   34247,  5, }, // America/Dawson
    {  421, 7848,   34252, 10, }, // America/Dawson_Creek
    {  421,  805,    1319,  6, }, // America/Denver
    {  421, 3465,    1325,  7, }, // America/Detroit
    {  421, 5739,    1332,  8, }, // America/Dominica
    {  421,  893,    1340,  8, }, // America/Edmonton
    {  421, 7869,    1348,  8, }, // America/Eirunepe
    {  421, 5441,  128318, 11, }, // America/El_Salvador
    {  421, 7886,    1364, 11, }, // America/Fort_Nelson
    {  421, 7906,    1375,  9, }, // America/Fortaleza
    {  421, 7924,   34262,  9, }, // America/Glace_Bay
    {  421, 6881,   34271,  7, }, // America/Goose_Bay
    {  421, 4612,   34278, 10, }, // America/Grand_Turk
    {  421, 5770,    1410,  7, }, // America/Grenada
    {  421, 5786,   34288,  9, }, // America/Guadeloupe
    {  421, 3760,    1426,  9, }, // America/Guatemala
    {  421, 4841,   34297,  8, }, // America/Guayaquil
    {  421, 6932,  128329,  7, }, // America/Guyana
    {  421, 1939,   34310,  8, }, // America/Halifax
    {  421, 2281,   34318,  6, }, // America/Havana
    {  421, 7942,  128336,  8, }, // America/Hermosillo
    {  421,  348,  128344, 13, }, // America/Indiana/Indianapolis
    {  421,  481,  128357, 14, }, // America/Indiana/Knox
    {  421, 7961,   34348, 17, }, // America/Indiana/Marengo
    {  421, 7985,  128371, 20, }, // America/Indiana/Petersburg
    {  421, 8012,   34385, 18, }, // America/Indiana/Tell_City
    {  421, 8038,  128391, 15, }, // America/Indiana/Vevay
    {  421, 8060,  128406, 16, }, // America/Indiana/Vincennes
    {  421, 8086,   34436, 17, }, // America/Indiana/Winamac
    {  421, 8110,    1570,  6, }, // America/Inuvik
    {  421,  660,    1576,  7, }, // America/Iqaluit
    {  421, 2799,   34461,  7, }, // America/Jamaica
    {  421, 5380,    1590,  4, }, // America/Juneau
    {  421,  521,  118356,  7, }, // America/Kentucky/Louisville
    {  421, 8125,   34472, 18, }, // America/Kentucky/Monticello
    {  421, 5704,  128422, 10, }, // America/Kralendijk
    {  421, 4376,   34500,  6, }, // America/La_Paz
    {  421, 7169,    1638,  4, }, // America/Lima
    {  421, 3239,  128432, 11, }, // America/Los_Angeles
    {  421, 5932,  128443, 18, }, // America/Lower_Princes
    {  421, 8153,   34536,  6, }, // America/Maceio
    {  421, 8168,   34542,  7, }, // America/Managua
    {  421, 1908,    1684,  6, }, // America/Manaus
    {  421, 5897,    1690,  6, }, // America/Marigot
    {  421, 5805,    1696,  8, }, // America/Martinique
    {  421, 8184,    1705,  9, }, // America/Matamoros
    {  421, 2917,   34549,  8, }, // America/Mazatlan
    {  421, 8202,    1722,  8, }, // America/Menominee
    {  421, 8220,    1730,  6, }, // America/Merida
    {  421, 8235,    1736, 10, }, // America/Metlakatla
    {  421, 2949,   34557, 12, }, // America/Mexico_City
    {  421, 4391,  128461,  7, }, // America/Miquelon
    {  421, 8254,    1759,  7, }, // America/Moncton
    {  421, 8270,   34569,  8, }, // America/Monterrey
    {  421, 4098,    1775, 10, }, // America/Montevideo
    {  421, 5824,   34577,  9, }, // America/Montserrat
    {  421, 5481,   34586,  5, }, // America/Nassau
    {  421, 2379,   34591,  6, }, // America/New_York
    {  421, 8288,    1809,  3, }, // America/Nome
    {  421, 1850,   34597,  6, }, // America/Noronha
    {  421, 8301,  128468, 21, }, // America/North_Dakota/Beulah
    {  421, 8329,   34625, 22, }, // America/North_Dakota/Center
    {  421, 8357,  128489, 24, }, // America/North_Dakota/New_Salem
    {  421,  393,   28283,  3, }, // America/Nuuk
    {  421, 8388,    1894,  7, }, // America/Ojinaga
    {  421, 2356,    1901,  6, }, // America/Panama
    {  421, 6173,    1907, 10, }, // America/Paramaribo
    {  421, 2973,   34673,  6, }, // America/Phoenix
    {  421, 3945,  128513, 12, }, // America/Port-au-Prince
    {  421, 5954,  128525, 13, }, // America/Port_of_Spain
    {  421, 8404,   34692, 10, }, // America/Porto_Velho
    {  421, 5843,   34702,  9, }, // America/Puerto_Rico
    {  421, 4030,   34711, 12, }, // America/Punta_Arenas
    {  421, 8424,  128538, 12, }, // America/Rankin_Inlet
    {  421, 8445,    1994,  6, }, // America/Recife
    {  421, 1995,  128550,  7, }, // America/Regina
    {  421, 8460,  118538,  7, }, // America/Resolute
    {  421,  695,   34754, 10, }, // America/Rio_Branco
    {  421, 8477,   34764,  8, }, // America/Santarem
    {  421, 2201,   34772,  9, }, // America/Santiago
    {  421, 6111,   34781, 13, }, // America/Santo_Domingo
    {  421, 1878,   34794,  9, }, // America/Sao_Paulo
    {  421, 6899,   34803, 12, }, // America/Scoresbysund
    {  421, 8494,    2074,  5, }, // America/Sitka
    {  421, 8508,  128557, 15, }, // America/St_Barthelemy
    {  421, 2061,  128572,  9, }, // America/St_Johns
    {  421, 5863,  128581, 10, }, // America/St_Kitts
    {  421, 5880,  128591, 11, }, // America/St_Lucia
    {  421,  855,  128602, 10, }, // America/St_Thomas
    {  421, 5913,   34863, 12, }, // America/St_Vincent
    {  421, 8530,   34875, 12, }, // America/Swift_Current
    {  421, 5461,    2156, 11, }, // America/Tegucigalpa
    {  421, 5756,    2167,  4, }, // America/Thule
    {  421,  313,    2171,  7, }, // America/Tijuana
    {  421,  608,    2178,  7, }, // America/Toronto
    {  421, 5688,    2185,  7, }, // America/Tortola
    {  421, 2093,    2192,  8, }, // America/Vancouver
    {  421, 2144,   34887,  8, }, // America/Whitehorse
    {  421,  734,   34895,  7, }, // America/Winnipeg
    {  421, 8552,   34902,  7, }, // America/Yakutat
    {  421, 6001,  128612,  5, }, // Antarctica/Casey
    {  421, 6094,   34914,  6, }, // Antarctica/Davis
    {  421, 6133,  128617, 12, }, // Antarctica/DumontDUrville
    {  421, 8568,  128629,  7, }, // Antarctica/Macquarie
    {  421, 7066,   34940,  5, }, // Antarctica/Mawson
    {  421, 7113,   34945,  8, }, // Antarctica/McMurdo
    {  421, 8589,    2272,  6, }, // Antarctica/Palmer
    {  421, 7226,    2278,  6, }, // Antarctica/Rothera
    {  421, 7295,  128636,  6, }, // Antarctica/Syowa
    {  421, 8607,    2289,  4, }, // Antarctica/Troll
    {  421, 7374,   34957,  6, }, // Antarctica/Vostok
    {  421, 1429,  128642, 12, }, // Arctic/Longyearbyen
    {  421, 5570,    2310,  4, }, // Asia/Aden
    {  421, 5395,   34974,  6, }, // Asia/Almaty
    {  421, 3968,   28663,  4, }, // Asia/Amman
    {  421, 5511,   34980,  6, }, // Asia/Anadyr
    {  421, 5523,    2331,  5, }, // Asia/Aqtau
    {  421, 5368,    2336,  6, }, // Asia/Aqtobe
    {  421,  964,  128654,  7, }, // Asia/Ashgabat
    {  421, 8624,   52009,  6, }, // Asia/Atyrau
    {  421, 3607,    2355,  6, }, // Asia/Baghdad
    {  421, 5534,    2361,  7, }, // Asia/Bahrain
    {  421, 3653,    2368,  4, }, // Asia/Baku
    {  421, 4466,    2372,  7, }, // Asia/Bangkok
    {  421, 3571,    2379,  7, }, // Asia/Barnaul
    {  421, 4086,   35005,  6, }, // Asia/Beirut
    {  421, 3778,    2392,  6, }, // Asia/Bishkek
    {  421, 5989,   35011,  6, }, // Asia/Brunei
    {  421, 4601,    2404,  4, }, // Asia/Chita
    {  421, 4511,    2408,  7, }, // Asia/Colombo
    {  421, 4540,    2415,  6, }, // Asia/Damascus
    {  421, 1093,     711,  4, }, // Asia/Dhaka
    {  421, 6192,    2426,  4, }, // Asia/Dili
    {  421, 3596,    2430,  5, }, // Asia/Dubai
    {  421, 6159,    2435,  7, }, // Asia/Dushanbe
    {  421, 8636,    2442,  9, }, // Asia/Famagusta
    {  421, 8651,    2451,  4, }, // Asia/Gaza
    {  421, 4795,   35018,  6, }, // Asia/Hebron
    {  421, 1259,   35024,  9, }, // Asia/Ho_Chi_Minh
    {  421, 2704,   35033,  9, }, // Asia/Hong_Kong
    {  421, 4771,    2475,  4, }, // Asia/Hovd
    {  421, 4184,    2479,  7, }, // Asia/Irkutsk
    {  421, 7006,    2486,  7, }, // Asia/Jakarta
    {  421, 6992,   35048,  8, }, // Asia/Jayapura
    {  421, 1290,  128661,  8, }, // Asia/Jerusalem
    {  421, 3560,    2510,  5, }, // Asia/Kabul
    {  421, 4316,    2529,  8, }, // Asia/Kamchatka
    {  421, 4239,    2540,  6, }, // Asia/Karachi
    {  421, 1185,    2546,  8, }, // Asia/Kathmandu
    {  421, 8661,   35065,  7, }, // Asia/Khandyga
    {  421,  992,   35072,  7, }, // Asia/Kolkata
    {  421, 4197,   35079, 11, }, // Asia/Krasnoyarsk
    {  421, 4859,   35090, 12, }, // Asia/Kuala_Lumpur
    {  421, 5976,    2592,  6, }, // Asia/Kuching
    {  421, 5547,   35102,  6, }, // Asia/Kuwait
    {  421, 1211,    2604,  5, }, // Asia/Macau
    {  421, 4017,    2609,  7, }, // Asia/Magadan
    {  421, 1349,    2616,  7, }, // Asia/Makassar
    {  421, 7182,    2623,  6, }, // Asia/Manila
    {  421, 6920,   35113,  6, }, // Asia/Muscat
    {  421, 2554,   35119,  8, }, // Asia/Nicosia
    {  421, 8675,   35127, 11, }, // Asia/Novokuznetsk
    {  421, 4135,    2653, 11, }, // Asia/Novosibirsk
    {  421, 4229,    2664,  4, }, // Asia/Omsk
    {  421, 7145,   35138,  4, }, // Asia/Oral
    {  421, 6961,   35142,  8, }, // Asia/Phnom_Penh
    {  421, 8693,  128669, 10, }, // Asia/Pontianak
    {  421, 4214,   35150,  9, }, // Asia/Pyongyang
    {  421, 5559,    2697,  5, }, // Asia/Qatar
    {  421, 8708,   35159,  8, }, // Asia/Qostanay
    {  421, 4269,   35167,  9, }, // Asia/Qyzylorda
    {  421, 3584,   35176,  5, }, // Asia/Riyadh
    {  421, 4408,  128679,  7, }, // Asia/Sakhalin
    {  421, 7245,    2739,  9, }, // Asia/Samarkand
    {  421, 3279,    2748,  4, }, // Asia/Seoul
    {  421, 1053,   35181,  6, }, // Asia/Shanghai
    {  421, 3300,    2758,  8, }, // Asia/Singapore
    {  421, 4297,   37158, 13, }, // Asia/Srednekolymsk
    {  421, 3263,   35200,  6, }, // Asia/Taipei
    {  421, 4781,    2785,  7, }, // Asia/Tashkent
    {  421, 3915,   35206,  7, }, // Asia/Tbilisi
    {  421, 2772,   35213,  7, }, // Asia/Tehran
    {  421, 1317,   35220,  5, }, // Asia/Thimphu
    {  421, 2821,    2810,  5, }, // Asia/Tokyo
    {  421, 4572,    2815,  5, }, // Asia/Tomsk
    {  421, 1021,   35225, 10, }, // Asia/Ulaanbaatar
    {  421, 1159,    2830,  6, }, // Asia/Urumqi
    {  421, 8722,    2836,  8, }, // Asia/Ust-Nera
    {  421, 6977,  128686,  9, }, // Asia/Vientiane
    {  421, 4710,    2851, 11, }, // Asia/Vladivostok
    {  421, 4828,   35252,  7, }, // Asia/Yakutsk
    {  421, 1235,   29043,  6, }, // Asia/Yangon
    {  421, 3883,    2875, 12, }, // Asia/Yekaterinburg
    {  421, 3747,    2887,  6, }, // Asia/Yerevan
    {  421, 3663,  128695, 15, }, // Atlantic/Azores
    {  421, 5671,  128710,  7, }, // Atlantic/Bermuda
    {  421, 6628,  128717, 16, }, // Atlantic/Canary
    {  421, 3727,   37237, 10, }, // Atlantic/Cape_Verde
    {  421, 1395,  128733, 14, }, // Atlantic/Faroe
    {  421, 8736,    2971,  7, }, // Atlantic/Madeira
    {  421, 2748,   35337,  9, }, // Atlantic/Reykjavik
    {  421, 7272,   35346, 13, }, // Atlantic/South_Georgia
    {  421, 6821,   37292, 11, }, // Atlantic/St_Helena
    {  421, 6644,    3029,  6, }, // Atlantic/Stanley
    {  421, 1670,    3035,  8, }, // Australia/Adelaide
    {  421, 1635,   35369,  8, }, // Australia/Brisbane
    {  421, 1799,   35377, 10, }, // Australia/Broken_Hill
    {  421, 1583,    3061,  6, }, // Australia/Darwin
    {  421, 3637,  128747,  5, }, // Australia/Eucla
    {  421, 1516,    3072,  6, }, // Australia/Hobart
    {  421, 8753,    3078,  8, }, // Australia/Lindeman
    {  421, 1547,   37322,  8, }, // Australia/Lord_Howe
    {  421, 1727,    3094,  7, }, // Australia/Melbourne
    {  421, 1762,    3101,  4, }, // Australia/Perth
    {  421, 1463,   35395,  6, }, // Australia/Sydney
    {  421, 6441,    3111,  9, }, // Europe/Amsterdam
    {  421, 6216,   29275,  6, }, // Europe/Andorra
    {  421, 3620,    3127,  8, }, // Europe/Astrakhan
    {  421, 2300,   35401,  5, }, // Europe/Athens
    {  421, 6336,    3139,  7, }, // Europe/Belgrade
    {  421, 4757,    3146,  6, }, // Europe/Berlin
    {  421, 6488,    3152, 10, }, // Europe/Bratislava
    {  421, 2167,   35413,  6, }, // Europe/Brussels
    {  421, 3928,   35419,  8, }, // Europe/Bucharest
    {  421, 3806,   35427, 10, }, // Europe/Budapest
    {  421, 8772,  128752,  8, }, // Europe/Busingen
    {  421, 2583,  128760,  7, }, // Europe/Chisinau
    {  421, 6289,   35452, 10, }, // Europe/Copenhagen
    {  421, 2338,   35462,  6, }, // Europe/Dublin
    {  421, 6307,    3232,  9, }, // Europe/Gibraltar
    {  421, 8788,   35468,  6, }, // Europe/Guernsey
    {  421, 6612,    3247,  8, }, // Europe/Helsinki
    {  421, 8804,  128767, 10, }, // Europe/Isle_of_Man
    {  421, 1130,   35484,  8, }, // Europe/Istanbul
    {  421, 8823,  128777,  5, }, // Europe/Jersey
    {  421, 3979,    3276, 11, }, // Europe/Kaliningrad
    {  421, 8929,    3287,  5, }, // Europe/Kirov
    {  421, 2527,    3292,  4, }, // Europe/Kyiv
    {  421, 3213,   35508,  7, }, // Europe/Lisbon
    {  421, 6506,   35515,  7, }, // Europe/Ljubljana
    {  421, 2501,    3313,  6, }, // Europe/London
    {  421, 6365,   35522, 10, }, // Europe/Luxembourg
    {  421, 4892,    3330,  6, }, // Europe/Madrid
    {  421, 6397,    3336,  5, }, // Europe/Malta
    {  421, 6582,  128782, 11, }, // Europe/Mariehamn
    {  421, 3693,    3350,  5, }, // Europe/Minsk
    {  421, 6410,    3355,  6, }, // Europe/Monaco
    {  421, 3537,    3361,  6, }, // Europe/Moscow
    {  421, 6458,    3367,  4, }, // Europe/Oslo
    {  421, 4284,   35540,  5, }, // Europe/Paris
    {  421, 6424,    3376,  9, }, // Europe/Podgorica
    {  421, 6275,    3385,  5, }, // Europe/Prague
    {  421, 8837,    3390,  4, }, // Europe/Riga
    {  421, 6324,    3394,  3, }, // Europe/Rome
    {  421, 4331,    3397,  6, }, // Europe/Samara
    {  421, 6470,   35545, 10, }, // Europe/San_Marino
    {  421, 6245,    3413,  7, }, // Europe/Sarajevo
    {  421, 4451,    3420,  7, }, // Europe/Saratov
    {  421, 8849,    3427, 10, }, // Europe/Simferopol
    {  421, 6383,  128793,  6, }, // Europe/Skopje
    {  421, 6599,   35579,  6, }, // Europe/Sofia
    {  421, 6523,   35585,  8, }, // Europe/Stockholm
    {  421, 8867,   35593,  5, }, // Europe/Tallinn
    {  421, 6202,    3461,  6, }, // Europe/Tirane
    {  421, 8882,  128799,  9, }, // Europe/Ulyanovsk
    {  421, 6352,    3476,  5, }, // Europe/Vaduz
    {  421, 6567,    3481,  7, }, // Europe/Vatican
    {  421, 6231,   37431,  5, }, // Europe/Vienna
    {  421, 8899,  128808,  6, }, // Europe/Vilnius
    {  421, 4727,    3499,  9, }, // Europe/Volgograd
    {  421, 3190,    3508,  7, }, // Europe/Warsaw
    {  421, 6261,    3515,  6, }, // Europe/Zagreb
    {  421, 6540,   35620,  5, }, // Europe/Zurich
    {  421, 5078,   35625, 12, }, // Indian/Antananarivo
    {  421, 6947,    3539,  5, }, // Indian/Chagos
    {  421, 6046,  128814, 14, }, // Indian/Christmas
    {  421, 6063,  128828, 16, }, // Indian/Cocos
    {  421, 5029,   52446,  6, }, // Indian/Comoro
    {  421, 6661,    3578,  8, }, // Indian/Kerguelen
    {  421, 7260,   35648,  4, }, // Indian/Mahe
    {  421, 7050,   35652,  7, }, // Indian/Maldives
    {  421, 4069,  128844,  9, }, // Indian/Mauritius
    {  421, 5098,   35669,  5, }, // Indian/Mayotte
    {  421, 7211,  128853,  8, }, // Indian/Reunion
    {  421, 4422,   35681,  5, }, // Pacific/Apia
    {  421,  932,    3620,  6, }, // Pacific/Auckland
    {  421, 3706,  128861,  8, }, // Pacific/Bougainville
    {  421, 3015,   37512,  5, }, // Pacific/Chatham
    {  421, 3157,  128869,  3, }, // Pacific/Chuuk
    {  421, 2237,  128872, 18, }, // Pacific/Easter
    {  421, 7360,    3654,  5, }, // Pacific/Efate
    {  421, 7327,    3659,  7, }, // Pacific/Fakaofo
    {  421, 3902,    3666,  4, }, // Pacific/Fiji
    {  421, 7343,    3670,  8, }, // Pacific/Funafuti
    {  421, 6678,    3678,  9, }, // Pacific/Galapagos
    {  421, 6696,   35712,  7, }, // Pacific/Gambier
    {  421, 3822,   35719, 11, }, // Pacific/Guadalcanal
    {  421, 6018,  128890,  4, }, // Pacific/Guam
    {  421, 2723,   35730,  8, }, // Pacific/Honolulu
    {  421, 3049,   35738,  9, }, // Pacific/Kanton
    {  421, 3998,    3734, 10, }, // Pacific/Kiritimati
    {  421, 7019,    3744,  6, }, // Pacific/Kosrae
    {  421, 2842,  128894,  9, }, // Pacific/Kwajalein
    {  421, 4877,    3759,  6, }, // Pacific/Majuro
    {  421, 4051,  128903, 16, }, // Pacific/Marquesas
    {  421, 8914,   35768,  6, }, // Pacific/Midway
    {  421, 7084,    3798,  5, }, // Pacific/Nauru
    {  421, 7132,    3803,  4, }, // Pacific/Niue
    {  421, 4168,    3807,  7, }, // Pacific/Norfolk
    {  421, 7098,    3814,  5, }, // Pacific/Noumea
    {  421, 3126,   35774,  9, }, // Pacific/Pago_Pago
    {  421, 7155,    3828,  5, }, // Pacific/Palau
    {  421, 7194,  128919, 18, }, // Pacific/Pitcairn
    {  421, 3096,  128937,  6, }, // Pacific/Pohnpei
    {  421, 4807,   37609, 11, }, // Pacific/Port_Moresby
    {  421, 6076,    3857,  9, }, // Pacific/Rarotonga
    {  421, 6031,   35800,  6, }, // Pacific/Saipan
    {  421, 7312,   35806,  6, }, // Pacific/Tahiti
    {  421, 6712,    3877,  6, }, // Pacific/Tarawa
    {  421, 4583,    3883,  9, }, // Pacific/Tongatapu
    {  421, 7392,   35812,  4, }, // Pacific/Wake
    {  421, 7405,   35816,  5, }, // Pacific/Wallis
    {  422,   14,       0,  6, }, // Africa/Asmara Machame/Latin/Tanzania
    {  422, 4435,       6,  8, }, // Africa/Sao_Tome
    {  422, 4252,      14,  8, }, // America/Asuncion
    {  422,  237,      22,  8, }, // America/Atikokan
    {  422, 7623,      30, 17, }, // America/Bahia_Banderas
    {  422, 3868,      47,  6, }, // America/Cancun
    {  422, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  422, 5723,      66,  7, }, // America/Curacao
    {  422,  481,      73, 13, }, // America/Indiana/Knox
    {  422, 7961,      86, 16, }, // America/Indiana/Marengo
    {  422, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  422, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  422, 8038,     139, 14, }, // America/Indiana/Vevay
    {  422, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  422, 8086,     171, 16, }, // America/Indiana/Winamac
    {  422, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  422, 5932,     207, 22, }, // America/Lower_Princes
    {  422, 8220,     229,  6, }, // America/Merida
    {  422, 1850,     235, 19, }, // America/Noronha
    {  422, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  422, 8329,     274, 20, }, // America/North_Dakota/Center
    {  422, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  422,  393,     317,  4, }, // America/Nuuk
    {  422, 6899,     321, 16, }, // America/Scoresbysund
    {  422, 8508,     337, 14, }, // America/St_Barthelemy
    {  422, 2061,     351, 10, }, // America/St_Johns
    {  422, 5863,     361,  9, }, // America/St_Kitts
    {  422, 5880,     370,  9, }, // America/St_Lucia
    {  422,  855,     379, 10, }, // America/St_Thomas
    {  422, 5913,     389, 11, }, // America/St_Vincent
    {  422, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  422, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  422, 1185,     448,  9, }, // Asia/Kathmandu
    {  422,  992,     457,  7, }, // Asia/Kolkata
    {  422, 1211,     464,  5, }, // Asia/Macau
    {  422, 1235,     477,  6, }, // Asia/Yangon
    {  422, 1395,     483,  5, }, // Atlantic/Faroe
    {  422, 6821,     488, 10, }, // Atlantic/St_Helena
    {  422, 2527,     514,  4, }, // Europe/Kyiv
    {  422, 7211,     547,  7, }, // Indian/Reunion
    {  422, 3157,     554,  5, }, // Pacific/Chuuk
    {  422, 3096,     586,  7, }, // Pacific/Pohnpei
    {  423, 6788,  128943,  6, }, // Africa/Abidjan Maithili/Devanagari/India
    {  423, 6760,  128949,  4, }, // Africa/Accra
    {  423, 5059,  128953, 10, }, // Africa/Addis_Ababa
    {  423, 7420,   88267,  9, }, // Africa/Algiers
    {  423,   14,  128963,  6, }, // Africa/Asmara
    {  423,   44,  128969,  5, }, // Africa/Bamako
    {  423, 5257,  128974,  5, }, // Africa/Bangui
    {  423, 6746,  128979,  5, }, // Africa/Banjul
    {  423, 7435,   88301,  5, }, // Africa/Bissau
    {  423, 4957,  128984,  9, }, // Africa/Blantyre
    {  423, 5287,  128993,  9, }, // Africa/Brazzaville
    {  423, 4922,  113451,  9, }, // Africa/Bujumbura
    {  423, 2320,  129002,  6, }, // Africa/Cairo
    {  423, 4117,   30065, 11, }, // Africa/Casablanca
    {  423, 7449,  129008,  5, }, // Africa/Ceuta
    {  423, 6773,   88345,  8, }, // Africa/Conakry
    {  423, 6840,   88353,  4, }, // Africa/Dakar
    {  423, 5130,  129013,  9, }, // Africa/Dar_es_Salaam
    {  423, 5043,   88368,  6, }, // Africa/Djibouti
    {  423, 5243,  129022,  5, }, // Africa/Douala
    {  423, 5166,  129027,  7, }, // Africa/El_Aaiun
    {  423, 6853,   30122,  8, }, // Africa/Freetown
    {  423, 4906,  129034,  7, }, // Africa/Gaborone
    {  423, 5015,   88403,  5, }, // Africa/Harare
    {  423, 4479,  129041, 11, }, // Africa/Johannesburg
    {  423, 4499,   30154,  4, }, // Africa/Juba
    {  423, 5151,  129052,  6, }, // Africa/Kampala
    {  423, 4524,  129058,  6, }, // Africa/Khartoum
    {  423, 4973,   88424,  6, }, // Africa/Kigali
    {  423, 5306,   88430,  7, }, // Africa/Kinshasa
    {  423, 4744,   88437,  5, }, // Africa/Lagos
    {  423, 5336,   88442,  9, }, // Africa/Libreville
    {  423, 6869,   88452,  3, }, // Africa/Lome
    {  423, 5211,   88455,  6, }, // Africa/Luanda
    {  423, 4939,   88461,  9, }, // Africa/Lubumbashi
    {  423, 5001,   30221,  6, }, // Africa/Lusaka
    {  423, 5322,   88470,  5, }, // Africa/Malabo
    {  423, 4987,  129064,  5, }, // Africa/Maputo
    {  423, 5197,  129069,  5, }, // Africa/Maseru
    {  423, 5182,  129074,  9, }, // Africa/Mbabane
    {  423, 5113,  113529,  8, }, // Africa/Mogadishu
    {  423, 7034,   88496,  9, }, // Africa/Monrovia
    {  423, 3853,   88505,  6, }, // Africa/Nairobi
    {  423, 5271,  129083,  8, }, // Africa/Ndjamena
    {  423, 5354,  129091,  7, }, // Africa/Niamey
    {  423, 6803,  129098,  6, }, // Africa/Nouakchott
    {  423, 6727,  129104,  7, }, // Africa/Ouagadougou
    {  423, 5225,   88541, 11, }, // Africa/Porto-Novo
    {  423, 4435,   88552,  7, }, // Africa/Sao_Tome
    {  423, 2866,   88559,  8, }, // Africa/Tripoli
    {  423, 6554,   30328,  7, }, // Africa/Tunis
    {  423, 4152,  129111,  7, }, // Africa/Windhoek
    {  423,  132,  129118,  3, }, // America/Adak
    {  423, 3346,  129121,  7, }, // America/Anchorage
    {  423, 5607,   88577,  8, }, // America/Anguilla
    {  423, 5624,  129128,  7, }, // America/Antigua
    {  423, 4554,  129135,  8, }, // America/Araguaina
    {  423,  166,  113601, 12, }, // America/Argentina/Buenos_Aires
    {  423,   91,   30390, 10, }, // America/Argentina/Catamarca
    {  423,  270,   88612,  8, }, // America/Argentina/Cordoba
    {  423,  441,  129143,  5, }, // America/Argentina/Jujuy
    {  423, 7462,  113634,  9, }, // America/Argentina/La_Rioja
    {  423,  565,  129148,  7, }, // America/Argentina/Mendoza
    {  423, 7489,  129155, 10, }, // America/Argentina/Rio_Gallegos
    {  423, 7520,  129165,  5, }, // America/Argentina/Salta
    {  423, 7544,  129170,  7, }, // America/Argentina/San_Juan
    {  423, 5580,  129177,  7, }, // America/Argentina/San_Luis
    {  423, 7571,   30460,  7, }, // America/Argentina/Tucuman
    {  423, 7597,  129184,  7, }, // America/Argentina/Ushuaia
    {  423, 5640,   88682,  5, }, // America/Aruba
    {  423, 4252,  129191,  8, }, // America/Asuncion
    {  423,  237,  129199,  7, }, // America/Atikokan
    {  423, 3679,   88703,  5, }, // America/Bahia
    {  423, 7623,  129206, 13, }, // America/Bahia_Banderas
    {  423, 5654,  129219,  7, }, // America/Barbados
    {  423, 7646,  129226,  5, }, // America/Belem
    {  423, 5407,   30531,  5, }, // America/Belize
    {  423, 7660,  129231, 12, }, // America/Blanc-Sablon
    {  423, 7681,  129243, 10, }, // America/Boa_Vista
    {  423, 4361,   88760,  6, }, // America/Bogota
    {  423, 7699,  129253,  5, }, // America/Boise
    {  423, 7713,  129258, 12, }, // America/Cambridge_Bay
    {  423, 7735,  129270, 14, }, // America/Campo_Grande
    {  423, 3868,  113766,  6, }, // America/Cancun
    {  423, 4694,   30602,  7, }, // America/Caracas
    {  423, 4345,   30609,  5, }, // America/Cayenne
    {  423, 5496,   30614,  5, }, // America/Cayman
    {  423, 2260,   88811,  6, }, // America/Chicago
    {  423, 7756,   88817,  8, }, // America/Chihuahua
    {  423, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  423, 5422,   88841, 11, }, // America/Costa_Rica
    {  423, 7796,   30642,  8, }, // America/Creston
    {  423, 3791,  129284,  6, }, // America/Cuiaba
    {  423, 5723,  129290,  9, }, // America/Curacao
    {  423, 7812,  129299,  7, }, // America/Danmarkshavn
    {  423, 7833,  129306,  5, }, // America/Dawson
    {  423, 7848,  129311, 11, }, // America/Dawson_Creek
    {  423,  805,  129322,  5, }, // America/Denver
    {  423, 3465,  129327,  8, }, // America/Detroit
    {  423, 5739,   88904,  8, }, // America/Dominica
    {  423,  893,  129335,  8, }, // America/Edmonton
    {  423, 7869,  129343,  7, }, // America/Eirunepe
    {  423, 5441,   88918, 11, }, // America/El_Salvador
    {  423, 7886,  129350, 11, }, // America/Fort_Nelson
    {  423, 7906,  129361, 10, }, // America/Fortaleza
    {  423, 7924,   30760,  8, }, // America/Glace_Bay
    {  423, 6881,   30769,  6, }, // America/Goose_Bay
    {  423, 4612,   88974, 11, }, // America/Grand_Turk
    {  423, 5770,   30789,  8, }, // America/Grenada
    {  423, 5786,  129371,  9, }, // America/Guadeloupe
    {  423, 3760,   30806, 10, }, // America/Guatemala
    {  423, 4841,  129380, 10, }, // America/Guayaquil
    {  423, 6932,   30824,  6, }, // America/Guyana
    {  423, 1939,   30830,  9, }, // America/Halifax
    {  423, 2281,   89018,  5, }, // America/Havana
    {  423, 7942,  129390,  8, }, // America/Hermosillo
    {  423,  348,   89034, 13, }, // America/Indiana/Indianapolis
    {  423,  481,  129398, 15, }, // America/Indiana/Knox
    {  423, 7961,  129413, 17, }, // America/Indiana/Marengo
    {  423, 7985,   89079, 20, }, // America/Indiana/Petersburg
    {  423, 8012,   89099, 18, }, // America/Indiana/Tell_City
    {  423, 8038,  129430, 14, }, // America/Indiana/Vevay
    {  423, 8060,  129444, 17, }, // America/Indiana/Vincennes
    {  423, 8086,  129461, 17, }, // America/Indiana/Winamac
    {  423, 8110,  129478,  6, }, // America/Inuvik
    {  423,  660,  129484,  7, }, // America/Iqaluit
    {  423, 2799,   89173,  5, }, // America/Jamaica
    {  423, 5380,  129491,  5, }, // America/Juneau
    {  423,  521,   89185,  8, }, // America/Kentucky/Louisville
    {  423, 8125,  129496, 18, }, // America/Kentucky/Monticello
    {  423, 5704,  129514, 10, }, // America/Kralendijk
    {  423, 4376,   31058,  6, }, // America/La_Paz
    {  423, 7169,   31064,  4, }, // America/Lima
    {  423, 3239,  129524, 11, }, // America/Los_Angeles
    {  423, 5932,  129535, 20, }, // America/Lower_Princes
    {  423, 8153,  129555,  5, }, // America/Maceio
    {  423, 8168,   31109,  6, }, // America/Managua
    {  423, 1908,   89274,  4, }, // America/Manaus
    {  423, 5897,  129560,  7, }, // America/Marigot
    {  423, 5805,   89285,  9, }, // America/Martinique
    {  423, 8184,   89294,  9, }, // America/Matamoros
    {  423, 2917,  129567,  7, }, // America/Mazatlan
    {  423, 8202,  129574,  8, }, // America/Menominee
    {  423, 8220,  129582,  6, }, // America/Merida
    {  423, 8235,  129588, 10, }, // America/Metlakatla
    {  423, 2949,  129598, 13, }, // America/Mexico_City
    {  423, 4391,  129611,  9, }, // America/Miquelon
    {  423, 8254,  129620,  8, }, // America/Moncton
    {  423, 8270,  129628,  8, }, // America/Monterrey
    {  423, 4098,  129636, 11, }, // America/Montevideo
    {  423, 5824,   89378,  9, }, // America/Montserrat
    {  423, 5481,  129647,  4, }, // America/Nassau
    {  423, 2379,  129651,  9, }, // America/New_York
    {  423, 8288,   89314,  3, }, // America/Nome
    {  423, 1850,   89401,  8, }, // America/Noronha
    {  423, 8301,  129660, 19, }, // America/North_Dakota/Beulah
    {  423, 8329,  129679, 18, }, // America/North_Dakota/Center
    {  423, 8357,  129697, 22, }, // America/North_Dakota/New_Salem
    {  423,  393,   89472,  3, }, // America/Nuuk
    {  423, 8388,  114339,  7, }, // America/Ojinaga
    {  423, 2356,   89484,  5, }, // America/Panama
    {  423, 6173,   89489, 10, }, // America/Paramaribo
    {  423, 2973,  114355,  7, }, // America/Phoenix
    {  423, 3945,  129719, 15, }, // America/Port-au-Prince
    {  423, 5954,  129734, 14, }, // America/Port_of_Spain
    {  423, 8404,  129748, 13, }, // America/Porto_Velho
    {  423, 5843,   89549, 11, }, // America/Puerto_Rico
    {  423, 4030,  129761, 12, }, // America/Punta_Arenas
    {  423, 8424,   89572, 12, }, // America/Rankin_Inlet
    {  423, 8445,  129773,  5, }, // America/Recife
    {  423, 1995,  129778,  6, }, // America/Regina
    {  423, 8460,  129784,  7, }, // America/Resolute
    {  423,  695,  129791, 12, }, // America/Rio_Branco
    {  423, 8477,  114445,  8, }, // America/Santarem
    {  423, 2201,   89620,  9, }, // America/Santiago
    {  423, 6111,  129803, 13, }, // America/Santo_Domingo
    {  423, 1878,   89642,  9, }, // America/Sao_Paulo
    {  423, 6899,  129816, 13, }, // America/Scoresbysund
    {  423, 8494,  129829,  5, }, // America/Sitka
    {  423, 8508,  129834, 13, }, // America/St_Barthelemy
    {  423, 2061,  129847, 10, }, // America/St_Johns
    {  423, 5863,   89694, 10, }, // America/St_Kitts
    {  423, 5880,  129857, 11, }, // America/St_Lucia
    {  423,  855,  129868,  9, }, // America/St_Thomas
    {  423, 5913,   89724, 12, }, // America/St_Vincent
    {  423, 8530,  129877, 13, }, // America/Swift_Current
    {  423, 5461,   31605, 12, }, // America/Tegucigalpa
    {  423, 5756,  129890,  4, }, // America/Thule
    {  423,  313,   89766,  7, }, // America/Tijuana
    {  423,  608,   89773,  6, }, // America/Toronto
    {  423, 5688,   89779,  8, }, // America/Tortola
    {  423, 2093,  129894,  7, }, // America/Vancouver
    {  423, 2144,  129901,  9, }, // America/Whitehorse
    {  423,  734,  129910,  7, }, // America/Winnipeg
    {  423, 8552,  129917,  6, }, // America/Yakutat
    {  423, 6001,   31673,  4, }, // Antarctica/Casey
    {  423, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  423, 8568,  129923,  8, }, // Antarctica/Macquarie
    {  423, 1429,  129931, 11, }, // Arctic/Longyearbyen
    {  423, 5570,  129942,  3, }, // Asia/Aden
    {  423, 5395,  129945,  7, }, // Asia/Almaty
    {  423, 3968,   89915,  6, }, // Asia/Amman
    {  423, 5511,  129952,  6, }, // Asia/Anadyr
    {  423, 5523,   31778,  6, }, // Asia/Aqtau
    {  423, 5368,   89927,  6, }, // Asia/Aqtobe
    {  423,  964,   31791,  8, }, // Asia/Ashgabat
    {  423, 8624,  129958,  6, }, // Asia/Atyrau
    {  423, 3607,   89938,  5, }, // Asia/Baghdad
    {  423, 5534,   89943,  5, }, // Asia/Bahrain
    {  423, 3653,   30654,  4, }, // Asia/Baku
    {  423, 4466,  129964,  6, }, // Asia/Bangkok
    {  423, 3571,   89954,  6, }, // Asia/Barnaul
    {  423, 4086,  114707,  5, }, // Asia/Beirut
    {  423, 3778,   31836,  7, }, // Asia/Bishkek
    {  423, 5989,   89965,  7, }, // Asia/Brunei
    {  423, 4601,  129970,  4, }, // Asia/Chita
    {  423, 4511,  129974,  7, }, // Asia/Colombo
    {  423, 4540,  129981,  6, }, // Asia/Damascus
    {  423, 1093,   89991,  4, }, // Asia/Dhaka
    {  423, 6192,   31872,  4, }, // Asia/Dili
    {  423, 3596,   89999,  4, }, // Asia/Dubai
    {  423, 6159,   90003,  7, }, // Asia/Dushanbe
    {  423, 8636,   31889, 10, }, // Asia/Famagusta
    {  423, 8651,   31899,  4, }, // Asia/Gaza
    {  423, 4795,  129987,  7, }, // Asia/Hebron
    {  423, 1259,   90033, 16, }, // Asia/Ho_Chi_Minh
    {  423, 2704,  129994,  9, }, // Asia/Hong_Kong
    {  423, 4771,  130003,  5, }, // Asia/Hovd
    {  423, 4184,  130008,  7, }, // Asia/Irkutsk
    {  423, 7006,   90073,  7, }, // Asia/Jakarta
    {  423, 6992,   31943,  7, }, // Asia/Jayapura
    {  423, 1290,  130015,  8, }, // Asia/Jerusalem
    {  423, 3560,   90086,  5, }, // Asia/Kabul
    {  423, 4316,  130023,  6, }, // Asia/Kamchatka
    {  423, 4239,   31972,  5, }, // Asia/Karachi
    {  423, 1185,   90098,  8, }, // Asia/Kathmandu
    {  423, 8661,  130029,  8, }, // Asia/Khandyga
    {  423,  992,   90113,  7, }, // Asia/Kolkata
    {  423, 4197,  130037, 15, }, // Asia/Krasnoyarsk
    {  423, 4859,  130052, 11, }, // Asia/Kuala_Lumpur
    {  423, 5976,  130063,  6, }, // Asia/Kuching
    {  423, 5547,   32034,  5, }, // Asia/Kuwait
    {  423, 1211,   90145,  4, }, // Asia/Macau
    {  423, 4017,  114825,  6, }, // Asia/Magadan
    {  423, 1349,  130069,  6, }, // Asia/Makassar
    {  423, 7182,   32060,  5, }, // Asia/Manila
    {  423, 6920,   90155,  5, }, // Asia/Muscat
    {  423, 2554,  130075,  8, }, // Asia/Nicosia
    {  423, 8675,  130083, 12, }, // Asia/Novokuznetsk
    {  423, 4135,  130095, 12, }, // Asia/Novosibirsk
    {  423, 4229,  130107,  5, }, // Asia/Omsk
    {  423, 7145,  130112,  4, }, // Asia/Oral
    {  423, 6961,  114860,  9, }, // Asia/Phnom_Penh
    {  423, 8693,  130116,  9, }, // Asia/Pontianak
    {  423, 4214,   90224, 10, }, // Asia/Pyongyang
    {  423, 5559,   90234,  3, }, // Asia/Qatar
    {  423, 8708,  130125,  8, }, // Asia/Qostanay
    {  423, 4269,  130133, 10, }, // Asia/Qyzylorda
    {  423, 3584,   90256,  5, }, // Asia/Riyadh
    {  423, 4408,  130143,  6, }, // Asia/Sakhalin
    {  423, 7245,   90267,  6, }, // Asia/Samarkand
    {  423, 3279,  130149,  5, }, // Asia/Seoul
    {  423, 1053,   32184,  6, }, // Asia/Shanghai
    {  423, 3300,   32190,  8, }, // Asia/Singapore
    {  423, 4297,  130154, 16, }, // Asia/Srednekolymsk
    {  423, 3263,  130170,  6, }, // Asia/Taipei
    {  423, 4781,   90304,  6, }, // Asia/Tashkent
    {  423, 3915,  130176,  7, }, // Asia/Tbilisi
    {  423, 2772,   32236,  6, }, // Asia/Tehran
    {  423, 1317,  130183,  6, }, // Asia/Thimphu
    {  423, 2821,   90322,  6, }, // Asia/Tokyo
    {  423, 4572,  130189,  5, }, // Asia/Tomsk
    {  423, 1021,  130194,  7, }, // Asia/Ulaanbaatar
    {  423, 1159,  130201,  6, }, // Asia/Urumqi
    {  423, 8722,  130207,  9, }, // Asia/Ust-Nera
    {  423, 6977,  130216,  9, }, // Asia/Vientiane
    {  423, 4710,  130225, 13, }, // Asia/Vladivostok
    {  423, 4828,  130238,  9, }, // Asia/Yakutsk
    {  423, 1235,  130247,  6, }, // Asia/Yangon
    {  423, 3883,  130253, 13, }, // Asia/Yekaterinburg
    {  423, 3747,  130266,  6, }, // Asia/Yerevan
    {  423, 3663,  130272,  6, }, // Atlantic/Azores
    {  423, 5671,   90409,  6, }, // Atlantic/Bermuda
    {  423, 6628,   32355,  6, }, // Atlantic/Canary
    {  423, 3727,  130278,  8, }, // Atlantic/Cape_Verde
    {  423, 1395,  130286,  4, }, // Atlantic/Faroe
    {  423, 8736,  130290,  5, }, // Atlantic/Madeira
    {  423, 2748,  130295,  8, }, // Atlantic/Reykjavik
    {  423, 7272,  130303, 15, }, // Atlantic/South_Georgia
    {  423, 6821,   90463, 11, }, // Atlantic/St_Helena
    {  423, 6644,   32413,  7, }, // Atlantic/Stanley
    {  423, 1670,   90481,  6, }, // Australia/Adelaide
    {  423, 1635,  130318,  8, }, // Australia/Brisbane
    {  423, 1799,  130326, 11, }, // Australia/Broken_Hill
    {  423, 1583,  130337,  7, }, // Australia/Darwin
    {  423, 3637,  130344,  6, }, // Australia/Eucla
    {  423, 1516,  130350,  6, }, // Australia/Hobart
    {  423, 8753,  130356,  7, }, // Australia/Lindeman
    {  423, 1547,  130363, 10, }, // Australia/Lord_Howe
    {  423, 1727,   90537,  8, }, // Australia/Melbourne
    {  423, 1762,   90545,  4, }, // Australia/Perth
    {  423, 1463,   32494,  5, }, // Australia/Sydney
    {  423, 6441,  130373,  9, }, // Europe/Amsterdam
    {  423, 6216,  130382,  6, }, // Europe/Andorra
    {  423, 3620,  130388, 10, }, // Europe/Astrakhan
    {  423, 2300,   90575,  5, }, // Europe/Athens
    {  423, 6336,   32532,  8, }, // Europe/Belgrade
    {  423, 4757,   90580,  6, }, // Europe/Berlin
    {  423, 6488,  130398, 11, }, // Europe/Bratislava
    {  423, 2167,  130409,  8, }, // Europe/Brussels
    {  423, 3928,  115164,  9, }, // Europe/Bucharest
    {  423, 3806,   90617,  9, }, // Europe/Budapest
    {  423, 8772,  130417,  7, }, // Europe/Busingen
    {  423, 2583,  130424,  7, }, // Europe/Chisinau
    {  423, 6289,  130431, 10, }, // Europe/Copenhagen
    {  423, 2338,   90652,  5, }, // Europe/Dublin
    {  423, 6307,   90657, 10, }, // Europe/Gibraltar
    {  423, 8788,  130441,  6, }, // Europe/Guernsey
    {  423, 6612,  130447,  8, }, // Europe/Helsinki
    {  423, 8804,  130455, 11, }, // Europe/Isle_of_Man
    {  423, 1130,  130466, 10, }, // Europe/Istanbul
    {  423, 8823,   90701,  5, }, // Europe/Jersey
    {  423, 3979,  130476, 11, }, // Europe/Kaliningrad
    {  423, 8929,   90718,  5, }, // Europe/Kirov
    {  423, 2527,   90723,  3, }, // Europe/Kyiv
    {  423, 3213,  130487,  5, }, // Europe/Lisbon
    {  423, 6506,  130492,  9, }, // Europe/Ljubljana
    {  423, 2501,   90737,  4, }, // Europe/London
    {  423, 6365,  130501,  9, }, // Europe/Luxembourg
    {  423, 4892,  130510,  7, }, // Europe/Madrid
    {  423, 6397,  130517,  5, }, // Europe/Malta
    {  423, 6582,  130522,  8, }, // Europe/Mariehamn
    {  423, 3693,  130530,  6, }, // Europe/Minsk
    {  423, 6410,  130536,  6, }, // Europe/Monaco
    {  423, 3537,  130542,  6, }, // Europe/Moscow
    {  423, 6458,   90785,  5, }, // Europe/Oslo
    {  423, 4284,   32766,  5, }, // Europe/Paris
    {  423, 6424,  130548,  9, }, // Europe/Podgorica
    {  423, 6275,   90800,  5, }, // Europe/Prague
    {  423, 8837,   32784,  4, }, // Europe/Riga
    {  423, 6324,   90432,  3, }, // Europe/Rome
    {  423, 4331,   90809,  5, }, // Europe/Samara
    {  423, 6470,  130557,  9, }, // Europe/San_Marino
    {  423, 6245,  130566,  7, }, // Europe/Sarajevo
    {  423, 4451,  130573,  7, }, // Europe/Saratov
    {  423, 8849,  130580, 11, }, // Europe/Simferopol
    {  423, 6383,  130591,  7, }, // Europe/Skopje
    {  423, 6599,  115326,  6, }, // Europe/Sofia
    {  423, 6523,  130598,  8, }, // Europe/Stockholm
    {  423, 8867,  130606,  5, }, // Europe/Tallinn
    {  423, 6202,  130611,  6, }, // Europe/Tirane
    {  423, 8882,  130617, 12, }, // Europe/Ulyanovsk
    {  423, 6352,  130629,  5, }, // Europe/Vaduz
    {  423, 6567,  130634,  6, }, // Europe/Vatican
    {  423, 6231,  130640,  5, }, // Europe/Vienna
    {  423, 8899,  130645,  7, }, // Europe/Vilnius
    {  423, 4727,  130652, 10, }, // Europe/Volgograd
    {  423, 3190,  130662,  5, }, // Europe/Warsaw
    {  423, 6261,  130667,  5, }, // Europe/Zagreb
    {  423, 6540,  130672,  7, }, // Europe/Zurich
    {  423, 5078,  130679, 12, }, // Indian/Antananarivo
    {  423, 6947,  130691,  4, }, // Indian/Chagos
    {  423, 6046,   90971,  7, }, // Indian/Christmas
    {  423, 6063,   90978,  5, }, // Indian/Cocos
    {  423, 5029,   90983,  6, }, // Indian/Comoro
    {  423, 6661,  130695,  9, }, // Indian/Kerguelen
    {  423, 7260,   32983,  4, }, // Indian/Mahe
    {  423, 7050,   90996,  6, }, // Indian/Maldives
    {  423, 4069,  130704,  6, }, // Indian/Mauritius
    {  423, 5098,  130710,  5, }, // Indian/Mayotte
    {  423, 7211,  130715,  8, }, // Indian/Reunion
    {  423,  932,  130723,  6, }, // Pacific/Auckland
    {  423, 3015,  130729,  4, }, // Pacific/Chatham
    {  423, 3157,     554,  5, }, // Pacific/Chuuk
    {  423, 2237,   91049,  5, }, // Pacific/Easter
    {  423, 6678,  130733,  9, }, // Pacific/Galapagos
    {  423, 4168,  130742,  6, }, // Pacific/Norfolk
    {  423, 3096,     586,  7, }, // Pacific/Pohnpei
    {  424,   14,       0,  6, }, // Africa/Asmara Makhuwa-Meetto/Latin/Mozambique
    {  424, 4435,       6,  8, }, // Africa/Sao_Tome
    {  424, 4252,      14,  8, }, // America/Asuncion
    {  424,  237,      22,  8, }, // America/Atikokan
    {  424, 7623,      30, 17, }, // America/Bahia_Banderas
    {  424, 3868,      47,  6, }, // America/Cancun
    {  424, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  424, 5723,      66,  7, }, // America/Curacao
    {  424,  481,      73, 13, }, // America/Indiana/Knox
    {  424, 7961,      86, 16, }, // America/Indiana/Marengo
    {  424, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  424, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  424, 8038,     139, 14, }, // America/Indiana/Vevay
    {  424, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  424, 8086,     171, 16, }, // America/Indiana/Winamac
    {  424, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  424, 5932,     207, 22, }, // America/Lower_Princes
    {  424, 8220,     229,  6, }, // America/Merida
    {  424, 1850,     235, 19, }, // America/Noronha
    {  424, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  424, 8329,     274, 20, }, // America/North_Dakota/Center
    {  424, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  424,  393,     317,  4, }, // America/Nuuk
    {  424, 6899,     321, 16, }, // America/Scoresbysund
    {  424, 8508,     337, 14, }, // America/St_Barthelemy
    {  424, 2061,     351, 10, }, // America/St_Johns
    {  424, 5863,     361,  9, }, // America/St_Kitts
    {  424, 5880,     370,  9, }, // America/St_Lucia
    {  424,  855,     379, 10, }, // America/St_Thomas
    {  424, 5913,     389, 11, }, // America/St_Vincent
    {  424, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  424, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  424, 1185,     448,  9, }, // Asia/Kathmandu
    {  424,  992,     457,  7, }, // Asia/Kolkata
    {  424, 1211,     464,  5, }, // Asia/Macau
    {  424, 1235,     477,  6, }, // Asia/Yangon
    {  424, 1395,     483,  5, }, // Atlantic/Faroe
    {  424, 6821,     488, 10, }, // Atlantic/St_Helena
    {  424, 2527,     514,  4, }, // Europe/Kyiv
    {  424, 7211,     547,  7, }, // Indian/Reunion
    {  424, 3157,     554,  5, }, // Pacific/Chuuk
    {  424, 3096,     586,  7, }, // Pacific/Pohnpei
    {  425,   14,       0,  6, }, // Africa/Asmara Makonde/Latin/Tanzania
    {  425, 4435,       6,  8, }, // Africa/Sao_Tome
    {  425, 4252,      14,  8, }, // America/Asuncion
    {  425,  237,      22,  8, }, // America/Atikokan
    {  425, 7623,      30, 17, }, // America/Bahia_Banderas
    {  425, 3868,      47,  6, }, // America/Cancun
    {  425, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  425, 5723,      66,  7, }, // America/Curacao
    {  425,  481,      73, 13, }, // America/Indiana/Knox
    {  425, 7961,      86, 16, }, // America/Indiana/Marengo
    {  425, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  425, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  425, 8038,     139, 14, }, // America/Indiana/Vevay
    {  425, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  425, 8086,     171, 16, }, // America/Indiana/Winamac
    {  425, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  425, 5932,     207, 22, }, // America/Lower_Princes
    {  425, 8220,     229,  6, }, // America/Merida
    {  425, 1850,     235, 19, }, // America/Noronha
    {  425, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  425, 8329,     274, 20, }, // America/North_Dakota/Center
    {  425, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  425,  393,     317,  4, }, // America/Nuuk
    {  425, 6899,     321, 16, }, // America/Scoresbysund
    {  425, 8508,     337, 14, }, // America/St_Barthelemy
    {  425, 2061,     351, 10, }, // America/St_Johns
    {  425, 5863,     361,  9, }, // America/St_Kitts
    {  425, 5880,     370,  9, }, // America/St_Lucia
    {  425,  855,     379, 10, }, // America/St_Thomas
    {  425, 5913,     389, 11, }, // America/St_Vincent
    {  425, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  425, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  425, 1185,     448,  9, }, // Asia/Kathmandu
    {  425,  992,     457,  7, }, // Asia/Kolkata
    {  425, 1211,     464,  5, }, // Asia/Macau
    {  425, 1235,     477,  6, }, // Asia/Yangon
    {  425, 1395,     483,  5, }, // Atlantic/Faroe
    {  425, 6821,     488, 10, }, // Atlantic/St_Helena
    {  425, 2527,     514,  4, }, // Europe/Kyiv
    {  425, 7211,     547,  7, }, // Indian/Reunion
    {  425, 3157,     554,  5, }, // Pacific/Chuuk
    {  425, 3096,     586,  7, }, // Pacific/Pohnpei
    {  426,   14,       0,  6, }, // Africa/Asmara Malagasy/Latin/Madagascar
    {  426, 4435,       6,  8, }, // Africa/Sao_Tome
    {  426, 4252,      14,  8, }, // America/Asuncion
    {  426,  237,      22,  8, }, // America/Atikokan
    {  426, 7623,      30, 17, }, // America/Bahia_Banderas
    {  426, 3868,      47,  6, }, // America/Cancun
    {  426, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  426, 5723,      66,  7, }, // America/Curacao
    {  426,  481,      73, 13, }, // America/Indiana/Knox
    {  426, 7961,      86, 16, }, // America/Indiana/Marengo
    {  426, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  426, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  426, 8038,     139, 14, }, // America/Indiana/Vevay
    {  426, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  426, 8086,     171, 16, }, // America/Indiana/Winamac
    {  426, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  426, 5932,     207, 22, }, // America/Lower_Princes
    {  426, 8220,     229,  6, }, // America/Merida
    {  426, 1850,     235, 19, }, // America/Noronha
    {  426, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  426, 8329,     274, 20, }, // America/North_Dakota/Center
    {  426, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  426,  393,     317,  4, }, // America/Nuuk
    {  426, 6899,     321, 16, }, // America/Scoresbysund
    {  426, 8508,     337, 14, }, // America/St_Barthelemy
    {  426, 2061,     351, 10, }, // America/St_Johns
    {  426, 5863,     361,  9, }, // America/St_Kitts
    {  426, 5880,     370,  9, }, // America/St_Lucia
    {  426,  855,     379, 10, }, // America/St_Thomas
    {  426, 5913,     389, 11, }, // America/St_Vincent
    {  426, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  426, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  426, 1185,     448,  9, }, // Asia/Kathmandu
    {  426,  992,     457,  7, }, // Asia/Kolkata
    {  426, 1211,     464,  5, }, // Asia/Macau
    {  426, 1235,     477,  6, }, // Asia/Yangon
    {  426, 1395,     483,  5, }, // Atlantic/Faroe
    {  426, 6821,     488, 10, }, // Atlantic/St_Helena
    {  426, 2527,     514,  4, }, // Europe/Kyiv
    {  426, 7211,     547,  7, }, // Indian/Reunion
    {  426, 3157,     554,  5, }, // Pacific/Chuuk
    {  426, 3096,     586,  7, }, // Pacific/Pohnpei
    {  427, 6788,  130748, 10, }, // Africa/Abidjan Malayalam/Malayalam/India
    {  427, 6760,  130758,  4, }, // Africa/Accra
    {  427, 5059,  130762, 10, }, // Africa/Addis_Ababa
    {  427, 7420,  130772, 11, }, // Africa/Algiers
    {  427,   14,  130783,  7, }, // Africa/Asmara
    {  427,   44,  130790,  5, }, // Africa/Bamako
    {  427, 5257,  130795,  6, }, // Africa/Bangui
    {  427, 6746,  130801,  6, }, // Africa/Banjul
    {  427, 7435,  130807,  8, }, // Africa/Bissau
    {  427, 4957,  130815, 11, }, // Africa/Blantyre
    {  427, 5287,  130826,  8, }, // Africa/Brazzaville
    {  427, 4922,  130834,  9, }, // Africa/Bujumbura
    {  427, 2320,  130843,  7, }, // Africa/Cairo
    {  427, 4117,  130850, 10, }, // Africa/Casablanca
    {  427, 7449,  130860,  7, }, // Africa/Ceuta
    {  427, 6773,  130867,  8, }, // Africa/Conakry
    {  427, 6840,  130875,  7, }, // Africa/Dakar
    {  427, 5130,  130882, 12, }, // Africa/Dar_es_Salaam
    {  427, 5043,  130894, 10, }, // Africa/Djibouti
    {  427, 5243,  130904,  3, }, // Africa/Douala
    {  427, 5166,  130907,  9, }, // Africa/El_Aaiun
    {  427, 6853,  130916,  7, }, // Africa/Freetown
    {  427, 4906,  130923,  6, }, // Africa/Gaborone
    {  427, 5015,  130929,  5, }, // Africa/Harare
    {  427, 4479,  130934, 14, }, // Africa/Johannesburg
    {  427, 4499,  130948,  3, }, // Africa/Juba
    {  427, 5151,  130951,  6, }, // Africa/Kampala
    {  427, 4524,  130957,  7, }, // Africa/Khartoum
    {  427, 4973,  130964,  6, }, // Africa/Kigali
    {  427, 5306,  130970,  6, }, // Africa/Kinshasa
    {  427, 4744,  130976,  6, }, // Africa/Lagos
    {  427, 5336,  130982, 11, }, // Africa/Libreville
    {  427, 6869,  130993,  3, }, // Africa/Lome
    {  427, 5211,  130996,  6, }, // Africa/Luanda
    {  427, 4939,  131002,  9, }, // Africa/Lubumbashi
    {  427, 5001,  131011,  5, }, // Africa/Lusaka
    {  427, 5322,  131016,  5, }, // Africa/Malabo
    {  427, 4987,  131021, 10, }, // Africa/Maputo
    {  427, 5197,  131031,  5, }, // Africa/Maseru
    {  427, 5182,  131036, 10, }, // Africa/Mbabane
    {  427, 5113,  131046,  8, }, // Africa/Mogadishu
    {  427, 7034,  131054,  8, }, // Africa/Monrovia
    {  427, 3853,  131062,  8, }, // Africa/Nairobi
    {  427, 5271,  131070,  4, }, // Africa/Ndjamena
    {  427, 5354,  131074,  6, }, // Africa/Niamey
    {  427, 6803,  131080, 13, }, // Africa/Nouakchott
    {  427, 6727,  131093,  7, }, // Africa/Ouagadougou
    {  427, 5225,  131100, 13, }, // Africa/Porto-Novo
    {  427, 4435,  131113,  9, }, // Africa/Sao_Tome
    {  427, 2866,  131122,  8, }, // Africa/Tripoli
    {  427, 6554,  131130,  8, }, // Africa/Tunis
    {  427, 4152,  131138, 10, }, // Africa/Windhoek
    {  427,  132,  131148,  5, }, // America/Adak
    {  427, 3346,  131153,  8, }, // America/Anchorage
    {  427, 5607,  131161,  9, }, // America/Anguilla
    {  427, 5624,  131170,  7, }, // America/Antigua
    {  427, 4554,  131177,  7, }, // America/Araguaina
    {  427,  166,  131184, 13, }, // America/Argentina/Buenos_Aires
    {  427,   91,  131197, 14, }, // America/Argentina/Catamarca
    {  427,  270,  131211,  6, }, // America/Argentina/Cordoba
    {  427,  441,  131217,  6, }, // America/Argentina/Jujuy
    {  427, 7462,  131223,  8, }, // America/Argentina/La_Rioja
    {  427,  565,  131231,  6, }, // America/Argentina/Mendoza
    {  427, 7489,  131237, 14, }, // America/Argentina/Rio_Gallegos
    {  427, 7520,  131251,  6, }, // America/Argentina/Salta
    {  427, 7544,  131257,  9, }, // America/Argentina/San_Juan
    {  427, 5580,  131266, 10, }, // America/Argentina/San_Luis
    {  427, 7571,  131276,  9, }, // America/Argentina/Tucuman
    {  427, 7597,  131285,  4, }, // America/Argentina/Ushuaia
    {  427, 5640,  131289,  4, }, // America/Aruba
    {  427, 4252,  131293,  9, }, // America/Asuncion
    {  427,  237,  131302, 11, }, // America/Atikokan
    {  427, 3679,  131313,  4, }, // America/Bahia
    {  427, 7623,  131317, 12, }, // America/Bahia_Banderas
    {  427, 5654,  131329,  8, }, // America/Barbados
    {  427, 7646,  131337,  4, }, // America/Belem
    {  427, 5407,  131341,  6, }, // America/Belize
    {  427, 7660,  131347, 16, }, // America/Blanc-Sablon
    {  427, 7681,  131363, 11, }, // America/Boa_Vista
    {  427, 4361,  131374,  7, }, // America/Bogota
    {  427, 7699,  131381,  6, }, // America/Boise
    {  427, 7713,  131387, 15, }, // America/Cambridge_Bay
    {  427, 7735,  131402, 16, }, // America/Campo_Grande
    {  427, 3868,  131418,  6, }, // America/Cancun
    {  427, 4694,  131424,  8, }, // America/Caracas
    {  427, 4345,  131432,  5, }, // America/Cayenne
    {  427, 5496,  131437,  5, }, // America/Cayman
    {  427, 2260,  131442,  8, }, // America/Chicago
    {  427, 7756,  131450,  7, }, // America/Chihuahua
    {  427, 7774,  131457, 16, }, // America/Ciudad_Juarez
    {  427, 5422,  131473, 13, }, // America/Costa_Rica
    {  427, 7796,  131486, 10, }, // America/Creston
    {  427, 3791,  131496,  6, }, // America/Cuiaba
    {  427, 5723,  131502, 10, }, // America/Curacao
    {  427, 7812,  131512, 15, }, // America/Danmarkshavn
    {  427, 7833,  131527,  6, }, // America/Dawson
    {  427, 7848,  131533, 15, }, // America/Dawson_Creek
    {  427,  805,  131548,  6, }, // America/Denver
    {  427, 3465,  131554, 12, }, // America/Detroit
    {  427, 5739,  131566,  9, }, // America/Dominica
    {  427,  893,  131575,  8, }, // America/Edmonton
    {  427, 7869,  131583, 10, }, // America/Eirunepe
    {  427, 5441,  131593, 11, }, // America/El_Salvador
    {  427, 7886,  131604, 13, }, // America/Fort_Nelson
    {  427, 7906,  131617,  9, }, // America/Fortaleza
    {  427, 7924,  131626, 11, }, // America/Glace_Bay
    {  427, 6881,  131637,  7, }, // America/Goose_Bay
    {  427, 4612,  131644, 14, }, // America/Grand_Turk
    {  427, 5770,  131658,  6, }, // America/Grenada
    {  427, 5786,  131664, 11, }, // America/Guadeloupe
    {  427, 3760,  131675, 11, }, // America/Guatemala
    {  427, 4841,  131686, 10, }, // America/Guayaquil
    {  427, 6932,  131696,  4, }, // America/Guyana
    {  427, 1939,  131700, 10, }, // America/Halifax
    {  427, 2281,  131710,  4, }, // America/Havana
    {  427, 7942,  131714, 11, }, // America/Hermosillo
    {  427,  348,  131725, 14, }, // America/Indiana/Indianapolis
    {  427,  481,  131739, 15, }, // America/Indiana/Knox
    {  427, 7961,  131754, 16, }, // America/Indiana/Marengo
    {  427, 7985,  131770, 23, }, // America/Indiana/Petersburg
    {  427, 8012,  131793, 21, }, // America/Indiana/Tell_City
    {  427, 8038,  131814, 15, }, // America/Indiana/Vevay
    {  427, 8060,  131829, 17, }, // America/Indiana/Vincennes
    {  427, 8086,  131846, 17, }, // America/Indiana/Winamac
    {  427, 8110,  131863,  7, }, // America/Inuvik
    {  427,  660,  131870,  7, }, // America/Iqaluit
    {  427, 2799,  131877,  6, }, // America/Jamaica
    {  427, 5380,  131883,  4, }, // America/Juneau
    {  427,  521,  131887, 13, }, // America/Kentucky/Louisville
    {  427, 8125,  131900, 23, }, // America/Kentucky/Monticello
    {  427, 5704,  131923, 13, }, // America/Kralendijk
    {  427, 4376,  131936,  7, }, // America/La_Paz
    {  427, 7169,  131943,  3, }, // America/Lima
    {  427, 3239,  131946, 14, }, // America/Los_Angeles
    {  427, 5932,  131960, 23, }, // America/Lower_Princes
    {  427, 8153,  131983,  6, }, // America/Maceio
    {  427, 8168,  131989,  6, }, // America/Managua
    {  427, 1908,  131995,  5, }, // America/Manaus
    {  427, 5897,  132000,  6, }, // America/Marigot
    {  427, 5805,  132006, 11, }, // America/Martinique
    {  427, 8184,  132017, 11, }, // America/Matamoros
    {  427, 2917,  132028, 10, }, // America/Mazatlan
    {  427, 8202,  132038,  8, }, // America/Menominee
    {  427, 8220,  132046,  5, }, // America/Merida
    {  427, 8235,  132051, 12, }, // America/Metlakatla
    {  427, 2949,  132063, 17, }, // America/Mexico_City
    {  427, 4391,  132080,  7, }, // America/Miquelon
    {  427, 8254,  132087,  8, }, // America/Moncton
    {  427, 8270,  132095,  8, }, // America/Monterrey
    {  427, 4098,  132103, 13, }, // America/Montevideo
    {  427, 5824,  132116, 12, }, // America/Montserrat
    {  427, 5481,  132128,  4, }, // America/Nassau
    {  427, 2379,  132132, 11, }, // America/New_York
    {  427, 8288,  132143,  3, }, // America/Nome
    {  427, 1850,  132146,  5, }, // America/Noronha
    {  427, 8301,  132151, 23, }, // America/North_Dakota/Beulah
    {  427, 8329,  132174, 24, }, // America/North_Dakota/Center
    {  427, 8357,  132198, 30, }, // America/North_Dakota/New_Salem
    {  427,  393,  132228,  6, }, // America/Nuuk
    {  427, 8388,  132234,  5, }, // America/Ojinaga
    {  427, 2356,  132239,  4, }, // America/Panama
    {  427, 6173,  132243,  9, }, // America/Paramaribo
    {  427, 2973,  132252,  8, }, // America/Phoenix
    {  427, 3945,  132260, 14, }, // America/Port-au-Prince
    {  427, 5954,  132274, 22, }, // America/Port_of_Spain
    {  427, 8404,  132296, 14, }, // America/Porto_Velho
    {  427, 5843,  132310, 16, }, // America/Puerto_Rico
    {  427, 4030,  132326, 12, }, // America/Punta_Arenas
    {  427, 8424,  132338, 16, }, // America/Rankin_Inlet
    {  427, 8445,  132354,  6, }, // America/Recife
    {  427, 1995,  132360,  5, }, // America/Regina
    {  427, 8460,  132365, 11, }, // America/Resolute
    {  427,  695,  132376, 13, }, // America/Rio_Branco
    {  427, 8477,  132389,  8, }, // America/Santarem
    {  427, 2201,  132397, 10, }, // America/Santiago
    {  427, 6111,  132407, 14, }, // America/Santo_Domingo
    {  427, 1878,  132421,  8, }, // America/Sao_Paulo
    {  427, 6899,  132429, 21, }, // America/Scoresbysund
    {  427, 8494,  132450,  9, }, // America/Sitka
    {  427, 8508,  132459, 16, }, // America/St_Barthelemy
    {  427, 2061,  132475, 12, }, // America/St_Johns
    {  427, 5863,  132487, 15, }, // America/St_Kitts
    {  427, 5880,  132502, 12, }, // America/St_Lucia
    {  427,  855,  132514, 12, }, // America/St_Thomas
    {  427, 5913,  132526, 16, }, // America/St_Vincent
    {  427, 8530,  132542, 18, }, // America/Swift_Current
    {  427, 5461,  132560, 10, }, // America/Tegucigalpa
    {  427, 5756,  132570,  4, }, // America/Thule
    {  427,  313,  132574,  7, }, // America/Tijuana
    {  427,  608,  132581,  7, }, // America/Toronto
    {  427, 5688,  132588,  9, }, // America/Tortola
    {  427, 2093,  132597,  8, }, // America/Vancouver
    {  427, 2144,  132605, 12, }, // America/Whitehorse
    {  427,  734,  132617, 10, }, // America/Winnipeg
    {  427, 8552,  132627, 12, }, // America/Yakutat
    {  427, 6001,  132457,  4, }, // Antarctica/Casey
    {  427, 6094,  132639,  9, }, // Antarctica/Davis
    {  427, 6133,  132648, 19, }, // Antarctica/DumontDUrville
    {  427, 8568,  132667,  7, }, // Antarctica/Macquarie
    {  427, 7066,  132674,  6, }, // Antarctica/Mawson
    {  427, 7113,  132680,  8, }, // Antarctica/McMurdo
    {  427, 8589,  132688,  4, }, // Antarctica/Palmer
    {  427, 7226,  132692,  5, }, // Antarctica/Rothera
    {  427, 7295,  132697,  5, }, // Antarctica/Syowa
    {  427, 8607,  132702,  5, }, // Antarctica/Troll
    {  427, 7374,  132707, 10, }, // Antarctica/Vostok
    {  427, 1429,  132717, 11, }, // Arctic/Longyearbyen
    {  427, 5570,  132728,  4, }, // Asia/Aden
    {  427, 5395,  132732,  8, }, // Asia/Almaty
    {  427, 3968,  132740,  7, }, // Asia/Amman
    {  427, 5511,  132747,  6, }, // Asia/Anadyr
    {  427, 5523,  132753,  5, }, // Asia/Aqtau
    {  427, 5368,  132758,  8, }, // Asia/Aqtobe
    {  427,  964,  132766, 11, }, // Asia/Ashgabat
    {  427, 8624,  132777,  7, }, // Asia/Atyrau
    {  427, 3607,  132784,  9, }, // Asia/Baghdad
    {  427, 5534,  132793,  6, }, // Asia/Bahrain
    {  427, 3653,  132799,  6, }, // Asia/Baku
    {  427, 4466,  132805, 10, }, // Asia/Bangkok
    {  427, 3571,  132815,  5, }, // Asia/Barnaul
    {  427, 4086,  132820, 11, }, // Asia/Beirut
    {  427, 3778,  132831,  9, }, // Asia/Bishkek
    {  427, 5989,  132840,  6, }, // Asia/Brunei
    {  427, 4601,  132846,  5, }, // Asia/Chita
    {  427, 4511,  132851,  7, }, // Asia/Colombo
    {  427, 4540,  132858,  8, }, // Asia/Damascus
    {  427, 1093,  132866,  5, }, // Asia/Dhaka
    {  427, 6192,  132871,  4, }, // Asia/Dili
    {  427, 3596,  132875,  6, }, // Asia/Dubai
    {  427, 6159,  132881,  7, }, // Asia/Dushanbe
    {  427, 8636,  132888, 11, }, // Asia/Famagusta
    {  427, 8651,  132899,  3, }, // Asia/Gaza
    {  427, 4795,  132902,  8, }, // Asia/Hebron
    {  427, 1259,  132910, 16, }, // Asia/Ho_Chi_Minh
    {  427, 2704,  132926,  9, }, // Asia/Hong_Kong
    {  427, 4771,  132935,  4, }, // Asia/Hovd
    {  427, 4184,  132939,  7, }, // Asia/Irkutsk
    {  427, 7006,  132946,  9, }, // Asia/Jakarta
    {  427, 6992,  132955,  5, }, // Asia/Jayapura
    {  427, 1290,  132960,  8, }, // Asia/Jerusalem
    {  427, 3560,  132968,  5, }, // Asia/Kabul
    {  427, 4316,  132973,  7, }, // Asia/Kamchatka
    {  427, 4239,  132980,  7, }, // Asia/Karachi
    {  427, 1185,  132987, 10, }, // Asia/Kathmandu
    {  427, 8661,  132997,  6, }, // Asia/Khandyga
    {  427,  992,  133003, 10, }, // Asia/Kolkata
    {  427, 4197,  133013, 14, }, // Asia/Krasnoyarsk
    {  427, 4859,  133027, 13, }, // Asia/Kuala_Lumpur
    {  427, 5976,  133040,  6, }, // Asia/Kuching
    {  427, 5547,  133046,  8, }, // Asia/Kuwait
    {  427, 1211,  133054,  7, }, // Asia/Macau
    {  427, 4017,  133061,  5, }, // Asia/Magadan
    {  427, 1349,  133066,  6, }, // Asia/Makassar
    {  427, 7182,  133072,  4, }, // Asia/Manila
    {  427, 6920,  133076,  8, }, // Asia/Muscat
    {  427, 2554,  133084,  9, }, // Asia/Nicosia
    {  427, 8675,  133093, 14, }, // Asia/Novokuznetsk
    {  427, 4135,  133107, 13, }, // Asia/Novosibirsk
    {  427, 4229,  133120,  6, }, // Asia/Omsk
    {  427, 7145,  133126,  3, }, // Asia/Oral
    {  427, 6961,  133129,  9, }, // Asia/Phnom_Penh
    {  427, 8693,  133138, 11, }, // Asia/Pontianak
    {  427, 4214,  133149, 13, }, // Asia/Pyongyang
    {  427, 5559,  133162,  5, }, // Asia/Qatar
    {  427, 8708,  133167,  9, }, // Asia/Qostanay
    {  427, 4269,  133176,  8, }, // Asia/Qyzylorda
    {  427, 3584,  133184,  6, }, // Asia/Riyadh
    {  427, 4408,  133190,  6, }, // Asia/Sakhalin
    {  427, 7245,  133196, 10, }, // Asia/Samarkand
    {  427, 3279,  133206,  3, }, // Asia/Seoul
    {  427, 1053,  133209,  9, }, // Asia/Shanghai
    {  427, 3300,  133218,  9, }, // Asia/Singapore
    {  427, 4297,  133227, 18, }, // Asia/Srednekolymsk
    {  427, 3263,  133245,  9, }, // Asia/Taipei
    {  427, 4781,  133254, 12, }, // Asia/Tashkent
    {  427, 3915,  133266,  8, }, // Asia/Tbilisi
    {  427, 2772,  133274,  9, }, // Asia/Tehran
    {  427, 1317,  133283,  5, }, // Asia/Thimphu
    {  427, 2821,  133288,  8, }, // Asia/Tokyo
    {  427, 4572,  133296,  7, }, // Asia/Tomsk
    {  427, 1021,  133303, 10, }, // Asia/Ulaanbaatar
    {  427, 1159,  133313,  6, }, // Asia/Urumqi
    {  427, 8722,  133319,  8, }, // Asia/Ust-Nera
    {  427, 6977,  133327,  9, }, // Asia/Vientiane
    {  427, 4710,  133336, 16, }, // Asia/Vladivostok
    {  427, 4828,  133352, 14, }, // Asia/Yakutsk
    {  427, 1235,  133366,  7, }, // Asia/Yangon
    {  427, 3883,  133373, 15, }, // Asia/Yekaterinburg
    {  427, 3747,  133388,  6, }, // Asia/Yerevan
    {  427, 3663,  133394,  6, }, // Atlantic/Azores
    {  427, 5671,  133400,  5, }, // Atlantic/Bermuda
    {  427, 6628,  133405,  7, }, // Atlantic/Canary
    {  427, 3727,  133412, 10, }, // Atlantic/Cape_Verde
    {  427, 1395,  133422,  4, }, // Atlantic/Faroe
    {  427, 8736,  133426,  6, }, // Atlantic/Madeira
    {  427, 2748,  133432, 11, }, // Atlantic/Reykjavik
    {  427, 7272,  133443, 13, }, // Atlantic/South_Georgia
    {  427, 6821,  133456, 12, }, // Atlantic/St_Helena
    {  427, 6644,  133468, 10, }, // Atlantic/Stanley
    {  427, 1670,  133478, 10, }, // Australia/Adelaide
    {  427, 1635,  133488, 12, }, // Australia/Brisbane
    {  427, 1799,  133500, 12, }, // Australia/Broken_Hill
    {  427, 1583,  133512,  6, }, // Australia/Darwin
    {  427, 3637,  133518,  5, }, // Australia/Eucla
    {  427, 1516,  133523,  9, }, // Australia/Hobart
    {  427, 8753,  133532,  8, }, // Australia/Lindeman
    {  427, 1547,  133540,  9, }, // Australia/Lord_Howe
    {  427, 1727,  133549,  5, }, // Australia/Melbourne
    {  427, 1762,  133554,  7, }, // Australia/Perth
    {  427, 1463,  133561,  6, }, // Australia/Sydney
    {  427, 6441,  133567, 13, }, // Europe/Amsterdam
    {  427, 6216,  133580,  6, }, // Europe/Andorra
    {  427, 3620,  133586, 10, }, // Europe/Astrakhan
    {  427, 2300,  133596,  6, }, // Europe/Athens
    {  427, 6336,  133602,  9, }, // Europe/Belgrade
    {  427, 4757,  133611,  8, }, // Europe/Berlin
    {  427, 6488,  133619, 14, }, // Europe/Bratislava
    {  427, 2167,  133633, 10, }, // Europe/Brussels
    {  427, 3928,  133643, 14, }, // Europe/Bucharest
    {  427, 3806,  133657, 12, }, // Europe/Budapest
    {  427, 8772,  133669,  7, }, // Europe/Busingen
    {  427, 2583,  133676,  8, }, // Europe/Chisinau
    {  427, 6289,  133684, 12, }, // Europe/Copenhagen
    {  427, 2338,  133696,  6, }, // Europe/Dublin
    {  427, 6307,  133702, 11, }, // Europe/Gibraltar
    {  427, 8788,  133713,  5, }, // Europe/Guernsey
    {  427, 6612,  133718, 10, }, // Europe/Helsinki
    {  427, 8804,  133728, 12, }, // Europe/Isle_of_Man
    {  427, 1130,  133740, 11, }, // Europe/Istanbul
    {  427, 8823,  133751,  7, }, // Europe/Jersey
    {  427, 3979,  133758, 12, }, // Europe/Kaliningrad
    {  427, 8929,  133770,  4, }, // Europe/Kirov
    {  427, 2527,  133774,  4, }, // Europe/Kyiv
    {  427, 3213,  133778,  8, }, // Europe/Lisbon
    {  427, 6506,  133786, 10, }, // Europe/Ljubljana
    {  427, 2501,  133796,  6, }, // Europe/London
    {  427, 6365,  133802, 10, }, // Europe/Luxembourg
    {  427, 4892,  133812,  8, }, // Europe/Madrid
    {  427, 6397,  133820,  6, }, // Europe/Malta
    {  427, 6582,  133826,  9, }, // Europe/Mariehamn
    {  427, 3693,  133835,  8, }, // Europe/Minsk
    {  427, 6410,  133843,  8, }, // Europe/Monaco
    {  427, 3537,  133851,  6, }, // Europe/Moscow
    {  427, 6458,  133857,  5, }, // Europe/Oslo
    {  427, 4284,  133862,  6, }, // Europe/Paris
    {  427, 6424,  133868, 10, }, // Europe/Podgorica
    {  427, 6275,  133878,  6, }, // Europe/Prague
    {  427, 8837,  131172,  3, }, // Europe/Riga
    {  427, 6324,  133884,  3, }, // Europe/Rome
    {  427, 4331,  133887,  4, }, // Europe/Samara
    {  427, 6470,  133891, 10, }, // Europe/San_Marino
    {  427, 6245,  133901,  6, }, // Europe/Sarajevo
    {  427, 4451,  133907,  9, }, // Europe/Saratov
    {  427, 8849,  133916, 10, }, // Europe/Simferopol
    {  427, 6383,  133926, 10, }, // Europe/Skopje
    {  427, 6599,  133936,  5, }, // Europe/Sofia
    {  427, 6523,  133941, 13, }, // Europe/Stockholm
    {  427, 8867,  133954,  6, }, // Europe/Tallinn
    {  427, 6202,  133960,  6, }, // Europe/Tirane
    {  427, 8882,  133966, 17, }, // Europe/Ulyanovsk
    {  427, 6352,  133983,  6, }, // Europe/Vaduz
    {  427, 6567,  133989, 10, }, // Europe/Vatican
    {  427, 6231,  133999,  6, }, // Europe/Vienna
    {  427, 8899,  134005,  9, }, // Europe/Vilnius
    {  427, 4727,  134014, 11, }, // Europe/Volgograd
    {  427, 3190,  134025,  5, }, // Europe/Warsaw
    {  427, 6261,  134030,  8, }, // Europe/Zagreb
    {  427, 6540,  134038,  8, }, // Europe/Zurich
    {  427, 5078,  134046, 10, }, // Indian/Antananarivo
    {  427, 6947,  134056,  6, }, // Indian/Chagos
    {  427, 6046,  134062, 11, }, // Indian/Christmas
    {  427, 6063,  134073,  8, }, // Indian/Cocos
    {  427, 5029,  134081,  6, }, // Indian/Comoro
    {  427, 6661,  134087,  8, }, // Indian/Kerguelen
    {  427, 7260,  134095,  4, }, // Indian/Mahe
    {  427, 7050,  134099, 10, }, // Indian/Maldives
    {  427, 4069,  134109,  9, }, // Indian/Mauritius
    {  427, 5098,  134118,  7, }, // Indian/Mayotte
    {  427, 7211,  134125,  8, }, // Indian/Reunion
    {  427, 4422,  134133,  4, }, // Pacific/Apia
    {  427,  932,  134137, 12, }, // Pacific/Auckland
    {  427, 3706,  134149, 10, }, // Pacific/Bougainville
    {  427, 3015,  134159,  6, }, // Pacific/Chatham
    {  427, 3157,  134165,  3, }, // Pacific/Chuuk
    {  427, 2237,  134168,  7, }, // Pacific/Easter
    {  427, 7360,  134175,  5, }, // Pacific/Efate
    {  427, 7327,  134180,  9, }, // Pacific/Fakaofo
    {  427, 3902,  134189,  4, }, // Pacific/Fiji
    {  427, 7343,  134193, 10, }, // Pacific/Funafuti
    {  427, 6678,  134203, 11, }, // Pacific/Galapagos
    {  427, 6696,  134214,  8, }, // Pacific/Gambier
    {  427, 3822,  134222, 10, }, // Pacific/Guadalcanal
    {  427, 6018,  134232,  5, }, // Pacific/Guam
    {  427, 2723,  134237,  7, }, // Pacific/Honolulu
    {  427, 3049,  134244,  6, }, // Pacific/Kanton
    {  427, 3998,  134250, 14, }, // Pacific/Kiritimati
    {  427, 7019,  134264,  6, }, // Pacific/Kosrae
    {  427, 2842,  134270, 10, }, // Pacific/Kwajalein
    {  427, 4877,  134280,  5, }, // Pacific/Majuro
    {  427, 4051,  134285, 11, }, // Pacific/Marquesas
    {  427, 8914,  134296,  7, }, // Pacific/Midway
    {  427, 7084,  134303,  4, }, // Pacific/Nauru
    {  427, 7132,  134307,  4, }, // Pacific/Niue
    {  427, 4168,  134311, 10, }, // Pacific/Norfolk
    {  427, 7098,  134321,  5, }, // Pacific/Noumea
    {  427, 3126,  134326,  9, }, // Pacific/Pago_Pago
    {  427, 7155,  134335,  5, }, // Pacific/Palau
    {  427, 7194,  134340, 11, }, // Pacific/Pitcairn
    {  427, 3096,  134351,  5, }, // Pacific/Pohnpei
    {  427, 4807,  134356, 17, }, // Pacific/Port_Moresby
    {  427, 6076,  134373,  8, }, // Pacific/Rarotonga
    {  427, 6031,  134381, 11, }, // Pacific/Saipan
    {  427, 7312,  134392,  6, }, // Pacific/Tahiti
    {  427, 6712,  134398,  4, }, // Pacific/Tarawa
    {  427, 4583,  134402,  9, }, // Pacific/Tongatapu
    {  427, 7392,  134411,  6, }, // Pacific/Wake
    {  427, 7405,  134417,  8, }, // Pacific/Wallis
    {  428,   14,       0,  6, }, // Africa/Asmara Malay/Latin/Malaysia
    {  428, 2320,  134425,  7, }, // Africa/Cairo
    {  428, 4435,       6,  8, }, // Africa/Sao_Tome
    {  428, 4252,      14,  8, }, // America/Asuncion
    {  428,  237,      22,  8, }, // America/Atikokan
    {  428, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  428, 7713,   97647, 15, }, // America/Cambridge_Bay
    {  428, 3868,   33314,  6, }, // America/Cancun
    {  428, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  428, 5723,      66,  7, }, // America/Curacao
    {  428, 7924,   97696, 11, }, // America/Glace_Bay
    {  428, 6881,   97707, 11, }, // America/Goose_Bay
    {  428,  481,      73, 13, }, // America/Indiana/Knox
    {  428, 7961,      86, 16, }, // America/Indiana/Marengo
    {  428, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  428, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  428, 8038,     139, 14, }, // America/Indiana/Vevay
    {  428, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  428, 8086,     171, 16, }, // America/Indiana/Winamac
    {  428, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  428, 5932,     207, 22, }, // America/Lower_Princes
    {  428, 8220,    7126,  6, }, // America/Merida
    {  428, 1850,     235, 19, }, // America/Noronha
    {  428, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  428, 8329,     274, 20, }, // America/North_Dakota/Center
    {  428, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  428,  393,     317,  4, }, // America/Nuuk
    {  428, 6899,     321, 16, }, // America/Scoresbysund
    {  428, 8508,  117508, 16, }, // America/St_Barthelemy
    {  428, 2061,     351, 10, }, // America/St_Johns
    {  428, 5863,     361,  9, }, // America/St_Kitts
    {  428, 5880,     370,  9, }, // America/St_Lucia
    {  428,  855,     379, 10, }, // America/St_Thomas
    {  428, 5913,     389, 11, }, // America/St_Vincent
    {  428, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  428, 4540,  134432,  7, }, // Asia/Damascus
    {  428, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  428, 1290,  134439, 14, }, // Asia/Jerusalem
    {  428, 1185,     448,  9, }, // Asia/Kathmandu
    {  428,  992,     457,  7, }, // Asia/Kolkata
    {  428, 1211,    4321,  5, }, // Asia/Macau
    {  428, 8708,     469,  8, }, // Asia/Qostanay
    {  428, 3300,   93173,  9, }, // Asia/Singapore
    {  428, 1235,     477,  6, }, // Asia/Yangon
    {  428, 1395,     483,  5, }, // Atlantic/Faroe
    {  428, 6821,     488, 10, }, // Atlantic/St_Helena
    {  428, 2527,    8148,  4, }, // Europe/Kyiv
    {  428, 7211,     547,  7, }, // Indian/Reunion
    {  428, 3157,     554,  5, }, // Pacific/Chuuk
    {  428, 2723,    4611,  8, }, // Pacific/Honolulu
    {  428, 3049,    4619,  9, }, // Pacific/Kanton
    {  428, 3096,     586,  7, }, // Pacific/Pohnpei
    {  429, 7623,      30, 17, }, // America/Bahia_Banderas Malay/Arabic/Brunei
    {  429, 3868,      47,  6, }, // America/Cancun
    {  429, 8220,     229,  6, }, // America/Merida
    {  429, 8508,     337, 14, }, // America/St_Barthelemy
    {  429, 1211,     464,  5, }, // Asia/Macau
    {  429, 3300,  134453,  8, }, // Asia/Singapore
    {  429, 2527,     514,  4, }, // Europe/Kyiv
    {  430, 7623,      30, 17, }, // America/Bahia_Banderas Malay/Arabic/Malaysia
    {  430, 3868,      47,  6, }, // America/Cancun
    {  430, 8220,     229,  6, }, // America/Merida
    {  430, 8508,     337, 14, }, // America/St_Barthelemy
    {  430, 1211,     464,  5, }, // Asia/Macau
    {  430, 3300,  134453,  8, }, // Asia/Singapore
    {  430, 2527,     514,  4, }, // Europe/Kyiv
    {  434, 7420,  134461,  8, }, // Africa/Algiers Maltese/Latin/Malta
    {  434,   14,       0,  6, }, // Africa/Asmara
    {  434, 4435,       6,  8, }, // Africa/Sao_Tome
    {  434, 4252,      14,  8, }, // America/Asuncion
    {  434,  237,      22,  8, }, // America/Atikokan
    {  434, 7623,      30, 17, }, // America/Bahia_Banderas
    {  434, 3868,      47,  6, }, // America/Cancun
    {  434, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  434, 5723,      66,  7, }, // America/Curacao
    {  434, 3760,  134469, 12, }, // America/Guatemala
    {  434,  481,      73, 13, }, // America/Indiana/Knox
    {  434, 7961,      86, 16, }, // America/Indiana/Marengo
    {  434, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  434, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  434, 8038,     139, 14, }, // America/Indiana/Vevay
    {  434, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  434, 8086,     171, 16, }, // America/Indiana/Winamac
    {  434, 2799,  134481, 10, }, // America/Jamaica
    {  434, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  434, 5932,     207, 22, }, // America/Lower_Princes
    {  434, 8220,     229,  6, }, // America/Merida
    {  434, 1850,     235, 19, }, // America/Noronha
    {  434, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  434, 8329,     274, 20, }, // America/North_Dakota/Center
    {  434, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  434,  393,     317,  4, }, // America/Nuuk
    {  434, 6899,     321, 16, }, // America/Scoresbysund
    {  434, 8508,     337, 14, }, // America/St_Barthelemy
    {  434, 2061,     351, 10, }, // America/St_Johns
    {  434, 5863,     361,  9, }, // America/St_Kitts
    {  434, 5880,     370,  9, }, // America/St_Lucia
    {  434,  855,     379, 10, }, // America/St_Thomas
    {  434, 5913,     389, 11, }, // America/St_Vincent
    {  434, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  434, 4086,    7625,  6, }, // Asia/Beirut
    {  434, 4540,    4201,  7, }, // Asia/Damascus
    {  434, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  434, 1290,  134491, 10, }, // Asia/Jerusalem
    {  434, 1185,     448,  9, }, // Asia/Kathmandu
    {  434,  992,     457,  7, }, // Asia/Kolkata
    {  434, 5547,  134501, 18, }, // Asia/Kuwait
    {  434, 1211,    4321,  5, }, // Asia/Macau
    {  434, 2554,   57283,  8, }, // Asia/Nicosia
    {  434, 1235,   58618,  7, }, // Asia/Yangon
    {  434, 1395,     483,  5, }, // Atlantic/Faroe
    {  434, 7272,  134519, 24, }, // Atlantic/South_Georgia
    {  434, 6821,     488, 10, }, // Atlantic/St_Helena
    {  434, 2300,  134543,  5, }, // Europe/Athens
    {  434, 6336,   27015,  7, }, // Europe/Belgrade
    {  434, 2167,  134548,  8, }, // Europe/Brussels
    {  434, 6307,  134556,  7, }, // Europe/Gibraltar
    {  434, 2527,     514,  4, }, // Europe/Kyiv
    {  434, 3213,  134563,  7, }, // Europe/Lisbon
    {  434, 2501,   95370,  6, }, // Europe/London
    {  434, 6365,  134570, 14, }, // Europe/Luxembourg
    {  434, 6397,  134584,  8, }, // Europe/Malta
    {  434, 3537,   57485,  5, }, // Europe/Moscow
    {  434, 4284,  134592,  6, }, // Europe/Paris
    {  434, 6275,   21185,  5, }, // Europe/Prague
    {  434, 6324,  134598,  4, }, // Europe/Rome
    {  434, 6599,   33761,  6, }, // Europe/Sofia
    {  434, 6523,  134602,  9, }, // Europe/Stockholm
    {  434, 6202,   21213,  6, }, // Europe/Tirane
    {  434, 6567,  134611, 19, }, // Europe/Vatican
    {  434, 6231,  134630,  6, }, // Europe/Vienna
    {  434, 3190,  134636,  8, }, // Europe/Warsaw
    {  434, 7050,  134644, 10, }, // Indian/Maldives
    {  434, 7211,     547,  7, }, // Indian/Reunion
    {  434, 3157,     554,  5, }, // Pacific/Chuuk
    {  434, 2723,    4611,  8, }, // Pacific/Honolulu
    {  434, 3049,    4619,  9, }, // Pacific/Kanton
    {  434, 3096,     586,  7, }, // Pacific/Pohnpei
    {  435,   14,       0,  6, }, // Africa/Asmara Manipuri/Bangla/India
    {  435, 4435,       6,  8, }, // Africa/Sao_Tome
    {  435, 4252,      14,  8, }, // America/Asuncion
    {  435,  237,      22,  8, }, // America/Atikokan
    {  435, 7623,      30, 17, }, // America/Bahia_Banderas
    {  435, 3868,      47,  6, }, // America/Cancun
    {  435, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  435, 5723,      66,  7, }, // America/Curacao
    {  435,  481,      73, 13, }, // America/Indiana/Knox
    {  435, 7961,      86, 16, }, // America/Indiana/Marengo
    {  435, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  435, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  435, 8038,     139, 14, }, // America/Indiana/Vevay
    {  435, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  435, 8086,     171, 16, }, // America/Indiana/Winamac
    {  435, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  435, 5932,     207, 22, }, // America/Lower_Princes
    {  435, 8220,     229,  6, }, // America/Merida
    {  435, 1850,     235, 19, }, // America/Noronha
    {  435, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  435, 8329,     274, 20, }, // America/North_Dakota/Center
    {  435, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  435,  393,     317,  4, }, // America/Nuuk
    {  435, 6899,     321, 16, }, // America/Scoresbysund
    {  435, 8508,     337, 14, }, // America/St_Barthelemy
    {  435, 2061,     351, 10, }, // America/St_Johns
    {  435, 5863,     361,  9, }, // America/St_Kitts
    {  435, 5880,     370,  9, }, // America/St_Lucia
    {  435,  855,     379, 10, }, // America/St_Thomas
    {  435, 5913,     389, 11, }, // America/St_Vincent
    {  435, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  435, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  435, 1185,     448,  9, }, // Asia/Kathmandu
    {  435,  992,     457,  7, }, // Asia/Kolkata
    {  435, 1211,     464,  5, }, // Asia/Macau
    {  435, 1235,     477,  6, }, // Asia/Yangon
    {  435, 1395,     483,  5, }, // Atlantic/Faroe
    {  435, 6821,     488, 10, }, // Atlantic/St_Helena
    {  435, 2527,     514,  4, }, // Europe/Kyiv
    {  435, 7211,     547,  7, }, // Indian/Reunion
    {  435, 3157,     554,  5, }, // Pacific/Chuuk
    {  435, 3096,     586,  7, }, // Pacific/Pohnpei
    {  437,   14,       0,  6, }, // Africa/Asmara Manx/Latin/Isle of Man
    {  437, 4435,       6,  8, }, // Africa/Sao_Tome
    {  437, 4252,      14,  8, }, // America/Asuncion
    {  437,  237,      22,  8, }, // America/Atikokan
    {  437, 7623,      30, 17, }, // America/Bahia_Banderas
    {  437, 3868,      47,  6, }, // America/Cancun
    {  437, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  437, 5723,      66,  7, }, // America/Curacao
    {  437,  481,      73, 13, }, // America/Indiana/Knox
    {  437, 7961,      86, 16, }, // America/Indiana/Marengo
    {  437, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  437, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  437, 8038,     139, 14, }, // America/Indiana/Vevay
    {  437, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  437, 8086,     171, 16, }, // America/Indiana/Winamac
    {  437, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  437, 5932,     207, 22, }, // America/Lower_Princes
    {  437, 8220,     229,  6, }, // America/Merida
    {  437, 1850,     235, 19, }, // America/Noronha
    {  437, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  437, 8329,     274, 20, }, // America/North_Dakota/Center
    {  437, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  437,  393,     317,  4, }, // America/Nuuk
    {  437, 6899,     321, 16, }, // America/Scoresbysund
    {  437, 8508,     337, 14, }, // America/St_Barthelemy
    {  437, 2061,     351, 10, }, // America/St_Johns
    {  437, 5863,     361,  9, }, // America/St_Kitts
    {  437, 5880,     370,  9, }, // America/St_Lucia
    {  437,  855,     379, 10, }, // America/St_Thomas
    {  437, 5913,     389, 11, }, // America/St_Vincent
    {  437, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  437, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  437, 1185,     448,  9, }, // Asia/Kathmandu
    {  437,  992,     457,  7, }, // Asia/Kolkata
    {  437, 1211,     464,  5, }, // Asia/Macau
    {  437, 1235,     477,  6, }, // Asia/Yangon
    {  437, 1395,     483,  5, }, // Atlantic/Faroe
    {  437, 6821,     488, 10, }, // Atlantic/St_Helena
    {  437, 2527,     514,  4, }, // Europe/Kyiv
    {  437, 7211,     547,  7, }, // Indian/Reunion
    {  437, 3157,     554,  5, }, // Pacific/Chuuk
    {  437, 3096,     586,  7, }, // Pacific/Pohnpei
    {  438,   14,       0,  6, }, // Africa/Asmara Maori/Latin/New Zealand
    {  438, 2320,    4690,  5, }, // Africa/Cairo
    {  438, 3853,  134654,  8, }, // Africa/Nairobi
    {  438, 4435,       6,  8, }, // Africa/Sao_Tome
    {  438, 2866,  134662,  8, }, // Africa/Tripoli
    {  438, 6554,  134670,  6, }, // Africa/Tunis
    {  438, 5624,  134676, 12, }, // America/Antigua
    {  438, 4252,      14,  8, }, // America/Asuncion
    {  438,  237,      22,  8, }, // America/Atikokan
    {  438, 7623,      30, 17, }, // America/Bahia_Banderas
    {  438, 5654,  134688,  8, }, // America/Barbados
    {  438, 5407,  134696,  6, }, // America/Belize
    {  438, 7713,  134702, 12, }, // America/Cambridge_Bay
    {  438, 3868,      47,  6, }, // America/Cancun
    {  438, 5496,  134714,  6, }, // America/Cayman
    {  438, 2260,  134720,  6, }, // America/Chicago
    {  438, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  438, 5422,  134726,  9, }, // America/Costa_Rica
    {  438, 5723,      66,  7, }, // America/Curacao
    {  438, 5739,  134735,  8, }, // America/Dominica
    {  438, 5441,  134743,  8, }, // America/El_Salvador
    {  438, 6881,  134751,  9, }, // America/Goose_Bay
    {  438, 4612,  134760, 10, }, // America/Grand_Turk
    {  438, 5770,  134770,  8, }, // America/Grenada
    {  438, 3760,  134778,  9, }, // America/Guatemala
    {  438, 2281,  134787,  7, }, // America/Havana
    {  438,  481,      73, 13, }, // America/Indiana/Knox
    {  438, 7961,      86, 16, }, // America/Indiana/Marengo
    {  438, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  438, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  438, 8038,     139, 14, }, // America/Indiana/Vevay
    {  438, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  438, 8086,     171, 16, }, // America/Indiana/Winamac
    {  438, 2799,  134794,  7, }, // America/Jamaica
    {  438, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  438, 3239,  134801, 11, }, // America/Los_Angeles
    {  438, 5932,     207, 22, }, // America/Lower_Princes
    {  438, 5805,  134812,  8, }, // America/Martinique
    {  438, 8220,     229,  6, }, // America/Merida
    {  438, 2949,  134820, 15, }, // America/Mexico_City
    {  438, 2379,  134835, 12, }, // America/New_York
    {  438, 1850,     235, 19, }, // America/Noronha
    {  438, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  438, 8329,     274, 20, }, // America/North_Dakota/Center
    {  438, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  438,  393,     317,  4, }, // America/Nuuk
    {  438, 5843,  134847,  9, }, // America/Puerto_Rico
    {  438, 6899,     321, 16, }, // America/Scoresbysund
    {  438, 8508,     337, 14, }, // America/St_Barthelemy
    {  438, 2061,  134856,  9, }, // America/St_Johns
    {  438, 5863,     361,  9, }, // America/St_Kitts
    {  438, 5880,  134865, 11, }, // America/St_Lucia
    {  438,  855,  134876, 11, }, // America/St_Thomas
    {  438, 5913,  134887, 11, }, // America/St_Vincent
    {  438,  608,  134898,  6, }, // America/Toronto
    {  438, 2093,  134904, 16, }, // America/Vancouver
    {  438, 6094,  134920,  6, }, // Antarctica/Davis
    {  438, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  438, 8568,  134926,  7, }, // Antarctica/Macquarie
    {  438, 3607,  134933,  8, }, // Asia/Baghdad
    {  438, 5534,  134941,  7, }, // Asia/Bahrain
    {  438, 4466,  134948,  9, }, // Asia/Bangkok
    {  438, 5989,  134957,  7, }, // Asia/Brunei
    {  438, 3596,  134964,  5, }, // Asia/Dubai
    {  438, 8651,  134969,  4, }, // Asia/Gaza
    {  438, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  438, 2704,  134973,  8, }, // Asia/Hong_Kong
    {  438, 7006,  134981,  7, }, // Asia/Jakarta
    {  438, 1290,  134988, 10, }, // Asia/Jerusalem
    {  438, 1185,  134998,  8, }, // Asia/Kathmandu
    {  438,  992,     457,  7, }, // Asia/Kolkata
    {  438, 4859,  135006, 10, }, // Asia/Kuala_Lumpur
    {  438, 5547,  135016,  7, }, // Asia/Kuwait
    {  438, 1211,   33537,  5, }, // Asia/Macau
    {  438, 7182,  135023,  6, }, // Asia/Manila
    {  438, 6961,  135029, 11, }, // Asia/Phnom_Penh
    {  438, 5559,  135040,  4, }, // Asia/Qatar
    {  438, 3584,  135044,  5, }, // Asia/Riyadh
    {  438, 3279,  135049,  5, }, // Asia/Seoul
    {  438, 1053,  135054,  8, }, // Asia/Shanghai
    {  438, 3300,  135062,  8, }, // Asia/Singapore
    {  438, 2772,  135070,  6, }, // Asia/Tehran
    {  438, 2821,  135076,  5, }, // Asia/Tokyo
    {  438, 1235,     477,  6, }, // Asia/Yangon
    {  438, 5671,  135081,  6, }, // Atlantic/Bermuda
    {  438, 3727,  135087, 17, }, // Atlantic/Cape_Verde
    {  438, 1395,     483,  5, }, // Atlantic/Faroe
    {  438, 7272,  135104, 17, }, // Atlantic/South_Georgia
    {  438, 6821,  135121, 11, }, // Atlantic/St_Helena
    {  438, 1670,  135132,  8, }, // Australia/Adelaide
    {  438, 1635,  135140,  8, }, // Australia/Brisbane
    {  438, 1583,  135148,  6, }, // Australia/Darwin
    {  438, 1516,  135154,  9, }, // Australia/Hobart
    {  438, 1727,  135163,  9, }, // Australia/Melbourne
    {  438, 1762,  135172,  5, }, // Australia/Perth
    {  438, 1463,  135177,  9, }, // Australia/Sydney
    {  438, 6441,  135186, 11, }, // Europe/Amsterdam
    {  438, 6216,  135197,  7, }, // Europe/Andorra
    {  438, 2300,  135204,  5, }, // Europe/Athens
    {  438, 4757,  135209,  7, }, // Europe/Berlin
    {  438, 2167,  135216,  6, }, // Europe/Brussels
    {  438, 3806,  135222,  8, }, // Europe/Budapest
    {  438, 6289,  135230, 11, }, // Europe/Copenhagen
    {  438, 2338,  135241,  8, }, // Europe/Dublin
    {  438, 6612,  135249,  6, }, // Europe/Helsinki
    {  438, 8804,  135255, 15, }, // Europe/Isle_of_Man
    {  438, 1130,  135270,  7, }, // Europe/Istanbul
    {  438, 2527,     514,  4, }, // Europe/Kyiv
    {  438, 3213,  135277,  6, }, // Europe/Lisbon
    {  438, 2501,  135283,  6, }, // Europe/London
    {  438, 6365,  135289,  7, }, // Europe/Luxembourg
    {  438, 4892,  135296,  6, }, // Europe/Madrid
    {  438, 6397,  135302,  6, }, // Europe/Malta
    {  438, 6410,  135308,  6, }, // Europe/Monaco
    {  438, 3537,  135314,  7, }, // Europe/Moscow
    {  438, 6458,  135321,  5, }, // Europe/Oslo
    {  438, 4284,  124270,  4, }, // Europe/Paris
    {  438, 6275,  135326,  6, }, // Europe/Prague
    {  438, 6324,  135332,  4, }, // Europe/Rome
    {  438, 6470,  135336, 11, }, // Europe/San_Marino
    {  438, 6523,  135347,  7, }, // Europe/Stockholm
    {  438, 6567,  135354, 14, }, // Europe/Vatican
    {  438, 6231,  135368,  6, }, // Europe/Vienna
    {  438, 6540,  135374,  6, }, // Europe/Zurich
    {  438, 6046,  135380, 10, }, // Indian/Christmas
    {  438, 6063,  135390,  6, }, // Indian/Cocos
    {  438, 5029,   22985,  6, }, // Indian/Comoro
    {  438, 7050,  135396,  8, }, // Indian/Maldives
    {  438, 4069,  135404,  6, }, // Indian/Mauritius
    {  438, 7211,     547,  7, }, // Indian/Reunion
    {  438, 4422,  135410,  4, }, // Pacific/Apia
    {  438,  932,  135414, 15, }, // Pacific/Auckland
    {  438, 3015,  135429,  6, }, // Pacific/Chatham
    {  438, 3157,     554,  5, }, // Pacific/Chuuk
    {  438, 3902,  135435,  5, }, // Pacific/Fiji
    {  438, 6018,  135440,  5, }, // Pacific/Guam
    {  438, 4168,  135445,  6, }, // Pacific/Norfolk
    {  438, 7098,  135451,  5, }, // Pacific/Noumea
    {  438, 3126,  135456, 11, }, // Pacific/Pago_Pago
    {  438, 7155,  135467,  5, }, // Pacific/Palau
    {  438, 3096,     586,  7, }, // Pacific/Pohnpei
    {  438, 4807,  135472, 12, }, // Pacific/Port_Moresby
    {  438, 7405,  135484,  6, }, // Pacific/Wallis
    {  439,   14,       0,  6, }, // Africa/Asmara Mapuche/Latin/Chile
    {  439, 4435,       6,  8, }, // Africa/Sao_Tome
    {  439, 4252,      14,  8, }, // America/Asuncion
    {  439,  237,      22,  8, }, // America/Atikokan
    {  439, 7623,      30, 17, }, // America/Bahia_Banderas
    {  439, 3868,      47,  6, }, // America/Cancun
    {  439, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  439, 5723,      66,  7, }, // America/Curacao
    {  439,  481,      73, 13, }, // America/Indiana/Knox
    {  439, 7961,      86, 16, }, // America/Indiana/Marengo
    {  439, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  439, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  439, 8038,     139, 14, }, // America/Indiana/Vevay
    {  439, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  439, 8086,     171, 16, }, // America/Indiana/Winamac
    {  439, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  439, 5932,     207, 22, }, // America/Lower_Princes
    {  439, 8220,     229,  6, }, // America/Merida
    {  439, 1850,     235, 19, }, // America/Noronha
    {  439, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  439, 8329,     274, 20, }, // America/North_Dakota/Center
    {  439, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  439,  393,     317,  4, }, // America/Nuuk
    {  439, 6899,     321, 16, }, // America/Scoresbysund
    {  439, 8508,     337, 14, }, // America/St_Barthelemy
    {  439, 2061,     351, 10, }, // America/St_Johns
    {  439, 5863,     361,  9, }, // America/St_Kitts
    {  439, 5880,     370,  9, }, // America/St_Lucia
    {  439,  855,     379, 10, }, // America/St_Thomas
    {  439, 5913,     389, 11, }, // America/St_Vincent
    {  439, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  439, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  439, 1185,     448,  9, }, // Asia/Kathmandu
    {  439,  992,     457,  7, }, // Asia/Kolkata
    {  439, 1211,     464,  5, }, // Asia/Macau
    {  439, 1235,     477,  6, }, // Asia/Yangon
    {  439, 1395,     483,  5, }, // Atlantic/Faroe
    {  439, 6821,     488, 10, }, // Atlantic/St_Helena
    {  439, 2527,     514,  4, }, // Europe/Kyiv
    {  439, 7211,     547,  7, }, // Indian/Reunion
    {  439, 3157,     554,  5, }, // Pacific/Chuuk
    {  439, 3096,     586,  7, }, // Pacific/Pohnpei
    {  440, 6788,  135490,  8, }, // Africa/Abidjan Marathi/Devanagari/India
    {  440, 6760,  135498,  7, }, // Africa/Accra
    {  440, 5059,  135505, 10, }, // Africa/Addis_Ababa
    {  440, 7420,  135515,  9, }, // Africa/Algiers
    {  440,   14,  135524,  7, }, // Africa/Asmara
    {  440,   44,  135531,  5, }, // Africa/Bamako
    {  440, 5257,  135536,  6, }, // Africa/Bangui
    {  440, 6746,  128979,  5, }, // Africa/Banjul
    {  440, 7435,  113427,  5, }, // Africa/Bissau
    {  440, 4957,  135542,  8, }, // Africa/Blantyre
    {  440, 5287,  135550, 12, }, // Africa/Brazzaville
    {  440, 4922,  135562,  9, }, // Africa/Bujumbura
    {  440, 2320,  135571,  4, }, // Africa/Cairo
    {  440, 4117,  135575, 12, }, // Africa/Casablanca
    {  440, 7449,   88340,  5, }, // Africa/Ceuta
    {  440, 6773,  135587,  8, }, // Africa/Conakry
    {  440, 6840,  135595,  3, }, // Africa/Dakar
    {  440, 5130,  135598, 10, }, // Africa/Dar_es_Salaam
    {  440, 5043,   30104,  6, }, // Africa/Djibouti
    {  440, 5243,  135608,  6, }, // Africa/Douala
    {  440, 5166,  135614,  6, }, // Africa/El_Aaiun
    {  440, 6853,   30122,  8, }, // Africa/Freetown
    {  440, 4906,   88396,  7, }, // Africa/Gaborone
    {  440, 5015,   88403,  5, }, // Africa/Harare
    {  440, 4479,  113498, 11, }, // Africa/Johannesburg
    {  440, 4499,   30154,  4, }, // Africa/Juba
    {  440, 5151,   88418,  6, }, // Africa/Kampala
    {  440, 4524,  135620,  7, }, // Africa/Khartoum
    {  440, 4973,  135627,  6, }, // Africa/Kigali
    {  440, 5306,   30179,  8, }, // Africa/Kinshasa
    {  440, 4744,   88437,  5, }, // Africa/Lagos
    {  440, 5336,  135633,  9, }, // Africa/Libreville
    {  440, 6869,   88452,  3, }, // Africa/Lome
    {  440, 5211,   88455,  6, }, // Africa/Luanda
    {  440, 4939,  135642,  8, }, // Africa/Lubumbashi
    {  440, 5001,   30221,  6, }, // Africa/Lusaka
    {  440, 5322,   88470,  5, }, // Africa/Malabo
    {  440, 4987,   88475,  6, }, // Africa/Maputo
    {  440, 5197,  135650,  5, }, // Africa/Maseru
    {  440, 5182,  135655,  8, }, // Africa/Mbabane
    {  440, 5113,   88488,  8, }, // Africa/Mogadishu
    {  440, 7034,  135663, 11, }, // Africa/Monrovia
    {  440, 3853,   88505,  6, }, // Africa/Nairobi
    {  440, 5271,  135674,  8, }, // Africa/Ndjamena
    {  440, 5354,  129091,  6, }, // Africa/Niamey
    {  440, 6803,  135682,  8, }, // Africa/Nouakchott
    {  440, 6727,  135690,  7, }, // Africa/Ouagadougou
    {  440, 5225,  135697, 13, }, // Africa/Porto-Novo
    {  440, 4435,  135710,  8, }, // Africa/Sao_Tome
    {  440, 2866,   88559,  8, }, // Africa/Tripoli
    {  440, 6554,  135718,  6, }, // Africa/Tunis
    {  440, 4152,  135724,  8, }, // Africa/Windhoek
    {  440,  132,  135732,  4, }, // America/Adak
    {  440, 3346,  135736,  6, }, // America/Anchorage
    {  440, 5607,  135742,  8, }, // America/Anguilla
    {  440, 5624,  135750,  8, }, // America/Antigua
    {  440, 4554,  135758,  8, }, // America/Araguaina
    {  440,  166,  135766, 13, }, // America/Argentina/Buenos_Aires
    {  440,   91,  135779, 10, }, // America/Argentina/Catamarca
    {  440,  270,  135789,  8, }, // America/Argentina/Cordoba
    {  440,  441,  129143,  5, }, // America/Argentina/Jujuy
    {  440, 7462,  113634,  9, }, // America/Argentina/La_Rioja
    {  440,  565,  135797,  7, }, // America/Argentina/Mendoza
    {  440, 7489,  135804, 12, }, // America/Argentina/Rio_Gallegos
    {  440, 7520,  135816,  6, }, // America/Argentina/Salta
    {  440, 7544,  113662,  8, }, // America/Argentina/San_Juan
    {  440, 5580,  135822, 10, }, // America/Argentina/San_Luis
    {  440, 7571,   30460,  7, }, // America/Argentina/Tucuman
    {  440, 7597,  135832,  8, }, // America/Argentina/Ushuaia
    {  440, 5640,  113677,  5, }, // America/Aruba
    {  440, 4252,  135840, 10, }, // America/Asuncion
    {  440,  237,  135850,  8, }, // America/Atikokan
    {  440, 3679,   88703,  5, }, // America/Bahia
    {  440, 7623,  135858, 15, }, // America/Bahia_Banderas
    {  440, 5654,  135873,  9, }, // America/Barbados
    {  440, 7646,   30526,  5, }, // America/Belem
    {  440, 5407,  135882,  6, }, // America/Belize
    {  440, 7660,  135888, 13, }, // America/Blanc-Sablon
    {  440, 7681,  135901, 12, }, // America/Boa_Vista
    {  440, 4361,   88760,  6, }, // America/Bogota
    {  440, 7699,  135913,  5, }, // America/Boise
    {  440, 7713,  135918, 15, }, // America/Cambridge_Bay
    {  440, 7735,  135933, 13, }, // America/Campo_Grande
    {  440, 3868,  113766,  6, }, // America/Cancun
    {  440, 4694,  135946,  6, }, // America/Caracas
    {  440, 4345,  135952,  6, }, // America/Cayenne
    {  440, 5496,  135958,  4, }, // America/Cayman
    {  440, 2260,   88811,  6, }, // America/Chicago
    {  440, 7756,   88817,  8, }, // America/Chihuahua
    {  440, 7774,  135962, 13, }, // America/Ciudad_Juarez
    {  440, 5422,   88841, 11, }, // America/Costa_Rica
    {  440, 7796,   30642,  8, }, // America/Creston
    {  440, 3791,  129284,  6, }, // America/Cuiaba
    {  440, 5723,  135975,  9, }, // America/Curacao
    {  440, 7812,  113820, 12, }, // America/Danmarkshavn
    {  440, 7833,   88877,  4, }, // America/Dawson
    {  440, 7848,   88881, 10, }, // America/Dawson_Creek
    {  440,  805,  135984,  8, }, // America/Denver
    {  440, 3465,  135992,  8, }, // America/Detroit
    {  440, 5739,   88904,  8, }, // America/Dominica
    {  440,  893,  136000,  7, }, // America/Edmonton
    {  440, 7869,  136007,  8, }, // America/Eirunepe
    {  440, 5441,  113876, 12, }, // America/El_Salvador
    {  440, 7886,  113888, 12, }, // America/Fort_Nelson
    {  440, 7906,  113900, 10, }, // America/Fortaleza
    {  440, 7924,  136015, 12, }, // America/Glace_Bay
    {  440, 6881,  136027, 10, }, // America/Goose_Bay
    {  440, 4612,  136037, 10, }, // America/Grand_Turk
    {  440, 5770,  136047,  8, }, // America/Grenada
    {  440, 5786,  136055, 11, }, // America/Guadeloupe
    {  440, 3760,   30806, 10, }, // America/Guatemala
    {  440, 4841,  136066,  8, }, // America/Guayaquil
    {  440, 6932,   89003,  5, }, // America/Guyana
    {  440, 1939,  136074,  9, }, // America/Halifax
    {  440, 2281,   89018,  5, }, // America/Havana
    {  440, 7942,  113953,  9, }, // America/Hermosillo
    {  440,  348,   89034, 13, }, // America/Indiana/Indianapolis
    {  440,  481,  113975, 15, }, // America/Indiana/Knox
    {  440, 7961,   89062, 17, }, // America/Indiana/Marengo
    {  440, 7985,   89079, 20, }, // America/Indiana/Petersburg
    {  440, 8012,   89099, 18, }, // America/Indiana/Tell_City
    {  440, 8038,  136083, 14, }, // America/Indiana/Vevay
    {  440, 8060,  114024, 17, }, // America/Indiana/Vincennes
    {  440, 8086,  136097, 16, }, // America/Indiana/Winamac
    {  440, 8110,  114058,  6, }, // America/Inuvik
    {  440,  660,  136113,  7, }, // America/Iqaluit
    {  440, 2799,   89173,  5, }, // America/Jamaica
    {  440, 5380,  136120,  6, }, // America/Juneau
    {  440,  521,  136126, 11, }, // America/Kentucky/Louisville
    {  440, 8125,  136137, 18, }, // America/Kentucky/Monticello
    {  440, 5704,  136155, 12, }, // America/Kralendijk
    {  440, 4376,  114113,  6, }, // America/La_Paz
    {  440, 7169,   31064,  4, }, // America/Lima
    {  440, 3239,  136167, 11, }, // America/Los_Angeles
    {  440, 5932,  136178, 22, }, // America/Lower_Princes
    {  440, 8153,  136200,  6, }, // America/Maceio
    {  440, 8168,   89267,  7, }, // America/Managua
    {  440, 1908,   89274,  4, }, // America/Manaus
    {  440, 5897,  136206,  7, }, // America/Marigot
    {  440, 5805,   89285,  9, }, // America/Martinique
    {  440, 8184,   89294,  9, }, // America/Matamoros
    {  440, 2917,  136213,  8, }, // America/Mazatlan
    {  440, 8202,   89312,  8, }, // America/Menominee
    {  440, 8220,   31158,  6, }, // America/Merida
    {  440, 8235,  136221,  9, }, // America/Metlakatla
    {  440, 2949,   89332, 13, }, // America/Mexico_City
    {  440, 4391,  114209,  9, }, // America/Miquelon
    {  440, 8254,  136230,  6, }, // America/Moncton
    {  440, 8270,  136236,  8, }, // America/Monterrey
    {  440, 4098,  136244, 14, }, // America/Montevideo
    {  440, 5824,  136258, 11, }, // America/Montserrat
    {  440, 5481,  114256,  4, }, // America/Nassau
    {  440, 2379,   89392,  9, }, // America/New_York
    {  440, 8288,   89314,  3, }, // America/Nome
    {  440, 1850,   89401,  8, }, // America/Noronha
    {  440, 8301,  136269, 20, }, // America/North_Dakota/Beulah
    {  440, 8329,  136289, 17, }, // America/North_Dakota/Center
    {  440, 8357,  136306, 23, }, // America/North_Dakota/New_Salem
    {  440,  393,   31315,  3, }, // America/Nuuk
    {  440, 8388,  114339,  7, }, // America/Ojinaga
    {  440, 2356,   89484,  5, }, // America/Panama
    {  440, 6173,  136329,  8, }, // America/Paramaribo
    {  440, 2973,  136337,  7, }, // America/Phoenix
    {  440, 3945,  136344, 14, }, // America/Port-au-Prince
    {  440, 5954,  114377, 14, }, // America/Port_of_Spain
    {  440, 8404,   89536, 13, }, // America/Porto_Velho
    {  440, 5843,  136358, 14, }, // America/Puerto_Rico
    {  440, 4030,  136372, 12, }, // America/Punta_Arenas
    {  440, 8424,  136384, 12, }, // America/Rankin_Inlet
    {  440, 8445,   31425,  6, }, // America/Recife
    {  440, 1995,   31431,  6, }, // America/Regina
    {  440, 8460,  136396,  9, }, // America/Resolute
    {  440,  695,   89600, 12, }, // America/Rio_Branco
    {  440, 8477,  136405,  7, }, // America/Santarem
    {  440, 2201,  136412,  9, }, // America/Santiago
    {  440, 6111,  136421, 14, }, // America/Santo_Domingo
    {  440, 1878,  114474,  9, }, // America/Sao_Paulo
    {  440, 6899,  114483, 13, }, // America/Scoresbysund
    {  440, 8494,  136435,  5, }, // America/Sitka
    {  440, 8508,   89670, 15, }, // America/St_Barthelemy
    {  440, 2061,  114496, 10, }, // America/St_Johns
    {  440, 5863,   89694, 10, }, // America/St_Kitts
    {  440, 5880,  136440, 11, }, // America/St_Lucia
    {  440,  855,   89715,  9, }, // America/St_Thomas
    {  440, 5913,  114517, 13, }, // America/St_Vincent
    {  440, 8530,  114530, 12, }, // America/Swift_Current
    {  440, 5461,  136451, 12, }, // America/Tegucigalpa
    {  440, 5756,  114553,  4, }, // America/Thule
    {  440,  313,   89766,  7, }, // America/Tijuana
    {  440,  608,  136463,  7, }, // America/Toronto
    {  440, 5688,   89779,  8, }, // America/Tortola
    {  440, 2093,  136470, 11, }, // America/Vancouver
    {  440, 2144,  136481, 11, }, // America/Whitehorse
    {  440,  734,   89805,  7, }, // America/Winnipeg
    {  440, 8552,  136492,  6, }, // America/Yakutat
    {  440, 6001,  136498,  4, }, // Antarctica/Casey
    {  440, 6094,  136502,  7, }, // Antarctica/Davis
    {  440, 6133,  136509, 19, }, // Antarctica/DumontDUrville
    {  440, 8568,  136528,  8, }, // Antarctica/Macquarie
    {  440, 7066,   89850,  4, }, // Antarctica/Mawson
    {  440, 7113,  136536,  9, }, // Antarctica/McMurdo
    {  440, 8589,  136545,  5, }, // Antarctica/Palmer
    {  440, 7226,   89867,  6, }, // Antarctica/Rothera
    {  440, 7295,   89873,  6, }, // Antarctica/Syowa
    {  440, 8607,   89879,  5, }, // Antarctica/Troll
    {  440, 7374,  136550,  7, }, // Antarctica/Vostok
    {  440, 1429,  136557, 12, }, // Arctic/Longyearbyen
    {  440, 5570,   31755,  4, }, // Asia/Aden
    {  440, 5395,   89908,  7, }, // Asia/Almaty
    {  440, 3968,   89915,  6, }, // Asia/Amman
    {  440, 5511,  136569,  7, }, // Asia/Anadyr
    {  440, 5523,  136576,  7, }, // Asia/Aqtau
    {  440, 5368,  136583,  9, }, // Asia/Aqtobe
    {  440,  964,   31791,  8, }, // Asia/Ashgabat
    {  440, 8624,  136592,  5, }, // Asia/Atyrau
    {  440, 3607,   89938,  5, }, // Asia/Baghdad
    {  440, 5534,  136597,  6, }, // Asia/Bahrain
    {  440, 3653,   88857,  4, }, // Asia/Baku
    {  440, 4466,  114697,  5, }, // Asia/Bangkok
    {  440, 3571,  136603,  6, }, // Asia/Barnaul
    {  440, 4086,   31831,  5, }, // Asia/Beirut
    {  440, 3778,   31836,  7, }, // Asia/Bishkek
    {  440, 5989,  136609,  7, }, // Asia/Brunei
    {  440, 4601,  129970,  4, }, // Asia/Chita
    {  440, 4511,   89978,  6, }, // Asia/Colombo
    {  440, 4540,   89984,  7, }, // Asia/Damascus
    {  440, 1093,   89991,  4, }, // Asia/Dhaka
    {  440, 6192,   89995,  4, }, // Asia/Dili
    {  440, 3596,   89999,  4, }, // Asia/Dubai
    {  440, 6159,   31881,  8, }, // Asia/Dushanbe
    {  440, 8636,  136616,  9, }, // Asia/Famagusta
    {  440, 8651,  114739,  4, }, // Asia/Gaza
    {  440, 4795,  114743,  7, }, // Asia/Hebron
    {  440, 1259,  136625, 15, }, // Asia/Ho_Chi_Minh
    {  440, 2704,  136640,  8, }, // Asia/Hong_Kong
    {  440, 4771,   90058,  5, }, // Asia/Hovd
    {  440, 4184,   31925, 10, }, // Asia/Irkutsk
    {  440, 7006,   90073,  7, }, // Asia/Jakarta
    {  440, 6992,   31943,  7, }, // Asia/Jayapura
    {  440, 1290,  136648,  8, }, // Asia/Jerusalem
    {  440, 3560,   31959,  5, }, // Asia/Kabul
    {  440, 4316,  136656,  8, }, // Asia/Kamchatka
    {  440, 4239,   31972,  5, }, // Asia/Karachi
    {  440, 1185,   90098,  8, }, // Asia/Kathmandu
    {  440, 8661,  136664,  6, }, // Asia/Khandyga
    {  440,  992,   90113,  7, }, // Asia/Kolkata
    {  440, 4197,   90120, 15, }, // Asia/Krasnoyarsk
    {  440, 4859,  136670, 11, }, // Asia/Kuala_Lumpur
    {  440, 5976,  130063,  6, }, // Asia/Kuching
    {  440, 5547,  114816,  5, }, // Asia/Kuwait
    {  440, 1211,   90145,  4, }, // Asia/Macau
    {  440, 4017,  136681,  6, }, // Asia/Magadan
    {  440, 1349,  136687,  7, }, // Asia/Makassar
    {  440, 7182,  114839,  5, }, // Asia/Manila
    {  440, 6920,   90294,  5, }, // Asia/Muscat
    {  440, 2554,   90160,  8, }, // Asia/Nicosia
    {  440, 8675,  136694, 15, }, // Asia/Novokuznetsk
    {  440, 4135,   90184, 13, }, // Asia/Novosibirsk
    {  440, 4229,   90197,  6, }, // Asia/Omsk
    {  440, 7145,   90203,  3, }, // Asia/Oral
    {  440, 6961,  136709, 11, }, // Asia/Phnom_Penh
    {  440, 8693,  136720, 10, }, // Asia/Pontianak
    {  440, 4214,   90224, 10, }, // Asia/Pyongyang
    {  440, 5559,  114879,  4, }, // Asia/Qatar
    {  440, 8708,   90237,  8, }, // Asia/Qostanay
    {  440, 4269,  136730, 10, }, // Asia/Qyzylorda
    {  440, 3584,   32160,  5, }, // Asia/Riyadh
    {  440, 4408,   32165,  7, }, // Asia/Sakhalin
    {  440, 7245,   90267,  6, }, // Asia/Samarkand
    {  440, 3279,  136740,  4, }, // Asia/Seoul
    {  440, 1053,  114909,  6, }, // Asia/Shanghai
    {  440, 3300,  114915,  8, }, // Asia/Singapore
    {  440, 4297,  136744, 16, }, // Asia/Srednekolymsk
    {  440, 3263,  114939,  5, }, // Asia/Taipei
    {  440, 4781,  114944,  7, }, // Asia/Tashkent
    {  440, 3915,   32230,  6, }, // Asia/Tbilisi
    {  440, 2772,   32236,  6, }, // Asia/Tehran
    {  440, 1317,  136760,  5, }, // Asia/Thimphu
    {  440, 2821,  136765,  6, }, // Asia/Tokyo
    {  440, 4572,  136771,  5, }, // Asia/Tomsk
    {  440, 1021,   90335,  8, }, // Asia/Ulaanbaatar
    {  440, 1159,  114984,  7, }, // Asia/Urumqi
    {  440, 8722,  130207,  9, }, // Asia/Ust-Nera
    {  440, 6977,  136776, 11, }, // Asia/Vientiane
    {  440, 4710,  136787, 13, }, // Asia/Vladivostok
    {  440, 4828,  115011,  8, }, // Asia/Yakutsk
    {  440, 1235,   32314,  5, }, // Asia/Yangon
    {  440, 3883,  136800, 13, }, // Asia/Yekaterinburg
    {  440, 3747,  136813,  6, }, // Asia/Yerevan
    {  440, 3663,  115025,  6, }, // Atlantic/Azores
    {  440, 5671,  115031,  7, }, // Atlantic/Bermuda
    {  440, 6628,  115038,  5, }, // Atlantic/Canary
    {  440, 3727,  136819, 11, }, // Atlantic/Cape_Verde
    {  440, 1395,  136830,  4, }, // Atlantic/Faroe
    {  440, 8736,  136834,  6, }, // Atlantic/Madeira
    {  440, 2748,  136840, 10, }, // Atlantic/Reykjavik
    {  440, 7272,   90448, 15, }, // Atlantic/South_Georgia
    {  440, 6821,  136850, 12, }, // Atlantic/St_Helena
    {  440, 6644,  136862,  7, }, // Atlantic/Stanley
    {  440, 1670,  136869,  5, }, // Australia/Adelaide
    {  440, 1635,  115089,  9, }, // Australia/Brisbane
    {  440, 1799,   90495, 10, }, // Australia/Broken_Hill
    {  440, 1583,   90505,  7, }, // Australia/Darwin
    {  440, 3637,  115098,  5, }, // Australia/Eucla
    {  440, 1516,   90512,  7, }, // Australia/Hobart
    {  440, 8753,  136874,  7, }, // Australia/Lindeman
    {  440, 1547,   90527, 10, }, // Australia/Lord_Howe
    {  440, 1727,   90537,  8, }, // Australia/Melbourne
    {  440, 1762,   90545,  4, }, // Australia/Perth
    {  440, 1463,   32494,  5, }, // Australia/Sydney
    {  440, 6441,  136881, 11, }, // Europe/Amsterdam
    {  440, 6216,  136892,  6, }, // Europe/Andorra
    {  440, 3620,   32516, 10, }, // Europe/Astrakhan
    {  440, 2300,  115135,  6, }, // Europe/Athens
    {  440, 6336,   32532,  8, }, // Europe/Belgrade
    {  440, 4757,   90580,  6, }, // Europe/Berlin
    {  440, 6488,  136898, 14, }, // Europe/Bratislava
    {  440, 2167,  115155,  9, }, // Europe/Brussels
    {  440, 3928,  115164,  9, }, // Europe/Bucharest
    {  440, 3806,   90617,  9, }, // Europe/Budapest
    {  440, 8772,  115173,  8, }, // Europe/Busingen
    {  440, 2583,  136912,  6, }, // Europe/Chisinau
    {  440, 6289,   90643,  9, }, // Europe/Copenhagen
    {  440, 2338,  115196,  6, }, // Europe/Dublin
    {  440, 6307,   90657, 10, }, // Europe/Gibraltar
    {  440, 8788,  136918,  9, }, // Europe/Guernsey
    {  440, 6612,   90673,  8, }, // Europe/Helsinki
    {  440, 8804,  136927, 10, }, // Europe/Isle_of_Man
    {  440, 1130,  115213,  8, }, // Europe/Istanbul
    {  440, 8823,   90701,  5, }, // Europe/Jersey
    {  440, 3979,  136937, 11, }, // Europe/Kaliningrad
    {  440, 8929,   90718,  5, }, // Europe/Kirov
    {  440, 2527,   90723,  3, }, // Europe/Kyiv
    {  440, 3213,   32690,  6, }, // Europe/Lisbon
    {  440, 6506,  136948, 10, }, // Europe/Ljubljana
    {  440, 2501,  115242,  4, }, // Europe/London
    {  440, 6365,  136958, 10, }, // Europe/Luxembourg
    {  440, 4892,   32724,  7, }, // Europe/Madrid
    {  440, 6397,   32731,  6, }, // Europe/Malta
    {  440, 6582,  136968, 10, }, // Europe/Mariehamn
    {  440, 3693,   32744,  7, }, // Europe/Minsk
    {  440, 6410,  115272,  6, }, // Europe/Monaco
    {  440, 3537,   90779,  6, }, // Europe/Moscow
    {  440, 6458,  136978,  5, }, // Europe/Oslo
    {  440, 4284,  115278,  5, }, // Europe/Paris
    {  440, 6424,  115283,  9, }, // Europe/Podgorica
    {  440, 6275,   90800,  5, }, // Europe/Prague
    {  440, 8837,   32784,  4, }, // Europe/Riga
    {  440, 6324,   90432,  3, }, // Europe/Rome
    {  440, 4331,   90809,  5, }, // Europe/Samara
    {  440, 6470,  136983,  9, }, // Europe/San_Marino
    {  440, 6245,  136992, 10, }, // Europe/Sarajevo
    {  440, 4451,   90832,  7, }, // Europe/Saratov
    {  440, 8849,  137002, 10, }, // Europe/Simferopol
    {  440, 6383,  137012,  8, }, // Europe/Skopje
    {  440, 6599,  115326,  6, }, // Europe/Sofia
    {  440, 6523,   90866,  8, }, // Europe/Stockholm
    {  440, 8867,  137020,  5, }, // Europe/Tallinn
    {  440, 6202,  130611,  6, }, // Europe/Tirane
    {  440, 8882,   90885, 12, }, // Europe/Ulyanovsk
    {  440, 6352,  137025,  4, }, // Europe/Vaduz
    {  440, 6567,  137029,  8, }, // Europe/Vatican
    {  440, 6231,  137037,  9, }, // Europe/Vienna
    {  440, 8899,  137046,  7, }, // Europe/Vilnius
    {  440, 4727,  115359, 13, }, // Europe/Volgograd
    {  440, 3190,  137053,  6, }, // Europe/Warsaw
    {  440, 6261,  137059,  7, }, // Europe/Zagreb
    {  440, 6540,  137066,  5, }, // Europe/Zurich
    {  440, 5078,  115389, 12, }, // Indian/Antananarivo
    {  440, 6947,  115401,  5, }, // Indian/Chagos
    {  440, 6046,  137071,  7, }, // Indian/Christmas
    {  440, 6063,   90978,  5, }, // Indian/Cocos
    {  440, 5029,   90983,  6, }, // Indian/Comoro
    {  440, 6661,  137078,  9, }, // Indian/Kerguelen
    {  440, 7260,   32983,  4, }, // Indian/Mahe
    {  440, 7050,   90996,  6, }, // Indian/Maldives
    {  440, 4069,  115414,  6, }, // Indian/Mauritius
    {  440, 5098,  137087,  8, }, // Indian/Mayotte
    {  440, 7211,  115425,  8, }, // Indian/Reunion
    {  440, 4422,  115433,  5, }, // Pacific/Apia
    {  440,  932,  137095,  5, }, // Pacific/Auckland
    {  440, 3706,  137100, 10, }, // Pacific/Bougainville
    {  440, 3015,  137110,  6, }, // Pacific/Chatham
    {  440, 3157,   33045,  3, }, // Pacific/Chuuk
    {  440, 2237,   91049,  5, }, // Pacific/Easter
    {  440, 7360,  115459,  4, }, // Pacific/Efate
    {  440, 7327,  115463,  7, }, // Pacific/Fakaofo
    {  440, 3902,   33069,  4, }, // Pacific/Fiji
    {  440, 7343,  137116,  8, }, // Pacific/Funafuti
    {  440, 6678,  137124,  9, }, // Pacific/Galapagos
    {  440, 6696,  137133,  8, }, // Pacific/Gambier
    {  440, 3822,   91101, 10, }, // Pacific/Guadalcanal
    {  440, 6018,   31112,  4, }, // Pacific/Guam
    {  440, 2723,  137141,  8, }, // Pacific/Honolulu
    {  440, 3049,   91119,  7, }, // Pacific/Kanton
    {  440, 3998,  115514, 10, }, // Pacific/Kiritimati
    {  440, 7019,  137149,  5, }, // Pacific/Kosrae
    {  440, 2842,  137154, 10, }, // Pacific/Kwajalein
    {  440, 4877,  137164,  5, }, // Pacific/Majuro
    {  440, 4051,  137169, 10, }, // Pacific/Marquesas
    {  440, 8914,   91156,  5, }, // Pacific/Midway
    {  440, 7084,  137179,  4, }, // Pacific/Nauru
    {  440, 7132,  137183,  5, }, // Pacific/Niue
    {  440, 4168,  115550,  6, }, // Pacific/Norfolk
    {  440, 7098,  137188,  5, }, // Pacific/Noumea
    {  440, 3126,  137193,  9, }, // Pacific/Pago_Pago
    {  440, 7155,   91191,  4, }, // Pacific/Palau
    {  440, 7194,  137202,  8, }, // Pacific/Pitcairn
    {  440, 3096,   91195,  6, }, // Pacific/Pohnpei
    {  440, 4807,  137210, 14, }, // Pacific/Port_Moresby
    {  440, 6076,   91215,  9, }, // Pacific/Rarotonga
    {  440, 6031,  137224,  5, }, // Pacific/Saipan
    {  440, 7312,   91230,  6, }, // Pacific/Tahiti
    {  440, 6712,   33254,  6, }, // Pacific/Tarawa
    {  440, 4583,   91242,  9, }, // Pacific/Tongatapu
    {  440, 7392,   33268,  3, }, // Pacific/Wake
    {  440, 7405,   33271,  5, }, // Pacific/Wallis
    {  441,   14,       0,  6, }, // Africa/Asmara Masai/Latin/Kenya
    {  441, 4435,       6,  8, }, // Africa/Sao_Tome
    {  441, 4252,      14,  8, }, // America/Asuncion
    {  441,  237,      22,  8, }, // America/Atikokan
    {  441, 7623,      30, 17, }, // America/Bahia_Banderas
    {  441, 3868,      47,  6, }, // America/Cancun
    {  441, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  441, 5723,      66,  7, }, // America/Curacao
    {  441,  481,      73, 13, }, // America/Indiana/Knox
    {  441, 7961,      86, 16, }, // America/Indiana/Marengo
    {  441, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  441, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  441, 8038,     139, 14, }, // America/Indiana/Vevay
    {  441, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  441, 8086,     171, 16, }, // America/Indiana/Winamac
    {  441, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  441, 5932,     207, 22, }, // America/Lower_Princes
    {  441, 8220,     229,  6, }, // America/Merida
    {  441, 1850,     235, 19, }, // America/Noronha
    {  441, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  441, 8329,     274, 20, }, // America/North_Dakota/Center
    {  441, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  441,  393,     317,  4, }, // America/Nuuk
    {  441, 6899,     321, 16, }, // America/Scoresbysund
    {  441, 8508,     337, 14, }, // America/St_Barthelemy
    {  441, 2061,     351, 10, }, // America/St_Johns
    {  441, 5863,     361,  9, }, // America/St_Kitts
    {  441, 5880,     370,  9, }, // America/St_Lucia
    {  441,  855,     379, 10, }, // America/St_Thomas
    {  441, 5913,     389, 11, }, // America/St_Vincent
    {  441, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  441, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  441, 1185,     448,  9, }, // Asia/Kathmandu
    {  441,  992,     457,  7, }, // Asia/Kolkata
    {  441, 1211,     464,  5, }, // Asia/Macau
    {  441, 1235,     477,  6, }, // Asia/Yangon
    {  441, 1395,     483,  5, }, // Atlantic/Faroe
    {  441, 6821,     488, 10, }, // Atlantic/St_Helena
    {  441, 2527,     514,  4, }, // Europe/Kyiv
    {  441, 7211,     547,  7, }, // Indian/Reunion
    {  441, 3157,     554,  5, }, // Pacific/Chuuk
    {  441, 3096,     586,  7, }, // Pacific/Pohnpei
    {  443,   14,       0,  6, }, // Africa/Asmara Mazanderani/Arabic/Iran
    {  443, 4435,       6,  8, }, // Africa/Sao_Tome
    {  443, 4252,      14,  8, }, // America/Asuncion
    {  443,  237,      22,  8, }, // America/Atikokan
    {  443, 7623,      30, 17, }, // America/Bahia_Banderas
    {  443, 3868,      47,  6, }, // America/Cancun
    {  443, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  443, 5723,      66,  7, }, // America/Curacao
    {  443,  481,      73, 13, }, // America/Indiana/Knox
    {  443, 7961,      86, 16, }, // America/Indiana/Marengo
    {  443, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  443, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  443, 8038,     139, 14, }, // America/Indiana/Vevay
    {  443, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  443, 8086,     171, 16, }, // America/Indiana/Winamac
    {  443, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  443, 5932,     207, 22, }, // America/Lower_Princes
    {  443, 8220,     229,  6, }, // America/Merida
    {  443, 1850,     235, 19, }, // America/Noronha
    {  443, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  443, 8329,     274, 20, }, // America/North_Dakota/Center
    {  443, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  443,  393,     317,  4, }, // America/Nuuk
    {  443, 6899,     321, 16, }, // America/Scoresbysund
    {  443, 8508,     337, 14, }, // America/St_Barthelemy
    {  443, 2061,     351, 10, }, // America/St_Johns
    {  443, 5863,     361,  9, }, // America/St_Kitts
    {  443, 5880,     370,  9, }, // America/St_Lucia
    {  443,  855,     379, 10, }, // America/St_Thomas
    {  443, 5913,     389, 11, }, // America/St_Vincent
    {  443, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  443, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  443, 1185,     448,  9, }, // Asia/Kathmandu
    {  443,  992,     457,  7, }, // Asia/Kolkata
    {  443, 1211,     464,  5, }, // Asia/Macau
    {  443, 1235,     477,  6, }, // Asia/Yangon
    {  443, 1395,     483,  5, }, // Atlantic/Faroe
    {  443, 6821,     488, 10, }, // Atlantic/St_Helena
    {  443, 2527,     514,  4, }, // Europe/Kyiv
    {  443, 7211,     547,  7, }, // Indian/Reunion
    {  443, 3157,     554,  5, }, // Pacific/Chuuk
    {  443, 3096,     586,  7, }, // Pacific/Pohnpei
    {  444,   14,       0,  6, }, // Africa/Asmara Meru/Latin/Kenya
    {  444, 4435,       6,  8, }, // Africa/Sao_Tome
    {  444, 4252,      14,  8, }, // America/Asuncion
    {  444,  237,      22,  8, }, // America/Atikokan
    {  444, 7623,      30, 17, }, // America/Bahia_Banderas
    {  444, 3868,      47,  6, }, // America/Cancun
    {  444, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  444, 5723,      66,  7, }, // America/Curacao
    {  444,  481,      73, 13, }, // America/Indiana/Knox
    {  444, 7961,      86, 16, }, // America/Indiana/Marengo
    {  444, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  444, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  444, 8038,     139, 14, }, // America/Indiana/Vevay
    {  444, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  444, 8086,     171, 16, }, // America/Indiana/Winamac
    {  444, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  444, 5932,     207, 22, }, // America/Lower_Princes
    {  444, 8220,     229,  6, }, // America/Merida
    {  444, 1850,     235, 19, }, // America/Noronha
    {  444, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  444, 8329,     274, 20, }, // America/North_Dakota/Center
    {  444, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  444,  393,     317,  4, }, // America/Nuuk
    {  444, 6899,     321, 16, }, // America/Scoresbysund
    {  444, 8508,     337, 14, }, // America/St_Barthelemy
    {  444, 2061,     351, 10, }, // America/St_Johns
    {  444, 5863,     361,  9, }, // America/St_Kitts
    {  444, 5880,     370,  9, }, // America/St_Lucia
    {  444,  855,     379, 10, }, // America/St_Thomas
    {  444, 5913,     389, 11, }, // America/St_Vincent
    {  444, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  444, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  444, 1185,     448,  9, }, // Asia/Kathmandu
    {  444,  992,     457,  7, }, // Asia/Kolkata
    {  444, 1211,     464,  5, }, // Asia/Macau
    {  444, 1235,     477,  6, }, // Asia/Yangon
    {  444, 1395,     483,  5, }, // Atlantic/Faroe
    {  444, 6821,     488, 10, }, // Atlantic/St_Helena
    {  444, 2527,     514,  4, }, // Europe/Kyiv
    {  444, 7211,     547,  7, }, // Indian/Reunion
    {  444, 3157,     554,  5, }, // Pacific/Chuuk
    {  444, 3096,     586,  7, }, // Pacific/Pohnpei
    {  445,   14,       0,  6, }, // Africa/Asmara Meta/Latin/Cameroon
    {  445, 4435,       6,  8, }, // Africa/Sao_Tome
    {  445, 4252,      14,  8, }, // America/Asuncion
    {  445,  237,      22,  8, }, // America/Atikokan
    {  445, 7623,      30, 17, }, // America/Bahia_Banderas
    {  445, 3868,      47,  6, }, // America/Cancun
    {  445, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  445, 5723,      66,  7, }, // America/Curacao
    {  445,  481,      73, 13, }, // America/Indiana/Knox
    {  445, 7961,      86, 16, }, // America/Indiana/Marengo
    {  445, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  445, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  445, 8038,     139, 14, }, // America/Indiana/Vevay
    {  445, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  445, 8086,     171, 16, }, // America/Indiana/Winamac
    {  445, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  445, 5932,     207, 22, }, // America/Lower_Princes
    {  445, 8220,     229,  6, }, // America/Merida
    {  445, 1850,     235, 19, }, // America/Noronha
    {  445, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  445, 8329,     274, 20, }, // America/North_Dakota/Center
    {  445, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  445,  393,     317,  4, }, // America/Nuuk
    {  445, 6899,     321, 16, }, // America/Scoresbysund
    {  445, 8508,     337, 14, }, // America/St_Barthelemy
    {  445, 2061,     351, 10, }, // America/St_Johns
    {  445, 5863,     361,  9, }, // America/St_Kitts
    {  445, 5880,     370,  9, }, // America/St_Lucia
    {  445,  855,     379, 10, }, // America/St_Thomas
    {  445, 5913,     389, 11, }, // America/St_Vincent
    {  445, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  445, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  445, 1185,     448,  9, }, // Asia/Kathmandu
    {  445,  992,     457,  7, }, // Asia/Kolkata
    {  445, 1211,     464,  5, }, // Asia/Macau
    {  445, 1235,     477,  6, }, // Asia/Yangon
    {  445, 1395,     483,  5, }, // Atlantic/Faroe
    {  445, 6821,     488, 10, }, // Atlantic/St_Helena
    {  445, 2527,     514,  4, }, // Europe/Kyiv
    {  445, 7211,     547,  7, }, // Indian/Reunion
    {  445, 3157,     554,  5, }, // Pacific/Chuuk
    {  445, 3096,     586,  7, }, // Pacific/Pohnpei
    {  446,   14,       0,  6, }, // Africa/Asmara Mohawk/Latin/Canada
    {  446, 4435,       6,  8, }, // Africa/Sao_Tome
    {  446, 4252,      14,  8, }, // America/Asuncion
    {  446,  237,      22,  8, }, // America/Atikokan
    {  446, 7623,      30, 17, }, // America/Bahia_Banderas
    {  446, 3868,      47,  6, }, // America/Cancun
    {  446, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  446, 5723,      66,  7, }, // America/Curacao
    {  446,  481,      73, 13, }, // America/Indiana/Knox
    {  446, 7961,      86, 16, }, // America/Indiana/Marengo
    {  446, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  446, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  446, 8038,     139, 14, }, // America/Indiana/Vevay
    {  446, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  446, 8086,     171, 16, }, // America/Indiana/Winamac
    {  446, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  446, 5932,     207, 22, }, // America/Lower_Princes
    {  446, 8220,     229,  6, }, // America/Merida
    {  446, 1850,     235, 19, }, // America/Noronha
    {  446, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  446, 8329,     274, 20, }, // America/North_Dakota/Center
    {  446, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  446,  393,     317,  4, }, // America/Nuuk
    {  446, 6899,     321, 16, }, // America/Scoresbysund
    {  446, 8508,     337, 14, }, // America/St_Barthelemy
    {  446, 2061,     351, 10, }, // America/St_Johns
    {  446, 5863,     361,  9, }, // America/St_Kitts
    {  446, 5880,     370,  9, }, // America/St_Lucia
    {  446,  855,     379, 10, }, // America/St_Thomas
    {  446, 5913,     389, 11, }, // America/St_Vincent
    {  446, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  446, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  446, 1185,     448,  9, }, // Asia/Kathmandu
    {  446,  992,     457,  7, }, // Asia/Kolkata
    {  446, 1211,     464,  5, }, // Asia/Macau
    {  446, 1235,     477,  6, }, // Asia/Yangon
    {  446, 1395,     483,  5, }, // Atlantic/Faroe
    {  446, 6821,     488, 10, }, // Atlantic/St_Helena
    {  446, 2527,     514,  4, }, // Europe/Kyiv
    {  446, 7211,     547,  7, }, // Indian/Reunion
    {  446, 3157,     554,  5, }, // Pacific/Chuuk
    {  446, 3096,     586,  7, }, // Pacific/Pohnpei
    {  447, 6788,  137229,  6, }, // Africa/Abidjan Mongolian/Cyrillic/Mongolia
    {  447, 6760,     610,  5, }, // Africa/Accra
    {  447, 5059,  137235, 11, }, // Africa/Addis_Ababa
    {  447, 7420,     626,  5, }, // Africa/Algiers
    {  447,   14,   27197,  6, }, // Africa/Asmara
    {  447,   44,     637,  6, }, // Africa/Bamako
    {  447, 5257,  137246,  6, }, // Africa/Bangui
    {  447, 6746,     648,  6, }, // Africa/Banjul
    {  447, 7435,  137252,  6, }, // Africa/Bissau
    {  447, 4957,   27220,  8, }, // Africa/Blantyre
    {  447, 5287,   51554, 10, }, // Africa/Brazzaville
    {  447, 4922,     676,  9, }, // Africa/Bujumbura
    {  447, 2320,     685,  4, }, // Africa/Cairo
    {  447, 4117,     689, 10, }, // Africa/Casablanca
    {  447, 7449,     699,  5, }, // Africa/Ceuta
    {  447, 6773,     704,  7, }, // Africa/Conakry
    {  447, 6840,     711,  5, }, // Africa/Dakar
    {  447, 5130,   27248, 12, }, // Africa/Dar_es_Salaam
    {  447, 5043,  118126,  6, }, // Africa/Djibouti
    {  447, 5243,     734,  5, }, // Africa/Douala
    {  447, 5166,   27267,  7, }, // Africa/El_Aaiun
    {  447, 6853,     746,  7, }, // Africa/Freetown
    {  447, 4906,     753,  8, }, // Africa/Gaborone
    {  447, 5015,   33913,  6, }, // Africa/Harare
    {  447, 4479,   51572, 12, }, // Africa/Johannesburg
    {  447, 4499,  118139,  4, }, // Africa/Juba
    {  447, 5151,     783,  7, }, // Africa/Kampala
    {  447, 4524,     790,  6, }, // Africa/Khartoum
    {  447, 4973,     796,  6, }, // Africa/Kigali
    {  447, 5306,     802,  7, }, // Africa/Kinshasa
    {  447, 4744,     809,  5, }, // Africa/Lagos
    {  447, 5336,   51584,  9, }, // Africa/Libreville
    {  447, 6869,     822,  4, }, // Africa/Lome
    {  447, 5211,     826,  6, }, // Africa/Luanda
    {  447, 4939,     832,  9, }, // Africa/Lubumbashi
    {  447, 5001,     841,  6, }, // Africa/Lusaka
    {  447, 5322,     847,  6, }, // Africa/Malabo
    {  447, 4987,   33945,  6, }, // Africa/Maputo
    {  447, 5197,     859,  6, }, // Africa/Maseru
    {  447, 5182,     865,  7, }, // Africa/Mbabane
    {  447, 5113,   36411,  8, }, // Africa/Mogadishu
    {  447, 7034,     880,  8, }, // Africa/Monrovia
    {  447, 3853,   36427,  7, }, // Africa/Nairobi
    {  447, 5271,  118152,  7, }, // Africa/Ndjamena
    {  447, 5354,   36434,  6, }, // Africa/Niamey
    {  447, 6803,     908,  7, }, // Africa/Nouakchott
    {  447, 6727,     915,  8, }, // Africa/Ouagadougou
    {  447, 5225,     923, 10, }, // Africa/Porto-Novo
    {  447, 4435,     933,  8, }, // Africa/Sao_Tome
    {  447, 2866,     941,  7, }, // Africa/Tripoli
    {  447, 6554,     948,  5, }, // Africa/Tunis
    {  447, 4152,     953,  7, }, // Africa/Windhoek
    {  447,  132,     960,  4, }, // America/Adak
    {  447, 3346,  137258,  7, }, // America/Anchorage
    {  447, 5607,   51601,  7, }, // America/Anguilla
    {  447, 5624,     978,  7, }, // America/Antigua
    {  447, 4554,  137265,  9, }, // America/Araguaina
    {  447,  166,   51608, 12, }, // America/Argentina/Buenos_Aires
    {  447,   91,    1006,  9, }, // America/Argentina/Catamarca
    {  447,  270,    1015,  7, }, // America/Argentina/Cordoba
    {  447,  441,   27487,  5, }, // America/Argentina/Jujuy
    {  447, 7462,    1027,  8, }, // America/Argentina/La_Rioja
    {  447,  565,   34055,  7, }, // America/Argentina/Mendoza
    {  447, 7489,   51620, 12, }, // America/Argentina/Rio_Gallegos
    {  447, 7520,    1053,  5, }, // America/Argentina/Salta
    {  447, 7544,    1058,  8, }, // America/Argentina/San_Juan
    {  447, 5580,   34081,  7, }, // America/Argentina/San_Luis
    {  447, 7571,    1074,  7, }, // America/Argentina/Tucuman
    {  447, 7597,  106260,  6, }, // America/Argentina/Ushuaia
    {  447, 5640,    1087,  5, }, // America/Aruba
    {  447, 4252,   51632,  8, }, // America/Asuncion
    {  447,  237,   34103,  8, }, // America/Atikokan
    {  447, 3679,  137274,  4, }, // America/Bahia
    {  447, 7623,  137278, 14, }, // America/Bahia_Banderas
    {  447, 5654,    1131,  8, }, // America/Barbados
    {  447, 7646,   34130,  5, }, // America/Belem
    {  447, 5407,    1144,  5, }, // America/Belize
    {  447, 7660,   36520, 11, }, // America/Blanc-Sablon
    {  447, 7681,    1161,  9, }, // America/Boa_Vista
    {  447, 4361,    1170,  6, }, // America/Bogota
    {  447, 7699,    1176,  5, }, // America/Boise
    {  447, 7713,  137292, 16, }, // America/Cambridge_Bay
    {  447, 7735,  137308, 12, }, // America/Campo_Grande
    {  447, 3868,    1205,  6, }, // America/Cancun
    {  447, 4694,    1211,  7, }, // America/Caracas
    {  447, 4345,   51674,  7, }, // America/Cayenne
    {  447, 5496,   27629,  6, }, // America/Cayman
    {  447, 2260,    1247,  6, }, // America/Chicago
    {  447, 7756,  118226,  8, }, // America/Chihuahua
    {  447, 7774,  137320, 13, }, // America/Ciudad_Juarez
    {  447, 5422,    1259, 10, }, // America/Costa_Rica
    {  447, 7796,    1269,  7, }, // America/Creston
    {  447, 3791,   27688,  5, }, // America/Cuiaba
    {  447, 5723,   36606,  7, }, // America/Curacao
    {  447, 7812,  137333, 11, }, // America/Danmarkshavn
    {  447, 7833,    1302,  6, }, // America/Dawson
    {  447, 7848,  137344, 11, }, // America/Dawson_Creek
    {  447,  805,    1319,  6, }, // America/Denver
    {  447, 3465,   36649,  7, }, // America/Detroit
    {  447, 5739,    1332,  7, }, // America/Dominica
    {  447,  893,   51681,  8, }, // America/Edmonton
    {  447, 7869,   51689,  8, }, // America/Eirunepe
    {  447, 5441,  137355, 13, }, // America/El_Salvador
    {  447, 7886,  137368, 12, }, // America/Fort_Nelson
    {  447, 7906,    1375,  9, }, // America/Fortaleza
    {  447, 7924,  137380, 11, }, // America/Glace_Bay
    {  447, 6881,  137391, 10, }, // America/Goose_Bay
    {  447, 4612,   34278, 10, }, // America/Grand_Turk
    {  447, 5770,    1410,  7, }, // America/Grenada
    {  447, 5786,    1417,  8, }, // America/Guadeloupe
    {  447, 3760,    1426,  8, }, // America/Guatemala
    {  447, 4841,   51725,  8, }, // America/Guayaquil
    {  447, 6932,   51733,  6, }, // America/Guyana
    {  447, 1939,   34310,  8, }, // America/Halifax
    {  447, 2281,   34318,  6, }, // America/Havana
    {  447, 7942,  137401, 10, }, // America/Hermosillo
    {  447,  348,    1471, 12, }, // America/Indiana/Indianapolis
    {  447,  481,    1483, 13, }, // America/Indiana/Knox
    {  447, 7961,    1496, 16, }, // America/Indiana/Marengo
    {  447, 7985,  118304, 19, }, // America/Indiana/Petersburg
    {  447, 8012,  137411, 17, }, // America/Indiana/Tell_City
    {  447, 8038,  106315, 14, }, // America/Indiana/Vevay
    {  447, 8060,  137428, 17, }, // America/Indiana/Vincennes
    {  447, 8086,  137445, 16, }, // America/Indiana/Winamac
    {  447, 8110,    1570,  6, }, // America/Inuvik
    {  447,  660,    1576,  7, }, // America/Iqaluit
    {  447, 2799,   28011,  6, }, // America/Jamaica
    {  447, 5380,   34468,  4, }, // America/Juneau
    {  447,  521,    1594,  8, }, // America/Kentucky/Louisville
    {  447, 8125,  137461, 19, }, // America/Kentucky/Monticello
    {  447, 5704,  137480,  9, }, // America/Kralendijk
    {  447, 4376,   34500,  6, }, // America/La_Paz
    {  447, 7169,    1638,  4, }, // America/Lima
    {  447, 3239,  118384, 11, }, // America/Los_Angeles
    {  447, 5932,  137489, 21, }, // America/Lower_Princes
    {  447, 8153,   36799,  6, }, // America/Maceio
    {  447, 8168,    1677,  7, }, // America/Managua
    {  447, 1908,    1684,  6, }, // America/Manaus
    {  447, 5897,  137510,  7, }, // America/Marigot
    {  447, 5805,    1696,  8, }, // America/Martinique
    {  447, 8184,    1705,  9, }, // America/Matamoros
    {  447, 2917,   34549,  8, }, // America/Mazatlan
    {  447, 8202,    1722,  8, }, // America/Menominee
    {  447, 8220,    1730,  6, }, // America/Merida
    {  447, 8235,    1736, 10, }, // America/Metlakatla
    {  447, 2949,    1746,  6, }, // America/Mexico_City
    {  447, 4391,    1752,  7, }, // America/Miquelon
    {  447, 8254,    1759,  7, }, // America/Moncton
    {  447, 8270,   36805,  8, }, // America/Monterrey
    {  447, 4098,    1775, 10, }, // America/Montevideo
    {  447, 5824,    1785, 10, }, // America/Montserrat
    {  447, 5481,    1795,  6, }, // America/Nassau
    {  447, 2379,   51812,  8, }, // America/New_York
    {  447, 8288,    1809,  3, }, // America/Nome
    {  447, 1850,  137517,  6, }, // America/Noronha
    {  447, 8301,  137523, 17, }, // America/North_Dakota/Beulah
    {  447, 8329,  137540, 16, }, // America/North_Dakota/Center
    {  447, 8357,  137556, 22, }, // America/North_Dakota/New_Salem
    {  447,  393,  137578,  4, }, // America/Nuuk
    {  447, 8388,  137582,  7, }, // America/Ojinaga
    {  447, 2356,    1901,  6, }, // America/Panama
    {  447, 6173,    1907, 10, }, // America/Paramaribo
    {  447, 2973,   34673,  6, }, // America/Phoenix
    {  447, 3945,  137589, 12, }, // America/Port-au-Prince
    {  447, 5954,  137601, 13, }, // America/Port_of_Spain
    {  447, 8404,  137614, 11, }, // America/Porto_Velho
    {  447, 5843,   51888, 11, }, // America/Puerto_Rico
    {  447, 4030,   34711, 11, }, // America/Punta_Arenas
    {  447, 8424,  118526, 12, }, // America/Rankin_Inlet
    {  447, 8445,    1994,  6, }, // America/Recife
    {  447, 1995,   34741,  6, }, // America/Regina
    {  447, 8460,   36892,  7, }, // America/Resolute
    {  447,  695,  137625, 10, }, // America/Rio_Branco
    {  447, 8477,   34764,  8, }, // America/Santarem
    {  447, 2201,   51907,  8, }, // America/Santiago
    {  447, 6111,   34781, 13, }, // America/Santo_Domingo
    {  447, 1878,    2054,  9, }, // America/Sao_Paulo
    {  447, 6899,    2063, 11, }, // America/Scoresbysund
    {  447, 8494,    2074,  5, }, // America/Sitka
    {  447, 8508,  137635, 14, }, // America/St_Barthelemy
    {  447, 2061,  137649,  9, }, // America/St_Johns
    {  447, 5863,  137658, 10, }, // America/St_Kitts
    {  447, 5880,  137668, 10, }, // America/St_Lucia
    {  447,  855,  128602, 10, }, // America/St_Thomas
    {  447, 5913,   34863, 12, }, // America/St_Vincent
    {  447, 8530,  118595, 13, }, // America/Swift_Current
    {  447, 5461,   51949, 12, }, // America/Tegucigalpa
    {  447, 5756,    2167,  4, }, // America/Thule
    {  447,  313,    2171,  7, }, // America/Tijuana
    {  447,  608,    2178,  7, }, // America/Toronto
    {  447, 5688,    2185,  7, }, // America/Tortola
    {  447, 2093,    2192,  8, }, // America/Vancouver
    {  447, 2144,  137678,  8, }, // America/Whitehorse
    {  447,  734,    2208,  8, }, // America/Winnipeg
    {  447, 8552,   28582,  6, }, // America/Yakutat
    {  447, 6001,  137686,  5, }, // Antarctica/Casey
    {  447, 6094,  118616,  5, }, // Antarctica/Davis
    {  447, 6133,  137691, 14, }, // Antarctica/DumontDUrville
    {  447, 8568,    2249,  8, }, // Antarctica/Macquarie
    {  447, 7066,    2257,  6, }, // Antarctica/Mawson
    {  447, 7113,  137705,  9, }, // Antarctica/McMurdo
    {  447, 8589,    2272,  6, }, // Antarctica/Palmer
    {  447, 7226,    2278,  6, }, // Antarctica/Rothera
    {  447, 7295,   28642,  4, }, // Antarctica/Syowa
    {  447, 8607,    2289,  5, }, // Antarctica/Troll
    {  447, 7374,   34957,  6, }, // Antarctica/Vostok
    {  447, 1429,  137714, 11, }, // Arctic/Longyearbyen
    {  447, 5570,    2310,  4, }, // Asia/Aden
    {  447, 5395,    2314,  6, }, // Asia/Almaty
    {  447, 3968,    2320,  5, }, // Asia/Amman
    {  447, 5511,    2325,  6, }, // Asia/Anadyr
    {  447, 5523,    2331,  5, }, // Asia/Aqtau
    {  447, 5368,  137725,  6, }, // Asia/Aqtobe
    {  447,  964,    2342,  7, }, // Asia/Ashgabat
    {  447, 8624,    2349,  6, }, // Asia/Atyrau
    {  447, 3607,    2355,  6, }, // Asia/Baghdad
    {  447, 5534,   37080,  7, }, // Asia/Bahrain
    {  447, 3653,    2368,  4, }, // Asia/Baku
    {  447, 4466,    2372,  7, }, // Asia/Bangkok
    {  447, 3571,    2379,  7, }, // Asia/Barnaul
    {  447, 4086,   28695,  6, }, // Asia/Beirut
    {  447, 3778,    2392,  6, }, // Asia/Bishkek
    {  447, 5989,   28707,  6, }, // Asia/Brunei
    {  447, 4601,    2404,  4, }, // Asia/Chita
    {  447, 4511,    2408,  7, }, // Asia/Colombo
    {  447, 4540,    2415,  6, }, // Asia/Damascus
    {  447, 1093,     711,  4, }, // Asia/Dhaka
    {  447, 6192,    2426,  4, }, // Asia/Dili
    {  447, 3596,   28728,  5, }, // Asia/Dubai
    {  447, 6159,    2435,  7, }, // Asia/Dushanbe
    {  447, 8636,    2442,  9, }, // Asia/Famagusta
    {  447, 8651,    2451,  4, }, // Asia/Gaza
    {  447, 4795,   35018,  6, }, // Asia/Hebron
    {  447, 1259,  137731, 13, }, // Asia/Ho_Chi_Minh
    {  447, 2704,   35033,  9, }, // Asia/Hong_Kong
    {  447, 4771,    2475,  4, }, // Asia/Hovd
    {  447, 4184,  137744,  5, }, // Asia/Irkutsk
    {  447, 7006,  118657,  7, }, // Asia/Jakarta
    {  447, 6992,  137749,  6, }, // Asia/Jayapura
    {  447, 1290,   37094,  9, }, // Asia/Jerusalem
    {  447, 3560,    2510,  5, }, // Asia/Kabul
    {  447, 4316,    2529,  8, }, // Asia/Kamchatka
    {  447, 4239,    2540,  6, }, // Asia/Karachi
    {  447, 1185,    2546,  8, }, // Asia/Kathmandu
    {  447, 8661,  118672,  7, }, // Asia/Khandyga
    {  447,  992,   52082,  9, }, // Asia/Kolkata
    {  447, 4197,   37110, 10, }, // Asia/Krasnoyarsk
    {  447, 4859,   35090, 12, }, // Asia/Kuala_Lumpur
    {  447, 5976,    2592,  5, }, // Asia/Kuching
    {  447, 5547,   28825,  6, }, // Asia/Kuwait
    {  447, 1211,    2604,  5, }, // Asia/Macau
    {  447, 4017,    2609,  7, }, // Asia/Magadan
    {  447, 1349,  118684,  8, }, // Asia/Makassar
    {  447, 7182,    2623,  6, }, // Asia/Manila
    {  447, 6920,   35113,  6, }, // Asia/Muscat
    {  447, 2554,  106464,  7, }, // Asia/Nicosia
    {  447, 8675,    2642, 11, }, // Asia/Novokuznetsk
    {  447, 4135,    2653, 11, }, // Asia/Novosibirsk
    {  447, 4229,    2664,  4, }, // Asia/Omsk
    {  447, 7145,   35138,  4, }, // Asia/Oral
    {  447, 6961,   52091,  8, }, // Asia/Phnom_Penh
    {  447, 8693,    2681,  9, }, // Asia/Pontianak
    {  447, 4214,  137755,  6, }, // Asia/Pyongyang
    {  447, 5559,    2697,  5, }, // Asia/Qatar
    {  447, 8708,   37137,  8, }, // Asia/Qostanay
    {  447, 4269,    2710,  8, }, // Asia/Qyzylorda
    {  447, 3584,   37154,  4, }, // Asia/Riyadh
    {  447, 4408,    2732,  7, }, // Asia/Sakhalin
    {  447, 7245,    2739,  9, }, // Asia/Samarkand
    {  447, 3279,  137761,  4, }, // Asia/Seoul
    {  447, 1053,   28933,  6, }, // Asia/Shanghai
    {  447, 3300,    2758,  8, }, // Asia/Singapore
    {  447, 4297,    2766, 13, }, // Asia/Srednekolymsk
    {  447, 3263,   61028,  6, }, // Asia/Taipei
    {  447, 4781,    2785,  7, }, // Asia/Tashkent
    {  447, 3915,   35206,  7, }, // Asia/Tbilisi
    {  447, 2772,    2796,  7, }, // Asia/Tehran
    {  447, 1317,    2803,  7, }, // Asia/Thimphu
    {  447, 2821,    2810,  5, }, // Asia/Tokyo
    {  447, 4572,    2815,  5, }, // Asia/Tomsk
    {  447, 1021,  137765, 11, }, // Asia/Ulaanbaatar
    {  447, 1159,  137776,  6, }, // Asia/Urumqi
    {  447, 8722,    2836,  8, }, // Asia/Ust-Nera
    {  447, 6977,   52134,  8, }, // Asia/Vientiane
    {  447, 4710,    2851, 11, }, // Asia/Vladivostok
    {  447, 4828,   37183,  6, }, // Asia/Yakutsk
    {  447, 1235,   29043,  6, }, // Asia/Yangon
    {  447, 3883,    2875, 12, }, // Asia/Yekaterinburg
    {  447, 3747,    2887,  6, }, // Asia/Yerevan
    {  447, 3663,    2893,  4, }, // Atlantic/Azores
    {  447, 5671,   35284,  7, }, // Atlantic/Bermuda
    {  447, 6628,  118711,  6, }, // Atlantic/Canary
    {  447, 3727,    2944, 10, }, // Atlantic/Cape_Verde
    {  447, 1395,    2954,  5, }, // Atlantic/Faroe
    {  447, 8736,   37264,  7, }, // Atlantic/Madeira
    {  447, 2748,   52214,  9, }, // Atlantic/Reykjavik
    {  447, 7272,  137782, 12, }, // Atlantic/South_Georgia
    {  447, 6821,  137794, 11, }, // Atlantic/St_Helena
    {  447, 6644,   52256,  6, }, // Atlantic/Stanley
    {  447, 1670,    3035,  8, }, // Australia/Adelaide
    {  447, 1635,    3043,  7, }, // Australia/Brisbane
    {  447, 1799,  137805, 11, }, // Australia/Broken_Hill
    {  447, 1583,    3061,  6, }, // Australia/Darwin
    {  447, 3637,  137816,  5, }, // Australia/Eucla
    {  447, 1516,    3072,  6, }, // Australia/Hobart
    {  447, 8753,  137821,  8, }, // Australia/Lindeman
    {  447, 1547,  137829,  8, }, // Australia/Lord_Howe
    {  447, 1727,   29252,  8, }, // Australia/Melbourne
    {  447, 1762,  137837,  4, }, // Australia/Perth
    {  447, 1463,   52276,  6, }, // Australia/Sydney
    {  447, 6441,    3111,  9, }, // Europe/Amsterdam
    {  447, 6216,    3120,  7, }, // Europe/Andorra
    {  447, 3620,   29281,  9, }, // Europe/Astrakhan
    {  447, 2300,    3135,  4, }, // Europe/Athens
    {  447, 6336,    3139,  7, }, // Europe/Belgrade
    {  447, 4757,    3146,  6, }, // Europe/Berlin
    {  447, 6488,    3152,  9, }, // Europe/Bratislava
    {  447, 2167,   52296,  8, }, // Europe/Brussels
    {  447, 3928,    3170,  8, }, // Europe/Bucharest
    {  447, 3806,    3178,  8, }, // Europe/Budapest
    {  447, 8772,   35437,  8, }, // Europe/Busingen
    {  447, 2583,  137841,  7, }, // Europe/Chisinau
    {  447, 6289,   35452, 10, }, // Europe/Copenhagen
    {  447, 2338,    3226,  6, }, // Europe/Dublin
    {  447, 6307,    3232,  9, }, // Europe/Gibraltar
    {  447, 8788,    3241,  6, }, // Europe/Guernsey
    {  447, 6612,   52333,  8, }, // Europe/Helsinki
    {  447, 8804,  137848,  8, }, // Europe/Isle_of_Man
    {  447, 1130,   35484,  8, }, // Europe/Istanbul
    {  447, 8823,  118769,  5, }, // Europe/Jersey
    {  447, 3979,    3276, 11, }, // Europe/Kaliningrad
    {  447, 8929,    3287,  5, }, // Europe/Kirov
    {  447, 2527,    3292,  4, }, // Europe/Kyiv
    {  447, 3213,  137856,  6, }, // Europe/Lisbon
    {  447, 6506,   29432,  7, }, // Europe/Ljubljana
    {  447, 2501,    3313,  6, }, // Europe/London
    {  447, 6365,   29445, 10, }, // Europe/Luxembourg
    {  447, 4892,    3330,  6, }, // Europe/Madrid
    {  447, 6397,   29461,  6, }, // Europe/Malta
    {  447, 6582,   35532,  8, }, // Europe/Mariehamn
    {  447, 3693,    3350,  5, }, // Europe/Minsk
    {  447, 6410,    3355,  6, }, // Europe/Monaco
    {  447, 3537,    3361,  6, }, // Europe/Moscow
    {  447, 6458,    3367,  4, }, // Europe/Oslo
    {  447, 4284,  137862,  5, }, // Europe/Paris
    {  447, 6424,    3376,  9, }, // Europe/Podgorica
    {  447, 6275,    3385,  5, }, // Europe/Prague
    {  447, 8837,    3390,  4, }, // Europe/Riga
    {  447, 6324,  137867,  3, }, // Europe/Rome
    {  447, 4331,    3397,  6, }, // Europe/Samara
    {  447, 6470,    3403, 10, }, // Europe/San_Marino
    {  447, 6245,    3413,  7, }, // Europe/Sarajevo
    {  447, 4451,    3420,  7, }, // Europe/Saratov
    {  447, 8849,   52357, 11, }, // Europe/Simferopol
    {  447, 6383,   52368,  6, }, // Europe/Skopje
    {  447, 6599,    3442,  4, }, // Europe/Sofia
    {  447, 6523,   35585,  8, }, // Europe/Stockholm
    {  447, 8867,    3455,  6, }, // Europe/Tallinn
    {  447, 6202,    3461,  6, }, // Europe/Tirane
    {  447, 8882,   52383,  9, }, // Europe/Ulyanovsk
    {  447, 6352,   35606,  5, }, // Europe/Vaduz
    {  447, 6567,    3481,  7, }, // Europe/Vatican
    {  447, 6231,    3488,  4, }, // Europe/Vienna
    {  447, 8899,   52392,  7, }, // Europe/Vilnius
    {  447, 4727,    3499,  9, }, // Europe/Volgograd
    {  447, 3190,    3508,  6, }, // Europe/Warsaw
    {  447, 6261,    3515,  6, }, // Europe/Zagreb
    {  447, 6540,   37442,  5, }, // Europe/Zurich
    {  447, 5078,   35625, 12, }, // Indian/Antananarivo
    {  447, 6947,    3539,  5, }, // Indian/Chagos
    {  447, 6046,  118785,  7, }, // Indian/Christmas
    {  447, 6063,    3556,  5, }, // Indian/Cocos
    {  447, 5029,   35642,  6, }, // Indian/Comoro
    {  447, 6661,    3578,  8, }, // Indian/Kerguelen
    {  447, 7260,   35648,  4, }, // Indian/Mahe
    {  447, 7050,   52464,  7, }, // Indian/Maldives
    {  447, 4069,    3595,  7, }, // Indian/Mauritius
    {  447, 5098,   56716,  7, }, // Indian/Mayotte
    {  447, 7211,   37501,  7, }, // Indian/Reunion
    {  447, 4422,    3616,  4, }, // Pacific/Apia
    {  447,  932,  137870,  7, }, // Pacific/Auckland
    {  447, 3706,   52487,  9, }, // Pacific/Bougainville
    {  447, 3015,    3634,  5, }, // Pacific/Chatham
    {  447, 3157,  137877,  4, }, // Pacific/Chuuk
    {  447, 2237,  118802,  5, }, // Pacific/Easter
    {  447, 7360,   52541,  5, }, // Pacific/Efate
    {  447, 7327,    3659,  7, }, // Pacific/Fakaofo
    {  447, 3902,  118807,  4, }, // Pacific/Fiji
    {  447, 7343,    3670,  8, }, // Pacific/Funafuti
    {  447, 6678,    3678,  9, }, // Pacific/Galapagos
    {  447, 6696,  137881,  7, }, // Pacific/Gambier
    {  447, 3822,    3713, 11, }, // Pacific/Guadalcanal
    {  447, 6018,    3724,  4, }, // Pacific/Guam
    {  447, 2723,   35730,  8, }, // Pacific/Honolulu
    {  447, 3049,   52590,  9, }, // Pacific/Kanton
    {  447, 3998,    3734, 10, }, // Pacific/Kiritimati
    {  447, 7019,  137888,  5, }, // Pacific/Kosrae
    {  447, 2842,  137893,  9, }, // Pacific/Kwajalein
    {  447, 4877,  118826,  6, }, // Pacific/Majuro
    {  447, 4051,  118832,  6, }, // Pacific/Marquesas
    {  447, 8914,   52622,  6, }, // Pacific/Midway
    {  447, 7084,    3798,  5, }, // Pacific/Nauru
    {  447, 7132,   52636,  4, }, // Pacific/Niue
    {  447, 4168,    3807,  7, }, // Pacific/Norfolk
    {  447, 7098,    3814,  5, }, // Pacific/Noumea
    {  447, 3126,   35774,  9, }, // Pacific/Pago_Pago
    {  447, 7155,    3828,  5, }, // Pacific/Palau
    {  447, 7194,   56789,  7, }, // Pacific/Pitcairn
    {  447, 3096,   37603,  6, }, // Pacific/Pohnpei
    {  447, 4807,    3846, 11, }, // Pacific/Port_Moresby
    {  447, 6076,    3857,  9, }, // Pacific/Rarotonga
    {  447, 6031,   29935,  6, }, // Pacific/Saipan
    {  447, 7312,    3872,  5, }, // Pacific/Tahiti
    {  447, 6712,    3877,  6, }, // Pacific/Tarawa
    {  447, 4583,    3883,  9, }, // Pacific/Tongatapu
    {  447, 7392,  137902,  3, }, // Pacific/Wake
    {  447, 7405,    3896,  6, }, // Pacific/Wallis
    {  448,   14,       0,  6, }, // Africa/Asmara Mongolian/Mongolian/China
    {  448, 4435,       6,  8, }, // Africa/Sao_Tome
    {  448, 4252,      14,  8, }, // America/Asuncion
    {  448,  237,      22,  8, }, // America/Atikokan
    {  448, 7623,      30, 17, }, // America/Bahia_Banderas
    {  448, 3868,      47,  6, }, // America/Cancun
    {  448, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  448, 5723,      66,  7, }, // America/Curacao
    {  448,  481,      73, 13, }, // America/Indiana/Knox
    {  448, 7961,      86, 16, }, // America/Indiana/Marengo
    {  448, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  448, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  448, 8038,     139, 14, }, // America/Indiana/Vevay
    {  448, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  448, 8086,     171, 16, }, // America/Indiana/Winamac
    {  448, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  448, 5932,     207, 22, }, // America/Lower_Princes
    {  448, 8220,     229,  6, }, // America/Merida
    {  448, 1850,     235, 19, }, // America/Noronha
    {  448, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  448, 8329,     274, 20, }, // America/North_Dakota/Center
    {  448, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  448,  393,     317,  4, }, // America/Nuuk
    {  448, 6899,     321, 16, }, // America/Scoresbysund
    {  448, 8508,     337, 14, }, // America/St_Barthelemy
    {  448, 2061,     351, 10, }, // America/St_Johns
    {  448, 5863,     361,  9, }, // America/St_Kitts
    {  448, 5880,     370,  9, }, // America/St_Lucia
    {  448,  855,     379, 10, }, // America/St_Thomas
    {  448, 5913,     389, 11, }, // America/St_Vincent
    {  448, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  448, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  448, 1185,     448,  9, }, // Asia/Kathmandu
    {  448,  992,     457,  7, }, // Asia/Kolkata
    {  448, 1211,     464,  5, }, // Asia/Macau
    {  448, 1235,     477,  6, }, // Asia/Yangon
    {  448, 1395,     483,  5, }, // Atlantic/Faroe
    {  448, 6821,     488, 10, }, // Atlantic/St_Helena
    {  448, 2527,     514,  4, }, // Europe/Kyiv
    {  448, 7211,     547,  7, }, // Indian/Reunion
    {  448, 3157,     554,  5, }, // Pacific/Chuuk
    {  448, 3096,     586,  7, }, // Pacific/Pohnpei
    {  449, 4771,  137905,  5, }, // Asia/Hovd Mongolian/Mongolian/Mongolia
    {  449, 1021,  137910, 13, }, // Asia/Ulaanbaatar
    {  450,   14,       0,  6, }, // Africa/Asmara Morisyen/Latin/Mauritius
    {  450, 4435,       6,  8, }, // Africa/Sao_Tome
    {  450, 4252,      14,  8, }, // America/Asuncion
    {  450,  237,      22,  8, }, // America/Atikokan
    {  450, 7623,      30, 17, }, // America/Bahia_Banderas
    {  450, 3868,      47,  6, }, // America/Cancun
    {  450, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  450, 5723,      66,  7, }, // America/Curacao
    {  450,  481,      73, 13, }, // America/Indiana/Knox
    {  450, 7961,      86, 16, }, // America/Indiana/Marengo
    {  450, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  450, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  450, 8038,     139, 14, }, // America/Indiana/Vevay
    {  450, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  450, 8086,     171, 16, }, // America/Indiana/Winamac
    {  450, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  450, 5932,     207, 22, }, // America/Lower_Princes
    {  450, 8220,     229,  6, }, // America/Merida
    {  450, 1850,     235, 19, }, // America/Noronha
    {  450, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  450, 8329,     274, 20, }, // America/North_Dakota/Center
    {  450, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  450,  393,     317,  4, }, // America/Nuuk
    {  450, 6899,     321, 16, }, // America/Scoresbysund
    {  450, 8508,     337, 14, }, // America/St_Barthelemy
    {  450, 2061,     351, 10, }, // America/St_Johns
    {  450, 5863,     361,  9, }, // America/St_Kitts
    {  450, 5880,     370,  9, }, // America/St_Lucia
    {  450,  855,     379, 10, }, // America/St_Thomas
    {  450, 5913,     389, 11, }, // America/St_Vincent
    {  450, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  450, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  450, 1185,     448,  9, }, // Asia/Kathmandu
    {  450,  992,     457,  7, }, // Asia/Kolkata
    {  450, 1211,     464,  5, }, // Asia/Macau
    {  450, 1235,     477,  6, }, // Asia/Yangon
    {  450, 1395,     483,  5, }, // Atlantic/Faroe
    {  450, 6821,     488, 10, }, // Atlantic/St_Helena
    {  450, 2527,     514,  4, }, // Europe/Kyiv
    {  450, 7211,     547,  7, }, // Indian/Reunion
    {  450, 3157,     554,  5, }, // Pacific/Chuuk
    {  450, 3096,     586,  7, }, // Pacific/Pohnpei
    {  451,   14,       0,  6, }, // Africa/Asmara Mundang/Latin/Cameroon
    {  451, 4435,       6,  8, }, // Africa/Sao_Tome
    {  451, 4252,      14,  8, }, // America/Asuncion
    {  451,  237,      22,  8, }, // America/Atikokan
    {  451, 7623,      30, 17, }, // America/Bahia_Banderas
    {  451, 3868,      47,  6, }, // America/Cancun
    {  451, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  451, 5723,      66,  7, }, // America/Curacao
    {  451,  481,      73, 13, }, // America/Indiana/Knox
    {  451, 7961,      86, 16, }, // America/Indiana/Marengo
    {  451, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  451, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  451, 8038,     139, 14, }, // America/Indiana/Vevay
    {  451, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  451, 8086,     171, 16, }, // America/Indiana/Winamac
    {  451, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  451, 5932,     207, 22, }, // America/Lower_Princes
    {  451, 8220,     229,  6, }, // America/Merida
    {  451, 1850,     235, 19, }, // America/Noronha
    {  451, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  451, 8329,     274, 20, }, // America/North_Dakota/Center
    {  451, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  451,  393,     317,  4, }, // America/Nuuk
    {  451, 6899,     321, 16, }, // America/Scoresbysund
    {  451, 8508,     337, 14, }, // America/St_Barthelemy
    {  451, 2061,     351, 10, }, // America/St_Johns
    {  451, 5863,     361,  9, }, // America/St_Kitts
    {  451, 5880,     370,  9, }, // America/St_Lucia
    {  451,  855,     379, 10, }, // America/St_Thomas
    {  451, 5913,     389, 11, }, // America/St_Vincent
    {  451, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  451, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  451, 1185,     448,  9, }, // Asia/Kathmandu
    {  451,  992,     457,  7, }, // Asia/Kolkata
    {  451, 1211,     464,  5, }, // Asia/Macau
    {  451, 1235,     477,  6, }, // Asia/Yangon
    {  451, 1395,     483,  5, }, // Atlantic/Faroe
    {  451, 6821,     488, 10, }, // Atlantic/St_Helena
    {  451, 2527,     514,  4, }, // Europe/Kyiv
    {  451, 7211,     547,  7, }, // Indian/Reunion
    {  451, 3157,     554,  5, }, // Pacific/Chuuk
    {  451, 3096,     586,  7, }, // Pacific/Pohnpei
    {  452,   14,       0,  6, }, // Africa/Asmara Muscogee/Latin/United States
    {  452, 4435,       6,  8, }, // Africa/Sao_Tome
    {  452, 4252,      14,  8, }, // America/Asuncion
    {  452,  237,      22,  8, }, // America/Atikokan
    {  452, 7623,      30, 17, }, // America/Bahia_Banderas
    {  452, 3868,      47,  6, }, // America/Cancun
    {  452, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  452, 5723,      66,  7, }, // America/Curacao
    {  452,  481,      73, 13, }, // America/Indiana/Knox
    {  452, 7961,      86, 16, }, // America/Indiana/Marengo
    {  452, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  452, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  452, 8038,     139, 14, }, // America/Indiana/Vevay
    {  452, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  452, 8086,     171, 16, }, // America/Indiana/Winamac
    {  452, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  452, 5932,     207, 22, }, // America/Lower_Princes
    {  452, 8220,     229,  6, }, // America/Merida
    {  452, 1850,     235, 19, }, // America/Noronha
    {  452, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  452, 8329,     274, 20, }, // America/North_Dakota/Center
    {  452, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  452,  393,     317,  4, }, // America/Nuuk
    {  452, 6899,     321, 16, }, // America/Scoresbysund
    {  452, 8508,     337, 14, }, // America/St_Barthelemy
    {  452, 2061,     351, 10, }, // America/St_Johns
    {  452, 5863,     361,  9, }, // America/St_Kitts
    {  452, 5880,     370,  9, }, // America/St_Lucia
    {  452,  855,     379, 10, }, // America/St_Thomas
    {  452, 5913,     389, 11, }, // America/St_Vincent
    {  452, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  452, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  452, 1185,     448,  9, }, // Asia/Kathmandu
    {  452,  992,     457,  7, }, // Asia/Kolkata
    {  452, 1211,     464,  5, }, // Asia/Macau
    {  452, 1235,     477,  6, }, // Asia/Yangon
    {  452, 1395,     483,  5, }, // Atlantic/Faroe
    {  452, 6821,     488, 10, }, // Atlantic/St_Helena
    {  452, 2527,     514,  4, }, // Europe/Kyiv
    {  452, 7211,     547,  7, }, // Indian/Reunion
    {  452, 3157,     554,  5, }, // Pacific/Chuuk
    {  452, 3096,     586,  7, }, // Pacific/Pohnpei
    {  453,   14,       0,  6, }, // Africa/Asmara Nama/Latin/Namibia
    {  453, 4435,       6,  8, }, // Africa/Sao_Tome
    {  453, 4252,      14,  8, }, // America/Asuncion
    {  453,  237,      22,  8, }, // America/Atikokan
    {  453, 7623,      30, 17, }, // America/Bahia_Banderas
    {  453, 3868,      47,  6, }, // America/Cancun
    {  453, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  453, 5723,      66,  7, }, // America/Curacao
    {  453,  481,      73, 13, }, // America/Indiana/Knox
    {  453, 7961,      86, 16, }, // America/Indiana/Marengo
    {  453, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  453, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  453, 8038,     139, 14, }, // America/Indiana/Vevay
    {  453, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  453, 8086,     171, 16, }, // America/Indiana/Winamac
    {  453, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  453, 5932,     207, 22, }, // America/Lower_Princes
    {  453, 8220,     229,  6, }, // America/Merida
    {  453, 1850,     235, 19, }, // America/Noronha
    {  453, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  453, 8329,     274, 20, }, // America/North_Dakota/Center
    {  453, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  453,  393,     317,  4, }, // America/Nuuk
    {  453, 6899,     321, 16, }, // America/Scoresbysund
    {  453, 8508,     337, 14, }, // America/St_Barthelemy
    {  453, 2061,     351, 10, }, // America/St_Johns
    {  453, 5863,     361,  9, }, // America/St_Kitts
    {  453, 5880,     370,  9, }, // America/St_Lucia
    {  453,  855,     379, 10, }, // America/St_Thomas
    {  453, 5913,     389, 11, }, // America/St_Vincent
    {  453, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  453, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  453, 1185,     448,  9, }, // Asia/Kathmandu
    {  453,  992,     457,  7, }, // Asia/Kolkata
    {  453, 1211,     464,  5, }, // Asia/Macau
    {  453, 1235,     477,  6, }, // Asia/Yangon
    {  453, 1395,     483,  5, }, // Atlantic/Faroe
    {  453, 6821,     488, 10, }, // Atlantic/St_Helena
    {  453, 2527,     514,  4, }, // Europe/Kyiv
    {  453, 7211,     547,  7, }, // Indian/Reunion
    {  453, 3157,     554,  5, }, // Pacific/Chuuk
    {  453, 3096,     586,  7, }, // Pacific/Pohnpei
    {  454,   14,       0,  6, }, // Africa/Asmara Navajo/Latin/United States
    {  454, 4435,       6,  8, }, // Africa/Sao_Tome
    {  454, 4252,      14,  8, }, // America/Asuncion
    {  454,  237,      22,  8, }, // America/Atikokan
    {  454, 7623,      30, 17, }, // America/Bahia_Banderas
    {  454, 3868,      47,  6, }, // America/Cancun
    {  454, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  454, 5723,      66,  7, }, // America/Curacao
    {  454,  481,      73, 13, }, // America/Indiana/Knox
    {  454, 7961,      86, 16, }, // America/Indiana/Marengo
    {  454, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  454, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  454, 8038,     139, 14, }, // America/Indiana/Vevay
    {  454, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  454, 8086,     171, 16, }, // America/Indiana/Winamac
    {  454, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  454, 5932,     207, 22, }, // America/Lower_Princes
    {  454, 8220,     229,  6, }, // America/Merida
    {  454, 1850,     235, 19, }, // America/Noronha
    {  454, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  454, 8329,     274, 20, }, // America/North_Dakota/Center
    {  454, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  454,  393,     317,  4, }, // America/Nuuk
    {  454, 6899,     321, 16, }, // America/Scoresbysund
    {  454, 8508,     337, 14, }, // America/St_Barthelemy
    {  454, 2061,     351, 10, }, // America/St_Johns
    {  454, 5863,     361,  9, }, // America/St_Kitts
    {  454, 5880,     370,  9, }, // America/St_Lucia
    {  454,  855,     379, 10, }, // America/St_Thomas
    {  454, 5913,     389, 11, }, // America/St_Vincent
    {  454, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  454, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  454, 1185,     448,  9, }, // Asia/Kathmandu
    {  454,  992,     457,  7, }, // Asia/Kolkata
    {  454, 1211,     464,  5, }, // Asia/Macau
    {  454, 1235,     477,  6, }, // Asia/Yangon
    {  454, 1395,     483,  5, }, // Atlantic/Faroe
    {  454, 6821,     488, 10, }, // Atlantic/St_Helena
    {  454, 2527,     514,  4, }, // Europe/Kyiv
    {  454, 7211,     547,  7, }, // Indian/Reunion
    {  454, 3157,     554,  5, }, // Pacific/Chuuk
    {  454, 3096,     586,  7, }, // Pacific/Pohnpei
    {  455, 6788,  137923,  8, }, // Africa/Abidjan Nepali/Devanagari/Nepal
    {  455, 6760,  137931,  5, }, // Africa/Accra
    {  455, 5059,  137936, 12, }, // Africa/Addis_Ababa
    {  455, 7420,  137948,  9, }, // Africa/Algiers
    {  455,   14,   29997,  7, }, // Africa/Asmara
    {  455,   44,  137957,  6, }, // Africa/Bamako
    {  455, 5257,  137963,  8, }, // Africa/Bangui
    {  455, 6746,   30016,  6, }, // Africa/Banjul
    {  455, 7435,  113427,  5, }, // Africa/Bissau
    {  455, 4957,  137971, 10, }, // Africa/Blantyre
    {  455, 5287,  137981, 12, }, // Africa/Brazzaville
    {  455, 4922,   30050, 10, }, // Africa/Bujumbura
    {  455, 2320,  137993,  5, }, // Africa/Cairo
    {  455, 4117,  137998, 12, }, // Africa/Casablanca
    {  455, 7449,   88340,  5, }, // Africa/Ceuta
    {  455, 6773,   88345,  8, }, // Africa/Conakry
    {  455, 6840,   30088,  5, }, // Africa/Dakar
    {  455, 5130,  138010, 12, }, // Africa/Dar_es_Salaam
    {  455, 5043,  138022,  7, }, // Africa/Djibouti
    {  455, 5243,  138029,  6, }, // Africa/Douala
    {  455, 5166,  138035,  9, }, // Africa/El_Aaiun
    {  455, 6853,  138044,  8, }, // Africa/Freetown
    {  455, 4906,  138052,  7, }, // Africa/Gaborone
    {  455, 5015,   88403,  5, }, // Africa/Harare
    {  455, 4479,  138059, 11, }, // Africa/Johannesburg
    {  455, 4499,   30154,  4, }, // Africa/Juba
    {  455, 5151,   30158,  8, }, // Africa/Kampala
    {  455, 4524,  138070,  7, }, // Africa/Khartoum
    {  455, 4973,   88424,  6, }, // Africa/Kigali
    {  455, 5306,   30179,  8, }, // Africa/Kinshasa
    {  455, 4744,   88437,  5, }, // Africa/Lagos
    {  455, 5336,  138077, 12, }, // Africa/Libreville
    {  455, 6869,   88452,  3, }, // Africa/Lome
    {  455, 5211,  138089,  8, }, // Africa/Luanda
    {  455, 4939,  138097, 10, }, // Africa/Lubumbashi
    {  455, 5001,   30221,  6, }, // Africa/Lusaka
    {  455, 5322,  138107,  6, }, // Africa/Malabo
    {  455, 4987,  138113,  6, }, // Africa/Maputo
    {  455, 5197,   30239,  6, }, // Africa/Maseru
    {  455, 5182,   88481,  7, }, // Africa/Mbabane
    {  455, 5113,  138119,  8, }, // Africa/Mogadishu
    {  455, 7034,  138127, 10, }, // Africa/Monrovia
    {  455, 3853,  138137,  7, }, // Africa/Nairobi
    {  455, 5271,  138144, 10, }, // Africa/Ndjamena
    {  455, 5354,  138154,  5, }, // Africa/Niamey
    {  455, 6803,  138159,  8, }, // Africa/Nouakchott
    {  455, 6727,  138167,  8, }, // Africa/Ouagadougou
    {  455, 5225,  138175, 11, }, // Africa/Porto-Novo
    {  455, 4435,  138186,  8, }, // Africa/Sao_Tome
    {  455, 2866,   88559,  8, }, // Africa/Tripoli
    {  455, 6554,  113566,  5, }, // Africa/Tunis
    {  455, 4152,  138194,  9, }, // Africa/Windhoek
    {  455,  132,  138203,  4, }, // America/Adak
    {  455, 3346,  138207,  8, }, // America/Anchorage
    {  455, 5607,  138215, 11, }, // America/Anguilla
    {  455, 5624,   30361,  9, }, // America/Antigua
    {  455, 4554,  138226,  9, }, // America/Araguaina
    {  455,  166,  138235, 12, }, // America/Argentina/Buenos_Aires
    {  455,   91,  138247, 10, }, // America/Argentina/Catamarca
    {  455,  270,  138257,  7, }, // America/Argentina/Cordoba
    {  455,  441,   30406,  5, }, // America/Argentina/Jujuy
    {  455, 7462,  113634,  9, }, // America/Argentina/La_Rioja
    {  455,  565,  138264,  8, }, // America/Argentina/Mendoza
    {  455, 7489,  138272, 14, }, // America/Argentina/Rio_Gallegos
    {  455, 7520,   30437,  6, }, // America/Argentina/Salta
    {  455, 7544,  138286,  9, }, // America/Argentina/San_Juan
    {  455, 5580,  138295,  8, }, // America/Argentina/San_Luis
    {  455, 7571,   30460,  7, }, // America/Argentina/Tucuman
    {  455, 7597,  138303,  6, }, // America/Argentina/Ushuaia
    {  455, 5640,  113677,  5, }, // America/Aruba
    {  455, 4252,  138309,  8, }, // America/Asuncion
    {  455,  237,  138317,  8, }, // America/Atikokan
    {  455, 3679,   30495,  6, }, // America/Bahia
    {  455, 7623,  138325, 16, }, // America/Bahia_Banderas
    {  455, 5654,  135873,  9, }, // America/Barbados
    {  455, 7646,   30526,  5, }, // America/Belem
    {  455, 5407,   30531,  5, }, // America/Belize
    {  455, 7660,  138341, 15, }, // America/Blanc-Sablon
    {  455, 7681,  138356, 10, }, // America/Boa_Vista
    {  455, 4361,   88760,  6, }, // America/Bogota
    {  455, 7699,  138366,  4, }, // America/Boise
    {  455, 7713,  138370, 14, }, // America/Cambridge_Bay
    {  455, 7735,  138384, 17, }, // America/Campo_Grande
    {  455, 3868,  113766,  6, }, // America/Cancun
    {  455, 4694,   30602,  7, }, // America/Caracas
    {  455, 4345,  138401,  8, }, // America/Cayenne
    {  455, 5496,  138409,  7, }, // America/Cayman
    {  455, 2260,   88811,  6, }, // America/Chicago
    {  455, 7756,  138416, 10, }, // America/Chihuahua
    {  455, 7774,  138426, 10, }, // America/Ciudad_Juarez
    {  455, 5422,  138436, 11, }, // America/Costa_Rica
    {  455, 7796,   30642,  8, }, // America/Creston
    {  455, 3791,  138447,  8, }, // America/Cuiaba
    {  455, 5723,   88859,  7, }, // America/Curacao
    {  455, 7812,  138455, 13, }, // America/Danmarkshavn
    {  455, 7833,  138468,  3, }, // America/Dawson
    {  455, 7848,  138471,  9, }, // America/Dawson_Creek
    {  455,  805,  138480,  6, }, // America/Denver
    {  455, 3465,  138486,  8, }, // America/Detroit
    {  455, 5739,   88904,  8, }, // America/Dominica
    {  455,  893,  138494,  9, }, // America/Edmonton
    {  455, 7869,  138503,  7, }, // America/Eirunepe
    {  455, 5441,  138510, 13, }, // America/El_Salvador
    {  455, 7886,  113888, 12, }, // America/Fort_Nelson
    {  455, 7906,  129361, 10, }, // America/Fortaleza
    {  455, 7924,   30760,  8, }, // America/Glace_Bay
    {  455, 6881,  138523,  6, }, // America/Goose_Bay
    {  455, 4612,  138529, 12, }, // America/Grand_Turk
    {  455, 5770,   30789,  8, }, // America/Grenada
    {  455, 5786,  138541,  9, }, // America/Guadeloupe
    {  455, 3760,   30806, 10, }, // America/Guatemala
    {  455, 4841,  138550,  9, }, // America/Guayaquil
    {  455, 6932,   30824,  6, }, // America/Guyana
    {  455, 1939,  138559, 13, }, // America/Halifax
    {  455, 2281,  138572,  5, }, // America/Havana
    {  455, 7942,  138577, 12, }, // America/Hermosillo
    {  455,  348,  138589, 14, }, // America/Indiana/Indianapolis
    {  455,  481,  138603, 15, }, // America/Indiana/Knox
    {  455, 7961,  138618, 19, }, // America/Indiana/Marengo
    {  455, 7985,  138637, 21, }, // America/Indiana/Petersburg
    {  455, 8012,  138658, 19, }, // America/Indiana/Tell_City
    {  455, 8038,  138677, 15, }, // America/Indiana/Vevay
    {  455, 8060,  138692,  9, }, // America/Indiana/Vincennes
    {  455, 8086,  138701, 18, }, // America/Indiana/Winamac
    {  455, 8110,  138719,  6, }, // America/Inuvik
    {  455,  660,  138725,  9, }, // America/Iqaluit
    {  455, 2799,  138734,  6, }, // America/Jamaica
    {  455, 5380,  138740,  5, }, // America/Juneau
    {  455,  521,  138745, 10, }, // America/Kentucky/Louisville
    {  455, 8125,  138755, 20, }, // America/Kentucky/Monticello
    {  455, 5704,  138775, 11, }, // America/Kralendijk
    {  455, 4376,   31058,  6, }, // America/La_Paz
    {  455, 7169,  114119,  4, }, // America/Lima
    {  455, 3239,  138786, 11, }, // America/Los_Angeles
    {  455, 5932,  138797, 23, }, // America/Lower_Princes
    {  455, 8153,  138820,  6, }, // America/Maceio
    {  455, 8168,  138826,  8, }, // America/Managua
    {  455, 1908,  138834,  6, }, // America/Manaus
    {  455, 5897,  138840,  8, }, // America/Marigot
    {  455, 5805,   89285,  9, }, // America/Martinique
    {  455, 8184,  138848, 12, }, // America/Matamoros
    {  455, 2917,  138860,  9, }, // America/Mazatlan
    {  455, 8202,   89312,  8, }, // America/Menominee
    {  455, 8220,   31158,  6, }, // America/Merida
    {  455, 8235,  138869, 11, }, // America/Metlakatla
    {  455, 2949,   89332, 13, }, // America/Mexico_City
    {  455, 4391,  138880,  8, }, // America/Miquelon
    {  455, 8254,  138888,  7, }, // America/Moncton
    {  455, 8270,  138895, 10, }, // America/Monterrey
    {  455, 4098,  138905, 12, }, // America/Montevideo
    {  455, 5824,  138917, 11, }, // America/Montserrat
    {  455, 5481,   31232,  7, }, // America/Nassau
    {  455, 2379,  138928,  9, }, // America/New_York
    {  455, 8288,   89314,  3, }, // America/Nome
    {  455, 1850,   89401,  8, }, // America/Noronha
    {  455, 8301,  138937, 19, }, // America/North_Dakota/Beulah
    {  455, 8329,  138956, 21, }, // America/North_Dakota/Center
    {  455, 8357,  138977, 24, }, // America/North_Dakota/New_Salem
    {  455,  393,   31315,  3, }, // America/Nuuk
    {  455, 8388,  114339,  7, }, // America/Ojinaga
    {  455, 2356,   31325,  6, }, // America/Panama
    {  455, 6173,  139001, 10, }, // America/Paramaribo
    {  455, 2973,  114355,  7, }, // America/Phoenix
    {  455, 3945,  139011, 16, }, // America/Port-au-Prince
    {  455, 5954,  139027, 15, }, // America/Port_of_Spain
    {  455, 8404,  139042, 13, }, // America/Porto_Velho
    {  455, 5843,  139055, 13, }, // America/Puerto_Rico
    {  455, 4030,  139068, 13, }, // America/Punta_Arenas
    {  455, 8424,  139081, 14, }, // America/Rankin_Inlet
    {  455, 8445,  139095,  6, }, // America/Recife
    {  455, 1995,   31431,  6, }, // America/Regina
    {  455, 8460,  139101,  7, }, // America/Resolute
    {  455,  695,  139108, 13, }, // America/Rio_Branco
    {  455, 8477,  139121,  9, }, // America/Santarem
    {  455, 2201,  139130,  9, }, // America/Santiago
    {  455, 6111,  139139, 15, }, // America/Santo_Domingo
    {  455, 1878,   89642,  9, }, // America/Sao_Paulo
    {  455, 6899,  139154, 17, }, // America/Scoresbysund
    {  455, 8494,   89664,  6, }, // America/Sitka
    {  455, 8508,  139171, 16, }, // America/St_Barthelemy
    {  455, 2061,  139187, 11, }, // America/St_Johns
    {  455, 5863,  139198, 12, }, // America/St_Kitts
    {  455, 5880,  139210, 12, }, // America/St_Lucia
    {  455,  855,  139222, 10, }, // America/St_Thomas
    {  455, 5913,  139232, 15, }, // America/St_Vincent
    {  455, 8530,  139247, 14, }, // America/Swift_Current
    {  455, 5461,  136451, 12, }, // America/Tegucigalpa
    {  455, 5756,  114553,  4, }, // America/Thule
    {  455,  313,   89766,  7, }, // America/Tijuana
    {  455,  608,  139261,  8, }, // America/Toronto
    {  455, 5688,  139269,  8, }, // America/Tortola
    {  455, 2093,  139277,  9, }, // America/Vancouver
    {  455, 2144,  139286, 10, }, // America/Whitehorse
    {  455,  734,  139296,  9, }, // America/Winnipeg
    {  455, 8552,  114591,  7, }, // America/Yakutat
    {  455, 6001,  139305,  4, }, // Antarctica/Casey
    {  455, 6094,   31677,  5, }, // Antarctica/Davis
    {  455, 6133,  139309, 20, }, // Antarctica/DumontDUrville
    {  455, 8568,  139329,  8, }, // Antarctica/Macquarie
    {  455, 7066,  139337,  5, }, // Antarctica/Mawson
    {  455, 7113,  139342,  9, }, // Antarctica/McMurdo
    {  455, 8589,  114636,  7, }, // Antarctica/Palmer
    {  455, 7226,   89867,  6, }, // Antarctica/Rothera
    {  455, 7295,  139351,  4, }, // Antarctica/Syowa
    {  455, 8607,   89879,  5, }, // Antarctica/Troll
    {  455, 7374,  139355,  7, }, // Antarctica/Vostok
    {  455, 1429,  139362, 11, }, // Arctic/Longyearbyen
    {  455, 5570,   31755,  4, }, // Asia/Aden
    {  455, 5395,   31759,  7, }, // Asia/Almaty
    {  455, 3968,   31766,  6, }, // Asia/Amman
    {  455, 5511,  139373,  7, }, // Asia/Anadyr
    {  455, 5523,  139380,  6, }, // Asia/Aqtau
    {  455, 5368,  139386,  6, }, // Asia/Aqtobe
    {  455,  964,  139392,  8, }, // Asia/Ashgabat
    {  455, 8624,  139400,  6, }, // Asia/Atyrau
    {  455, 3607,   89938,  5, }, // Asia/Baghdad
    {  455, 5534,  139406,  6, }, // Asia/Bahrain
    {  455, 3653,   88857,  4, }, // Asia/Baku
    {  455, 4466,  139412,  5, }, // Asia/Bangkok
    {  455, 3571,  114702,  5, }, // Asia/Barnaul
    {  455, 4086,  139417,  6, }, // Asia/Beirut
    {  455, 3778,  139423,  7, }, // Asia/Bishkek
    {  455, 5989,  139430,  7, }, // Asia/Brunei
    {  455, 4601,  129970,  4, }, // Asia/Chita
    {  455, 4511,  129974,  7, }, // Asia/Colombo
    {  455, 4540,  139437,  9, }, // Asia/Damascus
    {  455, 1093,   89991,  4, }, // Asia/Dhaka
    {  455, 6192,  139446,  6, }, // Asia/Dili
    {  455, 3596,  139452,  5, }, // Asia/Dubai
    {  455, 6159,  139457,  9, }, // Asia/Dushanbe
    {  455, 8636,   31889, 10, }, // Asia/Famagusta
    {  455, 8651,   31899,  4, }, // Asia/Gaza
    {  455, 4795,  139466,  7, }, // Asia/Hebron
    {  455, 1259,  139473, 15, }, // Asia/Ho_Chi_Minh
    {  455, 2704,  139488,  4, }, // Asia/Hong_Kong
    {  455, 4771,  139492,  5, }, // Asia/Hovd
    {  455, 4184,   90063, 10, }, // Asia/Irkutsk
    {  455, 7006,   31935,  8, }, // Asia/Jakarta
    {  455, 6992,   31943,  7, }, // Asia/Jayapura
    {  455, 1290,  136648,  8, }, // Asia/Jerusalem
    {  455, 3560,   90086,  5, }, // Asia/Kabul
    {  455, 4316,  139497,  8, }, // Asia/Kamchatka
    {  455, 4239,   31972,  5, }, // Asia/Karachi
    {  455, 1185,  139505, 10, }, // Asia/Kathmandu
    {  455, 8661,  139515,  8, }, // Asia/Khandyga
    {  455,  992,   90113,  7, }, // Asia/Kolkata
    {  455, 4197,   90120, 15, }, // Asia/Krasnoyarsk
    {  455, 4859,  139523, 11, }, // Asia/Kuala_Lumpur
    {  455, 5976,  139534,  5, }, // Asia/Kuching
    {  455, 5547,  114816,  5, }, // Asia/Kuwait
    {  455, 1211,  139539,  4, }, // Asia/Macau
    {  455, 4017,  139543,  7, }, // Asia/Magadan
    {  455, 1349,   32051,  9, }, // Asia/Makassar
    {  455, 7182,  114839,  5, }, // Asia/Manila
    {  455, 6920,  139550,  8, }, // Asia/Muscat
    {  455, 2554,   90160,  8, }, // Asia/Nicosia
    {  455, 8675,  139558, 12, }, // Asia/Novokuznetsk
    {  455, 4135,  139570, 13, }, // Asia/Novosibirsk
    {  455, 4229,   90197,  6, }, // Asia/Omsk
    {  455, 7145,   90203,  3, }, // Asia/Oral
    {  455, 6961,  139583,  9, }, // Asia/Phnom_Penh
    {  455, 8693,  139592, 10, }, // Asia/Pontianak
    {  455, 4214,  139602,  8, }, // Asia/Pyongyang
    {  455, 5559,  114879,  4, }, // Asia/Qatar
    {  455, 8708,  139610,  7, }, // Asia/Qostanay
    {  455, 4269,  130133, 10, }, // Asia/Qyzylorda
    {  455, 3584,   90256,  5, }, // Asia/Riyadh
    {  455, 4408,   32165,  7, }, // Asia/Sakhalin
    {  455, 7245,  139617,  8, }, // Asia/Samarkand
    {  455, 3279,   90273,  4, }, // Asia/Seoul
    {  455, 1053,  139625,  7, }, // Asia/Shanghai
    {  455, 3300,   32190,  8, }, // Asia/Singapore
    {  455, 4297,  139632, 17, }, // Asia/Srednekolymsk
    {  455, 3263,   32214,  5, }, // Asia/Taipei
    {  455, 4781,  139649,  9, }, // Asia/Tashkent
    {  455, 3915,   32228,  8, }, // Asia/Tbilisi
    {  455, 2772,  139658,  7, }, // Asia/Tehran
    {  455, 1317,  139665,  6, }, // Asia/Thimphu
    {  455, 2821,  136765,  6, }, // Asia/Tokyo
    {  455, 4572,  139671,  7, }, // Asia/Tomsk
    {  455, 1021,  139678,  8, }, // Asia/Ulaanbaatar
    {  455, 1159,   90343,  7, }, // Asia/Urumqi
    {  455, 8722,  139686,  9, }, // Asia/Ust-Nera
    {  455, 6977,  139695,  8, }, // Asia/Vientiane
    {  455, 4710,  139703, 13, }, // Asia/Vladivostok
    {  455, 4828,  139716,  9, }, // Asia/Yakutsk
    {  455, 1235,  139725,  7, }, // Asia/Yangon
    {  455, 3883,  139732, 11, }, // Asia/Yekaterinburg
    {  455, 3747,   32334,  7, }, // Asia/Yerevan
    {  455, 3663,  139743,  6, }, // Atlantic/Azores
    {  455, 5671,  115031,  7, }, // Atlantic/Bermuda
    {  455, 6628,  139749,  8, }, // Atlantic/Canary
    {  455, 3727,  139757,  9, }, // Atlantic/Cape_Verde
    {  455, 1395,  139766,  5, }, // Atlantic/Faroe
    {  455, 8736,  139771,  7, }, // Atlantic/Madeira
    {  455, 2748,  139778,  9, }, // Atlantic/Reykjavik
    {  455, 7272,  139787, 14, }, // Atlantic/South_Georgia
    {  455, 6821,  139801, 12, }, // Atlantic/St_Helena
    {  455, 6644,  139813, 10, }, // Atlantic/Stanley
    {  455, 1670,  139823,  6, }, // Australia/Adelaide
    {  455, 1635,  115089,  9, }, // Australia/Brisbane
    {  455, 1799,   90495, 10, }, // Australia/Broken_Hill
    {  455, 1583,   90505,  7, }, // Australia/Darwin
    {  455, 3637,  139829,  7, }, // Australia/Eucla
    {  455, 1516,  139836,  7, }, // Australia/Hobart
    {  455, 8753,  139843, 11, }, // Australia/Lindeman
    {  455, 1547,  139854,  9, }, // Australia/Lord_Howe
    {  455, 1727,  139863,  8, }, // Australia/Melbourne
    {  455, 1762,   90545,  4, }, // Australia/Perth
    {  455, 1463,  139871,  6, }, // Australia/Sydney
    {  455, 6441,  139877, 11, }, // Europe/Amsterdam
    {  455, 6216,  139888,  9, }, // Europe/Andorra
    {  455, 3620,  139897,  9, }, // Europe/Astrakhan
    {  455, 2300,   32526,  6, }, // Europe/Athens
    {  455, 6336,   32532,  8, }, // Europe/Belgrade
    {  455, 4757,   90580,  6, }, // Europe/Berlin
    {  455, 6488,  139906, 12, }, // Europe/Bratislava
    {  455, 2167,  130409,  8, }, // Europe/Brussels
    {  455, 3928,  139918,  9, }, // Europe/Bucharest
    {  455, 3806,   90617,  9, }, // Europe/Budapest
    {  455, 8772,  139927,  9, }, // Europe/Busingen
    {  455, 2583,  139936,  7, }, // Europe/Chisinau
    {  455, 6289,  139943, 11, }, // Europe/Copenhagen
    {  455, 2338,  115196,  6, }, // Europe/Dublin
    {  455, 6307,   32617, 11, }, // Europe/Gibraltar
    {  455, 8788,  139954,  8, }, // Europe/Guernsey
    {  455, 6612,  139962, 10, }, // Europe/Helsinki
    {  455, 8804,  139972, 13, }, // Europe/Isle_of_Man
    {  455, 1130,  139985,  9, }, // Europe/Istanbul
    {  455, 8823,   90701,  5, }, // Europe/Jersey
    {  455, 3979,  139994, 12, }, // Europe/Kaliningrad
    {  455, 8929,  140006,  5, }, // Europe/Kirov
    {  455, 2527,  140011,  3, }, // Europe/Kyiv
    {  455, 3213,  140014,  7, }, // Europe/Lisbon
    {  455, 6506,  140021,  9, }, // Europe/Ljubljana
    {  455, 2501,  140030,  5, }, // Europe/London
    {  455, 6365,  140035, 11, }, // Europe/Luxembourg
    {  455, 4892,  140046,  9, }, // Europe/Madrid
    {  455, 6397,   32731,  6, }, // Europe/Malta
    {  455, 6582,  140055, 13, }, // Europe/Mariehamn
    {  455, 3693,   32744,  7, }, // Europe/Minsk
    {  455, 6410,   90773,  6, }, // Europe/Monaco
    {  455, 3537,  140068,  5, }, // Europe/Moscow
    {  455, 6458,   90785,  5, }, // Europe/Oslo
    {  455, 4284,   32766,  5, }, // Europe/Paris
    {  455, 6424,  140073,  9, }, // Europe/Podgorica
    {  455, 6275,   90800,  5, }, // Europe/Prague
    {  455, 8837,   32784,  4, }, // Europe/Riga
    {  455, 6324,   90432,  3, }, // Europe/Rome
    {  455, 4331,   32791,  6, }, // Europe/Samara
    {  455, 6470,  140082, 10, }, // Europe/San_Marino
    {  455, 6245,  140092,  8, }, // Europe/Sarajevo
    {  455, 4451,  140100,  7, }, // Europe/Saratov
    {  455, 8849,  130580, 11, }, // Europe/Simferopol
    {  455, 6383,  140107,  7, }, // Europe/Skopje
    {  455, 6599,  115326,  6, }, // Europe/Sofia
    {  455, 6523,  140114,  9, }, // Europe/Stockholm
    {  455, 8867,  140123,  7, }, // Europe/Tallinn
    {  455, 6202,  130611,  6, }, // Europe/Tirane
    {  455, 8882,  140130, 12, }, // Europe/Ulyanovsk
    {  455, 6352,   32876,  5, }, // Europe/Vaduz
    {  455, 6567,  140142,  8, }, // Europe/Vatican
    {  455, 6231,  140150,  5, }, // Europe/Vienna
    {  455, 8899,  140155,  8, }, // Europe/Vilnius
    {  455, 4727,  140163, 11, }, // Europe/Volgograd
    {  455, 3190,  140174,  5, }, // Europe/Warsaw
    {  455, 6261,   32919,  7, }, // Europe/Zagreb
    {  455, 6540,  140179,  5, }, // Europe/Zurich
    {  455, 5078,  140184, 11, }, // Indian/Antananarivo
    {  455, 6947,  115401,  5, }, // Indian/Chagos
    {  455, 6046,  140195,  9, }, // Indian/Christmas
    {  455, 6063,   90978,  5, }, // Indian/Cocos
    {  455, 5029,   90983,  6, }, // Indian/Comoro
    {  455, 6661,  140204, 10, }, // Indian/Kerguelen
    {  455, 7260,   32983,  4, }, // Indian/Mahe
    {  455, 7050,  140214,  9, }, // Indian/Maldives
    {  455, 4069,  140223,  8, }, // Indian/Mauritius
    {  455, 5098,  137087,  8, }, // Indian/Mayotte
    {  455, 7211,  115425,  8, }, // Indian/Reunion
    {  455, 4422,  115433,  5, }, // Pacific/Apia
    {  455,  932,  140231,  9, }, // Pacific/Auckland
    {  455, 3706,  140240, 11, }, // Pacific/Bougainville
    {  455, 3015,   33040,  5, }, // Pacific/Chatham
    {  455, 3157,   33045,  3, }, // Pacific/Chuuk
    {  455, 2237,  140251,  5, }, // Pacific/Easter
    {  455, 7360,  140256,  5, }, // Pacific/Efate
    {  455, 7327,  115463,  7, }, // Pacific/Fakaofo
    {  455, 3902,   33069,  4, }, // Pacific/Fiji
    {  455, 7343,  140261,  8, }, // Pacific/Funafuti
    {  455, 6678,  140269,  8, }, // Pacific/Galapagos
    {  455, 6696,  140277, 10, }, // Pacific/Gambier
    {  455, 3822,  140287, 10, }, // Pacific/Guadalcanal
    {  455, 6018,   33111,  5, }, // Pacific/Guam
    {  455, 2723,   91111,  8, }, // Pacific/Honolulu
    {  455, 3049,  140297,  8, }, // Pacific/Kanton
    {  455, 3998,  140305, 10, }, // Pacific/Kiritimati
    {  455, 7019,  140315,  7, }, // Pacific/Kosrae
    {  455, 2842,  140322, 10, }, // Pacific/Kwajalein
    {  455, 4877,   91142,  6, }, // Pacific/Majuro
    {  455, 4051,   33162,  9, }, // Pacific/Marquesas
    {  455, 8914,   91156,  5, }, // Pacific/Midway
    {  455, 7084,  140332,  5, }, // Pacific/Nauru
    {  455, 7132,  140337,  4, }, // Pacific/Niue
    {  455, 4168,  140341,  7, }, // Pacific/Norfolk
    {  455, 7098,  140348,  6, }, // Pacific/Noumea
    {  455, 3126,   91182,  9, }, // Pacific/Pago_Pago
    {  455, 7155,  140354,  4, }, // Pacific/Palau
    {  455, 7194,  140358,  8, }, // Pacific/Pitcairn
    {  455, 3096,  140366,  6, }, // Pacific/Pohnpei
    {  455, 4807,   91201, 14, }, // Pacific/Port_Moresby
    {  455, 6076,  140372,  8, }, // Pacific/Rarotonga
    {  455, 6031,  140380,  5, }, // Pacific/Saipan
    {  455, 7312,   91230,  6, }, // Pacific/Tahiti
    {  455, 6712,  140385,  4, }, // Pacific/Tarawa
    {  455, 4583,  140389,  7, }, // Pacific/Tongatapu
    {  455, 7392,   33268,  3, }, // Pacific/Wake
    {  455, 7405,   33271,  5, }, // Pacific/Wallis
    {  457,   14,       0,  6, }, // Africa/Asmara Ngiemboon/Latin/Cameroon
    {  457, 4435,       6,  8, }, // Africa/Sao_Tome
    {  457, 4252,      14,  8, }, // America/Asuncion
    {  457,  237,      22,  8, }, // America/Atikokan
    {  457, 7623,      30, 17, }, // America/Bahia_Banderas
    {  457, 3868,      47,  6, }, // America/Cancun
    {  457, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  457, 5723,      66,  7, }, // America/Curacao
    {  457,  481,      73, 13, }, // America/Indiana/Knox
    {  457, 7961,      86, 16, }, // America/Indiana/Marengo
    {  457, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  457, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  457, 8038,     139, 14, }, // America/Indiana/Vevay
    {  457, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  457, 8086,     171, 16, }, // America/Indiana/Winamac
    {  457, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  457, 5932,     207, 22, }, // America/Lower_Princes
    {  457, 8220,     229,  6, }, // America/Merida
    {  457, 1850,     235, 19, }, // America/Noronha
    {  457, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  457, 8329,     274, 20, }, // America/North_Dakota/Center
    {  457, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  457,  393,     317,  4, }, // America/Nuuk
    {  457, 6899,     321, 16, }, // America/Scoresbysund
    {  457, 8508,     337, 14, }, // America/St_Barthelemy
    {  457, 2061,     351, 10, }, // America/St_Johns
    {  457, 5863,     361,  9, }, // America/St_Kitts
    {  457, 5880,     370,  9, }, // America/St_Lucia
    {  457,  855,     379, 10, }, // America/St_Thomas
    {  457, 5913,     389, 11, }, // America/St_Vincent
    {  457, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  457, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  457, 1185,     448,  9, }, // Asia/Kathmandu
    {  457,  992,     457,  7, }, // Asia/Kolkata
    {  457, 1211,     464,  5, }, // Asia/Macau
    {  457, 1235,     477,  6, }, // Asia/Yangon
    {  457, 1395,     483,  5, }, // Atlantic/Faroe
    {  457, 6821,     488, 10, }, // Atlantic/St_Helena
    {  457, 2527,     514,  4, }, // Europe/Kyiv
    {  457, 7211,     547,  7, }, // Indian/Reunion
    {  457, 3157,     554,  5, }, // Pacific/Chuuk
    {  457, 3096,     586,  7, }, // Pacific/Pohnpei
    {  458,   14,       0,  6, }, // Africa/Asmara Ngomba/Latin/Cameroon
    {  458, 4435,       6,  8, }, // Africa/Sao_Tome
    {  458, 4252,      14,  8, }, // America/Asuncion
    {  458,  237,      22,  8, }, // America/Atikokan
    {  458, 7623,      30, 17, }, // America/Bahia_Banderas
    {  458, 3868,      47,  6, }, // America/Cancun
    {  458, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  458, 5723,      66,  7, }, // America/Curacao
    {  458,  481,      73, 13, }, // America/Indiana/Knox
    {  458, 7961,      86, 16, }, // America/Indiana/Marengo
    {  458, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  458, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  458, 8038,     139, 14, }, // America/Indiana/Vevay
    {  458, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  458, 8086,     171, 16, }, // America/Indiana/Winamac
    {  458, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  458, 5932,     207, 22, }, // America/Lower_Princes
    {  458, 8220,     229,  6, }, // America/Merida
    {  458, 1850,     235, 19, }, // America/Noronha
    {  458, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  458, 8329,     274, 20, }, // America/North_Dakota/Center
    {  458, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  458,  393,     317,  4, }, // America/Nuuk
    {  458, 6899,     321, 16, }, // America/Scoresbysund
    {  458, 8508,     337, 14, }, // America/St_Barthelemy
    {  458, 2061,     351, 10, }, // America/St_Johns
    {  458, 5863,     361,  9, }, // America/St_Kitts
    {  458, 5880,     370,  9, }, // America/St_Lucia
    {  458,  855,     379, 10, }, // America/St_Thomas
    {  458, 5913,     389, 11, }, // America/St_Vincent
    {  458, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  458, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  458, 1185,     448,  9, }, // Asia/Kathmandu
    {  458,  992,     457,  7, }, // Asia/Kolkata
    {  458, 1211,     464,  5, }, // Asia/Macau
    {  458, 1235,     477,  6, }, // Asia/Yangon
    {  458, 1395,     483,  5, }, // Atlantic/Faroe
    {  458, 6821,     488, 10, }, // Atlantic/St_Helena
    {  458, 2527,     514,  4, }, // Europe/Kyiv
    {  458, 7211,     547,  7, }, // Indian/Reunion
    {  458, 3157,     554,  5, }, // Pacific/Chuuk
    {  458, 3096,     586,  7, }, // Pacific/Pohnpei
    {  459, 6788,  140396,  6, }, // Africa/Abidjan Nigerian Pidgin/Latin/Nigeria
    {  459, 6760,  140402,  4, }, // Africa/Accra
    {  459, 5059,  140406,  9, }, // Africa/Addis_Ababa
    {  459, 7420,  140415,  6, }, // Africa/Algiers
    {  459,   14,  140421,  6, }, // Africa/Asmara
    {  459,   44,  140427,  6, }, // Africa/Bamako
    {  459, 5257,  140433,  6, }, // Africa/Bangui
    {  459, 7435,    6313,  5, }, // Africa/Bissau
    {  459, 4957,  140439,  8, }, // Africa/Blantyre
    {  459, 5287,  140447,  8, }, // Africa/Brazzaville
    {  459, 4922,  140455,  9, }, // Africa/Bujumbura
    {  459, 2320,  140464,  5, }, // Africa/Cairo
    {  459, 4117,  140469, 10, }, // Africa/Casablanca
    {  459, 7449,  140479,  5, }, // Africa/Ceuta
    {  459, 6773,  140484,  7, }, // Africa/Conakry
    {  459, 6840,  140491,  5, }, // Africa/Dakar
    {  459, 5130,  140496, 13, }, // Africa/Dar_es_Salaam
    {  459, 5043,  140509,  6, }, // Africa/Djibouti
    {  459, 5243,  140515,  5, }, // Africa/Douala
    {  459, 5166,  140520,  7, }, // Africa/El_Aaiun
    {  459, 6853,  140527,  7, }, // Africa/Freetown
    {  459, 4906,  140534,  8, }, // Africa/Gaborone
    {  459, 5015,  140542,  6, }, // Africa/Harare
    {  459, 4479,  140548, 10, }, // Africa/Johannesburg
    {  459, 4499,  140558,  4, }, // Africa/Juba
    {  459, 5151,  140562,  7, }, // Africa/Kampala
    {  459, 4524,    6423,  6, }, // Africa/Khartoum
    {  459, 4973,  140569,  6, }, // Africa/Kigali
    {  459, 5306,  140575,  9, }, // Africa/Kinshasa
    {  459, 4744,  140584,  5, }, // Africa/Lagos
    {  459, 5336,  140589,  8, }, // Africa/Libreville
    {  459, 6869,  140597,  4, }, // Africa/Lome
    {  459, 5211,  140601,  6, }, // Africa/Luanda
    {  459, 4939,  140607, 10, }, // Africa/Lubumbashi
    {  459, 5001,  140617,  6, }, // Africa/Lusaka
    {  459, 5322,  140623,  6, }, // Africa/Malabo
    {  459, 4987,  140629,  6, }, // Africa/Maputo
    {  459, 5197,  140635,  7, }, // Africa/Maseru
    {  459, 5182,  140642,  7, }, // Africa/Mbabane
    {  459, 5113,  140649,  9, }, // Africa/Mogadishu
    {  459, 7034,  140658,  8, }, // Africa/Monrovia
    {  459, 3853,  140666,  7, }, // Africa/Nairobi
    {  459, 5271,  140673,  8, }, // Africa/Ndjamena
    {  459, 5354,  140681,  5, }, // Africa/Niamey
    {  459, 6803,  140686,  9, }, // Africa/Nouakchott
    {  459, 6727,  140695,  9, }, // Africa/Ouagadougou
    {  459, 5225,  140704,  9, }, // Africa/Porto-Novo
    {  459, 4435,  140713,  8, }, // Africa/Sao_Tome
    {  459, 2866,  140721,  8, }, // Africa/Tripoli
    {  459, 6554,   92461,  5, }, // Africa/Tunis
    {  459, 4152,  140729,  7, }, // Africa/Windhoek
    {  459,  132,  140736,  4, }, // America/Adak
    {  459, 3346,  140740,  8, }, // America/Anchorage
    {  459, 5607,  140748,  7, }, // America/Anguilla
    {  459, 5624,  140755,  8, }, // America/Antigua
    {  459, 4554,  140763, 10, }, // America/Araguaina
    {  459,  166,  140773, 12, }, // America/Argentina/Buenos_Aires
    {  459,   91,  140785,  8, }, // America/Argentina/Catamarca
    {  459,  270,  140793,  7, }, // America/Argentina/Cordoba
    {  459,  441,    6534,  5, }, // America/Argentina/Jujuy
    {  459, 7462,  140800,  9, }, // America/Argentina/La_Rioja
    {  459,  565,  140809,  7, }, // America/Argentina/Mendoza
    {  459, 7489,  140816, 12, }, // America/Argentina/Rio_Gallegos
    {  459, 7520,  140828,  5, }, // America/Argentina/Salta
    {  459, 7544,  140833,  8, }, // America/Argentina/San_Juan
    {  459, 5580,  140841,  8, }, // America/Argentina/San_Luis
    {  459, 7571,  140849,  7, }, // America/Argentina/Tucuman
    {  459, 7597,  140856,  6, }, // America/Argentina/Ushuaia
    {  459, 5640,   92485,  5, }, // America/Aruba
    {  459, 4252,  140862,  8, }, // America/Asuncion
    {  459,  237,  140870,  8, }, // America/Atikokan
    {  459, 3679,      30,  5, }, // America/Bahia
    {  459, 7623,  140878, 15, }, // America/Bahia_Banderas
    {  459, 5654,  140893,  8, }, // America/Barbados
    {  459, 7646,  140901,  5, }, // America/Belem
    {  459, 5407,  140906,  5, }, // America/Belize
    {  459, 7660,  140911, 12, }, // America/Blanc-Sablon
    {  459, 7681,  140923,  9, }, // America/Boa_Vista
    {  459, 7699,  140932,  5, }, // America/Boise
    {  459, 7713,  140937, 10, }, // America/Cambridge_Bay
    {  459, 7735,  140947, 12, }, // America/Campo_Grande
    {  459, 3868,    6682,  6, }, // America/Cancun
    {  459, 4694,  140959,  7, }, // America/Caracas
    {  459, 4345,  140966,  5, }, // America/Cayenne
    {  459, 5496,  140971,  5, }, // America/Cayman
    {  459, 2260,  140976,  7, }, // America/Chicago
    {  459, 7756,  140983,  9, }, // America/Chihuahua
    {  459, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  459, 5422,  140992, 10, }, // America/Costa_Rica
    {  459, 7796,  141002,  8, }, // America/Creston
    {  459, 3791,  141010,  7, }, // America/Cuiaba
    {  459, 5723,  141017,  8, }, // America/Curacao
    {  459, 7812,  141025, 12, }, // America/Danmarkshavn
    {  459, 7833,  141037,  6, }, // America/Dawson
    {  459, 7848,  141043, 11, }, // America/Dawson_Creek
    {  459,  805,  141054,  6, }, // America/Denver
    {  459, 3465,  141060,  7, }, // America/Detroit
    {  459, 5739,  141067,  8, }, // America/Dominica
    {  459,  893,  141075, 10, }, // America/Edmonton
    {  459, 7869,  141085,  8, }, // America/Eirunepe
    {  459, 5441,  141093, 10, }, // America/El_Salvador
    {  459, 7886,  141103, 11, }, // America/Fort_Nelson
    {  459, 7906,  141114,  9, }, // America/Fortaleza
    {  459, 7924,  141123,  7, }, // America/Glace_Bay
    {  459, 6881,  141130,  6, }, // America/Goose_Bay
    {  459, 4612,  141136,  9, }, // America/Grand_Turk
    {  459, 5770,  141145,  7, }, // America/Grenada
    {  459, 5786,  141152,  9, }, // America/Guadeloupe
    {  459, 3760,  141161,  9, }, // America/Guatemala
    {  459, 4841,  141170,  8, }, // America/Guayaquil
    {  459, 6932,  141178,  6, }, // America/Guyana
    {  459, 1939,  141184,  8, }, // America/Halifax
    {  459, 2281,  141192,  6, }, // America/Havana
    {  459, 7942,  141198,  9, }, // America/Hermosillo
    {  459,  348,  141207, 13, }, // America/Indiana/Indianapolis
    {  459,  481,  141220, 13, }, // America/Indiana/Knox
    {  459, 7961,  141233, 17, }, // America/Indiana/Marengo
    {  459, 7985,  141250, 17, }, // America/Indiana/Petersburg
    {  459, 8012,  141267, 18, }, // America/Indiana/Tell_City
    {  459, 8038,  141285, 14, }, // America/Indiana/Vevay
    {  459, 8060,  141299, 15, }, // America/Indiana/Vincennes
    {  459, 8086,  141314, 16, }, // America/Indiana/Winamac
    {  459, 8110,  141330,  6, }, // America/Inuvik
    {  459,  660,  141336,  7, }, // America/Iqaluit
    {  459, 2799,   92545,  7, }, // America/Jamaica
    {  459, 5380,  141343,  4, }, // America/Juneau
    {  459,  521,  141347,  6, }, // America/Kentucky/Louisville
    {  459, 8125,  141353, 22, }, // America/Kentucky/Monticello
    {  459, 5704,  141375, 11, }, // America/Kralendijk
    {  459, 7169,  141386,  4, }, // America/Lima
    {  459, 3239,  141390, 12, }, // America/Los_Angeles
    {  459, 5932,  141402, 20, }, // America/Lower_Princes
    {  459, 8153,  141422,  7, }, // America/Maceio
    {  459, 8168,   98036,  7, }, // America/Managua
    {  459, 1908,  141429,  6, }, // America/Manaus
    {  459, 5897,  141435,  7, }, // America/Marigot
    {  459, 5805,  141442,  7, }, // America/Martinique
    {  459, 8184,  141449,  9, }, // America/Matamoros
    {  459, 2917,  141458,  8, }, // America/Mazatlan
    {  459, 8202,  141466, 10, }, // America/Menominee
    {  459, 8220,  141476,  7, }, // America/Merida
    {  459, 8235,  141483, 10, }, // America/Metlakatla
    {  459, 2949,  141493, 13, }, // America/Mexico_City
    {  459, 4391,  141506,  7, }, // America/Miquelon
    {  459, 8254,  141513,  8, }, // America/Moncton
    {  459, 8270,  141521,  8, }, // America/Monterrey
    {  459, 4098,  141529, 10, }, // America/Montevideo
    {  459, 5824,  141539,  9, }, // America/Montserrat
    {  459, 5481,  141548,  6, }, // America/Nassau
    {  459, 2379,  141554,  7, }, // America/New_York
    {  459, 8288,  141561,  4, }, // America/Nome
    {  459, 1850,  141565,  7, }, // America/Noronha
    {  459, 8301,  141572, 18, }, // America/North_Dakota/Beulah
    {  459, 8329,  141590, 19, }, // America/North_Dakota/Center
    {  459, 8357,  141609, 22, }, // America/North_Dakota/New_Salem
    {  459,  393,     317,  4, }, // America/Nuuk
    {  459, 8388,  141631,  8, }, // America/Ojinaga
    {  459, 2356,  141639,  6, }, // America/Panama
    {  459, 6173,  141645, 10, }, // America/Paramaribo
    {  459, 2973,  141655,  6, }, // America/Phoenix
    {  459, 3945,  141661, 11, }, // America/Port-au-Prince
    {  459, 5954,  141672, 12, }, // America/Port_of_Spain
    {  459, 8404,  141684, 11, }, // America/Porto_Velho
    {  459, 5843,  141695, 12, }, // America/Puerto_Rico
    {  459, 4030,  141707, 13, }, // America/Punta_Arenas
    {  459, 8424,  141720, 12, }, // America/Rankin_Inlet
    {  459, 8445,  141732,  6, }, // America/Recife
    {  459, 1995,  141738,  6, }, // America/Regina
    {  459, 8460,  141744,  8, }, // America/Resolute
    {  459,  695,  141752, 10, }, // America/Rio_Branco
    {  459, 8477,  141762,  8, }, // America/Santarem
    {  459, 2201,  141770,  8, }, // America/Santiago
    {  459, 6111,  141778, 13, }, // America/Santo_Domingo
    {  459, 1878,  141791,  9, }, // America/Sao_Paulo
    {  459, 6899,  141800, 11, }, // America/Scoresbysund
    {  459, 8508,  141811, 14, }, // America/St_Barthelemy
    {  459, 2061,  141825,  8, }, // America/St_Johns
    {  459, 5863,  126132,  9, }, // America/St_Kitts
    {  459, 5880,  141833, 11, }, // America/St_Lucia
    {  459,  855,  141844, 10, }, // America/St_Thomas
    {  459, 5913,  141854, 12, }, // America/St_Vincent
    {  459, 8530,  141866, 13, }, // America/Swift_Current
    {  459, 5461,  141879, 11, }, // America/Tegucigalpa
    {  459, 5756,  141890,  4, }, // America/Thule
    {  459,  313,  141894,  8, }, // America/Tijuana
    {  459,  608,  141902,  7, }, // America/Toronto
    {  459, 5688,  141909,  6, }, // America/Tortola
    {  459, 2093,  141915,  7, }, // America/Vancouver
    {  459, 2144,  141922,  7, }, // America/Whitehorse
    {  459,  734,  141929,  7, }, // America/Winnipeg
    {  459, 8552,  141936,  7, }, // America/Yakutat
    {  459, 6001,  141943,  4, }, // Antarctica/Casey
    {  459, 6094,  141947,  5, }, // Antarctica/Davis
    {  459, 6133,  141952, 14, }, // Antarctica/DumontDUrville
    {  459, 8568,  141966,  7, }, // Antarctica/Macquarie
    {  459, 7066,  141973,  6, }, // Antarctica/Mawson
    {  459, 7113,  141979,  7, }, // Antarctica/McMurdo
    {  459, 8589,  141986,  4, }, // Antarctica/Palmer
    {  459, 7226,  141990,  7, }, // Antarctica/Rothera
    {  459, 7295,  141997,  5, }, // Antarctica/Syowa
    {  459, 8607,  123400,  4, }, // Antarctica/Troll
    {  459, 7374,  142002,  7, }, // Antarctica/Vostok
    {  459, 1429,  142009, 12, }, // Arctic/Longyearbyen
    {  459, 5570,  142021,  4, }, // Asia/Aden
    {  459, 5395,  142025,  6, }, // Asia/Almaty
    {  459, 3968,    5605,  4, }, // Asia/Amman
    {  459, 5511,  142031,  7, }, // Asia/Anadyr
    {  459, 5523,  142038,  5, }, // Asia/Aqtau
    {  459, 5368,  142043,  6, }, // Asia/Aqtobe
    {  459,  964,  142049,  8, }, // Asia/Ashgabat
    {  459, 8624,  142057,  6, }, // Asia/Atyrau
    {  459, 3607,  142063,  6, }, // Asia/Baghdad
    {  459, 5534,  142069,  6, }, // Asia/Bahrain
    {  459, 3653,  142075,  4, }, // Asia/Baku
    {  459, 4466,  142079,  6, }, // Asia/Bangkok
    {  459, 3571,  142085,  7, }, // Asia/Barnaul
    {  459, 4086,  142092,  5, }, // Asia/Beirut
    {  459, 3778,  142097,  7, }, // Asia/Bishkek
    {  459, 5989,  142104,  6, }, // Asia/Brunei
    {  459, 4601,  142110,  5, }, // Asia/Chita
    {  459, 4511,  142115,  7, }, // Asia/Colombo
    {  459, 4540,  142122,  8, }, // Asia/Damascus
    {  459, 1093,  142130,  4, }, // Asia/Dhaka
    {  459, 6192,  142134,  4, }, // Asia/Dili
    {  459, 6159,  142138,  8, }, // Asia/Dushanbe
    {  459, 8636,  142146,  7, }, // Asia/Famagusta
    {  459, 8651,   91569,  4, }, // Asia/Gaza
    {  459, 4795,  142153,  7, }, // Asia/Hebron
    {  459, 1259,  142160, 15, }, // Asia/Ho_Chi_Minh
    {  459, 2704,  142175,  9, }, // Asia/Hong_Kong
    {  459, 4184,  142184,  7, }, // Asia/Irkutsk
    {  459, 7006,  142191,  6, }, // Asia/Jakarta
    {  459, 6992,  142197,  8, }, // Asia/Jayapura
    {  459, 1290,  142205,  9, }, // Asia/Jerusalem
    {  459, 4316,  142214,  9, }, // Asia/Kamchatka
    {  459, 4239,  142223,  7, }, // Asia/Karachi
    {  459, 1185,  142230,  8, }, // Asia/Kathmandu
    {  459, 8661,  142238,  7, }, // Asia/Khandyga
    {  459,  992,  142245,  7, }, // Asia/Kolkata
    {  459, 4197,  142252, 10, }, // Asia/Krasnoyarsk
    {  459, 4859,  142262, 11, }, // Asia/Kuala_Lumpur
    {  459, 5547,  142273,  5, }, // Asia/Kuwait
    {  459, 1211,  142278,  5, }, // Asia/Macau
    {  459, 4017,  142283,  7, }, // Asia/Magadan
    {  459, 1349,  142290,  6, }, // Asia/Makassar
    {  459, 7182,  142296,  6, }, // Asia/Manila
    {  459, 6920,  142302,  6, }, // Asia/Muscat
    {  459, 2554,    5738,  7, }, // Asia/Nicosia
    {  459, 8675,  142308, 12, }, // Asia/Novokuznetsk
    {  459, 4135,  142320, 10, }, // Asia/Novosibirsk
    {  459, 4229,  142330,  4, }, // Asia/Omsk
    {  459, 7145,  142334,  5, }, // Asia/Oral
    {  459, 6961,  142339,  9, }, // Asia/Phnom_Penh
    {  459, 8693,  142348,  9, }, // Asia/Pontianak
    {  459, 4214,  142357,  9, }, // Asia/Pyongyang
    {  459, 5559,  142366,  4, }, // Asia/Qatar
    {  459, 8708,  142370,  8, }, // Asia/Qostanay
    {  459, 4269,  142378,  9, }, // Asia/Qyzylorda
    {  459, 3584,  142387,  5, }, // Asia/Riyadh
    {  459, 4408,  142392,  8, }, // Asia/Sakhalin
    {  459, 7245,  142400,  8, }, // Asia/Samarkand
    {  459, 3279,  142408,  3, }, // Asia/Seoul
    {  459, 1053,  142411,  8, }, // Asia/Shanghai
    {  459, 3300,  142419,  7, }, // Asia/Singapore
    {  459, 4297,  142426, 14, }, // Asia/Srednekolymsk
    {  459, 3263,  142440,  6, }, // Asia/Taipei
    {  459, 4781,  142446,  8, }, // Asia/Tashkent
    {  459, 3915,  142454,  7, }, // Asia/Tbilisi
    {  459, 2772,  142461,  5, }, // Asia/Tehran
    {  459, 1317,  142466,  5, }, // Asia/Thimphu
    {  459, 2821,  142471,  5, }, // Asia/Tokyo
    {  459, 1021,  142476,  8, }, // Asia/Ulaanbaatar
    {  459, 1159,  142484,  8, }, // Asia/Urumqi
    {  459, 8722,  142492,  9, }, // Asia/Ust-Nera
    {  459, 6977,  142501,  9, }, // Asia/Vientiane
    {  459, 4710,  142510, 11, }, // Asia/Vladivostok
    {  459, 4828,  142521,  7, }, // Asia/Yakutsk
    {  459, 1235,  142528,  6, }, // Asia/Yangon
    {  459, 3883,  142534, 12, }, // Asia/Yekaterinburg
    {  459, 3747,  142546,  8, }, // Asia/Yerevan
    {  459, 3663,  142554,  4, }, // Atlantic/Azores
    {  459, 5671,  142558,  7, }, // Atlantic/Bermuda
    {  459, 6628,  142565,  6, }, // Atlantic/Canary
    {  459, 3727,  142571,  7, }, // Atlantic/Cape_Verde
    {  459, 1395,  142578,  6, }, // Atlantic/Faroe
    {  459, 8736,  142584,  6, }, // Atlantic/Madeira
    {  459, 2748,  142590,  9, }, // Atlantic/Reykjavik
    {  459, 7272,  142599, 11, }, // Atlantic/South_Georgia
    {  459, 6821,  142610, 12, }, // Atlantic/St_Helena
    {  459, 6644,  142622,  6, }, // Atlantic/Stanley
    {  459, 1670,  142628,  6, }, // Australia/Adelaide
    {  459, 1635,  142634,  7, }, // Australia/Brisbane
    {  459, 1799,  142641, 10, }, // Australia/Broken_Hill
    {  459, 1583,  142651,  6, }, // Australia/Darwin
    {  459, 3637,  142657,  5, }, // Australia/Eucla
    {  459, 1516,  142662,  5, }, // Australia/Hobart
    {  459, 8753,  142667,  8, }, // Australia/Lindeman
    {  459, 1547,  142675,  7, }, // Australia/Lord_Howe
    {  459, 1727,  142682,  7, }, // Australia/Melbourne
    {  459, 1762,  142689,  4, }, // Australia/Perth
    {  459, 1463,  142693,  5, }, // Australia/Sydney
    {  459, 6441,  142698,  8, }, // Europe/Amsterdam
    {  459, 6216,  142706,  7, }, // Europe/Andorra
    {  459, 3620,  142713,  8, }, // Europe/Astrakhan
    {  459, 2300,  142721,  5, }, // Europe/Athens
    {  459, 6336,  142726,  8, }, // Europe/Belgrade
    {  459, 4757,  142734,  5, }, // Europe/Berlin
    {  459, 6488,  142739, 10, }, // Europe/Bratislava
    {  459, 2167,  142749,  7, }, // Europe/Brussels
    {  459, 3928,  142756,  8, }, // Europe/Bucharest
    {  459, 3806,  142764,  8, }, // Europe/Budapest
    {  459, 8772,  142772,  8, }, // Europe/Busingen
    {  459, 2583,  142780,  8, }, // Europe/Chisinau
    {  459, 6289,  142788, 10, }, // Europe/Copenhagen
    {  459, 2338,  142798,  7, }, // Europe/Dublin
    {  459, 6307,  142805,  9, }, // Europe/Gibraltar
    {  459, 8788,  142814,  6, }, // Europe/Guernsey
    {  459, 6612,  142820,  8, }, // Europe/Helsinki
    {  459, 8804,  142828, 10, }, // Europe/Isle_of_Man
    {  459, 1130,  142838,  8, }, // Europe/Istanbul
    {  459, 8823,  142846,  5, }, // Europe/Jersey
    {  459, 3979,  142851, 11, }, // Europe/Kaliningrad
    {  459, 8929,  142862,  5, }, // Europe/Kirov
    {  459, 2527,  142867,  4, }, // Europe/Kyiv
    {  459, 3213,  142871,  6, }, // Europe/Lisbon
    {  459, 6506,  142877,  8, }, // Europe/Ljubljana
    {  459, 2501,  142885,  7, }, // Europe/London
    {  459, 6365,  142892, 11, }, // Europe/Luxembourg
    {  459, 6397,  142903,  6, }, // Europe/Malta
    {  459, 6582,  142909,  9, }, // Europe/Mariehamn
    {  459, 6410,  142918,  7, }, // Europe/Monaco
    {  459, 3537,  142925,  6, }, // Europe/Moscow
    {  459, 6458,  142931,  5, }, // Europe/Oslo
    {  459, 4284,  142936,  5, }, // Europe/Paris
    {  459, 6424,  142941,  9, }, // Europe/Podgorica
    {  459, 6275,    8201,  4, }, // Europe/Prague
    {  459, 8837,  142950,  4, }, // Europe/Riga
    {  459, 6324,    6064,  3, }, // Europe/Rome
    {  459, 4331,  142954,  6, }, // Europe/Samara
    {  459, 6470,  142960, 10, }, // Europe/San_Marino
    {  459, 6245,  142970,  9, }, // Europe/Sarajevo
    {  459, 4451,  142979,  7, }, // Europe/Saratov
    {  459, 8849,  142986, 10, }, // Europe/Simferopol
    {  459, 6383,  142996,  7, }, // Europe/Skopje
    {  459, 6599,   21199,  5, }, // Europe/Sofia
    {  459, 6523,  143003,  8, }, // Europe/Stockholm
    {  459, 8867,  143011,  5, }, // Europe/Tallinn
    {  459, 6202,  143016,  6, }, // Europe/Tirane
    {  459, 8882,  143022,  9, }, // Europe/Ulyanovsk
    {  459, 6567,  143031,  7, }, // Europe/Vatican
    {  459, 6231,  143038,  6, }, // Europe/Vienna
    {  459, 8899,   44614,  7, }, // Europe/Vilnius
    {  459, 4727,  143044,  9, }, // Europe/Volgograd
    {  459, 3190,  143053,  5, }, // Europe/Warsaw
    {  459, 6261,  143058,  6, }, // Europe/Zagreb
    {  459, 6540,  143064,  5, }, // Europe/Zurich
    {  459, 5078,  143069, 12, }, // Indian/Antananarivo
    {  459, 6947,  143081,  6, }, // Indian/Chagos
    {  459, 6046,  143087,  7, }, // Indian/Christmas
    {  459, 6063,   93033,  5, }, // Indian/Cocos
    {  459, 5029,  143094,  9, }, // Indian/Comoro
    {  459, 6661,  143103,  9, }, // Indian/Kerguelen
    {  459, 7260,  143112,  5, }, // Indian/Mahe
    {  459, 7050,  143117,  8, }, // Indian/Maldives
    {  459, 4069,  143125,  8, }, // Indian/Mauritius
    {  459, 5098,  143133,  5, }, // Indian/Mayotte
    {  459, 7211,  143138,  8, }, // Indian/Reunion
    {  459, 4422,  143146,  4, }, // Pacific/Apia
    {  459,  932,  143150,  7, }, // Pacific/Auckland
    {  459, 3706,  143157, 10, }, // Pacific/Bougainville
    {  459, 3015,  143167,  6, }, // Pacific/Chatham
    {  459, 3157,  143173,  4, }, // Pacific/Chuuk
    {  459, 2237,  143177,  4, }, // Pacific/Easter
    {  459, 7360,  143181,  5, }, // Pacific/Efate
    {  459, 7327,  143186,  7, }, // Pacific/Fakaofo
    {  459, 3902,  143193,  4, }, // Pacific/Fiji
    {  459, 7343,  143197,  8, }, // Pacific/Funafuti
    {  459, 6678,  143205,  9, }, // Pacific/Galapagos
    {  459, 6696,  143214,  7, }, // Pacific/Gambier
    {  459, 3822,  143221, 11, }, // Pacific/Guadalcanal
    {  459, 3049,  143232, 10, }, // Pacific/Kanton
    {  459, 3998,  143242,  9, }, // Pacific/Kiritimati
    {  459, 7019,  143251,  7, }, // Pacific/Kosrae
    {  459, 2842,  143258,  8, }, // Pacific/Kwajalein
    {  459, 4877,  143266,  6, }, // Pacific/Majuro
    {  459, 4051,  143272,  9, }, // Pacific/Marquesas
    {  459, 8914,  143281,  5, }, // Pacific/Midway
    {  459, 7084,  143286,  5, }, // Pacific/Nauru
    {  459, 7132,  141554,  3, }, // Pacific/Niue
    {  459, 4168,  143291,  9, }, // Pacific/Norfolk
    {  459, 7098,  143300,  6, }, // Pacific/Noumea
    {  459, 3126,  143306,  9, }, // Pacific/Pago_Pago
    {  459, 7155,  143315,  5, }, // Pacific/Palau
    {  459, 7194,  143320,  6, }, // Pacific/Pitcairn
    {  459, 3096,  143326,  7, }, // Pacific/Pohnpei
    {  459, 4807,  143333, 12, }, // Pacific/Port_Moresby
    {  459, 6076,  143345,  9, }, // Pacific/Rarotonga
    {  459, 7312,  143354,  6, }, // Pacific/Tahiti
    {  459, 6712,  143360,  6, }, // Pacific/Tarawa
    {  459, 4583,  143366,  9, }, // Pacific/Tongatapu
    {  459, 7392,  143375,  3, }, // Pacific/Wake
    {  459, 7405,  143378,  4, }, // Pacific/Wallis
    {  460, 6788,  143382,  7, }, // Africa/Abidjan Nko/Nko/Guinea
    {  460, 6760,  143389,  5, }, // Africa/Accra
    {  460, 5059,  143394, 14, }, // Africa/Addis_Ababa
    {  460, 7420,  143408,  6, }, // Africa/Algiers
    {  460,   14,  143414,  7, }, // Africa/Asmara
    {  460,   44,  143421,  7, }, // Africa/Bamako
    {  460, 5257,  143428,  7, }, // Africa/Bangui
    {  460, 6746,  143435,  6, }, // Africa/Banjul
    {  460, 7435,  143441,  7, }, // Africa/Bissau
    {  460, 4957,  143448,  9, }, // Africa/Blantyre
    {  460, 5287,  143457,  9, }, // Africa/Brazzaville
    {  460, 4922,  143466, 11, }, // Africa/Bujumbura
    {  460, 2320,  143477,  4, }, // Africa/Cairo
    {  460, 4117,  143481, 11, }, // Africa/Casablanca
    {  460, 7449,  143492,  7, }, // Africa/Ceuta
    {  460, 6773,  143499,  8, }, // Africa/Conakry
    {  460, 6840,  143507,  9, }, // Africa/Dakar
    {  460, 5130,  143516, 12, }, // Africa/Dar_es_Salaam
    {  460, 5043,  143528,  7, }, // Africa/Djibouti
    {  460, 5243,  143535,  6, }, // Africa/Douala
    {  460, 5166,  143541,  6, }, // Africa/El_Aaiun
    {  460, 6853,  143547,  7, }, // Africa/Freetown
    {  460, 4906,  143554,  8, }, // Africa/Gaborone
    {  460, 5015,  143562,  6, }, // Africa/Harare
    {  460, 4479,  143568, 13, }, // Africa/Johannesburg
    {  460, 4499,  143581,  5, }, // Africa/Juba
    {  460, 5151,  143586,  7, }, // Africa/Kampala
    {  460, 4524,  143593,  7, }, // Africa/Khartoum
    {  460, 4973,  143600,  8, }, // Africa/Kigali
    {  460, 5306,  143608,  8, }, // Africa/Kinshasa
    {  460, 4744,  143616,  6, }, // Africa/Lagos
    {  460, 5336,  143622,  9, }, // Africa/Libreville
    {  460, 6869,  143631,  5, }, // Africa/Lome
    {  460, 5211,  143636,  8, }, // Africa/Luanda
    {  460, 4939,  143644, 10, }, // Africa/Lubumbashi
    {  460, 5001,  143654,  6, }, // Africa/Lusaka
    {  460, 5322,  143660,  6, }, // Africa/Malabo
    {  460, 4987,  143666,  7, }, // Africa/Maputo
    {  460, 5197,  143673,  7, }, // Africa/Maseru
    {  460, 5182,  143680,  6, }, // Africa/Mbabane
    {  460, 5113,  143686, 10, }, // Africa/Mogadishu
    {  460, 7034,  143696, 11, }, // Africa/Monrovia
    {  460, 3853,  143707,  8, }, // Africa/Nairobi
    {  460, 5271,  143715,  8, }, // Africa/Ndjamena
    {  460, 5354,  143723,  5, }, // Africa/Niamey
    {  460, 6803,  143728,  9, }, // Africa/Nouakchott
    {  460, 6727,  143737, 10, }, // Africa/Ouagadougou
    {  460, 5225,  143747, 13, }, // Africa/Porto-Novo
    {  460, 4435,  143760,  9, }, // Africa/Sao_Tome
    {  460, 2866,  143769, 16, }, // Africa/Tripoli
    {  460, 6554,  143785,  5, }, // Africa/Tunis
    {  460, 4152,  143790,  6, }, // Africa/Windhoek
    {  460,  132,  143796,  4, }, // America/Adak
    {  460, 3346,  143800,  8, }, // America/Anchorage
    {  460, 5607,  143808,  8, }, // America/Anguilla
    {  460, 5624,  143816,  8, }, // America/Antigua
    {  460, 4554,  143824, 22, }, // America/Araguaina
    {  460,  166,  143846,  8, }, // America/Argentina/Buenos_Aires
    {  460,   91,  143854, 10, }, // America/Argentina/Catamarca
    {  460,  270,  143864,  9, }, // America/Argentina/Cordoba
    {  460,  441,  143873,  7, }, // America/Argentina/Jujuy
    {  460, 7462,  143880, 10, }, // America/Argentina/La_Rioja
    {  460,  565,  143890,  8, }, // America/Argentina/Mendoza
    {  460, 7489,  143898, 16, }, // America/Argentina/Rio_Gallegos
    {  460, 7520,  143914,  7, }, // America/Argentina/Salta
    {  460, 7544,  143921, 11, }, // America/Argentina/San_Juan
    {  460, 5580,  143932, 10, }, // America/Argentina/San_Luis
    {  460, 7571,  143942,  7, }, // America/Argentina/Tucuman
    {  460, 7597,  143949,  7, }, // America/Argentina/Ushuaia
    {  460, 5640,  143956,  6, }, // America/Aruba
    {  460, 4252,  143962, 10, }, // America/Asuncion
    {  460,  237,  143972,  9, }, // America/Atikokan
    {  460, 3679,  143981,  7, }, // America/Bahia
    {  460, 7623,  143988, 19, }, // America/Bahia_Banderas
    {  460, 5654,  144007,  6, }, // America/Barbados
    {  460, 7646,  144013,  4, }, // America/Belem
    {  460, 5407,  144017,  9, }, // America/Belize
    {  460, 7660,  144026, 15, }, // America/Blanc-Sablon
    {  460, 7681,  144041, 14, }, // America/Boa_Vista
    {  460, 4361,  144055,  7, }, // America/Bogota
    {  460, 7699,  144062,  5, }, // America/Boise
    {  460, 7713,  144067, 12, }, // America/Cambridge_Bay
    {  460, 7735,  144079, 14, }, // America/Campo_Grande
    {  460, 3868,  144093,  7, }, // America/Cancun
    {  460, 4694,  144100,  6, }, // America/Caracas
    {  460, 4345,  144106,  7, }, // America/Cayenne
    {  460, 5496,  144113,  8, }, // America/Cayman
    {  460, 2260,  144121,  8, }, // America/Chicago
    {  460, 7756,  144129,  9, }, // America/Chihuahua
    {  460, 7774,  144138, 15, }, // America/Ciudad_Juarez
    {  460, 5422,  144153, 11, }, // America/Costa_Rica
    {  460, 7796,  144164,  9, }, // America/Creston
    {  460, 3791,  144173,  8, }, // America/Cuiaba
    {  460, 5723,  144181,  8, }, // America/Curacao
    {  460, 7812,  144189, 14, }, // America/Danmarkshavn
    {  460, 7833,  144203,  8, }, // America/Dawson
    {  460, 7848,  144211, 13, }, // America/Dawson_Creek
    {  460,  805,  144224,  7, }, // America/Denver
    {  460, 3465,  144231,  7, }, // America/Detroit
    {  460, 5739,  144238,  7, }, // America/Dominica
    {  460,  893,  144245, 10, }, // America/Edmonton
    {  460, 7869,  144255,  6, }, // America/Eirunepe
    {  460, 5441,  144261, 10, }, // America/El_Salvador
    {  460, 7886,  144271, 13, }, // America/Fort_Nelson
    {  460, 7906,  144284, 12, }, // America/Fortaleza
    {  460, 7924,  144296, 10, }, // America/Glace_Bay
    {  460, 6881,  144306, 10, }, // America/Goose_Bay
    {  460, 4612,  144316, 12, }, // America/Grand_Turk
    {  460, 5770,  144328,  8, }, // America/Grenada
    {  460, 5786,  144336, 10, }, // America/Guadeloupe
    {  460, 3760,  144346, 13, }, // America/Guatemala
    {  460, 4841,  144359,  9, }, // America/Guayaquil
    {  460, 6932,  144368,  7, }, // America/Guyana
    {  460, 1939,  144375,  8, }, // America/Halifax
    {  460, 2281,  144383,  7, }, // America/Havana
    {  460, 7942,  144390, 11, }, // America/Hermosillo
    {  460,  348,  144401, 13, }, // America/Indiana/Indianapolis
    {  460,  481,  144414, 17, }, // America/Indiana/Knox
    {  460, 7961,  144431, 20, }, // America/Indiana/Marengo
    {  460, 7985,  144451, 23, }, // America/Indiana/Petersburg
    {  460, 8012,  144474, 19, }, // America/Indiana/Tell_City
    {  460, 8038,  144493, 18, }, // America/Indiana/Vevay
    {  460, 8060,  144511, 19, }, // America/Indiana/Vincennes
    {  460, 8086,  144530, 17, }, // America/Indiana/Winamac
    {  460, 8110,  144547,  7, }, // America/Inuvik
    {  460,  660,  144554,  8, }, // America/Iqaluit
    {  460, 2799,  144562,  8, }, // America/Jamaica
    {  460, 5380,  144570,  7, }, // America/Juneau
    {  460,  521,  144577,  9, }, // America/Kentucky/Louisville
    {  460, 8125,  144586, 21, }, // America/Kentucky/Monticello
    {  460, 5704,  144607, 11, }, // America/Kralendijk
    {  460, 4376,  144618,  7, }, // America/La_Paz
    {  460, 7169,  144625,  5, }, // America/Lima
    {  460, 3239,  144630, 12, }, // America/Los_Angeles
    {  460, 5932,  144642, 19, }, // America/Lower_Princes
    {  460, 8153,  144661,  7, }, // America/Maceio
    {  460, 8168,  144668,  9, }, // America/Managua
    {  460, 1908,  144677,  6, }, // America/Manaus
    {  460, 5897,  144683,  8, }, // America/Marigot
    {  460, 5805,  144691,  8, }, // America/Martinique
    {  460, 8184,  144699,  7, }, // America/Matamoros
    {  460, 2917,  144706, 10, }, // America/Mazatlan
    {  460, 8202,  144716,  8, }, // America/Menominee
    {  460, 8220,  144724,  7, }, // America/Merida
    {  460, 8235,  144731, 12, }, // America/Metlakatla
    {  460, 2949,  144743,  8, }, // America/Mexico_City
    {  460, 4391,  144751,  8, }, // America/Miquelon
    {  460, 8254,  144759,  9, }, // America/Moncton
    {  460, 8270,  144768,  8, }, // America/Monterrey
    {  460, 4098,  144776, 13, }, // America/Montevideo
    {  460, 5824,  144789,  9, }, // America/Montserrat
    {  460, 5481,  144798,  7, }, // America/Nassau
    {  460, 2379,  144805,  8, }, // America/New_York
    {  460, 8288,  144813,  5, }, // America/Nome
    {  460, 1850,  144818,  9, }, // America/Noronha
    {  460, 8301,  144827, 24, }, // America/North_Dakota/Beulah
    {  460, 8329,  144851, 24, }, // America/North_Dakota/Center
    {  460, 8357,  144875, 26, }, // America/North_Dakota/New_Salem
    {  460,  393,  144901,  4, }, // America/Nuuk
    {  460, 8388,  144905,  8, }, // America/Ojinaga
    {  460, 2356,  144913,  6, }, // America/Panama
    {  460, 6173,  144919, 10, }, // America/Paramaribo
    {  460, 2973,  144929,  7, }, // America/Phoenix
    {  460, 3945,  144936, 13, }, // America/Port-au-Prince
    {  460, 5954,  144949, 12, }, // America/Port_of_Spain
    {  460, 8404,  144961, 14, }, // America/Porto_Velho
    {  460, 5843,  144975, 12, }, // America/Puerto_Rico
    {  460, 4030,  144987, 13, }, // America/Punta_Arenas
    {  460, 8424,  145000, 13, }, // America/Rankin_Inlet
    {  460, 8445,  145013,  5, }, // America/Recife
    {  460, 1995,  145018,  8, }, // America/Regina
    {  460, 8460,  145026,  8, }, // America/Resolute
    {  460,  695,  145034, 14, }, // America/Rio_Branco
    {  460, 8477,  145048,  8, }, // America/Santarem
    {  460, 2201,  145056, 11, }, // America/Santiago
    {  460, 6111,  145067, 13, }, // America/Santo_Domingo
    {  460, 1878,  145080,  9, }, // America/Sao_Paulo
    {  460, 6899,  145089, 16, }, // America/Scoresbysund
    {  460, 8494,  145105,  7, }, // America/Sitka
    {  460, 8508,  145112, 15, }, // America/St_Barthelemy
    {  460, 2061,  145127, 20, }, // America/St_Johns
    {  460, 5863,  145147, 12, }, // America/St_Kitts
    {  460, 5880,  145159,  8, }, // America/St_Lucia
    {  460,  855,  145167,  9, }, // America/St_Thomas
    {  460, 5913,  145176, 12, }, // America/St_Vincent
    {  460, 8530,  145188, 11, }, // America/Swift_Current
    {  460, 5461,  145199, 15, }, // America/Tegucigalpa
    {  460, 5756,  145214,  5, }, // America/Thule
    {  460,  313,  145219,  8, }, // America/Tijuana
    {  460,  608,  145227,  8, }, // America/Toronto
    {  460, 5688,  145235,  9, }, // America/Tortola
    {  460, 2093,  145244, 10, }, // America/Vancouver
    {  460, 2144,  145254,  8, }, // America/Whitehorse
    {  460,  734,  145262,  9, }, // America/Winnipeg
    {  460, 8552,  145271,  7, }, // America/Yakutat
    {  460, 6001,  145278, 16, }, // Antarctica/Casey
    {  460, 6094,  145294,  6, }, // Antarctica/Davis
    {  460, 6133,  145300, 13, }, // Antarctica/DumontDUrville
    {  460, 8568,  145313,  6, }, // Antarctica/Macquarie
    {  460, 7066,  145319,  8, }, // Antarctica/Mawson
    {  460, 7113,  145327, 10, }, // Antarctica/McMurdo
    {  460, 8589,  145337,  7, }, // Antarctica/Palmer
    {  460, 7226,  145344,  7, }, // Antarctica/Rothera
    {  460, 7295,  145351,  5, }, // Antarctica/Syowa
    {  460, 8607,  145356,  4, }, // Antarctica/Troll
    {  460, 7374,  145360,  8, }, // Antarctica/Vostok
    {  460, 1429,  145368, 17, }, // Arctic/Longyearbyen
    {  460, 5570,  145385,  5, }, // Asia/Aden
    {  460, 5395,  145390, 10, }, // Asia/Almaty
    {  460, 3968,  145400,  6, }, // Asia/Amman
    {  460, 5511,  145406,  6, }, // Asia/Anadyr
    {  460, 5523,  145412,  8, }, // Asia/Aqtau
    {  460, 5368,  145420, 10, }, // Asia/Aqtobe
    {  460,  964,  145430,  9, }, // Asia/Ashgabat
    {  460, 8624,  145439,  7, }, // Asia/Atyrau
    {  460, 3607,  145446,  8, }, // Asia/Baghdad
    {  460, 5534,  145454,  7, }, // Asia/Bahrain
    {  460, 3653,  145461,  5, }, // Asia/Baku
    {  460, 4466,  145466,  6, }, // Asia/Bangkok
    {  460, 3571,  145472,  7, }, // Asia/Barnaul
    {  460, 4086,  145479,  7, }, // Asia/Beirut
    {  460, 3778,  145486,  5, }, // Asia/Bishkek
    {  460, 5989,  145491,  8, }, // Asia/Brunei
    {  460, 4601,  145499,  5, }, // Asia/Chita
    {  460, 4511,  145504,  8, }, // Asia/Colombo
    {  460, 4540,  145512,  4, }, // Asia/Damascus
    {  460, 1093,  145516,  4, }, // Asia/Dhaka
    {  460, 6192,  145520,  4, }, // Asia/Dili
    {  460, 3596,  145524,  8, }, // Asia/Dubai
    {  460, 6159,  145532,  8, }, // Asia/Dushanbe
    {  460, 8636,  145540, 11, }, // Asia/Famagusta
    {  460, 8651,  145551,  7, }, // Asia/Gaza
    {  460, 4795,  145558,  8, }, // Asia/Hebron
    {  460, 1259,  145566, 14, }, // Asia/Ho_Chi_Minh
    {  460, 2704,  145580, 11, }, // Asia/Hong_Kong
    {  460, 4771,  145591,  5, }, // Asia/Hovd
    {  460, 4184,  145596, 11, }, // Asia/Irkutsk
    {  460, 7006,  145607,  8, }, // Asia/Jakarta
    {  460, 6992,  145615,  8, }, // Asia/Jayapura
    {  460, 1290,  145623, 11, }, // Asia/Jerusalem
    {  460, 3560,  145634,  5, }, // Asia/Kabul
    {  460, 4316,  145639, 12, }, // Asia/Kamchatka
    {  460, 4239,  145651,  6, }, // Asia/Karachi
    {  460, 1185,  145657, 10, }, // Asia/Kathmandu
    {  460, 8661,  145667, 10, }, // Asia/Khandyga
    {  460,  992,  145677,  9, }, // Asia/Kolkata
    {  460, 4197,  145686, 17, }, // Asia/Krasnoyarsk
    {  460, 4859,  145703, 12, }, // Asia/Kuala_Lumpur
    {  460, 5976,  145715,  7, }, // Asia/Kuching
    {  460, 5547,  145722,  7, }, // Asia/Kuwait
    {  460, 1211,  145729,  6, }, // Asia/Macau
    {  460, 4017,  145735,  8, }, // Asia/Magadan
    {  460, 1349,  145743,  6, }, // Asia/Makassar
    {  460, 7182,  145749,  5, }, // Asia/Manila
    {  460, 6920,  145754,  7, }, // Asia/Muscat
    {  460, 2554,  145761,  9, }, // Asia/Nicosia
    {  460, 8675,  145770, 16, }, // Asia/Novokuznetsk
    {  460, 4135,  145786, 14, }, // Asia/Novosibirsk
    {  460, 4229,  145800,  5, }, // Asia/Omsk
    {  460, 7145,  145805,  4, }, // Asia/Oral
    {  460, 6961,  145809, 11, }, // Asia/Phnom_Penh
    {  460, 8693,  145820, 10, }, // Asia/Pontianak
    {  460, 4214,  145830, 12, }, // Asia/Pyongyang
    {  460, 5559,  145842,  4, }, // Asia/Qatar
    {  460, 8708,  145846,  8, }, // Asia/Qostanay
    {  460, 4269,  145854, 11, }, // Asia/Qyzylorda
    {  460, 3584,  145865,  5, }, // Asia/Riyadh
    {  460, 4408,  145870,  6, }, // Asia/Sakhalin
    {  460, 7245,  145876,  9, }, // Asia/Samarkand
    {  460, 3279,  145885,  4, }, // Asia/Seoul
    {  460, 1053,  145889,  9, }, // Asia/Shanghai
    {  460, 3300,  145898,  9, }, // Asia/Singapore
    {  460, 4297,  145907, 17, }, // Asia/Srednekolymsk
    {  460, 3263,  145924,  7, }, // Asia/Taipei
    {  460, 4781,  145931,  9, }, // Asia/Tashkent
    {  460, 3915,  145940,  7, }, // Asia/Tbilisi
    {  460, 2772,  145947,  7, }, // Asia/Tehran
    {  460, 1317,  145954,  6, }, // Asia/Thimphu
    {  460, 2821,  145960,  7, }, // Asia/Tokyo
    {  460, 4572,  145967,  6, }, // Asia/Tomsk
    {  460, 1021,  145973,  9, }, // Asia/Ulaanbaatar
    {  460, 1159,  145982,  7, }, // Asia/Urumqi
    {  460, 8722,  145989,  9, }, // Asia/Ust-Nera
    {  460, 6977,  145998, 10, }, // Asia/Vientiane
    {  460, 4710,  146008, 15, }, // Asia/Vladivostok
    {  460, 4828,  146023,  8, }, // Asia/Yakutsk
    {  460, 1235,  146031,  8, }, // Asia/Yangon
    {  460, 3883,  146039, 14, }, // Asia/Yekaterinburg
    {  460, 3747,  146053,  8, }, // Asia/Yerevan
    {  460, 3663,  146061,  7, }, // Atlantic/Azores
    {  460, 5671,  146068,  9, }, // Atlantic/Bermuda
    {  460, 6628,  146077,  6, }, // Atlantic/Canary
    {  460, 3727,  146083, 14, }, // Atlantic/Cape_Verde
    {  460, 1395,  146097,  6, }, // Atlantic/Faroe
    {  460, 8736,  146103,  5, }, // Atlantic/Madeira
    {  460, 2748,  146108, 12, }, // Atlantic/Reykjavik
    {  460, 7272,  146120, 18, }, // Atlantic/South_Georgia
    {  460, 6821,  146138, 10, }, // Atlantic/St_Helena
    {  460, 6644,  146148,  8, }, // Atlantic/Stanley
    {  460, 1670,  146156,  7, }, // Australia/Adelaide
    {  460, 1635,  146163, 10, }, // Australia/Brisbane
    {  460, 1799,  146173, 12, }, // Australia/Broken_Hill
    {  460, 1583,  146185,  8, }, // Australia/Darwin
    {  460, 3637,  146193,  6, }, // Australia/Eucla
    {  460, 1516,  146199,  6, }, // Australia/Hobart
    {  460, 8753,  146205,  9, }, // Australia/Lindeman
    {  460, 1547,  146214, 10, }, // Australia/Lord_Howe
    {  460, 1727,  146224,  8, }, // Australia/Melbourne
    {  460, 1762,  146232,  4, }, // Australia/Perth
    {  460, 1463,  146236,  7, }, // Australia/Sydney
    {  460, 6441,  146243, 11, }, // Europe/Amsterdam
    {  460, 6216,  146254,  7, }, // Europe/Andorra
    {  460, 3620,  146261, 12, }, // Europe/Astrakhan
    {  460, 2300,  146273,  4, }, // Europe/Athens
    {  460, 6336,  146277,  9, }, // Europe/Belgrade
    {  460, 4757,  146286,  8, }, // Europe/Berlin
    {  460, 6488,  146294, 13, }, // Europe/Bratislava
    {  460, 2167,  146307,  9, }, // Europe/Brussels
    {  460, 3928,  146316,  7, }, // Europe/Bucharest
    {  460, 3806,  146323,  8, }, // Europe/Budapest
    {  460, 8772,  146331, 10, }, // Europe/Busingen
    {  460, 2583,  146341,  6, }, // Europe/Chisinau
    {  460, 6289,  146347,  8, }, // Europe/Copenhagen
    {  460, 2338,  146355,  7, }, // Europe/Dublin
    {  460, 6307,  146362, 11, }, // Europe/Gibraltar
    {  460, 8788,  146373,  9, }, // Europe/Guernsey
    {  460, 6612,  146382, 10, }, // Europe/Helsinki
    {  460, 8804,  146392,  8, }, // Europe/Isle_of_Man
    {  460, 1130,  146400,  9, }, // Europe/Istanbul
    {  460, 8823,  146409,  7, }, // Europe/Jersey
    {  460, 3979,  146416, 12, }, // Europe/Kaliningrad
    {  460, 8929,  146428,  6, }, // Europe/Kirov
    {  460, 2527,  146434,  6, }, // Europe/Kyiv
    {  460, 3213,  146440,  8, }, // Europe/Lisbon
    {  460, 6506,  146448, 10, }, // Europe/Ljubljana
    {  460, 2501,  146458,  7, }, // Europe/London
    {  460, 6365,  146465, 12, }, // Europe/Luxembourg
    {  460, 4892,  146477,  6, }, // Europe/Madrid
    {  460, 6397,  146483,  5, }, // Europe/Malta
    {  460, 6582,  146488,  7, }, // Europe/Mariehamn
    {  460, 3693,  146495,  5, }, // Europe/Minsk
    {  460, 6410,  146500,  7, }, // Europe/Monaco
    {  460, 3537,  146507,  7, }, // Europe/Moscow
    {  460, 6458,  146514,  6, }, // Europe/Oslo
    {  460, 4284,  146520,  6, }, // Europe/Paris
    {  460, 6424,  146526, 11, }, // Europe/Podgorica
    {  460, 6275,  146537,  5, }, // Europe/Prague
    {  460, 8837,  146542,  6, }, // Europe/Riga
    {  460, 6324,  146548,  3, }, // Europe/Rome
    {  460, 4331,  146551,  6, }, // Europe/Samara
    {  460, 6470,  146557, 11, }, // Europe/San_Marino
    {  460, 6245,  146568, 10, }, // Europe/Sarajevo
    {  460, 4451,  146578,  7, }, // Europe/Saratov
    {  460, 8849,  146585,  9, }, // Europe/Simferopol
    {  460, 6383,  146594,  9, }, // Europe/Skopje
    {  460, 6599,  146603,  7, }, // Europe/Sofia
    {  460, 6523,  146610,  8, }, // Europe/Stockholm
    {  460, 8867,  146618,  6, }, // Europe/Tallinn
    {  460, 6202,  146624,  6, }, // Europe/Tirane
    {  460, 8882,  146630, 12, }, // Europe/Ulyanovsk
    {  460, 6352,  146642,  8, }, // Europe/Vaduz
    {  460, 6567,  146650,  9, }, // Europe/Vatican
    {  460, 6231,  146659,  6, }, // Europe/Vienna
    {  460, 8899,  146665, 10, }, // Europe/Vilnius
    {  460, 4727,  146675, 12, }, // Europe/Volgograd
    {  460, 3190,  146687, 11, }, // Europe/Warsaw
    {  460, 6261,  146698,  8, }, // Europe/Zagreb
    {  460, 6540,  146706,  6, }, // Europe/Zurich
    {  460, 5078,  146712, 13, }, // Indian/Antananarivo
    {  460, 6947,  146725,  6, }, // Indian/Chagos
    {  460, 6046,  146731,  8, }, // Indian/Christmas
    {  460, 6063,  146739,  5, }, // Indian/Cocos
    {  460, 5029,  146744,  6, }, // Indian/Comoro
    {  460, 6661,  146750, 11, }, // Indian/Kerguelen
    {  460, 7260,  146761,  5, }, // Indian/Mahe
    {  460, 7050,  146766,  8, }, // Indian/Maldives
    {  460, 4069,  146774,  5, }, // Indian/Mauritius
    {  460, 5098,  146779,  5, }, // Indian/Mayotte
    {  460, 7211,  146784,  8, }, // Indian/Reunion
    {  460, 4422,  146792,  6, }, // Pacific/Apia
    {  460,  932,  146798,  7, }, // Pacific/Auckland
    {  460, 3706,  146805, 10, }, // Pacific/Bougainville
    {  460, 3015,  146815,  4, }, // Pacific/Chatham
    {  460, 3157,  146819,  4, }, // Pacific/Chuuk
    {  460, 2237,  146823,  6, }, // Pacific/Easter
    {  460, 7360,  146829,  6, }, // Pacific/Efate
    {  460, 7327,  146835,  7, }, // Pacific/Fakaofo
    {  460, 3902,  146842,  4, }, // Pacific/Fiji
    {  460, 7343,  146846,  9, }, // Pacific/Funafuti
    {  460, 6678,  146855, 10, }, // Pacific/Galapagos
    {  460, 6696,  146865,  9, }, // Pacific/Gambier
    {  460, 3822,  146874, 12, }, // Pacific/Guadalcanal
    {  460, 6018,  146886,  7, }, // Pacific/Guam
    {  460, 3049,  146893,  7, }, // Pacific/Kanton
    {  460, 3998,  146900, 10, }, // Pacific/Kiritimati
    {  460, 7019,  146910,  7, }, // Pacific/Kosrae
    {  460, 2842,  146917, 11, }, // Pacific/Kwajalein
    {  460, 4877,  146928,  7, }, // Pacific/Majuro
    {  460, 4051,  146935,  6, }, // Pacific/Marquesas
    {  460, 8914,  146941,  7, }, // Pacific/Midway
    {  460, 7084,  146948,  6, }, // Pacific/Nauru
    {  460, 7132,  146954,  6, }, // Pacific/Niue
    {  460, 4168,  146960,  8, }, // Pacific/Norfolk
    {  460, 7098,  146968,  7, }, // Pacific/Noumea
    {  460, 3126,  146975, 13, }, // Pacific/Pago_Pago
    {  460, 7155,  146988,  5, }, // Pacific/Palau
    {  460, 7194,  146993,  7, }, // Pacific/Pitcairn
    {  460, 3096,  147000,  8, }, // Pacific/Pohnpei
    {  460, 4807,  147008, 16, }, // Pacific/Port_Moresby
    {  460, 6076,  147024, 11, }, // Pacific/Rarotonga
    {  460, 6031,  147035,  7, }, // Pacific/Saipan
    {  460, 7312,  147042,  6, }, // Pacific/Tahiti
    {  460, 6712,  147048,  6, }, // Pacific/Tarawa
    {  460, 4583,  147054, 10, }, // Pacific/Tongatapu
    {  460, 7392,  147064,  5, }, // Pacific/Wake
    {  460, 7405,  147069,  6, }, // Pacific/Wallis
    {  461,   14,       0,  6, }, // Africa/Asmara Northern Luri/Arabic/Iran
    {  461, 4435,       6,  8, }, // Africa/Sao_Tome
    {  461, 4252,      14,  8, }, // America/Asuncion
    {  461,  237,      22,  8, }, // America/Atikokan
    {  461, 7623,      30, 17, }, // America/Bahia_Banderas
    {  461, 3868,      47,  6, }, // America/Cancun
    {  461, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  461, 5723,      66,  7, }, // America/Curacao
    {  461,  481,      73, 13, }, // America/Indiana/Knox
    {  461, 7961,      86, 16, }, // America/Indiana/Marengo
    {  461, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  461, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  461, 8038,     139, 14, }, // America/Indiana/Vevay
    {  461, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  461, 8086,     171, 16, }, // America/Indiana/Winamac
    {  461, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  461, 5932,     207, 22, }, // America/Lower_Princes
    {  461, 8220,     229,  6, }, // America/Merida
    {  461, 1850,     235, 19, }, // America/Noronha
    {  461, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  461, 8329,     274, 20, }, // America/North_Dakota/Center
    {  461, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  461,  393,     317,  4, }, // America/Nuuk
    {  461, 6899,     321, 16, }, // America/Scoresbysund
    {  461, 8508,     337, 14, }, // America/St_Barthelemy
    {  461, 2061,     351, 10, }, // America/St_Johns
    {  461, 5863,     361,  9, }, // America/St_Kitts
    {  461, 5880,     370,  9, }, // America/St_Lucia
    {  461,  855,     379, 10, }, // America/St_Thomas
    {  461, 5913,     389, 11, }, // America/St_Vincent
    {  461, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  461, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  461, 1185,     448,  9, }, // Asia/Kathmandu
    {  461,  992,     457,  7, }, // Asia/Kolkata
    {  461, 1211,     464,  5, }, // Asia/Macau
    {  461, 1235,     477,  6, }, // Asia/Yangon
    {  461, 1395,     483,  5, }, // Atlantic/Faroe
    {  461, 6821,     488, 10, }, // Atlantic/St_Helena
    {  461, 2527,     514,  4, }, // Europe/Kyiv
    {  461, 7211,     547,  7, }, // Indian/Reunion
    {  461, 3157,     554,  5, }, // Pacific/Chuuk
    {  461, 3096,     586,  7, }, // Pacific/Pohnpei
    {  463,   14,       0,  6, }, // Africa/Asmara Northern Sami/Latin/Norway
    {  463, 4435,       6,  8, }, // Africa/Sao_Tome
    {  463, 4252,      14,  8, }, // America/Asuncion
    {  463,  237,      22,  8, }, // America/Atikokan
    {  463, 7623,      30, 17, }, // America/Bahia_Banderas
    {  463, 3868,      47,  6, }, // America/Cancun
    {  463, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  463, 5723,      66,  7, }, // America/Curacao
    {  463,  481,      73, 13, }, // America/Indiana/Knox
    {  463, 7961,      86, 16, }, // America/Indiana/Marengo
    {  463, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  463, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  463, 8038,     139, 14, }, // America/Indiana/Vevay
    {  463, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  463, 8086,     171, 16, }, // America/Indiana/Winamac
    {  463, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  463, 5932,     207, 22, }, // America/Lower_Princes
    {  463, 8220,     229,  6, }, // America/Merida
    {  463, 1850,     235, 19, }, // America/Noronha
    {  463, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  463, 8329,     274, 20, }, // America/North_Dakota/Center
    {  463, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  463,  393,     317,  4, }, // America/Nuuk
    {  463, 1878,   26712,  9, }, // America/Sao_Paulo
    {  463, 6899,     321, 16, }, // America/Scoresbysund
    {  463, 8508,   35936, 16, }, // America/St_Barthelemy
    {  463, 2061,     351, 10, }, // America/St_Johns
    {  463, 5863,     361,  9, }, // America/St_Kitts
    {  463, 5880,     370,  9, }, // America/St_Lucia
    {  463,  855,     379, 10, }, // America/St_Thomas
    {  463, 5913,     389, 11, }, // America/St_Vincent
    {  463, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  463, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  463, 1185,     448,  9, }, // Asia/Kathmandu
    {  463,  992,     457,  7, }, // Asia/Kolkata
    {  463, 1211,     464,  5, }, // Asia/Macau
    {  463, 1235,     477,  6, }, // Asia/Yangon
    {  463, 1395,     483,  5, }, // Atlantic/Faroe
    {  463, 6821,     488, 10, }, // Atlantic/St_Helena
    {  463, 2527,     514,  4, }, // Europe/Kyiv
    {  463, 7211,     547,  7, }, // Indian/Reunion
    {  463, 3157,     554,  5, }, // Pacific/Chuuk
    {  463, 3096,     586,  7, }, // Pacific/Pohnpei
    {  464, 2320,    4690,  5, }, // Africa/Cairo Northern Sami/Latin/Finland
    {  464, 7812,  147075, 15, }, // America/Danmarkshavn
    {  464, 8301,  147090, 20, }, // America/North_Dakota/Beulah
    {  464, 8329,  147110, 22, }, // America/North_Dakota/Center
    {  464, 8357,  147132, 23, }, // America/North_Dakota/New_Salem
    {  464, 4540,   64297,  8, }, // Asia/Damascus
    {  464, 4184,  147155,  6, }, // Asia/Irkutsk
    {  464, 4316,   33510,  8, }, // Asia/Kamchatka
    {  464, 4197,    4288, 11, }, // Asia/Krasnoyarsk
    {  464, 8675,  147161, 11, }, // Asia/Novokuznetsk
    {  464, 4408,   33576,  7, }, // Asia/Sakhalin
    {  464, 2772,    7842,  7, }, // Asia/Tehran
    {  464, 1021,  147172, 10, }, // Asia/Ulaanbaatar
    {  464, 4828,  147182,  6, }, // Asia/Yakutsk
    {  464, 1235,   58618,  7, }, // Asia/Yangon
    {  464, 3883,    4380, 13, }, // Asia/Yekaterinburg
    {  464, 3747,    7898,  7, }, // Asia/Yerevan
    {  464, 3663,  147188,  6, }, // Atlantic/Azores
    {  464, 6628,  147194,  7, }, // Atlantic/Canary
    {  464, 3727,   57380,  9, }, // Atlantic/Cape_Verde
    {  464, 7272,  147201, 13, }, // Atlantic/South_Georgia
    {  464, 2300,   93210,  6, }, // Europe/Athens
    {  464, 6336,   27015,  7, }, // Europe/Belgrade
    {  464, 2167,   22766,  7, }, // Europe/Brussels
    {  464, 3928,   27029,  8, }, // Europe/Bucharest
    {  464, 6289,   58657,  9, }, // Europe/Copenhagen
    {  464, 6612,  147214,  7, }, // Europe/Helsinki
    {  464, 8804,  147221,  8, }, // Europe/Isle_of_Man
    {  464, 3213,   21137,  6, }, // Europe/Lisbon
    {  464, 6365,   27065,  9, }, // Europe/Luxembourg
    {  464, 3537,   22880,  6, }, // Europe/Moscow
    {  464, 6275,   36285,  5, }, // Europe/Prague
    {  464, 6324,    6064,  4, }, // Europe/Rome
    {  464, 6202,   21213,  6, }, // Europe/Tirane
    {  464, 8882,    8261,  9, }, // Europe/Ulyanovsk
    {  464, 6231,   57515,  4, }, // Europe/Vienna
    {  464, 3190,   93239,  7, }, // Europe/Warsaw
    {  464, 6540,    4577,  6, }, // Europe/Zurich
    {  464, 6046,  147229, 11, }, // Indian/Christmas
    {  464, 6063,    4583,  5, }, // Indian/Cocos
    {  464, 7050,  147240, 11, }, // Indian/Maldives
    {  464, 4051,  147251, 11, }, // Pacific/Marquesas
    {  466,   14,       0,  6, }, // Africa/Asmara Northern Sotho/Latin/South Africa
    {  466, 4435,       6,  8, }, // Africa/Sao_Tome
    {  466, 4252,      14,  8, }, // America/Asuncion
    {  466,  237,      22,  8, }, // America/Atikokan
    {  466, 7623,      30, 17, }, // America/Bahia_Banderas
    {  466, 3868,      47,  6, }, // America/Cancun
    {  466, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  466, 5723,      66,  7, }, // America/Curacao
    {  466,  481,      73, 13, }, // America/Indiana/Knox
    {  466, 7961,      86, 16, }, // America/Indiana/Marengo
    {  466, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  466, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  466, 8038,     139, 14, }, // America/Indiana/Vevay
    {  466, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  466, 8086,     171, 16, }, // America/Indiana/Winamac
    {  466, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  466, 5932,     207, 22, }, // America/Lower_Princes
    {  466, 8220,     229,  6, }, // America/Merida
    {  466, 1850,     235, 19, }, // America/Noronha
    {  466, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  466, 8329,     274, 20, }, // America/North_Dakota/Center
    {  466, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  466,  393,     317,  4, }, // America/Nuuk
    {  466, 6899,     321, 16, }, // America/Scoresbysund
    {  466, 8508,     337, 14, }, // America/St_Barthelemy
    {  466, 2061,     351, 10, }, // America/St_Johns
    {  466, 5863,     361,  9, }, // America/St_Kitts
    {  466, 5880,     370,  9, }, // America/St_Lucia
    {  466,  855,     379, 10, }, // America/St_Thomas
    {  466, 5913,     389, 11, }, // America/St_Vincent
    {  466, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  466, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  466, 1185,     448,  9, }, // Asia/Kathmandu
    {  466,  992,     457,  7, }, // Asia/Kolkata
    {  466, 1211,     464,  5, }, // Asia/Macau
    {  466, 1235,     477,  6, }, // Asia/Yangon
    {  466, 1395,     483,  5, }, // Atlantic/Faroe
    {  466, 6821,     488, 10, }, // Atlantic/St_Helena
    {  466, 2527,     514,  4, }, // Europe/Kyiv
    {  466, 7211,     547,  7, }, // Indian/Reunion
    {  466, 3157,     554,  5, }, // Pacific/Chuuk
    {  466, 3096,     586,  7, }, // Pacific/Pohnpei
    {  467,   14,       0,  6, }, // Africa/Asmara North Ndebele/Latin/Zimbabwe
    {  467, 4435,       6,  8, }, // Africa/Sao_Tome
    {  467, 4252,      14,  8, }, // America/Asuncion
    {  467,  237,      22,  8, }, // America/Atikokan
    {  467, 7623,      30, 17, }, // America/Bahia_Banderas
    {  467, 3868,      47,  6, }, // America/Cancun
    {  467, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  467, 5723,      66,  7, }, // America/Curacao
    {  467,  481,      73, 13, }, // America/Indiana/Knox
    {  467, 7961,      86, 16, }, // America/Indiana/Marengo
    {  467, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  467, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  467, 8038,     139, 14, }, // America/Indiana/Vevay
    {  467, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  467, 8086,     171, 16, }, // America/Indiana/Winamac
    {  467, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  467, 5932,     207, 22, }, // America/Lower_Princes
    {  467, 8220,     229,  6, }, // America/Merida
    {  467, 1850,     235, 19, }, // America/Noronha
    {  467, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  467, 8329,     274, 20, }, // America/North_Dakota/Center
    {  467, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  467,  393,     317,  4, }, // America/Nuuk
    {  467, 6899,     321, 16, }, // America/Scoresbysund
    {  467, 8508,     337, 14, }, // America/St_Barthelemy
    {  467, 2061,     351, 10, }, // America/St_Johns
    {  467, 5863,     361,  9, }, // America/St_Kitts
    {  467, 5880,     370,  9, }, // America/St_Lucia
    {  467,  855,     379, 10, }, // America/St_Thomas
    {  467, 5913,     389, 11, }, // America/St_Vincent
    {  467, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  467, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  467, 1185,     448,  9, }, // Asia/Kathmandu
    {  467,  992,     457,  7, }, // Asia/Kolkata
    {  467, 1211,     464,  5, }, // Asia/Macau
    {  467, 1235,     477,  6, }, // Asia/Yangon
    {  467, 1395,     483,  5, }, // Atlantic/Faroe
    {  467, 6821,     488, 10, }, // Atlantic/St_Helena
    {  467, 2527,     514,  4, }, // Europe/Kyiv
    {  467, 7211,     547,  7, }, // Indian/Reunion
    {  467, 3157,     554,  5, }, // Pacific/Chuuk
    {  467, 3096,     586,  7, }, // Pacific/Pohnpei
    {  468, 5059,    3902, 11, }, // Africa/Addis_Ababa Norwegian Bokmal/Latin/Norway
    {  468, 7420,   44088,  5, }, // Africa/Algiers
    {  468,   14,       0,  6, }, // Africa/Asmara
    {  468, 2320,    4690,  5, }, // Africa/Cairo
    {  468, 5130,   93447, 13, }, // Africa/Dar_es_Salaam
    {  468, 5166,   63491,  8, }, // Africa/El_Aaiun
    {  468, 6869,   20418,  4, }, // Africa/Lome
    {  468, 4435,       6,  8, }, // Africa/Sao_Tome
    {  468, 4554,   26559,  9, }, // America/Araguaina
    {  468,  270,   20479,  7, }, // America/Argentina/Cordoba
    {  468, 7571,   20486,  7, }, // America/Argentina/Tucuman
    {  468, 4252,      14,  8, }, // America/Asuncion
    {  468,  237,      22,  8, }, // America/Atikokan
    {  468, 7623,   20493, 14, }, // America/Bahia_Banderas
    {  468, 7646,   35896,  5, }, // America/Belem
    {  468, 4361,   20507,  6, }, // America/Bogota
    {  468, 3868,      47,  6, }, // America/Cancun
    {  468, 5496,  147262, 11, }, // America/Cayman
    {  468, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  468, 3791,   26574,  6, }, // America/Cuiaba
    {  468, 5723,      66,  7, }, // America/Curacao
    {  468,  481,      73, 13, }, // America/Indiana/Knox
    {  468, 7961,      86, 16, }, // America/Indiana/Marengo
    {  468, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  468, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  468, 8038,     139, 14, }, // America/Indiana/Vevay
    {  468, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  468, 8086,     171, 16, }, // America/Indiana/Winamac
    {  468, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  468, 5932,     207, 22, }, // America/Lower_Princes
    {  468, 8153,   26588,  6, }, // America/Maceio
    {  468, 8220,     229,  6, }, // America/Merida
    {  468, 2949,  147273,  9, }, // America/Mexico_City
    {  468, 1850,     235, 19, }, // America/Noronha
    {  468, 8301,   93718, 19, }, // America/North_Dakota/Beulah
    {  468, 8329,   93737, 19, }, // America/North_Dakota/Center
    {  468, 8357,   93756, 22, }, // America/North_Dakota/New_Salem
    {  468,  393,     317,  4, }, // America/Nuuk
    {  468, 8477,   26704,  8, }, // America/Santarem
    {  468, 1878,   26712,  9, }, // America/Sao_Paulo
    {  468, 6899,     321, 16, }, // America/Scoresbysund
    {  468, 8508,   26721, 16, }, // America/St_Barthelemy
    {  468, 2061,     351, 10, }, // America/St_Johns
    {  468, 5863,     361,  9, }, // America/St_Kitts
    {  468, 5880,     370,  9, }, // America/St_Lucia
    {  468,  855,     379, 10, }, // America/St_Thomas
    {  468, 5913,     389, 11, }, // America/St_Vincent
    {  468, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  468, 5523,    5609,  5, }, // Asia/Aqtau
    {  468, 5368,   57205,  6, }, // Asia/Aqtobe
    {  468,  964,   58572,  9, }, // Asia/Ashgabat
    {  468, 3607,    4162,  6, }, // Asia/Baghdad
    {  468, 3778,    4187,  7, }, // Asia/Bishkek
    {  468, 4601,  147282,  6, }, // Asia/Chita
    {  468, 4540,    4201,  8, }, // Asia/Damascus
    {  468, 6159,   58581,  8, }, // Asia/Dushanbe
    {  468, 1259,  147288, 16, }, // Asia/Ho_Chi_Minh
    {  468, 2704,    4231,  8, }, // Asia/Hong_Kong
    {  468, 6992,  147304,  8, }, // Asia/Jayapura
    {  468, 4316,    4261, 10, }, // Asia/Kamchatka
    {  468, 1185,    5711,  8, }, // Asia/Kathmandu
    {  468,  992,     457,  7, }, // Asia/Kolkata
    {  468, 4197,    4288, 11, }, // Asia/Krasnoyarsk
    {  468, 1211,     464,  5, }, // Asia/Macau
    {  468, 6920,    4326,  6, }, // Asia/Muscat
    {  468, 2554,    5738,  7, }, // Asia/Nicosia
    {  468, 8708,    7770,  8, }, // Asia/Qostanay
    {  468, 4269,   26897,  9, }, // Asia/Qyzylorda
    {  468, 4781,   58610,  8, }, // Asia/Tashkent
    {  468, 2772,    7842,  7, }, // Asia/Tehran
    {  468, 1317,    7849,  6, }, // Asia/Thimphu
    {  468, 1021,   26923, 10, }, // Asia/Ulaanbaatar
    {  468, 1159,   26933,  6, }, // Asia/Urumqi
    {  468, 4828,    7873,  7, }, // Asia/Yakutsk
    {  468, 1235,     477,  6, }, // Asia/Yangon
    {  468, 3883,    4380, 13, }, // Asia/Yekaterinburg
    {  468, 3747,    7898,  7, }, // Asia/Yerevan
    {  468, 3663,  147312,  7, }, // Atlantic/Azores
    {  468, 6628,  147319, 11, }, // Atlantic/Canary
    {  468, 3727,  147330, 10, }, // Atlantic/Cape_Verde
    {  468, 1395,  147340,  8, }, // Atlantic/Faroe
    {  468, 2748,   36176,  9, }, // Atlantic/Reykjavik
    {  468, 7272,  147348, 11, }, // Atlantic/South_Georgia
    {  468, 6821,     488, 10, }, // Atlantic/St_Helena
    {  468, 2300,    4438,  5, }, // Europe/Athens
    {  468, 6336,    8060,  7, }, // Europe/Belgrade
    {  468, 2167,    4450,  7, }, // Europe/Brussels
    {  468, 3928,  147359,  9, }, // Europe/Bucharest
    {  468, 8772,   27037,  8, }, // Europe/Busingen
    {  468, 2583,   63785,  8, }, // Europe/Chisinau
    {  468, 6289,   58657,  9, }, // Europe/Copenhagen
    {  468, 6612,  147368, 11, }, // Europe/Helsinki
    {  468, 8804,    4492,  3, }, // Europe/Isle_of_Man
    {  468, 2527,     514,  4, }, // Europe/Kyiv
    {  468, 3213,   21137,  6, }, // Europe/Lisbon
    {  468, 6365,    4513,  9, }, // Europe/Luxembourg
    {  468, 3537,   22880,  6, }, // Europe/Moscow
    {  468, 6275,   36285,  5, }, // Europe/Prague
    {  468, 6324,    6064,  4, }, // Europe/Rome
    {  468, 6202,   21213,  6, }, // Europe/Tirane
    {  468, 8882,    8261,  9, }, // Europe/Ulyanovsk
    {  468, 6567,  147379, 13, }, // Europe/Vatican
    {  468, 6231,   57515,  4, }, // Europe/Vienna
    {  468, 3190,   58675,  8, }, // Europe/Warsaw
    {  468, 6540,    4577,  6, }, // Europe/Zurich
    {  468, 6046,  147392, 12, }, // Indian/Christmas
    {  468, 6063,  147404, 10, }, // Indian/Cocos
    {  468, 5029,  147414,  8, }, // Indian/Comoro
    {  468, 7260,   27115,  4, }, // Indian/Mahe
    {  468, 7050,  147422,  9, }, // Indian/Maldives
    {  468, 7211,     547,  7, }, // Indian/Reunion
    {  468, 3157,     554,  5, }, // Pacific/Chuuk
    {  468, 2237,  147431,  8, }, // Pacific/Easter
    {  468, 6678,  147439, 14, }, // Pacific/Galapagos
    {  468, 2723,    4611,  8, }, // Pacific/Honolulu
    {  468, 3049,    4619,  9, }, // Pacific/Kanton
    {  468, 4168,  147453, 10, }, // Pacific/Norfolk
    {  468, 7098,    4628,  6, }, // Pacific/Noumea
    {  468, 3096,     586,  7, }, // Pacific/Pohnpei
    {  470, 5059,    3902, 11, }, // Africa/Addis_Ababa Norwegian Nynorsk/Latin/Norway
    {  470, 7420,   44088,  5, }, // Africa/Algiers
    {  470,   14,       0,  6, }, // Africa/Asmara
    {  470, 2320,    4690,  5, }, // Africa/Cairo
    {  470, 5130,   93447, 13, }, // Africa/Dar_es_Salaam
    {  470, 5166,   63491,  8, }, // Africa/El_Aaiun
    {  470, 6869,   20418,  4, }, // Africa/Lome
    {  470, 4435,       6,  8, }, // Africa/Sao_Tome
    {  470, 4554,   26559,  9, }, // America/Araguaina
    {  470,  270,   20479,  7, }, // America/Argentina/Cordoba
    {  470, 7571,   20486,  7, }, // America/Argentina/Tucuman
    {  470, 4252,      14,  8, }, // America/Asuncion
    {  470,  237,      22,  8, }, // America/Atikokan
    {  470, 7623,   20493, 14, }, // America/Bahia_Banderas
    {  470, 7646,   35896,  5, }, // America/Belem
    {  470, 4361,   20507,  6, }, // America/Bogota
    {  470, 3868,      47,  6, }, // America/Cancun
    {  470, 5496,  147463, 11, }, // America/Cayman
    {  470, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  470, 3791,  147474,  6, }, // America/Cuiaba
    {  470, 5723,      66,  7, }, // America/Curacao
    {  470,  481,      73, 13, }, // America/Indiana/Knox
    {  470, 7961,      86, 16, }, // America/Indiana/Marengo
    {  470, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  470, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  470, 8038,     139, 14, }, // America/Indiana/Vevay
    {  470, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  470, 8086,     171, 16, }, // America/Indiana/Winamac
    {  470, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  470, 5932,     207, 22, }, // America/Lower_Princes
    {  470, 8153,   26588,  6, }, // America/Maceio
    {  470, 8220,    7126,  6, }, // America/Merida
    {  470, 2949,  147273,  9, }, // America/Mexico_City
    {  470, 1850,     235, 19, }, // America/Noronha
    {  470, 8301,   93718, 19, }, // America/North_Dakota/Beulah
    {  470, 8329,   93737, 19, }, // America/North_Dakota/Center
    {  470, 8357,   93756, 22, }, // America/North_Dakota/New_Salem
    {  470,  393,     317,  4, }, // America/Nuuk
    {  470, 8477,   26704,  8, }, // America/Santarem
    {  470, 1878,   26712,  9, }, // America/Sao_Paulo
    {  470, 6899,     321, 16, }, // America/Scoresbysund
    {  470, 8508,   26721, 16, }, // America/St_Barthelemy
    {  470, 2061,     351, 10, }, // America/St_Johns
    {  470, 5863,     361,  9, }, // America/St_Kitts
    {  470, 5880,     370,  9, }, // America/St_Lucia
    {  470,  855,     379, 10, }, // America/St_Thomas
    {  470, 5913,     389, 11, }, // America/St_Vincent
    {  470, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  470, 5523,    5609,  5, }, // Asia/Aqtau
    {  470, 5368,    7606,  6, }, // Asia/Aqtobe
    {  470,  964,  147480,  8, }, // Asia/Ashgabat
    {  470, 3607,    4162,  6, }, // Asia/Baghdad
    {  470, 3778,    4187,  7, }, // Asia/Bishkek
    {  470, 4601,  147282,  6, }, // Asia/Chita
    {  470, 4540,    4201,  8, }, // Asia/Damascus
    {  470, 6159,   58581,  8, }, // Asia/Dushanbe
    {  470, 1259,  147288, 16, }, // Asia/Ho_Chi_Minh
    {  470, 2704,    4231,  8, }, // Asia/Hong_Kong
    {  470, 4771,   26835,  5, }, // Asia/Hovd
    {  470, 6992,  147304,  8, }, // Asia/Jayapura
    {  470, 4316,    4261, 10, }, // Asia/Kamchatka
    {  470, 1185,    5711,  8, }, // Asia/Kathmandu
    {  470,  992,     457,  7, }, // Asia/Kolkata
    {  470, 4197,    4288, 11, }, // Asia/Krasnoyarsk
    {  470, 1211,     464,  5, }, // Asia/Macau
    {  470, 6920,    4326,  6, }, // Asia/Muscat
    {  470, 2554,    5738,  7, }, // Asia/Nicosia
    {  470, 8708,    7770,  8, }, // Asia/Qostanay
    {  470, 4269,   26897,  9, }, // Asia/Qyzylorda
    {  470, 4781,   58610,  8, }, // Asia/Tashkent
    {  470, 2772,    7842,  7, }, // Asia/Tehran
    {  470, 1317,    7849,  6, }, // Asia/Thimphu
    {  470, 1021,   26923, 10, }, // Asia/Ulaanbaatar
    {  470, 1159,   26933,  6, }, // Asia/Urumqi
    {  470, 4828,    7873,  7, }, // Asia/Yakutsk
    {  470, 1235,  147488,  6, }, // Asia/Yangon
    {  470, 3883,    4380, 13, }, // Asia/Yekaterinburg
    {  470, 3747,    7898,  7, }, // Asia/Yerevan
    {  470, 3663,  147494,  7, }, // Atlantic/Azores
    {  470, 6628,  147501, 11, }, // Atlantic/Canary
    {  470, 3727,  147330, 10, }, // Atlantic/Cape_Verde
    {  470, 1395,  147512,  8, }, // Atlantic/Faroe
    {  470, 2748,   36176,  9, }, // Atlantic/Reykjavik
    {  470, 7272,  147348, 11, }, // Atlantic/South_Georgia
    {  470, 6821,     488, 10, }, // Atlantic/St_Helena
    {  470, 2300,    4438,  5, }, // Europe/Athens
    {  470, 6336,    8060,  7, }, // Europe/Belgrade
    {  470, 2167,    4450,  7, }, // Europe/Brussels
    {  470, 3928,  147359,  9, }, // Europe/Bucharest
    {  470, 8772,   27037,  8, }, // Europe/Busingen
    {  470, 2583,   63785,  8, }, // Europe/Chisinau
    {  470, 6289,   58657,  9, }, // Europe/Copenhagen
    {  470, 6612,  147368, 11, }, // Europe/Helsinki
    {  470, 8804,    4492,  3, }, // Europe/Isle_of_Man
    {  470, 2527,     514,  4, }, // Europe/Kyiv
    {  470, 3213,   21137,  6, }, // Europe/Lisbon
    {  470, 6365,    4513,  9, }, // Europe/Luxembourg
    {  470, 3537,   22880,  6, }, // Europe/Moscow
    {  470, 6275,   36285,  5, }, // Europe/Prague
    {  470, 6324,    6064,  4, }, // Europe/Rome
    {  470, 6202,   21213,  6, }, // Europe/Tirane
    {  470, 8882,    8261,  9, }, // Europe/Ulyanovsk
    {  470, 6567,  147379, 13, }, // Europe/Vatican
    {  470, 6231,   57515,  4, }, // Europe/Vienna
    {  470, 3190,   58675,  8, }, // Europe/Warsaw
    {  470, 6540,    4577,  6, }, // Europe/Zurich
    {  470, 6046,  147392, 12, }, // Indian/Christmas
    {  470, 6063,  147520, 10, }, // Indian/Cocos
    {  470, 5029,  147530,  8, }, // Indian/Comoro
    {  470, 7260,   27115,  4, }, // Indian/Mahe
    {  470, 7050,  147538,  9, }, // Indian/Maldives
    {  470, 7211,     547,  7, }, // Indian/Reunion
    {  470, 3157,     554,  5, }, // Pacific/Chuuk
    {  470, 2237,  147431,  8, }, // Pacific/Easter
    {  470, 6678,  147547, 14, }, // Pacific/Galapagos
    {  470, 2723,    4611,  8, }, // Pacific/Honolulu
    {  470, 3049,    4619,  9, }, // Pacific/Kanton
    {  470, 4168,  147453, 10, }, // Pacific/Norfolk
    {  470, 7098,    4628,  6, }, // Pacific/Noumea
    {  470, 3096,     586,  7, }, // Pacific/Pohnpei
    {  471,   14,       0,  6, }, // Africa/Asmara Nuer/Latin/South Sudan
    {  471, 4435,       6,  8, }, // Africa/Sao_Tome
    {  471, 4252,      14,  8, }, // America/Asuncion
    {  471,  237,      22,  8, }, // America/Atikokan
    {  471, 7623,      30, 17, }, // America/Bahia_Banderas
    {  471, 3868,      47,  6, }, // America/Cancun
    {  471, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  471, 5723,      66,  7, }, // America/Curacao
    {  471,  481,      73, 13, }, // America/Indiana/Knox
    {  471, 7961,      86, 16, }, // America/Indiana/Marengo
    {  471, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  471, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  471, 8038,     139, 14, }, // America/Indiana/Vevay
    {  471, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  471, 8086,     171, 16, }, // America/Indiana/Winamac
    {  471, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  471, 5932,     207, 22, }, // America/Lower_Princes
    {  471, 8220,     229,  6, }, // America/Merida
    {  471, 1850,     235, 19, }, // America/Noronha
    {  471, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  471, 8329,     274, 20, }, // America/North_Dakota/Center
    {  471, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  471,  393,     317,  4, }, // America/Nuuk
    {  471, 6899,     321, 16, }, // America/Scoresbysund
    {  471, 8508,     337, 14, }, // America/St_Barthelemy
    {  471, 2061,     351, 10, }, // America/St_Johns
    {  471, 5863,     361,  9, }, // America/St_Kitts
    {  471, 5880,     370,  9, }, // America/St_Lucia
    {  471,  855,     379, 10, }, // America/St_Thomas
    {  471, 5913,     389, 11, }, // America/St_Vincent
    {  471, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  471, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  471, 1185,     448,  9, }, // Asia/Kathmandu
    {  471,  992,     457,  7, }, // Asia/Kolkata
    {  471, 1211,     464,  5, }, // Asia/Macau
    {  471, 1235,     477,  6, }, // Asia/Yangon
    {  471, 1395,     483,  5, }, // Atlantic/Faroe
    {  471, 6821,     488, 10, }, // Atlantic/St_Helena
    {  471, 2527,     514,  4, }, // Europe/Kyiv
    {  471, 7211,     547,  7, }, // Indian/Reunion
    {  471, 3157,     554,  5, }, // Pacific/Chuuk
    {  471, 3096,     586,  7, }, // Pacific/Pohnpei
    {  472,   14,       0,  6, }, // Africa/Asmara Nyanja/Latin/Malawi
    {  472, 4435,       6,  8, }, // Africa/Sao_Tome
    {  472, 4252,      14,  8, }, // America/Asuncion
    {  472,  237,      22,  8, }, // America/Atikokan
    {  472, 7623,      30, 17, }, // America/Bahia_Banderas
    {  472, 3868,      47,  6, }, // America/Cancun
    {  472, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  472, 5723,      66,  7, }, // America/Curacao
    {  472,  481,      73, 13, }, // America/Indiana/Knox
    {  472, 7961,      86, 16, }, // America/Indiana/Marengo
    {  472, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  472, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  472, 8038,     139, 14, }, // America/Indiana/Vevay
    {  472, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  472, 8086,     171, 16, }, // America/Indiana/Winamac
    {  472, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  472, 5932,     207, 22, }, // America/Lower_Princes
    {  472, 8220,     229,  6, }, // America/Merida
    {  472, 1850,     235, 19, }, // America/Noronha
    {  472, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  472, 8329,     274, 20, }, // America/North_Dakota/Center
    {  472, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  472,  393,     317,  4, }, // America/Nuuk
    {  472, 6899,     321, 16, }, // America/Scoresbysund
    {  472, 8508,     337, 14, }, // America/St_Barthelemy
    {  472, 2061,     351, 10, }, // America/St_Johns
    {  472, 5863,     361,  9, }, // America/St_Kitts
    {  472, 5880,     370,  9, }, // America/St_Lucia
    {  472,  855,     379, 10, }, // America/St_Thomas
    {  472, 5913,     389, 11, }, // America/St_Vincent
    {  472, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  472, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  472, 1185,     448,  9, }, // Asia/Kathmandu
    {  472,  992,     457,  7, }, // Asia/Kolkata
    {  472, 1211,     464,  5, }, // Asia/Macau
    {  472, 1235,     477,  6, }, // Asia/Yangon
    {  472, 1395,     483,  5, }, // Atlantic/Faroe
    {  472, 6821,     488, 10, }, // Atlantic/St_Helena
    {  472, 2527,     514,  4, }, // Europe/Kyiv
    {  472, 7211,     547,  7, }, // Indian/Reunion
    {  472, 3157,     554,  5, }, // Pacific/Chuuk
    {  472, 3096,     586,  7, }, // Pacific/Pohnpei
    {  473,   14,       0,  6, }, // Africa/Asmara Nyankole/Latin/Uganda
    {  473, 4435,       6,  8, }, // Africa/Sao_Tome
    {  473, 4252,      14,  8, }, // America/Asuncion
    {  473,  237,      22,  8, }, // America/Atikokan
    {  473, 7623,      30, 17, }, // America/Bahia_Banderas
    {  473, 3868,      47,  6, }, // America/Cancun
    {  473, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  473, 5723,      66,  7, }, // America/Curacao
    {  473,  481,      73, 13, }, // America/Indiana/Knox
    {  473, 7961,      86, 16, }, // America/Indiana/Marengo
    {  473, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  473, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  473, 8038,     139, 14, }, // America/Indiana/Vevay
    {  473, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  473, 8086,     171, 16, }, // America/Indiana/Winamac
    {  473, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  473, 5932,     207, 22, }, // America/Lower_Princes
    {  473, 8220,     229,  6, }, // America/Merida
    {  473, 1850,     235, 19, }, // America/Noronha
    {  473, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  473, 8329,     274, 20, }, // America/North_Dakota/Center
    {  473, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  473,  393,     317,  4, }, // America/Nuuk
    {  473, 6899,     321, 16, }, // America/Scoresbysund
    {  473, 8508,     337, 14, }, // America/St_Barthelemy
    {  473, 2061,     351, 10, }, // America/St_Johns
    {  473, 5863,     361,  9, }, // America/St_Kitts
    {  473, 5880,     370,  9, }, // America/St_Lucia
    {  473,  855,     379, 10, }, // America/St_Thomas
    {  473, 5913,     389, 11, }, // America/St_Vincent
    {  473, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  473, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  473, 1185,     448,  9, }, // Asia/Kathmandu
    {  473,  992,     457,  7, }, // Asia/Kolkata
    {  473, 1211,     464,  5, }, // Asia/Macau
    {  473, 1235,     477,  6, }, // Asia/Yangon
    {  473, 1395,     483,  5, }, // Atlantic/Faroe
    {  473, 6821,     488, 10, }, // Atlantic/St_Helena
    {  473, 2527,     514,  4, }, // Europe/Kyiv
    {  473, 7211,     547,  7, }, // Indian/Reunion
    {  473, 3157,     554,  5, }, // Pacific/Chuuk
    {  473, 3096,     586,  7, }, // Pacific/Pohnpei
    {  474, 7420,  147561,  7, }, // Africa/Algiers Occitan/Latin/France
    {  474,   14,       0,  6, }, // Africa/Asmara
    {  474, 5043,  147568,  6, }, // Africa/Djibouti
    {  474, 5243,  147574,  5, }, // Africa/Douala
    {  474, 4435,       6,  8, }, // Africa/Sao_Tome
    {  474, 4252,      14,  8, }, // America/Asuncion
    {  474,  237,      22,  8, }, // America/Atikokan
    {  474, 7623,      30, 17, }, // America/Bahia_Banderas
    {  474, 3868,      47,  6, }, // America/Cancun
    {  474, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  474, 5723,      66,  7, }, // America/Curacao
    {  474,  481,      73, 13, }, // America/Indiana/Knox
    {  474, 7961,      86, 16, }, // America/Indiana/Marengo
    {  474, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  474, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  474, 8038,     139, 14, }, // America/Indiana/Vevay
    {  474, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  474, 8086,     171, 16, }, // America/Indiana/Winamac
    {  474, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  474, 5932,     207, 22, }, // America/Lower_Princes
    {  474, 8220,     229,  6, }, // America/Merida
    {  474, 1850,     235, 19, }, // America/Noronha
    {  474, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  474, 8329,     274, 20, }, // America/North_Dakota/Center
    {  474, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  474,  393,     317,  4, }, // America/Nuuk
    {  474, 6899,     321, 16, }, // America/Scoresbysund
    {  474, 8508,     337, 14, }, // America/St_Barthelemy
    {  474, 2061,     351, 10, }, // America/St_Johns
    {  474, 5863,     361,  9, }, // America/St_Kitts
    {  474, 5880,     370,  9, }, // America/St_Lucia
    {  474,  855,     379, 10, }, // America/St_Thomas
    {  474, 5913,     389, 11, }, // America/St_Vincent
    {  474, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  474, 4466,  147579,  7, }, // Asia/Bangkok
    {  474, 1093,   65717,  5, }, // Asia/Dhaka
    {  474, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  474, 3560,  147586,  5, }, // Asia/Kabul
    {  474, 1185,     448,  9, }, // Asia/Kathmandu
    {  474,  992,   20838,  7, }, // Asia/Kolkata
    {  474, 1211,     464,  5, }, // Asia/Macau
    {  474, 4408,  147591,  7, }, // Asia/Sakhalin
    {  474, 3300,    7809,  8, }, // Asia/Singapore
    {  474, 1021,   26923, 10, }, // Asia/Ulaanbaatar
    {  474, 1235,     477,  6, }, // Asia/Yangon
    {  474, 3883,  147598, 12, }, // Asia/Yekaterinburg
    {  474, 3663,  147610,  6, }, // Atlantic/Azores
    {  474, 6628,   44481,  8, }, // Atlantic/Canary
    {  474, 3727,   44489,  8, }, // Atlantic/Cape_Verde
    {  474, 1395,     483,  5, }, // Atlantic/Faroe
    {  474, 6821,     488, 10, }, // Atlantic/St_Helena
    {  474, 1670,  147616,  8, }, // Australia/Adelaide
    {  474, 6216,  147624,  7, }, // Europe/Andorra
    {  474, 2300,   21046,  6, }, // Europe/Athens
    {  474, 6336,   27015,  7, }, // Europe/Belgrade
    {  474, 2167,  147631,  9, }, // Europe/Brussels
    {  474, 3928,  147640,  8, }, // Europe/Bucharest
    {  474, 3806,  147648,  8, }, // Europe/Budapest
    {  474, 2527,     514,  4, }, // Europe/Kyiv
    {  474, 3213,   93414,  7, }, // Europe/Lisbon
    {  474, 2501,   21152,  7, }, // Europe/London
    {  474, 6365,  147656,  9, }, // Europe/Luxembourg
    {  474, 6410,  147665,  7, }, // Europe/Monaco
    {  474, 3537,  147672,  6, }, // Europe/Moscow
    {  474, 4284,   21180,  5, }, // Europe/Paris
    {  474, 6275,   21185,  5, }, // Europe/Prague
    {  474, 6324,    6064,  4, }, // Europe/Rome
    {  474, 7050,  147678,  8, }, // Indian/Maldives
    {  474, 7211,     547,  7, }, // Indian/Reunion
    {  474, 3157,     554,  5, }, // Pacific/Chuuk
    {  474, 3096,     586,  7, }, // Pacific/Pohnpei
    {  475, 6788,  147686,  6, }, // Africa/Abidjan Occitan/Latin/Spain
    {  475, 6760,   74129,  4, }, // Africa/Accra
    {  475, 5059,   20370, 10, }, // Africa/Addis_Ababa
    {  475, 7420,  147692,  5, }, // Africa/Algiers
    {  475, 7435,    6313,  5, }, // Africa/Bissau
    {  475, 2320,  147697,  9, }, // Africa/Cairo
    {  475, 5043,  147706,  6, }, // Africa/Djibouti
    {  475, 4524,  147712,  6, }, // Africa/Khartoum
    {  475, 6869,   20418,  4, }, // Africa/Lome
    {  475, 5113,   44110, 10, }, // Africa/Mogadishu
    {  475, 5271,  147718,  6, }, // Africa/Ndjamena
    {  475, 6803,  147724,  8, }, // Africa/Nouakchott
    {  475, 6727,   20449,  8, }, // Africa/Ouagadougou
    {  475, 5225,   61373,  9, }, // Africa/Porto-Novo
    {  475, 4435,  147732,  9, }, // Africa/Sao_Tome
    {  475, 2866,   20467,  7, }, // Africa/Tripoli
    {  475, 6554,   20474,  5, }, // Africa/Tunis
    {  475, 4554,   26559,  9, }, // America/Araguaina
    {  475,  270,  147741,  6, }, // America/Argentina/Cordoba
    {  475, 7489,   44137, 12, }, // America/Argentina/Rio_Gallegos
    {  475, 7571,   20486,  7, }, // America/Argentina/Tucuman
    {  475, 7623,   64671, 17, }, // America/Bahia_Banderas
    {  475, 7646,  147747,  5, }, // America/Belem
    {  475, 5407,  147752,  6, }, // America/Belize
    {  475, 4361,   20507,  6, }, // America/Bogota
    {  475, 4345,  147758,  6, }, // America/Cayenne
    {  475, 5496,   44173,  6, }, // America/Cayman
    {  475, 5422,  147764, 10, }, // America/Costa_Rica
    {  475, 3791,   26574,  6, }, // America/Cuiaba
    {  475, 7869,   26580,  8, }, // America/Eirunepe
    {  475, 4612,  147774, 10, }, // America/Grand_Turk
    {  475, 5770,   20519,  7, }, // America/Grenada
    {  475, 5786,   20526,  9, }, // America/Guadeloupe
    {  475, 2281,  147784, 10, }, // America/Havana
    {  475, 8153,   26588,  6, }, // America/Maceio
    {  475, 1908,  147794,  8, }, // America/Manaus
    {  475, 5805,   20554,  9, }, // America/Martinique
    {  475, 2949,  147802, 15, }, // America/Mexico_City
    {  475, 2379,  147817,  9, }, // America/New_York
    {  475, 8301,  147826, 24, }, // America/North_Dakota/Beulah
    {  475, 8329,  147850, 24, }, // America/North_Dakota/Center
    {  475, 8357,  147874, 27, }, // America/North_Dakota/New_Salem
    {  475, 2356,   44275,  6, }, // America/Panama
    {  475, 3945,  147901, 15, }, // America/Port-au-Prince
    {  475, 5954,  147916, 14, }, // America/Port_of_Spain
    {  475, 8477,   26704,  8, }, // America/Santarem
    {  475, 2201,   64260, 17, }, // America/Santiago
    {  475, 1878,  147930,  9, }, // America/Sao_Paulo
    {  475, 8508,  147939, 14, }, // America/St_Barthelemy
    {  475, 2061,  147953, 22, }, // America/St_Johns
    {  475, 5863,  147975, 14, }, // America/St_Kitts
    {  475, 5880,   74394, 11, }, // America/St_Lucia
    {  475, 5913,  147989, 11, }, // America/St_Vincent
    {  475, 5688,  148000,  7, }, // America/Tortola
    {  475, 5511,  148007,  6, }, // Asia/Anadyr
    {  475, 5523,    5609,  5, }, // Asia/Aqtau
    {  475, 5368,    7606,  6, }, // Asia/Aqtobe
    {  475,  964,  148013,  7, }, // Asia/Ashgabat
    {  475, 3607,    4162,  6, }, // Asia/Baghdad
    {  475, 5534,  148020,  6, }, // Asia/Bahrain
    {  475, 3653,   20747,  4, }, // Asia/Baku
    {  475, 3571,  148026,  7, }, // Asia/Barnaul
    {  475, 4601,   74433,  5, }, // Asia/Chita
    {  475, 4540,   74438,  7, }, // Asia/Damascus
    {  475, 6159,  148033,  7, }, // Asia/Dushanbe
    {  475, 1259,  148040, 18, }, // Asia/Ho_Chi_Minh
    {  475, 1290,  148058,  9, }, // Asia/Jerusalem
    {  475, 1185,   20830,  8, }, // Asia/Kathmandu
    {  475, 1349,    7736,  7, }, // Asia/Makassar
    {  475, 6920,   20857,  7, }, // Asia/Muscat
    {  475, 5559,   73150,  5, }, // Asia/Qatar
    {  475, 8708,   64342,  8, }, // Asia/Qostanay
    {  475, 4269,   26897,  9, }, // Asia/Qyzylorda
    {  475, 3584,    7787,  4, }, // Asia/Riyadh
    {  475, 4408,  148067,  7, }, // Asia/Sakhalin
    {  475, 7245,   20886, 10, }, // Asia/Samarkand
    {  475, 3279,  148074,  4, }, // Asia/Seoul
    {  475, 4297,    7817, 13, }, // Asia/Srednekolymsk
    {  475, 4781,   91755,  7, }, // Asia/Tashkent
    {  475, 3915,  148078,  7, }, // Asia/Tbilisi
    {  475, 2772,    7842,  7, }, // Asia/Tehran
    {  475, 1317,   62605,  5, }, // Asia/Thimphu
    {  475, 2821,    4356,  5, }, // Asia/Tokyo
    {  475, 1159,   26933,  6, }, // Asia/Urumqi
    {  475, 3747,  148085,  6, }, // Asia/Yerevan
    {  475, 5671,   44467,  8, }, // Atlantic/Bermuda
    {  475, 3727,   26968, 10, }, // Atlantic/Cape_Verde
    {  475, 1395,  148091, 11, }, // Atlantic/Faroe
    {  475, 2748,   20997,  9, }, // Atlantic/Reykjavik
    {  475, 7272,  148102, 16, }, // Atlantic/South_Georgia
    {  475, 6821,  148118, 11, }, // Atlantic/St_Helena
    {  475, 1463,  148129,  6, }, // Australia/Sydney
    {  475, 3620,   44523,  8, }, // Europe/Astrakhan
    {  475, 3928,   21073,  8, }, // Europe/Bucharest
    {  475, 8804,  148135, 11, }, // Europe/Isle_of_Man
    {  475, 1130,  148146,  8, }, // Europe/Istanbul
    {  475, 8929,   44561,  5, }, // Europe/Kirov
    {  475, 2527,    8148,  4, }, // Europe/Kyiv
    {  475, 3213,   21137,  6, }, // Europe/Lisbon
    {  475, 6506,   21143,  9, }, // Europe/Ljubljana
    {  475, 2501,  148154,  7, }, // Europe/London
    {  475, 6470,  148161, 11, }, // Europe/San_Marino
    {  475, 4451,  148172,  7, }, // Europe/Saratov
    {  475, 8849,   44589, 10, }, // Europe/Simferopol
    {  475, 6383,  148179,  6, }, // Europe/Skopje
    {  475, 6599,   21199,  5, }, // Europe/Sofia
    {  475, 6523,   21204,  8, }, // Europe/Stockholm
    {  475, 8867,   22925,  6, }, // Europe/Tallinn
    {  475, 6202,   21213,  6, }, // Europe/Tirane
    {  475, 8882,  148185,  9, }, // Europe/Ulyanovsk
    {  475, 6567,  148194, 11, }, // Europe/Vatican
    {  475, 6231,   21230,  5, }, // Europe/Vienna
    {  475, 8899,   64490,  5, }, // Europe/Vilnius
    {  475, 3190,   44621,  8, }, // Europe/Warsaw
    {  475, 6540,  148205,  6, }, // Europe/Zurich
    {  475, 6046,  148211,  5, }, // Indian/Christmas
    {  475, 5029,   65504,  7, }, // Indian/Comoro
    {  475, 7260,   27115,  4, }, // Indian/Mahe
    {  475, 4069,   21252,  7, }, // Indian/Mauritius
    {  475, 2237,  148216, 14, }, // Pacific/Easter
    {  475, 3902,  148230,  4, }, // Pacific/Fiji
    {  475, 6678,   21277,  9, }, // Pacific/Galapagos
    {  475, 4051,   44648,  9, }, // Pacific/Marquesas
    {  475, 7155,   65550,  6, }, // Pacific/Palau
    {  475, 7312,   44657,  6, }, // Pacific/Tahiti
    {  476, 6788,  148234,  7, }, // Africa/Abidjan Odia/Odia/India
    {  476, 6760,  148241,  5, }, // Africa/Accra
    {  476, 5059,  148246, 12, }, // Africa/Addis_Ababa
    {  476, 7420,  148258,  8, }, // Africa/Algiers
    {  476,   14,  148266,  5, }, // Africa/Asmara
    {  476,   44,  148271,  6, }, // Africa/Bamako
    {  476, 5257,  148277,  7, }, // Africa/Bangui
    {  476, 6746,  148284,  7, }, // Africa/Banjul
    {  476, 7435,  148291,  5, }, // Africa/Bissau
    {  476, 4957,  148296, 13, }, // Africa/Blantyre
    {  476, 5287,  148309, 12, }, // Africa/Brazzaville
    {  476, 4922,  148321, 10, }, // Africa/Bujumbura
    {  476, 2320,  148331,  5, }, // Africa/Cairo
    {  476, 4117,  148336, 12, }, // Africa/Casablanca
    {  476, 7449,  148348,  5, }, // Africa/Ceuta
    {  476, 6773,  148353,  8, }, // Africa/Conakry
    {  476, 6840,  148361,  4, }, // Africa/Dakar
    {  476, 5130,  148365, 15, }, // Africa/Dar_es_Salaam
    {  476, 5043,  148380,  6, }, // Africa/Djibouti
    {  476, 5243,  148386,  4, }, // Africa/Douala
    {  476, 5166,  148390,  9, }, // Africa/El_Aaiun
    {  476, 6853,  148399, 10, }, // Africa/Freetown
    {  476, 4906,  148409,  9, }, // Africa/Gaborone
    {  476, 5015,  148418,  5, }, // Africa/Harare
    {  476, 4479,  148423, 11, }, // Africa/Johannesburg
    {  476, 4499,  148434,  4, }, // Africa/Juba
    {  476, 5151,  148438,  8, }, // Africa/Kampala
    {  476, 4524,  148446,  7, }, // Africa/Khartoum
    {  476, 4973,  148453,  6, }, // Africa/Kigali
    {  476, 5306,  148459,  8, }, // Africa/Kinshasa
    {  476, 4744,  148467,  7, }, // Africa/Lagos
    {  476, 5336,  148474, 12, }, // Africa/Libreville
    {  476, 6869,  148486,  5, }, // Africa/Lome
    {  476, 5211,  148491,  7, }, // Africa/Luanda
    {  476, 4939,  148498, 10, }, // Africa/Lubumbashi
    {  476, 5001,  148508,  6, }, // Africa/Lusaka
    {  476, 5322,  148514,  6, }, // Africa/Malabo
    {  476, 4987,  148520,  6, }, // Africa/Maputo
    {  476, 5197,  148526,  6, }, // Africa/Maseru
    {  476, 5182,  148532,  7, }, // Africa/Mbabane
    {  476, 5113,  148539,  8, }, // Africa/Mogadishu
    {  476, 7034,  148547,  7, }, // Africa/Monrovia
    {  476, 3853,  148554,  7, }, // Africa/Nairobi
    {  476, 5271,  148561,  6, }, // Africa/Ndjamena
    {  476, 5354,  148567,  5, }, // Africa/Niamey
    {  476, 6803,  148572,  7, }, // Africa/Nouakchott
    {  476, 6727,  148579,  8, }, // Africa/Ouagadougou
    {  476, 5225,  148587, 11, }, // Africa/Porto-Novo
    {  476, 4435,  148598,  8, }, // Africa/Sao_Tome
    {  476, 2866,  148606,  8, }, // Africa/Tripoli
    {  476, 6554,  148614,  7, }, // Africa/Tunis
    {  476, 4152,  148621, 10, }, // Africa/Windhoek
    {  476,  132,  148631,  5, }, // America/Adak
    {  476, 3346,  148636,  8, }, // America/Anchorage
    {  476, 5607,  148644,  8, }, // America/Anguilla
    {  476, 5624,  148652,  8, }, // America/Antigua
    {  476, 4554,  148660,  8, }, // America/Araguaina
    {  476,  166,  148668, 14, }, // America/Argentina/Buenos_Aires
    {  476,   91,  148682, 10, }, // America/Argentina/Catamarca
    {  476,  270,  148692,  8, }, // America/Argentina/Cordoba
    {  476,  441,  148700,  5, }, // America/Argentina/Jujuy
    {  476, 7462,  148705,  8, }, // America/Argentina/La_Rioja
    {  476,  565,  148713,  8, }, // America/Argentina/Mendoza
    {  476, 7489,  148721, 14, }, // America/Argentina/Rio_Gallegos
    {  476, 7520,  148735,  5, }, // America/Argentina/Salta
    {  476, 7544,  148740, 10, }, // America/Argentina/San_Juan
    {  476, 5580,  148750, 12, }, // America/Argentina/San_Luis
    {  476, 7571,  148762,  9, }, // America/Argentina/Tucuman
    {  476, 7597,  148771,  5, }, // America/Argentina/Ushuaia
    {  476, 5640,  148776,  5, }, // America/Aruba
    {  476, 4252,  148781,  9, }, // America/Asuncion
    {  476,  237,  148790,  9, }, // America/Atikokan
    {  476, 3679,  148799,  5, }, // America/Bahia
    {  476, 7623,  148804, 18, }, // America/Bahia_Banderas
    {  476, 5654,  148822, 10, }, // America/Barbados
    {  476, 7646,  148832,  5, }, // America/Belem
    {  476, 5407,  148837,  6, }, // America/Belize
    {  476, 7660,  148843, 15, }, // America/Blanc-Sablon
    {  476, 7681,  148858, 10, }, // America/Boa_Vista
    {  476, 4361,  148868,  6, }, // America/Bogota
    {  476, 7699,  148874,  5, }, // America/Boise
    {  476, 7713,  148879, 13, }, // America/Cambridge_Bay
    {  476, 7735,  148892, 15, }, // America/Campo_Grande
    {  476, 3868,  148907,  7, }, // America/Cancun
    {  476, 4694,  148914,  8, }, // America/Caracas
    {  476, 4345,  148922,  8, }, // America/Cayenne
    {  476, 5496,  148930,  9, }, // America/Cayman
    {  476, 2260,  148939,  6, }, // America/Chicago
    {  476, 7756,  148945,  8, }, // America/Chihuahua
    {  476, 7774,  148953, 15, }, // America/Ciudad_Juarez
    {  476, 5422,  148968, 11, }, // America/Costa_Rica
    {  476, 7796,  148979, 10, }, // America/Creston
    {  476, 3791,  148989,  5, }, // America/Cuiaba
    {  476, 5723,  148994,  6, }, // America/Curacao
    {  476, 7812,  149000, 13, }, // America/Danmarkshavn
    {  476, 7833,  149013,  5, }, // America/Dawson
    {  476, 7848,  149018, 12, }, // America/Dawson_Creek
    {  476,  805,  149030,  7, }, // America/Denver
    {  476, 3465,  149037,  9, }, // America/Detroit
    {  476, 5739,  149046,  8, }, // America/Dominica
    {  476,  893,  149054,  9, }, // America/Edmonton
    {  476, 7869,  149063,  7, }, // America/Eirunepe
    {  476, 5441,  149070, 13, }, // America/El_Salvador
    {  476, 7886,  149083, 13, }, // America/Fort_Nelson
    {  476, 7906,  149096, 10, }, // America/Fortaleza
    {  476, 7924,  149106,  9, }, // America/Glace_Bay
    {  476, 6881,  149115,  7, }, // America/Goose_Bay
    {  476, 4612,  149122, 14, }, // America/Grand_Turk
    {  476, 5770,  149136,  8, }, // America/Grenada
    {  476, 5786,  149144, 10, }, // America/Guadeloupe
    {  476, 3760,  149154,  9, }, // America/Guatemala
    {  476, 4841,  149163,  9, }, // America/Guayaquil
    {  476, 6932,  149172,  5, }, // America/Guyana
    {  476, 1939,  149177, 12, }, // America/Halifax
    {  476, 2281,  149189,  6, }, // America/Havana
    {  476, 7942,  149195,  9, }, // America/Hermosillo
    {  476,  348,  149204, 14, }, // America/Indiana/Indianapolis
    {  476,  481,  149218, 17, }, // America/Indiana/Knox
    {  476, 7961,  149235, 17, }, // America/Indiana/Marengo
    {  476, 7985,  149252, 22, }, // America/Indiana/Petersburg
    {  476, 8012,  149274, 19, }, // America/Indiana/Tell_City
    {  476, 8038,  149293, 15, }, // America/Indiana/Vevay
    {  476, 8060,  149308, 21, }, // America/Indiana/Vincennes
    {  476, 8086,  149329, 18, }, // America/Indiana/Winamac
    {  476, 8110,  149347,  7, }, // America/Inuvik
    {  476,  660,  149354, 10, }, // America/Iqaluit
    {  476, 2799,  149364,  7, }, // America/Jamaica
    {  476, 5380,  149371,  6, }, // America/Juneau
    {  476,  521,  149377,  9, }, // America/Kentucky/Louisville
    {  476, 8125,  149386, 22, }, // America/Kentucky/Monticello
    {  476, 5704,  149408, 12, }, // America/Kralendijk
    {  476, 4376,  149420,  8, }, // America/La_Paz
    {  476, 7169,  149428,  4, }, // America/Lima
    {  476, 3239,  149432, 13, }, // America/Los_Angeles
    {  476, 5932,  149445, 23, }, // America/Lower_Princes
    {  476, 8153,  149468,  5, }, // America/Maceio
    {  476, 8168,  149473,  7, }, // America/Managua
    {  476, 1908,  149480,  8, }, // America/Manaus
    {  476, 5897,  149488,  8, }, // America/Marigot
    {  476, 5805,  149496, 12, }, // America/Martinique
    {  476, 8184,  149508, 10, }, // America/Matamoros
    {  476, 2917,  149518,  9, }, // America/Mazatlan
    {  476, 8202,  149527,  8, }, // America/Menominee
    {  476, 8220,  149535,  6, }, // America/Merida
    {  476, 8235,  149541, 10, }, // America/Metlakatla
    {  476, 2949,  149551, 13, }, // America/Mexico_City
    {  476, 4391,  149564, 10, }, // America/Miquelon
    {  476, 8254,  149574,  8, }, // America/Moncton
    {  476, 8270,  149582,  8, }, // America/Monterrey
    {  476, 4098,  149590, 10, }, // America/Montevideo
    {  476, 5824,  149600, 11, }, // America/Montserrat
    {  476, 5481,  149611,  5, }, // America/Nassau
    {  476, 2379,  149616, 11, }, // America/New_York
    {  476, 8288,  149627,  4, }, // America/Nome
    {  476, 1850,  149631,  8, }, // America/Noronha
    {  476, 8301,  149639, 20, }, // America/North_Dakota/Beulah
    {  476, 8329,  149659, 21, }, // America/North_Dakota/Center
    {  476, 8357,  149680, 25, }, // America/North_Dakota/New_Salem
    {  476,  393,  149705,  5, }, // America/Nuuk
    {  476, 8388,  149710,  7, }, // America/Ojinaga
    {  476, 2356,  149717,  6, }, // America/Panama
    {  476, 6173,  149723, 10, }, // America/Paramaribo
    {  476, 2973,  149733,  8, }, // America/Phoenix
    {  476, 3945,  149741, 18, }, // America/Port-au-Prince
    {  476, 5954,  149759, 17, }, // America/Port_of_Spain
    {  476, 8404,  149776, 11, }, // America/Porto_Velho
    {  476, 5843,  149787, 12, }, // America/Puerto_Rico
    {  476, 4030,  149799, 15, }, // America/Punta_Arenas
    {  476, 8424,  149814, 14, }, // America/Rankin_Inlet
    {  476, 8445,  149828,  6, }, // America/Recife
    {  476, 1995,  149834,  6, }, // America/Regina
    {  476, 8460,  149840,  8, }, // America/Resolute
    {  476,  695,  149848, 12, }, // America/Rio_Branco
    {  476, 8477,  149860, 11, }, // America/Santarem
    {  476, 2201,  149871,  9, }, // America/Santiago
    {  476, 6111,  149880, 15, }, // America/Santo_Domingo
    {  476, 1878,  149895,  9, }, // America/Sao_Paulo
    {  476, 6899,  149904, 17, }, // America/Scoresbysund
    {  476, 8494,  149921,  5, }, // America/Sitka
    {  476, 8508,  149926, 18, }, // America/St_Barthelemy
    {  476, 2061,  149944, 12, }, // America/St_Johns
    {  476, 5863,  149956, 13, }, // America/St_Kitts
    {  476, 5880,  149969, 13, }, // America/St_Lucia
    {  476,  855,  149982, 14, }, // America/St_Thomas
    {  476, 5913,  149996, 17, }, // America/St_Vincent
    {  476, 8530,  150013, 15, }, // America/Swift_Current
    {  476, 5461,  150028, 11, }, // America/Tegucigalpa
    {  476, 5756,  150039,  4, }, // America/Thule
    {  476,  313,  150043,  7, }, // America/Tijuana
    {  476,  608,  150050,  8, }, // America/Toronto
    {  476, 5688,  150058,  7, }, // America/Tortola
    {  476, 2093,  150065,  9, }, // America/Vancouver
    {  476, 2144,  150074, 11, }, // America/Whitehorse
    {  476,  734,  150085,  8, }, // America/Winnipeg
    {  476, 8552,  150093,  8, }, // America/Yakutat
    {  476, 6001,  148346,  4, }, // Antarctica/Casey
    {  476, 6094,  150101,  7, }, // Antarctica/Davis
    {  476, 6133,  150108, 19, }, // Antarctica/DumontDUrville
    {  476, 8568,  150127,  8, }, // Antarctica/Macquarie
    {  476, 7066,  150135,  6, }, // Antarctica/Mawson
    {  476, 7113,  150141, 13, }, // Antarctica/McMurdo
    {  476, 8589,  150154,  7, }, // Antarctica/Palmer
    {  476, 7226,  150161,  6, }, // Antarctica/Rothera
    {  476, 7295,  150167,  4, }, // Antarctica/Syowa
    {  476, 8607,  150171,  6, }, // Antarctica/Troll
    {  476, 7374,  150177,  9, }, // Antarctica/Vostok
    {  476, 1429,  150186, 13, }, // Arctic/Longyearbyen
    {  476, 5570,  150199,  6, }, // Asia/Aden
    {  476, 5395,  150205,  6, }, // Asia/Almaty
    {  476, 3968,  150211,  6, }, // Asia/Amman
    {  476, 5511,  150217,  8, }, // Asia/Anadyr
    {  476, 5523,  150225,  5, }, // Asia/Aqtau
    {  476, 5368,  150230,  7, }, // Asia/Aqtobe
    {  476,  964,  150237, 11, }, // Asia/Ashgabat
    {  476, 8624,  150248,  6, }, // Asia/Atyrau
    {  476, 3607,  150254,  8, }, // Asia/Baghdad
    {  476, 5534,  150262,  8, }, // Asia/Bahrain
    {  476, 3653,  148992,  4, }, // Asia/Baku
    {  476, 4466,  150270, 11, }, // Asia/Bangkok
    {  476, 3571,  150281,  8, }, // Asia/Barnaul
    {  476, 4086,  150289,  7, }, // Asia/Beirut
    {  476, 3778,  150296,  8, }, // Asia/Bishkek
    {  476, 5989,  150304,  7, }, // Asia/Brunei
    {  476, 4601,  150311,  4, }, // Asia/Chita
    {  476, 4511,  150315,  6, }, // Asia/Colombo
    {  476, 4540,  150321,  9, }, // Asia/Damascus
    {  476, 1093,  150330,  4, }, // Asia/Dhaka
    {  476, 6192,  150334,  6, }, // Asia/Dili
    {  476, 3596,  150340,  5, }, // Asia/Dubai
    {  476, 6159,  150345,  7, }, // Asia/Dushanbe
    {  476, 8636,  150352, 10, }, // Asia/Famagusta
    {  476, 8651,  150362,  4, }, // Asia/Gaza
    {  476, 4795,  150366,  8, }, // Asia/Hebron
    {  476, 1259,  150374, 16, }, // Asia/Ho_Chi_Minh
    {  476, 2704,  150390,  5, }, // Asia/Hong_Kong
    {  476, 4771,  150395,  6, }, // Asia/Hovd
    {  476, 4184,  150401,  8, }, // Asia/Irkutsk
    {  476, 7006,  150409,  9, }, // Asia/Jakarta
    {  476, 6992,  150418,  6, }, // Asia/Jayapura
    {  476, 1290,  150424,  8, }, // Asia/Jerusalem
    {  476, 3560,  150432,  5, }, // Asia/Kabul
    {  476, 4316,  150437,  8, }, // Asia/Kamchatka
    {  476, 4239,  150445,  5, }, // Asia/Karachi
    {  476, 1185,  150450,  9, }, // Asia/Kathmandu
    {  476, 8661,  150459,  9, }, // Asia/Khandyga
    {  476,  992,  150468,  7, }, // Asia/Kolkata
    {  476, 4197,  150475, 13, }, // Asia/Krasnoyarsk
    {  476, 4859,  150488, 12, }, // Asia/Kuala_Lumpur
    {  476, 5976,  150500,  5, }, // Asia/Kuching
    {  476, 5547,  150505,  4, }, // Asia/Kuwait
    {  476, 1211,  150509,  5, }, // Asia/Macau
    {  476, 4017,  150514,  8, }, // Asia/Magadan
    {  476, 1349,  150522,  9, }, // Asia/Makassar
    {  476, 7182,  150531,  6, }, // Asia/Manila
    {  476, 6920,  150537,  8, }, // Asia/Muscat
    {  476, 2554,  150545,  7, }, // Asia/Nicosia
    {  476, 8675,  150552, 13, }, // Asia/Novokuznetsk
    {  476, 4135,  150565, 12, }, // Asia/Novosibirsk
    {  476, 4229,  150577,  5, }, // Asia/Omsk
    {  476, 7145,  150582,  6, }, // Asia/Oral
    {  476, 6961,  150588, 11, }, // Asia/Phnom_Penh
    {  476, 8693,  150599, 12, }, // Asia/Pontianak
    {  476, 4214,  150611, 11, }, // Asia/Pyongyang
    {  476, 5559,  150622,  5, }, // Asia/Qatar
    {  476, 8708,  150627,  7, }, // Asia/Qostanay
    {  476, 4269,  150634, 10, }, // Asia/Qyzylorda
    {  476, 3584,  150644,  4, }, // Asia/Riyadh
    {  476, 4408,  150648,  7, }, // Asia/Sakhalin
    {  476, 7245,  150655,  7, }, // Asia/Samarkand
    {  476, 3279,  150662,  4, }, // Asia/Seoul
    {  476, 1053,  150666,  6, }, // Asia/Shanghai
    {  476, 3300,  150672, 11, }, // Asia/Singapore
    {  476, 4297,  150683, 15, }, // Asia/Srednekolymsk
    {  476, 3263,  150698,  6, }, // Asia/Taipei
    {  476, 4781,  150704,  8, }, // Asia/Tashkent
    {  476, 3915,  150712,  8, }, // Asia/Tbilisi
    {  476, 2772,  150720,  8, }, // Asia/Tehran
    {  476, 1317,  150728,  6, }, // Asia/Thimphu
    {  476, 2821,  150734,  5, }, // Asia/Tokyo
    {  476, 4572,  150739,  6, }, // Asia/Tomsk
    {  476, 1021,  150745, 12, }, // Asia/Ulaanbaatar
    {  476, 1159,  150757,  6, }, // Asia/Urumqi
    {  476, 8722,  150763, 10, }, // Asia/Ust-Nera
    {  476, 6977,  150773, 11, }, // Asia/Vientiane
    {  476, 4710,  150784, 14, }, // Asia/Vladivostok
    {  476, 4828,  150798,  8, }, // Asia/Yakutsk
    {  476, 1235,  150806,  9, }, // Asia/Yangon
    {  476, 3883,  150815, 14, }, // Asia/Yekaterinburg
    {  476, 3747,  150829,  8, }, // Asia/Yerevan
    {  476, 3663,  150837,  7, }, // Atlantic/Azores
    {  476, 5671,  150844,  7, }, // Atlantic/Bermuda
    {  476, 6628,  150851,  6, }, // Atlantic/Canary
    {  476, 3727,  150857, 11, }, // Atlantic/Cape_Verde
    {  476, 1395,  150868,  5, }, // Atlantic/Faroe
    {  476, 8736,  150873,  7, }, // Atlantic/Madeira
    {  476, 2748,  150880, 10, }, // Atlantic/Reykjavik
    {  476, 7272,  150890, 13, }, // Atlantic/South_Georgia
    {  476, 6821,  150903, 14, }, // Atlantic/St_Helena
    {  476, 6644,  150917,  7, }, // Atlantic/Stanley
    {  476, 1670,  150924,  8, }, // Australia/Adelaide
    {  476, 1635,  150932, 10, }, // Australia/Brisbane
    {  476, 1799,  150942, 15, }, // Australia/Broken_Hill
    {  476, 1583,  150957,  8, }, // Australia/Darwin
    {  476, 3637,  150965,  5, }, // Australia/Eucla
    {  476, 1516,  150970,  9, }, // Australia/Hobart
    {  476, 8753,  150979, 13, }, // Australia/Lindeman
    {  476, 1547,  150992,  9, }, // Australia/Lord_Howe
    {  476, 1727,  151001, 10, }, // Australia/Melbourne
    {  476, 1762,  151011,  6, }, // Australia/Perth
    {  476, 1463,  151017,  5, }, // Australia/Sydney
    {  476, 6441,  151022, 10, }, // Europe/Amsterdam
    {  476, 6216,  151032,  6, }, // Europe/Andorra
    {  476, 3620,  151038, 11, }, // Europe/Astrakhan
    {  476, 2300,  151049,  6, }, // Europe/Athens
    {  476, 6336,  151055, 10, }, // Europe/Belgrade
    {  476, 4757,  151065,  7, }, // Europe/Berlin
    {  476, 6488,  151072, 11, }, // Europe/Bratislava
    {  476, 2167,  151083,  8, }, // Europe/Brussels
    {  476, 3928,  151091, 10, }, // Europe/Bucharest
    {  476, 3806,  151101, 10, }, // Europe/Budapest
    {  476, 8772,  151111,  9, }, // Europe/Busingen
    {  476, 2583,  151120,  6, }, // Europe/Chisinau
    {  476, 6289,  151126, 11, }, // Europe/Copenhagen
    {  476, 2338,  151137,  6, }, // Europe/Dublin
    {  476, 6307,  151143, 11, }, // Europe/Gibraltar
    {  476, 8788,  151154,  7, }, // Europe/Guernsey
    {  476, 6612,  151161,  8, }, // Europe/Helsinki
    {  476, 8804,  151169, 13, }, // Europe/Isle_of_Man
    {  476, 1130,  151182,  9, }, // Europe/Istanbul
    {  476, 8823,  151191,  5, }, // Europe/Jersey
    {  476, 3979,  151196, 13, }, // Europe/Kaliningrad
    {  476, 8929,  151209,  5, }, // Europe/Kirov
    {  476, 2527,  151214,  5, }, // Europe/Kyiv
    {  476, 3213,  151219,  7, }, // Europe/Lisbon
    {  476, 6506,  151226, 10, }, // Europe/Ljubljana
    {  476, 2501,  151236,  6, }, // Europe/London
    {  476, 6365,  151242,  9, }, // Europe/Luxembourg
    {  476, 4892,  151251,  8, }, // Europe/Madrid
    {  476, 6397,  151259,  5, }, // Europe/Malta
    {  476, 6582,  151264, 10, }, // Europe/Mariehamn
    {  476, 3693,  151274,  6, }, // Europe/Minsk
    {  476, 6410,  151280,  6, }, // Europe/Monaco
    {  476, 3537,  151286,  5, }, // Europe/Moscow
    {  476, 6458,  151291,  4, }, // Europe/Oslo
    {  476, 4284,  151295,  6, }, // Europe/Paris
    {  476, 6424,  151301,  9, }, // Europe/Podgorica
    {  476, 6275,  151310,  6, }, // Europe/Prague
    {  476, 8837,  151316,  4, }, // Europe/Riga
    {  476, 6324,  151320,  4, }, // Europe/Rome
    {  476, 4331,  151324,  6, }, // Europe/Samara
    {  476, 6470,  151330, 11, }, // Europe/San_Marino
    {  476, 6245,  151341,  8, }, // Europe/Sarajevo
    {  476, 4451,  151349,  8, }, // Europe/Saratov
    {  476, 8849,  151357, 11, }, // Europe/Simferopol
    {  476, 6383,  151368,  7, }, // Europe/Skopje
    {  476, 6599,  151375,  5, }, // Europe/Sofia
    {  476, 6523,  151380,  9, }, // Europe/Stockholm
    {  476, 8867,  151389,  7, }, // Europe/Tallinn
    {  476, 6202,  151396,  6, }, // Europe/Tirane
    {  476, 8882,  151402, 11, }, // Europe/Ulyanovsk
    {  476, 6352,  151413,  5, }, // Europe/Vaduz
    {  476, 6567,  151418,  8, }, // Europe/Vatican
    {  476, 6231,  151426,  5, }, // Europe/Vienna
    {  476, 8899,  151431,  8, }, // Europe/Vilnius
    {  476, 4727,  151439, 11, }, // Europe/Volgograd
    {  476, 3190,  151450,  6, }, // Europe/Warsaw
    {  476, 6261,  151456,  8, }, // Europe/Zagreb
    {  476, 6540,  151464,  6, }, // Europe/Zurich
    {  476, 5078,  151470, 13, }, // Indian/Antananarivo
    {  476, 6947,  151483,  7, }, // Indian/Chagos
    {  476, 6046,  151490, 10, }, // Indian/Christmas
    {  476, 6063,  151500,  7, }, // Indian/Cocos
    {  476, 5029,  151507,  6, }, // Indian/Comoro
    {  476, 6661,  151513,  9, }, // Indian/Kerguelen
    {  476, 7260,  151522,  4, }, // Indian/Mahe
    {  476, 7050,  151526,  8, }, // Indian/Maldives
    {  476, 4069,  151534,  7, }, // Indian/Mauritius
    {  476, 5098,  151541,  6, }, // Indian/Mayotte
    {  476, 7211,  151547, 10, }, // Indian/Reunion
    {  476, 4422,  151557,  4, }, // Pacific/Apia
    {  476,  932,  151561,  7, }, // Pacific/Auckland
    {  476, 3706,  151568, 13, }, // Pacific/Bougainville
    {  476, 3015,  151581,  7, }, // Pacific/Chatham
    {  476, 3157,  151588,  5, }, // Pacific/Chuuk
    {  476, 2237,  151593,  7, }, // Pacific/Easter
    {  476, 7360,  151600,  6, }, // Pacific/Efate
    {  476, 7327,  151606,  7, }, // Pacific/Fakaofo
    {  476, 3902,  151613,  4, }, // Pacific/Fiji
    {  476, 7343,  151617,  8, }, // Pacific/Funafuti
    {  476, 6678,  151625,  9, }, // Pacific/Galapagos
    {  476, 6696,  151634, 10, }, // Pacific/Gambier
    {  476, 3822,  151644, 11, }, // Pacific/Guadalcanal
    {  476, 6018,  149477,  4, }, // Pacific/Guam
    {  476, 2723,  151655,  8, }, // Pacific/Honolulu
    {  476, 3049,  151663, 10, }, // Pacific/Kanton
    {  476, 3998,  151673, 10, }, // Pacific/Kiritimati
    {  476, 7019,  151683,  6, }, // Pacific/Kosrae
    {  476, 2842,  151689, 11, }, // Pacific/Kwajalein
    {  476, 4877,  151700,  6, }, // Pacific/Majuro
    {  476, 4051,  151706, 13, }, // Pacific/Marquesas
    {  476, 8914,  151719,  7, }, // Pacific/Midway
    {  476, 7084,  151726,  5, }, // Pacific/Nauru
    {  476, 7132,  151731,  4, }, // Pacific/Niue
    {  476, 4168,  151735,  7, }, // Pacific/Norfolk
    {  476, 7098,  151742,  5, }, // Pacific/Noumea
    {  476, 3126,  151747,  9, }, // Pacific/Pago_Pago
    {  476, 7155,  151756,  5, }, // Pacific/Palau
    {  476, 7194,  151761, 10, }, // Pacific/Pitcairn
    {  476, 3096,  151771,  7, }, // Pacific/Pohnpei
    {  476, 4807,  151778, 15, }, // Pacific/Port_Moresby
    {  476, 6076,  151793, 10, }, // Pacific/Rarotonga
    {  476, 6031,  151803,  7, }, // Pacific/Saipan
    {  476, 7312,  151810,  6, }, // Pacific/Tahiti
    {  476, 6712,  151816,  5, }, // Pacific/Tarawa
    {  476, 4583,  151821, 10, }, // Pacific/Tongatapu
    {  476, 7392,  151831,  4, }, // Pacific/Wake
    {  476, 7405,  151835,  7, }, // Pacific/Wallis
    {  477,   14,       0,  6, }, // Africa/Asmara Oromo/Latin/Ethiopia
    {  477, 4435,       6,  8, }, // Africa/Sao_Tome
    {  477, 4252,      14,  8, }, // America/Asuncion
    {  477,  237,      22,  8, }, // America/Atikokan
    {  477, 7623,      30, 17, }, // America/Bahia_Banderas
    {  477, 3868,      47,  6, }, // America/Cancun
    {  477, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  477, 5723,      66,  7, }, // America/Curacao
    {  477,  481,      73, 13, }, // America/Indiana/Knox
    {  477, 7961,      86, 16, }, // America/Indiana/Marengo
    {  477, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  477, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  477, 8038,     139, 14, }, // America/Indiana/Vevay
    {  477, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  477, 8086,     171, 16, }, // America/Indiana/Winamac
    {  477, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  477, 5932,     207, 22, }, // America/Lower_Princes
    {  477, 8220,     229,  6, }, // America/Merida
    {  477, 1850,     235, 19, }, // America/Noronha
    {  477, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  477, 8329,     274, 20, }, // America/North_Dakota/Center
    {  477, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  477,  393,     317,  4, }, // America/Nuuk
    {  477, 6899,     321, 16, }, // America/Scoresbysund
    {  477, 8508,     337, 14, }, // America/St_Barthelemy
    {  477, 2061,     351, 10, }, // America/St_Johns
    {  477, 5863,     361,  9, }, // America/St_Kitts
    {  477, 5880,     370,  9, }, // America/St_Lucia
    {  477,  855,     379, 10, }, // America/St_Thomas
    {  477, 5913,     389, 11, }, // America/St_Vincent
    {  477, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  477, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  477, 1185,     448,  9, }, // Asia/Kathmandu
    {  477,  992,     457,  7, }, // Asia/Kolkata
    {  477, 1211,     464,  5, }, // Asia/Macau
    {  477, 1235,     477,  6, }, // Asia/Yangon
    {  477, 1395,     483,  5, }, // Atlantic/Faroe
    {  477, 6821,     488, 10, }, // Atlantic/St_Helena
    {  477, 2527,     514,  4, }, // Europe/Kyiv
    {  477, 7211,     547,  7, }, // Indian/Reunion
    {  477, 3157,     554,  5, }, // Pacific/Chuuk
    {  477, 3096,     586,  7, }, // Pacific/Pohnpei
    {  479,   14,       0,  6, }, // Africa/Asmara Osage/Osage/United States
    {  479, 4435,       6,  8, }, // Africa/Sao_Tome
    {  479, 4252,      14,  8, }, // America/Asuncion
    {  479,  237,      22,  8, }, // America/Atikokan
    {  479, 7623,      30, 17, }, // America/Bahia_Banderas
    {  479, 3868,      47,  6, }, // America/Cancun
    {  479, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  479, 5723,      66,  7, }, // America/Curacao
    {  479,  481,      73, 13, }, // America/Indiana/Knox
    {  479, 7961,      86, 16, }, // America/Indiana/Marengo
    {  479, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  479, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  479, 8038,     139, 14, }, // America/Indiana/Vevay
    {  479, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  479, 8086,     171, 16, }, // America/Indiana/Winamac
    {  479, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  479, 5932,     207, 22, }, // America/Lower_Princes
    {  479, 8220,     229,  6, }, // America/Merida
    {  479, 1850,     235, 19, }, // America/Noronha
    {  479, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  479, 8329,     274, 20, }, // America/North_Dakota/Center
    {  479, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  479,  393,     317,  4, }, // America/Nuuk
    {  479, 6899,     321, 16, }, // America/Scoresbysund
    {  479, 8508,     337, 14, }, // America/St_Barthelemy
    {  479, 2061,     351, 10, }, // America/St_Johns
    {  479, 5863,     361,  9, }, // America/St_Kitts
    {  479, 5880,     370,  9, }, // America/St_Lucia
    {  479,  855,     379, 10, }, // America/St_Thomas
    {  479, 5913,     389, 11, }, // America/St_Vincent
    {  479, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  479, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  479, 1185,     448,  9, }, // Asia/Kathmandu
    {  479,  992,     457,  7, }, // Asia/Kolkata
    {  479, 1211,     464,  5, }, // Asia/Macau
    {  479, 1235,     477,  6, }, // Asia/Yangon
    {  479, 1395,     483,  5, }, // Atlantic/Faroe
    {  479, 6821,     488, 10, }, // Atlantic/St_Helena
    {  479, 2527,     514,  4, }, // Europe/Kyiv
    {  479, 7211,     547,  7, }, // Indian/Reunion
    {  479, 3157,     554,  5, }, // Pacific/Chuuk
    {  479, 3096,     586,  7, }, // Pacific/Pohnpei
    {  481,   14,       0,  6, }, // Africa/Asmara Ossetic/Cyrillic/Russia
    {  481, 4435,       6,  8, }, // Africa/Sao_Tome
    {  481, 4252,      14,  8, }, // America/Asuncion
    {  481,  237,      22,  8, }, // America/Atikokan
    {  481, 7623,      30, 17, }, // America/Bahia_Banderas
    {  481, 3868,      47,  6, }, // America/Cancun
    {  481, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  481, 5723,      66,  7, }, // America/Curacao
    {  481,  481,      73, 13, }, // America/Indiana/Knox
    {  481, 7961,      86, 16, }, // America/Indiana/Marengo
    {  481, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  481, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  481, 8038,     139, 14, }, // America/Indiana/Vevay
    {  481, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  481, 8086,     171, 16, }, // America/Indiana/Winamac
    {  481, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  481, 5932,     207, 22, }, // America/Lower_Princes
    {  481, 8220,     229,  6, }, // America/Merida
    {  481, 1850,     235, 19, }, // America/Noronha
    {  481, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  481, 8329,     274, 20, }, // America/North_Dakota/Center
    {  481, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  481,  393,     317,  4, }, // America/Nuuk
    {  481, 6899,     321, 16, }, // America/Scoresbysund
    {  481, 8508,     337, 14, }, // America/St_Barthelemy
    {  481, 2061,     351, 10, }, // America/St_Johns
    {  481, 5863,     361,  9, }, // America/St_Kitts
    {  481, 5880,     370,  9, }, // America/St_Lucia
    {  481,  855,     379, 10, }, // America/St_Thomas
    {  481, 5913,     389, 11, }, // America/St_Vincent
    {  481, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  481, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  481, 1185,     448,  9, }, // Asia/Kathmandu
    {  481,  992,     457,  7, }, // Asia/Kolkata
    {  481, 1211,     464,  5, }, // Asia/Macau
    {  481, 3915,   35206,  6, }, // Asia/Tbilisi
    {  481, 1235,     477,  6, }, // Asia/Yangon
    {  481, 1395,     483,  5, }, // Atlantic/Faroe
    {  481, 6821,     488, 10, }, // Atlantic/St_Helena
    {  481, 2527,     514,  4, }, // Europe/Kyiv
    {  481, 3693,    3350,  5, }, // Europe/Minsk
    {  481, 3537,  151842,  6, }, // Europe/Moscow
    {  481, 7211,     547,  7, }, // Indian/Reunion
    {  481, 3157,     554,  5, }, // Pacific/Chuuk
    {  481, 3096,     586,  7, }, // Pacific/Pohnpei
    {  482,   14,       0,  6, }, // Africa/Asmara Papiamento/Latin/Curacao
    {  482, 4435,       6,  8, }, // Africa/Sao_Tome
    {  482, 4252,      14,  8, }, // America/Asuncion
    {  482,  237,      22,  8, }, // America/Atikokan
    {  482, 7623,      30, 17, }, // America/Bahia_Banderas
    {  482, 3868,      47,  6, }, // America/Cancun
    {  482, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  482, 5723,  151848,  6, }, // America/Curacao
    {  482,  481,      73, 13, }, // America/Indiana/Knox
    {  482, 7961,      86, 16, }, // America/Indiana/Marengo
    {  482, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  482, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  482, 8038,     139, 14, }, // America/Indiana/Vevay
    {  482, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  482, 8086,     171, 16, }, // America/Indiana/Winamac
    {  482, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  482, 5932,     207, 22, }, // America/Lower_Princes
    {  482, 8220,     229,  6, }, // America/Merida
    {  482, 1850,     235, 19, }, // America/Noronha
    {  482, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  482, 8329,     274, 20, }, // America/North_Dakota/Center
    {  482, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  482,  393,     317,  4, }, // America/Nuuk
    {  482, 2356,   20665,  6, }, // America/Panama
    {  482, 6111,  151854, 19, }, // America/Santo_Domingo
    {  482, 6899,     321, 16, }, // America/Scoresbysund
    {  482, 8508,     337, 14, }, // America/St_Barthelemy
    {  482, 2061,    4093, 11, }, // America/St_Johns
    {  482, 5863,    4104, 10, }, // America/St_Kitts
    {  482, 5880,    4114, 10, }, // America/St_Lucia
    {  482,  855,    4124, 11, }, // America/St_Thomas
    {  482, 5913,    4135, 12, }, // America/St_Vincent
    {  482, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  482, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  482, 1290,  151873,  9, }, // Asia/Jerusalem
    {  482, 1185,     448,  9, }, // Asia/Kathmandu
    {  482,  992,     457,  7, }, // Asia/Kolkata
    {  482, 1211,     464,  5, }, // Asia/Macau
    {  482, 1235,     477,  6, }, // Asia/Yangon
    {  482, 3663,  151882,  6, }, // Atlantic/Azores
    {  482, 3727,  151888, 10, }, // Atlantic/Cape_Verde
    {  482, 1395,     483,  5, }, // Atlantic/Faroe
    {  482, 6821,     488, 10, }, // Atlantic/St_Helena
    {  482, 2300,  151898,  6, }, // Europe/Athens
    {  482, 2527,     514,  4, }, // Europe/Kyiv
    {  482, 2501,  151904,  6, }, // Europe/London
    {  482, 6324,    6064,  4, }, // Europe/Rome
    {  482, 7211,     547,  7, }, // Indian/Reunion
    {  482, 3157,     554,  5, }, // Pacific/Chuuk
    {  482, 3096,     586,  7, }, // Pacific/Pohnpei
    {  484, 6788,  151910,  6, }, // Africa/Abidjan Pashto/Arabic/Afghanistan
    {  484, 6760,  151916,  4, }, // Africa/Accra
    {  484, 5059,  151920, 10, }, // Africa/Addis_Ababa
    {  484, 7420,  151930,  7, }, // Africa/Algiers
    {  484,   14,  151937,  6, }, // Africa/Asmara
    {  484,   44,  151943,  6, }, // Africa/Bamako
    {  484, 5257,  151949,  6, }, // Africa/Bangui
    {  484, 6746,   10545,  6, }, // Africa/Banjul
    {  484, 7435,  151955,  4, }, // Africa/Bissau
    {  484, 4957,  151959,  7, }, // Africa/Blantyre
    {  484, 5287,  151966,  8, }, // Africa/Brazzaville
    {  484, 4922,  151974,  7, }, // Africa/Bujumbura
    {  484, 2320,  151981,  5, }, // Africa/Cairo
    {  484, 4117,  151986, 10, }, // Africa/Casablanca
    {  484, 7449,  151996,  5, }, // Africa/Ceuta
    {  484, 6773,  152001,  6, }, // Africa/Conakry
    {  484, 6840,  152007,  5, }, // Africa/Dakar
    {  484, 5130,   10617, 10, }, // Africa/Dar_es_Salaam
    {  484, 5043,  152012,  5, }, // Africa/Djibouti
    {  484, 5243,   10633,  5, }, // Africa/Douala
    {  484, 5166,  152017,  5, }, // Africa/El_Aaiun
    {  484, 6853,  152022,  6, }, // Africa/Freetown
    {  484, 4906,  152028,  6, }, // Africa/Gaborone
    {  484, 5015,  152034,  5, }, // Africa/Harare
    {  484, 4479,  152039,  9, }, // Africa/Johannesburg
    {  484, 4499,   10673,  4, }, // Africa/Juba
    {  484, 5151,  152048,  6, }, // Africa/Kampala
    {  484, 4524,  152054,  5, }, // Africa/Khartoum
    {  484, 4973,  152059,  6, }, // Africa/Kigali
    {  484, 5306,  152065,  7, }, // Africa/Kinshasa
    {  484, 4744,  106900,  5, }, // Africa/Lagos
    {  484, 5336,  152072,  6, }, // Africa/Libreville
    {  484, 6869,  152078,  4, }, // Africa/Lome
    {  484, 5211,  152082,  5, }, // Africa/Luanda
    {  484, 4939,  152087,  7, }, // Africa/Lubumbashi
    {  484, 5001,  103343,  6, }, // Africa/Lusaka
    {  484, 5322,   10740,  6, }, // Africa/Malabo
    {  484, 4987,  152094,  6, }, // Africa/Maputo
    {  484, 5197,  152100,  5, }, // Africa/Maseru
    {  484, 5182,  152105,  7, }, // Africa/Mbabane
    {  484, 5113,  152112,  7, }, // Africa/Mogadishu
    {  484, 7034,  152119,  8, }, // Africa/Monrovia
    {  484, 3853,  106973,  7, }, // Africa/Nairobi
    {  484, 5271,  152127,  7, }, // Africa/Ndjamena
    {  484, 5354,  152134,  4, }, // Africa/Niamey
    {  484, 6803,  152138,  6, }, // Africa/Nouakchott
    {  484, 6727,  152144,  8, }, // Africa/Ouagadougou
    {  484, 5225,  152152, 11, }, // Africa/Porto-Novo
    {  484, 4435,  152163,  7, }, // Africa/Sao_Tome
    {  484, 2866,  107027,  7, }, // Africa/Tripoli
    {  484, 6554,   10837,  4, }, // Africa/Tunis
    {  484, 4152,  152170,  6, }, // Africa/Windhoek
    {  484,  132,  152176,  4, }, // America/Adak
    {  484, 3346,  152180,  7, }, // America/Anchorage
    {  484, 5607,  152187,  6, }, // America/Anguilla
    {  484, 5624,  152193,  5, }, // America/Antigua
    {  484, 4554,  152198,  7, }, // America/Araguaina
    {  484,  166,  152205, 11, }, // America/Argentina/Buenos_Aires
    {  484,   91,  152216,  8, }, // America/Argentina/Catamarca
    {  484,  270,  152224,  7, }, // America/Argentina/Cordoba
    {  484,  441,  152231,  5, }, // America/Argentina/Jujuy
    {  484, 7462,  152236,  7, }, // America/Argentina/La_Rioja
    {  484,  565,  152243,  7, }, // America/Argentina/Mendoza
    {  484, 7489,  152250, 11, }, // America/Argentina/Rio_Gallegos
    {  484, 7520,  152261,  5, }, // America/Argentina/Salta
    {  484, 7544,  152266,  8, }, // America/Argentina/San_Juan
    {  484, 5580,  152274,  8, }, // America/Argentina/San_Luis
    {  484, 7571,  152282,  6, }, // America/Argentina/Tucuman
    {  484, 7597,  152288,  7, }, // America/Argentina/Ushuaia
    {  484, 5640,  152295,  5, }, // America/Aruba
    {  484, 4252,  152300,  8, }, // America/Asuncion
    {  484,  237,  152308,  7, }, // America/Atikokan
    {  484, 3679,  152315,  4, }, // America/Bahia
    {  484, 7623,  152319, 12, }, // America/Bahia_Banderas
    {  484, 5654,  107198,  8, }, // America/Barbados
    {  484, 7646,   11020,  3, }, // America/Belem
    {  484, 5407,  152331,  4, }, // America/Belize
    {  484, 7660,  152335, 12, }, // America/Blanc-Sablon
    {  484, 7681,  152347,  9, }, // America/Boa_Vista
    {  484, 4361,  152356,  5, }, // America/Bogota
    {  484, 7699,  152361,  3, }, // America/Boise
    {  484, 7713,  152364,  9, }, // America/Cambridge_Bay
    {  484, 7735,  152373,  9, }, // America/Campo_Grande
    {  484, 3868,  152382,  5, }, // America/Cancun
    {  484, 4694,  152387,  7, }, // America/Caracas
    {  484, 4345,  152394,  4, }, // America/Cayenne
    {  484, 5496,  152398,  4, }, // America/Cayman
    {  484, 2260,  152402,  6, }, // America/Chicago
    {  484, 7756,  152408,  7, }, // America/Chihuahua
    {  484, 7774,  152415, 13, }, // America/Ciudad_Juarez
    {  484, 5422,  152428,  9, }, // America/Costa_Rica
    {  484, 7796,  152437,  6, }, // America/Creston
    {  484, 3791,  152443,  6, }, // America/Cuiaba
    {  484, 5723,  152449,  7, }, // America/Curacao
    {  484, 7812,  152456,  9, }, // America/Danmarkshavn
    {  484, 7833,   11176,  5, }, // America/Dawson
    {  484, 7848,  152465,  9, }, // America/Dawson_Creek
    {  484,  805,  152474,  4, }, // America/Denver
    {  484, 3465,  152478,  7, }, // America/Detroit
    {  484, 5739,  152485,  8, }, // America/Dominica
    {  484,  893,  152493,  8, }, // America/Edmonton
    {  484, 7869,  152501,  7, }, // America/Eirunepe
    {  484, 5441,  152508, 11, }, // America/El_Salvador
    {  484, 7886,  152519, 10, }, // America/Fort_Nelson
    {  484, 7906,  152529,  8, }, // America/Fortaleza
    {  484, 7924,  152537,  8, }, // America/Glace_Bay
    {  484, 6881,  152545,  6, }, // America/Goose_Bay
    {  484, 4612,  152551,  7, }, // America/Grand_Turk
    {  484, 5770,  152558,  6, }, // America/Grenada
    {  484, 5786,  152564,  6, }, // America/Guadeloupe
    {  484, 3760,  152570,  8, }, // America/Guatemala
    {  484, 4841,  152578,  6, }, // America/Guayaquil
    {  484, 6932,  152584,  5, }, // America/Guyana
    {  484, 1939,  152589,  6, }, // America/Halifax
    {  484, 2281,  152595,  5, }, // America/Havana
    {  484, 7942,  152600,  8, }, // America/Hermosillo
    {  484,  348,  152608, 13, }, // America/Indiana/Indianapolis
    {  484,  481,  152621, 12, }, // America/Indiana/Knox
    {  484, 7961,  152633, 15, }, // America/Indiana/Marengo
    {  484, 7985,  152648, 16, }, // America/Indiana/Petersburg
    {  484, 8012,  152664, 15, }, // America/Indiana/Tell_City
    {  484, 8038,  152679, 13, }, // America/Indiana/Vevay
    {  484, 8060,  152692, 16, }, // America/Indiana/Vincennes
    {  484, 8086,  152708, 14, }, // America/Indiana/Winamac
    {  484, 8110,  152722,  4, }, // America/Inuvik
    {  484,  660,  152726,  5, }, // America/Iqaluit
    {  484, 2799,  152731,  6, }, // America/Jamaica
    {  484, 5380,  152737,  4, }, // America/Juneau
    {  484,  521,  152741,  8, }, // America/Kentucky/Louisville
    {  484, 8125,  152749, 17, }, // America/Kentucky/Monticello
    {  484, 5704,  152766,  8, }, // America/Kralendijk
    {  484, 4376,  104164,  6, }, // America/La_Paz
    {  484, 7169,  152774,  4, }, // America/Lima
    {  484, 3239,  152778, 10, }, // America/Los_Angeles
    {  484, 5932,  152788, 16, }, // America/Lower_Princes
    {  484, 8153,  152804,  5, }, // America/Maceio
    {  484, 8168,  152809,  5, }, // America/Managua
    {  484, 1908,  152814,  5, }, // America/Manaus
    {  484, 5897,  152819,  7, }, // America/Marigot
    {  484, 5805,  152826,  8, }, // America/Martinique
    {  484, 8184,  152834,  9, }, // America/Matamoros
    {  484, 2917,  152843,  7, }, // America/Mazatlan
    {  484, 8202,  152850,  7, }, // America/Menominee
    {  484, 8220,  152857,  6, }, // America/Merida
    {  484, 8235,  152863, 10, }, // America/Metlakatla
    {  484, 2949,  152873, 10, }, // America/Mexico_City
    {  484, 4391,  152883,  8, }, // America/Miquelon
    {  484, 8254,  152891,  7, }, // America/Moncton
    {  484, 8270,  152898,  6, }, // America/Monterrey
    {  484, 4098,  152904,  9, }, // America/Montevideo
    {  484, 5824,  152913,  9, }, // America/Montserrat
    {  484, 5481,  152922,  5, }, // America/Nassau
    {  484, 2379,  152927,  7, }, // America/New_York
    {  484, 8288,   11540,  3, }, // America/Nome
    {  484, 1850,  152934,  7, }, // America/Noronha
    {  484, 8301,  152941, 19, }, // America/North_Dakota/Beulah
    {  484, 8329,  152960, 18, }, // America/North_Dakota/Center
    {  484, 8357,  152978, 22, }, // America/North_Dakota/New_Salem
    {  484,  393,  153000,  4, }, // America/Nuuk
    {  484, 8388,  153004,  6, }, // America/Ojinaga
    {  484, 2356,  107960,  6, }, // America/Panama
    {  484, 6173,  153010,  9, }, // America/Paramaribo
    {  484, 2973,  153019,  5, }, // America/Phoenix
    {  484, 3945,  153024, 15, }, // America/Port-au-Prince
    {  484, 5954,  153039, 14, }, // America/Port_of_Spain
    {  484, 8404,  153053, 11, }, // America/Porto_Velho
    {  484, 5843,  153064, 10, }, // America/Puerto_Rico
    {  484, 4030,  153074, 11, }, // America/Punta_Arenas
    {  484, 8424,  153085, 10, }, // America/Rankin_Inlet
    {  484, 8445,  153095,  6, }, // America/Recife
    {  484, 1995,  153101,  6, }, // America/Regina
    {  484, 8460,  153107,  7, }, // America/Resolute
    {  484,  695,  153114, 10, }, // America/Rio_Branco
    {  484, 8477,  153124,  6, }, // America/Santarem
    {  484, 2201,  153130,  7, }, // America/Santiago
    {  484, 6111,  153137, 12, }, // America/Santo_Domingo
    {  484, 1878,  153149,  9, }, // America/Sao_Paulo
    {  484, 6899,  153158, 11, }, // America/Scoresbysund
    {  484, 8494,  153169,  5, }, // America/Sitka
    {  484, 8508,  153174, 13, }, // America/St_Barthelemy
    {  484, 2061,  153187,  9, }, // America/St_Johns
    {  484, 5863,  153196,  8, }, // America/St_Kitts
    {  484, 5880,  153204, 10, }, // America/St_Lucia
    {  484,  855,  153214, 10, }, // America/St_Thomas
    {  484, 5913,  153224, 10, }, // America/St_Vincent
    {  484, 8530,  153234, 10, }, // America/Swift_Current
    {  484, 5461,  153244,  9, }, // America/Tegucigalpa
    {  484, 5756,   11947,  3, }, // America/Thule
    {  484,  313,  153253,  6, }, // America/Tijuana
    {  484,  608,  153259,  6, }, // America/Toronto
    {  484, 5688,  153265,  7, }, // America/Tortola
    {  484, 2093,  153272,  7, }, // America/Vancouver
    {  484, 2144,  153279,  9, }, // America/Whitehorse
    {  484,  734,  153288,  6, }, // America/Winnipeg
    {  484, 8552,  153294,  6, }, // America/Yakutat
    {  484, 6001,  153300,  4, }, // Antarctica/Casey
    {  484, 6094,  153304,  4, }, // Antarctica/Davis
    {  484, 6133,  153308, 14, }, // Antarctica/DumontDUrville
    {  484, 8568,  153322,  6, }, // Antarctica/Macquarie
    {  484, 7066,  153328,  5, }, // Antarctica/Mawson
    {  484, 7113,  153333,  6, }, // Antarctica/McMurdo
    {  484, 8589,  153339,  5, }, // Antarctica/Palmer
    {  484, 7226,  153344,  5, }, // Antarctica/Rothera
    {  484, 7295,  153349,  4, }, // Antarctica/Syowa
    {  484, 8607,  153353,  4, }, // Antarctica/Troll
    {  484, 7374,  153357,  6, }, // Antarctica/Vostok
    {  484, 1429,  153363,  9, }, // Arctic/Longyearbyen
    {  484, 5570,  153372,  3, }, // Asia/Aden
    {  484, 5395,  153375,  6, }, // Asia/Almaty
    {  484, 3968,  108355,  5, }, // Asia/Amman
    {  484, 5511,  153381,  6, }, // Asia/Anadyr
    {  484, 5523,  153387,  5, }, // Asia/Aqtau
    {  484, 5368,  153392,  5, }, // Asia/Aqtobe
    {  484,  964,  153397,  8, }, // Asia/Ashgabat
    {  484, 8624,  153405,  6, }, // Asia/Atyrau
    {  484, 3607,   12109,  5, }, // Asia/Baghdad
    {  484, 5534,  153411,  5, }, // Asia/Bahrain
    {  484, 3653,  152447,  4, }, // Asia/Baku
    {  484, 4466,  153416,  5, }, // Asia/Bangkok
    {  484, 3571,  104899,  5, }, // Asia/Barnaul
    {  484, 4086,  153421,  5, }, // Asia/Beirut
    {  484, 3778,  153426,  5, }, // Asia/Bishkek
    {  484, 5989,  153431,  6, }, // Asia/Brunei
    {  484, 4601,  108439,  4, }, // Asia/Chita
    {  484, 4511,  153437,  6, }, // Asia/Colombo
    {  484, 4540,   12162,  4, }, // Asia/Damascus
    {  484, 1093,  153443,  5, }, // Asia/Dhaka
    {  484, 6192,  153448,  3, }, // Asia/Dili
    {  484, 3596,  153451,  4, }, // Asia/Dubai
    {  484, 6159,  153455,  6, }, // Asia/Dushanbe
    {  484, 8636,  153461,  8, }, // Asia/Famagusta
    {  484, 8651,  153469,  3, }, // Asia/Gaza
    {  484, 4795,  153472,  5, }, // Asia/Hebron
    {  484, 1259,  153477, 12, }, // Asia/Ho_Chi_Minh
    {  484, 2704,  153489,  9, }, // Asia/Hong_Kong
    {  484, 4771,  153498,  4, }, // Asia/Hovd
    {  484, 4184,  153502,  7, }, // Asia/Irkutsk
    {  484, 7006,  105006,  6, }, // Asia/Jakarta
    {  484, 6992,  153509,  8, }, // Asia/Jayapura
    {  484, 1290,  153517,  6, }, // Asia/Jerusalem
    {  484, 3560,  153523,  4, }, // Asia/Kabul
    {  484, 4316,  153527,  8, }, // Asia/Kamchatka
    {  484, 4239,  153535,  5, }, // Asia/Karachi
    {  484, 1185,  153540,  6, }, // Asia/Kathmandu
    {  484, 8661,  153546,  6, }, // Asia/Khandyga
    {  484,  992,  153552,  6, }, // Asia/Kolkata
    {  484, 4197,  153558, 11, }, // Asia/Krasnoyarsk
    {  484, 4859,  153569,  9, }, // Asia/Kuala_Lumpur
    {  484, 5976,  153578,  5, }, // Asia/Kuching
    {  484, 5547,  153583,  4, }, // Asia/Kuwait
    {  484, 1211,  153587,  4, }, // Asia/Macau
    {  484, 4017,  153591,  5, }, // Asia/Magadan
    {  484, 1349,  153596,  6, }, // Asia/Makassar
    {  484, 7182,  153602,  5, }, // Asia/Manila
    {  484, 6920,   12368,  4, }, // Asia/Muscat
    {  484, 2554,  153607,  7, }, // Asia/Nicosia
    {  484, 8675,  153614, 10, }, // Asia/Novokuznetsk
    {  484, 4135,  153624,  9, }, // Asia/Novosibirsk
    {  484, 4229,  105160,  5, }, // Asia/Omsk
    {  484, 7145,  153633,  4, }, // Asia/Oral
    {  484, 6961,  153637,  7, }, // Asia/Phnom_Penh
    {  484, 8693,  153644,  8, }, // Asia/Pontianak
    {  484, 4214,  153652, 10, }, // Asia/Pyongyang
    {  484, 5559,   12436,  3, }, // Asia/Qatar
    {  484, 8708,  153662,  7, }, // Asia/Qostanay
    {  484, 4269,  153669, 10, }, // Asia/Qyzylorda
    {  484, 3584,   12458,  4, }, // Asia/Riyadh
    {  484, 4408,  153679,  6, }, // Asia/Sakhalin
    {  484, 7245,   12468,  6, }, // Asia/Samarkand
    {  484, 3279,  105233,  4, }, // Asia/Seoul
    {  484, 1053,  153685,  7, }, // Asia/Shanghai
    {  484, 3300,  153692,  8, }, // Asia/Singapore
    {  484, 4297,  153700, 13, }, // Asia/Srednekolymsk
    {  484, 3263,  153713,  5, }, // Asia/Taipei
    {  484, 4781,  153718,  6, }, // Asia/Tashkent
    {  484, 3915,  153724,  6, }, // Asia/Tbilisi
    {  484, 2772,  153730,  5, }, // Asia/Tehran
    {  484, 1317,  153735,  6, }, // Asia/Thimphu
    {  484, 2821,  153741,  5, }, // Asia/Tokyo
    {  484, 4572,   10750,  5, }, // Asia/Tomsk
    {  484, 1021,  153746, 10, }, // Asia/Ulaanbaatar
    {  484, 1159,  153756,  7, }, // Asia/Urumqi
    {  484, 8722,  153763,  8, }, // Asia/Ust-Nera
    {  484, 6977,  153771,  6, }, // Asia/Vientiane
    {  484, 4710,  153777, 10, }, // Asia/Vladivostok
    {  484, 4828,  153787,  7, }, // Asia/Yakutsk
    {  484, 1235,  153794,  6, }, // Asia/Yangon
    {  484, 3883,  153800, 10, }, // Asia/Yekaterinburg
    {  484, 3747,  153810,  6, }, // Asia/Yerevan
    {  484, 3663,  153816,  6, }, // Atlantic/Azores
    {  484, 5671,   12615,  6, }, // Atlantic/Bermuda
    {  484, 6628,  153822,  5, }, // Atlantic/Canary
    {  484, 3727,  153827,  7, }, // Atlantic/Cape_Verde
    {  484, 1395,   12638,  4, }, // Atlantic/Faroe
    {  484, 8736,  153834,  5, }, // Atlantic/Madeira
    {  484, 2748,  153839,  8, }, // Atlantic/Reykjavik
    {  484, 7272,  153847, 12, }, // Atlantic/South_Georgia
    {  484, 6821,  153859, 11, }, // Atlantic/St_Helena
    {  484, 6644,  153870,  5, }, // Atlantic/Stanley
    {  484, 1670,  153875,  7, }, // Australia/Adelaide
    {  484, 1635,  153882,  6, }, // Australia/Brisbane
    {  484, 1799,  153888,  8, }, // Australia/Broken_Hill
    {  484, 1583,  153896,  5, }, // Australia/Darwin
    {  484, 3637,  153901,  6, }, // Australia/Eucla
    {  484, 1516,  153907,  6, }, // Australia/Hobart
    {  484, 8753,  153913,  8, }, // Australia/Lindeman
    {  484, 1547,  153921,  8, }, // Australia/Lord_Howe
    {  484, 1727,  153929,  7, }, // Australia/Melbourne
    {  484, 1762,  153936,  3, }, // Australia/Perth
    {  484, 1463,  153939,  4, }, // Australia/Sydney
    {  484, 6441,  153943,  8, }, // Europe/Amsterdam
    {  484, 6216,  153951,  6, }, // Europe/Andorra
    {  484, 3620,  153957,  9, }, // Europe/Astrakhan
    {  484, 2300,  153966,  5, }, // Europe/Athens
    {  484, 6336,  153971,  5, }, // Europe/Belgrade
    {  484, 4757,  153976,  4, }, // Europe/Berlin
    {  484, 6488,  153980,  9, }, // Europe/Bratislava
    {  484, 2167,  153989,  6, }, // Europe/Brussels
    {  484, 3928,  153995,  6, }, // Europe/Bucharest
    {  484, 3806,  154001,  6, }, // Europe/Budapest
    {  484, 8772,  154007,  7, }, // Europe/Busingen
    {  484, 2583,  154014,  6, }, // Europe/Chisinau
    {  484, 6289,  154020,  9, }, // Europe/Copenhagen
    {  484, 2338,  154029,  4, }, // Europe/Dublin
    {  484, 6307,  154033, 10, }, // Europe/Gibraltar
    {  484, 8788,  154043,  5, }, // Europe/Guernsey
    {  484, 6612,  154048,  7, }, // Europe/Helsinki
    {  484, 8804,  154055, 12, }, // Europe/Isle_of_Man
    {  484, 1130,  154067,  7, }, // Europe/Istanbul
    {  484, 8823,  105685,  4, }, // Europe/Jersey
    {  484, 3979,  154074, 10, }, // Europe/Kaliningrad
    {  484, 8929,  154084,  5, }, // Europe/Kirov
    {  484, 2527,  154089,  3, }, // Europe/Kyiv
    {  484, 3213,  154092,  6, }, // Europe/Lisbon
    {  484, 6506,  154098,  7, }, // Europe/Ljubljana
    {  484, 2501,   12926,  4, }, // Europe/London
    {  484, 6365,  154105, 10, }, // Europe/Luxembourg
    {  484, 4892,  154115,  6, }, // Europe/Madrid
    {  484, 6397,  154121,  5, }, // Europe/Malta
    {  484, 6582,  154126,  7, }, // Europe/Mariehamn
    {  484, 3693,  154133,  4, }, // Europe/Minsk
    {  484, 6410,  105768,  6, }, // Europe/Monaco
    {  484, 3537,  105774,  5, }, // Europe/Moscow
    {  484, 6458,  105779,  5, }, // Europe/Oslo
    {  484, 4284,  154137,  5, }, // Europe/Paris
    {  484, 6424,  154142,  9, }, // Europe/Podgorica
    {  484, 6275,  154151,  4, }, // Europe/Prague
    {  484, 8837,  152821,  4, }, // Europe/Riga
    {  484, 6324,   10756,  3, }, // Europe/Rome
    {  484, 4331,  106756,  5, }, // Europe/Samara
    {  484, 6470,  154155, 10, }, // Europe/San_Marino
    {  484, 6245,  154165,  8, }, // Europe/Sarajevo
    {  484, 4451,  154173,  5, }, // Europe/Saratov
    {  484, 8849,  154178,  9, }, // Europe/Simferopol
    {  484, 6383,  154187,  5, }, // Europe/Skopje
    {  484, 6599,  154192,  5, }, // Europe/Sofia
    {  484, 6523,  154197,  7, }, // Europe/Stockholm
    {  484, 8867,   13056,  5, }, // Europe/Tallinn
    {  484, 6202,   13061,  5, }, // Europe/Tirane
    {  484, 8882,  154204,  8, }, // Europe/Ulyanovsk
    {  484, 6352,  154212,  5, }, // Europe/Vaduz
    {  484, 6567,  154217,  7, }, // Europe/Vatican
    {  484, 6231,  154224,  5, }, // Europe/Vienna
    {  484, 8899,  154229,  7, }, // Europe/Vilnius
    {  484, 4727,  154236,  9, }, // Europe/Volgograd
    {  484, 3190,  105924,  5, }, // Europe/Warsaw
    {  484, 6261,  154245,  4, }, // Europe/Zagreb
    {  484, 6540,  154249,  4, }, // Europe/Zurich
    {  484, 5078,  154253, 11, }, // Indian/Antananarivo
    {  484, 6947,  154264,  5, }, // Indian/Chagos
    {  484, 6046,  154269,  6, }, // Indian/Christmas
    {  484, 6063,  154275,  5, }, // Indian/Cocos
    {  484, 5029,  105969,  6, }, // Indian/Comoro
    {  484, 6661,  154280,  7, }, // Indian/Kerguelen
    {  484, 7260,   13173,  4, }, // Indian/Mahe
    {  484, 7050,  154287,  6, }, // Indian/Maldives
    {  484, 4069,  154293,  7, }, // Indian/Mauritius
    {  484, 5098,  154300,  5, }, // Indian/Mayotte
    {  484, 7211,  154305,  8, }, // Indian/Reunion
    {  484, 4422,  154313,  4, }, // Pacific/Apia
    {  484,  932,  154317,  5, }, // Pacific/Auckland
    {  484, 3706,  154322,  8, }, // Pacific/Bougainville
    {  484, 3015,  154330,  5, }, // Pacific/Chatham
    {  484, 3157,  154335,  3, }, // Pacific/Chuuk
    {  484, 2237,  154338,  5, }, // Pacific/Easter
    {  484, 7360,  154343,  4, }, // Pacific/Efate
    {  484, 7327,  154347,  6, }, // Pacific/Fakaofo
    {  484, 3902,  154353,  3, }, // Pacific/Fiji
    {  484, 7343,   13250,  8, }, // Pacific/Funafuti
    {  484, 6678,  154356,  7, }, // Pacific/Galapagos
    {  484, 6696,  154363,  6, }, // Pacific/Gambier
    {  484, 3822,  154369, 10, }, // Pacific/Guadalcanal
    {  484, 6018,  152811,  4, }, // Pacific/Guam
    {  484, 2723,  154379,  8, }, // Pacific/Honolulu
    {  484, 3049,  154387,  7, }, // Pacific/Kanton
    {  484, 3998,  154394,  9, }, // Pacific/Kiritimati
    {  484, 7019,  154403,  6, }, // Pacific/Kosrae
    {  484, 2842,  154409,  7, }, // Pacific/Kwajalein
    {  484, 4877,  154416,  5, }, // Pacific/Majuro
    {  484, 4051,  154421,  8, }, // Pacific/Marquesas
    {  484, 8914,  154429,  5, }, // Pacific/Midway
    {  484, 7084,  154434,  5, }, // Pacific/Nauru
    {  484, 7132,  154439,  4, }, // Pacific/Niue
    {  484, 4168,  154443,  5, }, // Pacific/Norfolk
    {  484, 7098,   13355,  5, }, // Pacific/Noumea
    {  484, 3126,  154448,  9, }, // Pacific/Pago_Pago
    {  484, 7155,  154457,  4, }, // Pacific/Palau
    {  484, 7194,  154461,  8, }, // Pacific/Pitcairn
    {  484, 3096,  154469,  5, }, // Pacific/Pohnpei
    {  484, 4807,  154474, 11, }, // Pacific/Port_Moresby
    {  484, 6076,  154485,  9, }, // Pacific/Rarotonga
    {  484, 6031,  154494,  5, }, // Pacific/Saipan
    {  484, 7312,  154499,  5, }, // Pacific/Tahiti
    {  484, 6712,   13417,  6, }, // Pacific/Tarawa
    {  484, 4583,  154504,  9, }, // Pacific/Tongatapu
    {  484, 7392,  106252,  3, }, // Pacific/Wake
    {  484, 7405,  154513,  4, }, // Pacific/Wallis
    {  485, 5015,  154517,  5, }, // Africa/Harare Pashto/Arabic/Pakistan
    {  485, 5932,  154522, 25, }, // America/Lower_Princes
    {  486, 6788,  154547,  6, }, // Africa/Abidjan Persian/Arabic/Iran
    {  486, 6760,  151916,  4, }, // Africa/Accra
    {  486, 5059,  154553, 10, }, // Africa/Addis_Ababa
    {  486, 7420,  154563,  7, }, // Africa/Algiers
    {  486,   14,  154570,  5, }, // Africa/Asmara
    {  486,   44,  154575,  6, }, // Africa/Bamako
    {  486, 5257,  154581,  5, }, // Africa/Bangui
    {  486, 6746,   10545,  6, }, // Africa/Banjul
    {  486, 7435,  154586,  6, }, // Africa/Bissau
    {  486, 4957,  154592,  8, }, // Africa/Blantyre
    {  486, 5287,  154600,  7, }, // Africa/Brazzaville
    {  486, 4922,   10572,  9, }, // Africa/Bujumbura
    {  486, 2320,  151981,  5, }, // Africa/Cairo
    {  486, 4117,  154607, 10, }, // Africa/Casablanca
    {  486, 7449,  154617,  4, }, // Africa/Ceuta
    {  486, 6773,  103214,  7, }, // Africa/Conakry
    {  486, 6840,  154621,  5, }, // Africa/Dakar
    {  486, 5130,  103226,  9, }, // Africa/Dar_es_Salaam
    {  486, 5043,  154626,  6, }, // Africa/Djibouti
    {  486, 5243,  154632,  5, }, // Africa/Douala
    {  486, 5166,  154637,  6, }, // Africa/El_Aaiun
    {  486, 6853,  154643,  8, }, // Africa/Freetown
    {  486, 4906,  106851,  7, }, // Africa/Gaborone
    {  486, 5015,  154651,  5, }, // Africa/Harare
    {  486, 4479,  154656, 10, }, // Africa/Johannesburg
    {  486, 4499,   10673,  4, }, // Africa/Juba
    {  486, 5151,  154666,  7, }, // Africa/Kampala
    {  486, 4524,  154673,  6, }, // Africa/Khartoum
    {  486, 4973,  154679,  6, }, // Africa/Kigali
    {  486, 5306,  154685,  7, }, // Africa/Kinshasa
    {  486, 4744,  106900,  5, }, // Africa/Lagos
    {  486, 5336,  154692,  7, }, // Africa/Libreville
    {  486, 6869,  154699,  4, }, // Africa/Lome
    {  486, 5211,   10720,  6, }, // Africa/Luanda
    {  486, 4939,  154703,  9, }, // Africa/Lubumbashi
    {  486, 5001,  154712,  6, }, // Africa/Lusaka
    {  486, 5322,   10740,  6, }, // Africa/Malabo
    {  486, 4987,  154718,  6, }, // Africa/Maputo
    {  486, 5197,  154724,  5, }, // Africa/Maseru
    {  486, 5182,  154729,  7, }, // Africa/Mbabane
    {  486, 5113,  154736,  8, }, // Africa/Mogadishu
    {  486, 7034,  154744,  8, }, // Africa/Monrovia
    {  486, 3853,  154752,  7, }, // Africa/Nairobi
    {  486, 5271,  154759,  7, }, // Africa/Ndjamena
    {  486, 5354,  154766,  5, }, // Africa/Niamey
    {  486, 6803,  154771,  7, }, // Africa/Nouakchott
    {  486, 6727,  154778,  9, }, // Africa/Ouagadougou
    {  486, 5225,  154787, 10, }, // Africa/Porto-Novo
    {  486, 4435,  154797,  8, }, // Africa/Sao_Tome
    {  486, 2866,   10831,  6, }, // Africa/Tripoli
    {  486, 6554,   10837,  4, }, // Africa/Tunis
    {  486, 4152,  154805,  7, }, // Africa/Windhoek
    {  486,  132,  154812,  4, }, // America/Adak
    {  486, 3346,  154816,  7, }, // America/Anchorage
    {  486, 5607,  154823,  8, }, // America/Anguilla
    {  486, 5624,  154831,  7, }, // America/Antigua
    {  486, 4554,  154838,  9, }, // America/Araguaina
    {  486,  166,  154847, 11, }, // America/Argentina/Buenos_Aires
    {  486,   91,  154858,  9, }, // America/Argentina/Catamarca
    {  486,  270,  154867,  7, }, // America/Argentina/Cordoba
    {  486,  441,  154874,  5, }, // America/Argentina/Jujuy
    {  486, 7462,  154879,  7, }, // America/Argentina/La_Rioja
    {  486,  565,  154886,  6, }, // America/Argentina/Mendoza
    {  486, 7489,  154892,  9, }, // America/Argentina/Rio_Gallegos
    {  486, 7520,  107123,  5, }, // America/Argentina/Salta
    {  486, 7544,  154901,  7, }, // America/Argentina/San_Juan
    {  486, 5580,  154908,  8, }, // America/Argentina/San_Luis
    {  486, 7571,  154916,  6, }, // America/Argentina/Tucuman
    {  486, 7597,  152288,  7, }, // America/Argentina/Ushuaia
    {  486, 5640,  154922,  5, }, // America/Aruba
    {  486, 4252,  154927,  8, }, // America/Asuncion
    {  486,  237,  154935,  7, }, // America/Atikokan
    {  486, 3679,  154942,  4, }, // America/Bahia
    {  486, 7623,  154946, 13, }, // America/Bahia_Banderas
    {  486, 5654,  107198,  8, }, // America/Barbados
    {  486, 7646,   11020,  3, }, // America/Belem
    {  486, 5407,  152331,  4, }, // America/Belize
    {  486, 7660,  154959, 11, }, // America/Blanc-Sablon
    {  486, 7681,  154970,  9, }, // America/Boa_Vista
    {  486, 4361,  107225,  6, }, // America/Bogota
    {  486, 7699,  154979,  5, }, // America/Boise
    {  486, 7713,  154984,  9, }, // America/Cambridge_Bay
    {  486, 7735,  154993, 11, }, // America/Campo_Grande
    {  486, 3868,  155004,  6, }, // America/Cancun
    {  486, 4694,  152387,  7, }, // America/Caracas
    {  486, 4345,  152179,  4, }, // America/Cayenne
    {  486, 5496,  152398,  4, }, // America/Cayman
    {  486, 2260,  155010,  6, }, // America/Chicago
    {  486, 7756,  155016,  6, }, // America/Chihuahua
    {  486, 7774,  155022, 12, }, // America/Ciudad_Juarez
    {  486, 5422,  155034,  9, }, // America/Costa_Rica
    {  486, 7796,  155043,  6, }, // America/Creston
    {  486, 3791,  155049,  6, }, // America/Cuiaba
    {  486, 5723,  155055,  8, }, // America/Curacao
    {  486, 7812,  155063, 13, }, // America/Danmarkshavn
    {  486, 7833,   11176,  5, }, // America/Dawson
    {  486, 7848,  155076, 10, }, // America/Dawson_Creek
    {  486,  805,  155086,  4, }, // America/Denver
    {  486, 3465,  155090,  7, }, // America/Detroit
    {  486, 5739,  152485,  8, }, // America/Dominica
    {  486,  893,  155097,  8, }, // America/Edmonton
    {  486, 7869,  155105,  7, }, // America/Eirunepe
    {  486, 5441,  155112, 10, }, // America/El_Salvador
    {  486, 7886,  155122, 10, }, // America/Fort_Nelson
    {  486, 7906,  155132,  8, }, // America/Fortaleza
    {  486, 7924,  155140,  7, }, // America/Glace_Bay
    {  486, 6881,  155147,  6, }, // America/Goose_Bay
    {  486, 4612,  155153, 10, }, // America/Grand_Turk
    {  486, 5770,  155163,  6, }, // America/Grenada
    {  486, 5786,  155169,  7, }, // America/Guadeloupe
    {  486, 3760,  155176,  8, }, // America/Guatemala
    {  486, 4841,  155184,  8, }, // America/Guayaquil
    {  486, 6932,  155192,  5, }, // America/Guyana
    {  486, 1939,  155197,  6, }, // America/Halifax
    {  486, 2281,  155203,  6, }, // America/Havana
    {  486, 7942,  155209,  7, }, // America/Hermosillo
    {  486,  348,  155216, 13, }, // America/Indiana/Indianapolis
    {  486,  481,  155229, 14, }, // America/Indiana/Knox
    {  486, 7961,  155243, 16, }, // America/Indiana/Marengo
    {  486, 7985,  155259, 18, }, // America/Indiana/Petersburg
    {  486, 8012,  155277, 17, }, // America/Indiana/Tell_City
    {  486, 8038,  155294, 14, }, // America/Indiana/Vevay
    {  486, 8060,  155308, 15, }, // America/Indiana/Vincennes
    {  486, 8086,  155323, 17, }, // America/Indiana/Winamac
    {  486, 8110,  155340,  7, }, // America/Inuvik
    {  486,  660,  155347,  7, }, // America/Iqaluit
    {  486, 2799,  155354,  8, }, // America/Jamaica
    {  486, 5380,  152737,  4, }, // America/Juneau
    {  486,  521,  155362,  8, }, // America/Kentucky/Louisville
    {  486, 8125,  155370, 16, }, // America/Kentucky/Monticello
    {  486, 5704,  155386,  8, }, // America/Kralendijk
    {  486, 4376,  155394,  5, }, // America/La_Paz
    {  486, 7169,  152774,  4, }, // America/Lima
    {  486, 3239,  155399,  9, }, // America/Los_Angeles
    {  486, 5932,  155408, 23, }, // America/Lower_Princes
    {  486, 8153,  155431,  5, }, // America/Maceio
    {  486, 8168,  155436,  7, }, // America/Managua
    {  486, 1908,   11498,  6, }, // America/Manaus
    {  486, 5897,  155443,  7, }, // America/Marigot
    {  486, 5805,  155450,  8, }, // America/Martinique
    {  486, 8184,   11521,  9, }, // America/Matamoros
    {  486, 2917,  107772,  8, }, // America/Mazatlan
    {  486, 8202,  155458,  7, }, // America/Menominee
    {  486, 8220,  155465,  5, }, // America/Merida
    {  486, 8235,  155470, 10, }, // America/Metlakatla
    {  486, 2949,  155480, 10, }, // America/Mexico_City
    {  486, 4391,  155490,  6, }, // America/Miquelon
    {  486, 8254,  155496,  7, }, // America/Moncton
    {  486, 8270,  155503,  6, }, // America/Monterrey
    {  486, 4098,  155509, 11, }, // America/Montevideo
    {  486, 5824,  155520,  8, }, // America/Montserrat
    {  486, 5481,  155528,  6, }, // America/Nassau
    {  486, 2379,  155534,  7, }, // America/New_York
    {  486, 8288,   11540,  3, }, // America/Nome
    {  486, 1850,  155541,  7, }, // America/Noronha
    {  486, 8301,  155548, 20, }, // America/North_Dakota/Beulah
    {  486, 8329,  155568, 19, }, // America/North_Dakota/Center
    {  486, 8357,  155587, 22, }, // America/North_Dakota/New_Salem
    {  486,  393,  153000,  4, }, // America/Nuuk
    {  486, 8388,  155609,  8, }, // America/Ojinaga
    {  486, 2356,  107960,  6, }, // America/Panama
    {  486, 6173,  155617, 10, }, // America/Paramaribo
    {  486, 2973,  153019,  5, }, // America/Phoenix
    {  486, 3945,  155627,  9, }, // America/Port-au-Prince
    {  486, 5954,  155636, 12, }, // America/Port_of_Spain
    {  486, 8404,  155648,  9, }, // America/Porto_Velho
    {  486, 5843,  155657,  9, }, // America/Puerto_Rico
    {  486, 4030,  155666, 11, }, // America/Punta_Arenas
    {  486, 8424,  155677, 11, }, // America/Rankin_Inlet
    {  486, 8445,  155688,  6, }, // America/Recife
    {  486, 1995,  155694,  6, }, // America/Regina
    {  486, 8460,  155700,  6, }, // America/Resolute
    {  486,  695,  155706,  9, }, // America/Rio_Branco
    {  486, 8477,  155715,  6, }, // America/Santarem
    {  486, 2201,  155721,  8, }, // America/Santiago
    {  486, 6111,  155729, 13, }, // America/Santo_Domingo
    {  486, 1878,  155742, 10, }, // America/Sao_Paulo
    {  486, 6899,  155752, 12, }, // America/Scoresbysund
    {  486, 8494,  155764,  5, }, // America/Sitka
    {  486, 8508,  155769, 11, }, // America/St_Barthelemy
    {  486, 2061,  155780,  7, }, // America/St_Johns
    {  486, 5863,  155787,  8, }, // America/St_Kitts
    {  486, 5880,  155795,  9, }, // America/St_Lucia
    {  486,  855,  155804,  9, }, // America/St_Thomas
    {  486, 5913,  155813, 10, }, // America/St_Vincent
    {  486, 8530,  155823, 11, }, // America/Swift_Current
    {  486, 5461,  155834, 10, }, // America/Tegucigalpa
    {  486, 5756,   11947,  3, }, // America/Thule
    {  486,  313,  155844,  7, }, // America/Tijuana
    {  486,  608,  155851,  6, }, // America/Toronto
    {  486, 5688,   11944,  7, }, // America/Tortola
    {  486, 2093,  155857,  6, }, // America/Vancouver
    {  486, 2144,  155863,  9, }, // America/Whitehorse
    {  486,  734,  155872,  6, }, // America/Winnipeg
    {  486, 8552,  155878,  7, }, // America/Yakutat
    {  486, 6001,  104758,  4, }, // Antarctica/Casey
    {  486, 6094,  155885,  5, }, // Antarctica/Davis
    {  486, 6133,  155890, 12, }, // Antarctica/DumontDUrville
    {  486, 8568,  155902,  6, }, // Antarctica/Macquarie
    {  486, 7066,   12014,  6, }, // Antarctica/Mawson
    {  486, 7113,  155908,  8, }, // Antarctica/McMurdo
    {  486, 8589,  153339,  5, }, // Antarctica/Palmer
    {  486, 7226,  155916,  5, }, // Antarctica/Rothera
    {  486, 7295,  155921,  4, }, // Antarctica/Syowa
    {  486, 8607,   12047,  4, }, // Antarctica/Troll
    {  486, 7374,  104813,  5, }, // Antarctica/Vostok
    {  486, 1429,  155925, 10, }, // Arctic/Longyearbyen
    {  486, 5570,   12066,  3, }, // Asia/Aden
    {  486, 5395,  155935,  6, }, // Asia/Almaty
    {  486, 3968,  155941,  5, }, // Asia/Amman
    {  486, 5511,  155946,  6, }, // Asia/Anadyr
    {  486, 5523,  155952,  5, }, // Asia/Aqtau
    {  486, 5368,  155957,  6, }, // Asia/Aqtobe
    {  486,  964,  155963,  8, }, // Asia/Ashgabat
    {  486, 8624,  155971,  6, }, // Asia/Atyrau
    {  486, 3607,   12109,  5, }, // Asia/Baghdad
    {  486, 5534,  153411,  5, }, // Asia/Bahrain
    {  486, 3653,  152447,  4, }, // Asia/Baku
    {  486, 4466,  155977,  6, }, // Asia/Bangkok
    {  486, 3571,  155983,  8, }, // Asia/Barnaul
    {  486, 4086,  153421,  5, }, // Asia/Beirut
    {  486, 3778,  155991,  5, }, // Asia/Bishkek
    {  486, 5989,  155996,  6, }, // Asia/Brunei
    {  486, 4601,  156002,  4, }, // Asia/Chita
    {  486, 4511,  156006,  5, }, // Asia/Colombo
    {  486, 4540,   12162,  4, }, // Asia/Damascus
    {  486, 1093,  152177,  4, }, // Asia/Dhaka
    {  486, 6192,  156011,  4, }, // Asia/Dili
    {  486, 3596,  156015,  3, }, // Asia/Dubai
    {  486, 6159,  156018,  6, }, // Asia/Dushanbe
    {  486, 8636,  156024,  9, }, // Asia/Famagusta
    {  486, 8651,  153469,  3, }, // Asia/Gaza
    {  486, 4795,  156033,  6, }, // Asia/Hebron
    {  486, 1259,  156039, 13, }, // Asia/Ho_Chi_Minh
    {  486, 2704,  156052,  7, }, // Asia/Hong_Kong
    {  486, 4771,  156059,  4, }, // Asia/Hovd
    {  486, 4184,  156063,  8, }, // Asia/Irkutsk
    {  486, 7006,  156071,  7, }, // Asia/Jakarta
    {  486, 6992,  156078,  8, }, // Asia/Jayapura
    {  486, 1290,  156086,  7, }, // Asia/Jerusalem
    {  486, 3560,  153523,  4, }, // Asia/Kabul
    {  486, 4316,  153527,  8, }, // Asia/Kamchatka
    {  486, 4239,  156093,  5, }, // Asia/Karachi
    {  486, 1185,  156098,  8, }, // Asia/Kathmandu
    {  486, 8661,  156106,  7, }, // Asia/Khandyga
    {  486,  992,  156113,  5, }, // Asia/Kolkata
    {  486, 4197,  156118, 11, }, // Asia/Krasnoyarsk
    {  486, 4859,  156129, 11, }, // Asia/Kuala_Lumpur
    {  486, 5976,  156140,  6, }, // Asia/Kuching
    {  486, 5547,  156146,  4, }, // Asia/Kuwait
    {  486, 1211,  156150,  6, }, // Asia/Macau
    {  486, 4017,  108597,  7, }, // Asia/Magadan
    {  486, 1349,  156156,  7, }, // Asia/Makassar
    {  486, 7182,  156163,  5, }, // Asia/Manila
    {  486, 6920,   12368,  4, }, // Asia/Muscat
    {  486, 2554,  156168,  7, }, // Asia/Nicosia
    {  486, 8675,  156175, 11, }, // Asia/Novokuznetsk
    {  486, 4135,  156186, 11, }, // Asia/Novosibirsk
    {  486, 4229,  105160,  5, }, // Asia/Omsk
    {  486, 7145,  156197,  5, }, // Asia/Oral
    {  486, 6961,  156202,  7, }, // Asia/Phnom_Penh
    {  486, 8693,  156209,  9, }, // Asia/Pontianak
    {  486, 4214,  156218, 10, }, // Asia/Pyongyang
    {  486, 5559,   12436,  3, }, // Asia/Qatar
    {  486, 8708,  156228,  8, }, // Asia/Qostanay
    {  486, 4269,  156236, 11, }, // Asia/Qyzylorda
    {  486, 3584,  105212,  4, }, // Asia/Riyadh
    {  486, 4408,  156247,  7, }, // Asia/Sakhalin
    {  486, 7245,   12468,  6, }, // Asia/Samarkand
    {  486, 3279,  156254,  4, }, // Asia/Seoul
    {  486, 1053,  156258,  7, }, // Asia/Shanghai
    {  486, 3300,  156265,  7, }, // Asia/Singapore
    {  486, 4297,  156272, 12, }, // Asia/Srednekolymsk
    {  486, 3263,  156284,  5, }, // Asia/Taipei
    {  486, 4781,  156289,  6, }, // Asia/Tashkent
    {  486, 3915,  156295,  5, }, // Asia/Tbilisi
    {  486, 2772,  153730,  5, }, // Asia/Tehran
    {  486, 1317,  156300,  5, }, // Asia/Thimphu
    {  486, 2821,  156305,  5, }, // Asia/Tokyo
    {  486, 4572,  156310,  5, }, // Asia/Tomsk
    {  486, 1021,  156315, 11, }, // Asia/Ulaanbaatar
    {  486, 1159,  156326,  6, }, // Asia/Urumqi
    {  486, 8722,  156332,  8, }, // Asia/Ust-Nera
    {  486, 6977,  156340,  7, }, // Asia/Vientiane
    {  486, 4710,  156347, 11, }, // Asia/Vladivostok
    {  486, 4828,  156358,  7, }, // Asia/Yakutsk
    {  486, 1235,  156365,  6, }, // Asia/Yangon
    {  486, 3883,  156371, 10, }, // Asia/Yekaterinburg
    {  486, 3747,  156381,  6, }, // Asia/Yerevan
    {  486, 3663,  156387,  4, }, // Atlantic/Azores
    {  486, 5671,   12615,  6, }, // Atlantic/Bermuda
    {  486, 6628,  156391,  5, }, // Atlantic/Canary
    {  486, 3727,  156396,  7, }, // Atlantic/Cape_Verde
    {  486, 1395,   12638,  4, }, // Atlantic/Faroe
    {  486, 8736,  156403,  6, }, // Atlantic/Madeira
    {  486, 2748,  156409,  8, }, // Atlantic/Reykjavik
    {  486, 7272,  156417, 13, }, // Atlantic/South_Georgia
    {  486, 6821,  156430,  8, }, // Atlantic/St_Helena
    {  486, 6644,  156438,  7, }, // Atlantic/Stanley
    {  486, 1670,  156445,  6, }, // Australia/Adelaide
    {  486, 1635,  156451,  7, }, // Australia/Brisbane
    {  486, 1799,  156458,  9, }, // Australia/Broken_Hill
    {  486, 1583,  156467,  6, }, // Australia/Darwin
    {  486, 3637,  156473,  5, }, // Australia/Eucla
    {  486, 1516,   12721,  6, }, // Australia/Hobart
    {  486, 8753,  156478,  6, }, // Australia/Lindeman
    {  486, 1547,  156484,  6, }, // Australia/Lord_Howe
    {  486, 1727,  156490,  6, }, // Australia/Melbourne
    {  486, 1762,  153936,  3, }, // Australia/Perth
    {  486, 1463,  156496,  5, }, // Australia/Sydney
    {  486, 6441,  156501,  8, }, // Europe/Amsterdam
    {  486, 6216,  156509,  6, }, // Europe/Andorra
    {  486, 3620,  156515,  8, }, // Europe/Astrakhan
    {  486, 2300,  156523,  3, }, // Europe/Athens
    {  486, 6336,  156526,  6, }, // Europe/Belgrade
    {  486, 4757,  156532,  5, }, // Europe/Berlin
    {  486, 6488,  156537, 10, }, // Europe/Bratislava
    {  486, 2167,  156547,  6, }, // Europe/Brussels
    {  486, 3928,  153995,  6, }, // Europe/Bucharest
    {  486, 3806,  156553,  7, }, // Europe/Budapest
    {  486, 8772,  156560,  6, }, // Europe/Busingen
    {  486, 2583,  156566,  7, }, // Europe/Chisinau
    {  486, 6289,  156573,  6, }, // Europe/Copenhagen
    {  486, 2338,  156579,  6, }, // Europe/Dublin
    {  486, 6307,  156585, 10, }, // Europe/Gibraltar
    {  486, 8788,  156595,  5, }, // Europe/Guernsey
    {  486, 6612,  156600,  7, }, // Europe/Helsinki
    {  486, 8804,  156607,  9, }, // Europe/Isle_of_Man
    {  486, 1130,  156616,  8, }, // Europe/Istanbul
    {  486, 8823,  156624,  4, }, // Europe/Jersey
    {  486, 3979,  156628, 11, }, // Europe/Kaliningrad
    {  486, 8929,  154084,  5, }, // Europe/Kirov
    {  486, 2527,  154089,  3, }, // Europe/Kyiv
    {  486, 3213,  154092,  6, }, // Europe/Lisbon
    {  486, 6506,  156639,  9, }, // Europe/Ljubljana
    {  486, 2501,   12926,  4, }, // Europe/London
    {  486, 6365,  156648, 10, }, // Europe/Luxembourg
    {  486, 4892,  156658,  6, }, // Europe/Madrid
    {  486, 6397,  109265,  4, }, // Europe/Malta
    {  486, 6582,  156664, 10, }, // Europe/Mariehamn
    {  486, 3693,  156674,  5, }, // Europe/Minsk
    {  486, 6410,  105768,  6, }, // Europe/Monaco
    {  486, 3537,  154273,  4, }, // Europe/Moscow
    {  486, 6458,  107707,  4, }, // Europe/Oslo
    {  486, 4284,  154137,  5, }, // Europe/Paris
    {  486, 6424,  156679, 10, }, // Europe/Podgorica
    {  486, 6275,  105798,  4, }, // Europe/Prague
    {  486, 8837,  155445,  4, }, // Europe/Riga
    {  486, 6324,   11327,  2, }, // Europe/Rome
    {  486, 4331,  109308,  6, }, // Europe/Samara
    {  486, 6470,  156689, 10, }, // Europe/San_Marino
    {  486, 6245,  156699,  7, }, // Europe/Sarajevo
    {  486, 4451,   13022,  7, }, // Europe/Saratov
    {  486, 8849,  156706,  8, }, // Europe/Simferopol
    {  486, 6383,  156714,  7, }, // Europe/Skopje
    {  486, 6599,  156721,  5, }, // Europe/Sofia
    {  486, 6523,  156726,  7, }, // Europe/Stockholm
    {  486, 8867,  156733,  5, }, // Europe/Tallinn
    {  486, 6202,  156738,  6, }, // Europe/Tirane
    {  486, 8882,  156744, 10, }, // Europe/Ulyanovsk
    {  486, 6352,  156754,  6, }, // Europe/Vaduz
    {  486, 6567,  156760,  7, }, // Europe/Vatican
    {  486, 6231,  103492,  3, }, // Europe/Vienna
    {  486, 8899,  156767,  7, }, // Europe/Vilnius
    {  486, 4727,  156774,  8, }, // Europe/Volgograd
    {  486, 3190,  156782,  4, }, // Europe/Warsaw
    {  486, 6261,  156786,  5, }, // Europe/Zagreb
    {  486, 6540,  156791,  5, }, // Europe/Zurich
    {  486, 5078,  156796, 12, }, // Indian/Antananarivo
    {  486, 6947,  156808,  5, }, // Indian/Chagos
    {  486, 6046,  156813,  6, }, // Indian/Christmas
    {  486, 6063,  105964,  5, }, // Indian/Cocos
    {  486, 5029,  105969,  6, }, // Indian/Comoro
    {  486, 6661,  156819,  6, }, // Indian/Kerguelen
    {  486, 7260,  156825,  4, }, // Indian/Mahe
    {  486, 7050,  156829,  6, }, // Indian/Maldives
    {  486, 4069,  156835,  5, }, // Indian/Mauritius
    {  486, 5098,  156840,  5, }, // Indian/Mayotte
    {  486, 7211,  156845,  7, }, // Indian/Reunion
    {  486, 4422,  156852,  4, }, // Pacific/Apia
    {  486,  932,  156856,  6, }, // Pacific/Auckland
    {  486, 3706,  156862,  7, }, // Pacific/Bougainville
    {  486, 3015,  156869,  6, }, // Pacific/Chatham
    {  486, 3157,  154335,  3, }, // Pacific/Chuuk
    {  486, 2237,  156875,  5, }, // Pacific/Easter
    {  486, 7360,  156880,  5, }, // Pacific/Efate
    {  486, 7327,  156885,  8, }, // Pacific/Fakaofo
    {  486, 3902,  156893,  4, }, // Pacific/Fiji
    {  486, 7343,  156897,  8, }, // Pacific/Funafuti
    {  486, 6678,  109587,  9, }, // Pacific/Galapagos
    {  486, 6696,  156905,  6, }, // Pacific/Gambier
    {  486, 3822,  156911, 12, }, // Pacific/Guadalcanal
    {  486, 6018,  104212,  4, }, // Pacific/Guam
    {  486, 2723,   13283,  8, }, // Pacific/Honolulu
    {  486, 3049,  156923,  7, }, // Pacific/Kanton
    {  486, 3998,  156930,  9, }, // Pacific/Kiritimati
    {  486, 7019,  156939,  6, }, // Pacific/Kosrae
    {  486, 2842,  156945,  8, }, // Pacific/Kwajalein
    {  486, 4877,   13323,  6, }, // Pacific/Majuro
    {  486, 4051,  156953,  7, }, // Pacific/Marquesas
    {  486, 8914,  156960,  5, }, // Pacific/Midway
    {  486, 7084,  156965,  6, }, // Pacific/Nauru
    {  486, 7132,  156971,  5, }, // Pacific/Niue
    {  486, 4168,  156976,  7, }, // Pacific/Norfolk
    {  486, 7098,  156983,  5, }, // Pacific/Noumea
    {  486, 3126,  156988,  8, }, // Pacific/Pago_Pago
    {  486, 7155,  156996,  6, }, // Pacific/Palau
    {  486, 7194,  157002,  7, }, // Pacific/Pitcairn
    {  486, 3096,  157009,  5, }, // Pacific/Pohnpei
    {  486, 4807,  157014, 11, }, // Pacific/Port_Moresby
    {  486, 6076,  109724,  9, }, // Pacific/Rarotonga
    {  486, 6031,  157025,  6, }, // Pacific/Saipan
    {  486, 7312,  157031,  6, }, // Pacific/Tahiti
    {  486, 6712,   13417,  6, }, // Pacific/Tarawa
    {  486, 4583,  157037,  9, }, // Pacific/Tongatapu
    {  486, 7392,  106252,  3, }, // Pacific/Wake
    {  486, 7405,  157046,  5, }, // Pacific/Wallis
    {  488, 6788,  157051,  7, }, // Africa/Abidjan Polish/Latin/Poland
    {  488, 6760,    4649,  4, }, // Africa/Accra
    {  488, 5059,    3902, 11, }, // Africa/Addis_Ababa
    {  488, 7420,   58555,  6, }, // Africa/Algiers
    {  488,   14,       0,  6, }, // Africa/Asmara
    {  488, 5257,   26480,  5, }, // Africa/Bangui
    {  488, 6746,  157058,  7, }, // Africa/Banjul
    {  488, 4922,  157065,  9, }, // Africa/Bujumbura
    {  488, 2320,    4690,  4, }, // Africa/Cairo
    {  488, 6773,   26485,  7, }, // Africa/Conakry
    {  488, 5043,  157074,  7, }, // Africa/Djibouti
    {  488, 5243,   21372,  5, }, // Africa/Douala
    {  488, 5166,  157081,  7, }, // Africa/El_Aaiun
    {  488, 4499,  157088,  5, }, // Africa/Juba
    {  488, 4524,  124879,  7, }, // Africa/Khartoum
    {  488, 5306,  157093,  8, }, // Africa/Kinshasa
    {  488, 6869,   20418,  4, }, // Africa/Lome
    {  488, 5113,  157101,  9, }, // Africa/Mogadishu
    {  488, 5271,  157110,  8, }, // Africa/Ndjamena
    {  488, 6803,  157118,  9, }, // Africa/Nouakchott
    {  488, 6727,    4758,  8, }, // Africa/Ouagadougou
    {  488, 5225,   26542, 10, }, // Africa/Porto-Novo
    {  488, 4435,       6,  8, }, // Africa/Sao_Tome
    {  488, 2866,  157127,  8, }, // Africa/Tripoli
    {  488, 4152,   56808,  7, }, // Africa/Windhoek
    {  488,  270,   20479,  7, }, // America/Argentina/Cordoba
    {  488, 4252,      14,  8, }, // America/Asuncion
    {  488,  237,      22,  8, }, // America/Atikokan
    {  488, 3679,    6806,  8, }, // America/Bahia
    {  488, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  488, 7646,   35896,  5, }, // America/Belem
    {  488, 3868,      47,  6, }, // America/Cancun
    {  488, 4345,  122476,  7, }, // America/Cayenne
    {  488, 5496,  157135,  7, }, // America/Cayman
    {  488, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  488, 5422,  157142,  9, }, // America/Costa_Rica
    {  488, 3791,   26574,  6, }, // America/Cuiaba
    {  488, 5723,      66,  7, }, // America/Curacao
    {  488, 5739,   21666,  8, }, // America/Dominica
    {  488, 5441,  157151,  8, }, // America/El_Salvador
    {  488, 5786,  157159,  9, }, // America/Guadeloupe
    {  488, 3760,  134472,  9, }, // America/Guatemala
    {  488, 6932,  157168,  6, }, // America/Guyana
    {  488, 2281,  157174,  6, }, // America/Havana
    {  488,  481,      73, 13, }, // America/Indiana/Knox
    {  488, 7961,      86, 16, }, // America/Indiana/Marengo
    {  488, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  488, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  488, 8038,     139, 14, }, // America/Indiana/Vevay
    {  488, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  488, 8086,     171, 16, }, // America/Indiana/Winamac
    {  488, 2799,   33347,  7, }, // America/Jamaica
    {  488, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  488, 5932,     207, 22, }, // America/Lower_Princes
    {  488, 8153,   26588,  6, }, // America/Maceio
    {  488, 5805,  157180,  9, }, // America/Martinique
    {  488, 8220,    7126,  6, }, // America/Merida
    {  488, 2949,  157189, 15, }, // America/Mexico_City
    {  488, 2379,  157204,  9, }, // America/New_York
    {  488, 1850,     235, 19, }, // America/Noronha
    {  488, 8301,  157213, 23, }, // America/North_Dakota/Beulah
    {  488, 8329,  157236, 23, }, // America/North_Dakota/Center
    {  488, 8357,  157259, 26, }, // America/North_Dakota/New_Salem
    {  488,  393,     317,  4, }, // America/Nuuk
    {  488, 5954,   26691, 13, }, // America/Port_of_Spain
    {  488, 5843,  157285,  9, }, // America/Puerto_Rico
    {  488, 6899,     321, 16, }, // America/Scoresbysund
    {  488, 8508,   26721, 16, }, // America/St_Barthelemy
    {  488, 2061,     351, 10, }, // America/St_Johns
    {  488, 5863,   26749, 11, }, // America/St_Kitts
    {  488, 5880,   35964, 11, }, // America/St_Lucia
    {  488,  855,   44293, 12, }, // America/St_Thomas
    {  488, 5913,   26783, 13, }, // America/St_Vincent
    {  488, 5756,   26796,  7, }, // America/Thule
    {  488, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  488, 7374,    4147,  6, }, // Antarctica/Vostok
    {  488, 5395,  157294,  6, }, // Asia/Almaty
    {  488, 5523,    5609,  5, }, // Asia/Aqtau
    {  488, 5368,  157300, 10, }, // Asia/Aqtobe
    {  488,  964,  157310,  9, }, // Asia/Ashgabat
    {  488, 3607,    4162,  6, }, // Asia/Baghdad
    {  488, 5534,   57943,  7, }, // Asia/Bahrain
    {  488, 3571,  157319,  7, }, // Asia/Barnaul
    {  488, 4086,    7625,  6, }, // Asia/Beirut
    {  488, 3778,  157326,  7, }, // Asia/Bishkek
    {  488, 4601,  157333,  5, }, // Asia/Chita
    {  488, 4511,    5637,  7, }, // Asia/Colombo
    {  488, 4540,  157338,  8, }, // Asia/Damascus
    {  488, 3596,   57968,  5, }, // Asia/Dubai
    {  488, 6159,  157346,  8, }, // Asia/Dushanbe
    {  488, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  488, 2704,    4231,  8, }, // Asia/Hong_Kong
    {  488, 4771,  157354,  5, }, // Asia/Hovd
    {  488, 4184,  147155,  6, }, // Asia/Irkutsk
    {  488, 7006,  157359,  8, }, // Asia/Jakarta
    {  488, 1290,  157367, 10, }, // Asia/Jerusalem
    {  488, 4316,  157377,  9, }, // Asia/Kamchatka
    {  488, 4239,  157386,  7, }, // Asia/Karachi
    {  488, 1185,    5711,  8, }, // Asia/Kathmandu
    {  488, 8661,   58020,  8, }, // Asia/Khandyga
    {  488,  992,   26864,  7, }, // Asia/Kolkata
    {  488, 4197,    4288, 11, }, // Asia/Krasnoyarsk
    {  488, 5547,  157393,  6, }, // Asia/Kuwait
    {  488, 1211,   33537,  5, }, // Asia/Macau
    {  488, 6920,   26882,  6, }, // Asia/Muscat
    {  488, 2554,  157399,  7, }, // Asia/Nicosia
    {  488, 8675,  157406, 12, }, // Asia/Novokuznetsk
    {  488, 4135,  157418, 11, }, // Asia/Novosibirsk
    {  488, 7145,   58053,  6, }, // Asia/Oral
    {  488, 4214,   33562,  9, }, // Asia/Pyongyang
    {  488, 5559,    4332,  5, }, // Asia/Qatar
    {  488, 8708,  157429,  8, }, // Asia/Qostanay
    {  488, 4269,  157437,  9, }, // Asia/Qyzylorda
    {  488, 3584,   33571,  5, }, // Asia/Riyadh
    {  488, 4408,   58084,  8, }, // Asia/Sakhalin
    {  488, 7245,   26906, 10, }, // Asia/Samarkand
    {  488, 3279,    7798,  4, }, // Asia/Seoul
    {  488, 1053,  157446,  8, }, // Asia/Shanghai
    {  488, 3300,   20900,  8, }, // Asia/Singapore
    {  488, 4297,  157454, 15, }, // Asia/Srednekolymsk
    {  488, 3263,    7830,  6, }, // Asia/Taipei
    {  488, 4781,  157469,  9, }, // Asia/Tashkent
    {  488, 2772,    7842,  7, }, // Asia/Tehran
    {  488, 2821,    4356,  5, }, // Asia/Tokyo
    {  488, 1021,  157478, 10, }, // Asia/Ulaanbaatar
    {  488, 1159,  157488,  7, }, // Asia/Urumqi
    {  488, 8722,  157495,  9, }, // Asia/Ust-Nera
    {  488, 6977,  157504,  8, }, // Asia/Vientiane
    {  488, 4710,  157512, 11, }, // Asia/Vladivostok
    {  488, 4828,  147182,  6, }, // Asia/Yakutsk
    {  488, 1235,    7880,  6, }, // Asia/Yangon
    {  488, 3883,  157523, 13, }, // Asia/Yekaterinburg
    {  488, 3747,  157536,  6, }, // Asia/Yerevan
    {  488, 3663,  157542,  5, }, // Atlantic/Azores
    {  488, 5671,   58175,  7, }, // Atlantic/Bermuda
    {  488, 6628,  157547, 17, }, // Atlantic/Canary
    {  488, 3727,  157564, 29, }, // Atlantic/Cape_Verde
    {  488, 1395,  157593, 11, }, // Atlantic/Faroe
    {  488, 8736,  157604,  6, }, // Atlantic/Madeira
    {  488, 7272,  157610, 18, }, // Atlantic/South_Georgia
    {  488, 6821,  157628, 13, }, // Atlantic/St_Helena
    {  488, 6216,   33638,  6, }, // Europe/Andorra
    {  488, 3620,  157641,  9, }, // Europe/Astrakhan
    {  488, 2300,  157650,  5, }, // Europe/Athens
    {  488, 6336,   27015,  7, }, // Europe/Belgrade
    {  488, 6488,  157655, 10, }, // Europe/Bratislava
    {  488, 2167,  157665,  8, }, // Europe/Brussels
    {  488, 3928,  157673,  9, }, // Europe/Bucharest
    {  488, 3806,  157682,  9, }, // Europe/Budapest
    {  488, 8772,   62908, 21, }, // Europe/Busingen
    {  488, 2583,  157691,  9, }, // Europe/Chisinau
    {  488, 6289,  127206,  9, }, // Europe/Copenhagen
    {  488, 8804,  157700,  9, }, // Europe/Isle_of_Man
    {  488, 1130,  157709,  7, }, // Europe/Istanbul
    {  488, 8929,   78287,  5, }, // Europe/Kirov
    {  488, 2527,  157716,  5, }, // Europe/Kyiv
    {  488, 3213,  157721,  7, }, // Europe/Lisbon
    {  488, 6506,  157728,  7, }, // Europe/Ljubljana
    {  488, 2501,  157735,  6, }, // Europe/London
    {  488, 6365,    8172, 10, }, // Europe/Luxembourg
    {  488, 4892,  157741,  6, }, // Europe/Madrid
    {  488, 6582,   63826, 13, }, // Europe/Mariehamn
    {  488, 3693,  157747,  5, }, // Europe/Minsk
    {  488, 6410,    6044,  6, }, // Europe/Monaco
    {  488, 3537,   93233,  6, }, // Europe/Moscow
    {  488, 4284,  157752,  5, }, // Europe/Paris
    {  488, 6275,   21185,  5, }, // Europe/Prague
    {  488, 8837,  127376,  4, }, // Europe/Riga
    {  488, 6324,  157757,  4, }, // Europe/Rome
    {  488, 6245,  157761,  8, }, // Europe/Sarajevo
    {  488, 4451,  157769,  7, }, // Europe/Saratov
    {  488, 8849,  157776, 10, }, // Europe/Simferopol
    {  488, 6523,  157786,  9, }, // Europe/Stockholm
    {  488, 8867,   22925,  6, }, // Europe/Tallinn
    {  488, 6202,   21213,  6, }, // Europe/Tirane
    {  488, 8882,   78303,  9, }, // Europe/Ulyanovsk
    {  488, 6567,  157795,  7, }, // Europe/Vatican
    {  488, 6231,  157802,  6, }, // Europe/Vienna
    {  488, 8899,  157808,  5, }, // Europe/Vilnius
    {  488, 4727,  157813,  9, }, // Europe/Volgograd
    {  488, 3190,   58675,  8, }, // Europe/Warsaw
    {  488, 6261,  157822,  7, }, // Europe/Zagreb
    {  488, 6540,  157829,  6, }, // Europe/Zurich
    {  488, 5078,  157835, 12, }, // Indian/Antananarivo
    {  488, 6947,  157847,  6, }, // Indian/Chagos
    {  488, 6046,  157853, 23, }, // Indian/Christmas
    {  488, 6063,  157876, 14, }, // Indian/Cocos
    {  488, 5029,   58408,  6, }, // Indian/Comoro
    {  488, 6661,  157890, 16, }, // Indian/Kerguelen
    {  488, 7260,   27115,  4, }, // Indian/Mahe
    {  488, 7050,  128028,  8, }, // Indian/Maldives
    {  488, 5098,  157906,  7, }, // Indian/Mayotte
    {  488, 7211,     547,  7, }, // Indian/Reunion
    {  488, 3706,  157913, 20, }, // Pacific/Bougainville
    {  488, 3157,     554,  5, }, // Pacific/Chuuk
    {  488, 2237,  157933, 17, }, // Pacific/Easter
    {  488, 3902,  157950,  5, }, // Pacific/Fiji
    {  488, 6696,  157955, 14, }, // Pacific/Gambier
    {  488, 2723,    4611,  8, }, // Pacific/Honolulu
    {  488, 3049,    4619,  9, }, // Pacific/Kanton
    {  488, 4051,  157969,  7, }, // Pacific/Marquesas
    {  488, 7098,  124660,  5, }, // Pacific/Noumea
    {  488, 3096,     586,  7, }, // Pacific/Pohnpei
    {  489, 6760,   74129,  4, }, // Africa/Accra Portuguese/Latin/Brazil
    {  489, 5059,   35821, 10, }, // Africa/Addis_Ababa
    {  489, 7420,  147692,  5, }, // Africa/Algiers
    {  489,   14,       0,  6, }, // Africa/Asmara
    {  489, 6773,  157976,  7, }, // Africa/Conakry
    {  489, 5043,   26492,  7, }, // Africa/Djibouti
    {  489, 5166,   63491,  8, }, // Africa/El_Aaiun
    {  489, 4479,  157983, 11, }, // Africa/Johannesburg
    {  489, 4524,  157994,  6, }, // Africa/Khartoum
    {  489, 6869,   20418,  4, }, // Africa/Lome
    {  489, 5113,  158000, 10, }, // Africa/Mogadishu
    {  489, 7034,  140658,  8, }, // Africa/Monrovia
    {  489, 3853,  158010,  7, }, // Africa/Nairobi
    {  489, 5271,   44128,  9, }, // Africa/Ndjamena
    {  489, 5225,   26542, 10, }, // Africa/Porto-Novo
    {  489, 4435,       6,  8, }, // Africa/Sao_Tome
    {  489, 2866,   20467,  7, }, // Africa/Tripoli
    {  489, 6554,   92461,  5, }, // Africa/Tunis
    {  489, 5607,    4775,  7, }, // America/Anguilla
    {  489, 5624,  158017,  7, }, // America/Antigua
    {  489, 4554,   26559,  9, }, // America/Araguaina
    {  489,  270,   20479,  7, }, // America/Argentina/Cordoba
    {  489, 7571,  158024,  6, }, // America/Argentina/Tucuman
    {  489, 4252,  158030,  8, }, // America/Asuncion
    {  489,  237,      22,  8, }, // America/Atikokan
    {  489, 7623,   64671, 17, }, // America/Bahia_Banderas
    {  489, 7646,   35896,  5, }, // America/Belem
    {  489, 4361,   20507,  6, }, // America/Bogota
    {  489, 3868,      47,  6, }, // America/Cancun
    {  489, 4345,   44167,  6, }, // America/Cayenne
    {  489, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  489, 3791,   26574,  6, }, // America/Cuiaba
    {  489, 5723,      66,  7, }, // America/Curacao
    {  489, 7869,   26580,  8, }, // America/Eirunepe
    {  489, 5770,   20519,  7, }, // America/Grenada
    {  489, 5786,   20526,  9, }, // America/Guadeloupe
    {  489, 4841,  158038,  9, }, // America/Guayaquil
    {  489, 6932,  158047,  6, }, // America/Guyana
    {  489,  348,   74243, 12, }, // America/Indiana/Indianapolis
    {  489,  481,      73, 13, }, // America/Indiana/Knox
    {  489, 7961,      86, 16, }, // America/Indiana/Marengo
    {  489, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  489, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  489, 8038,     139, 14, }, // America/Indiana/Vevay
    {  489, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  489, 8086,     171, 16, }, // America/Indiana/Winamac
    {  489, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  489, 5932,     207, 22, }, // America/Lower_Princes
    {  489, 8153,   26588,  6, }, // America/Maceio
    {  489, 8168,   98036,  7, }, // America/Managua
    {  489, 5805,   20554,  9, }, // America/Martinique
    {  489, 8220,     229,  6, }, // America/Merida
    {  489, 2949,  158053, 16, }, // America/Mexico_City
    {  489, 4098,  158069, 10, }, // America/Montevideo
    {  489, 2379,   44194,  9, }, // America/New_York
    {  489, 1850,     235, 19, }, // America/Noronha
    {  489, 8301,  158079, 23, }, // America/North_Dakota/Beulah
    {  489, 8329,  158102, 23, }, // America/North_Dakota/Center
    {  489, 8357,  158125, 26, }, // America/North_Dakota/New_Salem
    {  489,  393,     317,  4, }, // America/Nuuk
    {  489, 2356,   20665,  6, }, // America/Panama
    {  489, 3945,   74358, 14, }, // America/Port-au-Prince
    {  489, 5843,   64979, 10, }, // America/Puerto_Rico
    {  489, 8477,   26704,  8, }, // America/Santarem
    {  489, 1878,   26712,  9, }, // America/Sao_Paulo
    {  489, 6899,     321, 16, }, // America/Scoresbysund
    {  489, 8508,  158151, 14, }, // America/St_Barthelemy
    {  489, 2061,   26737, 12, }, // America/St_Johns
    {  489, 5863,  158165, 13, }, // America/St_Kitts
    {  489, 5880,  158178, 11, }, // America/St_Lucia
    {  489,  855,   44293, 12, }, // America/St_Thomas
    {  489, 5913,  158189, 11, }, // America/St_Vincent
    {  489, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  489, 5570,   91532,  4, }, // Asia/Aden
    {  489, 3968,  158200,  3, }, // Asia/Amman
    {  489, 5523,    5609,  5, }, // Asia/Aqtau
    {  489, 5368,    7606,  6, }, // Asia/Aqtobe
    {  489,  964,  158203,  8, }, // Asia/Ashgabat
    {  489, 3607,   57937,  5, }, // Asia/Baghdad
    {  489, 5534,    4168,  7, }, // Asia/Bahrain
    {  489, 4086,  158211,  7, }, // Asia/Beirut
    {  489, 4540,   74438,  7, }, // Asia/Damascus
    {  489, 1093,   65717,  5, }, // Asia/Dhaka
    {  489, 6159,  158218,  8, }, // Asia/Dushanbe
    {  489, 1259,  158226, 21, }, // Asia/Ho_Chi_Minh
    {  489, 7006,  158247,  7, }, // Asia/Jakarta
    {  489, 1290,  158254,  9, }, // Asia/Jerusalem
    {  489, 3560,   74452,  5, }, // Asia/Kabul
    {  489, 1185,    5711,  8, }, // Asia/Kathmandu
    {  489,  992,  158263,  7, }, // Asia/Kolkata
    {  489, 1211,    4321,  5, }, // Asia/Macau
    {  489, 6920,   20857,  7, }, // Asia/Muscat
    {  489, 2554,  158270,  7, }, // Asia/Nicosia
    {  489, 5559,   73150,  5, }, // Asia/Qatar
    {  489, 3584,  158277,  5, }, // Asia/Riyadh
    {  489, 4408,  158282,  8, }, // Asia/Sakhalin
    {  489, 7245,   20886, 10, }, // Asia/Samarkand
    {  489, 3279,    7798,  4, }, // Asia/Seoul
    {  489, 1053,  158290,  6, }, // Asia/Shanghai
    {  489, 3300,   93173,  9, }, // Asia/Singapore
    {  489, 2772,  158296,  5, }, // Asia/Tehran
    {  489, 2821,  158301,  6, }, // Asia/Tokyo
    {  489, 1021,   26923, 10, }, // Asia/Ulaanbaatar
    {  489, 1235,  158307,  6, }, // Asia/Yangon
    {  489, 3883,  158313, 13, }, // Asia/Yekaterinburg
    {  489, 3663,   44461,  6, }, // Atlantic/Azores
    {  489, 5671,   74483,  8, }, // Atlantic/Bermuda
    {  489, 6628,  158326,  8, }, // Atlantic/Canary
    {  489, 3727,   26968, 10, }, // Atlantic/Cape_Verde
    {  489, 1395,  158334, 11, }, // Atlantic/Faroe
    {  489, 2748,   36176,  9, }, // Atlantic/Reykjavik
    {  489, 7272,  158345, 14, }, // Atlantic/South_Georgia
    {  489, 6821,   26997, 12, }, // Atlantic/St_Helena
    {  489, 6441,  158359,  8, }, // Europe/Amsterdam
    {  489, 3620,  158367,  7, }, // Europe/Astrakhan
    {  489, 2300,   27009,  6, }, // Europe/Athens
    {  489, 6336,   58906,  8, }, // Europe/Belgrade
    {  489, 4757,  158374,  6, }, // Europe/Berlin
    {  489, 2167,   74546,  8, }, // Europe/Brussels
    {  489, 3928,  158380,  9, }, // Europe/Bucharest
    {  489, 3806,  158389,  9, }, // Europe/Budapest
    {  489, 8772,   27037,  8, }, // Europe/Busingen
    {  489, 6289,   21081, 10, }, // Europe/Copenhagen
    {  489, 6612,  158398,  9, }, // Europe/Helsinki
    {  489, 8804,  158407, 11, }, // Europe/Isle_of_Man
    {  489, 1130,   57447,  8, }, // Europe/Istanbul
    {  489, 3979,   63006, 12, }, // Europe/Kaliningrad
    {  489, 2527,    8148,  4, }, // Europe/Kyiv
    {  489, 3213,   21137,  6, }, // Europe/Lisbon
    {  489, 6506,   21143,  9, }, // Europe/Ljubljana
    {  489, 2501,   21152,  7, }, // Europe/London
    {  489, 6365,   74574, 10, }, // Europe/Luxembourg
    {  489, 4892,   27075,  5, }, // Europe/Madrid
    {  489, 6410,  158418,  6, }, // Europe/Monaco
    {  489, 3537,   44576,  6, }, // Europe/Moscow
    {  489, 6275,   21185,  5, }, // Europe/Prague
    {  489, 6324,    6064,  4, }, // Europe/Rome
    {  489, 6599,  158424,  5, }, // Europe/Sofia
    {  489, 6523,   74615,  9, }, // Europe/Stockholm
    {  489, 6202,   21213,  6, }, // Europe/Tirane
    {  489, 8882,   27092,  9, }, // Europe/Ulyanovsk
    {  489, 6567,   74624,  8, }, // Europe/Vatican
    {  489, 6231,   21230,  5, }, // Europe/Vienna
    {  489, 4727,   63187, 10, }, // Europe/Volgograd
    {  489, 3190,  158429,  8, }, // Europe/Warsaw
    {  489, 6540,  158437,  7, }, // Europe/Zurich
    {  489, 5029,   65504,  7, }, // Indian/Comoro
    {  489, 7260,   27115,  4, }, // Indian/Mahe
    {  489, 7050,   74646,  8, }, // Indian/Maldives
    {  489, 4069,  158444,  8, }, // Indian/Mauritius
    {  489, 7211,  158452,  7, }, // Indian/Reunion
    {  489, 3015,  158459,  7, }, // Pacific/Chatham
    {  489, 3157,     554,  5, }, // Pacific/Chuuk
    {  489, 2237,  158466, 14, }, // Pacific/Easter
    {  489, 7360,   27148,  5, }, // Pacific/Efate
    {  489, 6678,   21277,  9, }, // Pacific/Galapagos
    {  489, 2723,    4611,  8, }, // Pacific/Honolulu
    {  489, 3049,    4619,  9, }, // Pacific/Kanton
    {  489, 7098,    4628,  6, }, // Pacific/Noumea
    {  489, 3096,     586,  7, }, // Pacific/Pohnpei
    {  489, 7312,  124697,  5, }, // Pacific/Tahiti
    {  489, 6712,  158480,  6, }, // Pacific/Tarawa
    {  490, 5059,  158486, 10, }, // Africa/Addis_Ababa Portuguese/Latin/Angola
    {  490,   44,   74138,  6, }, // Africa/Bamako
    {  490, 6840,  158496,  5, }, // Africa/Dakar
    {  490, 5043,  158501,  6, }, // Africa/Djibouti
    {  490, 5151,  158507,  7, }, // Africa/Kampala
    {  490, 5001,   74176,  6, }, // Africa/Lusaka
    {  490, 3853,  124956,  7, }, // Africa/Nairobi
    {  490, 5271,   65556,  8, }, // Africa/Ndjamena
    {  490, 5354,   26528,  6, }, // Africa/Niamey
    {  490, 5225,  158514, 10, }, // Africa/Porto-Novo
    {  490, 2866,   35888,  7, }, // Africa/Tripoli
    {  490, 6554,   74198,  5, }, // Africa/Tunis
    {  490, 4554,  158524,  9, }, // America/Araguaina
    {  490, 7571,   20486,  7, }, // America/Argentina/Tucuman
    {  490, 3679,   74203,  4, }, // America/Bahia
    {  490, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  490, 3868,   33314,  6, }, // America/Cancun
    {  490, 5496,  158533,  6, }, // America/Cayman
    {  490, 5723,  158539,  7, }, // America/Curacao
    {  490, 5739,  158546,  8, }, // America/Dominica
    {  490, 5441,    6806,  8, }, // America/El_Salvador
    {  490, 4098,  158554, 10, }, // America/Montevideo
    {  490, 5824,  158564, 10, }, // America/Montserrat
    {  490, 2379,  158574, 11, }, // America/New_York
    {  490, 3945,  158585, 14, }, // America/Port-au-Prince
    {  490, 5954,  158599, 16, }, // America/Port_of_Spain
    {  490, 2061,     351, 10, }, // America/St_Johns
    {  490,  855,     379, 10, }, // America/St_Thomas
    {  490, 5570,  158615,  4, }, // Asia/Aden
    {  490, 5523,   91325,  5, }, // Asia/Aqtau
    {  490, 5368,  158619,  6, }, // Asia/Aqtobe
    {  490, 3607,  158625,  7, }, // Asia/Baghdad
    {  490, 5534,  158632,  5, }, // Asia/Bahrain
    {  490, 4466,  158637, 11, }, // Asia/Bangkok
    {  490, 1093,  158496,  4, }, // Asia/Dhaka
    {  490, 4239,  158648,  7, }, // Asia/Karachi
    {  490, 1185,  158655,  8, }, // Asia/Kathmandu
    {  490, 5547,   36061,  6, }, // Asia/Kuwait
    {  490, 1349,   65218,  8, }, // Asia/Makassar
    {  490, 8708,     469,  8, }, // Asia/Qostanay
    {  490, 3263,  158663,  5, }, // Asia/Taipei
    {  490, 2772,  158668,  6, }, // Asia/Tehran
    {  490, 1317,    5758,  6, }, // Asia/Thimphu
    {  490, 1235,     477,  6, }, // Asia/Yangon
    {  490, 3747,   26947,  6, }, // Asia/Yerevan
    {  490, 1395,  158340,  5, }, // Atlantic/Faroe
    {  490, 2748,  158674, 12, }, // Atlantic/Reykjavik
    {  490, 1547,  158686, 17, }, // Australia/Lord_Howe
    {  490, 6441,  158703, 10, }, // Europe/Amsterdam
    {  490, 8772,  158713,  8, }, // Europe/Busingen
    {  490, 6289,  158721,  9, }, // Europe/Copenhagen
    {  490, 6612,  158730, 10, }, // Europe/Helsinki
    {  490, 3979,  158740, 12, }, // Europe/Kaliningrad
    {  490, 2501,   21152,  7, }, // Europe/London
    {  490, 4892,   63057,  6, }, // Europe/Madrid
    {  490, 6410,   74584,  6, }, // Europe/Monaco
    {  490, 3537,  158752,  7, }, // Europe/Moscow
    {  490, 6470,  158759, 11, }, // Europe/San_Marino
    {  490, 8867,  158770,  5, }, // Europe/Tallinn
    {  490, 6046,  158775, 13, }, // Indian/Christmas
    {  490, 6063,  158788, 11, }, // Indian/Cocos
    {  490, 7260,  158799,  4, }, // Indian/Mahe
    {  490, 4069,  158803,  8, }, // Indian/Mauritius
    {  490, 3015,   58451,  7, }, // Pacific/Chatham
    {  490, 2237,  158811, 14, }, // Pacific/Easter
    {  490, 7360,  158825,  5, }, // Pacific/Efate
    {  490, 7194,  158830, 14, }, // Pacific/Pitcairn
    {  490, 6712,  158844,  6, }, // Pacific/Tarawa
    {  491, 5059,  158486, 10, }, // Africa/Addis_Ababa Portuguese/Latin/Cape Verde
    {  491,   44,   74138,  6, }, // Africa/Bamako
    {  491, 6840,  158496,  5, }, // Africa/Dakar
    {  491, 5043,  158501,  6, }, // Africa/Djibouti
    {  491, 5151,  158507,  7, }, // Africa/Kampala
    {  491, 5001,   74176,  6, }, // Africa/Lusaka
    {  491, 3853,  124956,  7, }, // Africa/Nairobi
    {  491, 5271,   65556,  8, }, // Africa/Ndjamena
    {  491, 5354,   26528,  6, }, // Africa/Niamey
    {  491, 5225,  158514, 10, }, // Africa/Porto-Novo
    {  491, 2866,   35888,  7, }, // Africa/Tripoli
    {  491, 6554,   74198,  5, }, // Africa/Tunis
    {  491, 4554,  158524,  9, }, // America/Araguaina
    {  491, 7571,   20486,  7, }, // America/Argentina/Tucuman
    {  491, 3679,   74203,  4, }, // America/Bahia
    {  491, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  491, 3868,   33314,  6, }, // America/Cancun
    {  491, 5496,  158533,  6, }, // America/Cayman
    {  491, 5723,  158539,  7, }, // America/Curacao
    {  491, 5739,  158546,  8, }, // America/Dominica
    {  491, 5441,    6806,  8, }, // America/El_Salvador
    {  491, 4098,  158554, 10, }, // America/Montevideo
    {  491, 5824,  158564, 10, }, // America/Montserrat
    {  491, 2379,  158574, 11, }, // America/New_York
    {  491, 3945,  158585, 14, }, // America/Port-au-Prince
    {  491, 5954,  158599, 16, }, // America/Port_of_Spain
    {  491, 2061,     351, 10, }, // America/St_Johns
    {  491,  855,     379, 10, }, // America/St_Thomas
    {  491, 5570,  158615,  4, }, // Asia/Aden
    {  491, 5523,   91325,  5, }, // Asia/Aqtau
    {  491, 5368,  158619,  6, }, // Asia/Aqtobe
    {  491, 3607,  158625,  7, }, // Asia/Baghdad
    {  491, 5534,  158632,  5, }, // Asia/Bahrain
    {  491, 4466,  158637, 11, }, // Asia/Bangkok
    {  491, 1093,  158496,  4, }, // Asia/Dhaka
    {  491, 4239,  158648,  7, }, // Asia/Karachi
    {  491, 1185,  158655,  8, }, // Asia/Kathmandu
    {  491, 5547,   36061,  6, }, // Asia/Kuwait
    {  491, 1349,   65218,  8, }, // Asia/Makassar
    {  491, 8708,     469,  8, }, // Asia/Qostanay
    {  491, 3263,  158663,  5, }, // Asia/Taipei
    {  491, 2772,  158668,  6, }, // Asia/Tehran
    {  491, 1317,    5758,  6, }, // Asia/Thimphu
    {  491, 1235,     477,  6, }, // Asia/Yangon
    {  491, 3747,   26947,  6, }, // Asia/Yerevan
    {  491, 1395,  158340,  5, }, // Atlantic/Faroe
    {  491, 2748,  158674, 12, }, // Atlantic/Reykjavik
    {  491, 1547,  158686, 17, }, // Australia/Lord_Howe
    {  491, 6441,  158703, 10, }, // Europe/Amsterdam
    {  491, 8772,  158713,  8, }, // Europe/Busingen
    {  491, 6289,  158721,  9, }, // Europe/Copenhagen
    {  491, 6612,  158730, 10, }, // Europe/Helsinki
    {  491, 3979,  158740, 12, }, // Europe/Kaliningrad
    {  491, 2501,   21152,  7, }, // Europe/London
    {  491, 4892,   63057,  6, }, // Europe/Madrid
    {  491, 6410,   74584,  6, }, // Europe/Monaco
    {  491, 3537,  158752,  7, }, // Europe/Moscow
    {  491, 6470,  158759, 11, }, // Europe/San_Marino
    {  491, 8867,  158770,  5, }, // Europe/Tallinn
    {  491, 6046,  158775, 13, }, // Indian/Christmas
    {  491, 6063,  158788, 11, }, // Indian/Cocos
    {  491, 7260,  158799,  4, }, // Indian/Mahe
    {  491, 4069,  158803,  8, }, // Indian/Mauritius
    {  491, 3015,   58451,  7, }, // Pacific/Chatham
    {  491, 2237,  158811, 14, }, // Pacific/Easter
    {  491, 7360,  158825,  5, }, // Pacific/Efate
    {  491, 7194,  158830, 14, }, // Pacific/Pitcairn
    {  491, 6712,  158844,  6, }, // Pacific/Tarawa
    {  492, 5059,  158486, 10, }, // Africa/Addis_Ababa Portuguese/Latin/Equatorial Guinea
    {  492,   44,   74138,  6, }, // Africa/Bamako
    {  492, 6840,  158496,  5, }, // Africa/Dakar
    {  492, 5043,  158501,  6, }, // Africa/Djibouti
    {  492, 5151,  158507,  7, }, // Africa/Kampala
    {  492, 5001,   74176,  6, }, // Africa/Lusaka
    {  492, 3853,  124956,  7, }, // Africa/Nairobi
    {  492, 5271,   65556,  8, }, // Africa/Ndjamena
    {  492, 5354,   26528,  6, }, // Africa/Niamey
    {  492, 5225,  158514, 10, }, // Africa/Porto-Novo
    {  492, 2866,   35888,  7, }, // Africa/Tripoli
    {  492, 6554,   74198,  5, }, // Africa/Tunis
    {  492, 4554,  158524,  9, }, // America/Araguaina
    {  492, 7571,   20486,  7, }, // America/Argentina/Tucuman
    {  492, 3679,   74203,  4, }, // America/Bahia
    {  492, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  492, 3868,   33314,  6, }, // America/Cancun
    {  492, 5496,  158533,  6, }, // America/Cayman
    {  492, 5723,  158539,  7, }, // America/Curacao
    {  492, 5739,  158546,  8, }, // America/Dominica
    {  492, 5441,    6806,  8, }, // America/El_Salvador
    {  492, 4098,  158554, 10, }, // America/Montevideo
    {  492, 5824,  158564, 10, }, // America/Montserrat
    {  492, 2379,  158574, 11, }, // America/New_York
    {  492, 3945,  158585, 14, }, // America/Port-au-Prince
    {  492, 5954,  158599, 16, }, // America/Port_of_Spain
    {  492, 2061,     351, 10, }, // America/St_Johns
    {  492,  855,     379, 10, }, // America/St_Thomas
    {  492, 5570,  158615,  4, }, // Asia/Aden
    {  492, 5523,   91325,  5, }, // Asia/Aqtau
    {  492, 5368,  158619,  6, }, // Asia/Aqtobe
    {  492, 3607,  158625,  7, }, // Asia/Baghdad
    {  492, 5534,  158632,  5, }, // Asia/Bahrain
    {  492, 4466,  158637, 11, }, // Asia/Bangkok
    {  492, 1093,  158496,  4, }, // Asia/Dhaka
    {  492, 4239,  158648,  7, }, // Asia/Karachi
    {  492, 1185,  158655,  8, }, // Asia/Kathmandu
    {  492, 5547,   36061,  6, }, // Asia/Kuwait
    {  492, 1349,   65218,  8, }, // Asia/Makassar
    {  492, 8708,     469,  8, }, // Asia/Qostanay
    {  492, 3263,  158663,  5, }, // Asia/Taipei
    {  492, 2772,  158668,  6, }, // Asia/Tehran
    {  492, 1317,    5758,  6, }, // Asia/Thimphu
    {  492, 1235,     477,  6, }, // Asia/Yangon
    {  492, 3747,   26947,  6, }, // Asia/Yerevan
    {  492, 1395,  158340,  5, }, // Atlantic/Faroe
    {  492, 2748,  158674, 12, }, // Atlantic/Reykjavik
    {  492, 1547,  158686, 17, }, // Australia/Lord_Howe
    {  492, 6441,  158703, 10, }, // Europe/Amsterdam
    {  492, 8772,  158713,  8, }, // Europe/Busingen
    {  492, 6289,  158721,  9, }, // Europe/Copenhagen
    {  492, 6612,  158730, 10, }, // Europe/Helsinki
    {  492, 3979,  158740, 12, }, // Europe/Kaliningrad
    {  492, 2501,   21152,  7, }, // Europe/London
    {  492, 4892,   63057,  6, }, // Europe/Madrid
    {  492, 6410,   74584,  6, }, // Europe/Monaco
    {  492, 3537,  158752,  7, }, // Europe/Moscow
    {  492, 6470,  158759, 11, }, // Europe/San_Marino
    {  492, 8867,  158770,  5, }, // Europe/Tallinn
    {  492, 6046,  158775, 13, }, // Indian/Christmas
    {  492, 6063,  158788, 11, }, // Indian/Cocos
    {  492, 7260,  158799,  4, }, // Indian/Mahe
    {  492, 4069,  158803,  8, }, // Indian/Mauritius
    {  492, 3015,   58451,  7, }, // Pacific/Chatham
    {  492, 2237,  158811, 14, }, // Pacific/Easter
    {  492, 7360,  158825,  5, }, // Pacific/Efate
    {  492, 7194,  158830, 14, }, // Pacific/Pitcairn
    {  492, 6712,  158844,  6, }, // Pacific/Tarawa
    {  493, 5059,  158486, 10, }, // Africa/Addis_Ababa Portuguese/Latin/Guinea-Bissau
    {  493,   44,   74138,  6, }, // Africa/Bamako
    {  493, 6840,  158496,  5, }, // Africa/Dakar
    {  493, 5043,  158501,  6, }, // Africa/Djibouti
    {  493, 5151,  158507,  7, }, // Africa/Kampala
    {  493, 5001,   74176,  6, }, // Africa/Lusaka
    {  493, 3853,  124956,  7, }, // Africa/Nairobi
    {  493, 5271,   65556,  8, }, // Africa/Ndjamena
    {  493, 5354,   26528,  6, }, // Africa/Niamey
    {  493, 5225,  158514, 10, }, // Africa/Porto-Novo
    {  493, 2866,   35888,  7, }, // Africa/Tripoli
    {  493, 6554,   74198,  5, }, // Africa/Tunis
    {  493, 4554,  158524,  9, }, // America/Araguaina
    {  493, 7571,   20486,  7, }, // America/Argentina/Tucuman
    {  493, 3679,   74203,  4, }, // America/Bahia
    {  493, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  493, 3868,   33314,  6, }, // America/Cancun
    {  493, 5496,  158533,  6, }, // America/Cayman
    {  493, 5723,  158539,  7, }, // America/Curacao
    {  493, 5739,  158546,  8, }, // America/Dominica
    {  493, 5441,    6806,  8, }, // America/El_Salvador
    {  493, 4098,  158554, 10, }, // America/Montevideo
    {  493, 5824,  158564, 10, }, // America/Montserrat
    {  493, 2379,  158574, 11, }, // America/New_York
    {  493, 3945,  158585, 14, }, // America/Port-au-Prince
    {  493, 5954,  158599, 16, }, // America/Port_of_Spain
    {  493, 2061,     351, 10, }, // America/St_Johns
    {  493,  855,     379, 10, }, // America/St_Thomas
    {  493, 5570,  158615,  4, }, // Asia/Aden
    {  493, 5523,   91325,  5, }, // Asia/Aqtau
    {  493, 5368,  158619,  6, }, // Asia/Aqtobe
    {  493, 3607,  158625,  7, }, // Asia/Baghdad
    {  493, 5534,  158632,  5, }, // Asia/Bahrain
    {  493, 4466,  158637, 11, }, // Asia/Bangkok
    {  493, 1093,  158496,  4, }, // Asia/Dhaka
    {  493, 4239,  158648,  7, }, // Asia/Karachi
    {  493, 1185,  158655,  8, }, // Asia/Kathmandu
    {  493, 5547,   36061,  6, }, // Asia/Kuwait
    {  493, 1349,   65218,  8, }, // Asia/Makassar
    {  493, 8708,     469,  8, }, // Asia/Qostanay
    {  493, 3263,  158663,  5, }, // Asia/Taipei
    {  493, 2772,  158668,  6, }, // Asia/Tehran
    {  493, 1317,    5758,  6, }, // Asia/Thimphu
    {  493, 1235,     477,  6, }, // Asia/Yangon
    {  493, 3747,   26947,  6, }, // Asia/Yerevan
    {  493, 1395,  158340,  5, }, // Atlantic/Faroe
    {  493, 2748,  158674, 12, }, // Atlantic/Reykjavik
    {  493, 1547,  158686, 17, }, // Australia/Lord_Howe
    {  493, 6441,  158703, 10, }, // Europe/Amsterdam
    {  493, 8772,  158713,  8, }, // Europe/Busingen
    {  493, 6289,  158721,  9, }, // Europe/Copenhagen
    {  493, 6612,  158730, 10, }, // Europe/Helsinki
    {  493, 3979,  158740, 12, }, // Europe/Kaliningrad
    {  493, 2501,   21152,  7, }, // Europe/London
    {  493, 4892,   63057,  6, }, // Europe/Madrid
    {  493, 6410,   74584,  6, }, // Europe/Monaco
    {  493, 3537,  158752,  7, }, // Europe/Moscow
    {  493, 6470,  158759, 11, }, // Europe/San_Marino
    {  493, 8867,  158770,  5, }, // Europe/Tallinn
    {  493, 6046,  158775, 13, }, // Indian/Christmas
    {  493, 6063,  158788, 11, }, // Indian/Cocos
    {  493, 7260,  158799,  4, }, // Indian/Mahe
    {  493, 4069,  158803,  8, }, // Indian/Mauritius
    {  493, 3015,   58451,  7, }, // Pacific/Chatham
    {  493, 2237,  158811, 14, }, // Pacific/Easter
    {  493, 7360,  158825,  5, }, // Pacific/Efate
    {  493, 7194,  158830, 14, }, // Pacific/Pitcairn
    {  493, 6712,  158844,  6, }, // Pacific/Tarawa
    {  494, 5059,  158486, 10, }, // Africa/Addis_Ababa Portuguese/Latin/Luxembourg
    {  494,   44,   74138,  6, }, // Africa/Bamako
    {  494, 6840,  158496,  5, }, // Africa/Dakar
    {  494, 5043,  158501,  6, }, // Africa/Djibouti
    {  494, 5151,  158507,  7, }, // Africa/Kampala
    {  494, 5001,   74176,  6, }, // Africa/Lusaka
    {  494, 3853,  124956,  7, }, // Africa/Nairobi
    {  494, 5271,   65556,  8, }, // Africa/Ndjamena
    {  494, 5354,   26528,  6, }, // Africa/Niamey
    {  494, 5225,  158514, 10, }, // Africa/Porto-Novo
    {  494, 2866,   35888,  7, }, // Africa/Tripoli
    {  494, 6554,   74198,  5, }, // Africa/Tunis
    {  494, 4554,  158524,  9, }, // America/Araguaina
    {  494, 7571,   20486,  7, }, // America/Argentina/Tucuman
    {  494, 3679,   74203,  4, }, // America/Bahia
    {  494, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  494, 3868,   33314,  6, }, // America/Cancun
    {  494, 5496,  158533,  6, }, // America/Cayman
    {  494, 5723,  158539,  7, }, // America/Curacao
    {  494, 5739,  158546,  8, }, // America/Dominica
    {  494, 5441,    6806,  8, }, // America/El_Salvador
    {  494, 4098,  158554, 10, }, // America/Montevideo
    {  494, 5824,  158564, 10, }, // America/Montserrat
    {  494, 2379,  158574, 11, }, // America/New_York
    {  494, 3945,  158585, 14, }, // America/Port-au-Prince
    {  494, 5954,  158599, 16, }, // America/Port_of_Spain
    {  494, 2061,     351, 10, }, // America/St_Johns
    {  494,  855,     379, 10, }, // America/St_Thomas
    {  494, 5570,  158615,  4, }, // Asia/Aden
    {  494, 5523,   91325,  5, }, // Asia/Aqtau
    {  494, 5368,  158619,  6, }, // Asia/Aqtobe
    {  494, 3607,  158625,  7, }, // Asia/Baghdad
    {  494, 5534,  158632,  5, }, // Asia/Bahrain
    {  494, 4466,  158637, 11, }, // Asia/Bangkok
    {  494, 1093,  158496,  4, }, // Asia/Dhaka
    {  494, 4239,  158648,  7, }, // Asia/Karachi
    {  494, 1185,  158655,  8, }, // Asia/Kathmandu
    {  494, 5547,   36061,  6, }, // Asia/Kuwait
    {  494, 1349,   65218,  8, }, // Asia/Makassar
    {  494, 8708,     469,  8, }, // Asia/Qostanay
    {  494, 3263,  158663,  5, }, // Asia/Taipei
    {  494, 2772,  158668,  6, }, // Asia/Tehran
    {  494, 1317,    5758,  6, }, // Asia/Thimphu
    {  494, 1235,     477,  6, }, // Asia/Yangon
    {  494, 3747,   26947,  6, }, // Asia/Yerevan
    {  494, 1395,  158340,  5, }, // Atlantic/Faroe
    {  494, 2748,  158674, 12, }, // Atlantic/Reykjavik
    {  494, 1547,  158686, 17, }, // Australia/Lord_Howe
    {  494, 6441,  158703, 10, }, // Europe/Amsterdam
    {  494, 8772,  158713,  8, }, // Europe/Busingen
    {  494, 6289,  158721,  9, }, // Europe/Copenhagen
    {  494, 6612,  158730, 10, }, // Europe/Helsinki
    {  494, 3979,  158740, 12, }, // Europe/Kaliningrad
    {  494, 2501,   21152,  7, }, // Europe/London
    {  494, 4892,   63057,  6, }, // Europe/Madrid
    {  494, 6410,   74584,  6, }, // Europe/Monaco
    {  494, 3537,  158752,  7, }, // Europe/Moscow
    {  494, 6470,  158759, 11, }, // Europe/San_Marino
    {  494, 8867,  158770,  5, }, // Europe/Tallinn
    {  494, 6046,  158775, 13, }, // Indian/Christmas
    {  494, 6063,  158788, 11, }, // Indian/Cocos
    {  494, 7260,  158799,  4, }, // Indian/Mahe
    {  494, 4069,  158803,  8, }, // Indian/Mauritius
    {  494, 3015,   58451,  7, }, // Pacific/Chatham
    {  494, 2237,  158811, 14, }, // Pacific/Easter
    {  494, 7360,  158825,  5, }, // Pacific/Efate
    {  494, 7194,  158830, 14, }, // Pacific/Pitcairn
    {  494, 6712,  158844,  6, }, // Pacific/Tarawa
    {  495, 5059,  158486, 10, }, // Africa/Addis_Ababa Portuguese/Latin/Macao
    {  495,   44,   74138,  6, }, // Africa/Bamako
    {  495, 6840,  158496,  5, }, // Africa/Dakar
    {  495, 5043,  158501,  6, }, // Africa/Djibouti
    {  495, 5151,  158507,  7, }, // Africa/Kampala
    {  495, 5001,   74176,  6, }, // Africa/Lusaka
    {  495, 3853,  124956,  7, }, // Africa/Nairobi
    {  495, 5271,   65556,  8, }, // Africa/Ndjamena
    {  495, 5354,   26528,  6, }, // Africa/Niamey
    {  495, 5225,  158514, 10, }, // Africa/Porto-Novo
    {  495, 2866,   35888,  7, }, // Africa/Tripoli
    {  495, 6554,   74198,  5, }, // Africa/Tunis
    {  495, 4554,  158524,  9, }, // America/Araguaina
    {  495, 7571,   20486,  7, }, // America/Argentina/Tucuman
    {  495, 3679,   74203,  4, }, // America/Bahia
    {  495, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  495, 3868,   33314,  6, }, // America/Cancun
    {  495, 5496,  158533,  6, }, // America/Cayman
    {  495, 5723,  158539,  7, }, // America/Curacao
    {  495, 5739,  158546,  8, }, // America/Dominica
    {  495, 5441,    6806,  8, }, // America/El_Salvador
    {  495, 4098,  158554, 10, }, // America/Montevideo
    {  495, 5824,  158564, 10, }, // America/Montserrat
    {  495, 2379,  158574, 11, }, // America/New_York
    {  495, 3945,  158585, 14, }, // America/Port-au-Prince
    {  495, 5954,  158599, 16, }, // America/Port_of_Spain
    {  495, 2061,     351, 10, }, // America/St_Johns
    {  495,  855,     379, 10, }, // America/St_Thomas
    {  495, 5570,  158615,  4, }, // Asia/Aden
    {  495, 5523,   91325,  5, }, // Asia/Aqtau
    {  495, 5368,  158619,  6, }, // Asia/Aqtobe
    {  495, 3607,  158625,  7, }, // Asia/Baghdad
    {  495, 5534,  158632,  5, }, // Asia/Bahrain
    {  495, 4466,  158637, 11, }, // Asia/Bangkok
    {  495, 1093,  158496,  4, }, // Asia/Dhaka
    {  495, 4239,  158648,  7, }, // Asia/Karachi
    {  495, 1185,  158655,  8, }, // Asia/Kathmandu
    {  495, 5547,   36061,  6, }, // Asia/Kuwait
    {  495, 1349,   65218,  8, }, // Asia/Makassar
    {  495, 8708,     469,  8, }, // Asia/Qostanay
    {  495, 3263,  158663,  5, }, // Asia/Taipei
    {  495, 2772,  158668,  6, }, // Asia/Tehran
    {  495, 1317,    5758,  6, }, // Asia/Thimphu
    {  495, 1235,     477,  6, }, // Asia/Yangon
    {  495, 3747,   26947,  6, }, // Asia/Yerevan
    {  495, 1395,  158340,  5, }, // Atlantic/Faroe
    {  495, 2748,  158674, 12, }, // Atlantic/Reykjavik
    {  495, 1547,  158686, 17, }, // Australia/Lord_Howe
    {  495, 6441,  158703, 10, }, // Europe/Amsterdam
    {  495, 8772,  158713,  8, }, // Europe/Busingen
    {  495, 6289,  158721,  9, }, // Europe/Copenhagen
    {  495, 6612,  158730, 10, }, // Europe/Helsinki
    {  495, 3979,  158740, 12, }, // Europe/Kaliningrad
    {  495, 2501,   21152,  7, }, // Europe/London
    {  495, 4892,   63057,  6, }, // Europe/Madrid
    {  495, 6410,   74584,  6, }, // Europe/Monaco
    {  495, 3537,  158752,  7, }, // Europe/Moscow
    {  495, 6470,  158759, 11, }, // Europe/San_Marino
    {  495, 8867,  158770,  5, }, // Europe/Tallinn
    {  495, 6046,  158775, 13, }, // Indian/Christmas
    {  495, 6063,  158788, 11, }, // Indian/Cocos
    {  495, 7260,  158799,  4, }, // Indian/Mahe
    {  495, 4069,  158803,  8, }, // Indian/Mauritius
    {  495, 3015,   58451,  7, }, // Pacific/Chatham
    {  495, 2237,  158811, 14, }, // Pacific/Easter
    {  495, 7360,  158825,  5, }, // Pacific/Efate
    {  495, 7194,  158830, 14, }, // Pacific/Pitcairn
    {  495, 6712,  158844,  6, }, // Pacific/Tarawa
    {  496, 5059,  158486, 10, }, // Africa/Addis_Ababa Portuguese/Latin/Mozambique
    {  496,   44,   74138,  6, }, // Africa/Bamako
    {  496, 6840,  158496,  5, }, // Africa/Dakar
    {  496, 5043,  158501,  6, }, // Africa/Djibouti
    {  496, 5151,  158507,  7, }, // Africa/Kampala
    {  496, 5001,   74176,  6, }, // Africa/Lusaka
    {  496, 3853,  124956,  7, }, // Africa/Nairobi
    {  496, 5271,   65556,  8, }, // Africa/Ndjamena
    {  496, 5354,   26528,  6, }, // Africa/Niamey
    {  496, 5225,  158514, 10, }, // Africa/Porto-Novo
    {  496, 2866,   35888,  7, }, // Africa/Tripoli
    {  496, 6554,   74198,  5, }, // Africa/Tunis
    {  496, 4554,  158524,  9, }, // America/Araguaina
    {  496, 7571,   20486,  7, }, // America/Argentina/Tucuman
    {  496, 3679,   74203,  4, }, // America/Bahia
    {  496, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  496, 3868,   33314,  6, }, // America/Cancun
    {  496, 5496,  158533,  6, }, // America/Cayman
    {  496, 5723,  158539,  7, }, // America/Curacao
    {  496, 5739,  158546,  8, }, // America/Dominica
    {  496, 5441,    6806,  8, }, // America/El_Salvador
    {  496, 4098,  158554, 10, }, // America/Montevideo
    {  496, 5824,  158564, 10, }, // America/Montserrat
    {  496, 2379,  158574, 11, }, // America/New_York
    {  496, 3945,  158585, 14, }, // America/Port-au-Prince
    {  496, 5954,  158599, 16, }, // America/Port_of_Spain
    {  496, 2061,     351, 10, }, // America/St_Johns
    {  496,  855,     379, 10, }, // America/St_Thomas
    {  496, 5570,  158615,  4, }, // Asia/Aden
    {  496, 5523,   91325,  5, }, // Asia/Aqtau
    {  496, 5368,  158619,  6, }, // Asia/Aqtobe
    {  496, 3607,  158625,  7, }, // Asia/Baghdad
    {  496, 5534,  158632,  5, }, // Asia/Bahrain
    {  496, 4466,  158637, 11, }, // Asia/Bangkok
    {  496, 1093,  158496,  4, }, // Asia/Dhaka
    {  496, 4239,  158648,  7, }, // Asia/Karachi
    {  496, 1185,  158655,  8, }, // Asia/Kathmandu
    {  496, 5547,   36061,  6, }, // Asia/Kuwait
    {  496, 1349,   65218,  8, }, // Asia/Makassar
    {  496, 8708,     469,  8, }, // Asia/Qostanay
    {  496, 3263,  158663,  5, }, // Asia/Taipei
    {  496, 2772,  158668,  6, }, // Asia/Tehran
    {  496, 1317,    5758,  6, }, // Asia/Thimphu
    {  496, 1235,     477,  6, }, // Asia/Yangon
    {  496, 3747,   26947,  6, }, // Asia/Yerevan
    {  496, 1395,  158340,  5, }, // Atlantic/Faroe
    {  496, 2748,  158674, 12, }, // Atlantic/Reykjavik
    {  496, 1547,  158686, 17, }, // Australia/Lord_Howe
    {  496, 6441,  158703, 10, }, // Europe/Amsterdam
    {  496, 8772,  158713,  8, }, // Europe/Busingen
    {  496, 6289,  158721,  9, }, // Europe/Copenhagen
    {  496, 6612,  158730, 10, }, // Europe/Helsinki
    {  496, 3979,  158740, 12, }, // Europe/Kaliningrad
    {  496, 2501,   21152,  7, }, // Europe/London
    {  496, 4892,   63057,  6, }, // Europe/Madrid
    {  496, 6410,   74584,  6, }, // Europe/Monaco
    {  496, 3537,  158752,  7, }, // Europe/Moscow
    {  496, 6470,  158759, 11, }, // Europe/San_Marino
    {  496, 8867,  158770,  5, }, // Europe/Tallinn
    {  496, 6046,  158775, 13, }, // Indian/Christmas
    {  496, 6063,  158788, 11, }, // Indian/Cocos
    {  496, 7260,  158799,  4, }, // Indian/Mahe
    {  496, 4069,  158803,  8, }, // Indian/Mauritius
    {  496, 3015,   58451,  7, }, // Pacific/Chatham
    {  496, 2237,  158811, 14, }, // Pacific/Easter
    {  496, 7360,  158825,  5, }, // Pacific/Efate
    {  496, 7194,  158830, 14, }, // Pacific/Pitcairn
    {  496, 6712,  158844,  6, }, // Pacific/Tarawa
    {  497, 5059,  158486, 10, }, // Africa/Addis_Ababa Portuguese/Latin/Portugal
    {  497,   44,   74138,  6, }, // Africa/Bamako
    {  497, 6840,  158496,  5, }, // Africa/Dakar
    {  497, 5043,  158501,  6, }, // Africa/Djibouti
    {  497, 5151,  158507,  7, }, // Africa/Kampala
    {  497, 5001,   74176,  6, }, // Africa/Lusaka
    {  497, 3853,  124956,  7, }, // Africa/Nairobi
    {  497, 5271,   65556,  8, }, // Africa/Ndjamena
    {  497, 5354,   26528,  6, }, // Africa/Niamey
    {  497, 5225,  158514, 10, }, // Africa/Porto-Novo
    {  497, 2866,   35888,  7, }, // Africa/Tripoli
    {  497, 6554,   74198,  5, }, // Africa/Tunis
    {  497, 4554,  158524,  9, }, // America/Araguaina
    {  497, 7571,   20486,  7, }, // America/Argentina/Tucuman
    {  497, 3679,   74203,  4, }, // America/Bahia
    {  497, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  497, 3868,   33314,  6, }, // America/Cancun
    {  497, 5496,  158533,  6, }, // America/Cayman
    {  497, 5723,  158539,  7, }, // America/Curacao
    {  497, 5739,  158546,  8, }, // America/Dominica
    {  497, 5441,    6806,  8, }, // America/El_Salvador
    {  497, 4098,  158554, 10, }, // America/Montevideo
    {  497, 5824,  158564, 10, }, // America/Montserrat
    {  497, 2379,  158574, 11, }, // America/New_York
    {  497, 3945,  158585, 14, }, // America/Port-au-Prince
    {  497, 5954,  158599, 16, }, // America/Port_of_Spain
    {  497, 2061,     351, 10, }, // America/St_Johns
    {  497,  855,     379, 10, }, // America/St_Thomas
    {  497, 5570,  158615,  4, }, // Asia/Aden
    {  497, 5523,   91325,  5, }, // Asia/Aqtau
    {  497, 5368,  158619,  6, }, // Asia/Aqtobe
    {  497, 3607,  158625,  7, }, // Asia/Baghdad
    {  497, 5534,  158632,  5, }, // Asia/Bahrain
    {  497, 4466,  158637, 11, }, // Asia/Bangkok
    {  497, 1093,  158496,  4, }, // Asia/Dhaka
    {  497, 4239,  158648,  7, }, // Asia/Karachi
    {  497, 1185,  158655,  8, }, // Asia/Kathmandu
    {  497, 5547,   36061,  6, }, // Asia/Kuwait
    {  497, 1349,   65218,  8, }, // Asia/Makassar
    {  497, 8708,     469,  8, }, // Asia/Qostanay
    {  497, 3263,  158663,  5, }, // Asia/Taipei
    {  497, 2772,  158668,  6, }, // Asia/Tehran
    {  497, 1317,    5758,  6, }, // Asia/Thimphu
    {  497, 1235,     477,  6, }, // Asia/Yangon
    {  497, 3747,   26947,  6, }, // Asia/Yerevan
    {  497, 1395,  158340,  5, }, // Atlantic/Faroe
    {  497, 2748,  158674, 12, }, // Atlantic/Reykjavik
    {  497, 1547,  158686, 17, }, // Australia/Lord_Howe
    {  497, 6441,  158703, 10, }, // Europe/Amsterdam
    {  497, 8772,  158713,  8, }, // Europe/Busingen
    {  497, 6289,  158721,  9, }, // Europe/Copenhagen
    {  497, 6612,  158730, 10, }, // Europe/Helsinki
    {  497, 3979,  158740, 12, }, // Europe/Kaliningrad
    {  497, 2501,   21152,  7, }, // Europe/London
    {  497, 4892,   63057,  6, }, // Europe/Madrid
    {  497, 6410,   74584,  6, }, // Europe/Monaco
    {  497, 3537,  158752,  7, }, // Europe/Moscow
    {  497, 6470,  158759, 11, }, // Europe/San_Marino
    {  497, 8867,  158770,  5, }, // Europe/Tallinn
    {  497, 6046,  158775, 13, }, // Indian/Christmas
    {  497, 6063,  158788, 11, }, // Indian/Cocos
    {  497, 7260,  158799,  4, }, // Indian/Mahe
    {  497, 4069,  158803,  8, }, // Indian/Mauritius
    {  497, 3015,   58451,  7, }, // Pacific/Chatham
    {  497, 2237,  158811, 14, }, // Pacific/Easter
    {  497, 7360,  158825,  5, }, // Pacific/Efate
    {  497, 7194,  158830, 14, }, // Pacific/Pitcairn
    {  497, 6712,  158844,  6, }, // Pacific/Tarawa
    {  498, 5059,  158486, 10, }, // Africa/Addis_Ababa Portuguese/Latin/Sao Tome and Principe
    {  498,   44,   74138,  6, }, // Africa/Bamako
    {  498, 6840,  158496,  5, }, // Africa/Dakar
    {  498, 5043,  158501,  6, }, // Africa/Djibouti
    {  498, 5151,  158507,  7, }, // Africa/Kampala
    {  498, 5001,   74176,  6, }, // Africa/Lusaka
    {  498, 3853,  124956,  7, }, // Africa/Nairobi
    {  498, 5271,   65556,  8, }, // Africa/Ndjamena
    {  498, 5354,   26528,  6, }, // Africa/Niamey
    {  498, 5225,  158514, 10, }, // Africa/Porto-Novo
    {  498, 2866,   35888,  7, }, // Africa/Tripoli
    {  498, 6554,   74198,  5, }, // Africa/Tunis
    {  498, 4554,  158524,  9, }, // America/Araguaina
    {  498, 7571,   20486,  7, }, // America/Argentina/Tucuman
    {  498, 3679,   74203,  4, }, // America/Bahia
    {  498, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  498, 3868,   33314,  6, }, // America/Cancun
    {  498, 5496,  158533,  6, }, // America/Cayman
    {  498, 5723,  158539,  7, }, // America/Curacao
    {  498, 5739,  158546,  8, }, // America/Dominica
    {  498, 5441,    6806,  8, }, // America/El_Salvador
    {  498, 4098,  158554, 10, }, // America/Montevideo
    {  498, 5824,  158564, 10, }, // America/Montserrat
    {  498, 2379,  158574, 11, }, // America/New_York
    {  498, 3945,  158585, 14, }, // America/Port-au-Prince
    {  498, 5954,  158599, 16, }, // America/Port_of_Spain
    {  498, 2061,     351, 10, }, // America/St_Johns
    {  498,  855,     379, 10, }, // America/St_Thomas
    {  498, 5570,  158615,  4, }, // Asia/Aden
    {  498, 5523,   91325,  5, }, // Asia/Aqtau
    {  498, 5368,  158619,  6, }, // Asia/Aqtobe
    {  498, 3607,  158625,  7, }, // Asia/Baghdad
    {  498, 5534,  158632,  5, }, // Asia/Bahrain
    {  498, 4466,  158637, 11, }, // Asia/Bangkok
    {  498, 1093,  158496,  4, }, // Asia/Dhaka
    {  498, 4239,  158648,  7, }, // Asia/Karachi
    {  498, 1185,  158655,  8, }, // Asia/Kathmandu
    {  498, 5547,   36061,  6, }, // Asia/Kuwait
    {  498, 1349,   65218,  8, }, // Asia/Makassar
    {  498, 8708,     469,  8, }, // Asia/Qostanay
    {  498, 3263,  158663,  5, }, // Asia/Taipei
    {  498, 2772,  158668,  6, }, // Asia/Tehran
    {  498, 1317,    5758,  6, }, // Asia/Thimphu
    {  498, 1235,     477,  6, }, // Asia/Yangon
    {  498, 3747,   26947,  6, }, // Asia/Yerevan
    {  498, 1395,  158340,  5, }, // Atlantic/Faroe
    {  498, 2748,  158674, 12, }, // Atlantic/Reykjavik
    {  498, 1547,  158686, 17, }, // Australia/Lord_Howe
    {  498, 6441,  158703, 10, }, // Europe/Amsterdam
    {  498, 8772,  158713,  8, }, // Europe/Busingen
    {  498, 6289,  158721,  9, }, // Europe/Copenhagen
    {  498, 6612,  158730, 10, }, // Europe/Helsinki
    {  498, 3979,  158740, 12, }, // Europe/Kaliningrad
    {  498, 2501,   21152,  7, }, // Europe/London
    {  498, 4892,   63057,  6, }, // Europe/Madrid
    {  498, 6410,   74584,  6, }, // Europe/Monaco
    {  498, 3537,  158752,  7, }, // Europe/Moscow
    {  498, 6470,  158759, 11, }, // Europe/San_Marino
    {  498, 8867,  158770,  5, }, // Europe/Tallinn
    {  498, 6046,  158775, 13, }, // Indian/Christmas
    {  498, 6063,  158788, 11, }, // Indian/Cocos
    {  498, 7260,  158799,  4, }, // Indian/Mahe
    {  498, 4069,  158803,  8, }, // Indian/Mauritius
    {  498, 3015,   58451,  7, }, // Pacific/Chatham
    {  498, 2237,  158811, 14, }, // Pacific/Easter
    {  498, 7360,  158825,  5, }, // Pacific/Efate
    {  498, 7194,  158830, 14, }, // Pacific/Pitcairn
    {  498, 6712,  158844,  6, }, // Pacific/Tarawa
    {  499, 5059,  158486, 10, }, // Africa/Addis_Ababa Portuguese/Latin/Switzerland
    {  499,   44,   74138,  6, }, // Africa/Bamako
    {  499, 6840,  158496,  5, }, // Africa/Dakar
    {  499, 5043,  158501,  6, }, // Africa/Djibouti
    {  499, 5151,  158507,  7, }, // Africa/Kampala
    {  499, 5001,   74176,  6, }, // Africa/Lusaka
    {  499, 3853,  124956,  7, }, // Africa/Nairobi
    {  499, 5271,   65556,  8, }, // Africa/Ndjamena
    {  499, 5354,   26528,  6, }, // Africa/Niamey
    {  499, 5225,  158514, 10, }, // Africa/Porto-Novo
    {  499, 2866,   35888,  7, }, // Africa/Tripoli
    {  499, 6554,   74198,  5, }, // Africa/Tunis
    {  499, 4554,  158524,  9, }, // America/Araguaina
    {  499, 7571,   20486,  7, }, // America/Argentina/Tucuman
    {  499, 3679,   74203,  4, }, // America/Bahia
    {  499, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  499, 3868,   33314,  6, }, // America/Cancun
    {  499, 5496,  158533,  6, }, // America/Cayman
    {  499, 5723,  158539,  7, }, // America/Curacao
    {  499, 5739,  158546,  8, }, // America/Dominica
    {  499, 5441,    6806,  8, }, // America/El_Salvador
    {  499, 4098,  158554, 10, }, // America/Montevideo
    {  499, 5824,  158564, 10, }, // America/Montserrat
    {  499, 2379,  158574, 11, }, // America/New_York
    {  499, 3945,  158585, 14, }, // America/Port-au-Prince
    {  499, 5954,  158599, 16, }, // America/Port_of_Spain
    {  499, 2061,     351, 10, }, // America/St_Johns
    {  499,  855,     379, 10, }, // America/St_Thomas
    {  499, 5570,  158615,  4, }, // Asia/Aden
    {  499, 5523,   91325,  5, }, // Asia/Aqtau
    {  499, 5368,  158619,  6, }, // Asia/Aqtobe
    {  499, 3607,  158625,  7, }, // Asia/Baghdad
    {  499, 5534,  158632,  5, }, // Asia/Bahrain
    {  499, 4466,  158637, 11, }, // Asia/Bangkok
    {  499, 1093,  158496,  4, }, // Asia/Dhaka
    {  499, 4239,  158648,  7, }, // Asia/Karachi
    {  499, 1185,  158655,  8, }, // Asia/Kathmandu
    {  499, 5547,   36061,  6, }, // Asia/Kuwait
    {  499, 1349,   65218,  8, }, // Asia/Makassar
    {  499, 8708,     469,  8, }, // Asia/Qostanay
    {  499, 3263,  158663,  5, }, // Asia/Taipei
    {  499, 2772,  158668,  6, }, // Asia/Tehran
    {  499, 1317,    5758,  6, }, // Asia/Thimphu
    {  499, 1235,     477,  6, }, // Asia/Yangon
    {  499, 3747,   26947,  6, }, // Asia/Yerevan
    {  499, 1395,  158340,  5, }, // Atlantic/Faroe
    {  499, 2748,  158674, 12, }, // Atlantic/Reykjavik
    {  499, 1547,  158686, 17, }, // Australia/Lord_Howe
    {  499, 6441,  158703, 10, }, // Europe/Amsterdam
    {  499, 8772,  158713,  8, }, // Europe/Busingen
    {  499, 6289,  158721,  9, }, // Europe/Copenhagen
    {  499, 6612,  158730, 10, }, // Europe/Helsinki
    {  499, 3979,  158740, 12, }, // Europe/Kaliningrad
    {  499, 2501,   21152,  7, }, // Europe/London
    {  499, 4892,   63057,  6, }, // Europe/Madrid
    {  499, 6410,   74584,  6, }, // Europe/Monaco
    {  499, 3537,  158752,  7, }, // Europe/Moscow
    {  499, 6470,  158759, 11, }, // Europe/San_Marino
    {  499, 8867,  158770,  5, }, // Europe/Tallinn
    {  499, 6046,  158775, 13, }, // Indian/Christmas
    {  499, 6063,  158788, 11, }, // Indian/Cocos
    {  499, 7260,  158799,  4, }, // Indian/Mahe
    {  499, 4069,  158803,  8, }, // Indian/Mauritius
    {  499, 3015,   58451,  7, }, // Pacific/Chatham
    {  499, 2237,  158811, 14, }, // Pacific/Easter
    {  499, 7360,  158825,  5, }, // Pacific/Efate
    {  499, 7194,  158830, 14, }, // Pacific/Pitcairn
    {  499, 6712,  158844,  6, }, // Pacific/Tarawa
    {  500, 5059,  158486, 10, }, // Africa/Addis_Ababa Portuguese/Latin/Timor-Leste
    {  500,   44,   74138,  6, }, // Africa/Bamako
    {  500, 6840,  158496,  5, }, // Africa/Dakar
    {  500, 5043,  158501,  6, }, // Africa/Djibouti
    {  500, 5151,  158507,  7, }, // Africa/Kampala
    {  500, 5001,   74176,  6, }, // Africa/Lusaka
    {  500, 3853,  124956,  7, }, // Africa/Nairobi
    {  500, 5271,   65556,  8, }, // Africa/Ndjamena
    {  500, 5354,   26528,  6, }, // Africa/Niamey
    {  500, 5225,  158514, 10, }, // Africa/Porto-Novo
    {  500, 2866,   35888,  7, }, // Africa/Tripoli
    {  500, 6554,   74198,  5, }, // Africa/Tunis
    {  500, 4554,  158524,  9, }, // America/Araguaina
    {  500, 7571,   20486,  7, }, // America/Argentina/Tucuman
    {  500, 3679,   74203,  4, }, // America/Bahia
    {  500, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  500, 3868,   33314,  6, }, // America/Cancun
    {  500, 5496,  158533,  6, }, // America/Cayman
    {  500, 5723,  158539,  7, }, // America/Curacao
    {  500, 5739,  158546,  8, }, // America/Dominica
    {  500, 5441,    6806,  8, }, // America/El_Salvador
    {  500, 4098,  158554, 10, }, // America/Montevideo
    {  500, 5824,  158564, 10, }, // America/Montserrat
    {  500, 2379,  158574, 11, }, // America/New_York
    {  500, 3945,  158585, 14, }, // America/Port-au-Prince
    {  500, 5954,  158599, 16, }, // America/Port_of_Spain
    {  500, 2061,     351, 10, }, // America/St_Johns
    {  500,  855,     379, 10, }, // America/St_Thomas
    {  500, 5570,  158615,  4, }, // Asia/Aden
    {  500, 5523,   91325,  5, }, // Asia/Aqtau
    {  500, 5368,  158619,  6, }, // Asia/Aqtobe
    {  500, 3607,  158625,  7, }, // Asia/Baghdad
    {  500, 5534,  158632,  5, }, // Asia/Bahrain
    {  500, 4466,  158637, 11, }, // Asia/Bangkok
    {  500, 1093,  158496,  4, }, // Asia/Dhaka
    {  500, 4239,  158648,  7, }, // Asia/Karachi
    {  500, 1185,  158655,  8, }, // Asia/Kathmandu
    {  500, 5547,   36061,  6, }, // Asia/Kuwait
    {  500, 1349,   65218,  8, }, // Asia/Makassar
    {  500, 8708,     469,  8, }, // Asia/Qostanay
    {  500, 3263,  158663,  5, }, // Asia/Taipei
    {  500, 2772,  158668,  6, }, // Asia/Tehran
    {  500, 1317,    5758,  6, }, // Asia/Thimphu
    {  500, 1235,     477,  6, }, // Asia/Yangon
    {  500, 3747,   26947,  6, }, // Asia/Yerevan
    {  500, 1395,  158340,  5, }, // Atlantic/Faroe
    {  500, 2748,  158674, 12, }, // Atlantic/Reykjavik
    {  500, 1547,  158686, 17, }, // Australia/Lord_Howe
    {  500, 6441,  158703, 10, }, // Europe/Amsterdam
    {  500, 8772,  158713,  8, }, // Europe/Busingen
    {  500, 6289,  158721,  9, }, // Europe/Copenhagen
    {  500, 6612,  158730, 10, }, // Europe/Helsinki
    {  500, 3979,  158740, 12, }, // Europe/Kaliningrad
    {  500, 2501,   21152,  7, }, // Europe/London
    {  500, 4892,   63057,  6, }, // Europe/Madrid
    {  500, 6410,   74584,  6, }, // Europe/Monaco
    {  500, 3537,  158752,  7, }, // Europe/Moscow
    {  500, 6470,  158759, 11, }, // Europe/San_Marino
    {  500, 8867,  158770,  5, }, // Europe/Tallinn
    {  500, 6046,  158775, 13, }, // Indian/Christmas
    {  500, 6063,  158788, 11, }, // Indian/Cocos
    {  500, 7260,  158799,  4, }, // Indian/Mahe
    {  500, 4069,  158803,  8, }, // Indian/Mauritius
    {  500, 3015,   58451,  7, }, // Pacific/Chatham
    {  500, 2237,  158811, 14, }, // Pacific/Easter
    {  500, 7360,  158825,  5, }, // Pacific/Efate
    {  500, 7194,  158830, 14, }, // Pacific/Pitcairn
    {  500, 6712,  158844,  6, }, // Pacific/Tarawa
    {  501,   14,       0,  6, }, // Africa/Asmara Prussian/Latin/Poland
    {  501, 4435,       6,  8, }, // Africa/Sao_Tome
    {  501, 4252,      14,  8, }, // America/Asuncion
    {  501,  237,      22,  8, }, // America/Atikokan
    {  501, 7623,      30, 17, }, // America/Bahia_Banderas
    {  501, 3868,      47,  6, }, // America/Cancun
    {  501, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  501, 5723,      66,  7, }, // America/Curacao
    {  501,  481,      73, 13, }, // America/Indiana/Knox
    {  501, 7961,      86, 16, }, // America/Indiana/Marengo
    {  501, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  501, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  501, 8038,     139, 14, }, // America/Indiana/Vevay
    {  501, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  501, 8086,     171, 16, }, // America/Indiana/Winamac
    {  501, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  501, 5932,     207, 22, }, // America/Lower_Princes
    {  501, 8220,     229,  6, }, // America/Merida
    {  501, 1850,     235, 19, }, // America/Noronha
    {  501, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  501, 8329,     274, 20, }, // America/North_Dakota/Center
    {  501, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  501,  393,     317,  4, }, // America/Nuuk
    {  501, 6899,     321, 16, }, // America/Scoresbysund
    {  501, 8508,     337, 14, }, // America/St_Barthelemy
    {  501, 2061,     351, 10, }, // America/St_Johns
    {  501, 5863,     361,  9, }, // America/St_Kitts
    {  501, 5880,     370,  9, }, // America/St_Lucia
    {  501,  855,     379, 10, }, // America/St_Thomas
    {  501, 5913,     389, 11, }, // America/St_Vincent
    {  501, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  501, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  501, 1185,     448,  9, }, // Asia/Kathmandu
    {  501,  992,     457,  7, }, // Asia/Kolkata
    {  501, 1211,     464,  5, }, // Asia/Macau
    {  501, 1235,     477,  6, }, // Asia/Yangon
    {  501, 1395,     483,  5, }, // Atlantic/Faroe
    {  501, 6821,     488, 10, }, // Atlantic/St_Helena
    {  501, 2527,     514,  4, }, // Europe/Kyiv
    {  501, 7211,     547,  7, }, // Indian/Reunion
    {  501, 3157,     554,  5, }, // Pacific/Chuuk
    {  501, 3096,     586,  7, }, // Pacific/Pohnpei
    {  502, 6788,  158850,  7, }, // Africa/Abidjan Punjabi/Gurmukhi/India
    {  502, 6760,  158857,  5, }, // Africa/Accra
    {  502, 5059,  158862, 10, }, // Africa/Addis_Ababa
    {  502, 7420,  158872,  7, }, // Africa/Algiers
    {  502,   14,  158879,  6, }, // Africa/Asmara
    {  502,   44,  158885,  5, }, // Africa/Bamako
    {  502, 5257,  158890,  6, }, // Africa/Bangui
    {  502, 6746,  158896,  6, }, // Africa/Banjul
    {  502, 7435,  158902,  5, }, // Africa/Bissau
    {  502, 4957,  158907,  7, }, // Africa/Blantyre
    {  502, 5287,  158914, 11, }, // Africa/Brazzaville
    {  502, 4922,  158925,  9, }, // Africa/Bujumbura
    {  502, 2320,  158934,  4, }, // Africa/Cairo
    {  502, 4117,  158938, 10, }, // Africa/Casablanca
    {  502, 7449,  158948,  5, }, // Africa/Ceuta
    {  502, 6773,  158953,  7, }, // Africa/Conakry
    {  502, 6840,  158960,  4, }, // Africa/Dakar
    {  502, 5130,  158964, 11, }, // Africa/Dar_es_Salaam
    {  502, 5043,  158975,  6, }, // Africa/Djibouti
    {  502, 5243,  158981,  5, }, // Africa/Douala
    {  502, 5166,  158986,  7, }, // Africa/El_Aaiun
    {  502, 6853,  158993,  7, }, // Africa/Freetown
    {  502, 4906,  159000,  7, }, // Africa/Gaborone
    {  502, 5015,  159007,  5, }, // Africa/Harare
    {  502, 4479,  159012,  8, }, // Africa/Johannesburg
    {  502, 4499,  159020,  4, }, // Africa/Juba
    {  502, 5151,  159024,  6, }, // Africa/Kampala
    {  502, 4524,  159030,  6, }, // Africa/Khartoum
    {  502, 4973,  159036,  6, }, // Africa/Kigali
    {  502, 5306,  159042,  8, }, // Africa/Kinshasa
    {  502, 4744,  159050,  5, }, // Africa/Lagos
    {  502, 5336,  159055,  8, }, // Africa/Libreville
    {  502, 6869,  159063,  3, }, // Africa/Lome
    {  502, 5211,  159066,  6, }, // Africa/Luanda
    {  502, 4939,  159072, 10, }, // Africa/Lubumbashi
    {  502, 5001,  159082,  6, }, // Africa/Lusaka
    {  502, 5322,  159088,  6, }, // Africa/Malabo
    {  502, 4987,  159094,  6, }, // Africa/Maputo
    {  502, 5197,  159100,  5, }, // Africa/Maseru
    {  502, 5182,  159105,  8, }, // Africa/Mbabane
    {  502, 5113,  159113,  9, }, // Africa/Mogadishu
    {  502, 7034,  159122,  8, }, // Africa/Monrovia
    {  502, 3853,  159130,  6, }, // Africa/Nairobi
    {  502, 5271,  159136,  8, }, // Africa/Ndjamena
    {  502, 5354,  159144,  5, }, // Africa/Niamey
    {  502, 6803,  159149,  8, }, // Africa/Nouakchott
    {  502, 6727,  159157,  8, }, // Africa/Ouagadougou
    {  502, 5225,  159165, 10, }, // Africa/Porto-Novo
    {  502, 4435,  159175,  7, }, // Africa/Sao_Tome
    {  502, 2866,  159182,  8, }, // Africa/Tripoli
    {  502, 6554,  159190,  5, }, // Africa/Tunis
    {  502, 4152,  159195,  8, }, // Africa/Windhoek
    {  502,  132,  159203,  3, }, // America/Adak
    {  502, 3346,  159206,  6, }, // America/Anchorage
    {  502, 5607,  159212,  7, }, // America/Anguilla
    {  502, 5624,  159219,  7, }, // America/Antigua
    {  502, 4554,  159226,  8, }, // America/Araguaina
    {  502,  166,  159234, 10, }, // America/Argentina/Buenos_Aires
    {  502,   91,  159244,  9, }, // America/Argentina/Catamarca
    {  502,  270,  159253,  7, }, // America/Argentina/Cordoba
    {  502,  441,  159260,  5, }, // America/Argentina/Jujuy
    {  502, 7462,  159265,  8, }, // America/Argentina/La_Rioja
    {  502,  565,  159273,  8, }, // America/Argentina/Mendoza
    {  502, 7489,  159281, 11, }, // America/Argentina/Rio_Gallegos
    {  502, 7520,  159292,  5, }, // America/Argentina/Salta
    {  502, 7544,  159297,  8, }, // America/Argentina/San_Juan
    {  502, 5580,  159305,  8, }, // America/Argentina/San_Luis
    {  502, 7571,  159313,  6, }, // America/Argentina/Tucuman
    {  502, 7597,  159319,  7, }, // America/Argentina/Ushuaia
    {  502, 5640,  159326,  5, }, // America/Aruba
    {  502, 4252,  159331,  8, }, // America/Asuncion
    {  502,  237,  159339,  7, }, // America/Atikokan
    {  502, 3679,  159346,  5, }, // America/Bahia
    {  502, 7623,  159351, 13, }, // America/Bahia_Banderas
    {  502, 5654,  159364,  8, }, // America/Barbados
    {  502, 7646,  159372,  5, }, // America/Belem
    {  502, 5407,  159377,  6, }, // America/Belize
    {  502, 7660,  159383, 12, }, // America/Blanc-Sablon
    {  502, 7681,  159395,  9, }, // America/Boa_Vista
    {  502, 4361,  159404,  6, }, // America/Bogota
    {  502, 7699,  159410,  4, }, // America/Boise
    {  502, 7713,  159414, 11, }, // America/Cambridge_Bay
    {  502, 7735,  159425, 13, }, // America/Campo_Grande
    {  502, 3868,  159438,  6, }, // America/Cancun
    {  502, 4694,  159444,  7, }, // America/Caracas
    {  502, 4345,  159451,  6, }, // America/Cayenne
    {  502, 5496,  159457,  5, }, // America/Cayman
    {  502, 2260,  159462,  7, }, // America/Chicago
    {  502, 7756,  159469,  8, }, // America/Chihuahua
    {  502, 7774,  159477, 14, }, // America/Ciudad_Juarez
    {  502, 5422,  159491, 10, }, // America/Costa_Rica
    {  502, 7796,  159501,  7, }, // America/Creston
    {  502, 3791,  159508,  6, }, // America/Cuiaba
    {  502, 5723,  159514,  7, }, // America/Curacao
    {  502, 7812,  159521, 11, }, // America/Danmarkshavn
    {  502, 7833,  159532,  5, }, // America/Dawson
    {  502, 7848,  159537, 11, }, // America/Dawson_Creek
    {  502,  805,  159548,  5, }, // America/Denver
    {  502, 3465,  159553,  7, }, // America/Detroit
    {  502, 5739,  159560,  8, }, // America/Dominica
    {  502,  893,  159568,  7, }, // America/Edmonton
    {  502, 7869,  159575,  8, }, // America/Eirunepe
    {  502, 5441,  159583, 10, }, // America/El_Salvador
    {  502, 7886,  159593, 10, }, // America/Fort_Nelson
    {  502, 7906,  159603, 10, }, // America/Fortaleza
    {  502, 7924,  159613,  7, }, // America/Glace_Bay
    {  502, 6881,  159620,  7, }, // America/Goose_Bay
    {  502, 4612,  159627, 10, }, // America/Grand_Turk
    {  502, 5770,  159637,  8, }, // America/Grenada
    {  502, 5786,  159645,  8, }, // America/Guadeloupe
    {  502, 3760,  159653,  9, }, // America/Guatemala
    {  502, 4841,  159662,  8, }, // America/Guayaquil
    {  502, 6932,  159670,  5, }, // America/Guyana
    {  502, 1939,  159675,  8, }, // America/Halifax
    {  502, 2281,  159683,  5, }, // America/Havana
    {  502, 7942,  159688,  9, }, // America/Hermosillo
    {  502,  348,  159697, 12, }, // America/Indiana/Indianapolis
    {  502,  481,  159709, 13, }, // America/Indiana/Knox
    {  502, 7961,  159722, 16, }, // America/Indiana/Marengo
    {  502, 7985,  159738, 17, }, // America/Indiana/Petersburg
    {  502, 8012,  159755, 17, }, // America/Indiana/Tell_City
    {  502, 8038,  159772, 13, }, // America/Indiana/Vevay
    {  502, 8060,  159785, 17, }, // America/Indiana/Vincennes
    {  502, 8086,  159802, 15, }, // America/Indiana/Winamac
    {  502, 8110,  159817,  6, }, // America/Inuvik
    {  502,  660,  159823,  7, }, // America/Iqaluit
    {  502, 2799,  159830,  6, }, // America/Jamaica
    {  502, 5380,  159836,  5, }, // America/Juneau
    {  502,  521,  159841,  8, }, // America/Kentucky/Louisville
    {  502, 8125,  159849, 17, }, // America/Kentucky/Monticello
    {  502, 5704,  159866, 11, }, // America/Kralendijk
    {  502, 4376,  159877,  7, }, // America/La_Paz
    {  502, 7169,  159884,  4, }, // America/Lima
    {  502, 3239,  159888,  9, }, // America/Los_Angeles
    {  502, 5932,  159897, 20, }, // America/Lower_Princes
    {  502, 8153,  159917,  5, }, // America/Maceio
    {  502, 8168,  159922,  6, }, // America/Managua
    {  502, 1908,  159928,  4, }, // America/Manaus
    {  502, 5897,  159932,  7, }, // America/Marigot
    {  502, 5805,  159939,  8, }, // America/Martinique
    {  502, 8184,  159947,  8, }, // America/Matamoros
    {  502, 2917,  159955,  8, }, // America/Mazatlan
    {  502, 8202,  159963,  8, }, // America/Menominee
    {  502, 8220,  159971,  6, }, // America/Merida
    {  502, 8235,  159977, 10, }, // America/Metlakatla
    {  502, 2949,  159987, 12, }, // America/Mexico_City
    {  502, 4391,  159999,  6, }, // America/Miquelon
    {  502, 8254,  160005,  6, }, // America/Moncton
    {  502, 8270,  160011,  7, }, // America/Monterrey
    {  502, 4098,  160018, 10, }, // America/Montevideo
    {  502, 5824,  160028,  9, }, // America/Montserrat
    {  502, 5481,  160037,  5, }, // America/Nassau
    {  502, 2379,  160042,  8, }, // America/New_York
    {  502, 8288,  159965,  3, }, // America/Nome
    {  502, 1850,  160050,  7, }, // America/Noronha
    {  502, 8301,  160057, 18, }, // America/North_Dakota/Beulah
    {  502, 8329,  160075, 18, }, // America/North_Dakota/Center
    {  502, 8357,  160093, 21, }, // America/North_Dakota/New_Salem
    {  502,  393,  160114,  3, }, // America/Nuuk
    {  502, 8388,  160117,  7, }, // America/Ojinaga
    {  502, 2356,  160124,  5, }, // America/Panama
    {  502, 6173,  160129,  9, }, // America/Paramaribo
    {  502, 2973,  160138,  6, }, // America/Phoenix
    {  502, 3945,  160144, 13, }, // America/Port-au-Prince
    {  502, 5954,  160157, 12, }, // America/Port_of_Spain
    {  502, 8404,  160169, 12, }, // America/Porto_Velho
    {  502, 5843,  160181, 11, }, // America/Puerto_Rico
    {  502, 4030,  160192, 10, }, // America/Punta_Arenas
    {  502, 8424,  160202, 12, }, // America/Rankin_Inlet
    {  502, 8445,  160214,  6, }, // America/Recife
    {  502, 1995,  160220,  6, }, // America/Regina
    {  502, 8460,  160226,  9, }, // America/Resolute
    {  502,  695,  160235, 11, }, // America/Rio_Branco
    {  502, 8477,  160246,  7, }, // America/Santarem
    {  502, 2201,  160253,  8, }, // America/Santiago
    {  502, 6111,  160261, 13, }, // America/Santo_Domingo
    {  502, 1878,  160274,  8, }, // America/Sao_Paulo
    {  502, 6899,  160282, 13, }, // America/Scoresbysund
    {  502, 8494,  160295,  5, }, // America/Sitka
    {  502, 8508,  160300, 14, }, // America/St_Barthelemy
    {  502, 2061,  160314, 10, }, // America/St_Johns
    {  502, 5863,  160324,  9, }, // America/St_Kitts
    {  502, 5880,  160333, 10, }, // America/St_Lucia
    {  502,  855,  160343,  9, }, // America/St_Thomas
    {  502, 5913,  160352, 12, }, // America/St_Vincent
    {  502, 8530,  160364, 10, }, // America/Swift_Current
    {  502, 5461,  160374, 10, }, // America/Tegucigalpa
    {  502, 5756,  160384,  4, }, // America/Thule
    {  502,  313,  160388,  7, }, // America/Tijuana
    {  502,  608,  160395,  7, }, // America/Toronto
    {  502, 5688,  160402,  7, }, // America/Tortola
    {  502, 2093,  160409,  7, }, // America/Vancouver
    {  502, 2144,  160416,  8, }, // America/Whitehorse
    {  502,  734,  160424,  7, }, // America/Winnipeg
    {  502, 8552,  160431,  5, }, // America/Yakutat
    {  502, 6001,  160298,  4, }, // Antarctica/Casey
    {  502, 6094,  160436,  5, }, // Antarctica/Davis
    {  502, 6133,  160441, 16, }, // Antarctica/DumontDUrville
    {  502, 8568,  160457,  8, }, // Antarctica/Macquarie
    {  502, 7066,  160465,  4, }, // Antarctica/Mawson
    {  502, 7113,  160469,  8, }, // Antarctica/McMurdo
    {  502, 8589,  160477,  4, }, // Antarctica/Palmer
    {  502, 7226,  160481,  6, }, // Antarctica/Rothera
    {  502, 7295,  160487,  5, }, // Antarctica/Syowa
    {  502, 8607,  160492,  4, }, // Antarctica/Troll
    {  502, 7374,  160496,  6, }, // Antarctica/Vostok
    {  502, 1429,  160502, 11, }, // Arctic/Longyearbyen
    {  502, 5570,  160513,  4, }, // Asia/Aden
    {  502, 5395,  160517,  6, }, // Asia/Almaty
    {  502, 3968,  160523,  4, }, // Asia/Amman
    {  502, 5511,  160527,  7, }, // Asia/Anadyr
    {  502, 5523,  160534,  4, }, // Asia/Aqtau
    {  502, 5368,  160538,  6, }, // Asia/Aqtobe
    {  502,  964,  160544,  8, }, // Asia/Ashgabat
    {  502, 8624,  160552,  6, }, // Asia/Atyrau
    {  502, 3607,  160558,  5, }, // Asia/Baghdad
    {  502, 5534,  160563,  6, }, // Asia/Bahrain
    {  502, 3653,  160569,  4, }, // Asia/Baku
    {  502, 4466,  160573,  6, }, // Asia/Bangkok
    {  502, 3571,  160579,  5, }, // Asia/Barnaul
    {  502, 4086,  160584,  5, }, // Asia/Beirut
    {  502, 3778,  160589,  7, }, // Asia/Bishkek
    {  502, 5989,  160596,  6, }, // Asia/Brunei
    {  502, 4601,  160602,  4, }, // Asia/Chita
    {  502, 4511,  160606,  6, }, // Asia/Colombo
    {  502, 4540,  160612,  6, }, // Asia/Damascus
    {  502, 1093,  160618,  4, }, // Asia/Dhaka
    {  502, 6192,  160622,  4, }, // Asia/Dili
    {  502, 3596,  160626,  4, }, // Asia/Dubai
    {  502, 6159,  160630,  8, }, // Asia/Dushanbe
    {  502, 8636,  160638,  9, }, // Asia/Famagusta
    {  502, 8651,  160647,  5, }, // Asia/Gaza
    {  502, 4795,  160652,  6, }, // Asia/Hebron
    {  502, 1259,  160658, 16, }, // Asia/Ho_Chi_Minh
    {  502, 2704,  160674,  9, }, // Asia/Hong_Kong
    {  502, 4771,  160683,  4, }, // Asia/Hovd
    {  502, 4184,  160687,  7, }, // Asia/Irkutsk
    {  502, 7006,  160694,  6, }, // Asia/Jakarta
    {  502, 6992,  160700,  7, }, // Asia/Jayapura
    {  502, 1290,  160707,  7, }, // Asia/Jerusalem
    {  502, 3560,  160714,  5, }, // Asia/Kabul
    {  502, 4316,  160719,  6, }, // Asia/Kamchatka
    {  502, 4239,  160725,  5, }, // Asia/Karachi
    {  502, 1185,  160730,  8, }, // Asia/Kathmandu
    {  502, 8661,  160738,  7, }, // Asia/Khandyga
    {  502,  992,  160745,  7, }, // Asia/Kolkata
    {  502, 4197,  160752,  9, }, // Asia/Krasnoyarsk
    {  502, 4859,  160761, 10, }, // Asia/Kuala_Lumpur
    {  502, 5976,  160771,  6, }, // Asia/Kuching
    {  502, 5547,  160777,  5, }, // Asia/Kuwait
    {  502, 1211,  160782,  4, }, // Asia/Macau
    {  502, 4017,  160786,  6, }, // Asia/Magadan
    {  502, 1349,  160792,  5, }, // Asia/Makassar
    {  502, 7182,  160797,  5, }, // Asia/Manila
    {  502, 6920,  160802,  4, }, // Asia/Muscat
    {  502, 2554,  160806,  7, }, // Asia/Nicosia
    {  502, 8675,  160813, 13, }, // Asia/Novokuznetsk
    {  502, 4135,  160826, 11, }, // Asia/Novosibirsk
    {  502, 4229,  160837,  4, }, // Asia/Omsk
    {  502, 7145,  160841,  3, }, // Asia/Oral
    {  502, 6961,  160844,  9, }, // Asia/Phnom_Penh
    {  502, 8693,  160853,  9, }, // Asia/Pontianak
    {  502, 4214,  160862,  9, }, // Asia/Pyongyang
    {  502, 5559,  160871,  3, }, // Asia/Qatar
    {  502, 8708,  160874,  8, }, // Asia/Qostanay
    {  502, 4269,  160882, 10, }, // Asia/Qyzylorda
    {  502, 3584,  160892,  4, }, // Asia/Riyadh
    {  502, 4408,  160896,  5, }, // Asia/Sakhalin
    {  502, 7245,  160901,  6, }, // Asia/Samarkand
    {  502, 3279,  160907,  4, }, // Asia/Seoul
    {  502, 1053,  160911,  6, }, // Asia/Shanghai
    {  502, 3300,  160917,  8, }, // Asia/Singapore
    {  502, 4297,  160925, 12, }, // Asia/Srednekolymsk
    {  502, 3263,  160937,  4, }, // Asia/Taipei
    {  502, 4781,  160941,  7, }, // Asia/Tashkent
    {  502, 3915,  160948,  7, }, // Asia/Tbilisi
    {  502, 2772,  160955,  6, }, // Asia/Tehran
    {  502, 1317,  160961,  5, }, // Asia/Thimphu
    {  502, 2821,  160966,  5, }, // Asia/Tokyo
    {  502, 4572,  160971,  5, }, // Asia/Tomsk
    {  502, 1021,  160976,  8, }, // Asia/Ulaanbaatar
    {  502, 1159,  160984,  6, }, // Asia/Urumqi
    {  502, 8722,  160990,  8, }, // Asia/Ust-Nera
    {  502, 6977,  160998,  9, }, // Asia/Vientiane
    {  502, 4710,  161007, 10, }, // Asia/Vladivostok
    {  502, 4828,  161017,  6, }, // Asia/Yakutsk
    {  502, 1235,  161023,  5, }, // Asia/Yangon
    {  502, 3883,  161028, 10, }, // Asia/Yekaterinburg
    {  502, 3747,  161038,  6, }, // Asia/Yerevan
    {  502, 3663,  161044,  6, }, // Atlantic/Azores
    {  502, 5671,  161050,  6, }, // Atlantic/Bermuda
    {  502, 6628,  161056,  6, }, // Atlantic/Canary
    {  502, 3727,  161062,  7, }, // Atlantic/Cape_Verde
    {  502, 1395,  161069,  4, }, // Atlantic/Faroe
    {  502, 8736,  161073,  6, }, // Atlantic/Madeira
    {  502, 2748,  161079,  8, }, // Atlantic/Reykjavik
    {  502, 7272,  161087, 12, }, // Atlantic/South_Georgia
    {  502, 6821,  161099, 11, }, // Atlantic/St_Helena
    {  502, 6644,  161110,  6, }, // Atlantic/Stanley
    {  502, 1670,  161116,  6, }, // Australia/Adelaide
    {  502, 1635,  161122,  8, }, // Australia/Brisbane
    {  502, 1799,  161130, 10, }, // Australia/Broken_Hill
    {  502, 1583,  161140,  6, }, // Australia/Darwin
    {  502, 3637,  161146,  5, }, // Australia/Eucla
    {  502, 1516,  161151,  6, }, // Australia/Hobart
    {  502, 8753,  161157,  7, }, // Australia/Lindeman
    {  502, 1547,  161164,  9, }, // Australia/Lord_Howe
    {  502, 1727,  161173,  7, }, // Australia/Melbourne
    {  502, 1762,  161180,  3, }, // Australia/Perth
    {  502, 1463,  161183,  5, }, // Australia/Sydney
    {  502, 6441,  161188,  7, }, // Europe/Amsterdam
    {  502, 6216,  161195,  6, }, // Europe/Andorra
    {  502, 3620,  161201,  9, }, // Europe/Astrakhan
    {  502, 2300,  161210,  4, }, // Europe/Athens
    {  502, 6336,  161214,  8, }, // Europe/Belgrade
    {  502, 4757,  161222,  5, }, // Europe/Berlin
    {  502, 6488,  161227, 11, }, // Europe/Bratislava
    {  502, 2167,  161238,  6, }, // Europe/Brussels
    {  502, 3928,  161244,  8, }, // Europe/Bucharest
    {  502, 3806,  161252,  8, }, // Europe/Budapest
    {  502, 8772,  161260,  8, }, // Europe/Busingen
    {  502, 2583,  161268,  6, }, // Europe/Chisinau
    {  502, 6289,  161274,  8, }, // Europe/Copenhagen
    {  502, 2338,  161282,  5, }, // Europe/Dublin
    {  502, 6307,  161287,  8, }, // Europe/Gibraltar
    {  502, 8788,  161295,  5, }, // Europe/Guernsey
    {  502, 6612,  161300,  8, }, // Europe/Helsinki
    {  502, 8804,  161308, 10, }, // Europe/Isle_of_Man
    {  502, 1130,  161318,  8, }, // Europe/Istanbul
    {  502, 8823,  161326,  4, }, // Europe/Jersey
    {  502, 3979,  161330, 11, }, // Europe/Kaliningrad
    {  502, 8929,  161341,  5, }, // Europe/Kirov
    {  502, 2527,  161346,  3, }, // Europe/Kyiv
    {  502, 3213,  161349,  5, }, // Europe/Lisbon
    {  502, 6506,  161354,  9, }, // Europe/Ljubljana
    {  502, 2501,  161363,  4, }, // Europe/London
    {  502, 6365,  161367,  8, }, // Europe/Luxembourg
    {  502, 4892,  161375,  7, }, // Europe/Madrid
    {  502, 6397,  161382,  5, }, // Europe/Malta
    {  502, 6582,  161387, 10, }, // Europe/Mariehamn
    {  502, 3693,  161397,  5, }, // Europe/Minsk
    {  502, 6410,  161402,  6, }, // Europe/Monaco
    {  502, 3537,  161408,  5, }, // Europe/Moscow
    {  502, 6458,  161413,  4, }, // Europe/Oslo
    {  502, 4284,  161417,  5, }, // Europe/Paris
    {  502, 6424,  161422,  9, }, // Europe/Podgorica
    {  502, 6275,  161431,  5, }, // Europe/Prague
    {  502, 8837,  161436,  4, }, // Europe/Riga
    {  502, 6324,  161071,  3, }, // Europe/Rome
    {  502, 4331,  158880,  5, }, // Europe/Samara
    {  502, 6470,  161440,  9, }, // Europe/San_Marino
    {  502, 6245,  161449,  8, }, // Europe/Sarajevo
    {  502, 4451,  161457,  7, }, // Europe/Saratov
    {  502, 8849,  161464,  9, }, // Europe/Simferopol
    {  502, 6383,  161473,  6, }, // Europe/Skopje
    {  502, 6599,  161479,  5, }, // Europe/Sofia
    {  502, 6523,  161484,  7, }, // Europe/Stockholm
    {  502, 8867,  161491,  5, }, // Europe/Tallinn
    {  502, 6202,  161496,  6, }, // Europe/Tirane
    {  502, 8882,  161502,  9, }, // Europe/Ulyanovsk
    {  502, 6352,  161511,  6, }, // Europe/Vaduz
    {  502, 6567,  161517,  6, }, // Europe/Vatican
    {  502, 6231,  161523,  5, }, // Europe/Vienna
    {  502, 8899,  161528,  7, }, // Europe/Vilnius
    {  502, 4727,  161535, 10, }, // Europe/Volgograd
    {  502, 3190,  161545,  6, }, // Europe/Warsaw
    {  502, 6261,  161551,  6, }, // Europe/Zagreb
    {  502, 6540,  161557,  6, }, // Europe/Zurich
    {  502, 5078,  161563, 11, }, // Indian/Antananarivo
    {  502, 6947,  161574,  5, }, // Indian/Chagos
    {  502, 6046,  161579,  7, }, // Indian/Christmas
    {  502, 6063,  161586,  6, }, // Indian/Cocos
    {  502, 5029,  161592,  6, }, // Indian/Comoro
    {  502, 6661,  161598,  8, }, // Indian/Kerguelen
    {  502, 7260,  161606,  4, }, // Indian/Mahe
    {  502, 7050,  161610,  6, }, // Indian/Maldives
    {  502, 4069,  161616,  7, }, // Indian/Mauritius
    {  502, 5098,  161623,  6, }, // Indian/Mayotte
    {  502, 7211,  161629,  8, }, // Indian/Reunion
    {  502, 4422,  161637,  4, }, // Pacific/Apia
    {  502,  932,  161641,  6, }, // Pacific/Auckland
    {  502, 3706,  161647,  8, }, // Pacific/Bougainville
    {  502, 3015,  161655,  4, }, // Pacific/Chatham
    {  502, 3157,  161659,  3, }, // Pacific/Chuuk
    {  502, 2237,  161662,  4, }, // Pacific/Easter
    {  502, 7360,  161666,  4, }, // Pacific/Efate
    {  502, 7327,  161670,  6, }, // Pacific/Fakaofo
    {  502, 3902,  161676,  5, }, // Pacific/Fiji
    {  502, 7343,  161681,  8, }, // Pacific/Funafuti
    {  502, 6678,  161689,  7, }, // Pacific/Galapagos
    {  502, 6696,  161696,  7, }, // Pacific/Gambier
    {  502, 3822,  161703,  9, }, // Pacific/Guadalcanal
    {  502, 6018,  159925,  4, }, // Pacific/Guam
    {  502, 2723,  161712,  8, }, // Pacific/Honolulu
    {  502, 3049,  161720,  7, }, // Pacific/Kanton
    {  502, 3998,  161727,  9, }, // Pacific/Kiritimati
    {  502, 7019,  161736,  7, }, // Pacific/Kosrae
    {  502, 2842,  161743,  8, }, // Pacific/Kwajalein
    {  502, 4877,  161751,  6, }, // Pacific/Majuro
    {  502, 4051,  161757,  8, }, // Pacific/Marquesas
    {  502, 8914,  161765,  5, }, // Pacific/Midway
    {  502, 7084,  161770,  5, }, // Pacific/Nauru
    {  502, 7132,  161775,  4, }, // Pacific/Niue
    {  502, 4168,  161779,  6, }, // Pacific/Norfolk
    {  502, 7098,  161785,  5, }, // Pacific/Noumea
    {  502, 3126,  161790,  9, }, // Pacific/Pago_Pago
    {  502, 7155,  161799,  4, }, // Pacific/Palau
    {  502, 7194,  161803,  7, }, // Pacific/Pitcairn
    {  502, 3096,  161810,  7, }, // Pacific/Pohnpei
    {  502, 4807,  161817, 12, }, // Pacific/Port_Moresby
    {  502, 6076,  161829,  9, }, // Pacific/Rarotonga
    {  502, 6031,  161838,  5, }, // Pacific/Saipan
    {  502, 7312,  161843,  5, }, // Pacific/Tahiti
    {  502, 6712,  161848,  6, }, // Pacific/Tarawa
    {  502, 4583,  161854,  9, }, // Pacific/Tongatapu
    {  502, 7392,  161863,  3, }, // Pacific/Wake
    {  502, 7405,  161866,  5, }, // Pacific/Wallis
    {  503,   14,       0,  6, }, // Africa/Asmara Punjabi/Arabic/Pakistan
    {  503, 4435,       6,  8, }, // Africa/Sao_Tome
    {  503, 4252,      14,  8, }, // America/Asuncion
    {  503,  237,      22,  8, }, // America/Atikokan
    {  503, 7623,      30, 17, }, // America/Bahia_Banderas
    {  503, 3868,      47,  6, }, // America/Cancun
    {  503, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  503, 5723,      66,  7, }, // America/Curacao
    {  503,  481,      73, 13, }, // America/Indiana/Knox
    {  503, 7961,      86, 16, }, // America/Indiana/Marengo
    {  503, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  503, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  503, 8038,     139, 14, }, // America/Indiana/Vevay
    {  503, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  503, 8086,     171, 16, }, // America/Indiana/Winamac
    {  503, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  503, 5932,     207, 22, }, // America/Lower_Princes
    {  503, 8220,     229,  6, }, // America/Merida
    {  503, 1850,     235, 19, }, // America/Noronha
    {  503, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  503, 8329,     274, 20, }, // America/North_Dakota/Center
    {  503, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  503,  393,     317,  4, }, // America/Nuuk
    {  503, 6899,     321, 16, }, // America/Scoresbysund
    {  503, 8508,     337, 14, }, // America/St_Barthelemy
    {  503, 2061,     351, 10, }, // America/St_Johns
    {  503, 5863,     361,  9, }, // America/St_Kitts
    {  503, 5880,     370,  9, }, // America/St_Lucia
    {  503,  855,     379, 10, }, // America/St_Thomas
    {  503, 5913,     389, 11, }, // America/St_Vincent
    {  503, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  503, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  503, 1185,     448,  9, }, // Asia/Kathmandu
    {  503,  992,     457,  7, }, // Asia/Kolkata
    {  503, 1211,     464,  5, }, // Asia/Macau
    {  503, 1235,     477,  6, }, // Asia/Yangon
    {  503, 1395,     483,  5, }, // Atlantic/Faroe
    {  503, 6821,     488, 10, }, // Atlantic/St_Helena
    {  503, 2527,     514,  4, }, // Europe/Kyiv
    {  503, 7211,     547,  7, }, // Indian/Reunion
    {  503, 3157,     554,  5, }, // Pacific/Chuuk
    {  503, 3096,     586,  7, }, // Pacific/Pohnpei
    {  504, 6788,  161871,  6, }, // Africa/Abidjan Quechua/Latin/Peru
    {  504, 6760,   74129,  4, }, // Africa/Accra
    {  504, 5059,   20370, 10, }, // Africa/Addis_Ababa
    {  504, 7420,  147692,  5, }, // Africa/Algiers
    {  504,   14,       0,  6, }, // Africa/Asmara
    {  504, 2320,  161877,  8, }, // Africa/Cairo
    {  504, 5043,  147706,  6, }, // Africa/Djibouti
    {  504, 5243,   21372,  5, }, // Africa/Douala
    {  504, 4479,  161885, 13, }, // Africa/Johannesburg
    {  504, 4524,  147712,  6, }, // Africa/Khartoum
    {  504, 5113,   44110, 10, }, // Africa/Mogadishu
    {  504, 4435,       6,  8, }, // Africa/Sao_Tome
    {  504, 6554,  161898,  5, }, // Africa/Tunis
    {  504, 5607,    4775,  7, }, // America/Anguilla
    {  504, 4252,      14,  8, }, // America/Asuncion
    {  504,  237,      22,  8, }, // America/Atikokan
    {  504, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  504, 5407,  147752,  6, }, // America/Belize
    {  504, 3868,   33314,  6, }, // America/Cancun
    {  504, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  504, 5723,  161903,  7, }, // America/Curacao
    {  504, 5770,   20519,  7, }, // America/Grenada
    {  504, 5786,   20526,  9, }, // America/Guadeloupe
    {  504, 2281,   20535,  9, }, // America/Havana
    {  504,  481,      73, 13, }, // America/Indiana/Knox
    {  504, 7961,      86, 16, }, // America/Indiana/Marengo
    {  504, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  504, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  504, 8038,     139, 14, }, // America/Indiana/Vevay
    {  504, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  504, 8086,     171, 16, }, // America/Indiana/Winamac
    {  504, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  504, 5932,     207, 22, }, // America/Lower_Princes
    {  504, 5805,   20554,  9, }, // America/Martinique
    {  504, 8220,    7126,  6, }, // America/Merida
    {  504, 2949,  161910, 16, }, // America/Mexico_City
    {  504, 1850,     235, 19, }, // America/Noronha
    {  504, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  504, 8329,     274, 20, }, // America/North_Dakota/Center
    {  504, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  504,  393,     317,  4, }, // America/Nuuk
    {  504, 3945,  147901, 15, }, // America/Port-au-Prince
    {  504, 5954,  161926, 13, }, // America/Port_of_Spain
    {  504, 6899,     321, 16, }, // America/Scoresbysund
    {  504, 8508,  161939, 13, }, // America/St_Barthelemy
    {  504, 2061,  161952, 21, }, // America/St_Johns
    {  504, 5863,  161973, 13, }, // America/St_Kitts
    {  504, 5880,   95240, 11, }, // America/St_Lucia
    {  504,  855,  161986, 11, }, // America/St_Thomas
    {  504, 5913,   74405, 11, }, // America/St_Vincent
    {  504, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  504, 3968,   20730,  4, }, // Asia/Amman
    {  504, 5523,    5609,  5, }, // Asia/Aqtau
    {  504, 5368,    7606,  6, }, // Asia/Aqtobe
    {  504,  964,  148013,  7, }, // Asia/Ashgabat
    {  504, 3607,    4162,  6, }, // Asia/Baghdad
    {  504, 5534,  148020,  6, }, // Asia/Bahrain
    {  504, 4540,   74438,  7, }, // Asia/Damascus
    {  504, 6159,  148033,  7, }, // Asia/Dushanbe
    {  504, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  504, 7006,  161997,  7, }, // Asia/Jakarta
    {  504, 1185,   20830,  8, }, // Asia/Kathmandu
    {  504,  992,     457,  7, }, // Asia/Kolkata
    {  504, 1211,     464,  5, }, // Asia/Macau
    {  504, 1349,   94386,  7, }, // Asia/Makassar
    {  504, 6920,   20857,  7, }, // Asia/Muscat
    {  504, 4214,  162004,  8, }, // Asia/Pyongyang
    {  504, 5559,   73150,  5, }, // Asia/Qatar
    {  504, 8708,  162012,  8, }, // Asia/Qostanay
    {  504, 4269,   26897,  9, }, // Asia/Qyzylorda
    {  504, 3584,    7787,  4, }, // Asia/Riyadh
    {  504, 7245,   20886, 10, }, // Asia/Samarkand
    {  504, 3279,   20896,  4, }, // Asia/Seoul
    {  504, 1053,  162020,  7, }, // Asia/Shanghai
    {  504, 3300,   20900,  8, }, // Asia/Singapore
    {  504, 4781,   91755,  7, }, // Asia/Tashkent
    {  504, 3915,   78237,  6, }, // Asia/Tbilisi
    {  504, 2772,   58125,  7, }, // Asia/Tehran
    {  504, 2821,    4356,  5, }, // Asia/Tokyo
    {  504, 1021,   26923, 10, }, // Asia/Ulaanbaatar
    {  504, 1235,   20937,  6, }, // Asia/Yangon
    {  504, 3883,   74457, 13, }, // Asia/Yekaterinburg
    {  504, 3747,  148085,  6, }, // Asia/Yerevan
    {  504, 6628,   74497,  8, }, // Atlantic/Canary
    {  504, 3727,   26968, 10, }, // Atlantic/Cape_Verde
    {  504, 1395,     483,  5, }, // Atlantic/Faroe
    {  504, 7272,  162027, 15, }, // Atlantic/South_Georgia
    {  504, 6821,  148118, 11, }, // Atlantic/St_Helena
    {  504, 1670,   21030,  8, }, // Australia/Adelaide
    {  504, 1463,   22739,  6, }, // Australia/Sydney
    {  504, 2167,  162042,  8, }, // Europe/Brussels
    {  504, 3928,   21073,  8, }, // Europe/Bucharest
    {  504, 6289,   21081, 10, }, // Europe/Copenhagen
    {  504, 8804,  148135, 11, }, // Europe/Isle_of_Man
    {  504, 1130,  148146,  8, }, // Europe/Istanbul
    {  504, 3979,   63006, 12, }, // Europe/Kaliningrad
    {  504, 2527,    8148,  4, }, // Europe/Kyiv
    {  504, 3213,   21137,  6, }, // Europe/Lisbon
    {  504, 6506,   21143,  9, }, // Europe/Ljubljana
    {  504, 2501,   21152,  7, }, // Europe/London
    {  504, 6365,   74574, 10, }, // Europe/Luxembourg
    {  504, 3537,   21175,  5, }, // Europe/Moscow
    {  504, 6275,   21185,  5, }, // Europe/Prague
    {  504, 6324,    6064,  4, }, // Europe/Rome
    {  504, 6523,   74615,  9, }, // Europe/Stockholm
    {  504, 6202,   21213,  6, }, // Europe/Tirane
    {  504, 6567,  162050, 11, }, // Europe/Vatican
    {  504, 6231,   21230,  5, }, // Europe/Vienna
    {  504, 4727,   63187, 10, }, // Europe/Volgograd
    {  504, 3190,   21244,  8, }, // Europe/Warsaw
    {  504, 7050,   74646,  8, }, // Indian/Maldives
    {  504, 7211,     547,  7, }, // Indian/Reunion
    {  504, 3157,     554,  5, }, // Pacific/Chuuk
    {  504, 2723,    4611,  8, }, // Pacific/Honolulu
    {  504, 3049,    4619,  9, }, // Pacific/Kanton
    {  504, 3096,     586,  7, }, // Pacific/Pohnpei
    {  507, 5059,    3902, 11, }, // Africa/Addis_Ababa Romanian/Latin/Romania
    {  507, 7420,   44088,  5, }, // Africa/Algiers
    {  507,   14,       0,  6, }, // Africa/Asmara
    {  507, 5113,   44110, 10, }, // Africa/Mogadishu
    {  507, 5271,   44128,  9, }, // Africa/Ndjamena
    {  507, 4435,   58730,  8, }, // Africa/Sao_Tome
    {  507, 4252,      14,  8, }, // America/Asuncion
    {  507,  237,      22,  8, }, // America/Atikokan
    {  507, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  507, 3868,   33314,  6, }, // America/Cancun
    {  507, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  507, 5723,      66,  7, }, // America/Curacao
    {  507, 5786,   93640,  9, }, // America/Guadeloupe
    {  507,  481,      73, 13, }, // America/Indiana/Knox
    {  507, 7961,      86, 16, }, // America/Indiana/Marengo
    {  507, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  507, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  507, 8038,     139, 14, }, // America/Indiana/Vevay
    {  507, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  507, 8086,     171, 16, }, // America/Indiana/Winamac
    {  507, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  507, 5932,     207, 22, }, // America/Lower_Princes
    {  507, 5805,   20554,  9, }, // America/Martinique
    {  507, 8220,    7126,  6, }, // America/Merida
    {  507, 2949,  161910, 16, }, // America/Mexico_City
    {  507, 1850,     235, 19, }, // America/Noronha
    {  507, 8301,  162061, 22, }, // America/North_Dakota/Beulah
    {  507, 8329,  162083, 22, }, // America/North_Dakota/Center
    {  507, 8357,  162105, 25, }, // America/North_Dakota/New_Salem
    {  507,  393,     317,  4, }, // America/Nuuk
    {  507, 6899,     321, 16, }, // America/Scoresbysund
    {  507, 8508,   35936, 16, }, // America/St_Barthelemy
    {  507, 2061,     351, 10, }, // America/St_Johns
    {  507, 5863,     361,  9, }, // America/St_Kitts
    {  507, 5880,     370,  9, }, // America/St_Lucia
    {  507,  855,     379, 10, }, // America/St_Thomas
    {  507, 5913,     389, 11, }, // America/St_Vincent
    {  507, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  507, 7295,   62201,  5, }, // Antarctica/Syowa
    {  507, 5395,  117529,  6, }, // Asia/Almaty
    {  507, 5511,    7600,  6, }, // Asia/Anadyr
    {  507, 5523,    5609,  5, }, // Asia/Aqtau
    {  507, 5368,    7606,  6, }, // Asia/Aqtobe
    {  507,  964,  162130,  7, }, // Asia/Ashgabat
    {  507, 8624,  162137,  6, }, // Asia/Atyrau
    {  507, 3607,    4162,  6, }, // Asia/Baghdad
    {  507, 3778,  162143,  6, }, // Asia/Bishkek
    {  507, 4601,   93254,  4, }, // Asia/Chita
    {  507, 4540,   20757,  6, }, // Asia/Damascus
    {  507, 1093,   65717,  5, }, // Asia/Dhaka
    {  507, 6159,  162149,  7, }, // Asia/Dushanbe
    {  507, 1259,  162156,  9, }, // Asia/Ho_Chi_Minh
    {  507, 4184,  162165,  6, }, // Asia/Irkutsk
    {  507, 1290,  162171,  9, }, // Asia/Jerusalem
    {  507, 4316,  162180,  9, }, // Asia/Kamchatka
    {  507, 1185,     448,  9, }, // Asia/Kathmandu
    {  507,  992,   36053,  8, }, // Asia/Kolkata
    {  507, 4197,   26871, 11, }, // Asia/Krasnoyarsk
    {  507, 5547,  162189,  6, }, // Asia/Kuwait
    {  507, 1211,     464,  5, }, // Asia/Macau
    {  507, 8675,  162195, 11, }, // Asia/Novokuznetsk
    {  507, 7145,   58053,  6, }, // Asia/Oral
    {  507, 4214,  162206,  7, }, // Asia/Pyongyang
    {  507, 8708,     469,  8, }, // Asia/Qostanay
    {  507, 4269,   26897,  9, }, // Asia/Qyzylorda
    {  507, 3584,    7787,  4, }, // Asia/Riyadh
    {  507, 4408,   33576,  7, }, // Asia/Sakhalin
    {  507, 4297,    7817, 13, }, // Asia/Srednekolymsk
    {  507, 4781,  162213,  7, }, // Asia/Tashkent
    {  507, 2772,    7842,  7, }, // Asia/Tehran
    {  507, 1021,   26923, 10, }, // Asia/Ulaanbaatar
    {  507, 4828,  162220,  6, }, // Asia/Yakutsk
    {  507, 1235,     477,  6, }, // Asia/Yangon
    {  507, 3883,    7886, 12, }, // Asia/Yekaterinburg
    {  507, 3747,   26947,  6, }, // Asia/Yerevan
    {  507, 3663,    7905,  5, }, // Atlantic/Azores
    {  507, 6628,  162226,  6, }, // Atlantic/Canary
    {  507, 3727,  162232, 11, }, // Atlantic/Cape_Verde
    {  507, 1395,   20992,  5, }, // Atlantic/Faroe
    {  507, 7272,  162243, 14, }, // Atlantic/South_Georgia
    {  507, 6821,  162257,  9, }, // Atlantic/St_Helena
    {  507, 3620,   33644,  8, }, // Europe/Astrakhan
    {  507, 2300,   27009,  5, }, // Europe/Athens
    {  507, 6336,   27015,  7, }, // Europe/Belgrade
    {  507, 2167,   57661,  9, }, // Europe/Brussels
    {  507, 3928,  147359,  9, }, // Europe/Bucharest
    {  507, 3806,  162266,  9, }, // Europe/Budapest
    {  507, 2583,   63785,  8, }, // Europe/Chisinau
    {  507, 6289,  158721,  9, }, // Europe/Copenhagen
    {  507, 8804,  162275, 10, }, // Europe/Isle_of_Man
    {  507, 2527,    8148,  4, }, // Europe/Kyiv
    {  507, 3213,  124207,  8, }, // Europe/Lisbon
    {  507, 2501,   95370,  6, }, // Europe/London
    {  507, 6365,    4513,  9, }, // Europe/Luxembourg
    {  507, 3537,   74590,  7, }, // Europe/Moscow
    {  507, 6424,  162285,  9, }, // Europe/Podgorica
    {  507, 6275,   21185,  5, }, // Europe/Prague
    {  507, 6324,    6064,  4, }, // Europe/Rome
    {  507, 6202,   21213,  6, }, // Europe/Tirane
    {  507, 8882,   27092,  9, }, // Europe/Ulyanovsk
    {  507, 6231,   21230,  5, }, // Europe/Vienna
    {  507, 3190,  162294,  8, }, // Europe/Warsaw
    {  507, 6540,    4577,  6, }, // Europe/Zurich
    {  507, 5029,    4588,  6, }, // Indian/Comoro
    {  507, 7050,    8311,  7, }, // Indian/Maldives
    {  507, 7211,     547,  7, }, // Indian/Reunion
    {  507, 3157,     554,  5, }, // Pacific/Chuuk
    {  507, 2237,  162302, 15, }, // Pacific/Easter
    {  507, 2723,    4611,  8, }, // Pacific/Honolulu
    {  507, 3049,    4619,  9, }, // Pacific/Kanton
    {  507, 4051,  162317,  8, }, // Pacific/Marquesas
    {  507, 7194,  162325, 15, }, // Pacific/Pitcairn
    {  507, 3096,     586,  7, }, // Pacific/Pohnpei
    {  509, 5059,    3902, 11, }, // Africa/Addis_Ababa Romansh/Latin/Switzerland
    {  509, 7420,   58555,  6, }, // Africa/Algiers
    {  509,   14,       0,  6, }, // Africa/Asmara
    {  509, 5130,   78080, 10, }, // Africa/Dar_es_Salaam
    {  509, 5043,   78090,  9, }, // Africa/Djibouti
    {  509, 5166,   63491,  8, }, // Africa/El_Aaiun
    {  509, 4524,   26504,  7, }, // Africa/Khartoum
    {  509, 6869,   20418,  4, }, // Africa/Lome
    {  509, 5113,   78099, 10, }, // Africa/Mogadishu
    {  509, 4435,       6,  8, }, // Africa/Sao_Tome
    {  509, 3346,  162340,  6, }, // America/Anchorage
    {  509, 5607,  162346, 10, }, // America/Anguilla
    {  509, 4554,   26559,  9, }, // America/Araguaina
    {  509,  270,   20479,  7, }, // America/Argentina/Cordoba
    {  509,  441,  162356, 21, }, // America/Argentina/Jujuy
    {  509, 7544,  162377, 20, }, // America/Argentina/San_Juan
    {  509, 7571,   20486,  7, }, // America/Argentina/Tucuman
    {  509, 7597,   64664,  7, }, // America/Argentina/Ushuaia
    {  509, 5640,  162397, 10, }, // America/Aruba
    {  509, 4252,      14,  8, }, // America/Asuncion
    {  509,  237,      22,  8, }, // America/Atikokan
    {  509, 7623,      30, 17, }, // America/Bahia_Banderas
    {  509, 7646,   35896,  5, }, // America/Belem
    {  509, 5407,  162407,  8, }, // America/Belize
    {  509, 3868,      47,  6, }, // America/Cancun
    {  509, 5496,  162415, 13, }, // America/Cayman
    {  509, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  509, 3791,   26574,  6, }, // America/Cuiaba
    {  509, 5723,      66,  7, }, // America/Curacao
    {  509, 7869,   26580,  8, }, // America/Eirunepe
    {  509, 5441,    6806,  8, }, // America/El_Salvador
    {  509, 4612,  162428, 13, }, // America/Grand_Turk
    {  509, 5786,  162441, 11, }, // America/Guadeloupe
    {  509,  481,      73,  4, }, // America/Indiana/Knox
    {  509, 7961,      86,  7, }, // America/Indiana/Marengo
    {  509, 7985,     102, 10, }, // America/Indiana/Petersburg
    {  509, 8012,     121,  9, }, // America/Indiana/Tell_City
    {  509, 8038,     139,  5, }, // America/Indiana/Vevay
    {  509, 8060,     153,  9, }, // America/Indiana/Vincennes
    {  509, 8086,     171,  7, }, // America/Indiana/Winamac
    {  509, 2799,   95125,  8, }, // America/Jamaica
    {  509, 8125,     187, 10, }, // America/Kentucky/Monticello
    {  509, 5932,     207, 22, }, // America/Lower_Princes
    {  509, 2917,   26603,  8, }, // America/Mazatlan
    {  509, 8220,     229,  6, }, // America/Merida
    {  509, 2949,  162452, 15, }, // America/Mexico_City
    {  509, 4391,  162467, 12, }, // America/Miquelon
    {  509, 5824,  162479,  6, }, // America/Montserrat
    {  509, 1850,     235, 19, }, // America/Noronha
    {  509, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  509, 8329,  162485, 22, }, // America/North_Dakota/Center
    {  509, 8357,  162507, 24, }, // America/North_Dakota/New_Salem
    {  509,  393,   35928,  7, }, // America/Nuuk
    {  509, 1878,   26712,  9, }, // America/Sao_Paulo
    {  509, 6899,     321, 16, }, // America/Scoresbysund
    {  509, 8508,     337, 14, }, // America/St_Barthelemy
    {  509, 2061,   26737, 12, }, // America/St_Johns
    {  509, 5863,   26749, 11, }, // America/St_Kitts
    {  509, 5880,   95240, 11, }, // America/St_Lucia
    {  509,  855,   44293, 12, }, // America/St_Thomas
    {  509, 5913,   26783, 13, }, // America/St_Vincent
    {  509, 5688,  162531,  9, }, // America/Tortola
    {  509, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  509, 7113,  162540,  9, }, // Antarctica/McMurdo
    {  509, 7295,   62201,  5, }, // Antarctica/Syowa
    {  509, 5368,   57205,  6, }, // Asia/Aqtobe
    {  509,  964,   44322,  7, }, // Asia/Ashgabat
    {  509, 3607,    4162,  6, }, // Asia/Baghdad
    {  509, 3778,   57220,  8, }, // Asia/Bishkek
    {  509, 5989,  162549, 19, }, // Asia/Brunei
    {  509, 6159,   57228,  9, }, // Asia/Dushanbe
    {  509, 1259,  162568, 20, }, // Asia/Ho_Chi_Minh
    {  509, 2704,    4231,  8, }, // Asia/Hong_Kong
    {  509, 4316,   78179, 11, }, // Asia/Kamchatka
    {  509, 4239,   78190,  9, }, // Asia/Karachi
    {  509, 1185,     448,  9, }, // Asia/Kathmandu
    {  509,  992,     457,  7, }, // Asia/Kolkata
    {  509, 4197,    4288, 11, }, // Asia/Krasnoyarsk
    {  509, 1211,     464,  5, }, // Asia/Macau
    {  509, 1349,   65218,  8, }, // Asia/Makassar
    {  509, 6920,   20857,  7, }, // Asia/Muscat
    {  509, 4135,   78211, 11, }, // Asia/Novosibirsk
    {  509, 3584,   20874,  5, }, // Asia/Riyadh
    {  509, 4408,   58084,  8, }, // Asia/Sakhalin
    {  509, 7245,   20886, 10, }, // Asia/Samarkand
    {  509, 3279,    7798,  4, }, // Asia/Seoul
    {  509, 3300,   20900,  8, }, // Asia/Singapore
    {  509, 4781,   57328,  9, }, // Asia/Tashkent
    {  509, 2772,    7842,  7, }, // Asia/Tehran
    {  509, 2821,    4356,  5, }, // Asia/Tokyo
    {  509, 1159,   26933,  6, }, // Asia/Urumqi
    {  509, 4828,    7873,  7, }, // Asia/Yakutsk
    {  509, 1235,     477,  6, }, // Asia/Yangon
    {  509, 3883,    4380, 13, }, // Asia/Yekaterinburg
    {  509, 3747,    7898,  7, }, // Asia/Yerevan
    {  509, 3663,  162588,  6, }, // Atlantic/Azores
    {  509, 5671,   74483,  8, }, // Atlantic/Bermuda
    {  509, 6628,  162594, 15, }, // Atlantic/Canary
    {  509, 3727,   44489,  8, }, // Atlantic/Cape_Verde
    {  509, 1395,  162609, 12, }, // Atlantic/Faroe
    {  509, 7272,  162621, 15, }, // Atlantic/South_Georgia
    {  509, 6821,  162636, 11, }, // Atlantic/St_Helena
    {  509, 6644,  162647, 12, }, // Atlantic/Stanley
    {  509, 2300,    4438,  5, }, // Europe/Athens
    {  509, 2167,   57661,  9, }, // Europe/Brussels
    {  509, 3928,   21073,  8, }, // Europe/Bucharest
    {  509, 8788,  162659, 16, }, // Europe/Guernsey
    {  509, 8804,  162675,  7, }, // Europe/Isle_of_Man
    {  509, 8823,  162682, 12, }, // Europe/Jersey
    {  509, 2527,     514,  4, }, // Europe/Kyiv
    {  509, 3213,    4499,  8, }, // Europe/Lisbon
    {  509, 2501,   95370,  6, }, // Europe/London
    {  509, 6365,    4513,  9, }, // Europe/Luxembourg
    {  509, 3537,  162694,  6, }, // Europe/Moscow
    {  509, 6324,    6064,  4, }, // Europe/Rome
    {  509, 6202,   21213,  6, }, // Europe/Tirane
    {  509, 3190,   21244,  8, }, // Europe/Warsaw
    {  509, 6540,  162700,  6, }, // Europe/Zurich
    {  509, 6046,  162706, 16, }, // Indian/Christmas
    {  509, 6063,  162722, 11, }, // Indian/Cocos
    {  509, 5029,  162733,  7, }, // Indian/Comoro
    {  509, 7050,   74646,  8, }, // Indian/Maldives
    {  509, 7211,     547,  7, }, // Indian/Reunion
    {  509, 3157,  162740,  4, }, // Pacific/Chuuk
    {  509, 2237,  162744, 14, }, // Pacific/Easter
    {  509, 3902,   78353,  7, }, // Pacific/Fiji
    {  509, 6678,   21277,  9, }, // Pacific/Galapagos
    {  509, 6696,  162758,  7, }, // Pacific/Gambier
    {  509, 3822,  162765,  7, }, // Pacific/Guadalcanal
    {  509, 6018,  162772,  7, }, // Pacific/Guam
    {  509, 7019,  162779,  5, }, // Pacific/Kosrae
    {  509, 7084,  162784,  5, }, // Pacific/Nauru
    {  509, 7132,  162789,  5, }, // Pacific/Niue
    {  509, 4168,  162794,  8, }, // Pacific/Norfolk
    {  509, 7098,    4628,  6, }, // Pacific/Noumea
    {  509, 7155,  162802,  7, }, // Pacific/Palau
    {  509, 3096,  162809,  7, }, // Pacific/Pohnpei
    {  509, 6712,  162816, 12, }, // Pacific/Tarawa
    {  509, 4583,  162828, 10, }, // Pacific/Tongatapu
    {  509, 7405,  162838,  8, }, // Pacific/Wallis
    {  510,   14,       0,  6, }, // Africa/Asmara Rombo/Latin/Tanzania
    {  510, 4435,       6,  8, }, // Africa/Sao_Tome
    {  510, 4252,      14,  8, }, // America/Asuncion
    {  510,  237,      22,  8, }, // America/Atikokan
    {  510, 7623,      30, 17, }, // America/Bahia_Banderas
    {  510, 3868,      47,  6, }, // America/Cancun
    {  510, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  510, 5723,      66,  7, }, // America/Curacao
    {  510,  481,      73, 13, }, // America/Indiana/Knox
    {  510, 7961,      86, 16, }, // America/Indiana/Marengo
    {  510, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  510, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  510, 8038,     139, 14, }, // America/Indiana/Vevay
    {  510, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  510, 8086,     171, 16, }, // America/Indiana/Winamac
    {  510, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  510, 5932,     207, 22, }, // America/Lower_Princes
    {  510, 8220,     229,  6, }, // America/Merida
    {  510, 1850,     235, 19, }, // America/Noronha
    {  510, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  510, 8329,     274, 20, }, // America/North_Dakota/Center
    {  510, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  510,  393,     317,  4, }, // America/Nuuk
    {  510, 6899,     321, 16, }, // America/Scoresbysund
    {  510, 8508,     337, 14, }, // America/St_Barthelemy
    {  510, 2061,     351, 10, }, // America/St_Johns
    {  510, 5863,     361,  9, }, // America/St_Kitts
    {  510, 5880,     370,  9, }, // America/St_Lucia
    {  510,  855,     379, 10, }, // America/St_Thomas
    {  510, 5913,     389, 11, }, // America/St_Vincent
    {  510, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  510, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  510, 1185,     448,  9, }, // Asia/Kathmandu
    {  510,  992,     457,  7, }, // Asia/Kolkata
    {  510, 1211,     464,  5, }, // Asia/Macau
    {  510, 1235,     477,  6, }, // Asia/Yangon
    {  510, 1395,     483,  5, }, // Atlantic/Faroe
    {  510, 6821,     488, 10, }, // Atlantic/St_Helena
    {  510, 2527,     514,  4, }, // Europe/Kyiv
    {  510, 7211,     547,  7, }, // Indian/Reunion
    {  510, 3157,     554,  5, }, // Pacific/Chuuk
    {  510, 3096,     586,  7, }, // Pacific/Pohnpei
    {  511,   14,       0,  6, }, // Africa/Asmara Rundi/Latin/Burundi
    {  511, 4435,       6,  8, }, // Africa/Sao_Tome
    {  511, 4252,      14,  8, }, // America/Asuncion
    {  511,  237,      22,  8, }, // America/Atikokan
    {  511, 7623,      30, 17, }, // America/Bahia_Banderas
    {  511, 3868,      47,  6, }, // America/Cancun
    {  511, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  511, 5723,      66,  7, }, // America/Curacao
    {  511,  481,      73, 13, }, // America/Indiana/Knox
    {  511, 7961,      86, 16, }, // America/Indiana/Marengo
    {  511, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  511, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  511, 8038,     139, 14, }, // America/Indiana/Vevay
    {  511, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  511, 8086,     171, 16, }, // America/Indiana/Winamac
    {  511, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  511, 5932,     207, 22, }, // America/Lower_Princes
    {  511, 8220,     229,  6, }, // America/Merida
    {  511, 1850,     235, 19, }, // America/Noronha
    {  511, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  511, 8329,     274, 20, }, // America/North_Dakota/Center
    {  511, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  511,  393,     317,  4, }, // America/Nuuk
    {  511, 6899,     321, 16, }, // America/Scoresbysund
    {  511, 8508,     337, 14, }, // America/St_Barthelemy
    {  511, 2061,     351, 10, }, // America/St_Johns
    {  511, 5863,     361,  9, }, // America/St_Kitts
    {  511, 5880,     370,  9, }, // America/St_Lucia
    {  511,  855,     379, 10, }, // America/St_Thomas
    {  511, 5913,     389, 11, }, // America/St_Vincent
    {  511, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  511, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  511, 1185,     448,  9, }, // Asia/Kathmandu
    {  511,  992,     457,  7, }, // Asia/Kolkata
    {  511, 1211,     464,  5, }, // Asia/Macau
    {  511, 1235,     477,  6, }, // Asia/Yangon
    {  511, 1395,     483,  5, }, // Atlantic/Faroe
    {  511, 6821,     488, 10, }, // Atlantic/St_Helena
    {  511, 2527,     514,  4, }, // Europe/Kyiv
    {  511, 7211,     547,  7, }, // Indian/Reunion
    {  511, 3157,     554,  5, }, // Pacific/Chuuk
    {  511, 3096,     586,  7, }, // Pacific/Pohnpei
    {  512, 6788,   36346,  7, }, // Africa/Abidjan Russian/Cyrillic/Russia
    {  512, 6760,     610,  5, }, // Africa/Accra
    {  512, 5059,     615, 11, }, // Africa/Addis_Ababa
    {  512, 7420,     626,  5, }, // Africa/Algiers
    {  512,   14,   56236,  6, }, // Africa/Asmara
    {  512,   44,     637,  6, }, // Africa/Bamako
    {  512, 5257,     643,  5, }, // Africa/Bangui
    {  512, 6746,     648,  6, }, // Africa/Banjul
    {  512, 7435,     654,  5, }, // Africa/Bissau
    {  512, 4957,   27220,  8, }, // Africa/Blantyre
    {  512, 5287,   51554, 10, }, // Africa/Brazzaville
    {  512, 4922,     676,  9, }, // Africa/Bujumbura
    {  512, 2320,     685,  4, }, // Africa/Cairo
    {  512, 4117,     689, 10, }, // Africa/Casablanca
    {  512, 7449,     699,  5, }, // Africa/Ceuta
    {  512, 6773,     704,  7, }, // Africa/Conakry
    {  512, 6840,     711,  5, }, // Africa/Dakar
    {  512, 5130,   27248, 12, }, // Africa/Dar_es_Salaam
    {  512, 5043,   36379,  7, }, // Africa/Djibouti
    {  512, 5243,     734,  5, }, // Africa/Douala
    {  512, 5166,   27267,  7, }, // Africa/El_Aaiun
    {  512, 6853,     746,  7, }, // Africa/Freetown
    {  512, 4906,     753,  8, }, // Africa/Gaborone
    {  512, 5015,   33913,  6, }, // Africa/Harare
    {  512, 4479,   51572, 12, }, // Africa/Johannesburg
    {  512, 4499,   27305,  5, }, // Africa/Juba
    {  512, 5151,     783,  7, }, // Africa/Kampala
    {  512, 4524,     790,  6, }, // Africa/Khartoum
    {  512, 4973,     796,  6, }, // Africa/Kigali
    {  512, 5306,     802,  7, }, // Africa/Kinshasa
    {  512, 4744,     809,  5, }, // Africa/Lagos
    {  512, 5336,   51584,  9, }, // Africa/Libreville
    {  512, 6869,     822,  4, }, // Africa/Lome
    {  512, 5211,     826,  6, }, // Africa/Luanda
    {  512, 4939,     832,  9, }, // Africa/Lubumbashi
    {  512, 5001,     841,  6, }, // Africa/Lusaka
    {  512, 5322,     847,  6, }, // Africa/Malabo
    {  512, 4987,     853,  6, }, // Africa/Maputo
    {  512, 5197,     859,  6, }, // Africa/Maseru
    {  512, 5182,     865,  7, }, // Africa/Mbabane
    {  512, 5113,     872,  8, }, // Africa/Mogadishu
    {  512, 7034,   36419,  8, }, // Africa/Monrovia
    {  512, 3853,   36427,  7, }, // Africa/Nairobi
    {  512, 5271,   27386,  8, }, // Africa/Ndjamena
    {  512, 5354,   36434,  6, }, // Africa/Niamey
    {  512, 6803,     908,  7, }, // Africa/Nouakchott
    {  512, 6727,     915,  8, }, // Africa/Ouagadougou
    {  512, 5225,     923, 10, }, // Africa/Porto-Novo
    {  512, 4435,     933,  8, }, // Africa/Sao_Tome
    {  512, 2866,     941,  7, }, // Africa/Tripoli
    {  512, 6554,     948,  5, }, // Africa/Tunis
    {  512, 4152,     953,  7, }, // Africa/Windhoek
    {  512,  132,     960,  4, }, // America/Adak
    {  512, 3346,   51593,  8, }, // America/Anchorage
    {  512, 5607,   51601,  7, }, // America/Anguilla
    {  512, 5624,     978,  7, }, // America/Antigua
    {  512, 4554,     985,  9, }, // America/Araguaina
    {  512,  166,   51608, 12, }, // America/Argentina/Buenos_Aires
    {  512,   91,    1006,  9, }, // America/Argentina/Catamarca
    {  512,  270,    1015,  7, }, // America/Argentina/Cordoba
    {  512,  441,   27487,  5, }, // America/Argentina/Jujuy
    {  512, 7462,    1027,  8, }, // America/Argentina/La_Rioja
    {  512,  565,    1035,  7, }, // America/Argentina/Mendoza
    {  512, 7489,   51620, 12, }, // America/Argentina/Rio_Gallegos
    {  512, 7520,   27512,  6, }, // America/Argentina/Salta
    {  512, 7544,    1058,  8, }, // America/Argentina/San_Juan
    {  512, 5580,    1066,  8, }, // America/Argentina/San_Luis
    {  512, 7571,    1074,  7, }, // America/Argentina/Tucuman
    {  512, 7597,   27526,  5, }, // America/Argentina/Ushuaia
    {  512, 5640,    1087,  5, }, // America/Aruba
    {  512, 4252,   51632,  8, }, // America/Asuncion
    {  512,  237,    1099, 12, }, // America/Atikokan
    {  512, 3679,   36500,  4, }, // America/Bahia
    {  512, 7623,   51640, 16, }, // America/Bahia_Banderas
    {  512, 5654,    1131,  8, }, // America/Barbados
    {  512, 7646,    1139,  5, }, // America/Belem
    {  512, 5407,    1144,  5, }, // America/Belize
    {  512, 7660,    1149, 12, }, // America/Blanc-Sablon
    {  512, 7681,    1161,  9, }, // America/Boa_Vista
    {  512, 4361,    1170,  6, }, // America/Bogota
    {  512, 7699,   51656,  5, }, // America/Boise
    {  512, 7713,   51661, 13, }, // America/Cambridge_Bay
    {  512, 7735,    1193, 12, }, // America/Campo_Grande
    {  512, 3868,    1205,  6, }, // America/Cancun
    {  512, 4694,    1211,  7, }, // America/Caracas
    {  512, 4345,   51674,  7, }, // America/Cayenne
    {  512, 5496,  162846, 14, }, // America/Cayman
    {  512, 2260,    1247,  6, }, // America/Chicago
    {  512, 7756,    1253,  6, }, // America/Chihuahua
    {  512, 7774,   56292, 13, }, // America/Ciudad_Juarez
    {  512, 5422,    1259, 10, }, // America/Costa_Rica
    {  512, 7796,    1269,  7, }, // America/Creston
    {  512, 3791,   27688,  5, }, // America/Cuiaba
    {  512, 5723,   36606,  7, }, // America/Curacao
    {  512, 7812,    1290, 12, }, // America/Danmarkshavn
    {  512, 7833,    1302,  6, }, // America/Dawson
    {  512, 7848,    1308, 11, }, // America/Dawson_Creek
    {  512,  805,    1319,  6, }, // America/Denver
    {  512, 3465,   36649,  7, }, // America/Detroit
    {  512, 5739,    1332,  8, }, // America/Dominica
    {  512,  893,   51681,  8, }, // America/Edmonton
    {  512, 7869,   51689,  8, }, // America/Eirunepe
    {  512, 5441,   27765,  9, }, // America/El_Salvador
    {  512, 7886,   51697, 12, }, // America/Fort_Nelson
    {  512, 7906,    1375,  9, }, // America/Fortaleza
    {  512, 7924,   51709,  9, }, // America/Glace_Bay
    {  512, 6881,   51718,  7, }, // America/Goose_Bay
    {  512, 4612,    1400, 10, }, // America/Grand_Turk
    {  512, 5770,    1410,  7, }, // America/Grenada
    {  512, 5786,    1417,  9, }, // America/Guadeloupe
    {  512, 3760,    1426,  9, }, // America/Guatemala
    {  512, 4841,   51725,  8, }, // America/Guayaquil
    {  512, 6932,   51733,  6, }, // America/Guyana
    {  512, 1939,   51739,  8, }, // America/Halifax
    {  512, 2281,    1457,  6, }, // America/Havana
    {  512, 7942,   51747,  9, }, // America/Hermosillo
    {  512,  348,    1471, 12, }, // America/Indiana/Indianapolis
    {  512,  481,    1483, 13, }, // America/Indiana/Knox
    {  512, 7961,    1496, 16, }, // America/Indiana/Marengo
    {  512, 7985,    1512, 19, }, // America/Indiana/Petersburg
    {  512, 8012,    1531,  9, }, // America/Indiana/Tell_City
    {  512, 8038,   56323, 14, }, // America/Indiana/Vevay
    {  512, 8060,    1554,  9, }, // America/Indiana/Vincennes
    {  512, 8086,    1563,  7, }, // America/Indiana/Winamac
    {  512, 8110,    1570,  6, }, // America/Inuvik
    {  512,  660,    1576,  7, }, // America/Iqaluit
    {  512, 2799,   28011,  6, }, // America/Jamaica
    {  512, 5380,   36746,  5, }, // America/Juneau
    {  512,  521,    1594,  8, }, // America/Kentucky/Louisville
    {  512, 8125,    1602, 20, }, // America/Kentucky/Monticello
    {  512, 5704,   36751, 10, }, // America/Kralendijk
    {  512, 4376,    1632,  6, }, // America/La_Paz
    {  512, 7169,    1638,  4, }, // America/Lima
    {  512, 3239,   51761, 12, }, // America/Los_Angeles
    {  512, 5932,   56371, 18, }, // America/Lower_Princes
    {  512, 8153,   36799,  6, }, // America/Maceio
    {  512, 8168,    1677,  7, }, // America/Managua
    {  512, 1908,    1684,  6, }, // America/Manaus
    {  512, 5897,    1690,  6, }, // America/Marigot
    {  512, 5805,    1696,  9, }, // America/Martinique
    {  512, 8184,    1705,  9, }, // America/Matamoros
    {  512, 2917,    1714,  8, }, // America/Mazatlan
    {  512, 8202,    1722,  8, }, // America/Menominee
    {  512, 8220,    1730,  6, }, // America/Merida
    {  512, 8235,    1736, 10, }, // America/Metlakatla
    {  512, 2949,    1746,  6, }, // America/Mexico_City
    {  512, 4391,    1752,  7, }, // America/Miquelon
    {  512, 8254,    1759,  7, }, // America/Moncton
    {  512, 8270,   51794,  9, }, // America/Monterrey
    {  512, 4098,    1775, 10, }, // America/Montevideo
    {  512, 5824,    1785, 10, }, // America/Montserrat
    {  512, 5481,    1795,  6, }, // America/Nassau
    {  512, 2379,   51812,  8, }, // America/New_York
    {  512, 8288,    1809,  3, }, // America/Nome
    {  512, 1850,   56389,  7, }, // America/Noronha
    {  512, 8301,  162860, 22, }, // America/North_Dakota/Beulah
    {  512, 8329,  162882, 22, }, // America/North_Dakota/Center
    {  512, 8357,  162904, 27, }, // America/North_Dakota/New_Salem
    {  512,  393,    1890,  4, }, // America/Nuuk
    {  512, 8388,    1894,  7, }, // America/Ojinaga
    {  512, 2356,    1901,  6, }, // America/Panama
    {  512, 6173,    1907, 10, }, // America/Paramaribo
    {  512, 2973,    1917,  6, }, // America/Phoenix
    {  512, 3945,    1923, 12, }, // America/Port-au-Prince
    {  512, 5954,   28321, 13, }, // America/Port_of_Spain
    {  512, 8404,   28334, 11, }, // America/Porto_Velho
    {  512, 5843,   51888, 11, }, // America/Puerto_Rico
    {  512, 4030,    1970, 12, }, // America/Punta_Arenas
    {  512, 8424,    1982, 12, }, // America/Rankin_Inlet
    {  512, 8445,    1994,  6, }, // America/Recife
    {  512, 1995,   51899,  8, }, // America/Regina
    {  512, 8460,   56461,  8, }, // America/Resolute
    {  512,  695,    2015, 10, }, // America/Rio_Branco
    {  512, 8477,    2025,  8, }, // America/Santarem
    {  512, 2201,   51907,  8, }, // America/Santiago
    {  512, 6111,    2041, 13, }, // America/Santo_Domingo
    {  512, 1878,    2054,  9, }, // America/Sao_Paulo
    {  512, 6899,    2063, 11, }, // America/Scoresbysund
    {  512, 8494,    2074,  5, }, // America/Sitka
    {  512, 8508,    2079, 13, }, // America/St_Barthelemy
    {  512, 2061,   28481, 10, }, // America/St_Johns
    {  512, 5863,    2101,  9, }, // America/St_Kitts
    {  512, 5880,   56488, 10, }, // America/St_Lucia
    {  512,  855,    2121, 10, }, // America/St_Thomas
    {  512, 5913,    2131, 12, }, // America/St_Vincent
    {  512, 8530,    2143, 13, }, // America/Swift_Current
    {  512, 5461,   51949, 12, }, // America/Tegucigalpa
    {  512, 5756,    2167,  4, }, // America/Thule
    {  512,  313,    2171,  7, }, // America/Tijuana
    {  512,  608,    2178,  7, }, // America/Toronto
    {  512, 5688,    2185,  7, }, // America/Tortola
    {  512, 2093,    2192,  8, }, // America/Vancouver
    {  512, 2144,   28567,  8, }, // America/Whitehorse
    {  512,  734,    2208,  8, }, // America/Winnipeg
    {  512, 8552,   28582,  6, }, // America/Yakutat
    {  512, 6001,   37021,  5, }, // Antarctica/Casey
    {  512, 6094,   37026,  6, }, // Antarctica/Davis
    {  512, 6133,   51961, 14, }, // Antarctica/DumontDUrville
    {  512, 8568,    2249,  8, }, // Antarctica/Macquarie
    {  512, 7066,    2257,  6, }, // Antarctica/Mawson
    {  512, 7113,    2263,  9, }, // Antarctica/McMurdo
    {  512, 8589,    2272,  6, }, // Antarctica/Palmer
    {  512, 7226,    2278,  6, }, // Antarctica/Rothera
    {  512, 7295,   28642,  4, }, // Antarctica/Syowa
    {  512, 8607,   51975,  6, }, // Antarctica/Troll
    {  512, 7374,   34957,  6, }, // Antarctica/Vostok
    {  512, 1429,   51981,  7, }, // Arctic/Longyearbyen
    {  512, 5570,    2310,  4, }, // Asia/Aden
    {  512, 5395,    2314,  6, }, // Asia/Almaty
    {  512, 3968,    2320,  5, }, // Asia/Amman
    {  512, 5511,   52002,  7, }, // Asia/Anadyr
    {  512, 5523,    2331,  5, }, // Asia/Aqtau
    {  512, 5368,    2336,  6, }, // Asia/Aqtobe
    {  512,  964,    2342,  7, }, // Asia/Ashgabat
    {  512, 8624,    2349,  6, }, // Asia/Atyrau
    {  512, 3607,    2355,  6, }, // Asia/Baghdad
    {  512, 5534,   37080,  7, }, // Asia/Bahrain
    {  512, 3653,    2368,  4, }, // Asia/Baku
    {  512, 4466,    2372,  7, }, // Asia/Bangkok
    {  512, 3571,    2379,  7, }, // Asia/Barnaul
    {  512, 4086,   28695,  6, }, // Asia/Beirut
    {  512, 3778,    2392,  6, }, // Asia/Bishkek
    {  512, 5989,   28707,  6, }, // Asia/Brunei
    {  512, 4601,    2404,  4, }, // Asia/Chita
    {  512, 4511,    2408,  7, }, // Asia/Colombo
    {  512, 4540,    2415,  6, }, // Asia/Damascus
    {  512, 1093,    2421,  5, }, // Asia/Dhaka
    {  512, 6192,    2426,  4, }, // Asia/Dili
    {  512, 3596,   28728,  5, }, // Asia/Dubai
    {  512, 6159,    2435,  7, }, // Asia/Dushanbe
    {  512, 8636,    2442,  9, }, // Asia/Famagusta
    {  512, 8651,    2451,  4, }, // Asia/Gaza
    {  512, 4795,    2455,  6, }, // Asia/Hebron
    {  512, 1259,    2461,  7, }, // Asia/Ho_Chi_Minh
    {  512, 2704,   52034,  7, }, // Asia/Hong_Kong
    {  512, 4771,    2475,  4, }, // Asia/Hovd
    {  512, 4184,    2479,  7, }, // Asia/Irkutsk
    {  512, 7006,   28770,  8, }, // Asia/Jakarta
    {  512, 6992,   28778,  8, }, // Asia/Jayapura
    {  512, 1290,    2501,  9, }, // Asia/Jerusalem
    {  512, 3560,    2510,  5, }, // Asia/Kabul
    {  512, 4316,  162931, 24, }, // Asia/Kamchatka
    {  512, 4239,    2540,  6, }, // Asia/Karachi
    {  512, 1185,    2546,  8, }, // Asia/Kathmandu
    {  512, 8661,    2554,  7, }, // Asia/Khandyga
    {  512,  992,   52082,  9, }, // Asia/Kolkata
    {  512, 4197,   37110, 10, }, // Asia/Krasnoyarsk
    {  512, 4859,    2580, 12, }, // Asia/Kuala_Lumpur
    {  512, 5976,    2592,  6, }, // Asia/Kuching
    {  512, 5547,   28825,  6, }, // Asia/Kuwait
    {  512, 1211,    2604,  5, }, // Asia/Macau
    {  512, 4017,    2609,  7, }, // Asia/Magadan
    {  512, 1349,    2616,  7, }, // Asia/Makassar
    {  512, 7182,    2623,  6, }, // Asia/Manila
    {  512, 6920,    2629,  6, }, // Asia/Muscat
    {  512, 2554,  106464,  7, }, // Asia/Nicosia
    {  512, 8675,    2642, 11, }, // Asia/Novokuznetsk
    {  512, 4135,    2653, 11, }, // Asia/Novosibirsk
    {  512, 4229,    2664,  4, }, // Asia/Omsk
    {  512, 7145,   28871,  7, }, // Asia/Oral
    {  512, 6961,   52091,  8, }, // Asia/Phnom_Penh
    {  512, 8693,    2681,  9, }, // Asia/Pontianak
    {  512, 4214,   28895,  7, }, // Asia/Pyongyang
    {  512, 5559,    2697,  5, }, // Asia/Qatar
    {  512, 8708,   37137,  8, }, // Asia/Qostanay
    {  512, 4269,    2710,  9, }, // Asia/Qyzylorda
    {  512, 3584,   52099,  7, }, // Asia/Riyadh
    {  512, 4408,  162955, 11, }, // Asia/Sakhalin
    {  512, 7245,    2739,  9, }, // Asia/Samarkand
    {  512, 3279,    2748,  4, }, // Asia/Seoul
    {  512, 1053,   28933,  6, }, // Asia/Shanghai
    {  512, 3300,    2758,  8, }, // Asia/Singapore
    {  512, 4297,    2766, 13, }, // Asia/Srednekolymsk
    {  512, 3263,   28961,  6, }, // Asia/Taipei
    {  512, 4781,    2785,  7, }, // Asia/Tashkent
    {  512, 3915,   35206,  7, }, // Asia/Tbilisi
    {  512, 2772,    2796,  7, }, // Asia/Tehran
    {  512, 1317,    2803,  7, }, // Asia/Thimphu
    {  512, 2821,    2810,  5, }, // Asia/Tokyo
    {  512, 4572,    2815,  5, }, // Asia/Tomsk
    {  512, 1021,    2820, 10, }, // Asia/Ulaanbaatar
    {  512, 1159,    2830,  6, }, // Asia/Urumqi
    {  512, 8722,   52125,  9, }, // Asia/Ust-Nera
    {  512, 6977,   52134,  8, }, // Asia/Vientiane
    {  512, 4710,    2851, 11, }, // Asia/Vladivostok
    {  512, 4828,   37183,  6, }, // Asia/Yakutsk
    {  512, 1235,   56541,  5, }, // Asia/Yangon
    {  512, 3883,    2875, 12, }, // Asia/Yekaterinburg
    {  512, 3747,    2887,  6, }, // Asia/Yerevan
    {  512, 3663,  162966, 13, }, // Atlantic/Azores
    {  512, 5671,  162979, 15, }, // Atlantic/Bermuda
    {  512, 6628,  162994, 14, }, // Atlantic/Canary
    {  512, 3727,    2944, 10, }, // Atlantic/Cape_Verde
    {  512, 1395,  163008, 14, }, // Atlantic/Faroe
    {  512, 8736,   37264,  7, }, // Atlantic/Madeira
    {  512, 2748,   52214,  9, }, // Atlantic/Reykjavik
    {  512, 7272,  163022, 13, }, // Atlantic/South_Georgia
    {  512, 6821,  163035, 16, }, // Atlantic/St_Helena
    {  512, 6644,   52256,  6, }, // Atlantic/Stanley
    {  512, 1670,    3035,  8, }, // Australia/Adelaide
    {  512, 1635,    3043,  7, }, // Australia/Brisbane
    {  512, 1799,    3050, 11, }, // Australia/Broken_Hill
    {  512, 1583,    3061,  6, }, // Australia/Darwin
    {  512, 3637,   29240,  4, }, // Australia/Eucla
    {  512, 1516,    3072,  6, }, // Australia/Hobart
    {  512, 8753,    3078,  8, }, // Australia/Lindeman
    {  512, 1547,    3086,  8, }, // Australia/Lord_Howe
    {  512, 1727,   29252,  8, }, // Australia/Melbourne
    {  512, 1762,    3101,  4, }, // Australia/Perth
    {  512, 1463,   52276,  6, }, // Australia/Sydney
    {  512, 6441,    3111,  9, }, // Europe/Amsterdam
    {  512, 6216,    3120,  7, }, // Europe/Andorra
    {  512, 3620,   29281,  9, }, // Europe/Astrakhan
    {  512, 2300,   61051,  5, }, // Europe/Athens
    {  512, 6336,    3139,  7, }, // Europe/Belgrade
    {  512, 4757,    3146,  6, }, // Europe/Berlin
    {  512, 6488,    3152, 10, }, // Europe/Bratislava
    {  512, 2167,   52296,  8, }, // Europe/Brussels
    {  512, 3928,    3170,  8, }, // Europe/Bucharest
    {  512, 3806,    3178,  8, }, // Europe/Budapest
    {  512, 8772,  163051, 25, }, // Europe/Busingen
    {  512, 2583,    3209,  7, }, // Europe/Chisinau
    {  512, 6289,   52323, 10, }, // Europe/Copenhagen
    {  512, 2338,    3226,  6, }, // Europe/Dublin
    {  512, 6307,    3232,  9, }, // Europe/Gibraltar
    {  512, 8788,    3241,  6, }, // Europe/Guernsey
    {  512, 6612,   52333,  9, }, // Europe/Helsinki
    {  512, 8804,  163076,  7, }, // Europe/Isle_of_Man
    {  512, 1130,    3264,  7, }, // Europe/Istanbul
    {  512, 8823,   52351,  6, }, // Europe/Jersey
    {  512, 3979,    3276, 11, }, // Europe/Kaliningrad
    {  512, 8929,    3287,  5, }, // Europe/Kirov
    {  512, 2527,    3292,  4, }, // Europe/Kyiv
    {  512, 3213,    3296,  8, }, // Europe/Lisbon
    {  512, 6506,   29432,  7, }, // Europe/Ljubljana
    {  512, 2501,    3313,  6, }, // Europe/London
    {  512, 6365,   29445, 10, }, // Europe/Luxembourg
    {  512, 4892,    3330,  6, }, // Europe/Madrid
    {  512, 6397,   29461,  6, }, // Europe/Malta
    {  512, 6582,    3341,  9, }, // Europe/Mariehamn
    {  512, 3693,    3350,  5, }, // Europe/Minsk
    {  512, 6410,    3355,  6, }, // Europe/Monaco
    {  512, 3537,    3361,  6, }, // Europe/Moscow
    {  512, 6458,    3367,  4, }, // Europe/Oslo
    {  512, 4284,    3371,  5, }, // Europe/Paris
    {  512, 6424,    3376,  9, }, // Europe/Podgorica
    {  512, 6275,    3385,  5, }, // Europe/Prague
    {  512, 8837,    3390,  4, }, // Europe/Riga
    {  512, 6324,    3394,  3, }, // Europe/Rome
    {  512, 4331,    3397,  6, }, // Europe/Samara
    {  512, 6470,    3403, 10, }, // Europe/San_Marino
    {  512, 6245,    3413,  7, }, // Europe/Sarajevo
    {  512, 4451,    3420,  7, }, // Europe/Saratov
    {  512, 8849,   52357, 11, }, // Europe/Simferopol
    {  512, 6383,   52368,  6, }, // Europe/Skopje
    {  512, 6599,   37418,  5, }, // Europe/Sofia
    {  512, 6523,   52374,  9, }, // Europe/Stockholm
    {  512, 8867,    3455,  6, }, // Europe/Tallinn
    {  512, 6202,    3461,  6, }, // Europe/Tirane
    {  512, 8882,   52383,  9, }, // Europe/Ulyanovsk
    {  512, 6352,    3476,  5, }, // Europe/Vaduz
    {  512, 6567,    3481,  7, }, // Europe/Vatican
    {  512, 6231,    3488,  4, }, // Europe/Vienna
    {  512, 8899,   52392,  7, }, // Europe/Vilnius
    {  512, 4727,    3499,  9, }, // Europe/Volgograd
    {  512, 3190,    3508,  7, }, // Europe/Warsaw
    {  512, 6261,    3515,  6, }, // Europe/Zagreb
    {  512, 6540,   37442,  5, }, // Europe/Zurich
    {  512, 5078,    3527, 12, }, // Indian/Antananarivo
    {  512, 6947,    3539,  5, }, // Indian/Chagos
    {  512, 6046,  163083, 13, }, // Indian/Christmas
    {  512, 6063,  163096, 14, }, // Indian/Cocos
    {  512, 5029,  163110,  6, }, // Indian/Comoro
    {  512, 6661,    3578,  8, }, // Indian/Kerguelen
    {  512, 7260,   29694,  3, }, // Indian/Mahe
    {  512, 7050,  163116,  8, }, // Indian/Maldives
    {  512, 4069,   56708,  8, }, // Indian/Mauritius
    {  512, 5098,   56716,  7, }, // Indian/Mayotte
    {  512, 7211,   52480,  7, }, // Indian/Reunion
    {  512, 4422,    3616,  4, }, // Pacific/Apia
    {  512,  932,    3620,  6, }, // Pacific/Auckland
    {  512, 3706,   52487,  9, }, // Pacific/Bougainville
    {  512, 3015,    3634,  5, }, // Pacific/Chatham
    {  512, 3157,    3639,  4, }, // Pacific/Chuuk
    {  512, 2237,  163124,  9, }, // Pacific/Easter
    {  512, 7360,   52541,  5, }, // Pacific/Efate
    {  512, 7327,    3659,  7, }, // Pacific/Fakaofo
    {  512, 3902,   37539,  5, }, // Pacific/Fiji
    {  512, 7343,    3670,  8, }, // Pacific/Funafuti
    {  512, 6678,  163133, 18, }, // Pacific/Galapagos
    {  512, 6696,  163151, 11, }, // Pacific/Gambier
    {  512, 3822,    3713, 11, }, // Pacific/Guadalcanal
    {  512, 6018,    3724,  4, }, // Pacific/Guam
    {  512, 2723,   52582,  8, }, // Pacific/Honolulu
    {  512, 3049,  163162, 13, }, // Pacific/Kanton
    {  512, 3998,    3734, 10, }, // Pacific/Kiritimati
    {  512, 7019,    3744,  6, }, // Pacific/Kosrae
    {  512, 2842,   29841, 10, }, // Pacific/Kwajalein
    {  512, 4877,   37574,  7, }, // Pacific/Majuro
    {  512, 4051,  163175, 15, }, // Pacific/Marquesas
    {  512, 8914,  163190, 11, }, // Pacific/Midway
    {  512, 7084,    3798,  5, }, // Pacific/Nauru
    {  512, 7132,   52636,  4, }, // Pacific/Niue
    {  512, 4168,    3807,  7, }, // Pacific/Norfolk
    {  512, 7098,    3814,  5, }, // Pacific/Noumea
    {  512, 3126,    3819,  9, }, // Pacific/Pago_Pago
    {  512, 7155,    3828,  5, }, // Pacific/Palau
    {  512, 7194,   56789,  7, }, // Pacific/Pitcairn
    {  512, 3096,    3840,  6, }, // Pacific/Pohnpei
    {  512, 4807,    3846, 11, }, // Pacific/Port_Moresby
    {  512, 6076,    3857,  9, }, // Pacific/Rarotonga
    {  512, 6031,   29935,  6, }, // Pacific/Saipan
    {  512, 7312,    3872,  5, }, // Pacific/Tahiti
    {  512, 6712,    3877,  6, }, // Pacific/Tarawa
    {  512, 4583,    3883,  9, }, // Pacific/Tongatapu
    {  512, 7392,   29955,  4, }, // Pacific/Wake
    {  512, 7405,    3896,  6, }, // Pacific/Wallis
    {  518,   14,       0,  6, }, // Africa/Asmara Rwa/Latin/Tanzania
    {  518, 4435,       6,  8, }, // Africa/Sao_Tome
    {  518, 4252,      14,  8, }, // America/Asuncion
    {  518,  237,      22,  8, }, // America/Atikokan
    {  518, 7623,      30, 17, }, // America/Bahia_Banderas
    {  518, 3868,      47,  6, }, // America/Cancun
    {  518, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  518, 5723,      66,  7, }, // America/Curacao
    {  518,  481,      73, 13, }, // America/Indiana/Knox
    {  518, 7961,      86, 16, }, // America/Indiana/Marengo
    {  518, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  518, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  518, 8038,     139, 14, }, // America/Indiana/Vevay
    {  518, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  518, 8086,     171, 16, }, // America/Indiana/Winamac
    {  518, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  518, 5932,     207, 22, }, // America/Lower_Princes
    {  518, 8220,     229,  6, }, // America/Merida
    {  518, 1850,     235, 19, }, // America/Noronha
    {  518, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  518, 8329,     274, 20, }, // America/North_Dakota/Center
    {  518, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  518,  393,     317,  4, }, // America/Nuuk
    {  518, 6899,     321, 16, }, // America/Scoresbysund
    {  518, 8508,     337, 14, }, // America/St_Barthelemy
    {  518, 2061,     351, 10, }, // America/St_Johns
    {  518, 5863,     361,  9, }, // America/St_Kitts
    {  518, 5880,     370,  9, }, // America/St_Lucia
    {  518,  855,     379, 10, }, // America/St_Thomas
    {  518, 5913,     389, 11, }, // America/St_Vincent
    {  518, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  518, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  518, 1185,     448,  9, }, // Asia/Kathmandu
    {  518,  992,     457,  7, }, // Asia/Kolkata
    {  518, 1211,     464,  5, }, // Asia/Macau
    {  518, 1235,     477,  6, }, // Asia/Yangon
    {  518, 1395,     483,  5, }, // Atlantic/Faroe
    {  518, 6821,     488, 10, }, // Atlantic/St_Helena
    {  518, 2527,     514,  4, }, // Europe/Kyiv
    {  518, 7211,     547,  7, }, // Indian/Reunion
    {  518, 3157,     554,  5, }, // Pacific/Chuuk
    {  518, 3096,     586,  7, }, // Pacific/Pohnpei
    {  519,   14,       0,  6, }, // Africa/Asmara Saho/Latin/Eritrea
    {  519, 4435,       6,  8, }, // Africa/Sao_Tome
    {  519, 4252,      14,  8, }, // America/Asuncion
    {  519,  237,      22,  8, }, // America/Atikokan
    {  519, 7623,      30, 17, }, // America/Bahia_Banderas
    {  519, 3868,      47,  6, }, // America/Cancun
    {  519, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  519, 5723,      66,  7, }, // America/Curacao
    {  519,  481,      73, 13, }, // America/Indiana/Knox
    {  519, 7961,      86, 16, }, // America/Indiana/Marengo
    {  519, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  519, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  519, 8038,     139, 14, }, // America/Indiana/Vevay
    {  519, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  519, 8086,     171, 16, }, // America/Indiana/Winamac
    {  519, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  519, 5932,     207, 22, }, // America/Lower_Princes
    {  519, 8220,     229,  6, }, // America/Merida
    {  519, 1850,     235, 19, }, // America/Noronha
    {  519, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  519, 8329,     274, 20, }, // America/North_Dakota/Center
    {  519, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  519,  393,     317,  4, }, // America/Nuuk
    {  519, 6899,     321, 16, }, // America/Scoresbysund
    {  519, 8508,     337, 14, }, // America/St_Barthelemy
    {  519, 2061,     351, 10, }, // America/St_Johns
    {  519, 5863,     361,  9, }, // America/St_Kitts
    {  519, 5880,     370,  9, }, // America/St_Lucia
    {  519,  855,     379, 10, }, // America/St_Thomas
    {  519, 5913,     389, 11, }, // America/St_Vincent
    {  519, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  519, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  519, 1185,     448,  9, }, // Asia/Kathmandu
    {  519,  992,     457,  7, }, // Asia/Kolkata
    {  519, 1211,     464,  5, }, // Asia/Macau
    {  519, 1235,     477,  6, }, // Asia/Yangon
    {  519, 1395,     483,  5, }, // Atlantic/Faroe
    {  519, 6821,     488, 10, }, // Atlantic/St_Helena
    {  519, 2527,     514,  4, }, // Europe/Kyiv
    {  519, 7211,     547,  7, }, // Indian/Reunion
    {  519, 3157,     554,  5, }, // Pacific/Chuuk
    {  519, 3096,     586,  7, }, // Pacific/Pohnpei
    {  520,   14,       0,  6, }, // Africa/Asmara Sakha/Cyrillic/Russia
    {  520, 4435,       6,  8, }, // Africa/Sao_Tome
    {  520, 4252,      14,  8, }, // America/Asuncion
    {  520,  237,      22,  8, }, // America/Atikokan
    {  520, 7623,      30, 17, }, // America/Bahia_Banderas
    {  520, 3868,      47,  6, }, // America/Cancun
    {  520, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  520, 5723,      66,  7, }, // America/Curacao
    {  520,  481,      73, 13, }, // America/Indiana/Knox
    {  520, 7961,      86, 16, }, // America/Indiana/Marengo
    {  520, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  520, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  520, 8038,     139, 14, }, // America/Indiana/Vevay
    {  520, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  520, 8086,     171, 16, }, // America/Indiana/Winamac
    {  520, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  520, 5932,     207, 22, }, // America/Lower_Princes
    {  520, 8220,     229,  6, }, // America/Merida
    {  520, 1850,     235, 19, }, // America/Noronha
    {  520, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  520, 8329,     274, 20, }, // America/North_Dakota/Center
    {  520, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  520,  393,     317,  4, }, // America/Nuuk
    {  520, 6899,     321, 16, }, // America/Scoresbysund
    {  520, 8508,     337, 14, }, // America/St_Barthelemy
    {  520, 2061,     351, 10, }, // America/St_Johns
    {  520, 5863,     361,  9, }, // America/St_Kitts
    {  520, 5880,     370,  9, }, // America/St_Lucia
    {  520,  855,     379, 10, }, // America/St_Thomas
    {  520, 5913,     389, 11, }, // America/St_Vincent
    {  520, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  520, 5395,    2314,  6, }, // Asia/Almaty
    {  520, 5511,  163201,  7, }, // Asia/Anadyr
    {  520,  964,  163208,  8, }, // Asia/Ashgabat
    {  520, 3607,  163216,  7, }, // Asia/Baghdad
    {  520, 3653,  163223,  5, }, // Asia/Baku
    {  520, 3571,    2379,  7, }, // Asia/Barnaul
    {  520, 4601,  163228,  5, }, // Asia/Chita
    {  520, 4511,    2408,  7, }, // Asia/Colombo
    {  520, 4540,  163233,  6, }, // Asia/Damascus
    {  520, 3596,  163239,  6, }, // Asia/Dubai
    {  520, 1259,  163245, 10, }, // Asia/Ho_Chi_Minh
    {  520, 4184,  163255,  9, }, // Asia/Irkutsk
    {  520, 3560,  163264,  6, }, // Asia/Kabul
    {  520, 4316,  163270,  9, }, // Asia/Kamchatka
    {  520, 1185,     448,  9, }, // Asia/Kathmandu
    {  520, 8661,  163279,  8, }, // Asia/Khandyga
    {  520,  992,     457,  7, }, // Asia/Kolkata
    {  520, 4197,  163287, 12, }, // Asia/Krasnoyarsk
    {  520, 1211,     464,  5, }, // Asia/Macau
    {  520, 4017,  163299,  8, }, // Asia/Magadan
    {  520, 4135,  163307, 13, }, // Asia/Novosibirsk
    {  520, 4229,  163320,  6, }, // Asia/Omsk
    {  520, 4408,  163326,  8, }, // Asia/Sakhalin
    {  520, 7245,  163334,  9, }, // Asia/Samarkand
    {  520, 4297,  163343, 11, }, // Asia/Srednekolymsk
    {  520, 1021,  163354, 11, }, // Asia/Ulaanbaatar
    {  520, 1159,  163365,  6, }, // Asia/Urumqi
    {  520, 8722,  163371,  9, }, // Asia/Ust-Nera
    {  520, 4828,  163380, 10, }, // Asia/Yakutsk
    {  520, 1235,     477,  6, }, // Asia/Yangon
    {  520, 3883,  163390, 12, }, // Asia/Yekaterinburg
    {  520, 1395,     483,  5, }, // Atlantic/Faroe
    {  520, 6821,     488, 10, }, // Atlantic/St_Helena
    {  520, 3620,  163402,  9, }, // Europe/Astrakhan
    {  520, 1130,  163411,  8, }, // Europe/Istanbul
    {  520, 3979,    3276, 11, }, // Europe/Kaliningrad
    {  520, 2527,     514,  4, }, // Europe/Kyiv
    {  520, 3537,  163419,  7, }, // Europe/Moscow
    {  520, 4331,  163426,  7, }, // Europe/Samara
    {  520, 8849,   52357, 11, }, // Europe/Simferopol
    {  520, 8882,  163433, 11, }, // Europe/Ulyanovsk
    {  520, 7211,     547,  7, }, // Indian/Reunion
    {  520, 3157,     554,  5, }, // Pacific/Chuuk
    {  520, 3096,     586,  7, }, // Pacific/Pohnpei
    {  521,   14,       0,  6, }, // Africa/Asmara Samburu/Latin/Kenya
    {  521, 4435,       6,  8, }, // Africa/Sao_Tome
    {  521, 4252,      14,  8, }, // America/Asuncion
    {  521,  237,      22,  8, }, // America/Atikokan
    {  521, 7623,      30, 17, }, // America/Bahia_Banderas
    {  521, 3868,      47,  6, }, // America/Cancun
    {  521, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  521, 5723,      66,  7, }, // America/Curacao
    {  521,  481,      73, 13, }, // America/Indiana/Knox
    {  521, 7961,      86, 16, }, // America/Indiana/Marengo
    {  521, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  521, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  521, 8038,     139, 14, }, // America/Indiana/Vevay
    {  521, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  521, 8086,     171, 16, }, // America/Indiana/Winamac
    {  521, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  521, 5932,     207, 22, }, // America/Lower_Princes
    {  521, 8220,     229,  6, }, // America/Merida
    {  521, 1850,     235, 19, }, // America/Noronha
    {  521, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  521, 8329,     274, 20, }, // America/North_Dakota/Center
    {  521, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  521,  393,     317,  4, }, // America/Nuuk
    {  521, 6899,     321, 16, }, // America/Scoresbysund
    {  521, 8508,     337, 14, }, // America/St_Barthelemy
    {  521, 2061,     351, 10, }, // America/St_Johns
    {  521, 5863,     361,  9, }, // America/St_Kitts
    {  521, 5880,     370,  9, }, // America/St_Lucia
    {  521,  855,     379, 10, }, // America/St_Thomas
    {  521, 5913,     389, 11, }, // America/St_Vincent
    {  521, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  521, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  521, 1185,     448,  9, }, // Asia/Kathmandu
    {  521,  992,     457,  7, }, // Asia/Kolkata
    {  521, 1211,     464,  5, }, // Asia/Macau
    {  521, 1235,     477,  6, }, // Asia/Yangon
    {  521, 1395,     483,  5, }, // Atlantic/Faroe
    {  521, 6821,     488, 10, }, // Atlantic/St_Helena
    {  521, 2527,     514,  4, }, // Europe/Kyiv
    {  521, 7211,     547,  7, }, // Indian/Reunion
    {  521, 3157,     554,  5, }, // Pacific/Chuuk
    {  521, 3096,     586,  7, }, // Pacific/Pohnpei
    {  522,   14,       0,  6, }, // Africa/Asmara Sango/Latin/Central African Republic
    {  522, 4435,       6,  8, }, // Africa/Sao_Tome
    {  522, 4252,      14,  8, }, // America/Asuncion
    {  522,  237,      22,  8, }, // America/Atikokan
    {  522, 7623,      30, 17, }, // America/Bahia_Banderas
    {  522, 3868,      47,  6, }, // America/Cancun
    {  522, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  522, 5723,      66,  7, }, // America/Curacao
    {  522,  481,      73, 13, }, // America/Indiana/Knox
    {  522, 7961,      86, 16, }, // America/Indiana/Marengo
    {  522, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  522, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  522, 8038,     139, 14, }, // America/Indiana/Vevay
    {  522, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  522, 8086,     171, 16, }, // America/Indiana/Winamac
    {  522, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  522, 5932,     207, 22, }, // America/Lower_Princes
    {  522, 8220,     229,  6, }, // America/Merida
    {  522, 1850,     235, 19, }, // America/Noronha
    {  522, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  522, 8329,     274, 20, }, // America/North_Dakota/Center
    {  522, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  522,  393,     317,  4, }, // America/Nuuk
    {  522, 6899,     321, 16, }, // America/Scoresbysund
    {  522, 8508,     337, 14, }, // America/St_Barthelemy
    {  522, 2061,     351, 10, }, // America/St_Johns
    {  522, 5863,     361,  9, }, // America/St_Kitts
    {  522, 5880,     370,  9, }, // America/St_Lucia
    {  522,  855,     379, 10, }, // America/St_Thomas
    {  522, 5913,     389, 11, }, // America/St_Vincent
    {  522, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  522, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  522, 1185,     448,  9, }, // Asia/Kathmandu
    {  522,  992,     457,  7, }, // Asia/Kolkata
    {  522, 1211,     464,  5, }, // Asia/Macau
    {  522, 1235,     477,  6, }, // Asia/Yangon
    {  522, 1395,     483,  5, }, // Atlantic/Faroe
    {  522, 6821,     488, 10, }, // Atlantic/St_Helena
    {  522, 2527,     514,  4, }, // Europe/Kyiv
    {  522, 7211,     547,  7, }, // Indian/Reunion
    {  522, 3157,     554,  5, }, // Pacific/Chuuk
    {  522, 3096,     586,  7, }, // Pacific/Pohnpei
    {  523,   14,       0,  6, }, // Africa/Asmara Sangu/Latin/Tanzania
    {  523, 4435,       6,  8, }, // Africa/Sao_Tome
    {  523, 4252,      14,  8, }, // America/Asuncion
    {  523,  237,      22,  8, }, // America/Atikokan
    {  523, 7623,      30, 17, }, // America/Bahia_Banderas
    {  523, 3868,      47,  6, }, // America/Cancun
    {  523, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  523, 5723,      66,  7, }, // America/Curacao
    {  523,  481,      73, 13, }, // America/Indiana/Knox
    {  523, 7961,      86, 16, }, // America/Indiana/Marengo
    {  523, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  523, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  523, 8038,     139, 14, }, // America/Indiana/Vevay
    {  523, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  523, 8086,     171, 16, }, // America/Indiana/Winamac
    {  523, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  523, 5932,     207, 22, }, // America/Lower_Princes
    {  523, 8220,     229,  6, }, // America/Merida
    {  523, 1850,     235, 19, }, // America/Noronha
    {  523, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  523, 8329,     274, 20, }, // America/North_Dakota/Center
    {  523, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  523,  393,     317,  4, }, // America/Nuuk
    {  523, 6899,     321, 16, }, // America/Scoresbysund
    {  523, 8508,     337, 14, }, // America/St_Barthelemy
    {  523, 2061,     351, 10, }, // America/St_Johns
    {  523, 5863,     361,  9, }, // America/St_Kitts
    {  523, 5880,     370,  9, }, // America/St_Lucia
    {  523,  855,     379, 10, }, // America/St_Thomas
    {  523, 5913,     389, 11, }, // America/St_Vincent
    {  523, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  523, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  523, 1185,     448,  9, }, // Asia/Kathmandu
    {  523,  992,     457,  7, }, // Asia/Kolkata
    {  523, 1211,     464,  5, }, // Asia/Macau
    {  523, 1235,     477,  6, }, // Asia/Yangon
    {  523, 1395,     483,  5, }, // Atlantic/Faroe
    {  523, 6821,     488, 10, }, // Atlantic/St_Helena
    {  523, 2527,     514,  4, }, // Europe/Kyiv
    {  523, 7211,     547,  7, }, // Indian/Reunion
    {  523, 3157,     554,  5, }, // Pacific/Chuuk
    {  523, 3096,     586,  7, }, // Pacific/Pohnpei
    {  524,   14,       0,  6, }, // Africa/Asmara Sanskrit/Devanagari/India
    {  524, 4435,       6,  8, }, // Africa/Sao_Tome
    {  524, 4252,      14,  8, }, // America/Asuncion
    {  524,  237,      22,  8, }, // America/Atikokan
    {  524, 7623,      30, 17, }, // America/Bahia_Banderas
    {  524, 3868,      47,  6, }, // America/Cancun
    {  524, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  524, 5723,      66,  7, }, // America/Curacao
    {  524,  481,      73, 13, }, // America/Indiana/Knox
    {  524, 7961,      86, 16, }, // America/Indiana/Marengo
    {  524, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  524, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  524, 8038,     139, 14, }, // America/Indiana/Vevay
    {  524, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  524, 8086,     171, 16, }, // America/Indiana/Winamac
    {  524, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  524, 5932,     207, 22, }, // America/Lower_Princes
    {  524, 8220,     229,  6, }, // America/Merida
    {  524, 1850,     235, 19, }, // America/Noronha
    {  524, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  524, 8329,     274, 20, }, // America/North_Dakota/Center
    {  524, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  524,  393,     317,  4, }, // America/Nuuk
    {  524, 6899,     321, 16, }, // America/Scoresbysund
    {  524, 8508,     337, 14, }, // America/St_Barthelemy
    {  524, 2061,     351, 10, }, // America/St_Johns
    {  524, 5863,     361,  9, }, // America/St_Kitts
    {  524, 5880,     370,  9, }, // America/St_Lucia
    {  524,  855,     379, 10, }, // America/St_Thomas
    {  524, 5913,     389, 11, }, // America/St_Vincent
    {  524, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  524, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  524, 1185,     448,  9, }, // Asia/Kathmandu
    {  524,  992,     457,  7, }, // Asia/Kolkata
    {  524, 1211,     464,  5, }, // Asia/Macau
    {  524, 1235,     477,  6, }, // Asia/Yangon
    {  524, 1395,     483,  5, }, // Atlantic/Faroe
    {  524, 6821,     488, 10, }, // Atlantic/St_Helena
    {  524, 2527,     514,  4, }, // Europe/Kyiv
    {  524, 7211,     547,  7, }, // Indian/Reunion
    {  524, 3157,     554,  5, }, // Pacific/Chuuk
    {  524, 3096,     586,  7, }, // Pacific/Pohnpei
    {  525, 6788,  163444,  8, }, // Africa/Abidjan Santali/Ol Chiki/India
    {  525,   14,       0,  6, }, // Africa/Asmara
    {  525, 4435,       6,  8, }, // Africa/Sao_Tome
    {  525,  132,  163452,  4, }, // America/Adak
    {  525, 3346,  163456,  9, }, // America/Anchorage
    {  525, 5607,  163465,  7, }, // America/Anguilla
    {  525, 5624,  163472,  8, }, // America/Antigua
    {  525, 5640,  163480,  5, }, // America/Aruba
    {  525, 4252,      14,  8, }, // America/Asuncion
    {  525,  237,  163485,  8, }, // America/Atikokan
    {  525, 7623,  163493, 15, }, // America/Bahia_Banderas
    {  525, 5654,  163508,  8, }, // America/Barbados
    {  525, 5407,  163516,  6, }, // America/Belize
    {  525, 7660,  163522, 12, }, // America/Blanc-Sablon
    {  525, 7699,  163534,  5, }, // America/Boise
    {  525, 7713,  163539, 12, }, // America/Cambridge_Bay
    {  525, 3868,  163551,  6, }, // America/Cancun
    {  525, 5496,  163557,  5, }, // America/Cayman
    {  525, 2260,  163562,  6, }, // America/Chicago
    {  525, 7756,  163568, 11, }, // America/Chihuahua
    {  525, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  525, 5422,  163579, 10, }, // America/Costa_Rica
    {  525, 7796,  163589,  7, }, // America/Creston
    {  525, 5723,  163596,  8, }, // America/Curacao
    {  525, 7812,  163604, 11, }, // America/Danmarkshavn
    {  525, 7833,  163615,  6, }, // America/Dawson
    {  525, 7848,  163621, 11, }, // America/Dawson_Creek
    {  525,  805,  163632,  6, }, // America/Denver
    {  525, 3465,  163638,  8, }, // America/Detroit
    {  525, 5739,  163646,  8, }, // America/Dominica
    {  525,  893,  163654,  8, }, // America/Edmonton
    {  525, 5441,  163662, 11, }, // America/El_Salvador
    {  525, 7886,  163673, 11, }, // America/Fort_Nelson
    {  525, 7924,  163684,  7, }, // America/Glace_Bay
    {  525, 6881,  163691,  6, }, // America/Goose_Bay
    {  525, 4612,  163697, 11, }, // America/Grand_Turk
    {  525, 5770,  163708,  8, }, // America/Grenada
    {  525, 5786,  163716,  9, }, // America/Guadeloupe
    {  525, 3760,  163725, 10, }, // America/Guatemala
    {  525, 1939,  163735,  9, }, // America/Halifax
    {  525, 2281,  163744,  6, }, // America/Havana
    {  525, 7942,  163750, 10, }, // America/Hermosillo
    {  525,  348,  163760, 13, }, // America/Indiana/Indianapolis
    {  525,  481,  163773, 14, }, // America/Indiana/Knox
    {  525, 7961,  163787, 17, }, // America/Indiana/Marengo
    {  525, 7985,  163804, 21, }, // America/Indiana/Petersburg
    {  525, 8012,  163825, 18, }, // America/Indiana/Tell_City
    {  525, 8038,  163843, 17, }, // America/Indiana/Vevay
    {  525, 8060,  163860, 18, }, // America/Indiana/Vincennes
    {  525, 8086,  163878, 17, }, // America/Indiana/Winamac
    {  525, 8110,  163895,  6, }, // America/Inuvik
    {  525,  660,  163901,  8, }, // America/Iqaluit
    {  525, 2799,  163909,  7, }, // America/Jamaica
    {  525, 5380,  163916,  7, }, // America/Juneau
    {  525,  521,  163923,  9, }, // America/Kentucky/Louisville
    {  525, 8125,  163932, 18, }, // America/Kentucky/Monticello
    {  525, 5704,  163950, 12, }, // America/Kralendijk
    {  525, 3239,  163962, 11, }, // America/Los_Angeles
    {  525, 5932,  163973, 20, }, // America/Lower_Princes
    {  525, 8168,  163993,  8, }, // America/Managua
    {  525, 5897,  164001,  7, }, // America/Marigot
    {  525, 5805,  164008,  7, }, // America/Martinique
    {  525, 8184,  164015,  9, }, // America/Matamoros
    {  525, 2917,  164024,  8, }, // America/Mazatlan
    {  525, 8202,  164032,  8, }, // America/Menominee
    {  525, 8220,  164040,  6, }, // America/Merida
    {  525, 8235,  164046, 10, }, // America/Metlakatla
    {  525, 2949,  164056, 13, }, // America/Mexico_City
    {  525, 4391,  164069,  8, }, // America/Miquelon
    {  525, 8254,  164077,  7, }, // America/Moncton
    {  525, 8270,  164084,  8, }, // America/Monterrey
    {  525, 5824,  164092,  9, }, // America/Montserrat
    {  525, 5481,  164101,  5, }, // America/Nassau
    {  525, 2379,  164106,  8, }, // America/New_York
    {  525, 8288,  164114,  4, }, // America/Nome
    {  525, 1850,     235, 19, }, // America/Noronha
    {  525, 8301,  164118, 19, }, // America/North_Dakota/Beulah
    {  525, 8329,  164137, 19, }, // America/North_Dakota/Center
    {  525, 8357,  164156, 22, }, // America/North_Dakota/New_Salem
    {  525,  393,  164178,  3, }, // America/Nuuk
    {  525, 8388,  164181,  7, }, // America/Ojinaga
    {  525, 2356,  164188,  6, }, // America/Panama
    {  525, 2973,  164194,  7, }, // America/Phoenix
    {  525, 3945,  164201, 13, }, // America/Port-au-Prince
    {  525, 5954,  164214, 13, }, // America/Port_of_Spain
    {  525, 5843,  164227, 11, }, // America/Puerto_Rico
    {  525, 8424,  164238, 12, }, // America/Rankin_Inlet
    {  525, 1995,  164250,  8, }, // America/Regina
    {  525, 8460,  164258,  8, }, // America/Resolute
    {  525, 6111,  164266, 13, }, // America/Santo_Domingo
    {  525, 6899,  164279, 11, }, // America/Scoresbysund
    {  525, 8494,  164290,  5, }, // America/Sitka
    {  525, 8508,  164295, 15, }, // America/St_Barthelemy
    {  525, 2061,  164310, 10, }, // America/St_Johns
    {  525, 5863,  164320, 10, }, // America/St_Kitts
    {  525, 5880,  164330, 12, }, // America/St_Lucia
    {  525,  855,  164342, 12, }, // America/St_Thomas
    {  525, 5913,  164354, 12, }, // America/St_Vincent
    {  525, 8530,  164366, 13, }, // America/Swift_Current
    {  525, 5461,  164379, 11, }, // America/Tegucigalpa
    {  525, 5756,  164390,  5, }, // America/Thule
    {  525,  313,  164395,  8, }, // America/Tijuana
    {  525,  608,  164403,  7, }, // America/Toronto
    {  525, 5688,  164410,  7, }, // America/Tortola
    {  525, 2093,  164417,  9, }, // America/Vancouver
    {  525, 2144,  164426,  9, }, // America/Whitehorse
    {  525,  734,  164435,  8, }, // America/Winnipeg
    {  525, 8552,  164443,  7, }, // America/Yakutat
    {  525, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  525, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  525, 1185,     448,  9, }, // Asia/Kathmandu
    {  525,  992,     457,  7, }, // Asia/Kolkata
    {  525, 1211,     464,  5, }, // Asia/Macau
    {  525, 1235,     477,  6, }, // Asia/Yangon
    {  525, 5671,  164450,  9, }, // Atlantic/Bermuda
    {  525, 1395,     483,  5, }, // Atlantic/Faroe
    {  525, 6821,     488, 10, }, // Atlantic/St_Helena
    {  525, 2527,     514,  4, }, // Europe/Kyiv
    {  525, 7211,     547,  7, }, // Indian/Reunion
    {  525, 3157,     554,  5, }, // Pacific/Chuuk
    {  525, 3096,     586,  7, }, // Pacific/Pohnpei
    {  526,  237,      22,  8, }, // America/Atikokan Santali/Devanagari/India
    {  526, 7623,      30, 17, }, // America/Bahia_Banderas
    {  526, 3868,      47,  6, }, // America/Cancun
    {  526, 5723,      66,  7, }, // America/Curacao
    {  526,  481,      73, 13, }, // America/Indiana/Knox
    {  526, 7961,      86, 16, }, // America/Indiana/Marengo
    {  526, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  526, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  526, 8038,     139, 14, }, // America/Indiana/Vevay
    {  526, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  526, 8086,     171, 16, }, // America/Indiana/Winamac
    {  526, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  526, 5932,     207, 22, }, // America/Lower_Princes
    {  526, 8220,     229,  6, }, // America/Merida
    {  526, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  526, 8329,     274, 20, }, // America/North_Dakota/Center
    {  526, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  526,  393,     317,  4, }, // America/Nuuk
    {  526, 6899,     321, 16, }, // America/Scoresbysund
    {  526, 8508,     337, 14, }, // America/St_Barthelemy
    {  526, 2061,     351, 10, }, // America/St_Johns
    {  526, 5863,     361,  9, }, // America/St_Kitts
    {  526, 5880,     370,  9, }, // America/St_Lucia
    {  526,  855,     379, 10, }, // America/St_Thomas
    {  526, 5913,     389, 11, }, // America/St_Vincent
    {  527, 5059,    3902, 11, }, // Africa/Addis_Ababa Sardinian/Latin/Italy
    {  527, 7420,   95064,  6, }, // Africa/Algiers
    {  527,   14,       0,  6, }, // Africa/Asmara
    {  527, 2320,  164459,  8, }, // Africa/Cairo
    {  527, 5113,  164467, 10, }, // Africa/Mogadishu
    {  527, 7034,   44120,  8, }, // Africa/Monrovia
    {  527, 5225,  164477,  9, }, // Africa/Porto-Novo
    {  527, 4435,       6,  8, }, // Africa/Sao_Tome
    {  527, 2866,  164486,  7, }, // Africa/Tripoli
    {  527, 6554,  164493,  6, }, // Africa/Tunis
    {  527, 7571,   20486,  7, }, // America/Argentina/Tucuman
    {  527, 4252,      14,  8, }, // America/Asuncion
    {  527,  237,      22,  8, }, // America/Atikokan
    {  527, 7623,      30, 17, }, // America/Bahia_Banderas
    {  527, 3868,      47,  6, }, // America/Cancun
    {  527, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  527, 5723,      66,  7, }, // America/Curacao
    {  527, 2281,  164499,  7, }, // America/Havana
    {  527,  481,      73, 13, }, // America/Indiana/Knox
    {  527, 7961,      86, 16, }, // America/Indiana/Marengo
    {  527, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  527, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  527, 8038,     139, 14, }, // America/Indiana/Vevay
    {  527, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  527, 8086,     171, 16, }, // America/Indiana/Winamac
    {  527, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  527, 5932,     207, 22, }, // America/Lower_Princes
    {  527, 5805,   20554,  9, }, // America/Martinique
    {  527, 8220,     229,  6, }, // America/Merida
    {  527, 2949,  164506, 21, }, // America/Mexico_City
    {  527, 2379,  164527,  8, }, // America/New_York
    {  527, 1850,     235, 19, }, // America/Noronha
    {  527, 8301,  164535, 25, }, // America/North_Dakota/Beulah
    {  527, 8329,  164560, 25, }, // America/North_Dakota/Center
    {  527, 8357,  164585, 28, }, // America/North_Dakota/New_Salem
    {  527,  393,     317,  4, }, // America/Nuuk
    {  527, 2356,  164613,  6, }, // America/Panama
    {  527, 5954,  164619, 16, }, // America/Port_of_Spain
    {  527, 6899,     321, 16, }, // America/Scoresbysund
    {  527, 8508,  164635, 16, }, // America/St_Barthelemy
    {  527, 2061,  164651, 13, }, // America/St_Johns
    {  527, 5863,  164664, 14, }, // America/St_Kitts
    {  527, 5880,  164678, 12, }, // America/St_Lucia
    {  527,  855,  164690, 13, }, // America/St_Thomas
    {  527, 5913,  164703, 14, }, // America/St_Vincent
    {  527, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  527, 5523,    5609,  5, }, // Asia/Aqtau
    {  527, 5368,    7606,  6, }, // Asia/Aqtobe
    {  527, 5534,    4168,  7, }, // Asia/Bahrain
    {  527, 4601,   57622,  4, }, // Asia/Chita
    {  527, 4540,   20757,  7, }, // Asia/Damascus
    {  527, 1093,  158496,  4, }, // Asia/Dhaka
    {  527, 1259,  164717, 22, }, // Asia/Ho_Chi_Minh
    {  527, 7006,   95267,  8, }, // Asia/Jakarta
    {  527, 1290,   95275, 11, }, // Asia/Jerusalem
    {  527, 1185,     448,  9, }, // Asia/Kathmandu
    {  527,  992,   20838,  7, }, // Asia/Kolkata
    {  527, 4197,    4288, 11, }, // Asia/Krasnoyarsk
    {  527, 1211,     464,  5, }, // Asia/Macau
    {  527, 6920,   20857,  7, }, // Asia/Muscat
    {  527, 5559,  164739,  5, }, // Asia/Qatar
    {  527, 4269,   26897,  9, }, // Asia/Qyzylorda
    {  527, 3584,  164744,  5, }, // Asia/Riyadh
    {  527, 7245,   20886, 10, }, // Asia/Samarkand
    {  527, 3279,  164749,  4, }, // Asia/Seoul
    {  527, 2772,  164753,  7, }, // Asia/Tehran
    {  527, 1021,  164760, 10, }, // Asia/Ulaanbaatar
    {  527, 1235,     477,  6, }, // Asia/Yangon
    {  527, 3663,  164770,  7, }, // Atlantic/Azores
    {  527, 6628,  164777, 15, }, // Atlantic/Canary
    {  527, 3727,  164792, 10, }, // Atlantic/Cape_Verde
    {  527, 1395,  164802, 14, }, // Atlantic/Faroe
    {  527, 7272,  164816, 17, }, // Atlantic/South_Georgia
    {  527, 6821,  164833, 10, }, // Atlantic/St_Helena
    {  527, 2300,    5911,  5, }, // Europe/Athens
    {  527, 6336,  164843,  8, }, // Europe/Belgrade
    {  527, 4757,  164851,  7, }, // Europe/Berlin
    {  527, 2167,   57661,  9, }, // Europe/Brussels
    {  527, 3928,  164858,  8, }, // Europe/Bucharest
    {  527, 3806,  164866,  8, }, // Europe/Budapest
    {  527, 8772,   27037,  8, }, // Europe/Busingen
    {  527, 2338,  164874,  7, }, // Europe/Dublin
    {  527, 6307,   95341, 10, }, // Europe/Gibraltar
    {  527, 8804,  164881, 12, }, // Europe/Isle_of_Man
    {  527, 1130,  164893,  8, }, // Europe/Istanbul
    {  527, 2527,     514,  4, }, // Europe/Kyiv
    {  527, 3213,   93414,  7, }, // Europe/Lisbon
    {  527, 2501,   95370,  6, }, // Europe/London
    {  527, 6365,  134573, 11, }, // Europe/Luxembourg
    {  527, 6410,  164901,  6, }, // Europe/Monaco
    {  527, 3537,   95387,  5, }, // Europe/Moscow
    {  527, 4284,   95392,  6, }, // Europe/Paris
    {  527, 6275,   21185,  5, }, // Europe/Prague
    {  527, 6324,    6064,  4, }, // Europe/Rome
    {  527, 6470,  164907, 12, }, // Europe/San_Marino
    {  527, 6523,  164919,  9, }, // Europe/Stockholm
    {  527, 6202,   21213,  6, }, // Europe/Tirane
    {  527, 6567,  164928, 22, }, // Europe/Vatican
    {  527, 3190,  164950,  8, }, // Europe/Warsaw
    {  527, 6261,  164958,  8, }, // Europe/Zagreb
    {  527, 6540,  164966,  6, }, // Europe/Zurich
    {  527, 6046,  164972, 10, }, // Indian/Christmas
    {  527, 7050,   74646,  8, }, // Indian/Maldives
    {  527, 4069,  164982, 10, }, // Indian/Mauritius
    {  527, 5098,  164992,  6, }, // Indian/Mayotte
    {  527, 7211,  164998,  8, }, // Indian/Reunion
    {  527, 3157,     554,  5, }, // Pacific/Chuuk
    {  527, 6018,  165006,  4, }, // Pacific/Guam
    {  527, 3049,    4619,  9, }, // Pacific/Kanton
    {  527, 4051,  165010,  9, }, // Pacific/Marquesas
    {  527, 3096,     586,  7, }, // Pacific/Pohnpei
    {  528,   14,       0,  6, }, // Africa/Asmara Sena/Latin/Mozambique
    {  528, 4435,       6,  8, }, // Africa/Sao_Tome
    {  528, 4252,      14,  8, }, // America/Asuncion
    {  528,  237,      22,  8, }, // America/Atikokan
    {  528, 7623,      30, 17, }, // America/Bahia_Banderas
    {  528, 3868,      47,  6, }, // America/Cancun
    {  528, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  528, 5723,      66,  7, }, // America/Curacao
    {  528,  481,      73, 13, }, // America/Indiana/Knox
    {  528, 7961,      86, 16, }, // America/Indiana/Marengo
    {  528, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  528, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  528, 8038,     139, 14, }, // America/Indiana/Vevay
    {  528, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  528, 8086,     171, 16, }, // America/Indiana/Winamac
    {  528, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  528, 5932,     207, 22, }, // America/Lower_Princes
    {  528, 8220,     229,  6, }, // America/Merida
    {  528, 1850,     235, 19, }, // America/Noronha
    {  528, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  528, 8329,     274, 20, }, // America/North_Dakota/Center
    {  528, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  528,  393,     317,  4, }, // America/Nuuk
    {  528, 6899,     321, 16, }, // America/Scoresbysund
    {  528, 8508,     337, 14, }, // America/St_Barthelemy
    {  528, 2061,     351, 10, }, // America/St_Johns
    {  528, 5863,     361,  9, }, // America/St_Kitts
    {  528, 5880,     370,  9, }, // America/St_Lucia
    {  528,  855,     379, 10, }, // America/St_Thomas
    {  528, 5913,     389, 11, }, // America/St_Vincent
    {  528, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  528, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  528, 1185,     448,  9, }, // Asia/Kathmandu
    {  528,  992,     457,  7, }, // Asia/Kolkata
    {  528, 1211,     464,  5, }, // Asia/Macau
    {  528, 1235,     477,  6, }, // Asia/Yangon
    {  528, 1395,     483,  5, }, // Atlantic/Faroe
    {  528, 6821,     488, 10, }, // Atlantic/St_Helena
    {  528, 2527,     514,  4, }, // Europe/Kyiv
    {  528, 7211,     547,  7, }, // Indian/Reunion
    {  528, 3157,     554,  5, }, // Pacific/Chuuk
    {  528, 3096,     586,  7, }, // Pacific/Pohnpei
    {  529, 6788,     604,  6, }, // Africa/Abidjan Serbian/Cyrillic/Serbia
    {  529, 6760,   27178,  4, }, // Africa/Accra
    {  529, 5059,   33846, 10, }, // Africa/Addis_Ababa
    {  529, 7420,     626,  5, }, // Africa/Algiers
    {  529,   14,     631,  6, }, // Africa/Asmara
    {  529,   44,     637,  6, }, // Africa/Bamako
    {  529, 5257,   33856,  6, }, // Africa/Bangui
    {  529, 6746,     648,  6, }, // Africa/Banjul
    {  529, 7435,   33862,  5, }, // Africa/Bissau
    {  529, 4957,   33867,  7, }, // Africa/Blantyre
    {  529, 5287,   33874,  8, }, // Africa/Brazzaville
    {  529, 4922,   33882,  9, }, // Africa/Bujumbura
    {  529, 2320,   33891,  5, }, // Africa/Cairo
    {  529, 4117,   33896, 10, }, // Africa/Casablanca
    {  529, 7449,     699,  5, }, // Africa/Ceuta
    {  529, 6773,     704,  7, }, // Africa/Conakry
    {  529, 6840,     711,  5, }, // Africa/Dakar
    {  529, 5130,     716, 12, }, // Africa/Dar_es_Salaam
    {  529, 5043,     728,  6, }, // Africa/Djibouti
    {  529, 5243,     734,  5, }, // Africa/Douala
    {  529, 5166,   33906,  7, }, // Africa/El_Aaiun
    {  529, 6853,     746,  7, }, // Africa/Freetown
    {  529, 4906,     753,  7, }, // Africa/Gaborone
    {  529, 5015,   33913,  6, }, // Africa/Harare
    {  529, 4479,   33919, 11, }, // Africa/Johannesburg
    {  529, 4499,     779,  4, }, // Africa/Juba
    {  529, 5151,     783,  7, }, // Africa/Kampala
    {  529, 4524,   33930,  6, }, // Africa/Khartoum
    {  529, 4973,     796,  6, }, // Africa/Kigali
    {  529, 5306,     802,  7, }, // Africa/Kinshasa
    {  529, 4744,     809,  5, }, // Africa/Lagos
    {  529, 5336,     814,  8, }, // Africa/Libreville
    {  529, 6869,     822,  4, }, // Africa/Lome
    {  529, 5211,     826,  6, }, // Africa/Luanda
    {  529, 4939,     832,  9, }, // Africa/Lubumbashi
    {  529, 5001,     841,  6, }, // Africa/Lusaka
    {  529, 5322,     847,  6, }, // Africa/Malabo
    {  529, 4987,   33945,  6, }, // Africa/Maputo
    {  529, 5197,     859,  6, }, // Africa/Maseru
    {  529, 5182,     865,  7, }, // Africa/Mbabane
    {  529, 5113,     872,  7, }, // Africa/Mogadishu
    {  529, 7034,   33951,  9, }, // Africa/Monrovia
    {  529, 3853,   33960,  7, }, // Africa/Nairobi
    {  529, 5271,     895,  7, }, // Africa/Ndjamena
    {  529, 5354,   33967,  7, }, // Africa/Niamey
    {  529, 6803,     908,  7, }, // Africa/Nouakchott
    {  529, 6727,     915,  8, }, // Africa/Ouagadougou
    {  529, 5225,   33974, 10, }, // Africa/Porto-Novo
    {  529, 4435,   33984,  8, }, // Africa/Sao_Tome
    {  529, 2866,     941,  7, }, // Africa/Tripoli
    {  529, 6554,     948,  5, }, // Africa/Tunis
    {  529, 4152,     953,  7, }, // Africa/Windhoek
    {  529,  132,     960,  4, }, // America/Adak
    {  529, 3346,   33992,  7, }, // America/Anchorage
    {  529, 5607,   33999,  7, }, // America/Anguilla
    {  529, 5624,   34006,  7, }, // America/Antigua
    {  529, 4554,   34013, 10, }, // America/Araguaina
    {  529,  166,  165019, 12, }, // America/Argentina/Buenos_Aires
    {  529,   91,    1006,  9, }, // America/Argentina/Catamarca
    {  529,  270,   34035,  7, }, // America/Argentina/Cordoba
    {  529,  441,    1022,  5, }, // America/Argentina/Jujuy
    {  529, 7462,   34047,  8, }, // America/Argentina/La_Rioja
    {  529,  565,    1035,  7, }, // America/Argentina/Mendoza
    {  529, 7489,   34062, 11, }, // America/Argentina/Rio_Gallegos
    {  529, 7520,    1053,  5, }, // America/Argentina/Salta
    {  529, 7544,   34073,  8, }, // America/Argentina/San_Juan
    {  529, 5580,   34081,  7, }, // America/Argentina/San_Luis
    {  529, 7571,    1074,  7, }, // America/Argentina/Tucuman
    {  529, 7597,   34088,  7, }, // America/Argentina/Ushuaia
    {  529, 5640,    1087,  5, }, // America/Aruba
    {  529, 4252,   34095,  8, }, // America/Asuncion
    {  529,  237,  165031, 12, }, // America/Atikokan
    {  529, 3679,   34111,  5, }, // America/Bahia
    {  529, 7623,   34116, 14, }, // America/Bahia_Banderas
    {  529, 5654,    1131,  8, }, // America/Barbados
    {  529, 7646,   34130,  5, }, // America/Belem
    {  529, 5407,   34135,  6, }, // America/Belize
    {  529, 7660,   34141, 13, }, // America/Blanc-Sablon
    {  529, 7681,   34154,  9, }, // America/Boa_Vista
    {  529, 4361,    1170,  6, }, // America/Bogota
    {  529, 7699,   34163,  5, }, // America/Boise
    {  529, 7713,   34168, 11, }, // America/Cambridge_Bay
    {  529, 7735,   34179, 12, }, // America/Campo_Grande
    {  529, 3868,    1205,  6, }, // America/Cancun
    {  529, 4694,    1211,  7, }, // America/Caracas
    {  529, 4345,   34191,  5, }, // America/Cayenne
    {  529, 5496,  165043, 16, }, // America/Cayman
    {  529, 2260,    1247,  6, }, // America/Chicago
    {  529, 7756,  118226,  8, }, // America/Chihuahua
    {  529, 7774,  128289, 13, }, // America/Ciudad_Juarez
    {  529, 5422,   34218,  9, }, // America/Costa_Rica
    {  529, 7796,    1269,  7, }, // America/Creston
    {  529, 3791,    1276,  6, }, // America/Cuiaba
    {  529, 5723,   34227,  7, }, // America/Curacao
    {  529, 7812,   34234, 13, }, // America/Danmarkshavn
    {  529, 7833,   34247,  5, }, // America/Dawson
    {  529, 7848,   34252, 10, }, // America/Dawson_Creek
    {  529,  805,    1319,  6, }, // America/Denver
    {  529, 3465,    1325,  7, }, // America/Detroit
    {  529, 5739,    1332,  8, }, // America/Dominica
    {  529,  893,    1340,  8, }, // America/Edmonton
    {  529, 7869,    1348,  8, }, // America/Eirunepe
    {  529, 5441,    1356,  8, }, // America/El_Salvador
    {  529, 7886,    1364, 11, }, // America/Fort_Nelson
    {  529, 7906,    1375,  9, }, // America/Fortaleza
    {  529, 7924,   34262,  9, }, // America/Glace_Bay
    {  529, 6881,   34271,  7, }, // America/Goose_Bay
    {  529, 4612,   34278, 10, }, // America/Grand_Turk
    {  529, 5770,    1410,  7, }, // America/Grenada
    {  529, 5786,   34288,  9, }, // America/Guadeloupe
    {  529, 3760,    1426,  9, }, // America/Guatemala
    {  529, 4841,   34297,  8, }, // America/Guayaquil
    {  529, 6932,  128329,  7, }, // America/Guyana
    {  529, 1939,   34310,  8, }, // America/Halifax
    {  529, 2281,   34318,  6, }, // America/Havana
    {  529, 7942,   34324,  9, }, // America/Hermosillo
    {  529,  348,    1471, 12, }, // America/Indiana/Indianapolis
    {  529,  481,  128357, 14, }, // America/Indiana/Knox
    {  529, 7961,   34348, 17, }, // America/Indiana/Marengo
    {  529, 7985,  128371, 20, }, // America/Indiana/Petersburg
    {  529, 8012,   34385, 18, }, // America/Indiana/Tell_City
    {  529, 8038,   34403, 15, }, // America/Indiana/Vevay
    {  529, 8060,   34418, 18, }, // America/Indiana/Vincennes
    {  529, 8086,   34436, 17, }, // America/Indiana/Winamac
    {  529, 8110,    1570,  6, }, // America/Inuvik
    {  529,  660,   34453,  8, }, // America/Iqaluit
    {  529, 2799,   34461,  7, }, // America/Jamaica
    {  529, 5380,   34468,  4, }, // America/Juneau
    {  529,  521,  165059,  7, }, // America/Kentucky/Louisville
    {  529, 8125,   34472, 18, }, // America/Kentucky/Monticello
    {  529, 5704,  128422, 10, }, // America/Kralendijk
    {  529, 4376,   34500,  6, }, // America/La_Paz
    {  529, 7169,    1638,  4, }, // America/Lima
    {  529, 3239,   34506, 11, }, // America/Los_Angeles
    {  529, 5932,  165066, 21, }, // America/Lower_Princes
    {  529, 8153,   34536,  6, }, // America/Maceio
    {  529, 8168,   34542,  7, }, // America/Managua
    {  529, 1908,    1684,  6, }, // America/Manaus
    {  529, 5897,    1690,  6, }, // America/Marigot
    {  529, 5805,    1696,  8, }, // America/Martinique
    {  529, 8184,    1705,  9, }, // America/Matamoros
    {  529, 2917,   34549,  8, }, // America/Mazatlan
    {  529, 8202,    1722,  8, }, // America/Menominee
    {  529, 8220,    1730,  6, }, // America/Merida
    {  529, 8235,    1736, 10, }, // America/Metlakatla
    {  529, 2949,   34557, 12, }, // America/Mexico_City
    {  529, 4391,    1752,  7, }, // America/Miquelon
    {  529, 8254,    1759,  7, }, // America/Moncton
    {  529, 8270,   34569,  8, }, // America/Monterrey
    {  529, 4098,    1775, 10, }, // America/Montevideo
    {  529, 5824,   34577,  9, }, // America/Montserrat
    {  529, 5481,   34586,  5, }, // America/Nassau
    {  529, 2379,   34591,  6, }, // America/New_York
    {  529, 8288,    1809,  3, }, // America/Nome
    {  529, 1850,   34597,  6, }, // America/Noronha
    {  529, 8301,   34603, 22, }, // America/North_Dakota/Beulah
    {  529, 8329,   34625, 22, }, // America/North_Dakota/Center
    {  529, 8357,   34647, 26, }, // America/North_Dakota/New_Salem
    {  529,  393,  165087,  6, }, // America/Nuuk
    {  529, 8388,    1894,  7, }, // America/Ojinaga
    {  529, 2356,    1901,  6, }, // America/Panama
    {  529, 6173,    1907, 10, }, // America/Paramaribo
    {  529, 2973,    1917,  6, }, // America/Phoenix
    {  529, 3945,  128513, 12, }, // America/Port-au-Prince
    {  529, 5954,   34679, 13, }, // America/Port_of_Spain
    {  529, 8404,   34692, 10, }, // America/Porto_Velho
    {  529, 5843,  165093, 10, }, // America/Puerto_Rico
    {  529, 4030,   34711, 12, }, // America/Punta_Arenas
    {  529, 8424,   34723, 12, }, // America/Rankin_Inlet
    {  529, 8445,   34735,  6, }, // America/Recife
    {  529, 1995,   34741,  6, }, // America/Regina
    {  529, 8460,   34747,  7, }, // America/Resolute
    {  529,  695,   34754, 10, }, // America/Rio_Branco
    {  529, 8477,   34764,  8, }, // America/Santarem
    {  529, 2201,  165103,  8, }, // America/Santiago
    {  529, 6111,   34781, 13, }, // America/Santo_Domingo
    {  529, 1878,   34794,  9, }, // America/Sao_Paulo
    {  529, 6899,  165111, 12, }, // America/Scoresbysund
    {  529, 8494,    2074,  5, }, // America/Sitka
    {  529, 8508,   34815, 14, }, // America/St_Barthelemy
    {  529, 2061,   34829,  7, }, // America/St_Johns
    {  529, 5863,   34836,  9, }, // America/St_Kitts
    {  529, 5880,   34845, 10, }, // America/St_Lucia
    {  529,  855,   34855,  8, }, // America/St_Thomas
    {  529, 5913,   34863, 12, }, // America/St_Vincent
    {  529, 8530,   34875, 12, }, // America/Swift_Current
    {  529, 5461,    2156, 11, }, // America/Tegucigalpa
    {  529, 5756,    2167,  3, }, // America/Thule
    {  529,  313,    2171,  7, }, // America/Tijuana
    {  529,  608,    2178,  7, }, // America/Toronto
    {  529, 5688,    2185,  7, }, // America/Tortola
    {  529, 2093,    2192,  8, }, // America/Vancouver
    {  529, 2144,   34887,  8, }, // America/Whitehorse
    {  529,  734,   34895,  7, }, // America/Winnipeg
    {  529, 8552,   34902,  7, }, // America/Yakutat
    {  529, 6001,  128612,  5, }, // Antarctica/Casey
    {  529, 6094,   34914,  6, }, // Antarctica/Davis
    {  529, 6133,   34920, 13, }, // Antarctica/DumontDUrville
    {  529, 8568,   34933,  7, }, // Antarctica/Macquarie
    {  529, 7066,   34940,  5, }, // Antarctica/Mawson
    {  529, 7113,   34945,  8, }, // Antarctica/McMurdo
    {  529, 8589,    2272,  6, }, // Antarctica/Palmer
    {  529, 7226,    2278,  6, }, // Antarctica/Rothera
    {  529, 7295,   34953,  4, }, // Antarctica/Syowa
    {  529, 8607,    2289,  4, }, // Antarctica/Troll
    {  529, 7374,   34957,  6, }, // Antarctica/Vostok
    {  529, 1429,   34963, 11, }, // Arctic/Longyearbyen
    {  529, 5570,    2310,  4, }, // Asia/Aden
    {  529, 5395,   34974,  6, }, // Asia/Almaty
    {  529, 3968,   28663,  4, }, // Asia/Amman
    {  529, 5511,   34980,  6, }, // Asia/Anadyr
    {  529, 5523,    2331,  5, }, // Asia/Aqtau
    {  529, 5368,   34986,  7, }, // Asia/Aqtobe
    {  529,  964,    2342,  7, }, // Asia/Ashgabat
    {  529, 8624,   52009,  6, }, // Asia/Atyrau
    {  529, 3607,    2355,  6, }, // Asia/Baghdad
    {  529, 5534,    2361,  7, }, // Asia/Bahrain
    {  529, 3653,    2368,  4, }, // Asia/Baku
    {  529, 4466,    2372,  7, }, // Asia/Bangkok
    {  529, 3571,    2379,  7, }, // Asia/Barnaul
    {  529, 4086,   35005,  6, }, // Asia/Beirut
    {  529, 3778,    2392,  6, }, // Asia/Bishkek
    {  529, 5989,   35011,  6, }, // Asia/Brunei
    {  529, 4601,    2404,  4, }, // Asia/Chita
    {  529, 4511,    2408,  7, }, // Asia/Colombo
    {  529, 4540,    2415,  6, }, // Asia/Damascus
    {  529, 1093,     711,  4, }, // Asia/Dhaka
    {  529, 6192,    2426,  4, }, // Asia/Dili
    {  529, 3596,    2430,  5, }, // Asia/Dubai
    {  529, 6159,    2435,  7, }, // Asia/Dushanbe
    {  529, 8636,    2442,  9, }, // Asia/Famagusta
    {  529, 8651,    2451,  4, }, // Asia/Gaza
    {  529, 4795,   35018,  6, }, // Asia/Hebron
    {  529, 1259,   35024,  9, }, // Asia/Ho_Chi_Minh
    {  529, 2704,  165123,  8, }, // Asia/Hong_Kong
    {  529, 4771,    2475,  4, }, // Asia/Hovd
    {  529, 4184,   35042,  6, }, // Asia/Irkutsk
    {  529, 7006,    2486,  7, }, // Asia/Jakarta
    {  529, 6992,   35048,  8, }, // Asia/Jayapura
    {  529, 1290,   35056,  9, }, // Asia/Jerusalem
    {  529, 3560,    2510,  5, }, // Asia/Kabul
    {  529, 4316,    2529,  8, }, // Asia/Kamchatka
    {  529, 4239,    2540,  6, }, // Asia/Karachi
    {  529, 1185,    2546,  8, }, // Asia/Kathmandu
    {  529, 8661,   35065,  7, }, // Asia/Khandyga
    {  529,  992,   35072,  7, }, // Asia/Kolkata
    {  529, 4197,   35079, 11, }, // Asia/Krasnoyarsk
    {  529, 4859,   35090, 12, }, // Asia/Kuala_Lumpur
    {  529, 5976,    2592,  6, }, // Asia/Kuching
    {  529, 5547,   35102,  6, }, // Asia/Kuwait
    {  529, 1211,    2604,  5, }, // Asia/Macau
    {  529, 4017,    2609,  7, }, // Asia/Magadan
    {  529, 1349,    2616,  7, }, // Asia/Makassar
    {  529, 7182,    2623,  6, }, // Asia/Manila
    {  529, 6920,   35113,  6, }, // Asia/Muscat
    {  529, 2554,   35119,  8, }, // Asia/Nicosia
    {  529, 8675,   35127, 11, }, // Asia/Novokuznetsk
    {  529, 4135,    2653, 11, }, // Asia/Novosibirsk
    {  529, 4229,    2664,  4, }, // Asia/Omsk
    {  529, 7145,   35138,  4, }, // Asia/Oral
    {  529, 6961,   35142,  8, }, // Asia/Phnom_Penh
    {  529, 8693,  128669, 10, }, // Asia/Pontianak
    {  529, 4214,   35150,  9, }, // Asia/Pyongyang
    {  529, 5559,    2697,  5, }, // Asia/Qatar
    {  529, 8708,   35159,  8, }, // Asia/Qostanay
    {  529, 4269,   35167,  9, }, // Asia/Qyzylorda
    {  529, 3584,   35176,  5, }, // Asia/Riyadh
    {  529, 4408,    2732,  7, }, // Asia/Sakhalin
    {  529, 7245,    2739,  9, }, // Asia/Samarkand
    {  529, 3279,    2748,  4, }, // Asia/Seoul
    {  529, 1053,   35181,  6, }, // Asia/Shanghai
    {  529, 3300,    2758,  8, }, // Asia/Singapore
    {  529, 4297,   35187, 13, }, // Asia/Srednekolymsk
    {  529, 3263,   35200,  6, }, // Asia/Taipei
    {  529, 4781,    2785,  7, }, // Asia/Tashkent
    {  529, 3915,   35206,  7, }, // Asia/Tbilisi
    {  529, 2772,   35213,  7, }, // Asia/Tehran
    {  529, 1317,   35220,  5, }, // Asia/Thimphu
    {  529, 2821,    2810,  5, }, // Asia/Tokyo
    {  529, 4572,    2815,  5, }, // Asia/Tomsk
    {  529, 1021,   35225, 10, }, // Asia/Ulaanbaatar
    {  529, 1159,  165131,  6, }, // Asia/Urumqi
    {  529, 8722,    2836,  8, }, // Asia/Ust-Nera
    {  529, 6977,   35242, 10, }, // Asia/Vientiane
    {  529, 4710,    2851, 11, }, // Asia/Vladivostok
    {  529, 4828,   35252,  7, }, // Asia/Yakutsk
    {  529, 1235,   29043,  6, }, // Asia/Yangon
    {  529, 3883,   35259, 13, }, // Asia/Yekaterinburg
    {  529, 3747,   35272,  7, }, // Asia/Yerevan
    {  529, 3663,   35279,  5, }, // Atlantic/Azores
    {  529, 5671,   35284,  7, }, // Atlantic/Bermuda
    {  529, 6628,   35291, 15, }, // Atlantic/Canary
    {  529, 3727,   35306, 18, }, // Atlantic/Cape_Verde
    {  529, 1395,  165137, 13, }, // Atlantic/Faroe
    {  529, 8736,    2971,  7, }, // Atlantic/Madeira
    {  529, 2748,   35337,  9, }, // Atlantic/Reykjavik
    {  529, 7272,   35346, 13, }, // Atlantic/South_Georgia
    {  529, 6821,  165150, 12, }, // Atlantic/St_Helena
    {  529, 6644,    3029,  6, }, // Atlantic/Stanley
    {  529, 1670,  165162,  7, }, // Australia/Adelaide
    {  529, 1635,   35369,  8, }, // Australia/Brisbane
    {  529, 1799,   35377, 10, }, // Australia/Broken_Hill
    {  529, 1583,    3061,  6, }, // Australia/Darwin
    {  529, 3637,    3067,  5, }, // Australia/Eucla
    {  529, 1516,    3072,  6, }, // Australia/Hobart
    {  529, 8753,    3078,  8, }, // Australia/Lindeman
    {  529, 1547,   37322,  8, }, // Australia/Lord_Howe
    {  529, 1727,    3094,  7, }, // Australia/Melbourne
    {  529, 1762,    3101,  4, }, // Australia/Perth
    {  529, 1463,   35395,  6, }, // Australia/Sydney
    {  529, 6441,    3111,  9, }, // Europe/Amsterdam
    {  529, 6216,   29275,  6, }, // Europe/Andorra
    {  529, 3620,  165169,  8, }, // Europe/Astrakhan
    {  529, 2300,   35401,  5, }, // Europe/Athens
    {  529, 6336,   35406,  7, }, // Europe/Belgrade
    {  529, 4757,    3146,  6, }, // Europe/Berlin
    {  529, 6488,    3152, 10, }, // Europe/Bratislava
    {  529, 2167,   35413,  6, }, // Europe/Brussels
    {  529, 3928,   35419,  8, }, // Europe/Bucharest
    {  529, 3806,   35427, 10, }, // Europe/Budapest
    {  529, 8772,  165177,  8, }, // Europe/Busingen
    {  529, 2583,   35445,  7, }, // Europe/Chisinau
    {  529, 6289,   35452, 10, }, // Europe/Copenhagen
    {  529, 2338,   35462,  6, }, // Europe/Dublin
    {  529, 6307,    3232,  9, }, // Europe/Gibraltar
    {  529, 8788,   35468,  6, }, // Europe/Guernsey
    {  529, 6612,    3247,  8, }, // Europe/Helsinki
    {  529, 8804,  165185, 10, }, // Europe/Isle_of_Man
    {  529, 1130,   35484,  8, }, // Europe/Istanbul
    {  529, 8823,  128777,  5, }, // Europe/Jersey
    {  529, 3979,   35492, 11, }, // Europe/Kaliningrad
    {  529, 8929,    3287,  5, }, // Europe/Kirov
    {  529, 2527,   35503,  5, }, // Europe/Kyiv
    {  529, 3213,   35508,  7, }, // Europe/Lisbon
    {  529, 6506,   35515,  7, }, // Europe/Ljubljana
    {  529, 2501,    3313,  6, }, // Europe/London
    {  529, 6365,   35522, 10, }, // Europe/Luxembourg
    {  529, 4892,    3330,  6, }, // Europe/Madrid
    {  529, 6397,    3336,  5, }, // Europe/Malta
    {  529, 6582,   35532,  8, }, // Europe/Mariehamn
    {  529, 3693,    3350,  5, }, // Europe/Minsk
    {  529, 6410,    3355,  6, }, // Europe/Monaco
    {  529, 3537,    3361,  6, }, // Europe/Moscow
    {  529, 6458,    3367,  4, }, // Europe/Oslo
    {  529, 4284,   35540,  5, }, // Europe/Paris
    {  529, 6424,    3376,  9, }, // Europe/Podgorica
    {  529, 6275,    3385,  4, }, // Europe/Prague
    {  529, 8837,    3390,  4, }, // Europe/Riga
    {  529, 6324,    3394,  3, }, // Europe/Rome
    {  529, 4331,    3397,  6, }, // Europe/Samara
    {  529, 6470,   35545, 10, }, // Europe/San_Marino
    {  529, 6245,   35555,  8, }, // Europe/Sarajevo
    {  529, 4451,    3420,  7, }, // Europe/Saratov
    {  529, 8849,   35563, 10, }, // Europe/Simferopol
    {  529, 6383,   35573,  6, }, // Europe/Skopje
    {  529, 6599,   35579,  6, }, // Europe/Sofia
    {  529, 6523,   35585,  8, }, // Europe/Stockholm
    {  529, 8867,   35593,  5, }, // Europe/Tallinn
    {  529, 6202,    3461,  6, }, // Europe/Tirane
    {  529, 8882,   35598,  8, }, // Europe/Ulyanovsk
    {  529, 6352,   35606,  5, }, // Europe/Vaduz
    {  529, 6567,    3481,  7, }, // Europe/Vatican
    {  529, 6231,   35611,  3, }, // Europe/Vienna
    {  529, 8899,  165195,  6, }, // Europe/Vilnius
    {  529, 4727,    3499,  9, }, // Europe/Volgograd
    {  529, 3190,    3508,  7, }, // Europe/Warsaw
    {  529, 6261,    3515,  6, }, // Europe/Zagreb
    {  529, 6540,   35620,  5, }, // Europe/Zurich
    {  529, 5078,   35625, 12, }, // Indian/Antananarivo
    {  529, 6947,    3539,  5, }, // Indian/Chagos
    {  529, 6046,   35637,  5, }, // Indian/Christmas
    {  529, 6063,    3556,  5, }, // Indian/Cocos
    {  529, 5029,   35642,  6, }, // Indian/Comoro
    {  529, 6661,    3578,  8, }, // Indian/Kerguelen
    {  529, 7260,   35648,  4, }, // Indian/Mahe
    {  529, 7050,   35652,  7, }, // Indian/Maldives
    {  529, 4069,   35659, 10, }, // Indian/Mauritius
    {  529, 5098,   35669,  5, }, // Indian/Mayotte
    {  529, 7211,   35674,  7, }, // Indian/Reunion
    {  529, 4422,   35681,  5, }, // Pacific/Apia
    {  529,  932,   35686,  6, }, // Pacific/Auckland
    {  529, 3706,  128861,  8, }, // Pacific/Bougainville
    {  529, 3015,   37512,  5, }, // Pacific/Chatham
    {  529, 3157,    3639,  4, }, // Pacific/Chuuk
    {  529, 2237,   35698, 14, }, // Pacific/Easter
    {  529, 7360,    3654,  4, }, // Pacific/Efate
    {  529, 7327,    3659,  7, }, // Pacific/Fakaofo
    {  529, 3902,    3666,  4, }, // Pacific/Fiji
    {  529, 7343,    3670,  8, }, // Pacific/Funafuti
    {  529, 6678,    3678,  9, }, // Pacific/Galapagos
    {  529, 6696,   35712,  7, }, // Pacific/Gambier
    {  529, 3822,   35719, 11, }, // Pacific/Guadalcanal
    {  529, 6018,    3724,  4, }, // Pacific/Guam
    {  529, 2723,   35730,  8, }, // Pacific/Honolulu
    {  529, 3049,   35738,  9, }, // Pacific/Kanton
    {  529, 3998,    3734, 10, }, // Pacific/Kiritimati
    {  529, 7019,   35747,  5, }, // Pacific/Kosrae
    {  529, 2842,   35752, 10, }, // Pacific/Kwajalein
    {  529, 4877,   35762,  6, }, // Pacific/Majuro
    {  529, 4051,    3765,  6, }, // Pacific/Marquesas
    {  529, 8914,   35768,  6, }, // Pacific/Midway
    {  529, 7084,    3798,  5, }, // Pacific/Nauru
    {  529, 7132,    3803,  4, }, // Pacific/Niue
    {  529, 4168,    3807,  7, }, // Pacific/Norfolk
    {  529, 7098,    3814,  5, }, // Pacific/Noumea
    {  529, 3126,   35774,  9, }, // Pacific/Pago_Pago
    {  529, 7155,    3828,  5, }, // Pacific/Palau
    {  529, 7194,    3833,  7, }, // Pacific/Pitcairn
    {  529, 3096,   35783,  6, }, // Pacific/Pohnpei
    {  529, 4807,   35789, 11, }, // Pacific/Port_Moresby
    {  529, 6076,    3857,  9, }, // Pacific/Rarotonga
    {  529, 6031,   35800,  6, }, // Pacific/Saipan
    {  529, 7312,   35806,  6, }, // Pacific/Tahiti
    {  529, 6712,    3877,  6, }, // Pacific/Tarawa
    {  529, 4583,    3883,  9, }, // Pacific/Tongatapu
    {  529, 7392,   35812,  4, }, // Pacific/Wake
    {  529, 7405,   35816,  5, }, // Pacific/Wallis
    {  530,  348,  128344, 13, }, // America/Indiana/Indianapolis Serbian/Cyrillic/Bosnia and Herzegovina
    {  530, 8038,  165201, 14, }, // America/Indiana/Vevay
    {  530, 8060,  165215, 17, }, // America/Indiana/Vincennes
    {  530,  521,  165059,  6, }, // America/Kentucky/Louisville
    {  530, 8301,  165232, 22, }, // America/North_Dakota/Beulah
    {  530, 8329,  165254, 23, }, // America/North_Dakota/Center
    {  530, 8357,  165277, 27, }, // America/North_Dakota/New_Salem
    {  530, 3945,    1923, 12, }, // America/Port-au-Prince
    {  530, 5954,  128525, 13, }, // America/Port_of_Spain
    {  530, 5843,   34702,  9, }, // America/Puerto_Rico
    {  530, 1995,  128550,  7, }, // America/Regina
    {  530, 8460,  118538,  7, }, // America/Resolute
    {  530, 6899,   34803, 12, }, // America/Scoresbysund
    {  530, 8508,   36925, 13, }, // America/St_Barthelemy
    {  530, 2061,  128572,  9, }, // America/St_Johns
    {  530,  855,  128602, 10, }, // America/St_Thomas
    {  530, 8530,  165304, 12, }, // America/Swift_Current
    {  530, 6133,  165316, 13, }, // Antarctica/DumontDUrville
    {  530, 8568,  128629,  7, }, // Antarctica/Macquarie
    {  530, 1429,  165329,  7, }, // Arctic/Longyearbyen
    {  530, 2338,   35462,  6, }, // Europe/Dublin
    {  530, 2501,    3313,  6, }, // Europe/London
    {  530, 7132,  165336,  5, }, // Pacific/Niue
    {  533, 6788,   57696,  7, }, // Africa/Abidjan Serbian/Latin/Bosnia and Herzegovina
    {  533, 6760,    4649,  4, }, // Africa/Accra
    {  533, 5059,   35821, 10, }, // Africa/Addis_Ababa
    {  533, 7420,   57579,  5, }, // Africa/Algiers
    {  533,   14,   91251,  6, }, // Africa/Asmara
    {  533,   44,  165341,  6, }, // Africa/Bamako
    {  533, 5257,  165347,  6, }, // Africa/Bangui
    {  533, 6746,  165353,  6, }, // Africa/Banjul
    {  533, 7435,  165359,  5, }, // Africa/Bissau
    {  533, 4957,    6318,  7, }, // Africa/Blantyre
    {  533, 5287,    6326,  8, }, // Africa/Brazzaville
    {  533, 4922,  165364, 10, }, // Africa/Bujumbura
    {  533, 2320,    4690,  5, }, // Africa/Cairo
    {  533, 4117,   33276, 10, }, // Africa/Casablanca
    {  533, 7449,   21349,  5, }, // Africa/Ceuta
    {  533, 6773,    6367,  7, }, // Africa/Conakry
    {  533, 6840,   61161,  5, }, // Africa/Dakar
    {  533, 5130,    6374, 12, }, // Africa/Dar_es_Salaam
    {  533, 5043,   33286,  7, }, // Africa/Djibouti
    {  533, 5243,   21372,  5, }, // Africa/Douala
    {  533, 5166,    6393,  7, }, // Africa/El_Aaiun
    {  533, 6853,    6400,  7, }, // Africa/Freetown
    {  533, 4906,   61203,  7, }, // Africa/Gaborone
    {  533, 5015,   61211,  6, }, // Africa/Harare
    {  533, 4479,    6407, 11, }, // Africa/Johannesburg
    {  533, 4499,  124874,  5, }, // Africa/Juba
    {  533, 5151,  165374,  7, }, // Africa/Kampala
    {  533, 4524,    6423,  6, }, // Africa/Khartoum
    {  533, 4973,  124888,  6, }, // Africa/Kigali
    {  533, 5306,  122158,  7, }, // Africa/Kinshasa
    {  533, 4744,   61262,  5, }, // Africa/Lagos
    {  533, 5336,    6429,  8, }, // Africa/Libreville
    {  533, 6869,   61277,  4, }, // Africa/Lome
    {  533, 5211,  165381,  6, }, // Africa/Luanda
    {  533, 4939,  122180,  9, }, // Africa/Lubumbashi
    {  533, 5001,  165387,  6, }, // Africa/Lusaka
    {  533, 5322,  165393,  6, }, // Africa/Malabo
    {  533, 4987,  165399,  6, }, // Africa/Maputo
    {  533, 5197,   61303,  6, }, // Africa/Maseru
    {  533, 5182,  165405,  7, }, // Africa/Mbabane
    {  533, 5113,   33293,  7, }, // Africa/Mogadishu
    {  533, 7034,  122203,  9, }, // Africa/Monrovia
    {  533, 3853,   61333,  7, }, // Africa/Nairobi
    {  533, 5271,   57729,  8, }, // Africa/Ndjamena
    {  533, 5354,  165412,  7, }, // Africa/Niamey
    {  533, 6803,   57737,  7, }, // Africa/Nouakchott
    {  533, 6727,   26534,  8, }, // Africa/Ouagadougou
    {  533, 5225,   26542, 10, }, // Africa/Porto-Novo
    {  533, 4435,   91257,  8, }, // Africa/Sao_Tome
    {  533, 2866,   35888,  7, }, // Africa/Tripoli
    {  533, 6554,   91430,  5, }, // Africa/Tunis
    {  533, 4152,   21472,  7, }, // Africa/Windhoek
    {  533,  132,   97943,  4, }, // America/Adak
    {  533, 3346,  165419,  8, }, // America/Anchorage
    {  533, 5607,   33300,  7, }, // America/Anguilla
    {  533, 5624,   33307,  7, }, // America/Antigua
    {  533, 4554,  125041, 10, }, // America/Araguaina
    {  533,  166,  165427, 12, }, // America/Argentina/Buenos_Aires
    {  533,   91,    6518,  9, }, // America/Argentina/Catamarca
    {  533,  270,    6527,  7, }, // America/Argentina/Cordoba
    {  533,  441,  165439,  5, }, // America/Argentina/Jujuy
    {  533, 7462,    6539,  8, }, // America/Argentina/La_Rioja
    {  533,  565,   97634,  7, }, // America/Argentina/Mendoza
    {  533, 7489,  125081, 12, }, // America/Argentina/Rio_Gallegos
    {  533, 7520,   61439,  5, }, // America/Argentina/Salta
    {  533, 7544,  165444,  8, }, // America/Argentina/San_Juan
    {  533, 5580,  165452,  7, }, // America/Argentina/San_Luis
    {  533, 7571,    4842,  7, }, // America/Argentina/Tucuman
    {  533, 7597,  165459,  7, }, // America/Argentina/Ushuaia
    {  533, 5640,  165466,  5, }, // America/Aruba
    {  533, 4252,    6592,  8, }, // America/Asuncion
    {  533,  237,  165471, 12, }, // America/Atikokan
    {  533, 3679,  122369,  5, }, // America/Bahia
    {  533, 7623,  165483, 14, }, // America/Bahia_Banderas
    {  533, 5654,  122387,  8, }, // America/Barbados
    {  533, 7646,   61474,  5, }, // America/Belem
    {  533, 5407,  165497,  6, }, // America/Belize
    {  533, 7660,  165503, 13, }, // America/Blanc-Sablon
    {  533, 7681,  165516,  9, }, // America/Boa_Vista
    {  533, 4361,  165525,  6, }, // America/Bogota
    {  533, 7699,  165531,  5, }, // America/Boise
    {  533, 7713,  165536, 12, }, // America/Cambridge_Bay
    {  533, 7735,    4898, 12, }, // America/Campo_Grande
    {  533, 3868,    6682,  6, }, // America/Cancun
    {  533, 4694,    4910,  7, }, // America/Caracas
    {  533, 4345,    6688,  5, }, // America/Cayenne
    {  533, 5496,  165548, 16, }, // America/Cayman
    {  533, 2260,  165564,  6, }, // America/Chicago
    {  533, 7756,  125280,  8, }, // America/Chihuahua
    {  533, 7774,  165570, 13, }, // America/Ciudad_Juarez
    {  533, 5422,   33320,  9, }, // America/Costa_Rica
    {  533, 7796,    6736,  7, }, // America/Creston
    {  533, 3791,    4964,  6, }, // America/Cuiaba
    {  533, 5723,    6749,  7, }, // America/Curacao
    {  533, 7812,  165583, 13, }, // America/Danmarkshavn
    {  533, 7833,  125335,  5, }, // America/Dawson
    {  533, 7848,  125342, 10, }, // America/Dawson_Creek
    {  533,  805,   61557,  6, }, // America/Denver
    {  533, 3465,  122567,  7, }, // America/Detroit
    {  533, 5739,   21666,  8, }, // America/Dominica
    {  533,  893,   61580,  8, }, // America/Edmonton
    {  533, 7869,  165596,  8, }, // America/Eirunepe
    {  533, 5441,    6806,  8, }, // America/El_Salvador
    {  533, 7886,  125399, 11, }, // America/Fort_Nelson
    {  533, 7906,  165604,  9, }, // America/Fortaleza
    {  533, 7924,  165613,  9, }, // America/Glace_Bay
    {  533, 6881,  165622,  7, }, // America/Goose_Bay
    {  533, 4612,  165629, 10, }, // America/Grand_Turk
    {  533, 5770,  165639,  7, }, // America/Grenada
    {  533, 5786,   33329,  9, }, // America/Guadeloupe
    {  533, 3760,   33338,  9, }, // America/Guatemala
    {  533, 4841,  122647,  8, }, // America/Guayaquil
    {  533, 6932,   57584,  7, }, // America/Guyana
    {  533, 1939,    6905,  8, }, // America/Halifax
    {  533, 2281,  147788,  6, }, // America/Havana
    {  533, 7942,  165646, 10, }, // America/Hermosillo
    {  533,  348,  165656, 13, }, // America/Indiana/Indianapolis
    {  533,  481,  165669, 14, }, // America/Indiana/Knox
    {  533, 7961,  165683, 17, }, // America/Indiana/Marengo
    {  533, 7985,  165700, 20, }, // America/Indiana/Petersburg
    {  533, 8012,  165720, 18, }, // America/Indiana/Tell_City
    {  533, 8038,  165738, 14, }, // America/Indiana/Vevay
    {  533, 8060,  165752, 17, }, // America/Indiana/Vincennes
    {  533, 8086,  165769, 17, }, // America/Indiana/Winamac
    {  533, 8110,  122810,  6, }, // America/Inuvik
    {  533,  660,  165786,  8, }, // America/Iqaluit
    {  533, 2799,   33347,  7, }, // America/Jamaica
    {  533, 5380,  165794,  4, }, // America/Juneau
    {  533,  521,  165798,  6, }, // America/Kentucky/Louisville
    {  533, 8125,  165804, 18, }, // America/Kentucky/Monticello
    {  533, 5704,  165822, 10, }, // America/Kralendijk
    {  533, 4376,  165832,  6, }, // America/La_Paz
    {  533, 7169,   22041,  4, }, // America/Lima
    {  533, 3239,  165838, 11, }, // America/Los_Angeles
    {  533, 5932,  165849, 21, }, // America/Lower_Princes
    {  533, 8153,  122903,  6, }, // America/Maceio
    {  533, 8168,  122909,  7, }, // America/Managua
    {  533, 1908,  122916,  6, }, // America/Manaus
    {  533, 5897,   61871,  6, }, // America/Marigot
    {  533, 5805,    5260,  8, }, // America/Martinique
    {  533, 8184,  122931,  9, }, // America/Matamoros
    {  533, 2917,  165870,  8, }, // America/Mazatlan
    {  533, 8202,    7118,  8, }, // America/Menominee
    {  533, 8220,    7126,  6, }, // America/Merida
    {  533, 8235,  165878, 10, }, // America/Metlakatla
    {  533, 2949,  165888, 12, }, // America/Mexico_City
    {  533, 4391,    7149,  7, }, // America/Miquelon
    {  533, 8254,    7156,  7, }, // America/Moncton
    {  533, 8270,   61920,  8, }, // America/Monterrey
    {  533, 4098,  165900, 10, }, // America/Montevideo
    {  533, 5824,    7172,  9, }, // America/Montserrat
    {  533, 5481,  125812,  5, }, // America/Nassau
    {  533, 2379,  165910,  7, }, // America/New_York
    {  533, 8288,   21968,  3, }, // America/Nome
    {  533, 1850,    7194,  7, }, // America/Noronha
    {  533, 8301,  165917, 22, }, // America/North_Dakota/Beulah
    {  533, 8329,  165939, 23, }, // America/North_Dakota/Center
    {  533, 8357,  165962, 27, }, // America/North_Dakota/New_Salem
    {  533,  393,  165989,  6, }, // America/Nuuk
    {  533, 8388,    7276,  7, }, // America/Ojinaga
    {  533, 2356,  165995,  6, }, // America/Panama
    {  533, 6173,  166001, 10, }, // America/Paramaribo
    {  533, 2973,  125928,  6, }, // America/Phoenix
    {  533, 3945,  166011, 12, }, // America/Port-au-Prince
    {  533, 5954,  166023, 13, }, // America/Port_of_Spain
    {  533, 8404,  166036, 11, }, // America/Porto_Velho
    {  533, 5843,   33426,  9, }, // America/Puerto_Rico
    {  533, 4030,  125989, 12, }, // America/Punta_Arenas
    {  533, 8424,  126003, 12, }, // America/Rankin_Inlet
    {  533, 8445,  166047,  6, }, // America/Recife
    {  533, 1995,  166053,  8, }, // America/Regina
    {  533, 8460,  166061,  7, }, // America/Resolute
    {  533,  695,    5480, 10, }, // America/Rio_Branco
    {  533, 8477,  166068,  8, }, // America/Santarem
    {  533, 2201,  123192,  8, }, // America/Santiago
    {  533, 6111,  166076, 13, }, // America/Santo_Domingo
    {  533, 1878,  166089,  9, }, // America/Sao_Paulo
    {  533, 6899,  123220, 12, }, // America/Scoresbysund
    {  533, 8494,  166098,  5, }, // America/Sitka
    {  533, 8508,  126107, 13, }, // America/St_Barthelemy
    {  533, 2061,  126120, 10, }, // America/St_Johns
    {  533, 5863,  126132,  9, }, // America/St_Kitts
    {  533, 5880,  166103, 10, }, // America/St_Lucia
    {  533,  855,  126154, 10, }, // America/St_Thomas
    {  533, 5913,  166113, 12, }, // America/St_Vincent
    {  533, 8530,  126180, 12, }, // America/Swift_Current
    {  533, 5461,    5528, 11, }, // America/Tegucigalpa
    {  533, 5756,   22227,  3, }, // America/Thule
    {  533,  313,    7479,  7, }, // America/Tijuana
    {  533,  608,  166125,  7, }, // America/Toronto
    {  533, 5688,  166132,  7, }, // America/Tortola
    {  533, 2093,    7493,  8, }, // America/Vancouver
    {  533, 2144,  166139,  8, }, // America/Whitehorse
    {  533,  734,   62179,  7, }, // America/Winnipeg
    {  533, 8552,    7517,  7, }, // America/Yakutat
    {  533, 6001,    7524,  5, }, // Antarctica/Casey
    {  533, 6094,    7529,  6, }, // Antarctica/Davis
    {  533, 6133,  166147, 13, }, // Antarctica/DumontDUrville
    {  533, 8568,  123368,  7, }, // Antarctica/Macquarie
    {  533, 7066,  123375,  5, }, // Antarctica/Mawson
    {  533, 7113,  166160,  8, }, // Antarctica/McMurdo
    {  533, 8589,  126311,  6, }, // Antarctica/Palmer
    {  533, 7226,    5583,  6, }, // Antarctica/Rothera
    {  533, 7295,  123396,  4, }, // Antarctica/Syowa
    {  533, 8607,  123400,  4, }, // Antarctica/Troll
    {  533, 7374,  123406,  6, }, // Antarctica/Vostok
    {  533, 1429,  166168,  7, }, // Arctic/Longyearbyen
    {  533, 5570,   62217,  4, }, // Asia/Aden
    {  533, 5395,    7594,  6, }, // Asia/Almaty
    {  533, 3968,    5605,  4, }, // Asia/Amman
    {  533, 5511,    7600,  6, }, // Asia/Anadyr
    {  533, 5523,    5609,  5, }, // Asia/Aqtau
    {  533, 5368,   33435,  7, }, // Asia/Aqtobe
    {  533,  964,   33442,  7, }, // Asia/Ashgabat
    {  533, 8624,    7612,  6, }, // Asia/Atyrau
    {  533, 3607,    4162,  6, }, // Asia/Baghdad
    {  533, 5534,    4168,  7, }, // Asia/Bahrain
    {  533, 3653,   62255,  4, }, // Asia/Baku
    {  533, 4466,  123472,  7, }, // Asia/Bangkok
    {  533, 3571,  123480,  7, }, // Asia/Barnaul
    {  533, 4086,    7625,  6, }, // Asia/Beirut
    {  533, 3778,   33455,  6, }, // Asia/Bishkek
    {  533, 5989,   33461,  6, }, // Asia/Brunei
    {  533, 4601,   57622,  4, }, // Asia/Chita
    {  533, 4511,    5637,  7, }, // Asia/Colombo
    {  533, 4540,    4201,  6, }, // Asia/Damascus
    {  533, 1093,    4717,  4, }, // Asia/Dhaka
    {  533, 6192,  126467,  4, }, // Asia/Dili
    {  533, 3596,  123516,  5, }, // Asia/Dubai
    {  533, 6159,   33468,  7, }, // Asia/Dushanbe
    {  533, 8636,  166175,  9, }, // Asia/Famagusta
    {  533, 8651,   62313,  4, }, // Asia/Gaza
    {  533, 4795,   62318,  6, }, // Asia/Hebron
    {  533, 1259,   33475,  9, }, // Asia/Ho_Chi_Minh
    {  533, 2704,    4231,  8, }, // Asia/Hong_Kong
    {  533, 4771,  123546,  4, }, // Asia/Hovd
    {  533, 4184,  147155,  6, }, // Asia/Irkutsk
    {  533, 7006,   33484,  8, }, // Asia/Jakarta
    {  533, 6992,   33492,  9, }, // Asia/Jayapura
    {  533, 1290,  166184,  9, }, // Asia/Jerusalem
    {  533, 3560,   62370,  5, }, // Asia/Kabul
    {  533, 4316,   33510,  8, }, // Asia/Kamchatka
    {  533, 4239,   33518,  6, }, // Asia/Karachi
    {  533, 1185,    5711,  8, }, // Asia/Kathmandu
    {  533, 8661,   33524,  7, }, // Asia/Khandyga
    {  533,  992,   26864,  7, }, // Asia/Kolkata
    {  533, 4197,    4288, 11, }, // Asia/Krasnoyarsk
    {  533, 4859,  166193, 12, }, // Asia/Kuala_Lumpur
    {  533, 5976,   33531,  6, }, // Asia/Kuching
    {  533, 5547,    7725,  6, }, // Asia/Kuwait
    {  533, 1211,    7731,  5, }, // Asia/Macau
    {  533, 4017,  123625,  7, }, // Asia/Magadan
    {  533, 1349,    7736,  7, }, // Asia/Makassar
    {  533, 7182,  166205,  6, }, // Asia/Manila
    {  533, 6920,    4326,  6, }, // Asia/Muscat
    {  533, 2554,   33542,  8, }, // Asia/Nicosia
    {  533, 8675,   33550, 12, }, // Asia/Novokuznetsk
    {  533, 4135,   62477, 11, }, // Asia/Novosibirsk
    {  533, 4229,   62489,  4, }, // Asia/Omsk
    {  533, 7145,   62494,  4, }, // Asia/Oral
    {  533, 6961,   22518,  8, }, // Asia/Phnom_Penh
    {  533, 8693,  166211, 10, }, // Asia/Pontianak
    {  533, 4214,   33562,  9, }, // Asia/Pyongyang
    {  533, 5559,    4332,  5, }, // Asia/Qatar
    {  533, 8708,    7770,  8, }, // Asia/Qostanay
    {  533, 4269,    7778,  9, }, // Asia/Qyzylorda
    {  533, 3584,   33571,  5, }, // Asia/Riyadh
    {  533, 4408,   33576,  7, }, // Asia/Sakhalin
    {  533, 7245,   26906,  9, }, // Asia/Samarkand
    {  533, 3279,    7798,  4, }, // Asia/Seoul
    {  533, 1053,   33583,  6, }, // Asia/Shanghai
    {  533, 3300,   20900,  8, }, // Asia/Singapore
    {  533, 4297,  166221, 14, }, // Asia/Srednekolymsk
    {  533, 3263,    7830,  6, }, // Asia/Taipei
    {  533, 4781,   33589,  7, }, // Asia/Tashkent
    {  533, 3915,  126820,  7, }, // Asia/Tbilisi
    {  533, 2772,    7842,  7, }, // Asia/Tehran
    {  533, 1317,   91770,  5, }, // Asia/Thimphu
    {  533, 2821,    4356,  5, }, // Asia/Tokyo
    {  533, 4572,  123800,  5, }, // Asia/Tomsk
    {  533, 1021,   26923, 10, }, // Asia/Ulaanbaatar
    {  533, 1159,  166235,  6, }, // Asia/Urumqi
    {  533, 8722,  166241,  8, }, // Asia/Ust-Nera
    {  533, 6977,   33602, 10, }, // Asia/Vientiane
    {  533, 4710,   62644, 11, }, // Asia/Vladivostok
    {  533, 4828,    7873,  7, }, // Asia/Yakutsk
    {  533, 1235,    7880,  6, }, // Asia/Yangon
    {  533, 3883,    4380, 13, }, // Asia/Yekaterinburg
    {  533, 3747,    7898,  7, }, // Asia/Yerevan
    {  533, 3663,   33612,  5, }, // Atlantic/Azores
    {  533, 5671,   74483,  7, }, // Atlantic/Bermuda
    {  533, 6628,  166249, 15, }, // Atlantic/Canary
    {  533, 3727,  166264, 18, }, // Atlantic/Cape_Verde
    {  533, 1395,  166282, 13, }, // Atlantic/Faroe
    {  533, 8736,  166295,  7, }, // Atlantic/Madeira
    {  533, 2748,   33617,  9, }, // Atlantic/Reykjavik
    {  533, 7272,  166302, 15, }, // Atlantic/South_Georgia
    {  533, 6821,  166317, 12, }, // Atlantic/St_Helena
    {  533, 6644,    7965,  6, }, // Atlantic/Stanley
    {  533, 1670,  166329,  7, }, // Australia/Adelaide
    {  533, 1635,  166336,  8, }, // Australia/Brisbane
    {  533, 1799,  127037, 10, }, // Australia/Broken_Hill
    {  533, 1583,    7998,  6, }, // Australia/Darwin
    {  533, 3637,  166344,  5, }, // Australia/Eucla
    {  533, 1516,   62801,  6, }, // Australia/Hobart
    {  533, 8753,  124002,  8, }, // Australia/Lindeman
    {  533, 1547,   22727,  8, }, // Australia/Lord_Howe
    {  533, 1727,    8021,  7, }, // Australia/Melbourne
    {  533, 1762,   22735,  4, }, // Australia/Perth
    {  533, 1463,    8033,  6, }, // Australia/Sydney
    {  533, 6441,   62828,  9, }, // Europe/Amsterdam
    {  533, 6216,   33638,  6, }, // Europe/Andorra
    {  533, 3620,    8046,  8, }, // Europe/Astrakhan
    {  533, 2300,   33652,  5, }, // Europe/Athens
    {  533, 6336,    8060,  7, }, // Europe/Belgrade
    {  533, 4757,   62866,  6, }, // Europe/Berlin
    {  533, 6488,  166349, 10, }, // Europe/Bratislava
    {  533, 2167,   33657,  6, }, // Europe/Brussels
    {  533, 3928,   33663,  8, }, // Europe/Bucharest
    {  533, 3806,   33671, 10, }, // Europe/Budapest
    {  533, 8772,  166359,  8, }, // Europe/Busingen
    {  533, 2583,   33681,  8, }, // Europe/Chisinau
    {  533, 6289,    4475, 10, }, // Europe/Copenhagen
    {  533, 2338,   33689,  6, }, // Europe/Dublin
    {  533, 6307,  127223,  9, }, // Europe/Gibraltar
    {  533, 8788,   33695,  6, }, // Europe/Guernsey
    {  533, 6612,  127241,  8, }, // Europe/Helsinki
    {  533, 8804,   33701, 10, }, // Europe/Isle_of_Man
    {  533, 1130,   62990,  8, }, // Europe/Istanbul
    {  533, 8823,  166367,  6, }, // Europe/Jersey
    {  533, 3979,   33711, 12, }, // Europe/Kaliningrad
    {  533, 8929,  124195,  5, }, // Europe/Kirov
    {  533, 2527,   33723,  5, }, // Europe/Kyiv
    {  533, 3213,   33728,  7, }, // Europe/Lisbon
    {  533, 6506,  166373,  9, }, // Europe/Ljubljana
    {  533, 2501,   63039,  6, }, // Europe/London
    {  533, 6365,    8172, 10, }, // Europe/Luxembourg
    {  533, 4892,   63057,  6, }, // Europe/Madrid
    {  533, 6397,  166382,  5, }, // Europe/Malta
    {  533, 6582,   99286,  8, }, // Europe/Mariehamn
    {  533, 3693,   63069,  5, }, // Europe/Minsk
    {  533, 6410,    6044,  6, }, // Europe/Monaco
    {  533, 3537,   22880,  6, }, // Europe/Moscow
    {  533, 6458,  166387,  4, }, // Europe/Oslo
    {  533, 4284,   33735,  5, }, // Europe/Paris
    {  533, 6424,  166391,  9, }, // Europe/Podgorica
    {  533, 6275,    8201,  4, }, // Europe/Prague
    {  533, 8837,  166400,  4, }, // Europe/Riga
    {  533, 6324,   33740,  3, }, // Europe/Rome
    {  533, 4331,  166404,  6, }, // Europe/Samara
    {  533, 6470,  166410, 10, }, // Europe/San_Marino
    {  533, 6245,  166420,  8, }, // Europe/Sarajevo
    {  533, 4451,   78360,  7, }, // Europe/Saratov
    {  533, 8849,   33743, 11, }, // Europe/Simferopol
    {  533, 6383,   33754,  7, }, // Europe/Skopje
    {  533, 6599,   33761,  6, }, // Europe/Sofia
    {  533, 6523,    8242,  8, }, // Europe/Stockholm
    {  533, 8867,    8250,  5, }, // Europe/Tallinn
    {  533, 6202,   21213,  6, }, // Europe/Tirane
    {  533, 8882,    8261,  9, }, // Europe/Ulyanovsk
    {  533, 6352,   63171,  5, }, // Europe/Vaduz
    {  533, 6567,    6096,  7, }, // Europe/Vatican
    {  533, 6231,   33775,  3, }, // Europe/Vienna
    {  533, 8899,  166428,  7, }, // Europe/Vilnius
    {  533, 4727,   63187,  9, }, // Europe/Volgograd
    {  533, 3190,   33778,  7, }, // Europe/Warsaw
    {  533, 6261,   63205,  6, }, // Europe/Zagreb
    {  533, 6540,   33785,  5, }, // Europe/Zurich
    {  533, 5078,  166435, 12, }, // Indian/Antananarivo
    {  533, 6947,  124395,  5, }, // Indian/Chagos
    {  533, 6046,   33790,  5, }, // Indian/Christmas
    {  533, 6063,    4583,  5, }, // Indian/Cocos
    {  533, 5029,   22985,  6, }, // Indian/Comoro
    {  533, 6661,   22991,  8, }, // Indian/Kerguelen
    {  533, 7260,  158799,  4, }, // Indian/Mahe
    {  533, 7050,   33819,  7, }, // Indian/Maldives
    {  533, 4069,   33826, 10, }, // Indian/Mauritius
    {  533, 5098,    8318,  5, }, // Indian/Mayotte
    {  533, 7211,   91350,  7, }, // Indian/Reunion
    {  533, 4422,  124493,  5, }, // Pacific/Apia
    {  533,  932,    8324,  6, }, // Pacific/Auckland
    {  533, 3706,  166447,  8, }, // Pacific/Bougainville
    {  533, 3015,  127604,  5, }, // Pacific/Chatham
    {  533, 3157,   91357,  4, }, // Pacific/Chuuk
    {  533, 2237,  166455, 15, }, // Pacific/Easter
    {  533, 7360,  127627,  4, }, // Pacific/Efate
    {  533, 7327,  166470,  7, }, // Pacific/Fakaofo
    {  533, 3902,   33836,  5, }, // Pacific/Fiji
    {  533, 7343,   63335,  8, }, // Pacific/Funafuti
    {  533, 6678,   92233,  9, }, // Pacific/Galapagos
    {  533, 6696,  166477,  7, }, // Pacific/Gambier
    {  533, 3822,  124569, 11, }, // Pacific/Guadalcanal
    {  533, 6018,  124581,  4, }, // Pacific/Guam
    {  533, 2723,    4611,  8, }, // Pacific/Honolulu
    {  533, 3049,  124586,  9, }, // Pacific/Kanton
    {  533, 3998,   92267, 10, }, // Pacific/Kiritimati
    {  533, 7019,  166484,  5, }, // Pacific/Kosrae
    {  533, 2842,  166489, 11, }, // Pacific/Kwajalein
    {  533, 4877,   92315,  6, }, // Pacific/Majuro
    {  533, 4051,   36340,  6, }, // Pacific/Marquesas
    {  533, 8914,   63371,  6, }, // Pacific/Midway
    {  533, 7084,  166500,  5, }, // Pacific/Nauru
    {  533, 7132,  166505,  5, }, // Pacific/Niue
    {  533, 4168,     572,  7, }, // Pacific/Norfolk
    {  533, 7098,  124660,  5, }, // Pacific/Noumea
    {  533, 3126,  166510,  9, }, // Pacific/Pago_Pago
    {  533, 7155,   65776,  5, }, // Pacific/Palau
    {  533, 7194,    8422,  7, }, // Pacific/Pitcairn
    {  533, 3096,    8429,  6, }, // Pacific/Pohnpei
    {  533, 4807,  166519, 11, }, // Pacific/Port_Moresby
    {  533, 6076,  166530,  9, }, // Pacific/Rarotonga
    {  533, 6031,  166539,  6, }, // Pacific/Saipan
    {  533, 7312,   63456,  6, }, // Pacific/Tahiti
    {  533, 6712,   23159,  6, }, // Pacific/Tarawa
    {  533, 4583,  166545,  9, }, // Pacific/Tongatapu
    {  533, 7392,   63469,  4, }, // Pacific/Wake
    {  533, 7405,   33841,  5, }, // Pacific/Wallis
    {  534, 6788,   57696,  7, }, // Africa/Abidjan Serbian/Latin/Kosovo
    {  534, 6760,    4649,  4, }, // Africa/Accra
    {  534, 5059,   35821, 10, }, // Africa/Addis_Ababa
    {  534, 7420,   57579,  5, }, // Africa/Algiers
    {  534,   14,   91251,  6, }, // Africa/Asmara
    {  534,   44,  165341,  6, }, // Africa/Bamako
    {  534, 5257,  165347,  6, }, // Africa/Bangui
    {  534, 6746,  165353,  6, }, // Africa/Banjul
    {  534, 7435,  165359,  5, }, // Africa/Bissau
    {  534, 4957,    6318,  7, }, // Africa/Blantyre
    {  534, 5287,    6326,  8, }, // Africa/Brazzaville
    {  534, 4922,  165364, 10, }, // Africa/Bujumbura
    {  534, 2320,    4690,  5, }, // Africa/Cairo
    {  534, 4117,   33276, 10, }, // Africa/Casablanca
    {  534, 7449,   21349,  5, }, // Africa/Ceuta
    {  534, 6773,    6367,  7, }, // Africa/Conakry
    {  534, 6840,   61161,  5, }, // Africa/Dakar
    {  534, 5130,    6374, 12, }, // Africa/Dar_es_Salaam
    {  534, 5043,   33286,  7, }, // Africa/Djibouti
    {  534, 5243,   21372,  5, }, // Africa/Douala
    {  534, 5166,    6393,  7, }, // Africa/El_Aaiun
    {  534, 6853,    6400,  7, }, // Africa/Freetown
    {  534, 4906,   61203,  7, }, // Africa/Gaborone
    {  534, 5015,   61211,  6, }, // Africa/Harare
    {  534, 4479,    6407, 11, }, // Africa/Johannesburg
    {  534, 4499,  124874,  5, }, // Africa/Juba
    {  534, 5151,  165374,  7, }, // Africa/Kampala
    {  534, 4524,    6423,  6, }, // Africa/Khartoum
    {  534, 4973,  124888,  6, }, // Africa/Kigali
    {  534, 5306,  122158,  7, }, // Africa/Kinshasa
    {  534, 4744,   61262,  5, }, // Africa/Lagos
    {  534, 5336,    6429,  8, }, // Africa/Libreville
    {  534, 6869,   61277,  4, }, // Africa/Lome
    {  534, 5211,  165381,  6, }, // Africa/Luanda
    {  534, 4939,  122180,  9, }, // Africa/Lubumbashi
    {  534, 5001,  165387,  6, }, // Africa/Lusaka
    {  534, 5322,  165393,  6, }, // Africa/Malabo
    {  534, 4987,  165399,  6, }, // Africa/Maputo
    {  534, 5197,   61303,  6, }, // Africa/Maseru
    {  534, 5182,  165405,  7, }, // Africa/Mbabane
    {  534, 5113,   33293,  7, }, // Africa/Mogadishu
    {  534, 7034,  122203,  9, }, // Africa/Monrovia
    {  534, 3853,   61333,  7, }, // Africa/Nairobi
    {  534, 5271,   57729,  8, }, // Africa/Ndjamena
    {  534, 5354,  165412,  7, }, // Africa/Niamey
    {  534, 6803,   57737,  7, }, // Africa/Nouakchott
    {  534, 6727,   26534,  8, }, // Africa/Ouagadougou
    {  534, 5225,   26542, 10, }, // Africa/Porto-Novo
    {  534, 4435,   91257,  8, }, // Africa/Sao_Tome
    {  534, 2866,   35888,  7, }, // Africa/Tripoli
    {  534, 6554,   91430,  5, }, // Africa/Tunis
    {  534, 4152,   21472,  7, }, // Africa/Windhoek
    {  534,  132,   97943,  4, }, // America/Adak
    {  534, 3346,  165419,  8, }, // America/Anchorage
    {  534, 5607,   33300,  7, }, // America/Anguilla
    {  534, 5624,   33307,  7, }, // America/Antigua
    {  534, 4554,  125041, 10, }, // America/Araguaina
    {  534,  166,  165427, 12, }, // America/Argentina/Buenos_Aires
    {  534,   91,    6518,  9, }, // America/Argentina/Catamarca
    {  534,  270,    6527,  7, }, // America/Argentina/Cordoba
    {  534,  441,  165439,  5, }, // America/Argentina/Jujuy
    {  534, 7462,    6539,  8, }, // America/Argentina/La_Rioja
    {  534,  565,   97634,  7, }, // America/Argentina/Mendoza
    {  534, 7489,  125081, 12, }, // America/Argentina/Rio_Gallegos
    {  534, 7520,   61439,  5, }, // America/Argentina/Salta
    {  534, 7544,  165444,  8, }, // America/Argentina/San_Juan
    {  534, 5580,  165452,  7, }, // America/Argentina/San_Luis
    {  534, 7571,    4842,  7, }, // America/Argentina/Tucuman
    {  534, 7597,  165459,  7, }, // America/Argentina/Ushuaia
    {  534, 5640,  165466,  5, }, // America/Aruba
    {  534, 4252,    6592,  8, }, // America/Asuncion
    {  534,  237,  165471, 12, }, // America/Atikokan
    {  534, 3679,  122369,  5, }, // America/Bahia
    {  534, 7623,  165483, 14, }, // America/Bahia_Banderas
    {  534, 5654,  122387,  8, }, // America/Barbados
    {  534, 7646,   61474,  5, }, // America/Belem
    {  534, 5407,  165497,  6, }, // America/Belize
    {  534, 7660,  165503, 13, }, // America/Blanc-Sablon
    {  534, 7681,  165516,  9, }, // America/Boa_Vista
    {  534, 4361,  165525,  6, }, // America/Bogota
    {  534, 7699,  165531,  5, }, // America/Boise
    {  534, 7713,  165536, 12, }, // America/Cambridge_Bay
    {  534, 7735,    4898, 12, }, // America/Campo_Grande
    {  534, 3868,    6682,  6, }, // America/Cancun
    {  534, 4694,    4910,  7, }, // America/Caracas
    {  534, 4345,    6688,  5, }, // America/Cayenne
    {  534, 5496,  165548, 16, }, // America/Cayman
    {  534, 2260,  165564,  6, }, // America/Chicago
    {  534, 7756,  125280,  8, }, // America/Chihuahua
    {  534, 7774,  165570, 13, }, // America/Ciudad_Juarez
    {  534, 5422,   33320,  9, }, // America/Costa_Rica
    {  534, 7796,    6736,  7, }, // America/Creston
    {  534, 3791,    4964,  6, }, // America/Cuiaba
    {  534, 5723,    6749,  7, }, // America/Curacao
    {  534, 7812,  165583, 13, }, // America/Danmarkshavn
    {  534, 7833,  125335,  5, }, // America/Dawson
    {  534, 7848,  125342, 10, }, // America/Dawson_Creek
    {  534,  805,   61557,  6, }, // America/Denver
    {  534, 3465,  122567,  7, }, // America/Detroit
    {  534, 5739,   21666,  8, }, // America/Dominica
    {  534,  893,   61580,  8, }, // America/Edmonton
    {  534, 7869,  165596,  8, }, // America/Eirunepe
    {  534, 5441,    6806,  8, }, // America/El_Salvador
    {  534, 7886,  125399, 11, }, // America/Fort_Nelson
    {  534, 7906,  165604,  9, }, // America/Fortaleza
    {  534, 7924,  165613,  9, }, // America/Glace_Bay
    {  534, 6881,  165622,  7, }, // America/Goose_Bay
    {  534, 4612,  165629, 10, }, // America/Grand_Turk
    {  534, 5770,  165639,  7, }, // America/Grenada
    {  534, 5786,   33329,  9, }, // America/Guadeloupe
    {  534, 3760,   33338,  9, }, // America/Guatemala
    {  534, 4841,  122647,  8, }, // America/Guayaquil
    {  534, 6932,   57584,  7, }, // America/Guyana
    {  534, 1939,    6905,  8, }, // America/Halifax
    {  534, 2281,  147788,  6, }, // America/Havana
    {  534, 7942,  165646, 10, }, // America/Hermosillo
    {  534,  348,  122680, 12, }, // America/Indiana/Indianapolis
    {  534,  481,  165669, 14, }, // America/Indiana/Knox
    {  534, 7961,  165683, 17, }, // America/Indiana/Marengo
    {  534, 7985,  165700, 20, }, // America/Indiana/Petersburg
    {  534, 8012,  165720, 18, }, // America/Indiana/Tell_City
    {  534, 8038,  166554, 15, }, // America/Indiana/Vevay
    {  534, 8060,  166569, 18, }, // America/Indiana/Vincennes
    {  534, 8086,  165769, 17, }, // America/Indiana/Winamac
    {  534, 8110,  122810,  6, }, // America/Inuvik
    {  534,  660,  165786,  8, }, // America/Iqaluit
    {  534, 2799,   33347,  7, }, // America/Jamaica
    {  534, 5380,  165794,  4, }, // America/Juneau
    {  534,  521,  166587,  7, }, // America/Kentucky/Louisville
    {  534, 8125,  165804, 18, }, // America/Kentucky/Monticello
    {  534, 5704,  165822, 10, }, // America/Kralendijk
    {  534, 4376,  165832,  6, }, // America/La_Paz
    {  534, 7169,   22041,  4, }, // America/Lima
    {  534, 3239,  165838, 11, }, // America/Los_Angeles
    {  534, 5932,  165849, 21, }, // America/Lower_Princes
    {  534, 8153,  122903,  6, }, // America/Maceio
    {  534, 8168,  122909,  7, }, // America/Managua
    {  534, 1908,  122916,  6, }, // America/Manaus
    {  534, 5897,   61871,  6, }, // America/Marigot
    {  534, 5805,    5260,  8, }, // America/Martinique
    {  534, 8184,  122931,  9, }, // America/Matamoros
    {  534, 2917,  165870,  8, }, // America/Mazatlan
    {  534, 8202,    7118,  8, }, // America/Menominee
    {  534, 8220,    7126,  6, }, // America/Merida
    {  534, 8235,  165878, 10, }, // America/Metlakatla
    {  534, 2949,  165888, 12, }, // America/Mexico_City
    {  534, 4391,    7149,  7, }, // America/Miquelon
    {  534, 8254,    7156,  7, }, // America/Moncton
    {  534, 8270,   61920,  8, }, // America/Monterrey
    {  534, 4098,  165900, 10, }, // America/Montevideo
    {  534, 5824,    7172,  9, }, // America/Montserrat
    {  534, 5481,  125812,  5, }, // America/Nassau
    {  534, 2379,  165910,  7, }, // America/New_York
    {  534, 8288,   21968,  3, }, // America/Nome
    {  534, 1850,    7194,  7, }, // America/Noronha
    {  534, 8301,  166594, 22, }, // America/North_Dakota/Beulah
    {  534, 8329,  166616, 22, }, // America/North_Dakota/Center
    {  534, 8357,  166638, 26, }, // America/North_Dakota/New_Salem
    {  534,  393,  165989,  6, }, // America/Nuuk
    {  534, 8388,    7276,  7, }, // America/Ojinaga
    {  534, 2356,  165995,  6, }, // America/Panama
    {  534, 6173,  166001, 10, }, // America/Paramaribo
    {  534, 2973,  125928,  6, }, // America/Phoenix
    {  534, 3945,  125936, 12, }, // America/Port-au-Prince
    {  534, 5954,  166664, 13, }, // America/Port_of_Spain
    {  534, 8404,  166036, 11, }, // America/Porto_Velho
    {  534, 5843,  166677, 10, }, // America/Puerto_Rico
    {  534, 4030,  125989, 12, }, // America/Punta_Arenas
    {  534, 8424,  126003, 12, }, // America/Rankin_Inlet
    {  534, 8445,  166047,  6, }, // America/Recife
    {  534, 1995,  166687,  6, }, // America/Regina
    {  534, 8460,  126030,  7, }, // America/Resolute
    {  534,  695,    5480, 10, }, // America/Rio_Branco
    {  534, 8477,  166068,  8, }, // America/Santarem
    {  534, 2201,  123192,  8, }, // America/Santiago
    {  534, 6111,  166076, 13, }, // America/Santo_Domingo
    {  534, 1878,  166089,  9, }, // America/Sao_Paulo
    {  534, 6899,  166693, 12, }, // America/Scoresbysund
    {  534, 8494,  166098,  5, }, // America/Sitka
    {  534, 8508,  166705, 14, }, // America/St_Barthelemy
    {  534, 2061,  166719,  8, }, // America/St_Johns
    {  534, 5863,  126132,  9, }, // America/St_Kitts
    {  534, 5880,  166103, 10, }, // America/St_Lucia
    {  534,  855,  166727,  8, }, // America/St_Thomas
    {  534, 5913,  166113, 12, }, // America/St_Vincent
    {  534, 8530,   22204, 12, }, // America/Swift_Current
    {  534, 5461,    5528, 11, }, // America/Tegucigalpa
    {  534, 5756,   22227,  3, }, // America/Thule
    {  534,  313,    7479,  7, }, // America/Tijuana
    {  534,  608,  166125,  7, }, // America/Toronto
    {  534, 5688,  166132,  7, }, // America/Tortola
    {  534, 2093,    7493,  8, }, // America/Vancouver
    {  534, 2144,  166139,  8, }, // America/Whitehorse
    {  534,  734,   62179,  7, }, // America/Winnipeg
    {  534, 8552,    7517,  7, }, // America/Yakutat
    {  534, 6001,    7524,  5, }, // Antarctica/Casey
    {  534, 6094,    7529,  6, }, // Antarctica/Davis
    {  534, 6133,  166735, 13, }, // Antarctica/DumontDUrville
    {  534, 8568,  166748,  7, }, // Antarctica/Macquarie
    {  534, 7066,  123375,  5, }, // Antarctica/Mawson
    {  534, 7113,  166160,  8, }, // Antarctica/McMurdo
    {  534, 8589,  126311,  6, }, // Antarctica/Palmer
    {  534, 7226,    5583,  6, }, // Antarctica/Rothera
    {  534, 7295,  123396,  4, }, // Antarctica/Syowa
    {  534, 8607,  123400,  4, }, // Antarctica/Troll
    {  534, 7374,  123406,  6, }, // Antarctica/Vostok
    {  534, 1429,  166755, 11, }, // Arctic/Longyearbyen
    {  534, 5570,   62217,  4, }, // Asia/Aden
    {  534, 5395,    7594,  6, }, // Asia/Almaty
    {  534, 3968,    5605,  4, }, // Asia/Amman
    {  534, 5511,    7600,  6, }, // Asia/Anadyr
    {  534, 5523,    5609,  5, }, // Asia/Aqtau
    {  534, 5368,   33435,  7, }, // Asia/Aqtobe
    {  534,  964,   33442,  7, }, // Asia/Ashgabat
    {  534, 8624,    7612,  6, }, // Asia/Atyrau
    {  534, 3607,    4162,  6, }, // Asia/Baghdad
    {  534, 5534,    4168,  7, }, // Asia/Bahrain
    {  534, 3653,   62255,  4, }, // Asia/Baku
    {  534, 4466,  123472,  7, }, // Asia/Bangkok
    {  534, 3571,  123480,  7, }, // Asia/Barnaul
    {  534, 4086,    7625,  6, }, // Asia/Beirut
    {  534, 3778,   33455,  6, }, // Asia/Bishkek
    {  534, 5989,   33461,  6, }, // Asia/Brunei
    {  534, 4601,   57622,  4, }, // Asia/Chita
    {  534, 4511,    5637,  7, }, // Asia/Colombo
    {  534, 4540,    4201,  6, }, // Asia/Damascus
    {  534, 1093,    4717,  4, }, // Asia/Dhaka
    {  534, 6192,  126467,  4, }, // Asia/Dili
    {  534, 3596,  123516,  5, }, // Asia/Dubai
    {  534, 6159,   33468,  7, }, // Asia/Dushanbe
    {  534, 8636,  166175,  9, }, // Asia/Famagusta
    {  534, 8651,   62313,  4, }, // Asia/Gaza
    {  534, 4795,   62318,  6, }, // Asia/Hebron
    {  534, 1259,   33475,  9, }, // Asia/Ho_Chi_Minh
    {  534, 2704,    4231,  8, }, // Asia/Hong_Kong
    {  534, 4771,  123546,  4, }, // Asia/Hovd
    {  534, 4184,  147155,  6, }, // Asia/Irkutsk
    {  534, 7006,   33484,  8, }, // Asia/Jakarta
    {  534, 6992,   33492,  9, }, // Asia/Jayapura
    {  534, 1290,  166184,  9, }, // Asia/Jerusalem
    {  534, 3560,   62370,  5, }, // Asia/Kabul
    {  534, 4316,   33510,  8, }, // Asia/Kamchatka
    {  534, 4239,   33518,  6, }, // Asia/Karachi
    {  534, 1185,    5711,  8, }, // Asia/Kathmandu
    {  534, 8661,   33524,  7, }, // Asia/Khandyga
    {  534,  992,   26864,  7, }, // Asia/Kolkata
    {  534, 4197,    4288, 11, }, // Asia/Krasnoyarsk
    {  534, 4859,  166193, 12, }, // Asia/Kuala_Lumpur
    {  534, 5976,   33531,  6, }, // Asia/Kuching
    {  534, 5547,    7725,  6, }, // Asia/Kuwait
    {  534, 1211,    7731,  5, }, // Asia/Macau
    {  534, 4017,  123625,  7, }, // Asia/Magadan
    {  534, 1349,    7736,  7, }, // Asia/Makassar
    {  534, 7182,  166205,  6, }, // Asia/Manila
    {  534, 6920,    4326,  6, }, // Asia/Muscat
    {  534, 2554,   33542,  8, }, // Asia/Nicosia
    {  534, 8675,   33550, 12, }, // Asia/Novokuznetsk
    {  534, 4135,   62477, 11, }, // Asia/Novosibirsk
    {  534, 4229,   62489,  4, }, // Asia/Omsk
    {  534, 7145,   62494,  4, }, // Asia/Oral
    {  534, 6961,   22518,  8, }, // Asia/Phnom_Penh
    {  534, 8693,  166211, 10, }, // Asia/Pontianak
    {  534, 4214,   33562,  9, }, // Asia/Pyongyang
    {  534, 5559,    4332,  5, }, // Asia/Qatar
    {  534, 8708,    7770,  8, }, // Asia/Qostanay
    {  534, 4269,    7778,  9, }, // Asia/Qyzylorda
    {  534, 3584,   33571,  5, }, // Asia/Riyadh
    {  534, 4408,   33576,  7, }, // Asia/Sakhalin
    {  534, 7245,   26906,  9, }, // Asia/Samarkand
    {  534, 3279,    7798,  4, }, // Asia/Seoul
    {  534, 1053,   33583,  6, }, // Asia/Shanghai
    {  534, 3300,   20900,  8, }, // Asia/Singapore
    {  534, 4297,  166221, 14, }, // Asia/Srednekolymsk
    {  534, 3263,    7830,  6, }, // Asia/Taipei
    {  534, 4781,   33589,  7, }, // Asia/Tashkent
    {  534, 3915,  126820,  7, }, // Asia/Tbilisi
    {  534, 2772,    7842,  7, }, // Asia/Tehran
    {  534, 1317,   91770,  5, }, // Asia/Thimphu
    {  534, 2821,    4356,  5, }, // Asia/Tokyo
    {  534, 4572,  123800,  5, }, // Asia/Tomsk
    {  534, 1021,   26923, 10, }, // Asia/Ulaanbaatar
    {  534, 1159,  166235,  6, }, // Asia/Urumqi
    {  534, 8722,  166241,  8, }, // Asia/Ust-Nera
    {  534, 6977,   33602, 10, }, // Asia/Vientiane
    {  534, 4710,   62644, 11, }, // Asia/Vladivostok
    {  534, 4828,    7873,  7, }, // Asia/Yakutsk
    {  534, 1235,    7880,  6, }, // Asia/Yangon
    {  534, 3883,    4380, 13, }, // Asia/Yekaterinburg
    {  534, 3747,    7898,  7, }, // Asia/Yerevan
    {  534, 3663,   33612,  5, }, // Atlantic/Azores
    {  534, 5671,   74483,  7, }, // Atlantic/Bermuda
    {  534, 6628,  166249, 15, }, // Atlantic/Canary
    {  534, 3727,  166264, 18, }, // Atlantic/Cape_Verde
    {  534, 1395,  166282, 13, }, // Atlantic/Faroe
    {  534, 8736,  166295,  7, }, // Atlantic/Madeira
    {  534, 2748,   33617,  9, }, // Atlantic/Reykjavik
    {  534, 7272,  166302, 15, }, // Atlantic/South_Georgia
    {  534, 6821,  166317, 12, }, // Atlantic/St_Helena
    {  534, 6644,    7965,  6, }, // Atlantic/Stanley
    {  534, 1670,  166329,  7, }, // Australia/Adelaide
    {  534, 1635,  166336,  8, }, // Australia/Brisbane
    {  534, 1799,  127037, 10, }, // Australia/Broken_Hill
    {  534, 1583,    7998,  6, }, // Australia/Darwin
    {  534, 3637,  166344,  5, }, // Australia/Eucla
    {  534, 1516,   62801,  6, }, // Australia/Hobart
    {  534, 8753,  124002,  8, }, // Australia/Lindeman
    {  534, 1547,   22727,  8, }, // Australia/Lord_Howe
    {  534, 1727,    8021,  7, }, // Australia/Melbourne
    {  534, 1762,   22735,  4, }, // Australia/Perth
    {  534, 1463,    8033,  6, }, // Australia/Sydney
    {  534, 6441,   62828,  9, }, // Europe/Amsterdam
    {  534, 6216,   33638,  6, }, // Europe/Andorra
    {  534, 3620,    8046,  8, }, // Europe/Astrakhan
    {  534, 2300,   33652,  5, }, // Europe/Athens
    {  534, 6336,    8060,  7, }, // Europe/Belgrade
    {  534, 4757,   62866,  6, }, // Europe/Berlin
    {  534, 6488,  166349, 10, }, // Europe/Bratislava
    {  534, 2167,   33657,  6, }, // Europe/Brussels
    {  534, 3928,   33663,  8, }, // Europe/Bucharest
    {  534, 3806,   33671, 10, }, // Europe/Budapest
    {  534, 8772,  166359,  8, }, // Europe/Busingen
    {  534, 2583,   33681,  8, }, // Europe/Chisinau
    {  534, 6289,    4475, 10, }, // Europe/Copenhagen
    {  534, 2338,   33689,  6, }, // Europe/Dublin
    {  534, 6307,  127223,  9, }, // Europe/Gibraltar
    {  534, 8788,   33695,  6, }, // Europe/Guernsey
    {  534, 6612,  127241,  8, }, // Europe/Helsinki
    {  534, 8804,   33701, 10, }, // Europe/Isle_of_Man
    {  534, 1130,   62990,  8, }, // Europe/Istanbul
    {  534, 8823,  166367,  6, }, // Europe/Jersey
    {  534, 3979,   33711, 12, }, // Europe/Kaliningrad
    {  534, 8929,  124195,  5, }, // Europe/Kirov
    {  534, 2527,   33723,  5, }, // Europe/Kyiv
    {  534, 3213,   33728,  7, }, // Europe/Lisbon
    {  534, 6506,  166373,  9, }, // Europe/Ljubljana
    {  534, 2501,   63039,  6, }, // Europe/London
    {  534, 6365,    8172, 10, }, // Europe/Luxembourg
    {  534, 4892,   63057,  6, }, // Europe/Madrid
    {  534, 6397,  166382,  5, }, // Europe/Malta
    {  534, 6582,   99286,  8, }, // Europe/Mariehamn
    {  534, 3693,   63069,  5, }, // Europe/Minsk
    {  534, 6410,    6044,  6, }, // Europe/Monaco
    {  534, 3537,   22880,  6, }, // Europe/Moscow
    {  534, 6458,  166387,  4, }, // Europe/Oslo
    {  534, 4284,   33735,  5, }, // Europe/Paris
    {  534, 6424,  166391,  9, }, // Europe/Podgorica
    {  534, 6275,    8201,  4, }, // Europe/Prague
    {  534, 8837,  166400,  4, }, // Europe/Riga
    {  534, 6324,   33740,  3, }, // Europe/Rome
    {  534, 4331,  166404,  6, }, // Europe/Samara
    {  534, 6470,  166410, 10, }, // Europe/San_Marino
    {  534, 6245,  166420,  8, }, // Europe/Sarajevo
    {  534, 4451,   78360,  7, }, // Europe/Saratov
    {  534, 8849,   33743, 11, }, // Europe/Simferopol
    {  534, 6383,   33754,  7, }, // Europe/Skopje
    {  534, 6599,   33761,  6, }, // Europe/Sofia
    {  534, 6523,    8242,  8, }, // Europe/Stockholm
    {  534, 8867,    8250,  5, }, // Europe/Tallinn
    {  534, 6202,   21213,  6, }, // Europe/Tirane
    {  534, 8882,    8261,  9, }, // Europe/Ulyanovsk
    {  534, 6352,   63171,  5, }, // Europe/Vaduz
    {  534, 6567,    6096,  7, }, // Europe/Vatican
    {  534, 6231,   33775,  3, }, // Europe/Vienna
    {  534, 8899,  166428,  7, }, // Europe/Vilnius
    {  534, 4727,   63187,  9, }, // Europe/Volgograd
    {  534, 3190,   33778,  7, }, // Europe/Warsaw
    {  534, 6261,   63205,  6, }, // Europe/Zagreb
    {  534, 6540,   33785,  5, }, // Europe/Zurich
    {  534, 5078,  166435, 12, }, // Indian/Antananarivo
    {  534, 6947,  124395,  5, }, // Indian/Chagos
    {  534, 6046,   33790,  5, }, // Indian/Christmas
    {  534, 6063,    4583,  5, }, // Indian/Cocos
    {  534, 5029,   22985,  6, }, // Indian/Comoro
    {  534, 6661,   22991,  8, }, // Indian/Kerguelen
    {  534, 7260,  158799,  4, }, // Indian/Mahe
    {  534, 7050,   33819,  7, }, // Indian/Maldives
    {  534, 4069,   33826, 10, }, // Indian/Mauritius
    {  534, 5098,    8318,  5, }, // Indian/Mayotte
    {  534, 7211,   91350,  7, }, // Indian/Reunion
    {  534, 4422,  124493,  5, }, // Pacific/Apia
    {  534,  932,    8324,  6, }, // Pacific/Auckland
    {  534, 3706,  166447,  8, }, // Pacific/Bougainville
    {  534, 3015,  127604,  5, }, // Pacific/Chatham
    {  534, 3157,   91357,  4, }, // Pacific/Chuuk
    {  534, 2237,  166455, 15, }, // Pacific/Easter
    {  534, 7360,  127627,  4, }, // Pacific/Efate
    {  534, 7327,  166470,  7, }, // Pacific/Fakaofo
    {  534, 3902,   33836,  5, }, // Pacific/Fiji
    {  534, 7343,   63335,  8, }, // Pacific/Funafuti
    {  534, 6678,   92233,  9, }, // Pacific/Galapagos
    {  534, 6696,  166477,  7, }, // Pacific/Gambier
    {  534, 3822,  124569, 11, }, // Pacific/Guadalcanal
    {  534, 6018,  124581,  4, }, // Pacific/Guam
    {  534, 2723,    4611,  8, }, // Pacific/Honolulu
    {  534, 3049,  124586,  9, }, // Pacific/Kanton
    {  534, 3998,   92267, 10, }, // Pacific/Kiritimati
    {  534, 7019,  166484,  5, }, // Pacific/Kosrae
    {  534, 2842,  166489, 11, }, // Pacific/Kwajalein
    {  534, 4877,   92315,  6, }, // Pacific/Majuro
    {  534, 4051,   36340,  6, }, // Pacific/Marquesas
    {  534, 8914,   63371,  6, }, // Pacific/Midway
    {  534, 7084,  166500,  5, }, // Pacific/Nauru
    {  534, 7132,  166766,  4, }, // Pacific/Niue
    {  534, 4168,     572,  7, }, // Pacific/Norfolk
    {  534, 7098,  124660,  5, }, // Pacific/Noumea
    {  534, 3126,  166510,  9, }, // Pacific/Pago_Pago
    {  534, 7155,   65776,  5, }, // Pacific/Palau
    {  534, 7194,    8422,  7, }, // Pacific/Pitcairn
    {  534, 3096,    8429,  6, }, // Pacific/Pohnpei
    {  534, 4807,  166519, 11, }, // Pacific/Port_Moresby
    {  534, 6076,  166530,  9, }, // Pacific/Rarotonga
    {  534, 6031,  166539,  6, }, // Pacific/Saipan
    {  534, 7312,   63456,  6, }, // Pacific/Tahiti
    {  534, 6712,   23159,  6, }, // Pacific/Tarawa
    {  534, 4583,  166545,  9, }, // Pacific/Tongatapu
    {  534, 7392,   63469,  4, }, // Pacific/Wake
    {  534, 7405,   33841,  5, }, // Pacific/Wallis
    {  535, 6788,   57696,  7, }, // Africa/Abidjan Serbian/Latin/Montenegro
    {  535, 6760,    4649,  4, }, // Africa/Accra
    {  535, 5059,   35821, 10, }, // Africa/Addis_Ababa
    {  535, 7420,   57579,  5, }, // Africa/Algiers
    {  535,   14,   91251,  6, }, // Africa/Asmara
    {  535,   44,  165341,  6, }, // Africa/Bamako
    {  535, 5257,  165347,  6, }, // Africa/Bangui
    {  535, 6746,  165353,  6, }, // Africa/Banjul
    {  535, 7435,  165359,  5, }, // Africa/Bissau
    {  535, 4957,    6318,  7, }, // Africa/Blantyre
    {  535, 5287,    6326,  8, }, // Africa/Brazzaville
    {  535, 4922,  165364, 10, }, // Africa/Bujumbura
    {  535, 2320,    4690,  5, }, // Africa/Cairo
    {  535, 4117,   33276, 10, }, // Africa/Casablanca
    {  535, 7449,   21349,  5, }, // Africa/Ceuta
    {  535, 6773,    6367,  7, }, // Africa/Conakry
    {  535, 6840,   61161,  5, }, // Africa/Dakar
    {  535, 5130,    6374, 12, }, // Africa/Dar_es_Salaam
    {  535, 5043,   33286,  7, }, // Africa/Djibouti
    {  535, 5243,   21372,  5, }, // Africa/Douala
    {  535, 5166,    6393,  7, }, // Africa/El_Aaiun
    {  535, 6853,    6400,  7, }, // Africa/Freetown
    {  535, 4906,   61203,  7, }, // Africa/Gaborone
    {  535, 5015,   61211,  6, }, // Africa/Harare
    {  535, 4479,    6407, 11, }, // Africa/Johannesburg
    {  535, 4499,  124874,  5, }, // Africa/Juba
    {  535, 5151,  165374,  7, }, // Africa/Kampala
    {  535, 4524,    6423,  6, }, // Africa/Khartoum
    {  535, 4973,  124888,  6, }, // Africa/Kigali
    {  535, 5306,  122158,  7, }, // Africa/Kinshasa
    {  535, 4744,   61262,  5, }, // Africa/Lagos
    {  535, 5336,    6429,  8, }, // Africa/Libreville
    {  535, 6869,   61277,  4, }, // Africa/Lome
    {  535, 5211,  165381,  6, }, // Africa/Luanda
    {  535, 4939,  122180,  9, }, // Africa/Lubumbashi
    {  535, 5001,  165387,  6, }, // Africa/Lusaka
    {  535, 5322,  165393,  6, }, // Africa/Malabo
    {  535, 4987,  165399,  6, }, // Africa/Maputo
    {  535, 5197,   61303,  6, }, // Africa/Maseru
    {  535, 5182,  165405,  7, }, // Africa/Mbabane
    {  535, 5113,   33293,  7, }, // Africa/Mogadishu
    {  535, 7034,  122203,  9, }, // Africa/Monrovia
    {  535, 3853,   61333,  7, }, // Africa/Nairobi
    {  535, 5271,   57729,  8, }, // Africa/Ndjamena
    {  535, 5354,  165412,  7, }, // Africa/Niamey
    {  535, 6803,   57737,  7, }, // Africa/Nouakchott
    {  535, 6727,   26534,  8, }, // Africa/Ouagadougou
    {  535, 5225,   26542, 10, }, // Africa/Porto-Novo
    {  535, 4435,   91257,  8, }, // Africa/Sao_Tome
    {  535, 2866,   35888,  7, }, // Africa/Tripoli
    {  535, 6554,   91430,  5, }, // Africa/Tunis
    {  535, 4152,   21472,  7, }, // Africa/Windhoek
    {  535,  132,   97943,  4, }, // America/Adak
    {  535, 3346,  165419,  8, }, // America/Anchorage
    {  535, 5607,   33300,  7, }, // America/Anguilla
    {  535, 5624,   33307,  7, }, // America/Antigua
    {  535, 4554,  125041, 10, }, // America/Araguaina
    {  535,  166,  165427, 12, }, // America/Argentina/Buenos_Aires
    {  535,   91,    6518,  9, }, // America/Argentina/Catamarca
    {  535,  270,    6527,  7, }, // America/Argentina/Cordoba
    {  535,  441,  165439,  5, }, // America/Argentina/Jujuy
    {  535, 7462,    6539,  8, }, // America/Argentina/La_Rioja
    {  535,  565,   97634,  7, }, // America/Argentina/Mendoza
    {  535, 7489,  125081, 12, }, // America/Argentina/Rio_Gallegos
    {  535, 7520,   61439,  5, }, // America/Argentina/Salta
    {  535, 7544,  165444,  8, }, // America/Argentina/San_Juan
    {  535, 5580,  165452,  7, }, // America/Argentina/San_Luis
    {  535, 7571,    4842,  7, }, // America/Argentina/Tucuman
    {  535, 7597,  165459,  7, }, // America/Argentina/Ushuaia
    {  535, 5640,  165466,  5, }, // America/Aruba
    {  535, 4252,    6592,  8, }, // America/Asuncion
    {  535,  237,  165471, 12, }, // America/Atikokan
    {  535, 3679,  122369,  5, }, // America/Bahia
    {  535, 7623,  165483, 14, }, // America/Bahia_Banderas
    {  535, 5654,  122387,  8, }, // America/Barbados
    {  535, 7646,   61474,  5, }, // America/Belem
    {  535, 5407,  165497,  6, }, // America/Belize
    {  535, 7660,  165503, 13, }, // America/Blanc-Sablon
    {  535, 7681,  165516,  9, }, // America/Boa_Vista
    {  535, 4361,  165525,  6, }, // America/Bogota
    {  535, 7699,  165531,  5, }, // America/Boise
    {  535, 7713,  165536, 12, }, // America/Cambridge_Bay
    {  535, 7735,    4898, 12, }, // America/Campo_Grande
    {  535, 3868,    6682,  6, }, // America/Cancun
    {  535, 4694,    4910,  7, }, // America/Caracas
    {  535, 4345,    6688,  5, }, // America/Cayenne
    {  535, 5496,  165548, 16, }, // America/Cayman
    {  535, 2260,  165564,  6, }, // America/Chicago
    {  535, 7756,  125280,  8, }, // America/Chihuahua
    {  535, 7774,  165570, 13, }, // America/Ciudad_Juarez
    {  535, 5422,   33320,  9, }, // America/Costa_Rica
    {  535, 7796,    6736,  7, }, // America/Creston
    {  535, 3791,    4964,  6, }, // America/Cuiaba
    {  535, 5723,    6749,  7, }, // America/Curacao
    {  535, 7812,  165583, 13, }, // America/Danmarkshavn
    {  535, 7833,  125335,  5, }, // America/Dawson
    {  535, 7848,  125342, 10, }, // America/Dawson_Creek
    {  535,  805,   61557,  6, }, // America/Denver
    {  535, 3465,  122567,  7, }, // America/Detroit
    {  535, 5739,   21666,  8, }, // America/Dominica
    {  535,  893,   61580,  8, }, // America/Edmonton
    {  535, 7869,  165596,  8, }, // America/Eirunepe
    {  535, 5441,    6806,  8, }, // America/El_Salvador
    {  535, 7886,  125399, 11, }, // America/Fort_Nelson
    {  535, 7906,  165604,  9, }, // America/Fortaleza
    {  535, 7924,  165613,  9, }, // America/Glace_Bay
    {  535, 6881,  165622,  7, }, // America/Goose_Bay
    {  535, 4612,  165629, 10, }, // America/Grand_Turk
    {  535, 5770,  165639,  7, }, // America/Grenada
    {  535, 5786,   33329,  9, }, // America/Guadeloupe
    {  535, 3760,   33338,  9, }, // America/Guatemala
    {  535, 4841,  122647,  8, }, // America/Guayaquil
    {  535, 6932,   57584,  7, }, // America/Guyana
    {  535, 1939,    6905,  8, }, // America/Halifax
    {  535, 2281,  147788,  6, }, // America/Havana
    {  535, 7942,  165646, 10, }, // America/Hermosillo
    {  535,  348,  122680, 12, }, // America/Indiana/Indianapolis
    {  535,  481,  165669, 14, }, // America/Indiana/Knox
    {  535, 7961,  165683, 17, }, // America/Indiana/Marengo
    {  535, 7985,  165700, 20, }, // America/Indiana/Petersburg
    {  535, 8012,  165720, 18, }, // America/Indiana/Tell_City
    {  535, 8038,  166554, 15, }, // America/Indiana/Vevay
    {  535, 8060,  166569, 18, }, // America/Indiana/Vincennes
    {  535, 8086,  165769, 17, }, // America/Indiana/Winamac
    {  535, 8110,  122810,  6, }, // America/Inuvik
    {  535,  660,  165786,  8, }, // America/Iqaluit
    {  535, 2799,   33347,  7, }, // America/Jamaica
    {  535, 5380,  165794,  4, }, // America/Juneau
    {  535,  521,  166587,  7, }, // America/Kentucky/Louisville
    {  535, 8125,  165804, 18, }, // America/Kentucky/Monticello
    {  535, 5704,  165822, 10, }, // America/Kralendijk
    {  535, 4376,  165832,  6, }, // America/La_Paz
    {  535, 7169,   22041,  4, }, // America/Lima
    {  535, 3239,  165838, 11, }, // America/Los_Angeles
    {  535, 5932,  165849, 21, }, // America/Lower_Princes
    {  535, 8153,  122903,  6, }, // America/Maceio
    {  535, 8168,  122909,  7, }, // America/Managua
    {  535, 1908,  122916,  6, }, // America/Manaus
    {  535, 5897,   61871,  6, }, // America/Marigot
    {  535, 5805,    5260,  8, }, // America/Martinique
    {  535, 8184,  122931,  9, }, // America/Matamoros
    {  535, 2917,  165870,  8, }, // America/Mazatlan
    {  535, 8202,    7118,  8, }, // America/Menominee
    {  535, 8220,    7126,  6, }, // America/Merida
    {  535, 8235,  165878, 10, }, // America/Metlakatla
    {  535, 2949,  165888, 12, }, // America/Mexico_City
    {  535, 4391,    7149,  7, }, // America/Miquelon
    {  535, 8254,    7156,  7, }, // America/Moncton
    {  535, 8270,   61920,  8, }, // America/Monterrey
    {  535, 4098,  165900, 10, }, // America/Montevideo
    {  535, 5824,    7172,  9, }, // America/Montserrat
    {  535, 5481,  125812,  5, }, // America/Nassau
    {  535, 2379,  165910,  7, }, // America/New_York
    {  535, 8288,   21968,  3, }, // America/Nome
    {  535, 1850,    7194,  7, }, // America/Noronha
    {  535, 8301,  166594, 22, }, // America/North_Dakota/Beulah
    {  535, 8329,  166616, 22, }, // America/North_Dakota/Center
    {  535, 8357,  166638, 26, }, // America/North_Dakota/New_Salem
    {  535,  393,  165989,  6, }, // America/Nuuk
    {  535, 8388,    7276,  7, }, // America/Ojinaga
    {  535, 2356,  165995,  6, }, // America/Panama
    {  535, 6173,  166001, 10, }, // America/Paramaribo
    {  535, 2973,  125928,  6, }, // America/Phoenix
    {  535, 3945,  125936, 12, }, // America/Port-au-Prince
    {  535, 5954,  166664, 13, }, // America/Port_of_Spain
    {  535, 8404,  166036, 11, }, // America/Porto_Velho
    {  535, 5843,  166677, 10, }, // America/Puerto_Rico
    {  535, 4030,  125989, 12, }, // America/Punta_Arenas
    {  535, 8424,  126003, 12, }, // America/Rankin_Inlet
    {  535, 8445,  166047,  6, }, // America/Recife
    {  535, 1995,  166687,  6, }, // America/Regina
    {  535, 8460,  126030,  7, }, // America/Resolute
    {  535,  695,    5480, 10, }, // America/Rio_Branco
    {  535, 8477,  166068,  8, }, // America/Santarem
    {  535, 2201,  123192,  8, }, // America/Santiago
    {  535, 6111,  166076, 13, }, // America/Santo_Domingo
    {  535, 1878,  166089,  9, }, // America/Sao_Paulo
    {  535, 6899,  166693, 12, }, // America/Scoresbysund
    {  535, 8494,  166098,  5, }, // America/Sitka
    {  535, 8508,  166705, 14, }, // America/St_Barthelemy
    {  535, 2061,  166719,  8, }, // America/St_Johns
    {  535, 5863,  126132,  9, }, // America/St_Kitts
    {  535, 5880,  166103, 10, }, // America/St_Lucia
    {  535,  855,  166727,  8, }, // America/St_Thomas
    {  535, 5913,  166113, 12, }, // America/St_Vincent
    {  535, 8530,   22204, 12, }, // America/Swift_Current
    {  535, 5461,    5528, 11, }, // America/Tegucigalpa
    {  535, 5756,   22227,  3, }, // America/Thule
    {  535,  313,    7479,  7, }, // America/Tijuana
    {  535,  608,  166125,  7, }, // America/Toronto
    {  535, 5688,  166132,  7, }, // America/Tortola
    {  535, 2093,    7493,  8, }, // America/Vancouver
    {  535, 2144,  166139,  8, }, // America/Whitehorse
    {  535,  734,   62179,  7, }, // America/Winnipeg
    {  535, 8552,    7517,  7, }, // America/Yakutat
    {  535, 6001,    7524,  5, }, // Antarctica/Casey
    {  535, 6094,    7529,  6, }, // Antarctica/Davis
    {  535, 6133,  166735, 13, }, // Antarctica/DumontDUrville
    {  535, 8568,  166748,  7, }, // Antarctica/Macquarie
    {  535, 7066,  123375,  5, }, // Antarctica/Mawson
    {  535, 7113,  166160,  8, }, // Antarctica/McMurdo
    {  535, 8589,  126311,  6, }, // Antarctica/Palmer
    {  535, 7226,    5583,  6, }, // Antarctica/Rothera
    {  535, 7295,  123396,  4, }, // Antarctica/Syowa
    {  535, 8607,  123400,  4, }, // Antarctica/Troll
    {  535, 7374,  123406,  6, }, // Antarctica/Vostok
    {  535, 1429,  166755, 11, }, // Arctic/Longyearbyen
    {  535, 5570,   62217,  4, }, // Asia/Aden
    {  535, 5395,    7594,  6, }, // Asia/Almaty
    {  535, 3968,    5605,  4, }, // Asia/Amman
    {  535, 5511,    7600,  6, }, // Asia/Anadyr
    {  535, 5523,    5609,  5, }, // Asia/Aqtau
    {  535, 5368,   33435,  7, }, // Asia/Aqtobe
    {  535,  964,   33442,  7, }, // Asia/Ashgabat
    {  535, 8624,    7612,  6, }, // Asia/Atyrau
    {  535, 3607,    4162,  6, }, // Asia/Baghdad
    {  535, 5534,    4168,  7, }, // Asia/Bahrain
    {  535, 3653,   62255,  4, }, // Asia/Baku
    {  535, 4466,  123472,  7, }, // Asia/Bangkok
    {  535, 3571,  123480,  7, }, // Asia/Barnaul
    {  535, 4086,    7625,  6, }, // Asia/Beirut
    {  535, 3778,   33455,  6, }, // Asia/Bishkek
    {  535, 5989,   33461,  6, }, // Asia/Brunei
    {  535, 4601,   57622,  4, }, // Asia/Chita
    {  535, 4511,    5637,  7, }, // Asia/Colombo
    {  535, 4540,    4201,  6, }, // Asia/Damascus
    {  535, 1093,    4717,  4, }, // Asia/Dhaka
    {  535, 6192,  126467,  4, }, // Asia/Dili
    {  535, 3596,  123516,  5, }, // Asia/Dubai
    {  535, 6159,   33468,  7, }, // Asia/Dushanbe
    {  535, 8636,  166175,  9, }, // Asia/Famagusta
    {  535, 8651,   62313,  4, }, // Asia/Gaza
    {  535, 4795,   62318,  6, }, // Asia/Hebron
    {  535, 1259,   33475,  9, }, // Asia/Ho_Chi_Minh
    {  535, 2704,    4231,  8, }, // Asia/Hong_Kong
    {  535, 4771,  123546,  4, }, // Asia/Hovd
    {  535, 4184,  147155,  6, }, // Asia/Irkutsk
    {  535, 7006,   33484,  8, }, // Asia/Jakarta
    {  535, 6992,   33492,  9, }, // Asia/Jayapura
    {  535, 1290,  166184,  9, }, // Asia/Jerusalem
    {  535, 3560,   62370,  5, }, // Asia/Kabul
    {  535, 4316,   33510,  8, }, // Asia/Kamchatka
    {  535, 4239,   33518,  6, }, // Asia/Karachi
    {  535, 1185,    5711,  8, }, // Asia/Kathmandu
    {  535, 8661,   33524,  7, }, // Asia/Khandyga
    {  535,  992,   26864,  7, }, // Asia/Kolkata
    {  535, 4197,    4288, 11, }, // Asia/Krasnoyarsk
    {  535, 4859,  166193, 12, }, // Asia/Kuala_Lumpur
    {  535, 5976,   33531,  6, }, // Asia/Kuching
    {  535, 5547,    7725,  6, }, // Asia/Kuwait
    {  535, 1211,    7731,  5, }, // Asia/Macau
    {  535, 4017,  123625,  7, }, // Asia/Magadan
    {  535, 1349,    7736,  7, }, // Asia/Makassar
    {  535, 7182,  166205,  6, }, // Asia/Manila
    {  535, 6920,    4326,  6, }, // Asia/Muscat
    {  535, 2554,   33542,  8, }, // Asia/Nicosia
    {  535, 8675,   33550, 12, }, // Asia/Novokuznetsk
    {  535, 4135,   62477, 11, }, // Asia/Novosibirsk
    {  535, 4229,   62489,  4, }, // Asia/Omsk
    {  535, 7145,   62494,  4, }, // Asia/Oral
    {  535, 6961,   22518,  8, }, // Asia/Phnom_Penh
    {  535, 8693,  166211, 10, }, // Asia/Pontianak
    {  535, 4214,   33562,  9, }, // Asia/Pyongyang
    {  535, 5559,    4332,  5, }, // Asia/Qatar
    {  535, 8708,    7770,  8, }, // Asia/Qostanay
    {  535, 4269,    7778,  9, }, // Asia/Qyzylorda
    {  535, 3584,   33571,  5, }, // Asia/Riyadh
    {  535, 4408,   33576,  7, }, // Asia/Sakhalin
    {  535, 7245,   26906,  9, }, // Asia/Samarkand
    {  535, 3279,    7798,  4, }, // Asia/Seoul
    {  535, 1053,   33583,  6, }, // Asia/Shanghai
    {  535, 3300,   20900,  8, }, // Asia/Singapore
    {  535, 4297,  166221, 14, }, // Asia/Srednekolymsk
    {  535, 3263,    7830,  6, }, // Asia/Taipei
    {  535, 4781,   33589,  7, }, // Asia/Tashkent
    {  535, 3915,  126820,  7, }, // Asia/Tbilisi
    {  535, 2772,    7842,  7, }, // Asia/Tehran
    {  535, 1317,   91770,  5, }, // Asia/Thimphu
    {  535, 2821,    4356,  5, }, // Asia/Tokyo
    {  535, 4572,  123800,  5, }, // Asia/Tomsk
    {  535, 1021,   26923, 10, }, // Asia/Ulaanbaatar
    {  535, 1159,  166235,  6, }, // Asia/Urumqi
    {  535, 8722,  166241,  8, }, // Asia/Ust-Nera
    {  535, 6977,   33602, 10, }, // Asia/Vientiane
    {  535, 4710,   62644, 11, }, // Asia/Vladivostok
    {  535, 4828,    7873,  7, }, // Asia/Yakutsk
    {  535, 1235,    7880,  6, }, // Asia/Yangon
    {  535, 3883,    4380, 13, }, // Asia/Yekaterinburg
    {  535, 3747,    7898,  7, }, // Asia/Yerevan
    {  535, 3663,   33612,  5, }, // Atlantic/Azores
    {  535, 5671,   74483,  7, }, // Atlantic/Bermuda
    {  535, 6628,  166249, 15, }, // Atlantic/Canary
    {  535, 3727,  166264, 18, }, // Atlantic/Cape_Verde
    {  535, 1395,  166282, 13, }, // Atlantic/Faroe
    {  535, 8736,  166295,  7, }, // Atlantic/Madeira
    {  535, 2748,   33617,  9, }, // Atlantic/Reykjavik
    {  535, 7272,  166302, 15, }, // Atlantic/South_Georgia
    {  535, 6821,  166317, 12, }, // Atlantic/St_Helena
    {  535, 6644,    7965,  6, }, // Atlantic/Stanley
    {  535, 1670,  166329,  7, }, // Australia/Adelaide
    {  535, 1635,  166336,  8, }, // Australia/Brisbane
    {  535, 1799,  127037, 10, }, // Australia/Broken_Hill
    {  535, 1583,    7998,  6, }, // Australia/Darwin
    {  535, 3637,  166344,  5, }, // Australia/Eucla
    {  535, 1516,   62801,  6, }, // Australia/Hobart
    {  535, 8753,  124002,  8, }, // Australia/Lindeman
    {  535, 1547,   22727,  8, }, // Australia/Lord_Howe
    {  535, 1727,    8021,  7, }, // Australia/Melbourne
    {  535, 1762,   22735,  4, }, // Australia/Perth
    {  535, 1463,    8033,  6, }, // Australia/Sydney
    {  535, 6441,   62828,  9, }, // Europe/Amsterdam
    {  535, 6216,   33638,  6, }, // Europe/Andorra
    {  535, 3620,    8046,  8, }, // Europe/Astrakhan
    {  535, 2300,   33652,  5, }, // Europe/Athens
    {  535, 6336,    8060,  7, }, // Europe/Belgrade
    {  535, 4757,   62866,  6, }, // Europe/Berlin
    {  535, 6488,  166349, 10, }, // Europe/Bratislava
    {  535, 2167,   33657,  6, }, // Europe/Brussels
    {  535, 3928,   33663,  8, }, // Europe/Bucharest
    {  535, 3806,   33671, 10, }, // Europe/Budapest
    {  535, 8772,  166359,  8, }, // Europe/Busingen
    {  535, 2583,   33681,  8, }, // Europe/Chisinau
    {  535, 6289,    4475, 10, }, // Europe/Copenhagen
    {  535, 2338,   33689,  6, }, // Europe/Dublin
    {  535, 6307,  127223,  9, }, // Europe/Gibraltar
    {  535, 8788,   33695,  6, }, // Europe/Guernsey
    {  535, 6612,  127241,  8, }, // Europe/Helsinki
    {  535, 8804,   33701, 10, }, // Europe/Isle_of_Man
    {  535, 1130,   62990,  8, }, // Europe/Istanbul
    {  535, 8823,  166367,  6, }, // Europe/Jersey
    {  535, 3979,   33711, 12, }, // Europe/Kaliningrad
    {  535, 8929,  124195,  5, }, // Europe/Kirov
    {  535, 2527,   33723,  5, }, // Europe/Kyiv
    {  535, 3213,   33728,  7, }, // Europe/Lisbon
    {  535, 6506,  166373,  9, }, // Europe/Ljubljana
    {  535, 2501,   63039,  6, }, // Europe/London
    {  535, 6365,    8172, 10, }, // Europe/Luxembourg
    {  535, 4892,   63057,  6, }, // Europe/Madrid
    {  535, 6397,  166382,  5, }, // Europe/Malta
    {  535, 6582,   99286,  8, }, // Europe/Mariehamn
    {  535, 3693,   63069,  5, }, // Europe/Minsk
    {  535, 6410,    6044,  6, }, // Europe/Monaco
    {  535, 3537,   22880,  6, }, // Europe/Moscow
    {  535, 6458,  166387,  4, }, // Europe/Oslo
    {  535, 4284,   33735,  5, }, // Europe/Paris
    {  535, 6424,  166391,  9, }, // Europe/Podgorica
    {  535, 6275,    8201,  4, }, // Europe/Prague
    {  535, 8837,  166400,  4, }, // Europe/Riga
    {  535, 6324,   33740,  3, }, // Europe/Rome
    {  535, 4331,  166404,  6, }, // Europe/Samara
    {  535, 6470,  166410, 10, }, // Europe/San_Marino
    {  535, 6245,  166420,  8, }, // Europe/Sarajevo
    {  535, 4451,   78360,  7, }, // Europe/Saratov
    {  535, 8849,   33743, 11, }, // Europe/Simferopol
    {  535, 6383,   33754,  7, }, // Europe/Skopje
    {  535, 6599,   33761,  6, }, // Europe/Sofia
    {  535, 6523,    8242,  8, }, // Europe/Stockholm
    {  535, 8867,    8250,  5, }, // Europe/Tallinn
    {  535, 6202,   21213,  6, }, // Europe/Tirane
    {  535, 8882,    8261,  9, }, // Europe/Ulyanovsk
    {  535, 6352,   63171,  5, }, // Europe/Vaduz
    {  535, 6567,    6096,  7, }, // Europe/Vatican
    {  535, 6231,   33775,  3, }, // Europe/Vienna
    {  535, 8899,  166428,  7, }, // Europe/Vilnius
    {  535, 4727,   63187,  9, }, // Europe/Volgograd
    {  535, 3190,   33778,  7, }, // Europe/Warsaw
    {  535, 6261,   63205,  6, }, // Europe/Zagreb
    {  535, 6540,   33785,  5, }, // Europe/Zurich
    {  535, 5078,  166435, 12, }, // Indian/Antananarivo
    {  535, 6947,  124395,  5, }, // Indian/Chagos
    {  535, 6046,   33790,  5, }, // Indian/Christmas
    {  535, 6063,    4583,  5, }, // Indian/Cocos
    {  535, 5029,   22985,  6, }, // Indian/Comoro
    {  535, 6661,   22991,  8, }, // Indian/Kerguelen
    {  535, 7260,  158799,  4, }, // Indian/Mahe
    {  535, 7050,   33819,  7, }, // Indian/Maldives
    {  535, 4069,   33826, 10, }, // Indian/Mauritius
    {  535, 5098,    8318,  5, }, // Indian/Mayotte
    {  535, 7211,   91350,  7, }, // Indian/Reunion
    {  535, 4422,  124493,  5, }, // Pacific/Apia
    {  535,  932,    8324,  6, }, // Pacific/Auckland
    {  535, 3706,  166447,  8, }, // Pacific/Bougainville
    {  535, 3015,  127604,  5, }, // Pacific/Chatham
    {  535, 3157,   91357,  4, }, // Pacific/Chuuk
    {  535, 2237,  166455, 15, }, // Pacific/Easter
    {  535, 7360,  127627,  4, }, // Pacific/Efate
    {  535, 7327,  166470,  7, }, // Pacific/Fakaofo
    {  535, 3902,   33836,  5, }, // Pacific/Fiji
    {  535, 7343,   63335,  8, }, // Pacific/Funafuti
    {  535, 6678,   92233,  9, }, // Pacific/Galapagos
    {  535, 6696,  166477,  7, }, // Pacific/Gambier
    {  535, 3822,  124569, 11, }, // Pacific/Guadalcanal
    {  535, 6018,  124581,  4, }, // Pacific/Guam
    {  535, 2723,    4611,  8, }, // Pacific/Honolulu
    {  535, 3049,  124586,  9, }, // Pacific/Kanton
    {  535, 3998,   92267, 10, }, // Pacific/Kiritimati
    {  535, 7019,  166484,  5, }, // Pacific/Kosrae
    {  535, 2842,  166489, 11, }, // Pacific/Kwajalein
    {  535, 4877,   92315,  6, }, // Pacific/Majuro
    {  535, 4051,   36340,  6, }, // Pacific/Marquesas
    {  535, 8914,   63371,  6, }, // Pacific/Midway
    {  535, 7084,  166500,  5, }, // Pacific/Nauru
    {  535, 7132,  166766,  4, }, // Pacific/Niue
    {  535, 4168,     572,  7, }, // Pacific/Norfolk
    {  535, 7098,  124660,  5, }, // Pacific/Noumea
    {  535, 3126,  166510,  9, }, // Pacific/Pago_Pago
    {  535, 7155,   65776,  5, }, // Pacific/Palau
    {  535, 7194,    8422,  7, }, // Pacific/Pitcairn
    {  535, 3096,    8429,  6, }, // Pacific/Pohnpei
    {  535, 4807,  166519, 11, }, // Pacific/Port_Moresby
    {  535, 6076,  166530,  9, }, // Pacific/Rarotonga
    {  535, 6031,  166539,  6, }, // Pacific/Saipan
    {  535, 7312,   63456,  6, }, // Pacific/Tahiti
    {  535, 6712,   23159,  6, }, // Pacific/Tarawa
    {  535, 4583,  166545,  9, }, // Pacific/Tongatapu
    {  535, 7392,   63469,  4, }, // Pacific/Wake
    {  535, 7405,   33841,  5, }, // Pacific/Wallis
    {  536, 6788,   57696,  7, }, // Africa/Abidjan Serbian/Latin/Serbia
    {  536, 6760,    4649,  4, }, // Africa/Accra
    {  536, 5059,   35821, 10, }, // Africa/Addis_Ababa
    {  536, 7420,   57579,  5, }, // Africa/Algiers
    {  536,   14,   91251,  6, }, // Africa/Asmara
    {  536,   44,  165341,  6, }, // Africa/Bamako
    {  536, 5257,  165347,  6, }, // Africa/Bangui
    {  536, 6746,  165353,  6, }, // Africa/Banjul
    {  536, 7435,  165359,  5, }, // Africa/Bissau
    {  536, 4957,    6318,  7, }, // Africa/Blantyre
    {  536, 5287,    6326,  8, }, // Africa/Brazzaville
    {  536, 4922,  165364, 10, }, // Africa/Bujumbura
    {  536, 2320,    4690,  5, }, // Africa/Cairo
    {  536, 4117,   33276, 10, }, // Africa/Casablanca
    {  536, 7449,   21349,  5, }, // Africa/Ceuta
    {  536, 6773,    6367,  7, }, // Africa/Conakry
    {  536, 6840,   61161,  5, }, // Africa/Dakar
    {  536, 5130,    6374, 12, }, // Africa/Dar_es_Salaam
    {  536, 5043,   33286,  7, }, // Africa/Djibouti
    {  536, 5243,   21372,  5, }, // Africa/Douala
    {  536, 5166,    6393,  7, }, // Africa/El_Aaiun
    {  536, 6853,    6400,  7, }, // Africa/Freetown
    {  536, 4906,   61203,  7, }, // Africa/Gaborone
    {  536, 5015,   61211,  6, }, // Africa/Harare
    {  536, 4479,    6407, 11, }, // Africa/Johannesburg
    {  536, 4499,  124874,  5, }, // Africa/Juba
    {  536, 5151,  165374,  7, }, // Africa/Kampala
    {  536, 4524,    6423,  6, }, // Africa/Khartoum
    {  536, 4973,  124888,  6, }, // Africa/Kigali
    {  536, 5306,  122158,  7, }, // Africa/Kinshasa
    {  536, 4744,   61262,  5, }, // Africa/Lagos
    {  536, 5336,    6429,  8, }, // Africa/Libreville
    {  536, 6869,   61277,  4, }, // Africa/Lome
    {  536, 5211,  165381,  6, }, // Africa/Luanda
    {  536, 4939,  122180,  9, }, // Africa/Lubumbashi
    {  536, 5001,  165387,  6, }, // Africa/Lusaka
    {  536, 5322,  165393,  6, }, // Africa/Malabo
    {  536, 4987,  165399,  6, }, // Africa/Maputo
    {  536, 5197,   61303,  6, }, // Africa/Maseru
    {  536, 5182,  165405,  7, }, // Africa/Mbabane
    {  536, 5113,   33293,  7, }, // Africa/Mogadishu
    {  536, 7034,  122203,  9, }, // Africa/Monrovia
    {  536, 3853,   61333,  7, }, // Africa/Nairobi
    {  536, 5271,   57729,  8, }, // Africa/Ndjamena
    {  536, 5354,  165412,  7, }, // Africa/Niamey
    {  536, 6803,   57737,  7, }, // Africa/Nouakchott
    {  536, 6727,   26534,  8, }, // Africa/Ouagadougou
    {  536, 5225,   26542, 10, }, // Africa/Porto-Novo
    {  536, 4435,   91257,  8, }, // Africa/Sao_Tome
    {  536, 2866,   35888,  7, }, // Africa/Tripoli
    {  536, 6554,   91430,  5, }, // Africa/Tunis
    {  536, 4152,   21472,  7, }, // Africa/Windhoek
    {  536,  132,   97943,  4, }, // America/Adak
    {  536, 3346,  165419,  8, }, // America/Anchorage
    {  536, 5607,   33300,  7, }, // America/Anguilla
    {  536, 5624,   33307,  7, }, // America/Antigua
    {  536, 4554,  125041, 10, }, // America/Araguaina
    {  536,  166,  165427, 12, }, // America/Argentina/Buenos_Aires
    {  536,   91,    6518,  9, }, // America/Argentina/Catamarca
    {  536,  270,    6527,  7, }, // America/Argentina/Cordoba
    {  536,  441,  165439,  5, }, // America/Argentina/Jujuy
    {  536, 7462,    6539,  8, }, // America/Argentina/La_Rioja
    {  536,  565,   97634,  7, }, // America/Argentina/Mendoza
    {  536, 7489,  125081, 12, }, // America/Argentina/Rio_Gallegos
    {  536, 7520,   61439,  5, }, // America/Argentina/Salta
    {  536, 7544,  165444,  8, }, // America/Argentina/San_Juan
    {  536, 5580,  165452,  7, }, // America/Argentina/San_Luis
    {  536, 7571,    4842,  7, }, // America/Argentina/Tucuman
    {  536, 7597,  165459,  7, }, // America/Argentina/Ushuaia
    {  536, 5640,  165466,  5, }, // America/Aruba
    {  536, 4252,    6592,  8, }, // America/Asuncion
    {  536,  237,  165471, 12, }, // America/Atikokan
    {  536, 3679,  122369,  5, }, // America/Bahia
    {  536, 7623,  165483, 14, }, // America/Bahia_Banderas
    {  536, 5654,  122387,  8, }, // America/Barbados
    {  536, 7646,   61474,  5, }, // America/Belem
    {  536, 5407,  165497,  6, }, // America/Belize
    {  536, 7660,  165503, 13, }, // America/Blanc-Sablon
    {  536, 7681,  165516,  9, }, // America/Boa_Vista
    {  536, 4361,  165525,  6, }, // America/Bogota
    {  536, 7699,  165531,  5, }, // America/Boise
    {  536, 7713,  165536, 12, }, // America/Cambridge_Bay
    {  536, 7735,    4898, 12, }, // America/Campo_Grande
    {  536, 3868,    6682,  6, }, // America/Cancun
    {  536, 4694,    4910,  7, }, // America/Caracas
    {  536, 4345,    6688,  5, }, // America/Cayenne
    {  536, 5496,  165548, 16, }, // America/Cayman
    {  536, 2260,  165564,  6, }, // America/Chicago
    {  536, 7756,  125280,  8, }, // America/Chihuahua
    {  536, 7774,  165570, 13, }, // America/Ciudad_Juarez
    {  536, 5422,   33320,  9, }, // America/Costa_Rica
    {  536, 7796,    6736,  7, }, // America/Creston
    {  536, 3791,    4964,  6, }, // America/Cuiaba
    {  536, 5723,    6749,  7, }, // America/Curacao
    {  536, 7812,  165583, 13, }, // America/Danmarkshavn
    {  536, 7833,  125335,  5, }, // America/Dawson
    {  536, 7848,  125342, 10, }, // America/Dawson_Creek
    {  536,  805,   61557,  6, }, // America/Denver
    {  536, 3465,  122567,  7, }, // America/Detroit
    {  536, 5739,   21666,  8, }, // America/Dominica
    {  536,  893,   61580,  8, }, // America/Edmonton
    {  536, 7869,  165596,  8, }, // America/Eirunepe
    {  536, 5441,    6806,  8, }, // America/El_Salvador
    {  536, 7886,  125399, 11, }, // America/Fort_Nelson
    {  536, 7906,  165604,  9, }, // America/Fortaleza
    {  536, 7924,  165613,  9, }, // America/Glace_Bay
    {  536, 6881,  165622,  7, }, // America/Goose_Bay
    {  536, 4612,  165629, 10, }, // America/Grand_Turk
    {  536, 5770,  165639,  7, }, // America/Grenada
    {  536, 5786,   33329,  9, }, // America/Guadeloupe
    {  536, 3760,   33338,  9, }, // America/Guatemala
    {  536, 4841,  122647,  8, }, // America/Guayaquil
    {  536, 6932,   57584,  7, }, // America/Guyana
    {  536, 1939,    6905,  8, }, // America/Halifax
    {  536, 2281,  147788,  6, }, // America/Havana
    {  536, 7942,  165646, 10, }, // America/Hermosillo
    {  536,  348,  122680, 12, }, // America/Indiana/Indianapolis
    {  536,  481,  165669, 14, }, // America/Indiana/Knox
    {  536, 7961,  165683, 17, }, // America/Indiana/Marengo
    {  536, 7985,  165700, 20, }, // America/Indiana/Petersburg
    {  536, 8012,  165720, 18, }, // America/Indiana/Tell_City
    {  536, 8038,  166554, 15, }, // America/Indiana/Vevay
    {  536, 8060,  166569, 18, }, // America/Indiana/Vincennes
    {  536, 8086,  165769, 17, }, // America/Indiana/Winamac
    {  536, 8110,  122810,  6, }, // America/Inuvik
    {  536,  660,  165786,  8, }, // America/Iqaluit
    {  536, 2799,   33347,  7, }, // America/Jamaica
    {  536, 5380,  165794,  4, }, // America/Juneau
    {  536,  521,  166587,  7, }, // America/Kentucky/Louisville
    {  536, 8125,  165804, 18, }, // America/Kentucky/Monticello
    {  536, 5704,  165822, 10, }, // America/Kralendijk
    {  536, 4376,  165832,  6, }, // America/La_Paz
    {  536, 7169,   22041,  4, }, // America/Lima
    {  536, 3239,  165838, 11, }, // America/Los_Angeles
    {  536, 5932,  165849, 21, }, // America/Lower_Princes
    {  536, 8153,  122903,  6, }, // America/Maceio
    {  536, 8168,  122909,  7, }, // America/Managua
    {  536, 1908,  122916,  6, }, // America/Manaus
    {  536, 5897,   61871,  6, }, // America/Marigot
    {  536, 5805,    5260,  8, }, // America/Martinique
    {  536, 8184,  122931,  9, }, // America/Matamoros
    {  536, 2917,  165870,  8, }, // America/Mazatlan
    {  536, 8202,    7118,  8, }, // America/Menominee
    {  536, 8220,    7126,  6, }, // America/Merida
    {  536, 8235,  165878, 10, }, // America/Metlakatla
    {  536, 2949,  165888, 12, }, // America/Mexico_City
    {  536, 4391,    7149,  7, }, // America/Miquelon
    {  536, 8254,    7156,  7, }, // America/Moncton
    {  536, 8270,   61920,  8, }, // America/Monterrey
    {  536, 4098,  165900, 10, }, // America/Montevideo
    {  536, 5824,    7172,  9, }, // America/Montserrat
    {  536, 5481,  125812,  5, }, // America/Nassau
    {  536, 2379,  165910,  7, }, // America/New_York
    {  536, 8288,   21968,  3, }, // America/Nome
    {  536, 1850,    7194,  7, }, // America/Noronha
    {  536, 8301,  166594, 22, }, // America/North_Dakota/Beulah
    {  536, 8329,  166616, 22, }, // America/North_Dakota/Center
    {  536, 8357,  166638, 26, }, // America/North_Dakota/New_Salem
    {  536,  393,  165989,  6, }, // America/Nuuk
    {  536, 8388,    7276,  7, }, // America/Ojinaga
    {  536, 2356,  165995,  6, }, // America/Panama
    {  536, 6173,  166001, 10, }, // America/Paramaribo
    {  536, 2973,  125928,  6, }, // America/Phoenix
    {  536, 3945,  125936, 12, }, // America/Port-au-Prince
    {  536, 5954,  166664, 13, }, // America/Port_of_Spain
    {  536, 8404,  166036, 11, }, // America/Porto_Velho
    {  536, 5843,  166677, 10, }, // America/Puerto_Rico
    {  536, 4030,  125989, 12, }, // America/Punta_Arenas
    {  536, 8424,  126003, 12, }, // America/Rankin_Inlet
    {  536, 8445,  166047,  6, }, // America/Recife
    {  536, 1995,  166687,  6, }, // America/Regina
    {  536, 8460,  126030,  7, }, // America/Resolute
    {  536,  695,    5480, 10, }, // America/Rio_Branco
    {  536, 8477,  166068,  8, }, // America/Santarem
    {  536, 2201,  123192,  8, }, // America/Santiago
    {  536, 6111,  166076, 13, }, // America/Santo_Domingo
    {  536, 1878,  166089,  9, }, // America/Sao_Paulo
    {  536, 6899,  166693, 12, }, // America/Scoresbysund
    {  536, 8494,  166098,  5, }, // America/Sitka
    {  536, 8508,  166705, 14, }, // America/St_Barthelemy
    {  536, 2061,  166719,  8, }, // America/St_Johns
    {  536, 5863,  126132,  9, }, // America/St_Kitts
    {  536, 5880,  166103, 10, }, // America/St_Lucia
    {  536,  855,  166727,  8, }, // America/St_Thomas
    {  536, 5913,  166113, 12, }, // America/St_Vincent
    {  536, 8530,   22204, 12, }, // America/Swift_Current
    {  536, 5461,    5528, 11, }, // America/Tegucigalpa
    {  536, 5756,   22227,  3, }, // America/Thule
    {  536,  313,    7479,  7, }, // America/Tijuana
    {  536,  608,  166125,  7, }, // America/Toronto
    {  536, 5688,  166132,  7, }, // America/Tortola
    {  536, 2093,    7493,  8, }, // America/Vancouver
    {  536, 2144,  166139,  8, }, // America/Whitehorse
    {  536,  734,   62179,  7, }, // America/Winnipeg
    {  536, 8552,    7517,  7, }, // America/Yakutat
    {  536, 6001,    7524,  5, }, // Antarctica/Casey
    {  536, 6094,    7529,  6, }, // Antarctica/Davis
    {  536, 6133,  166735, 13, }, // Antarctica/DumontDUrville
    {  536, 8568,  166748,  7, }, // Antarctica/Macquarie
    {  536, 7066,  123375,  5, }, // Antarctica/Mawson
    {  536, 7113,  166160,  8, }, // Antarctica/McMurdo
    {  536, 8589,  126311,  6, }, // Antarctica/Palmer
    {  536, 7226,    5583,  6, }, // Antarctica/Rothera
    {  536, 7295,  123396,  4, }, // Antarctica/Syowa
    {  536, 8607,  123400,  4, }, // Antarctica/Troll
    {  536, 7374,  123406,  6, }, // Antarctica/Vostok
    {  536, 1429,  166755, 11, }, // Arctic/Longyearbyen
    {  536, 5570,   62217,  4, }, // Asia/Aden
    {  536, 5395,    7594,  6, }, // Asia/Almaty
    {  536, 3968,    5605,  4, }, // Asia/Amman
    {  536, 5511,    7600,  6, }, // Asia/Anadyr
    {  536, 5523,    5609,  5, }, // Asia/Aqtau
    {  536, 5368,   33435,  7, }, // Asia/Aqtobe
    {  536,  964,   33442,  7, }, // Asia/Ashgabat
    {  536, 8624,    7612,  6, }, // Asia/Atyrau
    {  536, 3607,    4162,  6, }, // Asia/Baghdad
    {  536, 5534,    4168,  7, }, // Asia/Bahrain
    {  536, 3653,   62255,  4, }, // Asia/Baku
    {  536, 4466,  123472,  7, }, // Asia/Bangkok
    {  536, 3571,  123480,  7, }, // Asia/Barnaul
    {  536, 4086,    7625,  6, }, // Asia/Beirut
    {  536, 3778,   33455,  6, }, // Asia/Bishkek
    {  536, 5989,   33461,  6, }, // Asia/Brunei
    {  536, 4601,   57622,  4, }, // Asia/Chita
    {  536, 4511,    5637,  7, }, // Asia/Colombo
    {  536, 4540,    4201,  6, }, // Asia/Damascus
    {  536, 1093,    4717,  4, }, // Asia/Dhaka
    {  536, 6192,  126467,  4, }, // Asia/Dili
    {  536, 3596,  123516,  5, }, // Asia/Dubai
    {  536, 6159,   33468,  7, }, // Asia/Dushanbe
    {  536, 8636,  166175,  9, }, // Asia/Famagusta
    {  536, 8651,   62313,  4, }, // Asia/Gaza
    {  536, 4795,   62318,  6, }, // Asia/Hebron
    {  536, 1259,   33475,  9, }, // Asia/Ho_Chi_Minh
    {  536, 2704,    4231,  8, }, // Asia/Hong_Kong
    {  536, 4771,  123546,  4, }, // Asia/Hovd
    {  536, 4184,  147155,  6, }, // Asia/Irkutsk
    {  536, 7006,   33484,  8, }, // Asia/Jakarta
    {  536, 6992,   33492,  9, }, // Asia/Jayapura
    {  536, 1290,  166184,  9, }, // Asia/Jerusalem
    {  536, 3560,   62370,  5, }, // Asia/Kabul
    {  536, 4316,   33510,  8, }, // Asia/Kamchatka
    {  536, 4239,   33518,  6, }, // Asia/Karachi
    {  536, 1185,    5711,  8, }, // Asia/Kathmandu
    {  536, 8661,   33524,  7, }, // Asia/Khandyga
    {  536,  992,   26864,  7, }, // Asia/Kolkata
    {  536, 4197,    4288, 11, }, // Asia/Krasnoyarsk
    {  536, 4859,  166193, 12, }, // Asia/Kuala_Lumpur
    {  536, 5976,   33531,  6, }, // Asia/Kuching
    {  536, 5547,    7725,  6, }, // Asia/Kuwait
    {  536, 1211,    7731,  5, }, // Asia/Macau
    {  536, 4017,  123625,  7, }, // Asia/Magadan
    {  536, 1349,    7736,  7, }, // Asia/Makassar
    {  536, 7182,  166205,  6, }, // Asia/Manila
    {  536, 6920,    4326,  6, }, // Asia/Muscat
    {  536, 2554,   33542,  8, }, // Asia/Nicosia
    {  536, 8675,   33550, 12, }, // Asia/Novokuznetsk
    {  536, 4135,   62477, 11, }, // Asia/Novosibirsk
    {  536, 4229,   62489,  4, }, // Asia/Omsk
    {  536, 7145,   62494,  4, }, // Asia/Oral
    {  536, 6961,   22518,  8, }, // Asia/Phnom_Penh
    {  536, 8693,  166211, 10, }, // Asia/Pontianak
    {  536, 4214,   33562,  9, }, // Asia/Pyongyang
    {  536, 5559,    4332,  5, }, // Asia/Qatar
    {  536, 8708,    7770,  8, }, // Asia/Qostanay
    {  536, 4269,    7778,  9, }, // Asia/Qyzylorda
    {  536, 3584,   33571,  5, }, // Asia/Riyadh
    {  536, 4408,   33576,  7, }, // Asia/Sakhalin
    {  536, 7245,   26906,  9, }, // Asia/Samarkand
    {  536, 3279,    7798,  4, }, // Asia/Seoul
    {  536, 1053,   33583,  6, }, // Asia/Shanghai
    {  536, 3300,   20900,  8, }, // Asia/Singapore
    {  536, 4297,  166221, 14, }, // Asia/Srednekolymsk
    {  536, 3263,    7830,  6, }, // Asia/Taipei
    {  536, 4781,   33589,  7, }, // Asia/Tashkent
    {  536, 3915,  126820,  7, }, // Asia/Tbilisi
    {  536, 2772,    7842,  7, }, // Asia/Tehran
    {  536, 1317,   91770,  5, }, // Asia/Thimphu
    {  536, 2821,    4356,  5, }, // Asia/Tokyo
    {  536, 4572,  123800,  5, }, // Asia/Tomsk
    {  536, 1021,   26923, 10, }, // Asia/Ulaanbaatar
    {  536, 1159,  166235,  6, }, // Asia/Urumqi
    {  536, 8722,  166241,  8, }, // Asia/Ust-Nera
    {  536, 6977,   33602, 10, }, // Asia/Vientiane
    {  536, 4710,   62644, 11, }, // Asia/Vladivostok
    {  536, 4828,    7873,  7, }, // Asia/Yakutsk
    {  536, 1235,    7880,  6, }, // Asia/Yangon
    {  536, 3883,    4380, 13, }, // Asia/Yekaterinburg
    {  536, 3747,    7898,  7, }, // Asia/Yerevan
    {  536, 3663,   33612,  5, }, // Atlantic/Azores
    {  536, 5671,   74483,  7, }, // Atlantic/Bermuda
    {  536, 6628,  166249, 15, }, // Atlantic/Canary
    {  536, 3727,  166264, 18, }, // Atlantic/Cape_Verde
    {  536, 1395,  166282, 13, }, // Atlantic/Faroe
    {  536, 8736,  166295,  7, }, // Atlantic/Madeira
    {  536, 2748,   33617,  9, }, // Atlantic/Reykjavik
    {  536, 7272,  166302, 15, }, // Atlantic/South_Georgia
    {  536, 6821,  166317, 12, }, // Atlantic/St_Helena
    {  536, 6644,    7965,  6, }, // Atlantic/Stanley
    {  536, 1670,  166329,  7, }, // Australia/Adelaide
    {  536, 1635,  166336,  8, }, // Australia/Brisbane
    {  536, 1799,  127037, 10, }, // Australia/Broken_Hill
    {  536, 1583,    7998,  6, }, // Australia/Darwin
    {  536, 3637,  166344,  5, }, // Australia/Eucla
    {  536, 1516,   62801,  6, }, // Australia/Hobart
    {  536, 8753,  124002,  8, }, // Australia/Lindeman
    {  536, 1547,   22727,  8, }, // Australia/Lord_Howe
    {  536, 1727,    8021,  7, }, // Australia/Melbourne
    {  536, 1762,   22735,  4, }, // Australia/Perth
    {  536, 1463,    8033,  6, }, // Australia/Sydney
    {  536, 6441,   62828,  9, }, // Europe/Amsterdam
    {  536, 6216,   33638,  6, }, // Europe/Andorra
    {  536, 3620,    8046,  8, }, // Europe/Astrakhan
    {  536, 2300,   33652,  5, }, // Europe/Athens
    {  536, 6336,    8060,  7, }, // Europe/Belgrade
    {  536, 4757,   62866,  6, }, // Europe/Berlin
    {  536, 6488,  166349, 10, }, // Europe/Bratislava
    {  536, 2167,   33657,  6, }, // Europe/Brussels
    {  536, 3928,   33663,  8, }, // Europe/Bucharest
    {  536, 3806,   33671, 10, }, // Europe/Budapest
    {  536, 8772,  166359,  8, }, // Europe/Busingen
    {  536, 2583,   33681,  8, }, // Europe/Chisinau
    {  536, 6289,    4475, 10, }, // Europe/Copenhagen
    {  536, 2338,   33689,  6, }, // Europe/Dublin
    {  536, 6307,  127223,  9, }, // Europe/Gibraltar
    {  536, 8788,   33695,  6, }, // Europe/Guernsey
    {  536, 6612,  127241,  8, }, // Europe/Helsinki
    {  536, 8804,   33701, 10, }, // Europe/Isle_of_Man
    {  536, 1130,   62990,  8, }, // Europe/Istanbul
    {  536, 8823,  166367,  6, }, // Europe/Jersey
    {  536, 3979,   33711, 12, }, // Europe/Kaliningrad
    {  536, 8929,  124195,  5, }, // Europe/Kirov
    {  536, 2527,   33723,  5, }, // Europe/Kyiv
    {  536, 3213,   33728,  7, }, // Europe/Lisbon
    {  536, 6506,  166373,  9, }, // Europe/Ljubljana
    {  536, 2501,   63039,  6, }, // Europe/London
    {  536, 6365,    8172, 10, }, // Europe/Luxembourg
    {  536, 4892,   63057,  6, }, // Europe/Madrid
    {  536, 6397,  166382,  5, }, // Europe/Malta
    {  536, 6582,   99286,  8, }, // Europe/Mariehamn
    {  536, 3693,   63069,  5, }, // Europe/Minsk
    {  536, 6410,    6044,  6, }, // Europe/Monaco
    {  536, 3537,   22880,  6, }, // Europe/Moscow
    {  536, 6458,  166387,  4, }, // Europe/Oslo
    {  536, 4284,   33735,  5, }, // Europe/Paris
    {  536, 6424,  166391,  9, }, // Europe/Podgorica
    {  536, 6275,    8201,  4, }, // Europe/Prague
    {  536, 8837,  166400,  4, }, // Europe/Riga
    {  536, 6324,   33740,  3, }, // Europe/Rome
    {  536, 4331,  166404,  6, }, // Europe/Samara
    {  536, 6470,  166410, 10, }, // Europe/San_Marino
    {  536, 6245,  166420,  8, }, // Europe/Sarajevo
    {  536, 4451,   78360,  7, }, // Europe/Saratov
    {  536, 8849,   33743, 11, }, // Europe/Simferopol
    {  536, 6383,   33754,  7, }, // Europe/Skopje
    {  536, 6599,   33761,  6, }, // Europe/Sofia
    {  536, 6523,    8242,  8, }, // Europe/Stockholm
    {  536, 8867,    8250,  5, }, // Europe/Tallinn
    {  536, 6202,   21213,  6, }, // Europe/Tirane
    {  536, 8882,    8261,  9, }, // Europe/Ulyanovsk
    {  536, 6352,   63171,  5, }, // Europe/Vaduz
    {  536, 6567,    6096,  7, }, // Europe/Vatican
    {  536, 6231,   33775,  3, }, // Europe/Vienna
    {  536, 8899,  166428,  7, }, // Europe/Vilnius
    {  536, 4727,   63187,  9, }, // Europe/Volgograd
    {  536, 3190,   33778,  7, }, // Europe/Warsaw
    {  536, 6261,   63205,  6, }, // Europe/Zagreb
    {  536, 6540,   33785,  5, }, // Europe/Zurich
    {  536, 5078,  166435, 12, }, // Indian/Antananarivo
    {  536, 6947,  124395,  5, }, // Indian/Chagos
    {  536, 6046,   33790,  5, }, // Indian/Christmas
    {  536, 6063,    4583,  5, }, // Indian/Cocos
    {  536, 5029,   22985,  6, }, // Indian/Comoro
    {  536, 6661,   22991,  8, }, // Indian/Kerguelen
    {  536, 7260,  158799,  4, }, // Indian/Mahe
    {  536, 7050,   33819,  7, }, // Indian/Maldives
    {  536, 4069,   33826, 10, }, // Indian/Mauritius
    {  536, 5098,    8318,  5, }, // Indian/Mayotte
    {  536, 7211,   91350,  7, }, // Indian/Reunion
    {  536, 4422,  124493,  5, }, // Pacific/Apia
    {  536,  932,    8324,  6, }, // Pacific/Auckland
    {  536, 3706,  166447,  8, }, // Pacific/Bougainville
    {  536, 3015,  127604,  5, }, // Pacific/Chatham
    {  536, 3157,   91357,  4, }, // Pacific/Chuuk
    {  536, 2237,  166455, 15, }, // Pacific/Easter
    {  536, 7360,  127627,  4, }, // Pacific/Efate
    {  536, 7327,  166470,  7, }, // Pacific/Fakaofo
    {  536, 3902,   33836,  5, }, // Pacific/Fiji
    {  536, 7343,   63335,  8, }, // Pacific/Funafuti
    {  536, 6678,   92233,  9, }, // Pacific/Galapagos
    {  536, 6696,  166477,  7, }, // Pacific/Gambier
    {  536, 3822,  124569, 11, }, // Pacific/Guadalcanal
    {  536, 6018,  124581,  4, }, // Pacific/Guam
    {  536, 2723,    4611,  8, }, // Pacific/Honolulu
    {  536, 3049,  124586,  9, }, // Pacific/Kanton
    {  536, 3998,   92267, 10, }, // Pacific/Kiritimati
    {  536, 7019,  166484,  5, }, // Pacific/Kosrae
    {  536, 2842,  166489, 11, }, // Pacific/Kwajalein
    {  536, 4877,   92315,  6, }, // Pacific/Majuro
    {  536, 4051,   36340,  6, }, // Pacific/Marquesas
    {  536, 8914,   63371,  6, }, // Pacific/Midway
    {  536, 7084,  166500,  5, }, // Pacific/Nauru
    {  536, 7132,  166766,  4, }, // Pacific/Niue
    {  536, 4168,     572,  7, }, // Pacific/Norfolk
    {  536, 7098,  124660,  5, }, // Pacific/Noumea
    {  536, 3126,  166510,  9, }, // Pacific/Pago_Pago
    {  536, 7155,   65776,  5, }, // Pacific/Palau
    {  536, 7194,    8422,  7, }, // Pacific/Pitcairn
    {  536, 3096,    8429,  6, }, // Pacific/Pohnpei
    {  536, 4807,  166519, 11, }, // Pacific/Port_Moresby
    {  536, 6076,  166530,  9, }, // Pacific/Rarotonga
    {  536, 6031,  166539,  6, }, // Pacific/Saipan
    {  536, 7312,   63456,  6, }, // Pacific/Tahiti
    {  536, 6712,   23159,  6, }, // Pacific/Tarawa
    {  536, 4583,  166545,  9, }, // Pacific/Tongatapu
    {  536, 7392,   63469,  4, }, // Pacific/Wake
    {  536, 7405,   33841,  5, }, // Pacific/Wallis
    {  537,   14,       0,  6, }, // Africa/Asmara Shambala/Latin/Tanzania
    {  537, 4435,       6,  8, }, // Africa/Sao_Tome
    {  537, 4252,      14,  8, }, // America/Asuncion
    {  537,  237,      22,  8, }, // America/Atikokan
    {  537, 7623,      30, 17, }, // America/Bahia_Banderas
    {  537, 3868,      47,  6, }, // America/Cancun
    {  537, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  537, 5723,      66,  7, }, // America/Curacao
    {  537,  481,      73, 13, }, // America/Indiana/Knox
    {  537, 7961,      86, 16, }, // America/Indiana/Marengo
    {  537, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  537, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  537, 8038,     139, 14, }, // America/Indiana/Vevay
    {  537, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  537, 8086,     171, 16, }, // America/Indiana/Winamac
    {  537, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  537, 5932,     207, 22, }, // America/Lower_Princes
    {  537, 8220,     229,  6, }, // America/Merida
    {  537, 1850,     235, 19, }, // America/Noronha
    {  537, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  537, 8329,     274, 20, }, // America/North_Dakota/Center
    {  537, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  537,  393,     317,  4, }, // America/Nuuk
    {  537, 6899,     321, 16, }, // America/Scoresbysund
    {  537, 8508,     337, 14, }, // America/St_Barthelemy
    {  537, 2061,     351, 10, }, // America/St_Johns
    {  537, 5863,     361,  9, }, // America/St_Kitts
    {  537, 5880,     370,  9, }, // America/St_Lucia
    {  537,  855,     379, 10, }, // America/St_Thomas
    {  537, 5913,     389, 11, }, // America/St_Vincent
    {  537, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  537, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  537, 1185,     448,  9, }, // Asia/Kathmandu
    {  537,  992,     457,  7, }, // Asia/Kolkata
    {  537, 1211,     464,  5, }, // Asia/Macau
    {  537, 1235,     477,  6, }, // Asia/Yangon
    {  537, 1395,     483,  5, }, // Atlantic/Faroe
    {  537, 6821,     488, 10, }, // Atlantic/St_Helena
    {  537, 2527,     514,  4, }, // Europe/Kyiv
    {  537, 7211,     547,  7, }, // Indian/Reunion
    {  537, 3157,     554,  5, }, // Pacific/Chuuk
    {  537, 3096,     586,  7, }, // Pacific/Pohnpei
    {  538,   14,       0,  6, }, // Africa/Asmara Shona/Latin/Zimbabwe
    {  538, 4435,       6,  8, }, // Africa/Sao_Tome
    {  538, 4252,      14,  8, }, // America/Asuncion
    {  538,  237,      22,  8, }, // America/Atikokan
    {  538, 7623,      30, 17, }, // America/Bahia_Banderas
    {  538, 3868,      47,  6, }, // America/Cancun
    {  538, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  538, 5723,      66,  7, }, // America/Curacao
    {  538,  481,      73, 13, }, // America/Indiana/Knox
    {  538, 7961,      86, 16, }, // America/Indiana/Marengo
    {  538, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  538, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  538, 8038,     139, 14, }, // America/Indiana/Vevay
    {  538, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  538, 8086,     171, 16, }, // America/Indiana/Winamac
    {  538, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  538, 5932,     207, 22, }, // America/Lower_Princes
    {  538, 8220,     229,  6, }, // America/Merida
    {  538, 1850,     235, 19, }, // America/Noronha
    {  538, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  538, 8329,     274, 20, }, // America/North_Dakota/Center
    {  538, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  538,  393,     317,  4, }, // America/Nuuk
    {  538, 6899,     321, 16, }, // America/Scoresbysund
    {  538, 8508,     337, 14, }, // America/St_Barthelemy
    {  538, 2061,     351, 10, }, // America/St_Johns
    {  538, 5863,     361,  9, }, // America/St_Kitts
    {  538, 5880,     370,  9, }, // America/St_Lucia
    {  538,  855,     379, 10, }, // America/St_Thomas
    {  538, 5913,     389, 11, }, // America/St_Vincent
    {  538, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  538, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  538, 1185,     448,  9, }, // Asia/Kathmandu
    {  538,  992,     457,  7, }, // Asia/Kolkata
    {  538, 1211,     464,  5, }, // Asia/Macau
    {  538, 1235,     477,  6, }, // Asia/Yangon
    {  538, 1395,     483,  5, }, // Atlantic/Faroe
    {  538, 6821,     488, 10, }, // Atlantic/St_Helena
    {  538, 2527,     514,  4, }, // Europe/Kyiv
    {  538, 7211,     547,  7, }, // Indian/Reunion
    {  538, 3157,     554,  5, }, // Pacific/Chuuk
    {  538, 3096,     586,  7, }, // Pacific/Pohnpei
    {  539,   14,       0,  6, }, // Africa/Asmara Sichuan Yi/Yi/China
    {  539, 4435,       6,  8, }, // Africa/Sao_Tome
    {  539, 4252,      14,  8, }, // America/Asuncion
    {  539,  237,      22,  8, }, // America/Atikokan
    {  539, 7623,      30, 17, }, // America/Bahia_Banderas
    {  539, 3868,      47,  6, }, // America/Cancun
    {  539, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  539, 5723,      66,  7, }, // America/Curacao
    {  539,  481,      73, 13, }, // America/Indiana/Knox
    {  539, 7961,      86, 16, }, // America/Indiana/Marengo
    {  539, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  539, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  539, 8038,     139, 14, }, // America/Indiana/Vevay
    {  539, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  539, 8086,     171, 16, }, // America/Indiana/Winamac
    {  539, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  539, 5932,     207, 22, }, // America/Lower_Princes
    {  539, 8220,     229,  6, }, // America/Merida
    {  539, 1850,     235, 19, }, // America/Noronha
    {  539, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  539, 8329,     274, 20, }, // America/North_Dakota/Center
    {  539, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  539,  393,     317,  4, }, // America/Nuuk
    {  539, 6899,     321, 16, }, // America/Scoresbysund
    {  539, 8508,     337, 14, }, // America/St_Barthelemy
    {  539, 2061,     351, 10, }, // America/St_Johns
    {  539, 5863,     361,  9, }, // America/St_Kitts
    {  539, 5880,     370,  9, }, // America/St_Lucia
    {  539,  855,     379, 10, }, // America/St_Thomas
    {  539, 5913,     389, 11, }, // America/St_Vincent
    {  539, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  539, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  539, 1185,     448,  9, }, // Asia/Kathmandu
    {  539,  992,     457,  7, }, // Asia/Kolkata
    {  539, 1211,     464,  5, }, // Asia/Macau
    {  539, 1235,     477,  6, }, // Asia/Yangon
    {  539, 1395,     483,  5, }, // Atlantic/Faroe
    {  539, 6821,     488, 10, }, // Atlantic/St_Helena
    {  539, 2527,     514,  4, }, // Europe/Kyiv
    {  539, 7211,     547,  7, }, // Indian/Reunion
    {  539, 3157,     554,  5, }, // Pacific/Chuuk
    {  539, 3096,     586,  7, }, // Pacific/Pohnpei
    {  540, 7420,   95064,  6, }, // Africa/Algiers Sicilian/Latin/Italy
    {  540,   14,       0,  6, }, // Africa/Asmara
    {  540, 2320,   20388,  5, }, // Africa/Cairo
    {  540, 4117,  166770, 11, }, // Africa/Casablanca
    {  540, 7449,  166781,  5, }, // Africa/Ceuta
    {  540, 5043,  166786,  7, }, // Africa/Djibouti
    {  540, 5113,  166793, 10, }, // Africa/Mogadishu
    {  540, 3853,  166803,  8, }, // Africa/Nairobi
    {  540, 4435,       6,  8, }, // Africa/Sao_Tome
    {  540, 2866,  166811,  7, }, // Africa/Tripoli
    {  540, 6554,  164493,  6, }, // Africa/Tunis
    {  540,  270,  166818,  7, }, // America/Argentina/Cordoba
    {  540, 4252,      14,  8, }, // America/Asuncion
    {  540,  237,      22,  8, }, // America/Atikokan
    {  540, 7623,      30, 17, }, // America/Bahia_Banderas
    {  540, 5407,  166825,  6, }, // America/Belize
    {  540, 7735,  166831, 11, }, // America/Campo_Grande
    {  540, 3868,      47,  6, }, // America/Cancun
    {  540, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  540, 5422,  166842, 10, }, // America/Costa_Rica
    {  540, 5723,      66,  7, }, // America/Curacao
    {  540, 3465,  166852,  7, }, // America/Detroit
    {  540, 5739,  166859,  8, }, // America/Dominica
    {  540, 5786,   95109,  9, }, // America/Guadeloupe
    {  540, 3760,   98027,  9, }, // America/Guatemala
    {  540, 6932,  158047,  6, }, // America/Guyana
    {  540,  481,      73, 13, }, // America/Indiana/Knox
    {  540, 7961,      86, 16, }, // America/Indiana/Marengo
    {  540, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  540, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  540, 8038,     139, 14, }, // America/Indiana/Vevay
    {  540, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  540, 8086,     171, 16, }, // America/Indiana/Winamac
    {  540, 2799,   95125,  8, }, // America/Jamaica
    {  540, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  540, 5932,     207, 22, }, // America/Lower_Princes
    {  540, 5805,   20554,  9, }, // America/Martinique
    {  540, 8220,     229,  6, }, // America/Merida
    {  540, 2949,  166867, 15, }, // America/Mexico_City
    {  540, 8270,  166882,  9, }, // America/Monterrey
    {  540, 5824,  166891, 10, }, // America/Montserrat
    {  540, 2379,  166901, 10, }, // America/New_York
    {  540, 1850,     235, 19, }, // America/Noronha
    {  540, 8301,  166911, 28, }, // America/North_Dakota/Beulah
    {  540, 8329,  166939, 28, }, // America/North_Dakota/Center
    {  540, 8357,  166967, 31, }, // America/North_Dakota/New_Salem
    {  540,  393,     317,  4, }, // America/Nuuk
    {  540, 2356,  164613,  6, }, // America/Panama
    {  540, 5954,  166998, 15, }, // America/Port_of_Spain
    {  540, 5843,  167013, 10, }, // America/Puerto_Rico
    {  540, 1995,  167023,  7, }, // America/Regina
    {  540, 6111,   98166, 13, }, // America/Santo_Domingo
    {  540, 1878,  167030,  9, }, // America/Sao_Paulo
    {  540, 6899,     321, 16, }, // America/Scoresbysund
    {  540, 8508,     337, 14, }, // America/St_Barthelemy
    {  540, 2061,     351, 10, }, // America/St_Johns
    {  540, 5863,     361,  9, }, // America/St_Kitts
    {  540, 5880,  167039, 11, }, // America/St_Lucia
    {  540,  855,     379, 10, }, // America/St_Thomas
    {  540, 5913,     389, 11, }, // America/St_Vincent
    {  540,  608,   98228,  7, }, // America/Toronto
    {  540, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  540, 3968,  167050,  5, }, // Asia/Amman
    {  540, 4511,  167055,  7, }, // Asia/Colombo
    {  540, 4540,   20757,  7, }, // Asia/Damascus
    {  540, 8651,  167062,  5, }, // Asia/Gaza
    {  540, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  540, 7006,   95267,  8, }, // Asia/Jakarta
    {  540, 1290,  167067, 11, }, // Asia/Jerusalem
    {  540, 1185,     448,  9, }, // Asia/Kathmandu
    {  540,  992,  167078,  7, }, // Asia/Kolkata
    {  540, 1211,    4321,  5, }, // Asia/Macau
    {  540, 6920,  167085,  7, }, // Asia/Muscat
    {  540, 2554,  167092,  7, }, // Asia/Nicosia
    {  540, 7245,  167099, 10, }, // Asia/Samarkand
    {  540, 3279,  167109,  4, }, // Asia/Seoul
    {  540, 3300,  167113,  9, }, // Asia/Singapore
    {  540, 4710,  167122, 11, }, // Asia/Vladivostok
    {  540, 1235,     477,  6, }, // Asia/Yangon
    {  540, 3883,  167133, 14, }, // Asia/Yekaterinburg
    {  540, 3663,  167147,  7, }, // Atlantic/Azores
    {  540, 5671,  167154,  7, }, // Atlantic/Bermuda
    {  540, 6628,   20967,  6, }, // Atlantic/Canary
    {  540, 3727,  167161, 10, }, // Atlantic/Cape_Verde
    {  540, 1395,     483,  5, }, // Atlantic/Faroe
    {  540, 7272,  167171, 19, }, // Atlantic/South_Georgia
    {  540, 6821,  167190, 10, }, // Atlantic/St_Helena
    {  540, 1670,  167200,  8, }, // Australia/Adelaide
    {  540, 2300,  134543,  5, }, // Europe/Athens
    {  540, 6336,  164843,  8, }, // Europe/Belgrade
    {  540, 4757,  167208,  7, }, // Europe/Berlin
    {  540, 2167,  167215,  8, }, // Europe/Brussels
    {  540, 2338,  167223,  8, }, // Europe/Dublin
    {  540, 6307,  167231, 11, }, // Europe/Gibraltar
    {  540, 6612,   44553,  8, }, // Europe/Helsinki
    {  540, 8804,  167242, 12, }, // Europe/Isle_of_Man
    {  540, 1130,  164893,  8, }, // Europe/Istanbul
    {  540, 2527,    8148,  4, }, // Europe/Kyiv
    {  540, 3213,   93414,  7, }, // Europe/Lisbon
    {  540, 2501,  167254,  7, }, // Europe/London
    {  540, 6365,  167261, 11, }, // Europe/Luxembourg
    {  540, 6397,  167272,  5, }, // Europe/Malta
    {  540, 6410,  164901,  6, }, // Europe/Monaco
    {  540, 3537,   95387,  5, }, // Europe/Moscow
    {  540, 4284,  167277,  7, }, // Europe/Paris
    {  540, 6275,   21185,  5, }, // Europe/Prague
    {  540, 6324,    6064,  4, }, // Europe/Rome
    {  540, 6470,   99312, 10, }, // Europe/San_Marino
    {  540, 6245,  167284,  8, }, // Europe/Sarajevo
    {  540, 6523,  167292,  9, }, // Europe/Stockholm
    {  540, 6202,   21213,  6, }, // Europe/Tirane
    {  540, 6567,   21222,  8, }, // Europe/Vatican
    {  540, 4727,  167301,  9, }, // Europe/Volgograd
    {  540, 3190,   95436,  8, }, // Europe/Warsaw
    {  540, 6261,  167310,  9, }, // Europe/Zagreb
    {  540, 6540,  164966,  6, }, // Europe/Zurich
    {  540, 6046,  167319,  6, }, // Indian/Christmas
    {  540, 7050,  167325,  7, }, // Indian/Maldives
    {  540, 7211,     547,  7, }, // Indian/Reunion
    {  540, 3157,     554,  5, }, // Pacific/Chuuk
    {  540, 3902,   95469,  4, }, // Pacific/Fiji
    {  540, 6678,  167332,  9, }, // Pacific/Galapagos
    {  540, 4051,   95473,  8, }, // Pacific/Marquesas
    {  540, 3096,     586,  7, }, // Pacific/Pohnpei
    {  541,   14,       0,  6, }, // Africa/Asmara Sidamo/Latin/Ethiopia
    {  541, 4435,       6,  8, }, // Africa/Sao_Tome
    {  541, 4252,      14,  8, }, // America/Asuncion
    {  541,  237,      22,  8, }, // America/Atikokan
    {  541, 7623,      30, 17, }, // America/Bahia_Banderas
    {  541, 3868,      47,  6, }, // America/Cancun
    {  541, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  541, 5723,      66,  7, }, // America/Curacao
    {  541,  481,      73, 13, }, // America/Indiana/Knox
    {  541, 7961,      86, 16, }, // America/Indiana/Marengo
    {  541, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  541, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  541, 8038,     139, 14, }, // America/Indiana/Vevay
    {  541, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  541, 8086,     171, 16, }, // America/Indiana/Winamac
    {  541, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  541, 5932,     207, 22, }, // America/Lower_Princes
    {  541, 8220,     229,  6, }, // America/Merida
    {  541, 1850,     235, 19, }, // America/Noronha
    {  541, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  541, 8329,     274, 20, }, // America/North_Dakota/Center
    {  541, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  541,  393,     317,  4, }, // America/Nuuk
    {  541, 6899,     321, 16, }, // America/Scoresbysund
    {  541, 8508,     337, 14, }, // America/St_Barthelemy
    {  541, 2061,     351, 10, }, // America/St_Johns
    {  541, 5863,     361,  9, }, // America/St_Kitts
    {  541, 5880,     370,  9, }, // America/St_Lucia
    {  541,  855,     379, 10, }, // America/St_Thomas
    {  541, 5913,     389, 11, }, // America/St_Vincent
    {  541, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  541, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  541, 1185,     448,  9, }, // Asia/Kathmandu
    {  541,  992,     457,  7, }, // Asia/Kolkata
    {  541, 1211,     464,  5, }, // Asia/Macau
    {  541, 1235,     477,  6, }, // Asia/Yangon
    {  541, 1395,     483,  5, }, // Atlantic/Faroe
    {  541, 6821,     488, 10, }, // Atlantic/St_Helena
    {  541, 2527,     514,  4, }, // Europe/Kyiv
    {  541, 7211,     547,  7, }, // Indian/Reunion
    {  541, 3157,     554,  5, }, // Pacific/Chuuk
    {  541, 3096,     586,  7, }, // Pacific/Pohnpei
    {  542, 6788,  157051,  7, }, // Africa/Abidjan Silesian/Latin/Poland
    {  542, 6760,    4649,  4, }, // Africa/Accra
    {  542, 5059,    3902, 11, }, // Africa/Addis_Ababa
    {  542, 7420,   44088,  5, }, // Africa/Algiers
    {  542,   14,       0,  6, }, // Africa/Asmara
    {  542, 5257,   26480,  5, }, // Africa/Bangui
    {  542, 6746,  157058,  7, }, // Africa/Banjul
    {  542, 4922,  157065,  9, }, // Africa/Bujumbura
    {  542, 2320,    4690,  4, }, // Africa/Cairo
    {  542, 6773,  167341,  7, }, // Africa/Conakry
    {  542, 5043,  157074,  7, }, // Africa/Djibouti
    {  542, 5243,   21372,  5, }, // Africa/Douala
    {  542, 5166,  157081,  7, }, // Africa/El_Aaiun
    {  542, 4499,  157088,  5, }, // Africa/Juba
    {  542, 4524,  124879,  7, }, // Africa/Khartoum
    {  542, 5306,  157093,  8, }, // Africa/Kinshasa
    {  542, 6869,   20418,  4, }, // Africa/Lome
    {  542, 5113,  157101,  9, }, // Africa/Mogadishu
    {  542, 5271,  157110,  8, }, // Africa/Ndjamena
    {  542, 6803,  157118,  9, }, // Africa/Nouakchott
    {  542, 6727,    4758,  8, }, // Africa/Ouagadougou
    {  542, 5225,   26542, 10, }, // Africa/Porto-Novo
    {  542, 4435,       6,  8, }, // Africa/Sao_Tome
    {  542, 2866,  157127,  8, }, // Africa/Tripoli
    {  542, 4152,   56808,  7, }, // Africa/Windhoek
    {  542,  565,  167348,  7, }, // America/Argentina/Mendoza
    {  542, 4252,  167355,  8, }, // America/Asuncion
    {  542,  237,      22,  8, }, // America/Atikokan
    {  542, 3679,    6806,  8, }, // America/Bahia
    {  542, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  542, 7646,   35896,  5, }, // America/Belem
    {  542, 3868,      47,  6, }, // America/Cancun
    {  542, 4345,  167363,  7, }, // America/Cayenne
    {  542, 5496,  157135,  7, }, // America/Cayman
    {  542, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  542, 5422,  157142,  9, }, // America/Costa_Rica
    {  542, 3791,   26574,  6, }, // America/Cuiaba
    {  542, 5723,      66,  7, }, // America/Curacao
    {  542, 5739,  167370,  8, }, // America/Dominica
    {  542, 5441,  157151,  8, }, // America/El_Salvador
    {  542, 5770,  167378,  7, }, // America/Grenada
    {  542, 5786,  157159,  9, }, // America/Guadeloupe
    {  542, 3760,  167385,  9, }, // America/Guatemala
    {  542, 6932,  157168,  6, }, // America/Guyana
    {  542, 2281,  157174,  6, }, // America/Havana
    {  542,  481,      73, 13, }, // America/Indiana/Knox
    {  542, 7961,      86, 16, }, // America/Indiana/Marengo
    {  542, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  542, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  542, 8038,     139, 14, }, // America/Indiana/Vevay
    {  542, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  542, 8086,     171, 16, }, // America/Indiana/Winamac
    {  542, 2799,   33347,  7, }, // America/Jamaica
    {  542, 8125,     187, 10, }, // America/Kentucky/Monticello
    {  542, 5932,     207, 22, }, // America/Lower_Princes
    {  542, 8153,  167394,  6, }, // America/Maceio
    {  542, 5805,  157180,  9, }, // America/Martinique
    {  542, 8202,  167400,  9, }, // America/Menominee
    {  542, 8220,    7126,  6, }, // America/Merida
    {  542, 2949,  157189, 15, }, // America/Mexico_City
    {  542, 5824,  167409, 10, }, // America/Montserrat
    {  542, 2379,  157204,  9, }, // America/New_York
    {  542, 1850,     235, 19, }, // America/Noronha
    {  542, 8301,  167419, 23, }, // America/North_Dakota/Beulah
    {  542, 8329,  167442, 23, }, // America/North_Dakota/Center
    {  542, 8357,  167465, 26, }, // America/North_Dakota/New_Salem
    {  542,  393,     317,  4, }, // America/Nuuk
    {  542, 5954,   26691, 13, }, // America/Port_of_Spain
    {  542, 5843,  157285,  9, }, // America/Puerto_Rico
    {  542, 6899,     321, 16, }, // America/Scoresbysund
    {  542, 8508,  167491, 16, }, // America/St_Barthelemy
    {  542, 2061,     351, 10, }, // America/St_Johns
    {  542, 5863,   26749, 11, }, // America/St_Kitts
    {  542, 5880,   35964, 11, }, // America/St_Lucia
    {  542,  855,   44293, 12, }, // America/St_Thomas
    {  542, 5913,   26783, 13, }, // America/St_Vincent
    {  542, 5756,   26796,  7, }, // America/Thule
    {  542, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  542, 7374,    4147,  6, }, // Antarctica/Vostok
    {  542, 1429,  167507, 12, }, // Arctic/Longyearbyen
    {  542, 5570,  167519,  4, }, // Asia/Aden
    {  542, 5395,  157294,  6, }, // Asia/Almaty
    {  542, 5523,    5609,  5, }, // Asia/Aqtau
    {  542, 5368,  157300, 10, }, // Asia/Aqtobe
    {  542,  964,  157310,  9, }, // Asia/Ashgabat
    {  542, 3607,    4162,  6, }, // Asia/Baghdad
    {  542, 5534,   57943,  7, }, // Asia/Bahrain
    {  542, 3571,  157319,  7, }, // Asia/Barnaul
    {  542, 4086,    7625,  6, }, // Asia/Beirut
    {  542, 3778,  157326,  7, }, // Asia/Bishkek
    {  542, 4601,  157333,  5, }, // Asia/Chita
    {  542, 4511,  167523,  7, }, // Asia/Colombo
    {  542, 4540,  157338,  8, }, // Asia/Damascus
    {  542, 3596,   57968,  5, }, // Asia/Dubai
    {  542, 6159,  157346,  8, }, // Asia/Dushanbe
    {  542, 4795,  167530,  6, }, // Asia/Hebron
    {  542, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  542, 2704,  167536,  8, }, // Asia/Hong_Kong
    {  542, 4771,  157354,  5, }, // Asia/Hovd
    {  542, 4184,  147155,  6, }, // Asia/Irkutsk
    {  542, 7006,  157359,  8, }, // Asia/Jakarta
    {  542, 1290,  157367, 10, }, // Asia/Jerusalem
    {  542, 4316,  157377,  9, }, // Asia/Kamchatka
    {  542, 4239,  157386,  7, }, // Asia/Karachi
    {  542, 1185,    5711,  8, }, // Asia/Kathmandu
    {  542, 8661,   58020,  8, }, // Asia/Khandyga
    {  542,  992,   26864,  7, }, // Asia/Kolkata
    {  542, 4197,    4288, 11, }, // Asia/Krasnoyarsk
    {  542, 5547,  157393,  6, }, // Asia/Kuwait
    {  542, 1211,   33537,  5, }, // Asia/Macau
    {  542, 6920,   26882,  6, }, // Asia/Muscat
    {  542, 2554,  157399,  7, }, // Asia/Nicosia
    {  542, 8675,  157406, 12, }, // Asia/Novokuznetsk
    {  542, 4135,  157418, 11, }, // Asia/Novosibirsk
    {  542, 4229,  167544,  4, }, // Asia/Omsk
    {  542, 7145,  167548,  4, }, // Asia/Oral
    {  542, 4214,  167552,  9, }, // Asia/Pyongyang
    {  542, 5559,    4332,  5, }, // Asia/Qatar
    {  542, 8708,  157429,  8, }, // Asia/Qostanay
    {  542, 4269,  157437,  9, }, // Asia/Qyzylorda
    {  542, 3584,   33571,  5, }, // Asia/Riyadh
    {  542, 4408,   58084,  8, }, // Asia/Sakhalin
    {  542, 7245,   26906, 10, }, // Asia/Samarkand
    {  542, 3279,    7798,  4, }, // Asia/Seoul
    {  542, 1053,  157446,  8, }, // Asia/Shanghai
    {  542, 3300,   20900,  8, }, // Asia/Singapore
    {  542, 4297,  157454, 15, }, // Asia/Srednekolymsk
    {  542, 3263,    7830,  6, }, // Asia/Taipei
    {  542, 4781,  167561,  8, }, // Asia/Tashkent
    {  542, 2772,    7842,  7, }, // Asia/Tehran
    {  542, 2821,    4356,  5, }, // Asia/Tokyo
    {  542, 4572,  167569,  5, }, // Asia/Tomsk
    {  542, 1021,  157478, 10, }, // Asia/Ulaanbaatar
    {  542, 1159,  157488,  7, }, // Asia/Urumqi
    {  542, 8722,  157495,  9, }, // Asia/Ust-Nera
    {  542, 6977,  167574,  8, }, // Asia/Vientiane
    {  542, 4710,  157512, 11, }, // Asia/Vladivostok
    {  542, 4828,  147182,  6, }, // Asia/Yakutsk
    {  542, 1235,    7880,  6, }, // Asia/Yangon
    {  542, 3883,  157523, 13, }, // Asia/Yekaterinburg
    {  542, 3747,  157536,  6, }, // Asia/Yerevan
    {  542, 3663,  157542,  5, }, // Atlantic/Azores
    {  542, 5671,   58175,  7, }, // Atlantic/Bermuda
    {  542, 6628,  167582, 16, }, // Atlantic/Canary
    {  542, 3727,  167598, 30, }, // Atlantic/Cape_Verde
    {  542, 1395,  167628, 11, }, // Atlantic/Faroe
    {  542, 8736,  157604,  6, }, // Atlantic/Madeira
    {  542, 7272,  167639, 18, }, // Atlantic/South_Georgia
    {  542, 6821,  167657, 14, }, // Atlantic/St_Helena
    {  542, 6216,   33638,  6, }, // Europe/Andorra
    {  542, 3620,  157641,  9, }, // Europe/Astrakhan
    {  542, 2300,  167671,  5, }, // Europe/Athens
    {  542, 6336,   27015,  7, }, // Europe/Belgrade
    {  542, 6488,  157655, 10, }, // Europe/Bratislava
    {  542, 2167,  157665,  8, }, // Europe/Brussels
    {  542, 3928,  157673,  9, }, // Europe/Bucharest
    {  542, 3806,  157682,  9, }, // Europe/Budapest
    {  542, 8772,   62908, 21, }, // Europe/Busingen
    {  542, 2583,  167676,  9, }, // Europe/Chisinau
    {  542, 6289,  167685,  9, }, // Europe/Copenhagen
    {  542, 8804,  157700,  9, }, // Europe/Isle_of_Man
    {  542, 1130,  157709,  7, }, // Europe/Istanbul
    {  542, 8929,   78287,  5, }, // Europe/Kirov
    {  542, 2527,  167694,  5, }, // Europe/Kyiv
    {  542, 3213,  167699,  7, }, // Europe/Lisbon
    {  542, 6506,  157728,  7, }, // Europe/Ljubljana
    {  542, 2501,  167706,  6, }, // Europe/London
    {  542, 6365,  167712, 10, }, // Europe/Luxembourg
    {  542, 4892,  157741,  6, }, // Europe/Madrid
    {  542, 6582,   63826, 13, }, // Europe/Mariehamn
    {  542, 3693,  157747,  5, }, // Europe/Minsk
    {  542, 6410,  167722,  6, }, // Europe/Monaco
    {  542, 3537,   93233,  6, }, // Europe/Moscow
    {  542, 6458,  167728,  4, }, // Europe/Oslo
    {  542, 4284,  157752,  5, }, // Europe/Paris
    {  542, 6275,   21185,  5, }, // Europe/Prague
    {  542, 8837,  127376,  4, }, // Europe/Riga
    {  542, 6324,  157757,  4, }, // Europe/Rome
    {  542, 6245,  157761,  8, }, // Europe/Sarajevo
    {  542, 4451,  167732,  7, }, // Europe/Saratov
    {  542, 8849,  157776, 10, }, // Europe/Simferopol
    {  542, 6523,  157786,  9, }, // Europe/Stockholm
    {  542, 8867,   22925,  6, }, // Europe/Tallinn
    {  542, 6202,   21213,  6, }, // Europe/Tirane
    {  542, 8882,   78303,  9, }, // Europe/Ulyanovsk
    {  542, 6567,  157795,  7, }, // Europe/Vatican
    {  542, 6231,  167739,  6, }, // Europe/Vienna
    {  542, 8899,  157808,  5, }, // Europe/Vilnius
    {  542, 4727,  157813,  9, }, // Europe/Volgograd
    {  542, 3190,   58675,  8, }, // Europe/Warsaw
    {  542, 6261,  157822,  7, }, // Europe/Zagreb
    {  542, 6540,  157829,  6, }, // Europe/Zurich
    {  542, 5078,  157835, 12, }, // Indian/Antananarivo
    {  542, 6947,  157847,  6, }, // Indian/Chagos
    {  542, 6046,  167745, 12, }, // Indian/Christmas
    {  542, 6063,  157876, 14, }, // Indian/Cocos
    {  542, 5029,  167757,  6, }, // Indian/Comoro
    {  542, 6661,  167763, 18, }, // Indian/Kerguelen
    {  542, 7260,   27115,  4, }, // Indian/Mahe
    {  542, 7050,  128028,  8, }, // Indian/Maldives
    {  542, 5098,  157906,  7, }, // Indian/Mayotte
    {  542, 7211,     547,  7, }, // Indian/Reunion
    {  542, 3706,  167781, 22, }, // Pacific/Bougainville
    {  542, 3157,     554,  5, }, // Pacific/Chuuk
    {  542, 2237,  167803, 17, }, // Pacific/Easter
    {  542, 3902,  157950,  5, }, // Pacific/Fiji
    {  542, 6696,  157955, 14, }, // Pacific/Gambier
    {  542, 3049,    4619,  9, }, // Pacific/Kanton
    {  542, 4051,  157969,  7, }, // Pacific/Marquesas
    {  542, 7098,  124660,  5, }, // Pacific/Noumea
    {  542, 3096,     586,  7, }, // Pacific/Pohnpei
    {  543, 6788,  167820,  7, }, // Africa/Abidjan Sindhi/Arabic/Pakistan
    {  543, 6760,  167827,  5, }, // Africa/Accra
    {  543, 5059,  167832, 10, }, // Africa/Addis_Ababa
    {  543, 7420,  167842,  6, }, // Africa/Algiers
    {  543,   14,  167848,  6, }, // Africa/Asmara
    {  543,   44,  167854,  6, }, // Africa/Bamako
    {  543, 5257,  167860,  4, }, // Africa/Bangui
    {  543, 6746,  167864,  6, }, // Africa/Banjul
    {  543, 7435,  167870,  5, }, // Africa/Bissau
    {  543, 4957,  167875,  7, }, // Africa/Blantyre
    {  543, 5287,  151966,  8, }, // Africa/Brazzaville
    {  543, 4922,  167882,  8, }, // Africa/Bujumbura
    {  543, 2320,  167890,  5, }, // Africa/Cairo
    {  543, 4117,  167895, 10, }, // Africa/Casablanca
    {  543, 7449,  167905,  5, }, // Africa/Ceuta
    {  543, 6773,  167910,  7, }, // Africa/Conakry
    {  543, 6840,  167917,  5, }, // Africa/Dakar
    {  543, 5130,  103226,  9, }, // Africa/Dar_es_Salaam
    {  543, 5043,  152012,  5, }, // Africa/Djibouti
    {  543, 5243,   10633,  5, }, // Africa/Douala
    {  543, 5166,  167922,  7, }, // Africa/El_Aaiun
    {  543, 6853,  167929,  9, }, // Africa/Freetown
    {  543, 4906,  167938,  7, }, // Africa/Gaborone
    {  543, 5015,   10659,  5, }, // Africa/Harare
    {  543, 4479,  167945,  9, }, // Africa/Johannesburg
    {  543, 4499,   10673,  4, }, // Africa/Juba
    {  543, 5151,  167954,  6, }, // Africa/Kampala
    {  543, 4524,   10686,  5, }, // Africa/Khartoum
    {  543, 4973,  167960,  6, }, // Africa/Kigali
    {  543, 5306,  167966,  6, }, // Africa/Kinshasa
    {  543, 4744,  106900,  5, }, // Africa/Lagos
    {  543, 5336,  167972,  8, }, // Africa/Libreville
    {  543, 6869,   10716,  3, }, // Africa/Lome
    {  543, 5211,  167980,  6, }, // Africa/Luanda
    {  543, 4939,  167986,  8, }, // Africa/Lubumbashi
    {  543, 5001,  167994,  6, }, // Africa/Lusaka
    {  543, 5322,  168000,  5, }, // Africa/Malabo
    {  543, 4987,  154718,  6, }, // Africa/Maputo
    {  543, 5197,   10752,  6, }, // Africa/Maseru
    {  543, 5182,  168005,  6, }, // Africa/Mbabane
    {  543, 5113,  168011,  8, }, // Africa/Mogadishu
    {  543, 7034,  154744,  8, }, // Africa/Monrovia
    {  543, 3853,   10780,  6, }, // Africa/Nairobi
    {  543, 5271,   10786,  7, }, // Africa/Ndjamena
    {  543, 5354,   10793,  5, }, // Africa/Niamey
    {  543, 6803,  168019,  7, }, // Africa/Nouakchott
    {  543, 6727,  168026, 12, }, // Africa/Ouagadougou
    {  543, 5225,  168038,  9, }, // Africa/Porto-Novo
    {  543, 4435,  168047,  8, }, // Africa/Sao_Tome
    {  543, 2866,  168055,  6, }, // Africa/Tripoli
    {  543, 6554,  168061,  5, }, // Africa/Tunis
    {  543, 4152,  168066,  6, }, // Africa/Windhoek
    {  543,  132,  168072,  3, }, // America/Adak
    {  543, 3346,  168075,  7, }, // America/Anchorage
    {  543, 5607,  168082,  7, }, // America/Anguilla
    {  543, 5624,  168089,  8, }, // America/Antigua
    {  543, 4554,  168097,  9, }, // America/Araguaina
    {  543,  166,  168106, 12, }, // America/Argentina/Buenos_Aires
    {  543,   91,  168118, 10, }, // America/Argentina/Catamarca
    {  543,  270,  168128,  7, }, // America/Argentina/Cordoba
    {  543,  441,  168135,  6, }, // America/Argentina/Jujuy
    {  543, 7462,  168141,  9, }, // America/Argentina/La_Rioja
    {  543,  565,  168150,  7, }, // America/Argentina/Mendoza
    {  543, 7489,  168157, 11, }, // America/Argentina/Rio_Gallegos
    {  543, 7520,  168168,  5, }, // America/Argentina/Salta
    {  543, 7544,  168173,  8, }, // America/Argentina/San_Juan
    {  543, 5580,  168181,  9, }, // America/Argentina/San_Luis
    {  543, 7571,  168190,  7, }, // America/Argentina/Tucuman
    {  543, 7597,  168197,  8, }, // America/Argentina/Ushuaia
    {  543, 5640,  154922,  5, }, // America/Aruba
    {  543, 4252,  168205,  8, }, // America/Asuncion
    {  543,  237,  168213,  7, }, // America/Atikokan
    {  543, 3679,  168220,  5, }, // America/Bahia
    {  543, 7623,  168225, 13, }, // America/Bahia_Banderas
    {  543, 5654,  168238,  8, }, // America/Barbados
    {  543, 7646,  168246,  4, }, // America/Belem
    {  543, 5407,  168250,  5, }, // America/Belize
    {  543, 7660,  168255, 11, }, // America/Blanc-Sablon
    {  543, 7681,  168266,  9, }, // America/Boa_Vista
    {  543, 4361,  168275,  5, }, // America/Bogota
    {  543, 7699,  168280,  4, }, // America/Boise
    {  543, 7713,  168284,  9, }, // America/Cambridge_Bay
    {  543, 7735,  168293, 12, }, // America/Campo_Grande
    {  543, 3868,  168305,  6, }, // America/Cancun
    {  543, 4694,  168311,  5, }, // America/Caracas
    {  543, 4345,  168316,  5, }, // America/Cayenne
    {  543, 5496,  168321,  5, }, // America/Cayman
    {  543, 2260,  168326,  5, }, // America/Chicago
    {  543, 7756,  168331, 10, }, // America/Chihuahua
    {  543, 7774,  168341, 11, }, // America/Ciudad_Juarez
    {  543, 5422,  168352, 10, }, // America/Costa_Rica
    {  543, 7796,  168362,  6, }, // America/Creston
    {  543, 3791,  168368,  7, }, // America/Cuiaba
    {  543, 5723,  168375,  9, }, // America/Curacao
    {  543, 7812,  168384, 11, }, // America/Danmarkshavn
    {  543, 7833,  168395,  5, }, // America/Dawson
    {  543, 7848,  168400, 10, }, // America/Dawson_Creek
    {  543,  805,  168410,  5, }, // America/Denver
    {  543, 3465,  168415,  8, }, // America/Detroit
    {  543, 5739,  168423,  8, }, // America/Dominica
    {  543,  893,  168431,  8, }, // America/Edmonton
    {  543, 7869,  168439,  7, }, // America/Eirunepe
    {  543, 5441,  168446, 11, }, // America/El_Salvador
    {  543, 7886,  168457, 10, }, // America/Fort_Nelson
    {  543, 7906,  168467,  9, }, // America/Fortaleza
    {  543, 7924,  168476,  7, }, // America/Glace_Bay
    {  543, 6881,  152545,  6, }, // America/Goose_Bay
    {  543, 4612,  168483,  9, }, // America/Grand_Turk
    {  543, 5770,  168492,  7, }, // America/Grenada
    {  543, 5786,  168499,  8, }, // America/Guadeloupe
    {  543, 3760,  168507, 10, }, // America/Guatemala
    {  543, 4841,  168517,  8, }, // America/Guayaquil
    {  543, 6932,  168525,  5, }, // America/Guyana
    {  543, 1939,  168530,  9, }, // America/Halifax
    {  543, 2281,  152595,  5, }, // America/Havana
    {  543, 7942,  168539,  7, }, // America/Hermosillo
    {  543,  348,  168546, 12, }, // America/Indiana/Indianapolis
    {  543,  481,  168558, 13, }, // America/Indiana/Knox
    {  543, 7961,  168571, 15, }, // America/Indiana/Marengo
    {  543, 7985,  168586, 17, }, // America/Indiana/Petersburg
    {  543, 8012,  168603, 16, }, // America/Indiana/Tell_City
    {  543, 8038,  168619, 13, }, // America/Indiana/Vevay
    {  543, 8060,  168632, 16, }, // America/Indiana/Vincennes
    {  543, 8086,  168648, 16, }, // America/Indiana/Winamac
    {  543, 8110,  168664,  5, }, // America/Inuvik
    {  543,  660,  168669,  7, }, // America/Iqaluit
    {  543, 2799,  168676,  6, }, // America/Jamaica
    {  543, 5380,  168682,  5, }, // America/Juneau
    {  543,  521,  168687,  8, }, // America/Kentucky/Louisville
    {  543, 8125,  168695, 17, }, // America/Kentucky/Monticello
    {  543, 5704,  168712,  8, }, // America/Kralendijk
    {  543, 4376,  168720,  5, }, // America/La_Paz
    {  543, 7169,   11456,  4, }, // America/Lima
    {  543, 3239,  168725, 10, }, // America/Los_Angeles
    {  543, 5932,  168735, 17, }, // America/Lower_Princes
    {  543, 8153,  168752,  6, }, // America/Maceio
    {  543, 8168,  155436,  7, }, // America/Managua
    {  543, 1908,  168758,  6, }, // America/Manaus
    {  543, 5897,  168764,  7, }, // America/Marigot
    {  543, 5805,  168771,  7, }, // America/Martinique
    {  543, 8184,  168778,  8, }, // America/Matamoros
    {  543, 2917,  168786,  7, }, // America/Mazatlan
    {  543, 8202,  168793,  8, }, // America/Menominee
    {  543, 8220,  168801,  6, }, // America/Merida
    {  543, 8235,  168807, 11, }, // America/Metlakatla
    {  543, 2949,  168818, 11, }, // America/Mexico_City
    {  543, 4391,  168829,  9, }, // America/Miquelon
    {  543, 8254,  168838,  6, }, // America/Moncton
    {  543, 8270,  168844,  7, }, // America/Monterrey
    {  543, 4098,  168851, 11, }, // America/Montevideo
    {  543, 5824,  168862,  8, }, // America/Montserrat
    {  543, 5481,  155528,  6, }, // America/Nassau
    {  543, 2379,  168870,  7, }, // America/New_York
    {  543, 8288,   11540,  3, }, // America/Nome
    {  543, 1850,  168877,  8, }, // America/Noronha
    {  543, 8301,  168885, 17, }, // America/North_Dakota/Beulah
    {  543, 8329,  168902, 16, }, // America/North_Dakota/Center
    {  543, 8357,  168918, 19, }, // America/North_Dakota/New_Salem
    {  543,  393,  168937,  4, }, // America/Nuuk
    {  543, 8388,  168941,  8, }, // America/Ojinaga
    {  543, 2356,  168949,  5, }, // America/Panama
    {  543, 6173,  168954, 10, }, // America/Paramaribo
    {  543, 2973,  168964,  6, }, // America/Phoenix
    {  543, 3945,  168970, 12, }, // America/Port-au-Prince
    {  543, 5954,  168982, 12, }, // America/Port_of_Spain
    {  543, 8404,  168994, 11, }, // America/Porto_Velho
    {  543, 5843,  169005, 10, }, // America/Puerto_Rico
    {  543, 4030,  169015, 11, }, // America/Punta_Arenas
    {  543, 8424,  169026, 11, }, // America/Rankin_Inlet
    {  543, 8445,  169037,  6, }, // America/Recife
    {  543, 1995,  169043,  6, }, // America/Regina
    {  543, 8460,  169049,  8, }, // America/Resolute
    {  543,  695,  169057, 10, }, // America/Rio_Branco
    {  543, 8477,  169067,  7, }, // America/Santarem
    {  543, 2201,  169074,  8, }, // America/Santiago
    {  543, 6111,  169082, 13, }, // America/Santo_Domingo
    {  543, 1878,  169095,  9, }, // America/Sao_Paulo
    {  543, 6899,  169104, 12, }, // America/Scoresbysund
    {  543, 8494,  169116,  4, }, // America/Sitka
    {  543, 8508,  169120, 12, }, // America/St_Barthelemy
    {  543, 2061,  169132,  9, }, // America/St_Johns
    {  543, 5863,  169141,  8, }, // America/St_Kitts
    {  543, 5880,  169149, 10, }, // America/St_Lucia
    {  543,  855,  169159,  9, }, // America/St_Thomas
    {  543, 5913,  169168, 11, }, // America/St_Vincent
    {  543, 8530,  169179, 10, }, // America/Swift_Current
    {  543, 5461,  169189, 10, }, // America/Tegucigalpa
    {  543, 5756,  169199,  4, }, // America/Thule
    {  543,  313,  169203,  7, }, // America/Tijuana
    {  543,  608,  169210,  6, }, // America/Toronto
    {  543, 5688,  169216,  7, }, // America/Tortola
    {  543, 2093,  169223,  7, }, // America/Vancouver
    {  543, 2144,  169230, 11, }, // America/Whitehorse
    {  543,  734,  169241,  7, }, // America/Winnipeg
    {  543, 8552,  169248,  7, }, // America/Yakutat
    {  543, 6001,  169255,  4, }, // Antarctica/Casey
    {  543, 6094,  169259,  4, }, // Antarctica/Davis
    {  543, 6133,  169263, 13, }, // Antarctica/DumontDUrville
    {  543, 8568,  169276,  7, }, // Antarctica/Macquarie
    {  543, 7066,  169283,  4, }, // Antarctica/Mawson
    {  543, 7113,  169287,  8, }, // Antarctica/McMurdo
    {  543, 8589,  169295,  4, }, // Antarctica/Palmer
    {  543, 7226,  169299,  5, }, // Antarctica/Rothera
    {  543, 7295,  169304,  5, }, // Antarctica/Syowa
    {  543, 8607,  169309,  4, }, // Antarctica/Troll
    {  543, 7374,  169313,  6, }, // Antarctica/Vostok
    {  543, 1429,  169319, 11, }, // Arctic/Longyearbyen
    {  543, 5570,   12066,  3, }, // Asia/Aden
    {  543, 5395,  169330,  6, }, // Asia/Almaty
    {  543, 3968,   11497,  4, }, // Asia/Amman
    {  543, 5511,  169336,  5, }, // Asia/Anadyr
    {  543, 5523,  169341,  6, }, // Asia/Aqtau
    {  543, 5368,  169347,  6, }, // Asia/Aqtobe
    {  543,  964,  169353,  7, }, // Asia/Ashgabat
    {  543, 8624,  169360,  7, }, // Asia/Atyrau
    {  543, 3607,   12109,  5, }, // Asia/Baghdad
    {  543, 5534,   12116,  5, }, // Asia/Bahrain
    {  543, 3653,  168133,  4, }, // Asia/Baku
    {  543, 4466,  169367,  6, }, // Asia/Bangkok
    {  543, 3571,  169373,  4, }, // Asia/Barnaul
    {  543, 4086,   12134,  5, }, // Asia/Beirut
    {  543, 3778,  169377,  5, }, // Asia/Bishkek
    {  543, 5989,  169382,  7, }, // Asia/Brunei
    {  543, 4601,  108439,  4, }, // Asia/Chita
    {  543, 4511,  169389,  6, }, // Asia/Colombo
    {  543, 4540,   12162,  4, }, // Asia/Damascus
    {  543, 1093,  169395,  4, }, // Asia/Dhaka
    {  543, 6192,  153448,  3, }, // Asia/Dili
    {  543, 3596,  169399,  4, }, // Asia/Dubai
    {  543, 6159,   12173,  7, }, // Asia/Dushanbe
    {  543, 8636,  156024,  9, }, // Asia/Famagusta
    {  543, 8651,  153469,  3, }, // Asia/Gaza
    {  543, 4795,   12192,  6, }, // Asia/Hebron
    {  543, 1259,  169403,  7, }, // Asia/Ho_Chi_Minh
    {  543, 2704,  169410,  9, }, // Asia/Hong_Kong
    {  543, 4771,  169419,  4, }, // Asia/Hovd
    {  543, 4184,  169423,  6, }, // Asia/Irkutsk
    {  543, 7006,  169429,  6, }, // Asia/Jakarta
    {  543, 6992,  169435,  7, }, // Asia/Jayapura
    {  543, 1290,  169442,  7, }, // Asia/Jerusalem
    {  543, 3560,  169449,  4, }, // Asia/Kabul
    {  543, 4316,  169453,  7, }, // Asia/Kamchatka
    {  543, 4239,  169460,  5, }, // Asia/Karachi
    {  543, 1185,  169465,  6, }, // Asia/Kathmandu
    {  543, 8661,  169471,  7, }, // Asia/Khandyga
    {  543,  992,  169478,  5, }, // Asia/Kolkata
    {  543, 4197,  169483, 11, }, // Asia/Krasnoyarsk
    {  543, 4859,  169494, 10, }, // Asia/Kuala_Lumpur
    {  543, 5976,  169504,  5, }, // Asia/Kuching
    {  543, 5547,  169509,  4, }, // Asia/Kuwait
    {  543, 1211,  169513,  5, }, // Asia/Macau
    {  543, 4017,  169518,  6, }, // Asia/Magadan
    {  543, 1349,  169524,  6, }, // Asia/Makassar
    {  543, 7182,  169530,  5, }, // Asia/Manila
    {  543, 6920,   12368,  4, }, // Asia/Muscat
    {  543, 2554,  169535,  7, }, // Asia/Nicosia
    {  543, 8675,  169542, 10, }, // Asia/Novokuznetsk
    {  543, 4135,  169552, 11, }, // Asia/Novosibirsk
    {  543, 4229,  169563,  5, }, // Asia/Omsk
    {  543, 7145,  169568,  5, }, // Asia/Oral
    {  543, 6961,  169573,  9, }, // Asia/Phnom_Penh
    {  543, 8693,  169582,  8, }, // Asia/Pontianak
    {  543, 4214,  169590, 10, }, // Asia/Pyongyang
    {  543, 5559,   12436,  3, }, // Asia/Qatar
    {  543, 8708,  169600,  7, }, // Asia/Qostanay
    {  543, 4269,  169607,  8, }, // Asia/Qyzylorda
    {  543, 3584,   12458,  4, }, // Asia/Riyadh
    {  543, 4408,  169615,  6, }, // Asia/Sakhalin
    {  543, 7245,   12468,  6, }, // Asia/Samarkand
    {  543, 3279,  169621,  4, }, // Asia/Seoul
    {  543, 1053,  169625,  7, }, // Asia/Shanghai
    {  543, 3300,  156265,  7, }, // Asia/Singapore
    {  543, 4297,  169632, 12, }, // Asia/Srednekolymsk
    {  543, 3263,  169644,  5, }, // Asia/Taipei
    {  543, 4781,  169649,  6, }, // Asia/Tashkent
    {  543, 3915,   12514,  6, }, // Asia/Tbilisi
    {  543, 2772,  153730,  5, }, // Asia/Tehran
    {  543, 1317,  169655,  4, }, // Asia/Thimphu
    {  543, 2821,  169659,  5, }, // Asia/Tokyo
    {  543, 4572,  169664,  4, }, // Asia/Tomsk
    {  543, 1021,  169668, 10, }, // Asia/Ulaanbaatar
    {  543, 1159,  169678,  6, }, // Asia/Urumqi
    {  543, 8722,  169684,  8, }, // Asia/Ust-Nera
    {  543, 6977,  169692,  7, }, // Asia/Vientiane
    {  543, 4710,  169699, 10, }, // Asia/Vladivostok
    {  543, 4828,  169709,  6, }, // Asia/Yakutsk
    {  543, 1235,  169715,  5, }, // Asia/Yangon
    {  543, 3883,  169720,  9, }, // Asia/Yekaterinburg
    {  543, 3747,  153810,  6, }, // Asia/Yerevan
    {  543, 3663,  169729,  5, }, // Atlantic/Azores
    {  543, 5671,   12615,  6, }, // Atlantic/Bermuda
    {  543, 6628,  169734,  5, }, // Atlantic/Canary
    {  543, 3727,  169739,  8, }, // Atlantic/Cape_Verde
    {  543, 1395,  169747,  4, }, // Atlantic/Faroe
    {  543, 8736,  169751,  6, }, // Atlantic/Madeira
    {  543, 2748,  169757,  6, }, // Atlantic/Reykjavik
    {  543, 7272,  169763, 10, }, // Atlantic/South_Georgia
    {  543, 6821,  169773, 11, }, // Atlantic/St_Helena
    {  543, 6644,  169784,  7, }, // Atlantic/Stanley
    {  543, 1670,  169791,  7, }, // Australia/Adelaide
    {  543, 1635,  169798,  6, }, // Australia/Brisbane
    {  543, 1799,  169804,  8, }, // Australia/Broken_Hill
    {  543, 1583,  169812,  5, }, // Australia/Darwin
    {  543, 3637,  169817,  5, }, // Australia/Eucla
    {  543, 1516,  169822,  6, }, // Australia/Hobart
    {  543, 8753,  169828,  6, }, // Australia/Lindeman
    {  543, 1547,  169834,  9, }, // Australia/Lord_Howe
    {  543, 1727,   12742,  7, }, // Australia/Melbourne
    {  543, 1762,  169843,  3, }, // Australia/Perth
    {  543, 1463,  169846,  4, }, // Australia/Sydney
    {  543, 6441,  169850,  9, }, // Europe/Amsterdam
    {  543, 6216,  153951,  6, }, // Europe/Andorra
    {  543, 3620,  156515,  8, }, // Europe/Astrakhan
    {  543, 2300,  169859,  5, }, // Europe/Athens
    {  543, 6336,   12785,  6, }, // Europe/Belgrade
    {  543, 4757,  153976,  4, }, // Europe/Berlin
    {  543, 6488,  169864,  9, }, // Europe/Bratislava
    {  543, 2167,  169873,  5, }, // Europe/Brussels
    {  543, 3928,  169878,  7, }, // Europe/Bucharest
    {  543, 3806,  169885,  7, }, // Europe/Budapest
    {  543, 8772,  169892,  6, }, // Europe/Busingen
    {  543, 2583,  169898,  7, }, // Europe/Chisinau
    {  543, 6289,  169905,  9, }, // Europe/Copenhagen
    {  543, 2338,  169914,  4, }, // Europe/Dublin
    {  543, 6307,  169918,  7, }, // Europe/Gibraltar
    {  543, 8788,  169925,  5, }, // Europe/Guernsey
    {  543, 6612,  169930,  7, }, // Europe/Helsinki
    {  543, 8804,  169937, 12, }, // Europe/Isle_of_Man
    {  543, 1130,  154067,  7, }, // Europe/Istanbul
    {  543, 8823,  169949,  4, }, // Europe/Jersey
    {  543, 3979,  169953, 10, }, // Europe/Kaliningrad
    {  543, 8929,  169963,  5, }, // Europe/Kirov
    {  543, 2527,  169968,  4, }, // Europe/Kyiv
    {  543, 3213,  169972,  4, }, // Europe/Lisbon
    {  543, 6506,  169976,  7, }, // Europe/Ljubljana
    {  543, 2501,  169983,  4, }, // Europe/London
    {  543, 6365,  169987,  7, }, // Europe/Luxembourg
    {  543, 4892,  169994,  5, }, // Europe/Madrid
    {  543, 6397,  169999,  5, }, // Europe/Malta
    {  543, 6582,  154126,  7, }, // Europe/Mariehamn
    {  543, 3693,  170004,  4, }, // Europe/Minsk
    {  543, 6410,  170008,  6, }, // Europe/Monaco
    {  543, 3537,  170014,  5, }, // Europe/Moscow
    {  543, 6458,  105779,  5, }, // Europe/Oslo
    {  543, 4284,  170019,  4, }, // Europe/Paris
    {  543, 6424,  170023,  9, }, // Europe/Podgorica
    {  543, 6275,  105798,  4, }, // Europe/Prague
    {  543, 8837,  156379,  3, }, // Europe/Riga
    {  543, 6324,   10756,  3, }, // Europe/Rome
    {  543, 4331,  106756,  5, }, // Europe/Samara
    {  543, 6470,  170032,  9, }, // Europe/San_Marino
    {  543, 6245,  170041,  7, }, // Europe/Sarajevo
    {  543, 4451,  170048,  5, }, // Europe/Saratov
    {  543, 8849,  170053,  8, }, // Europe/Simferopol
    {  543, 6383,  170061,  6, }, // Europe/Skopje
    {  543, 6599,  109352,  5, }, // Europe/Sofia
    {  543, 6523,  170067,  9, }, // Europe/Stockholm
    {  543, 8867,  170076,  4, }, // Europe/Tallinn
    {  543, 6202,  170080,  5, }, // Europe/Tirane
    {  543, 8882,  170085,  8, }, // Europe/Ulyanovsk
    {  543, 6352,  170093,  4, }, // Europe/Vaduz
    {  543, 6567,  170097,  6, }, // Europe/Vatican
    {  543, 6231,   11394,  4, }, // Europe/Vienna
    {  543, 8899,  170103,  6, }, // Europe/Vilnius
    {  543, 4727,  170109,  9, }, // Europe/Volgograd
    {  543, 3190,  105924,  5, }, // Europe/Warsaw
    {  543, 6261,   13116,  4, }, // Europe/Zagreb
    {  543, 6540,   13120,  5, }, // Europe/Zurich
    {  543, 5078,  170118, 12, }, // Indian/Antananarivo
    {  543, 6947,  105952,  5, }, // Indian/Chagos
    {  543, 6046,  170130,  5, }, // Indian/Christmas
    {  543, 6063,  170135,  5, }, // Indian/Cocos
    {  543, 5029,  170140,  6, }, // Indian/Comoro
    {  543, 6661,  170146,  8, }, // Indian/Kerguelen
    {  543, 7260,   13173,  4, }, // Indian/Mahe
    {  543, 7050,  154287,  6, }, // Indian/Maldives
    {  543, 4069,  170154,  6, }, // Indian/Mauritius
    {  543, 5098,  170160,  5, }, // Indian/Mayotte
    {  543, 7211,  154305,  8, }, // Indian/Reunion
    {  543, 4422,  109536,  4, }, // Pacific/Apia
    {  543,  932,  170165,  6, }, // Pacific/Auckland
    {  543, 3706,  170171, 11, }, // Pacific/Bougainville
    {  543, 3015,  170182,  4, }, // Pacific/Chatham
    {  543, 3157,  170186,  4, }, // Pacific/Chuuk
    {  543, 2237,  170190,  5, }, // Pacific/Easter
    {  543, 7360,  170195,  5, }, // Pacific/Efate
    {  543, 7327,  170200,  7, }, // Pacific/Fakaofo
    {  543, 3902,  154353,  3, }, // Pacific/Fiji
    {  543, 7343,  170207,  7, }, // Pacific/Funafuti
    {  543, 6678,  170214,  9, }, // Pacific/Galapagos
    {  543, 6696,  170223,  7, }, // Pacific/Gambier
    {  543, 3822,  170230, 11, }, // Pacific/Guadalcanal
    {  543, 6018,  104212,  4, }, // Pacific/Guam
    {  543, 2723,  170241,  9, }, // Pacific/Honolulu
    {  543, 3049,  170250,  8, }, // Pacific/Kanton
    {  543, 3998,  170258,  7, }, // Pacific/Kiritimati
    {  543, 7019,  170265,  7, }, // Pacific/Kosrae
    {  543, 2842,  170272,  7, }, // Pacific/Kwajalein
    {  543, 4877,  154416,  5, }, // Pacific/Majuro
    {  543, 4051,  170279,  7, }, // Pacific/Marquesas
    {  543, 8914,  170286,  4, }, // Pacific/Midway
    {  543, 7084,  170290,  5, }, // Pacific/Nauru
    {  543, 7132,  170295,  5, }, // Pacific/Niue
    {  543, 4168,  170300,  7, }, // Pacific/Norfolk
    {  543, 7098,  170307,  7, }, // Pacific/Noumea
    {  543, 3126,  170314,  9, }, // Pacific/Pago_Pago
    {  543, 7155,  170323,  5, }, // Pacific/Palau
    {  543, 7194,  170328,  6, }, // Pacific/Pitcairn
    {  543, 3096,  170334,  7, }, // Pacific/Pohnpei
    {  543, 4807,  170341, 11, }, // Pacific/Port_Moresby
    {  543, 6076,  170352,  9, }, // Pacific/Rarotonga
    {  543, 6031,  170361,  4, }, // Pacific/Saipan
    {  543, 7312,  170365,  5, }, // Pacific/Tahiti
    {  543, 6712,  170370,  5, }, // Pacific/Tarawa
    {  543, 4583,  170375,  9, }, // Pacific/Tongatapu
    {  543, 7392,  170384,  3, }, // Pacific/Wake
    {  543, 7405,  170387,  4, }, // Pacific/Wallis
    {  544,   14,       0,  6, }, // Africa/Asmara Sindhi/Devanagari/India
    {  544, 4435,       6,  8, }, // Africa/Sao_Tome
    {  544, 4252,      14,  8, }, // America/Asuncion
    {  544,  237,      22,  8, }, // America/Atikokan
    {  544, 7623,      30, 17, }, // America/Bahia_Banderas
    {  544, 3868,      47,  6, }, // America/Cancun
    {  544, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  544, 5723,      66,  7, }, // America/Curacao
    {  544,  481,      73, 13, }, // America/Indiana/Knox
    {  544, 7961,      86, 16, }, // America/Indiana/Marengo
    {  544, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  544, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  544, 8038,     139, 14, }, // America/Indiana/Vevay
    {  544, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  544, 8086,     171, 16, }, // America/Indiana/Winamac
    {  544, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  544, 5932,     207, 22, }, // America/Lower_Princes
    {  544, 8220,     229,  6, }, // America/Merida
    {  544, 1850,     235, 19, }, // America/Noronha
    {  544, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  544, 8329,     274, 20, }, // America/North_Dakota/Center
    {  544, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  544,  393,     317,  4, }, // America/Nuuk
    {  544, 6899,     321, 16, }, // America/Scoresbysund
    {  544, 8508,     337, 14, }, // America/St_Barthelemy
    {  544, 2061,     351, 10, }, // America/St_Johns
    {  544, 5863,     361,  9, }, // America/St_Kitts
    {  544, 5880,     370,  9, }, // America/St_Lucia
    {  544,  855,     379, 10, }, // America/St_Thomas
    {  544, 5913,     389, 11, }, // America/St_Vincent
    {  544, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  544, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  544, 1185,     448,  9, }, // Asia/Kathmandu
    {  544,  992,     457,  7, }, // Asia/Kolkata
    {  544, 1211,     464,  5, }, // Asia/Macau
    {  544, 1235,     477,  6, }, // Asia/Yangon
    {  544, 1395,     483,  5, }, // Atlantic/Faroe
    {  544, 6821,     488, 10, }, // Atlantic/St_Helena
    {  544, 2527,     514,  4, }, // Europe/Kyiv
    {  544, 7211,     547,  7, }, // Indian/Reunion
    {  544, 3157,     554,  5, }, // Pacific/Chuuk
    {  544, 3096,     586,  7, }, // Pacific/Pohnpei
    {  545, 6788,  170391,  7, }, // Africa/Abidjan Sinhala/Sinhala/Sri Lanka
    {  545, 6760,  170398,  5, }, // Africa/Accra
    {  545, 5059,  170403, 11, }, // Africa/Addis_Ababa
    {  545, 7420,  170414, 10, }, // Africa/Algiers
    {  545,   14,  170424,  7, }, // Africa/Asmara
    {  545,   44,  170431,  4, }, // Africa/Bamako
    {  545, 5257,  170435,  7, }, // Africa/Bangui
    {  545, 6746,  170442,  7, }, // Africa/Banjul
    {  545, 7435,  170449,  4, }, // Africa/Bissau
    {  545, 4957,  170453,  9, }, // Africa/Blantyre
    {  545, 5287,  170462, 10, }, // Africa/Brazzaville
    {  545, 4922,  170472, 10, }, // Africa/Bujumbura
    {  545, 2320,  170482,  5, }, // Africa/Cairo
    {  545, 4117,  170487, 10, }, // Africa/Casablanca
    {  545, 7449,  170497,  6, }, // Africa/Ceuta
    {  545, 6773,  170503,  8, }, // Africa/Conakry
    {  545, 6840,  170511,  4, }, // Africa/Dakar
    {  545, 5130,  170515, 14, }, // Africa/Dar_es_Salaam
    {  545, 5043,  170529,  6, }, // Africa/Djibouti
    {  545, 5243,  170535,  5, }, // Africa/Douala
    {  545, 5166,  170540,  9, }, // Africa/El_Aaiun
    {  545, 6853,  170549, 10, }, // Africa/Freetown
    {  545, 4906,  170559,  7, }, // Africa/Gaborone
    {  545, 5015,  170566,  5, }, // Africa/Harare
    {  545, 4479,  170571, 13, }, // Africa/Johannesburg
    {  545, 4499,  170584,  4, }, // Africa/Juba
    {  545, 5151,  170588,  7, }, // Africa/Kampala
    {  545, 4524,  170595,  8, }, // Africa/Khartoum
    {  545, 4973,  170603,  6, }, // Africa/Kigali
    {  545, 5306,  170609,  8, }, // Africa/Kinshasa
    {  545, 4744,  170617,  6, }, // Africa/Lagos
    {  545, 5336,  170623,  9, }, // Africa/Libreville
    {  545, 6869,  170632,  4, }, // Africa/Lome
    {  545, 5211,  170636,  7, }, // Africa/Luanda
    {  545, 4939,  170643, 10, }, // Africa/Lubumbashi
    {  545, 5001,  170653,  6, }, // Africa/Lusaka
    {  545, 5322,  170659,  4, }, // Africa/Malabo
    {  545, 4987,  170663,  5, }, // Africa/Maputo
    {  545, 5197,  170668,  5, }, // Africa/Maseru
    {  545, 5182,  170673,  7, }, // Africa/Mbabane
    {  545, 5113,  170680,  7, }, // Africa/Mogadishu
    {  545, 7034,  170687, 10, }, // Africa/Monrovia
    {  545, 3853,  170697,  7, }, // Africa/Nairobi
    {  545, 5271,  170704,  7, }, // Africa/Ndjamena
    {  545, 5354,  170711,  6, }, // Africa/Niamey
    {  545, 6803,  170717,  9, }, // Africa/Nouakchott
    {  545, 6727,  170726,  7, }, // Africa/Ouagadougou
    {  545, 5225,  170733, 11, }, // Africa/Porto-Novo
    {  545, 4435,  170744,  8, }, // Africa/Sao_Tome
    {  545, 2866,  170752,  9, }, // Africa/Tripoli
    {  545, 6554,  170761,  8, }, // Africa/Tunis
    {  545, 4152,  170769, 11, }, // Africa/Windhoek
    {  545,  132,  170780,  4, }, // America/Adak
    {  545, 3346,  170784,  8, }, // America/Anchorage
    {  545, 5607,  170792, 10, }, // America/Anguilla
    {  545, 5624,  170802,  9, }, // America/Antigua
    {  545, 4554,  170811,  9, }, // America/Araguaina
    {  545,  166,  170820, 14, }, // America/Argentina/Buenos_Aires
    {  545,   91,  170834,  7, }, // America/Argentina/Catamarca
    {  545,  270,  170841,  8, }, // America/Argentina/Cordoba
    {  545,  441,  170849,  6, }, // America/Argentina/Jujuy
    {  545, 7462,  170855,  9, }, // America/Argentina/La_Rioja
    {  545,  565,  170864,  8, }, // America/Argentina/Mendoza
    {  545, 7489,  170872, 12, }, // America/Argentina/Rio_Gallegos
    {  545, 7520,  170884,  6, }, // America/Argentina/Salta
    {  545, 7544,  170890, 11, }, // America/Argentina/San_Juan
    {  545, 5580,  170901, 11, }, // America/Argentina/San_Luis
    {  545, 7571,  170912,  7, }, // America/Argentina/Tucuman
    {  545, 7597,  170919,  8, }, // America/Argentina/Ushuaia
    {  545, 5640,  170927,  5, }, // America/Aruba
    {  545, 4252,  170932, 11, }, // America/Asuncion
    {  545,  237,  170943,  8, }, // America/Atikokan
    {  545, 3679,  170951,  5, }, // America/Bahia
    {  545, 7623,  170956, 15, }, // America/Bahia_Banderas
    {  545, 5654,  170971,  7, }, // America/Barbados
    {  545, 7646,  170978,  6, }, // America/Belem
    {  545, 5407,  170984,  6, }, // America/Belize
    {  545, 7660,  170990, 16, }, // America/Blanc-Sablon
    {  545, 7681,  171006, 10, }, // America/Boa_Vista
    {  545, 4361,  171016,  6, }, // America/Bogota
    {  545, 7699,  171022,  6, }, // America/Boise
    {  545, 7713,  171028, 17, }, // America/Cambridge_Bay
    {  545, 7735,  171045, 15, }, // America/Campo_Grande
    {  545, 3868,  171060,  8, }, // America/Cancun
    {  545, 4694,  171068,  5, }, // America/Caracas
    {  545, 4345,  171073,  6, }, // America/Cayenne
    {  545, 5496,  171079,  5, }, // America/Cayman
    {  545, 2260,  171084,  6, }, // America/Chicago
    {  545, 7756,  171090,  8, }, // America/Chihuahua
    {  545, 7774,  171098, 17, }, // America/Ciudad_Juarez
    {  545, 5422,  171115,  9, }, // America/Costa_Rica
    {  545, 7796,  171124, 10, }, // America/Creston
    {  545, 3791,  171134,  7, }, // America/Cuiaba
    {  545, 5723,  171141,  7, }, // America/Curacao
    {  545, 7812,  171148, 15, }, // America/Danmarkshavn
    {  545, 7833,  171163,  5, }, // America/Dawson
    {  545, 7848,  171168, 13, }, // America/Dawson_Creek
    {  545,  805,  171181,  7, }, // America/Denver
    {  545, 3465,  171188,  8, }, // America/Detroit
    {  545, 5739,  171196,  8, }, // America/Dominica
    {  545,  893,  171204,  9, }, // America/Edmonton
    {  545, 7869,  171213,  9, }, // America/Eirunepe
    {  545, 5441,  171222, 12, }, // America/El_Salvador
    {  545, 7886,  171234, 12, }, // America/Fort_Nelson
    {  545, 7906,  171246,  9, }, // America/Fortaleza
    {  545, 7924,  171255, 12, }, // America/Glace_Bay
    {  545, 6881,  171267, 10, }, // America/Goose_Bay
    {  545, 4612,  171277, 14, }, // America/Grand_Turk
    {  545, 5770,  171291,  8, }, // America/Grenada
    {  545, 5786,  171299, 10, }, // America/Guadeloupe
    {  545, 3760,  171309,  8, }, // America/Guatemala
    {  545, 4841,  171317, 11, }, // America/Guayaquil
    {  545, 6932,  171328,  5, }, // America/Guyana
    {  545, 1939,  171333,  9, }, // America/Halifax
    {  545, 2281,  171342,  5, }, // America/Havana
    {  545, 7942,  171347,  8, }, // America/Hermosillo
    {  545,  348,  171355, 14, }, // America/Indiana/Indianapolis
    {  545,  481,  171369, 17, }, // America/Indiana/Knox
    {  545, 7961,  171386, 18, }, // America/Indiana/Marengo
    {  545, 7985,  171404, 23, }, // America/Indiana/Petersburg
    {  545, 8012,  171427, 20, }, // America/Indiana/Tell_City
    {  545, 8038,  171447, 15, }, // America/Indiana/Vevay
    {  545, 8060,  171462, 19, }, // America/Indiana/Vincennes
    {  545, 8086,  171481, 18, }, // America/Indiana/Winamac
    {  545, 8110,  171499,  7, }, // America/Inuvik
    {  545,  660,  171506,  9, }, // America/Iqaluit
    {  545, 2799,  171515,  8, }, // America/Jamaica
    {  545, 5380,  171523,  4, }, // America/Juneau
    {  545,  521,  171527, 10, }, // America/Kentucky/Louisville
    {  545, 8125,  171537, 19, }, // America/Kentucky/Monticello
    {  545, 5704,  171556, 14, }, // America/Kralendijk
    {  545, 4376,  171570,  6, }, // America/La_Paz
    {  545, 7169,  171576,  4, }, // America/Lima
    {  545, 3239,  171580, 13, }, // America/Los_Angeles
    {  545, 5932,  171593, 24, }, // America/Lower_Princes
    {  545, 8153,  171617,  7, }, // America/Maceio
    {  545, 8168,  171624,  5, }, // America/Managua
    {  545, 1908,  171629,  6, }, // America/Manaus
    {  545, 5897,  171635,  8, }, // America/Marigot
    {  545, 5805,  171643,  8, }, // America/Martinique
    {  545, 8184,  171651,  8, }, // America/Matamoros
    {  545, 2917,  171659,  8, }, // America/Mazatlan
    {  545, 8202,  171667,  8, }, // America/Menominee
    {  545, 8220,  171675,  6, }, // America/Merida
    {  545, 8235,  171681, 10, }, // America/Metlakatla
    {  545, 2949,  171691, 13, }, // America/Mexico_City
    {  545, 4391,  171704,  9, }, // America/Miquelon
    {  545, 8254,  171713,  9, }, // America/Moncton
    {  545, 8270,  171722,  8, }, // America/Monterrey
    {  545, 4098,  171730, 12, }, // America/Montevideo
    {  545, 5824,  171742, 12, }, // America/Montserrat
    {  545, 5481,  171754,  5, }, // America/Nassau
    {  545, 2379,  171759,  8, }, // America/New_York
    {  545, 8288,  171767,  4, }, // America/Nome
    {  545, 1850,  171771,  8, }, // America/Noronha
    {  545, 8301,  171779, 19, }, // America/North_Dakota/Beulah
    {  545, 8329,  171798, 19, }, // America/North_Dakota/Center
    {  545, 8357,  171817, 22, }, // America/North_Dakota/New_Salem
    {  545,  393,  171839,  4, }, // America/Nuuk
    {  545, 8388,  171843,  6, }, // America/Ojinaga
    {  545, 2356,  171849,  5, }, // America/Panama
    {  545, 6173,  171854,  9, }, // America/Paramaribo
    {  545, 2973,  171863,  8, }, // America/Phoenix
    {  545, 3945,  171871, 18, }, // America/Port-au-Prince
    {  545, 5954,  171889, 19, }, // America/Port_of_Spain
    {  545, 8404,  171908, 13, }, // America/Porto_Velho
    {  545, 5843,  171921, 10, }, // America/Puerto_Rico
    {  545, 4030,  171931, 14, }, // America/Punta_Arenas
    {  545, 8424,  171945, 16, }, // America/Rankin_Inlet
    {  545, 8445,  171961,  6, }, // America/Recife
    {  545, 1995,  171967,  6, }, // America/Regina
    {  545, 8460,  171973,  8, }, // America/Resolute
    {  545,  695,  171981, 13, }, // America/Rio_Branco
    {  545, 8477,  171994,  8, }, // America/Santarem
    {  545, 2201,  172002,  9, }, // America/Santiago
    {  545, 6111,  172011, 15, }, // America/Santo_Domingo
    {  545, 1878,  172026,  9, }, // America/Sao_Paulo
    {  545, 6899,  172035, 13, }, // America/Scoresbysund
    {  545, 8494,  172048,  6, }, // America/Sitka
    {  545, 8508,  172054, 14, }, // America/St_Barthelemy
    {  545, 2061,  172068, 12, }, // America/St_Johns
    {  545, 5863,  172080, 12, }, // America/St_Kitts
    {  545, 5880,  172092, 12, }, // America/St_Lucia
    {  545,  855,  172104, 11, }, // America/St_Thomas
    {  545, 5913,  172115, 15, }, // America/St_Vincent
    {  545, 8530,  172130, 15, }, // America/Swift_Current
    {  545, 5461,  172145, 11, }, // America/Tegucigalpa
    {  545, 5756,  172156,  4, }, // America/Thule
    {  545,  313,  172160,  7, }, // America/Tijuana
    {  545,  608,  172167,  7, }, // America/Toronto
    {  545, 5688,  172174,  8, }, // America/Tortola
    {  545, 2093,  172182,  9, }, // America/Vancouver
    {  545, 2144,  172191, 11, }, // America/Whitehorse
    {  545,  734,  172202,  8, }, // America/Winnipeg
    {  545, 8552,  172210,  6, }, // America/Yakutat
    {  545, 6001,  172216,  4, }, // Antarctica/Casey
    {  545, 6094,  172220,  6, }, // Antarctica/Davis
    {  545, 6133,  172226, 18, }, // Antarctica/DumontDUrville
    {  545, 8568,  172244,  8, }, // Antarctica/Macquarie
    {  545, 7066,  172252,  6, }, // Antarctica/Mawson
    {  545, 7113,  172258, 10, }, // Antarctica/McMurdo
    {  545, 8589,  172268,  7, }, // Antarctica/Palmer
    {  545, 7226,  172275,  6, }, // Antarctica/Rothera
    {  545, 7295,  172281,  6, }, // Antarctica/Syowa
    {  545, 8607,  172287,  7, }, // Antarctica/Troll
    {  545, 7374,  172294,  8, }, // Antarctica/Vostok
    {  545, 1429,  172302, 16, }, // Arctic/Longyearbyen
    {  545, 5570,  172318,  5, }, // Asia/Aden
    {  545, 5395,  172323,  6, }, // Asia/Almaty
    {  545, 3968,  172329,  7, }, // Asia/Amman
    {  545, 5511,  172336,  9, }, // Asia/Anadyr
    {  545, 5523,  172345,  5, }, // Asia/Aqtau
    {  545, 5368,  172350,  7, }, // Asia/Aqtobe
    {  545,  964,  172357,  8, }, // Asia/Ashgabat
    {  545, 8624,  172365,  6, }, // Asia/Atyrau
    {  545, 3607,  172371,  8, }, // Asia/Baghdad
    {  545, 5534,  172379,  6, }, // Asia/Bahrain
    {  545, 3653,  171139,  4, }, // Asia/Baku
    {  545, 4466,  172385,  7, }, // Asia/Bangkok
    {  545, 3571,  172392,  7, }, // Asia/Barnaul
    {  545, 4086,  172399,  6, }, // Asia/Beirut
    {  545, 3778,  172405,  8, }, // Asia/Bishkek
    {  545, 5989,  172413,  6, }, // Asia/Brunei
    {  545, 4601,  172419,  4, }, // Asia/Chita
    {  545, 4511,  172423,  4, }, // Asia/Colombo
    {  545, 4540,  172427,  8, }, // Asia/Damascus
    {  545, 1093,  172435,  3, }, // Asia/Dhaka
    {  545, 6192,  172438,  4, }, // Asia/Dili
    {  545, 3596,  172442,  6, }, // Asia/Dubai
    {  545, 6159,  172448,  7, }, // Asia/Dushanbe
    {  545, 8636,  172455,  9, }, // Asia/Famagusta
    {  545, 8651,  172464,  4, }, // Asia/Gaza
    {  545, 4795,  172468,  8, }, // Asia/Hebron
    {  545, 1259,  172476, 13, }, // Asia/Ho_Chi_Minh
    {  545, 2704,  172489,  6, }, // Asia/Hong_Kong
    {  545, 4771,  172495,  6, }, // Asia/Hovd
    {  545, 4184,  172501, 10, }, // Asia/Irkutsk
    {  545, 7006,  172511,  6, }, // Asia/Jakarta
    {  545, 6992,  172517,  5, }, // Asia/Jayapura
    {  545, 1290,  172522,  7, }, // Asia/Jerusalem
    {  545, 3560,  172529,  6, }, // Asia/Kabul
    {  545, 4316,  172535,  8, }, // Asia/Kamchatka
    {  545, 4239,  172543,  8, }, // Asia/Karachi
    {  545, 1185,  172551,  8, }, // Asia/Kathmandu
    {  545, 8661,  172559,  7, }, // Asia/Khandyga
    {  545,  992,  172566,  6, }, // Asia/Kolkata
    {  545, 4197,  172572, 16, }, // Asia/Krasnoyarsk
    {  545, 4859,  172588, 13, }, // Asia/Kuala_Lumpur
    {  545, 5976,  172601,  5, }, // Asia/Kuching
    {  545, 5547,  172606,  6, }, // Asia/Kuwait
    {  545, 1211,  172612,  6, }, // Asia/Macau
    {  545, 4017,  172618,  5, }, // Asia/Magadan
    {  545, 1349,  172623,  9, }, // Asia/Makassar
    {  545, 7182,  172632,  6, }, // Asia/Manila
    {  545, 6920,  172638,  6, }, // Asia/Muscat
    {  545, 2554,  172644,  8, }, // Asia/Nicosia
    {  545, 8675,  172652, 16, }, // Asia/Novokuznetsk
    {  545, 4135,  172668, 13, }, // Asia/Novosibirsk
    {  545, 4229,  172681,  7, }, // Asia/Omsk
    {  545, 7145,  172688,  5, }, // Asia/Oral
    {  545, 6961,  172693,  9, }, // Asia/Phnom_Penh
    {  545, 8693,  172702, 10, }, // Asia/Pontianak
    {  545, 4214,  172712, 14, }, // Asia/Pyongyang
    {  545, 5559,  172726,  5, }, // Asia/Qatar
    {  545, 8708,  172731,  8, }, // Asia/Qostanay
    {  545, 4269,  172739, 12, }, // Asia/Qyzylorda
    {  545, 3584,  172751,  6, }, // Asia/Riyadh
    {  545, 4408,  172757,  8, }, // Asia/Sakhalin
    {  545, 7245,  172765, 11, }, // Asia/Samarkand
    {  545, 3279,  172776,  4, }, // Asia/Seoul
    {  545, 1053,  172780,  7, }, // Asia/Shanghai
    {  545, 3300,  172787, 11, }, // Asia/Singapore
    {  545, 4297,  172798, 18, }, // Asia/Srednekolymsk
    {  545, 3263,  172816,  6, }, // Asia/Taipei
    {  545, 4781,  172822,  9, }, // Asia/Tashkent
    {  545, 3915,  172831,  8, }, // Asia/Tbilisi
    {  545, 2772,  172839,  8, }, // Asia/Tehran
    {  545, 1317,  172847,  6, }, // Asia/Thimphu
    {  545, 2821,  172853,  6, }, // Asia/Tokyo
    {  545, 4572,  172859,  8, }, // Asia/Tomsk
    {  545, 1021,  172867, 10, }, // Asia/Ulaanbaatar
    {  545, 1159,  172877,  7, }, // Asia/Urumqi
    {  545, 8722,  172884, 10, }, // Asia/Ust-Nera
    {  545, 6977,  172894, 11, }, // Asia/Vientiane
    {  545, 4710,  172905, 13, }, // Asia/Vladivostok
    {  545, 4828,  172918,  9, }, // Asia/Yakutsk
    {  545, 1235,  172927,  7, }, // Asia/Yangon
    {  545, 3883,  172934, 13, }, // Asia/Yekaterinburg
    {  545, 3747,  172947,  7, }, // Asia/Yerevan
    {  545, 3663,  172954,  5, }, // Atlantic/Azores
    {  545, 5671,  172959,  9, }, // Atlantic/Bermuda
    {  545, 6628,  172968,  5, }, // Atlantic/Canary
    {  545, 3727,  172973, 10, }, // Atlantic/Cape_Verde
    {  545, 1395,  172983,  4, }, // Atlantic/Faroe
    {  545, 8736,  172987,  7, }, // Atlantic/Madeira
    {  545, 2748,  172994, 12, }, // Atlantic/Reykjavik
    {  545, 7272,  173006, 15, }, // Atlantic/South_Georgia
    {  545, 6821,  173021, 10, }, // Atlantic/St_Helena
    {  545, 6644,  173031,  8, }, // Atlantic/Stanley
    {  545, 1670,  173039,  7, }, // Australia/Adelaide
    {  545, 1635,  173046, 11, }, // Australia/Brisbane
    {  545, 1799,  173057, 13, }, // Australia/Broken_Hill
    {  545, 1583,  173070,  6, }, // Australia/Darwin
    {  545, 3637,  173076,  7, }, // Australia/Eucla
    {  545, 1516,  173083,  8, }, // Australia/Hobart
    {  545, 8753,  173091,  9, }, // Australia/Lindeman
    {  545, 1547,  173100, 11, }, // Australia/Lord_Howe
    {  545, 1727,  173111, 10, }, // Australia/Melbourne
    {  545, 1762,  173121,  5, }, // Australia/Perth
    {  545, 1463,  173126,  6, }, // Australia/Sydney
    {  545, 6441,  173132, 12, }, // Europe/Amsterdam
    {  545, 6216,  173144,  8, }, // Europe/Andorra
    {  545, 3620,  173152, 11, }, // Europe/Astrakhan
    {  545, 2300,  173163,  7, }, // Europe/Athens
    {  545, 6336,  173170, 13, }, // Europe/Belgrade
    {  545, 4757,  173183,  7, }, // Europe/Berlin
    {  545, 6488,  173190, 11, }, // Europe/Bratislava
    {  545, 2167,  173201,  9, }, // Europe/Brussels
    {  545, 3928,  173210, 10, }, // Europe/Bucharest
    {  545, 3806,  173220, 10, }, // Europe/Budapest
    {  545, 8772,  173230, 10, }, // Europe/Busingen
    {  545, 2583,  173240,  6, }, // Europe/Chisinau
    {  545, 6289,  173246, 10, }, // Europe/Copenhagen
    {  545, 2338,  173256,  7, }, // Europe/Dublin
    {  545, 6307,  173263, 10, }, // Europe/Gibraltar
    {  545, 8788,  173273,  7, }, // Europe/Guernsey
    {  545, 6612,  173280, 10, }, // Europe/Helsinki
    {  545, 8804,  173290, 14, }, // Europe/Isle_of_Man
    {  545, 1130,  173304, 11, }, // Europe/Istanbul
    {  545, 8823,  173315,  5, }, // Europe/Jersey
    {  545, 3979,  173320, 14, }, // Europe/Kaliningrad
    {  545, 8929,  173334,  6, }, // Europe/Kirov
    {  545, 2527,  173340,  4, }, // Europe/Kyiv
    {  545, 3213,  173344,  7, }, // Europe/Lisbon
    {  545, 6506,  173351, 11, }, // Europe/Ljubljana
    {  545, 2501,  173362,  6, }, // Europe/London
    {  545, 6365,  173368, 12, }, // Europe/Luxembourg
    {  545, 4892,  173380,  8, }, // Europe/Madrid
    {  545, 6397,  173388,  6, }, // Europe/Malta
    {  545, 6582,  173394, 10, }, // Europe/Mariehamn
    {  545, 3693,  173404,  8, }, // Europe/Minsk
    {  545, 6410,  173412,  6, }, // Europe/Monaco
    {  545, 3537,  173418,  7, }, // Europe/Moscow
    {  545, 6458,  173425,  5, }, // Europe/Oslo
    {  545, 4284,  173430,  6, }, // Europe/Paris
    {  545, 6424,  173436, 10, }, // Europe/Podgorica
    {  545, 6275,  173446,  7, }, // Europe/Prague
    {  545, 8837,  173453,  4, }, // Europe/Riga
    {  545, 6324,  173457,  4, }, // Europe/Rome
    {  545, 4331,  173461,  4, }, // Europe/Samara
    {  545, 6470,  173465, 11, }, // Europe/San_Marino
    {  545, 6245,  173476,  7, }, // Europe/Sarajevo
    {  545, 4451,  173483,  6, }, // Europe/Saratov
    {  545, 8849,  173489, 12, }, // Europe/Simferopol
    {  545, 6383,  173501,  8, }, // Europe/Skopje
    {  545, 6599,  173509,  6, }, // Europe/Sofia
    {  545, 6523,  173515, 10, }, // Europe/Stockholm
    {  545, 8867,  173525,  5, }, // Europe/Tallinn
    {  545, 6202,  173530,  6, }, // Europe/Tirane
    {  545, 8882,  173536, 12, }, // Europe/Ulyanovsk
    {  545, 6352,  173548,  5, }, // Europe/Vaduz
    {  545, 6567,  173553,  8, }, // Europe/Vatican
    {  545, 6231,  173561,  5, }, // Europe/Vienna
    {  545, 8899,  173566, 10, }, // Europe/Vilnius
    {  545, 4727,  173576, 13, }, // Europe/Volgograd
    {  545, 3190,  173589,  5, }, // Europe/Warsaw
    {  545, 6261,  173594,  7, }, // Europe/Zagreb
    {  545, 6540,  173601,  6, }, // Europe/Zurich
    {  545, 5078,  173607, 11, }, // Indian/Antananarivo
    {  545, 6947,  173618,  6, }, // Indian/Chagos
    {  545, 6046,  173624, 10, }, // Indian/Christmas
    {  545, 6063,  173634,  6, }, // Indian/Cocos
    {  545, 5029,  173640,  6, }, // Indian/Comoro
    {  545, 6661,  173646, 11, }, // Indian/Kerguelen
    {  545, 7260,  173657,  4, }, // Indian/Mahe
    {  545, 7050,  173661, 10, }, // Indian/Maldives
    {  545, 4069,  173671,  7, }, // Indian/Mauritius
    {  545, 5098,  173678,  5, }, // Indian/Mayotte
    {  545, 7211,  173683,  9, }, // Indian/Reunion
    {  545, 4422,  173692,  5, }, // Pacific/Apia
    {  545,  932,  173697,  8, }, // Pacific/Auckland
    {  545, 3706,  173705,  9, }, // Pacific/Bougainville
    {  545, 3015,  173714,  5, }, // Pacific/Chatham
    {  545, 3157,  173719,  4, }, // Pacific/Chuuk
    {  545, 2237,  173723,  6, }, // Pacific/Easter
    {  545, 7360,  173729,  5, }, // Pacific/Efate
    {  545, 7327,  173734,  7, }, // Pacific/Fakaofo
    {  545, 3902,  173741,  4, }, // Pacific/Fiji
    {  545, 7343,  173745,  7, }, // Pacific/Funafuti
    {  545, 6678,  173752,  7, }, // Pacific/Galapagos
    {  545, 6696,  173759,  9, }, // Pacific/Gambier
    {  545, 3822,  173768, 12, }, // Pacific/Guadalcanal
    {  545, 6018,  173780,  6, }, // Pacific/Guam
    {  545, 2723,  173786,  8, }, // Pacific/Honolulu
    {  545, 3049,  173794,  9, }, // Pacific/Kanton
    {  545, 3998,  173803,  7, }, // Pacific/Kiritimati
    {  545, 7019,  173810,  6, }, // Pacific/Kosrae
    {  545, 2842,  173816, 11, }, // Pacific/Kwajalein
    {  545, 4877,  173827,  5, }, // Pacific/Majuro
    {  545, 4051,  173832, 10, }, // Pacific/Marquesas
    {  545, 8914,  173842,  6, }, // Pacific/Midway
    {  545, 7084,  173848,  5, }, // Pacific/Nauru
    {  545, 7132,  173570,  4, }, // Pacific/Niue
    {  545, 4168,  173853,  6, }, // Pacific/Norfolk
    {  545, 7098,  173859,  6, }, // Pacific/Noumea
    {  545, 3126,  173865,  7, }, // Pacific/Pago_Pago
    {  545, 7155,  173872,  5, }, // Pacific/Palau
    {  545, 7194,  173877, 10, }, // Pacific/Pitcairn
    {  545, 3096,  173887,  8, }, // Pacific/Pohnpei
    {  545, 4807,  173895, 15, }, // Pacific/Port_Moresby
    {  545, 6076,  173910,  9, }, // Pacific/Rarotonga
    {  545, 6031,  173919,  7, }, // Pacific/Saipan
    {  545, 7312,  173926,  5, }, // Pacific/Tahiti
    {  545, 6712,  173931,  5, }, // Pacific/Tarawa
    {  545, 4583,  173936,  9, }, // Pacific/Tongatapu
    {  545, 7392,  173945,  4, }, // Pacific/Wake
    {  545, 7405,  173949,  6, }, // Pacific/Wallis
    {  546,   14,       0,  6, }, // Africa/Asmara Skolt Sami/Latin/Finland
    {  546, 4435,       6,  8, }, // Africa/Sao_Tome
    {  546, 4252,      14,  8, }, // America/Asuncion
    {  546,  237,      22,  8, }, // America/Atikokan
    {  546, 7623,      30, 17, }, // America/Bahia_Banderas
    {  546, 3868,      47,  6, }, // America/Cancun
    {  546, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  546, 5723,      66,  7, }, // America/Curacao
    {  546,  481,      73, 13, }, // America/Indiana/Knox
    {  546, 7961,      86, 16, }, // America/Indiana/Marengo
    {  546, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  546, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  546, 8038,     139, 14, }, // America/Indiana/Vevay
    {  546, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  546, 8086,     171, 16, }, // America/Indiana/Winamac
    {  546, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  546, 5932,     207, 22, }, // America/Lower_Princes
    {  546, 8220,     229,  6, }, // America/Merida
    {  546, 1850,     235, 19, }, // America/Noronha
    {  546, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  546, 8329,     274, 20, }, // America/North_Dakota/Center
    {  546, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  546,  393,     317,  4, }, // America/Nuuk
    {  546, 6899,     321, 16, }, // America/Scoresbysund
    {  546, 8508,     337, 14, }, // America/St_Barthelemy
    {  546, 2061,     351, 10, }, // America/St_Johns
    {  546, 5863,     361,  9, }, // America/St_Kitts
    {  546, 5880,     370,  9, }, // America/St_Lucia
    {  546,  855,     379, 10, }, // America/St_Thomas
    {  546, 5913,     389, 11, }, // America/St_Vincent
    {  546, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  546, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  546, 1185,     448,  9, }, // Asia/Kathmandu
    {  546,  992,     457,  7, }, // Asia/Kolkata
    {  546, 1211,     464,  5, }, // Asia/Macau
    {  546, 1235,     477,  6, }, // Asia/Yangon
    {  546, 1395,     483,  5, }, // Atlantic/Faroe
    {  546, 6821,     488, 10, }, // Atlantic/St_Helena
    {  546, 2527,     514,  4, }, // Europe/Kyiv
    {  546, 7211,     547,  7, }, // Indian/Reunion
    {  546, 3157,     554,  5, }, // Pacific/Chuuk
    {  546, 3096,     586,  7, }, // Pacific/Pohnpei
    {  547, 6760,   21292,  5, }, // Africa/Accra Slovak/Latin/Slovakia
    {  547, 5059,    3902, 11, }, // Africa/Addis_Ababa
    {  547, 7420,   57703,  5, }, // Africa/Algiers
    {  547,   14,       0,  6, }, // Africa/Asmara
    {  547, 2320,   57708,  6, }, // Africa/Cairo
    {  547, 6773,   26485,  7, }, // Africa/Conakry
    {  547, 5043,   33286,  7, }, // Africa/Djibouti
    {  547, 5166,  173955,  8, }, // Africa/El_Aaiun
    {  547, 4524,   57714,  7, }, // Africa/Khartoum
    {  547, 6869,   20418,  4, }, // Africa/Lome
    {  547, 5113,  173963,  8, }, // Africa/Mogadishu
    {  547, 5271,   44128,  9, }, // Africa/Ndjamena
    {  547, 4435,  173971, 11, }, // Africa/Sao_Tome
    {  547, 2866,   57756,  8, }, // Africa/Tripoli
    {  547,  270,   20479,  7, }, // America/Argentina/Cordoba
    {  547, 4252,      14,  8, }, // America/Asuncion
    {  547,  237,      22,  8, }, // America/Atikokan
    {  547, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  547, 7646,   35896,  5, }, // America/Belem
    {  547, 4361,   20507,  6, }, // America/Bogota
    {  547, 3868,      47,  6, }, // America/Cancun
    {  547, 5496,  173982, 16, }, // America/Cayman
    {  547, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  547, 5422,   33320,  9, }, // America/Costa_Rica
    {  547, 3791,   26574,  6, }, // America/Cuiaba
    {  547, 5723,      66,  7, }, // America/Curacao
    {  547, 5739,   21666,  8, }, // America/Dominica
    {  547, 7869,   26580,  8, }, // America/Eirunepe
    {  547, 5441,  173998,  8, }, // America/El_Salvador
    {  547,  481,      73, 13, }, // America/Indiana/Knox
    {  547, 7961,      86, 16, }, // America/Indiana/Marengo
    {  547, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  547, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  547, 8038,     139, 14, }, // America/Indiana/Vevay
    {  547, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  547, 8086,     171, 16, }, // America/Indiana/Winamac
    {  547, 2799,   33347,  7, }, // America/Jamaica
    {  547, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  547, 5932,     207, 22, }, // America/Lower_Princes
    {  547, 8153,   26588,  6, }, // America/Maceio
    {  547, 5805,    5260,  8, }, // America/Martinique
    {  547, 2917,   26603,  8, }, // America/Mazatlan
    {  547, 8220,     229,  6, }, // America/Merida
    {  547, 2949,   57601,  6, }, // America/Mexico_City
    {  547, 1850,     235, 19, }, // America/Noronha
    {  547, 8301,  174006, 22, }, // America/North_Dakota/Beulah
    {  547, 8329,  174028, 22, }, // America/North_Dakota/Center
    {  547, 8357,  174050, 25, }, // America/North_Dakota/New_Salem
    {  547,  393,     317,  4, }, // America/Nuuk
    {  547, 5843,   33426,  9, }, // America/Puerto_Rico
    {  547, 8477,   26704,  8, }, // America/Santarem
    {  547, 1878,   26712,  9, }, // America/Sao_Paulo
    {  547, 6899,     321, 16, }, // America/Scoresbysund
    {  547, 8508,  174075, 16, }, // America/St_Barthelemy
    {  547, 2061,     351, 10, }, // America/St_Johns
    {  547, 5863,     361,  9, }, // America/St_Kitts
    {  547, 5880,  174091, 11, }, // America/St_Lucia
    {  547,  855,  174102,  9, }, // America/St_Thomas
    {  547, 5913,  174111, 11, }, // America/St_Vincent
    {  547, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  547, 7295,  174122,  4, }, // Antarctica/Syowa
    {  547, 5395,  174126,  8, }, // Asia/Almaty
    {  547, 3968,   57924,  5, }, // Asia/Amman
    {  547, 5523,    5609,  5, }, // Asia/Aqtau
    {  547, 5368,    7606,  6, }, // Asia/Aqtobe
    {  547,  964,   57929,  8, }, // Asia/Ashgabat
    {  547, 3607,    4162,  6, }, // Asia/Baghdad
    {  547, 5534,   57943,  7, }, // Asia/Bahrain
    {  547, 4086,   57950,  6, }, // Asia/Beirut
    {  547, 3778,   33455,  6, }, // Asia/Bishkek
    {  547, 5989,   33461,  6, }, // Asia/Brunei
    {  547, 4601,   57622,  4, }, // Asia/Chita
    {  547, 4511,    5637,  7, }, // Asia/Colombo
    {  547, 4540,    4201,  6, }, // Asia/Damascus
    {  547, 1093,   57963,  5, }, // Asia/Dhaka
    {  547, 3596,   57968,  5, }, // Asia/Dubai
    {  547, 6159,   33468,  7, }, // Asia/Dushanbe
    {  547, 1259,  174134, 16, }, // Asia/Ho_Chi_Minh
    {  547, 2704,    4231,  8, }, // Asia/Hong_Kong
    {  547, 4771,  174150,  5, }, // Asia/Hovd
    {  547, 1290,   33501,  9, }, // Asia/Jerusalem
    {  547, 3560,   58000,  5, }, // Asia/Kabul
    {  547, 4316,   33510,  8, }, // Asia/Kamchatka
    {  547, 4239,  174155,  6, }, // Asia/Karachi
    {  547, 1185,  174161,  9, }, // Asia/Kathmandu
    {  547, 8661,   58020,  8, }, // Asia/Khandyga
    {  547,  992,   58028,  7, }, // Asia/Kolkata
    {  547, 4197,    4288, 11, }, // Asia/Krasnoyarsk
    {  547, 5976,   33531,  6, }, // Asia/Kuching
    {  547, 5547,    7725,  6, }, // Asia/Kuwait
    {  547, 1211,     464,  5, }, // Asia/Macau
    {  547, 6920,   26882,  6, }, // Asia/Muscat
    {  547, 2554,  174170,  7, }, // Asia/Nicosia
    {  547, 8675,   62465, 11, }, // Asia/Novokuznetsk
    {  547, 7145,  174177,  6, }, // Asia/Oral
    {  547, 6961,  174183, 10, }, // Asia/Phnom_Penh
    {  547, 4214,   58068, 11, }, // Asia/Pyongyang
    {  547, 5559,    4332,  5, }, // Asia/Qatar
    {  547, 8708,    7770,  8, }, // Asia/Qostanay
    {  547, 4269,   26897,  9, }, // Asia/Qyzylorda
    {  547, 3584,   58079,  5, }, // Asia/Riyadh
    {  547, 4408,   58084,  8, }, // Asia/Sakhalin
    {  547, 3279,   58092,  4, }, // Asia/Seoul
    {  547, 1053,   58096,  7, }, // Asia/Shanghai
    {  547, 3300,   20900,  8, }, // Asia/Singapore
    {  547, 3263,   58116,  9, }, // Asia/Taipei
    {  547, 4781,   33589,  7, }, // Asia/Tashkent
    {  547, 2772,   58125,  7, }, // Asia/Tehran
    {  547, 2821,    4356,  5, }, // Asia/Tokyo
    {  547, 1021,  174193,  9, }, // Asia/Ulaanbaatar
    {  547, 1159,   33596,  6, }, // Asia/Urumqi
    {  547, 8722,  174202,  8, }, // Asia/Ust-Nera
    {  547, 6977,   26939,  8, }, // Asia/Vientiane
    {  547, 4828,    7873,  7, }, // Asia/Yakutsk
    {  547, 1235,   20937,  6, }, // Asia/Yangon
    {  547, 3883,    4380, 13, }, // Asia/Yekaterinburg
    {  547, 3747,    7898,  7, }, // Asia/Yerevan
    {  547, 3663,  157542,  5, }, // Atlantic/Azores
    {  547, 5671,   58175,  7, }, // Atlantic/Bermuda
    {  547, 6628,  174210, 16, }, // Atlantic/Canary
    {  547, 3727,   58198,  8, }, // Atlantic/Cape_Verde
    {  547, 1395,   58206, 15, }, // Atlantic/Faroe
    {  547, 2748,   36176,  9, }, // Atlantic/Reykjavik
    {  547, 7272,  174226, 13, }, // Atlantic/South_Georgia
    {  547, 6821,  174239, 12, }, // Atlantic/St_Helena
    {  547, 3620,  174251,  9, }, // Europe/Astrakhan
    {  547, 2300,  174260,  5, }, // Europe/Athens
    {  547, 6336,  174265,  8, }, // Europe/Belgrade
    {  547, 4757,   21059,  6, }, // Europe/Berlin
    {  547, 2167,   27022,  6, }, // Europe/Brussels
    {  547, 3928,   58269,  8, }, // Europe/Bucharest
    {  547, 3806,   58277,  8, }, // Europe/Budapest
    {  547, 8772,   27037,  8, }, // Europe/Busingen
    {  547, 2583,  174273,  7, }, // Europe/Chisinau
    {  547, 6289,   58292,  5, }, // Europe/Copenhagen
    {  547, 6307,   91982,  9, }, // Europe/Gibraltar
    {  547, 8804,   58305, 10, }, // Europe/Isle_of_Man
    {  547, 2527,   58315,  5, }, // Europe/Kyiv
    {  547, 3213,   33728,  7, }, // Europe/Lisbon
    {  547, 6506,  174280,  7, }, // Europe/Ljubljana
    {  547, 2501,   58326,  6, }, // Europe/London
    {  547, 6365,  174287, 11, }, // Europe/Luxembourg
    {  547, 6410,    6044,  6, }, // Europe/Monaco
    {  547, 3537,   22880,  6, }, // Europe/Moscow
    {  547, 4284,  174298,  5, }, // Europe/Paris
    {  547, 6275,   36285,  5, }, // Europe/Prague
    {  547, 6324,  174303,  3, }, // Europe/Rome
    {  547, 6470,  142960, 10, }, // Europe/San_Marino
    {  547, 6523,   33767,  8, }, // Europe/Stockholm
    {  547, 6202,   21213,  6, }, // Europe/Tirane
    {  547, 8882,  174306,  9, }, // Europe/Ulyanovsk
    {  547, 6567,   58354,  7, }, // Europe/Vatican
    {  547, 6231,  174315,  6, }, // Europe/Vienna
    {  547, 3190,   33778,  7, }, // Europe/Warsaw
    {  547, 6261,  174321,  6, }, // Europe/Zagreb
    {  547, 6540,    4577,  6, }, // Europe/Zurich
    {  547, 6046,  174327, 15, }, // Indian/Christmas
    {  547, 6063,   58392, 16, }, // Indian/Cocos
    {  547, 5029,   58408,  6, }, // Indian/Comoro
    {  547, 6661,  174342, 10, }, // Indian/Kerguelen
    {  547, 7260,   27115,  4, }, // Indian/Mahe
    {  547, 7050,  174352,  7, }, // Indian/Maldives
    {  547, 4069,  174359,  9, }, // Indian/Mauritius
    {  547, 7211,     547,  7, }, // Indian/Reunion
    {  547, 3157,     554,  5, }, // Pacific/Chuuk
    {  547, 2237,  174368, 17, }, // Pacific/Easter
    {  547, 3902,   33836,  5, }, // Pacific/Fiji
    {  547, 6678,   58503,  8, }, // Pacific/Galapagos
    {  547, 2723,    4611,  8, }, // Pacific/Honolulu
    {  547, 3049,    4619,  9, }, // Pacific/Kanton
    {  547, 4051,   58529,  7, }, // Pacific/Marquesas
    {  547, 7098,    4628,  6, }, // Pacific/Noumea
    {  547, 3096,     586,  7, }, // Pacific/Pohnpei
    {  548, 6788,   57696,  7, }, // Africa/Abidjan Slovenian/Latin/Slovenia
    {  548, 6760,    4649,  4, }, // Africa/Accra
    {  548, 5059,   35821, 10, }, // Africa/Addis_Ababa
    {  548, 7420,   57579,  5, }, // Africa/Algiers
    {  548,   14,       0,  6, }, // Africa/Asmara
    {  548, 2320,    4690,  5, }, // Africa/Cairo
    {  548, 5043,   33286,  7, }, // Africa/Djibouti
    {  548, 4524,    6423,  6, }, // Africa/Khartoum
    {  548, 5306,  122158,  7, }, // Africa/Kinshasa
    {  548, 4939,  122180,  9, }, // Africa/Lubumbashi
    {  548, 5113,   57721,  8, }, // Africa/Mogadishu
    {  548, 4435,       6,  8, }, // Africa/Sao_Tome
    {  548, 5607,   33300,  7, }, // America/Anguilla
    {  548, 4252,      14,  8, }, // America/Asuncion
    {  548,  237,      22,  8, }, // America/Atikokan
    {  548, 7623,   64671, 17, }, // America/Bahia_Banderas
    {  548, 3868,   33314,  6, }, // America/Cancun
    {  548, 5496,    6694,  6, }, // America/Cayman
    {  548, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  548, 5422,   33320,  9, }, // America/Costa_Rica
    {  548, 5723,      66,  7, }, // America/Curacao
    {  548, 5739,   21666,  8, }, // America/Dominica
    {  548, 5441,    6806,  8, }, // America/El_Salvador
    {  548, 3760,   33338,  9, }, // America/Guatemala
    {  548, 6932,   57584,  7, }, // America/Guyana
    {  548,  481,      73, 13, }, // America/Indiana/Knox
    {  548, 7961,      86, 16, }, // America/Indiana/Marengo
    {  548, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  548, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  548, 8038,     139, 14, }, // America/Indiana/Vevay
    {  548, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  548, 8086,     171, 16, }, // America/Indiana/Winamac
    {  548, 2799,   33347,  7, }, // America/Jamaica
    {  548, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  548, 5932,     207, 22, }, // America/Lower_Princes
    {  548, 5805,    5260,  8, }, // America/Martinique
    {  548, 8220,    7126,  6, }, // America/Merida
    {  548, 2949,  161910, 16, }, // America/Mexico_City
    {  548, 1850,     235, 19, }, // America/Noronha
    {  548, 8301,  174385, 22, }, // America/North_Dakota/Beulah
    {  548, 8329,  174407, 22, }, // America/North_Dakota/Center
    {  548, 8357,  174429, 25, }, // America/North_Dakota/New_Salem
    {  548,  393,     317,  4, }, // America/Nuuk
    {  548, 5843,   33426,  9, }, // America/Puerto_Rico
    {  548, 6899,     321, 16, }, // America/Scoresbysund
    {  548, 8508,  166705, 14, }, // America/St_Barthelemy
    {  548, 2061,     351, 10, }, // America/St_Johns
    {  548, 5863,  174454, 11, }, // America/St_Kitts
    {  548, 5880,  166103, 10, }, // America/St_Lucia
    {  548,  855,  174465,  9, }, // America/St_Thomas
    {  548, 5913,  174474, 13, }, // America/St_Vincent
    {  548, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  548, 5395,    7594,  6, }, // Asia/Almaty
    {  548, 3968,    5605,  4, }, // Asia/Amman
    {  548, 5511,    7600,  6, }, // Asia/Anadyr
    {  548, 5523,    5609,  5, }, // Asia/Aqtau
    {  548, 5368,    7606,  6, }, // Asia/Aqtobe
    {  548,  964,   33442,  7, }, // Asia/Ashgabat
    {  548, 3607,    4162,  6, }, // Asia/Baghdad
    {  548, 5534,   57943,  7, }, // Asia/Bahrain
    {  548, 4086,    7625,  6, }, // Asia/Beirut
    {  548, 3778,   33455,  6, }, // Asia/Bishkek
    {  548, 5989,   33461,  6, }, // Asia/Brunei
    {  548, 4601,   57622,  4, }, // Asia/Chita
    {  548, 4511,    5637,  7, }, // Asia/Colombo
    {  548, 4540,    4201,  6, }, // Asia/Damascus
    {  548, 1093,    4717,  4, }, // Asia/Dhaka
    {  548, 3596,   57968,  5, }, // Asia/Dubai
    {  548, 6159,   33468,  7, }, // Asia/Dushanbe
    {  548, 1259,  174487,  8, }, // Asia/Ho_Chi_Minh
    {  548, 2704,    4231,  8, }, // Asia/Hong_Kong
    {  548, 7006,   33484,  8, }, // Asia/Jakarta
    {  548, 1290,   33501,  9, }, // Asia/Jerusalem
    {  548, 4316,   33510,  8, }, // Asia/Kamchatka
    {  548, 4239,   33518,  6, }, // Asia/Karachi
    {  548, 1185,    5711,  8, }, // Asia/Kathmandu
    {  548, 8661,   33524,  7, }, // Asia/Khandyga
    {  548,  992,   26864,  7, }, // Asia/Kolkata
    {  548, 4197,    4288, 11, }, // Asia/Krasnoyarsk
    {  548, 5547,    7725,  6, }, // Asia/Kuwait
    {  548, 1211,     464,  5, }, // Asia/Macau
    {  548, 1349,    7736,  7, }, // Asia/Makassar
    {  548, 6920,  174495,  6, }, // Asia/Muscat
    {  548, 2554,   33542,  8, }, // Asia/Nicosia
    {  548, 7145,   58053,  6, }, // Asia/Oral
    {  548, 4214,   33562,  9, }, // Asia/Pyongyang
    {  548, 5559,    4332,  5, }, // Asia/Qatar
    {  548, 8708,    7770,  8, }, // Asia/Qostanay
    {  548, 4269,  174501,  8, }, // Asia/Qyzylorda
    {  548, 3584,   33571,  5, }, // Asia/Riyadh
    {  548, 4408,   33576,  7, }, // Asia/Sakhalin
    {  548, 3279,    7798,  4, }, // Asia/Seoul
    {  548, 1053,   58096,  7, }, // Asia/Shanghai
    {  548, 3300,   20900,  8, }, // Asia/Singapore
    {  548, 4297,    7817, 13, }, // Asia/Srednekolymsk
    {  548, 3263,    7830,  6, }, // Asia/Taipei
    {  548, 4781,   33589,  7, }, // Asia/Tashkent
    {  548, 2772,    7842,  7, }, // Asia/Tehran
    {  548, 1317,   91770,  5, }, // Asia/Thimphu
    {  548, 2821,    4356,  5, }, // Asia/Tokyo
    {  548, 1021,   26923, 10, }, // Asia/Ulaanbaatar
    {  548, 1159,   33596,  6, }, // Asia/Urumqi
    {  548, 4828,    7873,  7, }, // Asia/Yakutsk
    {  548, 1235,    7880,  6, }, // Asia/Yangon
    {  548, 3883,    4380, 13, }, // Asia/Yekaterinburg
    {  548, 3747,   26947,  6, }, // Asia/Yerevan
    {  548, 3663,   33612,  5, }, // Atlantic/Azores
    {  548, 5671,   57626,  7, }, // Atlantic/Bermuda
    {  548, 6628,  174509, 14, }, // Atlantic/Canary
    {  548, 3727,  174523, 17, }, // Atlantic/Cape_Verde
    {  548, 1395,  174540, 12, }, // Atlantic/Faroe
    {  548, 7272,  174552, 13, }, // Atlantic/South_Georgia
    {  548, 6821,  174565, 10, }, // Atlantic/St_Helena
    {  548, 6216,   33638,  6, }, // Europe/Andorra
    {  548, 3620,   33644,  8, }, // Europe/Astrakhan
    {  548, 2300,    5911,  5, }, // Europe/Athens
    {  548, 6336,    8060,  7, }, // Europe/Belgrade
    {  548, 2167,  174575,  7, }, // Europe/Brussels
    {  548, 3928,  127165,  9, }, // Europe/Bucharest
    {  548, 3806,   33671, 10, }, // Europe/Budapest
    {  548, 2583,   33681,  8, }, // Europe/Chisinau
    {  548, 6289,  174582,  9, }, // Europe/Copenhagen
    {  548, 8804,   57670,  8, }, // Europe/Isle_of_Man
    {  548, 2527,   33723,  5, }, // Europe/Kyiv
    {  548, 3213,  157721,  7, }, // Europe/Lisbon
    {  548, 6365,    8172, 10, }, // Europe/Luxembourg
    {  548, 6410,    6044,  6, }, // Europe/Monaco
    {  548, 3537,   22880,  6, }, // Europe/Moscow
    {  548, 4284,   33735,  5, }, // Europe/Paris
    {  548, 6275,   21185,  5, }, // Europe/Prague
    {  548, 6324,   33740,  3, }, // Europe/Rome
    {  548, 6599,   33761,  6, }, // Europe/Sofia
    {  548, 8867,    8250,  5, }, // Europe/Tallinn
    {  548, 6202,   21213,  6, }, // Europe/Tirane
    {  548, 8882,    8261,  9, }, // Europe/Ulyanovsk
    {  548, 6567,    6096,  7, }, // Europe/Vatican
    {  548, 6231,  174591,  5, }, // Europe/Vienna
    {  548, 8899,   64490,  5, }, // Europe/Vilnius
    {  548, 3190,   33778,  7, }, // Europe/Warsaw
    {  548, 6540,    4577,  6, }, // Europe/Zurich
    {  548, 6046,  174596, 12, }, // Indian/Christmas
    {  548, 6063,  174608, 14, }, // Indian/Cocos
    {  548, 5029,   64524,  6, }, // Indian/Comoro
    {  548, 7050,   33819,  7, }, // Indian/Maldives
    {  548, 7211,     547,  7, }, // Indian/Reunion
    {  548, 3157,     554,  5, }, // Pacific/Chuuk
    {  548, 2237,  174622, 16, }, // Pacific/Easter
    {  548, 3902,   33836,  5, }, // Pacific/Fiji
    {  548, 2723,    4611,  8, }, // Pacific/Honolulu
    {  548, 3049,    4619,  9, }, // Pacific/Kanton
    {  548, 3096,     586,  7, }, // Pacific/Pohnpei
    {  549,   14,       0,  6, }, // Africa/Asmara Soga/Latin/Uganda
    {  549, 4435,       6,  8, }, // Africa/Sao_Tome
    {  549, 4252,      14,  8, }, // America/Asuncion
    {  549,  237,      22,  8, }, // America/Atikokan
    {  549, 7623,      30, 17, }, // America/Bahia_Banderas
    {  549, 3868,      47,  6, }, // America/Cancun
    {  549, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  549, 5723,      66,  7, }, // America/Curacao
    {  549,  481,      73, 13, }, // America/Indiana/Knox
    {  549, 7961,      86, 16, }, // America/Indiana/Marengo
    {  549, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  549, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  549, 8038,     139, 14, }, // America/Indiana/Vevay
    {  549, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  549, 8086,     171, 16, }, // America/Indiana/Winamac
    {  549, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  549, 5932,     207, 22, }, // America/Lower_Princes
    {  549, 8220,     229,  6, }, // America/Merida
    {  549, 1850,     235, 19, }, // America/Noronha
    {  549, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  549, 8329,     274, 20, }, // America/North_Dakota/Center
    {  549, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  549,  393,     317,  4, }, // America/Nuuk
    {  549, 6899,     321, 16, }, // America/Scoresbysund
    {  549, 8508,     337, 14, }, // America/St_Barthelemy
    {  549, 2061,     351, 10, }, // America/St_Johns
    {  549, 5863,     361,  9, }, // America/St_Kitts
    {  549, 5880,     370,  9, }, // America/St_Lucia
    {  549,  855,     379, 10, }, // America/St_Thomas
    {  549, 5913,     389, 11, }, // America/St_Vincent
    {  549, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  549, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  549, 1185,     448,  9, }, // Asia/Kathmandu
    {  549,  992,     457,  7, }, // Asia/Kolkata
    {  549, 1211,     464,  5, }, // Asia/Macau
    {  549, 1235,     477,  6, }, // Asia/Yangon
    {  549, 1395,     483,  5, }, // Atlantic/Faroe
    {  549, 6821,     488, 10, }, // Atlantic/St_Helena
    {  549, 2527,     514,  4, }, // Europe/Kyiv
    {  549, 7211,     547,  7, }, // Indian/Reunion
    {  549, 3157,     554,  5, }, // Pacific/Chuuk
    {  549, 3096,     586,  7, }, // Pacific/Pohnpei
    {  550, 6788,  174638,  8, }, // Africa/Abidjan Somali/Latin/Somalia
    {  550, 6760,    4649,  4, }, // Africa/Accra
    {  550, 7420,  174646,  8, }, // Africa/Algiers
    {  550,   14,       0,  6, }, // Africa/Asmara
    {  550,   44,  174654,  7, }, // Africa/Bamako
    {  550, 5257,  174661,  5, }, // Africa/Bangui
    {  550, 7435,  174666,  6, }, // Africa/Bissau
    {  550, 4957,  174672,  9, }, // Africa/Blantyre
    {  550, 5287,  174681,  9, }, // Africa/Brazzaville
    {  550, 2320,  174690,  7, }, // Africa/Cairo
    {  550, 4117,  174697, 12, }, // Africa/Casablanca
    {  550, 7449,   21349,  5, }, // Africa/Ceuta
    {  550, 6773,  174709,  7, }, // Africa/Conakry
    {  550, 5130,  174716, 10, }, // Africa/Dar_es_Salaam
    {  550, 5043,  174726,  7, }, // Africa/Djibouti
    {  550, 5243,  174733,  7, }, // Africa/Douala
    {  550, 5166,  174740,  9, }, // Africa/El_Aaiun
    {  550, 6853,  174749,  9, }, // Africa/Freetown
    {  550, 4906,  174758,  7, }, // Africa/Gaborone
    {  550, 5015,  174765,  7, }, // Africa/Harare
    {  550, 4479,  174772, 10, }, // Africa/Johannesburg
    {  550, 5151,  174782,  8, }, // Africa/Kampala
    {  550, 4524,  174790,  8, }, // Africa/Khartoum
    {  550, 4744,  174798,  7, }, // Africa/Lagos
    {  550, 5336,  174805,  8, }, // Africa/Libreville
    {  550, 6869,  174813,  4, }, // Africa/Lome
    {  550, 5211,  174817,  8, }, // Africa/Luanda
    {  550, 4939,  174825, 11, }, // Africa/Lubumbashi
    {  550, 5001,  174836,  7, }, // Africa/Lusaka
    {  550, 4987,  174843,  7, }, // Africa/Maputo
    {  550, 5197,  174850,  7, }, // Africa/Maseru
    {  550, 5182,  174857,  9, }, // Africa/Mbabane
    {  550, 5113,   26511,  8, }, // Africa/Mogadishu
    {  550, 7034,  174866,  9, }, // Africa/Monrovia
    {  550, 3853,  174875,  8, }, // Africa/Nairobi
    {  550, 5271,  174883,  8, }, // Africa/Ndjamena
    {  550, 5354,  165412,  6, }, // Africa/Niamey
    {  550, 6803,  174891,  7, }, // Africa/Nouakchott
    {  550, 6727,    4758,  8, }, // Africa/Ouagadougou
    {  550, 5225,  174898, 12, }, // Africa/Porto-Novo
    {  550, 4435,  174910,  8, }, // Africa/Sao_Tome
    {  550, 2866,  174918,  9, }, // Africa/Tripoli
    {  550, 6554,  174927,  6, }, // Africa/Tunis
    {  550, 4152,  174933,  8, }, // Africa/Windhoek
    {  550, 3346,  174941,  8, }, // America/Anchorage
    {  550, 5607,  174949,  8, }, // America/Anguilla
    {  550, 5624,  174957,  8, }, // America/Antigua
    {  550, 4554,  174965, 10, }, // America/Araguaina
    {  550,  166,  174975, 13, }, // America/Argentina/Buenos_Aires
    {  550,   91,  174988, 10, }, // America/Argentina/Catamarca
    {  550,  270,  174998,  8, }, // America/Argentina/Cordoba
    {  550, 7462,  175006,  9, }, // America/Argentina/La_Rioja
    {  550,  565,  175015,  9, }, // America/Argentina/Mendoza
    {  550, 7489,  175024, 12, }, // America/Argentina/Rio_Gallegos
    {  550, 7544,  175036, 10, }, // America/Argentina/San_Juan
    {  550, 5580,  175046,  9, }, // America/Argentina/San_Luis
    {  550, 7571,  175055,  9, }, // America/Argentina/Tucuman
    {  550, 7597,  175064,  7, }, // America/Argentina/Ushuaia
    {  550, 4252,  175071,  9, }, // America/Asuncion
    {  550,  237,  175080,  9, }, // America/Atikokan
    {  550, 3679,  175089,  7, }, // America/Bahia
    {  550, 7623,  175096, 15, }, // America/Bahia_Banderas
    {  550, 5407,  175111,  7, }, // America/Belize
    {  550, 7660,  175118, 13, }, // America/Blanc-Sablon
    {  550, 7681,  175131,  9, }, // America/Boa_Vista
    {  550, 7699,   21564,  5, }, // America/Boise
    {  550, 7713,  175140, 14, }, // America/Cambridge_Bay
    {  550, 7735,  175154, 14, }, // America/Campo_Grande
    {  550, 3868,  175168,  8, }, // America/Cancun
    {  550, 4694,  175176,  8, }, // America/Caracas
    {  550, 4345,  175184,  6, }, // America/Cayenne
    {  550, 5496,  175190,  7, }, // America/Cayman
    {  550, 2260,  175197,  7, }, // America/Chicago
    {  550, 7756,  175204,  8, }, // America/Chihuahua
    {  550, 7774,  175212, 16, }, // America/Ciudad_Juarez
    {  550, 5422,  175228, 11, }, // America/Costa_Rica
    {  550, 7796,  175239,  9, }, // America/Creston
    {  550, 3791,  175248,  7, }, // America/Cuiaba
    {  550, 5723,  175255,  8, }, // America/Curacao
    {  550, 7812,  175263, 13, }, // America/Danmarkshavn
    {  550, 7833,  175276,  6, }, // America/Dawson
    {  550, 7848,  175282, 13, }, // America/Dawson_Creek
    {  550,  805,  175295,  6, }, // America/Denver
    {  550, 3465,  175301,  8, }, // America/Detroit
    {  550, 5739,  175309,  9, }, // America/Dominica
    {  550, 7869,  175318,  8, }, // America/Eirunepe
    {  550, 5441,  175326, 12, }, // America/El_Salvador
    {  550, 7886,  175338, 11, }, // America/Fort_Nelson
    {  550, 7906,  175349,  9, }, // America/Fortaleza
    {  550, 7924,  175358, 11, }, // America/Glace_Bay
    {  550, 6881,  175369,  9, }, // America/Goose_Bay
    {  550, 4612,  175378, 11, }, // America/Grand_Turk
    {  550, 5770,  175389,  9, }, // America/Grenada
    {  550, 5786,  175398, 10, }, // America/Guadeloupe
    {  550, 3760,  175408, 11, }, // America/Guatemala
    {  550, 4841,  175419, 11, }, // America/Guayaquil
    {  550, 6932,  175430,  7, }, // America/Guyana
    {  550, 1939,  175437,  9, }, // America/Halifax
    {  550, 2281,  175446,  7, }, // America/Havana
    {  550, 7942,  175453,  9, }, // America/Hermosillo
    {  550,  348,  175462, 14, }, // America/Indiana/Indianapolis
    {  550,  481,  175476, 16, }, // America/Indiana/Knox
    {  550, 7961,  175492, 19, }, // America/Indiana/Marengo
    {  550, 7985,  175511, 20, }, // America/Indiana/Petersburg
    {  550, 8012,  175531, 19, }, // America/Indiana/Tell_City
    {  550, 8038,  175550, 18, }, // America/Indiana/Vevay
    {  550, 8060,  175568, 20, }, // America/Indiana/Vincennes
    {  550, 8086,  175588, 19, }, // America/Indiana/Winamac
    {  550, 8110,  175607,  7, }, // America/Inuvik
    {  550,  660,  175614,  8, }, // America/Iqaluit
    {  550, 2799,   98803,  7, }, // America/Jamaica
    {  550, 5380,  175622,  7, }, // America/Juneau
    {  550,  521,  175629,  8, }, // America/Kentucky/Louisville
    {  550, 8125,  175637, 19, }, // America/Kentucky/Monticello
    {  550, 5704,  175656, 11, }, // America/Kralendijk
    {  550, 4376,  175667,  8, }, // America/La_Paz
    {  550, 7169,  175675,  5, }, // America/Lima
    {  550, 3239,  175680, 12, }, // America/Los_Angeles
    {  550, 5932,  175692, 22, }, // America/Lower_Princes
    {  550, 8153,  175714,  8, }, // America/Maceio
    {  550, 8168,  175722,  9, }, // America/Managua
    {  550, 1908,  175731,  8, }, // America/Manaus
    {  550, 5897,  175739,  8, }, // America/Marigot
    {  550, 5805,  175747, 14, }, // America/Martinique
    {  550, 2917,  175761,  9, }, // America/Mazatlan
    {  550, 8202,  175770, 10, }, // America/Menominee
    {  550, 8220,  175780,  7, }, // America/Merida
    {  550, 8235,  175787, 11, }, // America/Metlakatla
    {  550, 2949,  165888, 12, }, // America/Mexico_City
    {  550, 4391,  175798,  9, }, // America/Miquelon
    {  550, 8254,  175807,  9, }, // America/Moncton
    {  550, 8270,  175816, 10, }, // America/Monterrey
    {  550, 4098,  175826, 13, }, // America/Montevideo
    {  550, 5824,  175839, 11, }, // America/Montserrat
    {  550, 5481,  175850,  6, }, // America/Nassau
    {  550, 2379,  175856, 10, }, // America/New_York
    {  550, 8288,  141561,  4, }, // America/Nome
    {  550, 1850,     235, 19, }, // America/Noronha
    {  550, 8301,  175866, 27, }, // America/North_Dakota/Beulah
    {  550, 8329,  175893, 28, }, // America/North_Dakota/Center
    {  550, 8357,  175921, 31, }, // America/North_Dakota/New_Salem
    {  550,  393,     317,  4, }, // America/Nuuk
    {  550, 8388,  175952,  8, }, // America/Ojinaga
    {  550, 2356,  175960,  7, }, // America/Panama
    {  550, 2973,  175967,  8, }, // America/Phoenix
    {  550, 3945,  175975, 16, }, // America/Port-au-Prince
    {  550, 5954,  175991, 15, }, // America/Port_of_Spain
    {  550, 5843,  176006, 12, }, // America/Puerto_Rico
    {  550, 4030,  176018, 13, }, // America/Punta_Arenas
    {  550, 8424,  176031, 14, }, // America/Rankin_Inlet
    {  550, 8445,  176045,  6, }, // America/Recife
    {  550, 1995,  176051,  7, }, // America/Regina
    {  550, 8460,  176058,  8, }, // America/Resolute
    {  550,  695,  176066, 13, }, // America/Rio_Branco
    {  550, 8477,  176079,  9, }, // America/Santarem
    {  550, 2201,  176088, 10, }, // America/Santiago
    {  550, 6111,  176098, 14, }, // America/Santo_Domingo
    {  550, 1878,  176112, 10, }, // America/Sao_Paulo
    {  550, 6899,  176122, 13, }, // America/Scoresbysund
    {  550, 8494,  176135,  6, }, // America/Sitka
    {  550, 8508,  176141, 14, }, // America/St_Barthelemy
    {  550, 2061,  176155,  8, }, // America/St_Johns
    {  550, 5863,  176163,  9, }, // America/St_Kitts
    {  550, 5880,  176172, 10, }, // America/St_Lucia
    {  550,  855,  176182, 10, }, // America/St_Thomas
    {  550, 5913,  176192, 11, }, // America/St_Vincent
    {  550, 8530,  176203, 13, }, // America/Swift_Current
    {  550, 5461,  176216, 11, }, // America/Tegucigalpa
    {  550, 5756,  176227,  4, }, // America/Thule
    {  550,  313,  176231,  9, }, // America/Tijuana
    {  550, 5688,  176240,  8, }, // America/Tortola
    {  550, 2093,  176248,  9, }, // America/Vancouver
    {  550, 2144,  176257,  9, }, // America/Whitehorse
    {  550,  734,  176266,  7, }, // America/Winnipeg
    {  550, 8552,  176273,  8, }, // America/Yakutat
    {  550, 6001,  176281,  6, }, // Antarctica/Casey
    {  550, 6094,  176287,  5, }, // Antarctica/Davis
    {  550, 6133,  176292, 15, }, // Antarctica/DumontDUrville
    {  550, 8568,  176307, 10, }, // Antarctica/Macquarie
    {  550, 7113,   74015,  8, }, // Antarctica/McMurdo
    {  550, 8589,  176317,  6, }, // Antarctica/Palmer
    {  550, 7226,    5583,  6, }, // Antarctica/Rothera
    {  550, 7295,  176323,  6, }, // Antarctica/Syowa
    {  550, 8607,  176329,  6, }, // Antarctica/Troll
    {  550, 7374,  176335,  6, }, // Antarctica/Vostok
    {  550, 1429,  176341, 11, }, // Arctic/Longyearbyen
    {  550, 5570,  176352,  5, }, // Asia/Aden
    {  550, 5395,    7594,  6, }, // Asia/Almaty
    {  550, 3968,  176357,  6, }, // Asia/Amman
    {  550, 5511,  176363,  7, }, // Asia/Anadyr
    {  550, 5523,  176370,  5, }, // Asia/Aqtau
    {  550, 8624,  176375,  7, }, // Asia/Atyrau
    {  550, 3607,  176382,  7, }, // Asia/Baghdad
    {  550, 5534,  176389,  7, }, // Asia/Bahrain
    {  550, 4466,  176396,  8, }, // Asia/Bangkok
    {  550, 3571,  176404,  9, }, // Asia/Barnaul
    {  550, 4086,  176413,  7, }, // Asia/Beirut
    {  550, 5989,  176420,  7, }, // Asia/Brunei
    {  550, 4601,  176427,  5, }, // Asia/Chita
    {  550, 4511,    5637,  7, }, // Asia/Colombo
    {  550, 4540,  176432,  8, }, // Asia/Damascus
    {  550, 1093,  176440,  6, }, // Asia/Dhaka
    {  550, 3596,   22390,  5, }, // Asia/Dubai
    {  550, 8651,  176446,  4, }, // Asia/Gaza
    {  550, 1259,  176450, 15, }, // Asia/Ho_Chi_Minh
    {  550, 2704,  176465, 11, }, // Asia/Hong_Kong
    {  550, 4771,  176476,  5, }, // Asia/Hovd
    {  550, 4184,  176481,  8, }, // Asia/Irkutsk
    {  550, 7006,  176489,  8, }, // Asia/Jakarta
    {  550, 6992,  176497,  8, }, // Asia/Jayapura
    {  550, 1290,  176505, 11, }, // Asia/Jerusalem
    {  550, 3560,  176516,  7, }, // Asia/Kabul
    {  550, 4316,  176523,  8, }, // Asia/Kamchatka
    {  550, 4239,  176531,  7, }, // Asia/Karachi
    {  550, 1185,    5711,  8, }, // Asia/Kathmandu
    {  550, 8661,  176538,  9, }, // Asia/Khandyga
    {  550,  992,  176547,  8, }, // Asia/Kolkata
    {  550, 4197,  176555, 13, }, // Asia/Krasnoyarsk
    {  550, 4859,  176568, 13, }, // Asia/Kuala_Lumpur
    {  550, 5976,  176581,  6, }, // Asia/Kuching
    {  550, 5547,  117688,  6, }, // Asia/Kuwait
    {  550, 1211,  176587,  5, }, // Asia/Macau
    {  550, 4017,  176592,  7, }, // Asia/Magadan
    {  550, 1349,    7736,  7, }, // Asia/Makassar
    {  550, 7182,  176599,  7, }, // Asia/Manila
    {  550, 6920,    4326,  6, }, // Asia/Muscat
    {  550, 2554,   22510,  8, }, // Asia/Nicosia
    {  550, 8675,  176606, 13, }, // Asia/Novokuznetsk
    {  550, 4135,  176619, 12, }, // Asia/Novosibirsk
    {  550, 6961,  176631,  9, }, // Asia/Phnom_Penh
    {  550, 8693,  176640, 10, }, // Asia/Pontianak
    {  550, 4214,  176650, 10, }, // Asia/Pyongyang
    {  550, 5559,  176660,  6, }, // Asia/Qatar
    {  550, 8708,     469,  8, }, // Asia/Qostanay
    {  550, 4269,  176666,  9, }, // Asia/Qyzylorda
    {  550, 3584,  176675,  6, }, // Asia/Riyadh
    {  550, 7245,  176681,  9, }, // Asia/Samarkand
    {  550, 3279,   58092,  4, }, // Asia/Seoul
    {  550, 1053,  176690,  9, }, // Asia/Shanghai
    {  550, 3300,  176699,  9, }, // Asia/Singapore
    {  550, 4297,  176708, 16, }, // Asia/Srednekolymsk
    {  550, 3263,  176724,  6, }, // Asia/Taipei
    {  550, 4781,  176730,  9, }, // Asia/Tashkent
    {  550, 3915,  176739,  8, }, // Asia/Tbilisi
    {  550, 2772,  176747,  7, }, // Asia/Tehran
    {  550, 1317,   62605,  5, }, // Asia/Thimphu
    {  550, 6977,  176754, 12, }, // Asia/Vientiane
    {  550, 4710,  176766, 12, }, // Asia/Vladivostok
    {  550, 4828,  176273,  5, }, // Asia/Yakutsk
    {  550, 1235,     477,  6, }, // Asia/Yangon
    {  550, 3883,  176778, 13, }, // Asia/Yekaterinburg
    {  550, 3747,  176791,  7, }, // Asia/Yerevan
    {  550, 3663,  176798,  6, }, // Atlantic/Azores
    {  550, 5671,  176804,  8, }, // Atlantic/Bermuda
    {  550, 6628,    4398,  6, }, // Atlantic/Canary
    {  550, 3727,  176812, 11, }, // Atlantic/Cape_Verde
    {  550, 1395,  176823,  5, }, // Atlantic/Faroe
    {  550, 8736,   99133,  6, }, // Atlantic/Madeira
    {  550, 2748,  176828,  9, }, // Atlantic/Reykjavik
    {  550, 7272,  176837, 13, }, // Atlantic/South_Georgia
    {  550, 6821,     488, 10, }, // Atlantic/St_Helena
    {  550, 6644,  176850,  9, }, // Atlantic/Stanley
    {  550, 1670,  176859,  8, }, // Australia/Adelaide
    {  550, 1635,  176867,  8, }, // Australia/Brisbane
    {  550, 1799,  176875, 11, }, // Australia/Broken_Hill
    {  550, 3637,   22722,  5, }, // Australia/Eucla
    {  550, 1516,  176886,  7, }, // Australia/Hobart
    {  550, 1547,  176893,  7, }, // Australia/Lord_Howe
    {  550, 1727,  176900,  7, }, // Australia/Melbourne
    {  550, 1762,  176907,  4, }, // Australia/Perth
    {  550, 1463,   22739,  6, }, // Australia/Sydney
    {  550, 6441,  176911, 10, }, // Europe/Amsterdam
    {  550, 6216,  176921,  7, }, // Europe/Andorra
    {  550, 3620,  176928, 11, }, // Europe/Astrakhan
    {  550, 2300,  176939,  5, }, // Europe/Athens
    {  550, 6336,  176944, 10, }, // Europe/Belgrade
    {  550, 4757,  176954,  7, }, // Europe/Berlin
    {  550, 6488,  176961, 11, }, // Europe/Bratislava
    {  550, 2167,  176972,  9, }, // Europe/Brussels
    {  550, 3928,  176981,  8, }, // Europe/Bucharest
    {  550, 3806,  176989,  8, }, // Europe/Budapest
    {  550, 8772,  176997,  9, }, // Europe/Busingen
    {  550, 2583,  177006,  8, }, // Europe/Chisinau
    {  550, 6289,  177014, 11, }, // Europe/Copenhagen
    {  550, 2338,  177025,  8, }, // Europe/Dublin
    {  550, 8788,  177033,  8, }, // Europe/Guernsey
    {  550, 6612,  177041,  9, }, // Europe/Helsinki
    {  550, 8804,  177050, 12, }, // Europe/Isle_of_Man
    {  550, 1130,  177062,  9, }, // Europe/Istanbul
    {  550, 8823,  177071,  7, }, // Europe/Jersey
    {  550, 8929,  177078,  6, }, // Europe/Kirov
    {  550, 2527,  177084,  6, }, // Europe/Kyiv
    {  550, 6506,  177090,  9, }, // Europe/Ljubljana
    {  550, 2501,  177099,  6, }, // Europe/London
    {  550, 6365,  177105, 10, }, // Europe/Luxembourg
    {  550, 4892,  177115,  7, }, // Europe/Madrid
    {  550, 6582,  177122,  9, }, // Europe/Mariehamn
    {  550, 3693,  177131,  6, }, // Europe/Minsk
    {  550, 6410,    6044,  6, }, // Europe/Monaco
    {  550, 3537,  177137,  6, }, // Europe/Moscow
    {  550, 4284,  177143,  7, }, // Europe/Paris
    {  550, 6424,  177150,  9, }, // Europe/Podgorica
    {  550, 6275,  177159,  6, }, // Europe/Prague
    {  550, 8837,   57491,  5, }, // Europe/Riga
    {  550, 6324,   63849,  5, }, // Europe/Rome
    {  550, 6470,  177165, 11, }, // Europe/San_Marino
    {  550, 6245,  177176,  9, }, // Europe/Sarajevo
    {  550, 4451,  177185,  8, }, // Europe/Saratov
    {  550, 8849,  177193, 10, }, // Europe/Simferopol
    {  550, 6383,  177203,  8, }, // Europe/Skopje
    {  550, 6599,   22919,  6, }, // Europe/Sofia
    {  550, 6523,  177211,  9, }, // Europe/Stockholm
    {  550, 8867,  177220,  6, }, // Europe/Tallinn
    {  550, 6202,  177226,  7, }, // Europe/Tirane
    {  550, 8882,  177233, 10, }, // Europe/Ulyanovsk
    {  550, 6352,  177243,  6, }, // Europe/Vaduz
    {  550, 6567,  177249,  8, }, // Europe/Vatican
    {  550, 6231,  177257,  7, }, // Europe/Vienna
    {  550, 8899,  177264,  8, }, // Europe/Vilnius
    {  550, 4727,  177272,  9, }, // Europe/Volgograd
    {  550, 6261,  177281,  6, }, // Europe/Zagreb
    {  550, 6540,  177287,  7, }, // Europe/Zurich
    {  550, 5078,  177294, 12, }, // Indian/Antananarivo
    {  550, 6947,  177306,  5, }, // Indian/Chagos
    {  550, 6046,  177311,  9, }, // Indian/Christmas
    {  550, 6063,    4583,  5, }, // Indian/Cocos
    {  550, 5029,   22985,  6, }, // Indian/Comoro
    {  550, 6661,  177320,  8, }, // Indian/Kerguelen
    {  550, 7050,  177328,  8, }, // Indian/Maldives
    {  550, 4069,  177336, 11, }, // Indian/Mauritius
    {  550, 5098,  177347,  7, }, // Indian/Mayotte
    {  550, 7211,  177354, 10, }, // Indian/Reunion
    {  550, 4422,  147686,  5, }, // Pacific/Apia
    {  550,  932,  177364,  7, }, // Pacific/Auckland
    {  550, 3706,  177371, 10, }, // Pacific/Bougainville
    {  550, 3015,  177381,  5, }, // Pacific/Chatham
    {  550, 3157,  177386,  4, }, // Pacific/Chuuk
    {  550, 2237,  177390,  6, }, // Pacific/Easter
    {  550, 7327,  177396,  6, }, // Pacific/Fakaofo
    {  550, 6678,  177402,  9, }, // Pacific/Galapagos
    {  550, 6696,  177411,  7, }, // Pacific/Gambier
    {  550, 3822,  177418, 11, }, // Pacific/Guadalcanal
    {  550, 6018,  177429,  5, }, // Pacific/Guam
    {  550, 3049,  127691,  9, }, // Pacific/Kanton
    {  550, 3998,  177434, 11, }, // Pacific/Kiritimati
    {  550, 7019,  177445,  6, }, // Pacific/Kosrae
    {  550, 2842,  177451, 10, }, // Pacific/Kwajalein
    {  550, 4877,  177461,  5, }, // Pacific/Majuro
    {  550, 4051,  177466, 10, }, // Pacific/Marquesas
    {  550, 7084,  177476,  6, }, // Pacific/Nauru
    {  550, 7132,  175856,  5, }, // Pacific/Niue
    {  550, 4168,  177482,  7, }, // Pacific/Norfolk
    {  550, 7098,  177489,  7, }, // Pacific/Noumea
    {  550, 3126,  177496,  9, }, // Pacific/Pago_Pago
    {  550, 7155,  177505,  5, }, // Pacific/Palau
    {  550, 7194,  177510,  8, }, // Pacific/Pitcairn
    {  550, 3096,  177518,  6, }, // Pacific/Pohnpei
    {  550, 4807,  177524, 13, }, // Pacific/Port_Moresby
    {  550, 6031,  177537,  6, }, // Pacific/Saipan
    {  550, 4583,  177543,  9, }, // Pacific/Tongatapu
    {  550, 7405,  177552,  5, }, // Pacific/Wallis
    {  554,   14,       0,  6, }, // Africa/Asmara Southern Kurdish/Arabic/Iran
    {  554, 4435,       6,  8, }, // Africa/Sao_Tome
    {  554, 4252,      14,  8, }, // America/Asuncion
    {  554,  237,      22,  8, }, // America/Atikokan
    {  554, 7623,      30, 17, }, // America/Bahia_Banderas
    {  554, 3868,      47,  6, }, // America/Cancun
    {  554, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  554, 5723,      66,  7, }, // America/Curacao
    {  554,  481,      73, 13, }, // America/Indiana/Knox
    {  554, 7961,      86, 16, }, // America/Indiana/Marengo
    {  554, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  554, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  554, 8038,     139, 14, }, // America/Indiana/Vevay
    {  554, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  554, 8086,     171, 16, }, // America/Indiana/Winamac
    {  554, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  554, 5932,     207, 22, }, // America/Lower_Princes
    {  554, 8220,     229,  6, }, // America/Merida
    {  554, 1850,     235, 19, }, // America/Noronha
    {  554, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  554, 8329,     274, 20, }, // America/North_Dakota/Center
    {  554, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  554,  393,     317,  4, }, // America/Nuuk
    {  554, 6899,     321, 16, }, // America/Scoresbysund
    {  554, 8508,     337, 14, }, // America/St_Barthelemy
    {  554, 2061,     351, 10, }, // America/St_Johns
    {  554, 5863,     361,  9, }, // America/St_Kitts
    {  554, 5880,     370,  9, }, // America/St_Lucia
    {  554,  855,     379, 10, }, // America/St_Thomas
    {  554, 5913,     389, 11, }, // America/St_Vincent
    {  554, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  554, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  554, 1185,     448,  9, }, // Asia/Kathmandu
    {  554,  992,     457,  7, }, // Asia/Kolkata
    {  554, 1211,     464,  5, }, // Asia/Macau
    {  554, 1235,     477,  6, }, // Asia/Yangon
    {  554, 1395,     483,  5, }, // Atlantic/Faroe
    {  554, 6821,     488, 10, }, // Atlantic/St_Helena
    {  554, 2527,     514,  4, }, // Europe/Kyiv
    {  554, 7211,     547,  7, }, // Indian/Reunion
    {  554, 3157,     554,  5, }, // Pacific/Chuuk
    {  554, 3096,     586,  7, }, // Pacific/Pohnpei
    {  556,   14,       0,  6, }, // Africa/Asmara Southern Sami/Latin/Sweden
    {  556, 4435,       6,  8, }, // Africa/Sao_Tome
    {  556, 4252,      14,  8, }, // America/Asuncion
    {  556,  237,      22,  8, }, // America/Atikokan
    {  556, 7623,      30, 17, }, // America/Bahia_Banderas
    {  556, 3868,      47,  6, }, // America/Cancun
    {  556, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  556, 5723,      66,  7, }, // America/Curacao
    {  556,  481,      73, 13, }, // America/Indiana/Knox
    {  556, 7961,      86, 16, }, // America/Indiana/Marengo
    {  556, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  556, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  556, 8038,     139, 14, }, // America/Indiana/Vevay
    {  556, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  556, 8086,     171, 16, }, // America/Indiana/Winamac
    {  556, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  556, 5932,     207, 22, }, // America/Lower_Princes
    {  556, 8220,     229,  6, }, // America/Merida
    {  556, 1850,     235, 19, }, // America/Noronha
    {  556, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  556, 8329,     274, 20, }, // America/North_Dakota/Center
    {  556, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  556,  393,     317,  4, }, // America/Nuuk
    {  556, 6899,     321, 16, }, // America/Scoresbysund
    {  556, 8508,     337, 14, }, // America/St_Barthelemy
    {  556, 2061,     351, 10, }, // America/St_Johns
    {  556, 5863,     361,  9, }, // America/St_Kitts
    {  556, 5880,     370,  9, }, // America/St_Lucia
    {  556,  855,     379, 10, }, // America/St_Thomas
    {  556, 5913,     389, 11, }, // America/St_Vincent
    {  556, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  556, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  556, 1185,     448,  9, }, // Asia/Kathmandu
    {  556,  992,     457,  7, }, // Asia/Kolkata
    {  556, 1211,     464,  5, }, // Asia/Macau
    {  556, 1235,     477,  6, }, // Asia/Yangon
    {  556, 1395,     483,  5, }, // Atlantic/Faroe
    {  556, 6821,     488, 10, }, // Atlantic/St_Helena
    {  556, 2527,     514,  4, }, // Europe/Kyiv
    {  556, 7211,     547,  7, }, // Indian/Reunion
    {  556, 3157,     554,  5, }, // Pacific/Chuuk
    {  556, 3096,     586,  7, }, // Pacific/Pohnpei
    {  558,   14,       0,  6, }, // Africa/Asmara Southern Sotho/Latin/South Africa
    {  558, 4435,       6,  8, }, // Africa/Sao_Tome
    {  558, 4252,      14,  8, }, // America/Asuncion
    {  558,  237,      22,  8, }, // America/Atikokan
    {  558, 7623,      30, 17, }, // America/Bahia_Banderas
    {  558, 3868,      47,  6, }, // America/Cancun
    {  558, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  558, 5723,      66,  7, }, // America/Curacao
    {  558,  481,      73, 13, }, // America/Indiana/Knox
    {  558, 7961,      86, 16, }, // America/Indiana/Marengo
    {  558, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  558, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  558, 8038,     139, 14, }, // America/Indiana/Vevay
    {  558, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  558, 8086,     171, 16, }, // America/Indiana/Winamac
    {  558, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  558, 5932,     207, 22, }, // America/Lower_Princes
    {  558, 8220,     229,  6, }, // America/Merida
    {  558, 1850,     235, 19, }, // America/Noronha
    {  558, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  558, 8329,     274, 20, }, // America/North_Dakota/Center
    {  558, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  558,  393,     317,  4, }, // America/Nuuk
    {  558, 6899,     321, 16, }, // America/Scoresbysund
    {  558, 8508,     337, 14, }, // America/St_Barthelemy
    {  558, 2061,     351, 10, }, // America/St_Johns
    {  558, 5863,     361,  9, }, // America/St_Kitts
    {  558, 5880,     370,  9, }, // America/St_Lucia
    {  558,  855,     379, 10, }, // America/St_Thomas
    {  558, 5913,     389, 11, }, // America/St_Vincent
    {  558, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  558, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  558, 1185,     448,  9, }, // Asia/Kathmandu
    {  558,  992,     457,  7, }, // Asia/Kolkata
    {  558, 1211,     464,  5, }, // Asia/Macau
    {  558, 1235,     477,  6, }, // Asia/Yangon
    {  558, 1395,     483,  5, }, // Atlantic/Faroe
    {  558, 6821,     488, 10, }, // Atlantic/St_Helena
    {  558, 2527,     514,  4, }, // Europe/Kyiv
    {  558, 7211,     547,  7, }, // Indian/Reunion
    {  558, 3157,     554,  5, }, // Pacific/Chuuk
    {  558, 3096,     586,  7, }, // Pacific/Pohnpei
    {  560,   14,       0,  6, }, // Africa/Asmara South Ndebele/Latin/South Africa
    {  560, 4435,       6,  8, }, // Africa/Sao_Tome
    {  560, 4252,      14,  8, }, // America/Asuncion
    {  560,  237,      22,  8, }, // America/Atikokan
    {  560, 7623,      30, 17, }, // America/Bahia_Banderas
    {  560, 3868,      47,  6, }, // America/Cancun
    {  560, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  560, 5723,      66,  7, }, // America/Curacao
    {  560,  481,      73, 13, }, // America/Indiana/Knox
    {  560, 7961,      86, 16, }, // America/Indiana/Marengo
    {  560, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  560, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  560, 8038,     139, 14, }, // America/Indiana/Vevay
    {  560, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  560, 8086,     171, 16, }, // America/Indiana/Winamac
    {  560, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  560, 5932,     207, 22, }, // America/Lower_Princes
    {  560, 8220,     229,  6, }, // America/Merida
    {  560, 1850,     235, 19, }, // America/Noronha
    {  560, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  560, 8329,     274, 20, }, // America/North_Dakota/Center
    {  560, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  560,  393,     317,  4, }, // America/Nuuk
    {  560, 6899,     321, 16, }, // America/Scoresbysund
    {  560, 8508,     337, 14, }, // America/St_Barthelemy
    {  560, 2061,     351, 10, }, // America/St_Johns
    {  560, 5863,     361,  9, }, // America/St_Kitts
    {  560, 5880,     370,  9, }, // America/St_Lucia
    {  560,  855,     379, 10, }, // America/St_Thomas
    {  560, 5913,     389, 11, }, // America/St_Vincent
    {  560, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  560, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  560, 1185,     448,  9, }, // Asia/Kathmandu
    {  560,  992,     457,  7, }, // Asia/Kolkata
    {  560, 1211,     464,  5, }, // Asia/Macau
    {  560, 1235,     477,  6, }, // Asia/Yangon
    {  560, 1395,     483,  5, }, // Atlantic/Faroe
    {  560, 6821,     488, 10, }, // Atlantic/St_Helena
    {  560, 2527,     514,  4, }, // Europe/Kyiv
    {  560, 7211,     547,  7, }, // Indian/Reunion
    {  560, 3157,     554,  5, }, // Pacific/Chuuk
    {  560, 3096,     586,  7, }, // Pacific/Pohnpei
    {  561, 6788,  161871,  6, }, // Africa/Abidjan Spanish/Latin/Spain
    {  561, 6760,   74129,  4, }, // Africa/Accra
    {  561, 5059,   20370, 10, }, // Africa/Addis_Ababa
    {  561, 7420,  147692,  5, }, // Africa/Algiers
    {  561,   14,       0,  6, }, // Africa/Asmara
    {  561, 7435,  177557,  5, }, // Africa/Bissau
    {  561, 2320,  161877,  8, }, // Africa/Cairo
    {  561, 6773,  174709,  7, }, // Africa/Conakry
    {  561, 5130,  177562, 12, }, // Africa/Dar_es_Salaam
    {  561, 5043,  147706,  6, }, // Africa/Djibouti
    {  561, 5243,   21372,  5, }, // Africa/Douala
    {  561, 5166,   63491,  8, }, // Africa/El_Aaiun
    {  561, 4479,  161885, 13, }, // Africa/Johannesburg
    {  561, 4524,  147712,  6, }, // Africa/Khartoum
    {  561, 6869,   20418,  4, }, // Africa/Lome
    {  561, 5113,   44110, 10, }, // Africa/Mogadishu
    {  561, 5271,  147718,  6, }, // Africa/Ndjamena
    {  561, 6803,  147724,  8, }, // Africa/Nouakchott
    {  561, 6727,   20449,  8, }, // Africa/Ouagadougou
    {  561, 5225,   61373,  9, }, // Africa/Porto-Novo
    {  561, 4435,  177574, 10, }, // Africa/Sao_Tome
    {  561, 2866,   20467,  7, }, // Africa/Tripoli
    {  561, 6554,   20474,  5, }, // Africa/Tunis
    {  561, 5607,    4775,  7, }, // America/Anguilla
    {  561, 4554,   26559,  9, }, // America/Araguaina
    {  561,  270,   20479,  7, }, // America/Argentina/Cordoba
    {  561, 7489,   44137, 12, }, // America/Argentina/Rio_Gallegos
    {  561, 7571,   20486,  7, }, // America/Argentina/Tucuman
    {  561, 4252,      14,  8, }, // America/Asuncion
    {  561,  237,      22,  8, }, // America/Atikokan
    {  561, 3679,      30,  5, }, // America/Bahia
    {  561, 7623,      30, 17, }, // America/Bahia_Banderas
    {  561, 7646,  147747,  5, }, // America/Belem
    {  561, 5407,  147752,  6, }, // America/Belize
    {  561, 4361,   20507,  6, }, // America/Bogota
    {  561, 3868,      47,  6, }, // America/Cancun
    {  561, 4345,  147758,  6, }, // America/Cayenne
    {  561, 5496,   20513,  6, }, // America/Cayman
    {  561, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  561, 3791,   26574,  6, }, // America/Cuiaba
    {  561, 5723,  161903,  7, }, // America/Curacao
    {  561, 7869,   26580,  8, }, // America/Eirunepe
    {  561, 4612,  147774, 10, }, // America/Grand_Turk
    {  561, 5770,   20519,  7, }, // America/Grenada
    {  561, 5786,   20526,  9, }, // America/Guadeloupe
    {  561, 2281,   20535,  9, }, // America/Havana
    {  561,  348,   74243, 12, }, // America/Indiana/Indianapolis
    {  561,  481,      73, 13, }, // America/Indiana/Knox
    {  561, 7961,      86, 16, }, // America/Indiana/Marengo
    {  561, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  561, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  561, 8038,     139, 14, }, // America/Indiana/Vevay
    {  561, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  561, 8086,     171, 16, }, // America/Indiana/Winamac
    {  561, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  561, 3239,  177584, 11, }, // America/Los_Angeles
    {  561, 5932,     207, 22, }, // America/Lower_Princes
    {  561, 8153,   26588,  6, }, // America/Maceio
    {  561, 1908,   64887,  6, }, // America/Manaus
    {  561, 5805,   20554,  9, }, // America/Martinique
    {  561, 2917,   26603,  8, }, // America/Mazatlan
    {  561, 8220,     229,  6, }, // America/Merida
    {  561, 2949,   57591, 16, }, // America/Mexico_City
    {  561, 4391,  177595,  8, }, // America/Miquelon
    {  561, 2379,   20578, 10, }, // America/New_York
    {  561, 1850,     235, 19, }, // America/Noronha
    {  561, 8301,   20588, 24, }, // America/North_Dakota/Beulah
    {  561, 8329,   20612, 24, }, // America/North_Dakota/Center
    {  561, 8357,  177603, 27, }, // America/North_Dakota/New_Salem
    {  561,  393,     317,  4, }, // America/Nuuk
    {  561, 2356,   20665,  6, }, // America/Panama
    {  561, 3945,  147901, 15, }, // America/Port-au-Prince
    {  561, 5954,  161926, 13, }, // America/Port_of_Spain
    {  561,  695,   74384, 10, }, // America/Rio_Branco
    {  561, 8477,   26704,  8, }, // America/Santarem
    {  561, 2201,   64260, 17, }, // America/Santiago
    {  561, 1878,   26712,  9, }, // America/Sao_Paulo
    {  561, 6899,     321, 16, }, // America/Scoresbysund
    {  561, 8508,  161939, 13, }, // America/St_Barthelemy
    {  561, 2061,  161952, 21, }, // America/St_Johns
    {  561, 5863,  177630, 13, }, // America/St_Kitts
    {  561, 5880,   74394, 11, }, // America/St_Lucia
    {  561,  855,     379, 10, }, // America/St_Thomas
    {  561, 5913,   74405, 11, }, // America/St_Vincent
    {  561, 5688,   20723,  7, }, // America/Tortola
    {  561, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  561, 5570,   74416,  4, }, // Asia/Aden
    {  561, 3968,   57924,  5, }, // Asia/Amman
    {  561, 5511,  148007,  6, }, // Asia/Anadyr
    {  561, 5523,    5609,  5, }, // Asia/Aqtau
    {  561, 5368,    7606,  6, }, // Asia/Aqtobe
    {  561,  964,  148013,  7, }, // Asia/Ashgabat
    {  561, 3607,    4162,  6, }, // Asia/Baghdad
    {  561, 5534,  148020,  6, }, // Asia/Bahrain
    {  561, 3653,   20747,  4, }, // Asia/Baku
    {  561, 3571,  148026,  7, }, // Asia/Barnaul
    {  561, 5989,   20751,  6, }, // Asia/Brunei
    {  561, 4601,   74433,  5, }, // Asia/Chita
    {  561, 4540,   74438,  7, }, // Asia/Damascus
    {  561, 1093,  158496,  4, }, // Asia/Dhaka
    {  561, 3596,  177643,  5, }, // Asia/Dubai
    {  561, 6159,  148033,  7, }, // Asia/Dushanbe
    {  561, 4795,   20771,  6, }, // Asia/Hebron
    {  561, 1259,  177648, 18, }, // Asia/Ho_Chi_Minh
    {  561, 7006,  161997,  7, }, // Asia/Jakarta
    {  561, 1290,  177666,  9, }, // Asia/Jerusalem
    {  561, 1185,   20830,  8, }, // Asia/Kathmandu
    {  561, 8661,   26856,  8, }, // Asia/Khandyga
    {  561,  992,   20838,  7, }, // Asia/Kolkata
    {  561, 1211,     464,  5, }, // Asia/Macau
    {  561, 4017,   20850,  7, }, // Asia/Magadan
    {  561, 1349,    7736,  7, }, // Asia/Makassar
    {  561, 6920,   20857,  7, }, // Asia/Muscat
    {  561, 5559,   73150,  5, }, // Asia/Qatar
    {  561, 8708,  162012,  8, }, // Asia/Qostanay
    {  561, 4269,   26897,  9, }, // Asia/Qyzylorda
    {  561, 3584,    7787,  4, }, // Asia/Riyadh
    {  561, 4408,  148067,  7, }, // Asia/Sakhalin
    {  561, 7245,   20886, 10, }, // Asia/Samarkand
    {  561, 3279,   20896,  4, }, // Asia/Seoul
    {  561, 1053,  177675,  8, }, // Asia/Shanghai
    {  561, 3300,   20900,  8, }, // Asia/Singapore
    {  561, 4297,    7817, 13, }, // Asia/Srednekolymsk
    {  561, 3263,  177683,  6, }, // Asia/Taipei
    {  561, 4781,   91755,  7, }, // Asia/Tashkent
    {  561, 3915,   78237,  6, }, // Asia/Tbilisi
    {  561, 2772,   58125,  7, }, // Asia/Tehran
    {  561, 1317,   62605,  5, }, // Asia/Thimphu
    {  561, 2821,    4356,  5, }, // Asia/Tokyo
    {  561, 1021,  177689, 10, }, // Asia/Ulaanbaatar
    {  561, 1159,   26933,  6, }, // Asia/Urumqi
    {  561, 6977,   20929,  8, }, // Asia/Vientiane
    {  561, 1235,  177699, 15, }, // Asia/Yangon
    {  561, 3883,  177714, 13, }, // Asia/Yekaterinburg
    {  561, 3747,  148085,  6, }, // Asia/Yerevan
    {  561, 5671,   74483,  8, }, // Atlantic/Bermuda
    {  561, 6628,   74497,  8, }, // Atlantic/Canary
    {  561, 3727,   26968, 10, }, // Atlantic/Cape_Verde
    {  561, 1395,  177727, 11, }, // Atlantic/Faroe
    {  561, 2748,   20997,  9, }, // Atlantic/Reykjavik
    {  561, 7272,  162027, 15, }, // Atlantic/South_Georgia
    {  561, 6821,  148118, 11, }, // Atlantic/St_Helena
    {  561, 1670,   21030,  8, }, // Australia/Adelaide
    {  561, 1463,  148129,  6, }, // Australia/Sydney
    {  561, 6441,   74529,  9, }, // Europe/Amsterdam
    {  561, 3620,   21038,  8, }, // Europe/Astrakhan
    {  561, 2300,   27009,  6, }, // Europe/Athens
    {  561, 6336,   58906,  8, }, // Europe/Belgrade
    {  561, 4757,   21059,  6, }, // Europe/Berlin
    {  561, 2167,  162042,  8, }, // Europe/Brussels
    {  561, 3928,   21073,  8, }, // Europe/Bucharest
    {  561, 2583,  142780,  8, }, // Europe/Chisinau
    {  561, 6289,   21081, 10, }, // Europe/Copenhagen
    {  561, 2338,   21091,  6, }, // Europe/Dublin
    {  561, 8788,   27045,  9, }, // Europe/Guernsey
    {  561, 8804,  148135, 11, }, // Europe/Isle_of_Man
    {  561, 1130,  148146,  8, }, // Europe/Istanbul
    {  561, 3979,   63006, 12, }, // Europe/Kaliningrad
    {  561, 8929,   44561,  5, }, // Europe/Kirov
    {  561, 2527,    8148,  4, }, // Europe/Kyiv
    {  561, 3213,   21137,  6, }, // Europe/Lisbon
    {  561, 6506,   21143,  9, }, // Europe/Ljubljana
    {  561, 2501,   21152,  7, }, // Europe/London
    {  561, 6365,   74574, 10, }, // Europe/Luxembourg
    {  561, 6410,   74584,  6, }, // Europe/Monaco
    {  561, 3537,   21175,  5, }, // Europe/Moscow
    {  561, 4284,   21180,  5, }, // Europe/Paris
    {  561, 6275,   21185,  5, }, // Europe/Prague
    {  561, 6324,    6064,  4, }, // Europe/Rome
    {  561, 4451,  148172,  7, }, // Europe/Saratov
    {  561, 8849,   74605, 10, }, // Europe/Simferopol
    {  561, 6383,  148179,  6, }, // Europe/Skopje
    {  561, 6599,   21199,  5, }, // Europe/Sofia
    {  561, 6523,   74615,  9, }, // Europe/Stockholm
    {  561, 8867,   22925,  6, }, // Europe/Tallinn
    {  561, 6202,   21213,  6, }, // Europe/Tirane
    {  561, 8882,  148185,  9, }, // Europe/Ulyanovsk
    {  561, 6567,  162050, 11, }, // Europe/Vatican
    {  561, 6231,   21230,  5, }, // Europe/Vienna
    {  561, 8899,   64490,  5, }, // Europe/Vilnius
    {  561, 4727,   63187, 10, }, // Europe/Volgograd
    {  561, 3190,   21244,  8, }, // Europe/Warsaw
    {  561, 6540,  148205,  6, }, // Europe/Zurich
    {  561, 6046,  177738,  7, }, // Indian/Christmas
    {  561, 5029,  162733,  7, }, // Indian/Comoro
    {  561, 7260,   27115,  4, }, // Indian/Mahe
    {  561, 7050,   74646,  8, }, // Indian/Maldives
    {  561, 4069,   74654,  8, }, // Indian/Mauritius
    {  561, 7211,   21260,  7, }, // Indian/Reunion
    {  561, 3157,     554,  5, }, // Pacific/Chuuk
    {  561, 2237,  177745, 14, }, // Pacific/Easter
    {  561, 3902,  148230,  4, }, // Pacific/Fiji
    {  561, 6678,   21277,  9, }, // Pacific/Galapagos
    {  561, 2723,   74691,  8, }, // Pacific/Honolulu
    {  561, 3049,    4619,  9, }, // Pacific/Kanton
    {  561, 7098,  124660,  5, }, // Pacific/Noumea
    {  561, 7155,   65550,  6, }, // Pacific/Palau
    {  561, 3096,     586,  7, }, // Pacific/Pohnpei
    {  561, 6031,  177759,  6, }, // Pacific/Saipan
    {  561, 7312,   44657,  6, }, // Pacific/Tahiti
    {  562, 6773,  177765,  7, }, // Africa/Conakry Spanish/Latin/Argentina
    {  562, 7886,  177772, 13, }, // America/Fort_Nelson
    {  562, 5481,  177785,  5, }, // America/Nassau
    {  562, 2201,   64260,  8, }, // America/Santiago
    {  562,  855,  177790, 11, }, // America/St_Thomas
    {  562, 8661,  177801,  8, }, // Asia/Khandyga
    {  562, 6628,  177809, 14, }, // Atlantic/Canary
    {  562, 8772,   27037,  8, }, // Europe/Busingen
    {  562, 2338,   21091,  6, }, // Europe/Dublin
    {  562, 8788,  177823,  8, }, // Europe/Guernsey
    {  562, 7392,  177831,  9, }, // Pacific/Wake
    {  563, 6773,  177765,  7, }, // Africa/Conakry Spanish/Latin/Belize
    {  563, 7886,  177772, 13, }, // America/Fort_Nelson
    {  563, 5481,  177785,  5, }, // America/Nassau
    {  563, 2201,   64260,  8, }, // America/Santiago
    {  563,  855,  177790, 11, }, // America/St_Thomas
    {  563, 8661,  177801,  8, }, // Asia/Khandyga
    {  563, 6628,  177809, 14, }, // Atlantic/Canary
    {  563, 8772,   27037,  8, }, // Europe/Busingen
    {  563, 2338,   21091,  6, }, // Europe/Dublin
    {  563, 8788,  177823,  8, }, // Europe/Guernsey
    {  563, 7392,  177831,  9, }, // Pacific/Wake
    {  564, 6773,  177765,  7, }, // Africa/Conakry Spanish/Latin/Bolivia
    {  564, 7886,  177772, 13, }, // America/Fort_Nelson
    {  564, 5481,  177785,  5, }, // America/Nassau
    {  564, 2201,   64260,  8, }, // America/Santiago
    {  564,  855,  177790, 11, }, // America/St_Thomas
    {  564, 8661,  177801,  8, }, // Asia/Khandyga
    {  564, 6628,  177809, 14, }, // Atlantic/Canary
    {  564, 8772,   27037,  8, }, // Europe/Busingen
    {  564, 2338,   21091,  6, }, // Europe/Dublin
    {  564, 8788,  177823,  8, }, // Europe/Guernsey
    {  564, 7392,  177831,  9, }, // Pacific/Wake
    {  565, 6773,  177765,  7, }, // Africa/Conakry Spanish/Latin/Brazil
    {  565, 7886,  177772, 13, }, // America/Fort_Nelson
    {  565, 5481,  177785,  5, }, // America/Nassau
    {  565, 2201,   64260,  8, }, // America/Santiago
    {  565,  855,  177790, 11, }, // America/St_Thomas
    {  565, 8661,  177801,  8, }, // Asia/Khandyga
    {  565, 6628,  177809, 14, }, // Atlantic/Canary
    {  565, 8772,   27037,  8, }, // Europe/Busingen
    {  565, 2338,   21091,  6, }, // Europe/Dublin
    {  565, 8788,  177823,  8, }, // Europe/Guernsey
    {  565, 7392,  177831,  9, }, // Pacific/Wake
    {  568, 6773,  177765,  7, }, // Africa/Conakry Spanish/Latin/Chile
    {  568, 7886,  177772, 13, }, // America/Fort_Nelson
    {  568, 5481,  177785,  5, }, // America/Nassau
    {  568, 2201,   64260,  8, }, // America/Santiago
    {  568,  855,  177790, 11, }, // America/St_Thomas
    {  568, 8661,  177801,  8, }, // Asia/Khandyga
    {  568, 6628,  177809, 14, }, // Atlantic/Canary
    {  568, 8772,   27037,  8, }, // Europe/Busingen
    {  568, 2338,   21091,  6, }, // Europe/Dublin
    {  568, 8788,  177823,  8, }, // Europe/Guernsey
    {  568, 7392,  177831,  9, }, // Pacific/Wake
    {  569, 6773,  177765,  7, }, // Africa/Conakry Spanish/Latin/Colombia
    {  569, 7886,  177772, 13, }, // America/Fort_Nelson
    {  569, 5481,  177785,  5, }, // America/Nassau
    {  569, 2201,   64260,  8, }, // America/Santiago
    {  569,  855,  177790, 11, }, // America/St_Thomas
    {  569, 8661,  177801,  8, }, // Asia/Khandyga
    {  569, 6628,  177809, 14, }, // Atlantic/Canary
    {  569, 8772,   27037,  8, }, // Europe/Busingen
    {  569, 2338,   21091,  6, }, // Europe/Dublin
    {  569, 8788,  177823,  8, }, // Europe/Guernsey
    {  569, 7392,  177831,  9, }, // Pacific/Wake
    {  570, 6773,  177765,  7, }, // Africa/Conakry Spanish/Latin/Costa Rica
    {  570, 7886,  177772, 13, }, // America/Fort_Nelson
    {  570, 5481,  177785,  5, }, // America/Nassau
    {  570, 2201,   64260,  8, }, // America/Santiago
    {  570,  855,  177790, 11, }, // America/St_Thomas
    {  570, 8661,  177801,  8, }, // Asia/Khandyga
    {  570, 6628,  177809, 14, }, // Atlantic/Canary
    {  570, 8772,   27037,  8, }, // Europe/Busingen
    {  570, 2338,   21091,  6, }, // Europe/Dublin
    {  570, 8788,  177823,  8, }, // Europe/Guernsey
    {  570, 7392,  177831,  9, }, // Pacific/Wake
    {  571, 6773,  177765,  7, }, // Africa/Conakry Spanish/Latin/Cuba
    {  571, 7886,  177772, 13, }, // America/Fort_Nelson
    {  571, 5481,  177785,  5, }, // America/Nassau
    {  571, 2201,   64260,  8, }, // America/Santiago
    {  571,  855,  177790, 11, }, // America/St_Thomas
    {  571, 8661,  177801,  8, }, // Asia/Khandyga
    {  571, 6628,  177809, 14, }, // Atlantic/Canary
    {  571, 8772,   27037,  8, }, // Europe/Busingen
    {  571, 2338,   21091,  6, }, // Europe/Dublin
    {  571, 8788,  177823,  8, }, // Europe/Guernsey
    {  571, 7392,  177831,  9, }, // Pacific/Wake
    {  572, 6773,  177765,  7, }, // Africa/Conakry Spanish/Latin/Dominican Republic
    {  572, 7886,  177772, 13, }, // America/Fort_Nelson
    {  572, 5481,  177785,  5, }, // America/Nassau
    {  572, 2201,   64260,  8, }, // America/Santiago
    {  572,  855,  177790, 11, }, // America/St_Thomas
    {  572, 8661,  177801,  8, }, // Asia/Khandyga
    {  572, 6628,  177809, 14, }, // Atlantic/Canary
    {  572, 8772,   27037,  8, }, // Europe/Busingen
    {  572, 2338,   21091,  6, }, // Europe/Dublin
    {  572, 8788,  177823,  8, }, // Europe/Guernsey
    {  572, 7392,  177831,  9, }, // Pacific/Wake
    {  573, 6773,  177765,  7, }, // Africa/Conakry Spanish/Latin/Ecuador
    {  573, 7886,  177772, 13, }, // America/Fort_Nelson
    {  573, 5481,  177785,  5, }, // America/Nassau
    {  573, 2201,   64260,  8, }, // America/Santiago
    {  573,  855,  177790, 11, }, // America/St_Thomas
    {  573, 8661,  177801,  8, }, // Asia/Khandyga
    {  573, 6628,  177809, 14, }, // Atlantic/Canary
    {  573, 8772,   27037,  8, }, // Europe/Busingen
    {  573, 2338,   21091,  6, }, // Europe/Dublin
    {  573, 8788,  177823,  8, }, // Europe/Guernsey
    {  573, 7392,  177831,  9, }, // Pacific/Wake
    {  574, 6773,  177765,  7, }, // Africa/Conakry Spanish/Latin/El Salvador
    {  574, 7886,  177772, 13, }, // America/Fort_Nelson
    {  574, 5481,  177785,  5, }, // America/Nassau
    {  574, 2201,   64260,  8, }, // America/Santiago
    {  574,  855,  177790, 11, }, // America/St_Thomas
    {  574, 8661,  177801,  8, }, // Asia/Khandyga
    {  574, 6628,  177809, 14, }, // Atlantic/Canary
    {  574, 8772,   27037,  8, }, // Europe/Busingen
    {  574, 2338,   21091,  6, }, // Europe/Dublin
    {  574, 8788,  177823,  8, }, // Europe/Guernsey
    {  574, 7392,  177831,  9, }, // Pacific/Wake
    {  576, 6773,  177765,  7, }, // Africa/Conakry Spanish/Latin/Guatemala
    {  576, 7886,  177772, 13, }, // America/Fort_Nelson
    {  576, 5481,  177785,  5, }, // America/Nassau
    {  576, 2201,   64260,  8, }, // America/Santiago
    {  576,  855,  177790, 11, }, // America/St_Thomas
    {  576, 8661,  177801,  8, }, // Asia/Khandyga
    {  576, 6628,  177809, 14, }, // Atlantic/Canary
    {  576, 8772,   27037,  8, }, // Europe/Busingen
    {  576, 2338,   21091,  6, }, // Europe/Dublin
    {  576, 8788,  177823,  8, }, // Europe/Guernsey
    {  576, 7392,  177831,  9, }, // Pacific/Wake
    {  577, 6773,  177765,  7, }, // Africa/Conakry Spanish/Latin/Honduras
    {  577, 7886,  177772, 13, }, // America/Fort_Nelson
    {  577, 5481,  177785,  5, }, // America/Nassau
    {  577, 2201,   64260,  8, }, // America/Santiago
    {  577,  855,  177790, 11, }, // America/St_Thomas
    {  577, 8661,  177801,  8, }, // Asia/Khandyga
    {  577, 6628,  177809, 14, }, // Atlantic/Canary
    {  577, 8772,   27037,  8, }, // Europe/Busingen
    {  577, 2338,   21091,  6, }, // Europe/Dublin
    {  577, 8788,  177823,  8, }, // Europe/Guernsey
    {  577, 7392,  177831,  9, }, // Pacific/Wake
    {  578, 6773,  177765,  7, }, // Africa/Conakry Spanish/Latin/Latin America
    {  578, 7886,  177772, 13, }, // America/Fort_Nelson
    {  578, 5481,  177785,  5, }, // America/Nassau
    {  578, 2201,   64260,  8, }, // America/Santiago
    {  578,  855,  177790, 11, }, // America/St_Thomas
    {  578, 8661,  177801,  8, }, // Asia/Khandyga
    {  578, 6628,  177809, 14, }, // Atlantic/Canary
    {  578, 8772,   27037,  8, }, // Europe/Busingen
    {  578, 2338,   21091,  6, }, // Europe/Dublin
    {  578, 8788,  177823,  8, }, // Europe/Guernsey
    {  578, 7392,  177831,  9, }, // Pacific/Wake
    {  579, 4922,  177840,  9, }, // Africa/Bujumbura Spanish/Latin/Mexico
    {  579, 5130,   91390, 13, }, // Africa/Dar_es_Salaam
    {  579, 7886,  125399, 11, }, // America/Fort_Nelson
    {  579, 5481,  177785,  5, }, // America/Nassau
    {  579,  695,  177849, 10, }, // America/Rio_Branco
    {  579, 2201,   64260,  8, }, // America/Santiago
    {  579, 5395,  177859,  6, }, // Asia/Almaty
    {  579, 5368,  177865,  6, }, // Asia/Aqtobe
    {  579, 8624,    7612,  6, }, // Asia/Atyrau
    {  579, 8661,  177801,  8, }, // Asia/Khandyga
    {  579, 6628,  177809, 14, }, // Atlantic/Canary
    {  579, 8772,   27037,  8, }, // Europe/Busingen
    {  579, 2338,   21091,  6, }, // Europe/Dublin
    {  579, 8788,  177823,  8, }, // Europe/Guernsey
    {  579, 2723,    4611,  8, }, // Pacific/Honolulu
    {  579, 7392,     593,  4, }, // Pacific/Wake
    {  580, 6773,  177765,  7, }, // Africa/Conakry Spanish/Latin/Nicaragua
    {  580, 7886,  177772, 13, }, // America/Fort_Nelson
    {  580, 5481,  177785,  5, }, // America/Nassau
    {  580, 2201,   64260,  8, }, // America/Santiago
    {  580,  855,  177790, 11, }, // America/St_Thomas
    {  580, 8661,  177801,  8, }, // Asia/Khandyga
    {  580, 6628,  177809, 14, }, // Atlantic/Canary
    {  580, 8772,   27037,  8, }, // Europe/Busingen
    {  580, 2338,   21091,  6, }, // Europe/Dublin
    {  580, 8788,  177823,  8, }, // Europe/Guernsey
    {  580, 7392,  177831,  9, }, // Pacific/Wake
    {  581, 6773,  177765,  7, }, // Africa/Conakry Spanish/Latin/Panama
    {  581, 7886,  177772, 13, }, // America/Fort_Nelson
    {  581, 5481,  177785,  5, }, // America/Nassau
    {  581, 2201,   64260,  8, }, // America/Santiago
    {  581,  855,  177790, 11, }, // America/St_Thomas
    {  581, 8661,  177801,  8, }, // Asia/Khandyga
    {  581, 6628,  177809, 14, }, // Atlantic/Canary
    {  581, 8772,   27037,  8, }, // Europe/Busingen
    {  581, 2338,   21091,  6, }, // Europe/Dublin
    {  581, 8788,  177823,  8, }, // Europe/Guernsey
    {  581, 7392,  177831,  9, }, // Pacific/Wake
    {  582, 6773,  177765,  7, }, // Africa/Conakry Spanish/Latin/Paraguay
    {  582, 7886,  177772, 13, }, // America/Fort_Nelson
    {  582, 5481,  177785,  5, }, // America/Nassau
    {  582, 2201,   64260,  8, }, // America/Santiago
    {  582,  855,  177790, 11, }, // America/St_Thomas
    {  582, 8661,  177801,  8, }, // Asia/Khandyga
    {  582, 6628,  177809, 14, }, // Atlantic/Canary
    {  582, 8772,   27037,  8, }, // Europe/Busingen
    {  582, 2338,   21091,  6, }, // Europe/Dublin
    {  582, 8788,  177823,  8, }, // Europe/Guernsey
    {  582, 7392,  177831,  9, }, // Pacific/Wake
    {  583, 6773,  177765,  7, }, // Africa/Conakry Spanish/Latin/Peru
    {  583, 7886,  177772, 13, }, // America/Fort_Nelson
    {  583, 5481,  177785,  5, }, // America/Nassau
    {  583, 2201,   64260,  8, }, // America/Santiago
    {  583,  855,  177790, 11, }, // America/St_Thomas
    {  583, 8661,  177801,  8, }, // Asia/Khandyga
    {  583, 6628,  177809, 14, }, // Atlantic/Canary
    {  583, 8772,   27037,  8, }, // Europe/Busingen
    {  583, 2338,   21091,  6, }, // Europe/Dublin
    {  583, 8788,  177823,  8, }, // Europe/Guernsey
    {  583, 7392,  177831,  9, }, // Pacific/Wake
    {  585, 6773,  177765,  7, }, // Africa/Conakry Spanish/Latin/Puerto Rico
    {  585, 7886,  177772, 13, }, // America/Fort_Nelson
    {  585, 5481,  177785,  5, }, // America/Nassau
    {  585, 2201,   64260,  8, }, // America/Santiago
    {  585,  855,  177790, 11, }, // America/St_Thomas
    {  585, 8661,  177801,  8, }, // Asia/Khandyga
    {  585, 6628,  177809, 14, }, // Atlantic/Canary
    {  585, 8772,   27037,  8, }, // Europe/Busingen
    {  585, 2338,   21091,  6, }, // Europe/Dublin
    {  585, 8788,  177823,  8, }, // Europe/Guernsey
    {  585, 7392,  177831,  9, }, // Pacific/Wake
    {  586, 6773,  177765,  7, }, // Africa/Conakry Spanish/Latin/United States
    {  586, 5043,  177871,  6, }, // Africa/Djibouti
    {  586, 7886,  125399, 11, }, // America/Fort_Nelson
    {  586, 5481,  177877,  6, }, // America/Nassau
    {  586, 2201,   64260,  8, }, // America/Santiago
    {  586, 8661,  177801,  8, }, // Asia/Khandyga
    {  586, 4214,  162004,  8, }, // Asia/Pyongyang
    {  586, 6628,  177809, 14, }, // Atlantic/Canary
    {  586, 3620,  177883,  9, }, // Europe/Astrakhan
    {  586, 8772,   27037,  8, }, // Europe/Busingen
    {  586, 2338,   21091,  6, }, // Europe/Dublin
    {  586, 8788,  177823,  8, }, // Europe/Guernsey
    {  586, 8929,  124195,  5, }, // Europe/Kirov
    {  586, 8882,  177892,  9, }, // Europe/Ulyanovsk
    {  586, 2723,    4611,  8, }, // Pacific/Honolulu
    {  586, 7392,     593,  4, }, // Pacific/Wake
    {  587, 6773,  177765,  7, }, // Africa/Conakry Spanish/Latin/Uruguay
    {  587, 7886,  177772, 13, }, // America/Fort_Nelson
    {  587, 5481,  177785,  5, }, // America/Nassau
    {  587, 2201,   64260,  8, }, // America/Santiago
    {  587,  855,  177790, 11, }, // America/St_Thomas
    {  587, 8661,  177801,  8, }, // Asia/Khandyga
    {  587, 6628,  177809, 14, }, // Atlantic/Canary
    {  587, 8772,   27037,  8, }, // Europe/Busingen
    {  587, 2338,   21091,  6, }, // Europe/Dublin
    {  587, 8788,  177823,  8, }, // Europe/Guernsey
    {  587, 7392,  177831,  9, }, // Pacific/Wake
    {  588, 6773,  177765,  7, }, // Africa/Conakry Spanish/Latin/Venezuela
    {  588, 7886,  177772, 13, }, // America/Fort_Nelson
    {  588, 5481,  177785,  5, }, // America/Nassau
    {  588, 2201,   64260,  8, }, // America/Santiago
    {  588,  855,  177790, 11, }, // America/St_Thomas
    {  588, 8661,  177801,  8, }, // Asia/Khandyga
    {  588, 6628,  177809, 14, }, // Atlantic/Canary
    {  588, 8772,   27037,  8, }, // Europe/Busingen
    {  588, 2338,   21091,  6, }, // Europe/Dublin
    {  588, 8788,  177823,  8, }, // Europe/Guernsey
    {  588, 7392,  177831,  9, }, // Pacific/Wake
    {  589,   14,       0,  6, }, // Africa/Asmara Standard Moroccan Tamazight/Tifinagh/Morocco
    {  589, 4435,       6,  8, }, // Africa/Sao_Tome
    {  589, 4252,      14,  8, }, // America/Asuncion
    {  589,  237,      22,  8, }, // America/Atikokan
    {  589, 7623,      30, 17, }, // America/Bahia_Banderas
    {  589, 3868,      47,  6, }, // America/Cancun
    {  589, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  589, 5723,      66,  7, }, // America/Curacao
    {  589,  481,      73, 13, }, // America/Indiana/Knox
    {  589, 7961,      86, 16, }, // America/Indiana/Marengo
    {  589, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  589, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  589, 8038,     139, 14, }, // America/Indiana/Vevay
    {  589, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  589, 8086,     171, 16, }, // America/Indiana/Winamac
    {  589, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  589, 5932,     207, 22, }, // America/Lower_Princes
    {  589, 8220,     229,  6, }, // America/Merida
    {  589, 1850,     235, 19, }, // America/Noronha
    {  589, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  589, 8329,     274, 20, }, // America/North_Dakota/Center
    {  589, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  589,  393,     317,  4, }, // America/Nuuk
    {  589, 6899,     321, 16, }, // America/Scoresbysund
    {  589, 8508,     337, 14, }, // America/St_Barthelemy
    {  589, 2061,     351, 10, }, // America/St_Johns
    {  589, 5863,     361,  9, }, // America/St_Kitts
    {  589, 5880,     370,  9, }, // America/St_Lucia
    {  589,  855,     379, 10, }, // America/St_Thomas
    {  589, 5913,     389, 11, }, // America/St_Vincent
    {  589, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  589, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  589, 1185,     448,  9, }, // Asia/Kathmandu
    {  589,  992,     457,  7, }, // Asia/Kolkata
    {  589, 1211,     464,  5, }, // Asia/Macau
    {  589, 1235,     477,  6, }, // Asia/Yangon
    {  589, 1395,     483,  5, }, // Atlantic/Faroe
    {  589, 6821,     488, 10, }, // Atlantic/St_Helena
    {  589, 2527,     514,  4, }, // Europe/Kyiv
    {  589, 7211,     547,  7, }, // Indian/Reunion
    {  589, 3157,     554,  5, }, // Pacific/Chuuk
    {  589, 3096,     586,  7, }, // Pacific/Pohnpei
    {  590,   14,       0,  6, }, // Africa/Asmara Sundanese/Latin/Indonesia
    {  590, 4435,       6,  8, }, // Africa/Sao_Tome
    {  590, 4252,      14,  8, }, // America/Asuncion
    {  590,  237,      22,  8, }, // America/Atikokan
    {  590, 7623,      30, 17, }, // America/Bahia_Banderas
    {  590, 3868,      47,  6, }, // America/Cancun
    {  590, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  590, 5723,      66,  7, }, // America/Curacao
    {  590,  481,      73, 13, }, // America/Indiana/Knox
    {  590, 7961,      86, 16, }, // America/Indiana/Marengo
    {  590, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  590, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  590, 8038,     139, 14, }, // America/Indiana/Vevay
    {  590, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  590, 8086,     171, 16, }, // America/Indiana/Winamac
    {  590, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  590, 5932,     207, 22, }, // America/Lower_Princes
    {  590, 8220,     229,  6, }, // America/Merida
    {  590, 1850,     235, 19, }, // America/Noronha
    {  590, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  590, 8329,     274, 20, }, // America/North_Dakota/Center
    {  590, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  590,  393,     317,  4, }, // America/Nuuk
    {  590, 6899,     321, 16, }, // America/Scoresbysund
    {  590, 8508,     337, 14, }, // America/St_Barthelemy
    {  590, 2061,     351, 10, }, // America/St_Johns
    {  590, 5863,     361,  9, }, // America/St_Kitts
    {  590, 5880,     370,  9, }, // America/St_Lucia
    {  590,  855,     379, 10, }, // America/St_Thomas
    {  590, 5913,     389, 11, }, // America/St_Vincent
    {  590, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  590, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  590, 1185,     448,  9, }, // Asia/Kathmandu
    {  590,  992,     457,  7, }, // Asia/Kolkata
    {  590, 1211,   33537,  5, }, // Asia/Macau
    {  590, 1235,     477,  6, }, // Asia/Yangon
    {  590, 1395,     483,  5, }, // Atlantic/Faroe
    {  590, 6821,     488, 10, }, // Atlantic/St_Helena
    {  590, 2527,     514,  4, }, // Europe/Kyiv
    {  590, 7211,     547,  7, }, // Indian/Reunion
    {  590, 3157,     554,  5, }, // Pacific/Chuuk
    {  590, 3096,     586,  7, }, // Pacific/Pohnpei
    {  591,   14,       0,  6, }, // Africa/Asmara Swahili/Latin/Tanzania
    {  591, 4435,       6,  8, }, // Africa/Sao_Tome
    {  591, 4252,      14,  8, }, // America/Asuncion
    {  591,  237,      22,  8, }, // America/Atikokan
    {  591, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  591, 3868,   33314,  6, }, // America/Cancun
    {  591, 7774,   91278, 13, }, // America/Ciudad_Juarez
    {  591, 5723,      66,  7, }, // America/Curacao
    {  591,  481,      73, 13, }, // America/Indiana/Knox
    {  591, 7961,      86, 16, }, // America/Indiana/Marengo
    {  591, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  591, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  591, 8038,     139, 14, }, // America/Indiana/Vevay
    {  591, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  591, 8086,     171, 16, }, // America/Indiana/Winamac
    {  591, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  591, 5932,     207, 22, }, // America/Lower_Princes
    {  591, 8220,    7126,  6, }, // America/Merida
    {  591, 2949,  177901, 14, }, // America/Mexico_City
    {  591, 1850,     235, 19, }, // America/Noronha
    {  591, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  591, 8329,     274, 20, }, // America/North_Dakota/Center
    {  591, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  591,  393,     317,  4, }, // America/Nuuk
    {  591, 6899,     321, 16, }, // America/Scoresbysund
    {  591, 8508,     337, 14, }, // America/St_Barthelemy
    {  591, 2061,     351, 10, }, // America/St_Johns
    {  591, 5863,     361,  9, }, // America/St_Kitts
    {  591, 5880,     370,  9, }, // America/St_Lucia
    {  591,  855,     379, 10, }, // America/St_Thomas
    {  591, 5913,     389, 11, }, // America/St_Vincent
    {  591, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  591, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  591, 1185,     448,  9, }, // Asia/Kathmandu
    {  591,  992,     457,  7, }, // Asia/Kolkata
    {  591, 1211,    4321,  5, }, // Asia/Macau
    {  591, 8708,     469,  8, }, // Asia/Qostanay
    {  591, 1235,   58618,  7, }, // Asia/Yangon
    {  591, 1395,     483,  5, }, // Atlantic/Faroe
    {  591, 7272,  177915, 14, }, // Atlantic/South_Georgia
    {  591, 6821,     488, 10, }, // Atlantic/St_Helena
    {  591, 2527,    8148,  4, }, // Europe/Kyiv
    {  591, 7211,     547,  7, }, // Indian/Reunion
    {  591, 3157,     554,  5, }, // Pacific/Chuuk
    {  591, 2723,    4611,  8, }, // Pacific/Honolulu
    {  591, 3049,    4619,  9, }, // Pacific/Kanton
    {  591, 3096,     586,  7, }, // Pacific/Pohnpei
    {  593, 5654,  177929,  7, }, // America/Barbados Swahili/Latin/Kenya
    {  593, 5422,   33320,  9, }, // America/Costa_Rica
    {  593, 5723,  177936,  7, }, // America/Curacao
    {  593, 5739,   21666,  8, }, // America/Dominica
    {  593, 5441,  177943,  9, }, // America/El_Salvador
    {  593, 5786,   20526,  9, }, // America/Guadeloupe
    {  593, 2799,    3995,  7, }, // America/Jamaica
    {  593, 8301,  177952, 24, }, // America/North_Dakota/Beulah
    {  593, 8329,  177976, 24, }, // America/North_Dakota/Center
    {  593, 8357,  178000, 27, }, // America/North_Dakota/New_Salem
    {  593, 5954,  178027, 20, }, // America/Port_of_Spain
    {  593, 5843,  178047,  9, }, // America/Puerto_Rico
    {  593, 4511,    5637,  7, }, // Asia/Colombo
    {  593, 1259,  178056, 19, }, // Asia/Ho_Chi_Minh
    {  593, 1211,    7731,  5, }, // Asia/Macau
    {  593, 3300,  178075,  8, }, // Asia/Singapore
    {  593, 1235,     477,  6, }, // Asia/Yangon
    {  593, 6628,    4398,  6, }, // Atlantic/Canary
    {  593, 6046,  178083,  8, }, // Indian/Christmas
    {  593, 7050,   33819,  7, }, // Indian/Maldives
    {  595,   14,       0,  6, }, // Africa/Asmara Swati/Latin/South Africa
    {  595, 4435,       6,  8, }, // Africa/Sao_Tome
    {  595, 4252,      14,  8, }, // America/Asuncion
    {  595,  237,      22,  8, }, // America/Atikokan
    {  595, 7623,      30, 17, }, // America/Bahia_Banderas
    {  595, 3868,      47,  6, }, // America/Cancun
    {  595, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  595, 5723,      66,  7, }, // America/Curacao
    {  595,  481,      73, 13, }, // America/Indiana/Knox
    {  595, 7961,      86, 16, }, // America/Indiana/Marengo
    {  595, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  595, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  595, 8038,     139, 14, }, // America/Indiana/Vevay
    {  595, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  595, 8086,     171, 16, }, // America/Indiana/Winamac
    {  595, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  595, 5932,     207, 22, }, // America/Lower_Princes
    {  595, 8220,     229,  6, }, // America/Merida
    {  595, 1850,     235, 19, }, // America/Noronha
    {  595, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  595, 8329,     274, 20, }, // America/North_Dakota/Center
    {  595, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  595,  393,     317,  4, }, // America/Nuuk
    {  595, 6899,     321, 16, }, // America/Scoresbysund
    {  595, 8508,     337, 14, }, // America/St_Barthelemy
    {  595, 2061,     351, 10, }, // America/St_Johns
    {  595, 5863,     361,  9, }, // America/St_Kitts
    {  595, 5880,     370,  9, }, // America/St_Lucia
    {  595,  855,     379, 10, }, // America/St_Thomas
    {  595, 5913,     389, 11, }, // America/St_Vincent
    {  595, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  595, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  595, 1185,     448,  9, }, // Asia/Kathmandu
    {  595,  992,     457,  7, }, // Asia/Kolkata
    {  595, 1211,     464,  5, }, // Asia/Macau
    {  595, 1235,     477,  6, }, // Asia/Yangon
    {  595, 1395,     483,  5, }, // Atlantic/Faroe
    {  595, 6821,     488, 10, }, // Atlantic/St_Helena
    {  595, 2527,     514,  4, }, // Europe/Kyiv
    {  595, 7211,     547,  7, }, // Indian/Reunion
    {  595, 3157,     554,  5, }, // Pacific/Chuuk
    {  595, 3096,     586,  7, }, // Pacific/Pohnpei
    {  597, 5059,    3902, 11, }, // Africa/Addis_Ababa Swedish/Latin/Sweden
    {  597, 7420,   44088,  5, }, // Africa/Algiers
    {  597,   14,       0,  6, }, // Africa/Asmara
    {  597, 2320,    4690,  5, }, // Africa/Cairo
    {  597, 5130,   91390, 13, }, // Africa/Dar_es_Salaam
    {  597, 5166,  173955,  8, }, // Africa/El_Aaiun
    {  597, 6869,   20418,  4, }, // Africa/Lome
    {  597, 5271,   44128,  9, }, // Africa/Ndjamena
    {  597, 4435,       6,  8, }, // Africa/Sao_Tome
    {  597, 4554,   26559,  9, }, // America/Araguaina
    {  597,  270,   20479,  7, }, // America/Argentina/Cordoba
    {  597,  441,  162356, 21, }, // America/Argentina/Jujuy
    {  597, 7489,   44137, 12, }, // America/Argentina/Rio_Gallegos
    {  597, 7571,   20486,  7, }, // America/Argentina/Tucuman
    {  597, 4252,      14,  8, }, // America/Asuncion
    {  597,  237,      22,  8, }, // America/Atikokan
    {  597, 7623,      30, 17, }, // America/Bahia_Banderas
    {  597, 7646,   35896,  5, }, // America/Belem
    {  597, 4361,   20507,  6, }, // America/Bogota
    {  597, 3868,      47,  6, }, // America/Cancun
    {  597, 5496,  178091, 11, }, // America/Cayman
    {  597, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  597, 3791,   26574,  6, }, // America/Cuiaba
    {  597, 5723,      66,  7, }, // America/Curacao
    {  597, 7869,   26580,  8, }, // America/Eirunepe
    {  597, 2281,   63505,  7, }, // America/Havana
    {  597,  481,      73, 13, }, // America/Indiana/Knox
    {  597, 7961,      86, 16, }, // America/Indiana/Marengo
    {  597, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  597, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  597, 8038,     139, 14, }, // America/Indiana/Vevay
    {  597, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  597, 8086,     171, 16, }, // America/Indiana/Winamac
    {  597, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  597, 5932,     207, 22, }, // America/Lower_Princes
    {  597, 8153,   26588,  6, }, // America/Maceio
    {  597, 2917,   26603,  8, }, // America/Mazatlan
    {  597, 8220,     229,  6, }, // America/Merida
    {  597, 1850,     235, 19, }, // America/Noronha
    {  597, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  597, 8329,     274, 20, }, // America/North_Dakota/Center
    {  597, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  597,  393,     317,  4, }, // America/Nuuk
    {  597, 8477,   26704,  8, }, // America/Santarem
    {  597, 1878,   26712,  9, }, // America/Sao_Paulo
    {  597, 6899,     321, 16, }, // America/Scoresbysund
    {  597, 8508,   26721, 16, }, // America/St_Barthelemy
    {  597, 2061,   26737, 12, }, // America/St_Johns
    {  597, 5863,   26749, 11, }, // America/St_Kitts
    {  597, 5880,   35964, 11, }, // America/St_Lucia
    {  597,  855,   44293, 12, }, // America/St_Thomas
    {  597, 5913,   26783, 13, }, // America/St_Vincent
    {  597, 5756,   26796,  7, }, // America/Thule
    {  597, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  597, 5523,    5609,  5, }, // Asia/Aqtau
    {  597, 5368,   57205,  6, }, // Asia/Aqtobe
    {  597,  964,    4153,  9, }, // Asia/Ashgabat
    {  597, 3607,    4162,  6, }, // Asia/Baghdad
    {  597, 3778,    4187,  7, }, // Asia/Bishkek
    {  597, 4601,  178102,  5, }, // Asia/Chita
    {  597, 4540,    4201,  8, }, // Asia/Damascus
    {  597, 6159,   58581,  8, }, // Asia/Dushanbe
    {  597, 1259,  178107, 18, }, // Asia/Ho_Chi_Minh
    {  597, 2704,    4231,  8, }, // Asia/Hong_Kong
    {  597, 4771,  174150,  5, }, // Asia/Hovd
    {  597, 4316,   58589,  9, }, // Asia/Kamchatka
    {  597, 1185,    5711,  8, }, // Asia/Kathmandu
    {  597, 8661,   58020,  8, }, // Asia/Khandyga
    {  597,  992,     457,  7, }, // Asia/Kolkata
    {  597, 4197,    4288, 11, }, // Asia/Krasnoyarsk
    {  597, 1211,     464,  5, }, // Asia/Macau
    {  597, 6920,    4326,  6, }, // Asia/Muscat
    {  597, 8708,    7770,  8, }, // Asia/Qostanay
    {  597, 4408,   58084,  8, }, // Asia/Sakhalin
    {  597, 4781,   58610,  8, }, // Asia/Tashkent
    {  597, 2772,    7842,  7, }, // Asia/Tehran
    {  597, 1159,   26933,  6, }, // Asia/Urumqi
    {  597, 4828,    7873,  7, }, // Asia/Yakutsk
    {  597, 1235,     477,  6, }, // Asia/Yangon
    {  597, 3883,    4380, 13, }, // Asia/Yekaterinburg
    {  597, 3747,    7898,  7, }, // Asia/Yerevan
    {  597, 3663,  178125,  8, }, // Atlantic/Azores
    {  597, 6628,  178133, 12, }, // Atlantic/Canary
    {  597, 3727,   57380,  9, }, // Atlantic/Cape_Verde
    {  597, 1395,  178145,  8, }, // Atlantic/Faroe
    {  597, 7272,  178153, 11, }, // Atlantic/South_Georgia
    {  597, 6821,  178164, 12, }, // Atlantic/St_Helena
    {  597, 2300,    5911,  4, }, // Europe/Athens
    {  597, 6336,   27015,  7, }, // Europe/Belgrade
    {  597, 2167,   64400,  7, }, // Europe/Brussels
    {  597, 3928,   27029,  8, }, // Europe/Bucharest
    {  597, 8772,   62908, 21, }, // Europe/Busingen
    {  597, 2583,   63785,  8, }, // Europe/Chisinau
    {  597, 6289,  178176,  9, }, // Europe/Copenhagen
    {  597, 6612,  147368, 11, }, // Europe/Helsinki
    {  597, 2527,    8148,  4, }, // Europe/Kyiv
    {  597, 3213,    4499,  8, }, // Europe/Lisbon
    {  597, 6365,    4513,  9, }, // Europe/Luxembourg
    {  597, 3537,   22880,  6, }, // Europe/Moscow
    {  597, 6275,    8201,  4, }, // Europe/Prague
    {  597, 6324,    6064,  3, }, // Europe/Rome
    {  597, 6202,   21213,  6, }, // Europe/Tirane
    {  597, 8882,    8261,  9, }, // Europe/Ulyanovsk
    {  597, 6567,  178185,  9, }, // Europe/Vatican
    {  597, 6231,   57515,  4, }, // Europe/Vienna
    {  597, 3190,   58675,  8, }, // Europe/Warsaw
    {  597, 6540,    4577,  6, }, // Europe/Zurich
    {  597, 6046,  178194,  5, }, // Indian/Christmas
    {  597, 6063,  178199, 10, }, // Indian/Cocos
    {  597, 5029,  178209,  9, }, // Indian/Comoro
    {  597, 6661,  178218, 14, }, // Indian/Kerguelen
    {  597, 7260,   27115,  4, }, // Indian/Mahe
    {  597, 7050,  178232, 10, }, // Indian/Maldives
    {  597, 7211,     547,  7, }, // Indian/Reunion
    {  597, 3157,     554,  5, }, // Pacific/Chuuk
    {  597, 2237,  178242,  6, }, // Pacific/Easter
    {  597, 6678,   21277,  9, }, // Pacific/Galapagos
    {  597, 6696,  178248, 12, }, // Pacific/Gambier
    {  597, 2723,    4611,  8, }, // Pacific/Honolulu
    {  597, 3049,    4619,  9, }, // Pacific/Kanton
    {  597, 4051,  178260, 14, }, // Pacific/Marquesas
    {  597, 8914,  178274, 11, }, // Pacific/Midway
    {  597, 7098,    4628,  6, }, // Pacific/Noumea
    {  597, 7194,  178285, 13, }, // Pacific/Pitcairn
    {  597, 3096,     586,  7, }, // Pacific/Pohnpei
    {  597, 7405,  178298,  8, }, // Pacific/Wallis
    {  600, 6760,   21292,  5, }, // Africa/Accra Swiss German/Latin/Switzerland
    {  600, 5059,    3902, 11, }, // Africa/Addis_Ababa
    {  600, 7420,   58555,  6, }, // Africa/Algiers
    {  600,   14,       0,  6, }, // Africa/Asmara
    {  600, 2320,    4690,  5, }, // Africa/Cairo
    {  600, 5130,   78080, 10, }, // Africa/Dar_es_Salaam
    {  600, 5043,   78090,  9, }, // Africa/Djibouti
    {  600, 5166,   63491,  8, }, // Africa/El_Aaiun
    {  600, 4524,   26504,  7, }, // Africa/Khartoum
    {  600, 5113,   78099, 10, }, // Africa/Mogadishu
    {  600, 6727,    4758,  8, }, // Africa/Ouagadougou
    {  600, 4435,       6,  8, }, // Africa/Sao_Tome
    {  600, 4252,      14,  8, }, // America/Asuncion
    {  600,  237,      22,  8, }, // America/Atikokan
    {  600, 7623,      30, 17, }, // America/Bahia_Banderas
    {  600, 3868,      47,  6, }, // America/Cancun
    {  600, 5496,  178306, 11, }, // America/Cayman
    {  600, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  600, 5723,      66,  7, }, // America/Curacao
    {  600, 5441,    6806,  8, }, // America/El_Salvador
    {  600, 2281,   63505,  7, }, // America/Havana
    {  600,  481,      73,  4, }, // America/Indiana/Knox
    {  600, 7961,      86, 16, }, // America/Indiana/Marengo
    {  600, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  600, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  600, 8038,     139, 14, }, // America/Indiana/Vevay
    {  600, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  600, 8086,     171, 16, }, // America/Indiana/Winamac
    {  600, 2799,    3995,  7, }, // America/Jamaica
    {  600, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  600, 5932,     207, 22, }, // America/Lower_Princes
    {  600, 8220,     229,  6, }, // America/Merida
    {  600, 2949,  178317, 14, }, // America/Mexico_City
    {  600, 1850,     235, 19, }, // America/Noronha
    {  600, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  600, 8329,     274, 20, }, // America/North_Dakota/Center
    {  600, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  600,  393,     317,  4, }, // America/Nuuk
    {  600, 5954,   26691, 13, }, // America/Port_of_Spain
    {  600, 6899,     321, 16, }, // America/Scoresbysund
    {  600, 8508,     337, 14, }, // America/St_Barthelemy
    {  600, 2061,     351, 10, }, // America/St_Johns
    {  600, 5863,     361,  9, }, // America/St_Kitts
    {  600, 5880,     370,  9, }, // America/St_Lucia
    {  600,  855,     379, 10, }, // America/St_Thomas
    {  600, 5913,     389, 11, }, // America/St_Vincent
    {  600, 6133,  127892, 16, }, // Antarctica/DumontDUrville
    {  600, 7374,  178331,  8, }, // Antarctica/Vostok
    {  600, 5368,    7606,  6, }, // Asia/Aqtobe
    {  600, 3607,    4162,  6, }, // Asia/Baghdad
    {  600, 3778,   57220,  8, }, // Asia/Bishkek
    {  600, 4540,    4201,  8, }, // Asia/Damascus
    {  600, 6159,   57228,  9, }, // Asia/Dushanbe
    {  600, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  600, 2704,    4231,  8, }, // Asia/Hong_Kong
    {  600, 4316,   78179, 11, }, // Asia/Kamchatka
    {  600, 1185,     448,  9, }, // Asia/Kathmandu
    {  600,  992,     457,  7, }, // Asia/Kolkata
    {  600, 4197,    4288, 11, }, // Asia/Krasnoyarsk
    {  600, 1211,     464,  5, }, // Asia/Macau
    {  600, 6920,  178339,  8, }, // Asia/Muscat
    {  600, 2554,    5738,  7, }, // Asia/Nicosia
    {  600, 4135,   78211, 11, }, // Asia/Novosibirsk
    {  600, 4214,   78222,  9, }, // Asia/Pyongyang
    {  600, 5559,    4332,  5, }, // Asia/Qatar
    {  600, 3584,    7787,  4, }, // Asia/Riyadh
    {  600, 4408,   58084,  8, }, // Asia/Sakhalin
    {  600, 3300,   20900,  8, }, // Asia/Singapore
    {  600, 3263,   78231,  6, }, // Asia/Taipei
    {  600, 4781,   57328,  9, }, // Asia/Tashkent
    {  600, 3915,   78237,  6, }, // Asia/Tbilisi
    {  600, 2772,    7842,  7, }, // Asia/Tehran
    {  600, 2821,    4356,  5, }, // Asia/Tokyo
    {  600, 1021,  178347, 11, }, // Asia/Ulaanbaatar
    {  600, 4710,    4361, 11, }, // Asia/Vladivostok
    {  600, 4828,    7873,  7, }, // Asia/Yakutsk
    {  600, 1235,     477,  6, }, // Asia/Yangon
    {  600, 3883,    4380, 13, }, // Asia/Yekaterinburg
    {  600, 3747,  128099,  6, }, // Asia/Yerevan
    {  600, 3663,    7905,  5, }, // Atlantic/Azores
    {  600, 5671,   74483,  8, }, // Atlantic/Bermuda
    {  600, 6628,   78243,  6, }, // Atlantic/Canary
    {  600, 3727,   57380,  9, }, // Atlantic/Cape_Verde
    {  600, 1395,   78250,  6, }, // Atlantic/Faroe
    {  600, 7272,  178358, 12, }, // Atlantic/South_Georgia
    {  600, 6821,     488, 10, }, // Atlantic/St_Helena
    {  600, 2300,    4438,  5, }, // Europe/Athens
    {  600, 2167,   22766,  7, }, // Europe/Brussels
    {  600, 3928,  178370, 10, }, // Europe/Bucharest
    {  600, 2583,  128130,  9, }, // Europe/Chisinau
    {  600, 6289,  178380,  8, }, // Europe/Copenhagen
    {  600, 2527,   78292,  4, }, // Europe/Kyiv
    {  600, 3213,    4499,  8, }, // Europe/Lisbon
    {  600, 6365,    4513,  9, }, // Europe/Luxembourg
    {  600, 3537,   57485,  6, }, // Europe/Moscow
    {  600, 6324,    6064,  3, }, // Europe/Rome
    {  600, 6202,   21213,  6, }, // Europe/Tirane
    {  600, 6231,   57515,  4, }, // Europe/Vienna
    {  600, 8899,  128004,  5, }, // Europe/Vilnius
    {  600, 3190,   57528,  8, }, // Europe/Warsaw
    {  600, 6540,    4577,  4, }, // Europe/Zurich
    {  600, 6046,  178388, 15, }, // Indian/Christmas
    {  600, 5029,    8305,  6, }, // Indian/Comoro
    {  600, 7050,    4594,  8, }, // Indian/Maldives
    {  600, 7211,     547,  7, }, // Indian/Reunion
    {  600, 3157,     554,  5, }, // Pacific/Chuuk
    {  600, 2237,  178403, 12, }, // Pacific/Easter
    {  600, 3902,   78353,  7, }, // Pacific/Fiji
    {  600, 3096,     586,  7, }, // Pacific/Pohnpei
    {  603, 6788,  178415,  5, }, // Africa/Abidjan Syriac/Syriac/Iraq
    {  603, 6760,  178420,  4, }, // Africa/Accra
    {  603, 5059,  178424,  9, }, // Africa/Addis_Ababa
    {  603, 7420,  178433,  5, }, // Africa/Algiers
    {  603,   14,  178438,  6, }, // Africa/Asmara
    {  603,   44,  178444,  6, }, // Africa/Bamako
    {  603, 5257,  178450,  7, }, // Africa/Bangui
    {  603, 6746,  178457,  6, }, // Africa/Banjul
    {  603, 7435,  178463,  5, }, // Africa/Bissau
    {  603, 4957,  178468,  6, }, // Africa/Blantyre
    {  603, 5287,  178474,  8, }, // Africa/Brazzaville
    {  603, 4922,  178482,  9, }, // Africa/Bujumbura
    {  603, 2320,  178491,  6, }, // Africa/Cairo
    {  603, 4117,  178497,  9, }, // Africa/Casablanca
    {  603, 7449,  178506,  4, }, // Africa/Ceuta
    {  603, 6773,  178510,  7, }, // Africa/Conakry
    {  603, 6840,  178517,  5, }, // Africa/Dakar
    {  603, 5130,  178522,  8, }, // Africa/Dar_es_Salaam
    {  603, 5043,  178530,  6, }, // Africa/Djibouti
    {  603, 5243,  178536,  5, }, // Africa/Douala
    {  603, 5166,  178541,  7, }, // Africa/El_Aaiun
    {  603, 6853,  178548,  7, }, // Africa/Freetown
    {  603, 4906,  178555,  6, }, // Africa/Gaborone
    {  603, 5015,  178561,  5, }, // Africa/Harare
    {  603, 4479,  178566, 10, }, // Africa/Johannesburg
    {  603, 4499,  178576,  4, }, // Africa/Juba
    {  603, 5151,  178580,  7, }, // Africa/Kampala
    {  603, 4524,  178587,  5, }, // Africa/Khartoum
    {  603, 4973,  178592,  6, }, // Africa/Kigali
    {  603, 5306,  178598,  7, }, // Africa/Kinshasa
    {  603, 4744,  178605,  5, }, // Africa/Lagos
    {  603, 5336,  178610,  8, }, // Africa/Libreville
    {  603, 6869,  178618,  4, }, // Africa/Lome
    {  603, 5211,  178622,  6, }, // Africa/Luanda
    {  603, 4939,  178628,  8, }, // Africa/Lubumbashi
    {  603, 5001,  178636,  6, }, // Africa/Lusaka
    {  603, 5322,  178642,  6, }, // Africa/Malabo
    {  603, 4987,  178648,  6, }, // Africa/Maputo
    {  603, 5197,  178654,  6, }, // Africa/Maseru
    {  603, 5182,  178660,  7, }, // Africa/Mbabane
    {  603, 5113,  178667,  7, }, // Africa/Mogadishu
    {  603, 7034,  178674,  8, }, // Africa/Monrovia
    {  603, 3853,  178682,  6, }, // Africa/Nairobi
    {  603, 5271,  178688,  6, }, // Africa/Ndjamena
    {  603, 5354,  178694,  5, }, // Africa/Niamey
    {  603, 6803,  178699,  7, }, // Africa/Nouakchott
    {  603, 6727,  178706, 11, }, // Africa/Ouagadougou
    {  603, 5225,  178717, 10, }, // Africa/Porto-Novo
    {  603, 4435,  178727,  8, }, // Africa/Sao_Tome
    {  603, 2866,  178735,  8, }, // Africa/Tripoli
    {  603, 6554,  178743,  4, }, // Africa/Tunis
    {  603, 4152,  178747,  7, }, // Africa/Windhoek
    {  603,  132,  178754,  4, }, // America/Adak
    {  603, 3346,  178758,  6, }, // America/Anchorage
    {  603, 5607,  178764,  6, }, // America/Anguilla
    {  603, 5624,  178770,  7, }, // America/Antigua
    {  603, 4554,  178777,  9, }, // America/Araguaina
    {  603,  166,  178786, 12, }, // America/Argentina/Buenos_Aires
    {  603,   91,  178798,  8, }, // America/Argentina/Catamarca
    {  603,  270,  178806,  7, }, // America/Argentina/Cordoba
    {  603,  441,  178813,  5, }, // America/Argentina/Jujuy
    {  603, 7462,  178818,  8, }, // America/Argentina/La_Rioja
    {  603,  565,  178826,  6, }, // America/Argentina/Mendoza
    {  603, 7489,  178832,  9, }, // America/Argentina/Rio_Gallegos
    {  603, 7520,  178841,  4, }, // America/Argentina/Salta
    {  603, 7544,  178845,  7, }, // America/Argentina/San_Juan
    {  603, 5580,  178852,  8, }, // America/Argentina/San_Luis
    {  603, 7571,  178860,  7, }, // America/Argentina/Tucuman
    {  603, 7597,  178867,  7, }, // America/Argentina/Ushuaia
    {  603, 5640,  178874,  5, }, // America/Aruba
    {  603, 4252,  178879,  9, }, // America/Asuncion
    {  603,  237,  178888,  8, }, // America/Atikokan
    {  603, 3679,  178896,  5, }, // America/Bahia
    {  603, 7623,  178901, 12, }, // America/Bahia_Banderas
    {  603, 5654,  178913,  7, }, // America/Barbados
    {  603, 7646,  178920,  5, }, // America/Belem
    {  603, 5407,  178925,  5, }, // America/Belize
    {  603, 7660,  178930, 12, }, // America/Blanc-Sablon
    {  603, 7681,  178942,  9, }, // America/Boa_Vista
    {  603, 4361,  178951,  6, }, // America/Bogota
    {  603, 7699,  178957,  5, }, // America/Boise
    {  603, 7713,  178962,  9, }, // America/Cambridge_Bay
    {  603, 7735,  178971, 11, }, // America/Campo_Grande
    {  603, 3868,  178982,  6, }, // America/Cancun
    {  603, 4694,  178988,  6, }, // America/Caracas
    {  603, 4345,  178994,  5, }, // America/Cayenne
    {  603, 5496,  178999,  6, }, // America/Cayman
    {  603, 2260,  179005,  4, }, // America/Chicago
    {  603, 7756,  179009,  6, }, // America/Chihuahua
    {  603, 7774,  179015, 12, }, // America/Ciudad_Juarez
    {  603, 5422,  179027, 10, }, // America/Costa_Rica
    {  603, 7796,  179037,  7, }, // America/Creston
    {  603, 3791,  179044,  6, }, // America/Cuiaba
    {  603, 5723,  179050,  7, }, // America/Curacao
    {  603, 7812,  179057, 10, }, // America/Danmarkshavn
    {  603, 7833,  179067,  5, }, // America/Dawson
    {  603, 7848,  179072, 10, }, // America/Dawson_Creek
    {  603,  805,  179082,  4, }, // America/Denver
    {  603, 3465,  179086,  7, }, // America/Detroit
    {  603, 5739,  179093,  8, }, // America/Dominica
    {  603,  893,  179101,  8, }, // America/Edmonton
    {  603, 7869,  179109,  8, }, // America/Eirunepe
    {  603, 5441,  179117, 10, }, // America/El_Salvador
    {  603, 7886,  179127, 11, }, // America/Fort_Nelson
    {  603, 7906,  179138,  9, }, // America/Fortaleza
    {  603, 7924,  179147,  8, }, // America/Glace_Bay
    {  603, 6881,  179155,  7, }, // America/Goose_Bay
    {  603, 4612,  179162, 10, }, // America/Grand_Turk
    {  603, 5770,  179172,  6, }, // America/Grenada
    {  603, 5786,  179178,  9, }, // America/Guadeloupe
    {  603, 3760,  179187,  8, }, // America/Guatemala
    {  603, 4841,  179195,  9, }, // America/Guayaquil
    {  603, 6932,  179204,  6, }, // America/Guyana
    {  603, 1939,  179210,  7, }, // America/Halifax
    {  603, 2281,  179217,  6, }, // America/Havana
    {  603, 7942,  179223,  8, }, // America/Hermosillo
    {  603,  348,  179231, 12, }, // America/Indiana/Indianapolis
    {  603,  481,  179243, 13, }, // America/Indiana/Knox
    {  603, 7961,  179256, 15, }, // America/Indiana/Marengo
    {  603, 7985,  179271, 19, }, // America/Indiana/Petersburg
    {  603, 8012,  179290, 20, }, // America/Indiana/Tell_City
    {  603, 8038,  179310, 14, }, // America/Indiana/Vevay
    {  603, 8060,  179324, 17, }, // America/Indiana/Vincennes
    {  603, 8086,  179341, 16, }, // America/Indiana/Winamac
    {  603, 8110,  179357,  6, }, // America/Inuvik
    {  603,  660,  179363,  7, }, // America/Iqaluit
    {  603, 2799,  179370,  6, }, // America/Jamaica
    {  603, 5380,  179376,  4, }, // America/Juneau
    {  603,  521,  179380,  7, }, // America/Kentucky/Louisville
    {  603, 8125,  179387, 18, }, // America/Kentucky/Monticello
    {  603, 5704,  179405,  7, }, // America/Kralendijk
    {  603, 4376,  179412,  6, }, // America/La_Paz
    {  603, 7169,  179418,  4, }, // America/Lima
    {  603, 3239,  179422, 10, }, // America/Los_Angeles
    {  603, 5932,  179432, 19, }, // America/Lower_Princes
    {  603, 8153,  179451,  6, }, // America/Maceio
    {  603, 8168,  179457,  7, }, // America/Managua
    {  603, 1908,  179464,  6, }, // America/Manaus
    {  603, 5897,  179470,  6, }, // America/Marigot
    {  603, 5805,  179476,  8, }, // America/Martinique
    {  603, 8184,  179484,  9, }, // America/Matamoros
    {  603, 2917,  179493,  6, }, // America/Mazatlan
    {  603, 8202,  179499,  7, }, // America/Menominee
    {  603, 8220,  179506,  5, }, // America/Merida
    {  603, 8235,  179511,  8, }, // America/Metlakatla
    {  603, 2949,  179519, 14, }, // America/Mexico_City
    {  603, 4391,  179533,  6, }, // America/Miquelon
    {  603, 8254,  179539,  7, }, // America/Moncton
    {  603, 8270,  179546,  7, }, // America/Monterrey
    {  603, 4098,  179553, 10, }, // America/Montevideo
    {  603, 5824,  179563,  9, }, // America/Montserrat
    {  603, 5481,  179572,  5, }, // America/Nassau
    {  603, 2379,  179577,  8, }, // America/New_York
    {  603, 8288,  179500,  3, }, // America/Nome
    {  603, 1850,  179585,  7, }, // America/Noronha
    {  603, 8301,  179592, 21, }, // America/North_Dakota/Beulah
    {  603, 8329,  179613, 21, }, // America/North_Dakota/Center
    {  603, 8357,  179634, 26, }, // America/North_Dakota/New_Salem
    {  603,  393,  179243,  3, }, // America/Nuuk
    {  603, 8388,  179660,  7, }, // America/Ojinaga
    {  603, 2356,  179667,  4, }, // America/Panama
    {  603, 6173,  179671, 10, }, // America/Paramaribo
    {  603, 2973,  179681,  6, }, // America/Phoenix
    {  603, 3945,  179687, 13, }, // America/Port-au-Prince
    {  603, 5954,  179700, 12, }, // America/Port_of_Spain
    {  603, 8404,  179712, 11, }, // America/Porto_Velho
    {  603, 5843,  179723, 10, }, // America/Puerto_Rico
    {  603, 4030,  179733, 11, }, // America/Punta_Arenas
    {  603, 8424,  179744, 11, }, // America/Rankin_Inlet
    {  603, 8445,  179755,  6, }, // America/Recife
    {  603, 1995,  179761,  6, }, // America/Regina
    {  603, 8460,  179767,  7, }, // America/Resolute
    {  603,  695,  179774,  9, }, // America/Rio_Branco
    {  603, 8477,  179783,  7, }, // America/Santarem
    {  603, 2201,  179790,  8, }, // America/Santiago
    {  603, 6111,  179798, 12, }, // America/Santo_Domingo
    {  603, 1878,  179810,  9, }, // America/Sao_Paulo
    {  603, 6899,  179819, 10, }, // America/Scoresbysund
    {  603, 8494,  179829,  5, }, // America/Sitka
    {  603, 8508,  179834, 11, }, // America/St_Barthelemy
    {  603, 2061,  179845,  8, }, // America/St_Johns
    {  603, 5863,  179853,  8, }, // America/St_Kitts
    {  603, 5880,  179861,  9, }, // America/St_Lucia
    {  603,  855,  179870,  8, }, // America/St_Thomas
    {  603, 5913,  179878,  8, }, // America/St_Vincent
    {  603, 8530,  179886,  6, }, // America/Swift_Current
    {  603, 5461,  179892, 11, }, // America/Tegucigalpa
    {  603, 5756,  179840,  3, }, // America/Thule
    {  603,  313,  179903,  7, }, // America/Tijuana
    {  603,  608,  179910,  7, }, // America/Toronto
    {  603, 5688,  179917,  7, }, // America/Tortola
    {  603, 2093,  179924,  7, }, // America/Vancouver
    {  603, 2144,  179931, 11, }, // America/Whitehorse
    {  603,  734,  179942,  6, }, // America/Winnipeg
    {  603, 8552,  179948,  6, }, // America/Yakutat
    {  603, 6001,  179954,  5, }, // Antarctica/Casey
    {  603, 6094,  179959,  4, }, // Antarctica/Davis
    {  603, 6133,  179963, 14, }, // Antarctica/DumontDUrville
    {  603, 8568,  179977,  7, }, // Antarctica/Macquarie
    {  603, 7066,  179984,  6, }, // Antarctica/Mawson
    {  603, 7113,  179990,  7, }, // Antarctica/McMurdo
    {  603, 8589,  179997,  6, }, // Antarctica/Palmer
    {  603, 7226,  180003,  6, }, // Antarctica/Rothera
    {  603, 7295,  180009,  4, }, // Antarctica/Syowa
    {  603, 8607,  180013,  4, }, // Antarctica/Troll
    {  603, 7374,  180017,  6, }, // Antarctica/Vostok
    {  603, 1429,  180023, 11, }, // Arctic/Longyearbyen
    {  603, 5570,  180034,  3, }, // Asia/Aden
    {  603, 5395,  180037,  6, }, // Asia/Almaty
    {  603, 3968,  180043,  4, }, // Asia/Amman
    {  603, 5511,  180047,  6, }, // Asia/Anadyr
    {  603, 5523,  180053,  5, }, // Asia/Aqtau
    {  603, 5368,  180058,  5, }, // Asia/Aqtobe
    {  603,  964,  180063,  7, }, // Asia/Ashgabat
    {  603, 8624,  180070,  5, }, // Asia/Atyrau
    {  603, 3607,  180075,  4, }, // Asia/Baghdad
    {  603, 5534,  180079,  5, }, // Asia/Bahrain
    {  603, 3653,  179048,  4, }, // Asia/Baku
    {  603, 4466,  180084,  6, }, // Asia/Bangkok
    {  603, 3571,  180090,  7, }, // Asia/Barnaul
    {  603, 4086,  180097,  5, }, // Asia/Beirut
    {  603, 3778,  180102,  6, }, // Asia/Bishkek
    {  603, 5989,  180108,  6, }, // Asia/Brunei
    {  603, 4601,  180114,  5, }, // Asia/Chita
    {  603, 4511,  180119,  7, }, // Asia/Colombo
    {  603, 4540,  180126,  6, }, // Asia/Damascus
    {  603, 1093,  180132,  3, }, // Asia/Dhaka
    {  603, 6192,  180135,  4, }, // Asia/Dili
    {  603, 3596,  180139,  5, }, // Asia/Dubai
    {  603, 6159,  180144,  7, }, // Asia/Dushanbe
    {  603, 8636,  180151,  9, }, // Asia/Famagusta
    {  603, 8651,  180160,  3, }, // Asia/Gaza
    {  603, 4795,  180163,  5, }, // Asia/Hebron
    {  603, 1259,  180168, 17, }, // Asia/Ho_Chi_Minh
    {  603, 2704,  180185,  9, }, // Asia/Hong_Kong
    {  603, 4771,  180194,  4, }, // Asia/Hovd
    {  603, 4184,  180198,  8, }, // Asia/Irkutsk
    {  603, 7006,  180206,  7, }, // Asia/Jakarta
    {  603, 6992,  180213,  7, }, // Asia/Jayapura
    {  603, 1290,  180220,  6, }, // Asia/Jerusalem
    {  603, 3560,  180226,  5, }, // Asia/Kabul
    {  603, 4316,  180231,  9, }, // Asia/Kamchatka
    {  603, 4239,  180240,  5, }, // Asia/Karachi
    {  603, 1185,  180245,  8, }, // Asia/Kathmandu
    {  603, 8661,  180253,  7, }, // Asia/Khandyga
    {  603,  992,  180260,  6, }, // Asia/Kolkata
    {  603, 4197,  180266, 10, }, // Asia/Krasnoyarsk
    {  603, 4859,  180276, 12, }, // Asia/Kuala_Lumpur
    {  603, 5976,  180288,  6, }, // Asia/Kuching
    {  603, 5547,  180294,  4, }, // Asia/Kuwait
    {  603, 1211,  179977,  5, }, // Asia/Macau
    {  603, 4017,  180298,  7, }, // Asia/Magadan
    {  603, 1349,  180305,  7, }, // Asia/Makassar
    {  603, 7182,  180312,  6, }, // Asia/Manila
    {  603, 6920,  180318,  4, }, // Asia/Muscat
    {  603, 2554,  180322,  7, }, // Asia/Nicosia
    {  603, 8675,  180329, 12, }, // Asia/Novokuznetsk
    {  603, 4135,  180341, 10, }, // Asia/Novosibirsk
    {  603, 4229,  180351,  5, }, // Asia/Omsk
    {  603, 7145,  180356,  5, }, // Asia/Oral
    {  603, 6961,  180361,  7, }, // Asia/Phnom_Penh
    {  603, 8693,  180368,  9, }, // Asia/Pontianak
    {  603, 4214,  180377,  8, }, // Asia/Pyongyang
    {  603, 5559,  180385,  3, }, // Asia/Qatar
    {  603, 8708,  180388,  8, }, // Asia/Qostanay
    {  603, 4269,  180396,  9, }, // Asia/Qyzylorda
    {  603, 3584,  180405,  4, }, // Asia/Riyadh
    {  603, 4408,  180409,  7, }, // Asia/Sakhalin
    {  603, 7245,  180416,  6, }, // Asia/Samarkand
    {  603, 3279,  180422,  4, }, // Asia/Seoul
    {  603, 1053,  180426,  7, }, // Asia/Shanghai
    {  603, 3300,  180433,  8, }, // Asia/Singapore
    {  603, 4297,  180441, 13, }, // Asia/Srednekolymsk
    {  603, 3263,  180454,  6, }, // Asia/Taipei
    {  603, 4781,  180460,  6, }, // Asia/Tashkent
    {  603, 3915,  180466,  5, }, // Asia/Tbilisi
    {  603, 2772,  180471,  4, }, // Asia/Tehran
    {  603, 1317,  180475,  5, }, // Asia/Thimphu
    {  603, 2821,  180480,  5, }, // Asia/Tokyo
    {  603, 4572,  180485,  5, }, // Asia/Tomsk
    {  603, 1021,  180490, 10, }, // Asia/Ulaanbaatar
    {  603, 1159,  180500,  7, }, // Asia/Urumqi
    {  603, 8722,  180507,  9, }, // Asia/Ust-Nera
    {  603, 6977,  180516,  8, }, // Asia/Vientiane
    {  603, 4710,  180524, 11, }, // Asia/Vladivostok
    {  603, 4828,  180535,  7, }, // Asia/Yakutsk
    {  603, 1235,  180542,  5, }, // Asia/Yangon
    {  603, 3883,  180547, 11, }, // Asia/Yekaterinburg
    {  603, 3747,  180558,  5, }, // Asia/Yerevan
    {  603, 3663,  180563, 11, }, // Atlantic/Azores
    {  603, 5671,  180574,  8, }, // Atlantic/Bermuda
    {  603, 6628,  180582,  5, }, // Atlantic/Canary
    {  603, 3727,  180587,  9, }, // Atlantic/Cape_Verde
    {  603, 1395,  180596,  4, }, // Atlantic/Faroe
    {  603, 8736,  180600,  6, }, // Atlantic/Madeira
    {  603, 2748,  180606,  7, }, // Atlantic/Reykjavik
    {  603, 7272,  180613, 11, }, // Atlantic/South_Georgia
    {  603, 6821,  180624, 11, }, // Atlantic/St_Helena
    {  603, 6644,  180635,  6, }, // Atlantic/Stanley
    {  603, 1670,  180134,  6, }, // Australia/Adelaide
    {  603, 1635,  180641,  7, }, // Australia/Brisbane
    {  603, 1799,  180648, 10, }, // Australia/Broken_Hill
    {  603, 1583,  180658,  5, }, // Australia/Darwin
    {  603, 3637,  180663,  5, }, // Australia/Eucla
    {  603, 1516,  180668,  5, }, // Australia/Hobart
    {  603, 8753,  180673,  7, }, // Australia/Lindeman
    {  603, 1547,  180680,  8, }, // Australia/Lord_Howe
    {  603, 1727,  180688,  7, }, // Australia/Melbourne
    {  603, 1762,  180695,  4, }, // Australia/Perth
    {  603, 1463,  180699,  5, }, // Australia/Sydney
    {  603, 6441,  180704,  7, }, // Europe/Amsterdam
    {  603, 6216,  180711,  6, }, // Europe/Andorra
    {  603, 3620,  180717,  7, }, // Europe/Astrakhan
    {  603, 2300,  180724,  5, }, // Europe/Athens
    {  603, 6336,  180729,  5, }, // Europe/Belgrade
    {  603, 4757,  180734,  5, }, // Europe/Berlin
    {  603, 6488,  180739,  9, }, // Europe/Bratislava
    {  603, 2167,  180748,  6, }, // Europe/Brussels
    {  603, 3928,  180754,  7, }, // Europe/Bucharest
    {  603, 3806,  180761,  6, }, // Europe/Budapest
    {  603, 8772,  180767,  7, }, // Europe/Busingen
    {  603, 2583,  180774,  7, }, // Europe/Chisinau
    {  603, 6289,  180781,  8, }, // Europe/Copenhagen
    {  603, 2338,  180789,  4, }, // Europe/Dublin
    {  603, 6307,  180793,  6, }, // Europe/Gibraltar
    {  603, 8788,  180799,  6, }, // Europe/Guernsey
    {  603, 6612,  180805,  6, }, // Europe/Helsinki
    {  603, 8804,  180811, 10, }, // Europe/Isle_of_Man
    {  603, 1130,  180821,  7, }, // Europe/Istanbul
    {  603, 8823,  180828,  5, }, // Europe/Jersey
    {  603, 3979,  180833, 11, }, // Europe/Kaliningrad
    {  603, 8929,  180844,  5, }, // Europe/Kirov
    {  603, 2527,  180849,  4, }, // Europe/Kyiv
    {  603, 3213,  180853,  6, }, // Europe/Lisbon
    {  603, 6506,  180859,  9, }, // Europe/Ljubljana
    {  603, 2501,  180868,  6, }, // Europe/London
    {  603, 6365,  180874,  9, }, // Europe/Luxembourg
    {  603, 4892,  180883,  5, }, // Europe/Madrid
    {  603, 6397,  180888,  5, }, // Europe/Malta
    {  603, 6582,  180893,  7, }, // Europe/Mariehamn
    {  603, 3693,  180900,  5, }, // Europe/Minsk
    {  603, 6410,  180905,  6, }, // Europe/Monaco
    {  603, 3537,  180911,  5, }, // Europe/Moscow
    {  603, 6458,  180916,  5, }, // Europe/Oslo
    {  603, 4284,  180921,  5, }, // Europe/Paris
    {  603, 6424,  180926,  9, }, // Europe/Podgorica
    {  603, 6275,  180935,  4, }, // Europe/Prague
    {  603, 8837,  180939,  4, }, // Europe/Riga
    {  603, 6324,  180943,  4, }, // Europe/Rome
    {  603, 4331,  178439,  5, }, // Europe/Samara
    {  603, 6470,  180947,  8, }, // Europe/San_Marino
    {  603, 6245,  180955,  7, }, // Europe/Sarajevo
    {  603, 4451,  180962,  6, }, // Europe/Saratov
    {  603, 8849,  180968,  7, }, // Europe/Simferopol
    {  603, 6383,  180975,  6, }, // Europe/Skopje
    {  603, 6599,  180981,  5, }, // Europe/Sofia
    {  603, 6523,  180986,  8, }, // Europe/Stockholm
    {  603, 8867,  180994,  5, }, // Europe/Tallinn
    {  603, 6202,  180999,  5, }, // Europe/Tirane
    {  603, 8882,  181004,  9, }, // Europe/Ulyanovsk
    {  603, 6352,  181013,  4, }, // Europe/Vaduz
    {  603, 6567,  181017,  7, }, // Europe/Vatican
    {  603, 6231,  181024,  5, }, // Europe/Vienna
    {  603, 8899,  181029,  5, }, // Europe/Vilnius
    {  603, 4727,  181034,  9, }, // Europe/Volgograd
    {  603, 3190,  181043,  5, }, // Europe/Warsaw
    {  603, 6261,  181048,  5, }, // Europe/Zagreb
    {  603, 6540,  181053,  5, }, // Europe/Zurich
    {  603, 5078,  181058, 11, }, // Indian/Antananarivo
    {  603, 6947,  181069,  5, }, // Indian/Chagos
    {  603, 6046,  181074,  7, }, // Indian/Christmas
    {  603, 6063,  181081,  5, }, // Indian/Cocos
    {  603, 5029,  181086,  6, }, // Indian/Comoro
    {  603, 6661,  181092,  8, }, // Indian/Kerguelen
    {  603, 7260,  181100,  4, }, // Indian/Mahe
    {  603, 7050,  181104, 15, }, // Indian/Maldives
    {  603, 4069,  181119,  7, }, // Indian/Mauritius
    {  603, 5098,  181126,  5, }, // Indian/Mayotte
    {  603, 7211,  181131,  7, }, // Indian/Reunion
    {  603, 4422,  181138,  4, }, // Pacific/Apia
    {  603,  932,  181142,  6, }, // Pacific/Auckland
    {  603, 3706,  181148,  9, }, // Pacific/Bougainville
    {  603, 3015,  181157,  5, }, // Pacific/Chatham
    {  603, 3157,  181162,  4, }, // Pacific/Chuuk
    {  603, 2237,  181166,  4, }, // Pacific/Easter
    {  603, 7360,  181170,  4, }, // Pacific/Efate
    {  603, 7327,  181174,  7, }, // Pacific/Fakaofo
    {  603, 3902,  181181,  4, }, // Pacific/Fiji
    {  603, 7343,  181185,  8, }, // Pacific/Funafuti
    {  603, 6678,  181193,  8, }, // Pacific/Galapagos
    {  603, 6696,  181201,  5, }, // Pacific/Gambier
    {  603, 3822,  181206, 11, }, // Pacific/Guadalcanal
    {  603, 6018,  179461,  4, }, // Pacific/Guam
    {  603, 3049,  181217,  6, }, // Pacific/Kanton
    {  603, 3998,  181223,  8, }, // Pacific/Kiritimati
    {  603, 7019,  181231,  6, }, // Pacific/Kosrae
    {  603, 2842,  181237,  8, }, // Pacific/Kwajalein
    {  603, 4877,  181245,  6, }, // Pacific/Majuro
    {  603, 4051,  181251,  9, }, // Pacific/Marquesas
    {  603, 8914,  181260,  6, }, // Pacific/Midway
    {  603, 7084,  181266,  5, }, // Pacific/Nauru
    {  603, 7132,  181271,  4, }, // Pacific/Niue
    {  603, 4168,  181275,  6, }, // Pacific/Norfolk
    {  603, 7098,  181281,  5, }, // Pacific/Noumea
    {  603, 3126,  181286,  9, }, // Pacific/Pago_Pago
    {  603, 7155,  181295,  4, }, // Pacific/Palau
    {  603, 7194,  181299,  8, }, // Pacific/Pitcairn
    {  603, 3096,  181307,  7, }, // Pacific/Pohnpei
    {  603, 4807,  181314, 13, }, // Pacific/Port_Moresby
    {  603, 6076,  181327,  9, }, // Pacific/Rarotonga
    {  603, 6031,  181336,  6, }, // Pacific/Saipan
    {  603, 7312,  181342,  6, }, // Pacific/Tahiti
    {  603, 6712,  181348,  6, }, // Pacific/Tarawa
    {  603, 4583,  181354,  9, }, // Pacific/Tongatapu
    {  603, 7392,  181363,  4, }, // Pacific/Wake
    {  603, 7405,  181367,  5, }, // Pacific/Wallis
    {  605,   14,       0,  6, }, // Africa/Asmara Tachelhit/Tifinagh/Morocco
    {  605, 4435,       6,  8, }, // Africa/Sao_Tome
    {  605, 4252,      14,  8, }, // America/Asuncion
    {  605,  237,      22,  8, }, // America/Atikokan
    {  605, 7623,      30, 17, }, // America/Bahia_Banderas
    {  605, 3868,      47,  6, }, // America/Cancun
    {  605, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  605, 5723,      66,  7, }, // America/Curacao
    {  605,  481,      73, 13, }, // America/Indiana/Knox
    {  605, 7961,      86, 16, }, // America/Indiana/Marengo
    {  605, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  605, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  605, 8038,     139, 14, }, // America/Indiana/Vevay
    {  605, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  605, 8086,     171, 16, }, // America/Indiana/Winamac
    {  605, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  605, 5932,     207, 22, }, // America/Lower_Princes
    {  605, 8220,     229,  6, }, // America/Merida
    {  605, 1850,     235, 19, }, // America/Noronha
    {  605, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  605, 8329,     274, 20, }, // America/North_Dakota/Center
    {  605, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  605,  393,     317,  4, }, // America/Nuuk
    {  605, 6899,     321, 16, }, // America/Scoresbysund
    {  605, 8508,     337, 14, }, // America/St_Barthelemy
    {  605, 2061,     351, 10, }, // America/St_Johns
    {  605, 5863,     361,  9, }, // America/St_Kitts
    {  605, 5880,     370,  9, }, // America/St_Lucia
    {  605,  855,     379, 10, }, // America/St_Thomas
    {  605, 5913,     389, 11, }, // America/St_Vincent
    {  605, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  605, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  605, 1185,     448,  9, }, // Asia/Kathmandu
    {  605,  992,     457,  7, }, // Asia/Kolkata
    {  605, 1211,     464,  5, }, // Asia/Macau
    {  605, 1235,     477,  6, }, // Asia/Yangon
    {  605, 1395,     483,  5, }, // Atlantic/Faroe
    {  605, 6821,     488, 10, }, // Atlantic/St_Helena
    {  605, 2527,     514,  4, }, // Europe/Kyiv
    {  605, 7211,     547,  7, }, // Indian/Reunion
    {  605, 3157,     554,  5, }, // Pacific/Chuuk
    {  605, 3096,     586,  7, }, // Pacific/Pohnpei
    {  607,   14,       0,  6, }, // Africa/Asmara Tai Dam/Tai Viet/Vietnam
    {  607, 4435,       6,  8, }, // Africa/Sao_Tome
    {  607, 4252,      14,  8, }, // America/Asuncion
    {  607,  237,      22,  8, }, // America/Atikokan
    {  607, 7623,      30, 17, }, // America/Bahia_Banderas
    {  607, 3868,      47,  6, }, // America/Cancun
    {  607, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  607, 5723,      66,  7, }, // America/Curacao
    {  607,  481,      73, 13, }, // America/Indiana/Knox
    {  607, 7961,      86, 16, }, // America/Indiana/Marengo
    {  607, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  607, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  607, 8038,     139, 14, }, // America/Indiana/Vevay
    {  607, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  607, 8086,     171, 16, }, // America/Indiana/Winamac
    {  607, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  607, 5932,     207, 22, }, // America/Lower_Princes
    {  607, 8220,     229,  6, }, // America/Merida
    {  607, 1850,     235, 19, }, // America/Noronha
    {  607, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  607, 8329,     274, 20, }, // America/North_Dakota/Center
    {  607, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  607,  393,     317,  4, }, // America/Nuuk
    {  607, 6899,     321, 16, }, // America/Scoresbysund
    {  607, 8508,     337, 14, }, // America/St_Barthelemy
    {  607, 2061,     351, 10, }, // America/St_Johns
    {  607, 5863,     361,  9, }, // America/St_Kitts
    {  607, 5880,     370,  9, }, // America/St_Lucia
    {  607,  855,     379, 10, }, // America/St_Thomas
    {  607, 5913,     389, 11, }, // America/St_Vincent
    {  607, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  607, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  607, 1185,     448,  9, }, // Asia/Kathmandu
    {  607,  992,     457,  7, }, // Asia/Kolkata
    {  607, 1211,     464,  5, }, // Asia/Macau
    {  607, 1235,     477,  6, }, // Asia/Yangon
    {  607, 1395,     483,  5, }, // Atlantic/Faroe
    {  607, 6821,     488, 10, }, // Atlantic/St_Helena
    {  607, 2527,     514,  4, }, // Europe/Kyiv
    {  607, 7211,     547,  7, }, // Indian/Reunion
    {  607, 3157,     554,  5, }, // Pacific/Chuuk
    {  607, 3096,     586,  7, }, // Pacific/Pohnpei
    {  608,   14,       0,  6, }, // Africa/Asmara Taita/Latin/Kenya
    {  608, 4435,       6,  8, }, // Africa/Sao_Tome
    {  608, 4252,      14,  8, }, // America/Asuncion
    {  608,  237,      22,  8, }, // America/Atikokan
    {  608, 7623,      30, 17, }, // America/Bahia_Banderas
    {  608, 3868,      47,  6, }, // America/Cancun
    {  608, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  608, 5723,      66,  7, }, // America/Curacao
    {  608,  481,      73, 13, }, // America/Indiana/Knox
    {  608, 7961,      86, 16, }, // America/Indiana/Marengo
    {  608, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  608, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  608, 8038,     139, 14, }, // America/Indiana/Vevay
    {  608, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  608, 8086,     171, 16, }, // America/Indiana/Winamac
    {  608, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  608, 5932,     207, 22, }, // America/Lower_Princes
    {  608, 8220,     229,  6, }, // America/Merida
    {  608, 1850,     235, 19, }, // America/Noronha
    {  608, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  608, 8329,     274, 20, }, // America/North_Dakota/Center
    {  608, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  608,  393,     317,  4, }, // America/Nuuk
    {  608, 6899,     321, 16, }, // America/Scoresbysund
    {  608, 8508,     337, 14, }, // America/St_Barthelemy
    {  608, 2061,     351, 10, }, // America/St_Johns
    {  608, 5863,     361,  9, }, // America/St_Kitts
    {  608, 5880,     370,  9, }, // America/St_Lucia
    {  608,  855,     379, 10, }, // America/St_Thomas
    {  608, 5913,     389, 11, }, // America/St_Vincent
    {  608, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  608, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  608, 1185,     448,  9, }, // Asia/Kathmandu
    {  608,  992,     457,  7, }, // Asia/Kolkata
    {  608, 1211,     464,  5, }, // Asia/Macau
    {  608, 1235,     477,  6, }, // Asia/Yangon
    {  608, 1395,     483,  5, }, // Atlantic/Faroe
    {  608, 6821,     488, 10, }, // Atlantic/St_Helena
    {  608, 2527,     514,  4, }, // Europe/Kyiv
    {  608, 7211,     547,  7, }, // Indian/Reunion
    {  608, 3157,     554,  5, }, // Pacific/Chuuk
    {  608, 3096,     586,  7, }, // Pacific/Pohnpei
    {  609, 6788,  181372,  7, }, // Africa/Abidjan Tajik/Cyrillic/Tajikistan
    {  609, 6760,     610,  5, }, // Africa/Accra
    {  609, 5059,     615, 11, }, // Africa/Addis_Ababa
    {  609, 7420,  181379,  8, }, // Africa/Algiers
    {  609,   14,   27197,  6, }, // Africa/Asmara
    {  609,   44,     637,  6, }, // Africa/Bamako
    {  609, 5257,   33856,  6, }, // Africa/Bangui
    {  609, 6746,  181387,  6, }, // Africa/Banjul
    {  609, 7435,     654,  5, }, // Africa/Bissau
    {  609, 4957,   27220,  8, }, // Africa/Blantyre
    {  609, 5287,     667,  9, }, // Africa/Brazzaville
    {  609, 4922,  181393,  9, }, // Africa/Bujumbura
    {  609, 2320,  181402,  6, }, // Africa/Cairo
    {  609, 4117,     689, 10, }, // Africa/Casablanca
    {  609, 7449,     699,  5, }, // Africa/Ceuta
    {  609, 6773,     704,  7, }, // Africa/Conakry
    {  609, 6840,     711,  5, }, // Africa/Dakar
    {  609, 5130,  181408, 10, }, // Africa/Dar_es_Salaam
    {  609, 5043,  181418,  6, }, // Africa/Djibouti
    {  609, 5243,     734,  5, }, // Africa/Douala
    {  609, 5166,  181424,  7, }, // Africa/El_Aaiun
    {  609, 6853,     746,  7, }, // Africa/Freetown
    {  609, 4906,     753,  8, }, // Africa/Gaborone
    {  609, 5015,   33913,  6, }, // Africa/Harare
    {  609, 4479,   51572, 12, }, // Africa/Johannesburg
    {  609, 4499,  181431,  4, }, // Africa/Juba
    {  609, 5151,     783,  7, }, // Africa/Kampala
    {  609, 4524,     790,  6, }, // Africa/Khartoum
    {  609, 4973,     796,  6, }, // Africa/Kigali
    {  609, 5306,     802,  7, }, // Africa/Kinshasa
    {  609, 4744,     809,  5, }, // Africa/Lagos
    {  609, 5336,   51584,  9, }, // Africa/Libreville
    {  609, 6869,     822,  4, }, // Africa/Lome
    {  609, 5211,     826,  6, }, // Africa/Luanda
    {  609, 4939,     832,  9, }, // Africa/Lubumbashi
    {  609, 5001,     841,  6, }, // Africa/Lusaka
    {  609, 5322,     847,  6, }, // Africa/Malabo
    {  609, 4987,     853,  6, }, // Africa/Maputo
    {  609, 5197,     859,  6, }, // Africa/Maseru
    {  609, 5182,     865,  7, }, // Africa/Mbabane
    {  609, 5113,     872,  8, }, // Africa/Mogadishu
    {  609, 7034,   36419,  8, }, // Africa/Monrovia
    {  609, 3853,   36427,  7, }, // Africa/Nairobi
    {  609, 5271,  181435,  7, }, // Africa/Ndjamena
    {  609, 5354,   36434,  6, }, // Africa/Niamey
    {  609, 6803,     908,  7, }, // Africa/Nouakchott
    {  609, 6727,     915,  8, }, // Africa/Ouagadougou
    {  609, 5225,     923, 10, }, // Africa/Porto-Novo
    {  609, 4435,     933,  8, }, // Africa/Sao_Tome
    {  609, 2866,     941,  7, }, // Africa/Tripoli
    {  609, 6554,     948,  5, }, // Africa/Tunis
    {  609, 4152,     953,  7, }, // Africa/Windhoek
    {  609,  132,     960,  4, }, // America/Adak
    {  609, 3346,  181442,  7, }, // America/Anchorage
    {  609, 5607,  181449,  6, }, // America/Anguilla
    {  609, 5624,     978,  7, }, // America/Antigua
    {  609, 4554,   36455,  9, }, // America/Araguaina
    {  609,  166,   51608, 12, }, // America/Argentina/Buenos_Aires
    {  609,   91,    1006,  9, }, // America/Argentina/Catamarca
    {  609,  270,   34035,  7, }, // America/Argentina/Cordoba
    {  609,  441,  181455,  5, }, // America/Argentina/Jujuy
    {  609, 7462,   34047,  8, }, // America/Argentina/La_Rioja
    {  609,  565,   34055,  7, }, // America/Argentina/Mendoza
    {  609, 7489,  118178, 12, }, // America/Argentina/Rio_Gallegos
    {  609, 7520,    1053,  5, }, // America/Argentina/Salta
    {  609, 7544,    1058,  8, }, // America/Argentina/San_Juan
    {  609, 5580,   36492,  8, }, // America/Argentina/San_Luis
    {  609, 7571,    1074,  7, }, // America/Argentina/Tucuman
    {  609, 7597,  106260,  6, }, // America/Argentina/Ushuaia
    {  609, 5640,    1087,  5, }, // America/Aruba
    {  609, 4252,   34095,  8, }, // America/Asuncion
    {  609,  237,   34103,  8, }, // America/Atikokan
    {  609, 3679,  181460,  5, }, // America/Bahia
    {  609, 7623,  181465, 17, }, // America/Bahia_Banderas
    {  609, 5654,    1131,  8, }, // America/Barbados
    {  609, 7646,   34130,  5, }, // America/Belem
    {  609, 5407,    1144,  5, }, // America/Belize
    {  609, 7660,   36520, 11, }, // America/Blanc-Sablon
    {  609, 7681,   34154,  9, }, // America/Boa_Vista
    {  609, 4361,    1170,  6, }, // America/Bogota
    {  609, 7699,  181482,  4, }, // America/Boise
    {  609, 7713,  181486, 11, }, // America/Cambridge_Bay
    {  609, 7735,   34179, 12, }, // America/Campo_Grande
    {  609, 3868,    1205,  6, }, // America/Cancun
    {  609, 4694,    1211,  7, }, // America/Caracas
    {  609, 4345,   36549,  5, }, // America/Cayenne
    {  609, 5496,   27629,  6, }, // America/Cayman
    {  609, 2260,    1247,  6, }, // America/Chicago
    {  609, 7756,  118226,  8, }, // America/Chihuahua
    {  609, 7774,  181497, 12, }, // America/Ciudad_Juarez
    {  609, 5422,   36584, 10, }, // America/Costa_Rica
    {  609, 7796,    1269,  7, }, // America/Creston
    {  609, 3791,   27688,  5, }, // America/Cuiaba
    {  609, 5723,   36606,  7, }, // America/Curacao
    {  609, 7812,  137333, 11, }, // America/Danmarkshavn
    {  609, 7833,    1302,  6, }, // America/Dawson
    {  609, 7848,  137344, 11, }, // America/Dawson_Creek
    {  609,  805,    1319,  6, }, // America/Denver
    {  609, 3465,   36649,  7, }, // America/Detroit
    {  609, 5739,    1332,  8, }, // America/Dominica
    {  609,  893,   51681,  8, }, // America/Edmonton
    {  609, 7869,   51689,  8, }, // America/Eirunepe
    {  609, 5441,   27765,  9, }, // America/El_Salvador
    {  609, 7886,    1364, 11, }, // America/Fort_Nelson
    {  609, 7906,    1375,  9, }, // America/Fortaleza
    {  609, 7924,  181509,  9, }, // America/Glace_Bay
    {  609, 6881,  181518,  7, }, // America/Goose_Bay
    {  609, 4612,  181525, 10, }, // America/Grand_Turk
    {  609, 5770,    1410,  7, }, // America/Grenada
    {  609, 5786,    1417,  9, }, // America/Guadeloupe
    {  609, 3760,    1426,  9, }, // America/Guatemala
    {  609, 4841,   36709,  7, }, // America/Guayaquil
    {  609, 6932,   51733,  6, }, // America/Guyana
    {  609, 1939,   51739,  8, }, // America/Halifax
    {  609, 2281,  181535,  6, }, // America/Havana
    {  609, 7942,  118295,  9, }, // America/Hermosillo
    {  609,  348,    1471, 12, }, // America/Indiana/Indianapolis
    {  609,  481,    1483, 13, }, // America/Indiana/Knox
    {  609, 7961,    1496, 16, }, // America/Indiana/Marengo
    {  609, 7985,  181541, 18, }, // America/Indiana/Petersburg
    {  609, 8012,  137411, 17, }, // America/Indiana/Tell_City
    {  609, 8038,  181559, 14, }, // America/Indiana/Vevay
    {  609, 8060,  181573, 16, }, // America/Indiana/Vincennes
    {  609, 8086,  137445, 16, }, // America/Indiana/Winamac
    {  609, 8110,    1570,  6, }, // America/Inuvik
    {  609,  660,    1576,  7, }, // America/Iqaluit
    {  609, 2799,   28011,  6, }, // America/Jamaica
    {  609, 5380,  181589,  4, }, // America/Juneau
    {  609,  521,    1594,  8, }, // America/Kentucky/Louisville
    {  609, 8125,  118364, 20, }, // America/Kentucky/Monticello
    {  609, 5704,  181593,  9, }, // America/Kralendijk
    {  609, 4376,    1632,  6, }, // America/La_Paz
    {  609, 7169,    1638,  4, }, // America/Lima
    {  609, 3239,  181602, 11, }, // America/Los_Angeles
    {  609, 5932,  181613, 20, }, // America/Lower_Princes
    {  609, 8153,   36799,  6, }, // America/Maceio
    {  609, 8168,    1677,  7, }, // America/Managua
    {  609, 1908,    1684,  6, }, // America/Manaus
    {  609, 5897,    1690,  6, }, // America/Marigot
    {  609, 5805,    1696,  9, }, // America/Martinique
    {  609, 8184,    1705,  9, }, // America/Matamoros
    {  609, 2917,   34549,  8, }, // America/Mazatlan
    {  609, 8202,    1722,  7, }, // America/Menominee
    {  609, 8220,    1730,  6, }, // America/Merida
    {  609, 8235,    1736, 10, }, // America/Metlakatla
    {  609, 2949,    1746,  6, }, // America/Mexico_City
    {  609, 4391,    1752,  7, }, // America/Miquelon
    {  609, 8254,    1759,  7, }, // America/Moncton
    {  609, 8270,   51794,  9, }, // America/Monterrey
    {  609, 4098,    1775, 10, }, // America/Montevideo
    {  609, 5824,   51803,  9, }, // America/Montserrat
    {  609, 5481,    1795,  6, }, // America/Nassau
    {  609, 2379,  181633,  7, }, // America/New_York
    {  609, 8288,    1809,  3, }, // America/Nome
    {  609, 1850,   51820,  7, }, // America/Noronha
    {  609, 8301,  181640, 21, }, // America/North_Dakota/Beulah
    {  609, 8329,  181661, 21, }, // America/North_Dakota/Center
    {  609, 8357,  181682, 24, }, // America/North_Dakota/New_Salem
    {  609,  393,    1890,  4, }, // America/Nuuk
    {  609, 8388,  137582,  7, }, // America/Ojinaga
    {  609, 2356,    1901,  6, }, // America/Panama
    {  609, 6173,    1907, 10, }, // America/Paramaribo
    {  609, 2973,    1917,  6, }, // America/Phoenix
    {  609, 3945,    1923, 12, }, // America/Port-au-Prince
    {  609, 5954,  181706, 13, }, // America/Port_of_Spain
    {  609, 8404,  181719, 11, }, // America/Porto_Velho
    {  609, 5843,   51888, 11, }, // America/Puerto_Rico
    {  609, 4030,   34711, 12, }, // America/Punta_Arenas
    {  609, 8424,   34723, 12, }, // America/Rankin_Inlet
    {  609, 8445,    1994,  6, }, // America/Recife
    {  609, 1995,   34741,  6, }, // America/Regina
    {  609, 8460,   36892,  7, }, // America/Resolute
    {  609,  695,   34754, 10, }, // America/Rio_Branco
    {  609, 8477,   34764,  8, }, // America/Santarem
    {  609, 2201,   51907,  8, }, // America/Santiago
    {  609, 6111,   34781, 13, }, // America/Santo_Domingo
    {  609, 1878,    2054,  9, }, // America/Sao_Paulo
    {  609, 6899,  118545, 16, }, // America/Scoresbysund
    {  609, 8494,    2074,  5, }, // America/Sitka
    {  609, 8508,  181730, 14, }, // America/St_Barthelemy
    {  609, 2061,  181744,  9, }, // America/St_Johns
    {  609, 5863,   34836,  9, }, // America/St_Kitts
    {  609, 5880,   56488, 10, }, // America/St_Lucia
    {  609,  855,  128602, 10, }, // America/St_Thomas
    {  609, 5913,   34863, 12, }, // America/St_Vincent
    {  609, 8530,  181753, 13, }, // America/Swift_Current
    {  609, 5461,    2156, 11, }, // America/Tegucigalpa
    {  609, 5756,    2167,  4, }, // America/Thule
    {  609,  313,    2171,  7, }, // America/Tijuana
    {  609,  608,    2178,  7, }, // America/Toronto
    {  609, 5688,    2185,  7, }, // America/Tortola
    {  609, 2093,    2192,  8, }, // America/Vancouver
    {  609, 2144,   28567,  8, }, // America/Whitehorse
    {  609,  734,    2208,  8, }, // America/Winnipeg
    {  609, 8552,   28582,  6, }, // America/Yakutat
    {  609, 6001,   37021,  5, }, // Antarctica/Casey
    {  609, 6094,  118616,  5, }, // Antarctica/Davis
    {  609, 6133,  181766, 14, }, // Antarctica/DumontDUrville
    {  609, 8568,  181780,  8, }, // Antarctica/Macquarie
    {  609, 7066,  181788,  6, }, // Antarctica/Mawson
    {  609, 7113,  181794,  8, }, // Antarctica/McMurdo
    {  609, 8589,    2272,  6, }, // Antarctica/Palmer
    {  609, 7226,    2278,  6, }, // Antarctica/Rothera
    {  609, 7295,   28642,  4, }, // Antarctica/Syowa
    {  609, 8607,    2289,  5, }, // Antarctica/Troll
    {  609, 7374,   34957,  6, }, // Antarctica/Vostok
    {  609, 1429,  181802, 10, }, // Arctic/Longyearbyen
    {  609, 5570,  181812,  4, }, // Asia/Aden
    {  609, 5395,  181816,  7, }, // Asia/Almaty
    {  609, 3968,  181823,  5, }, // Asia/Amman
    {  609, 5511,   34980,  6, }, // Asia/Anadyr
    {  609, 5523,    2331,  5, }, // Asia/Aqtau
    {  609, 5368,    2336,  6, }, // Asia/Aqtobe
    {  609,  964,  181828,  7, }, // Asia/Ashgabat
    {  609, 8624,   52009,  6, }, // Asia/Atyrau
    {  609, 3607,  181835,  6, }, // Asia/Baghdad
    {  609, 5534,  181841,  7, }, // Asia/Bahrain
    {  609, 3653,  181848,  4, }, // Asia/Baku
    {  609, 4466,    2372,  7, }, // Asia/Bangkok
    {  609, 3571,    2379,  7, }, // Asia/Barnaul
    {  609, 4086,   28695,  6, }, // Asia/Beirut
    {  609, 3778,    2392,  6, }, // Asia/Bishkek
    {  609, 5989,   28707,  6, }, // Asia/Brunei
    {  609, 4601,    2404,  4, }, // Asia/Chita
    {  609, 4511,    2408,  7, }, // Asia/Colombo
    {  609, 4540,  181852,  6, }, // Asia/Damascus
    {  609, 1093,    2421,  5, }, // Asia/Dhaka
    {  609, 6192,    2426,  4, }, // Asia/Dili
    {  609, 3596,   28728,  5, }, // Asia/Dubai
    {  609, 6159,    2435,  7, }, // Asia/Dushanbe
    {  609, 8636,    2442,  9, }, // Asia/Famagusta
    {  609, 8651,  181858,  5, }, // Asia/Gaza
    {  609, 4795,   35018,  6, }, // Asia/Hebron
    {  609, 1259,   35024,  9, }, // Asia/Ho_Chi_Minh
    {  609, 2704,    2468,  7, }, // Asia/Hong_Kong
    {  609, 4771,    2475,  4, }, // Asia/Hovd
    {  609, 4184,    2479,  7, }, // Asia/Irkutsk
    {  609, 7006,  181863,  7, }, // Asia/Jakarta
    {  609, 6992,  181870,  7, }, // Asia/Jayapura
    {  609, 1290,   37094,  9, }, // Asia/Jerusalem
    {  609, 3560,  181877,  5, }, // Asia/Kabul
    {  609, 4316,    2529,  8, }, // Asia/Kamchatka
    {  609, 4239,    2540,  6, }, // Asia/Karachi
    {  609, 1185,    2546,  8, }, // Asia/Kathmandu
    {  609, 8661,   35065,  7, }, // Asia/Khandyga
    {  609,  992,   37103,  7, }, // Asia/Kolkata
    {  609, 4197,   37110, 10, }, // Asia/Krasnoyarsk
    {  609, 4859,   35090, 12, }, // Asia/Kuala_Lumpur
    {  609, 5976,    2592,  6, }, // Asia/Kuching
    {  609, 5547,  181882,  6, }, // Asia/Kuwait
    {  609, 1211,    2604,  5, }, // Asia/Macau
    {  609, 4017,    2609,  7, }, // Asia/Magadan
    {  609, 1349,    2616,  7, }, // Asia/Makassar
    {  609, 7182,    2623,  6, }, // Asia/Manila
    {  609, 6920,    2629,  6, }, // Asia/Muscat
    {  609, 2554,  106464,  7, }, // Asia/Nicosia
    {  609, 8675,  181888, 12, }, // Asia/Novokuznetsk
    {  609, 4135,    2653, 11, }, // Asia/Novosibirsk
    {  609, 4229,    2664,  4, }, // Asia/Omsk
    {  609, 7145,   35138,  4, }, // Asia/Oral
    {  609, 6961,    2674,  7, }, // Asia/Phnom_Penh
    {  609, 8693,    2681,  9, }, // Asia/Pontianak
    {  609, 4214,   37131,  6, }, // Asia/Pyongyang
    {  609, 5559,  181900,  5, }, // Asia/Qatar
    {  609, 8708,   28902,  8, }, // Asia/Qostanay
    {  609, 4269,  181905,  9, }, // Asia/Qyzylorda
    {  609, 3584,  181914,  4, }, // Asia/Riyadh
    {  609, 4408,    2732,  7, }, // Asia/Sakhalin
    {  609, 7245,  106501,  9, }, // Asia/Samarkand
    {  609, 3279,    2748,  4, }, // Asia/Seoul
    {  609, 1053,   28933,  6, }, // Asia/Shanghai
    {  609, 3300,    2758,  8, }, // Asia/Singapore
    {  609, 4297,   37158, 13, }, // Asia/Srednekolymsk
    {  609, 3263,   28961,  6, }, // Asia/Taipei
    {  609, 4781,  181918,  7, }, // Asia/Tashkent
    {  609, 3915,   35206,  7, }, // Asia/Tbilisi
    {  609, 2772,  181925,  6, }, // Asia/Tehran
    {  609, 1317,  181931,  5, }, // Asia/Thimphu
    {  609, 2821,    2810,  5, }, // Asia/Tokyo
    {  609, 4572,    2815,  5, }, // Asia/Tomsk
    {  609, 1021,    2820, 10, }, // Asia/Ulaanbaatar
    {  609, 1159,    2830,  6, }, // Asia/Urumqi
    {  609, 8722,    2836,  8, }, // Asia/Ust-Nera
    {  609, 6977,  181936,  7, }, // Asia/Vientiane
    {  609, 4710,    2851, 11, }, // Asia/Vladivostok
    {  609, 4828,   37183,  6, }, // Asia/Yakutsk
    {  609, 1235,   56541,  5, }, // Asia/Yangon
    {  609, 3883,    2875, 12, }, // Asia/Yekaterinburg
    {  609, 3747,    2887,  6, }, // Asia/Yerevan
    {  609, 3663,  181943, 14, }, // Atlantic/Azores
    {  609, 5671,   35284,  7, }, // Atlantic/Bermuda
    {  609, 6628,  181957,  7, }, // Atlantic/Canary
    {  609, 3727,    2944, 10, }, // Atlantic/Cape_Verde
    {  609, 1395,    2954,  5, }, // Atlantic/Faroe
    {  609, 8736,   37264,  7, }, // Atlantic/Madeira
    {  609, 2748,   37271,  8, }, // Atlantic/Reykjavik
    {  609, 7272,  181964, 14, }, // Atlantic/South_Georgia
    {  609, 6821,  118740, 10, }, // Atlantic/St_Helena
    {  609, 6644,   52256,  6, }, // Atlantic/Stanley
    {  609, 1670,    3035,  8, }, // Australia/Adelaide
    {  609, 1635,    3043,  7, }, // Australia/Brisbane
    {  609, 1799,    3050, 11, }, // Australia/Broken_Hill
    {  609, 1583,    3061,  6, }, // Australia/Darwin
    {  609, 3637,  181978,  5, }, // Australia/Eucla
    {  609, 1516,    3072,  6, }, // Australia/Hobart
    {  609, 8753,    3078,  8, }, // Australia/Lindeman
    {  609, 1547,  137829,  8, }, // Australia/Lord_Howe
    {  609, 1727,    3094,  7, }, // Australia/Melbourne
    {  609, 1762,    3101,  4, }, // Australia/Perth
    {  609, 1463,   52276,  6, }, // Australia/Sydney
    {  609, 6441,    3111,  9, }, // Europe/Amsterdam
    {  609, 6216,    3120,  7, }, // Europe/Andorra
    {  609, 3620,   29281,  9, }, // Europe/Astrakhan
    {  609, 2300,   52291,  5, }, // Europe/Athens
    {  609, 6336,    3139,  7, }, // Europe/Belgrade
    {  609, 4757,    3146,  6, }, // Europe/Berlin
    {  609, 6488,    3152, 10, }, // Europe/Bratislava
    {  609, 2167,   52296,  7, }, // Europe/Brussels
    {  609, 3928,    3170,  8, }, // Europe/Bucharest
    {  609, 3806,    3178,  8, }, // Europe/Budapest
    {  609, 8772,   35437,  8, }, // Europe/Busingen
    {  609, 2583,  137841,  7, }, // Europe/Chisinau
    {  609, 6289,   52323, 10, }, // Europe/Copenhagen
    {  609, 2338,    3226,  6, }, // Europe/Dublin
    {  609, 6307,    3232,  9, }, // Europe/Gibraltar
    {  609, 8788,    3241,  6, }, // Europe/Guernsey
    {  609, 6612,    3247,  8, }, // Europe/Helsinki
    {  609, 8804,  181983, 11, }, // Europe/Isle_of_Man
    {  609, 1130,   35484,  8, }, // Europe/Istanbul
    {  609, 8823,  181994,  5, }, // Europe/Jersey
    {  609, 3979,    3276, 11, }, // Europe/Kaliningrad
    {  609, 8929,    3287,  5, }, // Europe/Kirov
    {  609, 2527,    3292,  4, }, // Europe/Kyiv
    {  609, 3213,    3296,  8, }, // Europe/Lisbon
    {  609, 6506,   29432,  7, }, // Europe/Ljubljana
    {  609, 2501,    3313,  6, }, // Europe/London
    {  609, 6365,   29445, 10, }, // Europe/Luxembourg
    {  609, 4892,    3330,  6, }, // Europe/Madrid
    {  609, 6397,    3336,  5, }, // Europe/Malta
    {  609, 6582,   35532,  8, }, // Europe/Mariehamn
    {  609, 3693,    3350,  5, }, // Europe/Minsk
    {  609, 6410,    3355,  6, }, // Europe/Monaco
    {  609, 3537,    3361,  6, }, // Europe/Moscow
    {  609, 6458,    3367,  4, }, // Europe/Oslo
    {  609, 4284,    3371,  5, }, // Europe/Paris
    {  609, 6424,  181999, 10, }, // Europe/Podgorica
    {  609, 6275,    3385,  5, }, // Europe/Prague
    {  609, 8837,    3390,  4, }, // Europe/Riga
    {  609, 6324,    3394,  3, }, // Europe/Rome
    {  609, 4331,    3397,  6, }, // Europe/Samara
    {  609, 6470,    3403, 10, }, // Europe/San_Marino
    {  609, 6245,    3413,  7, }, // Europe/Sarajevo
    {  609, 4451,    3420,  7, }, // Europe/Saratov
    {  609, 8849,    3427, 10, }, // Europe/Simferopol
    {  609, 6383,  118774,  6, }, // Europe/Skopje
    {  609, 6599,   37418,  5, }, // Europe/Sofia
    {  609, 6523,  182009,  8, }, // Europe/Stockholm
    {  609, 8867,    3455,  6, }, // Europe/Tallinn
    {  609, 6202,    3461,  5, }, // Europe/Tirane
    {  609, 8882,   37423,  8, }, // Europe/Ulyanovsk
    {  609, 6352,   35606,  5, }, // Europe/Vaduz
    {  609, 6567,    3481,  7, }, // Europe/Vatican
    {  609, 6231,    3488,  4, }, // Europe/Vienna
    {  609, 8899,   37436,  6, }, // Europe/Vilnius
    {  609, 4727,    3499,  9, }, // Europe/Volgograd
    {  609, 3190,    3508,  7, }, // Europe/Warsaw
    {  609, 6261,    3515,  6, }, // Europe/Zagreb
    {  609, 6540,  182017,  5, }, // Europe/Zurich
    {  609, 5078,    3527, 12, }, // Indian/Antananarivo
    {  609, 6947,    3539,  5, }, // Indian/Chagos
    {  609, 6046,  182022,  8, }, // Indian/Christmas
    {  609, 6063,    3556,  5, }, // Indian/Cocos
    {  609, 5029,   35642,  6, }, // Indian/Comoro
    {  609, 6661,  182030,  8, }, // Indian/Kerguelen
    {  609, 7260,   35648,  4, }, // Indian/Mahe
    {  609, 7050,    3589,  6, }, // Indian/Maldives
    {  609, 4069,   56708,  8, }, // Indian/Mauritius
    {  609, 5098,   56716,  7, }, // Indian/Mayotte
    {  609, 7211,   37501,  7, }, // Indian/Reunion
    {  609, 4422,    3616,  4, }, // Pacific/Apia
    {  609,  932,    3620,  6, }, // Pacific/Auckland
    {  609, 3706,  182038,  9, }, // Pacific/Bougainville
    {  609, 3015,   37512,  5, }, // Pacific/Chatham
    {  609, 3157,  128869,  3, }, // Pacific/Chuuk
    {  609, 2237,  118802,  5, }, // Pacific/Easter
    {  609, 7360,   52541,  5, }, // Pacific/Efate
    {  609, 7327,    3659,  7, }, // Pacific/Fakaofo
    {  609, 3902,  182047,  4, }, // Pacific/Fiji
    {  609, 7343,    3670,  8, }, // Pacific/Funafuti
    {  609, 6678,    3678,  9, }, // Pacific/Galapagos
    {  609, 6696,  182051,  6, }, // Pacific/Gambier
    {  609, 3822,   35719, 11, }, // Pacific/Guadalcanal
    {  609, 6018,    3724,  4, }, // Pacific/Guam
    {  609, 3049,    3728,  6, }, // Pacific/Kanton
    {  609, 3998,    3734, 10, }, // Pacific/Kiritimati
    {  609, 7019,    3744,  6, }, // Pacific/Kosrae
    {  609, 2842,  137893,  9, }, // Pacific/Kwajalein
    {  609, 4877,  118826,  6, }, // Pacific/Majuro
    {  609, 4051,  128903,  6, }, // Pacific/Marquesas
    {  609, 8914,  182057,  6, }, // Pacific/Midway
    {  609, 7084,    3798,  5, }, // Pacific/Nauru
    {  609, 7132,   52636,  4, }, // Pacific/Niue
    {  609, 4168,    3807,  7, }, // Pacific/Norfolk
    {  609, 7098,    3814,  5, }, // Pacific/Noumea
    {  609, 3126,   35774,  9, }, // Pacific/Pago_Pago
    {  609, 7155,    3828,  5, }, // Pacific/Palau
    {  609, 7194,   56789,  7, }, // Pacific/Pitcairn
    {  609, 3096,   37603,  6, }, // Pacific/Pohnpei
    {  609, 4807,   37609, 11, }, // Pacific/Port_Moresby
    {  609, 6076,    3857,  9, }, // Pacific/Rarotonga
    {  609, 6031,   29935,  6, }, // Pacific/Saipan
    {  609, 7312,    3872,  5, }, // Pacific/Tahiti
    {  609, 6712,    3877,  6, }, // Pacific/Tarawa
    {  609, 4583,    3883,  9, }, // Pacific/Tongatapu
    {  609, 7392,  182063,  4, }, // Pacific/Wake
    {  609, 7405,    3896,  6, }, // Pacific/Wallis
    {  610, 6788,  182067,  9, }, // Africa/Abidjan Tamil/Tamil/India
    {  610, 6760,  182076,  5, }, // Africa/Accra
    {  610, 5059,  182081, 11, }, // Africa/Addis_Ababa
    {  610, 7420,  182092, 10, }, // Africa/Algiers
    {  610,   14,  182102,  7, }, // Africa/Asmara
    {  610,   44,  182109,  5, }, // Africa/Bamako
    {  610, 5257,  182114,  8, }, // Africa/Bangui
    {  610, 6746,  182122,  7, }, // Africa/Banjul
    {  610, 7435,  182129,  6, }, // Africa/Bissau
    {  610, 4957,  182135, 11, }, // Africa/Blantyre
    {  610, 5287,  182146, 12, }, // Africa/Brazzaville
    {  610, 4922,  182158, 10, }, // Africa/Bujumbura
    {  610, 2320,  182168,  6, }, // Africa/Cairo
    {  610, 4117,  182174, 12, }, // Africa/Casablanca
    {  610, 7449,  182186,  8, }, // Africa/Ceuta
    {  610, 6773,  182194,  7, }, // Africa/Conakry
    {  610, 6840,  182201,  5, }, // Africa/Dakar
    {  610, 5130,  182206, 14, }, // Africa/Dar_es_Salaam
    {  610, 5043,  182220,  8, }, // Africa/Djibouti
    {  610, 5243,  182228,  5, }, // Africa/Douala
    {  610, 5166,  182233,  9, }, // Africa/El_Aaiun
    {  610, 6853,  182242, 10, }, // Africa/Freetown
    {  610, 4906,  182252,  7, }, // Africa/Gaborone
    {  610, 5015,  182259,  5, }, // Africa/Harare
    {  610, 4479,  182264, 14, }, // Africa/Johannesburg
    {  610, 4499,  182278,  4, }, // Africa/Juba
    {  610, 5151,  182282,  7, }, // Africa/Kampala
    {  610, 4524,  182289,  8, }, // Africa/Khartoum
    {  610, 4973,  182297,  5, }, // Africa/Kigali
    {  610, 5306,  182302,  7, }, // Africa/Kinshasa
    {  610, 4744,  182309,  6, }, // Africa/Lagos
    {  610, 5336,  182315, 12, }, // Africa/Libreville
    {  610, 6869,  182327,  4, }, // Africa/Lome
    {  610, 5211,  182331,  8, }, // Africa/Luanda
    {  610, 4939,  182339, 10, }, // Africa/Lubumbashi
    {  610, 5001,  182349,  6, }, // Africa/Lusaka
    {  610, 5322,  182355,  5, }, // Africa/Malabo
    {  610, 4987,  182360,  5, }, // Africa/Maputo
    {  610, 5197,  182365,  5, }, // Africa/Maseru
    {  610, 5182,  182370,  5, }, // Africa/Mbabane
    {  610, 5113,  182375,  8, }, // Africa/Mogadishu
    {  610, 7034,  182383, 10, }, // Africa/Monrovia
    {  610, 3853,  182393,  6, }, // Africa/Nairobi
    {  610, 5271,  182399,  5, }, // Africa/Ndjamena
    {  610, 5354,  182404,  6, }, // Africa/Niamey
    {  610, 6803,  182410, 10, }, // Africa/Nouakchott
    {  610, 6727,  182420,  9, }, // Africa/Ouagadougou
    {  610, 5225,  182429, 13, }, // Africa/Porto-Novo
    {  610, 4435,  182442,  9, }, // Africa/Sao_Tome
    {  610, 2866,  182451,  8, }, // Africa/Tripoli
    {  610, 6554,  182459,  6, }, // Africa/Tunis
    {  610, 4152,  182465, 10, }, // Africa/Windhoek
    {  610,  132,  182475,  4, }, // America/Adak
    {  610, 3346,  182479,  9, }, // America/Anchorage
    {  610, 5607,  182488,  9, }, // America/Anguilla
    {  610, 5624,  182497,  9, }, // America/Antigua
    {  610, 4554,  182506, 10, }, // America/Araguaina
    {  610,  166,  182516, 14, }, // America/Argentina/Buenos_Aires
    {  610,   91,  182530,  8, }, // America/Argentina/Catamarca
    {  610,  270,  182538,  8, }, // America/Argentina/Cordoba
    {  610,  441,  182546,  6, }, // America/Argentina/Jujuy
    {  610, 7462,  182552,  9, }, // America/Argentina/La_Rioja
    {  610,  565,  182561,  8, }, // America/Argentina/Mendoza
    {  610, 7489,  182569, 13, }, // America/Argentina/Rio_Gallegos
    {  610, 7520,  182582,  6, }, // America/Argentina/Salta
    {  610, 7544,  182588, 11, }, // America/Argentina/San_Juan
    {  610, 5580,  182599, 11, }, // America/Argentina/San_Luis
    {  610, 7571,  182610,  7, }, // America/Argentina/Tucuman
    {  610, 7597,  182617,  7, }, // America/Argentina/Ushuaia
    {  610, 5640,  182624,  5, }, // America/Aruba
    {  610, 4252,  182629,  9, }, // America/Asuncion
    {  610,  237,  182638,  8, }, // America/Atikokan
    {  610, 3679,  182646,  6, }, // America/Bahia
    {  610, 7623,  182652, 15, }, // America/Bahia_Banderas
    {  610, 5654,  182667,  9, }, // America/Barbados
    {  610, 7646,  182676,  6, }, // America/Belem
    {  610, 5407,  182682,  6, }, // America/Belize
    {  610, 7660,  182688, 16, }, // America/Blanc-Sablon
    {  610, 7681,  182704, 11, }, // America/Boa_Vista
    {  610, 4361,  182715,  6, }, // America/Bogota
    {  610, 7699,  182721,  6, }, // America/Boise
    {  610, 7713,  182727, 20, }, // America/Cambridge_Bay
    {  610, 7735,  182747, 15, }, // America/Campo_Grande
    {  610, 3868,  182762,  7, }, // America/Cancun
    {  610, 4694,  182769,  6, }, // America/Caracas
    {  610, 4345,  182775,  6, }, // America/Cayenne
    {  610, 5496,  182781,  5, }, // America/Cayman
    {  610, 2260,  182786,  6, }, // America/Chicago
    {  610, 7756,  182792,  6, }, // America/Chihuahua
    {  610, 7774,  182798, 14, }, // America/Ciudad_Juarez
    {  610, 5422,  182812, 11, }, // America/Costa_Rica
    {  610, 7796,  182823, 10, }, // America/Creston
    {  610, 3791,  182833,  6, }, // America/Cuiaba
    {  610, 5723,  182839,  8, }, // America/Curacao
    {  610, 7812,  182847, 16, }, // America/Danmarkshavn
    {  610, 7833,  182863,  7, }, // America/Dawson
    {  610, 7848,  182870, 14, }, // America/Dawson_Creek
    {  610,  805,  182884,  7, }, // America/Denver
    {  610, 3465,  182891, 10, }, // America/Detroit
    {  610, 5739,  182901,  8, }, // America/Dominica
    {  610,  893,  182909, 11, }, // America/Edmonton
    {  610, 7869,  182920,  7, }, // America/Eirunepe
    {  610, 5441,  182927, 13, }, // America/El_Salvador
    {  610, 7886,  182940, 15, }, // America/Fort_Nelson
    {  610, 7906,  182955, 11, }, // America/Fortaleza
    {  610, 7924,  182966, 14, }, // America/Glace_Bay
    {  610, 6881,  182980,  7, }, // America/Goose_Bay
    {  610, 4612,  182987, 14, }, // America/Grand_Turk
    {  610, 5770,  183001,  6, }, // America/Grenada
    {  610, 5786,  183007,  9, }, // America/Guadeloupe
    {  610, 3760,  183016,  8, }, // America/Guatemala
    {  610, 4841,  183024,  9, }, // America/Guayaquil
    {  610, 6932,  183033,  5, }, // America/Guyana
    {  610, 1939,  183038, 10, }, // America/Halifax
    {  610, 2281,  183048,  5, }, // America/Havana
    {  610, 7942,  183053, 10, }, // America/Hermosillo
    {  610,  348,  183063, 15, }, // America/Indiana/Indianapolis
    {  610,  481,  183078, 17, }, // America/Indiana/Knox
    {  610, 7961,  183095, 18, }, // America/Indiana/Marengo
    {  610, 7985,  183113, 26, }, // America/Indiana/Petersburg
    {  610, 8012,  183139, 22, }, // America/Indiana/Tell_City
    {  610, 8038,  183161, 15, }, // America/Indiana/Vevay
    {  610, 8060,  183176, 21, }, // America/Indiana/Vincennes
    {  610, 8086,  183197, 19, }, // America/Indiana/Winamac
    {  610, 8110,  183216,  7, }, // America/Inuvik
    {  610,  660,  183223,  9, }, // America/Iqaluit
    {  610, 2799,  183232,  7, }, // America/Jamaica
    {  610, 5380,  183239,  6, }, // America/Juneau
    {  610,  521,  183245, 12, }, // America/Kentucky/Louisville
    {  610, 8125,  183257, 21, }, // America/Kentucky/Monticello
    {  610, 5704,  183278, 11, }, // America/Kralendijk
    {  610, 4376,  183289,  7, }, // America/La_Paz
    {  610, 7169,  183296,  4, }, // America/Lima
    {  610, 3239,  183300, 13, }, // America/Los_Angeles
    {  610, 5932,  183313, 27, }, // America/Lower_Princes
    {  610, 8153,  183340,  6, }, // America/Maceio
    {  610, 8168,  183346,  8, }, // America/Managua
    {  610, 1908,  183354,  5, }, // America/Manaus
    {  610, 5897,  183359,  8, }, // America/Marigot
    {  610, 5805,  183367, 12, }, // America/Martinique
    {  610, 8184,  183379,  8, }, // America/Matamoros
    {  610, 2917,  183387,  7, }, // America/Mazatlan
    {  610, 8202,  183394,  8, }, // America/Menominee
    {  610, 8220,  183402,  6, }, // America/Merida
    {  610, 8235,  183408, 10, }, // America/Metlakatla
    {  610, 2949,  183418, 14, }, // America/Mexico_City
    {  610, 4391,  183432,  9, }, // America/Miquelon
    {  610, 8254,  183441,  8, }, // America/Moncton
    {  610, 8270,  183449, 10, }, // America/Monterrey
    {  610, 4098,  183459, 12, }, // America/Montevideo
    {  610, 5824,  183471, 10, }, // America/Montserrat
    {  610, 5481,  183481,  4, }, // America/Nassau
    {  610, 2379,  183485, 10, }, // America/New_York
    {  610, 8288,  183495,  4, }, // America/Nome
    {  610, 1850,  183499,  7, }, // America/Noronha
    {  610, 8301,  183506, 22, }, // America/North_Dakota/Beulah
    {  610, 8329,  183528, 21, }, // America/North_Dakota/Center
    {  610, 8357,  183549, 24, }, // America/North_Dakota/New_Salem
    {  610,  393,  183573,  4, }, // America/Nuuk
    {  610, 8388,  183577,  6, }, // America/Ojinaga
    {  610, 2356,  183583,  5, }, // America/Panama
    {  610, 6173,  183588,  7, }, // America/Paramaribo
    {  610, 2973,  183595,  9, }, // America/Phoenix
    {  610, 3945,  183604, 19, }, // America/Port-au-Prince
    {  610, 5954,  183623, 20, }, // America/Port_of_Spain
    {  610, 8404,  183643, 15, }, // America/Porto_Velho
    {  610, 5843,  183658, 13, }, // America/Puerto_Rico
    {  610, 4030,  183671, 14, }, // America/Punta_Arenas
    {  610, 8424,  183685, 16, }, // America/Rankin_Inlet
    {  610, 8445,  183701,  7, }, // America/Recife
    {  610, 1995,  183708,  6, }, // America/Regina
    {  610, 8460,  183714,  7, }, // America/Resolute
    {  610,  695,  183721, 13, }, // America/Rio_Branco
    {  610, 8477,  183734,  9, }, // America/Santarem
    {  610, 2201,  183743, 10, }, // America/Santiago
    {  610, 6111,  183753, 15, }, // America/Santo_Domingo
    {  610, 1878,  183768, 10, }, // America/Sao_Paulo
    {  610, 6899,  183778, 15, }, // America/Scoresbysund
    {  610, 8494,  183793,  6, }, // America/Sitka
    {  610, 8508,  183799, 19, }, // America/St_Barthelemy
    {  610, 2061,  183818, 15, }, // America/St_Johns
    {  610, 5863,  183833, 15, }, // America/St_Kitts
    {  610, 5880,  183848, 15, }, // America/St_Lucia
    {  610,  855,  183863, 14, }, // America/St_Thomas
    {  610, 5913,  183877, 19, }, // America/St_Vincent
    {  610, 8530,  183896, 16, }, // America/Swift_Current
    {  610, 5461,  183912, 11, }, // America/Tegucigalpa
    {  610, 5756,  183923,  4, }, // America/Thule
    {  610,  313,  183927,  8, }, // America/Tijuana
    {  610,  608,  183935,  8, }, // America/Toronto
    {  610, 5688,  183943,  8, }, // America/Tortola
    {  610, 2093,  183951,  9, }, // America/Vancouver
    {  610, 2144,  183960, 12, }, // America/Whitehorse
    {  610,  734,  183972, 10, }, // America/Winnipeg
    {  610, 8552,  183982,  6, }, // America/Yakutat
    {  610, 6001,  183988,  4, }, // Antarctica/Casey
    {  610, 6094,  183992,  6, }, // Antarctica/Davis
    {  610, 6133,  183998, 21, }, // Antarctica/DumontDUrville
    {  610, 8568,  184019, 10, }, // Antarctica/Macquarie
    {  610, 7066,  184029,  5, }, // Antarctica/Mawson
    {  610, 7113,  184034,  9, }, // Antarctica/McMurdo
    {  610, 8589,  184043,  7, }, // Antarctica/Palmer
    {  610, 7226,  184050,  6, }, // Antarctica/Rothera
    {  610, 7295,  184056,  6, }, // Antarctica/Syowa
    {  610, 8607,  184062,  6, }, // Antarctica/Troll
    {  610, 7374,  184068,  8, }, // Antarctica/Vostok
    {  610, 1429,  184076, 13, }, // Arctic/Longyearbyen
    {  610, 5570,  184089,  4, }, // Asia/Aden
    {  610, 5395,  184093,  7, }, // Asia/Almaty
    {  610, 3968,  184100,  7, }, // Asia/Amman
    {  610, 5511,  184107,  6, }, // Asia/Anadyr
    {  610, 5523,  184113,  6, }, // Asia/Aqtau
    {  610, 5368,  184119,  7, }, // Asia/Aqtobe
    {  610,  964,  184126,  9, }, // Asia/Ashgabat
    {  610, 8624,  184135,  5, }, // Asia/Atyrau
    {  610, 3607,  184140,  8, }, // Asia/Baghdad
    {  610, 5534,  184148,  7, }, // Asia/Bahrain
    {  610, 3653,  184155,  6, }, // Asia/Baku
    {  610, 4466,  184161,  8, }, // Asia/Bangkok
    {  610, 3571,  184169,  8, }, // Asia/Barnaul
    {  610, 4086,  184177,  8, }, // Asia/Beirut
    {  610, 3778,  184185,  8, }, // Asia/Bishkek
    {  610, 5989,  184193,  6, }, // Asia/Brunei
    {  610, 4601,  184199,  4, }, // Asia/Chita
    {  610, 4511,  184203,  8, }, // Asia/Colombo
    {  610, 4540,  184211,  8, }, // Asia/Damascus
    {  610, 1093,  184219,  6, }, // Asia/Dhaka
    {  610, 6192,  184225,  4, }, // Asia/Dili
    {  610, 3596,  184229,  6, }, // Asia/Dubai
    {  610, 6159,  184235,  7, }, // Asia/Dushanbe
    {  610, 8636,  184242, 10, }, // Asia/Famagusta
    {  610, 8651,  182174,  4, }, // Asia/Gaza
    {  610, 4795,  184252,  8, }, // Asia/Hebron
    {  610, 1259,  184260, 17, }, // Asia/Ho_Chi_Minh
    {  610, 2704,  184277,  8, }, // Asia/Hong_Kong
    {  610, 4771,  184285,  6, }, // Asia/Hovd
    {  610, 4184,  184291, 11, }, // Asia/Irkutsk
    {  610, 7006,  184302,  9, }, // Asia/Jakarta
    {  610, 6992,  184311,  7, }, // Asia/Jayapura
    {  610, 1290,  184318,  9, }, // Asia/Jerusalem
    {  610, 3560,  184327,  6, }, // Asia/Kabul
    {  610, 4316,  184333,  9, }, // Asia/Kamchatka
    {  610, 4239,  184342,  7, }, // Asia/Karachi
    {  610, 1185,  184349, 10, }, // Asia/Kathmandu
    {  610, 8661,  184359,  8, }, // Asia/Khandyga
    {  610,  992,  184367,  9, }, // Asia/Kolkata
    {  610, 4197,  184376, 16, }, // Asia/Krasnoyarsk
    {  610, 4859,  184392, 11, }, // Asia/Kuala_Lumpur
    {  610, 5976,  184403,  6, }, // Asia/Kuching
    {  610, 5547,  184409,  6, }, // Asia/Kuwait
    {  610, 1211,  184415,  5, }, // Asia/Macau
    {  610, 4017,  184420,  5, }, // Asia/Magadan
    {  610, 1349,  184425,  7, }, // Asia/Makassar
    {  610, 7182,  184432,  5, }, // Asia/Manila
    {  610, 6920,  184437,  6, }, // Asia/Muscat
    {  610, 2554,  184443,  8, }, // Asia/Nicosia
    {  610, 8675,  184451, 16, }, // Asia/Novokuznetsk
    {  610, 4135,  184467, 14, }, // Asia/Novosibirsk
    {  610, 4229,  184481,  7, }, // Asia/Omsk
    {  610, 7145,  184488,  4, }, // Asia/Oral
    {  610, 6961,  184492, 12, }, // Asia/Phnom_Penh
    {  610, 8693,  184504, 12, }, // Asia/Pontianak
    {  610, 4214,  184516, 10, }, // Asia/Pyongyang
    {  610, 5559,  184526,  7, }, // Asia/Qatar
    {  610, 8708,  184533,  8, }, // Asia/Qostanay
    {  610, 4269,  184541, 10, }, // Asia/Qyzylorda
    {  610, 3584,  184551,  6, }, // Asia/Riyadh
    {  610, 4408,  184557,  6, }, // Asia/Sakhalin
    {  610, 7245,  184563, 10, }, // Asia/Samarkand
    {  610, 3279,  184573,  6, }, // Asia/Seoul
    {  610, 1053,  184579,  8, }, // Asia/Shanghai
    {  610, 3300,  184587, 11, }, // Asia/Singapore
    {  610, 4297,  184598, 18, }, // Asia/Srednekolymsk
    {  610, 3263,  184616,  6, }, // Asia/Taipei
    {  610, 4781,  184622,  9, }, // Asia/Tashkent
    {  610, 3915,  184631,  8, }, // Asia/Tbilisi
    {  610, 2772,  184639,  8, }, // Asia/Tehran
    {  610, 1317,  184647,  6, }, // Asia/Thimphu
    {  610, 2821,  184653,  8, }, // Asia/Tokyo
    {  610, 4572,  184661,  8, }, // Asia/Tomsk
    {  610, 1021,  184669, 12, }, // Asia/Ulaanbaatar
    {  610, 1159,  184681,  7, }, // Asia/Urumqi
    {  610, 8722,  184688, 10, }, // Asia/Ust-Nera
    {  610, 6977,  184698, 10, }, // Asia/Vientiane
    {  610, 4710,  184708, 14, }, // Asia/Vladivostok
    {  610, 4828,  184722,  9, }, // Asia/Yakutsk
    {  610, 1235,  184731,  7, }, // Asia/Yangon
    {  610, 3883,  184738, 15, }, // Asia/Yekaterinburg
    {  610, 3747,  184753,  6, }, // Asia/Yerevan
    {  610, 3663,  184759,  6, }, // Atlantic/Azores
    {  610, 5671,  184765,  8, }, // Atlantic/Bermuda
    {  610, 6628,  184773,  5, }, // Atlantic/Canary
    {  610, 3727,  184778, 11, }, // Atlantic/Cape_Verde
    {  610, 1395,  184789,  4, }, // Atlantic/Faroe
    {  610, 8736,  184793,  5, }, // Atlantic/Madeira
    {  610, 2748,  184798, 10, }, // Atlantic/Reykjavik
    {  610, 7272,  184808, 15, }, // Atlantic/South_Georgia
    {  610, 6821,  184823, 15, }, // Atlantic/St_Helena
    {  610, 6644,  184838,  8, }, // Atlantic/Stanley
    {  610, 1670,  184846,  9, }, // Australia/Adelaide
    {  610, 1635,  184855, 10, }, // Australia/Brisbane
    {  610, 1799,  184865, 14, }, // Australia/Broken_Hill
    {  610, 1583,  184879,  8, }, // Australia/Darwin
    {  610, 3637,  184887,  6, }, // Australia/Eucla
    {  610, 1516,  184893,  7, }, // Australia/Hobart
    {  610, 8753,  184900,  9, }, // Australia/Lindeman
    {  610, 1547,  184909, 11, }, // Australia/Lord_Howe
    {  610, 1727,  184920, 10, }, // Australia/Melbourne
    {  610, 1762,  184930,  6, }, // Australia/Perth
    {  610, 1463,  184936,  6, }, // Australia/Sydney
    {  610, 6441,  184942, 12, }, // Europe/Amsterdam
    {  610, 6216,  184954,  7, }, // Europe/Andorra
    {  610, 3620,  184961, 11, }, // Europe/Astrakhan
    {  610, 2300,  184972,  6, }, // Europe/Athens
    {  610, 6336,  184978, 10, }, // Europe/Belgrade
    {  610, 4757,  184988,  8, }, // Europe/Berlin
    {  610, 6488,  184996, 11, }, // Europe/Bratislava
    {  610, 2167,  185007,  9, }, // Europe/Brussels
    {  610, 3928,  185016, 10, }, // Europe/Bucharest
    {  610, 3806,  185026, 10, }, // Europe/Budapest
    {  610, 8772,  185036,  9, }, // Europe/Busingen
    {  610, 2583,  185045,  7, }, // Europe/Chisinau
    {  610, 6289,  185052, 10, }, // Europe/Copenhagen
    {  610, 2338,  185062,  7, }, // Europe/Dublin
    {  610, 6307,  185069, 11, }, // Europe/Gibraltar
    {  610, 8788,  185080,  6, }, // Europe/Guernsey
    {  610, 6612,  185086, 10, }, // Europe/Helsinki
    {  610, 8804,  185096, 13, }, // Europe/Isle_of_Man
    {  610, 1130,  185109, 11, }, // Europe/Istanbul
    {  610, 8823,  185120,  6, }, // Europe/Jersey
    {  610, 3979,  185126, 13, }, // Europe/Kaliningrad
    {  610, 8929,  185139,  6, }, // Europe/Kirov
    {  610, 2527,  185145,  4, }, // Europe/Kyiv
    {  610, 3213,  185149,  7, }, // Europe/Lisbon
    {  610, 6506,  185156,  8, }, // Europe/Ljubljana
    {  610, 2501,  185164,  6, }, // Europe/London
    {  610, 6365,  185170, 11, }, // Europe/Luxembourg
    {  610, 4892,  185181,  8, }, // Europe/Madrid
    {  610, 6397,  185189,  6, }, // Europe/Malta
    {  610, 6582,  185195,  6, }, // Europe/Mariehamn
    {  610, 3693,  185201,  8, }, // Europe/Minsk
    {  610, 6410,  185209,  8, }, // Europe/Monaco
    {  610, 3537,  185217,  6, }, // Europe/Moscow
    {  610, 6458,  185223,  5, }, // Europe/Oslo
    {  610, 4284,  185228,  6, }, // Europe/Paris
    {  610, 6424,  185234, 10, }, // Europe/Podgorica
    {  610, 6275,  185244,  6, }, // Europe/Prague
    {  610, 8837,  182819,  4, }, // Europe/Riga
    {  610, 6324,  185250,  4, }, // Europe/Rome
    {  610, 4331,  185254,  5, }, // Europe/Samara
    {  610, 6470,  185259, 10, }, // Europe/San_Marino
    {  610, 6245,  185269,  6, }, // Europe/Sarajevo
    {  610, 4451,  185275,  6, }, // Europe/Saratov
    {  610, 8849,  185281, 13, }, // Europe/Simferopol
    {  610, 6383,  185294,  8, }, // Europe/Skopje
    {  610, 6599,  185302,  7, }, // Europe/Sofia
    {  610, 6523,  185309, 10, }, // Europe/Stockholm
    {  610, 8867,  185319,  5, }, // Europe/Tallinn
    {  610, 6202,  185324,  6, }, // Europe/Tirane
    {  610, 8882,  185330, 11, }, // Europe/Ulyanovsk
    {  610, 6352,  185341,  5, }, // Europe/Vaduz
    {  610, 6567,  185346,  7, }, // Europe/Vatican
    {  610, 6231,  185353,  7, }, // Europe/Vienna
    {  610, 8899,  185360,  9, }, // Europe/Vilnius
    {  610, 4727,  185369, 12, }, // Europe/Volgograd
    {  610, 3190,  185381,  6, }, // Europe/Warsaw
    {  610, 6261,  185387,  7, }, // Europe/Zagreb
    {  610, 6540,  185394,  6, }, // Europe/Zurich
    {  610, 5078,  185400, 10, }, // Indian/Antananarivo
    {  610, 6947,  185410,  6, }, // Indian/Chagos
    {  610, 6046,  185416, 11, }, // Indian/Christmas
    {  610, 6063,  185427,  6, }, // Indian/Cocos
    {  610, 5029,  185433,  5, }, // Indian/Comoro
    {  610, 6661,  185438, 10, }, // Indian/Kerguelen
    {  610, 7260,  185448,  4, }, // Indian/Mahe
    {  610, 7050,  185452, 12, }, // Indian/Maldives
    {  610, 4069,  185464,  9, }, // Indian/Mauritius
    {  610, 5098,  185473,  7, }, // Indian/Mayotte
    {  610, 7211,  185480,  9, }, // Indian/Reunion
    {  610, 4422,  185489,  5, }, // Pacific/Apia
    {  610,  932,  185494,  9, }, // Pacific/Auckland
    {  610, 3706,  185503, 14, }, // Pacific/Bougainville
    {  610, 3015,  185517,  7, }, // Pacific/Chatham
    {  610, 3157,  185524,  4, }, // Pacific/Chuuk
    {  610, 2237,  185528,  6, }, // Pacific/Easter
    {  610, 7360,  185534,  6, }, // Pacific/Efate
    {  610, 7327,  185540,  7, }, // Pacific/Fakaofo
    {  610, 3902,  185547,  5, }, // Pacific/Fiji
    {  610, 7343,  185552, 10, }, // Pacific/Funafuti
    {  610, 6678,  185562,  7, }, // Pacific/Galapagos
    {  610, 6696,  185569,  9, }, // Pacific/Gambier
    {  610, 3822,  185578, 13, }, // Pacific/Guadalcanal
    {  610, 6018,  185591,  6, }, // Pacific/Guam
    {  610, 2723,  185597,  8, }, // Pacific/Honolulu
    {  610, 3049,  185605,  9, }, // Pacific/Kanton
    {  610, 3998,  185614, 12, }, // Pacific/Kiritimati
    {  610, 7019,  185626,  6, }, // Pacific/Kosrae
    {  610, 2842,  185632, 11, }, // Pacific/Kwajalein
    {  610, 4877,  185643,  5, }, // Pacific/Majuro
    {  610, 4051,  185648, 12, }, // Pacific/Marquesas
    {  610, 8914,  185660,  6, }, // Pacific/Midway
    {  610, 7084,  185666,  5, }, // Pacific/Nauru
    {  610, 7132,  185671,  4, }, // Pacific/Niue
    {  610, 4168,  185675,  9, }, // Pacific/Norfolk
    {  610, 7098,  185684,  8, }, // Pacific/Noumea
    {  610, 3126,  185692,  9, }, // Pacific/Pago_Pago
    {  610, 7155,  185701,  5, }, // Pacific/Palau
    {  610, 7194,  185706, 12, }, // Pacific/Pitcairn
    {  610, 3096,  185718,  9, }, // Pacific/Pohnpei
    {  610, 4807,  185727, 15, }, // Pacific/Port_Moresby
    {  610, 6076,  185742,  9, }, // Pacific/Rarotonga
    {  610, 6031,  185751,  5, }, // Pacific/Saipan
    {  610, 7312,  185756,  5, }, // Pacific/Tahiti
    {  610, 6712,  185761,  5, }, // Pacific/Tarawa
    {  610, 4583,  185766, 10, }, // Pacific/Tongatapu
    {  610, 7392,  185776,  4, }, // Pacific/Wake
    {  610, 7405,  185780,  6, }, // Pacific/Wallis
    {  614,   14,       0,  6, }, // Africa/Asmara Taroko/Latin/Taiwan
    {  614, 4435,       6,  8, }, // Africa/Sao_Tome
    {  614, 3346,  185786, 19, }, // America/Anchorage
    {  614, 4252,      14,  8, }, // America/Asuncion
    {  614,  237,      22,  8, }, // America/Atikokan
    {  614, 7623,      30, 17, }, // America/Bahia_Banderas
    {  614, 7735,  185805, 16, }, // America/Campo_Grande
    {  614, 3868,      47,  6, }, // America/Cancun
    {  614, 2260,  185821, 18, }, // America/Chicago
    {  614, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  614, 5723,      66,  7, }, // America/Curacao
    {  614,  805,  185839, 17, }, // America/Denver
    {  614,  348,  185856, 25, }, // America/Indiana/Indianapolis
    {  614,  481,      73, 13, }, // America/Indiana/Knox
    {  614, 7961,      86, 16, }, // America/Indiana/Marengo
    {  614, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  614, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  614, 8038,     139, 14, }, // America/Indiana/Vevay
    {  614, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  614, 8086,     171, 16, }, // America/Indiana/Winamac
    {  614, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  614, 3239,  185881, 19, }, // America/Los_Angeles
    {  614, 5932,     207, 22, }, // America/Lower_Princes
    {  614, 8220,     229,  6, }, // America/Merida
    {  614, 2379,  185900, 18, }, // America/New_York
    {  614, 1850,     235, 19, }, // America/Noronha
    {  614, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  614, 8329,     274, 20, }, // America/North_Dakota/Center
    {  614, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  614,  393,     317,  4, }, // America/Nuuk
    {  614, 2973,  185918, 19, }, // America/Phoenix
    {  614,  695,  185937, 18, }, // America/Rio_Branco
    {  614, 1878,  185955, 20, }, // America/Sao_Paulo
    {  614, 6899,     321, 16, }, // America/Scoresbysund
    {  614, 8508,     337, 14, }, // America/St_Barthelemy
    {  614, 2061,     351, 10, }, // America/St_Johns
    {  614, 5863,     361,  9, }, // America/St_Kitts
    {  614, 5880,     370,  9, }, // America/St_Lucia
    {  614,  855,     379, 10, }, // America/St_Thomas
    {  614, 5913,     389, 11, }, // America/St_Vincent
    {  614, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  614, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  614, 1185,     448,  9, }, // Asia/Kathmandu
    {  614,  992,     457,  7, }, // Asia/Kolkata
    {  614, 1211,     464,  5, }, // Asia/Macau
    {  614, 1235,     477,  6, }, // Asia/Yangon
    {  614, 1395,     483,  5, }, // Atlantic/Faroe
    {  614, 6821,     488, 10, }, // Atlantic/St_Helena
    {  614, 2527,     514,  4, }, // Europe/Kyiv
    {  614, 7211,     547,  7, }, // Indian/Reunion
    {  614, 3157,     554,  5, }, // Pacific/Chuuk
    {  614, 2723,  185975, 20, }, // Pacific/Honolulu
    {  614, 3096,     586,  7, }, // Pacific/Pohnpei
    {  615,   14,       0,  6, }, // Africa/Asmara Tasawaq/Latin/Niger
    {  615, 4435,       6,  8, }, // Africa/Sao_Tome
    {  615, 4252,      14,  8, }, // America/Asuncion
    {  615,  237,      22,  8, }, // America/Atikokan
    {  615, 7623,      30, 17, }, // America/Bahia_Banderas
    {  615, 3868,      47,  6, }, // America/Cancun
    {  615, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  615, 5723,      66,  7, }, // America/Curacao
    {  615,  481,      73, 13, }, // America/Indiana/Knox
    {  615, 7961,      86, 16, }, // America/Indiana/Marengo
    {  615, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  615, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  615, 8038,     139, 14, }, // America/Indiana/Vevay
    {  615, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  615, 8086,     171, 16, }, // America/Indiana/Winamac
    {  615, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  615, 5932,     207, 22, }, // America/Lower_Princes
    {  615, 8220,     229,  6, }, // America/Merida
    {  615, 1850,     235, 19, }, // America/Noronha
    {  615, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  615, 8329,     274, 20, }, // America/North_Dakota/Center
    {  615, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  615,  393,     317,  4, }, // America/Nuuk
    {  615, 6899,     321, 16, }, // America/Scoresbysund
    {  615, 8508,     337, 14, }, // America/St_Barthelemy
    {  615, 2061,     351, 10, }, // America/St_Johns
    {  615, 5863,     361,  9, }, // America/St_Kitts
    {  615, 5880,     370,  9, }, // America/St_Lucia
    {  615,  855,     379, 10, }, // America/St_Thomas
    {  615, 5913,     389, 11, }, // America/St_Vincent
    {  615, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  615, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  615, 1185,     448,  9, }, // Asia/Kathmandu
    {  615,  992,     457,  7, }, // Asia/Kolkata
    {  615, 1211,     464,  5, }, // Asia/Macau
    {  615, 1235,     477,  6, }, // Asia/Yangon
    {  615, 1395,     483,  5, }, // Atlantic/Faroe
    {  615, 6821,     488, 10, }, // Atlantic/St_Helena
    {  615, 2527,     514,  4, }, // Europe/Kyiv
    {  615, 7211,     547,  7, }, // Indian/Reunion
    {  615, 3157,     554,  5, }, // Pacific/Chuuk
    {  615, 3096,     586,  7, }, // Pacific/Pohnpei
    {  616, 6788,  185995,  6, }, // Africa/Abidjan Tatar/Cyrillic/Russia
    {  616, 6760,     610,  5, }, // Africa/Accra
    {  616, 5059,     615, 11, }, // Africa/Addis_Ababa
    {  616, 7420,     626,  5, }, // Africa/Algiers
    {  616,   14,   56236,  6, }, // Africa/Asmara
    {  616,   44,     637,  6, }, // Africa/Bamako
    {  616, 5257,     643,  5, }, // Africa/Bangui
    {  616, 6746,  186001,  6, }, // Africa/Banjul
    {  616, 7435,  137252,  6, }, // Africa/Bissau
    {  616, 4957,   27220,  8, }, // Africa/Blantyre
    {  616, 5287,   51554, 10, }, // Africa/Brazzaville
    {  616, 4922,  186007,  9, }, // Africa/Bujumbura
    {  616, 2320,  186016,  6, }, // Africa/Cairo
    {  616, 4117,     689, 10, }, // Africa/Casablanca
    {  616, 7449,     699,  5, }, // Africa/Ceuta
    {  616, 6773,     704,  7, }, // Africa/Conakry
    {  616, 6840,     711,  5, }, // Africa/Dakar
    {  616, 5130,   27248, 12, }, // Africa/Dar_es_Salaam
    {  616, 5043,   36379,  7, }, // Africa/Djibouti
    {  616, 5243,     734,  5, }, // Africa/Douala
    {  616, 5166,   27267,  7, }, // Africa/El_Aaiun
    {  616, 6853,     746,  7, }, // Africa/Freetown
    {  616, 4906,     753,  8, }, // Africa/Gaborone
    {  616, 5015,   33913,  6, }, // Africa/Harare
    {  616, 4479,   51572, 12, }, // Africa/Johannesburg
    {  616, 4499,  186022,  4, }, // Africa/Juba
    {  616, 5151,     783,  7, }, // Africa/Kampala
    {  616, 4524,  186026,  6, }, // Africa/Khartoum
    {  616, 4973,     796,  6, }, // Africa/Kigali
    {  616, 5306,     802,  7, }, // Africa/Kinshasa
    {  616, 4744,     809,  5, }, // Africa/Lagos
    {  616, 5336,   51584,  9, }, // Africa/Libreville
    {  616, 6869,     822,  4, }, // Africa/Lome
    {  616, 5211,     826,  6, }, // Africa/Luanda
    {  616, 4939,     832,  9, }, // Africa/Lubumbashi
    {  616, 5001,     841,  6, }, // Africa/Lusaka
    {  616, 5322,     847,  6, }, // Africa/Malabo
    {  616, 4987,   33945,  6, }, // Africa/Maputo
    {  616, 5197,     859,  6, }, // Africa/Maseru
    {  616, 5182,     865,  7, }, // Africa/Mbabane
    {  616, 5113,     872,  8, }, // Africa/Mogadishu
    {  616, 7034,   36419,  8, }, // Africa/Monrovia
    {  616, 3853,   36427,  7, }, // Africa/Nairobi
    {  616, 5271,  186032,  6, }, // Africa/Ndjamena
    {  616, 5354,   36434,  6, }, // Africa/Niamey
    {  616, 6803,     908,  7, }, // Africa/Nouakchott
    {  616, 6727,     915,  8, }, // Africa/Ouagadougou
    {  616, 5225,     923, 10, }, // Africa/Porto-Novo
    {  616, 4435,     933,  8, }, // Africa/Sao_Tome
    {  616, 2866,     941,  7, }, // Africa/Tripoli
    {  616, 6554,     948,  5, }, // Africa/Tunis
    {  616, 4152,     953,  7, }, // Africa/Windhoek
    {  616,  132,     960,  4, }, // America/Adak
    {  616, 3346,   51593,  8, }, // America/Anchorage
    {  616, 5607,   51601,  7, }, // America/Anguilla
    {  616, 5624,     978,  7, }, // America/Antigua
    {  616, 4554,   36455,  9, }, // America/Araguaina
    {  616,  166,   51608, 12, }, // America/Argentina/Buenos_Aires
    {  616,   91,    1006,  9, }, // America/Argentina/Catamarca
    {  616,  270,   34035,  7, }, // America/Argentina/Cordoba
    {  616,  441,   27487,  5, }, // America/Argentina/Jujuy
    {  616, 7462,   34047,  8, }, // America/Argentina/La_Rioja
    {  616,  565,    1035,  7, }, // America/Argentina/Mendoza
    {  616, 7489,   51620, 12, }, // America/Argentina/Rio_Gallegos
    {  616, 7520,   27512,  6, }, // America/Argentina/Salta
    {  616, 7544,    1058,  8, }, // America/Argentina/San_Juan
    {  616, 5580,    1066,  8, }, // America/Argentina/San_Luis
    {  616, 7571,    1074,  7, }, // America/Argentina/Tucuman
    {  616, 7597,  106260,  6, }, // America/Argentina/Ushuaia
    {  616, 5640,    1087,  5, }, // America/Aruba
    {  616, 4252,   51632,  8, }, // America/Asuncion
    {  616,  237,   34103,  8, }, // America/Atikokan
    {  616, 3679,   36500,  4, }, // America/Bahia
    {  616, 7623,   51640, 16, }, // America/Bahia_Banderas
    {  616, 5654,    1131,  8, }, // America/Barbados
    {  616, 7646,   34130,  5, }, // America/Belem
    {  616, 5407,    1144,  5, }, // America/Belize
    {  616, 7660,   36520, 11, }, // America/Blanc-Sablon
    {  616, 7681,    1161,  9, }, // America/Boa_Vista
    {  616, 4361,    1170,  6, }, // America/Bogota
    {  616, 7699,   51656,  5, }, // America/Boise
    {  616, 7713,  186038, 12, }, // America/Cambridge_Bay
    {  616, 7735,  137308, 12, }, // America/Campo_Grande
    {  616, 3868,    1205,  6, }, // America/Cancun
    {  616, 4694,    1211,  7, }, // America/Caracas
    {  616, 4345,   51674,  7, }, // America/Cayenne
    {  616, 5496,  186050, 16, }, // America/Cayman
    {  616, 2260,    1247,  6, }, // America/Chicago
    {  616, 7756,  118226,  8, }, // America/Chihuahua
    {  616, 7774,   56292, 13, }, // America/Ciudad_Juarez
    {  616, 5422,    1259, 10, }, // America/Costa_Rica
    {  616, 7796,    1269,  7, }, // America/Creston
    {  616, 3791,   27688,  5, }, // America/Cuiaba
    {  616, 5723,   36606,  7, }, // America/Curacao
    {  616, 7812,   36613, 12, }, // America/Danmarkshavn
    {  616, 7833,    1302,  6, }, // America/Dawson
    {  616, 7848,    1308, 11, }, // America/Dawson_Creek
    {  616,  805,    1319,  6, }, // America/Denver
    {  616, 3465,   36649,  7, }, // America/Detroit
    {  616, 5739,    1332,  8, }, // America/Dominica
    {  616,  893,   51681,  8, }, // America/Edmonton
    {  616, 7869,   51689,  8, }, // America/Eirunepe
    {  616, 5441,   27765,  9, }, // America/El_Salvador
    {  616, 7886,   51697, 12, }, // America/Fort_Nelson
    {  616, 7906,    1375,  9, }, // America/Fortaleza
    {  616, 7924,  186066,  9, }, // America/Glace_Bay
    {  616, 6881,  186075, 11, }, // America/Goose_Bay
    {  616, 4612,  181525, 10, }, // America/Grand_Turk
    {  616, 5770,    1410,  7, }, // America/Grenada
    {  616, 5786,    1417,  9, }, // America/Guadeloupe
    {  616, 3760,    1426,  9, }, // America/Guatemala
    {  616, 4841,   51725,  8, }, // America/Guayaquil
    {  616, 6932,   51733,  6, }, // America/Guyana
    {  616, 1939,   51739,  8, }, // America/Halifax
    {  616, 2281,    1457,  6, }, // America/Havana
    {  616, 7942,   51747,  9, }, // America/Hermosillo
    {  616,  348,    1471, 12, }, // America/Indiana/Indianapolis
    {  616,  481,    1483, 13, }, // America/Indiana/Knox
    {  616, 7961,    1496, 16, }, // America/Indiana/Marengo
    {  616, 7985,  181541, 18, }, // America/Indiana/Petersburg
    {  616, 8012,  186086, 19, }, // America/Indiana/Tell_City
    {  616, 8038,   56323, 14, }, // America/Indiana/Vevay
    {  616, 8060,   56337, 18, }, // America/Indiana/Vincennes
    {  616, 8086,   56355, 16, }, // America/Indiana/Winamac
    {  616, 8110,    1570,  6, }, // America/Inuvik
    {  616,  660,    1576,  7, }, // America/Iqaluit
    {  616, 2799,   28011,  6, }, // America/Jamaica
    {  616, 5380,   36746,  5, }, // America/Juneau
    {  616,  521,  186105,  8, }, // America/Kentucky/Louisville
    {  616, 8125,  118364, 20, }, // America/Kentucky/Monticello
    {  616, 5704,   36751, 10, }, // America/Kralendijk
    {  616, 4376,    1632,  6, }, // America/La_Paz
    {  616, 7169,    1638,  4, }, // America/Lima
    {  616, 3239,   51761, 12, }, // America/Los_Angeles
    {  616, 5932,  186113, 22, }, // America/Lower_Princes
    {  616, 8153,   36799,  6, }, // America/Maceio
    {  616, 8168,    1677,  7, }, // America/Managua
    {  616, 1908,    1684,  6, }, // America/Manaus
    {  616, 5897,    1690,  6, }, // America/Marigot
    {  616, 5805,    1696,  9, }, // America/Martinique
    {  616, 8184,    1705,  9, }, // America/Matamoros
    {  616, 2917,    1714,  8, }, // America/Mazatlan
    {  616, 8202,    1722,  8, }, // America/Menominee
    {  616, 8220,    1730,  6, }, // America/Merida
    {  616, 8235,    1736, 10, }, // America/Metlakatla
    {  616, 2949,    1746,  6, }, // America/Mexico_City
    {  616, 4391,    1752,  7, }, // America/Miquelon
    {  616, 8254,    1759,  7, }, // America/Moncton
    {  616, 8270,   51794,  9, }, // America/Monterrey
    {  616, 4098,    1775, 10, }, // America/Montevideo
    {  616, 5824,   51803,  9, }, // America/Montserrat
    {  616, 5481,    1795,  6, }, // America/Nassau
    {  616, 2379,   51812,  8, }, // America/New_York
    {  616, 8288,  186135,  4, }, // America/Nome
    {  616, 1850,   56389,  7, }, // America/Noronha
    {  616, 8301,  186139, 20, }, // America/North_Dakota/Beulah
    {  616, 8329,  186159, 19, }, // America/North_Dakota/Center
    {  616, 8357,  186178, 24, }, // America/North_Dakota/New_Salem
    {  616,  393,    1890,  4, }, // America/Nuuk
    {  616, 8388,    1894,  7, }, // America/Ojinaga
    {  616, 2356,    1901,  6, }, // America/Panama
    {  616, 6173,    1907, 10, }, // America/Paramaribo
    {  616, 2973,   34673,  6, }, // America/Phoenix
    {  616, 3945,    1923, 12, }, // America/Port-au-Prince
    {  616, 5954,   28321, 13, }, // America/Port_of_Spain
    {  616, 8404,   28334, 11, }, // America/Porto_Velho
    {  616, 5843,   51888, 11, }, // America/Puerto_Rico
    {  616, 4030,    1970, 12, }, // America/Punta_Arenas
    {  616, 8424,    1982, 12, }, // America/Rankin_Inlet
    {  616, 8445,    1994,  6, }, // America/Recife
    {  616, 1995,   34741,  6, }, // America/Regina
    {  616, 8460,   36892,  7, }, // America/Resolute
    {  616,  695,  137625, 10, }, // America/Rio_Branco
    {  616, 8477,   34764,  8, }, // America/Santarem
    {  616, 2201,   51907,  8, }, // America/Santiago
    {  616, 6111,    2041, 13, }, // America/Santo_Domingo
    {  616, 1878,    2054,  9, }, // America/Sao_Paulo
    {  616, 6899,  118545, 16, }, // America/Scoresbysund
    {  616, 8494,    2074,  5, }, // America/Sitka
    {  616, 8508,  186202, 15, }, // America/St_Barthelemy
    {  616, 2061,   28481, 10, }, // America/St_Johns
    {  616, 5863,    2101,  9, }, // America/St_Kitts
    {  616, 5880,  186217, 10, }, // America/St_Lucia
    {  616,  855,    2121, 10, }, // America/St_Thomas
    {  616, 5913,    2131, 12, }, // America/St_Vincent
    {  616, 5461,   51949, 12, }, // America/Tegucigalpa
    {  616, 5756,    2167,  4, }, // America/Thule
    {  616,  313,    2171,  7, }, // America/Tijuana
    {  616,  608,    2178,  7, }, // America/Toronto
    {  616, 5688,    2185,  7, }, // America/Tortola
    {  616, 2093,    2192,  8, }, // America/Vancouver
    {  616, 2144,   28567,  8, }, // America/Whitehorse
    {  616,  734,    2208,  8, }, // America/Winnipeg
    {  616, 8552,   28582,  6, }, // America/Yakutat
    {  616, 6001,   37021,  5, }, // Antarctica/Casey
    {  616, 6094,  118616,  5, }, // Antarctica/Davis
    {  616, 6133,  137691, 14, }, // Antarctica/DumontDUrville
    {  616, 8568,    2249,  8, }, // Antarctica/Macquarie
    {  616, 7066,    2257,  6, }, // Antarctica/Mawson
    {  616, 7113,  118634,  8, }, // Antarctica/McMurdo
    {  616, 8589,    2272,  6, }, // Antarctica/Palmer
    {  616, 7226,    2278,  6, }, // Antarctica/Rothera
    {  616, 7295,   28642,  4, }, // Antarctica/Syowa
    {  616, 8607,   51975,  6, }, // Antarctica/Troll
    {  616, 7374,   34957,  6, }, // Antarctica/Vostok
    {  616, 1429,  186227, 11, }, // Arctic/Longyearbyen
    {  616, 5570,    2310,  4, }, // Asia/Aden
    {  616, 5395,   51988,  8, }, // Asia/Almaty
    {  616, 3968,  186238,  5, }, // Asia/Amman
    {  616, 5511,   52002,  7, }, // Asia/Anadyr
    {  616, 5523,    2331,  5, }, // Asia/Aqtau
    {  616, 5368,    2336,  6, }, // Asia/Aqtobe
    {  616,  964,    2342,  7, }, // Asia/Ashgabat
    {  616, 8624,    2349,  6, }, // Asia/Atyrau
    {  616, 3607,    2355,  6, }, // Asia/Baghdad
    {  616, 5534,   37080,  7, }, // Asia/Bahrain
    {  616, 3653,    2368,  4, }, // Asia/Baku
    {  616, 4466,    2372,  7, }, // Asia/Bangkok
    {  616, 3571,    2379,  7, }, // Asia/Barnaul
    {  616, 4086,  186243,  6, }, // Asia/Beirut
    {  616, 3778,    2392,  6, }, // Asia/Bishkek
    {  616, 5989,   28707,  6, }, // Asia/Brunei
    {  616, 4601,    2404,  4, }, // Asia/Chita
    {  616, 4511,    2408,  7, }, // Asia/Colombo
    {  616, 4540,  186249,  7, }, // Asia/Damascus
    {  616, 1093,    2421,  5, }, // Asia/Dhaka
    {  616, 6192,    2426,  4, }, // Asia/Dili
    {  616, 3596,   28728,  5, }, // Asia/Dubai
    {  616, 6159,    2435,  7, }, // Asia/Dushanbe
    {  616, 8636,    2442,  9, }, // Asia/Famagusta
    {  616, 8651,    2451,  4, }, // Asia/Gaza
    {  616, 4795,  186256,  6, }, // Asia/Hebron
    {  616, 1259,   35024,  9, }, // Asia/Ho_Chi_Minh
    {  616, 2704,   52034,  7, }, // Asia/Hong_Kong
    {  616, 4771,    2475,  4, }, // Asia/Hovd
    {  616, 4184,    2479,  7, }, // Asia/Irkutsk
    {  616, 7006,  186262,  7, }, // Asia/Jakarta
    {  616, 6992,  186269,  7, }, // Asia/Jayapura
    {  616, 1290,    2501,  9, }, // Asia/Jerusalem
    {  616, 3560,    2510,  5, }, // Asia/Kabul
    {  616, 4316,    2529,  8, }, // Asia/Kamchatka
    {  616, 4239,    2540,  6, }, // Asia/Karachi
    {  616, 1185,    2546,  8, }, // Asia/Kathmandu
    {  616, 8661,    2554,  7, }, // Asia/Khandyga
    {  616,  992,   52082,  9, }, // Asia/Kolkata
    {  616, 4197,   37110, 10, }, // Asia/Krasnoyarsk
    {  616, 4859,    2580, 12, }, // Asia/Kuala_Lumpur
    {  616, 5976,    2592,  6, }, // Asia/Kuching
    {  616, 5547,   28825,  6, }, // Asia/Kuwait
    {  616, 1211,    2604,  5, }, // Asia/Macau
    {  616, 4017,    2609,  7, }, // Asia/Magadan
    {  616, 1349,  118684,  8, }, // Asia/Makassar
    {  616, 7182,    2623,  6, }, // Asia/Manila
    {  616, 6920,    2629,  6, }, // Asia/Muscat
    {  616, 2554,  106464,  7, }, // Asia/Nicosia
    {  616, 8675,    2642, 11, }, // Asia/Novokuznetsk
    {  616, 4135,    2653, 11, }, // Asia/Novosibirsk
    {  616, 4229,    2664,  4, }, // Asia/Omsk
    {  616, 7145,   35138,  4, }, // Asia/Oral
    {  616, 6961,   52091,  8, }, // Asia/Phnom_Penh
    {  616, 8693,    2681,  9, }, // Asia/Pontianak
    {  616, 4214,   28895,  7, }, // Asia/Pyongyang
    {  616, 5559,    2697,  5, }, // Asia/Qatar
    {  616, 8708,   37137,  8, }, // Asia/Qostanay
    {  616, 4269,    2710,  9, }, // Asia/Qyzylorda
    {  616, 3584,   52099,  7, }, // Asia/Riyadh
    {  616, 4408,    2732,  7, }, // Asia/Sakhalin
    {  616, 7245,  186276,  9, }, // Asia/Samarkand
    {  616, 3279,    2748,  4, }, // Asia/Seoul
    {  616, 1053,   28933,  6, }, // Asia/Shanghai
    {  616, 3300,    2758,  8, }, // Asia/Singapore
    {  616, 4297,    2766, 13, }, // Asia/Srednekolymsk
    {  616, 3263,   61028,  6, }, // Asia/Taipei
    {  616, 4781,    2785,  7, }, // Asia/Tashkent
    {  616, 3915,   35206,  7, }, // Asia/Tbilisi
    {  616, 2772,  186285,  6, }, // Asia/Tehran
    {  616, 1317,    2803,  7, }, // Asia/Thimphu
    {  616, 2821,    2810,  5, }, // Asia/Tokyo
    {  616, 4572,    2815,  5, }, // Asia/Tomsk
    {  616, 1021,    2820, 10, }, // Asia/Ulaanbaatar
    {  616, 1159,    2830,  6, }, // Asia/Urumqi
    {  616, 8722,   52125,  9, }, // Asia/Ust-Nera
    {  616, 6977,   52134,  8, }, // Asia/Vientiane
    {  616, 4710,    2851, 11, }, // Asia/Vladivostok
    {  616, 4828,   37183,  6, }, // Asia/Yakutsk
    {  616, 1235,   56541,  5, }, // Asia/Yangon
    {  616, 3883,    2875, 12, }, // Asia/Yekaterinburg
    {  616, 3747,    2887,  6, }, // Asia/Yerevan
    {  616, 3663,  186291, 14, }, // Atlantic/Azores
    {  616, 5671,  186305, 16, }, // Atlantic/Bermuda
    {  616, 6628,  186321, 15, }, // Atlantic/Canary
    {  616, 3727,    2944, 10, }, // Atlantic/Cape_Verde
    {  616, 1395,  186336, 15, }, // Atlantic/Faroe
    {  616, 8736,   37264,  7, }, // Atlantic/Madeira
    {  616, 2748,   52214,  9, }, // Atlantic/Reykjavik
    {  616, 7272,  186351, 15, }, // Atlantic/South_Georgia
    {  616, 6821,  186366, 10, }, // Atlantic/St_Helena
    {  616, 6644,   52256,  6, }, // Atlantic/Stanley
    {  616, 1670,    3035,  8, }, // Australia/Adelaide
    {  616, 1635,    3043,  7, }, // Australia/Brisbane
    {  616, 1799,    3050, 11, }, // Australia/Broken_Hill
    {  616, 1583,    3061,  6, }, // Australia/Darwin
    {  616, 3637,   29240,  4, }, // Australia/Eucla
    {  616, 1516,    3072,  6, }, // Australia/Hobart
    {  616, 8753,    3078,  8, }, // Australia/Lindeman
    {  616, 1547,   37322,  8, }, // Australia/Lord_Howe
    {  616, 1727,   29252,  8, }, // Australia/Melbourne
    {  616, 1762,    3101,  4, }, // Australia/Perth
    {  616, 1463,   52276,  6, }, // Australia/Sydney
    {  616, 6441,    3111,  9, }, // Europe/Amsterdam
    {  616, 6216,    3120,  7, }, // Europe/Andorra
    {  616, 3620,  186376,  8, }, // Europe/Astrakhan
    {  616, 2300,    3135,  4, }, // Europe/Athens
    {  616, 6336,    3139,  7, }, // Europe/Belgrade
    {  616, 4757,    3146,  6, }, // Europe/Berlin
    {  616, 6488,    3152, 10, }, // Europe/Bratislava
    {  616, 2167,   52296,  8, }, // Europe/Brussels
    {  616, 3928,    3170,  8, }, // Europe/Bucharest
    {  616, 3806,    3178,  8, }, // Europe/Budapest
    {  616, 8772,   37368,  8, }, // Europe/Busingen
    {  616, 2583,    3209,  7, }, // Europe/Chisinau
    {  616, 6289,   52323, 10, }, // Europe/Copenhagen
    {  616, 2338,    3226,  6, }, // Europe/Dublin
    {  616, 6307,    3232,  9, }, // Europe/Gibraltar
    {  616, 8788,    3241,  6, }, // Europe/Guernsey
    {  616, 6612,   52333,  9, }, // Europe/Helsinki
    {  616, 8804,  186384, 10, }, // Europe/Isle_of_Man
    {  616, 1130,   35484,  8, }, // Europe/Istanbul
    {  616, 8823,   52351,  6, }, // Europe/Jersey
    {  616, 3979,    3276, 11, }, // Europe/Kaliningrad
    {  616, 8929,    3287,  5, }, // Europe/Kirov
    {  616, 2527,    3292,  4, }, // Europe/Kyiv
    {  616, 3213,    3296,  8, }, // Europe/Lisbon
    {  616, 6506,   29432,  7, }, // Europe/Ljubljana
    {  616, 2501,    3313,  6, }, // Europe/London
    {  616, 6365,   29445, 10, }, // Europe/Luxembourg
    {  616, 4892,    3330,  6, }, // Europe/Madrid
    {  616, 6397,   29461,  6, }, // Europe/Malta
    {  616, 6582,    3341,  9, }, // Europe/Mariehamn
    {  616, 3693,    3350,  5, }, // Europe/Minsk
    {  616, 6410,    3355,  6, }, // Europe/Monaco
    {  616, 3537,  186394,  6, }, // Europe/Moscow
    {  616, 6458,    3367,  4, }, // Europe/Oslo
    {  616, 4284,    3371,  5, }, // Europe/Paris
    {  616, 6424,    3376,  9, }, // Europe/Podgorica
    {  616, 6275,    3385,  5, }, // Europe/Prague
    {  616, 8837,    3390,  4, }, // Europe/Riga
    {  616, 6324,    3394,  3, }, // Europe/Rome
    {  616, 4331,    3397,  6, }, // Europe/Samara
    {  616, 6470,    3403, 10, }, // Europe/San_Marino
    {  616, 6245,    3413,  7, }, // Europe/Sarajevo
    {  616, 4451,    3420,  7, }, // Europe/Saratov
    {  616, 8849,   52357, 11, }, // Europe/Simferopol
    {  616, 6383,   52368,  6, }, // Europe/Skopje
    {  616, 6599,   37418,  5, }, // Europe/Sofia
    {  616, 6523,   52374,  9, }, // Europe/Stockholm
    {  616, 8867,    3455,  6, }, // Europe/Tallinn
    {  616, 6202,    3461,  5, }, // Europe/Tirane
    {  616, 8882,   52383,  9, }, // Europe/Ulyanovsk
    {  616, 6352,    3476,  5, }, // Europe/Vaduz
    {  616, 6567,    3481,  7, }, // Europe/Vatican
    {  616, 6231,    3488,  4, }, // Europe/Vienna
    {  616, 8899,   52392,  7, }, // Europe/Vilnius
    {  616, 4727,    3499,  9, }, // Europe/Volgograd
    {  616, 3190,    3508,  7, }, // Europe/Warsaw
    {  616, 6261,    3515,  6, }, // Europe/Zagreb
    {  616, 6540,   37442,  5, }, // Europe/Zurich
    {  616, 5078,    3527, 12, }, // Indian/Antananarivo
    {  616, 6947,    3539,  5, }, // Indian/Chagos
    {  616, 6046,  186400,  6, }, // Indian/Christmas
    {  616, 6063,    3556,  5, }, // Indian/Cocos
    {  616, 5029,   35642,  6, }, // Indian/Comoro
    {  616, 6661,    3578,  8, }, // Indian/Kerguelen
    {  616, 7260,   29694,  3, }, // Indian/Mahe
    {  616, 7050,  186406, 17, }, // Indian/Maldives
    {  616, 4069,   56708,  8, }, // Indian/Mauritius
    {  616, 5098,   56716,  7, }, // Indian/Mayotte
    {  616, 7211,   52480,  7, }, // Indian/Reunion
    {  616, 4422,    3616,  4, }, // Pacific/Apia
    {  616,  932,    3620,  6, }, // Pacific/Auckland
    {  616, 3706,   52487,  9, }, // Pacific/Bougainville
    {  616, 3015,    3634,  5, }, // Pacific/Chatham
    {  616, 3157,  118798,  4, }, // Pacific/Chuuk
    {  616, 2237,  186423, 12, }, // Pacific/Easter
    {  616, 7360,   52541,  5, }, // Pacific/Efate
    {  616, 7327,    3659,  7, }, // Pacific/Fakaofo
    {  616, 3902,   37539,  5, }, // Pacific/Fiji
    {  616, 7343,    3670,  8, }, // Pacific/Funafuti
    {  616, 6678,  186435, 19, }, // Pacific/Galapagos
    {  616, 6696,   52567,  6, }, // Pacific/Gambier
    {  616, 3822,    3713, 11, }, // Pacific/Guadalcanal
    {  616, 6018,    3724,  4, }, // Pacific/Guam
    {  616, 3049,    3728,  6, }, // Pacific/Kanton
    {  616, 3998,    3734, 10, }, // Pacific/Kiritimati
    {  616, 7019,  186454,  6, }, // Pacific/Kosrae
    {  616, 2842,   29841, 10, }, // Pacific/Kwajalein
    {  616, 4877,   37574,  7, }, // Pacific/Majuro
    {  616, 4051,  186460, 16, }, // Pacific/Marquesas
    {  616, 8914,   52622,  6, }, // Pacific/Midway
    {  616, 7084,    3798,  5, }, // Pacific/Nauru
    {  616, 7132,   52636,  4, }, // Pacific/Niue
    {  616, 4168,    3807,  7, }, // Pacific/Norfolk
    {  616, 7098,    3814,  5, }, // Pacific/Noumea
    {  616, 3126,   35774,  9, }, // Pacific/Pago_Pago
    {  616, 7155,    3828,  5, }, // Pacific/Palau
    {  616, 7194,   56789,  7, }, // Pacific/Pitcairn
    {  616, 3096,    3840,  6, }, // Pacific/Pohnpei
    {  616, 4807,    3846, 11, }, // Pacific/Port_Moresby
    {  616, 6076,    3857,  9, }, // Pacific/Rarotonga
    {  616, 6031,   29935,  6, }, // Pacific/Saipan
    {  616, 7312,    3872,  5, }, // Pacific/Tahiti
    {  616, 6712,    3877,  6, }, // Pacific/Tarawa
    {  616, 4583,    3883,  9, }, // Pacific/Tongatapu
    {  616, 7392,  182063,  4, }, // Pacific/Wake
    {  616, 7405,    3896,  6, }, // Pacific/Wallis
    {  617, 6788,  186476,  9, }, // Africa/Abidjan Telugu/Telugu/India
    {  617, 6760,  186485,  5, }, // Africa/Accra
    {  617, 5059,  186490, 12, }, // Africa/Addis_Ababa
    {  617, 7420,  186502, 10, }, // Africa/Algiers
    {  617,   14,  186512,  7, }, // Africa/Asmara
    {  617,   44,  186519,  6, }, // Africa/Bamako
    {  617, 5257,  186525,  7, }, // Africa/Bangui
    {  617, 6746,  186532,  6, }, // Africa/Banjul
    {  617, 7435,  186538,  8, }, // Africa/Bissau
    {  617, 4957,  186546, 10, }, // Africa/Blantyre
    {  617, 5287,  186556, 12, }, // Africa/Brazzaville
    {  617, 4922,  186568,  9, }, // Africa/Bujumbura
    {  617, 2320,  186577,  4, }, // Africa/Cairo
    {  617, 4117,  186581, 11, }, // Africa/Casablanca
    {  617, 7449,  186592,  6, }, // Africa/Ceuta
    {  617, 6773,  186598,  8, }, // Africa/Conakry
    {  617, 6840,  186606,  5, }, // Africa/Dakar
    {  617, 5130,  186611, 14, }, // Africa/Dar_es_Salaam
    {  617, 5043,  186625,  8, }, // Africa/Djibouti
    {  617, 5243,  186633,  4, }, // Africa/Douala
    {  617, 5166,  186637,  9, }, // Africa/El_Aaiun
    {  617, 6853,  186646,  8, }, // Africa/Freetown
    {  617, 4906,  186654,  7, }, // Africa/Gaborone
    {  617, 5015,  186661,  5, }, // Africa/Harare
    {  617, 4479,  186666, 14, }, // Africa/Johannesburg
    {  617, 4499,  186680,  4, }, // Africa/Juba
    {  617, 5151,  186684,  6, }, // Africa/Kampala
    {  617, 4524,  186690,  8, }, // Africa/Khartoum
    {  617, 4973,  186698,  5, }, // Africa/Kigali
    {  617, 5306,  186703,  8, }, // Africa/Kinshasa
    {  617, 4744,  186711,  6, }, // Africa/Lagos
    {  617, 5336,  186717, 11, }, // Africa/Libreville
    {  617, 6869,  186728,  4, }, // Africa/Lome
    {  617, 5211,  186732,  7, }, // Africa/Luanda
    {  617, 4939,  186739,  8, }, // Africa/Lubumbashi
    {  617, 5001,  186747,  6, }, // Africa/Lusaka
    {  617, 5322,  186753,  5, }, // Africa/Malabo
    {  617, 4987,  186758,  6, }, // Africa/Maputo
    {  617, 5197,  186764,  5, }, // Africa/Maseru
    {  617, 5182,  186769,  6, }, // Africa/Mbabane
    {  617, 5113,  186775,  8, }, // Africa/Mogadishu
    {  617, 7034,  186783, 10, }, // Africa/Monrovia
    {  617, 3853,  186793,  6, }, // Africa/Nairobi
    {  617, 5271,  186799,  8, }, // Africa/Ndjamena
    {  617, 5354,  186807,  6, }, // Africa/Niamey
    {  617, 6803,  186813, 10, }, // Africa/Nouakchott
    {  617, 6727,  186823,  9, }, // Africa/Ouagadougou
    {  617, 5225,  186832, 11, }, // Africa/Porto-Novo
    {  617, 4435,  186843,  9, }, // Africa/Sao_Tome
    {  617, 2866,  186852,  8, }, // Africa/Tripoli
    {  617, 6554,  186860,  8, }, // Africa/Tunis
    {  617, 4152,  186868,  9, }, // Africa/Windhoek
    {  617,  132,  186877,  5, }, // America/Adak
    {  617, 3346,  186882,  8, }, // America/Anchorage
    {  617, 5607,  186890, 10, }, // America/Anguilla
    {  617, 5624,  186900,  8, }, // America/Antigua
    {  617, 4554,  186908, 11, }, // America/Araguaina
    {  617,  166,  186919, 16, }, // America/Argentina/Buenos_Aires
    {  617,   91,  186935,  8, }, // America/Argentina/Catamarca
    {  617,  270,  186943,  8, }, // America/Argentina/Cordoba
    {  617,  441,  186951,  6, }, // America/Argentina/Jujuy
    {  617, 7462,  186957,  8, }, // America/Argentina/La_Rioja
    {  617,  565,  186965,  7, }, // America/Argentina/Mendoza
    {  617, 7489,  186972, 14, }, // America/Argentina/Rio_Gallegos
    {  617, 7520,  186986,  6, }, // America/Argentina/Salta
    {  617, 7544,  186992, 11, }, // America/Argentina/San_Juan
    {  617, 5580,  187003, 11, }, // America/Argentina/San_Luis
    {  617, 7571,  187014,  7, }, // America/Argentina/Tucuman
    {  617, 7597,  187021,  6, }, // America/Argentina/Ushuaia
    {  617, 5640,  187027,  5, }, // America/Aruba
    {  617, 4252,  187032, 11, }, // America/Asuncion
    {  617,  237,  187043,  8, }, // America/Atikokan
    {  617, 3679,  187051,  4, }, // America/Bahia
    {  617, 7623,  187055, 13, }, // America/Bahia_Banderas
    {  617, 5654,  187068,  9, }, // America/Barbados
    {  617, 7646,  187077,  6, }, // America/Belem
    {  617, 5407,  187083,  6, }, // America/Belize
    {  617, 7660,  187089, 15, }, // America/Blanc-Sablon
    {  617, 7681,  187104, 11, }, // America/Boa_Vista
    {  617, 4361,  187115,  5, }, // America/Bogota
    {  617, 7699,  187120,  6, }, // America/Boise
    {  617, 7713,  187126, 14, }, // America/Cambridge_Bay
    {  617, 7735,  187140, 13, }, // America/Campo_Grande
    {  617, 3868,  187153,  8, }, // America/Cancun
    {  617, 4694,  187161,  7, }, // America/Caracas
    {  617, 4345,  187168,  5, }, // America/Cayenne
    {  617, 5496,  187173,  6, }, // America/Cayman
    {  617, 2260,  187179,  6, }, // America/Chicago
    {  617, 7756,  187185,  6, }, // America/Chihuahua
    {  617, 7774,  187191, 17, }, // America/Ciudad_Juarez
    {  617, 5422,  187208, 11, }, // America/Costa_Rica
    {  617, 7796,  187219,  9, }, // America/Creston
    {  617, 3791,  187228,  5, }, // America/Cuiaba
    {  617, 5723,  187233,  7, }, // America/Curacao
    {  617, 7812,  187240, 15, }, // America/Danmarkshavn
    {  617, 7833,  187255,  5, }, // America/Dawson
    {  617, 7848,  187260, 12, }, // America/Dawson_Creek
    {  617,  805,  187272,  8, }, // America/Denver
    {  617, 3465,  187280, 10, }, // America/Detroit
    {  617, 5739,  187290,  8, }, // America/Dominica
    {  617,  893,  187298,  9, }, // America/Edmonton
    {  617, 7869,  187307,  7, }, // America/Eirunepe
    {  617, 5441,  187314, 13, }, // America/El_Salvador
    {  617, 7886,  187327, 14, }, // America/Fort_Nelson
    {  617, 7906,  187341,  9, }, // America/Fortaleza
    {  617, 7924,  187350,  9, }, // America/Glace_Bay
    {  617, 6881,  187359,  7, }, // America/Goose_Bay
    {  617, 4612,  187366, 13, }, // America/Grand_Turk
    {  617, 5770,  187379,  7, }, // America/Grenada
    {  617, 5786,  187386, 10, }, // America/Guadeloupe
    {  617, 3760,  187396, 10, }, // America/Guatemala
    {  617, 4841,  187406,  9, }, // America/Guayaquil
    {  617, 6932,  187415,  5, }, // America/Guyana
    {  617, 1939,  187420, 10, }, // America/Halifax
    {  617, 2281,  187430,  5, }, // America/Havana
    {  617, 7942,  187435, 12, }, // America/Hermosillo
    {  617,  348,  187447, 13, }, // America/Indiana/Indianapolis
    {  617,  481,  187460, 15, }, // America/Indiana/Knox
    {  617, 7961,  187475, 15, }, // America/Indiana/Marengo
    {  617, 7985,  187490, 22, }, // America/Indiana/Petersburg
    {  617, 8012,  187512, 18, }, // America/Indiana/Tell_City
    {  617, 8038,  187530, 14, }, // America/Indiana/Vevay
    {  617, 8060,  187544, 22, }, // America/Indiana/Vincennes
    {  617, 8086,  187566, 17, }, // America/Indiana/Winamac
    {  617, 8110,  187583,  7, }, // America/Inuvik
    {  617,  660,  187590,  9, }, // America/Iqaluit
    {  617, 2799,  187599,  5, }, // America/Jamaica
    {  617, 5380,  187604,  4, }, // America/Juneau
    {  617,  521,  187608,  8, }, // America/Kentucky/Louisville
    {  617, 8125,  187616, 24, }, // America/Kentucky/Monticello
    {  617, 5704,  187640, 12, }, // America/Kralendijk
    {  617, 4376,  187652,  7, }, // America/La_Paz
    {  617, 7169,  187659,  4, }, // America/Lima
    {  617, 3239,  187663, 12, }, // America/Los_Angeles
    {  617, 5932,  187675, 24, }, // America/Lower_Princes
    {  617, 8153,  187699,  6, }, // America/Maceio
    {  617, 8168,  187705,  7, }, // America/Managua
    {  617, 1908,  187712,  5, }, // America/Manaus
    {  617, 5897,  187717,  8, }, // America/Marigot
    {  617, 5805,  187725, 10, }, // America/Martinique
    {  617, 8184,  187735,  9, }, // America/Matamoros
    {  617, 2917,  187744, 10, }, // America/Mazatlan
    {  617, 8202,  187754,  8, }, // America/Menominee
    {  617, 8220,  187762,  6, }, // America/Merida
    {  617, 8235,  187768,  9, }, // America/Metlakatla
    {  617, 2949,  187777, 13, }, // America/Mexico_City
    {  617, 4391,  187790,  8, }, // America/Miquelon
    {  617, 8254,  187798, 11, }, // America/Moncton
    {  617, 8270,  187809,  9, }, // America/Monterrey
    {  617, 4098,  187818, 11, }, // America/Montevideo
    {  617, 5824,  187829, 11, }, // America/Montserrat
    {  617, 5481,  187840,  8, }, // America/Nassau
    {  617, 2379,  187848, 10, }, // America/New_York
    {  617, 8288,  187858,  4, }, // America/Nome
    {  617, 1850,  187862,  7, }, // America/Noronha
    {  617, 8301,  187869, 18, }, // America/North_Dakota/Beulah
    {  617, 8329,  187887, 19, }, // America/North_Dakota/Center
    {  617, 8357,  187906, 22, }, // America/North_Dakota/New_Salem
    {  617,  393,  187928,  4, }, // America/Nuuk
    {  617, 8388,  187932,  5, }, // America/Ojinaga
    {  617, 2356,  187937,  5, }, // America/Panama
    {  617, 6173,  187942,  9, }, // America/Paramaribo
    {  617, 2973,  187951,  8, }, // America/Phoenix
    {  617, 3945,  187959, 19, }, // America/Port-au-Prince
    {  617, 5954,  187978, 19, }, // America/Port_of_Spain
    {  617, 8404,  187997, 13, }, // America/Porto_Velho
    {  617, 5843,  188010, 13, }, // America/Puerto_Rico
    {  617, 4030,  188023, 12, }, // America/Punta_Arenas
    {  617, 8424,  188035, 17, }, // America/Rankin_Inlet
    {  617, 8445,  188052,  6, }, // America/Recife
    {  617, 1995,  188058,  6, }, // America/Regina
    {  617, 8460,  188064,  9, }, // America/Resolute
    {  617,  695,  188073, 12, }, // America/Rio_Branco
    {  617, 8477,  188085,  8, }, // America/Santarem
    {  617, 2201,  188093,  9, }, // America/Santiago
    {  617, 6111,  188102, 13, }, // America/Santo_Domingo
    {  617, 1878,  188115,  9, }, // America/Sao_Paulo
    {  617, 6899,  188124, 17, }, // America/Scoresbysund
    {  617, 8494,  188141,  6, }, // America/Sitka
    {  617, 8508,  188147, 17, }, // America/St_Barthelemy
    {  617, 2061,  188164, 14, }, // America/St_Johns
    {  617, 5863,  188178, 14, }, // America/St_Kitts
    {  617, 5880,  188192, 14, }, // America/St_Lucia
    {  617,  855,  188206, 13, }, // America/St_Thomas
    {  617, 5913,  188219, 17, }, // America/St_Vincent
    {  617, 8530,  188236, 15, }, // America/Swift_Current
    {  617, 5461,  188251, 11, }, // America/Tegucigalpa
    {  617, 5756,  188262,  4, }, // America/Thule
    {  617,  313,  188266,  8, }, // America/Tijuana
    {  617,  608,  188274,  6, }, // America/Toronto
    {  617, 5688,  188280,  8, }, // America/Tortola
    {  617, 2093,  188288,  9, }, // America/Vancouver
    {  617, 2144,  188297, 11, }, // America/Whitehorse
    {  617,  734,  188308, 10, }, // America/Winnipeg
    {  617, 8552,  188318,  7, }, // America/Yakutat
    {  617, 6001,  188325,  4, }, // Antarctica/Casey
    {  617, 6094,  188329,  6, }, // Antarctica/Davis
    {  617, 6133,  188335, 22, }, // Antarctica/DumontDUrville
    {  617, 8568,  188357,  5, }, // Antarctica/Macquarie
    {  617, 7066,  188362,  5, }, // Antarctica/Mawson
    {  617, 7113,  188367, 11, }, // Antarctica/McMurdo
    {  617, 8589,  188378,  7, }, // Antarctica/Palmer
    {  617, 7226,  188385,  6, }, // Antarctica/Rothera
    {  617, 7295,  188391,  6, }, // Antarctica/Syowa
    {  617, 8607,  188397,  6, }, // Antarctica/Troll
    {  617, 7374,  188403,  8, }, // Antarctica/Vostok
    {  617, 1429,  188411, 17, }, // Arctic/Longyearbyen
    {  617, 5570,  188428,  5, }, // Asia/Aden
    {  617, 5395,  188433,  7, }, // Asia/Almaty
    {  617, 3968,  188440,  6, }, // Asia/Amman
    {  617, 5511,  188446,  6, }, // Asia/Anadyr
    {  617, 5523,  188452,  7, }, // Asia/Aqtau
    {  617, 5368,  188459,  7, }, // Asia/Aqtobe
    {  617,  964,  188466, 10, }, // Asia/Ashgabat
    {  617, 8624,  188476,  5, }, // Asia/Atyrau
    {  617, 3607,  188481,  8, }, // Asia/Baghdad
    {  617, 5534,  188489,  9, }, // Asia/Bahrain
    {  617, 3653,  187231,  4, }, // Asia/Baku
    {  617, 4466,  188498,  9, }, // Asia/Bangkok
    {  617, 3571,  188507,  8, }, // Asia/Barnaul
    {  617, 4086,  188515,  5, }, // Asia/Beirut
    {  617, 3778,  188520,  8, }, // Asia/Bishkek
    {  617, 5989,  188528,  6, }, // Asia/Brunei
    {  617, 4601,  188534,  4, }, // Asia/Chita
    {  617, 4511,  188538,  6, }, // Asia/Colombo
    {  617, 4540,  188544,  8, }, // Asia/Damascus
    {  617, 1093,  188552,  4, }, // Asia/Dhaka
    {  617, 6192,  188556,  4, }, // Asia/Dili
    {  617, 3596,  188560,  6, }, // Asia/Dubai
    {  617, 6159,  188566,  7, }, // Asia/Dushanbe
    {  617, 8636,  188573,  9, }, // Asia/Famagusta
    {  617, 8651,  188582,  4, }, // Asia/Gaza
    {  617, 4795,  188586,  8, }, // Asia/Hebron
    {  617, 1259,  188594, 17, }, // Asia/Ho_Chi_Minh
    {  617, 2704,  188611,  8, }, // Asia/Hong_Kong
    {  617, 4771,  188619,  6, }, // Asia/Hovd
    {  617, 4184,  188625, 11, }, // Asia/Irkutsk
    {  617, 7006,  188636,  7, }, // Asia/Jakarta
    {  617, 6992,  188643,  5, }, // Asia/Jayapura
    {  617, 1290,  188648,  7, }, // Asia/Jerusalem
    {  617, 3560,  188655,  6, }, // Asia/Kabul
    {  617, 4316,  188661,  9, }, // Asia/Kamchatka
    {  617, 4239,  188670,  5, }, // Asia/Karachi
    {  617, 1185,  188675,  8, }, // Asia/Kathmandu
    {  617, 8661,  188683,  7, }, // Asia/Khandyga
    {  617,  992,  188690,  8, }, // Asia/Kolkata
    {  617, 4197,  188698, 14, }, // Asia/Krasnoyarsk
    {  617, 4859,  188712, 10, }, // Asia/Kuala_Lumpur
    {  617, 5976,  188722,  7, }, // Asia/Kuching
    {  617, 5547,  188729,  6, }, // Asia/Kuwait
    {  617, 1211,  188735,  5, }, // Asia/Macau
    {  617, 4017,  188740,  6, }, // Asia/Magadan
    {  617, 1349,  188746,  8, }, // Asia/Makassar
    {  617, 7182,  188754,  5, }, // Asia/Manila
    {  617, 6920,  188759,  6, }, // Asia/Muscat
    {  617, 2554,  188765,  8, }, // Asia/Nicosia
    {  617, 8675,  188773, 18, }, // Asia/Novokuznetsk
    {  617, 4135,  188791, 13, }, // Asia/Novosibirsk
    {  617, 4229,  188804,  7, }, // Asia/Omsk
    {  617, 7145,  188811,  4, }, // Asia/Oral
    {  617, 6961,  188815, 11, }, // Asia/Phnom_Penh
    {  617, 8693,  188826, 11, }, // Asia/Pontianak
    {  617, 4214,  188837, 10, }, // Asia/Pyongyang
    {  617, 5559,  188847,  5, }, // Asia/Qatar
    {  617, 8708,  188852,  8, }, // Asia/Qostanay
    {  617, 4269,  188860, 12, }, // Asia/Qyzylorda
    {  617, 3584,  188872,  6, }, // Asia/Riyadh
    {  617, 4408,  188878,  7, }, // Asia/Sakhalin
    {  617, 7245,  188885,  9, }, // Asia/Samarkand
    {  617, 3279,  188894,  6, }, // Asia/Seoul
    {  617, 1053,  188900,  5, }, // Asia/Shanghai
    {  617, 3300,  188905,  8, }, // Asia/Singapore
    {  617, 4297,  188913, 18, }, // Asia/Srednekolymsk
    {  617, 3263,  188931,  4, }, // Asia/Taipei
    {  617, 4781,  188935,  9, }, // Asia/Tashkent
    {  617, 3915,  188944,  8, }, // Asia/Tbilisi
    {  617, 2772,  188952,  8, }, // Asia/Tehran
    {  617, 1317,  188960,  5, }, // Asia/Thimphu
    {  617, 2821,  188965,  6, }, // Asia/Tokyo
    {  617, 4572,  188971,  8, }, // Asia/Tomsk
    {  617, 1021,  188979, 10, }, // Asia/Ulaanbaatar
    {  617, 1159,  188989,  8, }, // Asia/Urumqi
    {  617, 8722,  188997, 10, }, // Asia/Ust-Nera
    {  617, 6977,  189007,  9, }, // Asia/Vientiane
    {  617, 4710,  189016, 14, }, // Asia/Vladivostok
    {  617, 4828,  189030,  9, }, // Asia/Yakutsk
    {  617, 1235,  189039,  6, }, // Asia/Yangon
    {  617, 3883,  189045, 13, }, // Asia/Yekaterinburg
    {  617, 3747,  189058,  7, }, // Asia/Yerevan
    {  617, 3663,  189065,  7, }, // Atlantic/Azores
    {  617, 5671,  189072,  8, }, // Atlantic/Bermuda
    {  617, 6628,  189080,  5, }, // Atlantic/Canary
    {  617, 3727,  189085, 11, }, // Atlantic/Cape_Verde
    {  617, 1395,  189096,  4, }, // Atlantic/Faroe
    {  617, 8736,  189100,  5, }, // Atlantic/Madeira
    {  617, 2748,  189105, 10, }, // Atlantic/Reykjavik
    {  617, 7272,  189115, 15, }, // Atlantic/South_Georgia
    {  617, 6821,  189130, 14, }, // Atlantic/St_Helena
    {  617, 6644,  189144,  8, }, // Atlantic/Stanley
    {  617, 1670,  189152,  7, }, // Australia/Adelaide
    {  617, 1635,  189159, 13, }, // Australia/Brisbane
    {  617, 1799,  189172, 13, }, // Australia/Broken_Hill
    {  617, 1583,  189185,  8, }, // Australia/Darwin
    {  617, 3637,  189193,  6, }, // Australia/Eucla
    {  617, 1516,  189199,  7, }, // Australia/Hobart
    {  617, 8753,  189206,  9, }, // Australia/Lindeman
    {  617, 1547,  189215,  9, }, // Australia/Lord_Howe
    {  617, 1727,  189224, 10, }, // Australia/Melbourne
    {  617, 1762,  189234,  6, }, // Australia/Perth
    {  617, 1463,  189240,  6, }, // Australia/Sydney
    {  617, 6441,  189246, 14, }, // Europe/Amsterdam
    {  617, 6216,  189260,  6, }, // Europe/Andorra
    {  617, 3620,  189266, 10, }, // Europe/Astrakhan
    {  617, 2300,  189276,  7, }, // Europe/Athens
    {  617, 6336,  189283, 11, }, // Europe/Belgrade
    {  617, 4757,  189294,  8, }, // Europe/Berlin
    {  617, 6488,  189302, 12, }, // Europe/Bratislava
    {  617, 2167,  189314, 11, }, // Europe/Brussels
    {  617, 3928,  189325, 10, }, // Europe/Bucharest
    {  617, 3806,  189335, 10, }, // Europe/Budapest
    {  617, 8772,  189345,  7, }, // Europe/Busingen
    {  617, 2583,  189352,  8, }, // Europe/Chisinau
    {  617, 6289,  189360, 11, }, // Europe/Copenhagen
    {  617, 2338,  189371,  7, }, // Europe/Dublin
    {  617, 6307,  189378, 11, }, // Europe/Gibraltar
    {  617, 8788,  189389, 10, }, // Europe/Guernsey
    {  617, 6612,  189399,  9, }, // Europe/Helsinki
    {  617, 8804,  189408, 12, }, // Europe/Isle_of_Man
    {  617, 1130,  189420, 10, }, // Europe/Istanbul
    {  617, 8823,  189430,  6, }, // Europe/Jersey
    {  617, 3979,  189436, 13, }, // Europe/Kaliningrad
    {  617, 8929,  189449,  6, }, // Europe/Kirov
    {  617, 2527,  189455,  4, }, // Europe/Kyiv
    {  617, 3213,  189459,  7, }, // Europe/Lisbon
    {  617, 6506,  189466, 12, }, // Europe/Ljubljana
    {  617, 2501,  189478,  5, }, // Europe/London
    {  617, 6365,  189483, 11, }, // Europe/Luxembourg
    {  617, 4892,  189494,  8, }, // Europe/Madrid
    {  617, 6397,  189502,  6, }, // Europe/Malta
    {  617, 6582,  189508,  9, }, // Europe/Mariehamn
    {  617, 3693,  189517,  8, }, // Europe/Minsk
    {  617, 6410,  189525,  6, }, // Europe/Monaco
    {  617, 3537,  189531,  6, }, // Europe/Moscow
    {  617, 6458,  189537,  5, }, // Europe/Oslo
    {  617, 4284,  189542,  8, }, // Europe/Paris
    {  617, 6424,  189550, 10, }, // Europe/Podgorica
    {  617, 6275,  189560,  6, }, // Europe/Prague
    {  617, 8837,  189566,  4, }, // Europe/Riga
    {  617, 6324,  189570,  4, }, // Europe/Rome
    {  617, 4331,  189574,  4, }, // Europe/Samara
    {  617, 6470,  189578, 11, }, // Europe/San_Marino
    {  617, 6245,  189589,  7, }, // Europe/Sarajevo
    {  617, 4451,  189596,  6, }, // Europe/Saratov
    {  617, 8849,  189602, 13, }, // Europe/Simferopol
    {  617, 6383,  189615,  9, }, // Europe/Skopje
    {  617, 6599,  189624,  6, }, // Europe/Sofia
    {  617, 6523,  189630, 10, }, // Europe/Stockholm
    {  617, 8867,  189640,  8, }, // Europe/Tallinn
    {  617, 6202,  189648,  6, }, // Europe/Tirane
    {  617, 8882,  189654, 12, }, // Europe/Ulyanovsk
    {  617, 6352,  189666,  6, }, // Europe/Vaduz
    {  617, 6567,  189672,  7, }, // Europe/Vatican
    {  617, 6231,  189679,  7, }, // Europe/Vienna
    {  617, 8899,  189686,  9, }, // Europe/Vilnius
    {  617, 4727,  189695, 12, }, // Europe/Volgograd
    {  617, 3190,  189707,  6, }, // Europe/Warsaw
    {  617, 6261,  189713,  8, }, // Europe/Zagreb
    {  617, 6540,  189721,  8, }, // Europe/Zurich
    {  617, 5078,  189729, 12, }, // Indian/Antananarivo
    {  617, 6947,  189741,  6, }, // Indian/Chagos
    {  617, 6046,  189747,  9, }, // Indian/Christmas
    {  617, 6063,  189756,  6, }, // Indian/Cocos
    {  617, 5029,  189762,  6, }, // Indian/Comoro
    {  617, 6661,  189768, 12, }, // Indian/Kerguelen
    {  617, 7260,  189780,  4, }, // Indian/Mahe
    {  617, 7050,  189784, 10, }, // Indian/Maldives
    {  617, 4069,  189794,  7, }, // Indian/Mauritius
    {  617, 5098,  189801,  5, }, // Indian/Mayotte
    {  617, 7211,  189806,  9, }, // Indian/Reunion
    {  617, 4422,  189815,  5, }, // Pacific/Apia
    {  617,  932,  189820,  8, }, // Pacific/Auckland
    {  617, 3706,  189828, 15, }, // Pacific/Bougainville
    {  617, 3015,  189843,  5, }, // Pacific/Chatham
    {  617, 3157,  189848,  4, }, // Pacific/Chuuk
    {  617, 2237,  189852,  6, }, // Pacific/Easter
    {  617, 7360,  189858,  5, }, // Pacific/Efate
    {  617, 7327,  189863,  6, }, // Pacific/Fakaofo
    {  617, 3902,  189869,  4, }, // Pacific/Fiji
    {  617, 7343,  189873,  8, }, // Pacific/Funafuti
    {  617, 6678,  189881, 10, }, // Pacific/Galapagos
    {  617, 6696,  189891,  9, }, // Pacific/Gambier
    {  617, 3822,  189900, 13, }, // Pacific/Guadalcanal
    {  617, 6018,  189913,  6, }, // Pacific/Guam
    {  617, 2723,  189919,  8, }, // Pacific/Honolulu
    {  617, 3049,  189927,  9, }, // Pacific/Kanton
    {  617, 3998,  189936, 10, }, // Pacific/Kiritimati
    {  617, 7019,  189946,  6, }, // Pacific/Kosrae
    {  617, 2842,  189952, 10, }, // Pacific/Kwajalein
    {  617, 4877,  189962,  5, }, // Pacific/Majuro
    {  617, 4051,  189967, 12, }, // Pacific/Marquesas
    {  617, 8914,  189979,  6, }, // Pacific/Midway
    {  617, 7084,  189985,  4, }, // Pacific/Nauru
    {  617, 7132,  189989,  4, }, // Pacific/Niue
    {  617, 4168,  189993,  8, }, // Pacific/Norfolk
    {  617, 7098,  190001,  6, }, // Pacific/Noumea
    {  617, 3126,  190007,  9, }, // Pacific/Pago_Pago
    {  617, 7155,  190016,  6, }, // Pacific/Palau
    {  617, 7194,  190022, 11, }, // Pacific/Pitcairn
    {  617, 3096,  190033,  7, }, // Pacific/Pohnpei
    {  617, 4807,  190040, 15, }, // Pacific/Port_Moresby
    {  617, 6076,  190055,  8, }, // Pacific/Rarotonga
    {  617, 6031,  190063,  8, }, // Pacific/Saipan
    {  617, 7312,  190071,  5, }, // Pacific/Tahiti
    {  617, 6712,  190076,  5, }, // Pacific/Tarawa
    {  617, 4583,  190081,  9, }, // Pacific/Tongatapu
    {  617, 7392,  190090,  4, }, // Pacific/Wake
    {  617, 7405,  190094,  8, }, // Pacific/Wallis
    {  618,   14,       0,  6, }, // Africa/Asmara Teso/Latin/Uganda
    {  618, 4435,       6,  8, }, // Africa/Sao_Tome
    {  618, 4252,      14,  8, }, // America/Asuncion
    {  618,  237,      22,  8, }, // America/Atikokan
    {  618, 7623,      30, 17, }, // America/Bahia_Banderas
    {  618, 3868,      47,  6, }, // America/Cancun
    {  618, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  618, 5723,      66,  7, }, // America/Curacao
    {  618,  481,      73, 13, }, // America/Indiana/Knox
    {  618, 7961,      86, 16, }, // America/Indiana/Marengo
    {  618, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  618, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  618, 8038,     139, 14, }, // America/Indiana/Vevay
    {  618, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  618, 8086,     171, 16, }, // America/Indiana/Winamac
    {  618, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  618, 5932,     207, 22, }, // America/Lower_Princes
    {  618, 8220,     229,  6, }, // America/Merida
    {  618, 1850,     235, 19, }, // America/Noronha
    {  618, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  618, 8329,     274, 20, }, // America/North_Dakota/Center
    {  618, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  618,  393,     317,  4, }, // America/Nuuk
    {  618, 6899,     321, 16, }, // America/Scoresbysund
    {  618, 8508,     337, 14, }, // America/St_Barthelemy
    {  618, 2061,     351, 10, }, // America/St_Johns
    {  618, 5863,     361,  9, }, // America/St_Kitts
    {  618, 5880,     370,  9, }, // America/St_Lucia
    {  618,  855,     379, 10, }, // America/St_Thomas
    {  618, 5913,     389, 11, }, // America/St_Vincent
    {  618, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  618, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  618, 1185,     448,  9, }, // Asia/Kathmandu
    {  618,  992,     457,  7, }, // Asia/Kolkata
    {  618, 1211,     464,  5, }, // Asia/Macau
    {  618, 1235,     477,  6, }, // Asia/Yangon
    {  618, 1395,     483,  5, }, // Atlantic/Faroe
    {  618, 6821,     488, 10, }, // Atlantic/St_Helena
    {  618, 2527,     514,  4, }, // Europe/Kyiv
    {  618, 7211,     547,  7, }, // Indian/Reunion
    {  618, 3157,     554,  5, }, // Pacific/Chuuk
    {  618, 3096,     586,  7, }, // Pacific/Pohnpei
    {  620, 6788,  190102,  7, }, // Africa/Abidjan Thai/Thai/Thailand
    {  620, 6760,  190109,  6, }, // Africa/Accra
    {  620, 5059,  190115, 12, }, // Africa/Addis_Ababa
    {  620, 7420,  190127,  9, }, // Africa/Algiers
    {  620,   14,  190136,  7, }, // Africa/Asmara
    {  620,   44,  190143,  6, }, // Africa/Bamako
    {  620, 5257,  190149,  5, }, // Africa/Bangui
    {  620, 6746,  190154,  6, }, // Africa/Banjul
    {  620, 7435,  190160,  6, }, // Africa/Bissau
    {  620, 4957,  190166,  8, }, // Africa/Blantyre
    {  620, 5287,  190174,  9, }, // Africa/Brazzaville
    {  620, 4922,  190183,  9, }, // Africa/Bujumbura
    {  620, 2320,  190192,  4, }, // Africa/Cairo
    {  620, 4117,  190196, 12, }, // Africa/Casablanca
    {  620, 7449,  190208,  5, }, // Africa/Ceuta
    {  620, 6773,  190213,  7, }, // Africa/Conakry
    {  620, 6840,  190220,  6, }, // Africa/Dakar
    {  620, 5130,  190226, 12, }, // Africa/Dar_es_Salaam
    {  620, 5043,  190238,  6, }, // Africa/Djibouti
    {  620, 5243,  190244,  6, }, // Africa/Douala
    {  620, 5166,  190250, 10, }, // Africa/El_Aaiun
    {  620, 6853,  190260,  8, }, // Africa/Freetown
    {  620, 4906,  190268,  8, }, // Africa/Gaborone
    {  620, 5015,  190276,  6, }, // Africa/Harare
    {  620, 4479,  190282, 14, }, // Africa/Johannesburg
    {  620, 4499,  190296,  4, }, // Africa/Juba
    {  620, 5151,  190300,  7, }, // Africa/Kampala
    {  620, 4524,  190307,  7, }, // Africa/Khartoum
    {  620, 4973,  190314,  6, }, // Africa/Kigali
    {  620, 5306,  190320,  7, }, // Africa/Kinshasa
    {  620, 4744,  190327,  5, }, // Africa/Lagos
    {  620, 5336,  190332,  9, }, // Africa/Libreville
    {  620, 6869,  190341,  4, }, // Africa/Lome
    {  620, 5211,  190345,  7, }, // Africa/Luanda
    {  620, 4939,  190352,  9, }, // Africa/Lubumbashi
    {  620, 5001,  190361,  6, }, // Africa/Lusaka
    {  620, 5322,  190367,  6, }, // Africa/Malabo
    {  620, 4987,  190373,  6, }, // Africa/Maputo
    {  620, 5197,  190379,  6, }, // Africa/Maseru
    {  620, 5182,  190385,  9, }, // Africa/Mbabane
    {  620, 5113,  190394,  8, }, // Africa/Mogadishu
    {  620, 7034,  190402,  9, }, // Africa/Monrovia
    {  620, 3853,  190411,  8, }, // Africa/Nairobi
    {  620, 5271,  190419, 10, }, // Africa/Ndjamena
    {  620, 5354,  190429,  8, }, // Africa/Niamey
    {  620, 6803,  190437,  8, }, // Africa/Nouakchott
    {  620, 6727,  190445,  8, }, // Africa/Ouagadougou
    {  620, 5225,  190453, 11, }, // Africa/Porto-Novo
    {  620, 4435,  190464,  7, }, // Africa/Sao_Tome
    {  620, 2866,  190471,  7, }, // Africa/Tripoli
    {  620, 6554,  190478,  5, }, // Africa/Tunis
    {  620, 4152,  190483,  8, }, // Africa/Windhoek
    {  620,  132,  190491,  5, }, // America/Adak
    {  620, 3346,  190496,  9, }, // America/Anchorage
    {  620, 5607,  190505,  8, }, // America/Anguilla
    {  620, 5624,  190513,  7, }, // America/Antigua
    {  620, 4554,  190520,  9, }, // America/Araguaina
    {  620,  166,  190529, 11, }, // America/Argentina/Buenos_Aires
    {  620,   91,  190540, 10, }, // America/Argentina/Catamarca
    {  620,  270,  190550,  8, }, // America/Argentina/Cordoba
    {  620,  441,  190558,  5, }, // America/Argentina/Jujuy
    {  620, 7462,  190563,  8, }, // America/Argentina/La_Rioja
    {  620,  565,  190571,  7, }, // America/Argentina/Mendoza
    {  620, 7489,  190578, 12, }, // America/Argentina/Rio_Gallegos
    {  620, 7520,  190590,  5, }, // America/Argentina/Salta
    {  620, 7544,  190595,  6, }, // America/Argentina/San_Juan
    {  620, 5580,  190601,  8, }, // America/Argentina/San_Luis
    {  620, 7571,  190609,  7, }, // America/Argentina/Tucuman
    {  620, 7597,  190616,  8, }, // America/Argentina/Ushuaia
    {  620, 5640,  190624,  6, }, // America/Aruba
    {  620, 4252,  190630, 10, }, // America/Asuncion
    {  620,  237,  190640, 14, }, // America/Atikokan
    {  620, 3679,  190654,  6, }, // America/Bahia
    {  620, 7623,  190660, 14, }, // America/Bahia_Banderas
    {  620, 5654,  190674,  9, }, // America/Barbados
    {  620, 7646,  190683,  5, }, // America/Belem
    {  620, 5407,  190688,  5, }, // America/Belize
    {  620, 7660,  190693, 13, }, // America/Blanc-Sablon
    {  620, 7681,  190706,  8, }, // America/Boa_Vista
    {  620, 4361,  190714,  6, }, // America/Bogota
    {  620, 7699,  190720,  5, }, // America/Boise
    {  620, 7713,  190725, 13, }, // America/Cambridge_Bay
    {  620, 7735,  190738, 11, }, // America/Campo_Grande
    {  620, 3868,  190749,  6, }, // America/Cancun
    {  620, 4694,  190755,  7, }, // America/Caracas
    {  620, 4345,  190762,  5, }, // America/Cayenne
    {  620, 5496,  190767,  7, }, // America/Cayman
    {  620, 2260,  190774,  6, }, // America/Chicago
    {  620, 7756,  190780,  6, }, // America/Chihuahua
    {  620, 7774,  190786, 12, }, // America/Ciudad_Juarez
    {  620, 5422,  190798,  9, }, // America/Costa_Rica
    {  620, 7796,  190807,  7, }, // America/Creston
    {  620, 3791,  190814,  6, }, // America/Cuiaba
    {  620, 5723,  190820,  7, }, // America/Curacao
    {  620, 7812,  190827, 14, }, // America/Danmarkshavn
    {  620, 7833,  190841,  7, }, // America/Dawson
    {  620, 7848,  190848, 12, }, // America/Dawson_Creek
    {  620,  805,  190860,  8, }, // America/Denver
    {  620, 3465,  190868,  8, }, // America/Detroit
    {  620, 5739,  190876,  8, }, // America/Dominica
    {  620,  893,  190884,  9, }, // America/Edmonton
    {  620, 7869,  190893,  8, }, // America/Eirunepe
    {  620, 5441,  190901, 12, }, // America/El_Salvador
    {  620, 7886,  190913, 11, }, // America/Fort_Nelson
    {  620, 7906,  190924, 10, }, // America/Fortaleza
    {  620, 7924,  190934,  8, }, // America/Glace_Bay
    {  620, 6881,  190942,  7, }, // America/Goose_Bay
    {  620, 4612,  190949, 12, }, // America/Grand_Turk
    {  620, 5770,  190961,  7, }, // America/Grenada
    {  620, 5786,  190968,  9, }, // America/Guadeloupe
    {  620, 3760,  190977,  9, }, // America/Guatemala
    {  620, 4841,  190986,  8, }, // America/Guayaquil
    {  620, 6932,  190994,  7, }, // America/Guyana
    {  620, 1939,  191001,  9, }, // America/Halifax
    {  620, 2281,  191010,  6, }, // America/Havana
    {  620, 7942,  191016, 10, }, // America/Hermosillo
    {  620,  348,  191026, 14, }, // America/Indiana/Indianapolis
    {  620,  481,  191040, 16, }, // America/Indiana/Knox
    {  620, 7961,  191056, 18, }, // America/Indiana/Marengo
    {  620, 7985,  191074, 25, }, // America/Indiana/Petersburg
    {  620, 8012,  191099, 20, }, // America/Indiana/Tell_City
    {  620, 8038,  191119, 17, }, // America/Indiana/Vevay
    {  620, 8060,  191136, 20, }, // America/Indiana/Vincennes
    {  620, 8086,  191156, 18, }, // America/Indiana/Winamac
    {  620, 8110,  191174,  7, }, // America/Inuvik
    {  620,  660,  191181,  8, }, // America/Iqaluit
    {  620, 2799,  191189,  6, }, // America/Jamaica
    {  620, 5380,  191195,  4, }, // America/Juneau
    {  620,  521,  191199,  9, }, // America/Kentucky/Louisville
    {  620, 8125,  191208, 20, }, // America/Kentucky/Monticello
    {  620, 5704,  191228, 11, }, // America/Kralendijk
    {  620, 4376,  191239,  5, }, // America/La_Paz
    {  620, 7169,  191244,  4, }, // America/Lima
    {  620, 3239,  191248, 11, }, // America/Los_Angeles
    {  620, 5932,  191259, 22, }, // America/Lower_Princes
    {  620, 8153,  191281,  6, }, // America/Maceio
    {  620, 8168,  191287,  7, }, // America/Managua
    {  620, 1908,  191294,  7, }, // America/Manaus
    {  620, 5897,  191301,  8, }, // America/Marigot
    {  620, 5805,  191309,  9, }, // America/Martinique
    {  620, 8184,  191318,  9, }, // America/Matamoros
    {  620, 2917,  191327,  8, }, // America/Mazatlan
    {  620, 8202,  191335,  8, }, // America/Menominee
    {  620, 8220,  191343,  6, }, // America/Merida
    {  620, 8235,  191349, 10, }, // America/Metlakatla
    {  620, 2949,  191359, 12, }, // America/Mexico_City
    {  620, 4391,  191371,  7, }, // America/Miquelon
    {  620, 8254,  191378,  6, }, // America/Moncton
    {  620, 8270,  191384, 11, }, // America/Monterrey
    {  620, 4098,  191395, 11, }, // America/Montevideo
    {  620, 5824,  191406, 11, }, // America/Montserrat
    {  620, 5481,  191417,  5, }, // America/Nassau
    {  620, 2379,  191422,  8, }, // America/New_York
    {  620, 8288,  191430,  3, }, // America/Nome
    {  620, 1850,  191433,  7, }, // America/Noronha
    {  620, 8301,  191440, 19, }, // America/North_Dakota/Beulah
    {  620, 8329,  191459, 21, }, // America/North_Dakota/Center
    {  620, 8357,  191480, 21, }, // America/North_Dakota/New_Salem
    {  620,  393,  191501,  6, }, // America/Nuuk
    {  620, 8388,  191507,  8, }, // America/Ojinaga
    {  620, 2356,  191515,  6, }, // America/Panama
    {  620, 6173,  191521, 10, }, // America/Paramaribo
    {  620, 2973,  191531,  7, }, // America/Phoenix
    {  620, 3945,  191538, 12, }, // America/Port-au-Prince
    {  620, 5954,  191550, 12, }, // America/Port_of_Spain
    {  620, 8404,  191562, 11, }, // America/Porto_Velho
    {  620, 5843,  191573,  9, }, // America/Puerto_Rico
    {  620, 4030,  191582, 12, }, // America/Punta_Arenas
    {  620, 8424,  191594, 13, }, // America/Rankin_Inlet
    {  620, 8445,  191607,  6, }, // America/Recife
    {  620, 1995,  191613,  6, }, // America/Regina
    {  620, 8460,  191619,  7, }, // America/Resolute
    {  620,  695,  191626, 10, }, // America/Rio_Branco
    {  620, 8477,  191636,  8, }, // America/Santarem
    {  620, 2201,  191644,  9, }, // America/Santiago
    {  620, 6111,  191653, 13, }, // America/Santo_Domingo
    {  620, 1878,  191666,  8, }, // America/Sao_Paulo
    {  620, 6899,  191674, 13, }, // America/Scoresbysund
    {  620, 8494,  191687,  5, }, // America/Sitka
    {  620, 8508,  191692, 15, }, // America/St_Barthelemy
    {  620, 2061,  191707, 10, }, // America/St_Johns
    {  620, 5863,  191717, 10, }, // America/St_Kitts
    {  620, 5880,  191727, 11, }, // America/St_Lucia
    {  620,  855,  191738, 10, }, // America/St_Thomas
    {  620, 5913,  191748, 13, }, // America/St_Vincent
    {  620, 8530,  191761, 16, }, // America/Swift_Current
    {  620, 5461,  191777, 11, }, // America/Tegucigalpa
    {  620, 5756,  191788,  4, }, // America/Thule
    {  620,  313,  191792,  7, }, // America/Tijuana
    {  620,  608,  191799,  7, }, // America/Toronto
    {  620, 5688,  191806,  8, }, // America/Tortola
    {  620, 2093,  191814, 10, }, // America/Vancouver
    {  620, 2144,  191824,  9, }, // America/Whitehorse
    {  620,  734,  191833,  8, }, // America/Winnipeg
    {  620, 8552,  191841,  7, }, // America/Yakutat
    {  620, 6001,  191848,  6, }, // Antarctica/Casey
    {  620, 6094,  191854,  5, }, // Antarctica/Davis
    {  620, 6133,  191859, 16, }, // Antarctica/DumontDUrville
    {  620, 8568,  191875,  8, }, // Antarctica/Macquarie
    {  620, 7066,  191883,  7, }, // Antarctica/Mawson
    {  620, 7113,  191890,  8, }, // Antarctica/McMurdo
    {  620, 8589,  191898,  9, }, // Antarctica/Palmer
    {  620, 7226,  191907,  6, }, // Antarctica/Rothera
    {  620, 7295,  191913,  6, }, // Antarctica/Syowa
    {  620, 8607,  191919,  4, }, // Antarctica/Troll
    {  620, 7374,  191923,  6, }, // Antarctica/Vostok
    {  620, 1429,  191929, 14, }, // Arctic/Longyearbyen
    {  620, 5570,  191943,  5, }, // Asia/Aden
    {  620, 5395,  191948,  7, }, // Asia/Almaty
    {  620, 3968,  191955,  6, }, // Asia/Amman
    {  620, 5511,  191961,  8, }, // Asia/Anadyr
    {  620, 5523,  191969,  7, }, // Asia/Aqtau
    {  620, 5368,  191976,  7, }, // Asia/Aqtobe
    {  620,  964,  191983,  8, }, // Asia/Ashgabat
    {  620, 8624,  191991,  6, }, // Asia/Atyrau
    {  620, 3607,  191997,  6, }, // Asia/Baghdad
    {  620, 5534,  192003,  7, }, // Asia/Bahrain
    {  620, 3653,  192010,  4, }, // Asia/Baku
    {  620, 4466,  192014,  7, }, // Asia/Bangkok
    {  620, 3571,  192021,  9, }, // Asia/Barnaul
    {  620, 4086,  192030,  5, }, // Asia/Beirut
    {  620, 3778,  192035,  6, }, // Asia/Bishkek
    {  620, 5989,  192041,  5, }, // Asia/Brunei
    {  620, 4601,  192046,  4, }, // Asia/Chita
    {  620, 4511,  192050,  7, }, // Asia/Colombo
    {  620, 4540,  192057,  8, }, // Asia/Damascus
    {  620, 1093,  190220,  4, }, // Asia/Dhaka
    {  620, 6192,  192065,  4, }, // Asia/Dili
    {  620, 3596,  192069,  4, }, // Asia/Dubai
    {  620, 6159,  192073,  7, }, // Asia/Dushanbe
    {  620, 8636,  192080,  9, }, // Asia/Famagusta
    {  620, 8651,  192089,  4, }, // Asia/Gaza
    {  620, 4795,  192093,  6, }, // Asia/Hebron
    {  620, 1259,  192099, 12, }, // Asia/Ho_Chi_Minh
    {  620, 2704,  192111,  6, }, // Asia/Hong_Kong
    {  620, 4771,  192117,  5, }, // Asia/Hovd
    {  620, 4184,  192122, 10, }, // Asia/Irkutsk
    {  620, 7006,  192132,  8, }, // Asia/Jakarta
    {  620, 6992,  192140,  8, }, // Asia/Jayapura
    {  620, 1290,  192148,  9, }, // Asia/Jerusalem
    {  620, 3560,  192157,  5, }, // Asia/Kabul
    {  620, 4316,  192162,  8, }, // Asia/Kamchatka
    {  620, 4239,  192170,  6, }, // Asia/Karachi
    {  620, 1185,  192176,  8, }, // Asia/Kathmandu
    {  620, 8661,  192184,  8, }, // Asia/Khandyga
    {  620,  992,  192192,  7, }, // Asia/Kolkata
    {  620, 4197,  192199, 13, }, // Asia/Krasnoyarsk
    {  620, 4859,  192212, 13, }, // Asia/Kuala_Lumpur
    {  620, 5976,  192225,  5, }, // Asia/Kuching
    {  620, 5547,  192230,  5, }, // Asia/Kuwait
    {  620, 1211,  192235,  6, }, // Asia/Macau
    {  620, 4017,  192241,  7, }, // Asia/Magadan
    {  620, 1349,  192248,  9, }, // Asia/Makassar
    {  620, 7182,  192257,  6, }, // Asia/Manila
    {  620, 6920,  192263,  6, }, // Asia/Muscat
    {  620, 2554,  192269,  8, }, // Asia/Nicosia
    {  620, 8675,  192277, 13, }, // Asia/Novokuznetsk
    {  620, 4135,  192290, 13, }, // Asia/Novosibirsk
    {  620, 4229,  192303,  6, }, // Asia/Omsk
    {  620, 7145,  192309,  5, }, // Asia/Oral
    {  620, 6961,  192314,  6, }, // Asia/Phnom_Penh
    {  620, 8693,  192320, 10, }, // Asia/Pontianak
    {  620, 4214,  192330,  8, }, // Asia/Pyongyang
    {  620, 5559,  192338,  6, }, // Asia/Qatar
    {  620, 8708,  192344,  9, }, // Asia/Qostanay
    {  620, 4269,  192353, 10, }, // Asia/Qyzylorda
    {  620, 3584,  192363,  7, }, // Asia/Riyadh
    {  620, 4408,  192370,  7, }, // Asia/Sakhalin
    {  620, 7245,  192377, 11, }, // Asia/Samarkand
    {  620, 3279,  191621,  3, }, // Asia/Seoul
    {  620, 1053,  192388,  9, }, // Asia/Shanghai
    {  620, 3300,  192397,  8, }, // Asia/Singapore
    {  620, 4297,  192405, 15, }, // Asia/Srednekolymsk
    {  620, 3263,  192420,  4, }, // Asia/Taipei
    {  620, 4781,  192424,  8, }, // Asia/Tashkent
    {  620, 3915,  192432,  7, }, // Asia/Tbilisi
    {  620, 2772,  192439,  7, }, // Asia/Tehran
    {  620, 1317,  192446,  5, }, // Asia/Thimphu
    {  620, 2821,  192451,  7, }, // Asia/Tokyo
    {  620, 4572,  192458,  6, }, // Asia/Tomsk
    {  620, 1021,  192464, 11, }, // Asia/Ulaanbaatar
    {  620, 1159,  192475,  7, }, // Asia/Urumqi
    {  620, 8722,  192482,  9, }, // Asia/Ust-Nera
    {  620, 6977,  192491, 11, }, // Asia/Vientiane
    {  620, 4710,  192502, 12, }, // Asia/Vladivostok
    {  620, 4828,  192514,  8, }, // Asia/Yakutsk
    {  620, 1235,  192522,  8, }, // Asia/Yangon
    {  620, 3883,  192530, 16, }, // Asia/Yekaterinburg
    {  620, 3747,  192546,  7, }, // Asia/Yerevan
    {  620, 3663,  192553,  7, }, // Atlantic/Azores
    {  620, 5671,  192560, 10, }, // Atlantic/Bermuda
    {  620, 6628,  192570,  6, }, // Atlantic/Canary
    {  620, 3727,  192576,  9, }, // Atlantic/Cape_Verde
    {  620, 1395,  192585,  4, }, // Atlantic/Faroe
    {  620, 8736,  192589,  6, }, // Atlantic/Madeira
    {  620, 2748,  192595,  8, }, // Atlantic/Reykjavik
    {  620, 7272,  192603, 14, }, // Atlantic/South_Georgia
    {  620, 6821,  192617, 11, }, // Atlantic/St_Helena
    {  620, 6644,  192628,  8, }, // Atlantic/Stanley
    {  620, 1670,  192636,  7, }, // Australia/Adelaide
    {  620, 1635,  192643,  7, }, // Australia/Brisbane
    {  620, 1799,  192650, 12, }, // Australia/Broken_Hill
    {  620, 1583,  192662,  7, }, // Australia/Darwin
    {  620, 3637,  192669,  5, }, // Australia/Eucla
    {  620, 1516,  192674,  7, }, // Australia/Hobart
    {  620, 8753,  192681,  8, }, // Australia/Lindeman
    {  620, 1547,  192689,  9, }, // Australia/Lord_Howe
    {  620, 1727,  192698,  9, }, // Australia/Melbourne
    {  620, 1762,  192707,  6, }, // Australia/Perth
    {  620, 1463,  192713,  7, }, // Australia/Sydney
    {  620, 6441,  192720, 10, }, // Europe/Amsterdam
    {  620, 6216,  192730,  9, }, // Europe/Andorra
    {  620, 3620,  192739,  9, }, // Europe/Astrakhan
    {  620, 2300,  192748,  7, }, // Europe/Athens
    {  620, 6336,  192755,  7, }, // Europe/Belgrade
    {  620, 4757,  192762,  8, }, // Europe/Berlin
    {  620, 6488,  192770, 10, }, // Europe/Bratislava
    {  620, 2167,  192780,  9, }, // Europe/Brussels
    {  620, 3928,  192789,  7, }, // Europe/Bucharest
    {  620, 3806,  192796,  7, }, // Europe/Budapest
    {  620, 8772,  192803, 10, }, // Europe/Busingen
    {  620, 2583,  192813,  7, }, // Europe/Chisinau
    {  620, 6289,  192820, 10, }, // Europe/Copenhagen
    {  620, 2338,  192830,  7, }, // Europe/Dublin
    {  620, 6307,  192837, 10, }, // Europe/Gibraltar
    {  620, 8788,  192847, 10, }, // Europe/Guernsey
    {  620, 6612,  192857,  8, }, // Europe/Helsinki
    {  620, 8804,  192865,  7, }, // Europe/Isle_of_Man
    {  620, 1130,  192872,  9, }, // Europe/Istanbul
    {  620, 8823,  192881,  9, }, // Europe/Jersey
    {  620, 3979,  192890, 11, }, // Europe/Kaliningrad
    {  620, 8929,  192901,  5, }, // Europe/Kirov
    {  620, 2527,  192906,  5, }, // Europe/Kyiv
    {  620, 3213,  192911,  6, }, // Europe/Lisbon
    {  620, 6506,  192917,  9, }, // Europe/Ljubljana
    {  620, 2501,  192926,  6, }, // Europe/London
    {  620, 6365,  192932, 12, }, // Europe/Luxembourg
    {  620, 4892,  192944,  6, }, // Europe/Madrid
    {  620, 6397,  192950,  5, }, // Europe/Malta
    {  620, 6582,  192955,  9, }, // Europe/Mariehamn
    {  620, 3693,  192964,  6, }, // Europe/Minsk
    {  620, 6410,  192970,  6, }, // Europe/Monaco
    {  620, 3537,  192976,  5, }, // Europe/Moscow
    {  620, 6458,  192981,  5, }, // Europe/Oslo
    {  620, 4284,  192986,  5, }, // Europe/Paris
    {  620, 6424,  192991, 10, }, // Europe/Podgorica
    {  620, 6275,  193001,  4, }, // Europe/Prague
    {  620, 8837,  190803,  4, }, // Europe/Riga
    {  620, 6324,  192587,  3, }, // Europe/Rome
    {  620, 4331,  193005,  6, }, // Europe/Samara
    {  620, 6470,  193011,  9, }, // Europe/San_Marino
    {  620, 6245,  193020,  8, }, // Europe/Sarajevo
    {  620, 4451,  193028,  7, }, // Europe/Saratov
    {  620, 8849,  193035, 11, }, // Europe/Simferopol
    {  620, 6383,  193046,  7, }, // Europe/Skopje
    {  620, 6599,  193053,  6, }, // Europe/Sofia
    {  620, 6523,  193059,  9, }, // Europe/Stockholm
    {  620, 8867,  193068,  8, }, // Europe/Tallinn
    {  620, 6202,  193076,  6, }, // Europe/Tirane
    {  620, 8882,  193082,  9, }, // Europe/Ulyanovsk
    {  620, 6352,  193091,  5, }, // Europe/Vaduz
    {  620, 6567,  193096,  7, }, // Europe/Vatican
    {  620, 6231,  193103,  7, }, // Europe/Vienna
    {  620, 8899,  193110,  8, }, // Europe/Vilnius
    {  620, 4727,  193118,  9, }, // Europe/Volgograd
    {  620, 3190,  193127,  6, }, // Europe/Warsaw
    {  620, 6261,  193133,  7, }, // Europe/Zagreb
    {  620, 6540,  193140,  5, }, // Europe/Zurich
    {  620, 5078,  193145, 13, }, // Indian/Antananarivo
    {  620, 6947,  193158,  5, }, // Indian/Chagos
    {  620, 6046,  193163,  9, }, // Indian/Christmas
    {  620, 6063,  193172,  5, }, // Indian/Cocos
    {  620, 5029,  193177,  6, }, // Indian/Comoro
    {  620, 6661,  193183, 10, }, // Indian/Kerguelen
    {  620, 7260,  193193,  4, }, // Indian/Mahe
    {  620, 7050,  193197,  8, }, // Indian/Maldives
    {  620, 4069,  193205,  9, }, // Indian/Mauritius
    {  620, 5098,  193214,  6, }, // Indian/Mayotte
    {  620, 7211,  193220,  8, }, // Indian/Reunion
    {  620, 4422,  193228,  6, }, // Pacific/Apia
    {  620,  932,  193234,  8, }, // Pacific/Auckland
    {  620, 3706,  193242, 10, }, // Pacific/Bougainville
    {  620, 3015,  193252,  5, }, // Pacific/Chatham
    {  620, 3157,  193257,  4, }, // Pacific/Chuuk
    {  620, 2237,  193261,  8, }, // Pacific/Easter
    {  620, 7360,  193269,  6, }, // Pacific/Efate
    {  620, 7327,  193275,  7, }, // Pacific/Fakaofo
    {  620, 3902,  193282,  4, }, // Pacific/Fiji
    {  620, 7343,  193286,  8, }, // Pacific/Funafuti
    {  620, 6678,  193294,  9, }, // Pacific/Galapagos
    {  620, 6696,  193303,  9, }, // Pacific/Gambier
    {  620, 3822,  193312, 11, }, // Pacific/Guadalcanal
    {  620, 6018,  193323,  3, }, // Pacific/Guam
    {  620, 2723,  193326,  8, }, // Pacific/Honolulu
    {  620, 3049,  193334, 13, }, // Pacific/Kanton
    {  620, 3998,  193347, 10, }, // Pacific/Kiritimati
    {  620, 7019,  193357,  5, }, // Pacific/Kosrae
    {  620, 2842,  193362,  8, }, // Pacific/Kwajalein
    {  620, 4877,  193370,  6, }, // Pacific/Majuro
    {  620, 4051,  193376,  9, }, // Pacific/Marquesas
    {  620, 8914,  193385,  7, }, // Pacific/Midway
    {  620, 7084,  193392,  6, }, // Pacific/Nauru
    {  620, 7132,  193398,  6, }, // Pacific/Niue
    {  620, 4168,  193404,  9, }, // Pacific/Norfolk
    {  620, 7098,  193413,  6, }, // Pacific/Noumea
    {  620, 3126,  193419,  9, }, // Pacific/Pago_Pago
    {  620, 7155,  193428,  5, }, // Pacific/Palau
    {  620, 7194,  193433,  8, }, // Pacific/Pitcairn
    {  620, 3096,  193441,  6, }, // Pacific/Pohnpei
    {  620, 4807,  193447, 12, }, // Pacific/Port_Moresby
    {  620, 6076,  193459,  9, }, // Pacific/Rarotonga
    {  620, 6031,  193468,  5, }, // Pacific/Saipan
    {  620, 7312,  193473,  6, }, // Pacific/Tahiti
    {  620, 6712,  193479,  6, }, // Pacific/Tarawa
    {  620, 4583,  193485,  9, }, // Pacific/Tongatapu
    {  620, 7392,  193494,  3, }, // Pacific/Wake
    {  620, 7405,  193497,  6, }, // Pacific/Wallis
    {  621,   14,       0,  6, }, // Africa/Asmara Tibetan/Tibetan/China
    {  621, 4435,       6,  8, }, // Africa/Sao_Tome
    {  621, 4252,      14,  8, }, // America/Asuncion
    {  621,  237,      22,  8, }, // America/Atikokan
    {  621, 7623,      30, 17, }, // America/Bahia_Banderas
    {  621, 3868,      47,  6, }, // America/Cancun
    {  621, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  621, 5723,      66,  7, }, // America/Curacao
    {  621,  481,      73, 13, }, // America/Indiana/Knox
    {  621, 7961,      86, 16, }, // America/Indiana/Marengo
    {  621, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  621, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  621, 8038,     139, 14, }, // America/Indiana/Vevay
    {  621, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  621, 8086,     171, 16, }, // America/Indiana/Winamac
    {  621, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  621, 5932,     207, 22, }, // America/Lower_Princes
    {  621, 8220,     229,  6, }, // America/Merida
    {  621, 1850,     235, 19, }, // America/Noronha
    {  621, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  621, 8329,     274, 20, }, // America/North_Dakota/Center
    {  621, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  621,  393,     317,  4, }, // America/Nuuk
    {  621, 6899,     321, 16, }, // America/Scoresbysund
    {  621, 8508,     337, 14, }, // America/St_Barthelemy
    {  621, 2061,     351, 10, }, // America/St_Johns
    {  621, 5863,     361,  9, }, // America/St_Kitts
    {  621, 5880,     370,  9, }, // America/St_Lucia
    {  621,  855,     379, 10, }, // America/St_Thomas
    {  621, 5913,     389, 11, }, // America/St_Vincent
    {  621, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  621, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  621, 1185,     448,  9, }, // Asia/Kathmandu
    {  621,  992,     457,  7, }, // Asia/Kolkata
    {  621, 1211,     464,  5, }, // Asia/Macau
    {  621, 1235,     477,  6, }, // Asia/Yangon
    {  621, 1395,     483,  5, }, // Atlantic/Faroe
    {  621, 6821,     488, 10, }, // Atlantic/St_Helena
    {  621, 2527,     514,  4, }, // Europe/Kyiv
    {  621, 7211,     547,  7, }, // Indian/Reunion
    {  621, 3157,     554,  5, }, // Pacific/Chuuk
    {  621, 3096,     586,  7, }, // Pacific/Pohnpei
    {  623,   14,       0,  6, }, // Africa/Asmara Tigre/Ethiopic/Eritrea
    {  623, 4435,       6,  8, }, // Africa/Sao_Tome
    {  623, 4252,      14,  8, }, // America/Asuncion
    {  623,  237,      22,  8, }, // America/Atikokan
    {  623, 7623,      30, 17, }, // America/Bahia_Banderas
    {  623, 3868,      47,  6, }, // America/Cancun
    {  623, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  623, 5723,      66,  7, }, // America/Curacao
    {  623,  481,      73, 13, }, // America/Indiana/Knox
    {  623, 7961,      86, 16, }, // America/Indiana/Marengo
    {  623, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  623, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  623, 8038,     139, 14, }, // America/Indiana/Vevay
    {  623, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  623, 8086,     171, 16, }, // America/Indiana/Winamac
    {  623, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  623, 5932,     207, 22, }, // America/Lower_Princes
    {  623, 8220,     229,  6, }, // America/Merida
    {  623, 1850,     235, 19, }, // America/Noronha
    {  623, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  623, 8329,     274, 20, }, // America/North_Dakota/Center
    {  623, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  623,  393,     317,  4, }, // America/Nuuk
    {  623, 6899,     321, 16, }, // America/Scoresbysund
    {  623, 8508,     337, 14, }, // America/St_Barthelemy
    {  623, 2061,     351, 10, }, // America/St_Johns
    {  623, 5863,     361,  9, }, // America/St_Kitts
    {  623, 5880,     370,  9, }, // America/St_Lucia
    {  623,  855,     379, 10, }, // America/St_Thomas
    {  623, 5913,     389, 11, }, // America/St_Vincent
    {  623, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  623, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  623, 1185,     448,  9, }, // Asia/Kathmandu
    {  623,  992,     457,  7, }, // Asia/Kolkata
    {  623, 1211,     464,  5, }, // Asia/Macau
    {  623, 1235,     477,  6, }, // Asia/Yangon
    {  623, 1395,     483,  5, }, // Atlantic/Faroe
    {  623, 6821,     488, 10, }, // Atlantic/St_Helena
    {  623, 2527,     514,  4, }, // Europe/Kyiv
    {  623, 7211,     547,  7, }, // Indian/Reunion
    {  623, 3157,     554,  5, }, // Pacific/Chuuk
    {  623, 3096,     586,  7, }, // Pacific/Pohnpei
    {  624, 6788,  193503,  4, }, // Africa/Abidjan Tigrinya/Ethiopic/Ethiopia
    {  624, 6760,  193507,  3, }, // Africa/Accra
    {  624, 5059,  193510,  7, }, // Africa/Addis_Ababa
    {  624, 7420,  193517,  5, }, // Africa/Algiers
    {  624,   14,  193522,  4, }, // Africa/Asmara
    {  624,   44,    8496,  3, }, // Africa/Bamako
    {  624, 5257,  193526,  3, }, // Africa/Bangui
    {  624, 6746,    8503,  4, }, // Africa/Banjul
    {  624, 7435,  193529,  3, }, // Africa/Bissau
    {  624, 4957,    8510,  6, }, // Africa/Blantyre
    {  624, 5287,    8516,  5, }, // Africa/Brazzaville
    {  624, 4922,  193532,  5, }, // Africa/Bujumbura
    {  624, 2320,    8526,  3, }, // Africa/Cairo
    {  624, 4117,    8529,  6, }, // Africa/Casablanca
    {  624, 7449,  193537,  3, }, // Africa/Ceuta
    {  624, 6773,  193540,  4, }, // Africa/Conakry
    {  624, 6840,    8542,  3, }, // Africa/Dakar
    {  624, 5130,  193544,  9, }, // Africa/Dar_es_Salaam
    {  624, 5043,    8551,  3, }, // Africa/Djibouti
    {  624, 5243,  193553,  3, }, // Africa/Douala
    {  624, 5166,  193556,  6, }, // Africa/El_Aaiun
    {  624, 6853,    8564,  5, }, // Africa/Freetown
    {  624, 4906,    8569,  4, }, // Africa/Gaborone
    {  624, 5015,  193562,  3, }, // Africa/Harare
    {  624, 4479,    8576,  7, }, // Africa/Johannesburg
    {  624, 4499,    8583,  2, }, // Africa/Juba
    {  624, 5151,    8585,  4, }, // Africa/Kampala
    {  624, 4524,    8589,  4, }, // Africa/Khartoum
    {  624, 4973,    8593,  3, }, // Africa/Kigali
    {  624, 5306,    8596,  4, }, // Africa/Kinshasa
    {  624, 4744,    8600,  3, }, // Africa/Lagos
    {  624, 5336,  193565,  5, }, // Africa/Libreville
    {  624, 6869,  193570,  2, }, // Africa/Lome
    {  624, 5211,  193572,  4, }, // Africa/Luanda
    {  624, 4939,    8614,  5, }, // Africa/Lubumbashi
    {  624, 5001,    8619,  3, }, // Africa/Lusaka
    {  624, 5322,  193576,  3, }, // Africa/Malabo
    {  624, 4987,  193579,  3, }, // Africa/Maputo
    {  624, 5197,  193582,  3, }, // Africa/Maseru
    {  624, 5182,  193585,  4, }, // Africa/Mbabane
    {  624, 5113,  193589,  4, }, // Africa/Mogadishu
    {  624, 7034,  193593,  5, }, // Africa/Monrovia
    {  624, 3853,    8644,  4, }, // Africa/Nairobi
    {  624, 5271,  193598,  4, }, // Africa/Ndjamena
    {  624, 5354,  193602,  4, }, // Africa/Niamey
    {  624, 6803,  193606,  5, }, // Africa/Nouakchott
    {  624, 6727,  193611,  4, }, // Africa/Ouagadougou
    {  624, 5225,  193615,  6, }, // Africa/Porto-Novo
    {  624, 4435,  193621,  5, }, // Africa/Sao_Tome
    {  624, 2866,    8678,  4, }, // Africa/Tripoli
    {  624, 6554,  193626,  3, }, // Africa/Tunis
    {  624, 4152,    8685,  5, }, // Africa/Windhoek
    {  624,  132,  193629,  3, }, // America/Adak
    {  624, 3346,  193632,  5, }, // America/Anchorage
    {  624, 5607,  193637,  4, }, // America/Anguilla
    {  624, 5624,  193641,  4, }, // America/Antigua
    {  624, 4554,  193645,  5, }, // America/Araguaina
    {  624,  166,  193650,  9, }, // America/Argentina/Buenos_Aires
    {  624,   91,    8720,  5, }, // America/Argentina/Catamarca
    {  624,  270,    8725,  4, }, // America/Argentina/Cordoba
    {  624,  441,  193659,  3, }, // America/Argentina/Jujuy
    {  624, 7462,  193662,  5, }, // America/Argentina/La_Rioja
    {  624,  565,  193667,  4, }, // America/Argentina/Mendoza
    {  624, 7489,  193671,  7, }, // America/Argentina/Rio_Gallegos
    {  624, 7520,    8748,  3, }, // America/Argentina/Salta
    {  624, 7544,  193678,  6, }, // America/Argentina/San_Juan
    {  624, 5580,  193684,  6, }, // America/Argentina/San_Luis
    {  624, 7571,    8763,  4, }, // America/Argentina/Tucuman
    {  624, 7597,  193690,  4, }, // America/Argentina/Ushuaia
    {  624, 5640,  193694,  3, }, // America/Aruba
    {  624, 4252,  193697,  6, }, // America/Asuncion
    {  624,  237,  193703,  5, }, // America/Atikokan
    {  624, 3679,  193708,  3, }, // America/Bahia
    {  624, 7623,  193711, 11, }, // America/Bahia_Banderas
    {  624, 5654,  193722,  5, }, // America/Barbados
    {  624, 7646,  193727,  3, }, // America/Belem
    {  624, 5407,  193730,  3, }, // America/Belize
    {  624, 7660,  193733,  9, }, // America/Blanc-Sablon
    {  624, 7681,  193742,  6, }, // America/Boa_Vista
    {  624, 4361,    8824,  3, }, // America/Bogota
    {  624, 7699,  193748,  3, }, // America/Boise
    {  624, 7713,  193751,  8, }, // America/Cambridge_Bay
    {  624, 7735,  193759,  8, }, // America/Campo_Grande
    {  624, 3868,    8846,  4, }, // America/Cancun
    {  624, 4694,    8850,  4, }, // America/Caracas
    {  624, 4345,  193767,  3, }, // America/Cayenne
    {  624, 5496,    8858,  4, }, // America/Cayman
    {  624, 2260,    8862,  3, }, // America/Chicago
    {  624, 7756,  193770,  3, }, // America/Chihuahua
    {  624, 7774,  193773,  9, }, // America/Ciudad_Juarez
    {  624, 5422,  193782,  6, }, // America/Costa_Rica
    {  624, 7796,    8884,  5, }, // America/Creston
    {  624, 3791,  193788,  3, }, // America/Cuiaba
    {  624, 5723,  193791,  4, }, // America/Curacao
    {  624, 7812,    8897,  8, }, // America/Danmarkshavn
    {  624, 7833,    8905,  4, }, // America/Dawson
    {  624, 7848,  193795,  8, }, // America/Dawson_Creek
    {  624,  805,  193803,  4, }, // America/Denver
    {  624, 3465,    8921,  5, }, // America/Detroit
    {  624, 5739,  193807,  4, }, // America/Dominica
    {  624,  893,    8930,  6, }, // America/Edmonton
    {  624, 7869,  193811,  5, }, // America/Eirunepe
    {  624, 5441,  193816,  8, }, // America/El_Salvador
    {  624, 7886,  193824,  8, }, // America/Fort_Nelson
    {  624, 7906,  193832,  5, }, // America/Fortaleza
    {  624, 7924,  193837,  6, }, // America/Glace_Bay
    {  624, 6881,  193843,  5, }, // America/Goose_Bay
    {  624, 4612,  193848,  8, }, // America/Grand_Turk
    {  624, 5770,  193856,  4, }, // America/Grenada
    {  624, 5786,  193860,  4, }, // America/Guadeloupe
    {  624, 3760,    8987,  4, }, // America/Guatemala
    {  624, 4841,  193864,  4, }, // America/Guayaquil
    {  624, 6932,    8996,  3, }, // America/Guyana
    {  624, 1939,    8999,  5, }, // America/Halifax
    {  624, 2281,    9004,  3, }, // America/Havana
    {  624, 7942,  193868,  5, }, // America/Hermosillo
    {  624,  348,  193873,  8, }, // America/Indiana/Indianapolis
    {  624,  481,  193881, 10, }, // America/Indiana/Knox
    {  624, 7961,  193891, 11, }, // America/Indiana/Marengo
    {  624, 7985,  193902, 14, }, // America/Indiana/Petersburg
    {  624, 8012,  193916, 12, }, // America/Indiana/Tell_City
    {  624, 8038,  193928,  9, }, // America/Indiana/Vevay
    {  624, 8060,  193937, 12, }, // America/Indiana/Vincennes
    {  624, 8086,  193949, 11, }, // America/Indiana/Winamac
    {  624, 8110,    9101,  4, }, // America/Inuvik
    {  624,  660,  193960,  5, }, // America/Iqaluit
    {  624, 2799,    9110,  4, }, // America/Jamaica
    {  624, 5380,  193965,  3, }, // America/Juneau
    {  624,  521,  193968,  4, }, // America/Kentucky/Louisville
    {  624, 8125,  193972, 11, }, // America/Kentucky/Monticello
    {  624, 5704,  193983,  7, }, // America/Kralendijk
    {  624, 4376,    9140,  4, }, // America/La_Paz
    {  624, 7169,    9144,  2, }, // America/Lima
    {  624, 3239,  193990,  8, }, // America/Los_Angeles
    {  624, 5932,  193998, 14, }, // America/Lower_Princes
    {  624, 8153,  194012,  3, }, // America/Maceio
    {  624, 8168,  194015,  3, }, // America/Managua
    {  624, 1908,  194018,  4, }, // America/Manaus
    {  624, 5897,  194022,  4, }, // America/Marigot
    {  624, 5805,    9181,  5, }, // America/Martinique
    {  624, 8184,    9186,  5, }, // America/Matamoros
    {  624, 2917,    9191,  5, }, // America/Mazatlan
    {  624, 8202,    9196,  4, }, // America/Menominee
    {  624, 8220,  194026,  3, }, // America/Merida
    {  624, 8235,    9203,  6, }, // America/Metlakatla
    {  624, 2949,  194029,  8, }, // America/Mexico_City
    {  624, 4391,  194037,  4, }, // America/Miquelon
    {  624, 8254,  194041,  5, }, // America/Moncton
    {  624, 8270,  194046,  5, }, // America/Monterrey
    {  624, 4098,  194051,  6, }, // America/Montevideo
    {  624, 5824,  194057,  6, }, // America/Montserrat
    {  624, 5481,    9245,  3, }, // America/Nassau
    {  624, 2379,  194063,  6, }, // America/New_York
    {  624, 8288,  194069,  3, }, // America/Nome
    {  624, 1850,  194072,  4, }, // America/Noronha
    {  624, 8301,  194076, 12, }, // America/North_Dakota/Beulah
    {  624, 8329,  194088, 13, }, // America/North_Dakota/Center
    {  624, 8357,  194101, 15, }, // America/North_Dakota/New_Salem
    {  624,  393,  194116,  2, }, // America/Nuuk
    {  624, 8388,  194118,  4, }, // America/Ojinaga
    {  624, 2356,    9309,  3, }, // America/Panama
    {  624, 6173,    9312,  5, }, // America/Paramaribo
    {  624, 2973,    9317,  4, }, // America/Phoenix
    {  624, 3945,  194122, 10, }, // America/Port-au-Prince
    {  624, 5954,  194132, 10, }, // America/Port_of_Spain
    {  624, 8404,  194142,  7, }, // America/Porto_Velho
    {  624, 5843,  194149,  6, }, // America/Puerto_Rico
    {  624, 4030,  194155,  8, }, // America/Punta_Arenas
    {  624, 8424,  194163,  9, }, // America/Rankin_Inlet
    {  624, 8445,  194172,  3, }, // America/Recife
    {  624, 1995,    9372,  3, }, // America/Regina
    {  624, 8460,  194175,  4, }, // America/Resolute
    {  624,  695,  194179,  7, }, // America/Rio_Branco
    {  624, 8477,  194186,  5, }, // America/Santarem
    {  624, 2201,  194191,  5, }, // America/Santiago
    {  624, 6111,    9397,  8, }, // America/Santo_Domingo
    {  624, 1878,  194196,  6, }, // America/Sao_Paulo
    {  624, 6899,  194202,  7, }, // America/Scoresbysund
    {  624, 8494,    9419,  3, }, // America/Sitka
    {  624, 8508,  194209,  9, }, // America/St_Barthelemy
    {  624, 2061,  194218,  8, }, // America/St_Johns
    {  624, 5863,    9440,  7, }, // America/St_Kitts
    {  624, 5880,  194226,  8, }, // America/St_Lucia
    {  624,  855,    9455,  7, }, // America/St_Thomas
    {  624, 5913,    9462,  9, }, // America/St_Vincent
    {  624, 8530,  194234,  9, }, // America/Swift_Current
    {  624, 5461,  194243,  6, }, // America/Tegucigalpa
    {  624, 5756,  194249,  2, }, // America/Thule
    {  624,  313,  194251,  4, }, // America/Tijuana
    {  624,  608,    9492,  4, }, // America/Toronto
    {  624, 5688,    9496,  4, }, // America/Tortola
    {  624, 2093,  194255,  5, }, // America/Vancouver
    {  624, 2144,  194260,  5, }, // America/Whitehorse
    {  624,  734,  194265,  4, }, // America/Winnipeg
    {  624, 8552,    9515,  4, }, // America/Yakutat
    {  624, 6001,  194269,  3, }, // Antarctica/Casey
    {  624, 6094,  194272,  3, }, // Antarctica/Davis
    {  624, 6133,  194275, 10, }, // Antarctica/DumontDUrville
    {  624, 8568,  194285,  3, }, // Antarctica/Macquarie
    {  624, 7066,  194288,  4, }, // Antarctica/Mawson
    {  624, 7113,  194292,  5, }, // Antarctica/McMurdo
    {  624, 8589,    9547,  4, }, // Antarctica/Palmer
    {  624, 7226,  194297,  3, }, // Antarctica/Rothera
    {  624, 7295,    9554,  3, }, // Antarctica/Syowa
    {  624, 8607,    9557,  3, }, // Antarctica/Troll
    {  624, 7374,    9918,  4, }, // Antarctica/Vostok
    {  624, 1429,  194300,  8, }, // Arctic/Longyearbyen
    {  624, 5570,  194308,  3, }, // Asia/Aden
    {  624, 5395,  194311,  4, }, // Asia/Almaty
    {  624, 3968,  194315,  3, }, // Asia/Amman
    {  624, 5511,  194318,  4, }, // Asia/Anadyr
    {  624, 5523,  194322,  4, }, // Asia/Aqtau
    {  624, 5368,  194326,  4, }, // Asia/Aqtobe
    {  624,  964,  194330,  5, }, // Asia/Ashgabat
    {  624, 8624,  194335,  4, }, // Asia/Atyrau
    {  624, 3607,  194339,  4, }, // Asia/Baghdad
    {  624, 5534,  194343,  4, }, // Asia/Bahrain
    {  624, 3653,    8892,  2, }, // Asia/Baku
    {  624, 4466,  194347,  5, }, // Asia/Bangkok
    {  624, 3571,  194352,  5, }, // Asia/Barnaul
    {  624, 4086,  194357,  4, }, // Asia/Beirut
    {  624, 3778,    9626,  4, }, // Asia/Bishkek
    {  624, 5989,  194361,  4, }, // Asia/Brunei
    {  624, 4601,    9634,  2, }, // Asia/Chita
    {  624, 4511,    9636,  4, }, // Asia/Colombo
    {  624, 4540,    9640,  4, }, // Asia/Damascus
    {  624, 1093,    8542,  2, }, // Asia/Dhaka
    {  624, 6192,    9644,  2, }, // Asia/Dili
    {  624, 3596,    9646,  3, }, // Asia/Dubai
    {  624, 6159,  194365,  4, }, // Asia/Dushanbe
    {  624, 8636,    9653,  5, }, // Asia/Famagusta
    {  624, 8651,  194369,  2, }, // Asia/Gaza
    {  624, 4795,    9660,  4, }, // Asia/Hebron
    {  624, 1259,  194371, 10, }, // Asia/Ho_Chi_Minh
    {  624, 2704,    9675,  7, }, // Asia/Hong_Kong
    {  624, 4771,    9682,  3, }, // Asia/Hovd
    {  624, 4184,    9685,  6, }, // Asia/Irkutsk
    {  624, 7006,    9691,  4, }, // Asia/Jakarta
    {  624, 6992,    9695,  4, }, // Asia/Jayapura
    {  624, 1290,    9700,  5, }, // Asia/Jerusalem
    {  624, 3560,    9705,  3, }, // Asia/Kabul
    {  624, 4316,    9708,  5, }, // Asia/Kamchatka
    {  624, 4239,    9713,  3, }, // Asia/Karachi
    {  624, 1185,    9716,  5, }, // Asia/Kathmandu
    {  624, 8661,    9721,  4, }, // Asia/Khandyga
    {  624,  992,    9725,  4, }, // Asia/Kolkata
    {  624, 4197,    9729,  8, }, // Asia/Krasnoyarsk
    {  624, 4859,  194381,  7, }, // Asia/Kuala_Lumpur
    {  624, 5976,    9745,  4, }, // Asia/Kuching
    {  624, 5547,    9749,  3, }, // Asia/Kuwait
    {  624, 1211,  194388,  3, }, // Asia/Macau
    {  624, 4017,    9755,  4, }, // Asia/Magadan
    {  624, 1349,    9759,  4, }, // Asia/Makassar
    {  624, 7182,    9763,  3, }, // Asia/Manila
    {  624, 6920,    9766,  4, }, // Asia/Muscat
    {  624, 2554,  194391,  4, }, // Asia/Nicosia
    {  624, 8675,  194395,  8, }, // Asia/Novokuznetsk
    {  624, 4135,  194403,  7, }, // Asia/Novosibirsk
    {  624, 4229,    9788,  4, }, // Asia/Omsk
    {  624, 7145,    9792,  3, }, // Asia/Oral
    {  624, 6961,  194410,  6, }, // Asia/Phnom_Penh
    {  624, 8693,  194416,  6, }, // Asia/Pontianak
    {  624, 4214,    9808,  7, }, // Asia/Pyongyang
    {  624, 5559,  194422,  3, }, // Asia/Qatar
    {  624, 8708,    9818,  5, }, // Asia/Qostanay
    {  624, 4269,  194425,  5, }, // Asia/Qyzylorda
    {  624, 3584,  194430,  3, }, // Asia/Riyadh
    {  624, 4408,  194433,  4, }, // Asia/Sakhalin
    {  624, 7245,    9837,  6, }, // Asia/Samarkand
    {  624, 3279,  194437,  3, }, // Asia/Seoul
    {  624, 1053,  194440,  5, }, // Asia/Shanghai
    {  624, 3300,    9850,  5, }, // Asia/Singapore
    {  624, 4297,  194445,  9, }, // Asia/Srednekolymsk
    {  624, 3263,  194454,  4, }, // Asia/Taipei
    {  624, 4781,  194458,  5, }, // Asia/Tashkent
    {  624, 3915,  194463,  4, }, // Asia/Tbilisi
    {  624, 2772,  194467,  4, }, // Asia/Tehran
    {  624, 1317,    9882,  3, }, // Asia/Thimphu
    {  624, 2821,  194471,  3, }, // Asia/Tokyo
    {  624, 4572,    9888,  4, }, // Asia/Tomsk
    {  624, 1021,  194474,  7, }, // Asia/Ulaanbaatar
    {  624, 1159,    9900,  4, }, // Asia/Urumqi
    {  624, 8722,    9904,  6, }, // Asia/Ust-Nera
    {  624, 6977,  194481,  6, }, // Asia/Vientiane
    {  624, 4710,    9915,  7, }, // Asia/Vladivostok
    {  624, 4828,    9922,  5, }, // Asia/Yakutsk
    {  624, 1235,    9927,  4, }, // Asia/Yangon
    {  624, 3883,    9931,  8, }, // Asia/Yekaterinburg
    {  624, 3747,  194487,  4, }, // Asia/Yerevan
    {  624, 3663,  194491,  4, }, // Atlantic/Azores
    {  624, 5671,  194495,  4, }, // Atlantic/Bermuda
    {  624, 6628,    9951,  3, }, // Atlantic/Canary
    {  624, 3727,  194499,  6, }, // Atlantic/Cape_Verde
    {  624, 1395,  194505,  7, }, // Atlantic/Faroe
    {  624, 8736,  194512,  4, }, // Atlantic/Madeira
    {  624, 2748,  194516,  6, }, // Atlantic/Reykjavik
    {  624, 7272,  194522,  8, }, // Atlantic/South_Georgia
    {  624, 6821,  194530,  8, }, // Atlantic/St_Helena
    {  624, 6644,  194538,  4, }, // Atlantic/Stanley
    {  624, 1670,  194542,  5, }, // Australia/Adelaide
    {  624, 1635,  194547,  5, }, // Australia/Brisbane
    {  624, 1799,   10003,  7, }, // Australia/Broken_Hill
    {  624, 1583,   10010,  4, }, // Australia/Darwin
    {  624, 3637,  194552,  3, }, // Australia/Eucla
    {  624, 1516,   10017,  4, }, // Australia/Hobart
    {  624, 8753,   10021,  5, }, // Australia/Lindeman
    {  624, 1547,  194555,  6, }, // Australia/Lord_Howe
    {  624, 1727,  194561,  5, }, // Australia/Melbourne
    {  624, 1762,   10037,  3, }, // Australia/Perth
    {  624, 1463,   10040,  3, }, // Australia/Sydney
    {  624, 6441,  194566,  7, }, // Europe/Amsterdam
    {  624, 6216,  194573,  4, }, // Europe/Andorra
    {  624, 3620,  194577,  6, }, // Europe/Astrakhan
    {  624, 2300,  194583,  4, }, // Europe/Athens
    {  624, 6336,  194587,  5, }, // Europe/Belgrade
    {  624, 4757,   10070,  4, }, // Europe/Berlin
    {  624, 6488,   10074,  6, }, // Europe/Bratislava
    {  624, 2167,  194592,  5, }, // Europe/Brussels
    {  624, 3928,  194597,  5, }, // Europe/Bucharest
    {  624, 3806,  194602,  5, }, // Europe/Budapest
    {  624, 8772,   10095,  5, }, // Europe/Busingen
    {  624, 2583,  194607,  4, }, // Europe/Chisinau
    {  624, 6289,   10104,  6, }, // Europe/Copenhagen
    {  624, 2338,   10110,  4, }, // Europe/Dublin
    {  624, 6307,   10114,  6, }, // Europe/Gibraltar
    {  624, 8788,  194611,  4, }, // Europe/Guernsey
    {  624, 6612,   10124,  5, }, // Europe/Helsinki
    {  624, 8804,  194615,  9, }, // Europe/Isle_of_Man
    {  624, 1130,   10139,  6, }, // Europe/Istanbul
    {  624, 8823,  194624,  3, }, // Europe/Jersey
    {  624, 3979,   10148,  7, }, // Europe/Kaliningrad
    {  624, 8929,   10155,  3, }, // Europe/Kirov
    {  624, 2527,  194627,  3, }, // Europe/Kyiv
    {  624, 3213,   10161,  4, }, // Europe/Lisbon
    {  624, 6506,  194630,  6, }, // Europe/Ljubljana
    {  624, 2501,  194636,  4, }, // Europe/London
    {  624, 6365,   10175,  7, }, // Europe/Luxembourg
    {  624, 4892,   10182,  4, }, // Europe/Madrid
    {  624, 6397,   10186,  3, }, // Europe/Malta
    {  624, 6582,  194640,  5, }, // Europe/Mariehamn
    {  624, 3693,   10194,  4, }, // Europe/Minsk
    {  624, 6410,   10198,  3, }, // Europe/Monaco
    {  624, 3537,   10201,  3, }, // Europe/Moscow
    {  624, 6458,   10204,  3, }, // Europe/Oslo
    {  624, 4284,   10207,  3, }, // Europe/Paris
    {  624, 6424,  194645,  5, }, // Europe/Podgorica
    {  624, 6275,   10215,  3, }, // Europe/Prague
    {  624, 8837,   10218,  2, }, // Europe/Riga
    {  624, 6324,  194511,  2, }, // Europe/Rome
    {  624, 4331,   10222,  3, }, // Europe/Samara
    {  624, 6470,   10225,  6, }, // Europe/San_Marino
    {  624, 6245,  194650,  4, }, // Europe/Sarajevo
    {  624, 4451,   10235,  4, }, // Europe/Saratov
    {  624, 8849,   10239,  6, }, // Europe/Simferopol
    {  624, 6383,   10245,  4, }, // Europe/Skopje
    {  624, 6599,  194654,  3, }, // Europe/Sofia
    {  624, 6523,   10252,  6, }, // Europe/Stockholm
    {  624, 8867,   10258,  3, }, // Europe/Tallinn
    {  624, 6202,  194657,  3, }, // Europe/Tirane
    {  624, 8882,   10264,  7, }, // Europe/Ulyanovsk
    {  624, 6352,   10271,  3, }, // Europe/Vaduz
    {  624, 6567,   10274,  4, }, // Europe/Vatican
    {  624, 6231,  194660,  3, }, // Europe/Vienna
    {  624, 8899,  194663,  5, }, // Europe/Vilnius
    {  624, 4727,  194668,  6, }, // Europe/Volgograd
    {  624, 3190,  194674,  4, }, // Europe/Warsaw
    {  624, 6261,  194678,  4, }, // Europe/Zagreb
    {  624, 6540,   10298,  3, }, // Europe/Zurich
    {  624, 5078,  194682,  7, }, // Indian/Antananarivo
    {  624, 6947,   10308,  3, }, // Indian/Chagos
    {  624, 6046,  194689,  5, }, // Indian/Christmas
    {  624, 6063,   10313,  3, }, // Indian/Cocos
    {  624, 5029,   10316,  3, }, // Indian/Comoro
    {  624, 6661,  194694,  5, }, // Indian/Kerguelen
    {  624, 7260,   10324,  2, }, // Indian/Mahe
    {  624, 7050,  194699,  5, }, // Indian/Maldives
    {  624, 4069,  194704,  5, }, // Indian/Mauritius
    {  624, 5098,  194709,  3, }, // Indian/Mayotte
    {  624, 7211,  194712,  5, }, // Indian/Reunion
    {  624, 4422,  194717,  3, }, // Pacific/Apia
    {  624,  932,   10346,  5, }, // Pacific/Auckland
    {  624, 3706,  194720,  5, }, // Pacific/Bougainville
    {  624, 3015,   10357,  3, }, // Pacific/Chatham
    {  624, 3157,   10360,  2, }, // Pacific/Chuuk
    {  624, 2237,  194725,  7, }, // Pacific/Easter
    {  624, 7360,  194732,  3, }, // Pacific/Efate
    {  624, 7327,   10368,  4, }, // Pacific/Fakaofo
    {  624, 3902,   10372,  2, }, // Pacific/Fiji
    {  624, 7343,  194735,  4, }, // Pacific/Funafuti
    {  624, 6678,   10378,  5, }, // Pacific/Galapagos
    {  624, 6696,  194739,  5, }, // Pacific/Gambier
    {  624, 3822,  194744,  6, }, // Pacific/Guadalcanal
    {  624, 6018,  194750,  2, }, // Pacific/Guam
    {  624, 2723,   10398,  4, }, // Pacific/Honolulu
    {  624, 3049,  194752,  6, }, // Pacific/Kanton
    {  624, 3998,   10408,  5, }, // Pacific/Kiritimati
    {  624, 7019,   10413,  3, }, // Pacific/Kosrae
    {  624, 2842,   10416,  5, }, // Pacific/Kwajalein
    {  624, 4877,  194758,  3, }, // Pacific/Majuro
    {  624, 4051,  194761,  5, }, // Pacific/Marquesas
    {  624, 8914,   10430,  4, }, // Pacific/Midway
    {  624, 7084,   10434,  3, }, // Pacific/Nauru
    {  624, 7132,    9115,  2, }, // Pacific/Niue
    {  624, 4168,   10439,  5, }, // Pacific/Norfolk
    {  624, 7098,  194766,  3, }, // Pacific/Noumea
    {  624, 3126,   10448,  5, }, // Pacific/Pago_Pago
    {  624, 7155,   10453,  3, }, // Pacific/Palau
    {  624, 7194,  194769,  5, }, // Pacific/Pitcairn
    {  624, 3096,  194774,  4, }, // Pacific/Pohnpei
    {  624, 4807,  194778,  8, }, // Pacific/Port_Moresby
    {  624, 6076,   10475,  5, }, // Pacific/Rarotonga
    {  624, 6031,  194786,  4, }, // Pacific/Saipan
    {  624, 7312,   10484,  3, }, // Pacific/Tahiti
    {  624, 6712,   10487,  3, }, // Pacific/Tarawa
    {  624, 4583,   10490,  5, }, // Pacific/Tongatapu
    {  624, 7392,  194790,  2, }, // Pacific/Wake
    {  624, 7405,   10497,  3, }, // Pacific/Wallis
    {  626,   14,       0,  6, }, // Africa/Asmara Tok Pisin/Latin/Papua New Guinea
    {  626, 4435,       6,  8, }, // Africa/Sao_Tome
    {  626, 4252,      14,  8, }, // America/Asuncion
    {  626,  237,      22,  8, }, // America/Atikokan
    {  626, 7623,      30, 17, }, // America/Bahia_Banderas
    {  626, 3868,      47,  6, }, // America/Cancun
    {  626, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  626, 5723,      66,  7, }, // America/Curacao
    {  626,  481,      73, 13, }, // America/Indiana/Knox
    {  626, 7961,      86, 16, }, // America/Indiana/Marengo
    {  626, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  626, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  626, 8038,     139, 14, }, // America/Indiana/Vevay
    {  626, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  626, 8086,     171, 16, }, // America/Indiana/Winamac
    {  626, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  626, 5932,     207, 22, }, // America/Lower_Princes
    {  626, 8220,     229,  6, }, // America/Merida
    {  626, 1850,     235, 19, }, // America/Noronha
    {  626, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  626, 8329,     274, 20, }, // America/North_Dakota/Center
    {  626, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  626,  393,     317,  4, }, // America/Nuuk
    {  626, 6899,     321, 16, }, // America/Scoresbysund
    {  626, 8508,     337, 14, }, // America/St_Barthelemy
    {  626, 2061,     351, 10, }, // America/St_Johns
    {  626, 5863,     361,  9, }, // America/St_Kitts
    {  626, 5880,     370,  9, }, // America/St_Lucia
    {  626,  855,     379, 10, }, // America/St_Thomas
    {  626, 5913,     389, 11, }, // America/St_Vincent
    {  626, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  626, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  626, 1185,     448,  9, }, // Asia/Kathmandu
    {  626,  992,     457,  7, }, // Asia/Kolkata
    {  626, 1211,     464,  5, }, // Asia/Macau
    {  626, 1235,     477,  6, }, // Asia/Yangon
    {  626, 1395,     483,  5, }, // Atlantic/Faroe
    {  626, 6821,     488, 10, }, // Atlantic/St_Helena
    {  626, 2527,     514,  4, }, // Europe/Kyiv
    {  626, 7211,     547,  7, }, // Indian/Reunion
    {  626, 3157,     554,  5, }, // Pacific/Chuuk
    {  626, 3096,     586,  7, }, // Pacific/Pohnpei
    {  627,   14,       0,  6, }, // Africa/Asmara Tongan/Latin/Tonga
    {  627, 4435,   58730,  8, }, // Africa/Sao_Tome
    {  627, 4252,      14,  8, }, // America/Asuncion
    {  627,  237,      22,  8, }, // America/Atikokan
    {  627, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  627, 3868,   33314,  6, }, // America/Cancun
    {  627, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  627, 5723,      66,  7, }, // America/Curacao
    {  627,  481,      73, 13, }, // America/Indiana/Knox
    {  627, 7961,      86, 16, }, // America/Indiana/Marengo
    {  627, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  627, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  627, 8038,     139, 14, }, // America/Indiana/Vevay
    {  627, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  627, 8086,     171, 16, }, // America/Indiana/Winamac
    {  627, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  627, 5932,     207, 22, }, // America/Lower_Princes
    {  627, 8220,    7126,  6, }, // America/Merida
    {  627, 2379,  194792,  9, }, // America/New_York
    {  627, 1850,     235, 19, }, // America/Noronha
    {  627, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  627, 8329,     274, 20, }, // America/North_Dakota/Center
    {  627, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  627,  393,     317,  4, }, // America/Nuuk
    {  627, 6899,     321, 16, }, // America/Scoresbysund
    {  627, 8508,     337, 14, }, // America/St_Barthelemy
    {  627, 2061,     351, 10, }, // America/St_Johns
    {  627, 5863,     361,  9, }, // America/St_Kitts
    {  627, 5880,     370,  9, }, // America/St_Lucia
    {  627,  855,     379, 10, }, // America/St_Thomas
    {  627, 5913,     389, 11, }, // America/St_Vincent
    {  627, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  627, 4795,  194801,  8, }, // Asia/Hebron
    {  627, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  627, 1290,  194809, 10, }, // Asia/Jerusalem
    {  627, 1185,     448,  9, }, // Asia/Kathmandu
    {  627,  992,     457,  7, }, // Asia/Kolkata
    {  627, 1211,    4321,  5, }, // Asia/Macau
    {  627, 1235,   58618,  7, }, // Asia/Yangon
    {  627, 1395,     483,  5, }, // Atlantic/Faroe
    {  627, 6821,     488, 10, }, // Atlantic/St_Helena
    {  627, 1670,  194819,  8, }, // Australia/Adelaide
    {  627, 1635,  194827, 10, }, // Australia/Brisbane
    {  627, 1727,  194837,  9, }, // Australia/Melbourne
    {  627, 1463,  194846,  4, }, // Australia/Sydney
    {  627, 2300,  194850,  8, }, // Europe/Athens
    {  627, 2527,    8148,  4, }, // Europe/Kyiv
    {  627, 2501,  194858,  8, }, // Europe/London
    {  627, 3537,  194866,  7, }, // Europe/Moscow
    {  627, 4284,  194873,  6, }, // Europe/Paris
    {  627, 6324,  194879,  4, }, // Europe/Rome
    {  627, 6567,   64054,  8, }, // Europe/Vatican
    {  627, 7211,     547,  7, }, // Indian/Reunion
    {  627,  932,  194883,  9, }, // Pacific/Auckland
    {  627, 3706,  194892, 10, }, // Pacific/Bougainville
    {  627, 3157,  194902,  4, }, // Pacific/Chuuk
    {  627, 2237,  194906,  7, }, // Pacific/Easter
    {  627, 3902,  194913,  4, }, // Pacific/Fiji
    {  627, 6696,  194917,  7, }, // Pacific/Gambier
    {  627, 3822,  194924, 11, }, // Pacific/Guadalcanal
    {  627, 6018,  194935,  5, }, // Pacific/Guam
    {  627, 2723,    4611,  8, }, // Pacific/Honolulu
    {  627, 3049,  194940, 10, }, // Pacific/Kanton
    {  627, 3998,  194950, 10, }, // Pacific/Kiritimati
    {  627, 7019,  194960,  7, }, // Pacific/Kosrae
    {  627, 2842,  194967,  9, }, // Pacific/Kwajalein
    {  627, 4877,  194976,  6, }, // Pacific/Majuro
    {  627, 4051,  194982,  9, }, // Pacific/Marquesas
    {  627, 8914,  194991,  7, }, // Pacific/Midway
    {  627, 7084,  194998,  5, }, // Pacific/Nauru
    {  627, 7132,  195003,  4, }, // Pacific/Niue
    {  627, 4168,  195007,  7, }, // Pacific/Norfolk
    {  627, 7098,  124660,  5, }, // Pacific/Noumea
    {  627, 3126,  195014, 10, }, // Pacific/Pago_Pago
    {  627, 7194,  195024,  8, }, // Pacific/Pitcairn
    {  627, 3096,  195032,  6, }, // Pacific/Pohnpei
    {  627, 4807,  195038, 17, }, // Pacific/Port_Moresby
    {  627, 6076,  195055,  9, }, // Pacific/Rarotonga
    {  627, 6031,  195064,  8, }, // Pacific/Saipan
    {  627, 7312,  195072,  6, }, // Pacific/Tahiti
    {  627, 6712,  195078,  6, }, // Pacific/Tarawa
    {  627, 7392,  195084,  4, }, // Pacific/Wake
    {  627, 7405,  195088,  5, }, // Pacific/Wallis
    {  628,   14,       0,  6, }, // Africa/Asmara Tsonga/Latin/South Africa
    {  628, 4435,       6,  8, }, // Africa/Sao_Tome
    {  628, 4252,      14,  8, }, // America/Asuncion
    {  628,  237,      22,  8, }, // America/Atikokan
    {  628, 7623,      30, 17, }, // America/Bahia_Banderas
    {  628, 3868,      47,  6, }, // America/Cancun
    {  628, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  628, 5723,      66,  7, }, // America/Curacao
    {  628,  481,      73, 13, }, // America/Indiana/Knox
    {  628, 7961,      86, 16, }, // America/Indiana/Marengo
    {  628, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  628, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  628, 8038,     139, 14, }, // America/Indiana/Vevay
    {  628, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  628, 8086,     171, 16, }, // America/Indiana/Winamac
    {  628, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  628, 5932,     207, 22, }, // America/Lower_Princes
    {  628, 8220,     229,  6, }, // America/Merida
    {  628, 1850,     235, 19, }, // America/Noronha
    {  628, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  628, 8329,     274, 20, }, // America/North_Dakota/Center
    {  628, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  628,  393,     317,  4, }, // America/Nuuk
    {  628, 6899,     321, 16, }, // America/Scoresbysund
    {  628, 8508,     337, 14, }, // America/St_Barthelemy
    {  628, 2061,     351, 10, }, // America/St_Johns
    {  628, 5863,     361,  9, }, // America/St_Kitts
    {  628, 5880,     370,  9, }, // America/St_Lucia
    {  628,  855,     379, 10, }, // America/St_Thomas
    {  628, 5913,     389, 11, }, // America/St_Vincent
    {  628, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  628, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  628, 1185,     448,  9, }, // Asia/Kathmandu
    {  628,  992,     457,  7, }, // Asia/Kolkata
    {  628, 1211,     464,  5, }, // Asia/Macau
    {  628, 1235,     477,  6, }, // Asia/Yangon
    {  628, 1395,     483,  5, }, // Atlantic/Faroe
    {  628, 6821,     488, 10, }, // Atlantic/St_Helena
    {  628, 2527,     514,  4, }, // Europe/Kyiv
    {  628, 7211,     547,  7, }, // Indian/Reunion
    {  628, 3157,     554,  5, }, // Pacific/Chuuk
    {  628, 3096,     586,  7, }, // Pacific/Pohnpei
    {  629,   14,       0,  6, }, // Africa/Asmara Tswana/Latin/South Africa
    {  629, 4435,       6,  8, }, // Africa/Sao_Tome
    {  629, 4252,      14,  8, }, // America/Asuncion
    {  629,  237,      22,  8, }, // America/Atikokan
    {  629, 7623,      30, 17, }, // America/Bahia_Banderas
    {  629, 3868,      47,  6, }, // America/Cancun
    {  629, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  629, 5723,      66,  7, }, // America/Curacao
    {  629,  481,      73, 13, }, // America/Indiana/Knox
    {  629, 7961,      86, 16, }, // America/Indiana/Marengo
    {  629, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  629, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  629, 8038,     139, 14, }, // America/Indiana/Vevay
    {  629, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  629, 8086,     171, 16, }, // America/Indiana/Winamac
    {  629, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  629, 5932,     207, 22, }, // America/Lower_Princes
    {  629, 8220,     229,  6, }, // America/Merida
    {  629, 1850,     235, 19, }, // America/Noronha
    {  629, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  629, 8329,     274, 20, }, // America/North_Dakota/Center
    {  629, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  629,  393,     317,  4, }, // America/Nuuk
    {  629, 6899,     321, 16, }, // America/Scoresbysund
    {  629, 8508,     337, 14, }, // America/St_Barthelemy
    {  629, 2061,     351, 10, }, // America/St_Johns
    {  629, 5863,     361,  9, }, // America/St_Kitts
    {  629, 5880,     370,  9, }, // America/St_Lucia
    {  629,  855,     379, 10, }, // America/St_Thomas
    {  629, 5913,     389, 11, }, // America/St_Vincent
    {  629, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  629, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  629, 1185,     448,  9, }, // Asia/Kathmandu
    {  629,  992,     457,  7, }, // Asia/Kolkata
    {  629, 1211,     464,  5, }, // Asia/Macau
    {  629, 1235,     477,  6, }, // Asia/Yangon
    {  629, 1395,     483,  5, }, // Atlantic/Faroe
    {  629, 6821,     488, 10, }, // Atlantic/St_Helena
    {  629, 2527,     514,  4, }, // Europe/Kyiv
    {  629, 7211,     547,  7, }, // Indian/Reunion
    {  629, 3157,     554,  5, }, // Pacific/Chuuk
    {  629, 3096,     586,  7, }, // Pacific/Pohnpei
    {  631, 6760,    4649,  4, }, // Africa/Accra Turkish/Latin/Turkey
    {  631, 7420,  195093,  7, }, // Africa/Algiers
    {  631,   14,       0,  6, }, // Africa/Asmara
    {  631, 5287,   21334,  9, }, // Africa/Brazzaville
    {  631, 2320,  195100,  6, }, // Africa/Cairo
    {  631, 4117,   33276, 10, }, // Africa/Casablanca
    {  631, 7449,  117267,  5, }, // Africa/Ceuta
    {  631, 6773,    6367,  7, }, // Africa/Conakry
    {  631, 5130,  195106, 10, }, // Africa/Dar_es_Salaam
    {  631, 5043,   21366,  6, }, // Africa/Djibouti
    {  631, 5166,  195116,  5, }, // Africa/El_Aaiun
    {  631, 4499,  195121,  4, }, // Africa/Juba
    {  631, 4524,   63499,  6, }, // Africa/Khartoum
    {  631, 5306,   21414,  7, }, // Africa/Kinshasa
    {  631, 5336,    6429,  8, }, // Africa/Libreville
    {  631, 5113,  195125,  8, }, // Africa/Mogadishu
    {  631, 4435,       6,  8, }, // Africa/Sao_Tome
    {  631, 2866,   98500,  7, }, // Africa/Tripoli
    {  631, 6554,  195133,  5, }, // Africa/Tunis
    {  631, 4252,      14,  8, }, // America/Asuncion
    {  631,  237,      22,  8, }, // America/Atikokan
    {  631, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  631, 3868,   33314,  6, }, // America/Cancun
    {  631, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  631, 5422,   21621, 10, }, // America/Costa_Rica
    {  631, 5723,      66,  7, }, // America/Curacao
    {  631, 5739,   21666,  8, }, // America/Dominica
    {  631,  481,      73, 13, }, // America/Indiana/Knox
    {  631, 7961,      86, 16, }, // America/Indiana/Marengo
    {  631, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  631, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  631, 8038,     139, 14, }, // America/Indiana/Vevay
    {  631, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  631, 8086,     171, 16, }, // America/Indiana/Winamac
    {  631, 2799,    3995,  7, }, // America/Jamaica
    {  631, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  631, 5932,     207, 22, }, // America/Lower_Princes
    {  631, 8220,    7126,  6, }, // America/Merida
    {  631, 1850,     235, 19, }, // America/Noronha
    {  631, 8301,  195138, 20, }, // America/North_Dakota/Beulah
    {  631, 8329,  195158, 20, }, // America/North_Dakota/Center
    {  631, 8357,  195178, 23, }, // America/North_Dakota/New_Salem
    {  631,  393,     317,  4, }, // America/Nuuk
    {  631, 5843,  166677, 10, }, // America/Puerto_Rico
    {  631, 6899,     321, 16, }, // America/Scoresbysund
    {  631, 8508,  117508, 16, }, // America/St_Barthelemy
    {  631, 2061,     351, 10, }, // America/St_Johns
    {  631, 5863,     361,  9, }, // America/St_Kitts
    {  631, 5880,     370,  9, }, // America/St_Lucia
    {  631,  855,     379, 10, }, // America/St_Thomas
    {  631, 5913,     389, 11, }, // America/St_Vincent
    {  631, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  631, 7295,   62201,  5, }, // Antarctica/Syowa
    {  631, 5395,   22312,  6, }, // Asia/Almaty
    {  631, 5511,   22318,  6, }, // Asia/Anadyr
    {  631, 5523,  195201,  5, }, // Asia/Aqtau
    {  631, 5368,   63587,  6, }, // Asia/Aqtobe
    {  631,  964,  195206,  7, }, // Asia/Ashgabat
    {  631, 8624,  195213,  6, }, // Asia/Atyrau
    {  631, 3607,  195219,  6, }, // Asia/Baghdad
    {  631, 5534,  195225,  7, }, // Asia/Bahrain
    {  631, 3653,  195232,  4, }, // Asia/Baku
    {  631, 4086,   22361,  6, }, // Asia/Beirut
    {  631, 3778,   22367,  6, }, // Asia/Bishkek
    {  631, 4601,    7631,  4, }, // Asia/Chita
    {  631, 4511,    5637,  7, }, // Asia/Colombo
    {  631, 4540,  117590,  3, }, // Asia/Damascus
    {  631, 1093,   91557,  5, }, // Asia/Dhaka
    {  631, 6159,  195236,  7, }, // Asia/Dushanbe
    {  631, 8636,  195243, 10, }, // Asia/Famagusta
    {  631, 8651,  195253,  5, }, // Asia/Gaza
    {  631, 4795,  195258,  8, }, // Asia/Hebron
    {  631, 1259,  195266, 17, }, // Asia/Ho_Chi_Minh
    {  631, 4184,   22433,  7, }, // Asia/Irkutsk
    {  631, 7006,   22440,  7, }, // Asia/Jakarta
    {  631, 1290,  195283,  5, }, // Asia/Jerusalem
    {  631, 3560,   22464,  5, }, // Asia/Kabul
    {  631, 4316,   22469,  8, }, // Asia/Kamchatka
    {  631, 4239,    7687,  6, }, // Asia/Karachi
    {  631, 1185,    5711,  8, }, // Asia/Kathmandu
    {  631, 8661,   33524,  7, }, // Asia/Khandyga
    {  631,  992,  195288,  7, }, // Asia/Kolkata
    {  631, 5976,    7719,  6, }, // Asia/Kuching
    {  631, 5547,  195295,  6, }, // Asia/Kuwait
    {  631, 1211,    7731,  5, }, // Asia/Macau
    {  631, 6920,   26882,  6, }, // Asia/Muscat
    {  631, 2554,  195301,  7, }, // Asia/Nicosia
    {  631, 5559,    4332,  5, }, // Asia/Qatar
    {  631, 8708,     469,  8, }, // Asia/Qostanay
    {  631, 4269,  195308,  9, }, // Asia/Qyzylorda
    {  631, 3584,   20874,  5, }, // Asia/Riyadh
    {  631, 4408,   33576,  7, }, // Asia/Sakhalin
    {  631, 7245,  117728,  9, }, // Asia/Samarkand
    {  631, 3279,    7798,  4, }, // Asia/Seoul
    {  631, 1053,  117741,  7, }, // Asia/Shanghai
    {  631, 3300,   20900,  8, }, // Asia/Singapore
    {  631, 4781,  117762,  7, }, // Asia/Tashkent
    {  631, 3915,   78237,  6, }, // Asia/Tbilisi
    {  631, 2772,  195317,  6, }, // Asia/Tehran
    {  631, 1021,  195323, 10, }, // Asia/Ulaanbaatar
    {  631, 1159,   22607,  6, }, // Asia/Urumqi
    {  631, 1235,     477,  6, }, // Asia/Yangon
    {  631, 3747,  128099,  6, }, // Asia/Yerevan
    {  631, 3663,  195333, 12, }, // Atlantic/Azores
    {  631, 6628,  195345, 15, }, // Atlantic/Canary
    {  631, 1395,     483,  5, }, // Atlantic/Faroe
    {  631, 8736,  195360, 15, }, // Atlantic/Madeira
    {  631, 7272,  195375, 13, }, // Atlantic/South_Georgia
    {  631, 6821,     488, 10, }, // Atlantic/St_Helena
    {  631, 1463,   22739,  6, }, // Australia/Sydney
    {  631, 3620,   33644,  8, }, // Europe/Astrakhan
    {  631, 2300,   33652,  5, }, // Europe/Athens
    {  631, 6336,   27015,  7, }, // Europe/Belgrade
    {  631, 2167,  195388,  7, }, // Europe/Brussels
    {  631, 3928,  195395,  6, }, // Europe/Bucharest
    {  631, 3806,  195401,  9, }, // Europe/Budapest
    {  631, 8772,   27037,  8, }, // Europe/Busingen
    {  631, 2583,  195410,  7, }, // Europe/Chisinau
    {  631, 6289,    4475,  8, }, // Europe/Copenhagen
    {  631, 6307,  195417, 11, }, // Europe/Gibraltar
    {  631, 8804,  195428,  9, }, // Europe/Isle_of_Man
    {  631, 1130,   22825,  8, }, // Europe/Istanbul
    {  631, 2527,    8148,  4, }, // Europe/Kyiv
    {  631, 3213,  157721,  6, }, // Europe/Lisbon
    {  631, 2501,   95370,  6, }, // Europe/London
    {  631, 6365,  195437, 10, }, // Europe/Luxembourg
    {  631, 6410,    6044,  6, }, // Europe/Monaco
    {  631, 3537,   64446,  7, }, // Europe/Moscow
    {  631, 6275,    8201,  4, }, // Europe/Prague
    {  631, 6324,    6064,  4, }, // Europe/Rome
    {  631, 6245,  118027, 10, }, // Europe/Sarajevo
    {  631, 6383,  195447,  5, }, // Europe/Skopje
    {  631, 6599,   99330,  5, }, // Europe/Sofia
    {  631, 6523,    8242,  8, }, // Europe/Stockholm
    {  631, 6202,    8255,  5, }, // Europe/Tirane
    {  631, 6567,    6096,  7, }, // Europe/Vatican
    {  631, 6231,  118059,  6, }, // Europe/Vienna
    {  631, 3190,  195452,  7, }, // Europe/Warsaw
    {  631, 6540,  195459,  5, }, // Europe/Zurich
    {  631, 5029,    8305,  5, }, // Indian/Comoro
    {  631, 7050,  195464,  9, }, // Indian/Maldives
    {  631, 7211,     547,  7, }, // Indian/Reunion
    {  631, 3157,     554,  5, }, // Pacific/Chuuk
    {  631, 2237,  195473, 14, }, // Pacific/Easter
    {  631, 2723,    4611,  8, }, // Pacific/Honolulu
    {  631, 3049,    4619,  9, }, // Pacific/Kanton
    {  631, 4051,  195487, 14, }, // Pacific/Marquesas
    {  631, 3096,     586,  7, }, // Pacific/Pohnpei
    {  633, 6788,  195501,  6, }, // Africa/Abidjan Turkmen/Latin/Turkmenistan
    {  633, 6760,   21292,  5, }, // Africa/Accra
    {  633, 5059,   64622, 11, }, // Africa/Addis_Ababa
    {  633, 7420,   57579,  5, }, // Africa/Algiers
    {  633,   14,       0,  6, }, // Africa/Asmara
    {  633, 5257,   26480,  5, }, // Africa/Bangui
    {  633, 7435,    6313,  5, }, // Africa/Bissau
    {  633, 4957,  195507,  8, }, // Africa/Blantyre
    {  633, 5287,  195515,  9, }, // Africa/Brazzaville
    {  633, 2320,    4690,  4, }, // Africa/Cairo
    {  633, 4117,    4695, 10, }, // Africa/Casablanca
    {  633, 7449,   21349,  5, }, // Africa/Ceuta
    {  633, 6773,    6367,  7, }, // Africa/Conakry
    {  633, 5130,    6374, 12, }, // Africa/Dar_es_Salaam
    {  633, 5043,  158501,  6, }, // Africa/Djibouti
    {  633, 5243,   21372,  5, }, // Africa/Douala
    {  633, 5166,  195524,  7, }, // Africa/El_Aaiun
    {  633, 6853,    6400,  7, }, // Africa/Freetown
    {  633, 4479,  195531, 12, }, // Africa/Johannesburg
    {  633, 4524,   63499,  6, }, // Africa/Khartoum
    {  633, 5306,   21414,  7, }, // Africa/Kinshasa
    {  633, 5336,  195543,  8, }, // Africa/Libreville
    {  633, 4939,   21426,  9, }, // Africa/Lubumbashi
    {  633, 5113,  195551,  8, }, // Africa/Mogadishu
    {  633, 7034,  195559,  9, }, // Africa/Monrovia
    {  633, 3853,  195568,  7, }, // Africa/Nairobi
    {  633, 5271,  195575,  6, }, // Africa/Ndjamena
    {  633, 5354,  195581,  6, }, // Africa/Niamey
    {  633, 6803,  195587,  7, }, // Africa/Nouakchott
    {  633, 6727,   26534,  8, }, // Africa/Ouagadougou
    {  633, 5225,  195594, 10, }, // Africa/Porto-Novo
    {  633, 4435,  195604,  8, }, // Africa/Sao_Tome
    {  633, 4152,   56808,  7, }, // Africa/Windhoek
    {  633,  132,  195612, 10, }, // America/Adak
    {  633, 3346,  125023,  8, }, // America/Anchorage
    {  633, 5607,  195622,  7, }, // America/Anguilla
    {  633, 4554,  195629,  9, }, // America/Araguaina
    {  633,  166,  195638, 12, }, // America/Argentina/Buenos_Aires
    {  633,   91,    6518,  9, }, // America/Argentina/Catamarca
    {  633,  270,  195650,  7, }, // America/Argentina/Cordoba
    {  633,  441,  195657,  5, }, // America/Argentina/Jujuy
    {  633, 7462,  195662,  8, }, // America/Argentina/La_Rioja
    {  633,  565,   97634,  7, }, // America/Argentina/Mendoza
    {  633, 7489,    6547, 11, }, // America/Argentina/Rio_Gallegos
    {  633, 7544,    6563,  8, }, // America/Argentina/San_Juan
    {  633, 5580,  195670,  8, }, // America/Argentina/San_Luis
    {  633, 7571,    4842,  7, }, // America/Argentina/Tucuman
    {  633, 7597,  195678,  6, }, // America/Argentina/Ushuaia
    {  633, 4252,  195684,  8, }, // America/Asuncion
    {  633,  237,      22,  8, }, // America/Atikokan
    {  633, 3679,  195692,  5, }, // America/Bahia
    {  633, 7623,  195697, 17, }, // America/Bahia_Banderas
    {  633, 7646,  122396,  5, }, // America/Belem
    {  633, 5407,    6614,  5, }, // America/Belize
    {  633, 7660,    6620, 12, }, // America/Blanc-Sablon
    {  633, 7681,  195714,  9, }, // America/Boa_Vista
    {  633, 7699,  195723,  5, }, // America/Boise
    {  633, 7713,  195728, 11, }, // America/Cambridge_Bay
    {  633, 7735,  195739, 12, }, // America/Campo_Grande
    {  633, 3868,    6682,  6, }, // America/Cancun
    {  633, 4694,    4910,  7, }, // America/Caracas
    {  633, 4345,  195751,  7, }, // America/Cayenne
    {  633, 5496,  195758, 14, }, // America/Cayman
    {  633, 2260,    6700,  6, }, // America/Chicago
    {  633, 7756,   21613,  8, }, // America/Chihuahua
    {  633, 7774,  195772, 13, }, // America/Ciudad_Juarez
    {  633, 5422,  195785, 10, }, // America/Costa_Rica
    {  633, 7796,    6736,  7, }, // America/Creston
    {  633, 3791,  195795,  6, }, // America/Cuiaba
    {  633, 5723,  195801,  8, }, // America/Curacao
    {  633, 7812,  195809, 12, }, // America/Danmarkshavn
    {  633, 7833,    6769,  6, }, // America/Dawson
    {  633, 7848,  195821, 11, }, // America/Dawson_Creek
    {  633,  805,  195832,  6, }, // America/Denver
    {  633, 3465,  195838,  7, }, // America/Detroit
    {  633, 5739,   21666,  8, }, // America/Dominica
    {  633, 7869,  195845,  8, }, // America/Eirunepe
    {  633, 5441,  157151,  8, }, // America/El_Salvador
    {  633, 7924,  195853,  9, }, // America/Glace_Bay
    {  633, 6881,  195862,  7, }, // America/Goose_Bay
    {  633, 4612,  195869, 10, }, // America/Grand_Turk
    {  633, 5786,  157159,  9, }, // America/Guadeloupe
    {  633, 3760,  134472,  9, }, // America/Guatemala
    {  633, 4841,  195879,  7, }, // America/Guayaquil
    {  633, 6932,  195886,  6, }, // America/Guyana
    {  633, 1939,  195892,  8, }, // America/Halifax
    {  633, 2281,  195900,  6, }, // America/Havana
    {  633, 7942,  195906,  8, }, // America/Hermosillo
    {  633,  481,  195914, 13, }, // America/Indiana/Knox
    {  633, 7961,      86, 16, }, // America/Indiana/Marengo
    {  633, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  633, 8012,    6961, 18, }, // America/Indiana/Tell_City
    {  633, 8038,  195927, 14, }, // America/Indiana/Vevay
    {  633, 8060,  195941, 16, }, // America/Indiana/Vincennes
    {  633, 8086,    5161, 16, }, // America/Indiana/Winamac
    {  633, 8110,  195957,  6, }, // America/Inuvik
    {  633,  660,    5177,  7, }, // America/Iqaluit
    {  633, 2799,  195963,  7, }, // America/Jamaica
    {  633, 5380,  122825,  5, }, // America/Juneau
    {  633,  521,  195970,  8, }, // America/Kentucky/Louisville
    {  633, 8125,  195978, 19, }, // America/Kentucky/Monticello
    {  633, 5704,  195997, 10, }, // America/Kralendijk
    {  633, 4376,  196007,  6, }, // America/La_Paz
    {  633, 3239,  196013, 11, }, // America/Los_Angeles
    {  633, 5932,  196024, 21, }, // America/Lower_Princes
    {  633, 8153,  196045,  6, }, // America/Maceio
    {  633, 5897,   61871,  6, }, // America/Marigot
    {  633, 5805,   26594,  9, }, // America/Martinique
    {  633, 8202,    7118,  8, }, // America/Menominee
    {  633, 8220,    7126,  6, }, // America/Merida
    {  633, 2949,  122958,  6, }, // America/Mexico_City
    {  633, 4391,    7149,  7, }, // America/Miquelon
    {  633, 8254,    7156,  7, }, // America/Moncton
    {  633, 8270,  196051,  9, }, // America/Monterrey
    {  633, 4098,  196060, 10, }, // America/Montevideo
    {  633, 5824,  158564,  9, }, // America/Montserrat
    {  633, 2379,  196070,  8, }, // America/New_York
    {  633, 8288,   21968,  3, }, // America/Nome
    {  633, 1850,     235, 19, }, // America/Noronha
    {  633, 8301,  196078, 24, }, // America/North_Dakota/Beulah
    {  633, 8329,  196102, 24, }, // America/North_Dakota/Center
    {  633, 8357,  196126, 28, }, // America/North_Dakota/New_Salem
    {  633,  393,     317,  4, }, // America/Nuuk
    {  633, 8388,    7276,  7, }, // America/Ojinaga
    {  633, 2973,    7283,  6, }, // America/Phoenix
    {  633, 3945,  166011, 12, }, // America/Port-au-Prince
    {  633, 5954,  196154, 13, }, // America/Port_of_Spain
    {  633, 8404,  196167, 11, }, // America/Porto_Velho
    {  633, 5843,  196178, 11, }, // America/Puerto_Rico
    {  633, 4030,    7322, 12, }, // America/Punta_Arenas
    {  633, 8424,   93788, 12, }, // America/Rankin_Inlet
    {  633, 8445,  123152,  6, }, // America/Recife
    {  633, 1995,  196189,  7, }, // America/Regina
    {  633, 8460,  196196,  8, }, // America/Resolute
    {  633,  695,  196204, 10, }, // America/Rio_Branco
    {  633, 2201,  196214,  8, }, // America/Santiago
    {  633, 6111,    7362, 13, }, // America/Santo_Domingo
    {  633, 1878,  196222,  9, }, // America/Sao_Paulo
    {  633, 6899,  196231, 16, }, // America/Scoresbysund
    {  633, 8508,    7396, 13, }, // America/St_Barthelemy
    {  633, 2061,  196247,  9, }, // America/St_Johns
    {  633, 5863,  196256,  9, }, // America/St_Kitts
    {  633, 5880,  196265, 12, }, // America/St_Lucia
    {  633,  855,  196277, 10, }, // America/St_Thomas
    {  633, 5913,  196287, 12, }, // America/St_Vincent
    {  633, 8530,  196299, 12, }, // America/Swift_Current
    {  633, 5461,    5528, 11, }, // America/Tegucigalpa
    {  633, 5756,  123306,  4, }, // America/Thule
    {  633,  313,    7479,  7, }, // America/Tijuana
    {  633, 2093,  196311,  8, }, // America/Vancouver
    {  633, 2144,  196319,  8, }, // America/Whitehorse
    {  633, 8552,  196327,  7, }, // America/Yakutat
    {  633, 6001,  196334,  5, }, // Antarctica/Casey
    {  633, 6094,  196339,  6, }, // Antarctica/Davis
    {  633, 6133,  196345, 15, }, // Antarctica/DumontDUrville
    {  633, 8568,  196360,  8, }, // Antarctica/Macquarie
    {  633, 7066,   22285,  6, }, // Antarctica/Mawson
    {  633, 7113,  196368,  9, }, // Antarctica/McMurdo
    {  633, 7226,    5583,  6, }, // Antarctica/Rothera
    {  633, 7295,  196377,  5, }, // Antarctica/Syowa
    {  633, 8607,  123400,  4, }, // Antarctica/Troll
    {  633, 7374,    4147,  6, }, // Antarctica/Vostok
    {  633, 1429,  196382,  6, }, // Arctic/Longyearbyen
    {  633, 5523,    5609,  5, }, // Asia/Aqtau
    {  633, 5368,    7606,  6, }, // Asia/Aqtobe
    {  633,  964,   44322,  7, }, // Asia/Ashgabat
    {  633, 3607,  196388,  6, }, // Asia/Baghdad
    {  633, 5534,  196394,  7, }, // Asia/Bahrain
    {  633, 4086,  196401,  6, }, // Asia/Beirut
    {  633, 3778,   22367,  6, }, // Asia/Bishkek
    {  633, 5989,  196407,  6, }, // Asia/Brunei
    {  633, 4601,    7631,  4, }, // Asia/Chita
    {  633, 4511,    5637,  7, }, // Asia/Colombo
    {  633, 4540,    4201,  6, }, // Asia/Damascus
    {  633, 1093,   91557,  5, }, // Asia/Dhaka
    {  633, 3596,  196413,  5, }, // Asia/Dubai
    {  633, 6159,  195236,  7, }, // Asia/Dushanbe
    {  633, 4795,  196418,  6, }, // Asia/Hebron
    {  633, 1259,  196424,  7, }, // Asia/Ho_Chi_Minh
    {  633, 2704,  196431,  7, }, // Asia/Hong_Kong
    {  633, 4771,  196438,  4, }, // Asia/Hovd
    {  633, 6992,  196442,  8, }, // Asia/Jayapura
    {  633, 1290,  196450, 10, }, // Asia/Jerusalem
    {  633, 4316,   22469,  8, }, // Asia/Kamchatka
    {  633, 4239,    7687,  6, }, // Asia/Karachi
    {  633, 1185,    5711,  8, }, // Asia/Kathmandu
    {  633, 8661,   64327,  7, }, // Asia/Khandyga
    {  633,  992,   64334,  8, }, // Asia/Kolkata
    {  633, 4197,  196460, 11, }, // Asia/Krasnoyarsk
    {  633, 4859,    7707, 12, }, // Asia/Kuala_Lumpur
    {  633, 5976,    7719,  6, }, // Asia/Kuching
    {  633, 5547,  196471,  6, }, // Asia/Kuwait
    {  633, 1211,    7731,  5, }, // Asia/Macau
    {  633, 1349,    7736,  7, }, // Asia/Makassar
    {  633, 6920,   26882,  6, }, // Asia/Muscat
    {  633, 2554,  196477,  8, }, // Asia/Nicosia
    {  633, 8675,  196485, 11, }, // Asia/Novokuznetsk
    {  633, 4135,   78211, 11, }, // Asia/Novosibirsk
    {  633, 6961,   62499,  7, }, // Asia/Phnom_Penh
    {  633, 4214,  196496,  7, }, // Asia/Pyongyang
    {  633, 5559,    4332,  5, }, // Asia/Qatar
    {  633, 8708,  196503,  8, }, // Asia/Qostanay
    {  633, 4269,  196511,  9, }, // Asia/Qyzylorda
    {  633, 3584,  196520,  8, }, // Asia/Riyadh
    {  633, 4408,   33576,  7, }, // Asia/Sakhalin
    {  633, 3279,    7798,  4, }, // Asia/Seoul
    {  633, 1053,  196528,  6, }, // Asia/Shanghai
    {  633, 3300,   20900,  8, }, // Asia/Singapore
    {  633, 3263,  196534,  6, }, // Asia/Taipei
    {  633, 4781,  117762,  7, }, // Asia/Tashkent
    {  633, 2772,  196540,  6, }, // Asia/Tehran
    {  633, 1317,   91770,  5, }, // Asia/Thimphu
    {  633, 2821,    4356,  5, }, // Asia/Tokyo
    {  633, 1021,  147172, 10, }, // Asia/Ulaanbaatar
    {  633, 1159,   22607,  6, }, // Asia/Urumqi
    {  633, 6977,  196546,  8, }, // Asia/Vientiane
    {  633, 4710,    4361, 11, }, // Asia/Vladivostok
    {  633, 4828,  196554,  7, }, // Asia/Yakutsk
    {  633, 1235,  196561,  6, }, // Asia/Yangon
    {  633, 3883,  196567, 13, }, // Asia/Yekaterinburg
    {  633, 3747,  196580,  7, }, // Asia/Yerevan
    {  633, 3663,  196587, 12, }, // Atlantic/Azores
    {  633, 6628,  196599, 13, }, // Atlantic/Canary
    {  633, 3727,  196612, 10, }, // Atlantic/Cape_Verde
    {  633, 1395,  196622, 13, }, // Atlantic/Faroe
    {  633, 8736,  196635, 15, }, // Atlantic/Madeira
    {  633, 2748,  196650,  9, }, // Atlantic/Reykjavik
    {  633, 7272,  196659, 16, }, // Atlantic/South_Georgia
    {  633, 6821,  196675, 22, }, // Atlantic/St_Helena
    {  633, 6644,    7965,  6, }, // Atlantic/Stanley
    {  633, 1670,   21030,  8, }, // Australia/Adelaide
    {  633, 1635,  123965,  7, }, // Australia/Brisbane
    {  633, 1799,   93948, 10, }, // Australia/Broken_Hill
    {  633, 3637,  196697,  5, }, // Australia/Eucla
    {  633, 1547,  196702,  8, }, // Australia/Lord_Howe
    {  633, 1727,    8021,  7, }, // Australia/Melbourne
    {  633, 1762,   22735,  4, }, // Australia/Perth
    {  633, 1463,  196710,  6, }, // Australia/Sydney
    {  633, 3620,   33644,  8, }, // Europe/Astrakhan
    {  633, 2300,  196716,  5, }, // Europe/Athens
    {  633, 6336,   27015,  7, }, // Europe/Belgrade
    {  633, 6488,  196721, 10, }, // Europe/Bratislava
    {  633, 2167,  196731,  8, }, // Europe/Brussels
    {  633, 3928,  196739,  8, }, // Europe/Bucharest
    {  633, 3806,   22781,  8, }, // Europe/Budapest
    {  633, 8772,  196747,  9, }, // Europe/Busingen
    {  633, 2583,  196756,  8, }, // Europe/Chisinau
    {  633, 6289,  196764, 10, }, // Europe/Copenhagen
    {  633, 8788,  127234,  6, }, // Europe/Guernsey
    {  633, 8804,  196774,  9, }, // Europe/Isle_of_Man
    {  633, 1130,  124167,  7, }, // Europe/Istanbul
    {  633, 8823,  196783,  5, }, // Europe/Jersey
    {  633, 8929,   78287,  5, }, // Europe/Kirov
    {  633, 2527,  196788,  5, }, // Europe/Kyiv
    {  633, 3213,    4499,  8, }, // Europe/Lisbon
    {  633, 6506,  196793,  9, }, // Europe/Ljubljana
    {  633, 6365,  196802, 11, }, // Europe/Luxembourg
    {  633, 6582,  196813, 10, }, // Europe/Mariehamn
    {  633, 6410,    6044,  6, }, // Europe/Monaco
    {  633, 3537,   93233,  6, }, // Europe/Moscow
    {  633, 4284,  196823,  5, }, // Europe/Paris
    {  633, 6424,  196828,  9, }, // Europe/Podgorica
    {  633, 6275,   21185,  5, }, // Europe/Prague
    {  633, 6324,   33740,  3, }, // Europe/Rome
    {  633, 6470,    8214, 10, }, // Europe/San_Marino
    {  633, 6245,  196837,  8, }, // Europe/Sarajevo
    {  633, 4451,   78296,  7, }, // Europe/Saratov
    {  633, 6383,  196845,  6, }, // Europe/Skopje
    {  633, 6599,  196851,  6, }, // Europe/Sofia
    {  633, 6523,  196857,  8, }, // Europe/Stockholm
    {  633, 8867,   22925,  6, }, // Europe/Tallinn
    {  633, 6202,   21213,  6, }, // Europe/Tirane
    {  633, 8882,  196865,  9, }, // Europe/Ulyanovsk
    {  633, 6352,  196874,  5, }, // Europe/Vaduz
    {  633, 6567,  196879,  7, }, // Europe/Vatican
    {  633, 6231,  196886,  4, }, // Europe/Vienna
    {  633, 8899,  196890,  7, }, // Europe/Vilnius
    {  633, 4727,    4561,  9, }, // Europe/Volgograd
    {  633, 3190,  196897,  7, }, // Europe/Warsaw
    {  633, 6540,  196904,  6, }, // Europe/Zurich
    {  633, 5078,  196910, 12, }, // Indian/Antananarivo
    {  633, 6947,    8288,  5, }, // Indian/Chagos
    {  633, 6046,  196922,  9, }, // Indian/Christmas
    {  633, 6063,    4583,  5, }, // Indian/Cocos
    {  633, 5029,  196931, 13, }, // Indian/Comoro
    {  633, 6661,   22991,  8, }, // Indian/Kerguelen
    {  633, 7260,  196944,  4, }, // Indian/Mahe
    {  633, 7050,  196948,  9, }, // Indian/Maldives
    {  633, 4069,  196957,  8, }, // Indian/Mauritius
    {  633, 5098,  196965,  7, }, // Indian/Mayotte
    {  633, 7211,  196972,  8, }, // Indian/Reunion
    {  633,  932,  124498,  6, }, // Pacific/Auckland
    {  633, 3706,  196980,  8, }, // Pacific/Bougainville
    {  633, 3015,  196988,  5, }, // Pacific/Chatham
    {  633, 3157,   23025,  4, }, // Pacific/Chuuk
    {  633, 2237,  196993, 11, }, // Pacific/Easter
    {  633, 6678,  197004, 17, }, // Pacific/Galapagos
    {  633, 6696,  197021,  6, }, // Pacific/Gambier
    {  633, 3822,  197027, 11, }, // Pacific/Guadalcanal
    {  633, 2723,  197038,  8, }, // Pacific/Honolulu
    {  633, 3049,  124586,  9, }, // Pacific/Kanton
    {  633, 7019,  197046,  7, }, // Pacific/Kosrae
    {  633, 2842,  197053,  9, }, // Pacific/Kwajalein
    {  633, 4051,  197062, 14, }, // Pacific/Marquesas
    {  633, 8914,  197076,  6, }, // Pacific/Midway
    {  633, 7098,  124660,  5, }, // Pacific/Noumea
    {  633, 3126,    8413,  9, }, // Pacific/Pago_Pago
    {  633, 7194,    8422,  7, }, // Pacific/Pitcairn
    {  633, 3096,    8429,  6, }, // Pacific/Pohnpei
    {  633, 4807,  197082, 11, }, // Pacific/Port_Moresby
    {  633, 6031,  197093,  6, }, // Pacific/Saipan
    {  633, 7312,  124697,  5, }, // Pacific/Tahiti
    {  633, 7392,  197099,  4, }, // Pacific/Wake
    {  633, 7405,    8467,  6, }, // Pacific/Wallis
    {  634,   14,       0,  6, }, // Africa/Asmara Tyap/Latin/Nigeria
    {  634, 4435,       6,  8, }, // Africa/Sao_Tome
    {  634, 4252,      14,  8, }, // America/Asuncion
    {  634,  237,      22,  8, }, // America/Atikokan
    {  634, 7623,      30, 17, }, // America/Bahia_Banderas
    {  634, 3868,      47,  6, }, // America/Cancun
    {  634, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  634, 5723,      66,  7, }, // America/Curacao
    {  634,  481,      73, 13, }, // America/Indiana/Knox
    {  634, 7961,      86, 16, }, // America/Indiana/Marengo
    {  634, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  634, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  634, 8038,     139, 14, }, // America/Indiana/Vevay
    {  634, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  634, 8086,     171, 16, }, // America/Indiana/Winamac
    {  634, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  634, 5932,     207, 22, }, // America/Lower_Princes
    {  634, 8220,     229,  6, }, // America/Merida
    {  634, 1850,     235, 19, }, // America/Noronha
    {  634, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  634, 8329,     274, 20, }, // America/North_Dakota/Center
    {  634, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  634,  393,     317,  4, }, // America/Nuuk
    {  634, 6899,     321, 16, }, // America/Scoresbysund
    {  634, 8508,     337, 14, }, // America/St_Barthelemy
    {  634, 2061,     351, 10, }, // America/St_Johns
    {  634, 5863,     361,  9, }, // America/St_Kitts
    {  634, 5880,     370,  9, }, // America/St_Lucia
    {  634,  855,     379, 10, }, // America/St_Thomas
    {  634, 5913,     389, 11, }, // America/St_Vincent
    {  634, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  634, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  634, 1185,     448,  9, }, // Asia/Kathmandu
    {  634,  992,     457,  7, }, // Asia/Kolkata
    {  634, 1211,     464,  5, }, // Asia/Macau
    {  634, 1235,     477,  6, }, // Asia/Yangon
    {  634, 1395,     483,  5, }, // Atlantic/Faroe
    {  634, 6821,     488, 10, }, // Atlantic/St_Helena
    {  634, 2527,     514,  4, }, // Europe/Kyiv
    {  634, 7211,     547,  7, }, // Indian/Reunion
    {  634, 3157,     554,  5, }, // Pacific/Chuuk
    {  634, 3096,     586,  7, }, // Pacific/Pohnpei
    {  635, 6788,   27171,  7, }, // Africa/Abidjan Ukrainian/Cyrillic/Ukraine
    {  635, 6760,     610,  5, }, // Africa/Accra
    {  635, 5059,     615, 11, }, // Africa/Addis_Ababa
    {  635, 7420,     626,  5, }, // Africa/Algiers
    {  635,   14,     631,  6, }, // Africa/Asmara
    {  635,   44,     637,  6, }, // Africa/Bamako
    {  635, 5257,  197103,  5, }, // Africa/Bangui
    {  635, 6746,     648,  6, }, // Africa/Banjul
    {  635, 7435,   27215,  5, }, // Africa/Bissau
    {  635, 4957,   27220,  8, }, // Africa/Blantyre
    {  635, 5287,  197108, 10, }, // Africa/Brazzaville
    {  635, 4922,     676,  9, }, // Africa/Bujumbura
    {  635, 2320,  197118,  4, }, // Africa/Cairo
    {  635, 4117,     689, 10, }, // Africa/Casablanca
    {  635, 7449,     699,  5, }, // Africa/Ceuta
    {  635, 6773,  197122,  7, }, // Africa/Conakry
    {  635, 6840,     711,  5, }, // Africa/Dakar
    {  635, 5130,     716, 12, }, // Africa/Dar_es_Salaam
    {  635, 5043,  197129,  7, }, // Africa/Djibouti
    {  635, 5243,     734,  5, }, // Africa/Douala
    {  635, 5166,  197136,  7, }, // Africa/El_Aaiun
    {  635, 6853,  197143,  7, }, // Africa/Freetown
    {  635, 4906,  197150,  8, }, // Africa/Gaborone
    {  635, 5015,   33913,  6, }, // Africa/Harare
    {  635, 4479,  197158, 12, }, // Africa/Johannesburg
    {  635, 4499,   27305,  5, }, // Africa/Juba
    {  635, 5151,     783,  7, }, // Africa/Kampala
    {  635, 4524,     790,  6, }, // Africa/Khartoum
    {  635, 4973,   27310,  6, }, // Africa/Kigali
    {  635, 5306,   27316,  7, }, // Africa/Kinshasa
    {  635, 4744,  197170,  5, }, // Africa/Lagos
    {  635, 5336,  197175,  9, }, // Africa/Libreville
    {  635, 6869,     822,  4, }, // Africa/Lome
    {  635, 5211,     826,  6, }, // Africa/Luanda
    {  635, 4939,  197184,  9, }, // Africa/Lubumbashi
    {  635, 5001,     841,  6, }, // Africa/Lusaka
    {  635, 5322,     847,  6, }, // Africa/Malabo
    {  635, 4987,     853,  6, }, // Africa/Maputo
    {  635, 5197,     859,  6, }, // Africa/Maseru
    {  635, 5182,     865,  7, }, // Africa/Mbabane
    {  635, 5113,  197193,  8, }, // Africa/Mogadishu
    {  635, 7034,  197201,  8, }, // Africa/Monrovia
    {  635, 3853,   27379,  7, }, // Africa/Nairobi
    {  635, 5271,   27386,  8, }, // Africa/Ndjamena
    {  635, 5354,  197209,  6, }, // Africa/Niamey
    {  635, 6803,  197215,  8, }, // Africa/Nouakchott
    {  635, 6727,  197223,  8, }, // Africa/Ouagadougou
    {  635, 5225,     923, 10, }, // Africa/Porto-Novo
    {  635, 4435,     933,  8, }, // Africa/Sao_Tome
    {  635, 2866,  197231,  7, }, // Africa/Tripoli
    {  635, 6554,   27425,  5, }, // Africa/Tunis
    {  635, 4152,  197238,  7, }, // Africa/Windhoek
    {  635,  132,     960,  4, }, // America/Adak
    {  635, 3346,   51593,  8, }, // America/Anchorage
    {  635, 5607,  197245,  7, }, // America/Anguilla
    {  635, 5624,  197252,  7, }, // America/Antigua
    {  635, 4554,  197259,  9, }, // America/Araguaina
    {  635,  166,  197268, 12, }, // America/Argentina/Buenos_Aires
    {  635,   91,    1006,  9, }, // America/Argentina/Catamarca
    {  635,  270,    1015,  7, }, // America/Argentina/Cordoba
    {  635,  441,   27487,  5, }, // America/Argentina/Jujuy
    {  635, 7462,  197280,  8, }, // America/Argentina/La_Rioja
    {  635,  565,    1035,  7, }, // America/Argentina/Mendoza
    {  635, 7489,  197288, 12, }, // America/Argentina/Rio_Gallegos
    {  635, 7520,   27512,  6, }, // America/Argentina/Salta
    {  635, 7544,    1058,  8, }, // America/Argentina/San_Juan
    {  635, 5580,  197300,  8, }, // America/Argentina/San_Luis
    {  635, 7571,    1074,  7, }, // America/Argentina/Tucuman
    {  635, 7597,   27526,  5, }, // America/Argentina/Ushuaia
    {  635, 5640,    1087,  5, }, // America/Aruba
    {  635, 4252,  197308,  9, }, // America/Asuncion
    {  635,  237,  197317,  8, }, // America/Atikokan
    {  635, 3679,  137274,  4, }, // America/Bahia
    {  635, 7623,  197325, 13, }, // America/Bahia_Banderas
    {  635, 5654,    1131,  8, }, // America/Barbados
    {  635, 7646,    1139,  5, }, // America/Belem
    {  635, 5407,   27575,  5, }, // America/Belize
    {  635, 7660,   36520, 11, }, // America/Blanc-Sablon
    {  635, 7681,   27580,  9, }, // America/Boa_Vista
    {  635, 4361,  197338,  6, }, // America/Bogota
    {  635, 7699,   51656,  5, }, // America/Boise
    {  635, 7713,  197344, 13, }, // America/Cambridge_Bay
    {  635, 7735,  197357, 12, }, // America/Campo_Grande
    {  635, 3868,    1205,  6, }, // America/Cancun
    {  635, 4694,    1211,  7, }, // America/Caracas
    {  635, 4345,  197369,  6, }, // America/Cayenne
    {  635, 5496,  197375, 17, }, // America/Cayman
    {  635, 2260,  197392,  6, }, // America/Chicago
    {  635, 7756,  197398,  6, }, // America/Chihuahua
    {  635, 7774,  197404, 13, }, // America/Ciudad_Juarez
    {  635, 5422,  197417, 10, }, // America/Costa_Rica
    {  635, 7796,    1269,  7, }, // America/Creston
    {  635, 3791,   27688,  5, }, // America/Cuiaba
    {  635, 5723,   36606,  7, }, // America/Curacao
    {  635, 7812,    1290, 12, }, // America/Danmarkshavn
    {  635, 7833,    1302,  6, }, // America/Dawson
    {  635, 7848,  197427, 11, }, // America/Dawson_Creek
    {  635,  805,    1319,  6, }, // America/Denver
    {  635, 3465,   36649,  7, }, // America/Detroit
    {  635, 5739,  197438,  8, }, // America/Dominica
    {  635,  893,    1340,  8, }, // America/Edmonton
    {  635, 7869,   36664,  8, }, // America/Eirunepe
    {  635, 5441,   27765,  9, }, // America/El_Salvador
    {  635, 7886,   51697, 12, }, // America/Fort_Nelson
    {  635, 7906,    1375,  9, }, // America/Fortaleza
    {  635, 7924,  197446,  9, }, // America/Glace_Bay
    {  635, 6881,  197455,  7, }, // America/Goose_Bay
    {  635, 4612,  197462, 10, }, // America/Grand_Turk
    {  635, 5770,  197472,  7, }, // America/Grenada
    {  635, 5786,  197479,  9, }, // America/Guadeloupe
    {  635, 3760,  197488,  9, }, // America/Guatemala
    {  635, 4841,  197497,  8, }, // America/Guayaquil
    {  635, 6932,  197505,  6, }, // America/Guyana
    {  635, 1939,   27859,  8, }, // America/Halifax
    {  635, 2281,    1457,  6, }, // America/Havana
    {  635, 7942,  197511, 10, }, // America/Hermosillo
    {  635,  348,  197521, 12, }, // America/Indiana/Indianapolis
    {  635,  481,  197533, 13, }, // America/Indiana/Knox
    {  635, 7961,  197546, 16, }, // America/Indiana/Marengo
    {  635, 7985,  197562, 19, }, // America/Indiana/Petersburg
    {  635, 8012,  197581, 18, }, // America/Indiana/Tell_City
    {  635, 8038,  197599, 14, }, // America/Indiana/Vevay
    {  635, 8060,  197613, 16, }, // America/Indiana/Vincennes
    {  635, 8086,  197629, 16, }, // America/Indiana/Winamac
    {  635, 8110,   27998,  6, }, // America/Inuvik
    {  635,  660,  197645,  7, }, // America/Iqaluit
    {  635, 2799,   28011,  6, }, // America/Jamaica
    {  635, 5380,   36746,  5, }, // America/Juneau
    {  635,  521,  197652,  8, }, // America/Kentucky/Louisville
    {  635, 8125,  197660, 20, }, // America/Kentucky/Monticello
    {  635, 5704,   36751, 10, }, // America/Kralendijk
    {  635, 4376,    1632,  6, }, // America/La_Paz
    {  635, 7169,   28057,  4, }, // America/Lima
    {  635, 3239,   51761, 12, }, // America/Los_Angeles
    {  635, 5932,  197680, 20, }, // America/Lower_Princes
    {  635, 8153,   36799,  6, }, // America/Maceio
    {  635, 8168,  197700,  7, }, // America/Managua
    {  635, 1908,    1684,  6, }, // America/Manaus
    {  635, 5897,  197707,  6, }, // America/Marigot
    {  635, 5805,  197713,  9, }, // America/Martinique
    {  635, 8184,    1705,  9, }, // America/Matamoros
    {  635, 2917,    1714,  8, }, // America/Mazatlan
    {  635, 8202,   28128,  8, }, // America/Menominee
    {  635, 8220,  197722,  6, }, // America/Merida
    {  635, 8235,    1736, 10, }, // America/Metlakatla
    {  635, 2949,  197728,  6, }, // America/Mexico_City
    {  635, 4391,   28148,  7, }, // America/Miquelon
    {  635, 8254,    1759,  7, }, // America/Moncton
    {  635, 8270,   36805,  8, }, // America/Monterrey
    {  635, 4098,  197734, 10, }, // America/Montevideo
    {  635, 5824,    1785, 10, }, // America/Montserrat
    {  635, 5481,   34586,  5, }, // America/Nassau
    {  635, 2379,   51812,  8, }, // America/New_York
    {  635, 8288,    1809,  3, }, // America/Nome
    {  635, 1850,   56389,  7, }, // America/Noronha
    {  635, 8301,  197744, 22, }, // America/North_Dakota/Beulah
    {  635, 8329,  197766, 22, }, // America/North_Dakota/Center
    {  635, 8357,  197788, 26, }, // America/North_Dakota/New_Salem
    {  635,  393,    1890,  4, }, // America/Nuuk
    {  635, 8388,  197814,  7, }, // America/Ojinaga
    {  635, 2356,    1901,  6, }, // America/Panama
    {  635, 6173,    1907, 10, }, // America/Paramaribo
    {  635, 2973,   28303,  6, }, // America/Phoenix
    {  635, 3945,    1923, 12, }, // America/Port-au-Prince
    {  635, 5954,   28321, 13, }, // America/Port_of_Spain
    {  635, 8404,   28334, 11, }, // America/Porto_Velho
    {  635, 5843,  197821, 11, }, // America/Puerto_Rico
    {  635, 4030,    1970, 12, }, // America/Punta_Arenas
    {  635, 8424,  197832, 12, }, // America/Rankin_Inlet
    {  635, 8445,  197844,  6, }, // America/Recife
    {  635, 1995,   51899,  8, }, // America/Regina
    {  635, 8460,   56461,  8, }, // America/Resolute
    {  635,  695,  197850, 10, }, // America/Rio_Branco
    {  635, 8477,    2025,  8, }, // America/Santarem
    {  635, 2201,  197860,  8, }, // America/Santiago
    {  635, 6111,  197868, 13, }, // America/Santo_Domingo
    {  635, 1878,    2054,  9, }, // America/Sao_Paulo
    {  635, 6899,  197881, 16, }, // America/Scoresbysund
    {  635, 8494,   28463,  5, }, // America/Sitka
    {  635, 8508,  197897, 13, }, // America/St_Barthelemy
    {  635, 2061,   28481, 10, }, // America/St_Johns
    {  635, 5863,  197910, 10, }, // America/St_Kitts
    {  635, 5880,   28500, 10, }, // America/St_Lucia
    {  635,  855,    2121, 10, }, // America/St_Thomas
    {  635, 5913,   28510, 12, }, // America/St_Vincent
    {  635, 8530,  197920, 13, }, // America/Swift_Current
    {  635, 5461,  197933, 12, }, // America/Tegucigalpa
    {  635, 5756,    2167,  4, }, // America/Thule
    {  635,  313,  197945,  7, }, // America/Tijuana
    {  635,  608,    2178,  7, }, // America/Toronto
    {  635, 5688,    2185,  7, }, // America/Tortola
    {  635, 2093,    2192,  8, }, // America/Vancouver
    {  635, 2144,  197952,  8, }, // America/Whitehorse
    {  635,  734,  197960,  8, }, // America/Winnipeg
    {  635, 8552,   28582,  6, }, // America/Yakutat
    {  635, 6001,  197968,  5, }, // Antarctica/Casey
    {  635, 6094,  197973,  5, }, // Antarctica/Davis
    {  635, 6133,  197978, 14, }, // Antarctica/DumontDUrville
    {  635, 8568,  197992,  8, }, // Antarctica/Macquarie
    {  635, 7066,    2257,  6, }, // Antarctica/Mawson
    {  635, 7113,    2263,  9, }, // Antarctica/McMurdo
    {  635, 8589,    2272,  6, }, // Antarctica/Palmer
    {  635, 7226,    2278,  6, }, // Antarctica/Rothera
    {  635, 7295,  198000,  5, }, // Antarctica/Syowa
    {  635, 8607,    2289,  5, }, // Antarctica/Troll
    {  635, 7374,   34957,  6, }, // Antarctica/Vostok
    {  635, 1429,  198005,  7, }, // Arctic/Longyearbyen
    {  635, 5570,    2310,  4, }, // Asia/Aden
    {  635, 5395,   34974,  6, }, // Asia/Almaty
    {  635, 3968,    2320,  5, }, // Asia/Amman
    {  635, 5511,   34980,  6, }, // Asia/Anadyr
    {  635, 5523,    2331,  5, }, // Asia/Aqtau
    {  635, 5368,    2336,  6, }, // Asia/Aqtobe
    {  635,  964,   28681,  7, }, // Asia/Ashgabat
    {  635, 8624,   52009,  6, }, // Asia/Atyrau
    {  635, 3607,    2355,  6, }, // Asia/Baghdad
    {  635, 5534,   37080,  7, }, // Asia/Bahrain
    {  635, 3653,    2368,  4, }, // Asia/Baku
    {  635, 4466,    2372,  7, }, // Asia/Bangkok
    {  635, 3571,    2379,  7, }, // Asia/Barnaul
    {  635, 4086,   28695,  6, }, // Asia/Beirut
    {  635, 3778,   28701,  6, }, // Asia/Bishkek
    {  635, 5989,   28707,  6, }, // Asia/Brunei
    {  635, 4601,    2404,  4, }, // Asia/Chita
    {  635, 4511,    2408,  7, }, // Asia/Colombo
    {  635, 4540,    2415,  6, }, // Asia/Damascus
    {  635, 1093,    2421,  5, }, // Asia/Dhaka
    {  635, 6192,  198012,  4, }, // Asia/Dili
    {  635, 3596,   28728,  5, }, // Asia/Dubai
    {  635, 6159,    2435,  7, }, // Asia/Dushanbe
    {  635, 8636,    2442,  9, }, // Asia/Famagusta
    {  635, 8651,    2451,  4, }, // Asia/Gaza
    {  635, 4795,    2455,  6, }, // Asia/Hebron
    {  635, 1259,  198016,  7, }, // Asia/Ho_Chi_Minh
    {  635, 2704,   52034,  7, }, // Asia/Hong_Kong
    {  635, 4771,    2475,  4, }, // Asia/Hovd
    {  635, 4184,  198023,  8, }, // Asia/Irkutsk
    {  635, 7006,   28770,  8, }, // Asia/Jakarta
    {  635, 6992,   52041,  7, }, // Asia/Jayapura
    {  635, 1290,  198031,  8, }, // Asia/Jerusalem
    {  635, 3560,    2510,  5, }, // Asia/Kabul
    {  635, 4316,    2529,  8, }, // Asia/Kamchatka
    {  635, 4239,  198039,  6, }, // Asia/Karachi
    {  635, 1185,    2546,  8, }, // Asia/Kathmandu
    {  635, 8661,   35065,  7, }, // Asia/Khandyga
    {  635,  992,   37103,  7, }, // Asia/Kolkata
    {  635, 4197,  198045, 11, }, // Asia/Krasnoyarsk
    {  635, 4859,    2580, 12, }, // Asia/Kuala_Lumpur
    {  635, 5976,    2592,  6, }, // Asia/Kuching
    {  635, 5547,   28825,  6, }, // Asia/Kuwait
    {  635, 1211,    2604,  5, }, // Asia/Macau
    {  635, 4017,    2609,  7, }, // Asia/Magadan
    {  635, 1349,  118684,  8, }, // Asia/Makassar
    {  635, 7182,   28836,  6, }, // Asia/Manila
    {  635, 6920,    2629,  6, }, // Asia/Muscat
    {  635, 2554,  198056,  7, }, // Asia/Nicosia
    {  635, 8675,  198063, 12, }, // Asia/Novokuznetsk
    {  635, 4135,  198075, 12, }, // Asia/Novosibirsk
    {  635, 4229,  198087,  5, }, // Asia/Omsk
    {  635, 7145,   35138,  4, }, // Asia/Oral
    {  635, 6961,   52091,  8, }, // Asia/Phnom_Penh
    {  635, 8693,  198092,  9, }, // Asia/Pontianak
    {  635, 4214,   28895,  7, }, // Asia/Pyongyang
    {  635, 5559,    2697,  5, }, // Asia/Qatar
    {  635, 8708,   37137,  8, }, // Asia/Qostanay
    {  635, 4269,   35167,  9, }, // Asia/Qyzylorda
    {  635, 3584,  198101,  7, }, // Asia/Riyadh
    {  635, 4408,   28926,  7, }, // Asia/Sakhalin
    {  635, 7245,    2739,  9, }, // Asia/Samarkand
    {  635, 3279,    2748,  4, }, // Asia/Seoul
    {  635, 1053,   28933,  6, }, // Asia/Shanghai
    {  635, 3300,   28939,  8, }, // Asia/Singapore
    {  635, 4297,  198108, 16, }, // Asia/Srednekolymsk
    {  635, 3263,  198124,  6, }, // Asia/Taipei
    {  635, 4781,    2785,  7, }, // Asia/Tashkent
    {  635, 3915,   28967,  7, }, // Asia/Tbilisi
    {  635, 2772,    2796,  7, }, // Asia/Tehran
    {  635, 1317,   28981,  7, }, // Asia/Thimphu
    {  635, 2821,  198130,  5, }, // Asia/Tokyo
    {  635, 4572,  198135,  6, }, // Asia/Tomsk
    {  635, 1021,    2820, 10, }, // Asia/Ulaanbaatar
    {  635, 1159,  198141,  6, }, // Asia/Urumqi
    {  635, 8722,   52125,  9, }, // Asia/Ust-Nera
    {  635, 6977,  198147,  8, }, // Asia/Vientiane
    {  635, 4710,    2851, 11, }, // Asia/Vladivostok
    {  635, 4828,  198155,  7, }, // Asia/Yakutsk
    {  635, 1235,   56541,  5, }, // Asia/Yangon
    {  635, 3883,  198162, 12, }, // Asia/Yekaterinburg
    {  635, 3747,  198174,  6, }, // Asia/Yerevan
    {  635, 3663,  198180, 16, }, // Atlantic/Azores
    {  635, 5671,  128710,  7, }, // Atlantic/Bermuda
    {  635, 6628,  198196, 17, }, // Atlantic/Canary
    {  635, 3727,    2944, 10, }, // Atlantic/Cape_Verde
    {  635, 1395,  198213, 17, }, // Atlantic/Faroe
    {  635, 8736,   37264,  7, }, // Atlantic/Madeira
    {  635, 2748,  198230,  9, }, // Atlantic/Reykjavik
    {  635, 7272,  198239, 17, }, // Atlantic/South_Georgia
    {  635, 6821,  198256, 19, }, // Atlantic/St_Helena
    {  635, 6644,  198275,  6, }, // Atlantic/Stanley
    {  635, 1670,  198281,  8, }, // Australia/Adelaide
    {  635, 1635,  198289,  7, }, // Australia/Brisbane
    {  635, 1799,  198296, 11, }, // Australia/Broken_Hill
    {  635, 1583,   29234,  6, }, // Australia/Darwin
    {  635, 3637,  198307,  5, }, // Australia/Eucla
    {  635, 1516,  198312,  6, }, // Australia/Hobart
    {  635, 8753,  198318,  8, }, // Australia/Lindeman
    {  635, 1547,    3086,  8, }, // Australia/Lord_Howe
    {  635, 1727,   29252,  8, }, // Australia/Melbourne
    {  635, 1762,    3101,  4, }, // Australia/Perth
    {  635, 1463,   29260,  6, }, // Australia/Sydney
    {  635, 6441,    3111,  9, }, // Europe/Amsterdam
    {  635, 6216,    3120,  7, }, // Europe/Andorra
    {  635, 3620,   29281,  9, }, // Europe/Astrakhan
    {  635, 2300,  198326,  5, }, // Europe/Athens
    {  635, 6336,    3139,  7, }, // Europe/Belgrade
    {  635, 4757,   29302,  6, }, // Europe/Berlin
    {  635, 6488,    3152, 10, }, // Europe/Bratislava
    {  635, 2167,   52296,  8, }, // Europe/Brussels
    {  635, 3928,    3170,  8, }, // Europe/Bucharest
    {  635, 3806,    3178,  8, }, // Europe/Budapest
    {  635, 8772,   29333,  8, }, // Europe/Busingen
    {  635, 2583,  198331,  7, }, // Europe/Chisinau
    {  635, 6289,   52323, 10, }, // Europe/Copenhagen
    {  635, 2338,   29358,  6, }, // Europe/Dublin
    {  635, 6307,   29364,  9, }, // Europe/Gibraltar
    {  635, 8788,   29373,  6, }, // Europe/Guernsey
    {  635, 6612,  198338,  9, }, // Europe/Helsinki
    {  635, 8804,  198347, 10, }, // Europe/Isle_of_Man
    {  635, 1130,    3264,  7, }, // Europe/Istanbul
    {  635, 8823,  198357,  6, }, // Europe/Jersey
    {  635, 3979,   29405, 11, }, // Europe/Kaliningrad
    {  635, 8929,  198363,  5, }, // Europe/Kirov
    {  635, 2527,  198368,  4, }, // Europe/Kyiv
    {  635, 3213,   29425,  7, }, // Europe/Lisbon
    {  635, 6506,   29432,  7, }, // Europe/Ljubljana
    {  635, 2501,    3313,  6, }, // Europe/London
    {  635, 6365,   29445, 10, }, // Europe/Luxembourg
    {  635, 4892,    3330,  6, }, // Europe/Madrid
    {  635, 6397,   29461,  6, }, // Europe/Malta
    {  635, 6582,  198372,  9, }, // Europe/Mariehamn
    {  635, 3693,  198381,  6, }, // Europe/Minsk
    {  635, 6410,    3355,  6, }, // Europe/Monaco
    {  635, 3537,    3361,  6, }, // Europe/Moscow
    {  635, 6458,    3367,  4, }, // Europe/Oslo
    {  635, 4284,    3371,  5, }, // Europe/Paris
    {  635, 6424,  198387,  9, }, // Europe/Podgorica
    {  635, 6275,    3385,  5, }, // Europe/Prague
    {  635, 8837,    3390,  4, }, // Europe/Riga
    {  635, 6324,    3394,  3, }, // Europe/Rome
    {  635, 4331,    3397,  6, }, // Europe/Samara
    {  635, 6470,    3403, 10, }, // Europe/San_Marino
    {  635, 6245,  198396,  7, }, // Europe/Sarajevo
    {  635, 4451,    3420,  7, }, // Europe/Saratov
    {  635, 8849,  198403, 11, }, // Europe/Simferopol
    {  635, 6383,  198414,  6, }, // Europe/Skopje
    {  635, 6599,  198420,  5, }, // Europe/Sofia
    {  635, 6523,   52374,  9, }, // Europe/Stockholm
    {  635, 8867,  198425,  7, }, // Europe/Tallinn
    {  635, 6202,    3461,  6, }, // Europe/Tirane
    {  635, 8882,  198432, 10, }, // Europe/Ulyanovsk
    {  635, 6352,    3476,  5, }, // Europe/Vaduz
    {  635, 6567,    3481,  7, }, // Europe/Vatican
    {  635, 6231,  198442,  6, }, // Europe/Vienna
    {  635, 8899,   29603,  7, }, // Europe/Vilnius
    {  635, 4727,    3499,  9, }, // Europe/Volgograd
    {  635, 3190,    3508,  7, }, // Europe/Warsaw
    {  635, 6261,    3515,  6, }, // Europe/Zagreb
    {  635, 6540,  198448,  5, }, // Europe/Zurich
    {  635, 5078,  198453, 12, }, // Indian/Antananarivo
    {  635, 6947,  198465,  5, }, // Indian/Chagos
    {  635, 6046,  198470, 13, }, // Indian/Christmas
    {  635, 6063,  198483, 16, }, // Indian/Cocos
    {  635, 5029,   52446,  6, }, // Indian/Comoro
    {  635, 6661,  198499,  8, }, // Indian/Kerguelen
    {  635, 7260,   35648,  4, }, // Indian/Mahe
    {  635, 7050,  198507,  8, }, // Indian/Maldives
    {  635, 4069,  198515,  8, }, // Indian/Mauritius
    {  635, 5098,   56716,  7, }, // Indian/Mayotte
    {  635, 7211,  198523,  8, }, // Indian/Reunion
    {  635, 4422,  198531,  4, }, // Pacific/Apia
    {  635,  932,    3620,  6, }, // Pacific/Auckland
    {  635, 3706,  198535,  9, }, // Pacific/Bougainville
    {  635, 3015,    3634,  5, }, // Pacific/Chatham
    {  635, 3157,  118798,  4, }, // Pacific/Chuuk
    {  635, 2237,  198544, 12, }, // Pacific/Easter
    {  635, 7360,    3654,  5, }, // Pacific/Efate
    {  635, 7327,    3659,  7, }, // Pacific/Fakaofo
    {  635, 3902,  198556,  5, }, // Pacific/Fiji
    {  635, 7343,  198561,  8, }, // Pacific/Funafuti
    {  635, 6678,  198569,  9, }, // Pacific/Galapagos
    {  635, 6696,  198578,  6, }, // Pacific/Gambier
    {  635, 3822,    3713, 11, }, // Pacific/Guadalcanal
    {  635, 6018,    3724,  4, }, // Pacific/Guam
    {  635, 2723,   52582,  8, }, // Pacific/Honolulu
    {  635, 3049,  198584,  9, }, // Pacific/Kanton
    {  635, 3998,  198593, 10, }, // Pacific/Kiritimati
    {  635, 7019,    3744,  6, }, // Pacific/Kosrae
    {  635, 2842,   29841, 10, }, // Pacific/Kwajalein
    {  635, 4877,   37574,  7, }, // Pacific/Majuro
    {  635, 4051,  198603, 17, }, // Pacific/Marquesas
    {  635, 8914,  198620,  6, }, // Pacific/Midway
    {  635, 7084,    3798,  5, }, // Pacific/Nauru
    {  635, 7132,  198626,  4, }, // Pacific/Niue
    {  635, 4168,    3807,  7, }, // Pacific/Norfolk
    {  635, 7098,    3814,  5, }, // Pacific/Noumea
    {  635, 3126,    3819,  9, }, // Pacific/Pago_Pago
    {  635, 7155,    3828,  5, }, // Pacific/Palau
    {  635, 7194,  198630,  7, }, // Pacific/Pitcairn
    {  635, 3096,   35783,  6, }, // Pacific/Pohnpei
    {  635, 4807,   29915, 11, }, // Pacific/Port_Moresby
    {  635, 6076,    3857,  9, }, // Pacific/Rarotonga
    {  635, 6031,   29935,  6, }, // Pacific/Saipan
    {  635, 7312,  198637,  5, }, // Pacific/Tahiti
    {  635, 6712,    3877,  6, }, // Pacific/Tarawa
    {  635, 4583,    3883,  9, }, // Pacific/Tongatapu
    {  635, 7392,  182063,  4, }, // Pacific/Wake
    {  635, 7405,  198642,  6, }, // Pacific/Wallis
    {  636, 6760,   21292,  5, }, // Africa/Accra Upper Sorbian/Latin/Germany
    {  636, 5059,    3902, 11, }, // Africa/Addis_Ababa
    {  636, 7420,   58555,  6, }, // Africa/Algiers
    {  636,   14,       0,  6, }, // Africa/Asmara
    {  636, 2320,    4690,  5, }, // Africa/Cairo
    {  636, 5130,   78080, 10, }, // Africa/Dar_es_Salaam
    {  636, 5043,  198648,  7, }, // Africa/Djibouti
    {  636, 5166,   63491,  8, }, // Africa/El_Aaiun
    {  636, 4524,   26504,  7, }, // Africa/Khartoum
    {  636, 6869,   20418,  4, }, // Africa/Lome
    {  636, 5113,   57721,  8, }, // Africa/Mogadishu
    {  636, 6727,    4758,  8, }, // Africa/Ouagadougou
    {  636, 5225,   26542, 10, }, // Africa/Porto-Novo
    {  636, 4435,       6,  8, }, // Africa/Sao_Tome
    {  636, 2866,   57756,  8, }, // Africa/Tripoli
    {  636, 4252,      14,  8, }, // America/Asuncion
    {  636,  237,      22,  8, }, // America/Atikokan
    {  636, 7623,      30, 17, }, // America/Bahia_Banderas
    {  636, 4361,   20507,  6, }, // America/Bogota
    {  636, 3868,      47,  6, }, // America/Cancun
    {  636, 5496,  198655, 14, }, // America/Cayman
    {  636, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  636, 5723,      66,  7, }, // America/Curacao
    {  636, 2281,   63505,  7, }, // America/Havana
    {  636,  481,      73, 13, }, // America/Indiana/Knox
    {  636, 7961,      86, 16, }, // America/Indiana/Marengo
    {  636, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  636, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  636, 8038,     139, 14, }, // America/Indiana/Vevay
    {  636, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  636, 8086,     171, 16, }, // America/Indiana/Winamac
    {  636, 2799,    3995,  7, }, // America/Jamaica
    {  636, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  636, 5932,     207, 22, }, // America/Lower_Princes
    {  636, 8220,     229,  6, }, // America/Merida
    {  636, 2949,  198669, 12, }, // America/Mexico_City
    {  636, 1850,     235, 19, }, // America/Noronha
    {  636, 8301,  198681, 23, }, // America/North_Dakota/Beulah
    {  636, 8329,  198704, 23, }, // America/North_Dakota/Center
    {  636, 8357,  198727, 26, }, // America/North_Dakota/New_Salem
    {  636,  393,     317,  4, }, // America/Nuuk
    {  636, 5954,   26691, 13, }, // America/Port_of_Spain
    {  636, 1878,   26712,  9, }, // America/Sao_Paulo
    {  636, 6899,     321, 16, }, // America/Scoresbysund
    {  636, 8508,  127879, 13, }, // America/St_Barthelemy
    {  636, 2061,     351, 10, }, // America/St_Johns
    {  636, 5863,     361,  9, }, // America/St_Kitts
    {  636, 5880,     370,  9, }, // America/St_Lucia
    {  636,  855,     379, 10, }, // America/St_Thomas
    {  636, 5913,     389, 11, }, // America/St_Vincent
    {  636, 6133,  127892, 16, }, // Antarctica/DumontDUrville
    {  636, 7374,  198753, 17, }, // Antarctica/Vostok
    {  636, 5368,    7606,  6, }, // Asia/Aqtobe
    {  636, 3607,    4162,  6, }, // Asia/Baghdad
    {  636, 3778,   33455,  6, }, // Asia/Bishkek
    {  636, 4540,    4201,  8, }, // Asia/Damascus
    {  636, 6159,   33468,  7, }, // Asia/Dushanbe
    {  636, 1259,  198770, 20, }, // Asia/Ho_Chi_Minh
    {  636, 2704,    4231,  8, }, // Asia/Hong_Kong
    {  636, 6992,  127925, 11, }, // Asia/Jayapura
    {  636, 4316,   33510,  8, }, // Asia/Kamchatka
    {  636, 4239,   33518,  6, }, // Asia/Karachi
    {  636, 1185,     448,  9, }, // Asia/Kathmandu
    {  636,  992,   64334,  8, }, // Asia/Kolkata
    {  636, 4197,    4288, 11, }, // Asia/Krasnoyarsk
    {  636, 1211,     464,  5, }, // Asia/Macau
    {  636, 6920,    4326,  6, }, // Asia/Muscat
    {  636, 2554,    5738,  7, }, // Asia/Nicosia
    {  636, 8675,  198790, 12, }, // Asia/Novokuznetsk
    {  636, 4135,   78211, 11, }, // Asia/Novosibirsk
    {  636, 4214,   78222,  9, }, // Asia/Pyongyang
    {  636, 5559,    4332,  5, }, // Asia/Qatar
    {  636, 3584,    7787,  4, }, // Asia/Riyadh
    {  636, 4408,   58084,  8, }, // Asia/Sakhalin
    {  636, 3300,   20900,  8, }, // Asia/Singapore
    {  636, 3263,   78231,  6, }, // Asia/Taipei
    {  636, 4781,   33589,  7, }, // Asia/Tashkent
    {  636, 2772,    7842,  7, }, // Asia/Tehran
    {  636, 2821,    4356,  5, }, // Asia/Tokyo
    {  636, 4710,    4361, 11, }, // Asia/Vladivostok
    {  636, 4828,    7873,  7, }, // Asia/Yakutsk
    {  636, 1235,    7880,  6, }, // Asia/Yangon
    {  636, 3883,    4380, 13, }, // Asia/Yekaterinburg
    {  636, 3747,  198802,  8, }, // Asia/Yerevan
    {  636, 3663,  127956,  5, }, // Atlantic/Azores
    {  636, 5671,   58175,  7, }, // Atlantic/Bermuda
    {  636, 6628,  198810, 14, }, // Atlantic/Canary
    {  636, 3727,   57380,  9, }, // Atlantic/Cape_Verde
    {  636, 1395,  198824, 12, }, // Atlantic/Faroe
    {  636, 2748,   36176,  9, }, // Atlantic/Reykjavik
    {  636, 7272,  198836, 15, }, // Atlantic/South_Georgia
    {  636, 6821,     488, 10, }, // Atlantic/St_Helena
    {  636, 2300,    4438,  5, }, // Europe/Athens
    {  636, 6336,  198851,  8, }, // Europe/Belgrade
    {  636, 2167,   22766,  7, }, // Europe/Brussels
    {  636, 3928,   27029,  8, }, // Europe/Bucharest
    {  636, 8772,   27037,  8, }, // Europe/Busingen
    {  636, 2583,  198859,  8, }, // Europe/Chisinau
    {  636, 6289,    4475, 10, }, // Europe/Copenhagen
    {  636, 2527,   78292,  4, }, // Europe/Kyiv
    {  636, 3213,    4499,  8, }, // Europe/Lisbon
    {  636, 6365,    4513,  9, }, // Europe/Luxembourg
    {  636, 3537,   93233,  6, }, // Europe/Moscow
    {  636, 6275,   36285,  5, }, // Europe/Prague
    {  636, 6324,    6064,  3, }, // Europe/Rome
    {  636, 6202,   21213,  6, }, // Europe/Tirane
    {  636, 6567,    6096,  7, }, // Europe/Vatican
    {  636, 6231,   57515,  4, }, // Europe/Vienna
    {  636, 4727,    4561,  9, }, // Europe/Volgograd
    {  636, 3190,  128009,  7, }, // Europe/Warsaw
    {  636, 6540,    4577,  6, }, // Europe/Zurich
    {  636, 6046,  198867, 12, }, // Indian/Christmas
    {  636, 5029,   58408,  6, }, // Indian/Comoro
    {  636, 7050,  128028,  8, }, // Indian/Maldives
    {  636, 7211,     547,  7, }, // Indian/Reunion
    {  636, 3157,     554,  5, }, // Pacific/Chuuk
    {  636, 2237,  198879, 13, }, // Pacific/Easter
    {  636, 3902,  198892,  5, }, // Pacific/Fiji
    {  636, 3049,    4619,  9, }, // Pacific/Kanton
    {  636, 3096,     586,  7, }, // Pacific/Pohnpei
    {  637, 6788,  198897,  7, }, // Africa/Abidjan Urdu/Arabic/Pakistan
    {  637, 6760,  198904,  5, }, // Africa/Accra
    {  637, 5059,  103115, 10, }, // Africa/Addis_Ababa
    {  637, 7420,  198909,  7, }, // Africa/Algiers
    {  637,   14,  106755,  6, }, // Africa/Asmara
    {  637,   44,  103142,  5, }, // Africa/Bamako
    {  637, 5257,  198916,  6, }, // Africa/Bangui
    {  637, 6746,  198922,  5, }, // Africa/Banjul
    {  637, 7435,  103162,  5, }, // Africa/Bissau
    {  637, 4957,  198927,  8, }, // Africa/Blantyre
    {  637, 5287,  198935,  9, }, // Africa/Brazzaville
    {  637, 4922,  198944,  6, }, // Africa/Bujumbura
    {  637, 2320,  198950,  5, }, // Africa/Cairo
    {  637, 4117,  198955, 11, }, // Africa/Casablanca
    {  637, 7449,  198966,  5, }, // Africa/Ceuta
    {  637, 6773,  198971,  6, }, // Africa/Conakry
    {  637, 6840,  198977,  4, }, // Africa/Dakar
    {  637, 5130,   10617, 10, }, // Africa/Dar_es_Salaam
    {  637, 5043,  198981,  5, }, // Africa/Djibouti
    {  637, 5243,  198986,  5, }, // Africa/Douala
    {  637, 5166,  154637,  6, }, // Africa/El_Aaiun
    {  637, 6853,  198991,  8, }, // Africa/Freetown
    {  637, 4906,  198999,  5, }, // Africa/Gaborone
    {  637, 5015,  199004,  5, }, // Africa/Harare
    {  637, 4479,  103278,  9, }, // Africa/Johannesburg
    {  637, 4499,   10673,  4, }, // Africa/Juba
    {  637, 5151,  199009,  7, }, // Africa/Kampala
    {  637, 4524,   10686,  5, }, // Africa/Khartoum
    {  637, 4973,  199016,  5, }, // Africa/Kigali
    {  637, 5306,  199021,  6, }, // Africa/Kinshasa
    {  637, 4744,  106900,  5, }, // Africa/Lagos
    {  637, 5336,  199027,  9, }, // Africa/Libreville
    {  637, 6869,   10716,  3, }, // Africa/Lome
    {  637, 5211,  199036,  6, }, // Africa/Luanda
    {  637, 4939,  167986,  8, }, // Africa/Lubumbashi
    {  637, 5001,  199042,  7, }, // Africa/Lusaka
    {  637, 5322,  168000,  5, }, // Africa/Malabo
    {  637, 4987,  199049,  5, }, // Africa/Maputo
    {  637, 5197,  199054,  5, }, // Africa/Maseru
    {  637, 5182,  199059,  6, }, // Africa/Mbabane
    {  637, 5113,  154736,  8, }, // Africa/Mogadishu
    {  637, 7034,  154744,  8, }, // Africa/Monrovia
    {  637, 3853,  103386,  6, }, // Africa/Nairobi
    {  637, 5271,  199065,  9, }, // Africa/Ndjamena
    {  637, 5354,  154766,  5, }, // Africa/Niamey
    {  637, 6803,  199074,  7, }, // Africa/Nouakchott
    {  637, 6727,  199081,  9, }, // Africa/Ouagadougou
    {  637, 5225,  103422, 10, }, // Africa/Porto-Novo
    {  637, 4435,  199090,  7, }, // Africa/Sao_Tome
    {  637, 2866,  199097,  7, }, // Africa/Tripoli
    {  637, 6554,  199104,  5, }, // Africa/Tunis
    {  637, 4152,  199109,  6, }, // Africa/Windhoek
    {  637,  132,  152176,  4, }, // America/Adak
    {  637, 3346,  152180,  7, }, // America/Anchorage
    {  637, 5607,  199115,  7, }, // America/Anguilla
    {  637, 5624,  199122,  7, }, // America/Antigua
    {  637, 4554,  168097,  9, }, // America/Araguaina
    {  637,  166,  199129, 10, }, // America/Argentina/Buenos_Aires
    {  637,   91,  103509,  9, }, // America/Argentina/Catamarca
    {  637,  270,  199139,  7, }, // America/Argentina/Cordoba
    {  637,  441,  199146,  6, }, // America/Argentina/Jujuy
    {  637, 7462,  168141,  9, }, // America/Argentina/La_Rioja
    {  637,  565,  199152,  7, }, // America/Argentina/Mendoza
    {  637, 7489,  168157, 11, }, // America/Argentina/Rio_Gallegos
    {  637, 7520,  103559,  5, }, // America/Argentina/Salta
    {  637, 7544,  168173,  8, }, // America/Argentina/San_Juan
    {  637, 5580,  199159,  8, }, // America/Argentina/San_Luis
    {  637, 7571,  199167,  7, }, // America/Argentina/Tucuman
    {  637, 7597,  168197,  8, }, // America/Argentina/Ushuaia
    {  637, 5640,  154922,  5, }, // America/Aruba
    {  637, 4252,  199174,  7, }, // America/Asuncion
    {  637,  237,  199181,  7, }, // America/Atikokan
    {  637, 3679,  103633,  5, }, // America/Bahia
    {  637, 7623,  199188, 11, }, // America/Bahia_Banderas
    {  637, 5654,  103645,  8, }, // America/Barbados
    {  637, 7646,  199199,  5, }, // America/Belem
    {  637, 5407,  199204,  6, }, // America/Belize
    {  637, 7660,  199210, 11, }, // America/Blanc-Sablon
    {  637, 7681,  199221,  8, }, // America/Boa_Vista
    {  637, 4361,  199229,  5, }, // America/Bogota
    {  637, 7699,  199234,  5, }, // America/Boise
    {  637, 7713,  199239, 15, }, // America/Cambridge_Bay
    {  637, 7735,  199254, 11, }, // America/Campo_Grande
    {  637, 3868,  199265,  6, }, // America/Cancun
    {  637, 4694,  199271,  6, }, // America/Caracas
    {  637, 4345,  199277,  5, }, // America/Cayenne
    {  637, 5496,  199282,  5, }, // America/Cayman
    {  637, 2260,  199287,  5, }, // America/Chicago
    {  637, 7756,  199292,  8, }, // America/Chihuahua
    {  637, 7774,  152415, 13, }, // America/Ciudad_Juarez
    {  637, 5422,  199300, 10, }, // America/Costa_Rica
    {  637, 7796,  199310,  7, }, // America/Creston
    {  637, 3791,  199317,  7, }, // America/Cuiaba
    {  637, 5723,  199324,  8, }, // America/Curacao
    {  637, 7812,  103794, 11, }, // America/Danmarkshavn
    {  637, 7833,  199332,  5, }, // America/Dawson
    {  637, 7848,  199337, 10, }, // America/Dawson_Creek
    {  637,  805,  199347,  5, }, // America/Denver
    {  637, 3465,  199352,  7, }, // America/Detroit
    {  637, 5739,  199359,  7, }, // America/Dominica
    {  637,  893,  199366,  8, }, // America/Edmonton
    {  637, 7869,  199374,  7, }, // America/Eirunepe
    {  637, 5441,  199381, 10, }, // America/El_Salvador
    {  637, 7886,  103874, 10, }, // America/Fort_Nelson
    {  637, 7906,  199391,  9, }, // America/Fortaleza
    {  637, 7924,  199400, 13, }, // America/Glace_Bay
    {  637, 6881,  199413, 12, }, // America/Goose_Bay
    {  637, 4612,  199425,  8, }, // America/Grand_Turk
    {  637, 5770,  199433,  6, }, // America/Grenada
    {  637, 5786,  199439,  8, }, // America/Guadeloupe
    {  637, 3760,  199447, 10, }, // America/Guatemala
    {  637, 4841,  199457,  9, }, // America/Guayaquil
    {  637, 6932,  199466,  5, }, // America/Guyana
    {  637, 1939,  199471,  8, }, // America/Halifax
    {  637, 2281,  199479,  5, }, // America/Havana
    {  637, 7942,  199484,  8, }, // America/Hermosillo
    {  637,  348,  199492, 11, }, // America/Indiana/Indianapolis
    {  637,  481,  199503, 14, }, // America/Indiana/Knox
    {  637, 7961,  199517, 15, }, // America/Indiana/Marengo
    {  637, 7985,  199532, 17, }, // America/Indiana/Petersburg
    {  637, 8012,  199549, 16, }, // America/Indiana/Tell_City
    {  637, 8038,  199565, 13, }, // America/Indiana/Vevay
    {  637, 8060,  199578, 16, }, // America/Indiana/Vincennes
    {  637, 8086,  199594, 16, }, // America/Indiana/Winamac
    {  637, 8110,  199610,  6, }, // America/Inuvik
    {  637,  660,  199616,  8, }, // America/Iqaluit
    {  637, 2799,  199624,  7, }, // America/Jamaica
    {  637, 5380,  199631,  6, }, // America/Juneau
    {  637,  521,  199637,  9, }, // America/Kentucky/Louisville
    {  637, 8125,  199646, 18, }, // America/Kentucky/Monticello
    {  637, 5704,  199664, 10, }, // America/Kralendijk
    {  637, 4376,  104164,  6, }, // America/La_Paz
    {  637, 7169,  152774,  4, }, // America/Lima
    {  637, 3239,  152778, 10, }, // America/Los_Angeles
    {  637, 5932,  199674, 17, }, // America/Lower_Princes
    {  637, 8153,  168752,  6, }, // America/Maceio
    {  637, 8168,  199691,  6, }, // America/Managua
    {  637, 1908,  199697,  5, }, // America/Manaus
    {  637, 5897,  104220,  7, }, // America/Marigot
    {  637, 5805,  199702,  7, }, // America/Martinique
    {  637, 8184,  104235,  9, }, // America/Matamoros
    {  637, 2917,  199709,  7, }, // America/Mazatlan
    {  637, 8202,  199716,  8, }, // America/Menominee
    {  637, 8220,  199724,  6, }, // America/Merida
    {  637, 8235,  104266, 11, }, // America/Metlakatla
    {  637, 2949,  199730, 11, }, // America/Mexico_City
    {  637, 4391,  199741,  8, }, // America/Miquelon
    {  637, 8254,  199749,  6, }, // America/Moncton
    {  637, 8270,  199755,  7, }, // America/Monterrey
    {  637, 4098,  199762, 11, }, // America/Montevideo
    {  637, 5824,  199773,  9, }, // America/Montserrat
    {  637, 5481,  104328,  4, }, // America/Nassau
    {  637, 2379,  199782,  8, }, // America/New_York
    {  637, 8288,   11540,  3, }, // America/Nome
    {  637, 1850,  104341,  7, }, // America/Noronha
    {  637, 8301,  199790, 19, }, // America/North_Dakota/Beulah
    {  637, 8329,  199809, 16, }, // America/North_Dakota/Center
    {  637, 8357,  199825, 21, }, // America/North_Dakota/New_Salem
    {  637,  393,  103995,  3, }, // America/Nuuk
    {  637, 8388,  155609,  8, }, // America/Ojinaga
    {  637, 2356,  199846,  5, }, // America/Panama
    {  637, 6173,  199851,  9, }, // America/Paramaribo
    {  637, 2973,  153019,  5, }, // America/Phoenix
    {  637, 3945,  199860, 12, }, // America/Port-au-Prince
    {  637, 5954,  199872, 13, }, // America/Port_of_Spain
    {  637, 8404,  199885, 11, }, // America/Porto_Velho
    {  637, 5843,  199896, 11, }, // America/Puerto_Rico
    {  637, 4030,  199907, 11, }, // America/Punta_Arenas
    {  637, 8424,  199918, 11, }, // America/Rankin_Inlet
    {  637, 8445,  199929,  6, }, // America/Recife
    {  637, 1995,  169043,  6, }, // America/Regina
    {  637, 8460,  199935,  8, }, // America/Resolute
    {  637,  695,  199943, 11, }, // America/Rio_Branco
    {  637, 8477,  199954,  7, }, // America/Santarem
    {  637, 2201,  199961,  7, }, // America/Santiago
    {  637, 6111,  199968, 12, }, // America/Santo_Domingo
    {  637, 1878,  104578,  8, }, // America/Sao_Paulo
    {  637, 6899,  199980, 15, }, // America/Scoresbysund
    {  637, 8494,  199995,  5, }, // America/Sitka
    {  637, 8508,  200000, 13, }, // America/St_Barthelemy
    {  637, 2061,  200013,  9, }, // America/St_Johns
    {  637, 5863,  200022,  8, }, // America/St_Kitts
    {  637, 5880,  200030, 10, }, // America/St_Lucia
    {  637,  855,  200040, 10, }, // America/St_Thomas
    {  637, 5913,  200050, 10, }, // America/St_Vincent
    {  637, 8530,  200060, 10, }, // America/Swift_Current
    {  637, 5461,  200070, 11, }, // America/Tegucigalpa
    {  637, 5756,  200081,  5, }, // America/Thule
    {  637,  313,  200086,  7, }, // America/Tijuana
    {  637,  608,  200093,  6, }, // America/Toronto
    {  637, 5688,  200099,  7, }, // America/Tortola
    {  637, 2093,  153272,  7, }, // America/Vancouver
    {  637, 2144,  200106, 10, }, // America/Whitehorse
    {  637,  734,  200116,  7, }, // America/Winnipeg
    {  637, 8552,  200123,  6, }, // America/Yakutat
    {  637, 6001,  104758,  4, }, // Antarctica/Casey
    {  637, 6094,  200129,  4, }, // Antarctica/Davis
    {  637, 6133,  200133, 16, }, // Antarctica/DumontDUrville
    {  637, 8568,  200149,  7, }, // Antarctica/Macquarie
    {  637, 7066,  200156,  5, }, // Antarctica/Mawson
    {  637, 7113,  200161,  8, }, // Antarctica/McMurdo
    {  637, 8589,  200169,  5, }, // Antarctica/Palmer
    {  637, 7226,  104800,  7, }, // Antarctica/Rothera
    {  637, 7295,  104807,  5, }, // Antarctica/Syowa
    {  637, 8607,  200174,  4, }, // Antarctica/Troll
    {  637, 7374,  104812,  6, }, // Antarctica/Vostok
    {  637, 1429,  200178, 13, }, // Arctic/Longyearbyen
    {  637, 5570,   12066,  3, }, // Asia/Aden
    {  637, 5395,  200191,  6, }, // Asia/Almaty
    {  637, 3968,  200197,  5, }, // Asia/Amman
    {  637, 5511,  200202,  5, }, // Asia/Anadyr
    {  637, 5523,  200207,  5, }, // Asia/Aqtau
    {  637, 5368,  200212,  5, }, // Asia/Aqtobe
    {  637,  964,  200217,  5, }, // Asia/Ashgabat
    {  637, 8624,  200222,  6, }, // Asia/Atyrau
    {  637, 3607,   12109,  5, }, // Asia/Baghdad
    {  637, 5534,  153411,  5, }, // Asia/Bahrain
    {  637, 3653,  152447,  4, }, // Asia/Baku
    {  637, 4466,  153416,  5, }, // Asia/Bangkok
    {  637, 3571,  200228,  5, }, // Asia/Barnaul
    {  637, 4086,  153421,  5, }, // Asia/Beirut
    {  637, 3778,  153426,  5, }, // Asia/Bishkek
    {  637, 5989,  155996,  6, }, // Asia/Brunei
    {  637, 4601,  156002,  4, }, // Asia/Chita
    {  637, 4511,  153437,  6, }, // Asia/Colombo
    {  637, 4540,   12162,  4, }, // Asia/Damascus
    {  637, 1093,  200233,  5, }, // Asia/Dhaka
    {  637, 6192,  200238,  3, }, // Asia/Dili
    {  637, 3596,  200241,  4, }, // Asia/Dubai
    {  637, 6159,  200245,  7, }, // Asia/Dushanbe
    {  637, 8636,  200252,  9, }, // Asia/Famagusta
    {  637, 8651,  104972,  3, }, // Asia/Gaza
    {  637, 4795,  104975,  6, }, // Asia/Hebron
    {  637, 1259,  200261, 13, }, // Asia/Ho_Chi_Minh
    {  637, 2704,  104986,  9, }, // Asia/Hong_Kong
    {  637, 4771,  200274,  6, }, // Asia/Hovd
    {  637, 4184,  200280,  6, }, // Asia/Irkutsk
    {  637, 7006,  200286,  6, }, // Asia/Jakarta
    {  637, 6992,  200292,  7, }, // Asia/Jayapura
    {  637, 1290,  105021,  6, }, // Asia/Jerusalem
    {  637, 3560,  153523,  4, }, // Asia/Kabul
    {  637, 4316,  200299,  7, }, // Asia/Kamchatka
    {  637, 4239,  156093,  5, }, // Asia/Karachi
    {  637, 1185,  200306,  8, }, // Asia/Kathmandu
    {  637, 8661,  200314,  6, }, // Asia/Khandyga
    {  637,  992,  200320,  7, }, // Asia/Kolkata
    {  637, 4197,  200327, 11, }, // Asia/Krasnoyarsk
    {  637, 4859,  200338, 11, }, // Asia/Kuala_Lumpur
    {  637, 5976,  200349,  6, }, // Asia/Kuching
    {  637, 5547,  156146,  4, }, // Asia/Kuwait
    {  637, 1211,  200355,  4, }, // Asia/Macau
    {  637, 4017,  200359,  6, }, // Asia/Magadan
    {  637, 1349,  200365,  5, }, // Asia/Makassar
    {  637, 7182,  153602,  5, }, // Asia/Manila
    {  637, 6920,   12368,  4, }, // Asia/Muscat
    {  637, 2554,  200370,  6, }, // Asia/Nicosia
    {  637, 8675,  200376, 12, }, // Asia/Novokuznetsk
    {  637, 4135,  105149, 11, }, // Asia/Novosibirsk
    {  637, 4229,  105160,  5, }, // Asia/Omsk
    {  637, 7145,  156197,  5, }, // Asia/Oral
    {  637, 6961,  153637,  7, }, // Asia/Phnom_Penh
    {  637, 8693,  200388,  8, }, // Asia/Pontianak
    {  637, 4214,  200396, 10, }, // Asia/Pyongyang
    {  637, 5559,   12436,  3, }, // Asia/Qatar
    {  637, 8708,  200406,  7, }, // Asia/Qostanay
    {  637, 4269,  200413,  9, }, // Asia/Qyzylorda
    {  637, 3584,  105212,  4, }, // Asia/Riyadh
    {  637, 4408,  200422,  6, }, // Asia/Sakhalin
    {  637, 7245,   12468,  6, }, // Asia/Samarkand
    {  637, 3279,  200428,  5, }, // Asia/Seoul
    {  637, 1053,  153685,  7, }, // Asia/Shanghai
    {  637, 3300,  156265,  7, }, // Asia/Singapore
    {  637, 4297,  200433, 13, }, // Asia/Srednekolymsk
    {  637, 3263,  200446,  5, }, // Asia/Taipei
    {  637, 4781,  153718,  6, }, // Asia/Tashkent
    {  637, 3915,  200451,  6, }, // Asia/Tbilisi
    {  637, 2772,  200457,  5, }, // Asia/Tehran
    {  637, 1317,  200462,  5, }, // Asia/Thimphu
    {  637, 2821,  105295,  5, }, // Asia/Tokyo
    {  637, 4572,  200467,  5, }, // Asia/Tomsk
    {  637, 1021,  200472, 11, }, // Asia/Ulaanbaatar
    {  637, 1159,  200483,  7, }, // Asia/Urumqi
    {  637, 8722,  200490,  9, }, // Asia/Ust-Nera
    {  637, 6977,  200499,  8, }, // Asia/Vientiane
    {  637, 4710,  200507, 11, }, // Asia/Vladivostok
    {  637, 4828,  200518,  6, }, // Asia/Yakutsk
    {  637, 1235,  169715,  5, }, // Asia/Yangon
    {  637, 3883,  200524, 11, }, // Asia/Yekaterinburg
    {  637, 3747,  200535,  6, }, // Asia/Yerevan
    {  637, 3663,  200541,  6, }, // Atlantic/Azores
    {  637, 5671,   12615,  6, }, // Atlantic/Bermuda
    {  637, 6628,  155685,  5, }, // Atlantic/Canary
    {  637, 3727,  200547,  8, }, // Atlantic/Cape_Verde
    {  637, 1395,   12638,  4, }, // Atlantic/Faroe
    {  637, 8736,  200555,  6, }, // Atlantic/Madeira
    {  637, 2748,  200561,  7, }, // Atlantic/Reykjavik
    {  637, 7272,  105428, 12, }, // Atlantic/South_Georgia
    {  637, 6821,  200568, 11, }, // Atlantic/St_Helena
    {  637, 6644,  200579,  7, }, // Atlantic/Stanley
    {  637, 1670,  200586,  7, }, // Australia/Adelaide
    {  637, 1635,  105464,  6, }, // Australia/Brisbane
    {  637, 1799,  200593,  9, }, // Australia/Broken_Hill
    {  637, 1583,  200602,  5, }, // Australia/Darwin
    {  637, 3637,  200607,  6, }, // Australia/Eucla
    {  637, 1516,  200613,  6, }, // Australia/Hobart
    {  637, 8753,  200619,  7, }, // Australia/Lindeman
    {  637, 1547,  200626,  9, }, // Australia/Lord_Howe
    {  637, 1727,  156490,  6, }, // Australia/Melbourne
    {  637, 1762,  200635,  4, }, // Australia/Perth
    {  637, 1463,  200639,  4, }, // Australia/Sydney
    {  637, 6441,  200643,  8, }, // Europe/Amsterdam
    {  637, 6216,  200651,  6, }, // Europe/Andorra
    {  637, 3620,  200657,  8, }, // Europe/Astrakhan
    {  637, 2300,  200665,  6, }, // Europe/Athens
    {  637, 6336,   12785,  6, }, // Europe/Belgrade
    {  637, 4757,  153976,  4, }, // Europe/Berlin
    {  637, 6488,  200671, 10, }, // Europe/Bratislava
    {  637, 2167,  169873,  5, }, // Europe/Brussels
    {  637, 3928,  200681,  6, }, // Europe/Bucharest
    {  637, 3806,  200687,  6, }, // Europe/Budapest
    {  637, 8772,  200693,  5, }, // Europe/Busingen
    {  637, 2583,  200698,  7, }, // Europe/Chisinau
    {  637, 6289,  105627,  9, }, // Europe/Copenhagen
    {  637, 2338,  200705,  4, }, // Europe/Dublin
    {  637, 6307,  154033, 10, }, // Europe/Gibraltar
    {  637, 8788,  156595,  5, }, // Europe/Guernsey
    {  637, 6612,  200709,  7, }, // Europe/Helsinki
    {  637, 8804,  105666, 10, }, // Europe/Isle_of_Man
    {  637, 1130,  154067,  7, }, // Europe/Istanbul
    {  637, 8823,  105685,  4, }, // Europe/Jersey
    {  637, 3979,  200716, 11, }, // Europe/Kaliningrad
    {  637, 8929,  154084,  5, }, // Europe/Kirov
    {  637, 2527,  103208,  3, }, // Europe/Kyiv
    {  637, 3213,  169972,  4, }, // Europe/Lisbon
    {  637, 6506,  156639,  9, }, // Europe/Ljubljana
    {  637, 2501,   12926,  4, }, // Europe/London
    {  637, 6365,  200727,  7, }, // Europe/Luxembourg
    {  637, 4892,  200734,  5, }, // Europe/Madrid
    {  637, 6397,  105750,  5, }, // Europe/Malta
    {  637, 6582,  200739,  7, }, // Europe/Mariehamn
    {  637, 3693,  105763,  5, }, // Europe/Minsk
    {  637, 6410,  105768,  6, }, // Europe/Monaco
    {  637, 3537,  105774,  5, }, // Europe/Moscow
    {  637, 6458,  105779,  5, }, // Europe/Oslo
    {  637, 4284,  200746,  4, }, // Europe/Paris
    {  637, 6424,  200750,  9, }, // Europe/Podgorica
    {  637, 6275,  105798,  4, }, // Europe/Prague
    {  637, 8837,  155445,  4, }, // Europe/Riga
    {  637, 6324,   10756,  3, }, // Europe/Rome
    {  637, 4331,  106756,  5, }, // Europe/Samara
    {  637, 6470,  200759, 11, }, // Europe/San_Marino
    {  637, 6245,  170041,  7, }, // Europe/Sarajevo
    {  637, 4451,  200770,  6, }, // Europe/Saratov
    {  637, 8849,  170053,  8, }, // Europe/Simferopol
    {  637, 6383,  200776,  7, }, // Europe/Skopje
    {  637, 6599,  200783,  5, }, // Europe/Sofia
    {  637, 6523,  200788,  9, }, // Europe/Stockholm
    {  637, 8867,  200797,  4, }, // Europe/Tallinn
    {  637, 6202,  200801,  6, }, // Europe/Tirane
    {  637, 8882,  200807,  8, }, // Europe/Ulyanovsk
    {  637, 6352,  200815,  5, }, // Europe/Vaduz
    {  637, 6567,  200820,  6, }, // Europe/Vatican
    {  637, 6231,  154224,  5, }, // Europe/Vienna
    {  637, 8899,  200826,  7, }, // Europe/Vilnius
    {  637, 4727,  170109,  9, }, // Europe/Volgograd
    {  637, 3190,  105924,  5, }, // Europe/Warsaw
    {  637, 6261,  200833,  6, }, // Europe/Zagreb
    {  637, 6540,  200839,  5, }, // Europe/Zurich
    {  637, 5078,  200844, 12, }, // Indian/Antananarivo
    {  637, 6947,  105952,  5, }, // Indian/Chagos
    {  637, 6046,  200856,  5, }, // Indian/Christmas
    {  637, 6063,  105964,  5, }, // Indian/Cocos
    {  637, 5029,  105969,  6, }, // Indian/Comoro
    {  637, 6661,  200861,  8, }, // Indian/Kerguelen
    {  637, 7260,  105983,  4, }, // Indian/Mahe
    {  637, 7050,  200869,  6, }, // Indian/Maldives
    {  637, 4069,  200875,  6, }, // Indian/Mauritius
    {  637, 5098,  200881,  5, }, // Indian/Mayotte
    {  637, 7211,  200886,  8, }, // Indian/Reunion
    {  637, 4422,  154313,  4, }, // Pacific/Apia
    {  637,  932,  106021,  6, }, // Pacific/Auckland
    {  637, 3706,  200894,  8, }, // Pacific/Bougainville
    {  637, 3015,  200902,  5, }, // Pacific/Chatham
    {  637, 3157,  200907,  4, }, // Pacific/Chuuk
    {  637, 2237,  105549,  5, }, // Pacific/Easter
    {  637, 7360,  200911,  5, }, // Pacific/Efate
    {  637, 7327,  200916,  6, }, // Pacific/Fakaofo
    {  637, 3902,  200922,  3, }, // Pacific/Fiji
    {  637, 7343,  200925, 10, }, // Pacific/Funafuti
    {  637, 6678,  200935,  9, }, // Pacific/Galapagos
    {  637, 6696,  200944,  7, }, // Pacific/Gambier
    {  637, 3822,  200951, 11, }, // Pacific/Guadalcanal
    {  637, 6018,  199694,  4, }, // Pacific/Guam
    {  637, 2723,  200962,  8, }, // Pacific/Honolulu
    {  637, 3049,  200970,  8, }, // Pacific/Kanton
    {  637, 3998,  200978,  9, }, // Pacific/Kiritimati
    {  637, 7019,  200987,  7, }, // Pacific/Kosrae
    {  637, 2842,  156945,  8, }, // Pacific/Kwajalein
    {  637, 4877,  154416,  5, }, // Pacific/Majuro
    {  637, 4051,  200994,  8, }, // Pacific/Marquesas
    {  637, 8914,  201002,  4, }, // Pacific/Midway
    {  637, 7084,  201006,  5, }, // Pacific/Nauru
    {  637, 7132,  199633,  4, }, // Pacific/Niue
    {  637, 4168,  106166,  6, }, // Pacific/Norfolk
    {  637, 7098,  201011,  5, }, // Pacific/Noumea
    {  637, 3126,  170314,  9, }, // Pacific/Pago_Pago
    {  637, 7155,  201016,  4, }, // Pacific/Palau
    {  637, 7194,  201020,  7, }, // Pacific/Pitcairn
    {  637, 3096,  201027,  7, }, // Pacific/Pohnpei
    {  637, 4807,  201034, 12, }, // Pacific/Port_Moresby
    {  637, 6076,  106215,  9, }, // Pacific/Rarotonga
    {  637, 6031,  201046,  6, }, // Pacific/Saipan
    {  637, 7312,  201052,  6, }, // Pacific/Tahiti
    {  637, 6712,  201058,  5, }, // Pacific/Tarawa
    {  637, 4583,  201063,  9, }, // Pacific/Tongatapu
    {  637, 7392,  106252,  3, }, // Pacific/Wake
    {  637, 7405,  201072,  5, }, // Pacific/Wallis
    {  638, 6760,  151916,  4, }, // Africa/Accra Urdu/Arabic/India
    {  638, 7713,  201077,  9, }, // America/Cambridge_Bay
    {  638, 7924,  201086,  7, }, // America/Glace_Bay
    {  638, 6881,  201093,  6, }, // America/Goose_Bay
    {  638, 4612,  201099,  9, }, // America/Grand_Turk
    {  638, 6899,  201108, 14, }, // America/Scoresbysund
    {  638, 5756,  201122,  5, }, // America/Thule
    {  638, 3806,  201127,  8, }, // Europe/Budapest
    {  639,   14,       0,  6, }, // Africa/Asmara Uyghur/Arabic/China
    {  639, 4435,  201135,  8, }, // Africa/Sao_Tome
    {  639, 4252,  201143, 10, }, // America/Asuncion
    {  639,  237,      22,  8, }, // America/Atikokan
    {  639, 7623,      30, 17, }, // America/Bahia_Banderas
    {  639, 3868,      47,  6, }, // America/Cancun
    {  639, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  639, 5723,  201153,  6, }, // America/Curacao
    {  639,  481,      73, 13, }, // America/Indiana/Knox
    {  639, 7961,      86, 16, }, // America/Indiana/Marengo
    {  639, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  639, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  639, 8038,     139, 14, }, // America/Indiana/Vevay
    {  639, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  639, 8086,     171, 16, }, // America/Indiana/Winamac
    {  639, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  639, 5932,     207, 22, }, // America/Lower_Princes
    {  639, 8220,     229,  6, }, // America/Merida
    {  639, 1850,     235, 19, }, // America/Noronha
    {  639, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  639, 8329,     274, 20, }, // America/North_Dakota/Center
    {  639, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  639,  393,     317,  4, }, // America/Nuuk
    {  639, 6899,     321, 16, }, // America/Scoresbysund
    {  639, 8508,  201159, 18, }, // America/St_Barthelemy
    {  639, 2061,     351, 10, }, // America/St_Johns
    {  639, 5863,     361,  9, }, // America/St_Kitts
    {  639, 5880,     370,  9, }, // America/St_Lucia
    {  639,  855,     379, 10, }, // America/St_Thomas
    {  639, 5913,     389, 11, }, // America/St_Vincent
    {  639, 6133,  201177, 15, }, // Antarctica/DumontDUrville
    {  639, 1259,  201192, 14, }, // Asia/Ho_Chi_Minh
    {  639, 1185,     448,  9, }, // Asia/Kathmandu
    {  639,  992,     457,  7, }, // Asia/Kolkata
    {  639, 1211,     464,  5, }, // Asia/Macau
    {  639, 1235,     477,  6, }, // Asia/Yangon
    {  639, 1395,     483,  5, }, // Atlantic/Faroe
    {  639, 6821,     488, 10, }, // Atlantic/St_Helena
    {  639, 2527,     514,  4, }, // Europe/Kyiv
    {  639, 7211,  201206,  9, }, // Indian/Reunion
    {  639, 3157,     554,  5, }, // Pacific/Chuuk
    {  639, 3096,     586,  7, }, // Pacific/Pohnpei
    {  640, 6760,   21292,  5, }, // Africa/Accra Uzbek/Latin/Uzbekistan
    {  640, 5059,   64622, 11, }, // Africa/Addis_Ababa
    {  640, 7420,  201215,  6, }, // Africa/Algiers
    {  640,   14,   91251,  6, }, // Africa/Asmara
    {  640, 5257,   26480,  5, }, // Africa/Bangui
    {  640, 7435,    6313,  5, }, // Africa/Bissau
    {  640, 4957,  201221,  8, }, // Africa/Blantyre
    {  640, 5287,   21334,  9, }, // Africa/Brazzaville
    {  640, 2320,  201229,  6, }, // Africa/Cairo
    {  640, 4117,    4695, 10, }, // Africa/Casablanca
    {  640, 7449,   21349,  5, }, // Africa/Ceuta
    {  640, 6773,    6367,  7, }, // Africa/Conakry
    {  640, 5130,  201235, 12, }, // Africa/Dar_es_Salaam
    {  640, 5043,  158501,  6, }, // Africa/Djibouti
    {  640, 5243,   21372,  5, }, // Africa/Douala
    {  640, 5166,  201247,  7, }, // Africa/El_Aaiun
    {  640, 6853,    6400,  7, }, // Africa/Freetown
    {  640, 5015,  201254,  6, }, // Africa/Harare
    {  640, 4479,  201260, 12, }, // Africa/Johannesburg
    {  640, 4524,   21402,  6, }, // Africa/Khartoum
    {  640, 5336,    6429,  8, }, // Africa/Libreville
    {  640, 4987,  122189,  6, }, // Africa/Maputo
    {  640, 5113,  201272,  9, }, // Africa/Mogadishu
    {  640, 7034,  201281,  9, }, // Africa/Monrovia
    {  640, 3853,  201290,  7, }, // Africa/Nairobi
    {  640, 6727,   26534,  8, }, // Africa/Ouagadougou
    {  640, 4435,  195604,  8, }, // Africa/Sao_Tome
    {  640, 4152,  201297,  7, }, // Africa/Windhoek
    {  640,  132,  201304, 10, }, // America/Adak
    {  640, 3346,  201314,  7, }, // America/Anchorage
    {  640, 5607,   21486,  7, }, // America/Anguilla
    {  640,  166,  201321, 12, }, // America/Argentina/Buenos_Aires
    {  640,   91,    6518,  9, }, // America/Argentina/Catamarca
    {  640,  270,    6527,  7, }, // America/Argentina/Cordoba
    {  640, 7462,  201333,  8, }, // America/Argentina/La_Rioja
    {  640, 7489,  201341, 12, }, // America/Argentina/Rio_Gallegos
    {  640, 7544,  201353,  8, }, // America/Argentina/San_Juan
    {  640, 5580,  195670,  8, }, // America/Argentina/San_Luis
    {  640, 7571,    4842,  7, }, // America/Argentina/Tucuman
    {  640, 7597,  201361,  7, }, // America/Argentina/Ushuaia
    {  640, 4252,  201368,  7, }, // America/Asuncion
    {  640,  237,  201375, 12, }, // America/Atikokan
    {  640, 3679,  201387,  5, }, // America/Bahia
    {  640, 7623,  201392, 15, }, // America/Bahia_Banderas
    {  640, 5407,    6614,  5, }, // America/Belize
    {  640, 7660,    6620, 12, }, // America/Blanc-Sablon
    {  640, 7681,    6632,  9, }, // America/Boa_Vista
    {  640, 7699,   21564,  5, }, // America/Boise
    {  640, 7713,  201407, 11, }, // America/Cambridge_Bay
    {  640, 7735,  195739, 12, }, // America/Campo_Grande
    {  640, 3868,    6682,  6, }, // America/Cancun
    {  640, 4694,    4910,  7, }, // America/Caracas
    {  640, 4345,  201418,  7, }, // America/Cayenne
    {  640, 5496,  201425, 15, }, // America/Cayman
    {  640, 2260,  201440,  7, }, // America/Chicago
    {  640, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  640, 5422,  195785, 10, }, // America/Costa_Rica
    {  640, 7796,    6736,  7, }, // America/Creston
    {  640, 3791,   21631,  6, }, // America/Cuiaba
    {  640, 5723,  201447,  8, }, // America/Curacao
    {  640, 7812,  201455, 12, }, // America/Danmarkshavn
    {  640, 7833,    6769,  6, }, // America/Dawson
    {  640, 7848,  195821, 11, }, // America/Dawson_Creek
    {  640, 3465,   21659,  7, }, // America/Detroit
    {  640, 5739,   21666,  8, }, // America/Dominica
    {  640, 7869,  201467,  8, }, // America/Eirunepe
    {  640, 5441,    6806,  8, }, // America/El_Salvador
    {  640, 7924,  201475,  9, }, // America/Glace_Bay
    {  640, 6881,  201484,  7, }, // America/Goose_Bay
    {  640, 5786,  122638,  9, }, // America/Guadeloupe
    {  640, 3760,   33338,  9, }, // America/Guatemala
    {  640, 4841,  201491,  8, }, // America/Guayaquil
    {  640, 6932,    5098,  6, }, // America/Guyana
    {  640, 1939,  195892,  8, }, // America/Halifax
    {  640, 2281,  201499,  6, }, // America/Havana
    {  640, 7942,  201505,  9, }, // America/Hermosillo
    {  640,  481,  195914, 13, }, // America/Indiana/Knox
    {  640, 7961,      86, 16, }, // America/Indiana/Marengo
    {  640, 7985,  201514, 19, }, // America/Indiana/Petersburg
    {  640, 8012,    6961, 18, }, // America/Indiana/Tell_City
    {  640, 8038,  201533, 14, }, // America/Indiana/Vevay
    {  640, 8060,  201547, 16, }, // America/Indiana/Vincennes
    {  640, 8086,  201563, 16, }, // America/Indiana/Winamac
    {  640,  660,    5177,  7, }, // America/Iqaluit
    {  640, 2799,   21836,  7, }, // America/Jamaica
    {  640, 5380,  201579,  4, }, // America/Juneau
    {  640,  521,  201583,  8, }, // America/Kentucky/Louisville
    {  640, 8125,  201591, 20, }, // America/Kentucky/Monticello
    {  640, 5704,  201611, 10, }, // America/Kralendijk
    {  640, 4376,  196007,  6, }, // America/La_Paz
    {  640, 3239,  196013, 11, }, // America/Los_Angeles
    {  640, 5932,  201621, 21, }, // America/Lower_Princes
    {  640, 8153,   21910,  6, }, // America/Maceio
    {  640, 5897,   61871,  6, }, // America/Marigot
    {  640, 5805,   26594,  9, }, // America/Martinique
    {  640, 2917,    5279,  8, }, // America/Mazatlan
    {  640, 8202,    7118,  8, }, // America/Menominee
    {  640, 8220,    7126,  6, }, // America/Merida
    {  640, 2949,   21938,  6, }, // America/Mexico_City
    {  640, 4391,    7149,  7, }, // America/Miquelon
    {  640, 8254,    7156,  7, }, // America/Moncton
    {  640, 2379,  201642,  8, }, // America/New_York
    {  640, 8288,   21968,  3, }, // America/Nome
    {  640, 1850,   21971,  7, }, // America/Noronha
    {  640, 8301,  201650, 22, }, // America/North_Dakota/Beulah
    {  640, 8329,  201672, 23, }, // America/North_Dakota/Center
    {  640, 8357,  201695, 26, }, // America/North_Dakota/New_Salem
    {  640,  393,  201721,  6, }, // America/Nuuk
    {  640, 8388,  201727,  7, }, // America/Ojinaga
    {  640, 2973,    7283,  6, }, // America/Phoenix
    {  640, 3945,  166011, 12, }, // America/Port-au-Prince
    {  640, 5954,  201734, 13, }, // America/Port_of_Spain
    {  640, 8404,  201747, 11, }, // America/Porto_Velho
    {  640, 5843,  196178, 11, }, // America/Puerto_Rico
    {  640, 4030,    7322, 12, }, // America/Punta_Arenas
    {  640, 8424,   93788, 12, }, // America/Rankin_Inlet
    {  640, 8445,  123152,  6, }, // America/Recife
    {  640, 1995,  201758,  7, }, // America/Regina
    {  640, 8460,   22093,  8, }, // America/Resolute
    {  640,  695,  196204, 10, }, // America/Rio_Branco
    {  640, 2201,  201765,  8, }, // America/Santiago
    {  640, 6111,    7362, 13, }, // America/Santo_Domingo
    {  640, 1878,  196222,  9, }, // America/Sao_Paulo
    {  640, 6899,  201773, 16, }, // America/Scoresbysund
    {  640, 8508,    7396, 13, }, // America/St_Barthelemy
    {  640, 2061,  196247,  9, }, // America/St_Johns
    {  640, 5863,  196256,  9, }, // America/St_Kitts
    {  640, 5880,  201789, 12, }, // America/St_Lucia
    {  640,  855,  196277, 10, }, // America/St_Thomas
    {  640, 5913,  201801, 12, }, // America/St_Vincent
    {  640, 8530,  201813, 13, }, // America/Swift_Current
    {  640, 5461,    5528, 11, }, // America/Tegucigalpa
    {  640, 5756,  123306,  4, }, // America/Thule
    {  640,  313,   22230,  7, }, // America/Tijuana
    {  640, 2093,    7493,  8, }, // America/Vancouver
    {  640, 2144,  201826,  8, }, // America/Whitehorse
    {  640,  734,  201834,  8, }, // America/Winnipeg
    {  640, 6001,   22253,  5, }, // Antarctica/Casey
    {  640, 6094,   22258,  6, }, // Antarctica/Davis
    {  640, 6133,  201842, 15, }, // Antarctica/DumontDUrville
    {  640, 8568,  196360,  8, }, // Antarctica/Macquarie
    {  640, 7066,   22285,  6, }, // Antarctica/Mawson
    {  640, 7113,  196368,  9, }, // Antarctica/McMurdo
    {  640, 7226,    5583,  6, }, // Antarctica/Rothera
    {  640, 7295,   22300,  5, }, // Antarctica/Syowa
    {  640, 1429,  201857,  7, }, // Arctic/Longyearbyen
    {  640, 5570,  201864,  4, }, // Asia/Aden
    {  640, 5395,    7594,  6, }, // Asia/Almaty
    {  640, 3968,  201868,  5, }, // Asia/Amman
    {  640, 5511,    7600,  6, }, // Asia/Anadyr
    {  640, 5523,  201873,  5, }, // Asia/Aqtau
    {  640, 5368,  201878,  7, }, // Asia/Aqtobe
    {  640,  964,  201885,  8, }, // Asia/Ashgabat
    {  640, 8624,    7612,  6, }, // Asia/Atyrau
    {  640, 3607,  201893,  7, }, // Asia/Baghdad
    {  640, 5534,  201900,  7, }, // Asia/Bahrain
    {  640, 3653,  201907,  4, }, // Asia/Baku
    {  640, 4086,   36003,  6, }, // Asia/Beirut
    {  640, 5989,   22373,  6, }, // Asia/Brunei
    {  640, 4511,    5637,  7, }, // Asia/Colombo
    {  640, 4540,  201911,  7, }, // Asia/Damascus
    {  640, 1093,   91557,  5, }, // Asia/Dhaka
    {  640, 3596,   22390,  5, }, // Asia/Dubai
    {  640, 8651,  201918,  5, }, // Asia/Gaza
    {  640, 4795,  201923,  6, }, // Asia/Hebron
    {  640, 1259,  201929,  8, }, // Asia/Ho_Chi_Minh
    {  640, 2704,  196431,  7, }, // Asia/Hong_Kong
    {  640, 4771,  201937,  4, }, // Asia/Hovd
    {  640, 6992,  201941,  6, }, // Asia/Jayapura
    {  640, 1290,  201947,  6, }, // Asia/Jerusalem
    {  640, 3560,  201953,  5, }, // Asia/Kabul
    {  640, 1185,    5711,  8, }, // Asia/Kathmandu
    {  640, 8661,  201958,  7, }, // Asia/Khandyga
    {  640,  992,   64334,  8, }, // Asia/Kolkata
    {  640, 4859,    7707, 12, }, // Asia/Kuala_Lumpur
    {  640, 5547,  201965,  6, }, // Asia/Kuwait
    {  640, 1211,    7731,  5, }, // Asia/Macau
    {  640, 1349,    7736,  7, }, // Asia/Makassar
    {  640, 6920,   26882,  6, }, // Asia/Muscat
    {  640, 2554,   22510,  8, }, // Asia/Nicosia
    {  640, 7145,   58053,  6, }, // Asia/Oral
    {  640, 6961,   62499,  7, }, // Asia/Phnom_Penh
    {  640, 4214,   22526,  7, }, // Asia/Pyongyang
    {  640, 8708,  201971,  9, }, // Asia/Qostanay
    {  640, 4269,  201980, 10, }, // Asia/Qyzylorda
    {  640, 3584,  201990,  8, }, // Asia/Riyadh
    {  640, 4408,   22542,  7, }, // Asia/Sakhalin
    {  640, 7245,  201998,  9, }, // Asia/Samarkand
    {  640, 3279,    7798,  4, }, // Asia/Seoul
    {  640, 1053,  202007,  7, }, // Asia/Shanghai
    {  640, 3300,   20900,  8, }, // Asia/Singapore
    {  640, 4297,    7817, 13, }, // Asia/Srednekolymsk
    {  640, 3263,  202014,  6, }, // Asia/Taipei
    {  640, 4781,   36110,  8, }, // Asia/Tashkent
    {  640, 2772,  202020,  6, }, // Asia/Tehran
    {  640, 2821,    4356,  5, }, // Asia/Tokyo
    {  640, 1021,  147172, 10, }, // Asia/Ulaanbaatar
    {  640, 1159,  202026,  7, }, // Asia/Urumqi
    {  640, 6977,   22613,  8, }, // Asia/Vientiane
    {  640, 1235,    7880,  6, }, // Asia/Yangon
    {  640, 3663,  202033, 13, }, // Atlantic/Azores
    {  640, 5671,  202046, 16, }, // Atlantic/Bermuda
    {  640, 6628,  202062, 14, }, // Atlantic/Canary
    {  640, 3727,  202076, 10, }, // Atlantic/Cape_Verde
    {  640, 1395,  202086, 14, }, // Atlantic/Faroe
    {  640, 8736,  202100, 13, }, // Atlantic/Madeira
    {  640, 2748,   22676,  9, }, // Atlantic/Reykjavik
    {  640, 7272,  202113, 16, }, // Atlantic/South_Georgia
    {  640, 6821,  202129, 21, }, // Atlantic/St_Helena
    {  640, 6644,    7965,  6, }, // Atlantic/Stanley
    {  640, 1670,   21030,  8, }, // Australia/Adelaide
    {  640, 1635,  123965,  7, }, // Australia/Brisbane
    {  640, 1799,  202150, 11, }, // Australia/Broken_Hill
    {  640, 1583,    7998,  6, }, // Australia/Darwin
    {  640, 3637,  202161,  5, }, // Australia/Eucla
    {  640, 1516,  202166,  6, }, // Australia/Hobart
    {  640, 1547,  202172, 14, }, // Australia/Lord_Howe
    {  640, 1727,    8021,  7, }, // Australia/Melbourne
    {  640, 1762,   22735,  4, }, // Australia/Perth
    {  640, 1463,   22739,  6, }, // Australia/Sydney
    {  640, 3620,  117861,  8, }, // Europe/Astrakhan
    {  640, 2300,   22754,  5, }, // Europe/Athens
    {  640, 6336,   27015,  7, }, // Europe/Belgrade
    {  640, 2167,  202186,  8, }, // Europe/Brussels
    {  640, 3928,   22773,  8, }, // Europe/Bucharest
    {  640, 3806,  202194,  9, }, // Europe/Budapest
    {  640, 8772,  202203,  9, }, // Europe/Busingen
    {  640, 2583,  202212,  9, }, // Europe/Chisinau
    {  640, 6289,  196764, 10, }, // Europe/Copenhagen
    {  640, 8788,  127234,  6, }, // Europe/Guernsey
    {  640, 6612,  202221,  8, }, // Europe/Helsinki
    {  640, 8804,  202229,  9, }, // Europe/Isle_of_Man
    {  640, 8823,  196783,  5, }, // Europe/Jersey
    {  640, 2527,   22849,  5, }, // Europe/Kyiv
    {  640, 3213,    4499,  8, }, // Europe/Lisbon
    {  640, 6506,   22854,  9, }, // Europe/Ljubljana
    {  640, 6365,  202238, 11, }, // Europe/Luxembourg
    {  640, 6582,  202249, 10, }, // Europe/Mariehamn
    {  640, 6410,    6044,  6, }, // Europe/Monaco
    {  640, 3537,   22880,  6, }, // Europe/Moscow
    {  640, 4284,   58930,  5, }, // Europe/Paris
    {  640, 6424,  202259, 10, }, // Europe/Podgorica
    {  640, 6275,   21185,  5, }, // Europe/Prague
    {  640, 6324,   33740,  3, }, // Europe/Rome
    {  640, 6470,    8214, 10, }, // Europe/San_Marino
    {  640, 6245,   22905,  8, }, // Europe/Sarajevo
    {  640, 6383,   22913,  6, }, // Europe/Skopje
    {  640, 6599,   22919,  6, }, // Europe/Sofia
    {  640, 6523,  196857,  8, }, // Europe/Stockholm
    {  640, 8867,   22925,  6, }, // Europe/Tallinn
    {  640, 6202,   21213,  6, }, // Europe/Tirane
    {  640, 6352,   22931,  6, }, // Europe/Vaduz
    {  640, 6567,    6096,  7, }, // Europe/Vatican
    {  640, 6231,  202269,  4, }, // Europe/Vienna
    {  640, 8899,   99349,  7, }, // Europe/Vilnius
    {  640, 3190,  202273,  8, }, // Europe/Warsaw
    {  640, 6540,  202281,  6, }, // Europe/Zurich
    {  640, 5078,   99361, 12, }, // Indian/Antananarivo
    {  640, 6046,  202287, 15, }, // Indian/Christmas
    {  640, 6063,  202302, 14, }, // Indian/Cocos
    {  640, 5029,  202316, 14, }, // Indian/Comoro
    {  640, 6661,   22991,  8, }, // Indian/Kerguelen
    {  640, 7260,  124458,  3, }, // Indian/Mahe
    {  640, 7050,  202330, 15, }, // Indian/Maldives
    {  640, 4069,  202345,  8, }, // Indian/Mauritius
    {  640, 5098,  202353,  7, }, // Indian/Mayotte
    {  640, 7211,  202360,  8, }, // Indian/Reunion
    {  640,  932,  124498,  6, }, // Pacific/Auckland
    {  640, 3706,  124505,  8, }, // Pacific/Bougainville
    {  640, 3015,  202368, 12, }, // Pacific/Chatham
    {  640, 3157,  202380, 13, }, // Pacific/Chuuk
    {  640, 2237,  202393, 11, }, // Pacific/Easter
    {  640, 6696,  202404, 12, }, // Pacific/Gambier
    {  640, 3822,  124569, 11, }, // Pacific/Guadalcanal
    {  640, 2723,  197038,  8, }, // Pacific/Honolulu
    {  640, 3049,  202416, 15, }, // Pacific/Kanton
    {  640, 2842,   23094,  9, }, // Pacific/Kwajalein
    {  640, 4051,  202431, 15, }, // Pacific/Marquesas
    {  640, 8914,  202446, 15, }, // Pacific/Midway
    {  640, 7098,  124660,  5, }, // Pacific/Noumea
    {  640, 3126,    8413,  9, }, // Pacific/Pago_Pago
    {  640, 7194,    8422,  7, }, // Pacific/Pitcairn
    {  640, 3096,  202461, 12, }, // Pacific/Pohnpei
    {  640, 4807,  197082, 11, }, // Pacific/Port_Moresby
    {  640, 6031,   23153,  6, }, // Pacific/Saipan
    {  640, 7312,  202473, 11, }, // Pacific/Tahiti
    {  640, 6712,   23159,  6, }, // Pacific/Tarawa
    {  640, 7392,  202484, 10, }, // Pacific/Wake
    {  640, 7405,    8467,  6, }, // Pacific/Wallis
    {  641,   14,       0,  6, }, // Africa/Asmara Uzbek/Arabic/Afghanistan
    {  641, 4435,       6,  8, }, // Africa/Sao_Tome
    {  641, 4252,      14,  8, }, // America/Asuncion
    {  641,  237,      22,  8, }, // America/Atikokan
    {  641, 7623,      30, 17, }, // America/Bahia_Banderas
    {  641, 3868,      47,  6, }, // America/Cancun
    {  641, 5723,      66,  7, }, // America/Curacao
    {  641,  481,      73, 13, }, // America/Indiana/Knox
    {  641, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  641, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  641, 8038,     139, 14, }, // America/Indiana/Vevay
    {  641, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  641, 8086,     171, 16, }, // America/Indiana/Winamac
    {  641, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  641, 5932,     207, 22, }, // America/Lower_Princes
    {  641, 8220,     229,  6, }, // America/Merida
    {  641, 1850,     235, 19, }, // America/Noronha
    {  641, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  641, 8329,     274, 20, }, // America/North_Dakota/Center
    {  641, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  641,  393,     317,  4, }, // America/Nuuk
    {  641, 6899,     321, 16, }, // America/Scoresbysund
    {  641, 8508,     337, 14, }, // America/St_Barthelemy
    {  641, 2061,     351, 10, }, // America/St_Johns
    {  641, 5863,     361,  9, }, // America/St_Kitts
    {  641, 5880,     370,  9, }, // America/St_Lucia
    {  641,  855,     379, 10, }, // America/St_Thomas
    {  641, 5913,     389, 11, }, // America/St_Vincent
    {  641, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  641, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  641, 3560,  153523,  4, }, // Asia/Kabul
    {  641, 1185,     448,  9, }, // Asia/Kathmandu
    {  641,  992,     457,  7, }, // Asia/Kolkata
    {  641, 1211,     464,  5, }, // Asia/Macau
    {  641, 1235,     477,  6, }, // Asia/Yangon
    {  641, 1395,     483,  5, }, // Atlantic/Faroe
    {  641, 6821,     488, 10, }, // Atlantic/St_Helena
    {  641, 2527,     514,  4, }, // Europe/Kyiv
    {  641, 7211,     547,  7, }, // Indian/Reunion
    {  641, 3157,     554,  5, }, // Pacific/Chuuk
    {  641, 3096,     586,  7, }, // Pacific/Pohnpei
    {  642,   14,       0,  6, }, // Africa/Asmara Uzbek/Cyrillic/Uzbekistan
    {  642, 4435,       6,  8, }, // Africa/Sao_Tome
    {  642, 4252,      14,  8, }, // America/Asuncion
    {  642,  237,      22,  8, }, // America/Atikokan
    {  642, 7623,      30, 17, }, // America/Bahia_Banderas
    {  642, 3868,      47,  6, }, // America/Cancun
    {  642, 5723,      66,  7, }, // America/Curacao
    {  642,  481,      73, 13, }, // America/Indiana/Knox
    {  642, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  642, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  642, 8038,     139, 14, }, // America/Indiana/Vevay
    {  642, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  642, 8086,     171, 16, }, // America/Indiana/Winamac
    {  642, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  642, 5932,  202494, 22, }, // America/Lower_Princes
    {  642, 8220,     229,  6, }, // America/Merida
    {  642, 1850,     235, 19, }, // America/Noronha
    {  642, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  642, 8329,     274, 20, }, // America/North_Dakota/Center
    {  642, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  642,  393,     317,  4, }, // America/Nuuk
    {  642, 6899,     321, 16, }, // America/Scoresbysund
    {  642, 8508,     337, 14, }, // America/St_Barthelemy
    {  642, 2061,     351, 10, }, // America/St_Johns
    {  642, 5863,     361,  9, }, // America/St_Kitts
    {  642, 5880,     370,  9, }, // America/St_Lucia
    {  642,  855,     379, 10, }, // America/St_Thomas
    {  642, 5913,     389, 11, }, // America/St_Vincent
    {  642, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  642, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  642, 1185,     448,  9, }, // Asia/Kathmandu
    {  642,  992,     457,  7, }, // Asia/Kolkata
    {  642, 1211,     464,  5, }, // Asia/Macau
    {  642, 1235,     477,  6, }, // Asia/Yangon
    {  642, 1395,     483,  5, }, // Atlantic/Faroe
    {  642, 6821,     488, 10, }, // Atlantic/St_Helena
    {  642, 2527,     514,  4, }, // Europe/Kyiv
    {  642, 7211,     547,  7, }, // Indian/Reunion
    {  642, 3157,     554,  5, }, // Pacific/Chuuk
    {  642, 3096,     586,  7, }, // Pacific/Pohnpei
    {  643,   14,       0,  6, }, // Africa/Asmara Vai/Vai/Liberia
    {  643, 4435,       6,  8, }, // Africa/Sao_Tome
    {  643, 4252,      14,  8, }, // America/Asuncion
    {  643,  237,      22,  8, }, // America/Atikokan
    {  643, 7623,      30, 17, }, // America/Bahia_Banderas
    {  643, 3868,      47,  6, }, // America/Cancun
    {  643, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  643, 5723,      66,  7, }, // America/Curacao
    {  643,  481,      73, 13, }, // America/Indiana/Knox
    {  643, 7961,      86, 16, }, // America/Indiana/Marengo
    {  643, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  643, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  643, 8038,     139, 14, }, // America/Indiana/Vevay
    {  643, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  643, 8086,     171, 16, }, // America/Indiana/Winamac
    {  643, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  643, 5932,     207, 22, }, // America/Lower_Princes
    {  643, 8220,     229,  6, }, // America/Merida
    {  643, 1850,     235, 19, }, // America/Noronha
    {  643, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  643, 8329,     274, 20, }, // America/North_Dakota/Center
    {  643, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  643,  393,     317,  4, }, // America/Nuuk
    {  643, 6899,     321, 16, }, // America/Scoresbysund
    {  643, 8508,     337, 14, }, // America/St_Barthelemy
    {  643, 2061,     351, 10, }, // America/St_Johns
    {  643, 5863,     361,  9, }, // America/St_Kitts
    {  643, 5880,     370,  9, }, // America/St_Lucia
    {  643,  855,     379, 10, }, // America/St_Thomas
    {  643, 5913,     389, 11, }, // America/St_Vincent
    {  643, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  643, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  643, 1185,     448,  9, }, // Asia/Kathmandu
    {  643,  992,     457,  7, }, // Asia/Kolkata
    {  643, 1211,     464,  5, }, // Asia/Macau
    {  643, 1235,     477,  6, }, // Asia/Yangon
    {  643, 1395,     483,  5, }, // Atlantic/Faroe
    {  643, 6821,     488, 10, }, // Atlantic/St_Helena
    {  643, 2527,     514,  4, }, // Europe/Kyiv
    {  643, 7211,     547,  7, }, // Indian/Reunion
    {  643, 3157,     554,  5, }, // Pacific/Chuuk
    {  643, 3096,     586,  7, }, // Pacific/Pohnpei
    {  645,   14,       0,  6, }, // Africa/Asmara Venda/Latin/South Africa
    {  645, 4435,       6,  8, }, // Africa/Sao_Tome
    {  645, 4252,      14,  8, }, // America/Asuncion
    {  645,  237,      22,  8, }, // America/Atikokan
    {  645, 7623,      30, 17, }, // America/Bahia_Banderas
    {  645, 3868,      47,  6, }, // America/Cancun
    {  645, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  645, 5723,      66,  7, }, // America/Curacao
    {  645,  481,      73, 13, }, // America/Indiana/Knox
    {  645, 7961,      86, 16, }, // America/Indiana/Marengo
    {  645, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  645, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  645, 8038,     139, 14, }, // America/Indiana/Vevay
    {  645, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  645, 8086,     171, 16, }, // America/Indiana/Winamac
    {  645, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  645, 5932,     207, 22, }, // America/Lower_Princes
    {  645, 8220,     229,  6, }, // America/Merida
    {  645, 1850,     235, 19, }, // America/Noronha
    {  645, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  645, 8329,     274, 20, }, // America/North_Dakota/Center
    {  645, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  645,  393,     317,  4, }, // America/Nuuk
    {  645, 6899,     321, 16, }, // America/Scoresbysund
    {  645, 8508,     337, 14, }, // America/St_Barthelemy
    {  645, 2061,     351, 10, }, // America/St_Johns
    {  645, 5863,     361,  9, }, // America/St_Kitts
    {  645, 5880,     370,  9, }, // America/St_Lucia
    {  645,  855,     379, 10, }, // America/St_Thomas
    {  645, 5913,     389, 11, }, // America/St_Vincent
    {  645, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  645, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  645, 1185,     448,  9, }, // Asia/Kathmandu
    {  645,  992,     457,  7, }, // Asia/Kolkata
    {  645, 1211,     464,  5, }, // Asia/Macau
    {  645, 1235,     477,  6, }, // Asia/Yangon
    {  645, 1395,     483,  5, }, // Atlantic/Faroe
    {  645, 6821,     488, 10, }, // Atlantic/St_Helena
    {  645, 2527,     514,  4, }, // Europe/Kyiv
    {  645, 7211,     547,  7, }, // Indian/Reunion
    {  645, 3157,     554,  5, }, // Pacific/Chuuk
    {  645, 3096,     586,  7, }, // Pacific/Pohnpei
    {  646,   14,       0,  6, }, // Africa/Asmara Vietnamese/Latin/Vietnam
    {  646, 4435,       6,  8, }, // Africa/Sao_Tome
    {  646, 4252,      14,  8, }, // America/Asuncion
    {  646,  237,      22,  8, }, // America/Atikokan
    {  646, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  646, 3868,   33314,  6, }, // America/Cancun
    {  646, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  646, 5723,      66,  7, }, // America/Curacao
    {  646,  481,      73, 13, }, // America/Indiana/Knox
    {  646, 7961,      86, 16, }, // America/Indiana/Marengo
    {  646, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  646, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  646, 8038,     139, 14, }, // America/Indiana/Vevay
    {  646, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  646, 8086,     171, 16, }, // America/Indiana/Winamac
    {  646, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  646, 5932,     207, 22, }, // America/Lower_Princes
    {  646, 8220,    7126,  6, }, // America/Merida
    {  646, 1850,     235, 19, }, // America/Noronha
    {  646, 8301,  202516, 18, }, // America/North_Dakota/Beulah
    {  646, 8329,  202534, 18, }, // America/North_Dakota/Center
    {  646, 8357,  202552, 21, }, // America/North_Dakota/New_Salem
    {  646,  393,     317,  4, }, // America/Nuuk
    {  646, 6899,     321, 16, }, // America/Scoresbysund
    {  646, 8508,     337, 14, }, // America/St_Barthelemy
    {  646, 2061,     351, 10, }, // America/St_Johns
    {  646, 5863,     361,  9, }, // America/St_Kitts
    {  646, 5880,     370,  9, }, // America/St_Lucia
    {  646,  855,     379, 10, }, // America/St_Thomas
    {  646, 5913,     389, 11, }, // America/St_Vincent
    {  646, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  646, 1259,  202573, 14, }, // Asia/Ho_Chi_Minh
    {  646, 2704,  202587,  9, }, // Asia/Hong_Kong
    {  646, 1185,     448,  9, }, // Asia/Kathmandu
    {  646,  992,     457,  7, }, // Asia/Kolkata
    {  646, 1211,  202596,  6, }, // Asia/Macau
    {  646, 4214,  202602, 11, }, // Asia/Pyongyang
    {  646, 8708,     469,  8, }, // Asia/Qostanay
    {  646, 1053,  202613, 10, }, // Asia/Shanghai
    {  646, 3263,  202623,  7, }, // Asia/Taipei
    {  646, 1021,   26923, 10, }, // Asia/Ulaanbaatar
    {  646, 6977,  202630, 10, }, // Asia/Vientiane
    {  646, 1235,   58618,  7, }, // Asia/Yangon
    {  646, 1395,     483,  5, }, // Atlantic/Faroe
    {  646, 7272,  202640, 11, }, // Atlantic/South_Georgia
    {  646, 6821,     488, 10, }, // Atlantic/St_Helena
    {  646, 8804,  202651,  7, }, // Europe/Isle_of_Man
    {  646, 2527,    8148,  4, }, // Europe/Kyiv
    {  646, 3537,  202658, 10, }, // Europe/Moscow
    {  646, 6275,   36285,  5, }, // Europe/Prague
    {  646, 7211,     547,  7, }, // Indian/Reunion
    {  646, 3157,     554,  5, }, // Pacific/Chuuk
    {  646, 2723,    4611,  8, }, // Pacific/Honolulu
    {  646, 3049,    4619,  9, }, // Pacific/Kanton
    {  646, 3096,     586,  7, }, // Pacific/Pohnpei
    {  647,   14,       0,  6, }, // Africa/Asmara Volapuk/Latin/world
    {  647, 4435,       6,  8, }, // Africa/Sao_Tome
    {  647, 4252,      14,  8, }, // America/Asuncion
    {  647,  237,      22,  8, }, // America/Atikokan
    {  647, 7623,      30, 17, }, // America/Bahia_Banderas
    {  647, 3868,      47,  6, }, // America/Cancun
    {  647, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  647, 5723,      66,  7, }, // America/Curacao
    {  647,  481,      73, 13, }, // America/Indiana/Knox
    {  647, 7961,      86, 16, }, // America/Indiana/Marengo
    {  647, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  647, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  647, 8038,     139, 14, }, // America/Indiana/Vevay
    {  647, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  647, 8086,     171, 16, }, // America/Indiana/Winamac
    {  647, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  647, 5932,     207, 22, }, // America/Lower_Princes
    {  647, 8220,     229,  6, }, // America/Merida
    {  647, 1850,     235, 19, }, // America/Noronha
    {  647, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  647, 8329,     274, 20, }, // America/North_Dakota/Center
    {  647, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  647,  393,     317,  4, }, // America/Nuuk
    {  647, 6899,     321, 16, }, // America/Scoresbysund
    {  647, 8508,     337, 14, }, // America/St_Barthelemy
    {  647, 2061,     351, 10, }, // America/St_Johns
    {  647, 5863,     361,  9, }, // America/St_Kitts
    {  647, 5880,     370,  9, }, // America/St_Lucia
    {  647,  855,     379, 10, }, // America/St_Thomas
    {  647, 5913,     389, 11, }, // America/St_Vincent
    {  647, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  647, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  647, 1185,     448,  9, }, // Asia/Kathmandu
    {  647,  992,     457,  7, }, // Asia/Kolkata
    {  647, 1211,     464,  5, }, // Asia/Macau
    {  647, 1235,     477,  6, }, // Asia/Yangon
    {  647, 1395,     483,  5, }, // Atlantic/Faroe
    {  647, 6821,     488, 10, }, // Atlantic/St_Helena
    {  647, 2527,     514,  4, }, // Europe/Kyiv
    {  647, 7211,     547,  7, }, // Indian/Reunion
    {  647, 3157,     554,  5, }, // Pacific/Chuuk
    {  647, 3096,     586,  7, }, // Pacific/Pohnpei
    {  648,   14,       0,  6, }, // Africa/Asmara Vunjo/Latin/Tanzania
    {  648, 4435,       6,  8, }, // Africa/Sao_Tome
    {  648, 4252,      14,  8, }, // America/Asuncion
    {  648,  237,      22,  8, }, // America/Atikokan
    {  648, 7623,      30, 17, }, // America/Bahia_Banderas
    {  648, 3868,      47,  6, }, // America/Cancun
    {  648, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  648, 5723,      66,  7, }, // America/Curacao
    {  648,  481,      73, 13, }, // America/Indiana/Knox
    {  648, 7961,      86, 16, }, // America/Indiana/Marengo
    {  648, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  648, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  648, 8038,     139, 14, }, // America/Indiana/Vevay
    {  648, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  648, 8086,     171, 16, }, // America/Indiana/Winamac
    {  648, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  648, 5932,     207, 22, }, // America/Lower_Princes
    {  648, 8220,     229,  6, }, // America/Merida
    {  648, 1850,     235, 19, }, // America/Noronha
    {  648, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  648, 8329,     274, 20, }, // America/North_Dakota/Center
    {  648, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  648,  393,     317,  4, }, // America/Nuuk
    {  648, 6899,     321, 16, }, // America/Scoresbysund
    {  648, 8508,     337, 14, }, // America/St_Barthelemy
    {  648, 2061,     351, 10, }, // America/St_Johns
    {  648, 5863,     361,  9, }, // America/St_Kitts
    {  648, 5880,     370,  9, }, // America/St_Lucia
    {  648,  855,     379, 10, }, // America/St_Thomas
    {  648, 5913,     389, 11, }, // America/St_Vincent
    {  648, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  648, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  648, 1185,     448,  9, }, // Asia/Kathmandu
    {  648,  992,     457,  7, }, // Asia/Kolkata
    {  648, 1211,     464,  5, }, // Asia/Macau
    {  648, 1235,     477,  6, }, // Asia/Yangon
    {  648, 1395,     483,  5, }, // Atlantic/Faroe
    {  648, 6821,     488, 10, }, // Atlantic/St_Helena
    {  648, 2527,     514,  4, }, // Europe/Kyiv
    {  648, 7211,     547,  7, }, // Indian/Reunion
    {  648, 3157,     554,  5, }, // Pacific/Chuuk
    {  648, 3096,     586,  7, }, // Pacific/Pohnpei
    {  649,   14,       0,  6, }, // Africa/Asmara Walloon/Latin/Belgium
    {  649, 4435,       6,  8, }, // Africa/Sao_Tome
    {  649, 4252,      14,  8, }, // America/Asuncion
    {  649,  237,      22,  8, }, // America/Atikokan
    {  649, 7623,      30, 17, }, // America/Bahia_Banderas
    {  649, 3868,      47,  6, }, // America/Cancun
    {  649, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  649, 5723,      66,  7, }, // America/Curacao
    {  649,  481,      73, 13, }, // America/Indiana/Knox
    {  649, 7961,      86, 16, }, // America/Indiana/Marengo
    {  649, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  649, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  649, 8038,     139, 14, }, // America/Indiana/Vevay
    {  649, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  649, 8086,     171, 16, }, // America/Indiana/Winamac
    {  649, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  649, 5932,     207, 22, }, // America/Lower_Princes
    {  649, 8220,     229,  6, }, // America/Merida
    {  649, 1850,     235, 19, }, // America/Noronha
    {  649, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  649, 8329,     274, 20, }, // America/North_Dakota/Center
    {  649, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  649,  393,     317,  4, }, // America/Nuuk
    {  649, 6899,     321, 16, }, // America/Scoresbysund
    {  649, 8508,     337, 14, }, // America/St_Barthelemy
    {  649, 2061,     351, 10, }, // America/St_Johns
    {  649, 5863,     361,  9, }, // America/St_Kitts
    {  649, 5880,     370,  9, }, // America/St_Lucia
    {  649,  855,     379, 10, }, // America/St_Thomas
    {  649, 5913,     389, 11, }, // America/St_Vincent
    {  649, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  649, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  649, 1185,     448,  9, }, // Asia/Kathmandu
    {  649,  992,     457,  7, }, // Asia/Kolkata
    {  649, 1211,     464,  5, }, // Asia/Macau
    {  649, 1235,     477,  6, }, // Asia/Yangon
    {  649, 1395,     483,  5, }, // Atlantic/Faroe
    {  649, 6821,     488, 10, }, // Atlantic/St_Helena
    {  649, 2527,     514,  4, }, // Europe/Kyiv
    {  649, 7211,     547,  7, }, // Indian/Reunion
    {  649, 3157,     554,  5, }, // Pacific/Chuuk
    {  649, 3096,     586,  7, }, // Pacific/Pohnpei
    {  650, 6760,    4649,  4, }, // Africa/Accra Walser/Latin/Switzerland
    {  650, 7420,   58555,  6, }, // Africa/Algiers
    {  650,   14,       0,  6, }, // Africa/Asmara
    {  650, 2320,    4690,  5, }, // Africa/Cairo
    {  650, 5043,  202668,  7, }, // Africa/Djibouti
    {  650, 4524,    6423,  6, }, // Africa/Khartoum
    {  650, 5113,   57721,  8, }, // Africa/Mogadishu
    {  650, 6727,    4758,  8, }, // Africa/Ouagadougou
    {  650, 4435,       6,  8, }, // Africa/Sao_Tome
    {  650, 2866,   57756,  8, }, // Africa/Tripoli
    {  650,  270,    6527,  7, }, // America/Argentina/Cordoba
    {  650, 4252,      14,  8, }, // America/Asuncion
    {  650,  237,      22,  8, }, // America/Atikokan
    {  650, 7623,      30, 17, }, // America/Bahia_Banderas
    {  650, 3868,      47,  6, }, // America/Cancun
    {  650, 5496,  202675, 11, }, // America/Cayman
    {  650, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  650, 5723,      66,  7, }, // America/Curacao
    {  650, 2281,  202686,  7, }, // America/Havana
    {  650,  481,      73, 13, }, // America/Indiana/Knox
    {  650, 7961,      86, 16, }, // America/Indiana/Marengo
    {  650, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  650, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  650, 8038,     139, 14, }, // America/Indiana/Vevay
    {  650, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  650, 8086,     171, 16, }, // America/Indiana/Winamac
    {  650, 2799,    3995,  7, }, // America/Jamaica
    {  650, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  650, 5932,     207, 22, }, // America/Lower_Princes
    {  650, 8220,     229,  6, }, // America/Merida
    {  650, 5824,   21952,  8, }, // America/Montserrat
    {  650, 1850,     235, 19, }, // America/Noronha
    {  650, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  650, 8329,     274, 20, }, // America/North_Dakota/Center
    {  650, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  650,  393,     317,  4, }, // America/Nuuk
    {  650, 6899,     321, 16, }, // America/Scoresbysund
    {  650, 8508,     337, 14, }, // America/St_Barthelemy
    {  650, 2061,     351, 10, }, // America/St_Johns
    {  650, 5863,     361,  9, }, // America/St_Kitts
    {  650, 5880,     370,  9, }, // America/St_Lucia
    {  650,  855,     379, 10, }, // America/St_Thomas
    {  650, 5913,     389, 11, }, // America/St_Vincent
    {  650, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  650, 3607,    4162,  6, }, // Asia/Baghdad
    {  650, 4540,    4201,  8, }, // Asia/Damascus
    {  650, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  650, 2704,    4231,  8, }, // Asia/Hong_Kong
    {  650, 1185,     448,  9, }, // Asia/Kathmandu
    {  650,  992,     457,  7, }, // Asia/Kolkata
    {  650, 1211,     464,  5, }, // Asia/Macau
    {  650, 3300,   20900,  8, }, // Asia/Singapore
    {  650, 2821,    4356,  5, }, // Asia/Tokyo
    {  650, 1235,     477,  6, }, // Asia/Yangon
    {  650, 1395,     483,  5, }, // Atlantic/Faroe
    {  650, 2748,  202693,  8, }, // Atlantic/Reykjavik
    {  650, 6821,     488, 10, }, // Atlantic/St_Helena
    {  650, 1463,  202701,  6, }, // Australia/Sydney
    {  650, 6441,  202707,  9, }, // Europe/Amsterdam
    {  650, 6336,   27015,  7, }, // Europe/Belgrade
    {  650, 2167,   22766,  7, }, // Europe/Brussels
    {  650, 3928,  127165,  8, }, // Europe/Bucharest
    {  650, 3806,  124095,  8, }, // Europe/Budapest
    {  650, 6289,  178380,  8, }, // Europe/Copenhagen
    {  650, 1130,  202716, 14, }, // Europe/Istanbul
    {  650, 3979,  202730, 10, }, // Europe/Kaliningrad
    {  650, 2527,     514,  4, }, // Europe/Kyiv
    {  650, 6506,  202740,  6, }, // Europe/Ljubljana
    {  650, 6275,    8201,  4, }, // Europe/Prague
    {  650, 6324,    6064,  3, }, // Europe/Rome
    {  650, 8867,  202746,  5, }, // Europe/Tallinn
    {  650, 6202,    8255,  5, }, // Europe/Tirane
    {  650, 6567,    6096,  7, }, // Europe/Vatican
    {  650, 6231,   57515,  4, }, // Europe/Vienna
    {  650, 8899,  128004,  5, }, // Europe/Vilnius
    {  650, 4727,    4561,  9, }, // Europe/Volgograd
    {  650, 3190,  202751,  6, }, // Europe/Warsaw
    {  650, 6540,  202757,  5, }, // Europe/Zurich
    {  650, 7211,     547,  7, }, // Indian/Reunion
    {  650, 3157,     554,  5, }, // Pacific/Chuuk
    {  650, 3096,     586,  7, }, // Pacific/Pohnpei
    {  651,   14,       0,  6, }, // Africa/Asmara Warlpiri/Latin/Australia
    {  651, 4435,       6,  8, }, // Africa/Sao_Tome
    {  651, 4252,      14,  8, }, // America/Asuncion
    {  651,  237,      22,  8, }, // America/Atikokan
    {  651, 7623,      30, 17, }, // America/Bahia_Banderas
    {  651, 3868,      47,  6, }, // America/Cancun
    {  651, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  651, 5723,      66,  7, }, // America/Curacao
    {  651,  481,      73, 13, }, // America/Indiana/Knox
    {  651, 7961,      86, 16, }, // America/Indiana/Marengo
    {  651, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  651, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  651, 8038,     139, 14, }, // America/Indiana/Vevay
    {  651, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  651, 8086,     171, 16, }, // America/Indiana/Winamac
    {  651, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  651, 5932,     207, 22, }, // America/Lower_Princes
    {  651, 8220,     229,  6, }, // America/Merida
    {  651, 1850,     235, 19, }, // America/Noronha
    {  651, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  651, 8329,     274, 20, }, // America/North_Dakota/Center
    {  651, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  651,  393,     317,  4, }, // America/Nuuk
    {  651, 6899,     321, 16, }, // America/Scoresbysund
    {  651, 8508,     337, 14, }, // America/St_Barthelemy
    {  651, 2061,     351, 10, }, // America/St_Johns
    {  651, 5863,     361,  9, }, // America/St_Kitts
    {  651, 5880,     370,  9, }, // America/St_Lucia
    {  651,  855,     379, 10, }, // America/St_Thomas
    {  651, 5913,     389, 11, }, // America/St_Vincent
    {  651, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  651, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  651, 1185,     448,  9, }, // Asia/Kathmandu
    {  651,  992,     457,  7, }, // Asia/Kolkata
    {  651, 1211,     464,  5, }, // Asia/Macau
    {  651, 1235,     477,  6, }, // Asia/Yangon
    {  651, 1395,     483,  5, }, // Atlantic/Faroe
    {  651, 6821,     488, 10, }, // Atlantic/St_Helena
    {  651, 2527,     514,  4, }, // Europe/Kyiv
    {  651, 7211,     547,  7, }, // Indian/Reunion
    {  651, 3157,     554,  5, }, // Pacific/Chuuk
    {  651, 3096,     586,  7, }, // Pacific/Pohnpei
    {  652, 7420,   44088,  5, }, // Africa/Algiers Welsh/Latin/United Kingdom
    {  652,   14,       0,  6, }, // Africa/Asmara
    {  652, 4435,       6,  8, }, // Africa/Sao_Tome
    {  652, 7571,   20486,  7, }, // America/Argentina/Tucuman
    {  652, 4252,      14,  8, }, // America/Asuncion
    {  652,  237,      22,  8, }, // America/Atikokan
    {  652, 7623,  202762, 12, }, // America/Bahia_Banderas
    {  652, 7646,   35896,  5, }, // America/Belem
    {  652, 4361,   20507,  6, }, // America/Bogota
    {  652, 7713,  202774, 13, }, // America/Cambridge_Bay
    {  652, 3868,      47,  6, }, // America/Cancun
    {  652, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  652, 3791,   26574,  6, }, // America/Cuiaba
    {  652, 5723,      66,  7, }, // America/Curacao
    {  652, 7869,   26580,  8, }, // America/Eirunepe
    {  652,  481,      73, 13, }, // America/Indiana/Knox
    {  652, 7961,      86, 16, }, // America/Indiana/Marengo
    {  652, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  652, 8012,  202787, 19, }, // America/Indiana/Tell_City
    {  652, 8038,     139, 14, }, // America/Indiana/Vevay
    {  652, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  652, 8086,     171, 16, }, // America/Indiana/Winamac
    {  652, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  652, 5932,     207, 22, }, // America/Lower_Princes
    {  652, 8220,    7126,  6, }, // America/Merida
    {  652, 2949,  202806, 13, }, // America/Mexico_City
    {  652, 2379,  202819, 12, }, // America/New_York
    {  652, 1850,     235, 19, }, // America/Noronha
    {  652, 8301,  202831, 22, }, // America/North_Dakota/Beulah
    {  652, 8329,  202853, 22, }, // America/North_Dakota/Center
    {  652, 8357,  202875, 25, }, // America/North_Dakota/New_Salem
    {  652,  393,     317,  4, }, // America/Nuuk
    {  652, 6899,     321, 16, }, // America/Scoresbysund
    {  652, 8508,     337, 14, }, // America/St_Barthelemy
    {  652, 2061,     351, 10, }, // America/St_Johns
    {  652, 5863,     361,  9, }, // America/St_Kitts
    {  652, 5880,     370,  9, }, // America/St_Lucia
    {  652,  855,     379, 10, }, // America/St_Thomas
    {  652, 5913,     389, 11, }, // America/St_Vincent
    {  652, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  652, 8651,   63961,  4, }, // Asia/Gaza
    {  652, 1259,  202900, 17, }, // Asia/Ho_Chi_Minh
    {  652, 1290,  202917,  9, }, // Asia/Jerusalem
    {  652, 1185,     448,  9, }, // Asia/Kathmandu
    {  652,  992,     457,  7, }, // Asia/Kolkata
    {  652, 1211,    4321,  5, }, // Asia/Macau
    {  652, 8708,     469,  8, }, // Asia/Qostanay
    {  652, 3915,   78237,  6, }, // Asia/Tbilisi
    {  652, 1021,   26923, 10, }, // Asia/Ulaanbaatar
    {  652, 1235,     477,  6, }, // Asia/Yangon
    {  652, 6628,  202926, 19, }, // Atlantic/Canary
    {  652, 1395,  202945,  5, }, // Atlantic/Faroe
    {  652, 2748,   36176,  9, }, // Atlantic/Reykjavik
    {  652, 7272,  202950, 10, }, // Atlantic/South_Georgia
    {  652, 6821,     488, 10, }, // Atlantic/St_Helena
    {  652, 2167,  202960,  6, }, // Europe/Brussels
    {  652, 3928,  202966,  8, }, // Europe/Bucharest
    {  652, 2338,  202974,  5, }, // Europe/Dublin
    {  652, 8788,  202979, 11, }, // Europe/Guernsey
    {  652, 8804,  202990, 10, }, // Europe/Isle_of_Man
    {  652, 1130,  203000, 13, }, // Europe/Istanbul
    {  652, 2527,    8148,  4, }, // Europe/Kyiv
    {  652, 2501,  203013,  8, }, // Europe/London
    {  652, 6365,  203021, 10, }, // Europe/Luxembourg
    {  652, 6275,    8201,  4, }, // Europe/Prague
    {  652, 6324,  203031,  7, }, // Europe/Rome
    {  652, 6567,  203038,  9, }, // Europe/Vatican
    {  652, 6231,  203047,  6, }, // Europe/Vienna
    {  652, 6046,  203053, 14, }, // Indian/Christmas
    {  652, 7211,     547,  7, }, // Indian/Reunion
    {  652, 3157,     554,  5, }, // Pacific/Chuuk
    {  652, 2237,  203067, 11, }, // Pacific/Easter
    {  652, 2723,    4611,  8, }, // Pacific/Honolulu
    {  652, 3049,    4619,  9, }, // Pacific/Kanton
    {  652, 3096,     586,  7, }, // Pacific/Pohnpei
    {  653,   14,       0,  6, }, // Africa/Asmara Western Balochi/Arabic/Pakistan
    {  653, 4435,       6,  8, }, // Africa/Sao_Tome
    {  653, 4252,      14,  8, }, // America/Asuncion
    {  653,  237,      22,  8, }, // America/Atikokan
    {  653, 7623,      30, 17, }, // America/Bahia_Banderas
    {  653, 3868,      47,  6, }, // America/Cancun
    {  653, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  653, 5723,      66,  7, }, // America/Curacao
    {  653,  481,      73, 13, }, // America/Indiana/Knox
    {  653, 7961,      86, 16, }, // America/Indiana/Marengo
    {  653, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  653, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  653, 8038,     139, 14, }, // America/Indiana/Vevay
    {  653, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  653, 8086,     171, 16, }, // America/Indiana/Winamac
    {  653, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  653, 5932,     207, 22, }, // America/Lower_Princes
    {  653, 8220,     229,  6, }, // America/Merida
    {  653, 1850,     235, 19, }, // America/Noronha
    {  653, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  653, 8329,     274, 20, }, // America/North_Dakota/Center
    {  653, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  653,  393,     317,  4, }, // America/Nuuk
    {  653, 6899,     321, 16, }, // America/Scoresbysund
    {  653, 8508,     337, 14, }, // America/St_Barthelemy
    {  653, 2061,     351, 10, }, // America/St_Johns
    {  653, 5863,     361,  9, }, // America/St_Kitts
    {  653, 5880,     370,  9, }, // America/St_Lucia
    {  653,  855,     379, 10, }, // America/St_Thomas
    {  653, 5913,     389, 11, }, // America/St_Vincent
    {  653, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  653, 5570,   12066,  3, }, // Asia/Aden
    {  653, 5395,  203078,  6, }, // Asia/Almaty
    {  653, 3968,   11497,  4, }, // Asia/Amman
    {  653, 5523,  203084,  5, }, // Asia/Aqtau
    {  653, 5368,  203089,  6, }, // Asia/Aqtobe
    {  653,  964,  203095,  8, }, // Asia/Ashgabat
    {  653, 3607,   12109,  5, }, // Asia/Baghdad
    {  653, 5534,  153411,  5, }, // Asia/Bahrain
    {  653, 3653,  152447,  4, }, // Asia/Baku
    {  653, 3778,  203103,  6, }, // Asia/Bishkek
    {  653, 4511,  203109,  7, }, // Asia/Colombo
    {  653, 4540,   12162,  4, }, // Asia/Damascus
    {  653, 1093,  203116,  4, }, // Asia/Dhaka
    {  653, 3596,  153451,  4, }, // Asia/Dubai
    {  653, 6159,  156018,  6, }, // Asia/Dushanbe
    {  653, 8651,  153469,  3, }, // Asia/Gaza
    {  653, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  653, 1290,  203120,  7, }, // Asia/Jerusalem
    {  653, 3560,  153523,  4, }, // Asia/Kabul
    {  653, 4239,  156093,  5, }, // Asia/Karachi
    {  653, 1185,  203127,  6, }, // Asia/Kathmandu
    {  653,  992,  156113,  5, }, // Asia/Kolkata
    {  653, 5547,  203133,  4, }, // Asia/Kuwait
    {  653, 1211,     464,  5, }, // Asia/Macau
    {  653, 6920,   12368,  4, }, // Asia/Muscat
    {  653, 7145,  156197,  5, }, // Asia/Oral
    {  653, 5559,   12436,  3, }, // Asia/Qatar
    {  653, 4269,  203137,  9, }, // Asia/Qyzylorda
    {  653, 3584,  105212,  4, }, // Asia/Riyadh
    {  653, 7245,   12468,  6, }, // Asia/Samarkand
    {  653, 3279,  105233,  4, }, // Asia/Seoul
    {  653, 4781,  203146,  7, }, // Asia/Tashkent
    {  653, 3915,  203153,  7, }, // Asia/Tbilisi
    {  653, 2772,  153730,  5, }, // Asia/Tehran
    {  653, 1317,  203160,  5, }, // Asia/Thimphu
    {  653, 2821,  156305,  5, }, // Asia/Tokyo
    {  653, 4572,  156310,  5, }, // Asia/Tomsk
    {  653, 1021,  156315, 11, }, // Asia/Ulaanbaatar
    {  653, 1159,  203165,  6, }, // Asia/Urumqi
    {  653, 1235,     477,  6, }, // Asia/Yangon
    {  653, 1395,     483,  5, }, // Atlantic/Faroe
    {  653, 6821,     488, 10, }, // Atlantic/St_Helena
    {  653, 1130,  156616,  8, }, // Europe/Istanbul
    {  653, 8929,  154084,  5, }, // Europe/Kirov
    {  653, 2527,     514,  4, }, // Europe/Kyiv
    {  653, 4331,  109308,  6, }, // Europe/Samara
    {  653, 7050,  203171,  6, }, // Indian/Maldives
    {  653, 7211,     547,  7, }, // Indian/Reunion
    {  653, 3157,     554,  5, }, // Pacific/Chuuk
    {  653, 3096,     586,  7, }, // Pacific/Pohnpei
    {  658, 5059,    3902, 11, }, // Africa/Addis_Ababa Western Frisian/Latin/Netherlands
    {  658,   14,       0,  6, }, // Africa/Asmara
    {  658, 2320,   58717,  5, }, // Africa/Cairo
    {  658, 4524,   58722,  8, }, // Africa/Khartoum
    {  658, 6869,   20418,  4, }, // Africa/Lome
    {  658, 4435,   58730,  8, }, // Africa/Sao_Tome
    {  658,  270,   20479,  7, }, // America/Argentina/Cordoba
    {  658, 7489,   44137, 12, }, // America/Argentina/Rio_Gallegos
    {  658, 7571,   20486,  7, }, // America/Argentina/Tucuman
    {  658, 4252,      14,  8, }, // America/Asuncion
    {  658,  237,      22,  8, }, // America/Atikokan
    {  658, 7623,      30, 17, }, // America/Bahia_Banderas
    {  658, 7646,   35896,  5, }, // America/Belem
    {  658, 4361,   20507,  6, }, // America/Bogota
    {  658, 3868,   33314,  6, }, // America/Cancun
    {  658, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  658, 3791,   26574,  6, }, // America/Cuiaba
    {  658, 5723,      66,  7, }, // America/Curacao
    {  658, 5441,    6806,  8, }, // America/El_Salvador
    {  658,  481,      73, 13, }, // America/Indiana/Knox
    {  658, 7961,      86, 16, }, // America/Indiana/Marengo
    {  658, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  658, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  658, 8038,     139, 14, }, // America/Indiana/Vevay
    {  658, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  658, 8086,     171, 16, }, // America/Indiana/Winamac
    {  658, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  658, 5932,   58738, 24, }, // America/Lower_Princes
    {  658, 8153,   26588,  6, }, // America/Maceio
    {  658, 2917,   26603,  8, }, // America/Mazatlan
    {  658, 8220,     229,  6, }, // America/Merida
    {  658, 2949,  203177, 11, }, // America/Mexico_City
    {  658, 1850,     235, 19, }, // America/Noronha
    {  658, 8301,  203188, 20, }, // America/North_Dakota/Beulah
    {  658, 8329,  203208, 20, }, // America/North_Dakota/Center
    {  658, 8357,  203228, 23, }, // America/North_Dakota/New_Salem
    {  658,  393,     317,  4, }, // America/Nuuk
    {  658, 8404,  203251, 11, }, // America/Porto_Velho
    {  658, 1878,   26712,  9, }, // America/Sao_Paulo
    {  658, 6899,     321, 16, }, // America/Scoresbysund
    {  658, 8508,   26721, 16, }, // America/St_Barthelemy
    {  658, 2061,     351, 10, }, // America/St_Johns
    {  658, 5863,     361,  9, }, // America/St_Kitts
    {  658, 5880,     370,  9, }, // America/St_Lucia
    {  658,  855,     379, 10, }, // America/St_Thomas
    {  658, 5913,     389, 11, }, // America/St_Vincent
    {  658, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  658, 5395,   58773,  8, }, // Asia/Almaty
    {  658, 5368,   57205,  6, }, // Asia/Aqtobe
    {  658,  964,    4153,  9, }, // Asia/Ashgabat
    {  658, 3607,    4162,  6, }, // Asia/Baghdad
    {  658, 5534,    4168,  7, }, // Asia/Bahrain
    {  658, 3653,    4175,  5, }, // Asia/Baku
    {  658, 4086,    4180,  7, }, // Asia/Beirut
    {  658, 3778,    4187,  7, }, // Asia/Bishkek
    {  658, 6159,   58581,  8, }, // Asia/Dushanbe
    {  658, 1259,   58796, 15, }, // Asia/Ho_Chi_Minh
    {  658, 2704,    4231,  8, }, // Asia/Hong_Kong
    {  658, 4184,    4239,  8, }, // Asia/Irkutsk
    {  658, 1290,   33501,  9, }, // Asia/Jerusalem
    {  658, 4316,    4261, 10, }, // Asia/Kamchatka
    {  658, 1185,     448,  9, }, // Asia/Kathmandu
    {  658,  992,   36053,  8, }, // Asia/Kolkata
    {  658, 4197,    4288, 11, }, // Asia/Krasnoyarsk
    {  658, 5547,    4314,  7, }, // Asia/Kuwait
    {  658, 1211,     464,  5, }, // Asia/Macau
    {  658, 7182,   58811,  7, }, // Asia/Manila
    {  658, 6961,   93907, 10, }, // Asia/Phnom_Penh
    {  658, 3584,   20874,  5, }, // Asia/Riyadh
    {  658, 4408,   58084,  8, }, // Asia/Sakhalin
    {  658, 1053,   58818,  8, }, // Asia/Shanghai
    {  658, 4781,   58610,  8, }, // Asia/Tashkent
    {  658, 2772,    7842,  7, }, // Asia/Tehran
    {  658, 2821,    4356,  5, }, // Asia/Tokyo
    {  658, 4828,    4372,  8, }, // Asia/Yakutsk
    {  658, 1235,     477,  6, }, // Asia/Yangon
    {  658, 3883,   58826, 15, }, // Asia/Yekaterinburg
    {  658, 3747,    7898,  7, }, // Asia/Yerevan
    {  658, 3663,   58841,  6, }, // Atlantic/Azores
    {  658, 6628,  203262, 18, }, // Atlantic/Canary
    {  658, 3727,   58866, 10, }, // Atlantic/Cape_Verde
    {  658, 1395,   58876,  7, }, // Atlantic/Faroe
    {  658, 7272,  203280, 11, }, // Atlantic/South_Georgia
    {  658, 6821,   58895, 11, }, // Atlantic/St_Helena
    {  658, 2300,    4438,  6, }, // Europe/Athens
    {  658, 6336,   58906,  8, }, // Europe/Belgrade
    {  658, 4757,    4444,  6, }, // Europe/Berlin
    {  658, 2167,    4450,  7, }, // Europe/Brussels
    {  658, 3928,    4457,  9, }, // Europe/Bucharest
    {  658, 3806,    4466,  9, }, // Europe/Budapest
    {  658, 6289,    4475, 10, }, // Europe/Copenhagen
    {  658, 6612,   58297,  8, }, // Europe/Helsinki
    {  658, 1130,   58921,  9, }, // Europe/Istanbul
    {  658, 2527,     514,  4, }, // Europe/Kyiv
    {  658, 3213,    4499,  8, }, // Europe/Lisbon
    {  658, 2501,    4507,  6, }, // Europe/London
    {  658, 6365,    4513,  9, }, // Europe/Luxembourg
    {  658, 3537,    4522,  6, }, // Europe/Moscow
    {  658, 4284,    4528,  5, }, // Europe/Paris
    {  658, 6275,  203291,  6, }, // Europe/Prague
    {  658, 6202,   21213,  6, }, // Europe/Tirane
    {  658, 6567,  203297, 12, }, // Europe/Vatican
    {  658, 6231,   58948,  5, }, // Europe/Vienna
    {  658, 4727,    4561,  9, }, // Europe/Volgograd
    {  658, 3190,   57528,  8, }, // Europe/Warsaw
    {  658, 6540,    4577,  6, }, // Europe/Zurich
    {  658, 6947,   58953, 14, }, // Indian/Chagos
    {  658, 6046,  203309, 10, }, // Indian/Christmas
    {  658, 6063,  203319, 13, }, // Indian/Cocos
    {  658, 7260,   27115,  4, }, // Indian/Mahe
    {  658, 7050,   58995,  8, }, // Indian/Maldives
    {  658, 7211,     547,  7, }, // Indian/Reunion
    {  658, 3157,     554,  5, }, // Pacific/Chuuk
    {  658, 2237,  203332, 11, }, // Pacific/Easter
    {  658, 6678,   21277,  9, }, // Pacific/Galapagos
    {  658, 6696,  203343, 15, }, // Pacific/Gambier
    {  658, 3049,  203358, 15, }, // Pacific/Kanton
    {  658, 4051,  203373, 17, }, // Pacific/Marquesas
    {  658, 7098,    4628,  6, }, // Pacific/Noumea
    {  658, 3096,     586,  7, }, // Pacific/Pohnpei
    {  659,   14,       0,  6, }, // Africa/Asmara Wolaytta/Ethiopic/Ethiopia
    {  659, 4435,       6,  8, }, // Africa/Sao_Tome
    {  659, 4252,      14,  8, }, // America/Asuncion
    {  659,  237,      22,  8, }, // America/Atikokan
    {  659, 7623,      30, 17, }, // America/Bahia_Banderas
    {  659, 3868,      47,  6, }, // America/Cancun
    {  659, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  659, 5723,      66,  7, }, // America/Curacao
    {  659,  481,      73, 13, }, // America/Indiana/Knox
    {  659, 7961,      86, 16, }, // America/Indiana/Marengo
    {  659, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  659, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  659, 8038,     139, 14, }, // America/Indiana/Vevay
    {  659, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  659, 8086,     171, 16, }, // America/Indiana/Winamac
    {  659, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  659, 5932,     207, 22, }, // America/Lower_Princes
    {  659, 8220,     229,  6, }, // America/Merida
    {  659, 1850,     235, 19, }, // America/Noronha
    {  659, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  659, 8329,     274, 20, }, // America/North_Dakota/Center
    {  659, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  659,  393,     317,  4, }, // America/Nuuk
    {  659, 6899,     321, 16, }, // America/Scoresbysund
    {  659, 8508,     337, 14, }, // America/St_Barthelemy
    {  659, 2061,     351, 10, }, // America/St_Johns
    {  659, 5863,     361,  9, }, // America/St_Kitts
    {  659, 5880,     370,  9, }, // America/St_Lucia
    {  659,  855,     379, 10, }, // America/St_Thomas
    {  659, 5913,     389, 11, }, // America/St_Vincent
    {  659, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  659, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  659, 1185,     448,  9, }, // Asia/Kathmandu
    {  659,  992,     457,  7, }, // Asia/Kolkata
    {  659, 1211,     464,  5, }, // Asia/Macau
    {  659, 1235,     477,  6, }, // Asia/Yangon
    {  659, 1395,     483,  5, }, // Atlantic/Faroe
    {  659, 6821,     488, 10, }, // Atlantic/St_Helena
    {  659, 2527,     514,  4, }, // Europe/Kyiv
    {  659, 7211,     547,  7, }, // Indian/Reunion
    {  659, 3157,     554,  5, }, // Pacific/Chuuk
    {  659, 3096,     586,  7, }, // Pacific/Pohnpei
    {  660,   14,       0,  6, }, // Africa/Asmara Wolof/Latin/Senegal
    {  660, 4435,       6,  8, }, // Africa/Sao_Tome
    {  660, 4252,      14,  8, }, // America/Asuncion
    {  660,  237,      22,  8, }, // America/Atikokan
    {  660, 7623,      30, 17, }, // America/Bahia_Banderas
    {  660, 3868,      47,  6, }, // America/Cancun
    {  660, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  660, 5723,      66,  7, }, // America/Curacao
    {  660,  481,      73, 13, }, // America/Indiana/Knox
    {  660, 7961,      86, 16, }, // America/Indiana/Marengo
    {  660, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  660, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  660, 8038,     139, 14, }, // America/Indiana/Vevay
    {  660, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  660, 8086,     171, 16, }, // America/Indiana/Winamac
    {  660, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  660, 5932,     207, 22, }, // America/Lower_Princes
    {  660, 8220,     229,  6, }, // America/Merida
    {  660, 1850,     235, 19, }, // America/Noronha
    {  660, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  660, 8329,     274, 20, }, // America/North_Dakota/Center
    {  660, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  660,  393,     317,  4, }, // America/Nuuk
    {  660, 6899,     321, 16, }, // America/Scoresbysund
    {  660, 8508,     337, 14, }, // America/St_Barthelemy
    {  660, 2061,     351, 10, }, // America/St_Johns
    {  660, 5863,     361,  9, }, // America/St_Kitts
    {  660, 5880,     370,  9, }, // America/St_Lucia
    {  660,  855,     379, 10, }, // America/St_Thomas
    {  660, 5913,     389, 11, }, // America/St_Vincent
    {  660, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  660, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  660, 1185,     448,  9, }, // Asia/Kathmandu
    {  660,  992,     457,  7, }, // Asia/Kolkata
    {  660, 1211,     464,  5, }, // Asia/Macau
    {  660, 1235,     477,  6, }, // Asia/Yangon
    {  660, 1395,     483,  5, }, // Atlantic/Faroe
    {  660, 6821,     488, 10, }, // Atlantic/St_Helena
    {  660, 2527,     514,  4, }, // Europe/Kyiv
    {  660, 7211,     547,  7, }, // Indian/Reunion
    {  660, 3157,     554,  5, }, // Pacific/Chuuk
    {  660, 3096,     586,  7, }, // Pacific/Pohnpei
    {  661,   14,       0,  6, }, // Africa/Asmara Xhosa/Latin/South Africa
    {  661, 4435,       6,  8, }, // Africa/Sao_Tome
    {  661, 4252,      14,  8, }, // America/Asuncion
    {  661,  237,      22,  8, }, // America/Atikokan
    {  661, 7623,    3943, 14, }, // America/Bahia_Banderas
    {  661, 3868,   33314,  6, }, // America/Cancun
    {  661, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  661, 5723,      66,  7, }, // America/Curacao
    {  661,  481,      73, 13, }, // America/Indiana/Knox
    {  661, 7961,      86, 16, }, // America/Indiana/Marengo
    {  661, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  661, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  661, 8038,     139, 14, }, // America/Indiana/Vevay
    {  661, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  661, 8086,     171, 16, }, // America/Indiana/Winamac
    {  661, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  661, 5932,     207, 22, }, // America/Lower_Princes
    {  661, 8220,    7126,  6, }, // America/Merida
    {  661, 1850,     235, 19, }, // America/Noronha
    {  661, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  661, 8329,     274, 20, }, // America/North_Dakota/Center
    {  661, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  661,  393,     317,  4, }, // America/Nuuk
    {  661, 6899,     321, 16, }, // America/Scoresbysund
    {  661, 8508,     337, 14, }, // America/St_Barthelemy
    {  661, 2061,     351, 10, }, // America/St_Johns
    {  661, 5863,     361,  9, }, // America/St_Kitts
    {  661, 5880,     370,  9, }, // America/St_Lucia
    {  661,  855,     379, 10, }, // America/St_Thomas
    {  661, 5913,     389, 11, }, // America/St_Vincent
    {  661, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  661, 1259,     432, 16, }, // Asia/Ho_Chi_Minh
    {  661, 1185,     448,  9, }, // Asia/Kathmandu
    {  661,  992,     457,  7, }, // Asia/Kolkata
    {  661, 1211,     464,  5, }, // Asia/Macau
    {  661, 8708,     469,  8, }, // Asia/Qostanay
    {  661, 1235,     477,  6, }, // Asia/Yangon
    {  661, 1395,     483,  5, }, // Atlantic/Faroe
    {  661, 6821,     488, 10, }, // Atlantic/St_Helena
    {  661, 2527,     514,  4, }, // Europe/Kyiv
    {  661, 7211,     547,  7, }, // Indian/Reunion
    {  661, 3157,     554,  5, }, // Pacific/Chuuk
    {  661, 3096,     586,  7, }, // Pacific/Pohnpei
    {  662,   14,       0,  6, }, // Africa/Asmara Yangben/Latin/Cameroon
    {  662, 4435,       6,  8, }, // Africa/Sao_Tome
    {  662, 4252,      14,  8, }, // America/Asuncion
    {  662,  237,      22,  8, }, // America/Atikokan
    {  662, 7623,      30, 17, }, // America/Bahia_Banderas
    {  662, 3868,      47,  6, }, // America/Cancun
    {  662, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  662, 5723,      66,  7, }, // America/Curacao
    {  662,  481,      73, 13, }, // America/Indiana/Knox
    {  662, 7961,      86, 16, }, // America/Indiana/Marengo
    {  662, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  662, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  662, 8038,     139, 14, }, // America/Indiana/Vevay
    {  662, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  662, 8086,     171, 16, }, // America/Indiana/Winamac
    {  662, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  662, 5932,     207, 22, }, // America/Lower_Princes
    {  662, 8220,     229,  6, }, // America/Merida
    {  662, 1850,     235, 19, }, // America/Noronha
    {  662, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  662, 8329,     274, 20, }, // America/North_Dakota/Center
    {  662, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  662,  393,     317,  4, }, // America/Nuuk
    {  662, 6899,     321, 16, }, // America/Scoresbysund
    {  662, 8508,     337, 14, }, // America/St_Barthelemy
    {  662, 2061,     351, 10, }, // America/St_Johns
    {  662, 5863,     361,  9, }, // America/St_Kitts
    {  662, 5880,     370,  9, }, // America/St_Lucia
    {  662,  855,     379, 10, }, // America/St_Thomas
    {  662, 5913,     389, 11, }, // America/St_Vincent
    {  662, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  662, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  662, 1185,     448,  9, }, // Asia/Kathmandu
    {  662,  992,     457,  7, }, // Asia/Kolkata
    {  662, 1211,     464,  5, }, // Asia/Macau
    {  662, 1235,     477,  6, }, // Asia/Yangon
    {  662, 1395,     483,  5, }, // Atlantic/Faroe
    {  662, 6821,     488, 10, }, // Atlantic/St_Helena
    {  662, 2527,     514,  4, }, // Europe/Kyiv
    {  662, 7211,     547,  7, }, // Indian/Reunion
    {  662, 3157,     554,  5, }, // Pacific/Chuuk
    {  662, 3096,     586,  7, }, // Pacific/Pohnpei
    {  663, 6788,  203390, 10, }, // Africa/Abidjan Yiddish/Hebrew/Ukraine
    {  663,   14,  203400,  9, }, // Africa/Asmara
    {  663, 4435,       6,  8, }, // Africa/Sao_Tome
    {  663, 6554,  203409,  5, }, // Africa/Tunis
    {  663, 4152,  203414,  8, }, // Africa/Windhoek
    {  663, 4252,      14,  8, }, // America/Asuncion
    {  663,  237,      22,  8, }, // America/Atikokan
    {  663, 7623,      30, 17, }, // America/Bahia_Banderas
    {  663, 3868,      47,  6, }, // America/Cancun
    {  663, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  663, 5723,      66,  7, }, // America/Curacao
    {  663,  481,      73, 13, }, // America/Indiana/Knox
    {  663, 7961,      86, 16, }, // America/Indiana/Marengo
    {  663, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  663, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  663, 8038,     139, 14, }, // America/Indiana/Vevay
    {  663, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  663, 8086,     171, 16, }, // America/Indiana/Winamac
    {  663, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  663, 5932,     207, 22, }, // America/Lower_Princes
    {  663, 8220,     229,  6, }, // America/Merida
    {  663, 1850,     235, 19, }, // America/Noronha
    {  663, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  663, 8329,     274, 20, }, // America/North_Dakota/Center
    {  663, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  663,  393,     317,  4, }, // America/Nuuk
    {  663, 6899,     321, 16, }, // America/Scoresbysund
    {  663, 8508,     337, 14, }, // America/St_Barthelemy
    {  663, 2061,     351, 10, }, // America/St_Johns
    {  663, 5863,     361,  9, }, // America/St_Kitts
    {  663, 5880,     370,  9, }, // America/St_Lucia
    {  663,  855,     379, 10, }, // America/St_Thomas
    {  663, 5913,     389, 11, }, // America/St_Vincent
    {  663, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  663, 1259,  203422, 16, }, // Asia/Ho_Chi_Minh
    {  663, 1185,     448,  9, }, // Asia/Kathmandu
    {  663,  992,     457,  7, }, // Asia/Kolkata
    {  663, 1211,     464,  5, }, // Asia/Macau
    {  663, 1235,     477,  6, }, // Asia/Yangon
    {  663, 1395,     483,  5, }, // Atlantic/Faroe
    {  663, 6821,     488, 10, }, // Atlantic/St_Helena
    {  663, 2527,     514,  4, }, // Europe/Kyiv
    {  663, 7211,     547,  7, }, // Indian/Reunion
    {  663, 3157,     554,  5, }, // Pacific/Chuuk
    {  663, 3096,     586,  7, }, // Pacific/Pohnpei
    {  664,   14,       0,  6, }, // Africa/Asmara Yoruba/Latin/Nigeria
    {  664, 4435,       6,  8, }, // Africa/Sao_Tome
    {  664,  132,  203438,  9, }, // America/Adak
    {  664, 3346,  203447, 14, }, // America/Anchorage
    {  664, 5607,  203461, 11, }, // America/Anguilla
    {  664, 5624,  203472, 11, }, // America/Antigua
    {  664, 5640,  203483,  9, }, // America/Aruba
    {  664, 4252,      14,  8, }, // America/Asuncion
    {  664,  237,  203492, 12, }, // America/Atikokan
    {  664, 7623,  203504, 19, }, // America/Bahia_Banderas
    {  664, 5654,  203523, 15, }, // America/Barbados
    {  664, 5407,  203538, 12, }, // America/Belize
    {  664, 7660,  203550, 21, }, // America/Blanc-Sablon
    {  664, 7699,  203571, 10, }, // America/Boise
    {  664, 7713,  203581, 19, }, // America/Cambridge_Bay
    {  664, 3868,  203600, 10, }, // America/Cancun
    {  664, 5496,  203610, 10, }, // America/Cayman
    {  664, 2260,  203620, 11, }, // America/Chicago
    {  664, 7756,  203631, 12, }, // America/Chihuahua
    {  664, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  664, 5422,  203643, 14, }, // America/Costa_Rica
    {  664, 7796,  203657, 16, }, // America/Creston
    {  664, 5723,  203673, 11, }, // America/Curacao
    {  664, 7812,  203684, 15, }, // America/Danmarkshavn
    {  664, 7833,  203699, 10, }, // America/Dawson
    {  664, 7848,  203709, 16, }, // America/Dawson_Creek
    {  664,  805,  203725, 10, }, // America/Denver
    {  664, 3465,  203735, 11, }, // America/Detroit
    {  664, 5739,  203746, 12, }, // America/Dominica
    {  664,  893,  203758, 12, }, // America/Edmonton
    {  664, 5441,  203770, 14, }, // America/El_Salvador
    {  664, 7886,  203784, 15, }, // America/Fort_Nelson
    {  664, 7924,  203799, 13, }, // America/Glace_Bay
    {  664, 6881,  203812, 13, }, // America/Goose_Bay
    {  664, 4612,  203825, 14, }, // America/Grand_Turk
    {  664, 5770,  203839, 11, }, // America/Grenada
    {  664, 5786,  203850, 14, }, // America/Guadeloupe
    {  664, 3760,  203864, 13, }, // America/Guatemala
    {  664, 1939,  203877, 12, }, // America/Halifax
    {  664, 2281,  203889, 10, }, // America/Havana
    {  664, 7942,  203899, 13, }, // America/Hermosillo
    {  664,  348,  203912, 16, }, // America/Indiana/Indianapolis
    {  664,  481,  203928,  9, }, // America/Indiana/Knox
    {  664, 7961,  203937, 12, }, // America/Indiana/Marengo
    {  664, 7985,  203949, 14, }, // America/Indiana/Petersburg
    {  664, 8012,  203963, 13, }, // America/Indiana/Tell_City
    {  664, 8038,  203976,  9, }, // America/Indiana/Vevay
    {  664, 8060,  203985, 24, }, // America/Indiana/Vincennes
    {  664, 8086,  204009, 22, }, // America/Indiana/Winamac
    {  664, 8110,  204031, 10, }, // America/Inuvik
    {  664,  660,  204041, 11, }, // America/Iqaluit
    {  664, 2799,  204052, 11, }, // America/Jamaica
    {  664, 5380,  204063,  8, }, // America/Juneau
    {  664,  521,  204071, 12, }, // America/Kentucky/Louisville
    {  664, 8125,  204083, 14, }, // America/Kentucky/Monticello
    {  664, 5704,  204097, 17, }, // America/Kralendijk
    {  664, 3239,  204114, 15, }, // America/Los_Angeles
    {  664, 5932,  204129, 26, }, // America/Lower_Princes
    {  664, 8168,  204155, 11, }, // America/Managua
    {  664, 5897,  204166, 15, }, // America/Marigot
    {  664, 5805,  204181, 12, }, // America/Martinique
    {  664, 8184,  204193, 14, }, // America/Matamoros
    {  664, 2917,  204207, 13, }, // America/Mazatlan
    {  664, 8202,  204220, 12, }, // America/Menominee
    {  664, 8220,  204232, 10, }, // America/Merida
    {  664, 8235,  204242, 16, }, // America/Metlakatla
    {  664, 2949,  204258, 11, }, // America/Mexico_City
    {  664, 4391,  204269, 13, }, // America/Miquelon
    {  664, 8254,  204282, 11, }, // America/Moncton
    {  664, 8270,  204293, 11, }, // America/Monterrey
    {  664, 5824,  204304, 13, }, // America/Montserrat
    {  664, 5481,  204317,  9, }, // America/Nassau
    {  664, 2379,  204326, 12, }, // America/New_York
    {  664, 8288,  204338,  8, }, // America/Nome
    {  664, 1850,     235, 19, }, // America/Noronha
    {  664, 8301,  204346, 25, }, // America/North_Dakota/Beulah
    {  664, 8329,  204371, 22, }, // America/North_Dakota/Center
    {  664, 8357,  204393, 29, }, // America/North_Dakota/New_Salem
    {  664,  393,  204422,  8, }, // America/Nuuk
    {  664, 8388,  204430, 11, }, // America/Ojinaga
    {  664, 2356,  204441, 10, }, // America/Panama
    {  664, 2973,  204451, 10, }, // America/Phoenix
    {  664, 3945,  204461, 18, }, // America/Port-au-Prince
    {  664, 5954,  204479, 19, }, // America/Port_of_Spain
    {  664, 5843,  204498, 15, }, // America/Puerto_Rico
    {  664, 8424,  204513, 16, }, // America/Rankin_Inlet
    {  664, 1995,  204529, 10, }, // America/Regina
    {  664, 8460,  204539, 12, }, // America/Resolute
    {  664, 6111,  204551, 16, }, // America/Santo_Domingo
    {  664, 6899,  204567, 15, }, // America/Scoresbysund
    {  664, 8494,  204582,  8, }, // America/Sitka
    {  664, 8508,  204590, 12, }, // America/St_Barthelemy
    {  664, 2061,  204602, 14, }, // America/St_Johns
    {  664, 5863,  204616, 13, }, // America/St_Kitts
    {  664, 5880,  204629, 12, }, // America/St_Lucia
    {  664,  855,  204641, 14, }, // America/St_Thomas
    {  664, 5913,  204655, 12, }, // America/St_Vincent
    {  664, 8530,  204667, 19, }, // America/Swift_Current
    {  664, 5461,  204686, 14, }, // America/Tegucigalpa
    {  664, 5756,  204700,  8, }, // America/Thule
    {  664,  313,  204708, 11, }, // America/Tijuana
    {  664,  608,  204719, 11, }, // America/Toronto
    {  664, 5688,  204730, 10, }, // America/Tortola
    {  664, 2093,  204740, 11, }, // America/Vancouver
    {  664, 2144,  204751, 13, }, // America/Whitehorse
    {  664,  734,  204764, 12, }, // America/Winnipeg
    {  664, 8552,  204776, 10, }, // America/Yakutat
    {  664, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  664, 1259,  204786, 15, }, // Asia/Ho_Chi_Minh
    {  664, 1185,     448,  9, }, // Asia/Kathmandu
    {  664,  992,     457,  7, }, // Asia/Kolkata
    {  664, 1211,     464,  5, }, // Asia/Macau
    {  664, 1235,     477,  6, }, // Asia/Yangon
    {  664, 5671,  204801, 11, }, // Atlantic/Bermuda
    {  664, 1395,     483,  5, }, // Atlantic/Faroe
    {  664, 6821,     488, 10, }, // Atlantic/St_Helena
    {  664, 2527,     514,  4, }, // Europe/Kyiv
    {  664, 7211,     547,  7, }, // Indian/Reunion
    {  664, 3157,     554,  5, }, // Pacific/Chuuk
    {  664, 3096,     586,  7, }, // Pacific/Pohnpei
    {  665, 3346,  204812, 14, }, // America/Anchorage Yoruba/Latin/Benin
    {  665,  237,  204826, 12, }, // America/Atikokan
    {  665, 7623,  204838, 19, }, // America/Bahia_Banderas
    {  665, 5654,  204857, 15, }, // America/Barbados
    {  665, 5407,  204872, 12, }, // America/Belize
    {  665, 7660,  204884, 21, }, // America/Blanc-Sablon
    {  665, 7699,  204905, 10, }, // America/Boise
    {  665, 7756,  204915, 13, }, // America/Chihuahua
    {  665, 7796,  204928, 16, }, // America/Creston
    {  665, 5723,  204944, 12, }, // America/Curacao
    {  665,  481,  204956,  9, }, // America/Indiana/Knox
    {  665, 7961,  204965, 12, }, // America/Indiana/Marengo
    {  665, 8125,  204977, 14, }, // America/Kentucky/Monticello
    {  665, 5704,  204991, 17, }, // America/Kralendijk
    {  665, 5897,  205008, 15, }, // America/Marigot
    {  665, 2949,  205023, 11, }, // America/Mexico_City
    {  665, 4391,  205034, 13, }, // America/Miquelon
    {  665, 2061,  205047, 14, }, // America/St_Johns
    {  665,  855,  205061, 14, }, // America/St_Thomas
    {  665, 8530,  205075, 19, }, // America/Swift_Current
    {  665, 5671,  205094, 11, }, // Atlantic/Bermuda
    {  666,   14,       0,  6, }, // Africa/Asmara Zarma/Latin/Niger
    {  666, 4435,       6,  8, }, // Africa/Sao_Tome
    {  666, 4252,      14,  8, }, // America/Asuncion
    {  666,  237,      22,  8, }, // America/Atikokan
    {  666, 7623,      30, 17, }, // America/Bahia_Banderas
    {  666, 3868,      47,  6, }, // America/Cancun
    {  666, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  666, 5723,      66,  7, }, // America/Curacao
    {  666,  481,      73, 13, }, // America/Indiana/Knox
    {  666, 7961,      86, 16, }, // America/Indiana/Marengo
    {  666, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  666, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  666, 8038,     139, 14, }, // America/Indiana/Vevay
    {  666, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  666, 8086,     171, 16, }, // America/Indiana/Winamac
    {  666, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  666, 5932,     207, 22, }, // America/Lower_Princes
    {  666, 8220,     229,  6, }, // America/Merida
    {  666, 1850,     235, 19, }, // America/Noronha
    {  666, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  666, 8329,     274, 20, }, // America/North_Dakota/Center
    {  666, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  666,  393,     317,  4, }, // America/Nuuk
    {  666, 6899,     321, 16, }, // America/Scoresbysund
    {  666, 8508,     337, 14, }, // America/St_Barthelemy
    {  666, 2061,     351, 10, }, // America/St_Johns
    {  666, 5863,     361,  9, }, // America/St_Kitts
    {  666, 5880,     370,  9, }, // America/St_Lucia
    {  666,  855,     379, 10, }, // America/St_Thomas
    {  666, 5913,     389, 11, }, // America/St_Vincent
    {  666, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  666, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  666, 1185,     448,  9, }, // Asia/Kathmandu
    {  666,  992,     457,  7, }, // Asia/Kolkata
    {  666, 1211,     464,  5, }, // Asia/Macau
    {  666, 1235,     477,  6, }, // Asia/Yangon
    {  666, 1395,     483,  5, }, // Atlantic/Faroe
    {  666, 6821,     488, 10, }, // Atlantic/St_Helena
    {  666, 2527,     514,  4, }, // Europe/Kyiv
    {  666, 7211,     547,  7, }, // Indian/Reunion
    {  666, 3157,     554,  5, }, // Pacific/Chuuk
    {  666, 3096,     586,  7, }, // Pacific/Pohnpei
    {  667,   14,       0,  6, }, // Africa/Asmara Zhuang/Latin/China
    {  667, 4435,       6,  8, }, // Africa/Sao_Tome
    {  667, 4252,      14,  8, }, // America/Asuncion
    {  667,  237,      22,  8, }, // America/Atikokan
    {  667, 7623,      30, 17, }, // America/Bahia_Banderas
    {  667, 3868,      47,  6, }, // America/Cancun
    {  667, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  667, 5723,      66,  7, }, // America/Curacao
    {  667,  481,      73, 13, }, // America/Indiana/Knox
    {  667, 7961,      86, 16, }, // America/Indiana/Marengo
    {  667, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  667, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  667, 8038,     139, 14, }, // America/Indiana/Vevay
    {  667, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  667, 8086,     171, 16, }, // America/Indiana/Winamac
    {  667, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  667, 5932,     207, 22, }, // America/Lower_Princes
    {  667, 8220,     229,  6, }, // America/Merida
    {  667, 1850,     235, 19, }, // America/Noronha
    {  667, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  667, 8329,     274, 20, }, // America/North_Dakota/Center
    {  667, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  667,  393,     317,  4, }, // America/Nuuk
    {  667, 6899,     321, 16, }, // America/Scoresbysund
    {  667, 8508,     337, 14, }, // America/St_Barthelemy
    {  667, 2061,     351, 10, }, // America/St_Johns
    {  667, 5863,     361,  9, }, // America/St_Kitts
    {  667, 5880,     370,  9, }, // America/St_Lucia
    {  667,  855,     379, 10, }, // America/St_Thomas
    {  667, 5913,     389, 11, }, // America/St_Vincent
    {  667, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  667, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  667, 1185,     448,  9, }, // Asia/Kathmandu
    {  667,  992,     457,  7, }, // Asia/Kolkata
    {  667, 1211,     464,  5, }, // Asia/Macau
    {  667, 1235,     477,  6, }, // Asia/Yangon
    {  667, 1395,     483,  5, }, // Atlantic/Faroe
    {  667, 6821,     488, 10, }, // Atlantic/St_Helena
    {  667, 2527,     514,  4, }, // Europe/Kyiv
    {  667, 7211,     547,  7, }, // Indian/Reunion
    {  667, 3157,     554,  5, }, // Pacific/Chuuk
    {  667, 3096,     586,  7, }, // Pacific/Pohnpei
    {  668, 6788,  205105,  9, }, // Africa/Abidjan Zulu/Latin/South Africa
    {  668, 6760,  205114,  7, }, // Africa/Accra
    {  668, 5059,  205121, 13, }, // Africa/Addis_Ababa
    {  668, 7420,  205134,  9, }, // Africa/Algiers
    {  668,   14,  205143,  8, }, // Africa/Asmara
    {  668,   44,  205151,  8, }, // Africa/Bamako
    {  668, 5257,  205159,  8, }, // Africa/Bangui
    {  668, 6746,  205167,  8, }, // Africa/Banjul
    {  668, 7435,  205175,  8, }, // Africa/Bissau
    {  668, 4957,  205183, 10, }, // Africa/Blantyre
    {  668, 5287,  205193, 13, }, // Africa/Brazzaville
    {  668, 4922,  205206, 11, }, // Africa/Bujumbura
    {  668, 2320,  205217,  7, }, // Africa/Cairo
    {  668, 4117,  205224, 12, }, // Africa/Casablanca
    {  668, 7449,  205236,  7, }, // Africa/Ceuta
    {  668, 6773,  205243,  9, }, // Africa/Conakry
    {  668, 6840,  205252,  7, }, // Africa/Dakar
    {  668, 5130,  205259, 15, }, // Africa/Dar_es_Salaam
    {  668, 5043,  205274, 10, }, // Africa/Djibouti
    {  668, 5243,  205284,  8, }, // Africa/Douala
    {  668, 5166,  205292, 10, }, // Africa/El_Aaiun
    {  668, 6853,  205302, 10, }, // Africa/Freetown
    {  668, 4906,  205312, 10, }, // Africa/Gaborone
    {  668, 5015,  205322,  8, }, // Africa/Harare
    {  668, 4479,  205330, 14, }, // Africa/Johannesburg
    {  668, 4499,  205344,  5, }, // Africa/Juba
    {  668, 5151,  205349,  9, }, // Africa/Kampala
    {  668, 4524,  205358, 10, }, // Africa/Khartoum
    {  668, 4973,  205368,  8, }, // Africa/Kigali
    {  668, 5306,  205376, 10, }, // Africa/Kinshasa
    {  668, 4744,  205386,  7, }, // Africa/Lagos
    {  668, 5336,  205393, 12, }, // Africa/Libreville
    {  668, 6869,  205405,  6, }, // Africa/Lome
    {  668, 5211,  205411,  8, }, // Africa/Luanda
    {  668, 4939,  205419, 12, }, // Africa/Lubumbashi
    {  668, 5001,  205431,  7, }, // Africa/Lusaka
    {  668, 5322,  205438,  7, }, // Africa/Malabo
    {  668, 4987,  205445,  7, }, // Africa/Maputo
    {  668, 5197,  205452,  7, }, // Africa/Maseru
    {  668, 5182,  205459,  8, }, // Africa/Mbabane
    {  668, 5113,  205467, 11, }, // Africa/Mogadishu
    {  668, 7034,  205478, 10, }, // Africa/Monrovia
    {  668, 3853,  205488,  9, }, // Africa/Nairobi
    {  668, 5271,  205497, 10, }, // Africa/Ndjamena
    {  668, 5354,  205507,  8, }, // Africa/Niamey
    {  668, 6803,  205515, 12, }, // Africa/Nouakchott
    {  668, 6727,  205527, 13, }, // Africa/Ouagadougou
    {  668, 5225,  205540, 12, }, // Africa/Porto-Novo
    {  668, 4435,  205552, 10, }, // Africa/Sao_Tome
    {  668, 2866,  205562,  9, }, // Africa/Tripoli
    {  668, 6554,  205571,  7, }, // Africa/Tunis
    {  668, 4152,  205578, 10, }, // Africa/Windhoek
    {  668,  132,  205588,  6, }, // America/Adak
    {  668, 3346,  205594, 11, }, // America/Anchorage
    {  668, 5607,  205605, 10, }, // America/Anguilla
    {  668, 5624,  205615,  9, }, // America/Antigua
    {  668, 4554,  205624, 11, }, // America/Araguaina
    {  668,  166,  205635, 14, }, // America/Argentina/Buenos_Aires
    {  668,   91,  205649, 11, }, // America/Argentina/Catamarca
    {  668,  270,  205660,  9, }, // America/Argentina/Cordoba
    {  668,  441,  205669,  7, }, // America/Argentina/Jujuy
    {  668, 7462,  205676, 10, }, // America/Argentina/La_Rioja
    {  668,  565,  205686,  9, }, // America/Argentina/Mendoza
    {  668, 7489,  205695, 14, }, // America/Argentina/Rio_Gallegos
    {  668, 7520,  205709,  7, }, // America/Argentina/Salta
    {  668, 7544,  205716, 10, }, // America/Argentina/San_Juan
    {  668, 5580,  205726, 10, }, // America/Argentina/San_Luis
    {  668, 7571,  205736,  9, }, // America/Argentina/Tucuman
    {  668, 7597,  205745,  9, }, // America/Argentina/Ushuaia
    {  668, 5640,  205754,  7, }, // America/Aruba
    {  668, 4252,  205761, 10, }, // America/Asuncion
    {  668,  237,  205771, 10, }, // America/Atikokan
    {  668, 3679,  205781,  7, }, // America/Bahia
    {  668, 7623,  205788, 16, }, // America/Bahia_Banderas
    {  668, 5654,  205804, 10, }, // America/Barbados
    {  668, 7646,  205814,  7, }, // America/Belem
    {  668, 5407,  205821,  8, }, // America/Belize
    {  668, 7660,  205829, 14, }, // America/Blanc-Sablon
    {  668, 7681,  205843, 11, }, // America/Boa_Vista
    {  668, 4361,  205854,  8, }, // America/Bogota
    {  668, 7699,  205862,  7, }, // America/Boise
    {  668, 7713,  205869, 15, }, // America/Cambridge_Bay
    {  668, 7735,  205884, 14, }, // America/Campo_Grande
    {  668, 3868,  205898,  8, }, // America/Cancun
    {  668, 4694,  205906,  9, }, // America/Caracas
    {  668, 4345,  205915,  9, }, // America/Cayenne
    {  668, 5496,  205924,  8, }, // America/Cayman
    {  668, 2260,  205932,  9, }, // America/Chicago
    {  668, 7756,  205941, 11, }, // America/Chihuahua
    {  668, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  668, 5422,  205952, 12, }, // America/Costa_Rica
    {  668, 7796,  205964,  9, }, // America/Creston
    {  668, 3791,  205973,  8, }, // America/Cuiaba
    {  668, 5723,  205981,  9, }, // America/Curacao
    {  668, 7812,  205990, 14, }, // America/Danmarkshavn
    {  668, 7833,  206004,  8, }, // America/Dawson
    {  668, 7848,  206012, 14, }, // America/Dawson_Creek
    {  668,  805,  206026,  8, }, // America/Denver
    {  668, 3465,  206034,  9, }, // America/Detroit
    {  668, 5739,  206043, 10, }, // America/Dominica
    {  668,  893,  206053, 10, }, // America/Edmonton
    {  668, 7869,  206063, 10, }, // America/Eirunepe
    {  668, 5441,  206073, 13, }, // America/El_Salvador
    {  668, 7886,  206086, 13, }, // America/Fort_Nelson
    {  668, 7906,  206099, 11, }, // America/Fortaleza
    {  668, 7924,  206110, 11, }, // America/Glace_Bay
    {  668, 6881,  206121, 11, }, // America/Goose_Bay
    {  668, 4612,  206132, 12, }, // America/Grand_Turk
    {  668, 5770,  206144,  9, }, // America/Grenada
    {  668, 5786,  206153, 12, }, // America/Guadeloupe
    {  668, 3760,  206165, 11, }, // America/Guatemala
    {  668, 4841,  206176, 11, }, // America/Guayaquil
    {  668, 6932,  206187,  8, }, // America/Guyana
    {  668, 1939,  206195,  9, }, // America/Halifax
    {  668, 2281,  206204,  8, }, // America/Havana
    {  668, 7942,  206212, 12, }, // America/Hermosillo
    {  668,  348,  206224, 14, }, // America/Indiana/Indianapolis
    {  668,  481,  206238, 15, }, // America/Indiana/Knox
    {  668, 7961,  206253, 18, }, // America/Indiana/Marengo
    {  668, 7985,  206271, 21, }, // America/Indiana/Petersburg
    {  668, 8012,  206292, 20, }, // America/Indiana/Tell_City
    {  668, 8038,  206312, 16, }, // America/Indiana/Vevay
    {  668, 8060,  206328, 20, }, // America/Indiana/Vincennes
    {  668, 8086,  206348, 18, }, // America/Indiana/Winamac
    {  668, 8110,  206366,  8, }, // America/Inuvik
    {  668,  660,  206374,  9, }, // America/Iqaluit
    {  668, 2799,  206383,  9, }, // America/Jamaica
    {  668, 5380,  206392,  8, }, // America/Juneau
    {  668,  521,  206400, 12, }, // America/Kentucky/Louisville
    {  668, 8125,  206412, 22, }, // America/Kentucky/Monticello
    {  668, 5704,  206434, 12, }, // America/Kralendijk
    {  668, 4376,  206446,  8, }, // America/La_Paz
    {  668, 7169,  206454,  6, }, // America/Lima
    {  668, 3239,  206460, 13, }, // America/Los_Angeles
    {  668, 5932,  206473, 24, }, // America/Lower_Princes
    {  668, 8153,  206497,  8, }, // America/Maceio
    {  668, 8168,  206505,  9, }, // America/Managua
    {  668, 1908,  206514,  8, }, // America/Manaus
    {  668, 5897,  206522,  9, }, // America/Marigot
    {  668, 5805,  206531, 12, }, // America/Martinique
    {  668, 8184,  206543, 11, }, // America/Matamoros
    {  668, 2917,  206554, 10, }, // America/Mazatlan
    {  668, 8202,  206564, 11, }, // America/Menominee
    {  668, 8220,  206575,  8, }, // America/Merida
    {  668, 8235,  206583, 12, }, // America/Metlakatla
    {  668, 2949,  206595, 13, }, // America/Mexico_City
    {  668, 4391,  206608, 10, }, // America/Miquelon
    {  668, 8254,  206618,  9, }, // America/Moncton
    {  668, 8270,  206627, 11, }, // America/Monterrey
    {  668, 4098,  206638, 12, }, // America/Montevideo
    {  668, 5824,  206650, 12, }, // America/Montserrat
    {  668, 5481,  206662,  8, }, // America/Nassau
    {  668, 2379,  206670, 10, }, // America/New_York
    {  668, 8288,  206680,  6, }, // America/Nome
    {  668, 1850,  206686,  9, }, // America/Noronha
    {  668, 8301,  206695, 22, }, // America/North_Dakota/Beulah
    {  668, 8329,  206717, 22, }, // America/North_Dakota/Center
    {  668, 8357,  206739, 25, }, // America/North_Dakota/New_Salem
    {  668,  393,  206764,  6, }, // America/Nuuk
    {  668, 8388,  206770,  9, }, // America/Ojinaga
    {  668, 2356,  206779,  8, }, // America/Panama
    {  668, 6173,  206787, 12, }, // America/Paramaribo
    {  668, 2973,  206799,  9, }, // America/Phoenix
    {  668, 3945,  206808, 16, }, // America/Port-au-Prince
    {  668, 5954,  206824, 15, }, // America/Port_of_Spain
    {  668, 8404,  206839, 13, }, // America/Porto_Velho
    {  668, 5843,  206852, 13, }, // America/Puerto_Rico
    {  668, 4030,  206865, 14, }, // America/Punta_Arenas
    {  668, 8424,  206879, 14, }, // America/Rankin_Inlet
    {  668, 8445,  206893,  8, }, // America/Recife
    {  668, 1995,  206901,  8, }, // America/Regina
    {  668, 8460,  206909, 10, }, // America/Resolute
    {  668,  695,  206919, 12, }, // America/Rio_Branco
    {  668, 8477,  206931, 10, }, // America/Santarem
    {  668, 2201,  206941, 10, }, // America/Santiago
    {  668, 6111,  206951, 15, }, // America/Santo_Domingo
    {  668, 1878,  206966, 11, }, // America/Sao_Paulo
    {  668, 6899,  206977, 18, }, // America/Scoresbysund
    {  668, 8494,  206995,  7, }, // America/Sitka
    {  668, 8508,  207002, 16, }, // America/St_Barthelemy
    {  668, 2061,  207018, 12, }, // America/St_Johns
    {  668, 5863,  207030, 11, }, // America/St_Kitts
    {  668, 5880,  207041, 11, }, // America/St_Lucia
    {  668,  855,  207052, 12, }, // America/St_Thomas
    {  668, 5913,  207064, 13, }, // America/St_Vincent
    {  668, 8530,  207077, 15, }, // America/Swift_Current
    {  668, 5461,  207092, 13, }, // America/Tegucigalpa
    {  668, 5756,  207105,  7, }, // America/Thule
    {  668,  313,  207112,  9, }, // America/Tijuana
    {  668,  608,  207121,  9, }, // America/Toronto
    {  668, 5688,  207130,  9, }, // America/Tortola
    {  668, 2093,  207139, 11, }, // America/Vancouver
    {  668, 2144,  207150, 12, }, // America/Whitehorse
    {  668,  734,  207162, 10, }, // America/Winnipeg
    {  668, 8552,  207172,  9, }, // America/Yakutat
    {  668, 6001,  207181,  7, }, // Antarctica/Casey
    {  668, 6094,  207188,  7, }, // Antarctica/Davis
    {  668, 6133,  207195, 18, }, // Antarctica/DumontDUrville
    {  668, 8568,  207213, 11, }, // Antarctica/Macquarie
    {  668, 7066,  207224,  8, }, // Antarctica/Mawson
    {  668, 7113,  207232,  9, }, // Antarctica/McMurdo
    {  668, 8589,  207241,  8, }, // Antarctica/Palmer
    {  668, 7226,  207249,  9, }, // Antarctica/Rothera
    {  668, 7295,  207258,  7, }, // Antarctica/Syowa
    {  668, 8607,  207265,  7, }, // Antarctica/Troll
    {  668, 7374,  207272,  8, }, // Antarctica/Vostok
    {  668, 1429,  207280, 14, }, // Arctic/Longyearbyen
    {  668, 5570,  207294,  6, }, // Asia/Aden
    {  668, 5395,  207300,  8, }, // Asia/Almaty
    {  668, 3968,  207308,  7, }, // Asia/Amman
    {  668, 5511,  207315,  8, }, // Asia/Anadyr
    {  668, 5523,  207323,  7, }, // Asia/Aqtau
    {  668, 5368,  207330,  8, }, // Asia/Aqtobe
    {  668,  964,  207338, 10, }, // Asia/Ashgabat
    {  668, 3607,  207348,  9, }, // Asia/Baghdad
    {  668, 5534,  207357,  9, }, // Asia/Bahrain
    {  668, 3653,  207366,  6, }, // Asia/Baku
    {  668, 4466,  207372,  9, }, // Asia/Bangkok
    {  668, 3571,  207381,  9, }, // Asia/Barnaul
    {  668, 4086,  207390,  8, }, // Asia/Beirut
    {  668, 3778,  207398,  9, }, // Asia/Bishkek
    {  668, 5989,  207407,  8, }, // Asia/Brunei
    {  668, 4601,  207415,  7, }, // Asia/Chita
    {  668, 4511,  207422,  9, }, // Asia/Colombo
    {  668, 4540,  207431, 10, }, // Asia/Damascus
    {  668, 1093,  207441,  7, }, // Asia/Dhaka
    {  668, 6192,  207448,  6, }, // Asia/Dili
    {  668, 3596,  207454,  7, }, // Asia/Dubai
    {  668, 6159,  207461, 10, }, // Asia/Dushanbe
    {  668, 8651,  207471,  5, }, // Asia/Gaza
    {  668, 4795,  207476,  8, }, // Asia/Hebron
    {  668, 1259,  207484, 18, }, // Asia/Ho_Chi_Minh
    {  668, 2704,  207502, 11, }, // Asia/Hong_Kong
    {  668, 4771,  207513,  6, }, // Asia/Hovd
    {  668, 4184,  207519,  9, }, // Asia/Irkutsk
    {  668, 7006,  207528,  9, }, // Asia/Jakarta
    {  668, 6992,  207537, 10, }, // Asia/Jayapura
    {  668, 1290,  207547, 11, }, // Asia/Jerusalem
    {  668, 3560,  207558,  7, }, // Asia/Kabul
    {  668, 4316,  207565, 11, }, // Asia/Kamchatka
    {  668, 4239,  207576,  9, }, // Asia/Karachi
    {  668, 1185,  207585, 11, }, // Asia/Kathmandu
    {  668, 8661,  207596, 10, }, // Asia/Khandyga
    {  668,  992,  207606,  9, }, // Asia/Kolkata
    {  668, 4197,  207615, 13, }, // Asia/Krasnoyarsk
    {  668, 4859,  207628, 14, }, // Asia/Kuala_Lumpur
    {  668, 5976,  207642,  9, }, // Asia/Kuching
    {  668, 5547,  207651,  8, }, // Asia/Kuwait
    {  668, 1211,  207659,  7, }, // Asia/Macau
    {  668, 4017,  207666,  9, }, // Asia/Magadan
    {  668, 1349,  207675, 10, }, // Asia/Makassar
    {  668, 7182,  207685,  8, }, // Asia/Manila
    {  668, 6920,  207693,  8, }, // Asia/Muscat
    {  668, 2554,  207701,  9, }, // Asia/Nicosia
    {  668, 8675,  207710, 14, }, // Asia/Novokuznetsk
    {  668, 4135,  207724, 13, }, // Asia/Novosibirsk
    {  668, 4229,  207737,  6, }, // Asia/Omsk
    {  668, 7145,  207743,  6, }, // Asia/Oral
    {  668, 6961,  207749, 12, }, // Asia/Phnom_Penh
    {  668, 8693,  207761, 11, }, // Asia/Pontianak
    {  668, 4214,  207772, 11, }, // Asia/Pyongyang
    {  668, 5559,  207783,  7, }, // Asia/Qatar
    {  668, 8708,  207790, 10, }, // Asia/Qostanay
    {  668, 4269,  207800, 11, }, // Asia/Qyzylorda
    {  668, 3584,  207811,  8, }, // Asia/Riyadh
    {  668, 4408,  207819, 10, }, // Asia/Sakhalin
    {  668, 7245,  207829, 11, }, // Asia/Samarkand
    {  668, 3279,  207840,  7, }, // Asia/Seoul
    {  668, 1053,  207847, 10, }, // Asia/Shanghai
    {  668, 3300,  207857, 11, }, // Asia/Singapore
    {  668, 4297,  207868, 15, }, // Asia/Srednekolymsk
    {  668, 3263,  207883,  8, }, // Asia/Taipei
    {  668, 4781,  207891, 10, }, // Asia/Tashkent
    {  668, 3915,  207901,  9, }, // Asia/Tbilisi
    {  668, 2772,  207910,  8, }, // Asia/Tehran
    {  668, 1317,  207918,  9, }, // Asia/Thimphu
    {  668, 2821,  207927,  7, }, // Asia/Tokyo
    {  668, 4572,  207934,  7, }, // Asia/Tomsk
    {  668, 1021,  207941, 13, }, // Asia/Ulaanbaatar
    {  668, 1159,  207954,  8, }, // Asia/Urumqi
    {  668, 8722,  207962, 10, }, // Asia/Ust-Nera
    {  668, 6977,  207972, 11, }, // Asia/Vientiane
    {  668, 4710,  207983, 13, }, // Asia/Vladivostok
    {  668, 4828,  207996,  9, }, // Asia/Yakutsk
    {  668, 1235,  208005,  9, }, // Asia/Yangon
    {  668, 3883,  208014, 15, }, // Asia/Yekaterinburg
    {  668, 3747,  208029,  9, }, // Asia/Yerevan
    {  668, 3663,  208038,  8, }, // Atlantic/Azores
    {  668, 5671,  208046,  9, }, // Atlantic/Bermuda
    {  668, 6628,  208055,  8, }, // Atlantic/Canary
    {  668, 3727,  208063, 12, }, // Atlantic/Cape_Verde
    {  668, 1395,  208075,  7, }, // Atlantic/Faroe
    {  668, 8736,  208082,  9, }, // Atlantic/Madeira
    {  668, 2748,  208091, 11, }, // Atlantic/Reykjavik
    {  668, 7272,  208102, 15, }, // Atlantic/South_Georgia
    {  668, 6821,  208117, 12, }, // Atlantic/St_Helena
    {  668, 6644,  208129,  9, }, // Atlantic/Stanley
    {  668, 1670,  208138, 10, }, // Australia/Adelaide
    {  668, 1635,  208148, 10, }, // Australia/Brisbane
    {  668, 1799,  208158, 13, }, // Australia/Broken_Hill
    {  668, 1583,  208171,  8, }, // Australia/Darwin
    {  668, 3637,  208179,  7, }, // Australia/Eucla
    {  668, 1516,  208186,  8, }, // Australia/Hobart
    {  668, 8753,  208194, 10, }, // Australia/Lindeman
    {  668, 1547,  208204, 11, }, // Australia/Lord_Howe
    {  668, 1727,  208215, 11, }, // Australia/Melbourne
    {  668, 1762,  208226,  7, }, // Australia/Perth
    {  668, 1463,  208233,  8, }, // Australia/Sydney
    {  668, 6441,  208241, 11, }, // Europe/Amsterdam
    {  668, 6216,  208252,  9, }, // Europe/Andorra
    {  668, 3620,  208261, 11, }, // Europe/Astrakhan
    {  668, 2300,  208272,  8, }, // Europe/Athens
    {  668, 6336,  208280, 10, }, // Europe/Belgrade
    {  668, 4757,  208290,  8, }, // Europe/Berlin
    {  668, 6488,  208298, 12, }, // Europe/Bratislava
    {  668, 2167,  208310, 10, }, // Europe/Brussels
    {  668, 3928,  208320, 11, }, // Europe/Bucharest
    {  668, 3806,  208331, 10, }, // Europe/Budapest
    {  668, 8772,  208341, 10, }, // Europe/Busingen
    {  668, 2583,  208351, 10, }, // Europe/Chisinau
    {  668, 6289,  208361, 12, }, // Europe/Copenhagen
    {  668, 2338,  208373,  8, }, // Europe/Dublin
    {  668, 6307,  208381, 11, }, // Europe/Gibraltar
    {  668, 8788,  208392, 10, }, // Europe/Guernsey
    {  668, 6612,  208402, 10, }, // Europe/Helsinki
    {  668, 8804,  208412, 13, }, // Europe/Isle_of_Man
    {  668, 1130,  208425, 10, }, // Europe/Istanbul
    {  668, 8823,  208435,  8, }, // Europe/Jersey
    {  668, 3979,  208443, 13, }, // Europe/Kaliningrad
    {  668, 8929,  208456,  7, }, // Europe/Kirov
    {  668, 2527,  208463,  6, }, // Europe/Kyiv
    {  668, 3213,  208469,  8, }, // Europe/Lisbon
    {  668, 6506,  208477, 11, }, // Europe/Ljubljana
    {  668, 2501,  208488,  8, }, // Europe/London
    {  668, 6365,  208496, 12, }, // Europe/Luxembourg
    {  668, 4892,  208508,  8, }, // Europe/Madrid
    {  668, 6397,  208516,  7, }, // Europe/Malta
    {  668, 6582,  208523, 11, }, // Europe/Mariehamn
    {  668, 3693,  208534,  7, }, // Europe/Minsk
    {  668, 6410,  208541,  8, }, // Europe/Monaco
    {  668, 3537,  208549,  8, }, // Europe/Moscow
    {  668, 6458,  208557,  6, }, // Europe/Oslo
    {  668, 4284,  208563,  7, }, // Europe/Paris
    {  668, 6424,  208570, 11, }, // Europe/Podgorica
    {  668, 6275,  208581,  8, }, // Europe/Prague
    {  668, 8837,  208589,  6, }, // Europe/Riga
    {  668, 6324,  208595,  6, }, // Europe/Rome
    {  668, 4331,  208601,  8, }, // Europe/Samara
    {  668, 6470,  208609, 12, }, // Europe/San_Marino
    {  668, 6245,  208621, 10, }, // Europe/Sarajevo
    {  668, 4451,  208631,  9, }, // Europe/Saratov
    {  668, 8849,  208640, 12, }, // Europe/Simferopol
    {  668, 6383,  208652,  8, }, // Europe/Skopje
    {  668, 6599,  208660,  7, }, // Europe/Sofia
    {  668, 6523,  208667, 11, }, // Europe/Stockholm
    {  668, 8867,  208678,  9, }, // Europe/Tallinn
    {  668, 6202,  208687,  8, }, // Europe/Tirane
    {  668, 8882,  208695, 11, }, // Europe/Ulyanovsk
    {  668, 6352,  208706,  7, }, // Europe/Vaduz
    {  668, 6567,  208713,  9, }, // Europe/Vatican
    {  668, 6231,  208722,  8, }, // Europe/Vienna
    {  668, 8899,  208730,  9, }, // Europe/Vilnius
    {  668, 4727,  208739, 11, }, // Europe/Volgograd
    {  668, 3190,  208750,  8, }, // Europe/Warsaw
    {  668, 6261,  208758,  8, }, // Europe/Zagreb
    {  668, 6540,  208766,  8, }, // Europe/Zurich
    {  668, 5078,  208774, 14, }, // Indian/Antananarivo
    {  668, 6947,  208788,  8, }, // Indian/Chagos
    {  668, 6046,  208796, 10, }, // Indian/Christmas
    {  668, 6063,  208806,  7, }, // Indian/Cocos
    {  668, 5029,  208813,  8, }, // Indian/Comoro
    {  668, 6661,  208821, 11, }, // Indian/Kerguelen
    {  668, 7260,  208832,  5, }, // Indian/Mahe
    {  668, 7050,  208837, 10, }, // Indian/Maldives
    {  668, 4069,  208847, 11, }, // Indian/Mauritius
    {  668, 5098,  208858,  9, }, // Indian/Mayotte
    {  668, 7211,  208867,  9, }, // Indian/Reunion
    {  668, 4422,  208876,  6, }, // Pacific/Apia
    {  668,  932,  208882, 10, }, // Pacific/Auckland
    {  668, 3706,  208892, 14, }, // Pacific/Bougainville
    {  668, 3015,  208906,  9, }, // Pacific/Chatham
    {  668, 3157,  208915,  7, }, // Pacific/Chuuk
    {  668, 2237,  208922,  8, }, // Pacific/Easter
    {  668, 7360,  208930,  7, }, // Pacific/Efate
    {  668, 7327,  208937,  9, }, // Pacific/Fakaofo
    {  668, 3902,  208946,  6, }, // Pacific/Fiji
    {  668, 7343,  208952, 10, }, // Pacific/Funafuti
    {  668, 6678,  208962, 11, }, // Pacific/Galapagos
    {  668, 6696,  208973,  9, }, // Pacific/Gambier
    {  668, 3822,  208982, 13, }, // Pacific/Guadalcanal
    {  668, 6018,  208995,  6, }, // Pacific/Guam
    {  668, 2723,  209001, 10, }, // Pacific/Honolulu
    {  668, 3049,  209011, 11, }, // Pacific/Kanton
    {  668, 3998,  209022, 12, }, // Pacific/Kiritimati
    {  668, 7019,  209034,  8, }, // Pacific/Kosrae
    {  668, 2842,  209042, 11, }, // Pacific/Kwajalein
    {  668, 4877,  209053,  8, }, // Pacific/Majuro
    {  668, 4051,  209061, 11, }, // Pacific/Marquesas
    {  668, 8914,  209072,  8, }, // Pacific/Midway
    {  668, 7084,  209080,  7, }, // Pacific/Nauru
    {  668, 7132,  209087,  6, }, // Pacific/Niue
    {  668, 4168,  209093,  9, }, // Pacific/Norfolk
    {  668, 7098,  209102,  8, }, // Pacific/Noumea
    {  668, 3126,  209110, 11, }, // Pacific/Pago_Pago
    {  668, 7155,  209121,  7, }, // Pacific/Palau
    {  668, 7194,  209128, 10, }, // Pacific/Pitcairn
    {  668, 3096,  209138,  9, }, // Pacific/Pohnpei
    {  668, 4807,  209147, 14, }, // Pacific/Port_Moresby
    {  668, 6076,  209161, 11, }, // Pacific/Rarotonga
    {  668, 6031,  209172,  8, }, // Pacific/Saipan
    {  668, 7312,  209180,  8, }, // Pacific/Tahiti
    {  668, 6712,  209188,  8, }, // Pacific/Tarawa
    {  668, 4583,  209196, 11, }, // Pacific/Tongatapu
    {  668, 7392,  209207,  6, }, // Pacific/Wake
    {  668, 7405,  209213,  8, }, // Pacific/Wallis
    {  669, 6788,  209221,  7, }, // Africa/Abidjan Kaingang/Latin/Brazil
    {  669, 6760,    4649,  4, }, // Africa/Accra
    {  669, 5059,  209228, 11, }, // Africa/Addis_Ababa
    {  669, 7420,  209239,  5, }, // Africa/Algiers
    {  669,   14,  209244,  7, }, // Africa/Asmara
    {  669,   44,  209251,  6, }, // Africa/Bamako
    {  669, 5257,  209257,  4, }, // Africa/Bangui
    {  669, 6746,  209261,  6, }, // Africa/Banjul
    {  669, 7435,  209267,  5, }, // Africa/Bissau
    {  669, 4957,  209272,  8, }, // Africa/Blantyre
    {  669, 5287,  209280,  9, }, // Africa/Brazzaville
    {  669, 4922,  209289,  8, }, // Africa/Bujumbura
    {  669, 2320,    6346,  5, }, // Africa/Cairo
    {  669, 4117,  209297,  7, }, // Africa/Casablanca
    {  669, 7449,  209304,  6, }, // Africa/Ceuta
    {  669, 6773,  209310,  7, }, // Africa/Conakry
    {  669, 6840,  209317,  5, }, // Africa/Dakar
    {  669, 5130,  209322, 13, }, // Africa/Dar_es_Salaam
    {  669, 5043,  209335,  7, }, // Africa/Djibouti
    {  669, 5243,  209342,  6, }, // Africa/Douala
    {  669, 5166,  209348,  9, }, // Africa/El_Aaiun
    {  669, 6853,  209357,  7, }, // Africa/Freetown
    {  669, 4906,  209364,  8, }, // Africa/Gaborone
    {  669, 5015,   98353,  6, }, // Africa/Harare
    {  669, 4479,  209372, 13, }, // Africa/Johannesburg
    {  669, 4499,  209385,  4, }, // Africa/Juba
    {  669, 5151,  209389,  7, }, // Africa/Kampala
    {  669, 4524,  209396,  6, }, // Africa/Khartoum
    {  669, 4973,  209402,  6, }, // Africa/Kigali
    {  669, 5306,  209408,  6, }, // Africa/Kinshasa
    {  669, 4744,  209414,  4, }, // Africa/Lagos
    {  669, 5336,  209418,  9, }, // Africa/Libreville
    {  669, 6869,  209427,  4, }, // Africa/Lome
    {  669, 5211,  209431,  6, }, // Africa/Luanda
    {  669, 4939,  209437,  8, }, // Africa/Lubumbashi
    {  669, 5001,  209445,  6, }, // Africa/Lusaka
    {  669, 5322,  209451,  6, }, // Africa/Malabo
    {  669, 4987,  209457,  6, }, // Africa/Maputo
    {  669, 5197,  209463,  6, }, // Africa/Maseru
    {  669, 5182,  209469,  8, }, // Africa/Mbabane
    {  669, 5113,  209477, 10, }, // Africa/Mogadishu
    {  669, 7034,  209487,  9, }, // Africa/Monrovia
    {  669, 3853,  209496,  7, }, // Africa/Nairobi
    {  669, 5271,  209503, 11, }, // Africa/Ndjamena
    {  669, 5354,  209514,  6, }, // Africa/Niamey
    {  669, 6803,  209520,  9, }, // Africa/Nouakchott
    {  669, 6727,  209529, 10, }, // Africa/Ouagadougou
    {  669, 5225,  209539,  9, }, // Africa/Porto-Novo
    {  669, 4435,  209548,  7, }, // Africa/Sao_Tome
    {  669, 2866,  209555,  7, }, // Africa/Tripoli
    {  669, 6554,  209562,  6, }, // Africa/Tunis
    {  669, 4152,  209568,  8, }, // Africa/Windhoek
    {  669,  132,  209576,  5, }, // America/Adak
    {  669, 3346,  209581,  7, }, // America/Anchorage
    {  669, 5607,  209588,  5, }, // America/Anguilla
    {  669, 5624,  209593,  7, }, // America/Antigua
    {  669, 4554,  209600,  9, }, // America/Araguaina
    {  669,  166,  209609, 10, }, // America/Argentina/Buenos_Aires
    {  669,   91,  209619,  9, }, // America/Argentina/Catamarca
    {  669,  270,  209628,  7, }, // America/Argentina/Cordoba
    {  669,  441,  209635,  5, }, // America/Argentina/Jujuy
    {  669, 7462,  209640,  8, }, // America/Argentina/La_Rioja
    {  669,  565,  209648,  6, }, // America/Argentina/Mendoza
    {  669, 7489,  209654, 10, }, // America/Argentina/Rio_Gallegos
    {  669, 7520,  209664,  5, }, // America/Argentina/Salta
    {  669, 7544,  209669,  7, }, // America/Argentina/San_Juan
    {  669, 5580,  209676,  6, }, // America/Argentina/San_Luis
    {  669, 7571,  209682,  6, }, // America/Argentina/Tucuman
    {  669, 7597,  209688,  6, }, // America/Argentina/Ushuaia
    {  669, 5640,  209694,  5, }, // America/Aruba
    {  669, 4252,  209699,  6, }, // America/Asuncion
    {  669,  237,  209705,  8, }, // America/Atikokan
    {  669, 3679,  209713,  5, }, // America/Bahia
    {  669, 7623,  209718, 16, }, // America/Bahia_Banderas
    {  669, 5654,  209734,  7, }, // America/Barbados
    {  669, 7646,  209741,  5, }, // America/Belem
    {  669, 5407,  209746,  6, }, // America/Belize
    {  669, 7660,  209752, 12, }, // America/Blanc-Sablon
    {  669, 7681,  209764,  7, }, // America/Boa_Vista
    {  669, 4361,  209771,  6, }, // America/Bogota
    {  669, 7699,  209777,  5, }, // America/Boise
    {  669, 7713,  209782, 12, }, // America/Cambridge_Bay
    {  669, 7735,  209794,  6, }, // America/Campo_Grande
    {  669, 3868,  209800,  5, }, // America/Cancun
    {  669, 4694,  209805,  8, }, // America/Caracas
    {  669, 4345,  209813,  6, }, // America/Cayenne
    {  669, 5496,  209819,  6, }, // America/Cayman
    {  669, 2260,  209825,  6, }, // America/Chicago
    {  669, 7756,  209831, 10, }, // America/Chihuahua
    {  669, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  669, 5422,  209841, 11, }, // America/Costa_Rica
    {  669, 7796,  209852,  8, }, // America/Creston
    {  669, 3791,  209860,  6, }, // America/Cuiaba
    {  669, 5723,  209866,  8, }, // America/Curacao
    {  669, 7812,  209874, 13, }, // America/Danmarkshavn
    {  669, 7833,  209887,  6, }, // America/Dawson
    {  669, 7848,  209893, 12, }, // America/Dawson_Creek
    {  669,  805,  209905,  6, }, // America/Denver
    {  669, 3465,  209911,  9, }, // America/Detroit
    {  669, 5739,  209920,  8, }, // America/Dominica
    {  669,  893,  209928,  8, }, // America/Edmonton
    {  669, 7869,  209936,  8, }, // America/Eirunepe
    {  669, 5441,  209944, 11, }, // America/El_Salvador
    {  669, 7886,  209955, 11, }, // America/Fort_Nelson
    {  669, 7906,  209966,  9, }, // America/Fortaleza
    {  669, 7924,  209975,  9, }, // America/Glace_Bay
    {  669, 6881,  209984,  8, }, // America/Goose_Bay
    {  669, 4612,  209992,  9, }, // America/Grand_Turk
    {  669, 5770,  210001,  7, }, // America/Grenada
    {  669, 5786,  210008, 10, }, // America/Guadeloupe
    {  669, 3760,  210018, 10, }, // America/Guatemala
    {  669, 4841,  210028,  9, }, // America/Guayaquil
    {  669, 6932,  210037,  6, }, // America/Guyana
    {  669, 1939,  210043,  7, }, // America/Halifax
    {  669, 2281,  210050,  6, }, // America/Havana
    {  669, 7942,  210056,  9, }, // America/Hermosillo
    {  669,  348,  210065, 13, }, // America/Indiana/Indianapolis
    {  669,  481,  210078, 14, }, // America/Indiana/Knox
    {  669, 7961,  210092, 16, }, // America/Indiana/Marengo
    {  669, 7985,  210108, 19, }, // America/Indiana/Petersburg
    {  669, 8012,  210127, 17, }, // America/Indiana/Tell_City
    {  669, 8038,  210144, 14, }, // America/Indiana/Vevay
    {  669, 8060,  210158, 17, }, // America/Indiana/Vincennes
    {  669, 8086,  210175, 17, }, // America/Indiana/Winamac
    {  669, 8110,  210192,  7, }, // America/Inuvik
    {  669,  660,  210199,  8, }, // America/Iqaluit
    {  669, 2799,  210207,  7, }, // America/Jamaica
    {  669, 5380,  210214,  6, }, // America/Juneau
    {  669,  521,  210220,  9, }, // America/Kentucky/Louisville
    {  669, 8125,  210229, 17, }, // America/Kentucky/Monticello
    {  669, 5704,  210246,  9, }, // America/Kralendijk
    {  669, 4376,  210255,  7, }, // America/La_Paz
    {  669, 7169,  210262,  4, }, // America/Lima
    {  669, 3239,  210266, 11, }, // America/Los_Angeles
    {  669, 5932,  210277, 21, }, // America/Lower_Princes
    {  669, 8153,  210298,  6, }, // America/Maceio
    {  669, 8168,  210304,  8, }, // America/Managua
    {  669, 1908,  210312,  6, }, // America/Manaus
    {  669, 5897,  210318,  8, }, // America/Marigot
    {  669, 5805,  210326,  9, }, // America/Martinique
    {  669, 8184,  210335, 12, }, // America/Matamoros
    {  669, 2917,  210347,  8, }, // America/Mazatlan
    {  669, 8202,  210355,  8, }, // America/Menominee
    {  669, 8220,  210363,  6, }, // America/Merida
    {  669, 8235,  210369, 12, }, // America/Metlakatla
    {  669, 2949,  210381, 13, }, // America/Mexico_City
    {  669, 4391,  210394,  7, }, // America/Miquelon
    {  669, 8254,  210401,  6, }, // America/Moncton
    {  669, 8270,  210407,  7, }, // America/Monterrey
    {  669, 4098,  210414,  9, }, // America/Montevideo
    {  669, 5824,  210423,  9, }, // America/Montserrat
    {  669, 5481,  210432,  5, }, // America/Nassau
    {  669, 2379,  210437, 10, }, // America/New_York
    {  669, 8288,  210447,  4, }, // America/Nome
    {  669, 1850,  210451, 17, }, // America/Noronha
    {  669, 8301,  210468, 19, }, // America/North_Dakota/Beulah
    {  669, 8329,  210487, 20, }, // America/North_Dakota/Center
    {  669, 8357,  210507, 23, }, // America/North_Dakota/New_Salem
    {  669,  393,  210530,  3, }, // America/Nuuk
    {  669, 8388,  210533,  7, }, // America/Ojinaga
    {  669, 2356,  210540,  6, }, // America/Panama
    {  669, 6173,  210546, 10, }, // America/Paramaribo
    {  669, 2973,  210556,  7, }, // America/Phoenix
    {  669, 3945,  210563, 13, }, // America/Port-au-Prince
    {  669, 5954,  210576, 16, }, // America/Port_of_Spain
    {  669, 8404,  210592, 10, }, // America/Porto_Velho
    {  669, 5843,  166677, 10, }, // America/Puerto_Rico
    {  669, 4030,  210602, 10, }, // America/Punta_Arenas
    {  669, 8424,  210612, 11, }, // America/Rankin_Inlet
    {  669, 8445,  210623,  6, }, // America/Recife
    {  669, 1995,  210629,  6, }, // America/Regina
    {  669, 8460,  210635,  8, }, // America/Resolute
    {  669,  695,  210643,  9, }, // America/Rio_Branco
    {  669, 8477,  210652,  7, }, // America/Santarem
    {  669, 2201,  210659,  8, }, // America/Santiago
    {  669, 6111,  210667, 12, }, // America/Santo_Domingo
    {  669, 1878,  210679,  9, }, // America/Sao_Paulo
    {  669, 6899,  210688, 15, }, // America/Scoresbysund
    {  669, 8494,  210703,  5, }, // America/Sitka
    {  669, 8508,  210708, 12, }, // America/St_Barthelemy
    {  669, 2061,  210720,  8, }, // America/St_Johns
    {  669, 5863,  210728, 10, }, // America/St_Kitts
    {  669, 5880,  210738,  9, }, // America/St_Lucia
    {  669,  855,  210747,  8, }, // America/St_Thomas
    {  669, 5913,  210755,  9, }, // America/St_Vincent
    {  669, 8530,  210764, 15, }, // America/Swift_Current
    {  669, 5461,  210779, 11, }, // America/Tegucigalpa
    {  669, 5756,  210790,  4, }, // America/Thule
    {  669,  313,  210794,  6, }, // America/Tijuana
    {  669,  608,  210800,  6, }, // America/Toronto
    {  669, 5688,  210806,  7, }, // America/Tortola
    {  669, 2093,  210813,  8, }, // America/Vancouver
    {  669, 2144,  210821, 12, }, // America/Whitehorse
    {  669,  734,  210833,  7, }, // America/Winnipeg
    {  669, 8552,  210840,  9, }, // America/Yakutat
    {  669, 6001,  210849,  5, }, // Antarctica/Casey
    {  669, 6094,  210854,  6, }, // Antarctica/Davis
    {  669, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  669, 8568,  210860,  9, }, // Antarctica/Macquarie
    {  669, 7066,  210869,  6, }, // Antarctica/Mawson
    {  669, 7113,  210875,  9, }, // Antarctica/McMurdo
    {  669, 8589,  210884,  6, }, // Antarctica/Palmer
    {  669, 7226,    5583,  6, }, // Antarctica/Rothera
    {  669, 7295,  126319,  5, }, // Antarctica/Syowa
    {  669, 8607,  210890,  5, }, // Antarctica/Troll
    {  669, 7374,  210895,  8, }, // Antarctica/Vostok
    {  669, 1429,  210903, 14, }, // Arctic/Longyearbyen
    {  669, 5570,  210917,  4, }, // Asia/Aden
    {  669, 5395,  210921,  6, }, // Asia/Almaty
    {  669, 3968,  210927,  3, }, // Asia/Amman
    {  669, 5511,  210930,  7, }, // Asia/Anadyr
    {  669, 5523,  210937,  6, }, // Asia/Aqtau
    {  669, 5368,  210943,  7, }, // Asia/Aqtobe
    {  669,  964,  210950,  9, }, // Asia/Ashgabat
    {  669, 8624,  210959,  6, }, // Asia/Atyrau
    {  669, 3607,  210965,  6, }, // Asia/Baghdad
    {  669, 5534,  210971,  5, }, // Asia/Bahrain
    {  669, 3653,   22278,  4, }, // Asia/Baku
    {  669, 4466,  210976,  7, }, // Asia/Bangkok
    {  669, 3571,  210983,  7, }, // Asia/Barnaul
    {  669, 4086,  210990,  7, }, // Asia/Beirut
    {  669, 3778,  210997,  8, }, // Asia/Bishkek
    {  669, 5989,  211005,  6, }, // Asia/Brunei
    {  669, 4601,  211011,  4, }, // Asia/Chita
    {  669, 4511,  211015,  7, }, // Asia/Colombo
    {  669, 4540,  211022,  8, }, // Asia/Damascus
    {  669, 1093,  209317,  4, }, // Asia/Dhaka
    {  669, 6192,  211030,  4, }, // Asia/Dili
    {  669, 3596,  211034,  5, }, // Asia/Dubai
    {  669, 6159,  211039,  7, }, // Asia/Dushanbe
    {  669, 8636,  211046,  9, }, // Asia/Famagusta
    {  669, 8651,  211055,  4, }, // Asia/Gaza
    {  669, 4795,  211059,  6, }, // Asia/Hebron
    {  669, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  669, 2704,  211065,  7, }, // Asia/Hong_Kong
    {  669, 4771,  211072,  5, }, // Asia/Hovd
    {  669, 4184,  211077, 10, }, // Asia/Irkutsk
    {  669, 6992,  211087,  8, }, // Asia/Jayapura
    {  669, 1290,  211095,  9, }, // Asia/Jerusalem
    {  669, 3560,  211104,  5, }, // Asia/Kabul
    {  669, 4316,  211109,  8, }, // Asia/Kamchatka
    {  669, 4239,  211117,  6, }, // Asia/Karachi
    {  669, 1185,  211123,  9, }, // Asia/Kathmandu
    {  669, 8661,  211132,  6, }, // Asia/Khandyga
    {  669,  992,  211138,  7, }, // Asia/Kolkata
    {  669, 4197,  211145, 12, }, // Asia/Krasnoyarsk
    {  669, 4859,  211157, 12, }, // Asia/Kuala_Lumpur
    {  669, 5976,  211169,  5, }, // Asia/Kuching
    {  669, 5547,  211174,  7, }, // Asia/Kuwait
    {  669, 1211,  211181,  5, }, // Asia/Macau
    {  669, 4017,  211186,  7, }, // Asia/Magadan
    {  669, 7182,  211193,  6, }, // Asia/Manila
    {  669, 2554,  211199,  7, }, // Asia/Nicosia
    {  669, 8675,  211206, 16, }, // Asia/Novokuznetsk
    {  669, 4135,  211222, 13, }, // Asia/Novosibirsk
    {  669, 4229,  211235,  7, }, // Asia/Omsk
    {  669, 7145,  211242,  4, }, // Asia/Oral
    {  669, 6961,  211246,  9, }, // Asia/Phnom_Penh
    {  669, 8693,  211255, 10, }, // Asia/Pontianak
    {  669, 4214,  211265,  8, }, // Asia/Pyongyang
    {  669, 5559,    4332,  5, }, // Asia/Qatar
    {  669, 8708,  211273,  8, }, // Asia/Qostanay
    {  669, 4269,  211281,  9, }, // Asia/Qyzylorda
    {  669, 3584,  211290,  6, }, // Asia/Riyadh
    {  669, 4408,  211296,  8, }, // Asia/Sakhalin
    {  669, 7245,  211304, 10, }, // Asia/Samarkand
    {  669, 3279,  211314,  5, }, // Asia/Seoul
    {  669, 1053,  211319,  6, }, // Asia/Shanghai
    {  669, 3300,  211325,  8, }, // Asia/Singapore
    {  669, 4297,  211333, 15, }, // Asia/Srednekolymsk
    {  669, 3263,    7830,  6, }, // Asia/Taipei
    {  669, 4781,  211348,  9, }, // Asia/Tashkent
    {  669, 3915,  211357,  8, }, // Asia/Tbilisi
    {  669, 2772,  211365,  6, }, // Asia/Tehran
    {  669, 1317,  211371,  5, }, // Asia/Thimphu
    {  669, 2821,  211376,  6, }, // Asia/Tokyo
    {  669, 4572,  211382,  8, }, // Asia/Tomsk
    {  669, 1021,  211390, 10, }, // Asia/Ulaanbaatar
    {  669, 1159,  211400,  6, }, // Asia/Urumqi
    {  669, 8722,  211406, 10, }, // Asia/Ust-Nera
    {  669, 6977,  211416, 10, }, // Asia/Vientiane
    {  669, 4710,  211426, 13, }, // Asia/Vladivostok
    {  669, 4828,  211439, 11, }, // Asia/Yakutsk
    {  669, 1235,  211450,  6, }, // Asia/Yangon
    {  669, 3883,  211456, 13, }, // Asia/Yekaterinburg
    {  669, 3747,  211469,  6, }, // Asia/Yerevan
    {  669, 3663,  211475,  7, }, // Atlantic/Azores
    {  669, 5671,  211482,  7, }, // Atlantic/Bermuda
    {  669, 6628,  211489, 11, }, // Atlantic/Canary
    {  669, 3727,  211500,  7, }, // Atlantic/Cape_Verde
    {  669, 1395,     483,  5, }, // Atlantic/Faroe
    {  669, 8736,     448,  2, }, // Atlantic/Madeira
    {  669, 2748,  211507, 11, }, // Atlantic/Reykjavik
    {  669, 7272,  211518, 13, }, // Atlantic/South_Georgia
    {  669, 6821,  211531, 10, }, // Atlantic/St_Helena
    {  669, 6644,  211541,  7, }, // Atlantic/Stanley
    {  669, 1670,  211548,  8, }, // Australia/Adelaide
    {  669, 1635,  211556,  9, }, // Australia/Brisbane
    {  669, 1799,  211565, 10, }, // Australia/Broken_Hill
    {  669, 1583,  211575,  6, }, // Australia/Darwin
    {  669, 3637,  211581,  5, }, // Australia/Eucla
    {  669, 1516,  211586,  7, }, // Australia/Hobart
    {  669, 8753,  211593,  8, }, // Australia/Lindeman
    {  669, 1547,  211601, 10, }, // Australia/Lord_Howe
    {  669, 1727,  211611,  8, }, // Australia/Melbourne
    {  669, 1762,  211619,  5, }, // Australia/Perth
    {  669, 1463,  211624,  7, }, // Australia/Sydney
    {  669, 6441,  211631, 10, }, // Europe/Amsterdam
    {  669, 6216,  211641,  5, }, // Europe/Andorra
    {  669, 3620,  211646,  7, }, // Europe/Astrakhan
    {  669, 2300,  211653,  5, }, // Europe/Athens
    {  669, 6336,  211658,  8, }, // Europe/Belgrade
    {  669, 4757,  211666,  7, }, // Europe/Berlin
    {  669, 6488,  211673, 11, }, // Europe/Bratislava
    {  669, 2167,  211684,  7, }, // Europe/Brussels
    {  669, 3928,  211691, 10, }, // Europe/Bucharest
    {  669, 3806,  211701, 10, }, // Europe/Budapest
    {  669, 8772,  211711,  8, }, // Europe/Busingen
    {  669, 2583,  211719,  8, }, // Europe/Chisinau
    {  669, 6289,  211727,  9, }, // Europe/Copenhagen
    {  669, 2338,  211736,  6, }, // Europe/Dublin
    {  669, 6307,  211742,  8, }, // Europe/Gibraltar
    {  669, 8788,  211750,  8, }, // Europe/Guernsey
    {  669, 6612,  211758,  8, }, // Europe/Helsinki
    {  669, 8804,  211766, 15, }, // Europe/Isle_of_Man
    {  669, 1130,  211781,  8, }, // Europe/Istanbul
    {  669, 8823,  211789,  6, }, // Europe/Jersey
    {  669, 3979,  211795, 12, }, // Europe/Kaliningrad
    {  669, 8929,  211807,  6, }, // Europe/Kirov
    {  669, 2527,  211813,  6, }, // Europe/Kyiv
    {  669, 3213,  211819,  8, }, // Europe/Lisbon
    {  669, 6506,  211827,  8, }, // Europe/Ljubljana
    {  669, 2501,  211835,  6, }, // Europe/London
    {  669, 6365,  211841, 10, }, // Europe/Luxembourg
    {  669, 4892,  211851,  6, }, // Europe/Madrid
    {  669, 6397,  211857,  5, }, // Europe/Malta
    {  669, 6582,  211862,  8, }, // Europe/Mariehamn
    {  669, 3693,  211870,  7, }, // Europe/Minsk
    {  669, 6410,  211877,  6, }, // Europe/Monaco
    {  669, 3537,  211883,  7, }, // Europe/Moscow
    {  669, 6458,  211890,  5, }, // Europe/Oslo
    {  669, 4284,  211895,  6, }, // Europe/Paris
    {  669, 6424,  211901,  9, }, // Europe/Podgorica
    {  669, 6275,   21185,  5, }, // Europe/Prague
    {  669, 6324,  211910,  4, }, // Europe/Rome
    {  669, 4331,  211914,  6, }, // Europe/Samara
    {  669, 6470,  211920,  9, }, // Europe/San_Marino
    {  669, 6245,  211929, 17, }, // Europe/Sarajevo
    {  669, 4451,  211946,  8, }, // Europe/Saratov
    {  669, 8849,  211954, 10, }, // Europe/Simferopol
    {  669, 6383,  211964,  8, }, // Europe/Skopje
    {  669, 6599,  211972,  6, }, // Europe/Sofia
    {  669, 6523,  211978, 10, }, // Europe/Stockholm
    {  669, 8867,  211988,  5, }, // Europe/Tallinn
    {  669, 6202,  211993,  6, }, // Europe/Tirane
    {  669, 8882,  211999, 13, }, // Europe/Ulyanovsk
    {  669, 6352,  212012,  5, }, // Europe/Vaduz
    {  669, 6567,  212017,  8, }, // Europe/Vatican
    {  669, 6231,  212025,  5, }, // Europe/Vienna
    {  669, 8899,  212030,  9, }, // Europe/Vilnius
    {  669, 4727,  212039, 10, }, // Europe/Volgograd
    {  669, 3190,  212049,  9, }, // Europe/Warsaw
    {  669, 6261,  212058,  7, }, // Europe/Zagreb
    {  669, 6540,  212065,  6, }, // Europe/Zurich
    {  669, 5078,  212071, 11, }, // Indian/Antananarivo
    {  669, 6947,  212082,  4, }, // Indian/Chagos
    {  669, 6046,  212086,  9, }, // Indian/Christmas
    {  669, 6063,  212095,  7, }, // Indian/Cocos
    {  669, 5029,  212102,  6, }, // Indian/Comoro
    {  669, 6661,  212108,  8, }, // Indian/Kerguelen
    {  669, 7260,  212116,  4, }, // Indian/Mahe
    {  669, 7050,  212120,  7, }, // Indian/Maldives
    {  669, 4069,  212127,  7, }, // Indian/Mauritius
    {  669, 5098,  212134,  6, }, // Indian/Mayotte
    {  669, 7211,  212140,  6, }, // Indian/Reunion
    {  669, 4422,  124493,  5, }, // Pacific/Apia
    {  669,  932,  212146,  6, }, // Pacific/Auckland
    {  669, 3706,  212152,  9, }, // Pacific/Bougainville
    {  669, 3015,  212161,  7, }, // Pacific/Chatham
    {  669, 3157,     554,  5, }, // Pacific/Chuuk
    {  669, 2237,  212168, 17, }, // Pacific/Easter
    {  669, 7360,   27148,  5, }, // Pacific/Efate
    {  669, 7327,  212185,  7, }, // Pacific/Fakaofo
    {  669, 7343,  212192,  8, }, // Pacific/Funafuti
    {  669, 6678,  212200,  9, }, // Pacific/Galapagos
    {  669, 6696,  212209,  7, }, // Pacific/Gambier
    {  669, 3822,  212216, 12, }, // Pacific/Guadalcanal
    {  669, 6018,  212228,  5, }, // Pacific/Guam
    {  669, 2723,  212233,  8, }, // Pacific/Honolulu
    {  669, 3049,  212241,  9, }, // Pacific/Kanton
    {  669, 3998,  212250, 10, }, // Pacific/Kiritimati
    {  669, 7019,  212260,  8, }, // Pacific/Kosrae
    {  669, 2842,  212268,  9, }, // Pacific/Kwajalein
    {  669, 4877,  212277,  6, }, // Pacific/Majuro
    {  669, 4051,  212283, 11, }, // Pacific/Marquesas
    {  669, 8914,  212294,  8, }, // Pacific/Midway
    {  669, 7084,  212302,  6, }, // Pacific/Nauru
    {  669, 7132,  212308,  4, }, // Pacific/Niue
    {  669, 4168,  212312,  7, }, // Pacific/Norfolk
    {  669, 7098,  212319,  6, }, // Pacific/Noumea
    {  669, 7155,  212325,  5, }, // Pacific/Palau
    {  669, 7194,  212330,  8, }, // Pacific/Pitcairn
    {  669, 3096,     586,  7, }, // Pacific/Pohnpei
    {  669, 4807,  212338, 12, }, // Pacific/Port_Moresby
    {  669, 6076,  212350,  8, }, // Pacific/Rarotonga
    {  669, 6031,  212358,  5, }, // Pacific/Saipan
    {  669, 7312,  212363,  5, }, // Pacific/Tahiti
    {  669, 6712,  212368,  7, }, // Pacific/Tarawa
    {  669, 4583,  212375,  9, }, // Pacific/Tongatapu
    {  669, 7392,  212384,  4, }, // Pacific/Wake
    {  669, 7405,  212388,  6, }, // Pacific/Wallis
    {  670, 6788,  212394,  7, }, // Africa/Abidjan Nheengatu/Latin/Brazil
    {  670, 6760,  212401,  5, }, // Africa/Accra
    {  670, 5059,  212406, 11, }, // Africa/Addis_Ababa
    {  670, 7420,  212417,  4, }, // Africa/Algiers
    {  670,   14,  212421,  7, }, // Africa/Asmara
    {  670,   44,   98270,  6, }, // Africa/Bamako
    {  670, 5257,  212428,  4, }, // Africa/Bangui
    {  670, 6746,  212432,  4, }, // Africa/Banjul
    {  670, 7435,    6313,  5, }, // Africa/Bissau
    {  670, 4957,  212436,  8, }, // Africa/Blantyre
    {  670, 5287,  212444, 10, }, // Africa/Brazzaville
    {  670, 4922,  212454,  8, }, // Africa/Bujumbura
    {  670, 2320,  212462,  5, }, // Africa/Cairo
    {  670, 4117,  212467, 12, }, // Africa/Casablanca
    {  670, 7449,   21349,  5, }, // Africa/Ceuta
    {  670, 6773,  212479,  8, }, // Africa/Conakry
    {  670, 5130,  212487, 12, }, // Africa/Dar_es_Salaam
    {  670, 5043,  212499,  8, }, // Africa/Djibouti
    {  670, 5243,  212507,  6, }, // Africa/Douala
    {  670, 5166,  212513,  6, }, // Africa/El_Aaiun
    {  670, 6853,  212519,  6, }, // Africa/Freetown
    {  670, 4906,  212525,  8, }, // Africa/Gaborone
    {  670, 4479,  212533, 11, }, // Africa/Johannesburg
    {  670, 4499,  212544,  4, }, // Africa/Juba
    {  670, 5151,  212548,  6, }, // Africa/Kampala
    {  670, 4524,  212554,  4, }, // Africa/Khartoum
    {  670, 4973,  209402,  6, }, // Africa/Kigali
    {  670, 5306,  212558,  6, }, // Africa/Kinshasa
    {  670, 4744,  212564,  8, }, // Africa/Lagos
    {  670, 5336,  212572, 10, }, // Africa/Libreville
    {  670, 6869,  212582,  4, }, // Africa/Lome
    {  670, 5211,  212586,  5, }, // Africa/Luanda
    {  670, 4939,  212591,  8, }, // Africa/Lubumbashi
    {  670, 5322,   98421,  6, }, // Africa/Malabo
    {  670, 4987,  122189,  6, }, // Africa/Maputo
    {  670, 5182,  212599,  7, }, // Africa/Mbabane
    {  670, 5113,  212606, 10, }, // Africa/Mogadishu
    {  670, 7034,  212616,  8, }, // Africa/Monrovia
    {  670, 3853,  212624,  7, }, // Africa/Nairobi
    {  670, 5271,  212631, 11, }, // Africa/Ndjamena
    {  670, 5354,   26528,  6, }, // Africa/Niamey
    {  670, 6803,  212642, 10, }, // Africa/Nouakchott
    {  670, 6727,  212652,  9, }, // Africa/Ouagadougou
    {  670, 5225,  212661, 16, }, // Africa/Porto-Novo
    {  670, 4435,  212677,  7, }, // Africa/Sao_Tome
    {  670, 2866,  212684,  8, }, // Africa/Tripoli
    {  670, 6554,  212692,  8, }, // Africa/Tunis
    {  670, 4152,  212700,  7, }, // Africa/Windhoek
    {  670,  132,   97943,  5, }, // America/Adak
    {  670, 3346,  212707,  7, }, // America/Anchorage
    {  670, 5607,  212714,  5, }, // America/Anguilla
    {  670, 5624,  212719,  7, }, // America/Antigua
    {  670, 4554,  212726, 10, }, // America/Araguaina
    {  670,  166,  212736, 11, }, // America/Argentina/Buenos_Aires
    {  670,   91,  212747,  8, }, // America/Argentina/Catamarca
    {  670,  270,  212755,  6, }, // America/Argentina/Cordoba
    {  670,  441,  212761,  5, }, // America/Argentina/Jujuy
    {  670, 7462,  212766,  8, }, // America/Argentina/La_Rioja
    {  670,  565,  212774,  6, }, // America/Argentina/Mendoza
    {  670, 7489,  212780, 17, }, // America/Argentina/Rio_Gallegos
    {  670, 7520,  212797,  5, }, // America/Argentina/Salta
    {  670, 7544,  212802,  7, }, // America/Argentina/San_Juan
    {  670, 5580,  212809,  6, }, // America/Argentina/San_Luis
    {  670, 7571,  212815,  6, }, // America/Argentina/Tucuman
    {  670, 7597,  212821,  7, }, // America/Argentina/Ushuaia
    {  670, 4252,  212828,  7, }, // America/Asuncion
    {  670,  237,  212835,  7, }, // America/Atikokan
    {  670, 3679,  212842,  5, }, // America/Bahia
    {  670, 7623,  212847, 16, }, // America/Bahia_Banderas
    {  670, 5654,  212863,  8, }, // America/Barbados
    {  670, 7646,  212871,  4, }, // America/Belem
    {  670, 5407,  212875,  6, }, // America/Belize
    {  670, 7660,  212881, 15, }, // America/Blanc-Sablon
    {  670, 7681,  212896, 10, }, // America/Boa_Vista
    {  670, 4361,  212906,  6, }, // America/Bogota
    {  670, 7699,  212912,  6, }, // America/Boise
    {  670, 7713,  212918, 14, }, // America/Cambridge_Bay
    {  670, 7735,  212932,  9, }, // America/Campo_Grande
    {  670, 3868,  212941,  4, }, // America/Cancun
    {  670, 4694,    4910,  6, }, // America/Caracas
    {  670, 4345,  212945,  6, }, // America/Cayenne
    {  670, 5496,  212951,  5, }, // America/Cayman
    {  670, 2260,   97992,  6, }, // America/Chicago
    {  670, 7756,  212956,  6, }, // America/Chihuahua
    {  670, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  670, 5422,  212962,  9, }, // America/Costa_Rica
    {  670, 7796,  212971,  7, }, // America/Creston
    {  670, 3791,   21631,  6, }, // America/Cuiaba
    {  670, 5723,   98011,  7, }, // America/Curacao
    {  670, 7812,  212978, 13, }, // America/Danmarkshavn
    {  670, 7833,  212991,  5, }, // America/Dawson
    {  670, 7848,  212996, 12, }, // America/Dawson_Creek
    {  670,  805,  213008,  5, }, // America/Denver
    {  670, 5739,  213013,  8, }, // America/Dominica
    {  670,  893,  213021,  7, }, // America/Edmonton
    {  670, 7869,   26580,  8, }, // America/Eirunepe
    {  670, 5441,  213028,  9, }, // America/El_Salvador
    {  670, 7886,  213037, 10, }, // America/Fort_Nelson
    {  670, 7906,  213047,  8, }, // America/Fortaleza
    {  670, 7924,  213055, 19, }, // America/Glace_Bay
    {  670, 6881,  213074, 15, }, // America/Goose_Bay
    {  670, 4612,  213089, 11, }, // America/Grand_Turk
    {  670, 5770,  213100,  8, }, // America/Grenada
    {  670, 5786,  213108,  9, }, // America/Guadeloupe
    {  670, 3760,  213117,  9, }, // America/Guatemala
    {  670, 4841,  213126,  8, }, // America/Guayaquil
    {  670, 6932,  213134,  6, }, // America/Guyana
    {  670, 2281,  157174,  6, }, // America/Havana
    {  670, 7942,  213140,  8, }, // America/Hermosillo
    {  670,  348,  213148, 10, }, // America/Indiana/Indianapolis
    {  670,  481,  213158, 12, }, // America/Indiana/Knox
    {  670, 7961,  213170, 14, }, // America/Indiana/Marengo
    {  670, 7985,  213184, 20, }, // America/Indiana/Petersburg
    {  670, 8012,  213204, 22, }, // America/Indiana/Tell_City
    {  670, 8038,  213226, 13, }, // America/Indiana/Vevay
    {  670, 8060,  213239, 18, }, // America/Indiana/Vincennes
    {  670, 8086,  213257, 16, }, // America/Indiana/Winamac
    {  670, 8110,  213273,  7, }, // America/Inuvik
    {  670,  660,  213280,  8, }, // America/Iqaluit
    {  670, 2799,  213288,  7, }, // America/Jamaica
    {  670, 5380,  213295,  6, }, // America/Juneau
    {  670,  521,  213301, 10, }, // America/Kentucky/Louisville
    {  670, 8125,  213311, 16, }, // America/Kentucky/Monticello
    {  670, 5704,  213327, 10, }, // America/Kralendijk
    {  670, 7169,  213337,  4, }, // America/Lima
    {  670, 5932,  213341, 26, }, // America/Lower_Princes
    {  670, 8153,  213367,  6, }, // America/Maceio
    {  670, 8168,  213373,  8, }, // America/Managua
    {  670, 1908,  122916,  5, }, // America/Manaus
    {  670, 5897,  213381,  8, }, // America/Marigot
    {  670, 5805,  213389,  8, }, // America/Martinique
    {  670, 8184,  213397, 11, }, // America/Matamoros
    {  670, 2917,  213408,  8, }, // America/Mazatlan
    {  670, 8202,    7118,  8, }, // America/Menominee
    {  670, 8220,     229,  6, }, // America/Merida
    {  670, 8235,  213416, 12, }, // America/Metlakatla
    {  670, 2949,  213428, 13, }, // America/Mexico_City
    {  670, 4391,  213441,  6, }, // America/Miquelon
    {  670, 8254,  213447,  6, }, // America/Moncton
    {  670, 8270,  213453,  7, }, // America/Monterrey
    {  670, 4098,  213460, 10, }, // America/Montevideo
    {  670, 5824,  213470, 10, }, // America/Montserrat
    {  670, 5481,  125812,  5, }, // America/Nassau
    {  670, 2379,  213480, 12, }, // America/New_York
    {  670, 8288,  213492,  4, }, // America/Nome
    {  670, 1850,  213496, 13, }, // America/Noronha
    {  670, 8301,  213509, 21, }, // America/North_Dakota/Beulah
    {  670, 8329,  213530, 21, }, // America/North_Dakota/Center
    {  670, 8357,  213551, 28, }, // America/North_Dakota/New_Salem
    {  670,  393,  213579,  4, }, // America/Nuuk
    {  670, 8388,  213583,  7, }, // America/Ojinaga
    {  670, 2356,  213590,  6, }, // America/Panama
    {  670, 6173,   98830, 10, }, // America/Paramaribo
    {  670, 2973,  213596, 10, }, // America/Phoenix
    {  670, 3945,  213606, 18, }, // America/Port-au-Prince
    {  670, 5954,  213624, 17, }, // America/Port_of_Spain
    {  670, 8404,  213641, 20, }, // America/Porto_Velho
    {  670, 5843,  213661, 14, }, // America/Puerto_Rico
    {  670, 4030,  213675, 14, }, // America/Punta_Arenas
    {  670, 8424,  213689, 13, }, // America/Rankin_Inlet
    {  670, 8445,  123152,  6, }, // America/Recife
    {  670, 1995,  213702,  6, }, // America/Regina
    {  670, 8460,  213708,  8, }, // America/Resolute
    {  670,  695,  213716, 15, }, // America/Rio_Branco
    {  670, 8477,  213731,  6, }, // America/Santarem
    {  670, 2201,  213737,  8, }, // America/Santiago
    {  670, 6111,  213745, 11, }, // America/Santo_Domingo
    {  670, 1878,  213756,  8, }, // America/Sao_Paulo
    {  670, 6899,  213764, 11, }, // America/Scoresbysund
    {  670, 8494,  213775,  6, }, // America/Sitka
    {  670, 8508,  213781, 12, }, // America/St_Barthelemy
    {  670, 2061,  213793,  9, }, // America/St_Johns
    {  670, 5863,  213802, 12, }, // America/St_Kitts
    {  670, 5880,  213814, 11, }, // America/St_Lucia
    {  670,  855,  213825,  9, }, // America/St_Thomas
    {  670, 5913,  213834,  9, }, // America/St_Vincent
    {  670, 8530,  213843, 13, }, // America/Swift_Current
    {  670, 5461,  213856, 11, }, // America/Tegucigalpa
    {  670, 5756,  213867,  6, }, // America/Thule
    {  670,  313,  213873,  8, }, // America/Tijuana
    {  670,  608,  213881,  6, }, // America/Toronto
    {  670, 5688,  213887,  6, }, // America/Tortola
    {  670, 2093,  213893,  8, }, // America/Vancouver
    {  670, 2144,  213901, 15, }, // America/Whitehorse
    {  670,  734,  213916,  8, }, // America/Winnipeg
    {  670, 8552,  207174,  8, }, // America/Yakutat
    {  670, 6001,  213924,  5, }, // Antarctica/Casey
    {  670, 6094,  213929,  4, }, // Antarctica/Davis
    {  670, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  670, 8568,  213933,  9, }, // Antarctica/Macquarie
    {  670, 7066,  213942,  5, }, // Antarctica/Mawson
    {  670, 7113,  213947,  8, }, // Antarctica/McMurdo
    {  670, 8589,  213955,  7, }, // Antarctica/Palmer
    {  670, 7226,    5583,  6, }, // Antarctica/Rothera
    {  670, 7295,  176323,  6, }, // Antarctica/Syowa
    {  670, 7374,  213962,  8, }, // Antarctica/Vostok
    {  670, 1429,  213970, 18, }, // Arctic/Longyearbyen
    {  670, 5570,  213988,  3, }, // Asia/Aden
    {  670, 5395,  213991,  6, }, // Asia/Almaty
    {  670, 3968,  158200,  3, }, // Asia/Amman
    {  670, 5511,    7600,  5, }, // Asia/Anadyr
    {  670, 5523,  210937,  6, }, // Asia/Aqtau
    {  670, 5368,  213997,  7, }, // Asia/Aqtobe
    {  670,  964,  214004,  9, }, // Asia/Ashgabat
    {  670, 8624,    7612,  6, }, // Asia/Atyrau
    {  670, 3607,  214013,  5, }, // Asia/Baghdad
    {  670, 5534,  214018,  5, }, // Asia/Bahrain
    {  670, 4466,  214023,  8, }, // Asia/Bangkok
    {  670, 3571,  123480,  6, }, // Asia/Barnaul
    {  670, 4086,  158211,  7, }, // Asia/Beirut
    {  670, 3778,  214031,  8, }, // Asia/Bishkek
    {  670, 5989,  214039,  7, }, // Asia/Brunei
    {  670, 4601,  214046,  4, }, // Asia/Chita
    {  670, 4511,  214050,  6, }, // Asia/Colombo
    {  670, 4540,  214056,  8, }, // Asia/Damascus
    {  670, 1093,    4717,  4, }, // Asia/Dhaka
    {  670, 6192,  214064,  4, }, // Asia/Dili
    {  670, 6159,  214068,  6, }, // Asia/Dushanbe
    {  670, 8636,  214074,  9, }, // Asia/Famagusta
    {  670, 8651,   63961,  4, }, // Asia/Gaza
    {  670, 4795,  214083,  6, }, // Asia/Hebron
    {  670, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  670, 2704,  214089,  7, }, // Asia/Hong_Kong
    {  670, 4771,  214096,  6, }, // Asia/Hovd
    {  670, 4184,  214102, 10, }, // Asia/Irkutsk
    {  670, 7006,  214112,  6, }, // Asia/Jakarta
    {  670, 6992,  214118,  8, }, // Asia/Jayapura
    {  670, 1290,  214126,  9, }, // Asia/Jerusalem
    {  670, 3560,   92708,  4, }, // Asia/Kabul
    {  670, 4316,  214135,  8, }, // Asia/Kamchatka
    {  670, 4239,  214143,  6, }, // Asia/Karachi
    {  670, 1185,  214149,  8, }, // Asia/Kathmandu
    {  670, 8661,  214157,  6, }, // Asia/Khandyga
    {  670,  992,  214163,  7, }, // Asia/Kolkata
    {  670, 4197,  214170, 16, }, // Asia/Krasnoyarsk
    {  670, 4859,  214186, 12, }, // Asia/Kuala_Lumpur
    {  670, 5976,  214198,  6, }, // Asia/Kuching
    {  670, 5547,  207653,  7, }, // Asia/Kuwait
    {  670, 1211,   33537,  5, }, // Asia/Macau
    {  670, 4017,  214204,  6, }, // Asia/Magadan
    {  670, 7182,  135023,  6, }, // Asia/Manila
    {  670, 6920,  214210,  8, }, // Asia/Muscat
    {  670, 2554,   22510,  8, }, // Asia/Nicosia
    {  670, 8675,  214218, 19, }, // Asia/Novokuznetsk
    {  670, 4135,  214237, 17, }, // Asia/Novosibirsk
    {  670, 4229,  214254,  7, }, // Asia/Omsk
    {  670, 7145,  214261,  5, }, // Asia/Oral
    {  670, 6961,  214266,  8, }, // Asia/Phnom_Penh
    {  670, 8693,  214274,  9, }, // Asia/Pontianak
    {  670, 4214,  214283,  8, }, // Asia/Pyongyang
    {  670, 5559,  214291,  6, }, // Asia/Qatar
    {  670, 8708,  214297,  9, }, // Asia/Qostanay
    {  670, 4269,  214306,  8, }, // Asia/Qyzylorda
    {  670, 3584,  158277,  5, }, // Asia/Riyadh
    {  670, 4408,  214314,  8, }, // Asia/Sakhalin
    {  670, 7245,  214322,  8, }, // Asia/Samarkand
    {  670, 3279,   20896,  3, }, // Asia/Seoul
    {  670, 1053,  214330,  5, }, // Asia/Shanghai
    {  670, 3300,  211325,  8, }, // Asia/Singapore
    {  670, 4297,  214335, 18, }, // Asia/Srednekolymsk
    {  670, 4781,  214353,  8, }, // Asia/Tashkent
    {  670, 3915,  176739,  8, }, // Asia/Tbilisi
    {  670, 2772,  214361,  6, }, // Asia/Tehran
    {  670, 1317,  214367,  4, }, // Asia/Thimphu
    {  670, 2821,  214371,  6, }, // Asia/Tokyo
    {  670, 4572,  214377,  8, }, // Asia/Tomsk
    {  670, 1021,  214385, 10, }, // Asia/Ulaanbaatar
    {  670, 1159,  214395,  5, }, // Asia/Urumqi
    {  670, 8722,  214400, 10, }, // Asia/Ust-Nera
    {  670, 6977,  214410,  8, }, // Asia/Vientiane
    {  670, 4710,  214418, 14, }, // Asia/Vladivostok
    {  670, 4828,  214432, 10, }, // Asia/Yakutsk
    {  670, 1235,  214442,  4, }, // Asia/Yangon
    {  670, 3883,  214446, 11, }, // Asia/Yekaterinburg
    {  670, 3747,  214457,  6, }, // Asia/Yerevan
    {  670, 3663,  214463,  9, }, // Atlantic/Azores
    {  670, 5671,  214472,  6, }, // Atlantic/Bermuda
    {  670, 6628,  214478, 12, }, // Atlantic/Canary
    {  670, 3727,  214490, 12, }, // Atlantic/Cape_Verde
    {  670, 1395,  214502, 18, }, // Atlantic/Faroe
    {  670, 2748,  214520, 11, }, // Atlantic/Reykjavik
    {  670, 7272,  214531, 14, }, // Atlantic/South_Georgia
    {  670, 6821,  214545, 10, }, // Atlantic/St_Helena
    {  670, 6644,  214555,  8, }, // Atlantic/Stanley
    {  670, 1670,  214563,  8, }, // Australia/Adelaide
    {  670, 1635,  214571,  8, }, // Australia/Brisbane
    {  670, 1799,  214579, 17, }, // Australia/Broken_Hill
    {  670, 1583,  214596,  6, }, // Australia/Darwin
    {  670, 3637,  214602,  6, }, // Australia/Eucla
    {  670, 1516,  214608,  6, }, // Australia/Hobart
    {  670, 8753,  214614,  6, }, // Australia/Lindeman
    {  670, 1547,  214620, 11, }, // Australia/Lord_Howe
    {  670, 1727,  214631,  7, }, // Australia/Melbourne
    {  670, 1762,  214638,  6, }, // Australia/Perth
    {  670, 1463,  214644,  7, }, // Australia/Sydney
    {  670, 6441,  214651,  9, }, // Europe/Amsterdam
    {  670, 6216,  214660,  5, }, // Europe/Andorra
    {  670, 3620,  214665,  9, }, // Europe/Astrakhan
    {  670, 2300,  214674,  9, }, // Europe/Athens
    {  670, 6336,  214683,  9, }, // Europe/Belgrade
    {  670, 4757,  158374,  6, }, // Europe/Berlin
    {  670, 6488,  214692, 12, }, // Europe/Bratislava
    {  670, 2167,  214704,  8, }, // Europe/Brussels
    {  670, 3928,  214712,  8, }, // Europe/Bucharest
    {  670, 3806,  214720,  8, }, // Europe/Budapest
    {  670, 8772,  214728,  8, }, // Europe/Busingen
    {  670, 2583,  214736,  7, }, // Europe/Chisinau
    {  670, 6289,  214743,  8, }, // Europe/Copenhagen
    {  670, 2338,  214751,  6, }, // Europe/Dublin
    {  670, 6307,  214757,  9, }, // Europe/Gibraltar
    {  670, 8788,  214766,  9, }, // Europe/Guernsey
    {  670, 6612,  214775,  7, }, // Europe/Helsinki
    {  670, 8804,  214782, 10, }, // Europe/Isle_of_Man
    {  670, 1130,   57447,  8, }, // Europe/Istanbul
    {  670, 8823,  214792,  6, }, // Europe/Jersey
    {  670, 3979,  214798, 12, }, // Europe/Kaliningrad
    {  670, 8929,  214810,  6, }, // Europe/Kirov
    {  670, 2527,  214816,  6, }, // Europe/Kyiv
    {  670, 3213,   21137,  6, }, // Europe/Lisbon
    {  670, 6506,  214822, 10, }, // Europe/Ljubljana
    {  670, 2501,   21152,  7, }, // Europe/London
    {  670, 6365,  214832,  8, }, // Europe/Luxembourg
    {  670, 4892,   27075,  5, }, // Europe/Madrid
    {  670, 6397,  167272,  5, }, // Europe/Malta
    {  670, 6582,  214840,  7, }, // Europe/Mariehamn
    {  670, 3693,  214847,  6, }, // Europe/Minsk
    {  670, 6410,   99294,  6, }, // Europe/Monaco
    {  670, 3537,    4522,  6, }, // Europe/Moscow
    {  670, 6458,  214853,  5, }, // Europe/Oslo
    {  670, 6424,  214858, 10, }, // Europe/Podgorica
    {  670, 6275,   21185,  5, }, // Europe/Prague
    {  670, 6324,    6064,  4, }, // Europe/Rome
    {  670, 6470,  214868,  9, }, // Europe/San_Marino
    {  670, 6245,  214877,  8, }, // Europe/Sarajevo
    {  670, 4451,  214885,  8, }, // Europe/Saratov
    {  670, 8849,  214893,  8, }, // Europe/Simferopol
    {  670, 6383,  214901,  9, }, // Europe/Skopje
    {  670, 6599,  214910,  6, }, // Europe/Sofia
    {  670, 6523,   74615,  9, }, // Europe/Stockholm
    {  670, 8867,  211988,  4, }, // Europe/Tallinn
    {  670, 6202,   21213,  6, }, // Europe/Tirane
    {  670, 8882,  214916, 10, }, // Europe/Ulyanovsk
    {  670, 6352,  214926,  6, }, // Europe/Vaduz
    {  670, 6567,  214932,  8, }, // Europe/Vatican
    {  670, 6231,  214940,  6, }, // Europe/Vienna
    {  670, 8899,  214946,  9, }, // Europe/Vilnius
    {  670, 4727,  214955, 10, }, // Europe/Volgograd
    {  670, 3190,  214965, 10, }, // Europe/Warsaw
    {  670, 6261,  214975,  8, }, // Europe/Zagreb
    {  670, 6540,  214983,  6, }, // Europe/Zurich
    {  670, 5078,  214989, 10, }, // Indian/Antananarivo
    {  670, 6947,  214999,  8, }, // Indian/Chagos
    {  670, 6046,  215007, 12, }, // Indian/Christmas
    {  670, 6063,  215019,  8, }, // Indian/Cocos
    {  670, 5029,  215027, 10, }, // Indian/Comoro
    {  670, 6661,  215037,  7, }, // Indian/Kerguelen
    {  670, 7260,  215044,  3, }, // Indian/Mahe
    {  670, 7050,  215047, 11, }, // Indian/Maldives
    {  670, 4069,  215058,  9, }, // Indian/Mauritius
    {  670, 5098,  215067,  6, }, // Indian/Mayotte
    {  670, 7211,  215073, 13, }, // Indian/Reunion
    {  670, 4422,  215086,  5, }, // Pacific/Apia
    {  670,  932,  215091,  7, }, // Pacific/Auckland
    {  670, 3706,  215098,  9, }, // Pacific/Bougainville
    {  670, 3015,  215107,  6, }, // Pacific/Chatham
    {  670, 3157,     554,  5, }, // Pacific/Chuuk
    {  670, 2237,  215113, 16, }, // Pacific/Easter
    {  670, 7360,  215129,  5, }, // Pacific/Efate
    {  670, 7327,  215134,  7, }, // Pacific/Fakaofo
    {  670, 3902,  148230,  4, }, // Pacific/Fiji
    {  670, 6678,  215141, 12, }, // Pacific/Galapagos
    {  670, 6696,  215153,  7, }, // Pacific/Gambier
    {  670, 3822,  215160, 11, }, // Pacific/Guadalcanal
    {  670, 6018,  215171,  4, }, // Pacific/Guam
    {  670, 2723,  215175,  8, }, // Pacific/Honolulu
    {  670, 3049,  215183,  7, }, // Pacific/Kanton
    {  670, 7019,  215190,  7, }, // Pacific/Kosrae
    {  670, 2842,  215197,  8, }, // Pacific/Kwajalein
    {  670, 4877,  215205,  6, }, // Pacific/Majuro
    {  670, 4051,  215211, 10, }, // Pacific/Marquesas
    {  670, 8914,  215221,  7, }, // Pacific/Midway
    {  670, 7132,  215228,  4, }, // Pacific/Niue
    {  670, 4168,  215232,  8, }, // Pacific/Norfolk
    {  670, 7098,  215240,  6, }, // Pacific/Noumea
    {  670, 3126,  215246,  9, }, // Pacific/Pago_Pago
    {  670, 7155,  215255,  5, }, // Pacific/Palau
    {  670, 3096,     586,  7, }, // Pacific/Pohnpei
    {  670, 4807,  215260, 17, }, // Pacific/Port_Moresby
    {  670, 6076,  215277,  8, }, // Pacific/Rarotonga
    {  670, 6031,  215285,  5, }, // Pacific/Saipan
    {  670, 7312,  124697,  5, }, // Pacific/Tahiti
    {  670, 4583,  215290,  8, }, // Pacific/Tongatapu
    {  670, 7405,  215298,  8, }, // Pacific/Wallis
    {  671, 5863,  215306, 12, }, // America/St_Kitts Nheengatu/Latin/Colombia
    {  672, 5863,  215306, 12, }, // America/St_Kitts Nheengatu/Latin/Venezuela
    {  673,   14,       0,  6, }, // Africa/Asmara Haryanvi/Devanagari/India
    {  673, 4435,       6,  8, }, // Africa/Sao_Tome
    {  673, 4252,      14,  8, }, // America/Asuncion
    {  673,  237,      22,  8, }, // America/Atikokan
    {  673, 7623,      30, 17, }, // America/Bahia_Banderas
    {  673, 3868,      47,  6, }, // America/Cancun
    {  673, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  673, 5723,      66,  7, }, // America/Curacao
    {  673,  481,      73, 13, }, // America/Indiana/Knox
    {  673, 7961,      86, 16, }, // America/Indiana/Marengo
    {  673, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  673, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  673, 8038,     139, 14, }, // America/Indiana/Vevay
    {  673, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  673, 8086,     171, 16, }, // America/Indiana/Winamac
    {  673, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  673, 5932,     207, 22, }, // America/Lower_Princes
    {  673, 8220,     229,  6, }, // America/Merida
    {  673, 1850,     235, 19, }, // America/Noronha
    {  673, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  673, 8329,     274, 20, }, // America/North_Dakota/Center
    {  673, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  673,  393,     317,  4, }, // America/Nuuk
    {  673, 6899,     321, 16, }, // America/Scoresbysund
    {  673, 8508,     337, 14, }, // America/St_Barthelemy
    {  673, 2061,     351, 10, }, // America/St_Johns
    {  673, 5863,     361,  9, }, // America/St_Kitts
    {  673, 5880,     370,  9, }, // America/St_Lucia
    {  673,  855,     379, 10, }, // America/St_Thomas
    {  673, 5913,     389, 11, }, // America/St_Vincent
    {  673, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  673, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  673, 1185,     448,  9, }, // Asia/Kathmandu
    {  673,  992,     457,  7, }, // Asia/Kolkata
    {  673, 1211,     464,  5, }, // Asia/Macau
    {  673, 1235,     477,  6, }, // Asia/Yangon
    {  673, 1395,     483,  5, }, // Atlantic/Faroe
    {  673, 6821,     488, 10, }, // Atlantic/St_Helena
    {  673, 2527,     514,  4, }, // Europe/Kyiv
    {  673, 7211,     547,  7, }, // Indian/Reunion
    {  673, 3157,     554,  5, }, // Pacific/Chuuk
    {  673, 3096,     586,  7, }, // Pacific/Pohnpei
    {  674, 7420,   58555,  6, }, // Africa/Algiers Northern Frisian/Latin/Germany
    {  674,   14,       0,  6, }, // Africa/Asmara
    {  674, 2320,    4690,  5, }, // Africa/Cairo
    {  674, 4435,       6,  8, }, // Africa/Sao_Tome
    {  674, 2866,   57756,  8, }, // Africa/Tripoli
    {  674, 5624,  215318,  8, }, // America/Antigua
    {  674, 5640,  215326,  6, }, // America/Aruba
    {  674, 4252,      14,  8, }, // America/Asuncion
    {  674,  237,      22,  8, }, // America/Atikokan
    {  674, 7623,   64671, 17, }, // America/Bahia_Banderas
    {  674, 5654,  215332,  9, }, // America/Barbados
    {  674, 3868,      47,  6, }, // America/Cancun
    {  674, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  674, 5723,      66,  7, }, // America/Curacao
    {  674, 5739,  215341,  9, }, // America/Dominica
    {  674, 5770,  215350,  8, }, // America/Grenada
    {  674, 3760,  215358, 10, }, // America/Guatemala
    {  674, 6932,  175430,  7, }, // America/Guyana
    {  674,  481,      73, 13, }, // America/Indiana/Knox
    {  674, 7961,      86, 16, }, // America/Indiana/Marengo
    {  674, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  674, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  674, 8038,     139, 14, }, // America/Indiana/Vevay
    {  674, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  674, 8086,     171, 16, }, // America/Indiana/Winamac
    {  674, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  674, 5932,     207, 22, }, // America/Lower_Princes
    {  674, 8220,     229,  6, }, // America/Merida
    {  674, 2949,  215368, 13, }, // America/Mexico_City
    {  674, 1850,     235, 19, }, // America/Noronha
    {  674, 8301,  215381, 20, }, // America/North_Dakota/Beulah
    {  674, 8329,  215401, 20, }, // America/North_Dakota/Center
    {  674, 8357,  215421, 23, }, // America/North_Dakota/New_Salem
    {  674,  393,     317,  4, }, // America/Nuuk
    {  674, 5843,   22058, 11, }, // America/Puerto_Rico
    {  674, 6899,     321, 16, }, // America/Scoresbysund
    {  674, 8508,     337, 14, }, // America/St_Barthelemy
    {  674, 2061,     351, 10, }, // America/St_Johns
    {  674, 5863,     361,  9, }, // America/St_Kitts
    {  674, 5880,     370,  9, }, // America/St_Lucia
    {  674,  855,     379, 10, }, // America/St_Thomas
    {  674, 5913,     389, 11, }, // America/St_Vincent
    {  674, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  674, 7374,    4147,  6, }, // Antarctica/Vostok
    {  674,  964,  215444,  9, }, // Asia/Ashgabat
    {  674, 3607,    4162,  6, }, // Asia/Baghdad
    {  674, 5534,   73071,  8, }, // Asia/Bahrain
    {  674, 3778,   57220,  8, }, // Asia/Bishkek
    {  674, 4540,    4201,  8, }, // Asia/Damascus
    {  674, 6159,   57228,  9, }, // Asia/Dushanbe
    {  674, 1259,  215453, 17, }, // Asia/Ho_Chi_Minh
    {  674, 2704,    4231,  8, }, // Asia/Hong_Kong
    {  674, 4771,   78174,  5, }, // Asia/Hovd
    {  674, 4316,   78179, 11, }, // Asia/Kamchatka
    {  674, 4239,   78190,  9, }, // Asia/Karachi
    {  674, 1185,     448,  9, }, // Asia/Kathmandu
    {  674,  992,   26864,  7, }, // Asia/Kolkata
    {  674, 4197,    4288, 11, }, // Asia/Krasnoyarsk
    {  674, 5976,  215470,  9, }, // Asia/Kuching
    {  674, 1211,     464,  5, }, // Asia/Macau
    {  674, 6920,   26882,  6, }, // Asia/Muscat
    {  674, 2554,    5738,  7, }, // Asia/Nicosia
    {  674, 8675,  215479, 12, }, // Asia/Novokuznetsk
    {  674, 4135,   78211, 11, }, // Asia/Novosibirsk
    {  674, 4214,   78222,  9, }, // Asia/Pyongyang
    {  674, 5559,  215491,  6, }, // Asia/Qatar
    {  674, 3584,    7787,  4, }, // Asia/Riyadh
    {  674, 4408,   58084,  8, }, // Asia/Sakhalin
    {  674, 1053,   57309,  9, }, // Asia/Shanghai
    {  674, 3300,  215497,  9, }, // Asia/Singapore
    {  674, 3263,  215506,  6, }, // Asia/Taipei
    {  674, 4781,   57328,  9, }, // Asia/Tashkent
    {  674, 3915,   78237,  6, }, // Asia/Tbilisi
    {  674, 2772,    7842,  7, }, // Asia/Tehran
    {  674, 2821,    4356,  5, }, // Asia/Tokyo
    {  674, 1159,  215512,  9, }, // Asia/Urumqi
    {  674, 6977,  215521,  9, }, // Asia/Vientiane
    {  674, 4710,    4361, 11, }, // Asia/Vladivostok
    {  674, 4828,    7873,  7, }, // Asia/Yakutsk
    {  674, 1235,  215530,  7, }, // Asia/Yangon
    {  674, 3883,    4380, 13, }, // Asia/Yekaterinburg
    {  674, 3747,  127949,  7, }, // Asia/Yerevan
    {  674, 3663,  215537,  8, }, // Atlantic/Azores
    {  674, 5671,  215545,  8, }, // Atlantic/Bermuda
    {  674, 6628,  215553,  8, }, // Atlantic/Canary
    {  674, 3727,  215561,  9, }, // Atlantic/Cape_Verde
    {  674, 1395,  215570,  7, }, // Atlantic/Faroe
    {  674, 7272,  215577, 12, }, // Atlantic/South_Georgia
    {  674, 6821,     488, 10, }, // Atlantic/St_Helena
    {  674, 6216,   33638,  6, }, // Europe/Andorra
    {  674, 3620,   78278,  9, }, // Europe/Astrakhan
    {  674, 2300,  215589,  6, }, // Europe/Athens
    {  674, 6336,   27015,  7, }, // Europe/Belgrade
    {  674, 2167,   22766,  7, }, // Europe/Brussels
    {  674, 3928,   27029,  8, }, // Europe/Bucharest
    {  674, 6289,  215595, 11, }, // Europe/Copenhagen
    {  674, 6307,  215606, 10, }, // Europe/Gibraltar
    {  674, 8804,  215616,  9, }, // Europe/Isle_of_Man
    {  674, 2527,   78292,  4, }, // Europe/Kyiv
    {  674, 3213,   33728,  7, }, // Europe/Lisbon
    {  674, 6365,  215625, 11, }, // Europe/Luxembourg
    {  674, 6410,    6044,  6, }, // Europe/Monaco
    {  674, 3537,   57485,  6, }, // Europe/Moscow
    {  674, 4284,   63839,  6, }, // Europe/Paris
    {  674, 6275,    4533,  5, }, // Europe/Prague
    {  674, 8837,  215636,  5, }, // Europe/Riga
    {  674, 6324,   63849,  4, }, // Europe/Rome
    {  674, 4451,   78296,  7, }, // Europe/Saratov
    {  674, 6202,   21213,  6, }, // Europe/Tirane
    {  674, 8882,   78303,  9, }, // Europe/Ulyanovsk
    {  674, 6567,  215641, 13, }, // Europe/Vatican
    {  674, 6231,  215654,  4, }, // Europe/Vienna
    {  674, 4727,    4561,  9, }, // Europe/Volgograd
    {  674, 3190,   57528,  8, }, // Europe/Warsaw
    {  674, 6540,    4577,  6, }, // Europe/Zurich
    {  674, 6046,  215658,  9, }, // Indian/Christmas
    {  674, 6063,  215667, 13, }, // Indian/Cocos
    {  674, 7050,  215680, 10, }, // Indian/Maldives
    {  674, 7211,     547,  7, }, // Indian/Reunion
    {  674, 3157,     554,  5, }, // Pacific/Chuuk
    {  674, 2237,  215690, 13, }, // Pacific/Easter
    {  674, 3902,   78353,  7, }, // Pacific/Fiji
    {  674, 4168,  215703, 13, }, // Pacific/Norfolk
    {  674, 3096,     586,  7, }, // Pacific/Pohnpei
    {  675,   14,       0,  6, }, // Africa/Asmara Rajasthani/Devanagari/India
    {  675, 4435,       6,  8, }, // Africa/Sao_Tome
    {  675, 4252,      14,  8, }, // America/Asuncion
    {  675,  237,      22,  8, }, // America/Atikokan
    {  675, 7623,      30, 17, }, // America/Bahia_Banderas
    {  675, 3868,      47,  6, }, // America/Cancun
    {  675, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  675, 5723,      66,  7, }, // America/Curacao
    {  675,  481,      73, 13, }, // America/Indiana/Knox
    {  675, 7961,      86, 16, }, // America/Indiana/Marengo
    {  675, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  675, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  675, 8038,     139, 14, }, // America/Indiana/Vevay
    {  675, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  675, 8086,     171, 16, }, // America/Indiana/Winamac
    {  675, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  675, 5932,     207, 22, }, // America/Lower_Princes
    {  675, 8220,     229,  6, }, // America/Merida
    {  675, 1850,     235, 19, }, // America/Noronha
    {  675, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  675, 8329,     274, 20, }, // America/North_Dakota/Center
    {  675, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  675,  393,     317,  4, }, // America/Nuuk
    {  675, 6899,     321, 16, }, // America/Scoresbysund
    {  675, 8508,     337, 14, }, // America/St_Barthelemy
    {  675, 2061,     351, 10, }, // America/St_Johns
    {  675, 5863,     361,  9, }, // America/St_Kitts
    {  675, 5880,     370,  9, }, // America/St_Lucia
    {  675,  855,     379, 10, }, // America/St_Thomas
    {  675, 5913,     389, 11, }, // America/St_Vincent
    {  675, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  675, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  675, 1185,     448,  9, }, // Asia/Kathmandu
    {  675,  992,     457,  7, }, // Asia/Kolkata
    {  675, 1211,     464,  5, }, // Asia/Macau
    {  675, 1235,     477,  6, }, // Asia/Yangon
    {  675, 1395,     483,  5, }, // Atlantic/Faroe
    {  675, 6821,     488, 10, }, // Atlantic/St_Helena
    {  675, 2527,     514,  4, }, // Europe/Kyiv
    {  675, 7211,     547,  7, }, // Indian/Reunion
    {  675, 3157,     554,  5, }, // Pacific/Chuuk
    {  675, 3096,     586,  7, }, // Pacific/Pohnpei
    {  676,   14,       0,  6, }, // Africa/Asmara Moksha/Cyrillic/Russia
    {  676, 4435,       6,  8, }, // Africa/Sao_Tome
    {  676, 4252,      14,  8, }, // America/Asuncion
    {  676,  237,      22,  8, }, // America/Atikokan
    {  676, 7623,      30, 17, }, // America/Bahia_Banderas
    {  676, 3868,      47,  6, }, // America/Cancun
    {  676, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  676, 5723,      66,  7, }, // America/Curacao
    {  676,  481,      73, 13, }, // America/Indiana/Knox
    {  676, 7961,      86, 16, }, // America/Indiana/Marengo
    {  676, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  676, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  676, 8038,     139, 14, }, // America/Indiana/Vevay
    {  676, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  676, 8086,     171, 16, }, // America/Indiana/Winamac
    {  676, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  676, 5932,     207, 22, }, // America/Lower_Princes
    {  676, 8220,     229,  6, }, // America/Merida
    {  676, 1850,     235, 19, }, // America/Noronha
    {  676, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  676, 8329,     274, 20, }, // America/North_Dakota/Center
    {  676, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  676,  393,     317,  4, }, // America/Nuuk
    {  676, 6899,     321, 16, }, // America/Scoresbysund
    {  676, 8508,     337, 14, }, // America/St_Barthelemy
    {  676, 2061,     351, 10, }, // America/St_Johns
    {  676, 5863,     361,  9, }, // America/St_Kitts
    {  676, 5880,     370,  9, }, // America/St_Lucia
    {  676,  855,     379, 10, }, // America/St_Thomas
    {  676, 5913,     389, 11, }, // America/St_Vincent
    {  676, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  676, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  676, 1185,     448,  9, }, // Asia/Kathmandu
    {  676,  992,     457,  7, }, // Asia/Kolkata
    {  676, 1211,     464,  5, }, // Asia/Macau
    {  676, 1235,     477,  6, }, // Asia/Yangon
    {  676, 1395,     483,  5, }, // Atlantic/Faroe
    {  676, 6821,     488, 10, }, // Atlantic/St_Helena
    {  676, 2527,     514,  4, }, // Europe/Kyiv
    {  676, 7211,     547,  7, }, // Indian/Reunion
    {  676, 3157,     554,  5, }, // Pacific/Chuuk
    {  676, 3096,     586,  7, }, // Pacific/Pohnpei
    {  677,   14,       0,  6, }, // Africa/Asmara Toki Pona/Latin/world
    {  677, 4435,       6,  8, }, // Africa/Sao_Tome
    {  677, 4252,      14,  8, }, // America/Asuncion
    {  677,  237,      22,  8, }, // America/Atikokan
    {  677, 7623,      30, 17, }, // America/Bahia_Banderas
    {  677, 3868,      47,  6, }, // America/Cancun
    {  677, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  677, 5723,      66,  7, }, // America/Curacao
    {  677,  481,      73, 13, }, // America/Indiana/Knox
    {  677, 7961,      86, 16, }, // America/Indiana/Marengo
    {  677, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  677, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  677, 8038,     139, 14, }, // America/Indiana/Vevay
    {  677, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  677, 8086,     171, 16, }, // America/Indiana/Winamac
    {  677, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  677, 5932,     207, 22, }, // America/Lower_Princes
    {  677, 8220,     229,  6, }, // America/Merida
    {  677, 1850,     235, 19, }, // America/Noronha
    {  677, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  677, 8329,     274, 20, }, // America/North_Dakota/Center
    {  677, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  677,  393,     317,  4, }, // America/Nuuk
    {  677, 6899,     321, 16, }, // America/Scoresbysund
    {  677, 8508,     337, 14, }, // America/St_Barthelemy
    {  677, 2061,     351, 10, }, // America/St_Johns
    {  677, 5863,     361,  9, }, // America/St_Kitts
    {  677, 5880,     370,  9, }, // America/St_Lucia
    {  677,  855,     379, 10, }, // America/St_Thomas
    {  677, 5913,     389, 11, }, // America/St_Vincent
    {  677, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  677, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  677, 1185,     448,  9, }, // Asia/Kathmandu
    {  677,  992,     457,  7, }, // Asia/Kolkata
    {  677, 1211,     464,  5, }, // Asia/Macau
    {  677, 1235,     477,  6, }, // Asia/Yangon
    {  677, 1395,     483,  5, }, // Atlantic/Faroe
    {  677, 6821,     488, 10, }, // Atlantic/St_Helena
    {  677, 2527,     514,  4, }, // Europe/Kyiv
    {  677, 7211,     547,  7, }, // Indian/Reunion
    {  677, 3157,     554,  5, }, // Pacific/Chuuk
    {  677, 3096,     586,  7, }, // Pacific/Pohnpei
    {  678,   14,       0,  6, }, // Africa/Asmara Pijin/Latin/Solomon Islands
    {  678, 4435,       6,  8, }, // Africa/Sao_Tome
    {  678, 4252,      14,  8, }, // America/Asuncion
    {  678,  237,      22,  8, }, // America/Atikokan
    {  678, 7623,      30, 17, }, // America/Bahia_Banderas
    {  678, 3868,      47,  6, }, // America/Cancun
    {  678, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  678, 5723,      66,  7, }, // America/Curacao
    {  678,  481,      73, 13, }, // America/Indiana/Knox
    {  678, 7961,      86, 16, }, // America/Indiana/Marengo
    {  678, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  678, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  678, 8038,     139, 14, }, // America/Indiana/Vevay
    {  678, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  678, 8086,     171, 16, }, // America/Indiana/Winamac
    {  678, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  678, 5932,     207, 22, }, // America/Lower_Princes
    {  678, 8220,     229,  6, }, // America/Merida
    {  678, 1850,     235, 19, }, // America/Noronha
    {  678, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  678, 8329,     274, 20, }, // America/North_Dakota/Center
    {  678, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  678,  393,     317,  4, }, // America/Nuuk
    {  678, 6899,     321, 16, }, // America/Scoresbysund
    {  678, 8508,     337, 14, }, // America/St_Barthelemy
    {  678, 2061,     351, 10, }, // America/St_Johns
    {  678, 5863,     361,  9, }, // America/St_Kitts
    {  678, 5880,     370,  9, }, // America/St_Lucia
    {  678,  855,     379, 10, }, // America/St_Thomas
    {  678, 5913,     389, 11, }, // America/St_Vincent
    {  678, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  678, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  678, 1185,     448,  9, }, // Asia/Kathmandu
    {  678,  992,     457,  7, }, // Asia/Kolkata
    {  678, 1211,     464,  5, }, // Asia/Macau
    {  678, 1235,     477,  6, }, // Asia/Yangon
    {  678, 1395,     483,  5, }, // Atlantic/Faroe
    {  678, 6821,     488, 10, }, // Atlantic/St_Helena
    {  678, 2527,     514,  4, }, // Europe/Kyiv
    {  678, 7211,     547,  7, }, // Indian/Reunion
    {  678, 3157,     554,  5, }, // Pacific/Chuuk
    {  678, 3096,     586,  7, }, // Pacific/Pohnpei
    {  679,   14,       0,  6, }, // Africa/Asmara Obolo/Latin/Nigeria
    {  679, 4435,       6,  8, }, // Africa/Sao_Tome
    {  679, 4252,      14,  8, }, // America/Asuncion
    {  679,  237,      22,  8, }, // America/Atikokan
    {  679, 7623,      30, 17, }, // America/Bahia_Banderas
    {  679, 3868,      47,  6, }, // America/Cancun
    {  679, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  679, 5723,      66,  7, }, // America/Curacao
    {  679,  481,      73, 13, }, // America/Indiana/Knox
    {  679, 7961,      86, 16, }, // America/Indiana/Marengo
    {  679, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  679, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  679, 8038,     139, 14, }, // America/Indiana/Vevay
    {  679, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  679, 8086,     171, 16, }, // America/Indiana/Winamac
    {  679, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  679, 5932,     207, 22, }, // America/Lower_Princes
    {  679, 8220,     229,  6, }, // America/Merida
    {  679, 1850,     235, 19, }, // America/Noronha
    {  679, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  679, 8329,     274, 20, }, // America/North_Dakota/Center
    {  679, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  679,  393,     317,  4, }, // America/Nuuk
    {  679, 6899,     321, 16, }, // America/Scoresbysund
    {  679, 8508,     337, 14, }, // America/St_Barthelemy
    {  679, 2061,     351, 10, }, // America/St_Johns
    {  679, 5863,     361,  9, }, // America/St_Kitts
    {  679, 5880,     370,  9, }, // America/St_Lucia
    {  679,  855,     379, 10, }, // America/St_Thomas
    {  679, 5913,     389, 11, }, // America/St_Vincent
    {  679, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  679, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  679, 1185,     448,  9, }, // Asia/Kathmandu
    {  679,  992,     457,  7, }, // Asia/Kolkata
    {  679, 1211,     464,  5, }, // Asia/Macau
    {  679, 1235,     477,  6, }, // Asia/Yangon
    {  679, 1395,     483,  5, }, // Atlantic/Faroe
    {  679, 6821,     488, 10, }, // Atlantic/St_Helena
    {  679, 2527,     514,  4, }, // Europe/Kyiv
    {  679, 7211,     547,  7, }, // Indian/Reunion
    {  679, 3157,     554,  5, }, // Pacific/Chuuk
    {  679, 3096,     586,  7, }, // Pacific/Pohnpei
    {  680,   14,       0,  6, }, // Africa/Asmara Baluchi/Arabic/Pakistan
    {  680, 4435,       6,  8, }, // Africa/Sao_Tome
    {  680, 4252,      14,  8, }, // America/Asuncion
    {  680,  237,      22,  8, }, // America/Atikokan
    {  680, 7623,      30, 17, }, // America/Bahia_Banderas
    {  680, 3868,      47,  6, }, // America/Cancun
    {  680, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  680, 5723,      66,  7, }, // America/Curacao
    {  680,  481,      73, 13, }, // America/Indiana/Knox
    {  680, 7961,      86, 16, }, // America/Indiana/Marengo
    {  680, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  680, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  680, 8038,     139, 14, }, // America/Indiana/Vevay
    {  680, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  680, 8086,     171, 16, }, // America/Indiana/Winamac
    {  680, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  680, 5932,     207, 22, }, // America/Lower_Princes
    {  680, 8220,     229,  6, }, // America/Merida
    {  680, 1850,     235, 19, }, // America/Noronha
    {  680, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  680, 8329,     274, 20, }, // America/North_Dakota/Center
    {  680, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  680,  393,     317,  4, }, // America/Nuuk
    {  680, 6899,     321, 16, }, // America/Scoresbysund
    {  680, 8508,     337, 14, }, // America/St_Barthelemy
    {  680, 2061,     351, 10, }, // America/St_Johns
    {  680, 5863,     361,  9, }, // America/St_Kitts
    {  680, 5880,     370,  9, }, // America/St_Lucia
    {  680,  855,     379, 10, }, // America/St_Thomas
    {  680, 5913,     389, 11, }, // America/St_Vincent
    {  680, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  680, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  680, 1185,     448,  9, }, // Asia/Kathmandu
    {  680,  992,     457,  7, }, // Asia/Kolkata
    {  680, 1211,     464,  5, }, // Asia/Macau
    {  680, 1235,     477,  6, }, // Asia/Yangon
    {  680, 1395,     483,  5, }, // Atlantic/Faroe
    {  680, 6821,     488, 10, }, // Atlantic/St_Helena
    {  680, 2527,     514,  4, }, // Europe/Kyiv
    {  680, 7211,     547,  7, }, // Indian/Reunion
    {  680, 3157,     554,  5, }, // Pacific/Chuuk
    {  680, 3096,     586,  7, }, // Pacific/Pohnpei
    {  681, 6788,  215716,  7, }, // Africa/Abidjan Baluchi/Latin/Pakistan
    {  681, 6760,  215723,  5, }, // Africa/Accra
    {  681, 5059,  215728, 11, }, // Africa/Addis_Ababa
    {  681,   14,  215739,  6, }, // Africa/Asmara
    {  681,   44,  215745,  6, }, // Africa/Bamako
    {  681, 4957,  215751,  9, }, // Africa/Blantyre
    {  681, 5287,  215760, 11, }, // Africa/Brazzaville
    {  681, 4922,  215771,  9, }, // Africa/Bujumbura
    {  681, 2320,  215780,  5, }, // Africa/Cairo
    {  681, 4117,  215785, 10, }, // Africa/Casablanca
    {  681, 7449,  215795,  5, }, // Africa/Ceuta
    {  681, 6773,    6367,  7, }, // Africa/Conakry
    {  681, 5130,  215800, 12, }, // Africa/Dar_es_Salaam
    {  681, 5043,  215812,  7, }, // Africa/Djibouti
    {  681, 5243,  215819,  6, }, // Africa/Douala
    {  681, 5166,  215825,  7, }, // Africa/El_Aaiun
    {  681, 6853,  215832,  9, }, // Africa/Freetown
    {  681, 4906,  215841,  8, }, // Africa/Gaborone
    {  681, 5015,  215849,  6, }, // Africa/Harare
    {  681, 4479,  215855, 12, }, // Africa/Johannesburg
    {  681, 4499,  215867,  4, }, // Africa/Juba
    {  681, 5151,  140562,  7, }, // Africa/Kampala
    {  681, 4524,   26504,  7, }, // Africa/Khartoum
    {  681, 5306,  215871,  8, }, // Africa/Kinshasa
    {  681, 4744,  215879,  5, }, // Africa/Lagos
    {  681, 5336,  215884, 10, }, // Africa/Libreville
    {  681, 6869,  215894,  4, }, // Africa/Lome
    {  681, 5211,  215898,  6, }, // Africa/Luanda
    {  681, 4939,  140607, 10, }, // Africa/Lubumbashi
    {  681, 5001,  215904,  6, }, // Africa/Lusaka
    {  681, 5322,  215910,  6, }, // Africa/Malabo
    {  681, 4987,  215916,  6, }, // Africa/Maputo
    {  681, 5197,  215922,  6, }, // Africa/Maseru
    {  681, 5182,  215928,  7, }, // Africa/Mbabane
    {  681, 5113,  215935,  9, }, // Africa/Mogadishu
    {  681, 7034,  215944,  8, }, // Africa/Monrovia
    {  681, 3853,  215952,  6, }, // Africa/Nairobi
    {  681, 5271,  215958,  8, }, // Africa/Ndjamena
    {  681, 5354,  215966,  6, }, // Africa/Niamey
    {  681, 6803,   73579, 10, }, // Africa/Nouakchott
    {  681, 5225,  215972, 10, }, // Africa/Porto-Novo
    {  681, 4435,  215982,  8, }, // Africa/Sao_Tome
    {  681, 2866,  215990,  7, }, // Africa/Tripoli
    {  681, 6554,   74198,  5, }, // Africa/Tunis
    {  681, 4152,  215997,  7, }, // Africa/Windhoek
    {  681, 3346,  216004,  9, }, // America/Anchorage
    {  681, 5607,  216013,  8, }, // America/Anguilla
    {  681, 5624,  216021,  7, }, // America/Antigua
    {  681,   91,  216028, 10, }, // America/Argentina/Catamarca
    {  681,  270,  216038,  7, }, // America/Argentina/Cordoba
    {  681,  441,  216045,  5, }, // America/Argentina/Jujuy
    {  681,  565,  216050,  7, }, // America/Argentina/Mendoza
    {  681, 7571,    4842,  7, }, // America/Argentina/Tucuman
    {  681, 7597,  216057,  7, }, // America/Argentina/Ushuaia
    {  681, 5640,  216064,  5, }, // America/Aruba
    {  681, 4252,  216069,  9, }, // America/Asuncion
    {  681,  237,  216078,  8, }, // America/Atikokan
    {  681, 7623,  216086, 17, }, // America/Bahia_Banderas
    {  681, 7660,  216103, 13, }, // America/Blanc-Sablon
    {  681, 7681,  216116,  9, }, // America/Boa_Vista
    {  681, 7713,  216125, 11, }, // America/Cambridge_Bay
    {  681, 7735,  216136, 13, }, // America/Campo_Grande
    {  681, 3868,  216149,  8, }, // America/Cancun
    {  681, 4694,  216157,  8, }, // America/Caracas
    {  681, 4345,  216165,  7, }, // America/Cayenne
    {  681, 5496,   21601,  6, }, // America/Cayman
    {  681, 2260,  216172,  7, }, // America/Chicago
    {  681, 7756,  216179,  9, }, // America/Chihuahua
    {  681, 7774,  216188, 14, }, // America/Ciudad_Juarez
    {  681, 5422,  216202, 10, }, // America/Costa_Rica
    {  681, 7796,  216212,  7, }, // America/Creston
    {  681, 3791,  216219,  6, }, // America/Cuiaba
    {  681, 5723,  216225,  9, }, // America/Curacao
    {  681, 7812,  216234, 12, }, // America/Danmarkshavn
    {  681, 7848,  216246, 11, }, // America/Dawson_Creek
    {  681,  805,  216257,  6, }, // America/Denver
    {  681, 5739,  216263,  8, }, // America/Dominica
    {  681, 7869,  216271,  7, }, // America/Eirunepe
    {  681, 5441,  216278, 11, }, // America/El_Salvador
    {  681, 7906,  216289,  9, }, // America/Fortaleza
    {  681, 7924,  216298,  9, }, // America/Glace_Bay
    {  681, 5770,  216307,  7, }, // America/Grenada
    {  681, 5786,  216314,  8, }, // America/Guadeloupe
    {  681, 3760,  216322,  9, }, // America/Guatemala
    {  681, 4841,  216331,  8, }, // America/Guayaquil
    {  681, 6932,  216339,  6, }, // America/Guyana
    {  681, 1939,    6905,  8, }, // America/Halifax
    {  681, 2281,  216345,  6, }, // America/Havana
    {  681,  348,  216351, 12, }, // America/Indiana/Indianapolis
    {  681,  481,  216363, 14, }, // America/Indiana/Knox
    {  681, 7961,  216377, 15, }, // America/Indiana/Marengo
    {  681, 7985,  216392, 19, }, // America/Indiana/Petersburg
    {  681, 8012,  216411, 19, }, // America/Indiana/Tell_City
    {  681, 8038,  216430, 14, }, // America/Indiana/Vevay
    {  681, 8060,  216444, 18, }, // America/Indiana/Vincennes
    {  681, 8086,  216462, 16, }, // America/Indiana/Winamac
    {  681, 8110,  195957,  6, }, // America/Inuvik
    {  681,  660,  216478,  7, }, // America/Iqaluit
    {  681, 2799,  216485,  7, }, // America/Jamaica
    {  681, 5380,  216492,  5, }, // America/Juneau
    {  681,  521,  216497, 10, }, // America/Kentucky/Louisville
    {  681, 8125,  216507, 20, }, // America/Kentucky/Monticello
    {  681, 4376,  216527,  6, }, // America/La_Paz
    {  681, 7169,  216533,  4, }, // America/Lima
    {  681, 3239,  216537, 11, }, // America/Los_Angeles
    {  681, 8153,  216548,  6, }, // America/Maceio
    {  681, 5805,  213389,  7, }, // America/Martinique
    {  681, 8202,    7118,  8, }, // America/Menominee
    {  681, 2949,  216554, 13, }, // America/Mexico_City
    {  681, 4391,  216567,  8, }, // America/Miquelon
    {  681, 8254,    7156,  7, }, // America/Moncton
    {  681, 8270,  216575,  9, }, // America/Monterrey
    {  681, 4098,  216584, 10, }, // America/Montevideo
    {  681, 2379,  216594,  8, }, // America/New_York
    {  681, 1850,  216602,  7, }, // America/Noronha
    {  681, 8301,  216609, 19, }, // America/North_Dakota/Beulah
    {  681, 8329,  216628, 20, }, // America/North_Dakota/Center
    {  681, 8357,  216648, 22, }, // America/North_Dakota/New_Salem
    {  681, 2356,  216670,  6, }, // America/Panama
    {  681, 6173,  216676, 10, }, // America/Paramaribo
    {  681, 2973,  216686,  7, }, // America/Phoenix
    {  681, 3945,  216693, 13, }, // America/Port-au-Prince
    {  681, 5954,  216706, 10, }, // America/Port_of_Spain
    {  681, 8404,  216716, 11, }, // America/Porto_Velho
    {  681, 5843,  216727, 11, }, // America/Puerto_Rico
    {  681, 8445,  216738,  7, }, // America/Recife
    {  681, 8460,  216745,  7, }, // America/Resolute
    {  681,  695,  216752, 10, }, // America/Rio_Branco
    {  681, 2201,  216762,  8, }, // America/Santiago
    {  681, 6111,  216770, 13, }, // America/Santo_Domingo
    {  681, 1878,  216783,  9, }, // America/Sao_Paulo
    {  681, 6899,  216792, 14, }, // America/Scoresbysund
    {  681, 5880,  216806,  9, }, // America/St_Lucia
    {  681,  855,  216815,  9, }, // America/St_Thomas
    {  681, 5913,  216824, 11, }, // America/St_Vincent
    {  681, 8530,  216835, 13, }, // America/Swift_Current
    {  681, 5461,  216848, 12, }, // America/Tegucigalpa
    {  681, 5756,  216860,  4, }, // America/Thule
    {  681,  313,  216864,  7, }, // America/Tijuana
    {  681,  608,  216871,  7, }, // America/Toronto
    {  681, 5688,  216878,  7, }, // America/Tortola
    {  681, 2093,  216885,  8, }, // America/Vancouver
    {  681, 6001,  216893,  4, }, // Antarctica/Casey
    {  681, 6094,  216897,  5, }, // Antarctica/Davis
    {  681, 6133,  216902, 13, }, // Antarctica/DumontDUrville
    {  681, 8568,  216915,  7, }, // Antarctica/Macquarie
    {  681, 7295,  216922,  5, }, // Antarctica/Syowa
    {  681, 7374,    4147,  6, }, // Antarctica/Vostok
    {  681, 5570,  201864,  4, }, // Asia/Aden
    {  681, 5395,    7594,  6, }, // Asia/Almaty
    {  681, 3968,   57924,  5, }, // Asia/Amman
    {  681, 5511,    7600,  6, }, // Asia/Anadyr
    {  681, 5523,    5609,  5, }, // Asia/Aqtau
    {  681, 5368,  216927,  6, }, // Asia/Aqtobe
    {  681, 8624,    7612,  6, }, // Asia/Atyrau
    {  681, 3607,   57937,  6, }, // Asia/Baghdad
    {  681, 5534,  216933,  6, }, // Asia/Bahrain
    {  681, 3653,  142075,  4, }, // Asia/Baku
    {  681, 4466,  216939,  7, }, // Asia/Bangkok
    {  681, 4086,  216946,  5, }, // Asia/Beirut
    {  681, 5989,  216951,  6, }, // Asia/Brunei
    {  681, 4511,  216957,  7, }, // Asia/Colombo
    {  681, 4540,  216964,  8, }, // Asia/Damascus
    {  681, 1093,   57963,  5, }, // Asia/Dhaka
    {  681, 6192,  216972,  5, }, // Asia/Dili
    {  681, 3596,  216977,  5, }, // Asia/Dubai
    {  681, 6159,  216982,  9, }, // Asia/Dushanbe
    {  681, 8636,  216991,  9, }, // Asia/Famagusta
    {  681, 4795,   20771,  6, }, // Asia/Hebron
    {  681, 1259,  217000, 11, }, // Asia/Ho_Chi_Minh
    {  681, 2704,  217011,  9, }, // Asia/Hong_Kong
    {  681, 4771,  217020,  4, }, // Asia/Hovd
    {  681, 7006,  217024,  7, }, // Asia/Jakarta
    {  681, 6992,  201941,  6, }, // Asia/Jayapura
    {  681, 1290,  217031,  8, }, // Asia/Jerusalem
    {  681, 3560,  217039,  5, }, // Asia/Kabul
    {  681, 4239,  142223,  7, }, // Asia/Karachi
    {  681, 8661,   26856,  8, }, // Asia/Khandyga
    {  681,  992,   58028,  7, }, // Asia/Kolkata
    {  681, 4859,  217044, 12, }, // Asia/Kuala_Lumpur
    {  681, 5547,  217056,  5, }, // Asia/Kuwait
    {  681, 1211,  217061,  5, }, // Asia/Macau
    {  681, 6920,   26882,  6, }, // Asia/Muscat
    {  681, 2554,  217066,  8, }, // Asia/Nicosia
    {  681, 8675,   78199, 12, }, // Asia/Novokuznetsk
    {  681, 4135,   78211, 11, }, // Asia/Novosibirsk
    {  681, 7145,  217074,  4, }, // Asia/Oral
    {  681, 4214,  217078,  9, }, // Asia/Pyongyang
    {  681, 5559,  217087,  5, }, // Asia/Qatar
    {  681, 8708,  217092,  8, }, // Asia/Qostanay
    {  681, 4269,  217100,  9, }, // Asia/Qyzylorda
    {  681, 3584,  217109,  4, }, // Asia/Riyadh
    {  681, 3279,  217113,  4, }, // Asia/Seoul
    {  681, 1053,  162020,  7, }, // Asia/Shanghai
    {  681, 3300,  217117,  8, }, // Asia/Singapore
    {  681, 4297,  217125, 13, }, // Asia/Srednekolymsk
    {  681, 3263,  217138,  5, }, // Asia/Taipei
    {  681, 4781,  217143,  8, }, // Asia/Tashkent
    {  681, 3915,  217151,  8, }, // Asia/Tbilisi
    {  681, 2772,  217159,  6, }, // Asia/Tehran
    {  681, 1317,    7849,  6, }, // Asia/Thimphu
    {  681, 2821,  217165,  5, }, // Asia/Tokyo
    {  681, 1021,   58138,  9, }, // Asia/Ulaanbaatar
    {  681, 1159,   74057,  6, }, // Asia/Urumqi
    {  681, 6977,  217170,  9, }, // Asia/Vientiane
    {  681, 4710,  217179, 11, }, // Asia/Vladivostok
    {  681, 3747,  217190,  7, }, // Asia/Yerevan
    {  681, 3663,  151882,  6, }, // Atlantic/Azores
    {  681, 6628,  217197,  7, }, // Atlantic/Canary
    {  681, 3727,  217204,  9, }, // Atlantic/Cape_Verde
    {  681, 2748,  217213,  9, }, // Atlantic/Reykjavik
    {  681, 7272,  217222, 14, }, // Atlantic/South_Georgia
    {  681, 6821,  217236, 10, }, // Atlantic/St_Helena
    {  681, 6644,  217246,  6, }, // Atlantic/Stanley
    {  681, 1670,  217252,  6, }, // Australia/Adelaide
    {  681, 3637,  217258,  5, }, // Australia/Eucla
    {  681, 1727,  217263,  7, }, // Australia/Melbourne
    {  681, 1762,   22735,  4, }, // Australia/Perth
    {  681, 1463,  217270,  6, }, // Australia/Sydney
    {  681, 2300,  217276,  4, }, // Europe/Athens
    {  681, 6336,   91950,  7, }, // Europe/Belgrade
    {  681, 6488,  217280, 10, }, // Europe/Bratislava
    {  681, 3928,  217290,  9, }, // Europe/Bucharest
    {  681, 3806,  217299,  8, }, // Europe/Budapest
    {  681, 6289,  217307, 10, }, // Europe/Copenhagen
    {  681, 8788,  217317,  8, }, // Europe/Guernsey
    {  681, 1130,  217325,  8, }, // Europe/Istanbul
    {  681, 8823,  217333,  6, }, // Europe/Jersey
    {  681, 8929,  217339,  5, }, // Europe/Kirov
    {  681, 6506,  217344,  9, }, // Europe/Ljubljana
    {  681, 2501,  177099,  6, }, // Europe/London
    {  681, 6365,  217353, 10, }, // Europe/Luxembourg
    {  681, 6410,  217363,  6, }, // Europe/Monaco
    {  681, 3537,  217369,  5, }, // Europe/Moscow
    {  681, 6458,  217374,  4, }, // Europe/Oslo
    {  681, 4284,  217378,  6, }, // Europe/Paris
    {  681, 6424,  217384,  9, }, // Europe/Podgorica
    {  681, 6275,   92055,  4, }, // Europe/Prague
    {  681, 6324,   99309,  3, }, // Europe/Rome
    {  681, 6470,  217393, 10, }, // Europe/San_Marino
    {  681, 6245,  217403,  8, }, // Europe/Sarajevo
    {  681, 4451,  157769,  7, }, // Europe/Saratov
    {  681, 8849,  217411, 10, }, // Europe/Simferopol
    {  681, 6383,  217421,  5, }, // Europe/Skopje
    {  681, 6599,  217426,  5, }, // Europe/Sofia
    {  681, 6523,  217431, 10, }, // Europe/Stockholm
    {  681, 6202,  217441,  6, }, // Europe/Tirane
    {  681, 8882,  217447,  9, }, // Europe/Ulyanovsk
    {  681, 6352,  196874,  5, }, // Europe/Vaduz
    {  681, 6567,  196879,  7, }, // Europe/Vatican
    {  681, 6231,  217456,  6, }, // Europe/Vienna
    {  681, 8899,  217462,  7, }, // Europe/Vilnius
    {  681, 4727,    4561,  9, }, // Europe/Volgograd
    {  681, 3190,  217469,  5, }, // Europe/Warsaw
    {  681, 6540,  217474,  7, }, // Europe/Zurich
    {  681, 5078,  217481, 12, }, // Indian/Antananarivo
    {  681, 6947,  217493,  6, }, // Indian/Chagos
    {  681, 6046,  217499,  7, }, // Indian/Christmas
    {  681, 6063,  217506,  5, }, // Indian/Cocos
    {  681, 5029,  217511,  6, }, // Indian/Comoro
    {  681, 6661,  217517,  9, }, // Indian/Kerguelen
    {  681, 7260,   27115,  4, }, // Indian/Mahe
    {  681, 7050,  217526,  6, }, // Indian/Maldives
    {  681, 4069,  217532,  8, }, // Indian/Mauritius
    {  681, 5098,  217540,  7, }, // Indian/Mayotte
    {  681, 7211,  217547,  8, }, // Indian/Reunion
    {  681, 4422,  217555,  4, }, // Pacific/Apia
    {  681,  932,  217559,  7, }, // Pacific/Auckland
    {  681, 3706,  217566, 11, }, // Pacific/Bougainville
    {  681, 2237,  217577,  6, }, // Pacific/Easter
    {  681, 7360,  217583,  5, }, // Pacific/Efate
    {  681, 7327,  217588,  7, }, // Pacific/Fakaofo
    {  681, 3822,    8360, 11, }, // Pacific/Guadalcanal
    {  681, 7019,  217595,  6, }, // Pacific/Kosrae
    {  681, 4051,  217601,  8, }, // Pacific/Marquesas
    {  681, 4168,  217609,  7, }, // Pacific/Norfolk
    {  681, 7194,  217616,  8, }, // Pacific/Pitcairn
    {  682, 5059,    3902, 11, }, // Africa/Addis_Ababa Ligurian/Latin/Italy
    {  682, 7420,  217624,  4, }, // Africa/Algiers
    {  682,   14,       0,  6, }, // Africa/Asmara
    {  682, 7449,  217628,  5, }, // Africa/Ceuta
    {  682, 6869,   20418,  4, }, // Africa/Lome
    {  682, 5113,   44110, 10, }, // Africa/Mogadishu
    {  682, 3853,  217633,  7, }, // Africa/Nairobi
    {  682, 5271,   44128,  9, }, // Africa/Ndjamena
    {  682, 4435,   58730,  8, }, // Africa/Sao_Tome
    {  682, 6554,  217640,  6, }, // Africa/Tunis
    {  682,  270,  217646,  7, }, // America/Argentina/Cordoba
    {  682, 4252,      14,  8, }, // America/Asuncion
    {  682,  237,      22,  8, }, // America/Atikokan
    {  682, 7623,      30, 17, }, // America/Bahia_Banderas
    {  682, 4361,   44161,  6, }, // America/Bogota
    {  682, 3868,      47,  6, }, // America/Cancun
    {  682, 4345,  217653,  6, }, // America/Cayenne
    {  682, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  682, 5422,  147764, 10, }, // America/Costa_Rica
    {  682, 5723,      66,  7, }, // America/Curacao
    {  682, 5770,   20519,  7, }, // America/Grenada
    {  682, 5786,  217659, 10, }, // America/Guadeloupe
    {  682,  481,      73, 13, }, // America/Indiana/Knox
    {  682, 7961,      86, 16, }, // America/Indiana/Marengo
    {  682, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  682, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  682, 8038,     139, 14, }, // America/Indiana/Vevay
    {  682, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  682, 8086,     171, 16, }, // America/Indiana/Winamac
    {  682, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  682, 5932,     207, 22, }, // America/Lower_Princes
    {  682, 5805,   20554,  9, }, // America/Martinique
    {  682, 8220,     229,  6, }, // America/Merida
    {  682, 2949,  217669, 16, }, // America/Mexico_City
    {  682, 1850,     235, 19, }, // America/Noronha
    {  682, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  682, 8329,     274, 20, }, // America/North_Dakota/Center
    {  682, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  682,  393,     317,  4, }, // America/Nuuk
    {  682, 1878,  217685,  9, }, // America/Sao_Paulo
    {  682, 6899,     321, 16, }, // America/Scoresbysund
    {  682, 8508,  217694, 11, }, // America/St_Barthelemy
    {  682, 2061,     351, 10, }, // America/St_Johns
    {  682, 5863,  217705, 13, }, // America/St_Kitts
    {  682, 5880,  217718, 11, }, // America/St_Lucia
    {  682,  855,     379, 10, }, // America/St_Thomas
    {  682, 5913,  217729, 11, }, // America/St_Vincent
    {  682, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  682, 5534,    4168,  7, }, // Asia/Bahrain
    {  682, 4601,   93254,  4, }, // Asia/Chita
    {  682, 4540,   74438,  7, }, // Asia/Damascus
    {  682, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  682, 7006,   95267,  8, }, // Asia/Jakarta
    {  682, 1290,   95275, 11, }, // Asia/Jerusalem
    {  682, 1185,     448,  9, }, // Asia/Kathmandu
    {  682,  992,   36053,  8, }, // Asia/Kolkata
    {  682, 1211,     464,  5, }, // Asia/Macau
    {  682, 7245,   20886, 10, }, // Asia/Samarkand
    {  682, 3300,  217740,  8, }, // Asia/Singapore
    {  682, 1021,   26923, 10, }, // Asia/Ulaanbaatar
    {  682, 1235,     477,  6, }, // Asia/Yangon
    {  682, 3883,    7886, 12, }, // Asia/Yekaterinburg
    {  682, 3663,    7905,  5, }, // Atlantic/Azores
    {  682, 6628,  217748,  6, }, // Atlantic/Canary
    {  682, 3727,  217754, 11, }, // Atlantic/Cape_Verde
    {  682, 1395,   95299,  7, }, // Atlantic/Faroe
    {  682, 7272,  217765, 14, }, // Atlantic/South_Georgia
    {  682, 6821,   95321, 10, }, // Atlantic/St_Helena
    {  682, 6216,  147624,  7, }, // Europe/Andorra
    {  682, 2300,    5911,  5, }, // Europe/Athens
    {  682, 6336,  217779,  9, }, // Europe/Belgrade
    {  682, 2167,   57661,  9, }, // Europe/Brussels
    {  682, 2338,  217788,  6, }, // Europe/Dublin
    {  682, 6307,  217794,  7, }, // Europe/Gibraltar
    {  682, 8804,  217801, 11, }, // Europe/Isle_of_Man
    {  682, 2527,     514,  4, }, // Europe/Kyiv
    {  682, 3213,  217812,  7, }, // Europe/Lisbon
    {  682, 6506,  217819,  7, }, // Europe/Ljubljana
    {  682, 2501,   95370,  6, }, // Europe/London
    {  682, 6365,   74574, 10, }, // Europe/Luxembourg
    {  682, 6410,  217826,  6, }, // Europe/Monaco
    {  682, 3537,   95387,  5, }, // Europe/Moscow
    {  682, 6458,  217832,  4, }, // Europe/Oslo
    {  682, 4284,  167277,  7, }, // Europe/Paris
    {  682, 6275,   21185,  5, }, // Europe/Prague
    {  682, 6324,  217836,  5, }, // Europe/Rome
    {  682, 6470,   65868,  9, }, // Europe/San_Marino
    {  682, 6523,   95409,  9, }, // Europe/Stockholm
    {  682, 6202,  217841,  6, }, // Europe/Tirane
    {  682, 3190,   95436,  8, }, // Europe/Warsaw
    {  682, 6261,   95444,  8, }, // Europe/Zagreb
    {  682, 6540,   95452,  6, }, // Europe/Zurich
    {  682, 5029,  217847,  6, }, // Indian/Comoro
    {  682, 7050,    8311,  7, }, // Indian/Maldives
    {  682, 5098,  217853,  7, }, // Indian/Mayotte
    {  682, 7211,     547,  7, }, // Indian/Reunion
    {  682, 3157,     554,  5, }, // Pacific/Chuuk
    {  682, 2237,   44642,  6, }, // Pacific/Easter
    {  682, 3902,   95469,  4, }, // Pacific/Fiji
    {  682, 4051,  217860,  9, }, // Pacific/Marquesas
    {  682, 7098,    4628,  6, }, // Pacific/Noumea
    {  682, 3096,     586,  7, }, // Pacific/Pohnpei
    {  683,   14,       0,  6, }, // Africa/Asmara Rohingya/Hanifi/Myanmar
    {  683, 4435,       6,  8, }, // Africa/Sao_Tome
    {  683, 4252,      14,  8, }, // America/Asuncion
    {  683,  237,      22,  8, }, // America/Atikokan
    {  683, 7623,      30, 17, }, // America/Bahia_Banderas
    {  683, 3868,      47,  6, }, // America/Cancun
    {  683, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  683, 5723,      66,  7, }, // America/Curacao
    {  683,  481,      73, 13, }, // America/Indiana/Knox
    {  683, 7961,      86, 16, }, // America/Indiana/Marengo
    {  683, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  683, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  683, 8038,     139, 14, }, // America/Indiana/Vevay
    {  683, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  683, 8086,     171, 16, }, // America/Indiana/Winamac
    {  683, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  683, 5932,     207, 22, }, // America/Lower_Princes
    {  683, 8220,     229,  6, }, // America/Merida
    {  683, 1850,     235, 19, }, // America/Noronha
    {  683, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  683, 8329,     274, 20, }, // America/North_Dakota/Center
    {  683, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  683,  393,     317,  4, }, // America/Nuuk
    {  683, 6899,     321, 16, }, // America/Scoresbysund
    {  683, 8508,     337, 14, }, // America/St_Barthelemy
    {  683, 2061,     351, 10, }, // America/St_Johns
    {  683, 5863,     361,  9, }, // America/St_Kitts
    {  683, 5880,     370,  9, }, // America/St_Lucia
    {  683,  855,     379, 10, }, // America/St_Thomas
    {  683, 5913,     389, 11, }, // America/St_Vincent
    {  683, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  683, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  683, 1185,     448,  9, }, // Asia/Kathmandu
    {  683,  992,     457,  7, }, // Asia/Kolkata
    {  683, 1211,     464,  5, }, // Asia/Macau
    {  683, 1235,     477,  6, }, // Asia/Yangon
    {  683, 1395,     483,  5, }, // Atlantic/Faroe
    {  683, 6821,     488, 10, }, // Atlantic/St_Helena
    {  683, 2527,     514,  4, }, // Europe/Kyiv
    {  683, 7211,     547,  7, }, // Indian/Reunion
    {  683, 3157,     554,  5, }, // Pacific/Chuuk
    {  683, 3096,     586,  7, }, // Pacific/Pohnpei
    {  685, 6788,  198897,  7, }, // Africa/Abidjan Torwali/Arabic/Pakistan
    {  685, 6760,  198904,  5, }, // Africa/Accra
    {  685, 5059,  103115, 10, }, // Africa/Addis_Ababa
    {  685, 7420,  198909,  7, }, // Africa/Algiers
    {  685,   14,  106755,  6, }, // Africa/Asmara
    {  685,   44,  103142,  5, }, // Africa/Bamako
    {  685, 5257,  198916,  6, }, // Africa/Bangui
    {  685, 6746,  198922,  5, }, // Africa/Banjul
    {  685, 7435,  103162,  5, }, // Africa/Bissau
    {  685, 4957,  198927,  8, }, // Africa/Blantyre
    {  685, 5287,  198935,  9, }, // Africa/Brazzaville
    {  685, 4922,  198944,  6, }, // Africa/Bujumbura
    {  685, 2320,  198950,  5, }, // Africa/Cairo
    {  685, 4117,  198955, 11, }, // Africa/Casablanca
    {  685, 7449,  198966,  5, }, // Africa/Ceuta
    {  685, 6773,  198971,  6, }, // Africa/Conakry
    {  685, 6840,  198977,  4, }, // Africa/Dakar
    {  685, 5130,   10617, 10, }, // Africa/Dar_es_Salaam
    {  685, 5043,  198981,  5, }, // Africa/Djibouti
    {  685, 5243,  198986,  5, }, // Africa/Douala
    {  685, 5166,  154637,  6, }, // Africa/El_Aaiun
    {  685, 6853,  198991,  8, }, // Africa/Freetown
    {  685, 4906,  198999,  5, }, // Africa/Gaborone
    {  685, 5015,  199004,  5, }, // Africa/Harare
    {  685, 4479,  103278,  9, }, // Africa/Johannesburg
    {  685, 4499,   10673,  4, }, // Africa/Juba
    {  685, 5151,  199009,  7, }, // Africa/Kampala
    {  685, 4524,   10686,  5, }, // Africa/Khartoum
    {  685, 4973,  199016,  5, }, // Africa/Kigali
    {  685, 5306,  199021,  6, }, // Africa/Kinshasa
    {  685, 4744,  106900,  5, }, // Africa/Lagos
    {  685, 5336,  199027,  9, }, // Africa/Libreville
    {  685, 6869,   10716,  3, }, // Africa/Lome
    {  685, 5211,  199036,  6, }, // Africa/Luanda
    {  685, 4939,  167986,  8, }, // Africa/Lubumbashi
    {  685, 5001,  199042,  7, }, // Africa/Lusaka
    {  685, 5322,  168000,  5, }, // Africa/Malabo
    {  685, 4987,  199049,  5, }, // Africa/Maputo
    {  685, 5197,  199054,  5, }, // Africa/Maseru
    {  685, 5182,  199059,  6, }, // Africa/Mbabane
    {  685, 5113,  154736,  8, }, // Africa/Mogadishu
    {  685, 7034,  154744,  8, }, // Africa/Monrovia
    {  685, 3853,  103386,  6, }, // Africa/Nairobi
    {  685, 5271,  199065,  9, }, // Africa/Ndjamena
    {  685, 5354,  154766,  5, }, // Africa/Niamey
    {  685, 6803,  199074,  7, }, // Africa/Nouakchott
    {  685, 6727,  199081,  9, }, // Africa/Ouagadougou
    {  685, 5225,  103422, 10, }, // Africa/Porto-Novo
    {  685, 4435,  199090,  7, }, // Africa/Sao_Tome
    {  685, 2866,  199097,  7, }, // Africa/Tripoli
    {  685, 6554,  199104,  5, }, // Africa/Tunis
    {  685, 4152,  199109,  6, }, // Africa/Windhoek
    {  685,  132,  152176,  4, }, // America/Adak
    {  685, 3346,  152180,  7, }, // America/Anchorage
    {  685, 5607,  199115,  7, }, // America/Anguilla
    {  685, 5624,  199122,  7, }, // America/Antigua
    {  685, 4554,  168097,  9, }, // America/Araguaina
    {  685,  166,  199129, 10, }, // America/Argentina/Buenos_Aires
    {  685,   91,  217869,  9, }, // America/Argentina/Catamarca
    {  685,  270,  199139,  7, }, // America/Argentina/Cordoba
    {  685,  441,  199146,  6, }, // America/Argentina/Jujuy
    {  685, 7462,  168141,  9, }, // America/Argentina/La_Rioja
    {  685,  565,  199152,  7, }, // America/Argentina/Mendoza
    {  685, 7489,  168157, 11, }, // America/Argentina/Rio_Gallegos
    {  685, 7520,  103559,  5, }, // America/Argentina/Salta
    {  685, 7544,  168173,  8, }, // America/Argentina/San_Juan
    {  685, 5580,  199159,  8, }, // America/Argentina/San_Luis
    {  685, 7571,  199167,  7, }, // America/Argentina/Tucuman
    {  685, 7597,  168197,  8, }, // America/Argentina/Ushuaia
    {  685, 5640,  154922,  5, }, // America/Aruba
    {  685, 4252,  199174,  7, }, // America/Asuncion
    {  685,  237,  199181,  7, }, // America/Atikokan
    {  685, 3679,  103633,  5, }, // America/Bahia
    {  685, 7623,  199188, 11, }, // America/Bahia_Banderas
    {  685, 5654,  103645,  8, }, // America/Barbados
    {  685, 7646,  199199,  5, }, // America/Belem
    {  685, 5407,  199204,  6, }, // America/Belize
    {  685, 7660,  199210, 11, }, // America/Blanc-Sablon
    {  685, 7681,  199221,  8, }, // America/Boa_Vista
    {  685, 4361,  199229,  5, }, // America/Bogota
    {  685, 7699,  199234,  5, }, // America/Boise
    {  685, 7713,  217878, 15, }, // America/Cambridge_Bay
    {  685, 7735,  199254, 11, }, // America/Campo_Grande
    {  685, 3868,  199265,  6, }, // America/Cancun
    {  685, 4694,  217893,  6, }, // America/Caracas
    {  685, 4345,  217899,  5, }, // America/Cayenne
    {  685, 5496,  199282,  5, }, // America/Cayman
    {  685, 2260,  199287,  5, }, // America/Chicago
    {  685, 7756,  199292,  8, }, // America/Chihuahua
    {  685, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  685, 5422,  199300, 10, }, // America/Costa_Rica
    {  685, 7796,  199310,  7, }, // America/Creston
    {  685, 3791,  199317,  7, }, // America/Cuiaba
    {  685, 5723,  199324,  8, }, // America/Curacao
    {  685, 7812,  103794, 11, }, // America/Danmarkshavn
    {  685, 7833,  199332,  5, }, // America/Dawson
    {  685, 7848,  199337, 10, }, // America/Dawson_Creek
    {  685,  805,  199347,  5, }, // America/Denver
    {  685, 3465,  199352,  7, }, // America/Detroit
    {  685, 5739,  199359,  7, }, // America/Dominica
    {  685,  893,  199366,  8, }, // America/Edmonton
    {  685, 7869,  199374,  7, }, // America/Eirunepe
    {  685, 5441,  199381, 10, }, // America/El_Salvador
    {  685, 7886,  103874, 10, }, // America/Fort_Nelson
    {  685, 7906,  199391,  9, }, // America/Fortaleza
    {  685, 7924,  217904, 13, }, // America/Glace_Bay
    {  685, 6881,  217917, 12, }, // America/Goose_Bay
    {  685, 4612,  199425,  8, }, // America/Grand_Turk
    {  685, 5770,  199433,  6, }, // America/Grenada
    {  685, 5786,  199439,  8, }, // America/Guadeloupe
    {  685, 3760,  199447, 10, }, // America/Guatemala
    {  685, 4841,  199457,  9, }, // America/Guayaquil
    {  685, 6932,  199466,  5, }, // America/Guyana
    {  685, 1939,  199471,  8, }, // America/Halifax
    {  685, 2281,  199479,  5, }, // America/Havana
    {  685, 7942,  199484,  8, }, // America/Hermosillo
    {  685,  348,  199492, 11, }, // America/Indiana/Indianapolis
    {  685,  481,  199503, 14, }, // America/Indiana/Knox
    {  685, 7961,  199517, 15, }, // America/Indiana/Marengo
    {  685, 7985,  199532, 17, }, // America/Indiana/Petersburg
    {  685, 8012,  199549, 16, }, // America/Indiana/Tell_City
    {  685, 8038,  199565, 13, }, // America/Indiana/Vevay
    {  685, 8060,  199578, 16, }, // America/Indiana/Vincennes
    {  685, 8086,  199594, 16, }, // America/Indiana/Winamac
    {  685, 8110,  199610,  6, }, // America/Inuvik
    {  685,  660,  199616,  8, }, // America/Iqaluit
    {  685, 2799,  199624,  7, }, // America/Jamaica
    {  685, 5380,  199631,  6, }, // America/Juneau
    {  685,  521,  199637,  9, }, // America/Kentucky/Louisville
    {  685, 8125,  199646, 18, }, // America/Kentucky/Monticello
    {  685, 5704,  199664, 10, }, // America/Kralendijk
    {  685, 4376,  104164,  6, }, // America/La_Paz
    {  685, 7169,  152774,  4, }, // America/Lima
    {  685, 3239,  152778, 10, }, // America/Los_Angeles
    {  685, 5932,  199674, 17, }, // America/Lower_Princes
    {  685, 8153,  168752,  6, }, // America/Maceio
    {  685, 8168,  199691,  6, }, // America/Managua
    {  685, 1908,  199697,  5, }, // America/Manaus
    {  685, 5897,  104220,  7, }, // America/Marigot
    {  685, 5805,  199702,  7, }, // America/Martinique
    {  685, 8184,  104235,  9, }, // America/Matamoros
    {  685, 2917,  199709,  7, }, // America/Mazatlan
    {  685, 8202,  199716,  8, }, // America/Menominee
    {  685, 8220,  199724,  6, }, // America/Merida
    {  685, 8235,  104266, 11, }, // America/Metlakatla
    {  685, 2949,  199730, 11, }, // America/Mexico_City
    {  685, 4391,  199741,  8, }, // America/Miquelon
    {  685, 8254,  199749,  6, }, // America/Moncton
    {  685, 8270,  199755,  7, }, // America/Monterrey
    {  685, 4098,  199762, 11, }, // America/Montevideo
    {  685, 5824,  199773,  9, }, // America/Montserrat
    {  685, 5481,  104328,  4, }, // America/Nassau
    {  685, 2379,  199782,  8, }, // America/New_York
    {  685, 8288,   11540,  3, }, // America/Nome
    {  685, 1850,  104341,  7, }, // America/Noronha
    {  685, 8301,  199790, 19, }, // America/North_Dakota/Beulah
    {  685, 8329,  199809, 16, }, // America/North_Dakota/Center
    {  685, 8357,  199825, 21, }, // America/North_Dakota/New_Salem
    {  685,  393,  103995,  3, }, // America/Nuuk
    {  685, 8388,  155609,  8, }, // America/Ojinaga
    {  685, 2356,  199846,  5, }, // America/Panama
    {  685, 6173,  199851,  9, }, // America/Paramaribo
    {  685, 2973,  153019,  5, }, // America/Phoenix
    {  685, 3945,  199860, 12, }, // America/Port-au-Prince
    {  685, 5954,  199872, 13, }, // America/Port_of_Spain
    {  685, 8404,  199885, 11, }, // America/Porto_Velho
    {  685, 5843,  199896, 11, }, // America/Puerto_Rico
    {  685, 4030,  199907, 11, }, // America/Punta_Arenas
    {  685, 8424,  199918, 11, }, // America/Rankin_Inlet
    {  685, 8445,  199929,  6, }, // America/Recife
    {  685, 1995,  169043,  6, }, // America/Regina
    {  685, 8460,  199935,  8, }, // America/Resolute
    {  685,  695,  199943, 11, }, // America/Rio_Branco
    {  685, 8477,  199954,  7, }, // America/Santarem
    {  685, 2201,  199961,  7, }, // America/Santiago
    {  685, 6111,  199968, 12, }, // America/Santo_Domingo
    {  685, 1878,  104578,  8, }, // America/Sao_Paulo
    {  685, 6899,  199980, 15, }, // America/Scoresbysund
    {  685, 8494,  199995,  5, }, // America/Sitka
    {  685, 8508,  200000, 13, }, // America/St_Barthelemy
    {  685, 2061,  200013,  9, }, // America/St_Johns
    {  685, 5863,  200022,  8, }, // America/St_Kitts
    {  685, 5880,  200030, 10, }, // America/St_Lucia
    {  685,  855,  200040, 10, }, // America/St_Thomas
    {  685, 5913,  200050, 10, }, // America/St_Vincent
    {  685, 8530,  200060, 10, }, // America/Swift_Current
    {  685, 5461,  200070, 11, }, // America/Tegucigalpa
    {  685, 5756,  200081,  5, }, // America/Thule
    {  685,  313,  200086,  7, }, // America/Tijuana
    {  685,  608,  200093,  6, }, // America/Toronto
    {  685, 5688,  200099,  7, }, // America/Tortola
    {  685, 2093,  153272,  7, }, // America/Vancouver
    {  685, 2144,  200106, 10, }, // America/Whitehorse
    {  685,  734,  200116,  7, }, // America/Winnipeg
    {  685, 8552,  200123,  6, }, // America/Yakutat
    {  685, 6001,  104758,  4, }, // Antarctica/Casey
    {  685, 6094,  200129,  4, }, // Antarctica/Davis
    {  685, 6133,  200133, 16, }, // Antarctica/DumontDUrville
    {  685, 8568,  200149,  7, }, // Antarctica/Macquarie
    {  685, 7066,  200156,  5, }, // Antarctica/Mawson
    {  685, 7113,  200161,  8, }, // Antarctica/McMurdo
    {  685, 8589,  200169,  5, }, // Antarctica/Palmer
    {  685, 7226,  104800,  7, }, // Antarctica/Rothera
    {  685, 7295,  104807,  5, }, // Antarctica/Syowa
    {  685, 8607,  200174,  4, }, // Antarctica/Troll
    {  685, 7374,  104812,  6, }, // Antarctica/Vostok
    {  685, 1429,  200178, 13, }, // Arctic/Longyearbyen
    {  685, 5570,   12066,  3, }, // Asia/Aden
    {  685, 5395,  200191,  6, }, // Asia/Almaty
    {  685, 3968,  200197,  5, }, // Asia/Amman
    {  685, 5511,  200202,  5, }, // Asia/Anadyr
    {  685, 5523,  200207,  5, }, // Asia/Aqtau
    {  685, 5368,  200212,  5, }, // Asia/Aqtobe
    {  685,  964,  200217,  5, }, // Asia/Ashgabat
    {  685, 8624,  200222,  6, }, // Asia/Atyrau
    {  685, 3607,   12109,  5, }, // Asia/Baghdad
    {  685, 5534,  153411,  5, }, // Asia/Bahrain
    {  685, 3653,  152447,  4, }, // Asia/Baku
    {  685, 4466,  153416,  5, }, // Asia/Bangkok
    {  685, 3571,  200228,  5, }, // Asia/Barnaul
    {  685, 4086,  153421,  5, }, // Asia/Beirut
    {  685, 3778,  153426,  5, }, // Asia/Bishkek
    {  685, 5989,  155996,  6, }, // Asia/Brunei
    {  685, 4601,  156002,  4, }, // Asia/Chita
    {  685, 4511,  153437,  6, }, // Asia/Colombo
    {  685, 4540,   12162,  4, }, // Asia/Damascus
    {  685, 1093,  200233,  5, }, // Asia/Dhaka
    {  685, 6192,  200238,  3, }, // Asia/Dili
    {  685, 3596,  200241,  4, }, // Asia/Dubai
    {  685, 6159,  200245,  7, }, // Asia/Dushanbe
    {  685, 8636,  200252,  9, }, // Asia/Famagusta
    {  685, 8651,  104972,  3, }, // Asia/Gaza
    {  685, 4795,  104975,  6, }, // Asia/Hebron
    {  685, 1259,  200261, 13, }, // Asia/Ho_Chi_Minh
    {  685, 2704,  217929,  9, }, // Asia/Hong_Kong
    {  685, 4771,  200274,  6, }, // Asia/Hovd
    {  685, 4184,  200280,  6, }, // Asia/Irkutsk
    {  685, 7006,  200286,  6, }, // Asia/Jakarta
    {  685, 6992,  200292,  7, }, // Asia/Jayapura
    {  685, 1290,  105021,  6, }, // Asia/Jerusalem
    {  685, 3560,  217938,  4, }, // Asia/Kabul
    {  685, 4316,  200299,  7, }, // Asia/Kamchatka
    {  685, 4239,  156093,  5, }, // Asia/Karachi
    {  685, 1185,  217942,  8, }, // Asia/Kathmandu
    {  685, 8661,  200314,  6, }, // Asia/Khandyga
    {  685,  992,  217950,  7, }, // Asia/Kolkata
    {  685, 4197,  200327, 11, }, // Asia/Krasnoyarsk
    {  685, 4859,  200338, 11, }, // Asia/Kuala_Lumpur
    {  685, 5976,  200349,  6, }, // Asia/Kuching
    {  685, 5547,  156146,  4, }, // Asia/Kuwait
    {  685, 1211,  217957,  4, }, // Asia/Macau
    {  685, 4017,  200359,  6, }, // Asia/Magadan
    {  685, 1349,  200365,  5, }, // Asia/Makassar
    {  685, 7182,  153602,  5, }, // Asia/Manila
    {  685, 6920,   12368,  4, }, // Asia/Muscat
    {  685, 2554,  200370,  6, }, // Asia/Nicosia
    {  685, 8675,  200376, 12, }, // Asia/Novokuznetsk
    {  685, 4135,  105149, 11, }, // Asia/Novosibirsk
    {  685, 4229,  105160,  5, }, // Asia/Omsk
    {  685, 7145,  156197,  5, }, // Asia/Oral
    {  685, 6961,  153637,  7, }, // Asia/Phnom_Penh
    {  685, 8693,  200388,  8, }, // Asia/Pontianak
    {  685, 4214,  200396, 10, }, // Asia/Pyongyang
    {  685, 5559,   12436,  3, }, // Asia/Qatar
    {  685, 8708,  200406,  7, }, // Asia/Qostanay
    {  685, 4269,  200413,  9, }, // Asia/Qyzylorda
    {  685, 3584,  105212,  4, }, // Asia/Riyadh
    {  685, 4408,  200422,  6, }, // Asia/Sakhalin
    {  685, 7245,   12468,  6, }, // Asia/Samarkand
    {  685, 3279,  200428,  5, }, // Asia/Seoul
    {  685, 1053,  153685,  7, }, // Asia/Shanghai
    {  685, 3300,  156265,  7, }, // Asia/Singapore
    {  685, 4297,  200433, 13, }, // Asia/Srednekolymsk
    {  685, 3263,  200446,  5, }, // Asia/Taipei
    {  685, 4781,  153718,  6, }, // Asia/Tashkent
    {  685, 3915,  200451,  6, }, // Asia/Tbilisi
    {  685, 2772,  200457,  5, }, // Asia/Tehran
    {  685, 1317,  200462,  5, }, // Asia/Thimphu
    {  685, 2821,  105295,  5, }, // Asia/Tokyo
    {  685, 4572,  200467,  5, }, // Asia/Tomsk
    {  685, 1021,  200472, 11, }, // Asia/Ulaanbaatar
    {  685, 1159,  200483,  7, }, // Asia/Urumqi
    {  685, 8722,  200490,  9, }, // Asia/Ust-Nera
    {  685, 6977,  200499,  8, }, // Asia/Vientiane
    {  685, 4710,  200507, 11, }, // Asia/Vladivostok
    {  685, 4828,  200518,  6, }, // Asia/Yakutsk
    {  685, 1235,  169715,  5, }, // Asia/Yangon
    {  685, 3883,  200524, 11, }, // Asia/Yekaterinburg
    {  685, 3747,  200535,  6, }, // Asia/Yerevan
    {  685, 3663,  200541,  6, }, // Atlantic/Azores
    {  685, 5671,   12615,  6, }, // Atlantic/Bermuda
    {  685, 6628,  155685,  5, }, // Atlantic/Canary
    {  685, 3727,  200547,  8, }, // Atlantic/Cape_Verde
    {  685, 1395,   12638,  4, }, // Atlantic/Faroe
    {  685, 8736,  200555,  6, }, // Atlantic/Madeira
    {  685, 2748,  200561,  7, }, // Atlantic/Reykjavik
    {  685, 7272,  105428, 12, }, // Atlantic/South_Georgia
    {  685, 6821,  200568, 11, }, // Atlantic/St_Helena
    {  685, 6644,  200579,  7, }, // Atlantic/Stanley
    {  685, 1670,  217961,  7, }, // Australia/Adelaide
    {  685, 1635,  105464,  6, }, // Australia/Brisbane
    {  685, 1799,  200593,  9, }, // Australia/Broken_Hill
    {  685, 1583,  200602,  5, }, // Australia/Darwin
    {  685, 3637,  200607,  6, }, // Australia/Eucla
    {  685, 1516,  200613,  6, }, // Australia/Hobart
    {  685, 8753,  200619,  7, }, // Australia/Lindeman
    {  685, 1547,  200626,  9, }, // Australia/Lord_Howe
    {  685, 1727,  156490,  6, }, // Australia/Melbourne
    {  685, 1762,  200635,  4, }, // Australia/Perth
    {  685, 1463,  200639,  4, }, // Australia/Sydney
    {  685, 6441,  200643,  8, }, // Europe/Amsterdam
    {  685, 6216,  200651,  6, }, // Europe/Andorra
    {  685, 3620,  200657,  8, }, // Europe/Astrakhan
    {  685, 2300,  200665,  6, }, // Europe/Athens
    {  685, 6336,   12785,  6, }, // Europe/Belgrade
    {  685, 4757,  153976,  4, }, // Europe/Berlin
    {  685, 6488,  200671, 10, }, // Europe/Bratislava
    {  685, 2167,  169873,  5, }, // Europe/Brussels
    {  685, 3928,  200681,  6, }, // Europe/Bucharest
    {  685, 3806,  200687,  6, }, // Europe/Budapest
    {  685, 8772,  200693,  5, }, // Europe/Busingen
    {  685, 2583,  200698,  7, }, // Europe/Chisinau
    {  685, 6289,  105627,  9, }, // Europe/Copenhagen
    {  685, 2338,  200705,  4, }, // Europe/Dublin
    {  685, 6307,  154033, 10, }, // Europe/Gibraltar
    {  685, 8788,  156595,  5, }, // Europe/Guernsey
    {  685, 6612,  200709,  7, }, // Europe/Helsinki
    {  685, 8804,  105666, 10, }, // Europe/Isle_of_Man
    {  685, 1130,  154067,  7, }, // Europe/Istanbul
    {  685, 8823,  105685,  4, }, // Europe/Jersey
    {  685, 3979,  200716, 11, }, // Europe/Kaliningrad
    {  685, 8929,  154084,  5, }, // Europe/Kirov
    {  685, 2527,  103208,  3, }, // Europe/Kyiv
    {  685, 3213,  169972,  4, }, // Europe/Lisbon
    {  685, 6506,  156639,  9, }, // Europe/Ljubljana
    {  685, 2501,   12926,  4, }, // Europe/London
    {  685, 6365,  200727,  7, }, // Europe/Luxembourg
    {  685, 4892,  200734,  5, }, // Europe/Madrid
    {  685, 6397,  105750,  5, }, // Europe/Malta
    {  685, 6582,  200739,  7, }, // Europe/Mariehamn
    {  685, 3693,  105763,  5, }, // Europe/Minsk
    {  685, 6410,  105768,  6, }, // Europe/Monaco
    {  685, 3537,  105774,  5, }, // Europe/Moscow
    {  685, 6458,  105779,  5, }, // Europe/Oslo
    {  685, 4284,  200746,  4, }, // Europe/Paris
    {  685, 6424,  200750,  9, }, // Europe/Podgorica
    {  685, 6275,  105798,  4, }, // Europe/Prague
    {  685, 8837,  155445,  4, }, // Europe/Riga
    {  685, 6324,   10756,  3, }, // Europe/Rome
    {  685, 4331,  106756,  5, }, // Europe/Samara
    {  685, 6470,  200759, 11, }, // Europe/San_Marino
    {  685, 6245,  170041,  7, }, // Europe/Sarajevo
    {  685, 4451,  200770,  6, }, // Europe/Saratov
    {  685, 8849,  170053,  8, }, // Europe/Simferopol
    {  685, 6383,  200776,  7, }, // Europe/Skopje
    {  685, 6599,  200783,  5, }, // Europe/Sofia
    {  685, 6523,  200788,  9, }, // Europe/Stockholm
    {  685, 8867,  200797,  4, }, // Europe/Tallinn
    {  685, 6202,  200801,  6, }, // Europe/Tirane
    {  685, 8882,  200807,  8, }, // Europe/Ulyanovsk
    {  685, 6352,  200815,  5, }, // Europe/Vaduz
    {  685, 6567,  200820,  6, }, // Europe/Vatican
    {  685, 6231,  154224,  5, }, // Europe/Vienna
    {  685, 8899,  200826,  7, }, // Europe/Vilnius
    {  685, 4727,  170109,  9, }, // Europe/Volgograd
    {  685, 3190,  105924,  5, }, // Europe/Warsaw
    {  685, 6261,  200833,  6, }, // Europe/Zagreb
    {  685, 6540,  200839,  5, }, // Europe/Zurich
    {  685, 5078,  200844, 12, }, // Indian/Antananarivo
    {  685, 6947,  105952,  5, }, // Indian/Chagos
    {  685, 6046,  200856,  5, }, // Indian/Christmas
    {  685, 6063,  105964,  5, }, // Indian/Cocos
    {  685, 5029,  105969,  6, }, // Indian/Comoro
    {  685, 6661,  200861,  8, }, // Indian/Kerguelen
    {  685, 7260,  105983,  4, }, // Indian/Mahe
    {  685, 7050,  200869,  6, }, // Indian/Maldives
    {  685, 4069,  200875,  6, }, // Indian/Mauritius
    {  685, 5098,  200881,  5, }, // Indian/Mayotte
    {  685, 7211,  200886,  8, }, // Indian/Reunion
    {  685, 4422,  154313,  4, }, // Pacific/Apia
    {  685,  932,  106021,  6, }, // Pacific/Auckland
    {  685, 3706,  200894,  8, }, // Pacific/Bougainville
    {  685, 3015,  200902,  5, }, // Pacific/Chatham
    {  685, 3157,  200907,  4, }, // Pacific/Chuuk
    {  685, 2237,  105549,  5, }, // Pacific/Easter
    {  685, 7360,  200911,  5, }, // Pacific/Efate
    {  685, 7327,  200916,  6, }, // Pacific/Fakaofo
    {  685, 3902,  200922,  3, }, // Pacific/Fiji
    {  685, 7343,  200925, 10, }, // Pacific/Funafuti
    {  685, 6678,  200935,  9, }, // Pacific/Galapagos
    {  685, 6696,  200944,  7, }, // Pacific/Gambier
    {  685, 3822,  200951, 11, }, // Pacific/Guadalcanal
    {  685, 6018,  199694,  4, }, // Pacific/Guam
    {  685, 3049,  200970,  8, }, // Pacific/Kanton
    {  685, 3998,  200978,  9, }, // Pacific/Kiritimati
    {  685, 7019,  200987,  7, }, // Pacific/Kosrae
    {  685, 2842,  156945,  8, }, // Pacific/Kwajalein
    {  685, 4877,  154416,  5, }, // Pacific/Majuro
    {  685, 4051,  200994,  8, }, // Pacific/Marquesas
    {  685, 8914,  201002,  4, }, // Pacific/Midway
    {  685, 7084,  201006,  5, }, // Pacific/Nauru
    {  685, 7132,  199633,  4, }, // Pacific/Niue
    {  685, 4168,  106166,  6, }, // Pacific/Norfolk
    {  685, 7098,  201011,  5, }, // Pacific/Noumea
    {  685, 3126,  170314,  9, }, // Pacific/Pago_Pago
    {  685, 7155,  201016,  4, }, // Pacific/Palau
    {  685, 7194,  201020,  7, }, // Pacific/Pitcairn
    {  685, 3096,  201027,  7, }, // Pacific/Pohnpei
    {  685, 4807,  201034, 12, }, // Pacific/Port_Moresby
    {  685, 6076,  106215,  9, }, // Pacific/Rarotonga
    {  685, 6031,  201046,  6, }, // Pacific/Saipan
    {  685, 7312,  201052,  6, }, // Pacific/Tahiti
    {  685, 6712,  201058,  5, }, // Pacific/Tarawa
    {  685, 4583,  201063,  9, }, // Pacific/Tongatapu
    {  685, 7392,  106252,  3, }, // Pacific/Wake
    {  685, 7405,  201072,  5, }, // Pacific/Wallis
    {  686, 6788,  217968,  6, }, // Africa/Abidjan Anii/Latin/Benin
    {  686, 6760,    4649,  5, }, // Africa/Accra
    {  686, 5059,  217974, 10, }, // Africa/Addis_Ababa
    {  686, 7420,  217984,  6, }, // Africa/Algiers
    {  686,   14,  217990,  7, }, // Africa/Asmara
    {  686,   44,  217997,  7, }, // Africa/Bamako
    {  686, 5257,  218004,  6, }, // Africa/Bangui
    {  686, 7435,  218010,  5, }, // Africa/Bissau
    {  686, 4957,  218015,  8, }, // Africa/Blantyre
    {  686, 5287,  218023,  8, }, // Africa/Brazzaville
    {  686, 4922,  218031, 10, }, // Africa/Bujumbura
    {  686, 2320,  218041,  6, }, // Africa/Cairo
    {  686, 4117,  218047, 11, }, // Africa/Casablanca
    {  686, 7449,  218058,  6, }, // Africa/Ceuta
    {  686, 6773,  218064,  8, }, // Africa/Conakry
    {  686, 6840,  218072,  6, }, // Africa/Dakar
    {  686, 5130,  218078, 10, }, // Africa/Dar_es_Salaam
    {  686, 5043,  218088,  7, }, // Africa/Djibouti
    {  686, 5243,  218095,  7, }, // Africa/Douala
    {  686, 5166,  218102,  7, }, // Africa/El_Aaiun
    {  686, 6853,  218109,  8, }, // Africa/Freetown
    {  686, 5015,  218117,  7, }, // Africa/Harare
    {  686, 4479,  218124, 11, }, // Africa/Johannesburg
    {  686, 4499,  218135,  5, }, // Africa/Juba
    {  686, 5151,  218140,  8, }, // Africa/Kampala
    {  686, 4524,  218148,  7, }, // Africa/Khartoum
    {  686, 4973,  205370,  7, }, // Africa/Kigali
    {  686, 5306,  218155,  9, }, // Africa/Kinshasa
    {  686, 4744,    4745,  5, }, // Africa/Lagos
    {  686, 5336,  218164,  8, }, // Africa/Libreville
    {  686, 6869,  218172,  4, }, // Africa/Lome
    {  686, 5211,  218176,  8, }, // Africa/Luanda
    {  686, 4939,  205421, 11, }, // Africa/Lubumbashi
    {  686, 5001,  218184,  7, }, // Africa/Lusaka
    {  686, 5322,  218191,  7, }, // Africa/Malabo
    {  686, 4987,  218198,  7, }, // Africa/Maputo
    {  686, 5197,  218205,  7, }, // Africa/Maseru
    {  686, 5182,  218212,  8, }, // Africa/Mbabane
    {  686, 5113,  218220, 10, }, // Africa/Mogadishu
    {  686, 7034,  218230, 10, }, // Africa/Monrovia
    {  686, 3853,  205490,  8, }, // Africa/Nairobi
    {  686, 5271,  218240,  8, }, // Africa/Ndjamena
    {  686, 5354,  218248,  5, }, // Africa/Niamey
    {  686, 6803,  218253,  8, }, // Africa/Nouakchott
    {  686, 6727,  218261,  8, }, // Africa/Ouagadougou
    {  686, 5225,  218269, 11, }, // Africa/Porto-Novo
    {  686, 4435,  218280, 13, }, // Africa/Sao_Tome
    {  686, 2866,  205564,  8, }, // Africa/Tripoli
    {  686, 6554,  218293,  6, }, // Africa/Tunis
    {  686, 4152,  218299,  7, }, // Africa/Windhoek
    {  686,  132,  218306,  4, }, // America/Adak
    {  686, 3346,  218310,  9, }, // America/Anchorage
    {  686, 5607,  218319,  9, }, // America/Anguilla
    {  686, 5624,  218328,  9, }, // America/Antigua
    {  686, 4554,  218337, 10, }, // America/Araguaina
    {  686,  166,  218347, 13, }, // America/Argentina/Buenos_Aires
    {  686,   91,  218360, 10, }, // America/Argentina/Catamarca
    {  686,  270,  218370,  8, }, // America/Argentina/Cordoba
    {  686,  441,  218378,  6, }, // America/Argentina/Jujuy
    {  686, 7462,  218384,  9, }, // America/Argentina/La_Rioja
    {  686,  565,  218393,  8, }, // America/Argentina/Mendoza
    {  686, 7489,  218401, 12, }, // America/Argentina/Rio_Gallegos
    {  686, 7544,  218413,  9, }, // America/Argentina/San_Juan
    {  686, 5580,  175046,  9, }, // America/Argentina/San_Luis
    {  686, 7571,    4842,  7, }, // America/Argentina/Tucuman
    {  686, 7597,  218422,  9, }, // America/Argentina/Ushuaia
    {  686, 5640,  218431,  6, }, // America/Aruba
    {  686, 4252,  218437,  9, }, // America/Asuncion
    {  686,  237,      22,  8, }, // America/Atikokan
    {  686, 3679,  218446,  7, }, // America/Bahia
    {  686, 7623,  218453, 21, }, // America/Bahia_Banderas
    {  686, 5654,  218474, 10, }, // America/Barbados
    {  686, 7646,  218484,  5, }, // America/Belem
    {  686, 5407,  175111,  6, }, // America/Belize
    {  686, 7660,  218489, 11, }, // America/Blanc-Sablon
    {  686, 7681,  218500, 11, }, // America/Boa_Vista
    {  686, 7699,  218511,  7, }, // America/Boise
    {  686, 7713,  218518, 13, }, // America/Cambridge_Bay
    {  686, 7735,  218531, 13, }, // America/Campo_Grande
    {  686, 3868,  218544,  6, }, // America/Cancun
    {  686, 4694,  175176,  8, }, // America/Caracas
    {  686, 4345,  218550,  6, }, // America/Cayenne
    {  686, 5496,  218556,  8, }, // America/Cayman
    {  686, 2260,  218564,  8, }, // America/Chicago
    {  686, 7756,  218572,  8, }, // America/Chihuahua
    {  686, 7774,  218580, 16, }, // America/Ciudad_Juarez
    {  686, 5422,  218596, 11, }, // America/Costa_Rica
    {  686, 7796,  218607,  7, }, // America/Creston
    {  686, 3791,  218614,  7, }, // America/Cuiaba
    {  686, 5723,  218621,  9, }, // America/Curacao
    {  686, 7812,  218630, 13, }, // America/Danmarkshavn
    {  686, 7833,  218643,  6, }, // America/Dawson
    {  686, 7848,  218649, 12, }, // America/Dawson_Creek
    {  686,  805,  218661,  5, }, // America/Denver
    {  686, 3465,  218666,  7, }, // America/Detroit
    {  686, 5739,  218673,  9, }, // America/Dominica
    {  686,  893,  218682,  8, }, // America/Edmonton
    {  686, 7869,  218690,  9, }, // America/Eirunepe
    {  686, 5441,  218699, 12, }, // America/El_Salvador
    {  686, 7886,  218711, 11, }, // America/Fort_Nelson
    {  686, 7906,  218722, 10, }, // America/Fortaleza
    {  686, 7924,  218732,  9, }, // America/Glace_Bay
    {  686, 6881,  218741,  8, }, // America/Goose_Bay
    {  686, 4612,  218749, 10, }, // America/Grand_Turk
    {  686, 5770,  218759,  8, }, // America/Grenada
    {  686, 5786,  218767, 11, }, // America/Guadeloupe
    {  686, 3760,  218778, 11, }, // America/Guatemala
    {  686, 4841,  218789,  9, }, // America/Guayaquil
    {  686, 6932,  218798,  7, }, // America/Guyana
    {  686, 1939,  218805,  8, }, // America/Halifax
    {  686, 2281,  218813,  7, }, // America/Havana
    {  686, 7942,  218820,  9, }, // America/Hermosillo
    {  686,  348,  218829, 13, }, // America/Indiana/Indianapolis
    {  686,  481,  218842, 14, }, // America/Indiana/Knox
    {  686, 7961,  218856, 18, }, // America/Indiana/Marengo
    {  686, 7985,  218874, 21, }, // America/Indiana/Petersburg
    {  686, 8012,  218895, 19, }, // America/Indiana/Tell_City
    {  686, 8038,  218914, 16, }, // America/Indiana/Vevay
    {  686, 8060,  218930, 17, }, // America/Indiana/Vincennes
    {  686, 8086,  218947, 19, }, // America/Indiana/Winamac
    {  686, 8110,  218966,  6, }, // America/Inuvik
    {  686,  660,  218972,  9, }, // America/Iqaluit
    {  686, 2799,  218981,  7, }, // America/Jamaica
    {  686, 5380,  218988,  5, }, // America/Juneau
    {  686,  521,  218993,  8, }, // America/Kentucky/Louisville
    {  686, 8125,  219001, 20, }, // America/Kentucky/Monticello
    {  686, 5704,  219021, 10, }, // America/Kralendijk
    {  686, 4376,   21873,  6, }, // America/La_Paz
    {  686, 7169,  219031,  5, }, // America/Lima
    {  686, 3239,  219036, 12, }, // America/Los_Angeles
    {  686, 5932,  219048, 17, }, // America/Lower_Princes
    {  686, 8153,  219065,  8, }, // America/Maceio
    {  686, 8168,  175722,  9, }, // America/Managua
    {  686, 1908,  219073,  6, }, // America/Manaus
    {  686, 5897,  219079,  7, }, // America/Marigot
    {  686, 5805,  219086, 10, }, // America/Martinique
    {  686, 8184,  219096, 10, }, // America/Matamoros
    {  686, 2917,  219106,  9, }, // America/Mazatlan
    {  686, 8220,  219115,  7, }, // America/Merida
    {  686, 8235,  219122, 13, }, // America/Metlakatla
    {  686, 2949,  219135, 11, }, // America/Mexico_City
    {  686, 4391,  219146,  7, }, // America/Miquelon
    {  686, 8254,  219153,  6, }, // America/Moncton
    {  686, 8270,  219159,  8, }, // America/Monterrey
    {  686, 4098,  219167, 12, }, // America/Montevideo
    {  686, 5824,  219179, 10, }, // America/Montserrat
    {  686, 5481,  125812,  5, }, // America/Nassau
    {  686, 2379,  219189,  8, }, // America/New_York
    {  686, 1850,  219197,  7, }, // America/Noronha
    {  686, 8301,  219204, 19, }, // America/North_Dakota/Beulah
    {  686, 8329,  219223, 19, }, // America/North_Dakota/Center
    {  686, 8357,  219242, 22, }, // America/North_Dakota/New_Salem
    {  686,  393,     317,  4, }, // America/Nuuk
    {  686, 8388,  175952,  8, }, // America/Ojinaga
    {  686, 2356,  219264,  7, }, // America/Panama
    {  686, 6173,  219271, 11, }, // America/Paramaribo
    {  686, 2973,  219282,  6, }, // America/Phoenix
    {  686, 3945,  219288, 12, }, // America/Port-au-Prince
    {  686, 5954,  219300, 13, }, // America/Port_of_Spain
    {  686, 8404,  219313, 11, }, // America/Porto_Velho
    {  686, 5843,  219324, 11, }, // America/Puerto_Rico
    {  686, 8424,  219335, 14, }, // America/Rankin_Inlet
    {  686, 8445,  166047,  6, }, // America/Recife
    {  686, 1995,  219349,  7, }, // America/Regina
    {  686, 8460,  219356,  8, }, // America/Resolute
    {  686,  695,  219364, 12, }, // America/Rio_Branco
    {  686, 8477,  219376,  8, }, // America/Santarem
    {  686, 2201,  219384, 10, }, // America/Santiago
    {  686, 6111,  219394, 13, }, // America/Santo_Domingo
    {  686, 1878,  219407, 10, }, // America/Sao_Paulo
    {  686, 6899,  219417, 14, }, // America/Scoresbysund
    {  686, 8494,  219431,  6, }, // America/Sitka
    {  686, 8508,  219437, 14, }, // America/St_Barthelemy
    {  686, 2061,  219451,  9, }, // America/St_Johns
    {  686, 5863,  219460,  8, }, // America/St_Kitts
    {  686, 5880,  219468, 10, }, // America/St_Lucia
    {  686,  855,  219478, 11, }, // America/St_Thomas
    {  686, 5913,  219489, 10, }, // America/St_Vincent
    {  686, 8530,  219499, 16, }, // America/Swift_Current
    {  686, 5461,    5528, 11, }, // America/Tegucigalpa
    {  686, 5756,  219515,  5, }, // America/Thule
    {  686,  313,  219520,  8, }, // America/Tijuana
    {  686,  608,  219528,  8, }, // America/Toronto
    {  686, 5688,  219536,  8, }, // America/Tortola
    {  686, 2093,  219544,  8, }, // America/Vancouver
    {  686, 2144,  219552,  8, }, // America/Whitehorse
    {  686,  734,  219560,  8, }, // America/Winnipeg
    {  686, 6001,  219568,  5, }, // Antarctica/Casey
    {  686, 6094,  219573,  5, }, // Antarctica/Davis
    {  686, 6133,  219578, 12, }, // Antarctica/DumontDUrville
    {  686, 8568,  219590,  7, }, // Antarctica/Macquarie
    {  686, 7066,  219597,  5, }, // Antarctica/Mawson
    {  686, 7113,  219602,  9, }, // Antarctica/McMurdo
    {  686, 8589,  219611,  7, }, // Antarctica/Palmer
    {  686, 7226,  219618,  7, }, // Antarctica/Rothera
    {  686, 7295,  219625,  7, }, // Antarctica/Syowa
    {  686, 8607,  219632,  4, }, // Antarctica/Troll
    {  686, 7374,  219636,  6, }, // Antarctica/Vostok
    {  686, 1429,  219642, 12, }, // Arctic/Longyearbyen
    {  686, 5570,  219654,  4, }, // Asia/Aden
    {  686, 5395,  219658,  7, }, // Asia/Almaty
    {  686, 3968,  219665,  5, }, // Asia/Amman
    {  686, 5511,  219670,  7, }, // Asia/Anadyr
    {  686, 5523,  219677,  5, }, // Asia/Aqtau
    {  686, 5368,  219682,  7, }, // Asia/Aqtobe
    {  686,  964,  219689,  9, }, // Asia/Ashgabat
    {  686, 8624,  219698,  6, }, // Asia/Atyrau
    {  686, 3607,  219704,  7, }, // Asia/Baghdad
    {  686, 5534,  219711,  6, }, // Asia/Bahrain
    {  686, 3653,   63599,  5, }, // Asia/Baku
    {  686, 4466,  219717,  6, }, // Asia/Bangkok
    {  686, 3571,  219723,  8, }, // Asia/Barnaul
    {  686, 4086,  219731,  7, }, // Asia/Beirut
    {  686, 3778,  219738,  7, }, // Asia/Bishkek
    {  686, 5989,  219745,  6, }, // Asia/Brunei
    {  686, 4601,  219751,  6, }, // Asia/Chita
    {  686, 4511,  219757,  8, }, // Asia/Colombo
    {  686, 4540,  219765, 10, }, // Asia/Damascus
    {  686, 1093,  218072,  5, }, // Asia/Dhaka
    {  686, 6192,  219775,  5, }, // Asia/Dili
    {  686, 3596,  219780,  5, }, // Asia/Dubai
    {  686, 6159,  219785,  9, }, // Asia/Dushanbe
    {  686, 8636,  219794, 10, }, // Asia/Famagusta
    {  686, 8651,  219804,  5, }, // Asia/Gaza
    {  686, 4795,  219809,  6, }, // Asia/Hebron
    {  686, 1259,  219815, 15, }, // Asia/Ho_Chi_Minh
    {  686, 2704,  219830,  7, }, // Asia/Hong_Kong
    {  686, 4771,  219837,  5, }, // Asia/Hovd
    {  686, 4184,   62345,  5, }, // Asia/Irkutsk
    {  686, 7006,  219842,  7, }, // Asia/Jakarta
    {  686, 6992,  219849,  9, }, // Asia/Jayapura
    {  686, 1290,  219858,  9, }, // Asia/Jerusalem
    {  686, 4316,  219867, 10, }, // Asia/Kamchatka
    {  686, 4239,  219877,  6, }, // Asia/Karachi
    {  686, 1185,  219883,  9, }, // Asia/Kathmandu
    {  686, 8661,  219892,  8, }, // Asia/Khandyga
    {  686,  992,  219900,  7, }, // Asia/Kolkata
    {  686, 4197,  219907, 10, }, // Asia/Krasnoyarsk
    {  686, 4859,  219917, 14, }, // Asia/Kuala_Lumpur
    {  686, 5976,  219931,  5, }, // Asia/Kuching
    {  686, 5547,  219936,  7, }, // Asia/Kuwait
    {  686, 1211,  219943,  7, }, // Asia/Macau
    {  686, 4017,  219950,  7, }, // Asia/Magadan
    {  686, 1349,  219957,  8, }, // Asia/Makassar
    {  686, 7182,  219965,  7, }, // Asia/Manila
    {  686, 6920,  219972,  7, }, // Asia/Muscat
    {  686, 2554,   22510,  8, }, // Asia/Nicosia
    {  686, 8675,  219979, 10, }, // Asia/Novokuznetsk
    {  686, 4135,  219989, 10, }, // Asia/Novosibirsk
    {  686, 4229,  219999,  6, }, // Asia/Omsk
    {  686, 7145,  220005,  5, }, // Asia/Oral
    {  686, 6961,  220010,  7, }, // Asia/Phnom_Penh
    {  686, 8693,  220017, 10, }, // Asia/Pontianak
    {  686, 4214,  220027,  8, }, // Asia/Pyongyang
    {  686, 5559,  215491,  5, }, // Asia/Qatar
    {  686, 8708,  220035,  8, }, // Asia/Qostanay
    {  686, 4269,  220043, 10, }, // Asia/Qyzylorda
    {  686, 3584,  220053,  6, }, // Asia/Riyadh
    {  686, 4408,    7791,  7, }, // Asia/Sakhalin
    {  686, 7245,  220059,  9, }, // Asia/Samarkand
    {  686, 3279,  220068,  5, }, // Asia/Seoul
    {  686, 1053,  220073,  7, }, // Asia/Shanghai
    {  686, 3300,  220080,  9, }, // Asia/Singapore
    {  686, 4297,  220089, 11, }, // Asia/Srednekolymsk
    {  686, 3263,  220100,  6, }, // Asia/Taipei
    {  686, 4781,  220106,  8, }, // Asia/Tashkent
    {  686, 3915,  220114,  8, }, // Asia/Tbilisi
    {  686, 2772,  220122,  7, }, // Asia/Tehran
    {  686, 1317,  220129,  5, }, // Asia/Thimphu
    {  686, 2821,  220134,  7, }, // Asia/Tokyo
    {  686, 4572,  220141,  7, }, // Asia/Tomsk
    {  686, 1021,  220148, 10, }, // Asia/Ulaanbaatar
    {  686, 1159,  220158,  7, }, // Asia/Urumqi
    {  686, 8722,  220165,  9, }, // Asia/Ust-Nera
    {  686, 6977,  220174, 10, }, // Asia/Vientiane
    {  686, 4710,  220184, 11, }, // Asia/Vladivostok
    {  686, 4828,  176273,  5, }, // Asia/Yakutsk
    {  686, 1235,  220195,  6, }, // Asia/Yangon
    {  686, 3883,  220201, 12, }, // Asia/Yekaterinburg
    {  686, 3747,  220213,  8, }, // Asia/Yerevan
    {  686, 3663,  220221,  5, }, // Atlantic/Azores
    {  686, 5671,  220226,  8, }, // Atlantic/Bermuda
    {  686, 6628,  220234,  7, }, // Atlantic/Canary
    {  686, 3727,  220241,  7, }, // Atlantic/Cape_Verde
    {  686, 1395,  220248,  5, }, // Atlantic/Faroe
    {  686, 8736,  220253,  8, }, // Atlantic/Madeira
    {  686, 2748,  220261,  9, }, // Atlantic/Reykjavik
    {  686, 7272,  220270, 25, }, // Atlantic/South_Georgia
    {  686, 6821,  220295, 10, }, // Atlantic/St_Helena
    {  686, 6644,  220305,  7, }, // Atlantic/Stanley
    {  686, 1670,  220312,  7, }, // Australia/Adelaide
    {  686, 1635,  220319,  7, }, // Australia/Brisbane
    {  686, 1799,  220326, 10, }, // Australia/Broken_Hill
    {  686, 1583,  220336,  6, }, // Australia/Darwin
    {  686, 3637,  220342,  5, }, // Australia/Eucla
    {  686, 1516,  220347,  5, }, // Australia/Hobart
    {  686, 8753,  220352,  8, }, // Australia/Lindeman
    {  686, 1547,  220360,  8, }, // Australia/Lord_Howe
    {  686, 1727,  220368,  7, }, // Australia/Melbourne
    {  686, 1762,  220375,  4, }, // Australia/Perth
    {  686, 1463,  220379,  6, }, // Australia/Sydney
    {  686, 6441,  220385,  9, }, // Europe/Amsterdam
    {  686, 6216,  220394,  7, }, // Europe/Andorra
    {  686, 3620,  220401,  9, }, // Europe/Astrakhan
    {  686, 2300,  220410,  4, }, // Europe/Athens
    {  686, 6336,  220414,  8, }, // Europe/Belgrade
    {  686, 4757,  220422,  6, }, // Europe/Berlin
    {  686, 6488,  220428, 10, }, // Europe/Bratislava
    {  686, 2167,  220438,  6, }, // Europe/Brussels
    {  686, 3928,  220444,  7, }, // Europe/Bucharest
    {  686, 3806,  220451,  7, }, // Europe/Budapest
    {  686, 8772,  220458,  8, }, // Europe/Busingen
    {  686, 2583,  220466,  8, }, // Europe/Chisinau
    {  686, 6289,  220474,  8, }, // Europe/Copenhagen
    {  686, 2338,  220482,  6, }, // Europe/Dublin
    {  686, 6307,  215606,  9, }, // Europe/Gibraltar
    {  686, 8788,  220488,  8, }, // Europe/Guernsey
    {  686, 6612,  220496,  9, }, // Europe/Helsinki
    {  686, 8804,  220505, 16, }, // Europe/Isle_of_Man
    {  686, 1130,  220521,  9, }, // Europe/Istanbul
    {  686, 8823,  214792,  6, }, // Europe/Jersey
    {  686, 3979,  220530, 12, }, // Europe/Kaliningrad
    {  686, 8929,  220542,  6, }, // Europe/Kirov
    {  686, 2527,  220548,  5, }, // Europe/Kyiv
    {  686, 3213,    6009,  6, }, // Europe/Lisbon
    {  686, 6506,  220553,  9, }, // Europe/Ljubljana
    {  686, 2501,  220562,  6, }, // Europe/London
    {  686, 6365,  220568,  9, }, // Europe/Luxembourg
    {  686, 4892,  220577,  7, }, // Europe/Madrid
    {  686, 6582,  220584,  9, }, // Europe/Mariehamn
    {  686, 3693,  220593,  4, }, // Europe/Minsk
    {  686, 6410,  220597,  7, }, // Europe/Monaco
    {  686, 3537,  220604,  6, }, // Europe/Moscow
    {  686, 6458,  220610,  5, }, // Europe/Oslo
    {  686, 4284,   63839,  5, }, // Europe/Paris
    {  686, 6424,  220615, 10, }, // Europe/Podgorica
    {  686, 6275,    4533,  5, }, // Europe/Prague
    {  686, 8837,  220625,  5, }, // Europe/Riga
    {  686, 6324,   63849,  4, }, // Europe/Rome
    {  686, 4331,  220630,  7, }, // Europe/Samara
    {  686, 6470,  220637, 11, }, // Europe/San_Marino
    {  686, 6245,  220648,  9, }, // Europe/Sarajevo
    {  686, 4451,  220657,  8, }, // Europe/Saratov
    {  686, 6383,  220665,  6, }, // Europe/Skopje
    {  686, 6599,   22919,  6, }, // Europe/Sofia
    {  686, 6523,  220671,  8, }, // Europe/Stockholm
    {  686, 8867,    8250,  5, }, // Europe/Tallinn
    {  686, 6202,  220679,  7, }, // Europe/Tirane
    {  686, 8882,  220686,  8, }, // Europe/Ulyanovsk
    {  686, 6352,  220694,  5, }, // Europe/Vaduz
    {  686, 6567,  220699,  7, }, // Europe/Vatican
    {  686, 6231,  220706,  6, }, // Europe/Vienna
    {  686, 8899,  220712,  8, }, // Europe/Vilnius
    {  686, 4727,  220720, 10, }, // Europe/Volgograd
    {  686, 3190,  220730,  6, }, // Europe/Warsaw
    {  686, 6261,  220736,  6, }, // Europe/Zagreb
    {  686, 6540,  220742,  6, }, // Europe/Zurich
    {  686, 5078,  220748, 13, }, // Indian/Antananarivo
    {  686, 6947,  220761,  6, }, // Indian/Chagos
    {  686, 6046,  220767,  5, }, // Indian/Christmas
    {  686, 6063,  220772,  6, }, // Indian/Cocos
    {  686, 5029,  220778,  7, }, // Indian/Comoro
    {  686, 6661,  220785,  9, }, // Indian/Kerguelen
    {  686, 7260,  220794,  5, }, // Indian/Mahe
    {  686, 7050,  220799,  8, }, // Indian/Maldives
    {  686, 4069,  220807,  6, }, // Indian/Mauritius
    {  686, 5098,  220813,  7, }, // Indian/Mayotte
    {  686, 7211,  220820,  8, }, // Indian/Reunion
    {  686, 4422,  215086,  5, }, // Pacific/Apia
    {  686,  932,  220828,  7, }, // Pacific/Auckland
    {  686, 3706,  220835,  9, }, // Pacific/Bougainville
    {  686, 3015,  220844,  6, }, // Pacific/Chatham
    {  686, 3157,  220850,  4, }, // Pacific/Chuuk
    {  686, 2237,   57447,  4, }, // Pacific/Easter
    {  686, 7360,  220854,  6, }, // Pacific/Efate
    {  686, 7327,  220860,  9, }, // Pacific/Fakaofo
    {  686, 7343,  208954,  9, }, // Pacific/Funafuti
    {  686, 6678,    6198,  9, }, // Pacific/Galapagos
    {  686, 6696,  220869,  8, }, // Pacific/Gambier
    {  686, 3822,  220877, 12, }, // Pacific/Guadalcanal
    {  686, 6018,  177429,  5, }, // Pacific/Guam
    {  686, 3049,  220889,  6, }, // Pacific/Kanton
    {  686, 3998,  220895,  9, }, // Pacific/Kiritimati
    {  686, 7019,  220904,  6, }, // Pacific/Kosrae
    {  686, 2842,  220910,  9, }, // Pacific/Kwajalein
    {  686, 4877,  220919,  7, }, // Pacific/Majuro
    {  686, 4051,   23109,  8, }, // Pacific/Marquesas
    {  686, 8914,  220926,  6, }, // Pacific/Midway
    {  686, 7084,  220932,  6, }, // Pacific/Nauru
    {  686, 7132,  220938,  6, }, // Pacific/Niue
    {  686, 4168,  220944,  7, }, // Pacific/Norfolk
    {  686, 7098,  215240,  6, }, // Pacific/Noumea
    {  686, 3126,  220951, 10, }, // Pacific/Pago_Pago
    {  686, 7155,  220961,  7, }, // Pacific/Palau
    {  686, 7194,  220968,  7, }, // Pacific/Pitcairn
    {  686, 3096,  220975,  7, }, // Pacific/Pohnpei
    {  686, 4807,  220982, 13, }, // Pacific/Port_Moresby
    {  686, 6076,  220995,  9, }, // Pacific/Rarotonga
    {  686, 6031,  221004,  6, }, // Pacific/Saipan
    {  686, 7312,  209182,  7, }, // Pacific/Tahiti
    {  686, 6712,  221010,  7, }, // Pacific/Tarawa
    {  686, 4583,  221017, 10, }, // Pacific/Tongatapu
    {  686, 7392,  221027,  4, }, // Pacific/Wake
    {  686, 7405,  177552,  5, }, // Pacific/Wallis
    {  687, 6788,   29964,  7, }, // Africa/Abidjan Kangri/Devanagari/India
    {  687, 6760,   88252,  5, }, // Africa/Accra
    {  687, 5059,   88257, 10, }, // Africa/Addis_Ababa
    {  687, 7420,   88267,  9, }, // Africa/Algiers
    {  687,   14,   88276,  7, }, // Africa/Asmara
    {  687,   44,  128969,  5, }, // Africa/Bamako
    {  687, 5257,   88289,  6, }, // Africa/Bangui
    {  687, 6746,   88295,  6, }, // Africa/Banjul
    {  687, 7435,   88301,  5, }, // Africa/Bissau
    {  687, 4957,   88306,  9, }, // Africa/Blantyre
    {  687, 5287,   88315, 11, }, // Africa/Brazzaville
    {  687, 4922,   88326,  9, }, // Africa/Bujumbura
    {  687, 2320,   88335,  5, }, // Africa/Cairo
    {  687, 4117,   30065, 11, }, // Africa/Casablanca
    {  687, 7449,   88340,  5, }, // Africa/Ceuta
    {  687, 6773,   88345,  8, }, // Africa/Conakry
    {  687, 6840,   88353,  4, }, // Africa/Dakar
    {  687, 5130,   88357, 11, }, // Africa/Dar_es_Salaam
    {  687, 5043,   88368,  6, }, // Africa/Djibouti
    {  687, 5243,   88374,  5, }, // Africa/Douala
    {  687, 5166,  221031,  9, }, // Africa/El_Aaiun
    {  687, 6853,   88387,  9, }, // Africa/Freetown
    {  687, 4906,   88396,  7, }, // Africa/Gaborone
    {  687, 5015,   88403,  5, }, // Africa/Harare
    {  687, 4479,   88408, 10, }, // Africa/Johannesburg
    {  687, 4499,   30154,  4, }, // Africa/Juba
    {  687, 5151,   88418,  6, }, // Africa/Kampala
    {  687, 4524,   30166,  7, }, // Africa/Khartoum
    {  687, 4973,   88424,  6, }, // Africa/Kigali
    {  687, 5306,   88430,  7, }, // Africa/Kinshasa
    {  687, 4744,   88437,  5, }, // Africa/Lagos
    {  687, 5336,   88442, 10, }, // Africa/Libreville
    {  687, 6869,   88452,  3, }, // Africa/Lome
    {  687, 5211,   88455,  6, }, // Africa/Luanda
    {  687, 4939,   88461,  9, }, // Africa/Lubumbashi
    {  687, 5001,   30221,  6, }, // Africa/Lusaka
    {  687, 5322,   88470,  5, }, // Africa/Malabo
    {  687, 4987,   88475,  6, }, // Africa/Maputo
    {  687, 5197,   30239,  6, }, // Africa/Maseru
    {  687, 5182,   88481,  7, }, // Africa/Mbabane
    {  687, 5113,   88488,  8, }, // Africa/Mogadishu
    {  687, 7034,   88496,  9, }, // Africa/Monrovia
    {  687, 3853,   88505,  6, }, // Africa/Nairobi
    {  687, 5271,   88511, 10, }, // Africa/Ndjamena
    {  687, 5354,   88521,  6, }, // Africa/Niamey
    {  687, 6803,   88527,  7, }, // Africa/Nouakchott
    {  687, 6727,   88534,  7, }, // Africa/Ouagadougou
    {  687, 5225,   88541, 11, }, // Africa/Porto-Novo
    {  687, 4435,   88552,  7, }, // Africa/Sao_Tome
    {  687, 2866,   88559,  8, }, // Africa/Tripoli
    {  687, 6554,   30328,  7, }, // Africa/Tunis
    {  687, 4152,   88567,  7, }, // Africa/Windhoek
    {  687,  132,   88574,  3, }, // America/Adak
    {  687, 3346,   30347,  6, }, // America/Anchorage
    {  687, 5607,   88577,  8, }, // America/Anguilla
    {  687, 5624,   88585,  7, }, // America/Antigua
    {  687, 4554,   88592,  9, }, // America/Araguaina
    {  687,  166,   88601, 11, }, // America/Argentina/Buenos_Aires
    {  687,   91,   30390, 10, }, // America/Argentina/Catamarca
    {  687,  270,   88612,  8, }, // America/Argentina/Cordoba
    {  687,  441,   88620,  5, }, // America/Argentina/Jujuy
    {  687, 7462,   88625,  8, }, // America/Argentina/La_Rioja
    {  687,  565,   88633,  8, }, // America/Argentina/Mendoza
    {  687, 7489,   88641, 12, }, // America/Argentina/Rio_Gallegos
    {  687, 7520,   30437,  6, }, // America/Argentina/Salta
    {  687, 7544,   88653,  9, }, // America/Argentina/San_Juan
    {  687, 5580,   88662,  7, }, // America/Argentina/San_Luis
    {  687, 7571,   88669,  6, }, // America/Argentina/Tucuman
    {  687, 7597,   88675,  7, }, // America/Argentina/Ushuaia
    {  687, 5640,   88682,  5, }, // America/Aruba
    {  687, 4252,   88687,  8, }, // America/Asuncion
    {  687,  237,   88695,  8, }, // America/Atikokan
    {  687, 3679,   88703,  5, }, // America/Bahia
    {  687, 7623,   88708, 15, }, // America/Bahia_Banderas
    {  687, 5654,   88723,  8, }, // America/Barbados
    {  687, 7646,   30526,  5, }, // America/Belem
    {  687, 5407,   88731,  6, }, // America/Belize
    {  687, 7660,   88737, 13, }, // America/Blanc-Sablon
    {  687, 7681,   88750, 10, }, // America/Boa_Vista
    {  687, 4361,   88760,  6, }, // America/Bogota
    {  687, 7699,   88766,  5, }, // America/Boise
    {  687, 7713,   88771, 15, }, // America/Cambridge_Bay
    {  687, 7735,   88786, 13, }, // America/Campo_Grande
    {  687, 3868,   88799,  6, }, // America/Cancun
    {  687, 4694,   88805,  6, }, // America/Caracas
    {  687, 4345,   30609,  5, }, // America/Cayenne
    {  687, 5496,   30614,  5, }, // America/Cayman
    {  687, 2260,   88811,  6, }, // America/Chicago
    {  687, 7756,   88817,  8, }, // America/Chihuahua
    {  687, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  687, 5422,   88841, 11, }, // America/Costa_Rica
    {  687, 7796,   30642,  8, }, // America/Creston
    {  687, 3791,   88852,  7, }, // America/Cuiaba
    {  687, 5723,   88859,  7, }, // America/Curacao
    {  687, 7812,   88866, 11, }, // America/Danmarkshavn
    {  687, 7833,   88877,  4, }, // America/Dawson
    {  687, 7848,   88881, 10, }, // America/Dawson_Creek
    {  687,  805,   88891,  5, }, // America/Denver
    {  687, 3465,   88896,  8, }, // America/Detroit
    {  687, 5739,   88904,  8, }, // America/Dominica
    {  687,  893,   88912,  6, }, // America/Edmonton
    {  687, 7869,   30723,  7, }, // America/Eirunepe
    {  687, 5441,   88918, 11, }, // America/El_Salvador
    {  687, 7886,   88929, 13, }, // America/Fort_Nelson
    {  687, 7906,   88942, 12, }, // America/Fortaleza
    {  687, 7924,   88954, 11, }, // America/Glace_Bay
    {  687, 6881,   88965,  9, }, // America/Goose_Bay
    {  687, 4612,   88974, 11, }, // America/Grand_Turk
    {  687, 5770,   30789,  8, }, // America/Grenada
    {  687, 5786,   88985,  9, }, // America/Guadeloupe
    {  687, 3760,   30806, 10, }, // America/Guatemala
    {  687, 4841,   88994,  9, }, // America/Guayaquil
    {  687, 6932,   30824,  6, }, // America/Guyana
    {  687, 1939,   89008, 10, }, // America/Halifax
    {  687, 2281,   89018,  5, }, // America/Havana
    {  687, 7942,   89023, 11, }, // America/Hermosillo
    {  687,  348,   89034, 13, }, // America/Indiana/Indianapolis
    {  687,  481,   89047, 15, }, // America/Indiana/Knox
    {  687, 7961,   89062, 17, }, // America/Indiana/Marengo
    {  687, 7985,   89079, 20, }, // America/Indiana/Petersburg
    {  687, 8012,   89099, 18, }, // America/Indiana/Tell_City
    {  687, 8038,   89117, 14, }, // America/Indiana/Vevay
    {  687, 8060,   89131, 18, }, // America/Indiana/Vincennes
    {  687, 8086,   89149, 17, }, // America/Indiana/Winamac
    {  687, 8110,   30999,  6, }, // America/Inuvik
    {  687,  660,   89166,  7, }, // America/Iqaluit
    {  687, 2799,   89173,  5, }, // America/Jamaica
    {  687, 5380,   89178,  7, }, // America/Juneau
    {  687,  521,   89185,  8, }, // America/Kentucky/Louisville
    {  687, 8125,   89193, 17, }, // America/Kentucky/Monticello
    {  687, 5704,   89210, 12, }, // America/Kralendijk
    {  687, 4376,   89222,  7, }, // America/La_Paz
    {  687, 7169,   31064,  4, }, // America/Lima
    {  687, 3239,   89229, 11, }, // America/Los_Angeles
    {  687, 5932,   89240, 22, }, // America/Lower_Princes
    {  687, 8153,   89262,  5, }, // America/Maceio
    {  687, 8168,   89267,  7, }, // America/Managua
    {  687, 1908,   89274,  4, }, // America/Manaus
    {  687, 5897,   89278,  7, }, // America/Marigot
    {  687, 5805,   89285,  9, }, // America/Martinique
    {  687, 8184,   89294,  9, }, // America/Matamoros
    {  687, 2917,  221040,  8, }, // America/Mazatlan
    {  687, 8202,   89312,  8, }, // America/Menominee
    {  687, 8220,   31158,  6, }, // America/Merida
    {  687, 8235,   89320, 12, }, // America/Metlakatla
    {  687, 2949,   89332, 13, }, // America/Mexico_City
    {  687, 4391,   89345,  7, }, // America/Miquelon
    {  687, 8254,   89352,  6, }, // America/Moncton
    {  687, 8270,   89358,  9, }, // America/Monterrey
    {  687, 4098,   89367, 11, }, // America/Montevideo
    {  687, 5824,   89378,  9, }, // America/Montserrat
    {  687, 5481,   89387,  5, }, // America/Nassau
    {  687, 2379,   89392,  9, }, // America/New_York
    {  687, 8288,   89314,  3, }, // America/Nome
    {  687, 1850,   89401,  8, }, // America/Noronha
    {  687, 8301,   89409, 20, }, // America/North_Dakota/Beulah
    {  687, 8329,   89429, 19, }, // America/North_Dakota/Center
    {  687, 8357,   89448, 24, }, // America/North_Dakota/New_Salem
    {  687,  393,   89472,  3, }, // America/Nuuk
    {  687, 8388,   89475,  9, }, // America/Ojinaga
    {  687, 2356,   89484,  5, }, // America/Panama
    {  687, 6173,   89489, 10, }, // America/Paramaribo
    {  687, 2973,   89499,  8, }, // America/Phoenix
    {  687, 3945,   89507, 14, }, // America/Port-au-Prince
    {  687, 5954,   89521, 15, }, // America/Port_of_Spain
    {  687, 8404,   89536, 13, }, // America/Porto_Velho
    {  687, 5843,   89549, 11, }, // America/Puerto_Rico
    {  687, 4030,   89560, 12, }, // America/Punta_Arenas
    {  687, 8424,   89572, 12, }, // America/Rankin_Inlet
    {  687, 8445,   89584,  7, }, // America/Recife
    {  687, 1995,   31431,  6, }, // America/Regina
    {  687, 8460,   89591,  9, }, // America/Resolute
    {  687,  695,   89600, 12, }, // America/Rio_Branco
    {  687, 8477,   89612,  8, }, // America/Santarem
    {  687, 2201,   89620,  9, }, // America/Santiago
    {  687, 6111,   89629, 13, }, // America/Santo_Domingo
    {  687, 1878,   89642,  9, }, // America/Sao_Paulo
    {  687, 6899,   89651, 13, }, // America/Scoresbysund
    {  687, 8494,   89664,  6, }, // America/Sitka
    {  687, 8508,   89670, 15, }, // America/St_Barthelemy
    {  687, 2061,   89685,  9, }, // America/St_Johns
    {  687, 5863,   89694, 10, }, // America/St_Kitts
    {  687, 5880,   89704, 11, }, // America/St_Lucia
    {  687,  855,   89715,  9, }, // America/St_Thomas
    {  687, 5913,   89724, 12, }, // America/St_Vincent
    {  687, 8530,   89736, 13, }, // America/Swift_Current
    {  687, 5461,   89749, 11, }, // America/Tegucigalpa
    {  687, 5756,   89760,  6, }, // America/Thule
    {  687,  313,   89766,  7, }, // America/Tijuana
    {  687,  608,   89773,  6, }, // America/Toronto
    {  687, 5688,   89779,  8, }, // America/Tortola
    {  687, 2093,   89787,  7, }, // America/Vancouver
    {  687, 2144,   89794, 11, }, // America/Whitehorse
    {  687,  734,   89805,  7, }, // America/Winnipeg
    {  687, 8552,   89812,  7, }, // America/Yakutat
    {  687, 6001,   31673,  4, }, // Antarctica/Casey
    {  687, 6094,   89819,  5, }, // Antarctica/Davis
    {  687, 6133,   89824, 19, }, // Antarctica/DumontDUrville
    {  687, 8568,   89843,  7, }, // Antarctica/Macquarie
    {  687, 7066,   89850,  4, }, // Antarctica/Mawson
    {  687, 7113,   89854,  9, }, // Antarctica/McMurdo
    {  687, 8589,   89863,  4, }, // Antarctica/Palmer
    {  687, 7226,   89867,  6, }, // Antarctica/Rothera
    {  687, 7295,   89873,  6, }, // Antarctica/Syowa
    {  687, 8607,   89879,  5, }, // Antarctica/Troll
    {  687, 7374,   89884,  7, }, // Antarctica/Vostok
    {  687, 1429,   89891, 13, }, // Arctic/Longyearbyen
    {  687, 5570,  129942,  3, }, // Asia/Aden
    {  687, 5395,   89908,  7, }, // Asia/Almaty
    {  687, 3968,   89915,  6, }, // Asia/Amman
    {  687, 5511,   89921,  6, }, // Asia/Anadyr
    {  687, 5523,   31778,  6, }, // Asia/Aqtau
    {  687, 5368,   89927,  6, }, // Asia/Aqtobe
    {  687,  964,   31791,  8, }, // Asia/Ashgabat
    {  687, 8624,   89933,  5, }, // Asia/Atyrau
    {  687, 3607,   89938,  5, }, // Asia/Baghdad
    {  687, 5534,   89943,  5, }, // Asia/Bahrain
    {  687, 3653,   88857,  4, }, // Asia/Baku
    {  687, 4466,   89948,  6, }, // Asia/Bangkok
    {  687, 3571,   89954,  6, }, // Asia/Barnaul
    {  687, 4086,   89960,  5, }, // Asia/Beirut
    {  687, 3778,   31836,  7, }, // Asia/Bishkek
    {  687, 5989,   89965,  7, }, // Asia/Brunei
    {  687, 4601,   89972,  6, }, // Asia/Chita
    {  687, 4511,   89978,  6, }, // Asia/Colombo
    {  687, 4540,   89984,  7, }, // Asia/Damascus
    {  687, 1093,   89991,  4, }, // Asia/Dhaka
    {  687, 6192,   89995,  4, }, // Asia/Dili
    {  687, 3596,  221048,  3, }, // Asia/Dubai
    {  687, 6159,   90003,  7, }, // Asia/Dushanbe
    {  687, 8636,   90010, 11, }, // Asia/Famagusta
    {  687, 8651,   90021,  5, }, // Asia/Gaza
    {  687, 4795,   90026,  7, }, // Asia/Hebron
    {  687, 1259,  139473, 15, }, // Asia/Ho_Chi_Minh
    {  687, 2704,   90049,  9, }, // Asia/Hong_Kong
    {  687, 4771,   90058,  5, }, // Asia/Hovd
    {  687, 4184,   90063, 10, }, // Asia/Irkutsk
    {  687, 7006,   90073,  7, }, // Asia/Jakarta
    {  687, 6992,   31943,  7, }, // Asia/Jayapura
    {  687, 1290,   90080,  6, }, // Asia/Jerusalem
    {  687, 3560,  221051,  4, }, // Asia/Kabul
    {  687, 4316,   90091,  7, }, // Asia/Kamchatka
    {  687, 4239,   31972,  5, }, // Asia/Karachi
    {  687, 1185,   90098,  8, }, // Asia/Kathmandu
    {  687, 8661,   90106,  7, }, // Asia/Khandyga
    {  687,  992,   31994,  6, }, // Asia/Kolkata
    {  687, 4197,   90120, 15, }, // Asia/Krasnoyarsk
    {  687, 4859,   90135, 10, }, // Asia/Kuala_Lumpur
    {  687, 5976,   32028,  6, }, // Asia/Kuching
    {  687, 5547,   32034,  5, }, // Asia/Kuwait
    {  687, 1211,   90145,  4, }, // Asia/Macau
    {  687, 4017,   32044,  7, }, // Asia/Magadan
    {  687, 1349,   90149,  6, }, // Asia/Makassar
    {  687, 7182,   32060,  5, }, // Asia/Manila
    {  687, 6920,   90155,  5, }, // Asia/Muscat
    {  687, 2554,   90160,  8, }, // Asia/Nicosia
    {  687, 8675,   90168, 16, }, // Asia/Novokuznetsk
    {  687, 4135,   90184, 13, }, // Asia/Novosibirsk
    {  687, 4229,   90197,  6, }, // Asia/Omsk
    {  687, 7145,   90203,  3, }, // Asia/Oral
    {  687, 6961,  114860,  9, }, // Asia/Phnom_Penh
    {  687, 8693,   90215,  9, }, // Asia/Pontianak
    {  687, 4214,   90224, 10, }, // Asia/Pyongyang
    {  687, 5559,   90234,  3, }, // Asia/Qatar
    {  687, 8708,   90237,  8, }, // Asia/Qostanay
    {  687, 4269,   90245, 11, }, // Asia/Qyzylorda
    {  687, 3584,   90256,  5, }, // Asia/Riyadh
    {  687, 4408,   90261,  6, }, // Asia/Sakhalin
    {  687, 7245,   90267,  6, }, // Asia/Samarkand
    {  687, 3279,   90273,  4, }, // Asia/Seoul
    {  687, 1053,   90277,  5, }, // Asia/Shanghai
    {  687, 3300,   32190,  8, }, // Asia/Singapore
    {  687, 4297,   90282, 16, }, // Asia/Srednekolymsk
    {  687, 3263,  130170,  6, }, // Asia/Taipei
    {  687, 4781,   90304,  6, }, // Asia/Tashkent
    {  687, 3915,   90310,  7, }, // Asia/Tbilisi
    {  687, 2772,   32236,  6, }, // Asia/Tehran
    {  687, 1317,   90317,  5, }, // Asia/Thimphu
    {  687, 2821,   90322,  6, }, // Asia/Tokyo
    {  687, 4572,   90328,  7, }, // Asia/Tomsk
    {  687, 1021,   90335,  8, }, // Asia/Ulaanbaatar
    {  687, 1159,   90343,  7, }, // Asia/Urumqi
    {  687, 8722,   90350, 10, }, // Asia/Ust-Nera
    {  687, 6977,   90360,  9, }, // Asia/Vientiane
    {  687, 4710,   90369, 13, }, // Asia/Vladivostok
    {  687, 4828,   32305,  9, }, // Asia/Yakutsk
    {  687, 1235,   32314,  5, }, // Asia/Yangon
    {  687, 3883,   90382, 13, }, // Asia/Yekaterinburg
    {  687, 3747,   90395,  7, }, // Asia/Yerevan
    {  687, 3663,   90402,  7, }, // Atlantic/Azores
    {  687, 5671,   90409,  6, }, // Atlantic/Bermuda
    {  687, 6628,   90415,  6, }, // Atlantic/Canary
    {  687, 3727,   90421,  8, }, // Atlantic/Cape_Verde
    {  687, 1395,   90429,  5, }, // Atlantic/Faroe
    {  687, 8736,   90434,  5, }, // Atlantic/Madeira
    {  687, 2748,   90439,  9, }, // Atlantic/Reykjavik
    {  687, 7272,  221055, 13, }, // Atlantic/South_Georgia
    {  687, 6821,   90463, 11, }, // Atlantic/St_Helena
    {  687, 6644,   90474,  7, }, // Atlantic/Stanley
    {  687, 1670,   90481,  6, }, // Australia/Adelaide
    {  687, 1635,   90487,  8, }, // Australia/Brisbane
    {  687, 1799,   90495, 10, }, // Australia/Broken_Hill
    {  687, 1583,   90505,  7, }, // Australia/Darwin
    {  687, 3637,   32452,  6, }, // Australia/Eucla
    {  687, 1516,   90512,  7, }, // Australia/Hobart
    {  687, 8753,   90519,  8, }, // Australia/Lindeman
    {  687, 1547,   90527, 10, }, // Australia/Lord_Howe
    {  687, 1727,   90537,  8, }, // Australia/Melbourne
    {  687, 1762,   90545,  4, }, // Australia/Perth
    {  687, 1463,   32494,  5, }, // Australia/Sydney
    {  687, 6441,  221068,  9, }, // Europe/Amsterdam
    {  687, 6216,   90559,  6, }, // Europe/Andorra
    {  687, 3620,   90565, 10, }, // Europe/Astrakhan
    {  687, 2300,   90575,  5, }, // Europe/Athens
    {  687, 6336,   32532,  8, }, // Europe/Belgrade
    {  687, 4757,   90580,  6, }, // Europe/Berlin
    {  687, 6488,   90586, 12, }, // Europe/Bratislava
    {  687, 2167,   90598,  9, }, // Europe/Brussels
    {  687, 3928,   90607, 10, }, // Europe/Bucharest
    {  687, 3806,   90617,  9, }, // Europe/Budapest
    {  687, 8772,   90626, 10, }, // Europe/Busingen
    {  687, 2583,   90636,  7, }, // Europe/Chisinau
    {  687, 6289,   90643,  9, }, // Europe/Copenhagen
    {  687, 2338,   90652,  5, }, // Europe/Dublin
    {  687, 6307,   90657, 10, }, // Europe/Gibraltar
    {  687, 8788,   90667,  6, }, // Europe/Guernsey
    {  687, 6612,   90673,  8, }, // Europe/Helsinki
    {  687, 8804,   90681, 11, }, // Europe/Isle_of_Man
    {  687, 1130,   90692,  9, }, // Europe/Istanbul
    {  687, 8823,   90701,  5, }, // Europe/Jersey
    {  687, 3979,   90706, 12, }, // Europe/Kaliningrad
    {  687, 8929,   90718,  5, }, // Europe/Kirov
    {  687, 2527,   90723,  3, }, // Europe/Kyiv
    {  687, 3213,   32690,  6, }, // Europe/Lisbon
    {  687, 6506,   90726, 11, }, // Europe/Ljubljana
    {  687, 2501,   90737,  4, }, // Europe/London
    {  687, 6365,   90741, 10, }, // Europe/Luxembourg
    {  687, 4892,   90751,  7, }, // Europe/Madrid
    {  687, 6397,   32731,  6, }, // Europe/Malta
    {  687, 6582,   90758,  9, }, // Europe/Mariehamn
    {  687, 3693,   90767,  6, }, // Europe/Minsk
    {  687, 6410,   90773,  6, }, // Europe/Monaco
    {  687, 3537,   90779,  6, }, // Europe/Moscow
    {  687, 6458,   90785,  5, }, // Europe/Oslo
    {  687, 4284,  221077,  4, }, // Europe/Paris
    {  687, 6424,   90790, 10, }, // Europe/Podgorica
    {  687, 6275,   90800,  5, }, // Europe/Prague
    {  687, 8837,   90805,  4, }, // Europe/Riga
    {  687, 6324,   90432,  3, }, // Europe/Rome
    {  687, 4331,   90809,  5, }, // Europe/Samara
    {  687, 6470,   90814, 10, }, // Europe/San_Marino
    {  687, 6245,   90824,  8, }, // Europe/Sarajevo
    {  687, 4451,   90832,  7, }, // Europe/Saratov
    {  687, 8849,   90839, 12, }, // Europe/Simferopol
    {  687, 6383,   90851,  8, }, // Europe/Skopje
    {  687, 6599,   90859,  7, }, // Europe/Sofia
    {  687, 6523,   90866,  8, }, // Europe/Stockholm
    {  687, 8867,   90874,  5, }, // Europe/Tallinn
    {  687, 6202,   90879,  6, }, // Europe/Tirane
    {  687, 8882,   90885, 12, }, // Europe/Ulyanovsk
    {  687, 6352,   90897,  6, }, // Europe/Vaduz
    {  687, 6567,   90903,  6, }, // Europe/Vatican
    {  687, 6231,   90909,  5, }, // Europe/Vienna
    {  687, 8899,   90914,  8, }, // Europe/Vilnius
    {  687, 4727,   90922, 11, }, // Europe/Volgograd
    {  687, 3190,   90933,  5, }, // Europe/Warsaw
    {  687, 6261,   90938,  8, }, // Europe/Zagreb
    {  687, 6540,   90946,  9, }, // Europe/Zurich
    {  687, 5078,   90955, 11, }, // Indian/Antananarivo
    {  687, 6947,   90966,  5, }, // Indian/Chagos
    {  687, 6046,  221081,  8, }, // Indian/Christmas
    {  687, 6063,   90978,  5, }, // Indian/Cocos
    {  687, 5029,   90983,  6, }, // Indian/Comoro
    {  687, 6661,   90989,  7, }, // Indian/Kerguelen
    {  687, 7260,   32983,  4, }, // Indian/Mahe
    {  687, 7050,   90996,  6, }, // Indian/Maldives
    {  687, 4069,   91002,  6, }, // Indian/Mauritius
    {  687, 5098,   91008,  8, }, // Indian/Mayotte
    {  687, 7211,   91016,  8, }, // Indian/Reunion
    {  687, 4422,   91024,  5, }, // Pacific/Apia
    {  687,  932,   91029,  6, }, // Pacific/Auckland
    {  687, 3706,   91035,  8, }, // Pacific/Bougainville
    {  687, 3015,   91043,  4, }, // Pacific/Chatham
    {  687, 3157,   91047,  2, }, // Pacific/Chuuk
    {  687, 2237,   91049,  5, }, // Pacific/Easter
    {  687, 7360,   91054,  5, }, // Pacific/Efate
    {  687, 7327,   91059,  9, }, // Pacific/Fakaofo
    {  687, 3902,   91068,  5, }, // Pacific/Fiji
    {  687, 7343,   91073, 12, }, // Pacific/Funafuti
    {  687, 6678,   91085,  9, }, // Pacific/Galapagos
    {  687, 6696,   91094,  7, }, // Pacific/Gambier
    {  687, 3822,   91101, 10, }, // Pacific/Guadalcanal
    {  687, 6018,   31112,  4, }, // Pacific/Guam
    {  687, 3049,  221089,  5, }, // Pacific/Kanton
    {  687, 3998,   33132, 10, }, // Pacific/Kiritimati
    {  687, 7019,   91126,  6, }, // Pacific/Kosrae
    {  687, 2842,   91132, 10, }, // Pacific/Kwajalein
    {  687, 4877,   91142,  6, }, // Pacific/Majuro
    {  687, 4051,   91148,  8, }, // Pacific/Marquesas
    {  687, 8914,   91156,  5, }, // Pacific/Midway
    {  687, 7084,   91161,  4, }, // Pacific/Nauru
    {  687, 7132,   91165,  4, }, // Pacific/Niue
    {  687, 4168,   91169,  7, }, // Pacific/Norfolk
    {  687, 7098,   91176,  6, }, // Pacific/Noumea
    {  687, 3126,  221094,  9, }, // Pacific/Pago_Pago
    {  687, 7155,   91191,  4, }, // Pacific/Palau
    {  687, 7194,   33210,  8, }, // Pacific/Pitcairn
    {  687, 3096,   91195,  6, }, // Pacific/Pohnpei
    {  687, 4807,   91201, 14, }, // Pacific/Port_Moresby
    {  687, 6076,   91215,  9, }, // Pacific/Rarotonga
    {  687, 6031,   91224,  6, }, // Pacific/Saipan
    {  687, 7312,   91230,  6, }, // Pacific/Tahiti
    {  687, 6712,   91236,  6, }, // Pacific/Tarawa
    {  687, 4583,   91242,  9, }, // Pacific/Tongatapu
    {  687, 7392,   33268,  3, }, // Pacific/Wake
    {  687, 7405,   33271,  5, }, // Pacific/Wallis
    {  688, 6788,  195501,  6, }, // Africa/Abidjan Venetian/Latin/Italy
    {  688, 6760,   74129,  4, }, // Africa/Accra
    {  688, 5059,   35821, 10, }, // Africa/Addis_Ababa
    {  688, 7420,  221103,  6, }, // Africa/Algiers
    {  688,   14,  221109,  6, }, // Africa/Asmara
    {  688, 5257,  221115,  6, }, // Africa/Bangui
    {  688, 7435,  221121,  5, }, // Africa/Bissau
    {  688, 4922,  221126,  9, }, // Africa/Bujumbura
    {  688, 2320,  161877,  8, }, // Africa/Cairo
    {  688, 4117,  221135, 10, }, // Africa/Casablanca
    {  688, 7449,  166781,  5, }, // Africa/Ceuta
    {  688, 6773,  157976,  7, }, // Africa/Conakry
    {  688, 5130,   21354, 12, }, // Africa/Dar_es_Salaam
    {  688, 5043,  158501,  6, }, // Africa/Djibouti
    {  688, 5243,   21372,  5, }, // Africa/Douala
    {  688, 5166,    6393,  7, }, // Africa/El_Aaiun
    {  688, 5151,  221145,  7, }, // Africa/Kampala
    {  688, 4524,  157994,  6, }, // Africa/Khartoum
    {  688, 6869,   20418,  4, }, // Africa/Lome
    {  688, 4939,  221152, 10, }, // Africa/Lubumbashi
    {  688, 5197,  221162,  6, }, // Africa/Maseru
    {  688, 5182,  221168,  7, }, // Africa/Mbabane
    {  688, 5113,  221175,  8, }, // Africa/Mogadishu
    {  688, 5271,  195575,  6, }, // Africa/Ndjamena
    {  688, 6803,  147724,  8, }, // Africa/Nouakchott
    {  688, 6727,  221183,  8, }, // Africa/Ouagadougou
    {  688, 5225,   26542, 10, }, // Africa/Porto-Novo
    {  688, 4435,  221191, 15, }, // Africa/Sao_Tome
    {  688, 2866,  164486,  7, }, // Africa/Tripoli
    {  688, 6554,  221206,  6, }, // Africa/Tunis
    {  688, 5607,    4775,  7, }, // America/Anguilla
    {  688,  270,  221212,  7, }, // America/Argentina/Cordoba
    {  688, 7489,  221219, 11, }, // America/Argentina/Rio_Gallegos
    {  688, 4252,    6592,  8, }, // America/Asuncion
    {  688,  237,      22,  8, }, // America/Atikokan
    {  688, 7623,  221230, 16, }, // America/Bahia_Banderas
    {  688, 4361,   44161,  6, }, // America/Bogota
    {  688, 7735,  221246, 12, }, // America/Campo_Grande
    {  688, 3868,   33314,  6, }, // America/Cancun
    {  688, 4345,  147758,  6, }, // America/Cayenne
    {  688, 5496,   44173,  6, }, // America/Cayman
    {  688, 7774,      60,  6, }, // America/Ciudad_Juarez
    {  688, 3791,  221258,  6, }, // America/Cuiaba
    {  688, 5723,      66,  7, }, // America/Curacao
    {  688,  805,  221264,  6, }, // America/Denver
    {  688, 5739,  221270,  8, }, // America/Dominica
    {  688, 7869,   26580,  8, }, // America/Eirunepe
    {  688, 5770,   20519,  7, }, // America/Grenada
    {  688, 5786,   95109,  9, }, // America/Guadeloupe
    {  688, 2281,   95118,  7, }, // America/Havana
    {  688,  348,  221278, 12, }, // America/Indiana/Indianapolis
    {  688,  481,  221290, 14, }, // America/Indiana/Knox
    {  688, 7961,  221304, 17, }, // America/Indiana/Marengo
    {  688, 7985,  221321, 20, }, // America/Indiana/Petersburg
    {  688, 8012,  221341, 19, }, // America/Indiana/Tell_City
    {  688, 8038,  221360, 15, }, // America/Indiana/Vevay
    {  688, 8060,  221375, 19, }, // America/Indiana/Vincennes
    {  688, 8086,  221394, 17, }, // America/Indiana/Winamac
    {  688, 2799,  221411,  7, }, // America/Jamaica
    {  688, 8125,  221418, 21, }, // America/Kentucky/Monticello
    {  688, 3239,  221439, 11, }, // America/Los_Angeles
    {  688, 5932,     207, 22, }, // America/Lower_Princes
    {  688, 8153,   26588,  6, }, // America/Maceio
    {  688, 5805,  221450,  9, }, // America/Martinique
    {  688, 8220,     229,  6, }, // America/Merida
    {  688, 2949,  221459, 16, }, // America/Mexico_City
    {  688, 4391,  221475,  8, }, // America/Miquelon
    {  688, 8270,   21944,  8, }, // America/Monterrey
    {  688, 5824,    7172,  9, }, // America/Montserrat
    {  688, 1850,     235, 19, }, // America/Noronha
    {  688, 8301,  221483, 20, }, // America/North_Dakota/Beulah
    {  688, 8329,  221503, 20, }, // America/North_Dakota/Center
    {  688, 8357,  221523, 23, }, // America/North_Dakota/New_Salem
    {  688,  393,     317,  4, }, // America/Nuuk
    {  688, 2356,  164613,  6, }, // America/Panama
    {  688, 3945,  221546, 14, }, // America/Port-au-Prince
    {  688, 5954,  221560, 15, }, // America/Port_of_Spain
    {  688, 5843,   95222,  9, }, // America/Puerto_Rico
    {  688, 1878,  221575,  8, }, // America/Sao_Paulo
    {  688, 6899,     321, 16, }, // America/Scoresbysund
    {  688, 8508,  221583, 13, }, // America/St_Barthelemy
    {  688, 2061,  221596,  8, }, // America/St_Johns
    {  688, 5863,  221604, 12, }, // America/St_Kitts
    {  688, 5880,  221616,  8, }, // America/St_Lucia
    {  688,  855,  221197,  9, }, // America/St_Thomas
    {  688, 5913,  221624, 11, }, // America/St_Vincent
    {  688, 5688,  148000,  7, }, // America/Tortola
    {  688, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  688, 8568,  221635, 15, }, // Antarctica/Macquarie
    {  688, 5395,    7594,  6, }, // Asia/Almaty
    {  688, 3968,    5605,  4, }, // Asia/Amman
    {  688, 5511,  221650,  6, }, // Asia/Anadyr
    {  688, 5523,  221656,  5, }, // Asia/Aqtau
    {  688, 5368,    7606,  6, }, // Asia/Aqtobe
    {  688,  964,  221661,  7, }, // Asia/Ashgabat
    {  688, 8624,  221668,  6, }, // Asia/Atyrau
    {  688, 3607,    4162,  6, }, // Asia/Baghdad
    {  688, 5534,   63955,  6, }, // Asia/Bahrain
    {  688, 3778,   91536,  6, }, // Asia/Bishkek
    {  688, 5989,   20751,  6, }, // Asia/Brunei
    {  688, 4601,   93254,  4, }, // Asia/Chita
    {  688, 4511,  221674,  7, }, // Asia/Colombo
    {  688, 4540,   74438,  7, }, // Asia/Damascus
    {  688, 1093,  158496,  4, }, // Asia/Dhaka
    {  688, 3596,  221681,  5, }, // Asia/Dubai
    {  688, 6159,  221686,  7, }, // Asia/Dushanbe
    {  688, 8636,   95258,  9, }, // Asia/Famagusta
    {  688, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  688, 1290,  123559, 10, }, // Asia/Jerusalem
    {  688, 4316,  162180,  9, }, // Asia/Kamchatka
    {  688, 1185,  221693,  8, }, // Asia/Kathmandu
    {  688, 8661,  221701,  7, }, // Asia/Khandyga
    {  688,  992,   20838,  7, }, // Asia/Kolkata
    {  688, 4197,  221708, 11, }, // Asia/Krasnoyarsk
    {  688, 4859,  221719, 12, }, // Asia/Kuala_Lumpur
    {  688, 1211,  221731,  5, }, // Asia/Macau
    {  688, 1349,    7736,  7, }, // Asia/Makassar
    {  688, 6920,   20857,  7, }, // Asia/Muscat
    {  688, 8675,  221736, 13, }, // Asia/Novokuznetsk
    {  688, 7145,   65240,  5, }, // Asia/Oral
    {  688, 6961,   22518,  8, }, // Asia/Phnom_Penh
    {  688, 5559,   73150,  5, }, // Asia/Qatar
    {  688, 8708,  221749,  8, }, // Asia/Qostanay
    {  688, 4269,    7778,  9, }, // Asia/Qyzylorda
    {  688, 3584,    7787,  4, }, // Asia/Riyadh
    {  688, 4408,  158282,  7, }, // Asia/Sakhalin
    {  688, 7245,   20886, 10, }, // Asia/Samarkand
    {  688, 3279,    7798,  4, }, // Asia/Seoul
    {  688, 1053,  221757,  8, }, // Asia/Shanghai
    {  688, 4297,  221765, 13, }, // Asia/Srednekolymsk
    {  688, 3263,  177683,  6, }, // Asia/Taipei
    {  688, 4781,   91755,  7, }, // Asia/Tashkent
    {  688, 3915,  220114,  7, }, // Asia/Tbilisi
    {  688, 2772,    7842,  7, }, // Asia/Tehran
    {  688, 1317,  221778,  5, }, // Asia/Thimphu
    {  688, 1021,  221783, 10, }, // Asia/Ulaanbaatar
    {  688, 1159,  202026,  7, }, // Asia/Urumqi
    {  688, 8722,  221793,  9, }, // Asia/Ust-Nera
    {  688, 6977,   26939,  8, }, // Asia/Vientiane
    {  688, 1235,     477,  6, }, // Asia/Yangon
    {  688, 3883,  221802, 14, }, // Asia/Yekaterinburg
    {  688, 3747,  221816,  7, }, // Asia/Yerevan
    {  688, 3663,  221823, 11, }, // Atlantic/Azores
    {  688, 6628,  221834, 13, }, // Atlantic/Canary
    {  688, 3727,  221847,  9, }, // Atlantic/Cape_Verde
    {  688, 1395,  221856, 11, }, // Atlantic/Faroe
    {  688, 8736,  221867, 13, }, // Atlantic/Madeira
    {  688, 2748,  221880,  8, }, // Atlantic/Reykjavik
    {  688, 7272,  221888, 16, }, // Atlantic/South_Georgia
    {  688, 6821,  221904, 14, }, // Atlantic/St_Helena
    {  688, 1670,  221918,  8, }, // Australia/Adelaide
    {  688, 1635,  221926,  8, }, // Australia/Brisbane
    {  688, 1547,  221934, 15, }, // Australia/Lord_Howe
    {  688, 6441,  221949,  9, }, // Europe/Amsterdam
    {  688, 6216,   33638,  6, }, // Europe/Andorra
    {  688, 3620,   33644,  8, }, // Europe/Astrakhan
    {  688, 2300,    5911,  5, }, // Europe/Athens
    {  688, 6336,   58906,  8, }, // Europe/Belgrade
    {  688, 6488,  221958, 10, }, // Europe/Bratislava
    {  688, 2167,   27022,  6, }, // Europe/Brussels
    {  688, 3928,   21073,  8, }, // Europe/Bucharest
    {  688, 8772,   27037,  8, }, // Europe/Busingen
    {  688, 6289,   95331, 10, }, // Europe/Copenhagen
    {  688, 6307,  221968,  9, }, // Europe/Gibraltar
    {  688, 8804,  221977, 12, }, // Europe/Isle_of_Man
    {  688, 1130,  164893,  8, }, // Europe/Istanbul
    {  688, 3979,   63006, 12, }, // Europe/Kaliningrad
    {  688, 2527,    8148,  4, }, // Europe/Kyiv
    {  688, 3213,  157721,  7, }, // Europe/Lisbon
    {  688, 6506,  221989,  8, }, // Europe/Ljubljana
    {  688, 2501,   95370,  6, }, // Europe/London
    {  688, 6365,  221997, 10, }, // Europe/Luxembourg
    {  688, 6410,   44570,  6, }, // Europe/Monaco
    {  688, 3537,   95387,  5, }, // Europe/Moscow
    {  688, 6458,  222007,  4, }, // Europe/Oslo
    {  688, 4284,  222011,  6, }, // Europe/Paris
    {  688, 6424,  196828,  9, }, // Europe/Podgorica
    {  688, 6275,   21185,  5, }, // Europe/Prague
    {  688, 6324,    6064,  4, }, // Europe/Rome
    {  688, 6470,   65868,  9, }, // Europe/San_Marino
    {  688, 8849,  222017, 11, }, // Europe/Simferopol
    {  688, 6523,  222028,  8, }, // Europe/Stockholm
    {  688, 8867,    8250,  5, }, // Europe/Tallinn
    {  688, 6202,   21213,  6, }, // Europe/Tirane
    {  688, 8882,  222036,  8, }, // Europe/Ulyanovsk
    {  688, 6567,  222044,  7, }, // Europe/Vatican
    {  688, 6231,   21230,  5, }, // Europe/Vienna
    {  688, 4727,   63187, 10, }, // Europe/Volgograd
    {  688, 3190,   95436,  8, }, // Europe/Warsaw
    {  688, 6540,   95452,  6, }, // Europe/Zurich
    {  688, 6947,  222051,  6, }, // Indian/Chagos
    {  688, 6046,  222057, 15, }, // Indian/Christmas
    {  688, 6063,  222072, 11, }, // Indian/Cocos
    {  688, 5029,  222083, 12, }, // Indian/Comoro
    {  688, 7260,   27115,  4, }, // Indian/Mahe
    {  688, 7050,  222095, 13, }, // Indian/Maldives
    {  688, 4069,  222108, 14, }, // Indian/Mauritius
    {  688, 5098,  222122, 12, }, // Indian/Mayotte
    {  688, 7211,  222134, 13, }, // Indian/Reunion
    {  688, 3706,  222147, 18, }, // Pacific/Bougainville
    {  688, 3015,  222165, 12, }, // Pacific/Chatham
    {  688, 3157,  222177, 11, }, // Pacific/Chuuk
    {  688, 2237,  222188, 15, }, // Pacific/Easter
    {  688, 7360,  222203, 11, }, // Pacific/Efate
    {  688, 7327,  222214, 13, }, // Pacific/Fakaofo
    {  688, 7343,  222227, 14, }, // Pacific/Funafuti
    {  688, 6678,  167332,  9, }, // Pacific/Galapagos
    {  688, 6696,  222241, 13, }, // Pacific/Gambier
    {  688, 3822,  222254, 17, }, // Pacific/Guadalcanal
    {  688, 3049,  222271, 12, }, // Pacific/Kanton
    {  688, 3998,  222283, 16, }, // Pacific/Kiritimati
    {  688, 7019,  222299, 12, }, // Pacific/Kosrae
    {  688, 2842,  222311, 15, }, // Pacific/Kwajalein
    {  688, 4051,  222326, 14, }, // Pacific/Marquesas
    {  688, 8914,  222340, 12, }, // Pacific/Midway
    {  688, 7084,  222352, 11, }, // Pacific/Nauru
    {  688, 7132,  222363, 10, }, // Pacific/Niue
    {  688, 4168,  222373, 13, }, // Pacific/Norfolk
    {  688, 7098,    4628,  6, }, // Pacific/Noumea
    {  688, 3126,  135456, 11, }, // Pacific/Pago_Pago
    {  688, 7155,  222386,  5, }, // Pacific/Palau
    {  688, 7194,  222391, 14, }, // Pacific/Pitcairn
    {  688, 3096,  222405, 12, }, // Pacific/Pohnpei
    {  688, 4807,  222417, 13, }, // Pacific/Port_Moresby
    {  688, 6076,  222430, 15, }, // Pacific/Rarotonga
    {  688, 6031,  222445, 12, }, // Pacific/Saipan
    {  688, 7312,  222457, 11, }, // Pacific/Tahiti
    {  688, 6712,  222468, 13, }, // Pacific/Tarawa
    {  688, 4583,  222481, 15, }, // Pacific/Tongatapu
    {  688, 7392,  222496, 10, }, // Pacific/Wake
    {  688, 7405,  222506, 12, }, // Pacific/Wallis
    {  689, 6788,  222518,  7, }, // Africa/Abidjan Kuvi/Latin/India
    {  689, 6760,  222525,  4, }, // Africa/Accra
    {  689, 5059,  222529, 10, }, // Africa/Addis_Ababa
    {  689, 7420,  222539,  7, }, // Africa/Algiers
    {  689,   14,  222546,  6, }, // Africa/Asmara
    {  689,   44,  222552,  6, }, // Africa/Bamako
    {  689, 5257,  222558,  6, }, // Africa/Bangui
    {  689, 6746,  222564,  6, }, // Africa/Banjul
    {  689, 7435,  222570,  6, }, // Africa/Bissau
    {  689, 4957,  222576,  8, }, // Africa/Blantyre
    {  689, 5287,  222584,  9, }, // Africa/Brazzaville
    {  689, 4922,  222593,  9, }, // Africa/Bujumbura
    {  689, 2320,  222602,  5, }, // Africa/Cairo
    {  689, 4117,  222607, 10, }, // Africa/Casablanca
    {  689, 7449,  222617,  5, }, // Africa/Ceuta
    {  689, 6773,  222622,  7, }, // Africa/Conakry
    {  689, 6840,  222629,  4, }, // Africa/Dakar
    {  689, 5130,  222633, 13, }, // Africa/Dar_es_Salaam
    {  689, 5043,   26493,  6, }, // Africa/Djibouti
    {  689, 5243,  222646,  5, }, // Africa/Douala
    {  689, 5166,  222651,  9, }, // Africa/El_Aaiun
    {  689, 6853,  222660,  7, }, // Africa/Freetown
    {  689, 4906,  222667,  7, }, // Africa/Gaborone
    {  689, 5015,  222674,  5, }, // Africa/Harare
    {  689, 4479,  222679, 12, }, // Africa/Johannesburg
    {  689, 4499,  222691,  4, }, // Africa/Juba
    {  689, 5151,  222695,  6, }, // Africa/Kampala
    {  689, 4524,  222701,  7, }, // Africa/Khartoum
    {  689, 4973,  222708,  6, }, // Africa/Kigali
    {  689, 5306,  222714,  7, }, // Africa/Kinshasa
    {  689, 4744,  222721,  5, }, // Africa/Lagos
    {  689, 5336,  222726,  9, }, // Africa/Libreville
    {  689, 6869,    4087,  3, }, // Africa/Lome
    {  689, 5211,  222735,  6, }, // Africa/Luanda
    {  689, 4939,  222741,  9, }, // Africa/Lubumbashi
    {  689, 5001,  222750,  6, }, // Africa/Lusaka
    {  689, 5322,  222756,  6, }, // Africa/Malabo
    {  689, 4987,  222762,  6, }, // Africa/Maputo
    {  689, 5197,  222768,  6, }, // Africa/Maseru
    {  689, 5182,  222774,  7, }, // Africa/Mbabane
    {  689, 5113,  222781,  8, }, // Africa/Mogadishu
    {  689, 7034,  222789,  8, }, // Africa/Monrovia
    {  689, 3853,  222797,  7, }, // Africa/Nairobi
    {  689, 5271,  222804,  7, }, // Africa/Ndjamena
    {  689, 5354,  222811,  6, }, // Africa/Niamey
    {  689, 6803,  222817,  6, }, // Africa/Nouakchott
    {  689, 6727,  222823,  7, }, // Africa/Ouagadougou
    {  689, 5225,  222830, 10, }, // Africa/Porto-Novo
    {  689, 4435,  222840,  7, }, // Africa/Sao_Tome
    {  689, 2866,  222847,  7, }, // Africa/Tripoli
    {  689, 6554,  222854,  5, }, // Africa/Tunis
    {  689, 4152,  222859,  7, }, // Africa/Windhoek
    {  689,  132,  222866,  4, }, // America/Adak
    {  689, 3346,  222870,  7, }, // America/Anchorage
    {  689, 5607,  222877,  8, }, // America/Anguilla
    {  689, 5624,  222885,  7, }, // America/Antigua
    {  689, 4554,  222892,  9, }, // America/Araguaina
    {  689,  166,  222901, 13, }, // America/Argentina/Buenos_Aires
    {  689,   91,  222914,  9, }, // America/Argentina/Catamarca
    {  689,  270,  222923,  7, }, // America/Argentina/Cordoba
    {  689,  441,  222930,  5, }, // America/Argentina/Jujuy
    {  689, 7462,  222935,  8, }, // America/Argentina/La_Rioja
    {  689,  565,  222943,  7, }, // America/Argentina/Mendoza
    {  689, 7489,  222950, 13, }, // America/Argentina/Rio_Gallegos
    {  689, 7520,  222963,  5, }, // America/Argentina/Salta
    {  689, 7544,  222968,  8, }, // America/Argentina/San_Juan
    {  689, 5580,  222976,  8, }, // America/Argentina/San_Luis
    {  689, 7571,  222984,  6, }, // America/Argentina/Tucuman
    {  689, 7597,  222990,  7, }, // America/Argentina/Ushuaia
    {  689, 5640,  222997,  5, }, // America/Aruba
    {  689, 4252,  223002,  8, }, // America/Asuncion
    {  689,  237,  223010,  8, }, // America/Atikokan
    {  689, 3679,  223018,  5, }, // America/Bahia
    {  689, 7623,  223023, 13, }, // America/Bahia_Banderas
    {  689, 5654,  223036,  8, }, // America/Barbados
    {  689, 7646,  223044,  5, }, // America/Belem
    {  689, 5407,  223049,  5, }, // America/Belize
    {  689, 7660,  223054, 12, }, // America/Blanc-Sablon
    {  689, 7681,  223066,  9, }, // America/Boa_Vista
    {  689, 4361,  223075,  6, }, // America/Bogota
    {  689, 7699,  223081,  5, }, // America/Boise
    {  689, 7713,  223086, 10, }, // America/Cambridge_Bay
    {  689, 7735,  223096, 12, }, // America/Campo_Grande
    {  689, 3868,  223108,  6, }, // America/Cancun
    {  689, 4694,  223114,  7, }, // America/Caracas
    {  689, 4345,  223121,  5, }, // America/Cayenne
    {  689, 5496,  223126,  6, }, // America/Cayman
    {  689, 2260,  223132,  6, }, // America/Chicago
    {  689, 7756,  223138,  9, }, // America/Chihuahua
    {  689, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  689, 5422,  223147, 10, }, // America/Costa_Rica
    {  689, 7796,  223157,  7, }, // America/Creston
    {  689, 3791,  223164,  6, }, // America/Cuiaba
    {  689, 5723,  223170,  7, }, // America/Curacao
    {  689, 7812,  223177, 11, }, // America/Danmarkshavn
    {  689, 7833,  223188,  5, }, // America/Dawson
    {  689, 7848,  223193, 11, }, // America/Dawson_Creek
    {  689,  805,  223204,  6, }, // America/Denver
    {  689, 3465,  223210,  7, }, // America/Detroit
    {  689, 5739,  223217,  8, }, // America/Dominica
    {  689,  893,  223225,  8, }, // America/Edmonton
    {  689, 7869,  223233,  8, }, // America/Eirunepe
    {  689, 5441,  223241, 11, }, // America/El_Salvador
    {  689, 7886,  223252, 11, }, // America/Fort_Nelson
    {  689, 7906,  223263,  9, }, // America/Fortaleza
    {  689, 7924,  223272,  8, }, // America/Glace_Bay
    {  689, 6881,  223280,  7, }, // America/Goose_Bay
    {  689, 4612,  223287, 10, }, // America/Grand_Turk
    {  689, 5770,  223297,  7, }, // America/Grenada
    {  689, 5786,  223304, 10, }, // America/Guadeloupe
    {  689, 3760,  223314,  9, }, // America/Guatemala
    {  689, 4841,  223323,  8, }, // America/Guayaquil
    {  689, 6932,  223331,  6, }, // America/Guyana
    {  689, 1939,  223337,  8, }, // America/Halifax
    {  689, 2281,  223345,  6, }, // America/Havana
    {  689, 7942,    6920,  8, }, // America/Hermosillo
    {  689,  348,  223351, 12, }, // America/Indiana/Indianapolis
    {  689,  481,  223363, 14, }, // America/Indiana/Knox
    {  689, 7961,  223377, 17, }, // America/Indiana/Marengo
    {  689, 7985,  223394, 20, }, // America/Indiana/Petersburg
    {  689, 8012,  223414, 19, }, // America/Indiana/Tell_City
    {  689, 8038,  223433, 14, }, // America/Indiana/Vevay
    {  689, 8060,  223447, 17, }, // America/Indiana/Vincennes
    {  689, 8086,  223464, 17, }, // America/Indiana/Winamac
    {  689, 8110,  223481,  6, }, // America/Inuvik
    {  689,  660,  223487,  7, }, // America/Iqaluit
    {  689, 2799,  223494,  7, }, // America/Jamaica
    {  689, 5380,  223501,  5, }, // America/Juneau
    {  689,  521,  223506,  9, }, // America/Kentucky/Louisville
    {  689, 8125,  223515, 19, }, // America/Kentucky/Monticello
    {  689, 5704,  223534, 10, }, // America/Kralendijk
    {  689, 4376,  223544,  6, }, // America/La_Paz
    {  689, 7169,  157373,  4, }, // America/Lima
    {  689, 3239,  223550, 11, }, // America/Los_Angeles
    {  689, 5932,  223561, 21, }, // America/Lower_Princes
    {  689, 8153,  223582,  5, }, // America/Maceio
    {  689, 8168,  223587,  7, }, // America/Managua
    {  689, 1908,  223594,  6, }, // America/Manaus
    {  689, 5897,  223600,  7, }, // America/Marigot
    {  689, 5805,  223607,  8, }, // America/Martinique
    {  689, 8184,  223615,  9, }, // America/Matamoros
    {  689, 2917,  223624,  8, }, // America/Mazatlan
    {  689, 8202,  223632,  8, }, // America/Menominee
    {  689, 8220,  223640,  6, }, // America/Merida
    {  689, 8235,  223646, 10, }, // America/Metlakatla
    {  689, 2949,  223656, 12, }, // America/Mexico_City
    {  689, 4391,  223668,  7, }, // America/Miquelon
    {  689, 8254,  223675,  7, }, // America/Moncton
    {  689, 8270,  223682,  8, }, // America/Monterrey
    {  689, 4098,  223690, 10, }, // America/Montevideo
    {  689, 5824,  223700, 10, }, // America/Montserrat
    {  689, 5481,  223710,  5, }, // America/Nassau
    {  689, 2379,  223715,  9, }, // America/New_York
    {  689, 8288,    7120,  3, }, // America/Nome
    {  689, 1850,  223724,  7, }, // America/Noronha
    {  689, 8301,  223731, 20, }, // America/North_Dakota/Beulah
    {  689, 8329,  223751, 19, }, // America/North_Dakota/Center
    {  689, 8357,  223770, 23, }, // America/North_Dakota/New_Salem
    {  689,  393,  223793,  3, }, // America/Nuuk
    {  689, 8388,  223796,  7, }, // America/Ojinaga
    {  689, 2356,  223803,  6, }, // America/Panama
    {  689, 6173,  223809, 10, }, // America/Paramaribo
    {  689, 2973,  223819,  6, }, // America/Phoenix
    {  689, 3945,  223825, 14, }, // America/Port-au-Prince
    {  689, 5954,  223839, 13, }, // America/Port_of_Spain
    {  689, 8404,  223852, 11, }, // America/Porto_Velho
    {  689, 5843,  223863, 11, }, // America/Puerto_Rico
    {  689, 4030,  223874, 12, }, // America/Punta_Arenas
    {  689, 8424,  223886, 13, }, // America/Rankin_Inlet
    {  689, 8445,  223899,  6, }, // America/Recife
    {  689, 1995,  223905,  6, }, // America/Regina
    {  689, 8460,  223911,  8, }, // America/Resolute
    {  689,  695,  223919, 10, }, // America/Rio_Branco
    {  689, 8477,  223929,  6, }, // America/Santarem
    {  689, 2201,  223935,  8, }, // America/Santiago
    {  689, 6111,  223943, 13, }, // America/Santo_Domingo
    {  689, 1878,  223956,  9, }, // America/Sao_Paulo
    {  689, 6899,  223965, 11, }, // America/Scoresbysund
    {  689, 8494,  223976,  5, }, // America/Sitka
    {  689, 8508,  223981, 14, }, // America/St_Barthelemy
    {  689, 2061,  223995,  9, }, // America/St_Johns
    {  689, 5863,  224004, 10, }, // America/St_Kitts
    {  689, 5880,  224014, 10, }, // America/St_Lucia
    {  689,  855,  224024, 10, }, // America/St_Thomas
    {  689, 5913,  224034, 12, }, // America/St_Vincent
    {  689, 8530,  224046, 13, }, // America/Swift_Current
    {  689, 5461,  224059,  4, }, // America/Tegucigalpa
    {  689, 5756,  224063,  4, }, // America/Thule
    {  689,  313,  224067,  7, }, // America/Tijuana
    {  689,  608,  224074,  7, }, // America/Toronto
    {  689, 5688,  224081,  7, }, // America/Tortola
    {  689, 2093,  224088,  8, }, // America/Vancouver
    {  689, 2144,  224096,  9, }, // America/Whitehorse
    {  689,  734,  224105,  8, }, // America/Winnipeg
    {  689, 8552,  224113,  7, }, // America/Yakutat
    {  689, 6001,  224120,  5, }, // Antarctica/Casey
    {  689, 6094,  224125,  5, }, // Antarctica/Davis
    {  689, 6133,  224130, 17, }, // Antarctica/DumontDUrville
    {  689, 8568,  224147,  7, }, // Antarctica/Macquarie
    {  689, 7066,  224154,  6, }, // Antarctica/Mawson
    {  689, 7113,  224160,  9, }, // Antarctica/McMurdo
    {  689, 8589,  224169,  6, }, // Antarctica/Palmer
    {  689, 7226,  224175,  6, }, // Antarctica/Rothera
    {  689, 7295,  224181,  5, }, // Antarctica/Syowa
    {  689, 8607,  224186,  5, }, // Antarctica/Troll
    {  689, 7374,  224191,  6, }, // Antarctica/Vostok
    {  689, 1429,  224197, 10, }, // Arctic/Longyearbyen
    {  689, 5570,  224207,  4, }, // Asia/Aden
    {  689, 5395,  224211,  6, }, // Asia/Almaty
    {  689, 3968,  224217,  5, }, // Asia/Amman
    {  689, 5511,  224222,  6, }, // Asia/Anadyr
    {  689, 5523,  224228,  5, }, // Asia/Aqtau
    {  689, 5368,  224233,  7, }, // Asia/Aqtobe
    {  689,  964,  224240,  7, }, // Asia/Ashgabat
    {  689, 8624,  224247,  6, }, // Asia/Atyrau
    {  689, 3607,  224253,  6, }, // Asia/Baghdad
    {  689, 5534,  224259,  7, }, // Asia/Bahrain
    {  689, 3653,  224266,  4, }, // Asia/Baku
    {  689, 4466,  224270,  7, }, // Asia/Bangkok
    {  689, 3571,  224277,  7, }, // Asia/Barnaul
    {  689, 4086,  224284,  6, }, // Asia/Beirut
    {  689, 3778,  224290,  6, }, // Asia/Bishkek
    {  689, 5989,  224296,  6, }, // Asia/Brunei
    {  689, 4601,  224302,  4, }, // Asia/Chita
    {  689, 4511,  224306,  7, }, // Asia/Colombo
    {  689, 4540,  224313,  8, }, // Asia/Damascus
    {  689, 1093,  222867,  4, }, // Asia/Dhaka
    {  689, 6192,  224321,  4, }, // Asia/Dili
    {  689, 3596,  224325,  5, }, // Asia/Dubai
    {  689, 6159,  224330,  7, }, // Asia/Dushanbe
    {  689, 8636,  224337,  9, }, // Asia/Famagusta
    {  689, 8651,  224346,  4, }, // Asia/Gaza
    {  689, 4795,  224350,  6, }, // Asia/Hebron
    {  689, 1259,  224356, 15, }, // Asia/Ho_Chi_Minh
    {  689, 2704,  224371,  9, }, // Asia/Hong_Kong
    {  689, 4771,   26836,  4, }, // Asia/Hovd
    {  689, 4184,  224380,  7, }, // Asia/Irkutsk
    {  689, 7006,    4248,  7, }, // Asia/Jakarta
    {  689, 6992,  224387,  8, }, // Asia/Jayapura
    {  689, 1290,  224395,  9, }, // Asia/Jerusalem
    {  689, 3560,  224404,  5, }, // Asia/Kabul
    {  689, 4316,  224409,  8, }, // Asia/Kamchatka
    {  689, 4239,  224417,  6, }, // Asia/Karachi
    {  689, 1185,  224423,  8, }, // Asia/Kathmandu
    {  689, 8661,  224431,  7, }, // Asia/Khandyga
    {  689,  992,  224438,  7, }, // Asia/Kolkata
    {  689, 4197,  224445, 11, }, // Asia/Krasnoyarsk
    {  689, 4859,  224456, 12, }, // Asia/Kuala_Lumpur
    {  689, 5976,  224468,  6, }, // Asia/Kuching
    {  689, 5547,  224474,  6, }, // Asia/Kuwait
    {  689, 1211,  224480,  4, }, // Asia/Macau
    {  689, 4017,  224484,  7, }, // Asia/Magadan
    {  689, 1349,  224491,  7, }, // Asia/Makassar
    {  689, 7182,  224498,  6, }, // Asia/Manila
    {  689, 6920,  224504,  6, }, // Asia/Muscat
    {  689, 2554,  224510,  8, }, // Asia/Nicosia
    {  689, 8675,  224518, 12, }, // Asia/Novokuznetsk
    {  689, 4135,  224530, 11, }, // Asia/Novosibirsk
    {  689, 4229,  123801,  4, }, // Asia/Omsk
    {  689, 7145,   91266,  4, }, // Asia/Oral
    {  689, 6961,  224541,  9, }, // Asia/Phnom_Penh
    {  689, 8693,  224550,  9, }, // Asia/Pontianak
    {  689, 4214,  224559,  9, }, // Asia/Pyongyang
    {  689, 5559,  210375,  5, }, // Asia/Qatar
    {  689, 8708,  224568,  7, }, // Asia/Qostanay
    {  689, 4269,  224575,  8, }, // Asia/Qyzylorda
    {  689, 3584,  224583,  5, }, // Asia/Riyadh
    {  689, 4408,  224588,  7, }, // Asia/Sakhalin
    {  689, 7245,  224595,  9, }, // Asia/Samarkand
    {  689, 3279,  224604,  4, }, // Asia/Seoul
    {  689, 1053,  224608,  6, }, // Asia/Shanghai
    {  689, 3300,  224614,  8, }, // Asia/Singapore
    {  689, 4297,  224622, 13, }, // Asia/Srednekolymsk
    {  689, 3263,  224635,  6, }, // Asia/Taipei
    {  689, 4781,  224641,  7, }, // Asia/Tashkent
    {  689, 3915,  224648,  7, }, // Asia/Tbilisi
    {  689, 2772,  224655,  6, }, // Asia/Tehran
    {  689, 1317,  224661,  6, }, // Asia/Thimphu
    {  689, 2821,  224667,  5, }, // Asia/Tokyo
    {  689, 4572,  224672,  5, }, // Asia/Tomsk
    {  689, 1021,  224677,  9, }, // Asia/Ulaanbaatar
    {  689, 1159,  224686,  6, }, // Asia/Urumqi
    {  689, 8722,  224692,  9, }, // Asia/Ust-Nera
    {  689, 6977,  224701,  9, }, // Asia/Vientiane
    {  689, 4710,  224710, 11, }, // Asia/Vladivostok
    {  689, 4828,  224721,  7, }, // Asia/Yakutsk
    {  689, 1235,  224728,  6, }, // Asia/Yangon
    {  689, 3883,  224734, 13, }, // Asia/Yekaterinburg
    {  689, 3747,  224747,  7, }, // Asia/Yerevan
    {  689, 3663,  224754,  6, }, // Atlantic/Azores
    {  689, 5671,  224760,  7, }, // Atlantic/Bermuda
    {  689, 6628,  224767,  6, }, // Atlantic/Canary
    {  689, 3727,  224773,  9, }, // Atlantic/Cape_Verde
    {  689, 1395,  224782,  4, }, // Atlantic/Faroe
    {  689, 8736,  224786,  7, }, // Atlantic/Madeira
    {  689, 2748,  224793,  9, }, // Atlantic/Reykjavik
    {  689, 7272,  224802, 11, }, // Atlantic/South_Georgia
    {  689, 6821,  224813, 11, }, // Atlantic/St_Helena
    {  689, 6644,  224824,  6, }, // Atlantic/Stanley
    {  689, 1670,  224830,  8, }, // Australia/Adelaide
    {  689, 1635,  224838,  8, }, // Australia/Brisbane
    {  689, 1799,  224846, 11, }, // Australia/Broken_Hill
    {  689, 1583,  224857,  6, }, // Australia/Darwin
    {  689, 3637,  224863,  5, }, // Australia/Eucla
    {  689, 1516,  224868,  6, }, // Australia/Hobart
    {  689, 8753,  224874,  9, }, // Australia/Lindeman
    {  689, 1547,  224883,  9, }, // Australia/Lord_Howe
    {  689, 1727,  224892,  8, }, // Australia/Melbourne
    {  689, 1762,  224900,  4, }, // Australia/Perth
    {  689, 1463,  224904,  5, }, // Australia/Sydney
    {  689, 6441,  224909,  9, }, // Europe/Amsterdam
    {  689, 6216,  224918,  7, }, // Europe/Andorra
    {  689, 3620,  224925,  9, }, // Europe/Astrakhan
    {  689, 2300,  224934,  5, }, // Europe/Athens
    {  689, 6336,  224939,  8, }, // Europe/Belgrade
    {  689, 4757,  224947,  6, }, // Europe/Berlin
    {  689, 6488,  224953, 10, }, // Europe/Bratislava
    {  689, 2167,  224963,  8, }, // Europe/Brussels
    {  689, 3928,  224971,  8, }, // Europe/Bucharest
    {  689, 3806,  224979,  8, }, // Europe/Budapest
    {  689, 8772,  224987,  8, }, // Europe/Busingen
    {  689, 2583,  224995,  7, }, // Europe/Chisinau
    {  689, 6289,  225002, 10, }, // Europe/Copenhagen
    {  689, 2338,  225012,  5, }, // Europe/Dublin
    {  689, 6307,  225017,  9, }, // Europe/Gibraltar
    {  689, 8788,  225026,  6, }, // Europe/Guernsey
    {  689, 6612,  225032,  8, }, // Europe/Helsinki
    {  689, 8804,  225040, 11, }, // Europe/Isle_of_Man
    {  689, 1130,  225051,  8, }, // Europe/Istanbul
    {  689, 8823,  225059,  5, }, // Europe/Jersey
    {  689, 3979,  225064, 11, }, // Europe/Kaliningrad
    {  689, 8929,  225075,  5, }, // Europe/Kirov
    {  689, 2527,  225080,  4, }, // Europe/Kyiv
    {  689, 3213,  225084,  6, }, // Europe/Lisbon
    {  689, 6506,  225090,  9, }, // Europe/Ljubljana
    {  689, 2501,  225099,  4, }, // Europe/London
    {  689, 6365,  225103,  8, }, // Europe/Luxembourg
    {  689, 4892,  225111,  6, }, // Europe/Madrid
    {  689, 6397,  225117,  5, }, // Europe/Malta
    {  689, 6582,  225122,  8, }, // Europe/Mariehamn
    {  689, 3693,  225130,  5, }, // Europe/Minsk
    {  689, 6410,  225135,  6, }, // Europe/Monaco
    {  689, 3537,  225141,  5, }, // Europe/Moscow
    {  689, 6458,  225146,  4, }, // Europe/Oslo
    {  689, 4284,  225150,  5, }, // Europe/Paris
    {  689, 6424,  225155,  9, }, // Europe/Podgorica
    {  689, 6275,  225164,  4, }, // Europe/Prague
    {  689, 8837,  225168,  4, }, // Europe/Riga
    {  689, 6324,   27088,  3, }, // Europe/Rome
    {  689, 4331,  225172,  6, }, // Europe/Samara
    {  689, 6470,  225178, 10, }, // Europe/San_Marino
    {  689, 6245,  225188,  8, }, // Europe/Sarajevo
    {  689, 4451,  225196,  7, }, // Europe/Saratov
    {  689, 8849,  225203, 10, }, // Europe/Simferopol
    {  689, 6383,  225213,  5, }, // Europe/Skopje
    {  689, 6599,  225218,  6, }, // Europe/Sofia
    {  689, 6523,  225224,  7, }, // Europe/Stockholm
    {  689, 8867,  225231,  5, }, // Europe/Tallinn
    {  689, 6202,  225236,  6, }, // Europe/Tirane
    {  689, 8882,  225242,  9, }, // Europe/Ulyanovsk
    {  689, 6352,  225251,  5, }, // Europe/Vaduz
    {  689, 6567,  225256,  7, }, // Europe/Vatican
    {  689, 6231,  225263,  6, }, // Europe/Vienna
    {  689, 8899,  225269,  7, }, // Europe/Vilnius
    {  689, 4727,  225276,  9, }, // Europe/Volgograd
    {  689, 3190,  225285,  5, }, // Europe/Warsaw
    {  689, 6261,  225290, 11, }, // Europe/Zagreb
    {  689, 6540,  225301,  5, }, // Europe/Zurich
    {  689, 5078,  225306,  9, }, // Indian/Antananarivo
    {  689, 6947,  225315,  6, }, // Indian/Chagos
    {  689, 6046,  225321,  8, }, // Indian/Christmas
    {  689, 6063,  225329,  5, }, // Indian/Cocos
    {  689, 5029,  225334,  6, }, // Indian/Comoro
    {  689, 6661,  225340,  9, }, // Indian/Kerguelen
    {  689, 7260,  225349,  4, }, // Indian/Mahe
    {  689, 7050,  225353,  8, }, // Indian/Maldives
    {  689, 4069,  225361,  9, }, // Indian/Mauritius
    {  689, 5098,  225370,  5, }, // Indian/Mayotte
    {  689, 7211,  225375,  7, }, // Indian/Reunion
    {  689, 4422,  225382,  4, }, // Pacific/Apia
    {  689,  932,  225386,  6, }, // Pacific/Auckland
    {  689, 3706,  225392, 10, }, // Pacific/Bougainville
    {  689, 3015,  225402,  5, }, // Pacific/Chatham
    {  689, 3157,  225407,  3, }, // Pacific/Chuuk
    {  689, 2237,  225410,  5, }, // Pacific/Easter
    {  689, 7360,  225415,  5, }, // Pacific/Efate
    {  689, 7327,  225420,  7, }, // Pacific/Fakaofo
    {  689, 3902,  225427,  4, }, // Pacific/Fiji
    {  689, 7343,  225431,  8, }, // Pacific/Funafuti
    {  689, 6678,  225439,  9, }, // Pacific/Galapagos
    {  689, 6696,  225448,  7, }, // Pacific/Gambier
    {  689, 3822,  225455, 11, }, // Pacific/Guadalcanal
    {  689, 6018,  223591,  4, }, // Pacific/Guam
    {  689, 3049,  225466,  6, }, // Pacific/Kanton
    {  689, 3998,  225472, 10, }, // Pacific/Kiritimati
    {  689, 7019,  225482,  6, }, // Pacific/Kosrae
    {  689, 2842,  225488,  9, }, // Pacific/Kwajalein
    {  689, 4877,  225497,  6, }, // Pacific/Majuro
    {  689, 4051,  225503,  8, }, // Pacific/Marquesas
    {  689, 8914,  225511,  6, }, // Pacific/Midway
    {  689, 7084,  225517,  5, }, // Pacific/Nauru
    {  689, 7132,  225522,  4, }, // Pacific/Niue
    {  689, 4168,  225526,  6, }, // Pacific/Norfolk
    {  689, 7098,  225532,  5, }, // Pacific/Noumea
    {  689, 3126,  225537, 11, }, // Pacific/Pago_Pago
    {  689, 7155,  225548,  5, }, // Pacific/Palau
    {  689, 7194,  225553,  7, }, // Pacific/Pitcairn
    {  689, 3096,  225560,  6, }, // Pacific/Pohnpei
    {  689, 4807,  225566, 12, }, // Pacific/Port_Moresby
    {  689, 6076,  225578,  9, }, // Pacific/Rarotonga
    {  689, 6031,  225587,  6, }, // Pacific/Saipan
    {  689, 7312,  225593,  6, }, // Pacific/Tahiti
    {  689, 6712,  225599,  6, }, // Pacific/Tarawa
    {  689, 4583,  225605,  9, }, // Pacific/Tongatapu
    {  689, 7392,  225614,  3, }, // Pacific/Wake
    {  689, 7405,  225617,  6, }, // Pacific/Wallis
    {  690, 6788,   29964,  7, }, // Africa/Abidjan Kuvi/Devanagari/India
    {  690, 6760,   88252,  5, }, // Africa/Accra
    {  690, 5059,   88257, 10, }, // Africa/Addis_Ababa
    {  690, 7420,   88267,  9, }, // Africa/Algiers
    {  690,   14,   88276,  7, }, // Africa/Asmara
    {  690,   44,   88283,  6, }, // Africa/Bamako
    {  690, 5257,   88289,  6, }, // Africa/Bangui
    {  690, 6746,  225623,  9, }, // Africa/Banjul
    {  690, 7435,   88301,  5, }, // Africa/Bissau
    {  690, 4957,   88306,  9, }, // Africa/Blantyre
    {  690, 5287,   88315, 11, }, // Africa/Brazzaville
    {  690, 4922,   88326,  9, }, // Africa/Bujumbura
    {  690, 2320,   88335,  5, }, // Africa/Cairo
    {  690, 4117,   30065, 11, }, // Africa/Casablanca
    {  690, 7449,   88340,  5, }, // Africa/Ceuta
    {  690, 6773,   88345,  8, }, // Africa/Conakry
    {  690, 6840,   88353,  4, }, // Africa/Dakar
    {  690, 5130,   88357, 11, }, // Africa/Dar_es_Salaam
    {  690, 5043,   88368,  6, }, // Africa/Djibouti
    {  690, 5243,   88374,  5, }, // Africa/Douala
    {  690, 5166,   88379,  8, }, // Africa/El_Aaiun
    {  690, 6853,  225632,  9, }, // Africa/Freetown
    {  690, 4906,   88396,  7, }, // Africa/Gaborone
    {  690, 5015,   88403,  5, }, // Africa/Harare
    {  690, 4479,   88408, 10, }, // Africa/Johannesburg
    {  690, 4499,   30154,  4, }, // Africa/Juba
    {  690, 5151,   88418,  6, }, // Africa/Kampala
    {  690, 4524,  225641,  7, }, // Africa/Khartoum
    {  690, 4973,   88424,  6, }, // Africa/Kigali
    {  690, 5306,  225648,  7, }, // Africa/Kinshasa
    {  690, 4744,   88437,  5, }, // Africa/Lagos
    {  690, 5336,   88442, 10, }, // Africa/Libreville
    {  690, 6869,   88452,  3, }, // Africa/Lome
    {  690, 5211,   88455,  6, }, // Africa/Luanda
    {  690, 4939,  225655,  9, }, // Africa/Lubumbashi
    {  690, 5001,   30221,  6, }, // Africa/Lusaka
    {  690, 5322,   88470,  5, }, // Africa/Malabo
    {  690, 4987,   88475,  6, }, // Africa/Maputo
    {  690, 5197,   30239,  6, }, // Africa/Maseru
    {  690, 5182,  225664,  9, }, // Africa/Mbabane
    {  690, 5113,  225673,  8, }, // Africa/Mogadishu
    {  690, 7034,   88496,  9, }, // Africa/Monrovia
    {  690, 3853,  225681,  7, }, // Africa/Nairobi
    {  690, 5271,   88513,  8, }, // Africa/Ndjamena
    {  690, 5354,   88521,  6, }, // Africa/Niamey
    {  690, 6803,  225688,  7, }, // Africa/Nouakchott
    {  690, 6727,   88534,  7, }, // Africa/Ouagadougou
    {  690, 5225,   88541, 11, }, // Africa/Porto-Novo
    {  690, 4435,   88552,  7, }, // Africa/Sao_Tome
    {  690, 2866,   88559,  8, }, // Africa/Tripoli
    {  690, 6554,   30328,  7, }, // Africa/Tunis
    {  690, 4152,   88567,  7, }, // Africa/Windhoek
    {  690,  132,   88574,  3, }, // America/Adak
    {  690, 3346,   30347,  6, }, // America/Anchorage
    {  690, 5607,   88577,  8, }, // America/Anguilla
    {  690, 5624,   88585,  7, }, // America/Antigua
    {  690, 4554,   88592,  9, }, // America/Araguaina
    {  690,  166,   88601, 11, }, // America/Argentina/Buenos_Aires
    {  690,   91,   30390, 10, }, // America/Argentina/Catamarca
    {  690,  270,   88612,  8, }, // America/Argentina/Cordoba
    {  690,  441,   88620,  5, }, // America/Argentina/Jujuy
    {  690, 7462,   88625,  8, }, // America/Argentina/La_Rioja
    {  690,  565,   88633,  8, }, // America/Argentina/Mendoza
    {  690, 7489,   88641, 12, }, // America/Argentina/Rio_Gallegos
    {  690, 7520,   30437,  6, }, // America/Argentina/Salta
    {  690, 7544,  225695, 11, }, // America/Argentina/San_Juan
    {  690, 5580,  225706, 10, }, // America/Argentina/San_Luis
    {  690, 7571,   88669,  6, }, // America/Argentina/Tucuman
    {  690, 7597,  225716,  7, }, // America/Argentina/Ushuaia
    {  690, 5640,   88682,  5, }, // America/Aruba
    {  690, 4252,  225723,  8, }, // America/Asuncion
    {  690,  237,   88695,  8, }, // America/Atikokan
    {  690, 3679,   88703,  5, }, // America/Bahia
    {  690, 7623,   88708, 15, }, // America/Bahia_Banderas
    {  690, 5654,   88723,  8, }, // America/Barbados
    {  690, 7646,   30526,  5, }, // America/Belem
    {  690, 5407,   88731,  6, }, // America/Belize
    {  690, 7660,  225731, 12, }, // America/Blanc-Sablon
    {  690, 7681,   88750, 10, }, // America/Boa_Vista
    {  690, 4361,   88760,  6, }, // America/Bogota
    {  690, 7699,   88766,  5, }, // America/Boise
    {  690, 7713,  225743, 12, }, // America/Cambridge_Bay
    {  690, 7735,  225755, 16, }, // America/Campo_Grande
    {  690, 3868,  225771,  8, }, // America/Cancun
    {  690, 4694,   88805,  6, }, // America/Caracas
    {  690, 4345,   30609,  5, }, // America/Cayenne
    {  690, 5496,  138409,  7, }, // America/Cayman
    {  690, 2260,  225779,  6, }, // America/Chicago
    {  690, 7756,   88817,  8, }, // America/Chihuahua
    {  690, 5422,   88841, 11, }, // America/Costa_Rica
    {  690, 7796,   30642,  8, }, // America/Creston
    {  690, 3791,   88852,  7, }, // America/Cuiaba
    {  690, 5723,   88859,  7, }, // America/Curacao
    {  690, 7812,  225785, 11, }, // America/Danmarkshavn
    {  690, 7833,   88877,  4, }, // America/Dawson
    {  690, 7848,   88881, 10, }, // America/Dawson_Creek
    {  690,  805,   88891,  5, }, // America/Denver
    {  690, 3465,   88896,  8, }, // America/Detroit
    {  690, 5739,   88904,  8, }, // America/Dominica
    {  690,  893,   88912,  6, }, // America/Edmonton
    {  690, 7869,  225796,  8, }, // America/Eirunepe
    {  690, 5441,   88918, 11, }, // America/El_Salvador
    {  690, 7886,  225804, 13, }, // America/Fort_Nelson
    {  690, 7906,  225817, 12, }, // America/Fortaleza
    {  690, 7924,   30760,  8, }, // America/Glace_Bay
    {  690, 6881,  113910,  6, }, // America/Goose_Bay
    {  690, 4612,   88974, 11, }, // America/Grand_Turk
    {  690, 5770,   30789,  8, }, // America/Grenada
    {  690, 5786,   88985,  9, }, // America/Guadeloupe
    {  690, 3760,   30806, 10, }, // America/Guatemala
    {  690, 4841,   88994,  9, }, // America/Guayaquil
    {  690, 6932,   89003,  5, }, // America/Guyana
    {  690, 1939,  225829,  9, }, // America/Halifax
    {  690, 2281,   89018,  5, }, // America/Havana
    {  690, 7942,   89023, 11, }, // America/Hermosillo
    {  690,  348,   89034, 13, }, // America/Indiana/Indianapolis
    {  690,  481,  113975, 15, }, // America/Indiana/Knox
    {  690, 7961,   89062, 17, }, // America/Indiana/Marengo
    {  690, 7985,   89079, 20, }, // America/Indiana/Petersburg
    {  690, 8012,   89099, 18, }, // America/Indiana/Tell_City
    {  690, 8038,   89117, 14, }, // America/Indiana/Vevay
    {  690, 8060,   89131, 18, }, // America/Indiana/Vincennes
    {  690, 8086,   89149, 17, }, // America/Indiana/Winamac
    {  690, 8110,   30999,  6, }, // America/Inuvik
    {  690,  660,  225838,  8, }, // America/Iqaluit
    {  690, 2799,  138734,  6, }, // America/Jamaica
    {  690, 5380,   89178,  7, }, // America/Juneau
    {  690,  521,   89185,  8, }, // America/Kentucky/Louisville
    {  690, 8125,   89193, 17, }, // America/Kentucky/Monticello
    {  690, 5704,  225846, 13, }, // America/Kralendijk
    {  690, 4376,   89222,  7, }, // America/La_Paz
    {  690, 7169,   31064,  4, }, // America/Lima
    {  690, 3239,   89229, 11, }, // America/Los_Angeles
    {  690, 5932,   89240, 22, }, // America/Lower_Princes
    {  690, 8153,   89262,  5, }, // America/Maceio
    {  690, 8168,   89267,  7, }, // America/Managua
    {  690, 1908,  225859,  4, }, // America/Manaus
    {  690, 5897,  225863,  7, }, // America/Marigot
    {  690, 5805,   89285,  9, }, // America/Martinique
    {  690, 8184,   89294,  9, }, // America/Matamoros
    {  690, 2917,   89303,  9, }, // America/Mazatlan
    {  690, 8202,   89312,  8, }, // America/Menominee
    {  690, 8220,   31158,  6, }, // America/Merida
    {  690, 8235,   89320, 12, }, // America/Metlakatla
    {  690, 2949,   89332, 13, }, // America/Mexico_City
    {  690, 4391,   89345,  7, }, // America/Miquelon
    {  690, 8254,   89352,  6, }, // America/Moncton
    {  690, 8270,   89358,  9, }, // America/Monterrey
    {  690, 4098,   89367, 11, }, // America/Montevideo
    {  690, 5824,   89378,  9, }, // America/Montserrat
    {  690, 5481,   89387,  5, }, // America/Nassau
    {  690, 2379,   89392,  9, }, // America/New_York
    {  690, 8288,   89314,  3, }, // America/Nome
    {  690, 1850,   89401,  8, }, // America/Noronha
    {  690, 8301,  225870, 19, }, // America/North_Dakota/Beulah
    {  690, 8329,  225889, 18, }, // America/North_Dakota/Center
    {  690, 8357,  225907, 23, }, // America/North_Dakota/New_Salem
    {  690,  393,   89472,  3, }, // America/Nuuk
    {  690, 8388,  225930,  9, }, // America/Ojinaga
    {  690, 2356,   89484,  5, }, // America/Panama
    {  690, 6173,   89489, 10, }, // America/Paramaribo
    {  690, 2973,  225939,  8, }, // America/Phoenix
    {  690, 3945,   89507, 14, }, // America/Port-au-Prince
    {  690, 5954,  225947, 15, }, // America/Port_of_Spain
    {  690, 8404,   89536, 13, }, // America/Porto_Velho
    {  690, 5843,   89549, 11, }, // America/Puerto_Rico
    {  690, 4030,   89560, 12, }, // America/Punta_Arenas
    {  690, 8424,   89572, 12, }, // America/Rankin_Inlet
    {  690, 8445,  225962,  7, }, // America/Recife
    {  690, 1995,   31431,  6, }, // America/Regina
    {  690, 8460,   89591,  9, }, // America/Resolute
    {  690,  695,   89600, 12, }, // America/Rio_Branco
    {  690, 8477,  225969,  9, }, // America/Santarem
    {  690, 2201,  225978,  8, }, // America/Santiago
    {  690, 6111,   89629, 13, }, // America/Santo_Domingo
    {  690, 1878,   89642,  9, }, // America/Sao_Paulo
    {  690, 6899,   89651, 13, }, // America/Scoresbysund
    {  690, 8494,   89664,  6, }, // America/Sitka
    {  690, 8508,  225986, 15, }, // America/St_Barthelemy
    {  690, 2061,  114496, 10, }, // America/St_Johns
    {  690, 5863,   89694, 10, }, // America/St_Kitts
    {  690, 5880,   89704, 11, }, // America/St_Lucia
    {  690,  855,  226001,  9, }, // America/St_Thomas
    {  690, 5913,   89724, 12, }, // America/St_Vincent
    {  690, 8530,  226010, 13, }, // America/Swift_Current
    {  690, 5461,   89749, 11, }, // America/Tegucigalpa
    {  690, 5756,  129890,  4, }, // America/Thule
    {  690,  313,   89766,  7, }, // America/Tijuana
    {  690,  608,   89773,  6, }, // America/Toronto
    {  690, 5688,   89779,  8, }, // America/Tortola
    {  690, 2093,  226023, 10, }, // America/Vancouver
    {  690, 2144,   89794, 11, }, // America/Whitehorse
    {  690,  734,   89805,  7, }, // America/Winnipeg
    {  690, 8552,   89812,  7, }, // America/Yakutat
    {  690, 6001,   31673,  4, }, // Antarctica/Casey
    {  690, 6094,   89819,  5, }, // Antarctica/Davis
    {  690, 6133,   89824, 19, }, // Antarctica/DumontDUrville
    {  690, 8568,   89843,  7, }, // Antarctica/Macquarie
    {  690, 7066,   89850,  4, }, // Antarctica/Mawson
    {  690, 7113,  226033, 11, }, // Antarctica/McMurdo
    {  690, 8589,   89863,  4, }, // Antarctica/Palmer
    {  690, 7226,   89867,  6, }, // Antarctica/Rothera
    {  690, 7295,   89873,  6, }, // Antarctica/Syowa
    {  690, 8607,   89879,  5, }, // Antarctica/Troll
    {  690, 7374,   89884,  7, }, // Antarctica/Vostok
    {  690, 1429,  226044, 12, }, // Arctic/Longyearbyen
    {  690, 5570,   89904,  4, }, // Asia/Aden
    {  690, 5395,   89908,  7, }, // Asia/Almaty
    {  690, 3968,   89915,  6, }, // Asia/Amman
    {  690, 5511,   89921,  6, }, // Asia/Anadyr
    {  690, 5523,   31778,  6, }, // Asia/Aqtau
    {  690, 5368,  114670,  7, }, // Asia/Aqtobe
    {  690,  964,  226056,  8, }, // Asia/Ashgabat
    {  690, 8624,   89933,  5, }, // Asia/Atyrau
    {  690, 3607,   89938,  5, }, // Asia/Baghdad
    {  690, 5534,   89943,  5, }, // Asia/Bahrain
    {  690, 3653,   88857,  4, }, // Asia/Baku
    {  690, 4466,  226064,  9, }, // Asia/Bangkok
    {  690, 3571,   89954,  6, }, // Asia/Barnaul
    {  690, 4086,   89960,  5, }, // Asia/Beirut
    {  690, 3778,  226073,  7, }, // Asia/Bishkek
    {  690, 5989,  226080,  8, }, // Asia/Brunei
    {  690, 4601,  226088,  6, }, // Asia/Chita
    {  690, 4511,   89978,  6, }, // Asia/Colombo
    {  690, 4540,   89984,  7, }, // Asia/Damascus
    {  690, 1093,   89991,  4, }, // Asia/Dhaka
    {  690, 6192,   89995,  4, }, // Asia/Dili
    {  690, 3596,  226094,  5, }, // Asia/Dubai
    {  690, 6159,  226099,  7, }, // Asia/Dushanbe
    {  690, 8636,  226106, 11, }, // Asia/Famagusta
    {  690, 8651,   90021,  5, }, // Asia/Gaza
    {  690, 4795,   90026,  7, }, // Asia/Hebron
    {  690, 1259,   90033, 16, }, // Asia/Ho_Chi_Minh
    {  690, 2704,   90049,  9, }, // Asia/Hong_Kong
    {  690, 4771,   90058,  5, }, // Asia/Hovd
    {  690, 4184,   90063, 10, }, // Asia/Irkutsk
    {  690, 7006,   90073,  7, }, // Asia/Jakarta
    {  690, 6992,   31943,  7, }, // Asia/Jayapura
    {  690, 1290,  226117,  8, }, // Asia/Jerusalem
    {  690, 3560,   90086,  5, }, // Asia/Kabul
    {  690, 4316,   90091,  7, }, // Asia/Kamchatka
    {  690, 4239,   31972,  5, }, // Asia/Karachi
    {  690, 1185,   90098,  8, }, // Asia/Kathmandu
    {  690, 8661,  226125,  7, }, // Asia/Khandyga
    {  690,  992,   90113,  7, }, // Asia/Kolkata
    {  690, 4197,   90120, 15, }, // Asia/Krasnoyarsk
    {  690, 4859,   90135, 10, }, // Asia/Kuala_Lumpur
    {  690, 5976,   32028,  6, }, // Asia/Kuching
    {  690, 5547,  114816,  5, }, // Asia/Kuwait
    {  690, 1211,   90145,  4, }, // Asia/Macau
    {  690, 4017,   32044,  7, }, // Asia/Magadan
    {  690, 1349,   90149,  6, }, // Asia/Makassar
    {  690, 7182,   32060,  5, }, // Asia/Manila
    {  690, 6920,   90155,  5, }, // Asia/Muscat
    {  690, 2554,   90160,  8, }, // Asia/Nicosia
    {  690, 8675,   90168, 16, }, // Asia/Novokuznetsk
    {  690, 4135,   90184, 13, }, // Asia/Novosibirsk
    {  690, 4229,   90197,  6, }, // Asia/Omsk
    {  690, 7145,   90203,  3, }, // Asia/Oral
    {  690, 6961,  226132, 10, }, // Asia/Phnom_Penh
    {  690, 8693,   90215,  9, }, // Asia/Pontianak
    {  690, 4214,   90224, 10, }, // Asia/Pyongyang
    {  690, 5559,   90234,  3, }, // Asia/Qatar
    {  690, 8708,   90237,  8, }, // Asia/Qostanay
    {  690, 4269,  226142, 10, }, // Asia/Qyzylorda
    {  690, 3584,   90256,  5, }, // Asia/Riyadh
    {  690, 4408,  226152,  7, }, // Asia/Sakhalin
    {  690, 7245,   90267,  6, }, // Asia/Samarkand
    {  690, 3279,   90273,  4, }, // Asia/Seoul
    {  690, 1053,  226159,  6, }, // Asia/Shanghai
    {  690, 3300,   32190,  8, }, // Asia/Singapore
    {  690, 4297,   90282, 16, }, // Asia/Srednekolymsk
    {  690, 3263,  226165,  8, }, // Asia/Taipei
    {  690, 4781,  226173,  6, }, // Asia/Tashkent
    {  690, 3915,  226179,  9, }, // Asia/Tbilisi
    {  690, 2772,   32236,  6, }, // Asia/Tehran
    {  690, 1317,   90317,  5, }, // Asia/Thimphu
    {  690, 2821,   90322,  6, }, // Asia/Tokyo
    {  690, 4572,   90328,  7, }, // Asia/Tomsk
    {  690, 1021,   90335,  8, }, // Asia/Ulaanbaatar
    {  690, 1159,   90343,  7, }, // Asia/Urumqi
    {  690, 8722,   90350, 10, }, // Asia/Ust-Nera
    {  690, 6977,   90360,  9, }, // Asia/Vientiane
    {  690, 4710,   90369, 13, }, // Asia/Vladivostok
    {  690, 4828,   32305,  9, }, // Asia/Yakutsk
    {  690, 1235,  226188,  6, }, // Asia/Yangon
    {  690, 3883,   90382, 13, }, // Asia/Yekaterinburg
    {  690, 3747,   90395,  7, }, // Asia/Yerevan
    {  690, 3663,   90402,  7, }, // Atlantic/Azores
    {  690, 5671,   90409,  6, }, // Atlantic/Bermuda
    {  690, 6628,  226194,  8, }, // Atlantic/Canary
    {  690, 3727,   90421,  8, }, // Atlantic/Cape_Verde
    {  690, 1395,  226202,  6, }, // Atlantic/Faroe
    {  690, 8736,   90434,  5, }, // Atlantic/Madeira
    {  690, 2748,   90439,  9, }, // Atlantic/Reykjavik
    {  690, 7272,  226208, 18, }, // Atlantic/South_Georgia
    {  690, 6821,   90463, 11, }, // Atlantic/St_Helena
    {  690, 6644,  226226,  9, }, // Atlantic/Stanley
    {  690, 1670,   90481,  6, }, // Australia/Adelaide
    {  690, 1635,   90487,  8, }, // Australia/Brisbane
    {  690, 1799,   90495, 10, }, // Australia/Broken_Hill
    {  690, 1583,   90505,  7, }, // Australia/Darwin
    {  690, 3637,   32452,  6, }, // Australia/Eucla
    {  690, 1516,   90512,  7, }, // Australia/Hobart
    {  690, 8753,   90519,  8, }, // Australia/Lindeman
    {  690, 1547,   90527,  9, }, // Australia/Lord_Howe
    {  690, 1727,   90537,  8, }, // Australia/Melbourne
    {  690, 1762,   90545,  4, }, // Australia/Perth
    {  690, 1463,   32494,  5, }, // Australia/Sydney
    {  690, 6441,   90549, 10, }, // Europe/Amsterdam
    {  690, 6216,   90559,  6, }, // Europe/Andorra
    {  690, 3620,  226235, 10, }, // Europe/Astrakhan
    {  690, 2300,  226245,  6, }, // Europe/Athens
    {  690, 6336,   32532,  8, }, // Europe/Belgrade
    {  690, 4757,   90580,  6, }, // Europe/Berlin
    {  690, 6488,   90586, 12, }, // Europe/Bratislava
    {  690, 2167,   90598,  9, }, // Europe/Brussels
    {  690, 3928,   32570,  9, }, // Europe/Bucharest
    {  690, 3806,   90617,  9, }, // Europe/Budapest
    {  690, 8772,   90626, 10, }, // Europe/Busingen
    {  690, 2583,   90636,  7, }, // Europe/Chisinau
    {  690, 6289,   90643,  9, }, // Europe/Copenhagen
    {  690, 2338,   90652,  5, }, // Europe/Dublin
    {  690, 6307,   90657, 10, }, // Europe/Gibraltar
    {  690, 8788,   90667,  6, }, // Europe/Guernsey
    {  690, 6612,   90673,  8, }, // Europe/Helsinki
    {  690, 8804,  226251, 13, }, // Europe/Isle_of_Man
    {  690, 1130,   90692,  9, }, // Europe/Istanbul
    {  690, 8823,   90701,  5, }, // Europe/Jersey
    {  690, 3979,   90706, 12, }, // Europe/Kaliningrad
    {  690, 8929,   90718,  5, }, // Europe/Kirov
    {  690, 2527,   90723,  3, }, // Europe/Kyiv
    {  690, 3213,   32690,  6, }, // Europe/Lisbon
    {  690, 6506,   90726, 11, }, // Europe/Ljubljana
    {  690, 2501,   90737,  4, }, // Europe/London
    {  690, 6365,   90741, 10, }, // Europe/Luxembourg
    {  690, 4892,  140046,  9, }, // Europe/Madrid
    {  690, 6397,   32731,  6, }, // Europe/Malta
    {  690, 6582,   90758,  9, }, // Europe/Mariehamn
    {  690, 3693,   90767,  6, }, // Europe/Minsk
    {  690, 6410,   90773,  6, }, // Europe/Monaco
    {  690, 3537,   90779,  6, }, // Europe/Moscow
    {  690, 6458,   90785,  5, }, // Europe/Oslo
    {  690, 4284,   32766,  5, }, // Europe/Paris
    {  690, 6424,   90790, 10, }, // Europe/Podgorica
    {  690, 6275,   90800,  5, }, // Europe/Prague
    {  690, 8837,   90805,  4, }, // Europe/Riga
    {  690, 6324,   90432,  3, }, // Europe/Rome
    {  690, 4331,   90809,  5, }, // Europe/Samara
    {  690, 6470,   90814, 10, }, // Europe/San_Marino
    {  690, 6245,   90824,  8, }, // Europe/Sarajevo
    {  690, 4451,   90832,  7, }, // Europe/Saratov
    {  690, 8849,  226264, 12, }, // Europe/Simferopol
    {  690, 6383,  226276,  8, }, // Europe/Skopje
    {  690, 6599,  226284,  7, }, // Europe/Sofia
    {  690, 6523,   90866,  8, }, // Europe/Stockholm
    {  690, 8867,   90874,  5, }, // Europe/Tallinn
    {  690, 6202,   90879,  6, }, // Europe/Tirane
    {  690, 8882,   90885, 12, }, // Europe/Ulyanovsk
    {  690, 6352,   90897,  6, }, // Europe/Vaduz
    {  690, 6567,   90903,  6, }, // Europe/Vatican
    {  690, 6231,   90909,  5, }, // Europe/Vienna
    {  690, 8899,   90914,  8, }, // Europe/Vilnius
    {  690, 4727,   90922, 11, }, // Europe/Volgograd
    {  690, 3190,   90933,  5, }, // Europe/Warsaw
    {  690, 6261,   90938,  8, }, // Europe/Zagreb
    {  690, 6540,  226291,  9, }, // Europe/Zurich
    {  690, 5078,   90955, 11, }, // Indian/Antananarivo
    {  690, 6947,  115401,  5, }, // Indian/Chagos
    {  690, 6046,   90971,  7, }, // Indian/Christmas
    {  690, 6063,   90978,  5, }, // Indian/Cocos
    {  690, 5029,   90983,  6, }, // Indian/Comoro
    {  690, 6661,   90989,  7, }, // Indian/Kerguelen
    {  690, 7260,   32983,  4, }, // Indian/Mahe
    {  690, 7050,   90996,  6, }, // Indian/Maldives
    {  690, 4069,  226300,  6, }, // Indian/Mauritius
    {  690, 5098,   91008,  8, }, // Indian/Mayotte
    {  690, 7211,   91016,  8, }, // Indian/Reunion
    {  690, 4422,   91024,  5, }, // Pacific/Apia
    {  690,  932,  226306,  6, }, // Pacific/Auckland
    {  690, 3706,   91035,  8, }, // Pacific/Bougainville
    {  690, 3015,  226312,  6, }, // Pacific/Chatham
    {  690, 3157,   91047,  2, }, // Pacific/Chuuk
    {  690, 2237,  226318,  6, }, // Pacific/Easter
    {  690, 7360,  226324,  5, }, // Pacific/Efate
    {  690, 7327,  226329,  9, }, // Pacific/Fakaofo
    {  690, 3902,  226338,  5, }, // Pacific/Fiji
    {  690, 7343,  226343, 12, }, // Pacific/Funafuti
    {  690, 6678,   91085,  9, }, // Pacific/Galapagos
    {  690, 6696,  226355,  9, }, // Pacific/Gambier
    {  690, 3822,   91101, 10, }, // Pacific/Guadalcanal
    {  690, 6018,   31112,  4, }, // Pacific/Guam
    {  690, 3049,  226364,  5, }, // Pacific/Kanton
    {  690, 3998,   33132, 10, }, // Pacific/Kiritimati
    {  690, 7019,   91126,  6, }, // Pacific/Kosrae
    {  690, 2842,   91132, 10, }, // Pacific/Kwajalein
    {  690, 4877,   91142,  6, }, // Pacific/Majuro
    {  690, 4051,   91148,  8, }, // Pacific/Marquesas
    {  690, 8914,   91156,  5, }, // Pacific/Midway
    {  690, 7084,  226369,  6, }, // Pacific/Nauru
    {  690, 7132,   91165,  4, }, // Pacific/Niue
    {  690, 4168,  226375,  7, }, // Pacific/Norfolk
    {  690, 7098,  226382,  6, }, // Pacific/Noumea
    {  690, 3126,   91182,  9, }, // Pacific/Pago_Pago
    {  690, 7155,   91191,  4, }, // Pacific/Palau
    {  690, 7194,  137202,  8, }, // Pacific/Pitcairn
    {  690, 3096,  226388,  7, }, // Pacific/Pohnpei
    {  690, 4807,   91201, 14, }, // Pacific/Port_Moresby
    {  690, 6076,   91215,  9, }, // Pacific/Rarotonga
    {  690, 6031,   91224,  6, }, // Pacific/Saipan
    {  690, 7312,   91230,  6, }, // Pacific/Tahiti
    {  690, 6712,   91236,  6, }, // Pacific/Tarawa
    {  690, 4583,   91242,  9, }, // Pacific/Tongatapu
    {  690, 7392,   33268,  3, }, // Pacific/Wake
    {  690, 7405,   33271,  5, }, // Pacific/Wallis
    {  691, 6788,  148234,  7, }, // Africa/Abidjan Kuvi/Odia/India
    {  691, 6760,  226395,  5, }, // Africa/Accra
    {  691, 5059,  148246, 12, }, // Africa/Addis_Ababa
    {  691, 7420,  148258,  8, }, // Africa/Algiers
    {  691,   14,  148266,  5, }, // Africa/Asmara
    {  691,   44,  148271,  5, }, // Africa/Bamako
    {  691, 5257,  148277,  7, }, // Africa/Bangui
    {  691, 6746,  148284,  7, }, // Africa/Banjul
    {  691, 7435,  148291,  5, }, // Africa/Bissau
    {  691, 4957,  148296, 13, }, // Africa/Blantyre
    {  691, 5287,  226400, 12, }, // Africa/Brazzaville
    {  691, 4922,  226412, 10, }, // Africa/Bujumbura
    {  691, 2320,  148331,  4, }, // Africa/Cairo
    {  691, 4117,  148336, 12, }, // Africa/Casablanca
    {  691, 7449,  148348,  5, }, // Africa/Ceuta
    {  691, 6773,  226422,  7, }, // Africa/Conakry
    {  691, 6840,  148361,  4, }, // Africa/Dakar
    {  691, 5130,  148365, 15, }, // Africa/Dar_es_Salaam
    {  691, 5043,  226429,  5, }, // Africa/Djibouti
    {  691, 5243,  148386,  4, }, // Africa/Douala
    {  691, 5166,  148390,  9, }, // Africa/El_Aaiun
    {  691, 6853,  226434, 10, }, // Africa/Freetown
    {  691, 4906,  226444,  8, }, // Africa/Gaborone
    {  691, 5015,  148418,  5, }, // Africa/Harare
    {  691, 4479,  226452, 10, }, // Africa/Johannesburg
    {  691, 4499,  148434,  4, }, // Africa/Juba
    {  691, 5151,  148438,  8, }, // Africa/Kampala
    {  691, 4524,  226462,  7, }, // Africa/Khartoum
    {  691, 4973,  148453,  6, }, // Africa/Kigali
    {  691, 5306,  148459,  8, }, // Africa/Kinshasa
    {  691, 4744,  226469,  6, }, // Africa/Lagos
    {  691, 5336,  226475, 12, }, // Africa/Libreville
    {  691, 6869,  226487,  4, }, // Africa/Lome
    {  691, 5211,  148491,  7, }, // Africa/Luanda
    {  691, 4939,  148498, 10, }, // Africa/Lubumbashi
    {  691, 5001,  148508,  6, }, // Africa/Lusaka
    {  691, 5322,  148514,  5, }, // Africa/Malabo
    {  691, 4987,  148520,  6, }, // Africa/Maputo
    {  691, 5197,  148526,  6, }, // Africa/Maseru
    {  691, 5182,  226491,  9, }, // Africa/Mbabane
    {  691, 5113,  226500,  8, }, // Africa/Mogadishu
    {  691, 7034,  226508,  7, }, // Africa/Monrovia
    {  691, 3853,  226515,  6, }, // Africa/Nairobi
    {  691, 5271,  148561,  6, }, // Africa/Ndjamena
    {  691, 5354,  148567,  5, }, // Africa/Niamey
    {  691, 6803,  226521,  6, }, // Africa/Nouakchott
    {  691, 6727,  226527,  6, }, // Africa/Ouagadougou
    {  691, 5225,  226533,  7, }, // Africa/Porto-Novo
    {  691, 4435,  226540,  7, }, // Africa/Sao_Tome
    {  691, 2866,  226547,  7, }, // Africa/Tripoli
    {  691, 6554,  148614,  7, }, // Africa/Tunis
    {  691, 4152,  226554,  8, }, // Africa/Windhoek
    {  691,  132,  148631,  5, }, // America/Adak
    {  691, 3346,  148636,  8, }, // America/Anchorage
    {  691, 5607,  148644,  8, }, // America/Anguilla
    {  691, 5624,  148652,  8, }, // America/Antigua
    {  691, 4554,  148660,  8, }, // America/Araguaina
    {  691,  166,  226562, 13, }, // America/Argentina/Buenos_Aires
    {  691,   91,  226575, 10, }, // America/Argentina/Catamarca
    {  691,  270,  226585,  6, }, // America/Argentina/Cordoba
    {  691,  441,  226591,  5, }, // America/Argentina/Jujuy
    {  691, 7462,  226596,  9, }, // America/Argentina/La_Rioja
    {  691,  565,  226605,  7, }, // America/Argentina/Mendoza
    {  691, 7489,  226612, 12, }, // America/Argentina/Rio_Gallegos
    {  691, 7520,  226624,  6, }, // America/Argentina/Salta
    {  691, 7544,  226630, 11, }, // America/Argentina/San_Juan
    {  691, 5580,  226641,  9, }, // America/Argentina/San_Luis
    {  691, 7571,  226650,  6, }, // America/Argentina/Tucuman
    {  691, 7597,  226656,  7, }, // America/Argentina/Ushuaia
    {  691, 5640,  226663,  5, }, // America/Aruba
    {  691, 4252,  148781,  9, }, // America/Asuncion
    {  691,  237,  148790,  9, }, // America/Atikokan
    {  691, 3679,  148799,  5, }, // America/Bahia
    {  691, 7623,  148804, 18, }, // America/Bahia_Banderas
    {  691, 5654,  226668,  7, }, // America/Barbados
    {  691, 7646,  148832,  5, }, // America/Belem
    {  691, 5407,  148837,  6, }, // America/Belize
    {  691, 7660,  226675, 14, }, // America/Blanc-Sablon
    {  691, 7681,  226689, 10, }, // America/Boa_Vista
    {  691, 4361,  226699,  4, }, // America/Bogota
    {  691, 7699,  226703,  4, }, // America/Boise
    {  691, 7713,  148879, 13, }, // America/Cambridge_Bay
    {  691, 7735,  226707, 14, }, // America/Campo_Grande
    {  691, 3868,  148907,  7, }, // America/Cancun
    {  691, 4694,  148914,  8, }, // America/Caracas
    {  691, 4345,  148922,  8, }, // America/Cayenne
    {  691, 5496,  148930,  9, }, // America/Cayman
    {  691, 2260,  226721,  5, }, // America/Chicago
    {  691, 7756,  148945,  8, }, // America/Chihuahua
    {  691, 5422,  226726, 10, }, // America/Costa_Rica
    {  691, 7796,  226736,  9, }, // America/Creston
    {  691, 3791,  148989,  5, }, // America/Cuiaba
    {  691, 5723,  148994,  5, }, // America/Curacao
    {  691, 7812,  149000, 13, }, // America/Danmarkshavn
    {  691, 7833,  149013,  5, }, // America/Dawson
    {  691, 7848,  149018, 12, }, // America/Dawson_Creek
    {  691,  805,  226745,  7, }, // America/Denver
    {  691, 3465,  226752,  8, }, // America/Detroit
    {  691, 5739,  226760,  7, }, // America/Dominica
    {  691,  893,  226767,  9, }, // America/Edmonton
    {  691, 7869,  149063,  7, }, // America/Eirunepe
    {  691, 5441,  226776, 13, }, // America/El_Salvador
    {  691, 7886,  226789, 11, }, // America/Fort_Nelson
    {  691, 7906,  226800,  8, }, // America/Fortaleza
    {  691, 7924,  149106,  9, }, // America/Glace_Bay
    {  691, 6881,  149115,  7, }, // America/Goose_Bay
    {  691, 4612,  149122, 14, }, // America/Grand_Turk
    {  691, 5770,  149136,  8, }, // America/Grenada
    {  691, 5786,  226808,  9, }, // America/Guadeloupe
    {  691, 3760,  226817,  9, }, // America/Guatemala
    {  691, 4841,  149163,  9, }, // America/Guayaquil
    {  691, 6932,  149172,  5, }, // America/Guyana
    {  691, 1939,  149177, 12, }, // America/Halifax
    {  691, 2281,  226826,  5, }, // America/Havana
    {  691, 7942,  226831,  8, }, // America/Hermosillo
    {  691,  348,  226839, 13, }, // America/Indiana/Indianapolis
    {  691,  481,  149218, 17, }, // America/Indiana/Knox
    {  691, 7961,  226852, 16, }, // America/Indiana/Marengo
    {  691, 7985,  149252, 22, }, // America/Indiana/Petersburg
    {  691, 8012,  149274, 19, }, // America/Indiana/Tell_City
    {  691, 8038,  226868, 15, }, // America/Indiana/Vevay
    {  691, 8060,  149308, 21, }, // America/Indiana/Vincennes
    {  691, 8086,  149329, 18, }, // America/Indiana/Winamac
    {  691, 8110,  226883,  7, }, // America/Inuvik
    {  691,  660,  149354, 10, }, // America/Iqaluit
    {  691, 2799,  149364,  7, }, // America/Jamaica
    {  691, 5380,  149371,  6, }, // America/Juneau
    {  691,  521,  226890, 10, }, // America/Kentucky/Louisville
    {  691, 8125,  226900, 20, }, // America/Kentucky/Monticello
    {  691, 5704,  226920, 13, }, // America/Kralendijk
    {  691, 4376,  149420,  6, }, // America/La_Paz
    {  691, 7169,  149428,  4, }, // America/Lima
    {  691, 3239,  149432, 13, }, // America/Los_Angeles
    {  691, 5932,  149445, 23, }, // America/Lower_Princes
    {  691, 8153,  226933,  5, }, // America/Maceio
    {  691, 8168,  149473,  7, }, // America/Managua
    {  691, 1908,  149480,  8, }, // America/Manaus
    {  691, 5897,  226938,  7, }, // America/Marigot
    {  691, 5805,  149496, 12, }, // America/Martinique
    {  691, 8184,  226945,  8, }, // America/Matamoros
    {  691, 2917,  226953,  9, }, // America/Mazatlan
    {  691, 8202,  149527,  8, }, // America/Menominee
    {  691, 8220,  149535,  6, }, // America/Merida
    {  691, 8235,  149541, 10, }, // America/Metlakatla
    {  691, 2949,  149551, 13, }, // America/Mexico_City
    {  691, 4391,  226962,  9, }, // America/Miquelon
    {  691, 8254,  226971,  7, }, // America/Moncton
    {  691, 8270,  226978,  7, }, // America/Monterrey
    {  691, 4098,  226985, 10, }, // America/Montevideo
    {  691, 5824,  226995, 10, }, // America/Montserrat
    {  691, 5481,  149611,  5, }, // America/Nassau
    {  691, 2379,  227005, 10, }, // America/New_York
    {  691, 8288,  149627,  4, }, // America/Nome
    {  691, 1850,  227015,  5, }, // America/Noronha
    {  691, 8301,  227020, 19, }, // America/North_Dakota/Beulah
    {  691, 8329,  227039, 20, }, // America/North_Dakota/Center
    {  691, 8357,  227059, 24, }, // America/North_Dakota/New_Salem
    {  691,  393,  149705,  5, }, // America/Nuuk
    {  691, 8388,  227083,  7, }, // America/Ojinaga
    {  691, 2356,  149717,  6, }, // America/Panama
    {  691, 6173,  149723,  9, }, // America/Paramaribo
    {  691, 2973,  227090,  7, }, // America/Phoenix
    {  691, 3945,  227097, 17, }, // America/Port-au-Prince
    {  691, 5954,  227114, 16, }, // America/Port_of_Spain
    {  691, 8404,  149776, 11, }, // America/Porto_Velho
    {  691, 5843,  227130, 10, }, // America/Puerto_Rico
    {  691, 4030,  149799, 15, }, // America/Punta_Arenas
    {  691, 8424,  149814, 14, }, // America/Rankin_Inlet
    {  691, 8445,  227140,  6, }, // America/Recife
    {  691, 1995,  149834,  6, }, // America/Regina
    {  691, 8460,  149840,  8, }, // America/Resolute
    {  691,  695,  227146, 11, }, // America/Rio_Branco
    {  691, 8477,  227157,  9, }, // America/Santarem
    {  691, 2201,  149871,  8, }, // America/Santiago
    {  691, 6111,  227166, 12, }, // America/Santo_Domingo
    {  691, 1878,  149895,  9, }, // America/Sao_Paulo
    {  691, 6899,  227178, 14, }, // America/Scoresbysund
    {  691, 8494,  149921,  5, }, // America/Sitka
    {  691, 8508,  227192, 16, }, // America/St_Barthelemy
    {  691, 2061,  149944, 12, }, // America/St_Johns
    {  691, 5863,  227208, 13, }, // America/St_Kitts
    {  691, 5880,  149969, 13, }, // America/St_Lucia
    {  691,  855,  227221, 13, }, // America/St_Thomas
    {  691, 5913,  227234, 17, }, // America/St_Vincent
    {  691, 8530,  150013, 15, }, // America/Swift_Current
    {  691, 5461,  150028, 11, }, // America/Tegucigalpa
    {  691, 5756,  227251,  4, }, // America/Thule
    {  691,  313,  150043,  7, }, // America/Tijuana
    {  691,  608,  227255,  5, }, // America/Toronto
    {  691, 5688,  227260,  5, }, // America/Tortola
    {  691, 2093,  227265,  9, }, // America/Vancouver
    {  691, 2144,  150074, 11, }, // America/Whitehorse
    {  691,  734,  150085,  8, }, // America/Winnipeg
    {  691, 8552,  150093,  8, }, // America/Yakutat
    {  691, 6001,  227274,  4, }, // Antarctica/Casey
    {  691, 6094,  227278,  6, }, // Antarctica/Davis
    {  691, 6133,  227284, 19, }, // Antarctica/DumontDUrville
    {  691, 8568,  227303,  7, }, // Antarctica/Macquarie
    {  691, 7066,  227310,  5, }, // Antarctica/Mawson
    {  691, 7113,  227315, 10, }, // Antarctica/McMurdo
    {  691, 8589,  227325,  5, }, // Antarctica/Palmer
    {  691, 7226,  227330,  6, }, // Antarctica/Rothera
    {  691, 7295,  150167,  4, }, // Antarctica/Syowa
    {  691, 8607,  227336,  4, }, // Antarctica/Troll
    {  691, 7374,  227340,  7, }, // Antarctica/Vostok
    {  691, 1429,  227347, 10, }, // Arctic/Longyearbyen
    {  691, 5570,  150199,  6, }, // Asia/Aden
    {  691, 5395,  150205,  6, }, // Asia/Almaty
    {  691, 3968,  150211,  6, }, // Asia/Amman
    {  691, 5511,  150217,  8, }, // Asia/Anadyr
    {  691, 5523,  150225,  5, }, // Asia/Aqtau
    {  691, 5368,  227357,  6, }, // Asia/Aqtobe
    {  691,  964,  227363,  9, }, // Asia/Ashgabat
    {  691, 8624,  150248,  6, }, // Asia/Atyrau
    {  691, 3607,  150254,  8, }, // Asia/Baghdad
    {  691, 5534,  227372,  6, }, // Asia/Bahrain
    {  691, 3653,  148992,  4, }, // Asia/Baku
    {  691, 4466,  150270, 11, }, // Asia/Bangkok
    {  691, 3571,  150281,  8, }, // Asia/Barnaul
    {  691, 4086,  150289,  7, }, // Asia/Beirut
    {  691, 3778,  227378,  8, }, // Asia/Bishkek
    {  691, 5989,  150304,  7, }, // Asia/Brunei
    {  691, 4601,  150311,  4, }, // Asia/Chita
    {  691, 4511,  150315,  6, }, // Asia/Colombo
    {  691, 4540,  150321,  9, }, // Asia/Damascus
    {  691, 1093,  150849,  4, }, // Asia/Dhaka
    {  691, 6192,  150334,  6, }, // Asia/Dili
    {  691, 3596,  150340,  5, }, // Asia/Dubai
    {  691, 6159,  227386,  7, }, // Asia/Dushanbe
    {  691, 8636,  227393, 10, }, // Asia/Famagusta
    {  691, 8651,  150362,  4, }, // Asia/Gaza
    {  691, 4795,  150366,  8, }, // Asia/Hebron
    {  691, 1259,  227403, 15, }, // Asia/Ho_Chi_Minh
    {  691, 2704,  150390,  5, }, // Asia/Hong_Kong
    {  691, 4771,  227418,  6, }, // Asia/Hovd
    {  691, 4184,  150401,  8, }, // Asia/Irkutsk
    {  691, 7006,  227424,  7, }, // Asia/Jakarta
    {  691, 6992,  150418,  6, }, // Asia/Jayapura
    {  691, 1290,  150424,  8, }, // Asia/Jerusalem
    {  691, 3560,  227431,  5, }, // Asia/Kabul
    {  691, 4316,  150437,  8, }, // Asia/Kamchatka
    {  691, 4239,  150445,  5, }, // Asia/Karachi
    {  691, 1185,  227436,  9, }, // Asia/Kathmandu
    {  691, 8661,  227445,  9, }, // Asia/Khandyga
    {  691,  992,  150468,  7, }, // Asia/Kolkata
    {  691, 4197,  227454, 12, }, // Asia/Krasnoyarsk
    {  691, 4859,  150488, 12, }, // Asia/Kuala_Lumpur
    {  691, 5976,  150500,  5, }, // Asia/Kuching
    {  691, 5547,  150505,  4, }, // Asia/Kuwait
    {  691, 1211,  150509,  5, }, // Asia/Macau
    {  691, 4017,  150514,  8, }, // Asia/Magadan
    {  691, 1349,  150522,  9, }, // Asia/Makassar
    {  691, 7182,  150531,  6, }, // Asia/Manila
    {  691, 6920,  150537,  8, }, // Asia/Muscat
    {  691, 2554,  227466,  6, }, // Asia/Nicosia
    {  691, 8675,  227472, 11, }, // Asia/Novokuznetsk
    {  691, 4135,  227483, 10, }, // Asia/Novosibirsk
    {  691, 4229,  150577,  5, }, // Asia/Omsk
    {  691, 7145,  227493,  6, }, // Asia/Oral
    {  691, 6961,  227499, 10, }, // Asia/Phnom_Penh
    {  691, 8693,  227509, 11, }, // Asia/Pontianak
    {  691, 4214,  150611, 11, }, // Asia/Pyongyang
    {  691, 5559,  150622,  5, }, // Asia/Qatar
    {  691, 8708,  227520,  6, }, // Asia/Qostanay
    {  691, 4269,  227526,  9, }, // Asia/Qyzylorda
    {  691, 3584,  150644,  4, }, // Asia/Riyadh
    {  691, 4408,  227535,  7, }, // Asia/Sakhalin
    {  691, 7245,  150655,  7, }, // Asia/Samarkand
    {  691, 3279,  227542,  4, }, // Asia/Seoul
    {  691, 1053,  227546,  5, }, // Asia/Shanghai
    {  691, 3300,  150672, 11, }, // Asia/Singapore
    {  691, 4297,  227551, 14, }, // Asia/Srednekolymsk
    {  691, 3263,  150698,  6, }, // Asia/Taipei
    {  691, 4781,  227565,  8, }, // Asia/Tashkent
    {  691, 3915,  150712,  8, }, // Asia/Tbilisi
    {  691, 2772,  150720,  8, }, // Asia/Tehran
    {  691, 1317,  227573,  6, }, // Asia/Thimphu
    {  691, 2821,  227579,  4, }, // Asia/Tokyo
    {  691, 4572,  227583,  5, }, // Asia/Tomsk
    {  691, 1021,  150745, 12, }, // Asia/Ulaanbaatar
    {  691, 1159,  150757,  6, }, // Asia/Urumqi
    {  691, 8722,  227588, 11, }, // Asia/Ust-Nera
    {  691, 6977,  227599, 11, }, // Asia/Vientiane
    {  691, 4710,  227610, 13, }, // Asia/Vladivostok
    {  691, 4828,  150798,  8, }, // Asia/Yakutsk
    {  691, 1235,  150806,  9, }, // Asia/Yangon
    {  691, 3883,  150815, 14, }, // Asia/Yekaterinburg
    {  691, 3747,  227623,  8, }, // Asia/Yerevan
    {  691, 3663,  227631,  6, }, // Atlantic/Azores
    {  691, 5671,  227637,  6, }, // Atlantic/Bermuda
    {  691, 6628,  150851,  6, }, // Atlantic/Canary
    {  691, 3727,  227643, 11, }, // Atlantic/Cape_Verde
    {  691, 1395,  227654,  3, }, // Atlantic/Faroe
    {  691, 8736,  150873,  7, }, // Atlantic/Madeira
    {  691, 2748,  227657,  9, }, // Atlantic/Reykjavik
    {  691, 7272,  150890, 13, }, // Atlantic/South_Georgia
    {  691, 6821,  150903, 14, }, // Atlantic/St_Helena
    {  691, 6644,  227666,  8, }, // Atlantic/Stanley
    {  691, 1670,  227674,  7, }, // Australia/Adelaide
    {  691, 1635,  227681,  8, }, // Australia/Brisbane
    {  691, 1799,  227689, 10, }, // Australia/Broken_Hill
    {  691, 1583,  227699,  8, }, // Australia/Darwin
    {  691, 3637,  150965,  5, }, // Australia/Eucla
    {  691, 1516,  150970,  7, }, // Australia/Hobart
    {  691, 8753,  227707,  9, }, // Australia/Lindeman
    {  691, 1547,  227716, 10, }, // Australia/Lord_Howe
    {  691, 1727,  227726,  7, }, // Australia/Melbourne
    {  691, 1762,  227733,  4, }, // Australia/Perth
    {  691, 1463,  151017,  5, }, // Australia/Sydney
    {  691, 6441,  227737, 12, }, // Europe/Amsterdam
    {  691, 6216,  227749,  6, }, // Europe/Andorra
    {  691, 3620,  227755, 10, }, // Europe/Astrakhan
    {  691, 2300,  227765,  6, }, // Europe/Athens
    {  691, 6336,  227771,  9, }, // Europe/Belgrade
    {  691, 4757,  151065,  7, }, // Europe/Berlin
    {  691, 6488,  227780, 11, }, // Europe/Bratislava
    {  691, 2167,  227791, 10, }, // Europe/Brussels
    {  691, 3928,  227801,  9, }, // Europe/Bucharest
    {  691, 3806,  227810,  9, }, // Europe/Budapest
    {  691, 8772,  151111,  9, }, // Europe/Busingen
    {  691, 2583,  227819,  7, }, // Europe/Chisinau
    {  691, 6289,  227826, 10, }, // Europe/Copenhagen
    {  691, 2338,  151137,  6, }, // Europe/Dublin
    {  691, 6307,  151143, 11, }, // Europe/Gibraltar
    {  691, 8788,  151154,  7, }, // Europe/Guernsey
    {  691, 6612,  151161,  8, }, // Europe/Helsinki
    {  691, 8804,  227836, 15, }, // Europe/Isle_of_Man
    {  691, 1130,  151182,  9, }, // Europe/Istanbul
    {  691, 8823,  151191,  5, }, // Europe/Jersey
    {  691, 3979,  151196, 13, }, // Europe/Kaliningrad
    {  691, 8929,  227851,  4, }, // Europe/Kirov
    {  691, 2527,  151214,  5, }, // Europe/Kyiv
    {  691, 3213,  151219,  7, }, // Europe/Lisbon
    {  691, 6506,  151226, 10, }, // Europe/Ljubljana
    {  691, 2501,  151236,  6, }, // Europe/London
    {  691, 6365,  151242,  9, }, // Europe/Luxembourg
    {  691, 4892,  151251,  8, }, // Europe/Madrid
    {  691, 6397,  151259,  5, }, // Europe/Malta
    {  691, 6582,  227855,  9, }, // Europe/Mariehamn
    {  691, 3693,  151274,  6, }, // Europe/Minsk
    {  691, 6410,  227864,  4, }, // Europe/Monaco
    {  691, 3537,  150537,  4, }, // Europe/Moscow
    {  691, 6458,  227868,  4, }, // Europe/Oslo
    {  691, 4284,  151295,  6, }, // Europe/Paris
    {  691, 6424,  227872,  7, }, // Europe/Podgorica
    {  691, 6275,  151310,  6, }, // Europe/Prague
    {  691, 8837,  151316,  4, }, // Europe/Riga
    {  691, 6324,  227879,  3, }, // Europe/Rome
    {  691, 4331,  151324,  6, }, // Europe/Samara
    {  691, 6470,  151330, 11, }, // Europe/San_Marino
    {  691, 6245,  227882,  7, }, // Europe/Sarajevo
    {  691, 4451,  227889,  7, }, // Europe/Saratov
    {  691, 8849,  227896,  9, }, // Europe/Simferopol
    {  691, 6383,  227905,  6, }, // Europe/Skopje
    {  691, 6599,  227911,  5, }, // Europe/Sofia
    {  691, 6523,  227916, 10, }, // Europe/Stockholm
    {  691, 8867,  151389,  7, }, // Europe/Tallinn
    {  691, 6202,  227926,  7, }, // Europe/Tirane
    {  691, 8882,  227933, 10, }, // Europe/Ulyanovsk
    {  691, 6352,  227943,  5, }, // Europe/Vaduz
    {  691, 6567,  227948,  8, }, // Europe/Vatican
    {  691, 6231,  227956,  5, }, // Europe/Vienna
    {  691, 8899,  227961,  8, }, // Europe/Vilnius
    {  691, 4727,  227969,  9, }, // Europe/Volgograd
    {  691, 3190,  151450,  6, }, // Europe/Warsaw
    {  691, 6261,  151456,  8, }, // Europe/Zagreb
    {  691, 6540,  151464,  6, }, // Europe/Zurich
    {  691, 5078,  151470, 12, }, // Indian/Antananarivo
    {  691, 6947,  151483,  7, }, // Indian/Chagos
    {  691, 6046,  227978, 12, }, // Indian/Christmas
    {  691, 6063,  227990,  5, }, // Indian/Cocos
    {  691, 5029,  227995,  3, }, // Indian/Comoro
    {  691, 6661,  227998,  8, }, // Indian/Kerguelen
    {  691, 7260,  151522,  4, }, // Indian/Mahe
    {  691, 7050,  151526,  8, }, // Indian/Maldives
    {  691, 4069,  228006,  6, }, // Indian/Mauritius
    {  691, 5098,  228012,  5, }, // Indian/Mayotte
    {  691, 7211,  151547, 10, }, // Indian/Reunion
    {  691, 4422,  151557,  4, }, // Pacific/Apia
    {  691,  932,  151561,  7, }, // Pacific/Auckland
    {  691, 3706,  228017, 12, }, // Pacific/Bougainville
    {  691, 3015,  228029,  7, }, // Pacific/Chatham
    {  691, 3157,  151588,  5, }, // Pacific/Chuuk
    {  691, 2237,  151593,  7, }, // Pacific/Easter
    {  691, 7360,  151600,  6, }, // Pacific/Efate
    {  691, 7327,  228036,  5, }, // Pacific/Fakaofo
    {  691, 3902,  228041,  4, }, // Pacific/Fiji
    {  691, 7343,  151617,  8, }, // Pacific/Funafuti
    {  691, 6678,  228045,  8, }, // Pacific/Galapagos
    {  691, 6696,  151634, 10, }, // Pacific/Gambier
    {  691, 3822,  151644, 11, }, // Pacific/Guadalcanal
    {  691, 6018,  149477,  4, }, // Pacific/Guam
    {  691, 3049,  228053,  6, }, // Pacific/Kanton
    {  691, 3998,  151673, 10, }, // Pacific/Kiritimati
    {  691, 7019,  151683,  6, }, // Pacific/Kosrae
    {  691, 2842,  151689, 11, }, // Pacific/Kwajalein
    {  691, 4877,  151700,  5, }, // Pacific/Majuro
    {  691, 4051,  151706, 13, }, // Pacific/Marquesas
    {  691, 8914,  151719,  7, }, // Pacific/Midway
    {  691, 7084,  151726,  5, }, // Pacific/Nauru
    {  691, 7132,  151731,  4, }, // Pacific/Niue
    {  691, 4168,  228059,  6, }, // Pacific/Norfolk
    {  691, 7098,  228065,  5, }, // Pacific/Noumea
    {  691, 3126,  151747,  9, }, // Pacific/Pago_Pago
    {  691, 7155,  151756,  5, }, // Pacific/Palau
    {  691, 7194,  151761, 10, }, // Pacific/Pitcairn
    {  691, 3096,  228070,  6, }, // Pacific/Pohnpei
    {  691, 4807,  228076, 13, }, // Pacific/Port_Moresby
    {  691, 6076,  228089,  8, }, // Pacific/Rarotonga
    {  691, 6031,  151803,  7, }, // Pacific/Saipan
    {  691, 7312,  151810,  6, }, // Pacific/Tahiti
    {  691, 6712,  151816,  5, }, // Pacific/Tarawa
    {  691, 4583,  228097,  9, }, // Pacific/Tongatapu
    {  691, 7392,  151831,  4, }, // Pacific/Wake
    {  691, 7405,  151835,  7, }, // Pacific/Wallis
    {  692, 6788,  186476,  9, }, // Africa/Abidjan Kuvi/Telugu/India
    {  692, 6760,  186485,  5, }, // Africa/Accra
    {  692, 5059,  186490, 12, }, // Africa/Addis_Ababa
    {  692, 7420,  186502, 10, }, // Africa/Algiers
    {  692,   14,  186512,  7, }, // Africa/Asmara
    {  692,   44,  186519,  6, }, // Africa/Bamako
    {  692, 5257,  186525,  7, }, // Africa/Bangui
    {  692, 6746,  186532,  6, }, // Africa/Banjul
    {  692, 7435,  186538,  8, }, // Africa/Bissau
    {  692, 4957,  186546, 10, }, // Africa/Blantyre
    {  692, 5287,  186556, 12, }, // Africa/Brazzaville
    {  692, 4922,  186568,  9, }, // Africa/Bujumbura
    {  692, 2320,  186577,  4, }, // Africa/Cairo
    {  692, 4117,  186581, 11, }, // Africa/Casablanca
    {  692, 7449,  186592,  6, }, // Africa/Ceuta
    {  692, 6773,  186598,  8, }, // Africa/Conakry
    {  692, 6840,  186606,  5, }, // Africa/Dakar
    {  692, 5130,  186611, 14, }, // Africa/Dar_es_Salaam
    {  692, 5043,  186625,  8, }, // Africa/Djibouti
    {  692, 5243,  186633,  4, }, // Africa/Douala
    {  692, 5166,  186637,  9, }, // Africa/El_Aaiun
    {  692, 6853,  186646,  8, }, // Africa/Freetown
    {  692, 4906,  186654,  7, }, // Africa/Gaborone
    {  692, 5015,  186661,  5, }, // Africa/Harare
    {  692, 4479,  186666, 14, }, // Africa/Johannesburg
    {  692, 4499,  186680,  4, }, // Africa/Juba
    {  692, 5151,  186684,  6, }, // Africa/Kampala
    {  692, 4524,  186690,  8, }, // Africa/Khartoum
    {  692, 4973,  186698,  5, }, // Africa/Kigali
    {  692, 5306,  186703,  8, }, // Africa/Kinshasa
    {  692, 4744,  186711,  6, }, // Africa/Lagos
    {  692, 5336,  186717, 11, }, // Africa/Libreville
    {  692, 6869,  186728,  4, }, // Africa/Lome
    {  692, 5211,  186732,  7, }, // Africa/Luanda
    {  692, 4939,  186739,  8, }, // Africa/Lubumbashi
    {  692, 5001,  186747,  6, }, // Africa/Lusaka
    {  692, 5322,  186753,  5, }, // Africa/Malabo
    {  692, 4987,  186758,  6, }, // Africa/Maputo
    {  692, 5197,  186764,  5, }, // Africa/Maseru
    {  692, 5182,  186769,  6, }, // Africa/Mbabane
    {  692, 5113,  186775,  8, }, // Africa/Mogadishu
    {  692, 7034,  186783, 10, }, // Africa/Monrovia
    {  692, 3853,  186793,  6, }, // Africa/Nairobi
    {  692, 5271,  186799,  8, }, // Africa/Ndjamena
    {  692, 5354,  186807,  6, }, // Africa/Niamey
    {  692, 6803,  186813, 10, }, // Africa/Nouakchott
    {  692, 6727,  186823,  9, }, // Africa/Ouagadougou
    {  692, 5225,  186832, 11, }, // Africa/Porto-Novo
    {  692, 4435,  186843,  9, }, // Africa/Sao_Tome
    {  692, 2866,  186852,  8, }, // Africa/Tripoli
    {  692, 6554,  186860,  8, }, // Africa/Tunis
    {  692, 4152,  186868,  9, }, // Africa/Windhoek
    {  692,  132,  186877,  5, }, // America/Adak
    {  692, 3346,  186882,  8, }, // America/Anchorage
    {  692, 5607,  228106, 10, }, // America/Anguilla
    {  692, 5624,  186900,  8, }, // America/Antigua
    {  692, 4554,  186908, 11, }, // America/Araguaina
    {  692,  166,  186919, 16, }, // America/Argentina/Buenos_Aires
    {  692,   91,  186935,  8, }, // America/Argentina/Catamarca
    {  692,  270,  186943,  8, }, // America/Argentina/Cordoba
    {  692,  441,  186951,  6, }, // America/Argentina/Jujuy
    {  692, 7462,  186957,  8, }, // America/Argentina/La_Rioja
    {  692,  565,  186965,  7, }, // America/Argentina/Mendoza
    {  692, 7489,  186972, 14, }, // America/Argentina/Rio_Gallegos
    {  692, 7520,  186986,  6, }, // America/Argentina/Salta
    {  692, 7544,  186992, 11, }, // America/Argentina/San_Juan
    {  692, 5580,  187003, 11, }, // America/Argentina/San_Luis
    {  692, 7571,  187014,  7, }, // America/Argentina/Tucuman
    {  692, 7597,  187021,  6, }, // America/Argentina/Ushuaia
    {  692, 5640,  187027,  5, }, // America/Aruba
    {  692, 4252,  187032, 11, }, // America/Asuncion
    {  692,  237,  187043,  8, }, // America/Atikokan
    {  692, 3679,  187051,  4, }, // America/Bahia
    {  692, 7623,  187055, 13, }, // America/Bahia_Banderas
    {  692, 5654,  187068,  9, }, // America/Barbados
    {  692, 7646,  187077,  6, }, // America/Belem
    {  692, 5407,  187083,  6, }, // America/Belize
    {  692, 7660,  187089, 15, }, // America/Blanc-Sablon
    {  692, 7681,  187104, 11, }, // America/Boa_Vista
    {  692, 4361,  187115,  5, }, // America/Bogota
    {  692, 7699,  187120,  6, }, // America/Boise
    {  692, 7713,  187126, 14, }, // America/Cambridge_Bay
    {  692, 7735,  187140, 13, }, // America/Campo_Grande
    {  692, 3868,  187153,  8, }, // America/Cancun
    {  692, 4694,  187161,  7, }, // America/Caracas
    {  692, 4345,  187168,  5, }, // America/Cayenne
    {  692, 5496,  187173,  6, }, // America/Cayman
    {  692, 2260,  187179,  6, }, // America/Chicago
    {  692, 7756,  187185,  6, }, // America/Chihuahua
    {  692, 5422,  187208, 11, }, // America/Costa_Rica
    {  692, 7796,  187219,  9, }, // America/Creston
    {  692, 3791,  187228,  5, }, // America/Cuiaba
    {  692, 5723,  187233,  7, }, // America/Curacao
    {  692, 7812,  187240, 15, }, // America/Danmarkshavn
    {  692, 7833,  187255,  5, }, // America/Dawson
    {  692, 7848,  187260, 12, }, // America/Dawson_Creek
    {  692,  805,  187272,  8, }, // America/Denver
    {  692, 3465,  187280, 10, }, // America/Detroit
    {  692, 5739,  187290,  8, }, // America/Dominica
    {  692,  893,  187298,  9, }, // America/Edmonton
    {  692, 7869,  187307,  7, }, // America/Eirunepe
    {  692, 5441,  187314, 13, }, // America/El_Salvador
    {  692, 7886,  187327, 14, }, // America/Fort_Nelson
    {  692, 7906,  187341,  9, }, // America/Fortaleza
    {  692, 7924,  187350,  9, }, // America/Glace_Bay
    {  692, 6881,  187359,  7, }, // America/Goose_Bay
    {  692, 4612,  187366, 13, }, // America/Grand_Turk
    {  692, 5770,  187379,  7, }, // America/Grenada
    {  692, 5786,  187386, 10, }, // America/Guadeloupe
    {  692, 3760,  187396, 10, }, // America/Guatemala
    {  692, 4841,  187406,  9, }, // America/Guayaquil
    {  692, 6932,  187415,  5, }, // America/Guyana
    {  692, 1939,  187420, 10, }, // America/Halifax
    {  692, 2281,  187430,  5, }, // America/Havana
    {  692, 7942,  187435, 12, }, // America/Hermosillo
    {  692,  348,  187447, 13, }, // America/Indiana/Indianapolis
    {  692,  481,  187460, 15, }, // America/Indiana/Knox
    {  692, 7961,  187475, 15, }, // America/Indiana/Marengo
    {  692, 7985,  187490, 22, }, // America/Indiana/Petersburg
    {  692, 8012,  187512, 18, }, // America/Indiana/Tell_City
    {  692, 8038,  187530, 14, }, // America/Indiana/Vevay
    {  692, 8060,  187544, 22, }, // America/Indiana/Vincennes
    {  692, 8086,  187566, 17, }, // America/Indiana/Winamac
    {  692, 8110,  187583,  7, }, // America/Inuvik
    {  692,  660,  187590,  9, }, // America/Iqaluit
    {  692, 2799,  187599,  5, }, // America/Jamaica
    {  692, 5380,  187604,  4, }, // America/Juneau
    {  692,  521,  187608,  8, }, // America/Kentucky/Louisville
    {  692, 8125,  187616, 24, }, // America/Kentucky/Monticello
    {  692, 5704,  187640, 12, }, // America/Kralendijk
    {  692, 4376,  187652,  7, }, // America/La_Paz
    {  692, 7169,  187659,  4, }, // America/Lima
    {  692, 3239,  187663, 12, }, // America/Los_Angeles
    {  692, 5932,  187675, 24, }, // America/Lower_Princes
    {  692, 8153,  187699,  6, }, // America/Maceio
    {  692, 8168,  187705,  7, }, // America/Managua
    {  692, 1908,  187712,  5, }, // America/Manaus
    {  692, 5897,  187717,  8, }, // America/Marigot
    {  692, 5805,  187725, 10, }, // America/Martinique
    {  692, 8184,  187735,  9, }, // America/Matamoros
    {  692, 2917,  187744, 10, }, // America/Mazatlan
    {  692, 8202,  187754,  8, }, // America/Menominee
    {  692, 8220,  187762,  6, }, // America/Merida
    {  692, 8235,  187768,  9, }, // America/Metlakatla
    {  692, 2949,  187777, 13, }, // America/Mexico_City
    {  692, 4391,  187790,  8, }, // America/Miquelon
    {  692, 8254,  187798, 11, }, // America/Moncton
    {  692, 8270,  187809,  9, }, // America/Monterrey
    {  692, 4098,  187818, 11, }, // America/Montevideo
    {  692, 5824,  187829, 11, }, // America/Montserrat
    {  692, 5481,  187840,  8, }, // America/Nassau
    {  692, 2379,  187848, 10, }, // America/New_York
    {  692, 8288,  187858,  4, }, // America/Nome
    {  692, 1850,  187862,  7, }, // America/Noronha
    {  692, 8301,  187869, 18, }, // America/North_Dakota/Beulah
    {  692, 8329,  187887, 19, }, // America/North_Dakota/Center
    {  692, 8357,  187906, 22, }, // America/North_Dakota/New_Salem
    {  692,  393,  187928,  4, }, // America/Nuuk
    {  692, 8388,  187932,  5, }, // America/Ojinaga
    {  692, 2356,  187937,  5, }, // America/Panama
    {  692, 6173,  187942,  9, }, // America/Paramaribo
    {  692, 2973,  187951,  8, }, // America/Phoenix
    {  692, 3945,  187959, 19, }, // America/Port-au-Prince
    {  692, 5954,  187978, 19, }, // America/Port_of_Spain
    {  692, 8404,  187997, 13, }, // America/Porto_Velho
    {  692, 5843,  188010, 13, }, // America/Puerto_Rico
    {  692, 4030,  188023, 12, }, // America/Punta_Arenas
    {  692, 8424,  188035, 17, }, // America/Rankin_Inlet
    {  692, 8445,  188052,  6, }, // America/Recife
    {  692, 1995,  188058,  6, }, // America/Regina
    {  692, 8460,  188064,  9, }, // America/Resolute
    {  692,  695,  188073, 12, }, // America/Rio_Branco
    {  692, 8477,  188085,  8, }, // America/Santarem
    {  692, 2201,  188093,  9, }, // America/Santiago
    {  692, 6111,  188102, 13, }, // America/Santo_Domingo
    {  692, 1878,  188115,  9, }, // America/Sao_Paulo
    {  692, 6899,  188124, 17, }, // America/Scoresbysund
    {  692, 8494,  188141,  6, }, // America/Sitka
    {  692, 8508,  188147, 17, }, // America/St_Barthelemy
    {  692, 2061,  188164, 14, }, // America/St_Johns
    {  692, 5863,  188178, 14, }, // America/St_Kitts
    {  692, 5880,  188192, 14, }, // America/St_Lucia
    {  692,  855,  188206, 13, }, // America/St_Thomas
    {  692, 5913,  188219, 17, }, // America/St_Vincent
    {  692, 8530,  188236, 15, }, // America/Swift_Current
    {  692, 5461,  188251, 11, }, // America/Tegucigalpa
    {  692, 5756,  188262,  4, }, // America/Thule
    {  692,  313,  188266,  8, }, // America/Tijuana
    {  692,  608,  188274,  6, }, // America/Toronto
    {  692, 5688,  188280,  8, }, // America/Tortola
    {  692, 2093,  188288,  9, }, // America/Vancouver
    {  692, 2144,  188297, 11, }, // America/Whitehorse
    {  692,  734,  188308, 10, }, // America/Winnipeg
    {  692, 8552,  188318,  7, }, // America/Yakutat
    {  692, 6001,  188325,  4, }, // Antarctica/Casey
    {  692, 6094,  188329,  6, }, // Antarctica/Davis
    {  692, 6133,  188335, 22, }, // Antarctica/DumontDUrville
    {  692, 8568,  188357,  5, }, // Antarctica/Macquarie
    {  692, 7066,  188362,  5, }, // Antarctica/Mawson
    {  692, 7113,  188367, 11, }, // Antarctica/McMurdo
    {  692, 8589,  188378,  7, }, // Antarctica/Palmer
    {  692, 7226,  188385,  6, }, // Antarctica/Rothera
    {  692, 7295,  188391,  6, }, // Antarctica/Syowa
    {  692, 8607,  188397,  6, }, // Antarctica/Troll
    {  692, 7374,  188403,  8, }, // Antarctica/Vostok
    {  692, 1429,  188411, 17, }, // Arctic/Longyearbyen
    {  692, 5570,  188428,  5, }, // Asia/Aden
    {  692, 5395,  188433,  7, }, // Asia/Almaty
    {  692, 3968,  188440,  6, }, // Asia/Amman
    {  692, 5511,  188446,  6, }, // Asia/Anadyr
    {  692, 5523,  188452,  7, }, // Asia/Aqtau
    {  692, 5368,  188459,  7, }, // Asia/Aqtobe
    {  692,  964,  188466, 10, }, // Asia/Ashgabat
    {  692, 8624,  188476,  5, }, // Asia/Atyrau
    {  692, 3607,  188481,  8, }, // Asia/Baghdad
    {  692, 5534,  188489,  9, }, // Asia/Bahrain
    {  692, 3653,  187231,  4, }, // Asia/Baku
    {  692, 4466,  188498,  9, }, // Asia/Bangkok
    {  692, 3571,  188507,  8, }, // Asia/Barnaul
    {  692, 4086,  188515,  5, }, // Asia/Beirut
    {  692, 3778,  188520,  8, }, // Asia/Bishkek
    {  692, 5989,  188528,  6, }, // Asia/Brunei
    {  692, 4601,  188534,  4, }, // Asia/Chita
    {  692, 4511,  188538,  6, }, // Asia/Colombo
    {  692, 4540,  188544,  8, }, // Asia/Damascus
    {  692, 1093,  188552,  4, }, // Asia/Dhaka
    {  692, 6192,  188556,  4, }, // Asia/Dili
    {  692, 3596,  188560,  6, }, // Asia/Dubai
    {  692, 6159,  188566,  7, }, // Asia/Dushanbe
    {  692, 8636,  188573,  9, }, // Asia/Famagusta
    {  692, 8651,  188582,  4, }, // Asia/Gaza
    {  692, 4795,  188586,  8, }, // Asia/Hebron
    {  692, 1259,  188594, 17, }, // Asia/Ho_Chi_Minh
    {  692, 2704,  188611,  8, }, // Asia/Hong_Kong
    {  692, 4771,  188619,  6, }, // Asia/Hovd
    {  692, 4184,  188625, 11, }, // Asia/Irkutsk
    {  692, 7006,  188636,  7, }, // Asia/Jakarta
    {  692, 6992,  188643,  5, }, // Asia/Jayapura
    {  692, 1290,  188648,  7, }, // Asia/Jerusalem
    {  692, 3560,  188655,  6, }, // Asia/Kabul
    {  692, 4316,  188661,  9, }, // Asia/Kamchatka
    {  692, 4239,  188670,  5, }, // Asia/Karachi
    {  692, 1185,  188675,  8, }, // Asia/Kathmandu
    {  692, 8661,  188683,  7, }, // Asia/Khandyga
    {  692,  992,  188690,  8, }, // Asia/Kolkata
    {  692, 4197,  188698, 14, }, // Asia/Krasnoyarsk
    {  692, 4859,  188712, 10, }, // Asia/Kuala_Lumpur
    {  692, 5976,  188722,  7, }, // Asia/Kuching
    {  692, 5547,  188729,  6, }, // Asia/Kuwait
    {  692, 1211,  188735,  5, }, // Asia/Macau
    {  692, 4017,  188740,  6, }, // Asia/Magadan
    {  692, 1349,  188746,  8, }, // Asia/Makassar
    {  692, 7182,  188754,  5, }, // Asia/Manila
    {  692, 6920,  188759,  6, }, // Asia/Muscat
    {  692, 2554,  188765,  8, }, // Asia/Nicosia
    {  692, 8675,  188773, 18, }, // Asia/Novokuznetsk
    {  692, 4135,  188791, 13, }, // Asia/Novosibirsk
    {  692, 4229,  188804,  7, }, // Asia/Omsk
    {  692, 7145,  188811,  4, }, // Asia/Oral
    {  692, 6961,  188815, 11, }, // Asia/Phnom_Penh
    {  692, 8693,  188826, 11, }, // Asia/Pontianak
    {  692, 4214,  188837, 10, }, // Asia/Pyongyang
    {  692, 5559,  188847,  5, }, // Asia/Qatar
    {  692, 8708,  188852,  8, }, // Asia/Qostanay
    {  692, 4269,  188860, 12, }, // Asia/Qyzylorda
    {  692, 3584,  188872,  6, }, // Asia/Riyadh
    {  692, 4408,  188878,  7, }, // Asia/Sakhalin
    {  692, 7245,  188885,  9, }, // Asia/Samarkand
    {  692, 3279,  188894,  6, }, // Asia/Seoul
    {  692, 1053,  188900,  5, }, // Asia/Shanghai
    {  692, 3300,  188905,  8, }, // Asia/Singapore
    {  692, 4297,  188913, 18, }, // Asia/Srednekolymsk
    {  692, 3263,  188931,  4, }, // Asia/Taipei
    {  692, 4781,  188935,  9, }, // Asia/Tashkent
    {  692, 3915,  188944,  8, }, // Asia/Tbilisi
    {  692, 2772,  188952,  8, }, // Asia/Tehran
    {  692, 1317,  188960,  5, }, // Asia/Thimphu
    {  692, 2821,  188965,  6, }, // Asia/Tokyo
    {  692, 4572,  188971,  8, }, // Asia/Tomsk
    {  692, 1021,  188979, 10, }, // Asia/Ulaanbaatar
    {  692, 1159,  188989,  8, }, // Asia/Urumqi
    {  692, 8722,  188997, 10, }, // Asia/Ust-Nera
    {  692, 6977,  189007,  9, }, // Asia/Vientiane
    {  692, 4710,  189016, 14, }, // Asia/Vladivostok
    {  692, 4828,  189030,  9, }, // Asia/Yakutsk
    {  692, 1235,  189039,  6, }, // Asia/Yangon
    {  692, 3883,  189045, 13, }, // Asia/Yekaterinburg
    {  692, 3747,  189058,  7, }, // Asia/Yerevan
    {  692, 3663,  189065,  7, }, // Atlantic/Azores
    {  692, 5671,  189072,  8, }, // Atlantic/Bermuda
    {  692, 6628,  189080,  5, }, // Atlantic/Canary
    {  692, 3727,  189085, 11, }, // Atlantic/Cape_Verde
    {  692, 1395,  189096,  4, }, // Atlantic/Faroe
    {  692, 8736,  189100,  5, }, // Atlantic/Madeira
    {  692, 2748,  189105, 10, }, // Atlantic/Reykjavik
    {  692, 7272,  189115, 15, }, // Atlantic/South_Georgia
    {  692, 6821,  189130, 14, }, // Atlantic/St_Helena
    {  692, 6644,  189144,  8, }, // Atlantic/Stanley
    {  692, 1670,  228116, 36, }, // Australia/Adelaide
    {  692, 1635,  189159, 13, }, // Australia/Brisbane
    {  692, 1799,  189172, 13, }, // Australia/Broken_Hill
    {  692, 1583,  189185,  8, }, // Australia/Darwin
    {  692, 3637,  189193,  6, }, // Australia/Eucla
    {  692, 1516,  189199,  7, }, // Australia/Hobart
    {  692, 8753,  189206,  9, }, // Australia/Lindeman
    {  692, 1547,  189215,  9, }, // Australia/Lord_Howe
    {  692, 1727,  189224, 10, }, // Australia/Melbourne
    {  692, 1762,  189234,  6, }, // Australia/Perth
    {  692, 1463,  189240,  6, }, // Australia/Sydney
    {  692, 6441,  189246, 14, }, // Europe/Amsterdam
    {  692, 6216,  189260,  6, }, // Europe/Andorra
    {  692, 3620,  189266, 10, }, // Europe/Astrakhan
    {  692, 2300,  189276,  7, }, // Europe/Athens
    {  692, 6336,  189283, 11, }, // Europe/Belgrade
    {  692, 4757,  189294,  8, }, // Europe/Berlin
    {  692, 6488,  189302, 12, }, // Europe/Bratislava
    {  692, 2167,  189314, 11, }, // Europe/Brussels
    {  692, 3928,  189325, 10, }, // Europe/Bucharest
    {  692, 3806,  189335, 10, }, // Europe/Budapest
    {  692, 8772,  189345,  7, }, // Europe/Busingen
    {  692, 2583,  189352,  8, }, // Europe/Chisinau
    {  692, 6289,  189360, 11, }, // Europe/Copenhagen
    {  692, 2338,  189371,  7, }, // Europe/Dublin
    {  692, 6307,  189378, 11, }, // Europe/Gibraltar
    {  692, 8788,  189389, 10, }, // Europe/Guernsey
    {  692, 6612,  189399,  9, }, // Europe/Helsinki
    {  692, 8804,  189408, 12, }, // Europe/Isle_of_Man
    {  692, 1130,  189420, 10, }, // Europe/Istanbul
    {  692, 8823,  189430,  6, }, // Europe/Jersey
    {  692, 3979,  189436, 13, }, // Europe/Kaliningrad
    {  692, 8929,  189449,  6, }, // Europe/Kirov
    {  692, 2527,  189455,  4, }, // Europe/Kyiv
    {  692, 3213,  189459,  7, }, // Europe/Lisbon
    {  692, 6506,  189466, 12, }, // Europe/Ljubljana
    {  692, 2501,  189478,  5, }, // Europe/London
    {  692, 6365,  189483, 11, }, // Europe/Luxembourg
    {  692, 4892,  189494,  8, }, // Europe/Madrid
    {  692, 6397,  189502,  6, }, // Europe/Malta
    {  692, 6582,  189508,  9, }, // Europe/Mariehamn
    {  692, 3693,  189517,  8, }, // Europe/Minsk
    {  692, 6410,  189525,  6, }, // Europe/Monaco
    {  692, 3537,  189531,  6, }, // Europe/Moscow
    {  692, 6458,  189537,  5, }, // Europe/Oslo
    {  692, 4284,  189542,  8, }, // Europe/Paris
    {  692, 6424,  189550, 10, }, // Europe/Podgorica
    {  692, 6275,  189560,  6, }, // Europe/Prague
    {  692, 8837,  189566,  4, }, // Europe/Riga
    {  692, 6324,  189570,  4, }, // Europe/Rome
    {  692, 4331,  189574,  4, }, // Europe/Samara
    {  692, 6470,  189578, 11, }, // Europe/San_Marino
    {  692, 6245,  189589,  7, }, // Europe/Sarajevo
    {  692, 4451,  189596,  6, }, // Europe/Saratov
    {  692, 8849,  189602, 13, }, // Europe/Simferopol
    {  692, 6383,  189615,  9, }, // Europe/Skopje
    {  692, 6599,  189624,  6, }, // Europe/Sofia
    {  692, 6523,  189630, 10, }, // Europe/Stockholm
    {  692, 8867,  189640,  8, }, // Europe/Tallinn
    {  692, 6202,  189648,  6, }, // Europe/Tirane
    {  692, 8882,  189654, 12, }, // Europe/Ulyanovsk
    {  692, 6352,  189666,  6, }, // Europe/Vaduz
    {  692, 6567,  189672,  7, }, // Europe/Vatican
    {  692, 6231,  189679,  7, }, // Europe/Vienna
    {  692, 8899,  189686,  9, }, // Europe/Vilnius
    {  692, 4727,  189695, 12, }, // Europe/Volgograd
    {  692, 3190,  189707,  6, }, // Europe/Warsaw
    {  692, 6261,  189713,  8, }, // Europe/Zagreb
    {  692, 6540,  189721,  8, }, // Europe/Zurich
    {  692, 5078,  189729, 12, }, // Indian/Antananarivo
    {  692, 6947,  189741,  6, }, // Indian/Chagos
    {  692, 6046,  189747,  9, }, // Indian/Christmas
    {  692, 6063,  189756,  6, }, // Indian/Cocos
    {  692, 5029,  189762,  6, }, // Indian/Comoro
    {  692, 6661,  189768, 12, }, // Indian/Kerguelen
    {  692, 7260,  189780,  4, }, // Indian/Mahe
    {  692, 7050,  189784, 10, }, // Indian/Maldives
    {  692, 4069,  189794,  7, }, // Indian/Mauritius
    {  692, 5098,  189801,  5, }, // Indian/Mayotte
    {  692, 7211,  189806,  9, }, // Indian/Reunion
    {  692, 4422,  189815,  5, }, // Pacific/Apia
    {  692,  932,  189820,  8, }, // Pacific/Auckland
    {  692, 3706,  189828, 15, }, // Pacific/Bougainville
    {  692, 3015,  189843,  5, }, // Pacific/Chatham
    {  692, 3157,  189848,  4, }, // Pacific/Chuuk
    {  692, 2237,  189852,  6, }, // Pacific/Easter
    {  692, 7360,  189858,  5, }, // Pacific/Efate
    {  692, 7327,  189863,  6, }, // Pacific/Fakaofo
    {  692, 3902,  189869,  4, }, // Pacific/Fiji
    {  692, 7343,  189873,  8, }, // Pacific/Funafuti
    {  692, 6678,  189881, 10, }, // Pacific/Galapagos
    {  692, 6696,  189891,  9, }, // Pacific/Gambier
    {  692, 3822,  189900, 13, }, // Pacific/Guadalcanal
    {  692, 6018,  189913,  6, }, // Pacific/Guam
    {  692, 3049,  228152,  6, }, // Pacific/Kanton
    {  692, 3998,  189936, 10, }, // Pacific/Kiritimati
    {  692, 7019,  189946,  6, }, // Pacific/Kosrae
    {  692, 2842,  189952, 10, }, // Pacific/Kwajalein
    {  692, 4877,  189962,  5, }, // Pacific/Majuro
    {  692, 4051,  189967, 12, }, // Pacific/Marquesas
    {  692, 8914,  189979,  6, }, // Pacific/Midway
    {  692, 7084,  189985,  4, }, // Pacific/Nauru
    {  692, 7132,  189989,  4, }, // Pacific/Niue
    {  692, 4168,  189993,  8, }, // Pacific/Norfolk
    {  692, 7098,  190001,  6, }, // Pacific/Noumea
    {  692, 3126,  190007,  9, }, // Pacific/Pago_Pago
    {  692, 7155,  190016,  6, }, // Pacific/Palau
    {  692, 7194,  190022, 11, }, // Pacific/Pitcairn
    {  692, 3096,  190033,  7, }, // Pacific/Pohnpei
    {  692, 4807,  190040, 15, }, // Pacific/Port_Moresby
    {  692, 6076,  190055,  8, }, // Pacific/Rarotonga
    {  692, 6031,  190063,  8, }, // Pacific/Saipan
    {  692, 7312,  190071,  5, }, // Pacific/Tahiti
    {  692, 6712,  190076,  5, }, // Pacific/Tarawa
    {  692, 4583,  190081,  9, }, // Pacific/Tongatapu
    {  692, 7392,  190090,  4, }, // Pacific/Wake
    {  692, 7405,  190094,  8, }, // Pacific/Wallis
    {  693,   14,       0,  6, }, // Africa/Asmara Kara-Kalpak/Cyrillic/Uzbekistan
    {  693, 4435,       6,  8, }, // Africa/Sao_Tome
    {  693, 4252,      14,  8, }, // America/Asuncion
    {  693,  237,      22,  8, }, // America/Atikokan
    {  693, 7623,      30, 17, }, // America/Bahia_Banderas
    {  693, 3868,      47,  6, }, // America/Cancun
    {  693, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  693, 5723,      66,  7, }, // America/Curacao
    {  693,  481,      73, 13, }, // America/Indiana/Knox
    {  693, 7961,      86, 16, }, // America/Indiana/Marengo
    {  693, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  693, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  693, 8038,     139, 14, }, // America/Indiana/Vevay
    {  693, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  693, 8086,     171, 16, }, // America/Indiana/Winamac
    {  693, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  693, 5932,     207, 22, }, // America/Lower_Princes
    {  693, 8220,     229,  6, }, // America/Merida
    {  693, 1850,     235, 19, }, // America/Noronha
    {  693, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  693, 8329,     274, 20, }, // America/North_Dakota/Center
    {  693, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  693,  393,     317,  4, }, // America/Nuuk
    {  693, 6899,     321, 16, }, // America/Scoresbysund
    {  693, 8508,     337, 14, }, // America/St_Barthelemy
    {  693, 2061,     351, 10, }, // America/St_Johns
    {  693, 5863,     361,  9, }, // America/St_Kitts
    {  693, 5880,     370,  9, }, // America/St_Lucia
    {  693,  855,     379, 10, }, // America/St_Thomas
    {  693, 5913,     389, 11, }, // America/St_Vincent
    {  693, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  693, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  693, 1185,     448,  9, }, // Asia/Kathmandu
    {  693,  992,     457,  7, }, // Asia/Kolkata
    {  693, 1211,     464,  5, }, // Asia/Macau
    {  693, 1235,     477,  6, }, // Asia/Yangon
    {  693, 1395,     483,  5, }, // Atlantic/Faroe
    {  693, 6821,     488, 10, }, // Atlantic/St_Helena
    {  693, 2527,     514,  4, }, // Europe/Kyiv
    {  693, 7211,     547,  7, }, // Indian/Reunion
    {  693, 3157,     554,  5, }, // Pacific/Chuuk
    {  693, 3096,     586,  7, }, // Pacific/Pohnpei
    {  695,   14,       0,  6, }, // Africa/Asmara Swampy Cree/Canadian Aboriginal/Canada
    {  695, 4435,       6,  8, }, // Africa/Sao_Tome
    {  695, 4252,      14,  8, }, // America/Asuncion
    {  695,  237,  228158,  6, }, // America/Atikokan
    {  695, 7623,      30, 17, }, // America/Bahia_Banderas
    {  695, 3868,      47,  6, }, // America/Cancun
    {  695, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  695, 5723,      66,  7, }, // America/Curacao
    {  695,  893,  228164,  6, }, // America/Edmonton
    {  695,  481,      73, 13, }, // America/Indiana/Knox
    {  695, 7961,      86, 16, }, // America/Indiana/Marengo
    {  695, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  695, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  695, 8038,     139, 14, }, // America/Indiana/Vevay
    {  695, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  695, 8086,     171, 16, }, // America/Indiana/Winamac
    {  695, 8110,  228170,  4, }, // America/Inuvik
    {  695,  660,  228174,  6, }, // America/Iqaluit
    {  695, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  695, 5932,     207, 22, }, // America/Lower_Princes
    {  695, 8220,     229,  6, }, // America/Merida
    {  695, 1850,     235, 19, }, // America/Noronha
    {  695, 8301,     254, 20, }, // America/North_Dakota/Beulah
    {  695, 8329,     274, 20, }, // America/North_Dakota/Center
    {  695, 8357,     294, 23, }, // America/North_Dakota/New_Salem
    {  695,  393,     317,  4, }, // America/Nuuk
    {  695, 6899,     321, 16, }, // America/Scoresbysund
    {  695, 8508,     337, 14, }, // America/St_Barthelemy
    {  695, 2061,     351, 10, }, // America/St_Johns
    {  695, 5863,     361,  9, }, // America/St_Kitts
    {  695, 5880,     370,  9, }, // America/St_Lucia
    {  695,  855,     379, 10, }, // America/St_Thomas
    {  695, 5913,     389, 11, }, // America/St_Vincent
    {  695, 8530,  228180,  7, }, // America/Swift_Current
    {  695,  734,  228187,  4, }, // America/Winnipeg
    {  695, 6133,     400, 16, }, // Antarctica/DumontDUrville
    {  695, 1259,     432, 11, }, // Asia/Ho_Chi_Minh
    {  695, 1185,     448,  9, }, // Asia/Kathmandu
    {  695,  992,     457,  7, }, // Asia/Kolkata
    {  695, 1211,     464,  5, }, // Asia/Macau
    {  695, 1235,     477,  6, }, // Asia/Yangon
    {  695, 1395,     483,  5, }, // Atlantic/Faroe
    {  695, 6821,     488, 10, }, // Atlantic/St_Helena
    {  695, 2527,     514,  4, }, // Europe/Kyiv
    {  695, 7211,     547,  7, }, // Indian/Reunion
    {  695, 3157,     554,  5, }, // Pacific/Chuuk
    {  695, 3096,     586,  7, }, // Pacific/Pohnpei
    {  696,    0,       0,  0, } // Terminal row
}; // Exemplar city table

// Sorted by locale index, then iana name
static inline constexpr LocaleZoneNames localeZoneNameTable[] = {
    // locInd, ianaInd, (lngGen, lngStd, lngDst, srtGen, srtStd, srtDst){ind, sz}
    {    0, 2455,       0,       0,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC C/AnyScript/AnyTerritory
    {    0, 2338,       0,       0,      26,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {    0, 2501,       0,       0,      45,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {    0, 2723,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {    1, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Abkhazian/Cyrillic/Georgia
    {    1, 2338,       0,       0,      64,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {    1, 2501,       0,       0,      88,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {    2, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Afar/Latin/Ethiopia
    {    5, 2455,       0,     109,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Afrikaans/Latin/South Africa
    {    5, 2338,       0,       0,     138,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/Dublin
    {    5, 2501,       0,       0,     156,    0,    0,    0,  0,  0, 15,  0,  0,  0, }, // Europe/London
    {    7, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Aghem/Latin/Cameroon
    {    8, 2455,       0,     171,       0,    0,    0,    0,  0, 23,  0,  0,  3,  0, }, // Etc/UTC Akan/Latin/Ghana
    {    8, 2338,       0,       0,     194,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {    8, 2501,       0,       0,     217,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/London
    {    9, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Akoose/Latin/Cameroon
    {   10, 2455,       0,     234,       0,    0,    0,    0,  0, 27,  0,  0,  3,  0, }, // Etc/UTC Albanian/Latin/Albania
    {   10, 2338,       0,       0,     261,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/Dublin
    {   10, 2501,       0,       0,     286,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/London
    {   13, 2455,       0,     306,       0,    0,    0,    0,  0, 16,  0,  0,  3,  0, }, // Etc/UTC Amharic/Ethiopic/Ethiopia
    {   13, 2338,       0,       0,     322,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {   13, 2501,       0,       0,     342,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {   14, 2455,       0,     361,       0,    0,    0,    0,  0, 22,  0,  0,  3,  0, }, // Etc/UTC Arabic/Arabic/Egypt
    {   14, 2338,       0,       0,     383,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {   14, 2501,       0,       0,     403,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {   42, 2455,       0,     424,       0,    0,    0,    0,  0, 27,  0,  0,  3,  0, }, // Etc/UTC Aragonese/Latin/Spain
    {   43, 2455,       0,     451,       0,    0,    0,    0,  0, 34,  0,  0,  3,  0, }, // Etc/UTC Armenian/Armenian/Armenia
    {   43, 2338,       0,       0,     485,    0,    0,    0,  0,  0, 27,  0,  0,  0, }, // Europe/Dublin
    {   43, 2501,       0,       0,     512,    0,    0,    0,  0,  0, 27,  0,  0,  0, }, // Europe/London
    {   44, 2455,       0,     539,       0,    0,    0,    0,  0, 22,  0,  0,  3,  0, }, // Etc/UTC Assamese/Bangla/India
    {   44, 2338,       0,       0,     561,    0,    0,    0,  0,  0, 14,  0,  0,  0, }, // Europe/Dublin
    {   44, 2501,       0,       0,     575,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/London
    {   45, 2455,       0,     600,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Asturian/Latin/Spain
    {   45, 2338,       0,       0,     625,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {   45, 2501,       0,       0,     648,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {   45, 2723,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {   46, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Asu/Latin/Tanzania
    {   47, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Atsam/Latin/Nigeria
    {   48, 2455,       0,     671,       0,    0,    0,    0,  0, 36,  0,  0,  3,  0, }, // Etc/UTC Azerbaijani/Latin/Azerbaijan
    {   48, 2338,       0,       0,     707,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {   48, 2501,       0,       0,     726,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {   49, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Azerbaijani/Arabic/Iran
    {   52, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Azerbaijani/Cyrillic/Azerbaijan
    {   53, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Bafia/Latin/Cameroon
    {   54, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Bambara/Latin/Mali
    {   56, 2455,       0,     745,       0,    0,    0,    0,  0, 27,  0,  0,  3,  0, }, // Etc/UTC Bangla/Bangla/Bangladesh
    {   56, 2338,       0,       0,     772,    0,    0,    0,  0,  0, 15,  0,  0,  0, }, // Europe/Dublin
    {   56, 2501,       0,       0,     787,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/London
    {   58, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Basaa/Latin/Cameroon
    {   59, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Bashkir/Cyrillic/Russia
    {   60, 2455,       0,     812,       0,    0,    0,    0,  0, 27,  0,  0,  3,  0, }, // Etc/UTC Basque/Latin/Spain
    {   60, 2338,       0,       0,     839,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/Dublin
    {   60, 2501,       0,       0,     864,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {   60, 2723,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {   61, 2455,       0,     885,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Belarusian/Cyrillic/Belarus
    {   61, 2338,       0,       0,     914,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {   61, 2501,       0,       0,     938,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {   62, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Bemba/Latin/Zambia
    {   63, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Bena/Latin/Tanzania
    {   64, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Bhojpuri/Devanagari/India
    {   65, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Blin/Ethiopic/Eritrea
    {   66, 2455,       0,     957,       0,    0,    0,    0,  0, 22,  0,  0,  3,  0, }, // Etc/UTC Bodo/Devanagari/India
    {   66, 2338,       0,       0,     979,    0,    0,    0,  0,  0, 16,  0,  0,  0, }, // Europe/Dublin
    {   66, 2501,       0,       0,     995,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/London
    {   67, 2455,       0,    1013,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Bosnian/Latin/Bosnia and Herzegovina
    {   67, 2338,       0,       0,    1042,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {   67, 2501,       0,       0,    1066,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {   68, 2455,       0,    1090,       0,    0,    0,    0,  0, 32,  0,  0,  3,  0, }, // Etc/UTC Bosnian/Cyrillic/Bosnia and Herzegovina
    {   68, 2338,       0,       0,    1122,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {   68, 2501,       0,       0,    1146,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {   69, 2455,       0,    1169,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Breton/Latin/France
    {   69, 2338,       0,       0,    1195,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {   69, 2501,       0,       0,    1215,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/London
    {   70, 2455,       0,    1235,       0,    0,    0,    0,  0, 30,  0,  0,  3,  0, }, // Etc/UTC Bulgarian/Cyrillic/Bulgaria
    {   70, 2338,       0,       0,    1265,    0,    0,    0,  0,  0, 26,  0,  0,  0, }, // Europe/Dublin
    {   70, 2501,       0,       0,    1291,    0,    0,    0,  0,  0, 28,  0,  0,  0, }, // Europe/London
    {   71, 2455,       0,    1319,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Burmese/Myanmar/Myanmar
    {   71, 2338,       0,       0,    1348,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {   71, 2501,       0,       0,    1372,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/London
    {   72, 2455,       0,    1394,       0,    0,    0,    0,  0,  6,  0,  0,  3,  0, }, // Etc/UTC Cantonese/Traditional Han/Hong Kong
    {   72, 2338,       0,       0,    1400,    0,    0,    0,  0,  0,  7,  0,  0,  0, }, // Europe/Dublin
    {   72, 2501,       0,       0,    1407,    0,    0,    0,  0,  0,  6,  0,  0,  0, }, // Europe/London
    {   73, 2455,       0,    1413,       0,    0,    0,    0,  0,  6,  0,  0,  3,  0, }, // Etc/UTC Cantonese/Simplified Han/China
    {   73, 2338,       0,       0,    1419,    0,    0,    0,  0,  0,  7,  0,  0,  0, }, // Europe/Dublin
    {   73, 2501,       0,       0,    1426,    0,    0,    0,  0,  0,  6,  0,  0,  0, }, // Europe/London
    {   76, 2455,       0,    1432,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Catalan/Latin/Spain
    {   76, 2338,       0,       0,    1457,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {   76, 2501,       0,       0,    1481,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/London
    {   80, 2455,       0,       0,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Cebuano/Latin/Philippines
    {   80, 2338,       0,       0,      26,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {   80, 2501,       0,       0,      45,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {   81, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Central Atlas Tamazight/Latin/Morocco
    {   82, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Central Kurdish/Arabic/Iraq
    {   84, 2455,       0,    1503,       0,    0,    0,    0,  0, 46,  0,  0,  3,  0, }, // Etc/UTC Chakma/Chakma/Bangladesh
    {   84, 2338,       0,       0,    1549,    0,    0,    0,  0,  0, 36,  0,  0,  0, }, // Europe/Dublin
    {   84, 2501,       0,       0,    1585,    0,    0,    0,  0,  0, 42,  0,  0,  0, }, // Europe/London
    {   86, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Chechen/Cyrillic/Russia
    {   86, 2338,       0,       0,    1627,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {   86, 2501,       0,       0,    1648,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/London
    {   87, 2455,       0,    1670,       0,    0,    0,    0,  0, 14,  0,  0,  3,  0, }, // Etc/UTC Cherokee/Cherokee/United States
    {   87, 2338,       0,       0,    1684,    0,    0,    0,  0,  0, 15,  0,  0,  0, }, // Europe/Dublin
    {   87, 2501,       0,       0,    1699,    0,    0,    0,  0,  0, 12,  0,  0,  0, }, // Europe/London
    {   87, 2723,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {   88, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Chickasaw/Latin/United States
    {   89, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Chiga/Latin/Uganda
    {   90, 2455,       0,    1413,       0,    0,    0,    0,  0,  5,  0,  0,  3,  0, }, // Etc/UTC Chinese/Simplified Han/China
    {   90, 2338,       0,       0,    1419,    0,    0,    0,  0,  0,  7,  0,  0,  0, }, // Europe/Dublin
    {   90, 2501,       0,       0,    1426,    0,    0,    0,  0,  0,  6,  0,  0,  0, }, // Europe/London
    {   99, 2455,       0,    1711,       0,    0,    0,    0,  0,  6,  0,  0,  3,  0, }, // Etc/UTC Chinese/Traditional Han/Taiwan
    {   99, 2338,       0,       0,    1400,    0,    0,    0,  0,  0,  7,  0,  0,  0, }, // Europe/Dublin
    {   99, 2501,       0,       0,    1407,    0,    0,    0,  0,  0,  6,  0,  0,  0, }, // Europe/London
    {  100, 2455,       0,    1717,       0,    0,    0,    0,  0, 37,  0,  0,  3,  0, }, // Etc/UTC Church/Cyrillic/Russia
    {  101, 2455,       0,    1754,       0,    0,    9,    0,  0, 35,  0,  0, 10,  0, }, // Etc/UTC Chuvash/Cyrillic/Russia
    {  101, 2338,       0,       0,    1789,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/Dublin
    {  101, 2501,       0,       0,    1814,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/London
    {  101, 2723,       0,       0,       0,   19,   19,   42,  0,  0,  0, 23, 23, 20, }, // Pacific/Honolulu
    {  102, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Colognian/Latin/Germany
    {  102, 2338,       0,       0,    1836,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  102, 2501,       0,       0,    1858,    0,    0,    0,  0,  0, 30,  0,  0,  0, }, // Europe/London
    {  103, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Coptic/Coptic/Egypt
    {  104, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Cornish/Latin/United Kingdom
    {  105, 2455,       0,    1888,       0,    0,    0,    0,  0, 27,  0,  0,  3,  0, }, // Etc/UTC Corsican/Latin/France
    {  106, 2455,       0,    1915,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Croatian/Latin/Croatia
    {  106, 2338,       0,       0,    1944,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  106, 2501,       0,       0,    1968,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  108, 2455,       0,    1992,       0,    0,    0,    0,  0, 24,  0,  0,  3,  0, }, // Etc/UTC Czech/Latin/Czechia
    {  108, 2338,       0,       0,    2016,    0,    0,    0,  0,  0, 15,  0,  0,  0, }, // Europe/Dublin
    {  108, 2501,       0,       0,    2031,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/London
    {  108, 2723,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  109, 2455,       0,    2048,       0,    0,    0,    0,  0, 24,  0,  0,  3,  0, }, // Etc/UTC Danish/Latin/Denmark
    {  109, 2338,       0,       0,    2072,    0,    0,    0,  0,  0, 14,  0,  0,  0, }, // Europe/Dublin
    {  109, 2501,       0,       0,    2086,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/London
    {  111, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Divehi/Thaana/Maldives
    {  112, 2455,       0,    2103,       0,    0,    0,    0,  0, 17,  0,  0,  3,  0, }, // Etc/UTC Dogri/Devanagari/India
    {  112, 2338,       0,       0,    2120,    0,    0,    0,  0,  0, 15,  0,  0,  0, }, // Europe/Dublin
    {  112, 2501,       0,       0,    2135,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  113, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Duala/Latin/Cameroon
    {  114, 2455,       0,    2159,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Dutch/Latin/Netherlands
    {  114, 2338,       0,       0,    2184,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  114, 2501,       0,       0,    2203,    0,    0,    0,  0,  0, 16,  0,  0,  0, }, // Europe/London
    {  114, 2723,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  121, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Dzongkha/Tibetan/Bhutan
    {  121, 2338,       0,       0,    2219,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  121, 2501,       0,       0,    2242,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  122, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Embu/Latin/Kenya
    {  123, 2455,       0,       0,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC English/Latin/United States
    {  123, 2338,       0,       0,      26,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  123, 2501,       0,       0,      45,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  123, 2723,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  124, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC English/Deseret/United States
    {  126, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Anguilla
    {  127, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Antigua and Barbuda
    {  128, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Australia
    {  129, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Austria
    {  130, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Bahamas
    {  131, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Barbados
    {  132, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Belgium
    {  133, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Belize
    {  134, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Bermuda
    {  135, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Botswana
    {  136, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/British Indian Ocean Territory
    {  137, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/British Virgin Islands
    {  139, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Cameroon
    {  141, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Cayman Islands
    {  142, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Christmas Island
    {  143, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Cocos Islands
    {  144, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Cook Islands
    {  145, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Cyprus
    {  146, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Czechia
    {  147, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Denmark
    {  148, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Diego Garcia
    {  149, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Dominica
    {  150, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Eritrea
    {  151, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Eswatini
    {  152, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Europe
    {  153, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Falkland Islands
    {  154, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Fiji
    {  155, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Finland
    {  156, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/France
    {  157, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Gambia
    {  158, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Germany
    {  159, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Ghana
    {  160, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Gibraltar
    {  161, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Grenada
    {  163, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Guernsey
    {  164, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Guyana
    {  165, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Hong Kong
    {  166, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Hungary
    {  167, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/India
    {  168, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Indonesia
    {  169, 2338,       0,       0,      26,    0,    0,   65,  0,  0, 19,  0,  0,  3, }, // Europe/Dublin English/Latin/Ireland
    {  169, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  170, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Isle of Man
    {  171, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Israel
    {  172, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Italy
    {  173, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Jamaica
    {  174, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Jersey
    {  175, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Kenya
    {  176, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Kiribati
    {  177, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Lesotho
    {  178, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Liberia
    {  179, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Macao
    {  180, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Madagascar
    {  181, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Malawi
    {  182, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Malaysia
    {  183, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Maldives
    {  184, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Malta
    {  185, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Marshall Islands
    {  186, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Mauritius
    {  187, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Micronesia
    {  188, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Montserrat
    {  189, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Namibia
    {  190, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Nauru
    {  191, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Netherlands
    {  192, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/New Zealand
    {  193, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Nigeria
    {  194, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Niue
    {  195, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Norfolk Island
    {  196, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Northern Mariana Islands
    {  197, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Norway
    {  198, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Pakistan
    {  199, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Palau
    {  200, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Papua New Guinea
    {  202, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Pitcairn
    {  203, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Poland
    {  204, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Portugal
    {  206, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Romania
    {  207, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Rwanda
    {  208, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Saint Helena
    {  209, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Saint Kitts and Nevis
    {  210, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Saint Lucia
    {  211, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Saint Vincent and Grenadines
    {  212, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Samoa
    {  213, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Seychelles
    {  214, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Sierra Leone
    {  215, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Singapore
    {  216, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Sint Maarten
    {  217, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Slovakia
    {  218, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Slovenia
    {  219, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Solomon Islands
    {  220, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/South Africa
    {  221, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/South Georgia and South Sandwich Islands
    {  222, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/South Sudan
    {  223, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Spain
    {  224, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Sudan
    {  225, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Sweden
    {  226, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Switzerland
    {  227, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Tanzania
    {  228, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Tokelau
    {  229, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Tonga
    {  230, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Trinidad and Tobago
    {  231, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Turks and Caicos Islands
    {  232, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Tuvalu
    {  233, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Uganda
    {  234, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/United Arab Emirates
    {  235, 2501,       0,       0,      45,    0,    0,   68,  0,  0, 19,  0,  0,  3, }, // Europe/London English/Latin/United Kingdom
    {  235, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  238, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Vanuatu
    {  239, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/world
    {  240, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Zambia
    {  241, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Zimbabwe
    {  242, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC English/Shavian/United Kingdom
    {  243, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Erzya/Cyrillic/Russia
    {  244, 2455,       0,    2266,       0,    0,    0,    0,  0, 28,  0,  0,  3,  0, }, // Etc/UTC Esperanto/Latin/world
    {  244, 2338,       0,       0,    2294,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  244, 2501,       0,       0,    2313,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/London
    {  245, 2455,       0,    2331,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Estonian/Latin/Estonia
    {  245, 2338,       0,       0,    2356,    0,    0,    0,  0,  0, 12,  0,  0,  0, }, // Europe/Dublin
    {  245, 2501,       0,       0,    2368,    0,    0,    0,  0,  0, 13,  0,  0,  0, }, // Europe/London
    {  246, 2455,       0,    2381,       0,    0,    0,    0,  0, 22,  0,  0,  3,  0, }, // Etc/UTC Ewe/Latin/Ghana
    {  246, 2338,       0,       0,    2403,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  246, 2501,       0,       0,    2427,    0,    0,    0,  0,  0, 27,  0,  0,  0, }, // Europe/London
    {  248, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Ewondo/Latin/Cameroon
    {  249, 2455,       0,    2454,       0,    0,    0,    0,  0, 18,  0,  0,  3,  0, }, // Etc/UTC Faroese/Latin/Faroe Islands
    {  249, 2338,       0,       0,    2472,    0,    0,    0,  0,  0, 15,  0,  0,  0, }, // Europe/Dublin
    {  249, 2501,       0,       0,    2487,    0,    0,    0,  0,  0, 16,  0,  0,  0, }, // Europe/London
    {  251, 2455,       0,       0,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Filipino/Latin/Philippines
    {  251, 2338,       0,       0,    2503,    0,    0,    0,  0,  0, 27,  0,  0,  0, }, // Europe/Dublin
    {  251, 2501,       0,       0,    2530,    0,    0,    0,  0,  0, 27,  0,  0,  0, }, // Europe/London
    {  252, 2455,       0,    2557,       0,    0,    0,    0,  0, 13,  0,  0,  3,  0, }, // Etc/UTC Finnish/Latin/Finland
    {  252, 2338,       0,       0,    2570,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/Dublin
    {  252, 2501,       0,       0,    2587,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  253, 2455,       0,    2606,       0,    0,   71,    0,  0, 25,  0,  0,  2,  0, }, // Etc/UTC French/Latin/France
    {  253, 2338,       0,       0,    2631,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  253, 2501,       0,       0,    2653,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  253, 2723,       0,       0,       0,   73,    3,    6,  0,  0,  0,  2,  3,  3, }, // Pacific/Honolulu
    {  260, 2338,       0,       0,    2676,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin French/Latin/Canada
    {  260, 2501,       0,       0,    2700,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/London
    {  299, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Friulian/Latin/Italy
    {  300, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Fulah/Latin/Senegal
    {  306, 2455,       0,    2725,       0,    0,   75,    0,  0, 56,  0,  0,  6,  0, }, // Etc/UTC Fulah/Adlam/Guinea
    {  306, 2338,       0,       0,    2781,    0,    0,    0,  0,  0, 60,  0,  0,  0, }, // Europe/Dublin
    {  306, 2501,       0,       0,    2841,    0,    0,    0,  0,  0, 58,  0,  0,  0, }, // Europe/London
    {  306, 2723,       0,       0,       0,   81,   81,   87,  0,  0,  0,  6,  6,  6, }, // Pacific/Honolulu
    {  324, 2455,       0,    2899,       0,    0,    0,    0,  0, 34,  0,  0,  3,  0, }, // Etc/UTC Gaelic/Latin/United Kingdom
    {  324, 2338,       0,       0,    2933,    0,    0,   93,  0,  0, 19,  0,  0, 12, }, // Europe/Dublin
    {  324, 2501,       0,       0,    2952,    0,    0,  105,  0,  0, 25,  0,  0,  3, }, // Europe/London
    {  324, 2723,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  325, 2455,       0,    2977,       0,    0,    0,    0,  0, 36,  0,  0,  3,  0, }, // Etc/UTC Ga/Latin/Ghana
    {  325, 2338,       0,       0,    3013,    0,    0,    0,  0,  0, 27,  0,  0,  0, }, // Europe/Dublin
    {  325, 2501,       0,       0,    3040,    0,    0,    0,  0,  0, 42,  0,  0,  0, }, // Europe/London
    {  326, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Galician/Latin/Spain
    {  326, 2338,       0,       0,    3107,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  326, 2501,       0,       0,    3130,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  327, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Ganda/Latin/Uganda
    {  328, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Geez/Ethiopic/Ethiopia
    {  330, 2455,       0,    3153,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Georgian/Georgian/Georgia
    {  330, 2338,       0,       0,    3179,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/Dublin
    {  330, 2501,       0,       0,    3204,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  331, 2455,       0,    3227,       0,    0,    0,    0,  0, 21,  0,  0,  3,  0, }, // Etc/UTC German/Latin/Germany
    {  331, 2338,       0,       0,    3248,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/Dublin
    {  331, 2501,       0,       0,    3266,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/London
    {  338, 2455,       0,    3286,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Greek/Greek/Greece
    {  338, 2338,       0,       0,    3312,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  338, 2501,       0,       0,    3335,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/London
    {  340, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Guarani/Latin/Paraguay
    {  341, 2455,       0,    3355,       0,    0,    0,    0,  0, 20,  0,  0,  3,  0, }, // Etc/UTC Gujarati/Gujarati/India
    {  341, 2338,       0,       0,    3375,    0,    0,    0,  0,  0, 14,  0,  0,  0, }, // Europe/Dublin
    {  341, 2501,       0,       0,    3389,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  342, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Gusii/Latin/Kenya
    {  343, 2455,       0,    2606,       0,    0,   71,    0,  0, 25,  0,  0,  2,  0, }, // Etc/UTC Haitian/Latin/Haiti
    {  343, 2338,       0,       0,    2631,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  343, 2501,       0,       0,    2653,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  343, 2723,       0,       0,       0,   73,    3,    6,  0,  0,  0,  2,  3,  3, }, // Pacific/Honolulu
    {  344, 2455,       0,    3408,       0,    0,    0,    0,  0, 23,  0,  0,  3,  0, }, // Etc/UTC Hausa/Latin/Nigeria
    {  344, 2338,       0,       0,    3431,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  344, 2501,       0,       0,    3454,    0,    0,    0,  0,  0, 27,  0,  0,  0, }, // Europe/London
    {  345, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Hausa/Arabic/Nigeria
    {  346, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Hausa/Arabic/Sudan
    {  349, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Hawaiian/Latin/United States
    {  349, 2723,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  350, 2455,       0,    3481,       0,    0,    0,    0,  0, 19,  0,  0,  3,  0, }, // Etc/UTC Hebrew/Hebrew/Israel
    {  350, 2338,       0,       0,    3500,    0,    0,    0,  0,  0, 15,  0,  0,  0, }, // Europe/Dublin
    {  350, 2501,       0,       0,    3515,    0,    0,    0,  0,  0, 16,  0,  0,  0, }, // Europe/London
    {  351, 2455,       0,    3531,       0,    0,    0,    0,  0, 19,  0,  0,  3,  0, }, // Etc/UTC Hindi/Devanagari/India
    {  351, 2338,       0,       0,    3550,    0,    0,    0,  0,  0, 14,  0,  0,  0, }, // Europe/Dublin
    {  351, 2501,       0,       0,    3564,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  351, 2723,       0,       0,       0,  108,  108,    3,  0,  0,  0,  6,  6,  3, }, // Pacific/Honolulu
    {  352, 2455,       0,       0,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Hindi/Latin/India
    {  352, 2338,       0,       0,      26,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  352, 2501,       0,       0,      45,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  352, 2723,       0,       0,       0,    3,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  353, 2455,       0,    3588,       0,    0,    0,    0,  0, 19,  0,  0,  3,  0, }, // Etc/UTC Hungarian/Latin/Hungary
    {  353, 2338,       0,       0,    3607,    0,    0,    0,  0,  0, 12,  0,  0,  0, }, // Europe/Dublin
    {  353, 2501,       0,       0,    3619,    0,    0,    0,  0,  0, 14,  0,  0,  0, }, // Europe/London
    {  354, 2455,       0,    3633,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Icelandic/Latin/Iceland
    {  354, 2338,       0,       0,    3659,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  354, 2501,       0,       0,    3678,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  355, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Ido/Latin/world
    {  356, 2455,       0,    3699,       0,    0,    0,    0,  0, 19,  0,  0,  3,  0, }, // Etc/UTC Igbo/Latin/Nigeria
    {  356, 2338,       0,       0,    3718,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {  356, 2501,       0,       0,    3738,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/London
    {  357, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Inari Sami/Latin/Finland
    {  358, 2455,       0,    3758,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Indonesian/Latin/Indonesia
    {  358, 2338,       0,       0,    3787,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  358, 2501,       0,       0,    3809,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/London
    {  359, 2455,       0,    3834,       0,    0,    0,    0,  0, 28,  0,  0,  3,  0, }, // Etc/UTC Interlingua/Latin/world
    {  359, 2338,       0,       0,    3862,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {  359, 2501,       0,       0,    3883,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  360, 2455,       0,    3904,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Interlingue/Latin/Estonia
    {  360, 2338,       0,       0,    3930,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/Dublin
    {  360, 2501,       0,       0,    3955,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  361, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Inuktitut/Canadian Aboriginal/Canada
    {  363, 2455,       0,    3978,       0,    0,    0,    0,  0, 18,  0,  0,  3,  0, }, // Etc/UTC Irish/Latin/Ireland
    {  363, 2338,       0,       0,    3996,    0,    0,  114,  0,  0, 27,  0,  0,  3, }, // Europe/Dublin
    {  363, 2501,       0,       0,    4023,    0,    0,  117,  0,  0, 25,  0,  0,  3, }, // Europe/London
    {  365, 2455,       0,    4048,       0,    0,    0,    0,  0, 27,  0,  0,  3,  0, }, // Etc/UTC Italian/Latin/Italy
    {  365, 2338,       0,       0,    4075,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  365, 2501,       0,       0,    4098,    0,    0,    0,  0,  0, 26,  0,  0,  0, }, // Europe/London
    {  369, 2455,       0,    4124,       0,    0,    0,    0,  0,  5,  0,  0,  3,  0, }, // Etc/UTC Japanese/Japanese/Japan
    {  369, 2338,       0,       0,    4129,    0,    0,    0,  0,  0,  9,  0,  0,  0, }, // Europe/Dublin
    {  369, 2501,       0,       0,    4138,    0,    0,    0,  0,  0,  5,  0,  0,  0, }, // Europe/London
    {  370, 2455,       0,    4143,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Javanese/Latin/Indonesia
    {  370, 2338,       0,       0,    4172,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  370, 2501,       0,       0,    4194,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/London
    {  371, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Jju/Latin/Nigeria
    {  372, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Jola-Fonyi/Latin/Senegal
    {  373, 2455,       0,    4214,       0,    0,    0,    0,  0, 23,  0,  0,  3,  0, }, // Etc/UTC Kabuverdianu/Latin/Cape Verde
    {  373, 2338,       0,       0,    4237,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {  373, 2501,       0,       0,    4258,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/London
    {  374, 2455,       0,    4280,       0,    0,    0,    0,  0, 23,  0,  0,  3,  0, }, // Etc/UTC Kabyle/Latin/Algeria
    {  374, 2338,       0,       0,    4303,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {  374, 2501,       0,       0,    4324,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  375, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kako/Latin/Cameroon
    {  376, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kalaallisut/Latin/Greenland
    {  377, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kalenjin/Latin/Kenya
    {  378, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kamba/Latin/Kenya
    {  379, 2455,       0,    4348,       0,    0,    0,    0,  0, 21,  0,  0,  3,  0, }, // Etc/UTC Kannada/Kannada/India
    {  379, 2338,       0,       0,    4369,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/Dublin
    {  379, 2501,       0,       0,    4387,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  380, 2455,       0,    4406,       0,    0,    0,    0,  0, 23,  0,  0,  3,  0, }, // Etc/UTC Kashmiri/Arabic/India
    {  380, 2338,       0,       0,    4429,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/Dublin
    {  380, 2501,       0,       0,    4447,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  381, 2455,       0,    4466,       0,    0,    0,    0,  0, 23,  0,  0,  3,  0, }, // Etc/UTC Kashmiri/Devanagari/India
    {  382, 2455,       0,    4489,       0,    0,    0,    0,  0, 32,  0,  0,  3,  0, }, // Etc/UTC Kazakh/Cyrillic/Kazakhstan
    {  382, 2338,       0,       0,    4521,    0,    0,    0,  0,  0, 26,  0,  0,  0, }, // Europe/Dublin
    {  382, 2501,       0,       0,    4547,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  383, 2455,       0,    4571,       0,    0,    0,    0,  0, 32,  0,  0,  3,  0, }, // Etc/UTC Kazakh/Arabic/China
    {  383, 2338,       0,       0,    4603,    0,    0,    0,  0,  0, 26,  0,  0,  0, }, // Europe/Dublin
    {  383, 2501,       0,       0,    4629,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  384, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kenyang/Latin/Cameroon
    {  385, 2455,       0,    4653,       0,    0,    0,    0,  0, 22,  0,  0,  3,  0, }, // Etc/UTC Khmer/Khmer/Cambodia
    {  385, 2338,       0,       0,    4675,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/Dublin
    {  385, 2501,       0,       0,    4700,    0,    0,    0,  0,  0, 34,  0,  0,  0, }, // Europe/London
    {  386, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kiche/Latin/Guatemala
    {  387, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kikuyu/Latin/Kenya
    {  388, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kinyarwanda/Latin/Rwanda
    {  389, 2455,       0,    4734,       0,    0,    0,    0,  0, 19,  0,  0,  3,  0, }, // Etc/UTC Konkani/Devanagari/India
    {  389, 2338,       0,       0,    4753,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/Dublin
    {  389, 2501,       0,       0,    4771,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  390, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Konkani/Latin/India
    {  391, 2455,       0,    4790,       0,    0,    0,    0,  0,  6,  0,  0,  3,  0, }, // Etc/UTC Korean/Korean/South Korea
    {  391, 2338,       0,       0,    4796,    0,    0,    0,  0,  0,  8,  0,  0,  0, }, // Europe/Dublin
    {  391, 2501,       0,       0,    4804,    0,    0,    0,  0,  0,  9,  0,  0,  0, }, // Europe/London
    {  394, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Koyraboro Senni/Latin/Mali
    {  395, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Koyra Chiini/Latin/Mali
    {  396, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kpelle/Latin/Liberia
    {  398, 2455,       0,    4813,       0,    0,  120,    0,  0, 24,  0,  0,  3,  0, }, // Etc/UTC Kurdish/Latin/Turkey
    {  398, 2338,       0,       0,    4837,    0,    0,    0,  0,  0, 26,  0,  0,  0, }, // Europe/Dublin
    {  398, 2501,       0,       0,    4863,    0,    0,    0,  0,  0, 26,  0,  0,  0, }, // Europe/London
    {  399, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kwasio/Latin/Cameroon
    {  400, 2455,       0,    4889,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Kyrgyz/Cyrillic/Kyrgyzstan
    {  400, 2338,       0,       0,    4914,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  400, 2501,       0,       0,    4936,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  401, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Lakota/Latin/United States
    {  402, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Langi/Latin/Tanzania
    {  403, 2455,       0,    4959,       0,    0,    0,    0,  0, 18,  0,  0,  3,  0, }, // Etc/UTC Lao/Lao/Laos
    {  403, 2338,       0,       0,    4977,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  403, 2501,       0,       0,    5001,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  404, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Latin/Latin/Vatican City
    {  405, 2455,       0,    5025,       0,    0,    0,    0,  0, 31,  0,  0,  3,  0, }, // Etc/UTC Latvian/Latin/Latvia
    {  405, 2338,       0,       0,    5056,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  405, 2501,       0,       0,    5075,    0,    0,    0,  0,  0, 28,  0,  0,  0, }, // Europe/London
    {  406, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Lingala/Latin/Congo - Kinshasa
    {  410, 2455,       0,    5103,       0,    0,    0,    0,  0, 28,  0,  0,  3,  0, }, // Etc/UTC Lithuanian/Latin/Lithuania
    {  410, 2338,       0,       0,    5131,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  410, 2501,       0,       0,    5153,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/London
    {  411, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Lojban/Latin/world
    {  412, 2455,       0,    5178,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Lower Sorbian/Latin/Germany
    {  412, 2338,       0,       0,    5204,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  412, 2501,       0,       0,    5223,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  413, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Low German/Latin/Germany
    {  415, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Luba-Katanga/Latin/Congo - Kinshasa
    {  416, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Lule Sami/Latin/Sweden
    {  418, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Luo/Latin/Kenya
    {  419, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Luxembourgish/Latin/Luxembourg
    {  419, 2338,       0,       0,    5244,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/Dublin
    {  419, 2501,       0,       0,    5261,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  420, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Luyia/Latin/Kenya
    {  421, 2455,       0,    5280,       0,    0,    0,    0,  0, 30,  0,  0,  3,  0, }, // Etc/UTC Macedonian/Cyrillic/Macedonia
    {  421, 2338,       0,       0,    5310,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  421, 2501,       0,       0,    5332,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  422, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Machame/Latin/Tanzania
    {  423, 2455,       0,    3531,       0,    0,    0,    0,  0, 19,  0,  0,  3,  0, }, // Etc/UTC Maithili/Devanagari/India
    {  424, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Makhuwa-Meetto/Latin/Mozambique
    {  425, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Makonde/Latin/Tanzania
    {  426, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Malagasy/Latin/Madagascar
    {  427, 2455,       0,    5353,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Malayalam/Malayalam/India
    {  427, 2338,       0,       0,    5382,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  427, 2501,       0,       0,    5405,    0,    0,    0,  0,  0, 27,  0,  0,  0, }, // Europe/London
    {  428, 2455,       0,    5432,       0,    0,    0,    0,  0, 23,  0,  0,  3,  0, }, // Etc/UTC Malay/Latin/Malaysia
    {  428, 2338,       0,       0,    5455,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {  428, 2501,       0,       0,    5475,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/London
    {  429, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Malay/Arabic/Brunei
    {  430, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Malay/Arabic/Malaysia
    {  434, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Maltese/Latin/Malta
    {  435, 2455,       0,    5500,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Manipuri/Bangla/India
    {  436, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Manipuri/Meitei Mayek/India
    {  437, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Manx/Latin/Isle of Man
    {  438, 2455,       0,    5529,       0,    0,    0,    0,  0, 17,  0,  0,  3,  0, }, // Etc/UTC Maori/Latin/New Zealand
    {  438, 2338,       0,       0,    5546,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  438, 2501,       0,       0,    5565,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  439, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Mapuche/Latin/Chile
    {  440, 2455,       0,    4734,       0,    0,    0,    0,  0, 19,  0,  0,  3,  0, }, // Etc/UTC Marathi/Devanagari/India
    {  440, 2338,       0,       0,    5584,    0,    0,    0,  0,  0, 16,  0,  0,  0, }, // Europe/Dublin
    {  440, 2501,       0,       0,    5600,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  441, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Masai/Latin/Kenya
    {  443, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Mazanderani/Arabic/Iran
    {  444, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Meru/Latin/Kenya
    {  445, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Meta/Latin/Cameroon
    {  446, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Mohawk/Latin/Canada
    {  447, 2455,       0,    5619,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Mongolian/Cyrillic/Mongolia
    {  447, 2338,       0,       0,    5648,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {  447, 2501,       0,       0,    5669,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/London
    {  448, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Mongolian/Mongolian/China
    {  449, 2455,       0,    5687,       0,    0,    0,    0,  0, 33,  0,  0,  3,  0, }, // Etc/UTC Mongolian/Mongolian/Mongolia
    {  450, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Morisyen/Latin/Mauritius
    {  451, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Mundang/Latin/Cameroon
    {  452, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Muscogee/Latin/United States
    {  453, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Nama/Latin/Namibia
    {  454, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Navajo/Latin/United States
    {  455, 2455,       0,    5720,       0,    0,    0,    0,  0, 17,  0,  0,  3,  0, }, // Etc/UTC Nepali/Devanagari/Nepal
    {  455, 2338,       0,       0,    3550,    0,    0,    0,  0,  0, 14,  0,  0,  0, }, // Europe/Dublin
    {  455, 2501,       0,       0,    5737,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  457, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Ngiemboon/Latin/Cameroon
    {  458, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Ngomba/Latin/Cameroon
    {  459, 2455,       0,    5761,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Nigerian Pidgin/Latin/Nigeria
    {  459, 2338,       0,       0,    5790,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/Dublin
    {  459, 2501,       0,       0,    5807,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  460, 2455,       0,    5830,       0,    0,    0,    0,  0, 28,  0,  0,  3,  0, }, // Etc/UTC Nko/Nko/Guinea
    {  460, 2338,       0,       0,    5858,    0,    0,    0,  0,  0, 26,  0,  0,  0, }, // Europe/Dublin
    {  460, 2501,       0,       0,    5884,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  461, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Northern Luri/Arabic/Iran
    {  463, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Northern Sami/Latin/Norway
    {  464, 2455,       0,    5905,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Northern Sami/Latin/Finland
    {  464, 2338,       0,       0,    5934,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {  464, 2501,       0,       0,    5955,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/London
    {  466, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Northern Sotho/Latin/South Africa
    {  467, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC North Ndebele/Latin/Zimbabwe
    {  468, 2455,       0,    5975,       0,    0,    0,    0,  0, 23,  0,  0,  3,  0, }, // Etc/UTC Norwegian Bokmal/Latin/Norway
    {  468, 2338,       0,       0,    5998,    0,    0,    0,  0,  0, 14,  0,  0,  0, }, // Europe/Dublin
    {  468, 2501,       0,       0,    6012,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/London
    {  468, 2723,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  470, 2455,       0,    5975,       0,    0,    0,    0,  0, 23,  0,  0,  3,  0, }, // Etc/UTC Norwegian Nynorsk/Latin/Norway
    {  470, 2338,       0,       0,    6029,    0,    0,    0,  0,  0, 14,  0,  0,  0, }, // Europe/Dublin
    {  470, 2501,       0,       0,    6043,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/London
    {  470, 2723,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  471, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Nuer/Latin/South Sudan
    {  472, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Nyanja/Latin/Malawi
    {  473, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Nyankole/Latin/Uganda
    {  474, 2455,       0,    6060,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Occitan/Latin/France
    {  474, 2338,       0,       0,    6085,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  474, 2501,       0,       0,    6109,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  475, 2455,       0,    6130,       0,    0,    0,    0,  0, 24,  0,  0,  3,  0, }, // Etc/UTC Occitan/Latin/Spain
    {  475, 2338,       0,       0,    6154,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {  475, 2501,       0,       0,    6109,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  476, 2455,       0,    6175,       0,    0,    0,    0,  0, 21,  0,  0,  3,  0, }, // Etc/UTC Odia/Odia/India
    {  476, 2338,       0,       0,    6196,    0,    0,    0,  0,  0, 16,  0,  0,  0, }, // Europe/Dublin
    {  476, 2501,       0,       0,    6212,    0,    0,    0,  0,  0, 26,  0,  0,  0, }, // Europe/London
    {  477, 2455,       0,    6238,       0,    0,    0,    0,  0, 31,  0,  0,  3,  0, }, // Etc/UTC Oromo/Latin/Ethiopia
    {  477, 2338,       0,       0,    6269,    0,    0,    0,  0,  0, 32,  0,  0,  0, }, // Europe/Dublin
    {  477, 2501,       0,       0,    6301,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  479, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Osage/Osage/United States
    {  481, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Ossetic/Cyrillic/Russia
    {  482, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Papiamento/Latin/Curacao
    {  484, 2455,       0,    6324,       0,    0,    0,    0,  0, 16,  0,  0,  3,  0, }, // Etc/UTC Pashto/Arabic/Afghanistan
    {  484, 2338,       0,       0,    6340,    0,    0,    0,  0,  0, 15,  0,  0,  0, }, // Europe/Dublin
    {  484, 2501,       0,       0,    6355,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  486, 2455,       0,    6374,       0,    0,    0,    0,  0, 17,  0,  0,  3,  0, }, // Etc/UTC Persian/Arabic/Iran
    {  486, 2338,       0,       0,    6391,    0,    0,    0,  0,  0, 15,  0,  0,  0, }, // Europe/Dublin
    {  486, 2501,       0,       0,    6406,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  488, 2455,       0,    6427,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Polish/Latin/Poland
    {  488, 2338,       0,       0,    6456,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {  488, 2501,       0,       0,    6477,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/London
    {  489, 2455,       0,    6497,       0,    0,    0,    0,  0, 28,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Brazil
    {  489, 2338,       0,       0,    6525,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  489, 2501,       0,       0,    6548,    0,    0,    0,  0,  0, 26,  0,  0,  0, }, // Europe/London
    {  490, 2455,       0,    6574,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Angola
    {  490, 2338,       0,       0,    6599,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  490, 2501,       0,       0,    6623,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  491, 2455,       0,    6574,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Cape Verde
    {  491, 2338,       0,       0,    6599,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  491, 2501,       0,       0,    6623,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  492, 2455,       0,    6574,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Equatorial Guinea
    {  492, 2338,       0,       0,    6599,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  492, 2501,       0,       0,    6623,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  493, 2455,       0,    6574,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Guinea-Bissau
    {  493, 2338,       0,       0,    6599,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  493, 2501,       0,       0,    6623,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  494, 2455,       0,    6574,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Luxembourg
    {  494, 2338,       0,       0,    6599,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  494, 2501,       0,       0,    6623,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  495, 2455,       0,    6574,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Macao
    {  495, 2338,       0,       0,    6599,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  495, 2501,       0,       0,    6623,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  496, 2455,       0,    6574,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Mozambique
    {  496, 2338,       0,       0,    6599,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  496, 2501,       0,       0,    6623,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  497, 2455,       0,    6574,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Portugal
    {  497, 2338,       0,       0,    6599,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  497, 2501,       0,       0,    6623,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  498, 2455,       0,    6574,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Sao Tome and Principe
    {  498, 2338,       0,       0,    6599,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  498, 2501,       0,       0,    6623,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  499, 2455,       0,    6574,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Switzerland
    {  499, 2338,       0,       0,    6599,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  499, 2501,       0,       0,    6623,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  500, 2455,       0,    6574,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Timor-Leste
    {  500, 2338,       0,       0,    6599,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  500, 2501,       0,       0,    6623,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  501, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Prussian/Latin/Poland
    {  502, 2455,       0,    6646,       0,    0,    0,    0,  0, 22,  0,  0,  3,  0, }, // Etc/UTC Punjabi/Gurmukhi/India
    {  502, 2338,       0,       0,    6668,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/Dublin
    {  502, 2501,       0,       0,    6685,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  503, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Punjabi/Arabic/Pakistan
    {  504, 2455,       0,    6708,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Quechua/Latin/Peru
    {  504, 2338,       0,       0,    6737,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  504, 2501,       0,       0,    6761,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  507, 2455,       0,    6785,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Romanian/Latin/Romania
    {  507, 2338,       0,       0,    6811,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  507, 2501,       0,       0,    6833,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  509, 2455,       0,    6854,       0,    0,    0,    0,  0, 23,  0,  0,  3,  0, }, // Etc/UTC Romansh/Latin/Switzerland
    {  509, 2501,       0,       0,    6877,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/London
    {  510, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Rombo/Latin/Tanzania
    {  511, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Rundi/Latin/Burundi
    {  512, 2455,       0,    6899,       0,    0,    0,    0,  0, 32,  0,  0,  3,  0, }, // Etc/UTC Russian/Cyrillic/Russia
    {  512, 2338,       0,       0,    6931,    0,    0,    0,  0,  0, 27,  0,  0,  0, }, // Europe/Dublin
    {  512, 2501,       0,       0,    6958,    0,    0,    0,  0,  0, 28,  0,  0,  0, }, // Europe/London
    {  518, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Rwa/Latin/Tanzania
    {  519, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Saho/Latin/Eritrea
    {  520, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Sakha/Cyrillic/Russia
    {  521, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Samburu/Latin/Kenya
    {  522, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Sango/Latin/Central African Republic
    {  523, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Sangu/Latin/Tanzania
    {  524, 2455,       0,    6986,       0,    0,    0,    0,  0, 21,  0,  0,  3,  0, }, // Etc/UTC Sanskrit/Devanagari/India
    {  525, 2455,       0,    7007,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Santali/Ol Chiki/India
    {  526, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Santali/Devanagari/India
    {  527, 2455,       0,    7033,       0,    0,  123,    0,  0, 28,  0,  0,  3,  0, }, // Etc/UTC Sardinian/Latin/Italy
    {  527, 2338,       0,       0,    7061,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  527, 2501,       0,       0,    7084,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/London
    {  527, 2723,       0,       0,       0,  126,  126,  129,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  528, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Sena/Latin/Mozambique
    {  529, 2455,       0,    7106,       0,    0,    0,    0,  0, 30,  0,  0,  3,  0, }, // Etc/UTC Serbian/Cyrillic/Serbia
    {  529, 2338,       0,       0,    7136,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  529, 2501,       0,       0,    7159,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/London
    {  530, 2455,       0,    1090,       0,    0,    0,    0,  0, 32,  0,  0,  3,  0, }, // Etc/UTC Serbian/Cyrillic/Bosnia and Herzegovina
    {  530, 2338,       0,       0,    7181,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/Dublin
    {  530, 2501,       0,       0,    7206,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  533, 2455,       0,    7230,       0,    0,    0,    0,  0, 32,  0,  0,  3,  0, }, // Etc/UTC Serbian/Latin/Bosnia and Herzegovina
    {  533, 2338,       0,       0,    7262,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/Dublin
    {  533, 2501,       0,       0,    7287,    0,    0,    0,  0,  0, 26,  0,  0,  0, }, // Europe/London
    {  534, 2455,       0,    7313,       0,    0,    0,    0,  0, 30,  0,  0,  3,  0, }, // Etc/UTC Serbian/Latin/Kosovo
    {  534, 2338,       0,       0,    7343,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  534, 2501,       0,       0,    7366,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  535, 2455,       0,    7313,       0,    0,    0,    0,  0, 30,  0,  0,  3,  0, }, // Etc/UTC Serbian/Latin/Montenegro
    {  535, 2338,       0,       0,    7343,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  535, 2501,       0,       0,    7366,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  536, 2455,       0,    7313,       0,    0,    0,    0,  0, 30,  0,  0,  3,  0, }, // Etc/UTC Serbian/Latin/Serbia
    {  536, 2338,       0,       0,    7343,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  536, 2501,       0,       0,    7366,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  537, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Shambala/Latin/Tanzania
    {  538, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Shona/Latin/Zimbabwe
    {  539, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Sichuan Yi/Yi/China
    {  540, 2455,       0,    7389,       0,    0,    0,    0,  0, 24,  0,  0,  3,  0, }, // Etc/UTC Sicilian/Latin/Italy
    {  540, 2338,       0,       0,    7413,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {  540, 2501,       0,       0,    7433,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  541, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Sidamo/Latin/Ethiopia
    {  542, 2455,       0,    7454,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Silesian/Latin/Poland
    {  542, 2338,       0,       0,    7483,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  542, 2501,       0,       0,    7506,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  543, 2455,       0,    7527,       0,    0,  132,    0,  0, 15,  0,  0,  5,  0, }, // Etc/UTC Sindhi/Arabic/Pakistan
    {  543, 2338,       0,       0,    7542,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/Dublin
    {  543, 2501,       0,       0,    7560,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/London
    {  544, 2455,       0,    7585,       0,    0,    0,    0,  0, 17,  0,  0,  3,  0, }, // Etc/UTC Sindhi/Devanagari/India
    {  545, 2455,       0,    7602,       0,    0,    0,    0,  0, 18,  0,  0,  3,  0, }, // Etc/UTC Sinhala/Sinhala/Sri Lanka
    {  545, 2338,       0,       0,    7620,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {  545, 2501,       0,       0,    7640,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/London
    {  546, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Skolt Sami/Latin/Finland
    {  547, 2455,       0,    7665,       0,    0,    0,    0,  0, 24,  0,  0,  3,  0, }, // Etc/UTC Slovak/Latin/Slovakia
    {  547, 2338,       0,       0,    7689,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {  547, 2501,       0,       0,    7709,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/London
    {  547, 2723,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  548, 2455,       0,    7726,       0,    0,    0,    0,  0, 28,  0,  0,  3,  0, }, // Etc/UTC Slovenian/Latin/Slovenia
    {  548, 2338,       0,       0,    7754,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {  548, 2501,       0,       0,    7774,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  549, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Soga/Latin/Uganda
    {  550, 2455,       0,    7795,       0,    0,  137,    0,  0, 30,  0,  0, 11,  0, }, // Etc/UTC Somali/Latin/Somalia
    {  550, 2338,       0,       0,    7825,    0,    0,    0,  0,  0, 30,  0,  0,  0, }, // Europe/Dublin
    {  550, 2501,       0,       0,    7855,    0,    0,    0,  0,  0, 29,  0,  0,  0, }, // Europe/London
    {  554, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Southern Kurdish/Arabic/Iran
    {  556, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Southern Sami/Latin/Sweden
    {  558, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Southern Sotho/Latin/South Africa
    {  560, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC South Ndebele/Latin/South Africa
    {  561, 2455,       0,    7884,       0,    0,    0,    0,  0, 27,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Spain
    {  561, 2338,       0,       0,    7911,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/Dublin
    {  561, 2501,       0,       0,    7936,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  562, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Argentina
    {  562, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  563, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Belize
    {  563, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  564, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Bolivia
    {  564, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  565, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Brazil
    {  565, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  568, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Chile
    {  568, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  569, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Colombia
    {  569, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  570, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Costa Rica
    {  570, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  571, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Cuba
    {  571, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  572, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Dominican Republic
    {  572, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  573, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Ecuador
    {  573, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  574, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/El Salvador
    {  574, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  576, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Guatemala
    {  576, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  577, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Honduras
    {  577, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  578, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Latin America
    {  578, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  579, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Mexico
    {  579, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  580, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Nicaragua
    {  580, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  581, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Panama
    {  581, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  582, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Paraguay
    {  582, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  583, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Peru
    {  583, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  585, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Puerto Rico
    {  585, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  586, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/United States
    {  586, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  586, 2723,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  587, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Uruguay
    {  587, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  588, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Venezuela
    {  588, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  589, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Standard Moroccan Tamazight/Tifinagh/Morocco
    {  590, 2455,       0,    3758,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Sundanese/Latin/Indonesia
    {  591, 2455,       0,    7984,       0,    0,    0,    0,  0, 33,  0,  0,  3,  0, }, // Etc/UTC Swahili/Latin/Tanzania
    {  591, 2338,       0,       0,    8017,    0,    0,    0,  0,  0, 33,  0,  0,  0, }, // Europe/Dublin
    {  591, 2501,       0,       0,    8050,    0,    0,    0,  0,  0, 34,  0,  0,  0, }, // Europe/London
    {  593, 2455,       0,    8084,       0,    0,    0,    0,  0, 12,  0,  0,  3,  0, }, // Etc/UTC Swahili/Latin/Kenya
    {  595, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Swati/Latin/South Africa
    {  597, 2455,       0,    8096,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Swedish/Latin/Sweden
    {  597, 2338,       0,       0,    8122,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/Dublin
    {  597, 2501,       0,       0,    8140,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/London
    {  597, 2723,       0,       0,       0,  148,  159,  176,  0,  0,  0, 11, 17, 17, }, // Pacific/Honolulu
    {  600, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Swiss German/Latin/Switzerland
    {  603, 2455,       0,    8158,       0,    0,    0,    0,  0, 21,  0,  0,  3,  0, }, // Etc/UTC Syriac/Syriac/Iraq
    {  603, 2338,       0,       0,    8179,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  603, 2501,       0,       0,    8201,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  605, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Tachelhit/Tifinagh/Morocco
    {  607, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Tai Dam/Tai Viet/Vietnam
    {  608, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Taita/Latin/Kenya
    {  609, 2455,       0,    8224,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Tajik/Cyrillic/Tajikistan
    {  609, 2338,       0,       0,    8250,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/Dublin
    {  609, 2501,       0,       0,    8275,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/London
    {  610, 2455,       0,    8300,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Tamil/Tamil/India
    {  610, 2338,       0,       0,    8326,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  610, 2501,       0,       0,    8345,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  614, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Taroko/Latin/Taiwan
    {  615, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Tasawaq/Latin/Niger
    {  616, 2455,       0,    8366,       0,    0,    0,    0,  0, 31,  0,  0,  3,  0, }, // Etc/UTC Tatar/Cyrillic/Russia
    {  616, 2338,       0,       0,    8397,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  616, 2501,       0,       0,    8421,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  617, 2455,       0,    8442,       0,    0,    0,    0,  0, 21,  0,  0,  3,  0, }, // Etc/UTC Telugu/Telugu/India
    {  617, 2338,       0,       0,    8463,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {  617, 2501,       0,       0,    8483,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  618, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Teso/Latin/Uganda
    {  620, 2455,       0,    8502,       0,    0,    0,    0,  0, 17,  0,  0,  3,  0, }, // Etc/UTC Thai/Thai/Thailand
    {  620, 2338,       0,       0,    8519,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {  620, 2501,       0,       0,    8539,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/London
    {  621, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Tibetan/Tibetan/China
    {  623, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Tigre/Ethiopic/Eritrea
    {  624, 2455,       0,    8556,       0,    0,    0,    0,  0, 14,  0,  0,  3,  0, }, // Etc/UTC Tigrinya/Ethiopic/Ethiopia
    {  624, 2338,       0,       0,    8570,    0,    0,    0,  0,  0, 15,  0,  0,  0, }, // Europe/Dublin
    {  624, 2501,       0,       0,    8585,    0,    0,    0,  0,  0, 13,  0,  0,  0, }, // Europe/London
    {  626, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Tok Pisin/Latin/Papua New Guinea
    {  627, 2455,       0,    8598,       0,    0,  193,    0,  0, 17,  0,  0,  3,  0, }, // Etc/UTC Tongan/Latin/Tonga
    {  627, 2338,       0,       0,    8615,    0,    0,    0,  0,  0, 30,  0,  0,  0, }, // Europe/Dublin
    {  627, 2501,       0,       0,    8645,    0,    0,    0,  0,  0, 30,  0,  0,  0, }, // Europe/London
    {  627, 2723,       0,       0,       0,  196,  196,  199,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  628, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Tsonga/Latin/South Africa
    {  629, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Tswana/Latin/South Africa
    {  631, 2455,       0,    8675,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Turkish/Latin/Turkey
    {  631, 2338,       0,       0,    8700,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  631, 2501,       0,       0,    8722,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  633, 2455,       0,    8741,       0,    0,    0,    0,  0, 28,  0,  0,  3,  0, }, // Etc/UTC Turkmen/Latin/Turkmenistan
    {  633, 2338,       0,       0,    8769,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  633, 2501,       0,       0,    8793,    0,    0,    0,  0,  0, 29,  0,  0,  0, }, // Europe/London
    {  634, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Tyap/Latin/Nigeria
    {  635, 2455,       0,    8822,       0,    0,    0,    0,  0, 33,  0,  0,  3,  0, }, // Etc/UTC Ukrainian/Cyrillic/Ukraine
    {  635, 2338,       0,       0,    8855,    0,    0,    0,  0,  0, 26,  0,  0,  0, }, // Europe/Dublin
    {  635, 2501,       0,       0,    8881,    0,    0,    0,  0,  0, 34,  0,  0,  0, }, // Europe/London
    {  636, 2455,       0,    8915,       0,    0,    0,    0,  0, 24,  0,  0,  3,  0, }, // Etc/UTC Upper Sorbian/Latin/Germany
    {  636, 2338,       0,       0,    8939,    0,    0,    0,  0,  0, 15,  0,  0,  0, }, // Europe/Dublin
    {  636, 2501,       0,       0,    8954,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/London
    {  637, 2455,       0,    8972,       0,    0,    0,    0,  0, 24,  0,  0,  3,  0, }, // Etc/UTC Urdu/Arabic/Pakistan
    {  637, 2338,       0,       0,    8996,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/Dublin
    {  637, 2501,       0,       0,    9014,    0,    0,    0,  0,  0, 13,  0,  0,  0, }, // Europe/London
    {  639, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Uyghur/Arabic/China
    {  639, 2338,       0,       0,    9027,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {  639, 2501,       0,       0,    9048,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  640, 2455,       0,    9069,       0,    0,    0,    0,  0, 27,  0,  0,  3,  0, }, // Etc/UTC Uzbek/Latin/Uzbekistan
    {  640, 2338,       0,       0,    9096,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {  640, 2501,       0,       0,    9117,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  641, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Uzbek/Arabic/Afghanistan
    {  642, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Uzbek/Cyrillic/Uzbekistan
    {  642, 2338,       0,       0,    9138,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  642, 2501,       0,       0,    9157,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  643, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Vai/Vai/Liberia
    {  645, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Venda/Latin/South Africa
    {  646, 2455,       0,    9176,       0,    0,    0,    0,  0, 20,  0,  0,  3,  0, }, // Etc/UTC Vietnamese/Latin/Vietnam
    {  646, 2338,       0,       0,    9196,    0,    0,    0,  0,  0, 16,  0,  0,  0, }, // Europe/Dublin
    {  646, 2501,       0,       0,    9212,    0,    0,    0,  0,  0, 14,  0,  0,  0, }, // Europe/London
    {  646, 2723,       0,       0,       0,  202,    3,    6,  0,  0,  0,  7,  3,  3, }, // Pacific/Honolulu
    {  647, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Volapuk/Latin/world
    {  648, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Vunjo/Latin/Tanzania
    {  649, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Walloon/Latin/Belgium
    {  650, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Walser/Latin/Switzerland
    {  651, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Warlpiri/Latin/Australia
    {  652, 2455,       0,    9226,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Welsh/Latin/United Kingdom
    {  652, 2338,       0,       0,    9252,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  652, 2501,       0,       0,    9274,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/London
    {  653, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Western Balochi/Arabic/Pakistan
    {  658, 2455,       0,    9291,       0,    0,    0,    0,  0, 22,  0,  0,  3,  0, }, // Etc/UTC Western Frisian/Latin/Netherlands
    {  658, 2338,       0,       0,    9313,    0,    0,    0,  0,  0, 16,  0,  0,  0, }, // Europe/Dublin
    {  658, 2501,       0,       0,    9329,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/London
    {  659, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Wolaytta/Ethiopic/Ethiopia
    {  660, 2455,       0,    9346,       0,    0,    0,    0,  0, 31,  0,  0,  3,  0, }, // Etc/UTC Wolof/Latin/Senegal
    {  660, 2338,       0,       0,    9377,    0,    0,    0,  0,  0, 13,  0,  0,  0, }, // Europe/Dublin
    {  660, 2501,       0,       0,    9390,    0,    0,    0,  0,  0, 28,  0,  0,  0, }, // Europe/London
    {  661, 2455,       0,       0,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Xhosa/Latin/South Africa
    {  661, 2338,       0,       0,      26,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  661, 2501,       0,       0,      45,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  662, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Yangben/Latin/Cameroon
    {  663, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Yiddish/Hebrew/Ukraine
    {  664, 2455,       0,    9418,       0,    0,    0,    0,  0, 19,  0,  0,  3,  0, }, // Etc/UTC Yoruba/Latin/Nigeria
    {  664, 2338,       0,       0,    9437,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {  664, 2501,       0,       0,    9458,    0,    0,    0,  0,  0, 16,  0,  0,  0, }, // Europe/London
    {  665, 2455,       0,    9474,       0,    0,    0,    0,  0, 19,  0,  0,  3,  0, }, // Etc/UTC Yoruba/Latin/Benin
    {  665, 2338,       0,       0,    9493,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {  665, 2501,       0,       0,    9514,    0,    0,    0,  0,  0, 16,  0,  0,  0, }, // Europe/London
    {  666, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Zarma/Latin/Niger
    {  667, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Zhuang/Latin/China
    {  668, 2455,       0,    9530,       0,    0,    0,    0,  0, 31,  0,  0,  3,  0, }, // Etc/UTC Zulu/Latin/South Africa
    {  668, 2338,       0,       0,    9561,    0,    0,    0,  0,  0, 30,  0,  0,  0, }, // Europe/Dublin
    {  668, 2501,       0,       0,    9591,    0,    0,    0,  0,  0, 32,  0,  0,  0, }, // Europe/London
    {  669, 2455,       0,    9623,       0,    0,    0,    0,  0, 23,  0,  0,  3,  0, }, // Etc/UTC Kaingang/Latin/Brazil
    {  669, 2338,       0,       0,    9646,    0,    0,    0,  0,  0, 12,  0,  0,  0, }, // Europe/Dublin
    {  669, 2501,       0,       0,    9658,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  670, 2455,       0,    9677,       0,    0,    0,    0,  0, 34,  0,  0,  3,  0, }, // Etc/UTC Nheengatu/Latin/Brazil
    {  670, 2338,       0,       0,    9711,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  670, 2501,       0,       0,    9733,    0,    0,    0,  0,  0, 30,  0,  0,  0, }, // Europe/London
    {  673, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Haryanvi/Devanagari/India
    {  674, 2455,       0,    9763,       0,    0,    0,    0,  0, 18,  0,  0,  3,  0, }, // Etc/UTC Northern Frisian/Latin/Germany
    {  674, 2338,       0,       0,    9781,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  674, 2501,       0,       0,    9800,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/London
    {  675, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Rajasthani/Devanagari/India
    {  676, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Moksha/Cyrillic/Russia
    {  677, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Toki Pona/Latin/world
    {  678, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Pijin/Latin/Solomon Islands
    {  679, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Obolo/Latin/Nigeria
    {  680, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Baluchi/Arabic/Pakistan
    {  681, 2455,       0,    9817,       0,    0,    0,    0,  0, 20,  0,  0,  3,  0, }, // Etc/UTC Baluchi/Latin/Pakistan
    {  681, 2338,       0,       0,    9837,    0,    0,    0,  0,  0, 16,  0,  0,  0, }, // Europe/Dublin
    {  681, 2501,       0,       0,    9853,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/London
    {  682, 2455,       0,    9875,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Ligurian/Latin/Italy
    {  682, 2338,       0,       0,    9901,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  682, 2501,       0,       0,    9920,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/London
    {  683, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Rohingya/Hanifi/Myanmar
    {  685, 2455,       0,    8972,       0,    0,    0,    0,  0, 24,  0,  0,  3,  0, }, // Etc/UTC Torwali/Arabic/Pakistan
    {  685, 2338,       0,       0,    8996,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/Dublin
    {  685, 2501,       0,       0,    9014,    0,    0,    0,  0,  0, 13,  0,  0,  0, }, // Europe/London
    {  686, 2455,       0,    9940,       0,    0,  209,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Anii/Latin/Benin
    {  686, 2338,       0,       0,    9966,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/Dublin
    {  686, 2501,       0,       0,    9984,    0,    0,    0,  0,  0, 33,  0,  0,  0, }, // Europe/London
    {  687, 2455,       0,   10017,       0,    0,    0,    0,  0, 21,  0,  0,  3,  0, }, // Etc/UTC Kangri/Devanagari/India
    {  687, 2338,       0,       0,   10038,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/Dublin
    {  687, 2501,       0,       0,   10055,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/London
    {  688, 2455,       0,   10080,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Venetian/Latin/Italy
    {  688, 2338,       0,       0,   10105,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {  688, 2501,       0,       0,   10125,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/London
    {  689, 2455,       0,   10145,       0,    0,    0,    0,  0, 21,  0,  0,  3,  0, }, // Etc/UTC Kuvi/Latin/India
    {  689, 2338,       0,       0,   10166,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/Dublin
    {  689, 2501,       0,       0,   10184,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  690, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kuvi/Devanagari/India
    {  691, 2455,       0,   10205,       0,    0,    0,    0,  0, 22,  0,  0,  3,  0, }, // Etc/UTC Kuvi/Odia/India
    {  691, 2338,       0,       0,   10227,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  691, 2501,       0,       0,   10246,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/London
    {  692, 2455,       0,    8442,       0,    0,    0,    0,  0, 21,  0,  0,  3,  0, }, // Etc/UTC Kuvi/Telugu/India
    {  692, 2338,       0,       0,    8463,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {  692, 2501,       0,       0,    8483,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  693, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kara-Kalpak/Cyrillic/Uzbekistan
    {  695, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Swampy Cree/Canadian Aboriginal/Canada
    {  696,    0,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  0,  0, } // Terminal row
}; // Zone naming table

// Sorted by locale index, then meta key
static inline constexpr LocaleMetaZoneLongNames localeMetaZoneLongNameTable[] = {
    // locInd, metaKey, (generic, standard, DST){ind, sz}
    {    0,    0,       0,       9,      27,  9, 18, 16, }, // Acre C/AnyScript/AnyTerritory
    {    0,    5,       0,      43,       0,  0, 16,  0, }, // Afghanistan
    {    0,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central
    {    0,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {    0,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {    0,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {    0,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {    0,  128,     235,     246,     266, 11, 20, 18, }, // Almaty
    {    0,  135,     284,     295,     315, 11, 20, 18, }, // Amazon
    {    0,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {    0,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {    0,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {    0,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {    0,  207,     552,     563,     583, 11, 20, 18, }, // Anadyr
    {    0,  214,     601,     611,     630, 10, 19, 19, }, // Apia
    {    0,  219,     649,     659,     678, 10, 19, 17, }, // Aqtau
    {    0,  225,     695,     706,     726, 11, 20, 18, }, // Aqtobe
    {    0,  232,     744,     756,     777, 12, 21, 21, }, // Arabian
    {    0,  240,     798,     812,     835, 14, 23, 21, }, // Argentina
    {    0,  250,     856,     878,     909, 22, 31, 29, }, // Argentina_Western
    {    0,  268,     938,     950,     971, 12, 21, 19, }, // Armenia
    {    0,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {    0,  295,    1047,    1070,    1102, 23, 32, 32, }, // Australia_Central
    {    0,  313,    1134,    1165,    1205, 31, 40, 40, }, // Australia_CentralWestern
    {    0,  338,    1245,    1268,    1300, 23, 32, 32, }, // Australia_Eastern
    {    0,  356,    1332,    1355,    1387, 23, 32, 32, }, // Australia_Western
    {    0,  374,    1419,    1434,    1458, 15, 24, 22, }, // Azerbaijan
    {    0,  385,    1480,    1491,    1511, 11, 20, 18, }, // Azores
    {    0,  397,    1529,    1544,    1568, 15, 24, 22, }, // Bangladesh
    {    0,  415,       0,    1590,       0,  0, 11,  0, }, // Bhutan
    {    0,  422,       0,    1601,       0,  0, 12,  0, }, // Bolivia
    {    0,  437,    1613,    1626,    1648, 13, 22, 20, }, // Brasilia
    {    0,  454,       0,    1668,       0,  0, 11,  0, }, // Brunei
    {    0,  461,    1679,    1694,    1718, 15, 24, 22, }, // Cape_Verde
    {    0,  472,       0,    1740,       0,  0, 10,  0, }, // Casey
    {    0,  478,       0,    1750,       0,  0, 22,  0, }, // Chamorro
    {    0,  487,    1772,    1784,    1805, 12, 21, 21, }, // Chatham
    {    0,  495,    1826,    1836,    1855, 10, 19, 17, }, // Chile
    {    0,  501,    1872,    1882,    1901, 10, 19, 19, }, // China
    {    0,  507,       0,    1920,       0,  0, 21,  0, }, // Christmas
    {    0,  517,       0,    1941,       0,  0, 18,  0, }, // Cocos
    {    0,  523,    1959,    1972,    1994, 13, 22, 20, }, // Colombia
    {    0,  532,    2014,    2031,    2057, 17, 26, 24, }, // Cook
    {    0,  537,    2081,    2090,    2108,  9, 18, 18, }, // Cuba
    {    0,  548,       0,    2126,       0,  0, 10,  0, }, // Davis
    {    0,  564,       0,    2136,       0,  0, 21,  0, }, // DumontDUrville
    {    0,  601,       0,    2157,       0,  0, 16,  0, }, // East_Timor
    {    0,  612,    2173,    2191,    2218, 18, 27, 25, }, // Easter
    {    0,  619,       0,    2243,       0,  0, 12,  0, }, // Ecuador
    {    0,  627,    2255,    2276,    2306, 21, 30, 28, }, // Europe_Central
    {    0,  642,    2334,    2355,    2385, 21, 30, 28, }, // Europe_Eastern
    {    0,  657,       0,    2413,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {    0,  680,    2442,    2463,    2493, 21, 30, 28, }, // Europe_Western
    {    0,  695,    2521,    2542,    2572, 21, 30, 28, }, // Falkland
    {    0,  704,    2600,    2609,    2627,  9, 18, 16, }, // Fiji
    {    0,  709,       0,    2643,       0,  0, 18,  0, }, // French_Guiana
    {    0,  723,       0,    2661,       0,  0, 32,  0, }, // French_Southern
    {    0,  746,       0,    2693,       0,  0, 14,  0, }, // Galapagos
    {    0,  756,       0,    2707,       0,  0, 12,  0, }, // Gambier
    {    0,  764,    2719,    2731,    2752, 12, 21, 19, }, // Georgia
    {    0,  772,       0,    2771,       0,  0, 20,  0, }, // Gilbert_Islands
    {    0,  788,       0,    2791,       0,  0, 19,  0, }, // GMT
    {    0,  802,    2810,    2824,    2847, 14, 23, 21, }, // Greenland
    {    0,  830,    2868,    2887,    2915, 19, 28, 26, }, // Greenland_Eastern
    {    0,  848,    2941,    2960,    2988, 19, 28, 26, }, // Greenland_Western
    {    0,  866,       0,    3014,       0,  0, 18,  0, }, // Guam
    {    0,  871,       0,    3032,       0,  0, 18,  0, }, // Gulf
    {    0,  876,       0,    3050,       0,  0, 11,  0, }, // Guyana
    {    0,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {    0,  899,    3139,    3153,    3176, 14, 23, 21, }, // Hong_Kong
    {    0,  909,    3197,    3206,    3224,  9, 18, 16, }, // Hovd
    {    0,  914,       0,    3240,       0,  0, 19,  0, }, // India
    {    0,  920,       0,    3259,       0,  0, 17,  0, }, // Indian_Ocean
    {    0,  933,       0,    3276,       0,  0, 14,  0, }, // Indochina
    {    0,  943,       0,    3290,       0,  0, 22,  0, }, // Indonesia_Central
    {    0,  961,       0,    3312,       0,  0, 22,  0, }, // Indonesia_Eastern
    {    0,  979,       0,    3334,       0,  0, 22,  0, }, // Indonesia_Western
    {    0,  997,    3356,    3365,    3383,  9, 18, 18, }, // Iran
    {    0, 1008,    3401,    3413,    3434, 12, 21, 19, }, // Irkutsk
    {    0, 1016,    3453,    3464,    3484, 11, 20, 20, }, // Israel
    {    0, 1023,    3504,    3514,    3533, 10, 19, 19, }, // Japan
    {    0, 1029,    3552,    3566,    3589, 14, 23, 21, }, // Kamchatka
    {    0, 1047,       0,    3610,       0,  0, 15,  0, }, // Kazakhstan
    {    0, 1058,       0,    3625,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {    0, 1077,       0,    3645,       0,  0, 20,  0, }, // Kazakhstan_Western
    {    0, 1106,    3665,    3676,    3696, 11, 20, 20, }, // Korea
    {    0, 1112,       0,    3716,       0,  0, 11,  0, }, // Kosrae
    {    0, 1119,    3727,    3743,    3768, 16, 25, 23, }, // Krasnoyarsk
    {    0, 1151,       0,    3791,       0,  0, 15,  0, }, // Kyrgystan
    {    0, 1161,       0,    3806,       0,  0, 10,  0, }, // Lanka
    {    0, 1175,       0,    3816,       0,  0, 17,  0, }, // Line_Islands
    {    0, 1188,    3833,    3847,    3870, 14, 23, 23, }, // Lord_Howe
    {    0, 1198,    3893,    3903,    3922, 10, 19, 17, }, // Macau
    {    0, 1204,    3939,    3951,    3972, 12, 21, 19, }, // Magadan
    {    0, 1219,       0,    3991,       0,  0, 13,  0, }, // Malaysia
    {    0, 1228,       0,    4004,       0,  0, 13,  0, }, // Maldives
    {    0, 1237,       0,    4017,       0,  0, 14,  0, }, // Marquesas
    {    0, 1247,       0,    4031,       0,  0, 21,  0, }, // Marshall_Islands
    {    0, 1264,    4052,    4066,    4089, 14, 23, 21, }, // Mauritius
    {    0, 1274,       0,    4110,       0,  0, 11,  0, }, // Mawson
    {    0, 1281,    4121,    4141,    4170, 20, 29, 29, }, // Mexico_Pacific
    {    0, 1296,    4199,    4215,    4240, 16, 25, 23, }, // Mongolia
    {    0, 1305,    4263,    4274,    4294, 11, 20, 18, }, // Moscow
    {    0, 1312,       0,    4312,       0,  0, 12,  0, }, // Myanmar
    {    0, 1320,       0,    4324,       0,  0, 10,  0, }, // Nauru
    {    0, 1326,       0,    4334,       0,  0, 10,  0, }, // Nepal
    {    0, 1332,    4344,    4362,    4389, 18, 27, 25, }, // New_Caledonia
    {    0, 1346,    4414,    4430,    4455, 16, 25, 25, }, // New_Zealand
    {    0, 1358,    4480,    4497,    4523, 17, 26, 26, }, // Newfoundland
    {    0, 1371,       0,    4549,       0,  0,  9,  0, }, // Niue
    {    0, 1376,    4558,    4577,    4605, 19, 28, 28, }, // Norfolk
    {    0, 1384,    4633,    4657,    4690, 24, 33, 31, }, // Noronha
    {    0, 1392,       0,    4721,       0,  0, 29,  0, }, // North_Mariana
    {    0, 1406,    4750,    4766,    4791, 16, 25, 23, }, // Novosibirsk
    {    0, 1418,    4814,    4823,    4841,  9, 18, 16, }, // Omsk
    {    0, 1428,    4857,    4870,    4892, 13, 22, 20, }, // Pakistan
    {    0, 1437,       0,    4912,       0,  0, 10,  0, }, // Palau
    {    0, 1443,       0,    4922,       0,  0, 21,  0, }, // Papua_New_Guinea
    {    0, 1460,    4943,    4956,    4978, 13, 22, 20, }, // Paraguay
    {    0, 1469,    4998,    5007,    5025,  9, 18, 16, }, // Peru
    {    0, 1474,    5041,    5056,    5080, 15, 24, 22, }, // Philippines
    {    0, 1486,       0,    5102,       0,  0, 20,  0, }, // Phoenix_Islands
    {    0, 1502,    5122,    5148,    5183, 26, 35, 35, }, // Pierre_Miquelon
    {    0, 1518,       0,    5218,       0,  0, 13,  0, }, // Pitcairn
    {    0, 1527,       0,    5231,       0,  0, 12,  0, }, // Ponape
    {    0, 1534,       0,    5243,       0,  0, 16,  0, }, // Pyongyang
    {    0, 1544,    5259,    5273,    5296, 14, 23, 21, }, // Qyzylorda
    {    0, 1554,       0,    5317,       0,  0, 12,  0, }, // Reunion
    {    0, 1562,       0,    5329,       0,  0, 12,  0, }, // Rothera
    {    0, 1570,    5341,    5354,    5376, 13, 22, 20, }, // Sakhalin
    {    0, 1579,    5396,    5407,    5427, 11, 20, 18, }, // Samara
    {    0, 1596,    5445,    5464,    5492, 19, 28, 28, }, // Samoa
    {    0, 1602,       0,    5520,       0,  0, 15,  0, }, // Seychelles
    {    0, 1624,       0,    5535,       0,  0, 23,  0, }, // Singapore
    {    0, 1634,       0,    5558,       0,  0, 20,  0, }, // Solomon
    {    0, 1642,       0,    5578,       0,  0, 18,  0, }, // South_Georgia
    {    0, 1656,       0,    5596,       0,  0, 13,  0, }, // Suriname
    {    0, 1676,       0,    5609,       0,  0, 10,  0, }, // Syowa
    {    0, 1682,       0,    5619,       0,  0, 11,  0, }, // Tahiti
    {    0, 1689,    5630,    5641,    5661, 11, 20, 20, }, // Taipei
    {    0, 1696,       0,    5681,       0,  0, 15,  0, }, // Tajikistan
    {    0, 1724,       0,    5696,       0,  0, 12,  0, }, // Tokelau
    {    0, 1732,    5708,    5718,    5737, 10, 19, 17, }, // Tonga
    {    0, 1738,       0,    5754,       0,  0, 10,  0, }, // Truk
    {    0, 1750,    5764,    5781,    5807, 17, 26, 24, }, // Turkmenistan
    {    0, 1763,       0,    5831,       0,  0, 11,  0, }, // Tuvalu
    {    0, 1777,    5842,    5854,    5875, 12, 21, 19, }, // Uruguay
    {    0, 1792,    5894,    5909,    5933, 15, 24, 22, }, // Uzbekistan
    {    0, 1803,    5955,    5967,    5988, 12, 21, 19, }, // Vanuatu
    {    0, 1811,       0,    6007,       0,  0, 14,  0, }, // Venezuela
    {    0, 1821,    6021,    6037,    6062, 16, 25, 23, }, // Vladivostok
    {    0, 1833,    6085,    6099,    6122, 14, 23, 21, }, // Volgograd
    {    0, 1843,       0,    6143,       0,  0, 11,  0, }, // Vostok
    {    0, 1850,       0,    6154,       0,  0, 16,  0, }, // Wake
    {    0, 1855,       0,    6170,       0,  0, 20,  0, }, // Wallis
    {    0, 1862,    6190,    6202,    6223, 12, 21, 19, }, // Yakutsk
    {    0, 1870,    6242,    6260,    6287, 18, 27, 25, }, // Yekaterinburg
    {    0, 1892,       0,    6312,       0,  0, 10,  0, }, // Yukon
    {    1,    0,    6322,    6332,    6353, 10, 21, 19, }, // Acre Abkhazian/Cyrillic/Georgia
    {    1,    5,       0,    6372,       0,  0, 10,  0, }, // Afghanistan
    {    1,   17,       0,    6382,       0,  0, 16,  0, }, // Africa_Central
    {    1,   32,       0,    6398,       0,  0, 19,  0, }, // Africa_Eastern
    {    1,   65,       0,    6417,       0,  0, 15,  0, }, // Africa_Southern
    {    1,   81,    6432,    6452,    6490, 20, 38, 36, }, // Africa_Western
    {    1,  107,    6526,    6533,    6558,  7, 25, 23, }, // Alaska
    {    1,  128,    6581,    6595,    6620, 14, 25, 23, }, // Almaty
    {    1,  135,    6643,    6651,    6677,  8, 26, 24, }, // Amazon
    {    1,  142,    6701,    6718,    6753, 17, 35, 33, }, // America_Central
    {    1,  158,    6786,    6806,    6844, 20, 38, 36, }, // America_Eastern
    {    1,  174,    6880,    6912,    6955, 32, 43, 41, }, // America_Mountain
    {    1,  191,    6996,    7015,    7045, 19, 30, 28, }, // America_Pacific
    {    1,  207,    7073,    7089,    7112, 16, 23, 21, }, // Anadyr
    {    1,  214,    7133,    7137,    7159,  4, 22, 20, }, // Apia
    {    1,  219,    7179,    7190,    7213, 11, 23, 20, }, // Aqtau
    {    1,  225,    7233,    7245,    7268, 12, 23, 21, }, // Aqtobe
    {    1,  232,    7289,    7305,    7339, 16, 34, 32, }, // Arabian
    {    1,  240,    7371,    7380,    7407,  9, 27, 25, }, // Argentina
    {    1,  250,    7432,    7455,    7496, 23, 41, 39, }, // Argentina_Western
    {    1,  268,    7535,    7545,    7573, 10, 28, 26, }, // Armenia
    {    1,  286,    7599,    7617,    7646, 18, 29, 27, }, // Atlantic
    {    1,  295,    7673,    7692,    7729, 19, 37, 35, }, // Australia_Central
    {    1,  313,    7764,    7804,    7855, 40, 51, 49, }, // Australia_CentralWestern
    {    1,  338,    7904,    7926,    7966, 22, 40, 38, }, // Australia_Eastern
    {    1,  356,    8004,    8027,    8068, 23, 41, 39, }, // Australia_Western
    {    1,  374,    8107,    8118,    8147, 11, 29, 27, }, // Azerbaijan
    {    1,  385,    8174,    8190,    8224, 16, 34, 32, }, // Azores
    {    1,  397,    8256,    8265,    8292,  9, 27, 25, }, // Bangladesh
    {    1,  415,       0,    8317,       0,  0,  5,  0, }, // Bhutan
    {    1,  422,       0,    8322,       0,  0,  7,  0, }, // Bolivia
    {    1,  437,    8329,    8337,    8363,  8, 26, 24, }, // Brasilia
    {    1,  454,       0,    8387,       0,  0, 17,  0, }, // Brunei
    {    1,  461,    8404,    8414,    8442, 10, 28, 26, }, // Cape_Verde
    {    1,  472,       0,    8468,       0,  0,  5,  0, }, // Casey
    {    1,  478,       0,    8473,       0,  0,  7,  0, }, // Chamorro
    {    1,  487,    8480,    8485,    8508,  5, 23, 21, }, // Chatham
    {    1,  495,    8529,    8533,    8555,  4, 22, 20, }, // Chile
    {    1,  501,    8575,    8580,    8603,  5, 23, 21, }, // China
    {    1,  507,       0,    8624,       0,  0, 12,  0, }, // Christmas
    {    1,  517,       0,    8636,       0,  0, 17,  0, }, // Cocos
    {    1,  523,    8653,    8661,    8687,  8, 26, 24, }, // Colombia
    {    1,  532,    8711,    8730,    8767, 19, 37, 39, }, // Cook
    {    1,  537,    8806,    8810,    8832,  4, 22, 20, }, // Cuba
    {    1,  548,       0,    8852,       0,  0,  6,  0, }, // Davis
    {    1,  564,       0,    8858,       0,  0, 15,  0, }, // DumontDUrville
    {    1,  601,       0,    8873,       0,  0, 18,  0, }, // East_Timor
    {    1,  612,    8891,    8902,    8931, 11, 29, 27, }, // Easter
    {    1,  619,       0,    8958,       0,  0,  7,  0, }, // Ecuador
    {    1,  627,    8965,    8981,    9015, 16, 34, 32, }, // Europe_Central
    {    1,  642,    9047,    9066,    9103, 19, 37, 35, }, // Europe_Eastern
    {    1,  657,       0,    9138,       0,  0, 16,  0, }, // Europe_Further_Eastern
    {    1,  680,    9154,    9174,    9212, 20, 38, 36, }, // Europe_Western
    {    1,  695,    9248,    9268,    9306, 20, 38, 36, }, // Falkland
    {    1,  704,    9342,    9346,    9368,  4, 22, 20, }, // Fiji
    {    1,  709,       0,    9388,       0,  0, 17,  0, }, // French_Guiana
    {    1,  723,       0,    9405,       0,  0, 50,  0, }, // French_Southern
    {    1,  746,       0,    9455,       0,  0, 21,  0, }, // Galapagos
    {    1,  756,       0,    9476,       0,  0,  5,  0, }, // Gambier
    {    1,  764,    9481,    9490,    9517,  9, 27, 25, }, // Georgia
    {    1,  772,       0,    9542,       0,  0, 17,  0, }, // Gilbert_Islands
    {    1,  788,       0,    9559,       0,  0, 25,  0, }, // GMT
    {    1,  830,    9584,    9607,    9649, 23, 42, 39, }, // Greenland_Eastern
    {    1,  848,    9688,    9712,    9754, 24, 42, 40, }, // Greenland_Western
    {    1,  866,       0,    9794,       0,  0,  4,  0, }, // Guam
    {    1,  871,       0,    9798,       0,  0, 19,  0, }, // Gulf
    {    1,  876,       0,    9817,       0,  0,  6,  0, }, // Guyana
    {    1,  883,    9823,    9843,    9874, 20, 31, 29, }, // Hawaii_Aleutian
    {    1,  899,    9903,    9910,    9935,  7, 25, 23, }, // Hong_Kong
    {    1,  909,    9958,    9962,    9984,  4, 22, 20, }, // Hovd
    {    1,  914,       0,   10004,       0,  0,  5,  0, }, // India
    {    1,  920,       0,   10009,       0,  0, 15,  0, }, // Indian_Ocean
    {    1,  933,       0,   10024,       0,  0,  9,  0, }, // Indochina
    {    1,  943,       0,   10033,       0,  0, 19,  0, }, // Indonesia_Central
    {    1,  961,       0,   10052,       0,  0, 22,  0, }, // Indonesia_Eastern
    {    1,  979,       0,   10074,       0,  0, 23,  0, }, // Indonesia_Western
    {    1,  997,   10097,   10101,   10123,  4, 22, 20, }, // Iran
    {    1, 1008,   10143,   10150,   10175,  7, 25, 23, }, // Irkutsk
    {    1, 1016,   10198,   10205,   10230,  7, 25, 23, }, // Israel
    {    1, 1023,   10253,   10260,   10285,  7, 25, 23, }, // Japan
    {    1, 1029,   10308,   10333,   10376, 25, 43, 41, }, // Kamchatka
    {    1, 1058,       0,   10417,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {    1, 1077,       0,   10439,       0,  0, 23,  0, }, // Kazakhstan_Western
    {    1, 1106,   10462,   10468,   10492,  6, 24, 22, }, // Korea
    {    1, 1112,       0,   10514,       0,  0,  6,  0, }, // Kosrae
    {    1, 1119,   10520,   10531,   10560, 11, 29, 27, }, // Krasnoyarsk
    {    1, 1151,       0,   10587,       0,  0,  8,  0, }, // Kyrgystan
    {    1, 1161,       0,   10595,       0,  0,  9,  0, }, // Lanka
    {    1, 1175,       0,   10604,       0,  0, 13,  0, }, // Line_Islands
    {    1, 1188,   10617,   10625,   10651,  8, 26, 24, }, // Lord_Howe
    {    1, 1198,   10675,   10680,   10703,  5, 23, 21, }, // Macau
    {    1, 1204,   10724,   10731,   10756,  7, 25, 23, }, // Magadan
    {    1, 1219,       0,   10779,       0,  0,  8,  0, }, // Malaysia
    {    1, 1228,       0,   10787,       0,  0,  7,  0, }, // Maldives
    {    1, 1237,       0,   10794,       0,  0, 18,  0, }, // Marquesas
    {    1, 1247,       0,   10812,       0,  0, 26,  0, }, // Marshall_Islands
    {    1, 1264,   10838,   10845,   10870,  7, 25, 23, }, // Mauritius
    {    1, 1274,       0,   10893,       0,  0,  6,  0, }, // Mawson
    {    1, 1281,   10899,   10929,   10970, 30, 41, 39, }, // Mexico_Pacific
    {    1, 1296,   11009,   11019,   11047, 10, 28, 26, }, // Mongolia
    {    1, 1305,   11073,   11079,   11103,  6, 24, 22, }, // Moscow
    {    1, 1312,       0,   11125,       0,  0,  6,  0, }, // Myanmar
    {    1, 1320,       0,   11131,       0,  0,  5,  0, }, // Nauru
    {    1, 1326,       0,   11136,       0,  0,  5,  0, }, // Nepal
    {    1, 1332,   11141,   11154,   11185, 13, 31, 29, }, // New_Caledonia
    {    1, 1346,   11214,   11226,   11256, 12, 30, 28, }, // New_Zealand
    {    1, 1358,   11284,   11296,   11326, 12, 30, 28, }, // Newfoundland
    {    1, 1371,       0,   11354,       0,  0,  4,  0, }, // Niue
    {    1, 1376,   11358,   11365,   11390,  7, 25, 23, }, // Norfolk
    {    1, 1384,   11413,   11432,   11469, 19, 37, 35, }, // Noronha
    {    1, 1392,       0,   11504,       0,  0, 27,  0, }, // North_Mariana
    {    1, 1406,   11531,   11542,   11571, 11, 29, 27, }, // Novosibirsk
    {    1, 1418,   11598,   11602,   11624,  4, 22, 20, }, // Omsk
    {    1, 1428,   11644,   11652,   11678,  8, 26, 24, }, // Pakistan
    {    1, 1437,       0,   11702,       0,  0,  5,  0, }, // Palau
    {    1, 1443,       0,   11707,       0,  0, 19,  0, }, // Papua_New_Guinea
    {    1, 1460,   11726,   11734,   11760,  8, 26, 24, }, // Paraguay
    {    1, 1469,   11784,   11788,   11810,  4, 22, 20, }, // Peru
    {    1, 1474,   11830,   11838,   11864,  8, 26, 24, }, // Philippines
    {    1, 1486,       0,   11888,       0,  0, 15,  0, }, // Phoenix_Islands
    {    1, 1502,   11903,   11921,   11957, 18, 36, 34, }, // Pierre_Miquelon
    {    1, 1518,       0,   11991,       0,  0,  7,  0, }, // Pitcairn
    {    1, 1527,       0,   11998,       0,  0,  6,  0, }, // Ponape
    {    1, 1534,       0,   12004,       0,  0,  7,  0, }, // Pyongyang
    {    1, 1544,   12011,   12021,   12049, 10, 28, 26, }, // Qyzylorda
    {    1, 1554,       0,   12075,       0,  0,  7,  0, }, // Reunion
    {    1, 1562,       0,   12082,       0,  0,  6,  0, }, // Rothera
    {    1, 1570,   12088,   12095,   12120,  7, 25, 23, }, // Sakhalin
    {    1, 1579,   12143,   12157,   12182, 14, 25, 23, }, // Samara
    {    1, 1596,   12205,   12210,   12233,  5, 23, 21, }, // Samoa
    {    1, 1602,       0,   12254,       0,  0, 25,  0, }, // Seychelles
    {    1, 1624,       0,   12279,       0,  0,  8,  0, }, // Singapore
    {    1, 1634,       0,   12287,       0,  0, 26,  0, }, // Solomon
    {    1, 1642,       0,   12313,       0,  0, 16,  0, }, // South_Georgia
    {    1, 1656,       0,   12329,       0,  0,  7,  0, }, // Suriname
    {    1, 1676,       0,   12336,       0,  0,  5,  0, }, // Syowa
    {    1, 1682,       0,   12341,       0,  0,  5,  0, }, // Tahiti
    {    1, 1689,   12346,   12352,   12376,  6, 24, 22, }, // Taipei
    {    1, 1696,       0,   12398,       0,  0, 10,  0, }, // Tajikistan
    {    1, 1724,       0,   12408,       0,  0,  7,  0, }, // Tokelau
    {    1, 1732,   12415,   12420,   12443,  5, 23, 21, }, // Tonga
    {    1, 1738,       0,   12464,       0,  0,  4,  0, }, // Truk
    {    1, 1750,   12468,   12480,   12510, 12, 30, 28, }, // Turkmenistan
    {    1, 1763,       0,   12538,       0,  0,  6,  0, }, // Tuvalu
    {    1, 1777,   12544,   12551,   12576,  7, 25, 23, }, // Uruguay
    {    1, 1792,   12599,   12610,   12639, 11, 29, 27, }, // Uzbekistan
    {    1, 1803,   12666,   12673,   12698,  7, 25, 23, }, // Vanuatu
    {    1, 1811,       0,   12721,       0,  0,  9,  0, }, // Venezuela
    {    1, 1821,   12730,   12741,   12770, 11, 29, 27, }, // Vladivostok
    {    1, 1833,   12797,   12806,   12833,  9, 27, 25, }, // Volgograd
    {    1, 1843,       0,   12858,       0,  0, 10,  0, }, // Vostok
    {    1, 1850,       0,   12868,       0,  0,  4,  0, }, // Wake
    {    1, 1855,       0,   12872,       0,  0, 15,  0, }, // Wallis
    {    1, 1862,   12887,   12894,   12919,  7, 25, 23, }, // Yakutsk
    {    1, 1870,   12942,   12954,   12984, 12, 30, 28, }, // Yekaterinburg
    {    1, 1892,       0,   13012,       0,  0,  5,  0, }, // Yukon
    {    5,    5,       0,   13017,       0,  0, 14,  0, }, // Afghanistan Afrikaans/Latin/South Africa
    {    5,   17,       0,   13031,       0,  0, 19,  0, }, // Africa_Central
    {    5,   32,       0,   13050,       0,  0, 14,  0, }, // Africa_Eastern
    {    5,   65,       0,   13064,       0,  0, 24,  0, }, // Africa_Southern
    {    5,   81,   13088,   13102,   13125, 14, 23, 19, }, // Africa_Western
    {    5,  107,   13144,   13154,   13173, 10, 19, 16, }, // Alaska
    {    5,  135,   13189,   13200,   13220, 11, 20, 16, }, // Amazon
    {    5,  142,   13236,   13266,   13305, 30, 39, 36, }, // America_Central
    {    5,  158,   13341,   13372,   13412, 31, 40, 37, }, // America_Eastern
    {    5,  174,   13449,   13474,   13509, 25, 35, 32, }, // America_Mountain
    {    5,  191,   13541,   13554,   13576, 13, 22, 19, }, // America_Pacific
    {    5,  207,   13595,   13605,   13624, 10, 19, 15, }, // Anadyr
    {    5,  214,   13639,   13647,   13664,  8, 17, 14, }, // Apia
    {    5,  232,   13678,   13690,   13711, 12, 21, 18, }, // Arabian
    {    5,  240,   13729,   13743,   13766, 14, 23, 19, }, // Argentina
    {    5,  250,   13785,   13803,   13830, 18, 27, 23, }, // Argentina_Western
    {    5,  268,   13853,   13864,   13884, 11, 20, 16, }, // Armenia
    {    5,  286,   13900,   13914,   13937, 14, 23, 20, }, // Atlantic
    {    5,  295,   13957,   13981,   14014, 24, 33, 30, }, // Australia_Central
    {    5,  313,   14044,   14078,   14122, 34, 44, 40, }, // Australia_CentralWestern
    {    5,  338,   14162,   14187,   14221, 25, 34, 31, }, // Australia_Eastern
    {    5,  356,   14252,   14277,   14311, 25, 34, 31, }, // Australia_Western
    {    5,  374,   14342,   14357,   14381, 15, 24, 20, }, // Azerbaijan
    {    5,  385,   14401,   14410,   14428,  9, 18, 14, }, // Azores
    {    5,  397,   14442,   14456,   14479, 14, 23, 19, }, // Bangladesh
    {    5,  415,       0,   14498,       0,  0, 11,  0, }, // Bhutan
    {    5,  422,       0,   14509,       0,  0, 11,  0, }, // Bolivia
    {    5,  437,   14520,   14532,   14553, 12, 21, 17, }, // Brasilia
    {    5,  454,       0,   14570,       0,  0, 22,  0, }, // Brunei
    {    5,  461,   14592,   14606,   14629, 14, 23, 19, }, // Cape_Verde
    {    5,  478,       0,   14648,       0,  0, 21,  0, }, // Chamorro
    {    5,  487,   14669,   14680,   14700, 11, 20, 17, }, // Chatham
    {    5,  495,   14717,   14726,   14744,  9, 18, 14, }, // Chile
    {    5,  501,   14758,   14767,   14785,  9, 18, 15, }, // China
    {    5,  507,       0,   14800,       0,  0, 19,  0, }, // Christmas
    {    5,  517,       0,   14819,       0,  0, 16,  0, }, // Cocos
    {    5,  523,   14835,   14847,   14868, 12, 21, 17, }, // Colombia
    {    5,  532,   14885,   14900,   14924, 15, 24, 24, }, // Cook
    {    5,  537,   14948,   14956,   14973,  8, 17, 14, }, // Cuba
    {    5,  548,       0,   14987,       0,  0,  9,  0, }, // Davis
    {    5,  564,       0,   14996,       0,  0, 20,  0, }, // DumontDUrville
    {    5,  601,       0,   15016,       0,  0, 13,  0, }, // East_Timor
    {    5,  612,   15029,   15043,   15066, 14, 23, 19, }, // Easter
    {    5,  619,       0,   15085,       0,  0, 11,  0, }, // Ecuador
    {    5,  627,   15096,   15117,   15147, 21, 30, 26, }, // Europe_Central
    {    5,  642,   15173,   15189,   15214, 16, 25, 21, }, // Europe_Eastern
    {    5,  657,       0,   15235,       0,  0, 23,  0, }, // Europe_Further_Eastern
    {    5,  680,   15258,   15274,   15299, 16, 25, 21, }, // Europe_Western
    {    5,  695,   15320,   15339,   15367, 19, 28, 24, }, // Falkland
    {    5,  704,   15391,   15400,   15418,  9, 18, 14, }, // Fiji
    {    5,  709,       0,   15432,       0,  0, 16,  0, }, // French_Guiana
    {    5,  723,       0,   15448,       0,  0, 33,  0, }, // French_Southern
    {    5,  746,       0,   15481,       0,  0, 13,  0, }, // Galapagos
    {    5,  756,       0,   15494,       0,  0, 11,  0, }, // Gambier
    {    5,  764,   15505,   15516,   15536, 11, 20, 16, }, // Georgia
    {    5,  772,       0,   15552,       0,  0, 18,  0, }, // Gilbert_Islands
    {    5,  788,       0,   15570,       0,  0, 13,  0, }, // GMT
    {    5,  830,   15583,   15600,   15626, 17, 26, 22, }, // Greenland_Eastern
    {    5,  848,   15648,   15665,   15691, 17, 26, 22, }, // Greenland_Western
    {    5,  871,       0,   15713,       0,  0, 26,  0, }, // Gulf
    {    5,  876,       0,   15438,       0,  0, 10,  0, }, // Guyana
    {    5,  883,   15739,   15757,   15784, 18, 27, 24, }, // Hawaii_Aleutian
    {    5,  899,   15808,   15820,   15841, 12, 21, 17, }, // Hong_Kong
    {    5,  909,   15858,   15866,   15883,  8, 17, 13, }, // Hovd
    {    5,  914,       0,   15896,       0,  0, 18,  0, }, // India
    {    5,  920,       0,   15914,       0,  0, 18,  0, }, // Indian_Ocean
    {    5,  933,       0,   15932,       0,  0, 13,  0, }, // Indochina
    {    5,  943,       0,   15945,       0,  0, 24,  0, }, // Indonesia_Central
    {    5,  961,       0,   15969,       0,  0, 17,  0, }, // Indonesia_Eastern
    {    5,  979,       0,   15986,       0,  0, 17,  0, }, // Indonesia_Western
    {    5,  997,   16003,   16011,   16028,  8, 17, 14, }, // Iran
    {    5, 1008,   16042,   16054,   16075, 12, 21, 17, }, // Irkutsk
    {    5, 1016,   16092,   16102,   16121, 10, 19, 16, }, // Israel
    {    5, 1023,   16137,   16146,   16164,  9, 18, 15, }, // Japan
    {    5, 1029,   16179,   16207,   16244, 28, 37, 33, }, // Kamchatka
    {    5, 1047,       0,   16277,       0,  0, 13,  0, }, // Kazakhstan
    {    5, 1058,       0,   16290,       0,  0, 17,  0, }, // Kazakhstan_Eastern
    {    5, 1077,       0,   16307,       0,  0, 17,  0, }, // Kazakhstan_Western
    {    5, 1106,   16324,   16337,   16359, 13, 22, 19, }, // Korea
    {    5, 1112,       0,   16378,       0,  0, 10,  0, }, // Kosrae
    {    5, 1119,   16388,   16403,   16427, 15, 24, 20, }, // Krasnoyarsk
    {    5, 1151,       0,   16447,       0,  0, 13,  0, }, // Kyrgystan
    {    5, 1175,       0,   16460,       0,  0, 16,  0, }, // Line_Islands
    {    5, 1188,   16476,   16489,   16511, 13, 22, 19, }, // Lord_Howe
    {    5, 1204,   16530,   16541,   16561, 11, 20, 16, }, // Magadan
    {    5, 1219,       0,   16577,       0,  0, 12,  0, }, // Malaysia
    {    5, 1228,       0,   16589,       0,  0, 12,  0, }, // Maldives
    {    5, 1237,       0,   16601,       0,  0, 13,  0, }, // Marquesas
    {    5, 1247,       0,   16614,       0,  0, 19,  0, }, // Marshall_Islands
    {    5, 1264,   16633,   16646,   16668, 13, 22, 18, }, // Mauritius
    {    5, 1274,       0,   16686,       0,  0, 10,  0, }, // Mawson
    {    5, 1281,   16696,   16721,   16755, 25, 34, 31, }, // Mexico_Pacific
    {    5, 1296,   16786,   16801,   16825, 15, 24, 20, }, // Mongolia
    {    5, 1305,   16845,   16855,   16874, 10, 19, 15, }, // Moscow
    {    5, 1312,       0,   16889,       0,  0, 11,  0, }, // Myanmar
    {    5, 1320,       0,   16900,       0,  0,  9,  0, }, // Nauru
    {    5, 1326,       0,   16909,       0,  0,  9,  0, }, // Nepal
    {    5, 1332,   16918,   16936,   16963, 18, 27, 23, }, // New_Caledonia
    {    5, 1346,   16986,   17002,   17027, 16, 25, 22, }, // New_Zealand
    {    5, 1358,   17049,   17065,   17090, 16, 25, 22, }, // Newfoundland
    {    5, 1371,       0,   17112,       0,  0,  8,  0, }, // Niue
    {    5, 1376,   17120,   17137,   17163, 17, 26, 23, }, // Norfolk
    {    5, 1384,   17186,   17209,   17241, 23, 32, 28, }, // Noronha
    {    5, 1406,   17269,   17284,   17308, 15, 24, 20, }, // Novosibirsk
    {    5, 1418,   17328,   17336,   17353,  8, 17, 13, }, // Omsk
    {    5, 1428,   17366,   17378,   17399, 12, 21, 17, }, // Pakistan
    {    5, 1437,       0,   17416,       0,  0,  9,  0, }, // Palau
    {    5, 1443,       0,   17425,       0,  0, 22,  0, }, // Papua_New_Guinea
    {    5, 1460,   17447,   17459,   17480, 12, 21, 17, }, // Paraguay
    {    5, 1469,   17497,   17505,   17522,  8, 17, 13, }, // Peru
    {    5, 1474,   17535,   17549,   17572, 14, 23, 19, }, // Philippines
    {    5, 1486,       0,   17591,       0,  0, 17,  0, }, // Phoenix_Islands
    {    5, 1502,   17608,   17635,   17671, 27, 36, 33, }, // Pierre_Miquelon
    {    5, 1518,       0,   17704,       0,  0, 12,  0, }, // Pitcairn
    {    5, 1527,       0,   17716,       0,  0, 10,  0, }, // Ponape
    {    5, 1534,       0,   17726,       0,  0, 13,  0, }, // Pyongyang
    {    5, 1554,       0,   17739,       0,  0, 11,  0, }, // Reunion
    {    5, 1562,       0,   17750,       0,  0, 11,  0, }, // Rothera
    {    5, 1570,   17761,   17773,   17794, 12, 21, 17, }, // Sakhalin
    {    5, 1579,   17811,   17821,   17840, 10, 19, 16, }, // Samara
    {    5, 1596,   17856,   17865,   17883,  9, 18, 15, }, // Samoa
    {    5, 1602,       0,   17898,       0,  0, 13,  0, }, // Seychelles
    {    5, 1624,       0,   17911,       0,  0, 22,  0, }, // Singapore
    {    5, 1634,       0,   17933,       0,  0, 19,  0, }, // Solomon
    {    5, 1642,       0,   17952,       0,  0, 16,  0, }, // South_Georgia
    {    5, 1656,       0,   17968,       0,  0, 12,  0, }, // Suriname
    {    5, 1676,       0,   17980,       0,  0,  9,  0, }, // Syowa
    {    5, 1682,       0,   17989,       0,  0, 10,  0, }, // Tahiti
    {    5, 1689,   17999,   18009,   18028, 10, 19, 16, }, // Taipei
    {    5, 1696,       0,   18044,       0,  0, 15,  0, }, // Tajikistan
    {    5, 1724,       0,   18059,       0,  0, 11,  0, }, // Tokelau
    {    5, 1732,   18070,   18079,   18097,  9, 18, 14, }, // Tonga
    {    5, 1738,       0,   18111,       0,  0,  9,  0, }, // Truk
    {    5, 1750,   18120,   18136,   18161, 16, 25, 21, }, // Turkmenistan
    {    5, 1763,       0,   18182,       0,  0, 10,  0, }, // Tuvalu
    {    5, 1777,   18192,   18203,   18223, 11, 20, 16, }, // Uruguay
    {    5, 1792,   18239,   18254,   18278, 15, 24, 20, }, // Uzbekistan
    {    5, 1803,   18298,   18309,   18329, 11, 20, 16, }, // Vanuatu
    {    5, 1811,       0,   18345,       0,  0, 13,  0, }, // Venezuela
    {    5, 1821,   18358,   18373,   18397, 15, 24, 20, }, // Vladivostok
    {    5, 1833,   18417,   18430,   18452, 13, 22, 18, }, // Volgograd
    {    5, 1843,       0,   18470,       0,  0, 10,  0, }, // Vostok
    {    5, 1850,       0,   18480,       0,  0, 15,  0, }, // Wake
    {    5, 1855,       0,   18495,       0,  0, 20,  0, }, // Wallis
    {    5, 1862,   18515,   18527,   18548, 12, 21, 17, }, // Yakutsk
    {    5, 1870,   18565,   18582,   18608, 17, 26, 22, }, // Yekaterinburg
    {    5, 1892,       0,   18630,       0,  0,  8,  0, }, // Yukon
    {    8,    5,       0,   18638,       0,  0, 15,  0, }, // Afghanistan Akan/Latin/Ghana
    {    8,   17,       0,   18653,       0,  0, 20,  0, }, // Africa_Central
    {    8,   32,       0,   18673,       0,  0, 18,  0, }, // Africa_Eastern
    {    8,   65,       0,   18691,       0,  0, 27,  0, }, // Africa_Southern
    {    8,   81,   18718,   18735,   18760, 17, 25, 22, }, // Africa_Western
    {    8,  107,   18782,   18793,   18812, 11, 19, 16, }, // Alaska
    {    8,  135,   18828,   18839,   18858, 11, 19, 16, }, // Amazon
    {    8,  142,   18874,   18889,   18912, 15, 23, 20, }, // America_Central
    {    8,  158,   18680,   18932,   18951, 11, 19, 16, }, // America_Eastern
    {    8,  174,   18967,   18976,   18993,  9, 17, 14, }, // America_Mountain
    {    8,  191,   19007,   19019,   19039, 12, 20, 17, }, // America_Pacific
    {    8,  214,   19056,   19065,   19082,  9, 17, 14, }, // Apia
    {    8,  232,   19096,   19107,   19126, 11, 19, 16, }, // Arabian
    {    8,  240,   19142,   19156,   19178, 14, 22, 19, }, // Argentina
    {    8,  250,   19197,   19218,   19247, 21, 29, 26, }, // Argentina_Western
    {    8,  268,   19273,   19285,   19305, 12, 20, 17, }, // Armenia
    {    8,  286,   19322,   19335,   19356, 13, 21, 18, }, // Atlantic
    {    8,  295,   19374,   19398,   19430, 24, 32, 29, }, // Australia_Central
    {    8,  313,   19459,   19490,   19529, 31, 39, 36, }, // Australia_CentralWestern
    {    8,  338,   19565,   19585,   19613, 20, 28, 25, }, // Australia_Eastern
    {    8,  356,   19638,   19658,   19686, 20, 28, 25, }, // Australia_Western
    {    8,  374,   19711,   19725,   19747, 14, 22, 19, }, // Azerbaijan
    {    8,  385,   19766,   19777,   19796, 11, 19, 16, }, // Azores
    {    8,  397,   19812,   19828,   19852, 16, 24, 21, }, // Bangladesh
    {    8,  415,       0,   19873,       0,  0, 10,  0, }, // Bhutan
    {    8,  422,       0,   19883,       0,  0, 12,  0, }, // Bolivia
    {    8,  437,   19895,   19908,   19929, 13, 21, 18, }, // Brasilia
    {    8,  454,       0,   19947,       0,  0, 21,  0, }, // Brunei
    {    8,  461,   19968,   19980,   20000, 12, 20, 17, }, // Cape_Verde
    {    8,  478,       0,   20017,       0,  0, 20,  0, }, // Chamorro
    {    8,  487,   20037,   20048,   20067, 11, 19, 16, }, // Chatham
    {    8,  495,   20083,   20093,   20111, 10, 18, 15, }, // Chile
    {    8,  501,   20126,   20137,   20156, 11, 19, 16, }, // China
    {    8,  507,       0,   20172,       0,  0, 19,  0, }, // Christmas
    {    8,  517,       0,   20191,       0,  0, 18,  0, }, // Cocos
    {    8,  523,   20209,   20222,   20243, 13, 21, 18, }, // Colombia
    {    8,  532,   20261,   20276,   20299, 15, 23, 23, }, // Cook
    {    8,  537,   20322,   20331,   20348,  9, 17, 14, }, // Cuba
    {    8,  548,       0,   20362,       0,  0, 10,  0, }, // Davis
    {    8,  564,       0,   20372,       0,  0, 21,  0, }, // DumontDUrville
    {    8,  601,       0,   20393,       0,  0, 17,  0, }, // East_Timor
    {    8,  612,   20410,   20427,   20452, 17, 25, 22, }, // Easter
    {    8,  619,       0,   20474,       0,  0, 13,  0, }, // Ecuador
    {    8,  627,   20487,   20509,   20539, 22, 30, 27, }, // Europe_Central
    {    8,  642,   20566,   20584,   20610, 18, 26, 23, }, // Europe_Eastern
    {    8,  657,       0,   20633,       0,  0, 24,  0, }, // Europe_Further_Eastern
    {    8,  680,   20657,   20675,   20701, 18, 26, 23, }, // Europe_Western
    {    8,  695,   20724,   20742,   20768, 18, 26, 23, }, // Falkland
    {    8,  704,   20791,   20801,   20819, 10, 18, 15, }, // Fiji
    {    8,  709,       0,   20834,       0,  0, 19,  0, }, // French_Guiana
    {    8,  723,       0,   20853,       0,  0, 32,  0, }, // French_Southern
    {    8,  746,       0,   20885,       0,  0, 14,  0, }, // Galapagos
    {    8,  756,       0,   20899,       0,  0, 12,  0, }, // Gambier
    {    8,  764,   20911,   20923,   20943, 12, 20, 17, }, // Georgia
    {    8,  772,       0,   20960,       0,  0, 18,  0, }, // Gilbert_Islands
    {    8,  788,       0,   20978,       0,  0, 19,  0, }, // GMT
    {    8,  830,   20997,   21018,   21047, 21, 29, 26, }, // Greenland_Eastern
    {    8,  848,   21073,   21093,   21121, 20, 28, 25, }, // Greenland_Western
    {    8,  871,       0,   21146,       0,  0, 18,  0, }, // Gulf
    {    8,  876,       0,   20842,       0,  0, 11,  0, }, // Guyana
    {    8,  883,   21164,   21184,   21212, 20, 28, 25, }, // Hawaii_Aleutian
    {    8,  899,   21237,   21248,   21267, 11, 19, 16, }, // Hong_Kong
    {    8,  909,   21283,   21292,   21309,  9, 17, 14, }, // Hovd
    {    8,  914,       0,   21323,       0,  0, 18,  0, }, // India
    {    8,  920,       0,   21341,       0,  0, 13,  0, }, // Indian_Ocean
    {    8,  933,       0,   21354,       0,  0, 14,  0, }, // Indochina
    {    8,  943,       0,   21368,       0,  0, 26,  0, }, // Indonesia_Central
    {    8,  961,       0,   21394,       0,  0, 22,  0, }, // Indonesia_Eastern
    {    8,  979,       0,   21416,       0,  0, 22,  0, }, // Indonesia_Western
    {    8,  997,   21438,   21447,   21464,  9, 17, 14, }, // Iran
    {    8, 1008,   21478,   21490,   21510, 12, 20, 17, }, // Irkutsk
    {    8, 1016,   21527,   21538,   21557, 11, 19, 16, }, // Israel
    {    8, 1023,   21573,   21584,   21603, 11, 19, 16, }, // Japan
    {    8, 1047,       0,   21619,       0,  0, 14,  0, }, // Kazakhstan
    {    8, 1058,       0,   21633,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {    8, 1077,       0,   21654,       0,  0, 20,  0, }, // Kazakhstan_Western
    {    8, 1106,   21674,   21685,   21704, 11, 19, 16, }, // Korea
    {    8, 1112,       0,   21720,       0,  0, 11,  0, }, // Kosrae
    {    8, 1119,   21731,   21747,   21771, 16, 24, 21, }, // Krasnoyarsk
    {    8, 1151,       0,   21792,       0,  0, 14,  0, }, // Kyrgystan
    {    8, 1175,       0,   21806,       0,  0, 15,  0, }, // Line_Islands
    {    8, 1188,   21821,   21834,   21855, 13, 21, 18, }, // Lord_Howe
    {    8, 1204,   21873,   21885,   21905, 12, 20, 17, }, // Magadan
    {    8, 1219,       0,   21922,       0,  0, 13,  0, }, // Malaysia
    {    8, 1228,       0,   21935,       0,  0, 13,  0, }, // Maldives
    {    8, 1237,       0,   21948,       0,  0, 13,  0, }, // Marquesas
    {    8, 1247,       0,   21961,       0,  0, 18,  0, }, // Marshall_Islands
    {    8, 1264,   21979,   21994,   22017, 15, 23, 20, }, // Mauritius
    {    8, 1274,       0,   22037,       0,  0, 11,  0, }, // Mawson
    {    8, 1281,   22048,   22068,   22096, 20, 28, 25, }, // Mexico_Pacific
    {    8, 1296,   22121,   22135,   22157, 14, 22, 19, }, // Mongolia
    {    8, 1305,   22176,   22186,   22204, 10, 18, 15, }, // Moscow
    {    8, 1312,       0,   22219,       0,  0, 12,  0, }, // Myanmar
    {    8, 1320,       0,   22231,       0,  0, 10,  0, }, // Nauru
    {    8, 1326,       0,   22241,       0,  0, 10,  0, }, // Nepal
    {    8, 1332,   22251,   22272,   22301, 21, 29, 26, }, // New_Caledonia
    {    8, 1346,   22327,   22345,   22371, 18, 26, 23, }, // New_Zealand
    {    8, 1358,   22394,   22411,   22436, 17, 25, 22, }, // Newfoundland
    {    8, 1371,       0,   22458,       0,  0,  9,  0, }, // Niue
    {    8, 1376,   22467,   22485,   22511, 18, 26, 23, }, // Norfolk
    {    8, 1384,   22534,   22558,   22590, 24, 32, 29, }, // Noronha
    {    8, 1406,   22619,   22635,   22659, 16, 24, 21, }, // Novosibirsk
    {    8, 1418,   22680,   22689,   22706,  9, 17, 14, }, // Omsk
    {    8, 1428,   22720,   22733,   22754, 13, 21, 18, }, // Pakistan
    {    8, 1437,       0,   22772,       0,  0, 10,  0, }, // Palau
    {    8, 1443,       0,   22782,       0,  0, 22,  0, }, // Papua_New_Guinea
    {    8, 1460,   22804,   22817,   22838, 13, 21, 18, }, // Paraguay
    {    8, 1469,   22856,   22865,   22882,  9, 17, 14, }, // Peru
    {    8, 1474,   22896,   22908,   22928, 12, 20, 17, }, // Philippines
    {    8, 1486,       0,   22945,       0,  0, 18,  0, }, // Phoenix_Islands
    {    8, 1502,   22963,   22989,   23023, 26, 34, 31, }, // Pierre_Miquelon
    {    8, 1518,       0,   23054,       0,  0, 13,  0, }, // Pitcairn
    {    8, 1527,       0,   23067,       0,  0, 11,  0, }, // Ponape
    {    8, 1534,       0,   23078,       0,  0, 14,  0, }, // Pyongyang
    {    8, 1554,       0,   23092,       0,  0, 12,  0, }, // Reunion
    {    8, 1562,       0,   23104,       0,  0, 11,  0, }, // Rothera
    {    8, 1570,   23115,   23128,   23149, 13, 21, 18, }, // Sakhalin
    {    8, 1596,   23167,   23177,   23195, 10, 18, 15, }, // Samoa
    {    8, 1602,       0,   23210,       0,  0, 12,  0, }, // Seychelles
    {    8, 1624,       0,   23222,       0,  0, 20,  0, }, // Singapore
    {    8, 1634,       0,   23242,       0,  0, 19,  0, }, // Solomon
    {    8, 1642,       0,   23261,       0,  0, 21,  0, }, // South_Georgia
    {    8, 1656,       0,   23282,       0,  0, 13,  0, }, // Suriname
    {    8, 1676,       0,   23295,       0,  0, 10,  0, }, // Syowa
    {    8, 1682,       0,   23305,       0,  0, 11,  0, }, // Tahiti
    {    8, 1689,   23316,   23327,   23346, 11, 19, 16, }, // Taipei
    {    8, 1696,       0,   23362,       0,  0, 16,  0, }, // Tajikistan
    {    8, 1724,       0,   23378,       0,  0, 12,  0, }, // Tokelau
    {    8, 1732,   23390,   23400,   23418, 10, 18, 15, }, // Tonga
    {    8, 1738,       0,   23433,       0,  0, 10,  0, }, // Truk
    {    8, 1750,   23443,   23459,   23483, 16, 24, 21, }, // Turkmenistan
    {    8, 1763,       0,   23504,       0,  0, 11,  0, }, // Tuvalu
    {    8, 1777,   23515,   23528,   23549, 13, 21, 18, }, // Uruguay
    {    8, 1792,   23567,   23582,   23605, 15, 23, 20, }, // Uzbekistan
    {    8, 1803,   23625,   23637,   23657, 12, 20, 17, }, // Vanuatu
    {    8, 1811,       0,   23674,       0,  0, 14,  0, }, // Venezuela
    {    8, 1821,   23688,   23704,   23728, 16, 24, 21, }, // Vladivostok
    {    8, 1833,   23749,   23763,   23785, 14, 22, 19, }, // Volgograd
    {    8, 1843,       0,   23804,       0,  0, 11,  0, }, // Vostok
    {    8, 1850,       0,   23815,       0,  0, 16,  0, }, // Wake
    {    8, 1855,       0,   23831,       0,  0, 21,  0, }, // Wallis
    {    8, 1862,   23852,   23864,   23884, 12, 20, 17, }, // Yakutsk
    {    8, 1870,   23901,   23918,   23943, 17, 25, 22, }, // Yekaterinburg
    {    8, 1892,       0,   23965,       0,  0, 10,  0, }, // Yukon
    {   10,    0,   23975,   23993,   24021, 18, 28, 25, }, // Acre Albanian/Latin/Albania
    {   10,    5,       0,   24046,       0,  0, 18,  0, }, // Afghanistan
    {   10,   17,       0,   24064,       0,  0, 22,  0, }, // Africa_Central
    {   10,   32,       0,   24086,       0,  0, 21,  0, }, // Africa_Eastern
    {   10,   65,       0,   24107,       0,  0, 30,  0, }, // Africa_Southern
    {   10,   81,   24137,   24162,   24197, 25, 35, 32, }, // Africa_Western
    {   10,  107,   24229,   24242,   24265, 13, 23, 20, }, // Alaska
    {   10,  128,   24285,   24298,   24321, 13, 23, 20, }, // Almaty
    {   10,  135,   24341,   24355,   24379, 14, 24, 21, }, // Amazon
    {   10,  142,   24400,   24422,   24454, 22, 32, 29, }, // America_Central
    {   10,  158,   24483,   24504,   24535, 21, 31, 28, }, // America_Eastern
    {   10,  174,   24563,   24606,   24659, 43, 53, 50, }, // America_Mountain
    {   10,  191,   24709,   24759,   24819, 50, 60, 57, }, // America_Pacific
    {   10,  207,   24876,   24890,   24914, 14, 24, 21, }, // Anadyr
    {   10,  214,   24935,   24946,   24967, 11, 21, 18, }, // Apia
    {   10,  219,   24985,   24997,   25019, 12, 22, 19, }, // Aqtau
    {   10,  225,   25038,   25051,   25074, 13, 23, 20, }, // Aqtobe
    {   10,  232,   25094,   25103,   25122,  9, 19, 16, }, // Arabian
    {   10,  240,   25138,   25155,   25182, 17, 27, 24, }, // Argentina
    {   10,  250,   25206,   25235,   25274, 29, 39, 36, }, // Argentina_Western
    {   10,  268,   25310,   25324,   25348, 14, 24, 21, }, // Armenia
    {   10,  286,   25369,   25385,   25411, 16, 26, 23, }, // Atlantic
    {   10,  295,   25434,   25459,   25494, 25, 35, 32, }, // Australia_Central
    {   10,  313,   25526,   25563,   25610, 37, 47, 44, }, // Australia_CentralWestern
    {   10,  338,   25654,   25678,   25712, 24, 34, 31, }, // Australia_Eastern
    {   10,  356,   25743,   25771,   25809, 28, 38, 35, }, // Australia_Western
    {   10,  374,   25844,   25863,   25892, 19, 29, 26, }, // Azerbaijan
    {   10,  385,   25918,   25931,   25954, 13, 23, 20, }, // Azores
    {   10,  397,   25974,   25992,   26020, 18, 28, 25, }, // Bangladesh
    {   10,  415,       0,   26045,       0,  0, 13,  0, }, // Bhutan
    {   10,  422,       0,   26058,       0,  0, 14,  0, }, // Bolivia
    {   10,  437,   26072,   26087,   26112, 15, 25, 22, }, // Brasilia
    {   10,  454,       0,   26134,       0,  0, 24,  0, }, // Brunei
    {   10,  461,   26158,   26180,   26212, 22, 32, 29, }, // Cape_Verde
    {   10,  472,       0,   26241,       0,  0, 12,  0, }, // Casey
    {   10,  478,       0,   26253,       0,  0, 14,  0, }, // Chamorro
    {   10,  487,   26267,   26280,   26303, 13, 23, 20, }, // Chatham
    {   10,  495,   26323,   26334,   26355, 11, 21, 18, }, // Chile
    {   10,  501,   26373,   26384,   26405, 11, 21, 18, }, // China
    {   10,  507,       0,   26423,       0,  0, 32,  0, }, // Christmas
    {   10,  517,       0,   26455,       0,  0, 19,  0, }, // Cocos
    {   10,  523,   26474,   26489,   26514, 15, 25, 22, }, // Colombia
    {   10,  532,   26536,   26553,   26580, 17, 27, 24, }, // Cook
    {   10,  537,   26604,   26615,   26636, 11, 21, 18, }, // Cuba
    {   10,  548,       0,   26654,       0,  0, 14,  0, }, // Davis
    {   10,  564,       0,   26668,       0,  0, 22,  0, }, // DumontDUrville
    {   10,  601,       0,   26690,       0,  0, 20,  0, }, // East_Timor
    {   10,  612,   26710,   26735,   26770, 25, 35, 32, }, // Easter
    {   10,  619,       0,   26802,       0,  0, 15,  0, }, // Ecuador
    {   10,  627,   26817,   26839,   26871, 22, 32, 29, }, // Europe_Central
    {   10,  642,   26900,   26921,   26952, 21, 31, 28, }, // Europe_Eastern
    {   10,  657,       0,   26980,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {   10,  680,   27011,   27036,   27071, 25, 35, 32, }, // Europe_Western
    {   10,  695,   27103,   27125,   27157, 22, 32, 29, }, // Falkland
    {   10,  704,   27186,   27198,   27220, 12, 22, 19, }, // Fiji
    {   10,  709,       0,   27239,       0,  0, 23,  0, }, // French_Guiana
    {   10,  723,       0,   27262,       0,  0, 48,  0, }, // French_Southern
    {   10,  746,       0,   27310,       0,  0, 17,  0, }, // Galapagos
    {   10,  756,       0,   27327,       0,  0, 15,  0, }, // Gambier
    {   10,  764,   27342,   27358,   27384, 16, 26, 23, }, // Georgia
    {   10,  772,       0,   27407,       0,  0, 21,  0, }, // Gilbert_Islands
    {   10,  788,       0,   27428,       0,  0, 15,  0, }, // GMT
    {   10,  830,   27443,   27467,   27501, 24, 34, 31, }, // Greenland_Eastern
    {   10,  848,   27532,   27560,   27598, 28, 38, 35, }, // Greenland_Western
    {   10,  866,       0,   27633,       0,  0, 12,  0, }, // Guam
    {   10,  871,       0,   27645,       0,  0, 12,  0, }, // Gulf
    {   10,  876,       0,   27239,       0,  0, 14,  0, }, // Guyana
    {   10,  883,   27657,   27685,   27723, 28, 38, 35, }, // Hawaii_Aleutian
    {   10,  899,   27758,   27775,   27802, 17, 27, 24, }, // Hong_Kong
    {   10,  909,   27826,   27838,   27860, 12, 22, 19, }, // Hovd
    {   10,  914,       0,   27879,       0,  0, 22,  0, }, // India
    {   10,  920,       0,   27901,       0,  0, 20,  0, }, // Indian_Ocean
    {   10,  933,       0,   27921,       0,  0, 15,  0, }, // Indochina
    {   10,  943,       0,   27936,       0,  0, 25,  0, }, // Indonesia_Central
    {   10,  961,       0,   27961,       0,  0, 24,  0, }, // Indonesia_Eastern
    {   10,  979,       0,   27985,       0,  0, 28,  0, }, // Indonesia_Western
    {   10,  997,   28013,   28025,   28047, 12, 22, 19, }, // Iran
    {   10, 1008,   28066,   28081,   28106, 15, 25, 22, }, // Irkutsk
    {   10, 1016,   28128,   28142,   28166, 14, 24, 21, }, // Israel
    {   10, 1023,   28187,   28201,   28225, 14, 24, 21, }, // Japan
    {   10, 1029,   28246,   28276,   28316, 30, 40, 37, }, // Kamchatka
    {   10, 1047,       0,   28353,       0,  0, 18,  0, }, // Kazakhstan
    {   10, 1058,       0,   28371,       0,  0, 25,  0, }, // Kazakhstan_Eastern
    {   10, 1077,       0,   28396,       0,  0, 29,  0, }, // Kazakhstan_Western
    {   10, 1106,   28425,   28436,   28457, 11, 21, 18, }, // Korea
    {   10, 1112,       0,   28475,       0,  0, 12,  0, }, // Kosrae
    {   10, 1119,   28487,   28506,   28535, 19, 29, 26, }, // Krasnoyarsk
    {   10, 1151,       0,   28561,       0,  0, 15,  0, }, // Kyrgystan
    {   10, 1161,       0,   28576,       0,  0, 13,  0, }, // Lanka
    {   10, 1175,       0,   28589,       0,  0, 35,  0, }, // Line_Islands
    {   10, 1188,   28624,   28640,   28666, 16, 26, 23, }, // Lord_Howe
    {   10, 1198,   28689,   28701,   28723, 12, 22, 19, }, // Macau
    {   10, 1204,   28742,   28757,   28782, 15, 25, 22, }, // Magadan
    {   10, 1219,       0,   28804,       0,  0, 15,  0, }, // Malaysia
    {   10, 1228,       0,   28819,       0,  0, 15,  0, }, // Maldives
    {   10, 1237,       0,   28834,       0,  0, 21,  0, }, // Marquesas
    {   10, 1247,       0,   28855,       0,  0, 22,  0, }, // Marshall_Islands
    {   10, 1264,   28877,   28894,   28921, 17, 27, 24, }, // Mauritius
    {   10, 1274,       0,   28945,       0,  0, 14,  0, }, // Mawson
    {   10, 1281,   28959,   29009,   29069, 50, 60, 57, }, // Mexico_Pacific
    {   10, 1296,   29126,   29144,   29172, 18, 28, 25, }, // Mongolia
    {   10, 1305,   29197,   29209,   29231, 12, 22, 19, }, // Moscow
    {   10, 1312,       0,   29250,       0,  0, 15,  0, }, // Myanmar
    {   10, 1320,       0,   29265,       0,  0, 13,  0, }, // Nauru
    {   10, 1326,       0,   29278,       0,  0, 13,  0, }, // Nepal
    {   10, 1332,   29291,   29313,   29345, 22, 32, 29, }, // New_Caledonia
    {   10, 1346,   29374,   29394,   29424, 20, 30, 27, }, // New_Zealand
    {   10, 1358,   29451,   29485,   29529, 34, 44, 41, }, // Newfoundland
    {   10, 1371,       0,   29570,       0,  0, 12,  0, }, // Niue
    {   10, 1376,   29582,   29604,   29636, 22, 32, 29, }, // Norfolk
    {   10, 1384,   29665,   29691,   29727, 26, 36, 33, }, // Noronha
    {   10, 1392,       0,   29760,       0,  0, 35,  0, }, // North_Mariana
    {   10, 1406,   29795,   29814,   29843, 19, 29, 26, }, // Novosibirsk
    {   10, 1418,   29869,   29881,   29903, 12, 22, 19, }, // Omsk
    {   10, 1428,   29922,   29938,   29964, 16, 26, 23, }, // Pakistan
    {   10, 1437,       0,   29987,       0,  0, 12,  0, }, // Palau
    {   10, 1443,       0,   29999,       0,  0, 25,  0, }, // Papua_New_Guinea
    {   10, 1460,   30024,   30039,   30064, 15, 25, 22, }, // Paraguay
    {   10, 1469,   30086,   30098,   30120, 12, 22, 19, }, // Peru
    {   10, 1474,   30139,   30155,   30181, 16, 26, 23, }, // Philippines
    {   10, 1486,       0,   30204,       0,  0, 20,  0, }, // Phoenix_Islands
    {   10, 1502,   30224,   30251,   30288, 27, 37, 34, }, // Pierre_Miquelon
    {   10, 1518,       0,   30322,       0,  0, 15,  0, }, // Pitcairn
    {   10, 1527,       0,   30337,       0,  0, 14,  0, }, // Ponape
    {   10, 1534,       0,   30351,       0,  0, 14,  0, }, // Pyongyang
    {   10, 1544,   30365,   30381,   30407, 16, 26, 23, }, // Qyzylorda
    {   10, 1554,       0,   30430,       0,  0, 15,  0, }, // Reunion
    {   10, 1562,       0,   30445,       0,  0, 14,  0, }, // Rothera
    {   10, 1570,   30459,   30474,   30499, 15, 25, 22, }, // Sakhalin
    {   10, 1579,   30521,   30534,   30557, 13, 23, 20, }, // Samara
    {   10, 1596,   30577,   30589,   30611, 12, 22, 19, }, // Samoa
    {   10, 1602,       0,   30630,       0,  0, 17,  0, }, // Seychelles
    {   10, 1624,       0,   30647,       0,  0, 16,  0, }, // Singapore
    {   10, 1634,       0,   30663,       0,  0, 21,  0, }, // Solomon
    {   10, 1642,       0,   30684,       0,  0, 23,  0, }, // South_Georgia
    {   10, 1656,       0,   30707,       0,  0, 15,  0, }, // Suriname
    {   10, 1676,       0,   30722,       0,  0, 12,  0, }, // Syowa
    {   10, 1682,       0,   30734,       0,  0, 13,  0, }, // Tahiti
    {   10, 1689,   30747,   30760,   30783, 13, 23, 20, }, // Taipei
    {   10, 1696,       0,   30803,       0,  0, 19,  0, }, // Tajikistan
    {   10, 1724,       0,   30822,       0,  0, 14,  0, }, // Tokelau
    {   10, 1732,   30836,   30848,   30870, 12, 22, 19, }, // Tonga
    {   10, 1738,       0,   30889,       0,  0, 11,  0, }, // Truk
    {   10, 1750,   30900,   30920,   30950, 20, 30, 27, }, // Turkmenistan
    {   10, 1763,       0,   30977,       0,  0, 14,  0, }, // Tuvalu
    {   10, 1777,   30991,   31005,   31029, 14, 24, 21, }, // Uruguay
    {   10, 1792,   31050,   31068,   31096, 18, 28, 25, }, // Uzbekistan
    {   10, 1803,   31121,   31136,   31161, 15, 25, 22, }, // Vanuatu
    {   10, 1811,       0,   31183,       0,  0, 16,  0, }, // Venezuela
    {   10, 1821,   31199,   31218,   31247, 19, 29, 26, }, // Vladivostok
    {   10, 1833,   31273,   31290,   31317, 17, 27, 24, }, // Volgograd
    {   10, 1843,       0,   31341,       0,  0, 14,  0, }, // Vostok
    {   10, 1850,       0,   31355,       0,  0, 19,  0, }, // Wake
    {   10, 1855,       0,   31374,       0,  0, 26,  0, }, // Wallis
    {   10, 1862,   31400,   31415,   31440, 15, 25, 22, }, // Yakutsk
    {   10, 1870,   31462,   31482,   31512, 20, 30, 27, }, // Yekaterinburg
    {   10, 1892,       0,   31539,       0,  0, 13,  0, }, // Yukon
    {   13,    5,       0,   31552,       0,  0, 12,  0, }, // Afghanistan Amharic/Ethiopic/Ethiopia
    {   13,   17,       0,   31564,       0,  0, 16,  0, }, // Africa_Central
    {   13,   32,       0,   31580,       0,  0, 14,  0, }, // Africa_Eastern
    {   13,   65,       0,   31594,       0,  0, 18,  0, }, // Africa_Southern
    {   13,   81,   31612,   31626,   31645, 14, 19, 19, }, // Africa_Western
    {   13,  107,   31664,   31679,   31700, 15, 21, 19, }, // Alaska
    {   13,  135,   31719,   31734,   31754, 15, 20, 19, }, // Amazon
    {   13,  142,   31773,   31799,   31830, 26, 31, 20, }, // America_Central
    {   13,  158,   31850,   31865,   31885, 15, 20, 19, }, // America_Eastern
    {   13,  174,   31904,   31919,   31939, 15, 20, 17, }, // America_Mountain
    {   13,  191,   31956,   31971,   31991, 15, 20, 19, }, // America_Pacific
    {   13,  207,   32010,   32026,   32042, 16, 16, 21, }, // Anadyr
    {   13,  214,   32063,   32071,   32084,  8, 13, 15, }, // Apia
    {   13,  232,   32099,   32108,   32122,  9, 14, 18, }, // Arabian
    {   13,  240,   32140,   32158,   32180, 18, 22, 21, }, // Argentina
    {   13,  250,   32201,   32224,   32252, 23, 28, 27, }, // Argentina_Western
    {   13,  268,   32279,   32289,   32304, 10, 15, 15, }, // Armenia
    {   13,  286,   32319,   32337,   32360, 18, 23, 21, }, // Atlantic
    {   13,  295,   32381,   32406,   32436, 25, 30, 28, }, // Australia_Central
    {   13,  313,   32464,   32494,   32529, 30, 35, 34, }, // Australia_CentralWestern
    {   13,  338,   32563,   32588,   32618, 25, 30, 28, }, // Australia_Eastern
    {   13,  356,   32646,   32670,   32699, 24, 29, 28, }, // Australia_Western
    {   13,  374,   32727,   32738,   32754, 11, 16, 16, }, // Azerbaijan
    {   13,  385,   32770,   32779,   32793,  9, 14, 14, }, // Azores
    {   13,  397,   32807,   32818,   32834, 11, 16, 16, }, // Bangladesh
    {   13,  415,       0,   32850,       0,  0,  8,  0, }, // Bhutan
    {   13,  422,       0,   32858,       0,  0,  9,  0, }, // Bolivia
    {   13,  437,   32867,   32883,   32904, 16, 21, 19, }, // Brasilia
    {   13,  454,       0,   32923,       0,  0, 15,  0, }, // Brunei
    {   13,  461,   32938,   32949,   32965, 11, 16, 16, }, // Cape_Verde
    {   13,  478,       0,   32981,       0,  0, 13,  0, }, // Chamorro
    {   13,  487,   32994,   33002,   33015,  8, 13, 17, }, // Chatham
    {   13,  495,   33032,   33039,   33051,  7, 12, 12, }, // Chile
    {   13,  501,   33063,   33071,   33084,  8, 13, 17, }, // China
    {   13,  507,       0,   33101,       0,  0, 11,  0, }, // Christmas
    {   13,  517,       0,   33112,       0,  0, 13,  0, }, // Cocos
    {   13,  523,   33125,   33135,   33150, 10, 15, 15, }, // Colombia
    {   13,  532,   33165,   33177,   33194, 12, 17, 21, }, // Cook
    {   13,  537,   33215,   33221,   33233,  6, 12, 16, }, // Cuba
    {   13,  548,       0,   33249,       0,  0,  8,  0, }, // Davis
    {   13,  564,       0,   33257,       0,  0, 14,  0, }, // DumontDUrville
    {   13,  601,       0,   33271,       0,  0, 13,  0, }, // East_Timor
    {   13,  612,   33284,   33297,   33315, 13, 18, 18, }, // Easter
    {   13,  619,       0,   33333,       0,  0,  9,  0, }, // Ecuador
    {   13,  627,   33342,   33358,   33379, 16, 21, 21, }, // Europe_Central
    {   13,  642,   33400,   33415,   33435, 15, 20, 20, }, // Europe_Eastern
    {   13,  657,       0,   33455,       0,  0, 18,  0, }, // Europe_Further_Eastern
    {   13,  680,   33473,   33488,   33508, 15, 20, 20, }, // Europe_Western
    {   13,  695,   33528,   33544,   33565, 16, 21, 21, }, // Falkland
    {   13,  704,   33586,   33593,   33605,  7, 12, 12, }, // Fiji
    {   13,  709,       0,   33617,       0,  0, 14,  0, }, // French_Guiana
    {   13,  723,       0,   33631,       0,  0, 26,  0, }, // French_Southern
    {   13,  746,       0,   33657,       0,  0, 10,  0, }, // Galapagos
    {   13,  756,       0,   33667,       0,  0, 10,  0, }, // Gambier
    {   13,  764,   33677,   33687,   33702, 10, 15, 15, }, // Georgia
    {   13,  772,       0,   33717,       0,  0, 15,  0, }, // Gilbert_Islands
    {   13,  788,       0,   33732,       0,  0, 15,  0, }, // GMT
    {   13,  830,   33747,   33763,   33784, 16, 21, 21, }, // Greenland_Eastern
    {   13,  848,   33805,   33821,   33842, 16, 21, 21, }, // Greenland_Western
    {   13,  871,       0,   33863,       0,  0, 16,  0, }, // Gulf
    {   13,  876,       0,   33879,       0,  0,  8,  0, }, // Guyana
    {   13,  883,   33887,   33906,   33930, 19, 24, 23, }, // Hawaii_Aleutian
    {   13,  899,   33953,   33965,   33982, 12, 17, 17, }, // Hong_Kong
    {   13,  909,   33999,   34013,   34033, 14, 20, 18, }, // Hovd
    {   13,  914,       0,   34051,       0,  0, 13,  0, }, // India
    {   13,  920,       0,   34064,       0,  0, 14,  0, }, // Indian_Ocean
    {   13,  933,       0,   34078,       0,  0, 11,  0, }, // Indochina
    {   13,  943,       0,   34089,       0,  0, 18,  0, }, // Indonesia_Central
    {   13,  961,       0,   34107,       0,  0, 17,  0, }, // Indonesia_Eastern
    {   13,  979,       0,   34124,       0,  0, 17,  0, }, // Indonesia_Western
    {   13,  997,   34141,   34149,   34162,  8, 13, 17, }, // Iran
    {   13, 1008,   34179,   34196,   34219, 17, 23, 21, }, // Irkutsk
    {   13, 1016,   34240,   34250,   34265, 10, 15, 19, }, // Israel
    {   13, 1023,   34284,   34292,   34305,  8, 13, 17, }, // Japan
    {   13, 1029,   34322,   34338,   34366, 16, 28, 32, }, // Kamchatka
    {   13, 1047,       0,   34398,       0,  0, 10,  0, }, // Kazakhstan
    {   13, 1058,       0,   34408,       0,  0, 16,  0, }, // Kazakhstan_Eastern
    {   13, 1077,       0,   34424,       0,  0, 16,  0, }, // Kazakhstan_Western
    {   13, 1106,   34440,   34448,   34461,  8, 13, 17, }, // Korea
    {   13, 1112,       0,   34478,       0,  0,  9,  0, }, // Kosrae
    {   13, 1119,   34487,   34506,   34531, 19, 25, 23, }, // Krasnoyarsk
    {   13, 1151,       0,   34554,       0,  0, 11,  0, }, // Kyrgystan
    {   13, 1175,       0,   34565,       0,  0, 13,  0, }, // Line_Islands
    {   13, 1188,   34578,   34596,   34619, 18, 23, 21, }, // Lord_Howe
    {   13, 1204,   34640,   34656,   34676, 16, 20, 18, }, // Magadan
    {   13, 1219,       0,   34694,       0,  0, 10,  0, }, // Malaysia
    {   13, 1228,       0,   34704,       0,  0, 10,  0, }, // Maldives
    {   13, 1237,       0,   34714,       0,  0, 10,  0, }, // Marquesas
    {   13, 1247,       0,   34724,       0,  0, 14,  0, }, // Marshall_Islands
    {   13, 1264,   34738,   34749,   34765, 11, 16, 16, }, // Mauritius
    {   13, 1274,       0,   34781,       0,  0,  9,  0, }, // Mawson
    {   13, 1281,   34790,   34810,   34835, 20, 25, 24, }, // Mexico_Pacific
    {   13, 1296,   34859,   34870,   34893, 11, 23, 22, }, // Mongolia
    {   13, 1305,   34915,   34929,   34948, 14, 19, 18, }, // Moscow
    {   13, 1312,       0,   34966,       0,  0, 10,  0, }, // Myanmar
    {   13, 1320,       0,   34976,       0,  0,  8,  0, }, // Nauru
    {   13, 1326,       0,   34984,       0,  0,  8,  0, }, // Nepal
    {   13, 1332,   34992,   35005,   35023, 13, 18, 18, }, // New_Caledonia
    {   13, 1346,   35041,   35053,   35070, 12, 17, 21, }, // New_Zealand
    {   13, 1358,   35091,   35112,   35138, 21, 26, 25, }, // Newfoundland
    {   13, 1371,       0,   35163,       0,  0,  8,  0, }, // Niue
    {   13, 1376,   35171,   35186,   35213, 15, 27, 26, }, // Norfolk
    {   13, 1384,   35239,   35254,   35282, 15, 28, 28, }, // Noronha
    {   13, 1406,   35310,   35329,   35353, 19, 24, 22, }, // Novosibirsk
    {   13, 1418,   35375,   35391,   35411, 16, 20, 19, }, // Omsk
    {   13, 1428,   35430,   35440,   35455, 10, 15, 15, }, // Pakistan
    {   13, 1437,       0,   35470,       0,  0,  8,  0, }, // Palau
    {   13, 1443,       0,   35478,       0,  0, 13,  0, }, // Papua_New_Guinea
    {   13, 1460,   35491,   35500,   35514,  9, 14, 14, }, // Paraguay
    {   13, 1469,   35528,   35535,   35547,  7, 12, 12, }, // Peru
    {   13, 1474,   35559,   35568,   35582,  9, 14, 14, }, // Philippines
    {   13, 1486,       0,   35596,       0,  0, 14,  0, }, // Phoenix_Islands
    {   13, 1502,   35610,   35630,   35655, 20, 25, 29, }, // Pierre_Miquelon
    {   13, 1518,       0,   35684,       0,  0, 11,  0, }, // Pitcairn
    {   13, 1527,       0,   35695,       0,  0,  8,  0, }, // Ponape
    {   13, 1534,       0,   35703,       0,  0, 12,  0, }, // Pyongyang
    {   13, 1554,       0,   35715,       0,  0, 10,  0, }, // Reunion
    {   13, 1562,       0,   35725,       0,  0,  8,  0, }, // Rothera
    {   13, 1570,   35733,   35749,   35770, 16, 21, 20, }, // Sakhalin
    {   13, 1579,   35790,   35804,   35823, 14, 19, 18, }, // Samara
    {   13, 1596,   35841,   35849,   35862,  8, 13, 12, }, // Samoa
    {   13, 1602,       0,   35874,       0,  0,  9,  0, }, // Seychelles
    {   13, 1624,       0,   35883,       0,  0, 15,  0, }, // Singapore
    {   13, 1634,       0,   35898,       0,  0, 14,  0, }, // Solomon
    {   13, 1642,       0,   35912,       0,  0, 14,  0, }, // South_Georgia
    {   13, 1656,       0,   35926,       0,  0,  9,  0, }, // Suriname
    {   13, 1676,       0,   35935,       0,  0,  8,  0, }, // Syowa
    {   13, 1682,       0,   35943,       0,  0,  8,  0, }, // Tahiti
    {   13, 1689,   35951,   35960,   35974,  9, 14, 18, }, // Taipei
    {   13, 1696,       0,   35992,       0,  0, 11,  0, }, // Tajikistan
    {   13, 1724,       0,   36003,       0,  0,  9,  0, }, // Tokelau
    {   13, 1732,   36012,   36020,   36033,  8, 13, 13, }, // Tonga
    {   13, 1738,       0,   36046,       0,  0,  7,  0, }, // Truk
    {   13, 1750,   36053,   36066,   36084, 13, 18, 18, }, // Turkmenistan
    {   13, 1763,       0,   36102,       0,  0,  8,  0, }, // Tuvalu
    {   13, 1777,   36110,   36119,   36133,  9, 14, 14, }, // Uruguay
    {   13, 1792,   36147,   36159,   36176, 12, 17, 17, }, // Uzbekistan
    {   13, 1803,   36193,   36201,   36214,  8, 13, 13, }, // Vanuatu
    {   13, 1811,       0,   36227,       0,  0, 10,  0, }, // Venezuela
    {   13, 1821,   36237,   36256,   36280, 19, 24, 23, }, // Vladivostok
    {   13, 1833,   36303,   36320,   36341, 17, 21, 20, }, // Volgograd
    {   13, 1843,       0,   36361,       0,  0,  9,  0, }, // Vostok
    {   13, 1850,       0,   36370,       0,  0, 11,  0, }, // Wake
    {   13, 1855,       0,   36381,       0,  0, 15,  0, }, // Wallis
    {   13, 1862,   36396,   36412,   36432, 16, 20, 20, }, // Yakutsk
    {   13, 1870,   36452,   36471,   36495, 19, 24, 23, }, // Yekaterinburg
    {   13, 1892,       0,   36518,       0,  0,  7,  0, }, // Yukon
    {   14,    5,       0,   36525,       0,  0, 15,  0, }, // Afghanistan Arabic/Arabic/Egypt
    {   14,   17,       0,   36540,       0,  0, 17,  0, }, // Africa_Central
    {   14,   32,       0,   36557,       0,  0, 17,  0, }, // Africa_Eastern
    {   14,   65,       0,   36574,       0,  0, 18,  0, }, // Africa_Southern
    {   14,   81,   36592,   36609,   36633, 17, 24, 24, }, // Africa_Western
    {   14,  107,   36657,   36669,   36691, 12, 22, 19, }, // Alaska
    {   14,  135,   36710,   36724,   36745, 14, 21, 21, }, // Amazon
    {   14,  142,   36766,   36798,   36837, 32, 39, 39, }, // America_Central
    {   14,  158,   36876,   36907,   36945, 31, 38, 38, }, // America_Eastern
    {   14,  174,   36983,   37014,   37052, 31, 38, 38, }, // America_Mountain
    {   14,  191,   37090,   37109,   37135, 19, 26, 26, }, // America_Pacific
    {   14,  207,   37161,   37173,   37192, 12, 19, 22, }, // Anadyr
    {   14,  214,   37214,   37224,   37244, 10, 20, 20, }, // Apia
    {   14,  232,   37264,   37278,   37299, 14, 21, 21, }, // Arabian
    {   14,  240,   37320,   37335,   37357, 15, 22, 22, }, // Argentina
    {   14,  250,   37379,   37398,   37424, 19, 26, 26, }, // Argentina_Western
    {   14,  268,   37450,   37463,   37483, 13, 20, 20, }, // Armenia
    {   14,  286,   37503,   37516,   37538, 13, 22, 22, }, // Atlantic
    {   14,  295,   37560,   37578,   37603, 18, 25, 25, }, // Australia_Central
    {   14,  313,   37628,   37650,   37679, 22, 29, 29, }, // Australia_CentralWestern
    {   14,  338,   37708,   37726,   37751, 18, 25, 25, }, // Australia_Eastern
    {   14,  356,   37776,   37794,   37819, 18, 25, 25, }, // Australia_Western
    {   14,  374,   37844,   37858,   37879, 14, 21, 21, }, // Azerbaijan
    {   14,  385,   37900,   37911,   37929, 11, 18, 18, }, // Azores
    {   14,  397,   37947,   37961,   37982, 14, 21, 21, }, // Bangladesh
    {   14,  415,       0,   38003,       0,  0, 11,  0, }, // Bhutan
    {   14,  422,       0,   38014,       0,  0, 13,  0, }, // Bolivia
    {   14,  437,   38027,   38041,   38062, 14, 21, 21, }, // Brasilia
    {   14,  454,       0,   38083,       0,  0, 12,  0, }, // Brunei
    {   14,  461,   38095,   38113,   38138, 18, 25, 25, }, // Cape_Verde
    {   14,  478,       0,   38163,       0,  0, 13,  0, }, // Chamorro
    {   14,  487,   38176,   38188,   38207, 12, 19, 19, }, // Chatham
    {   14,  495,   38226,   38237,   38255, 11, 18, 18, }, // Chile
    {   14,  501,   38273,   38284,   38302, 11, 18, 18, }, // China
    {   14,  507,       0,   38320,       0,  0, 19,  0, }, // Christmas
    {   14,  517,       0,   38339,       0,  0, 15,  0, }, // Cocos
    {   14,  523,   38354,   38368,   38389, 14, 21, 21, }, // Colombia
    {   14,  532,   38410,   38424,   38444, 14, 20, 20, }, // Cook
    {   14,  537,   38464,   38474,   38491, 10, 17, 17, }, // Cuba
    {   14,  548,       0,   38508,       0,  0, 11,  0, }, // Davis
    {   14,  564,       0,   38519,       0,  0, 22,  0, }, // DumontDUrville
    {   14,  601,       0,   38541,       0,  0, 19,  0, }, // East_Timor
    {   14,  612,   38560,   38576,   38599, 16, 23, 23, }, // Easter
    {   14,  619,       0,   38622,       0,  0, 15,  0, }, // Ecuador
    {   14,  627,   38637,   38653,   38676, 16, 23, 23, }, // Europe_Central
    {   14,  642,   38699,   38715,   38738, 16, 23, 23, }, // Europe_Eastern
    {   14,  657,       0,   38761,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {   14,  680,   38790,   38806,   38829, 16, 23, 23, }, // Europe_Western
    {   14,  695,   38852,   38869,   38893, 17, 24, 24, }, // Falkland
    {   14,  704,   38917,   38927,   38944, 10, 17, 17, }, // Fiji
    {   14,  709,       0,   38961,       0,  0, 21,  0, }, // French_Guiana
    {   14,  723,       0,   38982,       0,  0, 47,  0, }, // French_Southern
    {   14,  746,       0,   39029,       0,  0, 14,  0, }, // Galapagos
    {   14,  756,       0,   39043,       0,  0, 12,  0, }, // Gambier
    {   14,  764,   39055,   39067,   39086, 12, 19, 19, }, // Georgia
    {   14,  772,       0,   39105,       0,  0, 16,  0, }, // Gilbert_Islands
    {   14,  788,       0,   39121,       0,  0, 12,  0, }, // GMT
    {   14,  830,   39133,   39151,   39176, 18, 25, 25, }, // Greenland_Eastern
    {   14,  848,   39201,   39219,   39244, 18, 25, 25, }, // Greenland_Western
    {   14,  866,       0,   39269,       0,  0, 10,  0, }, // Guam
    {   14,  871,       0,   39279,       0,  0, 12,  0, }, // Gulf
    {   14,  876,       0,   39291,       0,  0, 11,  0, }, // Guyana
    {   14,  883,   39302,   39321,   39347, 19, 26, 26, }, // Hawaii_Aleutian
    {   14,  899,   39373,   39388,   39410, 15, 22, 22, }, // Hong_Kong
    {   14,  909,   39432,   39442,   39459, 10, 17, 17, }, // Hovd
    {   14,  914,       0,   39476,       0,  0, 11,  0, }, // India
    {   14,  920,       0,   39487,       0,  0, 19,  0, }, // Indian_Ocean
    {   14,  933,       0,   39506,       0,  0, 19,  0, }, // Indochina
    {   14,  943,       0,   39525,       0,  0, 19,  0, }, // Indonesia_Central
    {   14,  961,       0,   39544,       0,  0, 19,  0, }, // Indonesia_Eastern
    {   14,  979,       0,   39563,       0,  0, 19,  0, }, // Indonesia_Western
    {   14,  997,   39582,   39593,   39611, 11, 18, 18, }, // Iran
    {   14, 1008,   39629,   39642,   39662, 13, 20, 20, }, // Irkutsk
    {   14, 1016,   39682,   39695,   39715, 13, 20, 20, }, // Israel
    {   14, 1023,   39735,   39748,   39768, 13, 20, 20, }, // Japan
    {   14, 1029,   39788,   39802,   39832, 14, 30, 37, }, // Kamchatka
    {   14, 1047,       0,   39869,       0,  0, 15,  0, }, // Kazakhstan
    {   14, 1058,       0,   39884,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {   14, 1077,       0,   39903,       0,  0, 19,  0, }, // Kazakhstan_Western
    {   14, 1106,   39922,   39933,   39951, 11, 18, 18, }, // Korea
    {   14, 1112,       0,   39969,       0,  0, 11,  0, }, // Kosrae
    {   14, 1119,   39980,   39997,   40021, 17, 24, 27, }, // Krasnoyarsk
    {   14, 1151,       0,   40048,       0,  0, 16,  0, }, // Kyrgystan
    {   14, 1175,       0,   40064,       0,  0, 14,  0, }, // Line_Islands
    {   14, 1188,   40078,   40092,   40113, 14, 21, 24, }, // Lord_Howe
    {   14, 1204,   40137,   40150,   40170, 13, 20, 20, }, // Magadan
    {   14, 1219,       0,   40190,       0,  0, 13,  0, }, // Malaysia
    {   14, 1228,       0,   40203,       0,  0, 18,  0, }, // Maldives
    {   14, 1237,       0,   40221,       0,  0, 14,  0, }, // Marquesas
    {   14, 1247,       0,   40235,       0,  0, 16,  0, }, // Marshall_Islands
    {   14, 1264,   40251,   40265,   40286, 14, 21, 21, }, // Mauritius
    {   14, 1274,       0,   40307,       0,  0, 12,  0, }, // Mawson
    {   14, 1281,   40319,   40346,   40380, 27, 34, 34, }, // Mexico_Pacific
    {   14, 1296,   40414,   40431,   40455, 17, 24, 24, }, // Mongolia
    {   14, 1305,   40479,   40490,   40508, 11, 18, 18, }, // Moscow
    {   14, 1312,       0,   40526,       0,  0, 13,  0, }, // Myanmar
    {   14, 1320,       0,   40539,       0,  0, 11,  0, }, // Nauru
    {   14, 1326,       0,   40550,       0,  0, 11,  0, }, // Nepal
    {   14, 1332,   40561,   40584,   40614, 23, 30, 30, }, // New_Caledonia
    {   14, 1346,   40644,   40659,   40681, 15, 22, 22, }, // New_Zealand
    {   14, 1358,   40703,   40721,   40746, 18, 25, 25, }, // Newfoundland
    {   14, 1371,       0,   40771,       0,  0, 10,  0, }, // Niue
    {   14, 1376,   40781,   40800,   40826, 19, 26, 26, }, // Norfolk
    {   14, 1384,   40852,   40877,   40908, 25, 31, 31, }, // Noronha
    {   14, 1392,       0,   40939,       0,  0, 26,  0, }, // North_Mariana
    {   14, 1406,   40965,   40982,   41006, 17, 24, 24, }, // Novosibirsk
    {   14, 1418,   41030,   41041,   41059, 11, 18, 18, }, // Omsk
    {   14, 1428,   41077,   41090,   41110, 13, 20, 20, }, // Pakistan
    {   14, 1437,       0,   41130,       0,  0, 11,  0, }, // Palau
    {   14, 1443,       0,   41141,       0,  0, 25,  0, }, // Papua_New_Guinea
    {   14, 1460,   41166,   41180,   41201, 14, 21, 21, }, // Paraguay
    {   14, 1469,   41222,   41232,   41249, 10, 17, 17, }, // Peru
    {   14, 1474,   41266,   41280,   41301, 14, 21, 21, }, // Philippines
    {   14, 1486,       0,   41322,       0,  0, 15,  0, }, // Phoenix_Islands
    {   14, 1502,   41337,   41361,   41392, 24, 31, 31, }, // Pierre_Miquelon
    {   14, 1518,       0,   41423,       0,  0, 13,  0, }, // Pitcairn
    {   14, 1527,       0,   41436,       0,  0, 12,  0, }, // Ponape
    {   14, 1534,       0,   41448,       0,  0, 16,  0, }, // Pyongyang
    {   14, 1554,       0,   41464,       0,  0, 13,  0, }, // Reunion
    {   14, 1562,       0,   41477,       0,  0, 12,  0, }, // Rothera
    {   14, 1570,   41489,   41502,   41522, 13, 20, 20, }, // Sakhalin
    {   14, 1579,   41542,   41554,   41565, 12, 11, 18, }, // Samara
    {   14, 1596,   41583,   41594,   41612, 11, 18, 18, }, // Samoa
    {   14, 1602,       0,   41630,       0,  0, 10,  0, }, // Seychelles
    {   14, 1624,       0,   41640,       0,  0, 14,  0, }, // Singapore
    {   14, 1634,       0,   41654,       0,  0, 16,  0, }, // Solomon
    {   14, 1642,       0,   41670,       0,  0, 17,  0, }, // South_Georgia
    {   14, 1656,       0,   41687,       0,  0, 13,  0, }, // Suriname
    {   14, 1676,       0,   41700,       0,  0, 12,  0, }, // Syowa
    {   14, 1682,       0,   41712,       0,  0, 12,  0, }, // Tahiti
    {   14, 1689,   41724,   41736,   41755, 12, 19, 19, }, // Taipei
    {   14, 1696,       0,   41774,       0,  0, 14,  0, }, // Tajikistan
    {   14, 1724,       0,   41788,       0,  0, 13,  0, }, // Tokelau
    {   14, 1732,   41801,   41812,   41830, 11, 18, 18, }, // Tonga
    {   14, 1738,       0,   41848,       0,  0,  9,  0, }, // Truk
    {   14, 1750,   41857,   41873,   41896, 16, 23, 23, }, // Turkmenistan
    {   14, 1763,       0,   41919,       0,  0, 12,  0, }, // Tuvalu
    {   14, 1777,   41931,   41945,   41966, 14, 21, 21, }, // Uruguay
    {   14, 1792,   41987,   42002,   42024, 15, 22, 22, }, // Uzbekistan
    {   14, 1803,   42046,   42059,   42079, 13, 20, 20, }, // Vanuatu
    {   14, 1811,       0,   42099,       0,  0, 13,  0, }, // Venezuela
    {   14, 1821,   42112,   42129,   42153, 17, 24, 24, }, // Vladivostok
    {   14, 1833,   42177,   42192,   42214, 15, 22, 22, }, // Volgograd
    {   14, 1843,       0,   42236,       0,  0, 12,  0, }, // Vostok
    {   14, 1850,       0,   42248,       0,  0, 15,  0, }, // Wake
    {   14, 1855,       0,   42263,       0,  0, 20,  0, }, // Wallis
    {   14, 1862,   42283,   42296,   42316, 13, 20, 20, }, // Yakutsk
    {   14, 1870,   42336,   42353,   42377, 17, 24, 24, }, // Yekaterinburg
    {   14, 1892,       0,   42401,       0,  0, 11,  0, }, // Yukon
    {   42,  142,   42412,   42444,   42485, 32, 41, 42, }, // America_Central Aragonese/Latin/Spain
    {   42,  158,   42527,   42560,   42602, 33, 42, 43, }, // America_Eastern
    {   42,  174,   42645,   42681,   42726, 36, 45, 46, }, // America_Mountain
    {   42,  191,   42772,   42809,   42855, 37, 46, 47, }, // America_Pacific
    {   42,  286,   42902,   42921,   42949, 19, 28, 29, }, // Atlantic
    {   42,  627,   42978,   43002,   43035, 24, 33, 34, }, // Europe_Central
    {   42,  642,   43069,   43092,   43124, 23, 32, 33, }, // Europe_Eastern
    {   42,  680,   43157,   43181,   43214, 24, 33, 34, }, // Europe_Western
    {   42,  788,       0,   43248,       0,  0, 31,  0, }, // GMT
    {   43,    5,       0,   43279,       0,  0, 18,  0, }, // Afghanistan Armenian/Armenian/Armenia
    {   43,   17,       0,   43297,       0,  0, 28,  0, }, // Africa_Central
    {   43,   32,       0,   43325,       0,  0, 25,  0, }, // Africa_Eastern
    {   43,   65,       0,   43350,       0,  0, 26,  0, }, // Africa_Southern
    {   43,   81,   43376,   43401,   43435, 25, 34, 34, }, // Africa_Western
    {   43,  107,   43469,   43486,   43512, 17, 26, 26, }, // Alaska
    {   43,  135,   43538,   43555,   43581, 17, 26, 26, }, // Amazon
    {   43,  142,   43607,   43636,   43674, 29, 38, 38, }, // America_Central
    {   43,  158,   43712,   43738,   43773, 26, 35, 35, }, // America_Eastern
    {   43,  174,   43808,   43830,   43861, 22, 31, 31, }, // America_Mountain
    {   43,  191,   43892,   43916,   43949, 24, 33, 33, }, // America_Pacific
    {   43,  214,   43982,   43996,   44019, 14, 23, 23, }, // Apia
    {   43,  232,   44042,   44067,   44101, 25, 34, 34, }, // Arabian
    {   43,  240,   44135,   44154,   44182, 19, 28, 28, }, // Argentina
    {   43,  250,   44210,   44238,   44275, 28, 37, 37, }, // Argentina_Western
    {   43,  268,   44312,   44329,   44355, 17, 26, 26, }, // Armenia
    {   43,  286,   44381,   44398,   44424, 17, 26, 26, }, // Atlantic
    {   43,  295,   44450,   44481,   44521, 31, 40, 40, }, // Australia_Central
    {   43,  313,   44561,   44601,   44650, 40, 49, 49, }, // Australia_CentralWestern
    {   43,  338,   44699,   44727,   44764, 28, 37, 37, }, // Australia_Eastern
    {   43,  356,   44801,   44829,   44866, 28, 37, 37, }, // Australia_Western
    {   43,  374,   44903,   44920,   44946, 17, 26, 26, }, // Azerbaijan
    {   43,  385,   44972,   44996,   45029, 24, 33, 33, }, // Azores
    {   43,  397,   45062,   45080,   45107, 18, 27, 27, }, // Bangladesh
    {   43,  415,       0,   45134,       0,  0, 15,  0, }, // Bhutan
    {   43,  422,       0,   45149,       0,  0, 17,  0, }, // Bolivia
    {   43,  437,   45166,   45184,   45211, 18, 27, 27, }, // Brasilia
    {   43,  454,       0,   45238,       0,  0, 16,  0, }, // Brunei
    {   43,  461,   45254,   45273,   45301, 19, 28, 28, }, // Cape_Verde
    {   43,  478,       0,   45329,       0,  0, 16,  0, }, // Chamorro
    {   43,  487,   45345,   45364,   45392, 19, 28, 28, }, // Chatham
    {   43,  495,   45420,   45433,   45455, 13, 22, 22, }, // Chile
    {   43,  501,   45477,   45494,   45520, 17, 26, 26, }, // China
    {   43,  507,       0,   45546,       0,  0, 27,  0, }, // Christmas
    {   43,  517,       0,   45573,       0,  0, 25,  0, }, // Cocos
    {   43,  523,   45598,   45617,   45645, 19, 28, 28, }, // Colombia
    {   43,  532,   45673,   45695,   45726, 22, 31, 35, }, // Cook
    {   43,  537,   45761,   45776,   45800, 15, 24, 24, }, // Cuba
    {   43,  548,       0,   45824,       0,  0, 15,  0, }, // Davis
    {   43,  564,       0,   45839,       0,  0, 26,  0, }, // DumontDUrville
    {   43,  601,       0,   45865,       0,  0, 23,  0, }, // East_Timor
    {   43,  612,   45888,   45907,   45935, 19, 28, 28, }, // Easter
    {   43,  619,       0,   45963,       0,  0, 16,  0, }, // Ecuador
    {   43,  627,   45979,   46007,   46044, 28, 37, 37, }, // Europe_Central
    {   43,  642,   46081,   46106,   46140, 25, 34, 34, }, // Europe_Eastern
    {   43,  657,       0,   46174,       0,  0, 14,  0, }, // Europe_Further_Eastern
    {   43,  680,   46188,   46213,   46247, 25, 34, 34, }, // Europe_Western
    {   43,  695,   46281,   46309,   46346, 28, 37, 37, }, // Falkland
    {   43,  704,   46383,   46396,   46418, 13, 22, 22, }, // Fiji
    {   43,  709,       0,   46440,       0,  0, 27,  0, }, // French_Guiana
    {   43,  723,       0,   46467,       0,  0, 43,  0, }, // French_Southern
    {   43,  746,       0,   46510,       0,  0, 29,  0, }, // Galapagos
    {   43,  756,       0,   46539,       0,  0, 23,  0, }, // Gambier
    {   43,  764,   46562,   46578,   46603, 16, 25, 25, }, // Georgia
    {   43,  772,       0,   46628,       0,  0, 25,  0, }, // Gilbert_Islands
    {   43,  788,       0,   46653,       0,  0, 16,  0, }, // GMT
    {   43,  830,   46669,   46698,   46736, 29, 38, 38, }, // Greenland_Eastern
    {   43,  848,   46774,   46803,   46841, 29, 38, 38, }, // Greenland_Western
    {   43,  871,       0,   46879,       0,  0, 28,  0, }, // Gulf
    {   43,  876,       0,   46907,       0,  0, 16,  0, }, // Guyana
    {   43,  883,   46923,   46948,   46982, 25, 34, 34, }, // Hawaii_Aleutian
    {   43,  899,   47016,   47032,   47057, 16, 25, 25, }, // Hong_Kong
    {   43,  909,   47082,   47095,   47117, 13, 22, 22, }, // Hovd
    {   43,  914,       0,   47139,       0,  0, 27,  0, }, // India
    {   43,  920,       0,   47166,       0,  0, 26,  0, }, // Indian_Ocean
    {   43,  933,       0,   47192,       0,  0, 20,  0, }, // Indochina
    {   43,  943,       0,   47212,       0,  0, 31,  0, }, // Indonesia_Central
    {   43,  961,       0,   47243,       0,  0, 28,  0, }, // Indonesia_Eastern
    {   43,  979,       0,   47271,       0,  0, 28,  0, }, // Indonesia_Western
    {   43,  997,   47299,   47312,   47334, 13, 22, 22, }, // Iran
    {   43, 1008,   47356,   47373,   47399, 17, 26, 26, }, // Irkutsk
    {   43, 1016,   47425,   47441,   47466, 16, 25, 25, }, // Israel
    {   43, 1023,   47491,   47508,   47534, 17, 26, 26, }, // Japan
    {   43, 1047,       0,   47560,       0,  0, 18,  0, }, // Kazakhstan
    {   43, 1058,       0,   47578,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {   43, 1077,       0,   47605,       0,  0, 27,  0, }, // Kazakhstan_Western
    {   43, 1106,   47632,   47647,   47671, 15, 24, 24, }, // Korea
    {   43, 1112,       0,   47695,       0,  0, 16,  0, }, // Kosrae
    {   43, 1119,   47711,   47731,   47760, 20, 29, 29, }, // Krasnoyarsk
    {   43, 1151,       0,   47789,       0,  0, 17,  0, }, // Kyrgystan
    {   43, 1175,       0,   47806,       0,  0, 21,  0, }, // Line_Islands
    {   43, 1188,   47827,   47845,   47872, 18, 27, 27, }, // Lord_Howe
    {   43, 1204,   47899,   47915,   47940, 16, 25, 25, }, // Magadan
    {   43, 1219,       0,   47965,       0,  0, 18,  0, }, // Malaysia
    {   43, 1228,       0,   47983,       0,  0, 18,  0, }, // Maldives
    {   43, 1237,       0,   48001,       0,  0, 26,  0, }, // Marquesas
    {   43, 1247,       0,   48027,       0,  0, 26,  0, }, // Marshall_Islands
    {   43, 1264,   48053,   48071,   48098, 18, 27, 27, }, // Mauritius
    {   43, 1274,       0,   48125,       0,  0, 16,  0, }, // Mawson
    {   43, 1281,   48141,   48175,   48218, 34, 43, 43, }, // Mexico_Pacific
    {   43, 1296,   48261,   48281,   48310, 20, 29, 29, }, // Mongolia
    {   43, 1305,   48339,   48355,   48380, 16, 25, 25, }, // Moscow
    {   43, 1312,       0,   48405,       0,  0, 16,  0, }, // Myanmar
    {   43, 1320,       0,   48421,       0,  0, 16,  0, }, // Nauru
    {   43, 1326,       0,   48437,       0,  0, 14,  0, }, // Nepal
    {   43, 1332,   48451,   48474,   48506, 23, 32, 32, }, // New_Caledonia
    {   43, 1346,   48538,   48560,   48591, 22, 31, 31, }, // New_Zealand
    {   43, 1358,   48622,   48645,   48677, 23, 32, 32, }, // Newfoundland
    {   43, 1371,       0,   48709,       0,  0, 15,  0, }, // Niue
    {   43, 1376,   48724,   48745,   48775, 21, 30, 30, }, // Norfolk
    {   43, 1384,   48805,   48835,   48874, 30, 39, 39, }, // Noronha
    {   43, 1406,   48913,   48933,   48962, 20, 29, 29, }, // Novosibirsk
    {   43, 1418,   48991,   49004,   49026, 13, 22, 22, }, // Omsk
    {   43, 1428,   49048,   49065,   49091, 17, 26, 26, }, // Pakistan
    {   43, 1437,       0,   49117,       0,  0, 16,  0, }, // Palau
    {   43, 1443,       0,   49133,       0,  0, 27,  0, }, // Papua_New_Guinea
    {   43, 1460,   49160,   49177,   49203, 17, 26, 26, }, // Paraguay
    {   43, 1469,   49229,   49243,   49266, 14, 23, 23, }, // Peru
    {   43, 1474,   49289,   49308,   49336, 19, 28, 28, }, // Philippines
    {   43, 1486,       0,   49364,       0,  0, 23,  0, }, // Phoenix_Islands
    {   43, 1502,   49387,   49414,   49450, 27, 36, 36, }, // Pierre_Miquelon
    {   43, 1518,       0,   49486,       0,  0, 16,  0, }, // Pitcairn
    {   43, 1527,       0,   49502,       0,  0, 20,  0, }, // Ponape
    {   43, 1534,       0,   49522,       0,  0, 16,  0, }, // Pyongyang
    {   43, 1554,       0,   49538,       0,  0, 18,  0, }, // Reunion
    {   43, 1562,       0,   49556,       0,  0, 16,  0, }, // Rothera
    {   43, 1570,   49572,   49588,   49613, 16, 25, 25, }, // Sakhalin
    {   43, 1596,   49638,   49653,   49677, 15, 24, 24, }, // Samoa
    {   43, 1602,       0,   49701,       0,  0, 26,  0, }, // Seychelles
    {   43, 1624,       0,   49727,       0,  0, 18,  0, }, // Singapore
    {   43, 1634,       0,   49745,       0,  0, 25,  0, }, // Solomon
    {   43, 1642,       0,   49770,       0,  0, 26,  0, }, // South_Georgia
    {   43, 1656,       0,   49796,       0,  0, 17,  0, }, // Suriname
    {   43, 1676,       0,   49813,       0,  0, 15,  0, }, // Syowa
    {   43, 1682,       0,   49828,       0,  0, 14,  0, }, // Tahiti
    {   43, 1689,   49842,   49857,   49881, 15, 24, 24, }, // Taipei
    {   43, 1696,       0,   49905,       0,  0, 18,  0, }, // Tajikistan
    {   43, 1724,       0,   49923,       0,  0, 18,  0, }, // Tokelau
    {   43, 1732,   49941,   49956,   49980, 15, 24, 24, }, // Tonga
    {   43, 1738,       0,   50004,       0,  0, 14,  0, }, // Truk
    {   43, 1750,   50018,   50039,   50069, 21, 30, 30, }, // Turkmenistan
    {   43, 1763,       0,   50099,       0,  0, 18,  0, }, // Tuvalu
    {   43, 1777,   50117,   50135,   50162, 18, 27, 27, }, // Uruguay
    {   43, 1792,   50189,   50208,   50236, 19, 28, 28, }, // Uzbekistan
    {   43, 1803,   50264,   50283,   50311, 19, 28, 28, }, // Vanuatu
    {   43, 1811,       0,   50339,       0,  0, 20,  0, }, // Venezuela
    {   43, 1821,   50359,   50379,   50408, 20, 29, 29, }, // Vladivostok
    {   43, 1833,   50437,   50455,   50482, 18, 27, 27, }, // Volgograd
    {   43, 1843,       0,   50509,       0,  0, 15,  0, }, // Vostok
    {   43, 1850,       0,   50524,       0,  0, 19,  0, }, // Wake
    {   43, 1855,       0,   50543,       0,  0, 27,  0, }, // Wallis
    {   43, 1862,   50570,   50587,   50613, 17, 26, 26, }, // Yakutsk
    {   43, 1870,   50639,   50661,   50692, 22, 31, 31, }, // Yekaterinburg
    {   43, 1892,       0,   50723,       0,  0, 15,  0, }, // Yukon
    {   44,    5,       0,   50738,       0,  0, 17,  0, }, // Afghanistan Assamese/Bangla/India
    {   44,   17,       0,   50755,       0,  0, 18,  0, }, // Africa_Central
    {   44,   32,       0,   50773,       0,  0, 17,  0, }, // Africa_Eastern
    {   44,   65,       0,   50790,       0,  0, 24,  0, }, // Africa_Southern
    {   44,   81,   50814,   50834,   50858, 20, 24, 33, }, // Africa_Western
    {   44,  107,   50891,   50904,   50921, 13, 17, 20, }, // Alaska
    {   44,  135,   50941,   50952,   50967, 11, 15, 24, }, // Amazon
    {   44,  142,   50991,   51021,   51055, 30, 34, 37, }, // America_Central
    {   44,  158,   51092,   51119,   51150, 27, 31, 34, }, // America_Eastern
    {   44,  174,   51184,   51214,   51248, 30, 34, 37, }, // America_Mountain
    {   44,  191,   51285,   51313,   51345, 28, 32, 26, }, // America_Pacific
    {   44,  214,   51371,   51383,   51399, 12, 16, 19, }, // Apia
    {   44,  232,   51418,   51429,   51444, 11, 15, 18, }, // Arabian
    {   44,  240,   51462,   51479,   51500, 17, 21, 30, }, // Argentina
    {   44,  250,   51530,   51557,   51588, 27, 31, 40, }, // Argentina_Western
    {   44,  268,   51628,   51644,   51664, 16, 20, 29, }, // Armenia
    {   44,  286,   51693,   51707,   51725, 14, 18, 21, }, // Atlantic
    {   44,  295,   51746,   51769,   51802, 23, 33, 36, }, // Australia_Central
    {   44,  313,   51838,   51877,   51920, 39, 43, 46, }, // Australia_CentralWestern
    {   44,  338,   51966,   51992,   52022, 26, 30, 33, }, // Australia_Eastern
    {   44,  356,   52055,   52083,   52115, 28, 32, 35, }, // Australia_Western
    {   44,  374,   52150,   52166,   52186, 16, 20, 29, }, // Azerbaijan
    {   44,  385,   52215,   52226,   52242, 11, 16, 25, }, // Azores
    {   44,  397,   52267,   52281,   52299, 14, 18, 27, }, // Bangladesh
    {   44,  415,       0,   52326,       0,  0, 11,  0, }, // Bhutan
    {   44,  422,       0,   52337,       0,  0, 14,  0, }, // Bolivia
    {   44,  437,   52351,   52368,   52389, 17, 21, 30, }, // Brasilia
    {   44,  454,       0,   52419,       0,  0, 23,  0, }, // Brunei
    {   44,  461,   52442,   52458,   52478, 16, 20, 29, }, // Cape_Verde
    {   44,  478,       0,   52507,       0,  0, 16,  0, }, // Chamorro
    {   44,  487,   52523,   52534,   52549, 11, 15, 18, }, // Chatham
    {   44,  495,   52567,   52577,   52591, 10, 14, 23, }, // Chile
    {   44,  501,   52614,   52623,   52636,  9, 13, 16, }, // China
    {   44,  507,       0,   52652,       0,  0, 22,  0, }, // Christmas
    {   44,  517,       0,   52674,       0,  0, 22,  0, }, // Cocos
    {   44,  523,   52696,   52711,   52730, 15, 19, 28, }, // Colombia
    {   44,  532,   52758,   52778,   52802, 20, 24, 38, }, // Cook
    {   44,  537,   52840,   52851,   52866, 11, 15, 18, }, // Cuba
    {   44,  548,       0,   52884,       0,  0, 11,  0, }, // Davis
    {   44,  564,       0,   52895,       0,  0, 23,  0, }, // DumontDUrville
    {   44,  601,       0,   52918,       0,  0, 15,  0, }, // East_Timor
    {   44,  612,   52933,   52951,   52973, 18, 22, 31, }, // Easter
    {   44,  619,       0,   53004,       0,  0, 13,  0, }, // Ecuador
    {   44,  627,   53017,   53035,   53057, 18, 22, 31, }, // Europe_Central
    {   44,  642,   53088,   53109,   53134, 21, 25, 34, }, // Europe_Eastern
    {   44,  657,       0,   53168,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {   44,  680,   53194,   53217,   53244, 23, 27, 36, }, // Europe_Western
    {   44,  695,   53280,   53304,   53332, 24, 28, 37, }, // Falkland
    {   44,  704,   53369,   53379,   53393, 10, 14, 23, }, // Fiji
    {   44,  709,       0,   53416,       0,  0, 20,  0, }, // French_Guiana
    {   44,  723,       0,   53436,       0,  0, 35,  0, }, // French_Southern
    {   44,  746,       0,   53471,       0,  0, 15,  0, }, // Galapagos
    {   44,  756,       0,   53486,       0,  0, 15,  0, }, // Gambier
    {   44,  764,   53501,   53515,   53533, 14, 18, 27, }, // Georgia
    {   44,  772,       0,   53560,       0,  0, 25,  0, }, // Gilbert_Islands
    {   44,  788,       0,   53585,       0,  0, 17,  0, }, // GMT
    {   44,  830,   53602,   53622,   53646, 20, 24, 33, }, // Greenland_Eastern
    {   44,  848,   53679,   53702,   53729, 23, 27, 36, }, // Greenland_Western
    {   44,  871,       0,   53765,       0,  0, 18,  0, }, // Gulf
    {   44,  876,       0,   53783,       0,  0, 13,  0, }, // Guyana
    {   44,  883,   53796,   53816,   53840, 20, 24, 27, }, // Hawaii_Aleutian
    {   44,  899,   53867,   53878,   53893, 11, 15, 24, }, // Hong_Kong
    {   44,  909,   53917,   53929,   53945, 12, 16, 25, }, // Hovd
    {   44,  914,       0,   53970,       0,  0, 16,  0, }, // India
    {   44,  920,       0,   53986,       0,  0, 20,  0, }, // Indian_Ocean
    {   44,  933,       0,   54006,       0,  0, 16,  0, }, // Indochina
    {   44,  943,       0,   54022,       0,  0, 23,  0, }, // Indonesia_Central
    {   44,  961,       0,   54045,       0,  0, 26,  0, }, // Indonesia_Eastern
    {   44,  979,       0,   54071,       0,  0, 28,  0, }, // Indonesia_Western
    {   44,  997,   54099,   54109,   54123, 10, 14, 17, }, // Iran
    {   44, 1008,   54140,   54155,   54174, 15, 19, 28, }, // Irkutsk
    {   44, 1016,   54202,   54214,   54230, 12, 16, 19, }, // Israel
    {   44, 1023,   54249,   54260,   54275, 11, 15, 18, }, // Japan
    {   44, 1047,       0,   54293,       0,  0, 16,  0, }, // Kazakhstan
    {   44, 1058,       0,   54309,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {   44, 1077,       0,   54329,       0,  0, 23,  0, }, // Kazakhstan_Western
    {   44, 1106,   54352,   54365,   54382, 13, 17, 20, }, // Korea
    {   44, 1112,       0,   54402,       0,  0, 13,  0, }, // Kosrae
    {   44, 1119,   54415,   54436,   54461, 21, 25, 33, }, // Krasnoyarsk
    {   44, 1151,       0,   54494,       0,  0, 18,  0, }, // Kyrgystan
    {   44, 1175,       0,   54512,       0,  0, 21,  0, }, // Line_Islands
    {   44, 1188,   54533,   54547,   54565, 14, 18, 21, }, // Lord_Howe
    {   44, 1204,   54586,   54599,   54616, 13, 17, 26, }, // Magadan
    {   44, 1219,       0,   54642,       0,  0, 17,  0, }, // Malaysia
    {   44, 1228,       0,   54659,       0,  0, 14,  0, }, // Maldives
    {   44, 1237,       0,   54673,       0,  0, 14,  0, }, // Marquesas
    {   44, 1247,       0,   54687,       0,  0, 26,  0, }, // Marshall_Islands
    {   44, 1264,   54713,   54725,   54741, 12, 16, 25, }, // Mauritius
    {   44, 1274,       0,   54766,       0,  0, 11,  0, }, // Mawson
    {   44, 1281,   54777,   54800,   54827, 23, 27, 30, }, // Mexico_Pacific
    {   44, 1296,   54857,   54872,   54891, 15, 19, 28, }, // Mongolia
    {   44, 1305,   54919,   54930,   54945, 11, 15, 24, }, // Moscow
    {   44, 1312,       0,   54969,       0,  0, 14,  0, }, // Myanmar
    {   44, 1320,       0,   54983,       0,  0, 11,  0, }, // Nauru
    {   44, 1326,       0,   54994,       0,  0, 11,  0, }, // Nepal
    {   44, 1332,   55005,   55026,   55051, 21, 25, 34, }, // New_Caledonia
    {   44, 1346,   55085,   55101,   55121, 16, 20, 23, }, // New_Zealand
    {   44, 1358,   55144,   55164,   55188, 20, 24, 27, }, // Newfoundland
    {   44, 1371,       0,   55215,       0,  0, 11,  0, }, // Niue
    {   44, 1376,   55226,   55244,   55268, 18, 24, 31, }, // Norfolk
    {   44, 1384,   55299,   55328,   55361, 29, 33, 42, }, // Noronha
    {   44, 1406,   55403,   55421,   55443, 18, 22, 31, }, // Novosibirsk
    {   44, 1418,   55474,   55485,   55500, 11, 15, 24, }, // Omsk
    {   44, 1428,   55524,   55539,   55558, 15, 19, 28, }, // Pakistan
    {   44, 1437,       0,   55586,       0,  0, 11,  0, }, // Palau
    {   44, 1443,       0,   55597,       0,  0, 21,  0, }, // Papua_New_Guinea
    {   44, 1460,   55618,   55632,   55650, 14, 18, 27, }, // Paraguay
    {   44, 1469,   55677,   55687,   55701, 10, 14, 23, }, // Peru
    {   44, 1474,   55724,   55738,   55756, 14, 18, 27, }, // Philippines
    {   44, 1486,       0,   55783,       0,  0, 24,  0, }, // Phoenix_Islands
    {   44, 1502,   55807,   55841,   55879, 34, 38, 41, }, // Pierre_Miquelon
    {   44, 1518,       0,   55920,       0,  0, 15,  0, }, // Pitcairn
    {   44, 1527,       0,   55935,       0,  0, 12,  0, }, // Ponape
    {   44, 1534,       0,   55947,       0,  0, 14,  0, }, // Pyongyang
    {   44, 1554,       0,   55961,       0,  0, 15,  0, }, // Reunion
    {   44, 1562,       0,   55976,       0,  0, 12,  0, }, // Rothera
    {   44, 1570,   55988,   56001,   56018, 13, 17, 26, }, // Sakhalin
    {   44, 1596,   56044,   56056,   56072, 12, 16, 19, }, // Samoa
    {   44, 1602,       0,   56091,       0,  0, 13,  0, }, // Seychelles
    {   44, 1624,       0,   56104,       0,  0, 18,  0, }, // Singapore
    {   44, 1634,       0,   56122,       0,  0, 24,  0, }, // Solomon
    {   44, 1642,       0,   56146,       0,  0, 21,  0, }, // South_Georgia
    {   44, 1656,       0,   56167,       0,  0, 13,  0, }, // Suriname
    {   44, 1676,       0,   56180,       0,  0, 11,  0, }, // Syowa
    {   44, 1682,       0,   56191,       0,  0, 12,  0, }, // Tahiti
    {   44, 1689,   56203,   56215,   56231, 12, 16, 19, }, // Taipei
    {   44, 1696,       0,   56250,       0,  0, 17,  0, }, // Tajikistan
    {   44, 1724,       0,   56267,       0,  0, 13,  0, }, // Tokelau
    {   44, 1732,   56280,   56290,   56304, 10, 14, 23, }, // Tonga
    {   44, 1738,       0,   56327,       0,  0, 11,  0, }, // Truk
    {   44, 1750,   56338,   56358,   56382, 20, 24, 33, }, // Turkmenistan
    {   44, 1763,       0,   56415,       0,  0, 13,  0, }, // Tuvalu
    {   44, 1777,   56428,   56441,   56458, 13, 17, 26, }, // Uruguay
    {   44, 1792,   56484,   56501,   56522, 17, 21, 30, }, // Uzbekistan
    {   44, 1803,   56552,   56564,   56580, 12, 16, 25, }, // Vanuatu
    {   44, 1811,       0,   56605,       0,  0, 16,  0, }, // Venezuela
    {   44, 1821,   56621,   56640,   56663, 19, 23, 32, }, // Vladivostok
    {   44, 1833,   56695,   56712,   56733, 17, 21, 30, }, // Volgograd
    {   44, 1843,       0,   56763,       0,  0, 12,  0, }, // Vostok
    {   44, 1850,       0,   56775,       0,  0, 15,  0, }, // Wake
    {   44, 1855,       0,   56790,       0,  0, 22,  0, }, // Wallis
    {   44, 1862,   56812,   56828,   56848, 16, 20, 29, }, // Yakutsk
    {   44, 1870,   56877,   56898,   56923, 21, 25, 34, }, // Yekaterinburg
    {   44, 1892,       0,   56957,       0,  0, 11,  0, }, // Yukon
    {   45,    0,   56968,   56979,   56999, 11, 20, 20, }, // Acre Asturian/Latin/Spain
    {   45,    5,       0,   57019,       0,  0, 17,  0, }, // Afghanistan
    {   45,   17,       0,   57036,       0,  0, 21,  0, }, // Africa_Central
    {   45,   32,       0,   57057,       0,  0, 22,  0, }, // Africa_Eastern
    {   45,   65,       0,   57079,       0,  0, 17,  0, }, // Africa_Southern
    {   45,   81,   57096,   57119,   57151, 23, 32, 32, }, // Africa_Western
    {   45,  107,   57183,   57196,   57218, 13, 22, 22, }, // Alaska
    {   45,  128,   57240,   57253,   57275, 13, 22, 22, }, // Almaty
    {   45,  135,   57297,   57314,   57340, 17, 26, 26, }, // Amazon
    {   45,  142,   57366,   57393,   57429, 27, 36, 36, }, // America_Central
    {   45,  158,   57465,   57493,   57530, 28, 37, 37, }, // America_Eastern
    {   45,  174,   57567,   57603,   57648, 36, 45, 45, }, // America_Mountain
    {   45,  191,   57693,   57725,   57766, 32, 41, 41, }, // America_Pacific
    {   45,  207,   57807,   57820,   57842, 13, 22, 22, }, // Anadyr
    {   45,  214,   57864,   57875,   57895, 11, 20, 20, }, // Apia
    {   45,  219,   57915,   57927,   57948, 12, 21, 21, }, // Aqtau
    {   45,  225,   57969,   57982,   58004, 13, 22, 22, }, // Aqtobe
    {   45,  232,   58026,   58039,   58061, 13, 22, 22, }, // Arabian
    {   45,  240,   58083,   58099,   58124, 16, 25, 25, }, // Argentina
    {   45,  250,   58149,   58176,   58212, 27, 36, 36, }, // Argentina_Western
    {   45,  268,   58248,   58262,   58285, 14, 23, 23, }, // Armenia
    {   45,  286,   58308,   58326,   58353, 18, 27, 27, }, // Atlantic
    {   45,  295,   58380,   58404,   58437, 24, 33, 33, }, // Australia_Central
    {   45,  313,   58470,   58504,   58547, 34, 43, 43, }, // Australia_CentralWestern
    {   45,  338,   58590,   58615,   58649, 25, 34, 34, }, // Australia_Eastern
    {   45,  356,   58683,   58709,   58744, 26, 35, 35, }, // Australia_Western
    {   45,  374,   58779,   58796,   58822, 17, 26, 26, }, // Azerbaijan
    {   45,  385,   58848,   58866,   58893, 18, 27, 27, }, // Azores
    {   45,  397,   58920,   58937,   58963, 17, 26, 26, }, // Bangladesh
    {   45,  415,       0,   58989,       0,  0, 13,  0, }, // Bhutan
    {   45,  422,       0,   59002,       0,  0, 15,  0, }, // Bolivia
    {   45,  437,   59017,   59033,   59058, 16, 25, 25, }, // Brasilia
    {   45,  454,       0,   59083,       0,  0, 25,  0, }, // Brunei
    {   45,  461,   59108,   59126,   59153, 18, 27, 27, }, // Cape_Verde
    {   45,  472,       0,   59180,       0,  0, 13,  0, }, // Casey
    {   45,  478,       0,   59193,       0,  0, 25,  0, }, // Chamorro
    {   45,  487,   59218,   59233,   59257, 15, 24, 24, }, // Chatham
    {   45,  495,   59281,   59294,   59316, 13, 22, 22, }, // Chile
    {   45,  501,   59338,   59351,   59373, 13, 22, 22, }, // China
    {   45,  507,       0,   59395,       0,  0, 35,  0, }, // Christmas
    {   45,  517,       0,   59430,       0,  0, 24,  0, }, // Cocos
    {   45,  523,   59454,   59470,   59495, 16, 25, 25, }, // Colombia
    {   45,  532,   59520,   59543,   59575, 23, 32, 38, }, // Cook
    {   45,  537,   59613,   59625,   59646, 12, 21, 21, }, // Cuba
    {   45,  548,       0,   59667,       0,  0, 13,  0, }, // Davis
    {   45,  564,       0,   59680,       0,  0, 24,  0, }, // DumontDUrville
    {   45,  601,       0,   59704,       0,  0, 22,  0, }, // East_Timor
    {   45,  612,   59726,   59752,   59787, 26, 35, 35, }, // Easter
    {   45,  619,       0,   59822,       0,  0, 14,  0, }, // Ecuador
    {   45,  627,   59836,   59857,   59887, 21, 30, 30, }, // Europe_Central
    {   45,  642,   59917,   59939,   59970, 22, 31, 31, }, // Europe_Eastern
    {   45,  657,       0,   60001,       0,  0, 30,  0, }, // Europe_Further_Eastern
    {   45,  680,   60031,   60055,   60088, 24, 33, 33, }, // Europe_Western
    {   45,  695,   60121,   60148,   60184, 27, 36, 36, }, // Falkland
    {   45,  704,   60220,   60232,   60253, 12, 21, 21, }, // Fiji
    {   45,  709,       0,   60274,       0,  0, 26,  0, }, // French_Guiana
    {   45,  723,       0,   60300,       0,  0, 34,  0, }, // French_Southern
    {   45,  746,       0,   60334,       0,  0, 17,  0, }, // Galapagos
    {   45,  756,       0,   60351,       0,  0, 15,  0, }, // Gambier
    {   45,  764,   60366,   60380,   60403, 14, 23, 23, }, // Georgia
    {   45,  772,       0,   60426,       0,  0, 26,  0, }, // Gilbert_Islands
    {   45,  788,       0,   60452,       0,  0, 23,  0, }, // GMT
    {   45,  830,   60475,   60503,   60540, 28, 37, 37, }, // Greenland_Eastern
    {   45,  848,   60577,   60607,   60646, 30, 39, 39, }, // Greenland_Western
    {   45,  866,       0,   60685,       0,  0, 21,  0, }, // Guam
    {   45,  871,       0,   60706,       0,  0, 23,  0, }, // Gulf
    {   45,  876,       0,   60274,       0,  0, 17,  0, }, // Guyana
    {   45,  883,   60729,   60754,   60788, 25, 34, 34, }, // Hawaii_Aleutian
    {   45,  899,   60822,   60839,   60865, 17, 26, 26, }, // Hong_Kong
    {   45,  909,   60891,   60903,   60924, 12, 21, 21, }, // Hovd
    {   45,  914,       0,   60945,       0,  0, 25,  0, }, // India
    {   45,  920,       0,   60970,       0,  0, 22,  0, }, // Indian_Ocean
    {   45,  933,       0,   60992,       0,  0, 16,  0, }, // Indochina
    {   45,  943,       0,   61008,       0,  0, 24,  0, }, // Indonesia_Central
    {   45,  961,       0,   61032,       0,  0, 25,  0, }, // Indonesia_Eastern
    {   45,  979,       0,   61057,       0,  0, 26,  0, }, // Indonesia_Western
    {   45,  997,   61083,   61094,   61114, 11, 20, 20, }, // Iran
    {   45, 1008,   61134,   61148,   61171, 14, 23, 23, }, // Irkutsk
    {   45, 1016,   61194,   61207,   61229, 13, 22, 22, }, // Israel
    {   45, 1023,   61251,   61264,   61286, 13, 22, 22, }, // Japan
    {   45, 1029,   61308,   61340,   61381, 32, 41, 41, }, // Kamchatka
    {   45, 1058,       0,   61422,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {   45, 1077,       0,   61449,       0,  0, 29,  0, }, // Kazakhstan_Western
    {   45, 1106,   61478,   61491,   61513, 13, 22, 22, }, // Korea
    {   45, 1112,       0,   61535,       0,  0, 14,  0, }, // Kosrae
    {   45, 1119,   61549,   61568,   61596, 19, 28, 28, }, // Krasnoyarsk
    {   45, 1151,       0,   61624,       0,  0, 19,  0, }, // Kyrgystan
    {   45, 1161,       0,   61643,       0,  0, 13,  0, }, // Lanka
    {   45, 1175,       0,   61656,       0,  0, 23,  0, }, // Line_Islands
    {   45, 1188,   61679,   61696,   61722, 17, 26, 26, }, // Lord_Howe
    {   45, 1198,   61748,   61761,   61783, 13, 22, 22, }, // Macau
    {   45, 1204,   61805,   61820,   61844, 15, 24, 24, }, // Magadan
    {   45, 1219,       0,   61868,       0,  0, 15,  0, }, // Malaysia
    {   45, 1228,       0,   61883,       0,  0, 20,  0, }, // Maldives
    {   45, 1237,       0,   61903,       0,  0, 21,  0, }, // Marquesas
    {   45, 1247,       0,   61924,       0,  0, 27,  0, }, // Marshall_Islands
    {   45, 1264,   61951,   61967,   61992, 16, 25, 25, }, // Mauritius
    {   45, 1274,       0,   62017,       0,  0, 14,  0, }, // Mawson
    {   45, 1281,   62031,   62058,   62094, 27, 36, 36, }, // Mexico_Pacific
    {   45, 1296,   62130,   62147,   62173, 17, 26, 26, }, // Mongolia
    {   45, 1305,   62199,   62212,   62234, 13, 22, 22, }, // Moscow
    {   45, 1312,       0,   62256,       0,  0, 15,  0, }, // Myanmar
    {   45, 1320,       0,   62271,       0,  0, 13,  0, }, // Nauru
    {   45, 1326,       0,   62284,       0,  0, 14,  0, }, // Nepal
    {   45, 1332,   62298,   62321,   62353, 23, 32, 32, }, // New_Caledonia
    {   45, 1346,   62385,   62406,   62436, 21, 30, 30, }, // New_Zealand
    {   45, 1358,   62466,   62486,   62515, 20, 29, 29, }, // Newfoundland
    {   45, 1371,       0,   62544,       0,  0, 12,  0, }, // Niue
    {   45, 1376,   62556,   62580,   62613, 24, 33, 33, }, // Norfolk
    {   45, 1384,   62646,   62673,   62709, 27, 36, 36, }, // Noronha
    {   45, 1392,       0,   62745,       0,  0, 37,  0, }, // North_Mariana
    {   45, 1406,   62782,   62801,   62829, 19, 28, 28, }, // Novosibirsk
    {   45, 1418,   62857,   62868,   62888, 11, 20, 20, }, // Omsk
    {   45, 1428,   62908,   62926,   62953, 18, 27, 27, }, // Pakistan
    {   45, 1437,       0,   62980,       0,  0, 13,  0, }, // Palau
    {   45, 1443,       0,   62993,       0,  0, 26,  0, }, // Papua_New_Guinea
    {   45, 1460,   63019,   63036,   63062, 17, 26, 26, }, // Paraguay
    {   45, 1469,   63088,   63101,   63123, 13, 22, 22, }, // Peru
    {   45, 1474,   63145,   63162,   63188, 17, 26, 26, }, // Philippines
    {   45, 1486,       0,   63214,       0,  0, 26,  0, }, // Phoenix_Islands
    {   45, 1502,   63240,   63271,   63311, 31, 40, 40, }, // Pierre_Miquelon
    {   45, 1518,       0,   63351,       0,  0, 16,  0, }, // Pitcairn
    {   45, 1527,       0,   63367,       0,  0, 14,  0, }, // Ponape
    {   45, 1534,       0,   63381,       0,  0, 17,  0, }, // Pyongyang
    {   45, 1544,   63398,   63415,   63441, 17, 26, 26, }, // Qyzylorda
    {   45, 1554,       0,   63467,       0,  0, 15,  0, }, // Reunion
    {   45, 1562,       0,   63482,       0,  0, 15,  0, }, // Rothera
    {   45, 1570,   63497,   63512,   63536, 15, 24, 24, }, // Sakhalin
    {   45, 1579,   63560,   63574,   63597, 14, 23, 23, }, // Samara
    {   45, 1596,   63620,   63633,   63655, 13, 22, 22, }, // Samoa
    {   45, 1602,       0,   63677,       0,  0, 20,  0, }, // Seychelles
    {   45, 1624,       0,   63697,       0,  0, 25,  0, }, // Singapore
    {   45, 1634,       0,   63722,       0,  0, 26,  0, }, // Solomon
    {   45, 1642,       0,   63748,       0,  0, 22,  0, }, // South_Georgia
    {   45, 1656,       0,   63770,       0,  0, 16,  0, }, // Suriname
    {   45, 1676,       0,   63786,       0,  0, 13,  0, }, // Syowa
    {   45, 1682,       0,   63799,       0,  0, 14,  0, }, // Tahiti
    {   45, 1689,   63813,   63827,   63850, 14, 23, 23, }, // Taipei
    {   45, 1696,       0,   63873,       0,  0, 20,  0, }, // Tajikistan
    {   45, 1724,       0,   63893,       0,  0, 15,  0, }, // Tokelau
    {   45, 1732,   63908,   63921,   63943, 13, 22, 22, }, // Tonga
    {   45, 1738,       0,   63965,       0,  0, 13,  0, }, // Truk
    {   45, 1750,   63978,   63999,   64029, 21, 30, 30, }, // Turkmenistan
    {   45, 1763,       0,   64059,       0,  0, 14,  0, }, // Tuvalu
    {   45, 1777,   64073,   64089,   64114, 16, 25, 25, }, // Uruguay
    {   45, 1792,   64139,   64159,   64188, 20, 29, 29, }, // Uzbekistan
    {   45, 1803,   64217,   64232,   64256, 15, 24, 24, }, // Vanuatu
    {   45, 1811,       0,   64280,       0,  0, 17,  0, }, // Venezuela
    {   45, 1821,   64297,   64316,   64344, 19, 28, 28, }, // Vladivostok
    {   45, 1833,   64372,   64389,   64415, 17, 26, 26, }, // Volgograd
    {   45, 1843,       0,   64441,       0,  0, 14,  0, }, // Vostok
    {   45, 1850,       0,   64455,       0,  0, 21,  0, }, // Wake
    {   45, 1855,       0,   64476,       0,  0, 23,  0, }, // Wallis
    {   45, 1862,   64499,   64514,   64538, 15, 24, 24, }, // Yakutsk
    {   45, 1870,   64562,   64584,   64615, 22, 31, 31, }, // Yekaterinburg
    {   48,    5,       0,   64646,       0,  0, 16,  0, }, // Afghanistan Azerbaijani/Latin/Azerbaijan
    {   48,   17,       0,   64662,       0,  0, 20,  0, }, // Africa_Central
    {   48,   32,       0,   64682,       0,  0, 18,  0, }, // Africa_Eastern
    {   48,   65,       0,   64700,       0,  0, 19,  0, }, // Africa_Southern
    {   48,   81,   64719,   64737,   64764, 18, 27, 22, }, // Africa_Western
    {   48,  107,   64786,   64799,   64821, 13, 22, 17, }, // Alaska
    {   48,  135,   64838,   64850,   64871, 12, 21, 16, }, // Amazon
    {   48,  142,   64887,   64915,   64952, 28, 37, 32, }, // America_Central
    {   48,  158,   64984,   65010,   65045, 26, 35, 30, }, // America_Eastern
    {   48,  174,   65075,   65102,   65138, 27, 36, 31, }, // America_Mountain
    {   48,  191,   65169,   65201,   65242, 32, 41, 36, }, // America_Pacific
    {   48,  214,   65278,   65288,   65307, 10, 19, 14, }, // Apia
    {   48,  232,   65321,   65336,   65360, 15, 24, 19, }, // Arabian
    {   48,  240,   65379,   65394,   65418, 15, 24, 19, }, // Argentina
    {   48,  250,   65437,   65458,   65488, 21, 30, 25, }, // Argentina_Western
    {   48,  268,   65513,   65529,   65554, 16, 25, 20, }, // Armenia
    {   48,  286,   65574,   65587,   65609, 13, 22, 18, }, // Atlantic
    {   48,  295,   65627,   65651,   65684, 24, 33, 28, }, // Australia_Central
    {   48,  313,   65712,   65742,   65781, 30, 39, 34, }, // Australia_CentralWestern
    {   48,  338,   65815,   65837,   65868, 22, 31, 26, }, // Australia_Eastern
    {   48,  356,   65720,   65750,   65789, 22, 31, 26, }, // Australia_Western
    {   48,  374,   65894,   65910,   65935, 16, 25, 20, }, // Azerbaijan
    {   48,  385,   65955,   65965,   65984, 10, 19, 14, }, // Azores
    {   48,  397,   65998,   66013,   66037, 15, 24, 19, }, // Bangladesh
    {   48,  415,       0,   66056,       0,  0, 11,  0, }, // Bhutan
    {   48,  422,       0,   66067,       0,  0, 14,  0, }, // Bolivia
    {   48,  437,   66081,   66096,   66120, 15, 24, 19, }, // Brasilia
    {   48,  454,       0,   66139,       0,  0, 23,  0, }, // Brunei
    {   48,  461,   66162,   66178,   66203, 16, 25, 20, }, // Cape_Verde
    {   48,  478,       0,   66223,       0,  0, 13,  0, }, // Chamorro
    {   48,  487,   66236,   66248,   66269, 12, 21, 16, }, // Chatham
    {   48,  495,   66285,   66295,   66314, 10, 19, 14, }, // Chile
    {   48,  501,   66328,   66337,   66355,  9, 18, 13, }, // China
    {   48,  507,       0,   66368,       0,  0, 17,  0, }, // Christmas
    {   48,  517,       0,   66385,       0,  0, 19,  0, }, // Cocos
    {   48,  523,   66404,   66419,   66443, 15, 24, 19, }, // Colombia
    {   48,  532,   66462,   66479,   66505, 17, 26, 27, }, // Cook
    {   48,  537,   66532,   66542,   66561, 10, 19, 14, }, // Cuba
    {   48,  548,       0,   66575,       0,  0, 11,  0, }, // Davis
    {   48,  564,       0,   66586,       0,  0, 19,  0, }, // DumontDUrville
    {   48,  601,       0,   66605,       0,  0, 17,  0, }, // East_Timor
    {   48,  612,   66622,   66639,   66665, 17, 26, 21, }, // Easter
    {   48,  619,       0,   66686,       0,  0, 13,  0, }, // Ecuador
    {   48,  627,   66699,   66719,   66748, 20, 29, 24, }, // Europe_Central
    {   48,  642,   66772,   66790,   66817, 18, 27, 22, }, // Europe_Eastern
    {   48,  657,       0,   66839,       0,  0, 24,  0, }, // Europe_Further_Eastern
    {   48,  680,   66863,   66881,   66908, 18, 27, 22, }, // Europe_Western
    {   48,  695,   66930,   66952,   66983, 22, 31, 26, }, // Falkland
    {   48,  704,   67009,   67019,   67038, 10, 19, 14, }, // Fiji
    {   48,  709,       0,   67052,       0,  0, 22,  0, }, // French_Guiana
    {   48,  723,       0,   67074,       0,  0, 33,  0, }, // French_Southern
    {   48,  746,       0,   67107,       0,  0, 15,  0, }, // Galapagos
    {   48,  756,       0,   67122,       0,  0, 13,  0, }, // Gambier
    {   48,  764,   67135,   67150,   67174, 15, 24, 19, }, // Georgia
    {   48,  772,       0,   67193,       0,  0, 21,  0, }, // Gilbert_Islands
    {   48,  788,       0,   67214,       0,  0, 18,  0, }, // GMT
    {   48,  830,   67232,   67255,   67287, 23, 32, 27, }, // Greenland_Eastern
    {   48,  848,   67314,   67337,   67369, 23, 32, 27, }, // Greenland_Western
    {   48,  871,       0,   67396,       0,  0, 12,  0, }, // Gulf
    {   48,  876,       0,   67408,       0,  0, 12,  0, }, // Guyana
    {   48,  883,   67420,   67437,   67463, 17, 26, 21, }, // Hawaii_Aleutian
    {   48,  899,   67484,   67499,   67523, 15, 24, 19, }, // Hong_Kong
    {   48,  909,   67542,   67552,   67571, 10, 19, 14, }, // Hovd
    {   48,  914,       0,   67585,       0,  0, 15,  0, }, // India
    {   48,  920,       0,   67600,       0,  0, 17,  0, }, // Indian_Ocean
    {   48,  933,       0,   67617,       0,  0, 13,  0, }, // Indochina
    {   48,  943,       0,   67630,       0,  0, 24,  0, }, // Indonesia_Central
    {   48,  961,       0,   67654,       0,  0, 22,  0, }, // Indonesia_Eastern
    {   48,  979,       0,   67676,       0,  0, 22,  0, }, // Indonesia_Western
    {   48,  997,   67698,   67708,   67727, 10, 19, 14, }, // Iran
    {   48, 1008,   67741,   67754,   67776, 13, 22, 17, }, // Irkutsk
    {   48, 1016,   67793,   67805,   67826, 12, 21, 16, }, // Israel
    {   48, 1023,   67842,   67856,   67879, 14, 23, 18, }, // Japan
    {   48, 1047,       0,   67897,       0,  0, 16,  0, }, // Kazakhstan
    {   48, 1058,       0,   67913,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {   48, 1077,       0,   67935,       0,  0, 22,  0, }, // Kazakhstan_Western
    {   48, 1106,   67957,   67969,   67990, 12, 21, 16, }, // Korea
    {   48, 1112,       0,   68006,       0,  0, 11,  0, }, // Kosrae
    {   48, 1119,   68017,   68034,   68060, 17, 26, 21, }, // Krasnoyarsk
    {   48, 1151,       0,   68081,       0,  0, 17,  0, }, // Kyrgystan
    {   48, 1175,       0,   68098,       0,  0, 18,  0, }, // Line_Islands
    {   48, 1188,   68116,   68130,   68153, 14, 23, 18, }, // Lord_Howe
    {   48, 1204,   68171,   68184,   68206, 13, 22, 17, }, // Magadan
    {   48, 1219,       0,   68223,       0,  0, 15,  0, }, // Malaysia
    {   48, 1228,       0,   68238,       0,  0, 12,  0, }, // Maldives
    {   48, 1237,       0,   68250,       0,  0, 14,  0, }, // Marquesas
    {   48, 1247,       0,   68264,       0,  0, 20,  0, }, // Marshall_Islands
    {   48, 1264,   68284,   68297,   68319, 13, 22, 17, }, // Mauritius
    {   48, 1274,       0,   68336,       0,  0, 12,  0, }, // Mawson
    {   48, 1281,   68348,   68373,   68407, 25, 34, 29, }, // Mexico_Pacific
    {   48, 1296,   68436,   68451,   68475, 15, 24, 19, }, // Mongolia
    {   48, 1305,   68494,   68506,   68527, 12, 21, 16, }, // Moscow
    {   48, 1312,       0,   68543,       0,  0, 12,  0, }, // Myanmar
    {   48, 1320,       0,   68555,       0,  0, 11,  0, }, // Nauru
    {   48, 1326,       0,   68566,       0,  0, 11,  0, }, // Nepal
    {   48, 1332,   68577,   68598,   68628, 21, 30, 25, }, // New_Caledonia
    {   48, 1346,   68653,   68673,   68702, 20, 29, 24, }, // New_Zealand
    {   48, 1358,   68726,   68744,   68771, 18, 27, 22, }, // Newfoundland
    {   48, 1371,       0,   68793,       0,  0, 10,  0, }, // Niue
    {   48, 1376,   68803,   68822,   68850, 19, 28, 23, }, // Norfolk
    {   48, 1384,   68873,   68898,   68932, 25, 34, 29, }, // Noronha
    {   48, 1406,   68961,   68978,   69004, 17, 26, 21, }, // Novosibirsk
    {   48, 1418,   69025,   69035,   69054, 10, 19, 14, }, // Omsk
    {   48, 1428,   69068,   69082,   69105, 14, 23, 18, }, // Pakistan
    {   48, 1437,       0,   69123,       0,  0, 11,  0, }, // Palau
    {   48, 1443,       0,   69134,       0,  0, 24,  0, }, // Papua_New_Guinea
    {   48, 1460,   69158,   69172,   69195, 14, 23, 18, }, // Paraguay
    {   48, 1469,   69213,   69223,   69242, 10, 19, 14, }, // Peru
    {   48, 1474,   69256,   69270,   69293, 14, 23, 18, }, // Philippines
    {   48, 1486,       0,   69311,       0,  0, 20,  0, }, // Phoenix_Islands
    {   48, 1502,   69331,   69361,   69400, 30, 39, 34, }, // Pierre_Miquelon
    {   48, 1518,       0,   69434,       0,  0, 13,  0, }, // Pitcairn
    {   48, 1527,       0,   69447,       0,  0, 12,  0, }, // Ponape
    {   48, 1534,       0,   69459,       0,  0, 13,  0, }, // Pyongyang
    {   48, 1554,       0,   69472,       0,  0,  8,  0, }, // Reunion
    {   48, 1562,       0,   69480,       0,  0, 12,  0, }, // Rothera
    {   48, 1570,   69492,   69505,   69527, 13, 22, 17, }, // Sakhalin
    {   48, 1579,   69544,   69556,   69577, 12, 21, 16, }, // Samara
    {   48, 1596,   69593,   69604,   69624, 11, 20, 15, }, // Samoa
    {   48, 1602,       0,   69639,       0,  0, 20,  0, }, // Seychelles
    {   48, 1624,       0,   69659,       0,  0, 14,  0, }, // Singapore
    {   48, 1634,       0,   69673,       0,  0, 21,  0, }, // Solomon
    {   48, 1642,       0,   69694,       0,  0, 20,  0, }, // South_Georgia
    {   48, 1656,       0,   69714,       0,  0, 13,  0, }, // Suriname
    {   48, 1676,       0,   69727,       0,  0, 11,  0, }, // Syowa
    {   48, 1682,       0,   69738,       0,  0, 12,  0, }, // Tahiti
    {   48, 1689,   69750,   69762,   69783, 12, 21, 16, }, // Taipei
    {   48, 1696,       0,   69799,       0,  0, 16,  0, }, // Tajikistan
    {   48, 1724,       0,   69815,       0,  0, 13,  0, }, // Tokelau
    {   48, 1732,   69828,   69839,   69859, 11, 20, 15, }, // Tonga
    {   48, 1738,       0,   69874,       0,  0, 10,  0, }, // Truk
    {   48, 1750,   69884,   69902,   69929, 18, 27, 22, }, // Turkmenistan
    {   48, 1763,       0,   69951,       0,  0, 12,  0, }, // Tuvalu
    {   48, 1777,   69963,   69976,   69998, 13, 22, 17, }, // Uruguay
    {   48, 1792,   70015,   70031,   70056, 16, 25, 20, }, // Uzbekistan
    {   48, 1803,   70076,   70089,   70111, 13, 22, 17, }, // Vanuatu
    {   48, 1811,       0,   70128,       0,  0, 15,  0, }, // Venezuela
    {   48, 1821,   70143,   70160,   70186, 17, 26, 21, }, // Vladivostok
    {   48, 1833,   70207,   70222,   70246, 15, 24, 19, }, // Volgograd
    {   48, 1843,       0,   70265,       0,  0, 12,  0, }, // Vostok
    {   48, 1850,       0,   70277,       0,  0, 10,  0, }, // Wake
    {   48, 1855,       0,   70287,       0,  0, 22,  0, }, // Wallis
    {   48, 1862,   70309,   70322,   70344, 13, 22, 17, }, // Yakutsk
    {   48, 1870,   70361,   70380,   70408, 19, 28, 23, }, // Yekaterinburg
    {   48, 1892,       0,   70431,       0,  0, 11,  0, }, // Yukon
    {   56,    0,   70442,   70450,   70463,  8, 13, 19, }, // Acre Bangla/Bangla/Bangladesh
    {   56,    5,       0,   70482,       0,  0, 16,  0, }, // Afghanistan
    {   56,   17,       0,   70498,       0,  0, 17,  0, }, // Africa_Central
    {   56,   32,       0,   70515,       0,  0, 18,  0, }, // Africa_Eastern
    {   56,   65,       0,   70533,       0,  0, 24,  0, }, // Africa_Southern
    {   56,   81,   70557,   70576,   70600, 19, 24, 32, }, // Africa_Western
    {   56,  107,   70632,   70644,   70661, 12, 17, 20, }, // Alaska
    {   56,  128,   70681,   70693,   70710, 12, 17, 23, }, // Almaty
    {   56,  135,   70733,   70743,   70758, 10, 15, 23, }, // Amazon
    {   56,  142,   70781,   70796,   70816, 15, 20, 23, }, // America_Central
    {   56,  158,   70839,   70857,   70879, 18, 22, 25, }, // America_Eastern
    {   56,  174,   70904,   70925,   70951, 21, 26, 27, }, // America_Mountain
    {   56,  191,   70978,   71010,   71047, 32, 37, 38, }, // America_Pacific
    {   56,  207,   71085,   71098,   71116, 13, 18, 26, }, // Anadyr
    {   56,  214,   71142,   71153,   71169, 11, 16, 17, }, // Apia
    {   56,  219,   71186,   71196,   71211, 10, 15, 21, }, // Aqtau
    {   56,  225,   71232,   71242,   71257, 10, 15, 21, }, // Aqtobe
    {   56,  232,   71278,   71287,   71301,  9, 14, 17, }, // Arabian
    {   56,  240,   71318,   71334,   71355, 16, 21, 29, }, // Argentina
    {   56,  250,   71384,   71408,   71439, 24, 31, 36, }, // Argentina_Western
    {   56,  268,   71475,   71490,   71510, 15, 20, 28, }, // Armenia
    {   56,  286,   71538,   71554,   71573, 16, 19, 22, }, // Atlantic
    {   56,  295,   71595,   71622,   71654, 27, 32, 35, }, // Australia_Central
    {   56,  313,   71689,   71724,   71764, 35, 40, 43, }, // Australia_CentralWestern
    {   56,  338,   71807,   71829,   71856, 22, 27, 30, }, // Australia_Eastern
    {   56,  356,   71886,   71910,   71939, 24, 29, 32, }, // Australia_Western
    {   56,  374,   71971,   71986,   72006, 15, 20, 28, }, // Azerbaijan
    {   56,  385,   72034,   72045,   72061, 11, 16, 24, }, // Azores
    {   56,  397,   72085,   72098,   72116, 13, 18, 26, }, // Bangladesh
    {   56,  415,       0,   72142,       0,  0, 10,  0, }, // Bhutan
    {   56,  422,       0,   72152,       0,  0, 14,  0, }, // Bolivia
    {   56,  437,   72166,   72182,   72203, 16, 21, 29, }, // Brasilia
    {   56,  454,       0,   72232,       0,  0, 23,  0, }, // Brunei
    {   56,  461,   72255,   72269,   72288, 14, 19, 27, }, // Cape_Verde
    {   56,  478,       0,   72315,       0,  0, 16,  0, }, // Chamorro
    {   56,  487,   72331,   72343,   72360, 12, 17, 20, }, // Chatham
    {   56,  495,   72380,   72389,   72403,  9, 14, 22, }, // Chile
    {   56,  501,   72425,   72433,   72446,  8, 13, 16, }, // China
    {   56,  507,       0,   72462,       0,  0, 19,  0, }, // Christmas
    {   56,  517,       0,   72481,       0,  0, 21,  0, }, // Cocos
    {   56,  523,   72502,   72518,   72539, 16, 21, 29, }, // Colombia
    {   56,  532,   72568,   72587,   72611, 19, 24, 39, }, // Cook
    {   56,  537,   72650,   72661,   72676, 11, 15, 18, }, // Cuba
    {   56,  548,       0,   72694,       0,  0, 10,  0, }, // Davis
    {   56,  564,       0,   72704,       0,  0, 22,  0, }, // DumontDUrville
    {   56,  601,       0,   72726,       0,  0, 15,  0, }, // East_Timor
    {   56,  612,   72741,   72758,   72780, 17, 22, 30, }, // Easter
    {   56,  619,       0,   72810,       0,  0, 13,  0, }, // Ecuador
    {   56,  627,   72823,   72841,   72864, 18, 23, 31, }, // Europe_Central
    {   56,  642,   72895,   72914,   72938, 19, 24, 32, }, // Europe_Eastern
    {   56,  657,       0,   72970,       0,  0, 30,  0, }, // Europe_Further_Eastern
    {   56,  680,   73000,   73020,   73045, 20, 25, 33, }, // Europe_Western
    {   56,  695,   73078,   73103,   73133, 25, 30, 38, }, // Falkland
    {   56,  704,   73171,   73180,   73194,  9, 14, 22, }, // Fiji
    {   56,  709,       0,   73216,       0,  0, 18,  0, }, // French_Guiana
    {   56,  723,       0,   73234,       0,  0, 33,  0, }, // French_Southern
    {   56,  746,       0,   73267,       0,  0, 14,  0, }, // Galapagos
    {   56,  756,       0,   73281,       0,  0, 17,  0, }, // Gambier
    {   56,  764,   73298,   73311,   73329, 13, 18, 26, }, // Georgia
    {   56,  772,       0,   73355,       0,  0, 24,  0, }, // Gilbert_Islands
    {   56,  788,       0,   73379,       0,  0, 16,  0, }, // GMT
    {   56,  830,   73395,   73418,   73446, 23, 28, 36, }, // Greenland_Eastern
    {   56,  848,   73482,   73506,   73535, 24, 29, 37, }, // Greenland_Western
    {   56,  866,       0,   73572,       0,  0, 15,  0, }, // Guam
    {   56,  871,       0,   73587,       0,  0, 19,  0, }, // Gulf
    {   56,  876,       0,   73606,       0,  0, 12,  0, }, // Guyana
    {   56,  883,   73618,   73636,   73659, 18, 23, 26, }, // Hawaii_Aleutian
    {   56,  899,   73685,   73695,   73710, 10, 15, 23, }, // Hong_Kong
    {   56,  909,   73733,   73742,   73756,  9, 14, 22, }, // Hovd
    {   56,  914,       0,   73778,       0,  0, 17,  0, }, // India
    {   56,  920,       0,   73795,       0,  0, 20,  0, }, // Indian_Ocean
    {   56,  933,       0,   73815,       0,  0, 13,  0, }, // Indochina
    {   56,  943,       0,   73828,       0,  0, 28,  0, }, // Indonesia_Central
    {   56,  961,       0,   73856,       0,  0, 23,  0, }, // Indonesia_Eastern
    {   56,  979,       0,   73879,       0,  0, 25,  0, }, // Indonesia_Western
    {   56,  997,   73904,   73913,   73927,  9, 14, 17, }, // Iran
    {   56, 1008,   73944,   73957,   73975, 13, 18, 26, }, // Irkutsk
    {   56, 1016,   74001,   74014,   74032, 13, 18, 21, }, // Israel
    {   56, 1023,   74053,   74063,   74078, 10, 15, 18, }, // Japan
    {   56, 1029,   74096,   74132,   74172, 36, 40, 47, }, // Kamchatka
    {   56, 1047,       0,   74219,       0,  0, 16,  0, }, // Kazakhstan
    {   56, 1058,       0,   74235,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {   56, 1077,       0,   74257,       0,  0, 23,  0, }, // Kazakhstan_Western
    {   56, 1106,   74280,   74293,   74311, 13, 18, 21, }, // Korea
    {   56, 1112,       0,   74332,       0,  0, 11,  0, }, // Kosrae
    {   56, 1119,   74343,   74364,   74390, 21, 26, 34, }, // Krasnoyarsk
    {   56, 1151,       0,   74424,       0,  0, 15,  0, }, // Kyrgystan
    {   56, 1161,       0,   74439,       0,  0, 10,  0, }, // Lanka
    {   56, 1175,       0,   74449,       0,  0, 20,  0, }, // Line_Islands
    {   56, 1188,   74469,   74485,   74506, 16, 21, 24, }, // Lord_Howe
    {   56, 1198,   74530,   74540,   74554, 10, 14, 21, }, // Macau
    {   56, 1204,   74575,   74589,   74608, 14, 19, 27, }, // Magadan
    {   56, 1219,       0,   74635,       0,  0, 16,  0, }, // Malaysia
    {   56, 1228,       0,   74651,       0,  0, 13,  0, }, // Maldives
    {   56, 1237,       0,   74664,       0,  0, 14,  0, }, // Marquesas
    {   56, 1247,       0,   74678,       0,  0, 23,  0, }, // Marshall_Islands
    {   56, 1264,   74701,   74712,   74728, 11, 16, 24, }, // Mauritius
    {   56, 1274,       0,   74752,       0,  0,  8,  0, }, // Mawson
    {   56, 1281,   74760,   74794,   74832, 34, 38, 42, }, // Mexico_Pacific
    {   56, 1296,   74874,   74889,   74909, 15, 20, 28, }, // Mongolia
    {   56, 1305,   74937,   74947,   74962, 10, 15, 23, }, // Moscow
    {   56, 1312,       0,   74985,       0,  0, 14,  0, }, // Myanmar
    {   56, 1320,       0,   74999,       0,  0, 10,  0, }, // Nauru
    {   56, 1326,       0,   75009,       0,  0, 10,  0, }, // Nepal
    {   56, 1332,   75019,   75041,   75068, 22, 27, 35, }, // New_Caledonia
    {   56, 1346,   75103,   75120,   75142, 17, 22, 25, }, // New_Zealand
    {   56, 1358,   75167,   75188,   75214, 21, 26, 29, }, // Newfoundland
    {   56, 1371,       0,   75243,       0,  0,  9,  0, }, // Niue
    {   56, 1376,   75252,   75268,   75289, 16, 21, 24, }, // Norfolk
    {   56, 1384,   75313,   75339,   75370, 26, 31, 39, }, // Noronha
    {   56, 1392,       0,   75409,       0,  0, 27,  0, }, // North_Mariana
    {   56, 1406,   75436,   75454,   75477, 18, 23, 31, }, // Novosibirsk
    {   56, 1418,   75508,   75518,   75533, 10, 15, 23, }, // Omsk
    {   56, 1428,   75556,   75570,   75589, 14, 19, 27, }, // Pakistan
    {   56, 1437,       0,   75616,       0,  0, 10,  0, }, // Palau
    {   56, 1443,       0,   75626,       0,  0, 21,  0, }, // Papua_New_Guinea
    {   56, 1460,   75647,   75663,   75684, 16, 21, 29, }, // Paraguay
    {   56, 1469,   75713,   75722,   75736,  9, 14, 22, }, // Peru
    {   56, 1474,   75758,   75771,   75789, 13, 18, 26, }, // Philippines
    {   56, 1486,       0,   75815,       0,  0, 23,  0, }, // Phoenix_Islands
    {   56, 1502,   75838,   75864,   75895, 26, 31, 34, }, // Pierre_Miquelon
    {   56, 1518,       0,   75929,       0,  0, 16,  0, }, // Pitcairn
    {   56, 1527,       0,   75945,       0,  0, 11,  0, }, // Ponape
    {   56, 1534,       0,   75956,       0,  0, 15,  0, }, // Pyongyang
    {   56, 1544,   75971,   75986,   76005, 15, 19, 26, }, // Qyzylorda
    {   56, 1554,       0,   76031,       0,  0, 14,  0, }, // Reunion
    {   56, 1562,       0,   76045,       0,  0, 10,  0, }, // Rothera
    {   56, 1570,   76055,   76067,   76084, 12, 17, 25, }, // Sakhalin
    {   56, 1579,   76109,   76120,   76135, 11, 15, 22, }, // Samara
    {   56, 1596,   76157,   76169,   76186, 12, 17, 20, }, // Samoa
    {   56, 1602,       0,   76206,       0,  0, 11,  0, }, // Seychelles
    {   56, 1624,       0,   76217,       0,  0, 19,  0, }, // Singapore
    {   56, 1634,       0,   76236,       0,  0, 21,  0, }, // Solomon
    {   56, 1642,       0,   76257,       0,  0, 20,  0, }, // South_Georgia
    {   56, 1656,       0,   76277,       0,  0, 12,  0, }, // Suriname
    {   56, 1676,       0,   76289,       0,  0, 14,  0, }, // Syowa
    {   56, 1682,       0,   76303,       0,  0, 11,  0, }, // Tahiti
    {   56, 1689,   76314,   76325,   76341, 11, 16, 19, }, // Taipei
    {   56, 1696,       0,   76360,       0,  0, 16,  0, }, // Tajikistan
    {   56, 1724,       0,   76376,       0,  0, 12,  0, }, // Tokelau
    {   56, 1732,   76388,   76399,   76415, 11, 16, 24, }, // Tonga
    {   56, 1738,       0,   76439,       0,  0,  8,  0, }, // Truk
    {   56, 1750,   76447,   76466,   76490, 19, 24, 32, }, // Turkmenistan
    {   56, 1763,       0,   76522,       0,  0, 11,  0, }, // Tuvalu
    {   56, 1777,   76533,   76546,   76564, 13, 18, 26, }, // Uruguay
    {   56, 1792,   76590,   76606,   76627, 16, 21, 29, }, // Uzbekistan
    {   56, 1803,   76656,   76670,   76689, 14, 19, 27, }, // Vanuatu
    {   56, 1811,       0,   76716,       0,  0, 16,  0, }, // Venezuela
    {   56, 1821,   76732,   76748,   76769, 16, 21, 29, }, // Vladivostok
    {   56, 1833,   76798,   76812,   76831, 14, 19, 27, }, // Volgograd
    {   56, 1843,       0,   76858,       0,  0,  9,  0, }, // Vostok
    {   56, 1850,       0,   76867,       0,  0, 16,  0, }, // Wake
    {   56, 1855,       0,   76883,       0,  0, 23,  0, }, // Wallis
    {   56, 1862,   76906,   76922,   76943, 16, 21, 29, }, // Yakutsk
    {   56, 1870,   76972,   76993,   77019, 21, 26, 34, }, // Yekaterinburg
    {   56, 1892,       0,   77053,       0,  0, 10,  0, }, // Yukon
    {   60,    0,   77063,   77075,   77097, 12, 22, 18, }, // Acre Basque/Latin/Spain
    {   60,    5,       0,   77115,       0,  0, 18,  0, }, // Afghanistan
    {   60,   17,       0,   77133,       0,  0, 25,  0, }, // Africa_Central
    {   60,   32,       0,   77158,       0,  0, 24,  0, }, // Africa_Eastern
    {   60,   65,       0,   77182,       0,  0, 25,  0, }, // Africa_Southern
    {   60,   81,   77207,   77234,   77271, 27, 37, 33, }, // Africa_Western
    {   60,  107,   77304,   77318,   77342, 14, 24, 20, }, // Alaska
    {   60,  128,   77362,   77376,   77400, 14, 24, 20, }, // Almaty
    {   60,  135,   77420,   77436,   77462, 16, 26, 22, }, // Amazon
    {   60,  142,   77484,   77515,   77556, 31, 41, 37, }, // America_Central
    {   60,  158,   77593,   77623,   77663, 30, 40, 36, }, // America_Eastern
    {   60,  174,   77699,   77731,   77773, 32, 42, 38, }, // America_Mountain
    {   60,  191,   77811,   77842,   77883, 31, 41, 37, }, // America_Pacific
    {   60,  207,   77920,   77936,   77962, 16, 26, 22, }, // Anadyr
    {   60,  214,   77984,   77996,   78018, 12, 22, 18, }, // Apia
    {   60,  219,   78036,   78049,   78072, 13, 23, 19, }, // Aqtau
    {   60,  225,   78091,   78105,   78129, 14, 24, 20, }, // Aqtobe
    {   60,  232,   78149,   78163,   78187, 14, 24, 20, }, // Arabian
    {   60,  240,   78207,   78224,   78251, 17, 27, 23, }, // Argentina
    {   60,  250,   78274,   78302,   78340, 28, 38, 34, }, // Argentina_Western
    {   60,  268,   78374,   78389,   78414, 15, 25, 21, }, // Armenia
    {   60,  286,   78435,   78467,   78509, 32, 42, 38, }, // Atlantic
    {   60,  295,   78547,   78575,   78613, 28, 38, 34, }, // Australia_Central
    {   60,  313,   78647,   78682,   78727, 35, 45, 41, }, // Australia_CentralWestern
    {   60,  338,   78768,   78795,   78832, 27, 37, 33, }, // Australia_Eastern
    {   60,  356,   78865,   78895,   78935, 30, 40, 36, }, // Australia_Western
    {   60,  374,   78971,   78989,   79017, 18, 28, 24, }, // Azerbaijan
    {   60,  385,   79041,   79057,   79083, 16, 26, 22, }, // Azores
    {   60,  397,   79105,   79124,   79153, 19, 29, 25, }, // Bangladesh
    {   60,  415,       0,   79178,       0,  0, 14,  0, }, // Bhutan
    {   60,  422,       0,   79192,       0,  0, 15,  0, }, // Bolivia
    {   60,  437,   79207,   79223,   79249, 16, 26, 22, }, // Brasilia
    {   60,  454,       0,   79271,       0,  0, 25,  0, }, // Brunei
    {   60,  461,   79296,   79314,   79342, 18, 28, 24, }, // Cape_Verde
    {   60,  472,       0,   79366,       0,  0, 13,  0, }, // Casey
    {   60,  478,       0,   79379,       0,  0, 26,  0, }, // Chamorro
    {   60,  487,   79405,   79420,   79445, 15, 25, 21, }, // Chatham
    {   60,  495,   79466,   79479,   79502, 13, 23, 19, }, // Chile
    {   60,  501,   79521,   79534,   79557, 13, 23, 19, }, // China
    {   60,  507,       0,   79576,       0,  0, 24,  0, }, // Christmas
    {   60,  517,       0,   79600,       0,  0, 23,  0, }, // Cocos
    {   60,  523,   79623,   79639,   79665, 16, 26, 22, }, // Colombia
    {   60,  532,   79687,   79709,   79741, 22, 32, 37, }, // Cook
    {   60,  537,   79778,   79790,   79812, 12, 22, 18, }, // Cuba
    {   60,  548,       0,   79830,       0,  0, 14,  0, }, // Davis
    {   60,  564,       0,   79844,       0,  0, 24,  0, }, // DumontDUrville
    {   60,  601,       0,   79868,       0,  0, 25,  0, }, // East_Timor
    {   60,  612,   79893,   79913,   79943, 20, 30, 26, }, // Easter
    {   60,  619,       0,   79969,       0,  0, 17,  0, }, // Ecuador
    {   60,  627,   79986,   80011,   80046, 25, 35, 31, }, // Europe_Central
    {   60,  642,   80077,   80101,   80135, 24, 34, 30, }, // Europe_Eastern
    {   60,  657,       0,   80165,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {   60,  680,   80196,   80223,   80260, 27, 37, 33, }, // Europe_Western
    {   60,  695,   80293,   80319,   80355, 26, 36, 32, }, // Falkland
    {   60,  704,   80387,   80399,   80421, 12, 22, 18, }, // Fiji
    {   60,  709,       0,   80439,       0,  0, 24,  0, }, // French_Guiana
    {   60,  723,       0,   80463,       0,  0, 57,  0, }, // French_Southern
    {   60,  746,       0,   80520,       0,  0, 19,  0, }, // Galapagos
    {   60,  756,       0,   80539,       0,  0, 19,  0, }, // Gambier
    {   60,  764,   80558,   80573,   80598, 15, 25, 21, }, // Georgia
    {   60,  772,       0,   80619,       0,  0, 25,  0, }, // Gilbert_Islands
    {   60,  788,       0,   80644,       0,  0, 31,  0, }, // GMT
    {   60,  830,   80675,   80704,   80743, 29, 39, 35, }, // Greenland_Eastern
    {   60,  848,   80778,   80810,   80852, 32, 42, 38, }, // Greenland_Western
    {   60,  866,       0,   80890,       0,  0, 23,  0, }, // Guam
    {   60,  871,       0,   80913,       0,  0, 23,  0, }, // Gulf
    {   60,  876,       0,   80936,       0,  0, 14,  0, }, // Guyana
    {   60,  883,   80950,   80983,   81026, 33, 43, 39, }, // Hawaii_Aleutian
    {   60,  899,   81065,   81081,   81107, 16, 26, 22, }, // Hong_Kong
    {   60,  909,   81129,   81143,   81167, 14, 24, 20, }, // Hovd
    {   60,  914,       0,   81187,       0,  0, 13,  0, }, // India
    {   60,  920,       0,   81200,       0,  0, 22,  0, }, // Indian_Ocean
    {   60,  933,       0,   81222,       0,  0, 17,  0, }, // Indochina
    {   60,  943,       0,   81239,       0,  0, 28,  0, }, // Indonesia_Central
    {   60,  961,       0,   81267,       0,  0, 27,  0, }, // Indonesia_Eastern
    {   60,  979,       0,   81294,       0,  0, 30,  0, }, // Indonesia_Western
    {   60,  997,   81324,   81336,   81358, 12, 22, 18, }, // Iran
    {   60, 1008,   81376,   81392,   81418, 16, 26, 22, }, // Irkutsk
    {   60, 1016,   81440,   81454,   81478, 14, 24, 20, }, // Israel
    {   60, 1023,   81498,   81513,   81538, 15, 25, 21, }, // Japan
    {   60, 1029,   81559,   81591,   81633, 32, 42, 38, }, // Kamchatka
    {   60, 1047,       0,   81671,       0,  0, 18,  0, }, // Kazakhstan
    {   60, 1058,       0,   81689,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {   60, 1077,       0,   81717,       0,  0, 31,  0, }, // Kazakhstan_Western
    {   60, 1106,   81748,   81761,   81784, 13, 23, 19, }, // Korea
    {   60, 1112,       0,   81803,       0,  0, 14,  0, }, // Kosrae
    {   60, 1119,   81817,   81837,   81867, 20, 30, 26, }, // Krasnoyarsk
    {   60, 1151,       0,   81893,       0,  0, 19,  0, }, // Kyrgystan
    {   60, 1161,       0,   81912,       0,  0, 13,  0, }, // Lanka
    {   60, 1175,       0,   81925,       0,  0, 22,  0, }, // Line_Islands
    {   60, 1188,   81947,   81964,   81991, 17, 27, 23, }, // Lord_Howe
    {   60, 1198,   82014,   82027,   82050, 13, 23, 19, }, // Macau
    {   60, 1204,   82069,   82084,   82109, 15, 25, 21, }, // Magadan
    {   60, 1219,       0,   82130,       0,  0, 16,  0, }, // Malaysia
    {   60, 1228,       0,   82146,       0,  0, 17,  0, }, // Maldives
    {   60, 1237,       0,   82163,       0,  0, 17,  0, }, // Marquesas
    {   60, 1247,       0,   82180,       0,  0, 26,  0, }, // Marshall_Islands
    {   60, 1264,   82206,   82222,   82248, 16, 26, 22, }, // Mauritius
    {   60, 1274,       0,   82270,       0,  0, 15,  0, }, // Mawson
    {   60, 1281,   82285,   82310,   82345, 25, 35, 31, }, // Mexico_Pacific
    {   60, 1296,   82376,   82396,   82426, 20, 30, 26, }, // Mongolia
    {   60, 1305,   82452,   82465,   82488, 13, 23, 19, }, // Moscow
    {   60, 1312,       0,   82507,       0,  0, 17,  0, }, // Myanmar
    {   60, 1320,       0,   82524,       0,  0, 13,  0, }, // Nauru
    {   60, 1326,       0,   82537,       0,  0, 13,  0, }, // Nepal
    {   60, 1332,   82550,   82573,   82606, 23, 33, 29, }, // New_Caledonia
    {   60, 1346,   82635,   82657,   82689, 22, 32, 28, }, // New_Zealand
    {   60, 1358,   82717,   82731,   82755, 14, 24, 20, }, // Newfoundland
    {   60, 1371,       0,   82775,       0,  0, 12,  0, }, // Niue
    {   60, 1376,   82787,   82812,   82847, 25, 35, 31, }, // Norfolk
    {   60, 1384,   82878,   82905,   82942, 27, 37, 33, }, // Noronha
    {   60, 1392,       0,   82975,       0,  0, 30,  0, }, // North_Mariana
    {   60, 1406,   83005,   83025,   83055, 20, 30, 26, }, // Novosibirsk
    {   60, 1418,   83081,   83094,   83117, 13, 23, 19, }, // Omsk
    {   60, 1428,   83136,   83152,   83178, 16, 26, 22, }, // Pakistan
    {   60, 1437,       0,   83200,       0,  0, 13,  0, }, // Palau
    {   60, 1443,       0,   83213,       0,  0, 25,  0, }, // Papua_New_Guinea
    {   60, 1460,   83238,   83254,   83280, 16, 26, 22, }, // Paraguay
    {   60, 1469,   83302,   83314,   83336, 12, 22, 18, }, // Peru
    {   60, 1474,   83354,   83372,   83400, 18, 28, 24, }, // Philippines
    {   60, 1486,       0,   83424,       0,  0, 25,  0, }, // Phoenix_Islands
    {   60, 1502,   83449,   83482,   83525, 33, 43, 39, }, // Pierre_Miquelon
    {   60, 1518,       0,   83564,       0,  0, 17,  0, }, // Pitcairn
    {   60, 1527,       0,   83581,       0,  0, 14,  0, }, // Ponape
    {   60, 1534,       0,   83595,       0,  0, 18,  0, }, // Pyongyang
    {   60, 1544,   83613,   83630,   83657, 17, 27, 23, }, // Qyzylorda
    {   60, 1554,       0,   83680,       0,  0, 16,  0, }, // Reunion
    {   60, 1562,       0,   83696,       0,  0, 15,  0, }, // Rothera
    {   60, 1570,   83711,   83727,   83753, 16, 26, 22, }, // Sakhalin
    {   60, 1579,   83775,   83789,   83813, 14, 24, 20, }, // Samara
    {   60, 1596,   83833,   83846,   83869, 13, 23, 19, }, // Samoa
    {   60, 1602,       0,   83888,       0,  0, 27,  0, }, // Seychelles
    {   60, 1624,       0,   83915,       0,  0, 28,  0, }, // Singapore
    {   60, 1634,       0,   83943,       0,  0, 25,  0, }, // Solomon
    {   60, 1642,       0,   83968,       0,  0, 28,  0, }, // South_Georgia
    {   60, 1656,       0,   83996,       0,  0, 15,  0, }, // Suriname
    {   60, 1676,       0,   84011,       0,  0, 13,  0, }, // Syowa
    {   60, 1682,       0,   84024,       0,  0, 14,  0, }, // Tahiti
    {   60, 1689,   84038,   84052,   84076, 14, 24, 20, }, // Taipei
    {   60, 1696,       0,   84096,       0,  0, 19,  0, }, // Tajikistan
    {   60, 1724,       0,   84115,       0,  0, 15,  0, }, // Tokelau
    {   60, 1732,   84130,   84143,   84166, 13, 23, 19, }, // Tonga
    {   60, 1738,       0,   84185,       0,  0, 14,  0, }, // Truk
    {   60, 1750,   84199,   84219,   84249, 20, 30, 26, }, // Turkmenistan
    {   60, 1763,       0,   84275,       0,  0, 14,  0, }, // Tuvalu
    {   60, 1777,   84289,   84304,   84329, 15, 25, 21, }, // Uruguay
    {   60, 1792,   84350,   84368,   84396, 18, 28, 24, }, // Uzbekistan
    {   60, 1803,   84420,   84435,   84460, 15, 25, 21, }, // Vanuatu
    {   60, 1811,       0,   84481,       0,  0, 17,  0, }, // Venezuela
    {   60, 1821,   84498,   84518,   84548, 20, 30, 26, }, // Vladivostok
    {   60, 1833,   84574,   84592,   84620, 18, 28, 24, }, // Volgograd
    {   60, 1843,       0,   84644,       0,  0, 15,  0, }, // Vostok
    {   60, 1850,       0,   84659,       0,  0, 19,  0, }, // Wake
    {   60, 1855,       0,   84678,       0,  0, 29,  0, }, // Wallis
    {   60, 1862,   84707,   84723,   84749, 16, 26, 22, }, // Yakutsk
    {   60, 1870,   84771,   84793,   84825, 22, 32, 28, }, // Yekaterinburg
    {   60, 1892,       0,   84853,       0,  0, 13,  0, }, // Yukon
    {   61,    5,       0,   84866,       0,  0, 17,  0, }, // Afghanistan Belarusian/Cyrillic/Belarus
    {   61,   17,       0,   84883,       0,  0, 24,  0, }, // Africa_Central
    {   61,   32,       0,   84907,       0,  0, 21,  0, }, // Africa_Eastern
    {   61,   65,       0,   84928,       0,  0, 22,  0, }, // Africa_Southern
    {   61,   81,   84950,   84971,   85003, 21, 32, 27, }, // Africa_Western
    {   61,  107,   85030,   85040,   85061, 10, 21, 16, }, // Alaska
    {   61,  135,   85077,   85090,   85114, 13, 24, 19, }, // Amazon
    {   61,  142,   85133,   85167,   85212, 34, 45, 40, }, // America_Central
    {   61,  158,   85252,   85283,   85325, 31, 42, 37, }, // America_Eastern
    {   61,  174,   85362,   85391,   85431, 29, 40, 35, }, // America_Mountain
    {   61,  191,   85466,   85482,   85509, 16, 27, 22, }, // America_Pacific
    {   61,  214,   85531,   85539,   85558,  8, 19, 14, }, // Apia
    {   61,  232,   85572,   85593,   85625, 21, 32, 27, }, // Arabian
    {   61,  240,   85652,   85667,   85693, 15, 26, 21, }, // Argentina
    {   61,  250,   85714,   85736,   85769, 22, 33, 28, }, // Argentina_Western
    {   61,  268,   85797,   85808,   85830, 11, 22, 17, }, // Armenia
    {   61,  286,   85847,   85861,   85886, 14, 25, 20, }, // Atlantic
    {   61,  295,   85906,   85931,   85967, 25, 36, 31, }, // Australia_Central
    {   61,  313,   85998,   86030,   86073, 32, 43, 38, }, // Australia_CentralWestern
    {   61,  338,   86111,   86133,   86166, 22, 33, 28, }, // Australia_Eastern
    {   61,  356,   86194,   86216,   86249, 22, 33, 28, }, // Australia_Western
    {   61,  374,   86277,   86293,   86320, 16, 27, 22, }, // Azerbaijan
    {   61,  385,   86342,   86363,   86395, 21, 32, 27, }, // Azores
    {   61,  397,   86422,   86435,   86459, 13, 24, 19, }, // Bangladesh
    {   61,  415,       0,   86478,       0,  0, 10,  0, }, // Bhutan
    {   61,  422,       0,   86488,       0,  0, 14,  0, }, // Bolivia
    {   61,  437,   86502,   86516,   86541, 14, 25, 20, }, // Brasilia
    {   61,  454,       0,   86561,       0,  0, 10,  0, }, // Brunei
    {   61,  461,   86571,   86585,   86610, 14, 25, 20, }, // Cape_Verde
    {   61,  478,       0,   86630,       0,  0, 10,  0, }, // Chamorro
    {   61,  487,   86640,   86650,   86671, 10, 21, 16, }, // Chatham
    {   61,  495,   86687,   86699,   86722, 12, 23, 18, }, // Chile
    {   61,  501,   86740,   86749,   86769,  9, 20, 15, }, // China
    {   61,  507,       0,   86784,       0,  0, 18,  0, }, // Christmas
    {   61,  517,       0,   86802,       0,  0, 22,  0, }, // Cocos
    {   61,  523,   86824,   86839,   86865, 15, 26, 21, }, // Colombia
    {   61,  532,   86886,   86903,   86931, 17, 28, 26, }, // Cook
    {   61,  537,   86957,   86965,   86984,  8, 19, 14, }, // Cuba
    {   61,  548,       0,   86998,       0,  0, 18,  0, }, // Davis
    {   61,  564,       0,   87016,       0,  0, 27,  0, }, // DumontDUrville
    {   61,  601,       0,   87043,       0,  0, 20,  0, }, // East_Timor
    {   61,  612,   87063,   87085,   87118, 22, 33, 28, }, // Easter
    {   61,  619,       0,   87146,       0,  0, 14,  0, }, // Ecuador
    {   61,  627,   87160,   87184,   87219, 24, 35, 30, }, // Europe_Central
    {   61,  642,   87249,   87270,   87302, 21, 32, 27, }, // Europe_Eastern
    {   61,  657,       0,   87329,       0,  0, 27,  0, }, // Europe_Further_Eastern
    {   61,  680,   87356,   87377,   87409, 21, 32, 27, }, // Europe_Western
    {   61,  695,   87436,   87461,   87497, 25, 36, 31, }, // Falkland
    {   61,  704,   87528,   87537,   87557,  9, 20, 15, }, // Fiji
    {   61,  709,       0,   87572,       0,  0, 22,  0, }, // French_Guiana
    {   61,  723,       0,   87594,       0,  0, 47,  0, }, // French_Southern
    {   61,  746,       0,   87641,       0,  0, 36,  0, }, // Galapagos
    {   61,  756,       0,   87677,       0,  0, 19,  0, }, // Gambier
    {   61,  764,   87696,   87709,   87733, 13, 24, 19, }, // Georgia
    {   61,  772,       0,   87752,       0,  0, 21,  0, }, // Gilbert_Islands
    {   61,  788,       0,   87773,       0,  0, 15,  0, }, // GMT
    {   61,  830,   87788,   87811,   87845, 23, 34, 29, }, // Greenland_Eastern
    {   61,  848,   87874,   87897,   87931, 23, 34, 29, }, // Greenland_Western
    {   61,  871,       0,   87960,       0,  0, 22,  0, }, // Gulf
    {   61,  876,       0,   87982,       0,  0,  9,  0, }, // Guyana
    {   61,  883,   87991,   88011,   88042, 20, 31, 26, }, // Hawaii_Aleutian
    {   61,  899,   88068,   88080,   88103, 12, 23, 18, }, // Hong_Kong
    {   61,  909,   88121,   88130,   88150,  9, 20, 15, }, // Hovd
    {   61,  914,       0,   88165,       0,  0,  9,  0, }, // India
    {   61,  920,       0,   88174,       0,  0, 21,  0, }, // Indian_Ocean
    {   61,  933,       0,   88195,       0,  0, 16,  0, }, // Indochina
    {   61,  943,       0,   88211,       0,  0, 26,  0, }, // Indonesia_Central
    {   61,  961,       0,   88237,       0,  0, 23,  0, }, // Indonesia_Eastern
    {   61,  979,       0,   88260,       0,  0, 23,  0, }, // Indonesia_Western
    {   61,  997,   88283,   88294,   88316, 11, 22, 17, }, // Iran
    {   61, 1008,   88333,   88344,   88366, 11, 22, 17, }, // Irkutsk
    {   61, 1016,   88383,   88397,   88422, 14, 25, 20, }, // Israel
    {   61, 1023,   88442,   88452,   88473, 10, 21, 16, }, // Japan
    {   61, 1047,       0,   88489,       0,  0, 16,  0, }, // Kazakhstan
    {   61, 1058,       0,   88505,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {   61, 1077,       0,   88528,       0,  0, 23,  0, }, // Kazakhstan_Western
    {   61, 1106,   88551,   88560,   88580,  9, 20, 15, }, // Korea
    {   61, 1112,       0,   88595,       0,  0, 19,  0, }, // Kosrae
    {   61, 1119,   88614,   88629,   88655, 15, 26, 21, }, // Krasnoyarsk
    {   61, 1151,       0,   88676,       0,  0, 15,  0, }, // Kyrgystan
    {   61, 1175,       0,   88691,       0,  0, 17,  0, }, // Line_Islands
    {   61, 1188,   88708,   88720,   88743, 12, 23, 18, }, // Lord_Howe
    {   61, 1204,   88761,   88775,   88800, 14, 25, 20, }, // Magadan
    {   61, 1219,       0,   88820,       0,  0, 12,  0, }, // Malaysia
    {   61, 1228,       0,   88832,       0,  0, 13,  0, }, // Maldives
    {   61, 1237,       0,   88845,       0,  0, 23,  0, }, // Marquesas
    {   61, 1247,       0,   88868,       0,  0, 23,  0, }, // Marshall_Islands
    {   61, 1264,   88891,   88903,   88926, 12, 23, 18, }, // Mauritius
    {   61, 1274,       0,   88944,       0,  0, 18,  0, }, // Mawson
    {   61, 1281,   88962,   88990,   89028, 28, 38, 34, }, // Mexico_Pacific
    {   61, 1296,   89062,   89077,   89103, 15, 26, 21, }, // Mongolia
    {   61, 1305,   89124,   89137,   89161, 13, 24, 19, }, // Moscow
    {   61, 1312,       0,   89180,       0,  0, 10,  0, }, // Myanmar
    {   61, 1320,       0,   89190,       0,  0,  9,  0, }, // Nauru
    {   61, 1326,       0,   89199,       0,  0, 13,  0, }, // Nepal
    {   61, 1332,   89212,   89231,   89261, 19, 30, 25, }, // New_Caledonia
    {   61, 1346,   89286,   89304,   89333, 18, 29, 24, }, // New_Zealand
    {   61, 1358,   89357,   89376,   89406, 19, 30, 25, }, // Newfoundland
    {   61, 1371,       0,   89431,       0,  0,  8,  0, }, // Niue
    {   61, 1376,   89439,   89459,   89490, 20, 31, 26, }, // Norfolk
    {   61, 1384,   89516,   89540,   89575, 24, 35, 30, }, // Noronha
    {   61, 1406,   89605,   89621,   89648, 16, 27, 22, }, // Novosibirsk
    {   61, 1418,   89670,   89679,   89699,  9, 20, 15, }, // Omsk
    {   61, 1428,   89714,   89729,   89755, 15, 26, 21, }, // Pakistan
    {   61, 1437,       0,   89776,       0,  0,  9,  0, }, // Palau
    {   61, 1443,       0,   89785,       0,  0, 22,  0, }, // Papua_New_Guinea
    {   61, 1460,   89807,   89819,   89842, 12, 23, 18, }, // Paraguay
    {   61, 1469,   89860,   89873,   89897, 13, 24, 19, }, // Peru
    {   61, 1474,   89916,   89930,   89955, 14, 25, 20, }, // Philippines
    {   61, 1486,       0,   89975,       0,  0, 19,  0, }, // Phoenix_Islands
    {   61, 1502,   89994,   90016,   90049, 22, 33, 39, }, // Pierre_Miquelon
    {   61, 1518,       0,   90088,       0,  0, 20,  0, }, // Pitcairn
    {   61, 1527,       0,   90108,       0,  0, 19,  0, }, // Ponape
    {   61, 1534,       0,   90127,       0,  0, 14,  0, }, // Pyongyang
    {   61, 1554,       0,   90141,       0,  0, 12,  0, }, // Reunion
    {   61, 1562,       0,   90153,       0,  0, 18,  0, }, // Rothera
    {   61, 1570,   90171,   90185,   90210, 14, 25, 20, }, // Sakhalin
    {   61, 1596,   90230,   90239,   90259,  9, 20, 15, }, // Samoa
    {   61, 1602,       0,   90274,       0,  0, 24,  0, }, // Seychelles
    {   61, 1624,       0,   90298,       0,  0, 15,  0, }, // Singapore
    {   61, 1634,       0,   90313,       0,  0, 24,  0, }, // Solomon
    {   61, 1642,       0,   90337,       0,  0, 21,  0, }, // South_Georgia
    {   61, 1656,       0,   90358,       0,  0, 12,  0, }, // Suriname
    {   61, 1676,       0,   90370,       0,  0, 16,  0, }, // Syowa
    {   61, 1682,       0,   90386,       0,  0,  9,  0, }, // Tahiti
    {   61, 1689,   90395,   90405,   90426, 10, 21, 16, }, // Taipei
    {   61, 1696,       0,   90442,       0,  0, 16,  0, }, // Tajikistan
    {   61, 1724,       0,   90458,       0,  0, 11,  0, }, // Tokelau
    {   61, 1732,   90469,   90478,   90498,  9, 20, 15, }, // Tonga
    {   61, 1738,       0,   90513,       0,  0,  9,  0, }, // Truk
    {   61, 1750,   90522,   90539,   90567, 17, 28, 23, }, // Turkmenistan
    {   61, 1763,       0,   90590,       0,  0, 10,  0, }, // Tuvalu
    {   61, 1777,   90600,   90614,   90639, 14, 25, 20, }, // Uruguay
    {   61, 1792,   90659,   90674,   90700, 15, 26, 21, }, // Uzbekistan
    {   61, 1803,   90721,   90732,   90754, 11, 22, 17, }, // Vanuatu
    {   61, 1811,       0,   90771,       0,  0, 16,  0, }, // Venezuela
    {   61, 1821,   90787,   90805,   90834, 18, 29, 24, }, // Vladivostok
    {   61, 1833,   90858,   90874,   90901, 16, 27, 22, }, // Volgograd
    {   61, 1843,       0,   90923,       0,  0, 18,  0, }, // Vostok
    {   61, 1850,       0,   90941,       0,  0, 17,  0, }, // Wake
    {   61, 1855,       0,   90958,       0,  0, 27,  0, }, // Wallis
    {   61, 1862,   90985,   90995,   91016, 10, 21, 16, }, // Yakutsk
    {   61, 1870,   91032,   91051,   91081, 19, 30, 25, }, // Yekaterinburg
    {   61, 1892,       0,   91106,       0,  0,  9,  0, }, // Yukon
    {   64,  788,       0,   91115,       0,  0, 17,  0, }, // GMT Bhojpuri/Devanagari/India
    {   66,    0,   91132,   91139,   91157,  7, 18, 12, }, // Acre Bodo/Devanagari/India
    {   66,    5,       0,   91169,       0,  0, 14,  0, }, // Afghanistan
    {   66,   17,       0,   91183,       0,  0, 15,  0, }, // Africa_Central
    {   66,   32,       0,   91198,       0,  0, 16,  0, }, // Africa_Eastern
    {   66,   65,       0,   91214,       0,  0, 25,  0, }, // Africa_Southern
    {   66,   81,   91239,   91257,   91283, 18, 26, 26, }, // Africa_Western
    {   66,  107,   91309,   91321,   91343, 12, 22, 19, }, // Alaska
    {   66,  128,   91362,   91373,   91394, 11, 21, 15, }, // Almaty
    {   66,  135,   91409,   91417,   91433,  8, 16, 16, }, // Amazon
    {   66,  142,   91449,   91471,   91504, 22, 33, 29, }, // America_Central
    {   66,  158,   91533,   91560,   91595, 27, 35, 34, }, // America_Eastern
    {   66,  174,   91629,   91657,   91693, 28, 36, 35, }, // America_Mountain
    {   66,  191,   91728,   91753,   91786, 25, 33, 32, }, // America_Pacific
    {   66,  207,   91818,   91827,   91848,  9, 21, 15, }, // Anadyr
    {   66,  214,   91863,   91871,   91887,  8, 16, 12, }, // Apia
    {   66,  219,   91899,   91908,   91930,  9, 22, 16, }, // Aqtau
    {   66,  225,   91946,   91955,   91978,  9, 23, 17, }, // Aqtobe
    {   66,  232,   91995,   92006,   92025, 11, 19, 18, }, // Arabian
    {   66,  240,   92043,   92057,   92079, 14, 22, 22, }, // Argentina
    {   66,  250,   92101,   92121,   92149, 20, 28, 28, }, // Argentina_Western
    {   66,  268,   92177,   92189,   92209, 12, 20, 20, }, // Armenia
    {   66,  286,   92229,   92241,   92261, 12, 20, 19, }, // Atlantic
    {   66,  295,   92280,   92300,   92330, 20, 30, 27, }, // Australia_Central
    {   66,  313,   92357,   92387,   92425, 30, 38, 37, }, // Australia_CentralWestern
    {   66,  338,   92462,   92486,   92520, 24, 34, 33, }, // Australia_Eastern
    {   66,  356,   92553,   92576,   92609, 23, 33, 32, }, // Australia_Western
    {   66,  374,   92641,   92654,   92675, 13, 21, 21, }, // Azerbaijan
    {   66,  385,   92696,   92707,   92726, 11, 19, 19, }, // Azores
    {   66,  397,   92745,   92756,   92775, 11, 19, 19, }, // Bangladesh
    {   66,  415,       0,   92794,       0,  0,  8,  0, }, // Bhutan
    {   66,  422,       0,   92802,       0,  0, 10,  0, }, // Bolivia
    {   66,  437,   92812,   92825,   92846, 13, 21, 21, }, // Brasilia
    {   66,  454,       0,   92867,       0,  0, 33,  0, }, // Brunei
    {   66,  461,   92900,   92913,   92934, 13, 21, 21, }, // Cape_Verde
    {   66,  478,       0,   92955,       0,  0, 19,  0, }, // Chamorro
    {   66,  487,   92974,   92982,   92998,  8, 16, 15, }, // Chatham
    {   66,  495,   93013,   93020,   93035,  7, 15, 15, }, // Chile
    {   66,  501,   93050,   93058,   93074,  8, 16, 15, }, // China
    {   66,  507,       0,   93089,       0,  0, 19,  0, }, // Christmas
    {   66,  517,       0,   93108,       0,  0, 18,  0, }, // Cocos
    {   66,  523,   93126,   93137,   93156, 11, 19, 19, }, // Colombia
    {   66,  532,   93175,   93192,   93217, 17, 25, 31, }, // Cook
    {   66,  537,   93248,   93256,   93272,  8, 16, 15, }, // Cuba
    {   66,  548,       0,   93287,       0,  0,  8,  0, }, // Davis
    {   66,  564,       0,   93295,       0,  0, 18,  0, }, // DumontDUrville
    {   66,  601,       0,   93313,       0,  0, 25,  0, }, // East_Timor
    {   66,  612,   93338,   93356,   93382, 18, 26, 26, }, // Easter
    {   66,  619,       0,   93408,       0,  0, 10,  0, }, // Ecuador
    {   66,  627,   93418,   93433,   93456, 15, 23, 23, }, // Europe_Central
    {   66,  642,   93479,   93499,   93527, 20, 28, 28, }, // Europe_Eastern
    {   66,  657,       0,   93555,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {   66,  680,   93584,   93600,   93624, 16, 24, 24, }, // Europe_Western
    {   66,  695,   93648,   93669,   93698, 21, 29, 29, }, // Falkland
    {   66,  704,   93727,   93734,   93749,  7, 15, 15, }, // Fiji
    {   66,  709,       0,   93764,       0,  0, 18,  0, }, // French_Guiana
    {   66,  723,       0,   93782,       0,  0, 35,  0, }, // French_Southern
    {   66,  746,       0,   93817,       0,  0, 12,  0, }, // Galapagos
    {   66,  756,       0,   93829,       0,  0, 12,  0, }, // Gambier
    {   66,  764,   93841,   93851,   93869, 10, 18, 18, }, // Georgia
    {   66,  772,       0,   93887,       0,  0, 22,  0, }, // Gilbert_Islands
    {   66,  788,       0,   93909,       0,  0, 20,  0, }, // GMT
    {   66,  830,   93929,   93948,   93975, 19, 27, 28, }, // Greenland_Eastern
    {   66,  848,   94003,   94022,   94049, 19, 27, 27, }, // Greenland_Western
    {   66,  866,       0,   94076,       0,  0, 19,  0, }, // Guam
    {   66,  871,       0,   94095,       0,  0, 16,  0, }, // Gulf
    {   66,  876,       0,   94111,       0,  0,  9,  0, }, // Guyana
    {   66,  883,   94120,   94139,   94166, 19, 27, 26, }, // Hawaii_Aleutian
    {   66,  899,   94192,   94199,   94214,  7, 15, 15, }, // Hong_Kong
    {   66,  909,   94229,   94237,   94253,  8, 16, 16, }, // Hovd
    {   66,  914,       0,   94269,       0,  0, 18,  0, }, // India
    {   66,  920,       0,   94287,       0,  0, 18,  0, }, // Indian_Ocean
    {   66,  933,       0,   94305,       0,  0, 24,  0, }, // Indochina
    {   66,  943,       0,   94329,       0,  0, 25,  0, }, // Indonesia_Central
    {   66,  961,       0,   94354,       0,  0, 33,  0, }, // Indonesia_Eastern
    {   66,  979,       0,   94387,       0,  0, 34,  0, }, // Indonesia_Western
    {   66,  997,   94421,   94428,   94443,  7, 15, 14, }, // Iran
    {   66, 1008,   94457,   94470,   94491, 13, 21, 22, }, // Irkutsk
    {   66, 1016,   94513,   94523,   94541, 10, 18, 17, }, // Israel
    {   66, 1023,   94558,   94566,   94582,  8, 16, 15, }, // Japan
    {   66, 1029,   94597,   94629,   94666, 32, 37, 31, }, // Kamchatka
    {   66, 1058,       0,   94697,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {   66, 1077,       0,   94716,       0,  0, 19,  0, }, // Kazakhstan_Western
    {   66, 1106,   94735,   94744,   94761,  9, 17, 16, }, // Korea
    {   66, 1112,       0,   94777,       0,  0,  8,  0, }, // Kosrae
    {   66, 1119,   94785,   94803,   94829, 18, 26, 27, }, // Krasnoyarsk
    {   66, 1151,       0,   94856,       0,  0, 14,  0, }, // Kyrgystan
    {   66, 1161,       0,   94870,       0,  0, 19,  0, }, // Lanka
    {   66, 1175,       0,   94889,       0,  0, 18,  0, }, // Line_Islands
    {   66, 1188,   94907,   94919,   94939, 12, 20, 19, }, // Lord_Howe
    {   66, 1198,   94958,   94966,   94986,  8, 20, 14, }, // Macau
    {   66, 1204,   95000,   95010,   95028, 10, 18, 18, }, // Magadan
    {   66, 1219,       0,   95046,       0,  0, 22,  0, }, // Malaysia
    {   66, 1228,       0,   95068,       0,  0, 11,  0, }, // Maldives
    {   66, 1237,       0,   95079,       0,  0, 12,  0, }, // Marquesas
    {   66, 1247,       0,   95091,       0,  0, 21,  0, }, // Marshall_Islands
    {   66, 1264,   95112,   95126,   95148, 14, 22, 22, }, // Mauritius
    {   66, 1274,       0,   95170,       0,  0,  7,  0, }, // Mawson
    {   66, 1281,   95177,   95198,   95227, 21, 29, 28, }, // Mexico_Pacific
    {   66, 1296,   95255,   95267,   95287, 12, 20, 20, }, // Mongolia
    {   66, 1305,   95307,   95315,   95331,  8, 16, 16, }, // Moscow
    {   66, 1312,       0,   95347,       0,  0, 23,  0, }, // Myanmar
    {   66, 1320,       0,   95370,       0,  0,  8,  0, }, // Nauru
    {   66, 1326,       0,   95378,       0,  0,  8,  0, }, // Nepal
    {   66, 1332,   95386,   95404,   95430, 18, 26, 26, }, // New_Caledonia
    {   66, 1346,   95456,   95469,   95490, 13, 21, 20, }, // New_Zealand
    {   66, 1358,   95510,   95527,   95552, 17, 25, 24, }, // Newfoundland
    {   66, 1371,       0,   95576,       0,  0,  7,  0, }, // Niue
    {   66, 1376,   95583,   95597,   95619, 14, 22, 21, }, // Norfolk
    {   66, 1384,   95640,   95663,   95694, 23, 31, 31, }, // Noronha
    {   66, 1392,       0,   95725,       0,  0, 28,  0, }, // North_Mariana
    {   66, 1406,   95753,   95768,   95791, 15, 23, 23, }, // Novosibirsk
    {   66, 1418,   95814,   95823,   95840,  9, 17, 17, }, // Omsk
    {   66, 1428,   95857,   95869,   95889, 12, 20, 20, }, // Pakistan
    {   66, 1437,       0,   95909,       0,  0,  8,  0, }, // Palau
    {   66, 1443,       0,   95917,       0,  0, 17,  0, }, // Papua_New_Guinea
    {   66, 1460,   95934,   95945,   95964, 11, 19, 19, }, // Paraguay
    {   66, 1469,   95983,   95990,   96005,  7, 15, 15, }, // Peru
    {   66, 1474,   96020,   96032,   96056, 12, 24, 18, }, // Philippines
    {   66, 1486,       0,   96074,       0,  0, 20,  0, }, // Phoenix_Islands
    {   66, 1502,   96094,   96121,   96156, 27, 35, 34, }, // Pierre_Miquelon
    {   66, 1518,       0,   96190,       0,  0, 11,  0, }, // Pitcairn
    {   66, 1527,       0,   96201,       0,  0,  8,  0, }, // Ponape
    {   66, 1534,       0,   96209,       0,  0, 10,  0, }, // Pyongyang
    {   66, 1544,   96219,   96236,   96263, 17, 27, 21, }, // Qyzylorda
    {   66, 1554,       0,   96284,       0,  0, 11,  0, }, // Reunion
    {   66, 1562,       0,   96295,       0,  0,  8,  0, }, // Rothera
    {   66, 1570,   96303,   96313,   96331, 10, 18, 18, }, // Sakhalin
    {   66, 1579,   96349,   96359,   96379, 10, 20, 14, }, // Samara
    {   66, 1596,   96393,   96402,   96419,  9, 17, 16, }, // Samoa
    {   66, 1602,       0,   96435,       0,  0, 10,  0, }, // Seychelles
    {   66, 1624,       0,   96445,       0,  0, 23,  0, }, // Singapore
    {   66, 1634,       0,   96468,       0,  0, 19,  0, }, // Solomon
    {   66, 1642,       0,   96487,       0,  0, 15,  0, }, // South_Georgia
    {   66, 1656,       0,   96502,       0,  0, 10,  0, }, // Suriname
    {   66, 1676,       0,   96512,       0,  0,  8,  0, }, // Syowa
    {   66, 1682,       0,   96520,       0,  0,  9,  0, }, // Tahiti
    {   66, 1689,   96529,   96538,   96555,  9, 17, 16, }, // Taipei
    {   66, 1696,       0,   96571,       0,  0, 14,  0, }, // Tajikistan
    {   66, 1724,       0,   96585,       0,  0,  8,  0, }, // Tokelau
    {   66, 1732,   96593,   96600,   96615,  7, 15, 15, }, // Tonga
    {   66, 1738,       0,   96630,       0,  0,  6,  0, }, // Truk
    {   66, 1750,   96636,   96653,   96678, 17, 25, 25, }, // Turkmenistan
    {   66, 1763,       0,   96703,       0,  0,  9,  0, }, // Tuvalu
    {   66, 1777,   96712,   96722,   96740, 10, 18, 18, }, // Uruguay
    {   66, 1792,   96758,   96772,   96794, 14, 22, 22, }, // Uzbekistan
    {   66, 1803,   96816,   96826,   96844, 10, 18, 18, }, // Vanuatu
    {   66, 1811,       0,   96862,       0,  0, 13,  0, }, // Venezuela
    {   66, 1821,   96875,   96890,   96913, 15, 23, 23, }, // Vladivostok
    {   66, 1833,   96936,   96949,   96970, 13, 21, 21, }, // Volgograd
    {   66, 1843,       0,   96881,       0,  0,  9,  0, }, // Vostok
    {   66, 1850,       0,   96991,       0,  0, 12,  0, }, // Wake
    {   66, 1855,       0,   97003,       0,  0, 19,  0, }, // Wallis
    {   66, 1862,   97022,   97034,   97054, 12, 20, 20, }, // Yakutsk
    {   66, 1870,   97074,   97092,   97118, 18, 26, 26, }, // Yekaterinburg
    {   66, 1892,       0,   97144,       0,  0,  7,  0, }, // Yukon
    {   67,    0,   97151,   97161,   97182, 10, 21, 29, }, // Acre Bosnian/Latin/Bosnia and Herzegovina
    {   67,    5,       0,   97211,       0,  0, 21,  0, }, // Afghanistan
    {   67,   17,       0,   97232,       0,  0, 24,  0, }, // Africa_Central
    {   67,   32,       0,   97256,       0,  0, 22,  0, }, // Africa_Eastern
    {   67,   65,       0,   97278,       0,  0, 31,  0, }, // Africa_Southern
    {   67,   81,   97309,   97331,   97364, 22, 33, 29, }, // Africa_Western
    {   67,  107,   97393,   97412,   97442, 19, 30, 26, }, // Alaska
    {   67,  128,   97468,   97480,   97503, 12, 23, 31, }, // Almaty
    {   67,  135,   97534,   97551,   97579, 17, 28, 24, }, // Amazon
    {   67,  142,   97603,   97637,   97682, 34, 45, 41, }, // America_Central
    {   67,  158,   97723,   97755,   97798, 32, 43, 39, }, // America_Eastern
    {   67,  174,   97837,   97871,   97916, 34, 45, 41, }, // America_Mountain
    {   67,  191,   97957,   97991,   98036, 34, 45, 41, }, // America_Pacific
    {   67,  207,   98077,   98089,   98112, 12, 23, 31, }, // Anadyr
    {   67,  214,   98143,   98158,   98184, 15, 26, 22, }, // Apia
    {   67,  219,   98206,   98218,   98241, 12, 23, 31, }, // Aqtau
    {   67,  225,   98272,   98285,   98309, 13, 24, 32, }, // Aqtobe
    {   67,  232,   98341,   98358,   98386, 17, 28, 24, }, // Arabian
    {   67,  240,   98410,   98429,   98459, 19, 30, 26, }, // Argentina
    {   67,  250,   98485,   98511,   98548, 26, 37, 33, }, // Argentina_Western
    {   67,  268,   98581,   98597,   98624, 16, 27, 23, }, // Armenia
    {   67,  286,   98647,   98681,   98726, 34, 45, 41, }, // Atlantic
    {   67,  295,   98767,   98796,   98836, 29, 40, 36, }, // Australia_Central
    {   67,  313,   98872,   98910,   98958, 38, 48, 44, }, // Australia_CentralWestern
    {   67,  338,   99002,   99029,   99067, 27, 38, 34, }, // Australia_Eastern
    {   67,  356,   99101,   99128,   99166, 27, 38, 34, }, // Australia_Western
    {   67,  374,   99200,   99222,   99255, 22, 33, 29, }, // Azerbaijan
    {   67,  385,   99284,   99299,   99325, 15, 26, 22, }, // Azores
    {   67,  397,   99347,   99366,   99396, 19, 30, 26, }, // Bangladesh
    {   67,  415,       0,   99422,       0,  0, 16,  0, }, // Bhutan
    {   67,  422,       0,   99438,       0,  0, 18,  0, }, // Bolivia
    {   67,  437,   99456,   99475,   99505, 19, 30, 26, }, // Brasilia
    {   67,  454,       0,   99531,       0,  0, 17,  0, }, // Brunei
    {   67,  461,   99548,   99567,   99597, 19, 30, 26, }, // Cape_Verde
    {   67,  478,       0,   99623,       0,  0, 27,  0, }, // Chamorro
    {   67,  487,   99650,   99666,   99693, 16, 27, 23, }, // Chatham
    {   67,  495,   99716,   99733,   99761, 17, 28, 24, }, // Chile
    {   67,  501,   99785,   99800,   99826, 15, 26, 22, }, // China
    {   67,  507,       0,   99848,       0,  0, 26,  0, }, // Christmas
    {   67,  517,       0,   99874,       0,  0, 25,  0, }, // Cocos
    {   67,  523,   99899,   99918,   99948, 19, 30, 26, }, // Colombia
    {   67,  532,   99974,  100001,  100039, 27, 38, 38, }, // Cook
    {   67,  537,  100077,  100093,  100120, 16, 27, 23, }, // Cuba
    {   67,  548,       0,  100143,       0,  0, 21,  0, }, // Davis
    {   67,  564,       0,  100164,       0,  0, 32,  0, }, // DumontDUrville
    {   67,  601,       0,  100196,       0,  0, 23,  0, }, // East_Timor
    {   67,  612,  100219,  100243,  100278, 24, 35, 31, }, // Easter
    {   67,  619,       0,  100309,       0,  0, 18,  0, }, // Ecuador
    {   67,  627,  100327,  100352,  100388, 25, 36, 32, }, // Europe_Central
    {   67,  642,  100420,  100443,  100477, 23, 34, 30, }, // Europe_Eastern
    {   67,  657,       0,  100507,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {   67,  680,  100536,  100559,  100593, 23, 34, 30, }, // Europe_Western
    {   67,  695,  100623,  100642,  100672, 19, 30, 26, }, // Falkland
    {   67,  704,  100698,  100716,  100745, 18, 29, 24, }, // Fiji
    {   67,  709,       0,  100769,       0,  0, 26,  0, }, // French_Guiana
    {   67,  723,       0,  100795,       0,  0, 52,  0, }, // French_Southern
    {   67,  746,       0,  100847,       0,  0, 19,  0, }, // Galapagos
    {   67,  756,       0,  100866,       0,  0, 19,  0, }, // Gambier
    {   67,  764,  100885,  100902,  100930, 17, 28, 24, }, // Georgia
    {   67,  772,       0,  100954,       0,  0, 31,  0, }, // Gilbert_Islands
    {   67,  788,       0,  100985,       0,  0, 16,  0, }, // GMT
    {   67,  830,  101001,  101027,  101064, 26, 37, 33, }, // Greenland_Eastern
    {   67,  848,  101097,  101123,  101160, 26, 37, 33, }, // Greenland_Western
    {   67,  866,       0,  101193,       0,  0, 21,  0, }, // Guam
    {   67,  871,       0,  101214,       0,  0, 27,  0, }, // Gulf
    {   67,  876,       0,  101241,       0,  0, 17,  0, }, // Guyana
    {   67,  883,  101258,  101285,  101323, 27, 38, 34, }, // Hawaii_Aleutian
    {   67,  899,  101357,  101375,  101404, 18, 29, 25, }, // Hong_Kong
    {   67,  909,  101429,  101444,  101470, 15, 26, 22, }, // Hovd
    {   67,  914,       0,  101492,       0,  0, 27,  0, }, // India
    {   67,  920,       0,  101519,       0,  0, 27,  0, }, // Indian_Ocean
    {   67,  933,       0,  101546,       0,  0, 19,  0, }, // Indochina
    {   67,  943,       0,  101565,       0,  0, 29,  0, }, // Indonesia_Central
    {   67,  961,       0,  101594,       0,  0, 27,  0, }, // Indonesia_Eastern
    {   67,  979,       0,  101621,       0,  0, 27,  0, }, // Indonesia_Western
    {   67,  997,  101648,  101663,  101689, 15, 26, 22, }, // Iran
    {   67, 1008,  101711,  101727,  101754, 16, 27, 23, }, // Irkutsk
    {   67, 1016,  101777,  101794,  101822, 17, 28, 24, }, // Israel
    {   67, 1023,  101846,  101862,  101889, 16, 27, 23, }, // Japan
    {   67, 1029,  101912,  101941,  101981, 29, 40, 48, }, // Kamchatka
    {   67, 1047,       0,  102029,       0,  0, 20,  0, }, // Kazakhstan
    {   67, 1058,       0,  102049,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {   67, 1077,       0,  102076,       0,  0, 27,  0, }, // Kazakhstan_Western
    {   67, 1106,  102103,  102119,  102146, 16, 27, 23, }, // Korea
    {   67, 1112,       0,  102169,       0,  0, 24,  0, }, // Kosrae
    {   67, 1119,  102193,  102213,  102244, 20, 31, 27, }, // Krasnoyarsk
    {   67, 1151,       0,  102271,       0,  0, 20,  0, }, // Kyrgystan
    {   67, 1161,       0,  102291,       0,  0, 11,  0, }, // Lanka
    {   67, 1175,       0,  102302,       0,  0, 24,  0, }, // Line_Islands
    {   67, 1188,  102326,  102352,  102389, 26, 37, 33, }, // Lord_Howe
    {   67, 1198,  102422,  102433,  102455, 11, 22, 30, }, // Macau
    {   67, 1204,  102485,  102503,  102532, 18, 29, 25, }, // Magadan
    {   67, 1219,       0,  102557,       0,  0, 18,  0, }, // Malaysia
    {   67, 1228,       0,  102575,       0,  0, 17,  0, }, // Maldives
    {   67, 1237,       0,  102592,       0,  0, 26,  0, }, // Marquesas
    {   67, 1247,       0,  102618,       0,  0, 30,  0, }, // Marshall_Islands
    {   67, 1264,  102648,  102667,  102697, 19, 30, 26, }, // Mauritius
    {   67, 1274,       0,  102723,       0,  0, 22,  0, }, // Mawson
    {   67, 1281,  102745,  102771,  102808, 26, 37, 33, }, // Mexico_Pacific
    {   67, 1296,  102841,  102861,  102892, 20, 31, 27, }, // Mongolia
    {   67, 1305,  102919,  102936,  102964, 17, 28, 24, }, // Moscow
    {   67, 1312,       0,  102988,       0,  0, 19,  0, }, // Myanmar
    {   67, 1320,       0,  103007,       0,  0, 23,  0, }, // Nauru
    {   67, 1326,       0,  103030,       0,  0, 16,  0, }, // Nepal
    {   67, 1332,  103046,  103070,  103105, 24, 35, 31, }, // New_Caledonia
    {   67, 1346,  103136,  103157,  103189, 21, 32, 28, }, // New_Zealand
    {   67, 1358,  103217,  103240,  103274, 23, 34, 30, }, // Newfoundland
    {   67, 1371,       0,  103304,       0,  0, 22,  0, }, // Niue
    {   67, 1376,  103326,  103343,  103371, 17, 28, 24, }, // Norfolk
    {   67, 1384,  103395,  103432,  103480, 37, 48, 44, }, // Noronha
    {   67, 1392,       0,  103524,       0,  0, 31,  0, }, // North_Mariana
    {   67, 1406,  103555,  103575,  103606, 20, 31, 27, }, // Novosibirsk
    {   67, 1418,  103633,  103646,  103670, 13, 24, 20, }, // Omsk
    {   67, 1428,  103690,  103709,  103739, 19, 30, 26, }, // Pakistan
    {   67, 1437,       0,  103765,       0,  0, 23,  0, }, // Palau
    {   67, 1443,       0,  103788,       0,  0, 30,  0, }, // Papua_New_Guinea
    {   67, 1460,  103818,  103837,  103867, 19, 30, 26, }, // Paraguay
    {   67, 1469,  103893,  103910,  103938, 17, 28, 24, }, // Peru
    {   67, 1474,  103962,  103980,  104009, 18, 29, 25, }, // Philippines
    {   67, 1486,       0,  104034,       0,  0, 26,  0, }, // Phoenix_Islands
    {   67, 1502,  104060,  104101,  104153, 41, 52, 48, }, // Pierre_Miquelon
    {   67, 1518,       0,  104201,       0,  0, 27,  0, }, // Pitcairn
    {   67, 1527,       0,  104228,       0,  0, 24,  0, }, // Ponape
    {   67, 1534,       0,  104252,       0,  0, 19,  0, }, // Pyongyang
    {   67, 1544,  104271,  104286,  104312, 15, 26, 34, }, // Qyzylorda
    {   67, 1554,       0,  104346,       0,  0, 18,  0, }, // Reunion
    {   67, 1562,       0,  104364,       0,  0, 23,  0, }, // Rothera
    {   67, 1570,  104387,  104405,  104434, 18, 29, 25, }, // Sakhalin
    {   67, 1579,  104459,  104471,  104494, 12, 23, 31, }, // Samara
    {   67, 1596,  104525,  104542,  104570, 17, 28, 24, }, // Samoa
    {   67, 1602,       0,  104594,       0,  0, 17,  0, }, // Seychelles
    {   67, 1624,       0,  104611,       0,  0, 30,  0, }, // Singapore
    {   67, 1634,       0,  104641,       0,  0, 31,  0, }, // Solomon
    {   67, 1642,       0,  104672,       0,  0, 24,  0, }, // South_Georgia
    {   67, 1656,       0,  104696,       0,  0, 18,  0, }, // Suriname
    {   67, 1676,       0,  104714,       0,  0, 21,  0, }, // Syowa
    {   67, 1682,       0,  104735,       0,  0, 18,  0, }, // Tahiti
    {   67, 1689,  104753,  104770,  104798, 17, 28, 24, }, // Taipei
    {   67, 1696,       0,  104822,       0,  0, 22,  0, }, // Tajikistan
    {   67, 1724,       0,  104844,       0,  0, 25,  0, }, // Tokelau
    {   67, 1732,  104869,  104886,  104914, 17, 28, 24, }, // Tonga
    {   67, 1738,       0,  104938,       0,  0, 13,  0, }, // Truk
    {   67, 1750,  104951,  104974,  105008, 23, 34, 30, }, // Turkmenistan
    {   67, 1763,       0,  105038,       0,  0, 19,  0, }, // Tuvalu
    {   67, 1777,  105057,  105075,  105104, 18, 29, 25, }, // Uruguay
    {   67, 1792,  105129,  105150,  105182, 21, 32, 28, }, // Uzbekistan
    {   67, 1803,  105210,  105230,  105261, 20, 31, 27, }, // Vanuatu
    {   67, 1811,       0,  105288,       0,  0, 21,  0, }, // Venezuela
    {   67, 1821,  105309,  105330,  105362, 21, 32, 28, }, // Vladivostok
    {   67, 1833,  105390,  105410,  105441, 20, 31, 27, }, // Volgograd
    {   67, 1843,       0,  105468,       0,  0, 22,  0, }, // Vostok
    {   67, 1850,       0,  105490,       0,  0, 22,  0, }, // Wake
    {   67, 1855,       0,  105512,       0,  0, 34,  0, }, // Wallis
    {   67, 1862,  105546,  105562,  105589, 16, 27, 23, }, // Yakutsk
    {   67, 1870,  105612,  105635,  105669, 23, 34, 30, }, // Yekaterinburg
    {   67, 1892,       0,  105699,       0,  0, 16,  0, }, // Yukon
    {   68,    0,  105715,  105725,  105746, 10, 21, 27, }, // Acre Bosnian/Cyrillic/Bosnia and Herzegovina
    {   68,    5,       0,  105773,       0,  0, 18,  0, }, // Afghanistan
    {   68,   17,       0,  105791,       0,  0, 25,  0, }, // Africa_Central
    {   68,   32,       0,  105816,       0,  0, 23,  0, }, // Africa_Eastern
    {   68,   65,       0,  105839,       0,  0, 21,  0, }, // Africa_Southern
    {   68,   81,  105860,  105883,  105917, 23, 34, 38, }, // Africa_Western
    {   68,  107,  105955,  105969,  105994, 14, 25, 20, }, // Alaska
    {   68,  128,  106014,  106026,  106049, 12, 23, 29, }, // Almaty
    {   68,  135,  106078,  106092,  106117, 14, 25, 29, }, // Amazon
    {   68,  142,  106146,  106163,  106191, 17, 28, 32, }, // America_Central
    {   68,  158,  106223,  106238,  106264, 15, 26, 30, }, // America_Eastern
    {   68,  174,  106294,  106311,  106339, 17, 28, 32, }, // America_Mountain
    {   68,  191,  106371,  106388,  106416, 17, 28, 32, }, // America_Pacific
    {   68,  207,  106448,  106460,  106483, 12, 23, 29, }, // Anadyr
    {   68,  214,  106512,  106525,  106549, 13, 24, 28, }, // Apia
    {   68,  219,  106577,  106590,  106614, 13, 24, 30, }, // Aqtau
    {   68,  225,  106644,  106657,  106681, 13, 24, 30, }, // Aqtobe
    {   68,  232,  106711,  106728,  106756, 17, 28, 32, }, // Arabian
    {   68,  240,  106788,  106805,  106833, 17, 28, 32, }, // Argentina
    {   68,  250,  106865,  106890,  106926, 25, 36, 40, }, // Argentina_Western
    {   68,  268,  106966,  106982,  107009, 16, 27, 31, }, // Armenia
    {   68,  286,  107040,  107056,  107083, 16, 27, 31, }, // Atlantic
    {   68,  295,  107114,  107144,  107185, 30, 41, 45, }, // Australia_Central
    {   68,  313,  107230,  107268,  107317, 38, 49, 53, }, // Australia_CentralWestern
    {   68,  338,  107370,  107398,  107437, 28, 39, 43, }, // Australia_Eastern
    {   68,  356,  107480,  107508,  107547, 28, 39, 43, }, // Australia_Western
    {   68,  374,  107590,  107608,  107637, 18, 29, 33, }, // Azerbaijan
    {   68,  385,  107670,  107683,  107707, 13, 24, 28, }, // Azores
    {   68,  397,  107735,  107752,  107780, 17, 28, 32, }, // Bangladesh
    {   68,  415,       0,  107812,       0,  0, 13,  0, }, // Bhutan
    {   68,  422,       0,  107825,       0,  0, 16,  0, }, // Bolivia
    {   68,  437,  107841,  107858,  107886, 17, 28, 32, }, // Brasilia
    {   68,  454,       0,  107918,       0,  0, 24,  0, }, // Brunei
    {   68,  461,  107942,  107961,  107991, 19, 30, 34, }, // Cape_Verde
    {   68,  478,       0,  108025,       0,  0, 14,  0, }, // Chamorro
    {   68,  487,  108039,  108052,  108076, 13, 24, 28, }, // Chatham
    {   68,  495,  108104,  108116,  108139, 12, 23, 27, }, // Chile
    {   68,  501,  108166,  108178,  108201, 12, 23, 27, }, // China
    {   68,  507,       0,  108228,       0,  0, 22,  0, }, // Christmas
    {   68,  517,       0,  108250,       0,  0, 29,  0, }, // Cocos
    {   68,  523,  108279,  108296,  108324, 17, 28, 32, }, // Colombia
    {   68,  532,  108356,  108377,  108409, 21, 32, 41, }, // Cook
    {   68,  537,  108450,  108462,  108485, 12, 23, 27, }, // Cuba
    {   68,  548,       0,  108512,       0,  0, 14,  0, }, // Davis
    {   68,  564,       0,  108526,       0,  0, 21,  0, }, // DumontDUrville
    {   68,  601,       0,  108547,       0,  0, 19,  0, }, // East_Timor
    {   68,  612,  108566,  108588,  108621, 22, 33, 37, }, // Easter
    {   68,  619,       0,  108658,       0,  0, 15,  0, }, // Ecuador
    {   68,  627,  108673,  108695,  108728, 22, 33, 37, }, // Europe_Central
    {   68,  642,  108765,  108788,  108822, 23, 34, 38, }, // Europe_Eastern
    {   68,  657,       0,  108860,       0,  0, 28,  0, }, // Europe_Further_Eastern
    {   68,  680,  108888,  108911,  108945, 23, 34, 38, }, // Europe_Western
    {   68,  695,  108983,  109009,  109046, 26, 37, 41, }, // Falkland
    {   68,  704,  109087,  109099,  109122, 12, 23, 27, }, // Fiji
    {   68,  709,       0,  109149,       0,  0, 25,  0, }, // French_Guiana
    {   68,  723,       0,  109174,       0,  0, 37,  0, }, // French_Southern
    {   68,  746,       0,  109211,       0,  0, 17,  0, }, // Galapagos
    {   68,  756,       0,  109228,       0,  0, 16,  0, }, // Gambier
    {   68,  764,  109244,  109259,  109285, 15, 26, 30, }, // Georgia
    {   68,  772,       0,  109315,       0,  0, 22,  0, }, // Gilbert_Islands
    {   68,  788,       0,  109337,       0,  0, 23,  0, }, // GMT
    {   68,  830,  109360,  109384,  109419, 24, 35, 39, }, // Greenland_Eastern
    {   68,  848,  109458,  109482,  109517, 24, 35, 39, }, // Greenland_Western
    {   68,  866,       0,  109556,       0,  0, 21,  0, }, // Guam
    {   68,  871,       0,  109577,       0,  0, 16,  0, }, // Gulf
    {   68,  876,       0,  109159,       0,  0, 15,  0, }, // Guyana
    {   68,  883,  109593,  109618,  109654, 25, 36, 40, }, // Hawaii_Aleutian
    {   68,  899,  109694,  109711,  109739, 17, 28, 33, }, // Hong_Kong
    {   68,  909,  109772,  109784,  109807, 12, 23, 27, }, // Hovd
    {   68,  914,       0,  109834,       0,  0, 27,  0, }, // India
    {   68,  920,       0,  109861,       0,  0, 25,  0, }, // Indian_Ocean
    {   68,  933,       0,  109886,       0,  0, 16,  0, }, // Indochina
    {   68,  943,       0,  109902,       0,  0, 30,  0, }, // Indonesia_Central
    {   68,  961,       0,  109932,       0,  0, 28,  0, }, // Indonesia_Eastern
    {   68,  979,       0,  109960,       0,  0, 28,  0, }, // Indonesia_Western
    {   68,  997,  109988,  110000,  110023, 12, 23, 27, }, // Iran
    {   68, 1008,  110050,  110064,  110089, 14, 25, 29, }, // Irkutsk
    {   68, 1016,  110118,  110135,  110163, 17, 28, 32, }, // Israel
    {   68, 1023,  110195,  110211,  110238, 16, 27, 31, }, // Japan
    {   68, 1029,  110269,  110299,  110340, 30, 41, 46, }, // Kamchatka
    {   68, 1047,       0,    3610,       0,  0, 15,  0, }, // Kazakhstan
    {   68, 1058,       0,  110386,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {   68, 1077,       0,  110414,       0,  0, 28,  0, }, // Kazakhstan_Western
    {   68, 1106,  110442,  110458,  110485, 16, 27, 31, }, // Korea
    {   68, 1112,       0,  110516,       0,  0, 13,  0, }, // Kosrae
    {   68, 1119,  110529,  110549,  110579, 20, 30, 34, }, // Krasnoyarsk
    {   68, 1151,       0,  110613,       0,  0, 17,  0, }, // Kyrgystan
    {   68, 1161,       0,  110630,       0,  0, 15,  0, }, // Lanka
    {   68, 1175,       0,  110645,       0,  0, 19,  0, }, // Line_Islands
    {   68, 1188,  110664,  110680,  110707, 16, 27, 31, }, // Lord_Howe
    {   68, 1198,  110738,  110749,  110771, 11, 22, 27, }, // Macau
    {   68, 1204,  110798,  110813,  110839, 15, 26, 29, }, // Magadan
    {   68, 1219,       0,  110868,       0,  0, 16,  0, }, // Malaysia
    {   68, 1228,       0,  110884,       0,  0, 15,  0, }, // Maldives
    {   68, 1237,       0,  110899,       0,  0, 14,  0, }, // Marquesas
    {   68, 1247,       0,  110913,       0,  0, 24,  0, }, // Marshall_Islands
    {   68, 1264,  110937,  110955,  110984, 18, 29, 33, }, // Mauritius
    {   68, 1274,       0,  111017,       0,  0, 13,  0, }, // Mawson
    {   68, 1281,  111030,  111056,  111093, 26, 37, 41, }, // Mexico_Pacific
    {   68, 1296,  111134,  111152,  111181, 18, 29, 33, }, // Mongolia
    {   68, 1305,  111214,  111228,  111253, 14, 25, 29, }, // Moscow
    {   68, 1312,       0,  111282,       0,  0, 16,  0, }, // Myanmar
    {   68, 1320,       0,  111298,       0,  0, 13,  0, }, // Nauru
    {   68, 1326,       0,  111311,       0,  0, 13,  0, }, // Nepal
    {   68, 1332,  111324,  111347,  111381, 23, 34, 38, }, // New_Caledonia
    {   68, 1346,  111419,  111438,  111468, 19, 30, 34, }, // New_Zealand
    {   68, 1358,  111502,  111521,  111551, 19, 30, 34, }, // Newfoundland
    {   68, 1371,       0,  111585,       0,  0, 12,  0, }, // Niue
    {   68, 1376,  111597,  111619,  111652, 22, 33, 37, }, // Norfolk
    {   68, 1384,  111689,  111715,  111752, 26, 37, 41, }, // Noronha
    {   68, 1392,       0,  111793,       0,  0, 31,  0, }, // North_Mariana
    {   68, 1406,  111824,  111844,  111874, 20, 30, 34, }, // Novosibirsk
    {   68, 1418,  111908,  111920,  111943, 12, 23, 27, }, // Omsk
    {   68, 1428,  111970,  111986,  112013, 16, 27, 31, }, // Pakistan
    {   68, 1437,       0,  112044,       0,  0, 13,  0, }, // Palau
    {   68, 1443,       0,  112057,       0,  0, 26,  0, }, // Papua_New_Guinea
    {   68, 1460,  112083,  112099,  112126, 16, 27, 31, }, // Paraguay
    {   68, 1469,  112157,  112169,  112192, 12, 23, 27, }, // Peru
    {   68, 1474,  112219,  112235,  112262, 16, 27, 31, }, // Philippines
    {   68, 1486,       0,  112293,       0,  0, 21,  0, }, // Phoenix_Islands
    {   68, 1502,  112314,  112340,  112377, 26, 37, 40, }, // Pierre_Miquelon
    {   68, 1518,       0,  112417,       0,  0, 15,  0, }, // Pitcairn
    {   68, 1527,       0,  112432,       0,  0, 14,  0, }, // Ponape
    {   68, 1534,       0,  112446,       0,  0, 17,  0, }, // Pyongyang
    {   68, 1544,  112463,  112478,  112504, 15, 26, 32, }, // Qyzylorda
    {   68, 1554,       0,  112536,       0,  0, 15,  0, }, // Reunion
    {   68, 1562,       0,  112551,       0,  0, 14,  0, }, // Rothera
    {   68, 1570,  112565,  112580,  112606, 15, 26, 30, }, // Sakhalin
    {   68, 1579,  112636,  112648,  112671, 12, 23, 29, }, // Samara
    {   68, 1596,  112700,  112713,  112737, 13, 24, 28, }, // Samoa
    {   68, 1602,       0,  112765,       0,  0, 15,  0, }, // Seychelles
    {   68, 1624,       0,  112780,       0,  0, 27,  0, }, // Singapore
    {   68, 1634,       0,  112807,       0,  0, 25,  0, }, // Solomon
    {   68, 1642,       0,  112832,       0,  0, 21,  0, }, // South_Georgia
    {   68, 1656,       0,  112853,       0,  0, 15,  0, }, // Suriname
    {   68, 1676,       0,  112868,       0,  0, 12,  0, }, // Syowa
    {   68, 1682,       0,  112880,       0,  0, 14,  0, }, // Tahiti
    {   68, 1689,  112894,  112908,  112933, 14, 25, 29, }, // Taipei
    {   68, 1696,       0,  112962,       0,  0, 18,  0, }, // Tajikistan
    {   68, 1724,       0,  112980,       0,  0, 15,  0, }, // Tokelau
    {   68, 1732,  112995,  113008,  113032, 13, 24, 28, }, // Tonga
    {   68, 1738,       0,  113060,       0,  0, 12,  0, }, // Truk
    {   68, 1750,  113072,  113092,  113123, 20, 31, 35, }, // Turkmenistan
    {   68, 1763,       0,  113158,       0,  0, 14,  0, }, // Tuvalu
    {   68, 1777,  113172,  113187,  113213, 15, 26, 30, }, // Uruguay
    {   68, 1792,  113243,  113261,  113290, 18, 29, 33, }, // Uzbekistan
    {   68, 1803,  113323,  113338,  113364, 15, 26, 30, }, // Vanuatu
    {   68, 1811,       0,  113394,       0,  0, 17,  0, }, // Venezuela
    {   68, 1821,  113411,  113430,  113460, 19, 30, 34, }, // Vladivostok
    {   68, 1833,  113494,  113511,  113539, 17, 28, 32, }, // Volgograd
    {   68, 1843,       0,  113571,       0,  0, 14,  0, }, // Vostok
    {   68, 1850,       0,  113585,       0,  0, 19,  0, }, // Wake
    {   68, 1855,       0,  113604,       0,  0, 29,  0, }, // Wallis
    {   68, 1862,  113633,  113648,  113674, 15, 26, 30, }, // Yakutsk
    {   68, 1870,  113704,  113725,  113757, 21, 32, 36, }, // Yekaterinburg
    {   68, 1892,       0,    6312,       0,  0, 10,  0, }, // Yukon
    {   69,    5,       0,  113793,       0,  0, 15,  0, }, // Afghanistan Breton/Latin/France
    {   69,   17,       0,  113808,       0,  0, 15,  0, }, // Africa_Central
    {   69,   32,       0,  113823,       0,  0, 19,  0, }, // Africa_Eastern
    {   69,   65,       0,  113842,       0,  0, 20,  0, }, // Africa_Southern
    {   69,   81,  113862,  113884,  113914, 22, 30, 27, }, // Africa_Western
    {   69,  107,  113941,  113951,  113969, 10, 18, 15, }, // Alaska
    {   69,  128,  113984,  113994,  114012, 10, 18, 15, }, // Almaty
    {   69,  135,  114027,  114040,  114061, 13, 21, 18, }, // Amazon
    {   69,  142,  114079,  114093,  114115, 14, 22, 19, }, // America_Central
    {   69,  158,  114134,  114146,  114166, 12, 20, 17, }, // America_Eastern
    {   69,  174,  114183,  114198,  114221, 15, 23, 20, }, // America_Mountain
    {   69,  191,  114241,  114254,  114275, 13, 21, 18, }, // America_Pacific
    {   69,  207,  114293,  114304,  114323, 11, 19, 16, }, // Anadyr
    {   69,  214,  114339,  114347,  114363,  8, 16, 13, }, // Apia
    {   69,  232,  114376,  114386,  114404, 10, 18, 15, }, // Arabian
    {   69,  240,  114419,  114434,  114457, 15, 23, 20, }, // Argentina
    {   69,  250,  114477,  114504,  114539, 27, 35, 32, }, // Argentina_Western
    {   69,  268,  114571,  114582,  114601, 11, 19, 16, }, // Armenia
    {   69,  286,  114617,  114632,  114655, 15, 23, 20, }, // Atlantic
    {   69,  295,  114675,  114693,  114719, 18, 26, 23, }, // Australia_Central
    {   69,  313,  114742,  114772,  114810, 30, 38, 35, }, // Australia_CentralWestern
    {   69,  338,  114845,  114867,  114897, 22, 30, 27, }, // Australia_Eastern
    {   69,  356,  114924,  114949,  114982, 25, 33, 30, }, // Australia_Western
    {   69,  374,  115012,  115027,  115050, 15, 23, 20, }, // Azerbaijan
    {   69,  385,  115070,  115083,  115104, 13, 21, 18, }, // Azores
    {   69,  397,  115122,  115136,  115158, 14, 22, 19, }, // Bangladesh
    {   69,  415,       0,  115177,       0,  0, 11,  0, }, // Bhutan
    {   69,  422,       0,  115188,       0,  0, 11,  0, }, // Bolivia
    {   69,  437,  115199,  115211,  115231, 12, 20, 17, }, // Brasilia
    {   69,  454,       0,  115248,       0,  0, 21,  0, }, // Brunei
    {   69,  461,  115269,  115286,  115311, 17, 25, 22, }, // Cape_Verde
    {   69,  478,       0,  115333,       0,  0, 12,  0, }, // Chamorro
    {   69,  487,  115345,  115356,  115375, 11, 19, 16, }, // Chatham
    {   69,  495,  115391,  115400,  115417,  9, 17, 14, }, // Chile
    {   69,  501,  115431,  115439,  115455,  8, 16, 13, }, // China
    {   69,  507,       0,  115468,       0,  0, 18,  0, }, // Christmas
    {   69,  517,       0,  115486,       0,  0, 15,  0, }, // Cocos
    {   69,  523,  115501,  115513,  115533, 12, 20, 17, }, // Colombia
    {   69,  532,  115550,  115564,  115586, 14, 22, 19, }, // Cook
    {   69,  537,  115605,  115613,  115629,  8, 16, 13, }, // Cuba
    {   69,  548,       0,  115642,       0,  0,  9,  0, }, // Davis
    {   69,  564,       0,  115651,       0,  0, 20,  0, }, // DumontDUrville
    {   69,  601,       0,  115671,       0,  0, 18,  0, }, // East_Timor
    {   69,  612,  115689,  115702,  115723, 13, 21, 18, }, // Easter
    {   69,  619,       0,  115741,       0,  0, 11,  0, }, // Ecuador
    {   69,  627,  115752,  115767,  115790, 15, 23, 20, }, // Europe_Central
    {   69,  642,  115810,  115829,  115856, 19, 27, 24, }, // Europe_Eastern
    {   69,  657,       0,  115880,       0,  0, 15,  0, }, // Europe_Further_Eastern
    {   69,  680,  115895,  115917,  115947, 22, 30, 27, }, // Europe_Western
    {   69,  695,  115974,  115992,  116018, 18, 26, 23, }, // Falkland
    {   69,  704,  116041,  116050,  116067,  9, 17, 14, }, // Fiji
    {   69,  709,       0,  116081,       0,  0, 17,  0, }, // French_Guiana
    {   69,  723,       0,  116098,       0,  0, 40,  0, }, // French_Southern
    {   69,  746,       0,  116138,       0,  0, 19,  0, }, // Galapagos
    {   69,  756,       0,  116157,       0,  0, 17,  0, }, // Gambier
    {   69,  764,  116174,  116184,  116202, 10, 18, 15, }, // Georgia
    {   69,  772,       0,  116217,       0,  0, 17,  0, }, // Gilbert_Islands
    {   69,  788,       0,  116234,       0,  0, 28,  0, }, // GMT
    {   69,  830,  116262,  116284,  116314, 22, 30, 27, }, // Greenland_Eastern
    {   69,  848,  116341,  116366,  116399, 25, 33, 30, }, // Greenland_Western
    {   69,  866,       0,  116429,       0,  0, 16,  0, }, // Guam
    {   69,  871,       0,  116445,       0,  0, 36,  0, }, // Gulf
    {   69,  876,       0,  116481,       0,  0, 10,  0, }, // Guyana
    {   69,  883,  116491,  116517,  116551, 26, 34, 31, }, // Hawaii_Aleutian
    {   69,  899,  116582,  116595,  116616, 13, 21, 18, }, // Hong_Kong
    {   69,  909,  116634,  116642,  116658,  8, 16, 13, }, // Hovd
    {   69,  914,       0,  116671,       0,  0, 17,  0, }, // India
    {   69,  920,       0,  116688,       0,  0, 17,  0, }, // Indian_Ocean
    {   69,  933,       0,  116705,       0,  0, 14,  0, }, // Indochina
    {   69,  943,       0,  116719,       0,  0, 19,  0, }, // Indonesia_Central
    {   69,  961,       0,  116738,       0,  0, 22,  0, }, // Indonesia_Eastern
    {   69,  979,       0,  116760,       0,  0, 25,  0, }, // Indonesia_Western
    {   69,  997,  116785,  116793,  116809,  8, 16, 13, }, // Iran
    {   69, 1008,  116822,  116833,  116852, 11, 19, 16, }, // Irkutsk
    {   69, 1016,  116868,  116878,  116896, 10, 18, 15, }, // Israel
    {   69, 1023,  116911,  116920,  116937,  9, 17, 14, }, // Japan
    {   69, 1058,       0,  116951,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {   69, 1077,       0,  116973,       0,  0, 25,  0, }, // Kazakhstan_Western
    {   69, 1106,  116998,  117007,  117024,  9, 17, 14, }, // Korea
    {   69, 1112,       0,  117038,       0,  0, 10,  0, }, // Kosrae
    {   69, 1119,  117048,  117063,  117086, 15, 23, 20, }, // Krasnoyarsk
    {   69, 1151,       0,  117106,       0,  0, 14,  0, }, // Kyrgystan
    {   69, 1161,       0,  117120,       0,  0, 13,  0, }, // Lanka
    {   69, 1175,       0,  117133,       0,  0, 16,  0, }, // Line_Islands
    {   69, 1188,  117149,  117162,  117183, 13, 21, 18, }, // Lord_Howe
    {   69, 1198,  117201,  117210,  117227,  9, 17, 14, }, // Macau
    {   69, 1204,  117241,  117252,  117271, 11, 19, 16, }, // Magadan
    {   69, 1219,       0,  117287,       0,  0, 12,  0, }, // Malaysia
    {   69, 1228,       0,  117299,       0,  0, 15,  0, }, // Maldives
    {   69, 1237,       0,  117314,       0,  0, 16,  0, }, // Marquesas
    {   69, 1247,       0,  117330,       0,  0, 18,  0, }, // Marshall_Islands
    {   69, 1264,  117348,  117357,  117374,  9, 17, 14, }, // Mauritius
    {   69, 1274,       0,  117388,       0,  0, 10,  0, }, // Mawson
    {   69, 1281,  117398,  117421,  117452, 23, 31, 28, }, // Mexico_Pacific
    {   69, 1296,  117480,  117495,  117518, 15, 23, 20, }, // Mongolia
    {   69, 1305,  117538,  117548,  117566, 10, 18, 15, }, // Moscow
    {   69, 1312,       0,  117581,       0,  0, 11,  0, }, // Myanmar
    {   69, 1320,       0,  117592,       0,  0,  9,  0, }, // Nauru
    {   69, 1326,       0,  117601,       0,  0,  9,  0, }, // Nepal
    {   69, 1332,  117610,  117629,  117656, 19, 27, 24, }, // New_Caledonia
    {   69, 1346,  117680,  117696,  117720, 16, 24, 21, }, // New_Zealand
    {   69, 1358,  117741,  117757,  117781, 16, 24, 21, }, // Newfoundland
    {   69, 1371,       0,  117802,       0,  0,  8,  0, }, // Niue
    {   69, 1376,  117810,  117826,  117850, 16, 24, 21, }, // Norfolk
    {   69, 1384,  117871,  117894,  117925, 23, 31, 28, }, // Noronha
    {   69, 1392,       0,  117953,       0,  0, 20,  0, }, // North_Mariana
    {   69, 1406,  117973,  117988,  118011, 15, 23, 20, }, // Novosibirsk
    {   69, 1418,  118031,  118039,  118055,  8, 16, 13, }, // Omsk
    {   69, 1428,  118068,  118080,  118100, 12, 20, 17, }, // Pakistan
    {   69, 1437,       0,  118117,       0,  0,  9,  0, }, // Palau
    {   69, 1443,       0,  118126,       0,  0, 22,  0, }, // Papua_New_Guinea
    {   69, 1460,  118148,  118160,  118180, 12, 20, 17, }, // Paraguay
    {   69, 1469,  118197,  118206,  118223,  9, 17, 14, }, // Peru
    {   69, 1474,  118237,  118253,  118277, 16, 24, 21, }, // Philippines
    {   69, 1486,       0,  118298,       0,  0, 17,  0, }, // Phoenix_Islands
    {   69, 1502,  118315,  118338,  118369, 23, 31, 28, }, // Pierre_Miquelon
    {   69, 1518,       0,  118397,       0,  0, 12,  0, }, // Pitcairn
    {   69, 1527,       0,  118409,       0,  0, 11,  0, }, // Ponape
    {   69, 1534,       0,  118420,       0,  0, 14,  0, }, // Pyongyang
    {   69, 1554,       0,  118434,       0,  0, 14,  0, }, // Reunion
    {   69, 1562,       0,  118448,       0,  0, 11,  0, }, // Rothera
    {   69, 1570,  118459,  118471,  118491, 12, 20, 17, }, // Sakhalin
    {   69, 1596,  118508,  118517,  118534,  9, 17, 14, }, // Samoa
    {   69, 1602,       0,  118548,       0,  0, 12,  0, }, // Seychelles
    {   69, 1624,       0,  118560,       0,  0, 21,  0, }, // Singapore
    {   69, 1634,       0,  118581,       0,  0, 17,  0, }, // Solomon
    {   69, 1642,       0,  118598,       0,  0, 17,  0, }, // South_Georgia
    {   69, 1656,       0,  118615,       0,  0, 11,  0, }, // Suriname
    {   69, 1676,       0,  118626,       0,  0,  9,  0, }, // Syowa
    {   69, 1682,       0,  118635,       0,  0, 10,  0, }, // Tahiti
    {   69, 1689,  118645,  118655,  118673, 10, 18, 15, }, // Taipei
    {   69, 1696,       0,  118688,       0,  0, 15,  0, }, // Tajikistan
    {   69, 1724,       0,  118703,       0,  0, 11,  0, }, // Tokelau
    {   69, 1732,  118714,  118723,  118740,  9, 17, 14, }, // Tonga
    {   69, 1738,       0,  118754,       0,  0,  9,  0, }, // Truk
    {   69, 1750,  118763,  118779,  118803, 16, 24, 21, }, // Turkmenistan
    {   69, 1763,       0,  118824,       0,  0, 10,  0, }, // Tuvalu
    {   69, 1777,  118834,  118845,  118864, 11, 19, 16, }, // Uruguay
    {   69, 1792,  118880,  118895,  118918, 15, 23, 20, }, // Uzbekistan
    {   69, 1803,  118938,  118949,  118968, 11, 19, 16, }, // Vanuatu
    {   69, 1811,       0,  118984,       0,  0, 13,  0, }, // Venezuela
    {   69, 1821,  118997,  119012,  119035, 15, 23, 20, }, // Vladivostok
    {   69, 1833,  119055,  119068,  119089, 13, 21, 18, }, // Volgograd
    {   69, 1843,       0,  119107,       0,  0, 10,  0, }, // Vostok
    {   69, 1850,       0,  119117,       0,  0, 15,  0, }, // Wake
    {   69, 1855,       0,  119132,       0,  0, 20,  0, }, // Wallis
    {   69, 1862,  119152,  119163,  119182, 11, 19, 16, }, // Yakutsk
    {   69, 1870,  119198,  119216,  119242, 18, 26, 23, }, // Yekaterinburg
    {   70,    5,       0,  119265,       0,  0, 19,  0, }, // Afghanistan Bulgarian/Cyrillic/Bulgaria
    {   70,   17,       0,  119284,       0,  0, 25,  0, }, // Africa_Central
    {   70,   32,       0,  119309,       0,  0, 23,  0, }, // Africa_Eastern
    {   70,   65,       0,  119332,       0,  0, 20,  0, }, // Africa_Southern
    {   70,   81,  119352,  119375,  119409, 23, 34, 36, }, // Africa_Western
    {   70,  107,  119445,  119451,  119476,  6, 25, 27, }, // Alaska
    {   70,  135,  119503,  119518,  119544, 15, 26, 28, }, // Amazon
    {   70,  142,  119572,  119606,  119651, 34, 45, 47, }, // America_Central
    {   70,  158,  119698,  119730,  119773, 32, 43, 45, }, // America_Eastern
    {   70,  174,  119818,  119852,  119897, 34, 45, 47, }, // America_Mountain
    {   70,  191,  119944,  119981,  120029, 37, 48, 50, }, // America_Pacific
    {   70,  207,  106448,  120079,  120104, 12, 25, 27, }, // Anadyr
    {   70,  214,  120131,  120135,  120158,  4, 23, 25, }, // Apia
    {   70,  232,  120183,  120196,  120220, 13, 24, 26, }, // Arabian
    {   70,  240,  120246,  120263,  120291, 17, 28, 30, }, // Argentina
    {   70,  250,  120321,  120345,  120380, 24, 35, 37, }, // Argentina_Western
    {   70,  268,  120417,  120431,  120456, 14, 25, 27, }, // Armenia
    {   70,  286,  120483,  120520,  120568, 37, 48, 50, }, // Atlantic
    {   70,  295,  120618,  120645,  120683, 27, 38, 40, }, // Australia_Central
    {   70,  313,  120723,  120758,  120804, 35, 46, 48, }, // Australia_CentralWestern
    {   70,  338,  120852,  120877,  120913, 25, 36, 38, }, // Australia_Eastern
    {   70,  356,  120951,  120976,  121012, 25, 36, 38, }, // Australia_Western
    {   70,  374,  121050,  121070,  121101, 20, 31, 33, }, // Azerbaijan
    {   70,  385,  121134,  121149,  121183, 15, 34, 36, }, // Azores
    {   70,  397,  121219,  121236,  121264, 17, 28, 30, }, // Bangladesh
    {   70,  415,       0,  121294,       0,  0, 14,  0, }, // Bhutan
    {   70,  422,       0,  121308,       0,  0, 16,  0, }, // Bolivia
    {   70,  437,  121324,  121339,  121365, 15, 26, 28, }, // Brasilia
    {   70,  454,       0,  121393,       0,  0, 17,  0, }, // Brunei
    {   70,  461,  121410,  121420,  121449, 10, 29, 31, }, // Cape_Verde
    {   70,  478,       0,  121480,       0,  0, 14,  0, }, // Chamorro
    {   70,  487,  121494,  121508,  121533, 14, 25, 27, }, // Chatham
    {   70,  495,  121560,  121574,  121599, 14, 25, 27, }, // Chile
    {   70,  501,  121626,  121640,  121665, 14, 25, 27, }, // China
    {   70,  507,       0,  121692,       0,  0, 16,  0, }, // Christmas
    {   70,  517,       0,  121708,       0,  0, 16,  0, }, // Cocos
    {   70,  523,  121724,  121741,  121769, 17, 28, 30, }, // Colombia
    {   70,  532,  121799,  121810,  121840, 11, 30, 32, }, // Cook
    {   70,  537,  121872,  121886,  121911, 14, 25, 27, }, // Cuba
    {   70,  548,       0,  121938,       0,  0,  6,  0, }, // Davis
    {   70,  564,       0,  121944,       0,  0, 12,  0, }, // DumontDUrville
    {   70,  601,       0,  121956,       0,  0, 21,  0, }, // East_Timor
    {   70,  612,  121977,  121995,  122032, 18, 37, 39, }, // Easter
    {   70,  619,       0,  122071,       0,  0, 16,  0, }, // Ecuador
    {   70,  627,  122087,  122112,  122148, 25, 36, 38, }, // Europe_Central
    {   70,  642,  122186,  122209,  122243, 23, 34, 36, }, // Europe_Eastern
    {   70,  657,       0,  122279,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {   70,  680,  122310,  122333,  122367, 23, 34, 29, }, // Europe_Western
    {   70,  695,  122396,  122415,  122453, 19, 38, 40, }, // Falkland
    {   70,  704,  122493,  122508,  122534, 15, 26, 28, }, // Fiji
    {   70,  709,       0,  122562,       0,  0, 14,  0, }, // French_Guiana
    {   70,  723,       0,  122576,       0,  0, 38,  0, }, // French_Southern
    {   70,  746,       0,  122614,       0,  0, 17,  0, }, // Galapagos
    {   70,  756,       0,  122631,       0,  0,  6,  0, }, // Gambier
    {   70,  764,  122637,  122652,  122678, 15, 26, 28, }, // Georgia
    {   70,  772,       0,  122706,       0,  0, 15,  0, }, // Gilbert_Islands
    {   70,  788,       0,  122721,       0,  0, 22,  0, }, // GMT
    {   70,  830,  122743,  122767,  122802, 24, 35, 37, }, // Greenland_Eastern
    {   70,  848,  122839,  122863,  122898, 24, 35, 37, }, // Greenland_Western
    {   70,  871,       0,  122935,       0,  0, 15,  0, }, // Gulf
    {   70,  876,       0,  122950,       0,  0,  5,  0, }, // Guyana
    {   70,  883,  122955,  122978,  123012, 23, 34, 36, }, // Hawaii_Aleutian
    {   70,  899,  123048,  123064,  123091, 16, 27, 29, }, // Hong_Kong
    {   70,  909,  123120,  123133,  123157, 13, 24, 26, }, // Hovd
    {   70,  914,       0,  123183,       0,  0, 14,  0, }, // India
    {   70,  920,       0,  123197,       0,  0, 14,  0, }, // Indian_Ocean
    {   70,  933,       0,  123211,       0,  0, 18,  0, }, // Indochina
    {   70,  943,       0,  123229,       0,  0, 27,  0, }, // Indonesia_Central
    {   70,  961,       0,  123256,       0,  0, 25,  0, }, // Indonesia_Eastern
    {   70,  979,       0,  123281,       0,  0, 25,  0, }, // Indonesia_Western
    {   70,  997,  123306,  123319,  123343, 13, 24, 26, }, // Iran
    {   70, 1008,  123369,  123383,  123408, 14, 25, 27, }, // Irkutsk
    {   70, 1016,  123435,  123450,  123476, 15, 26, 28, }, // Israel
    {   70, 1023,  123504,  123517,  123541, 13, 24, 26, }, // Japan
    {   70, 1029,  123567,  123596,  123636, 29, 40, 44, }, // Kamchatka
    {   70, 1047,       0,  123680,       0,  0, 18,  0, }, // Kazakhstan
    {   70, 1058,       0,  123698,       0,  0, 25,  0, }, // Kazakhstan_Eastern
    {   70, 1077,       0,  123723,       0,  0, 25,  0, }, // Kazakhstan_Western
    {   70, 1106,  123748,  123762,  123787, 14, 25, 27, }, // Korea
    {   70, 1112,       0,  123814,       0,  0,  6,  0, }, // Kosrae
    {   70, 1119,  123820,  123837,  123865, 17, 28, 30, }, // Krasnoyarsk
    {   70, 1151,       0,  123895,       0,  0, 19,  0, }, // Kyrgystan
    {   70, 1175,       0,  123914,       0,  0, 20,  0, }, // Line_Islands
    {   70, 1188,  123934,  123942,  123969,  8, 27, 29, }, // Lord_Howe
    {   70, 1204,  123998,  124014,  124041, 16, 27, 29, }, // Magadan
    {   70, 1219,       0,  124070,       0,  0, 17,  0, }, // Malaysia
    {   70, 1228,       0,  124087,       0,  0, 15,  0, }, // Maldives
    {   70, 1237,       0,  124102,       0,  0, 16,  0, }, // Marquesas
    {   70, 1247,       0,  124118,       0,  0, 17,  0, }, // Marshall_Islands
    {   70, 1264,  124135,  124143,  124170,  8, 27, 29, }, // Mauritius
    {   70, 1274,       0,  124199,       0,  0,  6,  0, }, // Mawson
    {   70, 1281,  124205,  124235,  124276, 30, 41, 43, }, // Mexico_Pacific
    {   70, 1296,  124319,  124337,  124366, 18, 29, 31, }, // Mongolia
    {   70, 1305,  124397,  124412,  124438, 15, 26, 28, }, // Moscow
    {   70, 1312,       0,  124466,       0,  0, 16,  0, }, // Myanmar
    {   70, 1320,       0,   11131,       0,  0,  5,  0, }, // Nauru
    {   70, 1326,       0,  124482,       0,  0, 14,  0, }, // Nepal
    {   70, 1332,  124496,  124516,  124547, 20, 31, 33, }, // New_Caledonia
    {   70, 1346,  124580,  124599,  124629, 19, 30, 32, }, // New_Zealand
    {   70, 1358,  124661,  124681,  124712, 20, 31, 33, }, // Newfoundland
    {   70, 1371,       0,   11354,       0,  0,  4,  0, }, // Niue
    {   70, 1376,  124745,  124761,  124788, 16, 27, 29, }, // Norfolk
    {   70, 1384,  124817,  124835,  124872, 18, 37, 39, }, // Noronha
    {   70, 1406,  124911,  124929,  124958, 18, 29, 31, }, // Novosibirsk
    {   70, 1418,  124989,  125000,  125022, 11, 22, 24, }, // Omsk
    {   70, 1428,  125046,  125063,  125091, 17, 28, 30, }, // Pakistan
    {   70, 1437,       0,   11702,       0,  0,  5,  0, }, // Palau
    {   70, 1443,       0,  125121,       0,  0, 17,  0, }, // Papua_New_Guinea
    {   70, 1460,  125138,  125155,  125183, 17, 28, 30, }, // Paraguay
    {   70, 1469,  125213,  125228,  125254, 15, 26, 28, }, // Peru
    {   70, 1474,  125282,  125298,  125325, 16, 27, 29, }, // Philippines
    {   70, 1486,       0,  125354,       0,  0, 14,  0, }, // Phoenix_Islands
    {   70, 1502,  125368,  125386,  125423, 18, 37, 39, }, // Pierre_Miquelon
    {   70, 1518,       0,   11991,       0,  0,  7,  0, }, // Pitcairn
    {   70, 1527,       0,  112432,       0,  0,  6,  0, }, // Ponape
    {   70, 1534,       0,  125462,       0,  0, 15,  0, }, // Pyongyang
    {   70, 1554,       0,  125477,       0,  0,  7,  0, }, // Reunion
    {   70, 1562,       0,   12082,       0,  0,  6,  0, }, // Rothera
    {   70, 1570,  125484,  125500,  125527, 16, 27, 29, }, // Sakhalin
    {   70, 1579,  112636,  125556,  125581, 12, 25, 27, }, // Samara
    {   70, 1596,  125608,  125623,  125649, 15, 26, 28, }, // Samoa
    {   70, 1602,       0,  125677,       0,  0,  7,  0, }, // Seychelles
    {   70, 1624,       0,  125684,       0,  0, 17,  0, }, // Singapore
    {   70, 1634,       0,  125701,       0,  0, 18,  0, }, // Solomon
    {   70, 1642,       0,  125719,       0,  0, 13,  0, }, // South_Georgia
    {   70, 1656,       0,  125732,       0,  0, 16,  0, }, // Suriname
    {   70, 1676,       0,  112868,       0,  0,  4,  0, }, // Syowa
    {   70, 1682,       0,  125748,       0,  0, 15,  0, }, // Tahiti
    {   70, 1689,  125763,  125768,  125792,  5, 24, 26, }, // Taipei
    {   70, 1696,       0,  125818,       0,  0, 20,  0, }, // Tajikistan
    {   70, 1724,       0,   12408,       0,  0,  7,  0, }, // Tokelau
    {   70, 1732,   12415,  125838,  125862,  5, 24, 26, }, // Tonga
    {   70, 1738,       0,  125888,       0,  0,  4,  0, }, // Truk
    {   70, 1750,  125892,  125913,  125945, 21, 32, 34, }, // Turkmenistan
    {   70, 1763,       0,   12538,       0,  0,  6,  0, }, // Tuvalu
    {   70, 1777,  125979,  125995,  126022, 16, 27, 29, }, // Uruguay
    {   70, 1792,  126051,  126070,  126100, 19, 30, 32, }, // Uzbekistan
    {   70, 1803,   12666,  126132,  126158,  7, 26, 28, }, // Vanuatu
    {   70, 1811,       0,  126186,       0,  0, 17,  0, }, // Venezuela
    {   70, 1821,  126203,  126223,  126254, 20, 31, 33, }, // Vladivostok
    {   70, 1833,  126287,  126305,  126334, 18, 29, 31, }, // Volgograd
    {   70, 1843,       0,  113571,       0,  0,  6,  0, }, // Vostok
    {   70, 1850,       0,  126365,       0,  0, 11,  0, }, // Wake
    {   70, 1855,       0,  126376,       0,  0, 14,  0, }, // Wallis
    {   70, 1862,  126390,  126403,  126429, 13, 26, 28, }, // Yakutsk
    {   70, 1870,  126457,  126478,  126510, 21, 32, 34, }, // Yekaterinburg
    {   70, 1892,       0,  126544,       0,  0,  4,  0, }, // Yukon
    {   71,    5,       0,  126548,       0,  0, 20,  0, }, // Afghanistan Burmese/Myanmar/Myanmar
    {   71,   17,       0,  126568,       0,  0, 17,  0, }, // Africa_Central
    {   71,   32,       0,  126585,       0,  0, 18,  0, }, // Africa_Eastern
    {   71,   65,       0,  126603,       0,  0, 18,  0, }, // Africa_Southern
    {   71,   81,  126621,  126640,  126664, 19, 24, 27, }, // Africa_Western
    {   71,  107,  126691,  126704,  126722, 13, 18, 26, }, // Alaska
    {   71,  135,  126748,  126761,  126779, 13, 18, 20, }, // Amazon
    {   71,  142,  126799,  126815,  126836, 16, 21, 29, }, // America_Central
    {   71,  158,  126865,  126882,  126904, 17, 22, 30, }, // America_Eastern
    {   71,  174,  126934,  126949,  126969, 15, 20, 28, }, // America_Mountain
    {   71,  191,  126997,  127010,  127028, 13, 18, 26, }, // America_Pacific
    {   71,  214,  127054,  127066,  127083, 12, 17, 20, }, // Apia
    {   71,  232,  127103,  127116,  127134, 13, 18, 21, }, // Arabian
    {   71,  240,  127155,  127174,  127198, 19, 24, 26, }, // Argentina
    {   71,  250,  127224,  127250,  127281, 26, 31, 34, }, // Argentina_Western
    {   71,  268,  127315,  127333,  127356, 18, 23, 26, }, // Armenia
    {   71,  286,  127382,  127399,  127421, 17, 22, 30, }, // Atlantic
    {   71,  295,  127451,  127477,  127508, 26, 31, 34, }, // Australia_Central
    {   71,  313,  127542,  127577,  127617, 35, 40, 43, }, // Australia_CentralWestern
    {   71,  338,  127660,  127680,  127705, 20, 25, 28, }, // Australia_Eastern
    {   71,  356,  127733,  127754,  127710, 21, 26, 23, }, // Australia_Western
    {   71,  374,  127780,  127799,  127823, 19, 24, 27, }, // Azerbaijan
    {   71,  385,  127850,  127868,  127891, 18, 23, 26, }, // Azores
    {   71,  397,  127917,  127938,  127964, 21, 26, 29, }, // Bangladesh
    {   71,  415,       0,  127993,       0,  0, 12,  0, }, // Bhutan
    {   71,  422,       0,  128005,       0,  0, 19,  0, }, // Bolivia
    {   71,  437,  128024,  128037,  128055, 13, 18, 21, }, // Brasilia
    {   71,  454,       0,  128076,       0,  0, 21,  0, }, // Brunei
    {   71,  461,  128097,  128112,  128132, 15, 20, 23, }, // Cape_Verde
    {   71,  478,       0,  128155,       0,  0, 16,  0, }, // Chamorro
    {   71,  487,  128171,  128184,  128202, 13, 18, 22, }, // Chatham
    {   71,  495,  128224,  128236,  128253, 12, 17, 20, }, // Chile
    {   71,  501,  128273,  128285,  128302, 12, 17, 20, }, // China
    {   71,  507,       0,  128322,       0,  0, 21,  0, }, // Christmas
    {   71,  517,       0,  128343,       0,  0, 20,  0, }, // Cocos
    {   71,  523,  128363,  128379,  128400, 16, 21, 24, }, // Colombia
    {   71,  532,  128424,  128443,  128467, 19, 24, 27, }, // Cook
    {   71,  537,  128494,  128508,  128527, 14, 19, 27, }, // Cuba
    {   71,  548,       0,  128554,       0,  0, 13,  0, }, // Davis
    {   71,  564,       0,  128567,       0,  0, 26,  0, }, // DumontDUrville
    {   71,  601,       0,  128593,       0,  0, 17,  0, }, // East_Timor
    {   71,  612,  128610,  128628,  128651, 18, 23, 26, }, // Easter
    {   71,  619,       0,  128677,       0,  0, 15,  0, }, // Ecuador
    {   71,  627,  128692,  128714,  128741, 22, 27, 30, }, // Europe_Central
    {   71,  642,  128771,  128788,  128810, 17, 22, 25, }, // Europe_Eastern
    {   71,  657,       0,  128835,       0,  0, 25,  0, }, // Europe_Further_Eastern
    {   71,  680,  128860,  128878,  128901, 18, 23, 26, }, // Europe_Western
    {   71,  695,  128927,  128950,  128978, 23, 28, 31, }, // Falkland
    {   71,  704,  129009,  129021,  129038, 12, 17, 20, }, // Fiji
    {   71,  709,       0,  129058,       0,  0, 23,  0, }, // French_Guiana
    {   71,  723,       0,  129081,       0,  0, 40,  0, }, // French_Southern
    {   71,  746,       0,  129121,       0,  0, 20,  0, }, // Galapagos
    {   71,  756,       0,  129141,       0,  0, 14,  0, }, // Gambier
    {   71,  764,  129155,  129172,  129194, 17, 22, 25, }, // Georgia
    {   71,  772,       0,  129219,       0,  0, 22,  0, }, // Gilbert_Islands
    {   71,  788,       0,  129241,       0,  0, 20,  0, }, // GMT
    {   71,  830,  129261,  129282,  129308, 21, 26, 33, }, // Greenland_Eastern
    {   71,  848,  129341,  129364,  129392, 23, 28, 35, }, // Greenland_Western
    {   71,  871,       0,  129427,       0,  0, 17,  0, }, // Gulf
    {   71,  876,       0,  129444,       0,  0, 16,  0, }, // Guyana
    {   71,  883,  129460,  129488,  129521, 28, 33, 41, }, // Hawaii_Aleutian
    {   71,  899,  129562,  129579,  129601, 17, 22, 25, }, // Hong_Kong
    {   71,  909,  129626,  129639,  129657, 13, 18, 21, }, // Hovd
    {   71,  914,       0,  129678,       0,  0, 19,  0, }, // India
    {   71,  920,       0,  129697,       0,  0, 22,  0, }, // Indian_Ocean
    {   71,  933,       0,  129719,       0,  0, 23,  0, }, // Indochina
    {   71,  943,       0,  129742,       0,  0, 31,  0, }, // Indonesia_Central
    {   71,  961,       0,  129773,       0,  0, 32,  0, }, // Indonesia_Eastern
    {   71,  979,       0,  129805,       0,  0, 33,  0, }, // Indonesia_Western
    {   71,  997,  129838,  129850,  129867, 12, 17, 20, }, // Iran
    {   71, 1008,  129887,  129902,  129922, 15, 20, 23, }, // Irkutsk
    {   71, 1016,  129945,  129959,  129978, 14, 19, 22, }, // Israel
    {   71, 1023,  130000,  130012,  130029, 12, 17, 20, }, // Japan
    {   71, 1047,       0,  130049,       0,  0, 16,  0, }, // Kazakhstan
    {   71, 1058,       0,  130065,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {   71, 1077,       0,  130086,       0,  0, 22,  0, }, // Kazakhstan_Western
    {   71, 1106,  130108,  130124,  130145, 16, 21, 24, }, // Korea
    {   71, 1112,       0,  130169,       0,  0, 17,  0, }, // Kosrae
    {   71, 1119,  130186,  130206,  130231, 20, 25, 28, }, // Krasnoyarsk
    {   71, 1151,       0,  130259,       0,  0, 17,  0, }, // Kyrgystan
    {   71, 1161,       0,  130276,       0,  0, 17,  0, }, // Lanka
    {   71, 1175,       0,  130293,       0,  0, 21,  0, }, // Line_Islands
    {   71, 1188,  130314,  130332,  130355, 18, 23, 26, }, // Lord_Howe
    {   71, 1198,  130381,  130394,  130412, 13, 18, 21, }, // Macau
    {   71, 1204,  130433,  130448,  130468, 15, 20, 23, }, // Magadan
    {   71, 1219,       0,  130491,       0,  0, 15,  0, }, // Malaysia
    {   71, 1228,       0,  130506,       0,  0, 17,  0, }, // Maldives
    {   71, 1237,       0,  130523,       0,  0, 17,  0, }, // Marquesas
    {   71, 1247,       0,  130540,       0,  0, 21,  0, }, // Marshall_Islands
    {   71, 1264,  130561,  130576,  130596, 15, 20, 23, }, // Mauritius
    {   71, 1274,       0,  130619,       0,  0, 15,  0, }, // Mawson
    {   71, 1281,  130634,  130658,  130687, 24, 29, 37, }, // Mexico_Pacific
    {   71, 1296,  130724,  130740,  130761, 16, 21, 24, }, // Mongolia
    {   71, 1305,  130785,  130800,  130820, 15, 20, 23, }, // Moscow
    {   71, 1312,       0,  130843,       0,  0, 13,  0, }, // Myanmar
    {   71, 1320,       0,  130856,       0,  0, 13,  0, }, // Nauru
    {   71, 1326,       0,  130869,       0,  0, 12,  0, }, // Nepal
    {   71, 1332,  130881,  130907,  130937, 26, 30, 33, }, // New_Caledonia
    {   71, 1346,  130970,  130986,  131007, 16, 21, 24, }, // New_Zealand
    {   71, 1358,  131031,  131050,  131074, 19, 24, 32, }, // Newfoundland
    {   71, 1371,       0,  131106,       0,  0, 13,  0, }, // Niue
    {   71, 1376,  131119,  131141,  131168, 22, 27, 35, }, // Norfolk
    {   71, 1384,  131203,  131231,  131264, 28, 33, 36, }, // Noronha
    {   71, 1406,  131300,  131323,  131351, 23, 28, 31, }, // Novosibirsk
    {   71, 1418,  131382,  131398,  131419, 16, 21, 24, }, // Omsk
    {   71, 1428,  131443,  131459,  131480, 16, 21, 24, }, // Pakistan
    {   71, 1437,       0,  131504,       0,  0, 13,  0, }, // Palau
    {   71, 1443,       0,  131517,       0,  0, 21,  0, }, // Papua_New_Guinea
    {   71, 1460,  131538,  131553,  131573, 15, 20, 23, }, // Paraguay
    {   71, 1469,  131596,  131608,  131625, 12, 17, 20, }, // Peru
    {   71, 1474,  131645,  131662,  131684, 17, 22, 25, }, // Philippines
    {   71, 1486,       0,  131709,       0,  0, 21,  0, }, // Phoenix_Islands
    {   71, 1502,  131730,  131759,  131793, 29, 34, 44, }, // Pierre_Miquelon
    {   71, 1518,       0,  131837,       0,  0, 17,  0, }, // Pitcairn
    {   71, 1527,       0,  131854,       0,  0, 16,  0, }, // Ponape
    {   71, 1534,       0,  131870,       0,  0, 15,  0, }, // Pyongyang
    {   71, 1554,       0,  131885,       0,  0, 15,  0, }, // Reunion
    {   71, 1562,       0,  131900,       0,  0, 14,  0, }, // Rothera
    {   71, 1570,  131914,  131928,  131947, 14, 19, 22, }, // Sakhalin
    {   71, 1596,  131969,  131982,  132000, 13, 18, 23, }, // Samoa
    {   71, 1602,       0,  132023,       0,  0, 13,  0, }, // Seychelles
    {   71, 1624,       0,  132036,       0,  0, 15,  0, }, // Singapore
    {   71, 1634,       0,  132051,       0,  0, 24,  0, }, // Solomon
    {   71, 1642,       0,  132075,       0,  0, 22,  0, }, // South_Georgia
    {   71, 1656,       0,  132097,       0,  0, 15,  0, }, // Suriname
    {   71, 1676,       0,  132112,       0,  0, 13,  0, }, // Syowa
    {   71, 1682,       0,  132125,       0,  0, 12,  0, }, // Tahiti
    {   71, 1689,  132137,  132151,  132170, 14, 19, 22, }, // Taipei
    {   71, 1696,       0,  132192,       0,  0, 20,  0, }, // Tajikistan
    {   71, 1724,       0,  132212,       0,  0, 15,  0, }, // Tokelau
    {   71, 1732,  132227,  132240,  132258, 13, 18, 21, }, // Tonga
    {   71, 1738,       0,  132279,       0,  0, 12,  0, }, // Truk
    {   71, 1750,  132291,  132313,  132340, 22, 27, 30, }, // Turkmenistan
    {   71, 1763,       0,  132370,       0,  0, 14,  0, }, // Tuvalu
    {   71, 1777,  132384,  132398,  132417, 14, 19, 22, }, // Uruguay
    {   71, 1792,  132439,  132458,  132482, 19, 24, 27, }, // Uzbekistan
    {   71, 1803,  132509,  132523,  132542, 14, 19, 22, }, // Vanuatu
    {   71, 1811,       0,  132564,       0,  0, 18,  0, }, // Venezuela
    {   71, 1821,  132582,  132605,  132633, 23, 28, 31, }, // Vladivostok
    {   71, 1833,  132664,  132683,  132707, 19, 24, 25, }, // Volgograd
    {   71, 1843,       0,  132732,       0,  0, 17,  0, }, // Vostok
    {   71, 1850,       0,  132749,       0,  0, 19,  0, }, // Wake
    {   71, 1855,       0,  132768,       0,  0, 27,  0, }, // Wallis
    {   71, 1862,  132795,  132811,  132832, 16, 21, 24, }, // Yakutsk
    {   71, 1870,  132856,  132879,  132907, 23, 28, 30, }, // Yekaterinburg
    {   71, 1892,       0,  132937,       0,  0, 14,  0, }, // Yukon
    {   72,    0,  132951,  132955,  132961,  4,  6,  6, }, // Acre Cantonese/Traditional Han/Hong Kong
    {   72,    5,       0,  132967,       0,  0,  5,  0, }, // Afghanistan
    {   72,   17,       0,  132972,       0,  0,  4,  0, }, // Africa_Central
    {   72,   32,       0,  132976,       0,  0,  4,  0, }, // Africa_Eastern
    {   72,   65,       0,  132980,       0,  0,  6,  0, }, // Africa_Southern
    {   72,   81,  132986,  132990,  132996,  4,  6,  6, }, // Africa_Western
    {   72,  107,  133002,  133008,  133016,  6,  8,  8, }, // Alaska
    {   72,  128,  133024,  133030,  133038,  6,  8,  8, }, // Almaty
    {   72,  135,  133046,  133051,  133058,  5,  7,  7, }, // Amazon
    {   72,  142,  133065,  133069,  133075,  4,  6,  6, }, // America_Central
    {   72,  158,  133081,  133085,  133091,  4,  6,  6, }, // America_Eastern
    {   72,  174,  133097,  133101,  133107,  4,  6,  6, }, // America_Mountain
    {   72,  191,  133113,  133118,  133125,  5,  7,  7, }, // America_Pacific
    {   72,  207,  133132,  133138,  133146,  6,  8,  8, }, // Anadyr
    {   72,  214,  133154,  133159,  133166,  5,  7,  7, }, // Apia
    {   72,  219,  133173,  133178,  133185,  5,  7,  7, }, // Aqtau
    {   72,  225,  133192,  133198,  133206,  6,  8,  8, }, // Aqtobe
    {   72,  232,  133214,  133219,  133226,  5,  7,  7, }, // Arabian
    {   72,  240,  133233,  133238,  133245,  5,  7,  7, }, // Argentina
    {   72,  250,  133252,  133259,  133268,  7,  9,  9, }, // Argentina_Western
    {   72,  268,  133277,  133283,  133291,  6,  8,  8, }, // Armenia
    {   72,  286,  133299,  133304,  133311,  5,  7,  7, }, // Atlantic
    {   72,  295,  133318,  133324,  133332,  6,  8,  8, }, // Australia_Central
    {   72,  313,  133340,  133347,  133356,  7,  9,  9, }, // Australia_CentralWestern
    {   72,  338,  133365,  133371,  133379,  6,  8,  8, }, // Australia_Eastern
    {   72,  356,  133387,  133393,  133401,  6,  8,  8, }, // Australia_Western
    {   72,  374,  133409,  133415,  133423,  6,  8,  8, }, // Azerbaijan
    {   72,  385,  133431,  133438,  133447,  7,  9,  9, }, // Azores
    {   72,  397,  133456,  133461,  133468,  5,  7,  7, }, // Bangladesh
    {   72,  415,       0,  133475,       0,  0,  4,  0, }, // Bhutan
    {   72,  422,       0,  133479,       0,  0,  6,  0, }, // Bolivia
    {   72,  437,  133485,  133491,  133499,  6,  8,  8, }, // Brasilia
    {   72,  454,       0,  133507,       0,  0,  4,  0, }, // Brunei
    {   72,  461,  133511,  133516,  133523,  5,  7,  7, }, // Cape_Verde
    {   72,  472,       0,  133530,       0,  0,  5,  0, }, // Casey
    {   72,  478,       0,  133535,       0,  0,  5,  0, }, // Chamorro
    {   72,  487,  133540,  133546,  133554,  6,  8,  8, }, // Chatham
    {   72,  495,  133562,  133566,  133572,  4,  6,  6, }, // Chile
    {   72,  501,  133578,  133582,  133588,  4,  6,  6, }, // China
    {   72,  507,       0,  133594,       0,  0,  5,  0, }, // Christmas
    {   72,  517,       0,  133599,       0,  0,  7,  0, }, // Cocos
    {   72,  523,  133606,  133612,  133620,  6,  8,  8, }, // Colombia
    {   72,  532,  133628,  133634,  133642,  6,  8,  9, }, // Cook
    {   72,  537,  133651,  133655,  133661,  4,  6,  6, }, // Cuba
    {   72,  548,       0,  133667,       0,  0,  5,  0, }, // Davis
    {   72,  564,       0,  133672,       0,  0,  7,  0, }, // DumontDUrville
    {   72,  601,       0,  133679,       0,  0,  5,  0, }, // East_Timor
    {   72,  612,  133684,  133690,  133698,  6,  8,  8, }, // Easter
    {   72,  619,       0,  133706,       0,  0,  5,  0, }, // Ecuador
    {   72,  627,  133711,  133715,  133721,  4,  6,  6, }, // Europe_Central
    {   72,  642,  133727,  133731,  133737,  4,  6,  6, }, // Europe_Eastern
    {   72,  657,       0,  133743,       0,  0,  6,  0, }, // Europe_Further_Eastern
    {   72,  680,  133749,  133753,  133759,  4,  6,  6, }, // Europe_Western
    {   72,  695,  133765,  133772,  133781,  7,  9,  9, }, // Falkland
    {   72,  704,  133790,  133794,  133800,  4,  6,  6, }, // Fiji
    {   72,  709,       0,  133806,       0,  0,  7,  0, }, // French_Guiana
    {   72,  723,       0,  133813,       0,  0,  9,  0, }, // French_Southern
    {   72,  746,       0,  133822,       0,  0,  8,  0, }, // Galapagos
    {   72,  756,       0,  133830,       0,  0,  7,  0, }, // Gambier
    {   72,  764,  133837,  133842,  133849,  5,  7,  7, }, // Georgia
    {   72,  772,       0,  133856,       0,  0,  8,  0, }, // Gilbert_Islands
    {   72,  788,       0,  133864,       0,  0,  8,  0, }, // GMT
    {   72,  830,  133872,  133879,  133888,  7,  9,  9, }, // Greenland_Eastern
    {   72,  848,  133897,  133904,  133913,  7,  9,  9, }, // Greenland_Western
    {   72,  866,       0,  133922,       0,  0,  6,  0, }, // Guam
    {   72,  871,       0,  133928,       0,  0,  9,  0, }, // Gulf
    {   72,  876,       0,  133937,       0,  0,  5,  0, }, // Guyana
    {   72,  883,  133942,  133951,  133962,  9, 11, 11, }, // Hawaii_Aleutian
    {   72,  899,  133973,  133977,  133983,  4,  6,  6, }, // Hong_Kong
    {   72,  909,  133989,  133994,  134001,  5,  7,  7, }, // Hovd
    {   72,  914,       0,  134008,       0,  0,  6,  0, }, // India
    {   72,  920,       0,  134014,       0,  0,  5,  0, }, // Indian_Ocean
    {   72,  933,       0,  134019,       0,  0,  6,  0, }, // Indochina
    {   72,  943,       0,  134025,       0,  0,  6,  0, }, // Indonesia_Central
    {   72,  961,       0,  134031,       0,  0,  6,  0, }, // Indonesia_Eastern
    {   72,  979,       0,  134037,       0,  0,  6,  0, }, // Indonesia_Western
    {   72,  997,  134043,  134047,  134053,  4,  6,  6, }, // Iran
    {   72, 1008,  134059,  134066,  134075,  7,  9,  9, }, // Irkutsk
    {   72, 1016,  134084,  134089,  134096,  5,  7,  7, }, // Israel
    {   72, 1023,  134103,  134107,  134113,  4,  6,  6, }, // Japan
    {   72, 1029,  134119,  134130,  134143, 11, 13, 15, }, // Kamchatka
    {   72, 1047,       0,  134158,       0,  0,  5,  0, }, // Kazakhstan
    {   72, 1058,       0,  134163,       0,  0,  6,  0, }, // Kazakhstan_Eastern
    {   72, 1077,       0,  134169,       0,  0,  6,  0, }, // Kazakhstan_Western
    {   72, 1106,  134175,  134179,  134185,  4,  6,  6, }, // Korea
    {   72, 1112,       0,  134191,       0,  0,  5,  0, }, // Kosrae
    {   72, 1119,  134196,  134206,  134218, 10, 12, 12, }, // Krasnoyarsk
    {   72, 1151,       0,  134230,       0,  0,  6,  0, }, // Kyrgystan
    {   72, 1161,       0,  134236,       0,  0,  4,  0, }, // Lanka
    {   72, 1175,       0,  134240,       0,  0,  6,  0, }, // Line_Islands
    {   72, 1188,  134246,  134252,  134260,  6,  8,  8, }, // Lord_Howe
    {   72, 1198,  134268,  134272,  134278,  4,  6,  6, }, // Macau
    {   72, 1204,  134284,  134289,  134296,  5,  7,  7, }, // Magadan
    {   72, 1219,       0,  134303,       0,  0,  6,  0, }, // Malaysia
    {   72, 1228,       0,  134309,       0,  0,  6,  0, }, // Maldives
    {   72, 1237,       0,  134315,       0,  0,  6,  0, }, // Marquesas
    {   72, 1247,       0,  134321,       0,  0,  7,  0, }, // Marshall_Islands
    {   72, 1264,  134328,  134334,  134342,  6,  8,  8, }, // Mauritius
    {   72, 1274,       0,  134350,       0,  0,  4,  0, }, // Mawson
    {   72, 1281,  134354,  134362,  134372,  8, 10, 10, }, // Mexico_Pacific
    {   72, 1296,  134382,  134388,  134396,  6,  8,  8, }, // Mongolia
    {   72, 1305,  134404,  134409,  134416,  5,  7,  7, }, // Moscow
    {   72, 1312,       0,  134423,       0,  0,  4,  0, }, // Myanmar
    {   72, 1320,       0,  134427,       0,  0,  4,  0, }, // Nauru
    {   72, 1326,       0,  134431,       0,  0,  5,  0, }, // Nepal
    {   72, 1332,  134436,  134444,  134454,  8, 10, 12, }, // New_Caledonia
    {   72, 1346,  134466,  134471,  134478,  5,  7,  7, }, // New_Zealand
    {   72, 1358,  134485,  134490,  134497,  5,  7,  7, }, // Newfoundland
    {   72, 1371,       0,  134504,       0,  0,  5,  0, }, // Niue
    {   72, 1376,  134509,  134515,  134523,  6,  8,  8, }, // Norfolk
    {   72, 1384,  134531,  134545,  134561, 14, 16, 16, }, // Noronha
    {   72, 1392,       0,  134577,       0,  0,  9,  0, }, // North_Mariana
    {   72, 1406,  134586,  134593,  134602,  7,  9,  9, }, // Novosibirsk
    {   72, 1418,  134611,  134617,  134625,  6,  8,  8, }, // Omsk
    {   72, 1428,  134633,  134639,  134647,  6,  8,  8, }, // Pakistan
    {   72, 1437,       0,  134655,       0,  0,  4,  0, }, // Palau
    {   72, 1443,       0,  134659,       0,  0,  9,  0, }, // Papua_New_Guinea
    {   72, 1460,  134668,  134673,  134680,  5,  7,  7, }, // Paraguay
    {   72, 1469,  134687,  134691,  134697,  4,  6,  6, }, // Peru
    {   72, 1474,  134703,  134708,  134715,  5,  7,  7, }, // Philippines
    {   72, 1486,       0,  134722,       0,  0,  6,  0, }, // Phoenix_Islands
    {   72, 1502,  134728,  134740,  134754, 12, 14, 14, }, // Pierre_Miquelon
    {   72, 1518,       0,  134768,       0,  0,  5,  0, }, // Pitcairn
    {   72, 1527,       0,  134773,       0,  0,  5,  0, }, // Ponape
    {   72, 1534,       0,  134778,       0,  0,  4,  0, }, // Pyongyang
    {   72, 1544,  134782,  134790,  134800,  8, 10, 10, }, // Qyzylorda
    {   72, 1554,       0,  134810,       0,  0,  5,  0, }, // Reunion
    {   72, 1562,       0,  134815,       0,  0,  5,  0, }, // Rothera
    {   72, 1570,  134820,  134825,  134832,  5,  7,  7, }, // Sakhalin
    {   72, 1579,  134839,  134844,  134851,  5,  7,  7, }, // Samara
    {   72, 1596,  134858,  134863,  134870,  5,  7,  7, }, // Samoa
    {   72, 1602,       0,  134877,       0,  0,  5,  0, }, // Seychelles
    {   72, 1624,       0,  134882,       0,  0,  7,  0, }, // Singapore
    {   72, 1634,       0,  134889,       0,  0,  7,  0, }, // Solomon
    {   72, 1642,       0,  134896,       0,  0,  6,  0, }, // South_Georgia
    {   72, 1656,       0,  134902,       0,  0,  5,  0, }, // Suriname
    {   72, 1676,       0,  134907,       0,  0,  6,  0, }, // Syowa
    {   72, 1682,       0,  134913,       0,  0,  5,  0, }, // Tahiti
    {   72, 1689,  134918,  134922,  134928,  4,  6,  6, }, // Taipei
    {   72, 1696,       0,  134934,       0,  0,  5,  0, }, // Tajikistan
    {   72, 1724,       0,  134939,       0,  0,  7,  0, }, // Tokelau
    {   72, 1732,  134946,  134950,  134956,  4,  6,  6, }, // Tonga
    {   72, 1738,       0,  134962,       0,  0,  5,  0, }, // Truk
    {   72, 1750,  134967,  134972,  134979,  5,  7,  7, }, // Turkmenistan
    {   72, 1763,       0,  134986,       0,  0,  5,  0, }, // Tuvalu
    {   72, 1777,  134991,  134996,  135003,  5,  7,  7, }, // Uruguay
    {   72, 1792,  135010,  135016,  135024,  6,  8,  8, }, // Uzbekistan
    {   72, 1803,  135032,  135037,  135044,  5,  7,  7, }, // Vanuatu
    {   72, 1811,       0,  135051,       0,  0,  6,  0, }, // Venezuela
    {   72, 1821,  135057,  135062,  135069,  5,  7,  7, }, // Vladivostok
    {   72, 1833,  135076,  135083,  135092,  7,  9,  9, }, // Volgograd
    {   72, 1843,       0,  135101,       0,  0,  6,  0, }, // Vostok
    {   72, 1850,       0,  135107,       0,  0,  5,  0, }, // Wake
    {   72, 1855,       0,  135112,       0,  0, 11,  0, }, // Wallis
    {   72, 1862,  135123,  135129,  135137,  6,  8,  8, }, // Yakutsk
    {   72, 1870,  135145,  135152,  135161,  7,  9,  9, }, // Yekaterinburg
    {   72, 1892,       0,  135170,       0,  0,  4,  0, }, // Yukon
    {   73,    0,  135174,  135178,  135184,  4,  6,  6, }, // Acre Cantonese/Simplified Han/China
    {   73,    5,       0,  135190,       0,  0,  5,  0, }, // Afghanistan
    {   73,   17,       0,  135195,       0,  0,  4,  0, }, // Africa_Central
    {   73,   32,       0,  135199,       0,  0,  4,  0, }, // Africa_Eastern
    {   73,   65,       0,  135203,       0,  0,  6,  0, }, // Africa_Southern
    {   73,   81,  135209,  135213,  135219,  4,  6,  6, }, // Africa_Western
    {   73,  107,  135225,  135231,  135239,  6,  8,  8, }, // Alaska
    {   73,  128,  135247,  135253,  135261,  6,  8,  8, }, // Almaty
    {   73,  135,  135269,  135274,  135281,  5,  7,  7, }, // Amazon
    {   73,  142,  135288,  135292,  135298,  4,  6,  6, }, // America_Central
    {   73,  158,  135304,  135308,  135314,  4,  6,  6, }, // America_Eastern
    {   73,  174,  135320,  135324,  135330,  4,  6,  6, }, // America_Mountain
    {   73,  191,  135336,  135341,  135348,  5,  7,  7, }, // America_Pacific
    {   73,  207,  135355,  135361,  135369,  6,  8,  8, }, // Anadyr
    {   73,  214,  135377,  135382,  135389,  5,  7,  7, }, // Apia
    {   73,  219,  135396,  135401,  135408,  5,  7,  7, }, // Aqtau
    {   73,  225,  135415,  135421,  135429,  6,  8,  8, }, // Aqtobe
    {   73,  232,  135437,  135442,  135449,  5,  7,  7, }, // Arabian
    {   73,  240,  135456,  135461,  135468,  5,  7,  7, }, // Argentina
    {   73,  250,  135475,  135482,  135491,  7,  9,  9, }, // Argentina_Western
    {   73,  268,  135500,  135506,  135514,  6,  8,  8, }, // Armenia
    {   73,  286,  135522,  135527,  135534,  5,  7,  7, }, // Atlantic
    {   73,  295,  135541,  135547,  135555,  6,  8,  8, }, // Australia_Central
    {   73,  313,  135563,  135570,  135579,  7,  9,  9, }, // Australia_CentralWestern
    {   73,  338,  135588,  135594,  135602,  6,  8,  8, }, // Australia_Eastern
    {   73,  356,  135610,  135616,  135624,  6,  8,  8, }, // Australia_Western
    {   73,  374,  135632,  135638,  135646,  6,  8,  8, }, // Azerbaijan
    {   73,  385,  135654,  135661,  135670,  7,  9,  9, }, // Azores
    {   73,  397,  135679,  135684,  135691,  5,  7,  7, }, // Bangladesh
    {   73,  415,       0,  135698,       0,  0,  4,  0, }, // Bhutan
    {   73,  422,       0,  135702,       0,  0,  6,  0, }, // Bolivia
    {   73,  437,  135708,  135714,  135722,  6,  8,  8, }, // Brasilia
    {   73,  454,       0,  135730,       0,  0,  4,  0, }, // Brunei
    {   73,  461,  135734,  135739,  135746,  5,  7,  7, }, // Cape_Verde
    {   73,  472,       0,  135753,       0,  0,  5,  0, }, // Casey
    {   73,  478,       0,  135758,       0,  0,  5,  0, }, // Chamorro
    {   73,  487,  135763,  135769,  135777,  6,  8,  8, }, // Chatham
    {   73,  495,  135785,  135789,  135795,  4,  6,  6, }, // Chile
    {   73,  501,  135801,  135805,  135811,  4,  6,  6, }, // China
    {   73,  507,       0,  135817,       0,  0,  5,  0, }, // Christmas
    {   73,  517,       0,  135822,       0,  0,  7,  0, }, // Cocos
    {   73,  523,  135829,  135835,  135843,  6,  8,  8, }, // Colombia
    {   73,  532,  135851,  135857,  135865,  6,  8,  9, }, // Cook
    {   73,  537,  135874,  135878,  135884,  4,  6,  6, }, // Cuba
    {   73,  548,       0,  135890,       0,  0,  5,  0, }, // Davis
    {   73,  564,       0,  135895,       0,  0,  7,  0, }, // DumontDUrville
    {   73,  601,       0,  135902,       0,  0,  5,  0, }, // East_Timor
    {   73,  612,  135907,  135913,  135921,  6,  8,  8, }, // Easter
    {   73,  619,       0,  135929,       0,  0,  5,  0, }, // Ecuador
    {   73,  627,  135934,  135938,  135944,  4,  6,  6, }, // Europe_Central
    {   73,  642,  135950,  135954,  135960,  4,  6,  6, }, // Europe_Eastern
    {   73,  657,       0,  135966,       0,  0,  6,  0, }, // Europe_Further_Eastern
    {   73,  680,  135972,  135976,  135982,  4,  6,  6, }, // Europe_Western
    {   73,  695,  135988,  135995,  136004,  7,  9,  9, }, // Falkland
    {   73,  704,  136013,  136017,  136023,  4,  6,  6, }, // Fiji
    {   73,  709,       0,  136029,       0,  0,  7,  0, }, // French_Guiana
    {   73,  723,       0,  136036,       0,  0,  9,  0, }, // French_Southern
    {   73,  746,       0,  136045,       0,  0,  8,  0, }, // Galapagos
    {   73,  756,       0,  136053,       0,  0,  7,  0, }, // Gambier
    {   73,  764,  136060,  136065,  136072,  5,  7,  7, }, // Georgia
    {   73,  772,       0,  136079,       0,  0,  8,  0, }, // Gilbert_Islands
    {   73,  788,       0,  136087,       0,  0,  8,  0, }, // GMT
    {   73,  830,  136095,  136102,  136111,  7,  9,  9, }, // Greenland_Eastern
    {   73,  848,  136120,  136127,  136136,  7,  9,  9, }, // Greenland_Western
    {   73,  866,       0,  136145,       0,  0,  6,  0, }, // Guam
    {   73,  871,       0,  136151,       0,  0,  9,  0, }, // Gulf
    {   73,  876,       0,  136160,       0,  0,  5,  0, }, // Guyana
    {   73,  883,  136165,  136174,  136185,  9, 11, 11, }, // Hawaii_Aleutian
    {   73,  899,  136196,  136200,  136206,  4,  6,  6, }, // Hong_Kong
    {   73,  909,  136212,  136217,  136224,  5,  7,  7, }, // Hovd
    {   73,  914,       0,  136231,       0,  0,  6,  0, }, // India
    {   73,  920,       0,  136237,       0,  0,  5,  0, }, // Indian_Ocean
    {   73,  933,       0,  136242,       0,  0,  6,  0, }, // Indochina
    {   73,  943,       0,  136248,       0,  0,  6,  0, }, // Indonesia_Central
    {   73,  961,       0,  136254,       0,  0,  6,  0, }, // Indonesia_Eastern
    {   73,  979,       0,  136260,       0,  0,  6,  0, }, // Indonesia_Western
    {   73,  997,  136266,  136270,  136276,  4,  6,  6, }, // Iran
    {   73, 1008,  136282,  136289,  136298,  7,  9,  9, }, // Irkutsk
    {   73, 1016,  136307,  136312,  136319,  5,  7,  7, }, // Israel
    {   73, 1023,  136326,  136330,  136336,  4,  6,  6, }, // Japan
    {   73, 1029,  136342,  136353,  136366, 11, 13, 15, }, // Kamchatka
    {   73, 1047,       0,  136381,       0,  0,  5,  0, }, // Kazakhstan
    {   73, 1058,       0,  136386,       0,  0,  6,  0, }, // Kazakhstan_Eastern
    {   73, 1077,       0,  136392,       0,  0,  6,  0, }, // Kazakhstan_Western
    {   73, 1106,  136398,  136402,  136408,  4,  6,  6, }, // Korea
    {   73, 1112,       0,  136414,       0,  0,  5,  0, }, // Kosrae
    {   73, 1119,  136419,  136429,  136441, 10, 12, 12, }, // Krasnoyarsk
    {   73, 1151,       0,  136453,       0,  0,  6,  0, }, // Kyrgystan
    {   73, 1161,       0,  136459,       0,  0,  4,  0, }, // Lanka
    {   73, 1175,       0,  136463,       0,  0,  6,  0, }, // Line_Islands
    {   73, 1188,  136469,  136475,  136483,  6,  8,  8, }, // Lord_Howe
    {   73, 1198,  136491,  136495,  136501,  4,  6,  6, }, // Macau
    {   73, 1204,  136507,  136512,  136519,  5,  7,  7, }, // Magadan
    {   73, 1219,       0,  136526,       0,  0,  6,  0, }, // Malaysia
    {   73, 1228,       0,  136532,       0,  0,  6,  0, }, // Maldives
    {   73, 1237,       0,  136538,       0,  0,  6,  0, }, // Marquesas
    {   73, 1247,       0,  136544,       0,  0,  7,  0, }, // Marshall_Islands
    {   73, 1264,  136551,  136557,  136565,  6,  8,  8, }, // Mauritius
    {   73, 1274,       0,  136573,       0,  0,  4,  0, }, // Mawson
    {   73, 1281,  136577,  136585,  136595,  8, 10, 10, }, // Mexico_Pacific
    {   73, 1296,  136605,  136611,  136619,  6,  8,  8, }, // Mongolia
    {   73, 1305,  136627,  136632,  136639,  5,  7,  7, }, // Moscow
    {   73, 1312,       0,  136646,       0,  0,  4,  0, }, // Myanmar
    {   73, 1320,       0,  136650,       0,  0,  4,  0, }, // Nauru
    {   73, 1326,       0,  136654,       0,  0,  5,  0, }, // Nepal
    {   73, 1332,  136659,  136667,  136677,  8, 10, 12, }, // New_Caledonia
    {   73, 1346,  136689,  136694,  136701,  5,  7,  7, }, // New_Zealand
    {   73, 1358,  136708,  136713,  136720,  5,  7,  7, }, // Newfoundland
    {   73, 1371,       0,  136727,       0,  0,  5,  0, }, // Niue
    {   73, 1376,  136732,  136738,  136746,  6,  8,  8, }, // Norfolk
    {   73, 1384,  136754,  136768,  136784, 14, 16, 16, }, // Noronha
    {   73, 1392,       0,  136800,       0,  0,  9,  0, }, // North_Mariana
    {   73, 1406,  136809,  136816,  136825,  7,  9,  9, }, // Novosibirsk
    {   73, 1418,  136834,  136840,  136848,  6,  8,  8, }, // Omsk
    {   73, 1428,  136856,  136862,  136870,  6,  8,  8, }, // Pakistan
    {   73, 1437,       0,  136878,       0,  0,  4,  0, }, // Palau
    {   73, 1443,       0,  136882,       0,  0,  9,  0, }, // Papua_New_Guinea
    {   73, 1460,  136891,  136896,  136903,  5,  7,  7, }, // Paraguay
    {   73, 1469,  136910,  136914,  136920,  4,  6,  6, }, // Peru
    {   73, 1474,  136926,  136931,  136938,  5,  7,  7, }, // Philippines
    {   73, 1486,       0,  136945,       0,  0,  6,  0, }, // Phoenix_Islands
    {   73, 1502,  136951,  136963,  136977, 12, 14, 14, }, // Pierre_Miquelon
    {   73, 1518,       0,  136991,       0,  0,  5,  0, }, // Pitcairn
    {   73, 1527,       0,  136996,       0,  0,  5,  0, }, // Ponape
    {   73, 1534,       0,  137001,       0,  0,  4,  0, }, // Pyongyang
    {   73, 1544,  137005,  137013,  137023,  8, 10, 10, }, // Qyzylorda
    {   73, 1554,       0,  137033,       0,  0,  5,  0, }, // Reunion
    {   73, 1562,       0,  137038,       0,  0,  5,  0, }, // Rothera
    {   73, 1570,  137043,  137048,  137055,  5,  7,  7, }, // Sakhalin
    {   73, 1579,  137062,  137067,  137074,  5,  7,  7, }, // Samara
    {   73, 1596,  137081,  137086,  137093,  5,  7,  7, }, // Samoa
    {   73, 1602,       0,  137100,       0,  0,  5,  0, }, // Seychelles
    {   73, 1624,       0,  137105,       0,  0,  7,  0, }, // Singapore
    {   73, 1634,       0,  137112,       0,  0,  7,  0, }, // Solomon
    {   73, 1642,       0,  137119,       0,  0,  6,  0, }, // South_Georgia
    {   73, 1656,       0,  137125,       0,  0,  5,  0, }, // Suriname
    {   73, 1676,       0,  137130,       0,  0,  6,  0, }, // Syowa
    {   73, 1682,       0,  137136,       0,  0,  5,  0, }, // Tahiti
    {   73, 1689,  137141,  137145,  137151,  4,  6,  6, }, // Taipei
    {   73, 1696,       0,  137157,       0,  0,  5,  0, }, // Tajikistan
    {   73, 1724,       0,  137162,       0,  0,  7,  0, }, // Tokelau
    {   73, 1732,  137169,  137173,  137179,  4,  6,  6, }, // Tonga
    {   73, 1738,       0,  137185,       0,  0,  5,  0, }, // Truk
    {   73, 1750,  137190,  137195,  137202,  5,  7,  7, }, // Turkmenistan
    {   73, 1763,       0,  137209,       0,  0,  5,  0, }, // Tuvalu
    {   73, 1777,  137214,  137219,  137226,  5,  7,  7, }, // Uruguay
    {   73, 1792,  137233,  137239,  137247,  6,  8,  8, }, // Uzbekistan
    {   73, 1803,  137255,  137260,  137267,  5,  7,  7, }, // Vanuatu
    {   73, 1811,       0,  137274,       0,  0,  6,  0, }, // Venezuela
    {   73, 1821,  137280,  137285,  137292,  5,  7,  7, }, // Vladivostok
    {   73, 1833,  137299,  137306,  137315,  7,  9,  9, }, // Volgograd
    {   73, 1843,       0,  137324,       0,  0,  6,  0, }, // Vostok
    {   73, 1850,       0,  137330,       0,  0,  5,  0, }, // Wake
    {   73, 1855,       0,  137335,       0,  0, 11,  0, }, // Wallis
    {   73, 1862,  137346,  137352,  137360,  6,  8,  8, }, // Yakutsk
    {   73, 1870,  137368,  137375,  137384,  7,  9,  9, }, // Yekaterinburg
    {   73, 1892,       0,  137393,       0,  0,  4,  0, }, // Yukon
    {   76,    5,       0,  137397,       0,  0, 20,  0, }, // Afghanistan Catalan/Latin/Spain
    {   76,   17,       0,  137417,       0,  0, 24,  0, }, // Africa_Central
    {   76,   32,       0,  137441,       0,  0, 25,  0, }, // Africa_Eastern
    {   76,   65,       0,  137466,       0,  0, 37,  0, }, // Africa_Southern
    {   76,   81,  137503,  137530,  137567, 27, 37, 35, }, // Africa_Western
    {   76,  107,   57183,  137602,  137625, 13, 23, 21, }, // Alaska
    {   76,  135,  137646,  137664,  137692, 18, 28, 26, }, // Amazon
    {   76,  142,  137718,  137749,  137790, 31, 41, 39, }, // America_Central
    {   76,  158,  137829,  137861,  137903, 32, 42, 40, }, // America_Eastern
    {   76,  174,  137943,  137978,  138023, 35, 45, 43, }, // America_Mountain
    {   76,  191,  138066,  138101,  138146, 35, 45, 43, }, // America_Pacific
    {   76,  207,  138189,  138202,  138225, 13, 23, 23, }, // Anadyr
    {   76,  214,   57864,  138248,  138269, 11, 21, 19, }, // Apia
    {   76,  232,  138288,  138297,  138316,  9, 19, 17, }, // Arabian
    {   76,  240,  138333,  138352,  138381, 19, 29, 27, }, // Argentina
    {   76,  250,  138408,  138437,  138476, 29, 39, 37, }, // Argentina_Western
    {   76,  268,  138513,  138527,  138551, 14, 24, 22, }, // Armenia
    {   76,  286,  138573,  138591,  138619, 18, 28, 26, }, // Atlantic
    {   76,  295,  138645,  138669,  138703, 24, 34, 32, }, // Australia_Central
    {   76,  313,  138735,  138769,  138813, 34, 44, 42, }, // Australia_CentralWestern
    {   76,  338,  138855,  138880,  138915, 25, 35, 33, }, // Australia_Eastern
    {   76,  356,  138948,  138975,  139012, 27, 37, 35, }, // Australia_Western
    {   76,  374,  139047,  139068,  139099, 21, 31, 29, }, // Azerbaijan
    {   76,  385,  139128,  139146,  139174, 18, 28, 26, }, // Azores
    {   76,  397,  139200,  139218,  139246, 18, 28, 26, }, // Bangladesh
    {   76,  415,       0,  139272,       0,  0, 14,  0, }, // Bhutan
    {   76,  422,       0,  139286,       0,  0, 15,  0, }, // Bolivia
    {   76,  437,  139301,  139317,  139343, 16, 26, 24, }, // Brasilia
    {   76,  454,       0,  139367,       0,  0, 25,  0, }, // Brunei
    {   76,  461,  139392,  139408,  139434, 16, 26, 24, }, // Cape_Verde
    {   76,  478,       0,  139458,       0,  0, 26,  0, }, // Chamorro
    {   76,  487,   59218,  139484,  139509, 15, 25, 23, }, // Chatham
    {   76,  495,  139532,  139544,  139566, 12, 22, 20, }, // Chile
    {   76,  501,  139586,  139601,  139626, 15, 25, 23, }, // China
    {   76,  507,       0,  139649,       0,  0, 18,  0, }, // Christmas
    {   76,  517,       0,  139667,       0,  0, 23,  0, }, // Cocos
    {   76,  523,  139690,  139706,  139732, 16, 26, 24, }, // Colombia
    {   76,  532,  139756,  139778,  139810, 22, 32, 35, }, // Cook
    {   76,  537,   59613,  139845,  139867, 12, 22, 20, }, // Cuba
    {   76,  548,       0,   59667,       0,  0, 13,  0, }, // Davis
    {   76,  564,       0,  139887,       0,  0, 24,  0, }, // DumontDUrville
    {   76,  601,       0,   59704,       0,  0, 22,  0, }, // East_Timor
    {   76,  612,  139911,  139935,  139969, 24, 34, 32, }, // Easter
    {   76,  619,       0,  140001,       0,  0, 17,  0, }, // Ecuador
    {   76,  627,  140018,  140039,  140070, 21, 31, 29, }, // Europe_Central
    {   76,  642,  140099,  140121,  140153, 22, 32, 30, }, // Europe_Eastern
    {   76,  657,       0,  140183,       0,  0, 34,  0, }, // Europe_Further_Eastern
    {   76,  680,  140217,  140241,  140275, 24, 34, 32, }, // Europe_Western
    {   76,  695,  140307,  140333,  140369, 26, 36, 34, }, // Falkland
    {   76,  704,  140403,  140415,  140437, 12, 22, 20, }, // Fiji
    {   76,  709,       0,  140457,       0,  0, 27,  0, }, // French_Guiana
    {   76,  723,       0,  140484,       0,  0, 63,  0, }, // French_Southern
    {   76,  746,       0,   60334,       0,  0, 17,  0, }, // Galapagos
    {   76,  756,       0,   60351,       0,  0, 15,  0, }, // Gambier
    {   76,  764,  140547,  140562,  140587, 15, 25, 23, }, // Georgia
    {   76,  772,       0,  140610,       0,  0, 25,  0, }, // Gilbert_Islands
    {   76,  788,       0,  140635,       0,  0, 29,  0, }, // GMT
    {   76,  802,  140664,  140683,  140712, 19, 29, 27, }, // Greenland
    {   76,  830,  140739,  140767,  140805, 28, 38, 36, }, // Greenland_Eastern
    {   76,  848,  140841,  140870,  140909, 29, 39, 37, }, // Greenland_Western
    {   76,  871,       0,  140946,       0,  0, 23,  0, }, // Gulf
    {   76,  876,       0,  140969,       0,  0, 14,  0, }, // Guyana
    {   76,  883,   60729,  140983,  141018, 25, 35, 33, }, // Hawaii_Aleutian
    {   76,  899,  141051,  141068,  141095, 17, 27, 25, }, // Hong_Kong
    {   76,  909,  141120,  141133,  141156, 13, 23, 21, }, // Hovd
    {   76,  914,       0,  141177,       0,  0, 15,  0, }, // India
    {   76,  920,       0,  141192,       0,  0, 20,  0, }, // Indian_Ocean
    {   76,  933,       0,  141212,       0,  0, 18,  0, }, // Indochina
    {   76,  943,       0,  141230,       0,  0, 24,  0, }, // Indonesia_Central
    {   76,  961,       0,  141254,       0,  0, 25,  0, }, // Indonesia_Eastern
    {   76,  979,       0,  141279,       0,  0, 26,  0, }, // Indonesia_Western
    {   76,  997,  141305,  141319,  141343, 14, 24, 22, }, // Iran
    {   76, 1008,   61134,  141365,  141389, 14, 24, 22, }, // Irkutsk
    {   76, 1016,   61194,  141411,  141434, 13, 23, 21, }, // Israel
    {   76, 1023,  141455,  141468,  141491, 13, 23, 21, }, // Japan
    {   76, 1029,  141512,  141529,  141573, 17, 44, 44, }, // Kamchatka
    {   76, 1047,       0,  141617,       0,  0, 19,  0, }, // Kazakhstan
    {   76, 1058,       0,  141636,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {   76, 1077,       0,  141664,       0,  0, 29,  0, }, // Kazakhstan_Western
    {   76, 1106,   61478,  141693,  141716, 13, 23, 21, }, // Korea
    {   76, 1112,       0,   61535,       0,  0, 14,  0, }, // Kosrae
    {   76, 1119,  141737,  141756,  141785, 19, 29, 27, }, // Krasnoyarsk
    {   76, 1151,       0,  141812,       0,  0, 20,  0, }, // Kyrgystan
    {   76, 1175,       0,  141832,       0,  0, 22,  0, }, // Line_Islands
    {   76, 1188,   61679,  141854,  141881, 17, 27, 27, }, // Lord_Howe
    {   76, 1198,  141908,  141921,  141944, 13, 23, 21, }, // Macau
    {   76, 1204,  141965,  141980,  142005, 15, 25, 23, }, // Magadan
    {   76, 1219,       0,  142028,       0,  0, 16,  0, }, // Malaysia
    {   76, 1228,       0,  142044,       0,  0, 20,  0, }, // Maldives
    {   76, 1237,       0,   61903,       0,  0, 21,  0, }, // Marquesas
    {   76, 1247,       0,  142064,       0,  0, 26,  0, }, // Marshall_Islands
    {   76, 1264,   61951,  142090,  142115, 15, 25, 23, }, // Mauritius
    {   76, 1274,       0,   62017,       0,  0, 14,  0, }, // Mawson
    {   76, 1281,  142138,  142163,  142198, 25, 35, 33, }, // Mexico_Pacific
    {   76, 1296,  142231,  142249,  142277, 18, 28, 26, }, // Mongolia
    {   76, 1305,  142303,  142317,  142341, 14, 24, 22, }, // Moscow
    {   76, 1312,       0,   62256,       0,  0, 15,  0, }, // Myanmar
    {   76, 1320,       0,   62271,       0,  0, 13,  0, }, // Nauru
    {   76, 1326,       0,   62284,       0,  0, 14,  0, }, // Nepal
    {   76, 1332,  142363,  142385,  142417, 22, 32, 30, }, // New_Caledonia
    {   76, 1346,  142447,  142467,  142497, 20, 30, 28, }, // New_Zealand
    {   76, 1358,  142525,  142542,  142569, 17, 27, 25, }, // Newfoundland
    {   76, 1371,       0,   62544,       0,  0, 12,  0, }, // Niue
    {   76, 1376,  142594,  142616,  142648, 22, 32, 30, }, // Norfolk
    {   76, 1384,   62646,  142678,  142715, 27, 37, 35, }, // Noronha
    {   76, 1406,  142750,  142770,  142800, 20, 30, 28, }, // Novosibirsk
    {   76, 1418,   62857,  142828,  142849, 11, 21, 19, }, // Omsk
    {   76, 1428,  142868,  142885,  142912, 17, 27, 25, }, // Pakistan
    {   76, 1437,       0,   62980,       0,  0, 13,  0, }, // Palau
    {   76, 1443,       0,  142937,       0,  0, 25,  0, }, // Papua_New_Guinea
    {   76, 1460,  142962,  142979,  143006, 17, 27, 25, }, // Paraguay
    {   76, 1469,   63088,  143031,  143054, 13, 23, 21, }, // Peru
    {   76, 1474,  143075,  143096,  143127, 21, 31, 29, }, // Philippines
    {   76, 1486,       0,  143156,       0,  0, 25,  0, }, // Phoenix_Islands
    {   76, 1502,  143181,  143213,  143255, 32, 42, 40, }, // Pierre_Miquelon
    {   76, 1518,       0,   63351,       0,  0, 16,  0, }, // Pitcairn
    {   76, 1527,       0,   63367,       0,  0, 14,  0, }, // Ponape
    {   76, 1534,       0,  143295,       0,  0, 17,  0, }, // Pyongyang
    {   76, 1554,       0,   63467,       0,  0, 14,  0, }, // Reunion
    {   76, 1562,       0,   63482,       0,  0, 15,  0, }, // Rothera
    {   76, 1570,  143312,  143328,  143354, 16, 26, 24, }, // Sakhalin
    {   76, 1579,   63560,  143378,  143402, 14, 24, 22, }, // Samara
    {   76, 1596,   63620,  143424,  143447, 13, 23, 21, }, // Samoa
    {   76, 1602,       0,  143468,       0,  0, 22,  0, }, // Seychelles
    {   76, 1624,       0,  143490,       0,  0, 16,  0, }, // Singapore
    {   76, 1634,       0,  143506,       0,  0, 24,  0, }, // Solomon
    {   76, 1642,       0,  143530,       0,  0, 23,  0, }, // South_Georgia
    {   76, 1656,       0,  143553,       0,  0, 15,  0, }, // Suriname
    {   76, 1676,       0,   63786,       0,  0, 13,  0, }, // Syowa
    {   76, 1682,       0,  143568,       0,  0, 14,  0, }, // Tahiti
    {   76, 1689,  143582,  143596,  143620, 14, 24, 22, }, // Taipei
    {   76, 1696,       0,  143642,       0,  0, 20,  0, }, // Tajikistan
    {   76, 1724,       0,   63893,       0,  0, 15,  0, }, // Tokelau
    {   76, 1732,   63908,  143662,  143685, 13, 23, 21, }, // Tonga
    {   76, 1738,       0,   63965,       0,  0, 13,  0, }, // Truk
    {   76, 1750,  143706,  143727,  143758, 21, 31, 29, }, // Turkmenistan
    {   76, 1763,       0,   64059,       0,  0, 14,  0, }, // Tuvalu
    {   76, 1777,  143787,  143804,  143831, 17, 27, 25, }, // Uruguay
    {   76, 1792,  143856,  143876,  143906, 20, 30, 28, }, // Uzbekistan
    {   76, 1803,  143934,  143948,  143972, 14, 24, 22, }, // Vanuatu
    {   76, 1811,       0,  143994,       0,  0, 17,  0, }, // Venezuela
    {   76, 1821,   64297,  144011,  144040, 19, 29, 27, }, // Vladivostok
    {   76, 1833,  144067,  144084,  144111, 17, 27, 25, }, // Volgograd
    {   76, 1843,       0,   64441,       0,  0, 14,  0, }, // Vostok
    {   76, 1850,       0,  144136,       0,  0, 22,  0, }, // Wake
    {   76, 1855,       0,  144158,       0,  0, 23,  0, }, // Wallis
    {   76, 1862,  144181,  144196,  144221, 15, 25, 23, }, // Yakutsk
    {   76, 1870,  144244,  144265,  144296, 21, 31, 29, }, // Yekaterinburg
    {   76, 1892,       0,  144325,       0,  0, 13,  0, }, // Yukon
    {   80,    5,       0,  144338,       0,  0, 19,  0, }, // Afghanistan Cebuano/Latin/Philippines
    {   80,   17,       0,  144357,       0,  0, 22,  0, }, // Africa_Central
    {   80,   32,       0,  144379,       0,  0, 19,  0, }, // Africa_Eastern
    {   80,   65,       0,  144398,       0,  0, 33,  0, }, // Africa_Southern
    {   80,   81,  144431,  144450,  144482, 19, 32, 32, }, // Africa_Western
    {   80,  107,  144514,  144528,  144555, 14, 27, 27, }, // Alaska
    {   80,  135,  144582,  144596,  144623, 14, 27, 27, }, // Amazon
    {   80,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {   80,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {   80,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {   80,  191,  144650,  144666,  144695, 16, 29, 29, }, // America_Pacific
    {   80,  214,  144724,  144736,  144761, 12, 25, 25, }, // Apia
    {   80,  232,  144786,  144800,  144827, 14, 27, 27, }, // Arabian
    {   80,  240,  144854,  144871,  144901, 17, 30, 30, }, // Argentina
    {   80,  250,  144931,  144956,  144994, 25, 38, 38, }, // Argentina_Western
    {   80,  268,  145032,  145047,  145075, 15, 28, 28, }, // Armenia
    {   80,  286,  145103,  145119,  145148, 16, 29, 29, }, // Atlantic
    {   80,  295,  145177,  145202,  145240, 25, 38, 38, }, // Australia_Central
    {   80,  313,  145278,  145311,  145357, 33, 46, 46, }, // Australia_CentralWestern
    {   80,  338,  145403,  145428,  145466, 25, 38, 38, }, // Australia_Eastern
    {   80,  356,  145504,  145529,  145567, 25, 38, 38, }, // Australia_Western
    {   80,  374,  145605,  145623,  145654, 18, 31, 31, }, // Azerbaijan
    {   80,  385,  145685,  145699,  145726, 14, 27, 27, }, // Azores
    {   80,  397,  145753,  145771,  145802, 18, 31, 31, }, // Bangladesh
    {   80,  415,       0,  145833,       0,  0, 14,  0, }, // Bhutan
    {   80,  422,       0,  145847,       0,  0, 15,  0, }, // Bolivia
    {   80,  437,  145862,  145878,  145907, 16, 29, 29, }, // Brasilia
    {   80,  454,       0,  145936,       0,  0, 25,  0, }, // Brunei
    {   80,  461,  145961,  145979,  146010, 18, 31, 31, }, // Cape_Verde
    {   80,  478,       0,  146041,       0,  0, 29,  0, }, // Chamorro
    {   80,  487,  146070,  146085,  146113, 15, 28, 28, }, // Chatham
    {   80,  495,  146141,  146154,  146180, 13, 26, 26, }, // Chile
    {   80,  501,  146206,  146219,  146245, 13, 26, 26, }, // China
    {   80,  507,       0,  146271,       0,  0, 24,  0, }, // Christmas
    {   80,  517,       0,  146295,       0,  0, 21,  0, }, // Cocos
    {   80,  523,  146316,  146332,  146361, 16, 29, 29, }, // Colombia
    {   80,  532,  146390,  146410,  146443, 20, 33, 43, }, // Cook
    {   80,  537,  146486,  146498,  146523, 12, 25, 25, }, // Cuba
    {   80,  548,       0,  146548,       0,  0, 13,  0, }, // Davis
    {   80,  564,       0,  146561,       0,  0, 24,  0, }, // DumontDUrville
    {   80,  601,       0,  146585,       0,  0, 18,  0, }, // East_Timor
    {   80,  612,  146603,  146624,  146658, 21, 34, 34, }, // Easter
    {   80,  619,       0,  146692,       0,  0, 15,  0, }, // Ecuador
    {   80,  627,  146707,  146729,  146764, 22, 35, 35, }, // Europe_Central
    {   80,  642,  146799,  146821,  146856, 22, 35, 35, }, // Europe_Eastern
    {   80,  657,       0,  146891,       0,  0, 30,  0, }, // Europe_Further_Eastern
    {   80,  680,  146921,  146943,  146978, 22, 35, 35, }, // Europe_Western
    {   80,  695,  147013,  147037,  147074, 24, 37, 37, }, // Falkland
    {   80,  704,  147111,  147123,  147148, 12, 25, 25, }, // Fiji
    {   80,  709,       0,  147173,       0,  0, 21,  0, }, // French_Guiana
    {   80,  723,       0,  147194,       0,  0, 36,  0, }, // French_Southern
    {   80,  746,       0,  147230,       0,  0, 17,  0, }, // Galapagos
    {   80,  756,       0,  147247,       0,  0, 15,  0, }, // Gambier
    {   80,  764,  147262,  147277,  147305, 15, 28, 28, }, // Georgia
    {   80,  772,       0,  147333,       0,  0, 23,  0, }, // Gilbert_Islands
    {   80,  788,       0,  147356,       0,  0, 22,  0, }, // GMT
    {   80,  830,  147378,  147400,  147435, 22, 35, 35, }, // Greenland_Eastern
    {   80,  848,  147470,  147492,  147527, 22, 35, 35, }, // Greenland_Western
    {   80,  871,       0,  147562,       0,  0, 25,  0, }, // Gulf
    {   80,  876,       0,  147587,       0,  0, 14,  0, }, // Guyana
    {   80,  883,  147601,  147624,  147660, 23, 36, 36, }, // Hawaii_Aleutian
    {   80,  899,  147696,  147713,  147743, 17, 30, 30, }, // Hong_Kong
    {   80,  909,  147773,  147785,  147810, 12, 25, 25, }, // Hovd
    {   80,  914,       0,  147835,       0,  0, 26,  0, }, // India
    {   80,  920,       0,  147861,       0,  0, 20,  0, }, // Indian_Ocean
    {   80,  933,       0,  147881,       0,  0, 17,  0, }, // Indochina
    {   80,  943,       0,  147898,       0,  0, 25,  0, }, // Indonesia_Central
    {   80,  961,       0,  147923,       0,  0, 25,  0, }, // Indonesia_Eastern
    {   80,  979,       0,  147948,       0,  0, 25,  0, }, // Indonesia_Western
    {   80,  997,  147973,  147985,  148010, 12, 25, 25, }, // Iran
    {   80, 1008,  148035,  148050,  148078, 15, 28, 28, }, // Irkutsk
    {   80, 1016,  148106,  148120,  148147, 14, 27, 27, }, // Israel
    {   80, 1023,  148174,  148187,  148213, 13, 26, 26, }, // Japan
    {   80, 1047,       0,  148239,       0,  0, 18,  0, }, // Kazakhstan
    {   80, 1058,       0,  148257,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {   80, 1077,       0,  148280,       0,  0, 23,  0, }, // Kazakhstan_Western
    {   80, 1106,  148303,  148316,  148342, 13, 26, 26, }, // Korea
    {   80, 1112,       0,  148368,       0,  0, 14,  0, }, // Kosrae
    {   80, 1119,  148382,  148401,  148433, 19, 32, 32, }, // Krasnoyarsk
    {   80, 1151,       0,  148465,       0,  0, 18,  0, }, // Kyrgystan
    {   80, 1175,       0,  148483,       0,  0, 20,  0, }, // Line_Islands
    {   80, 1188,  148503,  148520,  148550, 17, 30, 30, }, // Lord_Howe
    {   80, 1204,  148580,  148595,  148623, 15, 28, 28, }, // Magadan
    {   80, 1219,       0,  148651,       0,  0, 16,  0, }, // Malaysia
    {   80, 1228,       0,  148667,       0,  0, 16,  0, }, // Maldives
    {   80, 1237,       0,  148683,       0,  0, 17,  0, }, // Marquesas
    {   80, 1247,       0,  148700,       0,  0, 24,  0, }, // Marshall_Islands
    {   80, 1264,  148724,  148741,  148771, 17, 30, 30, }, // Mauritius
    {   80, 1274,       0,  148801,       0,  0, 14,  0, }, // Mawson
    {   80, 1281,  148815,  148838,  148874, 23, 36, 36, }, // Mexico_Pacific
    {   80, 1296,  148910,  148929,  148961, 19, 32, 32, }, // Mongolia
    {   80, 1305,  148993,  149007,  149034, 14, 27, 27, }, // Moscow
    {   80, 1312,       0,  149061,       0,  0, 15,  0, }, // Myanmar
    {   80, 1320,       0,  149076,       0,  0, 13,  0, }, // Nauru
    {   80, 1326,       0,  149089,       0,  0, 13,  0, }, // Nepal
    {   80, 1332,  149102,  149123,  149157, 21, 34, 34, }, // New_Caledonia
    {   80, 1346,  149191,  149210,  149242, 19, 32, 32, }, // New_Zealand
    {   80, 1358,  149274,  149294,  149327, 20, 33, 33, }, // Newfoundland
    {   80, 1371,       0,  149360,       0,  0, 12,  0, }, // Niue
    {   80, 1376,  149372,  149394,  149429, 22, 35, 35, }, // Norfolk
    {   80, 1384,  149464,  149491,  149531, 27, 40, 40, }, // Noronha
    {   80, 1406,  149571,  149590,  149622, 19, 32, 32, }, // Novosibirsk
    {   80, 1418,  149654,  149666,  149691, 12, 25, 25, }, // Omsk
    {   80, 1428,  149716,  149732,  149761, 16, 29, 29, }, // Pakistan
    {   80, 1437,       0,  149790,       0,  0, 13,  0, }, // Palau
    {   80, 1443,       0,  149803,       0,  0, 24,  0, }, // Papua_New_Guinea
    {   80, 1460,  149827,  149843,  149872, 16, 29, 29, }, // Paraguay
    {   80, 1469,  149901,  149913,  149938, 12, 25, 25, }, // Peru
    {   80, 1474,  149963,  149980,  150010, 17, 30, 30, }, // Philippines
    {   80, 1486,       0,  150040,       0,  0, 23,  0, }, // Phoenix_Islands
    {   80, 1502,  150063,  150092,  150134, 29, 42, 42, }, // Pierre_Miquelon
    {   80, 1518,       0,  150176,       0,  0, 16,  0, }, // Pitcairn
    {   80, 1527,       0,  150192,       0,  0, 14,  0, }, // Ponape
    {   80, 1534,       0,  150206,       0,  0, 17,  0, }, // Pyongyang
    {   80, 1554,       0,  150223,       0,  0, 15,  0, }, // Reunion
    {   80, 1562,       0,  150238,       0,  0, 15,  0, }, // Rothera
    {   80, 1570,  150253,  150269,  150298, 16, 29, 29, }, // Sakhalin
    {   80, 1596,  150327,  150340,  150366, 13, 26, 26, }, // Samoa
    {   80, 1602,       0,  150392,       0,  0, 18,  0, }, // Seychelles
    {   80, 1624,       0,  150410,       0,  0, 30,  0, }, // Singapore
    {   80, 1634,       0,  150440,       0,  0, 23,  0, }, // Solomon
    {   80, 1642,       0,  150463,       0,  0, 21,  0, }, // South_Georgia
    {   80, 1656,       0,  150484,       0,  0, 16,  0, }, // Suriname
    {   80, 1676,       0,  150500,       0,  0, 13,  0, }, // Syowa
    {   80, 1682,       0,  150513,       0,  0, 14,  0, }, // Tahiti
    {   80, 1689,  150527,  150541,  150568, 14, 27, 27, }, // Taipei
    {   80, 1696,       0,  150595,       0,  0, 18,  0, }, // Tajikistan
    {   80, 1724,       0,  150613,       0,  0, 15,  0, }, // Tokelau
    {   80, 1732,  150628,  150641,  150667, 13, 26, 26, }, // Tonga
    {   80, 1738,       0,  150693,       0,  0, 13,  0, }, // Truk
    {   80, 1750,  150706,  150726,  150759, 20, 33, 33, }, // Turkmenistan
    {   80, 1763,       0,  150792,       0,  0, 14,  0, }, // Tuvalu
    {   80, 1777,  150806,  150821,  150849, 15, 28, 28, }, // Uruguay
    {   80, 1792,  150877,  150895,  150926, 18, 31, 31, }, // Uzbekistan
    {   80, 1803,  150957,  150972,  151000, 15, 28, 28, }, // Vanuatu
    {   80, 1811,       0,  151028,       0,  0, 17,  0, }, // Venezuela
    {   80, 1821,  151045,  151064,  151096, 19, 32, 32, }, // Vladivostok
    {   80, 1833,  151128,  151145,  151175, 17, 30, 30, }, // Volgograd
    {   80, 1843,       0,  151205,       0,  0, 14,  0, }, // Vostok
    {   80, 1850,       0,  151219,       0,  0, 19,  0, }, // Wake
    {   80, 1855,       0,  151238,       0,  0, 23,  0, }, // Wallis
    {   80, 1862,  151261,  151276,  151304, 15, 28, 28, }, // Yakutsk
    {   80, 1870,  151332,  151353,  151387, 21, 34, 34, }, // Yekaterinburg
    {   80, 1892,       0,  151421,       0,  0, 13,  0, }, // Yukon
    {   84,    0,  151434,  151459,  151495, 25, 36, 48, }, // Acre Chakma/Chakma/Bangladesh
    {   84,    5,       0,  151543,       0,  0, 35,  0, }, // Afghanistan
    {   84,   17,       0,  151578,       0,  0, 40,  0, }, // Africa_Central
    {   84,   32,       0,  151618,       0,  0, 38,  0, }, // Africa_Eastern
    {   84,   65,       0,  151656,       0,  0, 49,  0, }, // Africa_Southern
    {   84,   81,  151705,  151747,  151800, 42, 53, 65, }, // Africa_Western
    {   84,  107,  151865,  151888,  151922, 23, 34, 38, }, // Alaska
    {   84,  128,  151960,  151985,  152021, 25, 36, 44, }, // Almaty
    {   84,  135,  152065,  152094,  152134, 29, 40, 48, }, // Amazon
    {   84,  142,  152182,  152214,  152259, 32, 45, 49, }, // America_Central
    {   84,  158,  152308,  152342,  152391, 34, 49, 53, }, // America_Eastern
    {   84,  174,  152444,  152482,  152535, 38, 53, 51, }, // America_Mountain
    {   84,  191,  152586,  152651,  152727, 65, 76, 78, }, // America_Pacific
    {   84,  207,  152805,  152834,  152874, 29, 40, 52, }, // Anadyr
    {   84,  214,  152926,  152949,  152983, 23, 34, 36, }, // Apia
    {   84,  219,  153019,  153044,  153080, 25, 36, 48, }, // Aqtau
    {   84,  225,  153128,  153155,  153193, 27, 38, 50, }, // Aqtobe
    {   84,  232,  153243,  153266,  153300, 23, 34, 42, }, // Arabian
    {   84,  240,  153342,  153375,  153419, 33, 44, 56, }, // Argentina
    {   84,  250,  153475,  153525,  153590, 50, 65, 73, }, // Argentina_Western
    {   84,  268,  153663,  153692,  153732, 29, 40, 52, }, // Armenia
    {   84,  286,  153784,  153821,  153869, 37, 48, 56, }, // Atlantic
    {   84,  295,  153925,  153988,  154062, 63, 74, 82, }, // Australia_Central
    {   84,  338,  154144,  154194,  154255, 50, 61, 69, }, // Australia_Eastern
    {   84,  356,  154324,  154378,  154443, 54, 65, 73, }, // Australia_Western
    {   84,  374,  154516,  154547,  154589, 31, 42, 54, }, // Azerbaijan
    {   84,  385,  154643,  154672,  154712, 29, 40, 52, }, // Azores
    {   84,  397,  154764,  154791,  154829, 27, 38, 50, }, // Bangladesh
    {   84,  415,       0,  154879,       0,  0, 23,  0, }, // Bhutan
    {   84,  422,       0,  154902,       0,  0, 27,  0, }, // Bolivia
    {   84,  437,  154929,  154958,  154998, 29, 40, 52, }, // Brasilia
    {   84,  454,       0,  155050,       0,  0, 46,  0, }, // Brunei
    {   84,  461,  155096,  155128,  155171, 32, 43, 55, }, // Cape_Verde
    {   84,  478,       0,  155226,       0,  0, 34,  0, }, // Chamorro
    {   84,  487,  155260,  155285,  155321, 25, 36, 44, }, // Chatham
    {   84,  495,  155365,  155386,  155418, 21, 32, 44, }, // Chile
    {   84,  501,  155462,  155483,  155515, 21, 32, 40, }, // China
    {   84,  507,       0,  155555,       0,  0, 50,  0, }, // Christmas
    {   84,  517,       0,  155605,       0,  0, 56,  0, }, // Cocos
    {   84,  523,  155661,  155692,  155734, 31, 42, 54, }, // Colombia
    {   84,  532,  155788,  155840,  155903, 52, 63, 92, }, // Cook
    {   84,  537,  155995,  156022,  156056, 27, 34, 42, }, // Cuba
    {   84,  548,       0,  156098,       0,  0, 25,  0, }, // Davis
    {   84,  564,       0,  156123,       0,  0, 55,  0, }, // DumontDUrville
    {   84,  601,       0,  156178,       0,  0, 38,  0, }, // East_Timor
    {   84,  612,  156216,  156262,  156319, 46, 57, 69, }, // Easter
    {   84,  619,       0,  156388,       0,  0, 33,  0, }, // Ecuador
    {   84,  627,  156421,  156469,  156528, 48, 59, 71, }, // Europe_Central
    {   84,  642,  156599,  156645,  156702, 46, 57, 69, }, // Europe_Eastern
    {   84,  657,       0,  156771,       0,  0, 71,  0, }, // Europe_Further_Eastern
    {   84,  680,  156842,  156890,  156951, 48, 61, 73, }, // Europe_Western
    {   84,  695,  157024,  157090,  157167, 66, 77, 89, }, // Falkland
    {   84,  704,  157256,  157277,  157309, 21, 32, 44, }, // Fiji
    {   84,  709,       0,  157353,       0,  0, 30,  0, }, // French_Guiana
    {   84,  723,       0,  157383,       0,  0, 66,  0, }, // French_Southern
    {   84,  746,       0,  157449,       0,  0, 27,  0, }, // Galapagos
    {   84,  756,       0,  157476,       0,  0, 33,  0, }, // Gambier
    {   84,  764,  157509,  157536,  157574, 27, 38, 50, }, // Georgia
    {   84,  772,       0,  157624,       0,  0, 54,  0, }, // Gilbert_Islands
    {   84,  788,       0,  157678,       0,  0, 34,  0, }, // GMT
    {   84,  830,  157712,  157764,  157827, 52, 63, 75, }, // Greenland_Eastern
    {   84,  848,  157902,  157958,  158025, 56, 67, 79, }, // Greenland_Western
    {   84,  866,       0,  158104,       0,  0, 30,  0, }, // Guam
    {   84,  871,       0,  158134,       0,  0, 46,  0, }, // Gulf
    {   84,  876,       0,  158180,       0,  0, 21,  0, }, // Guyana
    {   84,  883,  158201,  158249,  158300, 48, 51, 59, }, // Hawaii_Aleutian
    {   84,  899,  158359,  158385,  158422, 26, 37, 49, }, // Hong_Kong
    {   84,  909,  158471,  158496,  158532, 25, 36, 48, }, // Hovd
    {   84,  914,       0,  158580,       0,  0, 40,  0, }, // India
    {   84,  920,       0,  158620,       0,  0, 50,  0, }, // Indian_Ocean
    {   84,  933,       0,  158670,       0,  0, 33,  0, }, // Indochina
    {   84,  943,       0,  158703,       0,  0, 55,  0, }, // Indonesia_Central
    {   84,  961,       0,  158758,       0,  0, 48,  0, }, // Indonesia_Eastern
    {   84,  979,       0,  158806,       0,  0, 52,  0, }, // Indonesia_Western
    {   84,  997,  158858,  158881,  158915, 23, 34, 42, }, // Iran
    {   84, 1008,  158957,  158994,  159042, 37, 48, 60, }, // Irkutsk
    {   84, 1016,  159102,  159131,  159173, 29, 42, 50, }, // Israel
    {   84, 1023,  159223,  159244,  159276, 21, 32, 40, }, // Japan
    {   84, 1029,  159316,  159396,  159483, 80, 87,103, }, // Kamchatka
    {   84, 1058,       0,  159586,       0,  0, 55,  0, }, // Kazakhstan_Eastern
    {   84, 1077,       0,  159641,       0,  0, 59,  0, }, // Kazakhstan_Western
    {   84, 1106,  159700,  159727,  159765, 27, 38, 46, }, // Korea
    {   84, 1112,       0,  159811,       0,  0, 29,  0, }, // Kosrae
    {   84, 1119,  159840,  159881,  159933, 41, 52, 64, }, // Krasnoyarsk
    {   84, 1151,       0,  159997,       0,  0, 35,  0, }, // Kyrgystan
    {   84, 1161,       0,  160032,       0,  0, 21,  0, }, // Lanka
    {   84, 1175,       0,  160053,       0,  0, 52,  0, }, // Line_Islands
    {   84, 1188,  160105,  160137,  160180, 32, 43, 51, }, // Lord_Howe
    {   84, 1198,  160231,  160252,  160280, 21, 28, 44, }, // Macau
    {   84, 1204,  160324,  160351,  160389, 27, 38, 50, }, // Magadan
    {   84, 1219,       0,  160439,       0,  0, 29,  0, }, // Malaysia
    {   84, 1228,       0,  160468,       0,  0, 27,  0, }, // Maldives
    {   84, 1237,       0,  160495,       0,  0, 29,  0, }, // Marquesas
    {   84, 1247,       0,  160524,       0,  0, 56,  0, }, // Marshall_Islands
    {   84, 1264,  160580,  160607,  160645, 27, 38, 50, }, // Mauritius
    {   84, 1274,       0,  160695,       0,  0, 29,  0, }, // Mawson
    {   84, 1281,  160724,  160795,  160877, 71, 82, 90, }, // Mexico_Pacific
    {   84, 1296,  160967,  161001,  161046, 34, 45, 57, }, // Mongolia
    {   84, 1305,  161103,  161128,  161164, 25, 36, 48, }, // Moscow
    {   84, 1312,       0,  161212,       0,  0, 27,  0, }, // Myanmar
    {   84, 1320,       0,  161239,       0,  0, 23,  0, }, // Nauru
    {   84, 1326,       0,  161262,       0,  0, 23,  0, }, // Nepal
    {   84, 1332,  161285,  161327,  161380, 42, 53, 65, }, // New_Caledonia
    {   84, 1346,  161445,  161484,  161534, 39, 50, 58, }, // New_Zealand
    {   84, 1358,  161592,  161637,  161693, 45, 56, 64, }, // Newfoundland
    {   84, 1371,       0,  161757,       0,  0, 25,  0, }, // Niue
    {   84, 1376,  161782,  161842,  161913, 60, 71, 79, }, // Norfolk
    {   84, 1384,  161992,  162040,  162099, 48, 59, 71, }, // Noronha
    {   84, 1392,       0,  162170,       0,  0, 73,  0, }, // North_Mariana
    {   84, 1406,  162243,  162284,  162336, 41, 52, 64, }, // Novosibirsk
    {   84, 1418,  162400,  162429,  162469, 29, 40, 52, }, // Omsk
    {   84, 1428,  162521,  162550,  162590, 29, 40, 52, }, // Pakistan
    {   84, 1437,       0,  162642,       0,  0, 21,  0, }, // Palau
    {   84, 1443,       0,  162663,       0,  0, 39,  0, }, // Papua_New_Guinea
    {   84, 1460,  162702,  162731,  162771, 29, 40, 52, }, // Paraguay
    {   84, 1469,  162823,  162844,  162876, 21, 32, 44, }, // Peru
    {   84, 1474,  162920,  162949,  162989, 29, 40, 52, }, // Philippines
    {   84, 1486,       0,  163041,       0,  0, 60,  0, }, // Phoenix_Islands
    {   84, 1502,  163101,  163161,  163232, 60, 71, 79, }, // Pierre_Miquelon
    {   84, 1518,       0,  163311,       0,  0, 35,  0, }, // Pitcairn
    {   84, 1527,       0,  163346,       0,  0, 23,  0, }, // Ponape
    {   84, 1534,       0,  163369,       0,  0, 32,  0, }, // Pyongyang
    {   84, 1544,  163401,  163432,  163470, 31, 38, 54, }, // Qyzylorda
    {   84, 1554,       0,  163524,       0,  0, 35,  0, }, // Reunion
    {   84, 1562,       0,  163559,       0,  0, 23,  0, }, // Rothera
    {   84, 1570,  163582,  163607,  163643, 25, 36, 48, }, // Sakhalin
    {   84, 1579,  163691,  163710,  163736, 19, 26, 42, }, // Samara
    {   84, 1596,  163778,  163799,  163831, 21, 32, 44, }, // Samoa
    {   84, 1602,       0,  163875,       0,  0, 29,  0, }, // Seychelles
    {   84, 1624,       0,  163904,       0,  0, 38,  0, }, // Singapore
    {   84, 1634,       0,  163942,       0,  0, 60,  0, }, // Solomon
    {   84, 1642,       0,  164002,       0,  0, 40,  0, }, // South_Georgia
    {   84, 1656,       0,  164042,       0,  0, 27,  0, }, // Suriname
    {   84, 1676,       0,  164069,       0,  0, 21,  0, }, // Syowa
    {   84, 1682,       0,  164090,       0,  0, 23,  0, }, // Tahiti
    {   84, 1689,  164113,  164138,  164174, 25, 36, 44, }, // Taipei
    {   84, 1696,       0,  164218,       0,  0, 29,  0, }, // Tajikistan
    {   84, 1724,       0,  164247,       0,  0, 27,  0, }, // Tokelau
    {   84, 1732,  164274,  164297,  164331, 23, 34, 46, }, // Tonga
    {   84, 1738,       0,  164377,       0,  0, 21,  0, }, // Truk
    {   84, 1750,  164398,  164441,  164495, 43, 54, 66, }, // Turkmenistan
    {   84, 1763,       0,  164561,       0,  0, 23,  0, }, // Tuvalu
    {   84, 1777,  164584,  164613,  164653, 29, 40, 52, }, // Uruguay
    {   84, 1792,  164705,  164740,  164786, 35, 46, 58, }, // Uzbekistan
    {   84, 1803,  164844,  164869,  164905, 25, 36, 48, }, // Vanuatu
    {   84, 1811,       0,  164953,       0,  0, 31,  0, }, // Venezuela
    {   84, 1821,  164984,  165023,  165073, 39, 50, 62, }, // Vladivostok
    {   84, 1833,  165135,  165170,  165218, 35, 48, 58, }, // Volgograd
    {   84, 1843,       0,  165276,       0,  0, 27,  0, }, // Vostok
    {   84, 1850,       0,  165303,       0,  0, 44,  0, }, // Wake
    {   84, 1855,       0,  165347,       0,  0, 41,  0, }, // Wallis
    {   84, 1862,  165388,  165419,  165461, 31, 42, 54, }, // Yakutsk
    {   84, 1870,  165515,  165562,  165620, 47, 58, 70, }, // Yekaterinburg
    {   86,    5,       0,  165690,       0,  0,  6,  0, }, // Afghanistan Chechen/Cyrillic/Russia
    {   86,   17,       0,  165696,       0,  0, 14,  0, }, // Africa_Central
    {   86,   32,       0,  165710,       0,  0, 16,  0, }, // Africa_Eastern
    {   86,   65,       0,  165726,       0,  0, 13,  0, }, // Africa_Southern
    {   86,   81,  165739,  165755,  165787, 16, 32, 30, }, // Africa_Western
    {   86,  107,  119445,  165817,  165839,  6, 22, 20, }, // Alaska
    {   86,  135,    6643,  165859,  165883,  8, 24, 22, }, // Amazon
    {   86,  142,  165905,  165920,  165951, 15, 31, 29, }, // America_Central
    {   86,  158,  165980,  165997,  166030, 17, 33, 31, }, // America_Eastern
    {   86,  174,  166061,  166079,  166108, 18, 29, 27, }, // America_Mountain
    {   86,  191,  166135,  166152,  166180, 17, 28, 26, }, // America_Pacific
    {   86,  214,  166206,  166221,  166247, 15, 26, 24, }, // Apia
    {   86,  232,  166271,  166290,  166327, 19, 37, 36, }, // Arabian
    {   86,  240,    7371,  166363,  166388,  9, 25, 23, }, // Argentina
    {   86,  250,  166411,  166430,  166465, 19, 35, 33, }, // Argentina_Western
    {   86,  268,  166498,  166508,  166534, 10, 26, 24, }, // Armenia
    {   86,  286,  166558,  166572,  166597, 14, 25, 23, }, // Atlantic
    {   86,  295,  166620,  166636,  166668, 16, 32, 30, }, // Australia_Central
    {   86,  313,  166698,  166729,  166771, 31, 42, 40, }, // Australia_CentralWestern
    {   86,  338,  166811,  166829,  166863, 18, 34, 32, }, // Australia_Eastern
    {   86,  356,  166895,  166913,  166947, 18, 34, 32, }, // Australia_Western
    {   86,  374,   86281,  166979,  167006, 11, 27, 25, }, // Azerbaijan
    {   86,  385,  167031,  167047,  167079, 16, 32, 30, }, // Azores
    {   86,  397,    8256,  167109,  167134,  9, 25, 23, }, // Bangladesh
    {   86,  415,       0,    8317,       0,  0,  5,  0, }, // Bhutan
    {   86,  422,       0,    8322,       0,  0,  6,  0, }, // Bolivia
    {   86,  437,    8329,  167157,  167180,  7, 23, 21, }, // Brasilia
    {   86,  454,       0,  167201,       0,  0, 17,  0, }, // Brunei
    {   86,  461,    8404,  167218,  167244, 10, 26, 24, }, // Cape_Verde
    {   86,  478,       0,    8473,       0,  0,  7,  0, }, // Chamorro
    {   86,  487,    8480,  167268,  167289,  5, 21, 19, }, // Chatham
    {   86,  495,    8529,  167308,  167328,  4, 20, 18, }, // Chile
    {   86,  501,  167346,  167352,  167377,  6, 25, 23, }, // China
    {   86,  507,       0,  167400,       0,  0, 31,  0, }, // Christmas
    {   86,  517,       0,  167431,       0,  0, 16,  0, }, // Cocos
    {   86,  523,    8653,  167447,  167470,  7, 23, 21, }, // Colombia
    {   86,  532,  167491,  167504,  167533, 13, 29, 27, }, // Cook
    {   86,  537,    8806,  167560,  167580,  4, 20, 18, }, // Cuba
    {   86,  548,       0,  121938,       0,  0,  6,  0, }, // Davis
    {   86,  564,       0,  167598,       0,  0, 14,  0, }, // DumontDUrville
    {   86,  601,       0,  167612,       0,  0, 15,  0, }, // East_Timor
    {   86,  612,  167627,  167640,  167669, 13, 29, 27, }, // Easter
    {   86,  619,       0,   87146,       0,  0,  7,  0, }, // Ecuador
    {   86,  627,  167696,  167710,  167740, 14, 30, 28, }, // Europe_Central
    {   86,  642,  167768,  167784,  167816, 16, 32, 30, }, // Europe_Eastern
    {   86,  657,       0,  167846,       0,  0, 32,  0, }, // Europe_Further_Eastern
    {   86,  680,  167878,  167894,  167926, 16, 32, 30, }, // Europe_Western
    {   86,  695,  167956,  167976,  168012, 20, 36, 34, }, // Falkland
    {   86,  704,  122493,  168046,  168067,  5, 21, 19, }, // Fiji
    {   86,  709,       0,  168086,       0,  0, 17,  0, }, // French_Guiana
    {   86,  723,       0,  168103,       0,  0, 38,  0, }, // French_Southern
    {   86,  746,       0,  168141,       0,  0, 21,  0, }, // Galapagos
    {   86,  756,       0,  168162,       0,  0,  6,  0, }, // Gambier
    {   86,  764,  168168,  168178,  168204, 10, 26, 24, }, // Georgia
    {   86,  772,       0,  168228,       0,  0, 17,  0, }, // Gilbert_Islands
    {   86,  788,       0,  168245,       0,  0, 21,  0, }, // GMT
    {   86,  830,  168266,  168285,  168320, 19, 35, 33, }, // Greenland_Eastern
    {   86,  848,  168353,  168372,  168407, 19, 35, 33, }, // Greenland_Western
    {   86,  871,       0,  168440,       0,  0, 14,  0, }, // Gulf
    {   86,  876,       0,  168454,       0,  0,  6,  0, }, // Guyana
    {   86,  883,  168460,  168479,  168509, 19, 30, 28, }, // Hawaii_Aleutian
    {   86,  899,  168537,  168544,  168567,  7, 23, 21, }, // Hong_Kong
    {   86,  909,    9958,  168588,  168608,  4, 20, 18, }, // Hovd
    {   86,  914,       0,  168626,       0,  0,  6,  0, }, // India
    {   86,  920,       0,  168632,       0,  0, 11,  0, }, // Indian_Ocean
    {   86,  933,       0,  123211,       0,  0,  9,  0, }, // Indochina
    {   86,  943,       0,  168643,       0,  0, 16,  0, }, // Indonesia_Central
    {   86,  961,       0,  168659,       0,  0, 18,  0, }, // Indonesia_Eastern
    {   86,  979,       0,  168677,       0,  0, 18,  0, }, // Indonesia_Western
    {   86,  997,  168695,  168706,  168733, 11, 27, 25, }, // Iran
    {   86, 1008,   10143,  168758,  168781,  7, 23, 21, }, // Irkutsk
    {   86, 1016,   10198,  168802,  168825,  7, 23, 21, }, // Israel
    {   86, 1023,   88446,  168846,  168866,  4, 20, 18, }, // Japan
    {   86, 1058,       0,  168884,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {   86, 1077,       0,  168903,       0,  0, 19,  0, }, // Kazakhstan_Western
    {   86, 1106,  123748,  168922,  168943,  5, 21, 19, }, // Korea
    {   86, 1112,       0,  168962,       0,  0,  6,  0, }, // Kosrae
    {   86, 1119,  123820,  168968,  168994, 10, 26, 24, }, // Krasnoyarsk
    {   86, 1151,       0,   10587,       0,  0,  7,  0, }, // Kyrgystan
    {   86, 1175,       0,  169018,       0,  0, 12,  0, }, // Line_Islands
    {   86, 1188,   88712,  169030,  169054,  8, 24, 22, }, // Lord_Howe
    {   86, 1204,   10724,  169076,  169099,  7, 23, 21, }, // Magadan
    {   86, 1219,       0,  124070,       0,  0,  7,  0, }, // Malaysia
    {   86, 1228,       0,  169120,       0,  0,  9,  0, }, // Maldives
    {   86, 1237,       0,  169129,       0,  0, 16,  0, }, // Marquesas
    {   86, 1247,       0,  169145,       0,  0, 17,  0, }, // Marshall_Islands
    {   86, 1264,   10838,  169162,  169185,  7, 23, 21, }, // Mauritius
    {   86, 1274,       0,   10893,       0,  0,  6,  0, }, // Mawson
    {   86, 1281,  169206,  169232,  169269, 26, 37, 35, }, // Mexico_Pacific
    {   86, 1296,   11009,  169304,  169330, 10, 26, 24, }, // Mongolia
    {   86, 1305,   11073,  169354,  169376,  6, 22, 20, }, // Moscow
    {   86, 1312,       0,  169396,       0,  0,  6,  0, }, // Myanmar
    {   86, 1320,       0,   11131,       0,  0,  5,  0, }, // Nauru
    {   86, 1326,       0,   11136,       0,  0,  5,  0, }, // Nepal
    {   86, 1332,  169402,  169416,  169446, 14, 30, 28, }, // New_Caledonia
    {   86, 1346,  169474,  169487,  169516, 13, 29, 27, }, // New_Zealand
    {   86, 1358,  169543,  169555,  169583, 12, 28, 26, }, // Newfoundland
    {   86, 1371,       0,  169609,       0,  0,  4,  0, }, // Niue
    {   86, 1376,   11358,  169613,  169636,  7, 23, 21, }, // Norfolk
    {   86, 1384,  169657,  169676,  169711, 19, 35, 33, }, // Noronha
    {   86, 1406,   11531,  169744,  169771, 11, 27, 25, }, // Novosibirsk
    {   86, 1418,   11598,  169796,  169816,  4, 20, 18, }, // Omsk
    {   86, 1428,   11644,  169834,  169858,  8, 24, 22, }, // Pakistan
    {   86, 1437,       0,   11702,       0,  0,  5,  0, }, // Palau
    {   86, 1443,       0,  169880,       0,  0, 20,  0, }, // Papua_New_Guinea
    {   86, 1460,  125138,  169900,  169924,  8, 24, 22, }, // Paraguay
    {   86, 1469,   11784,  169946,  169966,  4, 20, 18, }, // Peru
    {   86, 1474,  169984,  169994,  170020, 10, 26, 24, }, // Philippines
    {   86, 1486,       0,  170044,       0,  0, 14,  0, }, // Phoenix_Islands
    {   86, 1502,  170058,  170079,  170116, 21, 37, 35, }, // Pierre_Miquelon
    {   86, 1518,       0,  170151,       0,  0,  7,  0, }, // Pitcairn
    {   86, 1527,       0,  170158,       0,  0, 14,  0, }, // Ponape
    {   86, 1534,       0,   90127,       0,  0,  7,  0, }, // Pyongyang
    {   86, 1554,       0,  170172,       0,  0,  7,  0, }, // Reunion
    {   86, 1562,       0,   12082,       0,  0,  6,  0, }, // Rothera
    {   86, 1570,   12088,  170179,  170202,  7, 23, 21, }, // Sakhalin
    {   86, 1596,   12205,  170223,  170244,  5, 21, 19, }, // Samoa
    {   86, 1602,       0,  170263,       0,  0, 18,  0, }, // Seychelles
    {   86, 1624,       0,   12279,       0,  0,  8,  0, }, // Singapore
    {   86, 1634,       0,  170281,       0,  0, 17,  0, }, // Solomon
    {   86, 1642,       0,  170298,       0,  0, 13,  0, }, // South_Georgia
    {   86, 1656,       0,   12329,       0,  0,  7,  0, }, // Suriname
    {   86, 1676,       0,   90382,       0,  0,  4,  0, }, // Syowa
    {   86, 1682,       0,  170311,       0,  0, 13,  0, }, // Tahiti
    {   86, 1689,  170324,  170331,  170354,  7, 23, 21, }, // Taipei
    {   86, 1696,       0,  125818,       0,  0, 11,  0, }, // Tajikistan
    {   86, 1724,       0,   12408,       0,  0,  7,  0, }, // Tokelau
    {   86, 1732,   12415,  170375,  170396,  5, 21, 19, }, // Tonga
    {   86, 1738,       0,  170415,       0,  0,  4,  0, }, // Truk
    {   86, 1750,   12468,  170419,  170443,  8, 24, 22, }, // Turkmenistan
    {   86, 1763,       0,   12538,       0,  0,  6,  0, }, // Tuvalu
    {   86, 1777,   90600,  170465,  170488,  7, 23, 21, }, // Uruguay
    {   86, 1792,  113243,  170509,  170536, 10, 27, 25, }, // Uzbekistan
    {   86, 1803,   12666,  170561,  170584,  7, 23, 21, }, // Vanuatu
    {   86, 1811,       0,  170605,       0,  0,  9,  0, }, // Venezuela
    {   86, 1821,   12730,  170614,  170641, 11, 27, 25, }, // Vladivostok
    {   86, 1833,   12797,  170666,  170691,  9, 25, 23, }, // Volgograd
    {   86, 1843,       0,  113571,       0,  0,  6,  0, }, // Vostok
    {   86, 1850,       0,  170714,       0,  0, 10,  0, }, // Wake
    {   86, 1855,       0,  170724,       0,  0, 18,  0, }, // Wallis
    {   86, 1862,  126390,  170742,  170764,  6, 22, 20, }, // Yakutsk
    {   86, 1870,   12942,  170784,  170812, 12, 28, 26, }, // Yekaterinburg
    {   87,    5,       0,  170838,       0,  0, 13,  0, }, // Afghanistan Cherokee/Cherokee/United States
    {   87,   17,       0,  170851,       0,  0, 15,  0, }, // Africa_Central
    {   87,   32,       0,  170866,       0,  0, 16,  0, }, // Africa_Eastern
    {   87,   65,       0,  170882,       0,  0, 22,  0, }, // Africa_Southern
    {   87,   81,  170904,  170920,  170942, 16, 22, 19, }, // Africa_Western
    {   87,  107,  170961,  170971,  170987, 10, 16, 16, }, // Alaska
    {   87,  135,  171003,  171013,  171029, 10, 16, 13, }, // Amazon
    {   87,  142,  171042,  171051,  171066,  9, 15, 15, }, // America_Central
    {   87,  158,  171081,  171094,  171113, 13, 19, 19, }, // America_Eastern
    {   87,  174,  171132,  171141,  171156,  9, 15, 15, }, // America_Mountain
    {   87,  191,  171171,  171181,  171197, 10, 16, 16, }, // America_Pacific
    {   87,  214,  171213,  171222,  171237,  9, 15, 15, }, // Apia
    {   87,  232,  171252,  171262,  171278, 10, 16, 16, }, // Arabian
    {   87,  240,  171294,  171306,  171324, 12, 18, 15, }, // Argentina
    {   87,  250,  171339,  171359,  171385, 20, 26, 29, }, // Argentina_Western
    {   87,  268,  171414,  171425,  171442, 11, 17, 14, }, // Armenia
    {   87,  286,  171456,  171466,  171482, 10, 16, 16, }, // Atlantic
    {   87,  295,  171498,  171512,  171532, 14, 20, 20, }, // Australia_Central
    {   87,  313,  171552,  171574,  171602, 22, 28, 28, }, // Australia_CentralWestern
    {   87,  338,  171630,  171648,  171672, 18, 24, 24, }, // Australia_Eastern
    {   87,  356,  171556,  171578,  171606, 18, 24, 24, }, // Australia_Western
    {   87,  374,  171696,  171708,  171726, 12, 18, 15, }, // Azerbaijan
    {   87,  385,  171741,  171751,  171767, 10, 16, 13, }, // Azores
    {   87,  397,  171780,  171792,  171810, 12, 18, 15, }, // Bangladesh
    {   87,  415,       0,  171825,       0,  0,  9,  0, }, // Bhutan
    {   87,  422,       0,  171834,       0,  0, 10,  0, }, // Bolivia
    {   87,  437,  171844,  171854,  171870, 10, 16, 13, }, // Brasilia
    {   87,  454,       0,  171883,       0,  0, 15,  0, }, // Brunei
    {   87,  461,  171898,  171915,  171938, 17, 23, 20, }, // Cape_Verde
    {   87,  478,       0,  171958,       0,  0, 15,  0, }, // Chamorro
    {   87,  487,  171973,  171982,  171997,  9, 15, 15, }, // Chatham
    {   87,  495,  172012,  172020,  172034,  8, 14, 11, }, // Chile
    {   87,  501,  172045,  172057,  172075, 12, 18, 18, }, // China
    {   87,  507,       0,  172093,       0,  0, 19,  0, }, // Christmas
    {   87,  517,       0,  172112,       0,  0, 15,  0, }, // Cocos
    {   87,  523,  172127,  172139,  172157, 12, 18, 15, }, // Colombia
    {   87,  532,  172172,  172192,  172218, 20, 26, 27, }, // Cook
    {   87,  537,  172245,  172253,  172267,  8, 14, 14, }, // Cuba
    {   87,  548,       0,  172281,       0,  0,  9,  0, }, // Davis
    {   87,  564,       0,  172290,       0,  0, 17,  0, }, // DumontDUrville
    {   87,  601,       0,  172307,       0,  0, 14,  0, }, // East_Timor
    {   87,  612,  172321,  172340,  172365, 19, 25, 22, }, // Easter
    {   87,  619,       0,  172387,       0,  0, 10,  0, }, // Ecuador
    {   87,  627,  172397,  172410,  172429, 13, 19, 16, }, // Europe_Central
    {   87,  642,  172445,  172462,  172485, 17, 23, 20, }, // Europe_Eastern
    {   87,  657,       0,  172505,       0,  0, 14,  0, }, // Europe_Further_Eastern
    {   87,  680,  172519,  172536,  172559, 17, 23, 20, }, // Europe_Western
    {   87,  695,  172579,  172593,  172613, 14, 20, 17, }, // Falkland
    {   87,  704,  172630,  172638,  172652,  8, 14, 11, }, // Fiji
    {   87,  709,       0,  172663,       0,  0, 14,  0, }, // French_Guiana
    {   87,  723,       0,  172677,       0,  0, 24,  0, }, // French_Southern
    {   87,  746,       0,  172701,       0,  0, 18,  0, }, // Galapagos
    {   87,  756,       0,  172719,       0,  0, 10,  0, }, // Gambier
    {   87,  764,  172729,  172739,  172755, 10, 16, 13, }, // Georgia
    {   87,  772,       0,  172768,       0,  0, 16,  0, }, // Gilbert_Islands
    {   87,  788,       0,  172784,       0,  0, 13,  0, }, // GMT
    {   87,  830,  172797,  172813,  172835, 16, 22, 19, }, // Greenland_Eastern
    {   87,  848,  172854,  172870,  172892, 16, 22, 19, }, // Greenland_Western
    {   87,  871,       0,  172911,       0,  0, 17,  0, }, // Gulf
    {   87,  876,       0,  172928,       0,  0,  9,  0, }, // Guyana
    {   87,  883,  172937,  172953,  172975, 16, 22, 22, }, // Hawaii_Aleutian
    {   87,  899,  172997,  173010,  173029, 13, 19, 16, }, // Hong_Kong
    {   87,  909,  173045,  173054,  173069,  9, 15, 12, }, // Hovd
    {   87,  914,       0,  173081,       0,  0, 17,  0, }, // India
    {   87,  920,       0,  173098,       0,  0, 16,  0, }, // Indian_Ocean
    {   87,  933,       0,  173114,       0,  0, 15,  0, }, // Indochina
    {   87,  943,       0,  173129,       0,  0, 16,  0, }, // Indonesia_Central
    {   87,  961,       0,  173145,       0,  0, 20,  0, }, // Indonesia_Eastern
    {   87,  979,       0,  173165,       0,  0, 20,  0, }, // Indonesia_Western
    {   87,  997,  173185,  173194,  173209,  9, 15, 15, }, // Iran
    {   87, 1008,  173224,  173234,  173250, 10, 16, 13, }, // Irkutsk
    {   87, 1016,  173263,  173273,  173289, 10, 16, 17, }, // Israel
    {   87, 1023,  173306,  173316,  173332, 10, 16, 16, }, // Japan
    {   87, 1047,       0,  173348,       0,  0, 12,  0, }, // Kazakhstan
    {   87, 1058,       0,  173360,       0,  0, 17,  0, }, // Kazakhstan_Eastern
    {   87, 1077,       0,  173377,       0,  0, 17,  0, }, // Kazakhstan_Western
    {   87, 1106,  173394,  173404,  173420, 10, 16, 16, }, // Korea
    {   87, 1112,       0,  173436,       0,  0,  9,  0, }, // Kosrae
    {   87, 1119,  173445,  173457,  173475, 12, 18, 15, }, // Krasnoyarsk
    {   87, 1151,       0,  173490,       0,  0, 11,  0, }, // Kyrgystan
    {   87, 1175,       0,  173501,       0,  0, 17,  0, }, // Line_Islands
    {   87, 1188,  173518,  173532,  173552, 14, 20, 20, }, // Lord_Howe
    {   87, 1204,  173572,  173582,  173598, 10, 16, 13, }, // Magadan
    {   87, 1219,       0,  173611,       0,  0, 11,  0, }, // Malaysia
    {   87, 1228,       0,  173622,       0,  0, 11,  0, }, // Maldives
    {   87, 1237,       0,  173633,       0,  0, 11,  0, }, // Marquesas
    {   87, 1247,       0,  173644,       0,  0, 15,  0, }, // Marshall_Islands
    {   87, 1264,  173659,  173670,  173687, 11, 17, 14, }, // Mauritius
    {   87, 1274,       0,  173701,       0,  0,  9,  0, }, // Mawson
    {   87, 1281,  173710,  173726,  173748, 16, 22, 22, }, // Mexico_Pacific
    {   87, 1296,  173770,  173783,  173802, 13, 19, 16, }, // Mongolia
    {   87, 1305,  173818,  173828,  173844, 10, 16, 13, }, // Moscow
    {   87, 1312,       0,  173857,       0,  0, 10,  0, }, // Myanmar
    {   87, 1320,       0,  173867,       0,  0,  8,  0, }, // Nauru
    {   87, 1326,       0,  173875,       0,  0,  9,  0, }, // Nepal
    {   87, 1332,  173884,  173899,  173920, 15, 21, 18, }, // New_Caledonia
    {   87, 1346,  173938,  173952,  173972, 14, 20, 20, }, // New_Zealand
    {   87, 1358,  173992,  174008,  174030, 16, 22, 22, }, // Newfoundland
    {   87, 1371,       0,  174052,       0,  0,  8,  0, }, // Niue
    {   87, 1376,  174060,  174077,  174100, 17, 23, 20, }, // Norfolk
    {   87, 1384,  174120,  174137,  174160, 17, 23, 20, }, // Noronha
    {   87, 1406,  174180,  174192,  174210, 12, 18, 15, }, // Novosibirsk
    {   87, 1418,  174225,  174235,  174251, 10, 16, 13, }, // Omsk
    {   87, 1428,  174264,  174275,  174292, 11, 17, 14, }, // Pakistan
    {   87, 1437,       0,  174306,       0,  0,  8,  0, }, // Palau
    {   87, 1443,       0,  174314,       0,  0, 15,  0, }, // Papua_New_Guinea
    {   87, 1460,  174329,  174338,  174353,  9, 15, 12, }, // Paraguay
    {   87, 1469,  174365,  174373,  174387,  8, 14, 11, }, // Peru
    {   87, 1474,  174398,  174410,  174428, 12, 18, 15, }, // Philippines
    {   87, 1486,       0,  174443,       0,  0, 18,  0, }, // Phoenix_Islands
    {   87, 1502,  174461,  174481,  174507, 20, 26, 26, }, // Pierre_Miquelon
    {   87, 1518,       0,  174533,       0,  0, 10,  0, }, // Pitcairn
    {   87, 1527,       0,  174543,       0,  0,  9,  0, }, // Ponape
    {   87, 1534,       0,  174552,       0,  0, 13,  0, }, // Pyongyang
    {   87, 1554,       0,  174565,       0,  0, 11,  0, }, // Reunion
    {   87, 1562,       0,  174576,       0,  0,  9,  0, }, // Rothera
    {   87, 1570,  174585,  174595,  174611, 10, 16, 13, }, // Sakhalin
    {   87, 1596,  174624,  174633,  174648,  9, 15, 15, }, // Samoa
    {   87, 1602,       0,  174663,       0,  0, 10,  0, }, // Seychelles
    {   87, 1624,       0,  174673,       0,  0, 17,  0, }, // Singapore
    {   87, 1634,       0,  174690,       0,  0, 16,  0, }, // Solomon
    {   87, 1642,       0,  174706,       0,  0, 15,  0, }, // South_Georgia
    {   87, 1656,       0,  174721,       0,  0, 10,  0, }, // Suriname
    {   87, 1676,       0,  174731,       0,  0,  9,  0, }, // Syowa
    {   87, 1682,       0,  174740,       0,  0,  9,  0, }, // Tahiti
    {   87, 1689,  174749,  174758,  174773,  9, 15, 15, }, // Taipei
    {   87, 1696,       0,  174788,       0,  0, 12,  0, }, // Tajikistan
    {   87, 1724,       0,  174800,       0,  0, 10,  0, }, // Tokelau
    {   87, 1732,  174810,  174819,  174834,  9, 15, 12, }, // Tonga
    {   87, 1738,       0,  174846,       0,  0,  8,  0, }, // Truk
    {   87, 1750,  174854,  174868,  174888, 14, 20, 17, }, // Turkmenistan
    {   87, 1763,       0,  174905,       0,  0,  9,  0, }, // Tuvalu
    {   87, 1777,  174914,  174923,  174938,  9, 15, 12, }, // Uruguay
    {   87, 1792,  174950,  174963,  174982, 13, 19, 16, }, // Uzbekistan
    {   87, 1803,  174998,  175008,  175024, 10, 16, 13, }, // Vanuatu
    {   87, 1811,       0,  175037,       0,  0, 11,  0, }, // Venezuela
    {   87, 1821,  175048,  175061,  175080, 13, 19, 16, }, // Vladivostok
    {   87, 1833,  175096,  175107,  175124, 11, 17, 14, }, // Volgograd
    {   87, 1843,       0,  175138,       0,  0, 10,  0, }, // Vostok
    {   87, 1850,       0,  175148,       0,  0, 15,  0, }, // Wake
    {   87, 1855,       0,  175163,       0,  0, 16,  0, }, // Wallis
    {   87, 1862,  175179,  175189,  175205, 10, 16, 13, }, // Yakutsk
    {   87, 1870,  175218,  175231,  175250, 13, 19, 16, }, // Yekaterinburg
    {   87, 1892,       0,  175266,       0,  0,  9,  0, }, // Yukon
    {   90,    0,  175275,  175280,  175287,  5,  7,  7, }, // Acre Chinese/Simplified Han/China
    {   90,    5,       0,  135190,       0,  0,  5,  0, }, // Afghanistan
    {   90,   17,       0,  175294,       0,  0,  6,  0, }, // Africa_Central
    {   90,   32,       0,  175300,       0,  0,  6,  0, }, // Africa_Eastern
    {   90,   65,       0,  135203,       0,  0,  6,  0, }, // Africa_Southern
    {   90,   81,  175306,  175312,  175320,  6,  8,  8, }, // Africa_Western
    {   90,  107,  135225,  135231,  135239,  6,  8,  8, }, // Alaska
    {   90,  128,  135247,  135253,  135261,  6,  8,  8, }, // Almaty
    {   90,  135,  135269,  135274,  135281,  5,  7,  7, }, // Amazon
    {   90,  142,  175328,  175334,  175342,  6,  8,  8, }, // America_Central
    {   90,  158,  175350,  175356,  175364,  6,  8,  8, }, // America_Eastern
    {   90,  174,  175372,  175378,  175386,  6,  8,  8, }, // America_Mountain
    {   90,  191,  175394,  175401,  175410,  7,  9,  9, }, // America_Pacific
    {   90,  207,  135355,  175419,  175427,  6,  8,  8, }, // Anadyr
    {   90,  214,  135377,  135382,  135389,  5,  7,  7, }, // Apia
    {   90,  219,  175435,  175440,  175447,  5,  7,  7, }, // Aqtau
    {   90,  225,  175454,  175460,  175468,  6,  8,  8, }, // Aqtobe
    {   90,  232,  135437,  135442,  135449,  5,  7,  7, }, // Arabian
    {   90,  240,  135456,  135461,  135468,  5,  7,  7, }, // Argentina
    {   90,  250,  135475,  135482,  135491,  7,  9,  9, }, // Argentina_Western
    {   90,  268,  135500,  135506,  135514,  6,  8,  8, }, // Armenia
    {   90,  286,  135522,  135527,  135534,  5,  7,  7, }, // Atlantic
    {   90,  295,  175476,  175484,  175494,  8, 10, 10, }, // Australia_Central
    {   90,  313,  175504,  175513,  175524,  9, 11, 11, }, // Australia_CentralWestern
    {   90,  338,  175535,  175543,  175553,  8, 10, 10, }, // Australia_Eastern
    {   90,  356,  175563,  175571,  175581,  8, 10, 10, }, // Australia_Western
    {   90,  374,  175591,  175597,  175605,  6,  8,  8, }, // Azerbaijan
    {   90,  385,  135654,  135661,  135670,  7,  9,  9, }, // Azores
    {   90,  397,  135679,  135684,  135691,  5,  7,  7, }, // Bangladesh
    {   90,  415,       0,  135698,       0,  0,  4,  0, }, // Bhutan
    {   90,  422,       0,  175613,       0,  0,  8,  0, }, // Bolivia
    {   90,  437,  135708,  135714,  135722,  6,  8,  8, }, // Brasilia
    {   90,  454,       0,  175621,       0,  0,  8,  0, }, // Brunei
    {   90,  461,  175629,  175634,  175641,  5,  7,  7, }, // Cape_Verde
    {   90,  472,       0,  175648,       0,  0,  4,  0, }, // Casey
    {   90,  478,       0,  175652,       0,  0,  5,  0, }, // Chamorro
    {   90,  487,  175657,  175662,  175669,  5,  7,  7, }, // Chatham
    {   90,  495,  135785,  135789,  135795,  4,  6,  6, }, // Chile
    {   90,  501,  135801,  135805,  135811,  4,  6,  6, }, // China
    {   90,  507,       0,  135817,       0,  0,  5,  0, }, // Christmas
    {   90,  517,       0,  135822,       0,  0,  7,  0, }, // Cocos
    {   90,  523,  135829,  135835,  135843,  6,  8,  8, }, // Colombia
    {   90,  532,  135851,  135857,  175676,  6,  8,  8, }, // Cook
    {   90,  537,  135874,  135878,  135884,  4,  6,  6, }, // Cuba
    {   90,  548,       0,  135890,       0,  0,  5,  0, }, // Davis
    {   90,  564,       0,  175684,       0,  0,  8,  0, }, // DumontDUrville
    {   90,  601,       0,  135902,       0,  0,  5,  0, }, // East_Timor
    {   90,  612,  135907,  135913,  135921,  6,  8,  8, }, // Easter
    {   90,  619,       0,  175692,       0,  0,  8,  0, }, // Ecuador
    {   90,  627,  135934,  135938,  135944,  4,  6,  6, }, // Europe_Central
    {   90,  642,  135950,  135954,  135960,  4,  6,  6, }, // Europe_Eastern
    {   90,  657,       0,  175700,       0,  0,  6,  0, }, // Europe_Further_Eastern
    {   90,  680,  135972,  135976,  135982,  4,  6,  6, }, // Europe_Western
    {   90,  695,  135988,  135995,  136004,  7,  9,  9, }, // Falkland
    {   90,  704,  136013,  136017,  136023,  4,  6,  6, }, // Fiji
    {   90,  709,       0,  175706,       0,  0,  9,  0, }, // French_Guiana
    {   90,  723,       0,  175715,       0,  0, 11,  0, }, // French_Southern
    {   90,  746,       0,  175726,       0,  0,  6,  0, }, // Galapagos
    {   90,  756,       0,  175732,       0,  0,  5,  0, }, // Gambier
    {   90,  764,  175737,  175743,  175751,  6,  8,  8, }, // Georgia
    {   90,  772,       0,  136079,       0,  0,  8,  0, }, // Gilbert_Islands
    {   90,  788,       0,  175759,       0,  0,  8,  0, }, // GMT
    {   90,  830,  175767,  175775,  175785,  8, 10, 10, }, // Greenland_Eastern
    {   90,  848,  175795,  175803,  175813,  8, 10, 10, }, // Greenland_Western
    {   90,  866,       0,  175823,       0,  0,  4,  0, }, // Guam
    {   90,  871,       0,  175827,       0,  0,  6,  0, }, // Gulf
    {   90,  876,       0,  136031,       0,  0,  5,  0, }, // Guyana
    {   90,  883,  136165,  136174,  136185,  9, 11, 11, }, // Hawaii_Aleutian
    {   90,  899,  136196,  136200,  136206,  4,  6,  6, }, // Hong_Kong
    {   90,  909,  136212,  136217,  136224,  5,  7,  7, }, // Hovd
    {   90,  914,       0,  175833,       0,  0,  4,  0, }, // India
    {   90,  920,       0,  136237,       0,  0,  5,  0, }, // Indian_Ocean
    {   90,  933,       0,  175837,       0,  0,  6,  0, }, // Indochina
    {   90,  943,       0,  175843,       0,  0,  9,  0, }, // Indonesia_Central
    {   90,  961,       0,  175852,       0,  0,  9,  0, }, // Indonesia_Eastern
    {   90,  979,       0,  175861,       0,  0,  9,  0, }, // Indonesia_Western
    {   90,  997,  136266,  136270,  136276,  4,  6,  6, }, // Iran
    {   90, 1008,  175870,  175877,  175886,  7,  9,  9, }, // Irkutsk
    {   90, 1016,  136307,  136312,  136319,  5,  7,  7, }, // Israel
    {   90, 1023,  136326,  136330,  136336,  4,  6,  6, }, // Japan
    {   90, 1029,  175895,  175910,  175927, 15, 17, 17, }, // Kamchatka
    {   90, 1047,       0,  175944,       0,  0,  7,  0, }, // Kazakhstan
    {   90, 1058,       0,  175951,       0,  0,  9,  0, }, // Kazakhstan_Eastern
    {   90, 1077,       0,  175960,       0,  0,  9,  0, }, // Kazakhstan_Western
    {   90, 1106,  136398,  136402,  136408,  4,  6,  6, }, // Korea
    {   90, 1112,       0,  175969,       0,  0,  5,  0, }, // Kosrae
    {   90, 1119,  136419,  136429,  136441, 10, 12, 12, }, // Krasnoyarsk
    {   90, 1151,       0,  175974,       0,  0,  8,  0, }, // Kyrgystan
    {   90, 1161,       0,  136459,       0,  0,  4,  0, }, // Lanka
    {   90, 1175,       0,  136463,       0,  0,  6,  0, }, // Line_Islands
    {   90, 1188,  136469,  136475,  136483,  6,  8,  8, }, // Lord_Howe
    {   90, 1198,  136491,  136495,  136501,  4,  6,  6, }, // Macau
    {   90, 1204,  136507,  136512,  136519,  5,  7,  7, }, // Magadan
    {   90, 1219,       0,  136526,       0,  0,  6,  0, }, // Malaysia
    {   90, 1228,       0,  175982,       0,  0,  6,  0, }, // Maldives
    {   90, 1237,       0,  175988,       0,  0,  8,  0, }, // Marquesas
    {   90, 1247,       0,  136544,       0,  0,  7,  0, }, // Marshall_Islands
    {   90, 1264,  175996,  176002,  176010,  6,  8,  8, }, // Mauritius
    {   90, 1274,       0,  136573,       0,  0,  4,  0, }, // Mawson
    {   90, 1281,  136577,  136585,  136595,  8, 10, 10, }, // Mexico_Pacific
    {   90, 1296,  136605,  136611,  136619,  6,  8,  8, }, // Mongolia
    {   90, 1305,  136627,  136632,  136639,  5,  7,  7, }, // Moscow
    {   90, 1312,       0,  136646,       0,  0,  4,  0, }, // Myanmar
    {   90, 1320,       0,  176018,       0,  0,  4,  0, }, // Nauru
    {   90, 1326,       0,  136654,       0,  0,  5,  0, }, // Nepal
    {   90, 1332,  136659,  136667,  176022,  8, 10, 10, }, // New_Caledonia
    {   90, 1346,  176032,  176037,  176044,  5,  7,  7, }, // New_Zealand
    {   90, 1358,  136708,  136713,  136720,  5,  7,  7, }, // Newfoundland
    {   90, 1371,       0,  176051,       0,  0,  4,  0, }, // Niue
    {   90, 1376,  136732,  136738,  136746,  6,  8,  8, }, // Norfolk
    {   90, 1384,  176055,  176068,  176083, 13, 15, 15, }, // Noronha
    {   90, 1392,       0,  136800,       0,  0,  9,  0, }, // North_Mariana
    {   90, 1406,  136809,  136816,  136825,  7,  9,  9, }, // Novosibirsk
    {   90, 1418,  136834,  136840,  136848,  6,  8,  8, }, // Omsk
    {   90, 1428,  136856,  136862,  136870,  6,  8,  8, }, // Pakistan
    {   90, 1437,       0,  176098,       0,  0,  4,  0, }, // Palau
    {   90, 1443,       0,  176102,       0,  0,  9,  0, }, // Papua_New_Guinea
    {   90, 1460,  136891,  136896,  136903,  5,  7,  7, }, // Paraguay
    {   90, 1469,  136910,  136914,  136920,  4,  6,  6, }, // Peru
    {   90, 1474,  136926,  136931,  136938,  5,  7,  7, }, // Philippines
    {   90, 1486,       0,  176111,       0,  0,  8,  0, }, // Phoenix_Islands
    {   90, 1502,  136951,  136963,  136977, 12, 14, 14, }, // Pierre_Miquelon
    {   90, 1518,       0,  176119,       0,  0,  6,  0, }, // Pitcairn
    {   90, 1527,       0,  136996,       0,  0,  5,  0, }, // Ponape
    {   90, 1534,       0,  137001,       0,  0,  4,  0, }, // Pyongyang
    {   90, 1544,  176125,  176132,  176141,  7,  9,  9, }, // Qyzylorda
    {   90, 1554,       0,  176150,       0,  0,  5,  0, }, // Reunion
    {   90, 1562,       0,  137038,       0,  0,  5,  0, }, // Rothera
    {   90, 1570,  137043,  137048,  137055,  5,  7,  7, }, // Sakhalin
    {   90, 1579,  137062,  137067,  137074,  5,  7,  7, }, // Samara
    {   90, 1596,  137081,  137086,  137093,  5,  7,  7, }, // Samoa
    {   90, 1602,       0,  176155,       0,  0,  5,  0, }, // Seychelles
    {   90, 1624,       0,  137105,       0,  0,  7,  0, }, // Singapore
    {   90, 1634,       0,  176160,       0,  0,  7,  0, }, // Solomon
    {   90, 1642,       0,  176167,       0,  0,  7,  0, }, // South_Georgia
    {   90, 1656,       0,  176174,       0,  0,  5,  0, }, // Suriname
    {   90, 1676,       0,  176179,       0,  0,  4,  0, }, // Syowa
    {   90, 1682,       0,  176183,       0,  0,  6,  0, }, // Tahiti
    {   90, 1689,  137141,  137145,  137151,  4,  6,  6, }, // Taipei
    {   90, 1696,       0,  176189,       0,  0,  7,  0, }, // Tajikistan
    {   90, 1724,       0,  176196,       0,  0,  5,  0, }, // Tokelau
    {   90, 1732,  176201,  176205,  176211,  4,  6,  6, }, // Tonga
    {   90, 1738,       0,  176217,       0,  0,  4,  0, }, // Truk
    {   90, 1750,  176221,  176228,  176237,  7,  9,  9, }, // Turkmenistan
    {   90, 1763,       0,  176246,       0,  0,  5,  0, }, // Tuvalu
    {   90, 1777,  137214,  137219,  137226,  5,  7,  7, }, // Uruguay
    {   90, 1792,  176251,  176259,  176269,  8, 10, 10, }, // Uzbekistan
    {   90, 1803,  176279,  176285,  176293,  6,  8,  8, }, // Vanuatu
    {   90, 1811,       0,  137274,       0,  0,  6,  0, }, // Venezuela
    {   90, 1821,  137280,  137285,  137292,  5,  7,  7, }, // Vladivostok
    {   90, 1833,  137299,  137306,  137315,  7,  9,  9, }, // Volgograd
    {   90, 1843,       0,  137324,       0,  0,  6,  0, }, // Vostok
    {   90, 1850,       0,  137330,       0,  0,  5,  0, }, // Wake
    {   90, 1855,       0,  176301,       0,  0,  9,  0, }, // Wallis
    {   90, 1862,  176310,  176316,  176324,  6,  8,  8, }, // Yakutsk
    {   90, 1870,  137368,  137375,  137384,  7,  9,  9, }, // Yekaterinburg
    {   90, 1892,       0,  137393,       0,  0,  4,  0, }, // Yukon
    {   95, 1219,       0,  136526,       0,  0,  6,  0, }, // Malaysia Chinese/Simplified Han/Singapore
    {   95, 1624,       0,  137105,       0,  0,  7,  0, }, // Singapore
    {   96,   65,       0,  176332,       0,  0,  4,  0, }, // Africa_Southern Chinese/Traditional Han/Hong Kong
    {   96,  142,  176336,  176342,  176350,  6,  8,  8, }, // America_Central
    {   96,  158,  176358,  176364,  176372,  6,  8,  8, }, // America_Eastern
    {   96,  174,  176380,  176386,  176394,  6,  8,  8, }, // America_Mountain
    {   96,  191,  176402,  176409,  176418,  7,  9,  9, }, // America_Pacific
    {   96,  374,  176427,  176433,  176441,  6,  8,  8, }, // Azerbaijan
    {   96,  461,  176449,  176454,  176461,  5,  7,  7, }, // Cape_Verde
    {   96,  517,       0,  176468,       0,  0,  7,  0, }, // Cocos
    {   96,  564,       0,  176475,       0,  0,  7,  0, }, // DumontDUrville
    {   96,  619,       0,  176482,       0,  0,  6,  0, }, // Ecuador
    {   96,  746,       0,  176488,       0,  0,  8,  0, }, // Galapagos
    {   96,  764,  176496,  176502,  176510,  6,  8,  8, }, // Georgia
    {   96,  871,       0,  176518,       0,  0,  7,  0, }, // Gulf
    {   96,  876,       0,  133808,       0,  0,  5,  0, }, // Guyana
    {   96,  914,       0,  176525,       0,  0,  4,  0, }, // India
    {   96, 1008,  176529,  176536,  176545,  7,  9,  9, }, // Irkutsk
    {   96, 1112,       0,  176554,       0,  0,  5,  0, }, // Kosrae
    {   96, 1228,       0,  176559,       0,  0,  6,  0, }, // Maldives
    {   96, 1237,       0,  176565,       0,  0,  6,  0, }, // Marquesas
    {   96, 1264,  176571,  176577,  176585,  6,  8,  8, }, // Mauritius
    {   96, 1320,       0,  176593,       0,  0,  4,  0, }, // Nauru
    {   96, 1332,  134436,  134444,  176597,  8, 10, 10, }, // New_Caledonia
    {   96, 1384,  176607,  176618,  176631, 11, 13, 13, }, // Noronha
    {   96, 1443,       0,  176644,       0,  0,  9,  0, }, // Papua_New_Guinea
    {   96, 1518,       0,  176653,       0,  0,  5,  0, }, // Pitcairn
    {   96, 1602,       0,  176658,       0,  0,  5,  0, }, // Seychelles
    {   96, 1624,       0,  176663,       0,  0,  5,  0, }, // Singapore
    {   96, 1634,       0,  176668,       0,  0,  7,  0, }, // Solomon
    {   96, 1656,       0,  176675,       0,  0,  5,  0, }, // Suriname
    {   96, 1732,  176680,  176684,  176690,  4,  6,  6, }, // Tonga
    {   96, 1763,       0,  176696,       0,  0,  5,  0, }, // Tuvalu
    {   96, 1803,  176701,  176707,  176715,  6,  8,  8, }, // Vanuatu
    {   96, 1862,  176723,  176729,  176737,  6,  8,  8, }, // Yakutsk
    {   97,   65,       0,  176332,       0,  0,  4,  0, }, // Africa_Southern Chinese/Traditional Han/Macao
    {   97,  142,  176336,  176342,  176350,  6,  8,  8, }, // America_Central
    {   97,  158,  176358,  176364,  176372,  6,  8,  8, }, // America_Eastern
    {   97,  174,  176380,  176386,  176394,  6,  8,  8, }, // America_Mountain
    {   97,  191,  176402,  176409,  176418,  7,  9,  9, }, // America_Pacific
    {   97,  374,  176427,  176433,  176441,  6,  8,  8, }, // Azerbaijan
    {   97,  461,  176449,  176454,  176461,  5,  7,  7, }, // Cape_Verde
    {   97,  517,       0,  176468,       0,  0,  7,  0, }, // Cocos
    {   97,  564,       0,  176475,       0,  0,  7,  0, }, // DumontDUrville
    {   97,  619,       0,  176482,       0,  0,  6,  0, }, // Ecuador
    {   97,  746,       0,  176488,       0,  0,  8,  0, }, // Galapagos
    {   97,  764,  176496,  176502,  176510,  6,  8,  8, }, // Georgia
    {   97,  871,       0,  176518,       0,  0,  7,  0, }, // Gulf
    {   97,  876,       0,  133808,       0,  0,  5,  0, }, // Guyana
    {   97,  914,       0,  176525,       0,  0,  4,  0, }, // India
    {   97, 1008,  176529,  176536,  176545,  7,  9,  9, }, // Irkutsk
    {   97, 1112,       0,  176554,       0,  0,  5,  0, }, // Kosrae
    {   97, 1228,       0,  176559,       0,  0,  6,  0, }, // Maldives
    {   97, 1237,       0,  176565,       0,  0,  6,  0, }, // Marquesas
    {   97, 1264,  176571,  176577,  176585,  6,  8,  8, }, // Mauritius
    {   97, 1320,       0,  176593,       0,  0,  4,  0, }, // Nauru
    {   97, 1332,  134436,  134444,  176597,  8, 10, 10, }, // New_Caledonia
    {   97, 1384,  176607,  176618,  176631, 11, 13, 13, }, // Noronha
    {   97, 1443,       0,  176644,       0,  0,  9,  0, }, // Papua_New_Guinea
    {   97, 1518,       0,  176653,       0,  0,  5,  0, }, // Pitcairn
    {   97, 1602,       0,  176658,       0,  0,  5,  0, }, // Seychelles
    {   97, 1624,       0,  176663,       0,  0,  5,  0, }, // Singapore
    {   97, 1634,       0,  176668,       0,  0,  7,  0, }, // Solomon
    {   97, 1656,       0,  176675,       0,  0,  5,  0, }, // Suriname
    {   97, 1732,  176680,  176684,  176690,  4,  6,  6, }, // Tonga
    {   97, 1763,       0,  176696,       0,  0,  5,  0, }, // Tuvalu
    {   97, 1803,  176701,  176707,  176715,  6,  8,  8, }, // Vanuatu
    {   97, 1862,  176723,  176729,  176737,  6,  8,  8, }, // Yakutsk
    {   99,    0,  132951,  132955,  132961,  4,  6,  6, }, // Acre Chinese/Traditional Han/Taiwan
    {   99,    5,       0,  132967,       0,  0,  5,  0, }, // Afghanistan
    {   99,   17,       0,  132972,       0,  0,  4,  0, }, // Africa_Central
    {   99,   32,       0,  132976,       0,  0,  4,  0, }, // Africa_Eastern
    {   99,   65,       0,  132980,       0,  0,  6,  0, }, // Africa_Southern
    {   99,   81,  132986,  132990,  132996,  4,  6,  6, }, // Africa_Western
    {   99,  107,  133002,  133008,  133016,  6,  8,  8, }, // Alaska
    {   99,  128,  133024,  133030,  133038,  6,  8,  8, }, // Almaty
    {   99,  135,  133046,  133051,  133058,  5,  7,  7, }, // Amazon
    {   99,  142,  133065,  133069,  133075,  4,  6,  6, }, // America_Central
    {   99,  158,  133081,  133085,  133091,  4,  6,  6, }, // America_Eastern
    {   99,  174,  133097,  133101,  133107,  4,  6,  6, }, // America_Mountain
    {   99,  191,  133113,  133118,  133125,  5,  7,  7, }, // America_Pacific
    {   99,  207,  133132,  133138,  133146,  6,  8,  8, }, // Anadyr
    {   99,  214,  133154,  133159,  133166,  5,  7,  7, }, // Apia
    {   99,  219,  133173,  133178,  133185,  5,  7,  7, }, // Aqtau
    {   99,  225,  133192,  133198,  133206,  6,  8,  8, }, // Aqtobe
    {   99,  232,  133214,  133219,  133226,  5,  7,  7, }, // Arabian
    {   99,  240,  133233,  133238,  133245,  5,  7,  7, }, // Argentina
    {   99,  250,  133252,  133259,  133268,  7,  9,  9, }, // Argentina_Western
    {   99,  268,  133277,  133283,  133291,  6,  8,  8, }, // Armenia
    {   99,  286,  133299,  133304,  133311,  5,  7,  7, }, // Atlantic
    {   99,  295,  133318,  133324,  133332,  6,  8,  8, }, // Australia_Central
    {   99,  313,  133340,  133347,  133356,  7,  9,  9, }, // Australia_CentralWestern
    {   99,  338,  133365,  133371,  133379,  6,  8,  8, }, // Australia_Eastern
    {   99,  356,  133387,  133393,  133401,  6,  8,  8, }, // Australia_Western
    {   99,  374,  133409,  133415,  133423,  6,  8,  8, }, // Azerbaijan
    {   99,  385,  133431,  133438,  133447,  7,  9,  9, }, // Azores
    {   99,  397,  133456,  133461,  133468,  5,  7,  7, }, // Bangladesh
    {   99,  415,       0,  133475,       0,  0,  4,  0, }, // Bhutan
    {   99,  422,       0,  133479,       0,  0,  6,  0, }, // Bolivia
    {   99,  437,  133485,  133491,  133499,  6,  8,  8, }, // Brasilia
    {   99,  454,       0,  133507,       0,  0,  4,  0, }, // Brunei
    {   99,  461,  133511,  133516,  133523,  5,  7,  7, }, // Cape_Verde
    {   99,  472,       0,  133530,       0,  0,  5,  0, }, // Casey
    {   99,  478,       0,  133535,       0,  0,  5,  0, }, // Chamorro
    {   99,  487,  133540,  133546,  133554,  6,  8,  8, }, // Chatham
    {   99,  495,  133562,  133566,  133572,  4,  6,  6, }, // Chile
    {   99,  501,  133578,  133582,  133588,  4,  6,  6, }, // China
    {   99,  507,       0,  133594,       0,  0,  5,  0, }, // Christmas
    {   99,  517,       0,  133599,       0,  0,  7,  0, }, // Cocos
    {   99,  523,  133606,  133612,  133620,  6,  8,  8, }, // Colombia
    {   99,  532,  133628,  133634,  133642,  6,  8,  9, }, // Cook
    {   99,  537,  133651,  133655,  133661,  4,  6,  6, }, // Cuba
    {   99,  548,       0,  133667,       0,  0,  5,  0, }, // Davis
    {   99,  564,       0,  133672,       0,  0,  7,  0, }, // DumontDUrville
    {   99,  601,       0,  133679,       0,  0,  5,  0, }, // East_Timor
    {   99,  612,  133684,  133690,  133698,  6,  8,  8, }, // Easter
    {   99,  619,       0,  133706,       0,  0,  5,  0, }, // Ecuador
    {   99,  627,  133711,  133715,  133721,  4,  6,  6, }, // Europe_Central
    {   99,  642,  133727,  133731,  133737,  4,  6,  6, }, // Europe_Eastern
    {   99,  657,       0,  133743,       0,  0,  6,  0, }, // Europe_Further_Eastern
    {   99,  680,  133749,  133753,  133759,  4,  6,  6, }, // Europe_Western
    {   99,  695,  133765,  133772,  133781,  7,  9,  9, }, // Falkland
    {   99,  704,  133790,  133794,  133800,  4,  6,  6, }, // Fiji
    {   99,  709,       0,  133806,       0,  0,  7,  0, }, // French_Guiana
    {   99,  723,       0,  133813,       0,  0,  9,  0, }, // French_Southern
    {   99,  746,       0,  133822,       0,  0,  8,  0, }, // Galapagos
    {   99,  756,       0,  133830,       0,  0,  7,  0, }, // Gambier
    {   99,  764,  133837,  133842,  133849,  5,  7,  7, }, // Georgia
    {   99,  772,       0,  133856,       0,  0,  8,  0, }, // Gilbert_Islands
    {   99,  788,       0,  133864,       0,  0,  8,  0, }, // GMT
    {   99,  830,  133872,  133879,  133888,  7,  9,  9, }, // Greenland_Eastern
    {   99,  848,  133897,  133904,  133913,  7,  9,  9, }, // Greenland_Western
    {   99,  866,       0,  133922,       0,  0,  6,  0, }, // Guam
    {   99,  871,       0,  133928,       0,  0,  9,  0, }, // Gulf
    {   99,  876,       0,  133937,       0,  0,  5,  0, }, // Guyana
    {   99,  883,  133942,  133951,  133962,  9, 11, 11, }, // Hawaii_Aleutian
    {   99,  899,  133973,  133977,  133983,  4,  6,  6, }, // Hong_Kong
    {   99,  909,  133989,  133994,  134001,  5,  7,  7, }, // Hovd
    {   99,  914,       0,  134008,       0,  0,  6,  0, }, // India
    {   99,  920,       0,  134014,       0,  0,  5,  0, }, // Indian_Ocean
    {   99,  933,       0,  176745,       0,  0,  6,  0, }, // Indochina
    {   99,  943,       0,  134025,       0,  0,  6,  0, }, // Indonesia_Central
    {   99,  961,       0,  134031,       0,  0,  6,  0, }, // Indonesia_Eastern
    {   99,  979,       0,  134037,       0,  0,  6,  0, }, // Indonesia_Western
    {   99,  997,  134043,  134047,  134053,  4,  6,  6, }, // Iran
    {   99, 1008,  134059,  134066,  134075,  7,  9,  9, }, // Irkutsk
    {   99, 1016,  134084,  134089,  134096,  5,  7,  7, }, // Israel
    {   99, 1023,  134103,  134107,  134113,  4,  6,  6, }, // Japan
    {   99, 1029,  134119,  134130,  134143, 11, 13, 15, }, // Kamchatka
    {   99, 1047,       0,  134158,       0,  0,  5,  0, }, // Kazakhstan
    {   99, 1058,       0,  134163,       0,  0,  6,  0, }, // Kazakhstan_Eastern
    {   99, 1077,       0,  134169,       0,  0,  6,  0, }, // Kazakhstan_Western
    {   99, 1106,  134175,  134179,  134185,  4,  6,  6, }, // Korea
    {   99, 1112,       0,  134191,       0,  0,  5,  0, }, // Kosrae
    {   99, 1119,  134196,  134206,  134218, 10, 12, 12, }, // Krasnoyarsk
    {   99, 1151,       0,  134230,       0,  0,  6,  0, }, // Kyrgystan
    {   99, 1161,       0,  134236,       0,  0,  4,  0, }, // Lanka
    {   99, 1175,       0,  134240,       0,  0,  6,  0, }, // Line_Islands
    {   99, 1188,  134246,  134252,  134260,  6,  8,  8, }, // Lord_Howe
    {   99, 1198,  134268,  134272,  134278,  4,  6,  6, }, // Macau
    {   99, 1204,  134284,  134289,  134296,  5,  7,  7, }, // Magadan
    {   99, 1219,       0,  134303,       0,  0,  6,  0, }, // Malaysia
    {   99, 1228,       0,  134309,       0,  0,  6,  0, }, // Maldives
    {   99, 1237,       0,  134315,       0,  0,  6,  0, }, // Marquesas
    {   99, 1247,       0,  134321,       0,  0,  7,  0, }, // Marshall_Islands
    {   99, 1264,  134328,  134334,  134342,  6,  8,  8, }, // Mauritius
    {   99, 1274,       0,  134350,       0,  0,  4,  0, }, // Mawson
    {   99, 1281,  134354,  134362,  134372,  8, 10, 10, }, // Mexico_Pacific
    {   99, 1296,  134382,  134388,  134396,  6,  8,  8, }, // Mongolia
    {   99, 1305,  134404,  134409,  134416,  5,  7,  7, }, // Moscow
    {   99, 1312,       0,  134423,       0,  0,  4,  0, }, // Myanmar
    {   99, 1320,       0,  134427,       0,  0,  4,  0, }, // Nauru
    {   99, 1326,       0,  134431,       0,  0,  5,  0, }, // Nepal
    {   99, 1332,  134436,  134444,  134454,  8, 10, 12, }, // New_Caledonia
    {   99, 1346,  134466,  134471,  134478,  5,  7,  7, }, // New_Zealand
    {   99, 1358,  134485,  134490,  134497,  5,  7,  7, }, // Newfoundland
    {   99, 1371,       0,  134504,       0,  0,  5,  0, }, // Niue
    {   99, 1376,  134509,  134515,  134523,  6,  8,  8, }, // Norfolk
    {   99, 1384,  134531,  134545,  134561, 14, 16, 16, }, // Noronha
    {   99, 1392,       0,  134577,       0,  0,  9,  0, }, // North_Mariana
    {   99, 1406,  134586,  134593,  134602,  7,  9,  9, }, // Novosibirsk
    {   99, 1418,  134611,  134617,  134625,  6,  8,  8, }, // Omsk
    {   99, 1428,  134633,  134639,  134647,  6,  8,  8, }, // Pakistan
    {   99, 1437,       0,  134655,       0,  0,  4,  0, }, // Palau
    {   99, 1443,       0,  134659,       0,  0,  9,  0, }, // Papua_New_Guinea
    {   99, 1460,  134668,  134673,  134680,  5,  7,  7, }, // Paraguay
    {   99, 1469,  134687,  134691,  134697,  4,  6,  6, }, // Peru
    {   99, 1474,  134703,  134708,  134715,  5,  7,  7, }, // Philippines
    {   99, 1486,       0,  134722,       0,  0,  6,  0, }, // Phoenix_Islands
    {   99, 1502,  176751,  176762,  176775, 11, 13, 13, }, // Pierre_Miquelon
    {   99, 1518,       0,  134768,       0,  0,  5,  0, }, // Pitcairn
    {   99, 1527,       0,  134773,       0,  0,  5,  0, }, // Ponape
    {   99, 1534,       0,  134778,       0,  0,  4,  0, }, // Pyongyang
    {   99, 1544,  134782,  134790,  134800,  8, 10, 10, }, // Qyzylorda
    {   99, 1554,       0,  134810,       0,  0,  5,  0, }, // Reunion
    {   99, 1562,       0,  134815,       0,  0,  5,  0, }, // Rothera
    {   99, 1570,  134820,  134825,  134832,  5,  7,  7, }, // Sakhalin
    {   99, 1579,  134839,  134844,  134851,  5,  7,  7, }, // Samara
    {   99, 1596,  134858,  134863,  134870,  5,  7,  7, }, // Samoa
    {   99, 1602,       0,  134877,       0,  0,  5,  0, }, // Seychelles
    {   99, 1624,       0,  134882,       0,  0,  7,  0, }, // Singapore
    {   99, 1634,       0,  134889,       0,  0,  7,  0, }, // Solomon
    {   99, 1642,       0,  134896,       0,  0,  6,  0, }, // South_Georgia
    {   99, 1656,       0,  134902,       0,  0,  5,  0, }, // Suriname
    {   99, 1676,       0,  134907,       0,  0,  6,  0, }, // Syowa
    {   99, 1682,       0,  134913,       0,  0,  5,  0, }, // Tahiti
    {   99, 1689,  134918,  134922,  134928,  4,  6,  6, }, // Taipei
    {   99, 1696,       0,  134934,       0,  0,  5,  0, }, // Tajikistan
    {   99, 1724,       0,  134939,       0,  0,  7,  0, }, // Tokelau
    {   99, 1732,  134946,  134950,  134956,  4,  6,  6, }, // Tonga
    {   99, 1738,       0,  134962,       0,  0,  5,  0, }, // Truk
    {   99, 1750,  134967,  134972,  134979,  5,  7,  7, }, // Turkmenistan
    {   99, 1763,       0,  134986,       0,  0,  5,  0, }, // Tuvalu
    {   99, 1777,  134991,  134996,  135003,  5,  7,  7, }, // Uruguay
    {   99, 1792,  135010,  135016,  135024,  6,  8,  8, }, // Uzbekistan
    {   99, 1803,  135032,  135037,  135044,  5,  7,  7, }, // Vanuatu
    {   99, 1811,       0,  135051,       0,  0,  6,  0, }, // Venezuela
    {   99, 1821,  135057,  135062,  135069,  5,  7,  7, }, // Vladivostok
    {   99, 1833,  135076,  135083,  135092,  7,  9,  9, }, // Volgograd
    {   99, 1843,       0,  135101,       0,  0,  6,  0, }, // Vostok
    {   99, 1850,       0,  135107,       0,  0,  5,  0, }, // Wake
    {   99, 1855,       0,  135112,       0,  0, 11,  0, }, // Wallis
    {   99, 1862,  135123,  135129,  135137,  6,  8,  8, }, // Yakutsk
    {   99, 1870,  135145,  135152,  135161,  7,  9,  9, }, // Yekaterinburg
    {   99, 1892,       0,  176788,       0,  0,  6,  0, }, // Yukon
    {  100,  142,  176794,  176820,  176854, 26, 34, 34, }, // America_Central Church/Cyrillic/Russia
    {  100,  158,  176888,  176916,  176952, 28, 36, 36, }, // America_Eastern
    {  100,  174,  176988,  177019,  177058, 31, 39, 39, }, // America_Mountain
    {  100,  191,  177097,  177118,  177147, 21, 29, 29, }, // America_Pacific
    {  100,  286,  177176,  177198,  177228, 22, 30, 30, }, // Atlantic
    {  100,  627,  177258,  177283,  177316, 25, 33, 33, }, // Europe_Central
    {  100,  642,  177349,  177376,  177411, 27, 35, 35, }, // Europe_Eastern
    {  100,  657,       0,  177446,       0,  0, 34,  0, }, // Europe_Further_Eastern
    {  100,  680,  177480,  177506,  177540, 26, 34, 34, }, // Europe_Western
    {  100,  788,       0,  177574,       0,  0, 28,  0, }, // GMT
    {  100, 1008,  177602,  177620,  177646, 18, 26, 26, }, // Irkutsk
    {  100, 1058,       0,  177672,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {  100, 1077,       0,  177694,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  100, 1119,  177715,  177735,  177763, 20, 28, 28, }, // Krasnoyarsk
    {  100, 1151,       0,  177791,       0,  0,  9,  0, }, // Kyrgystan
    {  100, 1204,  177800,  177819,  177846, 19, 27, 27, }, // Magadan
    {  100, 1305,  177873,  177891,  177917, 18, 26, 26, }, // Moscow
    {  100, 1406,  177943,  177964,  177993, 21, 29, 29, }, // Novosibirsk
    {  100, 1418,  178022,  178037,  178060, 15, 23, 23, }, // Omsk
    {  100, 1570,  178083,  178102,  178129, 19, 27, 27, }, // Sakhalin
    {  100, 1821,  178156,  178178,  178208, 22, 30, 30, }, // Vladivostok
    {  100, 1833,  178238,  178259,  178288, 21, 29, 29, }, // Volgograd
    {  100, 1862,  178317,  178334,  178359, 17, 25, 25, }, // Yakutsk
    {  100, 1870,  178384,  178409,  178442, 25, 33, 33, }, // Yekaterinburg
    {  101,    0,  178475,  178486,  178508, 11, 22, 19, }, // Acre Chuvash/Cyrillic/Russia
    {  101,    5,       0,  178527,       0,  0, 17,  0, }, // Afghanistan
    {  101,   17,       0,  178544,       0,  0, 17,  0, }, // Africa_Central
    {  101,   32,       0,  178561,       0,  0, 24,  0, }, // Africa_Eastern
    {  101,   65,       0,  178585,       0,  0, 20,  0, }, // Africa_Southern
    {  101,   81,  178605,  178623,  178652, 18, 29, 26, }, // Africa_Western
    {  101,  107,  178678,  178691,  178715, 13, 24, 21, }, // Alaska
    {  101,  128,  178736,  178749,  178773, 13, 24, 21, }, // Almaty
    {  101,  135,  178794,  178809,  178835, 15, 26, 23, }, // Amazon
    {  101,  142,  178858,  178876,  178905, 18, 29, 26, }, // America_Central
    {  101,  158,  178931,  178956,  178992, 25, 36, 33, }, // America_Eastern
    {  101,  174,  179025,  179051,  179088, 26, 37, 34, }, // America_Mountain
    {  101,  191,  179122,  179140,  179169, 18, 29, 26, }, // America_Pacific
    {  101,  207,  179195,  179209,  179234, 14, 25, 22, }, // Anadyr
    {  101,  214,  179256,  179267,  179289, 11, 22, 19, }, // Apia
    {  101,  219,  179308,  179320,  179343, 12, 23, 20, }, // Aqtau
    {  101,  225,  179363,  179376,  179400, 13, 24, 21, }, // Aqtobe
    {  101,  232,  179421,  179432,  179454, 11, 22, 19, }, // Arabian
    {  101,  240,  179473,  179489,  179516, 16, 27, 24, }, // Argentina
    {  101,  250,  179540,  179561,  179593, 21, 32, 29, }, // Argentina_Western
    {  101,  268,  179622,  179635,  179659, 13, 24, 21, }, // Armenia
    {  101,  286,  179680,  179696,  179723, 16, 27, 24, }, // Atlantic
    {  101,  295,  179747,  179766,  179796, 19, 30, 27, }, // Australia_Central
    {  101,  313,  179823,  179847,  179882, 24, 35, 32, }, // Australia_CentralWestern
    {  101,  338,  179914,  179940,  179977, 26, 37, 34, }, // Australia_Eastern
    {  101,  356,  180011,  180031,  180062, 20, 31, 28, }, // Australia_Western
    {  101,  374,  180090,  180108,  180137, 18, 29, 26, }, // Azerbaijan
    {  101,  385,  180163,  180184,  180216, 21, 32, 29, }, // Azores
    {  101,  397,  180245,  180261,  180288, 16, 27, 24, }, // Bangladesh
    {  101,  415,       0,  180312,       0,  0, 12,  0, }, // Bhutan
    {  101,  422,       0,  180324,       0,  0, 13,  0, }, // Bolivia
    {  101,  437,  180337,  180351,  180376, 14, 25, 22, }, // Brasilia
    {  101,  454,       0,  180398,       0,  0, 24,  0, }, // Brunei
    {  101,  461,  180422,  180439,  180467, 17, 28, 25, }, // Cape_Verde
    {  101,  472,       0,  180492,       0,  0, 12,  0, }, // Casey
    {  101,  478,       0,  180504,       0,  0, 14,  0, }, // Chamorro
    {  101,  487,  180518,  180530,  180553, 12, 23, 20, }, // Chatham
    {  101,  495,  180573,  180584,  180606, 11, 22, 19, }, // Chile
    {  101,  501,  180625,  180637,  180660, 12, 23, 20, }, // China
    {  101,  507,       0,  180680,       0,  0, 21,  0, }, // Christmas
    {  101,  517,       0,  180701,       0,  0, 22,  0, }, // Cocos
    {  101,  523,  180723,  180737,  180762, 14, 25, 22, }, // Colombia
    {  101,  532,  180784,  180806,  180839, 22, 33, 30, }, // Cook
    {  101,  537,  180869,  180880,  180902, 11, 22, 19, }, // Cuba
    {  101,  548,       0,  180921,       0,  0, 13,  0, }, // Davis
    {  101,  564,       0,  180934,       0,  0, 21,  0, }, // DumontDUrville
    {  101,  601,       0,  180955,       0,  0, 23,  0, }, // East_Timor
    {  101,  612,  180978,  180999,  181031, 21, 32, 29, }, // Easter
    {  101,  619,       0,  181060,       0,  0, 14,  0, }, // Ecuador
    {  101,  627,  181074,  181091,  181119, 17, 28, 25, }, // Europe_Central
    {  101,  642,  181144,  181168,  181203, 24, 35, 32, }, // Europe_Eastern
    {  101,  657,       0,  181235,       0,  0, 30,  0, }, // Europe_Further_Eastern
    {  101,  680,  181265,  181283,  181312, 18, 29, 26, }, // Europe_Western
    {  101,  695,  181338,  181363,  181399, 25, 36, 33, }, // Falkland
    {  101,  704,  181432,  181444,  181467, 12, 23, 20, }, // Fiji
    {  101,  709,       0,  181487,       0,  0, 20,  0, }, // French_Guiana
    {  101,  723,       0,  181507,       0,  0, 48,  0, }, // French_Southern
    {  101,  746,       0,  181555,       0,  0, 26,  0, }, // Galapagos
    {  101,  756,       0,  181581,       0,  0, 13,  0, }, // Gambier
    {  101,  764,  181594,  181606,  181629, 12, 23, 20, }, // Georgia
    {  101,  772,       0,  181649,       0,  0, 26,  0, }, // Gilbert_Islands
    {  101,  788,       0,  181675,       0,  0, 22,  0, }, // GMT
    {  101,  802,  181697,  181713,  181740, 16, 27, 24, }, // Greenland
    {  101,  830,  181764,  181791,  181829, 27, 38, 35, }, // Greenland_Eastern
    {  101,  848,  181864,  181885,  181917, 21, 32, 19, }, // Greenland_Western
    {  101,  866,       0,  181936,       0,  0, 22,  0, }, // Guam
    {  101,  871,       0,  181958,       0,  0, 18,  0, }, // Gulf
    {  101,  876,       0,  181976,       0,  0, 13,  0, }, // Guyana
    {  101,  883,  181989,  182008,  182038, 19, 30, 27, }, // Hawaii_Aleutian
    {  101,  899,  182065,  182079,  182104, 14, 25, 22, }, // Hong_Kong
    {  101,  909,  182126,  182137,  182159, 11, 22, 19, }, // Hovd
    {  101,  914,       0,  182178,       0,  0, 11,  0, }, // India
    {  101,  920,       0,  182189,       0,  0, 18,  0, }, // Indian_Ocean
    {  101,  933,       0,  182207,       0,  0, 16,  0, }, // Indochina
    {  101,  943,       0,  182223,       0,  0, 19,  0, }, // Indonesia_Central
    {  101,  961,       0,  182242,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  101,  979,       0,  182268,       0,  0, 20,  0, }, // Indonesia_Western
    {  101,  997,  182288,  182299,  182321, 11, 22, 19, }, // Iran
    {  101, 1008,  182340,  182354,  182379, 14, 25, 22, }, // Irkutsk
    {  101, 1016,  182401,  182415,  182440, 14, 25, 22, }, // Israel
    {  101, 1023,  182462,  182474,  182497, 12, 23, 20, }, // Japan
    {  101, 1029,  182517,  182548,  182590, 31, 42, 39, }, // Kamchatka
    {  101, 1047,       0,  182629,       0,  0, 16,  0, }, // Kazakhstan
    {  101, 1058,       0,  182645,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  101, 1077,       0,  182672,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  101, 1106,  182693,  182705,  182728, 12, 23, 20, }, // Korea
    {  101, 1112,       0,  182748,       0,  0, 13,  0, }, // Kosrae
    {  101, 1119,  182761,  182778,  182806, 17, 28, 25, }, // Krasnoyarsk
    {  101, 1151,       0,  182831,       0,  0, 14,  0, }, // Kyrgystan
    {  101, 1161,       0,  182845,       0,  0, 12,  0, }, // Lanka
    {  101, 1175,       0,  182857,       0,  0, 21,  0, }, // Line_Islands
    {  101, 1188,  182878,  182893,  182919, 15, 26, 23, }, // Lord_Howe
    {  101, 1198,  182942,  182954,  182977, 12, 23, 20, }, // Macau
    {  101, 1204,  182997,  183011,  183036, 14, 25, 22, }, // Magadan
    {  101, 1219,       0,  183058,       0,  0, 14,  0, }, // Malaysia
    {  101, 1228,       0,  183072,       0,  0, 17,  0, }, // Maldives
    {  101, 1237,       0,  183089,       0,  0, 25,  0, }, // Marquesas
    {  101, 1247,       0,  183114,       0,  0, 24,  0, }, // Marshall_Islands
    {  101, 1264,  183138,  183153,  183179, 15, 26, 23, }, // Mauritius
    {  101, 1274,       0,  183202,       0,  0, 13,  0, }, // Mawson
    {  101, 1281,  183215,  183241,  183278, 26, 37, 34, }, // Mexico_Pacific
    {  101, 1296,  183312,  183329,  183357, 17, 28, 25, }, // Mongolia
    {  101, 1305,  183382,  183395,  183419, 13, 24, 21, }, // Moscow
    {  101, 1312,       0,  183440,       0,  0, 13,  0, }, // Myanmar
    {  101, 1320,       0,  183453,       0,  0, 12,  0, }, // Nauru
    {  101, 1326,       0,  183465,       0,  0, 12,  0, }, // Nepal
    {  101, 1332,  183477,  183497,  183528, 20, 31, 28, }, // New_Caledonia
    {  101, 1346,  183556,  183575,  183605, 19, 30, 27, }, // New_Zealand
    {  101, 1358,  183632,  183651,  183681, 19, 30, 27, }, // Newfoundland
    {  101, 1371,       0,  183708,       0,  0, 11,  0, }, // Niue
    {  101, 1376,  183719,  183733,  183758, 14, 25, 22, }, // Norfolk
    {  101, 1384,  183780,  183806,  183843, 26, 37, 34, }, // Noronha
    {  101, 1392,       0,  183877,       0,  0, 35,  0, }, // North_Mariana
    {  101, 1406,  183912,  183930,  183959, 18, 29, 26, }, // Novosibirsk
    {  101, 1418,  183985,  183996,  184018, 11, 22, 19, }, // Omsk
    {  101, 1428,  184037,  184052,  184078, 15, 26, 23, }, // Pakistan
    {  101, 1437,       0,  184101,       0,  0, 12,  0, }, // Palau
    {  101, 1443,       0,  184113,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  101, 1460,  184139,  184154,  184180, 15, 26, 23, }, // Paraguay
    {  101, 1469,  184203,  184214,  184236, 11, 22, 19, }, // Peru
    {  101, 1474,  184255,  184273,  184302, 18, 29, 26, }, // Philippines
    {  101, 1486,       0,  184328,       0,  0, 13,  0, }, // Phoenix_Islands
    {  101, 1502,  184341,  184369,  184408, 28, 39, 36, }, // Pierre_Miquelon
    {  101, 1518,       0,  184444,       0,  0, 14,  0, }, // Pitcairn
    {  101, 1527,       0,  184458,       0,  0, 13,  0, }, // Ponape
    {  101, 1534,       0,   90127,       0,  0,  7,  0, }, // Pyongyang
    {  101, 1544,  184471,  184487,  184514, 16, 27, 24, }, // Qyzylorda
    {  101, 1554,       0,  184538,       0,  0, 14,  0, }, // Reunion
    {  101, 1562,       0,  184552,       0,  0, 13,  0, }, // Rothera
    {  101, 1570,  184565,  184579,  184604, 14, 25, 22, }, // Sakhalin
    {  101, 1579,  184626,  184638,  184661, 12, 23, 20, }, // Samara
    {  101, 1596,  184681,  184693,  184716, 12, 23, 20, }, // Samoa
    {  101, 1602,       0,  184736,       0,  0, 23,  0, }, // Seychelles
    {  101, 1624,       0,  184759,       0,  0, 15,  0, }, // Singapore
    {  101, 1634,       0,  184774,       0,  0, 14,  0, }, // Solomon
    {  101, 1642,       0,  184788,       0,  0, 20,  0, }, // South_Georgia
    {  101, 1656,       0,  184808,       0,  0, 14,  0, }, // Suriname
    {  101, 1676,       0,  184822,       0,  0, 11,  0, }, // Syowa
    {  101, 1682,       0,  184833,       0,  0, 12,  0, }, // Tahiti
    {  101, 1689,  184845,  184858,  184882, 13, 24, 21, }, // Taipei
    {  101, 1696,       0,  184903,       0,  0, 18,  0, }, // Tajikistan
    {  101, 1724,       0,  184921,       0,  0, 14,  0, }, // Tokelau
    {  101, 1732,  184935,  184947,  184970, 12, 23, 20, }, // Tonga
    {  101, 1738,       0,  184990,       0,  0, 11,  0, }, // Truk
    {  101, 1750,  185001,  185020,  185050, 19, 30, 27, }, // Turkmenistan
    {  101, 1763,       0,  185077,       0,  0, 13,  0, }, // Tuvalu
    {  101, 1777,  185090,  185104,  185129, 14, 25, 22, }, // Uruguay
    {  101, 1792,  185151,  185168,  185196, 17, 28, 25, }, // Uzbekistan
    {  101, 1803,  185221,  185235,  185260, 14, 25, 22, }, // Vanuatu
    {  101, 1811,       0,  185282,       0,  0, 16,  0, }, // Venezuela
    {  101, 1821,  185298,  185316,  185345, 18, 29, 26, }, // Vladivostok
    {  101, 1833,  185371,  185387,  185414, 16, 27, 24, }, // Volgograd
    {  101, 1843,       0,  185438,       0,  0, 13,  0, }, // Vostok
    {  101, 1850,       0,  185451,       0,  0, 11,  0, }, // Wake
    {  101, 1855,       0,  185462,       0,  0, 25,  0, }, // Wallis
    {  101, 1862,  185487,  185500,  185524, 13, 24, 21, }, // Yakutsk
    {  101, 1870,  185545,  185564,  185594, 19, 30, 27, }, // Yekaterinburg
    {  101, 1892,       0,  185621,       0,  0, 11,  0, }, // Yukon
    {  102,   17,       0,  185632,       0,  0, 28,  0, }, // Africa_Central Colognian/Latin/Germany
    {  102,   32,       0,  185660,       0,  0, 22,  0, }, // Africa_Eastern
    {  102,   65,       0,  185682,       0,  0, 23,  0, }, // Africa_Southern
    {  102,   81,  185705,  185728,  185763, 23, 35, 29, }, // Africa_Western
    {  102,  385,  185792,  185809,  185838, 17, 29, 23, }, // Azores
    {  102,  461,  185861,  185882,  185915, 21, 33, 27, }, // Cape_Verde
    {  102,  627,  185942,  185966,  186002, 24, 36, 30, }, // Europe_Central
    {  102,  642,  186032,  186052,  186084, 20, 32, 26, }, // Europe_Eastern
    {  102,  680,  186110,  186131,  186164, 21, 33, 27, }, // Europe_Western
    {  102,  788,       0,  186191,       0,  0, 27,  0, }, // GMT
    {  102,  920,       0,  186218,       0,  0, 29,  0, }, // Indian_Ocean
    {  102, 1264,  186247,  186265,  186295, 18, 30, 24, }, // Mauritius
    {  102, 1554,       0,  186319,       0,  0, 16,  0, }, // Reunion
    {  102, 1602,       0,  186335,       0,  0, 22,  0, }, // Seychelles
    {  105,  788,       0,  186357,       0,  0, 24,  0, }, // GMT Corsican/Latin/France
    {  106,    0,  186381,  186393,  186416, 12, 23, 19, }, // Acre Croatian/Latin/Croatia
    {  106,    5,       0,  186435,       0,  0, 21,  0, }, // Afghanistan
    {  106,   17,       0,  186456,       0,  0, 22,  0, }, // Africa_Central
    {  106,   32,       0,  186478,       0,  0, 22,  0, }, // Africa_Eastern
    {  106,   65,       0,  186500,       0,  0, 20,  0, }, // Africa_Southern
    {  106,   81,  186520,  186542,  186575, 22, 33, 29, }, // Africa_Western
    {  106,  107,  186604,  186619,  186645, 15, 26, 22, }, // Alaska
    {  106,  128,  186667,  186684,  186712, 17, 28, 24, }, // Almaty
    {  106,  135,  186736,  186753,  186781, 17, 28, 24, }, // Amazon
    {  106,  142,  186805,  186822,  186850, 17, 28, 24, }, // America_Central
    {  106,  158,   97740,   97772,   97815, 15, 26, 22, }, // America_Eastern
    {  106,  174,   97854,   97888,   97933, 17, 28, 24, }, // America_Mountain
    {  106,  191,   97974,   98008,   98053, 17, 28, 24, }, // America_Pacific
    {  106,  207,  186874,  186891,  186919, 17, 28, 24, }, // Anadyr
    {  106,  214,  186943,  186956,  186980, 13, 24, 20, }, // Apia
    {  106,  219,  187000,  187019,  187049, 19, 30, 26, }, // Aqtau
    {  106,  225,  187075,  187095,  187126, 20, 31, 27, }, // Aqtobe
    {  106,  232,  187153,  187168,  187194, 15, 26, 22, }, // Arabian
    {  106,  240,   98492,   98518,   98555, 19, 30, 26, }, // Argentina
    {  106,  250,  187216,  187242,  187279, 26, 37, 33, }, // Argentina_Western
    {  106,  268,  187312,  187328,  187355, 16, 27, 23, }, // Armenia
    {  106,  286,   98664,   98698,   98743, 17, 28, 24, }, // Atlantic
    {  106,  295,  187378,  187403,  187439, 25, 36, 32, }, // Australia_Central
    {  106,  313,  187471,  187507,  187554, 36, 47, 43, }, // Australia_CentralWestern
    {  106,  338,  187597,  187622,  187658, 25, 36, 32, }, // Australia_Eastern
    {  106,  356,  187690,  187715,  187751, 25, 36, 32, }, // Australia_Western
    {  106,  374,  187783,  187805,  187838, 22, 33, 29, }, // Azerbaijan
    {  106,  385,  187867,  187882,  187908, 15, 26, 22, }, // Azores
    {  106,  397,  187930,  187949,  187979, 19, 30, 26, }, // Bangladesh
    {  106,  415,       0,  188005,       0,  0, 16,  0, }, // Bhutan
    {  106,  422,       0,  188021,       0,  0, 18,  0, }, // Bolivia
    {  106,  437,  188039,  188056,  188084, 17, 28, 24, }, // Brasilia
    {  106,  454,       0,  188108,       0,  0, 28,  0, }, // Brunei
    {  106,  461,  188136,  188163,  188201, 27, 38, 34, }, // Cape_Verde
    {  106,  472,       0,  188235,       0,  0, 15,  0, }, // Casey
    {  106,  478,       0,  188250,       0,  0, 27,  0, }, // Chamorro
    {  106,  487,  188277,  188293,  188320, 16, 27, 23, }, // Chatham
    {  106,  495,  188343,  188360,  188388, 17, 28, 24, }, // Chile
    {  106,  501,  101550,  188412,  188438, 15, 26, 22, }, // China
    {  106,  507,       0,  188460,       0,  0, 22,  0, }, // Christmas
    {  106,  517,       0,  188482,       0,  0, 23,  0, }, // Cocos
    {  106,  523,  188505,  188524,  188554, 19, 30, 26, }, // Colombia
    {  106,  532,  188580,  188602,  188635, 22, 33, 46, }, // Cook
    {  106,  537,  188681,  188697,  188724, 16, 27, 23, }, // Cuba
    {  106,  548,       0,  188747,       0,  0, 14,  0, }, // Davis
    {  106,  564,       0,  188761,       0,  0, 25,  0, }, // DumontDUrville
    {  106,  601,       0,  188786,       0,  0, 23,  0, }, // East_Timor
    {  106,  612,  188809,  188832,  188866, 23, 34, 30, }, // Easter
    {  106,  619,       0,  188896,       0,  0, 18,  0, }, // Ecuador
    {  106,  627,  188914,  188937,  188971, 23, 34, 30, }, // Europe_Central
    {  106,  642,  189001,  189024,  189058, 23, 34, 30, }, // Europe_Eastern
    {  106,  657,       0,  189088,       0,  0, 30,  0, }, // Europe_Further_Eastern
    {  106,  680,  189118,  189141,  189175, 23, 34, 30, }, // Europe_Western
    {  106,  695,  189205,  189224,  189254, 19, 30, 26, }, // Falkland
    {  106,  704,  189280,  189295,  189321, 15, 26, 22, }, // Fiji
    {  106,  709,       0,  189343,       0,  0, 24,  0, }, // French_Guiana
    {  106,  723,       0,  189367,       0,  0, 36,  0, }, // French_Southern
    {  106,  746,       0,  189403,       0,  0, 18,  0, }, // Galapagos
    {  106,  756,       0,  189421,       0,  0, 16,  0, }, // Gambier
    {  106,  764,  189437,  189454,  189482, 17, 28, 24, }, // Georgia
    {  106,  772,       0,  189506,       0,  0, 25,  0, }, // Gilbert_Islands
    {  106,  788,       0,  189531,       0,  0, 19,  0, }, // GMT
    {  106,  830,  189550,  189576,  189613, 26, 37, 33, }, // Greenland_Eastern
    {  106,  848,  189646,  189672,  189709, 26, 37, 33, }, // Greenland_Western
    {  106,  866,       0,  189742,       0,  0, 26,  0, }, // Guam
    {  106,  871,       0,  189768,       0,  0, 28,  0, }, // Gulf
    {  106,  876,       0,  100778,       0,  0, 17,  0, }, // Guyana
    {  106,  883,  189796,  189821,  189857, 25, 36, 32, }, // Hawaii_Aleutian
    {  106,  899,  189889,  189907,  189936, 18, 29, 25, }, // Hong_Kong
    {  106,  909,  189961,  189976,  190002, 15, 26, 22, }, // Hovd
    {  106,  914,       0,  190024,       0,  0, 16,  0, }, // India
    {  106,  920,       0,  190040,       0,  0, 24,  0, }, // Indian_Ocean
    {  106,  933,       0,  190064,       0,  0, 19,  0, }, // Indochina
    {  106,  943,       0,  190083,       0,  0, 27,  0, }, // Indonesia_Central
    {  106,  961,       0,  190110,       0,  0, 27,  0, }, // Indonesia_Eastern
    {  106,  979,       0,  190137,       0,  0, 27,  0, }, // Indonesia_Western
    {  106,  997,  190164,  190179,  190205, 15, 26, 22, }, // Iran
    {  106, 1008,  190227,  190243,  190270, 16, 27, 23, }, // Irkutsk
    {  106, 1016,  190293,  190310,  190338, 17, 28, 24, }, // Israel
    {  106, 1023,  190362,  190378,  190405, 16, 27, 23, }, // Japan
    {  106, 1029,  190428,  190459,  190501, 31, 42, 38, }, // Kamchatka
    {  106, 1047,       0,  102029,       0,  0, 20,  0, }, // Kazakhstan
    {  106, 1058,       0,  102049,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  106, 1077,       0,  102076,       0,  0, 27,  0, }, // Kazakhstan_Western
    {  106, 1106,  190539,  190555,  190582, 16, 27, 23, }, // Korea
    {  106, 1112,       0,  190605,       0,  0, 14,  0, }, // Kosrae
    {  106, 1119,  190619,  190639,  190670, 20, 31, 27, }, // Krasnoyarsk
    {  106, 1151,       0,  102271,       0,  0, 20,  0, }, // Kyrgystan
    {  106, 1161,       0,  190697,       0,  0, 17,  0, }, // Lanka
    {  106, 1175,       0,  190714,       0,  0, 25,  0, }, // Line_Islands
    {  106, 1188,  190739,  190762,  190796, 23, 34, 30, }, // Lord_Howe
    {  106, 1198,  190826,  190842,  190869, 16, 27, 23, }, // Macau
    {  106, 1204,  190892,  190910,  190939, 18, 29, 25, }, // Magadan
    {  106, 1219,       0,  190964,       0,  0, 18,  0, }, // Malaysia
    {  106, 1228,       0,  190982,       0,  0, 17,  0, }, // Maldives
    {  106, 1237,       0,  190999,       0,  0, 19,  0, }, // Marquesas
    {  106, 1247,       0,  191018,       0,  0, 24,  0, }, // Marshall_Islands
    {  106, 1264,  191042,  191061,  191091, 19, 30, 26, }, // Mauritius
    {  106, 1274,       0,  191117,       0,  0, 17,  0, }, // Mawson
    {  106, 1281,  191134,  191160,  191197, 26, 37, 33, }, // Mexico_Pacific
    {  106, 1296,  191230,  191250,  191281, 20, 31, 27, }, // Mongolia
    {  106, 1305,  191308,  191325,  191353, 17, 28, 24, }, // Moscow
    {  106, 1312,       0,  191377,       0,  0, 18,  0, }, // Myanmar
    {  106, 1320,       0,  191395,       0,  0, 14,  0, }, // Nauru
    {  106, 1326,       0,  191409,       0,  0, 16,  0, }, // Nepal
    {  106, 1332,  191425,  191448,  191482, 23, 34, 30, }, // New_Caledonia
    {  106, 1346,  191512,  191533,  191565, 21, 32, 28, }, // New_Zealand
    {  106, 1358,  191593,  191616,  191650, 23, 34, 30, }, // Newfoundland
    {  106, 1371,       0,  191680,       0,  0, 13,  0, }, // Niue
    {  106, 1376,  191693,  191714,  191746, 21, 32, 28, }, // Norfolk
    {  106, 1384,  191774,  191807,  191851, 33, 44, 40, }, // Noronha
    {  106, 1392,       0,  191891,       0,  0, 33,  0, }, // North_Mariana
    {  106, 1406,  191924,  191944,  191975, 20, 31, 27, }, // Novosibirsk
    {  106, 1418,  192002,  192015,  192039, 13, 24, 20, }, // Omsk
    {  106, 1428,  192059,  192078,  192108, 19, 30, 26, }, // Pakistan
    {  106, 1437,       0,  192134,       0,  0, 14,  0, }, // Palau
    {  106, 1443,       0,  192148,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  106, 1460,  192174,  192193,  192223, 19, 30, 26, }, // Paraguay
    {  106, 1469,  192249,  192266,  192294, 17, 28, 24, }, // Peru
    {  106, 1474,  192318,  192336,  192365, 18, 29, 25, }, // Philippines
    {  106, 1486,       0,  192390,       0,  0, 21,  0, }, // Phoenix_Islands
    {  106, 1502,  192411,  192443,  192486, 32, 43, 39, }, // Pierre_Miquelon
    {  106, 1518,       0,  192525,       0,  0, 19,  0, }, // Pitcairn
    {  106, 1527,       0,  192544,       0,  0, 18,  0, }, // Ponape
    {  106, 1534,       0,  192562,       0,  0, 19,  0, }, // Pyongyang
    {  106, 1544,  192581,  192604,  192638, 23, 34, 30, }, // Qyzylorda
    {  106, 1554,       0,  192668,       0,  0, 16,  0, }, // Reunion
    {  106, 1562,       0,  192684,       0,  0, 15,  0, }, // Rothera
    {  106, 1570,  192699,  192717,  192746, 18, 29, 25, }, // Sakhalin
    {  106, 1579,  192771,  192787,  192814, 16, 27, 23, }, // Samara
    {  106, 1596,  192837,  192854,  192882, 17, 28, 24, }, // Samoa
    {  106, 1602,       0,  192906,       0,  0, 17,  0, }, // Seychelles
    {  106, 1624,       0,  192923,       0,  0, 19,  0, }, // Singapore
    {  106, 1634,       0,  192942,       0,  0, 25,  0, }, // Solomon
    {  106, 1642,       0,  192967,       0,  0, 22,  0, }, // South_Georgia
    {  106, 1656,       0,  192989,       0,  0, 18,  0, }, // Suriname
    {  106, 1676,       0,  193007,       0,  0, 13,  0, }, // Syowa
    {  106, 1682,       0,  193020,       0,  0, 16,  0, }, // Tahiti
    {  106, 1689,  193036,  193052,  193079, 16, 27, 23, }, // Taipei
    {  106, 1696,       0,  104822,       0,  0, 22,  0, }, // Tajikistan
    {  106, 1724,       0,  193102,       0,  0, 16,  0, }, // Tokelau
    {  106, 1732,  193118,  193131,  193155, 13, 24, 20, }, // Tonga
    {  106, 1738,       0,  193175,       0,  0, 14,  0, }, // Truk
    {  106, 1750,  104951,  104974,  105008, 23, 34, 30, }, // Turkmenistan
    {  106, 1763,       0,  193189,       0,  0, 15,  0, }, // Tuvalu
    {  106, 1777,  193204,  193222,  193251, 18, 29, 25, }, // Uruguay
    {  106, 1792,  105129,  105150,  105182, 21, 32, 28, }, // Uzbekistan
    {  106, 1803,  193276,  193292,  193319, 16, 27, 23, }, // Vanuatu
    {  106, 1811,       0,  193342,       0,  0, 19,  0, }, // Venezuela
    {  106, 1821,  193361,  193382,  193414, 21, 32, 28, }, // Vladivostok
    {  106, 1833,  193442,  193462,  193493, 20, 31, 27, }, // Volgograd
    {  106, 1843,       0,  105314,       0,  0, 16,  0, }, // Vostok
    {  106, 1850,       0,  193520,       0,  0, 18,  0, }, // Wake
    {  106, 1855,       0,  193538,       0,  0, 29,  0, }, // Wallis
    {  106, 1862,  193567,  193583,  193610, 16, 27, 23, }, // Yakutsk
    {  106, 1870,  193633,  193656,  193690, 23, 34, 30, }, // Yekaterinburg
    {  106, 1892,       0,  193720,       0,  0, 16,  0, }, // Yukon
    {  108,    0,  193736,  193748,  193771, 12, 23, 18, }, // Acre Czech/Latin/Czechia
    {  108,    5,       0,  193789,       0,  0, 13,  0, }, // Afghanistan
    {  108,   17,       0,  193802,       0,  0, 17,  0, }, // Africa_Central
    {  108,   32,       0,  193819,       0,  0, 18,  0, }, // Africa_Eastern
    {  108,   65,       0,  193837,       0,  0, 15,  0, }, // Africa_Southern
    {  108,   81,  193852,  193869,  193897, 17, 28, 23, }, // Africa_Western
    {  108,  107,  193920,  193932,  193955, 12, 23, 18, }, // Alaska
    {  108,  128,  193973,  193985,  194008, 12, 23, 18, }, // Almaty
    {  108,  135,  194026,  194039,  194063, 13, 24, 19, }, // Amazon
    {  108,  142,  194082,  194110,  194149, 28, 39, 34, }, // America_Central
    {  108,  158,  194183,  194210,  194248, 27, 38, 33, }, // America_Eastern
    {  108,  174,  194281,  194306,  194342, 25, 36, 31, }, // America_Mountain
    {  108,  191,  194373,  194401,  194440, 28, 39, 34, }, // America_Pacific
    {  108,  207,  194474,  194487,  194511, 13, 24, 19, }, // Anadyr
    {  108,  214,  194530,  194541,  194563, 11, 22, 17, }, // Apia
    {  108,  219,  194580,  194592,  194615, 12, 23, 18, }, // Aqtau
    {  108,  225,  194633,  194645,  194668, 12, 23, 18, }, // Aqtobe
    {  108,  232,  194686,  194697,  194719, 11, 22, 17, }, // Arabian
    {  108,  240,  194736,  194751,  194777, 15, 26, 21, }, // Argentina
    {  108,  250,  194798,  194819,  194851, 21, 32, 27, }, // Argentina_Western
    {  108,  268,  194878,  194890,  194913, 12, 23, 18, }, // Armenia
    {  108,  286,  194931,  194945,  194970, 14, 25, 20, }, // Atlantic
    {  108,  295,  194990,  195010,  195041, 20, 31, 26, }, // Australia_Central
    {  108,  313,  195067,  195095,  195134, 28, 39, 34, }, // Australia_CentralWestern
    {  108,  338,  195168,  195189,  195221, 21, 32, 27, }, // Australia_Eastern
    {  108,  356,  195248,  195268,  195299, 20, 31, 26, }, // Australia_Western
    {  108,  374,  195325,  195343,  195372, 18, 29, 24, }, // Azerbaijan
    {  108,  385,  195396,  195407,  195429, 11, 22, 17, }, // Azores
    {  108,  397,  195446,  195462,  195489, 16, 27, 22, }, // Bangladesh
    {  108,  415,       0,  195511,       0,  0, 13,  0, }, // Bhutan
    {  108,  422,       0,  195524,       0,  0, 14,  0, }, // Bolivia
    {  108,  437,  195538,  195553,  195579, 15, 26, 21, }, // Brasilia
    {  108,  454,       0,  195600,       0,  0, 13,  0, }, // Brunei
    {  108,  461,  195613,  195627,  195652, 14, 25, 20, }, // Cape_Verde
    {  108,  472,       0,  195672,       0,  0, 19,  0, }, // Casey
    {  108,  478,       0,  195691,       0,  0, 14,  0, }, // Chamorro
    {  108,  487,  195705,  195719,  195744, 14, 25, 20, }, // Chatham
    {  108,  495,  195764,  195775,  195797, 11, 22, 17, }, // Chile
    {  108,  501,  195814,  195824,  195845, 10, 21, 16, }, // China
    {  108,  507,       0,  195861,       0,  0, 21,  0, }, // Christmas
    {  108,  517,       0,  195882,       0,  0, 22,  0, }, // Cocos
    {  108,  523,  195904,  195919,  195945, 15, 26, 21, }, // Colombia
    {  108,  532,  195966,  195987,  196019, 21, 32, 27, }, // Cook
    {  108,  537,  196046,  196058,  196081, 12, 23, 18, }, // Cuba
    {  108,  548,       0,  196099,       0,  0, 20,  0, }, // Davis
    {  108,  564,       0,  196119,       0,  0, 29,  0, }, // DumontDUrville
    {  108,  601,       0,  196148,       0,  0, 19,  0, }, // East_Timor
    {  108,  612,  196167,  196192,  196228, 25, 36, 31, }, // Easter
    {  108,  619,       0,  196259,       0,  0, 14,  0, }, // Ecuador
    {  108,  627,  196273,  196291,  196320, 18, 29, 24, }, // Europe_Central
    {  108,  642,  196344,  196363,  196393, 19, 30, 25, }, // Europe_Eastern
    {  108,  657,       0,  196418,       0,  0, 24,  0, }, // Europe_Further_Eastern
    {  108,  680,  196442,  196460,  196489, 18, 29, 24, }, // Europe_Western
    {  108,  695,  196513,  196528,  196554, 15, 26, 21, }, // Falkland
    {  108,  704,  196575,  196588,  196612, 13, 24, 19, }, // Fiji
    {  108,  709,       0,  196631,       0,  0, 23,  0, }, // French_Guiana
    {  108,  723,       0,  196654,       0,  0, 47,  0, }, // French_Southern
    {  108,  746,       0,  196701,       0,  0, 14,  0, }, // Galapagos
    {  108,  756,       0,  196715,       0,  0, 14,  0, }, // Gambier
    {  108,  764,  196729,  196742,  196766, 13, 24, 19, }, // Georgia
    {  108,  772,       0,  196785,       0,  0, 24,  0, }, // Gilbert_Islands
    {  108,  788,       0,  196809,       0,  0, 24,  0, }, // GMT
    {  108,  830,  196833,  196851,  196880, 18, 29, 24, }, // Greenland_Eastern
    {  108,  848,  196904,  196921,  196949, 17, 28, 23, }, // Greenland_Western
    {  108,  866,       0,  196972,       0,  0, 11,  0, }, // Guam
    {  108,  871,       0,  196983,       0,  0, 30,  0, }, // Gulf
    {  108,  876,       0,  196642,       0,  0, 12,  0, }, // Guyana
    {  108,  883,  197013,  197034,  197066, 21, 32, 27, }, // Hawaii_Aleutian
    {  108,  899,  197093,  197108,  197134, 15, 26, 21, }, // Hong_Kong
    {  108,  909,  197155,  197166,  197188, 11, 22, 17, }, // Hovd
    {  108,  914,       0,  197205,       0,  0, 11,  0, }, // India
    {  108,  920,       0,  197216,       0,  0, 19,  0, }, // Indian_Ocean
    {  108,  933,       0,  197235,       0,  0, 14,  0, }, // Indochina
    {  108,  943,       0,  197249,       0,  0, 19,  0, }, // Indonesia_Central
    {  108,  961,       0,  197268,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  108,  979,       0,  197288,       0,  0, 19,  0, }, // Indonesia_Western
    {  108,  997,  197307,  197318,  197340, 11, 22, 17, }, // Iran
    {  108, 1008,  197357,  197369,  197392, 12, 23, 18, }, // Irkutsk
    {  108, 1016,  197410,  197423,  197447, 13, 24, 19, }, // Israel
    {  108, 1023,  197466,  197478,  197501, 12, 23, 18, }, // Japan
    {  108, 1029,  197519,  197547,  197586, 28, 39, 34, }, // Kamchatka
    {  108, 1047,       0,  197620,       0,  0, 17,  0, }, // Kazakhstan
    {  108, 1058,       0,  197637,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  108, 1077,       0,  197661,       0,  0, 23,  0, }, // Kazakhstan_Western
    {  108, 1106,  197684,  197696,  197719, 12, 23, 18, }, // Korea
    {  108, 1112,       0,  197737,       0,  0, 13,  0, }, // Kosrae
    {  108, 1119,  197750,  197766,  197793, 16, 27, 22, }, // Krasnoyarsk
    {  108, 1151,       0,  197815,       0,  0, 13,  0, }, // Kyrgystan
    {  108, 1161,       0,  197828,       0,  0, 13,  0, }, // Lanka
    {  108, 1175,       0,  197841,       0,  0, 23,  0, }, // Line_Islands
    {  108, 1188,  197864,  197886,  197919, 22, 33, 28, }, // Lord_Howe
    {  108, 1198,  197947,  197959,  197982, 12, 23, 18, }, // Macau
    {  108, 1204,  198000,  198014,  198039, 14, 25, 20, }, // Magadan
    {  108, 1219,       0,  198059,       0,  0, 12,  0, }, // Malaysia
    {  108, 1228,       0,  198071,       0,  0, 14,  0, }, // Maldives
    {  108, 1237,       0,  198085,       0,  0, 13,  0, }, // Marquesas
    {  108, 1247,       0,  198098,       0,  0, 25,  0, }, // Marshall_Islands
    {  108, 1264,  198123,  198138,  198164, 15, 26, 21, }, // Mauritius
    {  108, 1274,       0,  198185,       0,  0, 21,  0, }, // Mawson
    {  108, 1281,  198206,  198227,  198259, 21, 32, 27, }, // Mexico_Pacific
    {  108, 1296,  198286,  198302,  198329, 16, 27, 22, }, // Mongolia
    {  108, 1305,  198351,  198364,  198388, 13, 24, 19, }, // Moscow
    {  108, 1312,       0,  198407,       0,  0, 14,  0, }, // Myanmar
    {  108, 1320,       0,  198421,       0,  0, 11,  0, }, // Nauru
    {  108, 1326,       0,  198432,       0,  0, 12,  0, }, // Nepal
    {  108, 1332,  198444,  198462,  198491, 18, 29, 24, }, // New_Caledonia
    {  108, 1346,  198515,  198532,  198560, 17, 28, 23, }, // New_Zealand
    {  108, 1358,  198583,  198602,  198632, 19, 30, 25, }, // Newfoundland
    {  108, 1371,       0,  198657,       0,  0, 12,  0, }, // Niue
    {  108, 1376,  198669,  198683,  198708, 14, 25, 20, }, // Norfolk
    {  108, 1384,  198728,  198762,  198807, 34, 45, 40, }, // Noronha
    {  108, 1392,       0,  198847,       0,  0, 19,  0, }, // North_Mariana
    {  108, 1406,  198866,  198882,  198909, 16, 27, 22, }, // Novosibirsk
    {  108, 1418,  198931,  198940,  198960,  9, 20, 15, }, // Omsk
    {  108, 1428,  198975,  198990,  199016, 15, 26, 21, }, // Pakistan
    {  108, 1437,       0,  199037,       0,  0, 12,  0, }, // Palau
    {  108, 1443,       0,  199049,       0,  0, 21,  0, }, // Papua_New_Guinea
    {  108, 1460,  199070,  199085,  199111, 15, 26, 21, }, // Paraguay
    {  108, 1469,  199132,  199145,  199169, 13, 24, 19, }, // Peru
    {  108, 1474,  199188,  199202,  199227, 14, 25, 20, }, // Philippines
    {  108, 1486,       0,  199247,       0,  0, 22,  0, }, // Phoenix_Islands
    {  108, 1502,  199269,  199291,  199324, 22, 33, 28, }, // Pierre_Miquelon
    {  108, 1518,       0,  199352,       0,  0, 25,  0, }, // Pitcairn
    {  108, 1527,       0,  199377,       0,  0, 12,  0, }, // Ponape
    {  108, 1534,       0,  199389,       0,  0, 18,  0, }, // Pyongyang
    {  108, 1544,  199407,  199422,  199448, 15, 26, 21, }, // Qyzylorda
    {  108, 1554,       0,  199469,       0,  0, 14,  0, }, // Reunion
    {  108, 1562,       0,  199483,       0,  0, 21,  0, }, // Rothera
    {  108, 1570,  199504,  199519,  199545, 15, 26, 21, }, // Sakhalin
    {  108, 1579,  199566,  199578,  199601, 12, 23, 18, }, // Samara
    {  108, 1596,  199619,  199631,  199654, 12, 23, 18, }, // Samoa
    {  108, 1602,       0,  199672,       0,  0, 14,  0, }, // Seychelles
    {  108, 1624,       0,  199686,       0,  0, 15,  0, }, // Singapore
    {  108, 1634,       0,  199701,       0,  0, 25,  0, }, // Solomon
    {  108, 1642,       0,  199726,       0,  0, 17,  0, }, // South_Georgia
    {  108, 1656,       0,  199743,       0,  0, 14,  0, }, // Suriname
    {  108, 1676,       0,  199757,       0,  0, 16,  0, }, // Syowa
    {  108, 1682,       0,  199773,       0,  0, 12,  0, }, // Tahiti
    {  108, 1689,  199785,  199800,  199826, 15, 26, 21, }, // Taipei
    {  108, 1696,       0,  199847,       0,  0, 12,  0, }, // Tajikistan
    {  108, 1724,       0,  199859,       0,  0, 14,  0, }, // Tokelau
    {  108, 1732,  199873,  199884,  199906, 11, 22, 17, }, // Tonga
    {  108, 1738,       0,  199923,       0,  0, 12,  0, }, // Truk
    {  108, 1750,  199935,  199949,  199974, 14, 25, 20, }, // Turkmenistan
    {  108, 1763,       0,  199994,       0,  0, 12,  0, }, // Tuvalu
    {  108, 1777,  200006,  200020,  200045, 14, 25, 20, }, // Uruguay
    {  108, 1792,  200065,  200076,  200098, 11, 22, 17, }, // Uzbekistan
    {  108, 1803,  200115,  200128,  200152, 13, 24, 19, }, // Vanuatu
    {  108, 1811,       0,  200171,       0,  0, 15,  0, }, // Venezuela
    {  108, 1821,  200186,  200203,  200231, 17, 28, 23, }, // Vladivostok
    {  108, 1833,  200254,  200270,  200297, 16, 27, 22, }, // Volgograd
    {  108, 1843,       0,  200319,       0,  0, 18,  0, }, // Vostok
    {  108, 1850,       0,  200337,       0,  0, 16,  0, }, // Wake
    {  108, 1855,       0,  200353,       0,  0, 27,  0, }, // Wallis
    {  108, 1862,  200380,  200392,  200415, 12, 23, 18, }, // Yakutsk
    {  108, 1870,  200433,  200452,  200482, 19, 30, 25, }, // Yekaterinburg
    {  108, 1892,       0,  200507,       0,  0, 12,  0, }, // Yukon
    {  109,    0,  200519,  200527,  200541,  8, 14, 14, }, // Acre Danish/Latin/Denmark
    {  109,    5,       0,  200555,       0,  0, 12,  0, }, // Afghanistan
    {  109,   17,       0,  200567,       0,  0, 20,  0, }, // Africa_Central
    {  109,   32,       0,  200587,       0,  0, 16,  0, }, // Africa_Eastern
    {  109,   65,       0,  200603,       0,  0, 16,  0, }, // Africa_Southern
    {  109,   81,  200619,  200636,  200659, 17, 23, 23, }, // Africa_Western
    {  109,  107,  200682,  200692,  200708, 10, 16, 16, }, // Alaska
    {  109,  128,  200724,  200734,  200750, 10, 16, 16, }, // Almaty
    {  109,  135,  200766,  200778,  200796, 12, 18, 18, }, // Amazon
    {  109,  142,  200814,  200825,  200842, 11, 17, 17, }, // America_Central
    {  109,  158,  200859,  200870,  200887, 11, 17, 17, }, // America_Eastern
    {  109,  174,  200904,  200916,  200934, 12, 18, 18, }, // America_Mountain
    {  109,  191,  200952,  200963,  200980, 11, 17, 17, }, // America_Pacific
    {  109,  207,  200997,  201007,  201023, 10, 16, 16, }, // Anadyr
    {  109,  214,  201039,  201047,  201061,  8, 14, 14, }, // Apia
    {  109,  219,  201075,  201084,  201099,  9, 15, 15, }, // Aqtau
    {  109,  225,  201114,  201124,  201140, 10, 16, 16, }, // Aqtobe
    {  109,  232,  201156,  201167,  201184, 11, 17, 17, }, // Arabian
    {  109,  240,  201201,  201214,  201234, 13, 20, 20, }, // Argentina
    {  109,  250,  201254,  201272,  201296, 18, 24, 24, }, // Argentina_Western
    {  109,  268,  201320,  201331,  201348, 11, 17, 17, }, // Armenia
    {  109,  286,  201365,  201377,  201395, 12, 18, 18, }, // Atlantic
    {  109,  295,  201413,  201433,  201459, 20, 26, 26, }, // Australia_Central
    {  109,  313,  201485,  201513,  201547, 28, 34, 34, }, // Australia_CentralWestern
    {  109,  338,  201581,  201597,  201619, 16, 22, 22, }, // Australia_Eastern
    {  109,  356,  201641,  201658,  201681, 17, 23, 23, }, // Australia_Western
    {  109,  374,  201704,  201722,  201746, 18, 24, 24, }, // Azerbaijan
    {  109,  385,  201770,  201782,  201800, 12, 18, 18, }, // Azores
    {  109,  397,  201818,  201832,  201852, 14, 20, 20, }, // Bangladesh
    {  109,  415,       0,  201872,       0,  0, 10,  0, }, // Bhutan
    {  109,  422,       0,  201882,       0,  0, 14,  0, }, // Bolivia
    {  109,  437,  201896,  201911,  201932, 15, 21, 21, }, // Brasilia
    {  109,  454,       0,  201953,       0,  0, 21,  0, }, // Brunei
    {  109,  461,  201974,  201987,  202006, 13, 19, 19, }, // Cape_Verde
    {  109,  478,       0,  202025,       0,  0, 12,  0, }, // Chamorro
    {  109,  487,  202037,  202048,  202065, 11, 17, 17, }, // Chatham
    {  109,  495,  202082,  202094,  202112, 12, 18, 18, }, // Chile
    {  109,  501,  202130,  202142,  202160, 12, 18, 18, }, // China
    {  109,  507,       0,  202178,       0,  0, 17,  0, }, // Christmas
    {  109,  517,       0,  202195,       0,  0, 20,  0, }, // Cocos
    {  109,  523,  202215,  202230,  202251, 15, 21, 21, }, // Colombia
    {  109,  532,  202272,  202285,  202304, 13, 19, 19, }, // Cook
    {  109,  537,  202323,  202334,  202351, 11, 17, 17, }, // Cuba
    {  109,  548,       0,  202368,       0,  0,  9,  0, }, // Davis
    {  109,  564,       0,  202377,       0,  0, 20,  0, }, // DumontDUrville
    {  109,  601,       0,  202397,       0,  0, 12,  0, }, // East_Timor
    {  109,  612,  202409,  202421,  202439, 12, 18, 18, }, // Easter
    {  109,  619,       0,  202457,       0,  0, 16,  0, }, // Ecuador
    {  109,  627,  202473,  202493,  202519, 20, 26, 26, }, // Europe_Central
    {  109,  642,  202545,  202561,  202583, 16, 22, 22, }, // Europe_Eastern
    {  109,  657,       0,  202605,       0,  0, 21,  0, }, // Europe_Further_Eastern
    {  109,  680,  202626,  202643,  202666, 17, 23, 23, }, // Europe_Western
    {  109,  695,  202689,  202707,  202731, 18, 24, 24, }, // Falkland
    {  109,  704,  202755,  202767,  202785, 12, 18, 18, }, // Fiji
    {  109,  709,       0,  202803,       0,  0, 17,  0, }, // French_Guiana
    {  109,  723,       0,  202820,       0,  0, 46,  0, }, // French_Southern
    {  109,  746,       0,  202866,       0,  0, 13,  0, }, // Galapagos
    {  109,  756,       0,  202879,       0,  0, 11,  0, }, // Gambier
    {  109,  764,  202890,  202902,  202920, 12, 18, 18, }, // Georgia
    {  109,  772,       0,  202938,       0,  0, 16,  0, }, // Gilbert_Islands
    {  109,  788,       0,  202954,       0,  0,  3,  0, }, // GMT
    {  109,  830,  202957,  202974,  202997, 17, 23, 23, }, // Greenland_Eastern
    {  109,  848,  203020,  203038,  203062, 18, 24, 24, }, // Greenland_Western
    {  109,  866,       0,  203086,       0,  0, 14,  0, }, // Guam
    {  109,  871,       0,  203100,       0,  0, 21,  0, }, // Gulf
    {  109,  876,       0,  202810,       0,  0, 10,  0, }, // Guyana
    {  109,  883,  203121,  203140,  203165, 19, 25, 25, }, // Hawaii_Aleutian
    {  109,  899,  203190,  203202,  203220, 12, 18, 18, }, // Hong_Kong
    {  109,  909,  203238,  203246,  203260,  8, 14, 14, }, // Hovd
    {  109,  914,       0,  203274,       0,  0, 16,  0, }, // India
    {  109,  920,       0,  203290,       0,  0, 23,  0, }, // Indian_Ocean
    {  109,  933,       0,  203313,       0,  0, 12,  0, }, // Indochina
    {  109,  943,       0,  203325,       0,  0, 21,  0, }, // Indonesia_Central
    {  109,  961,       0,  203346,       0,  0, 17,  0, }, // Indonesia_Eastern
    {  109,  979,       0,  203363,       0,  0, 18,  0, }, // Indonesia_Western
    {  109,  997,  203381,  203391,  203407, 10, 16, 16, }, // Iran
    {  109, 1008,  203423,  203434,  203451, 11, 17, 17, }, // Irkutsk
    {  109, 1016,  203468,  203480,  203498, 12, 18, 18, }, // Israel
    {  109, 1023,  203516,  203527,  203544, 11, 17, 17, }, // Japan
    {  109, 1029,  203561,  203589,  203623, 28, 34, 34, }, // Kamchatka
    {  109, 1047,       0,  203657,       0,  0, 16,  0, }, // Kazakhstan
    {  109, 1058,       0,  203673,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  109, 1077,       0,  203692,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  109, 1106,  203712,  203724,  203742, 12, 18, 18, }, // Korea
    {  109, 1112,       0,  203760,       0,  0, 10,  0, }, // Kosrae
    {  109, 1119,  203770,  203785,  203806, 15, 21, 21, }, // Krasnoyarsk
    {  109, 1151,       0,  203827,       0,  0, 13,  0, }, // Kyrgystan
    {  109, 1161,       0,  203840,       0,  0,  9,  0, }, // Lanka
    {  109, 1175,       0,  203849,       0,  0, 14,  0, }, // Line_Islands
    {  109, 1188,  203863,  203876,  203895, 13, 19, 19, }, // Lord_Howe
    {  109, 1198,  203914,  203923,  203938,  9, 15, 15, }, // Macau
    {  109, 1204,  203953,  203964,  203981, 11, 17, 17, }, // Magadan
    {  109, 1219,       0,  203998,       0,  0, 12,  0, }, // Malaysia
    {  109, 1228,       0,  204010,       0,  0, 14,  0, }, // Maldives
    {  109, 1237,       0,  204024,       0,  0, 13,  0, }, // Marquesas
    {  109, 1247,       0,  204037,       0,  0, 17,  0, }, // Marshall_Islands
    {  109, 1264,  204054,  204067,  204086, 13, 19, 19, }, // Mauritius
    {  109, 1274,       0,  204105,       0,  0, 10,  0, }, // Mawson
    {  109, 1281,  204115,  204136,  204163, 21, 27, 27, }, // Mexico_Pacific
    {  109, 1296,  204190,  204204,  204224, 14, 20, 20, }, // Mongolia
    {  109, 1305,  204244,  204254,  204270, 10, 16, 16, }, // Moscow
    {  109, 1312,       0,  204286,       0,  0, 11,  0, }, // Myanmar
    {  109, 1320,       0,  204297,       0,  0,  9,  0, }, // Nauru
    {  109, 1326,       0,  204306,       0,  0, 14,  0, }, // Nepal
    {  109, 1332,  204320,  204337,  204360, 17, 23, 23, }, // New_Caledonia
    {  109, 1346,  204383,  204399,  204421, 16, 22, 22, }, // New_Zealand
    {  109, 1358,  204443,  204461,  204485, 18, 24, 24, }, // Newfoundland
    {  109, 1371,       0,  204509,       0,  0,  8,  0, }, // Niue
    {  109, 1376,  204517,  204535,  204559, 18, 24, 24, }, // Norfolk
    {  109, 1384,  204583,  204606,  204635, 23, 29, 29, }, // Noronha
    {  109, 1392,       0,  204664,       0,  0, 18,  0, }, // North_Mariana
    {  109, 1406,  204682,  204697,  204718, 15, 21, 21, }, // Novosibirsk
    {  109, 1418,  204739,  204747,  204761,  8, 14, 14, }, // Omsk
    {  109, 1428,  204775,  204789,  204809, 14, 20, 20, }, // Pakistan
    {  109, 1437,       0,  204829,       0,  0,  9,  0, }, // Palau
    {  109, 1443,       0,  204838,       0,  0, 19,  0, }, // Papua_New_Guinea
    {  109, 1460,  204857,  204873,  204895, 16, 22, 22, }, // Paraguay
    {  109, 1469,  204917,  204931,  204951, 14, 20, 20, }, // Peru
    {  109, 1474,  204971,  204985,  205005, 14, 20, 20, }, // Philippines
    {  109, 1486,       0,  205025,       0,  0, 14,  0, }, // Phoenix_Islands
    {  109, 1502,  205039,  205068,  205103, 29, 35, 35, }, // Pierre_Miquelon
    {  109, 1518,       0,  205138,       0,  0, 12,  0, }, // Pitcairn
    {  109, 1527,       0,  205150,       0,  0, 10,  0, }, // Ponape
    {  109, 1534,       0,  205160,       0,  0, 13,  0, }, // Pyongyang
    {  109, 1544,  205173,  205186,  205205, 13, 19, 19, }, // Qyzylorda
    {  109, 1554,       0,  205224,       0,  0, 11,  0, }, // Reunion
    {  109, 1562,       0,  205235,       0,  0, 11,  0, }, // Rothera
    {  109, 1570,  205246,  205258,  205276, 12, 18, 18, }, // Sakhalin
    {  109, 1579,  205294,  205304,  205320, 10, 16, 16, }, // Samara
    {  109, 1596,  205336,  205348,  205366, 12, 18, 18, }, // Samoa
    {  109, 1602,       0,  205384,       0,  0, 15,  0, }, // Seychelles
    {  109, 1624,       0,  205399,       0,  0, 13,  0, }, // Singapore
    {  109, 1634,       0,  205412,       0,  0, 16,  0, }, // Solomon
    {  109, 1642,       0,  205428,       0,  0, 17,  0, }, // South_Georgia
    {  109, 1656,       0,  205445,       0,  0, 11,  0, }, // Suriname
    {  109, 1676,       0,  205456,       0,  0,  9,  0, }, // Syowa
    {  109, 1682,       0,  205465,       0,  0, 10,  0, }, // Tahiti
    {  109, 1689,  205475,  205485,  205501, 10, 16, 16, }, // Taipei
    {  109, 1696,       0,  205517,       0,  0, 14,  0, }, // Tajikistan
    {  109, 1724,       0,  205531,       0,  0, 11,  0, }, // Tokelau
    {  109, 1732,  205542,  205554,  205572, 12, 18, 18, }, // Tonga
    {  109, 1738,       0,  205590,       0,  0,  9,  0, }, // Truk
    {  109, 1750,  205599,  205612,  205631, 13, 19, 19, }, // Turkmenistan
    {  109, 1763,       0,  205650,       0,  0, 10,  0, }, // Tuvalu
    {  109, 1777,  205660,  205675,  205696, 15, 21, 21, }, // Uruguay
    {  109, 1792,  205717,  205729,  205747, 12, 18, 18, }, // Uzbekistan
    {  109, 1803,  205765,  205776,  205793, 11, 17, 17, }, // Vanuatu
    {  109, 1811,       0,  205810,       0,  0, 16,  0, }, // Venezuela
    {  109, 1821,  205826,  205841,  205862, 15, 21, 21, }, // Vladivostok
    {  109, 1833,  205883,  205896,  205915, 13, 19, 19, }, // Volgograd
    {  109, 1843,       0,  205934,       0,  0, 10,  0, }, // Vostok
    {  109, 1850,       0,  205944,       0,  0, 15,  0, }, // Wake
    {  109, 1855,       0,  205959,       0,  0, 20,  0, }, // Wallis
    {  109, 1862,  205979,  205990,  206007, 11, 17, 17, }, // Yakutsk
    {  109, 1870,  206024,  206041,  206064, 17, 23, 23, }, // Yekaterinburg
    {  109, 1892,       0,  206087,       0,  0,  9,  0, }, // Yukon
    {  112,  142,  206096,  206121,  206151, 25, 30, 32, }, // America_Central Dogri/Devanagari/India
    {  112,  158,  206183,  206208,  206238, 25, 30, 32, }, // America_Eastern
    {  112,  174,  206270,  206296,  206327, 26, 31, 33, }, // America_Mountain
    {  112,  191,  206360,  206386,  206417, 26, 31, 33, }, // America_Pacific
    {  112,  286,  206450,  206463,  206481, 13, 18, 20, }, // Atlantic
    {  112,  627,  206501,  206519,  206542, 18, 23, 30, }, // Europe_Central
    {  112,  642,  206572,  206590,  206613, 18, 23, 30, }, // Europe_Eastern
    {  112,  680,  206643,  206661,  206684, 18, 23, 30, }, // Europe_Western
    {  112,  788,       0,   91115,       0,  0, 17,  0, }, // GMT
    {  114,    0,  206714,  206723,  206741,  9, 18, 14, }, // Acre Dutch/Latin/Netherlands
    {  114,    5,       0,  206755,       0,  0, 14,  0, }, // Afghanistan
    {  114,   17,       0,  206769,       0,  0, 24,  0, }, // Africa_Central
    {  114,   32,       0,  206793,       0,  0, 20,  0, }, // Africa_Eastern
    {  114,   65,       0,  206813,       0,  0, 20,  0, }, // Africa_Southern
    {  114,   81,  206833,  206853,  206882, 20, 29, 25, }, // Africa_Western
    {  114,  107,  206907,  206918,  206938, 11, 20, 16, }, // Alaska
    {  114,  128,  206954,  206967,  206989, 13, 22, 18, }, // Almaty
    {  114,  135,  207007,  207019,  207040, 12, 21, 17, }, // Amazon
    {  114,  142,  207057,  207069,  207090, 12, 21, 17, }, // America_Central
    {  114,  158,  207107,  207119,  207140, 12, 21, 17, }, // America_Eastern
    {  114,  174,  207157,  207170,  207192, 13, 22, 18, }, // America_Mountain
    {  114,  191,  207210,  207222,  207243, 12, 21, 17, }, // America_Pacific
    {  114,  207,  207260,  207271,  207291, 11, 20, 16, }, // Anadyr
    {  114,  214,  207307,  207316,  207334,  9, 18, 14, }, // Apia
    {  114,  219,  207348,  207358,  207377, 10, 19, 15, }, // Aqtau
    {  114,  225,  207392,  207403,  207423, 11, 20, 16, }, // Aqtobe
    {  114,  232,  207439,  207453,  207476, 14, 23, 19, }, // Arabian
    {  114,  240,  207495,  207511,  207536, 16, 25, 21, }, // Argentina
    {  114,  250,  207557,  207578,  207608, 21, 30, 26, }, // Argentina_Western
    {  114,  268,  207634,  207647,  207669, 13, 22, 18, }, // Armenia
    {  114,  286,  207687,  207700,  207722, 13, 22, 18, }, // Atlantic
    {  114,  295,  207740,  207764,  207797, 24, 33, 29, }, // Australia_Central
    {  114,  313,  207826,  207861,  207905, 35, 44, 40, }, // Australia_CentralWestern
    {  114,  338,  207945,  207967,  207998, 22, 31, 27, }, // Australia_Eastern
    {  114,  356,  208025,  208047,  208078, 22, 31, 27, }, // Australia_Western
    {  114,  374,  208105,  208125,  208154, 20, 29, 25, }, // Azerbaijan
    {  114,  385,  208179,  208190,  208210, 11, 20, 16, }, // Azores
    {  114,  397,  208226,  208240,  208263, 14, 23, 19, }, // Bangladesh
    {  114,  415,       0,  208282,       0,  0, 14,  0, }, // Bhutan
    {  114,  422,       0,  208296,       0,  0, 16,  0, }, // Bolivia
    {  114,  437,  208312,  208329,  208355, 17, 26, 22, }, // Brasilia
    {  114,  454,       0,  208377,       0,  0, 13,  0, }, // Brunei
    {  114,  461,  208390,  208408,  208435, 18, 27, 23, }, // Cape_Verde
    {  114,  472,       0,  208458,       0,  0, 10,  0, }, // Casey
    {  114,  478,       0,  208468,       0,  0, 13,  0, }, // Chamorro
    {  114,  487,  208481,  208493,  208514, 12, 21, 17, }, // Chatham
    {  114,  495,  208531,  208545,  208568, 14, 23, 19, }, // Chile
    {  114,  501,  208587,  208599,  208620, 12, 21, 17, }, // China
    {  114,  507,       0,  208637,       0,  0, 22,  0, }, // Christmas
    {  114,  517,       0,  208659,       0,  0, 18,  0, }, // Cocos
    {  114,  523,  208677,  208694,  208720, 17, 26, 22, }, // Colombia
    {  114,  532,  208742,  208759,  208785, 17, 26, 28, }, // Cook
    {  114,  537,  208813,  208826,  208848, 13, 22, 18, }, // Cuba
    {  114,  548,       0,  208866,       0,  0, 10,  0, }, // Davis
    {  114,  564,       0,  208876,       0,  0, 21,  0, }, // DumontDUrville
    {  114,  601,       0,  208897,       0,  0, 18,  0, }, // East_Timor
    {  114,  612,  208915,  208932,  208958, 17, 26, 22, }, // Easter
    {  114,  619,       0,  208980,       0,  0, 17,  0, }, // Ecuador
    {  114,  627,  208997,  209017,  209046, 20, 29, 25, }, // Europe_Central
    {  114,  642,  209071,  209089,  209116, 18, 27, 23, }, // Europe_Eastern
    {  114,  657,       0,  209139,       0,  0, 30,  0, }, // Europe_Further_Eastern
    {  114,  680,  209169,  209187,  209214, 18, 27, 23, }, // Europe_Western
    {  114,  695,  209237,  209258,  209288, 21, 30, 26, }, // Falkland
    {  114,  704,  209314,  209327,  209349, 13, 22, 18, }, // Fiji
    {  114,  709,       0,  209367,       0,  0, 19,  0, }, // French_Guiana
    {  114,  723,       0,  209386,       0,  0, 38,  0, }, // French_Southern
    {  114,  746,       0,  209424,       0,  0, 22,  0, }, // Galapagos
    {  114,  756,       0,  209446,       0,  0, 20,  0, }, // Gambier
    {  114,  764,  209466,  209481,  209505, 15, 24, 20, }, // Georgia
    {  114,  772,       0,  209525,       0,  0, 20,  0, }, // Gilbert_Islands
    {  114,  788,       0,    2791,       0,  0, 19,  0, }, // GMT
    {  114,  830,  209545,  209566,  209596, 21, 30, 26, }, // Greenland_Eastern
    {  114,  848,  209622,  209643,  209673, 21, 30, 26, }, // Greenland_Western
    {  114,  866,       0,  209699,       0,  0, 21,  0, }, // Guam
    {  114,  871,       0,  209720,       0,  0, 18,  0, }, // Gulf
    {  114,  876,       0,  209373,       0,  0, 13,  0, }, // Guyana
    {  114,  883,  209738,  209761,  209793, 23, 32, 28, }, // Hawaii_Aleutian
    {  114,  899,  209821,  209836,  209860, 15, 24, 20, }, // Hong_Kong
    {  114,  909,  209880,  209889,  209907,  9, 18, 14, }, // Hovd
    {  114,  914,       0,  209921,       0,  0, 12,  0, }, // India
    {  114,  920,       0,  209933,       0,  0, 20,  0, }, // Indian_Ocean
    {  114,  933,       0,  209953,       0,  0, 16,  0, }, // Indochina
    {  114,  943,       0,  209969,       0,  0, 26,  0, }, // Indonesia_Central
    {  114,  961,       0,  209995,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  114,  979,       0,  210017,       0,  0, 22,  0, }, // Indonesia_Western
    {  114,  997,  210039,  210051,  210072, 12, 21, 17, }, // Iran
    {  114, 1008,  210089,  210102,  210124, 13, 22, 18, }, // Irkutsk
    {  114, 1016,  210142,  210158,  210183, 16, 25, 21, }, // Israel
    {  114, 1023,  210204,  210216,  210237, 12, 21, 17, }, // Japan
    {  114, 1029,  210254,  210284,  210323, 30, 39, 35, }, // Kamchatka
    {  114, 1047,       0,  210358,       0,  0, 13,  0, }, // Kazakhstan
    {  114, 1058,       0,  210371,       0,  0, 18,  0, }, // Kazakhstan_Eastern
    {  114, 1077,       0,  210389,       0,  0, 18,  0, }, // Kazakhstan_Western
    {  114, 1106,  210407,  210421,  210444, 14, 23, 19, }, // Korea
    {  114, 1112,       0,  210463,       0,  0, 13,  0, }, // Kosrae
    {  114, 1119,  210476,  210492,  210517, 16, 25, 21, }, // Krasnoyarsk
    {  114, 1151,       0,  210538,       0,  0, 16,  0, }, // Kyrgystan
    {  114, 1161,       0,  210554,       0,  0, 10,  0, }, // Lanka
    {  114, 1175,       0,  210564,       0,  0, 18,  0, }, // Line_Islands
    {  114, 1188,  210582,  210605,  210637, 23, 32, 28, }, // Lord_Howe
    {  114, 1198,  210665,  210677,  210698, 12, 21, 17, }, // Macau
    {  114, 1204,  210715,  210727,  210748, 12, 21, 17, }, // Magadan
    {  114, 1219,       0,  210765,       0,  0, 16,  0, }, // Malaysia
    {  114, 1228,       0,  210781,       0,  0, 16,  0, }, // Maldives
    {  114, 1237,       0,  210797,       0,  0, 22,  0, }, // Marquesas
    {  114, 1247,       0,  210819,       0,  0, 21,  0, }, // Marshall_Islands
    {  114, 1264,  210840,  210857,  210883, 17, 26, 22, }, // Mauritius
    {  114, 1274,       0,  210905,       0,  0, 11,  0, }, // Mawson
    {  114, 1281,  210916,  210939,  210971, 23, 32, 28, }, // Mexico_Pacific
    {  114, 1296,  210999,  211015,  211040, 16, 25, 21, }, // Mongolia
    {  114, 1305,  211061,  211072,  211092, 11, 20, 16, }, // Moscow
    {  114, 1312,       0,  211108,       0,  0, 15,  0, }, // Myanmar
    {  114, 1320,       0,  211123,       0,  0, 15,  0, }, // Nauru
    {  114, 1326,       0,  211138,       0,  0, 13,  0, }, // Nepal
    {  114, 1332,  211151,  211174,  211206, 23, 32, 28, }, // New_Caledonia
    {  114, 1346,  211234,  211254,  211283, 20, 29, 25, }, // New_Zealand
    {  114, 1358,  211308,  211325,  211351, 17, 26, 22, }, // Newfoundland
    {  114, 1371,       0,  211373,       0,  0, 11,  0, }, // Niue
    {  114, 1376,  211384,  211404,  211433, 20, 29, 25, }, // Norfolk
    {  114, 1384,  211458,  211482,  211515, 24, 33, 29, }, // Noronha
    {  114, 1392,       0,  211544,       0,  0, 25,  0, }, // North_Mariana
    {  114, 1406,  211569,  211585,  211610, 16, 25, 21, }, // Novosibirsk
    {  114, 1418,  211631,  211640,  211658,  9, 18, 14, }, // Omsk
    {  114, 1428,  211672,  211688,  211713, 16, 25, 21, }, // Pakistan
    {  114, 1437,       0,  211734,       0,  0, 12,  0, }, // Palau
    {  114, 1443,       0,  211746,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  114, 1460,  211772,  211790,  211817, 18, 27, 23, }, // Paraguay
    {  114, 1469,  211840,  211854,  211877, 14, 23, 19, }, // Peru
    {  114, 1474,  211896,  211911,  211935, 15, 24, 20, }, // Philippines
    {  114, 1486,       0,  211955,       0,  0, 20,  0, }, // Phoenix_Islands
    {  114, 1502,  211975,  212004,  212042, 29, 38, 34, }, // Pierre_Miquelon
    {  114, 1518,       0,  212076,       0,  0, 21,  0, }, // Pitcairn
    {  114, 1527,       0,  212097,       0,  0, 12,  0, }, // Ponape
    {  114, 1534,       0,  212109,       0,  0, 14,  0, }, // Pyongyang
    {  114, 1544,  212123,  212137,  212160, 14, 23, 19, }, // Qyzylorda
    {  114, 1554,       0,  212179,       0,  0, 14,  0, }, // Reunion
    {  114, 1562,       0,  212193,       0,  0, 12,  0, }, // Rothera
    {  114, 1570,  212205,  212218,  212240, 13, 22, 18, }, // Sakhalin
    {  114, 1579,  212258,  212269,  212289, 11, 20, 16, }, // Samara
    {  114, 1596,  212305,  212319,  212342, 14, 23, 19, }, // Samoa
    {  114, 1602,       0,  212361,       0,  0, 14,  0, }, // Seychelles
    {  114, 1624,       0,  212375,       0,  0, 25,  0, }, // Singapore
    {  114, 1634,       0,  212400,       0,  0, 21,  0, }, // Solomon
    {  114, 1642,       0,  212421,       0,  0, 20,  0, }, // South_Georgia
    {  114, 1656,       0,  212441,       0,  0, 15,  0, }, // Suriname
    {  114, 1676,       0,  212456,       0,  0, 10,  0, }, // Syowa
    {  114, 1682,       0,  212466,       0,  0, 16,  0, }, // Tahiti
    {  114, 1689,  212482,  212493,  212513, 11, 20, 16, }, // Taipei
    {  114, 1696,       0,  212529,       0,  0, 15,  0, }, // Tajikistan
    {  114, 1724,       0,  212544,       0,  0, 21,  0, }, // Tokelau
    {  114, 1732,  212565,  212579,  212602, 14, 23, 19, }, // Tonga
    {  114, 1738,       0,  212621,       0,  0, 12,  0, }, // Truk
    {  114, 1750,  212633,  212648,  212672, 15, 24, 20, }, // Turkmenistan
    {  114, 1763,       0,  212692,       0,  0, 16,  0, }, // Tuvalu
    {  114, 1777,  212708,  212725,  212751, 17, 26, 22, }, // Uruguay
    {  114, 1792,  212773,  212787,  212810, 14, 23, 19, }, // Uzbekistan
    {  114, 1803,  212829,  212846,  212872, 17, 26, 22, }, // Vanuatu
    {  114, 1811,       0,  212894,       0,  0, 17,  0, }, // Venezuela
    {  114, 1821,  212911,  212927,  212952, 16, 25, 21, }, // Vladivostok
    {  114, 1833,  212973,  212987,  213010, 14, 23, 19, }, // Volgograd
    {  114, 1843,       0,  213029,       0,  0, 11,  0, }, // Vostok
    {  114, 1850,       0,  213040,       0,  0, 18,  0, }, // Wake
    {  114, 1855,       0,  213058,       0,  0, 23,  0, }, // Wallis
    {  114, 1862,  213081,  213094,  213116, 13, 22, 18, }, // Yakutsk
    {  114, 1870,  213134,  213154,  213183, 20, 29, 25, }, // Yekaterinburg
    {  114, 1892,       0,  213208,       0,  0, 10,  0, }, // Yukon
    {  120, 1656,       0,  212441,       0,  0, 15,  0, }, // Suriname Dutch/Latin/Suriname
    {  121,    5,       0,  213218,       0,  0, 19,  0, }, // Afghanistan Dzongkha/Tibetan/Bhutan
    {  121,   17,       0,  213237,       0,  0, 26,  0, }, // Africa_Central
    {  121,   32,       0,  213263,       0,  0, 24,  0, }, // Africa_Eastern
    {  121,   65,       0,  213287,       0,  0, 25,  0, }, // Africa_Southern
    {  121,   81,  213312,  213337,  213369, 25, 32, 34, }, // Africa_Western
    {  121,  107,  213403,  213417,  213438, 14, 21, 23, }, // Alaska
    {  121,  135,  213461,  213477,  213500, 16, 23, 25, }, // Amazon
    {  121,  142,  213525,  213556,  213594, 31, 38, 40, }, // America_Central
    {  121,  158,  213634,  213663,  213699, 29, 36, 38, }, // America_Eastern
    {  121,  174,  213737,  213765,  213800, 28, 35, 37, }, // America_Mountain
    {  121,  191,  213837,  213867,  213904, 30, 37, 39, }, // America_Pacific
    {  121,  232,  213943,  213961,  213986, 18, 25, 23, }, // Arabian
    {  121,  240,  214009,  214027,  214052, 18, 25, 27, }, // Argentina
    {  121,  250,  214079,  214107,  214142, 28, 35, 37, }, // Argentina_Western
    {  121,  268,  214179,  214196,  214220, 17, 24, 26, }, // Armenia
    {  121,  286,  214246,  214264,  214289, 18, 25, 27, }, // Atlantic
    {  121,  295,  214316,  214348,  214387, 32, 39, 41, }, // Australia_Central
    {  121,  313,  214428,  214463,  214506, 35, 43, 45, }, // Australia_CentralWestern
    {  121,  338,  214551,  214587,  214630, 36, 43, 45, }, // Australia_Eastern
    {  121,  356,  214432,  214468,  214511, 31, 38, 40, }, // Australia_Western
    {  121,  374,  214675,  214695,  214722, 20, 27, 29, }, // Azerbaijan
    {  121,  385,  214751,  214765,  214786, 14, 21, 23, }, // Azores
    {  121,  397,  214809,  214825,  214848, 16, 23, 25, }, // Bangladesh
    {  121,  415,       0,  214873,       0,  0, 16,  0, }, // Bhutan
    {  121,  422,       0,  214889,       0,  0, 18,  0, }, // Bolivia
    {  121,  437,  214907,  214924,  214948, 17, 24, 26, }, // Brasilia
    {  121,  461,  214974,  214988,  215009, 14, 21, 23, }, // Cape_Verde
    {  121,  495,  215032,  215044,  215063, 12, 19, 21, }, // Chile
    {  121,  501,  215084,  215097,  215117, 13, 20, 22, }, // China
    {  121,  507,       0,  215139,       0,  0, 27,  0, }, // Christmas
    {  121,  523,  215166,  215185,  215211, 19, 26, 28, }, // Colombia
    {  121,  537,  215239,  215253,  215274, 14, 21, 23, }, // Cuba
    {  121,  612,  215297,  215321,  215352, 24, 31, 33, }, // Easter
    {  121,  619,       0,  215385,       0,  0, 15,  0, }, // Ecuador
    {  121,  627,  215400,  215427,  215461, 27, 34, 36, }, // Europe_Central
    {  121,  642,  215497,  215522,  215554, 25, 32, 34, }, // Europe_Eastern
    {  121,  680,  215588,  215614,  215647, 26, 33, 35, }, // Europe_Western
    {  121,  695,  215682,  215708,  215741, 26, 33, 35, }, // Falkland
    {  121,  709,       0,  215776,       0,  0, 20,  0, }, // French_Guiana
    {  121,  746,       0,  215796,       0,  0, 17,  0, }, // Galapagos
    {  121,  764,  215813,  215825,  215844, 12, 19, 21, }, // Georgia
    {  121,  788,       0,  215865,       0,  0, 27,  0, }, // GMT
    {  121,  830,  215892,  215918,  215951, 26, 33, 35, }, // Greenland_Eastern
    {  121,  848,  215986,  216013,  216047, 27, 34, 36, }, // Greenland_Western
    {  121,  871,       0,  216083,       0,  0, 12,  0, }, // Gulf
    {  121,  876,       0,  216095,       0,  0, 13,  0, }, // Guyana
    {  121,  883,  216108,  216133,  216165, 25, 32, 34, }, // Hawaii_Aleutian
    {  121,  914,       0,  216199,       0,  0, 13,  0, }, // India
    {  121,  920,       0,  216212,       0,  0, 25,  0, }, // Indian_Ocean
    {  121,  933,       0,  216237,       0,  0, 20,  0, }, // Indochina
    {  121,  943,       0,  216257,       0,  0, 32,  0, }, // Indonesia_Central
    {  121,  961,       0,  216289,       0,  0, 30,  0, }, // Indonesia_Eastern
    {  121,  979,       0,  216319,       0,  0, 31,  0, }, // Indonesia_Western
    {  121,  997,  216350,  216363,  216383, 13, 20, 22, }, // Iran
    {  121, 1008,  216405,  216419,  216440, 14, 21, 23, }, // Irkutsk
    {  121, 1016,  216463,  216477,  216498, 14, 21, 23, }, // Israel
    {  121, 1023,  216521,  216533,  216552, 12, 19, 21, }, // Japan
    {  121, 1106,  216573,  216587,  216608, 14, 21, 23, }, // Korea
    {  121, 1119,  216631,  216651,  216678, 20, 27, 29, }, // Krasnoyarsk
    {  121, 1204,  216707,  216722,  216744, 15, 22, 24, }, // Magadan
    {  121, 1228,       0,  216768,       0,  0, 15,  0, }, // Maldives
    {  121, 1264,  216783,  216799,  216822, 16, 23, 25, }, // Mauritius
    {  121, 1305,  216847,  216860,  216880, 13, 20, 22, }, // Moscow
    {  121, 1326,       0,  216902,       0,  0, 13,  0, }, // Nepal
    {  121, 1346,  216915,  216934,  216960, 19, 26, 28, }, // New_Zealand
    {  121, 1358,  216988,  217010,  217039, 22, 29, 31, }, // Newfoundland
    {  121, 1384,  217070,  217101,  217139, 31, 38, 40, }, // Noronha
    {  121, 1406,  217179,  217201,  217230, 22, 29, 31, }, // Novosibirsk
    {  121, 1418,  217261,  217274,  217294, 13, 20, 22, }, // Omsk
    {  121, 1428,  217316,  217333,  217357, 17, 24, 26, }, // Pakistan
    {  121, 1460,  217383,  217400,  217424, 17, 24, 26, }, // Paraguay
    {  121, 1469,  217450,  217461,  217479, 11, 18, 20, }, // Peru
    {  121, 1502,  217499,  217526,  217560, 27, 34, 36, }, // Pierre_Miquelon
    {  121, 1554,       0,  217596,       0,  0, 19,  0, }, // Reunion
    {  121, 1570,  217615,  217629,  217650, 14, 21, 23, }, // Sakhalin
    {  121, 1602,       0,  217673,       0,  0, 14,  0, }, // Seychelles
    {  121, 1656,       0,  217687,       0,  0, 16,  0, }, // Suriname
    {  121, 1777,  217703,  217723,  217750, 20, 27, 29, }, // Uruguay
    {  121, 1811,       0,  217779,       0,  0, 20,  0, }, // Venezuela
    {  121, 1821,  217799,  217821,  217850, 22, 29, 31, }, // Vladivostok
    {  121, 1833,  217881,  217901,  217928, 20, 27, 29, }, // Volgograd
    {  121, 1862,  217957,  217972,  217994, 15, 22, 24, }, // Yakutsk
    {  121, 1870,  218018,  218040,  218069, 22, 29, 31, }, // Yekaterinburg
    {  123,    0,       0,       9,      27,  9, 18, 16, }, // Acre English/Latin/United States
    {  123,    5,       0,      43,       0,  0, 16,  0, }, // Afghanistan
    {  123,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central
    {  123,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  123,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  123,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  123,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  123,  128,     235,     246,     266, 11, 20, 18, }, // Almaty
    {  123,  135,     284,     295,     315, 11, 20, 18, }, // Amazon
    {  123,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  123,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  123,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  123,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  123,  207,     552,     563,     583, 11, 20, 18, }, // Anadyr
    {  123,  214,     601,     611,     630, 10, 19, 19, }, // Apia
    {  123,  219,     649,     659,     678, 10, 19, 17, }, // Aqtau
    {  123,  225,     695,     706,     726, 11, 20, 18, }, // Aqtobe
    {  123,  232,     744,     756,     777, 12, 21, 21, }, // Arabian
    {  123,  240,     798,     812,     835, 14, 23, 21, }, // Argentina
    {  123,  250,     856,     878,     909, 22, 31, 29, }, // Argentina_Western
    {  123,  268,     938,     950,     971, 12, 21, 19, }, // Armenia
    {  123,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  123,  295,    1047,    1070,    1102, 23, 32, 32, }, // Australia_Central
    {  123,  313,    1134,    1165,    1205, 31, 40, 40, }, // Australia_CentralWestern
    {  123,  338,    1245,    1268,    1300, 23, 32, 32, }, // Australia_Eastern
    {  123,  356,    1332,    1355,    1387, 23, 32, 32, }, // Australia_Western
    {  123,  374,    1419,    1434,    1458, 15, 24, 22, }, // Azerbaijan
    {  123,  385,    1480,    1491,    1511, 11, 20, 18, }, // Azores
    {  123,  397,    1529,    1544,    1568, 15, 24, 22, }, // Bangladesh
    {  123,  415,       0,    1590,       0,  0, 11,  0, }, // Bhutan
    {  123,  422,       0,    1601,       0,  0, 12,  0, }, // Bolivia
    {  123,  437,    1613,    1626,    1648, 13, 22, 20, }, // Brasilia
    {  123,  454,       0,    1668,       0,  0, 11,  0, }, // Brunei
    {  123,  461,    1679,    1694,    1718, 15, 24, 22, }, // Cape_Verde
    {  123,  472,       0,    1740,       0,  0, 10,  0, }, // Casey
    {  123,  478,       0,    1750,       0,  0, 22,  0, }, // Chamorro
    {  123,  487,    1772,    1784,    1805, 12, 21, 21, }, // Chatham
    {  123,  495,    1826,    1836,    1855, 10, 19, 17, }, // Chile
    {  123,  501,    1872,    1882,    1901, 10, 19, 19, }, // China
    {  123,  507,       0,    1920,       0,  0, 21,  0, }, // Christmas
    {  123,  517,       0,    1941,       0,  0, 18,  0, }, // Cocos
    {  123,  523,    1959,    1972,    1994, 13, 22, 20, }, // Colombia
    {  123,  532,    2014,    2031,    2057, 17, 26, 24, }, // Cook
    {  123,  537,    2081,    2090,    2108,  9, 18, 18, }, // Cuba
    {  123,  548,       0,    2126,       0,  0, 10,  0, }, // Davis
    {  123,  564,       0,    2136,       0,  0, 21,  0, }, // DumontDUrville
    {  123,  601,       0,    2157,       0,  0, 16,  0, }, // East_Timor
    {  123,  612,    2173,    2191,    2218, 18, 27, 25, }, // Easter
    {  123,  619,       0,    2243,       0,  0, 12,  0, }, // Ecuador
    {  123,  627,    2255,    2276,    2306, 21, 30, 28, }, // Europe_Central
    {  123,  642,    2334,    2355,    2385, 21, 30, 28, }, // Europe_Eastern
    {  123,  657,       0,    2413,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {  123,  680,    2442,    2463,    2493, 21, 30, 28, }, // Europe_Western
    {  123,  695,    2521,    2542,    2572, 21, 30, 28, }, // Falkland
    {  123,  704,    2600,    2609,    2627,  9, 18, 16, }, // Fiji
    {  123,  709,       0,    2643,       0,  0, 18,  0, }, // French_Guiana
    {  123,  723,       0,    2661,       0,  0, 32,  0, }, // French_Southern
    {  123,  746,       0,    2693,       0,  0, 14,  0, }, // Galapagos
    {  123,  756,       0,    2707,       0,  0, 12,  0, }, // Gambier
    {  123,  764,    2719,    2731,    2752, 12, 21, 19, }, // Georgia
    {  123,  772,       0,    2771,       0,  0, 20,  0, }, // Gilbert_Islands
    {  123,  788,       0,    2791,       0,  0, 19,  0, }, // GMT
    {  123,  802,    2810,    2824,    2847, 14, 23, 21, }, // Greenland
    {  123,  830,    2868,    2887,    2915, 19, 28, 26, }, // Greenland_Eastern
    {  123,  848,    2941,    2960,    2988, 19, 28, 26, }, // Greenland_Western
    {  123,  866,       0,    3014,       0,  0, 18,  0, }, // Guam
    {  123,  871,       0,    3032,       0,  0, 18,  0, }, // Gulf
    {  123,  876,       0,    3050,       0,  0, 11,  0, }, // Guyana
    {  123,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  123,  899,    3139,    3153,    3176, 14, 23, 21, }, // Hong_Kong
    {  123,  909,    3197,    3206,    3224,  9, 18, 16, }, // Hovd
    {  123,  914,       0,    3240,       0,  0, 19,  0, }, // India
    {  123,  920,       0,    3259,       0,  0, 17,  0, }, // Indian_Ocean
    {  123,  933,       0,    3276,       0,  0, 14,  0, }, // Indochina
    {  123,  943,       0,    3290,       0,  0, 22,  0, }, // Indonesia_Central
    {  123,  961,       0,    3312,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  123,  979,       0,    3334,       0,  0, 22,  0, }, // Indonesia_Western
    {  123,  997,    3356,    3365,    3383,  9, 18, 18, }, // Iran
    {  123, 1008,    3401,    3413,    3434, 12, 21, 19, }, // Irkutsk
    {  123, 1016,    3453,    3464,    3484, 11, 20, 20, }, // Israel
    {  123, 1023,    3504,    3514,    3533, 10, 19, 19, }, // Japan
    {  123, 1029,    3552,    3566,    3589, 14, 23, 21, }, // Kamchatka
    {  123, 1047,       0,    3610,       0,  0, 15,  0, }, // Kazakhstan
    {  123, 1058,       0,    3625,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  123, 1077,       0,    3645,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  123, 1106,    3665,    3676,    3696, 11, 20, 20, }, // Korea
    {  123, 1112,       0,    3716,       0,  0, 11,  0, }, // Kosrae
    {  123, 1119,    3727,    3743,    3768, 16, 25, 23, }, // Krasnoyarsk
    {  123, 1151,       0,    3791,       0,  0, 15,  0, }, // Kyrgystan
    {  123, 1161,       0,    3806,       0,  0, 10,  0, }, // Lanka
    {  123, 1175,       0,    3816,       0,  0, 17,  0, }, // Line_Islands
    {  123, 1188,    3833,    3847,    3870, 14, 23, 23, }, // Lord_Howe
    {  123, 1198,    3893,    3903,    3922, 10, 19, 17, }, // Macau
    {  123, 1204,    3939,    3951,    3972, 12, 21, 19, }, // Magadan
    {  123, 1219,       0,    3991,       0,  0, 13,  0, }, // Malaysia
    {  123, 1228,       0,    4004,       0,  0, 13,  0, }, // Maldives
    {  123, 1237,       0,    4017,       0,  0, 14,  0, }, // Marquesas
    {  123, 1247,       0,    4031,       0,  0, 21,  0, }, // Marshall_Islands
    {  123, 1264,    4052,    4066,    4089, 14, 23, 21, }, // Mauritius
    {  123, 1274,       0,    4110,       0,  0, 11,  0, }, // Mawson
    {  123, 1281,    4121,    4141,    4170, 20, 29, 29, }, // Mexico_Pacific
    {  123, 1296,    4199,    4215,    4240, 16, 25, 23, }, // Mongolia
    {  123, 1305,    4263,    4274,    4294, 11, 20, 18, }, // Moscow
    {  123, 1312,       0,    4312,       0,  0, 12,  0, }, // Myanmar
    {  123, 1320,       0,    4324,       0,  0, 10,  0, }, // Nauru
    {  123, 1326,       0,    4334,       0,  0, 10,  0, }, // Nepal
    {  123, 1332,    4344,    4362,    4389, 18, 27, 25, }, // New_Caledonia
    {  123, 1346,    4414,    4430,    4455, 16, 25, 25, }, // New_Zealand
    {  123, 1358,    4480,    4497,    4523, 17, 26, 26, }, // Newfoundland
    {  123, 1371,       0,    4549,       0,  0,  9,  0, }, // Niue
    {  123, 1376,    4558,    4577,    4605, 19, 28, 28, }, // Norfolk
    {  123, 1384,    4633,    4657,    4690, 24, 33, 31, }, // Noronha
    {  123, 1392,       0,    4721,       0,  0, 29,  0, }, // North_Mariana
    {  123, 1406,    4750,    4766,    4791, 16, 25, 23, }, // Novosibirsk
    {  123, 1418,    4814,    4823,    4841,  9, 18, 16, }, // Omsk
    {  123, 1428,    4857,    4870,    4892, 13, 22, 20, }, // Pakistan
    {  123, 1437,       0,    4912,       0,  0, 10,  0, }, // Palau
    {  123, 1443,       0,    4922,       0,  0, 21,  0, }, // Papua_New_Guinea
    {  123, 1460,    4943,    4956,    4978, 13, 22, 20, }, // Paraguay
    {  123, 1469,    4998,    5007,    5025,  9, 18, 16, }, // Peru
    {  123, 1474,    5041,    5056,    5080, 15, 24, 22, }, // Philippines
    {  123, 1486,       0,    5102,       0,  0, 20,  0, }, // Phoenix_Islands
    {  123, 1502,    5122,    5148,    5183, 26, 35, 35, }, // Pierre_Miquelon
    {  123, 1518,       0,    5218,       0,  0, 13,  0, }, // Pitcairn
    {  123, 1527,       0,    5231,       0,  0, 12,  0, }, // Ponape
    {  123, 1534,       0,    5243,       0,  0, 16,  0, }, // Pyongyang
    {  123, 1544,    5259,    5273,    5296, 14, 23, 21, }, // Qyzylorda
    {  123, 1554,       0,    5317,       0,  0, 12,  0, }, // Reunion
    {  123, 1562,       0,    5329,       0,  0, 12,  0, }, // Rothera
    {  123, 1570,    5341,    5354,    5376, 13, 22, 20, }, // Sakhalin
    {  123, 1579,    5396,    5407,    5427, 11, 20, 18, }, // Samara
    {  123, 1596,    5445,    5464,    5492, 19, 28, 28, }, // Samoa
    {  123, 1602,       0,    5520,       0,  0, 15,  0, }, // Seychelles
    {  123, 1624,       0,    5535,       0,  0, 23,  0, }, // Singapore
    {  123, 1634,       0,    5558,       0,  0, 20,  0, }, // Solomon
    {  123, 1642,       0,    5578,       0,  0, 18,  0, }, // South_Georgia
    {  123, 1656,       0,    5596,       0,  0, 13,  0, }, // Suriname
    {  123, 1676,       0,    5609,       0,  0, 10,  0, }, // Syowa
    {  123, 1682,       0,    5619,       0,  0, 11,  0, }, // Tahiti
    {  123, 1689,    5630,    5641,    5661, 11, 20, 20, }, // Taipei
    {  123, 1696,       0,    5681,       0,  0, 15,  0, }, // Tajikistan
    {  123, 1724,       0,    5696,       0,  0, 12,  0, }, // Tokelau
    {  123, 1732,    5708,    5718,    5737, 10, 19, 17, }, // Tonga
    {  123, 1738,       0,    5754,       0,  0, 10,  0, }, // Truk
    {  123, 1750,    5764,    5781,    5807, 17, 26, 24, }, // Turkmenistan
    {  123, 1763,       0,    5831,       0,  0, 11,  0, }, // Tuvalu
    {  123, 1777,    5842,    5854,    5875, 12, 21, 19, }, // Uruguay
    {  123, 1792,    5894,    5909,    5933, 15, 24, 22, }, // Uzbekistan
    {  123, 1803,    5955,    5967,    5988, 12, 21, 19, }, // Vanuatu
    {  123, 1811,       0,    6007,       0,  0, 14,  0, }, // Venezuela
    {  123, 1821,    6021,    6037,    6062, 16, 25, 23, }, // Vladivostok
    {  123, 1833,    6085,    6099,    6122, 14, 23, 21, }, // Volgograd
    {  123, 1843,       0,    6143,       0,  0, 11,  0, }, // Vostok
    {  123, 1850,       0,    6154,       0,  0, 16,  0, }, // Wake
    {  123, 1855,       0,    6170,       0,  0, 20,  0, }, // Wallis
    {  123, 1862,    6190,    6202,    6223, 12, 21, 19, }, // Yakutsk
    {  123, 1870,    6242,    6260,    6287, 18, 27, 25, }, // Yekaterinburg
    {  123, 1892,       0,    6312,       0,  0, 10,  0, }, // Yukon
    {  124,  107,  218100,  218119,  218155, 19, 36, 30, }, // Alaska English/Deseret/United States
    {  124,  142,  218185,  218206,  218244, 21, 38, 32, }, // America_Central
    {  124,  158,  218276,  218295,  218331, 19, 36, 30, }, // America_Eastern
    {  124,  174,  218361,  218380,  218416, 19, 36, 30, }, // America_Mountain
    {  124,  191,  218446,  218467,  218505, 21, 38, 32, }, // America_Pacific
    {  124,  286,  218537,  218560,  218600, 23, 40, 34, }, // Atlantic
    {  124,  899,  218634,  218654,  218691, 20, 37, 31, }, // Hong_Kong
    {  124, 1358,  218722,  218749,  218793, 27, 44, 38, }, // Newfoundland
    {  126,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Anguilla
    {  126,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  126,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  126,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  126,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  126,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  126,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  126, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  127,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Antigua and Barbuda
    {  127,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  127,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  127,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  127,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  127,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  127,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  127, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  128,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Australia
    {  128,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  128,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  128,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  128,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  128,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  128,  295,    1047,    1070,    1102, 23, 32, 32, }, // Australia_Central
    {  128,  313,    1134,    1165,    1205, 31, 40, 40, }, // Australia_CentralWestern
    {  128,  338,    1245,    1268,    1300, 23, 32, 32, }, // Australia_Eastern
    {  128,  356,    1332,    1355,    1387, 23, 32, 32, }, // Australia_Western
    {  128,  501,    1872,    1882,  218924, 10, 19, 17, }, // China
    {  128,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  128, 1023,    3504,    3514,  218941, 10, 19, 17, }, // Japan
    {  128, 1106,    3665,    3676,  218958, 11, 20, 18, }, // Korea
    {  128, 1188,    3833,    3847,    3870, 14, 23, 23, }, // Lord_Howe
    {  128, 1346,    4414,    4430,    4455, 16, 25, 25, }, // New_Zealand
    {  128, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  128, 1596,    5445,    5464,  218976, 19, 28, 17, }, // Samoa
    {  128, 1689,    5630,    5641,  218993, 11, 20, 18, }, // Taipei
    {  129,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Austria
    {  129,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  129,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  129,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  129,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  129,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  129,  627,    2255,    2276,    2306, 21, 30, 28, }, // Europe_Central
    {  129,  642,    2334,    2355,    2385, 21, 30, 28, }, // Europe_Eastern
    {  129,  680,    2442,    2463,    2493, 21, 30, 28, }, // Europe_Western
    {  129,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  129, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  130,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Bahamas
    {  130,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  130,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  130,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  130,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  130,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  130,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  130, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  131,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Barbados
    {  131,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  131,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  131,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  131,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  131,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  131,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  131, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  132,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Belgium
    {  132,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  132,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  132,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  132,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  132,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  132,  627,    2255,    2276,    2306, 21, 30, 28, }, // Europe_Central
    {  132,  642,    2334,    2355,    2385, 21, 30, 28, }, // Europe_Eastern
    {  132,  680,    2442,    2463,    2493, 21, 30, 28, }, // Europe_Western
    {  132,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  132, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  133,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Belize
    {  133,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  133,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  133,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  133,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  133,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  133,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  133, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  134,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Bermuda
    {  134,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  134,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  134,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  134,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  134,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  134,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  134, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  135,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Botswana
    {  135,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  135,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  135,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  135,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  135,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  135,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  135,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  135,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  135,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  135,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  135, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  136,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/British Indian Ocean Territory
    {  136,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  136,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  136,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  136,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  136,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  136,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  136, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  137,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/British Virgin Islands
    {  137,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  137,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  137,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  137,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  137,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  137,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  137, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  139,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Cameroon
    {  139,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  139,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  139,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  139,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  139,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  139,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  139,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  139,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  139,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  139,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  139, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  140,    5,       0,      43,       0,  0, 16,  0, }, // Afghanistan English/Latin/Canada
    {  140,  107,     184,     195,  219011, 11, 20, 27, }, // Alaska
    {  140,  142,     333,     345,  219038, 12, 21, 28, }, // America_Central
    {  140,  158,     387,     399,  219066, 12, 21, 28, }, // America_Eastern
    {  140,  240,     798,     812,     835, 14, 23, 21, }, // Argentina
    {  140,  313,    1134,    1165,    1205, 31, 40, 40, }, // Australia_CentralWestern
    {  140,  338,    1245,    1268,    1300, 23, 32, 32, }, // Australia_Eastern
    {  140,  356,    1332,    1355,    1387, 23, 32, 32, }, // Australia_Western
    {  140,  397,    1529,    1544,    1568, 15, 24, 22, }, // Bangladesh
    {  140,  415,       0,    1590,       0,  0, 11,  0, }, // Bhutan
    {  140,  437,    1613,    1626,    1648, 13, 22, 20, }, // Brasilia
    {  140,  454,       0,    1668,       0,  0, 11,  0, }, // Brunei
    {  140,  487,    1772,    1784,    1805, 12, 21, 21, }, // Chatham
    {  140,  507,       0,    1920,       0,  0, 21,  0, }, // Christmas
    {  140,  517,       0,    1941,       0,  0, 18,  0, }, // Cocos
    {  140,  523,    1959,    1972,    1994, 13, 22, 20, }, // Colombia
    {  140,  601,       0,    2157,       0,  0, 16,  0, }, // East_Timor
    {  140,  612,    2173,    2191,    2218, 18, 27, 25, }, // Easter
    {  140,  619,       0,    2243,       0,  0, 12,  0, }, // Ecuador
    {  140,  695,    2521,    2542,    2572, 21, 30, 28, }, // Falkland
    {  140,  723,       0,  219094,       0,  0, 34,  0, }, // French_Southern
    {  140,  746,       0,    2693,       0,  0, 14,  0, }, // Galapagos
    {  140,  830,    2868,    2887,    2915, 19, 28, 26, }, // Greenland_Eastern
    {  140,  871,       0,    3032,       0,  0, 18,  0, }, // Gulf
    {  140,  876,       0,    3050,       0,  0, 11,  0, }, // Guyana
    {  140,  914,       0,    3240,       0,  0, 19,  0, }, // India
    {  140,  933,       0,    3276,       0,  0, 14,  0, }, // Indochina
    {  140,  943,       0,    3290,       0,  0, 22,  0, }, // Indonesia_Central
    {  140,  961,       0,    3312,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  140,  979,       0,    3334,       0,  0, 22,  0, }, // Indonesia_Western
    {  140,  997,    3356,    3365,  219128,  9, 18, 25, }, // Iran
    {  140, 1219,       0,    3991,       0,  0, 13,  0, }, // Malaysia
    {  140, 1228,       0,    4004,       0,  0, 13,  0, }, // Maldives
    {  140, 1326,       0,    4334,       0,  0, 10,  0, }, // Nepal
    {  140, 1358,    4480,    4497,    4523, 17, 26, 26, }, // Newfoundland
    {  140, 1384,    4633,    4657,    4690, 24, 33, 31, }, // Noronha
    {  140, 1428,    4857,    4870,    4892, 13, 22, 20, }, // Pakistan
    {  140, 1460,    4943,    4956,    4978, 13, 22, 20, }, // Paraguay
    {  140, 1469,    4998,    5007,    5025,  9, 18, 16, }, // Peru
    {  140, 1502,  219153,  219182,  219220, 29, 38, 45, }, // Pierre_Miquelon
    {  140, 1596,    5445,    5464,  219265, 19, 28, 11, }, // Samoa
    {  140, 1777,    5842,    5854,    5875, 12, 21, 19, }, // Uruguay
    {  140, 1811,       0,    6007,       0,  0, 14,  0, }, // Venezuela
    {  140, 1855,       0,  219276,       0,  0, 22,  0, }, // Wallis
    {  141,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Cayman Islands
    {  141,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  141,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  141,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  141,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  141,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  141,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  141, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  142,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Christmas Island
    {  142,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  142,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  142,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  142,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  142,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  142,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  142, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  143,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Cocos Islands
    {  143,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  143,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  143,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  143,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  143,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  143,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  143, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  144,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Cook Islands
    {  144,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  144,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  144,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  144,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  144,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  144,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  144, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  145,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Cyprus
    {  145,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  145,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  145,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  145,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  145,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  145,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  145, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  146,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Czechia
    {  146,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  146,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  146,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  146,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  146,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  146,  627,    2255,    2276,    2306, 21, 30, 28, }, // Europe_Central
    {  146,  642,    2334,    2355,    2385, 21, 30, 28, }, // Europe_Eastern
    {  146,  680,    2442,    2463,    2493, 21, 30, 28, }, // Europe_Western
    {  146,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  146, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  147,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Denmark
    {  147,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  147,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  147,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  147,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  147,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  147,  627,    2255,    2276,    2306, 21, 30, 28, }, // Europe_Central
    {  147,  642,    2334,    2355,    2385, 21, 30, 28, }, // Europe_Eastern
    {  147,  680,    2442,    2463,    2493, 21, 30, 28, }, // Europe_Western
    {  147,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  147, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  148,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Diego Garcia
    {  148,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  148,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  148,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  148,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  148,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  148,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  148, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  149,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Dominica
    {  149,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  149,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  149,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  149,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  149,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  149,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  149, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  150,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Eritrea
    {  150,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  150,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  150,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  150,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  150,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  150,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  150,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  150,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  150,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  150,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  150, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  151,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Eswatini
    {  151,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  151,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  151,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  151,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  151,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  151,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  151,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  151,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  151,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  151,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  151, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  152,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Europe
    {  152,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  152,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  152,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  152,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  152,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  152,  627,    2255,    2276,    2306, 21, 30, 28, }, // Europe_Central
    {  152,  642,    2334,    2355,    2385, 21, 30, 28, }, // Europe_Eastern
    {  152,  680,    2442,    2463,    2493, 21, 30, 28, }, // Europe_Western
    {  152,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  152, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  153,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Falkland Islands
    {  153,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  153,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  153,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  153,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  153,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  153,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  153, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  154,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Fiji
    {  154,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  154,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  154,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  154,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  154,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  154,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  154, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  155,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Finland
    {  155,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  155,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  155,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  155,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  155,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  155,  627,    2255,    2276,    2306, 21, 30, 28, }, // Europe_Central
    {  155,  642,    2334,    2355,    2385, 21, 30, 28, }, // Europe_Eastern
    {  155,  680,    2442,    2463,    2493, 21, 30, 28, }, // Europe_Western
    {  155,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  155, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  156,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/France
    {  156,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  156,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  156,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  156,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  156,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  156,  627,    2255,    2276,    2306, 21, 30, 28, }, // Europe_Central
    {  156,  642,    2334,    2355,    2385, 21, 30, 28, }, // Europe_Eastern
    {  156,  680,    2442,    2463,    2493, 21, 30, 28, }, // Europe_Western
    {  156,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  156, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  157,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Gambia
    {  157,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  157,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  157,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  157,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  157,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  157,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  157,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  157,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  157,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  157,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  157, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  158,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Germany
    {  158,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  158,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  158,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  158,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  158,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  158,  627,    2255,    2276,    2306, 21, 30, 28, }, // Europe_Central
    {  158,  642,    2334,    2355,    2385, 21, 30, 28, }, // Europe_Eastern
    {  158,  680,    2442,    2463,    2493, 21, 30, 28, }, // Europe_Western
    {  158,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  158, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  159,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Ghana
    {  159,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  159,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  159,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  159,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  159,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  159,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  159,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  159,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  159,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  159,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  159, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  160,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Gibraltar
    {  160,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  160,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  160,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  160,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  160,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  160,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  160, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  161,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Grenada
    {  161,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  161,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  161,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  161,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  161,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  161,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  161, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  162,  478,       0,    1750,       0,  0, 22,  0, }, // Chamorro English/Latin/Guam
    {  163,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Guernsey
    {  163,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  163,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  163,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  163,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  163,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  163,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  163, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  164,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Guyana
    {  164,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  164,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  164,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  164,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  164,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  164,  876,       0,    3050,       0,  0, 11,  0, }, // Guyana
    {  164,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  164, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  165,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Hong Kong
    {  165,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  165,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  165,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  165,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  165,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  165,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  165,  899,    3139,    3153,    3176, 14, 23, 21, }, // Hong_Kong
    {  165, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  166,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Hungary
    {  166,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  166,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  166,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  166,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  166,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  166,  627,    2255,    2276,    2306, 21, 30, 28, }, // Europe_Central
    {  166,  642,    2334,    2355,    2385, 21, 30, 28, }, // Europe_Eastern
    {  166,  680,    2442,    2463,    2493, 21, 30, 28, }, // Europe_Western
    {  166,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  166, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  167,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/India
    {  167,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  167,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  167,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  167,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  167,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  167,  871,       0,    3032,       0,  0, 18,  0, }, // Gulf
    {  167,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  167,  909,  219298,  219308,  219327, 10, 19, 17, }, // Hovd
    {  167,  914,       0,    3240,       0,  0, 19,  0, }, // India
    {  167, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  168,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Indonesia
    {  168,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  168,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  168,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  168,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  168,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  168,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  168,  943,       0,    3290,       0,  0, 22,  0, }, // Indonesia_Central
    {  168,  961,       0,    3312,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  168,  979,       0,    3334,       0,  0, 22,  0, }, // Indonesia_Western
    {  168, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  169,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Ireland
    {  169,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  169,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  169,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  169,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  169,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  169,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  169, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  170,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Isle of Man
    {  170,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  170,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  170,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  170,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  170,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  170,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  170, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  171,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Israel
    {  171,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  171,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  171,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  171,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  171,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  171,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  171, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  172,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Italy
    {  172,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  172,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  172,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  172,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  172,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  172,  627,    2255,    2276,    2306, 21, 30, 28, }, // Europe_Central
    {  172,  642,    2334,    2355,    2385, 21, 30, 28, }, // Europe_Eastern
    {  172,  680,    2442,    2463,    2493, 21, 30, 28, }, // Europe_Western
    {  172,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  172, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  173,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Jamaica
    {  173,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  173,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  173,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  173,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  173,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  173,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  173, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  174,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Jersey
    {  174,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  174,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  174,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  174,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  174,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  174,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  174, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  175,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Kenya
    {  175,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  175,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  175,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  175,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  175,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  175,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  175,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  175,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  175,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  175,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  175, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  176,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Kiribati
    {  176,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  176,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  176,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  176,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  176,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  176,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  176, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  177,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Lesotho
    {  177,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  177,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  177,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  177,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  177,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  177,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  177,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  177,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  177,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  177,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  177, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  178,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Liberia
    {  178,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  178,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  178,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  178,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  178,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  178,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  178,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  178,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  178,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  178,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  178, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  179,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Macao
    {  179,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  179,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  179,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  179,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  179,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  179,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  179,  899,    3139,    3153,    3176, 14, 23, 21, }, // Hong_Kong
    {  179, 1198,    3893,    3903,    3922, 10, 19, 17, }, // Macau
    {  179, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  180,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Madagascar
    {  180,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  180,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  180,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  180,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  180,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  180,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  180,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  180,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  180,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  180,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  180, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  181,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Malawi
    {  181,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  181,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  181,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  181,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  181,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  181,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  181,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  181,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  181,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  181,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  181, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  182,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Malaysia
    {  182,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  182,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  182,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  182,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  182,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  182,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  182, 1219,       0,    3991,       0,  0, 13,  0, }, // Malaysia
    {  182, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  183,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Maldives
    {  183,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  183,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  183,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  183,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  183,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  183,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  183, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  184,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Malta
    {  184,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  184,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  184,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  184,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  184,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  184,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  184, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  185,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Marshall Islands
    {  185,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  185,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  185,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  185,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  185,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  185,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  186,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Mauritius
    {  186,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  186,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  186,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  186,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  186,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  186,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  186,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  186,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  186,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  186,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  186, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  187,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Micronesia
    {  187,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  187,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  187,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  187,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  187,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  187,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  187, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  188,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Montserrat
    {  188,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  188,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  188,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  188,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  188,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  188,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  188, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  189,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Namibia
    {  189,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  189,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  189,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  189,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  189,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  189,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  189,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  189,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  189,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  189,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  189, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  190,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Nauru
    {  190,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  190,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  190,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  190,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  190,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  190,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  190, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  191,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Netherlands
    {  191,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  191,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  191,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  191,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  191,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  191,  627,    2255,    2276,    2306, 21, 30, 28, }, // Europe_Central
    {  191,  642,    2334,    2355,    2385, 21, 30, 28, }, // Europe_Eastern
    {  191,  680,    2442,    2463,    2493, 21, 30, 28, }, // Europe_Western
    {  191,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  191, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  192,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/New Zealand
    {  192,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  192,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  192,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  192,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  192,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  192,  295,    1047,    1070,    1102, 23, 32, 32, }, // Australia_Central
    {  192,  313,    1134,    1165,    1205, 31, 40, 40, }, // Australia_CentralWestern
    {  192,  338,    1245,    1268,    1300, 23, 32, 32, }, // Australia_Eastern
    {  192,  356,    1332,    1355,    1387, 23, 32, 32, }, // Australia_Western
    {  192,  487,    1772,    1784,    1805, 12, 21, 21, }, // Chatham
    {  192,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  192, 1188,    3833,    3847,    3870, 14, 23, 23, }, // Lord_Howe
    {  192, 1346,    4414,    4430,    4455, 16, 25, 25, }, // New_Zealand
    {  192, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  193,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Nigeria
    {  193,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  193,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  193,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  193,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  193,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  193,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  193,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  193,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  193,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  193,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  193, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  194,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Niue
    {  194,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  194,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  194,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  194,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  194,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  194,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  194, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  195,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Norfolk Island
    {  195,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  195,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  195,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  195,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  195,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  195,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  195, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  196,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Northern Mariana Islands
    {  196,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  196,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  196,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  196,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  196,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  196,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  197,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Norway
    {  197,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  197,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  197,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  197,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  197,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  197,  627,    2255,    2276,    2306, 21, 30, 28, }, // Europe_Central
    {  197,  642,    2334,    2355,    2385, 21, 30, 28, }, // Europe_Eastern
    {  197,  680,    2442,    2463,    2493, 21, 30, 28, }, // Europe_Western
    {  197,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  197, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  198,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Pakistan
    {  198,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  198,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  198,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  198,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  198,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  198,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  198, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  199,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Palau
    {  199,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  199,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  199,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  199,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  199,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  199,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  199, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  200,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Papua New Guinea
    {  200,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  200,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  200,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  200,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  200,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  200,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  200, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  202,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Pitcairn
    {  202,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  202,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  202,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  202,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  202,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  202,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  202, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  203,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Poland
    {  203,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  203,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  203,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  203,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  203,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  203,  627,    2255,    2276,    2306, 21, 30, 28, }, // Europe_Central
    {  203,  642,    2334,    2355,    2385, 21, 30, 28, }, // Europe_Eastern
    {  203,  680,    2442,    2463,    2493, 21, 30, 28, }, // Europe_Western
    {  203,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  203, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  204,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Portugal
    {  204,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  204,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  204,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  204,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  204,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  204,  627,    2255,    2276,    2306, 21, 30, 28, }, // Europe_Central
    {  204,  642,    2334,    2355,    2385, 21, 30, 28, }, // Europe_Eastern
    {  204,  680,    2442,    2463,    2493, 21, 30, 28, }, // Europe_Western
    {  204,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  204, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  206,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Romania
    {  206,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  206,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  206,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  206,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  206,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  206,  627,    2255,    2276,    2306, 21, 30, 28, }, // Europe_Central
    {  206,  642,    2334,    2355,    2385, 21, 30, 28, }, // Europe_Eastern
    {  206,  680,    2442,    2463,    2493, 21, 30, 28, }, // Europe_Western
    {  206,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  206, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  207,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Rwanda
    {  207,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  207,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  207,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  207,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  207,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  207,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  207,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  207,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  207,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  207,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  207, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  208,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Saint Helena
    {  208,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  208,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  208,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  208,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  208,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  208,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  208, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  209,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Saint Kitts and Nevis
    {  209,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  209,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  209,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  209,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  209,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  209,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  209, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  210,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Saint Lucia
    {  210,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  210,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  210,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  210,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  210,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  210,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  210, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  211,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Saint Vincent and Grenadines
    {  211,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  211,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  211,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  211,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  211,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  211,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  211, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  212,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Samoa
    {  212,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  212,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  212,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  212,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  212,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  212,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  212, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  213,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Seychelles
    {  213,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  213,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  213,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  213,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  213,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  213,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  213, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  214,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Sierra Leone
    {  214,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  214,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  214,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  214,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  214,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  214,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  214,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  214,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  214,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  214,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  214, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  215,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Singapore
    {  215,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  215,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  215,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  215,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  215,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  215,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  215, 1219,       0,    3991,       0,  0, 13,  0, }, // Malaysia
    {  215, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  215, 1624,       0,    5535,       0,  0, 23,  0, }, // Singapore
    {  216,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Sint Maarten
    {  216,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  216,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  216,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  216,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  216,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  216,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  216, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  217,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Slovakia
    {  217,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  217,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  217,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  217,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  217,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  217,  627,    2255,    2276,    2306, 21, 30, 28, }, // Europe_Central
    {  217,  642,    2334,    2355,    2385, 21, 30, 28, }, // Europe_Eastern
    {  217,  680,    2442,    2463,    2493, 21, 30, 28, }, // Europe_Western
    {  217,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  217, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  218,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Slovenia
    {  218,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  218,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  218,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  218,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  218,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  218,  627,    2255,    2276,    2306, 21, 30, 28, }, // Europe_Central
    {  218,  642,    2334,    2355,    2385, 21, 30, 28, }, // Europe_Eastern
    {  218,  680,    2442,    2463,    2493, 21, 30, 28, }, // Europe_Western
    {  218,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  218, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  219,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Solomon Islands
    {  219,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  219,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  219,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  219,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  219,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  219,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  219, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  220,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/South Africa
    {  220,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  220,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  220,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  220,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  220,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  220,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  220,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  220,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  220,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  220,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  220, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  221,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/South Georgia and South Sandwich Islands
    {  221,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  221,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  221,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  221,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  221,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  221,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  221, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  222,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/South Sudan
    {  222,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  222,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  222,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  222,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  222,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  222,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  222,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  222,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  222,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  222,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  222, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  223,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Spain
    {  223,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  223,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  223,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  223,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  223,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  223,  627,    2255,    2276,    2306, 21, 30, 28, }, // Europe_Central
    {  223,  642,    2334,    2355,    2385, 21, 30, 28, }, // Europe_Eastern
    {  223,  680,    2442,    2463,    2493, 21, 30, 28, }, // Europe_Western
    {  223,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  223, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  224,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Sudan
    {  224,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  224,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  224,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  224,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  224,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  224,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  224,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  224,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  224,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  224,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  224, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  225,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Sweden
    {  225,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  225,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  225,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  225,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  225,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  225,  627,    2255,    2276,    2306, 21, 30, 28, }, // Europe_Central
    {  225,  642,    2334,    2355,    2385, 21, 30, 28, }, // Europe_Eastern
    {  225,  680,    2442,    2463,    2493, 21, 30, 28, }, // Europe_Western
    {  225,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  225, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  226,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Switzerland
    {  226,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  226,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  226,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  226,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  226,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  226,  627,    2255,    2276,    2306, 21, 30, 28, }, // Europe_Central
    {  226,  642,    2334,    2355,    2385, 21, 30, 28, }, // Europe_Eastern
    {  226,  680,    2442,    2463,    2493, 21, 30, 28, }, // Europe_Western
    {  226,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  226, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  227,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Tanzania
    {  227,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  227,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  227,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  227,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  227,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  227,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  227,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  227,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  227,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  227,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  227, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  228,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Tokelau
    {  228,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  228,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  228,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  228,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  228,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  228,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  228, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  229,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Tonga
    {  229,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  229,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  229,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  229,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  229,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  229,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  229, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  230,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Trinidad and Tobago
    {  230,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  230,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  230,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  230,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  230,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  230,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  230, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  231,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Turks and Caicos Islands
    {  231,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  231,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  231,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  231,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  231,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  231,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  231, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  232,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Tuvalu
    {  232,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  232,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  232,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  232,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  232,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  232,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  232, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  233,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Uganda
    {  233,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  233,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  233,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  233,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  233,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  233,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  233,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  233,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  233,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  233,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  233, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  234,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/United Arab Emirates
    {  234,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  234,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  234,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  234,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  234,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  234,  871,       0,    3032,       0,  0, 18,  0, }, // Gulf
    {  234,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  235,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/United Kingdom
    {  235,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  235,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  235,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  235,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  235,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  235,  627,    2255,    2276,    2306, 21, 30, 28, }, // Europe_Central
    {  235,  642,    2334,    2355,    2385, 21, 30, 28, }, // Europe_Eastern
    {  235,  680,    2442,    2463,    2493, 21, 30, 28, }, // Europe_Western
    {  235,  871,       0,    3032,       0,  0, 18,  0, }, // Gulf
    {  235,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  235, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  238,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Vanuatu
    {  238,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  238,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  238,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  238,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  238,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  238,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  238, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  239,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/world
    {  239,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  239,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  239,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  239,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  239,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  239,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  239, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  240,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Zambia
    {  240,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  240,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  240,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  240,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  240,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  240,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  240,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  240,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  240,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  240,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  240, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  241,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Zimbabwe
    {  241,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  241,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  241,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  241,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  241,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  241,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  241,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  241,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  241,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  241,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  241, 1502,  218831,  218856,  218890, 25, 34, 34, }, // Pierre_Miquelon
    {  243,    5,       0,  219344,       0,  0, 17,  0, }, // Afghanistan Erzya/Cyrillic/Russia
    {  243,  107,  219361,  219373,  219393, 12, 20, 19, }, // Alaska
    {  243,  135,  219412,  219425,  219446, 13, 21, 20, }, // Amazon
    {  243,  397,  219466,  219482,  219506, 16, 24, 23, }, // Bangladesh
    {  243,  437,  219529,  219542,  219563, 13, 21, 20, }, // Brasilia
    {  243,  495,  219583,  219593,  219611, 10, 18, 17, }, // Chile
    {  243,  501,  219628,  219639,  219658, 11, 19, 18, }, // China
    {  243,  537,  219676,  219686,  219704, 10, 18, 17, }, // Cuba
    {  243,  914,       0,  219721,       0,  0, 19,  0, }, // India
    {  243,  997,  219740,  219751,  219770, 11, 19, 18, }, // Iran
    {  243, 1023,  219788,  219800,  219820, 12, 20, 19, }, // Japan
    {  243, 1058,       0,  219839,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  243, 1077,       0,  219866,       0,  0, 28,  0, }, // Kazakhstan_Western
    {  243, 1106,  219894,  219905,  219924, 11, 19, 18, }, // Korea
    {  243, 1305,  219942,  219955,  219976, 13, 21, 20, }, // Moscow
    {  243, 1346,  219996,  220013,  220038, 17, 25, 24, }, // New_Zealand
    {  243, 1418,  220062,  220074,  220094, 12, 20, 19, }, // Omsk
    {  243, 1460,  220113,  220127,  220149, 14, 22, 21, }, // Paraguay
    {  243, 1570,  220170,  220184,  220206, 14, 22, 21, }, // Sakhalin
    {  243, 1777,  220227,  220240,  220261, 13, 21, 20, }, // Uruguay
    {  243, 1792,  220281,  220298,  220323, 17, 25, 24, }, // Uzbekistan
    {  244,    5,       0,  220347,       0,  0, 12,  0, }, // Afghanistan Esperanto/Latin/world
    {  244,   17,       0,  220359,       0,  0, 17,  0, }, // Africa_Central
    {  244,   32,       0,  220376,       0,  0, 18,  0, }, // Africa_Eastern
    {  244,   65,       0,  220394,       0,  0, 15,  0, }, // Africa_Southern
    {  244,   81,  220409,  220429,  220455, 20, 26, 27, }, // Africa_Western
    {  244,  107,  220482,  220494,  220512, 12, 18, 19, }, // Alaska
    {  244,  135,  220531,  220545,  220565, 14, 20, 21, }, // Amazon
    {  244,  142,  220586,  220610,  220640, 24, 30, 31, }, // America_Central
    {  244,  158,  220671,  220696,  220727, 25, 31, 32, }, // America_Eastern
    {  244,  174,  220759,  220784,  220815, 25, 31, 32, }, // America_Mountain
    {  244,  191,  220847,  220873,  220905, 26, 32, 33, }, // America_Pacific
    {  244,  214,  220938,  220949,  220967, 11, 18, 19, }, // Apia
    {  244,  232,  220986,  220997,  221014, 11, 17, 18, }, // Arabian
    {  244,  240,  221032,  221047,  221068, 15, 21, 22, }, // Argentina
    {  244,  250,  221090,  221114,  221144, 24, 30, 31, }, // Argentina_Western
    {  244,  268,  221175,  221187,  221205, 12, 18, 19, }, // Armenia
    {  244,  286,  221224,  221251,  221284, 27, 33, 34, }, // Atlantic
    {  244,  295,  221318,  221340,  221368, 22, 28, 29, }, // Australia_Central
    {  244,  313,  221397,  221427,  221463, 30, 36, 37, }, // Australia_CentralWestern
    {  244,  338,  221500,  221523,  221552, 23, 29, 30, }, // Australia_Eastern
    {  244,  356,  221402,  221432,  221468, 25, 31, 32, }, // Australia_Western
    {  244,  374,  221582,  221599,  221622, 17, 23, 24, }, // Azerbaijan
    {  244,  385,  221646,  221659,  221679, 13, 20, 21, }, // Azores
    {  244,  397,  221700,  221716,  221738, 16, 22, 23, }, // Bangladesh
    {  244,  415,       0,  221761,       0,  0, 12,  0, }, // Bhutan
    {  244,  422,       0,  221773,       0,  0, 13,  0, }, // Bolivia
    {  244,  437,  221786,  221800,  221820, 14, 20, 21, }, // Brasilia
    {  244,  454,       0,  221841,       0,  0, 13,  0, }, // Brunei
    {  244,  461,  221854,  221869,  221890, 15, 21, 22, }, // Cape_Verde
    {  244,  478,       0,  221912,       0,  0, 12,  0, }, // Chamorro
    {  244,  487,  221924,  221937,  221956, 13, 19, 20, }, // Chatham
    {  244,  495,  221976,  221987,  222004, 11, 17, 18, }, // Chile
    {  244,  501,  222022,  222032,  222048, 10, 16, 17, }, // China
    {  244,  507,       0,  222065,       0,  0, 21,  0, }, // Christmas
    {  244,  517,       0,  222086,       0,  0, 17,  0, }, // Cocos
    {  244,  523,  222103,  222117,  222137, 14, 20, 21, }, // Colombia
    {  244,  532,  222158,  222173,  222194, 15, 21, 22, }, // Cook
    {  244,  537,  222216,  222227,  222245, 11, 18, 19, }, // Cuba
    {  244,  548,       0,  222264,       0,  0, 12,  0, }, // Davis
    {  244,  564,       0,  222276,       0,  0, 23,  0, }, // DumontDUrville
    {  244,  601,       0,  222299,       0,  0, 19,  0, }, // East_Timor
    {  244,  612,  222318,  222334,  222356, 16, 22, 23, }, // Easter
    {  244,  619,       0,  222379,       0,  0, 14,  0, }, // Ecuador
    {  244,  627,  222393,  222410,  222433, 17, 23, 24, }, // Europe_Central
    {  244,  642,  222457,  222475,  222499, 18, 24, 25, }, // Europe_Eastern
    {  244,  657,       0,  222524,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {  244,  680,  222550,  222570,  222596, 20, 26, 27, }, // Europe_Western
    {  244,  695,  222623,  222638,  222659, 15, 21, 22, }, // Falkland
    {  244,  704,  222681,  222692,  222709, 11, 17, 18, }, // Fiji
    {  244,  709,       0,  222727,       0,  0, 20,  0, }, // French_Guiana
    {  244,  723,       0,  222747,       0,  0, 31,  0, }, // French_Southern
    {  244,  746,       0,  222778,       0,  0, 14,  0, }, // Galapagos
    {  244,  756,       0,  222792,       0,  0, 14,  0, }, // Gambier
    {  244,  764,  222806,  222820,  222840, 14, 20, 21, }, // Georgia
    {  244,  772,       0,  222861,       0,  0, 19,  0, }, // Gilbert_Islands
    {  244,  788,       0,  222880,       0,  0, 28,  0, }, // GMT
    {  244,  830,  222908,  222931,  222960, 23, 29, 30, }, // Greenland_Eastern
    {  244,  848,  222990,  223015,  223046, 25, 31, 32, }, // Greenland_Western
    {  244,  871,       0,  223078,       0,  0, 21,  0, }, // Gulf
    {  244,  876,       0,  223099,       0,  0, 12,  0, }, // Guyana
    {  244,  883,  223111,  223132,  223160, 21, 28, 29, }, // Hawaii_Aleutian
    {  244,  899,  223189,  223203,  223223, 14, 20, 21, }, // Hong_Kong
    {  244,  909,  223244,  223255,  223272, 11, 17, 18, }, // Hovd
    {  244,  914,       0,  223290,       0,  0, 17,  0, }, // India
    {  244,  920,       0,  223307,       0,  0, 16,  0, }, // Indian_Ocean
    {  244,  933,       0,  223323,       0,  0, 15,  0, }, // Indochina
    {  244,  943,       0,  223338,       0,  0, 21,  0, }, // Indonesia_Central
    {  244,  961,       0,  223359,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  244,  979,       0,  223381,       0,  0, 24,  0, }, // Indonesia_Western
    {  244,  997,  223405,  223416,  223433, 11, 17, 18, }, // Iran
    {  244, 1008,  223451,  223465,  223485, 14, 20, 21, }, // Irkutsk
    {  244, 1016,  223506,  223519,  223538, 13, 19, 20, }, // Israel
    {  244, 1023,  223558,  223570,  223588, 12, 18, 19, }, // Japan
    {  244, 1047,       0,  223607,       0,  0, 12,  0, }, // Kazakhstan
    {  244, 1058,       0,  223619,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  244, 1077,       0,  223638,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  244, 1106,  223659,  223670,  223687, 11, 17, 18, }, // Korea
    {  244, 1112,       0,  223705,       0,  0, 13,  0, }, // Kosrae
    {  244, 1119,  223718,  223736,  223760, 18, 24, 25, }, // Krasnoyarsk
    {  244, 1151,       0,  223785,       0,  0, 13,  0, }, // Kyrgystan
    {  244, 1175,       0,  223798,       0,  0, 21,  0, }, // Line_Islands
    {  244, 1188,  223819,  223835,  223858, 16, 23, 24, }, // Lord_Howe
    {  244, 1204,  223882,  223896,  223916, 14, 20, 21, }, // Magadan
    {  244, 1219,       0,  223937,       0,  0, 14,  0, }, // Malaysia
    {  244, 1228,       0,  223951,       0,  0, 13,  0, }, // Maldives
    {  244, 1237,       0,  223964,       0,  0, 18,  0, }, // Marquesas
    {  244, 1247,       0,  223982,       0,  0, 18,  0, }, // Marshall_Islands
    {  244, 1264,  224000,  224014,  224034, 14, 20, 21, }, // Mauritius
    {  244, 1274,       0,  224055,       0,  0, 13,  0, }, // Mawson
    {  244, 1281,  224068,  224090,  224118, 22, 28, 29, }, // Mexico_Pacific
    {  244, 1296,  224147,  224163,  224185, 16, 22, 23, }, // Mongolia
    {  244, 1305,  224208,  224220,  224238, 12, 18, 19, }, // Moscow
    {  244, 1312,       0,  224257,       0,  0, 11,  0, }, // Myanmar
    {  244, 1320,       0,  224268,       0,  0, 11,  0, }, // Nauru
    {  244, 1326,       0,  224279,       0,  0, 12,  0, }, // Nepal
    {  244, 1332,  224291,  224309,  224333, 18, 24, 25, }, // New_Caledonia
    {  244, 1346,  224358,  224374,  224396, 16, 22, 23, }, // New_Zealand
    {  244, 1358,  224419,  224434,  224456, 15, 22, 23, }, // Newfoundland
    {  244, 1371,       0,  224479,       0,  0, 10,  0, }, // Niue
    {  244, 1376,  224489,  224508,  224533, 19, 25, 26, }, // Norfolk
    {  244, 1384,  224559,  224585,  224618, 26, 33, 34, }, // Noronha
    {  244, 1406,  224652,  224670,  224694, 18, 24, 25, }, // Novosibirsk
    {  244, 1418,  224719,  224730,  224747, 11, 17, 18, }, // Omsk
    {  244, 1428,  224765,  224780,  224801, 15, 21, 22, }, // Pakistan
    {  244, 1437,       0,  224823,       0,  0, 12,  0, }, // Palau
    {  244, 1443,       0,  224835,       0,  0, 23,  0, }, // Papua_New_Guinea
    {  244, 1460,  224858,  224873,  224894, 15, 21, 22, }, // Paraguay
    {  244, 1469,  224916,  224927,  224944, 11, 17, 18, }, // Peru
    {  244, 1474,  224962,  224976,  224996, 14, 20, 21, }, // Philippines
    {  244, 1486,       0,  225017,       0,  0, 18,  0, }, // Phoenix_Islands
    {  244, 1502,  225035,  225067,  225106, 32, 39, 40, }, // Pierre_Miquelon
    {  244, 1518,       0,  225146,       0,  0, 19,  0, }, // Pitcairn
    {  244, 1527,       0,  225165,       0,  0, 13,  0, }, // Ponape
    {  244, 1534,       0,  225178,       0,  0, 16,  0, }, // Pyongyang
    {  244, 1554,       0,  225194,       0,  0, 13,  0, }, // Reunion
    {  244, 1562,       0,  225207,       0,  0, 14,  0, }, // Rothera
    {  244, 1570,  225221,  225235,  225255, 14, 20, 21, }, // Sakhalin
    {  244, 1596,  225276,  225287,  225304, 11, 17, 18, }, // Samoa
    {  244, 1602,       0,  225322,       0,  0, 13,  0, }, // Seychelles
    {  244, 1624,       0,  225335,       0,  0, 21,  0, }, // Singapore
    {  244, 1634,       0,  225356,       0,  0, 16,  0, }, // Solomon
    {  244, 1642,       0,  225372,       0,  0, 18,  0, }, // South_Georgia
    {  244, 1656,       0,  225390,       0,  0, 14,  0, }, // Suriname
    {  244, 1676,       0,  225404,       0,  0, 12,  0, }, // Syowa
    {  244, 1682,       0,  225416,       0,  0, 13,  0, }, // Tahiti
    {  244, 1689,  225429,  225441,  225459, 12, 18, 19, }, // Taipei
    {  244, 1696,       0,  225478,       0,  0, 12,  0, }, // Tajikistan
    {  244, 1724,       0,  225490,       0,  0, 13,  0, }, // Tokelau
    {  244, 1732,  225503,  225514,  225531, 11, 17, 18, }, // Tonga
    {  244, 1738,       0,  225549,       0,  0, 11,  0, }, // Truk
    {  244, 1750,  225560,  225574,  225594, 14, 20, 21, }, // Turkmenistan
    {  244, 1763,       0,  225615,       0,  0, 12,  0, }, // Tuvalu
    {  244, 1777,  225627,  225641,  225661, 14, 20, 21, }, // Uruguay
    {  244, 1792,  225682,  225694,  225712, 12, 18, 19, }, // Uzbekistan
    {  244, 1803,  225731,  225745,  225765, 14, 20, 21, }, // Vanuatu
    {  244, 1811,       0,  225786,       0,  0, 15,  0, }, // Venezuela
    {  244, 1821,  225801,  225819,  225843, 18, 24, 25, }, // Vladivostok
    {  244, 1833,  225868,  225884,  225906, 16, 22, 23, }, // Volgograd
    {  244, 1843,       0,  225929,       0,  0, 13,  0, }, // Vostok
    {  244, 1850,       0,  225942,       0,  0, 16,  0, }, // Wake
    {  244, 1855,       0,  225958,       0,  0, 24,  0, }, // Wallis
    {  244, 1862,  225982,  225996,  226016, 14, 20, 21, }, // Yakutsk
    {  244, 1870,  226037,  226057,  226083, 20, 26, 27, }, // Yekaterinburg
    {  244, 1892,       0,  226110,       0,  0, 13,  0, }, // Yukon
    {  245,    0,  226123,  226131,  226147,  8, 16, 12, }, // Acre Estonian/Latin/Estonia
    {  245,    5,       0,  226159,       0,  0, 15,  0, }, // Afghanistan
    {  245,   17,       0,  226174,       0,  0, 16,  0, }, // Africa_Central
    {  245,   32,       0,  226190,       0,  0, 15,  0, }, // Africa_Eastern
    {  245,   65,       0,  226205,       0,  0, 25,  0, }, // Africa_Southern
    {  245,   81,  226230,  226247,  226272, 17, 25, 21, }, // Africa_Western
    {  245,  107,  226293,  226303,  226321, 10, 18, 14, }, // Alaska
    {  245,  128,  226335,  226345,  226363, 10, 18, 14, }, // Almaty
    {  245,  135,  226377,  226390,  226411, 13, 21, 17, }, // Amazon
    {  245,  142,  226428,  226445,  226470, 17, 25, 21, }, // America_Central
    {  245,  158,  226491,  226505,  226527, 14, 22, 18, }, // America_Eastern
    {  245,  174,  226545,  226563,  226589, 18, 26, 22, }, // America_Mountain
    {  245,  191,  226611,  226629,  226655, 18, 26, 22, }, // America_Pacific
    {  245,  207,  226677,  226688,  226707, 11, 19, 15, }, // Anadyr
    {  245,  214,  226722,  226730,  226746,  8, 16, 12, }, // Apia
    {  245,  219,  226758,  226767,  226784,  9, 17, 13, }, // Aqtau
    {  245,  225,  226797,  226807,  226825, 10, 18, 14, }, // Aqtobe
    {  245,  232,  226839,  226850,  226869, 11, 19, 15, }, // Arabian
    {  245,  240,  226884,  226897,  226918, 13, 21, 17, }, // Argentina
    {  245,  250,  226935,  226954,  226981, 19, 27, 23, }, // Argentina_Western
    {  245,  268,  227004,  227016,  227036, 12, 20, 16, }, // Armenia
    {  245,  286,  227052,  227063,  227082, 11, 19, 15, }, // Atlantic
    {  245,  295,  227097,  227116,  227143, 19, 27, 23, }, // Australia_Central
    {  245,  313,  227166,  227191,  227224, 25, 33, 29, }, // Australia_CentralWestern
    {  245,  338,  227253,  227271,  227297, 18, 26, 22, }, // Australia_Eastern
    {  245,  356,  227319,  227339,  227367, 20, 28, 24, }, // Australia_Western
    {  245,  374,  227391,  227408,  227433, 17, 25, 21, }, // Azerbaijan
    {  245,  385,  227454,  227467,  227488, 13, 21, 17, }, // Azores
    {  245,  397,  227505,  227520,  227543, 15, 23, 19, }, // Bangladesh
    {  245,  415,       0,  227562,       0,  0, 11,  0, }, // Bhutan
    {  245,  422,       0,  227573,       0,  0, 12,  0, }, // Bolivia
    {  245,  437,  227585,  227598,  227619, 13, 21, 17, }, // Brasilia
    {  245,  454,       0,  227636,       0,  0, 10,  0, }, // Brunei
    {  245,  461,  227646,  227665,  227692, 19, 27, 23, }, // Cape_Verde
    {  245,  472,       0,  227715,       0,  0,  9,  0, }, // Casey
    {  245,  478,       0,  227724,       0,  0, 20,  0, }, // Chamorro
    {  245,  487,  227744,  227756,  227776, 12, 20, 16, }, // Chatham
    {  245,  495,  227792,  227802,  227820, 10, 18, 14, }, // Chile
    {  245,  501,  227834,  227843,  227860,  9, 17, 13, }, // China
    {  245,  507,       0,  227873,       0,  0, 14,  0, }, // Christmas
    {  245,  517,       0,  227887,       0,  0, 16,  0, }, // Cocos
    {  245,  523,  227903,  227915,  227935, 12, 20, 16, }, // Colombia
    {  245,  532,  227951,  227967,  227991, 16, 24, 28, }, // Cook
    {  245,  537,  228019,  228028,  228045,  9, 17, 13, }, // Cuba
    {  245,  548,       0,  228058,       0,  0, 10,  0, }, // Davis
    {  245,  564,       0,  228068,       0,  0, 22,  0, }, // DumontDUrville
    {  245,  601,       0,  228090,       0,  0, 14,  0, }, // East_Timor
    {  245,  612,  228104,  228122,  228148, 18, 26, 22, }, // Easter
    {  245,  619,       0,  228170,       0,  0, 12,  0, }, // Ecuador
    {  245,  627,  228182,  228198,  228222, 16, 24, 20, }, // Europe_Central
    {  245,  642,  228242,  228257,  228280, 15, 23, 19, }, // Europe_Eastern
    {  245,  657,       0,  228299,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {  245,  680,  228328,  228345,  228370, 17, 25, 21, }, // Europe_Western
    {  245,  695,  228391,  228411,  228439, 20, 28, 24, }, // Falkland
    {  245,  704,  228463,  228472,  228489,  9, 17, 13, }, // Fiji
    {  245,  709,       0,  228502,       0,  0, 22,  0, }, // French_Guiana
    {  245,  723,       0,  228524,       0,  0, 41,  0, }, // French_Southern
    {  245,  746,       0,  228565,       0,  0, 14,  0, }, // Galapagos
    {  245,  756,       0,  228579,       0,  0, 12,  0, }, // Gambier
    {  245,  764,  228591,  228602,  228621, 11, 19, 15, }, // Georgia
    {  245,  772,       0,  228636,       0,  0, 19,  0, }, // Gilbert_Islands
    {  245,  788,       0,  228655,       0,  0, 14,  0, }, // GMT
    {  245,  802,  228669,  228682,  228703, 13, 21, 17, }, // Greenland
    {  245,  830,  228720,  228737,  228762, 17, 25, 21, }, // Greenland_Eastern
    {  245,  848,  228783,  228802,  228829, 19, 27, 23, }, // Greenland_Western
    {  245,  866,       0,  228852,       0,  0, 17,  0, }, // Guam
    {  245,  871,       0,  228869,       0,  0, 23,  0, }, // Gulf
    {  245,  876,       0,  228892,       0,  0, 10,  0, }, // Guyana
    {  245,  883,  228902,  228920,  228946, 18, 26, 22, }, // Hawaii_Aleutian
    {  245,  899,  228968,  228981,  229002, 13, 21, 17, }, // Hong_Kong
    {  245,  909,  229019,  229028,  229045,  9, 17, 13, }, // Hovd
    {  245,  914,       0,  229058,       0,  0,  9,  0, }, // India
    {  245,  920,       0,  229067,       0,  0, 17,  0, }, // Indian_Ocean
    {  245,  933,       0,  229084,       0,  0, 13,  0, }, // Indochina
    {  245,  943,       0,  229097,       0,  0, 19,  0, }, // Indonesia_Central
    {  245,  961,       0,  229116,       0,  0, 18,  0, }, // Indonesia_Eastern
    {  245,  979,       0,  229134,       0,  0, 20,  0, }, // Indonesia_Western
    {  245,  997,  229154,  229164,  229182, 10, 18, 14, }, // Iran
    {  245, 1008,  229196,  229208,  229228, 12, 20, 16, }, // Irkutsk
    {  245, 1016,  229244,  229256,  229276, 12, 20, 16, }, // Israel
    {  245, 1023,  229292,  229303,  229322, 11, 19, 15, }, // Japan
    {  245, 1029,  229337,  229365,  229386, 28, 21, 17, }, // Kamchatka
    {  245, 1047,       0,  229403,       0,  0, 14,  0, }, // Kazakhstan
    {  245, 1058,       0,  229417,       0,  0, 18,  0, }, // Kazakhstan_Eastern
    {  245, 1077,       0,  229435,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  245, 1106,  229455,  229464,  229481,  9, 17, 13, }, // Korea
    {  245, 1112,       0,  229494,       0,  0, 10,  0, }, // Kosrae
    {  245, 1119,  229504,  229520,  229544, 16, 24, 20, }, // Krasnoyarsk
    {  245, 1151,       0,  229564,       0,  0, 15,  0, }, // Kyrgystan
    {  245, 1161,       0,  229579,       0,  0, 13,  0, }, // Lanka
    {  245, 1175,       0,  229592,       0,  0, 17,  0, }, // Line_Islands
    {  245, 1188,  229609,  229624,  229647, 15, 23, 19, }, // Lord_Howe
    {  245, 1198,  229666,  229675,  229692,  9, 17, 13, }, // Macau
    {  245, 1204,  229705,  229717,  229737, 12, 20, 16, }, // Magadan
    {  245, 1219,       0,  229753,       0,  0, 12,  0, }, // Malaysia
    {  245, 1228,       0,  229765,       0,  0, 12,  0, }, // Maldives
    {  245, 1237,       0,  229777,       0,  0, 16,  0, }, // Marquesas
    {  245, 1247,       0,  229793,       0,  0, 20,  0, }, // Marshall_Islands
    {  245, 1264,  229813,  229827,  229849, 14, 22, 18, }, // Mauritius
    {  245, 1274,       0,  229867,       0,  0, 11,  0, }, // Mawson
    {  245, 1281,  229878,  229904,  229938, 26, 34, 30, }, // Mexico_Pacific
    {  245, 1296,  229968,  229984,  230008, 16, 24, 20, }, // Mongolia
    {  245, 1305,  230028,  230038,  230056, 10, 18, 14, }, // Moscow
    {  245, 1312,       0,  230070,       0,  0,  9,  0, }, // Myanmar
    {  245, 1320,       0,  230079,       0,  0,  9,  0, }, // Nauru
    {  245, 1326,       0,  230088,       0,  0, 10,  0, }, // Nepal
    {  245, 1332,  230098,  230116,  230142, 18, 26, 22, }, // New_Caledonia
    {  245, 1346,  230164,  230179,  230202, 15, 23, 19, }, // New_Zealand
    {  245, 1358,  230221,  230238,  230263, 17, 25, 21, }, // Newfoundland
    {  245, 1371,       0,  230284,       0,  0,  8,  0, }, // Niue
    {  245, 1376,  230292,  230310,  230336, 18, 26, 22, }, // Norfolk
    {  245, 1384,  230358,  230381,  230412, 23, 31, 27, }, // Noronha
    {  245, 1392,       0,  230439,       0,  0, 18,  0, }, // North_Mariana
    {  245, 1406,  230457,  230473,  230497, 16, 24, 20, }, // Novosibirsk
    {  245, 1418,  230517,  230526,  230543,  9, 17, 13, }, // Omsk
    {  245, 1428,  230556,  230569,  230590, 13, 21, 17, }, // Pakistan
    {  245, 1437,       0,  230607,       0,  0,  9,  0, }, // Palau
    {  245, 1443,       0,  230616,       0,  0, 21,  0, }, // Papua_New_Guinea
    {  245, 1460,  230637,  230649,  230669, 12, 20, 16, }, // Paraguay
    {  245, 1469,  230685,  230694,  230711,  9, 17, 13, }, // Peru
    {  245, 1474,  230724,  230737,  230758, 13, 21, 17, }, // Philippines
    {  245, 1486,       0,  230775,       0,  0, 18,  0, }, // Phoenix_Islands
    {  245, 1502,  230793,  230824,  230863, 31, 39, 35, }, // Pierre_Miquelon
    {  245, 1518,       0,  230898,       0,  0, 13,  0, }, // Pitcairn
    {  245, 1527,       0,  230911,       0,  0, 11,  0, }, // Ponape
    {  245, 1534,       0,  230922,       0,  0, 14,  0, }, // Pyongyang
    {  245, 1544,  230936,  230949,  230970, 13, 21, 17, }, // Qyzylorda
    {  245, 1554,       0,  230987,       0,  0, 12,  0, }, // Reunion
    {  245, 1562,       0,  230999,       0,  0, 11,  0, }, // Rothera
    {  245, 1570,  231010,  231023,  231044, 13, 21, 17, }, // Sakhalin
    {  245, 1579,  231061,  231071,  231089, 10, 18, 14, }, // Samara
    {  245, 1596,  231103,  231112,  231129,  9, 17, 13, }, // Samoa
    {  245, 1602,       0,  231142,       0,  0, 12,  0, }, // Seychelles
    {  245, 1624,       0,  231154,       0,  0, 21,  0, }, // Singapore
    {  245, 1634,       0,  231175,       0,  0, 20,  0, }, // Solomon
    {  245, 1642,       0,  231195,       0,  0, 17,  0, }, // South_Georgia
    {  245, 1656,       0,  231212,       0,  0, 12,  0, }, // Suriname
    {  245, 1676,       0,  231224,       0,  0,  9,  0, }, // Syowa
    {  245, 1682,       0,  231233,       0,  0, 10,  0, }, // Tahiti
    {  245, 1689,  231243,  231253,  231271, 10, 18, 14, }, // Taipei
    {  245, 1696,       0,  231285,       0,  0, 16,  0, }, // Tajikistan
    {  245, 1724,       0,  231301,       0,  0, 11,  0, }, // Tokelau
    {  245, 1732,  231312,  231321,  231338,  9, 17, 13, }, // Tonga
    {  245, 1738,       0,  231351,       0,  0, 10,  0, }, // Truk
    {  245, 1750,  231361,  231378,  231403, 17, 25, 21, }, // Turkmenistan
    {  245, 1763,       0,  231424,       0,  0, 10,  0, }, // Tuvalu
    {  245, 1777,  231434,  231445,  231464, 11, 19, 15, }, // Uruguay
    {  245, 1792,  231479,  231494,  231517, 15, 23, 19, }, // Uzbekistan
    {  245, 1803,  231536,  231547,  231566, 11, 19, 15, }, // Vanuatu
    {  245, 1811,       0,  231581,       0,  0, 13,  0, }, // Venezuela
    {  245, 1821,  231594,  231610,  231634, 16, 24, 20, }, // Vladivostok
    {  245, 1833,  231654,  231668,  231690, 14, 22, 18, }, // Volgograd
    {  245, 1843,       0,  231708,       0,  0, 11,  0, }, // Vostok
    {  245, 1850,       0,  231719,       0,  0, 10,  0, }, // Wake
    {  245, 1855,       0,  231729,       0,  0, 21,  0, }, // Wallis
    {  245, 1862,  231750,  231762,  231782, 12, 20, 16, }, // Yakutsk
    {  245, 1870,  231798,  231816,  231842, 18, 26, 22, }, // Yekaterinburg
    {  245, 1892,       0,  231864,       0,  0, 10,  0, }, // Yukon
    {  246,    0,  231874,  231887,  231906, 13, 19, 23, }, // Acre Ewe/Latin/Ghana
    {  246,    5,       0,  231929,       0,  0, 21,  0, }, // Afghanistan
    {  246,   17,       0,  231950,       0,  0, 24,  0, }, // Africa_Central
    {  246,   32,       0,  231974,       0,  0, 21,  0, }, // Africa_Eastern
    {  246,   65,       0,  231995,       0,  0, 29,  0, }, // Africa_Southern
    {  246,   81,  232024,  232040,  232068, 16, 28, 31, }, // Africa_Western
    {  246,  107,  232099,  232114,  232137, 15, 23, 21, }, // Alaska
    {  246,  128,  232158,  232173,  232194, 15, 21, 25, }, // Almaty
    {  246,  135,  232219,  232234,  232257, 15, 23, 26, }, // Amazon
    {  246,  142,  232283,  232306,  232337, 23, 31, 29, }, // America_Central
    {  246,  158,  232366,  232391,  232423, 25, 32, 30, }, // America_Eastern
    {  246,  174,  232453,  232471,  232496, 18, 25, 23, }, // America_Mountain
    {  246,  191,  232519,  232535,  232559, 16, 24, 22, }, // America_Pacific
    {  246,  207,  232581,  232596,  232617, 15, 21, 23, }, // Anadyr
    {  246,  214,  232640,  232654,  232675, 14, 21, 19, }, // Apia
    {  246,  219,  232694,  232708,  232728, 14, 20, 24, }, // Aqtau
    {  246,  225,  232752,  232767,  232788, 15, 21, 16, }, // Aqtobe
    {  246,  232,  232804,  232820,  232843, 16, 23, 26, }, // Arabian
    {  246,  240,  232869,  232888,  232914, 19, 26, 29, }, // Argentina
    {  246,  250,  232943,  232971,  233006, 28, 35, 38, }, // Argentina_Western
    {  246,  268,  233044,  233061,  233085, 17, 24, 27, }, // Armenia
    {  246,  286,  233112,  233129,  233153, 17, 24, 22, }, // Atlantic
    {  246,  295,  233175,  233202,  233237, 27, 35, 36, }, // Australia_Central
    {  246,  313,  233273,  233320,  233363, 47, 43, 41, }, // Australia_CentralWestern
    {  246,  338,  233404,  233431,  233466, 27, 35, 33, }, // Australia_Eastern
    {  246,  356,  233499,  233526,  233561, 27, 35, 33, }, // Australia_Western
    {  246,  374,  233594,  233614,  233641, 20, 27, 30, }, // Azerbaijan
    {  246,  385,  233671,  233687,  233710, 16, 23, 26, }, // Azores
    {  246,  397,  233736,  233756,  233783, 20, 27, 30, }, // Bangladesh
    {  246,  415,       0,  233813,       0,  0, 16,  0, }, // Bhutan
    {  246,  422,       0,  233829,       0,  0, 17,  0, }, // Bolivia
    {  246,  437,  233846,  233864,  233889, 18, 25, 28, }, // Brasilia
    {  246,  454,       0,  233917,       0,  0, 27,  0, }, // Brunei
    {  246,  461,  233944,  233964,  233991, 20, 27, 30, }, // Cape_Verde
    {  246,  478,       0,  234021,       0,  0, 18,  0, }, // Chamorro
    {  246,  487,  234039,  234056,  234080, 17, 24, 22, }, // Chatham
    {  246,  495,  234102,  234117,  234139, 15, 22, 25, }, // Chile
    {  246,  501,  234164,  234179,  234201, 15, 22, 20, }, // China
    {  246,  507,       0,  234221,       0,  0, 26,  0, }, // Christmas
    {  246,  517,       0,  234247,       0,  0, 23,  0, }, // Cocos
    {  246,  523,  234270,  234288,  234313, 18, 25, 28, }, // Colombia
    {  246,  532,  234341,  234363,  234392, 22, 29, 32, }, // Cook
    {  246,  537,  234424,  234437,  234457, 13, 20, 18, }, // Cuba
    {  246,  548,       0,  234475,       0,  0, 15,  0, }, // Davis
    {  246,  564,       0,  234490,       0,  0, 26,  0, }, // DumontDUrville
    {  246,  601,       0,  234516,       0,  0, 20,  0, }, // East_Timor
    {  246,  612,  234536,  234559,  234589, 23, 30, 33, }, // Easter
    {  246,  619,       0,  234622,       0,  0, 17,  0, }, // Ecuador
    {  246,  627,  234639,  234663,  234694, 24, 31, 34, }, // Europe_Central
    {  246,  642,  234728,  234751,  234780, 23, 29, 31, }, // Europe_Eastern
    {  246,  680,  234811,  234835,  234866, 24, 31, 34, }, // Europe_Western
    {  246,  695,  234900,  234926,  234959, 26, 33, 36, }, // Falkland
    {  246,  704,  234995,  235009,  235030, 14, 21, 24, }, // Fiji
    {  246,  709,       0,  235054,       0,  0, 23,  0, }, // French_Guiana
    {  246,  723,       0,  235077,       0,  0, 37,  0, }, // French_Southern
    {  246,  746,       0,  235114,       0,  0, 19,  0, }, // Galapagos
    {  246,  756,       0,  235133,       0,  0, 17,  0, }, // Gambier
    {  246,  764,  235150,  235167,  235191, 17, 24, 27, }, // Georgia
    {  246,  772,       0,  235218,       0,  0, 25,  0, }, // Gilbert_Islands
    {  246,  788,       0,  235243,       0,  0, 19,  0, }, // GMT
    {  246,  830,  235262,  235285,  235316, 23, 31, 34, }, // Greenland_Eastern
    {  246,  848,  235350,  235374,  235405, 24, 31, 29, }, // Greenland_Western
    {  246,  871,       0,  235434,       0,  0, 21,  0, }, // Gulf
    {  246,  876,       0,  235455,       0,  0, 16,  0, }, // Guyana
    {  246,  883,  235471,  235495,  235526, 24, 31, 29, }, // Hawaii_Aleutian
    {  246,  899,  235555,  235574,  235600, 19, 26, 29, }, // Hong_Kong
    {  246,  909,  235629,  235643,  235664, 14, 21, 24, }, // Hovd
    {  246,  914,       0,  235688,       0,  0, 15,  0, }, // India
    {  246,  920,       0,  235703,       0,  0, 22,  0, }, // Indian_Ocean
    {  246,  933,       0,  235725,       0,  0, 19,  0, }, // Indochina
    {  246,  943,       0,  235744,       0,  0, 27,  0, }, // Indonesia_Central
    {  246,  961,       0,  235771,       0,  0, 27,  0, }, // Indonesia_Eastern
    {  246,  979,       0,  235798,       0,  0, 27,  0, }, // Indonesia_Western
    {  246,  997,  235825,  235839,  235860, 14, 21, 19, }, // Iran
    {  246, 1008,  235879,  235896,  235920, 17, 24, 27, }, // Irkutsk
    {  246, 1016,  235947,  235963,  235986, 16, 23, 21, }, // Israel
    {  246, 1023,  236007,  236022,  236044, 15, 22, 25, }, // Japan
    {  246, 1029,  236069,  236102,  236141, 33, 39, 41, }, // Kamchatka
    {  246, 1058,       0,  236182,       0,  0, 25,  0, }, // Kazakhstan_Eastern
    {  246, 1077,       0,  236207,       0,  0, 25,  0, }, // Kazakhstan_Western
    {  246, 1106,  236232,  236247,  236269, 15, 22, 25, }, // Korea
    {  246, 1112,       0,  236294,       0,  0, 16,  0, }, // Kosrae
    {  246, 1119,  236310,  236331,  236359, 21, 28, 31, }, // Krasnoyarsk
    {  246, 1151,       0,  236390,       0,  0, 19,  0, }, // Kyrgystan
    {  246, 1175,       0,  236409,       0,  0, 22,  0, }, // Line_Islands
    {  246, 1188,  236431,  236450,  236476, 19, 26, 24, }, // Lord_Howe
    {  246, 1198,  236500,  236514,  236534, 14, 20, 22, }, // Macau
    {  246, 1204,  236556,  236573,  236597, 17, 24, 27, }, // Magadan
    {  246, 1219,       0,  236624,       0,  0, 18,  0, }, // Malaysia
    {  246, 1228,       0,  236642,       0,  0, 18,  0, }, // Maldives
    {  246, 1237,       0,  236660,       0,  0, 19,  0, }, // Marquesas
    {  246, 1247,       0,  236679,       0,  0, 26,  0, }, // Marshall_Islands
    {  246, 1264,  236705,  236724,  236750, 19, 26, 29, }, // Mauritius
    {  246, 1274,       0,  236779,       0,  0, 16,  0, }, // Mawson
    {  246, 1281,  236795,  236820,  236852, 25, 32, 29, }, // Mexico_Pacific
    {  246, 1296,  236881,  236901,  236928, 20, 27, 30, }, // Mongolia
    {  246, 1305,  236958,  236974,  236997, 16, 23, 26, }, // Moscow
    {  246, 1312,       0,  237023,       0,  0, 17,  0, }, // Myanmar
    {  246, 1320,       0,  237040,       0,  0, 15,  0, }, // Nauru
    {  246, 1326,       0,  237055,       0,  0, 15,  0, }, // Nepal
    {  246, 1332,  237070,  237093,  237123, 23, 30, 33, }, // New_Caledonia
    {  246, 1346,  237156,  237177,  237205, 21, 28, 26, }, // New_Zealand
    {  246, 1358,  237231,  237252,  237280, 21, 28, 26, }, // Newfoundland
    {  246, 1371,       0,  237306,       0,  0, 14,  0, }, // Niue
    {  246, 1376,  237320,  237344,  237375, 24, 31, 34, }, // Norfolk
    {  246, 1384,  237409,  237438,  237474, 29, 36, 39, }, // Noronha
    {  246, 1406,  237513,  237534,  237562, 21, 28, 31, }, // Novosibirsk
    {  246, 1418,  237593,  237607,  237628, 14, 21, 24, }, // Omsk
    {  246, 1428,  237652,  237670,  237695, 18, 25, 28, }, // Pakistan
    {  246, 1437,       0,  237723,       0,  0, 15,  0, }, // Palau
    {  246, 1443,       0,  237738,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  246, 1460,  237764,  237782,  237807, 18, 25, 28, }, // Paraguay
    {  246, 1469,  237835,  237849,  237870, 14, 21, 23, }, // Peru
    {  246, 1474,  237893,  237913,  237940, 20, 27, 30, }, // Philippines
    {  246, 1486,       0,  237970,       0,  0, 25,  0, }, // Phoenix_Islands
    {  246, 1502,  237995,  238025,  238062, 30, 37, 35, }, // Pierre_Miquelon
    {  246, 1518,       0,  238097,       0,  0, 18,  0, }, // Pitcairn
    {  246, 1527,       0,  238115,       0,  0, 16,  0, }, // Ponape
    {  246, 1534,       0,  238131,       0,  0, 19,  0, }, // Pyongyang
    {  246, 1544,  238150,  238168,  238192, 18, 24, 28, }, // Qyzylorda
    {  246, 1554,       0,  238220,       0,  0, 17,  0, }, // Reunion
    {  246, 1562,       0,  238237,       0,  0, 17,  0, }, // Rothera
    {  246, 1570,  238254,  238272,  238297, 18, 25, 28, }, // Sakhalin
    {  246, 1579,  238325,  238340,  238361, 15, 21, 23, }, // Samara
    {  246, 1596,  238384,  238399,  238421, 15, 22, 20, }, // Samoa
    {  246, 1602,       0,  238441,       0,  0, 20,  0, }, // Seychelles
    {  246, 1624,       0,  238461,       0,  0, 26,  0, }, // Singapore
    {  246, 1634,       0,  238487,       0,  0, 25,  0, }, // Solomon
    {  246, 1642,       0,  238512,       0,  0, 23,  0, }, // South_Georgia
    {  246, 1656,       0,  238535,       0,  0, 17,  0, }, // Suriname
    {  246, 1676,       0,  238552,       0,  0, 15,  0, }, // Syowa
    {  246, 1682,       0,  238567,       0,  0, 16,  0, }, // Tahiti
    {  246, 1689,  238583,  238599,  238622, 16, 23, 21, }, // Taipei
    {  246, 1696,       0,  238643,       0,  0, 20,  0, }, // Tajikistan
    {  246, 1724,       0,  238663,       0,  0, 17,  0, }, // Tokelau
    {  246, 1732,  238680,  238695,  238717, 15, 22, 25, }, // Tonga
    {  246, 1738,       0,  238742,       0,  0, 15,  0, }, // Truk
    {  246, 1750,  238757,  238779,  238808, 22, 29, 32, }, // Turkmenistan
    {  246, 1763,       0,  238840,       0,  0, 16,  0, }, // Tuvalu
    {  246, 1777,  238856,  238873,  238897, 17, 24, 27, }, // Uruguay
    {  246, 1792,  238924,  238944,  238971, 20, 27, 30, }, // Uzbekistan
    {  246, 1803,  239001,  239018,  239042, 17, 24, 27, }, // Vanuatu
    {  246, 1811,       0,  239069,       0,  0, 19,  0, }, // Venezuela
    {  246, 1821,  239088,  239109,  239137, 21, 28, 31, }, // Vladivostok
    {  246, 1833,  239168,  239187,  239213, 19, 26, 28, }, // Volgograd
    {  246, 1843,       0,  239241,       0,  0, 16,  0, }, // Vostok
    {  246, 1850,       0,  239257,       0,  0, 21,  0, }, // Wake
    {  246, 1855,       0,  239278,       0,  0, 25,  0, }, // Wallis
    {  246, 1862,  239303,  239320,  239344, 17, 24, 27, }, // Yakutsk
    {  246, 1870,  239371,  239394,  239424, 23, 30, 33, }, // Yekaterinburg
    {  249,    5,       0,  239457,       0,  0, 14,  0, }, // Afghanistan Faroese/Latin/Faroe Islands
    {  249,   17,       0,  239471,       0,  0, 13,  0, }, // Africa_Central
    {  249,   32,       0,  239484,       0,  0, 16,  0, }, // Africa_Eastern
    {  249,   65,       0,  239500,       0,  0, 22,  0, }, // Africa_Southern
    {  249,   81,  239522,  239538,  239561, 16, 23, 22, }, // Africa_Western
    {  249,  107,  239583,  239593,  239610, 10, 17, 16, }, // Alaska
    {  249,  135,  239626,  239637,  239655, 11, 18, 17, }, // Amazon
    {  249,  142,  239672,  239683,  239701, 11, 18, 17, }, // America_Central
    {  249,  158,  239718,  239729,  239747, 11, 18, 17, }, // America_Eastern
    {  249,  174,  239764,  239776,  239795, 12, 19, 18, }, // America_Mountain
    {  249,  191,  239813,  239824,  239842, 11, 18, 17, }, // America_Pacific
    {  249,  214,  239859,  239867,  239882,  8, 15, 14, }, // Apia
    {  249,  232,  239896,  239907,  239925, 11, 18, 17, }, // Arabian
    {  249,  240,  239942,  239955,  239975, 13, 20, 19, }, // Argentina
    {  249,  250,  239994,  240014,  240041, 20, 27, 26, }, // Argentina_Western
    {  249,  268,  240067,  240078,  240096, 11, 18, 17, }, // Armenia
    {  249,  286,  240113,  240125,  240144, 12, 19, 18, }, // Atlantic
    {  249,  295,  240162,  240179,  240203, 17, 24, 23, }, // Australia_Central
    {  249,  313,  240226,  240249,  240279, 23, 30, 29, }, // Australia_CentralWestern
    {  249,  338,  240308,  240328,  240355, 20, 27, 26, }, // Australia_Eastern
    {  249,  356,  240229,  240252,  240282, 20, 27, 26, }, // Australia_Western
    {  249,  374,  240381,  240395,  240416, 14, 21, 20, }, // Azerbaijan
    {  249,  385,  240436,  240449,  240469, 13, 20, 19, }, // Azores
    {  249,  397,  240488,  240502,  240523, 14, 21, 20, }, // Bangladesh
    {  249,  415,       0,  240543,       0,  0,  9,  0, }, // Bhutan
    {  249,  422,       0,  240552,       0,  0, 11,  0, }, // Bolivia
    {  249,  437,  240563,  240575,  240594, 12, 19, 18, }, // Brasilia
    {  249,  454,       0,  240612,       0,  0, 21,  0, }, // Brunei
    {  249,  461,  240633,  240653,  240680, 20, 27, 26, }, // Cape_Verde
    {  249,  478,       0,  240706,       0,  0, 19,  0, }, // Chamorro
    {  249,  487,  240725,  240736,  240754, 11, 18, 17, }, // Chatham
    {  249,  495,  240771,  240779,  240794,  8, 15, 14, }, // Chile
    {  249,  501,  240808,  240816,  240831,  8, 15, 14, }, // China
    {  249,  507,       0,  240845,       0,  0, 13,  0, }, // Christmas
    {  249,  517,       0,  240858,       0,  0, 16,  0, }, // Cocos
    {  249,  523,  240874,  240886,  240905, 12, 19, 18, }, // Colombia
    {  249,  532,  240923,  240939,  240962, 16, 23, 22, }, // Cook
    {  249,  537,  240984,  240992,  241007,  8, 15, 14, }, // Cuba
    {  249,  548,       0,  241021,       0,  0,  9,  0, }, // Davis
    {  249,  564,       0,  241030,       0,  0, 20,  0, }, // DumontDUrville
    {  249,  601,       0,  241050,       0,  0, 15,  0, }, // East_Timor
    {  249,  612,  241065,  241080,  241102, 15, 22, 21, }, // Easter
    {  249,  619,       0,  241123,       0,  0, 11,  0, }, // Ecuador
    {  249,  627,  241134,  241147,  241167, 13, 20, 19, }, // Europe_Central
    {  249,  642,  241186,  241202,  241225, 16, 23, 22, }, // Europe_Eastern
    {  249,  657,       0,  241247,       0,  0, 23,  0, }, // Europe_Further_Eastern
    {  249,  680,  241270,  241286,  241309, 16, 23, 22, }, // Europe_Western
    {  249,  695,  241331,  241351,  241378, 20, 27, 26, }, // Falkland
    {  249,  704,  241404,  241412,  241427,  8, 15, 14, }, // Fiji
    {  249,  709,       0,  241441,       0,  0, 18,  0, }, // French_Guiana
    {  249,  723,       0,  241459,       0,  0, 41,  0, }, // French_Southern
    {  249,  746,       0,  241500,       0,  0, 13,  0, }, // Galapagos
    {  249,  756,       0,  241513,       0,  0, 11,  0, }, // Gambier
    {  249,  764,  241524,  241535,  241553, 11, 18, 17, }, // Georgia
    {  249,  772,       0,  241570,       0,  0, 18,  0, }, // Gilbert_Islands
    {  249,  788,       0,  241588,       0,  0, 18,  0, }, // GMT
    {  249,  830,  241606,  241627,  241655, 21, 28, 27, }, // Greenland_Eastern
    {  249,  848,  241682,  241703,  241731, 21, 28, 27, }, // Greenland_Western
    {  249,  871,       0,  241758,       0,  0, 15,  0, }, // Gulf
    {  249,  876,       0,  241449,       0,  0, 10,  0, }, // Guyana
    {  249,  883,  241773,  241792,  241818, 19, 26, 25, }, // Hawaii_Aleutian
    {  249,  899,  241843,  241856,  241876, 13, 20, 19, }, // Hong_Kong
    {  249,  909,  241895,  241903,  241918,  8, 15, 14, }, // Hovd
    {  249,  914,       0,  241932,       0,  0,  9,  0, }, // India
    {  249,  920,       0,  241941,       0,  0, 12,  0, }, // Indian_Ocean
    {  249,  933,       0,  241953,       0,  0, 12,  0, }, // Indochina
    {  249,  943,       0,  241965,       0,  0, 17,  0, }, // Indonesia_Central
    {  249,  961,       0,  241982,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  249,  979,       0,  242002,       0,  0, 20,  0, }, // Indonesia_Western
    {  249,  997,  242022,  242030,  242045,  8, 15, 14, }, // Iran
    {  249, 1008,  242059,  242070,  242088, 11, 18, 17, }, // Irkutsk
    {  249, 1016,  242105,  242115,  242132, 10, 17, 16, }, // Israel
    {  249, 1023,  242148,  242157,  242173,  9, 16, 15, }, // Japan
    {  249, 1047,       0,  242188,       0,  0, 13,  0, }, // Kazakhstan
    {  249, 1058,       0,  242201,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  249, 1077,       0,  242221,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  249, 1106,  242241,  242250,  242266,  9, 16, 15, }, // Korea
    {  249, 1112,       0,  242281,       0,  0, 10,  0, }, // Kosrae
    {  249, 1119,  242291,  242306,  242328, 15, 22, 21, }, // Krasnoyarsk
    {  249, 1151,       0,  242349,       0,  0, 12,  0, }, // Kyrgystan
    {  249, 1175,       0,  242361,       0,  0, 15,  0, }, // Line_Islands
    {  249, 1188,  242376,  242389,  242409, 13, 20, 19, }, // Lord_Howe
    {  249, 1204,  242428,  242439,  242457, 11, 18, 17, }, // Magadan
    {  249, 1219,       0,  242474,       0,  0, 12,  0, }, // Malaysia
    {  249, 1228,       0,  242486,       0,  0, 17,  0, }, // Maldives
    {  249, 1237,       0,  242503,       0,  0, 13,  0, }, // Marquesas
    {  249, 1247,       0,  242516,       0,  0, 19,  0, }, // Marshall_Islands
    {  249, 1264,  242535,  242547,  242566, 12, 19, 18, }, // Mauritius
    {  249, 1274,       0,  242584,       0,  0, 10,  0, }, // Mawson
    {  249, 1281,  242594,  242613,  242639, 19, 26, 25, }, // Mexico_Pacific
    {  249, 1296,  242664,  242678,  242699, 14, 21, 20, }, // Mongolia
    {  249, 1305,  242719,  242729,  242746, 10, 17, 16, }, // Moscow
    {  249, 1312,       0,  242762,       0,  0, 19,  0, }, // Myanmar
    {  249, 1320,       0,  242781,       0,  0,  9,  0, }, // Nauru
    {  249, 1326,       0,  242790,       0,  0,  9,  0, }, // Nepal
    {  249, 1332,  242799,  242814,  242836, 15, 22, 21, }, // New_Caledonia
    {  249, 1346,  242857,  242869,  242888, 12, 19, 18, }, // New_Zealand
    {  249, 1358,  242906,  242922,  242945, 16, 23, 22, }, // Newfoundland
    {  249, 1371,       0,  242967,       0,  0,  8,  0, }, // Niue
    {  249, 1376,  242975,  242991,  243014, 16, 23, 22, }, // Norfolk
    {  249, 1384,  243036,  243059,  243089, 23, 30, 29, }, // Noronha
    {  249, 1406,  243118,  243133,  243155, 15, 22, 21, }, // Novosibirsk
    {  249, 1418,  243176,  243184,  243199,  8, 15, 14, }, // Omsk
    {  249, 1428,  243213,  243225,  243244, 12, 19, 18, }, // Pakistan
    {  249, 1437,       0,  243262,       0,  0,  9,  0, }, // Palau
    {  249, 1443,       0,  243271,       0,  0, 18,  0, }, // Papua_New_Guinea
    {  249, 1460,  243289,  243301,  243320, 12, 19, 18, }, // Paraguay
    {  249, 1469,  243338,  243346,  243361,  8, 15, 14, }, // Peru
    {  249, 1474,  243375,  243392,  243416, 17, 24, 23, }, // Philippines
    {  249, 1486,       0,  243439,       0,  0, 18,  0, }, // Phoenix_Islands
    {  249, 1502,  243457,  243482,  243514, 25, 32, 31, }, // Pierre_Miquelon
    {  249, 1518,       0,  243545,       0,  0, 19,  0, }, // Pitcairn
    {  249, 1527,       0,  243564,       0,  0, 10,  0, }, // Ponape
    {  249, 1534,       0,  243574,       0,  0, 13,  0, }, // Pyongyang
    {  249, 1554,       0,  243587,       0,  0, 11,  0, }, // Reunion
    {  249, 1562,       0,  243598,       0,  0, 11,  0, }, // Rothera
    {  249, 1570,  243609,  243621,  243640, 12, 19, 18, }, // Sakhalin
    {  249, 1596,  243658,  243667,  243683,  9, 16, 15, }, // Samoa
    {  249, 1602,       0,  243698,       0,  0, 19,  0, }, // Seychelles
    {  249, 1624,       0,  243717,       0,  0, 12,  0, }, // Singapore
    {  249, 1634,       0,  243729,       0,  0, 18,  0, }, // Solomon
    {  249, 1642,       0,  243747,       0,  0, 23,  0, }, // South_Georgia
    {  249, 1656,       0,  243770,       0,  0, 11,  0, }, // Suriname
    {  249, 1676,       0,  243781,       0,  0,  9,  0, }, // Syowa
    {  249, 1682,       0,  243790,       0,  0, 10,  0, }, // Tahiti
    {  249, 1689,  243800,  243810,  243827, 10, 17, 16, }, // Taipei
    {  249, 1696,       0,  243843,       0,  0, 16,  0, }, // Tajikistan
    {  249, 1724,       0,  243859,       0,  0, 11,  0, }, // Tokelau
    {  249, 1732,  243870,  243879,  243895,  9, 16, 15, }, // Tonga
    {  249, 1738,       0,  243910,       0,  0,  9,  0, }, // Truk
    {  249, 1750,  243919,  243935,  243958, 16, 23, 22, }, // Turkmenistan
    {  249, 1763,       0,  243980,       0,  0, 10,  0, }, // Tuvalu
    {  249, 1777,  243990,  244001,  244019, 11, 18, 17, }, // Uruguay
    {  249, 1792,  244036,  244050,  244071, 14, 21, 20, }, // Uzbekistan
    {  249, 1803,  244091,  244102,  244120, 11, 18, 17, }, // Vanuatu
    {  249, 1811,       0,  244137,       0,  0, 13,  0, }, // Venezuela
    {  249, 1821,  244150,  244165,  244187, 15, 22, 21, }, // Vladivostok
    {  249, 1833,  244208,  244221,  244241, 13, 20, 19, }, // Volgograd
    {  249, 1843,       0,  244260,       0,  0, 10,  0, }, // Vostok
    {  249, 1850,       0,  244270,       0,  0, 13,  0, }, // Wake
    {  249, 1855,       0,  244283,       0,  0, 28,  0, }, // Wallis
    {  249, 1862,  244311,  244322,  244340, 11, 18, 17, }, // Yakutsk
    {  249, 1870,  244357,  244374,  244398, 17, 24, 23, }, // Yekaterinburg
    {  249, 1892,       0,  244421,       0,  0,  9,  0, }, // Yukon
    {  251,    5,       0,  144338,       0,  0, 19,  0, }, // Afghanistan Filipino/Latin/Philippines
    {  251,   17,       0,  244430,       0,  0, 22,  0, }, // Africa_Central
    {  251,   32,       0,  244452,       0,  0, 24,  0, }, // Africa_Eastern
    {  251,   65,       0,  244476,       0,  0, 20,  0, }, // Africa_Southern
    {  251,   81,  244496,  244520,  244556, 24, 36, 36, }, // Africa_Western
    {  251,  107,  144514,  244592,  244618, 14, 26, 23, }, // Alaska
    {  251,  135,  144582,  244641,  244667, 14, 26, 26, }, // Amazon
    {  251,  142,  244693,  244725,  244769, 32, 44, 44, }, // America_Central
    {  251,  158,  244813,  244849,  244898, 36, 49, 48, }, // America_Eastern
    {  251,  174,  244946,  244980,  245026, 34, 46, 46, }, // America_Mountain
    {  251,  191,  245072,  245108,  245156, 36, 48, 48, }, // America_Pacific
    {  251,  207,  245204,  245218,  245241, 14, 23, 21, }, // Anadyr
    {  251,  214,  144724,  245262,  245286, 12, 24, 21, }, // Apia
    {  251,  232,  144786,  245307,  245333, 14, 26, 23, }, // Arabian
    {  251,  240,  144854,  245356,  245385, 17, 29, 29, }, // Argentina
    {  251,  250,  245414,  245441,  245480, 27, 39, 39, }, // Argentina_Western
    {  251,  268,  145032,  245519,  245546, 15, 27, 27, }, // Armenia
    {  251,  286,  245573,  245590,  245619, 17, 29, 29, }, // Atlantic
    {  251,  295,  245648,  245673,  245710, 25, 37, 34, }, // Australia_Central
    {  251,  313,  245744,  245781,  245827, 37, 46, 44, }, // Australia_CentralWestern
    {  251,  338,  245871,  245898,  245937, 27, 39, 36, }, // Australia_Eastern
    {  251,  356,  245973,  246000,  246039, 27, 39, 36, }, // Australia_Western
    {  251,  374,  145605,  246075,  246105, 18, 30, 30, }, // Azerbaijan
    {  251,  385,  145685,  246135,  246161, 14, 26, 26, }, // Azores
    {  251,  397,  145753,  246187,  246217, 18, 30, 30, }, // Bangladesh
    {  251,  415,       0,  145833,       0,  0, 14,  0, }, // Bhutan
    {  251,  422,       0,  145847,       0,  0, 15,  0, }, // Bolivia
    {  251,  437,  145862,  246247,  246275, 16, 28, 28, }, // Brasilia
    {  251,  454,       0,  145936,       0,  0, 25,  0, }, // Brunei
    {  251,  461,  145961,  246303,  246333, 18, 30, 30, }, // Cape_Verde
    {  251,  478,       0,  246363,       0,  0, 28,  0, }, // Chamorro
    {  251,  487,  146070,  246391,  246418, 15, 27, 24, }, // Chatham
    {  251,  495,  146141,  246442,  246467, 13, 25, 25, }, // Chile
    {  251,  501,  146206,  246492,  246517, 13, 25, 22, }, // China
    {  251,  507,       0,  146271,       0,  0, 24,  0, }, // Christmas
    {  251,  517,       0,  146295,       0,  0, 21,  0, }, // Cocos
    {  251,  523,  146316,  246539,  246567, 16, 28, 28, }, // Colombia
    {  251,  532,  146390,  246595,  246627, 20, 32, 44, }, // Cook
    {  251,  537,  146486,  246671,  246695, 12, 24, 24, }, // Cuba
    {  251,  548,       0,  146548,       0,  0, 13,  0, }, // Davis
    {  251,  564,       0,  146561,       0,  0, 24,  0, }, // DumontDUrville
    {  251,  601,       0,  146585,       0,  0, 18,  0, }, // East_Timor
    {  251,  612,  146603,  246719,  246752, 21, 33, 33, }, // Easter
    {  251,  619,       0,  146692,       0,  0, 15,  0, }, // Ecuador
    {  251,  627,  246785,  246807,  246841, 22, 34, 34, }, // Europe_Central
    {  251,  642,  246875,  246899,  246935, 24, 36, 36, }, // Europe_Eastern
    {  251,  657,       0,  246971,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  251,  680,  247002,  247026,  247062, 24, 36, 36, }, // Europe_Western
    {  251,  695,  147013,  247098,  247134, 24, 36, 36, }, // Falkland
    {  251,  704,  147111,  247170,  247194, 12, 24, 24, }, // Fiji
    {  251,  709,       0,  147173,       0,  0, 21,  0, }, // French_Guiana
    {  251,  723,       0,  247218,       0,  0, 38,  0, }, // French_Southern
    {  251,  746,       0,  147230,       0,  0, 17,  0, }, // Galapagos
    {  251,  756,       0,  147247,       0,  0, 15,  0, }, // Gambier
    {  251,  764,  147262,  247256,  247283, 15, 27, 27, }, // Georgia
    {  251,  772,       0,  147333,       0,  0, 23,  0, }, // Gilbert_Islands
    {  251,  788,       0,    2791,       0,  0, 19,  0, }, // GMT
    {  251,  830,  247310,  247337,  247376, 27, 39, 39, }, // Greenland_Eastern
    {  251,  848,  247415,  247442,  247481, 27, 39, 39, }, // Greenland_Western
    {  251,  871,       0,  147575,       0,  0, 12,  0, }, // Gulf
    {  251,  876,       0,  147587,       0,  0, 14,  0, }, // Guyana
    {  251,  883,  147601,  247520,  247555, 23, 35, 35, }, // Hawaii_Aleutian
    {  251,  899,  147696,  247590,  247619, 17, 29, 29, }, // Hong_Kong
    {  251,  909,  147773,  247648,  247672, 12, 24, 24, }, // Hovd
    {  251,  914,       0,  247696,       0,  0, 25,  0, }, // India
    {  251,  920,       0,  147861,       0,  0, 20,  0, }, // Indian_Ocean
    {  251,  933,       0,  147881,       0,  0, 17,  0, }, // Indochina
    {  251,  943,       0,  247721,       0,  0, 25,  0, }, // Indonesia_Central
    {  251,  961,       0,  247746,       0,  0, 27,  0, }, // Indonesia_Eastern
    {  251,  979,       0,  247773,       0,  0, 27,  0, }, // Indonesia_Western
    {  251,  997,  147973,  247800,  247824, 12, 24, 21, }, // Iran
    {  251, 1008,  148035,  247845,  247872, 15, 27, 27, }, // Irkutsk
    {  251, 1016,  148106,  247899,  247925, 14, 26, 23, }, // Israel
    {  251, 1023,  148174,  247948,  247973, 13, 25, 22, }, // Japan
    {  251, 1029,  247995,  248027,  248068, 32, 41, 39, }, // Kamchatka
    {  251, 1047,       0,  248107,       0,  0, 18,  0, }, // Kazakhstan
    {  251, 1058,       0,  248125,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  251, 1077,       0,  248153,       0,  0, 28,  0, }, // Kazakhstan_Western
    {  251, 1106,  148303,  248181,  248206, 13, 25, 22, }, // Korea
    {  251, 1112,       0,  148368,       0,  0, 14,  0, }, // Kosrae
    {  251, 1119,  148382,  248228,  248259, 19, 31, 31, }, // Krasnoyarsk
    {  251, 1151,       0,  248290,       0,  0, 17,  0, }, // Kyrgystan
    {  251, 1175,       0,  148483,       0,  0, 20,  0, }, // Line_Islands
    {  251, 1188,  148503,  248307,  248336, 17, 29, 27, }, // Lord_Howe
    {  251, 1204,  148580,  248363,  248390, 15, 27, 27, }, // Magadan
    {  251, 1219,       0,  148651,       0,  0, 16,  0, }, // Malaysia
    {  251, 1228,       0,  148667,       0,  0, 16,  0, }, // Maldives
    {  251, 1237,       0,  148683,       0,  0, 17,  0, }, // Marquesas
    {  251, 1247,       0,  148700,       0,  0, 24,  0, }, // Marshall_Islands
    {  251, 1264,  148724,  248417,  248446, 17, 29, 29, }, // Mauritius
    {  251, 1274,       0,  148801,       0,  0, 14,  0, }, // Mawson
    {  251, 1281,  248475,  248501,  248539, 26, 38, 38, }, // Mexico_Pacific
    {  251, 1296,  248577,  248595,  248625, 18, 30, 30, }, // Mongolia
    {  251, 1305,  148993,  248655,  248681, 14, 26, 26, }, // Moscow
    {  251, 1312,       0,  149061,       0,  0, 15,  0, }, // Myanmar
    {  251, 1320,       0,  149076,       0,  0, 13,  0, }, // Nauru
    {  251, 1326,       0,  149089,       0,  0, 13,  0, }, // Nepal
    {  251, 1332,  149102,  248707,  248740, 21, 33, 33, }, // New_Caledonia
    {  251, 1346,  149191,  248773,  248804, 19, 31, 28, }, // New_Zealand
    {  251, 1358,  149274,  248832,  248864, 20, 32, 32, }, // Newfoundland
    {  251, 1371,       0,  149360,       0,  0, 12,  0, }, // Niue
    {  251, 1376,  149372,  248896,  248930, 22, 34, 31, }, // Norfolk
    {  251, 1384,  149464,  248961,  249000, 27, 39, 39, }, // Noronha
    {  251, 1406,  149571,  249039,  249070, 19, 31, 31, }, // Novosibirsk
    {  251, 1418,  149654,  249101,  249125, 12, 24, 24, }, // Omsk
    {  251, 1428,  149716,  249149,  249177, 16, 28, 28, }, // Pakistan
    {  251, 1437,       0,  149790,       0,  0, 13,  0, }, // Palau
    {  251, 1443,       0,  149803,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  251, 1460,  149827,  249205,  249233, 16, 28, 28, }, // Paraguay
    {  251, 1469,  149901,  249261,  249285, 12, 24, 24, }, // Peru
    {  251, 1474,  149963,  249309,  249338, 17, 29, 29, }, // Philippines
    {  251, 1486,       0,  150040,       0,  0, 23,  0, }, // Phoenix_Islands
    {  251, 1502,  249367,  249398,  249441, 31, 43, 43, }, // Pierre_Miquelon
    {  251, 1518,       0,  150176,       0,  0, 16,  0, }, // Pitcairn
    {  251, 1527,       0,  150192,       0,  0, 14,  0, }, // Ponape
    {  251, 1534,       0,  150206,       0,  0, 17,  0, }, // Pyongyang
    {  251, 1554,       0,  150223,       0,  0, 15,  0, }, // Reunion
    {  251, 1562,       0,  150238,       0,  0, 15,  0, }, // Rothera
    {  251, 1570,  150253,  249484,  249512, 16, 28, 28, }, // Sakhalin
    {  251, 1579,  249540,  249554,  249577, 14, 23, 15, }, // Samara
    {  251, 1596,  150327,  249592,  249617, 13, 25, 22, }, // Samoa
    {  251, 1602,       0,  150392,       0,  0, 18,  0, }, // Seychelles
    {  251, 1624,       0,  249639,       0,  0, 29,  0, }, // Singapore
    {  251, 1634,       0,  150440,       0,  0, 23,  0, }, // Solomon
    {  251, 1642,       0,  249668,       0,  0, 21,  0, }, // South_Georgia
    {  251, 1656,       0,  150484,       0,  0, 16,  0, }, // Suriname
    {  251, 1676,       0,  150500,       0,  0, 13,  0, }, // Syowa
    {  251, 1682,       0,  150513,       0,  0, 14,  0, }, // Tahiti
    {  251, 1689,  150527,  249689,  249715, 14, 26, 23, }, // Taipei
    {  251, 1696,       0,  150595,       0,  0, 18,  0, }, // Tajikistan
    {  251, 1724,       0,  150613,       0,  0, 15,  0, }, // Tokelau
    {  251, 1732,  150628,  249738,  249763, 13, 25, 25, }, // Tonga
    {  251, 1738,       0,  150693,       0,  0, 13,  0, }, // Truk
    {  251, 1750,  150706,  249788,  249820, 20, 32, 32, }, // Turkmenistan
    {  251, 1763,       0,  150792,       0,  0, 14,  0, }, // Tuvalu
    {  251, 1777,  150806,  249852,  249879, 15, 27, 27, }, // Uruguay
    {  251, 1792,  150877,  249906,  249936, 18, 30, 30, }, // Uzbekistan
    {  251, 1803,  150957,  249966,  249993, 15, 27, 27, }, // Vanuatu
    {  251, 1811,       0,  151028,       0,  0, 17,  0, }, // Venezuela
    {  251, 1821,  151045,  250020,  250051, 19, 31, 31, }, // Vladivostok
    {  251, 1833,  151128,  250082,  250111, 17, 29, 29, }, // Volgograd
    {  251, 1843,       0,  151205,       0,  0, 14,  0, }, // Vostok
    {  251, 1850,       0,  151219,       0,  0, 19,  0, }, // Wake
    {  251, 1855,       0,  151238,       0,  0, 23,  0, }, // Wallis
    {  251, 1862,  151261,  250140,  250167, 15, 27, 27, }, // Yakutsk
    {  251, 1870,  151332,  250194,  250227, 21, 33, 33, }, // Yekaterinburg
    {  251, 1892,       0,  151421,       0,  0, 13,  0, }, // Yukon
    {  252,    0,  250260,  250270,  250288, 10, 18, 14, }, // Acre Finnish/Latin/Finland
    {  252,    5,       0,  250302,       0,  0, 17,  0, }, // Afghanistan
    {  252,   17,       0,  250319,       0,  0, 18,  0, }, // Africa_Central
    {  252,   32,       0,  250337,       0,  0, 16,  0, }, // Africa_Eastern
    {  252,   65,       0,  250353,       0,  0, 18,  0, }, // Africa_Southern
    {  252,   81,  250371,  250389,  250415, 18, 26, 22, }, // Africa_Western
    {  252,  107,  250437,  250449,  250469, 12, 20, 16, }, // Alaska
    {  252,  128,  250485,  250497,  250517, 12, 20, 16, }, // Almaty
    {  252,  135,  250533,  250546,  250567, 13, 21, 17, }, // Amazon
    {  252,  142,  250584,  250610,  250644, 26, 34, 30, }, // America_Central
    {  252,  158,  250674,  250699,  250732, 25, 33, 29, }, // America_Eastern
    {  252,  174,  250761,  250779,  250805, 18, 26, 22, }, // America_Mountain
    {  252,  191,  250827,  250856,  250893, 29, 37, 33, }, // America_Pacific
    {  252,  207,  250926,  250939,  250960, 13, 21, 17, }, // Anadyr
    {  252,  214,  250977,  250987,  251005, 10, 18, 14, }, // Apia
    {  252,  219,  251019,  251031,  251051, 12, 20, 16, }, // Aqtau
    {  252,  225,  251067,  251079,  251099, 12, 20, 16, }, // Aqtobe
    {  252,  232,  251115,  251133,  251159, 18, 26, 22, }, // Arabian
    {  252,  240,  251181,  251197,  251221, 16, 24, 20, }, // Argentina
    {  252,  250,  251241,  251263,  251293, 22, 30, 26, }, // Argentina_Western
    {  252,  268,  251319,  251332,  251353, 13, 21, 17, }, // Armenia
    {  252,  286,  251370,  251391,  251420, 21, 29, 25, }, // Atlantic
    {  252,  295,  251445,  251466,  251495, 21, 29, 25, }, // Australia_Central
    {  252,  313,  251520,  251550,  251588, 30, 38, 34, }, // Australia_CentralWestern
    {  252,  338,  251622,  251641,  251668, 19, 27, 23, }, // Australia_Eastern
    {  252,  356,  251691,  251712,  251741, 21, 29, 25, }, // Australia_Western
    {  252,  374,  251766,  251784,  251810, 18, 26, 22, }, // Azerbaijan
    {  252,  385,  251832,  251844,  251864, 12, 20, 16, }, // Azores
    {  252,  397,  251880,  251897,  251922, 17, 25, 21, }, // Bangladesh
    {  252,  415,       0,  251943,       0,  0, 13,  0, }, // Bhutan
    {  252,  422,       0,  251956,       0,  0, 13,  0, }, // Bolivia
    {  252,  437,  251969,  251983,  252005, 14, 22, 18, }, // Brasilia
    {  252,  454,       0,  252023,       0,  0, 12,  0, }, // Brunei
    {  252,  461,  252035,  252050,  252073, 15, 23, 19, }, // Cape_Verde
    {  252,  472,       0,  252092,       0,  0, 11,  0, }, // Casey
    {  252,  478,       0,  252103,       0,  0, 14,  0, }, // Chamorro
    {  252,  487,  252117,  252131,  252153, 14, 22, 18, }, // Chatham
    {  252,  495,  252171,  252182,  252201, 11, 19, 15, }, // Chile
    {  252,  501,  252216,  252227,  252246, 11, 19, 15, }, // China
    {  252,  507,       0,  252261,       0,  0, 16,  0, }, // Christmas
    {  252,  517,       0,  252277,       0,  0, 18,  0, }, // Cocos
    {  252,  523,  252295,  252309,  252331, 14, 22, 18, }, // Colombia
    {  252,  532,  252349,  252367,  252393, 18, 26, 22, }, // Cook
    {  252,  537,  252415,  252426,  252445, 11, 19, 15, }, // Cuba
    {  252,  548,       0,  252460,       0,  0, 12,  0, }, // Davis
    {  252,  564,       0,  252472,       0,  0, 22,  0, }, // DumontDUrville
    {  252,  601,       0,  252494,       0,  0, 16,  0, }, // East_Timor
    {  252,  612,  252510,  252529,  252556, 19, 27, 23, }, // Easter
    {  252,  619,       0,  252579,       0,  0, 14,  0, }, // Ecuador
    {  252,  627,  252593,  252612,  252639, 19, 27, 23, }, // Europe_Central
    {  252,  642,  252662,  252679,  252704, 17, 25, 21, }, // Europe_Eastern
    {  252,  657,       0,  252725,       0,  0, 24,  0, }, // Europe_Further_Eastern
    {  252,  680,  252749,  252768,  252795, 19, 27, 23, }, // Europe_Western
    {  252,  695,  252818,  252840,  252870, 22, 30, 26, }, // Falkland
    {  252,  704,  252896,  252907,  252926, 11, 19, 15, }, // Fiji
    {  252,  709,       0,  252941,       0,  0, 21,  0, }, // French_Guiana
    {  252,  723,       0,  252962,       0,  0, 48,  0, }, // French_Southern
    {  252,  746,       0,  253010,       0,  0, 21,  0, }, // Galapagos
    {  252,  756,       0,  253031,       0,  0, 19,  0, }, // Gambier
    {  252,  764,  253050,  253063,  253084, 13, 21, 17, }, // Georgia
    {  252,  772,       0,  253101,       0,  0, 19,  0, }, // Gilbert_Islands
    {  252,  788,       0,  253120,       0,  0, 24,  0, }, // GMT
    {  252,  830,  253144,  253163,  253190, 19, 27, 23, }, // Greenland_Eastern
    {  252,  848,  253213,  253234,  253263, 21, 29, 25, }, // Greenland_Western
    {  252,  866,       0,  253288,       0,  0, 11,  0, }, // Guam
    {  252,  871,       0,  253299,       0,  0, 31,  0, }, // Gulf
    {  252,  876,       0,  253330,       0,  0, 12,  0, }, // Guyana
    {  252,  883,  253342,  253365,  253396, 23, 31, 27, }, // Hawaii_Aleutian
    {  252,  899,  253423,  253438,  253461, 15, 23, 19, }, // Hong_Kong
    {  252,  909,  253480,  253491,  253510, 11, 19, 15, }, // Hovd
    {  252,  914,       0,  253525,       0,  0, 11,  0, }, // India
    {  252,  920,       0,  253536,       0,  0, 22,  0, }, // Indian_Ocean
    {  252,  933,       0,  253558,       0,  0, 15,  0, }, // Indochina
    {  252,  943,       0,  253573,       0,  0, 21,  0, }, // Indonesia_Central
    {  252,  961,       0,  253594,       0,  0, 19,  0, }, // Indonesia_Eastern
    {  252,  979,       0,  253613,       0,  0, 21,  0, }, // Indonesia_Western
    {  252,  997,  253634,  253645,  253664, 11, 19, 15, }, // Iran
    {  252, 1008,  253679,  253693,  253715, 14, 22, 18, }, // Irkutsk
    {  252, 1016,  253733,  253746,  253767, 13, 21, 17, }, // Israel
    {  252, 1023,  253784,  253796,  253816, 12, 20, 16, }, // Japan
    {  252, 1029,  253832,  253847,  253870, 15, 23, 19, }, // Kamchatka
    {  252, 1047,       0,  253889,       0,  0, 16,  0, }, // Kazakhstan
    {  252, 1058,       0,  253905,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  252, 1077,       0,  253925,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  252, 1106,  253947,  253958,  253977, 11, 19, 15, }, // Korea
    {  252, 1112,       0,  253992,       0,  0, 12,  0, }, // Kosrae
    {  252, 1119,  254004,  254022,  254048, 18, 26, 22, }, // Krasnoyarsk
    {  252, 1151,       0,  254070,       0,  0, 14,  0, }, // Kyrgystan
    {  252, 1161,       0,  254084,       0,  0, 15,  0, }, // Lanka
    {  252, 1175,       0,  254099,       0,  0, 16,  0, }, // Line_Islands
    {  252, 1188,  254115,  254130,  254153, 15, 23, 19, }, // Lord_Howe
    {  252, 1198,  254172,  254183,  254202, 11, 19, 15, }, // Macau
    {  252, 1204,  254217,  254231,  254253, 14, 22, 18, }, // Magadan
    {  252, 1219,       0,  254271,       0,  0, 13,  0, }, // Malaysia
    {  252, 1228,       0,  254284,       0,  0, 16,  0, }, // Maldives
    {  252, 1237,       0,  254300,       0,  0, 21,  0, }, // Marquesas
    {  252, 1247,       0,  254321,       0,  0, 22,  0, }, // Marshall_Islands
    {  252, 1264,  254343,  254360,  254385, 17, 25, 21, }, // Mauritius
    {  252, 1274,       0,  254406,       0,  0, 13,  0, }, // Mawson
    {  252, 1281,  254419,  254444,  254477, 25, 33, 29, }, // Mexico_Pacific
    {  252, 1296,  254506,  254523,  254548, 17, 25, 21, }, // Mongolia
    {  252, 1305,  254569,  254582,  254603, 13, 21, 17, }, // Moscow
    {  252, 1312,       0,  254620,       0,  0, 14,  0, }, // Myanmar
    {  252, 1320,       0,  254634,       0,  0, 11,  0, }, // Nauru
    {  252, 1326,       0,  254645,       0,  0, 12,  0, }, // Nepal
    {  252, 1332,  254657,  254678,  254707, 21, 29, 25, }, // New_Caledonia
    {  252, 1346,  254732,  254752,  254780, 20, 28, 24, }, // New_Zealand
    {  252, 1358,  254804,  254823,  254850, 19, 27, 23, }, // Newfoundland
    {  252, 1371,       0,  254873,       0,  0, 10,  0, }, // Niue
    {  252, 1376,  254883,  254903,  254931, 20, 28, 24, }, // Norfolk
    {  252, 1384,  254955,  254980,  255013, 25, 33, 29, }, // Noronha
    {  252, 1392,       0,  255042,       0,  0, 23,  0, }, // North_Mariana
    {  252, 1406,  255065,  255083,  255109, 18, 26, 22, }, // Novosibirsk
    {  252, 1418,  255131,  255142,  255161, 11, 19, 15, }, // Omsk
    {  252, 1428,  255176,  255191,  255214, 15, 23, 19, }, // Pakistan
    {  252, 1437,       0,  255233,       0,  0, 11,  0, }, // Palau
    {  252, 1443,       0,  255244,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  252, 1460,  255268,  255282,  255304, 14, 22, 18, }, // Paraguay
    {  252, 1469,  255322,  255332,  255350, 10, 18, 14, }, // Peru
    {  252, 1474,  255364,  255381,  255406, 17, 25, 21, }, // Philippines
    {  252, 1486,       0,  255427,       0,  0, 19,  0, }, // Phoenix_Islands
    {  252, 1502,  255446,  255478,  255518, 32, 40, 36, }, // Pierre_Miquelon
    {  252, 1518,       0,  255554,       0,  0, 15,  0, }, // Pitcairn
    {  252, 1527,       0,  255569,       0,  0, 13,  0, }, // Ponape
    {  252, 1534,       0,  255582,       0,  0, 16,  0, }, // Pyongyang
    {  252, 1544,  255598,  255613,  255636, 15, 23, 19, }, // Qyzylorda
    {  252, 1554,       0,  255655,       0,  0, 14,  0, }, // Reunion
    {  252, 1562,       0,  255669,       0,  0, 13,  0, }, // Rothera
    {  252, 1570,  255682,  255696,  255718, 14, 22, 18, }, // Sakhalin
    {  252, 1579,  255736,  255748,  255768, 12, 20, 16, }, // Samara
    {  252, 1596,  255784,  255795,  255814, 11, 19, 15, }, // Samoa
    {  252, 1602,       0,  255829,       0,  0, 16,  0, }, // Seychelles
    {  252, 1624,       0,  255845,       0,  0, 15,  0, }, // Singapore
    {  252, 1634,       0,  255860,       0,  0, 19,  0, }, // Solomon
    {  252, 1642,       0,  255879,       0,  0, 19,  0, }, // South_Georgia
    {  252, 1656,       0,  255898,       0,  0, 14,  0, }, // Suriname
    {  252, 1676,       0,  255912,       0,  0, 11,  0, }, // Syowa
    {  252, 1682,       0,  255923,       0,  0, 12,  0, }, // Tahiti
    {  252, 1689,  255935,  255947,  255967, 12, 20, 16, }, // Taipei
    {  252, 1696,       0,  255983,       0,  0, 18,  0, }, // Tajikistan
    {  252, 1724,       0,  256001,       0,  0, 13,  0, }, // Tokelau
    {  252, 1732,  256014,  256025,  256044, 11, 19, 15, }, // Tonga
    {  252, 1738,       0,  256059,       0,  0, 12,  0, }, // Truk
    {  252, 1750,  256071,  256090,  256117, 19, 27, 23, }, // Turkmenistan
    {  252, 1763,       0,  256140,       0,  0, 12,  0, }, // Tuvalu
    {  252, 1777,  256152,  256165,  256186, 13, 21, 17, }, // Uruguay
    {  252, 1792,  256203,  256220,  256245, 17, 25, 21, }, // Uzbekistan
    {  252, 1803,  256266,  256279,  256300, 13, 21, 17, }, // Vanuatu
    {  252, 1811,       0,  256317,       0,  0, 15,  0, }, // Venezuela
    {  252, 1821,  256332,  256350,  256376, 18, 26, 22, }, // Vladivostok
    {  252, 1833,  256398,  256414,  256438, 16, 24, 20, }, // Volgograd
    {  252, 1843,       0,  256458,       0,  0, 13,  0, }, // Vostok
    {  252, 1850,       0,  256471,       0,  0, 10,  0, }, // Wake
    {  252, 1855,       0,  256481,       0,  0, 24,  0, }, // Wallis
    {  252, 1862,  256505,  256519,  256541, 14, 22, 18, }, // Yakutsk
    {  252, 1870,  256559,  256579,  256607, 20, 28, 24, }, // Yekaterinburg
    {  252, 1892,       0,  256631,       0,  0, 12,  0, }, // Yukon
    {  253,    0,  256643,  256658,  256681, 15, 23, 21, }, // Acre French/Latin/France
    {  253,    5,       0,  256702,       0,  0, 22,  0, }, // Afghanistan
    {  253,   17,       0,  256724,       0,  0, 32,  0, }, // Africa_Central
    {  253,   32,       0,  256756,       0,  0, 32,  0, }, // Africa_Eastern
    {  253,   65,       0,  256788,       0,  0, 35,  0, }, // Africa_Southern
    {  253,   81,  256823,  256849,  256883, 26, 34, 32, }, // Africa_Western
    {  253,  107,  256915,  256932,  256957, 17, 25, 23, }, // Alaska
    {  253,  128,  256980,  256996,  257020, 16, 24, 22, }, // Almaty
    {  253,  135,  257042,  257061,  257088, 19, 27, 25, }, // Amazon
    {  253,  142,  257113,  257143,  257181, 30, 38, 36, }, // America_Central
    {  253,  158,  257217,  257246,  257283, 29, 37, 35, }, // America_Eastern
    {  253,  174,  257318,  257337,  257364, 19, 27, 25, }, // America_Mountain
    {  253,  191,  257389,  257422,  257463, 33, 41, 39, }, // America_Pacific
    {  253,  207,  257502,  257516,  257538, 14, 22, 20, }, // Anadyr
    {  253,  214,  257558,  257570,  257590, 12, 20, 18, }, // Apia
    {  253,  219,  257608,  257622,  257644, 14, 22, 20, }, // Aqtau
    {  253,  225,  257664,  257678,  257700, 14, 22, 20, }, // Aqtobe
    {  253,  232,  257720,  257737,  257762, 17, 25, 23, }, // Arabian
    {  253,  240,  257785,  257805,  257830, 20, 25, 26, }, // Argentina
    {  253,  250,  257856,  257881,  257914, 25, 33, 31, }, // Argentina_Western
    {  253,  268,  257945,  257963,  257989, 18, 26, 21, }, // Armenia
    {  253,  286,  258010,  258031,  258060, 21, 29, 27, }, // Atlantic
    {  253,  295,  258087,  258117,  258155, 30, 38, 36, }, // Australia_Central
    {  253,  313,  258191,  258227,  258271, 36, 44, 42, }, // Australia_CentralWestern
    {  253,  338,  258313,  258342,  258379, 29, 37, 35, }, // Australia_Eastern
    {  253,  356,  258414,  258445,  258484, 31, 39, 37, }, // Australia_Western
    {  253,  374,  258521,  258543,  258573, 22, 30, 25, }, // Azerbaijan
    {  253,  385,  258598,  258614,  258638, 16, 24, 22, }, // Azores
    {  253,  397,  258660,  258679,  258706, 19, 27, 25, }, // Bangladesh
    {  253,  415,       0,  258731,       0,  0, 16,  0, }, // Bhutan
    {  253,  422,       0,  258747,       0,  0, 16,  0, }, // Bolivia
    {  253,  437,  258763,  258780,  258805, 17, 25, 23, }, // Brasilia
    {  253,  454,       0,  258828,       0,  0, 15,  0, }, // Brunei
    {  253,  461,  258843,  258860,  258885, 17, 25, 23, }, // Cape_Verde
    {  253,  472,       0,  258908,       0,  0, 14,  0, }, // Casey
    {  253,  478,       0,  258922,       0,  0, 18,  0, }, // Chamorro
    {  253,  487,  258940,  258962,  258992, 22, 30, 28, }, // Chatham
    {  253,  495,  259020,  259034,  259056, 14, 22, 20, }, // Chile
    {  253,  501,  259076,  259093,  259118, 17, 25, 20, }, // China
    {  253,  507,       0,  259138,       0,  0, 24,  0, }, // Christmas
    {  253,  517,       0,  259162,       0,  0, 20,  0, }, // Cocos
    {  253,  523,  259182,  259199,  259224, 17, 25, 23, }, // Colombia
    {  253,  532,  259247,  259266,  259293, 19, 27, 25, }, // Cook
    {  253,  537,  259318,  259331,  259352, 13, 21, 19, }, // Cuba
    {  253,  548,       0,  259371,       0,  0, 14,  0, }, // Davis
    {  253,  564,       0,  259385,       0,  0, 25,  0, }, // DumontDUrville
    {  253,  601,       0,  259410,       0,  0, 23,  0, }, // East_Timor
    {  253,  612,  259433,  259457,  259489, 24, 32, 30, }, // Easter
    {  253,  619,       0,  259519,       0,  0, 19,  0, }, // Ecuador
    {  253,  627,  259538,  259561,  259592, 23, 31, 29, }, // Europe_Central
    {  253,  642,  259621,  259644,  259675, 23, 31, 29, }, // Europe_Eastern
    {  253,  657,       0,  259704,       0,  0, 20,  0, }, // Europe_Further_Eastern
    {  253,  680,  259724,  259749,  259782, 25, 33, 31, }, // Europe_Western
    {  253,  695,  259813,  259837,  259869, 24, 32, 30, }, // Falkland
    {  253,  704,  259899,  259919,  259947, 20, 28, 26, }, // Fiji
    {  253,  709,       0,  259973,       0,  0, 28,  0, }, // French_Guiana
    {  253,  723,       0,  260001,       0,  0, 53,  0, }, // French_Southern
    {  253,  746,       0,  260054,       0,  0, 24,  0, }, // Galapagos
    {  253,  756,       0,  260078,       0,  0, 22,  0, }, // Gambier
    {  253,  764,  260100,  260119,  260146, 19, 27, 22, }, // Georgia
    {  253,  772,       0,  260168,       0,  0, 22,  0, }, // Gilbert_Islands
    {  253,  788,       0,  260190,       0,  0, 26,  0, }, // GMT
    {  253,  830,  260216,  260243,  260278, 27, 35, 33, }, // Greenland_Eastern
    {  253,  848,  260311,  260340,  260377, 29, 37, 35, }, // Greenland_Western
    {  253,  866,       0,  260412,       0,  0, 13,  0, }, // Guam
    {  253,  871,       0,  260425,       0,  0, 14,  0, }, // Gulf
    {  253,  876,       0,  260439,       0,  0, 15,  0, }, // Guyana
    {  253,  883,  260454,  260482,  260518, 28, 36, 34, }, // Hawaii_Aleutian
    {  253,  899,  260552,  260570,  260596, 18, 26, 24, }, // Hong_Kong
    {  253,  909,  260620,  260633,  260654, 13, 21, 19, }, // Hovd
    {  253,  914,       0,  260673,       0,  0, 15,  0, }, // India
    {  253,  920,       0,  260688,       0,  0, 23,  0, }, // Indian_Ocean
    {  253,  933,       0,  260711,       0,  0, 17,  0, }, // Indochina
    {  253,  943,       0,  260728,       0,  0, 26,  0, }, // Indonesia_Central
    {  253,  961,       0,  260754,       0,  0, 25,  0, }, // Indonesia_Eastern
    {  253,  979,       0,  260779,       0,  0, 27,  0, }, // Indonesia_Western
    {  253,  997,  260806,  260821,  260841, 15, 20, 18, }, // Iran
    {  253, 1008,  260859,  260875,  260899, 16, 24, 22, }, // Irkutsk
    {  253, 1016,  260921,  260935,  260957, 14, 22, 20, }, // Israel
    {  253, 1023,  260977,  260991,  261013, 14, 22, 20, }, // Japan
    {  253, 1029,  261033,  261066,  261107, 33, 41, 39, }, // Kamchatka
    {  253, 1047,       0,  261146,       0,  0, 19,  0, }, // Kazakhstan
    {  253, 1058,       0,  261165,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  253, 1077,       0,  261193,       0,  0, 30,  0, }, // Kazakhstan_Western
    {  253, 1106,  261223,  261240,  261265, 17, 25, 20, }, // Korea
    {  253, 1112,       0,  261285,       0,  0, 15,  0, }, // Kosrae
    {  253, 1119,  261300,  261320,  261348, 20, 28, 26, }, // Krasnoyarsk
    {  253, 1151,       0,  261374,       0,  0, 21,  0, }, // Kyrgystan
    {  253, 1161,       0,  261395,       0,  0, 14,  0, }, // Lanka
    {  253, 1175,       0,  261409,       0,  0, 26,  0, }, // Line_Islands
    {  253, 1188,  261435,  261453,  261479, 18, 26, 24, }, // Lord_Howe
    {  253, 1198,  261503,  261517,  261539, 14, 22, 20, }, // Macau
    {  253, 1204,  261559,  261575,  261599, 16, 24, 22, }, // Magadan
    {  253, 1219,       0,  261621,       0,  0, 20,  0, }, // Malaysia
    {  253, 1228,       0,  261641,       0,  0, 18,  0, }, // Maldives
    {  253, 1237,       0,  261659,       0,  0, 24,  0, }, // Marquesas
    {  253, 1247,       0,  261683,       0,  0, 23,  0, }, // Marshall_Islands
    {  253, 1264,  261706,  261722,  261746, 16, 24, 22, }, // Mauritius
    {  253, 1274,       0,  261768,       0,  0, 15,  0, }, // Mawson
    {  253, 1281,  261783,  261810,  261845, 27, 35, 33, }, // Mexico_Pacific
    {  253, 1296,  261878,  261897,  261924, 19, 27, 25, }, // Mongolia
    {  253, 1305,  261949,  261964,  261987, 15, 23, 21, }, // Moscow
    {  253, 1312,       0,  262008,       0,  0, 16,  0, }, // Myanmar
    {  253, 1320,       0,  262024,       0,  0, 14,  0, }, // Nauru
    {  253, 1326,       0,  262038,       0,  0, 14,  0, }, // Nepal
    {  253, 1332,  262052,  262082,  262120, 30, 38, 33, }, // New_Caledonia
    {  253, 1346,  262153,  262181,  262217, 28, 36, 34, }, // New_Zealand
    {  253, 1358,  262251,  262271,  262299, 20, 28, 26, }, // Newfoundland
    {  253, 1371,       0,  262325,       0,  0, 13,  0, }, // Niue
    {  253, 1376,  262338,  262360,  262390, 22, 30, 28, }, // Norfolk
    {  253, 1384,  262418,  262446,  262482, 28, 36, 34, }, // Noronha
    {  253, 1392,       0,  262516,       0,  0, 32,  0, }, // North_Mariana
    {  253, 1406,  262548,  262569,  262598, 21, 29, 27, }, // Novosibirsk
    {  253, 1418,  262625,  262638,  262659, 13, 21, 19, }, // Omsk
    {  253, 1428,  262678,  262695,  262720, 17, 25, 23, }, // Pakistan
    {  253, 1437,       0,  262743,       0,  0, 16,  0, }, // Palau
    {  253, 1443,       0,  262759,       0,  0, 37,  0, }, // Papua_New_Guinea
    {  253, 1460,  262796,  262813,  262838, 17, 25, 23, }, // Paraguay
    {  253, 1469,  262861,  262875,  262897, 14, 22, 20, }, // Peru
    {  253, 1474,  262917,  262938,  262967, 21, 29, 27, }, // Philippines
    {  253, 1486,       0,  262994,       0,  0, 22,  0, }, // Phoenix_Islands
    {  253, 1502,  263016,  263049,  263090, 33, 41, 39, }, // Pierre_Miquelon
    {  253, 1518,       0,  263129,       0,  0, 23,  0, }, // Pitcairn
    {  253, 1527,       0,  263152,       0,  0, 25,  0, }, // Ponape
    {  253, 1534,       0,  263177,       0,  0, 18,  0, }, // Pyongyang
    {  253, 1544,  263195,  263213,  263239, 18, 26, 24, }, // Qyzylorda
    {  253, 1554,       0,  263263,       0,  0, 19,  0, }, // Reunion
    {  253, 1562,       0,  263282,       0,  0, 16,  0, }, // Rothera
    {  253, 1570,  263298,  263316,  263342, 18, 26, 24, }, // Sakhalin
    {  253, 1579,  263366,  263381,  263404, 15, 23, 21, }, // Samara
    {  253, 1596,  263425,  263440,  263463, 15, 23, 21, }, // Samoa
    {  253, 1602,       0,  263484,       0,  0, 20,  0, }, // Seychelles
    {  253, 1624,       0,  263504,       0,  0, 18,  0, }, // Singapore
    {  253, 1634,       0,  263522,       0,  0, 22,  0, }, // Solomon
    {  253, 1642,       0,  263544,       0,  0, 23,  0, }, // South_Georgia
    {  253, 1656,       0,  263567,       0,  0, 17,  0, }, // Suriname
    {  253, 1676,       0,  263584,       0,  0, 14,  0, }, // Syowa
    {  253, 1682,       0,  263598,       0,  0, 15,  0, }, // Tahiti
    {  253, 1689,  263613,  263628,  263651, 15, 23, 21, }, // Taipei
    {  253, 1696,       0,  263672,       0,  0, 20,  0, }, // Tajikistan
    {  253, 1724,       0,  263692,       0,  0, 16,  0, }, // Tokelau
    {  253, 1732,  263708,  263723,  263746, 15, 23, 20, }, // Tonga
    {  253, 1738,       0,  263766,       0,  0, 14,  0, }, // Truk
    {  253, 1750,  263780,  263801,  263830, 21, 29, 27, }, // Turkmenistan
    {  253, 1763,       0,  263857,       0,  0, 16,  0, }, // Tuvalu
    {  253, 1777,  263873,  263891,  263917, 18, 26, 24, }, // Uruguay
    {  253, 1792,  263941,  263963,  263993, 22, 30, 28, }, // Uzbekistan
    {  253, 1803,  264021,  264037,  264061, 16, 24, 22, }, // Vanuatu
    {  253, 1811,       0,  264083,       0,  0, 18,  0, }, // Venezuela
    {  253, 1821,  264101,  264121,  264149, 20, 28, 26, }, // Vladivostok
    {  253, 1833,  264175,  264193,  264219, 18, 26, 24, }, // Volgograd
    {  253, 1843,       0,  264243,       0,  0, 15,  0, }, // Vostok
    {  253, 1850,       0,  264258,       0,  0, 19,  0, }, // Wake
    {  253, 1855,       0,  264277,       0,  0, 25,  0, }, // Wallis
    {  253, 1862,  264302,  264319,  264344, 17, 25, 23, }, // Yakutsk
    {  253, 1870,  264367,  264388,  264417, 21, 29, 27, }, // Yekaterinburg
    {  253, 1892,       0,  264444,       0,  0, 22,  0, }, // Yukon
    {  260,    0,  256643,  256658,  264466, 15, 23, 23, }, // Acre French/Latin/Canada
    {  260,   17,       0,  264489,       0,  0, 24,  0, }, // Africa_Central
    {  260,   32,       0,  264513,       0,  0, 25,  0, }, // Africa_Eastern
    {  260,   65,       0,  264538,       0,  0, 30,  0, }, // Africa_Southern
    {  260,   81,  256823,  256849,  264568, 26, 34, 34, }, // Africa_Western
    {  260,  107,  256915,  256932,  264602, 17, 25, 25, }, // Alaska
    {  260,  128,  256980,  256996,  264627, 16, 24, 24, }, // Almaty
    {  260,  135,  257042,  257061,  264651, 19, 27, 27, }, // Amazon
    {  260,  142,  260728,  264678,  264701, 15, 23, 23, }, // America_Central
    {  260,  158,  257217,  257246,  264724, 14, 22, 22, }, // America_Eastern
    {  260,  174,  257318,  257337,  264746, 19, 27, 27, }, // America_Mountain
    {  260,  191,  257389,  257422,  264773, 18, 26, 26, }, // America_Pacific
    {  260,  207,  257502,  257516,  264799, 14, 22, 22, }, // Anadyr
    {  260,  214,  257558,  257570,  264821, 12, 20, 20, }, // Apia
    {  260,  219,  257608,  257622,  264841, 14, 22, 22, }, // Aqtau
    {  260,  225,  257664,  257678,  264863, 14, 22, 22, }, // Aqtobe
    {  260,  232,  257720,  257737,  264885, 17, 25, 25, }, // Arabian
    {  260,  240,  257785,  257805,  264910, 20, 25, 28, }, // Argentina
    {  260,  250,  257856,  257881,  264938, 25, 33, 33, }, // Argentina_Western
    {  260,  268,  257945,  257963,  264971, 18, 26, 23, }, // Armenia
    {  260,  286,  258010,  258031,  264994, 21, 29, 29, }, // Atlantic
    {  260,  295,  258087,  258117,  265023, 30, 38, 38, }, // Australia_Central
    {  260,  313,  258191,  258227,  265061, 36, 44, 44, }, // Australia_CentralWestern
    {  260,  338,  258313,  258342,  265105, 29, 37, 37, }, // Australia_Eastern
    {  260,  356,  258414,  258445,  265142, 31, 39, 39, }, // Australia_Western
    {  260,  374,  258521,  258543,  265181, 22, 30, 27, }, // Azerbaijan
    {  260,  385,  258598,  258614,  265208, 16, 24, 24, }, // Azores
    {  260,  397,  258660,  258679,  265232, 19, 27, 27, }, // Bangladesh
    {  260,  437,  258763,  258780,  265259, 17, 25, 25, }, // Brasilia
    {  260,  454,       0,  265284,       0,  0, 15,  0, }, // Brunei
    {  260,  461,  258843,  258860,  265299, 17, 25, 25, }, // Cape_Verde
    {  260,  495,  259020,  259034,  265324, 14, 22, 22, }, // Chile
    {  260,  501,  265346,  265360,  265382, 14, 22, 22, }, // China
    {  260,  523,  259182,  259199,  265404, 17, 25, 25, }, // Colombia
    {  260,  532,  259247,  259266,  265429, 19, 27, 27, }, // Cook
    {  260,  537,  259318,  259331,  265456, 13, 21, 21, }, // Cuba
    {  260,  612,  259433,  259457,  265477, 24, 32, 32, }, // Easter
    {  260,  627,  265509,  265535,  265569, 26, 34, 34, }, // Europe_Central
    {  260,  642,  265603,  265629,  265663, 26, 34, 34, }, // Europe_Eastern
    {  260,  680,  265697,  265725,  265761, 28, 36, 36, }, // Europe_Western
    {  260,  695,  259813,  259837,  265797, 24, 32, 32, }, // Falkland
    {  260,  704,  259899,  259919,  265829, 20, 28, 28, }, // Fiji
    {  260,  709,       0,  265857,       0,  0, 25,  0, }, // French_Guiana
    {  260,  764,  260100,  260119,  265882, 19, 27, 24, }, // Georgia
    {  260,  830,  260216,  260243,  265906, 27, 35, 35, }, // Greenland_Eastern
    {  260,  848,  260311,  260340,  265941, 29, 37, 37, }, // Greenland_Western
    {  260,  883,  265978,  266004,  266038, 26, 34, 34, }, // Hawaii_Aleutian
    {  260,  899,  260552,  260570,  266072, 18, 26, 26, }, // Hong_Kong
    {  260,  909,  260620,  260633,  266098, 13, 21, 21, }, // Hovd
    {  260,  920,       0,  266119,       0,  0, 23,  0, }, // Indian_Ocean
    {  260,  997,  260806,  260821,  266142, 15, 20, 20, }, // Iran
    {  260, 1008,  260859,  260875,  266162, 16, 24, 24, }, // Irkutsk
    {  260, 1016,  260921,  260935,  266186, 14, 22, 22, }, // Israel
    {  260, 1023,  260977,  260991,  266208, 14, 22, 22, }, // Japan
    {  260, 1029,  261033,  261066,  266230, 33, 41, 41, }, // Kamchatka
    {  260, 1106,  261223,  261240,  266271, 17, 25, 22, }, // Korea
    {  260, 1119,  261300,  261320,  266293, 20, 28, 28, }, // Krasnoyarsk
    {  260, 1188,  261435,  261453,  266321, 18, 26, 26, }, // Lord_Howe
    {  260, 1198,  261503,  261517,  266347, 14, 22, 22, }, // Macau
    {  260, 1204,  261559,  261575,  266369, 16, 24, 24, }, // Magadan
    {  260, 1264,  261706,  261722,  266393, 16, 24, 24, }, // Mauritius
    {  260, 1281,  261783,  261810,  266417, 27, 35, 35, }, // Mexico_Pacific
    {  260, 1296,  261878,  261897,  266452, 19, 27, 27, }, // Mongolia
    {  260, 1305,  261949,  261964,  266479, 15, 23, 23, }, // Moscow
    {  260, 1332,  262052,  262082,  266502, 30, 38, 35, }, // New_Caledonia
    {  260, 1346,  262153,  262181,  266537, 28, 36, 36, }, // New_Zealand
    {  260, 1358,  262251,  262271,  266573, 20, 28, 28, }, // Newfoundland
    {  260, 1371,       0,  266601,       0,  0, 14,  0, }, // Niue
    {  260, 1376,  262338,  262360,  266615, 22, 30, 30, }, // Norfolk
    {  260, 1384,  262418,  262446,  266645, 28, 36, 36, }, // Noronha
    {  260, 1406,  262548,  262569,  266681, 21, 29, 29, }, // Novosibirsk
    {  260, 1418,  266710,  266722,  266742, 12, 20, 20, }, // Omsk
    {  260, 1428,  262678,  262695,  266762, 17, 25, 25, }, // Pakistan
    {  260, 1460,  262796,  262813,  266787, 17, 25, 25, }, // Paraguay
    {  260, 1469,  262861,  262875,  266812, 14, 22, 22, }, // Peru
    {  260, 1474,  262917,  262938,  266834, 21, 29, 29, }, // Philippines
    {  260, 1502,  263016,  263049,  266863, 33, 41, 41, }, // Pierre_Miquelon
    {  260, 1554,       0,  266904,       0,  0, 19,  0, }, // Reunion
    {  260, 1570,  263298,  263316,  266923, 18, 26, 26, }, // Sakhalin
    {  260, 1596,  263425,  263440,  266949, 15, 23, 23, }, // Samoa
    {  260, 1689,  263613,  263628,  266972, 15, 23, 23, }, // Taipei
    {  260, 1732,  263708,  263723,  266995, 15, 23, 22, }, // Tonga
    {  260, 1750,  263780,  263801,  267017, 21, 29, 29, }, // Turkmenistan
    {  260, 1777,  263873,  263891,  267046, 18, 26, 26, }, // Uruguay
    {  260, 1792,  263941,  263963,  267072, 22, 30, 30, }, // Uzbekistan
    {  260, 1803,  264021,  264037,  267102, 16, 24, 24, }, // Vanuatu
    {  260, 1821,  264101,  264121,  267126, 20, 28, 28, }, // Vladivostok
    {  260, 1833,  264175,  264193,  267154, 18, 26, 26, }, // Volgograd
    {  260, 1862,  264302,  264319,  267180, 17, 25, 25, }, // Yakutsk
    {  260, 1870,  264367,  264388,  267205, 21, 29, 29, }, // Yekaterinburg
    {  268,  709,       0,  259973,       0,  0, 28,  0, }, // French_Guiana French/Latin/French Guiana
    {  299,  627,  267234,  267255,  267285, 21, 30, 28, }, // Europe_Central Friulian/Latin/Italy
    {  299,  642,  267313,  267335,  267366, 22, 31, 29, }, // Europe_Eastern
    {  299,  680,  267395,  267418,  267450, 23, 32, 30, }, // Europe_Western
    {  299, 1305,  267480,  267493,  267515, 13, 22, 20, }, // Moscow
    {  306,    0,  267535,  267560,  267604, 25, 44, 36, }, // Acre Fulah/Adlam/Guinea
    {  306,    5,       0,  267640,       0,  0, 37,  0, }, // Afghanistan
    {  306,   17,       0,  267677,       0,  0, 46,  0, }, // Africa_Central
    {  306,   32,       0,  267723,       0,  0, 48,  0, }, // Africa_Eastern
    {  306,   65,       0,  267771,       0,  0, 67,  0, }, // Africa_Southern
    {  306,   81,  267838,  267888,  267957, 50, 69, 61, }, // Africa_Western
    {  306,  107,  268018,  268047,  268104, 29, 57, 49, }, // Alaska
    {  306,  128,  268153,  268180,  268226, 27, 46, 38, }, // Almaty
    {  306,  135,  268264,  268293,  268341, 29, 48, 40, }, // Amazon
    {  306,  142,  268381,  268442,  268522, 61, 80, 72, }, // America_Central
    {  306,  158,  268594,  268669,  268763, 75, 94, 86, }, // America_Eastern
    {  306,  174,  268849,  268922,  269014, 73, 92, 61, }, // America_Mountain
    {  306,  191,  269075,  269136,  269216, 61, 80, 72, }, // America_Pacific
    {  306,  207,  269288,  269317,  269365, 29, 48, 40, }, // Anadyr
    {  306,  214,  269405,  269432,  269478, 27, 46, 38, }, // Apia
    {  306,  219,  269516,  269541,  269585, 25, 44, 36, }, // Aqtau
    {  306,  225,  269621,  269650,  269698, 29, 48, 40, }, // Aqtobe
    {  306,  232,  269738,  269769,  269819, 31, 50, 42, }, // Arabian
    {  306,  240,  269861,  269896,  269950, 35, 54, 46, }, // Argentina
    {  306,  250,  269996,  270054,  270131, 58, 77, 69, }, // Argentina_Western
    {  306,  268,  270200,  270233,  270285, 33, 52, 44, }, // Armenia
    {  306,  286,  270329,  270352,  270394, 23, 42, 34, }, // Atlantic
    {  306,  295,  270428,  270480,  270551, 52, 71, 63, }, // Australia_Central
    {  306,  313,  270614,  270687,  270779, 73, 92, 84, }, // Australia_CentralWestern
    {  306,  338,  270863,  270923,  271002, 60, 79, 71, }, // Australia_Eastern
    {  306,  356,  271073,  271135,  271216, 62, 81, 73, }, // Australia_Western
    {  306,  374,  271289,  271324,  271378, 35, 54, 46, }, // Azerbaijan
    {  306,  385,  271424,  271451,  271497, 27, 46, 38, }, // Azores
    {  306,  397,  271535,  271572,  271628, 37, 56, 48, }, // Bangladesh
    {  306,  415,       0,  271676,       0,  0, 27,  0, }, // Bhutan
    {  306,  422,       0,  271703,       0,  0, 33,  0, }, // Bolivia
    {  306,  437,  271736,  271771,  271825, 35, 54, 46, }, // Brasilia
    {  306,  454,       0,  271871,       0,  0, 27,  0, }, // Brunei
    {  306,  461,  271898,  271934,  271989, 36, 55, 47, }, // Cape_Verde
    {  306,  472,       0,  272036,       0,  0, 27,  0, }, // Casey
    {  306,  478,       0,  272063,       0,  0, 48,  0, }, // Chamorro
    {  306,  487,  272111,  272138,  272184, 27, 46, 38, }, // Chatham
    {  306,  495,  272222,  272247,  272291, 25, 44, 36, }, // Chile
    {  306,  501,  272327,  272352,  272396, 25, 44, 36, }, // China
    {  306,  507,       0,  272432,       0,  0, 48,  0, }, // Christmas
    {  306,  517,       0,  272480,       0,  0, 27,  0, }, // Cocos
    {  306,  523,  272507,  272542,  272596, 35, 54, 46, }, // Colombia
    {  306,  532,  272642,  272680,  272737, 38, 57, 60, }, // Cook
    {  306,  537,  272797,  272824,  272870, 27, 46, 38, }, // Cuba
    {  306,  548,       0,  272908,       0,  0, 27,  0, }, // Davis
    {  306,  564,       0,  272935,       0,  0, 42,  0, }, // DumontDUrville
    {  306,  601,       0,  272977,       0,  0, 48,  0, }, // East_Timor
    {  306,  612,  273025,  273063,  273120, 38, 57, 49, }, // Easter
    {  306,  619,       0,  273169,       0,  0, 33,  0, }, // Ecuador
    {  306,  627,  273202,  273246,  273309, 44, 63, 55, }, // Europe_Central
    {  306,  642,  273364,  273412,  273479, 48, 67, 59, }, // Europe_Eastern
    {  306,  657,       0,  273538,       0,  0, 54,  0, }, // Europe_Further_Eastern
    {  306,  680,  273592,  273642,  273711, 50, 69, 61, }, // Europe_Western
    {  306,  695,  273772,  273816,  273879, 44, 63, 55, }, // Falkland
    {  306,  704,  273934,  273959,  274003, 25, 44, 36, }, // Fiji
    {  306,  709,       0,  274039,       0,  0, 42,  0, }, // French_Guiana
    {  306,  723,       0,  274081,       0,  0, 59,  0, }, // French_Southern
    {  306,  746,       0,  274140,       0,  0, 35,  0, }, // Galapagos
    {  306,  756,       0,  274175,       0,  0, 31,  0, }, // Gambier
    {  306,  764,  274206,  274235,  274283, 29, 48, 40, }, // Georgia
    {  306,  772,       0,  274323,       0,  0, 46,  0, }, // Gilbert_Islands
    {  306,  788,       0,  274369,       0,  0, 54,  0, }, // GMT
    {  306,  830,  274423,  274471,  274538, 48, 67, 59, }, // Greenland_Eastern
    {  306,  848,  274597,  274647,  274716, 50, 69, 61, }, // Greenland_Western
    {  306,  866,       0,  274777,       0,  0, 46,  0, }, // Guam
    {  306,  871,       0,  274823,       0,  0, 25,  0, }, // Gulf
    {  306,  876,       0,  274039,       0,  0, 29,  0, }, // Guyana
    {  306,  883,  274848,  274896,  274963, 48, 67, 59, }, // Hawaii_Aleutian
    {  306,  899,  275022,  275054,  275105, 32, 51, 43, }, // Hong_Kong
    {  306,  909,  275148,  275173,  275217, 25, 44, 36, }, // Hovd
    {  306,  914,       0,  275253,       0,  0, 46,  0, }, // India
    {  306,  920,       0,  275299,       0,  0, 42,  0, }, // Indian_Ocean
    {  306,  933,       0,  275341,       0,  0, 31,  0, }, // Indochina
    {  306,  943,       0,  275372,       0,  0, 54,  0, }, // Indonesia_Central
    {  306,  961,       0,  275426,       0,  0, 58,  0, }, // Indonesia_Eastern
    {  306,  979,       0,  275484,       0,  0, 60,  0, }, // Indonesia_Western
    {  306,  997,  275544,  275569,  275613, 25, 44, 36, }, // Iran
    {  306, 1008,  275649,  275682,  275734, 33, 52, 44, }, // Irkutsk
    {  306, 1016,  275778,  275811,  275863, 33, 52, 44, }, // Israel
    {  306, 1023,  275907,  275934,  275980, 27, 46, 38, }, // Japan
    {  306, 1058,       0,  276018,       0,  0, 37,  0, }, // Kazakhstan_Eastern
    {  306, 1077,       0,  276055,       0,  0, 58,  0, }, // Kazakhstan_Western
    {  306, 1106,  276113,  276144,  276194, 31, 50, 42, }, // Korea
    {  306, 1112,       0,  276236,       0,  0, 27,  0, }, // Kosrae
    {  306, 1119,  276263,  276304,  276362, 41, 58, 52, }, // Krasnoyarsk
    {  306, 1151,       0,  276414,       0,  0, 35,  0, }, // Kyrgystan
    {  306, 1175,       0,  276449,       0,  0, 40,  0, }, // Line_Islands
    {  306, 1188,  276489,  276523,  276576, 34, 53, 45, }, // Lord_Howe
    {  306, 1198,  276621,  276650,  276698, 29, 48, 40, }, // Macau
    {  306, 1204,  276738,  276767,  276815, 29, 48, 40, }, // Magadan
    {  306, 1219,       0,  276855,       0,  0, 33,  0, }, // Malaysia
    {  306, 1228,       0,  276888,       0,  0, 33,  0, }, // Maldives
    {  306, 1237,       0,  276921,       0,  0, 31,  0, }, // Marquesas
    {  306, 1247,       0,  276952,       0,  0, 42,  0, }, // Marshall_Islands
    {  306, 1264,  276994,  277023,  277071, 29, 48, 40, }, // Mauritius
    {  306, 1274,       0,  277111,       0,  0, 27,  0, }, // Mawson
    {  306, 1281,  277138,  277180,  277241, 42, 61, 53, }, // Mexico_Pacific
    {  306, 1296,  277294,  277327,  277379, 33, 52, 44, }, // Mongolia
    {  306, 1305,  277423,  277448,  277492, 25, 44, 36, }, // Moscow
    {  306, 1312,       0,  277528,       0,  0, 31,  0, }, // Myanmar
    {  306, 1320,       0,  277559,       0,  0, 25,  0, }, // Nauru
    {  306, 1326,       0,  277584,       0,  0, 25,  0, }, // Nepal
    {  306, 1332,  277609,  277655,  277720, 46, 65, 57, }, // New_Caledonia
    {  306, 1346,  277777,  277813,  277868, 36, 55, 47, }, // New_Zealand
    {  306, 1358,  277915,  277955,  278014, 40, 59, 51, }, // Newfoundland
    {  306, 1371,       0,  278065,       0,  0, 25,  0, }, // Niue
    {  306, 1376,  278090,  278136,  278201, 46, 65, 57, }, // Norfolk
    {  306, 1384,  278258,  278313,  278387, 55, 74, 66, }, // Noronha
    {  306, 1406,  278453,  278490,  278546, 37, 56, 48, }, // Novosibirsk
    {  306, 1418,  278594,  278621,  278667, 27, 46, 38, }, // Omsk
    {  306, 1428,  278705,  278738,  278790, 33, 52, 44, }, // Pakistan
    {  306, 1437,       0,  278834,       0,  0, 29,  0, }, // Palau
    {  306, 1443,       0,  278863,       0,  0, 45,  0, }, // Papua_New_Guinea
    {  306, 1460,  278908,  278939,  278989, 31, 50, 42, }, // Paraguay
    {  306, 1469,  279031,  279056,  279100, 25, 44, 59, }, // Peru
    {  306, 1474,  279159,  279190,  279240, 31, 50, 42, }, // Philippines
    {  306, 1486,       0,  279282,       0,  0, 44,  0, }, // Phoenix_Islands
    {  306, 1502,  279326,  279376,  279445, 50, 69, 61, }, // Pierre_Miquelon
    {  306, 1518,       0,  279506,       0,  0, 33,  0, }, // Pitcairn
    {  306, 1527,       0,  279539,       0,  0, 29,  0, }, // Ponape
    {  306, 1534,       0,  279568,       0,  0, 31,  0, }, // Pyongyang
    {  306, 1544,  279599,  279632,  279684, 33, 52, 44, }, // Qyzylorda
    {  306, 1554,       0,  279728,       0,  0, 29,  0, }, // Reunion
    {  306, 1562,       0,  279757,       0,  0, 27,  0, }, // Rothera
    {  306, 1570,  279784,  279815,  279865, 31, 50, 42, }, // Sakhalin
    {  306, 1596,  279907,  279934,  279980, 27, 46, 38, }, // Samoa
    {  306, 1602,       0,  280018,       0,  0, 29,  0, }, // Seychelles
    {  306, 1624,       0,  280047,       0,  0, 52,  0, }, // Singapore
    {  306, 1634,       0,  280099,       0,  0, 48,  0, }, // Solomon
    {  306, 1642,       0,  280147,       0,  0, 40,  0, }, // South_Georgia
    {  306, 1656,       0,  280187,       0,  0, 29,  0, }, // Suriname
    {  306, 1676,       0,  280216,       0,  0, 29,  0, }, // Syowa
    {  306, 1682,       0,  280245,       0,  0, 29,  0, }, // Tahiti
    {  306, 1689,  280274,  280301,  280347, 27, 46, 38, }, // Taipei
    {  306, 1696,       0,  280385,       0,  0, 37,  0, }, // Tajikistan
    {  306, 1724,       0,  280422,       0,  0, 35,  0, }, // Tokelau
    {  306, 1732,  280457,  280482,  280526, 25, 44, 36, }, // Tonga
    {  306, 1738,       0,  280562,       0,  0, 25,  0, }, // Truk
    {  306, 1750,  280587,  280630,  280692, 43, 62, 54, }, // Turkmenistan
    {  306, 1763,       0,  280746,       0,  0, 29,  0, }, // Tuvalu
    {  306, 1777,  280775,  280804,  280852, 29, 48, 40, }, // Uruguay
    {  306, 1792,  280892,  280929,  280985, 37, 56, 48, }, // Uzbekistan
    {  306, 1803,  281033,  281066,  281118, 33, 52, 44, }, // Vanuatu
    {  306, 1811,       0,  281162,       0,  0, 35,  0, }, // Venezuela
    {  306, 1821,  281197,  281232,  281286, 35, 54, 46, }, // Vladivostok
    {  306, 1833,  281332,  281369,  281425, 37, 56, 48, }, // Volgograd
    {  306, 1843,       0,  281473,       0,  0, 27,  0, }, // Vostok
    {  306, 1850,       0,  281500,       0,  0, 40,  0, }, // Wake
    {  306, 1855,       0,  281540,       0,  0, 44,  0, }, // Wallis
    {  306, 1862,  281584,  281617,  281669, 33, 52, 44, }, // Yakutsk
    {  306, 1870,  281713,  281756,  281818, 43, 62, 54, }, // Yekaterinburg
    {  324,    0,  281872,  281879,  281890,  7, 11, 19, }, // Acre Gaelic/Latin/United Kingdom
    {  324,    5,       0,  281909,       0,  0, 15,  0, }, // Afghanistan
    {  324,   17,       0,  281924,       0,  0, 17,  0, }, // Africa_Central
    {  324,   32,       0,  281941,       0,  0, 16,  0, }, // Africa_Eastern
    {  324,   65,       0,  281957,       0,  0, 16,  0, }, // Africa_Southern
    {  324,   81,  281973,  281989,  282009, 16, 20, 28, }, // Africa_Western
    {  324,  107,  282037,  282046,  282059,  9, 13, 21, }, // Alaska
    {  324,  128,  282080,  282089,  282102,  9, 13, 21, }, // Almaty
    {  324,  135,  282123,  282133,  282147, 10, 14, 22, }, // Amazon
    {  324,  142,  282169,  282190,  282215, 21, 25, 33, }, // America_Central
    {  324,  158,  282248,  282268,  282292, 20, 24, 32, }, // America_Eastern
    {  324,  174,  282324,  282344,  282368, 20, 24, 32, }, // America_Mountain
    {  324,  191,  282400,  282419,  282442, 19, 23, 31, }, // America_Pacific
    {  324,  207,  282473,  282482,  282495,  9, 13, 21, }, // Anadyr
    {  324,  214,  282516,  282523,  282534,  7, 11, 19, }, // Apia
    {  324,  219,  282553,  282561,  282573,  8, 12, 20, }, // Aqtau
    {  324,  225,  282593,  282602,  282615,  9, 13, 21, }, // Aqtobe
    {  324,  232,  282636,  282646,  282660, 10, 14, 22, }, // Arabian
    {  324,  240,  282682,  282700,  282722, 18, 22, 30, }, // Argentina
    {  324,  250,  282752,  282779,  282810, 27, 31, 39, }, // Argentina_Western
    {  324,  268,  282849,  282861,  282877, 12, 16, 24, }, // Armenia
    {  324,  286,  282901,  282919,  282941, 18, 22, 30, }, // Atlantic
    {  324,  295,  282971,  282991,  283015, 20, 24, 32, }, // Australia_Central
    {  324,  313,  283047,  283074,  283105, 27, 31, 39, }, // Australia_CentralWestern
    {  324,  338,  283144,  283163,  283186, 19, 23, 31, }, // Australia_Eastern
    {  324,  356,  283217,  283236,  283259, 19, 23, 31, }, // Australia_Western
    {  324,  374,  283290,  283305,  283324, 15, 19, 27, }, // Azerbaijan
    {  324,  385,  283351,  283374,  283401, 23, 27, 35, }, // Azores
    {  324,  397,  283436,  283449,  283466, 13, 17, 25, }, // Bangladesh
    {  324,  415,       0,  283491,       0,  0,  9,  0, }, // Bhutan
    {  324,  422,       0,  283500,       0,  0, 12,  0, }, // Bolivia
    {  324,  437,  283512,  283524,  283540, 12, 16, 24, }, // Brasilia
    {  324,  454,       0,  283564,       0,  0, 25,  0, }, // Brunei
    {  324,  461,  283589,  283605,  283625, 16, 20, 28, }, // Cape_Verde
    {  324,  472,       0,  283653,       0,  0,  9,  0, }, // Casey
    {  324,  478,       0,  283662,       0,  0, 11,  0, }, // Chamorro
    {  324,  487,  283673,  283683,  283697, 10, 14, 22, }, // Chatham
    {  324,  495,  283719,  283729,  283743, 10, 14, 22, }, // Chile
    {  324,  501,  283765,  283775,  283789, 10, 14, 22, }, // China
    {  324,  507,       0,  283811,       0,  0, 20,  0, }, // Christmas
    {  324,  517,       0,  283831,       0,  0, 18,  0, }, // Cocos
    {  324,  523,  283849,  283861,  283877, 12, 16, 24, }, // Colombia
    {  324,  532,  283901,  283917,  283937, 16, 20, 34, }, // Cook
    {  324,  537,  283971,  283978,  283989,  7, 11, 19, }, // Cuba
    {  324,  548,       0,  284008,       0,  0,  9,  0, }, // Davis
    {  324,  564,       0,  284017,       0,  0, 19,  0, }, // DumontDUrville
    {  324,  601,       0,  284036,       0,  0, 17,  0, }, // East_Timor
    {  324,  612,  284053,  284072,  284095, 19, 23, 31, }, // Easter
    {  324,  619,       0,  284126,       0,  0, 12,  0, }, // Ecuador
    {  324,  627,  284138,  284163,  284192, 25, 29, 37, }, // Europe_Central
    {  324,  642,  284229,  284253,  284281, 24, 28, 36, }, // Europe_Eastern
    {  324,  657,       0,  284317,       0,  0, 32,  0, }, // Europe_Further_Eastern
    {  324,  680,  284349,  284373,  284401, 24, 28, 36, }, // Europe_Western
    {  324,  695,  284437,  284463,  284493, 26, 30, 38, }, // Falkland
    {  324,  704,  284531,  284538,  284549,  7, 11, 19, }, // Fiji
    {  324,  709,       0,  284568,       0,  0, 23,  0, }, // French_Guiana
    {  324,  723,       0,  284591,       0,  0, 40,  0, }, // French_Southern
    {  324,  746,       0,  284631,       0,  0, 13,  0, }, // Galapagos
    {  324,  756,       0,  284644,       0,  0, 11,  0, }, // Gambier
    {  324,  764,  284655,  284672,  284693, 17, 21, 29, }, // Georgia
    {  324,  772,       0,  284722,       0,  0, 23,  0, }, // Gilbert_Islands
    {  324,  788,       0,    2791,       0,  0, 19,  0, }, // GMT
    {  324,  802,  284745,  284762,  284783, 17, 21, 29, }, // Greenland
    {  324,  830,  284812,  284835,  284862, 23, 27, 35, }, // Greenland_Eastern
    {  324,  848,  284897,  284920,  284947, 23, 27, 35, }, // Greenland_Western
    {  324,  866,       0,  284982,       0,  0,  7,  0, }, // Guam
    {  324,  871,       0,  284989,       0,  0, 13,  0, }, // Gulf
    {  324,  876,       0,  284568,       0,  0, 12,  0, }, // Guyana
    {  324,  883,  285002,  285037,  285076, 35, 39, 47, }, // Hawaii_Aleutian
    {  324,  899,  285123,  285135,  285151, 12, 16, 24, }, // Hong_Kong
    {  324,  909,  285175,  285182,  285193,  7, 11, 19, }, // Hovd
    {  324,  914,       0,  285212,       0,  0, 17,  0, }, // India
    {  324,  920,       0,  285229,       0,  0, 22,  0, }, // Indian_Ocean
    {  324,  933,       0,  285251,       0,  0, 17,  0, }, // Indochina
    {  324,  943,       0,  285268,       0,  0, 25,  0, }, // Indonesia_Central
    {  324,  961,       0,  285293,       0,  0, 24,  0, }, // Indonesia_Eastern
    {  324,  979,       0,  285317,       0,  0, 24,  0, }, // Indonesia_Western
    {  324,  997,  285341,  285350,  285363,  9, 13, 21, }, // Iran
    {  324, 1008,  285384,  285394,  285408, 10, 14, 22, }, // Irkutsk
    {  324, 1016,  285430,  285440,  285454, 10, 14, 22, }, // Israel
    {  324, 1023,  285476,  285490,  285508, 14, 18, 26, }, // Japan
    {  324, 1029,  285534,  285561,  285592, 27, 31, 39, }, // Kamchatka
    {  324, 1047,       0,  285631,       0,  0, 14,  0, }, // Kazakhstan
    {  324, 1058,       0,  285645,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {  324, 1077,       0,  285666,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  324, 1106,  285687,  285697,  285711, 10, 14, 22, }, // Korea
    {  324, 1112,       0,  285733,       0,  0,  9,  0, }, // Kosrae
    {  324, 1119,  285742,  285756,  285774, 14, 18, 26, }, // Krasnoyarsk
    {  324, 1151,       0,  285800,       0,  0, 14,  0, }, // Kyrgystan
    {  324, 1161,       0,  285814,       0,  0,  8,  0, }, // Lanka
    {  324, 1175,       0,  285822,       0,  0, 19,  0, }, // Line_Islands
    {  324, 1188,  285841,  285853,  285869, 12, 16, 24, }, // Lord_Howe
    {  324, 1198,  285893,  285903,  285917, 10, 14, 22, }, // Macau
    {  324, 1204,  285939,  285949,  285963, 10, 14, 22, }, // Magadan
    {  324, 1219,       0,  285985,       0,  0, 14,  0, }, // Malaysia
    {  324, 1228,       0,  285999,       0,  0, 26,  0, }, // Maldives
    {  324, 1237,       0,  286025,       0,  0, 23,  0, }, // Marquesas
    {  324, 1247,       0,  286048,       0,  0, 21,  0, }, // Marshall_Islands
    {  324, 1264,  286069,  286095,  286125, 26, 30, 38, }, // Mauritius
    {  324, 1274,       0,  286163,       0,  0, 10,  0, }, // Mawson
    {  324, 1281,  286173,  286202,  286235, 29, 33, 41, }, // Mexico_Pacific
    {  324, 1296,  286276,  286289,  286306, 13, 17, 25, }, // Mongolia
    {  324, 1305,  286331,  286340,  286353,  9, 13, 21, }, // Moscow
    {  324, 1312,       0,  286374,       0,  0, 10,  0, }, // Myanmar
    {  324, 1320,       0,  286384,       0,  0,  9,  0, }, // Nauru
    {  324, 1326,       0,  286393,       0,  0,  9,  0, }, // Nepal
    {  324, 1332,  286402,  286422,  286446, 20, 24, 32, }, // New_Caledonia
    {  324, 1346,  286478,  286497,  286520, 19, 23, 31, }, // New_Zealand
    {  324, 1358,  286551,  286568,  286589, 17, 21, 29, }, // Newfoundland
    {  324, 1371,       0,  286618,       0,  0,  7,  0, }, // Niue
    {  324, 1376,  286625,  286642,  286663, 17, 21, 29, }, // Norfolk
    {  324, 1384,  286692,  286714,  286740, 22, 26, 34, }, // Noronha
    {  324, 1392,       0,  286774,       0,  0, 34,  0, }, // North_Mariana
    {  324, 1406,  286808,  286822,  286840, 14, 18, 26, }, // Novosibirsk
    {  324, 1418,  286866,  286873,  286884,  7, 11, 19, }, // Omsk
    {  324, 1428,  286903,  286915,  286931, 12, 16, 24, }, // Pakistan
    {  324, 1437,       0,  286955,       0,  0,  9,  0, }, // Palau
    {  324, 1443,       0,  286964,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  324, 1460,  286990,  287003,  287020, 13, 17, 25, }, // Paraguay
    {  324, 1469,  287045,  287053,  287065,  8, 12, 20, }, // Peru
    {  324, 1474,  287085,  287110,  287139, 25, 29, 37, }, // Philippines
    {  324, 1486,       0,  287176,       0,  0, 19,  0, }, // Phoenix_Islands
    {  324, 1502,  287195,  287224,  287257, 29, 33, 41, }, // Pierre_Miquelon
    {  324, 1518,       0,  287298,       0,  0, 17,  0, }, // Pitcairn
    {  324, 1527,       0,  287315,       0,  0, 10,  0, }, // Ponape
    {  324, 1534,       0,  287325,       0,  0, 13,  0, }, // Pyongyang
    {  324, 1544,  287338,  287350,  287366, 12, 16, 24, }, // Qyzylorda
    {  324, 1554,       0,  287390,       0,  0, 10,  0, }, // Reunion
    {  324, 1562,       0,  287400,       0,  0, 10,  0, }, // Rothera
    {  324, 1570,  287410,  287421,  287436, 11, 15, 23, }, // Sakhalin
    {  324, 1579,  287459,  287468,  287481,  9, 13, 21, }, // Samara
    {  324, 1596,  287502,  287512,  287526, 10, 14, 22, }, // Samoa
    {  324, 1602,       0,  287548,       0,  0, 25,  0, }, // Seychelles
    {  324, 1624,       0,  287573,       0,  0, 12,  0, }, // Singapore
    {  324, 1634,       0,  287585,       0,  0, 20,  0, }, // Solomon
    {  324, 1642,       0,  287605,       0,  0, 17,  0, }, // South_Georgia
    {  324, 1656,       0,  287622,       0,  0, 11,  0, }, // Suriname
    {  324, 1676,       0,  287633,       0,  0,  9,  0, }, // Syowa
    {  324, 1682,       0,  287642,       0,  0,  9,  0, }, // Tahiti
    {  324, 1689,  287651,  287660,  287673,  9, 13, 21, }, // Taipei
    {  324, 1696,       0,  287694,       0,  0, 16,  0, }, // Tajikistan
    {  324, 1724,       0,  287710,       0,  0, 10,  0, }, // Tokelau
    {  324, 1732,  287720,  287728,  287740,  8, 12, 20, }, // Tonga
    {  324, 1738,       0,  287760,       0,  0,  8,  0, }, // Truk
    {  324, 1750,  287768,  287784,  287804, 16, 20, 28, }, // Turkmenistan
    {  324, 1763,       0,  287832,       0,  0, 10,  0, }, // Tuvalu
    {  324, 1777,  287842,  287854,  287870, 12, 16, 24, }, // Uruguay
    {  324, 1792,  287894,  287907,  287924, 13, 17, 25, }, // Uzbekistan
    {  324, 1803,  287949,  287959,  287973, 10, 14, 22, }, // Vanuatu
    {  324, 1811,       0,  287995,       0,  0, 17,  0, }, // Venezuela
    {  324, 1821,  288012,  288026,  288044, 14, 18, 26, }, // Vladivostok
    {  324, 1833,  288070,  288082,  288098, 12, 16, 24, }, // Volgograd
    {  324, 1843,       0,  288122,       0,  0,  9,  0, }, // Vostok
    {  324, 1850,       0,  288131,       0,  0, 14,  0, }, // Wake
    {  324, 1855,       0,  288145,       0,  0, 21,  0, }, // Wallis
    {  324, 1862,  288166,  288176,  288190, 10, 14, 22, }, // Yakutsk
    {  324, 1870,  288212,  288228,  288248, 16, 20, 28, }, // Yekaterinburg
    {  324, 1892,       0,  288276,       0,  0,  8,  0, }, // Yukon
    {  325,   17,       0,  288284,       0,  0, 16,  0, }, // Africa_Central Ga/Latin/Ghana
    {  325,   32,       0,  288300,       0,  0, 17,  0, }, // Africa_Eastern
    {  325,   65,       0,  288317,       0,  0, 15,  0, }, // Africa_Southern
    {  325,   81,  288332,  288349,  288379, 17, 30, 32, }, // Africa_Western
    {  325,  107,   18782,  288411,  288433,  9, 22, 24, }, // Alaska
    {  325,  142,  288457,  288478,  288512, 21, 34, 36, }, // America_Central
    {  325,  158,  288548,  288570,  288605, 22, 35, 37, }, // America_Eastern
    {  325,  174,  288642,  288664,  288699, 22, 35, 37, }, // America_Mountain
    {  325,  191,   19007,  288736,  288759, 10, 23, 25, }, // America_Pacific
    {  325,  286,   19322,  288784,  288808, 11, 24, 26, }, // Atlantic
    {  325,  385,   19766,  288834,  288856,  9, 22, 24, }, // Azores
    {  325,  461,  288880,  288893,  288919, 13, 26, 28, }, // Cape_Verde
    {  325,  537,   20322,  288947,  288967,  7, 20, 22, }, // Cuba
    {  325,  723,       0,  288989,       0,  0, 43,  0, }, // French_Southern
    {  325,  788,       0,  289032,       0,  0, 27,  0, }, // GMT
    {  325,  830,  289059,  289079,  289112, 20, 33, 35, }, // Greenland_Eastern
    {  325,  848,  289147,  289167,  289200, 20, 33, 35, }, // Greenland_Western
    {  325,  883,  289235,  289252,  289282, 17, 30, 32, }, // Hawaii_Aleutian
    {  325,  920,       0,  289314,       0,  0, 17,  0, }, // Indian_Ocean
    {  325, 1264,  289331,  289343,  289368, 12, 25, 27, }, // Mauritius
    {  325, 1281,  289395,  289412,  289442, 17, 30, 32, }, // Mexico_Pacific
    {  325, 1358,   22394,  289474,  289502, 15, 28, 30, }, // Newfoundland
    {  325, 1502,  289532,  289556,  289593, 24, 37, 39, }, // Pierre_Miquelon
    {  325, 1554,       0,   23092,       0,  0, 10,  0, }, // Reunion
    {  325, 1602,       0,  289632,       0,  0, 13,  0, }, // Seychelles
    {  326,    0,  289645,       0,       0, 12,  0,  0, }, // Acre Galician/Latin/Spain
    {  326,    5,       0,  289657,       0,  0, 18,  0, }, // Afghanistan
    {  326,   17,       0,  289675,       0,  0, 22,  0, }, // Africa_Central
    {  326,   32,       0,  289697,       0,  0, 23,  0, }, // Africa_Eastern
    {  326,   65,       0,  289720,       0,  0, 25,  0, }, // Africa_Southern
    {  326,   81,  289745,  289770,  289804, 25, 34, 34, }, // Africa_Western
    {  326,  107,  289838,  289852,  289875, 14, 23, 23, }, // Alaska
    {  326,  135,  289898,  289914,  289939, 16, 25, 25, }, // Amazon
    {  326,  142,  289964,  289990,  290025, 26, 35, 35, }, // America_Central
    {  326,  158,  290060,  290091,  290131, 31, 40, 40, }, // America_Eastern
    {  326,  174,  290171,  290204,  290246, 33, 42, 42, }, // America_Mountain
    {  326,  191,  290288,  290322,  290365, 34, 43, 43, }, // America_Pacific
    {  326,  207,  290408,  290422,  290448, 14, 26, 26, }, // Anadyr
    {  326,  214,  290474,  290486,  290507, 12, 21, 21, }, // Apia
    {  326,  232,  290528,  290538,  290557, 10, 19, 19, }, // Arabian
    {  326,  240,  290576,  290593,  290619, 17, 26, 26, }, // Argentina
    {  326,  250,  290645,  290673,  290710, 28, 37, 37, }, // Argentina_Western
    {  326,  268,  290747,  290762,  290786, 15, 24, 24, }, // Armenia
    {  326,  286,  290810,  290827,  290853, 17, 26, 26, }, // Atlantic
    {  326,  295,  290879,  290904,  290938, 25, 34, 34, }, // Australia_Central
    {  326,  313,  290972,  291008,  291053, 36, 45, 45, }, // Australia_CentralWestern
    {  326,  338,  291098,  291124,  291159, 26, 35, 35, }, // Australia_Eastern
    {  326,  356,  290972,  291008,  291053, 28, 37, 37, }, // Australia_Western
    {  326,  374,  291194,  291212,  291239, 18, 27, 27, }, // Azerbaijan
    {  326,  385,  291266,  291281,  291305, 15, 24, 24, }, // Azores
    {  326,  397,  291329,  291347,  291374, 18, 27, 27, }, // Bangladesh
    {  326,  415,       0,  291401,       0,  0, 13,  0, }, // Bhutan
    {  326,  422,       0,  291414,       0,  0, 15,  0, }, // Bolivia
    {  326,  437,  291429,  291445,  291470, 16, 25, 25, }, // Brasilia
    {  326,  454,       0,  291495,       0,  0, 25,  0, }, // Brunei
    {  326,  461,  291520,  291538,  291565, 18, 27, 27, }, // Cape_Verde
    {  326,  478,       0,  291592,       0,  0, 22,  0, }, // Chamorro
    {  326,  487,  291614,  291629,  291653, 15, 24, 24, }, // Chatham
    {  326,  495,  291677,  291690,  291712, 13, 22, 22, }, // Chile
    {  326,  501,  291734,  291747,  291769, 13, 22, 22, }, // China
    {  326,  507,       0,  291791,       0,  0, 22,  0, }, // Christmas
    {  326,  517,       0,  291813,       0,  0, 20,  0, }, // Cocos
    {  326,  523,  291833,  291849,  291874, 16, 25, 25, }, // Colombia
    {  326,  532,  291899,  291918,  291946, 19, 28, 34, }, // Cook
    {  326,  537,  291980,  291992,  292013, 12, 21, 21, }, // Cuba
    {  326,  548,       0,  292034,       0,  0, 13,  0, }, // Davis
    {  326,  564,       0,  292047,       0,  0, 24,  0, }, // DumontDUrville
    {  326,  601,       0,  292071,       0,  0, 19,  0, }, // East_Timor
    {  326,  612,  292090,  292112,  292143, 22, 31, 31, }, // Easter
    {  326,  619,       0,  292174,       0,  0, 15,  0, }, // Ecuador
    {  326,  627,  292189,  292211,  292242, 22, 31, 31, }, // Europe_Central
    {  326,  642,  292273,  292296,  292328, 23, 32, 32, }, // Europe_Eastern
    {  326,  657,       0,  292360,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {  326,  680,  292389,  292414,  292448, 25, 34, 34, }, // Europe_Western
    {  326,  695,  292482,  292505,  292537, 23, 32, 32, }, // Falkland
    {  326,  704,  292569,  292581,  292602, 12, 21, 21, }, // Fiji
    {  326,  709,       0,  292623,       0,  0, 23,  0, }, // French_Guiana
    {  326,  723,       0,  292646,       0,  0, 47,  0, }, // French_Southern
    {  326,  746,       0,  292693,       0,  0, 18,  0, }, // Galapagos
    {  326,  756,       0,  292711,       0,  0, 15,  0, }, // Gambier
    {  326,  764,  292726,  292741,  292765, 15, 24, 24, }, // Georgia
    {  326,  772,       0,  292789,       0,  0, 22,  0, }, // Gilbert_Islands
    {  326,  788,       0,  292811,       0,  0, 30,  0, }, // GMT
    {  326,  830,  292841,  292869,  292906, 28, 37, 37, }, // Greenland_Eastern
    {  326,  848,  292943,  292973,  293012, 30, 39, 39, }, // Greenland_Western
    {  326,  871,       0,  293051,       0,  0, 13,  0, }, // Gulf
    {  326,  876,       0,  292623,       0,  0, 14,  0, }, // Guyana
    {  326,  883,  293064,  293094,  293133, 30, 39, 39, }, // Hawaii_Aleutian
    {  326,  899,  293172,  293189,  293215, 17, 26, 26, }, // Hong_Kong
    {  326,  909,  293241,  293253,  293274, 12, 21, 21, }, // Hovd
    {  326,  914,       0,  293295,       0,  0, 13,  0, }, // India
    {  326,  920,       0,  293308,       0,  0, 21,  0, }, // Indian_Ocean
    {  326,  933,       0,  293329,       0,  0, 17,  0, }, // Indochina
    {  326,  943,       0,  293346,       0,  0, 25,  0, }, // Indonesia_Central
    {  326,  961,       0,  293371,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  326,  979,       0,  293397,       0,  0, 28,  0, }, // Indonesia_Western
    {  326,  997,  293425,  293437,  293458, 12, 21, 21, }, // Iran
    {  326, 1008,  293479,  293494,  293518, 15, 24, 24, }, // Irkutsk
    {  326, 1016,  293542,  293556,  293579, 14, 23, 23, }, // Israel
    {  326, 1023,  293602,  293615,  293637, 13, 22, 22, }, // Japan
    {  326, 1029,  293659,  293700,  293744, 41, 44, 44, }, // Kamchatka
    {  326, 1047,       0,  293788,       0,  0, 18,  0, }, // Kazakhstan
    {  326, 1058,       0,  293806,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  326, 1077,       0,  293833,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  326, 1106,  293862,  293875,  293897, 13, 22, 22, }, // Korea
    {  326, 1112,       0,  293919,       0,  0, 14,  0, }, // Kosrae
    {  326, 1119,  293933,  293952,  293980, 19, 28, 28, }, // Krasnoyarsk
    {  326, 1151,       0,  294008,       0,  0, 20,  0, }, // Kyrgystan
    {  326, 1175,       0,  294028,       0,  0, 22,  0, }, // Line_Islands
    {  326, 1188,  294050,  294067,  294093, 17, 26, 26, }, // Lord_Howe
    {  326, 1204,  294119,  294134,  294158, 15, 24, 24, }, // Magadan
    {  326, 1219,       0,  294182,       0,  0, 16,  0, }, // Malaysia
    {  326, 1228,       0,  294198,       0,  0, 17,  0, }, // Maldives
    {  326, 1237,       0,  294215,       0,  0, 18,  0, }, // Marquesas
    {  326, 1247,       0,  294233,       0,  0, 23,  0, }, // Marshall_Islands
    {  326, 1264,  294256,  294272,  294297, 16, 25, 25, }, // Mauritius
    {  326, 1274,       0,  294322,       0,  0, 14,  0, }, // Mawson
    {  326, 1281,  294336,  294361,  294395, 25, 34, 34, }, // Mexico_Pacific
    {  326, 1296,  294429,  294448,  294476, 19, 28, 28, }, // Mongolia
    {  326, 1305,  294504,  294519,  294543, 15, 24, 24, }, // Moscow
    {  326, 1312,       0,  294567,       0,  0, 15,  0, }, // Myanmar
    {  326, 1320,       0,  294582,       0,  0, 13,  0, }, // Nauru
    {  326, 1326,       0,  294595,       0,  0, 13,  0, }, // Nepal
    {  326, 1332,  294608,  294630,  294661, 22, 31, 31, }, // New_Caledonia
    {  326, 1346,  294692,  294713,  294743, 21, 30, 30, }, // New_Zealand
    {  326, 1358,  294773,  294791,  294818, 18, 27, 27, }, // Newfoundland
    {  326, 1371,       0,  294845,       0,  0, 12,  0, }, // Niue
    {  326, 1376,  294857,  294877,  294906, 20, 29, 29, }, // Norfolk
    {  326, 1384,  294935,  294962,  294998, 27, 36, 36, }, // Noronha
    {  326, 1406,  295034,  295053,  295081, 19, 28, 28, }, // Novosibirsk
    {  326, 1418,  295109,  295121,  295142, 12, 21, 21, }, // Omsk
    {  326, 1428,  295163,  295180,  295206, 17, 26, 26, }, // Pakistan
    {  326, 1437,       0,  295232,       0,  0, 13,  0, }, // Palau
    {  326, 1443,       0,  295245,       0,  0, 25,  0, }, // Papua_New_Guinea
    {  326, 1460,  295270,  295286,  295311, 16, 25, 25, }, // Paraguay
    {  326, 1469,  295336,  295348,  295369, 12, 21, 21, }, // Peru
    {  326, 1474,  295390,  295407,  295433, 17, 26, 26, }, // Philippines
    {  326, 1486,       0,  295459,       0,  0, 20,  0, }, // Phoenix_Islands
    {  326, 1502,  295479,  295511,  295552, 32, 41, 41, }, // Pierre_Miquelon
    {  326, 1518,       0,  295593,       0,  0, 16,  0, }, // Pitcairn
    {  326, 1527,       0,  295609,       0,  0, 15,  0, }, // Ponape
    {  326, 1534,       0,   63381,       0,  0, 17,  0, }, // Pyongyang
    {  326, 1554,       0,  295624,       0,  0, 15,  0, }, // Reunion
    {  326, 1562,       0,  295639,       0,  0, 15,  0, }, // Rothera
    {  326, 1570,  295654,  295670,  295695, 16, 25, 25, }, // Sakhalin
    {  326, 1579,  295720,  295734,  295760, 14, 26, 26, }, // Samara
    {  326, 1596,  295786,  295799,  295821, 13, 22, 22, }, // Samoa
    {  326, 1602,       0,  295843,       0,  0, 19,  0, }, // Seychelles
    {  326, 1624,       0,  295862,       0,  0, 16,  0, }, // Singapore
    {  326, 1634,       0,  295878,       0,  0, 22,  0, }, // Solomon
    {  326, 1642,       0,  295900,       0,  0, 22,  0, }, // South_Georgia
    {  326, 1656,       0,  295922,       0,  0, 16,  0, }, // Suriname
    {  326, 1676,       0,  295938,       0,  0, 13,  0, }, // Syowa
    {  326, 1682,       0,  295951,       0,  0, 14,  0, }, // Tahiti
    {  326, 1689,  295965,  295979,  296002, 14, 23, 23, }, // Taipei
    {  326, 1696,       0,  296025,       0,  0, 19,  0, }, // Tajikistan
    {  326, 1724,       0,  296044,       0,  0, 15,  0, }, // Tokelau
    {  326, 1732,  296059,  296072,  296094, 13, 22, 22, }, // Tonga
    {  326, 1738,       0,  296116,       0,  0, 13,  0, }, // Truk
    {  326, 1750,  296129,  296149,  296178, 20, 29, 29, }, // Turkmenistan
    {  326, 1763,       0,  296207,       0,  0, 14,  0, }, // Tuvalu
    {  326, 1777,  296221,  296236,  296260, 15, 24, 24, }, // Uruguay
    {  326, 1792,  296284,  296302,  296329, 18, 27, 27, }, // Uzbekistan
    {  326, 1803,  296356,  296371,  296395, 15, 24, 24, }, // Vanuatu
    {  326, 1811,       0,  296419,       0,  0, 17,  0, }, // Venezuela
    {  326, 1821,  296436,  296455,  296483, 19, 28, 28, }, // Vladivostok
    {  326, 1833,  296511,  296529,  296556, 18, 27, 27, }, // Volgograd
    {  326, 1843,       0,  296583,       0,  0, 14,  0, }, // Vostok
    {  326, 1850,       0,  296597,       0,  0, 17,  0, }, // Wake
    {  326, 1855,       0,  296614,       0,  0, 23,  0, }, // Wallis
    {  326, 1862,  296637,  296652,  296676, 15, 24, 24, }, // Yakutsk
    {  326, 1870,  296700,  296721,  296751, 21, 30, 30, }, // Yekaterinburg
    {  326, 1892,       0,  296781,       0,  0, 13,  0, }, // Yukon
    {  330,    5,       0,  296794,       0,  0, 13,  0, }, // Afghanistan Georgian/Georgian/Georgia
    {  330,   17,       0,  296807,       0,  0, 22,  0, }, // Africa_Central
    {  330,   32,       0,  296829,       0,  0, 22,  0, }, // Africa_Eastern
    {  330,   65,       0,  296851,       0,  0, 19,  0, }, // Africa_Southern
    {  330,   81,  296870,  296890,  296922, 20, 32, 29, }, // Africa_Western
    {  330,  107,  296951,  296962,  296985, 11, 23, 20, }, // Alaska
    {  330,  135,  297005,  297018,  297043, 13, 25, 22, }, // Amazon
    {  330,  142,  297065,  297097,  297141, 32, 44, 41, }, // America_Central
    {  330,  158,  297182,  297216,  297262, 34, 46, 43, }, // America_Eastern
    {  330,  174,  297305,  297336,  297379, 31, 43, 40, }, // America_Mountain
    {  330,  191,  297419,  297455,  297503, 36, 48, 45, }, // America_Pacific
    {  330,  214,  297548,  297557,  297578,  9, 21, 18, }, // Apia
    {  330,  232,  297596,  297608,  297632, 12, 24, 21, }, // Arabian
    {  330,  240,  297653,  297667,  297693, 14, 26, 23, }, // Argentina
    {  330,  250,  297716,  297739,  297774, 23, 35, 32, }, // Argentina_Western
    {  330,  268,  297806,  297818,  297842, 12, 24, 21, }, // Armenia
    {  330,  286,  297863,  297885,  297919, 22, 34, 31, }, // Atlantic
    {  330,  295,  297950,  297975,  298012, 25, 37, 34, }, // Australia_Central
    {  330,  313,  298046,  298083,  298132, 37, 49, 46, }, // Australia_CentralWestern
    {  330,  338,  298178,  298203,  298240, 25, 37, 34, }, // Australia_Eastern
    {  330,  356,  298060,  298097,  298146, 23, 35, 32, }, // Australia_Western
    {  330,  374,  298274,  298290,  298318, 16, 28, 25, }, // Azerbaijan
    {  330,  385,  298343,  298364,  298397, 21, 33, 30, }, // Azores
    {  330,  397,  298427,  298442,  298469, 15, 27, 24, }, // Bangladesh
    {  330,  415,       0,  298493,       0,  0, 11,  0, }, // Bhutan
    {  330,  422,       0,  298504,       0,  0, 12,  0, }, // Bolivia
    {  330,  437,  298516,  298529,  298554, 13, 25, 22, }, // Brasilia
    {  330,  454,       0,  298576,       0,  0, 22,  0, }, // Brunei
    {  330,  461,  298598,  298613,  298640, 15, 27, 24, }, // Cape_Verde
    {  330,  478,       0,  298664,       0,  0, 11,  0, }, // Chamorro
    {  330,  487,  298675,  298686,  298709, 11, 23, 20, }, // Chatham
    {  330,  495,  298729,  298738,  298759,  9, 21, 18, }, // Chile
    {  330,  501,  298777,  298788,  298811, 11, 23, 22, }, // China
    {  330,  507,       0,  298833,       0,  0, 18,  0, }, // Christmas
    {  330,  517,       0,  298851,       0,  0, 22,  0, }, // Cocos
    {  330,  523,  298873,  298886,  298911, 13, 25, 22, }, // Colombia
    {  330,  532,  298933,  298953,  298985, 20, 32, 38, }, // Cook
    {  330,  537,  299023,  299032,  299053,  9, 21, 18, }, // Cuba
    {  330,  548,       0,  299071,       0,  0, 11,  0, }, // Davis
    {  330,  564,       0,  299082,       0,  0, 18,  0, }, // DumontDUrville
    {  330,  601,       0,  299100,       0,  0, 22,  0, }, // East_Timor
    {  330,  612,  299122,  299143,  299176, 21, 33, 30, }, // Easter
    {  330,  619,       0,  299206,       0,  0, 13,  0, }, // Ecuador
    {  330,  627,  299219,  299241,  299275, 22, 34, 31, }, // Europe_Central
    {  330,  642,  299306,  299328,  299362, 22, 34, 31, }, // Europe_Eastern
    {  330,  657,       0,  299393,       0,  0, 30,  0, }, // Europe_Further_Eastern
    {  330,  680,  299423,  299443,  299475, 20, 32, 29, }, // Europe_Western
    {  330,  695,  299504,  299529,  299566, 25, 37, 34, }, // Falkland
    {  330,  704,  299600,  299609,  299630,  9, 21, 18, }, // Fiji
    {  330,  709,       0,  299648,       0,  0, 23,  0, }, // French_Guiana
    {  330,  723,       0,  299671,       0,  0, 37,  0, }, // French_Southern
    {  330,  746,       0,  299708,       0,  0, 15,  0, }, // Galapagos
    {  330,  756,       0,  299723,       0,  0, 13,  0, }, // Gambier
    {  330,  764,  299736,  299751,  299778, 15, 27, 24, }, // Georgia
    {  330,  772,       0,  299802,       0,  0, 24,  0, }, // Gilbert_Islands
    {  330,  788,       0,  299826,       0,  0, 21,  0, }, // GMT
    {  330,  830,  299847,  299873,  299911, 26, 38, 35, }, // Greenland_Eastern
    {  330,  848,  299946,  299970,  300006, 24, 36, 33, }, // Greenland_Western
    {  330,  871,       0,  300039,       0,  0, 31,  0, }, // Gulf
    {  330,  876,       0,  300070,       0,  0, 11,  0, }, // Guyana
    {  330,  883,  300081,  300103,  300137, 22, 34, 31, }, // Hawaii_Aleutian
    {  330,  899,  300168,  300181,  300206, 13, 25, 22, }, // Hong_Kong
    {  330,  909,  300228,  300238,  300260, 10, 22, 19, }, // Hovd
    {  330,  914,       0,  300279,       0,  0, 12,  0, }, // India
    {  330,  920,       0,  300291,       0,  0, 31,  0, }, // Indian_Ocean
    {  330,  933,       0,  300322,       0,  0, 15,  0, }, // Indochina
    {  330,  943,       0,  300337,       0,  0, 25,  0, }, // Indonesia_Central
    {  330,  961,       0,  300362,       0,  0, 25,  0, }, // Indonesia_Eastern
    {  330,  979,       0,  300387,       0,  0, 23,  0, }, // Indonesia_Western
    {  330,  997,  300410,  300420,  300442, 10, 22, 21, }, // Iran
    {  330, 1008,  300463,  300476,  300501, 13, 25, 22, }, // Irkutsk
    {  330, 1016,  300523,  300535,  300559, 12, 24, 21, }, // Israel
    {  330, 1023,  300580,  300592,  300616, 12, 24, 21, }, // Japan
    {  330, 1047,       0,  300637,       0,  0, 13,  0, }, // Kazakhstan
    {  330, 1058,       0,  300650,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  330, 1077,       0,  300674,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  330, 1106,  300696,  300706,  300728, 10, 22, 19, }, // Korea
    {  330, 1112,       0,  300747,       0,  0, 10,  0, }, // Kosrae
    {  330, 1119,  300757,  300774,  300803, 17, 29, 26, }, // Krasnoyarsk
    {  330, 1151,       0,  300829,       0,  0, 14,  0, }, // Kyrgystan
    {  330, 1175,       0,  300843,       0,  0, 21,  0, }, // Line_Islands
    {  330, 1188,  300864,  300877,  300902, 13, 25, 22, }, // Lord_Howe
    {  330, 1204,  300924,  300937,  300962, 13, 25, 22, }, // Magadan
    {  330, 1219,       0,  300984,       0,  0, 13,  0, }, // Malaysia
    {  330, 1228,       0,  300997,       0,  0, 14,  0, }, // Maldives
    {  330, 1237,       0,  301011,       0,  0, 23,  0, }, // Marquesas
    {  330, 1247,       0,  301034,       0,  0, 23,  0, }, // Marshall_Islands
    {  330, 1264,  301057,  301069,  301093, 12, 24, 21, }, // Mauritius
    {  330, 1274,       0,  301114,       0,  0, 12,  0, }, // Mawson
    {  330, 1281,  301126,  301153,  301192, 27, 39, 36, }, // Mexico_Pacific
    {  330, 1296,  301228,  301244,  301272, 16, 28, 25, }, // Mongolia
    {  330, 1305,  301297,  301309,  301333, 12, 24, 21, }, // Moscow
    {  330, 1312,       0,  301354,       0,  0, 13,  0, }, // Myanmar
    {  330, 1320,       0,  301367,       0,  0, 10,  0, }, // Nauru
    {  330, 1326,       0,  301377,       0,  0, 11,  0, }, // Nepal
    {  330, 1332,  301388,  301408,  301440, 20, 32, 29, }, // New_Caledonia
    {  330, 1346,  301469,  301488,  301519, 19, 31, 28, }, // New_Zealand
    {  330, 1358,  301547,  301565,  301595, 18, 30, 27, }, // Newfoundland
    {  330, 1371,       0,  301622,       0,  0,  9,  0, }, // Niue
    {  330, 1376,  301631,  301653,  301687, 22, 34, 31, }, // Norfolk
    {  330, 1384,  301718,  301742,  301778, 24, 36, 33, }, // Noronha
    {  330, 1406,  301811,  301828,  301857, 17, 29, 26, }, // Novosibirsk
    {  330, 1418,  301883,  301893,  301915, 10, 22, 19, }, // Omsk
    {  330, 1428,  301934,  301948,  301974, 14, 26, 23, }, // Pakistan
    {  330, 1437,       0,  301997,       0,  0, 10,  0, }, // Palau
    {  330, 1443,       0,  302007,       0,  0, 23,  0, }, // Papua_New_Guinea
    {  330, 1460,  302030,  302043,  302068, 13, 25, 22, }, // Paraguay
    {  330, 1469,  302090,  302099,  302120,  9, 21, 18, }, // Peru
    {  330, 1474,  302138,  302153,  302180, 15, 27, 24, }, // Philippines
    {  330, 1486,       0,  302204,       0,  0, 23,  0, }, // Phoenix_Islands
    {  330, 1502,  302227,  302254,  302293, 27, 39, 36, }, // Pierre_Miquelon
    {  330, 1518,       0,  302329,       0,  0, 13,  0, }, // Pitcairn
    {  330, 1527,       0,  302342,       0,  0, 11,  0, }, // Ponape
    {  330, 1534,       0,  302353,       0,  0, 13,  0, }, // Pyongyang
    {  330, 1554,       0,  302366,       0,  0, 14,  0, }, // Reunion
    {  330, 1562,       0,  302380,       0,  0, 11,  0, }, // Rothera
    {  330, 1570,  302391,  302404,  302429, 13, 25, 22, }, // Sakhalin
    {  330, 1596,  302451,  302461,  302483, 10, 22, 19, }, // Samoa
    {  330, 1602,       0,  302502,       0,  0, 23,  0, }, // Seychelles
    {  330, 1624,       0,  302525,       0,  0, 14,  0, }, // Singapore
    {  330, 1634,       0,  302539,       0,  0, 24,  0, }, // Solomon
    {  330, 1642,       0,  302563,       0,  0, 20,  0, }, // South_Georgia
    {  330, 1656,       0,  302583,       0,  0, 13,  0, }, // Suriname
    {  330, 1676,       0,  302596,       0,  0, 10,  0, }, // Syowa
    {  330, 1682,       0,  302606,       0,  0, 10,  0, }, // Tahiti
    {  330, 1689,  302616,  302627,  302650, 11, 23, 20, }, // Taipei
    {  330, 1696,       0,  302670,       0,  0, 13,  0, }, // Tajikistan
    {  330, 1724,       0,  302683,       0,  0, 12,  0, }, // Tokelau
    {  330, 1732,  302695,  302705,  302727, 10, 22, 19, }, // Tonga
    {  330, 1738,       0,  302746,       0,  0, 10,  0, }, // Truk
    {  330, 1750,  302756,  302771,  302798, 15, 27, 24, }, // Turkmenistan
    {  330, 1763,       0,  302822,       0,  0, 11,  0, }, // Tuvalu
    {  330, 1777,  302833,  302845,  302869, 12, 24, 21, }, // Uruguay
    {  330, 1792,  302890,  302903,  302928, 13, 25, 22, }, // Uzbekistan
    {  330, 1803,  302950,  302962,  302986, 12, 24, 21, }, // Vanuatu
    {  330, 1811,       0,  303007,       0,  0, 14,  0, }, // Venezuela
    {  330, 1821,  303021,  303038,  303067, 17, 29, 26, }, // Vladivostok
    {  330, 1833,  303093,  303108,  303135, 15, 27, 24, }, // Volgograd
    {  330, 1843,       0,  303026,       0,  0, 12,  0, }, // Vostok
    {  330, 1850,       0,  303159,       0,  0, 19,  0, }, // Wake
    {  330, 1855,       0,  303178,       0,  0, 21,  0, }, // Wallis
    {  330, 1862,  303199,  303212,  303237, 13, 25, 22, }, // Yakutsk
    {  330, 1870,  303259,  303277,  303307, 18, 30, 27, }, // Yekaterinburg
    {  330, 1892,       0,  303334,       0,  0, 11,  0, }, // Yukon
    {  331,    0,  303345,  303354,  303369,  9, 15, 15, }, // Acre German/Latin/Germany
    {  331,    5,       0,  303384,       0,  0, 16,  0, }, // Afghanistan
    {  331,   17,       0,  303400,       0,  0, 24,  0, }, // Africa_Central
    {  331,   32,       0,  303424,       0,  0, 20,  0, }, // Africa_Eastern
    {  331,   65,       0,  303444,       0,  0, 20,  0, }, // Africa_Southern
    {  331,   81,  303464,  303485,  303512, 21, 27, 27, }, // Africa_Western
    {  331,  107,  303539,  303550,  303567, 11, 17, 17, }, // Alaska
    {  331,  128,  303584,  303595,  303612, 11, 17, 17, }, // Almaty
    {  331,  135,  303629,  303642,  303661, 13, 19, 19, }, // Amazon
    {  331,  142,  303680,  303709,  303745, 29, 36, 36, }, // America_Central
    {  331,  158,  303781,  303812,  303850, 31, 38, 38, }, // America_Eastern
    {  331,  174,  303888,  303908,  303934, 20, 26, 26, }, // America_Mountain
    {  331,  191,  303960,  303992,  304031, 32, 39, 39, }, // America_Pacific
    {  331,  207,  304070,  304081,  304098, 11, 17, 17, }, // Anadyr
    {  331,  214,  304115,  304124,  304139,  9, 15, 15, }, // Apia
    {  331,  219,  304154,  304164,  304180, 10, 16, 16, }, // Aqtau
    {  331,  225,  304196,  304207,  304224, 11, 17, 17, }, // Aqtobe
    {  331,  232,  304241,  304255,  304275, 14, 20, 20, }, // Arabian
    {  331,  240,  304295,  304313,  304337, 18, 24, 24, }, // Argentina
    {  331,  250,  304361,  304383,  304411, 22, 28, 28, }, // Argentina_Western
    {  331,  268,  304439,  304454,  304475, 15, 21, 21, }, // Armenia
    {  331,  286,  304496,  304509,  304528, 13, 19, 19, }, // Atlantic
    {  331,  295,  304547,  304571,  304601, 24, 30, 30, }, // Australia_Central
    {  331,  313,  304631,  304661,  304697, 30, 36, 36, }, // Australia_CentralWestern
    {  331,  338,  304733,  304753,  304779, 20, 26, 26, }, // Australia_Eastern
    {  331,  356,  304640,  304670,  304706, 21, 27, 27, }, // Australia_Western
    {  331,  374,  304805,  304828,  304857, 23, 29, 29, }, // Azerbaijan
    {  331,  385,  304886,  304897,  304914, 11, 17, 17, }, // Azores
    {  331,  397,  304931,  304947,  304969, 16, 22, 22, }, // Bangladesh
    {  331,  415,       0,  304991,       0,  0, 11,  0, }, // Bhutan
    {  331,  422,       0,  305002,       0,  0, 18,  0, }, // Bolivia
    {  331,  437,  305020,  305033,  305052, 13, 19, 19, }, // Brasilia
    {  331,  454,       0,  305071,       0,  0, 22,  0, }, // Brunei
    {  331,  461,  305093,  305108,  305129, 15, 21, 21, }, // Cape_Verde
    {  331,  472,       0,  305150,       0,  0, 10,  0, }, // Casey
    {  331,  478,       0,  305160,       0,  0, 13,  0, }, // Chamorro
    {  331,  487,  305173,  305185,  305203, 12, 18, 18, }, // Chatham
    {  331,  495,  305221,  305237,  305259, 16, 22, 22, }, // Chile
    {  331,  501,  305281,  305297,  305319, 16, 22, 22, }, // China
    {  331,  507,       0,  305341,       0,  0, 20,  0, }, // Christmas
    {  331,  517,       0,  305361,       0,  0, 16,  0, }, // Cocos
    {  331,  523,  305377,  305396,  305421, 19, 25, 25, }, // Colombia
    {  331,  532,  305446,  305461,  305482, 15, 21, 21, }, // Cook
    {  331,  537,  305503,  305518,  305539, 15, 21, 21, }, // Cuba
    {  331,  548,       0,  305560,       0,  0, 10,  0, }, // Davis
    {  331,  564,       0,  305570,       0,  0, 21,  0, }, // DumontDUrville
    {  331,  601,       0,  305591,       0,  0, 13,  0, }, // East_Timor
    {  331,  612,  305604,  305619,  305640, 15, 21, 21, }, // Easter
    {  331,  619,       0,  305661,       0,  0, 20,  0, }, // Ecuador
    {  331,  627,  305681,  305703,  305731, 22, 28, 28, }, // Europe_Central
    {  331,  642,  305759,  305778,  305803, 19, 25, 25, }, // Europe_Eastern
    {  331,  657,       0,  305828,       0,  0, 18,  0, }, // Europe_Further_Eastern
    {  331,  680,  305846,  305866,  305892, 20, 26, 26, }, // Europe_Western
    {  331,  695,  305918,  305937,  305962, 19, 25, 25, }, // Falkland
    {  331,  704,  305987,  305999,  306017, 12, 18, 18, }, // Fiji
    {  331,  709,       0,  306035,       0,  0, 24,  0, }, // French_Guiana
    {  331,  723,       0,  306059,       0,  0, 42,  0, }, // French_Southern
    {  331,  746,       0,  306101,       0,  0, 14,  0, }, // Galapagos
    {  331,  756,       0,  306115,       0,  0, 12,  0, }, // Gambier
    {  331,  764,  306127,  306142,  306163, 15, 21, 21, }, // Georgia
    {  331,  772,       0,  306184,       0,  0, 19,  0, }, // Gilbert_Islands
    {  331,  788,       0,  306203,       0,  0, 23,  0, }, // GMT
    {  331,  830,  306226,  306242,  306264, 16, 22, 22, }, // Greenland_Eastern
    {  331,  848,  306286,  306303,  306326, 17, 23, 23, }, // Greenland_Western
    {  331,  866,       0,  306349,       0,  0,  9,  0, }, // Guam
    {  331,  871,       0,  306358,       0,  0,  9,  0, }, // Gulf
    {  331,  876,       0,  306367,       0,  0, 11,  0, }, // Guyana
    {  331,  883,  306378,  306397,  306422, 19, 25, 25, }, // Hawaii_Aleutian
    {  331,  899,  306447,  306460,  306479, 13, 19, 19, }, // Hong_Kong
    {  331,  909,  306498,  306508,  306524, 10, 16, 16, }, // Hovd
    {  331,  914,       0,  306540,       0,  0, 19,  0, }, // India
    {  331,  920,       0,  306559,       0,  0, 20,  0, }, // Indian_Ocean
    {  331,  933,       0,  306579,       0,  0, 14,  0, }, // Indochina
    {  331,  943,       0,  306593,       0,  0, 24,  0, }, // Indonesia_Central
    {  331,  961,       0,  306617,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  331,  979,       0,  306637,       0,  0, 21,  0, }, // Indonesia_Western
    {  331,  997,  306658,  306672,  306692, 14, 20, 20, }, // Iran
    {  331, 1008,  306712,  306726,  306746, 14, 20, 20, }, // Irkutsk
    {  331, 1016,  306766,  306782,  306804, 16, 22, 22, }, // Israel
    {  331, 1023,  306826,  306841,  306862, 15, 21, 21, }, // Japan
    {  331, 1029,  306883,  306899,  306921, 16, 22, 22, }, // Kamchatka
    {  331, 1047,       0,  306943,       0,  0, 16,  0, }, // Kazakhstan
    {  331, 1058,       0,  306959,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  331, 1077,       0,  306978,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  331, 1106,  306998,  307014,  307036, 16, 22, 22, }, // Korea
    {  331, 1112,       0,  307058,       0,  0, 11,  0, }, // Kosrae
    {  331, 1119,  307069,  307087,  307111, 18, 24, 24, }, // Krasnoyarsk
    {  331, 1151,       0,  307135,       0,  0, 16,  0, }, // Kyrgystan
    {  331, 1161,       0,  307151,       0,  0, 14,  0, }, // Lanka
    {  331, 1175,       0,  307165,       0,  0, 17,  0, }, // Line_Islands
    {  331, 1188,  307182,  307196,  307216, 14, 20, 20, }, // Lord_Howe
    {  331, 1198,  307236,  307246,  307262, 10, 16, 16, }, // Macau
    {  331, 1204,  307278,  307290,  307308, 12, 18, 18, }, // Magadan
    {  331, 1219,       0,  307326,       0,  0, 16,  0, }, // Malaysia
    {  331, 1228,       0,  307342,       0,  0, 14,  0, }, // Maldives
    {  331, 1237,       0,  307356,       0,  0, 14,  0, }, // Marquesas
    {  331, 1247,       0,  307370,       0,  0, 19,  0, }, // Marshall_Islands
    {  331, 1264,  307389,  307403,  307423, 14, 20, 20, }, // Mauritius
    {  331, 1274,       0,  307443,       0,  0, 11,  0, }, // Mawson
    {  331, 1281,  307454,  307478,  307509, 24, 31, 31, }, // Mexico_Pacific
    {  331, 1296,  307540,  307556,  307578, 16, 22, 22, }, // Mongolia
    {  331, 1305,  307600,  307613,  307632, 13, 19, 19, }, // Moscow
    {  331, 1312,       0,  307651,       0,  0, 12,  0, }, // Myanmar
    {  331, 1320,       0,  307663,       0,  0, 10,  0, }, // Nauru
    {  331, 1326,       0,  307673,       0,  0, 17,  0, }, // Nepal
    {  331, 1332,  307690,  307710,  307736, 20, 26, 26, }, // New_Caledonia
    {  331, 1346,  307762,  307777,  307798, 15, 21, 21, }, // New_Zealand
    {  331, 1358,  307819,  307835,  307857, 16, 22, 22, }, // Newfoundland
    {  331, 1371,       0,  307879,       0,  0,  9,  0, }, // Niue
    {  331, 1376,  307888,  307905,  307928, 17, 23, 23, }, // Norfolk
    {  331, 1384,  307951,  307975,  308005, 24, 30, 30, }, // Noronha
    {  331, 1392,       0,  308035,       0,  0, 23,  0, }, // North_Mariana
    {  331, 1406,  308058,  308076,  308100, 18, 24, 24, }, // Novosibirsk
    {  331, 1418,  308124,  308135,  308152, 11, 17, 17, }, // Omsk
    {  331, 1428,  308169,  308187,  308211, 18, 24, 24, }, // Pakistan
    {  331, 1437,       0,  308235,       0,  0, 10,  0, }, // Palau
    {  331, 1443,       0,  308245,       0,  0, 20,  0, }, // Papua_New_Guinea
    {  331, 1460,  308265,  308283,  308307, 18, 24, 24, }, // Paraguay
    {  331, 1469,  308331,  308347,  308369, 16, 22, 22, }, // Peru
    {  331, 1474,  308391,  308410,  308435, 19, 25, 25, }, // Philippines
    {  331, 1486,       0,  308460,       0,  0, 18,  0, }, // Phoenix_Islands
    {  331, 1502,  308478,  308506,  308540, 28, 34, 34, }, // Pierre_Miquelon
    {  331, 1518,       0,  308574,       0,  0, 19,  0, }, // Pitcairn
    {  331, 1527,       0,  308593,       0,  0, 11,  0, }, // Ponape
    {  331, 1534,       0,  308604,       0,  0, 14,  0, }, // Pyongyang
    {  331, 1544,  308618,  308633,  308654, 15, 21, 20, }, // Qyzylorda
    {  331, 1554,       0,  308674,       0,  0, 12,  0, }, // Reunion
    {  331, 1562,       0,  308686,       0,  0, 12,  0, }, // Rothera
    {  331, 1570,  308698,  308711,  308730, 13, 19, 19, }, // Sakhalin
    {  331, 1579,  308749,  308760,  308777, 11, 17, 17, }, // Samara
    {  331, 1596,  308794,  308804,  308820, 10, 16, 16, }, // Samoa
    {  331, 1602,       0,  308836,       0,  0, 15,  0, }, // Seychelles
    {  331, 1624,       0,  308851,       0,  0, 24,  0, }, // Singapore
    {  331, 1634,       0,  308875,       0,  0, 14,  0, }, // Solomon
    {  331, 1642,       0,  308889,       0,  0, 18,  0, }, // South_Georgia
    {  331, 1656,       0,  308907,       0,  0, 13,  0, }, // Suriname
    {  331, 1676,       0,  308920,       0,  0, 10,  0, }, // Syowa
    {  331, 1682,       0,  308930,       0,  0, 11,  0, }, // Tahiti
    {  331, 1689,  308941,  308952,  308969, 11, 17, 17, }, // Taipei
    {  331, 1696,       0,  308986,       0,  0, 18,  0, }, // Tajikistan
    {  331, 1724,       0,  309004,       0,  0, 12,  0, }, // Tokelau
    {  331, 1732,  309016,  309031,  309052, 15, 21, 21, }, // Tonga
    {  331, 1738,       0,  309073,       0,  0, 10,  0, }, // Truk
    {  331, 1750,  309083,  309100,  309123, 17, 23, 23, }, // Turkmenistan
    {  331, 1763,       0,  309146,       0,  0, 11,  0, }, // Tuvalu
    {  331, 1777,  309157,  309174,  309197, 17, 23, 23, }, // Uruguay
    {  331, 1792,  309220,  309235,  309256, 15, 21, 21, }, // Uzbekistan
    {  331, 1803,  309277,  309289,  309307, 12, 18, 18, }, // Vanuatu
    {  331, 1811,       0,  309325,       0,  0, 14,  0, }, // Venezuela
    {  331, 1821,  309339,  309357,  309381, 18, 24, 24, }, // Vladivostok
    {  331, 1833,  309405,  309421,  309443, 16, 22, 22, }, // Volgograd
    {  331, 1843,       0,  309465,       0,  0, 11,  0, }, // Vostok
    {  331, 1850,       0,  309476,       0,  0, 15,  0, }, // Wake
    {  331, 1855,       0,  309491,       0,  0, 22,  0, }, // Wallis
    {  331, 1862,  309513,  309527,  309547, 14, 20, 20, }, // Yakutsk
    {  331, 1870,  309567,  309587,  309613, 20, 26, 26, }, // Yekaterinburg
    {  331, 1892,       0,  309639,       0,  0, 10,  0, }, // Yukon
    {  337,  454,       0,  309649,       0,  0, 11,  0, }, // Brunei German/Latin/Switzerland
    {  337, 1634,       0,  309660,       0,  0, 18,  0, }, // Solomon
    {  338,    5,       0,  309678,       0,  0, 14,  0, }, // Afghanistan Greek/Greek/Greece
    {  338,   17,       0,  309692,       0,  0, 21,  0, }, // Africa_Central
    {  338,   32,       0,  309713,       0,  0, 22,  0, }, // Africa_Eastern
    {  338,   65,       0,  309735,       0,  0, 28,  0, }, // Africa_Southern
    {  338,   81,  309763,  309782,  309811, 19, 29, 26, }, // Africa_Western
    {  338,  107,  309837,  309848,  309869, 11, 21, 18, }, // Alaska
    {  338,  135,  309887,  309900,  309923, 13, 23, 20, }, // Amazon
    {  338,  142,  309943,  309972,  310011, 29, 39, 36, }, // America_Central
    {  338,  158,  310047,  310077,  310117, 30, 40, 37, }, // America_Eastern
    {  338,  174,  310154,  310181,  310218, 27, 37, 34, }, // America_Mountain
    {  338,  191,  310252,  310265,  310288, 13, 23, 20, }, // America_Pacific
    {  338,  207,  310308,  310319,  310340, 11, 21, 18, }, // Anadyr
    {  338,  214,  310358,  310366,  310384,  8, 18, 15, }, // Apia
    {  338,  232,  310399,  310410,  310431, 11, 21, 18, }, // Arabian
    {  338,  240,  310449,  310463,  310487, 14, 24, 21, }, // Argentina
    {  338,  250,  310508,  310530,  310562, 22, 32, 29, }, // Argentina_Western
    {  338,  268,  310591,  310603,  310625, 12, 22, 19, }, // Armenia
    {  338,  286,  310644,  310658,  310682, 14, 24, 21, }, // Atlantic
    {  338,  295,  310703,  310727,  310761, 24, 34, 31, }, // Australia_Central
    {  338,  313,  310792,  310820,  310858, 28, 38, 35, }, // Australia_CentralWestern
    {  338,  338,  310893,  310918,  310953, 25, 35, 32, }, // Australia_Eastern
    {  338,  356,  310985,  311007,  311039, 22, 32, 29, }, // Australia_Western
    {  338,  374,  311068,  311084,  311110, 16, 26, 23, }, // Azerbaijan
    {  338,  385,  311133,  311143,  311163, 10, 20, 17, }, // Azores
    {  338,  397,  311180,  311196,  311222, 16, 26, 23, }, // Bangladesh
    {  338,  415,       0,  311245,       0,  0, 11,  0, }, // Bhutan
    {  338,  422,       0,  311256,       0,  0, 12,  0, }, // Bolivia
    {  338,  437,  311268,  311282,  311306, 14, 24, 21, }, // Brasilia
    {  338,  454,       0,  311327,       0,  0, 24,  0, }, // Brunei
    {  338,  461,  311351,  311374,  311407, 23, 33, 30, }, // Cape_Verde
    {  338,  478,       0,  311437,       0,  0, 11,  0, }, // Chamorro
    {  338,  487,  311448,  311458,  311478, 10, 20, 17, }, // Chatham
    {  338,  495,  311495,  311504,  311523,  9, 19, 16, }, // Chile
    {  338,  501,  311539,  311548,  311567,  9, 19, 16, }, // China
    {  338,  507,       0,  311583,       0,  0, 23,  0, }, // Christmas
    {  338,  517,       0,  311606,       0,  0, 15,  0, }, // Cocos
    {  338,  523,  311621,  311634,  311657, 13, 23, 20, }, // Colombia
    {  338,  532,  311677,  311691,  311715, 14, 24, 21, }, // Cook
    {  338,  537,  311736,  311746,  311766, 10, 20, 17, }, // Cuba
    {  338,  548,       0,  311783,       0,  0, 11,  0, }, // Davis
    {  338,  564,       0,  311794,       0,  0, 21,  0, }, // DumontDUrville
    {  338,  601,       0,  311815,       0,  0, 20,  0, }, // East_Timor
    {  338,  612,  311835,  311850,  311875, 15, 25, 22, }, // Easter
    {  338,  619,       0,  311897,       0,  0, 14,  0, }, // Ecuador
    {  338,  627,  311911,  311932,  311963, 21, 31, 28, }, // Europe_Central
    {  338,  642,  311991,  312013,  312045, 22, 32, 29, }, // Europe_Eastern
    {  338,  657,       0,  312074,       0,  0, 32,  0, }, // Europe_Further_Eastern
    {  338,  680,  312106,  312125,  312154, 19, 29, 26, }, // Europe_Western
    {  338,  695,  312180,  312197,  312224, 17, 27, 24, }, // Falkland
    {  338,  704,  312248,  312257,  312276,  9, 19, 16, }, // Fiji
    {  338,  709,       0,  312292,       0,  0, 21,  0, }, // French_Guiana
    {  338,  723,       0,  312313,       0,  0, 34,  0, }, // French_Southern
    {  338,  746,       0,  312347,       0,  0, 15,  0, }, // Galapagos
    {  338,  756,       0,  312362,       0,  0, 12,  0, }, // Gambier
    {  338,  764,  312374,  312386,  312408, 12, 22, 19, }, // Georgia
    {  338,  772,       0,  312427,       0,  0, 19,  0, }, // Gilbert_Islands
    {  338,  788,       0,  312446,       0,  0, 19,  0, }, // GMT
    {  338,  830,  312465,  312491,  312527, 26, 36, 33, }, // Greenland_Eastern
    {  338,  848,  312560,  312583,  312616, 23, 33, 30, }, // Greenland_Western
    {  338,  866,       0,  312646,       0,  0, 10,  0, }, // Guam
    {  338,  871,       0,  312656,       0,  0, 10,  0, }, // Gulf
    {  338,  876,       0,  312666,       0,  0, 12,  0, }, // Guyana
    {  338,  883,  312678,  312704,  312740, 26, 36, 33, }, // Hawaii_Aleutian
    {  338,  899,  312773,  312788,  312813, 15, 25, 22, }, // Hong_Kong
    {  338,  909,  312835,  312844,  312863,  9, 19, 16, }, // Hovd
    {  338,  914,       0,  312879,       0,  0, 10,  0, }, // India
    {  338,  920,       0,  312889,       0,  0, 19,  0, }, // Indian_Ocean
    {  338,  933,       0,  312908,       0,  0, 13,  0, }, // Indochina
    {  338,  943,       0,  312921,       0,  0, 24,  0, }, // Indonesia_Central
    {  338,  961,       0,  312945,       0,  0, 25,  0, }, // Indonesia_Eastern
    {  338,  979,       0,  312970,       0,  0, 22,  0, }, // Indonesia_Western
    {  338,  997,  312992,  313000,  313018,  8, 18, 15, }, // Iran
    {  338, 1008,  313033,  313045,  313067, 12, 22, 19, }, // Irkutsk
    {  338, 1016,  313086,  313096,  313116, 10, 20, 17, }, // Israel
    {  338, 1023,  313133,  313145,  313167, 12, 22, 19, }, // Japan
    {  338, 1029,  313186,  313199,  313237, 13, 38, 35, }, // Kamchatka
    {  338, 1047,       0,  313272,       0,  0, 13,  0, }, // Kazakhstan
    {  338, 1058,       0,  313285,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  338, 1077,       0,  313309,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  338, 1106,  313330,  313340,  313360, 10, 20, 17, }, // Korea
    {  338, 1112,       0,  313377,       0,  0, 10,  0, }, // Kosrae
    {  338, 1119,  313387,  313403,  313429, 16, 26, 23, }, // Krasnoyarsk
    {  338, 1151,       0,  313452,       0,  0, 13,  0, }, // Kyrgystan
    {  338, 1175,       0,  313465,       0,  0, 14,  0, }, // Line_Islands
    {  338, 1188,  313479,  313493,  313517, 14, 24, 21, }, // Lord_Howe
    {  338, 1198,  313538,  313547,  313566,  9, 19, 16, }, // Macau
    {  338, 1204,  313582,  313595,  313618, 13, 23, 20, }, // Magadan
    {  338, 1219,       0,  313638,       0,  0, 13,  0, }, // Malaysia
    {  338, 1228,       0,  313651,       0,  0, 12,  0, }, // Maldives
    {  338, 1237,       0,  313663,       0,  0, 12,  0, }, // Marquesas
    {  338, 1247,       0,  313675,       0,  0, 16,  0, }, // Marshall_Islands
    {  338, 1264,  313691,  313704,  313727, 13, 23, 20, }, // Mauritius
    {  338, 1274,       0,  313747,       0,  0,  9,  0, }, // Mawson
    {  338, 1281,  313756,  313777,  313808, 21, 31, 28, }, // Mexico_Pacific
    {  338, 1296,  313836,  313852,  313878, 16, 26, 23, }, // Mongolia
    {  338, 1305,  313901,  313911,  313931, 10, 20, 17, }, // Moscow
    {  338, 1312,       0,  313948,       0,  0, 11,  0, }, // Myanmar
    {  338, 1320,       0,  313959,       0,  0, 11,  0, }, // Nauru
    {  338, 1326,       0,  313970,       0,  0,  9,  0, }, // Nepal
    {  338, 1332,  313979,  313998,  314027, 19, 29, 26, }, // New_Caledonia
    {  338, 1346,  314053,  314071,  314099, 18, 28, 25, }, // New_Zealand
    {  338, 1358,  314124,  314136,  314158, 12, 22, 19, }, // Newfoundland
    {  338, 1371,       0,  314177,       0,  0,  9,  0, }, // Niue
    {  338, 1376,  314186,  314203,  314230, 17, 27, 24, }, // Norfolk
    {  338, 1384,  314254,  314279,  314314, 25, 35, 32, }, // Noronha
    {  338, 1392,       0,  314346,       0,  0, 27,  0, }, // North_Mariana
    {  338, 1406,  314373,  314389,  314415, 16, 26, 23, }, // Novosibirsk
    {  338, 1418,  314438,  314446,  314464,  8, 18, 15, }, // Omsk
    {  338, 1428,  314479,  314491,  314513, 12, 22, 19, }, // Pakistan
    {  338, 1437,       0,  314532,       0,  0, 10,  0, }, // Palau
    {  338, 1443,       0,  314542,       0,  0, 25,  0, }, // Papua_New_Guinea
    {  338, 1460,  314567,  314581,  314605, 14, 24, 21, }, // Paraguay
    {  338, 1469,  314626,  314635,  314654,  9, 19, 16, }, // Peru
    {  338, 1474,  314670,  314684,  314708, 14, 24, 21, }, // Philippines
    {  338, 1486,       0,  314729,       0,  0, 16,  0, }, // Phoenix_Islands
    {  338, 1502,  314745,  314769,  314803, 24, 34, 31, }, // Pierre_Miquelon
    {  338, 1518,       0,  314834,       0,  0, 11,  0, }, // Pitcairn
    {  338, 1527,       0,  314845,       0,  0, 10,  0, }, // Ponape
    {  338, 1534,       0,  314855,       0,  0, 14,  0, }, // Pyongyang
    {  338, 1554,       0,  314869,       0,  0, 11,  0, }, // Reunion
    {  338, 1562,       0,  314880,       0,  0, 10,  0, }, // Rothera
    {  338, 1570,  314890,  314903,  314926, 13, 23, 20, }, // Sakhalin
    {  338, 1579,  314946,  314957,  314978, 11, 21, 18, }, // Samara
    {  338, 1596,  314996,  315005,  315024,  9, 19, 16, }, // Samoa
    {  338, 1602,       0,  315040,       0,  0, 13,  0, }, // Seychelles
    {  338, 1624,       0,  315053,       0,  0, 15,  0, }, // Singapore
    {  338, 1634,       0,  315068,       0,  0, 20,  0, }, // Solomon
    {  338, 1642,       0,  315088,       0,  0, 19,  0, }, // South_Georgia
    {  338, 1656,       0,  315107,       0,  0, 12,  0, }, // Suriname
    {  338, 1676,       0,  315119,       0,  0,  9,  0, }, // Syowa
    {  338, 1682,       0,  315128,       0,  0, 10,  0, }, // Tahiti
    {  338, 1689,  315138,  315148,  315168, 10, 20, 17, }, // Taipei
    {  338, 1696,       0,  315185,       0,  0, 15,  0, }, // Tajikistan
    {  338, 1724,       0,  315200,       0,  0, 12,  0, }, // Tokelau
    {  338, 1732,  315212,  315222,  315242, 10, 20, 17, }, // Tonga
    {  338, 1738,       0,  315259,       0,  0,  9,  0, }, // Truk
    {  338, 1750,  315268,  315285,  315312, 17, 27, 24, }, // Turkmenistan
    {  338, 1763,       0,  315336,       0,  0, 12,  0, }, // Tuvalu
    {  338, 1777,  315348,  315363,  315388, 15, 25, 22, }, // Uruguay
    {  338, 1792,  315410,  315426,  315452, 16, 26, 23, }, // Uzbekistan
    {  338, 1803,  315475,  315488,  315511, 13, 23, 20, }, // Vanuatu
    {  338, 1811,       0,  315531,       0,  0, 15,  0, }, // Venezuela
    {  338, 1821,  315546,  315561,  315586, 15, 25, 22, }, // Vladivostok
    {  338, 1833,  315608,  315624,  315650, 16, 26, 23, }, // Volgograd
    {  338, 1843,       0,  315673,       0,  0, 10,  0, }, // Vostok
    {  338, 1850,       0,  315683,       0,  0, 16,  0, }, // Wake
    {  338, 1855,       0,  315699,       0,  0, 23,  0, }, // Wallis
    {  338, 1862,  315722,  315735,  315758, 13, 23, 20, }, // Yakutsk
    {  338, 1870,  315778,  315798,  315828, 20, 30, 27, }, // Yekaterinburg
    {  338, 1892,       0,  315855,       0,  0, 11,  0, }, // Yukon
    {  340,  422,       0,  315866,       0,  0, 11,  0, }, // Bolivia Guarani/Latin/Paraguay
    {  340,  619,       0,  315877,       0,  0, 11,  0, }, // Ecuador
    {  340,  746,       0,  315888,       0,  0, 13,  0, }, // Galapagos
    {  340, 1811,       0,  315901,       0,  0, 13,  0, }, // Venezuela
    {  341,    0,  315914,  315921,  315938,  7, 17, 15, }, // Acre Gujarati/Gujarati/India
    {  341,    5,       0,  315953,       0,  0, 15,  0, }, // Afghanistan
    {  341,   17,       0,  315968,       0,  0, 16,  0, }, // Africa_Central
    {  341,   32,       0,  315984,       0,  0, 17,  0, }, // Africa_Eastern
    {  341,   65,       0,  316001,       0,  0, 23,  0, }, // Africa_Southern
    {  341,   81,  316024,  316042,  316065, 18, 23, 26, }, // Africa_Western
    {  341,  107,  316091,  316102,  316118, 11, 16, 16, }, // Alaska
    {  341,  128,  316134,  316145,  316166, 11, 21, 19, }, // Almaty
    {  341,  135,  316185,  316195,  316210, 10, 15, 18, }, // Amazon
    {  341,  142,  316228,  316255,  316287, 27, 32, 32, }, // America_Central
    {  341,  158,  316319,  316343,  316372, 24, 29, 29, }, // America_Eastern
    {  341,  174,  316401,  316426,  316456, 25, 30, 30, }, // America_Mountain
    {  341,  191,  316486,  316511,  316541, 25, 30, 30, }, // America_Pacific
    {  341,  207,  316571,  316581,  316601, 10, 20, 18, }, // Anadyr
    {  341,  214,  316619,  316628,  316642,  9, 14, 14, }, // Apia
    {  341,  219,  316656,  316666,  316686, 10, 20, 18, }, // Aqtau
    {  341,  225,  316704,  316714,  316734, 10, 20, 18, }, // Aqtobe
    {  341,  232,  316752,  316763,  316779, 11, 16, 16, }, // Arabian
    {  341,  240,  316795,  316810,  316830, 15, 20, 23, }, // Argentina
    {  341,  250,  316853,  316876,  316904, 23, 28, 31, }, // Argentina_Western
    {  341,  268,  316935,  316948,  316966, 13, 18, 21, }, // Armenia
    {  341,  286,  316987,  317000,  317018, 13, 18, 18, }, // Atlantic
    {  341,  295,  317036,  317061,  317091, 25, 30, 30, }, // Australia_Central
    {  341,  313,  317121,  317154,  317192, 33, 38, 38, }, // Australia_CentralWestern
    {  341,  338,  317230,  317253,  317281, 23, 28, 28, }, // Australia_Eastern
    {  341,  356,  317309,  317332,  317360, 23, 28, 28, }, // Australia_Western
    {  341,  374,  317388,  317400,  317417, 12, 17, 20, }, // Azerbaijan
    {  341,  385,  317437,  317447,  317462, 10, 15, 18, }, // Azores
    {  341,  397,  317480,  317494,  317513, 14, 19, 22, }, // Bangladesh
    {  341,  415,       0,  317535,       0,  0,  9,  0, }, // Bhutan
    {  341,  422,       0,  317544,       0,  0, 12,  0, }, // Bolivia
    {  341,  437,  317556,  317570,  317589, 14, 19, 22, }, // Brasilia
    {  341,  454,       0,  317611,       0,  0, 19,  0, }, // Brunei
    {  341,  461,  317630,  317643,  317661, 13, 18, 21, }, // Cape_Verde
    {  341,  478,       0,  317682,       0,  0, 15,  0, }, // Chamorro
    {  341,  487,  317697,  317707,  317722, 10, 15, 15, }, // Chatham
    {  341,  495,  317737,  317745,  317758,  8, 13, 16, }, // Chile
    {  341,  501,  317774,  317781,  317793,  7, 12, 12, }, // China
    {  341,  507,       0,  317805,       0,  0, 19,  0, }, // Christmas
    {  341,  517,       0,  317824,       0,  0, 19,  0, }, // Cocos
    {  341,  523,  317843,  317856,  317874, 13, 18, 21, }, // Colombia
    {  341,  532,  317895,  317912,  317934, 17, 22, 30, }, // Cook
    {  341,  537,  317964,  317974,  317989, 10, 15, 15, }, // Cuba
    {  341,  548,       0,  318004,       0,  0,  9,  0, }, // Davis
    {  341,  564,       0,  318013,       0,  0, 23,  0, }, // DumontDUrville
    {  341,  601,       0,  318036,       0,  0, 15,  0, }, // East_Timor
    {  341,  612,  318051,  318068,  318090, 17, 22, 25, }, // Easter
    {  341,  619,       0,  318115,       0,  0, 12,  0, }, // Ecuador
    {  341,  627,  318127,  318144,  318166, 17, 22, 25, }, // Europe_Central
    {  341,  642,  318191,  318210,  318234, 19, 24, 27, }, // Europe_Eastern
    {  341,  657,       0,  318261,       0,  0, 25,  0, }, // Europe_Further_Eastern
    {  341,  680,  318286,  318306,  318331, 20, 25, 28, }, // Europe_Western
    {  341,  695,  318359,  318381,  318408, 22, 27, 30, }, // Falkland
    {  341,  704,  318438,  318446,  318459,  8, 13, 16, }, // Fiji
    {  341,  709,       0,  318475,       0,  0, 16,  0, }, // French_Guiana
    {  341,  723,       0,  318491,       0,  0, 32,  0, }, // French_Southern
    {  341,  746,       0,  318523,       0,  0, 13,  0, }, // Galapagos
    {  341,  756,       0,  318536,       0,  0, 12,  0, }, // Gambier
    {  341,  764,  318548,  318562,  318581, 14, 19, 22, }, // Georgia
    {  341,  772,       0,  318603,       0,  0, 21,  0, }, // Gilbert_Islands
    {  341,  788,       0,  318624,       0,  0, 18,  0, }, // GMT
    {  341,  830,  318642,  318662,  318687, 20, 25, 28, }, // Greenland_Eastern
    {  341,  848,  318715,  318736,  318762, 21, 26, 29, }, // Greenland_Western
    {  341,  866,       0,  318791,       0,  0, 19,  0, }, // Guam
    {  341,  871,       0,  318810,       0,  0, 13,  0, }, // Gulf
    {  341,  876,       0,  318482,       0,  0,  9,  0, }, // Guyana
    {  341,  883,  318823,  318841,  318864, 18, 23, 23, }, // Hawaii_Aleutian
    {  341,  899,  318887,  318900,  318918, 13, 18, 21, }, // Hong_Kong
    {  341,  909,  318939,  318948,  318962,  9, 14, 17, }, // Hovd
    {  341,  914,       0,  318979,       0,  0, 15,  0, }, // India
    {  341,  920,       0,  318994,       0,  0, 18,  0, }, // Indian_Ocean
    {  341,  933,       0,  319012,       0,  0, 14,  0, }, // Indochina
    {  341,  943,       0,  319026,       0,  0, 20,  0, }, // Indonesia_Central
    {  341,  961,       0,  319046,       0,  0, 23,  0, }, // Indonesia_Eastern
    {  341,  979,       0,  319069,       0,  0, 23,  0, }, // Indonesia_Western
    {  341,  997,  319092,  319100,  319113,  8, 13, 13, }, // Iran
    {  341, 1008,  319126,  319140,  319159, 14, 19, 22, }, // Irkutsk
    {  341, 1016,  319181,  319191,  319206, 10, 15, 15, }, // Israel
    {  341, 1023,  319221,  319230,  319244,  9, 14, 14, }, // Japan
    {  341, 1029,  319258,  319288,  319328, 30, 40, 38, }, // Kamchatka
    {  341, 1047,       0,  319366,       0,  0, 14,  0, }, // Kazakhstan
    {  341, 1058,       0,  319380,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  341, 1077,       0,  319400,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  341, 1106,  319421,  319431,  319446, 10, 15, 15, }, // Korea
    {  341, 1112,       0,  319461,       0,  0,  9,  0, }, // Kosrae
    {  341, 1119,  319470,  319489,  319513, 19, 24, 27, }, // Krasnoyarsk
    {  341, 1151,       0,  319540,       0,  0, 17,  0, }, // Kyrgystan
    {  341, 1161,       0,  319557,       0,  0,  8,  0, }, // Lanka
    {  341, 1175,       0,  319565,       0,  0, 18,  0, }, // Line_Islands
    {  341, 1188,  319583,  319596,  319614, 13, 18, 18, }, // Lord_Howe
    {  341, 1198,  319632,  319640,  319658,  8, 18, 16, }, // Macau
    {  341, 1204,  319674,  319683,  319697,  9, 14, 17, }, // Magadan
    {  341, 1219,       0,  319714,       0,  0, 11,  0, }, // Malaysia
    {  341, 1228,       0,  319725,       0,  0, 10,  0, }, // Maldives
    {  341, 1237,       0,  319735,       0,  0, 14,  0, }, // Marquesas
    {  341, 1247,       0,  319749,       0,  0, 20,  0, }, // Marshall_Islands
    {  341, 1264,  319769,  319781,  319798, 12, 17, 20, }, // Mauritius
    {  341, 1274,       0,  319818,       0,  0,  8,  0, }, // Mawson
    {  341, 1281,  319826,  319846,  319871, 20, 25, 25, }, // Mexico_Pacific
    {  341, 1296,  319896,  319910,  319929, 14, 19, 21, }, // Mongolia
    {  341, 1305,  319950,  319960,  319975, 10, 15, 18, }, // Moscow
    {  341, 1312,       0,  319993,       0,  0, 12,  0, }, // Myanmar
    {  341, 1320,       0,  320005,       0,  0,  8,  0, }, // Nauru
    {  341, 1326,       0,  320013,       0,  0,  9,  0, }, // Nepal
    {  341, 1332,  320022,  320041,  320065, 19, 24, 27, }, // New_Caledonia
    {  341, 1346,  320092,  320107,  320127, 15, 20, 20, }, // New_Zealand
    {  341, 1358,  320147,  320166,  320190, 19, 24, 24, }, // Newfoundland
    {  341, 1371,       0,  320214,       0,  0,  8,  0, }, // Niue
    {  341, 1376,  320222,  320240,  320263, 18, 23, 23, }, // Norfolk
    {  341, 1384,  320286,  320311,  320341, 25, 30, 31, }, // Noronha
    {  341, 1392,       0,  320372,       0,  0, 24,  0, }, // North_Mariana
    {  341, 1406,  320396,  320412,  320433, 16, 21, 24, }, // Novosibirsk
    {  341, 1418,  320457,  320467,  320482, 10, 15, 18, }, // Omsk
    {  341, 1428,  320500,  320513,  320531, 13, 18, 21, }, // Pakistan
    {  341, 1437,       0,  320552,       0,  0,  8,  0, }, // Palau
    {  341, 1443,       0,  320560,       0,  0, 19,  0, }, // Papua_New_Guinea
    {  341, 1460,  320579,  320591,  320608, 12, 17, 20, }, // Paraguay
    {  341, 1469,  320628,  320636,  320649,  8, 13, 16, }, // Peru
    {  341, 1474,  320665,  320677,  320694, 12, 17, 20, }, // Philippines
    {  341, 1486,       0,  320714,       0,  0, 21,  0, }, // Phoenix_Islands
    {  341, 1502,  320735,  320763,  320796, 28, 33, 33, }, // Pierre_Miquelon
    {  341, 1518,       0,  320829,       0,  0, 13,  0, }, // Pitcairn
    {  341, 1527,       0,  320842,       0,  0,  9,  0, }, // Ponape
    {  341, 1534,       0,  320851,       0,  0, 14,  0, }, // Pyongyang
    {  341, 1544,  320865,  320879,  320903, 14, 24, 22, }, // Qyzylorda
    {  341, 1554,       0,  320925,       0,  0, 12,  0, }, // Reunion
    {  341, 1562,       0,  320937,       0,  0, 10,  0, }, // Rothera
    {  341, 1570,  320947,  320957,  320972, 10, 15, 18, }, // Sakhalin
    {  341, 1579,  320990,  320999,  321018,  9, 19, 17, }, // Samara
    {  341, 1596,  321035,  321043,  321056,  8, 13, 16, }, // Samoa
    {  341, 1602,       0,  321072,       0,  0, 10,  0, }, // Seychelles
    {  341, 1624,       0,  321082,       0,  0, 17,  0, }, // Singapore
    {  341, 1634,       0,  321099,       0,  0, 20,  0, }, // Solomon
    {  341, 1642,       0,  321119,       0,  0, 21,  0, }, // South_Georgia
    {  341, 1656,       0,  321140,       0,  0, 11,  0, }, // Suriname
    {  341, 1676,       0,  321151,       0,  0, 10,  0, }, // Syowa
    {  341, 1682,       0,  321161,       0,  0, 10,  0, }, // Tahiti
    {  341, 1689,  321171,  321181,  321196, 10, 15, 15, }, // Taipei
    {  341, 1696,       0,  321211,       0,  0, 15,  0, }, // Tajikistan
    {  341, 1724,       0,  321226,       0,  0, 10,  0, }, // Tokelau
    {  341, 1732,  321236,  321245,  321259,  9, 14, 17, }, // Tonga
    {  341, 1738,       0,  321276,       0,  0,  8,  0, }, // Truk
    {  341, 1750,  321284,  321302,  321325, 18, 23, 26, }, // Turkmenistan
    {  341, 1763,       0,  321351,       0,  0,  9,  0, }, // Tuvalu
    {  341, 1777,  321360,  321371,  321387, 11, 16, 19, }, // Uruguay
    {  341, 1792,  321406,  321422,  321443, 16, 21, 24, }, // Uzbekistan
    {  341, 1803,  321467,  321476,  321490,  9, 14, 17, }, // Vanuatu
    {  341, 1811,       0,  321507,       0,  0, 13,  0, }, // Venezuela
    {  341, 1821,  321520,  321537,  321559, 17, 22, 25, }, // Vladivostok
    {  341, 1833,  321584,  321599,  321619, 15, 20, 23, }, // Volgograd
    {  341, 1843,       0,  321526,       0,  0, 11,  0, }, // Vostok
    {  341, 1850,       0,  321642,       0,  0, 15,  0, }, // Wake
    {  341, 1855,       0,  321657,       0,  0, 22,  0, }, // Wallis
    {  341, 1862,  321679,  321692,  321710, 13, 18, 21, }, // Yakutsk
    {  341, 1870,  321731,  321748,  321770, 17, 22, 25, }, // Yekaterinburg
    {  341, 1892,       0,  321795,       0,  0,  9,  0, }, // Yukon
    {  343,    0,  256643,  256658,  256681, 15, 23, 21, }, // Acre Haitian/Latin/Haiti
    {  343,    5,       0,  256702,       0,  0, 22,  0, }, // Afghanistan
    {  343,   17,       0,  256724,       0,  0, 32,  0, }, // Africa_Central
    {  343,   32,       0,  256756,       0,  0, 32,  0, }, // Africa_Eastern
    {  343,   65,       0,  256788,       0,  0, 35,  0, }, // Africa_Southern
    {  343,   81,  256823,  256849,  256883, 26, 34, 32, }, // Africa_Western
    {  343,  107,  256915,  256932,  256957, 17, 25, 23, }, // Alaska
    {  343,  128,  256980,  256996,  257020, 16, 24, 22, }, // Almaty
    {  343,  135,  257042,  257061,  257088, 19, 27, 25, }, // Amazon
    {  343,  142,  257113,  257143,  257181, 30, 38, 36, }, // America_Central
    {  343,  158,  257217,  257246,  257283, 29, 37, 35, }, // America_Eastern
    {  343,  174,  257318,  257337,  257364, 19, 27, 25, }, // America_Mountain
    {  343,  191,  257389,  257422,  257463, 33, 41, 39, }, // America_Pacific
    {  343,  207,  257502,  257516,  257538, 14, 22, 20, }, // Anadyr
    {  343,  214,  257558,  257570,  257590, 12, 20, 18, }, // Apia
    {  343,  219,  257608,  257622,  257644, 14, 22, 20, }, // Aqtau
    {  343,  225,  257664,  257678,  257700, 14, 22, 20, }, // Aqtobe
    {  343,  232,  257720,  257737,  257762, 17, 25, 23, }, // Arabian
    {  343,  240,  257785,  257805,  257830, 20, 25, 26, }, // Argentina
    {  343,  250,  257856,  257881,  257914, 25, 33, 31, }, // Argentina_Western
    {  343,  268,  257945,  257963,  257989, 18, 26, 21, }, // Armenia
    {  343,  286,  258010,  258031,  258060, 21, 29, 27, }, // Atlantic
    {  343,  295,  258087,  258117,  258155, 30, 38, 36, }, // Australia_Central
    {  343,  313,  258191,  258227,  258271, 36, 44, 42, }, // Australia_CentralWestern
    {  343,  338,  258313,  258342,  258379, 29, 37, 35, }, // Australia_Eastern
    {  343,  356,  258414,  258445,  258484, 31, 39, 37, }, // Australia_Western
    {  343,  374,  258521,  258543,  258573, 22, 30, 25, }, // Azerbaijan
    {  343,  385,  258598,  258614,  258638, 16, 24, 22, }, // Azores
    {  343,  397,  258660,  258679,  258706, 19, 27, 25, }, // Bangladesh
    {  343,  415,       0,  258731,       0,  0, 16,  0, }, // Bhutan
    {  343,  422,       0,  258747,       0,  0, 16,  0, }, // Bolivia
    {  343,  437,  258763,  258780,  258805, 17, 25, 23, }, // Brasilia
    {  343,  454,       0,  258828,       0,  0, 15,  0, }, // Brunei
    {  343,  461,  258843,  258860,  258885, 17, 25, 23, }, // Cape_Verde
    {  343,  472,       0,  258908,       0,  0, 14,  0, }, // Casey
    {  343,  478,       0,  258922,       0,  0, 18,  0, }, // Chamorro
    {  343,  487,  258940,  258962,  258992, 22, 30, 28, }, // Chatham
    {  343,  495,  259020,  259034,  259056, 14, 22, 20, }, // Chile
    {  343,  501,  259076,  259093,  259118, 17, 25, 20, }, // China
    {  343,  507,       0,  259138,       0,  0, 24,  0, }, // Christmas
    {  343,  517,       0,  259162,       0,  0, 20,  0, }, // Cocos
    {  343,  523,  259182,  259199,  259224, 17, 25, 23, }, // Colombia
    {  343,  532,  259247,  259266,  259293, 19, 27, 25, }, // Cook
    {  343,  537,  259318,  259331,  259352, 13, 21, 19, }, // Cuba
    {  343,  548,       0,  259371,       0,  0, 14,  0, }, // Davis
    {  343,  564,       0,  259385,       0,  0, 25,  0, }, // DumontDUrville
    {  343,  601,       0,  259410,       0,  0, 23,  0, }, // East_Timor
    {  343,  612,  259433,  259457,  259489, 24, 32, 30, }, // Easter
    {  343,  619,       0,  259519,       0,  0, 19,  0, }, // Ecuador
    {  343,  627,  259538,  259561,  259592, 23, 31, 29, }, // Europe_Central
    {  343,  642,  259621,  259644,  259675, 23, 31, 29, }, // Europe_Eastern
    {  343,  657,       0,  259704,       0,  0, 20,  0, }, // Europe_Further_Eastern
    {  343,  680,  259724,  259749,  259782, 25, 33, 31, }, // Europe_Western
    {  343,  695,  259813,  259837,  259869, 24, 32, 30, }, // Falkland
    {  343,  704,  259899,  259919,  259947, 20, 28, 26, }, // Fiji
    {  343,  709,       0,  259973,       0,  0, 28,  0, }, // French_Guiana
    {  343,  723,       0,  260001,       0,  0, 53,  0, }, // French_Southern
    {  343,  746,       0,  260054,       0,  0, 24,  0, }, // Galapagos
    {  343,  756,       0,  260078,       0,  0, 22,  0, }, // Gambier
    {  343,  764,  260100,  260119,  260146, 19, 27, 22, }, // Georgia
    {  343,  772,       0,  260168,       0,  0, 22,  0, }, // Gilbert_Islands
    {  343,  788,       0,  260190,       0,  0, 26,  0, }, // GMT
    {  343,  830,  260216,  260243,  260278, 27, 35, 33, }, // Greenland_Eastern
    {  343,  848,  260311,  260340,  260377, 29, 37, 35, }, // Greenland_Western
    {  343,  866,       0,  260412,       0,  0, 13,  0, }, // Guam
    {  343,  871,       0,  260425,       0,  0, 14,  0, }, // Gulf
    {  343,  876,       0,  260439,       0,  0, 15,  0, }, // Guyana
    {  343,  883,  260454,  260482,  260518, 28, 36, 34, }, // Hawaii_Aleutian
    {  343,  899,  260552,  260570,  260596, 18, 26, 24, }, // Hong_Kong
    {  343,  909,  260620,  260633,  260654, 13, 21, 19, }, // Hovd
    {  343,  914,       0,  260673,       0,  0, 15,  0, }, // India
    {  343,  920,       0,  260688,       0,  0, 23,  0, }, // Indian_Ocean
    {  343,  933,       0,  260711,       0,  0, 17,  0, }, // Indochina
    {  343,  943,       0,  260728,       0,  0, 26,  0, }, // Indonesia_Central
    {  343,  961,       0,  260754,       0,  0, 25,  0, }, // Indonesia_Eastern
    {  343,  979,       0,  260779,       0,  0, 27,  0, }, // Indonesia_Western
    {  343,  997,  260806,  260821,  260841, 15, 20, 18, }, // Iran
    {  343, 1008,  260859,  260875,  260899, 16, 24, 22, }, // Irkutsk
    {  343, 1016,  260921,  260935,  260957, 14, 22, 20, }, // Israel
    {  343, 1023,  260977,  260991,  261013, 14, 22, 20, }, // Japan
    {  343, 1029,  261033,  261066,  261107, 33, 41, 39, }, // Kamchatka
    {  343, 1047,       0,  261146,       0,  0, 19,  0, }, // Kazakhstan
    {  343, 1058,       0,  261165,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  343, 1077,       0,  261193,       0,  0, 30,  0, }, // Kazakhstan_Western
    {  343, 1106,  261223,  261240,  261265, 17, 25, 20, }, // Korea
    {  343, 1112,       0,  261285,       0,  0, 15,  0, }, // Kosrae
    {  343, 1119,  261300,  261320,  261348, 20, 28, 26, }, // Krasnoyarsk
    {  343, 1151,       0,  261374,       0,  0, 21,  0, }, // Kyrgystan
    {  343, 1161,       0,  261395,       0,  0, 14,  0, }, // Lanka
    {  343, 1175,       0,  261409,       0,  0, 26,  0, }, // Line_Islands
    {  343, 1188,  261435,  261453,  261479, 18, 26, 24, }, // Lord_Howe
    {  343, 1198,  261503,  261517,  261539, 14, 22, 20, }, // Macau
    {  343, 1204,  261559,  261575,  261599, 16, 24, 22, }, // Magadan
    {  343, 1219,       0,  261621,       0,  0, 20,  0, }, // Malaysia
    {  343, 1228,       0,  261641,       0,  0, 18,  0, }, // Maldives
    {  343, 1237,       0,  261659,       0,  0, 24,  0, }, // Marquesas
    {  343, 1247,       0,  261683,       0,  0, 23,  0, }, // Marshall_Islands
    {  343, 1264,  261706,  261722,  261746, 16, 24, 22, }, // Mauritius
    {  343, 1274,       0,  261768,       0,  0, 15,  0, }, // Mawson
    {  343, 1281,  261783,  261810,  261845, 27, 35, 33, }, // Mexico_Pacific
    {  343, 1296,  261878,  261897,  261924, 19, 27, 25, }, // Mongolia
    {  343, 1305,  261949,  261964,  261987, 15, 23, 21, }, // Moscow
    {  343, 1312,       0,  262008,       0,  0, 16,  0, }, // Myanmar
    {  343, 1320,       0,  262024,       0,  0, 14,  0, }, // Nauru
    {  343, 1326,       0,  262038,       0,  0, 14,  0, }, // Nepal
    {  343, 1332,  262052,  262082,  262120, 30, 38, 33, }, // New_Caledonia
    {  343, 1346,  262153,  262181,  262217, 28, 36, 34, }, // New_Zealand
    {  343, 1358,  262251,  262271,  262299, 20, 28, 26, }, // Newfoundland
    {  343, 1371,       0,  262325,       0,  0, 13,  0, }, // Niue
    {  343, 1376,  262338,  262360,  262390, 22, 30, 28, }, // Norfolk
    {  343, 1384,  262418,  262446,  262482, 28, 36, 34, }, // Noronha
    {  343, 1392,       0,  262516,       0,  0, 32,  0, }, // North_Mariana
    {  343, 1406,  262548,  262569,  262598, 21, 29, 27, }, // Novosibirsk
    {  343, 1418,  262625,  262638,  262659, 13, 21, 19, }, // Omsk
    {  343, 1428,  262678,  262695,  262720, 17, 25, 23, }, // Pakistan
    {  343, 1437,       0,  262743,       0,  0, 16,  0, }, // Palau
    {  343, 1443,       0,  262759,       0,  0, 37,  0, }, // Papua_New_Guinea
    {  343, 1460,  262796,  262813,  262838, 17, 25, 23, }, // Paraguay
    {  343, 1469,  262861,  262875,  262897, 14, 22, 20, }, // Peru
    {  343, 1474,  262917,  262938,  262967, 21, 29, 27, }, // Philippines
    {  343, 1486,       0,  262994,       0,  0, 22,  0, }, // Phoenix_Islands
    {  343, 1502,  263016,  263049,  263090, 33, 41, 39, }, // Pierre_Miquelon
    {  343, 1518,       0,  263129,       0,  0, 23,  0, }, // Pitcairn
    {  343, 1527,       0,  263152,       0,  0, 25,  0, }, // Ponape
    {  343, 1534,       0,  263177,       0,  0, 18,  0, }, // Pyongyang
    {  343, 1544,  263195,  263213,  263239, 18, 26, 24, }, // Qyzylorda
    {  343, 1554,       0,  263263,       0,  0, 19,  0, }, // Reunion
    {  343, 1562,       0,  263282,       0,  0, 16,  0, }, // Rothera
    {  343, 1570,  263298,  263316,  263342, 18, 26, 24, }, // Sakhalin
    {  343, 1579,  263366,  263381,  263404, 15, 23, 21, }, // Samara
    {  343, 1596,  263425,  263440,  263463, 15, 23, 21, }, // Samoa
    {  343, 1602,       0,  263484,       0,  0, 20,  0, }, // Seychelles
    {  343, 1624,       0,  263504,       0,  0, 18,  0, }, // Singapore
    {  343, 1634,       0,  263522,       0,  0, 22,  0, }, // Solomon
    {  343, 1642,       0,  263544,       0,  0, 23,  0, }, // South_Georgia
    {  343, 1656,       0,  263567,       0,  0, 17,  0, }, // Suriname
    {  343, 1676,       0,  263584,       0,  0, 14,  0, }, // Syowa
    {  343, 1682,       0,  263598,       0,  0, 15,  0, }, // Tahiti
    {  343, 1689,  263613,  263628,  263651, 15, 23, 21, }, // Taipei
    {  343, 1696,       0,  263672,       0,  0, 20,  0, }, // Tajikistan
    {  343, 1724,       0,  263692,       0,  0, 16,  0, }, // Tokelau
    {  343, 1732,  263708,  263723,  263746, 15, 23, 20, }, // Tonga
    {  343, 1738,       0,  263766,       0,  0, 14,  0, }, // Truk
    {  343, 1750,  263780,  263801,  263830, 21, 29, 27, }, // Turkmenistan
    {  343, 1763,       0,  263857,       0,  0, 16,  0, }, // Tuvalu
    {  343, 1777,  263873,  263891,  263917, 18, 26, 24, }, // Uruguay
    {  343, 1792,  263941,  263963,  263993, 22, 30, 28, }, // Uzbekistan
    {  343, 1803,  264021,  264037,  264061, 16, 24, 22, }, // Vanuatu
    {  343, 1811,       0,  264083,       0,  0, 18,  0, }, // Venezuela
    {  343, 1821,  264101,  264121,  264149, 20, 28, 26, }, // Vladivostok
    {  343, 1833,  264175,  264193,  264219, 18, 26, 24, }, // Volgograd
    {  343, 1843,       0,  264243,       0,  0, 15,  0, }, // Vostok
    {  343, 1850,       0,  264258,       0,  0, 19,  0, }, // Wake
    {  343, 1855,       0,  264277,       0,  0, 25,  0, }, // Wallis
    {  343, 1862,  264302,  264319,  264344, 17, 25, 23, }, // Yakutsk
    {  343, 1870,  264367,  264388,  264417, 21, 29, 27, }, // Yekaterinburg
    {  343, 1892,       0,  264444,       0,  0, 22,  0, }, // Yukon
    {  344,    5,       0,  321804,       0,  0, 19,  0, }, // Afghanistan Hausa/Latin/Nigeria
    {  344,   17,       0,  321823,       0,  0, 25,  0, }, // Africa_Central
    {  344,   32,       0,  321848,       0,  0, 23,  0, }, // Africa_Eastern
    {  344,   65,       0,  321871,       0,  0, 32,  0, }, // Africa_Southern
    {  344,   81,  321903,  321926,  321959, 23, 33, 33, }, // Africa_Western
    {  344,  107,  321992,  322006,  322030, 14, 24, 22, }, // Alaska
    {  344,  135,  322052,  322066,  322090, 14, 24, 24, }, // Amazon
    {  344,  142,  322114,  322151,  322200, 37, 49, 45, }, // America_Central
    {  344,  158,  322245,  322281,  322326, 36, 45, 43, }, // America_Eastern
    {  344,  174,  322369,  322402,  322446, 33, 44, 39, }, // America_Mountain
    {  344,  191,  322485,  322509,  322544, 24, 35, 31, }, // America_Pacific
    {  344,  214,  322575,  322587,  322609, 12, 22, 20, }, // Apia
    {  344,  232,  322629,  322644,  322669, 15, 25, 23, }, // Arabian
    {  344,  240,  322692,  322709,  322736, 17, 27, 27, }, // Argentina
    {  344,  250,  322763,  322789,  322825, 26, 36, 36, }, // Argentina_Western
    {  344,  268,  322861,  322876,  322901, 15, 25, 25, }, // Armenia
    {  344,  286,  322926,  322971,  323026, 45, 55, 53, }, // Atlantic
    {  344,  295,  323079,  323106,  323144, 27, 38, 36, }, // Australia_Central
    {  344,  313,  323180,  323217,  323264, 37, 47, 45, }, // Australia_CentralWestern
    {  344,  338,  323309,  323337,  323373, 28, 36, 36, }, // Australia_Eastern
    {  344,  356,  323409,  323436,  323473, 27, 37, 35, }, // Australia_Western
    {  344,  374,  323508,  323526,  323554, 18, 28, 28, }, // Azerbaijan
    {  344,  385,  323582,  323596,  323622, 14, 26, 23, }, // Azores
    {  344,  397,  323645,  323663,  323691, 18, 28, 28, }, // Bangladesh
    {  344,  415,       0,  323719,       0,  0, 14,  0, }, // Bhutan
    {  344,  422,       0,  323733,       0,  0, 15,  0, }, // Bolivia
    {  344,  437,  323748,  323765,  323792, 17, 27, 27, }, // Brasilia
    {  344,  454,       0,  323819,       0,  0, 25,  0, }, // Brunei
    {  344,  461,  323844,  323862,  323890, 18, 28, 28, }, // Cape_Verde
    {  344,  478,       0,  323918,       0,  0, 26,  0, }, // Chamorro
    {  344,  487,  323944,  323959,  323984, 15, 25, 23, }, // Chatham
    {  344,  495,  324007,  324020,  324043, 13, 23, 23, }, // Chile
    {  344,  501,  324066,  324077,  324098, 11, 21, 19, }, // China
    {  344,  507,       0,  324117,       0,  0, 24,  0, }, // Christmas
    {  344,  517,       0,  324141,       0,  0, 21,  0, }, // Cocos
    {  344,  523,  324162,  324178,  324204, 16, 26, 26, }, // Colombia
    {  344,  532,  324230,  324250,  324280, 20, 30, 36, }, // Cook
    {  344,  537,  324316,  324330,  324352, 14, 22, 20, }, // Cuba
    {  344,  548,       0,  324372,       0,  0, 13,  0, }, // Davis
    {  344,  564,       0,  324385,       0,  0, 24,  0, }, // DumontDUrville
    {  344,  601,       0,  324409,       0,  0, 18,  0, }, // East_Timor
    {  344,  612,  324427,  324448,  324479, 21, 31, 31, }, // Easter
    {  344,  619,       0,  324510,       0,  0, 15,  0, }, // Ecuador
    {  344,  627,  324525,  324548,  324575, 23, 27, 30, }, // Europe_Central
    {  344,  642,  324605,  324625,  324649, 20, 24, 30, }, // Europe_Eastern
    {  344,  657,       0,  324679,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  344,  680,  324710,  324734,  324762, 24, 28, 38, }, // Europe_Western
    {  344,  695,  324800,  324824,  324858, 24, 34, 34, }, // Falkland
    {  344,  704,  324892,  324904,  324926, 12, 22, 22, }, // Fiji
    {  344,  709,       0,  324948,       0,  0, 21,  0, }, // French_Guiana
    {  344,  723,       0,  324969,       0,  0, 37,  0, }, // French_Southern
    {  344,  746,       0,  325006,       0,  0, 17,  0, }, // Galapagos
    {  344,  756,       0,  325023,       0,  0, 15,  0, }, // Gambier
    {  344,  764,  325038,  325053,  325078, 15, 25, 25, }, // Georgia
    {  344,  772,       0,  325103,       0,  0, 23,  0, }, // Gilbert_Islands
    {  344,  788,       0,  325126,       0,  0, 26,  0, }, // GMT
    {  344,  830,  325152,  325178,  325214, 26, 36, 34, }, // Greenland_Eastern
    {  344,  848,  325248,  325274,  325310, 26, 36, 34, }, // Greenland_Western
    {  344,  871,       0,  325344,       0,  0, 22,  0, }, // Gulf
    {  344,  876,       0,  325366,       0,  0, 14,  0, }, // Guyana
    {  344,  883,  325380,  325405,  325438, 25, 33, 31, }, // Hawaii_Aleutian
    {  344,  899,  325469,  325486,  325513, 17, 27, 27, }, // Hong_Kong
    {  344,  909,  325540,  325552,  325574, 12, 22, 22, }, // Hovd
    {  344,  914,       0,  325596,       0,  0, 24,  0, }, // India
    {  344,  920,       0,  325620,       0,  0, 20,  0, }, // Indian_Ocean
    {  344,  933,       0,  325640,       0,  0, 17,  0, }, // Indochina
    {  344,  943,       0,  325657,       0,  0, 28,  0, }, // Indonesia_Central
    {  344,  961,       0,  325685,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  344,  979,       0,  325711,       0,  0, 26,  0, }, // Indonesia_Western
    {  344,  997,  325737,  325749,  325771, 12, 22, 20, }, // Iran
    {  344, 1008,  325791,  325806,  325831, 15, 25, 25, }, // Irkutsk
    {  344, 1016,  325856,  325872,  325898, 16, 26, 31, }, // Israel
    {  344, 1023,  325929,  325942,  325965, 13, 23, 28, }, // Japan
    {  344, 1047,       0,  325993,       0,  0, 18,  0, }, // Kazakhstan
    {  344, 1058,       0,  326011,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  344, 1077,       0,  326038,       0,  0, 27,  0, }, // Kazakhstan_Western
    {  344, 1106,  326065,  326079,  326103, 14, 24, 22, }, // Korea
    {  344, 1112,       0,  326125,       0,  0, 14,  0, }, // Kosrae
    {  344, 1119,  326139,  326158,  326187, 19, 29, 29, }, // Krasnoyarsk
    {  344, 1151,       0,  326216,       0,  0, 18,  0, }, // Kyrgystan
    {  344, 1175,       0,  326234,       0,  0, 20,  0, }, // Line_Islands
    {  344, 1188,  326254,  326271,  326298, 17, 27, 30, }, // Lord_Howe
    {  344, 1204,  326328,  326343,  326368, 15, 25, 25, }, // Magadan
    {  344, 1219,       0,  326393,       0,  0, 16,  0, }, // Malaysia
    {  344, 1228,       0,  326409,       0,  0, 16,  0, }, // Maldives
    {  344, 1237,       0,  326425,       0,  0, 17,  0, }, // Marquesas
    {  344, 1247,       0,  326442,       0,  0, 24,  0, }, // Marshall_Islands
    {  344, 1264,  326466,  326483,  326510, 17, 27, 27, }, // Mauritius
    {  344, 1274,       0,  326537,       0,  0, 14,  0, }, // Mawson
    {  344, 1281,  326551,  326574,  326607, 23, 33, 31, }, // Mexico_Pacific
    {  344, 1296,  326638,  326657,  326686, 19, 29, 29, }, // Mongolia
    {  344, 1305,  326715,  326729,  326753, 14, 24, 24, }, // Moscow
    {  344, 1312,       0,  326777,       0,  0, 15,  0, }, // Myanmar
    {  344, 1320,       0,  326792,       0,  0, 13,  0, }, // Nauru
    {  344, 1326,       0,  326805,       0,  0, 13,  0, }, // Nepal
    {  344, 1332,  326818,  326839,  326870, 21, 31, 31, }, // New_Caledonia
    {  344, 1346,  326901,  326920,  326949, 19, 29, 27, }, // New_Zealand
    {  344, 1358,  326976,  326996,  327027, 20, 31, 27, }, // Newfoundland
    {  344, 1371,       0,  327054,       0,  0, 12,  0, }, // Niue
    {  344, 1376,  327066,  327088,  327120, 22, 32, 30, }, // Norfolk
    {  344, 1384,  327150,  327177,  327214, 27, 37, 37, }, // Noronha
    {  344, 1406,  327251,    4766,  327270, 19, 25, 29, }, // Novosibirsk
    {  344, 1418,  327299,  327311,  327333, 12, 22, 22, }, // Omsk
    {  344, 1428,  327355,  327371,  327397, 16, 26, 26, }, // Pakistan
    {  344, 1437,       0,  327423,       0,  0, 13,  0, }, // Palau
    {  344, 1443,       0,  327436,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  344, 1460,  327460,  327476,  327502, 16, 26, 26, }, // Paraguay
    {  344, 1469,  327528,  327540,  327562, 12, 22, 22, }, // Peru
    {  344, 1474,  327584,  327602,  327630, 18, 28, 28, }, // Philippines
    {  344, 1486,       0,  327658,       0,  0, 23,  0, }, // Phoenix_Islands
    {  344, 1502,  327681,  327711,  327751, 30, 40, 38, }, // Pierre_Miquelon
    {  344, 1518,       0,  327789,       0,  0, 16,  0, }, // Pitcairn
    {  344, 1527,       0,  327805,       0,  0, 14,  0, }, // Ponape
    {  344, 1534,       0,  327819,       0,  0, 17,  0, }, // Pyongyang
    {  344, 1554,       0,  327836,       0,  0, 15,  0, }, // Reunion
    {  344, 1562,       0,  327851,       0,  0, 15,  0, }, // Rothera
    {  344, 1570,  327866,  327882,  327908, 16, 26, 26, }, // Sakhalin
    {  344, 1596,  327934,  327947,  327970, 13, 23, 26, }, // Samoa
    {  344, 1602,       0,  327996,       0,  0, 18,  0, }, // Seychelles
    {  344, 1624,       0,  328014,       0,  0, 27,  0, }, // Singapore
    {  344, 1634,       0,  328041,       0,  0, 23,  0, }, // Solomon
    {  344, 1642,       0,  328064,       0,  0, 24,  0, }, // South_Georgia
    {  344, 1656,       0,  328088,       0,  0, 16,  0, }, // Suriname
    {  344, 1676,       0,  328104,       0,  0, 13,  0, }, // Syowa
    {  344, 1682,       0,  328117,       0,  0, 14,  0, }, // Tahiti
    {  344, 1689,  328131,  328145,  328169, 14, 24, 22, }, // Taipei
    {  344, 1696,       0,  328191,       0,  0, 18,  0, }, // Tajikistan
    {  344, 1724,       0,  328209,       0,  0, 15,  0, }, // Tokelau
    {  344, 1732,  328224,  328237,  328260, 13, 23, 23, }, // Tonga
    {  344, 1738,       0,  328283,       0,  0, 13,  0, }, // Truk
    {  344, 1750,  328296,  328316,  328346, 20, 30, 30, }, // Turkmenistan
    {  344, 1763,       0,  328376,       0,  0, 14,  0, }, // Tuvalu
    {  344, 1777,  328390,  328405,  328430, 15, 25, 25, }, // Uruguay
    {  344, 1792,  328455,  328473,  328501, 18, 28, 28, }, // Uzbekistan
    {  344, 1803,  328529,  328544,  328569, 15, 25, 25, }, // Vanuatu
    {  344, 1811,       0,  328594,       0,  0, 17,  0, }, // Venezuela
    {  344, 1821,  328611,  328630,  328659, 19, 29, 29, }, // Vladivostok
    {  344, 1833,  328688,  328705,  328732, 17, 27, 27, }, // Volgograd
    {  344, 1843,       0,  328759,       0,  0, 14,  0, }, // Vostok
    {  344, 1850,       0,  328773,       0,  0, 19,  0, }, // Wake
    {  344, 1855,       0,  328792,       0,  0, 24,  0, }, // Wallis
    {  344, 1862,  328816,  328831,  328856, 15, 25, 25, }, // Yakutsk
    {  344, 1870,  328881,  328902,  328933, 21, 31, 31, }, // Yekaterinburg
    {  344, 1892,       0,  328964,       0,  0, 13,  0, }, // Yukon
    {  350,    5,       0,  328977,       0,  0, 13,  0, }, // Afghanistan Hebrew/Hebrew/Israel
    {  350,   17,       0,  328990,       0,  0, 16,  0, }, // Africa_Central
    {  350,   32,       0,  329006,       0,  0, 16,  0, }, // Africa_Eastern
    {  350,   65,       0,  329022,       0,  0, 16,  0, }, // Africa_Southern
    {  350,   81,  329038,  329054,  329077, 16, 23, 22, }, // Africa_Western
    {  350,  107,  329099,  329109,  329126, 10, 17, 16, }, // Alaska
    {  350,  128,  329142,  329152,  329169, 10, 17, 16, }, // Almaty
    {  350,  135,  329185,  329196,  329214, 11, 18, 17, }, // Amazon
    {  350,  142,  329231,  329246,  329268, 15, 22, 21, }, // America_Central
    {  350,  158,  329289,  329305,  329328, 16, 23, 22, }, // America_Eastern
    {  350,  174,  329350,  329372,  329401, 22, 29, 28, }, // America_Mountain
    {  350,  191,  329429,  329444,  329466, 15, 22, 21, }, // America_Pacific
    {  350,  207,  329487,  329497,  329512, 10, 15, 14, }, // Anadyr
    {  350,  214,  329526,  329535,  329551,  9, 16, 15, }, // Apia
    {  350,  219,  329566,  329576,  329593, 10, 17, 16, }, // Aqtau
    {  350,  225,  329609,  329620,  329639, 11, 19, 18, }, // Aqtobe
    {  350,  232,  329657,  329673,  329696, 16, 23, 22, }, // Arabian
    {  350,  240,  329718,  329731,  329751, 13, 20, 19, }, // Argentina
    {  350,  250,  329770,  329788,  329813, 18, 25, 24, }, // Argentina_Western
    {  350,  268,  329837,  329848,  329866, 11, 18, 17, }, // Armenia
    {  350,  286,  329883,  329905,  329934, 22, 29, 28, }, // Atlantic
    {  350,  295,  329962,  329980,  330005, 18, 25, 24, }, // Australia_Central
    {  350,  313,  330029,  330052,  330082, 23, 30, 29, }, // Australia_CentralWestern
    {  350,  338,  330111,  330129,  330154, 18, 25, 24, }, // Australia_Eastern
    {  350,  356,  330178,  330196,  330221, 18, 25, 24, }, // Australia_Western
    {  350,  374,  330245,  330259,  330280, 14, 21, 20, }, // Azerbaijan
    {  350,  385,  330300,  330319,  330345, 19, 26, 25, }, // Azores
    {  350,  397,  330370,  330381,  330399, 11, 18, 17, }, // Bangladesh
    {  350,  415,       0,  330416,       0,  0, 10,  0, }, // Bhutan
    {  350,  422,       0,  330426,       0,  0, 12,  0, }, // Bolivia
    {  350,  437,  330438,  330450,  330469, 12, 19, 18, }, // Brasilia
    {  350,  454,       0,  330487,       0,  0, 20,  0, }, // Brunei
    {  350,  461,  330507,  330519,  330538, 12, 19, 18, }, // Cape_Verde
    {  350,  478,       0,  330556,       0,  0, 12,  0, }, // Chamorro
    {  350,  487,  330568,  330579,  330597, 11, 18, 17, }, // Chatham
    {  350,  495,  330614,  330624,  330641, 10, 17, 16, }, // Chile
    {  350,  501,  330657,  330665,  330680,  8, 15, 14, }, // China
    {  350,  507,       0,  330694,       0,  0, 16,  0, }, // Christmas
    {  350,  517,       0,  330710,       0,  0, 14,  0, }, // Cocos
    {  350,  523,  330724,  330737,  330757, 13, 20, 19, }, // Colombia
    {  350,  532,  330710,  330776,  330795, 12, 19, 25, }, // Cook
    {  350,  537,  330820,  330829,  330845,  9, 16, 15, }, // Cuba
    {  350,  548,       0,  330860,       0,  0, 11,  0, }, // Davis
    {  350,  564,       0,  330871,       0,  0, 20,  0, }, // DumontDUrville
    {  350,  601,       0,  330891,       0,  0, 15,  0, }, // East_Timor
    {  350,  612,  330906,  330919,  330939, 13, 20, 19, }, // Easter
    {  350,  619,       0,  330958,       0,  0, 12,  0, }, // Ecuador
    {  350,  627,  330970,  330986,  331009, 16, 23, 22, }, // Europe_Central
    {  350,  642,  331031,  331047,  331070, 16, 23, 22, }, // Europe_Eastern
    {  350,  657,       0,  331092,       0,  0, 10,  0, }, // Europe_Further_Eastern
    {  350,  680,  331102,  331118,  331141, 16, 23, 22, }, // Europe_Western
    {  350,  695,  331163,  331178,  331200, 15, 22, 21, }, // Falkland
    {  350,  704,  331221,  331231,  331248, 10, 17, 16, }, // Fiji
    {  350,  709,       0,  331264,       0,  0, 18,  0, }, // French_Guiana
    {  350,  723,       0,  331282,       0,  0, 41,  0, }, // French_Southern
    {  350,  746,       0,  331323,       0,  0, 16,  0, }, // Galapagos
    {  350,  756,       0,  331339,       0,  0, 15,  0, }, // Gambier
    {  350,  764,  331354,  331366,  331385, 12, 19, 18, }, // Georgia
    {  350,  772,       0,  331403,       0,  0, 15,  0, }, // Gilbert_Islands
    {  350,  788,       0,  331418,       0,  0, 13,  0, }, // GMT
    {  350,  830,  331431,  331448,  331472, 17, 24, 23, }, // Greenland_Eastern
    {  350,  848,  331495,  331512,  331536, 17, 24, 23, }, // Greenland_Western
    {  350,  866,       0,  331559,       0,  0,  9,  0, }, // Guam
    {  350,  871,       0,  331568,       0,  0, 17,  0, }, // Gulf
    {  350,  876,       0,  331264,       0,  0, 10,  0, }, // Guyana
    {  350,  883,  331585,  331611,  331644, 26, 33, 32, }, // Hawaii_Aleutian
    {  350,  899,  331676,  331690,  331711, 14, 21, 20, }, // Hong_Kong
    {  350,  909,  331731,  331740,  331756,  9, 16, 15, }, // Hovd
    {  350,  914,       0,  331771,       0,  0,  9,  0, }, // India
    {  350,  920,       0,  331780,       0,  0, 20,  0, }, // Indian_Ocean
    {  350,  933,       0,  331800,       0,  0, 13,  0, }, // Indochina
    {  350,  943,       0,  331813,       0,  0, 19,  0, }, // Indonesia_Central
    {  350,  961,       0,  331832,       0,  0, 19,  0, }, // Indonesia_Eastern
    {  350,  979,       0,  331851,       0,  0, 19,  0, }, // Indonesia_Western
    {  350,  997,  331870,  331880,  331897, 10, 17, 16, }, // Iran
    {  350, 1008,  331913,  331926,  331946, 13, 20, 19, }, // Irkutsk
    {  350, 1016,  331965,  331975,  331992, 10, 17, 16, }, // Israel
    {  350, 1023,  332008,  332016,  332031,  8, 15, 14, }, // Japan
    {  350, 1029,  332045,  332070,  332100, 25, 30, 29, }, // Kamchatka
    {  350, 1047,       0,  332129,       0,  0, 11,  0, }, // Kazakhstan
    {  350, 1058,       0,  332140,       0,  0, 16,  0, }, // Kazakhstan_Eastern
    {  350, 1077,       0,  332156,       0,  0, 16,  0, }, // Kazakhstan_Western
    {  350, 1106,  332172,  332183,  332201, 11, 18, 17, }, // Korea
    {  350, 1112,       0,  332218,       0,  0, 11,  0, }, // Kosrae
    {  350, 1119,  332229,  332244,  332266, 15, 22, 21, }, // Krasnoyarsk
    {  350, 1151,       0,  332287,       0,  0, 14,  0, }, // Kyrgystan
    {  350, 1175,       0,  332301,       0,  0, 13,  0, }, // Line_Islands
    {  350, 1188,  332314,  332331,  332355, 17, 24, 23, }, // Lord_Howe
    {  350, 1198,  332378,  332387,  332401,  9, 14, 13, }, // Macau
    {  350, 1204,  332414,  332423,  332439,  9, 16, 15, }, // Magadan
    {  350, 1219,       0,  332454,       0,  0, 10,  0, }, // Malaysia
    {  350, 1228,       0,  332464,       0,  0, 20,  0, }, // Maldives
    {  350, 1237,       0,  332484,       0,  0, 14,  0, }, // Marquesas
    {  350, 1247,       0,  332498,       0,  0, 13,  0, }, // Marshall_Islands
    {  350, 1264,  332511,  332525,  332546, 14, 21, 20, }, // Mauritius
    {  350, 1274,       0,  332566,       0,  0, 11,  0, }, // Mawson
    {  350, 1281,  332577,  332593,  332616, 16, 23, 22, }, // Mexico_Pacific
    {  350, 1296,  332638,  332654,  332677, 16, 23, 22, }, // Mongolia
    {  350, 1305,  332699,  332710,  332728, 11, 18, 17, }, // Moscow
    {  350, 1312,       0,  332745,       0,  0, 11,  0, }, // Myanmar
    {  350, 1320,       0,  332756,       0,  0, 10,  0, }, // Nauru
    {  350, 1326,       0,  332766,       0,  0,  9,  0, }, // Nepal
    {  350, 1332,  332775,  332793,  332818, 18, 25, 24, }, // New_Caledonia
    {  350, 1346,  332842,  332856,  332877, 14, 21, 20, }, // New_Zealand
    {  350, 1358,  332897,  332913,  332936, 16, 23, 22, }, // Newfoundland
    {  350, 1371,       0,  332958,       0,  0, 10,  0, }, // Niue
    {  350, 1376,  332968,  332983,  333005, 15, 22, 21, }, // Norfolk
    {  350, 1384,  333026,  333048,  333077, 22, 29, 28, }, // Noronha
    {  350, 1392,       0,  333105,       0,  0, 24,  0, }, // North_Mariana
    {  350, 1406,  333129,  333145,  333168, 16, 23, 22, }, // Novosibirsk
    {  350, 1418,  333190,  333200,  333217, 10, 17, 16, }, // Omsk
    {  350, 1428,  333233,  333244,  333262, 11, 18, 17, }, // Pakistan
    {  350, 1437,       0,  333279,       0,  0,  9,  0, }, // Palau
    {  350, 1443,       0,  333288,       0,  0, 23,  0, }, // Papua_New_Guinea
    {  350, 1460,  333311,  333323,  333342, 12, 19, 18, }, // Paraguay
    {  350, 1469,  333360,  333368,  333383,  8, 15, 14, }, // Peru
    {  350, 1474,  333397,  333412,  333434, 15, 22, 21, }, // Philippines
    {  350, 1486,       0,  333455,       0,  0, 15,  0, }, // Phoenix_Islands
    {  350, 1502,  333470,  333491,  333519, 21, 28, 27, }, // Pierre_Miquelon
    {  350, 1518,       0,  333546,       0,  0, 11,  0, }, // Pitcairn
    {  350, 1527,       0,  333557,       0,  0, 11,  0, }, // Ponape
    {  350, 1534,       0,  333568,       0,  0, 14,  0, }, // Pyongyang
    {  350, 1544,  333582,  333596,  333617, 14, 21, 20, }, // Qyzylorda
    {  350, 1554,       0,  333637,       0,  0, 12,  0, }, // Reunion
    {  350, 1562,       0,  333649,       0,  0, 11,  0, }, // Rothera
    {  350, 1570,  333660,  333670,  333687, 10, 17, 16, }, // Sakhalin
    {  350, 1579,  333703,  333712,  333726,  9, 14, 13, }, // Samara
    {  350, 1596,  333739,  333749,  333766, 10, 17, 16, }, // Samoa
    {  350, 1602,       0,  333782,       0,  0, 14,  0, }, // Seychelles
    {  350, 1624,       0,  333796,       0,  0, 12,  0, }, // Singapore
    {  350, 1634,       0,  333808,       0,  0, 13,  0, }, // Solomon
    {  350, 1642,       0,  333821,       0,  0, 18,  0, }, // South_Georgia
    {  350, 1656,       0,  333839,       0,  0, 12,  0, }, // Suriname
    {  350, 1676,       0,  333851,       0,  0, 11,  0, }, // Syowa
    {  350, 1682,       0,  333862,       0,  0, 10,  0, }, // Tahiti
    {  350, 1689,  333872,  333883,  333901, 11, 18, 17, }, // Taipei
    {  350, 1696,       0,  333918,       0,  0, 14,  0, }, // Tajikistan
    {  350, 1724,       0,  333932,       0,  0, 11,  0, }, // Tokelau
    {  350, 1732,  333943,  333953,  333970, 10, 17, 16, }, // Tonga
    {  350, 1738,       0,  333986,       0,  0,  9,  0, }, // Truk
    {  350, 1750,  333995,  334010,  334032, 15, 22, 21, }, // Turkmenistan
    {  350, 1763,       0,  334053,       0,  0, 11,  0, }, // Tuvalu
    {  350, 1777,  334064,  334078,  334099, 14, 21, 20, }, // Uruguay
    {  350, 1792,  334119,  334133,  334154, 14, 21, 20, }, // Uzbekistan
    {  350, 1803,  334174,  334185,  334203, 11, 18, 17, }, // Vanuatu
    {  350, 1811,       0,  334220,       0,  0, 12,  0, }, // Venezuela
    {  350, 1821,  334232,  334247,  334269, 15, 22, 21, }, // Vladivostok
    {  350, 1833,  334290,  334303,  334323, 13, 20, 19, }, // Volgograd
    {  350, 1843,       0,  334342,       0,  0, 11,  0, }, // Vostok
    {  350, 1850,       0,  334353,       0,  0, 13,  0, }, // Wake
    {  350, 1855,       0,  334366,       0,  0, 19,  0, }, // Wallis
    {  350, 1862,  334385,  334396,  334414, 11, 18, 17, }, // Yakutsk
    {  350, 1870,  334431,  334446,  334468, 15, 22, 21, }, // Yekaterinburg
    {  350, 1892,       0,  334489,       0,  0, 10,  0, }, // Yukon
    {  351,    5,       0,  334499,       0,  0, 16,  0, }, // Afghanistan Hindi/Devanagari/India
    {  351,   17,       0,  334515,       0,  0, 17,  0, }, // Africa_Central
    {  351,   32,       0,  334532,       0,  0, 19,  0, }, // Africa_Eastern
    {  351,   65,       0,  334551,       0,  0, 24,  0, }, // Africa_Southern
    {  351,   81,  334575,  334594,  334618, 19, 24, 32, }, // Africa_Western
    {  351,  107,  334650,  334661,  334678, 11, 17, 19, }, // Alaska
    {  351,  135,  334697,  334707,  334722, 10, 15, 23, }, // Amazon
    {  351,  142,  334745,  334772,  334804, 27, 32, 34, }, // America_Central
    {  351,  158,  334838,  334863,  334893, 25, 30, 32, }, // America_Eastern
    {  351,  174,  334925,  334951,  334982, 26, 31, 33, }, // America_Mountain
    {  351,  191,  335015,  335041,  335072, 26, 31, 33, }, // America_Pacific
    {  351,  207,  335105,  335116,  335132, 11, 16, 24, }, // Anadyr
    {  351,  214,  335156,  335164,  335177,  8, 13, 15, }, // Apia
    {  351,  232,  335192,  335199,  335211,  7, 12, 14, }, // Arabian
    {  351,  240,  335225,  335239,  335258, 14, 19, 27, }, // Argentina
    {  351,  250,  335285,  335307,  335334, 22, 27, 35, }, // Argentina_Western
    {  351,  268,  335369,  335382,  335400, 13, 18, 26, }, // Armenia
    {  351,  286,  335426,  335438,  335455, 12, 17, 19, }, // Atlantic
    {  351,  295,  335474,  335495,  335526, 21, 31, 33, }, // Australia_Central
    {  351,  313,  335559,  335593,  335632, 34, 39, 41, }, // Australia_CentralWestern
    {  351,  338,  335673,  335695,  335724, 22, 29, 31, }, // Australia_Eastern
    {  351,  356,  335755,  335778,  335807, 23, 29, 31, }, // Australia_Western
    {  351,  374,  335838,  335851,  335869, 13, 18, 26, }, // Azerbaijan
    {  351,  385,  335895,  335906,  335922, 11, 16, 24, }, // Azores
    {  351,  397,  335946,  335960,  335979, 14, 19, 27, }, // Bangladesh
    {  351,  415,       0,  336006,       0,  0,  9,  0, }, // Bhutan
    {  351,  422,       0,  336015,       0,  0, 12,  0, }, // Bolivia
    {  351,  437,  336027,  336042,  336062, 15, 20, 28, }, // Brasilia
    {  351,  454,       0,  336090,       0,  0, 21,  0, }, // Brunei
    {  351,  461,  336111,  336123,  336140, 12, 17, 25, }, // Cape_Verde
    {  351,  478,       0,  336165,       0,  0, 15,  0, }, // Chamorro
    {  351,  487,  336180,  336188,  336201,  8, 13, 15, }, // Chatham
    {  351,  495,  336216,  336224,  336237,  8, 13, 21, }, // Chile
    {  351,  501,  336258,  336265,  336277,  7, 12, 14, }, // China
    {  351,  507,       0,  336291,       0,  0, 17,  0, }, // Christmas
    {  351,  517,       0,  336308,       0,  0, 19,  0, }, // Cocos
    {  351,  523,  336327,  336339,  336356, 12, 17, 25, }, // Colombia
    {  351,  532,  336381,  336398,  336420, 17, 22, 37, }, // Cook
    {  351,  537,  336457,  336467,  336482, 10, 15, 17, }, // Cuba
    {  351,  548,       0,  336499,       0,  0,  9,  0, }, // Davis
    {  351,  564,       0,  336508,       0,  0, 23,  0, }, // DumontDUrville
    {  351,  601,       0,  336531,       0,  0, 16,  0, }, // East_Timor
    {  351,  612,  336547,  336562,  336582, 15, 20, 28, }, // Easter
    {  351,  619,       0,  336610,       0,  0, 12,  0, }, // Ecuador
    {  351,  627,  336622,  336638,  336659, 16, 21, 31, }, // Europe_Central
    {  351,  642,  336690,  336708,  336731, 18, 23, 31, }, // Europe_Eastern
    {  351,  657,       0,  336762,       0,  0, 23,  0, }, // Europe_Further_Eastern
    {  351,  680,  336785,  336804,  336828, 19, 24, 33, }, // Europe_Western
    {  351,  695,  336861,  336883,  336910, 22, 27, 35, }, // Falkland
    {  351,  704,  336945,  336954,  336968,  9, 14, 22, }, // Fiji
    {  351,  709,       0,  336990,       0,  0, 18,  0, }, // French_Guiana
    {  351,  723,       0,  337008,       0,  0, 33,  0, }, // French_Southern
    {  351,  746,       0,  337041,       0,  0, 16,  0, }, // Galapagos
    {  351,  756,       0,  337057,       0,  0, 11,  0, }, // Gambier
    {  351,  764,  337068,  337080,  337097, 12, 17, 25, }, // Georgia
    {  351,  772,       0,  337122,       0,  0, 22,  0, }, // Gilbert_Islands
    {  351,  788,       0,   91115,       0,  0, 17,  0, }, // GMT
    {  351,  830,  337144,  337164,  337189, 20, 25, 33, }, // Greenland_Eastern
    {  351,  848,  337222,  337243,  337269, 21, 26, 34, }, // Greenland_Western
    {  351,  871,       0,  337303,       0,  0, 14,  0, }, // Gulf
    {  351,  876,       0,  336998,       0,  0, 10,  0, }, // Guyana
    {  351,  883,  337317,  337333,  337354, 16, 21, 23, }, // Hawaii_Aleutian
    {  351,  899,  337377,  337390,  337408, 13, 18, 26, }, // Hong_Kong
    {  351,  909,  337434,  337443,  337457,  9, 14, 22, }, // Hovd
    {  351,  914,       0,  337479,       0,  0, 15,  0, }, // India
    {  351,  920,       0,  337494,       0,  0, 16,  0, }, // Indian_Ocean
    {  351,  933,       0,  337510,       0,  0, 13,  0, }, // Indochina
    {  351,  943,       0,  337523,       0,  0, 19,  0, }, // Indonesia_Central
    {  351,  961,       0,  337542,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  351,  979,       0,  337563,       0,  0, 22,  0, }, // Indonesia_Western
    {  351,  997,  337585,  337593,  337606,  8, 13, 15, }, // Iran
    {  351, 1008,  337621,  337635,  337654, 14, 19, 27, }, // Irkutsk
    {  351, 1016,  337681,  337692,  337708, 11, 16, 18, }, // Israel
    {  351, 1023,  337726,  337735,  337749,  9, 14, 16, }, // Japan
    {  351, 1029,  337765,  337794,  337828, 29, 34, 42, }, // Kamchatka
    {  351, 1047,       0,  337870,       0,  0, 14,  0, }, // Kazakhstan
    {  351, 1058,       0,  337884,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  351, 1077,       0,  337904,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  351, 1106,  337925,  337936,  337952, 11, 16, 18, }, // Korea
    {  351, 1112,       0,  337970,       0,  0, 10,  0, }, // Kosrae
    {  351, 1119,  337980,  337999,  338023, 19, 24, 32, }, // Krasnoyarsk
    {  351, 1151,       0,  338055,       0,  0, 16,  0, }, // Kyrgystan
    {  351, 1175,       0,  338071,       0,  0, 18,  0, }, // Line_Islands
    {  351, 1188,  338089,  338103,  338122, 14, 19, 21, }, // Lord_Howe
    {  351, 1204,  338143,  338154,  338170, 11, 16, 24, }, // Magadan
    {  351, 1219,       0,  338194,       0,  0, 11,  0, }, // Malaysia
    {  351, 1228,       0,  338205,       0,  0, 10,  0, }, // Maldives
    {  351, 1237,       0,  338215,       0,  0, 12,  0, }, // Marquesas
    {  351, 1247,       0,  338227,       0,  0, 20,  0, }, // Marshall_Islands
    {  351, 1264,  338247,  338257,  338272, 10, 15, 23, }, // Mauritius
    {  351, 1274,       0,  338295,       0,  0, 10,  0, }, // Mawson
    {  351, 1281,  338305,  338325,  338350, 20, 25, 27, }, // Mexico_Pacific
    {  351, 1296,  338377,  338390,  338408, 13, 18, 26, }, // Mongolia
    {  351, 1305,  338434,  338444,  338459, 10, 15, 23, }, // Moscow
    {  351, 1312,       0,  338482,       0,  0, 12,  0, }, // Myanmar
    {  351, 1320,       0,  338494,       0,  0,  8,  0, }, // Nauru
    {  351, 1326,       0,  338502,       0,  0,  9,  0, }, // Nepal
    {  351, 1332,  338511,  338530,  338554, 19, 24, 32, }, // New_Caledonia
    {  351, 1346,  338586,  338601,  338621, 15, 20, 22, }, // New_Zealand
    {  351, 1358,  338643,  338661,  338684, 18, 23, 25, }, // Newfoundland
    {  351, 1371,       0,  338709,       0,  0,  8,  0, }, // Niue
    {  351, 1376,  338717,  338734,  338756, 17, 22, 24, }, // Norfolk
    {  351, 1384,  338780,  338807,  338839, 27, 32, 40, }, // Noronha
    {  351, 1406,  338879,  338896,  338918, 17, 22, 30, }, // Novosibirsk
    {  351, 1418,  338948,  338958,  338973, 10, 15, 23, }, // Omsk
    {  351, 1428,  338996,  339009,  339027, 13, 18, 26, }, // Pakistan
    {  351, 1437,       0,  339053,       0,  0,  8,  0, }, // Palau
    {  351, 1443,       0,  339061,       0,  0, 19,  0, }, // Papua_New_Guinea
    {  351, 1460,  339080,  339092,  339109, 12, 17, 25, }, // Paraguay
    {  351, 1469,  339134,  339142,  339155,  8, 13, 21, }, // Peru
    {  351, 1474,  339176,  339188,  339205, 12, 17, 25, }, // Philippines
    {  351, 1486,       0,  339230,       0,  0, 22,  0, }, // Phoenix_Islands
    {  351, 1502,  339252,  339279,  339311, 27, 32, 34, }, // Pierre_Miquelon
    {  351, 1518,       0,  339345,       0,  0, 12,  0, }, // Pitcairn
    {  351, 1527,       0,  339357,       0,  0, 10,  0, }, // Ponape
    {  351, 1534,       0,  339367,       0,  0, 14,  0, }, // Pyongyang
    {  351, 1554,       0,  339381,       0,  0, 12,  0, }, // Reunion
    {  351, 1562,       0,  339393,       0,  0, 10,  0, }, // Rothera
    {  351, 1570,  339403,  339413,  339428, 10, 15, 23, }, // Sakhalin
    {  351, 1579,  339451,  339460,  339474,  9, 14, 22, }, // Samara
    {  351, 1596,  339496,  339504,  339517,  8, 13, 15, }, // Samoa
    {  351, 1602,       0,  339532,       0,  0, 11,  0, }, // Seychelles
    {  351, 1624,       0,  339543,       0,  0, 12,  0, }, // Singapore
    {  351, 1634,       0,  339555,       0,  0, 20,  0, }, // Solomon
    {  351, 1642,       0,  339575,       0,  0, 20,  0, }, // South_Georgia
    {  351, 1656,       0,  339595,       0,  0, 11,  0, }, // Suriname
    {  351, 1676,       0,  339606,       0,  0, 10,  0, }, // Syowa
    {  351, 1682,       0,  339616,       0,  0, 10,  0, }, // Tahiti
    {  351, 1689,  339626,  339635,  339649,  9, 14, 16, }, // Taipei
    {  351, 1696,       0,  339665,       0,  0, 15,  0, }, // Tajikistan
    {  351, 1724,       0,  339680,       0,  0, 11,  0, }, // Tokelau
    {  351, 1732,  339691,  339700,  339714,  9, 14, 22, }, // Tonga
    {  351, 1738,       0,  339736,       0,  0,  7,  0, }, // Truk
    {  351, 1750,  339743,  339761,  339784, 18, 23, 31, }, // Turkmenistan
    {  351, 1763,       0,  339815,       0,  0, 10,  0, }, // Tuvalu
    {  351, 1777,  339825,  339836,  339852, 11, 16, 24, }, // Uruguay
    {  351, 1792,  339876,  339893,  339915, 17, 22, 30, }, // Uzbekistan
    {  351, 1803,  339945,  339955,  339970, 10, 15, 23, }, // Vanuatu
    {  351, 1811,       0,  339993,       0,  0, 14,  0, }, // Venezuela
    {  351, 1821,  340007,  340024,  340046, 17, 22, 30, }, // Vladivostok
    {  351, 1833,  340076,  340091,  340111, 15, 20, 28, }, // Volgograd
    {  351, 1843,       0,  340013,       0,  0, 11,  0, }, // Vostok
    {  351, 1850,       0,  340139,       0,  0, 13,  0, }, // Wake
    {  351, 1855,       0,  340152,       0,  0, 22,  0, }, // Wallis
    {  351, 1862,   97022,  340174,  340192, 13, 18, 26, }, // Yakutsk
    {  351, 1870,  340218,  340235,  340257, 17, 22, 30, }, // Yekaterinburg
    {  351, 1892,       0,  340287,       0,  0,  9,  0, }, // Yukon
    {  352,    0,       0,       9,      27,  9, 18, 16, }, // Acre Hindi/Latin/India
    {  352,    5,       0,      43,       0,  0, 16,  0, }, // Afghanistan
    {  352,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central
    {  352,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  352,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  352,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  352,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  352,  128,     235,     246,     266, 11, 20, 18, }, // Almaty
    {  352,  135,     284,     295,     315, 11, 20, 18, }, // Amazon
    {  352,  142,  340296,  340322,  340357, 26, 35, 35, }, // America_Central
    {  352,  158,  340392,  340418,  340453, 26, 35, 35, }, // America_Eastern
    {  352,  174,  340488,  340515,  340551, 27, 36, 36, }, // America_Mountain
    {  352,  191,  340587,  340613,  340648, 26, 35, 35, }, // America_Pacific
    {  352,  207,     552,     563,     583, 11, 20, 18, }, // Anadyr
    {  352,  214,     601,     611,     630, 10, 19, 19, }, // Apia
    {  352,  219,     649,     659,     678, 10, 19, 17, }, // Aqtau
    {  352,  225,     695,     706,     726, 11, 20, 18, }, // Aqtobe
    {  352,  232,     744,     756,     777, 12, 21, 21, }, // Arabian
    {  352,  240,     798,     812,     835, 14, 23, 21, }, // Argentina
    {  352,  250,     856,     878,     909, 22, 31, 29, }, // Argentina_Western
    {  352,  268,     938,     950,     971, 12, 21, 19, }, // Armenia
    {  352,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  352,  295,    1047,    1070,    1102, 23, 32, 32, }, // Australia_Central
    {  352,  313,    1134,    1165,    1205, 31, 40, 40, }, // Australia_CentralWestern
    {  352,  338,    1245,    1268,    1300, 23, 32, 32, }, // Australia_Eastern
    {  352,  356,    1332,    1355,    1387, 23, 32, 32, }, // Australia_Western
    {  352,  374,    1419,    1434,    1458, 15, 24, 22, }, // Azerbaijan
    {  352,  385,    1480,    1491,    1511, 11, 20, 18, }, // Azores
    {  352,  397,    1529,    1544,    1568, 15, 24, 22, }, // Bangladesh
    {  352,  415,       0,    1590,       0,  0, 11,  0, }, // Bhutan
    {  352,  422,       0,    1601,       0,  0, 12,  0, }, // Bolivia
    {  352,  437,    1613,    1626,    1648, 13, 22, 20, }, // Brasilia
    {  352,  454,       0,    1668,       0,  0, 11,  0, }, // Brunei
    {  352,  461,    1679,    1694,    1718, 15, 24, 22, }, // Cape_Verde
    {  352,  472,       0,    1740,       0,  0, 10,  0, }, // Casey
    {  352,  478,       0,    1750,       0,  0, 22,  0, }, // Chamorro
    {  352,  487,    1772,    1784,    1805, 12, 21, 21, }, // Chatham
    {  352,  495,    1826,    1836,    1855, 10, 19, 17, }, // Chile
    {  352,  501,    1872,    1882,    1901, 10, 19, 19, }, // China
    {  352,  507,       0,    1920,       0,  0, 21,  0, }, // Christmas
    {  352,  517,       0,    1941,       0,  0, 18,  0, }, // Cocos
    {  352,  523,    1959,    1972,    1994, 13, 22, 20, }, // Colombia
    {  352,  532,    2014,    2031,    2057, 17, 26, 24, }, // Cook
    {  352,  537,    2081,    2090,    2108,  9, 18, 18, }, // Cuba
    {  352,  548,       0,    2126,       0,  0, 10,  0, }, // Davis
    {  352,  564,       0,    2136,       0,  0, 21,  0, }, // DumontDUrville
    {  352,  601,       0,    2157,       0,  0, 16,  0, }, // East_Timor
    {  352,  612,    2173,    2191,    2218, 18, 27, 25, }, // Easter
    {  352,  619,       0,    2243,       0,  0, 12,  0, }, // Ecuador
    {  352,  627,    2255,    2276,    2306, 21, 30, 28, }, // Europe_Central
    {  352,  642,    2334,    2355,    2385, 21, 30, 28, }, // Europe_Eastern
    {  352,  657,       0,    2413,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {  352,  680,    2442,    2463,    2493, 21, 30, 28, }, // Europe_Western
    {  352,  695,    2521,    2542,    2572, 21, 30, 28, }, // Falkland
    {  352,  704,    2600,    2609,    2627,  9, 18, 16, }, // Fiji
    {  352,  709,       0,    2643,       0,  0, 18,  0, }, // French_Guiana
    {  352,  723,       0,    2661,       0,  0, 32,  0, }, // French_Southern
    {  352,  746,       0,    2693,       0,  0, 14,  0, }, // Galapagos
    {  352,  756,       0,    2707,       0,  0, 12,  0, }, // Gambier
    {  352,  764,    2719,    2731,    2752, 12, 21, 19, }, // Georgia
    {  352,  772,       0,    2771,       0,  0, 20,  0, }, // Gilbert_Islands
    {  352,  788,       0,    2791,       0,  0, 19,  0, }, // GMT
    {  352,  802,    2810,    2824,    2847, 14, 23, 21, }, // Greenland
    {  352,  830,    2868,    2887,    2915, 19, 28, 26, }, // Greenland_Eastern
    {  352,  848,    2941,    2960,    2988, 19, 28, 26, }, // Greenland_Western
    {  352,  866,       0,    3014,       0,  0, 18,  0, }, // Guam
    {  352,  871,       0,    3032,       0,  0, 18,  0, }, // Gulf
    {  352,  876,       0,    3050,       0,  0, 11,  0, }, // Guyana
    {  352,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  352,  899,    3139,    3153,    3176, 14, 23, 21, }, // Hong_Kong
    {  352,  909,  219298,  219308,  219327, 10, 19, 17, }, // Hovd
    {  352,  914,       0,    3240,       0,  0, 19,  0, }, // India
    {  352,  920,       0,    3259,       0,  0, 17,  0, }, // Indian_Ocean
    {  352,  933,       0,    3276,       0,  0, 14,  0, }, // Indochina
    {  352,  943,       0,    3290,       0,  0, 22,  0, }, // Indonesia_Central
    {  352,  961,       0,    3312,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  352,  979,       0,    3334,       0,  0, 22,  0, }, // Indonesia_Western
    {  352,  997,    3356,    3365,    3383,  9, 18, 18, }, // Iran
    {  352, 1008,    3401,    3413,    3434, 12, 21, 19, }, // Irkutsk
    {  352, 1016,    3453,    3464,    3484, 11, 20, 20, }, // Israel
    {  352, 1023,    3504,    3514,    3533, 10, 19, 19, }, // Japan
    {  352, 1029,    3552,    3566,    3589, 14, 23, 21, }, // Kamchatka
    {  352, 1047,       0,    3610,       0,  0, 15,  0, }, // Kazakhstan
    {  352, 1058,       0,    3625,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  352, 1077,       0,    3645,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  352, 1106,    3665,    3676,    3696, 11, 20, 20, }, // Korea
    {  352, 1112,       0,    3716,       0,  0, 11,  0, }, // Kosrae
    {  352, 1119,    3727,    3743,    3768, 16, 25, 23, }, // Krasnoyarsk
    {  352, 1151,       0,    3791,       0,  0, 15,  0, }, // Kyrgystan
    {  352, 1161,       0,    3806,       0,  0, 10,  0, }, // Lanka
    {  352, 1175,       0,    3816,       0,  0, 17,  0, }, // Line_Islands
    {  352, 1188,    3833,    3847,    3870, 14, 23, 23, }, // Lord_Howe
    {  352, 1198,    3893,    3903,    3922, 10, 19, 17, }, // Macau
    {  352, 1204,    3939,    3951,    3972, 12, 21, 19, }, // Magadan
    {  352, 1219,       0,    3991,       0,  0, 13,  0, }, // Malaysia
    {  352, 1228,       0,    4004,       0,  0, 13,  0, }, // Maldives
    {  352, 1237,       0,    4017,       0,  0, 14,  0, }, // Marquesas
    {  352, 1247,       0,    4031,       0,  0, 21,  0, }, // Marshall_Islands
    {  352, 1264,    4052,    4066,    4089, 14, 23, 21, }, // Mauritius
    {  352, 1274,       0,    4110,       0,  0, 11,  0, }, // Mawson
    {  352, 1281,    4121,    4141,    4170, 20, 29, 29, }, // Mexico_Pacific
    {  352, 1296,    4199,    4215,    4240, 16, 25, 23, }, // Mongolia
    {  352, 1305,    4263,    4274,    4294, 11, 20, 18, }, // Moscow
    {  352, 1312,       0,    4312,       0,  0, 12,  0, }, // Myanmar
    {  352, 1320,       0,    4324,       0,  0, 10,  0, }, // Nauru
    {  352, 1326,       0,    4334,       0,  0, 10,  0, }, // Nepal
    {  352, 1332,    4344,    4362,    4389, 18, 27, 25, }, // New_Caledonia
    {  352, 1346,    4414,    4430,    4455, 16, 25, 25, }, // New_Zealand
    {  352, 1358,    4480,    4497,    4523, 17, 26, 26, }, // Newfoundland
    {  352, 1371,       0,    4549,       0,  0,  9,  0, }, // Niue
    {  352, 1376,    4558,    4577,    4605, 19, 28, 28, }, // Norfolk
    {  352, 1384,    4633,    4657,    4690, 24, 33, 31, }, // Noronha
    {  352, 1392,       0,    4721,       0,  0, 29,  0, }, // North_Mariana
    {  352, 1406,    4750,    4766,    4791, 16, 25, 23, }, // Novosibirsk
    {  352, 1418,    4814,    4823,    4841,  9, 18, 16, }, // Omsk
    {  352, 1428,    4857,    4870,    4892, 13, 22, 20, }, // Pakistan
    {  352, 1437,       0,    4912,       0,  0, 10,  0, }, // Palau
    {  352, 1443,       0,    4922,       0,  0, 21,  0, }, // Papua_New_Guinea
    {  352, 1460,    4943,    4956,    4978, 13, 22, 20, }, // Paraguay
    {  352, 1469,    4998,    5007,    5025,  9, 18, 16, }, // Peru
    {  352, 1474,    5041,    5056,    5080, 15, 24, 22, }, // Philippines
    {  352, 1486,       0,    5102,       0,  0, 20,  0, }, // Phoenix_Islands
    {  352, 1502,    5122,    5148,    5183, 26, 35, 35, }, // Pierre_Miquelon
    {  352, 1518,       0,    5218,       0,  0, 13,  0, }, // Pitcairn
    {  352, 1527,       0,    5231,       0,  0, 12,  0, }, // Ponape
    {  352, 1534,       0,    5243,       0,  0, 16,  0, }, // Pyongyang
    {  352, 1544,    5259,    5273,    5296, 14, 23, 21, }, // Qyzylorda
    {  352, 1554,       0,  340683,       0,  0, 12,  0, }, // Reunion
    {  352, 1562,       0,    5329,       0,  0, 12,  0, }, // Rothera
    {  352, 1570,    5341,    5354,    5376, 13, 22, 20, }, // Sakhalin
    {  352, 1579,    5396,    5407,    5427, 11, 20, 18, }, // Samara
    {  352, 1596,    5445,    5464,    5492, 19, 28, 28, }, // Samoa
    {  352, 1602,       0,    5520,       0,  0, 15,  0, }, // Seychelles
    {  352, 1624,       0,    5535,       0,  0, 23,  0, }, // Singapore
    {  352, 1634,       0,    5558,       0,  0, 20,  0, }, // Solomon
    {  352, 1642,       0,    5578,       0,  0, 18,  0, }, // South_Georgia
    {  352, 1656,       0,    5596,       0,  0, 13,  0, }, // Suriname
    {  352, 1676,       0,    5609,       0,  0, 10,  0, }, // Syowa
    {  352, 1682,       0,    5619,       0,  0, 11,  0, }, // Tahiti
    {  352, 1689,    5630,    5641,    5661, 11, 20, 20, }, // Taipei
    {  352, 1696,       0,    5681,       0,  0, 15,  0, }, // Tajikistan
    {  352, 1724,       0,    5696,       0,  0, 12,  0, }, // Tokelau
    {  352, 1732,    5708,    5718,    5737, 10, 19, 17, }, // Tonga
    {  352, 1738,       0,    5754,       0,  0, 10,  0, }, // Truk
    {  352, 1750,    5764,    5781,    5807, 17, 26, 24, }, // Turkmenistan
    {  352, 1763,       0,    5831,       0,  0, 11,  0, }, // Tuvalu
    {  352, 1777,    5842,    5854,    5875, 12, 21, 19, }, // Uruguay
    {  352, 1792,    5894,    5909,    5933, 15, 24, 22, }, // Uzbekistan
    {  352, 1803,    5955,    5967,    5988, 12, 21, 19, }, // Vanuatu
    {  352, 1811,       0,    6007,       0,  0, 14,  0, }, // Venezuela
    {  352, 1821,    6021,    6037,    6062, 16, 25, 23, }, // Vladivostok
    {  352, 1833,    6085,    6099,    6122, 14, 23, 21, }, // Volgograd
    {  352, 1843,       0,    6143,       0,  0, 11,  0, }, // Vostok
    {  352, 1850,       0,    6154,       0,  0, 16,  0, }, // Wake
    {  352, 1855,       0,    6170,       0,  0, 20,  0, }, // Wallis
    {  352, 1862,    6190,    6202,    6223, 12, 21, 19, }, // Yakutsk
    {  352, 1870,    6242,    6260,    6287, 18, 27, 25, }, // Yekaterinburg
    {  352, 1892,       0,    6312,       0,  0, 10,  0, }, // Yukon
    {  353,    0,  340695,  340703,  340715,  8, 12, 14, }, // Acre Hungarian/Latin/Hungary
    {  353,    5,       0,  340729,       0,  0, 16,  0, }, // Afghanistan
    {  353,   17,       0,  340745,       0,  0, 22,  0, }, // Africa_Central
    {  353,   32,       0,  340767,       0,  0, 22,  0, }, // Africa_Eastern
    {  353,   65,       0,  340789,       0,  0, 20,  0, }, // Africa_Southern
    {  353,   81,  340809,  340831,  340854, 22, 23, 24, }, // Africa_Western
    {  353,  107,  340878,  340890,  340906, 12, 16, 18, }, // Alaska
    {  353,  128,  340924,  340934,  340948, 10, 14, 16, }, // Almaty
    {  353,  135,  340964,  340977,  340995, 13, 18, 19, }, // Amazon
    {  353,  142,  341014,  341037,  341064, 23, 27, 29, }, // America_Central
    {  353,  158,  341093,  341115,  341141, 22, 26, 28, }, // America_Eastern
    {  353,  174,  341169,  341183,  341201, 14, 18, 20, }, // America_Mountain
    {  353,  191,  341221,  341239,  341261, 18, 22, 24, }, // America_Pacific
    {  353,  207,  341285,  341296,  341311, 11, 15, 17, }, // Anadyr
    {  353,  214,  341328,  341337,  341351,  9, 14, 15, }, // Apia
    {  353,  219,  341366,  341376,  341390, 10, 14, 16, }, // Aqtau
    {  353,  225,  341406,  341417,  341432, 11, 15, 17, }, // Aqtobe
    {  353,  232,  341449,  341457,  341470,  8, 13, 14, }, // Arabian
    {  353,  240,  341484,  341498,  341517, 14, 19, 20, }, // Argentina
    {  353,  250,  341537,  341562,  341588, 25, 26, 27, }, // Argentina_Western
    {  353,  268,  341615,  341632,  341654, 17, 22, 23, }, // Armenia
    {  353,  286,  341677,  341695,  341717, 18, 22, 24, }, // Atlantic
    {  353,  295,  341741,  341762,  341788, 21, 26, 27, }, // Australia_Central
    {  353,  313,  341815,  341843,  341876, 28, 33, 34, }, // Australia_CentralWestern
    {  353,  338,  341910,  341931,  341957, 21, 26, 27, }, // Australia_Eastern
    {  353,  356,  341821,  341849,  341882, 22, 27, 28, }, // Australia_Western
    {  353,  374,  341984,  342001,  342023, 17, 22, 23, }, // Azerbaijan
    {  353,  385,  342046,  342059,  342073, 13, 14, 15, }, // Azores
    {  353,  397,  342088,  342102,  342121, 14, 19, 20, }, // Bangladesh
    {  353,  415,       0,  342141,       0,  0, 10,  0, }, // Bhutan
    {  353,  422,       0,  342151,       0,  0, 17,  0, }, // Bolivia
    {  353,  437,  342168,  342181,  342199, 13, 18, 19, }, // Brasilia
    {  353,  454,       0,  342218,       0,  0, 23,  0, }, // Brunei
    {  353,  461,  342241,  342268,  342296, 27, 28, 29, }, // Cape_Verde
    {  353,  472,       0,  342325,       0,  0, 11,  0, }, // Casey
    {  353,  478,       0,  342336,       0,  0, 18,  0, }, // Chamorro
    {  353,  487,  342354,  342366,  342383, 12, 17, 18, }, // Chatham
    {  353,  495,  342401,  342415,  342430, 14, 15, 16, }, // Chile
    {  353,  501,  342446,  342455,  342469,  9, 14, 15, }, // China
    {  353,  507,       0,  342484,       0,  0, 21,  0, }, // Christmas
    {  353,  517,       0,  342505,       0,  0, 25,  0, }, // Cocos
    {  353,  523,  342530,  342543,  342561, 13, 18, 19, }, // Colombia
    {  353,  532,  342580,  342598,  342621, 18, 23, 28, }, // Cook
    {  353,  537,  342649,  342662,  342676, 13, 14, 15, }, // Cuba
    {  353,  548,       0,  342691,       0,  0, 10,  0, }, // Davis
    {  353,  564,       0,  342701,       0,  0, 22,  0, }, // DumontDUrville
    {  353,  601,       0,  342723,       0,  0, 21,  0, }, // East_Timor
    {  353,  612,  342744,  342766,  342789, 22, 23, 24, }, // Easter
    {  353,  619,       0,  342813,       0,  0, 17,  0, }, // Ecuador
    {  353,  627,  342830,  342851,  342873, 21, 22, 23, }, // Europe_Central
    {  353,  642,  342896,  342917,  342939, 21, 22, 23, }, // Europe_Eastern
    {  353,  657,       0,  342962,       0,  0, 11,  0, }, // Europe_Further_Eastern
    {  353,  680,  342973,  342995,  343018, 22, 23, 24, }, // Europe_Western
    {  353,  695,  343042,  343064,  343091, 22, 27, 28, }, // Falkland
    {  353,  704,  343119,  343129,  343144, 10, 15, 16, }, // Fiji
    {  353,  709,       0,  343160,       0,  0, 19,  0, }, // French_Guiana
    {  353,  723,       0,  343179,       0,  0, 31,  0, }, // French_Southern
    {  353,  746,       0,  343210,       0,  0, 19,  0, }, // Galapagos
    {  353,  756,       0,  343229,       0,  0, 12,  0, }, // Gambier
    {  353,  764,  343241,  343252,  343268, 11, 16, 17, }, // Georgia
    {  353,  772,       0,  343285,       0,  0, 21,  0, }, // Gilbert_Islands
    {  353,  788,       0,  343306,       0,  0, 29,  0, }, // GMT
    {  353,  830,  343335,  343358,  343382, 23, 24, 25, }, // Greenland_Eastern
    {  353,  848,  343407,  343431,  343456, 24, 25, 26, }, // Greenland_Western
    {  353,  866,       0,  343482,       0,  0, 13,  0, }, // Guam
    {  353,  871,       0,  343495,       0,  0, 17,  0, }, // Gulf
    {  353,  876,       0,  343512,       0,  0, 16,  0, }, // Guyana
    {  353,  883,  343528,  343549,  343571, 21, 22, 23, }, // Hawaii_Aleutian
    {  353,  899,  343594,  343611,  343629, 17, 18, 19, }, // Hong_Kong
    {  353,  909,  343648,  343657,  343671,  9, 14, 15, }, // Hovd
    {  353,  914,       0,  343686,       0,  0, 15,  0, }, // India
    {  353,  920,       0,  343701,       0,  0, 17,  0, }, // Indian_Ocean
    {  353,  933,       0,  343718,       0,  0, 13,  0, }, // Indochina
    {  353,  943,       0,  343731,       0,  0, 20,  0, }, // Indonesia_Central
    {  353,  961,       0,  343751,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  353,  979,       0,  343771,       0,  0, 26,  0, }, // Indonesia_Western
    {  353,  997,  343797,  343806,  343820,  9, 14, 15, }, // Iran
    {  353, 1008,  343835,  343848,  343866, 13, 18, 19, }, // Irkutsk
    {  353, 1016,  343885,  343896,  343912, 11, 16, 17, }, // Israel
    {  353, 1023,  343929,  343938,  343952,  9, 14, 15, }, // Japan
    {  353, 1029,  343967,  343996,  344029, 29, 33, 35, }, // Kamchatka
    {  353, 1047,       0,  344064,       0,  0, 15,  0, }, // Kazakhstan
    {  353, 1058,       0,  344079,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {  353, 1077,       0,  344100,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  353, 1106,  344122,  344132,  344147, 10, 15, 16, }, // Korea
    {  353, 1112,       0,  344163,       0,  0, 11,  0, }, // Kosrae
    {  353, 1119,  344174,  344192,  344215, 18, 23, 24, }, // Krasnoyarsk
    {  353, 1151,       0,  344239,       0,  0, 17,  0, }, // Kyrgystan
    {  353, 1161,       0,  344256,       0,  0, 10,  0, }, // Lanka
    {  353, 1175,       0,  344266,       0,  0, 17,  0, }, // Line_Islands
    {  353, 1188,  344283,  344304,  344330, 21, 26, 27, }, // Lord_Howe
    {  353, 1198,  344357,  344367,  344381, 10, 14, 16, }, // Macau
    {  353, 1204,  344397,  344409,  344426, 12, 17, 18, }, // Magadan
    {  353, 1219,       0,  344444,       0,  0, 13,  0, }, // Malaysia
    {  353, 1228,       0,  344457,       0,  0, 20,  0, }, // Maldives
    {  353, 1237,       0,  344477,       0,  0, 23,  0, }, // Marquesas
    {  353, 1247,       0,  344500,       0,  0, 22,  0, }, // Marshall_Islands
    {  353, 1264,  344522,  344540,  344559, 18, 19, 20, }, // Mauritius
    {  353, 1274,       0,  344579,       0,  0, 11,  0, }, // Mawson
    {  353, 1281,  344590,  344616,  344646, 26, 30, 32, }, // Mexico_Pacific
    {  353, 1296,  344678,  344692,  344711, 14, 19, 20, }, // Mongolia
    {  353, 1305,  344731,  344743,  344760, 12, 17, 18, }, // Moscow
    {  353, 1312,       0,  344778,       0,  0, 12,  0, }, // Myanmar
    {  353, 1320,       0,  344790,       0,  0, 10,  0, }, // Nauru
    {  353, 1326,       0,  344800,       0,  0, 10,  0, }, // Nepal
    {  353, 1332,  344810,  344827,  344849, 17, 22, 23, }, // New_Caledonia
    {  353, 1346,  344872,  344886,  344905, 14, 19, 20, }, // New_Zealand
    {  353, 1358,  344925,  344941,  344961, 16, 20, 22, }, // Newfoundland
    {  353, 1371,       0,  344983,       0,  0,  9,  0, }, // Niue
    {  353, 1376,  344992,  345013,  345039, 21, 26, 27, }, // Norfolk
    {  353, 1384,  345066,  345091,  345121, 25, 30, 31, }, // Noronha
    {  353, 1392,       0,  345152,       0,  0, 27,  0, }, // North_Mariana
    {  353, 1406,  345179,  345197,  345220, 18, 23, 24, }, // Novosibirsk
    {  353, 1418,  345244,  345254,  345269, 10, 15, 16, }, // Omsk
    {  353, 1428,  345285,  345299,  345318, 14, 19, 20, }, // Pakistan
    {  353, 1437,       0,  345338,       0,  0, 10,  0, }, // Palau
    {  353, 1443,       0,  345348,       0,  0, 20,  0, }, // Papua_New_Guinea
    {  353, 1460,  345368,  345381,  345399, 13, 18, 19, }, // Paraguay
    {  353, 1469,  345418,  345427,  345441,  9, 14, 15, }, // Peru
    {  353, 1474,  345456,  345475,  345499, 19, 24, 25, }, // Philippines
    {  353, 1486,       0,  345524,       0,  0, 26,  0, }, // Phoenix_Islands
    {  353, 1502,  345550,  345580,  345614, 30, 34, 36, }, // Pierre_Miquelon
    {  353, 1518,       0,  345650,       0,  0, 22,  0, }, // Pitcairn
    {  353, 1527,       0,  345672,       0,  0, 18,  0, }, // Ponape
    {  353, 1534,       0,  345690,       0,  0, 12,  0, }, // Pyongyang
    {  353, 1544,  345702,  345716,  345734, 14, 18, 20, }, // Qyzylorda
    {  353, 1554,       0,  345754,       0,  0, 12,  0, }, // Reunion
    {  353, 1562,       0,  345766,       0,  0, 12,  0, }, // Rothera
    {  353, 1570,  345778,  345791,  345809, 13, 18, 19, }, // Sakhalin
    {  353, 1579,  345828,  345840,  345856, 12, 16, 18, }, // Samara
    {  353, 1596,  345874,  345885,  345901, 11, 16, 17, }, // Samoa
    {  353, 1602,       0,  345918,       0,  0, 23,  0, }, // Seychelles
    {  353, 1624,       0,  345941,       0,  0, 19,  0, }, // Singapore
    {  353, 1634,       0,  345960,       0,  0, 21,  0, }, // Solomon
    {  353, 1642,       0,  345981,       0,  0, 17,  0, }, // South_Georgia
    {  353, 1656,       0,  345998,       0,  0, 13,  0, }, // Suriname
    {  353, 1676,       0,  346011,       0,  0, 10,  0, }, // Syowa
    {  353, 1682,       0,  346021,       0,  0, 10,  0, }, // Tahiti
    {  353, 1689,  346031,  346041,  346056, 10, 15, 16, }, // Taipei
    {  353, 1696,       0,  346072,       0,  0, 18,  0, }, // Tajikistan
    {  353, 1724,       0,  346090,       0,  0, 12,  0, }, // Tokelau
    {  353, 1732,  346102,  346112,  346127, 10, 15, 16, }, // Tonga
    {  353, 1738,       0,  346143,       0,  0,  9,  0, }, // Truk
    {  353, 1750,  346152,  346170,  346193, 18, 23, 24, }, // Turkmenistan
    {  353, 1763,       0,  346217,       0,  0, 11,  0, }, // Tuvalu
    {  353, 1777,  346228,  346240,  346257, 12, 17, 18, }, // Uruguay
    {  353, 1792,  346275,  346291,  346312, 16, 21, 22, }, // Uzbekistan
    {  353, 1803,  346334,  346346,  346363, 12, 17, 18, }, // Vanuatu
    {  353, 1811,       0,  346381,       0,  0, 14,  0, }, // Venezuela
    {  353, 1821,  346395,  346413,  346436, 18, 23, 24, }, // Vladivostok
    {  353, 1833,  346460,  346474,  346493, 14, 19, 20, }, // Volgograd
    {  353, 1843,       0,  346401,       0,  0, 12,  0, }, // Vostok
    {  353, 1850,       0,  346513,       0,  0, 16,  0, }, // Wake
    {  353, 1855,       0,  346529,       0,  0, 22,  0, }, // Wallis
    {  353, 1862,  346551,  346564,  346582, 13, 18, 19, }, // Yakutsk
    {  353, 1870,  346601,  346620,  346644, 19, 24, 25, }, // Yekaterinburg
    {  353, 1892,       0,  346669,       0,  0, 10,  0, }, // Yukon
    {  354,    5,       0,  346679,       0,  0, 14,  0, }, // Afghanistan Icelandic/Latin/Iceland
    {  354,   17,       0,  346693,       0,  0, 14,  0, }, // Africa_Central
    {  354,   32,       0,  346707,       0,  0, 17,  0, }, // Africa_Eastern
    {  354,   65,       0,  346724,       0,  0, 16,  0, }, // Africa_Southern
    {  354,   81,  346740,  346757,  346783, 17, 26, 25, }, // Africa_Western
    {  354,  107,  346808,  346821,  346840, 13, 19, 18, }, // Alaska
    {  354,  135,  346858,  346868,  346894, 10, 26, 25, }, // Amazon
    {  354,  142,  346919,  346958,  347003, 39, 45, 44, }, // America_Central
    {  354,  158,  347047,  347089,  347137, 42, 48, 47, }, // America_Eastern
    {  354,  174,  347184,  347204,  347230, 20, 26, 25, }, // America_Mountain
    {  354,  191,  347255,  347278,  347307, 23, 29, 28, }, // America_Pacific
    {  354,  207,  347335,  347348,  347367, 13, 19, 18, }, // Anadyr
    {  354,  214,  347385,  347396,  347413, 11, 17, 16, }, // Apia
    {  354,  232,  347429,  347439,  347458, 10, 19, 18, }, // Arabian
    {  354,  240,  347476,  347489,  347511, 13, 22, 21, }, // Argentina
    {  354,  250,  347532,  347552,  347581, 20, 29, 28, }, // Argentina_Western
    {  354,  268,  347609,  347620,  347640, 11, 20, 19, }, // Armenia
    {  354,  286,  347659,  347684,  347715, 25, 31, 30, }, // Atlantic
    {  354,  295,  347745,  347764,  347789, 19, 25, 24, }, // Australia_Central
    {  354,  313,  347813,  347843,  347879, 30, 36, 35, }, // Australia_CentralWestern
    {  354,  338,  347914,  347936,  347964, 22, 28, 27, }, // Australia_Eastern
    {  354,  356,  347991,  348013,  348041, 22, 28, 27, }, // Australia_Western
    {  354,  374,  348068,  348084,  348109, 16, 25, 24, }, // Azerbaijan
    {  354,  385,  348133,  348145,  348167, 12, 22, 21, }, // Azores
    {  354,  397,  348188,  348203,  348226, 15, 23, 22, }, // Bangladesh
    {  354,  415,       0,  348248,       0,  0,  9,  0, }, // Bhutan
    {  354,  422,       0,  348257,       0,  0, 11,  0, }, // Bolivia
    {  354,  437,  348268,  348280,  348301, 12, 21, 20, }, // Brasilia
    {  354,  454,       0,  348321,       0,  0, 10,  0, }, // Brunei
    {  354,  461,  348331,  348348,  348375, 17, 27, 26, }, // Cape_Verde
    {  354,  478,       0,  348401,       0,  0, 19,  0, }, // Chamorro
    {  354,  487,  348420,  348432,  348452, 12, 20, 19, }, // Chatham
    {  354,  495,  348471,  348479,  348496,  8, 17, 16, }, // Chile
    {  354,  501,  348512,  348520,  348537,  8, 17, 16, }, // China
    {  354,  507,       0,  348553,       0,  0, 13,  0, }, // Christmas
    {  354,  517,       0,  348566,       0,  0, 13,  0, }, // Cocos
    {  354,  523,  348579,  348591,  348612, 12, 21, 20, }, // Colombia
    {  354,  532,  348632,  348646,  348670, 14, 24, 27, }, // Cook
    {  354,  537,  348697,  348705,  348722,  8, 17, 16, }, // Cuba
    {  354,  548,       0,  348738,       0,  0, 10,  0, }, // Davis
    {  354,  564,       0,  348748,       0,  0, 23,  0, }, // DumontDUrville
    {  354,  601,       0,  348771,       0,  0, 18,  0, }, // East_Timor
    {  354,  612,  348789,  348802,  348824, 13, 22, 21, }, // Easter
    {  354,  619,       0,  348845,       0,  0, 11,  0, }, // Ecuador
    {  354,  627,  348856,  348870,  348893, 14, 23, 22, }, // Europe_Central
    {  354,  642,  348915,  348932,  348958, 17, 26, 25, }, // Europe_Eastern
    {  354,  657,       0,  348983,       0,  0, 22,  0, }, // Europe_Further_Eastern
    {  354,  680,  349005,  349022,  349048, 17, 26, 25, }, // Europe_Western
    {  354,  695,  349073,  349090,  349117, 17, 27, 26, }, // Falkland
    {  354,  704,  349143,  349156,  349179, 13, 23, 22, }, // Fiji
    {  354,  709,       0,  349201,       0,  0, 22,  0, }, // French_Guiana
    {  354,  723,       0,  349223,       0,  0, 55,  0, }, // French_Southern
    {  354,  746,       0,  349278,       0,  0, 14,  0, }, // Galapagos
    {  354,  756,       0,  349292,       0,  0, 12,  0, }, // Gambier
    {  354,  764,  349304,  349315,  349335, 11, 20, 19, }, // Georgia
    {  354,  772,       0,  349354,       0,  0, 20,  0, }, // Gilbert_Islands
    {  354,  788,       0,  349374,       0,  0, 20,  0, }, // GMT
    {  354,  830,  349394,  349414,  349443, 20, 29, 28, }, // Greenland_Eastern
    {  354,  848,  349471,  349491,  349520, 20, 29, 28, }, // Greenland_Western
    {  354,  871,       0,  349548,       0,  0, 24,  0, }, // Gulf
    {  354,  876,       0,  349572,       0,  0, 11,  0, }, // Guyana
    {  354,  883,  349583,  349605,  349633, 22, 28, 27, }, // Hawaii_Aleutian
    {  354,  899,  349660,  349674,  349696, 14, 22, 21, }, // Hong_Kong
    {  354,  909,  349717,  349726,  349743,  9, 17, 16, }, // Hovd
    {  354,  914,       0,  349759,       0,  0, 12,  0, }, // India
    {  354,  920,       0,  349771,       0,  0, 16,  0, }, // Indian_Ocean
    {  354,  933,       0,  349787,       0,  0, 12,  0, }, // Indochina
    {  354,  943,       0,  349799,       0,  0, 17,  0, }, // Indonesia_Central
    {  354,  961,       0,  349816,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  354,  979,       0,  349836,       0,  0, 20,  0, }, // Indonesia_Western
    {  354,  997,  349856,  349865,  349882,  9, 17, 16, }, // Iran
    {  354, 1008,  349898,  349912,  349932, 14, 20, 19, }, // Irkutsk
    {  354, 1016,  349951,  349962,  349981, 11, 19, 18, }, // Israel
    {  354, 1023,  349999,  350009,  350027, 10, 18, 17, }, // Japan
    {  354, 1029,  350044,  350075,  350112, 31, 37, 36, }, // Kamchatka
    {  354, 1047,       0,  350148,       0,  0, 16,  0, }, // Kazakhstan
    {  354, 1058,       0,  350164,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  354, 1077,       0,  350187,       0,  0, 23,  0, }, // Kazakhstan_Western
    {  354, 1106,  350210,  350219,  350237,  9, 18, 17, }, // Korea
    {  354, 1112,       0,  350254,       0,  0, 11,  0, }, // Kosrae
    {  354, 1119,  350265,  350283,  350307, 18, 24, 23, }, // Krasnoyarsk
    {  354, 1151,       0,  350330,       0,  0, 14,  0, }, // Kyrgystan
    {  354, 1175,       0,  350344,       0,  0, 13,  0, }, // Line_Islands
    {  354, 1188,  350357,  350378,  350405, 21, 27, 26, }, // Lord_Howe
    {  354, 1204,  350431,  350445,  350465, 14, 20, 19, }, // Magadan
    {  354, 1219,       0,  350484,       0,  0, 11,  0, }, // Malaysia
    {  354, 1228,       0,  350495,       0,  0, 15,  0, }, // Maldives
    {  354, 1237,       0,  350510,       0,  0, 26,  0, }, // Marquesas
    {  354, 1247,       0,  350536,       0,  0, 21,  0, }, // Marshall_Islands
    {  354, 1264,  350557,  350569,  350590, 12, 21, 20, }, // Mauritius
    {  354, 1274,       0,  350610,       0,  0, 11,  0, }, // Mawson
    {  354, 1281,  350621,  350643,  350681, 22, 38, 37, }, // Mexico_Pacific
    {  354, 1296,  350718,  350735,  350758, 17, 23, 22, }, // Mongolia
    {  354, 1305,  350780,  350790,  350809, 10, 19, 18, }, // Moscow
    {  354, 1312,       0,  350827,       0,  0, 12,  0, }, // Myanmar
    {  354, 1320,       0,  350839,       0,  0,  9,  0, }, // Nauru
    {  354, 1326,       0,  350848,       0,  0,  9,  0, }, // Nepal
    {  354, 1332,  350857,  350878,  350905, 21, 27, 26, }, // New_Caledonia
    {  354, 1346,  350931,  350951,  350977, 20, 26, 25, }, // New_Zealand
    {  354, 1358,  351002,  351022,  351048, 20, 26, 25, }, // Newfoundland
    {  354, 1371,       0,  351073,       0,  0,  9,  0, }, // Niue
    {  354, 1376,  351082,  351101,  351126, 19, 25, 24, }, // Norfolk
    {  354, 1384,  351150,  351176,  351208, 26, 32, 31, }, // Noronha
    {  354, 1406,  351239,  351257,  351281, 18, 24, 23, }, // Novosibirsk
    {  354, 1418,  351304,  351315,  351332, 11, 17, 16, }, // Omsk
    {  354, 1428,  351348,  351360,  351381, 12, 21, 20, }, // Pakistan
    {  354, 1437,       0,  351401,       0,  0,  8,  0, }, // Palau
    {  354, 1443,       0,  351409,       0,  0, 23,  0, }, // Papua_New_Guinea
    {  354, 1460,  351432,  351443,  351463, 11, 20, 19, }, // Paraguay
    {  354, 1469,  351482,  351490,  351507,  8, 17, 16, }, // Peru
    {  354, 1474,  351523,  351538,  351563, 15, 25, 24, }, // Philippines
    {  354, 1486,       0,  351587,       0,  0, 13,  0, }, // Phoenix_Islands
    {  354, 1502,  351600,  351632,  351670, 32, 38, 37, }, // Pierre_Miquelon
    {  354, 1518,       0,  351707,       0,  0, 13,  0, }, // Pitcairn
    {  354, 1527,       0,  351720,       0,  0, 11,  0, }, // Ponape
    {  354, 1534,       0,  351731,       0,  0, 16,  0, }, // Pyongyang
    {  354, 1554,       0,  351747,       0,  0, 12,  0, }, // Reunion
    {  354, 1562,       0,  351759,       0,  0, 12,  0, }, // Rothera
    {  354, 1570,  351771,  351786,  351807, 15, 21, 20, }, // Sakhalin
    {  354, 1579,  351827,  351840,  351859, 13, 19, 18, }, // Samara
    {  354, 1596,  351877,  351887,  351905, 10, 18, 17, }, // Samoa
    {  354, 1602,       0,  351922,       0,  0, 19,  0, }, // Seychelles
    {  354, 1624,       0,  351941,       0,  0, 12,  0, }, // Singapore
    {  354, 1634,       0,  351953,       0,  0, 16,  0, }, // Solomon
    {  354, 1642,       0,  351969,       0,  0, 17,  0, }, // South_Georgia
    {  354, 1656,       0,  351986,       0,  0, 11,  0, }, // Suriname
    {  354, 1676,       0,  351997,       0,  0, 10,  0, }, // Syowa
    {  354, 1682,       0,  352007,       0,  0, 11,  0, }, // Tahiti
    {  354, 1689,  352018,  352029,  352048, 11, 19, 18, }, // Taipei
    {  354, 1696,       0,  352066,       0,  0, 17,  0, }, // Tajikistan
    {  354, 1724,       0,  352083,       0,  0, 11,  0, }, // Tokelau
    {  354, 1732,  352094,  352103,  352121,  9, 18, 17, }, // Tonga
    {  354, 1738,       0,  352138,       0,  0, 10,  0, }, // Truk
    {  354, 1750,  352148,  352165,  352190, 17, 25, 24, }, // Turkmenistan
    {  354, 1763,       0,  352214,       0,  0, 10,  0, }, // Tuvalu
    {  354, 1777,  352224,  352234,  352253, 10, 19, 18, }, // Uruguay
    {  354, 1792,  352271,  352286,  352309, 15, 23, 22, }, // Uzbekistan
    {  354, 1803,  352331,  352343,  352363, 12, 20, 19, }, // Vanuatu
    {  354, 1811,       0,  352382,       0,  0, 13,  0, }, // Venezuela
    {  354, 1821,  352395,  352413,  352437, 18, 24, 23, }, // Vladivostok
    {  354, 1833,  352460,  352476,  352498, 16, 22, 21, }, // Volgograd
    {  354, 1843,       0,  352519,       0,  0, 11,  0, }, // Vostok
    {  354, 1850,       0,  352530,       0,  0, 16,  0, }, // Wake
    {  354, 1855,       0,  352546,       0,  0, 29,  0, }, // Wallis
    {  354, 1862,  352575,  352589,  352609, 14, 20, 19, }, // Yakutsk
    {  354, 1870,  352628,  352648,  352674, 20, 26, 25, }, // Yekaterinburg
    {  354, 1892,       0,  352699,       0,  0, 12,  0, }, // Yukon
    {  356,    5,       0,  352711,       0,  0, 15,  0, }, // Afghanistan Igbo/Latin/Nigeria
    {  356,   17,       0,  352726,       0,  0, 16,  0, }, // Africa_Central
    {  356,   32,       0,  352742,       0,  0, 33,  0, }, // Africa_Eastern
    {  356,   65,       0,  352775,       0,  0, 38,  0, }, // Africa_Southern
    {  356,   81,  352813,  352846,  352886, 33, 40, 42, }, // Africa_Western
    {  356,  107,  352928,  352938,  352955, 10, 17, 14, }, // Alaska
    {  356,  135,  352969,  352979,  352996, 10, 17, 19, }, // Amazon
    {  356,  142,  353015,  353033,  353058, 18, 25, 22, }, // America_Central
    {  356,  158,  352742,  353080,  353113, 26, 33, 30, }, // America_Eastern
    {  356,  174,  353143,  353160,  353184, 17, 24, 21, }, // America_Mountain
    {  356,  191,  353205,  353225,  353252, 20, 27, 24, }, // America_Pacific
    {  356,  214,  353276,  353284,  353299,  8, 15, 12, }, // Apia
    {  356,  232,  353311,  353319,  353334,  8, 15, 12, }, // Arabian
    {  356,  240,  353346,  353359,  353379, 13, 20, 22, }, // Argentina
    {  356,  250,  353401,  353437,  353480, 36, 43, 45, }, // Argentina_Western
    {  356,  268,  353525,  353536,  353554, 11, 18, 20, }, // Armenia
    {  356,  286,  353574,  353595,  353623, 21, 28, 25, }, // Atlantic
    {  356,  295,  353648,  353667,  353693, 19, 26, 23, }, // Australia_Central
    {  356,  313,  353716,  353758,  353807, 42, 49, 46, }, // Australia_CentralWestern
    {  356,  338,  353853,  353889,  353932, 36, 43, 40, }, // Australia_Eastern
    {  356,  356,  353972,  354008,  354051, 36, 43, 40, }, // Australia_Western
    {  356,  374,  354091,  354105,  354126, 14, 21, 23, }, // Azerbaijan
    {  356,  385,  354149,  354159,  354176, 10, 17, 19, }, // Azores
    {  356,  397,  354195,  354209,  354230, 14, 21, 23, }, // Bangladesh
    {  356,  415,       0,  354253,       0,  0, 10,  0, }, // Bhutan
    {  356,  422,       0,  354263,       0,  0, 11,  0, }, // Bolivia
    {  356,  437,  354274,  354286,  354305, 12, 19, 21, }, // Brasilia
    {  356,  454,       0,  354326,       0,  0, 21,  0, }, // Brunei
    {  356,  461,  354347,  354361,  354382, 14, 21, 23, }, // Cape_Verde
    {  356,  478,       0,  354405,       0,  0, 19,  0, }, // Chamorro
    {  356,  487,  354424,  354435,  354453, 11, 18, 15, }, // Chatham
    {  356,  495,  354468,  354477,  354493,  9, 16, 18, }, // Chile
    {  356,  501,  354511,  354520,  354536,  9, 16, 13, }, // China
    {  356,  507,       0,  354549,       0,  0, 22,  0, }, // Christmas
    {  356,  517,       0,  354571,       0,  0, 17,  0, }, // Cocos
    {  356,  523,  354588,  354600,  354619, 12, 19, 21, }, // Colombia
    {  356,  532,  354640,  354656,  354679, 16, 23, 31, }, // Cook
    {  356,  537,  354710,  354718,  354733,  8, 15, 21, }, // Cuba
    {  356,  548,       0,  354754,       0,  0,  9,  0, }, // Davis
    {  356,  564,       0,  354763,       0,  0, 20,  0, }, // DumontDUrville
    {  356,  601,       0,  354783,       0,  0, 32,  0, }, // East_Timor
    {  356,  612,  354815,  354848,  354888, 33, 40, 42, }, // Easter
    {  356,  619,       0,  354930,       0,  0, 11,  0, }, // Ecuador
    {  356,  627,  354941,  354966,  354998, 25, 32, 34, }, // Europe_Central
    {  356,  642,  355032,  355065,  355105, 33, 40, 42, }, // Europe_Eastern
    {  356,  657,       0,    2413,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {  356,  680,  355147,  355180,  355220, 33, 40, 42, }, // Europe_Western
    {  356,  695,  355262,  355282,  355309, 20, 27, 29, }, // Falkland
    {  356,  704,  355338,  355346,  355361,  8, 15, 17, }, // Fiji
    {  356,  709,       0,  355378,       0,  0, 17,  0, }, // French_Guiana
    {  356,  723,       0,  355395,       0,  0, 31,  0, }, // French_Southern
    {  356,  746,       0,  355426,       0,  0, 13,  0, }, // Galapagos
    {  356,  756,       0,  355439,       0,  0, 11,  0, }, // Gambier
    {  356,  764,  355450,  355461,  355479, 11, 18, 20, }, // Georgia
    {  356,  772,       0,  355499,       0,  0, 19,  0, }, // Gilbert_Islands
    {  356,  788,       0,  355518,       0,  0, 27,  0, }, // GMT
    {  356,  830,  355545,  355581,  355624, 36, 43, 45, }, // Greenland_Eastern
    {  356,  848,  355669,  355705,  355748, 36, 43, 45, }, // Greenland_Western
    {  356,  871,       0,  355793,       0,  0, 15,  0, }, // Gulf
    {  356,  876,       0,  355808,       0,  0, 10,  0, }, // Guyana
    {  356,  883,  355818,  355837,  355863, 19, 26, 23, }, // Hawaii_Aleutian
    {  356,  899,  355886,  355899,  355919, 13, 20, 22, }, // Hong_Kong
    {  356,  909,  355941,  355949,  355964,  8, 15, 17, }, // Hovd
    {  356,  914,       0,  355981,       0,  0, 16,  0, }, // India
    {  356,  920,       0,  355997,       0,  0, 17,  0, }, // Indian_Ocean
    {  356,  933,       0,  356014,       0,  0, 13,  0, }, // Indochina
    {  356,  943,       0,  356027,       0,  0, 19,  0, }, // Indonesia_Central
    {  356,  961,       0,  356046,       0,  0, 36,  0, }, // Indonesia_Eastern
    {  356,  979,       0,  356082,       0,  0, 36,  0, }, // Indonesia_Western
    {  356,  997,  356118,  356126,  356141,  8, 15, 12, }, // Iran
    {  356, 1008,  356153,  356164,  356182, 11, 18, 20, }, // Irkutsk
    {  356, 1016,  356202,  356212,  356229, 10, 17, 14, }, // Israel
    {  356, 1023,  356243,  356252,  356268,  9, 16, 13, }, // Japan
    {  356, 1047,       0,  356281,       0,  0, 14,  0, }, // Kazakhstan
    {  356, 1058,       0,  356295,       0,  0, 37,  0, }, // Kazakhstan_Eastern
    {  356, 1077,       0,  356332,       0,  0, 37,  0, }, // Kazakhstan_Western
    {  356, 1106,  356369,  356378,  356394,  9, 16, 13, }, // Korea
    {  356, 1112,       0,  356407,       0,  0, 10,  0, }, // Kosrae
    {  356, 1119,  356417,  356432,  356454, 15, 22, 24, }, // Krasnoyarsk
    {  356, 1151,       0,  356478,       0,  0, 14,  0, }, // Kyrgystan
    {  356, 1175,       0,  356492,       0,  0, 16,  0, }, // Line_Islands
    {  356, 1188,  356508,  356521,  356541, 13, 20, 17, }, // Lord_Howe
    {  356, 1204,  356558,  356569,  356587, 11, 18, 20, }, // Magadan
    {  356, 1219,       0,  356607,       0,  0, 12,  0, }, // Malaysia
    {  356, 1228,       0,  356619,       0,  0, 12,  0, }, // Maldives
    {  356, 1237,       0,  356631,       0,  0, 13,  0, }, // Marquesas
    {  356, 1247,       0,  356644,       0,  0, 20,  0, }, // Marshall_Islands
    {  356, 1264,  356664,  356677,  356697, 13, 20, 22, }, // Mauritius
    {  356, 1274,       0,  356719,       0,  0, 10,  0, }, // Mawson
    {  356, 1281,  356729,  356748,  356774, 19, 26, 23, }, // Mexico_Pacific
    {  356, 1296,  356797,  356812,  356834, 15, 22, 24, }, // Mongolia
    {  356, 1305,  356858,  356868,  356885, 10, 17, 19, }, // Moscow
    {  356, 1312,       0,  356904,       0,  0, 11,  0, }, // Myanmar
    {  356, 1320,       0,  356915,       0,  0,  9,  0, }, // Nauru
    {  356, 1326,       0,  356924,       0,  0,  9,  0, }, // Nepal
    {  356, 1332,  356933,  356950,  356974, 17, 24, 26, }, // New_Caledonia
    {  356, 1346,  357000,  357015,  357037, 15, 22, 19, }, // New_Zealand
    {  356, 1358,  357056,  357072,  357095, 16, 23, 20, }, // Newfoundland
    {  356, 1371,       0,  357115,       0,  0,  8,  0, }, // Niue
    {  356, 1376,  357123,  357141,  357166, 18, 25, 27, }, // Norfolk
    {  356, 1384,  357193,  357216,  357246, 23, 30, 32, }, // Noronha
    {  356, 1406,  357278,  357293,  357315, 15, 22, 24, }, // Novosibirsk
    {  356, 1418,  357339,  357347,  357362,  8, 15, 17, }, // Omsk
    {  356, 1428,  357379,  357391,  357410, 12, 19, 21, }, // Pakistan
    {  356, 1437,       0,  357431,       0,  0,  9,  0, }, // Palau
    {  356, 1443,       0,  357440,       0,  0, 20,  0, }, // Papua_New_Guinea
    {  356, 1460,  357460,  357472,  357491, 12, 19, 21, }, // Paraguay
    {  356, 1469,  357512,  357520,  357535,  8, 15, 17, }, // Peru
    {  356, 1474,  357552,  357566,  357587, 14, 21, 23, }, // Philippines
    {  356, 1486,       0,  357610,       0,  0, 19,  0, }, // Phoenix_Islands
    {  356, 1502,  357629,  357654,  357686, 25, 32, 29, }, // Pierre_Miquelon
    {  356, 1518,       0,  357715,       0,  0, 12,  0, }, // Pitcairn
    {  356, 1527,       0,  357727,       0,  0, 10,  0, }, // Ponape
    {  356, 1534,       0,  357737,       0,  0, 13,  0, }, // Pyongyang
    {  356, 1554,       0,  357750,       0,  0, 11,  0, }, // Reunion
    {  356, 1562,       0,  357761,       0,  0, 11,  0, }, // Rothera
    {  356, 1570,  357772,  357784,  357803, 12, 19, 21, }, // Sakhalin
    {  356, 1596,  357824,  357833,  357849,  9, 16, 13, }, // Samoa
    {  356, 1602,       0,  357862,       0,  0, 14,  0, }, // Seychelles
    {  356, 1624,       0,  357876,       0,  0, 20,  0, }, // Singapore
    {  356, 1634,       0,  357896,       0,  0, 19,  0, }, // Solomon
    {  356, 1642,       0,  357915,       0,  0, 17,  0, }, // South_Georgia
    {  356, 1656,       0,  357932,       0,  0, 12,  0, }, // Suriname
    {  356, 1676,       0,  357944,       0,  0,  9,  0, }, // Syowa
    {  356, 1682,       0,  357953,       0,  0, 10,  0, }, // Tahiti
    {  356, 1689,  357963,  357973,  357990, 10, 17, 14, }, // Taipei
    {  356, 1696,       0,  358004,       0,  0, 14,  0, }, // Tajikistan
    {  356, 1724,       0,  358018,       0,  0, 11,  0, }, // Tokelau
    {  356, 1732,  358029,  358038,  358054,  9, 16, 18, }, // Tonga
    {  356, 1738,       0,  358072,       0,  0,  9,  0, }, // Truk
    {  356, 1750,  358081,  358095,  358116, 14, 21, 23, }, // Turkmenistan
    {  356, 1763,       0,  358139,       0,  0, 10,  0, }, // Tuvalu
    {  356, 1777,  358149,  358160,  358178, 11, 18, 20, }, // Uruguay
    {  356, 1792,  358198,  358210,  358229, 12, 19, 21, }, // Uzbekistan
    {  356, 1803,  358250,  358261,  358279, 11, 18, 20, }, // Vanuatu
    {  356, 1811,       0,  358299,       0,  0, 13,  0, }, // Venezuela
    {  356, 1821,  358312,  358327,  358349, 15, 22, 24, }, // Vladivostok
    {  356, 1833,  358373,  358386,  358406, 13, 20, 22, }, // Volgograd
    {  356, 1843,       0,  358428,       0,  0, 10,  0, }, // Vostok
    {  356, 1850,       0,  358438,       0,  0, 15,  0, }, // Wake
    {  356, 1855,       0,  358453,       0,  0, 19,  0, }, // Wallis
    {  356, 1862,  358472,  358483,  358501, 11, 18, 20, }, // Yakutsk
    {  356, 1870,  358521,  358538,  358562, 17, 24, 26, }, // Yekaterinburg
    {  356, 1892,       0,  358588,       0,  0,  9,  0, }, // Yukon
    {  358,    0,  358597,  358607,  358625, 10, 18, 22, }, // Acre Indonesian/Latin/Indonesia
    {  358,    5,       0,  358647,       0,  0, 16,  0, }, // Afghanistan
    {  358,   17,       0,  358663,       0,  0, 19,  0, }, // Africa_Central
    {  358,   32,       0,  358682,       0,  0, 18,  0, }, // Africa_Eastern
    {  358,   65,       0,  358700,       0,  0, 28,  0, }, // Africa_Southern
    {  358,   81,  358728,  358746,  358772, 18, 26, 30, }, // Africa_Western
    {  358,  107,  358802,  358814,  358834, 12, 20, 24, }, // Alaska
    {  358,  128,  358858,  358870,  358890, 12, 20, 24, }, // Almaty
    {  358,  135,  358914,  358926,  358946, 12, 20, 24, }, // Amazon
    {  358,  142,  358970,  358982,  359002, 12, 20, 24, }, // America_Central
    {  358,  158,  359026,  359037,  359056, 11, 19, 23, }, // America_Eastern
    {  358,  174,  359079,  359095,  359119, 16, 24, 28, }, // America_Mountain
    {  358,  191,  359147,  359160,  359181, 13, 21, 25, }, // America_Pacific
    {  358,  207,  359206,  359218,  359238, 12, 20, 24, }, // Anadyr
    {  358,  214,  359262,  359272,  359290, 10, 18, 22, }, // Apia
    {  358,  219,  359312,  359323,  359342, 11, 19, 23, }, // Aqtau
    {  358,  225,  359365,  359377,  359397, 12, 20, 24, }, // Aqtobe
    {  358,  232,  359421,  359431,  359449, 10, 18, 22, }, // Arabian
    {  358,  240,  359471,  359486,  359509, 15, 23, 27, }, // Argentina
    {  358,  250,  359536,  359564,  359600, 28, 36, 40, }, // Argentina_Western
    {  358,  268,  359640,  359653,  359674, 13, 21, 25, }, // Armenia
    {  358,  286,  359699,  359713,  359735, 14, 22, 26, }, // Atlantic
    {  358,  295,  359761,  359783,  359813, 22, 30, 34, }, // Australia_Central
    {  358,  313,  359847,  359875,  359911, 28, 36, 40, }, // Australia_CentralWestern
    {  358,  338,  359951,  359972,  360001, 21, 29, 33, }, // Australia_Eastern
    {  358,  356,  360034,  360055,  360084, 21, 29, 33, }, // Australia_Western
    {  358,  374,  360117,  360133,  360157, 16, 24, 28, }, // Azerbaijan
    {  358,  385,  360185,  360197,  360217, 12, 20, 24, }, // Azores
    {  358,  397,  360241,  360257,  360281, 16, 24, 28, }, // Bangladesh
    {  358,  415,       0,  360309,       0,  0, 12,  0, }, // Bhutan
    {  358,  422,       0,  360321,       0,  0, 13,  0, }, // Bolivia
    {  358,  437,  360334,  360346,  360366, 12, 20, 24, }, // Brasilia
    {  358,  454,       0,  360390,       0,  0, 23,  0, }, // Brunei
    {  358,  461,  360413,  360432,  360459, 19, 27, 31, }, // Cape_Verde
    {  358,  472,       0,  360490,       0,  0, 11,  0, }, // Casey
    {  358,  478,       0,  360501,       0,  0, 22,  0, }, // Chamorro
    {  358,  487,  360523,  360536,  360557, 13, 21, 25, }, // Chatham
    {  358,  495,  360582,  360592,  360610, 10, 18, 22, }, // Chile
    {  358,  501,  360632,  360646,  360668, 14, 22, 26, }, // China
    {  358,  507,       0,  360694,       0,  0, 17,  0, }, // Christmas
    {  358,  517,       0,  360711,       0,  0, 21,  0, }, // Cocos
    {  358,  523,  360732,  360746,  360768, 14, 22, 26, }, // Colombia
    {  358,  532,  360794,  360809,  360832, 15, 23, 34, }, // Cook
    {  358,  537,  360866,  360876,  360894, 10, 18, 22, }, // Cuba
    {  358,  548,       0,  360916,       0,  0, 11,  0, }, // Davis
    {  358,  564,       0,  360927,       0,  0, 22,  0, }, // DumontDUrville
    {  358,  601,       0,  360949,       0,  0, 17,  0, }, // East_Timor
    {  358,  612,  360966,  360984,  361010, 18, 26, 30, }, // Easter
    {  358,  619,       0,  361040,       0,  0, 13,  0, }, // Ecuador
    {  358,  627,  361053,  361071,  361097, 18, 26, 30, }, // Europe_Central
    {  358,  642,  361127,  361144,  361169, 17, 25, 29, }, // Europe_Eastern
    {  358,  657,       0,  361198,       0,  0, 22,  0, }, // Europe_Further_Eastern
    {  358,  680,  361220,  361237,  361262, 17, 25, 29, }, // Europe_Western
    {  358,  695,  361291,  361315,  361347, 24, 32, 36, }, // Falkland
    {  358,  704,  361383,  361393,  361411, 10, 18, 22, }, // Fiji
    {  358,  709,       0,  361433,       0,  0, 20,  0, }, // French_Guiana
    {  358,  723,       0,  361453,       0,  0, 44,  0, }, // French_Southern
    {  358,  746,       0,  361497,       0,  0, 15,  0, }, // Galapagos
    {  358,  756,       0,  361512,       0,  0, 13,  0, }, // Gambier
    {  358,  764,  361525,  361538,  361559, 13, 21, 25, }, // Georgia
    {  358,  772,       0,  361584,       0,  0, 18,  0, }, // Gilbert_Islands
    {  358,  788,       0,    2791,       0,  0, 19,  0, }, // GMT
    {  358,  830,  361602,  361623,  361652, 21, 29, 33, }, // Greenland_Eastern
    {  358,  848,  361685,  361706,  361735, 21, 29, 33, }, // Greenland_Western
    {  358,  866,       0,  361768,       0,  0, 10,  0, }, // Guam
    {  358,  871,       0,  361778,       0,  0, 19,  0, }, // Gulf
    {  358,  876,       0,  361433,       0,  0, 12,  0, }, // Guyana
    {  358,  883,  361797,  361818,  361847, 21, 29, 33, }, // Hawaii_Aleutian
    {  358,  899,  361880,  361895,  361918, 15, 23, 27, }, // Hong_Kong
    {  358,  909,  361945,  361955,  361973, 10, 18, 22, }, // Hovd
    {  358,  914,       0,  361995,       0,  0, 11,  0, }, // India
    {  358,  920,       0,  362006,       0,  0, 21,  0, }, // Indian_Ocean
    {  358,  933,       0,  362027,       0,  0, 15,  0, }, // Indochina
    {  358,  943,       0,  362042,       0,  0, 22,  0, }, // Indonesia_Central
    {  358,  961,       0,  362064,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  358,  979,       0,  362085,       0,  0, 21,  0, }, // Indonesia_Western
    {  358,  997,  362106,  362116,  362134, 10, 18, 22, }, // Iran
    {  358, 1008,  362156,  362169,  362190, 13, 21, 25, }, // Irkutsk
    {  358, 1016,  362215,  362227,  362247, 12, 20, 24, }, // Israel
    {  358, 1023,  362271,  362283,  362303, 12, 20, 24, }, // Japan
    {  358, 1029,  362327,  362357,  362395, 30, 38, 42, }, // Kamchatka
    {  358, 1047,       0,  362437,       0,  0, 16,  0, }, // Kazakhstan
    {  358, 1058,       0,  362453,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {  358, 1077,       0,  362475,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  358, 1106,  362497,  362508,  362527, 11, 19, 23, }, // Korea
    {  358, 1112,       0,  362550,       0,  0, 12,  0, }, // Kosrae
    {  358, 1119,  362562,  362579,  362604, 17, 25, 29, }, // Krasnoyarsk
    {  358, 1151,       0,  362633,       0,  0, 16,  0, }, // Kyrgystan
    {  358, 1161,       0,  362649,       0,  0, 11,  0, }, // Lanka
    {  358, 1175,       0,  362660,       0,  0, 15,  0, }, // Line_Islands
    {  358, 1188,  362675,  362690,  362713, 15, 23, 27, }, // Lord_Howe
    {  358, 1198,  362740,  362751,  362770, 11, 19, 23, }, // Macau
    {  358, 1204,  362793,  362806,  362827, 13, 21, 25, }, // Magadan
    {  358, 1219,       0,  362852,       0,  0, 14,  0, }, // Malaysia
    {  358, 1228,       0,  362866,       0,  0, 14,  0, }, // Maldives
    {  358, 1237,       0,  362880,       0,  0, 15,  0, }, // Marquesas
    {  358, 1247,       0,  362895,       0,  0, 19,  0, }, // Marshall_Islands
    {  358, 1264,  362914,  362929,  362952, 15, 23, 27, }, // Mauritius
    {  358, 1274,       0,  362979,       0,  0, 12,  0, }, // Mawson
    {  358, 1281,  362991,  363012,  363041, 21, 29, 33, }, // Mexico_Pacific
    {  358, 1296,  363074,  363090,  363114, 16, 24, 28, }, // Mongolia
    {  358, 1305,  363142,  363154,  363174, 12, 20, 24, }, // Moscow
    {  358, 1312,       0,  363198,       0,  0, 13,  0, }, // Myanmar
    {  358, 1320,       0,  363211,       0,  0, 11,  0, }, // Nauru
    {  358, 1326,       0,  363222,       0,  0, 11,  0, }, // Nepal
    {  358, 1332,  363233,  363253,  363281, 20, 28, 32, }, // New_Caledonia
    {  358, 1346,  363313,  363332,  363359, 19, 27, 31, }, // New_Zealand
    {  358, 1358,  363390,  363408,  363434, 18, 26, 30, }, // Newfoundland
    {  358, 1371,       0,  363464,       0,  0, 10,  0, }, // Niue
    {  358, 1376,  363474,  363493,  363520, 19, 27, 31, }, // Norfolk
    {  358, 1384,  363551,  363576,  363609, 25, 33, 37, }, // Noronha
    {  358, 1392,       0,  363646,       0,  0, 24,  0, }, // North_Mariana
    {  358, 1406,  363670,  363687,  363712, 17, 25, 29, }, // Novosibirsk
    {  358, 1418,  363741,  363751,  363769, 10, 18, 22, }, // Omsk
    {  358, 1428,  363791,  363805,  363827, 14, 22, 26, }, // Pakistan
    {  358, 1437,       0,  363853,       0,  0, 11,  0, }, // Palau
    {  358, 1443,       0,  363864,       0,  0, 18,  0, }, // Papua_New_Guinea
    {  358, 1460,  363882,  363896,  363918, 14, 22, 26, }, // Paraguay
    {  358, 1469,  363944,  363954,  363972, 10, 18, 22, }, // Peru
    {  358, 1474,  363994,  364008,  364030, 14, 22, 26, }, // Philippines
    {  358, 1486,       0,  364056,       0,  0, 23,  0, }, // Phoenix_Islands
    {  358, 1502,  364079,  364110,  364149, 31, 39, 43, }, // Pierre_Miquelon
    {  358, 1518,       0,  364192,       0,  0, 14,  0, }, // Pitcairn
    {  358, 1527,       0,  364206,       0,  0, 12,  0, }, // Ponape
    {  358, 1534,       0,  364218,       0,  0, 15,  0, }, // Pyongyang
    {  358, 1544,  364233,  364248,  364271, 15, 23, 27, }, // Qyzylorda
    {  358, 1554,       0,  364298,       0,  0, 13,  0, }, // Reunion
    {  358, 1562,       0,  364311,       0,  0, 13,  0, }, // Rothera
    {  358, 1570,  364324,  364338,  364360, 14, 22, 26, }, // Sakhalin
    {  358, 1579,  364386,  364398,  364418, 12, 20, 24, }, // Samara
    {  358, 1596,  364442,  364453,  364472, 11, 19, 23, }, // Samoa
    {  358, 1602,       0,  364495,       0,  0, 16,  0, }, // Seychelles
    {  358, 1624,       0,  364511,       0,  0, 23,  0, }, // Singapore
    {  358, 1634,       0,  364534,       0,  0, 23,  0, }, // Solomon
    {  358, 1642,       0,  364557,       0,  0, 21,  0, }, // South_Georgia
    {  358, 1656,       0,  364578,       0,  0, 14,  0, }, // Suriname
    {  358, 1676,       0,  364592,       0,  0, 11,  0, }, // Syowa
    {  358, 1682,       0,  364603,       0,  0, 12,  0, }, // Tahiti
    {  358, 1689,  364615,  364627,  364647, 12, 20, 24, }, // Taipei
    {  358, 1696,       0,  364671,       0,  0, 16,  0, }, // Tajikistan
    {  358, 1724,       0,  364687,       0,  0, 13,  0, }, // Tokelau
    {  358, 1732,  364700,  364711,  364730, 11, 19, 23, }, // Tonga
    {  358, 1738,       0,  364753,       0,  0, 11,  0, }, // Truk
    {  358, 1750,  364764,  364782,  364808, 18, 26, 30, }, // Turkmenistan
    {  358, 1763,       0,  364838,       0,  0, 12,  0, }, // Tuvalu
    {  358, 1777,  364850,  364863,  364884, 13, 21, 25, }, // Uruguay
    {  358, 1792,  364909,  364925,  364949, 16, 24, 28, }, // Uzbekistan
    {  358, 1803,  364977,  364990,  365011, 13, 21, 25, }, // Vanuatu
    {  358, 1811,       0,  365036,       0,  0, 15,  0, }, // Venezuela
    {  358, 1821,  365051,  365068,  365093, 17, 25, 29, }, // Vladivostok
    {  358, 1833,  365122,  365137,  365160, 15, 23, 27, }, // Volgograd
    {  358, 1843,       0,  365187,       0,  0, 12,  0, }, // Vostok
    {  358, 1850,       0,  365199,       0,  0, 20,  0, }, // Wake
    {  358, 1855,       0,  365219,       0,  0, 23,  0, }, // Wallis
    {  358, 1862,  365242,  365255,  365276, 13, 21, 25, }, // Yakutsk
    {  358, 1870,  365301,  365320,  365347, 19, 27, 31, }, // Yekaterinburg
    {  358, 1892,       0,  365378,       0,  0, 11,  0, }, // Yukon
    {  359,    5,       0,  365389,       0,  0, 19,  0, }, // Afghanistan Interlingua/Latin/world
    {  359,   17,       0,  365408,       0,  0, 22,  0, }, // Africa_Central
    {  359,   32,       0,  365430,       0,  0, 22,  0, }, // Africa_Eastern
    {  359,   65,       0,  365452,       0,  0, 31,  0, }, // Africa_Southern
    {  359,   81,  365483,  365506,  365538, 23, 32, 30, }, // Africa_Western
    {  359,  107,  365568,  365582,  365603, 14, 21, 21, }, // Alaska
    {  359,  135,  365624,  365640,  365665, 16, 25, 23, }, // Amazon
    {  359,  142,   42412,  365688,  365707, 12, 19, 19, }, // America_Central
    {  359,  158,  365726,  365738,  365757, 12, 19, 19, }, // America_Eastern
    {  359,  174,  365776,  365794,  365819, 18, 25, 25, }, // America_Mountain
    {  359,  191,  365844,  365856,  365875, 12, 19, 19, }, // America_Pacific
    {  359,  214,  290474,  365894,  365915, 12, 21, 19, }, // Apia
    {  359,  232,  365934,  365944,  365963, 10, 19, 17, }, // Arabian
    {  359,  240,  365980,  365997,  366023, 17, 26, 24, }, // Argentina
    {  359,  250,  366047,  366075,  366112, 28, 37, 35, }, // Argentina_Western
    {  359,  268,  290747,  366147,  366171, 15, 24, 22, }, // Armenia
    {  359,  286,  366193,  366206,  366226, 13, 20, 20, }, // Atlantic
    {  359,  295,  366246,  366271,  366305, 25, 34, 32, }, // Australia_Central
    {  359,  313,  366337,  366372,  366416, 35, 44, 42, }, // Australia_CentralWestern
    {  359,  338,  366458,  366484,  366519, 26, 35, 33, }, // Australia_Eastern
    {  359,  356,  366552,  366580,  366617, 28, 37, 35, }, // Australia_Western
    {  359,  374,  366652,  366672,  366701, 20, 29, 27, }, // Azerbaijan
    {  359,  385,  366728,  366743,  366765, 15, 22, 22, }, // Azores
    {  359,  397,  291329,  366787,  366814, 18, 27, 25, }, // Bangladesh
    {  359,  415,       0,  366839,       0,  0, 14,  0, }, // Bhutan
    {  359,  422,       0,  291414,       0,  0, 15,  0, }, // Bolivia
    {  359,  437,  291429,  366853,  366878, 16, 25, 23, }, // Brasilia
    {  359,  454,       0,  291495,       0,  0, 25,  0, }, // Brunei
    {  359,  461,  366901,  366919,  366946, 18, 27, 25, }, // Cape_Verde
    {  359,  478,       0,  366971,       0,  0, 25,  0, }, // Chamorro
    {  359,  487,  291614,  366996,  367020, 15, 24, 22, }, // Chatham
    {  359,  495,  291677,  367042,  367064, 13, 22, 20, }, // Chile
    {  359,  501,  367084,  367097,  367119, 13, 22, 20, }, // China
    {  359,  507,       0,  367139,       0,  0, 24,  0, }, // Christmas
    {  359,  517,       0,  367163,       0,  0, 22,  0, }, // Cocos
    {  359,  523,  291833,  367185,  367210, 16, 25, 23, }, // Colombia
    {  359,  532,  367233,  367254,  367284, 21, 30, 28, }, // Cook
    {  359,  537,  291980,  367312,  367331, 12, 19, 19, }, // Cuba
    {  359,  548,       0,  292034,       0,  0, 13,  0, }, // Davis
    {  359,  564,       0,  292047,       0,  0, 24,  0, }, // DumontDUrville
    {  359,  601,       0,  367350,       0,  0, 21,  0, }, // East_Timor
    {  359,  612,  367371,  367396,  367430, 25, 34, 32, }, // Easter
    {  359,  619,       0,  292174,       0,  0, 15,  0, }, // Ecuador
    {  359,  627,  367462,  367484,  367513, 22, 29, 29, }, // Europe_Central
    {  359,  642,  367542,  367565,  367595, 23, 30, 30, }, // Europe_Eastern
    {  359,  657,       0,  367625,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {  359,  680,  367654,  367679,  367711, 25, 32, 32, }, // Europe_Western
    {  359,  695,  367743,  367768,  367802, 25, 34, 32, }, // Falkland
    {  359,  704,  367834,  367846,  367867, 12, 21, 19, }, // Fiji
    {  359,  709,       0,  367886,       0,  0, 23,  0, }, // French_Guiana
    {  359,  723,       0,  367909,       0,  0, 36,  0, }, // French_Southern
    {  359,  746,       0,  367945,       0,  0, 18,  0, }, // Galapagos
    {  359,  756,       0,  292711,       0,  0, 15,  0, }, // Gambier
    {  359,  764,  367963,  367978,  368002, 15, 24, 22, }, // Georgia
    {  359,  772,       0,  368024,       0,  0, 24,  0, }, // Gilbert_Islands
    {  359,  788,       0,  368048,       0,  0, 23,  0, }, // GMT
    {  359,  830,  368071,  368099,  368134, 28, 35, 35, }, // Greenland_Eastern
    {  359,  848,  368169,  368199,  368236, 30, 37, 37, }, // Greenland_Western
    {  359,  871,       0,  368273,       0,  0, 23,  0, }, // Gulf
    {  359,  876,       0,  368296,       0,  0, 14,  0, }, // Guyana
    {  359,  883,  368310,  368335,  368367, 25, 32, 32, }, // Hawaii_Aleutian
    {  359,  899,  368399,  368415,  368440, 16, 25, 23, }, // Hong_Kong
    {  359,  909,  293241,  368463,  368484, 12, 21, 19, }, // Hovd
    {  359,  914,       0,  368503,       0,  0, 22,  0, }, // India
    {  359,  920,       0,  368525,       0,  0, 22,  0, }, // Indian_Ocean
    {  359,  933,       0,  293329,       0,  0, 17,  0, }, // Indochina
    {  359,  943,       0,  293346,       0,  0, 25,  0, }, // Indonesia_Central
    {  359,  961,       0,  368547,       0,  0, 25,  0, }, // Indonesia_Eastern
    {  359,  979,       0,  368572,       0,  0, 26,  0, }, // Indonesia_Western
    {  359,  997,  368598,  368610,  368631, 12, 21, 19, }, // Iran
    {  359, 1008,  293479,  368650,  368672, 15, 22, 22, }, // Irkutsk
    {  359, 1016,  293542,  368694,  368717, 14, 23, 21, }, // Israel
    {  359, 1023,  368738,  368751,  368773, 13, 22, 20, }, // Japan
    {  359, 1047,       0,  368793,       0,  0, 18,  0, }, // Kazakhstan
    {  359, 1058,       0,  368811,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  359, 1077,       0,  368837,       0,  0, 27,  0, }, // Kazakhstan_Western
    {  359, 1106,  293862,  368864,  368886, 13, 22, 20, }, // Korea
    {  359, 1112,       0,  293919,       0,  0, 14,  0, }, // Kosrae
    {  359, 1119,  368906,  368925,  368951, 19, 26, 26, }, // Krasnoyarsk
    {  359, 1151,       0,  368977,       0,  0, 20,  0, }, // Kyrgystan
    {  359, 1175,       0,  368997,       0,  0, 26,  0, }, // Line_Islands
    {  359, 1188,  294050,  369023,  369049, 17, 26, 24, }, // Lord_Howe
    {  359, 1204,  294119,  369073,  369095, 15, 22, 22, }, // Magadan
    {  359, 1219,       0,  369117,       0,  0, 16,  0, }, // Malaysia
    {  359, 1228,       0,  369133,       0,  0, 17,  0, }, // Maldives
    {  359, 1237,       0,  369150,       0,  0, 17,  0, }, // Marquesas
    {  359, 1247,       0,  369167,       0,  0, 25,  0, }, // Marshall_Islands
    {  359, 1264,  369192,  369208,  369233, 16, 25, 23, }, // Mauritius
    {  359, 1274,       0,  294322,       0,  0, 14,  0, }, // Mawson
    {  359, 1281,  369256,  369281,  369313, 25, 32, 32, }, // Mexico_Pacific
    {  359, 1296,  369345,  369363,  369390, 18, 27, 25, }, // Mongolia
    {  359, 1305,  294504,  369415,  369437, 15, 22, 22, }, // Moscow
    {  359, 1312,       0,  294567,       0,  0, 15,  0, }, // Myanmar
    {  359, 1320,       0,  294582,       0,  0, 13,  0, }, // Nauru
    {  359, 1326,       0,  294595,       0,  0, 13,  0, }, // Nepal
    {  359, 1332,  369459,  369481,  369512, 22, 31, 29, }, // New_Caledonia
    {  359, 1346,  369541,  369561,  369590, 20, 29, 27, }, // New_Zealand
    {  359, 1358,  369617,  369634,  369658, 17, 24, 24, }, // Newfoundland
    {  359, 1371,       0,  294845,       0,  0, 12,  0, }, // Niue
    {  359, 1376,  369682,  369705,  369737, 23, 32, 30, }, // Norfolk
    {  359, 1384,  294935,  369767,  369803, 27, 36, 34, }, // Noronha
    {  359, 1406,  295034,  369837,  369863, 19, 26, 26, }, // Novosibirsk
    {  359, 1418,  295109,  369889,  369908, 12, 19, 19, }, // Omsk
    {  359, 1428,  369927,  369943,  369968, 16, 25, 23, }, // Pakistan
    {  359, 1437,       0,  295232,       0,  0, 13,  0, }, // Palau
    {  359, 1443,       0,  369991,       0,  0, 25,  0, }, // Papua_New_Guinea
    {  359, 1460,  370016,  370032,  370057, 16, 25, 23, }, // Paraguay
    {  359, 1469,  370080,  370092,  370113, 12, 21, 19, }, // Peru
    {  359, 1474,  370132,  370152,  370181, 20, 29, 27, }, // Philippines
    {  359, 1486,       0,  370208,       0,  0, 24,  0, }, // Phoenix_Islands
    {  359, 1502,  370232,  370263,  370301, 31, 38, 38, }, // Pierre_Miquelon
    {  359, 1518,       0,  295593,       0,  0, 16,  0, }, // Pitcairn
    {  359, 1527,       0,  370339,       0,  0, 14,  0, }, // Ponape
    {  359, 1534,       0,   63381,       0,  0, 17,  0, }, // Pyongyang
    {  359, 1554,       0,  370353,       0,  0, 15,  0, }, // Reunion
    {  359, 1562,       0,  295639,       0,  0, 15,  0, }, // Rothera
    {  359, 1570,  370368,  370384,  370407, 16, 23, 23, }, // Sakhalin
    {  359, 1596,  295786,  370430,  370452, 13, 22, 20, }, // Samoa
    {  359, 1602,       0,  370472,       0,  0, 19,  0, }, // Seychelles
    {  359, 1624,       0,  370491,       0,  0, 26,  0, }, // Singapore
    {  359, 1634,       0,  370517,       0,  0, 24,  0, }, // Solomon
    {  359, 1642,       0,  370541,       0,  0, 23,  0, }, // South_Georgia
    {  359, 1656,       0,  295922,       0,  0, 16,  0, }, // Suriname
    {  359, 1676,       0,  295938,       0,  0, 13,  0, }, // Syowa
    {  359, 1682,       0,  370564,       0,  0, 14,  0, }, // Tahiti
    {  359, 1689,  295965,  370578,  370601, 14, 23, 21, }, // Taipei
    {  359, 1696,       0,  370622,       0,  0, 18,  0, }, // Tajikistan
    {  359, 1724,       0,  296044,       0,  0, 15,  0, }, // Tokelau
    {  359, 1732,  296059,  370640,  370662, 13, 22, 20, }, // Tonga
    {  359, 1738,       0,  296116,       0,  0, 13,  0, }, // Truk
    {  359, 1750,  370682,  370702,  370731, 20, 29, 27, }, // Turkmenistan
    {  359, 1763,       0,  296207,       0,  0, 14,  0, }, // Tuvalu
    {  359, 1777,  370758,  370773,  370797, 15, 24, 22, }, // Uruguay
    {  359, 1792,  370819,  370837,  370864, 18, 27, 25, }, // Uzbekistan
    {  359, 1803,  296356,  370889,  370913, 15, 24, 22, }, // Vanuatu
    {  359, 1811,       0,  296419,       0,  0, 17,  0, }, // Venezuela
    {  359, 1821,  296436,  370935,  370961, 19, 26, 26, }, // Vladivostok
    {  359, 1833,  296511,  370987,  371011, 17, 24, 24, }, // Volgograd
    {  359, 1843,       0,  296583,       0,  0, 14,  0, }, // Vostok
    {  359, 1850,       0,  371035,       0,  0, 20,  0, }, // Wake
    {  359, 1855,       0,  296614,       0,  0, 23,  0, }, // Wallis
    {  359, 1862,  371055,  371070,  371092, 15, 22, 22, }, // Yakutsk
    {  359, 1870,  371114,  371134,  371161, 20, 27, 27, }, // Yekaterinburg
    {  359, 1892,       0,  296781,       0,  0, 13,  0, }, // Yukon
    {  360,  107,  371188,  371202,  371225, 14, 23, 22, }, // Alaska Interlingue/Latin/Estonia
    {  360,  142,  371247,  371270,  371302, 23, 32, 31, }, // America_Central
    {  360,  158,  371333,  371357,  371390, 24, 33, 32, }, // America_Eastern
    {  360,  174,  371422,  371444,  371475, 22, 31, 30, }, // America_Mountain
    {  360,  191,  371505,  371528,  371560, 23, 32, 31, }, // America_Pacific
    {  360,  286,  371591,  371615,  371648, 24, 33, 32, }, // Atlantic
    {  360,  385,  371680,  371697,  371723, 17, 26, 25, }, // Azores
    {  360,  507,       0,  371748,       0,  0, 26,  0, }, // Christmas
    {  360,  517,       0,  371774,       0,  0, 24,  0, }, // Cocos
    {  360,  532,  371798,  371821,  371853, 23, 32, 31, }, // Cook
    {  360,  537,  371884,  371896,  371917, 12, 21, 20, }, // Cuba
    {  360,  627,  371937,  371959,  371990, 22, 31, 30, }, // Europe_Central
    {  360,  642,  372020,  372038,  372065, 18, 27, 26, }, // Europe_Eastern
    {  360,  680,  372091,  372110,  372138, 19, 28, 27, }, // Europe_Western
    {  360,  695,  372165,  372192,  372228, 27, 36, 35, }, // Falkland
    {  360,  772,       0,  372263,       0,  0, 26,  0, }, // Gilbert_Islands
    {  360,  788,       0,  372289,       0,  0, 27,  0, }, // GMT
    {  360,  830,  372316,  372345,  372383, 29, 38, 37, }, // Greenland_Eastern
    {  360,  848,  372420,  372451,  372491, 31, 40, 39, }, // Greenland_Western
    {  360,  883,  372530,  372555,  372589, 25, 34, 33, }, // Hawaii_Aleutian
    {  360, 1175,       0,  372622,       0,  0, 23,  0, }, // Line_Islands
    {  360, 1188,  372645,  372671,  372706, 26, 35, 34, }, // Lord_Howe
    {  360, 1237,       0,  372740,       0,  0, 20,  0, }, // Marquesas
    {  360, 1247,       0,  372760,       0,  0, 27,  0, }, // Marshall_Islands
    {  360, 1281,  372787,  372809,  372840, 22, 31, 30, }, // Mexico_Pacific
    {  360, 1358,  372870,  372892,  372923, 22, 31, 30, }, // Newfoundland
    {  360, 1376,  372953,  372977,  373010, 24, 33, 32, }, // Norfolk
    {  360, 1486,       0,  373042,       0,  0, 24,  0, }, // Phoenix_Islands
    {  360, 1634,       0,  373066,       0,  0, 26,  0, }, // Solomon
    {  360, 1850,       0,  373092,       0,  0, 21,  0, }, // Wake
    {  360, 1892,       0,  373113,       0,  0, 15,  0, }, // Yukon
    {  363,    0,  373128,  373135,  373155,  7, 20, 17, }, // Acre Irish/Latin/Ireland
    {  363,    5,       0,  373172,       0,  0, 19,  0, }, // Afghanistan
    {  363,   17,       0,  373191,       0,  0, 19,  0, }, // Africa_Central
    {  363,   32,       0,  373210,       0,  0, 23,  0, }, // Africa_Eastern
    {  363,   65,       0,  373233,       0,  0, 20,  0, }, // Africa_Southern
    {  363,   81,  373253,  373275,  373310, 22, 35, 32, }, // Africa_Western
    {  363,  107,  373342,  373351,  373373,  9, 22, 19, }, // Alaska
    {  363,  128,  373392,  373401,  373423,  9, 22, 19, }, // Almaty
    {  363,  135,  373442,  373457,  373485, 15, 28, 25, }, // Amazon
    {  363,  142,  373510,  373538,  373579, 28, 41, 38, }, // America_Central
    {  363,  158,  373617,  373649,  373694, 32, 45, 42, }, // America_Eastern
    {  363,  174,  373736,  373765,  373807, 29, 42, 39, }, // America_Mountain
    {  363,  191,  373846,  373865,  373897, 19, 32, 29, }, // America_Pacific
    {  363,  207,  373926,  373935,  373957,  9, 22, 19, }, // Anadyr
    {  363,  214,  373976,  373983,  374003,  7, 20, 17, }, // Apia
    {  363,  219,  374020,  374028,  374049,  8, 21, 18, }, // Aqtau
    {  363,  225,  374067,  374076,  374098,  9, 22, 19, }, // Aqtobe
    {  363,  232,  374117,  374130,  374156, 13, 26, 23, }, // Arabian
    {  363,  240,  374179,  374196,  374226, 17, 30, 27, }, // Argentina
    {  363,  250,  374253,  374281,  374322, 28, 41, 38, }, // Argentina_Western
    {  363,  268,  374360,  374375,  374403, 15, 28, 25, }, // Armenia
    {  363,  286,  374428,  374444,  374473, 16, 29, 26, }, // Atlantic
    {  363,  295,  374499,  374518,  374550, 19, 32, 29, }, // Australia_Central
    {  363,  313,  374579,  374607,  374648, 28, 41, 38, }, // Australia_CentralWestern
    {  363,  338,  374686,  374710,  374747, 24, 37, 34, }, // Australia_Eastern
    {  363,  356,  374781,  374804,  374840, 23, 36, 33, }, // Australia_Western
    {  363,  374,  374873,  374893,  374926, 20, 33, 30, }, // Azerbaijan
    {  363,  385,  374956,  374967,  374991, 11, 24, 21, }, // Azores
    {  363,  397,  375012,  375030,  375061, 18, 31, 28, }, // Bangladesh
    {  363,  415,       0,  375089,       0,  0, 13,  0, }, // Bhutan
    {  363,  422,       0,  375102,       0,  0, 13,  0, }, // Bolivia
    {  363,  437,  375115,  375127,  375152, 12, 25, 22, }, // Brasilia
    {  363,  454,       0,  375174,       0,  0, 20,  0, }, // Brunei
    {  363,  461,  375194,  375207,  375233, 13, 26, 23, }, // Cape_Verde
    {  363,  472,       0,  375256,       0,  0, 17,  0, }, // Casey
    {  363,  478,       0,  375273,       0,  0, 25,  0, }, // Chamorro
    {  363,  487,  375298,  375308,  375331, 10, 23, 20, }, // Chatham
    {  363,  495,  375351,  375361,  375384, 10, 23, 20, }, // Chile
    {  363,  501,  375404,  375414,  375437, 10, 23, 20, }, // China
    {  363,  507,       0,  375457,       0,  0, 19,  0, }, // Christmas
    {  363,  517,       0,  375476,       0,  0, 16,  0, }, // Cocos
    {  363,  523,  375492,  375505,  375531, 13, 26, 23, }, // Colombia
    {  363,  532,  375554,  375569,  375597, 15, 28, 31, }, // Cook
    {  363,  537,  375628,  375636,  375657,  8, 21, 18, }, // Cuba
    {  363,  548,       0,  375675,       0,  0,  8,  0, }, // Davis
    {  363,  564,       0,  375683,       0,  0, 19,  0, }, // DumontDUrville
    {  363,  601,       0,  375702,       0,  0, 16,  0, }, // East_Timor
    {  363,  612,  375718,  375736,  375767, 18, 31, 28, }, // Easter
    {  363,  619,       0,  375795,       0,  0, 11,  0, }, // Ecuador
    {  363,  627,  375806,  375822,  375851, 16, 29, 26, }, // Europe_Central
    {  363,  642,  375877,  375898,  375932, 21, 34, 31, }, // Europe_Eastern
    {  363,  657,       0,  375963,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {  363,  680,  375989,  376009,  376042, 20, 33, 30, }, // Europe_Western
    {  363,  695,  376072,  376093,  376127, 21, 34, 31, }, // Falkland
    {  363,  704,  376158,  376167,  376189,  9, 22, 19, }, // Fiji
    {  363,  709,       0,  376208,       0,  0, 20,  0, }, // French_Guiana
    {  363,  723,       0,  376228,       0,  0, 52,  0, }, // French_Southern
    {  363,  746,       0,  376280,       0,  0, 20,  0, }, // Galapagos
    {  363,  756,       0,  376300,       0,  0, 10,  0, }, // Gambier
    {  363,  764,  376310,  376324,  376351, 14, 27, 24, }, // Georgia
    {  363,  772,       0,  376375,       0,  0, 14,  0, }, // Gilbert_Islands
    {  363,  788,       0,  376389,       0,  0, 17,  0, }, // GMT
    {  363,  830,  376406,  376432,  376471, 26, 39, 36, }, // Greenland_Eastern
    {  363,  848,  376507,  376532,  376570, 25, 38, 35, }, // Greenland_Western
    {  363,  866,       0,  376605,       0,  0, 21,  0, }, // Guam
    {  363,  871,       0,  376626,       0,  0, 30,  0, }, // Gulf
    {  363,  876,       0,  376656,       0,  0, 12,  0, }, // Guyana
    {  363,  883,  376668,  376684,  376713, 16, 29, 26, }, // Hawaii_Aleutian
    {  363,  899,  376739,  376751,  376776, 12, 25, 22, }, // Hong_Kong
    {  363,  909,  376798,  376805,  376825,  7, 20, 17, }, // Hovd
    {  363,  914,       0,  376842,       0,  0, 25,  0, }, // India
    {  363,  920,       0,  376867,       0,  0, 21,  0, }, // Indian_Ocean
    {  363,  933,       0,  376888,       0,  0, 15,  0, }, // Indochina
    {  363,  943,       0,  376903,       0,  0, 20,  0, }, // Indonesia_Central
    {  363,  961,       0,  376923,       0,  0, 25,  0, }, // Indonesia_Eastern
    {  363,  979,       0,  376948,       0,  0, 24,  0, }, // Indonesia_Western
    {  363,  997,  376972,  376986,  377013, 14, 27, 24, }, // Iran
    {  363, 1008,  377037,  377047,  377070, 10, 23, 20, }, // Irkutsk
    {  363, 1016,  377090,  377100,  377123, 10, 23, 20, }, // Israel
    {  363, 1023,  377143,  377157,  377184, 14, 27, 24, }, // Japan
    {  363, 1029,  377208,  377236,  377277, 28, 41, 38, }, // Kamchatka
    {  363, 1047,       0,  377315,       0,  0, 17,  0, }, // Kazakhstan
    {  363, 1058,       0,  377332,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  363, 1077,       0,  377358,       0,  0, 25,  0, }, // Kazakhstan_Western
    {  363, 1106,  377383,  377394,  377418, 11, 24, 21, }, // Korea
    {  363, 1112,       0,  377439,       0,  0,  9,  0, }, // Kosrae
    {  363, 1119,  377448,  377462,  377489, 14, 27, 24, }, // Krasnoyarsk
    {  363, 1151,       0,  377513,       0,  0, 18,  0, }, // Kyrgystan
    {  363, 1161,       0,  377531,       0,  0, 13,  0, }, // Lanka
    {  363, 1175,       0,  377544,       0,  0, 18,  0, }, // Line_Islands
    {  363, 1188,  377562,  377574,  377599, 12, 25, 22, }, // Lord_Howe
    {  363, 1198,  377621,  377630,  377652,  9, 22, 19, }, // Macau
    {  363, 1204,  377671,  377682,  377706, 11, 24, 21, }, // Magadan
    {  363, 1219,       0,  377727,       0,  0, 15,  0, }, // Malaysia
    {  363, 1228,       0,  377742,       0,  0, 20,  0, }, // Maldives
    {  363, 1237,       0,  377762,       0,  0, 23,  0, }, // Marquesas
    {  363, 1247,       0,  377785,       0,  0, 19,  0, }, // Marshall_Islands
    {  363, 1264,  377804,  377821,  377851, 17, 30, 27, }, // Mauritius
    {  363, 1274,       0,  377878,       0,  0,  9,  0, }, // Mawson
    {  363, 1281,  377887,  377918,  377962, 31, 44, 41, }, // Mexico_Pacific
    {  363, 1296,  378003,  378015,  378040, 12, 25, 22, }, // Mongolia
    {  363, 1305,  378062,  378071,  378093,  9, 22, 19, }, // Moscow
    {  363, 1312,       0,  378112,       0,  0, 11,  0, }, // Myanmar
    {  363, 1320,       0,  378123,       0,  0,  7,  0, }, // Nauru
    {  363, 1326,       0,  378130,       0,  0, 10,  0, }, // Nepal
    {  363, 1332,  378140,  378160,  378193, 20, 33, 30, }, // New_Caledonia
    {  363, 1346,  378223,  378243,  378276, 20, 33, 30, }, // New_Zealand
    {  363, 1358,  378306,  378324,  378355, 18, 31, 28, }, // Newfoundland
    {  363, 1371,       0,  378383,       0,  0,  7,  0, }, // Niue
    {  363, 1376,  378390,  378407,  378437, 17, 30, 27, }, // Norfolk
    {  363, 1384,  378464,  378486,  378521, 22, 35, 32, }, // Noronha
    {  363, 1392,       0,  378553,       0,  0, 32,  0, }, // North_Mariana
    {  363, 1406,  378585,  378599,  378626, 14, 27, 24, }, // Novosibirsk
    {  363, 1418,  378650,  378657,  378677,  7, 20, 17, }, // Omsk
    {  363, 1428,  378694,  378710,  378739, 16, 29, 26, }, // Pakistan
    {  363, 1437,       0,  378765,       0,  0, 16,  0, }, // Palau
    {  363, 1443,       0,  378781,       0,  0, 20,  0, }, // Papua_New_Guinea
    {  363, 1460,  378801,  378812,  378836, 11, 24, 21, }, // Paraguay
    {  363, 1469,  378857,  378867,  378890, 10, 23, 20, }, // Peru
    {  363, 1474,  378910,  378935,  378973, 25, 38, 35, }, // Philippines
    {  363, 1486,       0,  379008,       0,  0, 22,  0, }, // Phoenix_Islands
    {  363, 1502,  379030,  379057,  379097, 27, 40, 37, }, // Pierre_Miquelon
    {  363, 1518,       0,  379134,       0,  0, 11,  0, }, // Pitcairn
    {  363, 1527,       0,  379145,       0,  0, 11,  0, }, // Ponape
    {  363, 1534,       0,  379156,       0,  0, 12,  0, }, // Pyongyang
    {  363, 1544,  379168,  379180,  379205, 12, 25, 22, }, // Qyzylorda
    {  363, 1554,       0,  379227,       0,  0, 10,  0, }, // Reunion
    {  363, 1562,       0,  379237,       0,  0, 10,  0, }, // Rothera
    {  363, 1570,  379247,  379259,  379284, 12, 25, 22, }, // Sakhalin
    {  363, 1579,  379306,  379316,  379339, 10, 23, 20, }, // Samara
    {  363, 1596,  379359,  379367,  379388,  8, 21, 18, }, // Samoa
    {  363, 1602,       0,  379406,       0,  0, 13,  0, }, // Seychelles
    {  363, 1624,       0,  379419,       0,  0, 26,  0, }, // Singapore
    {  363, 1634,       0,  379445,       0,  0, 19,  0, }, // Solomon
    {  363, 1642,       0,  379464,       0,  0, 20,  0, }, // South_Georgia
    {  363, 1656,       0,  379484,       0,  0, 11,  0, }, // Suriname
    {  363, 1676,       0,  379495,       0,  0,  8,  0, }, // Syowa
    {  363, 1682,       0,  379503,       0,  0, 11,  0, }, // Tahiti
    {  363, 1689,  379514,  379523,  379545,  9, 22, 19, }, // Taipei
    {  363, 1696,       0,  379564,       0,  0, 21,  0, }, // Tajikistan
    {  363, 1724,       0,  379585,       0,  0, 17,  0, }, // Tokelau
    {  363, 1732,  379602,  379611,  379633,  9, 22, 19, }, // Tonga
    {  363, 1738,       0,  379652,       0,  0,  8,  0, }, // Truk
    {  363, 1750,  379660,  379682,  379717, 22, 35, 32, }, // Turkmenistan
    {  363, 1763,       0,  379749,       0,  0, 10,  0, }, // Tuvalu
    {  363, 1777,  379759,  379768,  379790,  9, 22, 19, }, // Uruguay
    {  363, 1792,  379809,  379831,  379866, 22, 35, 32, }, // Uzbekistan
    {  363, 1803,  379898,  379908,  379931, 10, 23, 20, }, // Vanuatu
    {  363, 1811,       0,  379951,       0,  0, 13,  0, }, // Venezuela
    {  363, 1821,  379964,  379978,  380005, 14, 27, 24, }, // Vladivostok
    {  363, 1833,  380029,  380041,  380066, 12, 25, 22, }, // Volgograd
    {  363, 1843,       0,  380088,       0,  0,  9,  0, }, // Vostok
    {  363, 1850,       0,  380097,       0,  0, 14,  0, }, // Wake
    {  363, 1855,       0,  380111,       0,  0, 21,  0, }, // Wallis
    {  363, 1862,  380132,  380142,  380165, 10, 23, 20, }, // Yakutsk
    {  363, 1870,  380185,  380201,  380230, 16, 29, 26, }, // Yekaterinburg
    {  363, 1892,       0,  380256,       0,  0,  8,  0, }, // Yukon
    {  365,    5,       0,  380264,       0,  0, 20,  0, }, // Afghanistan Italian/Latin/Italy
    {  365,   17,       0,  380284,       0,  0, 24,  0, }, // Africa_Central
    {  365,   32,       0,  380308,       0,  0, 25,  0, }, // Africa_Eastern
    {  365,   65,       0,  380333,       0,  0, 27,  0, }, // Africa_Southern
    {  365,   81,  380360,  380387,  380423, 27, 36, 34, }, // Africa_Western
    {  365,  107,  380457,  380472,  380496, 15, 24, 22, }, // Alaska
    {  365,  135,  380518,  380536,  380563, 18, 27, 25, }, // Amazon
    {  365,  142,  380588,  380604,  380629, 16, 25, 23, }, // America_Central
    {  365,  158,  380652,  380669,  380695, 17, 26, 24, }, // America_Eastern
    {  365,  174,  380719,  380744,  380778, 25, 34, 32, }, // America_Mountain
    {  365,  191,  380810,  380830,  380859, 20, 29, 27, }, // America_Pacific
    {  365,  207,  380886,  380899,  380921, 13, 22, 20, }, // Anadyr
    {  365,  214,  380941,  380952,  380972, 11, 20, 18, }, // Apia
    {  365,  232,  380990,  380999,  381017,  9, 18, 16, }, // Arabian
    {  365,  240,  381033,  381051,  381078, 18, 27, 25, }, // Argentina
    {  365,  250,  381103,  381133,  381172, 30, 39, 37, }, // Argentina_Western
    {  365,  268,  381209,  381225,  381250, 16, 25, 23, }, // Armenia
    {  365,  286,  381273,  381291,  381318, 18, 27, 25, }, // Atlantic
    {  365,  295,  381343,  381370,  381406, 27, 36, 34, }, // Australia_Central
    {  365,  313,  381440,  381475,  381519, 35, 44, 42, }, // Australia_CentralWestern
    {  365,  338,  381561,  381589,  381626, 28, 37, 35, }, // Australia_Eastern
    {  365,  356,  381661,  381691,  381730, 30, 39, 37, }, // Australia_Western
    {  365,  374,  381767,  381787,  381816, 20, 29, 27, }, // Azerbaijan
    {  365,  385,  381843,  381860,  381886, 17, 26, 24, }, // Azores
    {  365,  397,  381910,  381928,  381955, 18, 27, 25, }, // Bangladesh
    {  365,  415,       0,  381980,       0,  0, 14,  0, }, // Bhutan
    {  365,  422,       0,  381994,       0,  0, 17,  0, }, // Bolivia
    {  365,  437,  382011,  382026,  382050, 15, 24, 22, }, // Brasilia
    {  365,  454,       0,  382072,       0,  0, 25,  0, }, // Brunei
    {  365,  461,  382097,  382114,  382140, 17, 26, 24, }, // Cape_Verde
    {  365,  478,       0,  382164,       0,  0, 15,  0, }, // Chamorro
    {  365,  487,  382179,  382196,  382222, 17, 26, 24, }, // Chatham
    {  365,  495,  382246,  382258,  382279, 12, 21, 19, }, // Chile
    {  365,  501,  382298,  382312,  382335, 14, 23, 21, }, // China
    {  365,  507,       0,  382356,       0,  0, 24,  0, }, // Christmas
    {  365,  517,       0,  382380,       0,  0, 21,  0, }, // Cocos
    {  365,  523,  382401,  382419,  382446, 18, 27, 25, }, // Colombia
    {  365,  532,  382471,  382491,  382520, 20, 29, 33, }, // Cook
    {  365,  537,  382553,  382564,  382584, 11, 20, 18, }, // Cuba
    {  365,  548,       0,  382602,       0,  0, 12,  0, }, // Davis
    {  365,  564,       0,  382614,       0,  0, 23,  0, }, // DumontDUrville
    {  365,  601,       0,  382637,       0,  0, 16,  0, }, // East_Timor
    {  365,  612,  382653,  382677,  382710, 24, 33, 31, }, // Easter
    {  365,  619,       0,  382741,       0,  0, 16,  0, }, // Ecuador
    {  365,  627,  382757,  382781,  382814, 24, 33, 31, }, // Europe_Central
    {  365,  642,  382845,  382870,  382904, 25, 34, 32, }, // Europe_Eastern
    {  365,  657,       0,  382936,       0,  0, 39,  0, }, // Europe_Further_Eastern
    {  365,  680,  382975,  383002,  383038, 27, 36, 34, }, // Europe_Western
    {  365,  695,  383072,  383096,  383129, 24, 33, 31, }, // Falkland
    {  365,  704,  383160,  383174,  383197, 14, 23, 21, }, // Fiji
    {  365,  709,       0,  383218,       0,  0, 25,  0, }, // French_Guiana
    {  365,  723,       0,  383243,       0,  0, 46,  0, }, // French_Southern
    {  365,  746,       0,  383289,       0,  0, 19,  0, }, // Galapagos
    {  365,  756,       0,  383308,       0,  0, 14,  0, }, // Gambier
    {  365,  764,  383322,  383339,  383365, 17, 26, 24, }, // Georgia
    {  365,  772,       0,  383389,       0,  0, 23,  0, }, // Gilbert_Islands
    {  365,  788,       0,  383412,       0,  0, 30,  0, }, // GMT
    {  365,  830,  383442,  383473,  383513, 31, 40, 38, }, // Greenland_Eastern
    {  365,  848,  383551,  383584,  383626, 33, 42, 40, }, // Greenland_Western
    {  365,  871,       0,  383666,       0,  0, 13,  0, }, // Gulf
    {  365,  876,       0,  383679,       0,  0, 16,  0, }, // Guyana
    {  365,  883,  383695,  383726,  383766, 31, 40, 38, }, // Hawaii_Aleutian
    {  365,  899,  383804,  383820,  383845, 16, 25, 23, }, // Hong_Kong
    {  365,  909,  383868,  383879,  383899, 11, 20, 18, }, // Hovd
    {  365,  914,       0,  383917,       0,  0, 23,  0, }, // India
    {  365,  920,       0,  383940,       0,  0, 23,  0, }, // Indian_Ocean
    {  365,  933,       0,  383963,       0,  0, 17,  0, }, // Indochina
    {  365,  943,       0,  383980,       0,  0, 27,  0, }, // Indonesia_Central
    {  365,  961,       0,  384007,       0,  0, 28,  0, }, // Indonesia_Eastern
    {  365,  979,       0,  384035,       0,  0, 30,  0, }, // Indonesia_Western
    {  365,  997,  384065,  384078,  384100, 13, 22, 20, }, // Iran
    {  365, 1008,  384120,  384134,  384157, 14, 23, 21, }, // Irkutsk
    {  365, 1016,  384178,  384192,  384215, 14, 23, 21, }, // Israel
    {  365, 1023,  384236,  384252,  384277, 16, 25, 23, }, // Japan
    {  365, 1029,  384300,  384331,  384371, 31, 40, 38, }, // Kamchatka
    {  365, 1047,       0,  384409,       0,  0, 18,  0, }, // Kazakhstan
    {  365, 1058,       0,  384427,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  365, 1077,       0,  384455,       0,  0, 30,  0, }, // Kazakhstan_Western
    {  365, 1106,  384485,  384496,  384516, 11, 20, 18, }, // Korea
    {  365, 1112,       0,  384534,       0,  0, 14,  0, }, // Kosrae
    {  365, 1119,  384548,  384566,  384593, 18, 27, 25, }, // Krasnoyarsk
    {  365, 1151,       0,  384618,       0,  0, 20,  0, }, // Kyrgystan
    {  365, 1175,       0,  384638,       0,  0, 29,  0, }, // Line_Islands
    {  365, 1188,  384667,  384683,  384708, 16, 25, 23, }, // Lord_Howe
    {  365, 1198,  384731,  384743,  384764, 12, 21, 19, }, // Macau
    {  365, 1204,  384783,  384797,  384820, 14, 23, 21, }, // Magadan
    {  365, 1219,       0,  384841,       0,  0, 17,  0, }, // Malaysia
    {  365, 1228,       0,  384858,       0,  0, 17,  0, }, // Maldives
    {  365, 1237,       0,  384875,       0,  0, 18,  0, }, // Marquesas
    {  365, 1247,       0,  384893,       0,  0, 24,  0, }, // Marshall_Islands
    {  365, 1264,  384917,  384936,  384964, 19, 28, 26, }, // Mauritius
    {  365, 1274,       0,  384990,       0,  0, 13,  0, }, // Mawson
    {  365, 1281,  385003,  385029,  385064, 26, 35, 33, }, // Mexico_Pacific
    {  365, 1296,  385097,  385114,  385140, 17, 26, 24, }, // Mongolia
    {  365, 1305,  385164,  385176,  385197, 12, 21, 19, }, // Moscow
    {  365, 1312,       0,  385216,       0,  0, 18,  0, }, // Myanmar
    {  365, 1320,       0,  385234,       0,  0, 12,  0, }, // Nauru
    {  365, 1326,       0,  385246,       0,  0, 13,  0, }, // Nepal
    {  365, 1332,  385259,  385284,  385318, 25, 34, 32, }, // New_Caledonia
    {  365, 1346,  385350,  385373,  385405, 23, 32, 30, }, // New_Zealand
    {  365, 1358,  385435,  385451,  385476, 16, 25, 23, }, // Newfoundland
    {  365, 1371,       0,  385499,       0,  0, 11,  0, }, // Niue
    {  365, 1376,  385510,  385533,  385565, 23, 32, 30, }, // Norfolk
    {  365, 1384,  385595,  385621,  385656, 26, 35, 33, }, // Noronha
    {  365, 1406,  385689,  385707,  385734, 18, 27, 25, }, // Novosibirsk
    {  365, 1418,  385759,  385770,  385790, 11, 20, 18, }, // Omsk
    {  365, 1428,  385808,  385824,  385849, 16, 25, 23, }, // Pakistan
    {  365, 1437,       0,  385872,       0,  0, 12,  0, }, // Palau
    {  365, 1443,       0,  385884,       0,  0, 28,  0, }, // Papua_New_Guinea
    {  365, 1460,  385912,  385928,  385953, 16, 25, 23, }, // Paraguay
    {  365, 1469,  385976,  385988,  386009, 12, 21, 19, }, // Peru
    {  365, 1474,  386028,  386047,  386075, 19, 28, 26, }, // Philippines
    {  365, 1486,       0,  386101,       0,  0, 28,  0, }, // Phoenix_Islands
    {  365, 1502,  386129,  386159,  386198, 30, 39, 37, }, // Pierre_Miquelon
    {  365, 1518,       0,  386235,       0,  0, 18,  0, }, // Pitcairn
    {  365, 1527,       0,  386253,       0,  0, 14,  0, }, // Ponape
    {  365, 1534,       0,  386267,       0,  0, 16,  0, }, // Pyongyang
    {  365, 1554,       0,  386283,       0,  0, 15,  0, }, // Reunion
    {  365, 1562,       0,  386298,       0,  0, 14,  0, }, // Rothera
    {  365, 1570,  386312,  386327,  386351, 15, 24, 22, }, // Sakhalin
    {  365, 1579,  386373,  386386,  386408, 13, 22, 20, }, // Samara
    {  365, 1596,  386428,  386440,  386461, 12, 21, 19, }, // Samoa
    {  365, 1602,       0,  386480,       0,  0, 20,  0, }, // Seychelles
    {  365, 1624,       0,  386500,       0,  0, 16,  0, }, // Singapore
    {  365, 1634,       0,  386516,       0,  0, 24,  0, }, // Solomon
    {  365, 1642,       0,  386540,       0,  0, 25,  0, }, // South_Georgia
    {  365, 1656,       0,  386565,       0,  0, 16,  0, }, // Suriname
    {  365, 1676,       0,  386581,       0,  0, 12,  0, }, // Syowa
    {  365, 1682,       0,  386593,       0,  0, 13,  0, }, // Tahiti
    {  365, 1689,  386606,  386619,  386641, 13, 22, 20, }, // Taipei
    {  365, 1696,       0,  386661,       0,  0, 18,  0, }, // Tajikistan
    {  365, 1724,       0,  386679,       0,  0, 14,  0, }, // Tokelau
    {  365, 1732,  386693,  386705,  386726, 12, 21, 19, }, // Tonga
    {  365, 1738,       0,  386745,       0,  0, 13,  0, }, // Truk
    {  365, 1750,  386758,  386778,  386807, 20, 29, 27, }, // Turkmenistan
    {  365, 1763,       0,  386834,       0,  0, 13,  0, }, // Tuvalu
    {  365, 1777,  386847,  386863,  386888, 16, 25, 23, }, // Uruguay
    {  365, 1792,  386911,  386930,  386958, 19, 28, 26, }, // Uzbekistan
    {  365, 1803,  386984,  386999,  387023, 15, 24, 22, }, // Vanuatu
    {  365, 1811,       0,  387045,       0,  0, 17,  0, }, // Venezuela
    {  365, 1821,  387062,  387080,  387107, 18, 27, 25, }, // Vladivostok
    {  365, 1833,  387132,  387148,  387173, 16, 25, 23, }, // Volgograd
    {  365, 1843,       0,  387196,       0,  0, 13,  0, }, // Vostok
    {  365, 1850,       0,  387209,       0,  0, 22,  0, }, // Wake
    {  365, 1855,       0,  387231,       0,  0, 22,  0, }, // Wallis
    {  365, 1862,  387253,  387267,  387290, 14, 23, 21, }, // Yakutsk
    {  365, 1870,  387311,  387330,  387358, 19, 28, 26, }, // Yekaterinburg
    {  365, 1892,       0,  387384,       0,  0, 15,  0, }, // Yukon
    {  369,    0,  387399,  387404,  387410,  5,  6,  6, }, // Acre Japanese/Japanese/Japan
    {  369,    5,       0,  387416,       0,  0,  9,  0, }, // Afghanistan
    {  369,   17,       0,  387425,       0,  0,  8,  0, }, // Africa_Central
    {  369,   32,       0,  387433,       0,  0,  7,  0, }, // Africa_Eastern
    {  369,   65,       0,  387440,       0,  0,  8,  0, }, // Africa_Southern
    {  369,   81,  387448,  387455,  387463,  7,  8,  8, }, // Africa_Western
    {  369,  107,  387471,  387477,  387484,  6,  7,  7, }, // Alaska
    {  369,  128,  387491,  387498,  387506,  7,  8,  8, }, // Almaty
    {  369,  135,  387514,  387520,  387527,  6,  7,  7, }, // Amazon
    {  369,  142,  387534,  387542,  387551,  8,  9,  9, }, // America_Central
    {  369,  158,  387560,  387568,  387577,  8,  9,  9, }, // America_Eastern
    {  369,  174,  387586,  387594,  387603,  8,  9,  9, }, // America_Mountain
    {  369,  191,  387612,  387621,  387631,  9, 10, 10, }, // America_Pacific
    {  369,  207,  387641,  387648,  387656,  7,  8,  8, }, // Anadyr
    {  369,  214,  387664,  387669,  387675,  5,  6,  6, }, // Apia
    {  369,  219,  387681,  387687,  387694,  6,  7,  7, }, // Aqtau
    {  369,  225,  387701,  387707,  387714,  6,  7,  7, }, // Aqtobe
    {  369,  232,  387721,  387727,  387734,  6,  7,  7, }, // Arabian
    {  369,  240,  387741,  387749,  387758,  8,  9,  9, }, // Argentina
    {  369,  250,  387767,  387777,  387788, 10, 11, 11, }, // Argentina_Western
    {  369,  268,  387799,  387806,  387814,  7,  8,  8, }, // Armenia
    {  369,  286,  133299,  133304,  387822,  5,  6,  6, }, // Atlantic
    {  369,  295,  387828,  387839,  387851, 11, 12, 12, }, // Australia_Central
    {  369,  313,  387863,  387875,  387888, 12, 13, 13, }, // Australia_CentralWestern
    {  369,  338,  387901,  387912,  387924, 11, 12, 12, }, // Australia_Eastern
    {  369,  356,  387936,  387947,  387959, 11, 12, 12, }, // Australia_Western
    {  369,  374,  387971,  387981,  387992, 10, 11, 11, }, // Azerbaijan
    {  369,  385,  388003,  388009,  388016,  6,  7,  7, }, // Azores
    {  369,  397,  388023,  388032,  388042,  9, 10, 10, }, // Bangladesh
    {  369,  415,       0,  388052,       0,  0,  6,  0, }, // Bhutan
    {  369,  422,       0,  388058,       0,  0,  6,  0, }, // Bolivia
    {  369,  437,  388064,  388071,  388079,  7,  8,  8, }, // Brasilia
    {  369,  454,       0,  388087,       0,  0, 13,  0, }, // Brunei
    {  369,  461,  388100,  388108,  388117,  8,  9,  9, }, // Cape_Verde
    {  369,  472,       0,  388126,       0,  0,  8,  0, }, // Casey
    {  369,  478,       0,  388134,       0,  0,  6,  0, }, // Chamorro
    {  369,  487,  388140,  388146,  388153,  6,  7,  7, }, // Chatham
    {  369,  495,  388160,  388164,  388169,  4,  5,  5, }, // Chile
    {  369,  501,  388174,  388178,  388183,  4,  5,  5, }, // China
    {  369,  507,       0,  388188,       0,  0,  8,  0, }, // Christmas
    {  369,  517,       0,  388196,       0,  0,  7,  0, }, // Cocos
    {  369,  523,  388203,  388210,  388218,  7,  8,  8, }, // Colombia
    {  369,  532,  388226,  388233,  388241,  7,  8,  8, }, // Cook
    {  369,  537,  388249,  388255,  388262,  6,  7,  7, }, // Cuba
    {  369,  548,       0,  388269,       0,  0,  8,  0, }, // Davis
    {  369,  564,       0,  388277,       0,  0, 15,  0, }, // DumontDUrville
    {  369,  601,       0,  388292,       0,  0,  8,  0, }, // East_Timor
    {  369,  612,  388300,  388308,  388317,  8,  9,  9, }, // Easter
    {  369,  619,       0,  388326,       0,  0,  7,  0, }, // Ecuador
    {  369,  627,  388333,  388342,  388352,  9, 10, 10, }, // Europe_Central
    {  369,  642,  388362,  388370,  388379,  8,  9,  9, }, // Europe_Eastern
    {  369,  657,       0,  388388,       0,  0,  9,  0, }, // Europe_Further_Eastern
    {  369,  680,  388397,  388405,  388414,  8,  9,  9, }, // Europe_Western
    {  369,  695,  388423,  388434,  388446, 11, 12, 12, }, // Falkland
    {  369,  704,  388458,  388464,  388471,  6,  7,  7, }, // Fiji
    {  369,  709,       0,  388478,       0,  0,  7,  0, }, // French_Guiana
    {  369,  723,       0,  388485,       0,  0,  8,  0, }, // French_Southern
    {  369,  746,       0,  388493,       0,  0,  7,  0, }, // Galapagos
    {  369,  756,       0,  388500,       0,  0,  8,  0, }, // Gambier
    {  369,  764,  388508,  388515,  388523,  7,  8,  8, }, // Georgia
    {  369,  772,       0,  388531,       0,  0,  9,  0, }, // Gilbert_Islands
    {  369,  788,       0,  388540,       0,  0,  8,  0, }, // GMT
    {  369,  830,  388548,  388559,  388571, 11, 12, 12, }, // Greenland_Eastern
    {  369,  848,  388583,  388594,  388606, 11, 12, 12, }, // Greenland_Western
    {  369,  866,       0,  388618,       0,  0,  5,  0, }, // Guam
    {  369,  871,       0,  388623,       0,  0,  5,  0, }, // Gulf
    {  369,  876,       0,  388628,       0,  0,  6,  0, }, // Guyana
    {  369,  883,  388634,  388647,  388661, 13, 14, 14, }, // Hawaii_Aleutian
    {  369,  899,  133973,  133977,  388675,  4,  5,  5, }, // Hong_Kong
    {  369,  909,  388680,  388685,  388691,  5,  6,  6, }, // Hovd
    {  369,  914,       0,  388697,       0,  0,  6,  0, }, // India
    {  369,  920,       0,  388703,       0,  0,  6,  0, }, // Indian_Ocean
    {  369,  933,       0,  388709,       0,  0,  7,  0, }, // Indochina
    {  369,  943,       0,  388716,       0,  0, 10,  0, }, // Indonesia_Central
    {  369,  961,       0,  388726,       0,  0, 10,  0, }, // Indonesia_Eastern
    {  369,  979,       0,  388736,       0,  0, 10,  0, }, // Indonesia_Western
    {  369,  997,  388746,  388751,  388757,  5,  6,  6, }, // Iran
    {  369, 1008,  388763,  388771,  388780,  8,  9,  9, }, // Irkutsk
    {  369, 1016,  388789,  388796,  388804,  7,  8,  8, }, // Israel
    {  369, 1023,  134103,  134107,  388812,  4,  5,  5, }, // Japan
    {  369, 1029,  388817,  388836,  388856, 19, 20, 20, }, // Kamchatka
    {  369, 1047,       0,  388876,       0,  0,  8,  0, }, // Kazakhstan
    {  369, 1058,       0,  388884,       0,  0,  9,  0, }, // Kazakhstan_Eastern
    {  369, 1077,       0,  388893,       0,  0,  9,  0, }, // Kazakhstan_Western
    {  369, 1106,  388902,  388906,  388911,  4,  5,  5, }, // Korea
    {  369, 1112,       0,  388916,       0,  0,  6,  0, }, // Kosrae
    {  369, 1119,  388922,  388932,  388943, 10, 11, 11, }, // Krasnoyarsk
    {  369, 1151,       0,  388954,       0,  0,  6,  0, }, // Kyrgystan
    {  369, 1161,       0,  388960,       0,  0,  5,  0, }, // Lanka
    {  369, 1175,       0,  388965,       0,  0,  7,  0, }, // Line_Islands
    {  369, 1188,  388972,  388979,  388987,  7,  8,  8, }, // Lord_Howe
    {  369, 1198,  388995,  389000,  389006,  5,  6,  6, }, // Macau
    {  369, 1204,  389012,  389018,  389025,  6,  7,  7, }, // Magadan
    {  369, 1219,       0,  389032,       0,  0,  7,  0, }, // Malaysia
    {  369, 1228,       0,  389039,       0,  0,  7,  0, }, // Maldives
    {  369, 1237,       0,  389046,       0,  0,  7,  0, }, // Marquesas
    {  369, 1247,       0,  389053,       0,  0,  9,  0, }, // Marshall_Islands
    {  369, 1264,  389062,  389070,  389079,  8,  9,  9, }, // Mauritius
    {  369, 1274,       0,  389088,       0,  0,  8,  0, }, // Mawson
    {  369, 1281,  389096,  389105,  389115,  9, 10, 10, }, // Mexico_Pacific
    {  369, 1296,  389125,  389134,  389144,  9, 10, 10, }, // Mongolia
    {  369, 1305,  389154,  389160,  389167,  6,  7,  7, }, // Moscow
    {  369, 1312,       0,  389174,       0,  0,  7,  0, }, // Myanmar
    {  369, 1320,       0,  389181,       0,  0,  5,  0, }, // Nauru
    {  369, 1326,       0,  389186,       0,  0,  6,  0, }, // Nepal
    {  369, 1332,  389192,  389202,  389213, 10, 11, 11, }, // New_Caledonia
    {  369, 1346,  389224,  389234,  389245, 10, 11, 11, }, // New_Zealand
    {  369, 1358,  389256,  389268,  389281, 12, 13, 13, }, // Newfoundland
    {  369, 1371,       0,  389294,       0,  0,  5,  0, }, // Niue
    {  369, 1376,  389299,  389308,  389318,  9, 10, 10, }, // Norfolk
    {  369, 1384,  389328,  389344,  389361, 16, 17, 17, }, // Noronha
    {  369, 1392,       0,  389378,       0,  0,  9,  0, }, // North_Mariana
    {  369, 1406,  389387,  389397,  389408, 10, 11, 11, }, // Novosibirsk
    {  369, 1418,  389419,  389425,  389432,  6,  7,  7, }, // Omsk
    {  369, 1428,  389439,  389446,  389454,  7,  8,  8, }, // Pakistan
    {  369, 1437,       0,  389462,       0,  0,  5,  0, }, // Palau
    {  369, 1443,       0,  389467,       0,  0, 11,  0, }, // Papua_New_Guinea
    {  369, 1460,  389478,  389485,  389493,  7,  8,  8, }, // Paraguay
    {  369, 1469,  389501,  389506,  389512,  5,  6,  6, }, // Peru
    {  369, 1474,  389518,  389525,  389533,  7,  8,  8, }, // Philippines
    {  369, 1486,       0,  389541,       0,  0, 10,  0, }, // Phoenix_Islands
    {  369, 1502,  389551,  389566,  389582, 15, 16, 16, }, // Pierre_Miquelon
    {  369, 1518,       0,  389598,       0,  0,  7,  0, }, // Pitcairn
    {  369, 1527,       0,  389605,       0,  0,  5,  0, }, // Ponape
    {  369, 1534,       0,  389610,       0,  0,  4,  0, }, // Pyongyang
    {  369, 1544,  389614,  389621,  389629,  7,  8,  8, }, // Qyzylorda
    {  369, 1554,       0,  389637,       0,  0,  7,  0, }, // Reunion
    {  369, 1562,       0,  389644,       0,  0,  7,  0, }, // Rothera
    {  369, 1570,  389651,  389657,  389664,  6,  7,  7, }, // Sakhalin
    {  369, 1579,  389671,  389676,  389682,  5,  6,  6, }, // Samara
    {  369, 1596,  389688,  389693,  389699,  5,  6,  6, }, // Samoa
    {  369, 1602,       0,  389705,       0,  0,  7,  0, }, // Seychelles
    {  369, 1624,       0,  389712,       0,  0,  9,  0, }, // Singapore
    {  369, 1634,       0,  389721,       0,  0,  8,  0, }, // Solomon
    {  369, 1642,       0,  389729,       0,  0, 10,  0, }, // South_Georgia
    {  369, 1656,       0,  389739,       0,  0,  6,  0, }, // Suriname
    {  369, 1676,       0,  134907,       0,  0,  6,  0, }, // Syowa
    {  369, 1682,       0,  389745,       0,  0,  5,  0, }, // Tahiti
    {  369, 1689,  134918,  134922,  389750,  4,  5,  5, }, // Taipei
    {  369, 1696,       0,  389755,       0,  0,  8,  0, }, // Tajikistan
    {  369, 1724,       0,  389763,       0,  0,  6,  0, }, // Tokelau
    {  369, 1732,  389769,  389774,  389780,  5,  6,  6, }, // Tonga
    {  369, 1738,       0,  389786,       0,  0,  6,  0, }, // Truk
    {  369, 1750,  389792,  389802,  389813, 10, 11, 11, }, // Turkmenistan
    {  369, 1763,       0,  389824,       0,  0,  5,  0, }, // Tuvalu
    {  369, 1777,  389829,  389836,  389844,  7,  8,  8, }, // Uruguay
    {  369, 1792,  389852,  389861,  389871,  9, 10, 10, }, // Uzbekistan
    {  369, 1803,  389881,  389887,  389894,  6,  7,  7, }, // Vanuatu
    {  369, 1811,       0,  389901,       0,  0,  7,  0, }, // Venezuela
    {  369, 1821,  389908,  389917,  389927,  9, 10, 10, }, // Vladivostok
    {  369, 1833,  389937,  389946,  389956,  9, 10, 10, }, // Volgograd
    {  369, 1843,       0,  389966,       0,  0,  9,  0, }, // Vostok
    {  369, 1850,       0,  389975,       0,  0,  7,  0, }, // Wake
    {  369, 1855,       0,  389982,       0,  0, 10,  0, }, // Wallis
    {  369, 1862,  389992,  389999,  390007,  7,  8,  8, }, // Yakutsk
    {  369, 1870,  390015,  390025,  390036, 10, 11, 11, }, // Yekaterinburg
    {  369, 1892,       0,  390047,       0,  0,  6,  0, }, // Yukon
    {  370,    5,       0,  390053,       0,  0, 17,  0, }, // Afghanistan Javanese/Latin/Indonesia
    {  370,   17,       0,  390070,       0,  0, 19,  0, }, // Africa_Central
    {  370,   32,       0,  390089,       0,  0, 18,  0, }, // Africa_Eastern
    {  370,   65,       0,  390107,       0,  0, 26,  0, }, // Africa_Southern
    {  370,   81,  390133,  390151,  390177, 18, 26, 25, }, // Africa_Western
    {  370,  107,  390202,  390214,  390234, 12, 20, 19, }, // Alaska
    {  370,  135,  390253,  390265,  390285, 12, 20, 19, }, // Amazon
    {  370,  142,  390304,  390316,  390336, 12, 20, 19, }, // America_Central
    {  370,  158,  390355,  390372,  390397, 17, 25, 24, }, // America_Eastern
    {  370,  174,  390421,  390431,  390449, 10, 18, 17, }, // America_Mountain
    {  370,  191,  390466,  390479,  390500, 13, 21, 20, }, // America_Pacific
    {  370,  214,  390520,  390530,  390548, 10, 18, 17, }, // Apia
    {  370,  232,  390565,  390575,  390593, 10, 18, 17, }, // Arabian
    {  370,  240,  390610,  390625,  390648, 15, 23, 22, }, // Argentina
    {  370,  250,  390670,  390697,  390732, 27, 35, 34, }, // Argentina_Western
    {  370,  268,  390766,  390779,  390800, 13, 21, 20, }, // Armenia
    {  370,  286,  390820,  390834,  390856, 14, 22, 21, }, // Atlantic
    {  370,  295,  390877,  390899,  390929, 22, 30, 29, }, // Australia_Central
    {  370,  313,  390958,  390992,  391034, 34, 42, 41, }, // Australia_CentralWestern
    {  370,  338,  391075,  391102,  391137, 27, 35, 34, }, // Australia_Eastern
    {  370,  356,  391171,  391198,  391233, 27, 35, 34, }, // Australia_Western
    {  370,  374,  391267,  391283,  391307, 16, 24, 23, }, // Azerbaijan
    {  370,  385,  391330,  391342,  391362, 12, 20, 19, }, // Azores
    {  370,  397,  391381,  391397,  391421, 16, 24, 23, }, // Bangladesh
    {  370,  415,       0,  391444,       0,  0, 12,  0, }, // Bhutan
    {  370,  422,       0,  391456,       0,  0, 13,  0, }, // Bolivia
    {  370,  437,  391469,  391483,  391505, 14, 22, 21, }, // Brasilia
    {  370,  454,       0,  391526,       0,  0, 23,  0, }, // Brunei
    {  370,  461,  391549,  391568,  391595, 19, 27, 26, }, // Cape_Verde
    {  370,  478,       0,  391621,       0,  0, 14,  0, }, // Chamorro
    {  370,  487,  391635,  391648,  391669, 13, 21, 20, }, // Chatham
    {  370,  495,  391689,  391700,  391719, 11, 19, 18, }, // Chile
    {  370,  501,  391737,  391747,  391765, 10, 18, 17, }, // China
    {  370,  507,       0,  391782,       0,  0, 16,  0, }, // Christmas
    {  370,  517,       0,  391798,       0,  0, 20,  0, }, // Cocos
    {  370,  523,  391818,  391832,  391854, 14, 22, 21, }, // Colombia
    {  370,  532,  391875,  391894,  391921, 19, 27, 26, }, // Cook
    {  370,  537,  391947,  391957,  391975, 10, 18, 17, }, // Cuba
    {  370,  548,       0,  391992,       0,  0, 11,  0, }, // Davis
    {  370,  564,       0,  392003,       0,  0, 22,  0, }, // DumontDUrville
    {  370,  601,       0,  392025,       0,  0, 17,  0, }, // East_Timor
    {  370,  612,  392042,  392059,  392084, 17, 25, 24, }, // Easter
    {  370,  619,       0,  392108,       0,  0, 13,  0, }, // Ecuador
    {  370,  627,  392121,  392139,  392165, 18, 26, 25, }, // Europe_Central
    {  370,  642,  392190,  392213,  392244, 23, 31, 30, }, // Europe_Eastern
    {  370,  657,       0,  392274,       0,  0, 39,  0, }, // Europe_Further_Eastern
    {  370,  680,  392313,  392336,  392367, 23, 31, 30, }, // Europe_Western
    {  370,  695,  392397,  392420,  392451, 23, 31, 30, }, // Falkland
    {  370,  704,  392481,  392491,  392509, 10, 18, 17, }, // Fiji
    {  370,  709,       0,  392526,       0,  0, 20,  0, }, // French_Guiana
    {  370,  723,       0,  392546,       0,  0, 39,  0, }, // French_Southern
    {  370,  746,       0,  392585,       0,  0, 15,  0, }, // Galapagos
    {  370,  756,       0,  392600,       0,  0, 13,  0, }, // Gambier
    {  370,  764,  392613,  392626,  392647, 13, 21, 20, }, // Georgia
    {  370,  772,       0,  392667,       0,  0, 22,  0, }, // Gilbert_Islands
    {  370,  788,       0,  392689,       0,  0, 22,  0, }, // GMT
    {  370,  830,  392711,  392731,  392759, 20, 28, 27, }, // Greenland_Eastern
    {  370,  848,  392786,  392806,  392834, 20, 28, 27, }, // Greenland_Western
    {  370,  871,       0,  392861,       0,  0, 19,  0, }, // Gulf
    {  370,  876,       0,  392880,       0,  0, 12,  0, }, // Guyana
    {  370,  883,  392892,  392913,  392942, 21, 29, 28, }, // Hawaii_Aleutian
    {  370,  899,  392970,  392985,  393008, 15, 23, 22, }, // Hong_Kong
    {  370,  909,  393030,  393040,  393058, 10, 18, 17, }, // Hovd
    {  370,  914,       0,  393075,       0,  0, 19,  0, }, // India
    {  370,  920,       0,  393094,       0,  0, 19,  0, }, // Indian_Ocean
    {  370,  933,       0,  393113,       0,  0, 14,  0, }, // Indochina
    {  370,  943,       0,  393127,       0,  0, 22,  0, }, // Indonesia_Central
    {  370,  961,       0,  393149,       0,  0, 27,  0, }, // Indonesia_Eastern
    {  370,  979,       0,  393176,       0,  0, 27,  0, }, // Indonesia_Western
    {  370,  997,  393203,  393213,  393231, 10, 18, 17, }, // Iran
    {  370, 1008,  393248,  393261,  393282, 13, 21, 20, }, // Irkutsk
    {  370, 1016,  393302,  393314,  393334, 12, 20, 19, }, // Israel
    {  370, 1023,  393353,  393365,  393385, 12, 20, 19, }, // Japan
    {  370, 1047,       0,  393404,       0,  0, 16,  0, }, // Kazakhstan
    {  370, 1058,       0,  393420,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {  370, 1077,       0,  393442,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  370, 1106,  393464,  393475,  393494, 11, 19, 18, }, // Korea
    {  370, 1112,       0,  393512,       0,  0, 12,  0, }, // Kosrae
    {  370, 1119,  393524,  393541,  393566, 17, 25, 24, }, // Krasnoyarsk
    {  370, 1151,       0,  393590,       0,  0, 16,  0, }, // Kyrgystan
    {  370, 1175,       0,  393606,       0,  0, 19,  0, }, // Line_Islands
    {  370, 1188,  393625,  393640,  393663, 15, 23, 22, }, // Lord_Howe
    {  370, 1204,  393685,  393698,  393719, 13, 21, 20, }, // Magadan
    {  370, 1219,       0,  393739,       0,  0, 14,  0, }, // Malaysia
    {  370, 1228,       0,  393753,       0,  0, 14,  0, }, // Maldives
    {  370, 1237,       0,  393767,       0,  0, 15,  0, }, // Marquesas
    {  370, 1247,       0,  393782,       0,  0, 23,  0, }, // Marshall_Islands
    {  370, 1264,  393805,  393820,  393843, 15, 23, 22, }, // Mauritius
    {  370, 1274,       0,  393865,       0,  0, 12,  0, }, // Mawson
    {  370, 1281,  393877,  393898,  393927, 21, 29, 28, }, // Mexico_Pacific
    {  370, 1296,  393955,  393972,  393997, 17, 25, 24, }, // Mongolia
    {  370, 1305,  394021,  394033,  394053, 12, 20, 19, }, // Moscow
    {  370, 1312,       0,  394072,       0,  0, 13,  0, }, // Myanmar
    {  370, 1320,       0,  394085,       0,  0, 11,  0, }, // Nauru
    {  370, 1326,       0,  394096,       0,  0, 11,  0, }, // Nepal
    {  370, 1332,  394107,  394128,  394157, 21, 29, 28, }, // New_Caledonia
    {  370, 1346,  394185,  394205,  394233, 20, 28, 27, }, // New_Zealand
    {  370, 1358,  394260,  394278,  394304, 18, 26, 25, }, // Newfoundland
    {  370, 1371,       0,  394329,       0,  0, 10,  0, }, // Niue
    {  370, 1376,  394339,  394357,  394383, 18, 26, 25, }, // Norfolk
    {  370, 1384,  394408,  394433,  394466, 25, 33, 32, }, // Noronha
    {  370, 1406,  394498,  394515,  394540, 17, 25, 24, }, // Novosibirsk
    {  370, 1418,  394564,  394574,  394592, 10, 18, 17, }, // Omsk
    {  370, 1428,  394609,  394623,  394645, 14, 22, 21, }, // Pakistan
    {  370, 1437,       0,  394666,       0,  0, 11,  0, }, // Palau
    {  370, 1443,       0,  394677,       0,  0, 18,  0, }, // Papua_New_Guinea
    {  370, 1460,  394695,  394709,  394731, 14, 22, 21, }, // Paraguay
    {  370, 1469,  394752,  394762,  394780, 10, 18, 17, }, // Peru
    {  370, 1474,  394797,  394811,  394833, 14, 22, 21, }, // Philippines
    {  370, 1486,       0,  394854,       0,  0, 22,  0, }, // Phoenix_Islands
    {  370, 1502,  394876,  394907,  394946, 31, 39, 38, }, // Pierre_Miquelon
    {  370, 1518,       0,  394984,       0,  0, 14,  0, }, // Pitcairn
    {  370, 1527,       0,  394998,       0,  0, 12,  0, }, // Ponape
    {  370, 1534,       0,  395010,       0,  0, 15,  0, }, // Pyongyang
    {  370, 1554,       0,  395025,       0,  0, 13,  0, }, // Reunion
    {  370, 1562,       0,  395038,       0,  0, 13,  0, }, // Rothera
    {  370, 1570,  395051,  395065,  395087, 14, 22, 21, }, // Sakhalin
    {  370, 1596,  395108,  395119,  395138, 11, 19, 18, }, // Samoa
    {  370, 1602,       0,  395156,       0,  0, 16,  0, }, // Seychelles
    {  370, 1624,       0,  395172,       0,  0, 15,  0, }, // Singapore
    {  370, 1634,       0,  395187,       0,  0, 22,  0, }, // Solomon
    {  370, 1642,       0,  395209,       0,  0, 19,  0, }, // South_Georgia
    {  370, 1656,       0,  395228,       0,  0, 14,  0, }, // Suriname
    {  370, 1676,       0,  395242,       0,  0, 11,  0, }, // Syowa
    {  370, 1682,       0,  395253,       0,  0, 12,  0, }, // Tahiti
    {  370, 1689,  395265,  395277,  395297, 12, 20, 19, }, // Taipei
    {  370, 1696,       0,  395316,       0,  0, 16,  0, }, // Tajikistan
    {  370, 1724,       0,  395332,       0,  0, 13,  0, }, // Tokelau
    {  370, 1732,  395345,  395356,  395375, 11, 19, 18, }, // Tonga
    {  370, 1738,       0,  395393,       0,  0, 11,  0, }, // Truk
    {  370, 1750,  395404,  395422,  395448, 18, 26, 25, }, // Turkmenistan
    {  370, 1763,       0,  395473,       0,  0, 12,  0, }, // Tuvalu
    {  370, 1777,  395485,  395498,  395519, 13, 21, 20, }, // Uruguay
    {  370, 1792,  395539,  395555,  395579, 16, 24, 23, }, // Uzbekistan
    {  370, 1803,  395602,  395615,  395636, 13, 21, 20, }, // Vanuatu
    {  370, 1811,       0,  395656,       0,  0, 15,  0, }, // Venezuela
    {  370, 1821,  395671,  395688,  395713, 17, 25, 24, }, // Vladivostok
    {  370, 1833,  395737,  395752,  395775, 15, 23, 22, }, // Volgograd
    {  370, 1843,       0,  395797,       0,  0, 12,  0, }, // Vostok
    {  370, 1850,       0,  395809,       0,  0, 15,  0, }, // Wake
    {  370, 1855,       0,  395824,       0,  0, 23,  0, }, // Wallis
    {  370, 1862,  395847,  395860,  395881, 13, 21, 20, }, // Yakutsk
    {  370, 1870,  395901,  395920,  395947, 19, 27, 26, }, // Yekaterinburg
    {  370, 1892,       0,  395973,       0,  0, 11,  0, }, // Yukon
    {  373,   17,       0,  395984,       0,  0, 21,  0, }, // Africa_Central Kabuverdianu/Latin/Cape Verde
    {  373,   32,       0,  396005,       0,  0, 22,  0, }, // Africa_Eastern
    {  373,   65,       0,  396027,       0,  0, 20,  0, }, // Africa_Southern
    {  373,   81,  396047,  396070,  396100, 23, 30, 32, }, // Africa_Western
    {  373,  107,  396132,  396145,  396165, 13, 20, 22, }, // Alaska
    {  373,  142,  396187,  396198,  396216, 11, 18, 20, }, // America_Central
    {  373,  158,  396236,  396248,  396267, 12, 19, 21, }, // America_Eastern
    {  373,  174,  396288,  396303,  396325, 15, 22, 24, }, // America_Mountain
    {  373,  191,  396349,  396364,  396386, 15, 22, 24, }, // America_Pacific
    {  373,  286,  396410,  396426,  396449, 16, 23, 25, }, // Atlantic
    {  373,  295,  396474,  396498,  396529, 24, 31, 33, }, // Australia_Central
    {  373,  313,  396562,  396595,  396635, 33, 40, 42, }, // Australia_CentralWestern
    {  373,  338,  396677,  396702,  396734, 25, 32, 34, }, // Australia_Eastern
    {  373,  356,  396768,  396794,  396827, 26, 33, 35, }, // Australia_Western
    {  373,  461,  396862,  396879,  396903, 17, 24, 26, }, // Cape_Verde
    {  373,  537,  396929,  396940,  396958, 11, 18, 20, }, // Cuba
    {  373,  627,  396978,  396999,  397027, 21, 28, 30, }, // Europe_Central
    {  373,  642,  397057,  397079,  397108, 22, 29, 31, }, // Europe_Eastern
    {  373,  680,  397139,  397162,  397192, 23, 30, 32, }, // Europe_Western
    {  373,  788,       0,  397224,       0,  0, 22,  0, }, // GMT
    {  373,  830,  397246,  397273,  397307, 27, 34, 36, }, // Greenland_Eastern
    {  373,  848,  397343,  397371,  397406, 28, 35, 37, }, // Greenland_Western
    {  373,  883,  397443,  397464,  397492, 21, 28, 30, }, // Hawaii_Aleutian
    {  373, 1281,  397522,  397546,  397577, 24, 31, 33, }, // Mexico_Pacific
    {  373, 1358,  397610,  397626,  397649, 16, 23, 25, }, // Newfoundland
    {  373, 1502,  397674,  397700,  397733, 26, 33, 35, }, // Pierre_Miquelon
    {  373, 1892,       0,  397768,       0,  0, 12,  0, }, // Yukon
    {  374,    5,       0,  397780,       0,  0, 17,  0, }, // Afghanistan Kabyle/Latin/Algeria
    {  374,   17,       0,  397797,       0,  0, 24,  0, }, // Africa_Central
    {  374,   32,       0,  397821,       0,  0, 24,  0, }, // Africa_Eastern
    {  374,   65,       0,  397845,       0,  0, 30,  0, }, // Africa_Southern
    {  374,   81,  397875,  397897,  397927, 22, 30, 31, }, // Africa_Western
    {  374,  107,  397958,  397971,  397992, 13, 21, 22, }, // Alaska
    {  374,  135,  398014,  398027,  398048, 13, 21, 22, }, // Amazon
    {  374,  142,  398070,  398095,  398128, 25, 33, 34, }, // America_Central
    {  374,  158,  398162,  398186,  398218, 24, 32, 33, }, // America_Eastern
    {  374,  174,  398251,  398275,  398307, 24, 32, 33, }, // America_Mountain
    {  374,  191,  398340,  398351,  398370, 11, 19, 20, }, // America_Pacific
    {  374,  214,  398390,  398401,  398420, 11, 19, 20, }, // Apia
    {  374,  232,  398440,  398450,  398468, 10, 18, 19, }, // Arabian
    {  374,  240,  398487,  398502,  398525, 15, 23, 24, }, // Argentina
    {  374,  250,  398549,  398574,  398607, 25, 33, 34, }, // Argentina_Western
    {  374,  268,  398641,  398655,  398677, 14, 22, 23, }, // Armenia
    {  374,  286,  398700,  398712,  398732, 12, 20, 21, }, // Atlantic
    {  374,  295,  398753,  398778,  398811, 25, 33, 34, }, // Australia_Central
    {  374,  313,  398845,  398879,  398921, 34, 42, 43, }, // Australia_CentralWestern
    {  374,  338,  398964,  398989,  399022, 25, 33, 34, }, // Australia_Eastern
    {  374,  356,  399056,  399079,  399110, 23, 31, 30, }, // Australia_Western
    {  374,  374,  399140,  399156,  399180, 16, 24, 25, }, // Azerbaijan
    {  374,  385,  399205,  399218,  399239, 13, 21, 22, }, // Azores
    {  374,  397,  399261,  399277,  399301, 16, 24, 25, }, // Bangladesh
    {  374,  415,       0,  399326,       0,  0, 12,  0, }, // Bhutan
    {  374,  422,       0,  399338,       0,  0, 13,  0, }, // Bolivia
    {  374,  437,  399351,  399366,  399389, 15, 23, 24, }, // Brasilia
    {  374,  454,       0,  399413,       0,  0, 23,  0, }, // Brunei
    {  374,  461,  399436,  399450,  399472, 14, 22, 23, }, // Cape_Verde
    {  374,  478,       0,  399495,       0,  0, 22,  0, }, // Chamorro
    {  374,  487,  399517,  399530,  399551, 13, 21, 20, }, // Chatham
    {  374,  495,  399571,  399582,  399601, 11, 19, 20, }, // Chile
    {  374,  501,  399621,  399631,  399649, 10, 18, 19, }, // China
    {  374,  507,       0,  399668,       0,  0, 25,  0, }, // Christmas
    {  374,  517,       0,  399693,       0,  0, 22,  0, }, // Cocos
    {  374,  523,  399715,  399730,  399753, 15, 23, 24, }, // Colombia
    {  374,  532,  399693,  399777,  399806, 21, 29, 30, }, // Cook
    {  374,  537,  399836,  399847,  399866, 11, 19, 20, }, // Cuba
    {  374,  548,       0,  399886,       0,  0, 12,  0, }, // Davis
    {  374,  564,       0,  399898,       0,  0, 23,  0, }, // DumontDUrville
    {  374,  601,       0,  399921,       0,  0, 22,  0, }, // East_Timor
    {  374,  612,  399943,  399966,  399997, 23, 31, 32, }, // Easter
    {  374,  619,       0,  400029,       0,  0, 14,  0, }, // Ecuador
    {  374,  627,  400043,  400066,  400097, 23, 31, 32, }, // Europe_Central
    {  374,  642,  400129,  400152,  400183, 23, 31, 32, }, // Europe_Eastern
    {  374,  657,       0,  400215,       0,  0, 30,  0, }, // Europe_Further_Eastern
    {  374,  680,  400245,  400266,  400295, 21, 29, 28, }, // Europe_Western
    {  374,  695,  400323,  400349,  400383, 26, 34, 33, }, // Falkland
    {  374,  704,  400416,  400427,  400446, 11, 19, 20, }, // Fiji
    {  374,  709,       0,  400466,       0,  0, 24,  0, }, // French_Guiana
    {  374,  723,       0,  400490,       0,  0, 41,  0, }, // French_Southern
    {  374,  746,       0,  400531,       0,  0, 16,  0, }, // Galapagos
    {  374,  756,       0,  400547,       0,  0, 23,  0, }, // Gambier
    {  374,  764,  400570,  400584,  400606, 14, 22, 23, }, // Georgia
    {  374,  772,       0,  400629,       0,  0, 22,  0, }, // Gilbert_Islands
    {  374,  788,       0,  400651,       0,  0, 24,  0, }, // GMT
    {  374,  830,  400675,  400700,  400733, 25, 33, 34, }, // Greenland_Eastern
    {  374,  848,  400767,  400790,  400821, 23, 31, 32, }, // Greenland_Western
    {  374,  871,       0,  400853,       0,  0, 19,  0, }, // Gulf
    {  374,  876,       0,  400466,       0,  0, 13,  0, }, // Guyana
    {  374,  883,  400872,  400893,  400922, 21, 29, 30, }, // Hawaii_Aleutian
    {  374,  899,  400952,  400968,  400992, 16, 24, 25, }, // Hong_Kong
    {  374,  909,  401017,  401028,  401047, 11, 19, 20, }, // Hovd
    {  374,  914,       0,  401067,       0,  0, 20,  0, }, // India
    {  374,  920,       0,  401087,       0,  0, 20,  0, }, // Indian_Ocean
    {  374,  933,       0,  401107,       0,  0, 14,  0, }, // Indochina
    {  374,  943,       0,  401121,       0,  0, 27,  0, }, // Indonesia_Central
    {  374,  961,       0,  401148,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  374,  979,       0,  401174,       0,  0, 24,  0, }, // Indonesia_Western
    {  374,  997,  401198,  401209,  401228, 11, 19, 18, }, // Iran
    {  374, 1008,  401246,  401260,  401282, 14, 22, 23, }, // Irkutsk
    {  374, 1016,  401305,  401319,  401341, 14, 22, 23, }, // Israel
    {  374, 1023,  401364,  401376,  401396, 12, 20, 21, }, // Japan
    {  374, 1058,       0,  401417,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  374, 1077,       0,  401444,       0,  0, 27,  0, }, // Kazakhstan_Western
    {  374, 1106,  401471,  401483,  401503, 12, 20, 21, }, // Korea
    {  374, 1112,       0,  401524,       0,  0, 13,  0, }, // Kosrae
    {  374, 1119,  401537,  401555,  401580, 18, 25, 27, }, // Krasnoyarsk
    {  374, 1151,       0,  401607,       0,  0, 17,  0, }, // Kyrgystan
    {  374, 1175,       0,  401624,       0,  0, 19,  0, }, // Line_Islands
    {  374, 1188,  401643,  401659,  401682, 16, 23, 25, }, // Lord_Howe
    {  374, 1204,  401707,  401721,  401743, 14, 22, 23, }, // Magadan
    {  374, 1219,       0,  401766,       0,  0, 14,  0, }, // Malaysia
    {  374, 1228,       0,  401780,       0,  0, 13,  0, }, // Maldives
    {  374, 1237,       0,  401793,       0,  0, 22,  0, }, // Marquesas
    {  374, 1247,       0,  401815,       0,  0, 22,  0, }, // Marshall_Islands
    {  374, 1264,  401837,  401849,  401869, 12, 20, 21, }, // Mauritius
    {  374, 1274,       0,  401890,       0,  0, 13,  0, }, // Mawson
    {  374, 1281,  401903,  401923,  401951, 20, 28, 29, }, // Mexico_Pacific
    {  374, 1296,  401980,  401996,  402020, 16, 24, 25, }, // Mongolia
    {  374, 1305,  402045,  402057,  402077, 12, 20, 21, }, // Moscow
    {  374, 1312,       0,  402098,       0,  0, 14,  0, }, // Myanmar
    {  374, 1320,       0,  402112,       0,  0, 11,  0, }, // Nauru
    {  374, 1326,       0,  402123,       0,  0, 12,  0, }, // Nepal
    {  374, 1332,  402135,  402160,  402193, 25, 33, 34, }, // New_Caledonia
    {  374, 1346,  402227,  402249,  402279, 22, 30, 29, }, // New_Zealand
    {  374, 1358,  402308,  402328,  402356, 20, 28, 29, }, // Newfoundland
    {  374, 1371,       0,  402385,       0,  0, 11,  0, }, // Niue
    {  374, 1376,  402396,  402420,  402452, 24, 32, 33, }, // Norfolk
    {  374, 1384,  402485,  402510,  402543, 25, 33, 34, }, // Noronha
    {  374, 1406,  402577,  402595,  402620, 18, 25, 27, }, // Novosibirsk
    {  374, 1418,  402647,  402658,  402676, 11, 18, 20, }, // Omsk
    {  374, 1428,  402696,  402711,  402734, 15, 23, 24, }, // Pakistan
    {  374, 1437,       0,  402758,       0,  0, 12,  0, }, // Palau
    {  374, 1443,       0,  402770,       0,  0, 29,  0, }, // Papua_New_Guinea
    {  374, 1460,  402799,  402814,  402837, 15, 23, 24, }, // Paraguay
    {  374, 1469,  402861,  402872,  402891, 11, 19, 20, }, // Peru
    {  374, 1474,  402911,  402925,  402947, 14, 22, 23, }, // Philippines
    {  374, 1486,       0,  402970,       0,  0, 24,  0, }, // Phoenix_Islands
    {  374, 1502,  402994,  403018,  403050, 24, 32, 33, }, // Pierre_Miquelon
    {  374, 1518,       0,  403083,       0,  0, 14,  0, }, // Pitcairn
    {  374, 1527,       0,  403097,       0,  0, 13,  0, }, // Ponape
    {  374, 1534,       0,  403110,       0,  0, 16,  0, }, // Pyongyang
    {  374, 1554,       0,  403126,       0,  0, 15,  0, }, // Reunion
    {  374, 1562,       0,  403141,       0,  0, 14,  0, }, // Rothera
    {  374, 1570,  403155,  403170,  403192, 15, 22, 24, }, // Sakhalin
    {  374, 1596,  403216,  403228,  403248, 12, 20, 21, }, // Samoa
    {  374, 1602,       0,  403269,       0,  0, 13,  0, }, // Seychelles
    {  374, 1624,       0,  403282,       0,  0, 23,  0, }, // Singapore
    {  374, 1634,       0,  403305,       0,  0, 23,  0, }, // Solomon
    {  374, 1642,       0,  403328,       0,  0, 22,  0, }, // South_Georgia
    {  374, 1656,       0,  403350,       0,  0, 14,  0, }, // Suriname
    {  374, 1676,       0,  403364,       0,  0, 12,  0, }, // Syowa
    {  374, 1682,       0,  403376,       0,  0, 13,  0, }, // Tahiti
    {  374, 1689,  403389,  403402,  403423, 13, 21, 22, }, // Taipei
    {  374, 1696,       0,  403445,       0,  0, 17,  0, }, // Tajikistan
    {  374, 1724,       0,  403462,       0,  0, 14,  0, }, // Tokelau
    {  374, 1732,  403476,  403488,  403508, 12, 20, 21, }, // Tonga
    {  374, 1738,       0,  403529,       0,  0, 12,  0, }, // Truk
    {  374, 1750,  403541,  403560,  403587, 19, 27, 28, }, // Turkmenistan
    {  374, 1763,       0,  403615,       0,  0, 13,  0, }, // Tuvalu
    {  374, 1777,  403628,  403642,  403664, 14, 22, 23, }, // Uruguay
    {  374, 1792,  403687,  403704,  403729, 17, 25, 26, }, // Uzbekistan
    {  374, 1803,  403755,  403770,  403792, 15, 22, 24, }, // Vanuatu
    {  374, 1811,       0,  403816,       0,  0, 16,  0, }, // Venezuela
    {  374, 1821,  403832,  403850,  403875, 18, 25, 27, }, // Vladivostok
    {  374, 1833,  403902,  403918,  403941, 16, 23, 25, }, // Volgograd
    {  374, 1843,       0,  403966,       0,  0, 13,  0, }, // Vostok
    {  374, 1850,       0,  403979,       0,  0, 22,  0, }, // Wake
    {  374, 1855,       0,  404001,       0,  0, 26,  0, }, // Wallis
    {  374, 1862,  404027,  404041,  404063, 14, 22, 23, }, // Yakutsk
    {  374, 1870,  404086,  404106,  404133, 20, 27, 29, }, // Yekaterinburg
    {  379,    0,  404162,  404170,  404187,  8, 17, 15, }, // Acre Kannada/Kannada/India
    {  379,    5,       0,  404202,       0,  0, 15,  0, }, // Afghanistan
    {  379,   17,       0,  404217,       0,  0, 16,  0, }, // Africa_Central
    {  379,   32,       0,  404233,       0,  0, 17,  0, }, // Africa_Eastern
    {  379,   65,       0,  404250,       0,  0, 27,  0, }, // Africa_Southern
    {  379,   81,  404277,  404295,  404322, 18, 27, 25, }, // Africa_Western
    {  379,  107,  404347,  404358,  404378, 11, 20, 17, }, // Alaska
    {  379,  128,  404395,  404406,  404426, 11, 20, 18, }, // Almaty
    {  379,  135,  404444,  404455,  404475, 11, 20, 18, }, // Amazon
    {  379,  142,  404493,  404517,  404550, 24, 33, 31, }, // America_Central
    {  379,  158,  404581,  404605,  404638, 24, 33, 29, }, // America_Eastern
    {  379,  174,  404667,  404690,  404722, 23, 32, 28, }, // America_Mountain
    {  379,  191,  404750,  404776,  404811, 26, 35, 31, }, // America_Pacific
    {  379,  207,  404842,  404853,  404873, 11, 20, 16, }, // Anadyr
    {  379,  214,  404889,  404898,  404916,  9, 18, 14, }, // Apia
    {  379,  219,  404930,  404939,  404957,  9, 18, 16, }, // Aqtau
    {  379,  225,  404973,  404984,  405004, 11, 20, 18, }, // Aqtobe
    {  379,  232,  405022,  405034,  405055, 12, 21, 17, }, // Arabian
    {  379,  240,  405072,  405086,  405109, 14, 23, 21, }, // Argentina
    {  379,  250,  405130,  405151,  405181, 21, 30, 28, }, // Argentina_Western
    {  379,  268,  405209,  405221,  405242, 12, 21, 19, }, // Armenia
    {  379,  286,  405261,  405275,  405298, 14, 23, 19, }, // Atlantic
    {  379,  295,  405317,  405339,  405371, 22, 32, 28, }, // Australia_Central
    {  379,  313,  405399,  405429,  405468, 30, 39, 35, }, // Australia_CentralWestern
    {  379,  338,  405503,  405524,  405555, 21, 31, 27, }, // Australia_Eastern
    {  379,  356,  405582,  405604,  405636, 22, 32, 28, }, // Australia_Western
    {  379,  374,  405664,  405678,  405701, 14, 23, 21, }, // Azerbaijan
    {  379,  385,  405722,  405732,  405751, 10, 19, 17, }, // Azores
    {  379,  397,  405768,  405782,  405805, 14, 23, 21, }, // Bangladesh
    {  379,  415,       0,  405826,       0,  0, 10,  0, }, // Bhutan
    {  379,  422,       0,  405836,       0,  0, 12,  0, }, // Bolivia
    {  379,  437,  405848,  405863,  405886, 15, 23, 21, }, // Brasilia
    {  379,  454,       0,  405907,       0,  0, 20,  0, }, // Brunei
    {  379,  461,  405927,  405941,  405964, 14, 23, 21, }, // Cape_Verde
    {  379,  478,       0,  405985,       0,  0, 18,  0, }, // Chamorro
    {  379,  487,  406003,  406012,  406030,  9, 18, 14, }, // Chatham
    {  379,  495,  406044,  406052,  406069,  8, 17, 15, }, // Chile
    {  379,  501,  406084,  406092,  406109,  8, 17, 13, }, // China
    {  379,  507,       0,  406122,       0,  0, 19,  0, }, // Christmas
    {  379,  517,       0,  406141,       0,  0, 18,  0, }, // Cocos
    {  379,  523,  406159,  406171,  406192, 12, 21, 19, }, // Colombia
    {  379,  532,  406211,  406227,  406252, 16, 25, 33, }, // Cook
    {  379,  537,  406285,  406295,  406314, 10, 19, 15, }, // Cuba
    {  379,  548,       0,  406329,       0,  0, 10,  0, }, // Davis
    {  379,  564,       0,  406339,       0,  0, 21,  0, }, // DumontDUrville
    {  379,  601,       0,  406360,       0,  0, 16,  0, }, // East_Timor
    {  379,  612,  406376,  406392,  406417, 16, 25, 23, }, // Easter
    {  379,  619,       0,  406440,       0,  0, 13,  0, }, // Ecuador
    {  379,  627,  406453,  406471,  406498, 18, 27, 25, }, // Europe_Central
    {  379,  642,  406523,  406542,  406570, 19, 28, 26, }, // Europe_Eastern
    {  379,  657,       0,  406596,       0,  0, 28,  0, }, // Europe_Further_Eastern
    {  379,  680,  406624,  406644,  406673, 20, 29, 27, }, // Europe_Western
    {  379,  695,  406700,  406724,  406757, 24, 33, 31, }, // Falkland
    {  379,  704,  406788,  406796,  406813,  8, 17, 15, }, // Fiji
    {  379,  709,       0,  406828,       0,  0, 17,  0, }, // French_Guiana
    {  379,  723,       0,  406845,       0,  0, 35,  0, }, // French_Southern
    {  379,  746,       0,  406880,       0,  0, 14,  0, }, // Galapagos
    {  379,  756,       0,  406894,       0,  0, 14,  0, }, // Gambier
    {  379,  764,  406908,  406920,  406941, 12, 21, 19, }, // Georgia
    {  379,  772,       0,  406960,       0,  0, 21,  0, }, // Gilbert_Islands
    {  379,  788,       0,  406981,       0,  0, 25,  0, }, // GMT
    {  379,  830,  407006,  407030,  407063, 24, 33, 31, }, // Greenland_Eastern
    {  379,  848,  407094,  407119,  407153, 25, 34, 32, }, // Greenland_Western
    {  379,  866,       0,  407185,       0,  0, 19,  0, }, // Guam
    {  379,  871,       0,  407204,       0,  0, 18,  0, }, // Gulf
    {  379,  876,       0,  406836,       0,  0,  9,  0, }, // Guyana
    {  379,  883,  407222,  407242,  407271, 20, 29, 25, }, // Hawaii_Aleutian
    {  379,  899,  407296,  407311,  407335, 15, 24, 22, }, // Hong_Kong
    {  379,  909,  407357,  407366,  407384,  9, 18, 16, }, // Hovd
    {  379,  914,       0,  407400,       0,  0, 19,  0, }, // India
    {  379,  920,       0,  407419,       0,  0, 17,  0, }, // Indian_Ocean
    {  379,  933,       0,  407436,       0,  0, 12,  0, }, // Indochina
    {  379,  943,       0,  407448,       0,  0, 19,  0, }, // Indonesia_Central
    {  379,  961,       0,  407467,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  379,  979,       0,  407487,       0,  0, 20,  0, }, // Indonesia_Western
    {  379,  997,  407507,  407516,  407534,  9, 18, 14, }, // Iran
    {  379, 1008,  407548,  407563,  407587, 15, 24, 22, }, // Irkutsk
    {  379, 1016,  407609,  407620,  407640, 11, 20, 16, }, // Israel
    {  379, 1023,  407656,  407665,  407683,  9, 18, 14, }, // Japan
    {  379, 1029,  407697,  407731,  407774, 34, 43, 41, }, // Kamchatka
    {  379, 1047,       0,  407815,       0,  0, 16,  0, }, // Kazakhstan
    {  379, 1058,       0,  407831,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {  379, 1077,       0,  407853,       0,  0, 23,  0, }, // Kazakhstan_Western
    {  379, 1106,  407876,  407887,  407907, 11, 20, 16, }, // Korea
    {  379, 1112,       0,  407923,       0,  0,  9,  0, }, // Kosrae
    {  379, 1119,  407932,  407953,  407983, 21, 30, 28, }, // Krasnoyarsk
    {  379, 1151,       0,  408011,       0,  0, 16,  0, }, // Kyrgystan
    {  379, 1161,       0,  408027,       0,  0,  8,  0, }, // Lanka
    {  379, 1175,       0,  408035,       0,  0, 16,  0, }, // Line_Islands
    {  379, 1188,  408051,  408066,  408090, 15, 24, 22, }, // Lord_Howe
    {  379, 1198,  408112,  408121,  408139,  9, 18, 16, }, // Macau
    {  379, 1204,  408155,  408165,  408184, 10, 19, 17, }, // Magadan
    {  379, 1219,       0,  408201,       0,  0, 11,  0, }, // Malaysia
    {  379, 1228,       0,  408212,       0,  0, 14,  0, }, // Maldives
    {  379, 1237,       0,  408226,       0,  0, 15,  0, }, // Marquesas
    {  379, 1247,       0,  408241,       0,  0, 19,  0, }, // Marshall_Islands
    {  379, 1264,  408260,  408271,  408291, 11, 20, 18, }, // Mauritius
    {  379, 1274,       0,  408309,       0,  0, 12,  0, }, // Mawson
    {  379, 1281,  408321,  408343,  408374, 22, 31, 27, }, // Mexico_Pacific
    {  379, 1296,  408401,  408416,  408440, 15, 24, 22, }, // Mongolia
    {  379, 1305,  408462,  408472,  408491, 10, 19, 17, }, // Moscow
    {  379, 1312,       0,  408508,       0,  0, 14,  0, }, // Myanmar
    {  379, 1320,       0,  408522,       0,  0,  8,  0, }, // Nauru
    {  379, 1326,       0,  408530,       0,  0,  9,  0, }, // Nepal
    {  379, 1332,  408539,  408559,  408588, 20, 29, 28, }, // New_Caledonia
    {  379, 1346,  408616,  408633,  408659, 17, 26, 22, }, // New_Zealand
    {  379, 1358,  408681,  408702,  408732, 21, 30, 26, }, // Newfoundland
    {  379, 1371,       0,  408758,       0,  0,  8,  0, }, // Niue
    {  379, 1376,  408766,  408784,  408811, 18, 27, 25, }, // Norfolk
    {  379, 1384,  408836,  408861,  408895, 25, 34, 31, }, // Noronha
    {  379, 1392,       0,  408926,       0,  0, 25,  0, }, // North_Mariana
    {  379, 1406,  408951,  408970,  408998, 19, 28, 26, }, // Novosibirsk
    {  379, 1418,  409024,  409035,  409055, 11, 20, 18, }, // Omsk
    {  379, 1428,  409073,  409086,  409108, 13, 22, 20, }, // Pakistan
    {  379, 1437,       0,  409128,       0,  0, 10,  0, }, // Palau
    {  379, 1443,       0,  409138,       0,  0, 21,  0, }, // Papua_New_Guinea
    {  379, 1460,  409159,  409170,  409190, 11, 20, 18, }, // Paraguay
    {  379, 1469,  409208,  409216,  409233,  8, 17, 15, }, // Peru
    {  379, 1474,  409248,  409260,  409281, 12, 21, 19, }, // Philippines
    {  379, 1486,       0,  409300,       0,  0, 20,  0, }, // Phoenix_Islands
    {  379, 1502,  409320,  409351,  409391, 31, 40, 36, }, // Pierre_Miquelon
    {  379, 1518,       0,  409427,       0,  0, 15,  0, }, // Pitcairn
    {  379, 1527,       0,  409442,       0,  0, 10,  0, }, // Ponape
    {  379, 1534,       0,  409452,       0,  0, 16,  0, }, // Pyongyang
    {  379, 1544,  409468,  409482,  409505, 14, 23, 21, }, // Qyzylorda
    {  379, 1554,       0,  409526,       0,  0, 13,  0, }, // Reunion
    {  379, 1562,       0,  409539,       0,  0, 10,  0, }, // Rothera
    {  379, 1570,  409549,  409565,  409590, 16, 25, 23, }, // Sakhalin
    {  379, 1579,  409613,  409620,  409636,  7, 16, 14, }, // Samara
    {  379, 1596,  409650,  409659,  409677,  9, 18, 16, }, // Samoa
    {  379, 1602,       0,  409693,       0,  0, 12,  0, }, // Seychelles
    {  379, 1624,       0,  409705,       0,  0, 21,  0, }, // Singapore
    {  379, 1634,       0,  409726,       0,  0, 18,  0, }, // Solomon
    {  379, 1642,       0,  409744,       0,  0, 19,  0, }, // South_Georgia
    {  379, 1656,       0,  409763,       0,  0, 12,  0, }, // Suriname
    {  379, 1676,       0,  409775,       0,  0, 10,  0, }, // Syowa
    {  379, 1682,       0,  409785,       0,  0, 10,  0, }, // Tahiti
    {  379, 1689,  409795,  409803,  409820,  8, 17, 13, }, // Taipei
    {  379, 1696,       0,  409833,       0,  0, 14,  0, }, // Tajikistan
    {  379, 1724,       0,  409847,       0,  0, 12,  0, }, // Tokelau
    {  379, 1732,  409859,  409868,  409886,  9, 18, 16, }, // Tonga
    {  379, 1738,       0,  409902,       0,  0,  8,  0, }, // Truk
    {  379, 1750,  409910,  409931,  409961, 21, 30, 28, }, // Turkmenistan
    {  379, 1763,       0,  409989,       0,  0, 10,  0, }, // Tuvalu
    {  379, 1777,  409999,  410010,  410030, 11, 20, 18, }, // Uruguay
    {  379, 1792,  410048,  410065,  410091, 17, 26, 24, }, // Uzbekistan
    {  379, 1803,  410115,  410124,  410142,  9, 18, 16, }, // Vanuatu
    {  379, 1811,       0,  410158,       0,  0, 14,  0, }, // Venezuela
    {  379, 1821,  410172,  410190,  410217, 18, 27, 25, }, // Vladivostok
    {  379, 1833,  410242,  410258,  410283, 16, 25, 23, }, // Volgograd
    {  379, 1843,       0,  410306,       0,  0, 12,  0, }, // Vostok
    {  379, 1850,       0,  410318,       0,  0, 14,  0, }, // Wake
    {  379, 1855,       0,  410332,       0,  0, 27,  0, }, // Wallis
    {  379, 1862,  410359,  410372,  410394, 13, 22, 20, }, // Yakutsk
    {  379, 1870,  410414,  410432,  410459, 18, 27, 26, }, // Yekaterinburg
    {  379, 1892,       0,  410485,       0,  0, 10,  0, }, // Yukon
    {  380,    0,  410495,  410506,  410525, 11, 19, 17, }, // Acre Kashmiri/Arabic/India
    {  380,    5,       0,  410542,       0,  0, 16,  0, }, // Afghanistan
    {  380,   17,       0,  410558,       0,  0, 19,  0, }, // Africa_Central
    {  380,   32,       0,  410577,       0,  0, 19,  0, }, // Africa_Eastern
    {  380,   65,       0,  410596,       0,  0, 19,  0, }, // Africa_Southern
    {  380,   81,  410615,  410634,  410661, 19, 27, 25, }, // Africa_Western
    {  380,  107,  410686,  410699,  410720, 13, 21, 22, }, // Alaska
    {  380,  128,  410742,  410755,  410776, 13, 21, 19, }, // Almaty
    {  380,  135,  410795,  410808,  410829, 13, 21, 19, }, // Amazon
    {  380,  142,  410848,  410859,  410878, 11, 19, 20, }, // America_Central
    {  380,  158,  410898,  410909,  410928, 11, 19, 20, }, // America_Eastern
    {  380,  174,  410948,  410961,  410982, 13, 21, 22, }, // America_Mountain
    {  380,  191,  411004,  411017,  411038, 13, 21, 22, }, // America_Pacific
    {  380,  207,  411060,  411074,  411096, 14, 22, 19, }, // Anadyr
    {  380,  219,  411115,  411127,  411147, 12, 20, 18, }, // Aqtau
    {  380,  225,  411165,  411177,  411197, 12, 20, 18, }, // Aqtobe
    {  380,  232,  411215,  411229,  411251, 14, 22, 23, }, // Arabian
    {  380,  240,  411274,  411290,  411314, 16, 24, 22, }, // Argentina
    {  380,  250,  411336,  411358,  411388, 22, 30, 28, }, // Argentina_Western
    {  380,  268,  411416,  411431,  411454, 15, 23, 21, }, // Armenia
    {  380,  286,  411475,  411490,  411513, 15, 23, 24, }, // Atlantic
    {  380,  295,  411537,  411559,  411589, 22, 30, 31, }, // Australia_Central
    {  380,  313,  411620,  411648,  411684, 28, 36, 37, }, // Australia_CentralWestern
    {  380,  338,  411721,  411743,  411773, 22, 30, 31, }, // Australia_Eastern
    {  380,  356,  411804,  411826,  411857, 22, 31, 32, }, // Australia_Western
    {  380,  374,  411889,  411904,  411927, 15, 23, 21, }, // Azerbaijan
    {  380,  385,  411948,  411961,  411982, 13, 21, 18, }, // Azores
    {  380,  397,  412000,  412015,  412038, 15, 23, 21, }, // Bangladesh
    {  380,  415,       0,  412059,       0,  0, 12,  0, }, // Bhutan
    {  380,  422,       0,  412071,       0,  0, 14,  0, }, // Bolivia
    {  380,  437,  412085,  412100,  412123, 15, 23, 21, }, // Brasilia
    {  380,  454,       0,  412144,       0,  0, 24,  0, }, // Brunei
    {  380,  461,  412168,  412182,  412204, 14, 22, 15, }, // Cape_Verde
    {  380,  478,       0,  412219,       0,  0, 20,  0, }, // Chamorro
    {  380,  487,  412239,  412251,  412271, 12, 20, 18, }, // Chatham
    {  380,  495,  412289,  412299,  412317, 10, 18, 16, }, // Chile
    {  380,  501,  412333,  412344,  412363, 11, 19, 20, }, // China
    {  380,  507,       0,  412383,       0,  0, 12,  0, }, // Christmas
    {  380,  517,       0,  412395,       0,  0, 20,  0, }, // Cocos
    {  380,  523,  412415,  412430,  412453, 15, 23, 21, }, // Colombia
    {  380,  532,  412474,  412492,  412518, 18, 26, 28, }, // Cook
    {  380,  537,  412546,  412558,  412578, 12, 20, 21, }, // Cuba
    {  380,  548,       0,  412599,       0,  0, 11,  0, }, // Davis
    {  380,  564,       0,  412610,       0,  0, 22,  0, }, // DumontDUrville
    {  380,  601,       0,  412632,       0,  0, 18,  0, }, // East_Timor
    {  380,  612,  412650,  412669,  412696, 19, 27, 25, }, // Easter
    {  380,  619,       0,  412721,       0,  0, 14,  0, }, // Ecuador
    {  380,  627,  412735,  412753,  412779, 18, 26, 24, }, // Europe_Central
    {  380,  642,  412803,  412821,  412847, 18, 26, 24, }, // Europe_Eastern
    {  380,  657,       0,  412871,       0,  0, 21,  0, }, // Europe_Further_Eastern
    {  380,  680,  412892,  412911,  412938, 19, 27, 26, }, // Europe_Western
    {  380,  695,  412964,  412984,  413012, 20, 28, 26, }, // Falkland
    {  380,  704,  413038,  413049,  413068, 11, 19, 17, }, // Fiji
    {  380,  709,       0,  413085,       0,  0, 19,  0, }, // French_Guiana
    {  380,  723,       0,  413104,       0,  0, 30,  0, }, // French_Southern
    {  380,  746,       0,  413134,       0,  0, 15,  0, }, // Galapagos
    {  380,  756,       0,  413149,       0,  0, 14,  0, }, // Gambier
    {  380,  764,  413163,  413179,  413203, 16, 24, 22, }, // Georgia
    {  380,  772,       0,  413225,       0,  0, 20,  0, }, // Gilbert_Islands
    {  380,  788,       0,  413245,       0,  0, 20,  0, }, // GMT
    {  380,  830,  413265,  413290,  413323, 25, 33, 31, }, // Greenland_Eastern
    {  380,  848,  413354,  413379,  413412, 25, 33, 31, }, // Greenland_Western
    {  380,  866,       0,  413443,       0,  0, 11,  0, }, // Guam
    {  380,  871,       0,  413454,       0,  0, 18,  0, }, // Gulf
    {  380,  876,       0,  413472,       0,  0, 12,  0, }, // Guyana
    {  380,  883,  413484,  413507,  413538, 23, 31, 29, }, // Hawaii_Aleutian
    {  380,  899,  413567,  413581,  413603, 14, 22, 19, }, // Hong_Kong
    {  380,  909,  413622,  413632,  413650, 10, 18, 16, }, // Hovd
    {  380,  914,       0,  413666,       0,  0,  9,  0, }, // India
    {  380,  920,       0,  413675,       0,  0, 21,  0, }, // Indian_Ocean
    {  380,  933,       0,  413696,       0,  0, 16,  0, }, // Indochina
    {  380,  943,       0,  413712,       0,  0, 23,  0, }, // Indonesia_Central
    {  380,  961,       0,  413735,       0,  0, 24,  0, }, // Indonesia_Eastern
    {  380,  979,       0,  413759,       0,  0, 24,  0, }, // Indonesia_Western
    {  380,  997,  413783,  413797,  413819, 14, 22, 20, }, // Iran
    {  380, 1008,  413839,  413853,  413875, 14, 22, 20, }, // Irkutsk
    {  380, 1016,  413895,  413910,  413933, 15, 23, 24, }, // Israel
    {  380, 1023,  413957,  413969,  413989, 12, 20, 21, }, // Japan
    {  380, 1029,  414010,  414024,  414046, 14, 22, 20, }, // Kamchatka
    {  380, 1058,       0,  414066,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  380, 1077,       0,  414085,       0,  0, 19,  0, }, // Kazakhstan_Western
    {  380, 1106,  414104,  414116,  414136, 12, 20, 21, }, // Korea
    {  380, 1112,       0,  414157,       0,  0, 12,  0, }, // Kosrae
    {  380, 1119,  414169,  414187,  414213, 18, 26, 24, }, // Krasnoyarsk
    {  380, 1151,       0,  414237,       0,  0, 13,  0, }, // Kyrgystan
    {  380, 1161,       0,  414250,       0,  0, 11,  0, }, // Lanka
    {  380, 1175,       0,  414261,       0,  0, 20,  0, }, // Line_Islands
    {  380, 1188,  414281,  414296,  414319, 15, 23, 18, }, // Lord_Howe
    {  380, 1198,  414337,  414350,  414371, 13, 21, 19, }, // Macau
    {  380, 1204,  414390,  414403,  414424, 13, 21, 19, }, // Magadan
    {  380, 1219,       0,  414443,       0,  0, 14,  0, }, // Malaysia
    {  380, 1228,       0,  414457,       0,  0, 15,  0, }, // Maldives
    {  380, 1237,       0,  414472,       0,  0, 16,  0, }, // Marquesas
    {  380, 1247,       0,  414488,       0,  0, 20,  0, }, // Marshall_Islands
    {  380, 1264,  414508,  414521,  414542, 13, 21, 19, }, // Mauritius
    {  380, 1274,       0,  414561,       0,  0, 11,  0, }, // Mawson
    {  380, 1281,  414572,  414591,  414618, 19, 27, 27, }, // Mexico_Pacific
    {  380, 1296,  414645,  414659,  414681, 14, 22, 21, }, // Mongolia
    {  380, 1305,  414702,  414714,  414733, 12, 19, 17, }, // Moscow
    {  380, 1312,       0,  414750,       0,  0, 14,  0, }, // Myanmar
    {  380, 1320,       0,  414764,       0,  0, 14,  0, }, // Nauru
    {  380, 1326,       0,  414778,       0,  0, 12,  0, }, // Nepal
    {  380, 1332,  414790,  414810,  414838, 20, 28, 26, }, // New_Caledonia
    {  380, 1346,  414864,  414879,  414902, 15, 23, 23, }, // New_Zealand
    {  380, 1358,  414925,  414944,  414971, 19, 27, 27, }, // Newfoundland
    {  380, 1371,       0,  414998,       0,  0, 11,  0, }, // Niue
    {  380, 1376,  415009,  415022,  415043, 13, 21, 19, }, // Norfolk
    {  380, 1384,  415062,  415085,  415116, 23, 31, 29, }, // Noronha
    {  380, 1392,       0,  415145,       0,  0, 21,  0, }, // North_Mariana
    {  380, 1406,  415166,  415183,  415208, 17, 25, 23, }, // Novosibirsk
    {  380, 1418,  415231,  415242,  415261, 11, 19, 17, }, // Omsk
    {  380, 1428,  415278,  415292,  415314, 14, 22, 20, }, // Pakistan
    {  380, 1437,       0,  415334,       0,  0, 11,  0, }, // Palau
    {  380, 1443,       0,  415345,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  380, 1460,  415367,  415380,  415401, 13, 21, 19, }, // Paraguay
    {  380, 1469,  415420,  415431,  415450, 11, 19, 17, }, // Peru
    {  380, 1474,  415467,  415483,  415507, 16, 24, 22, }, // Philippines
    {  380, 1486,       0,  415529,       0,  0, 21,  0, }, // Phoenix_Islands
    {  380, 1502,  415550,  415576,  415610, 26, 34, 35, }, // Pierre_Miquelon
    {  380, 1518,       0,  415645,       0,  0, 14,  0, }, // Pitcairn
    {  380, 1527,       0,  415659,       0,  0, 11,  0, }, // Ponape
    {  380, 1534,       0,  415670,       0,  0, 14,  0, }, // Pyongyang
    {  380, 1544,  415684,  415697,  415718, 13, 21, 19, }, // Qyzylorda
    {  380, 1554,       0,  415737,       0,  0, 16,  0, }, // Reunion
    {  380, 1562,       0,  415753,       0,  0, 13,  0, }, // Rothera
    {  380, 1570,  415766,  415780,  415802, 14, 22, 20, }, // Sakhalin
    {  380, 1579,  415822,  415833,  415852, 11, 19, 17, }, // Samara
    {  380, 1596,  415869,  415880,  415899, 11, 19, 17, }, // Samoa
    {  380, 1602,       0,  415916,       0,  0, 13,  0, }, // Seychelles
    {  380, 1624,       0,  415929,       0,  0, 15,  0, }, // Singapore
    {  380, 1634,       0,  415944,       0,  0, 26,  0, }, // Solomon
    {  380, 1642,       0,  415970,       0,  0, 20,  0, }, // South_Georgia
    {  380, 1656,       0,  415990,       0,  0, 13,  0, }, // Suriname
    {  380, 1676,       0,  416003,       0,  0, 11,  0, }, // Syowa
    {  380, 1682,       0,  416014,       0,  0, 12,  0, }, // Tahiti
    {  380, 1689,  416026,  416036,  416054, 10, 18, 18, }, // Taipei
    {  380, 1696,       0,  416072,       0,  0, 13,  0, }, // Tajikistan
    {  380, 1724,       0,  416085,       0,  0, 12,  0, }, // Tokelau
    {  380, 1732,  416097,  416109,  416129, 12, 20, 18, }, // Tonga
    {  380, 1738,       0,  416147,       0,  0, 10,  0, }, // Truk
    {  380, 1750,  416157,  416172,  416195, 15, 23, 24, }, // Turkmenistan
    {  380, 1763,       0,  416219,       0,  0, 14,  0, }, // Tuvalu
    {  380, 1777,  416233,  416248,  416271, 15, 23, 21, }, // Uruguay
    {  380, 1792,  416292,  416309,  416334, 17, 25, 25, }, // Uzbekistan
    {  380, 1803,  416359,  416375,  416399, 16, 24, 22, }, // Vanuatu
    {  380, 1811,       0,  416421,       0,  0, 16,  0, }, // Venezuela
    {  380, 1821,  416437,  416454,  416479, 17, 25, 23, }, // Vladivostok
    {  380, 1833,  416502,  416517,  416540, 15, 23, 21, }, // Volgograd
    {  380, 1843,       0,  416442,       0,  0, 12,  0, }, // Vostok
    {  380, 1850,       0,  416561,       0,  0, 17,  0, }, // Wake
    {  380, 1855,       0,  416578,       0,  0, 26,  0, }, // Wallis
    {  380, 1862,  416604,  416617,  416638, 13, 21, 20, }, // Yakutsk
    {  380, 1870,  416658,  416677,  416704, 19, 27, 24, }, // Yekaterinburg
    {  380, 1892,       0,  416728,       0,  0, 10,  0, }, // Yukon
    {  381,  142,  416738,  416748,  416769, 10, 21, 18, }, // America_Central Kashmiri/Devanagari/India
    {  381,  158,  416787,  416796,  416816,  9, 20, 17, }, // America_Eastern
    {  381,  174,  416833,  416843,  416864, 10, 21, 18, }, // America_Mountain
    {  381,  191,  416882,  416892,  416913, 10, 21, 18, }, // America_Pacific
    {  381,  286,  416931,  416942,  416964, 11, 22, 19, }, // Atlantic
    {  381,  627,  416983,  416998,  417024, 15, 26, 19, }, // Europe_Central
    {  381,  642,  417043,  417058,  417084, 15, 26, 19, }, // Europe_Eastern
    {  381,  680,  417103,  417118,  417144, 15, 26, 19, }, // Europe_Western
    {  381,  788,       0,  417163,       0,  0, 15,  0, }, // GMT
    {  382,    5,       0,  417178,       0,  0, 16,  0, }, // Afghanistan Kazakh/Cyrillic/Kazakhstan
    {  382,   17,       0,  417194,       0,  0, 21,  0, }, // Africa_Central
    {  382,   32,       0,  417215,       0,  0, 19,  0, }, // Africa_Eastern
    {  382,   65,       0,  417234,       0,  0, 33,  0, }, // Africa_Southern
    {  382,   81,  417267,  417286,  417316, 19, 30, 25, }, // Africa_Western
    {  382,  107,  417341,  417354,  417378, 13, 24, 19, }, // Alaska
    {  382,  135,  417397,  417412,  417438, 15, 26, 21, }, // Amazon
    {  382,  142,  417459,  417491,  417534, 32, 43, 38, }, // America_Central
    {  382,  158,  417572,  417602,  417643, 30, 41, 36, }, // America_Eastern
    {  382,  174,  417679,  417707,  417746, 28, 39, 34, }, // America_Mountain
    {  382,  191,  417780,  417817,  417865, 37, 48, 43, }, // America_Pacific
    {  382,  214,  417908,  417919,  417941, 11, 22, 17, }, // Apia
    {  382,  232,  417958,  417978,  418009, 20, 31, 26, }, // Arabian
    {  382,  240,  418035,  418051,  418078, 16, 27, 22, }, // Argentina
    {  382,  250,  418100,  418122,  418155, 22, 33, 28, }, // Argentina_Western
    {  382,  268,  418183,  418197,  418222, 14, 25, 20, }, // Armenia
    {  382,  286,  418242,  418258,  418285, 16, 27, 22, }, // Atlantic
    {  382,  295,  418307,  418331,  418366, 24, 35, 30, }, // Australia_Central
    {  382,  313,  418396,  418426,  418467, 30, 41, 36, }, // Australia_CentralWestern
    {  382,  338,  418503,  418525,  418558, 22, 33, 28, }, // Australia_Eastern
    {  382,  356,  418586,  418608,  418641, 22, 33, 28, }, // Australia_Western
    {  382,  374,  418669,  418686,  418714, 17, 28, 23, }, // Azerbaijan
    {  382,  385,  418737,  418757,  418788, 20, 31, 26, }, // Azores
    {  382,  397,  418814,  418830,  418857, 16, 27, 22, }, // Bangladesh
    {  382,  415,       0,  418879,       0,  0, 12,  0, }, // Bhutan
    {  382,  422,       0,  418891,       0,  0, 14,  0, }, // Bolivia
    {  382,  437,  418905,  418920,  418946, 15, 26, 21, }, // Brasilia
    {  382,  454,       0,  418967,       0,  0, 24,  0, }, // Brunei
    {  382,  461,  418991,  419008,  419036, 17, 28, 23, }, // Cape_Verde
    {  382,  478,       0,  419059,       0,  0, 25,  0, }, // Chamorro
    {  382,  487,  419084,  419096,  419119, 12, 23, 18, }, // Chatham
    {  382,  495,  419137,  419148,  419170, 11, 22, 17, }, // Chile
    {  382,  501,  419187,  419199,  419222, 12, 23, 18, }, // China
    {  382,  507,       0,  419240,       0,  0, 25,  0, }, // Christmas
    {  382,  517,       0,  419265,       0,  0, 24,  0, }, // Cocos
    {  382,  523,  419289,  419304,  419330, 15, 26, 21, }, // Colombia
    {  382,  532,  419351,  419373,  419406, 22, 33, 37, }, // Cook
    {  382,  537,  419443,  419454,  419476, 11, 22, 17, }, // Cuba
    {  382,  548,       0,  419493,       0,  0, 13,  0, }, // Davis
    {  382,  564,       0,  419506,       0,  0, 21,  0, }, // DumontDUrville
    {  382,  601,       0,  419527,       0,  0, 18,  0, }, // East_Timor
    {  382,  612,  419545,  419563,  419592, 18, 29, 24, }, // Easter
    {  382,  619,       0,  419616,       0,  0, 14,  0, }, // Ecuador
    {  382,  627,  419630,  419651,  419683, 21, 32, 27, }, // Europe_Central
    {  382,  642,  419710,  419729,  419759, 19, 30, 25, }, // Europe_Eastern
    {  382,  657,       0,  419784,       0,  0, 24,  0, }, // Europe_Further_Eastern
    {  382,  680,  419808,  419827,  419857, 19, 30, 25, }, // Europe_Western
    {  382,  695,  419882,  419906,  419941, 24, 35, 30, }, // Falkland
    {  382,  704,  419971,  419983,  420006, 12, 23, 18, }, // Fiji
    {  382,  709,       0,  420024,       0,  0, 23,  0, }, // French_Guiana
    {  382,  723,       0,  420047,       0,  0, 49,  0, }, // French_Southern
    {  382,  746,       0,  420096,       0,  0, 16,  0, }, // Galapagos
    {  382,  756,       0,  420112,       0,  0, 13,  0, }, // Gambier
    {  382,  764,  420125,  420138,  420162, 13, 24, 19, }, // Georgia
    {  382,  772,       0,  420181,       0,  0, 26,  0, }, // Gilbert_Islands
    {  382,  788,       0,  420207,       0,  0, 14,  0, }, // GMT
    {  382,  830,  420221,  420244,  420278, 23, 34, 29, }, // Greenland_Eastern
    {  382,  848,  420307,  420330,  420364, 23, 34, 29, }, // Greenland_Western
    {  382,  871,       0,  420393,       0,  0, 32,  0, }, // Gulf
    {  382,  876,       0,  420425,       0,  0, 13,  0, }, // Guyana
    {  382,  883,  420438,  420470,  420513, 32, 43, 38, }, // Hawaii_Aleutian
    {  382,  899,  420551,  420565,  420590, 14, 25, 20, }, // Hong_Kong
    {  382,  909,  420610,  420621,  420643, 11, 22, 17, }, // Hovd
    {  382,  914,       0,  420660,       0,  0, 26,  0, }, // India
    {  382,  920,       0,  420686,       0,  0, 18,  0, }, // Indian_Ocean
    {  382,  933,       0,  420704,       0,  0, 16,  0, }, // Indochina
    {  382,  943,       0,  420720,       0,  0, 24,  0, }, // Indonesia_Central
    {  382,  961,       0,  420744,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  382,  979,       0,  420766,       0,  0, 22,  0, }, // Indonesia_Western
    {  382,  997,  420788,  420799,  420821, 11, 22, 17, }, // Iran
    {  382, 1008,  420838,  420852,  420877, 14, 25, 20, }, // Irkutsk
    {  382, 1016,  420897,  420911,  420936, 14, 25, 20, }, // Israel
    {  382, 1023,  420956,  420970,  420995, 14, 25, 20, }, // Japan
    {  382, 1047,       0,  421015,       0,  0, 16,  0, }, // Kazakhstan
    {  382, 1058,       0,  421031,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {  382, 1077,       0,  421053,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  382, 1106,  421075,  421087,  421110, 12, 23, 18, }, // Korea
    {  382, 1112,       0,  421128,       0,  0, 13,  0, }, // Kosrae
    {  382, 1119,  421141,  421158,  421186, 17, 28, 23, }, // Krasnoyarsk
    {  382, 1151,       0,  421209,       0,  0, 17,  0, }, // Kyrgystan
    {  382, 1175,       0,  421226,       0,  0, 20,  0, }, // Line_Islands
    {  382, 1188,  421246,  421261,  421287, 15, 26, 21, }, // Lord_Howe
    {  382, 1204,  421308,  421322,  421347, 14, 25, 20, }, // Magadan
    {  382, 1219,       0,  421367,       0,  0, 15,  0, }, // Malaysia
    {  382, 1228,       0,  421382,       0,  0, 23,  0, }, // Maldives
    {  382, 1237,       0,  421405,       0,  0, 22,  0, }, // Marquesas
    {  382, 1247,       0,  421427,       0,  0, 23,  0, }, // Marshall_Islands
    {  382, 1264,  421450,  421465,  421491, 15, 26, 21, }, // Mauritius
    {  382, 1274,       0,  421512,       0,  0, 13,  0, }, // Mawson
    {  382, 1281,  421525,  421551,  421588, 26, 37, 32, }, // Mexico_Pacific
    {  382, 1296,  421620,  421636,  421663, 16, 27, 22, }, // Mongolia
    {  382, 1305,  421685,  421698,  421722, 13, 24, 19, }, // Moscow
    {  382, 1312,       0,  421741,       0,  0, 13,  0, }, // Myanmar
    {  382, 1320,       0,  421754,       0,  0, 12,  0, }, // Nauru
    {  382, 1326,       0,  421766,       0,  0, 12,  0, }, // Nepal
    {  382, 1332,  421778,  421799,  421831, 21, 32, 27, }, // New_Caledonia
    {  382, 1346,  421858,  421878,  421909, 20, 31, 26, }, // New_Zealand
    {  382, 1358,  421935,  421954,  421984, 19, 30, 25, }, // Newfoundland
    {  382, 1371,       0,  422009,       0,  0, 11,  0, }, // Niue
    {  382, 1376,  422020,  422040,  422071, 20, 31, 26, }, // Norfolk
    {  382, 1384,  422097,  422123,  422160, 26, 37, 32, }, // Noronha
    {  382, 1406,  422192,  422208,  422235, 16, 27, 22, }, // Novosibirsk
    {  382, 1418,  422257,  422268,  422290, 11, 22, 17, }, // Omsk
    {  382, 1428,  422307,  422322,  422348, 15, 26, 21, }, // Pakistan
    {  382, 1437,       0,  422369,       0,  0, 12,  0, }, // Palau
    {  382, 1443,       0,  422381,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  382, 1460,  422407,  422422,  422448, 15, 26, 21, }, // Paraguay
    {  382, 1469,  422469,  422480,  422502, 11, 22, 17, }, // Peru
    {  382, 1474,  422519,  422543,  422578, 24, 35, 30, }, // Philippines
    {  382, 1486,       0,  422608,       0,  0, 22,  0, }, // Phoenix_Islands
    {  382, 1502,  422630,  422658,  422697, 28, 39, 34, }, // Pierre_Miquelon
    {  382, 1518,       0,  422731,       0,  0, 14,  0, }, // Pitcairn
    {  382, 1527,       0,  422745,       0,  0, 13,  0, }, // Ponape
    {  382, 1534,       0,  422758,       0,  0, 14,  0, }, // Pyongyang
    {  382, 1554,       0,  422772,       0,  0, 14,  0, }, // Reunion
    {  382, 1562,       0,  422786,       0,  0, 13,  0, }, // Rothera
    {  382, 1570,  422799,  422813,  422838, 14, 25, 20, }, // Sakhalin
    {  382, 1596,  422858,  422870,  422893, 12, 23, 18, }, // Samoa
    {  382, 1602,       0,  422911,       0,  0, 23,  0, }, // Seychelles
    {  382, 1624,       0,  422934,       0,  0, 26,  0, }, // Singapore
    {  382, 1634,       0,  422960,       0,  0, 23,  0, }, // Solomon
    {  382, 1642,       0,  422983,       0,  0, 23,  0, }, // South_Georgia
    {  382, 1656,       0,  423006,       0,  0, 14,  0, }, // Suriname
    {  382, 1676,       0,  423020,       0,  0, 11,  0, }, // Syowa
    {  382, 1682,       0,  423031,       0,  0, 12,  0, }, // Tahiti
    {  382, 1689,  423043,  423056,  423080, 13, 24, 19, }, // Taipei
    {  382, 1696,       0,  423099,       0,  0, 16,  0, }, // Tajikistan
    {  382, 1724,       0,  423115,       0,  0, 14,  0, }, // Tokelau
    {  382, 1732,  423129,  423141,  423164, 12, 23, 18, }, // Tonga
    {  382, 1738,       0,  423182,       0,  0, 11,  0, }, // Truk
    {  382, 1750,  423193,  423212,  423242, 19, 30, 25, }, // Turkmenistan
    {  382, 1763,       0,  423267,       0,  0, 13,  0, }, // Tuvalu
    {  382, 1777,  423280,  423294,  423319, 14, 25, 20, }, // Uruguay
    {  382, 1792,  423339,  423355,  423382, 16, 27, 22, }, // Uzbekistan
    {  382, 1803,  423404,  423418,  423443, 14, 25, 20, }, // Vanuatu
    {  382, 1811,       0,  423463,       0,  0, 16,  0, }, // Venezuela
    {  382, 1821,  423479,  423497,  423526, 18, 29, 24, }, // Vladivostok
    {  382, 1833,  423550,  423566,  423593, 16, 27, 22, }, // Volgograd
    {  382, 1843,       0,  423615,       0,  0, 13,  0, }, // Vostok
    {  382, 1850,       0,  423628,       0,  0, 17,  0, }, // Wake
    {  382, 1855,       0,  423645,       0,  0, 25,  0, }, // Wallis
    {  382, 1862,  423670,  423683,  423707, 13, 24, 19, }, // Yakutsk
    {  382, 1870,  423726,  423745,  423775, 19, 30, 25, }, // Yekaterinburg
    {  382, 1892,       0,  423800,       0,  0, 11,  0, }, // Yukon
    {  383,    5,       0,  423811,       0,  0, 16,  0, }, // Afghanistan Kazakh/Arabic/China
    {  383,   17,       0,  423827,       0,  0, 21,  0, }, // Africa_Central
    {  383,   32,       0,  423848,       0,  0, 19,  0, }, // Africa_Eastern
    {  383,   65,       0,  423867,       0,  0, 33,  0, }, // Africa_Southern
    {  383,   81,  423900,  423919,  423949, 19, 30, 25, }, // Africa_Western
    {  383,  107,  423974,  423987,  424011, 13, 24, 19, }, // Alaska
    {  383,  128,  424030,  424043,  424067, 13, 24, 19, }, // Almaty
    {  383,  135,  424086,  424099,  424123, 13, 24, 19, }, // Amazon
    {  383,  142,  424142,  424174,  424217, 32, 43, 37, }, // America_Central
    {  383,  158,  424254,  424284,  424325, 30, 41, 36, }, // America_Eastern
    {  383,  174,  424361,  424389,  424428, 28, 39, 34, }, // America_Mountain
    {  383,  191,  424462,  424499,  424547, 37, 48, 43, }, // America_Pacific
    {  383,  214,  424590,  424601,  424623, 11, 22, 17, }, // Apia
    {  383,  219,  424640,  424652,  424675, 12, 23, 18, }, // Aqtau
    {  383,  225,  424693,  424706,  424730, 13, 24, 19, }, // Aqtobe
    {  383,  232,  424749,  424769,  424800, 20, 31, 26, }, // Arabian
    {  383,  240,  424826,  424842,  424869, 16, 27, 22, }, // Argentina
    {  383,  250,  424891,  424913,  424946, 22, 33, 28, }, // Argentina_Western
    {  383,  268,  424974,  424988,  425013, 14, 25, 20, }, // Armenia
    {  383,  286,  425033,  425049,  425076, 16, 27, 22, }, // Atlantic
    {  383,  295,  425098,  425122,  425157, 24, 35, 30, }, // Australia_Central
    {  383,  313,  425187,  425217,  425258, 30, 41, 36, }, // Australia_CentralWestern
    {  383,  338,  425294,  425316,  425349, 22, 33, 28, }, // Australia_Eastern
    {  383,  356,  425377,  425399,  425432, 22, 33, 28, }, // Australia_Western
    {  383,  374,  425460,  425478,  425507, 18, 29, 24, }, // Azerbaijan
    {  383,  385,  425531,  425551,  425582, 20, 31, 26, }, // Azores
    {  383,  397,  425608,  425624,  425651, 16, 27, 22, }, // Bangladesh
    {  383,  415,       0,  425673,       0,  0, 12,  0, }, // Bhutan
    {  383,  422,       0,  425685,       0,  0, 14,  0, }, // Bolivia
    {  383,  437,  425699,  425714,  425740, 15, 26, 21, }, // Brasilia
    {  383,  454,       0,  425761,       0,  0, 24,  0, }, // Brunei
    {  383,  461,  425785,  425802,  425830, 17, 28, 23, }, // Cape_Verde
    {  383,  478,       0,  425853,       0,  0, 25,  0, }, // Chamorro
    {  383,  487,  425878,  425890,  425913, 12, 23, 18, }, // Chatham
    {  383,  495,  425931,  425942,  425964, 11, 22, 17, }, // Chile
    {  383,  501,  425981,  425993,  426016, 12, 23, 18, }, // China
    {  383,  507,       0,  426034,       0,  0, 24,  0, }, // Christmas
    {  383,  517,       0,  426058,       0,  0, 24,  0, }, // Cocos
    {  383,  523,  426082,  426097,  426123, 15, 26, 21, }, // Colombia
    {  383,  532,  426144,  426166,  426199, 22, 33, 28, }, // Cook
    {  383,  537,  426227,  426238,  426260, 11, 22, 17, }, // Cuba
    {  383,  548,       0,  426277,       0,  0, 13,  0, }, // Davis
    {  383,  564,       0,  426290,       0,  0, 21,  0, }, // DumontDUrville
    {  383,  601,       0,  426311,       0,  0, 18,  0, }, // East_Timor
    {  383,  612,  426329,  426347,  426376, 18, 29, 24, }, // Easter
    {  383,  619,       0,  426400,       0,  0, 14,  0, }, // Ecuador
    {  383,  627,  426414,  426435,  426467, 21, 32, 27, }, // Europe_Central
    {  383,  642,  426494,  426513,  426543, 19, 30, 25, }, // Europe_Eastern
    {  383,  657,       0,  426568,       0,  0, 24,  0, }, // Europe_Further_Eastern
    {  383,  680,  426592,  426611,  426641, 19, 30, 25, }, // Europe_Western
    {  383,  695,  426666,  426690,  426725, 24, 35, 30, }, // Falkland
    {  383,  704,  426755,  426766,  426788, 11, 22, 17, }, // Fiji
    {  383,  709,       0,  426805,       0,  0, 24,  0, }, // French_Guiana
    {  383,  723,       0,  426829,       0,  0, 50,  0, }, // French_Southern
    {  383,  746,       0,  426879,       0,  0, 16,  0, }, // Galapagos
    {  383,  756,       0,  426895,       0,  0, 12,  0, }, // Gambier
    {  383,  764,  426907,  426920,  426944, 13, 24, 19, }, // Georgia
    {  383,  772,       0,  426963,       0,  0, 26,  0, }, // Gilbert_Islands
    {  383,  788,       0,  426989,       0,  0, 14,  0, }, // GMT
    {  383,  830,  427003,  427026,  427060, 23, 34, 29, }, // Greenland_Eastern
    {  383,  848,  427089,  427112,  427146, 23, 34, 29, }, // Greenland_Western
    {  383,  871,       0,  427175,       0,  0, 32,  0, }, // Gulf
    {  383,  876,       0,  427207,       0,  0, 13,  0, }, // Guyana
    {  383,  883,  427220,  427252,  427295, 32, 43, 38, }, // Hawaii_Aleutian
    {  383,  899,  427333,  427346,  427370, 13, 24, 19, }, // Hong_Kong
    {  383,  909,  427389,  427400,  427422, 11, 22, 17, }, // Hovd
    {  383,  914,       0,  427439,       0,  0, 27,  0, }, // India
    {  383,  920,       0,  427466,       0,  0, 19,  0, }, // Indian_Ocean
    {  383,  933,       0,  427485,       0,  0, 18,  0, }, // Indochina
    {  383,  943,       0,  427503,       0,  0, 24,  0, }, // Indonesia_Central
    {  383,  961,       0,  427527,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  383,  979,       0,  427549,       0,  0, 22,  0, }, // Indonesia_Western
    {  383,  997,  427571,  427582,  427604, 11, 22, 17, }, // Iran
    {  383, 1008,  427621,  427635,  427660, 14, 25, 20, }, // Irkutsk
    {  383, 1016,  427680,  427693,  427717, 13, 24, 19, }, // Israel
    {  383, 1023,  427736,  427750,  427775, 14, 25, 20, }, // Japan
    {  383, 1047,       0,  427795,       0,  0, 16,  0, }, // Kazakhstan
    {  383, 1058,       0,  427811,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {  383, 1077,       0,  427833,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  383, 1106,  427855,  427868,  427892, 13, 24, 19, }, // Korea
    {  383, 1112,       0,  427911,       0,  0, 13,  0, }, // Kosrae
    {  383, 1119,  427924,  427942,  427971, 18, 29, 24, }, // Krasnoyarsk
    {  383, 1151,       0,  427995,       0,  0, 17,  0, }, // Kyrgystan
    {  383, 1175,       0,  428012,       0,  0, 20,  0, }, // Line_Islands
    {  383, 1188,  428032,  428047,  428073, 15, 26, 21, }, // Lord_Howe
    {  383, 1204,  428094,  428108,  428133, 14, 25, 20, }, // Magadan
    {  383, 1219,       0,  428153,       0,  0, 15,  0, }, // Malaysia
    {  383, 1228,       0,  428168,       0,  0, 22,  0, }, // Maldives
    {  383, 1237,       0,  428190,       0,  0, 22,  0, }, // Marquesas
    {  383, 1247,       0,  428212,       0,  0, 22,  0, }, // Marshall_Islands
    {  383, 1264,  428234,  428249,  428275, 15, 26, 21, }, // Mauritius
    {  383, 1274,       0,  428296,       0,  0, 13,  0, }, // Mawson
    {  383, 1281,  428309,  428335,  428372, 26, 37, 32, }, // Mexico_Pacific
    {  383, 1296,  428404,  428420,  428447, 16, 27, 22, }, // Mongolia
    {  383, 1305,  428469,  428482,  428506, 13, 24, 19, }, // Moscow
    {  383, 1312,       0,  428525,       0,  0, 13,  0, }, // Myanmar
    {  383, 1320,       0,  428538,       0,  0, 12,  0, }, // Nauru
    {  383, 1326,       0,  428550,       0,  0, 12,  0, }, // Nepal
    {  383, 1332,  428562,  428583,  428615, 21, 32, 27, }, // New_Caledonia
    {  383, 1346,  428642,  428662,  428693, 20, 31, 26, }, // New_Zealand
    {  383, 1358,  428719,  428738,  428768, 19, 30, 25, }, // Newfoundland
    {  383, 1371,       0,  428793,       0,  0, 11,  0, }, // Niue
    {  383, 1376,  428804,  428824,  428855, 20, 31, 26, }, // Norfolk
    {  383, 1384,  428881,  428907,  428944, 26, 37, 32, }, // Noronha
    {  383, 1406,  428976,  428992,  429019, 16, 27, 22, }, // Novosibirsk
    {  383, 1418,  429041,  429052,  429074, 11, 22, 17, }, // Omsk
    {  383, 1428,  429091,  429106,  429132, 15, 26, 21, }, // Pakistan
    {  383, 1437,       0,  429153,       0,  0, 12,  0, }, // Palau
    {  383, 1443,       0,  429165,       0,  0, 27,  0, }, // Papua_New_Guinea
    {  383, 1460,  429192,  429207,  429233, 15, 26, 21, }, // Paraguay
    {  383, 1469,  429254,  429265,  429287, 11, 22, 17, }, // Peru
    {  383, 1474,  429304,  429327,  429361, 23, 34, 29, }, // Philippines
    {  383, 1486,       0,  429390,       0,  0, 22,  0, }, // Phoenix_Islands
    {  383, 1502,  429412,  429439,  429477, 27, 38, 33, }, // Pierre_Miquelon
    {  383, 1518,       0,  429510,       0,  0, 14,  0, }, // Pitcairn
    {  383, 1527,       0,  429524,       0,  0, 13,  0, }, // Ponape
    {  383, 1534,       0,  429537,       0,  0, 14,  0, }, // Pyongyang
    {  383, 1544,  429551,  429567,  429594, 16, 27, 22, }, // Qyzylorda
    {  383, 1554,       0,  429616,       0,  0, 14,  0, }, // Reunion
    {  383, 1562,       0,  429630,       0,  0, 12,  0, }, // Rothera
    {  383, 1570,  429642,  429656,  429681, 14, 25, 20, }, // Sakhalin
    {  383, 1596,  429701,  429713,  429736, 12, 23, 18, }, // Samoa
    {  383, 1602,       0,  429754,       0,  0, 22,  0, }, // Seychelles
    {  383, 1624,       0,  429776,       0,  0, 26,  0, }, // Singapore
    {  383, 1634,       0,  429802,       0,  0, 23,  0, }, // Solomon
    {  383, 1642,       0,  429825,       0,  0, 23,  0, }, // South_Georgia
    {  383, 1656,       0,  429848,       0,  0, 14,  0, }, // Suriname
    {  383, 1676,       0,  429862,       0,  0, 11,  0, }, // Syowa
    {  383, 1682,       0,  429873,       0,  0, 12,  0, }, // Tahiti
    {  383, 1689,  429885,  429898,  429922, 13, 24, 19, }, // Taipei
    {  383, 1696,       0,  429941,       0,  0, 16,  0, }, // Tajikistan
    {  383, 1724,       0,  429957,       0,  0, 14,  0, }, // Tokelau
    {  383, 1732,  429971,  429983,  430006, 12, 23, 18, }, // Tonga
    {  383, 1738,       0,  430024,       0,  0, 11,  0, }, // Truk
    {  383, 1750,  430035,  430054,  430084, 19, 30, 25, }, // Turkmenistan
    {  383, 1763,       0,  430109,       0,  0, 13,  0, }, // Tuvalu
    {  383, 1777,  430122,  430136,  430161, 14, 25, 20, }, // Uruguay
    {  383, 1792,  430181,  430197,  430224, 16, 27, 22, }, // Uzbekistan
    {  383, 1803,  430246,  430260,  430285, 14, 25, 20, }, // Vanuatu
    {  383, 1811,       0,  430305,       0,  0, 16,  0, }, // Venezuela
    {  383, 1821,  430321,  430339,  430368, 18, 29, 24, }, // Vladivostok
    {  383, 1833,  430392,  430408,  430435, 16, 27, 22, }, // Volgograd
    {  383, 1843,       0,  430326,       0,  0, 13,  0, }, // Vostok
    {  383, 1850,       0,  430457,       0,  0, 17,  0, }, // Wake
    {  383, 1855,       0,  430474,       0,  0, 25,  0, }, // Wallis
    {  383, 1862,  430499,  430513,  430538, 14, 25, 20, }, // Yakutsk
    {  383, 1870,  430558,  430577,  430607, 19, 30, 25, }, // Yekaterinburg
    {  383, 1892,       0,  430632,       0,  0, 12,  0, }, // Yukon
    {  385,    5,       0,  430644,       0,  0, 24,  0, }, // Afghanistan Khmer/Khmer/Cambodia
    {  385,   17,       0,  430668,       0,  0, 24,  0, }, // Africa_Central
    {  385,   32,       0,  430692,       0,  0, 25,  0, }, // Africa_Eastern
    {  385,   65,       0,  430717,       0,  0, 27,  0, }, // Africa_Southern
    {  385,   81,  430744,  430769,  430803, 25, 34, 39, }, // Africa_Western
    {  385,  107,  430842,  430858,  430883, 16, 25, 26, }, // Alaska
    {  385,  135,  430909,  430927,  430953, 18, 26, 28, }, // Amazon
    {  385,  142,  430981,  431019,  431065, 38, 46, 45, }, // America_Central
    {  385,  158,  431110,  431142,  431183, 32, 41, 40, }, // America_Eastern
    {  385,  174,  431223,  431264,  431313, 41, 49, 48, }, // America_Mountain
    {  385,  191,  431361,  431385,  431417, 24, 32, 31, }, // America_Pacific
    {  385,  214,  431448,  431462,  431484, 14, 22, 23, }, // Apia
    {  385,  232,  431507,  431522,  431546, 15, 24, 24, }, // Arabian
    {  385,  240,  431570,  431588,  431614, 18, 26, 28, }, // Argentina
    {  385,  250,  431642,  431669,  431704, 27, 35, 37, }, // Argentina_Western
    {  385,  268,  431741,  431755,  431778, 14, 23, 26, }, // Armenia
    {  385,  286,  431804,  431822,  431849, 18, 27, 27, }, // Atlantic
    {  385,  295,  431876,  431900,  431934, 24, 34, 36, }, // Australia_Central
    {  385,  313,  431970,  432012,  432061, 42, 49, 50, }, // Australia_CentralWestern
    {  385,  338,  432111,  432137,  432172, 26, 35, 35, }, // Australia_Eastern
    {  385,  356,  432207,  432235,  432271, 28, 36, 35, }, // Australia_Western
    {  385,  374,  432306,  432326,  432355, 20, 29, 32, }, // Azerbaijan
    {  385,  385,  432387,  432401,  432424, 14, 23, 26, }, // Azores
    {  385,  397,  432450,  432468,  432495, 18, 27, 30, }, // Bangladesh
    {  385,  415,       0,  432525,       0,  0, 12,  0, }, // Bhutan
    {  385,  422,       0,  432537,       0,  0, 14,  0, }, // Bolivia
    {  385,  437,  432551,  432571,  432600, 20, 29, 33, }, // Brasilia
    {  385,  454,       0,  432633,       0,  0, 22,  0, }, // Brunei
    {  385,  461,  432655,  432670,  432694, 15, 24, 27, }, // Cape_Verde
    {  385,  478,       0,  432721,       0,  0, 25,  0, }, // Chamorro
    {  385,  487,  432746,  432759,  432781, 13, 22, 22, }, // Chatham
    {  385,  495,  432803,  432814,  432833, 11, 19, 21, }, // Chile
    {  385,  501,  432854,  432865,  432885, 11, 20, 20, }, // China
    {  385,  507,       0,  432905,       0,  0, 21,  0, }, // Christmas
    {  385,  517,       0,  432926,       0,  0, 23,  0, }, // Cocos
    {  385,  523,  432949,  432965,  432990, 16, 25, 28, }, // Colombia
    {  385,  532,  433018,  433039,  433069, 21, 30, 45, }, // Cook
    {  385,  537,  433114,  433127,  433149, 13, 22, 22, }, // Cuba
    {  385,  548,       0,  433171,       0,  0, 13,  0, }, // Davis
    {  385,  564,       0,  433184,       0,  0, 24,  0, }, // DumontDUrville
    {  385,  601,       0,  433208,       0,  0, 22,  0, }, // East_Timor
    {  385,  612,  433230,  433247,  433272, 17, 25, 27, }, // Easter
    {  385,  619,       0,  433299,       0,  0, 17,  0, }, // Ecuador
    {  385,  627,  433316,  433337,  433367, 21, 30, 33, }, // Europe_Central
    {  385,  642,  433400,  433424,  433458, 24, 34, 36, }, // Europe_Eastern
    {  385,  657,       0,  433494,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  385,  680,  433525,  433547,  433578, 22, 31, 34, }, // Europe_Western
    {  385,  695,  433612,  433638,  433673, 26, 35, 38, }, // Falkland
    {  385,  704,  433711,  433725,  433748, 14, 23, 28, }, // Fiji
    {  385,  709,       0,  433776,       0,  0, 23,  0, }, // French_Guiana
    {  385,  723,       0,  433799,       0,  0, 36,  0, }, // French_Southern
    {  385,  746,       0,  433835,       0,  0, 18,  0, }, // Galapagos
    {  385,  756,       0,  433853,       0,  0, 13,  0, }, // Gambier
    {  385,  764,  433866,  433883,  433909, 17, 26, 30, }, // Georgia
    {  385,  772,       0,  433939,       0,  0, 18,  0, }, // Gilbert_Islands
    {  385,  788,       0,  433957,       0,  0, 14,  0, }, // GMT
    {  385,  830,  433971,  433998,  434036, 27, 38, 37, }, // Greenland_Eastern
    {  385,  848,  434073,  434095,  434125, 22, 30, 32, }, // Greenland_Western
    {  385,  871,       0,  434157,       0,  0, 20,  0, }, // Gulf
    {  385,  876,       0,  433776,       0,  0, 16,  0, }, // Guyana
    {  385,  883,  434177,  434202,  434236, 25, 34, 33, }, // Hawaii_Aleutian
    {  385,  899,  434269,  434283,  434306, 14, 23, 26, }, // Hong_Kong
    {  385,  909,  434332,  434343,  434362, 11, 19, 23, }, // Hovd
    {  385,  914,       0,  434385,       0,  0, 21,  0, }, // India
    {  385,  920,       0,  434406,       0,  0, 23,  0, }, // Indian_Ocean
    {  385,  933,       0,  434429,       0,  0, 16,  0, }, // Indochina
    {  385,  943,       0,  434445,       0,  0, 27,  0, }, // Indonesia_Central
    {  385,  961,       0,  434472,       0,  0, 27,  0, }, // Indonesia_Eastern
    {  385,  979,       0,  434499,       0,  0, 27,  0, }, // Indonesia_Western
    {  385,  997,  434526,  434541,  434565, 15, 24, 24, }, // Iran
    {  385, 1008,  434589,  434606,  434631, 17, 25, 27, }, // Irkutsk
    {  385, 1016,  434658,  434676,  434703, 18, 27, 27, }, // Israel
    {  385, 1023,  434730,  434743,  434765, 13, 22, 21, }, // Japan
    {  385, 1047,       0,  434786,       0,  0, 22,  0, }, // Kazakhstan
    {  385, 1058,       0,  434808,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  385, 1077,       0,  434835,       0,  0, 31,  0, }, // Kazakhstan_Western
    {  385, 1106,  434866,  434879,  434901, 13, 22, 22, }, // Korea
    {  385, 1112,       0,  434923,       0,  0, 14,  0, }, // Kosrae
    {  385, 1119,  434937,  434955,  434982, 18, 27, 30, }, // Krasnoyarsk
    {  385, 1151,       0,  435012,       0,  0, 21,  0, }, // Kyrgystan
    {  385, 1175,       0,  435033,       0,  0, 15,  0, }, // Line_Islands
    {  385, 1188,  435048,  435060,  435082, 12, 22, 21, }, // Lord_Howe
    {  385, 1204,  435103,  435121,  435148, 18, 27, 30, }, // Magadan
    {  385, 1219,       0,  435178,       0,  0, 16,  0, }, // Malaysia
    {  385, 1228,       0,  435194,       0,  0, 16,  0, }, // Maldives
    {  385, 1237,       0,  435210,       0,  0, 21,  0, }, // Marquesas
    {  385, 1247,       0,  435231,       0,  0, 14,  0, }, // Marshall_Islands
    {  385, 1264,  435245,  435259,  435282, 14, 23, 24, }, // Mauritius
    {  385, 1274,       0,  435306,       0,  0, 16,  0, }, // Mawson
    {  385, 1281,  435322,  435350,  435386, 28, 36, 37, }, // Mexico_Pacific
    {  385, 1296,  435423,  435442,  435470, 19, 28, 31, }, // Mongolia
    {  385, 1305,  435501,  435515,  435538, 14, 23, 26, }, // Moscow
    {  385, 1312,       0,  435564,       0,  0, 18,  0, }, // Myanmar
    {  385, 1320,       0,  435582,       0,  0, 12,  0, }, // Nauru
    {  385, 1326,       0,  435594,       0,  0, 13,  0, }, // Nepal
    {  385, 1332,  435607,  435628,  435657, 21, 29, 31, }, // New_Caledonia
    {  385, 1346,  435688,  435706,  435733, 18, 27, 27, }, // New_Zealand
    {  385, 1358,  435760,  435779,  435809, 19, 30, 27, }, // Newfoundland
    {  385, 1371,       0,  435836,       0,  0, 12,  0, }, // Niue
    {  385, 1376,  435848,  435868,  435893, 20, 25, 27, }, // Norfolk
    {  385, 1384,  435920,  435949,  435986, 29, 37, 39, }, // Noronha
    {  385, 1406,  436025,  436045,  436074, 20, 29, 32, }, // Novosibirsk
    {  385, 1418,  436106,  436117,  436137, 11, 20, 23, }, // Omsk
    {  385, 1428,  436160,  436178,  436205, 18, 27, 30, }, // Pakistan
    {  385, 1437,       0,  436235,       0,  0, 12,  0, }, // Palau
    {  385, 1443,       0,  436247,       0,  0, 29,  0, }, // Papua_New_Guinea
    {  385, 1460,  436276,  436295,  436323, 19, 28, 30, }, // Paraguay
    {  385, 1469,  436353,  436366,  436388, 13, 22, 25, }, // Peru
    {  385, 1474,  436413,  436430,  436456, 17, 26, 30, }, // Philippines
    {  385, 1486,       0,  436486,       0,  0, 17,  0, }, // Phoenix_Islands
    {  385, 1502,  436503,  436534,  436572, 31, 38, 38, }, // Pierre_Miquelon
    {  385, 1518,       0,  436610,       0,  0, 13,  0, }, // Pitcairn
    {  385, 1527,       0,  436623,       0,  0, 15,  0, }, // Ponape
    {  385, 1534,       0,  436638,       0,  0, 15,  0, }, // Pyongyang
    {  385, 1554,       0,  436653,       0,  0, 15,  0, }, // Reunion
    {  385, 1562,       0,  436668,       0,  0, 16,  0, }, // Rothera
    {  385, 1570,  436684,  436701,  436727, 17, 26, 29, }, // Sakhalin
    {  385, 1596,  436756,  436769,  436790, 13, 21, 23, }, // Samoa
    {  385, 1602,       0,  436813,       0,  0, 15,  0, }, // Seychelles
    {  385, 1624,       0,  436828,       0,  0, 17,  0, }, // Singapore
    {  385, 1634,       0,  436845,       0,  0, 20,  0, }, // Solomon
    {  385, 1642,       0,  436865,       0,  0, 21,  0, }, // South_Georgia
    {  385, 1656,       0,  436886,       0,  0, 15,  0, }, // Suriname
    {  385, 1676,       0,  436901,       0,  0, 16,  0, }, // Syowa
    {  385, 1682,       0,  436917,       0,  0, 14,  0, }, // Tahiti
    {  385, 1689,  436931,  436944,  436966, 13, 22, 22, }, // Taipei
    {  385, 1696,       0,  436988,       0,  0, 17,  0, }, // Tajikistan
    {  385, 1724,       0,  437005,       0,  0, 14,  0, }, // Tokelau
    {  385, 1732,  437019,  437034,  437059, 15, 25, 27, }, // Tonga
    {  385, 1738,       0,  437086,       0,  0, 13,  0, }, // Truk
    {  385, 1750,  437099,  437120,  437149, 21, 29, 32, }, // Turkmenistan
    {  385, 1763,       0,  437181,       0,  0, 16,  0, }, // Tuvalu
    {  385, 1777,  437197,  437216,  437244, 19, 28, 32, }, // Uruguay
    {  385, 1792,  437276,  437297,  437327, 21, 30, 33, }, // Uzbekistan
    {  385, 1803,  437360,  437375,  437400, 15, 25, 27, }, // Vanuatu
    {  385, 1811,       0,  437427,       0,  0, 21,  0, }, // Venezuela
    {  385, 1821,  437448,  437471,  437503, 23, 32, 35, }, // Vladivostok
    {  385, 1833,  437538,  437558,  437587, 20, 29, 32, }, // Volgograd
    {  385, 1843,       0,  437619,       0,  0, 16,  0, }, // Vostok
    {  385, 1850,       0,  437635,       0,  0, 14,  0, }, // Wake
    {  385, 1855,       0,  437649,       0,  0, 27,  0, }, // Wallis
    {  385, 1862,  437676,  437695,  437723, 19, 28, 31, }, // Yakutsk
    {  385, 1870,  437754,  437776,  437807, 22, 31, 35, }, // Yekaterinburg
    {  385, 1892,       0,  437842,       0,  0, 11,  0, }, // Yukon
    {  388,  788,       0,    2791,       0,  0, 19,  0, }, // GMT Kinyarwanda/Latin/Rwanda
    {  389,    5,       0,  437853,       0,  0, 15,  0, }, // Afghanistan Konkani/Devanagari/India
    {  389,   17,       0,  437868,       0,  0, 16,  0, }, // Africa_Central
    {  389,   32,       0,  437884,       0,  0, 17,  0, }, // Africa_Eastern
    {  389,   65,       0,  437901,       0,  0, 27,  0, }, // Africa_Southern
    {  389,   81,  437928,  437946,  437973, 18, 27, 26, }, // Africa_Western
    {  389,  107,  437999,  438010,  438030, 11, 20, 18, }, // Alaska
    {  389,  135,  438048,  438058,  438077, 10, 19, 18, }, // Amazon
    {  389,  142,  438095,  438103,  438120,  8, 17, 15, }, // America_Central
    {  389,  158,  438135,  438144,  438162,  9, 18, 16, }, // America_Eastern
    {  389,  174,  438178,  438189,  438209, 11, 20, 18, }, // America_Mountain
    {  389,  191,  438227,  438238,  438258, 11, 20, 18, }, // America_Pacific
    {  389,  214,  438276,  438285,  438303,  9, 18, 16, }, // Apia
    {  389,  232,  438319,  438327,  438344,  8, 17, 15, }, // Arabian
    {  389,  240,  438359,  438373,  438396, 14, 23, 22, }, // Argentina
    {  389,  250,  438418,  438439,  438469, 21, 30, 29, }, // Argentina_Western
    {  389,  268,  438498,  438511,  438533, 13, 22, 21, }, // Armenia
    {  389,  286,  438554,  438566,  438587, 12, 21, 19, }, // Atlantic
    {  389,  295,  438606,  438626,  438655, 20, 29, 27, }, // Australia_Central
    {  389,  313,  438682,  438709,  438745, 27, 36, 34, }, // Australia_CentralWestern
    {  389,  338,  438779,  438800,  438830, 21, 30, 28, }, // Australia_Eastern
    {  389,  356,  438858,  438880,  438911, 22, 31, 29, }, // Australia_Western
    {  389,  374,  438940,  438952,  438973, 12, 21, 20, }, // Azerbaijan
    {  389,  385,  438993,  438999,  439018,  6, 19, 18, }, // Azores
    {  389,  397,  439036,  439049,  439071, 13, 22, 21, }, // Bangladesh
    {  389,  415,       0,  439092,       0,  0,  5,  0, }, // Bhutan
    {  389,  422,       0,  439097,       0,  0, 14,  0, }, // Bolivia
    {  389,  437,  439111,  439125,  439148, 14, 23, 22, }, // Brasilia
    {  389,  454,       0,  439170,       0,  0, 22,  0, }, // Brunei
    {  389,  461,  439192,  439205,  439227, 13, 22, 21, }, // Cape_Verde
    {  389,  478,       0,  439248,       0,  0, 19,  0, }, // Chamorro
    {  389,  487,  439267,  439275,  439292,  8, 17, 15, }, // Chatham
    {  389,  495,  439307,  439315,  439332,  8, 17, 16, }, // Chile
    {  389,  501,  439348,  439355,  439371,  7, 16, 14, }, // China
    {  389,  507,       0,  439385,       0,  0, 17,  0, }, // Christmas
    {  389,  517,       0,  439402,       0,  0, 15,  0, }, // Cocos
    {  389,  523,  439417,  439429,  439450, 12, 21, 20, }, // Colombia
    {  389,  532,  439470,  439483,  439505, 13, 22, 26, }, // Cook
    {  389,  537,  439531,  439541,  439560, 10, 19, 17, }, // Cuba
    {  389,  548,       0,  439577,       0,  0, 11,  0, }, // Davis
    {  389,  564,       0,  439588,       0,  0, 18,  0, }, // DumontDUrville
    {  389,  601,       0,  439606,       0,  0, 15,  0, }, // East_Timor
    {  389,  612,  439621,  439636,  439660, 15, 24, 23, }, // Easter
    {  389,  619,       0,  439683,       0,  0, 12,  0, }, // Ecuador
    {  389,  627,  439695,  439712,  439738, 17, 26, 25, }, // Europe_Central
    {  389,  642,  439763,  439781,  439808, 18, 27, 26, }, // Europe_Eastern
    {  389,  657,       0,  439834,       0,  0, 23,  0, }, // Europe_Further_Eastern
    {  389,  680,  439857,  439876,  439904, 19, 28, 27, }, // Europe_Western
    {  389,  695,  439931,  439949,  439976, 18, 27, 26, }, // Falkland
    {  389,  704,  440002,  440010,  440027,  8, 17, 16, }, // Fiji
    {  389,  709,       0,  440043,       0,  0, 16,  0, }, // French_Guiana
    {  389,  723,       0,  440059,       0,  0, 34,  0, }, // French_Southern
    {  389,  746,       0,  440093,       0,  0, 13,  0, }, // Galapagos
    {  389,  756,       0,  440106,       0,  0, 12,  0, }, // Gambier
    {  389,  764,  440118,  440130,  440151, 12, 21, 20, }, // Georgia
    {  389,  772,       0,  440171,       0,  0, 18,  0, }, // Gilbert_Islands
    {  389,  788,       0,  440189,       0,  0, 17,  0, }, // GMT
    {  389,  830,  440206,  440224,  440251, 18, 27, 26, }, // Greenland_Eastern
    {  389,  848,  440277,  440296,  440324, 19, 28, 27, }, // Greenland_Western
    {  389,  871,       0,  440351,       0,  0, 17,  0, }, // Gulf
    {  389,  876,       0,  440368,       0,  0, 10,  0, }, // Guyana
    {  389,  883,  440378,  440395,  440421, 17, 26, 24, }, // Hawaii_Aleutian
    {  389,  899,  440445,  440458,  440480, 13, 22, 21, }, // Hong_Kong
    {  389,  909,  440501,  440507,  440526,  6, 19, 18, }, // Hovd
    {  389,  914,       0,  440544,       0,  0, 19,  0, }, // India
    {  389,  920,       0,  440563,       0,  0, 17,  0, }, // Indian_Ocean
    {  389,  933,       0,  440580,       0,  0, 11,  0, }, // Indochina
    {  389,  943,       0,  440591,       0,  0, 19,  0, }, // Indonesia_Central
    {  389,  961,       0,  440610,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  389,  979,       0,  440630,       0,  0, 21,  0, }, // Indonesia_Western
    {  389,  997,   94421,  440651,  440668,  4, 17, 15, }, // Iran
    {  389, 1008,  440683,  440696,  440718, 13, 22, 21, }, // Irkutsk
    {  389, 1016,  440739,  440750,  440770, 11, 20, 18, }, // Israel
    {  389, 1023,  440788,  440796,  440813,  8, 17, 15, }, // Japan
    {  389, 1047,       0,  440828,       0,  0, 13,  0, }, // Kazakhstan
    {  389, 1058,       0,  440841,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  389, 1077,       0,  440860,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  389, 1106,  440880,  440890,  440909, 10, 19, 17, }, // Korea
    {  389, 1112,       0,  440926,       0,  0,  9,  0, }, // Kosrae
    {  389, 1119,  440935,  440954,  440982, 19, 28, 27, }, // Krasnoyarsk
    {  389, 1151,       0,  441009,       0,  0, 17,  0, }, // Kyrgystan
    {  389, 1175,       0,  441026,       0,  0, 14,  0, }, // Line_Islands
    {  389, 1188,  441040,  441054,  441077, 14, 23, 21, }, // Lord_Howe
    {  389, 1204,  441098,  441108,  441127, 10, 19, 18, }, // Magadan
    {  389, 1219,       0,  441145,       0,  0, 11,  0, }, // Malaysia
    {  389, 1228,       0,  441156,       0,  0, 10,  0, }, // Maldives
    {  389, 1237,       0,  441166,       0,  0, 12,  0, }, // Marquesas
    {  389, 1247,       0,  441178,       0,  0, 16,  0, }, // Marshall_Islands
    {  389, 1264,  441194,  441204,  441223, 10, 19, 18, }, // Mauritius
    {  389, 1274,       0,  441241,       0,  0,  8,  0, }, // Mawson
    {  389, 1281,  441249,  441269,  441298, 20, 29, 27, }, // Mexico_Pacific
    {  389, 1296,  441325,  441337,  441358, 12, 21, 20, }, // Mongolia
    {  389, 1305,  441378,  441388,  441407, 10, 19, 18, }, // Moscow
    {  389, 1312,       0,  441425,       0,  0, 12,  0, }, // Myanmar
    {  389, 1320,       0,  441437,       0,  0,  7,  0, }, // Nauru
    {  389, 1326,       0,  441444,       0,  0,  9,  0, }, // Nepal
    {  389, 1332,  441453,  441472,  441500, 19, 28, 27, }, // New_Caledonia
    {  389, 1346,  441527,  441542,  441566, 15, 24, 22, }, // New_Zealand
    {  389, 1358,  441588,  441604,  441629, 16, 25, 23, }, // Newfoundland
    {  389, 1371,       0,  441652,       0,  0,  9,  0, }, // Niue
    {  389, 1376,  441661,  441677,  441702, 16, 25, 24, }, // Norfolk
    {  389, 1384,  441726,  441750,  441783, 24, 33, 32, }, // Noronha
    {  389, 1406,  338879,  441815,  441841, 13, 26, 25, }, // Novosibirsk
    {  389, 1418,  441866,  441876,  441895, 10, 19, 18, }, // Omsk
    {  389, 1428,   95857,  441913,  441935,  9, 22, 21, }, // Pakistan
    {  389, 1437,       0,  441956,       0,  0,  8,  0, }, // Palau
    {  389, 1443,       0,  441964,       0,  0, 19,  0, }, // Papua_New_Guinea
    {  389, 1460,  441983,  441993,  442012, 10, 19, 18, }, // Paraguay
    {  389, 1469,  442030,  442038,  442055,  8, 17, 16, }, // Peru
    {  389, 1474,  442071,  442083,  442104, 12, 21, 20, }, // Philippines
    {  389, 1486,       0,  442124,       0,  0, 17,  0, }, // Phoenix_Islands
    {  389, 1502,  442141,  442165,  442198, 24, 33, 31, }, // Pierre_Miquelon
    {  389, 1518,       0,  442229,       0,  0, 11,  0, }, // Pitcairn
    {  389, 1527,       0,  442240,       0,  0,  9,  0, }, // Ponape
    {  389, 1534,       0,  442249,       0,  0, 14,  0, }, // Pyongyang
    {  389, 1554,       0,  442263,       0,  0, 12,  0, }, // Reunion
    {  389, 1562,       0,  442275,       0,  0, 10,  0, }, // Rothera
    {  389, 1570,  442285,  442294,  442312,  9, 18, 17, }, // Sakhalin
    {  389, 1596,  442329,  442338,  442356,  9, 18, 16, }, // Samoa
    {  389, 1602,       0,  442372,       0,  0, 11,  0, }, // Seychelles
    {  389, 1624,       0,  442383,       0,  0, 21,  0, }, // Singapore
    {  389, 1634,       0,  442404,       0,  0, 18,  0, }, // Solomon
    {  389, 1642,       0,  442422,       0,  0, 19,  0, }, // South_Georgia
    {  389, 1656,       0,  442441,       0,  0, 11,  0, }, // Suriname
    {  389, 1676,       0,  442452,       0,  0, 10,  0, }, // Syowa
    {  389, 1682,       0,  442462,       0,  0, 10,  0, }, // Tahiti
    {  389, 1689,  442472,  442481,  442499,  9, 18, 16, }, // Taipei
    {  389, 1696,       0,  442515,       0,  0, 14,  0, }, // Tajikistan
    {  389, 1724,       0,  442529,       0,  0, 10,  0, }, // Tokelau
    {  389, 1732,  442539,  442548,  442566,  9, 18, 17, }, // Tonga
    {  389, 1738,       0,  442583,       0,  0,  7,  0, }, // Truk
    {  389, 1750,  339743,  442590,  442617, 14, 27, 26, }, // Turkmenistan
    {  389, 1763,       0,  442643,       0,  0, 10,  0, }, // Tuvalu
    {  389, 1777,  442653,  442664,  442684, 11, 20, 19, }, // Uruguay
    {  389, 1792,  442703,  442720,  442746, 17, 26, 25, }, // Uzbekistan
    {  389, 1803,  442771,  442780,  442798,  9, 18, 17, }, // Vanuatu
    {  389, 1811,       0,  442815,       0,  0, 13,  0, }, // Venezuela
    {  389, 1821,  442828,  442845,  442871, 17, 26, 25, }, // Vladivostok
    {  389, 1833,  442896,  442913,  442939, 17, 26, 25, }, // Volgograd
    {  389, 1843,       0,  442834,       0,  0, 11,  0, }, // Vostok
    {  389, 1850,       0,  442964,       0,  0, 13,  0, }, // Wake
    {  389, 1855,       0,  442977,       0,  0, 22,  0, }, // Wallis
    {  389, 1862,  442999,  443011,  443032, 12, 21, 20, }, // Yakutsk
    {  389, 1870,  443052,  443069,  443095, 17, 26, 25, }, // Yekaterinburg
    {  389, 1892,       0,  443120,       0,  0,  9,  0, }, // Yukon
    {  390,  788,       0,    2791,       0,  0, 19,  0, }, // GMT Konkani/Latin/India
    {  391,    0,  443129,  443135,  443142,  6,  7, 10, }, // Acre Korean/Korean/South Korea
    {  391,    5,       0,  443152,       0,  0,  9,  0, }, // Afghanistan
    {  391,   17,       0,  443161,       0,  0,  9,  0, }, // Africa_Central
    {  391,   32,       0,  443170,       0,  0,  8,  0, }, // Africa_Eastern
    {  391,   65,       0,  443178,       0,  0,  8,  0, }, // Africa_Southern
    {  391,   81,  443186,  443194,  443203,  8,  9, 12, }, // Africa_Western
    {  391,  107,  443215,  443222,  443230,  7,  8, 11, }, // Alaska
    {  391,  128,  443241,  443250,  443260,  9, 10, 10, }, // Almaty
    {  391,  135,  443270,  443276,  443283,  6,  7, 10, }, // Amazon
    {  391,  142,  443293,  443300,  443308,  7,  8, 11, }, // America_Central
    {  391,  158,  443319,  443326,  443334,  7,  8, 11, }, // America_Eastern
    {  391,  174,  443345,  443352,  443360,  7,  8, 11, }, // America_Mountain
    {  391,  191,  443371,  443379,  443388,  8,  9, 12, }, // America_Pacific
    {  391,  207,  443400,  443407,  443415,  7,  8, 11, }, // Anadyr
    {  391,  214,  443426,  443432,  443439,  6,  7, 10, }, // Apia
    {  391,  219,  443449,  443458,  443468,  9, 10, 10, }, // Aqtau
    {  391,  225,  443478,  443487,  443497,  9, 10, 10, }, // Aqtobe
    {  391,  232,  443507,  443514,  443522,  7,  8, 11, }, // Arabian
    {  391,  240,  443533,  443541,  443550,  8,  9, 12, }, // Argentina
    {  391,  250,  443562,  443573,  443585, 11, 12, 15, }, // Argentina_Western
    {  391,  268,  443600,  443608,  443617,  8,  9, 12, }, // Armenia
    {  391,  286,  443629,  443635,  443642,  6,  7, 10, }, // Atlantic
    {  391,  295,  443652,  443665,  443679, 13, 14, 17, }, // Australia_Central
    {  391,  313,  443696,  443710,  443725, 14, 15, 18, }, // Australia_CentralWestern
    {  391,  338,  443743,  443756,  443770, 13, 14, 17, }, // Australia_Eastern
    {  391,  356,  443787,  443800,  443814, 13, 14, 17, }, // Australia_Western
    {  391,  374,  443831,  443840,  443850,  9, 10, 13, }, // Azerbaijan
    {  391,  385,  443863,  443870,  443878,  7,  8, 11, }, // Azores
    {  391,  397,  443889,  443897,  443906,  8,  9, 12, }, // Bangladesh
    {  391,  415,       0,  443918,       0,  0,  5,  0, }, // Bhutan
    {  391,  422,       0,  443923,       0,  0,  7,  0, }, // Bolivia
    {  391,  437,  443930,  443938,  443947,  8,  9, 12, }, // Brasilia
    {  391,  454,       0,  443959,       0,  0,  7,  0, }, // Brunei
    {  391,  461,  443966,  443975,  443985,  9, 10, 13, }, // Cape_Verde
    {  391,  472,       0,  443998,       0,  0,  6,  0, }, // Casey
    {  391,  478,       0,  444004,       0,  0,  6,  0, }, // Chamorro
    {  391,  487,  444010,  444015,  444021,  5,  6,  9, }, // Chatham
    {  391,  495,  444030,  444035,  444041,  5,  6,  9, }, // Chile
    {  391,  501,  444050,  444055,  444061,  5,  6,  9, }, // China
    {  391,  507,       0,  444070,       0,  0,  9,  0, }, // Christmas
    {  391,  517,       0,  444079,       0,  0,  9,  0, }, // Cocos
    {  391,  523,  444088,  444095,  444103,  7,  8, 11, }, // Colombia
    {  391,  532,  444114,  444121,  444129,  7,  8, 14, }, // Cook
    {  391,  537,  444143,  444148,  444154,  5,  6,  9, }, // Cuba
    {  391,  548,       0,  444163,       0,  0,  7,  0, }, // Davis
    {  391,  564,       0,  444170,       0,  0,  8,  0, }, // DumontDUrville
    {  391,  601,       0,  444178,       0,  0,  7,  0, }, // East_Timor
    {  391,  612,  444185,  444192,  444200,  7,  8, 11, }, // Easter
    {  391,  619,       0,  444211,       0,  0,  7,  0, }, // Ecuador
    {  391,  627,  444218,  444225,  444233,  7,  8, 11, }, // Europe_Central
    {  391,  642,  444244,  444250,  444257,  6,  7, 10, }, // Europe_Eastern
    {  391,  657,       0,  444267,       0,  0,  8,  0, }, // Europe_Further_Eastern
    {  391,  680,  444275,  444281,  444288,  6,  7, 10, }, // Europe_Western
    {  391,  695,  444298,  444308,  444319, 10, 11, 14, }, // Falkland
    {  391,  704,  444333,  444338,  444344,  5,  6,  9, }, // Fiji
    {  391,  709,       0,  444353,       0,  0, 12,  0, }, // French_Guiana
    {  391,  723,       0,  444365,       0,  0, 19,  0, }, // French_Southern
    {  391,  746,       0,  444384,       0,  0,  8,  0, }, // Galapagos
    {  391,  756,       0,  444392,       0,  0,  6,  0, }, // Gambier
    {  391,  764,  444398,  444404,  444411,  6,  7, 10, }, // Georgia
    {  391,  772,       0,  444421,       0,  0,  9,  0, }, // Gilbert_Islands
    {  391,  788,       0,  444430,       0,  0,  8,  0, }, // GMT
    {  391,  830,  444438,  444448,  444459, 10, 11, 14, }, // Greenland_Eastern
    {  391,  848,  444473,  444483,  444494, 10, 11, 14, }, // Greenland_Western
    {  391,  866,       0,  444508,       0,  0,  7,  0, }, // Guam
    {  391,  871,       0,  444515,       0,  0,  7,  0, }, // Gulf
    {  391,  876,       0,  444358,       0,  0,  7,  0, }, // Guyana
    {  391,  883,  444522,  444532,  444543, 10, 11, 14, }, // Hawaii_Aleutian
    {  391,  899,  444557,  444562,  444568,  5,  6,  9, }, // Hong_Kong
    {  391,  909,  444577,  444583,  444590,  6,  7, 10, }, // Hovd
    {  391,  914,       0,  444600,       0,  0,  6,  0, }, // India
    {  391,  920,       0,  444606,       0,  0,  6,  0, }, // Indian_Ocean
    {  391,  933,       0,  444612,       0,  0,  8,  0, }, // Indochina
    {  391,  943,       0,  444620,       0,  0, 11,  0, }, // Indonesia_Central
    {  391,  961,       0,  444631,       0,  0, 11,  0, }, // Indonesia_Eastern
    {  391,  979,       0,  444642,       0,  0, 11,  0, }, // Indonesia_Western
    {  391,  997,  444653,  444658,  444664,  5,  6,  9, }, // Iran
    {  391, 1008,  444673,  444681,  444690,  8,  9, 12, }, // Irkutsk
    {  391, 1016,  444702,  444709,  444717,  7,  8, 11, }, // Israel
    {  391, 1023,  444728,  444733,  444739,  5,  6,  9, }, // Japan
    {  391, 1029,  444748,  444764,  444781, 16, 17, 20, }, // Kamchatka
    {  391, 1047,       0,  444801,       0,  0,  8,  0, }, // Kazakhstan
    {  391, 1058,       0,  444809,       0,  0, 11,  0, }, // Kazakhstan_Eastern
    {  391, 1077,       0,  444820,       0,  0, 11,  0, }, // Kazakhstan_Western
    {  391, 1106,  444831,  444838,  444846,  7,  8, 11, }, // Korea
    {  391, 1112,       0,  444857,       0,  0,  8,  0, }, // Kosrae
    {  391, 1119,  444865,  444876,  444888, 11, 12, 15, }, // Krasnoyarsk
    {  391, 1151,       0,  444903,       0,  0,  9,  0, }, // Kyrgystan
    {  391, 1161,       0,  444912,       0,  0,  8,  0, }, // Lanka
    {  391, 1175,       0,  444920,       0,  0,  8,  0, }, // Line_Islands
    {  391, 1188,  444928,  444936,  444945,  8,  9, 12, }, // Lord_Howe
    {  391, 1198,  444957,  444963,  444972,  6,  9, 10, }, // Macau
    {  391, 1204,  444982,  444988,  444995,  6,  7, 10, }, // Magadan
    {  391, 1219,       0,  445005,       0,  0,  8,  0, }, // Malaysia
    {  391, 1228,       0,  445013,       0,  0,  6,  0, }, // Maldives
    {  391, 1237,       0,  445019,       0,  0, 10,  0, }, // Marquesas
    {  391, 1247,       0,  445029,       0,  0,  8,  0, }, // Marshall_Islands
    {  391, 1264,  445037,  445044,  445052,  7,  8, 11, }, // Mauritius
    {  391, 1274,       0,  445063,       0,  0,  5,  0, }, // Mawson
    {  391, 1281,  445068,  445078,  445089, 10, 11, 14, }, // Mexico_Pacific
    {  391, 1296,  445103,  445111,  445120,  8,  9, 12, }, // Mongolia
    {  391, 1305,  445132,  445139,  445147,  7,  8, 11, }, // Moscow
    {  391, 1312,       0,  445158,       0,  0,  6,  0, }, // Myanmar
    {  391, 1320,       0,  445164,       0,  0,  6,  0, }, // Nauru
    {  391, 1326,       0,  445170,       0,  0,  5,  0, }, // Nepal
    {  391, 1332,  445175,  445184,  445194,  9, 10, 13, }, // New_Caledonia
    {  391, 1346,  445207,  445214,  445222,  7,  8, 11, }, // New_Zealand
    {  391, 1358,  445233,  445241,  445250,  8,  9, 12, }, // Newfoundland
    {  391, 1371,       0,  445262,       0,  0,  6,  0, }, // Niue
    {  391, 1376,  445268,  445274,  445281,  6,  7, 10, }, // Norfolk
    {  391, 1384,  445291,  445304,  445318, 13, 14, 17, }, // Noronha
    {  391, 1392,       0,  445335,       0,  0, 14,  0, }, // North_Mariana
    {  391, 1406,  445349,  445359,  445370, 10, 11, 14, }, // Novosibirsk
    {  391, 1418,  445384,  445390,  445397,  6,  7, 10, }, // Omsk
    {  391, 1428,  445407,  445414,  445422,  7,  8, 11, }, // Pakistan
    {  391, 1437,       0,  445433,       0,  0,  6,  0, }, // Palau
    {  391, 1443,       0,  445439,       0,  0,  9,  0, }, // Papua_New_Guinea
    {  391, 1460,  445448,  445455,  445463,  7,  8, 11, }, // Paraguay
    {  391, 1469,  445474,  445479,  445485,  5,  6,  9, }, // Peru
    {  391, 1474,  445494,  445500,  445507,  6,  7, 10, }, // Philippines
    {  391, 1486,       0,  445517,       0,  0,  9,  0, }, // Phoenix_Islands
    {  391, 1502,  445526,  445539,  445553, 13, 14, 17, }, // Pierre_Miquelon
    {  391, 1518,       0,  445570,       0,  0,  6,  0, }, // Pitcairn
    {  391, 1527,       0,  445576,       0,  0,  6,  0, }, // Ponape
    {  391, 1534,       0,  443374,       0,  0,  5,  0, }, // Pyongyang
    {  391, 1544,  445582,  445590,  445601,  8, 11, 12, }, // Qyzylorda
    {  391, 1554,       0,  445613,       0,  0,  7,  0, }, // Reunion
    {  391, 1562,       0,  445620,       0,  0,  6,  0, }, // Rothera
    {  391, 1570,  445626,  445632,  445639,  6,  7, 10, }, // Sakhalin
    {  391, 1579,  445649,  445655,  445662,  6,  7, 10, }, // Samara
    {  391, 1596,  445672,  445678,  445685,  6,  7, 10, }, // Samoa
    {  391, 1602,       0,  445695,       0,  0,  6,  0, }, // Seychelles
    {  391, 1624,       0,  445701,       0,  0,  8,  0, }, // Singapore
    {  391, 1634,       0,  445709,       0,  0,  9,  0, }, // Solomon
    {  391, 1642,       0,  445718,       0,  0, 10,  0, }, // South_Georgia
    {  391, 1656,       0,  445728,       0,  0,  6,  0, }, // Suriname
    {  391, 1676,       0,  445734,       0,  0,  5,  0, }, // Syowa
    {  391, 1682,       0,  445739,       0,  0,  6,  0, }, // Tahiti
    {  391, 1689,  445745,  445750,  445756,  5,  6,  9, }, // Taipei
    {  391, 1696,       0,  445765,       0,  0,  8,  0, }, // Tajikistan
    {  391, 1724,       0,  445773,       0,  0,  7,  0, }, // Tokelau
    {  391, 1732,  445780,  445785,  445791,  5,  6,  9, }, // Tonga
    {  391, 1738,       0,  445800,       0,  0,  5,  0, }, // Truk
    {  391, 1750,  445805,  445815,  445826, 10, 11, 14, }, // Turkmenistan
    {  391, 1763,       0,  445840,       0,  0,  6,  0, }, // Tuvalu
    {  391, 1777,  445846,  445853,  445861,  7,  8, 11, }, // Uruguay
    {  391, 1792,  445872,  445881,  445891,  9, 10, 13, }, // Uzbekistan
    {  391, 1803,  445904,  445911,  445919,  7,  8, 11, }, // Vanuatu
    {  391, 1811,       0,  445930,       0,  0,  8,  0, }, // Venezuela
    {  391, 1821,  445938,  445948,  445959, 10, 11, 14, }, // Vladivostok
    {  391, 1833,  445973,  445981,  445990,  8,  9, 12, }, // Volgograd
    {  391, 1843,       0,  446002,       0,  0,  6,  0, }, // Vostok
    {  391, 1850,       0,  446008,       0,  0,  7,  0, }, // Wake
    {  391, 1855,       0,  446015,       0,  0, 12,  0, }, // Wallis
    {  391, 1862,  446027,  446034,  446042,  7,  8, 11, }, // Yakutsk
    {  391, 1870,  446053,  446063,  446074, 10, 11, 14, }, // Yekaterinburg
    {  391, 1892,       0,  446088,       0,  0,  5,  0, }, // Yukon
    {  393, 1106,  446093,  446098,  446104,  5,  6,  9, }, // Korea Korean/Korean/North Korea
    {  398,    5,       0,  446113,       0,  0, 17,  0, }, // Afghanistan Kurdish/Latin/Turkey
    {  398,   17,       0,  446130,       0,  0, 20,  0, }, // Africa_Central
    {  398,   32,       0,  446150,       0,  0, 23,  0, }, // Africa_Eastern
    {  398,   65,       0,  446173,       0,  0, 31,  0, }, // Africa_Southern
    {  398,   81,  446204,  446225,  446257, 21, 32, 31, }, // Africa_Western
    {  398,  107,  446288,  446302,  446327, 14, 25, 24, }, // Alaska
    {  398,  135,  446351,  446364,  446388, 13, 24, 23, }, // Amazon
    {  398,  142,  446411,  446443,  446486, 32, 43, 42, }, // America_Central
    {  398,  158,  446528,  446560,  446604, 32, 44, 43, }, // America_Eastern
    {  398,  174,  446647,  446678,  446720, 31, 42, 41, }, // America_Mountain
    {  398,  191,  446761,  446794,  446838, 33, 44, 43, }, // America_Pacific
    {  398,  214,  446881,  446893,  446916, 12, 23, 22, }, // Apia
    {  398,  232,  446938,  446954,  446981, 16, 27, 26, }, // Arabian
    {  398,  240,  447007,  447022,  447048, 15, 26, 25, }, // Argentina
    {  398,  250,  447073,  447095,  447128, 22, 33, 32, }, // Argentina_Western
    {  398,  268,  447160,  447177,  447205, 17, 28, 27, }, // Armenia
    {  398,  286,  447232,  447247,  447273, 15, 26, 25, }, // Atlantic
    {  398,  295,  447298,  447322,  447357, 24, 35, 34, }, // Australia_Central
    {  398,  313,  447391,  447426,  447472, 35, 46, 45, }, // Australia_CentralWestern
    {  398,  338,  447517,  447544,  447582, 27, 38, 37, }, // Australia_Eastern
    {  398,  356,  447619,  447644,  447680, 25, 36, 35, }, // Australia_Western
    {  398,  374,  447715,  447732,  447760, 17, 28, 27, }, // Azerbaijan
    {  398,  385,  447787,  447799,  447822, 12, 23, 22, }, // Azores
    {  398,  397,  447844,  447860,  447887, 16, 27, 26, }, // Bangladesh
    {  398,  415,       0,  447913,       0,  0, 12,  0, }, // Bhutan
    {  398,  422,       0,  447925,       0,  0, 15,  0, }, // Bolivia
    {  398,  437,  447940,  447956,  447983, 16, 27, 26, }, // Brasilia
    {  398,  454,       0,  448009,       0,  0, 24,  0, }, // Brunei
    {  398,  461,  448033,  448051,  448080, 18, 29, 28, }, // Cape_Verde
    {  398,  478,       0,  448108,       0,  0, 27,  0, }, // Chamorro
    {  398,  487,  448135,  448149,  448174, 14, 25, 24, }, // Chatham
    {  398,  495,  448198,  448210,  448233, 12, 23, 22, }, // Chile
    {  398,  501,  448255,  448265,  448286, 10, 21, 20, }, // China
    {  398,  507,       0,  448306,       0,  0, 23,  0, }, // Christmas
    {  398,  517,       0,  448329,       0,  0, 20,  0, }, // Cocos
    {  398,  523,  448349,  448366,  448394, 17, 28, 27, }, // Colombia
    {  398,  532,  448421,  448440,  448470, 19, 30, 32, }, // Cook
    {  398,  537,  448502,  448514,  448537, 12, 23, 22, }, // Cuba
    {  398,  548,       0,  448559,       0,  0, 12,  0, }, // Davis
    {  398,  564,       0,  448571,       0,  0, 24,  0, }, // DumontDUrville
    {  398,  601,       0,  448595,       0,  0, 21,  0, }, // East_Timor
    {  398,  612,  448616,  448639,  448673, 23, 34, 33, }, // Easter
    {  398,  619,       0,  448706,       0,  0, 14,  0, }, // Ecuador
    {  398,  627,  448720,  448740,  448771, 20, 31, 30, }, // Europe_Central
    {  398,  642,  448801,  448824,  448858, 23, 34, 33, }, // Europe_Eastern
    {  398,  657,       0,  448891,       0,  0, 32,  0, }, // Europe_Further_Eastern
    {  398,  680,  448923,  448944,  448976, 21, 32, 31, }, // Europe_Western
    {  398,  695,  449007,  449030,  449064, 23, 34, 33, }, // Falkland
    {  398,  704,  449097,  449109,  449132, 12, 23, 22, }, // Fiji
    {  398,  709,       0,  449154,       0,  0, 22,  0, }, // French_Guiana
    {  398,  723,       0,  449176,       0,  0, 34,  0, }, // French_Southern
    {  398,  746,       0,  449210,       0,  0, 16,  0, }, // Galapagos
    {  398,  756,       0,  449226,       0,  0, 14,  0, }, // Gambier
    {  398,  764,  449240,  449256,  449283, 16, 27, 26, }, // Georgia
    {  398,  772,       0,  449309,       0,  0, 22,  0, }, // Gilbert_Islands
    {  398,  788,       0,  449331,       0,  0, 26,  0, }, // GMT
    {  398,  830,  449357,  449381,  449416, 24, 35, 34, }, // Greenland_Eastern
    {  398,  848,  449450,  449472,  449505, 22, 33, 32, }, // Greenland_Western
    {  398,  871,       0,  449537,       0,  0, 24,  0, }, // Gulf
    {  398,  876,       0,  449561,       0,  0, 14,  0, }, // Guyana
    {  398,  883,  449575,  449597,  449630, 22, 33, 32, }, // Hawaii_Aleutian
    {  398,  899,  449662,  449678,  449705, 16, 27, 26, }, // Hong_Kong
    {  398,  909,  449731,  449742,  449764, 11, 22, 21, }, // Hovd
    {  398,  914,       0,  449785,       0,  0, 27,  0, }, // India
    {  398,  920,       0,  449812,       0,  0, 20,  0, }, // Indian_Ocean
    {  398,  933,       0,  449832,       0,  0, 15,  0, }, // Indochina
    {  398,  943,       0,  449847,       0,  0, 23,  0, }, // Indonesia_Central
    {  398,  961,       0,  449870,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  398,  979,       0,  449896,       0,  0, 24,  0, }, // Indonesia_Western
    {  398,  997,  449920,  449931,  449953, 11, 22, 21, }, // Iran
    {  398, 1008,  449974,  449988,  450013, 14, 25, 24, }, // Irkutsk
    {  398, 1016,  450037,  450050,  450074, 13, 24, 23, }, // Israel
    {  398, 1023,  450097,  450112,  450138, 15, 26, 25, }, // Japan
    {  398, 1047,       0,  450163,       0,  0, 17,  0, }, // Kazakhstan
    {  398, 1058,       0,  450180,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  398, 1077,       0,  450206,       0,  0, 24,  0, }, // Kazakhstan_Western
    {  398, 1106,  450230,  450242,  450265, 12, 23, 22, }, // Korea
    {  398, 1112,       0,  450287,       0,  0, 14,  0, }, // Kosrae
    {  398, 1119,  450301,  450319,  450348, 18, 29, 28, }, // Krasnoyarsk
    {  398, 1151,       0,  450376,       0,  0, 18,  0, }, // Kyrgystan
    {  398, 1175,       0,  450394,       0,  0, 20,  0, }, // Line_Islands
    {  398, 1188,  450414,  450431,  450459, 17, 28, 27, }, // Lord_Howe
    {  398, 1204,  450486,  450500,  450525, 14, 25, 24, }, // Magadan
    {  398, 1219,       0,  450549,       0,  0, 15,  0, }, // Malaysia
    {  398, 1228,       0,  450564,       0,  0, 14,  0, }, // Maldives
    {  398, 1237,       0,  450578,       0,  0, 16,  0, }, // Marquesas
    {  398, 1247,       0,  450594,       0,  0, 21,  0, }, // Marshall_Islands
    {  398, 1264,  450615,  450631,  450658, 16, 27, 26, }, // Mauritius
    {  398, 1274,       0,  450684,       0,  0, 13,  0, }, // Mawson
    {  398, 1281,  450697,  450721,  450756, 24, 35, 34, }, // Mexico_Pacific
    {  398, 1296,  450790,  450806,  450833, 16, 27, 26, }, // Mongolia
    {  398, 1305,  450859,  450874,  450900, 15, 26, 25, }, // Moscow
    {  398, 1312,       0,  450925,       0,  0, 14,  0, }, // Myanmar
    {  398, 1320,       0,  450939,       0,  0, 13,  0, }, // Nauru
    {  398, 1326,       0,  450952,       0,  0, 12,  0, }, // Nepal
    {  398, 1332,  450964,  450984,  451015, 20, 31, 30, }, // New_Caledonia
    {  398, 1346,  451045,  451063,  451092, 18, 29, 28, }, // New_Zealand
    {  398, 1358,  451120,  451139,  451169, 19, 30, 29, }, // Newfoundland
    {  398, 1371,       0,  451198,       0,  0, 12,  0, }, // Niue
    {  398, 1376,  451210,  451231,  451263, 21, 32, 31, }, // Norfolk
    {  398, 1384,  451294,  451321,  451359, 27, 38, 37, }, // Noronha
    {  398, 1406,  451396,  451414,  451443, 18, 29, 28, }, // Novosibirsk
    {  398, 1418,  451471,  451482,  451504, 11, 22, 21, }, // Omsk
    {  398, 1428,  451525,  451540,  451566, 15, 26, 25, }, // Pakistan
    {  398, 1437,       0,  451591,       0,  0, 13,  0, }, // Palau
    {  398, 1443,       0,  451604,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  398, 1460,  451630,  451645,  451671, 15, 26, 25, }, // Paraguay
    {  398, 1469,  451696,  451708,  451731, 12, 23, 22, }, // Peru
    {  398, 1474,  451753,  451767,  451792, 14, 25, 24, }, // Philippines
    {  398, 1486,       0,  451816,       0,  0, 22,  0, }, // Phoenix_Islands
    {  398, 1502,  451838,  451868,  451909, 30, 41, 40, }, // Pierre_Miquelon
    {  398, 1518,       0,  451949,       0,  0, 15,  0, }, // Pitcairn
    {  398, 1527,       0,  451964,       0,  0, 14,  0, }, // Ponape
    {  398, 1534,       0,  451978,       0,  0, 16,  0, }, // Pyongyang
    {  398, 1554,       0,  451994,       0,  0, 14,  0, }, // Reunion
    {  398, 1562,       0,  452008,       0,  0, 15,  0, }, // Rothera
    {  398, 1570,  452023,  452037,  452062, 14, 25, 24, }, // Sakhalin
    {  398, 1596,  452086,  452099,  452123, 13, 24, 23, }, // Samoa
    {  398, 1602,       0,  452146,       0,  0, 15,  0, }, // Seychelles
    {  398, 1624,       0,  452161,       0,  0, 26,  0, }, // Singapore
    {  398, 1634,       0,  452187,       0,  0, 22,  0, }, // Solomon
    {  398, 1642,       0,  452209,       0,  0, 21,  0, }, // South_Georgia
    {  398, 1656,       0,  452230,       0,  0, 14,  0, }, // Suriname
    {  398, 1676,       0,  452244,       0,  0, 13,  0, }, // Syowa
    {  398, 1682,       0,  452257,       0,  0, 14,  0, }, // Tahiti
    {  398, 1689,  452271,  452285,  452310, 14, 25, 24, }, // Taipei
    {  398, 1696,       0,  452334,       0,  0, 17,  0, }, // Tajikistan
    {  398, 1724,       0,  452351,       0,  0, 15,  0, }, // Tokelau
    {  398, 1732,  452366,  452379,  452403, 13, 24, 23, }, // Tonga
    {  398, 1738,       0,  452426,       0,  0, 12,  0, }, // Truk
    {  398, 1750,  452438,  452457,  452487, 19, 30, 29, }, // Turkmenistan
    {  398, 1763,       0,  452516,       0,  0, 14,  0, }, // Tuvalu
    {  398, 1777,  452530,  452544,  452569, 14, 25, 24, }, // Uruguay
    {  398, 1792,  452593,  452610,  452638, 17, 28, 27, }, // Uzbekistan
    {  398, 1803,  452665,  452680,  452706, 15, 26, 25, }, // Vanuatu
    {  398, 1811,       0,  452731,       0,  0, 17,  0, }, // Venezuela
    {  398, 1821,  452748,  452766,  452795, 18, 29, 28, }, // Vladivostok
    {  398, 1833,  452823,  452839,  452866, 16, 27, 26, }, // Volgograd
    {  398, 1843,       0,  452892,       0,  0, 13,  0, }, // Vostok
    {  398, 1850,       0,  452905,       0,  0, 19,  0, }, // Wake
    {  398, 1855,       0,  452924,       0,  0, 23,  0, }, // Wallis
    {  398, 1862,  452947,  452961,  452986, 14, 25, 24, }, // Yakutsk
    {  398, 1870,  453010,  453030,  453061, 20, 31, 30, }, // Yekaterinburg
    {  398, 1892,       0,  453091,       0,  0, 12,  0, }, // Yukon
    {  400,    5,       0,  453103,       0,  0, 19,  0, }, // Afghanistan Kyrgyz/Cyrillic/Kyrgyzstan
    {  400,   17,       0,  453122,       0,  0, 25,  0, }, // Africa_Central
    {  400,   32,       0,  453147,       0,  0, 21,  0, }, // Africa_Eastern
    {  400,   65,       0,  453168,       0,  0, 22,  0, }, // Africa_Southern
    {  400,   81,  453190,  453211,  453237, 21, 26, 27, }, // Africa_Western
    {  400,  107,  453264,  453279,  453300, 15, 21, 21, }, // Alaska
    {  400,  135,  453321,  453336,  453357, 15, 21, 21, }, // Amazon
    {  400,  142,  453378,  453410,  453450, 32, 40, 40, }, // America_Central
    {  400,  158,  453490,  453520,  453556, 30, 36, 36, }, // America_Eastern
    {  400,  174,  453592,  453620,  453654, 28, 34, 34, }, // America_Mountain
    {  400,  191,  453688,  453723,  453764, 35, 41, 41, }, // America_Pacific
    {  400,  214,  453805,  453818,  453837, 13, 19, 19, }, // Apia
    {  400,  232,  453856,  453871,  453891, 15, 20, 20, }, // Arabian
    {  400,  240,  453911,  453929,  453953, 18, 24, 24, }, // Argentina
    {  400,  250,  453977,  454001,  454031, 24, 30, 30, }, // Argentina_Western
    {  400,  268,  454061,  454077,  454098, 16, 21, 22, }, // Armenia
    {  400,  286,  454120,  454138,  454162, 18, 24, 24, }, // Atlantic
    {  400,  295,  454186,  454214,  454247, 28, 33, 34, }, // Australia_Central
    {  400,  313,  454281,  454315,  454354, 34, 39, 40, }, // Australia_CentralWestern
    {  400,  338,  454394,  454418,  454447, 24, 29, 30, }, // Australia_Eastern
    {  400,  356,  454477,  454501,  454530, 24, 29, 30, }, // Australia_Western
    {  400,  374,  454560,  454579,  454603, 19, 24, 25, }, // Azerbaijan
    {  400,  385,  454628,  454641,  454659, 13, 18, 20, }, // Azores
    {  400,  397,  454679,  454697,  454720, 18, 23, 24, }, // Bangladesh
    {  400,  415,       0,  454744,       0,  0, 14,  0, }, // Bhutan
    {  400,  422,       0,  454758,       0,  0, 16,  0, }, // Bolivia
    {  400,  437,  454774,  454791,  454814, 17, 23, 23, }, // Brasilia
    {  400,  454,       0,  454837,       0,  0, 26,  0, }, // Brunei
    {  400,  461,  454863,  454882,  454906, 19, 24, 25, }, // Cape_Verde
    {  400,  478,       0,  454931,       0,  0, 16,  0, }, // Chamorro
    {  400,  487,  454947,  454961,  454979, 14, 18, 20, }, // Chatham
    {  400,  495,  454999,  455012,  455031, 13, 19, 19, }, // Chile
    {  400,  501,  455050,  455064,  455083, 14, 19, 19, }, // China
    {  400,  507,       0,  455102,       0,  0, 25,  0, }, // Christmas
    {  400,  517,       0,  455127,       0,  0, 26,  0, }, // Cocos
    {  400,  523,  455153,  455170,  455193, 17, 23, 23, }, // Colombia
    {  400,  532,  455216,  455240,  455270, 24, 30, 36, }, // Cook
    {  400,  537,  455306,  455319,  455338, 13, 19, 19, }, // Cuba
    {  400,  548,       0,  455357,       0,  0, 14,  0, }, // Davis
    {  400,  564,       0,  455371,       0,  0, 22,  0, }, // DumontDUrville
    {  400,  601,       0,  455393,       0,  0, 20,  0, }, // East_Timor
    {  400,  612,  455413,  455436,  455464, 23, 28, 28, }, // Easter
    {  400,  619,       0,  455492,       0,  0, 16,  0, }, // Ecuador
    {  400,  627,  455508,  455533,  455563, 25, 30, 31, }, // Europe_Central
    {  400,  642,  455594,  455615,  455641, 21, 26, 27, }, // Europe_Eastern
    {  400,  657,       0,  455668,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  400,  680,  455699,  455720,  455746, 21, 26, 27, }, // Europe_Western
    {  400,  695,  455773,  455802,  455836, 29, 34, 35, }, // Falkland
    {  400,  704,  455871,  455884,  455902, 13, 18, 19, }, // Fiji
    {  400,  709,       0,  455921,       0,  0, 23,  0, }, // French_Guiana
    {  400,  723,       0,  455944,       0,  0, 40,  0, }, // French_Southern
    {  400,  746,       0,  455984,       0,  0, 18,  0, }, // Galapagos
    {  400,  756,       0,  456002,       0,  0, 15,  0, }, // Gambier
    {  400,  764,  456017,  456032,  456052, 15, 20, 21, }, // Georgia
    {  400,  772,       0,  456073,       0,  0, 16,  0, }, // Gilbert_Islands
    {  400,  788,       0,  456089,       0,  0, 28,  0, }, // GMT
    {  400,  830,  456117,  456142,  456173, 25, 31, 31, }, // Greenland_Eastern
    {  400,  848,  456204,  456229,  456260, 25, 31, 31, }, // Greenland_Western
    {  400,  871,       0,  456291,       0,  0, 28,  0, }, // Gulf
    {  400,  876,       0,  455929,       0,  0, 15,  0, }, // Guyana
    {  400,  883,  456319,  456340,  456367, 21, 27, 27, }, // Hawaii_Aleutian
    {  400,  899,  456394,  456410,  456431, 16, 21, 22, }, // Hong_Kong
    {  400,  909,  456453,  456466,  456484, 13, 18, 19, }, // Hovd
    {  400,  914,       0,  456503,       0,  0, 14,  0, }, // India
    {  400,  920,       0,  456517,       0,  0, 19,  0, }, // Indian_Ocean
    {  400,  933,       0,  456536,       0,  0, 18,  0, }, // Indochina
    {  400,  943,       0,  456554,       0,  0, 28,  0, }, // Indonesia_Central
    {  400,  961,       0,  456582,       0,  0, 24,  0, }, // Indonesia_Eastern
    {  400,  979,       0,  456606,       0,  0, 24,  0, }, // Indonesia_Western
    {  400,  997,  456630,  456643,  456661, 13, 18, 22, }, // Iran
    {  400, 1008,  456683,  456699,  456720, 16, 21, 21, }, // Irkutsk
    {  400, 1016,  456741,  456756,  456777, 15, 21, 21, }, // Israel
    {  400, 1023,  456798,  456812,  456831, 14, 19, 20, }, // Japan
    {  400, 1047,       0,  456851,       0,  0, 18,  0, }, // Kazakhstan
    {  400, 1058,       0,  456869,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  400, 1077,       0,  456893,       0,  0, 24,  0, }, // Kazakhstan_Western
    {  400, 1106,  456917,  456931,  456950, 14, 19, 20, }, // Korea
    {  400, 1112,       0,  456970,       0,  0, 15,  0, }, // Kosrae
    {  400, 1119,  456985,  457004,  457028, 19, 24, 25, }, // Krasnoyarsk
    {  400, 1151,       0,  457053,       0,  0, 19,  0, }, // Kyrgystan
    {  400, 1175,       0,  457072,       0,  0, 25,  0, }, // Line_Islands
    {  400, 1188,  457097,  457114,  457136, 17, 22, 23, }, // Lord_Howe
    {  400, 1204,  457159,  457175,  457196, 16, 21, 22, }, // Magadan
    {  400, 1219,       0,  457218,       0,  0, 17,  0, }, // Malaysia
    {  400, 1228,       0,  457235,       0,  0, 16,  0, }, // Maldives
    {  400, 1237,       0,  457251,       0,  0, 17,  0, }, // Marquesas
    {  400, 1247,       0,  457268,       0,  0, 28,  0, }, // Marshall_Islands
    {  400, 1264,  457296,  457313,  457335, 17, 22, 23, }, // Mauritius
    {  400, 1274,       0,  457358,       0,  0, 15,  0, }, // Mawson
    {  400, 1281,  457373,  457401,  457435, 28, 34, 34, }, // Mexico_Pacific
    {  400, 1296,  457469,  457488,  457512, 19, 24, 25, }, // Mongolia
    {  400, 1305,  457537,  457552,  457572, 15, 20, 21, }, // Moscow
    {  400, 1312,       0,  457593,       0,  0, 16,  0, }, // Myanmar
    {  400, 1320,       0,  457609,       0,  0, 14,  0, }, // Nauru
    {  400, 1326,       0,  457623,       0,  0, 14,  0, }, // Nepal
    {  400, 1332,  457637,  457660,  457689, 23, 29, 29, }, // New_Caledonia
    {  400, 1346,  457718,  457740,  457767, 22, 27, 27, }, // New_Zealand
    {  400, 1358,  457794,  457814,  457840, 20, 26, 26, }, // Newfoundland
    {  400, 1371,       0,  457866,       0,  0, 13,  0, }, // Niue
    {  400, 1376,  457879,  457895,  457917, 16, 22, 22, }, // Norfolk
    {  400, 1384,  457939,  457967,  458001, 28, 34, 34, }, // Noronha
    {  400, 1406,  458035,  458055,  458080, 20, 25, 26, }, // Novosibirsk
    {  400, 1418,  458106,  458119,  458137, 13, 18, 19, }, // Omsk
    {  400, 1428,  458156,  458173,  458195, 17, 22, 23, }, // Pakistan
    {  400, 1437,       0,  458218,       0,  0, 14,  0, }, // Palau
    {  400, 1443,       0,  458232,       0,  0, 25,  0, }, // Papua_New_Guinea
    {  400, 1460,  458257,  458274,  458297, 17, 23, 23, }, // Paraguay
    {  400, 1469,  458320,  458333,  458352, 13, 19, 19, }, // Peru
    {  400, 1474,  458371,  458400,  458435, 29, 35, 35, }, // Philippines
    {  400, 1486,       0,  458470,       0,  0, 27,  0, }, // Phoenix_Islands
    {  400, 1502,  458497,  458527,  458563, 30, 36, 36, }, // Pierre_Miquelon
    {  400, 1518,       0,  458599,       0,  0, 17,  0, }, // Pitcairn
    {  400, 1527,       0,  458616,       0,  0, 15,  0, }, // Ponape
    {  400, 1534,       0,  458631,       0,  0, 15,  0, }, // Pyongyang
    {  400, 1554,       0,  458646,       0,  0, 16,  0, }, // Reunion
    {  400, 1562,       0,  458662,       0,  0, 15,  0, }, // Rothera
    {  400, 1570,  458677,  458693,  458714, 16, 21, 22, }, // Sakhalin
    {  400, 1596,  458736,  458750,  458770, 14, 20, 20, }, // Samoa
    {  400, 1602,       0,  458790,       0,  0, 15,  0, }, // Seychelles
    {  400, 1624,       0,  458805,       0,  0, 17,  0, }, // Singapore
    {  400, 1634,       0,  458822,       0,  0, 28,  0, }, // Solomon
    {  400, 1642,       0,  458850,       0,  0, 22,  0, }, // South_Georgia
    {  400, 1656,       0,  458872,       0,  0, 17,  0, }, // Suriname
    {  400, 1676,       0,  458889,       0,  0, 13,  0, }, // Syowa
    {  400, 1682,       0,  458902,       0,  0, 14,  0, }, // Tahiti
    {  400, 1689,  458916,  458931,  458951, 15, 20, 20, }, // Taipei
    {  400, 1696,       0,  458971,       0,  0, 18,  0, }, // Tajikistan
    {  400, 1724,       0,  458989,       0,  0, 16,  0, }, // Tokelau
    {  400, 1732,  459005,  459019,  459039, 14, 20, 20, }, // Tonga
    {  400, 1738,       0,  459059,       0,  0, 13,  0, }, // Truk
    {  400, 1750,  459072,  459092,  459117, 20, 25, 26, }, // Turkmenistan
    {  400, 1763,       0,  459143,       0,  0, 15,  0, }, // Tuvalu
    {  400, 1777,  459158,  459174,  459196, 16, 22, 22, }, // Uruguay
    {  400, 1792,  459218,  459236,  459259, 18, 23, 24, }, // Uzbekistan
    {  400, 1803,  459283,  459299,  459320, 16, 21, 22, }, // Vanuatu
    {  400, 1811,       0,  459342,       0,  0, 18,  0, }, // Venezuela
    {  400, 1821,  459360,  459380,  459405, 20, 25, 26, }, // Vladivostok
    {  400, 1833,  459431,  459449,  459472, 18, 23, 24, }, // Volgograd
    {  400, 1843,       0,  459496,       0,  0, 15,  0, }, // Vostok
    {  400, 1850,       0,  459511,       0,  0, 25,  0, }, // Wake
    {  400, 1855,       0,  459536,       0,  0, 26,  0, }, // Wallis
    {  400, 1862,  459562,  459577,  459597, 15, 20, 21, }, // Yakutsk
    {  400, 1870,  459618,  459639,  459665, 21, 26, 27, }, // Yekaterinburg
    {  400, 1892,       0,  459692,       0,  0, 13,  0, }, // Yukon
    {  403,    0,  459705,  459717,  459737, 12, 20, 20, }, // Acre Lao/Lao/Laos
    {  403,    5,       0,  459757,       0,  0, 16,  0, }, // Afghanistan
    {  403,   17,       0,  459773,       0,  0, 19,  0, }, // Africa_Central
    {  403,   32,       0,  459792,       0,  0, 27,  0, }, // Africa_Eastern
    {  403,   65,       0,  459819,       0,  0, 19,  0, }, // Africa_Southern
    {  403,   81,  459838,  459865,  459903, 27, 38, 38, }, // Africa_Western
    {  403,  107,  459941,  459952,  459971, 11, 19, 18, }, // Alaska
    {  403,  128,  459989,  459999,  460017, 10, 18, 18, }, // Almaty
    {  403,  135,  460035,  460052,  460077, 17, 25, 25, }, // Amazon
    {  403,  142,  459971,  460102,  460117,  7, 15, 14, }, // America_Central
    {  403,  158,  460131,  460144,  460165, 13, 21, 20, }, // America_Eastern
    {  403,  174,  460185,  460198,  460219, 13, 21, 20, }, // America_Mountain
    {  403,  191,  460239,  460250,  460269, 11, 19, 18, }, // America_Pacific
    {  403,  214,  460287,  460296,  460313,  9, 17, 16, }, // Apia
    {  403,  219,  460329,  460340,  460359, 11, 19, 19, }, // Aqtau
    {  403,  225,  460378,  460389,  460408, 11, 19, 19, }, // Aqtobe
    {  403,  232,  460427,  460442,  460467, 15, 25, 18, }, // Arabian
    {  403,  240,  460485,  460502,  460531, 17, 29, 30, }, // Argentina
    {  403,  250,  460561,  460588,  460625, 27, 37, 37, }, // Argentina_Western
    {  403,  268,  460662,  460673,  460692, 11, 19, 19, }, // Armenia
    {  403,  286,  460711,  460726,  460749, 15, 23, 22, }, // Atlantic
    {  403,  295,  460771,  460792,  460824, 21, 32, 30, }, // Australia_Central
    {  403,  313,  460854,  460886,  460929, 32, 43, 42, }, // Australia_CentralWestern
    {  403,  338,  460971,  460999,  461039, 28, 40, 38, }, // Australia_Eastern
    {  403,  356,  461077,  461106,  461146, 29, 40, 38, }, // Australia_Western
    {  403,  374,  461184,  461199,  461222, 15, 23, 23, }, // Azerbaijan
    {  403,  385,  461245,  461261,  461287, 16, 26, 26, }, // Azores
    {  403,  397,  461313,  461328,  461351, 15, 23, 24, }, // Bangladesh
    {  403,  415,       0,  461375,       0,  0, 12,  0, }, // Bhutan
    {  403,  422,       0,  461387,       0,  0, 15,  0, }, // Bolivia
    {  403,  437,  461402,  461420,  461440, 18, 20, 32, }, // Brasilia
    {  403,  454,       0,  461472,       0,  0, 25,  0, }, // Brunei
    {  403,  461,  461497,  461511,  461537, 14, 26, 25, }, // Cape_Verde
    {  403,  472,       0,  461562,       0,  0,  8,  0, }, // Casey
    {  403,  478,       0,  461570,       0,  0, 14,  0, }, // Chamorro
    {  403,  487,  461584,  461596,  461619, 12, 23, 21, }, // Chatham
    {  403,  495,  461640,  461651,  461672, 11, 21, 21, }, // Chile
    {  403,  501,  461693,  461702,  461717,  9, 15, 19, }, // China
    {  403,  507,       0,  461736,       0,  0, 20,  0, }, // Christmas
    {  403,  517,       0,  461756,       0,  0, 17,  0, }, // Cocos
    {  403,  523,  461773,  461785,  461804, 12, 19, 20, }, // Colombia
    {  403,  532,  461824,  461839,  461862, 15, 23, 35, }, // Cook
    {  403,  537,  461897,  461906,  461926,  9, 20, 16, }, // Cuba
    {  403,  548,       0,  461942,       0,  0,  9,  0, }, // Davis
    {  403,  564,       0,  461951,       0,  0, 14,  0, }, // DumontDUrville
    {  403,  601,       0,  461965,       0,  0, 17,  0, }, // East_Timor
    {  403,  612,  461982,  462001,  462030, 19, 29, 29, }, // Easter
    {  403,  619,       0,  462059,       0,  0, 15,  0, }, // Ecuador
    {  403,  627,  462074,  462090,  462116, 16, 26, 28, }, // Europe_Central
    {  403,  642,  462144,  462168,  462204, 24, 36, 34, }, // Europe_Eastern
    {  403,  657,       0,  462238,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  403,  680,  462269,  462293,  462327, 24, 34, 34, }, // Europe_Western
    {  403,  695,  462361,  462387,  462423, 26, 36, 36, }, // Falkland
    {  403,  704,  462459,  462467,  462483,  8, 16, 16, }, // Fiji
    {  403,  709,       0,  462499,       0,  0, 21,  0, }, // French_Guiana
    {  403,  723,       0,  462520,       0,  0, 29,  0, }, // French_Southern
    {  403,  746,       0,  462549,       0,  0, 19,  0, }, // Galapagos
    {  403,  756,       0,  462568,       0,  0, 10,  0, }, // Gambier
    {  403,  764,  462578,  462587,  462604,  9, 17, 17, }, // Georgia
    {  403,  772,       0,  462621,       0,  0, 19,  0, }, // Gilbert_Islands
    {  403,  788,       0,  462640,       0,  0, 14,  0, }, // GMT
    {  403,  830,  462654,  462677,  462705, 23, 28, 28, }, // Greenland_Eastern
    {  403,  848,  462733,  462753,  462781, 20, 28, 27, }, // Greenland_Western
    {  403,  866,       0,  462808,       0,  0,  7,  0, }, // Guam
    {  403,  871,       0,  462815,       0,  0, 13,  0, }, // Gulf
    {  403,  876,       0,  462828,       0,  0, 11,  0, }, // Guyana
    {  403,  883,  462839,  462856,  462881, 17, 25, 24, }, // Hawaii_Aleutian
    {  403,  899,  462905,  462918,  462942, 13, 24, 25, }, // Hong_Kong
    {  403,  909,  462967,  462979,  463003, 12, 24, 24, }, // Hovd
    {  403,  914,       0,  463027,       0,  0, 11,  0, }, // India
    {  403,  920,       0,  463038,       0,  0, 20,  0, }, // Indian_Ocean
    {  403,  933,       0,  463058,       0,  0, 12,  0, }, // Indochina
    {  403,  943,       0,  463070,       0,  0, 23,  0, }, // Indonesia_Central
    {  403,  961,       0,  463093,       0,  0, 31,  0, }, // Indonesia_Eastern
    {  403,  979,       0,  463124,       0,  0, 31,  0, }, // Indonesia_Western
    {  403,  997,  463155,  463167,  463189, 12, 22, 22, }, // Iran
    {  403, 1008,  463211,  463228,  463254, 17, 26, 26, }, // Irkutsk
    {  403, 1016,  463280,  463297,  463324, 17, 27, 19, }, // Israel
    {  403, 1023,  463343,  463357,  463382, 14, 25, 23, }, // Japan
    {  403, 1047,       0,  463405,       0,  0, 13,  0, }, // Kazakhstan
    {  403, 1058,       0,  463418,       0,  0, 30,  0, }, // Kazakhstan_Eastern
    {  403, 1077,       0,  463448,       0,  0, 30,  0, }, // Kazakhstan_Western
    {  403, 1106,  463478,  463489,  463514, 11, 25, 23, }, // Korea
    {  403, 1112,       0,  463537,       0,  0,  9,  0, }, // Kosrae
    {  403, 1119,  463546,  463567,  463598, 21, 31, 31, }, // Krasnoyarsk
    {  403, 1151,       0,  463629,       0,  0, 13,  0, }, // Kyrgystan
    {  403, 1161,       0,  463642,       0,  0,  9,  0, }, // Lanka
    {  403, 1175,       0,  463651,       0,  0, 18,  0, }, // Line_Islands
    {  403, 1188,  463669,  463683,  463708, 14, 25, 28, }, // Lord_Howe
    {  403, 1198,  463736,  463746,  463764, 10, 18, 18, }, // Macau
    {  403, 1204,  463782,  463795,  463816, 13, 21, 21, }, // Magadan
    {  403, 1219,       0,  463837,       0,  0, 15,  0, }, // Malaysia
    {  403, 1228,       0,  463852,       0,  0, 10,  0, }, // Maldives
    {  403, 1237,       0,  463862,       0,  0, 12,  0, }, // Marquesas
    {  403, 1247,       0,  463874,       0,  0, 21,  0, }, // Marshall_Islands
    {  403, 1264,  463895,  463913,  463941, 18, 28, 30, }, // Mauritius
    {  403, 1274,       0,  463971,       0,  0,  9,  0, }, // Mawson
    {  403, 1281,  463980,  464000,  464028, 20, 28, 27, }, // Mexico_Pacific
    {  403, 1296,  464055,  464070,  464093, 15, 23, 22, }, // Mongolia
    {  403, 1305,  464115,  464128,  464151, 13, 23, 23, }, // Moscow
    {  403, 1312,       0,  464174,       0,  0,  9,  0, }, // Myanmar
    {  403, 1320,       0,  464183,       0,  0, 14,  0, }, // Nauru
    {  403, 1326,       0,  464197,       0,  0, 13,  0, }, // Nepal
    {  403, 1332,  464210,  464226,  464250, 16, 24, 24, }, // New_Caledonia
    {  403, 1346,  464274,  464290,  464316, 16, 26, 25, }, // New_Zealand
    {  403, 1358,  464341,  464358,  464386, 17, 28, 20, }, // Newfoundland
    {  403, 1371,       0,  464406,       0,  0, 10,  0, }, // Niue
    {  403, 1376,  464416,  464436,  464466, 20, 30, 30, }, // Norfolk
    {  403, 1384,  464496,  464516,  464544, 20, 28, 28, }, // Noronha
    {  403, 1392,       0,  464572,       0,  0, 24,  0, }, // North_Mariana
    {  403, 1406,  464596,  464618,  464650, 22, 32, 32, }, // Novosibirsk
    {  403, 1418,  464682,  464696,  464720, 14, 24, 24, }, // Omsk
    {  403, 1428,  464744,  464759,  464785, 15, 26, 28, }, // Pakistan
    {  403, 1437,       0,  464813,       0,  0, 10,  0, }, // Palau
    {  403, 1443,       0,  464823,       0,  0, 13,  0, }, // Papua_New_Guinea
    {  403, 1460,  464836,  464851,  464877, 15, 26, 26, }, // Paraguay
    {  403, 1469,  464903,  464914,  464937, 11, 23, 22, }, // Peru
    {  403, 1474,  464959,  464976,  465003, 17, 27, 27, }, // Philippines
    {  403, 1486,       0,  465030,       0,  0, 17,  0, }, // Phoenix_Islands
    {  403, 1502,  465047,  465077,  465117, 30, 40, 38, }, // Pierre_Miquelon
    {  403, 1518,       0,  465155,       0,  0, 12,  0, }, // Pitcairn
    {  403, 1527,       0,  465167,       0,  0,  9,  0, }, // Ponape
    {  403, 1534,       0,  465176,       0,  0, 10,  0, }, // Pyongyang
    {  403, 1544,  465186,  465197,  465216, 11, 19, 19, }, // Qyzylorda
    {  403, 1554,       0,  465235,       0,  0, 18,  0, }, // Reunion
    {  403, 1562,       0,  465253,       0,  0, 11,  0, }, // Rothera
    {  403, 1570,  465264,  465279,  465304, 15, 25, 25, }, // Sakhalin
    {  403, 1596,  465329,  465338,  465355,  9, 17, 17, }, // Samoa
    {  403, 1602,       0,  465372,       0,  0, 16,  0, }, // Seychelles
    {  403, 1624,       0,  465388,       0,  0, 15,  0, }, // Singapore
    {  403, 1634,       0,  465403,       0,  0, 19,  0, }, // Solomon
    {  403, 1642,       0,  465422,       0,  0, 12,  0, }, // South_Georgia
    {  403, 1656,       0,  465434,       0,  0, 15,  0, }, // Suriname
    {  403, 1676,       0,  465449,       0,  0,  9,  0, }, // Syowa
    {  403, 1682,       0,  465458,       0,  0, 10,  0, }, // Tahiti
    {  403, 1689,  465468,  465479,  465501, 11, 22, 21, }, // Taipei
    {  403, 1696,       0,  465522,       0,  0, 14,  0, }, // Tajikistan
    {  403, 1724,       0,  465536,       0,  0, 12,  0, }, // Tokelau
    {  403, 1732,  465548,  465557,  465574,  9, 17, 17, }, // Tonga
    {  403, 1738,       0,  465591,       0,  0,  7,  0, }, // Truk
    {  403, 1750,  465598,  465613,  465636, 15, 23, 23, }, // Turkmenistan
    {  403, 1763,       0,  465659,       0,  0, 10,  0, }, // Tuvalu
    {  403, 1777,  465669,  465685,  465711, 16, 26, 26, }, // Uruguay
    {  403, 1792,  465737,  465754,  465779, 17, 25, 25, }, // Uzbekistan
    {  403, 1803,  465804,  465816,  465836, 12, 20, 20, }, // Vanuatu
    {  403, 1811,       0,  465856,       0,  0, 20,  0, }, // Venezuela
    {  403, 1821,  465876,  465890,  465912, 14, 22, 22, }, // Vladivostok
    {  403, 1833,  465934,  465946,  465966, 12, 20, 20, }, // Volgograd
    {  403, 1843,       0,  465986,       0,  0, 11,  0, }, // Vostok
    {  403, 1850,       0,  465997,       0,  0, 11,  0, }, // Wake
    {  403, 1855,       0,  466008,       0,  0, 21,  0, }, // Wallis
    {  403, 1862,  466029,  466039,  466057, 10, 18, 18, }, // Yakutsk
    {  403, 1870,  466075,  466092,  466117, 17, 25, 25, }, // Yekaterinburg
    {  403, 1892,       0,  466142,       0,  0,  9,  0, }, // Yukon
    {  405,    5,       0,  466151,       0,  0, 18,  0, }, // Afghanistan Latvian/Latin/Latvia
    {  405,   17,       0,  466169,       0,  0, 20,  0, }, // Africa_Central
    {  405,   32,       0,  466189,       0,  0, 20,  0, }, // Africa_Eastern
    {  405,   65,       0,  466209,       0,  0, 27,  0, }, // Africa_Southern
    {  405,   81,  466236,  466255,  466281, 19, 26, 27, }, // Africa_Western
    {  405,  107,  466308,  466321,  466341, 13, 20, 21, }, // Alaska
    {  405,  135,  466362,  466376,  466397, 14, 21, 22, }, // Amazon
    {  405,  142,  466419,  466435,  466458, 16, 23, 24, }, // America_Central
    {  405,  158,  466482,  466496,  466517, 14, 21, 22, }, // America_Eastern
    {  405,  174,  466539,  466550,  466568, 11, 18, 19, }, // America_Mountain
    {  405,  191,  466587,  466605,  466630, 18, 25, 26, }, // America_Pacific
    {  405,  207,  466656,  466670,  466691, 14, 21, 22, }, // Anadyr
    {  405,  214,  466713,  466725,  466744, 12, 19, 20, }, // Apia
    {  405,  232,  466764,  466787,  466817, 23, 30, 31, }, // Arabian
    {  405,  240,  466848,  466864,  466887, 16, 23, 24, }, // Argentina
    {  405,  250,  466911,  466933,  466962, 22, 29, 30, }, // Argentina_Western
    {  405,  268,  466992,  467007,  467029, 15, 22, 23, }, // Armenia
    {  405,  286,  467052,  467068,  467091, 16, 23, 24, }, // Atlantic
    {  405,  295,  467115,  467143,  467178, 28, 35, 36, }, // Australia_Central
    {  405,  313,  467214,  467250,  467293, 36, 43, 44, }, // Australia_CentralWestern
    {  405,  338,  467337,  467363,  467396, 26, 33, 34, }, // Australia_Eastern
    {  405,  356,  467430,  467455,  467487, 25, 32, 33, }, // Australia_Western
    {  405,  374,  467520,  467539,  467565, 19, 26, 27, }, // Azerbaijan
    {  405,  385,  467592,  467608,  467631, 16, 23, 24, }, // Azores
    {  405,  397,  467655,  467672,  467696, 17, 24, 25, }, // Bangladesh
    {  405,  415,       0,  467721,       0,  0, 13,  0, }, // Bhutan
    {  405,  422,       0,  467734,       0,  0, 15,  0, }, // Bolivia
    {  405,  437,  467749,  467765,  467788, 16, 23, 24, }, // Brasilia
    {  405,  454,       0,  467812,       0,  0, 26,  0, }, // Brunei
    {  405,  461,  467838,  467854,  467877, 16, 23, 24, }, // Cape_Verde
    {  405,  478,       0,  467901,       0,  0, 20,  0, }, // Chamorro
    {  405,  487,  467921,  467934,  467954, 13, 20, 21, }, // Chatham
    {  405,  495,  467975,  467986,  468004, 11, 18, 19, }, // Chile
    {  405,  501,  468023,  468034,  468052, 11, 18, 19, }, // China
    {  405,  507,       0,  468071,       0,  0, 22,  0, }, // Christmas
    {  405,  517,       0,  468093,       0,  0, 27,  0, }, // Cocos
    {  405,  523,  468120,  468136,  468159, 16, 23, 24, }, // Colombia
    {  405,  532,  468183,  468198,  468220, 15, 22, 23, }, // Cook
    {  405,  537,  468243,  468254,  468272, 11, 18, 19, }, // Cuba
    {  405,  548,       0,  468291,       0,  0, 14,  0, }, // Davis
    {  405,  564,       0,  468305,       0,  0, 19,  0, }, // DumontDUrville
    {  405,  601,       0,  468324,       0,  0, 20,  0, }, // East_Timor
    {  405,  612,  468344,  468365,  468393, 21, 28, 29, }, // Easter
    {  405,  619,       0,  468422,       0,  0, 15,  0, }, // Ecuador
    {  405,  627,  468437,  468457,  468484, 20, 27, 28, }, // Europe_Central
    {  405,  642,  468512,  468532,  468559, 20, 27, 28, }, // Europe_Eastern
    {  405,  657,       0,  468587,       0,  0, 32,  0, }, // Europe_Further_Eastern
    {  405,  680,  468619,  468638,  468664, 19, 26, 27, }, // Europe_Western
    {  405,  695,  468691,  468721,  468758, 30, 37, 38, }, // Falkland
    {  405,  704,  468796,  468807,  468825, 11, 18, 19, }, // Fiji
    {  405,  709,       0,  468844,       0,  0, 23,  0, }, // French_Guiana
    {  405,  723,       0,  468867,       0,  0, 54,  0, }, // French_Southern
    {  405,  746,       0,  468921,       0,  0, 14,  0, }, // Galapagos
    {  405,  756,       0,  468935,       0,  0, 17,  0, }, // Gambier
    {  405,  764,  468952,  468966,  468987, 14, 21, 22, }, // Georgia
    {  405,  772,       0,  469009,       0,  0, 19,  0, }, // Gilbert_Islands
    {  405,  788,       0,  469028,       0,  0, 14,  0, }, // GMT
    {  405,  830,  469042,  469065,  469095, 23, 30, 31, }, // Greenland_Eastern
    {  405,  848,  469126,  469148,  469177, 22, 29, 30, }, // Greenland_Western
    {  405,  871,       0,  469207,       0,  0, 19,  0, }, // Gulf
    {  405,  876,       0,  469226,       0,  0, 13,  0, }, // Guyana
    {  405,  883,  469239,  469258,  469284, 19, 26, 27, }, // Hawaii_Aleutian
    {  405,  899,  469311,  469326,  469348, 15, 22, 23, }, // Hong_Kong
    {  405,  909,  469371,  469383,  469402, 12, 19, 20, }, // Hovd
    {  405,  914,       0,  469422,       0,  0, 20,  0, }, // India
    {  405,  920,       0,  469442,       0,  0, 20,  0, }, // Indian_Ocean
    {  405,  933,       0,  469462,       0,  0, 15,  0, }, // Indochina
    {  405,  943,       0,  469477,       0,  0, 24,  0, }, // Indonesia_Central
    {  405,  961,       0,  469501,       0,  0, 24,  0, }, // Indonesia_Eastern
    {  405,  979,       0,  469525,       0,  0, 23,  0, }, // Indonesia_Western
    {  405,  997,  469548,  469560,  469579, 12, 19, 20, }, // Iran
    {  405, 1008,  469599,  469614,  469636, 15, 22, 23, }, // Irkutsk
    {  405, 1016,  469659,  469673,  469694, 14, 21, 22, }, // Israel
    {  405, 1023,  469716,  469729,  469749, 13, 20, 21, }, // Japan
    {  405, 1029,  469770,  469802,  469841, 32, 39, 40, }, // Kamchatka
    {  405, 1047,       0,  469881,       0,  0, 17,  0, }, // Kazakhstan
    {  405, 1058,       0,  469898,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  405, 1077,       0,  469922,       0,  0, 23,  0, }, // Kazakhstan_Western
    {  405, 1106,  469945,  469958,  469978, 13, 20, 21, }, // Korea
    {  405, 1112,       0,  469999,       0,  0, 12,  0, }, // Kosrae
    {  405, 1119,  470011,  470030,  470056, 19, 26, 27, }, // Krasnoyarsk
    {  405, 1151,       0,  470083,       0,  0, 18,  0, }, // Kyrgystan
    {  405, 1175,       0,  470101,       0,  0, 17,  0, }, // Line_Islands
    {  405, 1188,  470118,  470140,  470169, 22, 29, 30, }, // Lord_Howe
    {  405, 1204,  470199,  470214,  470236, 15, 22, 23, }, // Magadan
    {  405, 1219,       0,  470259,       0,  0, 16,  0, }, // Malaysia
    {  405, 1228,       0,  470275,       0,  0, 16,  0, }, // Maldives
    {  405, 1237,       0,  470291,       0,  0, 18,  0, }, // Marquesas
    {  405, 1247,       0,  470309,       0,  0, 18,  0, }, // Marshall_Islands
    {  405, 1264,  470327,  470343,  470366, 16, 23, 24, }, // Mauritius
    {  405, 1274,       0,  470390,       0,  0, 13,  0, }, // Mawson
    {  405, 1281,  470403,  470441,  470486, 38, 45, 46, }, // Mexico_Pacific
    {  405, 1296,  470532,  470549,  470573, 17, 24, 25, }, // Mongolia
    {  405, 1305,  470598,  470612,  470633, 14, 21, 22, }, // Moscow
    {  405, 1312,       0,  470655,       0,  0, 13,  0, }, // Myanmar
    {  405, 1320,       0,  470668,       0,  0, 11,  0, }, // Nauru
    {  405, 1326,       0,  470679,       0,  0, 13,  0, }, // Nepal
    {  405, 1332,  470692,  470713,  470741, 21, 28, 29, }, // New_Caledonia
    {  405, 1346,  470770,  470788,  470813, 18, 25, 26, }, // New_Zealand
    {  405, 1358,  470839,  470858,  470884, 19, 26, 27, }, // Newfoundland
    {  405, 1371,       0,  470911,       0,  0, 11,  0, }, // Niue
    {  405, 1376,  470922,  470943,  470971, 21, 28, 29, }, // Norfolk
    {  405, 1384,  471000,  471025,  471057, 25, 32, 33, }, // Noronha
    {  405, 1406,  471090,  471109,  471135, 19, 26, 27, }, // Novosibirsk
    {  405, 1418,  471162,  471174,  471193, 12, 19, 20, }, // Omsk
    {  405, 1428,  471213,  471229,  471252, 16, 23, 24, }, // Pakistan
    {  405, 1437,       0,  471276,       0,  0, 11,  0, }, // Palau
    {  405, 1443,       0,  471287,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  405, 1460,  471311,  471327,  471350, 16, 23, 24, }, // Paraguay
    {  405, 1469,  471374,  471384,  471401, 10, 17, 18, }, // Peru
    {  405, 1474,  471419,  471433,  471454, 14, 21, 22, }, // Philippines
    {  405, 1486,       0,  471476,       0,  0, 18,  0, }, // Phoenix_Islands
    {  405, 1502,  471494,  471522,  471557, 28, 35, 36, }, // Pierre_Miquelon
    {  405, 1518,       0,  471593,       0,  0, 15,  0, }, // Pitcairn
    {  405, 1527,       0,  471608,       0,  0, 12,  0, }, // Ponape
    {  405, 1534,       0,  471620,       0,  0, 15,  0, }, // Pyongyang
    {  405, 1554,       0,  471635,       0,  0, 15,  0, }, // Reunion
    {  405, 1562,       0,  471650,       0,  0, 13,  0, }, // Rothera
    {  405, 1570,  471663,  471678,  471700, 15, 22, 23, }, // Sakhalin
    {  405, 1579,  471723,  471736,  471756, 13, 20, 21, }, // Samara
    {  405, 1596,  471777,  471788,  471806, 11, 18, 19, }, // Samoa
    {  405, 1602,       0,  471825,       0,  0, 18,  0, }, // Seychelles
    {  405, 1624,       0,  471843,       0,  0, 16,  0, }, // Singapore
    {  405, 1634,       0,  471859,       0,  0, 19,  0, }, // Solomon
    {  405, 1642,       0,  471878,       0,  0, 23,  0, }, // South_Georgia
    {  405, 1656,       0,  471901,       0,  0, 15,  0, }, // Suriname
    {  405, 1676,       0,  471916,       0,  0, 11,  0, }, // Syowa
    {  405, 1682,       0,  471927,       0,  0, 11,  0, }, // Tahiti
    {  405, 1689,  471938,  471950,  471969, 12, 19, 20, }, // Taipei
    {  405, 1696,       0,  471989,       0,  0, 19,  0, }, // Tajikistan
    {  405, 1724,       0,  472008,       0,  0, 13,  0, }, // Tokelau
    {  405, 1732,  472021,  472033,  472052, 12, 19, 20, }, // Tonga
    {  405, 1738,       0,  472072,       0,  0, 11,  0, }, // Truk
    {  405, 1750,  472083,  472103,  472130, 20, 27, 28, }, // Turkmenistan
    {  405, 1763,       0,  472158,       0,  0, 12,  0, }, // Tuvalu
    {  405, 1777,  472170,  472185,  472207, 15, 22, 23, }, // Uruguay
    {  405, 1792,  472230,  472248,  472273, 18, 25, 26, }, // Uzbekistan
    {  405, 1803,  472299,  472312,  472332, 13, 20, 21, }, // Vanuatu
    {  405, 1811,       0,  472353,       0,  0, 16,  0, }, // Venezuela
    {  405, 1821,  472369,  472388,  472414, 19, 26, 27, }, // Vladivostok
    {  405, 1833,  472441,  472458,  472482, 17, 24, 25, }, // Volgograd
    {  405, 1843,       0,  472507,       0,  0, 14,  0, }, // Vostok
    {  405, 1850,       0,  472521,       0,  0, 17,  0, }, // Wake
    {  405, 1855,       0,  472538,       0,  0, 24,  0, }, // Wallis
    {  405, 1862,  472562,  472577,  472599, 15, 22, 23, }, // Yakutsk
    {  405, 1870,  472622,  472643,  472671, 21, 28, 29, }, // Yekaterinburg
    {  405, 1892,       0,  472700,       0,  0, 13,  0, }, // Yukon
    {  406,   17,       0,  472713,       0,  0, 20,  0, }, // Africa_Central Lingala/Latin/Congo - Kinshasa
    {  406,   32,       0,  472733,       0,  0, 26,  0, }, // Africa_Eastern
    {  406,   65,       0,  472759,       0,  0, 24,  0, }, // Africa_Southern
    {  406,  788,       0,  472783,       0,  0, 17,  0, }, // GMT
    {  406, 1602,       0,  472800,       0,  0, 17,  0, }, // Seychelles
    {  410,    0,  472817,  472827,  472850, 10, 23, 18, }, // Acre Lithuanian/Latin/Lithuania
    {  410,    5,       0,  472868,       0,  0, 18,  0, }, // Afghanistan
    {  410,   17,       0,  472886,       0,  0, 24,  0, }, // Africa_Central
    {  410,   32,       0,  472910,       0,  0, 19,  0, }, // Africa_Eastern
    {  410,   65,       0,  472929,       0,  0, 20,  0, }, // Africa_Southern
    {  410,   81,  472949,  472970,  472998, 21, 28, 29, }, // Africa_Western
    {  410,  107,  473027,  473042,  473064, 15, 22, 23, }, // Alaska
    {  410,  128,  473087,  473101,  473122, 14, 21, 22, }, // Almaty
    {  410,  135,  473144,  473159,  473181, 15, 22, 23, }, // Amazon
    {  410,  142,  473204,  473234,  473271, 30, 37, 38, }, // America_Central
    {  410,  158,  473309,  473337,  473372, 28, 35, 36, }, // America_Eastern
    {  410,  174,  473408,  473437,  473473, 29, 36, 37, }, // America_Mountain
    {  410,  191,  473510,  473551,  473599, 41, 48, 49, }, // America_Pacific
    {  410,  207,  473648,  473663,  473685, 15, 22, 23, }, // Anadyr
    {  410,  214,  473708,  473721,  473741, 13, 20, 21, }, // Apia
    {  410,  219,  473762,  473774,  473793, 12, 19, 20, }, // Aqtau
    {  410,  225,  473813,  473827,  473848, 14, 21, 22, }, // Aqtobe
    {  410,  232,  473870,  473885,  473907, 15, 22, 23, }, // Arabian
    {  410,  240,  473930,  473947,  473971, 17, 24, 25, }, // Argentina
    {  410,  250,  473996,  474020,  474051, 24, 31, 32, }, // Argentina_Western
    {  410,  268,  474083,  474099,  474122, 16, 23, 24, }, // Armenia
    {  410,  286,  474146,  474160,  474181, 14, 21, 22, }, // Atlantic
    {  410,  295,  474203,  474231,  474266, 28, 35, 36, }, // Australia_Central
    {  410,  313,  474302,  474337,  474379, 35, 42, 43, }, // Australia_CentralWestern
    {  410,  338,  474422,  474445,  474475, 23, 30, 31, }, // Australia_Eastern
    {  410,  356,  474506,  474531,  474563, 25, 32, 33, }, // Australia_Western
    {  410,  374,  474596,  474615,  474641, 19, 26, 27, }, // Azerbaijan
    {  410,  385,  474668,  474685,  474709, 17, 24, 25, }, // Azores
    {  410,  397,  474734,  474751,  474775, 17, 24, 25, }, // Bangladesh
    {  410,  415,       0,  474800,       0,  0, 13,  0, }, // Bhutan
    {  410,  422,       0,  474813,       0,  0, 16,  0, }, // Bolivia
    {  410,  437,  474829,  474846,  474870, 17, 24, 25, }, // Brasilia
    {  410,  454,       0,  474895,       0,  0, 27,  0, }, // Brunei
    {  410,  461,  474922,  474944,  474973, 22, 29, 30, }, // Cape_Verde
    {  410,  472,       0,  475003,       0,  0, 13,  0, }, // Casey
    {  410,  478,       0,  475016,       0,  0, 13,  0, }, // Chamorro
    {  410,  487,  475029,  475042,  475062, 13, 20, 21, }, // Chatham
    {  410,  495,  475083,  475095,  475114, 12, 19, 20, }, // Chile
    {  410,  501,  475134,  475148,  475169, 14, 21, 22, }, // China
    {  410,  507,       0,  475191,       0,  0, 19,  0, }, // Christmas
    {  410,  517,       0,  475210,       0,  0, 18,  0, }, // Cocos
    {  410,  523,  475228,  475245,  475269, 17, 24, 25, }, // Colombia
    {  410,  532,  475294,  475310,  475333, 16, 23, 30, }, // Cook
    {  410,  537,  475363,  475375,  475394, 12, 19, 20, }, // Cuba
    {  410,  548,       0,  475414,       0,  0, 14,  0, }, // Davis
    {  410,  564,       0,  475428,       0,  0, 24,  0, }, // DumontDUrville
    {  410,  601,       0,  475452,       0,  0, 18,  0, }, // East_Timor
    {  410,  612,  475470,  475489,  475515, 19, 26, 27, }, // Easter
    {  410,  619,       0,  475542,       0,  0, 15,  0, }, // Ecuador
    {  410,  627,  475557,  475579,  475608, 22, 29, 30, }, // Europe_Central
    {  410,  642,  475638,  475657,  475683, 19, 26, 27, }, // Europe_Eastern
    {  410,  657,       0,  475710,       0,  0, 28,  0, }, // Europe_Further_Eastern
    {  410,  680,  475738,  475759,  475787, 21, 28, 29, }, // Europe_Western
    {  410,  695,  475816,  475837,  475865, 21, 28, 29, }, // Falkland
    {  410,  704,  475894,  475907,  475927, 13, 20, 21, }, // Fiji
    {  410,  709,       0,  475948,       0,  0, 26,  0, }, // French_Guiana
    {  410,  723,       0,  475974,       0,  0, 39,  0, }, // French_Southern
    {  410,  746,       0,  476013,       0,  0, 15,  0, }, // Galapagos
    {  410,  756,       0,  476028,       0,  0, 14,  0, }, // Gambier
    {  410,  764,  476042,  476057,  476079, 15, 22, 23, }, // Georgia
    {  410,  772,       0,  476102,       0,  0, 20,  0, }, // Gilbert_Islands
    {  410,  788,       0,  476122,       0,  0, 15,  0, }, // GMT
    {  410,  830,  476137,  476161,  476192, 24, 31, 32, }, // Greenland_Eastern
    {  410,  848,  476224,  476250,  476283, 26, 33, 34, }, // Greenland_Western
    {  410,  866,       0,  476317,       0,  0, 12,  0, }, // Guam
    {  410,  871,       0,  476329,       0,  0, 23,  0, }, // Gulf
    {  410,  876,       0,  476352,       0,  0, 14,  0, }, // Guyana
    {  410,  883,  476366,  476386,  476413, 20, 27, 28, }, // Hawaii_Aleutian
    {  410,  899,  476441,  476456,  476478, 15, 22, 23, }, // Hong_Kong
    {  410,  909,  476501,  476513,  476532, 12, 19, 20, }, // Hovd
    {  410,  914,       0,  476552,       0,  0, 14,  0, }, // India
    {  410,  920,       0,  476566,       0,  0, 24,  0, }, // Indian_Ocean
    {  410,  933,       0,  476590,       0,  0, 18,  0, }, // Indochina
    {  410,  943,       0,  476608,       0,  0, 28,  0, }, // Indonesia_Central
    {  410,  961,       0,  476636,       0,  0, 23,  0, }, // Indonesia_Eastern
    {  410,  979,       0,  476659,       0,  0, 25,  0, }, // Indonesia_Western
    {  410,  997,  476684,  476696,  476715, 12, 19, 20, }, // Iran
    {  410, 1008,  476735,  476750,  476772, 15, 22, 23, }, // Irkutsk
    {  410, 1016,  476795,  476810,  476832, 15, 22, 23, }, // Israel
    {  410, 1023,  476855,  476871,  476894, 16, 23, 24, }, // Japan
    {  410, 1029,  476918,  476950,  476989, 32, 39, 40, }, // Kamchatka
    {  410, 1047,       0,  477029,       0,  0, 18,  0, }, // Kazakhstan
    {  410, 1058,       0,  477047,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  410, 1077,       0,  477070,       0,  0, 25,  0, }, // Kazakhstan_Western
    {  410, 1106,  477095,  477109,  477130, 14, 21, 22, }, // Korea
    {  410, 1112,       0,  477152,       0,  0, 14,  0, }, // Kosrae
    {  410, 1119,  477166,  477185,  477211, 19, 26, 27, }, // Krasnoyarsk
    {  410, 1151,       0,  477238,       0,  0, 17,  0, }, // Kyrgystan
    {  410, 1161,       0,  477255,       0,  0, 13,  0, }, // Lanka
    {  410, 1175,       0,  477268,       0,  0, 17,  0, }, // Line_Islands
    {  410, 1188,  477285,  477301,  477324, 16, 23, 24, }, // Lord_Howe
    {  410, 1198,  477348,  477360,  477379, 12, 19, 20, }, // Macau
    {  410, 1204,  477399,  477414,  477436, 15, 22, 23, }, // Magadan
    {  410, 1219,       0,  477459,       0,  0, 17,  0, }, // Malaysia
    {  410, 1228,       0,  477476,       0,  0, 14,  0, }, // Maldives
    {  410, 1237,       0,  477490,       0,  0, 19,  0, }, // Marquesas
    {  410, 1247,       0,  477509,       0,  0, 19,  0, }, // Marshall_Islands
    {  410, 1264,  477528,  477546,  477571, 18, 25, 26, }, // Mauritius
    {  410, 1274,       0,  477597,       0,  0, 13,  0, }, // Mawson
    {  410, 1281,  477610,  477643,  477683, 33, 40, 41, }, // Mexico_Pacific
    {  410, 1296,  477724,  477742,  477767, 18, 25, 26, }, // Mongolia
    {  410, 1305,  477793,  477807,  477828, 14, 21, 22, }, // Moscow
    {  410, 1312,       0,  477850,       0,  0, 15,  0, }, // Myanmar
    {  410, 1320,       0,  477865,       0,  0, 12,  0, }, // Nauru
    {  410, 1326,       0,  477877,       0,  0, 13,  0, }, // Nepal
    {  410, 1332,  477890,  477918,  477953, 28, 35, 36, }, // New_Caledonia
    {  410, 1346,  477989,  478016,  478050, 27, 34, 35, }, // New_Zealand
    {  410, 1358,  478085,  478105,  478132, 20, 27, 28, }, // Newfoundland
    {  410, 1371,       0,  478160,       0,  0, 12,  0, }, // Niue
    {  410, 1376,  478172,  478192,  478219, 20, 27, 28, }, // Norfolk
    {  410, 1384,  478247,  478274,  478308, 27, 34, 35, }, // Noronha
    {  410, 1392,       0,  478343,       0,  0, 28,  0, }, // North_Mariana
    {  410, 1406,  478371,  478390,  478416, 19, 26, 27, }, // Novosibirsk
    {  410, 1418,  478443,  478455,  478474, 12, 19, 20, }, // Omsk
    {  410, 1428,  478494,  478510,  478533, 16, 23, 24, }, // Pakistan
    {  410, 1437,       0,  478557,       0,  0, 12,  0, }, // Palau
    {  410, 1443,       0,  478569,       0,  0, 31,  0, }, // Papua_New_Guinea
    {  410, 1460,  478600,  478618,  478643, 18, 25, 26, }, // Paraguay
    {  410, 1469,  478669,  478680,  478698, 11, 18, 19, }, // Peru
    {  410, 1474,  478717,  478732,  478754, 15, 22, 23, }, // Philippines
    {  410, 1486,       0,  478777,       0,  0, 19,  0, }, // Phoenix_Islands
    {  410, 1502,  478796,  478824,  478859, 28, 35, 36, }, // Pierre_Miquelon
    {  410, 1518,       0,  478895,       0,  0, 15,  0, }, // Pitcairn
    {  410, 1527,       0,  478910,       0,  0, 14,  0, }, // Ponape
    {  410, 1534,       0,  478924,       0,  0, 16,  0, }, // Pyongyang
    {  410, 1544,  478940,  478957,  478981, 17, 24, 25, }, // Qyzylorda
    {  410, 1554,       0,  479006,       0,  0, 15,  0, }, // Reunion
    {  410, 1562,       0,  479021,       0,  0, 14,  0, }, // Rothera
    {  410, 1570,  479035,  479051,  479074, 16, 23, 24, }, // Sakhalin
    {  410, 1579,  479098,  479112,  479133, 14, 21, 22, }, // Samara
    {  410, 1596,  479155,  479167,  479186, 12, 19, 20, }, // Samoa
    {  410, 1602,       0,  479206,       0,  0, 15,  0, }, // Seychelles
    {  410, 1624,       0,  479221,       0,  0, 16,  0, }, // Singapore
    {  410, 1634,       0,  479237,       0,  0, 21,  0, }, // Solomon
    {  410, 1642,       0,  479258,       0,  0, 23,  0, }, // South_Georgia
    {  410, 1656,       0,  479281,       0,  0, 15,  0, }, // Suriname
    {  410, 1676,       0,  479296,       0,  0, 13,  0, }, // Syowa
    {  410, 1682,       0,  479309,       0,  0, 14,  0, }, // Tahiti
    {  410, 1689,  479323,  479339,  479362, 16, 23, 24, }, // Taipei
    {  410, 1696,       0,  479386,       0,  0, 19,  0, }, // Tajikistan
    {  410, 1724,       0,  479405,       0,  0, 14,  0, }, // Tokelau
    {  410, 1732,  479419,  479432,  479452, 13, 20, 21, }, // Tonga
    {  410, 1738,       0,  479473,       0,  0, 11,  0, }, // Truk
    {  410, 1750,  479484,  479504,  479531, 20, 27, 28, }, // Turkmenistan
    {  410, 1763,       0,  479559,       0,  0, 13,  0, }, // Tuvalu
    {  410, 1777,  479572,  479589,  479613, 17, 24, 25, }, // Uruguay
    {  410, 1792,  479638,  479656,  479681, 18, 25, 26, }, // Uzbekistan
    {  410, 1803,  479707,  479721,  479742, 14, 21, 22, }, // Vanuatu
    {  410, 1811,       0,  479764,       0,  0, 17,  0, }, // Venezuela
    {  410, 1821,  479781,  479800,  479826, 19, 26, 27, }, // Vladivostok
    {  410, 1833,  479853,  479870,  479894, 17, 24, 25, }, // Volgograd
    {  410, 1843,       0,  479919,       0,  0, 14,  0, }, // Vostok
    {  410, 1850,       0,  479933,       0,  0, 18,  0, }, // Wake
    {  410, 1855,       0,  479951,       0,  0, 24,  0, }, // Wallis
    {  410, 1862,  479975,  479990,  480012, 15, 22, 23, }, // Yakutsk
    {  410, 1870,  480035,  480056,  480084, 21, 28, 29, }, // Yekaterinburg
    {  410, 1892,       0,  480113,       0,  0, 13,  0, }, // Yukon
    {  411,  523,  480126,  480148,  480176, 22, 28, 25, }, // Colombia Lojban/Latin/world
    {  411,  746,       0,  480201,       0,  0, 21,  0, }, // Galapagos
    {  412,    5,       0,  480222,       0,  0, 14,  0, }, // Afghanistan Lower Sorbian/Latin/Germany
    {  412,   17,       0,  480236,       0,  0, 18,  0, }, // Africa_Central
    {  412,   32,       0,  480254,       0,  0, 21,  0, }, // Africa_Eastern
    {  412,   65,       0,  480275,       0,  0, 23,  0, }, // Africa_Southern
    {  412,   81,  480298,  480320,  480353, 22, 33, 31, }, // Africa_Western
    {  412,  107,  480384,  480398,  480423, 14, 25, 23, }, // Alaska
    {  412,  135,  480446,  480460,  480485, 14, 25, 23, }, // Amazon
    {  412,  142,  480508,  480541,  480585, 33, 44, 42, }, // America_Central
    {  412,  158,  480627,  480661,  480706, 34, 45, 43, }, // America_Eastern
    {  412,  174,  480749,  480779,  480820, 30, 41, 39, }, // America_Mountain
    {  412,  191,  480859,  480892,  480936, 33, 44, 42, }, // America_Pacific
    {  412,  214,  480978,  480989,  481011, 11, 22, 20, }, // Apia
    {  412,  232,  481031,  481043,  481066, 12, 23, 21, }, // Arabian
    {  412,  240,  481087,  481102,  481128, 15, 26, 24, }, // Argentina
    {  412,  250,  481152,  481178,  481215, 26, 37, 35, }, // Argentina_Western
    {  412,  268,  481250,  481262,  481285, 12, 23, 21, }, // Armenia
    {  412,  286,  481306,  481320,  481345, 14, 25, 23, }, // Atlantic
    {  412,  295,  481368,  481389,  481421, 21, 32, 30, }, // Australia_Central
    {  412,  313,  481451,  481484,  481528, 33, 44, 42, }, // Australia_CentralWestern
    {  412,  338,  481570,  481594,  481629, 24, 35, 33, }, // Australia_Eastern
    {  412,  356,  481662,  481687,  481723, 25, 36, 34, }, // Australia_Western
    {  412,  374,  481757,  481776,  481806, 19, 30, 28, }, // Azerbaijan
    {  412,  385,  481834,  481845,  481867, 11, 22, 20, }, // Azores
    {  412,  397,  481887,  481903,  481930, 16, 27, 25, }, // Bangladesh
    {  412,  415,       0,  481955,       0,  0, 13,  0, }, // Bhutan
    {  412,  422,       0,  481968,       0,  0, 13,  0, }, // Bolivia
    {  412,  437,  481981,  481993,  482016, 12, 23, 21, }, // Brasilia
    {  412,  454,       0,  482037,       0,  0, 13,  0, }, // Brunei
    {  412,  461,  482050,  482064,  482089, 14, 25, 23, }, // Cape_Verde
    {  412,  478,       0,  482112,       0,  0, 14,  0, }, // Chamorro
    {  412,  487,  482126,  482140,  482165, 14, 25, 23, }, // Chatham
    {  412,  495,  482188,  482199,  482221, 11, 22, 20, }, // Chile
    {  412,  501,  482241,  482252,  482274, 11, 22, 20, }, // China
    {  412,  507,       0,  482294,       0,  0, 19,  0, }, // Christmas
    {  412,  517,       0,  482313,       0,  0, 20,  0, }, // Cocos
    {  412,  523,  482333,  482347,  482372, 14, 25, 23, }, // Colombia
    {  412,  532,  482395,  482414,  482444, 19, 30, 28, }, // Cook
    {  412,  537,  482472,  482484,  482507, 12, 23, 21, }, // Cuba
    {  412,  548,       0,  482528,       0,  0,  9,  0, }, // Davis
    {  412,  564,       0,  482537,       0,  0, 18,  0, }, // DumontDUrville
    {  412,  601,       0,  482555,       0,  0, 22,  0, }, // East_Timor
    {  412,  612,  482577,  482596,  482626, 19, 30, 28, }, // Easter
    {  412,  619,       0,  482654,       0,  0, 14,  0, }, // Ecuador
    {  412,  627,  482668,  482690,  482723, 22, 33, 31, }, // Europe_Central
    {  412,  642,  482754,  482778,  482813, 24, 35, 33, }, // Europe_Eastern
    {  412,  657,       0,  482846,       0,  0, 18,  0, }, // Europe_Further_Eastern
    {  412,  680,  482864,  482889,  482925, 25, 36, 34, }, // Europe_Western
    {  412,  695,  482959,  482974,  483000, 15, 26, 24, }, // Falkland
    {  412,  704,  483024,  483036,  483059, 12, 23, 21, }, // Fiji
    {  412,  709,       0,  483080,       0,  0, 22,  0, }, // French_Guiana
    {  412,  723,       0,  483102,       0,  0, 59,  0, }, // French_Southern
    {  412,  746,       0,  483161,       0,  0, 15,  0, }, // Galapagos
    {  412,  756,       0,  483176,       0,  0, 14,  0, }, // Gambier
    {  412,  764,  483190,  483203,  483227, 13, 24, 22, }, // Georgia
    {  412,  772,       0,  483249,       0,  0, 22,  0, }, // Gilbert_Islands
    {  412,  788,       0,  483271,       0,  0, 16,  0, }, // GMT
    {  412,  830,  483287,  483312,  483348, 25, 36, 34, }, // Greenland_Eastern
    {  412,  848,  483382,  483408,  483445, 26, 37, 35, }, // Greenland_Western
    {  412,  871,       0,  483480,       0,  0, 20,  0, }, // Gulf
    {  412,  876,       0,  483500,       0,  0, 12,  0, }, // Guyana
    {  412,  883,  483512,  483534,  483567, 22, 33, 31, }, // Hawaii_Aleutian
    {  412,  899,  483598,  483613,  483639, 15, 26, 24, }, // Hong_Kong
    {  412,  909,  483663,  483675,  483698, 12, 23, 21, }, // Hovd
    {  412,  914,       0,  483719,       0,  0, 11,  0, }, // India
    {  412,  920,       0,  483730,       0,  0, 20,  0, }, // Indian_Ocean
    {  412,  933,       0,  483750,       0,  0, 15,  0, }, // Indochina
    {  412,  943,       0,  483765,       0,  0, 20,  0, }, // Indonesia_Central
    {  412,  961,       0,  483785,       0,  0, 19,  0, }, // Indonesia_Eastern
    {  412,  979,       0,  483804,       0,  0, 24,  0, }, // Indonesia_Western
    {  412,  997,  483828,  483839,  483861, 11, 22, 20, }, // Iran
    {  412, 1008,  483881,  483893,  483916, 12, 23, 21, }, // Irkutsk
    {  412, 1016,  483937,  483950,  483974, 13, 24, 22, }, // Israel
    {  412, 1023,  483996,  484008,  484031, 12, 23, 21, }, // Japan
    {  412, 1047,       0,  484052,       0,  0, 14,  0, }, // Kazakhstan
    {  412, 1058,       0,  484066,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  412, 1077,       0,  484089,       0,  0, 24,  0, }, // Kazakhstan_Western
    {  412, 1106,  484113,  484125,  484148, 12, 23, 21, }, // Korea
    {  412, 1112,       0,  484169,       0,  0, 13,  0, }, // Kosrae
    {  412, 1119,  484182,  484198,  484225, 16, 27, 25, }, // Krasnoyarsk
    {  412, 1151,       0,  484250,       0,  0, 12,  0, }, // Kyrgystan
    {  412, 1175,       0,  484262,       0,  0, 20,  0, }, // Line_Islands
    {  412, 1188,  484282,  484300,  484329, 18, 29, 27, }, // Lord_Howe
    {  412, 1204,  484356,  484370,  484395, 14, 25, 23, }, // Magadan
    {  412, 1219,       0,  484418,       0,  0, 14,  0, }, // Malaysia
    {  412, 1228,       0,  484432,       0,  0, 14,  0, }, // Maldives
    {  412, 1237,       0,  484446,       0,  0, 13,  0, }, // Marquesas
    {  412, 1247,       0,  484459,       0,  0, 23,  0, }, // Marshall_Islands
    {  412, 1264,  484482,  484496,  484521, 14, 25, 23, }, // Mauritius
    {  412, 1274,       0,  484544,       0,  0, 10,  0, }, // Mawson
    {  412, 1281,  484554,  484575,  484607, 21, 32, 30, }, // Mexico_Pacific
    {  412, 1296,  484637,  484654,  484682, 17, 28, 26, }, // Mongolia
    {  412, 1305,  484708,  484721,  484745, 13, 24, 22, }, // Moscow
    {  412, 1312,       0,  484767,       0,  0, 14,  0, }, // Myanmar
    {  412, 1320,       0,  484781,       0,  0, 12,  0, }, // Nauru
    {  412, 1326,       0,  484793,       0,  0, 12,  0, }, // Nepal
    {  412, 1332,  484805,  484824,  484854, 19, 30, 28, }, // New_Caledonia
    {  412, 1346,  484882,  484900,  484929, 18, 29, 27, }, // New_Zealand
    {  412, 1358,  484956,  484975,  485005, 19, 30, 28, }, // Newfoundland
    {  412, 1371,       0,  485033,       0,  0, 11,  0, }, // Niue
    {  412, 1376,  485044,  485060,  485087, 16, 27, 25, }, // Norfolk
    {  412, 1384,  485112,  485135,  485169, 23, 34, 32, }, // Noronha
    {  412, 1406,  485201,  485217,  485244, 16, 27, 25, }, // Novosibirsk
    {  412, 1418,  485269,  485278,  485298,  9, 20, 18, }, // Omsk
    {  412, 1428,  485316,  485331,  485357, 15, 26, 24, }, // Pakistan
    {  412, 1437,       0,  485381,       0,  0, 12,  0, }, // Palau
    {  412, 1443,       0,  485393,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  412, 1460,  485415,  485430,  485456, 15, 26, 24, }, // Paraguay
    {  412, 1469,  485480,  485491,  485513, 11, 22, 20, }, // Peru
    {  412, 1474,  485533,  485547,  485572, 14, 25, 23, }, // Philippines
    {  412, 1486,       0,  485595,       0,  0, 22,  0, }, // Phoenix_Islands
    {  412, 1502,  485617,  485645,  485684, 28, 39, 37, }, // Pierre_Miquelon
    {  412, 1518,       0,  485721,       0,  0, 23,  0, }, // Pitcairn
    {  412, 1527,       0,  485744,       0,  0, 12,  0, }, // Ponape
    {  412, 1534,       0,  485756,       0,  0, 16,  0, }, // Pyongyang
    {  412, 1554,       0,  485772,       0,  0, 14,  0, }, // Reunion
    {  412, 1562,       0,  485786,       0,  0, 11,  0, }, // Rothera
    {  412, 1570,  485797,  485812,  485838, 15, 26, 24, }, // Sakhalin
    {  412, 1596,  485862,  485874,  485897, 12, 23, 21, }, // Samoa
    {  412, 1602,       0,  485918,       0,  0, 14,  0, }, // Seychelles
    {  412, 1624,       0,  485932,       0,  0, 15,  0, }, // Singapore
    {  412, 1634,       0,  485947,       0,  0, 14,  0, }, // Solomon
    {  412, 1642,       0,  485961,       0,  0, 25,  0, }, // South_Georgia
    {  412, 1656,       0,  485986,       0,  0, 14,  0, }, // Suriname
    {  412, 1676,       0,  486000,       0,  0,  9,  0, }, // Syowa
    {  412, 1682,       0,  486009,       0,  0, 13,  0, }, // Tahiti
    {  412, 1689,  486022,  486037,  486063, 15, 26, 24, }, // Taipei
    {  412, 1696,       0,  486087,       0,  0, 14,  0, }, // Tajikistan
    {  412, 1724,       0,  486101,       0,  0, 14,  0, }, // Tokelau
    {  412, 1732,  486115,  486127,  486150, 12, 23, 21, }, // Tonga
    {  412, 1738,       0,  486171,       0,  0, 12,  0, }, // Truk
    {  412, 1750,  486183,  486198,  486224, 15, 26, 24, }, // Turkmenistan
    {  412, 1763,       0,  486248,       0,  0, 12,  0, }, // Tuvalu
    {  412, 1777,  486260,  486274,  486299, 14, 25, 23, }, // Uruguay
    {  412, 1792,  486322,  486335,  486359, 13, 24, 22, }, // Uzbekistan
    {  412, 1803,  486381,  486394,  486418, 13, 24, 22, }, // Vanuatu
    {  412, 1811,       0,  486440,       0,  0, 15,  0, }, // Venezuela
    {  412, 1821,  486455,  486473,  486502, 18, 29, 27, }, // Vladivostok
    {  412, 1833,  486529,  486545,  486572, 16, 27, 25, }, // Volgograd
    {  412, 1843,       0,  486597,       0,  0, 10,  0, }, // Vostok
    {  412, 1850,       0,  486607,       0,  0, 13,  0, }, // Wake
    {  412, 1855,       0,  486620,       0,  0, 25,  0, }, // Wallis
    {  412, 1862,  486645,  486657,  486680, 12, 23, 21, }, // Yakutsk
    {  412, 1870,  486701,  486721,  486752, 20, 31, 29, }, // Yekaterinburg
    {  412, 1892,       0,  486781,       0,  0, 12,  0, }, // Yukon
    {  413,   17,       0,  486793,       0,  0, 24,  0, }, // Africa_Central Low German/Latin/Germany
    {  413,   32,       0,  486817,       0,  0, 20,  0, }, // Africa_Eastern
    {  413,   65,       0,  486837,       0,  0, 20,  0, }, // Africa_Southern
    {  413,   81,  486857,  486877,  486905, 20, 28, 26, }, // Africa_Western
    {  413,  142,  486931,  486961,  487000, 30, 39, 37, }, // America_Central
    {  413,  158,  487037,  487066,  487103, 29, 37, 35, }, // America_Eastern
    {  413,  174,  487138,  487164,  487199, 26, 35, 33, }, // America_Mountain
    {  413,  191,  487232,  487261,  487299, 29, 38, 36, }, // America_Pacific
    {  413,  232,  487335,  487348,  487369, 13, 21, 19, }, // Arabian
    {  413,  286,  487388,  487418,  487457, 30, 39, 37, }, // Atlantic
    {  413,  295,  487494,  487518,  487550, 24, 32, 30, }, // Australia_Central
    {  413,  313,  487580,  487608,  487644, 28, 36, 34, }, // Australia_CentralWestern
    {  413,  338,  487678,  487698,  487726, 20, 28, 26, }, // Australia_Eastern
    {  413,  356,  487752,  487772,  487800, 20, 28, 26, }, // Australia_Western
    {  413,  501,  487826,  487836,  487854, 10, 18, 16, }, // China
    {  413,  627,  487870,  487893,  487924, 23, 31, 29, }, // Europe_Central
    {  413,  642,  487953,  487972,  487999, 19, 27, 25, }, // Europe_Eastern
    {  413,  680,  488024,  488043,  488070, 19, 27, 25, }, // Europe_Western
    {  413,  788,       0,  488095,       0,  0, 19,  0, }, // GMT
    {  413,  914,       0,  488114,       0,  0, 11,  0, }, // India
    {  413,  933,       0,  488125,       0,  0, 22,  0, }, // Indochina
    {  413,  943,       0,  488147,       0,  0, 23,  0, }, // Indonesia_Central
    {  413,  961,       0,  488170,       0,  0, 19,  0, }, // Indonesia_Eastern
    {  413,  979,       0,  488189,       0,  0, 19,  0, }, // Indonesia_Western
    {  413, 1016,  488208,  488219,  488238, 11, 19, 17, }, // Israel
    {  413, 1023,  488255,  488269,  488291, 14, 22, 20, }, // Japan
    {  413, 1106,  488311,  488326,  488349, 15, 23, 21, }, // Korea
    {  413, 1305,  488370,  488381,  488400, 11, 19, 17, }, // Moscow
    {  419,    0,  488417,  488426,  488441,  9, 15, 15, }, // Acre Luxembourgish/Latin/Luxembourg
    {  419,    5,       0,  488456,       0,  0, 16,  0, }, // Afghanistan
    {  419,   17,       0,  488472,       0,  0, 23,  0, }, // Africa_Central
    {  419,   32,       0,  488495,       0,  0, 19,  0, }, // Africa_Eastern
    {  419,   65,       0,  488514,       0,  0, 19,  0, }, // Africa_Southern
    {  419,   81,  488533,  488553,  488579, 20, 26, 26, }, // Africa_Western
    {  419,  107,  488605,  488616,  488633, 11, 17, 17, }, // Alaska
    {  419,  128,  488650,  488661,  488678, 11, 17, 17, }, // Almaty
    {  419,  135,  488695,  488708,  488727, 13, 19, 19, }, // Amazon
    {  419,  142,  488746,  488773,  488807, 27, 34, 34, }, // America_Central
    {  419,  158,  488841,  488871,  488908, 30, 37, 37, }, // America_Eastern
    {  419,  174,  488945,  488964,  488989, 19, 25, 25, }, // America_Mountain
    {  419,  191,  489014,  489045,  489083, 31, 38, 38, }, // America_Pacific
    {  419,  207,  489121,  489132,  489149, 11, 17, 17, }, // Anadyr
    {  419,  232,  489166,  489179,  489198, 13, 19, 19, }, // Arabian
    {  419,  240,  489217,  489234,  489257, 17, 23, 23, }, // Argentina
    {  419,  250,  489280,  489301,  489328, 21, 27, 27, }, // Argentina_Western
    {  419,  268,  489355,  489369,  489389, 14, 20, 20, }, // Armenia
    {  419,  286,  489409,  489422,  489441, 13, 19, 19, }, // Atlantic
    {  419,  295,  489460,  489483,  489512, 23, 29, 29, }, // Australia_Central
    {  419,  313,  489541,  489570,  489605, 29, 35, 35, }, // Australia_CentralWestern
    {  419,  338,  489640,  489659,  489684, 19, 25, 25, }, // Australia_Eastern
    {  419,  356,  489550,  489579,  489614, 20, 26, 26, }, // Australia_Western
    {  419,  374,  489709,  489731,  489759, 22, 28, 28, }, // Azerbaijan
    {  419,  385,  489787,  489798,  489815, 11, 17, 17, }, // Azores
    {  419,  397,  489832,  489848,  489870, 16, 22, 22, }, // Bangladesh
    {  419,  415,       0,  489892,       0,  0, 11,  0, }, // Bhutan
    {  419,  422,       0,  489903,       0,  0, 17,  0, }, // Bolivia
    {  419,  437,  489920,  489933,  489952, 13, 19, 19, }, // Brasilia
    {  419,  454,       0,  489971,       0,  0, 11,  0, }, // Brunei
    {  419,  461,  489982,  489996,  490016, 14, 20, 20, }, // Cape_Verde
    {  419,  478,       0,  490036,       0,  0, 13,  0, }, // Chamorro
    {  419,  487,  490049,  490061,  490079, 12, 18, 18, }, // Chatham
    {  419,  495,  490097,  490112,  490133, 15, 21, 21, }, // Chile
    {  419,  501,  490154,  490169,  490190, 15, 21, 21, }, // China
    {  419,  507,       0,  490211,       0,  0, 22,  0, }, // Christmas
    {  419,  517,       0,  490233,       0,  0, 17,  0, }, // Cocos
    {  419,  523,  490250,  490268,  490292, 18, 24, 24, }, // Colombia
    {  419,  532,  490316,  490332,  490354, 16, 22, 22, }, // Cook
    {  419,  537,  490376,  490390,  490410, 14, 20, 20, }, // Cuba
    {  419,  548,       0,  490430,       0,  0, 10,  0, }, // Davis
    {  419,  564,       0,  490440,       0,  0, 21,  0, }, // DumontDUrville
    {  419,  601,       0,  490461,       0,  0, 13,  0, }, // East_Timor
    {  419,  612,  490474,  490492,  490516, 18, 24, 24, }, // Easter
    {  419,  619,       0,  490540,       0,  0, 19,  0, }, // Ecuador
    {  419,  627,  490559,  490580,  490607, 21, 27, 27, }, // Europe_Central
    {  419,  642,  490634,  490652,  490676, 18, 24, 24, }, // Europe_Eastern
    {  419,  680,  490700,  490719,  490744, 19, 25, 25, }, // Europe_Western
    {  419,  695,  490769,  490789,  490815, 20, 26, 26, }, // Falkland
    {  419,  704,  490841,  490853,  490871, 12, 18, 18, }, // Fiji
    {  419,  709,       0,  490889,       0,  0, 23,  0, }, // French_Guiana
    {  419,  723,       0,  490912,       0,  0, 41,  0, }, // French_Southern
    {  419,  746,       0,  490953,       0,  0, 14,  0, }, // Galapagos
    {  419,  756,       0,  490967,       0,  0, 12,  0, }, // Gambier
    {  419,  764,  490979,  490993,  491013, 14, 20, 20, }, // Georgia
    {  419,  772,       0,  491033,       0,  0, 20,  0, }, // Gilbert_Islands
    {  419,  788,       0,  491053,       0,  0, 22,  0, }, // GMT
    {  419,  830,  491075,  491091,  491113, 16, 22, 22, }, // Greenland_Eastern
    {  419,  848,  491135,  491152,  491175, 17, 23, 23, }, // Greenland_Western
    {  419,  866,       0,  491198,       0,  0,  9,  0, }, // Guam
    {  419,  871,       0,  491207,       0,  0,  9,  0, }, // Gulf
    {  419,  876,       0,  491216,       0,  0, 11,  0, }, // Guyana
    {  419,  883,  491227,  491246,  491271, 19, 25, 25, }, // Hawaii_Aleutian
    {  419,  899,  491296,  491310,  491330, 14, 20, 20, }, // Hong_Kong
    {  419,  909,  491350,  491359,  491374,  9, 15, 15, }, // Hovd
    {  419,  914,       0,  491389,       0,  0, 12,  0, }, // India
    {  419,  920,       0,  491401,       0,  0, 20,  0, }, // Indian_Ocean
    {  419,  933,       0,  491421,       0,  0, 14,  0, }, // Indochina
    {  419,  943,       0,  491435,       0,  0, 23,  0, }, // Indonesia_Central
    {  419,  961,       0,  491458,       0,  0, 19,  0, }, // Indonesia_Eastern
    {  419,  979,       0,  491477,       0,  0, 20,  0, }, // Indonesia_Western
    {  419,  997,  491497,  491510,  491529, 13, 19, 19, }, // Iran
    {  419, 1008,  491548,  491560,  491578, 12, 18, 18, }, // Irkutsk
    {  419, 1016,  491596,  491611,  491632, 15, 21, 21, }, // Israel
    {  419, 1023,  491653,  491667,  491687, 14, 20, 20, }, // Japan
    {  419, 1029,  491707,  491723,  491745, 16, 22, 22, }, // Kamchatka
    {  419, 1058,       0,  491767,       0,  0, 18,  0, }, // Kazakhstan_Eastern
    {  419, 1077,       0,  491785,       0,  0, 19,  0, }, // Kazakhstan_Western
    {  419, 1106,  491804,  491819,  491840, 15, 21, 21, }, // Korea
    {  419, 1112,       0,  491861,       0,  0, 11,  0, }, // Kosrae
    {  419, 1119,  491872,  491888,  491910, 16, 22, 22, }, // Krasnoyarsk
    {  419, 1151,       0,  491932,       0,  0, 16,  0, }, // Kyrgystan
    {  419, 1175,       0,  491948,       0,  0, 18,  0, }, // Line_Islands
    {  419, 1188,  491966,  491980,  492000, 14, 20, 20, }, // Lord_Howe
    {  419, 1204,  492020,  492032,  492050, 12, 18, 18, }, // Magadan
    {  419, 1219,       0,  492068,       0,  0, 15,  0, }, // Malaysia
    {  419, 1228,       0,  492083,       0,  0, 13,  0, }, // Maldives
    {  419, 1237,       0,  492096,       0,  0, 14,  0, }, // Marquesas
    {  419, 1247,       0,  492110,       0,  0, 20,  0, }, // Marshall_Islands
    {  419, 1264,  492130,  492144,  492164, 14, 20, 20, }, // Mauritius
    {  419, 1274,       0,  492184,       0,  0, 11,  0, }, // Mawson
    {  419, 1281,  492195,  492218,  492248, 23, 30, 30, }, // Mexico_Pacific
    {  419, 1296,  492278,  492294,  492316, 16, 22, 22, }, // Mongolia
    {  419, 1305,  492338,  492351,  492370, 13, 19, 19, }, // Moscow
    {  419, 1312,       0,  492389,       0,  0, 12,  0, }, // Myanmar
    {  419, 1320,       0,  492401,       0,  0, 10,  0, }, // Nauru
    {  419, 1326,       0,  492411,       0,  0, 16,  0, }, // Nepal
    {  419, 1332,  492427,  492446,  492471, 19, 25, 25, }, // New_Caledonia
    {  419, 1346,  492496,  492511,  492532, 15, 21, 21, }, // New_Zealand
    {  419, 1358,  492553,  492569,  492591, 16, 22, 22, }, // Newfoundland
    {  419, 1371,       0,  492613,       0,  0,  9,  0, }, // Niue
    {  419, 1376,  492622,  492641,  492666, 19, 25, 25, }, // Norfolk
    {  419, 1384,  492691,  492715,  492745, 24, 30, 30, }, // Noronha
    {  419, 1406,  492775,  492791,  492813, 16, 22, 22, }, // Novosibirsk
    {  419, 1418,  492835,  492844,  492859,  9, 15, 15, }, // Omsk
    {  419, 1428,  492874,  492891,  492914, 17, 23, 23, }, // Pakistan
    {  419, 1437,       0,  492937,       0,  0, 10,  0, }, // Palau
    {  419, 1443,       0,  492947,       0,  0, 20,  0, }, // Papua_New_Guinea
    {  419, 1460,  492967,  492986,  493011, 19, 25, 25, }, // Paraguay
    {  419, 1469,  493036,  493051,  493072, 15, 21, 21, }, // Peru
    {  419, 1474,  493093,  493112,  493137, 19, 25, 25, }, // Philippines
    {  419, 1486,       0,  493162,       0,  0, 19,  0, }, // Phoenix_Islands
    {  419, 1502,  493181,  493209,  493243, 28, 34, 34, }, // Pierre_Miquelon
    {  419, 1518,       0,  493277,       0,  0, 20,  0, }, // Pitcairn
    {  419, 1527,       0,  493297,       0,  0, 11,  0, }, // Ponape
    {  419, 1554,       0,  493308,       0,  0, 12,  0, }, // Reunion
    {  419, 1562,       0,  493320,       0,  0, 12,  0, }, // Rothera
    {  419, 1570,  493332,  493345,  493364, 13, 19, 19, }, // Sakhalin
    {  419, 1579,  493383,  493394,  493411, 11, 17, 17, }, // Samara
    {  419, 1596,  493428,  493438,  493454, 10, 16, 16, }, // Samoa
    {  419, 1602,       0,  493470,       0,  0, 15,  0, }, // Seychelles
    {  419, 1624,       0,  493485,       0,  0, 21,  0, }, // Singapore
    {  419, 1634,       0,  493506,       0,  0, 19,  0, }, // Solomon
    {  419, 1642,       0,  493525,       0,  0, 17,  0, }, // South_Georgia
    {  419, 1656,       0,  493542,       0,  0, 13,  0, }, // Suriname
    {  419, 1676,       0,  493555,       0,  0, 10,  0, }, // Syowa
    {  419, 1682,       0,  493565,       0,  0, 11,  0, }, // Tahiti
    {  419, 1689,  493576,  493587,  493604, 11, 17, 17, }, // Taipei
    {  419, 1696,       0,  493621,       0,  0, 18,  0, }, // Tajikistan
    {  419, 1724,       0,  493639,       0,  0, 12,  0, }, // Tokelau
    {  419, 1732,  493651,  493666,  493687, 15, 21, 21, }, // Tonga
    {  419, 1738,       0,  493708,       0,  0, 10,  0, }, // Truk
    {  419, 1750,  493718,  493735,  493758, 17, 23, 23, }, // Turkmenistan
    {  419, 1763,       0,  493781,       0,  0, 11,  0, }, // Tuvalu
    {  419, 1777,  493792,  493810,  493833, 18, 23, 24, }, // Uruguay
    {  419, 1792,  493857,  493872,  493893, 15, 21, 21, }, // Uzbekistan
    {  419, 1803,  493914,  493926,  493944, 12, 18, 18, }, // Vanuatu
    {  419, 1811,       0,  493962,       0,  0, 14,  0, }, // Venezuela
    {  419, 1821,  493976,  493992,  494014, 16, 22, 22, }, // Vladivostok
    {  419, 1833,  494036,  494050,  494070, 14, 20, 20, }, // Volgograd
    {  419, 1843,       0,  494090,       0,  0, 11,  0, }, // Vostok
    {  419, 1850,       0,  494101,       0,  0, 15,  0, }, // Wake
    {  419, 1855,       0,  494116,       0,  0, 20,  0, }, // Wallis
    {  419, 1862,  494136,  494148,  494166, 12, 18, 18, }, // Yakutsk
    {  419, 1870,  494184,  494203,  494228, 19, 25, 25, }, // Yekaterinburg
    {  421,    0,  105715,  105725,  494253, 10, 21, 29, }, // Acre Macedonian/Cyrillic/Macedonia
    {  421,    5,       0,  494282,       0,  0, 19,  0, }, // Afghanistan
    {  421,   17,       0,  494301,       0,  0, 22,  0, }, // Africa_Central
    {  421,   32,       0,  494323,       0,  0, 23,  0, }, // Africa_Eastern
    {  421,   65,       0,  494346,       0,  0, 34,  0, }, // Africa_Southern
    {  421,   81,  119352,  494380,  494414, 23, 34, 42, }, // Africa_Western
    {  421,  107,  494456,  494472,  494499, 16, 27, 35, }, // Alaska
    {  421,  135,  494534,  494549,  494575, 15, 26, 34, }, // Amazon
    {  421,  142,  494609,  494643,  494688, 34, 45, 53, }, // America_Central
    {  421,  158,  494741,  494773,  494816, 32, 43, 51, }, // America_Eastern
    {  421,  174,  494867,  494901,  494946, 34, 45, 53, }, // America_Mountain
    {  421,  191,  494999,  495033,  495078, 34, 45, 53, }, // America_Pacific
    {  421,  207,  495131,  495146,  495172, 15, 26, 21, }, // Anadyr
    {  421,  214,  495193,  495207,  495232, 14, 25, 20, }, // Apia
    {  421,  232,  495252,  495265,  495289, 13, 24, 32, }, // Arabian
    {  421,  240,  495321,  495339,  495368, 18, 29, 37, }, // Argentina
    {  421,  250,  495405,  495431,  495468, 26, 37, 45, }, // Argentina_Western
    {  421,  268,  495513,  495530,  495558, 17, 28, 23, }, // Armenia
    {  421,  286,  495581,  495596,  495622, 15, 26, 34, }, // Atlantic
    {  421,  295,  495656,  495685,  495725, 29, 40, 48, }, // Australia_Central
    {  421,  313,  495773,  495812,  495862, 39, 50, 58, }, // Australia_CentralWestern
    {  421,  338,  495920,  495947,  495985, 27, 38, 46, }, // Australia_Eastern
    {  421,  356,  496031,  496058,  496096, 27, 38, 46, }, // Australia_Western
    {  421,  374,  496142,  496161,  496191, 19, 30, 25, }, // Azerbaijan
    {  421,  385,  496216,  496240,  496275, 24, 35, 30, }, // Azores
    {  421,  397,  496305,  496323,  496352, 18, 29, 24, }, // Bangladesh
    {  421,  415,       0,  496376,       0,  0, 14,  0, }, // Bhutan
    {  421,  422,       0,  496390,       0,  0, 17,  0, }, // Bolivia
    {  421,  437,  496407,  496425,  496454, 18, 29, 37, }, // Brasilia
    {  421,  454,       0,  496491,       0,  0, 25,  0, }, // Brunei
    {  421,  461,  496516,  496535,  496565, 19, 30, 38, }, // Cape_Verde
    {  421,  478,       0,  496603,       0,  0, 15,  0, }, // Chamorro
    {  421,  487,  496618,  496632,  496657, 14, 25, 33, }, // Chatham
    {  421,  495,  496690,  496703,  496727, 13, 24, 32, }, // Chile
    {  421,  501,  496759,  496772,  496796, 13, 24, 32, }, // China
    {  421,  507,       0,  496828,       0,  0, 23,  0, }, // Christmas
    {  421,  517,       0,  496851,       0,  0, 25,  0, }, // Cocos
    {  421,  523,  496876,  496894,  496923, 18, 29, 37, }, // Colombia
    {  421,  532,  496960,  496983,  497017, 23, 34, 29, }, // Cook
    {  421,  537,  497046,  497059,  497083, 13, 24, 32, }, // Cuba
    {  421,  548,       0,  497115,       0,  0, 15,  0, }, // Davis
    {  421,  564,       0,  497130,       0,  0, 21,  0, }, // DumontDUrville
    {  421,  601,       0,  497151,       0,  0, 22,  0, }, // East_Timor
    {  421,  612,  497173,  497200,  497238, 27, 38, 33, }, // Easter
    {  421,  619,       0,  497271,       0,  0, 16,  0, }, // Ecuador
    {  421,  627,  497287,  497307,  497338, 20, 31, 26, }, // Europe_Central
    {  421,  642,  497364,  497385,  497417, 21, 32, 27, }, // Europe_Eastern
    {  421,  657,       0,  497444,       0,  0, 20,  0, }, // Europe_Further_Eastern
    {  421,  680,  497464,  497485,  497517, 21, 32, 27, }, // Europe_Western
    {  421,  695,  497544,  497572,  497611, 28, 39, 47, }, // Falkland
    {  421,  704,  497658,  497671,  497695, 13, 24, 19, }, // Fiji
    {  421,  709,       0,  497714,       0,  0, 26,  0, }, // French_Guiana
    {  421,  723,       0,  497740,       0,  0, 35,  0, }, // French_Southern
    {  421,  746,       0,  497775,       0,  0, 18,  0, }, // Galapagos
    {  421,  756,       0,  497793,       0,  0, 14,  0, }, // Gambier
    {  421,  764,  497807,  497823,  497850, 16, 27, 22, }, // Georgia
    {  421,  772,       0,  497872,       0,  0, 27,  0, }, // Gilbert_Islands
    {  421,  788,       0,  497899,       0,  0, 22,  0, }, // GMT
    {  421,  830,  497921,  497946,  497982, 25, 36, 44, }, // Greenland_Eastern
    {  421,  848,  498026,  498051,  498087, 25, 36, 44, }, // Greenland_Western
    {  421,  871,       0,  498131,       0,  0, 23,  0, }, // Gulf
    {  421,  876,       0,  498154,       0,  0, 16,  0, }, // Guyana
    {  421,  883,  498170,  498203,  498247, 33, 44, 52, }, // Hawaii_Aleutian
    {  421,  899,  498299,  498317,  498346, 18, 29, 24, }, // Hong_Kong
    {  421,  909,  498370,  498383,  498407, 13, 24, 19, }, // Hovd
    {  421,  914,       0,  498426,       0,  0, 15,  0, }, // India
    {  421,  920,       0,  498441,       0,  0, 22,  0, }, // Indian_Ocean
    {  421,  933,       0,  498463,       0,  0, 17,  0, }, // Indochina
    {  421,  943,       0,  498480,       0,  0, 29,  0, }, // Indonesia_Central
    {  421,  961,       0,  498509,       0,  0, 27,  0, }, // Indonesia_Eastern
    {  421,  979,       0,  498536,       0,  0, 27,  0, }, // Indonesia_Western
    {  421,  997,  498563,  498576,  498600, 13, 24, 32, }, // Iran
    {  421, 1008,  498632,  498648,  498675, 16, 27, 22, }, // Irkutsk
    {  421, 1016,  498697,  498712,  498738, 15, 26, 34, }, // Israel
    {  421, 1023,  498772,  498789,  498817, 17, 28, 36, }, // Japan
    {  421, 1047,       0,  498853,       0,  0, 18,  0, }, // Kazakhstan
    {  421, 1058,       0,  498871,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  421, 1077,       0,  498897,       0,  0, 26,  0, }, // Kazakhstan_Western
    {  421, 1106,  498923,  498938,  498964, 15, 26, 34, }, // Korea
    {  421, 1112,       0,  498998,       0,  0, 14,  0, }, // Kosrae
    {  421, 1119,  499012,  499032,  499063, 20, 31, 26, }, // Krasnoyarsk
    {  421, 1151,       0,  499089,       0,  0, 18,  0, }, // Kyrgystan
    {  421, 1175,       0,  499107,       0,  0, 24,  0, }, // Line_Islands
    {  421, 1188,  499131,  499148,  499176, 17, 28, 36, }, // Lord_Howe
    {  421, 1204,  499212,  499228,  499255, 16, 27, 22, }, // Magadan
    {  421, 1219,       0,  499277,       0,  0, 17,  0, }, // Malaysia
    {  421, 1228,       0,  499294,       0,  0, 16,  0, }, // Maldives
    {  421, 1237,       0,  499310,       0,  0, 17,  0, }, // Marquesas
    {  421, 1247,       0,  499327,       0,  0, 26,  0, }, // Marshall_Islands
    {  421, 1264,  499353,  499371,  499400, 18, 29, 37, }, // Mauritius
    {  421, 1274,       0,  499437,       0,  0, 14,  0, }, // Mawson
    {  421, 1281,  499451,  499477,  499514, 26, 37, 32, }, // Mexico_Pacific
    {  421, 1296,  499546,  499565,  499595, 19, 30, 25, }, // Mongolia
    {  421, 1305,  499620,  499635,  499661, 15, 26, 34, }, // Moscow
    {  421, 1312,       0,  499695,       0,  0, 16,  0, }, // Myanmar
    {  421, 1320,       0,  499711,       0,  0, 14,  0, }, // Nauru
    {  421, 1326,       0,  499725,       0,  0, 14,  0, }, // Nepal
    {  421, 1332,  499739,  499763,  499798, 24, 35, 30, }, // New_Caledonia
    {  421, 1346,  499828,  499847,  499877, 19, 30, 38, }, // New_Zealand
    {  421, 1358,  499915,  499935,  499966, 20, 31, 39, }, // Newfoundland
    {  421, 1371,       0,  500005,       0,  0, 13,  0, }, // Niue
    {  421, 1376,  500018,  500043,  500079, 25, 36, 44, }, // Norfolk
    {  421, 1384,  500123,  500150,  500188, 27, 38, 46, }, // Noronha
    {  421, 1406,  500234,  500254,  500285, 20, 31, 26, }, // Novosibirsk
    {  421, 1418,  500311,  500324,  500348, 13, 24, 19, }, // Omsk
    {  421, 1428,  500367,  500384,  500412, 17, 28, 23, }, // Pakistan
    {  421, 1437,       0,  500435,       0,  0, 14,  0, }, // Palau
    {  421, 1443,       0,  500449,       0,  0, 27,  0, }, // Papua_New_Guinea
    {  421, 1460,  500476,  500493,  500521, 17, 28, 36, }, // Paraguay
    {  421, 1469,  500557,  500570,  500594, 13, 24, 32, }, // Peru
    {  421, 1474,  500626,  500643,  500671, 17, 28, 23, }, // Philippines
    {  421, 1486,       0,  500694,       0,  0, 25,  0, }, // Phoenix_Islands
    {  421, 1502,  500719,  500747,  500786, 28, 39, 47, }, // Pierre_Miquelon
    {  421, 1518,       0,  500833,       0,  0, 16,  0, }, // Pitcairn
    {  421, 1527,       0,  500849,       0,  0, 15,  0, }, // Ponape
    {  421, 1534,       0,  500864,       0,  0, 18,  0, }, // Pyongyang
    {  421, 1554,       0,  500882,       0,  0, 17,  0, }, // Reunion
    {  421, 1562,       0,  500899,       0,  0, 15,  0, }, // Rothera
    {  421, 1570,  500914,  500930,  500957, 16, 27, 22, }, // Sakhalin
    {  421, 1579,  112636,  112648,  500979, 12, 23, 31, }, // Samara
    {  421, 1596,  501010,  501024,  501049, 14, 25, 20, }, // Samoa
    {  421, 1602,       0,  501069,       0,  0, 16,  0, }, // Seychelles
    {  421, 1624,       0,  501085,       0,  0, 17,  0, }, // Singapore
    {  421, 1634,       0,  501102,       0,  0, 27,  0, }, // Solomon
    {  421, 1642,       0,  501129,       0,  0, 22,  0, }, // South_Georgia
    {  421, 1656,       0,  501151,       0,  0, 16,  0, }, // Suriname
    {  421, 1676,       0,  501167,       0,  0, 15,  0, }, // Syowa
    {  421, 1682,       0,  501182,       0,  0, 15,  0, }, // Tahiti
    {  421, 1689,  501197,  501212,  501238, 15, 26, 34, }, // Taipei
    {  421, 1696,       0,  501272,       0,  0, 19,  0, }, // Tajikistan
    {  421, 1724,       0,  501291,       0,  0, 16,  0, }, // Tokelau
    {  421, 1732,  501307,  501321,  501346, 14, 25, 20, }, // Tonga
    {  421, 1738,       0,  501366,       0,  0, 13,  0, }, // Truk
    {  421, 1750,  501379,  501400,  501432, 21, 32, 27, }, // Turkmenistan
    {  421, 1763,       0,  501459,       0,  0, 15,  0, }, // Tuvalu
    {  421, 1777,  501474,  501490,  501517, 16, 27, 35, }, // Uruguay
    {  421, 1792,  501552,  501571,  501601, 19, 30, 25, }, // Uzbekistan
    {  421, 1803,  501626,  501642,  501669, 16, 27, 22, }, // Vanuatu
    {  421, 1811,       0,  501691,       0,  0, 18,  0, }, // Venezuela
    {  421, 1821,  501709,  501729,  501760, 20, 31, 26, }, // Vladivostok
    {  421, 1833,  501786,  501804,  501833, 18, 29, 37, }, // Volgograd
    {  421, 1843,       0,  501870,       0,  0, 15,  0, }, // Vostok
    {  421, 1850,       0,  501885,       0,  0, 22,  0, }, // Wake
    {  421, 1855,       0,  501907,       0,  0, 23,  0, }, // Wallis
    {  421, 1862,  501930,  501946,  501973, 16, 27, 22, }, // Yakutsk
    {  421, 1870,  501995,  502016,  502048, 21, 32, 27, }, // Yekaterinburg
    {  421, 1892,       0,  502075,       0,  0, 14,  0, }, // Yukon
    {  423,    5,       0,  502089,       0,  0, 16,  0, }, // Afghanistan Maithili/Devanagari/India
    {  423,   17,       0,  502105,       0,  0, 17,  0, }, // Africa_Central
    {  423,   32,       0,  502122,       0,  0, 17,  0, }, // Africa_Eastern
    {  423,   65,       0,  502139,       0,  0, 24,  0, }, // Africa_Southern
    {  423,   81,  502163,  502182,  502206, 19, 24, 21, }, // Africa_Western
    {  423,  107,  334650,  502227,  502243, 11, 16, 18, }, // Alaska
    {  423,  135,  502261,  502271,  502286, 10, 15, 14, }, // Amazon
    {  423,  142,  334745,  334772,  334804, 27, 32, 34, }, // America_Central
    {  423,  158,  334838,  334863,  334893, 25, 30, 32, }, // America_Eastern
    {  423,  174,  334925,  334951,  334982, 26, 31, 33, }, // America_Mountain
    {  423,  191,  335015,  335041,  335072, 26, 31, 33, }, // America_Pacific
    {  423,  232,  502300,  502312,  502329, 12, 17, 19, }, // Arabian
    {  423,  240,  502348,  502363,  502383, 15, 20, 19, }, // Argentina
    {  423,  250,  502402,  502425,  502453, 23, 28, 27, }, // Argentina_Western
    {  423,  268,  502480,  502494,  502513, 14, 19, 18, }, // Armenia
    {  423,  286,  335426,  335438,  335455, 12, 17, 19, }, // Atlantic
    {  423,  295,  502531,  502552,  502579, 21, 27, 29, }, // Australia_Central
    {  423,  313,  502608,  502638,  502673, 30, 35, 37, }, // Australia_CentralWestern
    {  423,  338,  502710,  502732,  502760, 22, 28, 30, }, // Australia_Eastern
    {  423,  356,  502790,  502814,  502844, 24, 30, 32, }, // Australia_Western
    {  423,  374,  502876,  502889,  502907, 13, 18, 17, }, // Azerbaijan
    {  423,  385,  502924,  502935,  502951, 11, 16, 15, }, // Azores
    {  423,  397,  502966,  502979,  502997, 13, 18, 17, }, // Bangladesh
    {  423,  415,       0,  503014,       0,  0, 10,  0, }, // Bhutan
    {  423,  422,       0,  503024,       0,  0, 13,  0, }, // Bolivia
    {  423,  437,  503037,  503052,  503072, 15, 20, 19, }, // Brasilia
    {  423,  454,       0,  503091,       0,  0, 21,  0, }, // Brunei
    {  423,  461,  503112,  503126,  503144, 14, 18, 17, }, // Cape_Verde
    {  423,  487,  503161,  503170,  503184,  9, 14, 16, }, // Chatham
    {  423,  495,  503200,  503209,  503223,  9, 14, 13, }, // Chile
    {  423,  501,  503236,  503245,  503259,  9, 14, 16, }, // China
    {  423,  507,       0,  503275,       0,  0, 18,  0, }, // Christmas
    {  423,  517,       0,  503293,       0,  0, 16,  0, }, // Cocos
    {  423,  523,  503309,  503322,  503340, 13, 18, 17, }, // Colombia
    {  423,  537,  503357,  503368,  503384, 11, 16, 18, }, // Cuba
    {  423,  601,       0,  503402,       0,  0, 16,  0, }, // East_Timor
    {  423,  612,  503418,  503435,  503457, 17, 22, 21, }, // Easter
    {  423,  619,       0,  503478,       0,  0, 13,  0, }, // Ecuador
    {  423,  627,  336622,  336638,  336659, 16, 21, 31, }, // Europe_Central
    {  423,  642,  336690,  336708,  336731, 18, 23, 31, }, // Europe_Eastern
    {  423,  657,       0,  503491,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {  423,  680,  336785,  336804,  336828, 19, 24, 33, }, // Europe_Western
    {  423,  695,  503517,  503536,  503560, 19, 24, 23, }, // Falkland
    {  423,  709,       0,  503583,       0,  0, 18,  0, }, // French_Guiana
    {  423,  723,       0,  503601,       0,  0, 44,  0, }, // French_Southern
    {  423,  746,       0,  503645,       0,  0, 14,  0, }, // Galapagos
    {  423,  764,  503659,  503672,  503690, 13, 18, 17, }, // Georgia
    {  423,  788,       0,   91115,       0,  0, 17,  0, }, // GMT
    {  423,  830,  503707,  503727,  503752, 20, 25, 24, }, // Greenland_Eastern
    {  423,  848,  503776,  503798,  503825, 22, 27, 26, }, // Greenland_Western
    {  423,  871,       0,  503851,       0,  0, 13,  0, }, // Gulf
    {  423,  876,       0,  503590,       0,  0, 11,  0, }, // Guyana
    {  423,  883,  503864,  503883,  503907, 19, 24, 26, }, // Hawaii_Aleutian
    {  423,  899,  503933,  503947,  503966, 14, 19, 18, }, // Hong_Kong
    {  423,  909,  503984,  503994,  504009, 10, 15, 14, }, // Hovd
    {  423,  914,       0,  337479,       0,  0, 15,  0, }, // India
    {  423,  920,       0,  504023,       0,  0, 17,  0, }, // Indian_Ocean
    {  423,  933,       0,  504040,       0,  0, 14,  0, }, // Indochina
    {  423,  943,       0,  504054,       0,  0, 20,  0, }, // Indonesia_Central
    {  423,  961,       0,  504074,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  423,  979,       0,  504095,       0,  0, 23,  0, }, // Indonesia_Western
    {  423,  997,  504118,  504127,  504140,  9, 13, 16, }, // Iran
    {  423, 1008,  504156,  504170,  504189, 14, 19, 18, }, // Irkutsk
    {  423, 1016,  504207,  504218,  504234, 11, 16, 18, }, // Israel
    {  423, 1023,  504252,  504262,  504277, 10, 15, 17, }, // Japan
    {  423, 1058,       0,  504294,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  423, 1077,       0,  504313,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  423, 1106,  504334,  504345,  504361, 11, 16, 18, }, // Korea
    {  423, 1119,  504379,  504394,  504419, 15, 25, 24, }, // Krasnoyarsk
    {  423, 1151,       0,  504443,       0,  0, 11,  0, }, // Kyrgystan
    {  423, 1188,  504454,  504469,  504489, 15, 20, 22, }, // Lord_Howe
    {  423, 1204,  504511,  504522,  504538, 11, 16, 15, }, // Magadan
    {  423, 1219,       0,  504553,       0,  0, 12,  0, }, // Malaysia
    {  423, 1228,       0,  504565,       0,  0, 11,  0, }, // Maldives
    {  423, 1264,  504576,  504587,  504603, 11, 16, 15, }, // Mauritius
    {  423, 1281,  504618,  504639,  504665, 21, 26, 28, }, // Mexico_Pacific
    {  423, 1296,  504693,  504705,  504722, 12, 17, 16, }, // Mongolia
    {  423, 1305,  504738,  504749,  504765, 11, 16, 15, }, // Moscow
    {  423, 1312,       0,  504780,       0,  0, 11,  0, }, // Myanmar
    {  423, 1326,       0,  504791,       0,  0, 10,  0, }, // Nepal
    {  423, 1332,  504801,  504820,  504844, 19, 24, 26, }, // New_Caledonia
    {  423, 1346,  504870,  504885,  504905, 15, 20, 22, }, // New_Zealand
    {  423, 1358,  504927,  504945,  504968, 18, 23, 25, }, // Newfoundland
    {  423, 1376,  504993,  505011,  505034, 18, 23, 25, }, // Norfolk
    {  423, 1384,  505059,  505084,  505114, 25, 30, 29, }, // Noronha
    {  423, 1406,  505143,  505161,  505184, 18, 23, 22, }, // Novosibirsk
    {  423, 1418,  505206,  505217,  505233, 11, 16, 15, }, // Omsk
    {  423, 1428,  505248,  505262,  505281, 14, 19, 18, }, // Pakistan
    {  423, 1460,  505299,  505311,  505328, 12, 17, 16, }, // Paraguay
    {  423, 1469,  505344,  505353,  505367,  9, 14, 13, }, // Peru
    {  423, 1474,  505380,  505392,  505409, 12, 17, 16, }, // Philippines
    {  423, 1502,  505425,  505452,  505484, 27, 32, 34, }, // Pierre_Miquelon
    {  423, 1534,       0,  505518,       0,  0, 15,  0, }, // Pyongyang
    {  423, 1554,       0,  505533,       0,  0, 13,  0, }, // Reunion
    {  423, 1570,  505546,  505557,  505573, 11, 16, 15, }, // Sakhalin
    {  423, 1596,  505588,  505597,  505611,  9, 14, 13, }, // Samoa
    {  423, 1602,       0,  505624,       0,  0, 12,  0, }, // Seychelles
    {  423, 1624,       0,  505636,       0,  0, 18,  0, }, // Singapore
    {  423, 1642,       0,  505654,       0,  0, 20,  0, }, // South_Georgia
    {  423, 1656,       0,  505674,       0,  0, 12,  0, }, // Suriname
    {  423, 1689,  505686,  505697,  505713, 11, 16, 18, }, // Taipei
    {  423, 1696,       0,   96571,       0,  0, 11,  0, }, // Tajikistan
    {  423, 1750,  505731,  505750,  505774, 19, 24, 23, }, // Turkmenistan
    {  423, 1777,  505797,  505809,  505826, 12, 17, 16, }, // Uruguay
    {  423, 1792,  505842,  505858,  505879, 16, 21, 20, }, // Uzbekistan
    {  423, 1811,       0,  505899,       0,  0, 14,  0, }, // Venezuela
    {  423, 1821,  505913,  505931,  505954, 18, 23, 22, }, // Vladivostok
    {  423, 1833,  505976,  505992,  506013, 16, 21, 20, }, // Volgograd
    {  423, 1862,  506033,  506046,  506064, 13, 18, 17, }, // Yakutsk
    {  423, 1870,  506081,  506099,  506122, 18, 23, 22, }, // Yekaterinburg
    {  423, 1892,       0,  506144,       0,  0, 10,  0, }, // Yukon
    {  427,    0,  506154,  506166,  506191, 12, 25, 22, }, // Acre Malayalam/Malayalam/India
    {  427,    5,       0,  506213,       0,  0, 18,  0, }, // Afghanistan
    {  427,   17,       0,  506231,       0,  0, 18,  0, }, // Africa_Central
    {  427,   32,       0,  506249,       0,  0, 21,  0, }, // Africa_Eastern
    {  427,   65,       0,  506270,       0,  0, 32,  0, }, // Africa_Southern
    {  427,   81,  506302,  506325,  506361, 23, 36, 35, }, // Africa_Western
    {  427,  107,  506396,  506408,  506432, 12, 24, 21, }, // Alaska
    {  427,  128,  506453,  506463,  506486, 10, 23, 20, }, // Almaty
    {  427,  135,  506506,  506516,  506539, 10, 23, 22, }, // Amazon
    {  427,  142,  506561,  506590,  506632, 29, 42, 38, }, // America_Central
    {  427,  158,  506670,  506699,  506741, 29, 42, 38, }, // America_Eastern
    {  427,  174,  506779,  506807,  506848, 28, 41, 37, }, // America_Mountain
    {  427,  191,  506885,  506913,  506954, 28, 41, 37, }, // America_Pacific
    {  427,  207,  506991,  507002,  507026, 11, 24, 21, }, // Anadyr
    {  427,  214,  507047,  507056,  507078,  9, 22, 18, }, // Apia
    {  427,  219,  507096,  507106,  507129, 10, 23, 20, }, // Aqtau
    {  427,  225,  507149,  507161,  507186, 12, 25, 22, }, // Aqtobe
    {  427,  232,  507208,  507220,  507245, 12, 25, 21, }, // Arabian
    {  427,  240,  507266,  507279,  507305, 13, 26, 25, }, // Argentina
    {  427,  250,  507330,  507353,  507389, 23, 36, 35, }, // Argentina_Western
    {  427,  268,  507424,  507436,  507461, 12, 25, 24, }, // Armenia
    {  427,  286,  507485,  507504,  507536, 19, 32, 28, }, // Atlantic
    {  427,  295,  507564,  507587,  507624, 23, 37, 33, }, // Australia_Central
    {  427,  313,  507657,  507691,  507738, 34, 47, 43, }, // Australia_CentralWestern
    {  427,  338,  507781,  507805,  507843, 24, 38, 34, }, // Australia_Eastern
    {  427,  356,  507877,  507903,  507943, 26, 40, 36, }, // Australia_Western
    {  427,  374,  507979,  507992,  508018, 13, 26, 25, }, // Azerbaijan
    {  427,  385,  508043,  508054,  508078, 11, 24, 23, }, // Azores
    {  427,  397,  508101,  508116,  508144, 15, 28, 27, }, // Bangladesh
    {  427,  415,       0,  508171,       0,  0, 12,  0, }, // Bhutan
    {  427,  422,       0,  508183,       0,  0, 12,  0, }, // Bolivia
    {  427,  437,  508195,  508208,  508234, 13, 26, 25, }, // Brasilia
    {  427,  454,       0,  508259,       0,  0, 22,  0, }, // Brunei
    {  427,  461,  508281,  508296,  508324, 15, 28, 27, }, // Cape_Verde
    {  427,  478,       0,  508351,       0,  0, 23,  0, }, // Chamorro
    {  427,  487,  508374,  508385,  508409, 11, 24, 23, }, // Chatham
    {  427,  495,  508432,  508441,  508463,  9, 22, 21, }, // Chile
    {  427,  501,  508484,  508492,  508513,  8, 21, 17, }, // China
    {  427,  507,       0,  508530,       0,  0, 22,  0, }, // Christmas
    {  427,  517,       0,  508552,       0,  0, 22,  0, }, // Cocos
    {  427,  523,  508574,  508586,  508611, 12, 25, 24, }, // Colombia
    {  427,  532,  508635,  508655,  508688, 20, 33, 38, }, // Cook
    {  427,  537,  508726,  508736,  508759, 10, 23, 19, }, // Cuba
    {  427,  548,       0,  508778,       0,  0, 11,  0, }, // Davis
    {  427,  564,       0,  508789,       0,  0, 25,  0, }, // DumontDUrville
    {  427,  601,       0,  508814,       0,  0, 18,  0, }, // East_Timor
    {  427,  612,  508832,  508851,  508883, 19, 32, 31, }, // Easter
    {  427,  619,       0,  508914,       0,  0, 12,  0, }, // Ecuador
    {  427,  627,  508926,  508947,  508981, 21, 34, 32, }, // Europe_Central
    {  427,  642,  509013,  509034,  509068, 21, 34, 32, }, // Europe_Eastern
    {  427,  657,       0,  509100,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {  427,  680,  509129,  509152,  509188, 23, 36, 35, }, // Europe_Western
    {  427,  695,  509223,  509249,  509288, 26, 39, 38, }, // Falkland
    {  427,  704,  509326,  509335,  509357,  9, 22, 21, }, // Fiji
    {  427,  709,       0,  509378,       0,  0, 17,  0, }, // French_Guiana
    {  427,  723,       0,  509395,       0,  0, 31,  0, }, // French_Southern
    {  427,  746,       0,  509426,       0,  0, 15,  0, }, // Galapagos
    {  427,  756,       0,  509441,       0,  0, 13,  0, }, // Gambier
    {  427,  764,  509454,  509467,  509493, 13, 26, 25, }, // Georgia
    {  427,  772,       0,  509518,       0,  0, 24,  0, }, // Gilbert_Islands
    {  427,  788,       0,  509542,       0,  0, 20,  0, }, // GMT
    {  427,  830,  509562,  509585,  509621, 23, 36, 35, }, // Greenland_Eastern
    {  427,  848,  509656,  509681,  509719, 25, 38, 37, }, // Greenland_Western
    {  427,  866,       0,  509756,       0,  0, 23,  0, }, // Guam
    {  427,  871,       0,  509779,       0,  0, 22,  0, }, // Gulf
    {  427,  876,       0,  509386,       0,  0,  9,  0, }, // Guyana
    {  427,  883,  509801,  509819,  509850, 18, 31, 27, }, // Hawaii_Aleutian
    {  427,  899,  509877,  509892,  509920, 15, 28, 27, }, // Hong_Kong
    {  427,  909,  509947,  509956,  509978,  9, 22, 21, }, // Hovd
    {  427,  914,       0,  509999,       0,  0, 25,  0, }, // India
    {  427,  920,       0,  510024,       0,  0, 22,  0, }, // Indian_Ocean
    {  427,  933,       0,  510046,       0,  0, 12,  0, }, // Indochina
    {  427,  943,       0,  510058,       0,  0, 20,  0, }, // Indonesia_Central
    {  427,  961,       0,  510078,       0,  0, 23,  0, }, // Indonesia_Eastern
    {  427,  979,       0,  510101,       0,  0, 25,  0, }, // Indonesia_Western
    {  427,  997,  510126,  510135,  510157,  9, 22, 18, }, // Iran
    {  427, 1008,  510175,  510187,  510212, 12, 25, 25, }, // Irkutsk
    {  427, 1016,  510237,  510250,  510276, 13, 26, 22, }, // Israel
    {  427, 1023,  510298,  510309,  510333, 11, 24, 20, }, // Japan
    {  427, 1029,  510353,  510384,  510428, 31, 44, 41, }, // Kamchatka
    {  427, 1047,       0,  510469,       0,  0, 15,  0, }, // Kazakhstan
    {  427, 1058,       0,  510484,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  427, 1077,       0,  510507,       0,  0, 25,  0, }, // Kazakhstan_Western
    {  427, 1106,  510532,  510543,  510567, 11, 24, 20, }, // Korea
    {  427, 1112,       0,  510587,       0,  0, 10,  0, }, // Kosrae
    {  427, 1119,  510597,  510619,  510654, 22, 35, 34, }, // Krasnoyarsk
    {  427, 1151,       0,  510688,       0,  0, 15,  0, }, // Kyrgystan
    {  427, 1161,       0,  510703,       0,  0,  9,  0, }, // Lanka
    {  427, 1175,       0,  510712,       0,  0, 17,  0, }, // Line_Islands
    {  427, 1188,  510729,  510744,  510772, 15, 28, 24, }, // Lord_Howe
    {  427, 1198,  510796,  510804,  510825,  8, 21, 18, }, // Macau
    {  427, 1204,  510843,  510853,  510876, 10, 23, 22, }, // Magadan
    {  427, 1219,       0,  510898,       0,  0, 11,  0, }, // Malaysia
    {  427, 1228,       0,  510909,       0,  0, 15,  0, }, // Maldives
    {  427, 1237,       0,  510924,       0,  0, 13,  0, }, // Marquesas
    {  427, 1247,       0,  510937,       0,  0, 19,  0, }, // Marshall_Islands
    {  427, 1264,  510956,  510970,  510997, 14, 27, 26, }, // Mauritius
    {  427, 1274,       0,  511023,       0,  0,  9,  0, }, // Mawson
    {  427, 1281,  511032,  511054,  511090, 22, 36, 31, }, // Mexico_Pacific
    {  427, 1296,  511121,  511136,  511164, 15, 28, 27, }, // Mongolia
    {  427, 1305,  511191,  511202,  511226, 11, 24, 24, }, // Moscow
    {  427, 1312,       0,  511250,       0,  0, 13,  0, }, // Myanmar
    {  427, 1320,       0,  511263,       0,  0,  9,  0, }, // Nauru
    {  427, 1326,       0,  511272,       0,  0, 12,  0, }, // Nepal
    {  427, 1332,  511284,  511303,  511335, 19, 32, 31, }, // New_Caledonia
    {  427, 1346,  511366,  511382,  511411, 16, 29, 25, }, // New_Zealand
    {  427, 1358,  511436,  511458,  511493, 22, 35, 31, }, // Newfoundland
    {  427, 1371,       0,  511524,       0,  0, 11,  0, }, // Niue
    {  427, 1376,  511535,  511557,  511592, 22, 35, 31, }, // Norfolk
    {  427, 1384,  511623,  511645,  511680, 22, 35, 34, }, // Noronha
    {  427, 1392,       0,  511714,       0,  0, 29,  0, }, // North_Mariana
    {  427, 1406,  511743,  511762,  511793, 19, 31, 31, }, // Novosibirsk
    {  427, 1418,  511824,  511838,  511865, 14, 27, 26, }, // Omsk
    {  427, 1428,  511891,  511907,  511936, 16, 29, 28, }, // Pakistan
    {  427, 1437,       0,  511964,       0,  0, 10,  0, }, // Palau
    {  427, 1443,       0,  511974,       0,  0, 23,  0, }, // Papua_New_Guinea
    {  427, 1460,  511997,  512009,  512034, 12, 25, 24, }, // Paraguay
    {  427, 1469,  512058,  512067,  512089,  9, 22, 21, }, // Peru
    {  427, 1474,  512110,  512124,  512151, 14, 27, 26, }, // Philippines
    {  427, 1486,       0,  512177,       0,  0, 21,  0, }, // Phoenix_Islands
    {  427, 1502,  512198,  512228,  512271, 30, 43, 39, }, // Pierre_Miquelon
    {  427, 1518,       0,  512310,       0,  0, 14,  0, }, // Pitcairn
    {  427, 1527,       0,  512324,       0,  0, 13,  0, }, // Ponape
    {  427, 1534,       0,  512337,       0,  0, 18,  0, }, // Pyongyang
    {  427, 1544,  512355,  512368,  512394, 13, 26, 23, }, // Qyzylorda
    {  427, 1554,       0,  512417,       0,  0, 13,  0, }, // Reunion
    {  427, 1562,       0,  512430,       0,  0, 10,  0, }, // Rothera
    {  427, 1570,  512440,  512451,  512475, 11, 24, 23, }, // Sakhalin
    {  427, 1579,  512498,  512507,  512529,  9, 22, 19, }, // Samara
    {  427, 1596,  512548,  512557,  512579,  9, 22, 22, }, // Samoa
    {  427, 1602,       0,  512601,       0,  0, 12,  0, }, // Seychelles
    {  427, 1624,       0,  512613,       0,  0, 27,  0, }, // Singapore
    {  427, 1634,       0,  512640,       0,  0, 19,  0, }, // Solomon
    {  427, 1642,       0,  512659,       0,  0, 21,  0, }, // South_Georgia
    {  427, 1656,       0,  512680,       0,  0, 14,  0, }, // Suriname
    {  427, 1676,       0,  512694,       0,  0,  9,  0, }, // Syowa
    {  427, 1682,       0,  512703,       0,  0, 11,  0, }, // Tahiti
    {  427, 1689,  512714,  512728,  512755, 14, 27, 23, }, // Taipei
    {  427, 1696,       0,  512778,       0,  0, 18,  0, }, // Tajikistan
    {  427, 1724,       0,  512796,       0,  0, 13,  0, }, // Tokelau
    {  427, 1732,  512809,  512818,  512840,  9, 22, 21, }, // Tonga
    {  427, 1738,       0,  512861,       0,  0, 11,  0, }, // Truk
    {  427, 1750,  512872,  512894,  512929, 22, 35, 34, }, // Turkmenistan
    {  427, 1763,       0,  512963,       0,  0, 11,  0, }, // Tuvalu
    {  427, 1777,  512974,  512986,  513011, 12, 25, 24, }, // Uruguay
    {  427, 1792,  513035,  513055,  513088, 20, 33, 32, }, // Uzbekistan
    {  427, 1803,  513120,  513132,  513157, 12, 25, 24, }, // Vanuatu
    {  427, 1811,       0,  513181,       0,  0, 14,  0, }, // Venezuela
    {  427, 1821,  513195,  513216,  513250, 21, 34, 33, }, // Vladivostok
    {  427, 1833,  513283,  513299,  513328, 16, 29, 28, }, // Volgograd
    {  427, 1843,       0,  513201,       0,  0, 15,  0, }, // Vostok
    {  427, 1850,       0,  513356,       0,  0, 18,  0, }, // Wake
    {  427, 1855,       0,  513374,       0,  0, 27,  0, }, // Wallis
    {  427, 1862,  513401,  513416,  513444, 15, 28, 27, }, // Yakutsk
    {  427, 1870,  513471,  513492,  513526, 21, 34, 33, }, // Yekaterinburg
    {  427, 1892,       0,  513559,       0,  0, 12,  0, }, // Yukon
    {  428,    0,  358597,  513571,  358625, 10, 17, 22, }, // Acre Malay/Latin/Malaysia
    {  428,    5,       0,  513588,       0,  0, 17,  0, }, // Afghanistan
    {  428,   17,       0,  358663,       0,  0, 19,  0, }, // Africa_Central
    {  428,   32,       0,  358682,       0,  0, 18,  0, }, // Africa_Eastern
    {  428,   65,       0,  513605,       0,  0, 27,  0, }, // Africa_Southern
    {  428,   81,  358728,  513632,  358772, 18, 25, 30, }, // Africa_Western
    {  428,  107,  358802,  513657,  513676, 12, 19, 18, }, // Alaska
    {  428,  128,  358858,  513694,  358890, 12, 19, 24, }, // Almaty
    {  428,  135,  358914,  513713,  358946, 12, 19, 24, }, // Amazon
    {  428,  142,  513732,  513743,  513761, 11, 18, 18, }, // America_Central
    {  428,  158,  359026,  513779,  513797, 11, 18, 17, }, // America_Eastern
    {  428,  174,  513814,  513831,  513855, 17, 24, 23, }, // America_Mountain
    {  428,  191,  359147,  513878,  513898, 13, 20, 19, }, // America_Pacific
    {  428,  207,  359206,  513917,  359238, 12, 19, 24, }, // Anadyr
    {  428,  214,  359262,  513936,  513953, 10, 17, 16, }, // Apia
    {  428,  219,  359312,  513969,  359342, 11, 20, 23, }, // Aqtau
    {  428,  225,  359365,  513989,  359397, 12, 19, 24, }, // Aqtobe
    {  428,  232,  359421,  514008,  514025, 10, 17, 16, }, // Arabian
    {  428,  240,  359471,  514041,  359509, 15, 22, 27, }, // Argentina
    {  428,  250,  514063,  514084,  514112, 21, 28, 33, }, // Argentina_Western
    {  428,  268,  359640,  514145,  359674, 13, 20, 25, }, // Armenia
    {  428,  286,  359699,  514165,  514186, 14, 21, 20, }, // Atlantic
    {  428,  295,  514206,  514228,  514257, 22, 29, 28, }, // Australia_Central
    {  428,  313,  359847,  514285,  514320, 28, 35, 34, }, // Australia_CentralWestern
    {  428,  338,  514354,  514375,  514403, 21, 28, 27, }, // Australia_Eastern
    {  428,  356,  514430,  514451,  514479, 21, 28, 27, }, // Australia_Western
    {  428,  374,  360117,  514506,  360157, 16, 23, 28, }, // Azerbaijan
    {  428,  385,  360185,  514529,  360217, 12, 19, 24, }, // Azores
    {  428,  397,  360241,  514548,  360281, 16, 23, 28, }, // Bangladesh
    {  428,  415,       0,  360309,       0,  0, 12,  0, }, // Bhutan
    {  428,  422,       0,  360321,       0,  0, 13,  0, }, // Bolivia
    {  428,  437,  514571,  514585,  514606, 14, 21, 26, }, // Brasilia
    {  428,  454,       0,  360390,       0,  0, 23,  0, }, // Brunei
    {  428,  461,  360413,  514632,  360459, 19, 26, 31, }, // Cape_Verde
    {  428,  478,       0,  514658,       0,  0, 21,  0, }, // Chamorro
    {  428,  487,  360523,  514679,  514699, 13, 20, 19, }, // Chatham
    {  428,  495,  514718,  514729,  514747, 11, 18, 23, }, // Chile
    {  428,  501,  514770,  514781,  514799, 11, 18, 17, }, // China
    {  428,  507,       0,  514816,       0,  0, 21,  0, }, // Christmas
    {  428,  517,       0,  360711,       0,  0, 21,  0, }, // Cocos
    {  428,  523,  514837,  514851,  514872, 14, 21, 26, }, // Colombia
    {  428,  532,  514898,  514918,  514945, 20, 27, 40, }, // Cook
    {  428,  537,  514985,  514995,  515012, 10, 17, 16, }, // Cuba
    {  428,  548,       0,  360916,       0,  0, 11,  0, }, // Davis
    {  428,  564,       0,  360927,       0,  0, 22,  0, }, // DumontDUrville
    {  428,  601,       0,  515028,       0,  0, 17,  0, }, // East_Timor
    {  428,  612,  515045,  515063,  515088, 18, 25, 30, }, // Easter
    {  428,  619,       0,  515118,       0,  0, 13,  0, }, // Ecuador
    {  428,  627,  515131,  515150,  515176, 19, 26, 31, }, // Europe_Central
    {  428,  642,  515207,  515225,  515250, 18, 25, 30, }, // Europe_Eastern
    {  428,  657,       0,  515280,       0,  0, 24,  0, }, // Europe_Further_Eastern
    {  428,  680,  515304,  515322,  515347, 18, 25, 30, }, // Europe_Western
    {  428,  695,  361291,  515377,  361347, 24, 31, 36, }, // Falkland
    {  428,  704,  361383,  515408,  361411, 10, 17, 22, }, // Fiji
    {  428,  709,       0,  515425,       0,  0, 21,  0, }, // French_Guiana
    {  428,  723,       0,  515446,       0,  0, 36,  0, }, // French_Southern
    {  428,  746,       0,  361497,       0,  0, 15,  0, }, // Galapagos
    {  428,  756,       0,  361512,       0,  0, 13,  0, }, // Gambier
    {  428,  764,  361525,  515482,  361559, 13, 20, 25, }, // Georgia
    {  428,  772,       0,  515502,       0,  0, 23,  0, }, // Gilbert_Islands
    {  428,  788,       0,  515525,       0,  0, 19,  0, }, // GMT
    {  428,  830,  361602,  515544,  361652, 21, 28, 33, }, // Greenland_Eastern
    {  428,  848,  361685,  515572,  361735, 21, 28, 33, }, // Greenland_Western
    {  428,  871,       0,  515600,       0,  0, 18,  0, }, // Gulf
    {  428,  876,       0,  361433,       0,  0, 12,  0, }, // Guyana
    {  428,  883,  361797,  515618,  515646, 21, 28, 27, }, // Hawaii_Aleutian
    {  428,  899,  361880,  515673,  361918, 15, 22, 27, }, // Hong_Kong
    {  428,  909,  361945,  515695,  361973, 10, 17, 22, }, // Hovd
    {  428,  914,       0,  515712,       0,  0, 18,  0, }, // India
    {  428,  920,       0,  515730,       0,  0, 18,  0, }, // Indian_Ocean
    {  428,  933,       0,  362027,       0,  0, 15,  0, }, // Indochina
    {  428,  943,       0,  362042,       0,  0, 22,  0, }, // Indonesia_Central
    {  428,  961,       0,  362064,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  428,  979,       0,  362085,       0,  0, 21,  0, }, // Indonesia_Western
    {  428,  997,  362106,  515748,  515765, 10, 17, 16, }, // Iran
    {  428, 1008,  362156,  515781,  362190, 13, 20, 25, }, // Irkutsk
    {  428, 1016,  362215,  515801,  515820, 12, 19, 18, }, // Israel
    {  428, 1023,  515838,  515849,  515867, 11, 18, 17, }, // Japan
    {  428, 1029,  515884,  515914,  362395, 30, 37, 42, }, // Kamchatka
    {  428, 1047,       0,  362437,       0,  0, 16,  0, }, // Kazakhstan
    {  428, 1058,       0,  362453,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {  428, 1077,       0,  362475,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  428, 1106,  362497,  515951,  515969, 11, 18, 17, }, // Korea
    {  428, 1112,       0,  362550,       0,  0, 12,  0, }, // Kosrae
    {  428, 1119,  362562,  515986,  362604, 17, 24, 29, }, // Krasnoyarsk
    {  428, 1151,       0,  516010,       0,  0, 15,  0, }, // Kyrgystan
    {  428, 1175,       0,  516025,       0,  0, 20,  0, }, // Line_Islands
    {  428, 1188,  362675,  516045,  516067, 15, 22, 21, }, // Lord_Howe
    {  428, 1198,  516088,  516099,  516117, 11, 18, 23, }, // Macau
    {  428, 1204,  362793,  516140,  362827, 13, 20, 25, }, // Magadan
    {  428, 1219,       0,  362852,       0,  0, 14,  0, }, // Malaysia
    {  428, 1228,       0,  516160,       0,  0, 14,  0, }, // Maldives
    {  428, 1237,       0,  362880,       0,  0, 15,  0, }, // Marquesas
    {  428, 1247,       0,  516174,       0,  0, 24,  0, }, // Marshall_Islands
    {  428, 1264,  362914,  516198,  362952, 15, 22, 27, }, // Mauritius
    {  428, 1274,       0,  362979,       0,  0, 12,  0, }, // Mawson
    {  428, 1281,  516220,  516240,  516267, 20, 27, 26, }, // Mexico_Pacific
    {  428, 1296,  363074,  516293,  363114, 16, 23, 28, }, // Mongolia
    {  428, 1305,  516316,  516328,  516347, 12, 19, 24, }, // Moscow
    {  428, 1312,       0,  363198,       0,  0, 13,  0, }, // Myanmar
    {  428, 1320,       0,  363211,       0,  0, 11,  0, }, // Nauru
    {  428, 1326,       0,  363222,       0,  0, 11,  0, }, // Nepal
    {  428, 1332,  516371,  516390,  516416, 19, 26, 31, }, // New_Caledonia
    {  428, 1346,  516447,  516464,  516488, 17, 24, 23, }, // New_Zealand
    {  428, 1358,  363390,  516511,  516536, 18, 25, 24, }, // Newfoundland
    {  428, 1371,       0,  363464,       0,  0, 10,  0, }, // Niue
    {  428, 1376,  516560,  516583,  516613, 23, 30, 29, }, // Norfolk
    {  428, 1384,  363551,  516642,  363609, 25, 32, 37, }, // Noronha
    {  428, 1392,       0,  516674,       0,  0, 29,  0, }, // North_Mariana
    {  428, 1406,  363670,  516703,  363712, 17, 24, 29, }, // Novosibirsk
    {  428, 1418,  363741,  516727,  363769, 10, 17, 22, }, // Omsk
    {  428, 1428,  363791,  516744,  363827, 14, 21, 26, }, // Pakistan
    {  428, 1437,       0,  363853,       0,  0, 11,  0, }, // Palau
    {  428, 1443,       0,  516765,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  428, 1460,  363882,  516787,  363918, 14, 21, 26, }, // Paraguay
    {  428, 1469,  363944,  516808,  363972, 10, 17, 22, }, // Peru
    {  428, 1474,  363994,  516825,  364030, 14, 21, 26, }, // Philippines
    {  428, 1486,       0,  364056,       0,  0, 23,  0, }, // Phoenix_Islands
    {  428, 1502,  364079,  516846,  516884, 31, 38, 37, }, // Pierre_Miquelon
    {  428, 1518,       0,  364192,       0,  0, 14,  0, }, // Pitcairn
    {  428, 1527,       0,  364206,       0,  0, 12,  0, }, // Ponape
    {  428, 1534,       0,  364218,       0,  0, 15,  0, }, // Pyongyang
    {  428, 1544,  364233,  516921,  364271, 15, 22, 27, }, // Qyzylorda
    {  428, 1554,       0,  364298,       0,  0, 13,  0, }, // Reunion
    {  428, 1562,       0,  364311,       0,  0, 13,  0, }, // Rothera
    {  428, 1570,  364324,  516943,  364360, 14, 21, 26, }, // Sakhalin
    {  428, 1579,  364386,  516964,  364418, 12, 19, 24, }, // Samara
    {  428, 1596,  364442,  516983,  364472, 11, 18, 23, }, // Samoa
    {  428, 1602,       0,  364495,       0,  0, 16,  0, }, // Seychelles
    {  428, 1624,       0,  517001,       0,  0, 22,  0, }, // Singapore
    {  428, 1634,       0,  364534,       0,  0, 23,  0, }, // Solomon
    {  428, 1642,       0,  364557,       0,  0, 21,  0, }, // South_Georgia
    {  428, 1656,       0,  364578,       0,  0, 14,  0, }, // Suriname
    {  428, 1676,       0,  364592,       0,  0, 11,  0, }, // Syowa
    {  428, 1682,       0,  364603,       0,  0, 12,  0, }, // Tahiti
    {  428, 1689,  364615,  517023,  517042, 12, 19, 18, }, // Taipei
    {  428, 1696,       0,  364671,       0,  0, 16,  0, }, // Tajikistan
    {  428, 1724,       0,  364687,       0,  0, 13,  0, }, // Tokelau
    {  428, 1732,  364700,  517060,  364730, 11, 18, 23, }, // Tonga
    {  428, 1738,       0,  364753,       0,  0, 11,  0, }, // Truk
    {  428, 1750,  364764,  517078,  364808, 18, 25, 30, }, // Turkmenistan
    {  428, 1763,       0,  364838,       0,  0, 12,  0, }, // Tuvalu
    {  428, 1777,  364850,  517103,  364884, 13, 20, 25, }, // Uruguay
    {  428, 1792,  364909,  517123,  364949, 16, 23, 28, }, // Uzbekistan
    {  428, 1803,  364977,  517146,  365011, 13, 20, 25, }, // Vanuatu
    {  428, 1811,       0,  365036,       0,  0, 15,  0, }, // Venezuela
    {  428, 1821,  365051,  517166,  365093, 17, 24, 29, }, // Vladivostok
    {  428, 1833,  365122,  517190,  365160, 15, 22, 27, }, // Volgograd
    {  428, 1843,       0,  365187,       0,  0, 12,  0, }, // Vostok
    {  428, 1850,       0,  517212,       0,  0, 16,  0, }, // Wake
    {  428, 1855,       0,  365219,       0,  0, 23,  0, }, // Wallis
    {  428, 1862,  365242,  517228,  365276, 13, 20, 25, }, // Yakutsk
    {  428, 1870,  365301,  517248,  365347, 19, 26, 31, }, // Yekaterinburg
    {  428, 1892,       0,  517274,       0,  0, 10,  0, }, // Yukon
    {  429,  454,       0,  517284,       0,  0, 20,  0, }, // Brunei Malay/Arabic/Brunei
    {  429,  914,       0,  517304,       0,  0, 18,  0, }, // India
    {  429,  920,       0,  517322,       0,  0, 17,  0, }, // Indian_Ocean
    {  429,  943,       0,  517339,       0,  0, 18,  0, }, // Indonesia_Central
    {  429,  961,       0,  517357,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  429,  979,       0,  517377,       0,  0, 20,  0, }, // Indonesia_Western
    {  429, 1219,       0,  517397,       0,  0, 11,  0, }, // Malaysia
    {  430,  454,       0,  517284,       0,  0, 20,  0, }, // Brunei Malay/Arabic/Malaysia
    {  430,  914,       0,  517304,       0,  0, 18,  0, }, // India
    {  430,  920,       0,  517322,       0,  0, 17,  0, }, // Indian_Ocean
    {  430,  943,       0,  517339,       0,  0, 18,  0, }, // Indonesia_Central
    {  430,  961,       0,  517357,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  430,  979,       0,  517377,       0,  0, 20,  0, }, // Indonesia_Western
    {  430, 1219,       0,  517397,       0,  0, 11,  0, }, // Malaysia
    {  434,  627,  517408,  517428,  517457, 20, 29, 29, }, // Europe_Central Maltese/Latin/Malta
    {  434,  788,       0,    2791,       0,  0, 19,  0, }, // GMT
    {  435,  142,  517486,  517515,  517555, 29, 40, 36, }, // America_Central Manipuri/Bangla/India
    {  435,  158,  517591,  517618,  517656, 27, 38, 34, }, // America_Eastern
    {  435,  174,  517690,  517718,  517757, 28, 39, 35, }, // America_Mountain
    {  435,  191,  517792,  517819,  517857, 27, 38, 34, }, // America_Pacific
    {  435,  286,  517891,  517905,  517930, 14, 25, 21, }, // Atlantic
    {  435,  627,  517951,  517977,  518014, 26, 37, 30, }, // Europe_Central
    {  435,  642,  518044,  518068,  518103, 24, 35, 28, }, // Europe_Eastern
    {  435,  680,  518131,  518156,  518192, 25, 36, 29, }, // Europe_Western
    {  435,  788,       0,  518221,       0,  0, 17,  0, }, // GMT
    {  438,    5,       0,  518238,       0,  0, 15,  0, }, // Afghanistan Maori/Latin/New Zealand
    {  438,   17,       0,  518253,       0,  0, 23,  0, }, // Africa_Central
    {  438,   32,       0,  518276,       0,  0, 27,  0, }, // Africa_Eastern
    {  438,   65,       0,  518303,       0,  0, 35,  0, }, // Africa_Southern
    {  438,   81,  518338,  518361,  518393, 23, 32, 31, }, // Africa_Western
    {  438,  107,  518424,  518433,  518452,  9, 19, 16, }, // Alaska
    {  438,  135,  518468,  518478,  518498, 10, 20, 18, }, // Amazon
    {  438,  142,  518516,  518525,  518544,  9, 19, 16, }, // America_Central
    {  438,  158,  518560,  518570,  518590, 10, 20, 17, }, // America_Eastern
    {  438,  174,  518607,  518616,  518635,  9, 19, 16, }, // America_Mountain
    {  438,  191,  518651,  518658,  518675,  7, 17, 14, }, // America_Pacific
    {  438,  214,  518689,  518696,  518713,  7, 17, 14, }, // Apia
    {  438,  232,  518727,  518736,  518755,  9, 19, 16, }, // Arabian
    {  438,  240,  518771,  518781,  518801, 10, 20, 18, }, // Argentina
    {  438,  250,  518819,  518839,  518869, 20, 30, 28, }, // Argentina_Western
    {  438,  268,  518897,  518906,  518925,  9, 19, 17, }, // Armenia
    {  438,  286,  518942,  518953,  518974, 11, 21, 18, }, // Atlantic
    {  438,  295,  518992,  519013,  519044, 21, 31, 28, }, // Australia_Central
    {  438,  313,  519072,  519097,  519132, 25, 35, 32, }, // Australia_CentralWestern
    {  438,  338,  519164,  519192,  519230, 28, 38, 35, }, // Australia_Eastern
    {  438,  356,  519265,  519289,  519323, 24, 34, 31, }, // Australia_Western
    {  438,  374,  519354,  519368,  519392, 14, 24, 22, }, // Azerbaijan
    {  438,  385,  519414,  519423,  519442,  9, 19, 17, }, // Azores
    {  438,  397,  519459,  519472,  519495, 13, 23, 21, }, // Bangladesh
    {  438,  415,       0,  519516,       0,  0,  9,  0, }, // Bhutan
    {  438,  422,       0,  519525,       0,  0, 10,  0, }, // Bolivia
    {  438,  437,  519535,  519545,  519565, 10, 20, 18, }, // Brasilia
    {  438,  454,       0,  519583,       0,  0, 21,  0, }, // Brunei
    {  438,  461,  519604,  519626,  519658, 22, 32, 30, }, // Cape_Verde
    {  438,  478,       0,  519688,       0,  0, 21,  0, }, // Chamorro
    {  438,  487,  519709,  519718,  519737,  9, 19, 16, }, // Chatham
    {  438,  495,  519753,  519760,  519777,  7, 17, 15, }, // Chile
    {  438,  501,  519792,  519800,  519818,  8, 18, 15, }, // China
    {  438,  507,       0,  519833,       0,  0, 26,  0, }, // Christmas
    {  438,  517,       0,  519859,       0,  0, 23,  0, }, // Cocos
    {  438,  523,  519882,  519894,  519916, 12, 22, 20, }, // Colombia
    {  438,  532,  519936,  519950,  519974, 14, 24, 29, }, // Cook
    {  438,  537,  520003,  520011,  520029,  8, 18, 15, }, // Cuba
    {  438,  548,       0,  520044,       0,  0,  9,  0, }, // Davis
    {  438,  564,       0,  520053,       0,  0, 19,  0, }, // DumontDUrville
    {  438,  601,       0,  520072,       0,  0, 24,  0, }, // East_Timor
    {  438,  612,  520096,  520121,  520154, 25, 33, 31, }, // Easter
    {  438,  619,       0,  520185,       0,  0, 10,  0, }, // Ecuador
    {  438,  627,  520195,  520210,  520235, 15, 25, 23, }, // Europe_Central
    {  438,  642,  520258,  520274,  520300, 16, 26, 24, }, // Europe_Eastern
    {  438,  657,       0,  520324,       0,  0, 27,  0, }, // Europe_Further_Eastern
    {  438,  680,  520351,  520363,  520385, 12, 22, 20, }, // Europe_Western
    {  438,  695,  520405,  520427,  520459, 22, 32, 30, }, // Falkland
    {  438,  704,  520489,  520497,  520515,  8, 18, 16, }, // Fiji
    {  438,  709,       0,  520531,       0,  0, 13,  0, }, // French_Guiana
    {  438,  723,       0,  520544,       0,  0, 48,  0, }, // French_Southern
    {  438,  746,       0,  520592,       0,  0, 12,  0, }, // Galapagos
    {  438,  756,       0,  520604,       0,  0, 10,  0, }, // Gambier
    {  438,  764,  520614,  520622,  520640,  8, 18, 16, }, // Georgia
    {  438,  772,       0,  520656,       0,  0, 11,  0, }, // Gilbert_Islands
    {  438,  788,       0,  520667,       0,  0, 20,  0, }, // GMT
    {  438,  830,  520687,  520716,  520757, 29, 41, 39, }, // Greenland_Eastern
    {  438,  848,  520796,  520821,  520858, 25, 37, 35, }, // Greenland_Western
    {  438,  871,       0,  520893,       0,  0, 19,  0, }, // Gulf
    {  438,  876,       0,  520912,       0,  0,  9,  0, }, // Guyana
    {  438,  883,  520921,  520939,  520967, 18, 28, 25, }, // Hawaii_Aleutian
    {  438,  899,  520992,  521003,  521024, 11, 21, 19, }, // Hong_Kong
    {  438,  909,  521043,  521050,  521067,  7, 17, 15, }, // Hovd
    {  438,  914,       0,  521082,       0,  0,  7,  0, }, // India
    {  438,  920,       0,  521089,       0,  0, 18,  0, }, // Indian_Ocean
    {  438,  933,       0,  521107,       0,  0, 12,  0, }, // Indochina
    {  438,  943,       0,  521119,       0,  0, 20,  0, }, // Indonesia_Central
    {  438,  961,       0,  521139,       0,  0, 27,  0, }, // Indonesia_Eastern
    {  438,  979,       0,  521166,       0,  0, 23,  0, }, // Indonesia_Western
    {  438,  997,  521189,  521197,  521215,  8, 18, 15, }, // Iran
    {  438, 1008,  521230,  521240,  521260, 10, 20, 18, }, // Irkutsk
    {  438, 1016,  521278,  521289,  521310, 11, 21, 18, }, // Israel
    {  438, 1023,  521328,  521337,  521356,  9, 19, 16, }, // Japan
    {  438, 1047,       0,  521372,       0,  0, 12,  0, }, // Kazakhstan
    {  438, 1058,       0,  521384,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  438, 1077,       0,  521410,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  438, 1106,  521432,  521440,  521458,  8, 18, 15, }, // Korea
    {  438, 1112,       0,  521473,       0,  0,  9,  0, }, // Kosrae
    {  438, 1119,  521482,  521496,  521520, 14, 24, 22, }, // Krasnoyarsk
    {  438, 1151,       0,  521542,       0,  0, 12,  0, }, // Kyrgystan
    {  438, 1175,       0,  521554,       0,  0, 26,  0, }, // Line_Islands
    {  438, 1188,  521580,  521592,  521614, 12, 22, 19, }, // Lord_Howe
    {  438, 1204,  521633,  521643,  521663, 10, 20, 18, }, // Magadan
    {  438, 1219,       0,  521681,       0,  0,  9,  0, }, // Malaysia
    {  438, 1228,       0,  521690,       0,  0, 11,  0, }, // Maldives
    {  438, 1237,       0,  521701,       0,  0, 12,  0, }, // Marquesas
    {  438, 1247,       0,  521713,       0,  0, 20,  0, }, // Marshall_Islands
    {  438, 1264,  521733,  521742,  521761,  9, 19, 17, }, // Mauritius
    {  438, 1274,       0,  521778,       0,  0,  9,  0, }, // Mawson
    {  438, 1281,  521787,  521801,  521825, 14, 24, 21, }, // Mexico_Pacific
    {  438, 1296,  521846,  521860,  521884, 14, 24, 22, }, // Mongolia
    {  438, 1305,  521906,  521916,  521936, 10, 20, 18, }, // Moscow
    {  438, 1312,       0,  521954,       0,  0,  7,  0, }, // Myanmar
    {  438, 1320,       0,  521961,       0,  0,  8,  0, }, // Nauru
    {  438, 1326,       0,  521969,       0,  0,  9,  0, }, // Nepal
    {  438, 1332,  521978,  521994,  522020, 16, 26, 24, }, // New_Caledonia
    {  438, 1346,  522044,  522055,  522076, 11, 21, 18, }, // New_Zealand
    {  438, 1358,  522094,  522109,  522134, 15, 25, 22, }, // Newfoundland
    {  438, 1371,       0,  522156,       0,  0,  7,  0, }, // Niue
    {  438, 1376,  522163,  522185,  522217, 22, 32, 29, }, // Norfolk
    {  438, 1384,  522246,  522268,  522300, 22, 32, 30, }, // Noronha
    {  438, 1406,  522330,  522344,  522368, 14, 24, 22, }, // Novosibirsk
    {  438, 1418,  522390,  522397,  522414,  7, 17, 15, }, // Omsk
    {  438, 1428,  522429,  522440,  522461, 11, 21, 19, }, // Pakistan
    {  438, 1437,       0,  522480,       0,  0,  8,  0, }, // Palau
    {  438, 1443,       0,  522488,       0,  0, 15,  0, }, // Papua_New_Guinea
    {  438, 1460,  522503,  522513,  522533, 10, 20, 18, }, // Paraguay
    {  438, 1469,  522551,  522558,  522575,  7, 17, 15, }, // Peru
    {  438, 1474,  522590,  522601,  522622, 11, 21, 19, }, // Philippines
    {  438, 1486,       0,  522641,       0,  0, 24,  0, }, // Phoenix_Islands
    {  438, 1502,  522665,  522690,  522727, 25, 37, 34, }, // Pierre_Miquelon
    {  438, 1518,       0,  522761,       0,  0, 11,  0, }, // Pitcairn
    {  438, 1527,       0,  522772,       0,  0,  9,  0, }, // Ponape
    {  438, 1534,       0,  522781,       0,  0, 12,  0, }, // Pyongyang
    {  438, 1554,       0,  522793,       0,  0, 10,  0, }, // Reunion
    {  438, 1562,       0,  522803,       0,  0, 10,  0, }, // Rothera
    {  438, 1570,  522813,  522824,  522845, 11, 21, 19, }, // Sakhalin
    {  438, 1596,  522864,  522872,  522890,  8, 18, 15, }, // Samoa
    {  438, 1602,       0,  522905,       0,  0, 10,  0, }, // Seychelles
    {  438, 1624,       0,  522915,       0,  0, 21,  0, }, // Singapore
    {  438, 1634,       0,  522936,       0,  0, 22,  0, }, // Solomon
    {  438, 1642,       0,  522958,       0,  0, 20,  0, }, // South_Georgia
    {  438, 1656,       0,  522978,       0,  0, 11,  0, }, // Suriname
    {  438, 1676,       0,  522989,       0,  0,  8,  0, }, // Syowa
    {  438, 1682,       0,  522997,       0,  0,  9,  0, }, // Tahiti
    {  438, 1689,  523006,  523015,  523034,  9, 19, 16, }, // Taipei
    {  438, 1696,       0,  523050,       0,  0, 14,  0, }, // Tajikistan
    {  438, 1724,       0,  523064,       0,  0, 10,  0, }, // Tokelau
    {  438, 1732,  523074,  523082,  523100,  8, 18, 16, }, // Tonga
    {  438, 1738,       0,  523116,       0,  0,  8,  0, }, // Truk
    {  438, 1750,  523124,  523140,  523166, 16, 26, 24, }, // Turkmenistan
    {  438, 1763,       0,  523190,       0,  0,  9,  0, }, // Tuvalu
    {  438, 1777,  523199,  523208,  523227,  9, 19, 17, }, // Uruguay
    {  438, 1792,  523244,  523258,  523282, 14, 24, 22, }, // Uzbekistan
    {  438, 1803,  523304,  523315,  523336, 11, 21, 19, }, // Vanuatu
    {  438, 1811,       0,  523355,       0,  0, 12,  0, }, // Venezuela
    {  438, 1821,  523367,  523381,  523405, 14, 24, 22, }, // Vladivostok
    {  438, 1833,  523427,  523439,  523461, 12, 22, 20, }, // Volgograd
    {  438, 1843,       0,  523481,       0,  0,  9,  0, }, // Vostok
    {  438, 1850,       0,  523490,       0,  0, 17,  0, }, // Wake
    {  438, 1855,       0,  523507,       0,  0, 20,  0, }, // Wallis
    {  438, 1862,  523527,  523537,  523557, 10, 20, 18, }, // Yakutsk
    {  438, 1870,  523575,  523591,  523617, 16, 26, 24, }, // Yekaterinburg
    {  438, 1892,       0,  523641,       0,  0,  8,  0, }, // Yukon
    {  440,    0,  523649,  523656,  523669,  7, 13, 15, }, // Acre Marathi/Devanagari/India
    {  440,    5,       0,  523684,       0,  0, 15,  0, }, // Afghanistan
    {  440,   17,       0,  523699,       0,  0, 17,  0, }, // Africa_Central
    {  440,   32,       0,  523716,       0,  0, 17,  0, }, // Africa_Eastern
    {  440,   65,       0,  523733,       0,  0, 25,  0, }, // Africa_Southern
    {  440,   81,  523758,  523776,  523801, 18, 25, 26, }, // Africa_Western
    {  440,  107,  437999,  523827,  523845, 11, 18, 23, }, // Alaska
    {  440,  128,  523868,  523879,  523896, 11, 17, 24, }, // Almaty
    {  440,  135,  523920,  523932,  523951, 12, 19, 20, }, // Amazon
    {  440,  142,  523971,  523983,  524002, 12, 19, 24, }, // America_Central
    {  440,  158,  524026,  524039,  524059, 13, 20, 25, }, // America_Eastern
    {  440,  174,  438178,  524084,  524102, 11, 18, 23, }, // America_Mountain
    {  440,  191,  524125,  524136,  524154, 11, 18, 23, }, // America_Pacific
    {  440,  207,  524177,  524188,  524204, 11, 16, 23, }, // Anadyr
    {  440,  214,  524227,  524236,  524252,  9, 16, 21, }, // Apia
    {  440,  219,  524273,  524285,  524303, 12, 18, 25, }, // Aqtau
    {  440,  225,  524328,  524341,  524360, 13, 19, 26, }, // Aqtobe
    {  440,  232,  524386,  524397,  524415, 11, 18, 23, }, // Arabian
    {  440,  240,  438359,  524438,  524459, 14, 21, 22, }, // Argentina
    {  440,  250,  524481,  524503,  524532, 22, 29, 30, }, // Argentina_Western
    {  440,  268,  438498,  524562,  524582, 13, 20, 21, }, // Armenia
    {  440,  286,  524603,  524615,  524634, 12, 19, 24, }, // Atlantic
    {  440,  295,  524658,  524678,  524705, 20, 27, 32, }, // Australia_Central
    {  440,  313,  524737,  524764,  524798, 27, 34, 39, }, // Australia_CentralWestern
    {  440,  338,  524837,  524858,  524886, 21, 28, 33, }, // Australia_Eastern
    {  440,  356,  524919,  524941,  524970, 22, 29, 34, }, // Australia_Western
    {  440,  374,  525004,  525016,  525035, 12, 19, 20, }, // Azerbaijan
    {  440,  385,  525055,  525067,  525086, 12, 19, 20, }, // Azores
    {  440,  397,  439036,  525106,  525126, 13, 20, 21, }, // Bangladesh
    {  440,  415,       0,  525147,       0,  0,  9,  0, }, // Bhutan
    {  440,  422,       0,  439097,       0,  0, 14,  0, }, // Bolivia
    {  440,  437,  439111,  525156,  525177, 14, 21, 22, }, // Brasilia
    {  440,  454,       0,  525199,       0,  0, 20,  0, }, // Brunei
    {  440,  461,  525219,  525234,  525256, 15, 22, 23, }, // Cape_Verde
    {  440,  478,       0,  525279,       0,  0, 17,  0, }, // Chamorro
    {  440,  487,  439267,  525296,  525311,  8, 15, 20, }, // Chatham
    {  440,  495,  439307,  525331,  525346,  8, 15, 16, }, // Chile
    {  440,  501,  525362,  525370,  525385,  8, 15, 20, }, // China
    {  440,  507,       0,  525405,       0,  0, 15,  0, }, // Christmas
    {  440,  517,       0,  525420,       0,  0, 14,  0, }, // Cocos
    {  440,  523,  439417,  525434,  525453, 12, 19, 20, }, // Colombia
    {  440,  532,  525473,  525485,  525504, 12, 19, 25, }, // Cook
    {  440,  537,  525529,  525539,  525556, 10, 17, 22, }, // Cuba
    {  440,  548,       0,  525578,       0,  0, 11,  0, }, // Davis
    {  440,  564,       0,  525589,       0,  0, 23,  0, }, // DumontDUrville
    {  440,  601,       0,  525612,       0,  0, 15,  0, }, // East_Timor
    {  440,  612,  525627,  525640,  525660, 13, 20, 21, }, // Easter
    {  440,  619,       0,  439683,       0,  0, 12,  0, }, // Ecuador
    {  440,  627,  525681,  525699,  525724, 18, 25, 26, }, // Europe_Central
    {  440,  642,  525750,  525768,  525793, 18, 25, 26, }, // Europe_Eastern
    {  440,  657,       0,  525819,       0,  0, 27,  0, }, // Europe_Further_Eastern
    {  440,  680,  525846,  525865,  525891, 19, 26, 27, }, // Europe_Western
    {  440,  695,  525918,  525933,  525955, 15, 22, 23, }, // Falkland
    {  440,  704,  440002,  525978,  525993,  8, 15, 16, }, // Fiji
    {  440,  709,       0,  440043,       0,  0, 16,  0, }, // French_Guiana
    {  440,  723,       0,  526009,       0,  0, 33,  0, }, // French_Southern
    {  440,  746,       0,  526042,       0,  0, 13,  0, }, // Galapagos
    {  440,  756,       0,  526055,       0,  0, 12,  0, }, // Gambier
    {  440,  764,  440118,  526067,  526086, 12, 19, 20, }, // Georgia
    {  440,  772,       0,  526106,       0,  0, 17,  0, }, // Gilbert_Islands
    {  440,  788,       0,  526123,       0,  0, 18,  0, }, // GMT
    {  440,  830,  526141,  526159,  526184, 18, 25, 26, }, // Greenland_Eastern
    {  440,  848,  526210,  526229,  526255, 19, 26, 27, }, // Greenland_Western
    {  440,  866,       0,  526282,       0,  0, 14,  0, }, // Guam
    {  440,  871,       0,  526296,       0,  0, 15,  0, }, // Gulf
    {  440,  876,       0,  440050,       0,  0,  9,  0, }, // Guyana
    {  440,  883,  526311,  526325,  526346, 14, 21, 26, }, // Hawaii_Aleutian
    {  440,  899,  440445,  526372,  526392, 13, 20, 21, }, // Hong_Kong
    {  440,  909,  526413,  526424,  526442, 11, 18, 19, }, // Hovd
    {  440,  914,       0,  526461,       0,  0, 17,  0, }, // India
    {  440,  920,       0,  526478,       0,  0, 16,  0, }, // Indian_Ocean
    {  440,  933,       0,  526494,       0,  0, 13,  0, }, // Indochina
    {  440,  943,       0,  526507,       0,  0, 20,  0, }, // Indonesia_Central
    {  440,  961,       0,  526527,       0,  0, 24,  0, }, // Indonesia_Eastern
    {  440,  979,       0,  526551,       0,  0, 22,  0, }, // Indonesia_Western
    {  440,  997,  526573,  526581,  526596,  8, 15, 20, }, // Iran
    {  440, 1008,  526616,  526629,  526649, 13, 20, 21, }, // Irkutsk
    {  440, 1016,  526670,  526677,  526695,  7, 18, 23, }, // Israel
    {  440, 1023,  440788,  526718,  526733,  8, 15, 20, }, // Japan
    {  440, 1029,  526753,  526786,  526825, 33, 39, 46, }, // Kamchatka
    {  440, 1047,       0,  440847,       0,  0, 13,  0, }, // Kazakhstan
    {  440, 1058,       0,  526871,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  440, 1077,       0,  526890,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  440, 1106,  440880,  526910,  526927, 10, 17, 22, }, // Korea
    {  440, 1112,       0,  526949,       0,  0, 11,  0, }, // Kosrae
    {  440, 1119,  440935,  526960,  526986, 19, 26, 27, }, // Krasnoyarsk
    {  440, 1151,       0,  527013,       0,  0, 14,  0, }, // Kyrgystan
    {  440, 1161,       0,  527027,       0,  0,  8,  0, }, // Lanka
    {  440, 1175,       0,  527035,       0,  0, 13,  0, }, // Line_Islands
    {  440, 1188,  441040,  527048,  527069, 14, 21, 26, }, // Lord_Howe
    {  440, 1198,  527095,  527103,  527117,  8, 14, 21, }, // Macau
    {  440, 1204,  527138,  527148,  527165, 10, 17, 18, }, // Magadan
    {  440, 1219,       0,  441145,       0,  0, 11,  0, }, // Malaysia
    {  440, 1228,       0,  527183,       0,  0, 10,  0, }, // Maldives
    {  440, 1237,       0,  527193,       0,  0, 15,  0, }, // Marquesas
    {  440, 1247,       0,  527208,       0,  0, 15,  0, }, // Marshall_Islands
    {  440, 1264,  527223,  527233,  527250, 10, 17, 18, }, // Mauritius
    {  440, 1274,       0,  441241,       0,  0,  8,  0, }, // Mawson
    {  440, 1281,  527268,  527288,  527315, 20, 27, 32, }, // Mexico_Pacific
    {  440, 1296,  527347,  527361,  527382, 14, 21, 22, }, // Mongolia
    {  440, 1305,  441378,  527404,  527421, 10, 17, 18, }, // Moscow
    {  440, 1312,       0,  441425,       0,  0, 12,  0, }, // Myanmar
    {  440, 1320,       0,  527439,       0,  0,  8,  0, }, // Nauru
    {  440, 1326,       0,  441444,       0,  0,  9,  0, }, // Nepal
    {  440, 1332,  527447,  527466,  527492, 19, 26, 27, }, // New_Caledonia
    {  440, 1346,  527519,  527532,  527552, 13, 20, 25, }, // New_Zealand
    {  440, 1358,  527577,  527594,  527618, 17, 24, 29, }, // Newfoundland
    {  440, 1371,       0,  441652,       0,  0,  9,  0, }, // Niue
    {  440, 1376,  527647,  527661,  527682, 14, 21, 26, }, // Norfolk
    {  440, 1384,  441726,  527708,  527739, 24, 31, 32, }, // Noronha
    {  440, 1392,       0,  527771,       0,  0, 20,  0, }, // North_Mariana
    {  440, 1406,  527791,  527808,  527832, 17, 24, 25, }, // Novosibirsk
    {  440, 1418,  441866,  527857,  527874, 10, 17, 18, }, // Omsk
    {  440, 1428,  527892,  527905,  527925, 13, 20, 21, }, // Pakistan
    {  440, 1437,       0,  441956,       0,  0,  8,  0, }, // Palau
    {  440, 1443,       0,  527946,       0,  0, 19,  0, }, // Papua_New_Guinea
    {  440, 1460,  527965,  527977,  527996, 12, 19, 20, }, // Paraguay
    {  440, 1469,  528016,  528024,  528039,  8, 15, 16, }, // Peru
    {  440, 1474,  528055,  528067,  528086, 12, 19, 20, }, // Philippines
    {  440, 1486,       0,  528106,       0,  0, 17,  0, }, // Phoenix_Islands
    {  440, 1502,  528123,  528150,  528185, 27, 35, 40, }, // Pierre_Miquelon
    {  440, 1518,       0,  528225,       0,  0, 12,  0, }, // Pitcairn
    {  440, 1527,       0,  528237,       0,  0, 10,  0, }, // Ponape
    {  440, 1534,       0,  442249,       0,  0, 14,  0, }, // Pyongyang
    {  440, 1544,  528247,  528262,  528283, 15, 21, 28, }, // Qyzylorda
    {  440, 1554,       0,  442263,       0,  0, 12,  0, }, // Reunion
    {  440, 1562,       0,  442275,       0,  0, 10,  0, }, // Rothera
    {  440, 1570,  442285,  528311,  528327,  9, 16, 17, }, // Sakhalin
    {  440, 1579,  528344,  528353,  528368,  9, 15, 22, }, // Samara
    {  440, 1596,  442329,  528390,  528406,  9, 16, 21, }, // Samoa
    {  440, 1602,       0,  442372,       0,  0, 11,  0, }, // Seychelles
    {  440, 1624,       0,  528427,       0,  0, 19,  0, }, // Singapore
    {  440, 1634,       0,  528446,       0,  0, 16,  0, }, // Solomon
    {  440, 1642,       0,  442422,       0,  0, 19,  0, }, // South_Georgia
    {  440, 1656,       0,  442441,       0,  0, 11,  0, }, // Suriname
    {  440, 1676,       0,  442452,       0,  0, 10,  0, }, // Syowa
    {  440, 1682,       0,  442462,       0,  0, 10,  0, }, // Tahiti
    {  440, 1689,  442472,  528462,  528478,  9, 16, 21, }, // Taipei
    {  440, 1696,       0,  528499,       0,  0, 15,  0, }, // Tajikistan
    {  440, 1724,       0,  528514,       0,  0, 11,  0, }, // Tokelau
    {  440, 1732,  442539,  528525,  528541,  9, 16, 17, }, // Tonga
    {  440, 1738,       0,  528558,       0,  0,  7,  0, }, // Truk
    {  440, 1750,  528565,  528583,  528608, 18, 25, 26, }, // Turkmenistan
    {  440, 1763,       0,  528634,       0,  0, 10,  0, }, // Tuvalu
    {  440, 1777,  442653,  528644,  528662, 11, 18, 19, }, // Uruguay
    {  440, 1792,  528681,  528696,  528718, 15, 22, 23, }, // Uzbekistan
    {  440, 1803,  528741,  528752,  528770, 11, 18, 19, }, // Vanuatu
    {  440, 1811,       0,  528789,       0,  0, 15,  0, }, // Venezuela
    {  440, 1821,  442828,  528804,  528828, 17, 24, 25, }, // Vladivostok
    {  440, 1833,  442896,  528853,  528877, 17, 24, 25, }, // Volgograd
    {  440, 1843,       0,  528902,       0,  0, 13,  0, }, // Vostok
    {  440, 1850,       0,  528915,       0,  0, 11,  0, }, // Wake
    {  440, 1855,       0,  528926,       0,  0, 20,  0, }, // Wallis
    {  440, 1862,  528946,  528958,  528977, 12, 19, 20, }, // Yakutsk
    {  440, 1870,  528997,  529012,  529034, 15, 22, 23, }, // Yekaterinburg
    {  440, 1892,       0,  529057,       0,  0,  9,  0, }, // Yukon
    {  447,    5,       0,  529066,       0,  0, 15,  0, }, // Afghanistan Mongolian/Cyrillic/Mongolia
    {  447,   17,       0,  529081,       0,  0, 16,  0, }, // Africa_Central
    {  447,   32,       0,  529097,       0,  0, 17,  0, }, // Africa_Eastern
    {  447,   65,       0,  529114,       0,  0, 27,  0, }, // Africa_Southern
    {  447,   81,  529141,  529160,  529188, 19, 28, 24, }, // Africa_Western
    {  447,  107,  529212,  529224,  529245, 12, 21, 17, }, // Alaska
    {  447,  135,  529262,  529273,  529293, 11, 20, 16, }, // Amazon
    {  447,  142,  529309,  529316,  529332,  7, 16, 12, }, // America_Central
    {  447,  158,  529344,  529359,  529383, 15, 24, 20, }, // America_Eastern
    {  447,  174,  529403,  529412,  529430,  9, 18, 14, }, // America_Mountain
    {  447,  191,  529444,  529461,  529487, 17, 26, 22, }, // America_Pacific
    {  447,  214,  529509,  529521,  529542, 12, 21, 17, }, // Apia
    {  447,  232,  529559,  529569,  529588, 10, 19, 15, }, // Arabian
    {  447,  240,  529603,  529616,  529638, 13, 22, 18, }, // Argentina
    {  447,  250,  529656,  529676,  529705, 20, 29, 25, }, // Argentina_Western
    {  447,  268,  529730,  529742,  529763, 12, 21, 17, }, // Armenia
    {  447,  286,  529780,  529792,  529813, 12, 21, 17, }, // Atlantic
    {  447,  295,  529830,  529848,  529875, 18, 27, 23, }, // Australia_Central
    {  447,  313,  529898,  529930,  529971, 32, 41, 37, }, // Australia_CentralWestern
    {  447,  338,  530008,  530027,  530062, 19, 35, 31, }, // Australia_Eastern
    {  447,  356,  530093,  530114,  530151, 21, 37, 33, }, // Australia_Western
    {  447,  374,  530184,  530199,  530223, 15, 24, 20, }, // Azerbaijan
    {  447,  385,  530243,  530253,  530272, 10, 19, 15, }, // Azores
    {  447,  397,  530287,  530303,  530328, 16, 25, 21, }, // Bangladesh
    {  447,  415,       0,  530349,       0,  0, 10,  0, }, // Bhutan
    {  447,  422,       0,  530359,       0,  0, 12,  0, }, // Bolivia
    {  447,  437,  530371,  530384,  530406, 13, 22, 18, }, // Brasilia
    {  447,  454,       0,  530424,       0,  0, 23,  0, }, // Brunei
    {  447,  461,  530447,  530463,  530488, 16, 25, 21, }, // Cape_Verde
    {  447,  478,       0,  530509,       0,  0, 15,  0, }, // Chamorro
    {  447,  487,  530524,  530535,  530555, 11, 20, 16, }, // Chatham
    {  447,  495,  530571,  530581,  530600, 10, 19, 15, }, // Chile
    {  447,  501,  530615,  530626,  530646, 11, 20, 16, }, // China
    {  447,  507,       0,  530662,       0,  0, 17,  0, }, // Christmas
    {  447,  517,       0,  530679,       0,  0, 15,  0, }, // Cocos
    {  447,  523,  530694,  530706,  530727, 12, 21, 17, }, // Colombia
    {  447,  532,  530744,  530764,  530793, 20, 29, 31, }, // Cook
    {  447,  537,  530824,  530833,  530851,  9, 18, 14, }, // Cuba
    {  447,  548,       0,  530865,       0,  0, 12,  0, }, // Davis
    {  447,  564,       0,  530877,       0,  0, 20,  0, }, // DumontDUrville
    {  447,  601,       0,  530897,       0,  0, 16,  0, }, // East_Timor
    {  447,  612,  530913,  530930,  530956, 17, 26, 22, }, // Easter
    {  447,  619,       0,  530978,       0,  0, 13,  0, }, // Ecuador
    {  447,  627,  530991,  531006,  531030, 15, 24, 20, }, // Europe_Central
    {  447,  642,  531050,  531066,  531091, 16, 25, 21, }, // Europe_Eastern
    {  447,  657,       0,  531112,       0,  0, 22,  0, }, // Europe_Further_Eastern
    {  447,  680,  531134,  531152,  531179, 18, 27, 23, }, // Europe_Western
    {  447,  695,  531202,  531226,  531259, 24, 33, 29, }, // Falkland
    {  447,  704,  531288,  531300,  531321, 12, 21, 17, }, // Fiji
    {  447,  709,       0,  531338,       0,  0, 22,  0, }, // French_Guiana
    {  447,  723,       0,  531360,       0,  0, 33,  0, }, // French_Southern
    {  447,  746,       0,  531393,       0,  0, 15,  0, }, // Galapagos
    {  447,  756,       0,  531408,       0,  0, 14,  0, }, // Gambier
    {  447,  764,  531422,  531433,  531453, 11, 20, 16, }, // Georgia
    {  447,  772,       0,  531469,       0,  0, 24,  0, }, // Gilbert_Islands
    {  447,  788,       0,  531493,       0,  0, 14,  0, }, // GMT
    {  447,  830,  531507,  531526,  531554, 19, 28, 24, }, // Greenland_Eastern
    {  447,  848,  531578,  531599,  531629, 21, 30, 26, }, // Greenland_Western
    {  447,  871,       0,  531655,       0,  0, 21,  0, }, // Gulf
    {  447,  876,       0,  531676,       0,  0, 14,  0, }, // Guyana
    {  447,  883,  531690,  531707,  531733, 17, 26, 22, }, // Hawaii_Aleutian
    {  447,  899,  531755,  531771,  531796, 16, 25, 21, }, // Hong_Kong
    {  447,  909,  531817,  531827,  531846, 10, 19, 15, }, // Hovd
    {  447,  914,       0,  531861,       0,  0, 14,  0, }, // India
    {  447,  920,       0,  531875,       0,  0, 21,  0, }, // Indian_Ocean
    {  447,  933,       0,  531896,       0,  0, 27,  0, }, // Indochina
    {  447,  943,       0,  531923,       0,  0, 18,  0, }, // Indonesia_Central
    {  447,  961,       0,  531941,       0,  0, 19,  0, }, // Indonesia_Eastern
    {  447,  979,       0,  531960,       0,  0, 21,  0, }, // Indonesia_Western
    {  447,  997,  531981,  531990,  532008,  9, 18, 14, }, // Iran
    {  447, 1008,  532022,  532035,  532057, 13, 22, 18, }, // Irkutsk
    {  447, 1016,  532075,  532088,  532110, 13, 22, 18, }, // Israel
    {  447, 1023,  532128,  532137,  532155,  9, 18, 14, }, // Japan
    {  447, 1047,       0,  532169,       0,  0, 14,  0, }, // Kazakhstan
    {  447, 1058,       0,  532183,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  447, 1077,       0,  532202,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  447, 1106,  532223,  532237,  532260, 14, 23, 19, }, // Korea
    {  447, 1112,       0,  532279,       0,  0, 13,  0, }, // Kosrae
    {  447, 1119,  532292,  532309,  532335, 17, 26, 22, }, // Krasnoyarsk
    {  447, 1151,       0,  532357,       0,  0, 13,  0, }, // Kyrgystan
    {  447, 1175,       0,  532370,       0,  0, 17,  0, }, // Line_Islands
    {  447, 1188,  532387,  532403,  532428, 16, 25, 21, }, // Lord_Howe
    {  447, 1204,  532449,  532461,  532482, 12, 21, 17, }, // Magadan
    {  447, 1219,       0,  532499,       0,  0, 12,  0, }, // Malaysia
    {  447, 1228,       0,  532511,       0,  0, 14,  0, }, // Maldives
    {  447, 1237,       0,  532525,       0,  0, 21,  0, }, // Marquesas
    {  447, 1247,       0,  532546,       0,  0, 22,  0, }, // Marshall_Islands
    {  447, 1264,  532568,  532581,  532603, 13, 22, 18, }, // Mauritius
    {  447, 1274,       0,  532621,       0,  0, 11,  0, }, // Mawson
    {  447, 1281,  532632,  532656,  532689, 24, 33, 29, }, // Mexico_Pacific
    {  447, 1296,  532718,  532735,  532761, 17, 26, 22, }, // Mongolia
    {  447, 1305,  532783,  532797,  532820, 14, 23, 19, }, // Moscow
    {  447, 1312,       0,  532839,       0,  0, 13,  0, }, // Myanmar
    {  447, 1320,       0,  532852,       0,  0, 13,  0, }, // Nauru
    {  447, 1326,       0,  532865,       0,  0, 10,  0, }, // Nepal
    {  447, 1332,  532875,  532894,  532922, 19, 28, 24, }, // New_Caledonia
    {  447, 1346,  532946,  532963,  532989, 17, 26, 22, }, // New_Zealand
    {  447, 1358,  533011,  533030,  533058, 19, 28, 24, }, // Newfoundland
    {  447, 1371,       0,  533082,       0,  0, 12,  0, }, // Niue
    {  447, 1376,  533094,  533111,  533137, 17, 26, 22, }, // Norfolk
    {  447, 1384,  533159,  533185,  533220, 26, 35, 31, }, // Noronha
    {  447, 1406,  533251,  533269,  533296, 18, 27, 23, }, // Novosibirsk
    {  447, 1418,  533319,  533330,  533350, 11, 20, 16, }, // Omsk
    {  447, 1428,  533366,  533379,  533401, 13, 22, 18, }, // Pakistan
    {  447, 1437,       0,  533419,       0,  0, 13,  0, }, // Palau
    {  447, 1443,       0,  533432,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  447, 1460,  533454,  533467,  533489, 13, 22, 18, }, // Paraguay
    {  447, 1469,  533507,  533519,  533540, 12, 21, 17, }, // Peru
    {  447, 1474,  533557,  533571,  533594, 14, 23, 19, }, // Philippines
    {  447, 1486,       0,  533613,       0,  0, 16,  0, }, // Phoenix_Islands
    {  447, 1502,  533629,  533654,  533688, 25, 34, 30, }, // Pierre_Miquelon
    {  447, 1518,       0,  533718,       0,  0, 14,  0, }, // Pitcairn
    {  447, 1527,       0,  533732,       0,  0, 14,  0, }, // Ponape
    {  447, 1534,       0,  533746,       0,  0, 11,  0, }, // Pyongyang
    {  447, 1554,       0,  533757,       0,  0, 12,  0, }, // Reunion
    {  447, 1562,       0,  533769,       0,  0, 14,  0, }, // Rothera
    {  447, 1570,  533783,  533795,  533816, 12, 21, 17, }, // Sakhalin
    {  447, 1596,  533833,  533846,  533868, 13, 22, 18, }, // Samoa
    {  447, 1602,       0,  533886,       0,  0, 22,  0, }, // Seychelles
    {  447, 1624,       0,  533908,       0,  0, 14,  0, }, // Singapore
    {  447, 1634,       0,  533922,       0,  0, 21,  0, }, // Solomon
    {  447, 1642,       0,  533943,       0,  0, 20,  0, }, // South_Georgia
    {  447, 1656,       0,  533963,       0,  0, 13,  0, }, // Suriname
    {  447, 1676,       0,  533976,       0,  0, 12,  0, }, // Syowa
    {  447, 1682,       0,  533988,       0,  0, 13,  0, }, // Tahiti
    {  447, 1689,  534001,  534012,  534032, 11, 20, 16, }, // Taipei
    {  447, 1696,       0,  534048,       0,  0, 15,  0, }, // Tajikistan
    {  447, 1724,       0,  534063,       0,  0, 15,  0, }, // Tokelau
    {  447, 1732,  534078,  534091,  534113, 13, 22, 18, }, // Tonga
    {  447, 1738,       0,  534131,       0,  0, 11,  0, }, // Truk
    {  447, 1750,  534142,  534159,  534185, 17, 26, 22, }, // Turkmenistan
    {  447, 1763,       0,  534207,       0,  0, 14,  0, }, // Tuvalu
    {  447, 1777,  534221,  534233,  534254, 12, 21, 17, }, // Uruguay
    {  447, 1792,  534271,  534286,  534310, 15, 24, 20, }, // Uzbekistan
    {  447, 1803,  534330,  534345,  534369, 15, 24, 20, }, // Vanuatu
    {  447, 1811,       0,  534389,       0,  0, 15,  0, }, // Venezuela
    {  447, 1821,  534404,  534422,  534449, 18, 27, 23, }, // Vladivostok
    {  447, 1833,  534472,  534487,  534511, 15, 24, 20, }, // Volgograd
    {  447, 1843,       0,  534531,       0,  0, 13,  0, }, // Vostok
    {  447, 1850,       0,  534544,       0,  0, 19,  0, }, // Wake
    {  447, 1855,       0,  534563,       0,  0, 24,  0, }, // Wallis
    {  447, 1862,  534587,  534597,  534616, 10, 19, 15, }, // Yakutsk
    {  447, 1870,  534631,  534650,  534678, 19, 28, 24, }, // Yekaterinburg
    {  447, 1892,       0,  534702,       0,  0, 15,  0, }, // Yukon
    {  449,  142,  534717,  534724,  534743,  7, 19, 16, }, // America_Central Mongolian/Mongolian/Mongolia
    {  449,  158,  534759,  534778,  534809, 19, 31, 28, }, // America_Eastern
    {  449,  174,  534837,  534854,  534883, 17, 29, 22, }, // America_Mountain
    {  449,  191,  534905,  534927,  534962, 22, 35, 31, }, // America_Pacific
    {  449,  286,  534993,  535010,  535038, 17, 28, 23, }, // Atlantic
    {  449,  627,  535061,  535081,  535113, 20, 32, 28, }, // Europe_Central
    {  449,  642,  535141,  535163,  535197, 22, 34, 30, }, // Europe_Eastern
    {  449,  680,  535227,  535251,  535287, 24, 36, 32, }, // Europe_Western
    {  449,  788,       0,  535319,       0,  0, 17,  0, }, // GMT
    {  455,    5,       0,  535336,       0,  0, 15,  0, }, // Afghanistan Nepali/Devanagari/Nepal
    {  455,   17,       0,  535351,       0,  0, 21,  0, }, // Africa_Central
    {  455,   32,       0,  535372,       0,  0, 18,  0, }, // Africa_Eastern
    {  455,   65,       0,  535390,       0,  0, 18,  0, }, // Africa_Southern
    {  455,   81,  535408,  535426,  535449, 18, 23, 31, }, // Africa_Western
    {  455,  107,  535480,  535492,  535509, 12, 17, 17, }, // Alaska
    {  455,  135,  535526,  535536,  535551, 10, 15, 23, }, // Amazon
    {  455,  142,  535574,  535587,  535605, 13, 18, 18, }, // America_Central
    {  455,  158,  334853,  334878,  535623, 10, 15, 15, }, // America_Eastern
    {  455,  174,  535638,  535648,  535663, 10, 15, 15, }, // America_Mountain
    {  455,  191,  535678,  535691,  535709, 13, 18, 18, }, // America_Pacific
    {  455,  214,  535727,  535736,  535750,  9, 14, 14, }, // Apia
    {  455,  232,  535764,  535772,  535785,  8, 13, 13, }, // Arabian
    {  455,  240,  535798,  535812,  535831, 14, 19, 27, }, // Argentina
    {  455,  250,  535858,  535880,  535907, 22, 27, 35, }, // Argentina_Western
    {  455,  268,  535942,  535955,  535973, 13, 18, 26, }, // Armenia
    {  455,  286,  535999,  536013,  536032, 14, 19, 19, }, // Atlantic
    {  455,  295,  536051,  536076,  536106, 25, 30, 30, }, // Australia_Central
    {  455,  313,  536136,  536169,  536207, 33, 38, 38, }, // Australia_CentralWestern
    {  455,  338,  536245,  536267,  536294, 22, 27, 27, }, // Australia_Eastern
    {  455,  356,  536146,  536179,  536217, 23, 28, 28, }, // Australia_Western
    {  455,  374,  536321,  536333,  536350, 12, 17, 25, }, // Azerbaijan
    {  455,  385,  536375,  536386,  536402, 11, 16, 24, }, // Azores
    {  455,  397,  536426,  536439,  536457, 13, 18, 26, }, // Bangladesh
    {  455,  415,       0,  536483,       0,  0, 10,  0, }, // Bhutan
    {  455,  422,       0,  536493,       0,  0, 12,  0, }, // Bolivia
    {  455,  437,  536505,  536519,  536538, 14, 19, 27, }, // Brasilia
    {  455,  454,       0,  536565,       0,  0, 21,  0, }, // Brunei
    {  455,  461,  536586,  536599,  536617, 13, 18, 26, }, // Cape_Verde
    {  455,  478,       0,  536643,       0,  0, 17,  0, }, // Chamorro
    {  455,  487,  536660,  536669,  536683,  9, 14, 14, }, // Chatham
    {  455,  495,  336216,  336224,  336237,  8, 13, 21, }, // Chile
    {  455,  501,  336258,  336265,  536697,  7, 12, 12, }, // China
    {  455,  507,       0,  536709,       0,  0, 17,  0, }, // Christmas
    {  455,  517,       0,  536726,       0,  0, 14,  0, }, // Cocos
    {  455,  523,  536740,  536755,  536775, 15, 20, 28, }, // Colombia
    {  455,  532,  536803,  536815,  536832, 12, 17, 29, }, // Cook
    {  455,  537,  536861,  536873,  536890, 12, 17, 17, }, // Cuba
    {  455,  548,       0,  536907,       0,  0,  9,  0, }, // Davis
    {  455,  564,       0,  536916,       0,  0, 23,  0, }, // DumontDUrville
    {  455,  601,       0,  536939,       0,  0, 16,  0, }, // East_Timor
    {  455,  612,  536955,  536969,  536988, 14, 19, 22, }, // Easter
    {  455,  619,       0,  537010,       0,  0, 12,  0, }, // Ecuador
    {  455,  627,  537022,  537044,  537071, 22, 27, 35, }, // Europe_Central
    {  455,  642,  537106,  537125,  537149, 19, 24, 32, }, // Europe_Eastern
    {  455,  657,       0,  537181,       0,  0, 22,  0, }, // Europe_Further_Eastern
    {  455,  680,  537203,  537223,  537081, 20, 25, 25, }, // Europe_Western
    {  455,  695,  537248,  537268,  537293, 20, 25, 33, }, // Falkland
    {  455,  704,  537326,  537334,  537347,  8, 13, 21, }, // Fiji
    {  455,  709,       0,  537368,       0,  0, 18,  0, }, // French_Guiana
    {  455,  723,       0,  537386,       0,  0, 32,  0, }, // French_Southern
    {  455,  746,       0,  537418,       0,  0, 14,  0, }, // Galapagos
    {  455,  756,       0,  537432,       0,  0, 14,  0, }, // Gambier
    {  455,  764,  537446,  537457,  537473, 11, 16, 24, }, // Georgia
    {  455,  772,       0,  537497,       0,  0, 17,  0, }, // Gilbert_Islands
    {  455,  788,       0,  537514,       0,  0, 16,  0, }, // GMT
    {  455,  830,  537530,  537555,  537585, 25, 30, 38, }, // Greenland_Eastern
    {  455,  848,  537623,  537649,  537680, 26, 31, 39, }, // Greenland_Western
    {  455,  871,       0,  537719,       0,  0, 13,  0, }, // Gulf
    {  455,  876,       0,  336998,       0,  0, 10,  0, }, // Guyana
    {  455,  883,  537732,  537748,  537769, 16, 21, 21, }, // Hawaii_Aleutian
    {  455,  899,  537790,  537798,  537811,  8, 13, 21, }, // Hong_Kong
    {  455,  909,  537832,  537841,  537855,  9, 14, 22, }, // Hovd
    {  455,  914,       0,  337479,       0,  0, 15,  0, }, // India
    {  455,  920,       0,  537877,       0,  0, 17,  0, }, // Indian_Ocean
    {  455,  933,       0,  537894,       0,  0, 14,  0, }, // Indochina
    {  455,  943,       0,  537908,       0,  0, 25,  0, }, // Indonesia_Central
    {  455,  961,       0,  537933,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  455,  979,       0,  537955,       0,  0, 23,  0, }, // Indonesia_Western
    {  455,  997,  537978,  537987,  538001,  9, 14, 14, }, // Iran
    {  455, 1008,  538015,  538029,  538048, 14, 19, 27, }, // Irkutsk
    {  455, 1016,  538075,  538085,  538100, 10, 15, 15, }, // Israel
    {  455, 1023,  337726,  337735,  538115,  9, 14, 14, }, // Japan
    {  455, 1047,       0,  538129,       0,  0, 15,  0, }, // Kazakhstan
    {  455, 1058,       0,  538144,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  455, 1077,       0,  538164,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  455, 1106,  538184,  538196,  538213, 12, 17, 17, }, // Korea
    {  455, 1112,       0,  337970,       0,  0, 10,  0, }, // Kosrae
    {  455, 1119,  538230,  538246,  538267, 16, 21, 29, }, // Krasnoyarsk
    {  455, 1151,       0,  538296,       0,  0, 15,  0, }, // Kyrgystan
    {  455, 1175,       0,  538311,       0,  0, 13,  0, }, // Line_Islands
    {  455, 1188,  538324,  538337,  538355, 13, 18, 18, }, // Lord_Howe
    {  455, 1204,  338143,  338154,  338170, 11, 16, 24, }, // Magadan
    {  455, 1219,       0,  538373,       0,  0, 11,  0, }, // Malaysia
    {  455, 1228,       0,  538384,       0,  0, 13,  0, }, // Maldives
    {  455, 1237,       0,  538397,       0,  0, 12,  0, }, // Marquesas
    {  455, 1247,       0,  538409,       0,  0, 15,  0, }, // Marshall_Islands
    {  455, 1264,  538424,  538434,  538449, 10, 15, 23, }, // Mauritius
    {  455, 1274,       0,  538472,       0,  0,  9,  0, }, // Mawson
    {  455, 1281,  538481,  538503,  538530, 22, 27, 27, }, // Mexico_Pacific
    {  455, 1296,  538557,  538571,  538590, 14, 19, 27, }, // Mongolia
    {  455, 1305,  538617,  538626,  538640,  9, 14, 22, }, // Moscow
    {  455, 1312,       0,  538662,       0,  0, 12,  0, }, // Myanmar
    {  455, 1320,       0,  538674,       0,  0,  9,  0, }, // Nauru
    {  455, 1326,       0,  538683,       0,  0, 10,  0, }, // Nepal
    {  455, 1332,  538693,  538712,  538736, 19, 24, 32, }, // New_Caledonia
    {  455, 1346,  538768,  538785,  538807, 17, 22, 22, }, // New_Zealand
    {  455, 1358,  538829,  538852,  538879, 23, 27, 26, }, // Newfoundland
    {  455, 1371,       0,  538905,       0,  0,  8,  0, }, // Niue
    {  455, 1376,  538913,  538931,  538954, 18, 23, 31, }, // Norfolk
    {  455, 1384,  538985,  539010,  539040, 25, 30, 38, }, // Noronha
    {  455, 1406,  539078,  539095,  539117, 17, 22, 30, }, // Novosibirsk
    {  455, 1418,  338948,  338958,  338973, 10, 15, 23, }, // Omsk
    {  455, 1428,  539147,  539161,  539180, 14, 19, 27, }, // Pakistan
    {  455, 1437,       0,  539207,       0,  0,  9,  0, }, // Palau
    {  455, 1443,       0,  539216,       0,  0, 19,  0, }, // Papua_New_Guinea
    {  455, 1460,  539235,  539247,  539264, 12, 17, 25, }, // Paraguay
    {  455, 1469,  539289,  339142,  539297,  8, 13, 21, }, // Peru
    {  455, 1474,  539318,  539330,  539347, 12, 17, 25, }, // Philippines
    {  455, 1486,       0,  539372,       0,  0, 16,  0, }, // Phoenix_Islands
    {  455, 1502,  539388,  539419,  539455, 31, 36, 36, }, // Pierre_Miquelon
    {  455, 1518,       0,  539491,       0,  0, 11,  0, }, // Pitcairn
    {  455, 1527,       0,  339357,       0,  0, 10,  0, }, // Ponape
    {  455, 1534,       0,  539502,       0,  0, 12,  0, }, // Pyongyang
    {  455, 1554,       0,  539514,       0,  0, 12,  0, }, // Reunion
    {  455, 1562,       0,  339393,       0,  0, 10,  0, }, // Rothera
    {  455, 1570,  539526,  539537,  539553, 11, 16, 24, }, // Sakhalin
    {  455, 1596,  539577,  539586,  539600,  9, 14, 14, }, // Samoa
    {  455, 1602,       0,  539614,       0,  0, 14,  0, }, // Seychelles
    {  455, 1624,       0,  539628,       0,  0, 17,  0, }, // Singapore
    {  455, 1634,       0,  539645,       0,  0, 16,  0, }, // Solomon
    {  455, 1642,       0,  539661,       0,  0, 18,  0, }, // South_Georgia
    {  455, 1656,       0,  539679,       0,  0, 12,  0, }, // Suriname
    {  455, 1676,       0,  339606,       0,  0, 10,  0, }, // Syowa
    {  455, 1682,       0,  339616,       0,  0, 10,  0, }, // Tahiti
    {  455, 1689,  539691,  539701,  539716, 10, 15, 15, }, // Taipei
    {  455, 1696,       0,  539731,       0,  0, 14,  0, }, // Tajikistan
    {  455, 1724,       0,  539745,       0,  0, 11,  0, }, // Tokelau
    {  455, 1732,  339691,  339700,  339714,  9, 14, 22, }, // Tonga
    {  455, 1738,       0,  539756,       0,  0,  8,  0, }, // Truk
    {  455, 1750,  339743,  339761,  539764, 18, 23, 36, }, // Turkmenistan
    {  455, 1763,       0,  539800,       0,  0, 10,  0, }, // Tuvalu
    {  455, 1777,  339825,  539810,  339852, 11, 16, 24, }, // Uruguay
    {  455, 1792,  539826,  539842,  539863, 16, 21, 29, }, // Uzbekistan
    {  455, 1803,  539892,  539903,  539919, 11, 16, 24, }, // Vanuatu
    {  455, 1811,       0,  539943,       0,  0, 15,  0, }, // Venezuela
    {  455, 1821,  539958,  539975,  539997, 17, 22, 30, }, // Vladivostok
    {  455, 1833,  540027,  540042,  540062, 15, 20, 28, }, // Volgograd
    {  455, 1843,       0,  539964,       0,  0, 11,  0, }, // Vostok
    {  455, 1850,       0,  540090,       0,  0, 12,  0, }, // Wake
    {  455, 1855,       0,  540102,       0,  0, 19,  0, }, // Wallis
    {  455, 1862,  540121,  540132,  540148, 11, 16, 24, }, // Yakutsk
    {  455, 1870,  540172,  540189,  540211, 17, 22, 30, }, // Yekaterinburg
    {  455, 1892,       0,  540241,       0,  0, 11,  0, }, // Yukon
    {  456,  914,       0,  337479,       0,  0, 15,  0, }, // India Nepali/Devanagari/India
    {  459,    5,       0,  540252,       0,  0, 15,  0, }, // Afghanistan Nigerian Pidgin/Latin/Nigeria
    {  459,   17,       0,  540267,       0,  0, 17,  0, }, // Africa_Central
    {  459,   32,       0,  540284,       0,  0, 15,  0, }, // Africa_Eastern
    {  459,   65,       0,  540299,       0,  0, 22,  0, }, // Africa_Southern
    {  459,   81,  540321,  540338,  540361, 17, 23, 28, }, // Africa_Western
    {  459,  107,  540389,  540400,  540417, 11, 17, 18, }, // Alaska
    {  459,  135,  540435,  540446,  540463, 11, 17, 22, }, // Amazon
    {  459,  142,  540485,  540514,  540549, 29, 35, 36, }, // America_Central
    {  459,  158,  540585,  540614,  540649, 29, 35, 36, }, // America_Eastern
    {  459,  174,  540685,  540716,  540753, 31, 37, 38, }, // America_Mountain
    {  459,  191,  540791,  540822,  540859, 31, 37, 38, }, // America_Pacific
    {  459,  214,  540897,  540906,  540921,  9, 15, 16, }, // Apia
    {  459,  232,  540937,  540948,  540965, 11, 17, 18, }, // Arabian
    {  459,  240,  540983,  540996,  541015, 13, 19, 24, }, // Argentina
    {  459,  250,  541039,  541060,  541087, 21, 27, 32, }, // Argentina_Western
    {  459,  268,  541119,  541132,  541151, 13, 19, 24, }, // Armenia
    {  459,  286,  541175,  541188,  541207, 13, 19, 20, }, // Atlantic
    {  459,  295,  541227,  541246,  541271, 19, 25, 26, }, // Australia_Central
    {  459,  313,  541297,  541324,  541357, 27, 33, 34, }, // Australia_CentralWestern
    {  459,  338,  541391,  541410,  541435, 19, 25, 26, }, // Australia_Eastern
    {  459,  356,  541461,  541482,  541509, 21, 27, 28, }, // Australia_Western
    {  459,  374,  541537,  541552,  541573, 15, 21, 26, }, // Azerbaijan
    {  459,  385,  541599,  541608,  541623,  9, 15, 20, }, // Azores
    {  459,  397,  541643,  541658,  541679, 15, 21, 22, }, // Bangladesh
    {  459,  415,       0,  541701,       0,  0, 10,  0, }, // Bhutan
    {  459,  422,       0,  541711,       0,  0, 18,  0, }, // Bolivia
    {  459,  437,  541729,  541742,  541761, 13, 19, 24, }, // Brasilia
    {  459,  454,       0,  541785,       0,  0, 21,  0, }, // Brunei
    {  459,  461,  541806,  541819,  541838, 13, 19, 24, }, // Cape_Verde
    {  459,  478,       0,  541862,       0,  0, 18,  0, }, // Chamorro
    {  459,  487,  541880,  541891,  541908, 11, 17, 18, }, // Chatham
    {  459,  495,  541926,  541936,  541952, 10, 16, 21, }, // Chile
    {  459,  501,  541973,  541984,  542001, 11, 17, 18, }, // China
    {  459,  507,       0,  542019,       0,  0, 19,  0, }, // Christmas
    {  459,  517,       0,  542038,       0,  0, 18,  0, }, // Cocos
    {  459,  523,  542056,  542069,  542088, 13, 19, 24, }, // Colombia
    {  459,  532,  542112,  542128,  542150, 16, 22, 31, }, // Cook
    {  459,  537,  542181,  542190,  542205,  9, 15, 16, }, // Cuba
    {  459,  548,       0,  542221,       0,  0, 10,  0, }, // Davis
    {  459,  564,       0,  542231,       0,  0, 19,  0, }, // DumontDUrville
    {  459,  601,       0,  542250,       0,  0, 13,  0, }, // East_Timor
    {  459,  612,  542263,  542272,  542287,  9, 15, 20, }, // Easter
    {  459,  619,       0,  542307,       0,  0, 13,  0, }, // Ecuador
    {  459,  627,  542320,  542336,  542358, 16, 22, 27, }, // Europe_Central
    {  459,  642,  542385,  542401,  542423, 16, 22, 27, }, // Europe_Eastern
    {  459,  657,       0,  542450,       0,  0, 20,  0, }, // Europe_Further_Eastern
    {  459,  680,  542470,  542488,  542512, 18, 24, 29, }, // Europe_Western
    {  459,  695,  542541,  542555,  542575, 14, 20, 25, }, // Falkland
    {  459,  704,  542600,  542609,  542624,  9, 15, 20, }, // Fiji
    {  459,  709,       0,  542644,       0,  0, 18,  0, }, // French_Guiana
    {  459,  723,       0,  542662,       0,  0, 30,  0, }, // French_Southern
    {  459,  746,       0,  542692,       0,  0, 14,  0, }, // Galapagos
    {  459,  756,       0,  542706,       0,  0, 12,  0, }, // Gambier
    {  459,  764,  542718,  542729,  542746, 11, 17, 22, }, // Georgia
    {  459,  772,       0,  542768,       0,  0, 19,  0, }, // Gilbert_Islands
    {  459,  788,       0,  542787,       0,  0, 17,  0, }, // GMT
    {  459,  830,  542804,  542822,  542846, 18, 24, 29, }, // Greenland_Eastern
    {  459,  848,  542875,  542894,  542919, 19, 25, 30, }, // Greenland_Western
    {  459,  871,       0,  542949,       0,  0, 16,  0, }, // Gulf
    {  459,  876,       0,  542965,       0,  0, 11,  0, }, // Guyana
    {  459,  883,  542976,  542996,  543022, 20, 26, 27, }, // Hawaii_Aleutian
    {  459,  899,  543049,  543063,  543083, 14, 20, 25, }, // Hong_Kong
    {  459,  909,  543108,  543117,  543132,  9, 15, 20, }, // Hovd
    {  459,  914,       0,  543152,       0,  0, 16,  0, }, // India
    {  459,  920,       0,  543168,       0,  0, 18,  0, }, // Indian_Ocean
    {  459,  933,       0,  543186,       0,  0, 15,  0, }, // Indochina
    {  459,  943,       0,  543201,       0,  0, 22,  0, }, // Indonesia_Central
    {  459,  961,       0,  543223,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  459,  979,       0,  543245,       0,  0, 24,  0, }, // Indonesia_Western
    {  459,  997,  543269,  543278,  543293,  9, 15, 16, }, // Iran
    {  459, 1008,  543309,  543321,  543339, 12, 18, 23, }, // Irkutsk
    {  459, 1016,  543362,  543372,  543388, 10, 16, 17, }, // Israel
    {  459, 1023,  543405,  543415,  543431, 10, 16, 17, }, // Japan
    {  459, 1047,       0,  543448,       0,  0, 14,  0, }, // Kazakhstan
    {  459, 1058,       0,  543462,       0,  0, 18,  0, }, // Kazakhstan_Eastern
    {  459, 1077,       0,  543480,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  459, 1106,  543500,  543510,  543526, 10, 16, 17, }, // Korea
    {  459, 1112,       0,  543543,       0,  0, 12,  0, }, // Kosrae
    {  459, 1119,  543555,  543570,  543591, 15, 21, 26, }, // Krasnoyarsk
    {  459, 1151,       0,  543617,       0,  0, 13,  0, }, // Kyrgystan
    {  459, 1175,       0,  543630,       0,  0, 17,  0, }, // Line_Islands
    {  459, 1188,  543647,  543659,  543677, 12, 18, 19, }, // Lord_Howe
    {  459, 1204,  543696,  543708,  543726, 12, 18, 23, }, // Magadan
    {  459, 1219,       0,  543749,       0,  0, 13,  0, }, // Malaysia
    {  459, 1228,       0,  543762,       0,  0, 12,  0, }, // Maldives
    {  459, 1237,       0,  543774,       0,  0, 14,  0, }, // Marquesas
    {  459, 1247,       0,  543788,       0,  0, 19,  0, }, // Marshall_Islands
    {  459, 1264,  543807,  543820,  543839, 13, 19, 24, }, // Mauritius
    {  459, 1274,       0,  543863,       0,  0, 11,  0, }, // Mawson
    {  459, 1281,  543874,  543895,  543922, 21, 27, 28, }, // Mexico_Pacific
    {  459, 1296,  543950,  543963,  543982, 13, 19, 24, }, // Mongolia
    {  459, 1305,  544006,  544017,  544034, 11, 17, 22, }, // Moscow
    {  459, 1312,       0,  544056,       0,  0, 11,  0, }, // Myanmar
    {  459, 1320,       0,  544067,       0,  0, 10,  0, }, // Nauru
    {  459, 1326,       0,  544077,       0,  0, 10,  0, }, // Nepal
    {  459, 1332,  544087,  544105,  544129, 18, 24, 29, }, // New_Caledonia
    {  459, 1346,  544158,  544173,  544194, 15, 21, 22, }, // New_Zealand
    {  459, 1358,  544216,  544233,  544256, 17, 23, 24, }, // Newfoundland
    {  459, 1371,       0,  544280,       0,  0,  9,  0, }, // Niue
    {  459, 1376,  544289,  544310,  544337, 21, 27, 28, }, // Norfolk
    {  459, 1384,  544365,  544389,  544419, 24, 30, 35, }, // Noronha
    {  459, 1406,  544454,  544469,  544490, 15, 21, 26, }, // Novosibirsk
    {  459, 1418,  544516,  544525,  544540,  9, 15, 20, }, // Omsk
    {  459, 1428,  544560,  544573,  544592, 13, 19, 24, }, // Pakistan
    {  459, 1437,       0,  544616,       0,  0, 10,  0, }, // Palau
    {  459, 1443,       0,  544626,       0,  0, 19,  0, }, // Papua_New_Guinea
    {  459, 1460,  544645,  544658,  544677, 13, 19, 24, }, // Paraguay
    {  459, 1469,  544701,  544710,  544725,  9, 15, 20, }, // Peru
    {  459, 1474,  544745,  544757,  544775, 12, 18, 22, }, // Philippines
    {  459, 1486,       0,  544797,       0,  0, 19,  0, }, // Phoenix_Islands
    {  459, 1502,  544816,  544840,  544871, 24, 31, 32, }, // Pierre_Miquelon
    {  459, 1518,       0,  544903,       0,  0, 11,  0, }, // Pitcairn
    {  459, 1527,       0,  544914,       0,  0, 11,  0, }, // Ponape
    {  459, 1534,       0,  544925,       0,  0, 14,  0, }, // Pyongyang
    {  459, 1554,       0,  544939,       0,  0, 13,  0, }, // Reunion
    {  459, 1562,       0,  544952,       0,  0, 12,  0, }, // Rothera
    {  459, 1570,  544964,  544977,  544996, 13, 19, 24, }, // Sakhalin
    {  459, 1596,  545020,  545030,  545046, 10, 16, 17, }, // Samoa
    {  459, 1602,       0,  545063,       0,  0, 13,  0, }, // Seychelles
    {  459, 1624,       0,  545076,       0,  0, 12,  0, }, // Singapore
    {  459, 1634,       0,  545088,       0,  0, 21,  0, }, // Solomon
    {  459, 1642,       0,  545109,       0,  0, 16,  0, }, // South_Georgia
    {  459, 1656,       0,  545125,       0,  0, 12,  0, }, // Suriname
    {  459, 1676,       0,  545137,       0,  0, 10,  0, }, // Syowa
    {  459, 1682,       0,  545147,       0,  0, 11,  0, }, // Tahiti
    {  459, 1689,  545158,  545169,  545186, 11, 17, 18, }, // Taipei
    {  459, 1696,       0,  545204,       0,  0, 15,  0, }, // Tajikistan
    {  459, 1724,       0,  545219,       0,  0, 12,  0, }, // Tokelau
    {  459, 1732,  545231,  545242,  545259, 11, 17, 22, }, // Tonga
    {  459, 1738,       0,  545281,       0,  0,  9,  0, }, // Truk
    {  459, 1750,  545290,  545306,  545328, 16, 22, 27, }, // Turkmenistan
    {  459, 1763,       0,  545355,       0,  0, 11,  0, }, // Tuvalu
    {  459, 1777,  545366,  545379,  545398, 13, 19, 24, }, // Uruguay
    {  459, 1792,  545422,  545437,  545458, 15, 21, 26, }, // Uzbekistan
    {  459, 1803,  545484,  545496,  545514, 12, 18, 18, }, // Vanuatu
    {  459, 1811,       0,  545532,       0,  0, 15,  0, }, // Venezuela
    {  459, 1821,  545547,  545563,  545585, 16, 22, 27, }, // Vladivostok
    {  459, 1833,  545612,  545626,  545646, 14, 20, 25, }, // Volgograd
    {  459, 1843,       0,  545671,       0,  0, 12,  0, }, // Vostok
    {  459, 1850,       0,  545683,       0,  0, 15,  0, }, // Wake
    {  459, 1855,       0,  545698,       0,  0, 20,  0, }, // Wallis
    {  459, 1862,  545718,  545730,  545748, 12, 18, 23, }, // Yakutsk
    {  459, 1870,  545771,  545788,  545811, 17, 23, 28, }, // Yekaterinburg
    {  459, 1892,       0,  545839,       0,  0, 10,  0, }, // Yukon
    {  460,    5,       0,  545849,       0,  0, 23,  0, }, // Afghanistan Nko/Nko/Guinea
    {  460,   17,       0,  545872,       0,  0, 34,  0, }, // Africa_Central
    {  460,   32,       0,  545906,       0,  0, 33,  0, }, // Africa_Eastern
    {  460,   65,       0,  545939,       0,  0, 35,  0, }, // Africa_Southern
    {  460,   81,  545974,  546002,  546036, 28, 34, 34, }, // Africa_Western
    {  460,  107,  546070,  546087,  546114, 17, 27, 22, }, // Alaska
    {  460,  135,  546136,  546153,  546179, 17, 26, 22, }, // Amazon
    {  460,  142,  546201,  546225,  546267, 24, 42, 29, }, // America_Central
    {  460,  158,  546296,  546328,  546369, 32, 41, 29, }, // America_Eastern
    {  460,  174,  546398,  546417,  546446, 19, 29, 24, }, // America_Mountain
    {  460,  191,  546470,  546509,  546558, 39, 49, 44, }, // America_Pacific
    {  460,  214,  546602,  546617,  546642, 15, 25, 33, }, // Apia
    {  460,  232,  546675,  546692,  546719, 17, 27, 33, }, // Arabian
    {  460,  240,  546752,  546772,  546802, 20, 30, 25, }, // Argentina
    {  460,  250,  546827,  546854,  546879, 27, 25, 32, }, // Argentina_Western
    {  460,  268,  546911,  546928,  546955, 17, 27, 33, }, // Armenia
    {  460,  286,  546988,  547010,  547042, 22, 32, 27, }, // Atlantic
    {  460,  295,  547069,  547093,  547127, 24, 34, 29, }, // Australia_Central
    {  460,  313,  547156,  547188,  547230, 32, 42, 37, }, // Australia_CentralWestern
    {  460,  338,  547267,  547292,  547327, 25, 35, 30, }, // Australia_Eastern
    {  460,  356,  547357,  547382,  547417, 25, 35, 30, }, // Australia_Western
    {  460,  374,  547447,  547470,  547503, 23, 33, 39, }, // Azerbaijan
    {  460,  385,  547542,  547557,  547582, 15, 25, 20, }, // Azores
    {  460,  397,  547602,  547622,  547652, 20, 30, 25, }, // Bangladesh
    {  460,  415,       0,  547677,       0,  0, 15,  0, }, // Bhutan
    {  460,  422,       0,  547692,       0,  0, 16,  0, }, // Bolivia
    {  460,  437,  547708,  547727,  547756, 19, 29, 24, }, // Brasilia
    {  460,  454,       0,  547780,       0,  0, 28,  0, }, // Brunei
    {  460,  461,  547808,  547832,  547862, 24, 30, 30, }, // Cape_Verde
    {  460,  478,       0,  547892,       0,  0, 27,  0, }, // Chamorro
    {  460,  487,  547919,  547932,  547955, 13, 23, 18, }, // Chatham
    {  460,  495,  547973,  547986,  548009, 13, 23, 18, }, // Chile
    {  460,  501,  548027,  548039,  548061, 12, 22, 20, }, // China
    {  460,  507,       0,  548081,       0,  0, 22,  0, }, // Christmas
    {  460,  517,       0,  548103,       0,  0, 18,  0, }, // Cocos
    {  460,  523,  548121,  548138,  548165, 17, 27, 22, }, // Colombia
    {  460,  532,  548187,  548199,  548221, 12, 22, 30, }, // Cook
    {  460,  537,  548251,  548266,  548291, 15, 25, 20, }, // Cuba
    {  460,  548,       0,  548311,       0,  0, 15,  0, }, // Davis
    {  460,  564,       0,  548326,       0,  0, 23,  0, }, // DumontDUrville
    {  460,  601,       0,  548349,       0,  0, 21,  0, }, // East_Timor
    {  460,  612,  548370,  548390,  548420, 20, 30, 25, }, // Easter
    {  460,  619,       0,  548445,       0,  0, 17,  0, }, // Ecuador
    {  460,  627,  548462,  548483,  548514, 21, 31, 26, }, // Europe_Central
    {  460,  642,  548540,  548562,  548594, 22, 32, 27, }, // Europe_Eastern
    {  460,  657,       0,  548621,       0,  0, 21,  0, }, // Europe_Further_Eastern
    {  460,  680,  548642,  548664,  548696, 22, 32, 27, }, // Europe_Western
    {  460,  695,  548723,  548745,  548777, 22, 32, 27, }, // Falkland
    {  460,  704,  548804,  548817,  548840, 13, 23, 31, }, // Fiji
    {  460,  709,       0,  548871,       0,  0, 27,  0, }, // French_Guiana
    {  460,  723,       0,  548898,       0,  0, 50,  0, }, // French_Southern
    {  460,  746,       0,  548948,       0,  0, 20,  0, }, // Galapagos
    {  460,  756,       0,  548968,       0,  0, 18,  0, }, // Gambier
    {  460,  764,  548986,  549004,  549032, 18, 28, 34, }, // Georgia
    {  460,  772,       0,  549066,       0,  0, 22,  0, }, // Gilbert_Islands
    {  460,  788,       0,  549088,       0,  0, 16,  0, }, // GMT
    {  460,  830,  549104,  549130,  549166, 26, 36, 31, }, // Greenland_Eastern
    {  460,  848,  549197,  549223,  549259, 26, 36, 31, }, // Greenland_Western
    {  460,  871,       0,  549290,       0,  0, 14,  0, }, // Gulf
    {  460,  876,       0,  548882,       0,  0, 16,  0, }, // Guyana
    {  460,  883,  549304,  549331,  549368, 27, 37, 32, }, // Hawaii_Aleutian
    {  460,  899,  549400,  549420,  549450, 20, 30, 28, }, // Hong_Kong
    {  460,  909,  549478,  549492,  549516, 14, 24, 22, }, // Hovd
    {  460,  914,       0,  549538,       0,  0, 25,  0, }, // India
    {  460,  920,       0,  549563,       0,  0, 25,  0, }, // Indian_Ocean
    {  460,  933,       0,  549588,       0,  0, 19,  0, }, // Indochina
    {  460,  943,       0,  549607,       0,  0, 24,  0, }, // Indonesia_Central
    {  460,  961,       0,  549631,       0,  0, 25,  0, }, // Indonesia_Eastern
    {  460,  979,       0,  549656,       0,  0, 25,  0, }, // Indonesia_Western
    {  460,  997,  549681,  549695,  549719, 14, 24, 19, }, // Iran
    {  460, 1008,  549738,  549756,  549784, 18, 28, 23, }, // Irkutsk
    {  460, 1016,  549807,  549828,  549859, 21, 31, 37, }, // Israel
    {  460, 1023,  549896,  549912,  549938, 16, 26, 24, }, // Japan
    {  460, 1047,       0,  549962,       0,  0, 21,  0, }, // Kazakhstan
    {  460, 1058,       0,  549983,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  460, 1077,       0,  550011,       0,  0, 28,  0, }, // Kazakhstan_Western
    {  460, 1106,  550039,  550053,  550077, 14, 24, 22, }, // Korea
    {  460, 1112,       0,  550099,       0,  0, 16,  0, }, // Kosrae
    {  460, 1119,  550115,  550139,  550173, 24, 34, 29, }, // Krasnoyarsk
    {  460, 1151,       0,  550202,       0,  0, 23,  0, }, // Kyrgystan
    {  460, 1175,       0,  550225,       0,  0, 26,  0, }, // Line_Islands
    {  460, 1188,  550251,  550264,  550287, 13, 23, 18, }, // Lord_Howe
    {  460, 1204,  550305,  550322,  550349, 17, 27, 22, }, // Magadan
    {  460, 1219,       0,  550371,       0,  0, 16,  0, }, // Malaysia
    {  460, 1228,       0,  550387,       0,  0, 21,  0, }, // Maldives
    {  460, 1237,       0,  550408,       0,  0, 17,  0, }, // Marquesas
    {  460, 1247,       0,  550425,       0,  0, 20,  0, }, // Marshall_Islands
    {  460, 1264,  550445,  550460,  550481, 15, 21, 21, }, // Mauritius
    {  460, 1274,       0,  550502,       0,  0, 17,  0, }, // Mawson
    {  460, 1281,  550519,  550548,  550587, 29, 39, 34, }, // Mexico_Pacific
    {  460, 1296,  550621,  550638,  550665, 17, 27, 25, }, // Mongolia
    {  460, 1305,  550690,  550706,  550732, 16, 26, 21, }, // Moscow
    {  460, 1312,       0,  550753,       0,  0, 18,  0, }, // Myanmar
    {  460, 1320,       0,  550771,       0,  0, 15,  0, }, // Nauru
    {  460, 1326,       0,  550786,       0,  0, 14,  0, }, // Nepal
    {  460, 1332,  550800,  550824,  550858, 24, 34, 29, }, // New_Caledonia
    {  460, 1346,  550887,  550908,  550939, 21, 31, 26, }, // New_Zealand
    {  460, 1358,  550965,  550989,  551023, 24, 34, 29, }, // Newfoundland
    {  460, 1371,       0,  551052,       0,  0, 14,  0, }, // Niue
    {  460, 1376,  551066,  551087,  551118, 21, 31, 26, }, // Norfolk
    {  460, 1384,  551144,  551176,  551218, 32, 42, 37, }, // Noronha
    {  460, 1406,  551255,  551275,  551305, 20, 30, 25, }, // Novosibirsk
    {  460, 1418,  551330,  551344,  551368, 14, 24, 19, }, // Omsk
    {  460, 1428,  551387,  551406,  551435, 19, 29, 24, }, // Pakistan
    {  460, 1437,       0,  551459,       0,  0, 14,  0, }, // Palau
    {  460, 1443,       0,  551473,       0,  0, 29,  0, }, // Papua_New_Guinea
    {  460, 1460,  551502,  551520,  551548, 18, 28, 23, }, // Paraguay
    {  460, 1469,  551571,  551585,  551609, 14, 24, 19, }, // Peru
    {  460, 1474,  551628,  551644,  551670, 16, 26, 21, }, // Philippines
    {  460, 1486,       0,  551691,       0,  0, 23,  0, }, // Phoenix_Islands
    {  460, 1502,  551714,  551743,  551782, 29, 39, 34, }, // Pierre_Miquelon
    {  460, 1518,       0,  551816,       0,  0, 17,  0, }, // Pitcairn
    {  460, 1527,       0,  551833,       0,  0, 15,  0, }, // Ponape
    {  460, 1534,       0,  551848,       0,  0, 21,  0, }, // Pyongyang
    {  460, 1554,       0,  551869,       0,  0, 18,  0, }, // Reunion
    {  460, 1562,       0,  551887,       0,  0, 16,  0, }, // Rothera
    {  460, 1570,  551903,  551921,  551949, 18, 28, 23, }, // Sakhalin
    {  460, 1596,  551972,  551988,  552014, 16, 26, 21, }, // Samoa
    {  460, 1602,       0,  552035,       0,  0, 17,  0, }, // Seychelles
    {  460, 1624,       0,  552052,       0,  0, 28,  0, }, // Singapore
    {  460, 1634,       0,  552080,       0,  0, 21,  0, }, // Solomon
    {  460, 1642,       0,  552101,       0,  0, 28,  0, }, // South_Georgia
    {  460, 1656,       0,  552129,       0,  0, 16,  0, }, // Suriname
    {  460, 1676,       0,  552145,       0,  0, 15,  0, }, // Syowa
    {  460, 1682,       0,  552160,       0,  0, 15,  0, }, // Tahiti
    {  460, 1689,  552175,  552191,  552217, 16, 26, 24, }, // Taipei
    {  460, 1696,       0,  552241,       0,  0, 20,  0, }, // Tajikistan
    {  460, 1724,       0,  552261,       0,  0, 16,  0, }, // Tokelau
    {  460, 1732,  552277,  552293,  552319, 16, 26, 21, }, // Tonga
    {  460, 1738,       0,  552340,       0,  0, 12,  0, }, // Truk
    {  460, 1750,  552352,  552376,  552410, 24, 34, 29, }, // Turkmenistan
    {  460, 1763,       0,  552439,       0,  0, 17,  0, }, // Tuvalu
    {  460, 1777,  552456,  552475,  552504, 19, 29, 24, }, // Uruguay
    {  460, 1792,  552528,  552551,  552584, 23, 33, 28, }, // Uzbekistan
    {  460, 1803,  552612,  552631,  552660, 19, 29, 24, }, // Vanuatu
    {  460, 1811,       0,  552684,       0,  0, 21,  0, }, // Venezuela
    {  460, 1821,  552705,  552728,  552761, 23, 33, 28, }, // Vladivostok
    {  460, 1833,  552789,  552811,  552843, 22, 32, 27, }, // Volgograd
    {  460, 1843,       0,  552870,       0,  0, 17,  0, }, // Vostok
    {  460, 1850,       0,  552887,       0,  0, 16,  0, }, // Wake
    {  460, 1855,       0,  552903,       0,  0, 25,  0, }, // Wallis
    {  460, 1862,  552928,  552947,  552976, 19, 29, 24, }, // Yakutsk
    {  460, 1870,  553000,  553023,  553056, 23, 33, 28, }, // Yekaterinburg
    {  460, 1892,       0,  553084,       0,  0, 25,  0, }, // Yukon
    {  461,  142,  553109,  553122,  553147, 13, 25, 21, }, // America_Central Northern Luri/Arabic/Iran
    {  463,  627,  553168,  553186,  553211, 18, 25, 24, }, // Europe_Central Northern Sami/Latin/Norway
    {  463,  642,  553235,  553254,  553280, 19, 26, 25, }, // Europe_Eastern
    {  463,  680,  553305,  553323,  553348, 18, 25, 24, }, // Europe_Western
    {  463,  788,       0,  553372,       0,  0, 21,  0, }, // GMT
    {  463, 1305,  553393,  553404,  553422, 11, 18, 17, }, // Moscow
    {  464,    5,       0,  553439,       0,  0, 17,  0, }, // Afghanistan Northern Sami/Latin/Finland
    {  464,   17,       0,  553456,       0,  0, 18,  0, }, // Africa_Central
    {  464,   32,       0,  553474,       0,  0, 19,  0, }, // Africa_Eastern
    {  464,   65,       0,  553493,       0,  0, 23,  0, }, // Africa_Southern
    {  464,   81,  553516,  553534,  553557, 18, 23, 24, }, // Africa_Western
    {  464,  107,  553581,  553592,  553608, 11, 16, 17, }, // Alaska
    {  464,  135,  553625,  553637,  553654, 12, 17, 18, }, // Amazon
    {  464,  142,  553200,  553672,  553689, 11, 17, 18, }, // America_Central
    {  464,  158,  553707,  553719,  553736, 12, 17, 18, }, // America_Eastern
    {  464,  174,  553754,  553765,  553781, 11, 16, 17, }, // America_Mountain
    {  464,  191,  553798,  553812,  553831, 14, 19, 20, }, // America_Pacific
    {  464,  214,  553851,  553860,  553874,  9, 14, 15, }, // Apia
    {  464,  232,  553889,  553900,  553916, 11, 16, 17, }, // Arabian
    {  464,  240,  553933,  553947,  553966, 14, 19, 20, }, // Argentina
    {  464,  250,  553986,  554006,  554031, 20, 25, 26, }, // Argentina_Western
    {  464,  268,  554057,  554069,  554086, 12, 17, 18, }, // Armenia
    {  464,  286,  554104,  554119,  554139, 15, 20, 21, }, // Atlantic
    {  464,  295,  554160,  554180,  554205, 20, 25, 26, }, // Australia_Central
    {  464,  313,  554231,  554263,  554300, 32, 37, 38, }, // Australia_CentralWestern
    {  464,  338,  554338,  554359,  554385, 21, 26, 27, }, // Australia_Eastern
    {  464,  356,  554412,  554432,  554457, 20, 25, 26, }, // Australia_Western
    {  464,  374,  554483,  554499,  554520, 16, 21, 22, }, // Azerbaijan
    {  464,  385,  554542,  554554,  554571, 12, 17, 18, }, // Azores
    {  464,  397,  554589,  554605,  554626, 16, 21, 22, }, // Bangladesh
    {  464,  415,       0,  554648,       0,  0, 12,  0, }, // Bhutan
    {  464,  422,       0,  554660,       0,  0, 12,  0, }, // Bolivia
    {  464,  437,  554672,  554685,  554703, 13, 18, 19, }, // Brasilia
    {  464,  454,       0,  554722,       0,  0, 23,  0, }, // Brunei
    {  464,  461,  554745,  554759,  554778, 14, 19, 20, }, // Cape_Verde
    {  464,  478,       0,  554798,       0,  0, 17,  0, }, // Chamorro
    {  464,  487,  554815,  554828,  554846, 13, 18, 19, }, // Chatham
    {  464,  495,  554865,  554875,  554890, 10, 15, 16, }, // Chile
    {  464,  501,  554906,  554917,  554933, 11, 16, 17, }, // China
    {  464,  507,       0,  554950,       0,  0, 16,  0, }, // Christmas
    {  464,  517,       0,  554966,       0,  0, 17,  0, }, // Cocos
    {  464,  523,  554983,  554996,  555014, 13, 18, 19, }, // Colombia
    {  464,  532,  555033,  555049,  555070, 16, 21, 34, }, // Cook
    {  464,  537,  555104,  555113,  555127,  9, 14, 15, }, // Cuba
    {  464,  548,       0,  555142,       0,  0, 11,  0, }, // Davis
    {  464,  564,       0,  555153,       0,  0, 21,  0, }, // DumontDUrville
    {  464,  601,       0,  555174,       0,  0, 18,  0, }, // East_Timor
    {  464,  612,  555192,  555209,  555231, 17, 22, 23, }, // Easter
    {  464,  619,       0,  555254,       0,  0, 13,  0, }, // Ecuador
    {  464,  627,  555267,  555285,  555308, 18, 23, 24, }, // Europe_Central
    {  464,  642,  555332,  555351,  555375, 19, 24, 25, }, // Europe_Eastern
    {  464,  657,       0,  555400,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  464,  680,  555431,  555449,  555472, 18, 23, 24, }, // Europe_Western
    {  464,  695,  555496,  555516,  555541, 20, 25, 26, }, // Falkland
    {  464,  704,  555567,  555576,  555590,  9, 14, 15, }, // Fiji
    {  464,  709,       0,  555605,       0,  0, 23,  0, }, // French_Guiana
    {  464,  723,       0,  555628,       0,  0, 35,  0, }, // French_Southern
    {  464,  746,       0,  555663,       0,  0, 15,  0, }, // Galapagos
    {  464,  756,       0,  555678,       0,  0, 13,  0, }, // Gambier
    {  464,  764,  555691,  555703,  555720, 12, 17, 18, }, // Georgia
    {  464,  772,       0,  555738,       0,  0, 19,  0, }, // Gilbert_Islands
    {  464,  788,       0,  555757,       0,  0, 15,  0, }, // GMT
    {  464,  830,  555772,  555796,  555825, 24, 29, 30, }, // Greenland_Eastern
    {  464,  848,  555855,  555878,  555906, 23, 28, 29, }, // Greenland_Western
    {  464,  871,       0,  555935,       0,  0, 15,  0, }, // Gulf
    {  464,  876,       0,  555617,       0,  0, 11,  0, }, // Guyana
    {  464,  883,  555950,  555972,  555999, 22, 27, 28, }, // Hawaii_Aleutian
    {  464,  899,  556027,  556042,  556062, 15, 20, 21, }, // Hong_Kong
    {  464,  909,  556083,  556093,  556108, 10, 15, 16, }, // Hovd
    {  464,  914,       0,  556124,       0,  0, 15,  0, }, // India
    {  464,  920,       0,  556139,       0,  0, 13,  0, }, // Indian_Ocean
    {  464,  933,       0,  556152,       0,  0, 15,  0, }, // Indochina
    {  464,  943,       0,  556167,       0,  0, 20,  0, }, // Indonesia_Central
    {  464,  961,       0,  556187,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  464,  979,       0,  556208,       0,  0, 20,  0, }, // Indonesia_Western
    {  464,  997,  556228,  556238,  556253, 10, 15, 16, }, // Iran
    {  464, 1008,  556269,  556281,  556298, 12, 17, 18, }, // Irkutsk
    {  464, 1016,  556316,  556328,  556345, 12, 17, 18, }, // Israel
    {  464, 1023,  556363,  556374,  556390, 11, 16, 17, }, // Japan
    {  464, 1058,       0,  556407,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {  464, 1077,       0,  556429,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  464, 1106,  556450,  556460,  556475, 10, 15, 16, }, // Korea
    {  464, 1112,       0,  556491,       0,  0, 12,  0, }, // Kosrae
    {  464, 1119,  556503,  556520,  556542, 17, 22, 23, }, // Krasnoyarsk
    {  464, 1151,       0,  556565,       0,  0, 13,  0, }, // Kyrgystan
    {  464, 1175,       0,  556578,       0,  0, 16,  0, }, // Line_Islands
    {  464, 1188,  556594,  556608,  556627, 14, 19, 20, }, // Lord_Howe
    {  464, 1204,  556647,  556660,  556678, 13, 18, 19, }, // Magadan
    {  464, 1219,       0,  556697,       0,  0, 12,  0, }, // Malaysia
    {  464, 1228,       0,  556709,       0,  0, 17,  0, }, // Maldives
    {  464, 1237,       0,  556726,       0,  0, 17,  0, }, // Marquesas
    {  464, 1247,       0,  556743,       0,  0, 20,  0, }, // Marshall_Islands
    {  464, 1264,  556763,  556778,  556798, 15, 20, 21, }, // Mauritius
    {  464, 1274,       0,  556819,       0,  0, 12,  0, }, // Mawson
    {  464, 1281,  556831,  556853,  556880, 22, 27, 28, }, // Mexico_Pacific
    {  464, 1296,  556908,  556924,  556945, 16, 21, 22, }, // Mongolia
    {  464, 1305,  556967,  556978,  556994, 11, 16, 17, }, // Moscow
    {  464, 1312,       0,  557011,       0,  0, 13,  0, }, // Myanmar
    {  464, 1320,       0,  557024,       0,  0, 10,  0, }, // Nauru
    {  464, 1326,       0,  557034,       0,  0, 11,  0, }, // Nepal
    {  464, 1332,  557045,  557064,  557088, 19, 24, 25, }, // New_Caledonia
    {  464, 1346,  557113,  557130,  557152, 17, 22, 23, }, // New_Zealand
    {  464, 1358,  557175,  557193,  557216, 18, 23, 24, }, // Newfoundland
    {  464, 1371,       0,  557240,       0,  0, 10,  0, }, // Niue
    {  464, 1376,       0,  557250,       0,  0, 17,  0, }, // Norfolk
    {  464, 1384,  557267,  557291,  557320, 24, 29, 30, }, // Noronha
    {  464, 1406,  557350,  557367,  557389, 17, 22, 23, }, // Novosibirsk
    {  464, 1418,  557412,  557422,  557437, 10, 15, 16, }, // Omsk
    {  464, 1428,  557453,  557467,  557486, 14, 19, 20, }, // Pakistan
    {  464, 1437,       0,  557506,       0,  0, 11,  0, }, // Palau
    {  464, 1443,       0,  557517,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  464, 1460,  557539,  557553,  557572, 14, 19, 20, }, // Paraguay
    {  464, 1469,  557592,  557601,  557615,  9, 14, 15, }, // Peru
    {  464, 1474,  557630,  557648,  557671, 18, 23, 24, }, // Philippines
    {  464, 1486,       0,  557695,       0,  0, 19,  0, }, // Phoenix_Islands
    {  464, 1502,  557714,  557739,  557769, 25, 30, 31, }, // Pierre_Miquelon
    {  464, 1518,       0,  557800,       0,  0, 20,  0, }, // Pitcairn
    {  464, 1527,       0,  557820,       0,  0, 11,  0, }, // Ponape
    {  464, 1534,       0,  557831,       0,  0, 15,  0, }, // Pyongyang
    {  464, 1554,       0,  557846,       0,  0, 13,  0, }, // Reunion
    {  464, 1562,       0,  557859,       0,  0, 12,  0, }, // Rothera
    {  464, 1570,  557871,  557884,  557902, 13, 18, 19, }, // Sakhalin
    {  464, 1596,  557921,  557931,  557946, 10, 15, 16, }, // Samoa
    {  464, 1602,       0,  557962,       0,  0, 16,  0, }, // Seychelles
    {  464, 1624,       0,  557978,       0,  0, 19,  0, }, // Singapore
    {  464, 1634,       0,  557997,       0,  0, 19,  0, }, // Solomon
    {  464, 1642,       0,  558016,       0,  0, 18,  0, }, // South_Georgia
    {  464, 1656,       0,  558034,       0,  0, 13,  0, }, // Suriname
    {  464, 1676,       0,  558047,       0,  0, 10,  0, }, // Syowa
    {  464, 1682,       0,  558057,       0,  0, 11,  0, }, // Tahiti
    {  464, 1689,  558068,  558080,  558097, 12, 17, 18, }, // Taipei
    {  464, 1696,       0,  558115,       0,  0, 16,  0, }, // Tajikistan
    {  464, 1724,       0,  558131,       0,  0, 13,  0, }, // Tokelau
    {  464, 1732,  558144,  558154,  558169, 10, 15, 16, }, // Tonga
    {  464, 1738,       0,  558185,       0,  0, 11,  0, }, // Truk
    {  464, 1750,  558196,  558214,  558237, 18, 23, 24, }, // Turkmenistan
    {  464, 1763,       0,  558261,       0,  0, 11,  0, }, // Tuvalu
    {  464, 1777,  558272,  558285,  558303, 13, 18, 19, }, // Uruguay
    {  464, 1792,  558322,  558338,  558359, 16, 21, 22, }, // Uzbekistan
    {  464, 1803,  558381,  558393,  558410, 12, 17, 18, }, // Vanuatu
    {  464, 1811,       0,  558428,       0,  0, 14,  0, }, // Venezuela
    {  464, 1821,  558442,  558459,  558481, 17, 22, 23, }, // Vladivostok
    {  464, 1833,  558504,  558519,  558539, 15, 20, 21, }, // Volgograd
    {  464, 1843,       0,  558560,       0,  0, 12,  0, }, // Vostok
    {  464, 1850,       0,  558572,       0,  0, 14,  0, }, // Wake
    {  464, 1855,       0,  558586,       0,  0, 22,  0, }, // Wallis
    {  464, 1862,  558608,  558620,  558637, 12, 17, 18, }, // Yakutsk
    {  464, 1870,  558655,  558674,  558698, 19, 24, 25, }, // Yekaterinburg
    {  466,  788,       0,  558723,       0,  0, 26,  0, }, // GMT Northern Sotho/Latin/South Africa
    {  468,    0,  200519,  558749,  558763,  8, 14, 14, }, // Acre Norwegian Bokmal/Latin/Norway
    {  468,    5,       0,  558777,       0,  0, 12,  0, }, // Afghanistan
    {  468,   17,       0,  558789,       0,  0, 20,  0, }, // Africa_Central
    {  468,   32,       0,  558809,       0,  0, 16,  0, }, // Africa_Eastern
    {  468,   65,       0,  558825,       0,  0, 16,  0, }, // Africa_Southern
    {  468,   81,  558841,  558858,  558881, 17, 23, 23, }, // Africa_Western
    {  468,  107,  558904,  558916,  558934, 12, 18, 18, }, // Alaska
    {  468,  128,  200724,  558952,  558971, 10, 19, 17, }, // Almaty
    {  468,  135,  558988,  559009,  559031, 21, 22, 22, }, // Amazon
    {  468,  142,  559053,  559091,  559130, 38, 39, 39, }, // America_Central
    {  468,  158,  559169,  559211,  559254, 42, 43, 43, }, // America_Eastern
    {  468,  174,  559297,  559331,  559366, 34, 35, 35, }, // America_Mountain
    {  468,  191,  559401,  559450,  559500, 49, 50, 50, }, // America_Pacific
    {  468,  207,  559550,  559570,  559596, 20, 26, 26, }, // Anadyr
    {  468,  214,  559622,  559639,  559657, 17, 18, 18, }, // Apia
    {  468,  219,  201075,  559675,  559693,  9, 18, 16, }, // Aqtau
    {  468,  225,  201114,  559709,  559728, 10, 19, 17, }, // Aqtobe
    {  468,  232,  559745,  559756,  559775, 11, 19, 17, }, // Arabian
    {  468,  240,  201258,  201276,  201300, 14, 20, 20, }, // Argentina
    {  468,  250,  559792,  559810,  559834, 18, 24, 24, }, // Argentina_Western
    {  468,  268,  559858,  559869,  559886, 11, 17, 17, }, // Armenia
    {  468,  286,  559903,  559954,  560006, 51, 52, 52, }, // Atlantic
    {  468,  295,  560058,  560078,  560104, 20, 26, 26, }, // Australia_Central
    {  468,  313,  560130,  560155,  560186, 25, 31, 31, }, // Australia_CentralWestern
    {  468,  338,  560217,  560233,  560255, 16, 22, 22, }, // Australia_Eastern
    {  468,  356,  560277,  560294,  560317, 17, 23, 23, }, // Australia_Western
    {  468,  374,  560340,  560358,  560382, 18, 24, 24, }, // Azerbaijan
    {  468,  385,  560406,  560417,  560434, 11, 17, 17, }, // Azores
    {  468,  397,  560451,  560468,  560491, 17, 23, 23, }, // Bangladesh
    {  468,  415,       0,  560514,       0,  0, 12,  0, }, // Bhutan
    {  468,  422,       0,  560526,       0,  0, 14,  0, }, // Bolivia
    {  468,  437,  560540,  560561,  560583, 21, 22, 22, }, // Brasilia
    {  468,  454,       0,  560605,       0,  0, 30,  0, }, // Brunei
    {  468,  461,  560635,  560650,  560671, 15, 21, 21, }, // Cape_Verde
    {  468,  472,       0,  560692,       0,  0,  9,  0, }, // Casey
    {  468,  478,       0,  560701,       0,  0, 21,  0, }, // Chamorro
    {  468,  487,  560722,  560742,  560763, 20, 21, 21, }, // Chatham
    {  468,  495,  560784,  560796,  560814, 12, 18, 18, }, // Chile
    {  468,  501,  560832,  560844,  560862, 12, 18, 18, }, // China
    {  468,  507,       0,  560880,       0,  0, 25,  0, }, // Christmas
    {  468,  517,       0,  560905,       0,  0, 23,  0, }, // Cocos
    {  468,  523,  560928,  560943,  560964, 15, 21, 21, }, // Colombia
    {  468,  532,  560985,  561007,  561030, 22, 23, 28, }, // Cook
    {  468,  537,  561058,  561069,  561086, 11, 17, 17, }, // Cuba
    {  468,  548,       0,  561103,       0,  0, 18,  0, }, // Davis
    {  468,  564,       0,  561121,       0,  0, 29,  0, }, // DumontDUrville
    {  468,  601,       0,  561150,       0,  0, 17,  0, }, // East_Timor
    {  468,  612,  561167,  561188,  561210, 21, 22, 22, }, // Easter
    {  468,  619,       0,  561232,       0,  0, 16,  0, }, // Ecuador
    {  468,  627,  561248,  561268,  561294, 20, 26, 26, }, // Europe_Central
    {  468,  642,  561320,  561336,  561358, 16, 22, 22, }, // Europe_Eastern
    {  468,  657,       0,  561380,       0,  0, 22,  0, }, // Europe_Further_Eastern
    {  468,  680,  561402,  561419,  561442, 17, 23, 23, }, // Europe_Western
    {  468,  695,  561465,  561492,  561520, 27, 28, 28, }, // Falkland
    {  468,  704,  561548,  561560,  561578, 12, 18, 18, }, // Fiji
    {  468,  709,       0,  561596,       0,  0, 26,  0, }, // French_Guiana
    {  468,  723,       0,  561622,       0,  0, 38,  0, }, // French_Southern
    {  468,  746,       0,  561660,       0,  0, 27,  0, }, // Galapagos
    {  468,  756,       0,  561687,       0,  0, 20,  0, }, // Gambier
    {  468,  764,  561707,  561719,  561737, 12, 18, 18, }, // Georgia
    {  468,  772,       0,  561755,       0,  0, 25,  0, }, // Gilbert_Islands
    {  468,  788,       0,  561780,       0,  0, 19,  0, }, // GMT
    {  468,  830,  561799,  561816,  561839, 17, 23, 23, }, // Greenland_Eastern
    {  468,  848,  561862,  561880,  561904, 18, 24, 24, }, // Greenland_Western
    {  468,  866,       0,  561928,       0,  0,  8,  0, }, // Guam
    {  468,  871,       0,  561936,       0,  0, 24,  0, }, // Gulf
    {  468,  876,       0,  561960,       0,  0, 11,  0, }, // Guyana
    {  468,  883,  561971,  562002,  562034, 31, 32, 32, }, // Hawaii_Aleutian
    {  468,  899,  562066,  562087,  562109, 21, 22, 22, }, // Hong_Kong
    {  468,  909,  562131,  562149,  562168, 18, 19, 19, }, // Hovd
    {  468,  914,       0,  562187,       0,  0, 10,  0, }, // India
    {  468,  920,       0,  562197,       0,  0, 23,  0, }, // Indian_Ocean
    {  468,  933,       0,  562220,       0,  0, 16,  0, }, // Indochina
    {  468,  943,       0,  562236,       0,  0, 21,  0, }, // Indonesia_Central
    {  468,  961,       0,  562257,       0,  0, 17,  0, }, // Indonesia_Eastern
    {  468,  979,       0,  562274,       0,  0, 18,  0, }, // Indonesia_Western
    {  468,  997,  562292,  562302,  562318, 10, 16, 16, }, // Iran
    {  468, 1008,  562334,  562354,  562375, 20, 21, 21, }, // Irkutsk
    {  468, 1016,  562396,  562408,  562426, 12, 18, 18, }, // Israel
    {  468, 1023,  562444,  562455,  562472, 11, 17, 17, }, // Japan
    {  468, 1029,  562489,  562529,  562575, 40, 46, 46, }, // Kamchatka
    {  468, 1047,       0,  203676,       0,  0, 16,  0, }, // Kazakhstan
    {  468, 1058,       0,  562621,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  468, 1077,       0,  562640,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  468, 1106,  562660,  562672,  562690, 12, 18, 18, }, // Korea
    {  468, 1112,       0,  562708,       0,  0, 19,  0, }, // Kosrae
    {  468, 1119,  562727,  562751,  562776, 24, 25, 25, }, // Krasnoyarsk
    {  468, 1151,       0,  562801,       0,  0, 13,  0, }, // Kyrgystan
    {  468, 1161,       0,  562814,       0,  0,  9,  0, }, // Lanka
    {  468, 1175,       0,  562823,       0,  0, 23,  0, }, // Line_Islands
    {  468, 1188,  562846,  562872,  562899, 26, 27, 27, }, // Lord_Howe
    {  468, 1198,  562926,  562935,  562953,  9, 18, 16, }, // Macau
    {  468, 1204,  562969,  562989,  563010, 20, 21, 21, }, // Magadan
    {  468, 1219,       0,  563031,       0,  0, 13,  0, }, // Malaysia
    {  468, 1228,       0,  563044,       0,  0, 13,  0, }, // Maldives
    {  468, 1237,       0,  563057,       0,  0, 27,  0, }, // Marquesas
    {  468, 1247,       0,  563084,       0,  0, 17,  0, }, // Marshall_Islands
    {  468, 1264,  563101,  563114,  563133, 13, 19, 19, }, // Mauritius
    {  468, 1274,       0,  563152,       0,  0, 19,  0, }, // Mawson
    {  468, 1281,  563171,  563216,  563262, 45, 46, 46, }, // Mexico_Pacific
    {  468, 1296,  563308,  563331,  563355, 23, 24, 24, }, // Mongolia
    {  468, 1305,  563379,  563398,  563418, 19, 20, 20, }, // Moscow
    {  468, 1312,       0,  563438,       0,  0, 13,  0, }, // Myanmar
    {  468, 1320,       0,  563451,       0,  0, 11,  0, }, // Nauru
    {  468, 1326,       0,  563462,       0,  0, 11,  0, }, // Nepal
    {  468, 1332,  563473,  563486,  563505, 13, 19, 19, }, // New_Caledonia
    {  468, 1346,  563524,  563540,  563562, 16, 22, 22, }, // New_Zealand
    {  468, 1358,  563584,  563609,  563635, 25, 26, 26, }, // Newfoundland
    {  468, 1371,       0,  563661,       0,  0, 17,  0, }, // Niue
    {  468, 1376,  563678,  563701,  563725, 23, 24, 24, }, // Norfolk
    {  468, 1384,  563749,  563781,  563814, 32, 33, 33, }, // Noronha
    {  468, 1392,       0,  563847,       0,  0, 18,  0, }, // North_Mariana
    {  468, 1406,  563865,  563889,  563914, 24, 25, 25, }, // Novosibirsk
    {  468, 1418,  563939,  563956,  563974, 17, 18, 18, }, // Omsk
    {  468, 1428,  563992,  564006,  564026, 14, 20, 20, }, // Pakistan
    {  468, 1437,       0,  564046,       0,  0, 12,  0, }, // Palau
    {  468, 1443,       0,  564058,       0,  0, 12,  0, }, // Papua_New_Guinea
    {  468, 1460,  564070,  564086,  564108, 16, 22, 22, }, // Paraguay
    {  468, 1469,  564130,  564142,  564160, 12, 18, 18, }, // Peru
    {  468, 1474,  564178,  564192,  564212, 14, 20, 20, }, // Philippines
    {  468, 1486,       0,  564232,       0,  0, 25,  0, }, // Phoenix_Islands
    {  468, 1502,  564257,  564294,  564332, 37, 38, 38, }, // Pierre_Miquelon
    {  468, 1518,       0,  564370,       0,  0, 21,  0, }, // Pitcairn
    {  468, 1527,       0,  564391,       0,  0, 20,  0, }, // Ponape
    {  468, 1534,       0,  564411,       0,  0, 22,  0, }, // Pyongyang
    {  468, 1544,  205173,  564433,  564455, 13, 22, 20, }, // Qyzylorda
    {  468, 1554,       0,  564475,       0,  0, 20,  0, }, // Reunion
    {  468, 1562,       0,  564495,       0,  0, 20,  0, }, // Rothera
    {  468, 1570,  564515,  564536,  564558, 21, 22, 22, }, // Sakhalin
    {  468, 1579,  564580,  564600,  564626, 20, 26, 26, }, // Samara
    {  468, 1596,  564652,  564664,  564682, 12, 18, 18, }, // Samoa
    {  468, 1602,       0,  564700,       0,  0, 15,  0, }, // Seychelles
    {  468, 1624,       0,  564715,       0,  0, 14,  0, }, // Singapore
    {  468, 1634,       0,  564729,       0,  0, 13,  0, }, // Solomon
    {  468, 1642,       0,  564742,       0,  0, 24,  0, }, // South_Georgia
    {  468, 1656,       0,  564766,       0,  0, 13,  0, }, // Suriname
    {  468, 1676,       0,  564779,       0,  0, 18,  0, }, // Syowa
    {  468, 1682,       0,  564797,       0,  0, 12,  0, }, // Tahiti
    {  468, 1689,  564809,  564828,  564848, 19, 20, 20, }, // Taipei
    {  468, 1696,       0,  564868,       0,  0, 14,  0, }, // Tajikistan
    {  468, 1724,       0,  564882,       0,  0, 20,  0, }, // Tokelau
    {  468, 1732,  564902,  564914,  564932, 12, 18, 18, }, // Tonga
    {  468, 1738,       0,  564950,       0,  0, 23,  0, }, // Truk
    {  468, 1750,  564973,  564986,  565005, 13, 19, 19, }, // Turkmenistan
    {  468, 1763,       0,  565024,       0,  0, 11,  0, }, // Tuvalu
    {  468, 1777,  565035,  565050,  565071, 15, 21, 21, }, // Uruguay
    {  468, 1792,  565092,  565104,  565122, 12, 18, 18, }, // Uzbekistan
    {  468, 1803,  565140,  565153,  565172, 13, 19, 19, }, // Vanuatu
    {  468, 1811,       0,  565191,       0,  0, 16,  0, }, // Venezuela
    {  468, 1821,  565207,  565231,  565256, 24, 25, 25, }, // Vladivostok
    {  468, 1833,  565281,  565303,  565326, 22, 23, 23, }, // Volgograd
    {  468, 1843,       0,  565349,       0,  0, 19,  0, }, // Vostok
    {  468, 1850,       0,  565368,       0,  0, 24,  0, }, // Wake
    {  468, 1855,       0,  565392,       0,  0, 35,  0, }, // Wallis
    {  468, 1862,  565427,  565447,  565468, 20, 21, 21, }, // Yakutsk
    {  468, 1870,  565489,  565515,  565542, 26, 27, 27, }, // Yekaterinburg
    {  468, 1892,       0,  565569,       0,  0, 18,  0, }, // Yukon
    {  470,    0,  200519,  558749,  558763,  8, 14, 14, }, // Acre Norwegian Nynorsk/Latin/Norway
    {  470,    5,       0,  558777,       0,  0, 12,  0, }, // Afghanistan
    {  470,   17,       0,  558789,       0,  0, 20,  0, }, // Africa_Central
    {  470,   32,       0,  565587,       0,  0, 17,  0, }, // Africa_Eastern
    {  470,   65,       0,  558825,       0,  0, 16,  0, }, // Africa_Southern
    {  470,   81,  558841,  565604,  565629, 17, 25, 23, }, // Africa_Western
    {  470,  107,  558904,  558916,  565652, 12, 18, 18, }, // Alaska
    {  470,  128,  200724,  558952,  558971, 10, 19, 17, }, // Almaty
    {  470,  135,  558988,  559009,  565670, 21, 22, 22, }, // Amazon
    {  470,  142,  565692,  565726,  565761, 34, 35, 35, }, // America_Central
    {  470,  158,  565796,  565839,  565883, 43, 44, 44, }, // America_Eastern
    {  470,  174,  559297,  559331,  565927, 34, 35, 35, }, // America_Mountain
    {  470,  191,  565962,  566011,  566061, 49, 50, 50, }, // America_Pacific
    {  470,  207,  559550,  559570,  559596, 20, 26, 26, }, // Anadyr
    {  470,  214,  559622,  559639,  566111, 17, 18, 18, }, // Apia
    {  470,  219,  201075,  559675,  559693,  9, 18, 16, }, // Aqtau
    {  470,  225,  201114,  559709,  559728, 10, 19, 17, }, // Aqtobe
    {  470,  232,  559745,  566129,  566146, 11, 17, 17, }, // Arabian
    {  470,  240,  201258,  201276,  566163, 14, 20, 20, }, // Argentina
    {  470,  250,  559792,  559810,  566183, 18, 24, 24, }, // Argentina_Western
    {  470,  268,  559858,  559869,  566207, 11, 17, 17, }, // Armenia
    {  470,  286,  559903,  559954,  566224, 51, 52, 52, }, // Atlantic
    {  470,  295,  560058,  566276,  566304, 20, 28, 26, }, // Australia_Central
    {  470,  313,  560130,  566330,  566363, 25, 33, 31, }, // Australia_CentralWestern
    {  470,  338,  566394,  566411,  566436, 17, 25, 23, }, // Australia_Eastern
    {  470,  356,  560277,  566459,  566484, 17, 25, 23, }, // Australia_Western
    {  470,  374,  560340,  560358,  566507, 18, 24, 24, }, // Azerbaijan
    {  470,  385,  560406,  560417,  566531, 11, 17, 17, }, // Azores
    {  470,  397,  560451,  560468,  566548, 17, 23, 23, }, // Bangladesh
    {  470,  415,       0,  560514,       0,  0, 12,  0, }, // Bhutan
    {  470,  422,       0,  560526,       0,  0, 14,  0, }, // Bolivia
    {  470,  437,  560540,  560561,  566571, 21, 22, 22, }, // Brasilia
    {  470,  454,       0,  560605,       0,  0, 30,  0, }, // Brunei
    {  470,  461,  560635,  560650,  566593, 15, 21, 21, }, // Cape_Verde
    {  470,  472,       0,  560692,       0,  0,  9,  0, }, // Casey
    {  470,  478,       0,  560701,       0,  0, 21,  0, }, // Chamorro
    {  470,  487,  560722,  560742,  566614, 20, 21, 21, }, // Chatham
    {  470,  495,  560784,  560796,  566635, 12, 18, 18, }, // Chile
    {  470,  501,  560832,  560844,  566653, 12, 18, 18, }, // China
    {  470,  507,       0,  560880,       0,  0, 25,  0, }, // Christmas
    {  470,  517,       0,  566671,       0,  0, 23,  0, }, // Cocos
    {  470,  523,  566694,  566709,  566730, 15, 21, 21, }, // Colombia
    {  470,  532,  566751,  566773,  566796, 22, 23, 23, }, // Cook
    {  470,  537,  566819,  566830,  566847, 11, 17, 17, }, // Cuba
    {  470,  548,       0,  561103,       0,  0, 18,  0, }, // Davis
    {  470,  564,       0,  566864,       0,  0, 29,  0, }, // DumontDUrville
    {  470,  601,       0,  566893,       0,  0, 18,  0, }, // East_Timor
    {  470,  612,  561167,  561188,  566911, 21, 22, 22, }, // Easter
    {  470,  619,       0,  561232,       0,  0, 16,  0, }, // Ecuador
    {  470,  627,  561248,  566933,  566961, 20, 28, 26, }, // Europe_Central
    {  470,  642,  566987,  567004,  567029, 17, 25, 23, }, // Europe_Eastern
    {  470,  657,       0,  567052,       0,  0, 23,  0, }, // Europe_Further_Eastern
    {  470,  680,  561402,  567075,  567100, 17, 25, 23, }, // Europe_Western
    {  470,  695,  567123,  567150,  567178, 27, 28, 28, }, // Falkland
    {  470,  704,  561548,  561560,  567206, 12, 18, 18, }, // Fiji
    {  470,  709,       0,  561596,       0,  0, 26,  0, }, // French_Guiana
    {  470,  723,       0,  567224,       0,  0, 38,  0, }, // French_Southern
    {  470,  746,       0,  567262,       0,  0, 27,  0, }, // Galapagos
    {  470,  756,       0,  561687,       0,  0, 20,  0, }, // Gambier
    {  470,  764,  561707,  561719,  567289, 12, 18, 18, }, // Georgia
    {  470,  772,       0,  567307,       0,  0, 25,  0, }, // Gilbert_Islands
    {  470,  788,       0,  561780,       0,  0, 19,  0, }, // GMT
    {  470,  830,  567332,  567350,  567374, 18, 24, 24, }, // Greenland_Eastern
    {  470,  848,  561862,  561880,  567398, 18, 24, 24, }, // Greenland_Western
    {  470,  866,       0,  561928,       0,  0,  8,  0, }, // Guam
    {  470,  871,       0,  561936,       0,  0, 24,  0, }, // Gulf
    {  470,  876,       0,  561960,       0,  0, 11,  0, }, // Guyana
    {  470,  883,  561971,  562002,  567422, 31, 32, 32, }, // Hawaii_Aleutian
    {  470,  899,  567454,  567474,  567500, 20, 26, 26, }, // Hong_Kong
    {  470,  909,  562131,  562149,  567526, 18, 19, 19, }, // Hovd
    {  470,  914,       0,  562187,       0,  0, 10,  0, }, // India
    {  470,  920,       0,  562197,       0,  0, 23,  0, }, // Indian_Ocean
    {  470,  933,       0,  562220,       0,  0, 16,  0, }, // Indochina
    {  470,  943,       0,  562236,       0,  0, 21,  0, }, // Indonesia_Central
    {  470,  961,       0,  567545,       0,  0, 18,  0, }, // Indonesia_Eastern
    {  470,  979,       0,  562274,       0,  0, 18,  0, }, // Indonesia_Western
    {  470,  997,  562292,  562302,  567563, 10, 16, 16, }, // Iran
    {  470, 1008,  562334,  562354,  567579, 20, 21, 21, }, // Irkutsk
    {  470, 1016,  562396,  562408,  567600, 12, 18, 18, }, // Israel
    {  470, 1023,  562444,  562455,  567618, 11, 17, 17, }, // Japan
    {  470, 1029,  562489,  562529,  562575, 40, 46, 46, }, // Kamchatka
    {  470, 1047,       0,  203676,       0,  0, 16,  0, }, // Kazakhstan
    {  470, 1058,       0,  567635,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  470, 1077,       0,  562640,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  470, 1106,  562660,  562672,  567655, 12, 18, 18, }, // Korea
    {  470, 1112,       0,  562708,       0,  0, 19,  0, }, // Kosrae
    {  470, 1119,  562727,  562751,  567673, 24, 25, 25, }, // Krasnoyarsk
    {  470, 1151,       0,  562801,       0,  0, 13,  0, }, // Kyrgystan
    {  470, 1161,       0,  562814,       0,  0,  9,  0, }, // Lanka
    {  470, 1175,       0,  567698,       0,  0, 22,  0, }, // Line_Islands
    {  470, 1188,  562846,  562872,  567720, 26, 27, 27, }, // Lord_Howe
    {  470, 1198,  562926,  562935,  562953,  9, 18, 16, }, // Macau
    {  470, 1204,  562969,  562989,  567747, 20, 21, 21, }, // Magadan
    {  470, 1219,       0,  563031,       0,  0, 13,  0, }, // Malaysia
    {  470, 1228,       0,  563044,       0,  0, 13,  0, }, // Maldives
    {  470, 1237,       0,  567768,       0,  0, 27,  0, }, // Marquesas
    {  470, 1247,       0,  563084,       0,  0, 17,  0, }, // Marshall_Islands
    {  470, 1264,  563101,  563114,  567795, 13, 19, 19, }, // Mauritius
    {  470, 1274,       0,  563152,       0,  0, 19,  0, }, // Mawson
    {  470, 1281,  567814,  567859,  567905, 45, 46, 46, }, // Mexico_Pacific
    {  470, 1296,  563308,  563331,  567951, 23, 24, 24, }, // Mongolia
    {  470, 1305,  563379,  563398,  567975, 19, 20, 20, }, // Moscow
    {  470, 1312,       0,  563438,       0,  0, 13,  0, }, // Myanmar
    {  470, 1320,       0,  563451,       0,  0, 11,  0, }, // Nauru
    {  470, 1326,       0,  563462,       0,  0, 11,  0, }, // Nepal
    {  470, 1332,  563473,  563486,  567995, 13, 19, 19, }, // New_Caledonia
    {  470, 1346,  568014,  568029,  568050, 15, 21, 21, }, // New_Zealand
    {  470, 1358,  563584,  563609,  568071, 25, 26, 26, }, // Newfoundland
    {  470, 1371,       0,  563661,       0,  0, 17,  0, }, // Niue
    {  470, 1376,  563678,  563701,  568097, 23, 24, 24, }, // Norfolk
    {  470, 1384,  563749,  563781,  568121, 32, 33, 33, }, // Noronha
    {  470, 1392,       0,  563847,       0,  0, 18,  0, }, // North_Mariana
    {  470, 1406,  563865,  563889,  568154, 24, 25, 25, }, // Novosibirsk
    {  470, 1418,  563939,  563956,  568179, 17, 18, 18, }, // Omsk
    {  470, 1428,  563992,  564006,  568197, 14, 20, 20, }, // Pakistan
    {  470, 1437,       0,  564046,       0,  0, 12,  0, }, // Palau
    {  470, 1443,       0,  564058,       0,  0, 12,  0, }, // Papua_New_Guinea
    {  470, 1460,  564070,  564086,  568217, 16, 22, 22, }, // Paraguay
    {  470, 1469,  564130,  564142,  568239, 12, 18, 18, }, // Peru
    {  470, 1474,  564178,  564192,  568257, 14, 20, 20, }, // Philippines
    {  470, 1486,       0,  568277,       0,  0, 25,  0, }, // Phoenix_Islands
    {  470, 1502,  564257,  564294,  568302, 37, 38, 38, }, // Pierre_Miquelon
    {  470, 1518,       0,  564370,       0,  0, 21,  0, }, // Pitcairn
    {  470, 1527,       0,  564391,       0,  0, 20,  0, }, // Ponape
    {  470, 1534,       0,  564411,       0,  0, 22,  0, }, // Pyongyang
    {  470, 1544,  205173,  564433,  564455, 13, 22, 20, }, // Qyzylorda
    {  470, 1554,       0,  564475,       0,  0, 20,  0, }, // Reunion
    {  470, 1562,       0,  564495,       0,  0, 20,  0, }, // Rothera
    {  470, 1570,  564515,  564536,  568340, 21, 22, 22, }, // Sakhalin
    {  470, 1579,  564580,  564600,  564626, 20, 26, 26, }, // Samara
    {  470, 1596,  564652,  564664,  568362, 12, 18, 18, }, // Samoa
    {  470, 1602,       0,  564700,       0,  0, 15,  0, }, // Seychelles
    {  470, 1624,       0,  564715,       0,  0, 14,  0, }, // Singapore
    {  470, 1634,       0,  564729,       0,  0, 13,  0, }, // Solomon
    {  470, 1642,       0,  564742,       0,  0, 24,  0, }, // South_Georgia
    {  470, 1656,       0,  564766,       0,  0, 13,  0, }, // Suriname
    {  470, 1676,       0,  564779,       0,  0, 18,  0, }, // Syowa
    {  470, 1682,       0,  564797,       0,  0, 12,  0, }, // Tahiti
    {  470, 1689,  564809,  564828,  568380, 19, 20, 20, }, // Taipei
    {  470, 1696,       0,  564868,       0,  0, 14,  0, }, // Tajikistan
    {  470, 1724,       0,  564882,       0,  0, 20,  0, }, // Tokelau
    {  470, 1732,  564902,  564914,  568400, 12, 18, 18, }, // Tonga
    {  470, 1738,       0,  568418,       0,  0, 23,  0, }, // Truk
    {  470, 1750,  564973,  564986,  568441, 13, 19, 19, }, // Turkmenistan
    {  470, 1763,       0,  565024,       0,  0, 11,  0, }, // Tuvalu
    {  470, 1777,  565035,  565050,  568460, 15, 21, 21, }, // Uruguay
    {  470, 1792,  565092,  565104,  568481, 12, 18, 18, }, // Uzbekistan
    {  470, 1803,  565140,  565153,  568499, 13, 19, 19, }, // Vanuatu
    {  470, 1811,       0,  565191,       0,  0, 16,  0, }, // Venezuela
    {  470, 1821,  565207,  565231,  568518, 24, 25, 25, }, // Vladivostok
    {  470, 1833,  565281,  565303,  568543, 22, 23, 23, }, // Volgograd
    {  470, 1843,       0,  565349,       0,  0, 19,  0, }, // Vostok
    {  470, 1850,       0,  565368,       0,  0, 24,  0, }, // Wake
    {  470, 1855,       0,  568566,       0,  0, 35,  0, }, // Wallis
    {  470, 1862,  565427,  565447,  568601, 20, 21, 21, }, // Yakutsk
    {  470, 1870,  565489,  565515,  568622, 26, 27, 27, }, // Yekaterinburg
    {  470, 1892,       0,  565569,       0,  0, 18,  0, }, // Yukon
    {  474,    5,       0,  568649,       0,  0, 16,  0, }, // Afghanistan Occitan/Latin/France
    {  474,   17,       0,  568665,       0,  0, 21,  0, }, // Africa_Central
    {  474,   32,       0,  568686,       0,  0, 21,  0, }, // Africa_Eastern
    {  474,   65,       0,  568707,       0,  0, 16,  0, }, // Africa_Southern
    {  474,   81,  568723,  568747,  568782, 24, 35, 32, }, // Africa_Western
    {  474,  107,   57184,  568814,  137626, 12, 23, 20, }, // Alaska
    {  474,  135,  568837,  568854,  568882, 17, 28, 22, }, // Amazon
    {  474,  142,  568904,  568916,  568939, 12, 23, 20, }, // America_Central
    {  474,  158,  568959,  568971,  568994, 12, 23, 20, }, // America_Eastern
    {  474,  174,  569014,  569032,  569061, 18, 29, 26, }, // America_Mountain
    {  474,  191,  369257,  569087,  569113, 15, 26, 23, }, // America_Pacific
    {  474,  214,   57865,  569136,  569156, 10, 20, 17, }, // Apia
    {  474,  232,   58027,  569173,  569196, 12, 23, 20, }, // Arabian
    {  474,  240,  569216,  569231,  569257, 15, 26, 23, }, // Argentina
    {  474,  250,  569280,  569305,  569341, 25, 36, 33, }, // Argentina_Western
    {  474,  268,   58249,  569374,  569398, 13, 24, 21, }, // Armenia
    {  474,  286,   42903,  569419,  569447, 17, 28, 25, }, // Atlantic
    {  474,  295,  569472,  569496,  569530, 24, 34, 32, }, // Australia_Central
    {  474,  313,  569562,  569595,  569638, 33, 43, 52, }, // Australia_CentralWestern
    {  474,  338,  138856,  569690,  138916, 24, 34, 32, }, // Australia_Eastern
    {  474,  356,  138949,  569724,  139013, 26, 36, 34, }, // Australia_Western
    {  474,  374,  569760,  569776,  569803, 16, 27, 24, }, // Azerbaijan
    {  474,  385,  569827,  569844,  569872, 17, 28, 25, }, // Azores
    {  474,  397,  569897,  569914,  569942, 17, 28, 25, }, // Bangladesh
    {  474,  415,       0,  569967,       0,  0, 12,  0, }, // Bhutan
    {  474,  422,       0,   59003,       0,  0, 14,  0, }, // Bolivia
    {  474,  437,   59018,  569979,  570005, 15, 26, 23, }, // Brasilia
    {  474,  454,       0,  139368,       0,  0, 13,  0, }, // Brunei
    {  474,  461,  570028,  570043,  570069, 15, 26, 23, }, // Cape_Verde
    {  474,  478,       0,  570092,       0,  0, 25,  0, }, // Chamorro
    {  474,  487,   59219,  570117,  139510, 14, 24, 22, }, // Chatham
    {  474,  495,   59282,  570141,  570164, 12, 23, 20, }, // Chile
    {  474,  501,   59339,  570184,  570207, 12, 23, 20, }, // China
    {  474,  507,       0,  570227,       0,  0, 22,  0, }, // Christmas
    {  474,  517,       0,  570249,       0,  0, 22,  0, }, // Cocos
    {  474,  523,  139691,  570271,  139733, 15, 26, 23, }, // Colombia
    {  474,  532,  570297,  570318,  570349, 21, 31, 35, }, // Cook
    {  474,  537,   59614,  570384,  139868, 11, 22, 19, }, // Cuba
    {  474,  548,       0,   59668,       0,  0, 12,  0, }, // Davis
    {  474,  564,       0,   59681,       0,  0, 23,  0, }, // DumontDUrville
    {  474,  601,       0,  570406,       0,  0, 21,  0, }, // East_Timor
    {  474,  612,  570427,  570451,  570486, 24, 35, 32, }, // Easter
    {  474,  619,       0,  570518,       0,  0, 13,  0, }, // Ecuador
    {  474,  627,  570531,  570552,  570584, 21, 32, 29, }, // Europe_Central
    {  474,  642,  570613,  570634,  570666, 21, 32, 29, }, // Europe_Eastern
    {  474,  657,       0,  570695,       0,  0, 34,  0, }, // Europe_Further_Eastern
    {  474,  680,  570729,  570751,  570784, 22, 33, 30, }, // Europe_Western
    {  474,  695,  570814,  570839,  570875, 25, 36, 33, }, // Falkland
    {  474,  704,  140404,  570908,  140438, 11, 21, 19, }, // Fiji
    {  474,  709,       0,  570929,       0,  0, 26,  0, }, // French_Guiana
    {  474,  723,       0,  570955,       0,  0, 51,  0, }, // French_Southern
    {  474,  746,       0,  571006,       0,  0, 16,  0, }, // Galapagos
    {  474,  756,       0,   60352,       0,  0, 14,  0, }, // Gambier
    {  474,  764,  140548,  571022,  140588, 14, 25, 22, }, // Georgia
    {  474,  772,       0,  140611,       0,  0, 24,  0, }, // Gilbert_Islands
    {  474,  788,       0,  571047,       0,  0, 28,  0, }, // GMT
    {  474,  830,  571075,  571102,  571140, 27, 38, 35, }, // Greenland_Eastern
    {  474,  848,  571175,  571203,  571242, 28, 39, 36, }, // Greenland_Western
    {  474,  871,       0,  571278,       0,  0, 23,  0, }, // Gulf
    {  474,  876,       0,  570929,       0,  0, 17,  0, }, // Guyana
    {  474,  883,  571301,  571324,  571358, 23, 34, 31, }, // Hawaii_Aleutian
    {  474,  899,  141052,  571389,  141096, 16, 27, 24, }, // Hong_Kong
    {  474,  909,   60892,  571416,  571438, 11, 22, 19, }, // Hovd
    {  474,  914,       0,  571457,       0,  0, 25,  0, }, // India
    {  474,  920,       0,  571482,       0,  0, 20,  0, }, // Indian_Ocean
    {  474,  933,       0,   60993,       0,  0, 15,  0, }, // Indochina
    {  474,  943,       0,  571502,       0,  0, 25,  0, }, // Indonesia_Central
    {  474,  961,       0,  571527,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  474,  979,       0,  571553,       0,  0, 28,  0, }, // Indonesia_Western
    {  474,  997,  571581,  571591,  571612, 10, 21, 18, }, // Iran
    {  474, 1008,   61135,  571630,  141390, 13, 24, 21, }, // Irkutsk
    {  474, 1016,  571654,  571666,  571689, 12, 23, 20, }, // Israel
    {  474, 1023,  368739,  571709,  571732, 12, 23, 20, }, // Japan
    {  474, 1058,       0,  571752,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  474, 1077,       0,  571779,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  474, 1106,  571808,  571820,  571843, 12, 23, 20, }, // Korea
    {  474, 1112,       0,   61536,       0,  0, 13,  0, }, // Kosrae
    {  474, 1119,   61550,  571863,  571892, 18, 29, 26, }, // Krasnoyarsk
    {  474, 1151,       0,  571918,       0,  0, 17,  0, }, // Kyrgystan
    {  474, 1175,       0,  571935,       0,  0, 32,  0, }, // Line_Islands
    {  474, 1188,   61680,  571967,  571993, 16, 26, 24, }, // Lord_Howe
    {  474, 1204,  141966,  572017,  142006, 14, 25, 22, }, // Magadan
    {  474, 1219,       0,  142029,       0,  0, 15,  0, }, // Malaysia
    {  474, 1228,       0,  572042,       0,  0, 16,  0, }, // Maldives
    {  474, 1237,       0,  572058,       0,  0, 26,  0, }, // Marquesas
    {  474, 1247,       0,  572084,       0,  0, 25,  0, }, // Marshall_Islands
    {  474, 1264,   61952,  572109,  142116, 14, 25, 22, }, // Mauritius
    {  474, 1274,       0,   62018,       0,  0, 13,  0, }, // Mawson
    {  474, 1281,  572134,  572157,  572191, 23, 34, 31, }, // Mexico_Pacific
    {  474, 1296,  369346,  572222,  572250, 17, 28, 25, }, // Mongolia
    {  474, 1305,  572275,  572288,  572312, 13, 24, 21, }, // Moscow
    {  474, 1312,       0,   62257,       0,  0, 14,  0, }, // Myanmar
    {  474, 1320,       0,   62272,       0,  0, 12,  0, }, // Nauru
    {  474, 1326,       0,  294596,       0,  0, 12,  0, }, // Nepal
    {  474, 1332,  572333,  572354,  572385, 21, 31, 29, }, // New_Caledonia
    {  474, 1346,  572414,  572433,  572462, 19, 29, 27, }, // New_Zealand
    {  474, 1358,  572489,  572506,  572534, 17, 28, 25, }, // Newfoundland
    {  474, 1371,       0,   62545,       0,  0, 11,  0, }, // Niue
    {  474, 1376,  142595,  572559,  142649, 21, 31, 29, }, // Norfolk
    {  474, 1384,  572590,  572613,  572647, 23, 34, 31, }, // Noronha
    {  474, 1406,   62783,  572678,  572707, 18, 29, 26, }, // Novosibirsk
    {  474, 1418,   62858,  572733,  142850, 10, 21, 18, }, // Omsk
    {  474, 1428,  369928,  572754,  572780, 15, 26, 23, }, // Pakistan
    {  474, 1437,       0,  572803,       0,  0, 13,  0, }, // Palau
    {  474, 1443,       0,  572816,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  474, 1460,  370017,  572840,  572866, 15, 26, 23, }, // Paraguay
    {  474, 1469,  572889,  572900,  572922, 11, 22, 19, }, // Peru
    {  474, 1474,   63146,  572941,  572968, 16, 27, 24, }, // Philippines
    {  474, 1486,       0,  572992,       0,  0, 22,  0, }, // Phoenix_Islands
    {  474, 1502,  573014,  573042,  573081, 28, 39, 36, }, // Pierre_Miquelon
    {  474, 1518,       0,   63352,       0,  0, 15,  0, }, // Pitcairn
    {  474, 1527,       0,  295610,       0,  0, 14,  0, }, // Ponape
    {  474, 1534,       0,   63382,       0,  0, 16,  0, }, // Pyongyang
    {  474, 1554,       0,  573117,       0,  0, 17,  0, }, // Reunion
    {  474, 1562,       0,   63483,       0,  0, 14,  0, }, // Rothera
    {  474, 1570,  573134,  573148,  573173, 14, 25, 22, }, // Sakhalin
    {  474, 1596,   63621,  573195,  143448, 12, 22, 20, }, // Samoa
    {  474, 1602,       0,  573217,       0,  0, 17,  0, }, // Seychelles
    {  474, 1624,       0,  573234,       0,  0, 15,  0, }, // Singapore
    {  474, 1634,       0,  573249,       0,  0, 24,  0, }, // Solomon
    {  474, 1642,       0,  143531,       0,  0, 22,  0, }, // South_Georgia
    {  474, 1656,       0,  143554,       0,  0, 14,  0, }, // Suriname
    {  474, 1676,       0,   63787,       0,  0, 12,  0, }, // Syowa
    {  474, 1682,       0,   63800,       0,  0, 13,  0, }, // Tahiti
    {  474, 1689,  143583,  573273,  143621, 13, 24, 21, }, // Taipei
    {  474, 1696,       0,  370623,       0,  0, 17,  0, }, // Tajikistan
    {  474, 1724,       0,   63894,       0,  0, 14,  0, }, // Tokelau
    {  474, 1732,   63909,  573297,  143686, 12, 22, 20, }, // Tonga
    {  474, 1738,       0,   63966,       0,  0, 12,  0, }, // Truk
    {  474, 1750,  370683,  573319,  573349, 19, 30, 27, }, // Turkmenistan
    {  474, 1763,       0,   64060,       0,  0, 13,  0, }, // Tuvalu
    {  474, 1777,  573376,  573392,  573419, 16, 27, 24, }, // Uruguay
    {  474, 1792,  370820,  573443,  573471, 17, 28, 25, }, // Uzbekistan
    {  474, 1803,   64218,  573496,  573520, 14, 24, 22, }, // Vanuatu
    {  474, 1811,       0,  143995,       0,  0, 16,  0, }, // Venezuela
    {  474, 1821,   64298,  573542,  144041, 18, 29, 26, }, // Vladivostok
    {  474, 1833,  144068,  573571,  144112, 16, 27, 24, }, // Volgograd
    {  474, 1843,       0,   64442,       0,  0, 13,  0, }, // Vostok
    {  474, 1850,       0,  573598,       0,  0, 18,  0, }, // Wake
    {  474, 1855,       0,  296615,       0,  0, 22,  0, }, // Wallis
    {  474, 1862,   64500,  573616,  573641, 14, 25, 22, }, // Yakutsk
    {  474, 1870,  573663,  573681,  573710, 18, 29, 26, }, // Yekaterinburg
    {  474, 1892,       0,  144326,       0,  0, 12,  0, }, // Yukon
    {  475,   17,       0,  568665,       0,  0, 20,  0, }, // Africa_Central Occitan/Latin/Spain
    {  475,   32,       0,  573736,       0,  0, 21,  0, }, // Africa_Eastern
    {  475,   81,  573757,  573780,  573813, 23, 33, 31, }, // Africa_Western
    {  475,  107,   57184,  573844,  137626, 12, 22, 20, }, // Alaska
    {  475,  135,  573866,  573882,  573908, 16, 26, 24, }, // Amazon
    {  475,  142,  573932,  573943,  573964, 11, 21, 19, }, // America_Central
    {  475,  158,  573983,  573995,  574017, 12, 22, 20, }, // America_Eastern
    {  475,  174,  574037,  574062,  574097, 25, 35, 33, }, // America_Mountain
    {  475,  191,  574130,  574146,  574172, 16, 26, 24, }, // America_Pacific
    {  475,  232,   58027,  574196,  574218, 12, 22, 20, }, // Arabian
    {  475,  240,  569216,  574238,  569257, 15, 25, 23, }, // Argentina
    {  475,  250,  574263,  574288,  574324, 25, 36, 34, }, // Argentina_Western
    {  475,  268,   58249,  574358,  569398, 13, 23, 21, }, // Armenia
    {  475,  286,  574381,  574397,  574423, 16, 26, 24, }, // Atlantic
    {  475,  295,  574447,  574470,  574503, 23, 33, 31, }, // Australia_Central
    {  475,  313,  574534,  574566,  574609, 32, 43, 40, }, // Australia_CentralWestern
    {  475,  338,  574649,  574673,  574707, 24, 34, 32, }, // Australia_Eastern
    {  475,  356,  574739,  574765,  574801, 26, 36, 44, }, // Australia_Western
    {  475,  374,  569760,  574845,  569803, 16, 26, 24, }, // Azerbaijan
    {  475,  385,  574871,  574885,  574909, 14, 24, 22, }, // Azores
    {  475,  397,  569897,  574931,  139247, 17, 27, 25, }, // Bangladesh
    {  475,  437,   59018,  574958,  570005, 15, 25, 23, }, // Brasilia
    {  475,  461,  570028,  574983,  570069, 15, 25, 23, }, // Cape_Verde
    {  475,  495,   59282,  575008,  570164, 12, 22, 20, }, // Chile
    {  475,  501,   59339,  575030,  570207, 12, 22, 20, }, // China
    {  475,  507,       0,  575052,       0,  0, 22,  0, }, // Christmas
    {  475,  517,       0,  575074,       0,  0, 22,  0, }, // Cocos
    {  475,  523,  139691,  575096,  139733, 15, 25, 23, }, // Colombia
    {  475,  532,  575121,  575139,  575167, 18, 28, 32, }, // Cook
    {  475,  537,   59614,  575199,  139868, 11, 21, 19, }, // Cuba
    {  475,  601,       0,  575220,       0,  0, 21,  0, }, // East_Timor
    {  475,  612,  575241,  575264,  575297, 23, 33, 31, }, // Easter
    {  475,  627,  575328,  575348,  575378, 20, 30, 28, }, // Europe_Central
    {  475,  642,  570613,  575406,  570666, 21, 31, 29, }, // Europe_Eastern
    {  475,  657,       0,  575437,       0,  0, 32,  0, }, // Europe_Further_Eastern
    {  475,  680,  575469,  575492,  575525, 23, 33, 31, }, // Europe_Western
    {  475,  695,  575556,  575578,  575610, 22, 32, 30, }, // Falkland
    {  475,  709,       0,  575640,       0,  0, 25,  0, }, // French_Guiana
    {  475,  723,       0,  575665,       0,  0, 47,  0, }, // French_Southern
    {  475,  746,       0,   60335,       0,  0, 16,  0, }, // Galapagos
    {  475,  764,  140548,  575712,  140588, 14, 24, 22, }, // Georgia
    {  475,  772,       0,  575736,       0,  0, 21,  0, }, // Gilbert_Islands
    {  475,  788,       0,  575757,       0,  0, 30,  0, }, // GMT
    {  475,  830,  575787,  575814,  575851, 27, 37, 35, }, // Greenland_Eastern
    {  475,  848,  575886,  575915,  575954, 29, 39, 37, }, // Greenland_Western
    {  475,  876,       0,  140970,       0,  0, 13,  0, }, // Guyana
    {  475,  883,  571301,  575991,  571358, 23, 33, 31, }, // Hawaii_Aleutian
    {  475,  899,  141052,  576024,  141096, 16, 26, 24, }, // Hong_Kong
    {  475,  909,   60892,  576050,  571438, 11, 21, 19, }, // Hovd
    {  475,  914,       0,  576071,       0,  0, 24,  0, }, // India
    {  475,  920,       0,  576095,       0,  0, 19,  0, }, // Indian_Ocean
    {  475,  943,       0,  576114,       0,  0, 24,  0, }, // Indonesia_Central
    {  475,  961,       0,  576138,       0,  0, 25,  0, }, // Indonesia_Eastern
    {  475,  979,       0,  576163,       0,  0, 27,  0, }, // Indonesia_Western
    {  475,  997,  571581,  576190,  571612, 10, 20, 18, }, // Iran
    {  475, 1008,   61135,  576210,  141390, 13, 23, 21, }, // Irkutsk
    {  475, 1016,  571654,  576233,  571689, 12, 22, 20, }, // Israel
    {  475, 1023,  368739,  576255,  571732, 12, 22, 20, }, // Japan
    {  475, 1058,       0,  576277,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  475, 1077,       0,  576303,       0,  0, 28,  0, }, // Kazakhstan_Western
    {  475, 1106,  571808,  576331,  571843, 12, 22, 20, }, // Korea
    {  475, 1119,   61550,  576353,  571892, 18, 28, 26, }, // Krasnoyarsk
    {  475, 1175,       0,  576381,       0,  0, 29,  0, }, // Line_Islands
    {  475, 1204,  141966,  576410,  142006, 14, 24, 22, }, // Magadan
    {  475, 1237,       0,  576434,       0,  0, 23,  0, }, // Marquesas
    {  475, 1247,       0,  576457,       0,  0, 22,  0, }, // Marshall_Islands
    {  475, 1264,   61952,  576479,  142116, 14, 24, 22, }, // Mauritius
    {  475, 1281,  576503,  576528,  576563, 25, 35, 33, }, // Mexico_Pacific
    {  475, 1296,  369346,  576596,  572250, 17, 27, 25, }, // Mongolia
    {  475, 1305,  572275,  576623,  572312, 13, 23, 21, }, // Moscow
    {  475, 1326,       0,  576646,       0,  0, 14,  0, }, // Nepal
    {  475, 1358,  572489,  576660,  572534, 17, 27, 25, }, // Newfoundland
    {  475, 1376,  576687,  576708,  576739, 21, 31, 29, }, // Norfolk
    {  475, 1384,   62647,  576768,  142716, 26, 36, 34, }, // Noronha
    {  475, 1406,   62783,  576804,  572707, 18, 28, 26, }, // Novosibirsk
    {  475, 1418,   62858,  576832,  142850, 10, 20, 18, }, // Omsk
    {  475, 1428,  576852,  576869,  576896, 17, 27, 25, }, // Pakistan
    {  475, 1460,  370017,  576921,  572866, 15, 25, 23, }, // Paraguay
    {  475, 1469,  572889,  576946,  576967, 11, 21, 18, }, // Peru
    {  475, 1474,   63146,  576985,  572968, 16, 26, 24, }, // Philippines
    {  475, 1486,       0,  577011,       0,  0, 19,  0, }, // Phoenix_Islands
    {  475, 1502,  573014,  577030,  573081, 28, 38, 36, }, // Pierre_Miquelon
    {  475, 1554,       0,  577068,       0,  0, 14,  0, }, // Reunion
    {  475, 1570,  573134,  577082,  573173, 14, 24, 22, }, // Sakhalin
    {  475, 1634,       0,  577106,       0,  0, 21,  0, }, // Solomon
    {  475, 1642,       0,  577127,       0,  0, 23,  0, }, // South_Georgia
    {  475, 1689,  143583,  577150,  143621, 13, 23, 21, }, // Taipei
    {  475, 1750,  370683,  577173,  573349, 19, 29, 27, }, // Turkmenistan
    {  475, 1777,  577202,  577215,  577238, 13, 23, 21, }, // Uruguay
    {  475, 1792,  370820,  577259,  573471, 17, 27, 25, }, // Uzbekistan
    {  475, 1821,   64298,  577286,  144041, 18, 28, 26, }, // Vladivostok
    {  475, 1833,  144068,  577314,  144112, 16, 26, 24, }, // Volgograd
    {  475, 1850,       0,  577340,       0,  0, 18,  0, }, // Wake
    {  475, 1862,   64500,  577358,  573641, 14, 24, 22, }, // Yakutsk
    {  475, 1870,  573663,  577382,  573710, 18, 28, 26, }, // Yekaterinburg
    {  476,    0,  577410,  577417,  577429,  7, 12, 15, }, // Acre Odia/Odia/India
    {  476,    5,       0,  577444,       0,  0, 15,  0, }, // Afghanistan
    {  476,   17,       0,  577459,       0,  0, 16,  0, }, // Africa_Central
    {  476,   32,       0,  577475,       0,  0, 17,  0, }, // Africa_Eastern
    {  476,   65,       0,  577492,       0,  0, 26,  0, }, // Africa_Southern
    {  476,   81,  577518,  577536,  577562, 18, 26, 25, }, // Africa_Western
    {  476,  107,  577587,  577598,  577617, 11, 19, 19, }, // Alaska
    {  476,  128,  577636,  577646,  577661, 10, 15, 18, }, // Almaty
    {  476,  135,  577679,  577689,  577707, 10, 18, 23, }, // Amazon
    {  476,  142,  577730,  577743,  577764, 13, 21, 21, }, // America_Central
    {  476,  158,  577785,  577799,  577821, 14, 22, 22, }, // America_Eastern
    {  476,  174,  577843,  577855,  577875, 12, 20, 20, }, // America_Mountain
    {  476,  191,  577895,  577907,  577927, 12, 20, 20, }, // America_Pacific
    {  476,  207,  577947,  577958,  577974, 11, 16, 19, }, // Anadyr
    {  476,  214,  577993,  578001,  578017,  8, 16, 16, }, // Apia
    {  476,  219,  578033,  578041,  578054,  8, 13, 16, }, // Aqtau
    {  476,  225,  578070,  578080,  578095, 10, 15, 18, }, // Aqtobe
    {  476,  232,  578113,  578122,  578139,  9, 17, 17, }, // Arabian
    {  476,  240,  578156,  578171,  578194, 15, 23, 26, }, // Argentina
    {  476,  250,  578220,  578242,  578272, 22, 30, 33, }, // Argentina_Western
    {  476,  268,  578305,  578317,  578337, 12, 20, 23, }, // Armenia
    {  476,  286,  578360,  578374,  578396, 14, 22, 22, }, // Atlantic
    {  476,  295,  578418,  578437,  578464, 19, 27, 27, }, // Australia_Central
    {  476,  313,  578491,  578517,  578551, 26, 34, 34, }, // Australia_CentralWestern
    {  476,  338,  578585,  578605,  578633, 20, 28, 28, }, // Australia_Eastern
    {  476,  356,  578661,  578682,  578711, 21, 29, 29, }, // Australia_Western
    {  476,  374,  578740,  578754,  578775, 14, 21, 21, }, // Azerbaijan
    {  476,  385,  578796,  578807,  578823, 11, 16, 22, }, // Azores
    {  476,  397,  578845,  578857,  578877, 12, 20, 25, }, // Bangladesh
    {  476,  415,       0,  578902,       0,  0,  9,  0, }, // Bhutan
    {  476,  422,       0,  578911,       0,  0, 10,  0, }, // Bolivia
    {  476,  437,  578921,  578934,  578955, 13, 21, 24, }, // Brasilia
    {  476,  454,       0,  578979,       0,  0, 21,  0, }, // Brunei
    {  476,  461,  579000,  579015,  579038, 15, 23, 28, }, // Cape_Verde
    {  476,  478,       0,  579066,       0,  0, 15,  0, }, // Chamorro
    {  476,  487,  579081,  579092,  579111, 11, 19, 19, }, // Chatham
    {  476,  495,  579130,  579138,  579154,  8, 16, 19, }, // Chile
    {  476,  501,  579173,  579180,  579192,  7, 12, 15, }, // China
    {  476,  507,       0,  579207,       0,  0, 20,  0, }, // Christmas
    {  476,  517,       0,  579227,       0,  0, 22,  0, }, // Cocos
    {  476,  523,  579249,  579260,  579276, 11, 16, 22, }, // Colombia
    {  476,  532,  579298,  579318,  579343, 20, 25, 38, }, // Cook
    {  476,  537,  579381,  579391,  579409, 10, 18, 18, }, // Cuba
    {  476,  548,       0,  579427,       0,  0, 11,  0, }, // Davis
    {  476,  564,       0,  579438,       0,  0, 23,  0, }, // DumontDUrville
    {  476,  601,       0,  579461,       0,  0, 17,  0, }, // East_Timor
    {  476,  612,  579478,  579499,  579528, 21, 29, 32, }, // Easter
    {  476,  619,       0,  579560,       0,  0, 11,  0, }, // Ecuador
    {  476,  627,  579571,  579592,  579616, 21, 24, 32, }, // Europe_Central
    {  476,  642,  579648,  579670,  579700, 22, 30, 33, }, // Europe_Eastern
    {  476,  657,       0,  579733,       0,  0, 32,  0, }, // Europe_Further_Eastern
    {  476,  680,  579765,  579788,  579819, 23, 31, 34, }, // Europe_Western
    {  476,  695,  579853,  579877,  579906, 24, 29, 35, }, // Falkland
    {  476,  704,  579941,  579949,  579962,  8, 13, 21, }, // Fiji
    {  476,  709,       0,  579983,       0,  0, 17,  0, }, // French_Guiana
    {  476,  723,       0,  580000,       0,  0, 37,  0, }, // French_Southern
    {  476,  746,       0,  580037,       0,  0, 14,  0, }, // Galapagos
    {  476,  756,       0,  580051,       0,  0, 13,  0, }, // Gambier
    {  476,  764,  580064,  580074,  580092, 10, 18, 17, }, // Georgia
    {  476,  772,       0,  580109,       0,  0, 24,  0, }, // Gilbert_Islands
    {  476,  788,       0,  580133,       0,  0, 18,  0, }, // GMT
    {  476,  830,  580151,  580174,  580205, 23, 31, 34, }, // Greenland_Eastern
    {  476,  848,  580239,  580263,  580295, 24, 32, 32, }, // Greenland_Western
    {  476,  871,       0,  580327,       0,  0, 16,  0, }, // Gulf
    {  476,  876,       0,  580343,       0,  0,  9,  0, }, // Guyana
    {  476,  883,  580352,  580368,  580392, 16, 24, 24, }, // Hawaii_Aleutian
    {  476,  899,  580416,  580425,  580439,  9, 14, 20, }, // Hong_Kong
    {  476,  909,  580459,  580469,  580484, 10, 15, 21, }, // Hovd
    {  476,  914,       0,  580505,       0,  0, 15,  0, }, // India
    {  476,  920,       0,  580520,       0,  0, 17,  0, }, // Indian_Ocean
    {  476,  933,       0,  580537,       0,  0, 14,  0, }, // Indochina
    {  476,  943,       0,  580551,       0,  0, 19,  0, }, // Indonesia_Central
    {  476,  961,       0,  580570,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  476,  979,       0,  580590,       0,  0, 21,  0, }, // Indonesia_Western
    {  476,  997,  580611,  580619,  580632,  8, 13, 16, }, // Iran
    {  476, 1008,  580648,  580661,  580680, 13, 19, 27, }, // Irkutsk
    {  476, 1016,  580707,  580718,  580737, 11, 19, 19, }, // Israel
    {  476, 1023,  580756,  580765,  580779,  9, 14, 17, }, // Japan
    {  476, 1029,  580796,  580824,  580857, 28, 33, 36, }, // Kamchatka
    {  476, 1047,       0,  580893,       0,  0, 15,  0, }, // Kazakhstan
    {  476, 1058,       0,  580908,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {  476, 1077,       0,  580929,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  476, 1106,  580950,  580959,  580973,  9, 14, 17, }, // Korea
    {  476, 1112,       0,  580990,       0,  0, 10,  0, }, // Kosrae
    {  476, 1119,  581000,  581017,  581042, 17, 25, 28, }, // Krasnoyarsk
    {  476, 1151,       0,  581070,       0,  0, 15,  0, }, // Kyrgystan
    {  476, 1161,       0,  581085,       0,  0,  9,  0, }, // Lanka
    {  476, 1175,       0,  581094,       0,  0, 21,  0, }, // Line_Islands
    {  476, 1188,  581115,  581128,  581149, 13, 21, 21, }, // Lord_Howe
    {  476, 1198,  581170,  581179,  581192,  9, 13, 16, }, // Macau
    {  476, 1204,  581208,  581220,  581240, 12, 20, 23, }, // Magadan
    {  476, 1219,       0,  581263,       0,  0, 11,  0, }, // Malaysia
    {  476, 1228,       0,  581274,       0,  0, 12,  0, }, // Maldives
    {  476, 1237,       0,  581286,       0,  0, 17,  0, }, // Marquesas
    {  476, 1247,       0,  581303,       0,  0, 24,  0, }, // Marshall_Islands
    {  476, 1264,  581327,  581339,  581359, 12, 20, 19, }, // Mauritius
    {  476, 1274,       0,  581378,       0,  0, 11,  0, }, // Mawson
    {  476, 1281,  581389,  581412,  581443, 23, 31, 31, }, // Mexico_Pacific
    {  476, 1296,  581474,  581490,  581511, 16, 21, 29, }, // Mongolia
    {  476, 1305,  581540,  581549,  581566,  9, 17, 20, }, // Moscow
    {  476, 1312,       0,  581586,       0,  0, 13,  0, }, // Myanmar
    {  476, 1320,       0,  581599,       0,  0,  9,  0, }, // Nauru
    {  476, 1326,       0,  581608,       0,  0,  9,  0, }, // Nepal
    {  476, 1332,  581617,  581635,  581661, 18, 26, 29, }, // New_Caledonia
    {  476, 1346,  581690,  581705,  581728, 15, 23, 23, }, // New_Zealand
    {  476, 1358,  581751,  581773,  581803, 22, 30, 30, }, // Newfoundland
    {  476, 1371,       0,  581833,       0,  0,  8,  0, }, // Niue
    {  476, 1376,  581841,  581858,  581883, 17, 25, 28, }, // Norfolk
    {  476, 1384,  581911,  581938,  581973, 27, 35, 38, }, // Noronha
    {  476, 1392,       0,  582011,       0,  0, 27,  0, }, // North_Mariana
    {  476, 1406,  582038,  582054,  582078, 16, 24, 27, }, // Novosibirsk
    {  476, 1418,  582105,  582114,  582131,  9, 17, 20, }, // Omsk
    {  476, 1428,  582151,  582164,  582182, 13, 18, 24, }, // Pakistan
    {  476, 1437,       0,  582206,       0,  0,  9,  0, }, // Palau
    {  476, 1443,       0,  582215,       0,  0, 18,  0, }, // Papua_New_Guinea
    {  476, 1460,  582233,  582244,  582263, 11, 19, 22, }, // Paraguay
    {  476, 1469,  582285,  582293,  582309,  8, 16, 19, }, // Peru
    {  476, 1474,  582328,  582342,  582364, 14, 22, 25, }, // Philippines
    {  476, 1486,       0,  582389,       0,  0, 22,  0, }, // Phoenix_Islands
    {  476, 1502,  582411,  582442,  582481, 31, 39, 39, }, // Pierre_Miquelon
    {  476, 1518,       0,  582520,       0,  0, 14,  0, }, // Pitcairn
    {  476, 1527,       0,  582534,       0,  0, 10,  0, }, // Ponape
    {  476, 1534,       0,  582544,       0,  0, 15,  0, }, // Pyongyang
    {  476, 1544,  582559,  582573,  582592, 14, 19, 22, }, // Qyzylorda
    {  476, 1554,       0,  582614,       0,  0, 14,  0, }, // Reunion
    {  476, 1562,       0,  582628,       0,  0, 10,  0, }, // Rothera
    {  476, 1570,  582638,  582649,  582668, 11, 19, 22, }, // Sakhalin
    {  476, 1579,  582690,  582699,  582713,  9, 14, 17, }, // Samara
    {  476, 1596,  582730,  582739,  582753,  9, 14, 17, }, // Samoa
    {  476, 1602,       0,  582770,       0,  0, 15,  0, }, // Seychelles
    {  476, 1624,       0,  582785,       0,  0, 20,  0, }, // Singapore
    {  476, 1634,       0,  582805,       0,  0, 21,  0, }, // Solomon
    {  476, 1642,       0,  582826,       0,  0, 17,  0, }, // South_Georgia
    {  476, 1656,       0,  582843,       0,  0, 13,  0, }, // Suriname
    {  476, 1676,       0,  582856,       0,  0,  8,  0, }, // Syowa
    {  476, 1682,       0,  582864,       0,  0, 10,  0, }, // Tahiti
    {  476, 1689,  582874,  582884,  582899, 10, 15, 18, }, // Taipei
    {  476, 1696,       0,  582917,       0,  0, 15,  0, }, // Tajikistan
    {  476, 1724,       0,  582932,       0,  0, 11,  0, }, // Tokelau
    {  476, 1732,  582943,  582953,  582968, 10, 15, 23, }, // Tonga
    {  476, 1738,       0,  582991,       0,  0,  9,  0, }, // Truk
    {  476, 1750,  583000,  583018,  583044, 18, 26, 25, }, // Turkmenistan
    {  476, 1763,       0,  583069,       0,  0, 10,  0, }, // Tuvalu
    {  476, 1777,  583079,  583089,  583107, 10, 18, 21, }, // Uruguay
    {  476, 1792,  583128,  583143,  583163, 15, 20, 28, }, // Uzbekistan
    {  476, 1803,  583191,  583202,  583218, 11, 16, 24, }, // Vanuatu
    {  476, 1811,       0,  583242,       0,  0, 13,  0, }, // Venezuela
    {  476, 1821,  583255,  583273,  583299, 18, 26, 29, }, // Vladivostok
    {  476, 1833,  583328,  583343,  583366, 15, 23, 26, }, // Volgograd
    {  476, 1843,       0,  583392,       0,  0, 13,  0, }, // Vostok
    {  476, 1850,       0,  583405,       0,  0, 15,  0, }, // Wake
    {  476, 1855,       0,  583420,       0,  0, 22,  0, }, // Wallis
    {  476, 1862,  583442,  583454,  583474, 12, 20, 23, }, // Yakutsk
    {  476, 1870,  583497,  583515,  583541, 18, 26, 29, }, // Yekaterinburg
    {  476, 1892,       0,  583570,       0,  0, 10,  0, }, // Yukon
    {  477,    5,       0,  583580,       0,  0, 21,  0, }, // Afghanistan Oromo/Latin/Ethiopia
    {  477,   17,       0,  583601,       0,  0, 32,  0, }, // Africa_Central
    {  477,   32,       0,  583633,       0,  0, 21,  0, }, // Africa_Eastern
    {  477,   65,       0,  583654,       0,  0, 37,  0, }, // Africa_Southern
    {  477,   81,  583691,  583714,  583751, 23, 37, 29, }, // Africa_Western
    {  477,  107,  583780,  583797,  583828, 17, 31, 24, }, // Alaska
    {  477,  135,  583852,  583868,  583898, 16, 30, 22, }, // Amazon
    {  477,  142,  583920,  583944,  583982, 24, 38, 31, }, // America_Central
    {  477,  158,  584013,  584027,  584055, 14, 28, 21, }, // America_Eastern
    {  477,  174,  584076,  584095,  584128, 19, 33, 26, }, // America_Mountain
    {  477,  191,  584154,  584171,  584202, 17, 31, 24, }, // America_Pacific
    {  477,  214,  584226,  584239,  584266, 13, 27, 20, }, // Apia
    {  477,  232,  584286,  584301,  584330, 15, 29, 22, }, // Arabian
    {  477,  240,  584352,  584372,  584406, 20, 34, 26, }, // Argentina
    {  477,  250,  584432,  584459,  584500, 27, 41, 33, }, // Argentina_Western
    {  477,  268,  584533,  584552,  584585, 19, 33, 25, }, // Armenia
    {  477,  286,  584610,  584630,  584664, 20, 34, 27, }, // Atlantic
    {  477,  295,  584691,  584730,  584783, 39, 53, 46, }, // Australia_Central
    {  477,  313,  584829,  584874,  584933, 45, 59, 52, }, // Australia_CentralWestern
    {  477,  338,  584985,  585014,  585057, 29, 43, 36, }, // Australia_Eastern
    {  477,  356,  585093,  585123,  585196, 30, 73, 38, }, // Australia_Western
    {  477,  374,  585234,  585256,  585292, 22, 36, 28, }, // Azerbaijan
    {  477,  385,  585320,  585337,  585368, 17, 31, 23, }, // Azores
    {  477,  397,  585391,  585413,  585449, 22, 36, 28, }, // Bangladesh
    {  477,  415,       0,  585477,       0,  0, 17,  0, }, // Bhutan
    {  477,  422,       0,  585494,       0,  0, 19,  0, }, // Bolivia
    {  477,  437,  585513,  585534,  585569, 21, 35, 27, }, // Brasilia
    {  477,  454,       0,  585596,       0,  0, 27,  0, }, // Brunei
    {  477,  461,  585623,  585644,  585679, 21, 35, 27, }, // Cape_Verde
    {  477,  478,       0,  585706,       0,  0, 30,  0, }, // Chamorro
    {  477,  487,  585736,  585752,  585782, 16, 30, 23, }, // Chatham
    {  477,  495,  585805,  585821,  585851, 16, 30, 22, }, // Chile
    {  477,  501,  585873,  585891,  585923, 18, 32, 25, }, // China
    {  477,  507,       0,  585948,       0,  0, 24,  0, }, // Christmas
    {  477,  517,       0,  585972,       0,  0, 23,  0, }, // Cocos
    {  477,  523,  585995,  586014,  586047, 19, 33, 25, }, // Colombia
    {  477,  532,  586072,  586094,  586130, 22, 36, 36, }, // Cook
    {  477,  537,  586166,  586181,  586210, 15, 29, 22, }, // Cuba
    {  477,  548,       0,  586232,       0,  0, 16,  0, }, // Davis
    {  477,  564,       0,  586248,       0,  0, 25,  0, }, // DumontDUrville
    {  477,  601,       0,  586273,       0,  0, 22,  0, }, // East_Timor
    {  477,  612,  586295,  586315,  586349, 20, 34, 26, }, // Easter
    {  477,  619,       0,  586375,       0,  0, 20,  0, }, // Ecuador
    {  477,  627,  586395,  586430,  586479, 35, 49, 41, }, // Europe_Central
    {  477,  642,  586520,  586544,  586583, 24, 39, 31, }, // Europe_Eastern
    {  477,  657,       0,  586614,       0,  0, 35,  0, }, // Europe_Further_Eastern
    {  477,  680,  586649,  586675,  586715, 26, 40, 32, }, // Europe_Western
    {  477,  695,  586747,  586775,  586817, 28, 42, 34, }, // Falkland
    {  477,  704,  586851,  586866,  586895, 15, 29, 21, }, // Fiji
    {  477,  709,       0,  586916,       0,  0, 25,  0, }, // French_Guiana
    {  477,  723,       0,  586941,       0,  0, 39,  0, }, // French_Southern
    {  477,  746,       0,  586980,       0,  0, 20,  0, }, // Galapagos
    {  477,  756,       0,  587000,       0,  0, 19,  0, }, // Gambier
    {  477,  764,  587019,  587037,  587069, 18, 32, 24, }, // Georgia
    {  477,  772,       0,  587093,       0,  0, 26,  0, }, // Gilbert_Islands
    {  477,  788,       0,  587119,       0,  0, 36,  0, }, // GMT
    {  477,  830,  587155,  587181,  587221, 26, 40, 32, }, // Greenland_Eastern
    {  477,  848,  587253,  587280,  587321, 27, 41, 33, }, // Greenland_Western
    {  477,  866,       0,  587354,       0,  0, 29,  0, }, // Guam
    {  477,  871,       0,  587383,       0,  0, 30,  0, }, // Gulf
    {  477,  876,       0,  587413,       0,  0, 17,  0, }, // Guyana
    {  477,  883,  587430,  587457,  587498, 27, 41, 34, }, // Hawaii_Aleutian
    {  477,  899,  587532,  587552,  587586, 20, 34, 26, }, // Hong_Kong
    {  477,  909,  587612,  587626,  587654, 14, 28, 20, }, // Hovd
    {  477,  914,       0,  587674,       0,  0, 29,  0, }, // India
    {  477,  920,       0,  587703,       0,  0, 23,  0, }, // Indian_Ocean
    {  477,  933,       0,  587726,       0,  0, 23,  0, }, // Indochina
    {  477,  943,       0,  587749,       0,  0, 39,  0, }, // Indonesia_Central
    {  477,  961,       0,  587788,       0,  0, 28,  0, }, // Indonesia_Eastern
    {  477,  979,       0,  587816,       0,  0, 30,  0, }, // Indonesia_Western
    {  477,  997,  587846,  587860,  587888, 14, 28, 21, }, // Iran
    {  477, 1008,  587909,  587925,  587955, 16, 30, 22, }, // Irkutsk
    {  477, 1016,  587977,  587995,  588027, 18, 32, 25, }, // Israel
    {  477, 1023,  588052,  588069,  588100, 17, 31, 24, }, // Japan
    {  477, 1047,       0,  588124,       0,  0, 22,  0, }, // Kazakhstan
    {  477, 1058,       0,  588146,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  477, 1077,       0,  588174,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  477, 1106,  588203,  588220,  588251, 17, 31, 24, }, // Korea
    {  477, 1112,       0,  588275,       0,  0, 18,  0, }, // Kosrae
    {  477, 1119,  588293,  588313,  588347, 20, 34, 26, }, // Krasnoyarsk
    {  477, 1151,       0,  588373,       0,  0, 23,  0, }, // Kyrgystan
    {  477, 1161,       0,  588396,       0,  0, 16,  0, }, // Lanka
    {  477, 1175,       0,  588412,       0,  0, 22,  0, }, // Line_Islands
    {  477, 1188,  588434,  588452,  588484, 18, 32, 25, }, // Lord_Howe
    {  477, 1204,  588509,  588525,  588555, 16, 30, 22, }, // Magadan
    {  477, 1219,       0,  588577,       0,  0, 21,  0, }, // Malaysia
    {  477, 1228,       0,  588598,       0,  0, 18,  0, }, // Maldives
    {  477, 1237,       0,  588616,       0,  0, 18,  0, }, // Marquesas
    {  477, 1247,       0,  588634,       0,  0, 27,  0, }, // Marshall_Islands
    {  477, 1264,  588661,  588683,  588718, 22, 35, 27, }, // Mauritius
    {  477, 1274,       0,  588745,       0,  0, 15,  0, }, // Mawson
    {  477, 1281,  588760,  588788,  588830, 28, 42, 35, }, // Mexico_Pacific
    {  477, 1296,  588865,  588885,  588919, 20, 34, 26, }, // Mongolia
    {  477, 1305,  588945,  588961,  588991, 16, 30, 22, }, // Moscow
    {  477, 1312,       0,  589013,       0,  0, 21,  0, }, // Myanmar
    {  477, 1320,       0,  589034,       0,  0, 17,  0, }, // Nauru
    {  477, 1326,       0,  589051,       0,  0, 17,  0, }, // Nepal
    {  477, 1332,  589068,  589097,  589140, 29, 43, 35, }, // New_Caledonia
    {  477, 1346,  589175,  589195,  589229, 20, 34, 27, }, // New_Zealand
    {  477, 1358,  589256,  589281,  589320, 25, 39, 32, }, // Newfoundland
    {  477, 1371,       0,  589352,       0,  0, 13,  0, }, // Niue
    {  477, 1376,  589365,  589388,  589425, 23, 37, 30, }, // Norfolk
    {  477, 1384,  589455,  589483,  589525, 28, 42, 34, }, // Noronha
    {  477, 1392,       0,  589559,       0,  0, 36,  0, }, // North_Mariana
    {  477, 1406,  589595,  589616,  589651, 21, 35, 27, }, // Novosibirsk
    {  477, 1418,  589678,  589691,  589718, 13, 27, 19, }, // Omsk
    {  477, 1428,  589737,  589756,  589789, 19, 33, 25, }, // Pakistan
    {  477, 1437,       0,  589814,       0,  0, 15,  0, }, // Palau
    {  477, 1443,       0,  589829,       0,  0, 31,  0, }, // Papua_New_Guinea
    {  477, 1460,  589860,  589882,  589918, 22, 36, 28, }, // Paraguay
    {  477, 1469,  589946,  589961,  589990, 15, 29, 21, }, // Peru
    {  477, 1474,  590011,  590030,  590063, 19, 33, 25, }, // Philippines
    {  477, 1486,       0,  590088,       0,  0, 26,  0, }, // Phoenix_Islands
    {  477, 1502,  590114,  590147,  590194, 33, 47, 40, }, // Pierre_Miquelon
    {  477, 1518,       0,  590234,       0,  0, 17,  0, }, // Pitcairn
    {  477, 1527,       0,  590251,       0,  0, 15,  0, }, // Ponape
    {  477, 1534,       0,  590266,       0,  0, 21,  0, }, // Pyongyang
    {  477, 1554,       0,  590287,       0,  0, 21,  0, }, // Reunion
    {  477, 1562,       0,  590308,       0,  0, 16,  0, }, // Rothera
    {  477, 1570,  590324,  590341,  590372, 17, 31, 23, }, // Sakhalin
    {  477, 1596,  590395,  590412,  590443, 17, 31, 24, }, // Samoa
    {  477, 1602,       0,  590467,       0,  0, 18,  0, }, // Seychelles
    {  477, 1624,       0,  590485,       0,  0, 33,  0, }, // Singapore
    {  477, 1634,       0,  590518,       0,  0, 26,  0, }, // Solomon
    {  477, 1642,       0,  590544,       0,  0, 25,  0, }, // South_Georgia
    {  477, 1656,       0,  590569,       0,  0, 18,  0, }, // Suriname
    {  477, 1676,       0,  590587,       0,  0, 14,  0, }, // Syowa
    {  477, 1682,       0,  590601,       0,  0, 15,  0, }, // Tahiti
    {  477, 1689,  590616,  590633,  590664, 17, 31, 24, }, // Taipei
    {  477, 1696,       0,  590688,       0,  0, 20,  0, }, // Tajikistan
    {  477, 1724,       0,  590708,       0,  0, 17,  0, }, // Tokelau
    {  477, 1732,  590725,  590739,  590767, 14, 28, 20, }, // Tonga
    {  477, 1738,       0,  590787,       0,  0, 14,  0, }, // Truk
    {  477, 1750,  590801,  590824,  590861, 23, 37, 29, }, // Turkmenistan
    {  477, 1763,       0,  590890,       0,  0, 15,  0, }, // Tuvalu
    {  477, 1777,  590905,  590924,  590957, 19, 33, 25, }, // Uruguay
    {  477, 1792,  590982,  591003,  591038, 21, 35, 27, }, // Uzbekistan
    {  477, 1803,  591065,  591082,  591113, 17, 31, 23, }, // Vanuatu
    {  477, 1811,       0,  591136,       0,  0, 21,  0, }, // Venezuela
    {  477, 1821,  591157,  591177,  591211, 20, 34, 26, }, // Vladivostok
    {  477, 1833,  591237,  591255,  591287, 18, 32, 24, }, // Volgograd
    {  477, 1843,       0,  591311,       0,  0, 15,  0, }, // Vostok
    {  477, 1850,       0,  591326,       0,  0, 19,  0, }, // Wake
    {  477, 1855,       0,  591345,       0,  0, 25,  0, }, // Wallis
    {  477, 1862,  591370,  591386,  591415, 16, 29, 22, }, // Yakutsk
    {  477, 1870,  591437,  591459,  591495, 22, 36, 28, }, // Yekaterinburg
    {  477, 1892,       0,  591523,       0,  0, 16,  0, }, // Yukon
    {  481,  627,  591539,  591565,  591602, 26, 37, 35, }, // Europe_Central Ossetic/Cyrillic/Russia
    {  481,  642,  591637,  591660,  591694, 23, 34, 32, }, // Europe_Eastern
    {  481,  680,  591726,  591752,  591789, 26, 37, 35, }, // Europe_Western
    {  481,  764,  591824,  591843,  591873, 19, 30, 28, }, // Georgia
    {  481,  788,       0,  591901,       0,  0, 25,  0, }, // GMT
    {  481, 1305,  591926,  591941,  591967, 15, 26, 24, }, // Moscow
    {  482,   17,       0,  395984,       0,  0, 21,  0, }, // Africa_Central Papiamento/Latin/Curacao
    {  482,   32,       0,  591991,       0,  0, 17,  0, }, // Africa_Eastern
    {  482,  385,  592008,  592021,  592041, 13, 20, 32, }, // Azores
    {  482,  627,  396978,  592073,  592101, 21, 28, 40, }, // Europe_Central
    {  482,  642,  592141,  592158,  592182, 17, 24, 36, }, // Europe_Eastern
    {  482,  680,  592218,  592236,  592261, 18, 25, 37, }, // Europe_Western
    {  482,  788,       0,    2791,       0,  0, 19,  0, }, // GMT
    {  484,    5,       0,  592298,       0,  0, 13,  0, }, // Afghanistan Pashto/Arabic/Afghanistan
    {  484,   17,       0,  592311,       0,  0, 16,  0, }, // Africa_Central
    {  484,   32,       0,  592327,       0,  0, 15,  0, }, // Africa_Eastern
    {  484,   65,       0,  592342,       0,  0, 23,  0, }, // Africa_Southern
    {  484,   81,  592365,  592382,  592408, 17, 26, 26, }, // Africa_Western
    {  484,  107,  592434,  592444,  592461, 10, 17, 24, }, // Alaska
    {  484,  135,  592485,  592496,  592514, 11, 18, 16, }, // Amazon
    {  484,  142,  592530,  592539,  592555,  9, 16, 23, }, // America_Central
    {  484,  158,  592578,  592586,  592601,  8, 15, 19, }, // America_Eastern
    {  484,  174,  592620,  592631,  592647, 11, 16, 20, }, // America_Mountain
    {  484,  191,  592667,  592676,  592694,  9, 18, 20, }, // America_Pacific
    {  484,  214,  592714,  592722,  592737,  8, 15, 22, }, // Apia
    {  484,  232,  592759,  592767,  592782,  8, 15, 22, }, // Arabian
    {  484,  240,  592804,  592816,  592835, 12, 19, 17, }, // Argentina
    {  484,  250,  592852,  592871,  592897, 19, 26, 24, }, // Argentina_Western
    {  484,  268,  592921,  592933,  592952, 12, 19, 17, }, // Armenia
    {  484,  286,  592969,  592981,  593000, 12, 19, 23, }, // Atlantic
    {  484,  295,  593023,  593041,  593066, 18, 25, 32, }, // Australia_Central
    {  484,  313,  593098,  593123,  593155, 25, 32, 39, }, // Australia_CentralWestern
    {  484,  338,  593194,  593211,  593235, 17, 24, 31, }, // Australia_Eastern
    {  484,  356,  593266,  593285,  593311, 19, 26, 33, }, // Australia_Western
    {  484,  374,  593344,  593359,  593379, 15, 20, 19, }, // Azerbaijan
    {  484,  385,  593398,  593408,  593425, 10, 17, 15, }, // Azores
    {  484,  397,  593440,  593453,  593472, 13, 19, 18, }, // Bangladesh
    {  484,  415,       0,  593490,       0,  0, 10,  0, }, // Bhutan
    {  484,  422,       0,  593500,       0,  0, 11,  0, }, // Bolivia
    {  484,  437,  593511,  593521,  593538, 10, 17, 15, }, // Brasilia
    {  484,  454,       0,  593553,       0,  0, 21,  0, }, // Brunei
    {  484,  461,  593574,  593586,  593605, 12, 19, 16, }, // Cape_Verde
    {  484,  478,       0,  593621,       0,  0, 16,  0, }, // Chamorro
    {  484,  487,  593637,  593646,  593662,  9, 16, 23, }, // Chatham
    {  484,  495,  593685,  593692,  593706,  7, 14, 12, }, // Chile
    {  484,  501,  593718,  593725,  593739,  7, 14, 20, }, // China
    {  484,  507,       0,  593759,       0,  0, 15,  0, }, // Christmas
    {  484,  517,       0,  593774,       0,  0, 18,  0, }, // Cocos
    {  484,  523,  593792,  593803,  593821, 11, 18, 16, }, // Colombia
    {  484,  532,  593837,  593853,  593876, 16, 23, 25, }, // Cook
    {  484,  537,  593901,  593910,  593926,  9, 16, 20, }, // Cuba
    {  484,  548,       0,  593946,       0,  0,  8,  0, }, // Davis
    {  484,  564,       0,  593954,       0,  0, 14,  0, }, // DumontDUrville
    {  484,  601,       0,  593968,       0,  0, 14,  0, }, // East_Timor
    {  484,  612,  593982,  593996,  594017, 14, 21, 19, }, // Easter
    {  484,  619,       0,  594036,       0,  0, 13,  0, }, // Ecuador
    {  484,  627,  594049,  594066,  594090, 17, 24, 21, }, // Europe_Central
    {  484,  642,  594111,  594127,  594150, 16, 23, 21, }, // Europe_Eastern
    {  484,  657,       0,  594171,       0,  0, 20,  0, }, // Europe_Further_Eastern
    {  484,  680,  594191,  594209,  594234, 18, 25, 23, }, // Europe_Western
    {  484,  695,  594257,  594272,  594296, 15, 24, 22, }, // Falkland
    {  484,  704,  594318,  594325,  594339,  7, 14, 14, }, // Fiji
    {  484,  709,       0,  594353,       0,  0, 18,  0, }, // French_Guiana
    {  484,  723,       0,  594371,       0,  0, 29,  0, }, // French_Southern
    {  484,  746,       0,  594400,       0,  0, 11,  0, }, // Galapagos
    {  484,  756,       0,  594411,       0,  0, 10,  0, }, // Gambier
    {  484,  764,  594421,  594431,  594448, 10, 17, 19, }, // Georgia
    {  484,  772,       0,  594467,       0,  0, 18,  0, }, // Gilbert_Islands
    {  484,  788,       0,  594485,       0,  0, 18,  0, }, // GMT
    {  484,  830,  594503,  594522,  594548, 19, 26, 24, }, // Greenland_Eastern
    {  484,  848,  594572,  594591,  594617, 19, 26, 24, }, // Greenland_Western
    {  484,  871,       0,  594641,       0,  0, 15,  0, }, // Gulf
    {  484,  876,       0,  594656,       0,  0, 13,  0, }, // Guyana
    {  484,  883,  594669,  594686,  594710, 17, 24, 26, }, // Hawaii_Aleutian
    {  484,  899,  594736,  594749,  594769, 13, 20, 18, }, // Hong_Kong
    {  484,  909,  594787,  594795,  594810,  8, 15, 13, }, // Hovd
    {  484,  914,       0,  594823,       0,  0, 14,  0, }, // India
    {  484,  920,       0,  594837,       0,  0, 15,  0, }, // Indian_Ocean
    {  484,  933,       0,  594852,       0,  0, 13,  0, }, // Indochina
    {  484,  943,       0,  594865,       0,  0, 18,  0, }, // Indonesia_Central
    {  484,  961,       0,  594883,       0,  0, 13,  0, }, // Indonesia_Eastern
    {  484,  979,       0,  594896,       0,  0, 20,  0, }, // Indonesia_Western
    {  484,  997,  594916,  594925,  594941,  9, 16, 23, }, // Iran
    {  484, 1008,  594964,  594975,  594993, 11, 18, 16, }, // Irkutsk
    {  484, 1016,  595009,  595020,  595038, 11, 18, 25, }, // Israel
    {  484, 1023,  595063,  595072,  595088,  9, 16, 23, }, // Japan
    {  484, 1047,       0,  595111,       0,  0, 12,  0, }, // Kazakhstan
    {  484, 1058,       0,  595123,       0,  0, 17,  0, }, // Kazakhstan_Eastern
    {  484, 1077,       0,  595140,       0,  0, 19,  0, }, // Kazakhstan_Western
    {  484, 1106,  595159,  595170,  595188, 11, 18, 25, }, // Korea
    {  484, 1112,       0,  595213,       0,  0, 10,  0, }, // Kosrae
    {  484, 1119,  595223,  595238,  595260, 15, 22, 20, }, // Krasnoyarsk
    {  484, 1151,       0,  595280,       0,  0, 13,  0, }, // Kyrgystan
    {  484, 1175,       0,  595293,       0,  0, 17,  0, }, // Line_Islands
    {  484, 1188,  595310,  595322,  595341, 12, 19, 26, }, // Lord_Howe
    {  484, 1204,  595367,  595377,  595394, 10, 17, 15, }, // Magadan
    {  484, 1219,       0,  595409,       0,  0, 12,  0, }, // Malaysia
    {  484, 1228,       0,  595421,       0,  0, 10,  0, }, // Maldives
    {  484, 1237,       0,  595431,       0,  0, 10,  0, }, // Marquesas
    {  484, 1247,       0,  595441,       0,  0, 18,  0, }, // Marshall_Islands
    {  484, 1264,  595459,  595470,  595488, 11, 18, 18, }, // Mauritius
    {  484, 1274,       0,  595506,       0,  0,  9,  0, }, // Mawson
    {  484, 1281,  595515,  595531,  595554, 16, 23, 25, }, // Mexico_Pacific
    {  484, 1296,  595579,  595593,  595614, 14, 21, 19, }, // Mongolia
    {  484, 1305,  595633,  595642,  595658,  9, 16, 13, }, // Moscow
    {  484, 1312,       0,  595671,       0,  0, 11,  0, }, // Myanmar
    {  484, 1320,       0,  595682,       0,  0,  9,  0, }, // Nauru
    {  484, 1326,       0,  595691,       0,  0,  9,  0, }, // Nepal
    {  484, 1332,  595700,  595717,  595741, 17, 24, 21, }, // New_Caledonia
    {  484, 1346,  595762,  595776,  595797, 14, 21, 28, }, // New_Zealand
    {  484, 1358,  595825,  595842,  595867, 17, 25, 31, }, // Newfoundland
    {  484, 1371,       0,  595898,       0,  0,  8,  0, }, // Niue
    {  484, 1376,  595906,  595921,  595946, 15, 25, 23, }, // Norfolk
    {  484, 1384,  595969,  595991,  596020, 22, 29, 27, }, // Noronha
    {  484, 1406,  596047,  596060,  596080, 13, 20, 18, }, // Novosibirsk
    {  484, 1418,  596098,  596107,  596123,  9, 16, 14, }, // Omsk
    {  484, 1428,  596137,  596148,  596166, 11, 18, 16, }, // Pakistan
    {  484, 1437,       0,  596182,       0,  0,  9,  0, }, // Palau
    {  484, 1443,       0,  596191,       0,  0, 17,  0, }, // Papua_New_Guinea
    {  484, 1460,  596208,  596220,  596239, 12, 19, 17, }, // Paraguay
    {  484, 1469,  596256,  596264,  596279,  8, 15, 13, }, // Peru
    {  484, 1474,  596292,  596302,  596319, 10, 17, 15, }, // Philippines
    {  484, 1486,       0,  596334,       0,  0, 16,  0, }, // Phoenix_Islands
    {  484, 1502,  596350,  596373,  596403, 23, 30, 32, }, // Pierre_Miquelon
    {  484, 1518,       0,  596435,       0,  0, 13,  0, }, // Pitcairn
    {  484, 1527,       0,  596448,       0,  0,  9,  0, }, // Ponape
    {  484, 1534,       0,  596457,       0,  0, 14,  0, }, // Pyongyang
    {  484, 1554,       0,  596471,       0,  0, 12,  0, }, // Reunion
    {  484, 1562,       0,  596483,       0,  0,  9,  0, }, // Rothera
    {  484, 1570,  596492,  596501,  596517,  9, 16, 14, }, // Sakhalin
    {  484, 1596,  596531,  596539,  596554,  8, 15, 22, }, // Samoa
    {  484, 1602,       0,  596576,       0,  0, 10,  0, }, // Seychelles
    {  484, 1624,       0,  596586,       0,  0, 18,  0, }, // Singapore
    {  484, 1634,       0,  596604,       0,  0, 19,  0, }, // Solomon
    {  484, 1642,       0,  596623,       0,  0, 17,  0, }, // South_Georgia
    {  484, 1656,       0,  596640,       0,  0, 11,  0, }, // Suriname
    {  484, 1676,       0,  596651,       0,  0,  8,  0, }, // Syowa
    {  484, 1682,       0,  596659,       0,  0,  9,  0, }, // Tahiti
    {  484, 1689,  596668,  596677,  596693,  9, 16, 23, }, // Taipei
    {  484, 1696,       0,  596716,       0,  0, 12,  0, }, // Tajikistan
    {  484, 1724,       0,  596728,       0,  0, 11,  0, }, // Tokelau
    {  484, 1732,  596739,  596748,  596765,  9, 17, 14, }, // Tonga
    {  484, 1738,       0,  596779,       0,  0,  7,  0, }, // Truk
    {  484, 1750,  596786,  596800,  596820, 14, 20, 18, }, // Turkmenistan
    {  484, 1763,       0,  596838,       0,  0, 10,  0, }, // Tuvalu
    {  484, 1777,  596848,  596860,  596879, 12, 19, 17, }, // Uruguay
    {  484, 1792,  596896,  596908,  596927, 12, 19, 17, }, // Uzbekistan
    {  484, 1803,  596944,  596955,  596972, 11, 17, 15, }, // Vanuatu
    {  484, 1811,       0,  596987,       0,  0, 12,  0, }, // Venezuela
    {  484, 1821,  596999,  597013,  597034, 14, 21, 19, }, // Vladivostok
    {  484, 1833,  597053,  597066,  597086, 13, 20, 18, }, // Volgograd
    {  484, 1843,       0,  597104,       0,  0, 10,  0, }, // Vostok
    {  484, 1850,       0,  597114,       0,  0, 12,  0, }, // Wake
    {  484, 1855,       0,  597126,       0,  0, 18,  0, }, // Wallis
    {  484, 1862,  597144,  597155,  597173, 11, 18, 18, }, // Yakutsk
    {  484, 1870,  597191,  597206,  597229, 15, 23, 22, }, // Yekaterinburg
    {  484, 1892,       0,  597251,       0,  0, 11,  0, }, // Yukon
    {  485,  107,  592434,  592444,  597262, 10, 17, 26, }, // Alaska Pashto/Arabic/Pakistan
    {  485,  142,  592530,  592539,  597288,  9, 16, 18, }, // America_Central
    {  485,  158,  592578,  592586,  597306,  8, 15, 19, }, // America_Eastern
    {  485,  174,  592620,  592631,  597325, 11, 16, 20, }, // America_Mountain
    {  485,  191,  592667,  592676,  597345,  9, 18, 20, }, // America_Pacific
    {  485,  214,  592714,  592722,  597365,  8, 15, 17, }, // Apia
    {  485,  232,  592759,  592767,  597382,  8, 15, 18, }, // Arabian
    {  485,  286,  592969,  592981,  597400, 12, 19, 23, }, // Atlantic
    {  485,  356,  593266,  593285,  597423, 19, 26, 28, }, // Australia_Western
    {  485,  501,  593718,  593725,  597451,  7, 14, 20, }, // China
    {  485,  537,  593901,  593910,  597471,  9, 16, 20, }, // Cuba
    {  485,  680,  597491,  597508,  597533, 17, 25, 25, }, // Europe_Western
    {  485,  723,       0,  597558,       0,  0, 29,  0, }, // French_Southern
    {  485,  883,  594669,  594686,  597587, 17, 24, 26, }, // Hawaii_Aleutian
    {  485,  997,  594916,  594925,  597613,  9, 16, 18, }, // Iran
    {  485, 1016,  595009,  595020,  597631, 11, 18, 21, }, // Israel
    {  485, 1023,  595063,  595072,  597652,  9, 16, 20, }, // Japan
    {  485, 1106,  595159,  595170,  597672, 11, 18, 25, }, // Korea
    {  485, 1175,       0,  597697,       0,  0, 17,  0, }, // Line_Islands
    {  485, 1188,  595310,  595322,  597714, 12, 19, 23, }, // Lord_Howe
    {  485, 1281,  595515,  595531,  597737, 16, 23, 25, }, // Mexico_Pacific
    {  485, 1346,  595762,  595776,  597762, 14, 21, 29, }, // New_Zealand
    {  485, 1358,  595825,  595842,  597791, 17, 25, 27, }, // Newfoundland
    {  485, 1502,  596350,  596373,  597818, 23, 30, 32, }, // Pierre_Miquelon
    {  485, 1596,  596531,  596539,  597850,  8, 15, 24, }, // Samoa
    {  486,    5,       0,  597874,       0,  0, 13,  0, }, // Afghanistan Persian/Arabic/Iran
    {  486,   17,       0,  597887,       0,  0, 15,  0, }, // Africa_Central
    {  486,   32,       0,  597902,       0,  0, 14,  0, }, // Africa_Eastern
    {  486,   65,       0,  597916,       0,  0, 20,  0, }, // Africa_Southern
    {  486,   81,  597936,  597950,  597969, 14, 19, 23, }, // Africa_Western
    {  486,  107,  597992,  598002,  598017, 10, 15, 19, }, // Alaska
    {  486,  128,  598036,  598047,  598063, 11, 16, 20, }, // Almaty
    {  486,  135,  598083,  598093,  598108, 10, 15, 19, }, // Amazon
    {  486,  142,  598127,  598142,  598162, 15, 20, 24, }, // America_Central
    {  486,  158,  598186,  598200,  598219, 14, 19, 23, }, // America_Eastern
    {  486,  174,  598242,  598261,  598285, 19, 24, 28, }, // America_Mountain
    {  486,  191,  598313,  598327,  598346, 14, 19, 23, }, // America_Pacific
    {  486,  207,  598369,  598379,  598394, 10, 15, 19, }, // Anadyr
    {  486,  214,  598413,  598421,  598434,  8, 13, 17, }, // Apia
    {  486,  232,  598451,  598462,  598478, 11, 16, 20, }, // Arabian
    {  486,  240,  598498,  598510,  598527, 12, 17, 21, }, // Argentina
    {  486,  250,  598548,  598564,  598585, 16, 21, 25, }, // Argentina_Western
    {  486,  268,  598610,  598622,  598639, 12, 17, 21, }, // Armenia
    {  486,  286,  598660,  598672,  598689, 12, 17, 21, }, // Atlantic
    {  486,  295,  598710,  598727,  598749, 17, 22, 26, }, // Australia_Central
    {  486,  313,  598775,  598798,  598826, 23, 28, 32, }, // Australia_CentralWestern
    {  486,  338,  598858,  598876,  598899, 18, 23, 27, }, // Australia_Eastern
    {  486,  356,  598926,  598944,  598967, 18, 23, 27, }, // Australia_Western
    {  486,  374,  598994,  599014,  599039, 20, 25, 29, }, // Azerbaijan
    {  486,  385,  599068,  599076,  599089,  8, 13, 17, }, // Azores
    {  486,  397,  599106,  599117,  599133, 11, 16, 20, }, // Bangladesh
    {  486,  415,       0,  599153,       0,  0,  9,  0, }, // Bhutan
    {  486,  422,       0,  599162,       0,  0, 10,  0, }, // Bolivia
    {  486,  437,  599172,  599184,  599201, 12, 17, 21, }, // Brasilia
    {  486,  454,       0,  599222,       0,  0, 20,  0, }, // Brunei
    {  486,  461,  599242,  599253,  599269, 11, 16, 20, }, // Cape_Verde
    {  486,  478,       0,  599289,       0,  0, 15,  0, }, // Chamorro
    {  486,  487,  599304,  599314,  599329, 10, 15, 19, }, // Chatham
    {  486,  495,  599348,  599356,  599369,  8, 13, 17, }, // Chile
    {  486,  501,  599386,  599393,  599405,  7, 12, 16, }, // China
    {  486,  507,       0,  599421,       0,  0, 17,  0, }, // Christmas
    {  486,  517,       0,  599438,       0,  0, 15,  0, }, // Cocos
    {  486,  523,  599453,  599463,  599478, 10, 15, 19, }, // Colombia
    {  486,  532,  599438,  599497,  599515, 13, 18, 22, }, // Cook
    {  486,  537,  599537,  599545,  599558,  8, 13, 17, }, // Cuba
    {  486,  548,       0,  599575,       0,  0,  9,  0, }, // Davis
    {  486,  564,       0,  599584,       0,  0, 16,  0, }, // DumontDUrville
    {  486,  601,       0,  599600,       0,  0, 14,  0, }, // East_Timor
    {  486,  612,  599614,  599630,  599651, 16, 21, 25, }, // Easter
    {  486,  619,       0,  599676,       0,  0, 11,  0, }, // Ecuador
    {  486,  627,  599687,  599701,  599720, 14, 19, 23, }, // Europe_Central
    {  486,  642,  599743,  599756,  599774, 13, 18, 22, }, // Europe_Eastern
    {  486,  657,       0,  599796,       0,  0, 36,  0, }, // Europe_Further_Eastern
    {  486,  680,  599832,  599845,  599863, 13, 18, 22, }, // Europe_Western
    {  486,  695,  599885,  599902,  599924, 17, 22, 26, }, // Falkland
    {  486,  704,  599950,  599958,  599971,  8, 13, 17, }, // Fiji
    {  486,  709,       0,  599988,       0,  0, 16,  0, }, // French_Guiana
    {  486,  723,       0,  600004,       0,  0, 37,  0, }, // French_Southern
    {  486,  746,       0,  600041,       0,  0, 13,  0, }, // Galapagos
    {  486,  756,       0,  600054,       0,  0, 10,  0, }, // Gambier
    {  486,  764,  600064,  600075,  600091, 11, 16, 20, }, // Georgia
    {  486,  772,       0,  600111,       0,  0, 16,  0, }, // Gilbert_Islands
    {  486,  788,       0,  600127,       0,  0, 11,  0, }, // GMT
    {  486,  830,  600138,  600153,  600173, 15, 20, 24, }, // Greenland_Eastern
    {  486,  848,  600197,  600212,  600232, 15, 20, 24, }, // Greenland_Western
    {  486,  866,       0,  600256,       0,  0, 13,  0, }, // Guam
    {  486,  871,       0,  600269,       0,  0, 18,  0, }, // Gulf
    {  486,  876,       0,  599988,       0,  0,  9,  0, }, // Guyana
    {  486,  883,  600287,  600303,  600324, 16, 21, 25, }, // Hawaii_Aleutian
    {  486,  899,  600349,  600360,  600376, 11, 16, 20, }, // Hong_Kong
    {  486,  909,  600396,  600404,  600417,  8, 13, 17, }, // Hovd
    {  486,  914,       0,  600434,       0,  0,  7,  0, }, // India
    {  486,  920,       0,  600441,       0,  0, 15,  0, }, // Indian_Ocean
    {  486,  933,       0,  600456,       0,  0, 11,  0, }, // Indochina
    {  486,  943,       0,  600467,       0,  0, 16,  0, }, // Indonesia_Central
    {  486,  961,       0,  600483,       0,  0, 15,  0, }, // Indonesia_Eastern
    {  486,  979,       0,  600498,       0,  0, 15,  0, }, // Indonesia_Western
    {  486,  997,  600513,  600522,  600536,  9, 14, 18, }, // Iran
    {  486, 1008,  600554,  600566,  600583, 12, 17, 21, }, // Irkutsk
    {  486, 1016,  600604,  600615,  600631, 11, 16, 20, }, // Israel
    {  486, 1023,  600651,  600659,  600672,  8, 13, 17, }, // Japan
    {  486, 1029,  600689,  600714,  600744, 25, 30, 34, }, // Kamchatka
    {  486, 1047,       0,  600778,       0,  0, 12,  0, }, // Kazakhstan
    {  486, 1058,       0,  600790,       0,  0, 16,  0, }, // Kazakhstan_Eastern
    {  486, 1077,       0,  600806,       0,  0, 16,  0, }, // Kazakhstan_Western
    {  486, 1106,  600822,  600829,  600841,  7, 12, 16, }, // Korea
    {  486, 1112,       0,  600857,       0,  0, 10,  0, }, // Kosrae
    {  486, 1119,  600867,  600882,  600902, 15, 20, 24, }, // Krasnoyarsk
    {  486, 1151,       0,  600926,       0,  0, 13,  0, }, // Kyrgystan
    {  486, 1161,       0,  600939,       0,  0,  9,  0, }, // Lanka
    {  486, 1175,       0,  600948,       0,  0, 14,  0, }, // Line_Islands
    {  486, 1188,  600962,  600971,  600985,  9, 14, 18, }, // Lord_Howe
    {  486, 1198,  601003,  601013,  601028, 10, 15, 19, }, // Macau
    {  486, 1204,  601047,  601058,  601074, 11, 16, 20, }, // Magadan
    {  486, 1219,       0,  601094,       0,  0,  9,  0, }, // Malaysia
    {  486, 1228,       0,  601103,       0,  0, 10,  0, }, // Maldives
    {  486, 1237,       0,  601113,       0,  0, 11,  0, }, // Marquesas
    {  486, 1247,       0,  601124,       0,  0, 16,  0, }, // Marshall_Islands
    {  486, 1264,  601140,  601149,  601163,  9, 14, 18, }, // Mauritius
    {  486, 1274,       0,  601181,       0,  0, 10,  0, }, // Mawson
    {  486, 1281,  601191,  601204,  601222, 13, 18, 22, }, // Mexico_Pacific
    {  486, 1296,  601244,  601259,  601279, 15, 20, 24, }, // Mongolia
    {  486, 1305,  601303,  601311,  601324,  8, 13, 17, }, // Moscow
    {  486, 1312,       0,  601341,       0,  0, 11,  0, }, // Myanmar
    {  486, 1320,       0,  601352,       0,  0, 10,  0, }, // Nauru
    {  486, 1326,       0,  601362,       0,  0,  8,  0, }, // Nepal
    {  486, 1332,  601370,  601388,  601411, 18, 23, 27, }, // New_Caledonia
    {  486, 1346,  601438,  601450,  601467, 12, 17, 21, }, // New_Zealand
    {  486, 1358,  601488,  601502,  601521, 14, 19, 23, }, // Newfoundland
    {  486, 1371,       0,  601544,       0,  0,  9,  0, }, // Niue
    {  486, 1376,  601553,  601571,  601594, 18, 23, 27, }, // Norfolk
    {  486, 1384,  601621,  601643,  601670, 22, 27, 31, }, // Noronha
    {  486, 1392,       0,  601701,       0,  0, 24,  0, }, // North_Mariana
    {  486, 1406,  601725,  601739,  601758, 14, 19, 23, }, // Novosibirsk
    {  486, 1418,  601781,  601790,  601804,  9, 14, 18, }, // Omsk
    {  486, 1428,  601822,  601833,  601849, 11, 16, 20, }, // Pakistan
    {  486, 1437,       0,  601869,       0,  0, 10,  0, }, // Palau
    {  486, 1443,       0,  601879,       0,  0, 18,  0, }, // Papua_New_Guinea
    {  486, 1460,  601897,  601909,  601926, 12, 17, 21, }, // Paraguay
    {  486, 1469,  601947,  601954,  601966,  7, 12, 16, }, // Peru
    {  486, 1474,  601982,  601993,  602009, 11, 16, 20, }, // Philippines
    {  486, 1486,       0,  602029,       0,  0, 16,  0, }, // Phoenix_Islands
    {  486, 1502,  602045,  602065,  602090, 20, 25, 29, }, // Pierre_Miquelon
    {  486, 1518,       0,  602119,       0,  0, 12,  0, }, // Pitcairn
    {  486, 1527,       0,  602131,       0,  0, 10,  0, }, // Ponape
    {  486, 1534,       0,  602141,       0,  0, 14,  0, }, // Pyongyang
    {  486, 1544,  602155,  602170,  602190, 15, 20, 24, }, // Qyzylorda
    {  486, 1554,       0,  602214,       0,  0, 11,  0, }, // Reunion
    {  486, 1562,       0,  602225,       0,  0,  9,  0, }, // Rothera
    {  486, 1570,  602234,  602245,  602261, 11, 16, 20, }, // Sakhalin
    {  486, 1579,  602281,  602291,  602306, 10, 15, 19, }, // Samara
    {  486, 1596,  602325,  602334,  602348,  9, 14, 18, }, // Samoa
    {  486, 1602,       0,  602366,       0,  0,  8,  0, }, // Seychelles
    {  486, 1624,       0,  602374,       0,  0, 11,  0, }, // Singapore
    {  486, 1634,       0,  602385,       0,  0, 16,  0, }, // Solomon
    {  486, 1642,       0,  602401,       0,  0, 17,  0, }, // South_Georgia
    {  486, 1656,       0,  602418,       0,  0, 11,  0, }, // Suriname
    {  486, 1676,       0,  602429,       0,  0,  8,  0, }, // Syowa
    {  486, 1682,       0,  602437,       0,  0, 10,  0, }, // Tahiti
    {  486, 1689,  602447,  602456,  602470,  9, 14, 18, }, // Taipei
    {  486, 1696,       0,  602488,       0,  0, 13,  0, }, // Tajikistan
    {  486, 1724,       0,  602501,       0,  0, 11,  0, }, // Tokelau
    {  486, 1732,  602512,  602521,  602535,  9, 14, 18, }, // Tonga
    {  486, 1738,       0,  602553,       0,  0,  9,  0, }, // Truk
    {  486, 1750,  602562,  602575,  602593, 13, 18, 22, }, // Turkmenistan
    {  486, 1763,       0,  602615,       0,  0, 10,  0, }, // Tuvalu
    {  486, 1777,  602625,  602636,  602652, 11, 16, 20, }, // Uruguay
    {  486, 1792,  602672,  602684,  602701, 12, 17, 21, }, // Uzbekistan
    {  486, 1803,  602722,  602732,  602747, 10, 15, 19, }, // Vanuatu
    {  486, 1811,       0,  602766,       0,  0, 11,  0, }, // Venezuela
    {  486, 1821,  602777,  602792,  602812, 15, 20, 24, }, // Vladivostok
    {  486, 1833,  602836,  602848,  602865, 12, 17, 21, }, // Volgograd
    {  486, 1843,       0,  602886,       0,  0,  9,  0, }, // Vostok
    {  486, 1850,       0,  602895,       0,  0, 14,  0, }, // Wake
    {  486, 1855,       0,  602909,       0,  0, 18,  0, }, // Wallis
    {  486, 1862,  602927,  602938,  602954, 11, 16, 20, }, // Yakutsk
    {  486, 1870,  602974,  602989,  603009, 15, 20, 24, }, // Yekaterinburg
    {  486, 1892,       0,  603033,       0,  0,  9,  0, }, // Yukon
    {  488,    5,       0,  603042,       0,  0, 15,  0, }, // Afghanistan Polish/Latin/Poland
    {  488,   17,       0,  603057,       0,  0, 23,  0, }, // Africa_Central
    {  488,   32,       0,  603080,       0,  0, 24,  0, }, // Africa_Eastern
    {  488,   65,       0,  603104,       0,  0, 25,  0, }, // Africa_Southern
    {  488,   81,  603129,  603153,  603189, 24, 36, 30, }, // Africa_Western
    {  488,  107,  603219,  603230,  603255, 11, 25, 19, }, // Alaska
    {  488,  128,  603274,  603286,  603310, 12, 24, 18, }, // Almaty
    {  488,  135,  603328,  603342,  603368, 14, 26, 20, }, // Amazon
    {  488,  142,  603388,  603412,  603448, 24, 36, 30, }, // America_Central
    {  488,  158,  603478,  603503,  603540, 25, 37, 31, }, // America_Eastern
    {  488,  174,  603571,  603582,  603605, 11, 23, 17, }, // America_Mountain
    {  488,  191,  603622,  603637,  603664, 15, 27, 21, }, // America_Pacific
    {  488,  207,  603685,  603696,  603719, 11, 23, 17, }, // Anadyr
    {  488,  214,  603736,  603745,  603768,  9, 23, 17, }, // Apia
    {  488,  219,  603785,  603799,  603825, 14, 26, 20, }, // Aqtau
    {  488,  225,  603845,  603861,  603889, 16, 28, 22, }, // Aqtobe
    {  488,  232,  603911,  603932,  603967, 21, 35, 29, }, // Arabian
    {  488,  240,  603996,  604010,  604038, 14, 28, 22, }, // Argentina
    {  488,  250,  604060,  604084,  604122, 24, 38, 32, }, // Argentina_Western
    {  488,  268,  604154,  604166,  604192, 12, 26, 20, }, // Armenia
    {  488,  286,  604212,  604227,  604254, 15, 27, 21, }, // Atlantic
    {  488,  295,  604275,  604300,  604337, 25, 37, 31, }, // Australia_Central
    {  488,  313,  604368,  604403,  604450, 35, 47, 41, }, // Australia_CentralWestern
    {  488,  338,  604491,  604517,  604555, 26, 38, 32, }, // Australia_Eastern
    {  488,  356,  604587,  604613,  604651, 26, 38, 32, }, // Australia_Western
    {  488,  374,  604683,  604699,  604729, 16, 30, 24, }, // Azerbaijan
    {  488,  385,  604753,  604763,  604787, 10, 24, 18, }, // Azores
    {  488,  397,  604805,  604820,  604849, 15, 29, 23, }, // Bangladesh
    {  488,  415,       0,  604872,       0,  0, 11,  0, }, // Bhutan
    {  488,  422,       0,  604883,       0,  0, 12,  0, }, // Bolivia
    {  488,  437,  604895,  604908,  604935, 13, 27, 21, }, // Brasilia
    {  488,  454,       0,  604956,       0,  0, 11,  0, }, // Brunei
    {  488,  461,  604967,  604997,  605041, 30, 44, 38, }, // Cape_Verde
    {  488,  478,       0,  605079,       0,  0, 13,  0, }, // Chamorro
    {  488,  487,  605092,  605104,  605130, 12, 26, 20, }, // Chatham
    {  488,  495,  605150,  605160,  605184, 10, 24, 18, }, // Chile
    {  488,  501,  605202,  605212,  605236, 10, 24, 18, }, // China
    {  488,  507,       0,  605254,       0,  0, 28,  0, }, // Christmas
    {  488,  517,       0,  605282,       0,  0, 19,  0, }, // Cocos
    {  488,  523,  605301,  605314,  605341, 13, 27, 21, }, // Colombia
    {  488,  532,  605362,  605378,  605408, 16, 30, 24, }, // Cook
    {  488,  537,  605432,  605441,  605464,  9, 23, 17, }, // Cuba
    {  488,  548,       0,  605481,       0,  0, 10,  0, }, // Davis
    {  488,  564,       0,  605491,       0,  0, 21,  0, }, // DumontDUrville
    {  488,  601,       0,  605512,       0,  0, 19,  0, }, // East_Timor
    {  488,  612,  605531,  605553,  605589, 22, 36, 30, }, // Easter
    {  488,  619,       0,  605619,       0,  0, 12,  0, }, // Ecuador
    {  488,  627,  605631,  605654,  605689, 23, 35, 29, }, // Europe_Central
    {  488,  642,  605718,  605742,  605778, 24, 36, 30, }, // Europe_Eastern
    {  488,  657,       0,  605808,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  488,  680,  605839,  605863,  605899, 24, 36, 30, }, // Europe_Western
    {  488,  695,  605929,  605943,  605971, 14, 28, 22, }, // Falkland
    {  488,  704,  605993,  606003,  606027, 10, 24, 18, }, // Fiji
    {  488,  709,       0,  606045,       0,  0, 21,  0, }, // French_Guiana
    {  488,  723,       0,  606066,       0,  0, 51,  0, }, // French_Southern
    {  488,  746,       0,  606117,       0,  0, 14,  0, }, // Galapagos
    {  488,  756,       0,  606131,       0,  0, 19,  0, }, // Gambier
    {  488,  764,  606150,  606161,  606186, 11, 25, 19, }, // Georgia
    {  488,  772,       0,  606205,       0,  0, 19,  0, }, // Gilbert_Islands
    {  488,  788,       0,  606224,       0,  0, 16,  0, }, // GMT
    {  488,  830,  606240,  606265,  606304, 25, 39, 33, }, // Greenland_Eastern
    {  488,  848,  606337,  606362,  606401, 25, 39, 33, }, // Greenland_Western
    {  488,  871,       0,  606434,       0,  0, 18,  0, }, // Gulf
    {  488,  876,       0,  606045,       0,  0, 11,  0, }, // Guyana
    {  488,  883,  606452,  606470,  606502, 18, 32, 26, }, // Hawaii_Aleutian
    {  488,  899,  606528,  606541,  606568, 13, 27, 21, }, // Hong_Kong
    {  488,  909,  606589,  606599,  606623, 10, 24, 18, }, // Hovd
    {  488,  914,       0,  606641,       0,  0, 25,  0, }, // India
    {  488,  920,       0,  606666,       0,  0, 19,  0, }, // Indian_Ocean
    {  488,  933,       0,  606685,       0,  0, 16,  0, }, // Indochina
    {  488,  943,       0,  606701,       0,  0, 23,  0, }, // Indonesia_Central
    {  488,  961,       0,  606724,       0,  0, 24,  0, }, // Indonesia_Eastern
    {  488,  979,       0,  606748,       0,  0, 24,  0, }, // Indonesia_Western
    {  488,  997,  606772,  606781,  606804,  9, 23, 17, }, // Iran
    {  488, 1008,  606821,  606832,  606857, 11, 25, 19, }, // Irkutsk
    {  488, 1016,  606876,  606887,  606912, 11, 25, 19, }, // Israel
    {  488, 1023,  606931,  606943,  606969, 12, 26, 20, }, // Japan
    {  488, 1029,  606989,  607018,  607059, 29, 41, 35, }, // Kamchatka
    {  488, 1047,       0,  607094,       0,  0, 15,  0, }, // Kazakhstan
    {  488, 1058,       0,  607109,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  488, 1077,       0,  607133,       0,  0, 24,  0, }, // Kazakhstan_Western
    {  488, 1106,  607157,  607167,  607191, 10, 24, 18, }, // Korea
    {  488, 1112,       0,  607209,       0,  0, 11,  0, }, // Kosrae
    {  488, 1119,  607220,  607236,  607266, 16, 30, 24, }, // Krasnoyarsk
    {  488, 1151,       0,  607290,       0,  0, 14,  0, }, // Kyrgystan
    {  488, 1175,       0,  607304,       0,  0, 17,  0, }, // Line_Islands
    {  488, 1188,  607321,  607335,  607363, 14, 28, 22, }, // Lord_Howe
    {  488, 1204,  607385,  607397,  607423, 12, 26, 20, }, // Magadan
    {  488, 1219,       0,  607443,       0,  0, 12,  0, }, // Malaysia
    {  488, 1228,       0,  607455,       0,  0, 13,  0, }, // Maldives
    {  488, 1237,       0,  607468,       0,  0, 12,  0, }, // Marquesas
    {  488, 1247,       0,  607480,       0,  0, 20,  0, }, // Marshall_Islands
    {  488, 1264,  607500,  607514,  607542, 14, 28, 22, }, // Mauritius
    {  488, 1274,       0,  607564,       0,  0, 11,  0, }, // Mawson
    {  488, 1281,  607575,  607599,  607635, 24, 36, 30, }, // Mexico_Pacific
    {  488, 1296,  607665,  607680,  607709, 15, 29, 23, }, // Mongolia
    {  488, 1305,  607732,  607743,  607768, 11, 25, 19, }, // Moscow
    {  488, 1312,       0,  607787,       0,  0, 11,  0, }, // Myanmar
    {  488, 1320,       0,  607798,       0,  0, 10,  0, }, // Nauru
    {  488, 1326,       0,  607808,       0,  0, 10,  0, }, // Nepal
    {  488, 1332,  607818,  607837,  607870, 19, 33, 27, }, // New_Caledonia
    {  488, 1346,  607897,  607915,  607947, 18, 32, 26, }, // New_Zealand
    {  488, 1358,  607973,  607993,  608027, 20, 34, 28, }, // Newfoundland
    {  488, 1371,       0,  608055,       0,  0,  9,  0, }, // Niue
    {  488, 1376,  608064,  608076,  608102, 12, 26, 20, }, // Norfolk
    {  488, 1384,  608122,  608146,  608184, 24, 38, 32, }, // Noronha
    {  488, 1406,  608216,  608232,  608262, 16, 30, 24, }, // Novosibirsk
    {  488, 1418,  608286,  608295,  608318,  9, 23, 17, }, // Omsk
    {  488, 1428,  608335,  608348,  608375, 13, 27, 21, }, // Pakistan
    {  488, 1437,       0,  608396,       0,  0, 10,  0, }, // Palau
    {  488, 1443,       0,  608406,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  488, 1460,  608428,  608441,  608468, 13, 27, 21, }, // Paraguay
    {  488, 1469,  608489,  608498,  608521,  9, 23, 17, }, // Peru
    {  488, 1474,  608538,  608551,  608578, 13, 27, 21, }, // Philippines
    {  488, 1486,       0,  608599,       0,  0, 11,  0, }, // Phoenix_Islands
    {  488, 1502,  608610,  608638,  608680, 28, 42, 36, }, // Pierre_Miquelon
    {  488, 1518,       0,  608716,       0,  0, 13,  0, }, // Pitcairn
    {  488, 1527,       0,  608729,       0,  0, 12,  0, }, // Ponape
    {  488, 1534,       0,  608741,       0,  0, 14,  0, }, // Pyongyang
    {  488, 1544,  608755,  608771,  608799, 16, 28, 22, }, // Qyzylorda
    {  488, 1554,       0,  608821,       0,  0, 12,  0, }, // Reunion
    {  488, 1562,       0,  608833,       0,  0, 12,  0, }, // Rothera
    {  488, 1570,  608845,  608858,  608885, 13, 27, 21, }, // Sakhalin
    {  488, 1579,  608906,  608917,  608940, 11, 23, 17, }, // Samara
    {  488, 1596,  608957,  608967,  608991, 10, 24, 18, }, // Samoa
    {  488, 1602,       0,  609009,       0,  0, 12,  0, }, // Seychelles
    {  488, 1624,       0,  609021,       0,  0, 13,  0, }, // Singapore
    {  488, 1634,       0,  609034,       0,  0, 19,  0, }, // Solomon
    {  488, 1642,       0,  609053,       0,  0, 23,  0, }, // South_Georgia
    {  488, 1656,       0,  609076,       0,  0, 12,  0, }, // Suriname
    {  488, 1676,       0,  609088,       0,  0, 10,  0, }, // Syowa
    {  488, 1682,       0,  609098,       0,  0, 11,  0, }, // Tahiti
    {  488, 1689,  609109,  609120,  609145, 11, 25, 19, }, // Taipei
    {  488, 1696,       0,  609164,       0,  0, 16,  0, }, // Tajikistan
    {  488, 1724,       0,  609180,       0,  0, 12,  0, }, // Tokelau
    {  488, 1732,  609192,  609202,  609226, 10, 24, 18, }, // Tonga
    {  488, 1738,       0,  609244,       0,  0, 10,  0, }, // Truk
    {  488, 1750,  609254,  609271,  609302, 17, 31, 25, }, // Turkmenistan
    {  488, 1763,       0,  609327,       0,  0, 11,  0, }, // Tuvalu
    {  488, 1777,  609338,  609350,  609376, 12, 26, 20, }, // Uruguay
    {  488, 1792,  609396,  609411,  609440, 15, 29, 23, }, // Uzbekistan
    {  488, 1803,  609463,  609475,  609501, 12, 26, 20, }, // Vanuatu
    {  488, 1811,       0,  609521,       0,  0, 14,  0, }, // Venezuela
    {  488, 1821,  609535,  609551,  609581, 16, 30, 24, }, // Vladivostok
    {  488, 1833,  609605,  609619,  609647, 14, 28, 22, }, // Volgograd
    {  488, 1843,       0,  609669,       0,  0, 11,  0, }, // Vostok
    {  488, 1850,       0,  609680,       0,  0,  9,  0, }, // Wake
    {  488, 1855,       0,  609689,       0,  0, 20,  0, }, // Wallis
    {  488, 1862,  609709,  609720,  609745, 11, 25, 19, }, // Yakutsk
    {  488, 1870,  609764,  609782,  609814, 18, 32, 26, }, // Yekaterinburg
    {  488, 1892,       0,  609840,       0,  0, 10,  0, }, // Yukon
    {  489,    0,  609850,  609865,  609887, 15, 22, 24, }, // Acre Portuguese/Latin/Brazil
    {  489,    5,       0,  609911,       0,  0, 22,  0, }, // Afghanistan
    {  489,   17,       0,  609933,       0,  0, 25,  0, }, // Africa_Central
    {  489,   32,       0,  609958,       0,  0, 26,  0, }, // Africa_Eastern
    {  489,   65,       0,  609984,       0,  0, 24,  0, }, // Africa_Southern
    {  489,   81,  610008,  610035,  610069, 27, 34, 36, }, // Africa_Western
    {  489,  107,  610105,  610122,  610146, 17, 24, 26, }, // Alaska
    {  489,  128,  610172,  610189,  610213, 17, 24, 26, }, // Almaty
    {  489,  135,  610239,  610258,  610284, 19, 26, 28, }, // Amazon
    {  489,  142,  610312,  610327,  610349, 15, 22, 24, }, // America_Central
    {  489,  158,  610373,  610389,  610412, 16, 23, 25, }, // America_Eastern
    {  489,  174,  610437,  610458,  610486, 21, 28, 30, }, // America_Mountain
    {  489,  191,  610516,  610535,  610561, 19, 26, 28, }, // America_Pacific
    {  489,  207,  610589,  610606,  610630, 17, 24, 26, }, // Anadyr
    {  489,  214,  610656,  610671,  610693, 15, 22, 24, }, // Apia
    {  489,  219,  610717,  610733,  610756, 16, 23, 25, }, // Aqtau
    {  489,  225,  610781,  610798,  610822, 17, 24, 26, }, // Aqtobe
    {  489,  232,  610848,  610865,  610889, 17, 24, 26, }, // Arabian
    {  489,  240,  610915,  610935,  610962, 20, 27, 29, }, // Argentina
    {  489,  250,  610991,  611021,  611058, 30, 37, 39, }, // Argentina_Western
    {  489,  268,  611097,  611115,  611140, 18, 25, 27, }, // Armenia
    {  489,  286,  611167,  611187,  611214, 20, 27, 29, }, // Atlantic
    {  489,  295,  611243,  611271,  611306, 28, 35, 37, }, // Australia_Central
    {  489,  313,  611343,  611380,  611424, 37, 44, 46, }, // Australia_CentralWestern
    {  489,  338,  611470,  611499,  611535, 29, 36, 38, }, // Australia_Eastern
    {  489,  356,  611573,  611603,  611640, 30, 37, 39, }, // Australia_Western
    {  489,  374,  611679,  611701,  611730, 22, 29, 31, }, // Azerbaijan
    {  489,  385,  611761,  611779,  611804, 18, 25, 27, }, // Azores
    {  489,  397,  611831,  611852,  611880, 21, 28, 30, }, // Bangladesh
    {  489,  415,       0,  611910,       0,  0, 16,  0, }, // Bhutan
    {  489,  422,       0,  611926,       0,  0, 18,  0, }, // Bolivia
    {  489,  437,  611944,  611963,  611989, 19, 26, 28, }, // Brasilia
    {  489,  454,       0,  612017,       0,  0, 28,  0, }, // Brunei
    {  489,  461,  612045,  612066,  612094, 21, 28, 30, }, // Cape_Verde
    {  489,  478,       0,  612124,       0,  0, 19,  0, }, // Chamorro
    {  489,  487,  612143,  612161,  612186, 18, 25, 27, }, // Chatham
    {  489,  495,  612213,  612229,  612252, 16, 23, 25, }, // Chile
    {  489,  501,  612277,  612293,  612316, 16, 23, 25, }, // China
    {  489,  507,       0,  612341,       0,  0, 25,  0, }, // Christmas
    {  489,  517,       0,  612366,       0,  0, 22,  0, }, // Cocos
    {  489,  523,  612388,  612407,  612433, 19, 26, 28, }, // Colombia
    {  489,  532,  612461,  612483,  612512, 22, 29, 36, }, // Cook
    {  489,  537,  612548,  612563,  612585, 15, 22, 24, }, // Cuba
    {  489,  548,       0,  612609,       0,  0, 16,  0, }, // Davis
    {  489,  564,       0,  612625,       0,  0, 27,  0, }, // DumontDUrville
    {  489,  601,       0,  612652,       0,  0, 22,  0, }, // East_Timor
    {  489,  612,  612674,  612699,  612731, 25, 32, 34, }, // Easter
    {  489,  619,       0,  612765,       0,  0, 18,  0, }, // Ecuador
    {  489,  627,  612783,  612808,  612840, 25, 32, 34, }, // Europe_Central
    {  489,  642,  612874,  612900,  612933, 26, 33, 35, }, // Europe_Eastern
    {  489,  657,       0,  612968,       0,  0, 32,  0, }, // Europe_Further_Eastern
    {  489,  680,  613000,  613027,  613061, 27, 34, 36, }, // Europe_Western
    {  489,  695,  613097,  613123,  613156, 26, 33, 35, }, // Falkland
    {  489,  704,  613191,  613206,  613228, 15, 22, 24, }, // Fiji
    {  489,  709,       0,  613252,       0,  0, 26,  0, }, // French_Guiana
    {  489,  723,       0,  613278,       0,  0, 52,  0, }, // French_Southern
    {  489,  746,       0,  613330,       0,  0, 20,  0, }, // Galapagos
    {  489,  756,       0,  613350,       0,  0, 18,  0, }, // Gambier
    {  489,  764,  613368,  613386,  613411, 18, 25, 27, }, // Georgia
    {  489,  772,       0,  613438,       0,  0, 26,  0, }, // Gilbert_Islands
    {  489,  788,       0,  613464,       0,  0, 33,  0, }, // GMT
    {  489,  830,  613497,  613527,  613564, 30, 37, 39, }, // Greenland_Eastern
    {  489,  848,  613603,  613635,  613674, 32, 39, 41, }, // Greenland_Western
    {  489,  866,       0,  613715,       0,  0, 22,  0, }, // Guam
    {  489,  871,       0,  613737,       0,  0, 16,  0, }, // Gulf
    {  489,  876,       0,  613252,       0,  0, 17,  0, }, // Guyana
    {  489,  883,  613753,  613785,  613824, 32, 39, 41, }, // Hawaii_Aleutian
    {  489,  899,  613865,  613885,  613912, 20, 27, 29, }, // Hong_Kong
    {  489,  909,  613941,  613956,  613978, 15, 22, 24, }, // Hovd
    {  489,  914,       0,  614002,       0,  0, 23,  0, }, // India
    {  489,  920,       0,  614025,       0,  0, 24,  0, }, // Indian_Ocean
    {  489,  933,       0,  614049,       0,  0, 20,  0, }, // Indochina
    {  489,  943,       0,  614069,       0,  0, 28,  0, }, // Indonesia_Central
    {  489,  961,       0,  614097,       0,  0, 29,  0, }, // Indonesia_Eastern
    {  489,  979,       0,  614126,       0,  0, 30,  0, }, // Indonesia_Western
    {  489,  997,  614156,  614170,  614191, 14, 21, 23, }, // Iran
    {  489, 1008,  614214,  614232,  614257, 18, 25, 27, }, // Irkutsk
    {  489, 1016,  614284,  614301,  614325, 17, 24, 26, }, // Israel
    {  489, 1023,  614351,  614367,  614390, 16, 23, 25, }, // Japan
    {  489, 1029,  614415,  614450,  614492, 35, 42, 44, }, // Kamchatka
    {  489, 1047,       0,  614536,       0,  0, 22,  0, }, // Kazakhstan
    {  489, 1058,       0,  614558,       0,  0, 31,  0, }, // Kazakhstan_Eastern
    {  489, 1077,       0,  614589,       0,  0, 32,  0, }, // Kazakhstan_Western
    {  489, 1106,  614621,  614638,  614662, 17, 24, 26, }, // Korea
    {  489, 1112,       0,  614688,       0,  0, 17,  0, }, // Kosrae
    {  489, 1119,  614705,  614727,  614756, 22, 29, 31, }, // Krasnoyarsk
    {  489, 1151,       0,  614787,       0,  0, 22,  0, }, // Kyrgystan
    {  489, 1161,       0,  614809,       0,  0, 16,  0, }, // Lanka
    {  489, 1175,       0,  614825,       0,  0, 26,  0, }, // Line_Islands
    {  489, 1188,  614851,  614871,  614898, 20, 27, 29, }, // Lord_Howe
    {  489, 1198,  614927,  614943,  614966, 16, 23, 25, }, // Macau
    {  489, 1204,  614991,  615009,  615034, 18, 25, 27, }, // Magadan
    {  489, 1219,       0,  615061,       0,  0, 18,  0, }, // Malaysia
    {  489, 1228,       0,  615079,       0,  0, 26,  0, }, // Maldives
    {  489, 1237,       0,  615105,       0,  0, 21,  0, }, // Marquesas
    {  489, 1247,       0,  615126,       0,  0, 26,  0, }, // Marshall_Islands
    {  489, 1264,  615152,  615171,  615197, 19, 26, 28, }, // Mauritius
    {  489, 1274,       0,  615225,       0,  0, 17,  0, }, // Mawson
    {  489, 1281,  615242,  615270,  615305, 28, 35, 37, }, // Mexico_Pacific
    {  489, 1296,  615342,  615363,  615391, 21, 28, 30, }, // Mongolia
    {  489, 1305,  615421,  615438,  615462, 17, 24, 26, }, // Moscow
    {  489, 1312,       0,  615488,       0,  0, 18,  0, }, // Myanmar
    {  489, 1320,       0,  615506,       0,  0, 16,  0, }, // Nauru
    {  489, 1326,       0,  615522,       0,  0, 16,  0, }, // Nepal
    {  489, 1332,  615538,  615563,  615595, 25, 32, 34, }, // New_Caledonia
    {  489, 1346,  615629,  615653,  615684, 24, 31, 33, }, // New_Zealand
    {  489, 1358,  615717,  615738,  615766, 21, 28, 30, }, // Newfoundland
    {  489, 1371,       0,  615796,       0,  0, 15,  0, }, // Niue
    {  489, 1376,  615811,  615834,  615864, 23, 30, 32, }, // Norfolk
    {  489, 1384,  615896,  615926,  615963, 30, 37, 39, }, // Noronha
    {  489, 1392,       0,  616002,       0,  0, 34,  0, }, // North_Mariana
    {  489, 1406,  616036,  616058,  616087, 22, 29, 31, }, // Novosibirsk
    {  489, 1418,  616118,  616133,  616155, 15, 22, 24, }, // Omsk
    {  489, 1428,  616179,  616199,  616226, 20, 27, 29, }, // Pakistan
    {  489, 1437,       0,  616255,       0,  0, 16,  0, }, // Palau
    {  489, 1443,       0,  616271,       0,  0, 27,  0, }, // Papua_New_Guinea
    {  489, 1460,  616298,  616317,  616343, 19, 26, 28, }, // Paraguay
    {  489, 1469,  616371,  616386,  616408, 15, 22, 24, }, // Peru
    {  489, 1474,  616432,  616453,  616481, 21, 28, 30, }, // Philippines
    {  489, 1486,       0,  616511,       0,  0, 23,  0, }, // Phoenix_Islands
    {  489, 1502,  616534,  616565,  616603, 31, 38, 37, }, // Pierre_Miquelon
    {  489, 1518,       0,  616640,       0,  0, 19,  0, }, // Pitcairn
    {  489, 1527,       0,  616659,       0,  0, 17,  0, }, // Ponape
    {  489, 1534,       0,  616676,       0,  0, 20,  0, }, // Pyongyang
    {  489, 1544,  616696,  616716,  616743, 20, 27, 29, }, // Qyzylorda
    {  489, 1554,       0,  616772,       0,  0, 18,  0, }, // Reunion
    {  489, 1562,       0,  616790,       0,  0, 18,  0, }, // Rothera
    {  489, 1570,  616808,  616827,  616853, 19, 26, 28, }, // Sakhalin
    {  489, 1579,  616881,  616898,  616922, 17, 24, 26, }, // Samara
    {  489, 1596,  616948,  616964,  616987, 16, 23, 25, }, // Samoa
    {  489, 1602,       0,  617012,       0,  0, 20,  0, }, // Seychelles
    {  489, 1624,       0,  617032,       0,  0, 27,  0, }, // Singapore
    {  489, 1634,       0,  617059,       0,  0, 25,  0, }, // Solomon
    {  489, 1642,       0,  617084,       0,  0, 25,  0, }, // South_Georgia
    {  489, 1656,       0,  617109,       0,  0, 19,  0, }, // Suriname
    {  489, 1676,       0,  617128,       0,  0, 16,  0, }, // Syowa
    {  489, 1682,       0,  617144,       0,  0, 16,  0, }, // Tahiti
    {  489, 1689,  617160,  617177,  617201, 17, 24, 26, }, // Taipei
    {  489, 1696,       0,  617227,       0,  0, 22,  0, }, // Tajikistan
    {  489, 1724,       0,  617249,       0,  0, 18,  0, }, // Tokelau
    {  489, 1732,  617267,  617283,  617306, 16, 23, 25, }, // Tonga
    {  489, 1738,       0,  617331,       0,  0, 16,  0, }, // Truk
    {  489, 1750,  617347,  617371,  617402, 24, 31, 33, }, // Turkmenistan
    {  489, 1763,       0,  617435,       0,  0, 17,  0, }, // Tuvalu
    {  489, 1777,  617452,  617470,  617495, 18, 25, 27, }, // Uruguay
    {  489, 1792,  617522,  617544,  617573, 22, 29, 31, }, // Uzbekistan
    {  489, 1803,  617604,  617622,  617647, 18, 25, 27, }, // Vanuatu
    {  489, 1811,       0,  617674,       0,  0, 20,  0, }, // Venezuela
    {  489, 1821,  617694,  617716,  617745, 22, 29, 31, }, // Vladivostok
    {  489, 1833,  617776,  617797,  617825, 21, 28, 30, }, // Volgograd
    {  489, 1843,       0,  617855,       0,  0, 17,  0, }, // Vostok
    {  489, 1850,       0,  617872,       0,  0, 22,  0, }, // Wake
    {  489, 1855,       0,  617894,       0,  0, 26,  0, }, // Wallis
    {  489, 1862,  617920,  617938,  617963, 18, 25, 27, }, // Yakutsk
    {  489, 1870,  617990,  618014,  618045, 24, 31, 33, }, // Yekaterinburg
    {  489, 1892,       0,  618078,       0,  0, 16,  0, }, // Yukon
    {  490,    0,  618094,  618106,  618125, 12, 19, 21, }, // Acre Portuguese/Latin/Angola
    {  490,    5,       0,  618146,       0,  0, 19,  0, }, // Afghanistan
    {  490,   17,       0,  618165,       0,  0, 22,  0, }, // Africa_Central
    {  490,   32,       0,  618187,       0,  0, 23,  0, }, // Africa_Eastern
    {  490,   65,       0,  618210,       0,  0, 21,  0, }, // Africa_Southern
    {  490,   81,  618231,  618255,  618286, 24, 31, 33, }, // Africa_Western
    {  490,  107,  618319,  618333,  618354, 14, 21, 23, }, // Alaska
    {  490,  128,  618377,  618391,  618412, 14, 21, 23, }, // Almaty
    {  490,  135,  618435,  618451,  618474, 16, 23, 25, }, // Amazon
    {  490,  142,  618499,  618527,  618562, 28, 35, 37, }, // America_Central
    {  490,  158,  618599,  618628,  618664, 29, 36, 38, }, // America_Eastern
    {  490,  174,  618702,  618734,  618773, 32, 39, 41, }, // America_Mountain
    {  490,  191,  618814,  618846,  618885, 32, 39, 41, }, // America_Pacific
    {  490,  207,  618926,  618940,  618961, 14, 21, 23, }, // Anadyr
    {  490,  214,  618984,  618996,  619015, 12, 19, 21, }, // Apia
    {  490,  219,  619036,  619049,  619069, 13, 20, 22, }, // Aqtau
    {  490,  225,  619091,  619105,  619126, 14, 21, 23, }, // Aqtobe
    {  490,  232,  619149,  619163,  619184, 14, 21, 23, }, // Arabian
    {  490,  240,  619207,  619224,  619248, 17, 24, 26, }, // Argentina
    {  490,  250,  619274,  619301,  619335, 27, 34, 36, }, // Argentina_Western
    {  490,  268,  619371,  619386,  619408, 15, 22, 24, }, // Armenia
    {  490,  286,  619432,  619449,  619473, 17, 24, 26, }, // Atlantic
    {  490,  295,  619499,  619524,  619556, 25, 32, 34, }, // Australia_Central
    {  490,  313,  619590,  619625,  619667, 35, 42, 44, }, // Australia_CentralWestern
    {  490,  338,  619711,  619737,  619770, 26, 33, 35, }, // Australia_Eastern
    {  490,  356,  619805,  619832,  619866, 27, 34, 36, }, // Australia_Western
    {  490,  374,  619902,  619920,  619945, 18, 25, 27, }, // Azerbaijan
    {  490,  385,  619972,  619987,  620009, 15, 22, 24, }, // Azores
    {  490,  397,  620033,  620052,  620078, 19, 26, 28, }, // Bangladesh
    {  490,  415,       0,  620106,       0,  0, 13,  0, }, // Bhutan
    {  490,  422,       0,  620119,       0,  0, 15,  0, }, // Bolivia
    {  490,  437,  139301,  620134,  620157, 16, 23, 25, }, // Brasilia
    {  490,  454,       0,  620182,       0,  0, 25,  0, }, // Brunei
    {  490,  461,  620207,  620225,  620250, 18, 25, 27, }, // Cape_Verde
    {  490,  478,       0,  620277,       0,  0, 23,  0, }, // Chamorro
    {  490,  487,   59218,  620300,  620322, 15, 22, 24, }, // Chatham
    {  490,  495,  620346,  620359,  620379, 13, 20, 22, }, // Chile
    {  490,  501,  620401,  620414,  620434, 13, 20, 22, }, // China
    {  490,  507,       0,  620456,       0,  0, 21,  0, }, // Christmas
    {  490,  517,       0,  620477,       0,  0, 20,  0, }, // Cocos
    {  490,  523,  620497,  620513,  620536, 16, 23, 25, }, // Colombia
    {  490,  532,  620561,  620580,  620606, 19, 26, 28, }, // Cook
    {  490,  537,   59613,  620634,  620653, 12, 19, 21, }, // Cuba
    {  490,  548,       0,   59667,       0,  0, 13,  0, }, // Davis
    {  490,  564,       0,   59680,       0,  0, 24,  0, }, // DumontDUrville
    {  490,  601,       0,  620674,       0,  0, 19,  0, }, // East_Timor
    {  490,  612,  620693,  620715,  620744, 22, 29, 31, }, // Easter
    {  490,  619,       0,  620775,       0,  0, 15,  0, }, // Ecuador
    {  490,  627,  620790,  620812,  620841, 22, 29, 31, }, // Europe_Central
    {  490,  642,  620872,  620895,  620925, 23, 30, 32, }, // Europe_Eastern
    {  490,  657,       0,  620957,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  490,  680,  620988,  621012,  621043, 24, 31, 33, }, // Europe_Western
    {  490,  695,  621076,  621099,  621129, 23, 30, 32, }, // Falkland
    {  490,  704,  140403,  621161,  621180, 12, 19, 21, }, // Fiji
    {  490,  709,       0,  621201,       0,  0, 23,  0, }, // French_Guiana
    {  490,  723,       0,  621224,       0,  0, 48,  0, }, // French_Southern
    {  490,  746,       0,  621272,       0,  0, 18,  0, }, // Galapagos
    {  490,  756,       0,   60351,       0,  0, 15,  0, }, // Gambier
    {  490,  764,  621290,  621305,  621327, 15, 22, 24, }, // Georgia
    {  490,  772,       0,  621351,       0,  0, 22,  0, }, // Gilbert_Islands
    {  490,  788,       0,  621373,       0,  0, 17,  0, }, // GMT
    {  490,  830,  621390,  621418,  621453, 28, 35, 37, }, // Greenland_Eastern
    {  490,  848,  621490,  621519,  621555, 29, 36, 38, }, // Greenland_Western
    {  490,  866,       0,  621593,       0,  0, 19,  0, }, // Guam
    {  490,  871,       0,  621612,       0,  0, 20,  0, }, // Gulf
    {  490,  876,       0,  621201,       0,  0, 14,  0, }, // Guyana
    {  490,  883,  621632,  621655,  621685, 23, 30, 32, }, // Hawaii_Aleutian
    {  490,  899,  141051,  621717,  621741, 17, 24, 26, }, // Hong_Kong
    {  490,  909,   60891,  621767,  621786, 12, 19, 21, }, // Hovd
    {  490,  914,       0,  621807,       0,  0, 20,  0, }, // India
    {  490,  920,       0,  621827,       0,  0, 21,  0, }, // Indian_Ocean
    {  490,  933,       0,  621848,       0,  0, 17,  0, }, // Indochina
    {  490,  943,       0,  621865,       0,  0, 25,  0, }, // Indonesia_Central
    {  490,  961,       0,  621890,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  490,  979,       0,  621916,       0,  0, 27,  0, }, // Indonesia_Western
    {  490,  997,  621943,  621955,  621974, 12, 19, 21, }, // Iran
    {  490, 1008,  621995,  622010,  622032, 15, 22, 24, }, // Irkutsk
    {  490, 1016,  622056,  622070,  622091, 14, 21, 23, }, // Israel
    {  490, 1023,  622114,  622127,  622147, 13, 20, 22, }, // Japan
    {  490, 1029,  622169,  622201,  622240, 32, 39, 41, }, // Kamchatka
    {  490, 1047,       0,  622281,       0,  0, 19,  0, }, // Kazakhstan
    {  490, 1058,       0,  622300,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  490, 1077,       0,  622328,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  490, 1106,  622357,  622371,  622392, 14, 21, 23, }, // Korea
    {  490, 1112,       0,   61535,       0,  0, 14,  0, }, // Kosrae
    {  490, 1119,   61549,  622415,  622441, 19, 26, 28, }, // Krasnoyarsk
    {  490, 1151,       0,  622469,       0,  0, 19,  0, }, // Kyrgystan
    {  490, 1161,       0,  622488,       0,  0, 17,  0, }, // Lanka
    {  490, 1175,       0,  622505,       0,  0, 19,  0, }, // Line_Islands
    {  490, 1188,   61679,  622524,  622548, 17, 24, 26, }, // Lord_Howe
    {  490, 1198,  141908,  622574,  622594, 13, 20, 22, }, // Macau
    {  490, 1204,  141965,  622616,  622638, 15, 22, 24, }, // Magadan
    {  490, 1219,       0,  622662,       0,  0, 15,  0, }, // Malaysia
    {  490, 1228,       0,  622677,       0,  0, 17,  0, }, // Maldives
    {  490, 1237,       0,  622694,       0,  0, 24,  0, }, // Marquesas
    {  490, 1247,       0,  622718,       0,  0, 23,  0, }, // Marshall_Islands
    {  490, 1264,  622741,  622757,  622780, 16, 23, 25, }, // Mauritius
    {  490, 1274,       0,   62017,       0,  0, 14,  0, }, // Mawson
    {  490, 1281,  622805,  622830,  622862, 25, 32, 34, }, // Mexico_Pacific
    {  490, 1296,  622896,  622914,  622939, 18, 25, 27, }, // Mongolia
    {  490, 1305,  622966,  622981,  623003, 15, 22, 24, }, // Moscow
    {  490, 1312,       0,  623027,       0,  0, 15,  0, }, // Myanmar
    {  490, 1320,       0,   62271,       0,  0, 13,  0, }, // Nauru
    {  490, 1326,       0,  623042,       0,  0, 13,  0, }, // Nepal
    {  490, 1332,  623055,  623077,  623106, 22, 29, 31, }, // New_Caledonia
    {  490, 1346,  623137,  623158,  623186, 21, 28, 30, }, // New_Zealand
    {  490, 1358,  623216,  623234,  623259, 18, 25, 27, }, // Newfoundland
    {  490, 1371,       0,  623286,       0,  0, 12,  0, }, // Niue
    {  490, 1376,  623298,  623318,  623345, 20, 27, 29, }, // Norfolk
    {  490, 1384,   62646,  623374,  623408, 27, 34, 36, }, // Noronha
    {  490, 1392,       0,  623444,       0,  0, 31,  0, }, // North_Mariana
    {  490, 1406,   62782,  623475,  623501, 19, 26, 28, }, // Novosibirsk
    {  490, 1418,  623529,  623541,  623560, 12, 19, 21, }, // Omsk
    {  490, 1428,  623581,  623598,  623622, 17, 24, 26, }, // Pakistan
    {  490, 1437,       0,   62980,       0,  0, 13,  0, }, // Palau
    {  490, 1443,       0,  623648,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  490, 1460,  623672,  623688,  623711, 16, 23, 25, }, // Paraguay
    {  490, 1469,  623736,  623748,  623767, 12, 19, 21, }, // Peru
    {  490, 1474,  623788,  623806,  623831, 18, 25, 27, }, // Philippines
    {  490, 1486,       0,  623858,       0,  0, 20,  0, }, // Phoenix_Islands
    {  490, 1502,  623878,  623906,  623941, 28, 35, 37, }, // Pierre_Miquelon
    {  490, 1518,       0,   63351,       0,  0, 16,  0, }, // Pitcairn
    {  490, 1527,       0,   63367,       0,  0, 14,  0, }, // Ponape
    {  490, 1534,       0,  143295,       0,  0, 17,  0, }, // Pyongyang
    {  490, 1544,   63398,  623978,  624002, 17, 24, 26, }, // Qyzylorda
    {  490, 1554,       0,  624028,       0,  0, 15,  0, }, // Reunion
    {  490, 1562,       0,   63482,       0,  0, 15,  0, }, // Rothera
    {  490, 1570,  624043,  624059,  624082, 16, 23, 25, }, // Sakhalin
    {  490, 1579,   63560,  624107,  624128, 14, 21, 23, }, // Samara
    {  490, 1596,   63620,  624151,  624171, 13, 20, 22, }, // Samoa
    {  490, 1602,       0,  624193,       0,  0, 18,  0, }, // Seychelles
    {  490, 1624,       0,  624211,       0,  0, 24,  0, }, // Singapore
    {  490, 1634,       0,  624235,       0,  0, 22,  0, }, // Solomon
    {  490, 1642,       0,  624257,       0,  0, 22,  0, }, // South_Georgia
    {  490, 1656,       0,  624279,       0,  0, 16,  0, }, // Suriname
    {  490, 1676,       0,   63786,       0,  0, 13,  0, }, // Syowa
    {  490, 1682,       0,  624295,       0,  0, 13,  0, }, // Tahiti
    {  490, 1689,   63813,  624308,  624328, 13, 20, 22, }, // Taipei
    {  490, 1696,       0,  624350,       0,  0, 19,  0, }, // Tajikistan
    {  490, 1724,       0,   63893,       0,  0, 15,  0, }, // Tokelau
    {  490, 1732,   63908,  624369,  624389, 13, 20, 22, }, // Tonga
    {  490, 1738,       0,   63965,       0,  0, 13,  0, }, // Truk
    {  490, 1750,  624411,  624433,  624462, 22, 29, 31, }, // Turkmenistan
    {  490, 1763,       0,   64059,       0,  0, 14,  0, }, // Tuvalu
    {  490, 1777,  624493,  624508,  624530, 15, 22, 24, }, // Uruguay
    {  490, 1792,  624554,  624573,  624599, 19, 26, 28, }, // Uzbekistan
    {  490, 1803,  624627,  624642,  624664, 15, 22, 24, }, // Vanuatu
    {  490, 1811,       0,  624688,       0,  0, 17,  0, }, // Venezuela
    {  490, 1821,   64297,  624705,  624731, 19, 26, 28, }, // Vladivostok
    {  490, 1833,  624759,  624777,  624802, 18, 25, 27, }, // Volgograd
    {  490, 1843,       0,   64441,       0,  0, 14,  0, }, // Vostok
    {  490, 1850,       0,  624829,       0,  0, 17,  0, }, // Wake
    {  490, 1855,       0,  624846,       0,  0, 23,  0, }, // Wallis
    {  490, 1862,   64499,  624869,  624891, 15, 22, 24, }, // Yakutsk
    {  490, 1870,  624915,  624936,  624964, 21, 28, 30, }, // Yekaterinburg
    {  490, 1892,       0,  624994,       0,  0, 13,  0, }, // Yukon
    {  491,    0,  618094,  618106,  618125, 12, 19, 21, }, // Acre Portuguese/Latin/Cape Verde
    {  491,    5,       0,  618146,       0,  0, 19,  0, }, // Afghanistan
    {  491,   17,       0,  618165,       0,  0, 22,  0, }, // Africa_Central
    {  491,   32,       0,  618187,       0,  0, 23,  0, }, // Africa_Eastern
    {  491,   65,       0,  618210,       0,  0, 21,  0, }, // Africa_Southern
    {  491,   81,  618231,  618255,  618286, 24, 31, 33, }, // Africa_Western
    {  491,  107,  618319,  618333,  618354, 14, 21, 23, }, // Alaska
    {  491,  128,  618377,  618391,  618412, 14, 21, 23, }, // Almaty
    {  491,  135,  618435,  618451,  618474, 16, 23, 25, }, // Amazon
    {  491,  142,  618499,  618527,  618562, 28, 35, 37, }, // America_Central
    {  491,  158,  618599,  618628,  618664, 29, 36, 38, }, // America_Eastern
    {  491,  174,  618702,  618734,  618773, 32, 39, 41, }, // America_Mountain
    {  491,  191,  618814,  618846,  618885, 32, 39, 41, }, // America_Pacific
    {  491,  207,  618926,  618940,  618961, 14, 21, 23, }, // Anadyr
    {  491,  214,  618984,  618996,  619015, 12, 19, 21, }, // Apia
    {  491,  219,  619036,  619049,  619069, 13, 20, 22, }, // Aqtau
    {  491,  225,  619091,  619105,  619126, 14, 21, 23, }, // Aqtobe
    {  491,  232,  619149,  619163,  619184, 14, 21, 23, }, // Arabian
    {  491,  240,  619207,  619224,  619248, 17, 24, 26, }, // Argentina
    {  491,  250,  619274,  619301,  619335, 27, 34, 36, }, // Argentina_Western
    {  491,  268,  619371,  619386,  619408, 15, 22, 24, }, // Armenia
    {  491,  286,  619432,  619449,  619473, 17, 24, 26, }, // Atlantic
    {  491,  295,  619499,  619524,  619556, 25, 32, 34, }, // Australia_Central
    {  491,  313,  619590,  619625,  619667, 35, 42, 44, }, // Australia_CentralWestern
    {  491,  338,  619711,  619737,  619770, 26, 33, 35, }, // Australia_Eastern
    {  491,  356,  619805,  619832,  619866, 27, 34, 36, }, // Australia_Western
    {  491,  374,  619902,  619920,  619945, 18, 25, 27, }, // Azerbaijan
    {  491,  385,  619972,  619987,  620009, 15, 22, 24, }, // Azores
    {  491,  397,  620033,  620052,  620078, 19, 26, 28, }, // Bangladesh
    {  491,  415,       0,  620106,       0,  0, 13,  0, }, // Bhutan
    {  491,  422,       0,  620119,       0,  0, 15,  0, }, // Bolivia
    {  491,  437,  139301,  620134,  620157, 16, 23, 25, }, // Brasilia
    {  491,  454,       0,  620182,       0,  0, 25,  0, }, // Brunei
    {  491,  461,  620207,  620225,  620250, 18, 25, 27, }, // Cape_Verde
    {  491,  478,       0,  620277,       0,  0, 23,  0, }, // Chamorro
    {  491,  487,   59218,  620300,  620322, 15, 22, 24, }, // Chatham
    {  491,  495,  620346,  620359,  620379, 13, 20, 22, }, // Chile
    {  491,  501,  620401,  620414,  620434, 13, 20, 22, }, // China
    {  491,  507,       0,  620456,       0,  0, 21,  0, }, // Christmas
    {  491,  517,       0,  620477,       0,  0, 20,  0, }, // Cocos
    {  491,  523,  620497,  620513,  620536, 16, 23, 25, }, // Colombia
    {  491,  532,  620561,  620580,  620606, 19, 26, 28, }, // Cook
    {  491,  537,   59613,  620634,  620653, 12, 19, 21, }, // Cuba
    {  491,  548,       0,   59667,       0,  0, 13,  0, }, // Davis
    {  491,  564,       0,   59680,       0,  0, 24,  0, }, // DumontDUrville
    {  491,  601,       0,  620674,       0,  0, 19,  0, }, // East_Timor
    {  491,  612,  620693,  620715,  620744, 22, 29, 31, }, // Easter
    {  491,  619,       0,  620775,       0,  0, 15,  0, }, // Ecuador
    {  491,  627,  620790,  620812,  620841, 22, 29, 31, }, // Europe_Central
    {  491,  642,  620872,  620895,  620925, 23, 30, 32, }, // Europe_Eastern
    {  491,  657,       0,  620957,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  491,  680,  620988,  621012,  621043, 24, 31, 33, }, // Europe_Western
    {  491,  695,  621076,  621099,  621129, 23, 30, 32, }, // Falkland
    {  491,  704,  140403,  621161,  621180, 12, 19, 21, }, // Fiji
    {  491,  709,       0,  621201,       0,  0, 23,  0, }, // French_Guiana
    {  491,  723,       0,  621224,       0,  0, 48,  0, }, // French_Southern
    {  491,  746,       0,  621272,       0,  0, 18,  0, }, // Galapagos
    {  491,  756,       0,   60351,       0,  0, 15,  0, }, // Gambier
    {  491,  764,  621290,  621305,  621327, 15, 22, 24, }, // Georgia
    {  491,  772,       0,  621351,       0,  0, 22,  0, }, // Gilbert_Islands
    {  491,  788,       0,  621373,       0,  0, 17,  0, }, // GMT
    {  491,  830,  621390,  621418,  621453, 28, 35, 37, }, // Greenland_Eastern
    {  491,  848,  621490,  621519,  621555, 29, 36, 38, }, // Greenland_Western
    {  491,  866,       0,  621593,       0,  0, 19,  0, }, // Guam
    {  491,  871,       0,  621612,       0,  0, 20,  0, }, // Gulf
    {  491,  876,       0,  621201,       0,  0, 14,  0, }, // Guyana
    {  491,  883,  621632,  621655,  621685, 23, 30, 32, }, // Hawaii_Aleutian
    {  491,  899,  141051,  621717,  621741, 17, 24, 26, }, // Hong_Kong
    {  491,  909,   60891,  621767,  621786, 12, 19, 21, }, // Hovd
    {  491,  914,       0,  621807,       0,  0, 20,  0, }, // India
    {  491,  920,       0,  621827,       0,  0, 21,  0, }, // Indian_Ocean
    {  491,  933,       0,  621848,       0,  0, 17,  0, }, // Indochina
    {  491,  943,       0,  621865,       0,  0, 25,  0, }, // Indonesia_Central
    {  491,  961,       0,  621890,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  491,  979,       0,  621916,       0,  0, 27,  0, }, // Indonesia_Western
    {  491,  997,  621943,  621955,  621974, 12, 19, 21, }, // Iran
    {  491, 1008,  621995,  622010,  622032, 15, 22, 24, }, // Irkutsk
    {  491, 1016,  622056,  622070,  622091, 14, 21, 23, }, // Israel
    {  491, 1023,  622114,  622127,  622147, 13, 20, 22, }, // Japan
    {  491, 1029,  622169,  622201,  622240, 32, 39, 41, }, // Kamchatka
    {  491, 1047,       0,  622281,       0,  0, 19,  0, }, // Kazakhstan
    {  491, 1058,       0,  622300,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  491, 1077,       0,  622328,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  491, 1106,  622357,  622371,  622392, 14, 21, 23, }, // Korea
    {  491, 1112,       0,   61535,       0,  0, 14,  0, }, // Kosrae
    {  491, 1119,   61549,  622415,  622441, 19, 26, 28, }, // Krasnoyarsk
    {  491, 1151,       0,  622469,       0,  0, 19,  0, }, // Kyrgystan
    {  491, 1161,       0,  622488,       0,  0, 17,  0, }, // Lanka
    {  491, 1175,       0,  622505,       0,  0, 19,  0, }, // Line_Islands
    {  491, 1188,   61679,  622524,  622548, 17, 24, 26, }, // Lord_Howe
    {  491, 1198,  141908,  622574,  622594, 13, 20, 22, }, // Macau
    {  491, 1204,  141965,  622616,  622638, 15, 22, 24, }, // Magadan
    {  491, 1219,       0,  622662,       0,  0, 15,  0, }, // Malaysia
    {  491, 1228,       0,  622677,       0,  0, 17,  0, }, // Maldives
    {  491, 1237,       0,  622694,       0,  0, 24,  0, }, // Marquesas
    {  491, 1247,       0,  622718,       0,  0, 23,  0, }, // Marshall_Islands
    {  491, 1264,  622741,  622757,  622780, 16, 23, 25, }, // Mauritius
    {  491, 1274,       0,   62017,       0,  0, 14,  0, }, // Mawson
    {  491, 1281,  622805,  622830,  622862, 25, 32, 34, }, // Mexico_Pacific
    {  491, 1296,  622896,  622914,  622939, 18, 25, 27, }, // Mongolia
    {  491, 1305,  622966,  622981,  623003, 15, 22, 24, }, // Moscow
    {  491, 1312,       0,  623027,       0,  0, 15,  0, }, // Myanmar
    {  491, 1320,       0,   62271,       0,  0, 13,  0, }, // Nauru
    {  491, 1326,       0,  623042,       0,  0, 13,  0, }, // Nepal
    {  491, 1332,  623055,  623077,  623106, 22, 29, 31, }, // New_Caledonia
    {  491, 1346,  623137,  623158,  623186, 21, 28, 30, }, // New_Zealand
    {  491, 1358,  623216,  623234,  623259, 18, 25, 27, }, // Newfoundland
    {  491, 1371,       0,  623286,       0,  0, 12,  0, }, // Niue
    {  491, 1376,  623298,  623318,  623345, 20, 27, 29, }, // Norfolk
    {  491, 1384,   62646,  623374,  623408, 27, 34, 36, }, // Noronha
    {  491, 1392,       0,  623444,       0,  0, 31,  0, }, // North_Mariana
    {  491, 1406,   62782,  623475,  623501, 19, 26, 28, }, // Novosibirsk
    {  491, 1418,  623529,  623541,  623560, 12, 19, 21, }, // Omsk
    {  491, 1428,  623581,  623598,  623622, 17, 24, 26, }, // Pakistan
    {  491, 1437,       0,   62980,       0,  0, 13,  0, }, // Palau
    {  491, 1443,       0,  623648,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  491, 1460,  623672,  623688,  623711, 16, 23, 25, }, // Paraguay
    {  491, 1469,  623736,  623748,  623767, 12, 19, 21, }, // Peru
    {  491, 1474,  623788,  623806,  623831, 18, 25, 27, }, // Philippines
    {  491, 1486,       0,  623858,       0,  0, 20,  0, }, // Phoenix_Islands
    {  491, 1502,  623878,  623906,  623941, 28, 35, 37, }, // Pierre_Miquelon
    {  491, 1518,       0,   63351,       0,  0, 16,  0, }, // Pitcairn
    {  491, 1527,       0,   63367,       0,  0, 14,  0, }, // Ponape
    {  491, 1534,       0,  143295,       0,  0, 17,  0, }, // Pyongyang
    {  491, 1544,   63398,  623978,  624002, 17, 24, 26, }, // Qyzylorda
    {  491, 1554,       0,  624028,       0,  0, 15,  0, }, // Reunion
    {  491, 1562,       0,   63482,       0,  0, 15,  0, }, // Rothera
    {  491, 1570,  624043,  624059,  624082, 16, 23, 25, }, // Sakhalin
    {  491, 1579,   63560,  624107,  624128, 14, 21, 23, }, // Samara
    {  491, 1596,   63620,  624151,  624171, 13, 20, 22, }, // Samoa
    {  491, 1602,       0,  624193,       0,  0, 18,  0, }, // Seychelles
    {  491, 1624,       0,  624211,       0,  0, 24,  0, }, // Singapore
    {  491, 1634,       0,  624235,       0,  0, 22,  0, }, // Solomon
    {  491, 1642,       0,  624257,       0,  0, 22,  0, }, // South_Georgia
    {  491, 1656,       0,  624279,       0,  0, 16,  0, }, // Suriname
    {  491, 1676,       0,   63786,       0,  0, 13,  0, }, // Syowa
    {  491, 1682,       0,  624295,       0,  0, 13,  0, }, // Tahiti
    {  491, 1689,   63813,  624308,  624328, 13, 20, 22, }, // Taipei
    {  491, 1696,       0,  624350,       0,  0, 19,  0, }, // Tajikistan
    {  491, 1724,       0,   63893,       0,  0, 15,  0, }, // Tokelau
    {  491, 1732,   63908,  624369,  624389, 13, 20, 22, }, // Tonga
    {  491, 1738,       0,   63965,       0,  0, 13,  0, }, // Truk
    {  491, 1750,  624411,  624433,  624462, 22, 29, 31, }, // Turkmenistan
    {  491, 1763,       0,   64059,       0,  0, 14,  0, }, // Tuvalu
    {  491, 1777,  624493,  624508,  624530, 15, 22, 24, }, // Uruguay
    {  491, 1792,  624554,  624573,  624599, 19, 26, 28, }, // Uzbekistan
    {  491, 1803,  624627,  624642,  624664, 15, 22, 24, }, // Vanuatu
    {  491, 1811,       0,  624688,       0,  0, 17,  0, }, // Venezuela
    {  491, 1821,   64297,  624705,  624731, 19, 26, 28, }, // Vladivostok
    {  491, 1833,  624759,  624777,  624802, 18, 25, 27, }, // Volgograd
    {  491, 1843,       0,   64441,       0,  0, 14,  0, }, // Vostok
    {  491, 1850,       0,  624829,       0,  0, 17,  0, }, // Wake
    {  491, 1855,       0,  624846,       0,  0, 23,  0, }, // Wallis
    {  491, 1862,   64499,  624869,  624891, 15, 22, 24, }, // Yakutsk
    {  491, 1870,  624915,  624936,  624964, 21, 28, 30, }, // Yekaterinburg
    {  491, 1892,       0,  624994,       0,  0, 13,  0, }, // Yukon
    {  492,    0,  618094,  618106,  618125, 12, 19, 21, }, // Acre Portuguese/Latin/Equatorial Guinea
    {  492,    5,       0,  618146,       0,  0, 19,  0, }, // Afghanistan
    {  492,   17,       0,  618165,       0,  0, 22,  0, }, // Africa_Central
    {  492,   32,       0,  618187,       0,  0, 23,  0, }, // Africa_Eastern
    {  492,   65,       0,  618210,       0,  0, 21,  0, }, // Africa_Southern
    {  492,   81,  618231,  618255,  618286, 24, 31, 33, }, // Africa_Western
    {  492,  107,  618319,  618333,  618354, 14, 21, 23, }, // Alaska
    {  492,  128,  618377,  618391,  618412, 14, 21, 23, }, // Almaty
    {  492,  135,  618435,  618451,  618474, 16, 23, 25, }, // Amazon
    {  492,  142,  618499,  618527,  618562, 28, 35, 37, }, // America_Central
    {  492,  158,  618599,  618628,  618664, 29, 36, 38, }, // America_Eastern
    {  492,  174,  618702,  618734,  618773, 32, 39, 41, }, // America_Mountain
    {  492,  191,  618814,  618846,  618885, 32, 39, 41, }, // America_Pacific
    {  492,  207,  618926,  618940,  618961, 14, 21, 23, }, // Anadyr
    {  492,  214,  618984,  618996,  619015, 12, 19, 21, }, // Apia
    {  492,  219,  619036,  619049,  619069, 13, 20, 22, }, // Aqtau
    {  492,  225,  619091,  619105,  619126, 14, 21, 23, }, // Aqtobe
    {  492,  232,  619149,  619163,  619184, 14, 21, 23, }, // Arabian
    {  492,  240,  619207,  619224,  619248, 17, 24, 26, }, // Argentina
    {  492,  250,  619274,  619301,  619335, 27, 34, 36, }, // Argentina_Western
    {  492,  268,  619371,  619386,  619408, 15, 22, 24, }, // Armenia
    {  492,  286,  619432,  619449,  619473, 17, 24, 26, }, // Atlantic
    {  492,  295,  619499,  619524,  619556, 25, 32, 34, }, // Australia_Central
    {  492,  313,  619590,  619625,  619667, 35, 42, 44, }, // Australia_CentralWestern
    {  492,  338,  619711,  619737,  619770, 26, 33, 35, }, // Australia_Eastern
    {  492,  356,  619805,  619832,  619866, 27, 34, 36, }, // Australia_Western
    {  492,  374,  619902,  619920,  619945, 18, 25, 27, }, // Azerbaijan
    {  492,  385,  619972,  619987,  620009, 15, 22, 24, }, // Azores
    {  492,  397,  620033,  620052,  620078, 19, 26, 28, }, // Bangladesh
    {  492,  415,       0,  620106,       0,  0, 13,  0, }, // Bhutan
    {  492,  422,       0,  620119,       0,  0, 15,  0, }, // Bolivia
    {  492,  437,  139301,  620134,  620157, 16, 23, 25, }, // Brasilia
    {  492,  454,       0,  620182,       0,  0, 25,  0, }, // Brunei
    {  492,  461,  620207,  620225,  620250, 18, 25, 27, }, // Cape_Verde
    {  492,  478,       0,  620277,       0,  0, 23,  0, }, // Chamorro
    {  492,  487,   59218,  620300,  620322, 15, 22, 24, }, // Chatham
    {  492,  495,  620346,  620359,  620379, 13, 20, 22, }, // Chile
    {  492,  501,  620401,  620414,  620434, 13, 20, 22, }, // China
    {  492,  507,       0,  620456,       0,  0, 21,  0, }, // Christmas
    {  492,  517,       0,  620477,       0,  0, 20,  0, }, // Cocos
    {  492,  523,  620497,  620513,  620536, 16, 23, 25, }, // Colombia
    {  492,  532,  620561,  620580,  620606, 19, 26, 28, }, // Cook
    {  492,  537,   59613,  620634,  620653, 12, 19, 21, }, // Cuba
    {  492,  548,       0,   59667,       0,  0, 13,  0, }, // Davis
    {  492,  564,       0,   59680,       0,  0, 24,  0, }, // DumontDUrville
    {  492,  601,       0,  620674,       0,  0, 19,  0, }, // East_Timor
    {  492,  612,  620693,  620715,  620744, 22, 29, 31, }, // Easter
    {  492,  619,       0,  620775,       0,  0, 15,  0, }, // Ecuador
    {  492,  627,  620790,  620812,  620841, 22, 29, 31, }, // Europe_Central
    {  492,  642,  620872,  620895,  620925, 23, 30, 32, }, // Europe_Eastern
    {  492,  657,       0,  620957,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  492,  680,  620988,  621012,  621043, 24, 31, 33, }, // Europe_Western
    {  492,  695,  621076,  621099,  621129, 23, 30, 32, }, // Falkland
    {  492,  704,  140403,  621161,  621180, 12, 19, 21, }, // Fiji
    {  492,  709,       0,  621201,       0,  0, 23,  0, }, // French_Guiana
    {  492,  723,       0,  621224,       0,  0, 48,  0, }, // French_Southern
    {  492,  746,       0,  621272,       0,  0, 18,  0, }, // Galapagos
    {  492,  756,       0,   60351,       0,  0, 15,  0, }, // Gambier
    {  492,  764,  621290,  621305,  621327, 15, 22, 24, }, // Georgia
    {  492,  772,       0,  621351,       0,  0, 22,  0, }, // Gilbert_Islands
    {  492,  788,       0,  621373,       0,  0, 17,  0, }, // GMT
    {  492,  830,  621390,  621418,  621453, 28, 35, 37, }, // Greenland_Eastern
    {  492,  848,  621490,  621519,  621555, 29, 36, 38, }, // Greenland_Western
    {  492,  866,       0,  621593,       0,  0, 19,  0, }, // Guam
    {  492,  871,       0,  621612,       0,  0, 20,  0, }, // Gulf
    {  492,  876,       0,  621201,       0,  0, 14,  0, }, // Guyana
    {  492,  883,  621632,  621655,  621685, 23, 30, 32, }, // Hawaii_Aleutian
    {  492,  899,  141051,  621717,  621741, 17, 24, 26, }, // Hong_Kong
    {  492,  909,   60891,  621767,  621786, 12, 19, 21, }, // Hovd
    {  492,  914,       0,  621807,       0,  0, 20,  0, }, // India
    {  492,  920,       0,  621827,       0,  0, 21,  0, }, // Indian_Ocean
    {  492,  933,       0,  621848,       0,  0, 17,  0, }, // Indochina
    {  492,  943,       0,  621865,       0,  0, 25,  0, }, // Indonesia_Central
    {  492,  961,       0,  621890,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  492,  979,       0,  621916,       0,  0, 27,  0, }, // Indonesia_Western
    {  492,  997,  621943,  621955,  621974, 12, 19, 21, }, // Iran
    {  492, 1008,  621995,  622010,  622032, 15, 22, 24, }, // Irkutsk
    {  492, 1016,  622056,  622070,  622091, 14, 21, 23, }, // Israel
    {  492, 1023,  622114,  622127,  622147, 13, 20, 22, }, // Japan
    {  492, 1029,  622169,  622201,  622240, 32, 39, 41, }, // Kamchatka
    {  492, 1047,       0,  622281,       0,  0, 19,  0, }, // Kazakhstan
    {  492, 1058,       0,  622300,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  492, 1077,       0,  622328,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  492, 1106,  622357,  622371,  622392, 14, 21, 23, }, // Korea
    {  492, 1112,       0,   61535,       0,  0, 14,  0, }, // Kosrae
    {  492, 1119,   61549,  622415,  622441, 19, 26, 28, }, // Krasnoyarsk
    {  492, 1151,       0,  622469,       0,  0, 19,  0, }, // Kyrgystan
    {  492, 1161,       0,  622488,       0,  0, 17,  0, }, // Lanka
    {  492, 1175,       0,  622505,       0,  0, 19,  0, }, // Line_Islands
    {  492, 1188,   61679,  622524,  622548, 17, 24, 26, }, // Lord_Howe
    {  492, 1198,  141908,  622574,  622594, 13, 20, 22, }, // Macau
    {  492, 1204,  141965,  622616,  622638, 15, 22, 24, }, // Magadan
    {  492, 1219,       0,  622662,       0,  0, 15,  0, }, // Malaysia
    {  492, 1228,       0,  622677,       0,  0, 17,  0, }, // Maldives
    {  492, 1237,       0,  622694,       0,  0, 24,  0, }, // Marquesas
    {  492, 1247,       0,  622718,       0,  0, 23,  0, }, // Marshall_Islands
    {  492, 1264,  622741,  622757,  622780, 16, 23, 25, }, // Mauritius
    {  492, 1274,       0,   62017,       0,  0, 14,  0, }, // Mawson
    {  492, 1281,  622805,  622830,  622862, 25, 32, 34, }, // Mexico_Pacific
    {  492, 1296,  622896,  622914,  622939, 18, 25, 27, }, // Mongolia
    {  492, 1305,  622966,  622981,  623003, 15, 22, 24, }, // Moscow
    {  492, 1312,       0,  623027,       0,  0, 15,  0, }, // Myanmar
    {  492, 1320,       0,   62271,       0,  0, 13,  0, }, // Nauru
    {  492, 1326,       0,  623042,       0,  0, 13,  0, }, // Nepal
    {  492, 1332,  623055,  623077,  623106, 22, 29, 31, }, // New_Caledonia
    {  492, 1346,  623137,  623158,  623186, 21, 28, 30, }, // New_Zealand
    {  492, 1358,  623216,  623234,  623259, 18, 25, 27, }, // Newfoundland
    {  492, 1371,       0,  623286,       0,  0, 12,  0, }, // Niue
    {  492, 1376,  623298,  623318,  623345, 20, 27, 29, }, // Norfolk
    {  492, 1384,   62646,  623374,  623408, 27, 34, 36, }, // Noronha
    {  492, 1392,       0,  623444,       0,  0, 31,  0, }, // North_Mariana
    {  492, 1406,   62782,  623475,  623501, 19, 26, 28, }, // Novosibirsk
    {  492, 1418,  623529,  623541,  623560, 12, 19, 21, }, // Omsk
    {  492, 1428,  623581,  623598,  623622, 17, 24, 26, }, // Pakistan
    {  492, 1437,       0,   62980,       0,  0, 13,  0, }, // Palau
    {  492, 1443,       0,  623648,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  492, 1460,  623672,  623688,  623711, 16, 23, 25, }, // Paraguay
    {  492, 1469,  623736,  623748,  623767, 12, 19, 21, }, // Peru
    {  492, 1474,  623788,  623806,  623831, 18, 25, 27, }, // Philippines
    {  492, 1486,       0,  623858,       0,  0, 20,  0, }, // Phoenix_Islands
    {  492, 1502,  623878,  623906,  623941, 28, 35, 37, }, // Pierre_Miquelon
    {  492, 1518,       0,   63351,       0,  0, 16,  0, }, // Pitcairn
    {  492, 1527,       0,   63367,       0,  0, 14,  0, }, // Ponape
    {  492, 1534,       0,  143295,       0,  0, 17,  0, }, // Pyongyang
    {  492, 1544,   63398,  623978,  624002, 17, 24, 26, }, // Qyzylorda
    {  492, 1554,       0,  624028,       0,  0, 15,  0, }, // Reunion
    {  492, 1562,       0,   63482,       0,  0, 15,  0, }, // Rothera
    {  492, 1570,  624043,  624059,  624082, 16, 23, 25, }, // Sakhalin
    {  492, 1579,   63560,  624107,  624128, 14, 21, 23, }, // Samara
    {  492, 1596,   63620,  624151,  624171, 13, 20, 22, }, // Samoa
    {  492, 1602,       0,  624193,       0,  0, 18,  0, }, // Seychelles
    {  492, 1624,       0,  624211,       0,  0, 24,  0, }, // Singapore
    {  492, 1634,       0,  624235,       0,  0, 22,  0, }, // Solomon
    {  492, 1642,       0,  624257,       0,  0, 22,  0, }, // South_Georgia
    {  492, 1656,       0,  624279,       0,  0, 16,  0, }, // Suriname
    {  492, 1676,       0,   63786,       0,  0, 13,  0, }, // Syowa
    {  492, 1682,       0,  624295,       0,  0, 13,  0, }, // Tahiti
    {  492, 1689,   63813,  624308,  624328, 13, 20, 22, }, // Taipei
    {  492, 1696,       0,  624350,       0,  0, 19,  0, }, // Tajikistan
    {  492, 1724,       0,   63893,       0,  0, 15,  0, }, // Tokelau
    {  492, 1732,   63908,  624369,  624389, 13, 20, 22, }, // Tonga
    {  492, 1738,       0,   63965,       0,  0, 13,  0, }, // Truk
    {  492, 1750,  624411,  624433,  624462, 22, 29, 31, }, // Turkmenistan
    {  492, 1763,       0,   64059,       0,  0, 14,  0, }, // Tuvalu
    {  492, 1777,  624493,  624508,  624530, 15, 22, 24, }, // Uruguay
    {  492, 1792,  624554,  624573,  624599, 19, 26, 28, }, // Uzbekistan
    {  492, 1803,  624627,  624642,  624664, 15, 22, 24, }, // Vanuatu
    {  492, 1811,       0,  624688,       0,  0, 17,  0, }, // Venezuela
    {  492, 1821,   64297,  624705,  624731, 19, 26, 28, }, // Vladivostok
    {  492, 1833,  624759,  624777,  624802, 18, 25, 27, }, // Volgograd
    {  492, 1843,       0,   64441,       0,  0, 14,  0, }, // Vostok
    {  492, 1850,       0,  624829,       0,  0, 17,  0, }, // Wake
    {  492, 1855,       0,  624846,       0,  0, 23,  0, }, // Wallis
    {  492, 1862,   64499,  624869,  624891, 15, 22, 24, }, // Yakutsk
    {  492, 1870,  624915,  624936,  624964, 21, 28, 30, }, // Yekaterinburg
    {  492, 1892,       0,  624994,       0,  0, 13,  0, }, // Yukon
    {  493,    0,  618094,  618106,  618125, 12, 19, 21, }, // Acre Portuguese/Latin/Guinea-Bissau
    {  493,    5,       0,  618146,       0,  0, 19,  0, }, // Afghanistan
    {  493,   17,       0,  618165,       0,  0, 22,  0, }, // Africa_Central
    {  493,   32,       0,  618187,       0,  0, 23,  0, }, // Africa_Eastern
    {  493,   65,       0,  618210,       0,  0, 21,  0, }, // Africa_Southern
    {  493,   81,  618231,  618255,  618286, 24, 31, 33, }, // Africa_Western
    {  493,  107,  618319,  618333,  618354, 14, 21, 23, }, // Alaska
    {  493,  128,  618377,  618391,  618412, 14, 21, 23, }, // Almaty
    {  493,  135,  618435,  618451,  618474, 16, 23, 25, }, // Amazon
    {  493,  142,  618499,  618527,  618562, 28, 35, 37, }, // America_Central
    {  493,  158,  618599,  618628,  618664, 29, 36, 38, }, // America_Eastern
    {  493,  174,  618702,  618734,  618773, 32, 39, 41, }, // America_Mountain
    {  493,  191,  618814,  618846,  618885, 32, 39, 41, }, // America_Pacific
    {  493,  207,  618926,  618940,  618961, 14, 21, 23, }, // Anadyr
    {  493,  214,  618984,  618996,  619015, 12, 19, 21, }, // Apia
    {  493,  219,  619036,  619049,  619069, 13, 20, 22, }, // Aqtau
    {  493,  225,  619091,  619105,  619126, 14, 21, 23, }, // Aqtobe
    {  493,  232,  619149,  619163,  619184, 14, 21, 23, }, // Arabian
    {  493,  240,  619207,  619224,  619248, 17, 24, 26, }, // Argentina
    {  493,  250,  619274,  619301,  619335, 27, 34, 36, }, // Argentina_Western
    {  493,  268,  619371,  619386,  619408, 15, 22, 24, }, // Armenia
    {  493,  286,  619432,  619449,  619473, 17, 24, 26, }, // Atlantic
    {  493,  295,  619499,  619524,  619556, 25, 32, 34, }, // Australia_Central
    {  493,  313,  619590,  619625,  619667, 35, 42, 44, }, // Australia_CentralWestern
    {  493,  338,  619711,  619737,  619770, 26, 33, 35, }, // Australia_Eastern
    {  493,  356,  619805,  619832,  619866, 27, 34, 36, }, // Australia_Western
    {  493,  374,  619902,  619920,  619945, 18, 25, 27, }, // Azerbaijan
    {  493,  385,  619972,  619987,  620009, 15, 22, 24, }, // Azores
    {  493,  397,  620033,  620052,  620078, 19, 26, 28, }, // Bangladesh
    {  493,  415,       0,  620106,       0,  0, 13,  0, }, // Bhutan
    {  493,  422,       0,  620119,       0,  0, 15,  0, }, // Bolivia
    {  493,  437,  139301,  620134,  620157, 16, 23, 25, }, // Brasilia
    {  493,  454,       0,  620182,       0,  0, 25,  0, }, // Brunei
    {  493,  461,  620207,  620225,  620250, 18, 25, 27, }, // Cape_Verde
    {  493,  478,       0,  620277,       0,  0, 23,  0, }, // Chamorro
    {  493,  487,   59218,  620300,  620322, 15, 22, 24, }, // Chatham
    {  493,  495,  620346,  620359,  620379, 13, 20, 22, }, // Chile
    {  493,  501,  620401,  620414,  620434, 13, 20, 22, }, // China
    {  493,  507,       0,  620456,       0,  0, 21,  0, }, // Christmas
    {  493,  517,       0,  620477,       0,  0, 20,  0, }, // Cocos
    {  493,  523,  620497,  620513,  620536, 16, 23, 25, }, // Colombia
    {  493,  532,  620561,  620580,  620606, 19, 26, 28, }, // Cook
    {  493,  537,   59613,  620634,  620653, 12, 19, 21, }, // Cuba
    {  493,  548,       0,   59667,       0,  0, 13,  0, }, // Davis
    {  493,  564,       0,   59680,       0,  0, 24,  0, }, // DumontDUrville
    {  493,  601,       0,  620674,       0,  0, 19,  0, }, // East_Timor
    {  493,  612,  620693,  620715,  620744, 22, 29, 31, }, // Easter
    {  493,  619,       0,  620775,       0,  0, 15,  0, }, // Ecuador
    {  493,  627,  620790,  620812,  620841, 22, 29, 31, }, // Europe_Central
    {  493,  642,  620872,  620895,  620925, 23, 30, 32, }, // Europe_Eastern
    {  493,  657,       0,  620957,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  493,  680,  620988,  621012,  621043, 24, 31, 33, }, // Europe_Western
    {  493,  695,  621076,  621099,  621129, 23, 30, 32, }, // Falkland
    {  493,  704,  140403,  621161,  621180, 12, 19, 21, }, // Fiji
    {  493,  709,       0,  621201,       0,  0, 23,  0, }, // French_Guiana
    {  493,  723,       0,  621224,       0,  0, 48,  0, }, // French_Southern
    {  493,  746,       0,  621272,       0,  0, 18,  0, }, // Galapagos
    {  493,  756,       0,   60351,       0,  0, 15,  0, }, // Gambier
    {  493,  764,  621290,  621305,  621327, 15, 22, 24, }, // Georgia
    {  493,  772,       0,  621351,       0,  0, 22,  0, }, // Gilbert_Islands
    {  493,  788,       0,  621373,       0,  0, 17,  0, }, // GMT
    {  493,  830,  621390,  621418,  621453, 28, 35, 37, }, // Greenland_Eastern
    {  493,  848,  621490,  621519,  621555, 29, 36, 38, }, // Greenland_Western
    {  493,  866,       0,  621593,       0,  0, 19,  0, }, // Guam
    {  493,  871,       0,  621612,       0,  0, 20,  0, }, // Gulf
    {  493,  876,       0,  621201,       0,  0, 14,  0, }, // Guyana
    {  493,  883,  621632,  621655,  621685, 23, 30, 32, }, // Hawaii_Aleutian
    {  493,  899,  141051,  621717,  621741, 17, 24, 26, }, // Hong_Kong
    {  493,  909,   60891,  621767,  621786, 12, 19, 21, }, // Hovd
    {  493,  914,       0,  621807,       0,  0, 20,  0, }, // India
    {  493,  920,       0,  621827,       0,  0, 21,  0, }, // Indian_Ocean
    {  493,  933,       0,  621848,       0,  0, 17,  0, }, // Indochina
    {  493,  943,       0,  621865,       0,  0, 25,  0, }, // Indonesia_Central
    {  493,  961,       0,  621890,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  493,  979,       0,  621916,       0,  0, 27,  0, }, // Indonesia_Western
    {  493,  997,  621943,  621955,  621974, 12, 19, 21, }, // Iran
    {  493, 1008,  621995,  622010,  622032, 15, 22, 24, }, // Irkutsk
    {  493, 1016,  622056,  622070,  622091, 14, 21, 23, }, // Israel
    {  493, 1023,  622114,  622127,  622147, 13, 20, 22, }, // Japan
    {  493, 1029,  622169,  622201,  622240, 32, 39, 41, }, // Kamchatka
    {  493, 1047,       0,  622281,       0,  0, 19,  0, }, // Kazakhstan
    {  493, 1058,       0,  622300,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  493, 1077,       0,  622328,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  493, 1106,  622357,  622371,  622392, 14, 21, 23, }, // Korea
    {  493, 1112,       0,   61535,       0,  0, 14,  0, }, // Kosrae
    {  493, 1119,   61549,  622415,  622441, 19, 26, 28, }, // Krasnoyarsk
    {  493, 1151,       0,  622469,       0,  0, 19,  0, }, // Kyrgystan
    {  493, 1161,       0,  622488,       0,  0, 17,  0, }, // Lanka
    {  493, 1175,       0,  622505,       0,  0, 19,  0, }, // Line_Islands
    {  493, 1188,   61679,  622524,  622548, 17, 24, 26, }, // Lord_Howe
    {  493, 1198,  141908,  622574,  622594, 13, 20, 22, }, // Macau
    {  493, 1204,  141965,  622616,  622638, 15, 22, 24, }, // Magadan
    {  493, 1219,       0,  622662,       0,  0, 15,  0, }, // Malaysia
    {  493, 1228,       0,  622677,       0,  0, 17,  0, }, // Maldives
    {  493, 1237,       0,  622694,       0,  0, 24,  0, }, // Marquesas
    {  493, 1247,       0,  622718,       0,  0, 23,  0, }, // Marshall_Islands
    {  493, 1264,  622741,  622757,  622780, 16, 23, 25, }, // Mauritius
    {  493, 1274,       0,   62017,       0,  0, 14,  0, }, // Mawson
    {  493, 1281,  622805,  622830,  622862, 25, 32, 34, }, // Mexico_Pacific
    {  493, 1296,  622896,  622914,  622939, 18, 25, 27, }, // Mongolia
    {  493, 1305,  622966,  622981,  623003, 15, 22, 24, }, // Moscow
    {  493, 1312,       0,  623027,       0,  0, 15,  0, }, // Myanmar
    {  493, 1320,       0,   62271,       0,  0, 13,  0, }, // Nauru
    {  493, 1326,       0,  623042,       0,  0, 13,  0, }, // Nepal
    {  493, 1332,  623055,  623077,  623106, 22, 29, 31, }, // New_Caledonia
    {  493, 1346,  623137,  623158,  623186, 21, 28, 30, }, // New_Zealand
    {  493, 1358,  623216,  623234,  623259, 18, 25, 27, }, // Newfoundland
    {  493, 1371,       0,  623286,       0,  0, 12,  0, }, // Niue
    {  493, 1376,  623298,  623318,  623345, 20, 27, 29, }, // Norfolk
    {  493, 1384,   62646,  623374,  623408, 27, 34, 36, }, // Noronha
    {  493, 1392,       0,  623444,       0,  0, 31,  0, }, // North_Mariana
    {  493, 1406,   62782,  623475,  623501, 19, 26, 28, }, // Novosibirsk
    {  493, 1418,  623529,  623541,  623560, 12, 19, 21, }, // Omsk
    {  493, 1428,  623581,  623598,  623622, 17, 24, 26, }, // Pakistan
    {  493, 1437,       0,   62980,       0,  0, 13,  0, }, // Palau
    {  493, 1443,       0,  623648,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  493, 1460,  623672,  623688,  623711, 16, 23, 25, }, // Paraguay
    {  493, 1469,  623736,  623748,  623767, 12, 19, 21, }, // Peru
    {  493, 1474,  623788,  623806,  623831, 18, 25, 27, }, // Philippines
    {  493, 1486,       0,  623858,       0,  0, 20,  0, }, // Phoenix_Islands
    {  493, 1502,  623878,  623906,  623941, 28, 35, 37, }, // Pierre_Miquelon
    {  493, 1518,       0,   63351,       0,  0, 16,  0, }, // Pitcairn
    {  493, 1527,       0,   63367,       0,  0, 14,  0, }, // Ponape
    {  493, 1534,       0,  143295,       0,  0, 17,  0, }, // Pyongyang
    {  493, 1544,   63398,  623978,  624002, 17, 24, 26, }, // Qyzylorda
    {  493, 1554,       0,  624028,       0,  0, 15,  0, }, // Reunion
    {  493, 1562,       0,   63482,       0,  0, 15,  0, }, // Rothera
    {  493, 1570,  624043,  624059,  624082, 16, 23, 25, }, // Sakhalin
    {  493, 1579,   63560,  624107,  624128, 14, 21, 23, }, // Samara
    {  493, 1596,   63620,  624151,  624171, 13, 20, 22, }, // Samoa
    {  493, 1602,       0,  624193,       0,  0, 18,  0, }, // Seychelles
    {  493, 1624,       0,  624211,       0,  0, 24,  0, }, // Singapore
    {  493, 1634,       0,  624235,       0,  0, 22,  0, }, // Solomon
    {  493, 1642,       0,  624257,       0,  0, 22,  0, }, // South_Georgia
    {  493, 1656,       0,  624279,       0,  0, 16,  0, }, // Suriname
    {  493, 1676,       0,   63786,       0,  0, 13,  0, }, // Syowa
    {  493, 1682,       0,  624295,       0,  0, 13,  0, }, // Tahiti
    {  493, 1689,   63813,  624308,  624328, 13, 20, 22, }, // Taipei
    {  493, 1696,       0,  624350,       0,  0, 19,  0, }, // Tajikistan
    {  493, 1724,       0,   63893,       0,  0, 15,  0, }, // Tokelau
    {  493, 1732,   63908,  624369,  624389, 13, 20, 22, }, // Tonga
    {  493, 1738,       0,   63965,       0,  0, 13,  0, }, // Truk
    {  493, 1750,  624411,  624433,  624462, 22, 29, 31, }, // Turkmenistan
    {  493, 1763,       0,   64059,       0,  0, 14,  0, }, // Tuvalu
    {  493, 1777,  624493,  624508,  624530, 15, 22, 24, }, // Uruguay
    {  493, 1792,  624554,  624573,  624599, 19, 26, 28, }, // Uzbekistan
    {  493, 1803,  624627,  624642,  624664, 15, 22, 24, }, // Vanuatu
    {  493, 1811,       0,  624688,       0,  0, 17,  0, }, // Venezuela
    {  493, 1821,   64297,  624705,  624731, 19, 26, 28, }, // Vladivostok
    {  493, 1833,  624759,  624777,  624802, 18, 25, 27, }, // Volgograd
    {  493, 1843,       0,   64441,       0,  0, 14,  0, }, // Vostok
    {  493, 1850,       0,  624829,       0,  0, 17,  0, }, // Wake
    {  493, 1855,       0,  624846,       0,  0, 23,  0, }, // Wallis
    {  493, 1862,   64499,  624869,  624891, 15, 22, 24, }, // Yakutsk
    {  493, 1870,  624915,  624936,  624964, 21, 28, 30, }, // Yekaterinburg
    {  493, 1892,       0,  624994,       0,  0, 13,  0, }, // Yukon
    {  494,    0,  618094,  618106,  618125, 12, 19, 21, }, // Acre Portuguese/Latin/Luxembourg
    {  494,    5,       0,  618146,       0,  0, 19,  0, }, // Afghanistan
    {  494,   17,       0,  618165,       0,  0, 22,  0, }, // Africa_Central
    {  494,   32,       0,  618187,       0,  0, 23,  0, }, // Africa_Eastern
    {  494,   65,       0,  618210,       0,  0, 21,  0, }, // Africa_Southern
    {  494,   81,  618231,  618255,  618286, 24, 31, 33, }, // Africa_Western
    {  494,  107,  618319,  618333,  618354, 14, 21, 23, }, // Alaska
    {  494,  128,  618377,  618391,  618412, 14, 21, 23, }, // Almaty
    {  494,  135,  618435,  618451,  618474, 16, 23, 25, }, // Amazon
    {  494,  142,  618499,  618527,  618562, 28, 35, 37, }, // America_Central
    {  494,  158,  618599,  618628,  618664, 29, 36, 38, }, // America_Eastern
    {  494,  174,  618702,  618734,  618773, 32, 39, 41, }, // America_Mountain
    {  494,  191,  618814,  618846,  618885, 32, 39, 41, }, // America_Pacific
    {  494,  207,  618926,  618940,  618961, 14, 21, 23, }, // Anadyr
    {  494,  214,  618984,  618996,  619015, 12, 19, 21, }, // Apia
    {  494,  219,  619036,  619049,  619069, 13, 20, 22, }, // Aqtau
    {  494,  225,  619091,  619105,  619126, 14, 21, 23, }, // Aqtobe
    {  494,  232,  619149,  619163,  619184, 14, 21, 23, }, // Arabian
    {  494,  240,  619207,  619224,  619248, 17, 24, 26, }, // Argentina
    {  494,  250,  619274,  619301,  619335, 27, 34, 36, }, // Argentina_Western
    {  494,  268,  619371,  619386,  619408, 15, 22, 24, }, // Armenia
    {  494,  286,  619432,  619449,  619473, 17, 24, 26, }, // Atlantic
    {  494,  295,  619499,  619524,  619556, 25, 32, 34, }, // Australia_Central
    {  494,  313,  619590,  619625,  619667, 35, 42, 44, }, // Australia_CentralWestern
    {  494,  338,  619711,  619737,  619770, 26, 33, 35, }, // Australia_Eastern
    {  494,  356,  619805,  619832,  619866, 27, 34, 36, }, // Australia_Western
    {  494,  374,  619902,  619920,  619945, 18, 25, 27, }, // Azerbaijan
    {  494,  385,  619972,  619987,  620009, 15, 22, 24, }, // Azores
    {  494,  397,  620033,  620052,  620078, 19, 26, 28, }, // Bangladesh
    {  494,  415,       0,  620106,       0,  0, 13,  0, }, // Bhutan
    {  494,  422,       0,  620119,       0,  0, 15,  0, }, // Bolivia
    {  494,  437,  139301,  620134,  620157, 16, 23, 25, }, // Brasilia
    {  494,  454,       0,  620182,       0,  0, 25,  0, }, // Brunei
    {  494,  461,  620207,  620225,  620250, 18, 25, 27, }, // Cape_Verde
    {  494,  478,       0,  620277,       0,  0, 23,  0, }, // Chamorro
    {  494,  487,   59218,  620300,  620322, 15, 22, 24, }, // Chatham
    {  494,  495,  620346,  620359,  620379, 13, 20, 22, }, // Chile
    {  494,  501,  620401,  620414,  620434, 13, 20, 22, }, // China
    {  494,  507,       0,  620456,       0,  0, 21,  0, }, // Christmas
    {  494,  517,       0,  620477,       0,  0, 20,  0, }, // Cocos
    {  494,  523,  620497,  620513,  620536, 16, 23, 25, }, // Colombia
    {  494,  532,  620561,  620580,  620606, 19, 26, 28, }, // Cook
    {  494,  537,   59613,  620634,  620653, 12, 19, 21, }, // Cuba
    {  494,  548,       0,   59667,       0,  0, 13,  0, }, // Davis
    {  494,  564,       0,   59680,       0,  0, 24,  0, }, // DumontDUrville
    {  494,  601,       0,  620674,       0,  0, 19,  0, }, // East_Timor
    {  494,  612,  620693,  620715,  620744, 22, 29, 31, }, // Easter
    {  494,  619,       0,  620775,       0,  0, 15,  0, }, // Ecuador
    {  494,  627,  620790,  620812,  620841, 22, 29, 31, }, // Europe_Central
    {  494,  642,  620872,  620895,  620925, 23, 30, 32, }, // Europe_Eastern
    {  494,  657,       0,  620957,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  494,  680,  620988,  621012,  621043, 24, 31, 33, }, // Europe_Western
    {  494,  695,  621076,  621099,  621129, 23, 30, 32, }, // Falkland
    {  494,  704,  140403,  621161,  621180, 12, 19, 21, }, // Fiji
    {  494,  709,       0,  621201,       0,  0, 23,  0, }, // French_Guiana
    {  494,  723,       0,  621224,       0,  0, 48,  0, }, // French_Southern
    {  494,  746,       0,  621272,       0,  0, 18,  0, }, // Galapagos
    {  494,  756,       0,   60351,       0,  0, 15,  0, }, // Gambier
    {  494,  764,  621290,  621305,  621327, 15, 22, 24, }, // Georgia
    {  494,  772,       0,  621351,       0,  0, 22,  0, }, // Gilbert_Islands
    {  494,  788,       0,  621373,       0,  0, 17,  0, }, // GMT
    {  494,  830,  621390,  621418,  621453, 28, 35, 37, }, // Greenland_Eastern
    {  494,  848,  621490,  621519,  621555, 29, 36, 38, }, // Greenland_Western
    {  494,  866,       0,  621593,       0,  0, 19,  0, }, // Guam
    {  494,  871,       0,  621612,       0,  0, 20,  0, }, // Gulf
    {  494,  876,       0,  621201,       0,  0, 14,  0, }, // Guyana
    {  494,  883,  621632,  621655,  621685, 23, 30, 32, }, // Hawaii_Aleutian
    {  494,  899,  141051,  621717,  621741, 17, 24, 26, }, // Hong_Kong
    {  494,  909,   60891,  621767,  621786, 12, 19, 21, }, // Hovd
    {  494,  914,       0,  621807,       0,  0, 20,  0, }, // India
    {  494,  920,       0,  621827,       0,  0, 21,  0, }, // Indian_Ocean
    {  494,  933,       0,  621848,       0,  0, 17,  0, }, // Indochina
    {  494,  943,       0,  621865,       0,  0, 25,  0, }, // Indonesia_Central
    {  494,  961,       0,  621890,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  494,  979,       0,  621916,       0,  0, 27,  0, }, // Indonesia_Western
    {  494,  997,  621943,  621955,  621974, 12, 19, 21, }, // Iran
    {  494, 1008,  621995,  622010,  622032, 15, 22, 24, }, // Irkutsk
    {  494, 1016,  622056,  622070,  622091, 14, 21, 23, }, // Israel
    {  494, 1023,  622114,  622127,  622147, 13, 20, 22, }, // Japan
    {  494, 1029,  622169,  622201,  622240, 32, 39, 41, }, // Kamchatka
    {  494, 1047,       0,  622281,       0,  0, 19,  0, }, // Kazakhstan
    {  494, 1058,       0,  622300,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  494, 1077,       0,  622328,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  494, 1106,  622357,  622371,  622392, 14, 21, 23, }, // Korea
    {  494, 1112,       0,   61535,       0,  0, 14,  0, }, // Kosrae
    {  494, 1119,   61549,  622415,  622441, 19, 26, 28, }, // Krasnoyarsk
    {  494, 1151,       0,  622469,       0,  0, 19,  0, }, // Kyrgystan
    {  494, 1161,       0,  622488,       0,  0, 17,  0, }, // Lanka
    {  494, 1175,       0,  622505,       0,  0, 19,  0, }, // Line_Islands
    {  494, 1188,   61679,  622524,  622548, 17, 24, 26, }, // Lord_Howe
    {  494, 1198,  141908,  622574,  622594, 13, 20, 22, }, // Macau
    {  494, 1204,  141965,  622616,  622638, 15, 22, 24, }, // Magadan
    {  494, 1219,       0,  622662,       0,  0, 15,  0, }, // Malaysia
    {  494, 1228,       0,  622677,       0,  0, 17,  0, }, // Maldives
    {  494, 1237,       0,  622694,       0,  0, 24,  0, }, // Marquesas
    {  494, 1247,       0,  622718,       0,  0, 23,  0, }, // Marshall_Islands
    {  494, 1264,  622741,  622757,  622780, 16, 23, 25, }, // Mauritius
    {  494, 1274,       0,   62017,       0,  0, 14,  0, }, // Mawson
    {  494, 1281,  622805,  622830,  622862, 25, 32, 34, }, // Mexico_Pacific
    {  494, 1296,  622896,  622914,  622939, 18, 25, 27, }, // Mongolia
    {  494, 1305,  622966,  622981,  623003, 15, 22, 24, }, // Moscow
    {  494, 1312,       0,  623027,       0,  0, 15,  0, }, // Myanmar
    {  494, 1320,       0,   62271,       0,  0, 13,  0, }, // Nauru
    {  494, 1326,       0,  623042,       0,  0, 13,  0, }, // Nepal
    {  494, 1332,  623055,  623077,  623106, 22, 29, 31, }, // New_Caledonia
    {  494, 1346,  623137,  623158,  623186, 21, 28, 30, }, // New_Zealand
    {  494, 1358,  623216,  623234,  623259, 18, 25, 27, }, // Newfoundland
    {  494, 1371,       0,  623286,       0,  0, 12,  0, }, // Niue
    {  494, 1376,  623298,  623318,  623345, 20, 27, 29, }, // Norfolk
    {  494, 1384,   62646,  623374,  623408, 27, 34, 36, }, // Noronha
    {  494, 1392,       0,  623444,       0,  0, 31,  0, }, // North_Mariana
    {  494, 1406,   62782,  623475,  623501, 19, 26, 28, }, // Novosibirsk
    {  494, 1418,  623529,  623541,  623560, 12, 19, 21, }, // Omsk
    {  494, 1428,  623581,  623598,  623622, 17, 24, 26, }, // Pakistan
    {  494, 1437,       0,   62980,       0,  0, 13,  0, }, // Palau
    {  494, 1443,       0,  623648,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  494, 1460,  623672,  623688,  623711, 16, 23, 25, }, // Paraguay
    {  494, 1469,  623736,  623748,  623767, 12, 19, 21, }, // Peru
    {  494, 1474,  623788,  623806,  623831, 18, 25, 27, }, // Philippines
    {  494, 1486,       0,  623858,       0,  0, 20,  0, }, // Phoenix_Islands
    {  494, 1502,  623878,  623906,  623941, 28, 35, 37, }, // Pierre_Miquelon
    {  494, 1518,       0,   63351,       0,  0, 16,  0, }, // Pitcairn
    {  494, 1527,       0,   63367,       0,  0, 14,  0, }, // Ponape
    {  494, 1534,       0,  143295,       0,  0, 17,  0, }, // Pyongyang
    {  494, 1544,   63398,  623978,  624002, 17, 24, 26, }, // Qyzylorda
    {  494, 1554,       0,  624028,       0,  0, 15,  0, }, // Reunion
    {  494, 1562,       0,   63482,       0,  0, 15,  0, }, // Rothera
    {  494, 1570,  624043,  624059,  624082, 16, 23, 25, }, // Sakhalin
    {  494, 1579,   63560,  624107,  624128, 14, 21, 23, }, // Samara
    {  494, 1596,   63620,  624151,  624171, 13, 20, 22, }, // Samoa
    {  494, 1602,       0,  624193,       0,  0, 18,  0, }, // Seychelles
    {  494, 1624,       0,  624211,       0,  0, 24,  0, }, // Singapore
    {  494, 1634,       0,  624235,       0,  0, 22,  0, }, // Solomon
    {  494, 1642,       0,  624257,       0,  0, 22,  0, }, // South_Georgia
    {  494, 1656,       0,  624279,       0,  0, 16,  0, }, // Suriname
    {  494, 1676,       0,   63786,       0,  0, 13,  0, }, // Syowa
    {  494, 1682,       0,  624295,       0,  0, 13,  0, }, // Tahiti
    {  494, 1689,   63813,  624308,  624328, 13, 20, 22, }, // Taipei
    {  494, 1696,       0,  624350,       0,  0, 19,  0, }, // Tajikistan
    {  494, 1724,       0,   63893,       0,  0, 15,  0, }, // Tokelau
    {  494, 1732,   63908,  624369,  624389, 13, 20, 22, }, // Tonga
    {  494, 1738,       0,   63965,       0,  0, 13,  0, }, // Truk
    {  494, 1750,  624411,  624433,  624462, 22, 29, 31, }, // Turkmenistan
    {  494, 1763,       0,   64059,       0,  0, 14,  0, }, // Tuvalu
    {  494, 1777,  624493,  624508,  624530, 15, 22, 24, }, // Uruguay
    {  494, 1792,  624554,  624573,  624599, 19, 26, 28, }, // Uzbekistan
    {  494, 1803,  624627,  624642,  624664, 15, 22, 24, }, // Vanuatu
    {  494, 1811,       0,  624688,       0,  0, 17,  0, }, // Venezuela
    {  494, 1821,   64297,  624705,  624731, 19, 26, 28, }, // Vladivostok
    {  494, 1833,  624759,  624777,  624802, 18, 25, 27, }, // Volgograd
    {  494, 1843,       0,   64441,       0,  0, 14,  0, }, // Vostok
    {  494, 1850,       0,  624829,       0,  0, 17,  0, }, // Wake
    {  494, 1855,       0,  624846,       0,  0, 23,  0, }, // Wallis
    {  494, 1862,   64499,  624869,  624891, 15, 22, 24, }, // Yakutsk
    {  494, 1870,  624915,  624936,  624964, 21, 28, 30, }, // Yekaterinburg
    {  494, 1892,       0,  624994,       0,  0, 13,  0, }, // Yukon
    {  495,    0,  618094,  618106,  618125, 12, 19, 21, }, // Acre Portuguese/Latin/Macao
    {  495,    5,       0,  618146,       0,  0, 19,  0, }, // Afghanistan
    {  495,   17,       0,  618165,       0,  0, 22,  0, }, // Africa_Central
    {  495,   32,       0,  618187,       0,  0, 23,  0, }, // Africa_Eastern
    {  495,   65,       0,  618210,       0,  0, 21,  0, }, // Africa_Southern
    {  495,   81,  618231,  618255,  618286, 24, 31, 33, }, // Africa_Western
    {  495,  107,  618319,  618333,  618354, 14, 21, 23, }, // Alaska
    {  495,  128,  618377,  618391,  618412, 14, 21, 23, }, // Almaty
    {  495,  135,  618435,  618451,  618474, 16, 23, 25, }, // Amazon
    {  495,  142,  618499,  618527,  618562, 28, 35, 37, }, // America_Central
    {  495,  158,  618599,  618628,  618664, 29, 36, 38, }, // America_Eastern
    {  495,  174,  618702,  618734,  618773, 32, 39, 41, }, // America_Mountain
    {  495,  191,  618814,  618846,  618885, 32, 39, 41, }, // America_Pacific
    {  495,  207,  618926,  618940,  618961, 14, 21, 23, }, // Anadyr
    {  495,  214,  618984,  618996,  619015, 12, 19, 21, }, // Apia
    {  495,  219,  619036,  619049,  619069, 13, 20, 22, }, // Aqtau
    {  495,  225,  619091,  619105,  619126, 14, 21, 23, }, // Aqtobe
    {  495,  232,  619149,  619163,  619184, 14, 21, 23, }, // Arabian
    {  495,  240,  619207,  619224,  619248, 17, 24, 26, }, // Argentina
    {  495,  250,  619274,  619301,  619335, 27, 34, 36, }, // Argentina_Western
    {  495,  268,  619371,  619386,  619408, 15, 22, 24, }, // Armenia
    {  495,  286,  619432,  619449,  619473, 17, 24, 26, }, // Atlantic
    {  495,  295,  619499,  619524,  619556, 25, 32, 34, }, // Australia_Central
    {  495,  313,  619590,  619625,  619667, 35, 42, 44, }, // Australia_CentralWestern
    {  495,  338,  619711,  619737,  619770, 26, 33, 35, }, // Australia_Eastern
    {  495,  356,  619805,  619832,  619866, 27, 34, 36, }, // Australia_Western
    {  495,  374,  619902,  619920,  619945, 18, 25, 27, }, // Azerbaijan
    {  495,  385,  619972,  619987,  620009, 15, 22, 24, }, // Azores
    {  495,  397,  620033,  620052,  620078, 19, 26, 28, }, // Bangladesh
    {  495,  415,       0,  620106,       0,  0, 13,  0, }, // Bhutan
    {  495,  422,       0,  620119,       0,  0, 15,  0, }, // Bolivia
    {  495,  437,  139301,  620134,  620157, 16, 23, 25, }, // Brasilia
    {  495,  454,       0,  620182,       0,  0, 25,  0, }, // Brunei
    {  495,  461,  620207,  620225,  620250, 18, 25, 27, }, // Cape_Verde
    {  495,  478,       0,  620277,       0,  0, 23,  0, }, // Chamorro
    {  495,  487,   59218,  620300,  620322, 15, 22, 24, }, // Chatham
    {  495,  495,  620346,  620359,  620379, 13, 20, 22, }, // Chile
    {  495,  501,  620401,  620414,  620434, 13, 20, 22, }, // China
    {  495,  507,       0,  620456,       0,  0, 21,  0, }, // Christmas
    {  495,  517,       0,  620477,       0,  0, 20,  0, }, // Cocos
    {  495,  523,  620497,  620513,  620536, 16, 23, 25, }, // Colombia
    {  495,  532,  620561,  620580,  620606, 19, 26, 28, }, // Cook
    {  495,  537,   59613,  620634,  620653, 12, 19, 21, }, // Cuba
    {  495,  548,       0,   59667,       0,  0, 13,  0, }, // Davis
    {  495,  564,       0,   59680,       0,  0, 24,  0, }, // DumontDUrville
    {  495,  601,       0,  620674,       0,  0, 19,  0, }, // East_Timor
    {  495,  612,  620693,  620715,  620744, 22, 29, 31, }, // Easter
    {  495,  619,       0,  620775,       0,  0, 15,  0, }, // Ecuador
    {  495,  627,  620790,  620812,  620841, 22, 29, 31, }, // Europe_Central
    {  495,  642,  620872,  620895,  620925, 23, 30, 32, }, // Europe_Eastern
    {  495,  657,       0,  620957,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  495,  680,  620988,  621012,  621043, 24, 31, 33, }, // Europe_Western
    {  495,  695,  621076,  621099,  621129, 23, 30, 32, }, // Falkland
    {  495,  704,  140403,  621161,  621180, 12, 19, 21, }, // Fiji
    {  495,  709,       0,  621201,       0,  0, 23,  0, }, // French_Guiana
    {  495,  723,       0,  621224,       0,  0, 48,  0, }, // French_Southern
    {  495,  746,       0,  621272,       0,  0, 18,  0, }, // Galapagos
    {  495,  756,       0,   60351,       0,  0, 15,  0, }, // Gambier
    {  495,  764,  621290,  621305,  621327, 15, 22, 24, }, // Georgia
    {  495,  772,       0,  621351,       0,  0, 22,  0, }, // Gilbert_Islands
    {  495,  788,       0,  621373,       0,  0, 17,  0, }, // GMT
    {  495,  830,  621390,  621418,  621453, 28, 35, 37, }, // Greenland_Eastern
    {  495,  848,  621490,  621519,  621555, 29, 36, 38, }, // Greenland_Western
    {  495,  866,       0,  621593,       0,  0, 19,  0, }, // Guam
    {  495,  871,       0,  621612,       0,  0, 20,  0, }, // Gulf
    {  495,  876,       0,  621201,       0,  0, 14,  0, }, // Guyana
    {  495,  883,  621632,  621655,  621685, 23, 30, 32, }, // Hawaii_Aleutian
    {  495,  899,  141051,  621717,  621741, 17, 24, 26, }, // Hong_Kong
    {  495,  909,   60891,  621767,  621786, 12, 19, 21, }, // Hovd
    {  495,  914,       0,  621807,       0,  0, 20,  0, }, // India
    {  495,  920,       0,  621827,       0,  0, 21,  0, }, // Indian_Ocean
    {  495,  933,       0,  621848,       0,  0, 17,  0, }, // Indochina
    {  495,  943,       0,  621865,       0,  0, 25,  0, }, // Indonesia_Central
    {  495,  961,       0,  621890,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  495,  979,       0,  621916,       0,  0, 27,  0, }, // Indonesia_Western
    {  495,  997,  621943,  621955,  621974, 12, 19, 21, }, // Iran
    {  495, 1008,  621995,  622010,  622032, 15, 22, 24, }, // Irkutsk
    {  495, 1016,  622056,  622070,  622091, 14, 21, 23, }, // Israel
    {  495, 1023,  622114,  622127,  622147, 13, 20, 22, }, // Japan
    {  495, 1029,  622169,  622201,  622240, 32, 39, 41, }, // Kamchatka
    {  495, 1047,       0,  622281,       0,  0, 19,  0, }, // Kazakhstan
    {  495, 1058,       0,  622300,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  495, 1077,       0,  622328,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  495, 1106,  622357,  622371,  622392, 14, 21, 23, }, // Korea
    {  495, 1112,       0,   61535,       0,  0, 14,  0, }, // Kosrae
    {  495, 1119,   61549,  622415,  622441, 19, 26, 28, }, // Krasnoyarsk
    {  495, 1151,       0,  622469,       0,  0, 19,  0, }, // Kyrgystan
    {  495, 1161,       0,  622488,       0,  0, 17,  0, }, // Lanka
    {  495, 1175,       0,  622505,       0,  0, 19,  0, }, // Line_Islands
    {  495, 1188,   61679,  622524,  622548, 17, 24, 26, }, // Lord_Howe
    {  495, 1198,  141908,  622574,  622594, 13, 20, 22, }, // Macau
    {  495, 1204,  141965,  622616,  622638, 15, 22, 24, }, // Magadan
    {  495, 1219,       0,  622662,       0,  0, 15,  0, }, // Malaysia
    {  495, 1228,       0,  622677,       0,  0, 17,  0, }, // Maldives
    {  495, 1237,       0,  622694,       0,  0, 24,  0, }, // Marquesas
    {  495, 1247,       0,  622718,       0,  0, 23,  0, }, // Marshall_Islands
    {  495, 1264,  622741,  622757,  622780, 16, 23, 25, }, // Mauritius
    {  495, 1274,       0,   62017,       0,  0, 14,  0, }, // Mawson
    {  495, 1281,  622805,  622830,  622862, 25, 32, 34, }, // Mexico_Pacific
    {  495, 1296,  622896,  622914,  622939, 18, 25, 27, }, // Mongolia
    {  495, 1305,  622966,  622981,  623003, 15, 22, 24, }, // Moscow
    {  495, 1312,       0,  623027,       0,  0, 15,  0, }, // Myanmar
    {  495, 1320,       0,   62271,       0,  0, 13,  0, }, // Nauru
    {  495, 1326,       0,  623042,       0,  0, 13,  0, }, // Nepal
    {  495, 1332,  623055,  623077,  623106, 22, 29, 31, }, // New_Caledonia
    {  495, 1346,  623137,  623158,  623186, 21, 28, 30, }, // New_Zealand
    {  495, 1358,  623216,  623234,  623259, 18, 25, 27, }, // Newfoundland
    {  495, 1371,       0,  623286,       0,  0, 12,  0, }, // Niue
    {  495, 1376,  623298,  623318,  623345, 20, 27, 29, }, // Norfolk
    {  495, 1384,   62646,  623374,  623408, 27, 34, 36, }, // Noronha
    {  495, 1392,       0,  623444,       0,  0, 31,  0, }, // North_Mariana
    {  495, 1406,   62782,  623475,  623501, 19, 26, 28, }, // Novosibirsk
    {  495, 1418,  623529,  623541,  623560, 12, 19, 21, }, // Omsk
    {  495, 1428,  623581,  623598,  623622, 17, 24, 26, }, // Pakistan
    {  495, 1437,       0,   62980,       0,  0, 13,  0, }, // Palau
    {  495, 1443,       0,  623648,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  495, 1460,  623672,  623688,  623711, 16, 23, 25, }, // Paraguay
    {  495, 1469,  623736,  623748,  623767, 12, 19, 21, }, // Peru
    {  495, 1474,  623788,  623806,  623831, 18, 25, 27, }, // Philippines
    {  495, 1486,       0,  623858,       0,  0, 20,  0, }, // Phoenix_Islands
    {  495, 1502,  623878,  623906,  623941, 28, 35, 37, }, // Pierre_Miquelon
    {  495, 1518,       0,   63351,       0,  0, 16,  0, }, // Pitcairn
    {  495, 1527,       0,   63367,       0,  0, 14,  0, }, // Ponape
    {  495, 1534,       0,  143295,       0,  0, 17,  0, }, // Pyongyang
    {  495, 1544,   63398,  623978,  624002, 17, 24, 26, }, // Qyzylorda
    {  495, 1554,       0,  624028,       0,  0, 15,  0, }, // Reunion
    {  495, 1562,       0,   63482,       0,  0, 15,  0, }, // Rothera
    {  495, 1570,  624043,  624059,  624082, 16, 23, 25, }, // Sakhalin
    {  495, 1579,   63560,  624107,  624128, 14, 21, 23, }, // Samara
    {  495, 1596,   63620,  624151,  624171, 13, 20, 22, }, // Samoa
    {  495, 1602,       0,  624193,       0,  0, 18,  0, }, // Seychelles
    {  495, 1624,       0,  624211,       0,  0, 24,  0, }, // Singapore
    {  495, 1634,       0,  624235,       0,  0, 22,  0, }, // Solomon
    {  495, 1642,       0,  624257,       0,  0, 22,  0, }, // South_Georgia
    {  495, 1656,       0,  624279,       0,  0, 16,  0, }, // Suriname
    {  495, 1676,       0,   63786,       0,  0, 13,  0, }, // Syowa
    {  495, 1682,       0,  624295,       0,  0, 13,  0, }, // Tahiti
    {  495, 1689,   63813,  624308,  624328, 13, 20, 22, }, // Taipei
    {  495, 1696,       0,  624350,       0,  0, 19,  0, }, // Tajikistan
    {  495, 1724,       0,   63893,       0,  0, 15,  0, }, // Tokelau
    {  495, 1732,   63908,  624369,  624389, 13, 20, 22, }, // Tonga
    {  495, 1738,       0,   63965,       0,  0, 13,  0, }, // Truk
    {  495, 1750,  624411,  624433,  624462, 22, 29, 31, }, // Turkmenistan
    {  495, 1763,       0,   64059,       0,  0, 14,  0, }, // Tuvalu
    {  495, 1777,  624493,  624508,  624530, 15, 22, 24, }, // Uruguay
    {  495, 1792,  624554,  624573,  624599, 19, 26, 28, }, // Uzbekistan
    {  495, 1803,  624627,  624642,  624664, 15, 22, 24, }, // Vanuatu
    {  495, 1811,       0,  624688,       0,  0, 17,  0, }, // Venezuela
    {  495, 1821,   64297,  624705,  624731, 19, 26, 28, }, // Vladivostok
    {  495, 1833,  624759,  624777,  624802, 18, 25, 27, }, // Volgograd
    {  495, 1843,       0,   64441,       0,  0, 14,  0, }, // Vostok
    {  495, 1850,       0,  624829,       0,  0, 17,  0, }, // Wake
    {  495, 1855,       0,  624846,       0,  0, 23,  0, }, // Wallis
    {  495, 1862,   64499,  624869,  624891, 15, 22, 24, }, // Yakutsk
    {  495, 1870,  624915,  624936,  624964, 21, 28, 30, }, // Yekaterinburg
    {  495, 1892,       0,  624994,       0,  0, 13,  0, }, // Yukon
    {  496,    0,  618094,  618106,  618125, 12, 19, 21, }, // Acre Portuguese/Latin/Mozambique
    {  496,    5,       0,  618146,       0,  0, 19,  0, }, // Afghanistan
    {  496,   17,       0,  618165,       0,  0, 22,  0, }, // Africa_Central
    {  496,   32,       0,  618187,       0,  0, 23,  0, }, // Africa_Eastern
    {  496,   65,       0,  618210,       0,  0, 21,  0, }, // Africa_Southern
    {  496,   81,  618231,  618255,  618286, 24, 31, 33, }, // Africa_Western
    {  496,  107,  618319,  618333,  618354, 14, 21, 23, }, // Alaska
    {  496,  128,  618377,  618391,  618412, 14, 21, 23, }, // Almaty
    {  496,  135,  618435,  618451,  618474, 16, 23, 25, }, // Amazon
    {  496,  142,  618499,  618527,  618562, 28, 35, 37, }, // America_Central
    {  496,  158,  618599,  618628,  618664, 29, 36, 38, }, // America_Eastern
    {  496,  174,  618702,  618734,  618773, 32, 39, 41, }, // America_Mountain
    {  496,  191,  618814,  618846,  618885, 32, 39, 41, }, // America_Pacific
    {  496,  207,  618926,  618940,  618961, 14, 21, 23, }, // Anadyr
    {  496,  214,  618984,  618996,  619015, 12, 19, 21, }, // Apia
    {  496,  219,  619036,  619049,  619069, 13, 20, 22, }, // Aqtau
    {  496,  225,  619091,  619105,  619126, 14, 21, 23, }, // Aqtobe
    {  496,  232,  619149,  619163,  619184, 14, 21, 23, }, // Arabian
    {  496,  240,  619207,  619224,  619248, 17, 24, 26, }, // Argentina
    {  496,  250,  619274,  619301,  619335, 27, 34, 36, }, // Argentina_Western
    {  496,  268,  619371,  619386,  619408, 15, 22, 24, }, // Armenia
    {  496,  286,  619432,  619449,  619473, 17, 24, 26, }, // Atlantic
    {  496,  295,  619499,  619524,  619556, 25, 32, 34, }, // Australia_Central
    {  496,  313,  619590,  619625,  619667, 35, 42, 44, }, // Australia_CentralWestern
    {  496,  338,  619711,  619737,  619770, 26, 33, 35, }, // Australia_Eastern
    {  496,  356,  619805,  619832,  619866, 27, 34, 36, }, // Australia_Western
    {  496,  374,  619902,  619920,  619945, 18, 25, 27, }, // Azerbaijan
    {  496,  385,  619972,  619987,  620009, 15, 22, 24, }, // Azores
    {  496,  397,  620033,  620052,  620078, 19, 26, 28, }, // Bangladesh
    {  496,  415,       0,  620106,       0,  0, 13,  0, }, // Bhutan
    {  496,  422,       0,  620119,       0,  0, 15,  0, }, // Bolivia
    {  496,  437,  139301,  620134,  620157, 16, 23, 25, }, // Brasilia
    {  496,  454,       0,  620182,       0,  0, 25,  0, }, // Brunei
    {  496,  461,  620207,  620225,  620250, 18, 25, 27, }, // Cape_Verde
    {  496,  478,       0,  620277,       0,  0, 23,  0, }, // Chamorro
    {  496,  487,   59218,  620300,  620322, 15, 22, 24, }, // Chatham
    {  496,  495,  620346,  620359,  620379, 13, 20, 22, }, // Chile
    {  496,  501,  620401,  620414,  620434, 13, 20, 22, }, // China
    {  496,  507,       0,  620456,       0,  0, 21,  0, }, // Christmas
    {  496,  517,       0,  620477,       0,  0, 20,  0, }, // Cocos
    {  496,  523,  620497,  620513,  620536, 16, 23, 25, }, // Colombia
    {  496,  532,  620561,  620580,  620606, 19, 26, 28, }, // Cook
    {  496,  537,   59613,  620634,  620653, 12, 19, 21, }, // Cuba
    {  496,  548,       0,   59667,       0,  0, 13,  0, }, // Davis
    {  496,  564,       0,   59680,       0,  0, 24,  0, }, // DumontDUrville
    {  496,  601,       0,  620674,       0,  0, 19,  0, }, // East_Timor
    {  496,  612,  620693,  620715,  620744, 22, 29, 31, }, // Easter
    {  496,  619,       0,  620775,       0,  0, 15,  0, }, // Ecuador
    {  496,  627,  620790,  620812,  620841, 22, 29, 31, }, // Europe_Central
    {  496,  642,  620872,  620895,  620925, 23, 30, 32, }, // Europe_Eastern
    {  496,  657,       0,  620957,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  496,  680,  620988,  621012,  621043, 24, 31, 33, }, // Europe_Western
    {  496,  695,  621076,  621099,  621129, 23, 30, 32, }, // Falkland
    {  496,  704,  140403,  621161,  621180, 12, 19, 21, }, // Fiji
    {  496,  709,       0,  621201,       0,  0, 23,  0, }, // French_Guiana
    {  496,  723,       0,  621224,       0,  0, 48,  0, }, // French_Southern
    {  496,  746,       0,  621272,       0,  0, 18,  0, }, // Galapagos
    {  496,  756,       0,   60351,       0,  0, 15,  0, }, // Gambier
    {  496,  764,  621290,  621305,  621327, 15, 22, 24, }, // Georgia
    {  496,  772,       0,  621351,       0,  0, 22,  0, }, // Gilbert_Islands
    {  496,  788,       0,  621373,       0,  0, 17,  0, }, // GMT
    {  496,  830,  621390,  621418,  621453, 28, 35, 37, }, // Greenland_Eastern
    {  496,  848,  621490,  621519,  621555, 29, 36, 38, }, // Greenland_Western
    {  496,  866,       0,  621593,       0,  0, 19,  0, }, // Guam
    {  496,  871,       0,  621612,       0,  0, 20,  0, }, // Gulf
    {  496,  876,       0,  621201,       0,  0, 14,  0, }, // Guyana
    {  496,  883,  621632,  621655,  621685, 23, 30, 32, }, // Hawaii_Aleutian
    {  496,  899,  141051,  621717,  621741, 17, 24, 26, }, // Hong_Kong
    {  496,  909,   60891,  621767,  621786, 12, 19, 21, }, // Hovd
    {  496,  914,       0,  621807,       0,  0, 20,  0, }, // India
    {  496,  920,       0,  621827,       0,  0, 21,  0, }, // Indian_Ocean
    {  496,  933,       0,  621848,       0,  0, 17,  0, }, // Indochina
    {  496,  943,       0,  621865,       0,  0, 25,  0, }, // Indonesia_Central
    {  496,  961,       0,  621890,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  496,  979,       0,  621916,       0,  0, 27,  0, }, // Indonesia_Western
    {  496,  997,  621943,  621955,  621974, 12, 19, 21, }, // Iran
    {  496, 1008,  621995,  622010,  622032, 15, 22, 24, }, // Irkutsk
    {  496, 1016,  622056,  622070,  622091, 14, 21, 23, }, // Israel
    {  496, 1023,  622114,  622127,  622147, 13, 20, 22, }, // Japan
    {  496, 1029,  622169,  622201,  622240, 32, 39, 41, }, // Kamchatka
    {  496, 1047,       0,  622281,       0,  0, 19,  0, }, // Kazakhstan
    {  496, 1058,       0,  622300,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  496, 1077,       0,  622328,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  496, 1106,  622357,  622371,  622392, 14, 21, 23, }, // Korea
    {  496, 1112,       0,   61535,       0,  0, 14,  0, }, // Kosrae
    {  496, 1119,   61549,  622415,  622441, 19, 26, 28, }, // Krasnoyarsk
    {  496, 1151,       0,  622469,       0,  0, 19,  0, }, // Kyrgystan
    {  496, 1161,       0,  622488,       0,  0, 17,  0, }, // Lanka
    {  496, 1175,       0,  622505,       0,  0, 19,  0, }, // Line_Islands
    {  496, 1188,   61679,  622524,  622548, 17, 24, 26, }, // Lord_Howe
    {  496, 1198,  141908,  622574,  622594, 13, 20, 22, }, // Macau
    {  496, 1204,  141965,  622616,  622638, 15, 22, 24, }, // Magadan
    {  496, 1219,       0,  622662,       0,  0, 15,  0, }, // Malaysia
    {  496, 1228,       0,  622677,       0,  0, 17,  0, }, // Maldives
    {  496, 1237,       0,  622694,       0,  0, 24,  0, }, // Marquesas
    {  496, 1247,       0,  622718,       0,  0, 23,  0, }, // Marshall_Islands
    {  496, 1264,  622741,  622757,  622780, 16, 23, 25, }, // Mauritius
    {  496, 1274,       0,   62017,       0,  0, 14,  0, }, // Mawson
    {  496, 1281,  622805,  622830,  622862, 25, 32, 34, }, // Mexico_Pacific
    {  496, 1296,  622896,  622914,  622939, 18, 25, 27, }, // Mongolia
    {  496, 1305,  622966,  622981,  623003, 15, 22, 24, }, // Moscow
    {  496, 1312,       0,  623027,       0,  0, 15,  0, }, // Myanmar
    {  496, 1320,       0,   62271,       0,  0, 13,  0, }, // Nauru
    {  496, 1326,       0,  623042,       0,  0, 13,  0, }, // Nepal
    {  496, 1332,  623055,  623077,  623106, 22, 29, 31, }, // New_Caledonia
    {  496, 1346,  623137,  623158,  623186, 21, 28, 30, }, // New_Zealand
    {  496, 1358,  623216,  623234,  623259, 18, 25, 27, }, // Newfoundland
    {  496, 1371,       0,  623286,       0,  0, 12,  0, }, // Niue
    {  496, 1376,  623298,  623318,  623345, 20, 27, 29, }, // Norfolk
    {  496, 1384,   62646,  623374,  623408, 27, 34, 36, }, // Noronha
    {  496, 1392,       0,  623444,       0,  0, 31,  0, }, // North_Mariana
    {  496, 1406,   62782,  623475,  623501, 19, 26, 28, }, // Novosibirsk
    {  496, 1418,  623529,  623541,  623560, 12, 19, 21, }, // Omsk
    {  496, 1428,  623581,  623598,  623622, 17, 24, 26, }, // Pakistan
    {  496, 1437,       0,   62980,       0,  0, 13,  0, }, // Palau
    {  496, 1443,       0,  623648,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  496, 1460,  623672,  623688,  623711, 16, 23, 25, }, // Paraguay
    {  496, 1469,  623736,  623748,  623767, 12, 19, 21, }, // Peru
    {  496, 1474,  623788,  623806,  623831, 18, 25, 27, }, // Philippines
    {  496, 1486,       0,  623858,       0,  0, 20,  0, }, // Phoenix_Islands
    {  496, 1502,  623878,  623906,  623941, 28, 35, 37, }, // Pierre_Miquelon
    {  496, 1518,       0,   63351,       0,  0, 16,  0, }, // Pitcairn
    {  496, 1527,       0,   63367,       0,  0, 14,  0, }, // Ponape
    {  496, 1534,       0,  143295,       0,  0, 17,  0, }, // Pyongyang
    {  496, 1544,   63398,  623978,  624002, 17, 24, 26, }, // Qyzylorda
    {  496, 1554,       0,  624028,       0,  0, 15,  0, }, // Reunion
    {  496, 1562,       0,   63482,       0,  0, 15,  0, }, // Rothera
    {  496, 1570,  624043,  624059,  624082, 16, 23, 25, }, // Sakhalin
    {  496, 1579,   63560,  624107,  624128, 14, 21, 23, }, // Samara
    {  496, 1596,   63620,  624151,  624171, 13, 20, 22, }, // Samoa
    {  496, 1602,       0,  624193,       0,  0, 18,  0, }, // Seychelles
    {  496, 1624,       0,  624211,       0,  0, 24,  0, }, // Singapore
    {  496, 1634,       0,  624235,       0,  0, 22,  0, }, // Solomon
    {  496, 1642,       0,  624257,       0,  0, 22,  0, }, // South_Georgia
    {  496, 1656,       0,  624279,       0,  0, 16,  0, }, // Suriname
    {  496, 1676,       0,   63786,       0,  0, 13,  0, }, // Syowa
    {  496, 1682,       0,  624295,       0,  0, 13,  0, }, // Tahiti
    {  496, 1689,   63813,  624308,  624328, 13, 20, 22, }, // Taipei
    {  496, 1696,       0,  624350,       0,  0, 19,  0, }, // Tajikistan
    {  496, 1724,       0,   63893,       0,  0, 15,  0, }, // Tokelau
    {  496, 1732,   63908,  624369,  624389, 13, 20, 22, }, // Tonga
    {  496, 1738,       0,   63965,       0,  0, 13,  0, }, // Truk
    {  496, 1750,  624411,  624433,  624462, 22, 29, 31, }, // Turkmenistan
    {  496, 1763,       0,   64059,       0,  0, 14,  0, }, // Tuvalu
    {  496, 1777,  624493,  624508,  624530, 15, 22, 24, }, // Uruguay
    {  496, 1792,  624554,  624573,  624599, 19, 26, 28, }, // Uzbekistan
    {  496, 1803,  624627,  624642,  624664, 15, 22, 24, }, // Vanuatu
    {  496, 1811,       0,  624688,       0,  0, 17,  0, }, // Venezuela
    {  496, 1821,   64297,  624705,  624731, 19, 26, 28, }, // Vladivostok
    {  496, 1833,  624759,  624777,  624802, 18, 25, 27, }, // Volgograd
    {  496, 1843,       0,   64441,       0,  0, 14,  0, }, // Vostok
    {  496, 1850,       0,  624829,       0,  0, 17,  0, }, // Wake
    {  496, 1855,       0,  624846,       0,  0, 23,  0, }, // Wallis
    {  496, 1862,   64499,  624869,  624891, 15, 22, 24, }, // Yakutsk
    {  496, 1870,  624915,  624936,  624964, 21, 28, 30, }, // Yekaterinburg
    {  496, 1892,       0,  624994,       0,  0, 13,  0, }, // Yukon
    {  497,    0,  618094,  618106,  618125, 12, 19, 21, }, // Acre Portuguese/Latin/Portugal
    {  497,    5,       0,  618146,       0,  0, 19,  0, }, // Afghanistan
    {  497,   17,       0,  618165,       0,  0, 22,  0, }, // Africa_Central
    {  497,   32,       0,  618187,       0,  0, 23,  0, }, // Africa_Eastern
    {  497,   65,       0,  618210,       0,  0, 21,  0, }, // Africa_Southern
    {  497,   81,  618231,  618255,  618286, 24, 31, 33, }, // Africa_Western
    {  497,  107,  618319,  618333,  618354, 14, 21, 23, }, // Alaska
    {  497,  128,  618377,  618391,  618412, 14, 21, 23, }, // Almaty
    {  497,  135,  618435,  618451,  618474, 16, 23, 25, }, // Amazon
    {  497,  142,  618499,  618527,  618562, 28, 35, 37, }, // America_Central
    {  497,  158,  618599,  618628,  618664, 29, 36, 38, }, // America_Eastern
    {  497,  174,  618702,  618734,  618773, 32, 39, 41, }, // America_Mountain
    {  497,  191,  618814,  618846,  618885, 32, 39, 41, }, // America_Pacific
    {  497,  207,  618926,  618940,  618961, 14, 21, 23, }, // Anadyr
    {  497,  214,  618984,  618996,  619015, 12, 19, 21, }, // Apia
    {  497,  219,  619036,  619049,  619069, 13, 20, 22, }, // Aqtau
    {  497,  225,  619091,  619105,  619126, 14, 21, 23, }, // Aqtobe
    {  497,  232,  619149,  619163,  619184, 14, 21, 23, }, // Arabian
    {  497,  240,  619207,  619224,  619248, 17, 24, 26, }, // Argentina
    {  497,  250,  619274,  619301,  619335, 27, 34, 36, }, // Argentina_Western
    {  497,  268,  619371,  619386,  619408, 15, 22, 24, }, // Armenia
    {  497,  286,  619432,  619449,  619473, 17, 24, 26, }, // Atlantic
    {  497,  295,  619499,  619524,  619556, 25, 32, 34, }, // Australia_Central
    {  497,  313,  619590,  619625,  619667, 35, 42, 44, }, // Australia_CentralWestern
    {  497,  338,  619711,  619737,  619770, 26, 33, 35, }, // Australia_Eastern
    {  497,  356,  619805,  619832,  619866, 27, 34, 36, }, // Australia_Western
    {  497,  374,  619902,  619920,  619945, 18, 25, 27, }, // Azerbaijan
    {  497,  385,  619972,  619987,  620009, 15, 22, 24, }, // Azores
    {  497,  397,  620033,  620052,  620078, 19, 26, 28, }, // Bangladesh
    {  497,  415,       0,  620106,       0,  0, 13,  0, }, // Bhutan
    {  497,  422,       0,  620119,       0,  0, 15,  0, }, // Bolivia
    {  497,  437,  139301,  620134,  620157, 16, 23, 25, }, // Brasilia
    {  497,  454,       0,  620182,       0,  0, 25,  0, }, // Brunei
    {  497,  461,  620207,  620225,  620250, 18, 25, 27, }, // Cape_Verde
    {  497,  478,       0,  620277,       0,  0, 23,  0, }, // Chamorro
    {  497,  487,   59218,  620300,  620322, 15, 22, 24, }, // Chatham
    {  497,  495,  620346,  620359,  620379, 13, 20, 22, }, // Chile
    {  497,  501,  620401,  620414,  620434, 13, 20, 22, }, // China
    {  497,  507,       0,  620456,       0,  0, 21,  0, }, // Christmas
    {  497,  517,       0,  620477,       0,  0, 20,  0, }, // Cocos
    {  497,  523,  620497,  620513,  620536, 16, 23, 25, }, // Colombia
    {  497,  532,  620561,  620580,  620606, 19, 26, 28, }, // Cook
    {  497,  537,   59613,  620634,  620653, 12, 19, 21, }, // Cuba
    {  497,  548,       0,   59667,       0,  0, 13,  0, }, // Davis
    {  497,  564,       0,   59680,       0,  0, 24,  0, }, // DumontDUrville
    {  497,  601,       0,  620674,       0,  0, 19,  0, }, // East_Timor
    {  497,  612,  620693,  620715,  620744, 22, 29, 31, }, // Easter
    {  497,  619,       0,  620775,       0,  0, 15,  0, }, // Ecuador
    {  497,  627,  620790,  620812,  620841, 22, 29, 31, }, // Europe_Central
    {  497,  642,  620872,  620895,  620925, 23, 30, 32, }, // Europe_Eastern
    {  497,  657,       0,  620957,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  497,  680,  620988,  621012,  621043, 24, 31, 33, }, // Europe_Western
    {  497,  695,  621076,  621099,  621129, 23, 30, 32, }, // Falkland
    {  497,  704,  140403,  621161,  621180, 12, 19, 21, }, // Fiji
    {  497,  709,       0,  621201,       0,  0, 23,  0, }, // French_Guiana
    {  497,  723,       0,  621224,       0,  0, 48,  0, }, // French_Southern
    {  497,  746,       0,  621272,       0,  0, 18,  0, }, // Galapagos
    {  497,  756,       0,   60351,       0,  0, 15,  0, }, // Gambier
    {  497,  764,  621290,  621305,  621327, 15, 22, 24, }, // Georgia
    {  497,  772,       0,  621351,       0,  0, 22,  0, }, // Gilbert_Islands
    {  497,  788,       0,  621373,       0,  0, 17,  0, }, // GMT
    {  497,  830,  621390,  621418,  621453, 28, 35, 37, }, // Greenland_Eastern
    {  497,  848,  621490,  621519,  621555, 29, 36, 38, }, // Greenland_Western
    {  497,  866,       0,  621593,       0,  0, 19,  0, }, // Guam
    {  497,  871,       0,  621612,       0,  0, 20,  0, }, // Gulf
    {  497,  876,       0,  621201,       0,  0, 14,  0, }, // Guyana
    {  497,  883,  621632,  621655,  621685, 23, 30, 32, }, // Hawaii_Aleutian
    {  497,  899,  141051,  621717,  621741, 17, 24, 26, }, // Hong_Kong
    {  497,  909,   60891,  621767,  621786, 12, 19, 21, }, // Hovd
    {  497,  914,       0,  621807,       0,  0, 20,  0, }, // India
    {  497,  920,       0,  621827,       0,  0, 21,  0, }, // Indian_Ocean
    {  497,  933,       0,  621848,       0,  0, 17,  0, }, // Indochina
    {  497,  943,       0,  621865,       0,  0, 25,  0, }, // Indonesia_Central
    {  497,  961,       0,  621890,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  497,  979,       0,  621916,       0,  0, 27,  0, }, // Indonesia_Western
    {  497,  997,  621943,  621955,  621974, 12, 19, 21, }, // Iran
    {  497, 1008,  621995,  622010,  622032, 15, 22, 24, }, // Irkutsk
    {  497, 1016,  622056,  622070,  622091, 14, 21, 23, }, // Israel
    {  497, 1023,  622114,  622127,  622147, 13, 20, 22, }, // Japan
    {  497, 1029,  622169,  622201,  622240, 32, 39, 41, }, // Kamchatka
    {  497, 1047,       0,  622281,       0,  0, 19,  0, }, // Kazakhstan
    {  497, 1058,       0,  622300,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  497, 1077,       0,  622328,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  497, 1106,  622357,  622371,  622392, 14, 21, 23, }, // Korea
    {  497, 1112,       0,   61535,       0,  0, 14,  0, }, // Kosrae
    {  497, 1119,   61549,  622415,  622441, 19, 26, 28, }, // Krasnoyarsk
    {  497, 1151,       0,  622469,       0,  0, 19,  0, }, // Kyrgystan
    {  497, 1161,       0,  622488,       0,  0, 17,  0, }, // Lanka
    {  497, 1175,       0,  622505,       0,  0, 19,  0, }, // Line_Islands
    {  497, 1188,   61679,  622524,  622548, 17, 24, 26, }, // Lord_Howe
    {  497, 1198,  141908,  622574,  622594, 13, 20, 22, }, // Macau
    {  497, 1204,  141965,  622616,  622638, 15, 22, 24, }, // Magadan
    {  497, 1219,       0,  622662,       0,  0, 15,  0, }, // Malaysia
    {  497, 1228,       0,  622677,       0,  0, 17,  0, }, // Maldives
    {  497, 1237,       0,  622694,       0,  0, 24,  0, }, // Marquesas
    {  497, 1247,       0,  622718,       0,  0, 23,  0, }, // Marshall_Islands
    {  497, 1264,  622741,  622757,  622780, 16, 23, 25, }, // Mauritius
    {  497, 1274,       0,   62017,       0,  0, 14,  0, }, // Mawson
    {  497, 1281,  622805,  622830,  622862, 25, 32, 34, }, // Mexico_Pacific
    {  497, 1296,  622896,  622914,  622939, 18, 25, 27, }, // Mongolia
    {  497, 1305,  622966,  622981,  623003, 15, 22, 24, }, // Moscow
    {  497, 1312,       0,  623027,       0,  0, 15,  0, }, // Myanmar
    {  497, 1320,       0,   62271,       0,  0, 13,  0, }, // Nauru
    {  497, 1326,       0,  623042,       0,  0, 13,  0, }, // Nepal
    {  497, 1332,  623055,  623077,  623106, 22, 29, 31, }, // New_Caledonia
    {  497, 1346,  623137,  623158,  623186, 21, 28, 30, }, // New_Zealand
    {  497, 1358,  623216,  623234,  623259, 18, 25, 27, }, // Newfoundland
    {  497, 1371,       0,  623286,       0,  0, 12,  0, }, // Niue
    {  497, 1376,  623298,  623318,  623345, 20, 27, 29, }, // Norfolk
    {  497, 1384,   62646,  623374,  623408, 27, 34, 36, }, // Noronha
    {  497, 1392,       0,  623444,       0,  0, 31,  0, }, // North_Mariana
    {  497, 1406,   62782,  623475,  623501, 19, 26, 28, }, // Novosibirsk
    {  497, 1418,  623529,  623541,  623560, 12, 19, 21, }, // Omsk
    {  497, 1428,  623581,  623598,  623622, 17, 24, 26, }, // Pakistan
    {  497, 1437,       0,   62980,       0,  0, 13,  0, }, // Palau
    {  497, 1443,       0,  623648,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  497, 1460,  623672,  623688,  623711, 16, 23, 25, }, // Paraguay
    {  497, 1469,  623736,  623748,  623767, 12, 19, 21, }, // Peru
    {  497, 1474,  623788,  623806,  623831, 18, 25, 27, }, // Philippines
    {  497, 1486,       0,  623858,       0,  0, 20,  0, }, // Phoenix_Islands
    {  497, 1502,  623878,  623906,  623941, 28, 35, 37, }, // Pierre_Miquelon
    {  497, 1518,       0,   63351,       0,  0, 16,  0, }, // Pitcairn
    {  497, 1527,       0,   63367,       0,  0, 14,  0, }, // Ponape
    {  497, 1534,       0,  143295,       0,  0, 17,  0, }, // Pyongyang
    {  497, 1544,   63398,  623978,  624002, 17, 24, 26, }, // Qyzylorda
    {  497, 1554,       0,  624028,       0,  0, 15,  0, }, // Reunion
    {  497, 1562,       0,   63482,       0,  0, 15,  0, }, // Rothera
    {  497, 1570,  624043,  624059,  624082, 16, 23, 25, }, // Sakhalin
    {  497, 1579,   63560,  624107,  624128, 14, 21, 23, }, // Samara
    {  497, 1596,   63620,  624151,  624171, 13, 20, 22, }, // Samoa
    {  497, 1602,       0,  624193,       0,  0, 18,  0, }, // Seychelles
    {  497, 1624,       0,  624211,       0,  0, 24,  0, }, // Singapore
    {  497, 1634,       0,  624235,       0,  0, 22,  0, }, // Solomon
    {  497, 1642,       0,  624257,       0,  0, 22,  0, }, // South_Georgia
    {  497, 1656,       0,  624279,       0,  0, 16,  0, }, // Suriname
    {  497, 1676,       0,   63786,       0,  0, 13,  0, }, // Syowa
    {  497, 1682,       0,  624295,       0,  0, 13,  0, }, // Tahiti
    {  497, 1689,   63813,  624308,  624328, 13, 20, 22, }, // Taipei
    {  497, 1696,       0,  624350,       0,  0, 19,  0, }, // Tajikistan
    {  497, 1724,       0,   63893,       0,  0, 15,  0, }, // Tokelau
    {  497, 1732,   63908,  624369,  624389, 13, 20, 22, }, // Tonga
    {  497, 1738,       0,   63965,       0,  0, 13,  0, }, // Truk
    {  497, 1750,  624411,  624433,  624462, 22, 29, 31, }, // Turkmenistan
    {  497, 1763,       0,   64059,       0,  0, 14,  0, }, // Tuvalu
    {  497, 1777,  624493,  624508,  624530, 15, 22, 24, }, // Uruguay
    {  497, 1792,  624554,  624573,  624599, 19, 26, 28, }, // Uzbekistan
    {  497, 1803,  624627,  624642,  624664, 15, 22, 24, }, // Vanuatu
    {  497, 1811,       0,  624688,       0,  0, 17,  0, }, // Venezuela
    {  497, 1821,   64297,  624705,  624731, 19, 26, 28, }, // Vladivostok
    {  497, 1833,  624759,  624777,  624802, 18, 25, 27, }, // Volgograd
    {  497, 1843,       0,   64441,       0,  0, 14,  0, }, // Vostok
    {  497, 1850,       0,  624829,       0,  0, 17,  0, }, // Wake
    {  497, 1855,       0,  624846,       0,  0, 23,  0, }, // Wallis
    {  497, 1862,   64499,  624869,  624891, 15, 22, 24, }, // Yakutsk
    {  497, 1870,  624915,  624936,  624964, 21, 28, 30, }, // Yekaterinburg
    {  497, 1892,       0,  624994,       0,  0, 13,  0, }, // Yukon
    {  498,    0,  618094,  618106,  618125, 12, 19, 21, }, // Acre Portuguese/Latin/Sao Tome and Principe
    {  498,    5,       0,  618146,       0,  0, 19,  0, }, // Afghanistan
    {  498,   17,       0,  618165,       0,  0, 22,  0, }, // Africa_Central
    {  498,   32,       0,  618187,       0,  0, 23,  0, }, // Africa_Eastern
    {  498,   65,       0,  618210,       0,  0, 21,  0, }, // Africa_Southern
    {  498,   81,  618231,  618255,  618286, 24, 31, 33, }, // Africa_Western
    {  498,  107,  618319,  618333,  618354, 14, 21, 23, }, // Alaska
    {  498,  128,  618377,  618391,  618412, 14, 21, 23, }, // Almaty
    {  498,  135,  618435,  618451,  618474, 16, 23, 25, }, // Amazon
    {  498,  142,  618499,  618527,  618562, 28, 35, 37, }, // America_Central
    {  498,  158,  618599,  618628,  618664, 29, 36, 38, }, // America_Eastern
    {  498,  174,  618702,  618734,  618773, 32, 39, 41, }, // America_Mountain
    {  498,  191,  618814,  618846,  618885, 32, 39, 41, }, // America_Pacific
    {  498,  207,  618926,  618940,  618961, 14, 21, 23, }, // Anadyr
    {  498,  214,  618984,  618996,  619015, 12, 19, 21, }, // Apia
    {  498,  219,  619036,  619049,  619069, 13, 20, 22, }, // Aqtau
    {  498,  225,  619091,  619105,  619126, 14, 21, 23, }, // Aqtobe
    {  498,  232,  619149,  619163,  619184, 14, 21, 23, }, // Arabian
    {  498,  240,  619207,  619224,  619248, 17, 24, 26, }, // Argentina
    {  498,  250,  619274,  619301,  619335, 27, 34, 36, }, // Argentina_Western
    {  498,  268,  619371,  619386,  619408, 15, 22, 24, }, // Armenia
    {  498,  286,  619432,  619449,  619473, 17, 24, 26, }, // Atlantic
    {  498,  295,  619499,  619524,  619556, 25, 32, 34, }, // Australia_Central
    {  498,  313,  619590,  619625,  619667, 35, 42, 44, }, // Australia_CentralWestern
    {  498,  338,  619711,  619737,  619770, 26, 33, 35, }, // Australia_Eastern
    {  498,  356,  619805,  619832,  619866, 27, 34, 36, }, // Australia_Western
    {  498,  374,  619902,  619920,  619945, 18, 25, 27, }, // Azerbaijan
    {  498,  385,  619972,  619987,  620009, 15, 22, 24, }, // Azores
    {  498,  397,  620033,  620052,  620078, 19, 26, 28, }, // Bangladesh
    {  498,  415,       0,  620106,       0,  0, 13,  0, }, // Bhutan
    {  498,  422,       0,  620119,       0,  0, 15,  0, }, // Bolivia
    {  498,  437,  139301,  620134,  620157, 16, 23, 25, }, // Brasilia
    {  498,  454,       0,  620182,       0,  0, 25,  0, }, // Brunei
    {  498,  461,  620207,  620225,  620250, 18, 25, 27, }, // Cape_Verde
    {  498,  478,       0,  620277,       0,  0, 23,  0, }, // Chamorro
    {  498,  487,   59218,  620300,  620322, 15, 22, 24, }, // Chatham
    {  498,  495,  620346,  620359,  620379, 13, 20, 22, }, // Chile
    {  498,  501,  620401,  620414,  620434, 13, 20, 22, }, // China
    {  498,  507,       0,  620456,       0,  0, 21,  0, }, // Christmas
    {  498,  517,       0,  620477,       0,  0, 20,  0, }, // Cocos
    {  498,  523,  620497,  620513,  620536, 16, 23, 25, }, // Colombia
    {  498,  532,  620561,  620580,  620606, 19, 26, 28, }, // Cook
    {  498,  537,   59613,  620634,  620653, 12, 19, 21, }, // Cuba
    {  498,  548,       0,   59667,       0,  0, 13,  0, }, // Davis
    {  498,  564,       0,   59680,       0,  0, 24,  0, }, // DumontDUrville
    {  498,  601,       0,  620674,       0,  0, 19,  0, }, // East_Timor
    {  498,  612,  620693,  620715,  620744, 22, 29, 31, }, // Easter
    {  498,  619,       0,  620775,       0,  0, 15,  0, }, // Ecuador
    {  498,  627,  620790,  620812,  620841, 22, 29, 31, }, // Europe_Central
    {  498,  642,  620872,  620895,  620925, 23, 30, 32, }, // Europe_Eastern
    {  498,  657,       0,  620957,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  498,  680,  620988,  621012,  621043, 24, 31, 33, }, // Europe_Western
    {  498,  695,  621076,  621099,  621129, 23, 30, 32, }, // Falkland
    {  498,  704,  140403,  621161,  621180, 12, 19, 21, }, // Fiji
    {  498,  709,       0,  621201,       0,  0, 23,  0, }, // French_Guiana
    {  498,  723,       0,  621224,       0,  0, 48,  0, }, // French_Southern
    {  498,  746,       0,  621272,       0,  0, 18,  0, }, // Galapagos
    {  498,  756,       0,   60351,       0,  0, 15,  0, }, // Gambier
    {  498,  764,  621290,  621305,  621327, 15, 22, 24, }, // Georgia
    {  498,  772,       0,  621351,       0,  0, 22,  0, }, // Gilbert_Islands
    {  498,  788,       0,  621373,       0,  0, 17,  0, }, // GMT
    {  498,  830,  621390,  621418,  621453, 28, 35, 37, }, // Greenland_Eastern
    {  498,  848,  621490,  621519,  621555, 29, 36, 38, }, // Greenland_Western
    {  498,  866,       0,  621593,       0,  0, 19,  0, }, // Guam
    {  498,  871,       0,  621612,       0,  0, 20,  0, }, // Gulf
    {  498,  876,       0,  621201,       0,  0, 14,  0, }, // Guyana
    {  498,  883,  621632,  621655,  621685, 23, 30, 32, }, // Hawaii_Aleutian
    {  498,  899,  141051,  621717,  621741, 17, 24, 26, }, // Hong_Kong
    {  498,  909,   60891,  621767,  621786, 12, 19, 21, }, // Hovd
    {  498,  914,       0,  621807,       0,  0, 20,  0, }, // India
    {  498,  920,       0,  621827,       0,  0, 21,  0, }, // Indian_Ocean
    {  498,  933,       0,  621848,       0,  0, 17,  0, }, // Indochina
    {  498,  943,       0,  621865,       0,  0, 25,  0, }, // Indonesia_Central
    {  498,  961,       0,  621890,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  498,  979,       0,  621916,       0,  0, 27,  0, }, // Indonesia_Western
    {  498,  997,  621943,  621955,  621974, 12, 19, 21, }, // Iran
    {  498, 1008,  621995,  622010,  622032, 15, 22, 24, }, // Irkutsk
    {  498, 1016,  622056,  622070,  622091, 14, 21, 23, }, // Israel
    {  498, 1023,  622114,  622127,  622147, 13, 20, 22, }, // Japan
    {  498, 1029,  622169,  622201,  622240, 32, 39, 41, }, // Kamchatka
    {  498, 1047,       0,  622281,       0,  0, 19,  0, }, // Kazakhstan
    {  498, 1058,       0,  622300,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  498, 1077,       0,  622328,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  498, 1106,  622357,  622371,  622392, 14, 21, 23, }, // Korea
    {  498, 1112,       0,   61535,       0,  0, 14,  0, }, // Kosrae
    {  498, 1119,   61549,  622415,  622441, 19, 26, 28, }, // Krasnoyarsk
    {  498, 1151,       0,  622469,       0,  0, 19,  0, }, // Kyrgystan
    {  498, 1161,       0,  622488,       0,  0, 17,  0, }, // Lanka
    {  498, 1175,       0,  622505,       0,  0, 19,  0, }, // Line_Islands
    {  498, 1188,   61679,  622524,  622548, 17, 24, 26, }, // Lord_Howe
    {  498, 1198,  141908,  622574,  622594, 13, 20, 22, }, // Macau
    {  498, 1204,  141965,  622616,  622638, 15, 22, 24, }, // Magadan
    {  498, 1219,       0,  622662,       0,  0, 15,  0, }, // Malaysia
    {  498, 1228,       0,  622677,       0,  0, 17,  0, }, // Maldives
    {  498, 1237,       0,  622694,       0,  0, 24,  0, }, // Marquesas
    {  498, 1247,       0,  622718,       0,  0, 23,  0, }, // Marshall_Islands
    {  498, 1264,  622741,  622757,  622780, 16, 23, 25, }, // Mauritius
    {  498, 1274,       0,   62017,       0,  0, 14,  0, }, // Mawson
    {  498, 1281,  622805,  622830,  622862, 25, 32, 34, }, // Mexico_Pacific
    {  498, 1296,  622896,  622914,  622939, 18, 25, 27, }, // Mongolia
    {  498, 1305,  622966,  622981,  623003, 15, 22, 24, }, // Moscow
    {  498, 1312,       0,  623027,       0,  0, 15,  0, }, // Myanmar
    {  498, 1320,       0,   62271,       0,  0, 13,  0, }, // Nauru
    {  498, 1326,       0,  623042,       0,  0, 13,  0, }, // Nepal
    {  498, 1332,  623055,  623077,  623106, 22, 29, 31, }, // New_Caledonia
    {  498, 1346,  623137,  623158,  623186, 21, 28, 30, }, // New_Zealand
    {  498, 1358,  623216,  623234,  623259, 18, 25, 27, }, // Newfoundland
    {  498, 1371,       0,  623286,       0,  0, 12,  0, }, // Niue
    {  498, 1376,  623298,  623318,  623345, 20, 27, 29, }, // Norfolk
    {  498, 1384,   62646,  623374,  623408, 27, 34, 36, }, // Noronha
    {  498, 1392,       0,  623444,       0,  0, 31,  0, }, // North_Mariana
    {  498, 1406,   62782,  623475,  623501, 19, 26, 28, }, // Novosibirsk
    {  498, 1418,  623529,  623541,  623560, 12, 19, 21, }, // Omsk
    {  498, 1428,  623581,  623598,  623622, 17, 24, 26, }, // Pakistan
    {  498, 1437,       0,   62980,       0,  0, 13,  0, }, // Palau
    {  498, 1443,       0,  623648,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  498, 1460,  623672,  623688,  623711, 16, 23, 25, }, // Paraguay
    {  498, 1469,  623736,  623748,  623767, 12, 19, 21, }, // Peru
    {  498, 1474,  623788,  623806,  623831, 18, 25, 27, }, // Philippines
    {  498, 1486,       0,  623858,       0,  0, 20,  0, }, // Phoenix_Islands
    {  498, 1502,  623878,  623906,  623941, 28, 35, 37, }, // Pierre_Miquelon
    {  498, 1518,       0,   63351,       0,  0, 16,  0, }, // Pitcairn
    {  498, 1527,       0,   63367,       0,  0, 14,  0, }, // Ponape
    {  498, 1534,       0,  143295,       0,  0, 17,  0, }, // Pyongyang
    {  498, 1544,   63398,  623978,  624002, 17, 24, 26, }, // Qyzylorda
    {  498, 1554,       0,  624028,       0,  0, 15,  0, }, // Reunion
    {  498, 1562,       0,   63482,       0,  0, 15,  0, }, // Rothera
    {  498, 1570,  624043,  624059,  624082, 16, 23, 25, }, // Sakhalin
    {  498, 1579,   63560,  624107,  624128, 14, 21, 23, }, // Samara
    {  498, 1596,   63620,  624151,  624171, 13, 20, 22, }, // Samoa
    {  498, 1602,       0,  624193,       0,  0, 18,  0, }, // Seychelles
    {  498, 1624,       0,  624211,       0,  0, 24,  0, }, // Singapore
    {  498, 1634,       0,  624235,       0,  0, 22,  0, }, // Solomon
    {  498, 1642,       0,  624257,       0,  0, 22,  0, }, // South_Georgia
    {  498, 1656,       0,  624279,       0,  0, 16,  0, }, // Suriname
    {  498, 1676,       0,   63786,       0,  0, 13,  0, }, // Syowa
    {  498, 1682,       0,  624295,       0,  0, 13,  0, }, // Tahiti
    {  498, 1689,   63813,  624308,  624328, 13, 20, 22, }, // Taipei
    {  498, 1696,       0,  624350,       0,  0, 19,  0, }, // Tajikistan
    {  498, 1724,       0,   63893,       0,  0, 15,  0, }, // Tokelau
    {  498, 1732,   63908,  624369,  624389, 13, 20, 22, }, // Tonga
    {  498, 1738,       0,   63965,       0,  0, 13,  0, }, // Truk
    {  498, 1750,  624411,  624433,  624462, 22, 29, 31, }, // Turkmenistan
    {  498, 1763,       0,   64059,       0,  0, 14,  0, }, // Tuvalu
    {  498, 1777,  624493,  624508,  624530, 15, 22, 24, }, // Uruguay
    {  498, 1792,  624554,  624573,  624599, 19, 26, 28, }, // Uzbekistan
    {  498, 1803,  624627,  624642,  624664, 15, 22, 24, }, // Vanuatu
    {  498, 1811,       0,  624688,       0,  0, 17,  0, }, // Venezuela
    {  498, 1821,   64297,  624705,  624731, 19, 26, 28, }, // Vladivostok
    {  498, 1833,  624759,  624777,  624802, 18, 25, 27, }, // Volgograd
    {  498, 1843,       0,   64441,       0,  0, 14,  0, }, // Vostok
    {  498, 1850,       0,  624829,       0,  0, 17,  0, }, // Wake
    {  498, 1855,       0,  624846,       0,  0, 23,  0, }, // Wallis
    {  498, 1862,   64499,  624869,  624891, 15, 22, 24, }, // Yakutsk
    {  498, 1870,  624915,  624936,  624964, 21, 28, 30, }, // Yekaterinburg
    {  498, 1892,       0,  624994,       0,  0, 13,  0, }, // Yukon
    {  499,    0,  618094,  618106,  618125, 12, 19, 21, }, // Acre Portuguese/Latin/Switzerland
    {  499,    5,       0,  618146,       0,  0, 19,  0, }, // Afghanistan
    {  499,   17,       0,  618165,       0,  0, 22,  0, }, // Africa_Central
    {  499,   32,       0,  618187,       0,  0, 23,  0, }, // Africa_Eastern
    {  499,   65,       0,  618210,       0,  0, 21,  0, }, // Africa_Southern
    {  499,   81,  618231,  618255,  618286, 24, 31, 33, }, // Africa_Western
    {  499,  107,  618319,  618333,  618354, 14, 21, 23, }, // Alaska
    {  499,  128,  618377,  618391,  618412, 14, 21, 23, }, // Almaty
    {  499,  135,  618435,  618451,  618474, 16, 23, 25, }, // Amazon
    {  499,  142,  618499,  618527,  618562, 28, 35, 37, }, // America_Central
    {  499,  158,  618599,  618628,  618664, 29, 36, 38, }, // America_Eastern
    {  499,  174,  618702,  618734,  618773, 32, 39, 41, }, // America_Mountain
    {  499,  191,  618814,  618846,  618885, 32, 39, 41, }, // America_Pacific
    {  499,  207,  618926,  618940,  618961, 14, 21, 23, }, // Anadyr
    {  499,  214,  618984,  618996,  619015, 12, 19, 21, }, // Apia
    {  499,  219,  619036,  619049,  619069, 13, 20, 22, }, // Aqtau
    {  499,  225,  619091,  619105,  619126, 14, 21, 23, }, // Aqtobe
    {  499,  232,  619149,  619163,  619184, 14, 21, 23, }, // Arabian
    {  499,  240,  619207,  619224,  619248, 17, 24, 26, }, // Argentina
    {  499,  250,  619274,  619301,  619335, 27, 34, 36, }, // Argentina_Western
    {  499,  268,  619371,  619386,  619408, 15, 22, 24, }, // Armenia
    {  499,  286,  619432,  619449,  619473, 17, 24, 26, }, // Atlantic
    {  499,  295,  619499,  619524,  619556, 25, 32, 34, }, // Australia_Central
    {  499,  313,  619590,  619625,  619667, 35, 42, 44, }, // Australia_CentralWestern
    {  499,  338,  619711,  619737,  619770, 26, 33, 35, }, // Australia_Eastern
    {  499,  356,  619805,  619832,  619866, 27, 34, 36, }, // Australia_Western
    {  499,  374,  619902,  619920,  619945, 18, 25, 27, }, // Azerbaijan
    {  499,  385,  619972,  619987,  620009, 15, 22, 24, }, // Azores
    {  499,  397,  620033,  620052,  620078, 19, 26, 28, }, // Bangladesh
    {  499,  415,       0,  620106,       0,  0, 13,  0, }, // Bhutan
    {  499,  422,       0,  620119,       0,  0, 15,  0, }, // Bolivia
    {  499,  437,  139301,  620134,  620157, 16, 23, 25, }, // Brasilia
    {  499,  454,       0,  620182,       0,  0, 25,  0, }, // Brunei
    {  499,  461,  620207,  620225,  620250, 18, 25, 27, }, // Cape_Verde
    {  499,  478,       0,  620277,       0,  0, 23,  0, }, // Chamorro
    {  499,  487,   59218,  620300,  620322, 15, 22, 24, }, // Chatham
    {  499,  495,  620346,  620359,  620379, 13, 20, 22, }, // Chile
    {  499,  501,  620401,  620414,  620434, 13, 20, 22, }, // China
    {  499,  507,       0,  620456,       0,  0, 21,  0, }, // Christmas
    {  499,  517,       0,  620477,       0,  0, 20,  0, }, // Cocos
    {  499,  523,  620497,  620513,  620536, 16, 23, 25, }, // Colombia
    {  499,  532,  620561,  620580,  620606, 19, 26, 28, }, // Cook
    {  499,  537,   59613,  620634,  620653, 12, 19, 21, }, // Cuba
    {  499,  548,       0,   59667,       0,  0, 13,  0, }, // Davis
    {  499,  564,       0,   59680,       0,  0, 24,  0, }, // DumontDUrville
    {  499,  601,       0,  620674,       0,  0, 19,  0, }, // East_Timor
    {  499,  612,  620693,  620715,  620744, 22, 29, 31, }, // Easter
    {  499,  619,       0,  620775,       0,  0, 15,  0, }, // Ecuador
    {  499,  627,  620790,  620812,  620841, 22, 29, 31, }, // Europe_Central
    {  499,  642,  620872,  620895,  620925, 23, 30, 32, }, // Europe_Eastern
    {  499,  657,       0,  620957,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  499,  680,  620988,  621012,  621043, 24, 31, 33, }, // Europe_Western
    {  499,  695,  621076,  621099,  621129, 23, 30, 32, }, // Falkland
    {  499,  704,  140403,  621161,  621180, 12, 19, 21, }, // Fiji
    {  499,  709,       0,  621201,       0,  0, 23,  0, }, // French_Guiana
    {  499,  723,       0,  621224,       0,  0, 48,  0, }, // French_Southern
    {  499,  746,       0,  621272,       0,  0, 18,  0, }, // Galapagos
    {  499,  756,       0,   60351,       0,  0, 15,  0, }, // Gambier
    {  499,  764,  621290,  621305,  621327, 15, 22, 24, }, // Georgia
    {  499,  772,       0,  621351,       0,  0, 22,  0, }, // Gilbert_Islands
    {  499,  788,       0,  621373,       0,  0, 17,  0, }, // GMT
    {  499,  830,  621390,  621418,  621453, 28, 35, 37, }, // Greenland_Eastern
    {  499,  848,  621490,  621519,  621555, 29, 36, 38, }, // Greenland_Western
    {  499,  866,       0,  621593,       0,  0, 19,  0, }, // Guam
    {  499,  871,       0,  621612,       0,  0, 20,  0, }, // Gulf
    {  499,  876,       0,  621201,       0,  0, 14,  0, }, // Guyana
    {  499,  883,  621632,  621655,  621685, 23, 30, 32, }, // Hawaii_Aleutian
    {  499,  899,  141051,  621717,  621741, 17, 24, 26, }, // Hong_Kong
    {  499,  909,   60891,  621767,  621786, 12, 19, 21, }, // Hovd
    {  499,  914,       0,  621807,       0,  0, 20,  0, }, // India
    {  499,  920,       0,  621827,       0,  0, 21,  0, }, // Indian_Ocean
    {  499,  933,       0,  621848,       0,  0, 17,  0, }, // Indochina
    {  499,  943,       0,  621865,       0,  0, 25,  0, }, // Indonesia_Central
    {  499,  961,       0,  621890,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  499,  979,       0,  621916,       0,  0, 27,  0, }, // Indonesia_Western
    {  499,  997,  621943,  621955,  621974, 12, 19, 21, }, // Iran
    {  499, 1008,  621995,  622010,  622032, 15, 22, 24, }, // Irkutsk
    {  499, 1016,  622056,  622070,  622091, 14, 21, 23, }, // Israel
    {  499, 1023,  622114,  622127,  622147, 13, 20, 22, }, // Japan
    {  499, 1029,  622169,  622201,  622240, 32, 39, 41, }, // Kamchatka
    {  499, 1047,       0,  622281,       0,  0, 19,  0, }, // Kazakhstan
    {  499, 1058,       0,  622300,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  499, 1077,       0,  622328,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  499, 1106,  622357,  622371,  622392, 14, 21, 23, }, // Korea
    {  499, 1112,       0,   61535,       0,  0, 14,  0, }, // Kosrae
    {  499, 1119,   61549,  622415,  622441, 19, 26, 28, }, // Krasnoyarsk
    {  499, 1151,       0,  622469,       0,  0, 19,  0, }, // Kyrgystan
    {  499, 1161,       0,  622488,       0,  0, 17,  0, }, // Lanka
    {  499, 1175,       0,  622505,       0,  0, 19,  0, }, // Line_Islands
    {  499, 1188,   61679,  622524,  622548, 17, 24, 26, }, // Lord_Howe
    {  499, 1198,  141908,  622574,  622594, 13, 20, 22, }, // Macau
    {  499, 1204,  141965,  622616,  622638, 15, 22, 24, }, // Magadan
    {  499, 1219,       0,  622662,       0,  0, 15,  0, }, // Malaysia
    {  499, 1228,       0,  622677,       0,  0, 17,  0, }, // Maldives
    {  499, 1237,       0,  622694,       0,  0, 24,  0, }, // Marquesas
    {  499, 1247,       0,  622718,       0,  0, 23,  0, }, // Marshall_Islands
    {  499, 1264,  622741,  622757,  622780, 16, 23, 25, }, // Mauritius
    {  499, 1274,       0,   62017,       0,  0, 14,  0, }, // Mawson
    {  499, 1281,  622805,  622830,  622862, 25, 32, 34, }, // Mexico_Pacific
    {  499, 1296,  622896,  622914,  622939, 18, 25, 27, }, // Mongolia
    {  499, 1305,  622966,  622981,  623003, 15, 22, 24, }, // Moscow
    {  499, 1312,       0,  623027,       0,  0, 15,  0, }, // Myanmar
    {  499, 1320,       0,   62271,       0,  0, 13,  0, }, // Nauru
    {  499, 1326,       0,  623042,       0,  0, 13,  0, }, // Nepal
    {  499, 1332,  623055,  623077,  623106, 22, 29, 31, }, // New_Caledonia
    {  499, 1346,  623137,  623158,  623186, 21, 28, 30, }, // New_Zealand
    {  499, 1358,  623216,  623234,  623259, 18, 25, 27, }, // Newfoundland
    {  499, 1371,       0,  623286,       0,  0, 12,  0, }, // Niue
    {  499, 1376,  623298,  623318,  623345, 20, 27, 29, }, // Norfolk
    {  499, 1384,   62646,  623374,  623408, 27, 34, 36, }, // Noronha
    {  499, 1392,       0,  623444,       0,  0, 31,  0, }, // North_Mariana
    {  499, 1406,   62782,  623475,  623501, 19, 26, 28, }, // Novosibirsk
    {  499, 1418,  623529,  623541,  623560, 12, 19, 21, }, // Omsk
    {  499, 1428,  623581,  623598,  623622, 17, 24, 26, }, // Pakistan
    {  499, 1437,       0,   62980,       0,  0, 13,  0, }, // Palau
    {  499, 1443,       0,  623648,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  499, 1460,  623672,  623688,  623711, 16, 23, 25, }, // Paraguay
    {  499, 1469,  623736,  623748,  623767, 12, 19, 21, }, // Peru
    {  499, 1474,  623788,  623806,  623831, 18, 25, 27, }, // Philippines
    {  499, 1486,       0,  623858,       0,  0, 20,  0, }, // Phoenix_Islands
    {  499, 1502,  623878,  623906,  623941, 28, 35, 37, }, // Pierre_Miquelon
    {  499, 1518,       0,   63351,       0,  0, 16,  0, }, // Pitcairn
    {  499, 1527,       0,   63367,       0,  0, 14,  0, }, // Ponape
    {  499, 1534,       0,  143295,       0,  0, 17,  0, }, // Pyongyang
    {  499, 1544,   63398,  623978,  624002, 17, 24, 26, }, // Qyzylorda
    {  499, 1554,       0,  624028,       0,  0, 15,  0, }, // Reunion
    {  499, 1562,       0,   63482,       0,  0, 15,  0, }, // Rothera
    {  499, 1570,  624043,  624059,  624082, 16, 23, 25, }, // Sakhalin
    {  499, 1579,   63560,  624107,  624128, 14, 21, 23, }, // Samara
    {  499, 1596,   63620,  624151,  624171, 13, 20, 22, }, // Samoa
    {  499, 1602,       0,  624193,       0,  0, 18,  0, }, // Seychelles
    {  499, 1624,       0,  624211,       0,  0, 24,  0, }, // Singapore
    {  499, 1634,       0,  624235,       0,  0, 22,  0, }, // Solomon
    {  499, 1642,       0,  624257,       0,  0, 22,  0, }, // South_Georgia
    {  499, 1656,       0,  624279,       0,  0, 16,  0, }, // Suriname
    {  499, 1676,       0,   63786,       0,  0, 13,  0, }, // Syowa
    {  499, 1682,       0,  624295,       0,  0, 13,  0, }, // Tahiti
    {  499, 1689,   63813,  624308,  624328, 13, 20, 22, }, // Taipei
    {  499, 1696,       0,  624350,       0,  0, 19,  0, }, // Tajikistan
    {  499, 1724,       0,   63893,       0,  0, 15,  0, }, // Tokelau
    {  499, 1732,   63908,  624369,  624389, 13, 20, 22, }, // Tonga
    {  499, 1738,       0,   63965,       0,  0, 13,  0, }, // Truk
    {  499, 1750,  624411,  624433,  624462, 22, 29, 31, }, // Turkmenistan
    {  499, 1763,       0,   64059,       0,  0, 14,  0, }, // Tuvalu
    {  499, 1777,  624493,  624508,  624530, 15, 22, 24, }, // Uruguay
    {  499, 1792,  624554,  624573,  624599, 19, 26, 28, }, // Uzbekistan
    {  499, 1803,  624627,  624642,  624664, 15, 22, 24, }, // Vanuatu
    {  499, 1811,       0,  624688,       0,  0, 17,  0, }, // Venezuela
    {  499, 1821,   64297,  624705,  624731, 19, 26, 28, }, // Vladivostok
    {  499, 1833,  624759,  624777,  624802, 18, 25, 27, }, // Volgograd
    {  499, 1843,       0,   64441,       0,  0, 14,  0, }, // Vostok
    {  499, 1850,       0,  624829,       0,  0, 17,  0, }, // Wake
    {  499, 1855,       0,  624846,       0,  0, 23,  0, }, // Wallis
    {  499, 1862,   64499,  624869,  624891, 15, 22, 24, }, // Yakutsk
    {  499, 1870,  624915,  624936,  624964, 21, 28, 30, }, // Yekaterinburg
    {  499, 1892,       0,  624994,       0,  0, 13,  0, }, // Yukon
    {  500,    0,  618094,  618106,  618125, 12, 19, 21, }, // Acre Portuguese/Latin/Timor-Leste
    {  500,    5,       0,  618146,       0,  0, 19,  0, }, // Afghanistan
    {  500,   17,       0,  618165,       0,  0, 22,  0, }, // Africa_Central
    {  500,   32,       0,  618187,       0,  0, 23,  0, }, // Africa_Eastern
    {  500,   65,       0,  618210,       0,  0, 21,  0, }, // Africa_Southern
    {  500,   81,  618231,  618255,  618286, 24, 31, 33, }, // Africa_Western
    {  500,  107,  618319,  618333,  618354, 14, 21, 23, }, // Alaska
    {  500,  128,  618377,  618391,  618412, 14, 21, 23, }, // Almaty
    {  500,  135,  618435,  618451,  618474, 16, 23, 25, }, // Amazon
    {  500,  142,  618499,  618527,  618562, 28, 35, 37, }, // America_Central
    {  500,  158,  618599,  618628,  618664, 29, 36, 38, }, // America_Eastern
    {  500,  174,  618702,  618734,  618773, 32, 39, 41, }, // America_Mountain
    {  500,  191,  618814,  618846,  618885, 32, 39, 41, }, // America_Pacific
    {  500,  207,  618926,  618940,  618961, 14, 21, 23, }, // Anadyr
    {  500,  214,  618984,  618996,  619015, 12, 19, 21, }, // Apia
    {  500,  219,  619036,  619049,  619069, 13, 20, 22, }, // Aqtau
    {  500,  225,  619091,  619105,  619126, 14, 21, 23, }, // Aqtobe
    {  500,  232,  619149,  619163,  619184, 14, 21, 23, }, // Arabian
    {  500,  240,  619207,  619224,  619248, 17, 24, 26, }, // Argentina
    {  500,  250,  619274,  619301,  619335, 27, 34, 36, }, // Argentina_Western
    {  500,  268,  619371,  619386,  619408, 15, 22, 24, }, // Armenia
    {  500,  286,  619432,  619449,  619473, 17, 24, 26, }, // Atlantic
    {  500,  295,  619499,  619524,  619556, 25, 32, 34, }, // Australia_Central
    {  500,  313,  619590,  619625,  619667, 35, 42, 44, }, // Australia_CentralWestern
    {  500,  338,  619711,  619737,  619770, 26, 33, 35, }, // Australia_Eastern
    {  500,  356,  619805,  619832,  619866, 27, 34, 36, }, // Australia_Western
    {  500,  374,  619902,  619920,  619945, 18, 25, 27, }, // Azerbaijan
    {  500,  385,  619972,  619987,  620009, 15, 22, 24, }, // Azores
    {  500,  397,  620033,  620052,  620078, 19, 26, 28, }, // Bangladesh
    {  500,  415,       0,  620106,       0,  0, 13,  0, }, // Bhutan
    {  500,  422,       0,  620119,       0,  0, 15,  0, }, // Bolivia
    {  500,  437,  139301,  620134,  620157, 16, 23, 25, }, // Brasilia
    {  500,  454,       0,  620182,       0,  0, 25,  0, }, // Brunei
    {  500,  461,  620207,  620225,  620250, 18, 25, 27, }, // Cape_Verde
    {  500,  478,       0,  620277,       0,  0, 23,  0, }, // Chamorro
    {  500,  487,   59218,  620300,  620322, 15, 22, 24, }, // Chatham
    {  500,  495,  620346,  620359,  620379, 13, 20, 22, }, // Chile
    {  500,  501,  620401,  620414,  620434, 13, 20, 22, }, // China
    {  500,  507,       0,  620456,       0,  0, 21,  0, }, // Christmas
    {  500,  517,       0,  620477,       0,  0, 20,  0, }, // Cocos
    {  500,  523,  620497,  620513,  620536, 16, 23, 25, }, // Colombia
    {  500,  532,  620561,  620580,  620606, 19, 26, 28, }, // Cook
    {  500,  537,   59613,  620634,  620653, 12, 19, 21, }, // Cuba
    {  500,  548,       0,   59667,       0,  0, 13,  0, }, // Davis
    {  500,  564,       0,   59680,       0,  0, 24,  0, }, // DumontDUrville
    {  500,  601,       0,  620674,       0,  0, 19,  0, }, // East_Timor
    {  500,  612,  620693,  620715,  620744, 22, 29, 31, }, // Easter
    {  500,  619,       0,  620775,       0,  0, 15,  0, }, // Ecuador
    {  500,  627,  620790,  620812,  620841, 22, 29, 31, }, // Europe_Central
    {  500,  642,  620872,  620895,  620925, 23, 30, 32, }, // Europe_Eastern
    {  500,  657,       0,  620957,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  500,  680,  620988,  621012,  621043, 24, 31, 33, }, // Europe_Western
    {  500,  695,  621076,  621099,  621129, 23, 30, 32, }, // Falkland
    {  500,  704,  140403,  621161,  621180, 12, 19, 21, }, // Fiji
    {  500,  709,       0,  621201,       0,  0, 23,  0, }, // French_Guiana
    {  500,  723,       0,  621224,       0,  0, 48,  0, }, // French_Southern
    {  500,  746,       0,  621272,       0,  0, 18,  0, }, // Galapagos
    {  500,  756,       0,   60351,       0,  0, 15,  0, }, // Gambier
    {  500,  764,  621290,  621305,  621327, 15, 22, 24, }, // Georgia
    {  500,  772,       0,  621351,       0,  0, 22,  0, }, // Gilbert_Islands
    {  500,  788,       0,  621373,       0,  0, 17,  0, }, // GMT
    {  500,  830,  621390,  621418,  621453, 28, 35, 37, }, // Greenland_Eastern
    {  500,  848,  621490,  621519,  621555, 29, 36, 38, }, // Greenland_Western
    {  500,  866,       0,  621593,       0,  0, 19,  0, }, // Guam
    {  500,  871,       0,  621612,       0,  0, 20,  0, }, // Gulf
    {  500,  876,       0,  621201,       0,  0, 14,  0, }, // Guyana
    {  500,  883,  621632,  621655,  621685, 23, 30, 32, }, // Hawaii_Aleutian
    {  500,  899,  141051,  621717,  621741, 17, 24, 26, }, // Hong_Kong
    {  500,  909,   60891,  621767,  621786, 12, 19, 21, }, // Hovd
    {  500,  914,       0,  621807,       0,  0, 20,  0, }, // India
    {  500,  920,       0,  621827,       0,  0, 21,  0, }, // Indian_Ocean
    {  500,  933,       0,  621848,       0,  0, 17,  0, }, // Indochina
    {  500,  943,       0,  621865,       0,  0, 25,  0, }, // Indonesia_Central
    {  500,  961,       0,  621890,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  500,  979,       0,  621916,       0,  0, 27,  0, }, // Indonesia_Western
    {  500,  997,  621943,  621955,  621974, 12, 19, 21, }, // Iran
    {  500, 1008,  621995,  622010,  622032, 15, 22, 24, }, // Irkutsk
    {  500, 1016,  622056,  622070,  622091, 14, 21, 23, }, // Israel
    {  500, 1023,  622114,  622127,  622147, 13, 20, 22, }, // Japan
    {  500, 1029,  622169,  622201,  622240, 32, 39, 41, }, // Kamchatka
    {  500, 1047,       0,  622281,       0,  0, 19,  0, }, // Kazakhstan
    {  500, 1058,       0,  622300,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  500, 1077,       0,  622328,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  500, 1106,  622357,  622371,  622392, 14, 21, 23, }, // Korea
    {  500, 1112,       0,   61535,       0,  0, 14,  0, }, // Kosrae
    {  500, 1119,   61549,  622415,  622441, 19, 26, 28, }, // Krasnoyarsk
    {  500, 1151,       0,  622469,       0,  0, 19,  0, }, // Kyrgystan
    {  500, 1161,       0,  622488,       0,  0, 17,  0, }, // Lanka
    {  500, 1175,       0,  622505,       0,  0, 19,  0, }, // Line_Islands
    {  500, 1188,   61679,  622524,  622548, 17, 24, 26, }, // Lord_Howe
    {  500, 1198,  141908,  622574,  622594, 13, 20, 22, }, // Macau
    {  500, 1204,  141965,  622616,  622638, 15, 22, 24, }, // Magadan
    {  500, 1219,       0,  622662,       0,  0, 15,  0, }, // Malaysia
    {  500, 1228,       0,  622677,       0,  0, 17,  0, }, // Maldives
    {  500, 1237,       0,  622694,       0,  0, 24,  0, }, // Marquesas
    {  500, 1247,       0,  622718,       0,  0, 23,  0, }, // Marshall_Islands
    {  500, 1264,  622741,  622757,  622780, 16, 23, 25, }, // Mauritius
    {  500, 1274,       0,   62017,       0,  0, 14,  0, }, // Mawson
    {  500, 1281,  622805,  622830,  622862, 25, 32, 34, }, // Mexico_Pacific
    {  500, 1296,  622896,  622914,  622939, 18, 25, 27, }, // Mongolia
    {  500, 1305,  622966,  622981,  623003, 15, 22, 24, }, // Moscow
    {  500, 1312,       0,  623027,       0,  0, 15,  0, }, // Myanmar
    {  500, 1320,       0,   62271,       0,  0, 13,  0, }, // Nauru
    {  500, 1326,       0,  623042,       0,  0, 13,  0, }, // Nepal
    {  500, 1332,  623055,  623077,  623106, 22, 29, 31, }, // New_Caledonia
    {  500, 1346,  623137,  623158,  623186, 21, 28, 30, }, // New_Zealand
    {  500, 1358,  623216,  623234,  623259, 18, 25, 27, }, // Newfoundland
    {  500, 1371,       0,  623286,       0,  0, 12,  0, }, // Niue
    {  500, 1376,  623298,  623318,  623345, 20, 27, 29, }, // Norfolk
    {  500, 1384,   62646,  623374,  623408, 27, 34, 36, }, // Noronha
    {  500, 1392,       0,  623444,       0,  0, 31,  0, }, // North_Mariana
    {  500, 1406,   62782,  623475,  623501, 19, 26, 28, }, // Novosibirsk
    {  500, 1418,  623529,  623541,  623560, 12, 19, 21, }, // Omsk
    {  500, 1428,  623581,  623598,  623622, 17, 24, 26, }, // Pakistan
    {  500, 1437,       0,   62980,       0,  0, 13,  0, }, // Palau
    {  500, 1443,       0,  623648,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  500, 1460,  623672,  623688,  623711, 16, 23, 25, }, // Paraguay
    {  500, 1469,  623736,  623748,  623767, 12, 19, 21, }, // Peru
    {  500, 1474,  623788,  623806,  623831, 18, 25, 27, }, // Philippines
    {  500, 1486,       0,  623858,       0,  0, 20,  0, }, // Phoenix_Islands
    {  500, 1502,  623878,  623906,  623941, 28, 35, 37, }, // Pierre_Miquelon
    {  500, 1518,       0,   63351,       0,  0, 16,  0, }, // Pitcairn
    {  500, 1527,       0,   63367,       0,  0, 14,  0, }, // Ponape
    {  500, 1534,       0,  143295,       0,  0, 17,  0, }, // Pyongyang
    {  500, 1544,   63398,  623978,  624002, 17, 24, 26, }, // Qyzylorda
    {  500, 1554,       0,  624028,       0,  0, 15,  0, }, // Reunion
    {  500, 1562,       0,   63482,       0,  0, 15,  0, }, // Rothera
    {  500, 1570,  624043,  624059,  624082, 16, 23, 25, }, // Sakhalin
    {  500, 1579,   63560,  624107,  624128, 14, 21, 23, }, // Samara
    {  500, 1596,   63620,  624151,  624171, 13, 20, 22, }, // Samoa
    {  500, 1602,       0,  624193,       0,  0, 18,  0, }, // Seychelles
    {  500, 1624,       0,  624211,       0,  0, 24,  0, }, // Singapore
    {  500, 1634,       0,  624235,       0,  0, 22,  0, }, // Solomon
    {  500, 1642,       0,  624257,       0,  0, 22,  0, }, // South_Georgia
    {  500, 1656,       0,  624279,       0,  0, 16,  0, }, // Suriname
    {  500, 1676,       0,   63786,       0,  0, 13,  0, }, // Syowa
    {  500, 1682,       0,  624295,       0,  0, 13,  0, }, // Tahiti
    {  500, 1689,   63813,  624308,  624328, 13, 20, 22, }, // Taipei
    {  500, 1696,       0,  624350,       0,  0, 19,  0, }, // Tajikistan
    {  500, 1724,       0,   63893,       0,  0, 15,  0, }, // Tokelau
    {  500, 1732,   63908,  624369,  624389, 13, 20, 22, }, // Tonga
    {  500, 1738,       0,   63965,       0,  0, 13,  0, }, // Truk
    {  500, 1750,  624411,  624433,  624462, 22, 29, 31, }, // Turkmenistan
    {  500, 1763,       0,   64059,       0,  0, 14,  0, }, // Tuvalu
    {  500, 1777,  624493,  624508,  624530, 15, 22, 24, }, // Uruguay
    {  500, 1792,  624554,  624573,  624599, 19, 26, 28, }, // Uzbekistan
    {  500, 1803,  624627,  624642,  624664, 15, 22, 24, }, // Vanuatu
    {  500, 1811,       0,  624688,       0,  0, 17,  0, }, // Venezuela
    {  500, 1821,   64297,  624705,  624731, 19, 26, 28, }, // Vladivostok
    {  500, 1833,  624759,  624777,  624802, 18, 25, 27, }, // Volgograd
    {  500, 1843,       0,   64441,       0,  0, 14,  0, }, // Vostok
    {  500, 1850,       0,  624829,       0,  0, 17,  0, }, // Wake
    {  500, 1855,       0,  624846,       0,  0, 23,  0, }, // Wallis
    {  500, 1862,   64499,  624869,  624891, 15, 22, 24, }, // Yakutsk
    {  500, 1870,  624915,  624936,  624964, 21, 28, 30, }, // Yekaterinburg
    {  500, 1892,       0,  624994,       0,  0, 13,  0, }, // Yukon
    {  501,  142,  625007,  625031,  625062, 24, 31, 31, }, // America_Central Prussian/Latin/Poland
    {  501,  158,  625093,  625117,  625148, 24, 31, 31, }, // America_Eastern
    {  501,  174,  625179,  625200,  625228, 21, 28, 28, }, // America_Mountain
    {  501,  191,  625256,  625281,  625313, 25, 32, 32, }, // America_Pacific
    {  501,  286,  625345,  625361,  625384, 16, 23, 23, }, // Atlantic
    {  501,  627,  625407,  625430,  625460, 23, 30, 30, }, // Europe_Central
    {  501,  642,  625490,  625513,  625543, 23, 30, 30, }, // Europe_Eastern
    {  501,  680,  625573,  625598,  625630, 25, 32, 32, }, // Europe_Western
    {  501,  788,       0,  625662,       0,  0, 15,  0, }, // GMT
    {  502,    5,       0,  625677,       0,  0, 16,  0, }, // Afghanistan Punjabi/Gurmukhi/India
    {  502,   17,       0,  625693,       0,  0, 18,  0, }, // Africa_Central
    {  502,   32,       0,  625711,       0,  0, 17,  0, }, // Africa_Eastern
    {  502,   65,       0,  625728,       0,  0, 24,  0, }, // Africa_Southern
    {  502,   81,  625752,  625769,  625792, 17, 23, 27, }, // Africa_Western
    {  502,  107,  625819,  625830,  625847, 11, 17, 19, }, // Alaska
    {  502,  128,  625866,  625877,  625894, 11, 17, 21, }, // Almaty
    {  502,  135,  625915,  625926,  625943, 11, 17, 21, }, // Amazon
    {  502,  142,  625964,  625988,  626018, 24, 30, 32, }, // America_Central
    {  502,  158,  626050,  626073,  626102, 23, 29, 31, }, // America_Eastern
    {  502,  174,  626133,  626158,  626189, 25, 31, 33, }, // America_Mountain
    {  502,  191,  626222,  626247,  626278, 25, 31, 33, }, // America_Pacific
    {  502,  214,  626311,  626320,  626335,  9, 15, 17, }, // Apia
    {  502,  219,  626352,  626362,  626378, 10, 16, 20, }, // Aqtau
    {  502,  225,  626398,  626408,  626424, 10, 16, 20, }, // Aqtobe
    {  502,  232,  626444,  626453,  626468,  9, 15, 17, }, // Arabian
    {  502,  240,  626485,  626498,  626517, 13, 19, 23, }, // Argentina
    {  502,  250,  626540,  626559,  626584, 19, 25, 29, }, // Argentina_Western
    {  502,  268,  626613,  626625,  626643, 12, 18, 22, }, // Armenia
    {  502,  286,  626665,  626678,  626697, 13, 19, 21, }, // Atlantic
    {  502,  295,  626718,  626740,  626768, 22, 28, 30, }, // Australia_Central
    {  502,  313,  626798,  626826,  626860, 28, 34, 36, }, // Australia_CentralWestern
    {  502,  338,  626896,  626917,  626944, 21, 27, 29, }, // Australia_Eastern
    {  502,  356,  626973,  626994,  627021, 21, 27, 29, }, // Australia_Western
    {  502,  374,  627050,  627065,  627086, 15, 21, 25, }, // Azerbaijan
    {  502,  385,  627111,  627122,  627139, 11, 17, 21, }, // Azores
    {  502,  397,  627160,  627174,  627194, 14, 20, 24, }, // Bangladesh
    {  502,  415,       0,  627218,       0,  0, 10,  0, }, // Bhutan
    {  502,  422,       0,  627228,       0,  0, 12,  0, }, // Bolivia
    {  502,  437,  627240,  627255,  627276, 15, 21, 25, }, // Brasilia
    {  502,  454,       0,  627301,       0,  0, 20,  0, }, // Brunei
    {  502,  461,  627321,  627333,  627351, 12, 18, 22, }, // Cape_Verde
    {  502,  472,       0,  627373,       0,  0,  9,  0, }, // Casey
    {  502,  478,       0,  627382,       0,  0, 17,  0, }, // Chamorro
    {  502,  487,  627399,  627408,  627423,  9, 15, 17, }, // Chatham
    {  502,  495,  627440,  627449,  627464,  9, 15, 19, }, // Chile
    {  502,  501,  627483,  627491,  627505,  8, 14, 16, }, // China
    {  502,  507,       0,  627521,       0,  0, 19,  0, }, // Christmas
    {  502,  517,       0,  627540,       0,  0, 16,  0, }, // Cocos
    {  502,  523,  627556,  627568,  627586, 12, 18, 22, }, // Colombia
    {  502,  532,  627608,  627624,  627646, 16, 22, 30, }, // Cook
    {  502,  537,  627676,  627686,  627702, 10, 16, 18, }, // Cuba
    {  502,  548,       0,  627720,       0,  0, 10,  0, }, // Davis
    {  502,  564,       0,  627730,       0,  0, 21,  0, }, // DumontDUrville
    {  502,  601,       0,  627751,       0,  0, 16,  0, }, // East_Timor
    {  502,  612,  627767,  627783,  627805, 16, 22, 26, }, // Easter
    {  502,  619,       0,  627831,       0,  0, 12,  0, }, // Ecuador
    {  502,  627,  627843,  627857,  627877, 14, 20, 24, }, // Europe_Central
    {  502,  642,  627901,  627917,  627939, 16, 22, 26, }, // Europe_Eastern
    {  502,  657,       0,  627965,       0,  0, 20,  0, }, // Europe_Further_Eastern
    {  502,  680,  627985,  628001,  628023, 16, 22, 26, }, // Europe_Western
    {  502,  695,  628049,  628070,  628097, 21, 27, 31, }, // Falkland
    {  502,  704,  628128,  628139,  628156, 11, 17, 21, }, // Fiji
    {  502,  709,       0,  628177,       0,  0, 17,  0, }, // French_Guiana
    {  502,  723,       0,  628194,       0,  0, 30,  0, }, // French_Southern
    {  502,  746,       0,  628224,       0,  0, 13,  0, }, // Galapagos
    {  502,  756,       0,  628237,       0,  0, 12,  0, }, // Gambier
    {  502,  764,  628249,  628260,  628277, 11, 17, 21, }, // Georgia
    {  502,  772,       0,  628298,       0,  0, 18,  0, }, // Gilbert_Islands
    {  502,  788,       0,  628316,       0,  0, 17,  0, }, // GMT
    {  502,  830,  628333,  628353,  628379, 20, 26, 30, }, // Greenland_Eastern
    {  502,  848,  628409,  628429,  628455, 20, 26, 30, }, // Greenland_Western
    {  502,  866,       0,  628485,       0,  0,  9,  0, }, // Guam
    {  502,  871,       0,  628494,       0,  0, 15,  0, }, // Gulf
    {  502,  876,       0,  628509,       0,  0, 11,  0, }, // Guyana
    {  502,  883,  628520,  628540,  628566, 20, 26, 28, }, // Hawaii_Aleutian
    {  502,  899,  628594,  628608,  628628, 14, 20, 24, }, // Hong_Kong
    {  502,  909,  628652,  628661,  628676,  9, 15, 19, }, // Hovd
    {  502,  914,       0,  628695,       0,  0, 16,  0, }, // India
    {  502,  920,       0,  628711,       0,  0, 18,  0, }, // Indian_Ocean
    {  502,  933,       0,  628729,       0,  0, 14,  0, }, // Indochina
    {  502,  943,       0,  628743,       0,  0, 20,  0, }, // Indonesia_Central
    {  502,  961,       0,  628763,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  502,  979,       0,  628784,       0,  0, 21,  0, }, // Indonesia_Western
    {  502,  997,  628805,  628814,  628829,  9, 15, 17, }, // Iran
    {  502, 1008,  628846,  628858,  628876, 12, 18, 22, }, // Irkutsk
    {  502, 1016,  628898,  628910,  628928, 12, 18, 20, }, // Israel
    {  502, 1023,  628948,  628957,  628972,  9, 15, 17, }, // Japan
    {  502, 1047,       0,  628989,       0,  0, 15,  0, }, // Kazakhstan
    {  502, 1058,       0,  629004,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {  502, 1077,       0,  629025,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  502, 1106,  629046,  629057,  629074, 11, 17, 19, }, // Korea
    {  502, 1112,       0,  629093,       0,  0, 10,  0, }, // Kosrae
    {  502, 1119,  629103,  629120,  629143, 17, 23, 27, }, // Krasnoyarsk
    {  502, 1151,       0,  629170,       0,  0, 14,  0, }, // Kyrgystan
    {  502, 1161,       0,  629184,       0,  0,  9,  0, }, // Lanka
    {  502, 1175,       0,  629193,       0,  0, 16,  0, }, // Line_Islands
    {  502, 1188,  629209,  629223,  629243, 14, 20, 22, }, // Lord_Howe
    {  502, 1198,  629265,  629274,  629289,  9, 15, 19, }, // Macau
    {  502, 1204,  629308,  629319,  629336, 11, 17, 21, }, // Magadan
    {  502, 1219,       0,  629357,       0,  0, 12,  0, }, // Malaysia
    {  502, 1228,       0,  629369,       0,  0, 11,  0, }, // Maldives
    {  502, 1237,       0,  629380,       0,  0, 13,  0, }, // Marquesas
    {  502, 1247,       0,  629393,       0,  0, 18,  0, }, // Marshall_Islands
    {  502, 1264,  629411,  629423,  629441, 12, 18, 22, }, // Mauritius
    {  502, 1274,       0,  629463,       0,  0,  9,  0, }, // Mawson
    {  502, 1281,  629472,  629492,  629518, 20, 26, 28, }, // Mexico_Pacific
    {  502, 1296,  629546,  629559,  629578, 13, 19, 23, }, // Mongolia
    {  502, 1305,  629601,  629611,  629627, 10, 16, 20, }, // Moscow
    {  502, 1312,       0,  629647,       0,  0, 12,  0, }, // Myanmar
    {  502, 1320,       0,  629659,       0,  0, 10,  0, }, // Nauru
    {  502, 1326,       0,  629669,       0,  0, 10,  0, }, // Nepal
    {  502, 1332,  629679,  629697,  629721, 18, 24, 28, }, // New_Caledonia
    {  502, 1346,  629749,  629764,  629785, 15, 21, 23, }, // New_Zealand
    {  502, 1358,  629808,  629825,  629848, 17, 23, 25, }, // Newfoundland
    {  502, 1371,       0,  629873,       0,  0,  8,  0, }, // Niue
    {  502, 1376,  629881,  629899,  629923, 18, 24, 26, }, // Norfolk
    {  502, 1384,  629949,  629972,  630001, 23, 29, 33, }, // Noronha
    {  502, 1392,       0,  630034,       0,  0, 24,  0, }, // North_Mariana
    {  502, 1406,  630058,  630074,  630096, 16, 22, 26, }, // Novosibirsk
    {  502, 1418,  630122,  630131,  630146,  9, 15, 19, }, // Omsk
    {  502, 1428,  630165,  630178,  630197, 13, 19, 23, }, // Pakistan
    {  502, 1437,       0,  630220,       0,  0,  9,  0, }, // Palau
    {  502, 1443,       0,  630229,       0,  0, 19,  0, }, // Papua_New_Guinea
    {  502, 1460,  630248,  630260,  630278, 12, 18, 22, }, // Paraguay
    {  502, 1469,  630300,  630309,  630324,  9, 15, 19, }, // Peru
    {  502, 1474,  630343,  630356,  630375, 13, 19, 23, }, // Philippines
    {  502, 1486,       0,  630398,       0,  0, 18,  0, }, // Phoenix_Islands
    {  502, 1502,  630416,  630442,  630474, 26, 32, 34, }, // Pierre_Miquelon
    {  502, 1518,       0,  630508,       0,  0, 12,  0, }, // Pitcairn
    {  502, 1527,       0,  630520,       0,  0, 11,  0, }, // Ponape
    {  502, 1534,       0,  630531,       0,  0, 14,  0, }, // Pyongyang
    {  502, 1544,  630545,  630559,  630579, 14, 20, 24, }, // Qyzylorda
    {  502, 1554,       0,  630603,       0,  0, 13,  0, }, // Reunion
    {  502, 1562,       0,  630616,       0,  0, 11,  0, }, // Rothera
    {  502, 1570,  630627,  630637,  630653, 10, 16, 20, }, // Sakhalin
    {  502, 1596,  630673,  630683,  630699, 10, 16, 18, }, // Samoa
    {  502, 1602,       0,  630717,       0,  0, 11,  0, }, // Seychelles
    {  502, 1624,       0,  630728,       0,  0, 19,  0, }, // Singapore
    {  502, 1634,       0,  630747,       0,  0, 19,  0, }, // Solomon
    {  502, 1642,       0,  630766,       0,  0, 17,  0, }, // South_Georgia
    {  502, 1656,       0,  630783,       0,  0, 12,  0, }, // Suriname
    {  502, 1676,       0,  630795,       0,  0, 10,  0, }, // Syowa
    {  502, 1682,       0,  630805,       0,  0, 11,  0, }, // Tahiti
    {  502, 1689,  630816,  630825,  630840,  9, 15, 17, }, // Taipei
    {  502, 1696,       0,  630857,       0,  0, 15,  0, }, // Tajikistan
    {  502, 1724,       0,  630872,       0,  0, 12,  0, }, // Tokelau
    {  502, 1732,  630884,  630894,  630910, 10, 16, 20, }, // Tonga
    {  502, 1738,       0,  630930,       0,  0,  8,  0, }, // Truk
    {  502, 1750,  630938,  630955,  630978, 17, 23, 27, }, // Turkmenistan
    {  502, 1763,       0,  631005,       0,  0, 11,  0, }, // Tuvalu
    {  502, 1777,  631016,  631027,  631044, 11, 17, 21, }, // Uruguay
    {  502, 1792,  631065,  631081,  631103, 16, 22, 26, }, // Uzbekistan
    {  502, 1803,  631129,  631141,  631159, 12, 18, 22, }, // Vanuatu
    {  502, 1811,       0,  631181,       0,  0, 15,  0, }, // Venezuela
    {  502, 1821,  631196,  631211,  631232, 15, 21, 25, }, // Vladivostok
    {  502, 1833,  631257,  631272,  631293, 15, 21, 25, }, // Volgograd
    {  502, 1843,       0,  631318,       0,  0, 11,  0, }, // Vostok
    {  502, 1850,       0,  631329,       0,  0, 15,  0, }, // Wake
    {  502, 1855,       0,  631344,       0,  0, 21,  0, }, // Wallis
    {  502, 1862,  631365,  631376,  631393, 11, 17, 21, }, // Yakutsk
    {  502, 1870,  631414,  631429,  631450, 15, 21, 25, }, // Yekaterinburg
    {  502, 1892,       0,  631475,       0,  0, 10,  0, }, // Yukon
    {  504,    5,       0,  631485,       0,  0, 18,  0, }, // Afghanistan Quechua/Latin/Peru
    {  504,   17,       0,  631503,       0,  0, 22,  0, }, // Africa_Central
    {  504,   32,       0,  631525,       0,  0, 23,  0, }, // Africa_Eastern
    {  504,   65,       0,  631548,       0,  0, 17,  0, }, // Africa_Southern
    {  504,   81,  631565,  631590,  631624, 25, 34, 44, }, // Africa_Western
    {  504,  107,  631668,  631682,  631705, 14, 23, 24, }, // Alaska
    {  504,  135,  631729,  631745,  631770, 16, 25, 26, }, // Amazon
    {  504,  142,  631796,  631808,  631830, 12, 22, 22, }, // America_Central
    {  504,  158,  631852,  631865,  631887, 13, 22, 23, }, // America_Eastern
    {  504,  174,  631910,  631928,  631955, 18, 27, 28, }, // America_Mountain
    {  504,  191,  631983,  632000,  632026, 17, 26, 27, }, // America_Pacific
    {  504,  214,  618984,  632053,  632074, 12, 21, 22, }, // Apia
    {  504,  232,  632096,  632110,  632133, 14, 23, 24, }, // Arabian
    {  504,  240,  632157,  632174,  632200, 17, 26, 27, }, // Argentina
    {  504,  250,  632227,  632254,  632290, 27, 36, 37, }, // Argentina_Western
    {  504,  268,  632327,  632342,  632366, 15, 24, 25, }, // Armenia
    {  504,  286,  632391,  632409,  632436, 18, 27, 28, }, // Atlantic
    {  504,  295,  632464,  632489,  632523, 25, 34, 35, }, // Australia_Central
    {  504,  313,  632558,  632594,  632639, 36, 45, 46, }, // Australia_CentralWestern
    {  504,  338,  632685,  632711,  632746, 26, 35, 36, }, // Australia_Eastern
    {  504,  356,  632782,  632810,  632847, 28, 37, 38, }, // Australia_Western
    {  504,  374,  632885,  632903,  632930, 18, 27, 28, }, // Azerbaijan
    {  504,  385,  632958,  632976,  633003, 18, 27, 28, }, // Azores
    {  504,  397,  139200,  633031,  633058, 18, 27, 28, }, // Bangladesh
    {  504,  415,       0,   58989,       0,  0, 13,  0, }, // Bhutan
    {  504,  422,       0,    1601,       0,  0, 12,  0, }, // Bolivia
    {  504,  437,   59017,  633086,  633111, 16, 25, 26, }, // Brasilia
    {  504,  454,       0,  139367,       0,  0, 25,  0, }, // Brunei
    {  504,  461,  620207,  633137,  633164, 18, 27, 28, }, // Cape_Verde
    {  504,  478,       0,  633192,       0,  0, 25,  0, }, // Chamorro
    {  504,  487,   59218,  633217,  633241, 15, 24, 25, }, // Chatham
    {  504,  495,   59281,  633266,  633288, 13, 22, 23, }, // Chile
    {  504,  501,   59338,  633311,  633333, 13, 22, 23, }, // China
    {  504,  507,       0,  633356,       0,  0, 22,  0, }, // Christmas
    {  504,  517,       0,  633378,       0,  0, 19,  0, }, // Cocos
    {  504,  523,   59454,  633397,  633422, 16, 25, 26, }, // Colombia
    {  504,  532,  633448,  633466,  633493, 18, 27, 28, }, // Cook
    {  504,  537,   59613,  633521,  633542, 12, 21, 22, }, // Cuba
    {  504,  548,       0,   59667,       0,  0, 13,  0, }, // Davis
    {  504,  564,       0,   59680,       0,  0, 24,  0, }, // DumontDUrville
    {  504,  601,       0,   59704,       0,  0, 22,  0, }, // East_Timor
    {  504,  612,  633564,  633586,  633617, 22, 31, 32, }, // Easter
    {  504,  619,       0,  633649,       0,  0, 15,  0, }, // Ecuador
    {  504,  627,  633664,  633686,  633717, 22, 31, 32, }, // Europe_Central
    {  504,  642,  633749,  633772,  633804, 23, 32, 33, }, // Europe_Eastern
    {  504,  657,       0,  633837,       0,  0, 37,  0, }, // Europe_Further_Eastern
    {  504,  680,  633874,  633899,  633933, 25, 34, 35, }, // Europe_Western
    {  504,  695,  633968,  633990,  634021, 22, 31, 32, }, // Falkland
    {  504,  704,  140403,  634053,  634074, 12, 21, 22, }, // Fiji
    {  504,  709,       0,  634096,       0,  0, 24,  0, }, // French_Guiana
    {  504,  723,       0,  634120,       0,  0, 35,  0, }, // French_Southern
    {  504,  746,       0,  634155,       0,  0, 23,  0, }, // Galapagos
    {  504,  756,       0,   60351,       0,  0, 15,  0, }, // Gambier
    {  504,  764,  634178,  634193,  634217, 15, 24, 25, }, // Georgia
    {  504,  772,       0,  634242,       0,  0, 21,  0, }, // Gilbert_Islands
    {  504,  788,       0,  634263,       0,  0, 31,  0, }, // GMT
    {  504,  830,   60475,  634294,  634322, 19, 28, 29, }, // Greenland_Eastern
    {  504,  848,  634351,  634381,  634420, 30, 39, 40, }, // Greenland_Western
    {  504,  871,       0,  634460,       0,  0, 23,  0, }, // Gulf
    {  504,  876,       0,  140969,       0,  0, 14,  0, }, // Guyana
    {  504,  883,  634483,  634506,  634538, 23, 32, 33, }, // Hawaii_Aleutian
    {  504,  899,  141051,  634571,  634597, 17, 26, 27, }, // Hong_Kong
    {  504,  909,   60891,  634624,  634645, 12, 21, 22, }, // Hovd
    {  504,  914,       0,  634667,       0,  0, 22,  0, }, // India
    {  504,  920,       0,  634689,       0,  0, 22,  0, }, // Indian_Ocean
    {  504,  933,       0,  634711,       0,  0, 17,  0, }, // Indochina
    {  504,  943,       0,  634728,       0,  0, 25,  0, }, // Indonesia_Central
    {  504,  961,       0,  634753,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  504,  979,       0,  634779,       0,  0, 28,  0, }, // Indonesia_Western
    {  504,  997,  634807,  634819,  634840, 12, 21, 22, }, // Iran
    {  504, 1008,  621995,  634862,  634886, 15, 24, 25, }, // Irkutsk
    {  504, 1016,  622056,  634911,  634934, 14, 23, 24, }, // Israel
    {  504, 1023,  634958,  634971,  634993, 13, 22, 23, }, // Japan
    {  504, 1047,       0,  635016,       0,  0, 18,  0, }, // Kazakhstan
    {  504, 1058,       0,  635034,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  504, 1077,       0,  635061,       0,  0, 28,  0, }, // Kazakhstan_Western
    {  504, 1106,   61478,  635089,  635111, 13, 22, 23, }, // Korea
    {  504, 1112,       0,   61535,       0,  0, 14,  0, }, // Kosrae
    {  504, 1119,   61549,  635134,  635162, 19, 28, 29, }, // Krasnoyarsk
    {  504, 1151,       0,  635191,       0,  0, 18,  0, }, // Kyrgystan
    {  504, 1175,       0,  635209,       0,  0, 30,  0, }, // Line_Islands
    {  504, 1188,   61679,  635239,  635265, 17, 26, 27, }, // Lord_Howe
    {  504, 1204,  141965,  635292,  635316, 15, 24, 25, }, // Magadan
    {  504, 1219,       0,   61868,       0,  0, 15,  0, }, // Malaysia
    {  504, 1228,       0,  635341,       0,  0, 16,  0, }, // Maldives
    {  504, 1237,       0,  635357,       0,  0, 17,  0, }, // Marquesas
    {  504, 1247,       0,  635374,       0,  0, 22,  0, }, // Marshall_Islands
    {  504, 1264,  635396,  635412,  635437, 16, 25, 26, }, // Mauritius
    {  504, 1274,       0,   62017,       0,  0, 14,  0, }, // Mawson
    {  504, 1281,  635463,  635489,  635524, 26, 35, 36, }, // Mexico_Pacific
    {  504, 1296,  635560,  635578,  635605, 18, 27, 28, }, // Mongolia
    {  504, 1305,   62199,  635633,  635655, 13, 22, 23, }, // Moscow
    {  504, 1312,       0,   62256,       0,  0, 15,  0, }, // Myanmar
    {  504, 1320,       0,   62271,       0,  0, 13,  0, }, // Nauru
    {  504, 1326,       0,  635678,       0,  0, 13,  0, }, // Nepal
    {  504, 1332,   62298,  635691,  635723, 23, 32, 33, }, // New_Caledonia
    {  504, 1346,   62385,  635756,  635786, 21, 30, 31, }, // New_Zealand
    {  504, 1358,  142525,  635817,  635843, 17, 26, 27, }, // Newfoundland
    {  504, 1371,       0,   62544,       0,  0, 12,  0, }, // Niue
    {  504, 1376,  635870,  635893,  635925, 23, 32, 33, }, // Norfolk
    {  504, 1384,   62646,  635958,  635994, 27, 36, 37, }, // Noronha
    {  504, 1406,   62782,  636031,  636059, 19, 28, 29, }, // Novosibirsk
    {  504, 1418,  623529,  636088,  636109, 12, 21, 22, }, // Omsk
    {  504, 1428,  636131,  636147,  636172, 16, 25, 26, }, // Pakistan
    {  504, 1437,       0,   62980,       0,  0, 13,  0, }, // Palau
    {  504, 1443,       0,  636198,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  504, 1460,  636224,  636240,  636265, 16, 25, 26, }, // Paraguay
    {  504, 1469,  636291,  636303,  636324, 12, 21, 22, }, // Peru
    {  504, 1474,  636346,  636363,  636389, 17, 26, 27, }, // Philippines
    {  504, 1486,       0,  636416,       0,  0, 21,  0, }, // Phoenix_Islands
    {  504, 1502,  636437,  636465,  636502, 28, 37, 38, }, // Pierre_Miquelon
    {  504, 1518,       0,   63351,       0,  0, 16,  0, }, // Pitcairn
    {  504, 1527,       0,  636540,       0,  0, 15,  0, }, // Ponape
    {  504, 1534,       0,  636555,       0,  0, 16,  0, }, // Pyongyang
    {  504, 1554,       0,  636571,       0,  0, 15,  0, }, // Reunion
    {  504, 1562,       0,   63482,       0,  0, 15,  0, }, // Rothera
    {  504, 1570,  636586,  636602,  636627, 16, 25, 26, }, // Sakhalin
    {  504, 1596,   63620,  636653,  636675, 13, 22, 23, }, // Samoa
    {  504, 1602,       0,  636698,       0,  0, 18,  0, }, // Seychelles
    {  504, 1624,       0,  636716,       0,  0, 25,  0, }, // Singapore
    {  504, 1634,       0,  636741,       0,  0, 21,  0, }, // Solomon
    {  504, 1642,       0,  636762,       0,  0, 23,  0, }, // South_Georgia
    {  504, 1656,       0,  143553,       0,  0, 15,  0, }, // Suriname
    {  504, 1676,       0,   63786,       0,  0, 13,  0, }, // Syowa
    {  504, 1682,       0,   63799,       0,  0, 14,  0, }, // Tahiti
    {  504, 1689,   63813,  636785,  636808, 14, 23, 24, }, // Taipei
    {  504, 1696,       0,  636832,       0,  0, 18,  0, }, // Tajikistan
    {  504, 1724,       0,   63893,       0,  0, 15,  0, }, // Tokelau
    {  504, 1732,   63908,  636850,  636872, 13, 22, 23, }, // Tonga
    {  504, 1738,       0,   63965,       0,  0, 13,  0, }, // Truk
    {  504, 1750,  636895,  636915,  636944, 20, 29, 30, }, // Turkmenistan
    {  504, 1763,       0,   64059,       0,  0, 14,  0, }, // Tuvalu
    {  504, 1777,  636974,  636989,  637013, 15, 24, 25, }, // Uruguay
    {  504, 1792,  637038,  637056,  637083, 18, 27, 28, }, // Uzbekistan
    {  504, 1803,   64217,  637111,  637135, 15, 24, 25, }, // Vanuatu
    {  504, 1811,       0,   64280,       0,  0, 17,  0, }, // Venezuela
    {  504, 1821,   64297,  637160,  637188, 19, 28, 29, }, // Vladivostok
    {  504, 1833,  624759,  637217,  637244, 18, 27, 28, }, // Volgograd
    {  504, 1843,       0,   64441,       0,  0, 14,  0, }, // Vostok
    {  504, 1850,       0,  637272,       0,  0, 17,  0, }, // Wake
    {  504, 1855,       0,   64476,       0,  0, 23,  0, }, // Wallis
    {  504, 1862,   64499,  637289,  637313, 15, 24, 25, }, // Yakutsk
    {  504, 1870,  637338,  637359,  637389, 21, 30, 31, }, // Yekaterinburg
    {  504, 1892,       0,  637420,       0,  0,  9,  0, }, // Yukon
    {  505,  422,       0,    1601,       0,  0, 12,  0, }, // Bolivia Quechua/Latin/Bolivia
    {  505, 1469,  636291,  636303,  636324, 12, 21, 22, }, // Peru
    {  506,  619,       0,  633649,       0,  0, 15,  0, }, // Ecuador Quechua/Latin/Ecuador
    {  506, 1469,  636291,  636303,  636324, 12, 21, 22, }, // Peru
    {  507,    0,  637429,  637437,  637454,  8, 17, 16, }, // Acre Romanian/Latin/Romania
    {  507,    5,       0,  637470,       0,  0, 18,  0, }, // Afghanistan
    {  507,   17,       0,  637488,       0,  0, 20,  0, }, // Africa_Central
    {  507,   32,       0,  637508,       0,  0, 21,  0, }, // Africa_Eastern
    {  507,   65,       0,  637529,       0,  0, 23,  0, }, // Africa_Southern
    {  507,   81,  637552,  637575,  637609, 23, 34, 33, }, // Africa_Western
    {  507,  107,  637642,  637656,  637679, 14, 23, 22, }, // Alaska
    {  507,  128,  637701,  637711,  637730, 10, 19, 18, }, // Almaty
    {  507,  135,  637748,  637762,  637787, 14, 25, 24, }, // Amazon
    {  507,  142,  637811,  637838,  637874, 27, 36, 35, }, // America_Central
    {  507,  158,  637909,  637937,  637974, 28, 37, 36, }, // America_Eastern
    {  507,  174,  638010,  638042,  638085, 32, 43, 42, }, // America_Mountain
    {  507,  191,  638127,  638159,  638202, 32, 43, 42, }, // America_Pacific
    {  507,  207,  638244,  638258,  638281, 14, 23, 22, }, // Anadyr
    {  507,  214,  638303,  638315,  638336, 12, 21, 20, }, // Apia
    {  507,  219,  638356,  638365,  638383,  9, 18, 25, }, // Aqtau
    {  507,  225,  638408,  638418,  638437, 10, 19, 26, }, // Aqtobe
    {  507,  232,  638463,  638472,  638490,  9, 18, 17, }, // Arabian
    {  507,  240,  638507,  638521,  638546, 14, 25, 24, }, // Argentina
    {  507,  250,  638570,  638596,  638633, 26, 37, 36, }, // Argentina_Western
    {  507,  268,  638669,  638681,  638704, 12, 23, 22, }, // Armenia
    {  507,  286,  638726,  638759,  638803, 33, 44, 43, }, // Atlantic
    {  507,  295,  638846,  638869,  638903, 23, 34, 33, }, // Australia_Central
    {  507,  313,  638936,  638970,  639015, 34, 45, 44, }, // Australia_CentralWestern
    {  507,  338,  639059,  639083,  639118, 24, 35, 34, }, // Australia_Eastern
    {  507,  356,  639152,  639178,  639215, 26, 37, 36, }, // Australia_Western
    {  507,  374,  639251,  639270,  639300, 19, 30, 29, }, // Azerbaijan
    {  507,  385,  639329,  639342,  639364, 13, 22, 21, }, // Azores
    {  507,  397,  639385,  639403,  639430, 18, 27, 26, }, // Bangladesh
    {  507,  415,       0,  639456,       0,  0, 14,  0, }, // Bhutan
    {  507,  422,       0,  639470,       0,  0, 12,  0, }, // Bolivia
    {  507,  437,  639482,  639495,  639519, 13, 24, 23, }, // Brasilia
    {  507,  454,       0,  639542,       0,  0, 25,  0, }, // Brunei
    {  507,  461,  639567,  639586,  639614, 19, 28, 27, }, // Cape_Verde
    {  507,  478,       0,  639641,       0,  0, 16,  0, }, // Chamorro
    {  507,  487,  639657,  639672,  639696, 15, 24, 23, }, // Chatham
    {  507,  495,  639719,  639732,  639754, 13, 22, 21, }, // Chile
    {  507,  501,  639775,  639785,  639806, 10, 21, 20, }, // China
    {  507,  507,       0,  639826,       0,  0, 24,  0, }, // Christmas
    {  507,  517,       0,  639850,       0,  0, 19,  0, }, // Cocos
    {  507,  523,  639869,  639882,  639906, 13, 24, 23, }, // Colombia
    {  507,  532,  639929,  639947,  639976, 18, 29, 28, }, // Cook
    {  507,  537,  640004,  640013,  640033,  9, 20, 19, }, // Cuba
    {  507,  548,       0,  640052,       0,  0, 13,  0, }, // Davis
    {  507,  564,       0,  640065,       0,  0, 24,  0, }, // DumontDUrville
    {  507,  601,       0,  640089,       0,  0, 20,  0, }, // East_Timor
    {  507,  612,  640109,  640132,  640164, 23, 32, 31, }, // Easter
    {  507,  619,       0,  640195,       0,  0, 15,  0, }, // Ecuador
    {  507,  627,  640210,  640230,  640261, 20, 31, 30, }, // Europe_Central
    {  507,  642,  640291,  640309,  640338, 18, 29, 28, }, // Europe_Eastern
    {  507,  657,       0,  640366,       0,  0, 30,  0, }, // Europe_Further_Eastern
    {  507,  680,  640396,  640415,  640445, 19, 30, 29, }, // Europe_Western
    {  507,  695,  640474,  640499,  640533, 25, 34, 33, }, // Falkland
    {  507,  704,  640566,  640578,  640599, 12, 21, 20, }, // Fiji
    {  507,  709,       0,  640619,       0,  0, 23,  0, }, // French_Guiana
    {  507,  723,       0,  640642,       0,  0, 51,  0, }, // French_Southern
    {  507,  746,       0,  640693,       0,  0, 17,  0, }, // Galapagos
    {  507,  756,       0,  640710,       0,  0, 15,  0, }, // Gambier
    {  507,  764,  640725,  640737,  640760, 12, 23, 22, }, // Georgia
    {  507,  772,       0,  640782,       0,  0, 21,  0, }, // Gilbert_Islands
    {  507,  788,       0,  640803,       0,  0, 17,  0, }, // GMT
    {  507,  830,  640820,  640845,  640881, 25, 36, 35, }, // Greenland_Eastern
    {  507,  848,  640916,  640943,  640981, 27, 38, 37, }, // Greenland_Western
    {  507,  871,       0,  641018,       0,  0, 23,  0, }, // Gulf
    {  507,  876,       0,  640619,       0,  0, 14,  0, }, // Guyana
    {  507,  883,  641041,  641064,  641096, 23, 32, 31, }, // Hawaii_Aleutian
    {  507,  899,  641127,  641144,  641170, 17, 26, 25, }, // Hong_Kong
    {  507,  909,  641195,  641207,  641228, 12, 21, 20, }, // Hovd
    {  507,  914,       0,  641248,       0,  0, 10,  0, }, // India
    {  507,  920,       0,  641258,       0,  0, 20,  0, }, // Indian_Ocean
    {  507,  933,       0,  641278,       0,  0, 14,  0, }, // Indochina
    {  507,  943,       0,  641292,       0,  0, 23,  0, }, // Indonesia_Central
    {  507,  961,       0,  641315,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  507,  979,       0,  641336,       0,  0, 22,  0, }, // Indonesia_Western
    {  507,  997,  641358,  641370,  641393, 12, 23, 22, }, // Iran
    {  507, 1008,  641415,  641429,  641452, 14, 23, 22, }, // Irkutsk
    {  507, 1016,  641474,  641488,  641513, 14, 25, 24, }, // Israel
    {  507, 1023,  641537,  641549,  641572, 12, 23, 22, }, // Japan
    {  507, 1029,  641594,  641625,  641665, 31, 40, 39, }, // Kamchatka
    {  507, 1047,       0,  641704,       0,  0, 17,  0, }, // Kazakhstan
    {  507, 1058,       0,  641721,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  507, 1077,       0,  641747,       0,  0, 27,  0, }, // Kazakhstan_Western
    {  507, 1106,  641774,  641784,  641805, 10, 21, 20, }, // Korea
    {  507, 1112,       0,  641825,       0,  0, 14,  0, }, // Kosrae
    {  507, 1119,  641839,  641858,  641886, 19, 28, 27, }, // Krasnoyarsk
    {  507, 1151,       0,  641913,       0,  0, 18,  0, }, // Kyrgystan
    {  507, 1175,       0,  641931,       0,  0, 21,  0, }, // Line_Islands
    {  507, 1188,  641952,  641969,  641995, 17, 26, 25, }, // Lord_Howe
    {  507, 1204,  642020,  642035,  642059, 15, 24, 23, }, // Magadan
    {  507, 1219,       0,  642082,       0,  0, 16,  0, }, // Malaysia
    {  507, 1228,       0,  642098,       0,  0, 15,  0, }, // Maldives
    {  507, 1237,       0,  642113,       0,  0, 22,  0, }, // Marquesas
    {  507, 1247,       0,  642135,       0,  0, 22,  0, }, // Marshall_Islands
    {  507, 1264,  642157,  642174,  642200, 17, 26, 25, }, // Mauritius
    {  507, 1274,       0,  642225,       0,  0, 14,  0, }, // Mawson
    {  507, 1281,  642239,  642265,  642302, 26, 37, 36, }, // Mexico_Pacific
    {  507, 1296,  642338,  642356,  642383, 18, 27, 26, }, // Mongolia
    {  507, 1305,  642409,  642421,  642444, 12, 23, 22, }, // Moscow
    {  507, 1312,       0,  642466,       0,  0, 15,  0, }, // Myanmar
    {  507, 1320,       0,  642481,       0,  0, 13,  0, }, // Nauru
    {  507, 1326,       0,  642494,       0,  0, 13,  0, }, // Nepal
    {  507, 1332,  642507,  642525,  642554, 18, 29, 28, }, // New_Caledonia
    {  507, 1346,  642582,  642599,  642627, 17, 28, 27, }, // New_Zealand
    {  507, 1358,  642654,  642674,  642703, 20, 29, 28, }, // Newfoundland
    {  507, 1371,       0,  642731,       0,  0, 12,  0, }, // Niue
    {  507, 1376,  642743,  642762,  642792, 19, 30, 29, }, // Norfolk
    {  507, 1384,  642821,  642848,  642884, 27, 36, 35, }, // Noronha
    {  507, 1406,  642919,  642938,  642966, 19, 28, 27, }, // Novosibirsk
    {  507, 1418,  642993,  643005,  643026, 12, 21, 20, }, // Omsk
    {  507, 1428,  643046,  643062,  643089, 16, 27, 26, }, // Pakistan
    {  507, 1437,       0,  643115,       0,  0, 13,  0, }, // Palau
    {  507, 1443,       0,  643128,       0,  0, 25,  0, }, // Papua_New_Guinea
    {  507, 1460,  643153,  643169,  643194, 16, 25, 24, }, // Paraguay
    {  507, 1469,  643218,  643230,  643251, 12, 21, 20, }, // Peru
    {  507, 1474,  643271,  643287,  643312, 16, 25, 24, }, // Philippines
    {  507, 1486,       0,  643336,       0,  0, 21,  0, }, // Phoenix_Islands
    {  507, 1502,  643357,  643389,  643430, 32, 41, 40, }, // Pierre_Miquelon
    {  507, 1518,       0,  643470,       0,  0, 16,  0, }, // Pitcairn
    {  507, 1527,       0,  643486,       0,  0, 14,  0, }, // Ponape
    {  507, 1534,       0,  643500,       0,  0, 17,  0, }, // Pyongyang
    {  507, 1554,       0,  643517,       0,  0, 15,  0, }, // Reunion
    {  507, 1562,       0,  643532,       0,  0, 15,  0, }, // Rothera
    {  507, 1570,  643547,  643562,  643586, 15, 24, 23, }, // Sakhalin
    {  507, 1579,  643609,  643623,  643646, 14, 23, 22, }, // Samara
    {  507, 1596,  643668,  643681,  643703, 13, 22, 21, }, // Samoa
    {  507, 1602,       0,  643724,       0,  0, 18,  0, }, // Seychelles
    {  507, 1624,       0,  643742,       0,  0, 17,  0, }, // Singapore
    {  507, 1634,       0,  643759,       0,  0, 21,  0, }, // Solomon
    {  507, 1642,       0,  643780,       0,  0, 19,  0, }, // South_Georgia
    {  507, 1656,       0,  643799,       0,  0, 15,  0, }, // Suriname
    {  507, 1676,       0,  643814,       0,  0, 13,  0, }, // Syowa
    {  507, 1682,       0,  643827,       0,  0, 14,  0, }, // Tahiti
    {  507, 1689,  643841,  643855,  643878, 14, 23, 22, }, // Taipei
    {  507, 1696,       0,  643900,       0,  0, 19,  0, }, // Tajikistan
    {  507, 1724,       0,  643919,       0,  0, 15,  0, }, // Tokelau
    {  507, 1732,  643934,  643947,  643969, 13, 22, 21, }, // Tonga
    {  507, 1738,       0,  643990,       0,  0, 13,  0, }, // Truk
    {  507, 1750,  644003,  644023,  644052, 20, 29, 28, }, // Turkmenistan
    {  507, 1763,       0,  644080,       0,  0, 14,  0, }, // Tuvalu
    {  507, 1777,  644094,  644109,  644135, 15, 26, 25, }, // Uruguay
    {  507, 1792,  644160,  644178,  644205, 18, 27, 26, }, // Uzbekistan
    {  507, 1803,  644231,  644246,  644270, 15, 24, 23, }, // Vanuatu
    {  507, 1811,       0,  644293,       0,  0, 14,  0, }, // Venezuela
    {  507, 1821,  644307,  644326,  644354, 19, 28, 27, }, // Vladivostok
    {  507, 1833,  644381,  644398,  644424, 17, 26, 25, }, // Volgograd
    {  507, 1843,       0,  644449,       0,  0, 14,  0, }, // Vostok
    {  507, 1850,       0,  644463,       0,  0, 16,  0, }, // Wake
    {  507, 1855,       0,  644479,       0,  0, 24,  0, }, // Wallis
    {  507, 1862,  644503,  644517,  644540, 14, 23, 22, }, // Yakutsk
    {  507, 1870,  644562,  644582,  644611, 20, 29, 28, }, // Yekaterinburg
    {  507, 1892,       0,  644639,       0,  0, 13,  0, }, // Yukon
    {  509,  142,  644652,  644664,  644688, 12, 24, 20, }, // America_Central Romansh/Latin/Switzerland
    {  509,  158,  644708,  644721,  644746, 13, 25, 21, }, // America_Eastern
    {  509,  174,  644767,  644783,  644811, 16, 28, 24, }, // America_Mountain
    {  509,  191,  644835,  644847,  644871, 12, 24, 20, }, // America_Pacific
    {  509,  286,  644891,  644904,  644929, 13, 25, 21, }, // Atlantic
    {  509,  627,  644950,  644975,  645012, 25, 37, 33, }, // Europe_Central
    {  509,  642,  645045,  645071,  645109, 26, 38, 34, }, // Europe_Eastern
    {  509,  680,  645143,  645168,  645205, 25, 37, 33, }, // Europe_Western
    {  509,  788,       0,  645238,       0,  0, 14,  0, }, // GMT
    {  512,    0,  645252,  645262,  645284, 10, 22, 17, }, // Acre Russian/Cyrillic/Russia
    {  512,    5,       0,    6372,       0,  0, 10,  0, }, // Afghanistan
    {  512,   17,       0,  645301,       0,  0, 18,  0, }, // Africa_Central
    {  512,   32,       0,  645319,       0,  0, 16,  0, }, // Africa_Eastern
    {  512,   65,       0,  645335,       0,  0, 12,  0, }, // Africa_Southern
    {  512,   81,  645347,  645362,  645396, 15, 34, 29, }, // Africa_Western
    {  512,  107,  119445,  645425,  645450,  6, 25, 20, }, // Alaska
    {  512,  128,  645470,  645484,  645510, 14, 26, 21, }, // Almaty
    {  512,  135,    6643,  645531,  645558,  8, 27, 22, }, // Amazon
    {  512,  142,  645580,  645599,  645637, 19, 38, 33, }, // America_Central
    {  512,  158,  645670,  645687,  645723, 17, 36, 31, }, // America_Eastern
    {  512,  174,  645754,  645785,  645828, 31, 43, 38, }, // America_Mountain
    {  512,  191,  645866,  645885,  645916, 19, 31, 26, }, // America_Pacific
    {  512,  207,  645942,  645958,  645983, 16, 25, 20, }, // Anadyr
    {  512,  214,    7133,  646003,  646026,  4, 23, 18, }, // Apia
    {  512,  219,  646044,  646055,  646079, 11, 24, 18, }, // Aqtau
    {  512,  225,  646097,  646109,  646133, 12, 24, 19, }, // Aqtobe
    {  512,  232,  646152,  646169,  646205, 17, 36, 31, }, // Arabian
    {  512,  240,    7371,  646236,  646264,  9, 28, 23, }, // Argentina
    {  512,  250,  646287,  646305,  646342, 18, 37, 32, }, // Argentina_Western
    {  512,  268,  418183,  646374,  646400,  7, 26, 21, }, // Armenia
    {  512,  286,  646421,  646440,  646471, 19, 31, 26, }, // Atlantic
    {  512,  295,  646497,  646518,  646558, 21, 40, 35, }, // Australia_Central
    {  512,  313,  646593,  646630,  646679, 37, 49, 44, }, // Australia_CentralWestern
    {  512,  338,  646723,  646742,  646780, 19, 38, 33, }, // Australia_Eastern
    {  512,  356,  646813,  646831,  646868, 18, 37, 32, }, // Australia_Western
    {  512,  374,   86281,  646900,  646930, 11, 30, 25, }, // Azerbaijan
    {  512,  385,  646955,  646968,  647000, 13, 32, 27, }, // Azores
    {  512,  397,    8256,  647027,  647055,  9, 28, 23, }, // Bangladesh
    {  512,  415,       0,    8317,       0,  0,  5,  0, }, // Bhutan
    {  512,  422,       0,  418891,       0,  0,  7,  0, }, // Bolivia
    {  512,  437,  418905,  647078,  647105,  8, 27, 22, }, // Brasilia
    {  512,  454,       0,  167201,       0,  0, 17,  0, }, // Brunei
    {  512,  461,    8404,  647127,  647156, 10, 29, 24, }, // Cape_Verde
    {  512,  472,       0,  180492,       0,  0,  5,  0, }, // Casey
    {  512,  478,       0,    8473,       0,  0,  7,  0, }, // Chamorro
    {  512,  487,    8480,  647180,  647204,  5, 24, 19, }, // Chatham
    {  512,  495,    8529,  647223,  647246,  4, 23, 18, }, // Chile
    {  512,  501,  121626,  647264,  647288,  5, 24, 19, }, // China
    {  512,  507,       0,  647307,       0,  0, 13,  0, }, // Christmas
    {  512,  517,       0,  647320,       0,  0, 14,  0, }, // Cocos
    {  512,  523,  419289,  647334,  647361,  8, 27, 22, }, // Colombia
    {  512,  532,  647383,  647395,  647426, 12, 31, 30, }, // Cook
    {  512,  537,    8806,  647456,  647479,  4, 23, 18, }, // Cuba
    {  512,  548,       0,  121938,       0,  0,  6,  0, }, // Davis
    {  512,  564,       0,  167598,       0,  0, 14,  0, }, // DumontDUrville
    {  512,  601,       0,  647497,       0,  0, 15,  0, }, // East_Timor
    {  512,  612,  647512,  647521,  647549,  9, 28, 23, }, // Easter
    {  512,  619,       0,   87146,       0,  0,  7,  0, }, // Ecuador
    {  512,  627,  647572,  647590,  647627, 18, 37, 32, }, // Europe_Central
    {  512,  642,  647659,  647675,  647710, 16, 35, 30, }, // Europe_Eastern
    {  512,  657,       0,  647740,       0,  0, 16,  0, }, // Europe_Further_Eastern
    {  512,  680,  647756,  647771,  647805, 15, 34, 29, }, // Europe_Western
    {  512,  695,  647834,  647851,  647887, 17, 36, 31, }, // Falkland
    {  512,  704,  122493,  647918,  647942,  5, 24, 19, }, // Fiji
    {  512,  709,       0,  647961,       0,  0, 18,  0, }, // French_Guiana
    {  512,  723,       0,  647979,       0,  0, 45,  0, }, // French_Southern
    {  512,  746,       0,  648024,       0,  0, 18,  0, }, // Galapagos
    {  512,  756,       0,  168162,       0,  0,  6,  0, }, // Gambier
    {  512,  764,  420125,  648042,  648067,  6, 25, 20, }, // Georgia
    {  512,  772,       0,  648087,       0,  0, 13,  0, }, // Gilbert_Islands
    {  512,  788,       0,  648100,       0,  0, 25,  0, }, // GMT
    {  512,  830,  648125,  648145,  648183, 20, 38, 34, }, // Greenland_Eastern
    {  512,  848,  648217,  648236,  648274, 19, 38, 33, }, // Greenland_Western
    {  512,  866,       0,    9794,       0,  0,  4,  0, }, // Guam
    {  512,  871,       0,  648307,       0,  0, 16,  0, }, // Gulf
    {  512,  876,       0,  168454,       0,  0,  6,  0, }, // Guyana
    {  512,  883,  648323,  648347,  648383, 24, 36, 31, }, // Hawaii_Aleutian
    {  512,  899,  168537,  648414,  648440,  7, 26, 21, }, // Hong_Kong
    {  512,  909,    9958,  648461,  648484,  4, 23, 18, }, // Hovd
    {  512,  914,       0,  219721,       0,  0,  5,  0, }, // India
    {  512,  920,       0,  648502,       0,  0, 15,  0, }, // Indian_Ocean
    {  512,  933,       0,  123211,       0,  0,  9,  0, }, // Indochina
    {  512,  943,       0,  648517,       0,  0, 21,  0, }, // Indonesia_Central
    {  512,  961,       0,  648538,       0,  0, 19,  0, }, // Indonesia_Eastern
    {  512,  979,       0,  648557,       0,  0, 18,  0, }, // Indonesia_Western
    {  512,  997,   10097,  648575,  648598,  4, 23, 18, }, // Iran
    {  512, 1008,   10143,  648616,  648642,  7, 26, 21, }, // Irkutsk
    {  512, 1016,   10198,  648663,  648689,  7, 26, 21, }, // Israel
    {  512, 1023,  219788,  648710,  648735,  6, 25, 20, }, // Japan
    {  512, 1029,  648755,  648779,  648822, 24, 43, 38, }, // Kamchatka
    {  512, 1047,       0,   88489,       0,  0,  9,  0, }, // Kazakhstan
    {  512, 1058,       0,  648860,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  512, 1077,       0,  648879,       0,  0, 18,  0, }, // Kazakhstan_Western
    {  512, 1106,  421075,  648897,  648921,  5, 24, 19, }, // Korea
    {  512, 1112,       0,   10514,       0,  0,  6,  0, }, // Kosrae
    {  512, 1119,  123820,  648940,  648969, 10, 29, 24, }, // Krasnoyarsk
    {  512, 1151,       0,  648993,       0,  0,  8,  0, }, // Kyrgystan
    {  512, 1161,       0,   10595,       0,  0,  9,  0, }, // Lanka
    {  512, 1175,       0,  649001,       0,  0,  9,  0, }, // Line_Islands
    {  512, 1188,   88712,  649010,  649037,  8, 27, 22, }, // Lord_Howe
    {  512, 1198,   10675,  649059,  649083,  5, 24, 19, }, // Macau
    {  512, 1204,   10724,  649102,  649128,  7, 26, 21, }, // Magadan
    {  512, 1219,       0,  421367,       0,  0,  8,  0, }, // Malaysia
    {  512, 1228,       0,  649149,       0,  0,  8,  0, }, // Maldives
    {  512, 1237,       0,  649157,       0,  0, 15,  0, }, // Marquesas
    {  512, 1247,       0,  649172,       0,  0, 18,  0, }, // Marshall_Islands
    {  512, 1264,  183138,  649190,  649217,  8, 27, 22, }, // Mauritius
    {  512, 1274,       0,   10893,       0,  0,  6,  0, }, // Mawson
    {  512, 1281,  649239,  649271,  649315, 32, 44, 39, }, // Mexico_Pacific
    {  512, 1296,   11009,  649354,  649383, 10, 29, 24, }, // Mongolia
    {  512, 1305,   11073,  649407,  649432,  6, 25, 20, }, // Moscow
    {  512, 1312,       0,  169396,       0,  0,  6,  0, }, // Myanmar
    {  512, 1320,       0,   11131,       0,  0,  5,  0, }, // Nauru
    {  512, 1326,       0,   11136,       0,  0,  5,  0, }, // Nepal
    {  512, 1332,  649452,  649467,  649501, 15, 34, 29, }, // New_Caledonia
    {  512, 1346,  649530,  649544,  649577, 14, 33, 28, }, // New_Zealand
    {  512, 1358,  169543,  649605,  649636, 12, 31, 26, }, // Newfoundland
    {  512, 1371,       0,  169609,       0,  0,  4,  0, }, // Niue
    {  512, 1376,   11358,  649662,  649688,  7, 26, 21, }, // Norfolk
    {  512, 1384,  169657,  649709,  649747, 19, 38, 33, }, // Noronha
    {  512, 1392,       0,  649780,       0,  0, 24,  0, }, // North_Mariana
    {  512, 1406,   11531,  649804,  649834, 11, 30, 25, }, // Novosibirsk
    {  512, 1418,   11598,  649859,  649882,  4, 23, 18, }, // Omsk
    {  512, 1428,   11644,  649900,  649927,  8, 27, 22, }, // Pakistan
    {  512, 1437,       0,   11702,       0,  0,  5,  0, }, // Palau
    {  512, 1443,       0,  649949,       0,  0, 20,  0, }, // Papua_New_Guinea
    {  512, 1460,  125138,  649969,  649996,  8, 27, 22, }, // Paraguay
    {  512, 1469,   11784,  650018,  650041,  4, 23, 18, }, // Peru
    {  512, 1474,  650059,  650068,  650096,  9, 28, 23, }, // Philippines
    {  512, 1486,       0,  650119,       0,  0, 11,  0, }, // Phoenix_Islands
    {  512, 1502,  650130,  650148,  650185, 18, 37, 32, }, // Pierre_Miquelon
    {  512, 1518,       0,  170151,       0,  0,  7,  0, }, // Pitcairn
    {  512, 1527,       0,   11998,       0,  0,  6,  0, }, // Ponape
    {  512, 1534,       0,   90127,       0,  0,  7,  0, }, // Pyongyang
    {  512, 1544,   12011,  650217,  650246, 10, 29, 24, }, // Qyzylorda
    {  512, 1554,       0,  170172,       0,  0,  7,  0, }, // Reunion
    {  512, 1562,       0,   12082,       0,  0,  6,  0, }, // Rothera
    {  512, 1570,   12088,  650270,  650296,  7, 26, 21, }, // Sakhalin
    {  512, 1579,  650317,  650331,  650358, 14, 27, 22, }, // Samara
    {  512, 1596,   12205,  650380,  650404,  5, 24, 19, }, // Samoa
    {  512, 1602,       0,  650423,       0,  0, 19,  0, }, // Seychelles
    {  512, 1624,       0,   12279,       0,  0,  8,  0, }, // Singapore
    {  512, 1634,       0,  650442,       0,  0, 18,  0, }, // Solomon
    {  512, 1642,       0,  650460,       0,  0, 13,  0, }, // South_Georgia
    {  512, 1656,       0,   12329,       0,  0,  7,  0, }, // Suriname
    {  512, 1676,       0,   90382,       0,  0,  4,  0, }, // Syowa
    {  512, 1682,       0,   12341,       0,  0,  5,  0, }, // Tahiti
    {  512, 1689,  170324,  650473,  650499,  7, 26, 21, }, // Taipei
    {  512, 1696,       0,  125818,       0,  0, 11,  0, }, // Tajikistan
    {  512, 1724,       0,   12408,       0,  0,  7,  0, }, // Tokelau
    {  512, 1732,   12415,  650520,  650544,  5, 24, 19, }, // Tonga
    {  512, 1738,       0,   12464,       0,  0,  4,  0, }, // Truk
    {  512, 1750,   12468,  650563,  650594, 12, 31, 26, }, // Turkmenistan
    {  512, 1763,       0,   12538,       0,  0,  6,  0, }, // Tuvalu
    {  512, 1777,   90600,  650620,  650646,  7, 26, 21, }, // Uruguay
    {  512, 1792,  113243,  650667,  650696, 10, 29, 24, }, // Uzbekistan
    {  512, 1803,   12666,  650720,  650746,  7, 26, 21, }, // Vanuatu
    {  512, 1811,       0,  170605,       0,  0,  9,  0, }, // Venezuela
    {  512, 1821,   12730,  650767,  650797, 11, 30, 25, }, // Vladivostok
    {  512, 1833,   12797,  650822,  650850,  9, 28, 23, }, // Volgograd
    {  512, 1843,       0,  113571,       0,  0,  6,  0, }, // Vostok
    {  512, 1850,       0,   90954,       0,  0,  4,  0, }, // Wake
    {  512, 1855,       0,  650873,       0,  0, 15,  0, }, // Wallis
    {  512, 1862,  126390,  650888,  650913,  6, 25, 20, }, // Yakutsk
    {  512, 1870,   12942,  650933,  650964, 12, 31, 26, }, // Yekaterinburg
    {  512, 1892,       0,  126544,       0,  0,  4,  0, }, // Yukon
    {  520,  232,  650990,  651000,  651016, 10, 16, 18, }, // Arabian Sakha/Cyrillic/Russia
    {  520,  268,  651034,  651045,  651062, 11, 17, 19, }, // Armenia
    {  520,  295,  651081,  651100,  651125, 19, 25, 27, }, // Australia_Central
    {  520,  338,  651152,  651171,  651196, 19, 25, 27, }, // Australia_Eastern
    {  520,  356,  651223,  651243,  651269, 20, 26, 28, }, // Australia_Western
    {  520,  501,  651297,  651307,  651323, 10, 16, 18, }, // China
    {  520,  764,  651341,  651354,  651373, 13, 19, 21, }, // Georgia
    {  520,  788,       0,  202954,       0,  0,  3,  0, }, // GMT
    {  520,  914,       0,  651394,       0,  0, 18,  0, }, // India
    {  520,  997,  651412,  651422,  651438, 10, 16, 18, }, // Iran
    {  520, 1023,  651456,  651469,  651488, 13, 19, 21, }, // Japan
    {  520, 1058,       0,  651509,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  520, 1077,       0,  651529,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  520, 1106,  651550,  651561,  651578, 11, 17, 19, }, // Korea
    {  520, 1119,  651597,  651614,  651637, 17, 23, 25, }, // Krasnoyarsk
    {  520, 1151,       0,  651662,       0,  0, 15,  0, }, // Kyrgystan
    {  520, 1204,  651677,  651690,  651709, 13, 19, 21, }, // Magadan
    {  520, 1296,  651730,  651746,  651768, 16, 22, 24, }, // Mongolia
    {  520, 1305,  651792,  651804,  651822, 12, 18, 20, }, // Moscow
    {  520, 1346,  651842,  651860,  651883, 18, 23, 25, }, // New_Zealand
    {  520, 1406,  651908,  651926,  651950, 18, 24, 26, }, // Novosibirsk
    {  520, 1418,  651976,  651987,  652004, 11, 17, 19, }, // Omsk
    {  520, 1428,  652023,  652037,  652057, 14, 20, 22, }, // Pakistan
    {  520, 1570,  652079,  652092,  652111, 13, 19, 21, }, // Sakhalin
    {  520, 1821,  652132,  652148,  652173, 16, 25, 27, }, // Vladivostok
    {  520, 1833,  652200,  652214,  652234, 14, 20, 22, }, // Volgograd
    {  520, 1862,  652256,  652271,  652292, 15, 21, 23, }, // Yakutsk
    {  520, 1870,  652315,  652332,  652355, 17, 23, 25, }, // Yekaterinburg
    {  524,  142,  652380,  652404,  652434, 24, 30, 29, }, // America_Central Sanskrit/Devanagari/India
    {  524,  158,  652463,  652488,  652519, 25, 31, 30, }, // America_Eastern
    {  524,  174,  652549,  652572,  652601, 23, 29, 28, }, // America_Mountain
    {  524,  191,  652629,  652659,  652695, 30, 36, 35, }, // America_Pacific
    {  524,  286,  652730,  652744,  652764, 14, 20, 19, }, // Atlantic
    {  524,  627,  652783,  652800,  652823, 17, 23, 25, }, // Europe_Central
    {  524,  642,  652848,  652866,  652890, 18, 24, 26, }, // Europe_Eastern
    {  524,  680,  652916,  652938,  652966, 22, 28, 30, }, // Europe_Western
    {  524,  788,       0,  652996,       0,  0, 17,  0, }, // GMT
    {  525,  107,  653013,  653025,  653047, 12, 22, 18, }, // Alaska Santali/Ol Chiki/India
    {  525,  135,  653065,  653076,  653096, 11, 20, 17, }, // Amazon
    {  525,  142,  653113,  653124,  653145, 11, 21, 17, }, // America_Central
    {  525,  158,  653162,  653173,  653194, 11, 21, 18, }, // America_Eastern
    {  525,  174,  653212,  653224,  653246, 12, 22, 18, }, // America_Mountain
    {  525,  191,  653264,  653277,  653300, 13, 23, 19, }, // America_Pacific
    {  525,  240,  653319,  653334,  653358, 15, 24, 21, }, // Argentina
    {  525,  250,  653379,  653399,  653429, 20, 30, 26, }, // Argentina_Western
    {  525,  286,  653455,  653468,  653491, 13, 23, 19, }, // Atlantic
    {  525,  537,  653510,  653520,  653536, 10, 16, 16, }, // Cuba
    {  525,  627,  653552,  653573,  653604, 21, 31, 27, }, // Europe_Central
    {  525,  642,  653631,  653651,  653681, 20, 30, 26, }, // Europe_Eastern
    {  525,  680,  653707,  653728,  653759, 21, 31, 27, }, // Europe_Western
    {  525,  788,       0,  653786,       0,  0, 16,  0, }, // GMT
    {  525,  830,  653802,  653822,  653848, 20, 26, 26, }, // Greenland_Eastern
    {  525,  848,  653874,  653893,  653918, 19, 25, 25, }, // Greenland_Western
    {  525,  883,  653943,  653964,  653991, 21, 27, 27, }, // Hawaii_Aleutian
    {  525, 1281,  654018,  654039,  654066, 21, 27, 27, }, // Mexico_Pacific
    {  525, 1358,  654093,  654112,  654137, 19, 25, 25, }, // Newfoundland
    {  525, 1502,  654162,  654190,  654224, 28, 34, 34, }, // Pierre_Miquelon
    {  525, 1892,       0,  654258,       0,  0, 10,  0, }, // Yukon
    {  527,    0,  654268,  654279,  654300, 11, 21, 18, }, // Acre Sardinian/Latin/Italy
    {  527,    5,       0,  654318,       0,  0, 20,  0, }, // Afghanistan
    {  527,   17,       0,  654338,       0,  0, 25,  0, }, // Africa_Central
    {  527,   32,       0,  654363,       0,  0, 25,  0, }, // Africa_Eastern
    {  527,   65,       0,  654388,       0,  0, 37,  0, }, // Africa_Southern
    {  527,   81,  654425,  654452,  654489, 27, 37, 34, }, // Africa_Western
    {  527,  107,  654523,  654538,  654563, 15, 25, 22, }, // Alaska
    {  527,  128,  654585,  654598,  654621, 13, 23, 20, }, // Almaty
    {  527,  135,  654641,  654659,  654687, 18, 28, 25, }, // Amazon
    {  527,  142,  654712,  654729,  654756, 17, 27, 24, }, // America_Central
    {  527,  158,  380652,  654780,  380695, 17, 27, 24, }, // America_Eastern
    {  527,  174,  654807,  654830,  654863, 23, 33, 30, }, // America_Mountain
    {  527,  191,  654893,  654916,  654949, 23, 33, 30, }, // America_Pacific
    {  527,  207,  654979,  654992,  655015, 13, 23, 20, }, // Anadyr
    {  527,  214,  655035,  655046,  655067, 11, 21, 18, }, // Apia
    {  527,  219,  655085,  655097,  655119, 12, 22, 19, }, // Aqtau
    {  527,  225,  655138,  655151,  655174, 13, 23, 20, }, // Aqtobe
    {  527,  232,  655194,  655203,  655222,  9, 19, 16, }, // Arabian
    {  527,  240,  655238,  655256,  655284, 18, 28, 25, }, // Argentina
    {  527,  250,  655309,  655339,  655379, 30, 40, 37, }, // Argentina_Western
    {  527,  268,  655416,  655432,  655458, 16, 26, 23, }, // Armenia
    {  527,  286,  655481,  655499,  655527, 18, 28, 25, }, // Atlantic
    {  527,  295,  655552,  655580,  655618, 28, 38, 35, }, // Australia_Central
    {  527,  313,  655653,  655691,  655739, 38, 48, 45, }, // Australia_CentralWestern
    {  527,  338,  655784,  655812,  655850, 28, 38, 35, }, // Australia_Eastern
    {  527,  356,  655885,  655915,  655955, 30, 40, 37, }, // Australia_Western
    {  527,  374,  655992,  656012,  656042, 20, 30, 27, }, // Azerbaijan
    {  527,  385,  656069,  656087,  656115, 18, 28, 25, }, // Azores
    {  527,  397,  656140,  656160,  656190, 20, 30, 27, }, // Bangladesh
    {  527,  415,       0,  656217,       0,  0, 16,  0, }, // Bhutan
    {  527,  422,       0,  656233,       0,  0, 17,  0, }, // Bolivia
    {  527,  437,  656250,  656265,  656290, 15, 25, 22, }, // Brasilia
    {  527,  454,       0,  656312,       0,  0, 16,  0, }, // Brunei
    {  527,  461,  656328,  656348,  656378, 20, 30, 27, }, // Cape_Verde
    {  527,  472,       0,  656405,       0,  0, 12,  0, }, // Casey
    {  527,  478,       0,  656417,       0,  0, 25,  0, }, // Chamorro
    {  527,  487,  656442,  656460,  656488, 18, 28, 25, }, // Chatham
    {  527,  495,  656513,  656528,  656553, 15, 25, 22, }, // Chile
    {  527,  501,  656575,  656590,  656615, 15, 25, 22, }, // China
    {  527,  507,       0,  656637,       0,  0, 31,  0, }, // Christmas
    {  527,  517,       0,  656668,       0,  0, 23,  0, }, // Cocos
    {  527,  523,  656691,  656709,  656737, 18, 28, 25, }, // Colombia
    {  527,  532,  656762,  656784,  656816, 22, 32, 29, }, // Cook
    {  527,  537,  656845,  656856,  656877, 11, 21, 18, }, // Cuba
    {  527,  548,       0,  656895,       0,  0, 12,  0, }, // Davis
    {  527,  564,       0,  656907,       0,  0, 23,  0, }, // DumontDUrville
    {  527,  601,       0,  656930,       0,  0, 19,  0, }, // East_Timor
    {  527,  612,  656949,  656972,  657005, 23, 33, 30, }, // Easter
    {  527,  619,       0,  657035,       0,  0, 16,  0, }, // Ecuador
    {  527,  627,  657051,  657076,  657111, 25, 35, 32, }, // Europe_Central
    {  527,  642,  657143,  657168,  657203, 25, 35, 32, }, // Europe_Eastern
    {  527,  657,       0,  657235,       0,  0, 46,  0, }, // Europe_Further_Eastern
    {  527,  680,  657281,  657308,  657345, 27, 37, 34, }, // Europe_Western
    {  527,  695,  657379,  657405,  657441, 26, 36, 33, }, // Falkland
    {  527,  704,  657474,  657489,  657514, 15, 25, 22, }, // Fiji
    {  527,  709,       0,  657536,       0,  0, 26,  0, }, // French_Guiana
    {  527,  723,       0,  657562,       0,  0, 50,  0, }, // French_Southern
    {  527,  746,       0,  657612,       0,  0, 20,  0, }, // Galapagos
    {  527,  756,       0,  657632,       0,  0, 14,  0, }, // Gambier
    {  527,  764,  657646,  657663,  657690, 17, 27, 24, }, // Georgia
    {  527,  772,       0,  657714,       0,  0, 25,  0, }, // Gilbert_Islands
    {  527,  788,       0,  657739,       0,  0, 32,  0, }, // GMT
    {  527,  830,  657771,  657802,  657843, 31, 41, 38, }, // Greenland_Eastern
    {  527,  848,  657881,  657914,  657957, 33, 43, 40, }, // Greenland_Western
    {  527,  866,       0,  657997,       0,  0, 21,  0, }, // Guam
    {  527,  871,       0,  658018,       0,  0, 25,  0, }, // Gulf
    {  527,  876,       0,  658043,       0,  0, 16,  0, }, // Guyana
    {  527,  883,  658059,  658093,  658137, 34, 44, 41, }, // Hawaii_Aleutian
    {  527,  899,  658178,  658194,  658220, 16, 26, 23, }, // Hong_Kong
    {  527,  909,  658243,  658254,  658275, 11, 21, 18, }, // Hovd
    {  527,  914,       0,  658293,       0,  0, 24,  0, }, // India
    {  527,  920,       0,  658317,       0,  0, 24,  0, }, // Indian_Ocean
    {  527,  933,       0,  658341,       0,  0, 18,  0, }, // Indochina
    {  527,  943,       0,  658359,       0,  0, 28,  0, }, // Indonesia_Central
    {  527,  961,       0,  658387,       0,  0, 28,  0, }, // Indonesia_Eastern
    {  527,  979,       0,  658415,       0,  0, 30,  0, }, // Indonesia_Western
    {  527,  997,  658445,  658458,  658481, 13, 23, 20, }, // Iran
    {  527, 1008,  658501,  658515,  658539, 14, 24, 21, }, // Irkutsk
    {  527, 1016,  658560,  658574,  658598, 14, 24, 21, }, // Israel
    {  527, 1023,  658619,  658636,  658663, 17, 27, 24, }, // Japan
    {  527, 1029,  658687,  658718,  658759, 31, 41, 38, }, // Kamchatka
    {  527, 1047,       0,  658797,       0,  0, 20,  0, }, // Kazakhstan
    {  527, 1058,       0,  658817,       0,  0, 30,  0, }, // Kazakhstan_Eastern
    {  527, 1077,       0,  658847,       0,  0, 32,  0, }, // Kazakhstan_Western
    {  527, 1106,  384485,  658879,  384516, 11, 21, 18, }, // Korea
    {  527, 1112,       0,  658900,       0,  0, 13,  0, }, // Kosrae
    {  527, 1119,  658913,  658931,  658959, 18, 28, 25, }, // Krasnoyarsk
    {  527, 1151,       0,  658984,       0,  0, 22,  0, }, // Kyrgystan
    {  527, 1161,       0,  659006,       0,  0, 12,  0, }, // Lanka
    {  527, 1175,       0,  659018,       0,  0, 29,  0, }, // Line_Islands
    {  527, 1188,  659047,  659063,  659089, 16, 26, 23, }, // Lord_Howe
    {  527, 1198,  659112,  659124,  659146, 12, 22, 19, }, // Macau
    {  527, 1204,  659165,  659179,  659203, 14, 24, 21, }, // Magadan
    {  527, 1219,       0,  659224,       0,  0, 17,  0, }, // Malaysia
    {  527, 1228,       0,  659241,       0,  0, 19,  0, }, // Maldives
    {  527, 1237,       0,  659260,       0,  0, 20,  0, }, // Marquesas
    {  527, 1247,       0,  659280,       0,  0, 26,  0, }, // Marshall_Islands
    {  527, 1264,  659306,  659327,  659358, 21, 31, 28, }, // Mauritius
    {  527, 1274,       0,  659386,       0,  0, 13,  0, }, // Mawson
    {  527, 1281,  659399,  659428,  659467, 29, 39, 36, }, // Mexico_Pacific
    {  527, 1296,  659503,  659520,  659547, 17, 27, 24, }, // Mongolia
    {  527, 1305,  659571,  659583,  659605, 12, 22, 19, }, // Moscow
    {  527, 1312,       0,  659624,       0,  0, 17,  0, }, // Myanmar
    {  527, 1320,       0,  659641,       0,  0, 12,  0, }, // Nauru
    {  527, 1326,       0,  659653,       0,  0, 15,  0, }, // Nepal
    {  527, 1332,  659668,  659691,  659724, 23, 33, 30, }, // New_Caledonia
    {  527, 1346,  659754,  659775,  659806, 21, 31, 28, }, // New_Zealand
    {  527, 1358,  659834,  659850,  659876, 16, 26, 23, }, // Newfoundland
    {  527, 1371,       0,  659899,       0,  0, 11,  0, }, // Niue
    {  527, 1376,  659910,  659932,  659964, 22, 32, 29, }, // Norfolk
    {  527, 1384,  659993,  660022,  660061, 29, 39, 36, }, // Noronha
    {  527, 1392,       0,  660097,       0,  0, 42,  0, }, // North_Mariana
    {  527, 1406,  660139,  660157,  660185, 18, 28, 25, }, // Novosibirsk
    {  527, 1418,  660210,  660221,  660242, 11, 21, 18, }, // Omsk
    {  527, 1428,  660260,  660278,  660306, 18, 28, 25, }, // Pakistan
    {  527, 1437,       0,  660331,       0,  0, 12,  0, }, // Palau
    {  527, 1443,       0,  660343,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  527, 1460,  660369,  660387,  660415, 18, 28, 25, }, // Paraguay
    {  527, 1469,  660440,  660454,  660478, 14, 24, 21, }, // Peru
    {  527, 1474,  660499,  660519,  660549, 20, 30, 27, }, // Philippines
    {  527, 1486,       0,  660576,       0,  0, 32,  0, }, // Phoenix_Islands
    {  527, 1502,  660608,  660638,  660678, 30, 40, 37, }, // Pierre_Miquelon
    {  527, 1518,       0,  660715,       0,  0, 19,  0, }, // Pitcairn
    {  527, 1527,       0,  660734,       0,  0, 14,  0, }, // Ponape
    {  527, 1534,       0,  660748,       0,  0, 16,  0, }, // Pyongyang
    {  527, 1544,  660764,  660780,  660806, 16, 26, 23, }, // Qyzylorda
    {  527, 1554,       0,  660829,       0,  0, 18,  0, }, // Reunion
    {  527, 1562,       0,  660847,       0,  0, 14,  0, }, // Rothera
    {  527, 1570,  660861,  660876,  660901, 15, 25, 22, }, // Sakhalin
    {  527, 1579,  660923,  660936,  660959, 13, 23, 20, }, // Samara
    {  527, 1596,  660979,  660995,  661021, 16, 26, 23, }, // Samoa
    {  527, 1602,       0,  661044,       0,  0, 21,  0, }, // Seychelles
    {  527, 1624,       0,  661065,       0,  0, 16,  0, }, // Singapore
    {  527, 1634,       0,  661081,       0,  0, 26,  0, }, // Solomon
    {  527, 1642,       0,  661107,       0,  0, 27,  0, }, // South_Georgia
    {  527, 1656,       0,  661134,       0,  0, 18,  0, }, // Suriname
    {  527, 1676,       0,  661152,       0,  0, 12,  0, }, // Syowa
    {  527, 1682,       0,  661164,       0,  0, 13,  0, }, // Tahiti
    {  527, 1689,  661177,  661190,  661213, 13, 23, 20, }, // Taipei
    {  527, 1696,       0,  661233,       0,  0, 20,  0, }, // Tajikistan
    {  527, 1724,       0,  661253,       0,  0, 17,  0, }, // Tokelau
    {  527, 1732,  661270,  661285,  661310, 15, 25, 22, }, // Tonga
    {  527, 1738,       0,  661332,       0,  0, 15,  0, }, // Truk
    {  527, 1750,  661347,  661369,  661401, 22, 32, 29, }, // Turkmenistan
    {  527, 1763,       0,  661430,       0,  0, 16,  0, }, // Tuvalu
    {  527, 1777,  661446,  661462,  661488, 16, 26, 23, }, // Uruguay
    {  527, 1792,  661511,  661530,  661559, 19, 29, 26, }, // Uzbekistan
    {  527, 1803,  661585,  661602,  661629, 17, 27, 24, }, // Vanuatu
    {  527, 1811,       0,  661653,       0,  0, 20,  0, }, // Venezuela
    {  527, 1821,  661673,  661691,  661719, 18, 28, 25, }, // Vladivostok
    {  527, 1833,  661744,  661760,  661786, 16, 26, 23, }, // Volgograd
    {  527, 1843,       0,  661809,       0,  0, 13,  0, }, // Vostok
    {  527, 1850,       0,  661822,       0,  0, 22,  0, }, // Wake
    {  527, 1855,       0,  661844,       0,  0, 22,  0, }, // Wallis
    {  527, 1862,  661866,  661880,  661904, 14, 24, 21, }, // Yakutsk
    {  527, 1870,  661925,  661945,  661975, 20, 30, 27, }, // Yekaterinburg
    {  527, 1892,       0,  662002,       0,  0, 15,  0, }, // Yukon
    {  529,    0,  105715,  105725,  105746, 10, 21, 27, }, // Acre Serbian/Cyrillic/Serbia
    {  529,    5,       0,  662017,       0,  0, 16,  0, }, // Afghanistan
    {  529,   17,       0,  662033,       0,  0, 23,  0, }, // Africa_Central
    {  529,   32,       0,  662056,       0,  0, 21,  0, }, // Africa_Eastern
    {  529,   65,       0,  662077,       0,  0, 19,  0, }, // Africa_Southern
    {  529,   81,  662096,  662117,  662149, 21, 32, 27, }, // Africa_Western
    {  529,  107,  105955,  662176,  662200,  6, 24, 19, }, // Alaska
    {  529,  128,  106014,  106026,  106049, 12, 23, 29, }, // Almaty
    {  529,  135,  662219,  662231,  662255, 12, 24, 19, }, // Amazon
    {  529,  142,  662274,  662305,  662347, 31, 42, 37, }, // America_Central
    {  529,  158,  662384,  662413,  662453, 29, 40, 35, }, // America_Eastern
    {  529,  174,  662488,  662519,  662561, 31, 42, 37, }, // America_Mountain
    {  529,  191,  662598,  662629,  662671, 31, 42, 37, }, // America_Pacific
    {  529,  207,  106448,  106460,  106483, 12, 23, 29, }, // Anadyr
    {  529,  214,  662708,  662719,  662742, 11, 23, 18, }, // Apia
    {  529,  219,  106577,  106590,  106614, 13, 24, 30, }, // Aqtau
    {  529,  225,  106644,  106657,  106681, 13, 24, 30, }, // Aqtobe
    {  529,  232,  662760,  662775,  662801, 15, 26, 21, }, // Arabian
    {  529,  240,  662822,  662837,  662864, 15, 27, 22, }, // Argentina
    {  529,  250,  662886,  662909,  662944, 23, 35, 30, }, // Argentina_Western
    {  529,  268,  662974,  662989,  663016, 15, 27, 22, }, // Armenia
    {  529,  286,  495581,  495596,  663038, 15, 26, 21, }, // Atlantic
    {  529,  295,  663059,  663087,  663126, 28, 39, 34, }, // Australia_Central
    {  529,  313,  663160,  663196,  663243, 36, 47, 42, }, // Australia_CentralWestern
    {  529,  338,  663285,  663311,  663348, 26, 37, 32, }, // Australia_Eastern
    {  529,  356,  663380,  663406,  663443, 26, 37, 32, }, // Australia_Western
    {  529,  374,  663475,  663491,  663519, 16, 28, 23, }, // Azerbaijan
    {  529,  385,  663542,  663553,  663576, 11, 23, 18, }, // Azores
    {  529,  397,  663594,  663609,  663636, 15, 27, 22, }, // Bangladesh
    {  529,  415,       0,  663658,       0,  0, 11,  0, }, // Bhutan
    {  529,  422,       0,  663669,       0,  0, 14,  0, }, // Bolivia
    {  529,  437,  663683,  663698,  663725, 15, 27, 22, }, // Brasilia
    {  529,  454,       0,  663747,       0,  0, 22,  0, }, // Brunei
    {  529,  461,  663769,  663793,  663829, 24, 36, 31, }, // Cape_Verde
    {  529,  478,       0,  663860,       0,  0, 12,  0, }, // Chamorro
    {  529,  487,  663872,  663883,  663906, 11, 23, 18, }, // Chatham
    {  529,  495,  663924,  663934,  663956, 10, 22, 17, }, // Chile
    {  529,  501,  663973,  663983,  664007, 10, 24, 17, }, // China
    {  529,  507,       0,  664024,       0,  0, 20,  0, }, // Christmas
    {  529,  517,       0,  664044,       0,  0, 27,  0, }, // Cocos
    {  529,  523,  664071,  664086,  664113, 15, 27, 22, }, // Colombia
    {  529,  532,  664135,  664154,  664185, 19, 31, 31, }, // Cook
    {  529,  537,    8806,  664216,  664238,  4, 22, 17, }, // Cuba
    {  529,  548,       0,  664255,       0,  0, 12,  0, }, // Davis
    {  529,  564,       0,  664267,       0,  0, 19,  0, }, // DumontDUrville
    {  529,  601,       0,  664286,       0,  0, 19,  0, }, // East_Timor
    {  529,  612,  664305,  664325,  664357, 20, 32, 27, }, // Easter
    {  529,  619,       0,  664384,       0,  0, 13,  0, }, // Ecuador
    {  529,  627,  664397,  664417,  664448, 20, 31, 26, }, // Europe_Central
    {  529,  642,  497364,  497385,  664474, 21, 32, 27, }, // Europe_Eastern
    {  529,  657,       0,  664501,       0,  0, 25,  0, }, // Europe_Further_Eastern
    {  529,  680,  497464,  497485,  664526, 21, 32, 27, }, // Europe_Western
    {  529,  695,  664553,  664577,  664613, 24, 36, 31, }, // Falkland
    {  529,  704,  664644,  664654,  664676, 10, 22, 17, }, // Fiji
    {  529,  709,       0,  664693,       0,  0, 23,  0, }, // French_Guiana
    {  529,  723,       0,  497740,       0,  0, 35,  0, }, // French_Southern
    {  529,  746,       0,  664716,       0,  0, 15,  0, }, // Galapagos
    {  529,  756,       0,  664731,       0,  0, 13,  0, }, // Gambier
    {  529,  764,  664744,  664757,  664782, 13, 25, 20, }, // Georgia
    {  529,  772,       0,  664802,       0,  0, 20,  0, }, // Gilbert_Islands
    {  529,  788,       0,  664822,       0,  0, 23,  0, }, // GMT
    {  529,  830,  109360,  664845,  664879, 16, 34, 29, }, // Greenland_Eastern
    {  529,  848,  109458,  664908,  664942, 16, 34, 29, }, // Greenland_Western
    {  529,  866,       0,  109556,       0,  0, 21,  0, }, // Guam
    {  529,  871,       0,  664971,       0,  0, 14,  0, }, // Gulf
    {  529,  876,       0,  664703,       0,  0, 13,  0, }, // Guyana
    {  529,  883,  664985,  665008,  665042, 23, 34, 29, }, // Hawaii_Aleutian
    {  529,  899,  665071,  665086,  665113, 15, 27, 22, }, // Hong_Kong
    {  529,  909,  665135,  665145,  665167, 10, 22, 17, }, // Hovd
    {  529,  914,       0,  665184,       0,  0, 25,  0, }, // India
    {  529,  920,       0,  665209,       0,  0, 23,  0, }, // Indian_Ocean
    {  529,  933,       0,  665232,       0,  0, 14,  0, }, // Indochina
    {  529,  943,       0,  665246,       0,  0, 28,  0, }, // Indonesia_Central
    {  529,  961,       0,  665274,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  529,  979,       0,  665300,       0,  0, 26,  0, }, // Indonesia_Western
    {  529,  997,  665326,  665336,  665358, 10, 22, 17, }, // Iran
    {  529, 1008,  665375,  665387,  665411, 12, 24, 19, }, // Irkutsk
    {  529, 1016,  123435,  665430,  665456, 15, 26, 21, }, // Israel
    {  529, 1023,  665477,  665491,  665516, 14, 25, 20, }, // Japan
    {  529, 1029,  110269,  110299,  665536, 30, 41, 47, }, // Kamchatka
    {  529, 1047,       0,  123680,       0,  0, 18,  0, }, // Kazakhstan
    {  529, 1058,       0,  665583,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  529, 1077,       0,  665609,       0,  0, 26,  0, }, // Kazakhstan_Western
    {  529, 1106,  665635,  665649,  665674, 14, 25, 20, }, // Korea
    {  529, 1112,       0,  665694,       0,  0, 11,  0, }, // Kosrae
    {  529, 1119,  665705,  665722,  665751, 17, 29, 24, }, // Krasnoyarsk
    {  529, 1151,       0,  665775,       0,  0, 15,  0, }, // Kyrgystan
    {  529, 1161,       0,  110630,       0,  0, 15,  0, }, // Lanka
    {  529, 1175,       0,  665790,       0,  0, 17,  0, }, // Line_Islands
    {  529, 1188,  665807,  665821,  665847, 14, 26, 21, }, // Lord_Howe
    {  529, 1198,  110738,  110749,  665868, 11, 22, 28, }, // Macau
    {  529, 1204,  665896,  665909,  665934, 13, 25, 20, }, // Magadan
    {  529, 1219,       0,  665954,       0,  0, 14,  0, }, // Malaysia
    {  529, 1228,       0,  665968,       0,  0, 13,  0, }, // Maldives
    {  529, 1237,       0,  665981,       0,  0, 12,  0, }, // Marquesas
    {  529, 1247,       0,  665993,       0,  0, 22,  0, }, // Marshall_Islands
    {  529, 1264,  666015,  666031,  666059, 16, 28, 23, }, // Mauritius
    {  529, 1274,       0,  666082,       0,  0, 11,  0, }, // Mawson
    {  529, 1281,  666093,  666109,  666143, 16, 34, 29, }, // Mexico_Pacific
    {  529, 1296,  666172,  666188,  666216, 16, 28, 23, }, // Mongolia
    {  529, 1305,  666239,  666251,  666275, 12, 24, 19, }, // Moscow
    {  529, 1312,       0,  666294,       0,  0, 14,  0, }, // Myanmar
    {  529, 1320,       0,  666308,       0,  0, 11,  0, }, // Nauru
    {  529, 1326,       0,  666319,       0,  0, 11,  0, }, // Nepal
    {  529, 1332,  666330,  666351,  666384, 21, 33, 28, }, // New_Caledonia
    {  529, 1346,  666412,  666429,  666458, 17, 29, 24, }, // New_Zealand
    {  529, 1358,  111502,  666482,  666511, 11, 29, 24, }, // Newfoundland
    {  529, 1371,       0,  666535,       0,  0, 10,  0, }, // Niue
    {  529, 1376,  666545,  666565,  666597, 20, 32, 27, }, // Norfolk
    {  529, 1384,  666624,  666648,  666684, 24, 36, 31, }, // Noronha
    {  529, 1392,       0,  111793,       0,  0, 31,  0, }, // North_Mariana
    {  529, 1406,  666715,  666732,  666761, 17, 29, 24, }, // Novosibirsk
    {  529, 1418,  666785,  666795,  666817, 10, 22, 17, }, // Omsk
    {  529, 1428,  666834,  666848,  666874, 14, 26, 21, }, // Pakistan
    {  529, 1437,       0,  666895,       0,  0, 11,  0, }, // Palau
    {  529, 1443,       0,  666906,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  529, 1460,  666930,  666944,  666970, 14, 26, 21, }, // Paraguay
    {  529, 1469,  666991,  667001,  667023, 10, 22, 17, }, // Peru
    {  529, 1474,  667040,  667054,  667080, 14, 26, 21, }, // Philippines
    {  529, 1486,       0,  667101,       0,  0, 19,  0, }, // Phoenix_Islands
    {  529, 1502,  112314,  667120,  667156, 18, 36, 31, }, // Pierre_Miquelon
    {  529, 1518,       0,  667187,       0,  0, 13,  0, }, // Pitcairn
    {  529, 1527,       0,  667200,       0,  0, 12,  0, }, // Ponape
    {  529, 1534,       0,  667212,       0,  0, 17,  0, }, // Pyongyang
    {  529, 1544,  112463,  112478,  112504, 15, 26, 32, }, // Qyzylorda
    {  529, 1554,       0,  667229,       0,  0, 13,  0, }, // Reunion
    {  529, 1562,       0,  667242,       0,  0, 12,  0, }, // Rothera
    {  529, 1570,  667254,  667267,  667292, 13, 25, 20, }, // Sakhalin
    {  529, 1579,  112636,  112648,  112671, 12, 23, 29, }, // Samara
    {  529, 1596,  667312,  667323,  667346, 11, 23, 18, }, // Samoa
    {  529, 1602,       0,  667364,       0,  0, 13,  0, }, // Seychelles
    {  529, 1624,       0,  667377,       0,  0, 26,  0, }, // Singapore
    {  529, 1634,       0,  667403,       0,  0, 23,  0, }, // Solomon
    {  529, 1642,       0,  667426,       0,  0, 19,  0, }, // South_Georgia
    {  529, 1656,       0,  667445,       0,  0, 13,  0, }, // Suriname
    {  529, 1676,       0,  667458,       0,  0, 10,  0, }, // Syowa
    {  529, 1682,       0,  667468,       0,  0, 12,  0, }, // Tahiti
    {  529, 1689,  667480,  667492,  667516, 12, 24, 19, }, // Taipei
    {  529, 1696,       0,  667535,       0,  0, 16,  0, }, // Tajikistan
    {  529, 1724,       0,  667551,       0,  0, 13,  0, }, // Tokelau
    {  529, 1732,  667564,  667575,  667598, 11, 23, 18, }, // Tonga
    {  529, 1738,       0,  667616,       0,  0, 10,  0, }, // Truk
    {  529, 1750,  667626,  667644,  667674, 18, 30, 25, }, // Turkmenistan
    {  529, 1763,       0,  667699,       0,  0, 12,  0, }, // Tuvalu
    {  529, 1777,  667711,  667724,  667749, 13, 25, 20, }, // Uruguay
    {  529, 1792,  667769,  667785,  667813, 16, 28, 23, }, // Uzbekistan
    {  529, 1803,  667836,  667849,  667874, 13, 25, 20, }, // Vanuatu
    {  529, 1811,       0,  667894,       0,  0, 15,  0, }, // Venezuela
    {  529, 1821,  667909,  667926,  667955, 17, 29, 24, }, // Vladivostok
    {  529, 1833,  667979,  667994,  668021, 15, 27, 22, }, // Volgograd
    {  529, 1843,       0,  668043,       0,  0, 12,  0, }, // Vostok
    {  529, 1850,       0,  668055,       0,  0, 17,  0, }, // Wake
    {  529, 1855,       0,  668072,       0,  0, 27,  0, }, // Wallis
    {  529, 1862,  668099,  668112,  668137, 13, 25, 20, }, // Yakutsk
    {  529, 1870,  668157,  668176,  668207, 19, 31, 26, }, // Yekaterinburg
    {  529, 1892,       0,  502084,       0,  0,  5,  0, }, // Yukon
    {  530,    5,       0,  105773,       0,  0, 18,  0, }, // Afghanistan Serbian/Cyrillic/Bosnia and Herzegovina
    {  530,   17,       0,  105791,       0,  0, 25,  0, }, // Africa_Central
    {  530,   32,       0,  105816,       0,  0, 23,  0, }, // Africa_Eastern
    {  530,   65,       0,  105839,       0,  0, 21,  0, }, // Africa_Southern
    {  530,   81,  105860,  105883,  668233, 23, 34, 29, }, // Africa_Western
    {  530,  107,  105955,  668262,  668288,  6, 26, 21, }, // Alaska
    {  530,  135,  106078,  668309,  668335, 14, 26, 21, }, // Amazon
    {  530,  142,  668356,  668390,  668435, 34, 45, 40, }, // America_Central
    {  530,  158,  668475,  668507,  668550, 32, 43, 38, }, // America_Eastern
    {  530,  174,  668588,  668622,  668667, 34, 45, 40, }, // America_Mountain
    {  530,  191,  668707,  668741,  668786, 34, 45, 40, }, // America_Pacific
    {  530,  214,  106512,  668826,  668851, 13, 25, 20, }, // Apia
    {  530,  232,  106711,  106728,  668871, 17, 28, 23, }, // Arabian
    {  530,  240,  106788,  668894,  668923, 17, 29, 24, }, // Argentina
    {  530,  250,  106865,  668947,  668984, 25, 37, 32, }, // Argentina_Western
    {  530,  268,  669016,  669033,  669062, 17, 29, 24, }, // Armenia
    {  530,  286,  669086,  669103,  669131, 17, 28, 23, }, // Atlantic
    {  530,  295,  107114,  107144,  669154, 30, 41, 36, }, // Australia_Central
    {  530,  313,  107230,  107268,  669190, 38, 49, 44, }, // Australia_CentralWestern
    {  530,  338,  107370,  107398,  669234, 28, 39, 34, }, // Australia_Eastern
    {  530,  356,  107480,  107508,  669268, 28, 39, 34, }, // Australia_Western
    {  530,  374,  107590,  669302,  669332, 18, 30, 25, }, // Azerbaijan
    {  530,  385,  107670,  669357,  669382, 13, 25, 20, }, // Azores
    {  530,  397,  107735,  669402,  669431, 17, 29, 24, }, // Bangladesh
    {  530,  415,       0,  107812,       0,  0, 13,  0, }, // Bhutan
    {  530,  422,       0,  107825,       0,  0, 16,  0, }, // Bolivia
    {  530,  437,  107841,  669455,  669484, 17, 29, 24, }, // Brasilia
    {  530,  454,       0,  669508,       0,  0, 24,  0, }, // Brunei
    {  530,  461,  669532,  669558,  669596, 26, 38, 33, }, // Cape_Verde
    {  530,  478,       0,  108025,       0,  0, 14,  0, }, // Chamorro
    {  530,  487,  108039,  669629,  669654, 13, 25, 20, }, // Chatham
    {  530,  495,  108104,  669674,  669698, 12, 24, 19, }, // Chile
    {  530,  501,  108166,  669717,  669743, 12, 26, 19, }, // China
    {  530,  507,       0,  669762,       0,  0, 22,  0, }, // Christmas
    {  530,  517,       0,  669784,       0,  0, 32,  0, }, // Cocos
    {  530,  523,  108279,  669816,  669845, 17, 29, 24, }, // Colombia
    {  530,  532,  669869,  669890,  669923, 21, 33, 32, }, // Cook
    {  530,  537,    8806,  669955,  669979,  4, 24, 19, }, // Cuba
    {  530,  548,       0,  108512,       0,  0, 14,  0, }, // Davis
    {  530,  564,       0,  669998,       0,  0, 21,  0, }, // DumontDUrville
    {  530,  601,       0,  670019,       0,  0, 21,  0, }, // East_Timor
    {  530,  612,  108566,  670040,  670074, 22, 34, 29, }, // Easter
    {  530,  619,       0,  108658,       0,  0, 15,  0, }, // Ecuador
    {  530,  627,  670103,  670125,  670158, 22, 33, 28, }, // Europe_Central
    {  530,  642,  108765,  108788,  670186, 23, 34, 29, }, // Europe_Eastern
    {  530,  680,  108888,  108911,  670215, 23, 34, 29, }, // Europe_Western
    {  530,  695,  108983,  670244,  670282, 26, 38, 33, }, // Falkland
    {  530,  704,  109087,  670315,  670339, 12, 24, 19, }, // Fiji
    {  530,  709,       0,  109149,       0,  0, 25,  0, }, // French_Guiana
    {  530,  723,       0,  109174,       0,  0, 37,  0, }, // French_Southern
    {  530,  746,       0,  109211,       0,  0, 17,  0, }, // Galapagos
    {  530,  756,       0,  670358,       0,  0, 15,  0, }, // Gambier
    {  530,  764,  109244,  670373,  670400, 15, 27, 22, }, // Georgia
    {  530,  772,       0,  670422,       0,  0, 25,  0, }, // Gilbert_Islands
    {  530,  788,       0,  670447,       0,  0, 25,  0, }, // GMT
    {  530,  830,  109360,  670472,  670508, 16, 36, 31, }, // Greenland_Eastern
    {  530,  848,  109458,  670539,  670575, 16, 36, 31, }, // Greenland_Western
    {  530,  871,       0,  109577,       0,  0, 16,  0, }, // Gulf
    {  530,  876,       0,  109159,       0,  0, 15,  0, }, // Guyana
    {  530,  883,  109593,  109618,  670606, 25, 36, 31, }, // Hawaii_Aleutian
    {  530,  899,  109694,  670637,  670666, 17, 29, 24, }, // Hong_Kong
    {  530,  909,  109772,  670690,  670714, 12, 24, 19, }, // Hovd
    {  530,  914,       0,  109834,       0,  0, 27,  0, }, // India
    {  530,  920,       0,  109861,       0,  0, 25,  0, }, // Indian_Ocean
    {  530,  933,       0,  109886,       0,  0, 16,  0, }, // Indochina
    {  530,  943,       0,  109902,       0,  0, 30,  0, }, // Indonesia_Central
    {  530,  961,       0,  109932,       0,  0, 28,  0, }, // Indonesia_Eastern
    {  530,  979,       0,  109960,       0,  0, 28,  0, }, // Indonesia_Western
    {  530,  997,  109988,  670733,  670757, 12, 24, 19, }, // Iran
    {  530, 1008,  110050,  670776,  670802, 14, 26, 21, }, // Irkutsk
    {  530, 1016,  110118,  110135,  670823, 17, 28, 23, }, // Israel
    {  530, 1023,  110195,  110211,  670846, 16, 27, 22, }, // Japan
    {  530, 1047,       0,  670868,       0,  0, 20,  0, }, // Kazakhstan
    {  530, 1058,       0,  110386,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  530, 1077,       0,  110414,       0,  0, 28,  0, }, // Kazakhstan_Western
    {  530, 1106,  110442,  110458,  670888, 16, 27, 22, }, // Korea
    {  530, 1112,       0,  110516,       0,  0, 13,  0, }, // Kosrae
    {  530, 1119,  670910,  670929,  670960, 19, 31, 26, }, // Krasnoyarsk
    {  530, 1151,       0,  110613,       0,  0, 17,  0, }, // Kyrgystan
    {  530, 1175,       0,  670986,       0,  0, 23,  0, }, // Line_Islands
    {  530, 1188,  110664,  671009,  671037, 16, 28, 23, }, // Lord_Howe
    {  530, 1204,  110798,  671060,  671087, 15, 27, 22, }, // Magadan
    {  530, 1219,       0,  110868,       0,  0, 16,  0, }, // Malaysia
    {  530, 1228,       0,  110884,       0,  0, 15,  0, }, // Maldives
    {  530, 1237,       0,  110899,       0,  0, 14,  0, }, // Marquesas
    {  530, 1247,       0,  110913,       0,  0, 24,  0, }, // Marshall_Islands
    {  530, 1264,  110937,  671109,  671139, 18, 30, 25, }, // Mauritius
    {  530, 1274,       0,  111017,       0,  0, 13,  0, }, // Mawson
    {  530, 1281,  666093,  671164,  671200, 16, 36, 31, }, // Mexico_Pacific
    {  530, 1296,  111134,  671231,  671261, 18, 30, 26, }, // Mongolia
    {  530, 1305,  111214,  671287,  671313, 14, 26, 21, }, // Moscow
    {  530, 1312,       0,  671334,       0,  0, 15,  0, }, // Myanmar
    {  530, 1320,       0,  111298,       0,  0, 13,  0, }, // Nauru
    {  530, 1326,       0,  111311,       0,  0, 13,  0, }, // Nepal
    {  530, 1332,  111324,  671349,  671384, 23, 35, 30, }, // New_Caledonia
    {  530, 1346,  111419,  671414,  671445, 19, 31, 26, }, // New_Zealand
    {  530, 1358,  111502,  671471,  671502, 11, 31, 26, }, // Newfoundland
    {  530, 1371,       0,  671528,       0,  0, 13,  0, }, // Niue
    {  530, 1376,  671541,  671563,  671597, 22, 34, 29, }, // Norfolk
    {  530, 1384,  111689,  671626,  671664, 26, 38, 33, }, // Noronha
    {  530, 1406,  671697,  671716,  671747, 19, 31, 26, }, // Novosibirsk
    {  530, 1418,  111908,  671773,  671797, 12, 24, 19, }, // Omsk
    {  530, 1428,  111970,  671816,  671844, 16, 28, 23, }, // Pakistan
    {  530, 1437,       0,  112044,       0,  0, 13,  0, }, // Palau
    {  530, 1443,       0,  112057,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  530, 1460,  112083,  671867,  671895, 16, 28, 23, }, // Paraguay
    {  530, 1469,  112157,  671918,  671942, 12, 24, 19, }, // Peru
    {  530, 1474,  112219,  671961,  671989, 16, 28, 23, }, // Philippines
    {  530, 1486,       0,  112293,       0,  0, 21,  0, }, // Phoenix_Islands
    {  530, 1502,  112314,  672012,  672050, 18, 38, 33, }, // Pierre_Miquelon
    {  530, 1518,       0,  112417,       0,  0, 15,  0, }, // Pitcairn
    {  530, 1527,       0,  672083,       0,  0, 14,  0, }, // Ponape
    {  530, 1534,       0,  672097,       0,  0, 19,  0, }, // Pyongyang
    {  530, 1554,       0,  672116,       0,  0, 15,  0, }, // Reunion
    {  530, 1562,       0,  112551,       0,  0, 14,  0, }, // Rothera
    {  530, 1570,  112565,  672131,  672158, 15, 27, 22, }, // Sakhalin
    {  530, 1596,  112700,  672180,  672205, 13, 25, 20, }, // Samoa
    {  530, 1602,       0,  112765,       0,  0, 15,  0, }, // Seychelles
    {  530, 1624,       0,  672225,       0,  0, 28,  0, }, // Singapore
    {  530, 1634,       0,  112807,       0,  0, 25,  0, }, // Solomon
    {  530, 1642,       0,  112832,       0,  0, 21,  0, }, // South_Georgia
    {  530, 1656,       0,  112853,       0,  0, 15,  0, }, // Suriname
    {  530, 1676,       0,  112868,       0,  0, 12,  0, }, // Syowa
    {  530, 1682,       0,  112880,       0,  0, 14,  0, }, // Tahiti
    {  530, 1689,  112894,  672253,  672279, 14, 26, 21, }, // Taipei
    {  530, 1696,       0,  112962,       0,  0, 18,  0, }, // Tajikistan
    {  530, 1724,       0,  112980,       0,  0, 15,  0, }, // Tokelau
    {  530, 1732,  112995,  672300,  672325, 13, 25, 20, }, // Tonga
    {  530, 1738,       0,  672345,       0,  0, 11,  0, }, // Truk
    {  530, 1750,  113072,  672356,  672388, 20, 32, 27, }, // Turkmenistan
    {  530, 1763,       0,  113158,       0,  0, 14,  0, }, // Tuvalu
    {  530, 1777,  113172,  672415,  672442, 15, 27, 22, }, // Uruguay
    {  530, 1792,  113243,  672464,  672494, 18, 30, 25, }, // Uzbekistan
    {  530, 1803,  113323,  672519,  672546, 15, 27, 22, }, // Vanuatu
    {  530, 1811,       0,  113394,       0,  0, 17,  0, }, // Venezuela
    {  530, 1821,  113411,  672568,  672599, 19, 31, 26, }, // Vladivostok
    {  530, 1833,  113494,  672625,  672654, 17, 29, 24, }, // Volgograd
    {  530, 1843,       0,  113571,       0,  0, 14,  0, }, // Vostok
    {  530, 1850,       0,  672678,       0,  0, 19,  0, }, // Wake
    {  530, 1855,       0,  672697,       0,  0, 29,  0, }, // Wallis
    {  530, 1862,  113633,  672726,  672753, 15, 27, 22, }, // Yakutsk
    {  530, 1870,  113704,  672775,  672808, 21, 33, 28, }, // Yekaterinburg
    {  533,    0,  672836,  672846,  672867, 10, 21, 29, }, // Acre Serbian/Latin/Bosnia and Herzegovina
    {  533,    5,       0,  672896,       0,  0, 18,  0, }, // Afghanistan
    {  533,   17,       0,  672914,       0,  0, 25,  0, }, // Africa_Central
    {  533,   32,       0,  672939,       0,  0, 23,  0, }, // Africa_Eastern
    {  533,   65,       0,  672962,       0,  0, 21,  0, }, // Africa_Southern
    {  533,   81,  672983,  673006,  673040, 23, 34, 31, }, // Africa_Western
    {  533,  107,   97393,  673071,  673098,  7, 27, 24, }, // Alaska
    {  533,  128,  673122,  673134,  673157, 12, 23, 31, }, // Almaty
    {  533,  135,  673188,  673202,  673228, 14, 26, 23, }, // Amazon
    {  533,  142,   97603,   97637,  673251, 34, 45, 42, }, // America_Central
    {  533,  158,   97723,   97755,  673293, 32, 43, 40, }, // America_Eastern
    {  533,  174,   97837,   97871,  673333, 34, 45, 42, }, // America_Mountain
    {  533,  191,   97957,   97991,  673375, 34, 45, 41, }, // America_Pacific
    {  533,  207,   98077,   98089,   98112, 12, 23, 31, }, // Anadyr
    {  533,  214,  673416,  673429,  673454, 13, 25, 22, }, // Apia
    {  533,  219,  673476,  673489,  673513, 13, 24, 32, }, // Aqtau
    {  533,  225,  673545,  673558,  673582, 13, 24, 32, }, // Aqtobe
    {  533,  232,   98341,   98358,  673614, 17, 28, 25, }, // Arabian
    {  533,  240,  673639,  673656,  673685, 17, 29, 26, }, // Argentina
    {  533,  250,  673711,  673736,  673773, 25, 37, 34, }, // Argentina_Western
    {  533,  268,  673807,  673824,  673853, 17, 29, 26, }, // Armenia
    {  533,  286,  673879,  673896,  673924, 17, 28, 25, }, // Atlantic
    {  533,  295,  673949,  673979,  674020, 30, 41, 38, }, // Australia_Central
    {  533,  313,   98872,  674058,  674107, 38, 49, 46, }, // Australia_CentralWestern
    {  533,  338,  674153,  674181,  674220, 28, 39, 36, }, // Australia_Eastern
    {  533,  356,  674256,  674284,  674323, 28, 39, 36, }, // Australia_Western
    {  533,  374,  674359,  674378,  674409, 19, 31, 28, }, // Azerbaijan
    {  533,  385,  674437,  674450,  674475, 13, 25, 22, }, // Azores
    {  533,  397,  674497,  674514,  674543, 17, 29, 26, }, // Bangladesh
    {  533,  415,       0,  674569,       0,  0, 13,  0, }, // Bhutan
    {  533,  422,       0,  674582,       0,  0, 16,  0, }, // Bolivia
    {  533,  437,  674598,  674615,  674644, 17, 29, 26, }, // Brasilia
    {  533,  454,       0,  674670,       0,  0, 24,  0, }, // Brunei
    {  533,  461,  674694,  674720,  674758, 26, 38, 35, }, // Cape_Verde
    {  533,  478,       0,  674793,       0,  0, 14,  0, }, // Chamorro
    {  533,  487,  674807,  674820,  674845, 13, 25, 22, }, // Chatham
    {  533,  495,  674867,  674879,  674903, 12, 24, 21, }, // Chile
    {  533,  501,  674924,   99800,  674936, 12, 26, 21, }, // China
    {  533,  507,       0,  674957,       0,  0, 22,  0, }, // Christmas
    {  533,  517,       0,  674979,       0,  0, 32,  0, }, // Cocos
    {  533,  523,  675011,  675028,  675057, 17, 29, 26, }, // Colombia
    {  533,  532,  675083,  675104,  675137, 21, 33, 34, }, // Cook
    {  533,  537,   14948,  675171,  675195,  4, 24, 21, }, // Cuba
    {  533,  548,       0,  675216,       0,  0, 14,  0, }, // Davis
    {  533,  564,       0,  675230,       0,  0, 21,  0, }, // DumontDUrville
    {  533,  601,       0,  675251,       0,  0, 21,  0, }, // East_Timor
    {  533,  612,  675272,  675295,  675330, 23, 35, 32, }, // Easter
    {  533,  619,       0,  675362,       0,  0, 15,  0, }, // Ecuador
    {  533,  627,  675377,  675400,  675434, 23, 34, 31, }, // Europe_Central
    {  533,  642,  100420,  100443,  675465, 23, 34, 31, }, // Europe_Eastern
    {  533,  657,       0,  675496,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {  533,  680,  100536,  100559,  675522, 23, 34, 31, }, // Europe_Western
    {  533,  695,  675553,  675579,  675617, 26, 38, 35, }, // Falkland
    {  533,  704,  675652,  675665,  675690, 13, 25, 22, }, // Fiji
    {  533,  709,       0,  675712,       0,  0, 25,  0, }, // French_Guiana
    {  533,  723,       0,  675737,       0,  0, 37,  0, }, // French_Southern
    {  533,  746,       0,  675774,       0,  0, 17,  0, }, // Galapagos
    {  533,  756,       0,  675791,       0,  0, 15,  0, }, // Gambier
    {  533,  764,  675806,  675821,  675848, 15, 27, 24, }, // Georgia
    {  533,  772,       0,  675872,       0,  0, 25,  0, }, // Gilbert_Islands
    {  533,  788,       0,  675897,       0,  0, 26,  0, }, // GMT
    {  533,  830,  675923,  675939,  675975, 16, 36, 33, }, // Greenland_Eastern
    {  533,  848,  676008,  676024,  676060, 16, 36, 33, }, // Greenland_Western
    {  533,  866,       0,  101193,       0,  0, 21,  0, }, // Guam
    {  533,  871,       0,  676093,       0,  0, 16,  0, }, // Gulf
    {  533,  876,       0,  675722,       0,  0, 15,  0, }, // Guyana
    {  533,  883,  676109,  676134,  676170, 25, 36, 33, }, // Hawaii_Aleutian
    {  533,  899,  676203,  676220,  676249, 17, 29, 26, }, // Hong_Kong
    {  533,  909,  676275,  676287,  676311, 12, 24, 21, }, // Hovd
    {  533,  914,       0,  101492,       0,  0, 27,  0, }, // India
    {  533,  920,       0,  676332,       0,  0, 25,  0, }, // Indian_Ocean
    {  533,  933,       0,  676357,       0,  0, 16,  0, }, // Indochina
    {  533,  943,       0,  676373,       0,  0, 30,  0, }, // Indonesia_Central
    {  533,  961,       0,  676403,       0,  0, 28,  0, }, // Indonesia_Eastern
    {  533,  979,       0,  676431,       0,  0, 28,  0, }, // Indonesia_Western
    {  533,  997,  676459,  676471,  676495, 12, 24, 21, }, // Iran
    {  533, 1008,  676516,  676530,  676556, 14, 26, 23, }, // Irkutsk
    {  533, 1016,  101777,  101794,  676579, 17, 28, 25, }, // Israel
    {  533, 1023,  101846,  101862,  676604, 16, 27, 24, }, // Japan
    {  533, 1029,  676628,  676658,  676699, 30, 41, 49, }, // Kamchatka
    {  533, 1047,       0,  676748,       0,  0, 20,  0, }, // Kazakhstan
    {  533, 1058,       0,  676768,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  533, 1077,       0,  676796,       0,  0, 28,  0, }, // Kazakhstan_Western
    {  533, 1106,  102103,  102119,  676824, 16, 27, 24, }, // Korea
    {  533, 1112,       0,  676848,       0,  0, 13,  0, }, // Kosrae
    {  533, 1119,  676861,  676880,  676911, 19, 31, 28, }, // Krasnoyarsk
    {  533, 1151,       0,  676939,       0,  0, 17,  0, }, // Kyrgystan
    {  533, 1161,       0,  676956,       0,  0, 15,  0, }, // Lanka
    {  533, 1175,       0,  676971,       0,  0, 23,  0, }, // Line_Islands
    {  533, 1188,  676994,  677010,  677038, 16, 28, 25, }, // Lord_Howe
    {  533, 1198,  102422,  102433,  102455, 11, 22, 30, }, // Macau
    {  533, 1204,  677063,  677078,  677105, 15, 27, 24, }, // Magadan
    {  533, 1219,       0,  677129,       0,  0, 16,  0, }, // Malaysia
    {  533, 1228,       0,  677145,       0,  0, 15,  0, }, // Maldives
    {  533, 1237,       0,  677160,       0,  0, 14,  0, }, // Marquesas
    {  533, 1247,       0,  677174,       0,  0, 24,  0, }, // Marshall_Islands
    {  533, 1264,  677198,  677216,  677246, 18, 30, 27, }, // Mauritius
    {  533, 1274,       0,  677273,       0,  0, 13,  0, }, // Mawson
    {  533, 1281,  677286,  677302,  677338, 16, 36, 33, }, // Mexico_Pacific
    {  533, 1296,  677371,  677389,  677419, 18, 30, 28, }, // Mongolia
    {  533, 1305,  677447,  677461,  677487, 14, 26, 23, }, // Moscow
    {  533, 1312,       0,  677510,       0,  0, 15,  0, }, // Myanmar
    {  533, 1320,       0,  677525,       0,  0, 13,  0, }, // Nauru
    {  533, 1326,       0,  677538,       0,  0, 13,  0, }, // Nepal
    {  533, 1332,  677551,  677574,  677609, 23, 35, 32, }, // New_Caledonia
    {  533, 1346,  677641,  677660,  677691, 19, 31, 28, }, // New_Zealand
    {  533, 1358,   29457,  677719,  677751, 12, 32, 29, }, // Newfoundland
    {  533, 1371,       0,  677780,       0,  0, 13,  0, }, // Niue
    {  533, 1376,  677793,  677815,  677849, 22, 34, 31, }, // Norfolk
    {  533, 1384,  677880,  677907,  677946, 27, 39, 36, }, // Noronha
    {  533, 1392,       0,  103524,       0,  0, 31,  0, }, // North_Mariana
    {  533, 1406,  677982,  678001,  678032, 19, 31, 28, }, // Novosibirsk
    {  533, 1418,  678060,  678072,  678096, 12, 24, 21, }, // Omsk
    {  533, 1428,  678117,  678133,  678161, 16, 28, 25, }, // Pakistan
    {  533, 1437,       0,  678186,       0,  0, 13,  0, }, // Palau
    {  533, 1443,       0,  678199,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  533, 1460,  678225,  678241,  678269, 16, 28, 25, }, // Paraguay
    {  533, 1469,  678294,  678306,  678330, 12, 24, 21, }, // Peru
    {  533, 1474,  678351,  678367,  678395, 16, 28, 25, }, // Philippines
    {  533, 1486,       0,  678420,       0,  0, 21,  0, }, // Phoenix_Islands
    {  533, 1502,  678441,  678459,  678497, 18, 38, 35, }, // Pierre_Miquelon
    {  533, 1518,       0,  678532,       0,  0, 15,  0, }, // Pitcairn
    {  533, 1527,       0,  678547,       0,  0, 14,  0, }, // Ponape
    {  533, 1534,       0,  104252,       0,  0, 19,  0, }, // Pyongyang
    {  533, 1544,  104271,  104286,  104312, 15, 26, 34, }, // Qyzylorda
    {  533, 1554,       0,  678561,       0,  0, 15,  0, }, // Reunion
    {  533, 1562,       0,  678576,       0,  0, 14,  0, }, // Rothera
    {  533, 1570,  678590,  678605,  678632, 15, 27, 24, }, // Sakhalin
    {  533, 1579,  104459,  104471,  104494, 12, 23, 31, }, // Samara
    {  533, 1596,  678656,  678669,  678694, 13, 25, 22, }, // Samoa
    {  533, 1602,       0,  678716,       0,  0, 15,  0, }, // Seychelles
    {  533, 1624,       0,  678731,       0,  0, 28,  0, }, // Singapore
    {  533, 1634,       0,  678759,       0,  0, 25,  0, }, // Solomon
    {  533, 1642,       0,  678784,       0,  0, 23,  0, }, // South_Georgia
    {  533, 1656,       0,  678807,       0,  0, 15,  0, }, // Suriname
    {  533, 1676,       0,  678822,       0,  0, 12,  0, }, // Syowa
    {  533, 1682,       0,  678834,       0,  0, 14,  0, }, // Tahiti
    {  533, 1689,  678848,  678862,  678888, 14, 26, 23, }, // Taipei
    {  533, 1696,       0,  678911,       0,  0, 19,  0, }, // Tajikistan
    {  533, 1724,       0,  678930,       0,  0, 15,  0, }, // Tokelau
    {  533, 1732,  678945,  678958,  678983, 13, 25, 22, }, // Tonga
    {  533, 1738,       0,  679005,       0,  0, 11,  0, }, // Truk
    {  533, 1750,  679016,  679036,  679068, 20, 32, 29, }, // Turkmenistan
    {  533, 1763,       0,  679097,       0,  0, 14,  0, }, // Tuvalu
    {  533, 1777,  679111,  679126,  679153, 15, 27, 24, }, // Uruguay
    {  533, 1792,  679177,  679195,  679225, 18, 30, 27, }, // Uzbekistan
    {  533, 1803,  679252,  679267,  679294, 15, 27, 24, }, // Vanuatu
    {  533, 1811,       0,  679318,       0,  0, 17,  0, }, // Venezuela
    {  533, 1821,  679335,  679354,  679385, 19, 31, 28, }, // Vladivostok
    {  533, 1833,  679413,  679430,  679459, 17, 29, 26, }, // Volgograd
    {  533, 1843,       0,  679485,       0,  0, 14,  0, }, // Vostok
    {  533, 1850,       0,  679499,       0,  0, 19,  0, }, // Wake
    {  533, 1855,       0,  679518,       0,  0, 29,  0, }, // Wallis
    {  533, 1862,  679547,  679562,  679589, 15, 27, 24, }, // Yakutsk
    {  533, 1870,  679613,  679634,  679667, 21, 33, 30, }, // Yekaterinburg
    {  533, 1892,       0,   31545,       0,  0,  5,  0, }, // Yukon
    {  534,    0,  672836,  672846,  672867, 10, 21, 29, }, // Acre Serbian/Latin/Kosovo
    {  534,    5,       0,  679697,       0,  0, 16,  0, }, // Afghanistan
    {  534,   17,       0,  679713,       0,  0, 23,  0, }, // Africa_Central
    {  534,   32,       0,  679736,       0,  0, 21,  0, }, // Africa_Eastern
    {  534,   65,       0,  679757,       0,  0, 19,  0, }, // Africa_Southern
    {  534,   81,  679776,  679797,  679829, 21, 32, 28, }, // Africa_Western
    {  534,  107,   97393,  679857,  679882,  7, 25, 21, }, // Alaska
    {  534,  128,  673122,  673134,  673157, 12, 23, 31, }, // Almaty
    {  534,  135,  679903,  679915,  679939, 12, 24, 20, }, // Amazon
    {  534,  142,  679959,  679990,  680032, 31, 42, 38, }, // America_Central
    {  534,  158,  680070,  680099,  680139, 29, 40, 36, }, // America_Eastern
    {  534,  174,  680175,  680206,  680248, 31, 42, 38, }, // America_Mountain
    {  534,  191,  680286,  680317,  680359, 31, 42, 38, }, // America_Pacific
    {  534,  207,   98077,   98089,   98112, 12, 23, 31, }, // Anadyr
    {  534,  214,  680397,  680408,  680431, 11, 23, 19, }, // Apia
    {  534,  219,  673476,  673489,  673513, 13, 24, 32, }, // Aqtau
    {  534,  225,  673545,  673558,  673582, 13, 24, 32, }, // Aqtobe
    {  534,  232,  680450,  680465,  680491, 15, 26, 22, }, // Arabian
    {  534,  240,  680513,  680528,  680555, 15, 27, 23, }, // Argentina
    {  534,  250,  680578,  680601,  680636, 23, 35, 31, }, // Argentina_Western
    {  534,  268,  680667,  680682,  680709, 15, 27, 23, }, // Armenia
    {  534,  286,  680732,  680747,  680773, 15, 26, 22, }, // Atlantic
    {  534,  295,  680795,  680823,  680862, 28, 39, 35, }, // Australia_Central
    {  534,  313,  680897,  680933,  680980, 36, 47, 43, }, // Australia_CentralWestern
    {  534,  338,  681023,  681049,  681086, 26, 37, 33, }, // Australia_Eastern
    {  534,  356,  681119,  681145,  681182, 26, 37, 33, }, // Australia_Western
    {  534,  374,  681215,  681232,  681261, 17, 29, 25, }, // Azerbaijan
    {  534,  385,  681286,  681297,  681320, 11, 23, 19, }, // Azores
    {  534,  397,  681339,  681354,  681381, 15, 27, 23, }, // Bangladesh
    {  534,  415,       0,  681404,       0,  0, 11,  0, }, // Bhutan
    {  534,  422,       0,  681415,       0,  0, 14,  0, }, // Bolivia
    {  534,  437,  681429,  681444,  681471, 15, 27, 23, }, // Brasilia
    {  534,  454,       0,  681494,       0,  0, 22,  0, }, // Brunei
    {  534,  461,  681516,  681540,  681576, 24, 36, 32, }, // Cape_Verde
    {  534,  478,       0,  681608,       0,  0, 12,  0, }, // Chamorro
    {  534,  487,  681620,  681631,  681654, 11, 23, 19, }, // Chatham
    {  534,  495,  681673,  681683,  681705, 10, 22, 18, }, // Chile
    {  534,  501,  681723,  681733,  681757, 10, 24, 18, }, // China
    {  534,  507,       0,  681775,       0,  0, 20,  0, }, // Christmas
    {  534,  517,       0,  681795,       0,  0, 27,  0, }, // Cocos
    {  534,  523,  681822,  681837,  681864, 15, 27, 23, }, // Colombia
    {  534,  532,  681887,  681906,  681937, 19, 31, 32, }, // Cook
    {  534,  537,   14948,  681969,  681991,  4, 22, 18, }, // Cuba
    {  534,  548,       0,  682009,       0,  0, 12,  0, }, // Davis
    {  534,  564,       0,  682021,       0,  0, 19,  0, }, // DumontDUrville
    {  534,  601,       0,  682040,       0,  0, 19,  0, }, // East_Timor
    {  534,  612,  682059,  682080,  682113, 21, 33, 29, }, // Easter
    {  534,  619,       0,  682142,       0,  0, 13,  0, }, // Ecuador
    {  534,  627,  682155,  682176,  682208, 21, 32, 28, }, // Europe_Central
    {  534,  642,  682236,  682257,  682289, 21, 32, 28, }, // Europe_Eastern
    {  534,  657,       0,  675496,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {  534,  680,  682317,  682338,  682370, 21, 32, 28, }, // Europe_Western
    {  534,  695,  682398,  682422,  682458, 24, 36, 32, }, // Falkland
    {  534,  704,  682490,  682501,  682524, 11, 23, 19, }, // Fiji
    {  534,  709,       0,  682543,       0,  0, 23,  0, }, // French_Guiana
    {  534,  723,       0,  682566,       0,  0, 35,  0, }, // French_Southern
    {  534,  746,       0,  682601,       0,  0, 15,  0, }, // Galapagos
    {  534,  756,       0,  682616,       0,  0, 13,  0, }, // Gambier
    {  534,  764,  682629,  682642,  682667, 13, 25, 21, }, // Georgia
    {  534,  772,       0,  682688,       0,  0, 20,  0, }, // Gilbert_Islands
    {  534,  788,       0,  682708,       0,  0, 24,  0, }, // GMT
    {  534,  830,  675923,  682732,  682766, 16, 34, 30, }, // Greenland_Eastern
    {  534,  848,  676008,  682796,  682830, 16, 34, 30, }, // Greenland_Western
    {  534,  866,       0,  101193,       0,  0, 21,  0, }, // Guam
    {  534,  871,       0,  682860,       0,  0, 14,  0, }, // Gulf
    {  534,  876,       0,  682553,       0,  0, 13,  0, }, // Guyana
    {  534,  883,  682874,  682897,  682931, 23, 34, 30, }, // Hawaii_Aleutian
    {  534,  899,  682961,  682976,  683003, 15, 27, 23, }, // Hong_Kong
    {  534,  909,  683026,  683036,  683058, 10, 22, 18, }, // Hovd
    {  534,  914,       0,  683076,       0,  0, 25,  0, }, // India
    {  534,  920,       0,  683101,       0,  0, 23,  0, }, // Indian_Ocean
    {  534,  933,       0,  683124,       0,  0, 14,  0, }, // Indochina
    {  534,  943,       0,  683138,       0,  0, 28,  0, }, // Indonesia_Central
    {  534,  961,       0,  683166,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  534,  979,       0,  683192,       0,  0, 26,  0, }, // Indonesia_Western
    {  534,  997,  683218,  683228,  683250, 10, 22, 18, }, // Iran
    {  534, 1008,  683268,  683280,  683304, 12, 24, 20, }, // Irkutsk
    {  534, 1016,  683324,  683339,  683365, 15, 26, 22, }, // Israel
    {  534, 1023,  683387,  683401,  683426, 14, 25, 21, }, // Japan
    {  534, 1029,  676628,  676658,  676699, 30, 41, 49, }, // Kamchatka
    {  534, 1047,       0,  683447,       0,  0, 18,  0, }, // Kazakhstan
    {  534, 1058,       0,  683465,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  534, 1077,       0,  683491,       0,  0, 26,  0, }, // Kazakhstan_Western
    {  534, 1106,  683517,  683531,  683556, 14, 25, 21, }, // Korea
    {  534, 1112,       0,  683577,       0,  0, 11,  0, }, // Kosrae
    {  534, 1119,  683588,  683605,  683634, 17, 29, 25, }, // Krasnoyarsk
    {  534, 1151,       0,  683659,       0,  0, 15,  0, }, // Kyrgystan
    {  534, 1161,       0,  676956,       0,  0, 15,  0, }, // Lanka
    {  534, 1175,       0,  683674,       0,  0, 17,  0, }, // Line_Islands
    {  534, 1188,  683691,  683705,  683731, 14, 26, 22, }, // Lord_Howe
    {  534, 1198,  102422,  102433,  102455, 11, 22, 30, }, // Macau
    {  534, 1204,  683753,  683766,  683791, 13, 25, 21, }, // Magadan
    {  534, 1219,       0,  683812,       0,  0, 14,  0, }, // Malaysia
    {  534, 1228,       0,  683826,       0,  0, 13,  0, }, // Maldives
    {  534, 1237,       0,  683839,       0,  0, 12,  0, }, // Marquesas
    {  534, 1247,       0,  683851,       0,  0, 22,  0, }, // Marshall_Islands
    {  534, 1264,  683873,  683889,  683917, 16, 28, 24, }, // Mauritius
    {  534, 1274,       0,  683941,       0,  0, 11,  0, }, // Mawson
    {  534, 1281,  677286,  683952,  683986, 16, 34, 30, }, // Mexico_Pacific
    {  534, 1296,  684016,  684032,  684060, 16, 28, 24, }, // Mongolia
    {  534, 1305,  684084,  684096,  684120, 12, 24, 20, }, // Moscow
    {  534, 1312,       0,  684140,       0,  0, 14,  0, }, // Myanmar
    {  534, 1320,       0,  684154,       0,  0, 11,  0, }, // Nauru
    {  534, 1326,       0,  684165,       0,  0, 11,  0, }, // Nepal
    {  534, 1332,  684176,  684197,  684230, 21, 33, 29, }, // New_Caledonia
    {  534, 1346,  684259,  684276,  684305, 17, 29, 25, }, // New_Zealand
    {  534, 1358,   29457,  684330,  684360, 12, 30, 26, }, // Newfoundland
    {  534, 1371,       0,  684386,       0,  0, 10,  0, }, // Niue
    {  534, 1376,  684396,  684416,  684448, 20, 32, 28, }, // Norfolk
    {  534, 1384,  684476,  684501,  684538, 25, 37, 33, }, // Noronha
    {  534, 1392,       0,  103524,       0,  0, 31,  0, }, // North_Mariana
    {  534, 1406,  684571,  684588,  684617, 17, 29, 25, }, // Novosibirsk
    {  534, 1418,  684642,  684652,  684674, 10, 22, 18, }, // Omsk
    {  534, 1428,  684692,  684706,  684732, 14, 26, 22, }, // Pakistan
    {  534, 1437,       0,  684754,       0,  0, 11,  0, }, // Palau
    {  534, 1443,       0,  684765,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  534, 1460,  684789,  684803,  684829, 14, 26, 22, }, // Paraguay
    {  534, 1469,  684851,  684861,  684883, 10, 22, 18, }, // Peru
    {  534, 1474,  684901,  684915,  684941, 14, 26, 22, }, // Philippines
    {  534, 1486,       0,  684963,       0,  0, 19,  0, }, // Phoenix_Islands
    {  534, 1502,  678441,  684982,  685018, 18, 36, 32, }, // Pierre_Miquelon
    {  534, 1518,       0,  685050,       0,  0, 13,  0, }, // Pitcairn
    {  534, 1527,       0,  685063,       0,  0, 12,  0, }, // Ponape
    {  534, 1534,       0,  685075,       0,  0, 17,  0, }, // Pyongyang
    {  534, 1544,  104271,  104286,  104312, 15, 26, 34, }, // Qyzylorda
    {  534, 1554,       0,  685092,       0,  0, 13,  0, }, // Reunion
    {  534, 1562,       0,  685105,       0,  0, 12,  0, }, // Rothera
    {  534, 1570,  685117,  685130,  685155, 13, 25, 21, }, // Sakhalin
    {  534, 1579,  104459,  104471,  104494, 12, 23, 31, }, // Samara
    {  534, 1596,  685176,  685187,  685210, 11, 23, 19, }, // Samoa
    {  534, 1602,       0,  685229,       0,  0, 13,  0, }, // Seychelles
    {  534, 1624,       0,  685242,       0,  0, 26,  0, }, // Singapore
    {  534, 1634,       0,  685268,       0,  0, 23,  0, }, // Solomon
    {  534, 1642,       0,  685291,       0,  0, 21,  0, }, // South_Georgia
    {  534, 1656,       0,  685312,       0,  0, 13,  0, }, // Suriname
    {  534, 1676,       0,  685325,       0,  0, 10,  0, }, // Syowa
    {  534, 1682,       0,  685335,       0,  0, 12,  0, }, // Tahiti
    {  534, 1689,  685347,  685359,  685383, 12, 24, 20, }, // Taipei
    {  534, 1696,       0,  685403,       0,  0, 17,  0, }, // Tajikistan
    {  534, 1724,       0,  685420,       0,  0, 13,  0, }, // Tokelau
    {  534, 1732,  685433,  685444,  685467, 11, 23, 19, }, // Tonga
    {  534, 1738,       0,  685486,       0,  0, 10,  0, }, // Truk
    {  534, 1750,  685496,  685514,  685544, 18, 30, 26, }, // Turkmenistan
    {  534, 1763,       0,  685570,       0,  0, 12,  0, }, // Tuvalu
    {  534, 1777,  685582,  685595,  685620, 13, 25, 21, }, // Uruguay
    {  534, 1792,  685641,  685657,  685685, 16, 28, 24, }, // Uzbekistan
    {  534, 1803,  685709,  685722,  685747, 13, 25, 21, }, // Vanuatu
    {  534, 1811,       0,  685768,       0,  0, 15,  0, }, // Venezuela
    {  534, 1821,  685783,  685800,  685829, 17, 29, 25, }, // Vladivostok
    {  534, 1833,  685854,  685869,  685896, 15, 27, 23, }, // Volgograd
    {  534, 1843,       0,  685919,       0,  0, 12,  0, }, // Vostok
    {  534, 1850,       0,  685931,       0,  0, 17,  0, }, // Wake
    {  534, 1855,       0,  685948,       0,  0, 27,  0, }, // Wallis
    {  534, 1862,  685975,  685988,  686013, 13, 25, 21, }, // Yakutsk
    {  534, 1870,  686034,  686053,  686084, 19, 31, 27, }, // Yekaterinburg
    {  534, 1892,       0,   31545,       0,  0,  5,  0, }, // Yukon
    {  535,    0,  672836,  672846,  672867, 10, 21, 29, }, // Acre Serbian/Latin/Montenegro
    {  535,    5,       0,  679697,       0,  0, 16,  0, }, // Afghanistan
    {  535,   17,       0,  679713,       0,  0, 23,  0, }, // Africa_Central
    {  535,   32,       0,  679736,       0,  0, 21,  0, }, // Africa_Eastern
    {  535,   65,       0,  679757,       0,  0, 19,  0, }, // Africa_Southern
    {  535,   81,  679776,  679797,  679829, 21, 32, 28, }, // Africa_Western
    {  535,  107,   97393,  679857,  679882,  7, 25, 21, }, // Alaska
    {  535,  128,  673122,  673134,  673157, 12, 23, 31, }, // Almaty
    {  535,  135,  679903,  679915,  679939, 12, 24, 20, }, // Amazon
    {  535,  142,  679959,  679990,  680032, 31, 42, 38, }, // America_Central
    {  535,  158,  680070,  680099,  680139, 29, 40, 36, }, // America_Eastern
    {  535,  174,  680175,  680206,  680248, 31, 42, 38, }, // America_Mountain
    {  535,  191,  680286,  680317,  680359, 31, 42, 38, }, // America_Pacific
    {  535,  207,   98077,   98089,   98112, 12, 23, 31, }, // Anadyr
    {  535,  214,  680397,  680408,  680431, 11, 23, 19, }, // Apia
    {  535,  219,  673476,  673489,  673513, 13, 24, 32, }, // Aqtau
    {  535,  225,  673545,  673558,  673582, 13, 24, 32, }, // Aqtobe
    {  535,  232,  680450,  680465,  680491, 15, 26, 22, }, // Arabian
    {  535,  240,  680513,  680528,  680555, 15, 27, 23, }, // Argentina
    {  535,  250,  680578,  680601,  680636, 23, 35, 31, }, // Argentina_Western
    {  535,  268,  680667,  680682,  680709, 15, 27, 23, }, // Armenia
    {  535,  286,  680732,  680747,  680773, 15, 26, 22, }, // Atlantic
    {  535,  295,  680795,  680823,  680862, 28, 39, 35, }, // Australia_Central
    {  535,  313,  680897,  680933,  680980, 36, 47, 43, }, // Australia_CentralWestern
    {  535,  338,  681023,  681049,  681086, 26, 37, 33, }, // Australia_Eastern
    {  535,  356,  681119,  681145,  681182, 26, 37, 33, }, // Australia_Western
    {  535,  374,  681215,  681232,  681261, 17, 29, 25, }, // Azerbaijan
    {  535,  385,  681286,  681297,  681320, 11, 23, 19, }, // Azores
    {  535,  397,  681339,  681354,  681381, 15, 27, 23, }, // Bangladesh
    {  535,  415,       0,  681404,       0,  0, 11,  0, }, // Bhutan
    {  535,  422,       0,  681415,       0,  0, 14,  0, }, // Bolivia
    {  535,  437,  681429,  681444,  681471, 15, 27, 23, }, // Brasilia
    {  535,  454,       0,  681494,       0,  0, 22,  0, }, // Brunei
    {  535,  461,  681516,  681540,  681576, 24, 36, 32, }, // Cape_Verde
    {  535,  478,       0,  681608,       0,  0, 12,  0, }, // Chamorro
    {  535,  487,  681620,  681631,  681654, 11, 23, 19, }, // Chatham
    {  535,  495,  681673,  681683,  681705, 10, 22, 18, }, // Chile
    {  535,  501,  681723,  681733,  681757, 10, 24, 18, }, // China
    {  535,  507,       0,  681775,       0,  0, 20,  0, }, // Christmas
    {  535,  517,       0,  681795,       0,  0, 27,  0, }, // Cocos
    {  535,  523,  681822,  681837,  681864, 15, 27, 23, }, // Colombia
    {  535,  532,  681887,  681906,  681937, 19, 31, 32, }, // Cook
    {  535,  537,   14948,  681969,  681991,  4, 22, 18, }, // Cuba
    {  535,  548,       0,  682009,       0,  0, 12,  0, }, // Davis
    {  535,  564,       0,  682021,       0,  0, 19,  0, }, // DumontDUrville
    {  535,  601,       0,  682040,       0,  0, 19,  0, }, // East_Timor
    {  535,  612,  682059,  682080,  682113, 21, 33, 29, }, // Easter
    {  535,  619,       0,  682142,       0,  0, 13,  0, }, // Ecuador
    {  535,  627,  682155,  682176,  682208, 21, 32, 28, }, // Europe_Central
    {  535,  642,  682236,  682257,  682289, 21, 32, 28, }, // Europe_Eastern
    {  535,  657,       0,  675496,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {  535,  680,  682317,  682338,  682370, 21, 32, 28, }, // Europe_Western
    {  535,  695,  682398,  682422,  682458, 24, 36, 32, }, // Falkland
    {  535,  704,  682490,  682501,  682524, 11, 23, 19, }, // Fiji
    {  535,  709,       0,  682543,       0,  0, 23,  0, }, // French_Guiana
    {  535,  723,       0,  682566,       0,  0, 35,  0, }, // French_Southern
    {  535,  746,       0,  682601,       0,  0, 15,  0, }, // Galapagos
    {  535,  756,       0,  682616,       0,  0, 13,  0, }, // Gambier
    {  535,  764,  682629,  682642,  682667, 13, 25, 21, }, // Georgia
    {  535,  772,       0,  682688,       0,  0, 20,  0, }, // Gilbert_Islands
    {  535,  788,       0,  682708,       0,  0, 24,  0, }, // GMT
    {  535,  830,  675923,  682732,  682766, 16, 34, 30, }, // Greenland_Eastern
    {  535,  848,  676008,  682796,  682830, 16, 34, 30, }, // Greenland_Western
    {  535,  866,       0,  101193,       0,  0, 21,  0, }, // Guam
    {  535,  871,       0,  682860,       0,  0, 14,  0, }, // Gulf
    {  535,  876,       0,  682553,       0,  0, 13,  0, }, // Guyana
    {  535,  883,  682874,  682897,  682931, 23, 34, 30, }, // Hawaii_Aleutian
    {  535,  899,  682961,  682976,  683003, 15, 27, 23, }, // Hong_Kong
    {  535,  909,  683026,  683036,  683058, 10, 22, 18, }, // Hovd
    {  535,  914,       0,  683076,       0,  0, 25,  0, }, // India
    {  535,  920,       0,  683101,       0,  0, 23,  0, }, // Indian_Ocean
    {  535,  933,       0,  683124,       0,  0, 14,  0, }, // Indochina
    {  535,  943,       0,  683138,       0,  0, 28,  0, }, // Indonesia_Central
    {  535,  961,       0,  683166,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  535,  979,       0,  683192,       0,  0, 26,  0, }, // Indonesia_Western
    {  535,  997,  683218,  683228,  683250, 10, 22, 18, }, // Iran
    {  535, 1008,  683268,  683280,  683304, 12, 24, 20, }, // Irkutsk
    {  535, 1016,  683324,  683339,  683365, 15, 26, 22, }, // Israel
    {  535, 1023,  683387,  683401,  683426, 14, 25, 21, }, // Japan
    {  535, 1029,  676628,  676658,  676699, 30, 41, 49, }, // Kamchatka
    {  535, 1047,       0,  683447,       0,  0, 18,  0, }, // Kazakhstan
    {  535, 1058,       0,  683465,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  535, 1077,       0,  683491,       0,  0, 26,  0, }, // Kazakhstan_Western
    {  535, 1106,  683517,  683531,  683556, 14, 25, 21, }, // Korea
    {  535, 1112,       0,  683577,       0,  0, 11,  0, }, // Kosrae
    {  535, 1119,  683588,  683605,  683634, 17, 29, 25, }, // Krasnoyarsk
    {  535, 1151,       0,  683659,       0,  0, 15,  0, }, // Kyrgystan
    {  535, 1161,       0,  676956,       0,  0, 15,  0, }, // Lanka
    {  535, 1175,       0,  683674,       0,  0, 17,  0, }, // Line_Islands
    {  535, 1188,  683691,  683705,  683731, 14, 26, 22, }, // Lord_Howe
    {  535, 1198,  102422,  102433,  102455, 11, 22, 30, }, // Macau
    {  535, 1204,  683753,  683766,  683791, 13, 25, 21, }, // Magadan
    {  535, 1219,       0,  683812,       0,  0, 14,  0, }, // Malaysia
    {  535, 1228,       0,  683826,       0,  0, 13,  0, }, // Maldives
    {  535, 1237,       0,  683839,       0,  0, 12,  0, }, // Marquesas
    {  535, 1247,       0,  683851,       0,  0, 22,  0, }, // Marshall_Islands
    {  535, 1264,  683873,  683889,  683917, 16, 28, 24, }, // Mauritius
    {  535, 1274,       0,  683941,       0,  0, 11,  0, }, // Mawson
    {  535, 1281,  677286,  683952,  683986, 16, 34, 30, }, // Mexico_Pacific
    {  535, 1296,  684016,  684032,  684060, 16, 28, 24, }, // Mongolia
    {  535, 1305,  684084,  684096,  684120, 12, 24, 20, }, // Moscow
    {  535, 1312,       0,  684140,       0,  0, 14,  0, }, // Myanmar
    {  535, 1320,       0,  684154,       0,  0, 11,  0, }, // Nauru
    {  535, 1326,       0,  684165,       0,  0, 11,  0, }, // Nepal
    {  535, 1332,  684176,  684197,  684230, 21, 33, 29, }, // New_Caledonia
    {  535, 1346,  684259,  684276,  684305, 17, 29, 25, }, // New_Zealand
    {  535, 1358,   29457,  684330,  684360, 12, 30, 26, }, // Newfoundland
    {  535, 1371,       0,  684386,       0,  0, 10,  0, }, // Niue
    {  535, 1376,  684396,  684416,  684448, 20, 32, 28, }, // Norfolk
    {  535, 1384,  684476,  684501,  684538, 25, 37, 33, }, // Noronha
    {  535, 1392,       0,  103524,       0,  0, 31,  0, }, // North_Mariana
    {  535, 1406,  684571,  684588,  684617, 17, 29, 25, }, // Novosibirsk
    {  535, 1418,  684642,  684652,  684674, 10, 22, 18, }, // Omsk
    {  535, 1428,  684692,  684706,  684732, 14, 26, 22, }, // Pakistan
    {  535, 1437,       0,  684754,       0,  0, 11,  0, }, // Palau
    {  535, 1443,       0,  684765,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  535, 1460,  684789,  684803,  684829, 14, 26, 22, }, // Paraguay
    {  535, 1469,  684851,  684861,  684883, 10, 22, 18, }, // Peru
    {  535, 1474,  684901,  684915,  684941, 14, 26, 22, }, // Philippines
    {  535, 1486,       0,  684963,       0,  0, 19,  0, }, // Phoenix_Islands
    {  535, 1502,  678441,  684982,  685018, 18, 36, 32, }, // Pierre_Miquelon
    {  535, 1518,       0,  685050,       0,  0, 13,  0, }, // Pitcairn
    {  535, 1527,       0,  685063,       0,  0, 12,  0, }, // Ponape
    {  535, 1534,       0,  685075,       0,  0, 17,  0, }, // Pyongyang
    {  535, 1544,  104271,  104286,  104312, 15, 26, 34, }, // Qyzylorda
    {  535, 1554,       0,  685092,       0,  0, 13,  0, }, // Reunion
    {  535, 1562,       0,  685105,       0,  0, 12,  0, }, // Rothera
    {  535, 1570,  685117,  685130,  685155, 13, 25, 21, }, // Sakhalin
    {  535, 1579,  104459,  104471,  104494, 12, 23, 31, }, // Samara
    {  535, 1596,  685176,  685187,  685210, 11, 23, 19, }, // Samoa
    {  535, 1602,       0,  685229,       0,  0, 13,  0, }, // Seychelles
    {  535, 1624,       0,  685242,       0,  0, 26,  0, }, // Singapore
    {  535, 1634,       0,  685268,       0,  0, 23,  0, }, // Solomon
    {  535, 1642,       0,  685291,       0,  0, 21,  0, }, // South_Georgia
    {  535, 1656,       0,  685312,       0,  0, 13,  0, }, // Suriname
    {  535, 1676,       0,  685325,       0,  0, 10,  0, }, // Syowa
    {  535, 1682,       0,  685335,       0,  0, 12,  0, }, // Tahiti
    {  535, 1689,  685347,  685359,  685383, 12, 24, 20, }, // Taipei
    {  535, 1696,       0,  685403,       0,  0, 17,  0, }, // Tajikistan
    {  535, 1724,       0,  685420,       0,  0, 13,  0, }, // Tokelau
    {  535, 1732,  685433,  685444,  685467, 11, 23, 19, }, // Tonga
    {  535, 1738,       0,  685486,       0,  0, 10,  0, }, // Truk
    {  535, 1750,  685496,  685514,  685544, 18, 30, 26, }, // Turkmenistan
    {  535, 1763,       0,  685570,       0,  0, 12,  0, }, // Tuvalu
    {  535, 1777,  685582,  685595,  685620, 13, 25, 21, }, // Uruguay
    {  535, 1792,  685641,  685657,  685685, 16, 28, 24, }, // Uzbekistan
    {  535, 1803,  685709,  685722,  685747, 13, 25, 21, }, // Vanuatu
    {  535, 1811,       0,  685768,       0,  0, 15,  0, }, // Venezuela
    {  535, 1821,  685783,  685800,  685829, 17, 29, 25, }, // Vladivostok
    {  535, 1833,  685854,  685869,  685896, 15, 27, 23, }, // Volgograd
    {  535, 1843,       0,  685919,       0,  0, 12,  0, }, // Vostok
    {  535, 1850,       0,  685931,       0,  0, 17,  0, }, // Wake
    {  535, 1855,       0,  685948,       0,  0, 27,  0, }, // Wallis
    {  535, 1862,  685975,  685988,  686013, 13, 25, 21, }, // Yakutsk
    {  535, 1870,  686034,  686053,  686084, 19, 31, 27, }, // Yekaterinburg
    {  535, 1892,       0,   31545,       0,  0,  5,  0, }, // Yukon
    {  536,    0,  672836,  672846,  672867, 10, 21, 29, }, // Acre Serbian/Latin/Serbia
    {  536,    5,       0,  679697,       0,  0, 16,  0, }, // Afghanistan
    {  536,   17,       0,  679713,       0,  0, 23,  0, }, // Africa_Central
    {  536,   32,       0,  679736,       0,  0, 21,  0, }, // Africa_Eastern
    {  536,   65,       0,  679757,       0,  0, 19,  0, }, // Africa_Southern
    {  536,   81,  679776,  679797,  679829, 21, 32, 28, }, // Africa_Western
    {  536,  107,   97393,  679857,  679882,  7, 25, 21, }, // Alaska
    {  536,  128,  673122,  673134,  673157, 12, 23, 31, }, // Almaty
    {  536,  135,  679903,  679915,  679939, 12, 24, 20, }, // Amazon
    {  536,  142,  679959,  679990,  680032, 31, 42, 38, }, // America_Central
    {  536,  158,  680070,  680099,  680139, 29, 40, 36, }, // America_Eastern
    {  536,  174,  680175,  680206,  680248, 31, 42, 38, }, // America_Mountain
    {  536,  191,  680286,  680317,  680359, 31, 42, 38, }, // America_Pacific
    {  536,  207,   98077,   98089,   98112, 12, 23, 31, }, // Anadyr
    {  536,  214,  680397,  680408,  680431, 11, 23, 19, }, // Apia
    {  536,  219,  673476,  673489,  673513, 13, 24, 32, }, // Aqtau
    {  536,  225,  673545,  673558,  673582, 13, 24, 32, }, // Aqtobe
    {  536,  232,  680450,  680465,  680491, 15, 26, 22, }, // Arabian
    {  536,  240,  680513,  680528,  680555, 15, 27, 23, }, // Argentina
    {  536,  250,  680578,  680601,  680636, 23, 35, 31, }, // Argentina_Western
    {  536,  268,  680667,  680682,  680709, 15, 27, 23, }, // Armenia
    {  536,  286,  680732,  680747,  680773, 15, 26, 22, }, // Atlantic
    {  536,  295,  680795,  680823,  680862, 28, 39, 35, }, // Australia_Central
    {  536,  313,  680897,  680933,  680980, 36, 47, 43, }, // Australia_CentralWestern
    {  536,  338,  681023,  681049,  681086, 26, 37, 33, }, // Australia_Eastern
    {  536,  356,  681119,  681145,  681182, 26, 37, 33, }, // Australia_Western
    {  536,  374,  681215,  681232,  681261, 17, 29, 25, }, // Azerbaijan
    {  536,  385,  681286,  681297,  681320, 11, 23, 19, }, // Azores
    {  536,  397,  681339,  681354,  681381, 15, 27, 23, }, // Bangladesh
    {  536,  415,       0,  681404,       0,  0, 11,  0, }, // Bhutan
    {  536,  422,       0,  681415,       0,  0, 14,  0, }, // Bolivia
    {  536,  437,  681429,  681444,  681471, 15, 27, 23, }, // Brasilia
    {  536,  454,       0,  681494,       0,  0, 22,  0, }, // Brunei
    {  536,  461,  681516,  681540,  681576, 24, 36, 32, }, // Cape_Verde
    {  536,  478,       0,  681608,       0,  0, 12,  0, }, // Chamorro
    {  536,  487,  681620,  681631,  681654, 11, 23, 19, }, // Chatham
    {  536,  495,  681673,  681683,  681705, 10, 22, 18, }, // Chile
    {  536,  501,  681723,  681733,  681757, 10, 24, 18, }, // China
    {  536,  507,       0,  681775,       0,  0, 20,  0, }, // Christmas
    {  536,  517,       0,  681795,       0,  0, 27,  0, }, // Cocos
    {  536,  523,  681822,  681837,  681864, 15, 27, 23, }, // Colombia
    {  536,  532,  681887,  681906,  681937, 19, 31, 32, }, // Cook
    {  536,  537,   14948,  681969,  681991,  4, 22, 18, }, // Cuba
    {  536,  548,       0,  682009,       0,  0, 12,  0, }, // Davis
    {  536,  564,       0,  682021,       0,  0, 19,  0, }, // DumontDUrville
    {  536,  601,       0,  682040,       0,  0, 19,  0, }, // East_Timor
    {  536,  612,  682059,  682080,  682113, 21, 33, 29, }, // Easter
    {  536,  619,       0,  682142,       0,  0, 13,  0, }, // Ecuador
    {  536,  627,  682155,  682176,  682208, 21, 32, 28, }, // Europe_Central
    {  536,  642,  682236,  682257,  682289, 21, 32, 28, }, // Europe_Eastern
    {  536,  657,       0,  675496,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {  536,  680,  682317,  682338,  682370, 21, 32, 28, }, // Europe_Western
    {  536,  695,  682398,  682422,  682458, 24, 36, 32, }, // Falkland
    {  536,  704,  682490,  682501,  682524, 11, 23, 19, }, // Fiji
    {  536,  709,       0,  682543,       0,  0, 23,  0, }, // French_Guiana
    {  536,  723,       0,  682566,       0,  0, 35,  0, }, // French_Southern
    {  536,  746,       0,  682601,       0,  0, 15,  0, }, // Galapagos
    {  536,  756,       0,  682616,       0,  0, 13,  0, }, // Gambier
    {  536,  764,  682629,  682642,  682667, 13, 25, 21, }, // Georgia
    {  536,  772,       0,  682688,       0,  0, 20,  0, }, // Gilbert_Islands
    {  536,  788,       0,  682708,       0,  0, 24,  0, }, // GMT
    {  536,  830,  675923,  682732,  682766, 16, 34, 30, }, // Greenland_Eastern
    {  536,  848,  676008,  682796,  682830, 16, 34, 30, }, // Greenland_Western
    {  536,  866,       0,  101193,       0,  0, 21,  0, }, // Guam
    {  536,  871,       0,  682860,       0,  0, 14,  0, }, // Gulf
    {  536,  876,       0,  682553,       0,  0, 13,  0, }, // Guyana
    {  536,  883,  682874,  682897,  682931, 23, 34, 30, }, // Hawaii_Aleutian
    {  536,  899,  682961,  682976,  683003, 15, 27, 23, }, // Hong_Kong
    {  536,  909,  683026,  683036,  683058, 10, 22, 18, }, // Hovd
    {  536,  914,       0,  683076,       0,  0, 25,  0, }, // India
    {  536,  920,       0,  683101,       0,  0, 23,  0, }, // Indian_Ocean
    {  536,  933,       0,  683124,       0,  0, 14,  0, }, // Indochina
    {  536,  943,       0,  683138,       0,  0, 28,  0, }, // Indonesia_Central
    {  536,  961,       0,  683166,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  536,  979,       0,  683192,       0,  0, 26,  0, }, // Indonesia_Western
    {  536,  997,  683218,  683228,  683250, 10, 22, 18, }, // Iran
    {  536, 1008,  683268,  683280,  683304, 12, 24, 20, }, // Irkutsk
    {  536, 1016,  683324,  683339,  683365, 15, 26, 22, }, // Israel
    {  536, 1023,  683387,  683401,  683426, 14, 25, 21, }, // Japan
    {  536, 1029,  676628,  676658,  676699, 30, 41, 49, }, // Kamchatka
    {  536, 1047,       0,  683447,       0,  0, 18,  0, }, // Kazakhstan
    {  536, 1058,       0,  683465,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  536, 1077,       0,  683491,       0,  0, 26,  0, }, // Kazakhstan_Western
    {  536, 1106,  683517,  683531,  683556, 14, 25, 21, }, // Korea
    {  536, 1112,       0,  683577,       0,  0, 11,  0, }, // Kosrae
    {  536, 1119,  683588,  683605,  683634, 17, 29, 25, }, // Krasnoyarsk
    {  536, 1151,       0,  683659,       0,  0, 15,  0, }, // Kyrgystan
    {  536, 1161,       0,  676956,       0,  0, 15,  0, }, // Lanka
    {  536, 1175,       0,  683674,       0,  0, 17,  0, }, // Line_Islands
    {  536, 1188,  683691,  683705,  683731, 14, 26, 22, }, // Lord_Howe
    {  536, 1198,  102422,  102433,  102455, 11, 22, 30, }, // Macau
    {  536, 1204,  683753,  683766,  683791, 13, 25, 21, }, // Magadan
    {  536, 1219,       0,  683812,       0,  0, 14,  0, }, // Malaysia
    {  536, 1228,       0,  683826,       0,  0, 13,  0, }, // Maldives
    {  536, 1237,       0,  683839,       0,  0, 12,  0, }, // Marquesas
    {  536, 1247,       0,  683851,       0,  0, 22,  0, }, // Marshall_Islands
    {  536, 1264,  683873,  683889,  683917, 16, 28, 24, }, // Mauritius
    {  536, 1274,       0,  683941,       0,  0, 11,  0, }, // Mawson
    {  536, 1281,  677286,  683952,  683986, 16, 34, 30, }, // Mexico_Pacific
    {  536, 1296,  684016,  684032,  684060, 16, 28, 24, }, // Mongolia
    {  536, 1305,  684084,  684096,  684120, 12, 24, 20, }, // Moscow
    {  536, 1312,       0,  684140,       0,  0, 14,  0, }, // Myanmar
    {  536, 1320,       0,  684154,       0,  0, 11,  0, }, // Nauru
    {  536, 1326,       0,  684165,       0,  0, 11,  0, }, // Nepal
    {  536, 1332,  684176,  684197,  684230, 21, 33, 29, }, // New_Caledonia
    {  536, 1346,  684259,  684276,  684305, 17, 29, 25, }, // New_Zealand
    {  536, 1358,   29457,  684330,  684360, 12, 30, 26, }, // Newfoundland
    {  536, 1371,       0,  684386,       0,  0, 10,  0, }, // Niue
    {  536, 1376,  684396,  684416,  684448, 20, 32, 28, }, // Norfolk
    {  536, 1384,  684476,  684501,  684538, 25, 37, 33, }, // Noronha
    {  536, 1392,       0,  103524,       0,  0, 31,  0, }, // North_Mariana
    {  536, 1406,  684571,  684588,  684617, 17, 29, 25, }, // Novosibirsk
    {  536, 1418,  684642,  684652,  684674, 10, 22, 18, }, // Omsk
    {  536, 1428,  684692,  684706,  684732, 14, 26, 22, }, // Pakistan
    {  536, 1437,       0,  684754,       0,  0, 11,  0, }, // Palau
    {  536, 1443,       0,  684765,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  536, 1460,  684789,  684803,  684829, 14, 26, 22, }, // Paraguay
    {  536, 1469,  684851,  684861,  684883, 10, 22, 18, }, // Peru
    {  536, 1474,  684901,  684915,  684941, 14, 26, 22, }, // Philippines
    {  536, 1486,       0,  684963,       0,  0, 19,  0, }, // Phoenix_Islands
    {  536, 1502,  678441,  684982,  685018, 18, 36, 32, }, // Pierre_Miquelon
    {  536, 1518,       0,  685050,       0,  0, 13,  0, }, // Pitcairn
    {  536, 1527,       0,  685063,       0,  0, 12,  0, }, // Ponape
    {  536, 1534,       0,  685075,       0,  0, 17,  0, }, // Pyongyang
    {  536, 1544,  104271,  104286,  104312, 15, 26, 34, }, // Qyzylorda
    {  536, 1554,       0,  685092,       0,  0, 13,  0, }, // Reunion
    {  536, 1562,       0,  685105,       0,  0, 12,  0, }, // Rothera
    {  536, 1570,  685117,  685130,  685155, 13, 25, 21, }, // Sakhalin
    {  536, 1579,  104459,  104471,  104494, 12, 23, 31, }, // Samara
    {  536, 1596,  685176,  685187,  685210, 11, 23, 19, }, // Samoa
    {  536, 1602,       0,  685229,       0,  0, 13,  0, }, // Seychelles
    {  536, 1624,       0,  685242,       0,  0, 26,  0, }, // Singapore
    {  536, 1634,       0,  685268,       0,  0, 23,  0, }, // Solomon
    {  536, 1642,       0,  685291,       0,  0, 21,  0, }, // South_Georgia
    {  536, 1656,       0,  685312,       0,  0, 13,  0, }, // Suriname
    {  536, 1676,       0,  685325,       0,  0, 10,  0, }, // Syowa
    {  536, 1682,       0,  685335,       0,  0, 12,  0, }, // Tahiti
    {  536, 1689,  685347,  685359,  685383, 12, 24, 20, }, // Taipei
    {  536, 1696,       0,  685403,       0,  0, 17,  0, }, // Tajikistan
    {  536, 1724,       0,  685420,       0,  0, 13,  0, }, // Tokelau
    {  536, 1732,  685433,  685444,  685467, 11, 23, 19, }, // Tonga
    {  536, 1738,       0,  685486,       0,  0, 10,  0, }, // Truk
    {  536, 1750,  685496,  685514,  685544, 18, 30, 26, }, // Turkmenistan
    {  536, 1763,       0,  685570,       0,  0, 12,  0, }, // Tuvalu
    {  536, 1777,  685582,  685595,  685620, 13, 25, 21, }, // Uruguay
    {  536, 1792,  685641,  685657,  685685, 16, 28, 24, }, // Uzbekistan
    {  536, 1803,  685709,  685722,  685747, 13, 25, 21, }, // Vanuatu
    {  536, 1811,       0,  685768,       0,  0, 15,  0, }, // Venezuela
    {  536, 1821,  685783,  685800,  685829, 17, 29, 25, }, // Vladivostok
    {  536, 1833,  685854,  685869,  685896, 15, 27, 23, }, // Volgograd
    {  536, 1843,       0,  685919,       0,  0, 12,  0, }, // Vostok
    {  536, 1850,       0,  685931,       0,  0, 17,  0, }, // Wake
    {  536, 1855,       0,  685948,       0,  0, 27,  0, }, // Wallis
    {  536, 1862,  685975,  685988,  686013, 13, 25, 21, }, // Yakutsk
    {  536, 1870,  686034,  686053,  686084, 19, 31, 27, }, // Yekaterinburg
    {  536, 1892,       0,   31545,       0,  0,  5,  0, }, // Yukon
    {  539,  788,       0,  686111,       0,  0,  6,  0, }, // GMT Sichuan Yi/Yi/China
    {  540,    5,       0,  686117,       0,  0, 20,  0, }, // Afghanistan Sicilian/Latin/Italy
    {  540,   17,       0,  686137,       0,  0, 24,  0, }, // Africa_Central
    {  540,   32,       0,  686161,       0,  0, 26,  0, }, // Africa_Eastern
    {  540,   65,       0,  686187,       0,  0, 35,  0, }, // Africa_Southern
    {  540,   81,  686222,  686248,  686281, 26, 33, 33, }, // Africa_Western
    {  540,  107,  686314,  686329,  686351, 15, 22, 22, }, // Alaska
    {  540,  135,  686373,  686391,  686416, 18, 25, 25, }, // Amazon
    {  540,  142,  686441,  686453,  686472, 12, 19, 19, }, // America_Central
    {  540,  158,  686491,  686504,  686524, 13, 20, 20, }, // America_Eastern
    {  540,  174,  686544,  686559,  686581, 15, 22, 22, }, // America_Mountain
    {  540,  191,  686603,  686618,  686640, 15, 22, 22, }, // America_Pacific
    {  540,  214,  686662,  686673,  686691, 11, 18, 18, }, // Apia
    {  540,  232,  686709,  686719,  686736, 10, 17, 17, }, // Arabian
    {  540,  240,  686753,  686771,  686796, 18, 25, 25, }, // Argentina
    {  540,  250,  686821,  686850,  686886, 29, 36, 36, }, // Argentina_Western
    {  540,  268,  686922,  686938,  686961, 16, 23, 23, }, // Armenia
    {  540,  286,  686984,  687002,  687027, 18, 25, 25, }, // Atlantic
    {  540,  295,  687052,  687079,  687113, 27, 34, 34, }, // Australia_Central
    {  540,  313,  687147,  687185,  687230, 38, 45, 45, }, // Australia_CentralWestern
    {  540,  338,  687275,  687304,  687340, 29, 36, 36, }, // Australia_Eastern
    {  540,  356,  687376,  687405,  687441, 29, 36, 36, }, // Australia_Western
    {  540,  374,  687477,  687496,  687522, 19, 26, 26, }, // Azerbaijan
    {  540,  385,  687548,  687564,  687587, 16, 23, 23, }, // Azores
    {  540,  397,  687610,  687627,  687651, 17, 24, 24, }, // Bangladesh
    {  540,  415,       0,  687675,       0,  0, 13,  0, }, // Bhutan
    {  540,  422,       0,  687688,       0,  0, 14,  0, }, // Bolivia
    {  540,  437,  687702,  687717,  687739, 15, 22, 22, }, // Brasilia
    {  540,  454,       0,  687761,       0,  0, 13,  0, }, // Brunei
    {  540,  461,  687774,  687791,  687815, 17, 24, 24, }, // Cape_Verde
    {  540,  478,       0,  687839,       0,  0, 15,  0, }, // Chamorro
    {  540,  487,  687854,  687868,  687889, 14, 21, 21, }, // Chatham
    {  540,  495,  687910,  687921,  687939, 11, 18, 18, }, // Chile
    {  540,  501,  687957,  687968,  687986, 11, 18, 18, }, // China
    {  540,  507,       0,  688004,       0,  0, 24,  0, }, // Christmas
    {  540,  517,       0,  688028,       0,  0, 20,  0, }, // Cocos
    {  540,  523,  688048,  688063,  688085, 15, 22, 22, }, // Colombia
    {  540,  532,  688107,  688126,  688152, 19, 26, 26, }, // Cook
    {  540,  537,  688178,  688190,  688209, 12, 19, 19, }, // Cuba
    {  540,  548,       0,  688228,       0,  0, 12,  0, }, // Davis
    {  540,  564,       0,  688240,       0,  0, 23,  0, }, // DumontDUrville
    {  540,  601,       0,  688263,       0,  0, 23,  0, }, // East_Timor
    {  540,  612,  688286,  688309,  688339, 23, 30, 30, }, // Easter
    {  540,  619,       0,  688369,       0,  0, 16,  0, }, // Ecuador
    {  540,  627,  688385,  688405,  688432, 20, 27, 27, }, // Europe_Central
    {  540,  642,  688459,  688485,  688518, 26, 33, 33, }, // Europe_Eastern
    {  540,  657,       0,  688551,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  540,  680,  688582,  688608,  688641, 26, 33, 33, }, // Europe_Western
    {  540,  695,  688674,  688697,  688727, 23, 30, 30, }, // Falkland
    {  540,  704,  688757,  688768,  688786, 11, 18, 18, }, // Fiji
    {  540,  709,       0,  688804,       0,  0, 22,  0, }, // French_Guiana
    {  540,  723,       0,  688826,       0,  0, 41,  0, }, // French_Southern
    {  540,  746,       0,  688867,       0,  0, 16,  0, }, // Galapagos
    {  540,  756,       0,  688883,       0,  0, 14,  0, }, // Gambier
    {  540,  764,  688897,  688911,  688932, 14, 21, 21, }, // Georgia
    {  540,  772,       0,  688953,       0,  0, 22,  0, }, // Gilbert_Islands
    {  540,  788,       0,  688975,       0,  0, 24,  0, }, // GMT
    {  540,  802,  688999,  689017,  689042, 18, 25, 25, }, // Greenland
    {  540,  830,  689067,  689095,  689130, 28, 35, 38, }, // Greenland_Eastern
    {  540,  848,  689168,  689196,  689231, 28, 35, 35, }, // Greenland_Western
    {  540,  871,       0,  689266,       0,  0, 20,  0, }, // Gulf
    {  540,  876,       0,  688804,       0,  0, 13,  0, }, // Guyana
    {  540,  883,  689286,  689301,  689323, 15, 22, 22, }, // Hawaii_Aleutian
    {  540,  899,  689345,  689361,  689384, 16, 23, 23, }, // Hong_Kong
    {  540,  909,  689407,  689418,  689436, 11, 18, 18, }, // Hovd
    {  540,  914,       0,  689454,       0,  0, 21,  0, }, // India
    {  540,  920,       0,  689475,       0,  0, 23,  0, }, // Indian_Ocean
    {  540,  933,       0,  689498,       0,  0, 17,  0, }, // Indochina
    {  540,  943,       0,  689515,       0,  0, 27,  0, }, // Indonesia_Central
    {  540,  961,       0,  689542,       0,  0, 29,  0, }, // Indonesia_Eastern
    {  540,  979,       0,  689571,       0,  0, 29,  0, }, // Indonesia_Western
    {  540,  997,  689600,  689613,  689633, 13, 20, 20, }, // Iran
    {  540, 1008,  689653,  689668,  689690, 15, 22, 22, }, // Irkutsk
    {  540, 1016,  689712,  689727,  689749, 15, 22, 22, }, // Israel
    {  540, 1023,  689771,  689786,  689808, 15, 22, 22, }, // Japan
    {  540, 1047,       0,  689830,       0,  0, 18,  0, }, // Kazakhstan
    {  540, 1058,       0,  689848,       0,  0, 29,  0, }, // Kazakhstan_Eastern
    {  540, 1077,       0,  689877,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  540, 1106,  689906,  689918,  689937, 12, 19, 19, }, // Korea
    {  540, 1112,       0,  689956,       0,  0, 13,  0, }, // Kosrae
    {  540, 1119,  689969,  689987,  690012, 18, 25, 25, }, // Krasnoyarsk
    {  540, 1151,       0,  690037,       0,  0, 20,  0, }, // Kyrgystan
    {  540, 1175,       0,  690057,       0,  0, 19,  0, }, // Line_Islands
    {  540, 1188,  690076,  690092,  690115, 16, 23, 23, }, // Lord_Howe
    {  540, 1204,  690138,  690151,  690171, 13, 20, 20, }, // Magadan
    {  540, 1219,       0,  690191,       0,  0, 14,  0, }, // Malaysia
    {  540, 1228,       0,  690205,       0,  0, 14,  0, }, // Maldives
    {  540, 1237,       0,  690219,       0,  0, 15,  0, }, // Marquesas
    {  540, 1247,       0,  690234,       0,  0, 23,  0, }, // Marshall_Islands
    {  540, 1264,  690257,  690273,  690296, 16, 23, 23, }, // Mauritius
    {  540, 1274,       0,  690319,       0,  0, 13,  0, }, // Mawson
    {  540, 1281,  690332,  690355,  690385, 23, 30, 30, }, // Mexico_Pacific
    {  540, 1296,  690415,  690433,  690458, 18, 25, 25, }, // Mongolia
    {  540, 1305,  690483,  690495,  690514, 12, 19, 19, }, // Moscow
    {  540, 1312,       0,  690533,       0,  0, 14,  0, }, // Myanmar
    {  540, 1320,       0,  690547,       0,  0, 12,  0, }, // Nauru
    {  540, 1326,       0,  690559,       0,  0, 12,  0, }, // Nepal
    {  540, 1332,  690571,  690592,  690620, 21, 28, 28, }, // New_Caledonia
    {  540, 1346,  690648,  690668,  690695, 20, 27, 27, }, // New_Zealand
    {  540, 1358,  690722,  690738,  690761, 16, 23, 23, }, // Newfoundland
    {  540, 1371,       0,  690784,       0,  0, 11,  0, }, // Niue
    {  540, 1376,  690795,  690817,  690846, 22, 29, 29, }, // Norfolk
    {  540, 1384,  690875,  690901,  690934, 26, 33, 33, }, // Noronha
    {  540, 1406,  690967,  690985,  691010, 18, 25, 25, }, // Novosibirsk
    {  540, 1418,  691035,  691046,  691064, 11, 18, 18, }, // Omsk
    {  540, 1428,  691082,  691097,  691119, 15, 22, 22, }, // Pakistan
    {  540, 1437,       0,  691141,       0,  0, 12,  0, }, // Palau
    {  540, 1443,       0,  691153,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  540, 1460,  691177,  691192,  691214, 15, 22, 22, }, // Paraguay
    {  540, 1469,  691236,  691247,  691265, 11, 18, 18, }, // Peru
    {  540, 1474,  691283,  691299,  691322, 16, 23, 23, }, // Philippines
    {  540, 1486,       0,  691345,       0,  0, 22,  0, }, // Phoenix_Islands
    {  540, 1502,  691367,  691394,  691428, 27, 34, 34, }, // Pierre_Miquelon
    {  540, 1518,       0,  691462,       0,  0, 15,  0, }, // Pitcairn
    {  540, 1527,       0,  691477,       0,  0, 13,  0, }, // Ponape
    {  540, 1534,       0,  691490,       0,  0, 16,  0, }, // Pyongyang
    {  540, 1554,       0,  691506,       0,  0, 14,  0, }, // Reunion
    {  540, 1562,       0,  691520,       0,  0, 14,  0, }, // Rothera
    {  540, 1570,  691534,  691549,  691571, 15, 22, 22, }, // Sakhalin
    {  540, 1596,  691593,  691605,  691624, 12, 19, 19, }, // Samoa
    {  540, 1602,       0,  691643,       0,  0, 17,  0, }, // Seychelles
    {  540, 1624,       0,  691660,       0,  0, 16,  0, }, // Singapore
    {  540, 1634,       0,  691676,       0,  0, 23,  0, }, // Solomon
    {  540, 1642,       0,  691699,       0,  0, 26,  0, }, // South_Georgia
    {  540, 1656,       0,  691725,       0,  0, 15,  0, }, // Suriname
    {  540, 1676,       0,  691740,       0,  0, 12,  0, }, // Syowa
    {  540, 1682,       0,  691752,       0,  0, 13,  0, }, // Tahiti
    {  540, 1689,  691765,  691778,  691798, 13, 20, 20, }, // Taipei
    {  540, 1696,       0,  691818,       0,  0, 18,  0, }, // Tajikistan
    {  540, 1724,       0,  691836,       0,  0, 14,  0, }, // Tokelau
    {  540, 1732,  691850,  691862,  691881, 12, 19, 19, }, // Tonga
    {  540, 1738,       0,  691900,       0,  0, 12,  0, }, // Truk
    {  540, 1750,  691912,  691931,  691957, 19, 26, 26, }, // Turkmenistan
    {  540, 1763,       0,  691983,       0,  0, 13,  0, }, // Tuvalu
    {  540, 1777,  691996,  692012,  692035, 16, 23, 23, }, // Uruguay
    {  540, 1792,  692058,  692077,  692103, 19, 26, 26, }, // Uzbekistan
    {  540, 1803,  692129,  692143,  692164, 14, 21, 21, }, // Vanuatu
    {  540, 1811,       0,  692185,       0,  0, 17,  0, }, // Venezuela
    {  540, 1821,  692202,  692220,  692245, 18, 25, 25, }, // Vladivostok
    {  540, 1833,  692270,  692286,  692309, 16, 23, 23, }, // Volgograd
    {  540, 1843,       0,  692332,       0,  0, 13,  0, }, // Vostok
    {  540, 1850,       0,  692345,       0,  0, 19,  0, }, // Wake
    {  540, 1855,       0,  692364,       0,  0, 22,  0, }, // Wallis
    {  540, 1862,  692386,  692400,  692421, 14, 21, 21, }, // Yakutsk
    {  540, 1870,  692442,  692463,  692491, 21, 28, 28, }, // Yekaterinburg
    {  540, 1892,       0,  692519,       0,  0, 12,  0, }, // Yukon
    {  542,    5,       0,   13017,       0,  0, 10,  0, }, // Afghanistan Silesian/Latin/Poland
    {  542,   17,       0,  692531,       0,  0, 27,  0, }, // Africa_Central
    {  542,   32,       0,  692558,       0,  0, 24,  0, }, // Africa_Eastern
    {  542,   65,       0,  692582,       0,  0, 25,  0, }, // Africa_Southern
    {  542,   81,  692607,  692631,  692668, 24, 37, 31, }, // Africa_Western
    {  542,  107,  692699,  692714,  692742, 15, 28, 22, }, // Alaska
    {  542,  128,  603274,  692764,  692789, 12, 25, 19, }, // Almaty
    {  542,  135,  692808,  692822,  692849, 14, 27, 21, }, // Amazon
    {  542,  142,  692870,  692898,  692939, 28, 41, 35, }, // America_Central
    {  542,  158,  603478,  692974,  693012, 25, 38, 32, }, // America_Eastern
    {  542,  174,  693044,  693055,  693079, 11, 24, 18, }, // America_Mountain
    {  542,  191,  603622,  693097,  693125, 15, 28, 22, }, // America_Pacific
    {  542,  207,  603685,  693147,  693171, 11, 24, 18, }, // Anadyr
    {  542,  219,  603785,  693189,  693216, 14, 27, 21, }, // Aqtau
    {  542,  225,  603845,  693237,  693266, 16, 29, 23, }, // Aqtobe
    {  542,  232,  693289,  693305,  693341, 16, 36, 30, }, // Arabian
    {  542,  240,  693371,  693380,  693409,  9, 29, 23, }, // Argentina
    {  542,  250,  693432,  693451,  693490, 19, 39, 33, }, // Argentina_Western
    {  542,  268,  693523,  693531,  693559,  8, 28, 22, }, // Armenia
    {  542,  286,  604212,  693581,  693609, 15, 28, 22, }, // Atlantic
    {  542,  295,  693631,  693660,  693702, 29, 42, 36, }, // Australia_Central
    {  542,  313,  693738,  693777,  693829, 39, 52, 46, }, // Australia_CentralWestern
    {  542,  338,  693875,  693901,  693940, 26, 39, 33, }, // Australia_Eastern
    {  542,  356,  693751,  693790,  693842, 26, 39, 33, }, // Australia_Western
    {  542,  374,  604688,  693973,  694004, 11, 31, 25, }, // Azerbaijan
    {  542,  385,  604758,  694029,  694054,  5, 25, 19, }, // Azores
    {  542,  397,  604810,  694073,  694103, 10, 30, 24, }, // Bangladesh
    {  542,  422,       0,  694127,       0,  0,  8,  0, }, // Bolivia
    {  542,  437,  115203,  694135,  694163,  8, 28, 22, }, // Brasilia
    {  542,  461,  694185,  694211,  694257, 26, 46, 40, }, // Cape_Verde
    {  542,  478,       0,  605084,       0,  0,  8,  0, }, // Chamorro
    {  542,  495,  605150,  694297,  694322, 10, 25, 19, }, // Chile
    {  542,  501,  605207,  694341,  694366,  5, 25, 19, }, // China
    {  542,  507,       0,  694385,       0,  0, 12,  0, }, // Christmas
    {  542,  517,       0,  605287,       0,  0, 14,  0, }, // Cocos
    {  542,  523,  475228,  694397,  694426,  9, 29, 23, }, // Colombia
    {  542,  532,  605367,  694449,  694480, 11, 31, 25, }, // Cook
    {  542,  537,   14948,  694505,  694529,  4, 24, 18, }, // Cuba
    {  542,  564,       0,   14996,       0,  0, 16,  0, }, // DumontDUrville
    {  542,  601,       0,  605517,       0,  0, 14,  0, }, // East_Timor
    {  542,  612,  694547,  694564,  694601, 17, 37, 31, }, // Easter
    {  542,  619,       0,  448712,       0,  0,  7,  0, }, // Ecuador
    {  542,  627,  694632,  694659,  694699, 27, 40, 34, }, // Europe_Central
    {  542,  642,  694733,  694757,  694794, 24, 37, 31, }, // Europe_Eastern
    {  542,  657,       0,  694825,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  542,  680,  694856,  694880,  694917, 24, 37, 31, }, // Europe_Western
    {  542,  695,  605934,  694948,  694977,  9, 29, 23, }, // Falkland
    {  542,  704,  605998,  695000,  695025,  5, 25, 19, }, // Fiji
    {  542,  709,       0,  695044,       0,  0, 16,  0, }, // French_Guiana
    {  542,  723,       0,  695060,       0,  0, 45,  0, }, // French_Southern
    {  542,  746,       0,  606117,       0,  0, 14,  0, }, // Galapagos
    {  542,  756,       0,  606136,       0,  0, 14,  0, }, // Gambier
    {  542,  764,  695105,  695112,  695139,  7, 27, 21, }, // Georgia
    {  542,  772,       0,  695160,       0,  0, 16,  0, }, // Gilbert_Islands
    {  542,  788,       0,  695176,       0,  0, 19,  0, }, // GMT
    {  542,  830,  695195,  695216,  695257, 21, 41, 35, }, // Greenland_Eastern
    {  542,  848,  695292,  695313,  695354, 21, 41, 35, }, // Greenland_Western
    {  542,  871,       0,  606439,       0,  0, 13,  0, }, // Gulf
    {  542,  876,       0,  241449,       0,  0,  6,  0, }, // Guyana
    {  542,  883,  606457,  695389,  695422, 13, 33, 27, }, // Hawaii_Aleutian
    {  542,  899,  695449,  695457,  695485,  8, 28, 22, }, // Hong_Kong
    {  542,  909,  606594,  695507,  695532,  5, 25, 19, }, // Hovd
    {  542,  914,       0,  695551,       0,  0, 26,  0, }, // India
    {  542,  920,       0,  695577,       0,  0, 14,  0, }, // Indian_Ocean
    {  542,  933,       0,  695591,       0,  0, 16,  0, }, // Indochina
    {  542,  943,       0,  695607,       0,  0, 23,  0, }, // Indonesia_Central
    {  542,  961,       0,  695630,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  542,  979,       0,  695650,       0,  0, 20,  0, }, // Indonesia_Western
    {  542, 1008,  556269,  695670,  695696,  6, 26, 20, }, // Irkutsk
    {  542, 1016,   28134,  695716,  695742,  6, 26, 20, }, // Israel
    {  542, 1023,  695762,  695770,  695798,  8, 28, 22, }, // Japan
    {  542, 1029,  606989,  695820,  695862, 29, 42, 36, }, // Kamchatka
    {  542, 1058,       0,  607114,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  542, 1077,       0,  607138,       0,  0, 19,  0, }, // Kazakhstan_Western
    {  542, 1119,   16388,  695898,  695929, 11, 31, 25, }, // Krasnoyarsk
    {  542, 1151,       0,   16447,       0,  0,  9,  0, }, // Kyrgystan
    {  542, 1175,       0,    3816,       0,  0, 12,  0, }, // Line_Islands
    {  542, 1188,    3833,  695954,  695983,  9, 29, 23, }, // Lord_Howe
    {  542, 1219,       0,  696006,       0,  0,  8,  0, }, // Malaysia
    {  542, 1228,       0,  607460,       0,  0,  8,  0, }, // Maldives
    {  542, 1237,       0,  607473,       0,  0,  7,  0, }, // Marquesas
    {  542, 1247,       0,  607485,       0,  0, 15,  0, }, // Marshall_Islands
    {  542, 1281,  607575,  696014,  696051, 24, 37, 31, }, // Mexico_Pacific
    {  542, 1296,  607670,  696082,  696112, 10, 30, 24, }, // Mongolia
    {  542, 1305,  607737,  696136,  696157,  6, 21, 15, }, // Moscow
    {  542, 1312,       0,  350827,       0,  0,  6,  0, }, // Myanmar
    {  542, 1332,  696172,  696187,  696222, 15, 35, 29, }, // New_Caledonia
    {  542, 1346,  696251,  696265,  696299, 14, 34, 28, }, // New_Zealand
    {  542, 1358,  696327,  696343,  696379, 16, 36, 30, }, // Newfoundland
    {  542, 1384,    4633,  696409,  696448, 19, 39, 33, }, // Noronha
    {  542, 1406,  608221,  696481,  696512, 11, 31, 25, }, // Novosibirsk
    {  542, 1418,  696537,  696541,  696565,  4, 24, 18, }, // Omsk
    {  542, 1443,       0,  696583,       0,  0, 17,  0, }, // Papua_New_Guinea
    {  542, 1460,  608433,  696600,  696628,  8, 28, 22, }, // Paraguay
    {  542, 1469,  608489,  696650,  696674,  9, 24, 18, }, // Peru
    {  542, 1474,  608543,  696692,  696720,  8, 28, 22, }, // Philippines
    {  542, 1486,       0,  696742,       0,  0,  6,  0, }, // Phoenix_Islands
    {  542, 1502,  608615,  696748,  696791, 23, 43, 37, }, // Pierre_Miquelon
    {  542, 1527,       0,    5231,       0,  0,  7,  0, }, // Ponape
    {  542, 1534,       0,  696828,       0,  0,  9,  0, }, // Pyongyang
    {  542, 1544,  608755,  696837,  696866, 16, 29, 23, }, // Qyzylorda
    {  542, 1570,  212205,  696889,  696917,  8, 28, 22, }, // Sakhalin
    {  542, 1579,  608906,  696939,  696963, 11, 24, 18, }, // Samara
    {  542, 1602,       0,  609014,       0,  0,  7,  0, }, // Seychelles
    {  542, 1624,       0,   63714,       0,  0,  8,  0, }, // Singapore
    {  542, 1634,       0,  696981,       0,  0, 14,  0, }, // Solomon
    {  542, 1642,       0,  696995,       0,  0, 18,  0, }, // South_Georgia
    {  542, 1656,       0,    5596,       0,  0,  7,  0, }, // Suriname
    {  542, 1689,  104753,  697013,  697039,  6, 26, 20, }, // Taipei
    {  542, 1696,       0,  609169,       0,  0, 11,  0, }, // Tajikistan
    {  542, 1732,  697059,  697064,  697089,  5, 25, 19, }, // Tonga
    {  542, 1738,       0,    5754,       0,  0,  5,  0, }, // Truk
    {  542, 1750,  697108,  697120,  697152, 12, 32, 26, }, // Turkmenistan
    {  542, 1777,  609343,  697178,  697205,  7, 27, 21, }, // Uruguay
    {  542, 1811,       0,  697226,       0,  0,  9,  0, }, // Venezuela
    {  542, 1821,  609540,  697235,  697266, 11, 31, 25, }, // Vladivostok
    {  542, 1833,  609610,  697291,  697320,  9, 29, 23, }, // Volgograd
    {  542, 1843,       0,   18470,       0,  0,  6,  0, }, // Vostok
    {  542, 1855,       0,  144166,       0,  0, 15,  0, }, // Wallis
    {  542, 1862,  558608,  697343,  697369,  6, 26, 20, }, // Yakutsk
    {  542, 1870,  609769,  697389,  697422, 13, 33, 27, }, // Yekaterinburg
    {  542, 1892,       0,  697449,       0,  0, 13,  0, }, // Yukon
    {  543,    5,       0,  697462,       0,  0, 16,  0, }, // Afghanistan Sindhi/Arabic/Pakistan
    {  543,   17,       0,  697478,       0,  0, 19,  0, }, // Africa_Central
    {  543,   32,       0,  697497,       0,  0, 18,  0, }, // Africa_Eastern
    {  543,   65,       0,  697515,       0,  0, 24,  0, }, // Africa_Southern
    {  543,   81,  697539,  697558,  697584, 19, 26, 30, }, // Africa_Western
    {  543,  107,  697614,  697627,  697647, 13, 20, 22, }, // Alaska
    {  543,  135,  697669,  697683,  697704, 14, 21, 25, }, // Amazon
    {  543,  142,  697729,  697738,  697754,  9, 16, 18, }, // America_Central
    {  543,  158,  697772,  697781,  697797,  9, 16, 18, }, // America_Eastern
    {  543,  174,  697815,  697824,  697840,  9, 16, 18, }, // America_Mountain
    {  543,  191,  697858,  697868,  697885, 10, 17, 19, }, // America_Pacific
    {  543,  214,  697904,  697915,  697933, 11, 18, 20, }, // Apia
    {  543,  232,  697953,  697965,  697984, 12, 19, 21, }, // Arabian
    {  543,  240,  698005,  698020,  698039, 15, 19, 26, }, // Argentina
    {  543,  250,  698065,  698086,  698114, 21, 28, 32, }, // Argentina_Western
    {  543,  268,  698146,  698160,  698181, 14, 21, 25, }, // Armenia
    {  543,  286,  698206,  698221,  698243, 15, 22, 24, }, // Atlantic
    {  543,  295,  698267,  698288,  698316, 21, 28, 30, }, // Australia_Central
    {  543,  313,  698346,  698373,  698407, 27, 34, 36, }, // Australia_CentralWestern
    {  543,  338,  698443,  698463,  698491, 20, 28, 30, }, // Australia_Eastern
    {  543,  356,  698521,  698542,  698570, 21, 28, 30, }, // Australia_Western
    {  543,  374,  698600,  698617,  698641, 17, 24, 28, }, // Azerbaijan
    {  543,  385,  698669,  698681,  698700, 12, 19, 23, }, // Azores
    {  543,  397,  698723,  698738,  698760, 15, 22, 26, }, // Bangladesh
    {  543,  415,       0,  698786,       0,  0, 12,  0, }, // Bhutan
    {  543,  422,       0,  698798,       0,  0, 12,  0, }, // Bolivia
    {  543,  437,  698810,  698824,  698846, 14, 22, 26, }, // Brasilia
    {  543,  454,       0,  698872,       0,  0, 24,  0, }, // Brunei
    {  543,  461,  698896,  698910,  698931, 14, 21, 25, }, // Cape_Verde
    {  543,  478,       0,  698956,       0,  0, 19,  0, }, // Chamorro
    {  543,  487,  698975,  698986,  699004, 11, 18, 20, }, // Chatham
    {  543,  495,  699024,  699034,  699051, 10, 17, 21, }, // Chile
    {  543,  501,  699072,  699082,  699101, 10, 19, 21, }, // China
    {  543,  507,       0,  699122,       0,  0, 21,  0, }, // Christmas
    {  543,  517,       0,  699143,       0,  0, 21,  0, }, // Cocos
    {  543,  523,  699164,  699178,  699199, 14, 21, 25, }, // Colombia
    {  543,  532,  699224,  699243,  699269, 19, 26, 33, }, // Cook
    {  543,  537,  699302,  699314,  699333, 12, 19, 21, }, // Cuba
    {  543,  548,       0,  699354,       0,  0, 11,  0, }, // Davis
    {  543,  564,       0,  699365,       0,  0, 20,  0, }, // DumontDUrville
    {  543,  601,       0,  699385,       0,  0, 17,  0, }, // East_Timor
    {  543,  612,  699402,  699423,  699451, 21, 28, 32, }, // Easter
    {  543,  619,       0,  699483,       0,  0, 15,  0, }, // Ecuador
    {  543,  627,  699498,  699513,  699535, 15, 22, 26, }, // Europe_Central
    {  543,  642,  699561,  699576,  699598, 15, 22, 26, }, // Europe_Eastern
    {  543,  657,       0,  699624,       0,  0, 20,  0, }, // Europe_Further_Eastern
    {  543,  680,  699644,  699659,  699681, 15, 22, 24, }, // Europe_Western
    {  543,  695,  699705,  699729,  699760, 24, 31, 35, }, // Falkland
    {  543,  704,  699795,  699805,  699822, 10, 17, 21, }, // Fiji
    {  543,  709,       0,  699843,       0,  0, 21,  0, }, // French_Guiana
    {  543,  723,       0,  699864,       0,  0, 30,  0, }, // French_Southern
    {  543,  746,       0,  699894,       0,  0, 16,  0, }, // Galapagos
    {  543,  756,       0,  699910,       0,  0, 14,  0, }, // Gambier
    {  543,  764,  699924,  699937,  699957, 13, 20, 24, }, // Georgia
    {  543,  772,       0,  699981,       0,  0, 21,  0, }, // Gilbert_Islands
    {  543,  788,       0,  700002,       0,  0, 17,  0, }, // GMT
    {  543,  830,  700019,  700041,  700070, 22, 29, 33, }, // Greenland_Eastern
    {  543,  848,  700103,  700125,  700154, 22, 29, 33, }, // Greenland_Western
    {  543,  871,       0,  700187,       0,  0, 16,  0, }, // Gulf
    {  543,  876,       0,  700203,       0,  0, 11,  0, }, // Guyana
    {  543,  883,  700214,  700234,  700261, 20, 27, 29, }, // Hawaii_Aleutian
    {  543,  899,  700290,  700306,  700329, 16, 23, 27, }, // Hong_Kong
    {  543,  909,  700356,  700367,  700385, 11, 18, 22, }, // Hovd
    {  543,  914,       0,  700407,       0,  0, 18,  0, }, // India
    {  543,  920,       0,  700425,       0,  0, 15,  0, }, // Indian_Ocean
    {  543,  933,       0,  700440,       0,  0, 17,  0, }, // Indochina
    {  543,  943,       0,  700457,       0,  0, 22,  0, }, // Indonesia_Central
    {  543,  961,       0,  700479,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  543,  979,       0,  700500,       0,  0, 22,  0, }, // Indonesia_Western
    {  543,  997,  700522,  700534,  700553, 12, 19, 21, }, // Iran
    {  543, 1008,  700574,  700587,  700607, 13, 20, 22, }, // Irkutsk
    {  543, 1016,  700629,  700643,  700664, 14, 21, 23, }, // Israel
    {  543, 1023,  700687,  700699,  700718, 12, 19, 21, }, // Japan
    {  543, 1047,       0,  700739,       0,  0, 12,  0, }, // Kazakhstan
    {  543, 1058,       0,  700751,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  543, 1077,       0,  700771,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  543, 1106,  700792,  700804,  700823, 12, 19, 21, }, // Korea
    {  543, 1112,       0,  700844,       0,  0, 14,  0, }, // Kosrae
    {  543, 1119,  700858,  700876,  700901, 18, 25, 27, }, // Krasnoyarsk
    {  543, 1151,       0,  700928,       0,  0, 15,  0, }, // Kyrgystan
    {  543, 1175,       0,  700943,       0,  0, 20,  0, }, // Line_Islands
    {  543, 1188,  700963,  700979,  701002, 16, 23, 25, }, // Lord_Howe
    {  543, 1204,  701027,  701040,  701060, 13, 20, 22, }, // Magadan
    {  543, 1219,       0,  701082,       0,  0, 15,  0, }, // Malaysia
    {  543, 1228,       0,  701097,       0,  0, 13,  0, }, // Maldives
    {  543, 1237,       0,  701110,       0,  0, 14,  0, }, // Marquesas
    {  543, 1247,       0,  701124,       0,  0, 21,  0, }, // Marshall_Islands
    {  543, 1264,  701145,  701159,  701180, 14, 21, 25, }, // Mauritius
    {  543, 1274,       0,  701205,       0,  0, 13,  0, }, // Mawson
    {  543, 1281,  701218,  701236,  701264, 18, 28, 30, }, // Mexico_Pacific
    {  543, 1296,  701294,  701311,  701335, 17, 24, 28, }, // Mongolia
    {  543, 1305,  701363,  701375,  701394, 12, 19, 21, }, // Moscow
    {  543, 1312,       0,  701415,       0,  0, 14,  0, }, // Myanmar
    {  543, 1320,       0,  701429,       0,  0, 13,  0, }, // Nauru
    {  543, 1326,       0,  701442,       0,  0, 12,  0, }, // Nepal
    {  543, 1332,  701454,  701474,  701501, 20, 27, 31, }, // New_Caledonia
    {  543, 1346,  701532,  701548,  701571, 16, 23, 25, }, // New_Zealand
    {  543, 1358,  701596,  701618,  701647, 22, 29, 31, }, // Newfoundland
    {  543, 1371,       0,  701678,       0,  0, 12,  0, }, // Niue
    {  543, 1376,  701690,  701713,  701743, 23, 30, 32, }, // Norfolk
    {  543, 1384,  701775,  701800,  701832, 25, 32, 33, }, // Noronha
    {  543, 1406,  701865,  701883,  701908, 18, 25, 27, }, // Novosibirsk
    {  543, 1418,  701935,  701947,  701966, 12, 19, 21, }, // Omsk
    {  543, 1428,  701987,  702001,  702022, 14, 21, 25, }, // Pakistan
    {  543, 1437,       0,  702047,       0,  0, 12,  0, }, // Palau
    {  543, 1443,       0,  702059,       0,  0, 20,  0, }, // Papua_New_Guinea
    {  543, 1460,  702079,  702094,  702116, 15, 22, 26, }, // Paraguay
    {  543, 1469,  702142,  702153,  702171, 11, 18, 22, }, // Peru
    {  543, 1474,  702193,  702206,  702226, 13, 20, 24, }, // Philippines
    {  543, 1486,       0,  702250,       0,  0, 22,  0, }, // Phoenix_Islands
    {  543, 1502,  702272,  702297,  702329, 25, 32, 34, }, // Pierre_Miquelon
    {  543, 1518,       0,  702363,       0,  0, 13,  0, }, // Pitcairn
    {  543, 1527,       0,  702376,       0,  0, 13,  0, }, // Ponape
    {  543, 1534,       0,  702389,       0,  0, 17,  0, }, // Pyongyang
    {  543, 1554,       0,  702406,       0,  0, 15,  0, }, // Reunion
    {  543, 1562,       0,  702421,       0,  0, 13,  0, }, // Rothera
    {  543, 1570,  702434,  702447,  702467, 13, 20, 22, }, // Sakhalin
    {  543, 1596,  702489,  702501,  702520, 12, 19, 21, }, // Samoa
    {  543, 1602,       0,  702541,       0,  0, 13,  0, }, // Seychelles
    {  543, 1624,       0,  702554,       0,  0, 21,  0, }, // Singapore
    {  543, 1634,       0,  702575,       0,  0, 22,  0, }, // Solomon
    {  543, 1642,       0,  702597,       0,  0, 17,  0, }, // South_Georgia
    {  543, 1656,       0,  702614,       0,  0, 15,  0, }, // Suriname
    {  543, 1676,       0,  702629,       0,  0, 13,  0, }, // Syowa
    {  543, 1682,       0,  702642,       0,  0, 13,  0, }, // Tahiti
    {  543, 1689,  702655,  702667,  702686, 12, 19, 21, }, // Taipei
    {  543, 1696,       0,  702707,       0,  0, 15,  0, }, // Tajikistan
    {  543, 1724,       0,  702722,       0,  0, 15,  0, }, // Tokelau
    {  543, 1732,  702737,  702749,  702768, 12, 19, 23, }, // Tonga
    {  543, 1738,       0,  702791,       0,  0, 11,  0, }, // Truk
    {  543, 1750,  702802,  702819,  702843, 17, 24, 28, }, // Turkmenistan
    {  543, 1763,       0,  702871,       0,  0, 13,  0, }, // Tuvalu
    {  543, 1777,  702884,  702899,  702921, 15, 22, 26, }, // Uruguay
    {  543, 1792,  702947,  702962,  702984, 15, 22, 26, }, // Uzbekistan
    {  543, 1803,  703010,  703024,  703045, 14, 21, 23, }, // Vanuatu
    {  543, 1811,       0,  703068,       0,  0, 15,  0, }, // Venezuela
    {  543, 1821,  703083,  703100,  703124, 17, 24, 27, }, // Vladivostok
    {  543, 1833,  703151,  703167,  703190, 16, 23, 25, }, // Volgograd
    {  543, 1843,       0,  703215,       0,  0, 13,  0, }, // Vostok
    {  543, 1850,       0,  703228,       0,  0, 19,  0, }, // Wake
    {  543, 1855,       0,  703247,       0,  0, 19,  0, }, // Wallis
    {  543, 1862,  703266,  703279,  703299, 13, 20, 22, }, // Yakutsk
    {  543, 1870,  703321,  703338,  703362, 17, 24, 26, }, // Yekaterinburg
    {  543, 1892,       0,  703388,       0,  0, 11,  0, }, // Yukon
    {  544,  142,  703399,  703410,  703428, 11, 18, 20, }, // America_Central Sindhi/Devanagari/India
    {  544,  158,  703448,  703457,  703473,  9, 16, 18, }, // America_Eastern
    {  544,  174,  703491,  703502,  703520, 11, 18, 20, }, // America_Mountain
    {  544,  191,  703540,  703552,  703571, 12, 19, 21, }, // America_Pacific
    {  544,  286,  703592,  703605,  703625, 13, 20, 22, }, // Atlantic
    {  544,  627,  703647,  703665,  703691, 18, 26, 29, }, // Europe_Central
    {  544,  642,  703720,  703737,  703761, 17, 24, 27, }, // Europe_Eastern
    {  544,  680,  703788,  703807,  703833, 19, 26, 29, }, // Europe_Western
    {  544,  788,       0,  703862,       0,  0, 19,  0, }, // GMT
    {  545,    5,       0,  703881,       0,  0, 17,  0, }, // Afghanistan Sinhala/Sinhala/Sri Lanka
    {  545,   17,       0,  703898,       0,  0, 23,  0, }, // Africa_Central
    {  545,   32,       0,  703921,       0,  0, 25,  0, }, // Africa_Eastern
    {  545,   65,       0,  703946,       0,  0, 22,  0, }, // Africa_Southern
    {  545,   81,  703968,  703990,  704018, 22, 28, 30, }, // Africa_Western
    {  545,  107,  704048,  704060,  704078, 12, 18, 21, }, // Alaska
    {  545,  135,  704099,  704112,  704131, 13, 19, 21, }, // Amazon
    {  545,  142,  704152,  704179,  704212, 27, 33, 36, }, // America_Central
    {  545,  158,  704248,  704277,  704312, 29, 35, 38, }, // America_Eastern
    {  545,  174,  704350,  704376,  704408, 26, 32, 35, }, // America_Mountain
    {  545,  191,  704443,  704472,  704507, 29, 35, 38, }, // America_Pacific
    {  545,  214,  704545,  704556,  704573, 11, 17, 16, }, // Apia
    {  545,  232,  704589,  704600,  704617, 11, 17, 17, }, // Arabian
    {  545,  240,  704634,  704650,  704672, 16, 22, 24, }, // Argentina
    {  545,  250,  704696,  704718,  704746, 22, 28, 30, }, // Argentina_Western
    {  545,  268,  704776,  704791,  704812, 15, 21, 24, }, // Armenia
    {  545,  286,  704836,  704853,  704876, 17, 23, 26, }, // Atlantic
    {  545,  295,  704902,  704929,  704955, 27, 26, 33, }, // Australia_Central
    {  545,  313,  704988,  705021,  705060, 33, 39, 39, }, // Australia_CentralWestern
    {  545,  338,  705099,  705128,  705163, 29, 35, 35, }, // Australia_Eastern
    {  545,  356,  705198,  705224,  705067, 26, 32, 32, }, // Australia_Western
    {  545,  374,  705256,  705273,  705296, 17, 23, 26, }, // Azerbaijan
    {  545,  385,  705322,  705335,  705354, 13, 19, 22, }, // Azores
    {  545,  397,  705376,  705390,  705410, 14, 20, 22, }, // Bangladesh
    {  545,  415,       0,  705432,       0,  0, 11,  0, }, // Bhutan
    {  545,  422,       0,  705443,       0,  0, 14,  0, }, // Bolivia
    {  545,  437,  705457,  705470,  705489, 13, 19, 21, }, // Brasilia
    {  545,  454,       0,  705510,       0,  0, 23,  0, }, // Brunei
    {  545,  461,  705533,  705547,  705567, 14, 20, 22, }, // Cape_Verde
    {  545,  478,       0,  705589,       0,  0, 11,  0, }, // Chamorro
    {  545,  487,  705600,  705611,  705628, 11, 17, 16, }, // Chatham
    {  545,  495,  705644,  705654,  705670, 10, 16, 18, }, // Chile
    {  545,  501,  705688,  705697,  705712,  9, 15, 15, }, // China
    {  545,  507,       0,  705727,       0,  0, 22,  0, }, // Christmas
    {  545,  517,       0,  705749,       0,  0, 18,  0, }, // Cocos
    {  545,  523,  705767,  705783,  705805, 16, 22, 24, }, // Colombia
    {  545,  532,  705829,  705845,  705867, 16, 22, 29, }, // Cook
    {  545,  537,  705896,  705910,  705930, 14, 20, 23, }, // Cuba
    {  545,  548,       0,  705953,       0,  0, 12,  0, }, // Davis
    {  545,  564,       0,  705965,       0,  0, 24,  0, }, // DumontDUrville
    {  545,  601,       0,  705989,       0,  0, 21,  0, }, // East_Timor
    {  545,  612,  706010,  706028,  706052, 18, 24, 26, }, // Easter
    {  545,  619,       0,  706078,       0,  0, 14,  0, }, // Ecuador
    {  545,  627,  706092,  706112,  706138, 20, 26, 29, }, // Europe_Central
    {  545,  642,  706167,  706189,  706217, 22, 28, 31, }, // Europe_Eastern
    {  545,  657,       0,  706248,       0,  0, 27,  0, }, // Europe_Further_Eastern
    {  545,  680,  706275,  706294,  706319, 19, 25, 28, }, // Europe_Western
    {  545,  695,  706347,  706368,  706395, 21, 27, 29, }, // Falkland
    {  545,  704,  706424,  706434,  706450, 10, 16, 19, }, // Fiji
    {  545,  709,       0,  706469,       0,  0, 17,  0, }, // French_Guiana
    {  545,  723,       0,  706486,       0,  0, 38,  0, }, // French_Southern
    {  545,  746,       0,  706524,       0,  0, 13,  0, }, // Galapagos
    {  545,  756,       0,  706537,       0,  0, 15,  0, }, // Gambier
    {  545,  764,  706552,  706568,  706590, 16, 22, 25, }, // Georgia
    {  545,  772,       0,  706615,       0,  0, 21,  0, }, // Gilbert_Islands
    {  545,  788,       0,  706636,       0,  0, 22,  0, }, // GMT
    {  545,  830,  706658,  706684,  706716, 26, 32, 34, }, // Greenland_Eastern
    {  545,  848,  706750,  706773,  706802, 23, 29, 31, }, // Greenland_Western
    {  545,  871,       0,  706833,       0,  0, 11,  0, }, // Gulf
    {  545,  876,       0,  706476,       0,  0, 10,  0, }, // Guyana
    {  545,  883,  706844,  706867,  706896, 23, 29, 32, }, // Hawaii_Aleutian
    {  545,  899,  706928,  706940,  706958, 12, 18, 21, }, // Hong_Kong
    {  545,  909,  706979,  706991,  707009, 12, 18, 21, }, // Hovd
    {  545,  914,       0,  707030,       0,  0, 15,  0, }, // India
    {  545,  920,       0,  707045,       0,  0, 19,  0, }, // Indian_Ocean
    {  545,  933,       0,  707064,       0,  0, 14,  0, }, // Indochina
    {  545,  943,       0,  707078,       0,  0, 26,  0, }, // Indonesia_Central
    {  545,  961,       0,  707104,       0,  0, 28,  0, }, // Indonesia_Eastern
    {  545,  979,       0,  707132,       0,  0, 25,  0, }, // Indonesia_Western
    {  545,  997,  707157,  707167,  707183, 10, 16, 14, }, // Iran
    {  545, 1008,  707197,  707214,  707237, 17, 23, 26, }, // Irkutsk
    {  545, 1016,  707263,  707277,  707297, 14, 20, 20, }, // Israel
    {  545, 1023,  707317,  707327,  707343, 10, 16, 16, }, // Japan
    {  545, 1047,       0,  707359,       0,  0, 14,  0, }, // Kazakhstan
    {  545, 1058,       0,  707373,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  545, 1077,       0,  707396,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  545, 1106,  707416,  707430,  707450, 14, 20, 20, }, // Korea
    {  545, 1112,       0,  707470,       0,  0, 12,  0, }, // Kosrae
    {  545, 1119,  707482,  707504,  707532, 22, 28, 31, }, // Krasnoyarsk
    {  545, 1151,       0,  707563,       0,  0, 17,  0, }, // Kyrgystan
    {  545, 1161,       0,  707580,       0,  0, 16,  0, }, // Lanka
    {  545, 1175,       0,  707596,       0,  0, 17,  0, }, // Line_Islands
    {  545, 1188,  707613,  707630,  707653, 17, 23, 22, }, // Lord_Howe
    {  545, 1204,  707675,  707687,  707705, 12, 18, 21, }, // Magadan
    {  545, 1219,       0,  707726,       0,  0, 16,  0, }, // Malaysia
    {  545, 1228,       0,  707742,       0,  0, 16,  0, }, // Maldives
    {  545, 1237,       0,  707758,       0,  0, 17,  0, }, // Marquesas
    {  545, 1247,       0,  707775,       0,  0, 19,  0, }, // Marshall_Islands
    {  545, 1264,  707794,  707806,  707824, 12, 18, 20, }, // Mauritius
    {  545, 1274,       0,  707844,       0,  0, 13,  0, }, // Mawson
    {  545, 1281,  707857,  707880,  707909, 23, 29, 32, }, // Mexico_Pacific
    {  545, 1296,  707941,  707958,  707981, 17, 23, 26, }, // Mongolia
    {  545, 1305,  708007,  708020,  708039, 13, 19, 22, }, // Moscow
    {  545, 1312,       0,  708061,       0,  0, 15,  0, }, // Myanmar
    {  545, 1320,       0,  708076,       0,  0, 12,  0, }, // Nauru
    {  545, 1326,       0,  708088,       0,  0, 11,  0, }, // Nepal
    {  545, 1332,  708099,  708119,  708145, 20, 26, 29, }, // New_Caledonia
    {  545, 1346,  708174,  708188,  708208, 14, 20, 19, }, // New_Zealand
    {  545, 1358,  708227,  708246,  708271, 19, 25, 28, }, // Newfoundland
    {  545, 1371,       0,  708299,       0,  0, 10,  0, }, // Niue
    {  545, 1376,  708309,  708329,  708355, 20, 26, 29, }, // Norfolk
    {  545, 1384,  708384,  708411,  708444, 27, 33, 35, }, // Noronha
    {  545, 1406,  708479,  708498,  708523, 19, 25, 28, }, // Novosibirsk
    {  545, 1418,  708551,  708564,  708583, 13, 19, 22, }, // Omsk
    {  545, 1428,  708605,  708620,  708641, 15, 21, 23, }, // Pakistan
    {  545, 1437,       0,  708664,       0,  0, 11,  0, }, // Palau
    {  545, 1443,       0,  708675,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  545, 1460,  708699,  708712,  708731, 13, 19, 21, }, // Paraguay
    {  545, 1469,  708752,  708762,  708778, 10, 16, 18, }, // Peru
    {  545, 1474,  708796,  708809,  708828, 13, 19, 22, }, // Philippines
    {  545, 1486,       0,  708850,       0,  0, 20,  0, }, // Phoenix_Islands
    {  545, 1502,  708870,  708898,  708932, 28, 34, 37, }, // Pierre_Miquelon
    {  545, 1518,       0,  708969,       0,  0, 16,  0, }, // Pitcairn
    {  545, 1527,       0,  708985,       0,  0, 11,  0, }, // Ponape
    {  545, 1534,       0,  708996,       0,  0, 17,  0, }, // Pyongyang
    {  545, 1554,       0,  709013,       0,  0, 15,  0, }, // Reunion
    {  545, 1562,       0,  709028,       0,  0, 12,  0, }, // Rothera
    {  545, 1570,  709040,  709052,  709070, 12, 18, 21, }, // Sakhalin
    {  545, 1596,  709091,  709103,  709121, 12, 18, 21, }, // Samoa
    {  545, 1602,       0,  709142,       0,  0, 14,  0, }, // Seychelles
    {  545, 1624,       0,  709156,       0,  0, 16,  0, }, // Singapore
    {  545, 1634,       0,  709172,       0,  0, 19,  0, }, // Solomon
    {  545, 1642,       0,  709191,       0,  0, 18,  0, }, // South_Georgia
    {  545, 1656,       0,  709209,       0,  0, 13,  0, }, // Suriname
    {  545, 1676,       0,  709222,       0,  0, 12,  0, }, // Syowa
    {  545, 1682,       0,  709234,       0,  0, 12,  0, }, // Tahiti
    {  545, 1689,  709246,  709258,  709276, 12, 18, 18, }, // Taipei
    {  545, 1696,       0,  709294,       0,  0, 16,  0, }, // Tajikistan
    {  545, 1724,       0,  709310,       0,  0, 14,  0, }, // Tokelau
    {  545, 1732,  709324,  709335,  709352, 11, 17, 20, }, // Tonga
    {  545, 1738,       0,  709372,       0,  0, 10,  0, }, // Truk
    {  545, 1750,  709382,  709402,  709428, 20, 26, 29, }, // Turkmenistan
    {  545, 1763,       0,  709457,       0,  0, 12,  0, }, // Tuvalu
    {  545, 1777,  709469,  709482,  709501, 13, 19, 21, }, // Uruguay
    {  545, 1792,  709522,  709540,  709564, 18, 24, 27, }, // Uzbekistan
    {  545, 1803,  709591,  709603,  709621, 12, 18, 20, }, // Vanuatu
    {  545, 1811,       0,  709641,       0,  0, 16,  0, }, // Venezuela
    {  545, 1821,  709657,  709676,  709701, 19, 25, 28, }, // Vladivostok
    {  545, 1833,  709729,  709748,  709773, 19, 25, 28, }, // Volgograd
    {  545, 1843,       0,  709801,       0,  0, 14,  0, }, // Vostok
    {  545, 1850,       0,  709815,       0,  0, 16,  0, }, // Wake
    {  545, 1855,       0,  709831,       0,  0, 22,  0, }, // Wallis
    {  545, 1862,  709853,  709868,  709889, 15, 21, 24, }, // Yakutsk
    {  545, 1870,  709913,  709933,  709959, 20, 26, 29, }, // Yekaterinburg
    {  545, 1892,       0,  709988,       0,  0, 12,  0, }, // Yukon
    {  547,    0,  193736,  710000,  710023, 12, 23, 18, }, // Acre Slovak/Latin/Slovakia
    {  547,    5,       0,  710041,       0,  0, 12,  0, }, // Afghanistan
    {  547,   17,       0,  710053,       0,  0, 17,  0, }, // Africa_Central
    {  547,   32,       0,  193819,       0,  0, 18,  0, }, // Africa_Eastern
    {  547,   65,       0,  710070,       0,  0, 15,  0, }, // Africa_Southern
    {  547,   81,  193852,  710085,  710113, 17, 28, 23, }, // Africa_Western
    {  547,  107,  193920,  710136,  710159, 12, 23, 18, }, // Alaska
    {  547,  128,  710177,  710190,  710214, 13, 24, 19, }, // Almaty
    {  547,  135,  194026,  710233,  710257, 13, 24, 19, }, // Amazon
    {  547,  142,  710276,  710304,  710343, 28, 39, 34, }, // America_Central
    {  547,  158,  710377,  710404,  710442, 27, 38, 33, }, // America_Eastern
    {  547,  174,  194281,  710475,  710511, 25, 36, 31, }, // America_Mountain
    {  547,  191,  710542,  710572,  710613, 30, 41, 36, }, // America_Pacific
    {  547,  207,  710649,  710662,  710686, 13, 24, 19, }, // Anadyr
    {  547,  214,  194530,  710705,  710727, 11, 22, 17, }, // Apia
    {  547,  219,  710744,  710756,  710779, 12, 23, 18, }, // Aqtau
    {  547,  225,  710797,  710809,  710832, 12, 23, 18, }, // Aqtobe
    {  547,  232,  194686,  710850,  710872, 11, 22, 17, }, // Arabian
    {  547,  240,  710889,  710904,  710930, 15, 26, 21, }, // Argentina
    {  547,  250,  710951,  710972,  711004, 21, 32, 27, }, // Argentina_Western
    {  547,  268,  711031,  711043,  711066, 12, 23, 18, }, // Armenia
    {  547,  286,  194931,  711084,  711109, 14, 25, 20, }, // Atlantic
    {  547,  295,  711129,  711149,  711180, 20, 31, 26, }, // Australia_Central
    {  547,  313,  711206,  711234,  711273, 28, 39, 34, }, // Australia_CentralWestern
    {  547,  338,  711307,  711328,  711360, 21, 32, 27, }, // Australia_Eastern
    {  547,  356,  711387,  711407,  711438, 20, 31, 26, }, // Australia_Western
    {  547,  374,  711464,  711482,  711511, 18, 29, 24, }, // Azerbaijan
    {  547,  385,  195396,  711535,  711557, 11, 22, 17, }, // Azores
    {  547,  397,  711574,  711590,  711617, 16, 27, 22, }, // Bangladesh
    {  547,  415,       0,  711639,       0,  0, 13,  0, }, // Bhutan
    {  547,  422,       0,  711652,       0,  0, 14,  0, }, // Bolivia
    {  547,  437,  711666,  711679,  711703, 13, 24, 19, }, // Brasilia
    {  547,  454,       0,  195600,       0,  0, 13,  0, }, // Brunei
    {  547,  461,  195613,  711722,  711747, 14, 25, 20, }, // Cape_Verde
    {  547,  472,       0,  195672,       0,  0, 19,  0, }, // Casey
    {  547,  478,       0,  195691,       0,  0, 14,  0, }, // Chamorro
    {  547,  487,  195705,  711767,  711792, 14, 25, 20, }, // Chatham
    {  547,  495,  711812,  711822,  711843, 10, 21, 16, }, // Chile
    {  547,  501,  711859,  711869,  711890, 10, 21, 16, }, // China
    {  547,  507,       0,  711906,       0,  0, 22,  0, }, // Christmas
    {  547,  517,       0,  711928,       0,  0, 23,  0, }, // Cocos
    {  547,  523,  195904,  711951,  711977, 15, 26, 21, }, // Colombia
    {  547,  532,  711998,  712020,  712053, 22, 33, 28, }, // Cook
    {  547,  537,  712081,  712093,  712116, 12, 23, 18, }, // Cuba
    {  547,  548,       0,  712134,       0,  0, 21,  0, }, // Davis
    {  547,  564,       0,  712155,       0,  0, 30,  0, }, // DumontDUrville
    {  547,  601,       0,  196148,       0,  0, 19,  0, }, // East_Timor
    {  547,  612,  712185,  712209,  712244, 24, 35, 30, }, // Easter
    {  547,  619,       0,  196259,       0,  0, 14,  0, }, // Ecuador
    {  547,  627,  712274,  712292,  712321, 18, 29, 24, }, // Europe_Central
    {  547,  642,  712345,  712364,  712394, 19, 30, 25, }, // Europe_Eastern
    {  547,  657,       0,  712419,       0,  0, 10,  0, }, // Europe_Further_Eastern
    {  547,  680,  712429,  712447,  712476, 18, 29, 24, }, // Europe_Western
    {  547,  695,  196513,  712500,  712526, 15, 26, 21, }, // Falkland
    {  547,  704,  196575,  712547,  712571, 13, 24, 19, }, // Fiji
    {  547,  709,       0,  712590,       0,  0, 22,  0, }, // French_Guiana
    {  547,  723,       0,  712612,       0,  0, 46,  0, }, // French_Southern
    {  547,  746,       0,  712658,       0,  0, 14,  0, }, // Galapagos
    {  547,  756,       0,  196715,       0,  0, 14,  0, }, // Gambier
    {  547,  764,  712672,  712685,  712709, 13, 24, 19, }, // Georgia
    {  547,  772,       0,  712728,       0,  0, 25,  0, }, // Gilbert_Islands
    {  547,  788,       0,  712753,       0,  0, 16,  0, }, // GMT
    {  547,  830,  712769,  712787,  712816, 18, 29, 24, }, // Greenland_Eastern
    {  547,  848,  712840,  712857,  712885, 17, 28, 23, }, // Greenland_Western
    {  547,  866,       0,  712908,       0,  0, 11,  0, }, // Guam
    {  547,  871,       0,  712919,       0,  0, 31,  0, }, // Gulf
    {  547,  876,       0,  196642,       0,  0, 12,  0, }, // Guyana
    {  547,  883,  197013,  712950,  712982, 21, 32, 27, }, // Hawaii_Aleutian
    {  547,  899,  713009,  713023,  713048, 14, 25, 20, }, // Hong_Kong
    {  547,  909,  713068,  713080,  713103, 12, 23, 18, }, // Hovd
    {  547,  914,       0,  197205,       0,  0, 11,  0, }, // India
    {  547,  920,       0,  713121,       0,  0, 19,  0, }, // Indian_Ocean
    {  547,  933,       0,  713140,       0,  0, 14,  0, }, // Indochina
    {  547,  943,       0,  713154,       0,  0, 20,  0, }, // Indonesia_Central
    {  547,  961,       0,  713174,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  547,  979,       0,  713195,       0,  0, 20,  0, }, // Indonesia_Western
    {  547,  997,  713215,  713226,  713248, 11, 22, 17, }, // Iran
    {  547, 1008,  197357,  713265,  713288, 12, 23, 18, }, // Irkutsk
    {  547, 1016,  197410,  713306,  713330, 13, 24, 19, }, // Israel
    {  547, 1023,  197466,  713349,  713372, 12, 23, 18, }, // Japan
    {  547, 1029,  713390,  713417,  713455, 27, 38, 37, }, // Kamchatka
    {  547, 1047,       0,  713492,       0,  0, 17,  0, }, // Kazakhstan
    {  547, 1058,       0,  713509,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  547, 1077,       0,  713533,       0,  0, 23,  0, }, // Kazakhstan_Western
    {  547, 1106,  713556,  713568,  713591, 12, 23, 18, }, // Korea
    {  547, 1112,       0,  197737,       0,  0, 13,  0, }, // Kosrae
    {  547, 1119,  197750,  713609,  713636, 16, 27, 22, }, // Krasnoyarsk
    {  547, 1151,       0,  713658,       0,  0, 13,  0, }, // Kyrgystan
    {  547, 1161,       0,  713671,       0,  0, 13,  0, }, // Lanka
    {  547, 1175,       0,  713684,       0,  0, 24,  0, }, // Line_Islands
    {  547, 1188,  197864,  713708,  713741, 22, 33, 28, }, // Lord_Howe
    {  547, 1198,  713769,  713781,  713804, 12, 23, 18, }, // Macau
    {  547, 1204,  198000,  713822,  713847, 14, 25, 20, }, // Magadan
    {  547, 1219,       0,  713867,       0,  0, 15,  0, }, // Malaysia
    {  547, 1228,       0,  713882,       0,  0, 13,  0, }, // Maldives
    {  547, 1237,       0,  713895,       0,  0, 13,  0, }, // Marquesas
    {  547, 1247,       0,  713908,       0,  0, 26,  0, }, // Marshall_Islands
    {  547, 1264,  713934,  713949,  713975, 15, 26, 21, }, // Mauritius
    {  547, 1274,       0,  713996,       0,  0, 22,  0, }, // Mawson
    {  547, 1281,  714018,  714041,  714075, 23, 34, 29, }, // Mexico_Pacific
    {  547, 1296,  714104,  714120,  714147, 16, 27, 22, }, // Mongolia
    {  547, 1305,  714169,  714182,  714206, 13, 24, 19, }, // Moscow
    {  547, 1312,       0,  714225,       0,  0, 14,  0, }, // Myanmar
    {  547, 1320,       0,  714239,       0,  0, 12,  0, }, // Nauru
    {  547, 1326,       0,  714251,       0,  0, 12,  0, }, // Nepal
    {  547, 1332,  714263,  714281,  714310, 18, 29, 24, }, // New_Caledonia
    {  547, 1346,  198515,  714334,  714362, 17, 28, 23, }, // New_Zealand
    {  547, 1358,  198583,  714385,  714415, 19, 30, 25, }, // Newfoundland
    {  547, 1371,       0,  198657,       0,  0, 12,  0, }, // Niue
    {  547, 1376,  714440,  714453,  714477, 13, 24, 19, }, // Norfolk
    {  547, 1384,  714496,  714530,  714575, 34, 45, 40, }, // Noronha
    {  547, 1392,       0,  714615,       0,  0, 19,  0, }, // North_Mariana
    {  547, 1406,  198866,  714634,  714661, 16, 27, 22, }, // Novosibirsk
    {  547, 1418,  198931,  714683,  714703,  9, 20, 15, }, // Omsk
    {  547, 1428,  714718,  714733,  714759, 15, 26, 21, }, // Pakistan
    {  547, 1437,       0,  199037,       0,  0, 12,  0, }, // Palau
    {  547, 1443,       0,  714780,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  547, 1460,  714802,  714817,  714843, 15, 26, 21, }, // Paraguay
    {  547, 1469,  714864,  714877,  714901, 13, 24, 19, }, // Peru
    {  547, 1474,  714920,  714934,  714959, 14, 25, 20, }, // Philippines
    {  547, 1486,       0,  714979,       0,  0, 23,  0, }, // Phoenix_Islands
    {  547, 1502,  199269,  715002,  715035, 22, 33, 28, }, // Pierre_Miquelon
    {  547, 1518,       0,  715063,       0,  0, 26,  0, }, // Pitcairn
    {  547, 1527,       0,  199377,       0,  0, 12,  0, }, // Ponape
    {  547, 1534,       0,  715089,       0,  0, 17,  0, }, // Pyongyang
    {  547, 1544,  715106,  715121,  715147, 15, 26, 21, }, // Qyzylorda
    {  547, 1554,       0,  199469,       0,  0, 14,  0, }, // Reunion
    {  547, 1562,       0,  715168,       0,  0, 22,  0, }, // Rothera
    {  547, 1570,  199504,  715190,  715216, 15, 26, 21, }, // Sakhalin
    {  547, 1579,  715237,  715249,  715272, 12, 23, 18, }, // Samara
    {  547, 1596,  199619,  715290,  715313, 12, 23, 18, }, // Samoa
    {  547, 1602,       0,  199672,       0,  0, 14,  0, }, // Seychelles
    {  547, 1624,       0,  715331,       0,  0, 26,  0, }, // Singapore
    {  547, 1634,       0,  715357,       0,  0, 25,  0, }, // Solomon
    {  547, 1642,       0,  715382,       0,  0, 18,  0, }, // South_Georgia
    {  547, 1656,       0,  199743,       0,  0, 14,  0, }, // Suriname
    {  547, 1676,       0,  199757,       0,  0, 16,  0, }, // Syowa
    {  547, 1682,       0,  199773,       0,  0, 12,  0, }, // Tahiti
    {  547, 1689,  199785,  715400,  715426, 15, 26, 21, }, // Taipei
    {  547, 1696,       0,  715447,       0,  0, 12,  0, }, // Tajikistan
    {  547, 1724,       0,  199859,       0,  0, 14,  0, }, // Tokelau
    {  547, 1732,  199873,  715459,  715481, 11, 22, 17, }, // Tonga
    {  547, 1738,       0,  199923,       0,  0, 12,  0, }, // Truk
    {  547, 1750,  715498,  715512,  715537, 14, 25, 20, }, // Turkmenistan
    {  547, 1763,       0,  199994,       0,  0, 12,  0, }, // Tuvalu
    {  547, 1777,  715557,  715571,  715596, 14, 25, 20, }, // Uruguay
    {  547, 1792,  200065,  715616,  715638, 11, 22, 17, }, // Uzbekistan
    {  547, 1803,  200115,  715655,  715679, 13, 24, 19, }, // Vanuatu
    {  547, 1811,       0,  200171,       0,  0, 15,  0, }, // Venezuela
    {  547, 1821,  200186,  715698,  715726, 17, 28, 23, }, // Vladivostok
    {  547, 1833,  200254,  715749,  715776, 16, 27, 22, }, // Volgograd
    {  547, 1843,       0,  200319,       0,  0, 18,  0, }, // Vostok
    {  547, 1850,       0,  200337,       0,  0, 16,  0, }, // Wake
    {  547, 1855,       0,  715798,       0,  0, 28,  0, }, // Wallis
    {  547, 1862,  200380,  715826,  715849, 12, 23, 18, }, // Yakutsk
    {  547, 1870,  715867,  715886,  715916, 19, 30, 25, }, // Yekaterinburg
    {  547, 1892,       0,  200507,       0,  0, 12,  0, }, // Yukon
    {  548,    5,       0,  715941,       0,  0, 17,  0, }, // Afghanistan Slovenian/Latin/Slovenia
    {  548,   17,       0,  715958,       0,  0, 20,  0, }, // Africa_Central
    {  548,   32,       0,  715978,       0,  0, 18,  0, }, // Africa_Eastern
    {  548,   65,       0,  715996,       0,  0, 16,  0, }, // Africa_Southern
    {  548,   81,  716012,  716030,  716059, 18, 29, 26, }, // Africa_Western
    {  548,  107,  716085,  716096,  716118, 11, 22, 19, }, // Alaska
    {  548,  135,  716137,  716150,  716174, 13, 24, 21, }, // Amazon
    {  548,  142,  716195,  716208,  716232, 13, 24, 21, }, // America_Central
    {  548,  158,  716253,  716264,  716286, 11, 22, 19, }, // America_Eastern
    {  548,  174,  716305,  716315,  716336, 10, 21, 18, }, // America_Mountain
    {  548,  191,  716354,  716367,  716391, 13, 24, 21, }, // America_Pacific
    {  548,  207,  716412,  716425,  716449, 13, 24, 21, }, // Anadyr
    {  548,  214,  716470,  716479,  716499,  9, 20, 17, }, // Apia
    {  548,  232,  716516,  716527,  716549, 11, 22, 19, }, // Arabian
    {  548,  240,  716568,  716583,  716609, 15, 26, 23, }, // Argentina
    {  548,  250,  716632,  716655,  716689, 23, 34, 31, }, // Argentina_Western
    {  548,  268,  716720,  716732,  716755, 12, 23, 20, }, // Armenia
    {  548,  286,  716775,  716788,  716812, 13, 24, 21, }, // Atlantic
    {  548,  295,  716833,  716857,  716892, 24, 35, 32, }, // Australia_Central
    {  548,  313,  716924,  716956,  716999, 32, 43, 40, }, // Australia_CentralWestern
    {  548,  338,  717039,  717061,  717094, 22, 33, 30, }, // Australia_Eastern
    {  548,  356,  717124,  717146,  717179, 22, 33, 30, }, // Australia_Western
    {  548,  374,  717209,  717227,  717256, 18, 29, 26, }, // Azerbaijan
    {  548,  385,  717282,  717293,  717315, 11, 22, 19, }, // Azores
    {  548,  397,  717334,  717349,  717375, 15, 26, 23, }, // Bangladesh
    {  548,  415,       0,  717398,       0,  0, 12,  0, }, // Bhutan
    {  548,  422,       0,  717410,       0,  0, 14,  0, }, // Bolivia
    {  548,  437,  717424,  717437,  717461, 13, 24, 21, }, // Brasilia
    {  548,  454,       0,  717482,       0,  0, 13,  0, }, // Brunei
    {  548,  461,  717495,  717509,  717534, 14, 25, 22, }, // Cape_Verde
    {  548,  478,       0,  717556,       0,  0, 23,  0, }, // Chamorro
    {  548,  487,  717579,  717591,  717614, 12, 23, 20, }, // Chatham
    {  548,  495,  717634,  717644,  717665, 10, 21, 18, }, // Chile
    {  548,  501,  717683,  717695,  717718, 12, 23, 20, }, // China
    {  548,  507,       0,  717738,       0,  0, 17,  0, }, // Christmas
    {  548,  517,       0,  717755,       0,  0, 19,  0, }, // Cocos
    {  548,  523,  717774,  717789,  717815, 15, 26, 23, }, // Colombia
    {  548,  532,  717838,  717855,  717883, 17, 28, 32, }, // Cook
    {  548,  537,  717915,  717927,  717950, 12, 23, 20, }, // Cuba
    {  548,  548,       0,  717970,       0,  0, 10,  0, }, // Davis
    {  548,  564,       0,  717980,       0,  0, 21,  0, }, // DumontDUrville
    {  548,  601,       0,  718001,       0,  0, 19,  0, }, // East_Timor
    {  548,  612,  718020,  718041,  718073, 21, 32, 29, }, // Easter
    {  548,  619,       0,  718102,       0,  0, 14,  0, }, // Ecuador
    {  548,  627,  718116,  718135,  718165, 19, 30, 27, }, // Europe_Central
    {  548,  642,  718192,  718211,  718241, 19, 30, 27, }, // Europe_Eastern
    {  548,  657,       0,  718268,       0,  0, 27,  0, }, // Europe_Further_Eastern
    {  548,  680,  718295,  718314,  718344, 19, 30, 27, }, // Europe_Western
    {  548,  695,  718371,  718394,  718428, 23, 34, 31, }, // Falkland
    {  548,  704,  718459,  718472,  718496, 13, 24, 21, }, // Fiji
    {  548,  709,       0,  718517,       0,  0, 22,  0, }, // French_Guiana
    {  548,  723,       0,  718539,       0,  0, 34,  0, }, // French_Southern
    {  548,  746,       0,  718573,       0,  0, 13,  0, }, // Galapagos
    {  548,  756,       0,  718586,       0,  0, 14,  0, }, // Gambier
    {  548,  764,  718600,  718613,  718637, 13, 24, 21, }, // Georgia
    {  548,  772,       0,  718658,       0,  0, 21,  0, }, // Gilbert_Islands
    {  548,  788,       0,  718679,       0,  0, 22,  0, }, // GMT
    {  548,  830,  718701,  718723,  718756, 22, 33, 30, }, // Greenland_Eastern
    {  548,  848,  718786,  718808,  718841, 22, 33, 30, }, // Greenland_Western
    {  548,  871,       0,  718871,       0,  0, 23,  0, }, // Gulf
    {  548,  876,       0,  718894,       0,  0, 13,  0, }, // Guyana
    {  548,  883,  718907,  718928,  718960, 21, 32, 29, }, // Hawaii_Aleutian
    {  548,  899,  718989,  719003,  719028, 14, 25, 22, }, // Hong_Kong
    {  548,  909,  719050,  719061,  719083, 11, 22, 19, }, // Hovd
    {  548,  914,       0,  719102,       0,  0, 23,  0, }, // India
    {  548,  920,       0,  719125,       0,  0, 20,  0, }, // Indian_Ocean
    {  548,  933,       0,  719145,       0,  0, 16,  0, }, // Indochina
    {  548,  943,       0,  719161,       0,  0, 25,  0, }, // Indonesia_Central
    {  548,  961,       0,  719186,       0,  0, 24,  0, }, // Indonesia_Eastern
    {  548,  979,       0,  719210,       0,  0, 24,  0, }, // Indonesia_Western
    {  548,  997,  719234,  719245,  719267, 11, 22, 19, }, // Iran
    {  548, 1008,  719286,  719298,  719321, 12, 23, 20, }, // Irkutsk
    {  548, 1016,  719341,  719354,  719378, 13, 24, 21, }, // Israel
    {  548, 1023,  719399,  719411,  719434, 12, 23, 20, }, // Japan
    {  548, 1029,  719454,  719481,  719519, 27, 38, 35, }, // Kamchatka
    {  548, 1047,       0,  719554,       0,  0, 16,  0, }, // Kazakhstan
    {  548, 1058,       0,  719570,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  548, 1077,       0,  719594,       0,  0, 24,  0, }, // Kazakhstan_Western
    {  548, 1106,  719618,  719630,  719653, 12, 23, 20, }, // Korea
    {  548, 1112,       0,  719673,       0,  0, 13,  0, }, // Kosrae
    {  548, 1119,  719686,  719702,  719729, 16, 27, 24, }, // Krasnoyarsk
    {  548, 1151,       0,  719753,       0,  0, 18,  0, }, // Kyrgystan
    {  548, 1175,       0,  719771,       0,  0, 21,  0, }, // Line_Islands
    {  548, 1188,  719792,  719811,  719841, 19, 30, 27, }, // Lord_Howe
    {  548, 1204,  719868,  719882,  719907, 14, 25, 22, }, // Magadan
    {  548, 1219,       0,  719929,       0,  0, 14,  0, }, // Malaysia
    {  548, 1228,       0,  719943,       0,  0, 13,  0, }, // Maldives
    {  548, 1237,       0,  719956,       0,  0, 19,  0, }, // Marquesas
    {  548, 1247,       0,  719975,       0,  0, 22,  0, }, // Marshall_Islands
    {  548, 1264,  719997,  720012,  720038, 15, 26, 23, }, // Mauritius
    {  548, 1274,       0,  720061,       0,  0, 13,  0, }, // Mawson
    {  548, 1281,  720074,  720095,  720127, 21, 32, 29, }, // Mexico_Pacific
    {  548, 1296,  720156,  720172,  720199, 16, 27, 24, }, // Mongolia
    {  548, 1305,  720223,  720236,  720260, 13, 24, 21, }, // Moscow
    {  548, 1312,       0,  720281,       0,  0, 14,  0, }, // Myanmar
    {  548, 1320,       0,  720295,       0,  0, 13,  0, }, // Nauru
    {  548, 1326,       0,  720308,       0,  0, 12,  0, }, // Nepal
    {  548, 1332,  720320,  720340,  720371, 20, 31, 28, }, // New_Caledonia
    {  548, 1346,  720399,  720416,  720444, 17, 28, 25, }, // New_Zealand
    {  548, 1358,  720469,  720488,  720518, 19, 30, 27, }, // Newfoundland
    {  548, 1371,       0,  720545,       0,  0, 12,  0, }, // Niue
    {  548, 1376,  720557,  720577,  720608, 20, 31, 28, }, // Norfolk
    {  548, 1384,  720636,  720660,  720695, 24, 35, 32, }, // Noronha
    {  548, 1406,  720727,  720743,  720770, 16, 27, 24, }, // Novosibirsk
    {  548, 1418,  720794,  720803,  720823,  9, 20, 17, }, // Omsk
    {  548, 1428,  720840,  720855,  720881, 15, 26, 23, }, // Pakistan
    {  548, 1437,       0,  720904,       0,  0, 12,  0, }, // Palau
    {  548, 1443,       0,  720916,       0,  0, 13,  0, }, // Papua_New_Guinea
    {  548, 1460,  720929,  720944,  720970, 15, 26, 23, }, // Paraguay
    {  548, 1469,  720993,  721005,  721028, 12, 23, 20, }, // Peru
    {  548, 1474,  721048,  721062,  721087, 14, 25, 22, }, // Philippines
    {  548, 1486,       0,  721109,       0,  0, 18,  0, }, // Phoenix_Islands
    {  548, 1502,  721127,  721156,  721196, 29, 40, 37, }, // Pierre_Miquelon
    {  548, 1518,       0,  721233,       0,  0, 15,  0, }, // Pitcairn
    {  548, 1527,       0,  721248,       0,  0, 12,  0, }, // Ponape
    {  548, 1534,       0,  721260,       0,  0, 15,  0, }, // Pyongyang
    {  548, 1554,       0,  721275,       0,  0, 14,  0, }, // Reunion
    {  548, 1562,       0,  721289,       0,  0, 13,  0, }, // Rothera
    {  548, 1570,  721302,  721316,  721341, 14, 25, 22, }, // Sakhalin
    {  548, 1579,  721363,  721375,  721398, 12, 23, 20, }, // Samara
    {  548, 1596,  721418,  721431,  721455, 13, 24, 21, }, // Samoa
    {  548, 1602,       0,  721476,       0,  0, 13,  0, }, // Seychelles
    {  548, 1624,       0,  721489,       0,  0, 26,  0, }, // Singapore
    {  548, 1634,       0,  721515,       0,  0, 20,  0, }, // Solomon
    {  548, 1642,       0,  721535,       0,  0, 19,  0, }, // South_Georgia
    {  548, 1656,       0,  721554,       0,  0, 14,  0, }, // Suriname
    {  548, 1676,       0,  721568,       0,  0, 10,  0, }, // Syowa
    {  548, 1682,       0,  721578,       0,  0, 14,  0, }, // Tahiti
    {  548, 1689,  721592,  721605,  721629, 13, 24, 21, }, // Taipei
    {  548, 1696,       0,  721650,       0,  0, 18,  0, }, // Tajikistan
    {  548, 1724,       0,  721668,       0,  0, 14,  0, }, // Tokelau
    {  548, 1732,  721682,  721695,  721719, 13, 24, 21, }, // Tonga
    {  548, 1738,       0,  721740,       0,  0, 15,  0, }, // Truk
    {  548, 1750,  721755,  721774,  721804, 19, 30, 27, }, // Turkmenistan
    {  548, 1763,       0,  721831,       0,  0, 14,  0, }, // Tuvalu
    {  548, 1777,  721845,  721859,  721884, 14, 25, 22, }, // Uruguay
    {  548, 1792,  721906,  721923,  721951, 17, 28, 25, }, // Uzbekistan
    {  548, 1803,  721976,  721991,  722017, 15, 26, 23, }, // Vanuatu
    {  548, 1811,       0,  722040,       0,  0, 15,  0, }, // Venezuela
    {  548, 1821,  722055,  722072,  722100, 17, 28, 25, }, // Vladivostok
    {  548, 1833,  722125,  722141,  722168, 16, 27, 24, }, // Volgograd
    {  548, 1843,       0,  722192,       0,  0, 12,  0, }, // Vostok
    {  548, 1850,       0,  722204,       0,  0, 14,  0, }, // Wake
    {  548, 1855,       0,  722218,       0,  0, 21,  0, }, // Wallis
    {  548, 1862,  722239,  722251,  722274, 12, 23, 20, }, // Yakutsk
    {  548, 1870,  722294,  722313,  722343, 19, 30, 27, }, // Yekaterinburg
    {  548, 1892,       0,  722370,       0,  0, 12,  0, }, // Yukon
    {  550,    0,  722382,  722395,  722422, 13, 27, 22, }, // Acre Somali/Latin/Somalia
    {  550,    5,       0,  722444,       0,  0, 21,  0, }, // Afghanistan
    {  550,   17,       0,  722465,       0,  0, 24,  0, }, // Africa_Central
    {  550,   32,       0,  722489,       0,  0, 21,  0, }, // Africa_Eastern
    {  550,   65,       0,  722510,       0,  0, 38,  0, }, // Africa_Southern
    {  550,   81,  722548,  722572,  722610, 24, 38, 35, }, // Africa_Western
    {  550,  107,  722645,  722659,  722687, 14, 28, 27, }, // Alaska
    {  550,  128,  722714,  722728,  722756, 14, 28, 35, }, // Almaty
    {  550,  135,  722791,  722805,  722833, 14, 28, 25, }, // Amazon
    {  550,  142,  722858,  722894,  722944, 36, 50, 49, }, // America_Central
    {  550,  158,  722993,  723029,  723076, 36, 47, 46, }, // America_Eastern
    {  550,  174,  723122,  723162,  723213, 40, 51, 49, }, // America_Mountain
    {  550,  191,  723262,  723300,  723349, 38, 49, 48, }, // America_Pacific
    {  550,  207,  723397,  723412,  723441, 15, 29, 24, }, // Anadyr
    {  550,  214,  723465,  723478,  723505, 13, 27, 26, }, // Apia
    {  550,  219,  723531,  723544,  723571, 13, 27, 31, }, // Aqtau
    {  550,  225,  723602,  723616,  723644, 14, 28, 32, }, // Aqtobe
    {  550,  232,  723676,  723691,  723720, 15, 29, 28, }, // Arabian
    {  550,  240,  723748,  723765,  723797, 17, 32, 29, }, // Argentina
    {  550,  250,  723826,  723854,  723896, 28, 42, 39, }, // Argentina_Western
    {  550,  268,  723935,  723952,  723983, 17, 31, 28, }, // Armenia
    {  550,  286,  724011,  724050,  724100, 39, 50, 49, }, // Atlantic
    {  550,  295,  724149,  724178,  724221, 29, 43, 42, }, // Australia_Central
    {  550,  313,  724263,  724302,  724355, 39, 53, 48, }, // Australia_CentralWestern
    {  550,  338,  724403,  724429,  724471, 26, 42, 39, }, // Australia_Eastern
    {  550,  356,  724510,  724539,  724582, 29, 43, 42, }, // Australia_Western
    {  550,  374,  724624,  724642,  724674, 18, 32, 29, }, // Azerbaijan
    {  550,  385,  724703,  724717,  724745, 14, 28, 25, }, // Azores
    {  550,  397,  724770,  724789,  724822, 19, 33, 30, }, // Bangladesh
    {  550,  415,       0,  724852,       0,  0, 14,  0, }, // Bhutan
    {  550,  422,       0,  724866,       0,  0, 17,  0, }, // Bolivia
    {  550,  437,  724883,  724902,  724935, 19, 33, 30, }, // Brasilia
    {  550,  454,       0,  724965,       0,  0, 27,  0, }, // Brunei
    {  550,  461,  724992,  725011,  725044, 19, 33, 30, }, // Cape_Verde
    {  550,  478,       0,  725074,       0,  0, 14,  0, }, // Chamorro
    {  550,  487,  725088,  725102,  725130, 14, 28, 27, }, // Chatham
    {  550,  495,  725157,  725169,  725195, 12, 26, 23, }, // Chile
    {  550,  501,  725218,  725234,  725264, 16, 30, 29, }, // China
    {  550,  507,       0,  725293,       0,  0, 23,  0, }, // Christmas
    {  550,  517,       0,  725316,       0,  0, 20,  0, }, // Cocos
    {  550,  523,  725336,  725353,  725384, 17, 31, 28, }, // Colombia
    {  550,  532,  725412,  725433,  725468, 21, 35, 39, }, // Cook
    {  550,  537,  725507,  725520,  725547, 13, 27, 26, }, // Cuba
    {  550,  548,       0,  725573,       0,  0, 13,  0, }, // Davis
    {  550,  564,       0,  725586,       0,  0, 26,  0, }, // DumontDUrville
    {  550,  601,       0,  725612,       0,  0, 19,  0, }, // East_Timor
    {  550,  612,  725631,  725652,  725687, 21, 35, 32, }, // Easter
    {  550,  619,       0,  725719,       0,  0, 17,  0, }, // Ecuador
    {  550,  627,  725736,  725759,  725796, 23, 37, 34, }, // Europe_Central
    {  550,  642,  725830,  725850,  725884, 20, 34, 31, }, // Europe_Eastern
    {  550,  657,       0,  725915,       0,  0, 27,  0, }, // Europe_Further_Eastern
    {  550,  680,  725942,  725965,  726002, 23, 37, 34, }, // Europe_Western
    {  550,  695,  726036,  726062,  726102, 26, 40, 37, }, // Falkland
    {  550,  704,  726139,  726151,  726177, 12, 26, 23, }, // Fiji
    {  550,  709,       0,  726200,       0,  0, 21,  0, }, // French_Guiana
    {  550,  723,       0,  726221,       0,  0, 42,  0, }, // French_Southern
    {  550,  746,       0,  726263,       0,  0, 17,  0, }, // Galapagos
    {  550,  756,       0,  726280,       0,  0, 16,  0, }, // Gambier
    {  550,  764,  726296,  726312,  726342, 16, 30, 27, }, // Georgia
    {  550,  772,       0,  726369,       0,  0, 23,  0, }, // Gilbert_Islands
    {  550,  788,       0,  726392,       0,  0, 18,  0, }, // GMT
    {  550,  830,  726410,  726438,  726477, 28, 39, 36, }, // Greenland_Eastern
    {  550,  848,  726513,  726541,  726583, 28, 42, 39, }, // Greenland_Western
    {  550,  871,       0,  726622,       0,  0, 15,  0, }, // Gulf
    {  550,  876,       0,  726637,       0,  0, 15,  0, }, // Guyana
    {  550,  883,  726652,  726676,  726714, 24, 38, 37, }, // Hawaii_Aleutian
    {  550,  899,  726751,  726770,  726803, 19, 33, 30, }, // Hong_Kong
    {  550,  909,  726833,  726846,  726873, 13, 27, 24, }, // Hovd
    {  550,  914,       0,  726897,       0,  0, 29,  0, }, // India
    {  550,  920,       0,  726926,       0,  0, 25,  0, }, // Indian_Ocean
    {  550,  933,       0,  726951,       0,  0, 18,  0, }, // Indochina
    {  550,  943,       0,  726969,       0,  0, 30,  0, }, // Indonesia_Central
    {  550,  961,       0,  726999,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  550,  979,       0,  727019,       0,  0, 30,  0, }, // Indonesia_Western
    {  550,  997,  727049,  727063,  727091, 14, 28, 27, }, // Iran
    {  550, 1008,  727118,  727134,  727164, 16, 30, 27, }, // Irkutsk
    {  550, 1016,  727191,  727207,  727237, 16, 30, 29, }, // Israel
    {  550, 1023,  727266,  727280,  727308, 14, 28, 27, }, // Japan
    {  550, 1029,  727335,  727368,  727415, 33, 47, 42, }, // Kamchatka
    {  550, 1047,       0,  727457,       0,  0, 20,  0, }, // Kazakhstan
    {  550, 1058,       0,  727477,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  550, 1077,       0,  727504,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  550, 1106,  727533,  727548,  727577, 15, 29, 28, }, // Korea
    {  550, 1112,       0,  727605,       0,  0, 14,  0, }, // Kosrae
    {  550, 1119,  727619,  727640,  727675, 21, 35, 32, }, // Krasnoyarsk
    {  550, 1151,       0,  727707,       0,  0, 20,  0, }, // Kyrgystan
    {  550, 1175,       0,  727727,       0,  0, 19,  0, }, // Line_Islands
    {  550, 1188,  727746,  727761,  727790, 15, 29, 28, }, // Lord_Howe
    {  550, 1204,  727818,  727832,  727861, 14, 29, 26, }, // Magadan
    {  550, 1219,       0,  727887,       0,  0, 18,  0, }, // Malaysia
    {  550, 1228,       0,  727905,       0,  0, 16,  0, }, // Maldives
    {  550, 1237,       0,  727921,       0,  0, 18,  0, }, // Marquesas
    {  550, 1247,       0,  727939,       0,  0, 26,  0, }, // Marshall_Islands
    {  550, 1264,  727965,  727984,  728017, 19, 33, 30, }, // Mauritius
    {  550, 1274,       0,  728047,       0,  0, 14,  0, }, // Mawson
    {  550, 1281,  728061,  728088,  728129, 27, 41, 40, }, // Mexico_Pacific
    {  550, 1296,  728169,  728189,  728223, 20, 34, 31, }, // Mongolia
    {  550, 1305,  728254,  728268,  728296, 14, 28, 25, }, // Moscow
    {  550, 1312,       0,  728321,       0,  0, 17,  0, }, // Myanmar
    {  550, 1320,       0,  728338,       0,  0, 14,  0, }, // Nauru
    {  550, 1326,       0,  728352,       0,  0, 15,  0, }, // Nepal
    {  550, 1332,  728367,  728390,  728428, 23, 38, 35, }, // New_Caledonia
    {  550, 1346,  728463,  728484,  728519, 21, 35, 34, }, // New_Zealand
    {  550, 1358,  728553,  728574,  728609, 21, 35, 34, }, // Newfoundland
    {  550, 1371,       0,  728367,       0,  0, 13,  0, }, // Niue
    {  550, 1376,  728643,  728665,  728701, 22, 36, 34, }, // Norfolk
    {  550, 1384,  728735,  728763,  728806, 28, 43, 40, }, // Noronha
    {  550, 1406,  728846,  728866,  728900, 20, 34, 31, }, // Novosibirsk
    {  550, 1418,  728931,  728943,  728969, 12, 26, 23, }, // Omsk
    {  550, 1428,  728992,  729009,  729040, 17, 31, 28, }, // Pakistan
    {  550, 1437,       0,  729068,       0,  0, 13,  0, }, // Palau
    {  550, 1443,       0,  729081,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  550, 1460,  729107,  729124,  729155, 17, 31, 28, }, // Paraguay
    {  550, 1469,  729183,  729196,  729223, 13, 27, 24, }, // Peru
    {  550, 1474,  729247,  729263,  729293, 16, 30, 27, }, // Philippines
    {  550, 1486,       0,  729320,       0,  0, 25,  0, }, // Phoenix_Islands
    {  550, 1502,  729345,  729374,  729414, 29, 40, 42, }, // Pierre_Miquelon
    {  550, 1518,       0,  729456,       0,  0, 15,  0, }, // Pitcairn
    {  550, 1527,       0,  729471,       0,  0, 14,  0, }, // Ponape
    {  550, 1534,       0,  729485,       0,  0, 18,  0, }, // Pyongyang
    {  550, 1544,  729503,  729520,  729551, 17, 31, 35, }, // Qyzylorda
    {  550, 1554,       0,  729586,       0,  0, 18,  0, }, // Reunion
    {  550, 1562,       0,  729604,       0,  0, 14,  0, }, // Rothera
    {  550, 1570,  729618,  729634,  729664, 16, 30, 27, }, // Sakhalin
    {  550, 1579,  729691,  729706,  729735, 15, 29, 24, }, // Samara
    {  550, 1596,  729759,  729772,  729799, 13, 27, 26, }, // Samoa
    {  550, 1602,       0,  729825,       0,  0, 17,  0, }, // Seychelles
    {  550, 1624,       0,  729842,       0,  0, 17,  0, }, // Singapore
    {  550, 1634,       0,  729859,       0,  0, 24,  0, }, // Solomon
    {  550, 1642,       0,  729883,       0,  0, 21,  0, }, // South_Georgia
    {  550, 1656,       0,  729904,       0,  0, 16,  0, }, // Suriname
    {  550, 1676,       0,  729920,       0,  0, 14,  0, }, // Syowa
    {  550, 1682,       0,  729934,       0,  0, 14,  0, }, // Tahiti
    {  550, 1689,  729948,  729962,  729990, 14, 28, 27, }, // Taipei
    {  550, 1696,       0,  730017,       0,  0, 18,  0, }, // Tajikistan
    {  550, 1724,       0,  730035,       0,  0, 15,  0, }, // Tokelau
    {  550, 1732,  730050,  730063,  730090, 13, 27, 24, }, // Tonga
    {  550, 1738,       0,  730114,       0,  0, 12,  0, }, // Truk
    {  550, 1750,  730126,  730147,  730181, 21, 34, 31, }, // Turkmenistan
    {  550, 1763,       0,  730212,       0,  0, 14,  0, }, // Tuvalu
    {  550, 1777,  730226,  730242,  730272, 16, 30, 27, }, // Uruguay
    {  550, 1792,  730299,  730317,  730349, 18, 32, 29, }, // Uzbekistan
    {  550, 1803,  730378,  730393,  730422, 15, 29, 26, }, // Vanuatu
    {  550, 1811,       0,  730448,       0,  0, 19,  0, }, // Venezuela
    {  550, 1821,  730467,  730487,  730521, 20, 34, 31, }, // Vladivostok
    {  550, 1833,  730552,  730569,  730600, 17, 31, 28, }, // Volgograd
    {  550, 1843,       0,  730628,       0,  0, 14,  0, }, // Vostok
    {  550, 1850,       0,  730642,       0,  0, 21,  0, }, // Wake
    {  550, 1855,       0,  730663,       0,  0, 22,  0, }, // Wallis
    {  550, 1862,  730685,  730700,  730727, 15, 27, 24, }, // Yakutsk
    {  550, 1870,  730751,  730772,  730807, 21, 35, 32, }, // Yekaterinburg
    {  550, 1892,       0,  730839,       0,  0, 13,  0, }, // Yukon
    {  558,  788,       0,    2791,       0,  0, 19,  0, }, // GMT Southern Sotho/Latin/South Africa
    {  561,    0,  730852,  730864,  730885, 12, 21, 22, }, // Acre Spanish/Latin/Spain
    {  561,    5,       0,  289657,       0,  0, 18,  0, }, // Afghanistan
    {  561,   17,       0,  730907,       0,  0, 22,  0, }, // Africa_Central
    {  561,   32,       0,  730929,       0,  0, 23,  0, }, // Africa_Eastern
    {  561,   65,       0,  730952,       0,  0, 17,  0, }, // Africa_Southern
    {  561,   81,  730969,  730994,  731028, 25, 34, 35, }, // Africa_Western
    {  561,  107,  365568,  731063,  731086, 14, 23, 24, }, // Alaska
    {  561,  135,  731110,  731127,  731153, 17, 26, 27, }, // Amazon
    {  561,  142,   42412,  289990,   42485, 12, 21, 22, }, // America_Central
    {  561,  158,   42527,  731180,   42602, 13, 22, 23, }, // America_Eastern
    {  561,  174,  731202,  731230,  731267, 28, 37, 38, }, // America_Mountain
    {  561,  191,  731305,  731322,  731348, 17, 26, 27, }, // America_Pacific
    {  561,  207,  290408,  731375,  731398, 14, 23, 24, }, // Anadyr
    {  561,  214,  290474,  290486,  731422, 12, 21, 25, }, // Apia
    {  561,  219,  731447,  731460,  731482, 13, 22, 23, }, // Aqtau
    {  561,  225,  731505,  731519,  731542, 14, 23, 24, }, // Aqtobe
    {  561,  232,  731566,  731580,  731603, 14, 23, 24, }, // Arabian
    {  561,  240,  365980,  731627,  731653, 17, 26, 27, }, // Argentina
    {  561,  250,  366047,  731680,  731717, 28, 37, 38, }, // Argentina_Western
    {  561,  268,  290747,  290762,  731755, 15, 24, 25, }, // Armenia
    {  561,  286,  731780,  731798,  731825, 18, 27, 28, }, // Atlantic
    {  561,  295,  366246,  731853,  731887, 25, 34, 35, }, // Australia_Central
    {  561,  313,  731922,  731955,  731997, 33, 42, 43, }, // Australia_CentralWestern
    {  561,  338,  366458,  732040,  732075, 26, 35, 36, }, // Australia_Eastern
    {  561,  356,  366552,  732111,  732148, 28, 37, 38, }, // Australia_Western
    {  561,  374,  732186,  732204,  732231, 18, 27, 28, }, // Azerbaijan
    {  561,  385,  732259,  732277,  732304, 18, 27, 28, }, // Azores
    {  561,  397,  732332,  732349,  732375, 17, 26, 27, }, // Bangladesh
    {  561,  415,       0,  291401,       0,  0, 13,  0, }, // Bhutan
    {  561,  422,       0,  291414,       0,  0, 15,  0, }, // Bolivia
    {  561,  437,  291429,  291445,  732402, 16, 25, 26, }, // Brasilia
    {  561,  454,       0,  732428,       0,  0, 14,  0, }, // Brunei
    {  561,  461,  291520,  291538,  732442, 18, 27, 28, }, // Cape_Verde
    {  561,  478,       0,  732470,       0,  0, 25,  0, }, // Chamorro
    {  561,  487,  291614,  291629,  732495, 15, 24, 25, }, // Chatham
    {  561,  495,  291677,  291690,  732520, 13, 22, 23, }, // Chile
    {  561,  501,  367084,  732543,  732565, 13, 22, 23, }, // China
    {  561,  507,       0,  732588,       0,  0, 26,  0, }, // Christmas
    {  561,  517,       0,  732614,       0,  0, 23,  0, }, // Cocos
    {  561,  523,  291833,  291849,  732637, 16, 25, 26, }, // Colombia
    {  561,  532,  732663,  732685,  732716, 22, 31, 38, }, // Cook
    {  561,  537,  291980,  291992,  732754, 12, 21, 22, }, // Cuba
    {  561,  548,       0,  292034,       0,  0, 13,  0, }, // Davis
    {  561,  564,       0,  292047,       0,  0, 24,  0, }, // DumontDUrville
    {  561,  601,       0,  732776,       0,  0, 22,  0, }, // East_Timor
    {  561,  612,  732798,  732823,  732857, 25, 34, 35, }, // Easter
    {  561,  619,       0,  292174,       0,  0, 15,  0, }, // Ecuador
    {  561,  627,  367462,  732892,  732923, 22, 31, 32, }, // Europe_Central
    {  561,  642,  367542,  732955,  732987, 23, 32, 33, }, // Europe_Eastern
    {  561,  657,       0,  733020,       0,  0, 35,  0, }, // Europe_Further_Eastern
    {  561,  680,  367654,  733055,  733089, 25, 34, 35, }, // Europe_Western
    {  561,  695,  733124,  733150,  733185, 26, 35, 36, }, // Falkland
    {  561,  704,  733221,  733233,  733254, 12, 21, 22, }, // Fiji
    {  561,  709,       0,  733276,       0,  0, 27,  0, }, // French_Guiana
    {  561,  723,       0,  733303,       0,  0, 51,  0, }, // French_Southern
    {  561,  746,       0,  733354,       0,  0, 17,  0, }, // Galapagos
    {  561,  756,       0,  292711,       0,  0, 15,  0, }, // Gambier
    {  561,  764,  367963,  733371,  733395, 15, 24, 25, }, // Georgia
    {  561,  772,       0,  733420,       0,  0, 25,  0, }, // Gilbert_Islands
    {  561,  788,       0,  733445,       0,  0, 31,  0, }, // GMT
    {  561,  830,  368071,  733476,  733513, 28, 37, 38, }, // Greenland_Eastern
    {  561,  848,  368169,  733551,  733590, 30, 39, 40, }, // Greenland_Western
    {  561,  866,       0,   60685,       0,  0, 21,  0, }, // Guam
    {  561,  871,       0,  733630,       0,  0, 23,  0, }, // Gulf
    {  561,  876,       0,  368296,       0,  0, 14,  0, }, // Guyana
    {  561,  883,  733653,  733677,  733710, 24, 33, 34, }, // Hawaii_Aleutian
    {  561,  899,  293172,  293189,  733744, 17, 26, 27, }, // Hong_Kong
    {  561,  909,  293241,  293253,  733771, 12, 21, 22, }, // Hovd
    {  561,  914,       0,  733793,       0,  0, 25,  0, }, // India
    {  561,  920,       0,  733818,       0,  0, 22,  0, }, // Indian_Ocean
    {  561,  933,       0,  293329,       0,  0, 17,  0, }, // Indochina
    {  561,  943,       0,  733840,       0,  0, 25,  0, }, // Indonesia_Central
    {  561,  961,       0,  733865,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  561,  979,       0,  733891,       0,  0, 28,  0, }, // Indonesia_Western
    {  561,  997,  293425,  293437,  733919, 12, 21, 22, }, // Iran
    {  561, 1008,  293479,  293494,  733941, 15, 24, 25, }, // Irkutsk
    {  561, 1016,  293542,  293556,  733966, 14, 23, 24, }, // Israel
    {  561, 1023,  733990,  734003,  734025, 13, 22, 23, }, // Japan
    {  561, 1029,  734048,  734065,  734091, 17, 26, 27, }, // Kamchatka
    {  561, 1047,       0,  734118,       0,  0, 18,  0, }, // Kazakhstan
    {  561, 1058,       0,  734136,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  561, 1077,       0,  734163,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  561, 1106,  293862,  293875,  734192, 13, 22, 23, }, // Korea
    {  561, 1112,       0,  293919,       0,  0, 14,  0, }, // Kosrae
    {  561, 1119,  368906,  734215,  734243, 19, 28, 29, }, // Krasnoyarsk
    {  561, 1151,       0,  734272,       0,  0, 18,  0, }, // Kyrgystan
    {  561, 1161,       0,  734290,       0,  0, 17,  0, }, // Lanka
    {  561, 1175,       0,  734307,       0,  0, 34,  0, }, // Line_Islands
    {  561, 1188,  294050,  294067,  734341, 17, 26, 27, }, // Lord_Howe
    {  561, 1198,  734368,  734381,  734403, 13, 22, 23, }, // Macau
    {  561, 1204,  734426,  734441,  734465, 15, 24, 25, }, // Magadan
    {  561, 1219,       0,  734490,       0,  0, 15,  0, }, // Malaysia
    {  561, 1228,       0,  734505,       0,  0, 16,  0, }, // Maldives
    {  561, 1237,       0,  369150,       0,  0, 17,  0, }, // Marquesas
    {  561, 1247,       0,  734521,       0,  0, 26,  0, }, // Marshall_Islands
    {  561, 1264,  294256,  294272,  734547, 16, 25, 26, }, // Mauritius
    {  561, 1274,       0,  294322,       0,  0, 14,  0, }, // Mawson
    {  561, 1281,  734573,  734600,  734636, 27, 36, 37, }, // Mexico_Pacific
    {  561, 1296,  734673,  734691,  734718, 18, 27, 28, }, // Mongolia
    {  561, 1305,  734746,  734759,  734781, 13, 22, 23, }, // Moscow
    {  561, 1312,       0,  294567,       0,  0, 15,  0, }, // Myanmar
    {  561, 1320,       0,  294582,       0,  0, 13,  0, }, // Nauru
    {  561, 1326,       0,  294595,       0,  0, 13,  0, }, // Nepal
    {  561, 1332,  734804,  734827,  734859, 23, 32, 33, }, // New_Caledonia
    {  561, 1346,  734892,  734913,  734943, 21, 30, 31, }, // New_Zealand
    {  561, 1358,  369617,  734974,  735000, 17, 26, 27, }, // Newfoundland
    {  561, 1371,       0,  294845,       0,  0, 12,  0, }, // Niue
    {  561, 1376,  735027,  735050,  735082, 23, 32, 33, }, // Norfolk
    {  561, 1384,  294935,  294962,  735115, 27, 36, 37, }, // Noronha
    {  561, 1392,       0,  735152,       0,  0, 36,  0, }, // North_Mariana
    {  561, 1406,  295034,  295053,  735188, 19, 28, 29, }, // Novosibirsk
    {  561, 1418,  295109,  295121,  735217, 12, 21, 22, }, // Omsk
    {  561, 1428,  735239,  735255,  735280, 16, 25, 26, }, // Pakistan
    {  561, 1437,       0,  735306,       0,  0, 14,  0, }, // Palau
    {  561, 1443,       0,  735320,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  561, 1460,  370016,  735346,  735371, 16, 25, 26, }, // Paraguay
    {  561, 1469,  735397,  735409,  735430, 12, 21, 22, }, // Peru
    {  561, 1474,  295390,  295407,  735452, 17, 26, 27, }, // Philippines
    {  561, 1486,       0,  735479,       0,  0, 23,  0, }, // Phoenix_Islands
    {  561, 1502,  735502,  735530,  735567, 28, 37, 38, }, // Pierre_Miquelon
    {  561, 1518,       0,  295593,       0,  0, 16,  0, }, // Pitcairn
    {  561, 1527,       0,  295609,       0,  0, 15,  0, }, // Ponape
    {  561, 1534,       0,   63381,       0,  0, 17,  0, }, // Pyongyang
    {  561, 1544,   63398,   63415,  735605, 17, 26, 27, }, // Qyzylorda
    {  561, 1554,       0,  295624,       0,  0, 15,  0, }, // Reunion
    {  561, 1562,       0,  295639,       0,  0, 15,  0, }, // Rothera
    {  561, 1570,  735632,  735647,  735671, 15, 24, 25, }, // Sakhalin
    {  561, 1579,  295720,  735696,  735719, 14, 23, 24, }, // Samara
    {  561, 1596,  295786,  295799,  735743, 13, 22, 23, }, // Samoa
    {  561, 1602,       0,  735766,       0,  0, 18,  0, }, // Seychelles
    {  561, 1624,       0,  295862,       0,  0, 16,  0, }, // Singapore
    {  561, 1634,       0,  735784,       0,  0, 25,  0, }, // Solomon
    {  561, 1642,       0,  735809,       0,  0, 23,  0, }, // South_Georgia
    {  561, 1656,       0,  295922,       0,  0, 15,  0, }, // Suriname
    {  561, 1676,       0,  295938,       0,  0, 13,  0, }, // Syowa
    {  561, 1682,       0,  295951,       0,  0, 14,  0, }, // Tahiti
    {  561, 1689,  735832,  735846,  735869, 14, 23, 24, }, // Taipei
    {  561, 1696,       0,  735893,       0,  0, 18,  0, }, // Tajikistan
    {  561, 1724,       0,  296044,       0,  0, 15,  0, }, // Tokelau
    {  561, 1732,  296059,  296072,  735911, 13, 22, 23, }, // Tonga
    {  561, 1738,       0,  296116,       0,  0, 13,  0, }, // Truk
    {  561, 1750,  296129,  296149,  735934, 20, 29, 30, }, // Turkmenistan
    {  561, 1763,       0,  296207,       0,  0, 14,  0, }, // Tuvalu
    {  561, 1777,  370758,  735964,  735988, 15, 24, 25, }, // Uruguay
    {  561, 1792,  296284,  296302,  736013, 18, 27, 28, }, // Uzbekistan
    {  561, 1803,  296356,  296371,  736041, 15, 24, 25, }, // Vanuatu
    {  561, 1811,       0,  296419,       0,  0, 17,  0, }, // Venezuela
    {  561, 1821,  296436,  296455,  736066, 19, 28, 29, }, // Vladivostok
    {  561, 1833,  296511,  296529,  736095, 18, 27, 28, }, // Volgograd
    {  561, 1843,       0,  296583,       0,  0, 14,  0, }, // Vostok
    {  561, 1850,       0,  736123,       0,  0, 20,  0, }, // Wake
    {  561, 1855,       0,  736143,       0,  0, 23,  0, }, // Wallis
    {  561, 1862,  371055,  736166,  736190, 15, 24, 25, }, // Yakutsk
    {  561, 1870,  296700,  296721,  736215, 21, 30, 31, }, // Yekaterinburg
    {  561, 1892,       0,  736246,       0,  0, 13,  0, }, // Yukon
    {  562,  174,  736259,  736277,  736304, 18, 27, 28, }, // America_Mountain Spanish/Latin/Argentina
    {  562,  240,  365980,  731627,  731653, 17, 26, 27, }, // Argentina
    {  562,  250,  366047,  731680,  731717, 28, 37, 38, }, // Argentina_Western
    {  562,  517,       0,  736332,       0,  0, 19,  0, }, // Cocos
    {  562,  532,  736351,  736373,  736404, 22, 31, 38, }, // Cook
    {  562,  612,  736442,  736467,  736498, 25, 31, 35, }, // Easter
    {  562,  627,  367462,  732892,  732923, 22, 31, 32, }, // Europe_Central
    {  562,  642,  736533,  736556,  736588, 23, 32, 33, }, // Europe_Eastern
    {  562,  657,       0,  736621,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  562,  680,  736654,  736678,  736711, 24, 33, 34, }, // Europe_Western
    {  562,  695,  736745,  736771,  736806, 26, 35, 36, }, // Falkland
    {  562,  723,       0,  736842,       0,  0, 52,  0, }, // French_Southern
    {  562,  772,       0,  736894,       0,  0, 21,  0, }, // Gilbert_Islands
    {  562,  788,       0,  733445,       0,  0, 31,  0, }, // GMT
    {  562,  914,       0,  736915,       0,  0, 13,  0, }, // India
    {  562, 1247,       0,  736928,       0,  0, 22,  0, }, // Marshall_Islands
    {  562, 1312,       0,  736950,       0,  0, 26,  0, }, // Myanmar
    {  562, 1376,  736976,  736999,  737031, 23, 32, 33, }, // Norfolk
    {  562, 1534,       0,  737064,       0,  0, 16,  0, }, // Pyongyang
    {  562, 1634,       0,  737080,       0,  0, 21,  0, }, // Solomon
    {  562, 1850,       0,  737101,       0,  0, 17,  0, }, // Wake
    {  563,  174,  736259,  736277,  736304, 18, 27, 28, }, // America_Mountain Spanish/Latin/Belize
    {  563,  517,       0,  736332,       0,  0, 19,  0, }, // Cocos
    {  563,  532,  736351,  736373,  736404, 22, 31, 38, }, // Cook
    {  563,  612,  736442,  736467,  736498, 25, 31, 35, }, // Easter
    {  563,  627,  367462,  732892,  732923, 22, 31, 32, }, // Europe_Central
    {  563,  642,  736533,  736556,  736588, 23, 32, 33, }, // Europe_Eastern
    {  563,  657,       0,  736621,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  563,  680,  736654,  736678,  736711, 24, 33, 34, }, // Europe_Western
    {  563,  695,  736745,  736771,  736806, 26, 35, 36, }, // Falkland
    {  563,  723,       0,  736842,       0,  0, 52,  0, }, // French_Southern
    {  563,  772,       0,  736894,       0,  0, 21,  0, }, // Gilbert_Islands
    {  563,  788,       0,  733445,       0,  0, 31,  0, }, // GMT
    {  563,  914,       0,  736915,       0,  0, 13,  0, }, // India
    {  563, 1247,       0,  736928,       0,  0, 22,  0, }, // Marshall_Islands
    {  563, 1312,       0,  736950,       0,  0, 26,  0, }, // Myanmar
    {  563, 1376,  736976,  736999,  737031, 23, 32, 33, }, // Norfolk
    {  563, 1534,       0,  737064,       0,  0, 16,  0, }, // Pyongyang
    {  563, 1634,       0,  737080,       0,  0, 21,  0, }, // Solomon
    {  563, 1850,       0,  737101,       0,  0, 17,  0, }, // Wake
    {  564,  174,  736259,  736277,  736304, 18, 27, 28, }, // America_Mountain Spanish/Latin/Bolivia
    {  564,  422,       0,  291414,       0,  0, 15,  0, }, // Bolivia
    {  564,  517,       0,  736332,       0,  0, 19,  0, }, // Cocos
    {  564,  532,  736351,  736373,  736404, 22, 31, 38, }, // Cook
    {  564,  612,  736442,  736467,  736498, 25, 31, 35, }, // Easter
    {  564,  627,  367462,  732892,  732923, 22, 31, 32, }, // Europe_Central
    {  564,  642,  736533,  736556,  736588, 23, 32, 33, }, // Europe_Eastern
    {  564,  657,       0,  736621,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  564,  680,  736654,  736678,  736711, 24, 33, 34, }, // Europe_Western
    {  564,  695,  736745,  736771,  736806, 26, 35, 36, }, // Falkland
    {  564,  723,       0,  736842,       0,  0, 52,  0, }, // French_Southern
    {  564,  772,       0,  736894,       0,  0, 21,  0, }, // Gilbert_Islands
    {  564,  788,       0,  733445,       0,  0, 31,  0, }, // GMT
    {  564,  914,       0,  736915,       0,  0, 13,  0, }, // India
    {  564, 1247,       0,  736928,       0,  0, 22,  0, }, // Marshall_Islands
    {  564, 1312,       0,  736950,       0,  0, 26,  0, }, // Myanmar
    {  564, 1376,  736976,  736999,  737031, 23, 32, 33, }, // Norfolk
    {  564, 1534,       0,  737064,       0,  0, 16,  0, }, // Pyongyang
    {  564, 1634,       0,  737080,       0,  0, 21,  0, }, // Solomon
    {  564, 1850,       0,  737101,       0,  0, 17,  0, }, // Wake
    {  565,  174,  736259,  736277,  736304, 18, 27, 28, }, // America_Mountain Spanish/Latin/Brazil
    {  565,  517,       0,  736332,       0,  0, 19,  0, }, // Cocos
    {  565,  532,  736351,  736373,  736404, 22, 31, 38, }, // Cook
    {  565,  612,  736442,  736467,  736498, 25, 31, 35, }, // Easter
    {  565,  627,  367462,  732892,  732923, 22, 31, 32, }, // Europe_Central
    {  565,  642,  736533,  736556,  736588, 23, 32, 33, }, // Europe_Eastern
    {  565,  657,       0,  736621,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  565,  680,  736654,  736678,  736711, 24, 33, 34, }, // Europe_Western
    {  565,  695,  736745,  736771,  736806, 26, 35, 36, }, // Falkland
    {  565,  723,       0,  736842,       0,  0, 52,  0, }, // French_Southern
    {  565,  772,       0,  736894,       0,  0, 21,  0, }, // Gilbert_Islands
    {  565,  788,       0,  733445,       0,  0, 31,  0, }, // GMT
    {  565,  914,       0,  736915,       0,  0, 13,  0, }, // India
    {  565, 1247,       0,  736928,       0,  0, 22,  0, }, // Marshall_Islands
    {  565, 1312,       0,  736950,       0,  0, 26,  0, }, // Myanmar
    {  565, 1376,  736976,  736999,  737031, 23, 32, 33, }, // Norfolk
    {  565, 1534,       0,  737064,       0,  0, 16,  0, }, // Pyongyang
    {  565, 1634,       0,  737080,       0,  0, 21,  0, }, // Solomon
    {  565, 1850,       0,  737101,       0,  0, 17,  0, }, // Wake
    {  568,  174,  736259,  736277,  736304, 18, 27, 28, }, // America_Mountain Spanish/Latin/Chile
    {  568,  495,  291677,  291690,  732520, 13, 22, 23, }, // Chile
    {  568,  517,       0,  736332,       0,  0, 19,  0, }, // Cocos
    {  568,  532,  736351,  736373,  736404, 22, 31, 38, }, // Cook
    {  568,  612,  736442,  736467,  736498, 25, 31, 35, }, // Easter
    {  568,  627,  367462,  732892,  732923, 22, 31, 32, }, // Europe_Central
    {  568,  642,  736533,  736556,  736588, 23, 32, 33, }, // Europe_Eastern
    {  568,  657,       0,  736621,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  568,  680,  736654,  736678,  736711, 24, 33, 34, }, // Europe_Western
    {  568,  695,  736745,  736771,  736806, 26, 35, 36, }, // Falkland
    {  568,  723,       0,  736842,       0,  0, 52,  0, }, // French_Southern
    {  568,  772,       0,  736894,       0,  0, 21,  0, }, // Gilbert_Islands
    {  568,  788,       0,  733445,       0,  0, 31,  0, }, // GMT
    {  568,  914,       0,  736915,       0,  0, 13,  0, }, // India
    {  568, 1247,       0,  736928,       0,  0, 22,  0, }, // Marshall_Islands
    {  568, 1312,       0,  736950,       0,  0, 26,  0, }, // Myanmar
    {  568, 1376,  736976,  736999,  737031, 23, 32, 33, }, // Norfolk
    {  568, 1534,       0,  737064,       0,  0, 16,  0, }, // Pyongyang
    {  568, 1634,       0,  737080,       0,  0, 21,  0, }, // Solomon
    {  568, 1850,       0,  737101,       0,  0, 17,  0, }, // Wake
    {  569,  174,  736259,  736277,  736304, 18, 27, 28, }, // America_Mountain Spanish/Latin/Colombia
    {  569,  517,       0,  736332,       0,  0, 19,  0, }, // Cocos
    {  569,  523,  291833,  291849,  732637, 16, 25, 26, }, // Colombia
    {  569,  532,  736351,  736373,  736404, 22, 31, 38, }, // Cook
    {  569,  612,  736442,  736467,  736498, 25, 31, 35, }, // Easter
    {  569,  627,  367462,  732892,  732923, 22, 31, 32, }, // Europe_Central
    {  569,  642,  736533,  736556,  736588, 23, 32, 33, }, // Europe_Eastern
    {  569,  657,       0,  736621,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  569,  680,  736654,  736678,  736711, 24, 33, 34, }, // Europe_Western
    {  569,  695,  736745,  736771,  736806, 26, 35, 36, }, // Falkland
    {  569,  723,       0,  736842,       0,  0, 52,  0, }, // French_Southern
    {  569,  772,       0,  736894,       0,  0, 21,  0, }, // Gilbert_Islands
    {  569,  788,       0,  733445,       0,  0, 31,  0, }, // GMT
    {  569,  914,       0,  736915,       0,  0, 13,  0, }, // India
    {  569, 1247,       0,  736928,       0,  0, 22,  0, }, // Marshall_Islands
    {  569, 1312,       0,  736950,       0,  0, 26,  0, }, // Myanmar
    {  569, 1376,  736976,  736999,  737031, 23, 32, 33, }, // Norfolk
    {  569, 1534,       0,  737064,       0,  0, 16,  0, }, // Pyongyang
    {  569, 1634,       0,  737080,       0,  0, 21,  0, }, // Solomon
    {  569, 1850,       0,  737101,       0,  0, 17,  0, }, // Wake
    {  570,  174,  736259,  736277,  736304, 18, 27, 28, }, // America_Mountain Spanish/Latin/Costa Rica
    {  570,  517,       0,  736332,       0,  0, 19,  0, }, // Cocos
    {  570,  532,  736351,  736373,  736404, 22, 31, 38, }, // Cook
    {  570,  612,  736442,  736467,  736498, 25, 31, 35, }, // Easter
    {  570,  627,  367462,  732892,  732923, 22, 31, 32, }, // Europe_Central
    {  570,  642,  736533,  736556,  736588, 23, 32, 33, }, // Europe_Eastern
    {  570,  657,       0,  736621,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  570,  680,  736654,  736678,  736711, 24, 33, 34, }, // Europe_Western
    {  570,  695,  736745,  736771,  736806, 26, 35, 36, }, // Falkland
    {  570,  723,       0,  736842,       0,  0, 52,  0, }, // French_Southern
    {  570,  772,       0,  736894,       0,  0, 21,  0, }, // Gilbert_Islands
    {  570,  788,       0,  733445,       0,  0, 31,  0, }, // GMT
    {  570,  914,       0,  736915,       0,  0, 13,  0, }, // India
    {  570, 1247,       0,  736928,       0,  0, 22,  0, }, // Marshall_Islands
    {  570, 1312,       0,  736950,       0,  0, 26,  0, }, // Myanmar
    {  570, 1376,  736976,  736999,  737031, 23, 32, 33, }, // Norfolk
    {  570, 1534,       0,  737064,       0,  0, 16,  0, }, // Pyongyang
    {  570, 1634,       0,  737080,       0,  0, 21,  0, }, // Solomon
    {  570, 1850,       0,  737101,       0,  0, 17,  0, }, // Wake
    {  571,  174,  736259,  736277,  736304, 18, 27, 28, }, // America_Mountain Spanish/Latin/Cuba
    {  571,  517,       0,  736332,       0,  0, 19,  0, }, // Cocos
    {  571,  532,  736351,  736373,  736404, 22, 31, 38, }, // Cook
    {  571,  612,  736442,  736467,  736498, 25, 31, 35, }, // Easter
    {  571,  627,  367462,  732892,  732923, 22, 31, 32, }, // Europe_Central
    {  571,  642,  736533,  736556,  736588, 23, 32, 33, }, // Europe_Eastern
    {  571,  657,       0,  736621,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  571,  680,  736654,  736678,  736711, 24, 33, 34, }, // Europe_Western
    {  571,  695,  736745,  736771,  736806, 26, 35, 36, }, // Falkland
    {  571,  723,       0,  736842,       0,  0, 52,  0, }, // French_Southern
    {  571,  772,       0,  736894,       0,  0, 21,  0, }, // Gilbert_Islands
    {  571,  788,       0,  733445,       0,  0, 31,  0, }, // GMT
    {  571,  914,       0,  736915,       0,  0, 13,  0, }, // India
    {  571, 1247,       0,  736928,       0,  0, 22,  0, }, // Marshall_Islands
    {  571, 1312,       0,  736950,       0,  0, 26,  0, }, // Myanmar
    {  571, 1376,  736976,  736999,  737031, 23, 32, 33, }, // Norfolk
    {  571, 1534,       0,  737064,       0,  0, 16,  0, }, // Pyongyang
    {  571, 1634,       0,  737080,       0,  0, 21,  0, }, // Solomon
    {  571, 1850,       0,  737101,       0,  0, 17,  0, }, // Wake
    {  572,  174,  736259,  736277,  736304, 18, 27, 28, }, // America_Mountain Spanish/Latin/Dominican Republic
    {  572,  517,       0,  736332,       0,  0, 19,  0, }, // Cocos
    {  572,  532,  736351,  736373,  736404, 22, 31, 38, }, // Cook
    {  572,  612,  736442,  736467,  736498, 25, 31, 35, }, // Easter
    {  572,  627,  367462,  732892,  732923, 22, 31, 32, }, // Europe_Central
    {  572,  642,  736533,  736556,  736588, 23, 32, 33, }, // Europe_Eastern
    {  572,  657,       0,  736621,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  572,  680,  736654,  736678,  736711, 24, 33, 34, }, // Europe_Western
    {  572,  695,  736745,  736771,  736806, 26, 35, 36, }, // Falkland
    {  572,  723,       0,  736842,       0,  0, 52,  0, }, // French_Southern
    {  572,  772,       0,  736894,       0,  0, 21,  0, }, // Gilbert_Islands
    {  572,  788,       0,  733445,       0,  0, 31,  0, }, // GMT
    {  572,  914,       0,  736915,       0,  0, 13,  0, }, // India
    {  572, 1247,       0,  736928,       0,  0, 22,  0, }, // Marshall_Islands
    {  572, 1312,       0,  736950,       0,  0, 26,  0, }, // Myanmar
    {  572, 1376,  736976,  736999,  737031, 23, 32, 33, }, // Norfolk
    {  572, 1534,       0,  737064,       0,  0, 16,  0, }, // Pyongyang
    {  572, 1634,       0,  737080,       0,  0, 21,  0, }, // Solomon
    {  572, 1850,       0,  737101,       0,  0, 17,  0, }, // Wake
    {  573,  174,  736259,  736277,  736304, 18, 27, 28, }, // America_Mountain Spanish/Latin/Ecuador
    {  573,  517,       0,  736332,       0,  0, 19,  0, }, // Cocos
    {  573,  532,  736351,  736373,  736404, 22, 31, 38, }, // Cook
    {  573,  612,  736442,  736467,  736498, 25, 31, 35, }, // Easter
    {  573,  619,       0,  292174,       0,  0, 15,  0, }, // Ecuador
    {  573,  627,  367462,  732892,  732923, 22, 31, 32, }, // Europe_Central
    {  573,  642,  736533,  736556,  736588, 23, 32, 33, }, // Europe_Eastern
    {  573,  657,       0,  736621,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  573,  680,  736654,  736678,  736711, 24, 33, 34, }, // Europe_Western
    {  573,  695,  736745,  736771,  736806, 26, 35, 36, }, // Falkland
    {  573,  723,       0,  736842,       0,  0, 52,  0, }, // French_Southern
    {  573,  772,       0,  736894,       0,  0, 21,  0, }, // Gilbert_Islands
    {  573,  788,       0,  733445,       0,  0, 31,  0, }, // GMT
    {  573,  914,       0,  736915,       0,  0, 13,  0, }, // India
    {  573, 1247,       0,  736928,       0,  0, 22,  0, }, // Marshall_Islands
    {  573, 1312,       0,  736950,       0,  0, 26,  0, }, // Myanmar
    {  573, 1376,  736976,  736999,  737031, 23, 32, 33, }, // Norfolk
    {  573, 1534,       0,  737064,       0,  0, 16,  0, }, // Pyongyang
    {  573, 1634,       0,  737080,       0,  0, 21,  0, }, // Solomon
    {  573, 1850,       0,  737101,       0,  0, 17,  0, }, // Wake
    {  574,  174,  736259,  736277,  736304, 18, 27, 28, }, // America_Mountain Spanish/Latin/El Salvador
    {  574,  517,       0,  736332,       0,  0, 19,  0, }, // Cocos
    {  574,  532,  736351,  736373,  736404, 22, 31, 38, }, // Cook
    {  574,  612,  736442,  736467,  736498, 25, 31, 35, }, // Easter
    {  574,  627,  367462,  732892,  732923, 22, 31, 32, }, // Europe_Central
    {  574,  642,  736533,  736556,  736588, 23, 32, 33, }, // Europe_Eastern
    {  574,  657,       0,  736621,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  574,  680,  736654,  736678,  736711, 24, 33, 34, }, // Europe_Western
    {  574,  695,  736745,  736771,  736806, 26, 35, 36, }, // Falkland
    {  574,  723,       0,  736842,       0,  0, 52,  0, }, // French_Southern
    {  574,  772,       0,  736894,       0,  0, 21,  0, }, // Gilbert_Islands
    {  574,  788,       0,  733445,       0,  0, 31,  0, }, // GMT
    {  574,  914,       0,  736915,       0,  0, 13,  0, }, // India
    {  574, 1247,       0,  736928,       0,  0, 22,  0, }, // Marshall_Islands
    {  574, 1312,       0,  736950,       0,  0, 26,  0, }, // Myanmar
    {  574, 1376,  736976,  736999,  737031, 23, 32, 33, }, // Norfolk
    {  574, 1534,       0,  737064,       0,  0, 16,  0, }, // Pyongyang
    {  574, 1634,       0,  737080,       0,  0, 21,  0, }, // Solomon
    {  574, 1850,       0,  737101,       0,  0, 17,  0, }, // Wake
    {  576,  174,  736259,  736277,  736304, 18, 27, 28, }, // America_Mountain Spanish/Latin/Guatemala
    {  576,  517,       0,  736332,       0,  0, 19,  0, }, // Cocos
    {  576,  532,  736351,  736373,  736404, 22, 31, 38, }, // Cook
    {  576,  612,  736442,  736467,  736498, 25, 31, 35, }, // Easter
    {  576,  627,  367462,  732892,  732923, 22, 31, 32, }, // Europe_Central
    {  576,  642,  736533,  736556,  736588, 23, 32, 33, }, // Europe_Eastern
    {  576,  657,       0,  736621,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  576,  680,  736654,  736678,  736711, 24, 33, 34, }, // Europe_Western
    {  576,  695,  736745,  736771,  736806, 26, 35, 36, }, // Falkland
    {  576,  723,       0,  736842,       0,  0, 52,  0, }, // French_Southern
    {  576,  772,       0,  736894,       0,  0, 21,  0, }, // Gilbert_Islands
    {  576,  788,       0,  733445,       0,  0, 31,  0, }, // GMT
    {  576,  914,       0,  736915,       0,  0, 13,  0, }, // India
    {  576, 1247,       0,  736928,       0,  0, 22,  0, }, // Marshall_Islands
    {  576, 1312,       0,  736950,       0,  0, 26,  0, }, // Myanmar
    {  576, 1376,  736976,  736999,  737031, 23, 32, 33, }, // Norfolk
    {  576, 1534,       0,  737064,       0,  0, 16,  0, }, // Pyongyang
    {  576, 1634,       0,  737080,       0,  0, 21,  0, }, // Solomon
    {  576, 1850,       0,  737101,       0,  0, 17,  0, }, // Wake
    {  577,  174,  736259,  736277,  736304, 18, 27, 28, }, // America_Mountain Spanish/Latin/Honduras
    {  577,  517,       0,  736332,       0,  0, 19,  0, }, // Cocos
    {  577,  532,  736351,  736373,  736404, 22, 31, 38, }, // Cook
    {  577,  612,  736442,  736467,  736498, 25, 31, 35, }, // Easter
    {  577,  627,  367462,  732892,  732923, 22, 31, 32, }, // Europe_Central
    {  577,  642,  736533,  736556,  736588, 23, 32, 33, }, // Europe_Eastern
    {  577,  657,       0,  736621,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  577,  680,  736654,  736678,  736711, 24, 33, 34, }, // Europe_Western
    {  577,  695,  736745,  736771,  736806, 26, 35, 36, }, // Falkland
    {  577,  723,       0,  736842,       0,  0, 52,  0, }, // French_Southern
    {  577,  772,       0,  736894,       0,  0, 21,  0, }, // Gilbert_Islands
    {  577,  788,       0,  733445,       0,  0, 31,  0, }, // GMT
    {  577,  914,       0,  736915,       0,  0, 13,  0, }, // India
    {  577, 1247,       0,  736928,       0,  0, 22,  0, }, // Marshall_Islands
    {  577, 1312,       0,  736950,       0,  0, 26,  0, }, // Myanmar
    {  577, 1376,  736976,  736999,  737031, 23, 32, 33, }, // Norfolk
    {  577, 1534,       0,  737064,       0,  0, 16,  0, }, // Pyongyang
    {  577, 1634,       0,  737080,       0,  0, 21,  0, }, // Solomon
    {  577, 1850,       0,  737101,       0,  0, 17,  0, }, // Wake
    {  578,  174,  736259,  736277,  736304, 18, 27, 28, }, // America_Mountain Spanish/Latin/Latin America
    {  578,  517,       0,  736332,       0,  0, 19,  0, }, // Cocos
    {  578,  532,  736351,  736373,  736404, 22, 31, 38, }, // Cook
    {  578,  612,  736442,  736467,  736498, 25, 31, 35, }, // Easter
    {  578,  627,  367462,  732892,  732923, 22, 31, 32, }, // Europe_Central
    {  578,  642,  736533,  736556,  736588, 23, 32, 33, }, // Europe_Eastern
    {  578,  657,       0,  736621,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  578,  680,  736654,  736678,  736711, 24, 33, 34, }, // Europe_Western
    {  578,  695,  736745,  736771,  736806, 26, 35, 36, }, // Falkland
    {  578,  723,       0,  736842,       0,  0, 52,  0, }, // French_Southern
    {  578,  772,       0,  736894,       0,  0, 21,  0, }, // Gilbert_Islands
    {  578,  788,       0,  733445,       0,  0, 31,  0, }, // GMT
    {  578,  914,       0,  736915,       0,  0, 13,  0, }, // India
    {  578, 1247,       0,  736928,       0,  0, 22,  0, }, // Marshall_Islands
    {  578, 1312,       0,  736950,       0,  0, 26,  0, }, // Myanmar
    {  578, 1376,  736976,  736999,  737031, 23, 32, 33, }, // Norfolk
    {  578, 1534,       0,  737064,       0,  0, 16,  0, }, // Pyongyang
    {  578, 1634,       0,  737080,       0,  0, 21,  0, }, // Solomon
    {  578, 1850,       0,  737101,       0,  0, 17,  0, }, // Wake
    {  579,  174,  736259,  736277,  736304, 18, 27, 28, }, // America_Mountain Spanish/Latin/Mexico
    {  579,  214,  290474,  290486,  737118, 12, 21, 22, }, // Apia
    {  579,  507,       0,  737140,       0,  0, 26,  0, }, // Christmas
    {  579,  612,  737166,  732823,  732857, 22, 34, 35, }, // Easter
    {  579,  627,  367462,  732892,  732923, 22, 31, 32, }, // Europe_Central
    {  579,  642,  367542,  732955,  732987, 23, 32, 33, }, // Europe_Eastern
    {  579,  680,  367654,  733055,  733089, 25, 34, 35, }, // Europe_Western
    {  579,  695,  737188,  737210,  737241, 22, 31, 32, }, // Falkland
    {  579,  723,       0,  736842,       0,  0, 52,  0, }, // French_Southern
    {  579,  772,       0,  737273,       0,  0, 25,  0, }, // Gilbert_Islands
    {  579,  788,       0,  733445,       0,  0, 31,  0, }, // GMT
    {  579,  914,       0,  736915,       0,  0, 13,  0, }, // India
    {  579, 1312,       0,  736950,       0,  0, 26,  0, }, // Myanmar
    {  579, 1376,  736976,  736999,  737031, 23, 32, 33, }, // Norfolk
    {  579, 1850,       0,  737298,       0,  0, 20,  0, }, // Wake
    {  580,  174,  736259,  736277,  736304, 18, 27, 28, }, // America_Mountain Spanish/Latin/Nicaragua
    {  580,  517,       0,  736332,       0,  0, 19,  0, }, // Cocos
    {  580,  532,  736351,  736373,  736404, 22, 31, 38, }, // Cook
    {  580,  612,  736442,  736467,  736498, 25, 31, 35, }, // Easter
    {  580,  627,  367462,  732892,  732923, 22, 31, 32, }, // Europe_Central
    {  580,  642,  736533,  736556,  736588, 23, 32, 33, }, // Europe_Eastern
    {  580,  657,       0,  736621,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  580,  680,  736654,  736678,  736711, 24, 33, 34, }, // Europe_Western
    {  580,  695,  736745,  736771,  736806, 26, 35, 36, }, // Falkland
    {  580,  723,       0,  736842,       0,  0, 52,  0, }, // French_Southern
    {  580,  772,       0,  736894,       0,  0, 21,  0, }, // Gilbert_Islands
    {  580,  788,       0,  733445,       0,  0, 31,  0, }, // GMT
    {  580,  914,       0,  736915,       0,  0, 13,  0, }, // India
    {  580, 1247,       0,  736928,       0,  0, 22,  0, }, // Marshall_Islands
    {  580, 1312,       0,  736950,       0,  0, 26,  0, }, // Myanmar
    {  580, 1376,  736976,  736999,  737031, 23, 32, 33, }, // Norfolk
    {  580, 1534,       0,  737064,       0,  0, 16,  0, }, // Pyongyang
    {  580, 1634,       0,  737080,       0,  0, 21,  0, }, // Solomon
    {  580, 1850,       0,  737101,       0,  0, 17,  0, }, // Wake
    {  581,  174,  736259,  736277,  736304, 18, 27, 28, }, // America_Mountain Spanish/Latin/Panama
    {  581,  517,       0,  736332,       0,  0, 19,  0, }, // Cocos
    {  581,  532,  736351,  736373,  736404, 22, 31, 38, }, // Cook
    {  581,  612,  736442,  736467,  736498, 25, 31, 35, }, // Easter
    {  581,  627,  367462,  732892,  732923, 22, 31, 32, }, // Europe_Central
    {  581,  642,  736533,  736556,  736588, 23, 32, 33, }, // Europe_Eastern
    {  581,  657,       0,  736621,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  581,  680,  736654,  736678,  736711, 24, 33, 34, }, // Europe_Western
    {  581,  695,  736745,  736771,  736806, 26, 35, 36, }, // Falkland
    {  581,  723,       0,  736842,       0,  0, 52,  0, }, // French_Southern
    {  581,  772,       0,  736894,       0,  0, 21,  0, }, // Gilbert_Islands
    {  581,  788,       0,  733445,       0,  0, 31,  0, }, // GMT
    {  581,  914,       0,  736915,       0,  0, 13,  0, }, // India
    {  581, 1247,       0,  736928,       0,  0, 22,  0, }, // Marshall_Islands
    {  581, 1312,       0,  736950,       0,  0, 26,  0, }, // Myanmar
    {  581, 1376,  736976,  736999,  737031, 23, 32, 33, }, // Norfolk
    {  581, 1534,       0,  737064,       0,  0, 16,  0, }, // Pyongyang
    {  581, 1634,       0,  737080,       0,  0, 21,  0, }, // Solomon
    {  581, 1850,       0,  737101,       0,  0, 17,  0, }, // Wake
    {  582,  174,  736259,  736277,  736304, 18, 27, 28, }, // America_Mountain Spanish/Latin/Paraguay
    {  582,  517,       0,  736332,       0,  0, 19,  0, }, // Cocos
    {  582,  532,  736351,  736373,  736404, 22, 31, 38, }, // Cook
    {  582,  612,  736442,  736467,  736498, 25, 31, 35, }, // Easter
    {  582,  627,  367462,  732892,  732923, 22, 31, 32, }, // Europe_Central
    {  582,  642,  736533,  736556,  736588, 23, 32, 33, }, // Europe_Eastern
    {  582,  657,       0,  736621,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  582,  680,  736654,  736678,  736711, 24, 33, 34, }, // Europe_Western
    {  582,  695,  736745,  736771,  736806, 26, 35, 36, }, // Falkland
    {  582,  723,       0,  736842,       0,  0, 52,  0, }, // French_Southern
    {  582,  772,       0,  736894,       0,  0, 21,  0, }, // Gilbert_Islands
    {  582,  788,       0,  733445,       0,  0, 31,  0, }, // GMT
    {  582,  914,       0,  736915,       0,  0, 13,  0, }, // India
    {  582, 1247,       0,  736928,       0,  0, 22,  0, }, // Marshall_Islands
    {  582, 1312,       0,  736950,       0,  0, 26,  0, }, // Myanmar
    {  582, 1376,  736976,  736999,  737031, 23, 32, 33, }, // Norfolk
    {  582, 1534,       0,  737064,       0,  0, 16,  0, }, // Pyongyang
    {  582, 1634,       0,  737080,       0,  0, 21,  0, }, // Solomon
    {  582, 1850,       0,  737101,       0,  0, 17,  0, }, // Wake
    {  583,  174,  736259,  736277,  736304, 18, 27, 28, }, // America_Mountain Spanish/Latin/Peru
    {  583,  517,       0,  736332,       0,  0, 19,  0, }, // Cocos
    {  583,  532,  736351,  736373,  736404, 22, 31, 38, }, // Cook
    {  583,  612,  736442,  736467,  736498, 25, 31, 35, }, // Easter
    {  583,  627,  367462,  732892,  732923, 22, 31, 32, }, // Europe_Central
    {  583,  642,  736533,  736556,  736588, 23, 32, 33, }, // Europe_Eastern
    {  583,  657,       0,  736621,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  583,  680,  736654,  736678,  736711, 24, 33, 34, }, // Europe_Western
    {  583,  695,  736745,  736771,  736806, 26, 35, 36, }, // Falkland
    {  583,  723,       0,  736842,       0,  0, 52,  0, }, // French_Southern
    {  583,  772,       0,  736894,       0,  0, 21,  0, }, // Gilbert_Islands
    {  583,  788,       0,  733445,       0,  0, 31,  0, }, // GMT
    {  583,  914,       0,  736915,       0,  0, 13,  0, }, // India
    {  583, 1247,       0,  736928,       0,  0, 22,  0, }, // Marshall_Islands
    {  583, 1312,       0,  736950,       0,  0, 26,  0, }, // Myanmar
    {  583, 1376,  736976,  736999,  737031, 23, 32, 33, }, // Norfolk
    {  583, 1469,  735397,  735409,  735430, 12, 21, 22, }, // Peru
    {  583, 1534,       0,  737064,       0,  0, 16,  0, }, // Pyongyang
    {  583, 1634,       0,  737080,       0,  0, 21,  0, }, // Solomon
    {  583, 1850,       0,  737101,       0,  0, 17,  0, }, // Wake
    {  585,  174,  736259,  736277,  736304, 18, 27, 28, }, // America_Mountain Spanish/Latin/Puerto Rico
    {  585,  517,       0,  736332,       0,  0, 19,  0, }, // Cocos
    {  585,  532,  736351,  736373,  736404, 22, 31, 38, }, // Cook
    {  585,  612,  736442,  736467,  736498, 25, 31, 35, }, // Easter
    {  585,  627,  367462,  732892,  732923, 22, 31, 32, }, // Europe_Central
    {  585,  642,  736533,  736556,  736588, 23, 32, 33, }, // Europe_Eastern
    {  585,  657,       0,  736621,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  585,  680,  736654,  736678,  736711, 24, 33, 34, }, // Europe_Western
    {  585,  695,  736745,  736771,  736806, 26, 35, 36, }, // Falkland
    {  585,  723,       0,  736842,       0,  0, 52,  0, }, // French_Southern
    {  585,  772,       0,  736894,       0,  0, 21,  0, }, // Gilbert_Islands
    {  585,  788,       0,  733445,       0,  0, 31,  0, }, // GMT
    {  585,  914,       0,  736915,       0,  0, 13,  0, }, // India
    {  585, 1247,       0,  736928,       0,  0, 22,  0, }, // Marshall_Islands
    {  585, 1312,       0,  736950,       0,  0, 26,  0, }, // Myanmar
    {  585, 1376,  736976,  736999,  737031, 23, 32, 33, }, // Norfolk
    {  585, 1534,       0,  737064,       0,  0, 16,  0, }, // Pyongyang
    {  585, 1634,       0,  737080,       0,  0, 21,  0, }, // Solomon
    {  585, 1850,       0,  737101,       0,  0, 17,  0, }, // Wake
    {  586,  107,  365568,  731063,  731086, 14, 23, 24, }, // Alaska Spanish/Latin/United States
    {  586,  142,   42412,  289990,   42485, 12, 21, 22, }, // America_Central
    {  586,  158,   42527,  731180,   42602, 13, 22, 23, }, // America_Eastern
    {  586,  174,  731202,  731230,  731267, 28, 37, 38, }, // America_Mountain
    {  586,  191,  731305,  731322,  731348, 17, 26, 27, }, // America_Pacific
    {  586,  214,  290474,  290486,  737118, 12, 21, 22, }, // Apia
    {  586,  286,  731780,  731798,  731825, 18, 27, 28, }, // Atlantic
    {  586,  478,       0,  737318,       0,  0, 16,  0, }, // Chamorro
    {  586,  627,  367462,  732892,  732923, 22, 31, 32, }, // Europe_Central
    {  586,  642,  367542,  732955,  732987, 23, 32, 33, }, // Europe_Eastern
    {  586,  680,  367654,  733055,  733089, 25, 34, 35, }, // Europe_Western
    {  586,  723,       0,  736842,       0,  0, 52,  0, }, // French_Southern
    {  586,  788,       0,  733445,       0,  0, 31,  0, }, // GMT
    {  586,  883,  733653,  733677,  733710, 24, 33, 34, }, // Hawaii_Aleutian
    {  586,  914,       0,  736915,       0,  0, 13,  0, }, // India
    {  586,  920,       0,  737334,       0,  0, 22,  0, }, // Indian_Ocean
    {  586, 1237,       0,  737356,       0,  0, 27,  0, }, // Marquesas
    {  586, 1312,       0,  736950,       0,  0, 26,  0, }, // Myanmar
    {  586, 1486,       0,  737383,       0,  0, 23,  0, }, // Phoenix_Islands
    {  587,  174,  736259,  736277,  736304, 18, 27, 28, }, // America_Mountain Spanish/Latin/Uruguay
    {  587,  517,       0,  736332,       0,  0, 19,  0, }, // Cocos
    {  587,  532,  736351,  736373,  736404, 22, 31, 38, }, // Cook
    {  587,  612,  736442,  736467,  736498, 25, 31, 35, }, // Easter
    {  587,  627,  367462,  732892,  732923, 22, 31, 32, }, // Europe_Central
    {  587,  642,  736533,  736556,  736588, 23, 32, 33, }, // Europe_Eastern
    {  587,  657,       0,  736621,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  587,  680,  736654,  736678,  736711, 24, 33, 34, }, // Europe_Western
    {  587,  695,  736745,  736771,  736806, 26, 35, 36, }, // Falkland
    {  587,  723,       0,  736842,       0,  0, 52,  0, }, // French_Southern
    {  587,  772,       0,  736894,       0,  0, 21,  0, }, // Gilbert_Islands
    {  587,  788,       0,  733445,       0,  0, 31,  0, }, // GMT
    {  587,  914,       0,  736915,       0,  0, 13,  0, }, // India
    {  587, 1247,       0,  736928,       0,  0, 22,  0, }, // Marshall_Islands
    {  587, 1312,       0,  736950,       0,  0, 26,  0, }, // Myanmar
    {  587, 1376,  736976,  736999,  737031, 23, 32, 33, }, // Norfolk
    {  587, 1534,       0,  737064,       0,  0, 16,  0, }, // Pyongyang
    {  587, 1634,       0,  737080,       0,  0, 21,  0, }, // Solomon
    {  587, 1777,  370758,  735964,  735988, 15, 24, 25, }, // Uruguay
    {  587, 1850,       0,  737101,       0,  0, 17,  0, }, // Wake
    {  588,  174,  736259,  736277,  736304, 18, 27, 28, }, // America_Mountain Spanish/Latin/Venezuela
    {  588,  517,       0,  736332,       0,  0, 19,  0, }, // Cocos
    {  588,  532,  736351,  736373,  736404, 22, 31, 38, }, // Cook
    {  588,  612,  736442,  736467,  736498, 25, 31, 35, }, // Easter
    {  588,  627,  367462,  732892,  732923, 22, 31, 32, }, // Europe_Central
    {  588,  642,  736533,  736556,  736588, 23, 32, 33, }, // Europe_Eastern
    {  588,  657,       0,  736621,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  588,  680,  736654,  736678,  736711, 24, 33, 34, }, // Europe_Western
    {  588,  695,  736745,  736771,  736806, 26, 35, 36, }, // Falkland
    {  588,  723,       0,  736842,       0,  0, 52,  0, }, // French_Southern
    {  588,  772,       0,  736894,       0,  0, 21,  0, }, // Gilbert_Islands
    {  588,  788,       0,  733445,       0,  0, 31,  0, }, // GMT
    {  588,  914,       0,  736915,       0,  0, 13,  0, }, // India
    {  588, 1247,       0,  736928,       0,  0, 22,  0, }, // Marshall_Islands
    {  588, 1312,       0,  736950,       0,  0, 26,  0, }, // Myanmar
    {  588, 1376,  736976,  736999,  737031, 23, 32, 33, }, // Norfolk
    {  588, 1534,       0,  737064,       0,  0, 16,  0, }, // Pyongyang
    {  588, 1634,       0,  737080,       0,  0, 21,  0, }, // Solomon
    {  588, 1811,       0,  296419,       0,  0, 17,  0, }, // Venezuela
    {  588, 1850,       0,  737101,       0,  0, 17,  0, }, // Wake
    {  590,  142,  358970,  358982,  737406, 12, 20, 23, }, // America_Central Sundanese/Latin/Indonesia
    {  590,  158,  737429,  737440,  737459, 11, 19, 22, }, // America_Eastern
    {  590,  174,  737481,  737497,  737521, 16, 24, 27, }, // America_Mountain
    {  590,  191,  359147,  359160,  737548, 13, 21, 24, }, // America_Pacific
    {  590,  286,  359699,  359713,  737572, 14, 22, 25, }, // Atlantic
    {  590,  523,  360732,  360746,  737597, 14, 22, 25, }, // Colombia
    {  590,  627,  737622,  737640,  737666, 18, 26, 29, }, // Europe_Central
    {  590,  642,  737695,  737712,  737737, 17, 25, 28, }, // Europe_Eastern
    {  590,  680,  737765,  737782,  737807, 17, 25, 28, }, // Europe_Western
    {  590,  746,       0,  361497,       0,  0, 15,  0, }, // Galapagos
    {  590,  788,       0,  737835,       0,  0, 15,  0, }, // GMT
    {  591,    5,       0,  737850,       0,  0, 18,  0, }, // Afghanistan Swahili/Latin/Tanzania
    {  591,   17,       0,  737868,       0,  0, 21,  0, }, // Africa_Central
    {  591,   32,       0,  737889,       0,  0, 23,  0, }, // Africa_Eastern
    {  591,   65,       0,  737912,       0,  0, 31,  0, }, // Africa_Southern
    {  591,   81,  737943,  737966,  738000, 23, 34, 41, }, // Africa_Western
    {  591,  107,  738041,  738054,  738078, 13, 24, 23, }, // Alaska
    {  591,  135,  738101,  738114,  738138, 13, 24, 31, }, // Amazon
    {  591,  142,  738169,  738180,  738202, 11, 22, 21, }, // America_Central
    {  591,  158,  738223,  738239,  738266, 16, 27, 26, }, // America_Eastern
    {  591,  174,  738292,  738307,  738333, 15, 26, 25, }, // America_Mountain
    {  591,  191,  738358,  738373,  738399, 15, 26, 25, }, // America_Pacific
    {  591,  207,  738424,  738437,  738461, 13, 24, 25, }, // Anadyr
    {  591,  214,  738486,  738497,  738519, 11, 22, 21, }, // Apia
    {  591,  232,  738540,  738555,  738581, 15, 26, 25, }, // Arabian
    {  591,  240,  738606,  738622,  738649, 16, 27, 34, }, // Argentina
    {  591,  250,  738683,  738713,  738754, 30, 41, 48, }, // Argentina_Western
    {  591,  268,  738802,  738816,  738841, 14, 25, 32, }, // Armenia
    {  591,  286,  738873,  738889,  738916, 16, 27, 26, }, // Atlantic
    {  591,  295,  738942,  738966,  739001, 24, 35, 34, }, // Australia_Central
    {  591,  313,  739035,  739072,  739120, 37, 48, 47, }, // Australia_CentralWestern
    {  591,  338,  739167,  739193,  739234, 26, 41, 40, }, // Australia_Eastern
    {  591,  356,  739274,  739300,  739337, 26, 37, 36, }, // Australia_Western
    {  591,  374,  739373,  739390,  739418, 17, 28, 35, }, // Azerbaijan
    {  591,  385,  739453,  739466,  739490, 13, 24, 31, }, // Azores
    {  591,  397,  739521,  739538,  739566, 17, 28, 35, }, // Bangladesh
    {  591,  415,       0,  739601,       0,  0, 13,  0, }, // Bhutan
    {  591,  422,       0,  739614,       0,  0, 14,  0, }, // Bolivia
    {  591,  437,  739628,  739643,  739669, 15, 26, 33, }, // Brasilia
    {  591,  454,       0,  739702,       0,  0, 24,  0, }, // Brunei
    {  591,  461,  739726,  739743,  739771, 17, 28, 35, }, // Cape_Verde
    {  591,  478,       0,  739806,       0,  0, 26,  0, }, // Chamorro
    {  591,  487,  739832,  739846,  739871, 14, 25, 24, }, // Chatham
    {  591,  495,  739895,  739907,  739930, 12, 23, 30, }, // Chile
    {  591,  501,  739960,  739973,  739997, 13, 24, 23, }, // China
    {  591,  507,       0,  740020,       0,  0, 26,  0, }, // Christmas
    {  591,  517,       0,  740046,       0,  0, 23,  0, }, // Cocos
    {  591,  523,  740069,  740084,  740110, 15, 26, 33, }, // Colombia
    {  591,  532,  740143,  740165,  740198, 22, 33, 41, }, // Cook
    {  591,  537,  740239,  740250,  740272, 11, 22, 21, }, // Cuba
    {  591,  548,       0,  740293,       0,  0, 12,  0, }, // Davis
    {  591,  564,       0,  740305,       0,  0, 23,  0, }, // DumontDUrville
    {  591,  601,       0,  740328,       0,  0, 22,  0, }, // East_Timor
    {  591,  612,  740350,  740374,  740409, 24, 35, 42, }, // Easter
    {  591,  619,       0,  740451,       0,  0, 13,  0, }, // Ecuador
    {  591,  627,  740464,  740484,  740515, 20, 31, 38, }, // Europe_Central
    {  591,  642,  740553,  740579,  740616, 26, 37, 44, }, // Europe_Eastern
    {  591,  657,       0,  740660,       0,  0, 32,  0, }, // Europe_Further_Eastern
    {  591,  680,  740692,  740718,  740755, 26, 37, 44, }, // Europe_Western
    {  591,  695,  740799,  740825,  740862, 26, 37, 44, }, // Falkland
    {  591,  704,  740906,  740917,  740939, 11, 22, 29, }, // Fiji
    {  591,  709,       0,  740968,       0,  0, 25,  0, }, // French_Guiana
    {  591,  723,       0,  740993,       0,  0, 39,  0, }, // French_Southern
    {  591,  746,       0,  741032,       0,  0, 16,  0, }, // Galapagos
    {  591,  756,       0,  741048,       0,  0, 14,  0, }, // Gambier
    {  591,  764,  741062,  741074,  741097, 12, 23, 30, }, // Georgia
    {  591,  772,       0,  741127,       0,  0, 25,  0, }, // Gilbert_Islands
    {  591,  788,       0,  741152,       0,  0, 16,  0, }, // GMT
    {  591,  830,  741168,  741194,  741231, 26, 37, 44, }, // Greenland_Eastern
    {  591,  848,  741275,  741301,  741338, 26, 37, 44, }, // Greenland_Western
    {  591,  871,       0,  741382,       0,  0, 22,  0, }, // Gulf
    {  591,  876,       0,  741404,       0,  0, 13,  0, }, // Guyana
    {  591,  883,  741417,  741439,  741472, 22, 33, 32, }, // Hawaii_Aleutian
    {  591,  899,  741504,  741520,  741547, 16, 27, 34, }, // Hong_Kong
    {  591,  909,  741581,  741592,  741614, 11, 22, 29, }, // Hovd
    {  591,  914,       0,  741643,       0,  0, 23,  0, }, // India
    {  591,  920,       0,  741666,       0,  0, 19,  0, }, // Indian_Ocean
    {  591,  933,       0,  741685,       0,  0, 16,  0, }, // Indochina
    {  591,  943,       0,  741701,       0,  0, 24,  0, }, // Indonesia_Central
    {  591,  961,       0,  741725,       0,  0, 30,  0, }, // Indonesia_Eastern
    {  591,  979,       0,  741755,       0,  0, 30,  0, }, // Indonesia_Western
    {  591,  997,  741785,  741796,  741818, 11, 22, 21, }, // Iran
    {  591, 1008,  741839,  741853,  741878, 14, 25, 32, }, // Irkutsk
    {  591, 1016,  741910,  741924,  741949, 14, 25, 24, }, // Israel
    {  591, 1023,  741973,  741985,  742009, 12, 24, 22, }, // Japan
    {  591, 1029,  742031,  742062,  742104, 31, 42, 43, }, // Kamchatka
    {  591, 1047,       0,  742147,       0,  0, 17,  0, }, // Kazakhstan
    {  591, 1058,       0,  742164,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  591, 1077,       0,  742191,       0,  0, 27,  0, }, // Kazakhstan_Western
    {  591, 1106,  742218,  742230,  742253, 12, 23, 22, }, // Korea
    {  591, 1112,       0,  742275,       0,  0, 13,  0, }, // Kosrae
    {  591, 1119,  742288,  742306,  742334, 18, 28, 36, }, // Krasnoyarsk
    {  591, 1151,       0,  742370,       0,  0, 16,  0, }, // Kyrgystan
    {  591, 1175,       0,  742386,       0,  0, 22,  0, }, // Line_Islands
    {  591, 1188,  742408,  742424,  742451, 16, 27, 26, }, // Lord_Howe
    {  591, 1204,  742477,  742491,  742516, 14, 25, 32, }, // Magadan
    {  591, 1219,       0,  742548,       0,  0, 15,  0, }, // Malaysia
    {  591, 1228,       0,  742563,       0,  0, 15,  0, }, // Maldives
    {  591, 1237,       0,  742578,       0,  0, 16,  0, }, // Marquesas
    {  591, 1247,       0,  742594,       0,  0, 26,  0, }, // Marshall_Islands
    {  591, 1264,  742620,  742633,  742657, 13, 24, 31, }, // Mauritius
    {  591, 1274,       0,  742688,       0,  0, 13,  0, }, // Mawson
    {  591, 1281,  742701,  742727,  742764, 26, 37, 36, }, // Mexico_Pacific
    {  591, 1296,  742800,  742817,  742845, 17, 28, 35, }, // Mongolia
    {  591, 1305,  742880,  742893,  742917, 13, 24, 31, }, // Moscow
    {  591, 1312,       0,  742948,       0,  0, 14,  0, }, // Myanmar
    {  591, 1320,       0,  742962,       0,  0, 12,  0, }, // Nauru
    {  591, 1326,       0,  742974,       0,  0, 12,  0, }, // Nepal
    {  591, 1332,  742986,  743006,  743037, 20, 31, 38, }, // New_Caledonia
    {  591, 1346,  743075,  743093,  743122, 18, 29, 28, }, // New_Zealand
    {  591, 1358,  743150,  743169,  743199, 19, 30, 29, }, // Newfoundland
    {  591, 1371,       0,  743228,       0,  0, 11,  0, }, // Niue
    {  591, 1376,  743239,  743264,  743300, 25, 36, 43, }, // Norfolk
    {  591, 1384,  743343,  743369,  743406, 26, 37, 44, }, // Noronha
    {  591, 1406,  743450,  743468,  743497, 18, 29, 36, }, // Novosibirsk
    {  591, 1418,  743533,  743544,  743566, 11, 22, 29, }, // Omsk
    {  591, 1428,  743595,  743610,  743636, 15, 26, 33, }, // Pakistan
    {  591, 1437,       0,  743669,       0,  0, 12,  0, }, // Palau
    {  591, 1443,       0,  743681,       0,  0, 23,  0, }, // Papua_New_Guinea
    {  591, 1460,  743704,  743719,  743745, 15, 26, 33, }, // Paraguay
    {  591, 1469,  743778,  743789,  743811, 11, 22, 29, }, // Peru
    {  591, 1474,  743840,  743856,  743883, 16, 27, 34, }, // Philippines
    {  591, 1486,       0,  743917,       0,  0, 25,  0, }, // Phoenix_Islands
    {  591, 1502,  743942,  743973,  744015, 31, 42, 41, }, // Pierre_Miquelon
    {  591, 1518,       0,  744056,       0,  0, 15,  0, }, // Pitcairn
    {  591, 1527,       0,  744071,       0,  0, 13,  0, }, // Ponape
    {  591, 1534,       0,  744084,       0,  0, 16,  0, }, // Pyongyang
    {  591, 1554,       0,  744100,       0,  0, 14,  0, }, // Reunion
    {  591, 1562,       0,  744114,       0,  0, 14,  0, }, // Rothera
    {  591, 1570,  744128,  744143,  744169, 15, 26, 33, }, // Sakhalin
    {  591, 1579,  744202,  744215,  744239, 13, 24, 25, }, // Samara
    {  591, 1596,  744264,  744276,  744299, 12, 23, 30, }, // Samoa
    {  591, 1602,       0,  744329,       0,  0, 18,  0, }, // Seychelles
    {  591, 1624,       0,  744347,       0,  0, 27,  0, }, // Singapore
    {  591, 1634,       0,  744374,       0,  0, 25,  0, }, // Solomon
    {  591, 1642,       0,  744399,       0,  0, 21,  0, }, // South_Georgia
    {  591, 1656,       0,  744420,       0,  0, 15,  0, }, // Suriname
    {  591, 1676,       0,  744435,       0,  0, 12,  0, }, // Syowa
    {  591, 1682,       0,  744447,       0,  0, 13,  0, }, // Tahiti
    {  591, 1689,  744460,  744473,  744497, 13, 24, 23, }, // Taipei
    {  591, 1696,       0,  744520,       0,  0, 17,  0, }, // Tajikistan
    {  591, 1724,       0,  744537,       0,  0, 14,  0, }, // Tokelau
    {  591, 1732,  744551,  744563,  744586, 12, 23, 30, }, // Tonga
    {  591, 1738,       0,  744616,       0,  0, 12,  0, }, // Truk
    {  591, 1750,  744628,  744647,  744677, 19, 30, 37, }, // Turkmenistan
    {  591, 1763,       0,  744714,       0,  0, 13,  0, }, // Tuvalu
    {  591, 1777,  744727,  744741,  744766, 14, 25, 32, }, // Uruguay
    {  591, 1792,  744798,  744815,  744843, 17, 28, 35, }, // Uzbekistan
    {  591, 1803,  744878,  744892,  744917, 14, 25, 32, }, // Vanuatu
    {  591, 1811,       0,  744949,       0,  0, 16,  0, }, // Venezuela
    {  591, 1821,  744965,  744983,  745012, 18, 29, 36, }, // Vladivostok
    {  591, 1833,  745048,  745064,  745091, 16, 27, 34, }, // Volgograd
    {  591, 1843,       0,  745125,       0,  0, 13,  0, }, // Vostok
    {  591, 1850,       0,  745138,       0,  0, 22,  0, }, // Wake
    {  591, 1855,       0,  745160,       0,  0, 23,  0, }, // Wallis
    {  591, 1862,  745183,  745197,  745222, 14, 25, 32, }, // Yakutsk
    {  591, 1870,  745254,  745274,  745305, 20, 31, 38, }, // Yekaterinburg
    {  591, 1892,       0,  745343,       0,  0, 12,  0, }, // Yukon
    {  593,    5,       0,  745355,       0,  0, 19,  0, }, // Afghanistan Swahili/Latin/Kenya
    {  593,   81,  737943,  737966,  745374, 23, 34, 41, }, // Africa_Western
    {  593,  135,  738101,  738114,  745415, 13, 24, 31, }, // Amazon
    {  593,  240,  745446,  745461,  745487, 15, 26, 30, }, // Argentina
    {  593,  250,  745517,  745546,  745586, 29, 40, 47, }, // Argentina_Western
    {  593,  268,  738802,  738816,  745633, 14, 25, 32, }, // Armenia
    {  593,  313,  745665,  745704,  745753, 39, 49, 48, }, // Australia_CentralWestern
    {  593,  374,  745801,  745817,  745844, 16, 27, 34, }, // Azerbaijan
    {  593,  385,  739453,  739466,  745878, 13, 24, 31, }, // Azores
    {  593,  397,  745909,  745927,  745956, 18, 29, 36, }, // Bangladesh
    {  593,  415,       0,  745992,       0,  0, 13,  0, }, // Bhutan
    {  593,  437,  746005,  746019,  746044, 14, 25, 32, }, // Brasilia
    {  593,  461,  746076,  746091,  746117, 15, 26, 33, }, // Cape_Verde
    {  593,  523,  740069,  740084,  746150, 15, 26, 33, }, // Colombia
    {  593,  532,  740143,  740165,  746183, 22, 33, 45, }, // Cook
    {  593,  537,  740239,  746228,  740272, 11, 22, 21, }, // Cuba
    {  593,  612,  740350,  740374,  746250, 24, 35, 42, }, // Easter
    {  593,  627,  740464,  740484,  746292, 20, 31, 38, }, // Europe_Central
    {  593,  642,  740553,  740579,  746330, 26, 37, 44, }, // Europe_Eastern
    {  593,  680,  740692,  740718,  746374, 26, 37, 44, }, // Europe_Western
    {  593,  709,       0,  740968,       0,  0, 13,  0, }, // French_Guiana
    {  593,  764,  746418,  746431,  746455, 13, 24, 31, }, // Georgia
    {  593,  830,  741168,  741194,  746486, 26, 37, 44, }, // Greenland_Eastern
    {  593,  871,       0,  746530,       0,  0, 23,  0, }, // Gulf
    {  593,  899,  741504,  741520,  746553, 16, 27, 34, }, // Hong_Kong
    {  593,  909,  741581,  741592,  746587, 11, 22, 29, }, // Hovd
    {  593,  997,  746616,  746628,  746651, 12, 23, 22, }, // Iran
    {  593, 1008,  741839,  741853,  746673, 14, 25, 32, }, // Irkutsk
    {  593, 1023,  746705,  741985,  746718, 13, 24, 23, }, // Japan
    {  593, 1058,       0,  746741,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  593, 1077,       0,  746769,       0,  0, 28,  0, }, // Kazakhstan_Western
    {  593, 1119,  742288,  742306,  746797, 18, 28, 36, }, // Krasnoyarsk
    {  593, 1204,  742477,  742491,  746833, 14, 25, 32, }, // Magadan
    {  593, 1219,       0,  746865,       0,  0, 14,  0, }, // Malaysia
    {  593, 1228,       0,  746879,       0,  0, 14,  0, }, // Maldives
    {  593, 1264,  742620,  742633,  746893, 13, 24, 31, }, // Mauritius
    {  593, 1296,  746924,  746942,  746971, 18, 29, 36, }, // Mongolia
    {  593, 1305,  742880,  742893,  747007, 13, 24, 31, }, // Moscow
    {  593, 1312,       0,  742948,       0,  0, 13,  0, }, // Myanmar
    {  593, 1326,       0,  747038,       0,  0, 13,  0, }, // Nepal
    {  593, 1332,  747051,  747072,  747104, 21, 32, 39, }, // New_Caledonia
    {  593, 1346,  747143,  747160,  747188, 17, 28, 27, }, // New_Zealand
    {  593, 1376,  743239,  743264,  747215, 25, 36, 43, }, // Norfolk
    {  593, 1406,  743450,  743468,  747258, 18, 29, 36, }, // Novosibirsk
    {  593, 1418,  743533,  743544,  747294, 11, 22, 29, }, // Omsk
    {  593, 1428,  747323,  747339,  747366, 16, 27, 34, }, // Pakistan
    {  593, 1443,       0,  743681,       0,  0, 12,  0, }, // Papua_New_Guinea
    {  593, 1460,  743704,  743719,  747400, 15, 26, 33, }, // Paraguay
    {  593, 1469,  743778,  743789,  747433, 11, 22, 29, }, // Peru
    {  593, 1474,  743840,  743856,  747462, 16, 27, 34, }, // Philippines
    {  593, 1486,       0,  747496,       0,  0, 25,  0, }, // Phoenix_Islands
    {  593, 1570,  744128,  744143,  747521, 15, 26, 33, }, // Sakhalin
    {  593, 1596,  744264,  744276,  747554, 12, 23, 22, }, // Samoa
    {  593, 1624,       0,  747576,       0,  0, 26,  0, }, // Singapore
    {  593, 1642,       0,  747602,       0,  0, 19,  0, }, // South_Georgia
    {  593, 1696,       0,  747621,       0,  0, 17,  0, }, // Tajikistan
    {  593, 1732,  744551,  744563,  747638, 12, 23, 30, }, // Tonga
    {  593, 1750,  747668,  747688,  747719, 20, 31, 38, }, // Turkmenistan
    {  593, 1777,  744727,  744741,  747757, 14, 25, 32, }, // Uruguay
    {  593, 1792,  747789,  747807,  747836, 18, 29, 36, }, // Uzbekistan
    {  593, 1803,  744878,  744892,  747872, 14, 25, 32, }, // Vanuatu
    {  593, 1821,  744965,  744983,  747904, 18, 29, 36, }, // Vladivostok
    {  593, 1833,  745048,  745064,  747940, 16, 27, 34, }, // Volgograd
    {  593, 1862,  745183,  745197,  747974, 14, 25, 32, }, // Yakutsk
    {  593, 1870,  745254,  745274,  748006, 20, 31, 38, }, // Yekaterinburg
    {  597,    0,  748044,  748063,  748088, 19, 25, 25, }, // Acre Swedish/Latin/Sweden
    {  597,    5,       0,  558777,       0,  0, 12,  0, }, // Afghanistan
    {  597,   17,       0,  748113,       0,  0, 20,  0, }, // Africa_Central
    {  597,   32,       0,  748133,       0,  0, 16,  0, }, // Africa_Eastern
    {  597,   65,       0,  748149,       0,  0, 16,  0, }, // Africa_Southern
    {  597,   81,  748165,  748182,  748205, 17, 23, 23, }, // Africa_Western
    {  597,  107,  748228,  748237,  748254,  9, 17, 17, }, // Alaska
    {  597,  128,  748271,  748280,  748295,  9, 15, 15, }, // Almaty
    {  597,  135,  559045,  748310,  748329, 11, 19, 19, }, // Amazon
    {  597,  142,  748348,  748373,  748404, 25, 31, 31, }, // America_Central
    {  597,  158,  748435,  748456,  748483, 21, 27, 27, }, // America_Eastern
    {  597,  174,  748510,  748528,  748554, 18, 26, 26, }, // America_Mountain
    {  597,  191,  748580,  748602,  748630, 22, 28, 28, }, // America_Pacific
    {  597,  207,  748658,  748667,  748682,  9, 15, 15, }, // Anadyr
    {  597,  214,  748697,  748704,  748719,  7, 15, 15, }, // Apia
    {  597,  219,  748734,  748742,  748756,  8, 14, 14, }, // Aqtau
    {  597,  225,  748770,  748779,  748794,  9, 15, 15, }, // Aqtobe
    {  597,  232,  748809,  748825,  748847, 16, 22, 22, }, // Arabian
    {  597,  240,  748869,  748886,  748909, 17, 23, 23, }, // Argentina
    {  597,  250,  748932,  748950,  748974, 18, 24, 24, }, // Argentina_Western
    {  597,  268,  748998,  749010,  749028, 12, 18, 18, }, // Armenia
    {  597,  286,  749046,  749070,  749100, 24, 30, 30, }, // Atlantic
    {  597,  295,  749130,  749151,  749178, 21, 27, 27, }, // Australia_Central
    {  597,  313,  749205,  749230,  749261, 25, 31, 31, }, // Australia_CentralWestern
    {  597,  338,  749292,  749309,  749332, 17, 23, 23, }, // Australia_Eastern
    {  597,  356,  749355,  749373,  749397, 18, 24, 24, }, // Australia_Western
    {  597,  374,  749421,  749439,  749463, 18, 24, 24, }, // Azerbaijan
    {  597,  385,  749487,  749498,  749515, 11, 17, 17, }, // Azores
    {  597,  397,  560451,  560468,  566548, 17, 23, 23, }, // Bangladesh
    {  597,  415,       0,  560514,       0,  0, 12,  0, }, // Bhutan
    {  597,  422,       0,  560526,       0,  0, 14,  0, }, // Bolivia
    {  597,  437,  560597,  749532,  749551, 11, 19, 19, }, // Brasilia
    {  597,  454,       0,  749570,       0,  0,  9,  0, }, // Brunei
    {  597,  461,  749579,  749591,  749611, 12, 20, 20, }, // Cape_Verde
    {  597,  472,       0,  749631,       0,  0,  8,  0, }, // Casey
    {  597,  478,       0,  560714,       0,  0, 11,  0, }, // Chamorro
    {  597,  487,  749639,  749649,  749667, 10, 18, 18, }, // Chatham
    {  597,  495,  560784,  560796,  566635, 12, 18, 18, }, // Chile
    {  597,  501,  560832,  560844,  566653, 12, 18, 18, }, // China
    {  597,  507,       0,  749685,       0,  0, 10,  0, }, // Christmas
    {  597,  517,       0,  749695,       0,  0, 17,  0, }, // Cocos
    {  597,  523,  560928,  560943,  749712, 15, 21, 21, }, // Colombia
    {  597,  532,  749733,  749747,  749767, 14, 20, 20, }, // Cook
    {  597,  537,  566819,  566830,  566847, 11, 17, 17, }, // Cuba
    {  597,  548,       0,  561116,       0,  0,  8,  0, }, // Davis
    {  597,  564,       0,  749787,       0,  0, 20,  0, }, // DumontDUrville
    {  597,  601,       0,  749807,       0,  0, 15,  0, }, // East_Timor
    {  597,  612,  749822,  749830,  749847,  8, 17, 17, }, // Easter
    {  597,  619,       0,  561232,       0,  0, 16,  0, }, // Ecuador
    {  597,  627,  749864,  749884,  749910, 20, 26, 26, }, // Europe_Central
    {  597,  642,  749936,  749952,  749974, 16, 22, 22, }, // Europe_Eastern
    {  597,  657,       0,  749996,       0,  0, 14,  0, }, // Europe_Further_Eastern
    {  597,  680,  750010,  750027,  750050, 17, 23, 23, }, // Europe_Western
    {  597,  695,  750073,  750092,  750117, 19, 25, 25, }, // Falkland
    {  597,  704,  750142,  750149,  750164,  7, 15, 15, }, // Fiji
    {  597,  709,       0,  750179,       0,  0, 17,  0, }, // French_Guiana
    {  597,  723,       0,  750196,       0,  0, 29,  0, }, // French_Southern
    {  597,  746,       0,  750225,       0,  0, 12,  0, }, // Galapagos
    {  597,  756,       0,  750237,       0,  0, 10,  0, }, // Gambier
    {  597,  764,  561707,  561719,  567289, 12, 18, 18, }, // Georgia
    {  597,  772,       0,  750247,       0,  0, 11,  0, }, // Gilbert_Islands
    {  597,  788,       0,  750258,       0,  0, 12,  0, }, // GMT
    {  597,  830,  750270,  750287,  750310, 17, 23, 23, }, // Greenland_Eastern
    {  597,  848,  750333,  750351,  750375, 18, 24, 24, }, // Greenland_Western
    {  597,  866,       0,  750399,       0,  0,  7,  0, }, // Guam
    {  597,  871,       0,  750406,       0,  0, 17,  0, }, // Gulf
    {  597,  876,       0,  561616,       0,  0,  9,  0, }, // Guyana
    {  597,  883,  750423,  750434,  750453, 11, 19, 19, }, // Hawaii_Aleutian
    {  597,  899,  562123,  750472,  750491, 11, 19, 19, }, // Hong_Kong
    {  597,  909,  750510,  750518,  750534,  8, 16, 16, }, // Hovd
    {  597,  914,       0,  562187,       0,  0, 10,  0, }, // India
    {  597,  920,       0,  750550,       0,  0, 33,  0, }, // Indian_Ocean
    {  597,  933,       0,  562220,       0,  0, 16,  0, }, // Indochina
    {  597,  943,       0,  750583,       0,  0, 21,  0, }, // Indonesia_Central
    {  597,  961,       0,  750604,       0,  0, 17,  0, }, // Indonesia_Eastern
    {  597,  979,       0,  750621,       0,  0, 18,  0, }, // Indonesia_Western
    {  597,  997,  562292,  562302,  567563, 10, 16, 16, }, // Iran
    {  597, 1008,  750639,  750649,  750667, 10, 18, 18, }, // Irkutsk
    {  597, 1016,  750685,  750698,  750717, 13, 19, 19, }, // Israel
    {  597, 1023,  562444,  562455,  567618, 11, 17, 17, }, // Japan
    {  597, 1029,  750736,  750748,  750766, 12, 18, 18, }, // Kamchatka
    {  597, 1047,       0,  750784,       0,  0, 15,  0, }, // Kazakhstan
    {  597, 1058,       0,  750799,       0,  0, 18,  0, }, // Kazakhstan_Eastern
    {  597, 1077,       0,  750817,       0,  0, 19,  0, }, // Kazakhstan_Western
    {  597, 1106,  562660,  562672,  567655, 12, 18, 18, }, // Korea
    {  597, 1112,       0,  562721,       0,  0,  9,  0, }, // Kosrae
    {  597, 1119,  567687,  750836,  750858, 14, 22, 22, }, // Krasnoyarsk
    {  597, 1151,       0,  750880,       0,  0, 13,  0, }, // Kyrgystan
    {  597, 1161,       0,  750893,       0,  0, 12,  0, }, // Lanka
    {  597, 1175,       0,  750905,       0,  0, 14,  0, }, // Line_Islands
    {  597, 1188,  750919,  750931,  750951, 12, 20, 20, }, // Lord_Howe
    {  597, 1198,  750971,  750979,  750993,  8, 14, 14, }, // Macau
    {  597, 1204,  567761,  751007,  751025, 10, 18, 18, }, // Magadan
    {  597, 1219,       0,  563031,       0,  0, 13,  0, }, // Malaysia
    {  597, 1228,       0,  751043,       0,  0, 13,  0, }, // Maldives
    {  597, 1237,       0,  751056,       0,  0, 12,  0, }, // Marquesas
    {  597, 1247,       0,  751068,       0,  0, 18,  0, }, // Marshall_Islands
    {  597, 1264,  751086,  751098,  751118, 12, 20, 20, }, // Mauritius
    {  597, 1274,       0,  563165,       0,  0,  9,  0, }, // Mawson
    {  597, 1281,  751138,  751161,  751195, 23, 34, 34, }, // Mexico_Pacific
    {  597, 1296,  751229,  751243,  751265, 14, 22, 22, }, // Mongolia
    {  597, 1305,  751287,  751296,  751313,  9, 17, 17, }, // Moscow
    {  597, 1312,       0,  751330,       0,  0, 13,  0, }, // Myanmar
    {  597, 1320,       0,  751343,       0,  0,  8,  0, }, // Nauru
    {  597, 1326,       0,  751351,       0,  0, 14,  0, }, // Nepal
    {  597, 1332,  751365,  751382,  751407, 17, 25, 25, }, // New_Caledonia
    {  597, 1346,  751432,  751447,  751468, 15, 21, 21, }, // New_Zealand
    {  597, 1358,  563649,  751489,  751512, 15, 23, 23, }, // Newfoundland
    {  597, 1371,       0,  563674,       0,  0,  7,  0, }, // Niue
    {  597, 1376,  751535,  751549,  751569, 14, 20, 20, }, // Norfolk
    {  597, 1384,  751589,  751611,  751641, 22, 30, 30, }, // Noronha
    {  597, 1392,       0,  751671,       0,  0, 19,  0, }, // North_Mariana
    {  597, 1406,  563928,  751690,  751712, 14, 22, 22, }, // Novosibirsk
    {  597, 1418,  751734,  751741,  751756,  7, 15, 15, }, // Omsk
    {  597, 1428,  563992,  564006,  568197, 14, 20, 20, }, // Pakistan
    {  597, 1437,       0,  751771,       0,  0,  8,  0, }, // Palau
    {  597, 1443,       0,  751779,       0,  0, 21,  0, }, // Papua_New_Guinea
    {  597, 1460,  564070,  564086,  568217, 16, 22, 22, }, // Paraguay
    {  597, 1469,  564130,  564142,  568239, 12, 18, 18, }, // Peru
    {  597, 1474,  564178,  564192,  568257, 14, 20, 20, }, // Philippines
    {  597, 1486,       0,  751800,       0,  0, 12,  0, }, // Phoenix_Islands
    {  597, 1502,  751812,  751840,  751875, 28, 35, 35, }, // Pierre_Miquelon
    {  597, 1518,       0,  564383,       0,  0, 11,  0, }, // Pitcairn
    {  597, 1527,       0,  751910,       0,  0,  9,  0, }, // Ponape
    {  597, 1534,       0,  751919,       0,  0, 12,  0, }, // Pyongyang
    {  597, 1544,  751931,  751943,  751961, 12, 18, 18, }, // Qyzylorda
    {  597, 1554,       0,  564488,       0,  0, 10,  0, }, // Reunion
    {  597, 1562,       0,  564508,       0,  0, 10,  0, }, // Rothera
    {  597, 1570,  751979,  751990,  752009, 11, 19, 19, }, // Sakhalin
    {  597, 1579,  752028,  752037,  752052,  9, 15, 15, }, // Samara
    {  597, 1596,  564652,  564664,  568362, 12, 18, 18, }, // Samoa
    {  597, 1602,       0,  752067,       0,  0, 15,  0, }, // Seychelles
    {  597, 1624,       0,  752082,       0,  0, 12,  0, }, // Singapore
    {  597, 1634,       0,  752094,       0,  0, 17,  0, }, // Solomon
    {  597, 1642,       0,  752111,       0,  0, 15,  0, }, // South_Georgia
    {  597, 1656,       0,  752126,       0,  0, 10,  0, }, // Suriname
    {  597, 1676,       0,  752136,       0,  0,  8,  0, }, // Syowa
    {  597, 1682,       0,  752144,       0,  0,  9,  0, }, // Tahiti
    {  597, 1689,  752153,  752162,  752179,  9, 17, 17, }, // Taipei
    {  597, 1696,       0,  752196,       0,  0, 15,  0, }, // Tajikistan
    {  597, 1724,       0,  752211,       0,  0, 10,  0, }, // Tokelau
    {  597, 1732,  752221,  752229,  752245,  8, 16, 16, }, // Tonga
    {  597, 1738,       0,  752261,       0,  0,  8,  0, }, // Truk
    {  597, 1750,  564973,  564986,  568441, 13, 19, 19, }, // Turkmenistan
    {  597, 1763,       0,  752269,       0,  0,  9,  0, }, // Tuvalu
    {  597, 1777,  565035,  565050,  568460, 15, 21, 21, }, // Uruguay
    {  597, 1792,  752278,  752290,  752308, 12, 18, 18, }, // Uzbekistan
    {  597, 1803,  752326,  752336,  752354, 10, 18, 18, }, // Vanuatu
    {  597, 1811,       0,  565191,       0,  0, 16,  0, }, // Venezuela
    {  597, 1821,  565270,  752372,  752394, 14, 22, 22, }, // Vladivostok
    {  597, 1833,  565340,  752416,  752436, 12, 20, 20, }, // Volgograd
    {  597, 1843,       0,  565362,       0,  0,  9,  0, }, // Vostok
    {  597, 1850,       0,  752456,       0,  0, 14,  0, }, // Wake
    {  597, 1855,       0,  752470,       0,  0, 28,  0, }, // Wallis
    {  597, 1862,  565482,  752498,  752516, 10, 18, 18, }, // Yakutsk
    {  597, 1870,  565556,  752534,  752558, 16, 24, 24, }, // Yekaterinburg
    {  597, 1892,       0,  752582,       0,  0,  8,  0, }, // Yukon
    {  600,    0,  752590,  752599,  752618,  9, 19, 15, }, // Acre Swiss German/Latin/Switzerland
    {  600,    5,       0,  752633,       0,  0, 18,  0, }, // Afghanistan
    {  600,   17,       0,  752651,       0,  0, 24,  0, }, // Africa_Central
    {  600,   32,       0,  752675,       0,  0, 22,  0, }, // Africa_Eastern
    {  600,   65,       0,  752697,       0,  0, 21,  0, }, // Africa_Southern
    {  600,   81,  752718,  752741,  752774, 23, 33, 29, }, // Africa_Western
    {  600,  107,  752803,  752814,  752835, 11, 21, 17, }, // Alaska
    {  600,  128,  752852,  752863,  752884, 11, 21, 17, }, // Almaty
    {  600,  135,  752901,  752914,  752937, 13, 23, 19, }, // Amazon
    {  600,  142,  752956,  752977,  753008, 21, 31, 27, }, // America_Central
    {  600,  627,  753035,  753057,  753089, 22, 32, 28, }, // Europe_Central
    {  600,  642,  753117,  753138,  753169, 21, 31, 27, }, // Europe_Eastern
    {  600,  680,  753196,  753218,  753250, 22, 32, 28, }, // Europe_Western
    {  600, 1305,  753278,  753291,  753314, 13, 23, 19, }, // Moscow
    {  603,    0,  753333,  753346,  753380, 13, 34, 20, }, // Acre Syriac/Syriac/Iraq
    {  603,    5,       0,  753400,       0,  0, 15,  0, }, // Afghanistan
    {  603,   17,       0,  753415,       0,  0, 19,  0, }, // Africa_Central
    {  603,   32,       0,  753434,       0,  0, 17,  0, }, // Africa_Eastern
    {  603,   65,       0,  753451,       0,  0, 27,  0, }, // Africa_Southern
    {  603,   81,  753478,  753495,  753522, 17, 27, 24, }, // Africa_Western
    {  603,  107,  753546,  753558,  753580, 12, 22, 22, }, // Alaska
    {  603,  128,  753602,  753614,  753636, 12, 22, 22, }, // Almaty
    {  603,  135,  753658,  753668,  753688, 10, 20, 20, }, // Amazon
    {  603,  142,  753708,  753735,  753772, 27, 37, 37, }, // America_Central
    {  603,  158,  753809,  753837,  753875, 28, 38, 38, }, // America_Eastern
    {  603,  174,  753913,  753940,  753977, 27, 37, 37, }, // America_Mountain
    {  603,  191,  754014,  754041,  754078, 27, 37, 37, }, // America_Pacific
    {  603,  207,  754115,  754127,  754149, 12, 22, 22, }, // Anadyr
    {  603,  214,  754171,  754181,  754201, 10, 20, 20, }, // Apia
    {  603,  219,  754221,  754231,  754252, 10, 21, 21, }, // Aqtau
    {  603,  225,  754273,  754284,  754305, 11, 21, 21, }, // Aqtobe
    {  603,  232,  754326,  754337,  754358, 11, 21, 21, }, // Arabian
    {  603,  240,  754379,  754393,  754417, 14, 24, 24, }, // Argentina
    {  603,  250,  754441,  754463,  754495, 22, 32, 32, }, // Argentina_Western
    {  603,  268,  754527,  754539,  754561, 12, 22, 22, }, // Armenia
    {  603,  286,  754583,  754612,  754651, 29, 39, 39, }, // Atlantic
    {  603,  295,  754690,  754711,  754742, 21, 31, 31, }, // Australia_Central
    {  603,  313,  754773,  754802,  754841, 29, 39, 39, }, // Australia_CentralWestern
    {  603,  338,  754880,  754902,  754934, 22, 32, 32, }, // Australia_Eastern
    {  603,  356,  754966,  754988,  755020, 22, 32, 32, }, // Australia_Western
    {  603,  374,  755052,  755066,  755090, 14, 24, 24, }, // Azerbaijan
    {  603,  385,  755114,  755124,  755144, 10, 20, 20, }, // Azores
    {  603,  397,  755164,  755178,  755202, 14, 24, 24, }, // Bangladesh
    {  603,  415,       0,  755226,       0,  0, 11,  0, }, // Bhutan
    {  603,  422,       0,  755237,       0,  0, 13,  0, }, // Bolivia
    {  603,  437,  755250,  755264,  755288, 14, 24, 24, }, // Brasilia
    {  603,  454,       0,  755312,       0,  0, 21,  0, }, // Brunei
    {  603,  461,  755333,  755348,  755373, 15, 25, 25, }, // Cape_Verde
    {  603,  472,       0,  755398,       0,  0, 11,  0, }, // Casey
    {  603,  478,       0,  755409,       0,  0, 23,  0, }, // Chamorro
    {  603,  487,  755432,  755443,  755464, 11, 21, 21, }, // Chatham
    {  603,  495,  755485,  755496,  755517, 11, 21, 21, }, // Chile
    {  603,  501,  755538,  755547,  755566,  9, 19, 19, }, // China
    {  603,  507,       0,  755585,       0,  0, 20,  0, }, // Christmas
    {  603,  517,       0,  755605,       0,  0, 19,  0, }, // Cocos
    {  603,  523,  755624,  755638,  755662, 14, 24, 24, }, // Colombia
    {  603,  532,  755686,  755702,  755728, 16, 26, 26, }, // Cook
    {  603,  537,  755754,  755764,  755784, 10, 20, 20, }, // Cuba
    {  603,  548,       0,  755804,       0,  0, 10,  0, }, // Davis
    {  603,  564,       0,  755814,       0,  0, 20,  0, }, // DumontDUrville
    {  603,  601,       0,  755834,       0,  0, 16,  0, }, // East_Timor
    {  603,  612,  755850,  755866,  755892, 16, 26, 26, }, // Easter
    {  603,  619,       0,  755918,       0,  0, 13,  0, }, // Ecuador
    {  603,  627,  755931,  755950,  755979, 19, 29, 29, }, // Europe_Central
    {  603,  642,  756008,  756028,  756058, 20, 30, 30, }, // Europe_Eastern
    {  603,  657,       0,  756088,       0,  0, 23,  0, }, // Europe_Further_Eastern
    {  603,  680,  756111,  756131,  756161, 20, 30, 30, }, // Europe_Western
    {  603,  695,  756191,  756211,  756241, 20, 30, 30, }, // Falkland
    {  603,  704,  756271,  756281,  756301, 10, 20, 20, }, // Fiji
    {  603,  709,       0,  756321,       0,  0, 20,  0, }, // French_Guiana
    {  603,  723,       0,  756341,       0,  0, 29,  0, }, // French_Southern
    {  603,  746,       0,  756370,       0,  0, 14,  0, }, // Galapagos
    {  603,  756,       0,  756384,       0,  0, 11,  0, }, // Gambier
    {  603,  764,  756395,  756407,  756429, 12, 22, 22, }, // Georgia
    {  603,  772,       0,  756451,       0,  0, 20,  0, }, // Gilbert_Islands
    {  603,  788,       0,  756471,       0,  0, 11,  0, }, // GMT
    {  603,  830,  756482,  756502,  756532, 20, 30, 30, }, // Greenland_Eastern
    {  603,  848,  756562,  756584,  756616, 22, 32, 32, }, // Greenland_Western
    {  603,  866,       0,  756648,       0,  0, 20,  0, }, // Guam
    {  603,  871,       0,  756668,       0,  0, 21,  0, }, // Gulf
    {  603,  876,       0,  756321,       0,  0, 12,  0, }, // Guyana
    {  603,  883,  756689,  756708,  756737, 19, 29, 29, }, // Hawaii_Aleutian
    {  603,  899,  756766,  756781,  756806, 15, 25, 25, }, // Hong_Kong
    {  603,  909,  756831,  756836,  756856,  5, 20, 20, }, // Hovd
    {  603,  914,       0,  756876,       0,  0, 20,  0, }, // India
    {  603,  920,       0,  756896,       0,  0, 20,  0, }, // Indian_Ocean
    {  603,  933,       0,  756916,       0,  0, 13,  0, }, // Indochina
    {  603,  943,       0,  756929,       0,  0, 20,  0, }, // Indonesia_Central
    {  603,  961,       0,  756949,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  603,  979,       0,  756970,       0,  0, 21,  0, }, // Indonesia_Western
    {  603,  997,  756991,  757002,  757023, 11, 21, 21, }, // Iran
    {  603, 1008,  757044,  757058,  757082, 14, 24, 24, }, // Irkutsk
    {  603, 1016,  757106,  757119,  757142, 13, 23, 23, }, // Israel
    {  603, 1023,  757165,  757174,  757193,  9, 19, 19, }, // Japan
    {  603, 1029,  757212,  757239,  757276, 27, 37, 37, }, // Kamchatka
    {  603, 1058,       0,  757313,       0,  0, 18,  0, }, // Kazakhstan_Eastern
    {  603, 1077,       0,  757331,       0,  0, 18,  0, }, // Kazakhstan_Western
    {  603, 1106,  757349,  757361,  757383, 12, 22, 22, }, // Korea
    {  603, 1112,       0,  757405,       0,  0, 11,  0, }, // Kosrae
    {  603, 1119,  757416,  757432,  757458, 16, 26, 26, }, // Krasnoyarsk
    {  603, 1151,       0,  757484,       0,  0, 16,  0, }, // Kyrgystan
    {  603, 1161,       0,  757500,       0,  0, 11,  0, }, // Lanka
    {  603, 1175,       0,  757511,       0,  0, 18,  0, }, // Line_Islands
    {  603, 1188,  757529,  757543,  757567, 14, 24, 24, }, // Lord_Howe
    {  603, 1198,  757591,  757602,  757623, 11, 21, 21, }, // Macau
    {  603, 1204,  757644,  757657,  757679, 13, 22, 22, }, // Magadan
    {  603, 1219,       0,  757701,       0,  0, 12,  0, }, // Malaysia
    {  603, 1228,       0,  757713,       0,  0, 21,  0, }, // Maldives
    {  603, 1237,       0,  757734,       0,  0, 15,  0, }, // Marquesas
    {  603, 1247,       0,  757749,       0,  0, 20,  0, }, // Marshall_Islands
    {  603, 1264,  757769,  757783,  757807, 14, 24, 24, }, // Mauritius
    {  603, 1274,       0,  757831,       0,  0, 12,  0, }, // Mawson
    {  603, 1281,  757843,  757862,  757891, 19, 29, 29, }, // Mexico_Pacific
    {  603, 1296,  757920,  757936,  757962, 16, 26, 26, }, // Mongolia
    {  603, 1305,  757988,  757999,  758020, 11, 21, 21, }, // Moscow
    {  603, 1312,       0,  758041,       0,  0, 13,  0, }, // Myanmar
    {  603, 1320,       0,  758054,       0,  0, 11,  0, }, // Nauru
    {  603, 1326,       0,  758065,       0,  0, 11,  0, }, // Nepal
    {  603, 1332,  758076,  758094,  758122, 18, 28, 28, }, // New_Caledonia
    {  603, 1346,  758150,  758165,  758190, 15, 25, 25, }, // New_Zealand
    {  603, 1358,  758215,  758231,  758257, 16, 26, 26, }, // Newfoundland
    {  603, 1371,       0,  758283,       0,  0, 10,  0, }, // Niue
    {  603, 1376,  758293,  758312,  758341, 19, 29, 29, }, // Norfolk
    {  603, 1384,  758370,  758391,  758422, 21, 31, 31, }, // Noronha
    {  603, 1392,       0,  758453,       0,  0, 28,  0, }, // North_Mariana
    {  603, 1406,  758481,  758497,  758523, 16, 26, 26, }, // Novosibirsk
    {  603, 1418,  758549,  758559,  758579, 10, 20, 20, }, // Omsk
    {  603, 1428,  758599,  758612,  758635, 13, 23, 23, }, // Pakistan
    {  603, 1437,       0,  758658,       0,  0, 10,  0, }, // Palau
    {  603, 1443,       0,  758668,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  603, 1460,  758690,  758702,  758724, 12, 22, 22, }, // Paraguay
    {  603, 1469,  758746,  758756,  758776, 10, 20, 20, }, // Peru
    {  603, 1474,  758796,  758811,  758836, 15, 25, 25, }, // Philippines
    {  603, 1486,       0,  758861,       0,  0, 20,  0, }, // Phoenix_Islands
    {  603, 1502,  758881,  758904,  758937, 23, 33, 33, }, // Pierre_Miquelon
    {  603, 1518,       0,  758970,       0,  0, 14,  0, }, // Pitcairn
    {  603, 1527,       0,  758984,       0,  0, 12,  0, }, // Ponape
    {  603, 1534,       0,  758996,       0,  0, 14,  0, }, // Pyongyang
    {  603, 1544,  759010,  759025,  759050, 15, 25, 25, }, // Qyzylorda
    {  603, 1554,       0,  759075,       0,  0, 13,  0, }, // Reunion
    {  603, 1562,       0,  759088,       0,  0, 12,  0, }, // Rothera
    {  603, 1570,  759100,  759113,  759136, 13, 23, 23, }, // Sakhalin
    {  603, 1579,  759159,  759170,  759191, 11, 21, 21, }, // Samara
    {  603, 1596,  759212,  759222,  759242, 10, 20, 20, }, // Samoa
    {  603, 1602,       0,  759262,       0,  0, 12,  0, }, // Seychelles
    {  603, 1624,       0,  759274,       0,  0, 24,  0, }, // Singapore
    {  603, 1634,       0,  759298,       0,  0, 20,  0, }, // Solomon
    {  603, 1642,       0,  759318,       0,  0, 21,  0, }, // South_Georgia
    {  603, 1656,       0,  759339,       0,  0, 13,  0, }, // Suriname
    {  603, 1676,       0,  759352,       0,  0, 10,  0, }, // Syowa
    {  603, 1682,       0,  759362,       0,  0, 11,  0, }, // Tahiti
    {  603, 1689,  759373,  759385,  759407, 12, 22, 22, }, // Taipei
    {  603, 1696,       0,  759429,       0,  0, 15,  0, }, // Tajikistan
    {  603, 1724,       0,  759444,       0,  0, 13,  0, }, // Tokelau
    {  603, 1732,  759457,  759467,  759487, 10, 20, 20, }, // Tonga
    {  603, 1738,       0,  759507,       0,  0, 10,  0, }, // Truk
    {  603, 1750,  759517,  759533,  759559, 16, 26, 26, }, // Turkmenistan
    {  603, 1763,       0,  759585,       0,  0, 12,  0, }, // Tuvalu
    {  603, 1777,  759597,  759611,  759635, 14, 24, 24, }, // Uruguay
    {  603, 1792,  759659,  759674,  759699, 15, 25, 25, }, // Uzbekistan
    {  603, 1803,  759724,  759737,  759760, 13, 23, 23, }, // Vanuatu
    {  603, 1811,       0,  759783,       0,  0, 13,  0, }, // Venezuela
    {  603, 1821,  759796,  759813,  759840, 17, 27, 27, }, // Vladivostok
    {  603, 1833,  759867,  759882,  759907, 15, 25, 25, }, // Volgograd
    {  603, 1843,       0,  759932,       0,  0, 12,  0, }, // Vostok
    {  603, 1850,       0,  759944,       0,  0, 18,  0, }, // Wake
    {  603, 1855,       0,  759962,       0,  0, 19,  0, }, // Wallis
    {  603, 1862,  759981,  759994,  760017, 13, 23, 23, }, // Yakutsk
    {  603, 1870,  760040,  760057,  760084, 17, 27, 27, }, // Yekaterinburg
    {  603, 1892,       0,  760111,       0,  0, 11,  0, }, // Yukon
    {  609,    5,       0,  760122,       0,  0, 16,  0, }, // Afghanistan Tajik/Cyrillic/Tajikistan
    {  609,   17,       0,  760138,       0,  0, 21,  0, }, // Africa_Central
    {  609,   32,       0,  760159,       0,  0, 19,  0, }, // Africa_Eastern
    {  609,   65,       0,  760178,       0,  0, 31,  0, }, // Africa_Southern
    {  609,   81,  760209,  760228,  760258, 19, 30, 30, }, // Africa_Western
    {  609,  107,  760288,  760300,  760323, 12, 23, 20, }, // Alaska
    {  609,  135,  760343,  760355,  760378, 12, 23, 23, }, // Amazon
    {  609,  142,  760401,  760414,  760438, 13, 24, 21, }, // America_Central
    {  609,  158,  760459,  760470,  760492, 11, 22, 19, }, // America_Eastern
    {  609,  174,  760511,  760521,  760542, 10, 21, 18, }, // America_Mountain
    {  609,  191,  760560,  760578,  760607, 18, 29, 26, }, // America_Pacific
    {  609,  214,  760633,  760643,  760664, 10, 21, 18, }, // Apia
    {  609,  232,  760682,  760697,  760723, 15, 26, 23, }, // Arabian
    {  609,  240,  760746,  760761,  760787, 15, 26, 26, }, // Argentina
    {  609,  250,  760813,  760835,  760868, 22, 33, 33, }, // Argentina_Western
    {  609,  268,  760901,  760917,  760944, 16, 27, 24, }, // Armenia
    {  609,  286,  760968,  760983,  761009, 15, 26, 23, }, // Atlantic
    {  609,  295,  761032,  761056,  761091, 24, 35, 31, }, // Australia_Central
    {  609,  313,  761122,  761153,  761194, 31, 41, 38, }, // Australia_CentralWestern
    {  609,  338,  761232,  761254,  761287, 22, 33, 30, }, // Australia_Eastern
    {  609,  356,  761317,  761339,  761372, 22, 33, 30, }, // Australia_Western
    {  609,  374,  761402,  761418,  761445, 16, 27, 27, }, // Azerbaijan
    {  609,  385,  761472,  761482,  761503, 10, 21, 21, }, // Azores
    {  609,  397,  761524,  761539,  761565, 15, 26, 26, }, // Bangladesh
    {  609,  415,       0,  761591,       0,  0, 11,  0, }, // Bhutan
    {  609,  422,       0,  761602,       0,  0, 13,  0, }, // Bolivia
    {  609,  437,  761615,  761629,  761654, 14, 25, 25, }, // Brasilia
    {  609,  454,       0,  761679,       0,  0, 23,  0, }, // Brunei
    {  609,  461,  761702,  761718,  761745, 16, 27, 27, }, // Cape_Verde
    {  609,  478,       0,  761772,       0,  0, 24,  0, }, // Chamorro
    {  609,  487,  761796,  761807,  761829, 11, 22, 19, }, // Chatham
    {  609,  495,  761848,  761858,  761879, 10, 21, 21, }, // Chile
    {  609,  501,  761900,  761909,  761929,  9, 20, 17, }, // China
    {  609,  507,       0,  761946,       0,  0, 25,  0, }, // Christmas
    {  609,  517,       0,  761971,       0,  0, 21,  0, }, // Cocos
    {  609,  523,  761992,  762006,  762031, 14, 25, 25, }, // Colombia
    {  609,  532,  762056,  762075,  762105, 19, 30, 36, }, // Cook
    {  609,  537,  762141,  762151,  762172, 10, 21, 18, }, // Cuba
    {  609,  548,       0,  762190,       0,  0, 11,  0, }, // Davis
    {  609,  564,       0,  762201,       0,  0, 19,  0, }, // DumontDUrville
    {  609,  601,       0,  762220,       0,  0, 18,  0, }, // East_Timor
    {  609,  612,  762238,  762257,  762287, 19, 30, 30, }, // Easter
    {  609,  619,       0,  762317,       0,  0, 13,  0, }, // Ecuador
    {  609,  627,  762330,  762352,  762385, 22, 33, 33, }, // Europe_Central
    {  609,  642,  762418,  762438,  762469, 20, 31, 31, }, // Europe_Eastern
    {  609,  657,       0,  762500,       0,  0, 19,  0, }, // Europe_Further_Eastern
    {  609,  680,  762519,  762539,  762570, 20, 31, 31, }, // Europe_Western
    {  609,  695,  762601,  762625,  762660, 24, 35, 35, }, // Falkland
    {  609,  704,  762695,  762705,  762726, 10, 21, 21, }, // Fiji
    {  609,  709,       0,  762747,       0,  0, 21,  0, }, // French_Guiana
    {  609,  723,       0,  762768,       0,  0, 35,  0, }, // French_Southern
    {  609,  746,       0,  762803,       0,  0, 15,  0, }, // Galapagos
    {  609,  756,       0,  762818,       0,  0, 12,  0, }, // Gambier
    {  609,  764,  762830,  762845,  762871, 15, 26, 26, }, // Georgia
    {  609,  772,       0,  762897,       0,  0, 23,  0, }, // Gilbert_Islands
    {  609,  788,       0,  762920,       0,  0, 20,  0, }, // GMT
    {  609,  830,  762940,  762963,  762997, 23, 34, 34, }, // Greenland_Eastern
    {  609,  848,  763031,  763054,  763088, 23, 34, 34, }, // Greenland_Western
    {  609,  871,       0,  763122,       0,  0, 28,  0, }, // Gulf
    {  609,  876,       0,  763150,       0,  0, 12,  0, }, // Guyana
    {  609,  883,  763162,  763181,  763211, 19, 30, 27, }, // Hawaii_Aleutian
    {  609,  899,  763238,  763251,  763275, 13, 24, 24, }, // Hong_Kong
    {  609,  909,  763299,  763309,  763330, 10, 21, 21, }, // Hovd
    {  609,  914,       0,  763351,       0,  0, 26,  0, }, // India
    {  609,  920,       0,  763377,       0,  0, 18,  0, }, // Indian_Ocean
    {  609,  933,       0,  763395,       0,  0, 14,  0, }, // Indochina
    {  609,  943,       0,  763409,       0,  0, 24,  0, }, // Indonesia_Central
    {  609,  961,       0,  763433,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  609,  979,       0,  763455,       0,  0, 22,  0, }, // Indonesia_Western
    {  609,  997,  763477,  763487,  763508, 10, 21, 18, }, // Iran
    {  609, 1008,  763526,  763539,  763563, 13, 24, 24, }, // Irkutsk
    {  609, 1016,  763587,  763599,  763622, 12, 23, 20, }, // Israel
    {  609, 1023,  763642,  763653,  763675, 11, 22, 20, }, // Japan
    {  609, 1047,       0,  763695,       0,  0, 16,  0, }, // Kazakhstan
    {  609, 1058,       0,  763711,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  609, 1077,       0,  763734,       0,  0, 23,  0, }, // Kazakhstan_Western
    {  609, 1106,  763757,  763768,  763790, 11, 22, 19, }, // Korea
    {  609, 1112,       0,  763809,       0,  0, 12,  0, }, // Kosrae
    {  609, 1119,  763821,  763837,  763864, 16, 27, 27, }, // Krasnoyarsk
    {  609, 1151,       0,  763891,       0,  0, 17,  0, }, // Kyrgystan
    {  609, 1175,       0,  763908,       0,  0, 19,  0, }, // Line_Islands
    {  609, 1188,  763927,  763940,  763965, 13, 25, 22, }, // Lord_Howe
    {  609, 1204,  763987,  764000,  764024, 13, 24, 24, }, // Magadan
    {  609, 1219,       0,  764048,       0,  0, 14,  0, }, // Malaysia
    {  609, 1228,       0,  764062,       0,  0, 12,  0, }, // Maldives
    {  609, 1237,       0,  764074,       0,  0, 12,  0, }, // Marquesas
    {  609, 1247,       0,  764086,       0,  0, 23,  0, }, // Marshall_Islands
    {  609, 1264,  764109,  764123,  764148, 14, 25, 25, }, // Mauritius
    {  609, 1274,       0,  764173,       0,  0, 12,  0, }, // Mawson
    {  609, 1281,  764185,  764211,  764248, 26, 37, 34, }, // Mexico_Pacific
    {  609, 1296,  764282,  764298,  764325, 16, 27, 27, }, // Mongolia
    {  609, 1305,  764352,  764364,  764387, 12, 23, 23, }, // Moscow
    {  609, 1312,       0,  764410,       0,  0, 11,  0, }, // Myanmar
    {  609, 1320,       0,  764421,       0,  0, 11,  0, }, // Nauru
    {  609, 1326,       0,  764432,       0,  0, 11,  0, }, // Nepal
    {  609, 1332,  764443,  764463,  764494, 20, 31, 31, }, // New_Caledonia
    {  609, 1346,  764525,  764544,  764574, 19, 30, 27, }, // New_Zealand
    {  609, 1358,  764601,  764618,  764646, 17, 28, 25, }, // Newfoundland
    {  609, 1371,       0,  764671,       0,  0, 10,  0, }, // Niue
    {  609, 1376,  764681,  764702,  764734, 21, 32, 29, }, // Norfolk
    {  609, 1384,  764763,  764788,  764824, 25, 36, 36, }, // Noronha
    {  609, 1406,  764860,  764877,  764905, 17, 28, 28, }, // Novosibirsk
    {  609, 1418,  764933,  764943,  764963, 10, 20, 21, }, // Omsk
    {  609, 1428,  764984,  764998,  765023, 14, 25, 25, }, // Pakistan
    {  609, 1437,       0,  765048,       0,  0, 11,  0, }, // Palau
    {  609, 1443,       0,  765059,       0,  0, 23,  0, }, // Papua_New_Guinea
    {  609, 1460,  765082,  765096,  765121, 14, 25, 25, }, // Paraguay
    {  609, 1469,  765146,  765156,  765177, 10, 21, 21, }, // Peru
    {  609, 1474,  765198,  765212,  765237, 14, 25, 25, }, // Philippines
    {  609, 1486,       0,  765262,       0,  0, 22,  0, }, // Phoenix_Islands
    {  609, 1502,  765284,  765310,  765347, 26, 37, 34, }, // Pierre_Miquelon
    {  609, 1518,       0,  765381,       0,  0, 13,  0, }, // Pitcairn
    {  609, 1527,       0,  765394,       0,  0, 11,  0, }, // Ponape
    {  609, 1534,       0,  765405,       0,  0, 12,  0, }, // Pyongyang
    {  609, 1554,       0,  765417,       0,  0, 13,  0, }, // Reunion
    {  609, 1562,       0,  765430,       0,  0, 12,  0, }, // Rothera
    {  609, 1570,  765442,  765455,  765479, 13, 24, 24, }, // Sakhalin
    {  609, 1596,  765503,  765514,  765536, 11, 22, 19, }, // Samoa
    {  609, 1602,       0,  765555,       0,  0, 12,  0, }, // Seychelles
    {  609, 1624,       0,  765567,       0,  0, 25,  0, }, // Singapore
    {  609, 1634,       0,  765592,       0,  0, 23,  0, }, // Solomon
    {  609, 1642,       0,  765615,       0,  0, 20,  0, }, // South_Georgia
    {  609, 1656,       0,  765635,       0,  0, 13,  0, }, // Suriname
    {  609, 1676,       0,  765648,       0,  0, 10,  0, }, // Syowa
    {  609, 1682,       0,  765658,       0,  0, 11,  0, }, // Tahiti
    {  609, 1689,  765669,  765681,  765704, 12, 23, 20, }, // Taipei
    {  609, 1696,       0,  765724,       0,  0, 16,  0, }, // Tajikistan
    {  609, 1724,       0,  765740,       0,  0, 13,  0, }, // Tokelau
    {  609, 1732,  765753,  765764,  765786, 11, 22, 22, }, // Tonga
    {  609, 1738,       0,  765808,       0,  0,  9,  0, }, // Truk
    {  609, 1750,  765817,  765835,  765864, 18, 29, 29, }, // Turkmenistan
    {  609, 1763,       0,  765893,       0,  0, 12,  0, }, // Tuvalu
    {  609, 1777,  765905,  765918,  765942, 13, 24, 24, }, // Uruguay
    {  609, 1792,  765966,  765982,  766009, 16, 27, 27, }, // Uzbekistan
    {  609, 1803,  766036,  766049,  766073, 13, 24, 24, }, // Vanuatu
    {  609, 1811,       0,  766097,       0,  0, 15,  0, }, // Venezuela
    {  609, 1821,  766112,  766129,  766157, 17, 28, 28, }, // Vladivostok
    {  609, 1833,  766185,  766200,  766226, 15, 26, 26, }, // Volgograd
    {  609, 1843,       0,  766252,       0,  0, 12,  0, }, // Vostok
    {  609, 1850,       0,  766264,       0,  0, 20,  0, }, // Wake
    {  609, 1855,       0,  766284,       0,  0, 22,  0, }, // Wallis
    {  609, 1862,  766306,  766318,  766340, 12, 22, 22, }, // Yakutsk
    {  609, 1870,  766362,  766380,  766409, 18, 29, 29, }, // Yekaterinburg
    {  609, 1892,       0,  766438,       0,  0, 10,  0, }, // Yukon
    {  610,    0,  766448,  766459,  766473, 11, 14, 16, }, // Acre Tamil/Tamil/India
    {  610,    5,       0,  766489,       0,  0, 20,  0, }, // Afghanistan
    {  610,   17,       0,  766509,       0,  0, 23,  0, }, // Africa_Central
    {  610,   32,       0,  766532,       0,  0, 24,  0, }, // Africa_Eastern
    {  610,   65,       0,  766556,       0,  0, 29,  0, }, // Africa_Southern
    {  610,   81,  766585,  766608,  766639, 23, 31, 28, }, // Africa_Western
    {  610,  107,  766667,  766680,  766701, 13, 21, 20, }, // Alaska
    {  610,  128,  766721,  766734,  766750, 13, 16, 18, }, // Almaty
    {  610,  135,  766768,  766781,  766802, 13, 21, 18, }, // Amazon
    {  610,  142,  766820,  766832,  766852, 12, 20, 19, }, // America_Central
    {  610,  158,  766871,  766888,  766913, 17, 25, 24, }, // America_Eastern
    {  610,  174,  766937,  766952,  766975, 15, 23, 22, }, // America_Mountain
    {  610,  191,  766997,  767010,  767031, 13, 21, 20, }, // America_Pacific
    {  610,  207,  767051,  767063,  767080, 12, 17, 19, }, // Anadyr
    {  610,  214,  767099,  767110,  767129, 11, 19, 18, }, // Apia
    {  610,  219,  767147,  767158,  767172, 11, 14, 16, }, // Aqtau
    {  610,  225,  767188,  767201,  767217, 13, 16, 18, }, // Aqtobe
    {  610,  232,  767235,  767247,  767267, 12, 20, 19, }, // Arabian
    {  610,  240,  767286,  767303,  767328, 17, 25, 22, }, // Argentina
    {  610,  250,  767350,  767378,  767414, 28, 36, 33, }, // Argentina_Western
    {  610,  268,  767447,  767461,  767483, 14, 22, 19, }, // Armenia
    {  610,  286,  767502,  767519,  767544, 17, 25, 24, }, // Atlantic
    {  610,  295,  767568,  767591,  767624, 23, 33, 32, }, // Australia_Central
    {  610,  313,  767656,  767692,  767736, 36, 44, 43, }, // Australia_CentralWestern
    {  610,  338,  767779,  767807,  767845, 28, 38, 37, }, // Australia_Eastern
    {  610,  356,  767882,  767909,  767946, 27, 37, 36, }, // Australia_Western
    {  610,  374,  767982,  767998,  768022, 16, 24, 21, }, // Azerbaijan
    {  610,  385,  768043,  768055,  768075, 12, 20, 18, }, // Azores
    {  610,  397,  768093,  768106,  768127, 13, 21, 18, }, // Bangladesh
    {  610,  415,       0,  768145,       0,  0, 12,  0, }, // Bhutan
    {  610,  422,       0,  768157,       0,  0, 14,  0, }, // Bolivia
    {  610,  437,  768171,  768187,  768211, 16, 24, 21, }, // Brasilia
    {  610,  454,       0,  768232,       0,  0, 23,  0, }, // Brunei
    {  610,  461,  768255,  768272,  768297, 17, 25, 22, }, // Cape_Verde
    {  610,  478,       0,  768319,       0,  0, 20,  0, }, // Chamorro
    {  610,  487,  768339,  768352,  768373, 13, 21, 20, }, // Chatham
    {  610,  495,  768393,  768403,  768421, 10, 18, 15, }, // Chile
    {  610,  501,  768436,  768445,  768462,  9, 17, 16, }, // China
    {  610,  507,       0,  768478,       0,  0, 22,  0, }, // Christmas
    {  610,  517,       0,  768500,       0,  0, 20,  0, }, // Cocos
    {  610,  523,  768520,  768535,  768558, 15, 23, 20, }, // Colombia
    {  610,  532,  768578,  768596,  768622, 18, 26, 27, }, // Cook
    {  610,  537,  768649,  768661,  768681, 12, 20, 19, }, // Cuba
    {  610,  548,       0,  768700,       0,  0, 12,  0, }, // Davis
    {  610,  564,       0,  768712,       0,  0, 27,  0, }, // DumontDUrville
    {  610,  601,       0,  768739,       0,  0, 20,  0, }, // East_Timor
    {  610,  612,  768759,  768776,  768801, 17, 25, 22, }, // Easter
    {  610,  619,       0,  768823,       0,  0, 14,  0, }, // Ecuador
    {  610,  627,  768837,  768858,  768887, 21, 29, 26, }, // Europe_Central
    {  610,  642,  768913,  768939,  768973, 26, 34, 31, }, // Europe_Eastern
    {  610,  657,       0,  769004,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {  610,  680,  769030,  769055,  769088, 25, 33, 30, }, // Europe_Western
    {  610,  695,  769118,  769143,  769176, 25, 33, 30, }, // Falkland
    {  610,  704,  769206,  769217,  769236, 11, 19, 16, }, // Fiji
    {  610,  709,       0,  769252,       0,  0, 20,  0, }, // French_Guiana
    {  610,  723,       0,  769272,       0,  0, 39,  0, }, // French_Southern
    {  610,  746,       0,  769311,       0,  0, 13,  0, }, // Galapagos
    {  610,  756,       0,  769324,       0,  0, 15,  0, }, // Gambier
    {  610,  764,  769339,  769353,  769375, 14, 22, 19, }, // Georgia
    {  610,  772,       0,  769394,       0,  0, 23,  0, }, // Gilbert_Islands
    {  610,  788,       0,  769417,       0,  0, 23,  0, }, // GMT
    {  610,  830,  769440,  769466,  769500, 26, 34, 31, }, // Greenland_Eastern
    {  610,  848,  769531,  769556,  769589, 25, 33, 30, }, // Greenland_Western
    {  610,  866,       0,  769619,       0,  0, 12,  0, }, // Guam
    {  610,  871,       0,  769631,       0,  0, 21,  0, }, // Gulf
    {  610,  876,       0,  769261,       0,  0, 11,  0, }, // Guyana
    {  610,  883,  769652,  769672,  769700, 20, 28, 27, }, // Hawaii_Aleutian
    {  610,  899,  769727,  769741,  769763, 14, 22, 19, }, // Hong_Kong
    {  610,  909,  769782,  769794,  769814, 12, 20, 17, }, // Hovd
    {  610,  914,       0,  769831,       0,  0, 20,  0, }, // India
    {  610,  920,       0,  769851,       0,  0, 25,  0, }, // Indian_Ocean
    {  610,  933,       0,  769876,       0,  0, 14,  0, }, // Indochina
    {  610,  943,       0,  769890,       0,  0, 23,  0, }, // Indonesia_Central
    {  610,  961,       0,  769913,       0,  0, 28,  0, }, // Indonesia_Eastern
    {  610,  979,       0,  769941,       0,  0, 27,  0, }, // Indonesia_Western
    {  610,  997,  769968,  769979,  769998, 11, 19, 18, }, // Iran
    {  610, 1008,  770016,  770033,  770058, 17, 25, 22, }, // Irkutsk
    {  610, 1016,  770080,  770093,  770114, 13, 21, 20, }, // Israel
    {  610, 1023,  770134,  770147,  770168, 13, 21, 20, }, // Japan
    {  610, 1029,  770188,  770222,  770259, 34, 37, 39, }, // Kamchatka
    {  610, 1047,       0,  770298,       0,  0, 15,  0, }, // Kazakhstan
    {  610, 1058,       0,  770313,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  610, 1077,       0,  770336,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  610, 1106,  770358,  770369,  770388, 11, 19, 18, }, // Korea
    {  610, 1112,       0,  770406,       0,  0, 12,  0, }, // Kosrae
    {  610, 1119,  770418,  770439,  770468, 21, 29, 26, }, // Krasnoyarsk
    {  610, 1151,       0,  770494,       0,  0, 18,  0, }, // Kyrgystan
    {  610, 1161,       0,  770512,       0,  0, 11,  0, }, // Lanka
    {  610, 1175,       0,  770523,       0,  0, 18,  0, }, // Line_Islands
    {  610, 1188,  770541,  770558,  770583, 17, 25, 24, }, // Lord_Howe
    {  610, 1198,  770607,  770620,  770636, 13, 16, 18, }, // Macau
    {  610, 1204,  770654,  770665,  770684, 11, 19, 16, }, // Magadan
    {  610, 1219,       0,  770700,       0,  0, 12,  0, }, // Malaysia
    {  610, 1228,       0,  770712,       0,  0, 18,  0, }, // Maldives
    {  610, 1237,       0,  770730,       0,  0, 18,  0, }, // Marquesas
    {  610, 1247,       0,  770748,       0,  0, 21,  0, }, // Marshall_Islands
    {  610, 1264,  770769,  770784,  770807, 15, 23, 20, }, // Mauritius
    {  610, 1274,       0,  770827,       0,  0, 11,  0, }, // Mawson
    {  610, 1281,  770838,  770861,  770892, 23, 31, 30, }, // Mexico_Pacific
    {  610, 1296,  770922,  770938,  770962, 16, 24, 21, }, // Mongolia
    {  610, 1305,  770983,  770995,  771015, 12, 20, 17, }, // Moscow
    {  610, 1312,       0,  771032,       0,  0, 15,  0, }, // Myanmar
    {  610, 1320,       0,  771047,       0,  0, 11,  0, }, // Nauru
    {  610, 1326,       0,  771058,       0,  0, 11,  0, }, // Nepal
    {  610, 1332,  771069,  771089,  771117, 20, 28, 25, }, // New_Caledonia
    {  610, 1346,  771142,  771160,  771186, 18, 26, 25, }, // New_Zealand
    {  610, 1358,  771211,  771235,  771267, 24, 32, 31, }, // Newfoundland
    {  610, 1371,       0,  771298,       0,  0, 10,  0, }, // Niue
    {  610, 1376,  771308,  771328,  771356, 20, 28, 27, }, // Norfolk
    {  610, 1384,  771383,  771410,  771446, 27, 36, 33, }, // Noronha
    {  610, 1392,       0,  771479,       0,  0, 26,  0, }, // North_Mariana
    {  610, 1406,  771505,  771525,  771553, 20, 28, 25, }, // Novosibirsk
    {  610, 1418,  771578,  771591,  771612, 13, 21, 18, }, // Omsk
    {  610, 1428,  771630,  771646,  771670, 16, 24, 21, }, // Pakistan
    {  610, 1437,       0,  771691,       0,  0, 11,  0, }, // Palau
    {  610, 1443,       0,  771702,       0,  0, 23,  0, }, // Papua_New_Guinea
    {  610, 1460,  771725,  771738,  771759, 13, 21, 18, }, // Paraguay
    {  610, 1469,  771777,  771787,  771805, 10, 18, 15, }, // Peru
    {  610, 1474,  771820,  771836,  771860, 16, 24, 21, }, // Philippines
    {  610, 1486,       0,  771881,       0,  0, 23,  0, }, // Phoenix_Islands
    {  610, 1502,  771904,  771939,  771982, 35, 43, 42, }, // Pierre_Miquelon
    {  610, 1518,       0,  772024,       0,  0, 18,  0, }, // Pitcairn
    {  610, 1527,       0,  772042,       0,  0, 12,  0, }, // Ponape
    {  610, 1534,       0,  772054,       0,  0, 16,  0, }, // Pyongyang
    {  610, 1544,  772070,  772086,  772105, 16, 19, 21, }, // Qyzylorda
    {  610, 1554,       0,  772126,       0,  0, 15,  0, }, // Reunion
    {  610, 1562,       0,  772141,       0,  0, 12,  0, }, // Rothera
    {  610, 1570,  772153,  772165,  772185, 12, 20, 17, }, // Sakhalin
    {  610, 1579,  772202,  772212,  772225, 10, 13, 15, }, // Samara
    {  610, 1596,  772240,  772251,  772270, 11, 19, 18, }, // Samoa
    {  610, 1602,       0,  772288,       0,  0, 14,  0, }, // Seychelles
    {  610, 1624,       0,  772302,       0,  0, 25,  0, }, // Singapore
    {  610, 1634,       0,  772327,       0,  0, 20,  0, }, // Solomon
    {  610, 1642,       0,  772347,       0,  0, 21,  0, }, // South_Georgia
    {  610, 1656,       0,  772368,       0,  0, 14,  0, }, // Suriname
    {  610, 1676,       0,  772382,       0,  0, 12,  0, }, // Syowa
    {  610, 1682,       0,  772394,       0,  0, 11,  0, }, // Tahiti
    {  610, 1689,  772405,  772417,  772437, 12, 20, 19, }, // Taipei
    {  610, 1696,       0,  772456,       0,  0, 17,  0, }, // Tajikistan
    {  610, 1724,       0,  772473,       0,  0, 16,  0, }, // Tokelau
    {  610, 1732,  772489,  772501,  772521, 12, 20, 17, }, // Tonga
    {  610, 1738,       0,  772538,       0,  0, 10,  0, }, // Truk
    {  610, 1750,  772548,  772570,  772600, 22, 30, 27, }, // Turkmenistan
    {  610, 1763,       0,  772627,       0,  0, 12,  0, }, // Tuvalu
    {  610, 1777,  772639,  772652,  772673, 13, 21, 18, }, // Uruguay
    {  610, 1792,  772691,  772710,  772737, 19, 27, 24, }, // Uzbekistan
    {  610, 1803,  772761,  772776,  772799, 15, 23, 20, }, // Vanuatu
    {  610, 1811,       0,  772819,       0,  0, 14,  0, }, // Venezuela
    {  610, 1821,  772833,  772853,  772881, 20, 28, 25, }, // Vladivostok
    {  610, 1833,  772906,  772924,  772950, 18, 26, 23, }, // Volgograd
    {  610, 1843,       0,  772839,       0,  0, 14,  0, }, // Vostok
    {  610, 1850,       0,  772973,       0,  0, 15,  0, }, // Wake
    {  610, 1855,       0,  772988,       0,  0, 30,  0, }, // Wallis
    {  610, 1862,  773018,  773033,  773056, 15, 23, 20, }, // Yakutsk
    {  610, 1870,  773076,  773097,  773126, 21, 29, 26, }, // Yekaterinburg
    {  610, 1892,       0,  773152,       0,  0, 12,  0, }, // Yukon
    {  611,  914,       0,  769831,       0,  0, 20,  0, }, // India Tamil/Tamil/Malaysia
    {  611, 1219,       0,  770700,       0,  0, 12,  0, }, // Malaysia
    {  611, 1624,       0,  772302,       0,  0, 25,  0, }, // Singapore
    {  612,  914,       0,  769831,       0,  0, 20,  0, }, // India Tamil/Tamil/Singapore
    {  612, 1219,       0,  770700,       0,  0, 12,  0, }, // Malaysia
    {  612, 1624,       0,  772302,       0,  0, 25,  0, }, // Singapore
    {  614,  142,  773164,  773181,  773205, 17, 24, 32, }, // America_Central Taroko/Latin/Taiwan
    {  614,  158,  773237,  773254,  773278, 17, 24, 32, }, // America_Eastern
    {  614,  174,  773310,  773328,  773353, 18, 25, 28, }, // America_Mountain
    {  614,  191,  773381,  773396,  773426, 15, 30, 38, }, // America_Pacific
    {  614,  286,  773464,  773483,  773509, 19, 26, 34, }, // Atlantic
    {  614,  627,  773543,  773554,  773572, 11, 18, 26, }, // Europe_Central
    {  614,  642,  773598,  773609,  773627, 11, 18, 26, }, // Europe_Eastern
    {  614,  680,  773653,  773663,  773680, 10, 17, 25, }, // Europe_Western
    {  614,  788,       0,  773705,       0,  0, 20,  0, }, // GMT
    {  616,    0,  773725,  773735,  773754, 10, 19, 16, }, // Acre Tatar/Cyrillic/Russia
    {  616,    5,       0,  773770,       0,  0, 16,  0, }, // Afghanistan
    {  616,   17,       0,  773786,       0,  0, 18,  0, }, // Africa_Central
    {  616,   32,       0,  773804,       0,  0, 22,  0, }, // Africa_Eastern
    {  616,   65,       0,  773826,       0,  0, 20,  0, }, // Africa_Southern
    {  616,   81,  773846,  773868,  773899, 22, 31, 28, }, // Africa_Western
    {  616,  107,  773927,  773940,  773962, 13, 22, 19, }, // Alaska
    {  616,  135,  773981,  773996,  774020, 15, 24, 21, }, // Amazon
    {  616,  142,  774041,  774067,  774100, 26, 33, 32, }, // America_Central
    {  616,  158,  774132,  774162,  774199, 30, 37, 36, }, // America_Eastern
    {  616,  174,  774235,  774260,  774292, 25, 32, 31, }, // America_Mountain
    {  616,  191,  774323,  774354,  774392, 31, 38, 37, }, // America_Pacific
    {  616,  207,  774429,  774443,  774466, 14, 23, 20, }, // Anadyr
    {  616,  214,  774486,  774497,  774517, 11, 20, 17, }, // Apia
    {  616,  232,  774534,  774546,  774567, 12, 21, 18, }, // Arabian
    {  616,  240,  774585,  774601,  774626, 16, 25, 22, }, // Argentina
    {  616,  250,  774648,  774673,  774707, 25, 34, 31, }, // Argentina_Western
    {  616,  268,  774738,  774752,  774775, 14, 23, 20, }, // Armenia
    {  616,  286,  774795,  774825,  774862, 30, 37, 36, }, // Atlantic
    {  616,  295,  774898,  774919,  774949, 21, 30, 27, }, // Australia_Central
    {  616,  313,  774976,  775006,  775045, 30, 39, 36, }, // Australia_CentralWestern
    {  616,  338,  775081,  775106,  775140, 25, 34, 31, }, // Australia_Eastern
    {  616,  356,  775171,  775196,  775230, 25, 34, 31, }, // Australia_Western
    {  616,  374,  775261,  775278,  775304, 17, 26, 23, }, // Azerbaijan
    {  616,  385,  775327,  775348,  775378, 21, 30, 27, }, // Azores
    {  616,  397,  775405,  775421,  775446, 16, 25, 22, }, // Bangladesh
    {  616,  415,       0,  775468,       0,  0, 12,  0, }, // Bhutan
    {  616,  422,       0,  775480,       0,  0, 14,  0, }, // Bolivia
    {  616,  437,  775494,  775509,  775533, 15, 24, 21, }, // Brasilia
    {  616,  454,       0,  775554,       0,  0, 24,  0, }, // Brunei
    {  616,  461,  775578,  775595,  775621, 17, 26, 23, }, // Cape_Verde
    {  616,  478,       0,  775644,       0,  0, 23,  0, }, // Chamorro
    {  616,  487,  775667,  775679,  775700, 12, 21, 18, }, // Chatham
    {  616,  495,  775718,  775729,  775749, 11, 20, 17, }, // Chile
    {  616,  501,  775766,  775778,  775799, 12, 21, 18, }, // China
    {  616,  507,       0,  775817,       0,  0, 20,  0, }, // Christmas
    {  616,  517,       0,  775837,       0,  0, 22,  0, }, // Cocos
    {  616,  523,  775859,  775874,  775898, 15, 24, 21, }, // Colombia
    {  616,  532,  775919,  775939,  775968, 20, 29, 33, }, // Cook
    {  616,  537,  776001,  776012,  776032, 11, 20, 17, }, // Cuba
    {  616,  548,       0,  776049,       0,  0, 12,  0, }, // Davis
    {  616,  564,       0,  776061,       0,  0, 21,  0, }, // DumontDUrville
    {  616,  601,       0,  776082,       0,  0, 21,  0, }, // East_Timor
    {  616,  612,  776103,  776122,  776150, 19, 28, 25, }, // Easter
    {  616,  619,       0,  776175,       0,  0, 14,  0, }, // Ecuador
    {  616,  627,  776189,  776207,  776232, 18, 25, 24, }, // Europe_Central
    {  616,  642,  776256,  776278,  776307, 22, 29, 28, }, // Europe_Eastern
    {  616,  657,       0,  776335,       0,  0, 27,  0, }, // Europe_Further_Eastern
    {  616,  680,  776362,  776384,  776413, 22, 29, 28, }, // Europe_Western
    {  616,  695,  776441,  776466,  776500, 25, 34, 31, }, // Falkland
    {  616,  704,  776531,  776543,  776564, 12, 21, 18, }, // Fiji
    {  616,  709,       0,  776582,       0,  0, 21,  0, }, // French_Guiana
    {  616,  723,       0,  776603,       0,  0, 36,  0, }, // French_Southern
    {  616,  746,       0,  776639,       0,  0, 26,  0, }, // Galapagos
    {  616,  756,       0,  776665,       0,  0, 13,  0, }, // Gambier
    {  616,  764,  776678,  776691,  776713, 13, 22, 19, }, // Georgia
    {  616,  772,       0,  776732,       0,  0, 24,  0, }, // Gilbert_Islands
    {  616,  788,       0,  776756,       0,  0, 21,  0, }, // GMT
    {  616,  830,  776777,  776803,  776838, 26, 35, 32, }, // Greenland_Eastern
    {  616,  848,  776870,  776896,  776931, 26, 35, 32, }, // Greenland_Western
    {  616,  871,       0,  776963,       0,  0, 32,  0, }, // Gulf
    {  616,  876,       0,  776995,       0,  0, 13,  0, }, // Guyana
    {  616,  883,  777008,  777026,  777053, 18, 27, 24, }, // Hawaii_Aleutian
    {  616,  899,  777077,  777091,  777114, 14, 23, 20, }, // Hong_Kong
    {  616,  909,  777134,  777145,  777165, 11, 20, 17, }, // Hovd
    {  616,  914,       0,  777182,       0,  0, 20,  0, }, // India
    {  616,  920,       0,  777202,       0,  0, 18,  0, }, // Indian_Ocean
    {  616,  933,       0,  777220,       0,  0, 17,  0, }, // Indochina
    {  616,  943,       0,  777237,       0,  0, 21,  0, }, // Indonesia_Central
    {  616,  961,       0,  777258,       0,  0, 25,  0, }, // Indonesia_Eastern
    {  616,  979,       0,  777283,       0,  0, 25,  0, }, // Indonesia_Western
    {  616,  997,  777308,  777319,  777339, 11, 20, 17, }, // Iran
    {  616, 1008,  777356,  777370,  777393, 14, 23, 20, }, // Irkutsk
    {  616, 1016,  777413,  777426,  777448, 13, 22, 19, }, // Israel
    {  616, 1023,  777467,  777478,  777498, 11, 20, 17, }, // Japan
    {  616, 1029,  777515,  777546,  777586, 31, 40, 37, }, // Kamchatka
    {  616, 1047,       0,  777623,       0,  0, 16,  0, }, // Kazakhstan
    {  616, 1058,       0,  777639,       0,  0, 25,  0, }, // Kazakhstan_Eastern
    {  616, 1077,       0,  777664,       0,  0, 25,  0, }, // Kazakhstan_Western
    {  616, 1106,  777689,  777701,  777722, 12, 21, 18, }, // Korea
    {  616, 1112,       0,  777740,       0,  0, 13,  0, }, // Kosrae
    {  616, 1119,  777753,  777770,  777796, 17, 26, 23, }, // Krasnoyarsk
    {  616, 1151,       0,  777819,       0,  0, 17,  0, }, // Kyrgystan
    {  616, 1175,       0,  777836,       0,  0, 21,  0, }, // Line_Islands
    {  616, 1188,  777857,  777872,  777896, 15, 24, 21, }, // Lord_Howe
    {  616, 1204,  777917,  777931,  777954, 14, 23, 20, }, // Magadan
    {  616, 1219,       0,  777974,       0,  0, 15,  0, }, // Malaysia
    {  616, 1228,       0,  777989,       0,  0, 24,  0, }, // Maldives
    {  616, 1237,       0,  778013,       0,  0, 23,  0, }, // Marquesas
    {  616, 1247,       0,  778036,       0,  0, 24,  0, }, // Marshall_Islands
    {  616, 1264,  778060,  778075,  778099, 15, 24, 21, }, // Mauritius
    {  616, 1274,       0,  778120,       0,  0, 13,  0, }, // Mawson
    {  616, 1281,  778133,  778159,  778193, 26, 34, 32, }, // Mexico_Pacific
    {  616, 1296,  778225,  778242,  778268, 17, 26, 23, }, // Mongolia
    {  616, 1305,  778291,  778304,  778326, 13, 22, 19, }, // Moscow
    {  616, 1312,       0,  778345,       0,  0, 13,  0, }, // Myanmar
    {  616, 1320,       0,  778358,       0,  0, 12,  0, }, // Nauru
    {  616, 1326,       0,  778370,       0,  0, 12,  0, }, // Nepal
    {  616, 1332,  778382,  778402,  778431, 20, 29, 26, }, // New_Caledonia
    {  616, 1346,  778457,  778476,  778504, 19, 28, 25, }, // New_Zealand
    {  616, 1358,  778529,  778548,  778576, 19, 28, 25, }, // Newfoundland
    {  616, 1371,       0,  778601,       0,  0, 11,  0, }, // Niue
    {  616, 1376,  778612,  778633,  778663, 21, 30, 27, }, // Norfolk
    {  616, 1384,  778690,  778716,  778751, 26, 35, 32, }, // Noronha
    {  616, 1406,  778783,  778801,  778828, 18, 27, 24, }, // Novosibirsk
    {  616, 1418,  778852,  778863,  778883, 11, 20, 17, }, // Omsk
    {  616, 1428,  778900,  778915,  778939, 15, 24, 21, }, // Pakistan
    {  616, 1437,       0,  778960,       0,  0, 12,  0, }, // Palau
    {  616, 1443,       0,  778972,       0,  0, 23,  0, }, // Papua_New_Guinea
    {  616, 1460,  778995,  779010,  779034, 15, 24, 21, }, // Paraguay
    {  616, 1469,  779055,  779066,  779086, 11, 20, 17, }, // Peru
    {  616, 1474,  779103,  779118,  779142, 15, 24, 21, }, // Philippines
    {  616, 1486,       0,  779163,       0,  0, 23,  0, }, // Phoenix_Islands
    {  616, 1502,  779186,  779213,  779249, 27, 36, 33, }, // Pierre_Miquelon
    {  616, 1518,       0,  779282,       0,  0, 14,  0, }, // Pitcairn
    {  616, 1527,       0,  779296,       0,  0, 13,  0, }, // Ponape
    {  616, 1534,       0,  779309,       0,  0, 14,  0, }, // Pyongyang
    {  616, 1554,       0,  779323,       0,  0, 14,  0, }, // Reunion
    {  616, 1562,       0,  779337,       0,  0, 13,  0, }, // Rothera
    {  616, 1570,  779350,  779364,  779387, 14, 23, 20, }, // Sakhalin
    {  616, 1579,  779407,  779420,  779442, 13, 22, 19, }, // Samara
    {  616, 1596,  779461,  779473,  779494, 12, 21, 18, }, // Samoa
    {  616, 1602,       0,  779512,       0,  0, 23,  0, }, // Seychelles
    {  616, 1624,       0,  779535,       0,  0, 24,  0, }, // Singapore
    {  616, 1634,       0,  779559,       0,  0, 24,  0, }, // Solomon
    {  616, 1642,       0,  779583,       0,  0, 22,  0, }, // South_Georgia
    {  616, 1656,       0,  779605,       0,  0, 14,  0, }, // Suriname
    {  616, 1676,       0,  779619,       0,  0, 11,  0, }, // Syowa
    {  616, 1682,       0,  779630,       0,  0, 12,  0, }, // Tahiti
    {  616, 1689,  779642,  779655,  779677, 13, 22, 19, }, // Taipei
    {  616, 1696,       0,  779696,       0,  0, 16,  0, }, // Tajikistan
    {  616, 1724,       0,  779712,       0,  0, 14,  0, }, // Tokelau
    {  616, 1732,  779726,  779738,  779759, 12, 21, 18, }, // Tonga
    {  616, 1738,       0,  779777,       0,  0, 11,  0, }, // Truk
    {  616, 1750,  779788,  779806,  779833, 18, 27, 24, }, // Turkmenistan
    {  616, 1763,       0,  779857,       0,  0, 13,  0, }, // Tuvalu
    {  616, 1777,  779870,  779884,  779907, 14, 23, 20, }, // Uruguay
    {  616, 1792,  779927,  779943,  779968, 16, 25, 22, }, // Uzbekistan
    {  616, 1803,  779990,  780004,  780027, 14, 23, 20, }, // Vanuatu
    {  616, 1811,       0,  780047,       0,  0, 16,  0, }, // Venezuela
    {  616, 1821,  780063,  780081,  780108, 18, 27, 24, }, // Vladivostok
    {  616, 1833,  780132,  780148,  780173, 16, 25, 22, }, // Volgograd
    {  616, 1843,       0,  780195,       0,  0, 13,  0, }, // Vostok
    {  616, 1850,       0,  780208,       0,  0, 18,  0, }, // Wake
    {  616, 1855,       0,  780226,       0,  0, 24,  0, }, // Wallis
    {  616, 1862,  780250,  780263,  780285, 13, 22, 19, }, // Yakutsk
    {  616, 1870,  780304,  780323,  780351, 19, 28, 25, }, // Yekaterinburg
    {  616, 1892,       0,  780376,       0,  0, 11,  0, }, // Yukon
    {  617,    0,  780387,  780396,  780415,  9, 19, 15, }, // Acre Telugu/Telugu/India
    {  617,    5,       0,  780430,       0,  0, 17,  0, }, // Afghanistan
    {  617,   17,       0,  780447,       0,  0, 21,  0, }, // Africa_Central
    {  617,   32,       0,  780468,       0,  0, 19,  0, }, // Africa_Eastern
    {  617,   65,       0,  780487,       0,  0, 29,  0, }, // Africa_Southern
    {  617,   81,  780516,  780535,  780564, 19, 29, 25, }, // Africa_Western
    {  617,  107,  780589,  780601,  780623, 12, 22, 26, }, // Alaska
    {  617,  128,  780649,  780661,  780683, 12, 22, 18, }, // Almaty
    {  617,  135,  780701,  780713,  780735, 12, 22, 18, }, // Amazon
    {  617,  142,  780753,  780763,  780783, 10, 20, 24, }, // America_Central
    {  617,  158,  780807,  780818,  780839, 11, 21, 25, }, // America_Eastern
    {  617,  174,  780864,  780878,  780902, 14, 24, 28, }, // America_Mountain
    {  617,  191,  780930,  780942,  780964, 12, 22, 26, }, // America_Pacific
    {  617,  207,  780990,  781001,  781025, 11, 24, 20, }, // Anadyr
    {  617,  214,  781045,  781055,  781075, 10, 20, 15, }, // Apia
    {  617,  219,  781090,  781102,  781124, 12, 22, 18, }, // Aqtau
    {  617,  225,  781142,  781154,  781176, 12, 22, 18, }, // Aqtobe
    {  617,  232,  781194,  781207,  781230, 13, 23, 27, }, // Arabian
    {  617,  240,  781257,  781272,  781297, 15, 25, 21, }, // Argentina
    {  617,  250,  781318,  781340,  781372, 22, 32, 28, }, // Argentina_Western
    {  617,  268,  781400,  781414,  781438, 14, 24, 20, }, // Armenia
    {  617,  286,  781458,  781473,  781498, 15, 25, 29, }, // Atlantic
    {  617,  295,  781527,  781549,  781581, 22, 32, 36, }, // Australia_Central
    {  617,  313,  781617,  781646,  781686, 29, 40, 43, }, // Australia_CentralWestern
    {  617,  338,  781729,  781752,  781786, 23, 34, 38, }, // Australia_Eastern
    {  617,  356,  781824,  781652,  781847, 23, 34, 38, }, // Australia_Western
    {  617,  374,  781885,  781900,  781925, 15, 25, 21, }, // Azerbaijan
    {  617,  385,  781946,  781958,  781980, 12, 22, 18, }, // Azores
    {  617,  397,  781998,  782013,  782038, 15, 25, 21, }, // Bangladesh
    {  617,  415,       0,  782059,       0,  0, 11,  0, }, // Bhutan
    {  617,  422,       0,  782070,       0,  0, 13,  0, }, // Bolivia
    {  617,  437,  782083,  782098,  782123, 15, 25, 21, }, // Brasilia
    {  617,  454,       0,  782144,       0,  0, 20,  0, }, // Brunei
    {  617,  461,  782164,  782180,  782206, 16, 26, 22, }, // Cape_Verde
    {  617,  478,       0,  782228,       0,  0, 22,  0, }, // Chamorro
    {  617,  487,  782250,  782260,  782280, 10, 20, 24, }, // Chatham
    {  617,  495,  782304,  782313,  782332,  9, 19, 15, }, // Chile
    {  617,  501,  782347,  782356,  782375,  9, 19, 23, }, // China
    {  617,  507,       0,  782398,       0,  0, 19,  0, }, // Christmas
    {  617,  517,       0,  782417,       0,  0, 17,  0, }, // Cocos
    {  617,  523,  782434,  782447,  782470, 13, 23, 19, }, // Colombia
    {  617,  532,  782489,  782504,  782529, 15, 25, 26, }, // Cook
    {  617,  537,  782555,  782566,  782587, 11, 21, 25, }, // Cuba
    {  617,  548,       0,  782612,       0,  0, 11,  0, }, // Davis
    {  617,  564,       0,  782623,       0,  0, 27,  0, }, // DumontDUrville
    {  617,  601,       0,  782650,       0,  0, 18,  0, }, // East_Timor
    {  617,  612,  782668,  782684,  782710, 16, 26, 22, }, // Easter
    {  617,  619,       0,  782732,       0,  0, 13,  0, }, // Ecuador
    {  617,  627,  782745,  782768,  782801, 23, 33, 29, }, // Europe_Central
    {  617,  642,  782830,  782851,  782882, 21, 31, 27, }, // Europe_Eastern
    {  617,  657,       0,  782909,       0,  0, 27,  0, }, // Europe_Further_Eastern
    {  617,  680,  782936,  782957,  782988, 21, 31, 27, }, // Europe_Western
    {  617,  695,  783015,  783038,  783071, 23, 33, 29, }, // Falkland
    {  617,  704,  783100,  783109,  783128,  9, 19, 15, }, // Fiji
    {  617,  709,       0,  783143,       0,  0, 18,  0, }, // French_Guiana
    {  617,  723,       0,  783161,       0,  0, 38,  0, }, // French_Southern
    {  617,  746,       0,  783199,       0,  0, 15,  0, }, // Galapagos
    {  617,  756,       0,  783214,       0,  0, 13,  0, }, // Gambier
    {  617,  764,  783227,  783240,  783263, 13, 23, 19, }, // Georgia
    {  617,  772,       0,  783282,       0,  0, 20,  0, }, // Gilbert_Islands
    {  617,  788,       0,  783302,       0,  0, 21,  0, }, // GMT
    {  617,  830,  783323,  783349,  783385, 26, 36, 32, }, // Greenland_Eastern
    {  617,  848,  783417,  783443,  783479, 26, 36, 32, }, // Greenland_Western
    {  617,  866,       0,  783511,       0,  0, 21,  0, }, // Guam
    {  617,  871,       0,  783532,       0,  0, 20,  0, }, // Gulf
    {  617,  876,       0,  783151,       0,  0, 10,  0, }, // Guyana
    {  617,  883,  783552,  783573,  783604, 21, 31, 35, }, // Hawaii_Aleutian
    {  617,  899,  783639,  783652,  783675, 13, 23, 19, }, // Hong_Kong
    {  617,  909,  783694,  783705,  783726, 11, 21, 17, }, // Hovd
    {  617,  914,       0,  783743,       0,  0, 22,  0, }, // India
    {  617,  920,       0,  783765,       0,  0, 21,  0, }, // Indian_Ocean
    {  617,  933,       0,  783786,       0,  0, 13,  0, }, // Indochina
    {  617,  943,       0,  783799,       0,  0, 24,  0, }, // Indonesia_Central
    {  617,  961,       0,  783823,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  617,  979,       0,  783845,       0,  0, 22,  0, }, // Indonesia_Western
    {  617,  997,  783867,  783877,  783897, 10, 20, 24, }, // Iran
    {  617, 1008,  783921,  783937,  783963, 16, 26, 22, }, // Irkutsk
    {  617, 1016,  783985,  783999,  784023, 14, 24, 28, }, // Israel
    {  617, 1023,  784051,  784061,  784081, 10, 20, 24, }, // Japan
    {  617, 1029,  784105,  784141,  784187, 36, 46, 42, }, // Kamchatka
    {  617, 1047,       0,  784229,       0,  0, 15,  0, }, // Kazakhstan
    {  617, 1058,       0,  784244,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  617, 1077,       0,  784267,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  617, 1106,  784289,  784301,  784323, 12, 22, 26, }, // Korea
    {  617, 1112,       0,  784349,       0,  0, 13,  0, }, // Kosrae
    {  617, 1119,  784362,  784383,  784414, 21, 31, 27, }, // Krasnoyarsk
    {  617, 1151,       0,  784441,       0,  0, 17,  0, }, // Kyrgystan
    {  617, 1161,       0,  784458,       0,  0,  9,  0, }, // Lanka
    {  617, 1175,       0,  784467,       0,  0, 15,  0, }, // Line_Islands
    {  617, 1188,  784482,  784498,  784524, 16, 26, 21, }, // Lord_Howe
    {  617, 1198,  784545,  784555,  784575, 10, 20, 16, }, // Macau
    {  617, 1204,  784591,  784602,  784623, 11, 21, 17, }, // Magadan
    {  617, 1219,       0,  784640,       0,  0, 12,  0, }, // Malaysia
    {  617, 1228,       0,  784652,       0,  0, 14,  0, }, // Maldives
    {  617, 1237,       0,  784666,       0,  0, 17,  0, }, // Marquesas
    {  617, 1247,       0,  784683,       0,  0, 18,  0, }, // Marshall_Islands
    {  617, 1264,  784701,  784713,  784735, 12, 22, 18, }, // Mauritius
    {  617, 1274,       0,  784753,       0,  0, 10,  0, }, // Mawson
    {  617, 1281,  784763,  784785,  784817, 22, 32, 36, }, // Mexico_Pacific
    {  617, 1296,  784853,  784868,  784893, 15, 25, 21, }, // Mongolia
    {  617, 1305,  784914,  784925,  784946, 11, 21, 17, }, // Moscow
    {  617, 1312,       0,  784963,       0,  0, 13,  0, }, // Myanmar
    {  617, 1320,       0,  784976,       0,  0,  9,  0, }, // Nauru
    {  617, 1326,       0,  784985,       0,  0, 11,  0, }, // Nepal
    {  617, 1332,  784996,  785016,  785046, 20, 30, 26, }, // New_Caledonia
    {  617, 1346,  785072,  785090,  785118, 18, 28, 32, }, // New_Zealand
    {  617, 1358,  785150,  785172,  785204, 22, 32, 36, }, // Newfoundland
    {  617, 1371,       0,  785240,       0,  0,  9,  0, }, // Niue
    {  617, 1376,  785249,  785267,  785295, 18, 28, 23, }, // Norfolk
    {  617, 1384,  785318,  785344,  785380, 26, 36, 32, }, // Noronha
    {  617, 1392,       0,  785412,       0,  0, 25,  0, }, // North_Mariana
    {  617, 1406,  785437,  785456,  785485, 19, 29, 25, }, // Novosibirsk
    {  617, 1418,  785510,  785522,  785544, 12, 22, 18, }, // Omsk
    {  617, 1428,  785562,  785577,  785602, 15, 25, 21, }, // Pakistan
    {  617, 1437,       0,  785623,       0,  0, 11,  0, }, // Palau
    {  617, 1443,       0,  785634,       0,  0, 23,  0, }, // Papua_New_Guinea
    {  617, 1460,  785657,  785669,  785691, 12, 22, 18, }, // Paraguay
    {  617, 1469,  785709,  785718,  785737,  9, 19, 15, }, // Peru
    {  617, 1474,  785752,  785767,  785792, 15, 25, 21, }, // Philippines
    {  617, 1486,       0,  785813,       0,  0, 19,  0, }, // Phoenix_Islands
    {  617, 1502,  785832,  785868,  785914, 36, 46, 49, }, // Pierre_Miquelon
    {  617, 1518,       0,  785963,       0,  0, 16,  0, }, // Pitcairn
    {  617, 1527,       0,  785979,       0,  0, 11,  0, }, // Ponape
    {  617, 1534,       0,  785990,       0,  0, 15,  0, }, // Pyongyang
    {  617, 1544,  786005,  786020,  786045, 15, 25, 21, }, // Qyzylorda
    {  617, 1554,       0,  786066,       0,  0, 14,  0, }, // Reunion
    {  617, 1562,       0,  786080,       0,  0, 11,  0, }, // Rothera
    {  617, 1570,  786091,  786103,  786125, 12, 22, 18, }, // Sakhalin
    {  617, 1579,  786143,  786153,  786173, 10, 20, 16, }, // Samara
    {  617, 1596,  786189,  786199,  786219, 10, 20, 24, }, // Samoa
    {  617, 1602,       0,  786243,       0,  0, 13,  0, }, // Seychelles
    {  617, 1624,       0,  786256,       0,  0, 23,  0, }, // Singapore
    {  617, 1634,       0,  786279,       0,  0, 17,  0, }, // Solomon
    {  617, 1642,       0,  786296,       0,  0, 20,  0, }, // South_Georgia
    {  617, 1656,       0,  786316,       0,  0, 13,  0, }, // Suriname
    {  617, 1676,       0,  786329,       0,  0, 11,  0, }, // Syowa
    {  617, 1682,       0,  786340,       0,  0, 10,  0, }, // Tahiti
    {  617, 1689,  786350,  786359,  786378,  9, 19, 23, }, // Taipei
    {  617, 1696,       0,  786401,       0,  0, 16,  0, }, // Tajikistan
    {  617, 1724,       0,  786417,       0,  0, 13,  0, }, // Tokelau
    {  617, 1732,  786430,  786440,  786460, 10, 20, 16, }, // Tonga
    {  617, 1738,       0,  786476,       0,  0,  8,  0, }, // Truk
    {  617, 1750,  786484,  786506,  786538, 22, 32, 28, }, // Turkmenistan
    {  617, 1763,       0,  786566,       0,  0, 11,  0, }, // Tuvalu
    {  617, 1777,  786577,  786589,  786611, 12, 22, 18, }, // Uruguay
    {  617, 1792,  786629,  786647,  786675, 18, 28, 24, }, // Uzbekistan
    {  617, 1803,  786699,  786709,  786729, 10, 20, 16, }, // Vanuatu
    {  617, 1811,       0,  786745,       0,  0, 13,  0, }, // Venezuela
    {  617, 1821,  786758,  786777,  786806, 19, 29, 25, }, // Vladivostok
    {  617, 1833,  786831,  786848,  786875, 17, 27, 23, }, // Volgograd
    {  617, 1843,       0,  786764,       0,  0, 13,  0, }, // Vostok
    {  617, 1850,       0,  786898,       0,  0, 14,  0, }, // Wake
    {  617, 1855,       0,  786912,       0,  0, 24,  0, }, // Wallis
    {  617, 1862,  786936,  786951,  786976, 15, 25, 21, }, // Yakutsk
    {  617, 1870,  786997,  787017,  787047, 20, 30, 26, }, // Yekaterinburg
    {  617, 1892,       0,  787073,       0,  0, 11,  0, }, // Yukon
    {  620,    0,  787084,  787093,  787109,  9, 16, 16, }, // Acre Thai/Thai/Thailand
    {  620,    5,       0,  787125,       0,  0, 15,  0, }, // Afghanistan
    {  620,   17,       0,  787140,       0,  0, 15,  0, }, // Africa_Central
    {  620,   32,       0,  787155,       0,  0, 19,  0, }, // Africa_Eastern
    {  620,   65,       0,  787174,       0,  0, 14,  0, }, // Africa_Southern
    {  620,   81,  787188,  787206,  787231, 18, 25, 25, }, // Africa_Western
    {  620,  107,  787256,  787267,  787285, 11, 18, 20, }, // Alaska
    {  620,  128,  787305,  787316,  787334, 11, 18, 18, }, // Almaty
    {  620,  135,  787352,  787363,  787381, 11, 18, 18, }, // Amazon
    {  620,  142,  787399,  787424,  787456, 25, 32, 31, }, // America_Central
    {  620,  158,  787487,  787516,  787552, 29, 36, 35, }, // America_Eastern
    {  620,  174,  787587,  787613,  787646, 26, 33, 32, }, // America_Mountain
    {  620,  191,  787678,  787703,  787735, 25, 32, 31, }, // America_Pacific
    {  620,  207,  787766,  787778,  787797, 12, 19, 19, }, // Anadyr
    {  620,  214,  787816,  787826,  787843, 10, 17, 16, }, // Apia
    {  620,  219,  787859,  787870,  787888, 11, 18, 18, }, // Aqtau
    {  620,  225,  787906,  787917,  787935, 11, 18, 18, }, // Aqtobe
    {  620,  232,  787953,  787963,  787980, 10, 17, 16, }, // Arabian
    {  620,  240,  787996,  788011,  788033, 15, 22, 22, }, // Argentina
    {  620,  250,  788055,  788080,  788115, 25, 35, 35, }, // Argentina_Western
    {  620,  268,  788150,  788164,  788185, 14, 21, 21, }, // Armenia
    {  620,  286,  788206,  788219,  788239, 13, 20, 22, }, // Atlantic
    {  620,  295,  788261,  788279,  788313, 18, 34, 33, }, // Australia_Central
    {  620,  313,  788346,  788380,  788421, 34, 41, 40, }, // Australia_CentralWestern
    {  620,  338,  788461,  788483,  788518, 22, 35, 34, }, // Australia_Eastern
    {  620,  356,  788552,  788573,  788607, 21, 34, 33, }, // Australia_Western
    {  620,  374,  788640,  788656,  788679, 16, 23, 23, }, // Azerbaijan
    {  620,  385,  788702,  788713,  788731, 11, 18, 18, }, // Azores
    {  620,  397,  788749,  788762,  788782, 13, 20, 20, }, // Bangladesh
    {  620,  415,       0,  788802,       0,  0,  9,  0, }, // Bhutan
    {  620,  422,       0,  788811,       0,  0, 12,  0, }, // Bolivia
    {  620,  437,  788823,  788836,  788856, 13, 20, 20, }, // Brasilia
    {  620,  454,       0,  788876,       0,  0, 19,  0, }, // Brunei
    {  620,  461,  788895,  788908,  788928, 13, 20, 20, }, // Cape_Verde
    {  620,  472,       0,  788948,       0,  0, 10,  0, }, // Casey
    {  620,  478,       0,  788958,       0,  0, 12,  0, }, // Chamorro
    {  620,  487,  788970,  788979,  788995,  9, 16, 15, }, // Chatham
    {  620,  495,  789010,  789018,  789033,  8, 15, 15, }, // Chile
    {  620,  501,  789048,  789055,  789069,  7, 14, 13, }, // China
    {  620,  507,       0,  789082,       0,  0, 17,  0, }, // Christmas
    {  620,  517,       0,  789099,       0,  0, 17,  0, }, // Cocos
    {  620,  523,  789116,  789129,  789149, 13, 20, 20, }, // Colombia
    {  620,  532,  789169,  789184,  789206, 15, 22, 27, }, // Cook
    {  620,  537,  789233,  789242,  789258,  9, 16, 18, }, // Cuba
    {  620,  548,       0,  789276,       0,  0,  9,  0, }, // Davis
    {  620,  564,       0,  789285,       0,  0, 20,  0, }, // DumontDUrville
    {  620,  601,       0,  789305,       0,  0, 18,  0, }, // East_Timor
    {  620,  612,  789323,  789339,  789362, 16, 23, 23, }, // Easter
    {  620,  619,       0,  789385,       0,  0, 13,  0, }, // Ecuador
    {  620,  627,  789398,  789411,  789431, 13, 20, 20, }, // Europe_Central
    {  620,  642,  789451,  789468,  789492, 17, 24, 24, }, // Europe_Eastern
    {  620,  657,       0,  789516,       0,  0, 20,  0, }, // Europe_Further_Eastern
    {  620,  680,  789536,  789552,  789575, 16, 23, 23, }, // Europe_Western
    {  620,  695,  789598,  789620,  789649, 22, 29, 29, }, // Falkland
    {  620,  704,  789678,  789686,  789701,  8, 15, 15, }, // Fiji
    {  620,  709,       0,  789716,       0,  0, 16,  0, }, // French_Guiana
    {  620,  723,       0,  789732,       0,  0, 33,  0, }, // French_Southern
    {  620,  746,       0,  789765,       0,  0, 13,  0, }, // Galapagos
    {  620,  756,       0,  789778,       0,  0, 13,  0, }, // Gambier
    {  620,  764,  789791,  789803,  789822, 12, 19, 19, }, // Georgia
    {  620,  772,       0,  789841,       0,  0, 21,  0, }, // Gilbert_Islands
    {  620,  788,       0,  789862,       0,  0, 17,  0, }, // GMT
    {  620,  830,  789879,  789900,  789928, 21, 28, 28, }, // Greenland_Eastern
    {  620,  848,  789956,  789976,  790003, 20, 27, 27, }, // Greenland_Western
    {  620,  866,       0,  790030,       0,  0,  7,  0, }, // Guam
    {  620,  871,       0,  790037,       0,  0,  9,  0, }, // Gulf
    {  620,  876,       0,  790046,       0,  0, 11,  0, }, // Guyana
    {  620,  883,  790057,  790076,  790102, 19, 26, 25, }, // Hawaii_Aleutian
    {  620,  899,  790127,  790137,  790154, 10, 17, 17, }, // Hong_Kong
    {  620,  909,  790171,  790180,  790196,  9, 16, 16, }, // Hovd
    {  620,  914,       0,  790212,       0,  0, 11,  0, }, // India
    {  620,  920,       0,  790223,       0,  0, 19,  0, }, // Indian_Ocean
    {  620,  933,       0,  790242,       0,  0, 12,  0, }, // Indochina
    {  620,  943,       0,  790254,       0,  0, 22,  0, }, // Indonesia_Central
    {  620,  961,       0,  790276,       0,  0, 27,  0, }, // Indonesia_Eastern
    {  620,  979,       0,  790303,       0,  0, 26,  0, }, // Indonesia_Western
    {  620,  997,  790329,  790340,  790358, 11, 18, 17, }, // Iran
    {  620, 1008,  790375,  790389,  790410, 14, 21, 21, }, // Irkutsk
    {  620, 1016,  790431,  790443,  790462, 12, 19, 18, }, // Israel
    {  620, 1023,  790480,  790491,  790509, 11, 18, 17, }, // Japan
    {  620, 1029,  790526,  790538,  790566, 12, 28, 35, }, // Kamchatka
    {  620, 1047,       0,  790601,       0,  0, 13,  0, }, // Kazakhstan
    {  620, 1058,       0,  790614,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {  620, 1077,       0,  790635,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  620, 1106,  790655,  790665,  790682, 10, 17, 16, }, // Korea
    {  620, 1112,       0,  790698,       0,  0,  9,  0, }, // Kosrae
    {  620, 1119,  790707,  790722,  790744, 15, 22, 22, }, // Krasnoyarsk
    {  620, 1151,       0,  790766,       0,  0, 15,  0, }, // Kyrgystan
    {  620, 1161,       0,  790781,       0,  0,  9,  0, }, // Lanka
    {  620, 1175,       0,  790790,       0,  0, 16,  0, }, // Line_Islands
    {  620, 1188,  790806,  790819,  790839, 13, 20, 19, }, // Lord_Howe
    {  620, 1198,  790858,  790868,  790885, 10, 17, 17, }, // Macau
    {  620, 1204,  790902,  790913,  790931, 11, 18, 18, }, // Magadan
    {  620, 1219,       0,  790949,       0,  0, 12,  0, }, // Malaysia
    {  620, 1228,       0,  790961,       0,  0, 12,  0, }, // Maldives
    {  620, 1237,       0,  790973,       0,  0, 13,  0, }, // Marquesas
    {  620, 1247,       0,  790986,       0,  0, 21,  0, }, // Marshall_Islands
    {  620, 1264,  791007,  791020,  791040, 13, 20, 23, }, // Mauritius
    {  620, 1274,       0,  791063,       0,  0, 11,  0, }, // Mawson
    {  620, 1281,  791074,  791093,  791119, 19, 26, 25, }, // Mexico_Pacific
    {  620, 1296,  791144,  791159,  791181, 15, 22, 22, }, // Mongolia
    {  620, 1305,  791203,  791212,  791228,  9, 16, 16, }, // Moscow
    {  620, 1312,       0,  791244,       0,  0,  8,  0, }, // Myanmar
    {  620, 1320,       0,  791252,       0,  0, 10,  0, }, // Nauru
    {  620, 1326,       0,  791262,       0,  0,  9,  0, }, // Nepal
    {  620, 1332,  791271,  791288,  791312, 17, 24, 24, }, // New_Caledonia
    {  620, 1346,  791336,  791350,  791371, 14, 21, 20, }, // New_Zealand
    {  620, 1358,  791391,  791408,  791432, 17, 24, 23, }, // Newfoundland
    {  620, 1371,       0,  791455,       0,  0, 10,  0, }, // Niue
    {  620, 1376,  791465,  791482,  791506, 17, 24, 23, }, // Norfolk
    {  620, 1384,  791529,  791551,  791580, 22, 29, 32, }, // Noronha
    {  620, 1392,       0,  791612,       0,  0, 25,  0, }, // North_Mariana
    {  620, 1406,  791637,  791652,  791674, 15, 22, 22, }, // Novosibirsk
    {  620, 1418,  791696,  791706,  791723, 10, 17, 17, }, // Omsk
    {  620, 1428,  791740,  791752,  791771, 12, 19, 19, }, // Pakistan
    {  620, 1437,       0,  791790,       0,  0,  9,  0, }, // Palau
    {  620, 1443,       0,  791799,       0,  0, 16,  0, }, // Papua_New_Guinea
    {  620, 1460,  791815,  791827,  791846, 12, 19, 19, }, // Paraguay
    {  620, 1469,  791865,  791873,  791888,  8, 15, 15, }, // Peru
    {  620, 1474,  791903,  791917,  791938, 14, 21, 21, }, // Philippines
    {  620, 1486,       0,  791959,       0,  0, 19,  0, }, // Phoenix_Islands
    {  620, 1502,  791978,  792001,  792031, 23, 30, 32, }, // Pierre_Miquelon
    {  620, 1518,       0,  792063,       0,  0, 12,  0, }, // Pitcairn
    {  620, 1527,       0,  792075,       0,  0, 10,  0, }, // Ponape
    {  620, 1534,       0,  792085,       0,  0, 12,  0, }, // Pyongyang
    {  620, 1544,  792097,  792113,  792136, 16, 23, 23, }, // Qyzylorda
    {  620, 1554,       0,  792159,       0,  0, 12,  0, }, // Reunion
    {  620, 1562,       0,  792171,       0,  0, 10,  0, }, // Rothera
    {  620, 1570,  792181,  792192,  792210, 11, 18, 18, }, // Sakhalin
    {  620, 1579,  792228,  792238,  792255, 10, 17, 17, }, // Samara
    {  620, 1596,  792272,  792281,  792297,  9, 16, 16, }, // Samoa
    {  620, 1602,       0,  792313,       0,  0, 11,  0, }, // Seychelles
    {  620, 1624,       0,  792324,       0,  0, 12,  0, }, // Singapore
    {  620, 1634,       0,  792336,       0,  0, 19,  0, }, // Solomon
    {  620, 1642,       0,  792355,       0,  0, 17,  0, }, // South_Georgia
    {  620, 1656,       0,  792372,       0,  0, 12,  0, }, // Suriname
    {  620, 1676,       0,  792384,       0,  0, 10,  0, }, // Syowa
    {  620, 1682,       0,  792394,       0,  0, 10,  0, }, // Tahiti
    {  620, 1689,  792404,  792412,  792427,  8, 15, 14, }, // Taipei
    {  620, 1696,       0,  792441,       0,  0, 14,  0, }, // Tajikistan
    {  620, 1724,       0,  792455,       0,  0, 11,  0, }, // Tokelau
    {  620, 1732,  792466,  792475,  792491,  9, 16, 16, }, // Tonga
    {  620, 1738,       0,  792507,       0,  0,  7,  0, }, // Truk
    {  620, 1750,  792514,  792532,  792557, 18, 25, 25, }, // Turkmenistan
    {  620, 1763,       0,  792582,       0,  0, 10,  0, }, // Tuvalu
    {  620, 1777,  792592,  792604,  792623, 12, 19, 19, }, // Uruguay
    {  620, 1792,  792642,  792657,  792679, 15, 22, 22, }, // Uzbekistan
    {  620, 1803,  792701,  792713,  792732, 12, 19, 19, }, // Vanuatu
    {  620, 1811,       0,  792751,       0,  0, 14,  0, }, // Venezuela
    {  620, 1821,  792765,  792780,  792802, 15, 22, 22, }, // Vladivostok
    {  620, 1833,  792824,  792837,  792857, 13, 20, 20, }, // Volgograd
    {  620, 1843,       0,  792877,       0,  0, 10,  0, }, // Vostok
    {  620, 1850,       0,  792887,       0,  0, 11,  0, }, // Wake
    {  620, 1855,       0,  792898,       0,  0, 19,  0, }, // Wallis
    {  620, 1862,  792917,  792929,  792948, 12, 19, 19, }, // Yakutsk
    {  620, 1870,  792967,  792985,  793010, 18, 25, 25, }, // Yekaterinburg
    {  620, 1892,       0,  793035,       0,  0,  9,  0, }, // Yukon
    {  624,    0,  793044,  793050,  793064,  6, 14, 11, }, // Acre Tigrinya/Ethiopic/Ethiopia
    {  624,    5,       0,  793075,       0,  0, 13,  0, }, // Afghanistan
    {  624,   17,       0,  793088,       0,  0, 13,  0, }, // Africa_Central
    {  624,   32,       0,  793101,       0,  0, 12,  0, }, // Africa_Eastern
    {  624,   65,       0,  793113,       0,  0, 11,  0, }, // Africa_Southern
    {  624,   81,  793124,  793136,  793156, 12, 20, 17, }, // Africa_Western
    {  624,  107,  793173,  793180,  793195,  7, 15, 15, }, // Alaska
    {  624,  135,  793210,  793217,  793232,  7, 15, 12, }, // Amazon
    {  624,  142,  793244,  793257,  793278, 13, 21, 21, }, // America_Central
    {  624,  158,  793299,  793310,  793331, 11, 21, 21, }, // America_Eastern
    {  624,  174,  793352,  793360,  793378,  8, 18, 18, }, // America_Mountain
    {  624,  191,  793396,  793406,  793421, 10, 15, 15, }, // America_Pacific
    {  624,  214,  793436,  793445,  793459,  9, 14, 14, }, // Apia
    {  624,  232,  793473,  793482,  793496,  9, 14, 14, }, // Arabian
    {  624,  240,  793510,  793519,  793536,  9, 17, 14, }, // Argentina
    {  624,  250,  793550,  793565,  793588, 15, 23, 20, }, // Argentina_Western
    {  624,  268,  793608,  793619,  793635, 11, 16, 16, }, // Armenia
    {  624,  286,  793651,  793663,  793680, 12, 17, 17, }, // Atlantic
    {  624,  295,  793697,  793709,  793732, 12, 23, 23, }, // Australia_Central
    {  624,  313,  793755,  793773,  793796, 18, 23, 23, }, // Australia_CentralWestern
    {  624,  338,  793819,  793838,  793862, 19, 24, 24, }, // Australia_Eastern
    {  624,  356,  793886,  793905,  793929, 19, 24, 24, }, // Australia_Western
    {  624,  374,  793953,  793965,  793982, 12, 17, 17, }, // Azerbaijan
    {  624,  385,  793999,  794009,  794024, 10, 15, 15, }, // Azores
    {  624,  397,  794039,  794051,  794068, 12, 17, 17, }, // Bangladesh
    {  624,  415,       0,  794085,       0,  0, 10,  0, }, // Bhutan
    {  624,  422,       0,  794095,       0,  0,  7,  0, }, // Bolivia
    {  624,  437,  794102,  794110,  794126,  8, 16, 13, }, // Brasilia
    {  624,  454,       0,  794139,       0,  0, 15,  0, }, // Brunei
    {  624,  461,  794154,  794163,  794180,  9, 17, 14, }, // Cape_Verde
    {  624,  478,       0,  794194,       0,  0, 14,  0, }, // Chamorro
    {  624,  487,  794208,  794218,  794233, 10, 15, 15, }, // Chatham
    {  624,  495,  794248,  794253,  794266,  5, 13, 10, }, // Chile
    {  624,  501,  794276,  794285,  794299,  9, 14, 14, }, // China
    {  624,  507,       0,  794313,       0,  0, 13,  0, }, // Christmas
    {  624,  517,       0,  794326,       0,  0, 13,  0, }, // Cocos
    {  624,  523,  794339,  794347,  794363,  8, 16, 13, }, // Colombia
    {  624,  532,  794376,  794388,  794405, 12, 17, 21, }, // Cook
    {  624,  537,  794426,  794434,  794447,  8, 13, 13, }, // Cuba
    {  624,  548,       0,  794460,       0,  0,  9,  0, }, // Davis
    {  624,  564,       0,  794469,       0,  0, 13,  0, }, // DumontDUrville
    {  624,  601,       0,  794482,       0,  0, 14,  0, }, // East_Timor
    {  624,  612,  794496,  794506,  794524, 10, 18, 15, }, // Easter
    {  624,  619,       0,  794539,       0,  0,  7,  0, }, // Ecuador
    {  624,  627,  794546,  794562,  794583, 16, 21, 12, }, // Europe_Central
    {  624,  642,  794595,  794610,  794630, 15, 20, 17, }, // Europe_Eastern
    {  624,  657,       0,  794647,       0,  0, 20,  0, }, // Europe_Further_Eastern
    {  624,  680,  794667,  794684,  794706, 17, 22, 22, }, // Europe_Western
    {  624,  695,  794728,  794741,  794762, 13, 21, 18, }, // Falkland
    {  624,  704,  794780,  794788,  794801,  8, 13, 13, }, // Fiji
    {  624,  709,       0,  794814,       0,  0, 13,  0, }, // French_Guiana
    {  624,  723,       0,  794827,       0,  0, 18,  0, }, // French_Southern
    {  624,  746,       0,  794845,       0,  0,  8,  0, }, // Galapagos
    {  624,  756,       0,  794853,       0,  0, 11,  0, }, // Gambier
    {  624,  764,  794864,  794874,  794889, 10, 15, 15, }, // Georgia
    {  624,  772,       0,  794904,       0,  0, 16,  0, }, // Gilbert_Islands
    {  624,  788,       0,  202954,       0,  0,  3,  0, }, // GMT
    {  624,  830,  794920,  794938,  794961, 18, 23, 23, }, // Greenland_Eastern
    {  624,  848,  794984,  795001,  795023, 17, 22, 22, }, // Greenland_Western
    {  624,  871,       0,  795045,       0,  0, 14,  0, }, // Gulf
    {  624,  876,       0,  795059,       0,  0,  6,  0, }, // Guyana
    {  624,  883,  795065,  795081,  795102, 16, 21, 21, }, // Hawaii_Aleutian
    {  624,  899,  795123,  795136,  795154, 13, 18, 18, }, // Hong_Kong
    {  624,  909,  795172,  795181,  795195,  9, 14, 14, }, // Hovd
    {  624,  914,       0,  795209,       0,  0, 14,  0, }, // India
    {  624,  920,       0,  795223,       0,  0, 13,  0, }, // Indian_Ocean
    {  624,  933,       0,  795236,       0,  0, 12,  0, }, // Indochina
    {  624,  943,       0,  795248,       0,  0, 18,  0, }, // Indonesia_Central
    {  624,  961,       0,  795266,       0,  0, 18,  0, }, // Indonesia_Eastern
    {  624,  979,       0,  795284,       0,  0, 18,  0, }, // Indonesia_Western
    {  624,  997,  795302,  795311,  795325,  9, 14, 14, }, // Iran
    {  624, 1008,  795339,  795351,  795368, 12, 17, 17, }, // Irkutsk
    {  624, 1016,  795385,  795396,  795412, 11, 16, 16, }, // Israel
    {  624, 1023,  795428,  795437,  795451,  9, 14, 14, }, // Japan
    {  624, 1047,       0,  795465,       0,  0, 12,  0, }, // Kazakhstan
    {  624, 1058,       0,  795477,       0,  0, 17,  0, }, // Kazakhstan_Eastern
    {  624, 1077,       0,  795494,       0,  0, 17,  0, }, // Kazakhstan_Western
    {  624, 1106,  795511,  795521,  795536, 10, 15, 15, }, // Korea
    {  624, 1112,       0,  795551,       0,  0, 10,  0, }, // Kosrae
    {  624, 1119,  795561,  795575,  795594, 14, 19, 19, }, // Krasnoyarsk
    {  624, 1151,       0,  795613,       0,  0, 12,  0, }, // Kyrgystan
    {  624, 1175,       0,  795625,       0,  0, 14,  0, }, // Line_Islands
    {  624, 1188,  795639,  795651,  795668, 12, 17, 17, }, // Lord_Howe
    {  624, 1204,  795685,  795695,  795710, 10, 15, 15, }, // Magadan
    {  624, 1219,       0,  795725,       0,  0, 10,  0, }, // Malaysia
    {  624, 1228,       0,  795735,       0,  0, 11,  0, }, // Maldives
    {  624, 1237,       0,  795746,       0,  0, 12,  0, }, // Marquesas
    {  624, 1247,       0,  795758,       0,  0, 15,  0, }, // Marshall_Islands
    {  624, 1264,  795773,  795781,  795797,  8, 16, 13, }, // Mauritius
    {  624, 1274,       0,  795810,       0,  0, 10,  0, }, // Mawson
    {  624, 1281,  795820,  795835,  795855, 15, 20, 20, }, // Mexico_Pacific
    {  624, 1296,  795875,  795887,  795904, 12, 17, 17, }, // Mongolia
    {  624, 1305,  795921,  795931,  795946, 10, 15, 15, }, // Moscow
    {  624, 1312,       0,  795961,       0,  0, 11,  0, }, // Myanmar
    {  624, 1320,       0,  795972,       0,  0,  9,  0, }, // Nauru
    {  624, 1326,       0,  795981,       0,  0,  9,  0, }, // Nepal
    {  624, 1332,  795990,  796004,  796023, 14, 19, 19, }, // New_Caledonia
    {  624, 1346,  796042,  796055,  796073, 13, 18, 18, }, // New_Zealand
    {  624, 1358,  796091,  796106,  796126, 15, 20, 20, }, // Newfoundland
    {  624, 1371,       0,  796146,       0,  0,  8,  0, }, // Niue
    {  624, 1376,  796154,  796169,  796189, 15, 20, 20, }, // Norfolk
    {  624, 1384,  796209,  796224,  796247, 15, 23, 20, }, // Noronha
    {  624, 1406,  796267,  796279,  796296, 12, 17, 17, }, // Novosibirsk
    {  624, 1418,  796313,  796323,  796338, 10, 15, 15, }, // Omsk
    {  624, 1428,  796353,  796364,  796380, 11, 16, 16, }, // Pakistan
    {  624, 1437,       0,  796396,       0,  0,  9,  0, }, // Palau
    {  624, 1443,       0,  796405,       0,  0, 14,  0, }, // Papua_New_Guinea
    {  624, 1460,  796419,  796426,  796441,  7, 15, 12, }, // Paraguay
    {  624, 1469,  796453,  796458,  796471,  5, 13, 10, }, // Peru
    {  624, 1474,  796481,  796491,  796506, 10, 15, 15, }, // Philippines
    {  624, 1486,       0,  796521,       0,  0, 15,  0, }, // Phoenix_Islands
    {  624, 1502,  796536,  796555,  796579, 19, 24, 24, }, // Pierre_Miquelon
    {  624, 1518,       0,  796603,       0,  0, 12,  0, }, // Pitcairn
    {  624, 1527,       0,  796615,       0,  0,  9,  0, }, // Ponape
    {  624, 1534,       0,  796624,       0,  0, 13,  0, }, // Pyongyang
    {  624, 1554,       0,  796637,       0,  0,  8,  0, }, // Reunion
    {  624, 1562,       0,  796645,       0,  0,  9,  0, }, // Rothera
    {  624, 1570,  796654,  796665,  796681, 11, 16, 16, }, // Sakhalin
    {  624, 1596,  796697,  796706,  796720,  9, 14, 14, }, // Samoa
    {  624, 1602,       0,  796734,       0,  0,  7,  0, }, // Seychelles
    {  624, 1624,       0,  796741,       0,  0, 15,  0, }, // Singapore
    {  624, 1634,       0,  796756,       0,  0, 15,  0, }, // Solomon
    {  624, 1642,       0,  796771,       0,  0, 11,  0, }, // South_Georgia
    {  624, 1656,       0,  796782,       0,  0,  7,  0, }, // Suriname
    {  624, 1676,       0,  796789,       0,  0,  9,  0, }, // Syowa
    {  624, 1682,       0,  796798,       0,  0,  9,  0, }, // Tahiti
    {  624, 1689,  796807,  796815,  796828,  8, 13, 13, }, // Taipei
    {  624, 1696,       0,  796841,       0,  0, 12,  0, }, // Tajikistan
    {  624, 1724,       0,  796853,       0,  0, 10,  0, }, // Tokelau
    {  624, 1732,  796863,  796872,  796886,  9, 14, 14, }, // Tonga
    {  624, 1738,       0,  796900,       0,  0,  8,  0, }, // Truk
    {  624, 1750,  796908,  796922,  796941, 14, 19, 19, }, // Turkmenistan
    {  624, 1763,       0,  796960,       0,  0,  9,  0, }, // Tuvalu
    {  624, 1777,  796969,  796976,  796991,  7, 15, 12, }, // Uruguay
    {  624, 1792,  797003,  797016,  797034, 13, 18, 18, }, // Uzbekistan
    {  624, 1803,  797052,  797062,  797077, 10, 15, 15, }, // Vanuatu
    {  624, 1811,       0,  797092,       0,  0,  8,  0, }, // Venezuela
    {  624, 1821,  797100,  797113,  797131, 13, 18, 18, }, // Vladivostok
    {  624, 1833,  797149,  797161,  797178, 12, 17, 17, }, // Volgograd
    {  624, 1843,       0,  797195,       0,  0, 10,  0, }, // Vostok
    {  624, 1850,       0,  797205,       0,  0, 12,  0, }, // Wake
    {  624, 1855,       0,  797217,       0,  0, 13,  0, }, // Wallis
    {  624, 1862,  797230,  797241,  797257, 11, 16, 16, }, // Yakutsk
    {  624, 1870,  797273,  797287,  797306, 14, 19, 19, }, // Yekaterinburg
    {  624, 1892,       0,  797325,       0,  0,  9,  0, }, // Yukon
    {  627,    0,  797334,  797349,  797377, 15, 28, 27, }, // Acre Tongan/Latin/Tonga
    {  627,    5,       0,  797404,       0,  0, 23,  0, }, // Afghanistan
    {  627,   17,       0,  797427,       0,  0, 22,  0, }, // Africa_Central
    {  627,   32,       0,  797449,       0,  0, 24,  0, }, // Africa_Eastern
    {  627,   65,       0,  797473,       0,  0, 23,  0, }, // Africa_Southern
    {  627,   81,  797496,  797520,  797557, 24, 37, 36, }, // Africa_Western
    {  627,  107,  797593,  797610,  797640, 17, 30, 29, }, // Alaska
    {  627,  128,  797669,  797686,  797716, 17, 30, 29, }, // Almaty
    {  627,  135,  797745,  797762,  797792, 17, 30, 29, }, // Amazon
    {  627,  142,  797821,  797851,  797894, 30, 43, 42, }, // America_Central
    {  627,  158,  797936,  797968,  798013, 32, 45, 44, }, // America_Eastern
    {  627,  174,  798057,  798090,  798136, 33, 46, 45, }, // America_Mountain
    {  627,  191,  798181,  798215,  798262, 34, 47, 46, }, // America_Pacific
    {  627,  207,  798308,  798331,  798367, 23, 36, 35, }, // Anadyr
    {  627,  214,  798402,  798415,  798441, 13, 26, 25, }, // Apia
    {  627,  219,  798466,  798482,  798511, 16, 29, 28, }, // Aqtau
    {  627,  225,  798539,  798556,  798586, 17, 30, 29, }, // Aqtobe
    {  627,  232,  798615,  798631,  798660, 16, 29, 28, }, // Arabian
    {  627,  240,  798688,  798707,  798739, 19, 32, 31, }, // Argentina
    {  627,  250,  798770,  798796,  798835, 26, 39, 38, }, // Argentina_Western
    {  627,  268,  798873,  798889,  798918, 16, 29, 28, }, // Armenia
    {  627,  286,  798946,  798984,  799035, 38, 51, 50, }, // Atlantic
    {  627,  295,  799085,  799111,  799150, 26, 39, 38, }, // Australia_Central
    {  627,  313,  799188,  799221,  799267, 33, 46, 45, }, // Australia_CentralWestern
    {  627,  338,  799312,  799340,  799381, 28, 41, 40, }, // Australia_Eastern
    {  627,  356,  799421,  799449,  799490, 28, 41, 40, }, // Australia_Western
    {  627,  374,  799530,  799550,  799583, 20, 33, 32, }, // Azerbaijan
    {  627,  385,  799615,  799632,  799662, 17, 30, 29, }, // Azores
    {  627,  397,  799691,  799711,  799744, 20, 33, 32, }, // Bangladesh
    {  627,  415,       0,  799776,       0,  0, 15,  0, }, // Bhutan
    {  627,  422,       0,  799791,       0,  0, 16,  0, }, // Bolivia
    {  627,  437,  799807,  799824,  799854, 17, 30, 29, }, // Brasilia
    {  627,  454,       0,  799883,       0,  0, 16,  0, }, // Brunei
    {  627,  461,  799899,  799918,  799950, 19, 32, 31, }, // Cape_Verde
    {  627,  472,       0,  799981,       0,  0, 14,  0, }, // Casey
    {  627,  478,       0,  799995,       0,  0, 15,  0, }, // Chamorro
    {  627,  487,  800010,  800027,  800057, 17, 30, 29, }, // Chatham
    {  627,  495,  800086,  800099,  800125, 13, 26, 25, }, // Chile
    {  627,  501,  800150,  800165,  800193, 15, 28, 27, }, // China
    {  627,  507,       0,  800220,       0,  0, 23,  0, }, // Christmas
    {  627,  517,       0,  800243,       0,  0, 19,  0, }, // Cocos
    {  627,  523,  800262,  800280,  800311, 18, 31, 30, }, // Colombia
    {  627,  532,  800341,  800362,  800396, 21, 34, 33, }, // Cook
    {  627,  537,  800429,  800443,  800470, 14, 27, 26, }, // Cuba
    {  627,  548,       0,  800496,       0,  0, 15,  0, }, // Davis
    {  627,  564,       0,  800511,       0,  0, 22,  0, }, // DumontDUrville
    {  627,  601,       0,  800533,       0,  0, 21,  0, }, // East_Timor
    {  627,  612,  800554,  800570,  800599, 16, 29, 28, }, // Easter
    {  627,  619,       0,  800627,       0,  0, 17,  0, }, // Ecuador
    {  627,  627,  800644,  800665,  800699, 21, 34, 33, }, // Europe_Central
    {  627,  642,  800732,  800755,  800791, 23, 36, 35, }, // Europe_Eastern
    {  627,  657,       0,  800826,       0,  0, 28,  0, }, // Europe_Further_Eastern
    {  627,  680,  800854,  800877,  800913, 23, 36, 35, }, // Europe_Western
    {  627,  695,  800948,  800974,  801013, 26, 39, 38, }, // Falkland
    {  627,  704,  801051,  801064,  801090, 13, 26, 25, }, // Fiji
    {  627,  709,       0,  801115,       0,  0, 28,  0, }, // French_Guiana
    {  627,  723,       0,  801143,       0,  0, 27,  0, }, // French_Southern
    {  627,  746,       0,  801170,       0,  0, 19,  0, }, // Galapagos
    {  627,  756,       0,  801189,       0,  0, 16,  0, }, // Gambier
    {  627,  764,  801205,  801220,  801248, 15, 28, 27, }, // Georgia
    {  627,  772,       0,  801275,       0,  0, 17,  0, }, // Gilbert_Islands
    {  627,  788,       0,  801292,       0,  0, 25,  0, }, // GMT
    {  627,  830,  801317,  801342,  801380, 25, 38, 37, }, // Greenland_Eastern
    {  627,  848,  801417,  801442,  801480, 25, 38, 37, }, // Greenland_Western
    {  627,  866,       0,  801517,       0,  0, 14,  0, }, // Guam
    {  627,  871,       0,  801531,       0,  0, 15,  0, }, // Gulf
    {  627,  876,       0,  801115,       0,  0, 15,  0, }, // Guyana
    {  627,  883,  801546,  801569,  801605, 23, 36, 35, }, // Hawaii_Aleutian
    {  627,  899,  801640,  801660,  801693, 20, 33, 32, }, // Hong_Kong
    {  627,  909,  801725,  801740,  801768, 15, 28, 27, }, // Hovd
    {  627,  914,       0,  801795,       0,  0, 16,  0, }, // India
    {  627,  920,       0,  801811,       0,  0, 21,  0, }, // Indian_Ocean
    {  627,  933,       0,  801832,       0,  0, 21,  0, }, // Indochina
    {  627,  943,       0,  801853,       0,  0, 25,  0, }, // Indonesia_Central
    {  627,  961,       0,  801878,       0,  0, 27,  0, }, // Indonesia_Eastern
    {  627,  979,       0,  801905,       0,  0, 27,  0, }, // Indonesia_Western
    {  627,  997,  801932,  801948,  801977, 16, 29, 28, }, // Iran
    {  627, 1008,  802005,  802030,  802068, 25, 38, 37, }, // Irkutsk
    {  627, 1016,  802105,  802122,  802152, 17, 30, 29, }, // Israel
    {  627, 1023,  802181,  802197,  802226, 16, 29, 28, }, // Japan
    {  627, 1029,  802254,  802287,  802333, 33, 46, 45, }, // Kamchatka
    {  627, 1047,       0,  802378,       0,  0, 19,  0, }, // Kazakhstan
    {  627, 1058,       0,  802397,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  627, 1077,       0,  802423,       0,  0, 26,  0, }, // Kazakhstan_Western
    {  627, 1106,  802449,  802463,  802490, 14, 27, 26, }, // Korea
    {  627, 1112,       0,  802516,       0,  0, 16,  0, }, // Kosrae
    {  627, 1119,  802532,  802561,  802603, 29, 42, 41, }, // Krasnoyarsk
    {  627, 1151,       0,  802644,       0,  0, 19,  0, }, // Kyrgystan
    {  627, 1161,       0,  802663,       0,  0, 16,  0, }, // Lanka
    {  627, 1175,       0,  802679,       0,  0, 22,  0, }, // Line_Islands
    {  627, 1188,  802701,  802723,  802758, 22, 35, 34, }, // Lord_Howe
    {  627, 1198,  802792,  802806,  802833, 14, 27, 26, }, // Macau
    {  627, 1204,  802859,  802882,  802918, 23, 36, 35, }, // Magadan
    {  627, 1219,       0,  802953,       0,  0, 17,  0, }, // Malaysia
    {  627, 1228,       0,  802970,       0,  0, 19,  0, }, // Maldives
    {  627, 1237,       0,  802989,       0,  0, 17,  0, }, // Marquesas
    {  627, 1247,       0,  803006,       0,  0, 23,  0, }, // Marshall_Islands
    {  627, 1264,  803029,  803048,  803080, 19, 32, 31, }, // Mauritius
    {  627, 1274,       0,  803111,       0,  0, 16,  0, }, // Mawson
    {  627, 1281,  803127,  803154,  803194, 27, 40, 39, }, // Mexico_Pacific
    {  627, 1296,  803233,  803252,  803284, 19, 32, 31, }, // Mongolia
    {  627, 1305,  803315,  803337,  803372, 22, 35, 34, }, // Moscow
    {  627, 1312,       0,  803406,       0,  0, 13,  0, }, // Myanmar
    {  627, 1320,       0,  803419,       0,  0, 14,  0, }, // Nauru
    {  627, 1326,       0,  803433,       0,  0, 15,  0, }, // Nepal
    {  627, 1332,  803448,  803471,  803507, 23, 36, 35, }, // New_Caledonia
    {  627, 1346,  803542,  803559,  803589, 17, 30, 29, }, // New_Zealand
    {  627, 1358,  803618,  803641,  803677, 23, 36, 35, }, // Newfoundland
    {  627, 1371,       0,  803712,       0,  0, 13,  0, }, // Niue
    {  627, 1376,  803725,  803741,  803770, 16, 29, 28, }, // Norfolk
    {  627, 1384,  803798,  803826,  803867, 28, 41, 40, }, // Noronha
    {  627, 1392,       0,  803907,       0,  0, 23,  0, }, // North_Mariana
    {  627, 1406,  803930,  803957,  803997, 27, 40, 39, }, // Novosibirsk
    {  627, 1418,  804036,  804059,  804095, 23, 36, 35, }, // Omsk
    {  627, 1428,  804130,  804149,  804181, 19, 32, 31, }, // Pakistan
    {  627, 1437,       0,  804212,       0,  0, 14,  0, }, // Palau
    {  627, 1443,       0,  804226,       0,  0, 21,  0, }, // Papua_New_Guinea
    {  627, 1460,  804247,  804264,  804294, 17, 30, 29, }, // Paraguay
    {  627, 1469,  804323,  804336,  804362, 13, 26, 25, }, // Peru
    {  627, 1474,  804387,  804405,  804436, 18, 31, 30, }, // Philippines
    {  627, 1486,       0,  804466,       0,  0, 26,  0, }, // Phoenix_Islands
    {  627, 1502,  804492,  804520,  804561, 28, 41, 40, }, // Pierre_Miquelon
    {  627, 1518,       0,  804601,       0,  0, 17,  0, }, // Pitcairn
    {  627, 1527,       0,  804618,       0,  0, 15,  0, }, // Ponape
    {  627, 1534,       0,  804633,       0,  0, 19,  0, }, // Pyongyang
    {  627, 1544,  804652,  804669,  804699, 17, 30, 29, }, // Qyzylorda
    {  627, 1554,       0,  804728,       0,  0, 17,  0, }, // Reunion
    {  627, 1562,       0,  804745,       0,  0, 15,  0, }, // Rothera
    {  627, 1570,  804760,  804783,  804819, 23, 36, 35, }, // Sakhalin
    {  627, 1579,  804854,  804875,  804909, 21, 34, 33, }, // Samara
    {  627, 1596,  804942,  804958,  804987, 16, 29, 28, }, // Samoa
    {  627, 1602,       0,  805015,       0,  0, 25,  0, }, // Seychelles
    {  627, 1624,       0,  805040,       0,  0, 17,  0, }, // Singapore
    {  627, 1634,       0,  805057,       0,  0, 25,  0, }, // Solomon
    {  627, 1642,       0,  805082,       0,  0, 20,  0, }, // South_Georgia
    {  627, 1656,       0,  805102,       0,  0, 17,  0, }, // Suriname
    {  627, 1676,       0,  805119,       0,  0, 14,  0, }, // Syowa
    {  627, 1682,       0,  805133,       0,  0, 15,  0, }, // Tahiti
    {  627, 1689,  805148,  805163,  805191, 15, 28, 27, }, // Taipei
    {  627, 1696,       0,  805218,       0,  0, 19,  0, }, // Tajikistan
    {  627, 1724,       0,  805237,       0,  0, 16,  0, }, // Tokelau
    {  627, 1732,  805253,  805267,  805294, 14, 27, 26, }, // Tonga
    {  627, 1738,       0,  805320,       0,  0, 13,  0, }, // Truk
    {  627, 1750,  805333,  805356,  805392, 23, 36, 35, }, // Turkmenistan
    {  627, 1763,       0,  805427,       0,  0, 15,  0, }, // Tuvalu
    {  627, 1777,  805442,  805459,  805489, 17, 30, 29, }, // Uruguay
    {  627, 1792,  805518,  805539,  805573, 21, 34, 33, }, // Uzbekistan
    {  627, 1803,  805606,  805622,  805651, 16, 29, 28, }, // Vanuatu
    {  627, 1811,       0,  805679,       0,  0, 18,  0, }, // Venezuela
    {  627, 1821,  805697,  805726,  805768, 29, 42, 41, }, // Vladivostok
    {  627, 1833,  805809,  805836,  805876, 27, 40, 39, }, // Volgograd
    {  627, 1843,       0,  805915,       0,  0, 17,  0, }, // Vostok
    {  627, 1850,       0,  805932,       0,  0, 17,  0, }, // Wake
    {  627, 1855,       0,  805949,       0,  0, 24,  0, }, // Wallis
    {  627, 1862,  805973,  805999,  806038, 26, 39, 38, }, // Yakutsk
    {  627, 1870,  806076,  806106,  806149, 30, 43, 42, }, // Yekaterinburg
    {  627, 1892,       0,  806191,       0,  0, 15,  0, }, // Yukon
    {  629,  788,       0,  806206,       0,  0, 49,  0, }, // GMT Tswana/Latin/South Africa
    {  631,    0,  806255,  806265,  806284, 10, 19, 14, }, // Acre Turkish/Latin/Turkey
    {  631,    5,       0,  806298,       0,  0, 16,  0, }, // Afghanistan
    {  631,   17,       0,  806314,       0,  0, 17,  0, }, // Africa_Central
    {  631,   32,       0,  806331,       0,  0, 17,  0, }, // Africa_Eastern
    {  631,   65,       0,  806348,       0,  0, 27,  0, }, // Africa_Southern
    {  631,   81,  806375,  806392,  806418, 17, 26, 21, }, // Africa_Western
    {  631,  107,  806439,  806451,  806472, 12, 21, 16, }, // Alaska
    {  631,  128,  806488,  806500,  806521, 12, 21, 16, }, // Almaty
    {  631,  135,  806537,  806549,  806570, 12, 21, 16, }, // Amazon
    {  631,  142,  806586,  806613,  806649, 27, 36, 31, }, // America_Central
    {  631,  158,  806680,  806704,  806737, 24, 33, 28, }, // America_Eastern
    {  631,  174,  806765,  806788,  806820, 23, 32, 27, }, // America_Mountain
    {  631,  191,  806847,  806874,  806910, 27, 36, 31, }, // America_Pacific
    {  631,  207,  806941,  806953,  806974, 12, 21, 16, }, // Anadyr
    {  631,  214,  806990,  807000,  807019, 10, 19, 14, }, // Apia
    {  631,  219,  807033,  807044,  807064, 11, 20, 15, }, // Aqtau
    {  631,  225,  807079,  807091,  807112, 12, 21, 16, }, // Aqtobe
    {  631,  232,  807128,  807143,  807167, 15, 24, 19, }, // Arabian
    {  631,  240,  807186,  807200,  807223, 14, 23, 18, }, // Argentina
    {  631,  250,  807241,  807260,  807288, 19, 28, 23, }, // Argentina_Western
    {  631,  268,  807311,  807327,  807352, 16, 25, 20, }, // Armenia
    {  631,  286,  807372,  807386,  807409, 14, 23, 18, }, // Atlantic
    {  631,  295,  807427,  807448,  807478, 21, 30, 25, }, // Australia_Central
    {  631,  313,  807503,  807527,  807560, 24, 33, 28, }, // Australia_CentralWestern
    {  631,  338,  807588,  807609,  807639, 21, 30, 25, }, // Australia_Eastern
    {  631,  356,  807506,  807530,  807563, 21, 30, 25, }, // Australia_Western
    {  631,  374,  807664,  807680,  807705, 16, 25, 20, }, // Azerbaijan
    {  631,  385,  807725,  807738,  807760, 13, 22, 17, }, // Azores
    {  631,  397,  807777,  807792,  807816, 15, 24, 19, }, // Bangladesh
    {  631,  415,       0,  807835,       0,  0, 11,  0, }, // Bhutan
    {  631,  422,       0,  807846,       0,  0, 13,  0, }, // Bolivia
    {  631,  437,  807859,  807873,  807896, 14, 23, 18, }, // Brasilia
    {  631,  454,       0,  807914,       0,  0, 25,  0, }, // Brunei
    {  631,  461,  807939,  807955,  807980, 16, 25, 20, }, // Cape_Verde
    {  631,  472,       0,  808000,       0,  0, 11,  0, }, // Casey
    {  631,  478,       0,  808011,       0,  0, 14,  0, }, // Chamorro
    {  631,  487,  808025,  808038,  808060, 13, 22, 17, }, // Chatham
    {  631,  495,  808077,  808087,  808106, 10, 19, 14, }, // Chile
    {  631,  501,  808120,  808129,  808147,  9, 18, 13, }, // China
    {  631,  507,       0,  808160,       0,  0, 21,  0, }, // Christmas
    {  631,  517,       0,  808181,       0,  0, 19,  0, }, // Cocos
    {  631,  523,  808200,  808215,  808239, 15, 24, 19, }, // Colombia
    {  631,  532,  808258,  808276,  808303, 18, 27, 27, }, // Cook
    {  631,  537,  808330,  808340,  808359, 10, 19, 14, }, // Cuba
    {  631,  548,       0,  808373,       0,  0, 11,  0, }, // Davis
    {  631,  564,       0,  808384,       0,  0, 22,  0, }, // DumontDUrville
    {  631,  601,       0,  808406,       0,  0, 16,  0, }, // East_Timor
    {  631,  612,  808422,  808442,  808471, 20, 29, 24, }, // Easter
    {  631,  619,       0,  808495,       0,  0, 13,  0, }, // Ecuador
    {  631,  627,  808508,  808525,  808551, 17, 26, 21, }, // Europe_Central
    {  631,  642,  808572,  808589,  808615, 17, 26, 21, }, // Europe_Eastern
    {  631,  657,       0,  808636,       0,  0, 23,  0, }, // Europe_Further_Eastern
    {  631,  680,  808659,  808676,  808702, 17, 26, 21, }, // Europe_Western
    {  631,  695,  808723,  808745,  808776, 22, 31, 26, }, // Falkland
    {  631,  704,  808802,  808812,  808831, 10, 19, 14, }, // Fiji
    {  631,  709,       0,  808845,       0,  0, 22,  0, }, // French_Guiana
    {  631,  723,       0,  808867,       0,  0, 33,  0, }, // French_Southern
    {  631,  746,       0,  808900,       0,  0, 15,  0, }, // Galapagos
    {  631,  756,       0,  808915,       0,  0, 13,  0, }, // Gambier
    {  631,  764,  808928,  808943,  808967, 15, 24, 19, }, // Georgia
    {  631,  772,       0,  808986,       0,  0, 21,  0, }, // Gilbert_Islands
    {  631,  788,       0,  809007,       0,  0, 24,  0, }, // GMT
    {  631,  830,  809031,  809050,  809078, 19, 28, 23, }, // Greenland_Eastern
    {  631,  848,  809101,  809120,  809148, 19, 28, 23, }, // Greenland_Western
    {  631,  866,       0,  809171,       0,  0, 19,  0, }, // Guam
    {  631,  871,       0,  809190,       0,  0, 12,  0, }, // Gulf
    {  631,  876,       0,  809202,       0,  0, 12,  0, }, // Guyana
    {  631,  883,  809214,  809232,  809259, 18, 27, 22, }, // Hawaii_Aleutian
    {  631,  899,  809281,  809296,  809320, 15, 24, 19, }, // Hong_Kong
    {  631,  909,  809339,  809349,  809368, 10, 19, 14, }, // Hovd
    {  631,  914,       0,  809382,       0,  0, 24,  0, }, // India
    {  631,  920,       0,  809406,       0,  0, 19,  0, }, // Indian_Ocean
    {  631,  933,       0,  809425,       0,  0, 14,  0, }, // Indochina
    {  631,  943,       0,  809439,       0,  0, 20,  0, }, // Indonesia_Central
    {  631,  961,       0,  809459,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  631,  979,       0,  809479,       0,  0, 20,  0, }, // Indonesia_Western
    {  631,  997,  809499,  809509,  809528, 10, 19, 14, }, // Iran
    {  631, 1008,  809542,  809555,  809577, 13, 22, 17, }, // Irkutsk
    {  631, 1016,  809594,  809606,  809627, 12, 21, 16, }, // Israel
    {  631, 1023,  809643,  809656,  809678, 13, 22, 17, }, // Japan
    {  631, 1029,  809695,  809724,  809762, 29, 38, 33, }, // Kamchatka
    {  631, 1047,       0,  809795,       0,  0, 16,  0, }, // Kazakhstan
    {  631, 1058,       0,  809811,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {  631, 1077,       0,  809832,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  631, 1106,  809853,  809863,  809882, 10, 19, 14, }, // Korea
    {  631, 1112,       0,  809896,       0,  0, 12,  0, }, // Kosrae
    {  631, 1119,  809908,  809925,  809951, 17, 26, 21, }, // Krasnoyarsk
    {  631, 1151,       0,  809972,       0,  0, 17,  0, }, // Kyrgystan
    {  631, 1161,       0,  809989,       0,  0, 11,  0, }, // Lanka
    {  631, 1175,       0,  810000,       0,  0, 18,  0, }, // Line_Islands
    {  631, 1188,  810018,  810033,  810057, 15, 24, 19, }, // Lord_Howe
    {  631, 1198,  810076,  810087,  810107, 11, 20, 15, }, // Macau
    {  631, 1204,  810122,  810135,  810157, 13, 22, 17, }, // Magadan
    {  631, 1219,       0,  810174,       0,  0, 13,  0, }, // Malaysia
    {  631, 1228,       0,  810187,       0,  0, 15,  0, }, // Maldives
    {  631, 1237,       0,  810202,       0,  0, 20,  0, }, // Marquesas
    {  631, 1247,       0,  810222,       0,  0, 22,  0, }, // Marshall_Islands
    {  631, 1264,  810244,  810259,  810283, 15, 24, 19, }, // Mauritius
    {  631, 1274,       0,  810302,       0,  0, 12,  0, }, // Mawson
    {  631, 1281,  810314,  810342,  810379, 28, 37, 32, }, // Mexico_Pacific
    {  631, 1296,  810411,  810427,  810452, 16, 25, 20, }, // Mongolia
    {  631, 1305,  810472,  810485,  810507, 13, 22, 17, }, // Moscow
    {  631, 1312,       0,  810524,       0,  0, 13,  0, }, // Myanmar
    {  631, 1320,       0,  810537,       0,  0, 11,  0, }, // Nauru
    {  631, 1326,       0,  810548,       0,  0, 11,  0, }, // Nepal
    {  631, 1332,  810559,  810579,  810608, 20, 29, 24, }, // New_Caledonia
    {  631, 1346,  810632,  810650,  810677, 18, 27, 22, }, // New_Zealand
    {  631, 1358,  810699,  810717,  810744, 18, 27, 22, }, // Newfoundland
    {  631, 1371,       0,  810766,       0,  0, 10,  0, }, // Niue
    {  631, 1376,  810776,  810795,  810823, 19, 28, 23, }, // Norfolk
    {  631, 1384,  810846,  810871,  810905, 25, 34, 29, }, // Noronha
    {  631, 1392,       0,  810934,       0,  0, 27,  0, }, // North_Mariana
    {  631, 1406,  810961,  810978,  811004, 17, 26, 21, }, // Novosibirsk
    {  631, 1418,  811025,  811035,  811054, 10, 19, 14, }, // Omsk
    {  631, 1428,  811068,  811082,  811105, 14, 23, 18, }, // Pakistan
    {  631, 1437,       0,  811123,       0,  0, 11,  0, }, // Palau
    {  631, 1443,       0,  811134,       0,  0, 21,  0, }, // Papua_New_Guinea
    {  631, 1460,  811155,  811169,  811192, 14, 23, 18, }, // Paraguay
    {  631, 1469,  811210,  811220,  811239, 10, 19, 14, }, // Peru
    {  631, 1474,  811253,  811269,  811294, 16, 25, 20, }, // Philippines
    {  631, 1486,       0,  811314,       0,  0, 21,  0, }, // Phoenix_Islands
    {  631, 1502,  811335,  811365,  811404, 30, 39, 34, }, // Pierre_Miquelon
    {  631, 1518,       0,  811438,       0,  0, 14,  0, }, // Pitcairn
    {  631, 1527,       0,  811452,       0,  0, 12,  0, }, // Ponape
    {  631, 1534,       0,  811464,       0,  0, 15,  0, }, // Pyongyang
    {  631, 1544,  811479,  811494,  811518, 15, 24, 19, }, // Qyzylorda
    {  631, 1554,       0,  811537,       0,  0, 13,  0, }, // Reunion
    {  631, 1562,       0,  811550,       0,  0, 13,  0, }, // Rothera
    {  631, 1570,  811563,  811576,  811598, 13, 22, 17, }, // Sakhalin
    {  631, 1579,  811615,  811627,  811648, 12, 21, 16, }, // Samara
    {  631, 1596,  811664,  811675,  811695, 11, 20, 15, }, // Samoa
    {  631, 1602,       0,  811710,       0,  0, 15,  0, }, // Seychelles
    {  631, 1624,       0,  811725,       0,  0, 23,  0, }, // Singapore
    {  631, 1634,       0,  811748,       0,  0, 21,  0, }, // Solomon
    {  631, 1642,       0,  811769,       0,  0, 19,  0, }, // South_Georgia
    {  631, 1656,       0,  811788,       0,  0, 13,  0, }, // Suriname
    {  631, 1676,       0,  811801,       0,  0, 11,  0, }, // Syowa
    {  631, 1682,       0,  811812,       0,  0, 12,  0, }, // Tahiti
    {  631, 1689,  811824,  811836,  811857, 12, 21, 16, }, // Taipei
    {  631, 1696,       0,  811873,       0,  0, 16,  0, }, // Tajikistan
    {  631, 1724,       0,  811889,       0,  0, 13,  0, }, // Tokelau
    {  631, 1732,  811902,  811913,  811933, 11, 20, 15, }, // Tonga
    {  631, 1738,       0,  811948,       0,  0, 11,  0, }, // Truk
    {  631, 1750,  811959,  811977,  812004, 18, 27, 22, }, // Turkmenistan
    {  631, 1763,       0,  812026,       0,  0, 12,  0, }, // Tuvalu
    {  631, 1777,  812038,  812051,  812073, 13, 22, 17, }, // Uruguay
    {  631, 1792,  812090,  812106,  812131, 16, 25, 20, }, // Uzbekistan
    {  631, 1803,  812151,  812164,  812186, 13, 22, 17, }, // Vanuatu
    {  631, 1811,       0,  812203,       0,  0, 15,  0, }, // Venezuela
    {  631, 1821,  812218,  812235,  812261, 17, 26, 21, }, // Vladivostok
    {  631, 1833,  812282,  812297,  812321, 15, 24, 19, }, // Volgograd
    {  631, 1843,       0,  812340,       0,  0, 12,  0, }, // Vostok
    {  631, 1850,       0,  812352,       0,  0, 16,  0, }, // Wake
    {  631, 1855,       0,  812368,       0,  0, 22,  0, }, // Wallis
    {  631, 1862,  812390,  812403,  812425, 13, 22, 17, }, // Yakutsk
    {  631, 1870,  812442,  812461,  812489, 19, 28, 23, }, // Yekaterinburg
    {  631, 1892,       0,  812512,       0,  0, 11,  0, }, // Yukon
    {  633,    5,       0,  812523,       0,  0, 16,  0, }, // Afghanistan Turkmen/Latin/Turkmenistan
    {  633,   17,       0,  812539,       0,  0, 20,  0, }, // Africa_Central
    {  633,   32,       0,  812559,       0,  0, 21,  0, }, // Africa_Eastern
    {  633,   65,       0,  812580,       0,  0, 29,  0, }, // Africa_Southern
    {  633,   81,  812609,  812630,  812660, 21, 30, 29, }, // Africa_Western
    {  633,  107,  812689,  812702,  812724, 13, 22, 21, }, // Alaska
    {  633,  135,  812745,  812757,  812778, 12, 21, 20, }, // Amazon
    {  633,  142,  812798,  812813,  812843, 15, 30, 29, }, // America_Central
    {  633,  158,  812872,  812905,  812947, 33, 42, 41, }, // America_Eastern
    {  633,  174,  812988,  813016,  813053, 28, 37, 36, }, // America_Mountain
    {  633,  191,  813089,  813125,  813170, 36, 45, 44, }, // America_Pacific
    {  633,  207,  813214,       0,       0, 12,  0,  0, }, // Anadyr
    {  633,  214,  813226,  813236,  813255, 10, 19, 18, }, // Apia
    {  633,  232,  813273,  813292,  813320, 19, 28, 27, }, // Arabian
    {  633,  240,  813347,  813362,  813386, 15, 24, 23, }, // Argentina
    {  633,  250,  813409,  813433,  813466, 24, 33, 32, }, // Argentina_Western
    {  633,  268,  813498,  813514,  813539, 16, 25, 24, }, // Armenia
    {  633,  286,  813563,  813577,  813600, 14, 23, 22, }, // Atlantic
    {  633,  295,  813622,  813646,  813679, 24, 33, 32, }, // Australia_Central
    {  633,  313,  813711,  813744,  813786, 33, 42, 41, }, // Australia_CentralWestern
    {  633,  338,  813827,  813852,  813886, 25, 34, 33, }, // Australia_Eastern
    {  633,  356,  813919,  813944,  813978, 25, 34, 33, }, // Australia_Western
    {  633,  374,  814011,  814027,  814052, 16, 25, 24, }, // Azerbaijan
    {  633,  385,  814076,  814094,  814121, 18, 27, 26, }, // Azores
    {  633,  397,  814147,  814162,  814186, 15, 24, 23, }, // Bangladesh
    {  633,  415,       0,  814209,       0,  0, 11,  0, }, // Bhutan
    {  633,  422,       0,  814220,       0,  0, 14,  0, }, // Bolivia
    {  633,  437,  814234,  814249,  814273, 15, 24, 23, }, // Brasilia
    {  633,  454,       0,  814296,       0,  0, 23,  0, }, // Brunei
    {  633,  461,  814319,  814335,  814360, 16, 25, 24, }, // Cape_Verde
    {  633,  478,       0,  814384,       0,  0, 13,  0, }, // Chamorro
    {  633,  487,  814397,  814408,  814428, 11, 20, 19, }, // Chatham
    {  633,  495,  814447,  814457,  814476, 10, 19, 18, }, // Chile
    {  633,  501,  814494,  814505,  814525, 11, 20, 19, }, // China
    {  633,  507,       0,  814544,       0,  0, 21,  0, }, // Christmas
    {  633,  517,       0,  814565,       0,  0, 19,  0, }, // Cocos
    {  633,  523,  814584,  814599,  814623, 15, 24, 23, }, // Colombia
    {  633,  532,  814646,  814663,  814689, 17, 26, 25, }, // Cook
    {  633,  537,  814714,  814724,  814743, 10, 19, 18, }, // Cuba
    {  633,  548,       0,  814761,       0,  0, 12,  0, }, // Davis
    {  633,  564,       0,  814773,       0,  0, 21,  0, }, // DumontDUrville
    {  633,  601,       0,  814794,       0,  0, 20,  0, }, // East_Timor
    {  633,  612,  814814,  814831,  814857, 17, 26, 25, }, // Easter
    {  633,  619,       0,  814882,       0,  0, 13,  0, }, // Ecuador
    {  633,  627,  814895,  814916,  814946, 21, 30, 29, }, // Europe_Central
    {  633,  642,  814975,  814997,  815028, 22, 31, 30, }, // Europe_Eastern
    {  633,  657,       0,  815058,       0,  0, 27,  0, }, // Europe_Further_Eastern
    {  633,  680,  815085,  815107,  815138, 22, 31, 30, }, // Europe_Western
    {  633,  695,  815168,  815190,  815221, 22, 31, 30, }, // Falkland
    {  633,  704,  815251,  815261,  815280, 10, 19, 18, }, // Fiji
    {  633,  709,       0,  815298,       0,  0, 22,  0, }, // French_Guiana
    {  633,  723,       0,  815320,       0,  0, 43,  0, }, // French_Southern
    {  633,  746,       0,  815363,       0,  0, 23,  0, }, // Galapagos
    {  633,  756,       0,  815386,       0,  0, 12,  0, }, // Gambier
    {  633,  764,  815398,  815411,  815433, 13, 22, 21, }, // Georgia
    {  633,  772,       0,  815454,       0,  0, 21,  0, }, // Gilbert_Islands
    {  633,  788,       0,  815475,       0,  0, 20,  0, }, // GMT
    {  633,  830,  815495,  815521,  815556, 26, 35, 34, }, // Greenland_Eastern
    {  633,  848,  815590,  815616,  815651, 26, 35, 34, }, // Greenland_Western
    {  633,  871,       0,  815685,       0,  0, 26,  0, }, // Gulf
    {  633,  876,       0,  815711,       0,  0, 12,  0, }, // Guyana
    {  633,  883,  815723,  815740,  815766, 17, 26, 25, }, // Hawaii_Aleutian
    {  633,  899,  815791,  815804,  815826, 13, 22, 21, }, // Hong_Kong
    {  633,  909,  815847,  815857,  815876, 10, 19, 18, }, // Hovd
    {  633,  914,       0,  815894,       0,  0, 24,  0, }, // India
    {  633,  920,       0,  815918,       0,  0, 18,  0, }, // Indian_Ocean
    {  633,  933,       0,  815936,       0,  0, 16,  0, }, // Indochina
    {  633,  943,       0,  815952,       0,  0, 24,  0, }, // Indonesia_Central
    {  633,  961,       0,  815976,       0,  0, 25,  0, }, // Indonesia_Eastern
    {  633,  979,       0,  816001,       0,  0, 25,  0, }, // Indonesia_Western
    {  633,  997,  816026,  816037,  816057, 11, 20, 19, }, // Iran
    {  633, 1008,  816076,  816089,  816111, 13, 22, 21, }, // Irkutsk
    {  633, 1016,  816132,  816145,  816167, 13, 22, 21, }, // Israel
    {  633, 1023,  816188,  816202,  816225, 14, 23, 22, }, // Japan
    {  633, 1029,  816247,       0,       0, 30,  0,  0, }, // Kamchatka
    {  633, 1047,       0,  816277,       0,  0, 16,  0, }, // Kazakhstan
    {  633, 1058,       0,  816293,       0,  0, 25,  0, }, // Kazakhstan_Eastern
    {  633, 1077,       0,  816318,       0,  0, 25,  0, }, // Kazakhstan_Western
    {  633, 1106,  816343,  816355,  816376, 12, 21, 20, }, // Korea
    {  633, 1112,       0,  816396,       0,  0, 13,  0, }, // Kosrae
    {  633, 1119,  816409,  816426,  816452, 17, 26, 25, }, // Krasnoyarsk
    {  633, 1151,       0,  816477,       0,  0, 17,  0, }, // Kyrgystan
    {  633, 1175,       0,  816494,       0,  0, 18,  0, }, // Line_Islands
    {  633, 1188,  816512,  816526,  816549, 14, 23, 22, }, // Lord_Howe
    {  633, 1204,  816571,  816584,  816606, 13, 22, 21, }, // Magadan
    {  633, 1219,       0,  816627,       0,  0, 15,  0, }, // Malaysia
    {  633, 1228,       0,  816642,       0,  0, 15,  0, }, // Maldives
    {  633, 1237,       0,  816657,       0,  0, 20,  0, }, // Marquesas
    {  633, 1247,       0,  816677,       0,  0, 21,  0, }, // Marshall_Islands
    {  633, 1264,  816698,  816712,  816735, 14, 23, 22, }, // Mauritius
    {  633, 1274,       0,  816757,       0,  0, 12,  0, }, // Mawson
    {  633, 1281,  816769,  816795,  816830, 26, 35, 34, }, // Mexico_Pacific
    {  633, 1296,  816864,  816880,  816905, 16, 25, 24, }, // Mongolia
    {  633, 1305,  816929,  816941,  816962, 12, 21, 20, }, // Moscow
    {  633, 1312,       0,  816982,       0,  0, 12,  0, }, // Myanmar
    {  633, 1320,       0,  816994,       0,  0, 11,  0, }, // Nauru
    {  633, 1326,       0,  817005,       0,  0, 11,  0, }, // Nepal
    {  633, 1332,  817016,  817037,  817067, 21, 30, 29, }, // New_Caledonia
    {  633, 1346,  817096,  817116,  817145, 20, 29, 28, }, // New_Zealand
    {  633, 1358,  817173,  817191,  817218, 18, 27, 26, }, // Newfoundland
    {  633, 1371,       0,  817244,       0,  0, 10,  0, }, // Niue
    {  633, 1376,  817254,  817273,  817301, 19, 28, 27, }, // Norfolk
    {  633, 1384,  817328,  817353,  817387, 25, 34, 33, }, // Noronha
    {  633, 1406,  817420,  817437,  817463, 17, 26, 24, }, // Novosibirsk
    {  633, 1418,  817487,  817497,  817516, 10, 19, 18, }, // Omsk
    {  633, 1428,  817534,  817548,  817571, 14, 23, 22, }, // Pakistan
    {  633, 1437,       0,  817593,       0,  0, 11,  0, }, // Palau
    {  633, 1443,       0,  817604,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  633, 1460,  817630,  817644,  817667, 14, 23, 22, }, // Paraguay
    {  633, 1469,  817689,  817699,  817718, 10, 19, 18, }, // Peru
    {  633, 1474,  817736,  817753,  817779, 17, 26, 25, }, // Philippines
    {  633, 1486,       0,  817804,       0,  0, 20,  0, }, // Phoenix_Islands
    {  633, 1502,  817824,  817843,  817877, 19, 34, 33, }, // Pierre_Miquelon
    {  633, 1518,       0,  817910,       0,  0, 13,  0, }, // Pitcairn
    {  633, 1527,       0,  817923,       0,  0, 12,  0, }, // Ponape
    {  633, 1534,       0,  817935,       0,  0, 13,  0, }, // Pyongyang
    {  633, 1554,       0,  817948,       0,  0, 14,  0, }, // Reunion
    {  633, 1562,       0,  817962,       0,  0, 12,  0, }, // Rothera
    {  633, 1570,  817974,  817987,  818009, 13, 22, 21, }, // Sakhalin
    {  633, 1579,  818030,       0,       0, 12,  0,  0, }, // Samara
    {  633, 1596,  818042,  818053,  818073, 11, 20, 19, }, // Samoa
    {  633, 1602,       0,  818092,       0,  0, 20,  0, }, // Seychelles
    {  633, 1624,       0,  818112,       0,  0, 14,  0, }, // Singapore
    {  633, 1634,       0,  818126,       0,  0, 21,  0, }, // Solomon
    {  633, 1642,       0,  818147,       0,  0, 22,  0, }, // South_Georgia
    {  633, 1656,       0,  818169,       0,  0, 13,  0, }, // Suriname
    {  633, 1676,       0,  818182,       0,  0, 11,  0, }, // Syowa
    {  633, 1682,       0,  818193,       0,  0, 11,  0, }, // Tahiti
    {  633, 1689,  818204,  818216,  818237, 12, 21, 20, }, // Taipei
    {  633, 1696,       0,  818257,       0,  0, 16,  0, }, // Tajikistan
    {  633, 1724,       0,  818273,       0,  0, 13,  0, }, // Tokelau
    {  633, 1732,  818286,  818297,  818317, 11, 20, 19, }, // Tonga
    {  633, 1738,       0,  818336,       0,  0, 10,  0, }, // Truk
    {  633, 1750,  818346,  818364,  818391, 18, 27, 26, }, // Turkmenistan
    {  633, 1763,       0,  818417,       0,  0, 12,  0, }, // Tuvalu
    {  633, 1777,  818429,  818442,  818464, 13, 22, 21, }, // Uruguay
    {  633, 1792,  818485,  818501,  818526, 16, 25, 24, }, // Uzbekistan
    {  633, 1803,  818550,  818563,  818585, 13, 22, 21, }, // Vanuatu
    {  633, 1811,       0,  818606,       0,  0, 15,  0, }, // Venezuela
    {  633, 1821,  818621,  818638,  818664, 17, 26, 25, }, // Vladivostok
    {  633, 1833,  818689,  818704,  818728, 15, 24, 23, }, // Volgograd
    {  633, 1843,       0,  818751,       0,  0, 12,  0, }, // Vostok
    {  633, 1850,       0,  818763,       0,  0, 16,  0, }, // Wake
    {  633, 1855,       0,  818779,       0,  0, 22,  0, }, // Wallis
    {  633, 1862,  818801,  818814,  818836, 13, 22, 21, }, // Yakutsk
    {  633, 1870,  818857,  818876,  818904, 19, 28, 27, }, // Yekaterinburg
    {  633, 1892,       0,  818931,       0,  0, 11,  0, }, // Yukon
    {  635,    0,  818942,  818951,  818973,  9, 22, 17, }, // Acre Ukrainian/Cyrillic/Ukraine
    {  635,    5,       0,  818990,       0,  0, 22,  0, }, // Afghanistan
    {  635,   17,       0,  819012,       0,  0, 31,  0, }, // Africa_Central
    {  635,   32,       0,  819043,       0,  0, 27,  0, }, // Africa_Eastern
    {  635,   65,       0,  819070,       0,  0, 29,  0, }, // Africa_Southern
    {  635,   81,  819099,  819127,  819167, 28, 40, 35, }, // Africa_Western
    {  635,  107,  819202,  819220,  819250, 18, 30, 25, }, // Alaska
    {  635,  135,  819275,  819295,  819327, 20, 32, 27, }, // Amazon
    {  635,  142,  819354,  819396,  819450, 42, 54, 49, }, // America_Central
    {  635,  158,  819499,  819537,  819587, 38, 50, 45, }, // America_Eastern
    {  635,  174,  819632,  819671,  819722, 39, 51, 46, }, // America_Mountain
    {  635,  191,  819768,  819813,  819870, 45, 57, 52, }, // America_Pacific
    {  635,  207,  819922,  819933,  819957, 11, 24, 19, }, // Anadyr
    {  635,  214,  819976,  819991,  820018, 15, 27, 22, }, // Apia
    {  635,  232,  820040,  820058,  820088, 18, 30, 25, }, // Arabian
    {  635,  240,  820113,  820135,  820169, 22, 34, 29, }, // Argentina
    {  635,  250,  820198,  820227,  820268, 29, 41, 39, }, // Argentina_Western
    {  635,  268,  820307,  820327,  820359, 20, 32, 27, }, // Armenia
    {  635,  286,  820386,  820406,  820438, 20, 32, 27, }, // Atlantic
    {  635,  295,  820465,  820498,  820543, 33, 45, 40, }, // Australia_Central
    {  635,  313,  820583,  820625,  820679, 42, 54, 49, }, // Australia_CentralWestern
    {  635,  338,  820728,  820757,  820798, 29, 41, 36, }, // Australia_Eastern
    {  635,  356,  820834,  820864,  820906, 30, 42, 37, }, // Australia_Western
    {  635,  374,  820943,  820968,  821005, 25, 37, 32, }, // Azerbaijan
    {  635,  385,  821037,  821067,  821109, 30, 42, 37, }, // Azores
    {  635,  397,  821146,  821166,  821198, 20, 32, 27, }, // Bangladesh
    {  635,  415,       0,  821225,       0,  0, 17,  0, }, // Bhutan
    {  635,  422,       0,  821242,       0,  0, 21,  0, }, // Bolivia
    {  635,  437,  821263,  821284,  821317, 21, 33, 28, }, // Brasilia
    {  635,  454,       0,  821345,       0,  0, 17,  0, }, // Brunei
    {  635,  461,  821362,  821393,  821436, 31, 43, 38, }, // Cape_Verde
    {  635,  478,       0,  821474,       0,  0, 42,  0, }, // Chamorro
    {  635,  487,  821516,  821544,  821584, 28, 40, 35, }, // Chatham
    {  635,  495,  821619,  821638,  821669, 19, 31, 26, }, // Chile
    {  635,  501,  821695,  821714,  821745, 19, 31, 26, }, // China
    {  635,  507,       0,  821771,       0,  0, 26,  0, }, // Christmas
    {  635,  517,       0,  821797,       0,  0, 30,  0, }, // Cocos
    {  635,  523,  821827,  821849,  821883, 22, 34, 29, }, // Colombia
    {  635,  532,  821912,  821937,  821974, 25, 37, 32, }, // Cook
    {  635,  537,  822006,  822022,  822050, 16, 28, 23, }, // Cuba
    {  635,  548,       0,  822073,       0,  0, 25,  0, }, // Davis
    {  635,  564,       0,  822098,       0,  0, 34,  0, }, // DumontDUrville
    {  635,  601,       0,  822132,       0,  0, 26,  0, }, // East_Timor
    {  635,  612,  822158,  822183,  822220, 25, 37, 32, }, // Easter
    {  635,  619,       0,  822252,       0,  0, 19,  0, }, // Ecuador
    {  635,  627,  822271,  822302,  822345, 31, 43, 38, }, // Europe_Central
    {  635,  642,  822383,  822410,  822449, 27, 39, 34, }, // Europe_Eastern
    {  635,  657,       0,  822483,       0,  0, 35,  0, }, // Europe_Further_Eastern
    {  635,  680,  822518,  822546,  822586, 28, 40, 35, }, // Europe_Western
    {  635,  695,  822621,  822655,  822701, 34, 46, 41, }, // Falkland
    {  635,  704,  822742,  822758,  822786, 16, 28, 23, }, // Fiji
    {  635,  709,       0,  822809,       0,  0, 27,  0, }, // French_Guiana
    {  635,  723,       0,  822836,       0,  0, 59,  0, }, // French_Southern
    {  635,  746,       0,  822895,       0,  0, 19,  0, }, // Galapagos
    {  635,  756,       0,  822914,       0,  0, 26,  0, }, // Gambier
    {  635,  764,  822940,  822960,  822992, 20, 32, 27, }, // Georgia
    {  635,  772,       0,  823019,       0,  0, 29,  0, }, // Gilbert_Islands
    {  635,  788,       0,  823048,       0,  0, 12,  0, }, // GMT
    {  635,  830,  823060,  823089,  823130, 29, 41, 36, }, // Greenland_Eastern
    {  635,  848,  823166,  823196,  823238, 30, 42, 37, }, // Greenland_Western
    {  635,  866,       0,  823275,       0,  0, 24,  0, }, // Guam
    {  635,  871,       0,  823299,       0,  0, 24,  0, }, // Gulf
    {  635,  876,       0,  823323,       0,  0, 16,  0, }, // Guyana
    {  635,  883,  823339,  823368,  823409, 29, 41, 36, }, // Hawaii_Aleutian
    {  635,  899,  823445,  823464,  823495, 19, 31, 26, }, // Hong_Kong
    {  635,  909,  823521,  823537,  823565, 16, 28, 23, }, // Hovd
    {  635,  914,       0,  823588,       0,  0, 31,  0, }, // India
    {  635,  920,       0,  823619,       0,  0, 29,  0, }, // Indian_Ocean
    {  635,  933,       0,  823648,       0,  0, 20,  0, }, // Indochina
    {  635,  943,       0,  823668,       0,  0, 33,  0, }, // Indonesia_Central
    {  635,  961,       0,  823701,       0,  0, 29,  0, }, // Indonesia_Eastern
    {  635,  979,       0,  823730,       0,  0, 30,  0, }, // Indonesia_Western
    {  635,  997,  823760,  823778,  823808, 18, 30, 25, }, // Iran
    {  635, 1008,  823833,  823852,  823883, 19, 31, 26, }, // Irkutsk
    {  635, 1016,  823909,  823930,  823963, 21, 33, 28, }, // Israel
    {  635, 1023,  823991,  824009,  824039, 18, 30, 25, }, // Japan
    {  635, 1029,  824064,  824084,  824116, 20, 32, 27, }, // Kamchatka
    {  635, 1047,       0,  824143,       0,  0, 21,  0, }, // Kazakhstan
    {  635, 1058,       0,  824164,       0,  0, 29,  0, }, // Kazakhstan_Eastern
    {  635, 1077,       0,  824193,       0,  0, 30,  0, }, // Kazakhstan_Western
    {  635, 1106,  824223,  824242,  824273, 19, 31, 26, }, // Korea
    {  635, 1112,       0,  824299,       0,  0, 26,  0, }, // Kosrae
    {  635, 1119,  824325,  824347,  824381, 22, 34, 29, }, // Krasnoyarsk
    {  635, 1151,       0,  824410,       0,  0, 22,  0, }, // Kyrgystan
    {  635, 1161,       0,  824432,       0,  0, 10,  0, }, // Lanka
    {  635, 1175,       0,  824442,       0,  0, 24,  0, }, // Line_Islands
    {  635, 1188,  824466,  824494,  824534, 28, 40, 35, }, // Lord_Howe
    {  635, 1204,  824569,  824590,  824623, 21, 33, 28, }, // Magadan
    {  635, 1219,       0,  824651,       0,  0, 19,  0, }, // Malaysia
    {  635, 1228,       0,  824670,       0,  0, 21,  0, }, // Maldives
    {  635, 1237,       0,  824691,       0,  0, 31,  0, }, // Marquesas
    {  635, 1247,       0,  824722,       0,  0, 32,  0, }, // Marshall_Islands
    {  635, 1264,  824754,  824782,  824822, 28, 40, 35, }, // Mauritius
    {  635, 1274,       0,  824857,       0,  0, 26,  0, }, // Mawson
    {  635, 1281,  824883,  824916,  824961, 33, 45, 40, }, // Mexico_Pacific
    {  635, 1296,  825001,  825023,  825057, 22, 34, 29, }, // Mongolia
    {  635, 1305,  825086,  825106,  825138, 20, 32, 27, }, // Moscow
    {  635, 1312,       0,  825165,       0,  0, 17,  0, }, // Myanmar
    {  635, 1320,       0,  825182,       0,  0, 25,  0, }, // Nauru
    {  635, 1326,       0,  825207,       0,  0, 17,  0, }, // Nepal
    {  635, 1332,  825224,  825260,  825308, 36, 48, 43, }, // New_Caledonia
    {  635, 1346,  825351,  825376,  825413, 25, 37, 32, }, // New_Zealand
    {  635, 1358,  825445,  825477,  825521, 32, 44, 30, }, // Newfoundland
    {  635, 1371,       0,  825551,       0,  0, 24,  0, }, // Niue
    {  635, 1376,  825575,  825602,  825641, 27, 39, 34, }, // Norfolk
    {  635, 1384,  825675,  825717,  825771, 42, 54, 49, }, // Noronha
    {  635, 1406,  825820,  825843,  825878, 23, 35, 30, }, // Novosibirsk
    {  635, 1418,  825908,  825924,  825952, 16, 28, 23, }, // Omsk
    {  635, 1428,  825975,  825995,  826027, 20, 32, 27, }, // Pakistan
    {  635, 1437,       0,  826054,       0,  0, 25,  0, }, // Palau
    {  635, 1443,       0,  826079,       0,  0, 38,  0, }, // Papua_New_Guinea
    {  635, 1460,  826117,  826139,  826173, 22, 34, 29, }, // Paraguay
    {  635, 1469,  826202,  826217,  826244, 15, 27, 22, }, // Peru
    {  635, 1474,  826266,  826288,  826322, 22, 34, 29, }, // Philippines
    {  635, 1486,       0,  826351,       0,  0, 27,  0, }, // Phoenix_Islands
    {  635, 1502,  826378,  826417,  826468, 39, 51, 46, }, // Pierre_Miquelon
    {  635, 1518,       0,  826514,       0,  0, 28,  0, }, // Pitcairn
    {  635, 1527,       0,  826542,       0,  0, 26,  0, }, // Ponape
    {  635, 1534,       0,  826568,       0,  0, 19,  0, }, // Pyongyang
    {  635, 1544,  826587,  826601,  826628, 14, 27, 22, }, // Qyzylorda
    {  635, 1554,       0,  826650,       0,  0, 28,  0, }, // Reunion
    {  635, 1562,       0,  826678,       0,  0, 26,  0, }, // Rothera
    {  635, 1570,  826704,  826725,  826758, 21, 33, 28, }, // Sakhalin
    {  635, 1579,  826786,  826805,  826836, 19, 31, 26, }, // Samara
    {  635, 1596,  826862,  826887,  826924, 25, 37, 32, }, // Samoa
    {  635, 1602,       0,  826956,       0,  0, 33,  0, }, // Seychelles
    {  635, 1624,       0,  826989,       0,  0, 20,  0, }, // Singapore
    {  635, 1634,       0,  827009,       0,  0, 32,  0, }, // Solomon
    {  635, 1642,       0,  827041,       0,  0, 37,  0, }, // South_Georgia
    {  635, 1656,       0,  827078,       0,  0, 19,  0, }, // Suriname
    {  635, 1676,       0,  827097,       0,  0, 24,  0, }, // Syowa
    {  635, 1682,       0,  827121,       0,  0, 25,  0, }, // Tahiti
    {  635, 1689,  827146,  827163,  827192, 17, 29, 24, }, // Taipei
    {  635, 1696,       0,  827216,       0,  0, 23,  0, }, // Tajikistan
    {  635, 1724,       0,  827239,       0,  0, 28,  0, }, // Tokelau
    {  635, 1732,  827267,  827293,  827331, 26, 38, 33, }, // Tonga
    {  635, 1738,       0,  827364,       0,  0, 25,  0, }, // Truk
    {  635, 1750,  827389,  827413,  827449, 24, 36, 31, }, // Turkmenistan
    {  635, 1763,       0,  827480,       0,  0, 27,  0, }, // Tuvalu
    {  635, 1777,  827507,  827525,  827555, 18, 30, 25, }, // Uruguay
    {  635, 1792,  827580,  827602,  827636, 22, 34, 29, }, // Uzbekistan
    {  635, 1803,  827665,  827693,  827733, 28, 40, 35, }, // Vanuatu
    {  635, 1811,       0,  827768,       0,  0, 20,  0, }, // Venezuela
    {  635, 1821,  827788,  827812,  827848, 24, 36, 31, }, // Vladivostok
    {  635, 1833,  827879,  827902,  827937, 23, 35, 30, }, // Volgograd
    {  635, 1843,       0,  827967,       0,  0, 26,  0, }, // Vostok
    {  635, 1850,       0,  827993,       0,  0, 24,  0, }, // Wake
    {  635, 1855,       0,  828017,       0,  0, 36,  0, }, // Wallis
    {  635, 1862,  828053,  828071,  828101, 18, 30, 25, }, // Yakutsk
    {  635, 1870,  828126,  828151,  828188, 25, 37, 32, }, // Yekaterinburg
    {  635, 1892,       0,  828220,       0,  0, 29,  0, }, // Yukon
    {  636,    5,       0,  828249,       0,  0, 13,  0, }, // Afghanistan Upper Sorbian/Latin/Germany
    {  636,   17,       0,  828262,       0,  0, 20,  0, }, // Africa_Central
    {  636,   32,       0,  828282,       0,  0, 18,  0, }, // Africa_Eastern
    {  636,   65,       0,  828300,       0,  0, 16,  0, }, // Africa_Southern
    {  636,   81,  828316,  828333,  828361, 17, 28, 23, }, // Africa_Western
    {  636,  107,  828384,  828397,  828421, 13, 24, 19, }, // Alaska
    {  636,  135,  828440,  828454,  828479, 14, 25, 20, }, // Amazon
    {  636,  142,  828499,  828528,  828568, 29, 40, 35, }, // America_Central
    {  636,  158,  828603,  828631,  828670, 28, 39, 34, }, // America_Eastern
    {  636,  174,  828704,  828730,  828767, 26, 37, 32, }, // America_Mountain
    {  636,  191,  828799,  828828,  828868, 29, 40, 35, }, // America_Pacific
    {  636,  214,  828903,  828914,  828936, 11, 22, 17, }, // Apia
    {  636,  232,  828953,  828964,  828986, 11, 22, 17, }, // Arabian
    {  636,  240,  829003,  829018,  829044, 15, 26, 21, }, // Argentina
    {  636,  250,  829065,  829086,  829118, 21, 32, 27, }, // Argentina_Western
    {  636,  268,  829145,  829157,  829180, 12, 23, 18, }, // Armenia
    {  636,  286,  829198,  829212,  829237, 14, 25, 20, }, // Atlantic
    {  636,  295,  829257,  829278,  829310, 21, 32, 27, }, // Australia_Central
    {  636,  313,  829337,  829366,  829406, 29, 40, 34, }, // Australia_CentralWestern
    {  636,  338,  829440,  829461,  829493, 21, 32, 27, }, // Australia_Eastern
    {  636,  356,  829520,  829540,  829571, 20, 31, 26, }, // Australia_Western
    {  636,  374,  829597,  829615,  829644, 18, 29, 24, }, // Azerbaijan
    {  636,  385,  829668,  829679,  829701, 11, 22, 17, }, // Azores
    {  636,  397,  829718,  829734,  829761, 16, 27, 22, }, // Bangladesh
    {  636,  415,       0,  829783,       0,  0, 13,  0, }, // Bhutan
    {  636,  422,       0,  829796,       0,  0, 13,  0, }, // Bolivia
    {  636,  437,  829809,  829823,  829848, 14, 25, 20, }, // Brasilia
    {  636,  454,       0,  829868,       0,  0, 13,  0, }, // Brunei
    {  636,  461,  829881,  829895,  829920, 14, 25, 20, }, // Cape_Verde
    {  636,  478,       0,  829940,       0,  0, 15,  0, }, // Chamorro
    {  636,  487,  829955,  829969,  829994, 14, 25, 20, }, // Chatham
    {  636,  495,  830014,  830025,  830047, 11, 22, 17, }, // Chile
    {  636,  501,  830064,  830075,  830097, 11, 22, 17, }, // China
    {  636,  507,       0,  830114,       0,  0, 18,  0, }, // Christmas
    {  636,  517,       0,  830132,       0,  0, 20,  0, }, // Cocos
    {  636,  523,  830152,  830166,  830191, 14, 25, 20, }, // Colombia
    {  636,  532,  830211,  830230,  830260, 19, 30, 25, }, // Cook
    {  636,  537,  830285,  830296,  830318, 11, 22, 17, }, // Cuba
    {  636,  548,       0,  830335,       0,  0, 11,  0, }, // Davis
    {  636,  564,       0,  830346,       0,  0, 24,  0, }, // DumontDUrville
    {  636,  601,       0,  830370,       0,  0, 20,  0, }, // East_Timor
    {  636,  612,  830390,  830409,  830439, 19, 30, 25, }, // Easter
    {  636,  619,       0,  830464,       0,  0, 14,  0, }, // Ecuador
    {  636,  627,  830478,  830497,  830527, 19, 30, 25, }, // Europe_Central
    {  636,  642,  830552,  830571,  830601, 19, 30, 25, }, // Europe_Eastern
    {  636,  657,       0,  830626,       0,  0, 18,  0, }, // Europe_Further_Eastern
    {  636,  680,  830644,  830662,  830691, 18, 29, 24, }, // Europe_Western
    {  636,  695,  830715,  830730,  830756, 15, 26, 21, }, // Falkland
    {  636,  704,  830777,  830789,  830812, 12, 23, 18, }, // Fiji
    {  636,  709,       0,  830830,       0,  0, 21,  0, }, // French_Guiana
    {  636,  723,       0,  830851,       0,  0, 50,  0, }, // French_Southern
    {  636,  746,       0,  830901,       0,  0, 15,  0, }, // Galapagos
    {  636,  756,       0,  830916,       0,  0, 14,  0, }, // Gambier
    {  636,  764,  830930,  830943,  830967, 13, 24, 19, }, // Georgia
    {  636,  772,       0,  830986,       0,  0, 22,  0, }, // Gilbert_Islands
    {  636,  788,       0,  831008,       0,  0, 16,  0, }, // GMT
    {  636,  830,  831024,  831046,  831079, 22, 33, 28, }, // Greenland_Eastern
    {  636,  848,  831107,  831128,  831160, 21, 32, 27, }, // Greenland_Western
    {  636,  871,       0,  831187,       0,  0, 20,  0, }, // Gulf
    {  636,  876,       0,  830839,       0,  0, 12,  0, }, // Guyana
    {  636,  883,  831207,  831229,  831262, 22, 33, 28, }, // Hawaii_Aleutian
    {  636,  899,  831290,  831305,  831331, 15, 26, 21, }, // Hong_Kong
    {  636,  909,  831352,  831364,  831387, 12, 23, 18, }, // Hovd
    {  636,  914,       0,  831405,       0,  0, 11,  0, }, // India
    {  636,  920,       0,  831416,       0,  0, 19,  0, }, // Indian_Ocean
    {  636,  933,       0,  831435,       0,  0, 15,  0, }, // Indochina
    {  636,  943,       0,  831450,       0,  0, 20,  0, }, // Indonesia_Central
    {  636,  961,       0,  831470,       0,  0, 16,  0, }, // Indonesia_Eastern
    {  636,  979,       0,  831486,       0,  0, 19,  0, }, // Indonesia_Western
    {  636,  997,  831505,  831516,  831538, 11, 22, 17, }, // Iran
    {  636, 1008,  719286,  831555,  831578, 12, 23, 18, }, // Irkutsk
    {  636, 1016,  831596,  831609,  831633, 13, 24, 19, }, // Israel
    {  636, 1023,  831652,  831664,  831687, 12, 23, 18, }, // Japan
    {  636, 1047,       0,  831705,       0,  0, 13,  0, }, // Kazakhstan
    {  636, 1058,       0,  831718,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {  636, 1077,       0,  831739,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  636, 1106,  831759,  831771,  831794, 12, 23, 18, }, // Korea
    {  636, 1112,       0,  831812,       0,  0, 13,  0, }, // Kosrae
    {  636, 1119,  719686,  831825,  831852, 16, 27, 22, }, // Krasnoyarsk
    {  636, 1151,       0,  831874,       0,  0, 12,  0, }, // Kyrgystan
    {  636, 1175,       0,  831886,       0,  0, 20,  0, }, // Line_Islands
    {  636, 1188,  831906,  831924,  831953, 18, 29, 24, }, // Lord_Howe
    {  636, 1204,  719868,  831977,  832002, 14, 25, 20, }, // Magadan
    {  636, 1219,       0,  832022,       0,  0, 14,  0, }, // Malaysia
    {  636, 1228,       0,  832036,       0,  0, 14,  0, }, // Maldives
    {  636, 1237,       0,  832050,       0,  0, 15,  0, }, // Marquesas
    {  636, 1247,       0,  832065,       0,  0, 23,  0, }, // Marshall_Islands
    {  636, 1264,  832088,  832103,  832129, 15, 26, 21, }, // Mauritius
    {  636, 1274,       0,  720061,       0,  0, 13,  0, }, // Mawson
    {  636, 1281,  832150,  832171,  832203, 21, 32, 27, }, // Mexico_Pacific
    {  636, 1296,  832230,  832247,  832275, 17, 28, 23, }, // Mongolia
    {  636, 1305,  832298,  832311,  832335, 13, 24, 19, }, // Moscow
    {  636, 1312,       0,  832354,       0,  0, 14,  0, }, // Myanmar
    {  636, 1320,       0,  832368,       0,  0, 12,  0, }, // Nauru
    {  636, 1326,       0,  832380,       0,  0, 12,  0, }, // Nepal
    {  636, 1332,  832392,  832410,  832439, 18, 29, 24, }, // New_Caledonia
    {  636, 1346,  832463,  832481,  832510, 18, 29, 24, }, // New_Zealand
    {  636, 1358,  832534,  832553,  832583, 19, 30, 25, }, // Newfoundland
    {  636, 1371,       0,  832608,       0,  0, 11,  0, }, // Niue
    {  636, 1376,  832619,  832635,  832662, 16, 27, 22, }, // Norfolk
    {  636, 1384,  832684,  832713,  832753, 29, 40, 35, }, // Noronha
    {  636, 1406,  832788,  832804,  832831, 16, 27, 22, }, // Novosibirsk
    {  636, 1418,  720794,  832853,  832873,  9, 20, 15, }, // Omsk
    {  636, 1428,  832888,  832903,  832929, 15, 26, 21, }, // Pakistan
    {  636, 1437,       0,  832950,       0,  0, 12,  0, }, // Palau
    {  636, 1443,       0,  832962,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  636, 1460,  832984,  832999,  833025, 15, 26, 21, }, // Paraguay
    {  636, 1469,  833046,  833057,  833079, 11, 22, 17, }, // Peru
    {  636, 1474,  833096,  833110,  833135, 14, 25, 20, }, // Philippines
    {  636, 1486,       0,  833155,       0,  0, 22,  0, }, // Phoenix_Islands
    {  636, 1502,  833177,  833208,  833250, 31, 42, 37, }, // Pierre_Miquelon
    {  636, 1518,       0,  833287,       0,  0, 23,  0, }, // Pitcairn
    {  636, 1527,       0,  833310,       0,  0, 13,  0, }, // Ponape
    {  636, 1534,       0,  833323,       0,  0, 16,  0, }, // Pyongyang
    {  636, 1554,       0,  833339,       0,  0, 14,  0, }, // Reunion
    {  636, 1562,       0,  833353,       0,  0, 14,  0, }, // Rothera
    {  636, 1570,  833367,  833382,  833408, 15, 26, 21, }, // Sakhalin
    {  636, 1596,  833429,  833441,  833464, 12, 23, 18, }, // Samoa
    {  636, 1602,       0,  833482,       0,  0, 15,  0, }, // Seychelles
    {  636, 1624,       0,  833497,       0,  0, 15,  0, }, // Singapore
    {  636, 1634,       0,  833512,       0,  0, 22,  0, }, // Solomon
    {  636, 1642,       0,  833534,       0,  0, 18,  0, }, // South_Georgia
    {  636, 1656,       0,  833552,       0,  0, 14,  0, }, // Suriname
    {  636, 1676,       0,  833566,       0,  0, 12,  0, }, // Syowa
    {  636, 1682,       0,  833578,       0,  0, 13,  0, }, // Tahiti
    {  636, 1689,  833591,  833604,  833628, 13, 24, 19, }, // Taipei
    {  636, 1696,       0,  833647,       0,  0, 13,  0, }, // Tajikistan
    {  636, 1724,       0,  833660,       0,  0, 14,  0, }, // Tokelau
    {  636, 1732,  833674,  833686,  833709, 12, 23, 18, }, // Tonga
    {  636, 1738,       0,  833727,       0,  0, 12,  0, }, // Truk
    {  636, 1750,  833739,  833753,  833778, 14, 25, 20, }, // Turkmenistan
    {  636, 1763,       0,  833798,       0,  0, 13,  0, }, // Tuvalu
    {  636, 1777,  833811,  833825,  833850, 14, 25, 20, }, // Uruguay
    {  636, 1792,  833870,  833882,  833905, 12, 23, 18, }, // Uzbekistan
    {  636, 1803,  833923,  833937,  833962, 14, 25, 20, }, // Vanuatu
    {  636, 1811,       0,  833982,       0,  0, 15,  0, }, // Venezuela
    {  636, 1821,  833997,  834015,  834044, 18, 29, 24, }, // Vladivostok
    {  636, 1833,  834068,  834084,  834111, 16, 27, 22, }, // Volgograd
    {  636, 1843,       0,  834133,       0,  0, 13,  0, }, // Vostok
    {  636, 1850,       0,  834146,       0,  0, 13,  0, }, // Wake
    {  636, 1855,       0,  834159,       0,  0, 25,  0, }, // Wallis
    {  636, 1862,  722239,  834184,  834207, 12, 23, 18, }, // Yakutsk
    {  636, 1870,  834225,  834245,  834276, 20, 31, 26, }, // Yekaterinburg
    {  636, 1892,       0,  834302,       0,  0, 12,  0, }, // Yukon
    {  637,    5,       0,  834314,       0,  0, 16,  0, }, // Afghanistan Urdu/Arabic/Pakistan
    {  637,   17,       0,  834330,       0,  0, 16,  0, }, // Africa_Central
    {  637,   32,       0,  834346,       0,  0, 17,  0, }, // Africa_Eastern
    {  637,   65,       0,  834363,       0,  0, 25,  0, }, // Africa_Southern
    {  637,   81,  834388,  834405,  834430, 17, 25, 21, }, // Africa_Western
    {  637,  107,  834451,  834462,  834482, 11, 20, 19, }, // Alaska
    {  637,  135,  834501,  834512,  834533, 11, 21, 26, }, // Amazon
    {  637,  142,  834559,  834569,  834588, 10, 19, 18, }, // America_Central
    {  637,  158,  834606,  834617,  834637, 11, 20, 19, }, // America_Eastern
    {  637,  174,  834656,  834668,  834689, 12, 21, 20, }, // America_Mountain
    {  637,  191,  834709,  834719,  834738, 10, 19, 18, }, // America_Pacific
    {  637,  207,  834756,  834766,  834785, 10, 19, 14, }, // Anadyr
    {  637,  214,  834799,  834809,  834827, 10, 18, 18, }, // Apia
    {  637,  232,  834845,  834855,  834872, 10, 17, 16, }, // Arabian
    {  637,  240,  834888,  834903,  834925, 15, 22, 28, }, // Argentina
    {  637,  250,  834953,  834974,  835002, 21, 28, 34, }, // Argentina_Western
    {  637,  268,  835036,  835050,  835071, 14, 21, 27, }, // Armenia
    {  637,  286,  835098,  835110,  835131, 12, 21, 20, }, // Atlantic
    {  637,  295,  835151,  835170,  835198, 19, 28, 27, }, // Australia_Central
    {  637,  313,  835225,  835251,  835286, 26, 35, 34, }, // Australia_CentralWestern
    {  637,  338,  835320,  835340,  835369, 20, 29, 28, }, // Australia_Eastern
    {  637,  356,  835397,  835417,  835446, 20, 29, 28, }, // Australia_Western
    {  637,  374,  835474,  835491,  835515, 17, 24, 30, }, // Azerbaijan
    {  637,  385,  835545,  835558,  835578, 13, 20, 26, }, // Azores
    {  637,  397,  835604,  835620,  835643, 16, 23, 29, }, // Bangladesh
    {  637,  415,       0,  835672,       0,  0, 13,  0, }, // Bhutan
    {  637,  422,       0,  835685,       0,  0, 14,  0, }, // Bolivia
    {  637,  437,  835699,  835712,  835734, 13, 22, 17, }, // Brasilia
    {  637,  454,       0,  835751,       0,  0, 21,  0, }, // Brunei
    {  637,  461,  835772,  835785,  835806, 13, 21, 17, }, // Cape_Verde
    {  637,  478,       0,  835823,       0,  0, 19,  0, }, // Chamorro
    {  637,  487,  835842,  835852,  835871, 10, 19, 18, }, // Chatham
    {  637,  495,  835889,  835899,  835916, 10, 17, 23, }, // Chile
    {  637,  501,  835939,  835949,  835965, 10, 16, 17, }, // China
    {  637,  507,       0,  835982,       0,  0, 17,  0, }, // Christmas
    {  637,  517,       0,  835999,       0,  0, 18,  0, }, // Cocos
    {  637,  523,  836017,  836029,  836050, 12, 21, 27, }, // Colombia
    {  637,  532,  836077,  836092,  836115, 15, 23, 23, }, // Cook
    {  637,  537,  836138,  836148,  836167, 10, 19, 18, }, // Cuba
    {  637,  548,       0,  836185,       0,  0,  9,  0, }, // Davis
    {  637,  564,       0,  836194,       0,  0, 21,  0, }, // DumontDUrville
    {  637,  601,       0,  836215,       0,  0, 16,  0, }, // East_Timor
    {  637,  612,  836231,  836250,  836276, 19, 26, 32, }, // Easter
    {  637,  619,       0,  836308,       0,  0, 15,  0, }, // Ecuador
    {  637,  627,  836323,  836338,  836361, 15, 23, 29, }, // Europe_Central
    {  637,  642,  836390,  836407,  836431, 17, 24, 30, }, // Europe_Eastern
    {  637,  657,       0,  836461,       0,  0, 20,  0, }, // Europe_Further_Eastern
    {  637,  680,  836481,  836498,  836522, 17, 24, 30, }, // Europe_Western
    {  637,  695,  836552,  836575,  836605, 23, 30, 36, }, // Falkland
    {  637,  704,  836641,  836649,  836665,  8, 16, 12, }, // Fiji
    {  637,  709,       0,  836677,       0,  0, 18,  0, }, // French_Guiana
    {  637,  723,       0,  836695,       0,  0, 29,  0, }, // French_Southern
    {  637,  746,       0,  836724,       0,  0, 16,  0, }, // Galapagos
    {  637,  756,       0,  836740,       0,  0, 12,  0, }, // Gambier
    {  637,  764,  836752,  836765,  836785, 13, 20, 26, }, // Georgia
    {  637,  772,       0,  836811,       0,  0, 18,  0, }, // Gilbert_Islands
    {  637,  788,       0,  836829,       0,  0, 18,  0, }, // GMT
    {  637,  830,  836847,  836867,  836896, 20, 29, 35, }, // Greenland_Eastern
    {  637,  848,  836931,  836951,  836980, 20, 29, 35, }, // Greenland_Western
    {  637,  871,       0,  837015,       0,  0, 18,  0, }, // Gulf
    {  637,  876,       0,  836683,       0,  0, 12,  0, }, // Guyana
    {  637,  883,  837033,  837052,  837080, 19, 28, 27, }, // Hawaii_Aleutian
    {  637,  899,  413567,  413581,  837107, 14, 22, 18, }, // Hong_Kong
    {  637,  909,  837125,  837134,  837151,  9, 17, 13, }, // Hovd
    {  637,  914,       0,  837164,       0,  0, 22,  0, }, // India
    {  637,  920,       0,  837186,       0,  0, 12,  0, }, // Indian_Ocean
    {  637,  933,       0,  837198,       0,  0, 12,  0, }, // Indochina
    {  637,  943,       0,  837210,       0,  0, 19,  0, }, // Indonesia_Central
    {  637,  961,       0,  837229,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  637,  979,       0,  837249,       0,  0, 20,  0, }, // Indonesia_Western
    {  637,  997,  837269,  837281,  837300, 12, 19, 18, }, // Iran
    {  637, 1008,  837318,  837329,  837348, 11, 19, 15, }, // Irkutsk
    {  637, 1016,  837363,  837377,  837398, 14, 21, 20, }, // Israel
    {  637, 1023,  837418,  837428,  837446, 10, 18, 18, }, // Japan
    {  637, 1029,  837464,  837490,  837525, 26, 35, 30, }, // Kamchatka
    {  637, 1047,       0,  837555,       0,  0, 15,  0, }, // Kazakhstan
    {  637, 1058,       0,  837570,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {  637, 1077,       0,  837591,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  637, 1106,  837612,  837622,  837640, 10, 18, 18, }, // Korea
    {  637, 1112,       0,  837658,       0,  0, 10,  0, }, // Kosrae
    {  637, 1119,  837668,  837684,  837707, 16, 23, 20, }, // Krasnoyarsk
    {  637, 1151,       0,  837727,       0,  0, 14,  0, }, // Kyrgystan
    {  637, 1175,       0,  837741,       0,  0, 17,  0, }, // Line_Islands
    {  637, 1188,  837758,  837772,  837795, 14, 23, 22, }, // Lord_Howe
    {  637, 1204,  837817,  837828,  837848, 11, 20, 15, }, // Magadan
    {  637, 1219,       0,  837863,       0,  0, 11,  0, }, // Malaysia
    {  637, 1228,       0,  837874,       0,  0, 13,  0, }, // Maldives
    {  637, 1237,       0,  837887,       0,  0, 12,  0, }, // Marquesas
    {  637, 1247,       0,  837899,       0,  0, 18,  0, }, // Marshall_Islands
    {  637, 1264,  837917,  837928,  837947, 11, 19, 15, }, // Mauritius
    {  637, 1274,       0,  837962,       0,  0, 10,  0, }, // Mawson
    {  637, 1281,  837972,  837990,  838017, 18, 27, 26, }, // Mexico_Pacific
    {  637, 1296,  838043,  838059,  838083, 16, 24, 20, }, // Mongolia
    {  637, 1305,  838103,  838113,  838132, 10, 19, 14, }, // Moscow
    {  637, 1312,       0,  838146,       0,  0, 12,  0, }, // Myanmar
    {  637, 1320,       0,  838158,       0,  0, 10,  0, }, // Nauru
    {  637, 1326,       0,  838168,       0,  0, 12,  0, }, // Nepal
    {  637, 1332,  838180,  838198,  838224, 18, 26, 22, }, // New_Caledonia
    {  637, 1346,  838246,  838263,  838287, 17, 24, 23, }, // New_Zealand
    {  637, 1358,  414925,  838310,  414971, 19, 28, 27, }, // Newfoundland
    {  637, 1371,       0,  838338,       0,  0,  9,  0, }, // Niue
    {  637, 1376,  838347,  838367,  838394, 20, 27, 25, }, // Norfolk
    {  637, 1384,  838419,  838443,  838474, 24, 31, 27, }, // Noronha
    {  637, 1406,  838501,  838516,  838539, 15, 23, 19, }, // Novosibirsk
    {  637, 1418,  838558,  838568,  838586, 10, 18, 14, }, // Omsk
    {  637, 1428,  838600,  838614,  838635, 14, 21, 27, }, // Pakistan
    {  637, 1437,       0,  838662,       0,  0,  9,  0, }, // Palau
    {  637, 1443,       0,  838671,       0,  0, 18,  0, }, // Papua_New_Guinea
    {  637, 1460,  838689,  838704,  838726, 15, 22, 28, }, // Paraguay
    {  637, 1469,  838754,  838765,  838783, 11, 18, 24, }, // Peru
    {  637, 1474,  838807,  838818,  838837, 11, 19, 15, }, // Philippines
    {  637, 1486,       0,  838852,       0,  0, 18,  0, }, // Phoenix_Islands
    {  637, 1502,  838870,  838896,  838931, 26, 35, 34, }, // Pierre_Miquelon
    {  637, 1518,       0,  838965,       0,  0, 12,  0, }, // Pitcairn
    {  637, 1527,       0,  838977,       0,  0, 11,  0, }, // Ponape
    {  637, 1534,       0,  838988,       0,  0, 14,  0, }, // Pyongyang
    {  637, 1554,       0,  839002,       0,  0, 13,  0, }, // Reunion
    {  637, 1562,       0,  839015,       0,  0, 14,  0, }, // Rothera
    {  637, 1570,  839029,  839040,  839059, 11, 19, 15, }, // Sakhalin
    {  637, 1579,  839074,  839084,  839103, 10, 19, 14, }, // Samara
    {  637, 1596,  839117,  839127,  839145, 10, 18, 18, }, // Samoa
    {  637, 1602,       0,  839163,       0,  0, 11,  0, }, // Seychelles
    {  637, 1624,       0,  839174,       0,  0, 20,  0, }, // Singapore
    {  637, 1634,       0,  839194,       0,  0, 18,  0, }, // Solomon
    {  637, 1642,       0,  839212,       0,  0, 17,  0, }, // South_Georgia
    {  637, 1656,       0,  839229,       0,  0, 14,  0, }, // Suriname
    {  637, 1676,       0,  839243,       0,  0, 10,  0, }, // Syowa
    {  637, 1682,       0,  839253,       0,  0, 11,  0, }, // Tahiti
    {  637, 1689,  839264,  839278,  839301, 14, 23, 21, }, // Taipei
    {  637, 1696,       0,  839322,       0,  0, 15,  0, }, // Tajikistan
    {  637, 1724,       0,  839337,       0,  0, 12,  0, }, // Tokelau
    {  637, 1732,  839349,  839359,  839377, 10, 18, 14, }, // Tonga
    {  637, 1738,       0,  839391,       0,  0,  8,  0, }, // Truk
    {  637, 1750,  839399,  839416,  839440, 17, 24, 30, }, // Turkmenistan
    {  637, 1763,       0,  839470,       0,  0, 10,  0, }, // Tuvalu
    {  637, 1777,  839480,  839495,  839517, 15, 22, 28, }, // Uruguay
    {  637, 1792,  839545,  839560,  839582, 15, 22, 28, }, // Uzbekistan
    {  637, 1803,  839610,  839622,  839642, 12, 20, 16, }, // Vanuatu
    {  637, 1811,       0,  839658,       0,  0, 16,  0, }, // Venezuela
    {  637, 1821,  839674,  839690,  839714, 16, 24, 20, }, // Vladivostok
    {  637, 1833,  839734,  839748,  839771, 14, 23, 18, }, // Volgograd
    {  637, 1843,       0,  839789,       0,  0, 13,  0, }, // Vostok
    {  637, 1850,       0,  839802,       0,  0, 15,  0, }, // Wake
    {  637, 1855,       0,  839817,       0,  0, 20,  0, }, // Wallis
    {  637, 1862,  839837,  839848,  839868, 11, 20, 15, }, // Yakutsk
    {  637, 1870,  839883,  839899,  839924, 16, 25, 20, }, // Yekaterinburg
    {  637, 1892,       0,  416728,       0,  0, 10,  0, }, // Yukon
    {  638,    5,       0,  839944,       0,  0, 14,  0, }, // Afghanistan Urdu/Arabic/India
    {  638,  135,  839958,  839970,  839990, 12, 20, 16, }, // Amazon
    {  638,  232,  840006,  840014,  834872,  8, 16, 16, }, // Arabian
    {  638,  250,  840030,  840049,  840076, 19, 27, 23, }, // Argentina_Western
    {  638,  268,  840099,  840111,  840131, 12, 20, 16, }, // Armenia
    {  638,  374,  840147,  840162,  840185, 15, 23, 19, }, // Azerbaijan
    {  638,  397,  840204,  840218,  840240, 14, 22, 18, }, // Bangladesh
    {  638,  415,       0,  840258,       0,  0, 11,  0, }, // Bhutan
    {  638,  422,       0,  840269,       0,  0, 12,  0, }, // Bolivia
    {  638,  437,  835699,  840281,  835734, 13, 21, 17, }, // Brasilia
    {  638,  495,  840302,  840310,  840326,  8, 16, 12, }, // Chile
    {  638,  523,  836017,  840338,  840358, 12, 20, 16, }, // Colombia
    {  638,  612,  840374,  840391,  840416, 17, 25, 21, }, // Easter
    {  638,  619,       0,  840437,       0,  0, 13,  0, }, // Ecuador
    {  638,  627,  840450,  836338,  836361, 16, 23, 29, }, // Europe_Central
    {  638,  695,  840466,  840487,  840516, 21, 29, 25, }, // Falkland
    {  638,  709,       0,  840541,       0,  0, 16,  0, }, // French_Guiana
    {  638,  746,       0,  840557,       0,  0, 14,  0, }, // Galapagos
    {  638,  764,  839218,  840571,  840590, 11, 19, 15, }, // Georgia
    {  638,  788,       0,  840605,       0,  0, 16,  0, }, // GMT
    {  638,  871,       0,  840621,       0,  0, 17,  0, }, // Gulf
    {  638,  876,       0,  840547,       0,  0, 10,  0, }, // Guyana
    {  638,  914,       0,  840638,       0,  0, 18,  0, }, // India
    {  638,  997,  840656,  840666,  837300, 10, 18, 18, }, // Iran
    {  638, 1016,  840684,  840696,  837398, 12, 20, 20, }, // Israel
    {  638, 1058,       0,  840716,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  638, 1077,       0,  840735,       0,  0, 19,  0, }, // Kazakhstan_Western
    {  638, 1151,       0,  840754,       0,  0, 12,  0, }, // Kyrgystan
    {  638, 1228,       0,  840766,       0,  0, 11,  0, }, // Maldives
    {  638, 1326,       0,  840777,       0,  0, 10,  0, }, // Nepal
    {  638, 1346,  840787,  840802,  838287, 15, 23, 23, }, // New_Zealand
    {  638, 1384,  840825,  840847,  838474, 22, 30, 27, }, // Noronha
    {  638, 1428,  840877,  840889,  840909, 12, 20, 16, }, // Pakistan
    {  638, 1460,  840925,  840938,  840959, 13, 21, 17, }, // Paraguay
    {  638, 1469,  840976,  840985,  841002,  9, 17, 13, }, // Peru
    {  638, 1562,       0,  841015,       0,  0, 12,  0, }, // Rothera
    {  638, 1656,       0,  841027,       0,  0, 12,  0, }, // Suriname
    {  638, 1696,       0,  416072,       0,  0, 13,  0, }, // Tajikistan
    {  638, 1750,  416157,  416172,  841039, 15, 23, 19, }, // Turkmenistan
    {  638, 1777,  841058,  841071,  841092, 13, 21, 17, }, // Uruguay
    {  638, 1792,  841109,  841122,  841143, 13, 21, 17, }, // Uzbekistan
    {  638, 1811,       0,  841160,       0,  0, 14,  0, }, // Venezuela
    {  638, 1843,       0,  841174,       0,  0, 11,  0, }, // Vostok
    {  639,    0,  841185,  841196,  841217, 11, 21, 18, }, // Acre Uyghur/Arabic/China
    {  639,    5,       0,  841235,       0,  0, 17,  0, }, // Afghanistan
    {  639,   17,       0,  841252,       0,  0, 21,  0, }, // Africa_Central
    {  639,   32,       0,  841273,       0,  0, 20,  0, }, // Africa_Eastern
    {  639,   65,       0,  841293,       0,  0, 31,  0, }, // Africa_Southern
    {  639,   81,  841324,  841344,  841374, 20, 30, 27, }, // Africa_Western
    {  639,  107,  841401,  841415,  841439, 14, 24, 21, }, // Alaska
    {  639,  128,  841460,  841473,  841496, 13, 23, 20, }, // Almaty
    {  639,  135,  841516,  841529,  841552, 13, 23, 20, }, // Amazon
    {  639,  142,  841572,  841591,  841620, 19, 29, 26, }, // America_Central
    {  639,  158,  841646,  841664,  841692, 18, 28, 25, }, // America_Eastern
    {  639,  174,  841717,  841726,  841745,  9, 19, 16, }, // America_Mountain
    {  639,  191,  841761,  841778,  841805, 17, 27, 24, }, // America_Pacific
    {  639,  207,  841829,  841842,  841865, 13, 23, 20, }, // Anadyr
    {  639,  219,  841885,  841897,  841919, 12, 22, 19, }, // Aqtau
    {  639,  225,  841938,  841951,  841974, 13, 23, 20, }, // Aqtobe
    {  639,  232,  841994,  842005,  842026, 11, 21, 18, }, // Arabian
    {  639,  240,  842044,  842060,  842086, 16, 26, 23, }, // Argentina
    {  639,  250,  842109,  842132,  842165, 23, 33, 30, }, // Argentina_Western
    {  639,  268,  842195,  842210,  842235, 15, 25, 22, }, // Armenia
    {  639,  286,  842257,  842279,  842311, 22, 32, 29, }, // Atlantic
    {  639,  295,  842340,  842371,  842412, 31, 41, 38, }, // Australia_Central
    {  639,  313,  842450,  842488,  842537, 38, 49, 45, }, // Australia_CentralWestern
    {  639,  338,  842582,  842612,  842652, 30, 40, 37, }, // Australia_Eastern
    {  639,  356,  842689,  842719,  842759, 30, 40, 37, }, // Australia_Western
    {  639,  374,  842796,  842813,  842840, 17, 27, 24, }, // Azerbaijan
    {  639,  385,  842864,  842875,  842896, 11, 21, 18, }, // Azores
    {  639,  397,  842914,  842928,  842952, 14, 24, 21, }, // Bangladesh
    {  639,  415,       0,  842973,       0,  0, 11,  0, }, // Bhutan
    {  639,  422,       0,  842984,       0,  0, 14,  0, }, // Bolivia
    {  639,  437,  842998,  843014,  843040, 16, 26, 23, }, // Brasilia
    {  639,  454,       0,  843063,       0,  0, 24,  0, }, // Brunei
    {  639,  461,  843087,  843105,  843133, 18, 28, 25, }, // Cape_Verde
    {  639,  472,       0,  843158,       0,  0, 11,  0, }, // Casey
    {  639,  478,       0,  843169,       0,  0, 23,  0, }, // Chamorro
    {  639,  487,  843192,  843203,  843224, 11, 21, 18, }, // Chatham
    {  639,  495,  843242,  843252,  843272, 10, 20, 17, }, // Chile
    {  639,  501,  843289,  843300,  843321, 11, 21, 18, }, // China
    {  639,  507,       0,  843339,       0,  0, 22,  0, }, // Christmas
    {  639,  517,       0,  843361,       0,  0, 18,  0, }, // Cocos
    {  639,  523,  843379,  843394,  843419, 15, 25, 22, }, // Colombia
    {  639,  532,  843441,  843460,  843489, 19, 29, 32, }, // Cook
    {  639,  537,  843521,  843531,  843551, 10, 20, 17, }, // Cuba
    {  639,  548,       0,  843568,       0,  0, 11,  0, }, // Davis
    {  639,  564,       0,  843579,       0,  0, 19,  0, }, // DumontDUrville
    {  639,  601,       0,  843598,       0,  0, 18,  0, }, // East_Timor
    {  639,  612,  843616,  843635,  843666, 19, 31, 26, }, // Easter
    {  639,  619,       0,  843692,       0,  0, 14,  0, }, // Ecuador
    {  639,  627,  843706,  843727,  843758, 21, 31, 28, }, // Europe_Central
    {  639,  642,  843786,  843806,  843836, 20, 30, 27, }, // Europe_Eastern
    {  639,  680,  843863,  843883,  843913, 20, 30, 27, }, // Europe_Western
    {  639,  695,  843940,  843964,  843998, 24, 34, 31, }, // Falkland
    {  639,  704,  844029,  844039,  844059, 10, 20, 17, }, // Fiji
    {  639,  709,       0,  844076,       0,  0, 35,  0, }, // French_Guiana
    {  639,  723,       0,  844111,       0,  0, 49,  0, }, // French_Southern
    {  639,  746,       0,  844160,       0,  0, 15,  0, }, // Galapagos
    {  639,  756,       0,  844175,       0,  0, 14,  0, }, // Gambier
    {  639,  764,  844189,  844203,  844227, 14, 24, 21, }, // Georgia
    {  639,  772,       0,  844248,       0,  0, 23,  0, }, // Gilbert_Islands
    {  639,  788,       0,  844271,       0,  0, 14,  0, }, // GMT
    {  639,  830,  844285,  844307,  844339, 22, 32, 29, }, // Greenland_Eastern
    {  639,  848,  844368,  844390,  844422, 22, 32, 29, }, // Greenland_Western
    {  639,  866,       0,  844451,       0,  0, 21,  0, }, // Guam
    {  639,  871,       0,  844472,       0,  0, 20,  0, }, // Gulf
    {  639,  876,       0,  844097,       0,  0, 14,  0, }, // Guyana
    {  639,  883,  844492,  844511,  844540, 19, 29, 26, }, // Hawaii_Aleutian
    {  639,  899,  844566,  844579,  844602, 13, 23, 20, }, // Hong_Kong
    {  639,  909,  844622,  844632,  844652, 10, 20, 17, }, // Hovd
    {  639,  914,       0,  844669,       0,  0, 25,  0, }, // India
    {  639,  920,       0,  844694,       0,  0, 18,  0, }, // Indian_Ocean
    {  639,  933,       0,  844712,       0,  0, 16,  0, }, // Indochina
    {  639,  943,       0,  844728,       0,  0, 25,  0, }, // Indonesia_Central
    {  639,  961,       0,  844753,       0,  0, 24,  0, }, // Indonesia_Eastern
    {  639,  979,       0,  844777,       0,  0, 24,  0, }, // Indonesia_Western
    {  639,  997,  844801,  844812,  844833, 11, 21, 18, }, // Iran
    {  639, 1008,  844851,  844865,  844889, 14, 24, 21, }, // Irkutsk
    {  639, 1016,  844910,  844927,  844954, 17, 27, 24, }, // Israel
    {  639, 1023,  844978,  844992,  845016, 14, 24, 21, }, // Japan
    {  639, 1029,  845037,  845067,  845107, 30, 40, 37, }, // Kamchatka
    {  639, 1058,       0,  845144,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  639, 1077,       0,  845167,       0,  0, 23,  0, }, // Kazakhstan_Western
    {  639, 1106,  845190,  845202,  845224, 12, 22, 19, }, // Korea
    {  639, 1112,       0,  845243,       0,  0, 13,  0, }, // Kosrae
    {  639, 1119,  845256,  845274,  845302, 18, 28, 25, }, // Krasnoyarsk
    {  639, 1151,       0,  845327,       0,  0, 17,  0, }, // Kyrgystan
    {  639, 1161,       0,  845344,       0,  0, 16,  0, }, // Lanka
    {  639, 1175,       0,  845360,       0,  0, 20,  0, }, // Line_Islands
    {  639, 1188,  845380,  845394,  845418, 14, 24, 21, }, // Lord_Howe
    {  639, 1198,  845439,  845451,  845473, 12, 22, 19, }, // Macau
    {  639, 1204,  845492,  845505,  845528, 13, 23, 20, }, // Magadan
    {  639, 1219,       0,  845548,       0,  0, 15,  0, }, // Malaysia
    {  639, 1228,       0,  845563,       0,  0, 13,  0, }, // Maldives
    {  639, 1237,       0,  845576,       0,  0, 12,  0, }, // Marquesas
    {  639, 1247,       0,  845588,       0,  0, 22,  0, }, // Marshall_Islands
    {  639, 1264,  845610,  845626,  845652, 16, 26, 23, }, // Mauritius
    {  639, 1274,       0,  845675,       0,  0, 12,  0, }, // Mawson
    {  639, 1281,  845687,  845712,  845747, 25, 35, 32, }, // Mexico_Pacific
    {  639, 1296,  845779,  845795,  845821, 16, 26, 23, }, // Mongolia
    {  639, 1305,  845844,  845856,  845878, 12, 22, 19, }, // Moscow
    {  639, 1312,       0,  845897,       0,  0, 11,  0, }, // Myanmar
    {  639, 1320,       0,  845908,       0,  0, 11,  0, }, // Nauru
    {  639, 1326,       0,  845919,       0,  0, 11,  0, }, // Nepal
    {  639, 1332,  845930,  845951,  845982, 21, 31, 28, }, // New_Caledonia
    {  639, 1346,  846010,  846030,  846060, 20, 30, 27, }, // New_Zealand
    {  639, 1358,  846087,  846105,  846133, 18, 28, 25, }, // Newfoundland
    {  639, 1371,       0,  846158,       0,  0, 12,  0, }, // Niue
    {  639, 1376,  846170,  846193,  846226, 23, 33, 30, }, // Norfolk
    {  639, 1384,  846256,  846278,  846310, 22, 32, 29, }, // Noronha
    {  639, 1392,       0,  846339,       0,  0, 32,  0, }, // North_Mariana
    {  639, 1406,  846371,  846388,  846415, 17, 27, 24, }, // Novosibirsk
    {  639, 1418,  846439,  846450,  846471, 11, 21, 18, }, // Omsk
    {  639, 1428,  846489,  846503,  846527, 14, 24, 21, }, // Pakistan
    {  639, 1437,       0,  846548,       0,  0, 11,  0, }, // Palau
    {  639, 1443,       0,  846559,       0,  0, 28,  0, }, // Papua_New_Guinea
    {  639, 1460,  846587,  846601,  846625, 14, 24, 21, }, // Paraguay
    {  639, 1469,  846646,  846656,  846676, 10, 20, 17, }, // Peru
    {  639, 1474,  846693,  846707,  846731, 14, 24, 21, }, // Philippines
    {  639, 1486,       0,  846752,       0,  0, 22,  0, }, // Phoenix_Islands
    {  639, 1502,  846774,  846803,  846842, 29, 39, 36, }, // Pierre_Miquelon
    {  639, 1518,       0,  846878,       0,  0, 13,  0, }, // Pitcairn
    {  639, 1527,       0,  846891,       0,  0, 12,  0, }, // Ponape
    {  639, 1544,  846903,  846919,  846945, 16, 26, 23, }, // Qyzylorda
    {  639, 1554,       0,  846968,       0,  0, 15,  0, }, // Reunion
    {  639, 1562,       0,  846983,       0,  0, 12,  0, }, // Rothera
    {  639, 1570,  846995,  847008,  847031, 13, 23, 20, }, // Sakhalin
    {  639, 1579,  847051,  847063,  847085, 12, 22, 19, }, // Samara
    {  639, 1596,  847104,  847116,  847138, 12, 22, 22, }, // Samoa
    {  639, 1602,       0,  847160,       0,  0, 12,  0, }, // Seychelles
    {  639, 1624,       0,  847172,       0,  0, 14,  0, }, // Singapore
    {  639, 1634,       0,  847186,       0,  0, 23,  0, }, // Solomon
    {  639, 1642,       0,  847209,       0,  0, 21,  0, }, // South_Georgia
    {  639, 1656,       0,  847230,       0,  0, 13,  0, }, // Suriname
    {  639, 1676,       0,  847243,       0,  0, 10,  0, }, // Syowa
    {  639, 1682,       0,  847253,       0,  0, 11,  0, }, // Tahiti
    {  639, 1689,  847264,  847276,  847298, 12, 22, 19, }, // Taipei
    {  639, 1696,       0,  847317,       0,  0, 16,  0, }, // Tajikistan
    {  639, 1724,       0,  847333,       0,  0, 13,  0, }, // Tokelau
    {  639, 1732,  847346,  847357,  847378, 11, 21, 18, }, // Tonga
    {  639, 1738,       0,  847396,       0,  0,  9,  0, }, // Truk
    {  639, 1750,  847405,  847423,  847451, 18, 28, 25, }, // Turkmenistan
    {  639, 1763,       0,  847476,       0,  0, 12,  0, }, // Tuvalu
    {  639, 1777,  847488,  847502,  847526, 14, 24, 21, }, // Uruguay
    {  639, 1792,  847547,  847564,  847591, 17, 27, 24, }, // Uzbekistan
    {  639, 1803,  847615,  847629,  847653, 14, 24, 21, }, // Vanuatu
    {  639, 1811,       0,  847674,       0,  0, 16,  0, }, // Venezuela
    {  639, 1821,  847690,  847708,  847736, 18, 28, 25, }, // Vladivostok
    {  639, 1833,  847761,  847776,  847801, 15, 25, 22, }, // Volgograd
    {  639, 1843,       0,  847696,       0,  0, 12,  0, }, // Vostok
    {  639, 1850,       0,  847823,       0,  0, 17,  0, }, // Wake
    {  639, 1855,       0,  847840,       0,  0, 22,  0, }, // Wallis
    {  639, 1862,  847862,  847875,  847898, 13, 23, 20, }, // Yakutsk
    {  639, 1870,  847918,  847937,  847966, 19, 29, 26, }, // Yekaterinburg
    {  640,    5,       0,  847992,       0,  0, 17,  0, }, // Afghanistan Uzbek/Latin/Uzbekistan
    {  640,   17,       0,  848009,       0,  0, 21,  0, }, // Africa_Central
    {  640,   32,       0,  848030,       0,  0, 20,  0, }, // Africa_Eastern
    {  640,   65,       0,  848050,       0,  0, 29,  0, }, // Africa_Southern
    {  640,   81,  848079,  848099,  848128, 20, 29, 26, }, // Africa_Western
    {  640,  107,  848154,  848167,  848189, 13, 22, 19, }, // Alaska
    {  640,  135,  848208,  848222,  848245, 14, 23, 20, }, // Amazon
    {  640,  142,  848265,  848287,  848318, 22, 31, 28, }, // America_Central
    {  640,  158,  848346,  848367,  848397, 21, 30, 27, }, // America_Eastern
    {  640,  174,  848424,  848441,  848467, 17, 26, 23, }, // America_Mountain
    {  640,  191,  848490,  848508,  848535, 18, 27, 24, }, // America_Pacific
    {  640,  214,  848559,  848569,  848588, 10, 19, 16, }, // Apia
    {  640,  232,  848604,  848628,  848661, 24, 33, 30, }, // Arabian
    {  640,  240,  848691,  848706,  848730, 15, 24, 21, }, // Argentina
    {  640,  250,  848751,  848774,  848806, 23, 32, 29, }, // Argentina_Western
    {  640,  268,  848835,  848851,  848876, 16, 25, 22, }, // Armenia
    {  640,  286,  848898,  848913,  848937, 15, 24, 21, }, // Atlantic
    {  640,  295,  848958,  848983,  849017, 25, 34, 31, }, // Australia_Central
    {  640,  313,  849048,  849081,  849123, 33, 42, 39, }, // Australia_CentralWestern
    {  640,  338,  849162,  849186,  849219, 24, 33, 30, }, // Australia_Eastern
    {  640,  356,  849249,  849273,  849306, 24, 33, 30, }, // Australia_Western
    {  640,  374,  849336,  849352,  849377, 16, 25, 22, }, // Azerbaijan
    {  640,  385,  849399,  849418,  849446, 19, 28, 25, }, // Azores
    {  640,  397,  849471,  849487,  849512, 16, 25, 22, }, // Bangladesh
    {  640,  415,       0,  849534,       0,  0, 11,  0, }, // Bhutan
    {  640,  422,       0,  849545,       0,  0, 14,  0, }, // Bolivia
    {  640,  437,  849559,  849574,  849598, 15, 24, 21, }, // Brasilia
    {  640,  454,       0,  849619,       0,  0, 23,  0, }, // Brunei
    {  640,  461,  849642,  849658,  849683, 16, 25, 22, }, // Cape_Verde
    {  640,  478,       0,  849705,       0,  0, 23,  0, }, // Chamorro
    {  640,  487,  849728,  849740,  849761, 12, 21, 18, }, // Chatham
    {  640,  495,  849779,  849790,  849810, 11, 20, 17, }, // Chile
    {  640,  501,  849827,  849838,  849858, 11, 20, 17, }, // China
    {  640,  507,       0,  849875,       0,  0, 21,  0, }, // Christmas
    {  640,  517,       0,  849896,       0,  0, 20,  0, }, // Cocos
    {  640,  523,  849916,  849931,  849955, 15, 24, 21, }, // Colombia
    {  640,  532,  849976,  849994,  850021, 18, 27, 30, }, // Cook
    {  640,  537,  850051,  850061,  850080, 10, 19, 16, }, // Cuba
    {  640,  548,       0,  850096,       0,  0, 12,  0, }, // Davis
    {  640,  564,       0,  850108,       0,  0, 21,  0, }, // DumontDUrville
    {  640,  601,       0,  850129,       0,  0, 19,  0, }, // East_Timor
    {  640,  612,  850148,  850165,  850191, 17, 26, 23, }, // Easter
    {  640,  619,       0,  850214,       0,  0, 13,  0, }, // Ecuador
    {  640,  627,  850227,  850249,  850280, 22, 31, 28, }, // Europe_Central
    {  640,  642,  850308,  850329,  850359, 21, 30, 27, }, // Europe_Eastern
    {  640,  657,       0,  850386,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {  640,  680,  850412,  850433,  850463, 21, 30, 27, }, // Europe_Western
    {  640,  695,  850490,  850513,  850545, 23, 32, 29, }, // Falkland
    {  640,  704,  850574,  850584,  850603, 10, 19, 16, }, // Fiji
    {  640,  709,       0,  850619,       0,  0, 22,  0, }, // French_Guiana
    {  640,  723,       0,  850641,       0,  0, 45,  0, }, // French_Southern
    {  640,  746,       0,  850686,       0,  0, 15,  0, }, // Galapagos
    {  640,  756,       0,  850701,       0,  0, 12,  0, }, // Gambier
    {  640,  764,  850713,  850726,  850748, 13, 22, 19, }, // Georgia
    {  640,  772,       0,  850767,       0,  0, 22,  0, }, // Gilbert_Islands
    {  640,  788,       0,  850789,       0,  0, 23,  0, }, // GMT
    {  640,  830,  850812,  850837,  850871, 25, 34, 31, }, // Greenland_Eastern
    {  640,  848,  850902,  850927,  850961, 25, 34, 31, }, // Greenland_Western
    {  640,  871,       0,  850992,       0,  0, 28,  0, }, // Gulf
    {  640,  876,       0,  851020,       0,  0, 12,  0, }, // Guyana
    {  640,  883,  851032,  851050,  851077, 18, 27, 24, }, // Hawaii_Aleutian
    {  640,  899,  851101,  851114,  851136, 13, 22, 19, }, // Hong_Kong
    {  640,  909,  851155,  851165,  851184, 10, 19, 16, }, // Hovd
    {  640,  914,       0,  851200,       0,  0, 24,  0, }, // India
    {  640,  920,       0,  851224,       0,  0, 17,  0, }, // Indian_Ocean
    {  640,  933,       0,  851241,       0,  0, 16,  0, }, // Indochina
    {  640,  943,       0,  851257,       0,  0, 25,  0, }, // Indonesia_Central
    {  640,  961,       0,  851282,       0,  0, 24,  0, }, // Indonesia_Eastern
    {  640,  979,       0,  851306,       0,  0, 24,  0, }, // Indonesia_Western
    {  640,  997,  851330,  851340,  851359, 10, 19, 16, }, // Iran
    {  640, 1008,  851375,  851388,  851410, 13, 22, 19, }, // Irkutsk
    {  640, 1016,  851429,  851441,  851462, 12, 21, 18, }, // Israel
    {  640, 1023,  851480,  851494,  851517, 14, 23, 20, }, // Japan
    {  640, 1047,       0,  851537,       0,  0, 17,  0, }, // Kazakhstan
    {  640, 1058,       0,  851554,       0,  0, 25,  0, }, // Kazakhstan_Eastern
    {  640, 1077,       0,  851579,       0,  0, 25,  0, }, // Kazakhstan_Western
    {  640, 1106,  851604,  851616,  851637, 12, 21, 18, }, // Korea
    {  640, 1112,       0,  851655,       0,  0, 12,  0, }, // Kosrae
    {  640, 1119,  851667,  851684,  851710, 17, 26, 23, }, // Krasnoyarsk
    {  640, 1151,       0,  851733,       0,  0, 18,  0, }, // Kyrgystan
    {  640, 1175,       0,  851751,       0,  0, 19,  0, }, // Line_Islands
    {  640, 1188,  851770,  851784,  851807, 14, 23, 20, }, // Lord_Howe
    {  640, 1204,  851827,  851840,  851862, 13, 22, 19, }, // Magadan
    {  640, 1219,       0,  851881,       0,  0, 15,  0, }, // Malaysia
    {  640, 1228,       0,  851896,       0,  0, 21,  0, }, // Maldives
    {  640, 1237,       0,  851917,       0,  0, 21,  0, }, // Marquesas
    {  640, 1247,       0,  851938,       0,  0, 23,  0, }, // Marshall_Islands
    {  640, 1264,  851961,  851975,  851998, 14, 23, 20, }, // Mauritius
    {  640, 1274,       0,  852018,       0,  0, 12,  0, }, // Mawson
    {  640, 1281,  852030,  852056,  852091, 26, 35, 32, }, // Mexico_Pacific
    {  640, 1296,  852123,  852139,  852164, 16, 25, 22, }, // Mongolia
    {  640, 1305,  852186,  852198,  852219, 12, 21, 18, }, // Moscow
    {  640, 1312,       0,  852237,       0,  0, 12,  0, }, // Myanmar
    {  640, 1320,       0,  852249,       0,  0, 11,  0, }, // Nauru
    {  640, 1326,       0,  852260,       0,  0, 11,  0, }, // Nepal
    {  640, 1332,  852271,  852293,  852324, 22, 31, 28, }, // New_Caledonia
    {  640, 1346,  852352,  852373,  852403, 21, 30, 27, }, // New_Zealand
    {  640, 1358,  852430,  852448,  852475, 18, 27, 24, }, // Newfoundland
    {  640, 1371,       0,  852499,       0,  0, 11,  0, }, // Niue
    {  640, 1376,  852510,  852529,  852557, 19, 28, 25, }, // Norfolk
    {  640, 1384,  852582,  852607,  852641, 25, 34, 31, }, // Noronha
    {  640, 1406,  852672,  852689,  852715, 17, 26, 23, }, // Novosibirsk
    {  640, 1418,  852738,  852748,  852767, 10, 19, 16, }, // Omsk
    {  640, 1428,  852783,  852797,  852820, 14, 23, 20, }, // Pakistan
    {  640, 1437,       0,  852840,       0,  0, 11,  0, }, // Palau
    {  640, 1443,       0,  852851,       0,  0, 25,  0, }, // Papua_New_Guinea
    {  640, 1460,  852876,  852890,  852913, 14, 23, 20, }, // Paraguay
    {  640, 1469,  852933,  852943,  852962, 10, 19, 16, }, // Peru
    {  640, 1474,  852978,  852992,  853015, 14, 23, 20, }, // Philippines
    {  640, 1486,       0,  853035,       0,  0, 21,  0, }, // Phoenix_Islands
    {  640, 1502,  853056,  853081,  853115, 25, 34, 31, }, // Pierre_Miquelon
    {  640, 1518,       0,  853146,       0,  0, 13,  0, }, // Pitcairn
    {  640, 1527,       0,  853159,       0,  0, 12,  0, }, // Ponape
    {  640, 1534,       0,  853171,       0,  0, 13,  0, }, // Pyongyang
    {  640, 1554,       0,  853184,       0,  0, 14,  0, }, // Reunion
    {  640, 1562,       0,  853198,       0,  0, 12,  0, }, // Rothera
    {  640, 1570,  853210,  853223,  853245, 13, 22, 19, }, // Sakhalin
    {  640, 1596,  853264,  853275,  853295, 11, 20, 17, }, // Samoa
    {  640, 1602,       0,  853312,       0,  0, 22,  0, }, // Seychelles
    {  640, 1624,       0,  853334,       0,  0, 14,  0, }, // Singapore
    {  640, 1634,       0,  853348,       0,  0, 22,  0, }, // Solomon
    {  640, 1642,       0,  853370,       0,  0, 22,  0, }, // South_Georgia
    {  640, 1656,       0,  853392,       0,  0, 13,  0, }, // Suriname
    {  640, 1676,       0,  853405,       0,  0, 11,  0, }, // Syowa
    {  640, 1682,       0,  853416,       0,  0, 11,  0, }, // Tahiti
    {  640, 1689,  853427,  853439,  853460, 12, 21, 18, }, // Taipei
    {  640, 1696,       0,  853478,       0,  0, 16,  0, }, // Tajikistan
    {  640, 1724,       0,  853494,       0,  0, 13,  0, }, // Tokelau
    {  640, 1732,  853507,  853518,  853538, 11, 20, 17, }, // Tonga
    {  640, 1738,       0,  853555,       0,  0, 11,  0, }, // Truk
    {  640, 1750,  853566,  853584,  853611, 18, 27, 24, }, // Turkmenistan
    {  640, 1763,       0,  853635,       0,  0, 12,  0, }, // Tuvalu
    {  640, 1777,  853647,  853660,  853682, 13, 22, 19, }, // Uruguay
    {  640, 1792,  853701,  853718,  853744, 17, 26, 23, }, // Uzbekistan
    {  640, 1803,  853767,  853780,  853802, 13, 22, 19, }, // Vanuatu
    {  640, 1811,       0,  853821,       0,  0, 15,  0, }, // Venezuela
    {  640, 1821,  853836,  853853,  853879, 17, 26, 23, }, // Vladivostok
    {  640, 1833,  853902,  853917,  853941, 15, 24, 21, }, // Volgograd
    {  640, 1843,       0,  853962,       0,  0, 12,  0, }, // Vostok
    {  640, 1850,       0,  853974,       0,  0, 16,  0, }, // Wake
    {  640, 1855,       0,  853990,       0,  0, 22,  0, }, // Wallis
    {  640, 1862,  854012,  854025,  854047, 13, 22, 19, }, // Yakutsk
    {  640, 1870,  854066,  854085,  854113, 19, 28, 25, }, // Yekaterinburg
    {  640, 1892,       0,  854138,       0,  0, 11,  0, }, // Yukon
    {  641,    5,       0,  854149,       0,  0, 14,  0, }, // Afghanistan Uzbek/Arabic/Afghanistan
    {  642,    5,       0,  854163,       0,  0, 16,  0, }, // Afghanistan Uzbek/Cyrillic/Uzbekistan
    {  642,   17,       0,  854179,       0,  0, 21,  0, }, // Africa_Central
    {  642,   32,       0,  854200,       0,  0, 19,  0, }, // Africa_Eastern
    {  642,   65,       0,  854219,       0,  0, 20,  0, }, // Africa_Southern
    {  642,   81,  854239,  854258,  854286, 19, 28, 24, }, // Africa_Western
    {  642,  107,  854310,  854322,  854343, 12, 21, 21, }, // Alaska
    {  642,  135,  854364,  854378,  854401, 14, 23, 19, }, // Amazon
    {  642,  142,  854420,  854435,  854474, 15, 39, 39, }, // America_Central
    {  642,  158,  854513,  854541,  854578, 28, 37, 37, }, // America_Eastern
    {  642,  174,  854615,  854640,  854674, 25, 34, 34, }, // America_Mountain
    {  642,  191,  854708,  854741,  854783, 33, 42, 42, }, // America_Pacific
    {  642,  232,  854825,  854840,  854864, 15, 24, 24, }, // Arabian
    {  642,  240,  854888,  854903,  854927, 15, 24, 20, }, // Argentina
    {  642,  250,  854947,  854969,  855000, 22, 31, 27, }, // Argentina_Western
    {  642,  268,  855027,  855043,  855068, 16, 25, 21, }, // Armenia
    {  642,  286,  855089,  855104,  855128, 15, 24, 24, }, // Atlantic
    {  642,  295,  855152,  855176,  855209, 24, 33, 33, }, // Australia_Central
    {  642,  313,  855242,  855273,  855313, 31, 40, 40, }, // Australia_CentralWestern
    {  642,  338,  855353,  855375,  855406, 22, 31, 31, }, // Australia_Eastern
    {  642,  356,  855437,  855459,  855490, 22, 31, 31, }, // Australia_Western
    {  642,  374,  855521,  855537,  855562, 16, 25, 21, }, // Azerbaijan
    {  642,  385,  855583,  855593,  855612, 10, 19, 15, }, // Azores
    {  642,  397,  855627,  855642,  855666, 15, 24, 20, }, // Bangladesh
    {  642,  415,       0,  855686,       0,  0, 11,  0, }, // Bhutan
    {  642,  422,       0,  855697,       0,  0, 13,  0, }, // Bolivia
    {  642,  437,  855710,  855724,  855747, 14, 23, 19, }, // Brasilia
    {  642,  454,       0,  855766,       0,  0, 23,  0, }, // Brunei
    {  642,  461,  855789,  855805,  855830, 16, 25, 21, }, // Cape_Verde
    {  642,  478,       0,  855851,       0,  0, 13,  0, }, // Chamorro
    {  642,  487,  855864,  855876,  855897, 12, 21, 21, }, // Chatham
    {  642,  495,  855918,  855928,  855947, 10, 19, 15, }, // Chile
    {  642,  501,  855962,  855973,  855993, 11, 20, 20, }, // China
    {  642,  507,       0,  856013,       0,  0, 21,  0, }, // Christmas
    {  642,  517,       0,  856034,       0,  0, 20,  0, }, // Cocos
    {  642,  523,  856054,  856068,  856091, 14, 23, 19, }, // Colombia
    {  642,  532,  856110,  856128,  856155, 18, 27, 28, }, // Cook
    {  642,  537,  856183,  856193,  856212, 10, 19, 19, }, // Cuba
    {  642,  548,       0,  856231,       0,  0, 11,  0, }, // Davis
    {  642,  564,       0,  856242,       0,  0, 20,  0, }, // DumontDUrville
    {  642,  601,       0,  856262,       0,  0, 18,  0, }, // East_Timor
    {  642,  612,  856280,  856297,  856323, 17, 26, 22, }, // Easter
    {  642,  619,       0,  856345,       0,  0, 13,  0, }, // Ecuador
    {  642,  627,  856358,  856379,  856409, 21, 30, 26, }, // Europe_Central
    {  642,  642,  856435,  856454,  856482, 19, 28, 24, }, // Europe_Eastern
    {  642,  680,  856506,  856525,  856553, 19, 28, 24, }, // Europe_Western
    {  642,  695,  856577,  856601,  856634, 24, 33, 29, }, // Falkland
    {  642,  704,  856663,  856673,  856692, 10, 19, 15, }, // Fiji
    {  642,  709,       0,  856707,       0,  0, 22,  0, }, // French_Guiana
    {  642,  723,       0,  856729,       0,  0, 35,  0, }, // French_Southern
    {  642,  746,       0,  856764,       0,  0, 15,  0, }, // Galapagos
    {  642,  756,       0,  856779,       0,  0, 13,  0, }, // Gambier
    {  642,  764,  856792,  856804,  856825, 12, 21, 17, }, // Georgia
    {  642,  772,       0,  856842,       0,  0, 22,  0, }, // Gilbert_Islands
    {  642,  788,       0,  856864,       0,  0, 13,  0, }, // GMT
    {  642,  830,  856877,  856900,  856932, 23, 32, 28, }, // Greenland_Eastern
    {  642,  848,  856960,  856983,  857015, 23, 32, 28, }, // Greenland_Western
    {  642,  871,       0,  857043,       0,  0, 12,  0, }, // Gulf
    {  642,  876,       0,  857055,       0,  0, 12,  0, }, // Guyana
    {  642,  883,  857067,  857085,  857112, 18, 27, 27, }, // Hawaii_Aleutian
    {  642,  899,  857139,  857152,  857174, 13, 22, 18, }, // Hong_Kong
    {  642,  909,  857192,  857202,  857221, 10, 19, 15, }, // Hovd
    {  642,  914,       0,  857236,       0,  0, 15,  0, }, // India
    {  642,  920,       0,  857251,       0,  0, 17,  0, }, // Indian_Ocean
    {  642,  933,       0,  857268,       0,  0, 16,  0, }, // Indochina
    {  642,  943,       0,  857284,       0,  0, 24,  0, }, // Indonesia_Central
    {  642,  961,       0,  857308,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  642,  979,       0,  857330,       0,  0, 22,  0, }, // Indonesia_Western
    {  642,  997,  857352,  857362,  857381, 10, 19, 19, }, // Iran
    {  642, 1008,  857400,  857413,  857435, 13, 22, 18, }, // Irkutsk
    {  642, 1016,  857453,  857465,  857486, 12, 21, 21, }, // Israel
    {  642, 1023,  857507,  857519,  857540, 12, 21, 21, }, // Japan
    {  642, 1058,       0,  857561,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  642, 1077,       0,  857584,       0,  0, 23,  0, }, // Kazakhstan_Western
    {  642, 1106,  857607,  857618,  857638, 11, 20, 20, }, // Korea
    {  642, 1112,       0,  857658,       0,  0, 12,  0, }, // Kosrae
    {  642, 1119,  857670,  857686,  857711, 16, 25, 21, }, // Krasnoyarsk
    {  642, 1151,       0,  857732,       0,  0, 17,  0, }, // Kyrgystan
    {  642, 1175,       0,  857749,       0,  0, 19,  0, }, // Line_Islands
    {  642, 1188,  857768,  857783,  857807, 15, 24, 24, }, // Lord_Howe
    {  642, 1204,  857831,  857844,  857866, 13, 22, 18, }, // Magadan
    {  642, 1219,       0,  857884,       0,  0, 14,  0, }, // Malaysia
    {  642, 1228,       0,  857898,       0,  0, 15,  0, }, // Maldives
    {  642, 1237,       0,  857913,       0,  0, 14,  0, }, // Marquesas
    {  642, 1247,       0,  857927,       0,  0, 22,  0, }, // Marshall_Islands
    {  642, 1264,  857949,  857963,  857986, 14, 23, 19, }, // Mauritius
    {  642, 1274,       0,  858005,       0,  0, 13,  0, }, // Mawson
    {  642, 1296,  858018,  858034,  858059, 16, 25, 21, }, // Mongolia
    {  642, 1305,  858080,  858092,  858113, 12, 21, 17, }, // Moscow
    {  642, 1312,       0,  858130,       0,  0, 12,  0, }, // Myanmar
    {  642, 1320,       0,  858142,       0,  0, 11,  0, }, // Nauru
    {  642, 1326,       0,  858153,       0,  0, 11,  0, }, // Nepal
    {  642, 1332,  858164,  858184,  858213, 20, 29, 25, }, // New_Caledonia
    {  642, 1346,  858238,  858257,  858285, 19, 28, 28, }, // New_Zealand
    {  642, 1358,  858313,  858331,  858358, 18, 27, 27, }, // Newfoundland
    {  642, 1371,       0,  858385,       0,  0, 10,  0, }, // Niue
    {  642, 1376,  858395,  858414,  858442, 19, 28, 24, }, // Norfolk
    {  642, 1384,  858466,  858491,  858525, 25, 34, 30, }, // Noronha
    {  642, 1406,  858555,  858572,  858598, 17, 26, 22, }, // Novosibirsk
    {  642, 1418,  764943,  858620,  858639, 10, 19, 15, }, // Omsk
    {  642, 1428,  858654,  858668,  858691, 14, 23, 19, }, // Pakistan
    {  642, 1437,       0,  858710,       0,  0, 11,  0, }, // Palau
    {  642, 1443,       0,  858721,       0,  0, 23,  0, }, // Papua_New_Guinea
    {  642, 1460,  858744,  858758,  858781, 14, 23, 19, }, // Paraguay
    {  642, 1469,  858800,  858810,  858829, 10, 19, 15, }, // Peru
    {  642, 1474,  858844,  858858,  858881, 14, 23, 19, }, // Philippines
    {  642, 1486,       0,  858900,       0,  0, 21,  0, }, // Phoenix_Islands
    {  642, 1502,  858921,  858947,  858982, 26, 35, 35, }, // Pierre_Miquelon
    {  642, 1518,       0,  859017,       0,  0, 13,  0, }, // Pitcairn
    {  642, 1527,       0,  859030,       0,  0, 12,  0, }, // Ponape
    {  642, 1554,       0,  859042,       0,  0, 13,  0, }, // Reunion
    {  642, 1562,       0,  859055,       0,  0, 12,  0, }, // Rothera
    {  642, 1570,  859067,  859080,  859102, 13, 22, 18, }, // Sakhalin
    {  642, 1596,  859120,  859131,  859151, 11, 20, 20, }, // Samoa
    {  642, 1602,       0,  859171,       0,  0, 21,  0, }, // Seychelles
    {  642, 1624,       0,  859192,       0,  0, 14,  0, }, // Singapore
    {  642, 1634,       0,  859206,       0,  0, 22,  0, }, // Solomon
    {  642, 1642,       0,  859228,       0,  0, 22,  0, }, // South_Georgia
    {  642, 1656,       0,  859250,       0,  0, 13,  0, }, // Suriname
    {  642, 1676,       0,  859263,       0,  0, 11,  0, }, // Syowa
    {  642, 1682,       0,  859274,       0,  0, 11,  0, }, // Tahiti
    {  642, 1689,  859285,  859297,  859318, 12, 21, 21, }, // Taipei
    {  642, 1696,       0,  859339,       0,  0, 16,  0, }, // Tajikistan
    {  642, 1724,       0,  859355,       0,  0, 13,  0, }, // Tokelau
    {  642, 1732,  859368,  859379,  859399, 11, 20, 16, }, // Tonga
    {  642, 1738,       0,  859415,       0,  0, 10,  0, }, // Truk
    {  642, 1750,  859425,  859443,  859470, 18, 27, 23, }, // Turkmenistan
    {  642, 1763,       0,  859493,       0,  0, 12,  0, }, // Tuvalu
    {  642, 1777,  859505,  859518,  859540, 13, 22, 18, }, // Uruguay
    {  642, 1792,  859558,  859574,  859599, 16, 25, 21, }, // Uzbekistan
    {  642, 1803,  859620,  859633,  859655, 13, 22, 18, }, // Vanuatu
    {  642, 1811,       0,  859673,       0,  0, 15,  0, }, // Venezuela
    {  642, 1821,  859688,  859705,  859731, 17, 26, 22, }, // Vladivostok
    {  642, 1833,  859753,  859768,  859792, 15, 24, 20, }, // Volgograd
    {  642, 1843,       0,  859812,       0,  0, 12,  0, }, // Vostok
    {  642, 1850,       0,  859824,       0,  0, 16,  0, }, // Wake
    {  642, 1855,       0,  859840,       0,  0, 22,  0, }, // Wallis
    {  642, 1862,  766318,  859862,  859883, 12, 21, 17, }, // Yakutsk
    {  642, 1870,  859900,  859918,  859945, 18, 27, 23, }, // Yekaterinburg
    {  646,    0,  859968,  859976,  859990,  8, 14, 15, }, // Acre Vietnamese/Latin/Vietnam
    {  646,    5,       0,  860005,       0,  0, 15,  0, }, // Afghanistan
    {  646,   17,       0,  860020,       0,  0, 13,  0, }, // Africa_Central
    {  646,   32,       0,  860033,       0,  0, 12,  0, }, // Africa_Eastern
    {  646,   65,       0,  860045,       0,  0, 17,  0, }, // Africa_Southern
    {  646,   81,  860062,  860073,  860090, 11, 17, 18, }, // Africa_Western
    {  646,  107,  860108,  860118,  860134, 10, 16, 17, }, // Alaska
    {  646,  128,  860151,  860161,  860177, 10, 16, 17, }, // Almaty
    {  646,  135,  860194,  860204,  860220, 10, 16, 17, }, // Amazon
    {  646,  142,  860237,  860251,  860271, 14, 20, 21, }, // America_Central
    {  646,  158,  860292,  860305,  860324, 13, 19, 20, }, // America_Eastern
    {  646,  174,  860344,  860356,  860374, 12, 18, 19, }, // America_Mountain
    {  646,  191,  860393,  860412,  860437, 19, 25, 26, }, // America_Pacific
    {  646,  207,  860463,  860473,  860489, 10, 16, 17, }, // Anadyr
    {  646,  214,  860506,  860514,  860528,  8, 14, 15, }, // Apia
    {  646,  219,  860543,  860552,  860567,  9, 15, 16, }, // Aqtau
    {  646,  225,  860583,  860593,  860609, 10, 16, 17, }, // Aqtobe
    {  646,  232,  860626,  860635,  860650,  9, 15, 16, }, // Arabian
    {  646,  240,  860666,  860679,  860698, 13, 19, 20, }, // Argentina
    {  646,  250,  860718,  860740,  860768, 22, 28, 29, }, // Argentina_Western
    {  646,  268,  860797,  860808,  860825, 11, 17, 18, }, // Armenia
    {  646,  286,  860843,  860860,  860883, 17, 23, 24, }, // Atlantic
    {  646,  295,  860907,  860931,  860961, 24, 30, 31, }, // Australia_Central
    {  646,  313,  860992,  861020,  861054, 28, 34, 35, }, // Australia_CentralWestern
    {  646,  338,  861089,  861112,  861141, 23, 29, 30, }, // Australia_Eastern
    {  646,  356,  861171,  861193,  861221, 22, 28, 29, }, // Australia_Western
    {  646,  374,  861250,  861264,  861284, 14, 20, 21, }, // Azerbaijan
    {  646,  385,  861305,  861315,  861331, 10, 16, 17, }, // Azores
    {  646,  397,  861348,  861362,  861382, 14, 20, 21, }, // Bangladesh
    {  646,  415,       0,  861403,       0,  0, 10,  0, }, // Bhutan
    {  646,  422,       0,  861413,       0,  0, 11,  0, }, // Bolivia
    {  646,  437,  861424,  861436,  861454, 12, 18, 19, }, // Brasilia
    {  646,  454,       0,  861473,       0,  0, 21,  0, }, // Brunei
    {  646,  461,  861494,  861508,  861528, 14, 20, 21, }, // Cape_Verde
    {  646,  472,       0,  861549,       0,  0,  9,  0, }, // Casey
    {  646,  478,       0,  861558,       0,  0, 12,  0, }, // Chamorro
    {  646,  487,  861570,  861581,  861598, 11, 17, 18, }, // Chatham
    {  646,  495,  861616,  861625,  861640,  9, 15, 16, }, // Chile
    {  646,  501,  861656,  861670,  861690, 14, 20, 21, }, // China
    {  646,  507,       0,  861711,       0,  0, 17,  0, }, // Christmas
    {  646,  517,       0,  861728,       0,  0, 18,  0, }, // Cocos
    {  646,  523,  861746,  861758,  861776, 12, 18, 19, }, // Colombia
    {  646,  532,  861795,  861812,  861835, 17, 23, 28, }, // Cook
    {  646,  537,  861863,  861871,  861885,  8, 14, 15, }, // Cuba
    {  646,  548,       0,  861900,       0,  0,  9,  0, }, // Davis
    {  646,  564,       0,  861909,       0,  0, 20,  0, }, // DumontDUrville
    {  646,  601,       0,  861929,       0,  0, 14,  0, }, // East_Timor
    {  646,  612,  861943,  861960,  861983, 17, 23, 24, }, // Easter
    {  646,  619,       0,  862007,       0,  0, 11,  0, }, // Ecuador
    {  646,  627,  862018,  862030,  862048, 12, 18, 19, }, // Europe_Central
    {  646,  642,  862067,  862078,  862095, 11, 17, 18, }, // Europe_Eastern
    {  646,  657,       0,  862113,       0,  0, 21,  0, }, // Europe_Further_Eastern
    {  646,  680,  862134,  862144,  862160, 10, 16, 17, }, // Europe_Western
    {  646,  695,  862177,  862198,  862225, 21, 27, 28, }, // Falkland
    {  646,  704,  862253,  862261,  862275,  8, 14, 15, }, // Fiji
    {  646,  709,       0,  862290,       0,  0, 21,  0, }, // French_Guiana
    {  646,  723,       0,  862311,       0,  0, 28,  0, }, // French_Southern
    {  646,  746,       0,  862339,       0,  0, 13,  0, }, // Galapagos
    {  646,  756,       0,  862352,       0,  0, 11,  0, }, // Gambier
    {  646,  764,  862363,  862374,  862391, 11, 17, 18, }, // Georgia
    {  646,  772,       0,  862409,       0,  0, 20,  0, }, // Gilbert_Islands
    {  646,  788,       0,  862429,       0,  0, 24,  0, }, // GMT
    {  646,  830,  862453,  862476,  862505, 23, 29, 30, }, // Greenland_Eastern
    {  646,  848,  862535,  862557,  862585, 22, 28, 29, }, // Greenland_Western
    {  646,  866,       0,  862614,       0,  0, 14,  0, }, // Guam
    {  646,  871,       0,  862628,       0,  0, 19,  0, }, // Gulf
    {  646,  876,       0,  862647,       0,  0, 10,  0, }, // Guyana
    {  646,  883,  862657,  862673,  862695, 16, 22, 23, }, // Hawaii_Aleutian
    {  646,  899,  862718,  862731,  862750, 13, 19, 20, }, // Hong_Kong
    {  646,  909,  862770,  862778,  862792,  8, 14, 15, }, // Hovd
    {  646,  914,       0,  862807,       0,  0, 15,  0, }, // India
    {  646,  920,       0,  862822,       0,  0, 15,  0, }, // Indian_Ocean
    {  646,  933,       0,  862837,       0,  0, 14,  0, }, // Indochina
    {  646,  943,       0,  862851,       0,  0, 24,  0, }, // Indonesia_Central
    {  646,  961,       0,  862875,       0,  0, 23,  0, }, // Indonesia_Eastern
    {  646,  979,       0,  862898,       0,  0, 22,  0, }, // Indonesia_Western
    {  646,  997,  862920,  862928,  862942,  8, 14, 15, }, // Iran
    {  646, 1008,  862957,  862968,  862985, 11, 17, 18, }, // Irkutsk
    {  646, 1016,  863003,  863013,  863029, 10, 16, 17, }, // Israel
    {  646, 1023,  863046,  863058,  863076, 12, 18, 19, }, // Japan
    {  646, 1029,  863095,  863123,  863157, 28, 34, 35, }, // Kamchatka
    {  646, 1047,       0,  863192,       0,  0, 14,  0, }, // Kazakhstan
    {  646, 1058,       0,  863206,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  646, 1077,       0,  863230,       0,  0, 23,  0, }, // Kazakhstan_Western
    {  646, 1106,  863253,  863265,  863283, 12, 18, 19, }, // Korea
    {  646, 1112,       0,  863302,       0,  0, 10,  0, }, // Kosrae
    {  646, 1119,  863312,  863327,  863348, 15, 21, 22, }, // Krasnoyarsk
    {  646, 1151,       0,  863370,       0,  0, 13,  0, }, // Kyrgystan
    {  646, 1161,       0,  863383,       0,  0,  9,  0, }, // Lanka
    {  646, 1175,       0,  863392,       0,  0, 17,  0, }, // Line_Islands
    {  646, 1188,  863409,  863422,  863441, 13, 19, 20, }, // Lord_Howe
    {  646, 1198,  863461,  863471,  863487, 10, 16, 17, }, // Macau
    {  646, 1204,  863504,  863515,  863532, 11, 17, 18, }, // Magadan
    {  646, 1219,       0,  863550,       0,  0, 12,  0, }, // Malaysia
    {  646, 1228,       0,  863562,       0,  0, 12,  0, }, // Maldives
    {  646, 1237,       0,  863574,       0,  0, 13,  0, }, // Marquesas
    {  646, 1247,       0,  863587,       0,  0, 21,  0, }, // Marshall_Islands
    {  646, 1264,  863608,  863621,  863640, 13, 19, 20, }, // Mauritius
    {  646, 1274,       0,  863660,       0,  0, 10,  0, }, // Mawson
    {  646, 1281,  863670,  863696,  863728, 26, 32, 33, }, // Mexico_Pacific
    {  646, 1296,  863761,  863775,  863795, 14, 20, 21, }, // Mongolia
    {  646, 1305,  863816,  863828,  863846, 12, 18, 19, }, // Moscow
    {  646, 1312,       0,  863865,       0,  0, 11,  0, }, // Myanmar
    {  646, 1320,       0,  863876,       0,  0,  9,  0, }, // Nauru
    {  646, 1326,       0,  863885,       0,  0,  9,  0, }, // Nepal
    {  646, 1332,  863894,  863911,  863934, 17, 23, 24, }, // New_Caledonia
    {  646, 1346,  863958,  863973,  863994, 15, 21, 22, }, // New_Zealand
    {  646, 1358,  864016,  864032,  864054, 16, 22, 23, }, // Newfoundland
    {  646, 1371,       0,  864077,       0,  0,  8,  0, }, // Niue
    {  646, 1376,  864085,  864100,  864121, 15, 21, 22, }, // Norfolk
    {  646, 1384,  864143,  864166,  864195, 23, 29, 30, }, // Noronha
    {  646, 1392,       0,  864225,       0,  0, 24,  0, }, // North_Mariana
    {  646, 1406,  864249,  864264,  864285, 15, 21, 22, }, // Novosibirsk
    {  646, 1418,  864307,  864315,  864329,  8, 14, 15, }, // Omsk
    {  646, 1428,  864344,  864356,  864374, 12, 18, 19, }, // Pakistan
    {  646, 1437,       0,  864393,       0,  0,  9,  0, }, // Palau
    {  646, 1443,       0,  864402,       0,  0, 20,  0, }, // Papua_New_Guinea
    {  646, 1460,  864422,  864434,  864452, 12, 18, 19, }, // Paraguay
    {  646, 1469,  864471,  864479,  864493,  8, 14, 15, }, // Peru
    {  646, 1474,  864508,  864521,  864540, 13, 19, 20, }, // Philippines
    {  646, 1486,       0,  864560,       0,  0, 20,  0, }, // Phoenix_Islands
    {  646, 1502,  864580,  864606,  864638, 26, 32, 35, }, // Pierre_Miquelon
    {  646, 1518,       0,  864673,       0,  0, 12,  0, }, // Pitcairn
    {  646, 1527,       0,  864685,       0,  0, 10,  0, }, // Ponape
    {  646, 1534,       0,  864695,       0,  0, 15,  0, }, // Pyongyang
    {  646, 1544,  864710,  864723,  864742, 13, 19, 20, }, // Qyzylorda
    {  646, 1554,       0,  864762,       0,  0, 11,  0, }, // Reunion
    {  646, 1562,       0,  864773,       0,  0, 11,  0, }, // Rothera
    {  646, 1570,  864784,  864796,  864814, 12, 18, 19, }, // Sakhalin
    {  646, 1579,  864833,  864843,  864859, 10, 16, 17, }, // Samara
    {  646, 1596,  864876,  864885,  864900,  9, 15, 18, }, // Samoa
    {  646, 1602,       0,  864918,       0,  0, 14,  0, }, // Seychelles
    {  646, 1624,       0,  864932,       0,  0, 13,  0, }, // Singapore
    {  646, 1634,       0,  864945,       0,  0, 20,  0, }, // Solomon
    {  646, 1642,       0,  864965,       0,  0, 15,  0, }, // South_Georgia
    {  646, 1656,       0,  864980,       0,  0, 12,  0, }, // Suriname
    {  646, 1676,       0,  864992,       0,  0,  9,  0, }, // Syowa
    {  646, 1682,       0,  865001,       0,  0, 10,  0, }, // Tahiti
    {  646, 1689,  865011,  865022,  865039, 11, 17, 18, }, // Taipei
    {  646, 1696,       0,  865057,       0,  0, 14,  0, }, // Tajikistan
    {  646, 1724,       0,  865071,       0,  0, 11,  0, }, // Tokelau
    {  646, 1732,  865082,  865091,  865106,  9, 15, 16, }, // Tonga
    {  646, 1738,       0,  865122,       0,  0,  9,  0, }, // Truk
    {  646, 1750,  865131,  865147,  865169, 16, 22, 23, }, // Turkmenistan
    {  646, 1763,       0,  865192,       0,  0, 10,  0, }, // Tuvalu
    {  646, 1777,  865202,  865213,  865230, 11, 17, 18, }, // Uruguay
    {  646, 1792,  865248,  865262,  865282, 14, 20, 21, }, // Uzbekistan
    {  646, 1803,  865303,  865314,  865331, 11, 17, 18, }, // Vanuatu
    {  646, 1811,       0,  865349,       0,  0, 13,  0, }, // Venezuela
    {  646, 1821,  865362,  865377,  865398, 15, 21, 22, }, // Vladivostok
    {  646, 1833,  865420,  865433,  865452, 13, 19, 20, }, // Volgograd
    {  646, 1843,       0,  865472,       0,  0, 10,  0, }, // Vostok
    {  646, 1850,       0,  865482,       0,  0, 12,  0, }, // Wake
    {  646, 1855,       0,  865494,       0,  0, 20,  0, }, // Wallis
    {  646, 1862,  865514,  865525,  865542, 11, 17, 18, }, // Yakutsk
    {  646, 1870,  865560,  865577,  865600, 17, 23, 24, }, // Yekaterinburg
    {  646, 1892,       0,  865624,       0,  0,  9,  0, }, // Yukon
    {  650,  286,  865633,  865646,  865667, 13, 21, 19, }, // Atlantic Walser/Latin/Switzerland
    {  650,  627,  865686,  865705,  865732, 19, 27, 25, }, // Europe_Central
    {  650,  642,  865757,  865773,  865797, 16, 24, 22, }, // Europe_Eastern
    {  650,  680,  865819,  865836,  865861, 17, 25, 23, }, // Europe_Western
    {  652,    5,       0,  865884,       0,  0, 17,  0, }, // Afghanistan Welsh/Latin/United Kingdom
    {  652,   17,       0,  865901,       0,  0, 24,  0, }, // Africa_Central
    {  652,   32,       0,  865925,       0,  0, 21,  0, }, // Africa_Eastern
    {  652,   65,       0,  865946,       0,  0, 24,  0, }, // Africa_Southern
    {  652,   81,  865970,  865993,  866024, 23, 31, 27, }, // Africa_Western
    {  652,  107,  866051,  866063,  866083, 12, 20, 16, }, // Alaska
    {  652,  135,  866099,  866113,  866135, 14, 22, 18, }, // Amazon
    {  652,  142,  866153,  866185,  866225, 32, 40, 36, }, // America_Central
    {  652,  158,  866261,  866290,  866327, 29, 37, 33, }, // America_Eastern
    {  652,  174,  866360,  866392,  866432, 32, 40, 36, }, // America_Mountain
    {  652,  191,  866468,  866503,  866546, 35, 43, 39, }, // America_Pacific
    {  652,  214,  866585,  866595,  866613, 10, 18, 14, }, // Apia
    {  652,  232,  866627,  866641,  866663, 14, 22, 18, }, // Arabian
    {  652,  240,  866681,  866698,  866720, 17, 22, 18, }, // Argentina
    {  652,  250,  866738,  866762,  866794, 24, 32, 28, }, // Argentina_Western
    {  652,  268,  866822,  866835,  866856, 13, 21, 17, }, // Armenia
    {  652,  286,  866873,  866897,  866929, 24, 32, 28, }, // Atlantic
    {  652,  295,  866957,  866983,  867017, 26, 34, 30, }, // Australia_Central
    {  652,  313,  867047,  867083,  867127, 36, 44, 40, }, // Australia_CentralWestern
    {  652,  338,  867167,  867190,  867221, 23, 31, 27, }, // Australia_Eastern
    {  652,  356,  867248,  867273,  867306, 25, 33, 29, }, // Australia_Western
    {  652,  374,  867335,  867351,  867375, 16, 24, 20, }, // Azerbaijan
    {  652,  385,  867395,  867410,  867433, 15, 23, 19, }, // Azores
    {  652,  397,  867452,  867468,  867492, 16, 24, 20, }, // Bangladesh
    {  652,  415,       0,  867512,       0,  0, 12,  0, }, // Bhutan
    {  652,  422,       0,  867524,       0,  0, 13,  0, }, // Bolivia
    {  652,  437,  867537,  867551,  867573, 14, 22, 18, }, // Brasilia
    {  652,  454,       0,  867591,       0,  0, 23,  0, }, // Brunei
    {  652,  461,  867614,  867630,  867654, 16, 24, 20, }, // Cape_Verde
    {  652,  478,       0,  867674,       0,  0, 14,  0, }, // Chamorro
    {  652,  487,  867688,  867701,  867722, 13, 21, 17, }, // Chatham
    {  652,  495,  867739,  867750,  867769, 11, 19, 15, }, // Chile
    {  652,  501,  867784,  867797,  867818, 13, 21, 17, }, // China
    {  652,  507,       0,  867835,       0,  0, 20,  0, }, // Christmas
    {  652,  517,       0,  867855,       0,  0, 20,  0, }, // Cocos
    {  652,  523,  867875,  867889,  867911, 14, 22, 18, }, // Colombia
    {  652,  532,  867929,  867948,  867975, 19, 27, 30, }, // Cook
    {  652,  537,  868005,  868016,  868035, 11, 19, 14, }, // Cuba
    {  652,  548,       0,  868049,       0,  0, 11,  0, }, // Davis
    {  652,  564,       0,  868060,       0,  0, 22,  0, }, // DumontDUrville
    {  652,  601,       0,  868082,       0,  0, 19,  0, }, // East_Timor
    {  652,  612,  868101,  868118,  868143, 17, 25, 21, }, // Easter
    {  652,  619,       0,  868164,       0,  0, 13,  0, }, // Ecuador
    {  652,  627,  868177,  868199,  868229, 22, 30, 26, }, // Europe_Central
    {  652,  642,  868255,  868274,  868301, 19, 27, 23, }, // Europe_Eastern
    {  652,  657,       0,  868324,       0,  0, 24,  0, }, // Europe_Further_Eastern
    {  652,  680,  868348,  868369,  868398, 21, 29, 25, }, // Europe_Western
    {  652,  695,  868423,  868456,  868497, 33, 41, 37, }, // Falkland
    {  652,  704,  868534,  868544,  868562, 10, 18, 14, }, // Fiji
    {  652,  709,       0,  868576,       0,  0, 21,  0, }, // French_Guiana
    {  652,  723,       0,  868597,       0,  0, 43,  0, }, // French_Southern
    {  652,  746,       0,  868640,       0,  0, 15,  0, }, // Galapagos
    {  652,  756,       0,  868655,       0,  0, 13,  0, }, // Gambier
    {  652,  764,  868668,  868681,  868702, 13, 21, 17, }, // Georgia
    {  652,  772,       0,  868719,       0,  0, 22,  0, }, // Gilbert_Islands
    {  652,  788,       0,  868741,       0,  0, 23,  0, }, // GMT
    {  652,  830,  868764,  868789,  868822, 25, 33, 29, }, // Greenland_Eastern
    {  652,  848,  868851,  868878,  868913, 27, 35, 31, }, // Greenland_Western
    {  652,  871,       0,  868944,       0,  0, 21,  0, }, // Gulf
    {  652,  876,       0,  868965,       0,  0, 12,  0, }, // Guyana
    {  652,  883,  868977,  868998,  869027, 21, 29, 25, }, // Hawaii_Aleutian
    {  652,  899,  869052,  869067,  869090, 15, 23, 19, }, // Hong_Kong
    {  652,  909,  869109,  869119,  869137, 10, 18, 14, }, // Hovd
    {  652,  914,       0,  869151,       0,  0, 11,  0, }, // India
    {  652,  920,       0,  869162,       0,  0, 19,  0, }, // Indian_Ocean
    {  652,  933,       0,  869181,       0,  0, 18,  0, }, // Indochina
    {  652,  943,       0,  869199,       0,  0, 26,  0, }, // Indonesia_Central
    {  652,  961,       0,  869225,       0,  0, 23,  0, }, // Indonesia_Eastern
    {  652,  979,       0,  869248,       0,  0, 25,  0, }, // Indonesia_Western
    {  652,  997,  869273,  869283,  869301, 10, 18, 14, }, // Iran
    {  652, 1008,  869315,  869328,  869349, 13, 21, 17, }, // Irkutsk
    {  652, 1016,  869366,  869378,  869398, 12, 20, 16, }, // Israel
    {  652, 1023,  869414,  869425,  869444, 11, 19, 15, }, // Japan
    {  652, 1047,       0,  869459,       0,  0, 16,  0, }, // Kazakhstan
    {  652, 1058,       0,  869475,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  652, 1077,       0,  869499,       0,  0, 26,  0, }, // Kazakhstan_Western
    {  652, 1106,  869525,  869536,  869555, 11, 19, 15, }, // Korea
    {  652, 1112,       0,  869570,       0,  0, 12,  0, }, // Kosrae
    {  652, 1119,  869582,  869599,  869624, 17, 25, 21, }, // Krasnoyarsk
    {  652, 1151,       0,  869645,       0,  0, 16,  0, }, // Kyrgystan
    {  652, 1175,       0,  869661,       0,  0, 19,  0, }, // Line_Islands
    {  652, 1188,  869680,  869702,  869732, 22, 30, 26, }, // Lord_Howe
    {  652, 1204,  869758,  869771,  869792, 13, 21, 17, }, // Magadan
    {  652, 1219,       0,  869809,       0,  0, 14,  0, }, // Malaysia
    {  652, 1228,       0,  869823,       0,  0, 16,  0, }, // Maldives
    {  652, 1237,       0,  869839,       0,  0, 15,  0, }, // Marquesas
    {  652, 1247,       0,  869854,       0,  0, 23,  0, }, // Marshall_Islands
    {  652, 1264,  869877,  869892,  869915, 15, 23, 19, }, // Mauritius
    {  652, 1274,       0,  869934,       0,  0, 12,  0, }, // Mawson
    {  652, 1281,  869946,  869968,  869998, 22, 30, 26, }, // Mexico_Pacific
    {  652, 1296,  870024,  870040,  870064, 16, 24, 20, }, // Mongolia
    {  652, 1305,  870084,  870096,  870116, 12, 20, 16, }, // Moscow
    {  652, 1312,       0,  870132,       0,  0, 13,  0, }, // Myanmar
    {  652, 1320,       0,  870145,       0,  0, 11,  0, }, // Nauru
    {  652, 1326,       0,  870156,       0,  0, 11,  0, }, // Nepal
    {  652, 1332,  870167,  870189,  870219, 22, 30, 26, }, // New_Caledonia
    {  652, 1346,  870245,  870264,  870291, 19, 27, 23, }, // New_Zealand
    {  652, 1358,  870314,  870332,  870358, 18, 26, 22, }, // Newfoundland
    {  652, 1371,       0,  870380,       0,  0, 10,  0, }, // Niue
    {  652, 1376,  870390,  870408,  870434, 18, 26, 22, }, // Norfolk
    {  652, 1384,  870456,  870481,  870514, 25, 33, 29, }, // Noronha
    {  652, 1406,  870543,  870560,  870585, 17, 25, 21, }, // Novosibirsk
    {  652, 1418,  870606,  870616,  870634, 10, 18, 14, }, // Omsk
    {  652, 1428,  870648,  870662,  870684, 14, 22, 18, }, // Pakistan
    {  652, 1437,       0,  870702,       0,  0, 11,  0, }, // Palau
    {  652, 1443,       0,  870713,       0,  0, 25,  0, }, // Papua_New_Guinea
    {  652, 1460,  870738,  870752,  870774, 14, 22, 18, }, // Paraguay
    {  652, 1469,  870792,  870803,  870822, 11, 19, 15, }, // Peru
    {  652, 1474,  870837,  870852,  870875, 15, 23, 19, }, // Philippines
    {  652, 1486,       0,  870894,       0,  0, 22,  0, }, // Phoenix_Islands
    {  652, 1502,  870916,  870946,  870984, 30, 38, 34, }, // Pierre_Miquelon
    {  652, 1518,       0,  871018,       0,  0, 14,  0, }, // Pitcairn
    {  652, 1527,       0,  871032,       0,  0, 13,  0, }, // Ponape
    {  652, 1534,       0,  871045,       0,  0, 15,  0, }, // Pyongyang
    {  652, 1554,       0,  871060,       0,  0, 13,  0, }, // Reunion
    {  652, 1562,       0,  871073,       0,  0, 13,  0, }, // Rothera
    {  652, 1570,  871086,  871100,  871122, 14, 22, 18, }, // Sakhalin
    {  652, 1579,  871140,  871152,  871172, 12, 20, 16, }, // Samara
    {  652, 1596,  871188,  871199,  871218, 11, 19, 15, }, // Samoa
    {  652, 1602,       0,  871233,       0,  0, 16,  0, }, // Seychelles
    {  652, 1624,       0,  871249,       0,  0, 15,  0, }, // Singapore
    {  652, 1634,       0,  871264,       0,  0, 22,  0, }, // Solomon
    {  652, 1642,       0,  871286,       0,  0, 16,  0, }, // South_Georgia
    {  652, 1656,       0,  871302,       0,  0, 14,  0, }, // Suriname
    {  652, 1676,       0,  871316,       0,  0, 11,  0, }, // Syowa
    {  652, 1682,       0,  871327,       0,  0, 12,  0, }, // Tahiti
    {  652, 1689,  871339,  871351,  871371, 12, 20, 16, }, // Taipei
    {  652, 1696,       0,  871387,       0,  0, 16,  0, }, // Tajikistan
    {  652, 1724,       0,  871403,       0,  0, 13,  0, }, // Tokelau
    {  652, 1732,  871416,  871427,  871446, 11, 19, 15, }, // Tonga
    {  652, 1738,       0,  871461,       0,  0, 11,  0, }, // Truk
    {  652, 1750,  871472,  871490,  871516, 18, 26, 22, }, // Turkmenistan
    {  652, 1763,       0,  871538,       0,  0, 12,  0, }, // Tuvalu
    {  652, 1777,  871550,  871563,  871584, 13, 21, 17, }, // Uruguay
    {  652, 1792,  871601,  871617,  871641, 16, 24, 20, }, // Uzbekistan
    {  652, 1803,  871661,  871674,  871695, 13, 21, 17, }, // Vanuatu
    {  652, 1811,       0,  871712,       0,  0, 15,  0, }, // Venezuela
    {  652, 1821,  871727,  871744,  871769, 17, 25, 21, }, // Vladivostok
    {  652, 1833,  871790,  871805,  871828, 15, 23, 19, }, // Volgograd
    {  652, 1843,       0,  871847,       0,  0, 12,  0, }, // Vostok
    {  652, 1850,       0,  871859,       0,  0, 15,  0, }, // Wake
    {  652, 1855,       0,  871874,       0,  0, 21,  0, }, // Wallis
    {  652, 1862,  871895,  871908,  871929, 13, 21, 17, }, // Yakutsk
    {  652, 1870,  871946,  871965,  871992, 19, 27, 23, }, // Yekaterinburg
    {  652, 1892,       0,  872015,       0,  0, 11,  0, }, // Yukon
    {  653,    5,       0,  872026,       0,  0, 14,  0, }, // Afghanistan Western Balochi/Arabic/Pakistan
    {  653, 1058,       0,  872040,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  653, 1077,       0,  872064,       0,  0, 23,  0, }, // Kazakhstan_Western
    {  653, 1151,       0,  872087,       0,  0, 15,  0, }, // Kyrgystan
    {  653, 1696,       0,  872102,       0,  0, 14,  0, }, // Tajikistan
    {  658,    0,  872116,  872125,  872142,  9, 17, 15, }, // Acre Western Frisian/Latin/Netherlands
    {  658,    5,       0,  872157,       0,  0, 15,  0, }, // Afghanistan
    {  658,   17,       0,  872172,       0,  0, 25,  0, }, // Africa_Central
    {  658,   32,       0,  872197,       0,  0, 21,  0, }, // Africa_Eastern
    {  658,   65,       0,  872218,       0,  0, 20,  0, }, // Africa_Southern
    {  658,   81,  872238,  872259,  872288, 21, 29, 27, }, // Africa_Western
    {  658,  107,  872315,  872326,  872345, 11, 19, 17, }, // Alaska
    {  658,  128,  872362,  872375,  872396, 13, 21, 19, }, // Almaty
    {  658,  135,  872415,  872427,  872447, 12, 20, 18, }, // Amazon
    {  658,  142,  872465,  872477,  872497, 12, 20, 18, }, // America_Central
    {  658,  158,  872515,  872527,  872547, 12, 20, 18, }, // America_Eastern
    {  658,  174,  872565,  872578,  872599, 13, 21, 19, }, // America_Mountain
    {  658,  191,  872618,  872630,  872650, 12, 20, 18, }, // America_Pacific
    {  658,  207,  872668,  872679,  872698, 11, 19, 17, }, // Anadyr
    {  658,  214,  872715,  872724,  872741,  9, 17, 15, }, // Apia
    {  658,  219,  872756,  872766,  872784, 10, 18, 16, }, // Aqtau
    {  658,  225,  872800,  872811,  872830, 11, 19, 17, }, // Aqtobe
    {  658,  232,  872847,  872860,  872881, 13, 21, 19, }, // Arabian
    {  658,  240,  872900,  872916,  872940, 16, 24, 22, }, // Argentina
    {  658,  250,  872962,  872983,  873012, 21, 29, 27, }, // Argentina_Western
    {  658,  268,  873039,  873052,  873073, 13, 21, 19, }, // Armenia
    {  658,  286,  873092,  873105,  873126, 13, 21, 19, }, // Atlantic
    {  658,  295,  873145,  873168,  873199, 23, 31, 29, }, // Australia_Central
    {  658,  313,  873228,  873262,  873304, 34, 42, 40, }, // Australia_CentralWestern
    {  658,  338,  873344,  873365,  873394, 21, 29, 27, }, // Australia_Eastern
    {  658,  356,  873421,  873442,  873471, 21, 29, 27, }, // Australia_Western
    {  658,  374,  873498,  873519,  873548, 21, 29, 27, }, // Azerbaijan
    {  658,  385,  873575,  873586,  873605, 11, 19, 17, }, // Azores
    {  658,  397,  873622,  873636,  873658, 14, 22, 20, }, // Bangladesh
    {  658,  415,       0,  873678,       0,  0, 15,  0, }, // Bhutan
    {  658,  422,       0,  873693,       0,  0, 17,  0, }, // Bolivia
    {  658,  437,  873710,  873728,  873754, 18, 26, 24, }, // Brasilia
    {  658,  454,       0,  873778,       0,  0, 13,  0, }, // Brunei
    {  658,  461,  873791,  873808,  873833, 17, 25, 23, }, // Cape_Verde
    {  658,  478,       0,  873856,       0,  0, 13,  0, }, // Chamorro
    {  658,  487,  873869,  873881,  873901, 12, 20, 18, }, // Chatham
    {  658,  495,  873919,  873933,  873955, 14, 22, 20, }, // Chile
    {  658,  501,  873975,  873988,  874009, 13, 21, 19, }, // China
    {  658,  507,       0,  874028,       0,  0, 18,  0, }, // Christmas
    {  658,  517,       0,  874046,       0,  0, 18,  0, }, // Cocos
    {  658,  523,  874064,  874082,  874108, 18, 26, 24, }, // Colombia
    {  658,  532,  874132,  874148,  874172, 16, 24, 28, }, // Cook
    {  658,  537,  874200,  874214,  874236, 14, 22, 20, }, // Cuba
    {  658,  548,       0,  874256,       0,  0, 10,  0, }, // Davis
    {  658,  564,       0,  874266,       0,  0, 21,  0, }, // DumontDUrville
    {  658,  601,       0,  874287,       0,  0, 18,  0, }, // East_Timor
    {  658,  612,  874305,  874324,  874351, 19, 27, 25, }, // Easter
    {  658,  619,       0,  874376,       0,  0, 18,  0, }, // Ecuador
    {  658,  627,  874394,  874416,  874446, 22, 30, 28, }, // Europe_Central
    {  658,  642,  874474,  874494,  874522, 20, 28, 26, }, // Europe_Eastern
    {  658,  657,       0,  874548,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  658,  680,  874579,  874599,  874627, 20, 28, 26, }, // Europe_Western
    {  658,  695,  874653,  874673,  874701, 20, 28, 26, }, // Falkland
    {  658,  704,  874727,  874739,  874759, 12, 20, 18, }, // Fiji
    {  658,  709,       0,  874777,       0,  0, 20,  0, }, // French_Guiana
    {  658,  723,       0,  874797,       0,  0, 35,  0, }, // French_Southern
    {  658,  746,       0,  874832,       0,  0, 22,  0, }, // Galapagos
    {  658,  756,       0,  874854,       0,  0, 20,  0, }, // Gambier
    {  658,  764,  874874,  874888,  874910, 14, 22, 20, }, // Georgia
    {  658,  772,       0,  874930,       0,  0, 20,  0, }, // Gilbert_Islands
    {  658,  788,       0,    2791,       0,  0, 19,  0, }, // GMT
    {  658,  830,  874950,  874971,  875000, 21, 29, 27, }, // Greenland_Eastern
    {  658,  848,  875027,  875048,  875077, 21, 29, 27, }, // Greenland_Western
    {  658,  866,       0,  875104,       0,  0, 20,  0, }, // Guam
    {  658,  871,       0,  875124,       0,  0, 17,  0, }, // Gulf
    {  658,  876,       0,  874783,       0,  0, 14,  0, }, // Guyana
    {  658,  883,  875141,  875163,  875193, 22, 30, 28, }, // Hawaii_Aleutian
    {  658,  899,  875221,  875236,  875259, 15, 23, 21, }, // Hong_Kong
    {  658,  909,  875280,  875289,  875306,  9, 17, 15, }, // Hovd
    {  658,  914,       0,  875321,       0,  0, 14,  0, }, // India
    {  658,  920,       0,  875335,       0,  0, 19,  0, }, // Indian_Ocean
    {  658,  933,       0,  875354,       0,  0, 16,  0, }, // Indochina
    {  658,  943,       0,  875370,       0,  0, 25,  0, }, // Indonesia_Central
    {  658,  961,       0,  875395,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  658,  979,       0,  875416,       0,  0, 21,  0, }, // Indonesia_Western
    {  658,  997,  875437,  875450,  875471, 13, 21, 19, }, // Iran
    {  658, 1008,  875490,  875503,  875524, 13, 21, 19, }, // Irkutsk
    {  658, 1016,  875543,  875558,  875581, 15, 23, 21, }, // Israel
    {  658, 1023,  875602,  875615,  875636, 13, 21, 19, }, // Japan
    {  658, 1029,  875655,  875685,  875723, 30, 38, 36, }, // Kamchatka
    {  658, 1047,       0,  875759,       0,  0, 15,  0, }, // Kazakhstan
    {  658, 1058,       0,  875774,       0,  0, 18,  0, }, // Kazakhstan_Eastern
    {  658, 1077,       0,  875792,       0,  0, 18,  0, }, // Kazakhstan_Western
    {  658, 1106,  875810,  875825,  875848, 15, 23, 21, }, // Korea
    {  658, 1112,       0,  875869,       0,  0, 13,  0, }, // Kosrae
    {  658, 1119,  875882,  875898,  875922, 16, 24, 22, }, // Krasnoyarsk
    {  658, 1151,       0,  875944,       0,  0, 15,  0, }, // Kyrgystan
    {  658, 1161,       0,  875959,       0,  0, 10,  0, }, // Lanka
    {  658, 1175,       0,  875969,       0,  0, 18,  0, }, // Line_Islands
    {  658, 1188,  875987,  876010,  876041, 23, 31, 29, }, // Lord_Howe
    {  658, 1198,  876070,  876082,  876102, 12, 20, 18, }, // Macau
    {  658, 1204,  876120,  876132,  876152, 12, 20, 18, }, // Magadan
    {  658, 1219,       0,  876170,       0,  0, 15,  0, }, // Malaysia
    {  658, 1228,       0,  876185,       0,  0, 15,  0, }, // Maldives
    {  658, 1237,       0,  876200,       0,  0, 22,  0, }, // Marquesas
    {  658, 1247,       0,  876222,       0,  0, 21,  0, }, // Marshall_Islands
    {  658, 1264,  876243,  876261,  876287, 18, 26, 24, }, // Mauritius
    {  658, 1274,       0,  876311,       0,  0, 11,  0, }, // Mawson
    {  658, 1281,  876322,  876347,  876380, 25, 33, 32, }, // Mexico_Pacific
    {  658, 1296,  876412,  876428,  876452, 16, 24, 22, }, // Mongolia
    {  658, 1305,  876474,  876485,  876504, 11, 19, 17, }, // Moscow
    {  658, 1312,       0,  876521,       0,  0, 15,  0, }, // Myanmar
    {  658, 1320,       0,  876536,       0,  0, 16,  0, }, // Nauru
    {  658, 1326,       0,  876552,       0,  0, 13,  0, }, // Nepal
    {  658, 1332,  876565,  876585,  876613, 20, 28, 26, }, // New_Caledonia
    {  658, 1346,  876639,  876657,  876683, 18, 26, 24, }, // New_Zealand
    {  658, 1358,  876707,  876726,  876753, 19, 27, 25, }, // Newfoundland
    {  658, 1371,       0,  876778,       0,  0, 11,  0, }, // Niue
    {  658, 1376,  876789,  876809,  876837, 20, 28, 26, }, // Norfolk
    {  658, 1384,  876863,  876887,  876919, 24, 32, 30, }, // Noronha
    {  658, 1392,       0,  876949,       0,  0, 25,  0, }, // North_Mariana
    {  658, 1406,  876974,  876990,  877014, 16, 24, 22, }, // Novosibirsk
    {  658, 1418,  877036,  877045,  877062,  9, 17, 15, }, // Omsk
    {  658, 1428,  877077,  877094,  877119, 17, 25, 23, }, // Pakistan
    {  658, 1437,       0,  877142,       0,  0, 12,  0, }, // Palau
    {  658, 1443,       0,  877154,       0,  0, 25,  0, }, // Papua_New_Guinea
    {  658, 1460,  877179,  877198,  877225, 19, 27, 25, }, // Paraguay
    {  658, 1469,  877250,  877265,  877288, 15, 23, 21, }, // Peru
    {  658, 1474,  877309,  877324,  877347, 15, 23, 21, }, // Philippines
    {  658, 1486,       0,  877368,       0,  0, 20,  0, }, // Phoenix_Islands
    {  658, 1502,  877388,  877417,  877454, 29, 37, 35, }, // Pierre_Miquelon
    {  658, 1518,       0,  877489,       0,  0, 22,  0, }, // Pitcairn
    {  658, 1527,       0,  877511,       0,  0, 12,  0, }, // Ponape
    {  658, 1534,       0,  877523,       0,  0, 14,  0, }, // Pyongyang
    {  658, 1544,  877537,  877551,  877573, 14, 22, 20, }, // Qyzylorda
    {  658, 1554,       0,  877593,       0,  0, 14,  0, }, // Reunion
    {  658, 1562,       0,  877607,       0,  0, 12,  0, }, // Rothera
    {  658, 1570,  877619,  877632,  877653, 13, 21, 19, }, // Sakhalin
    {  658, 1579,  877672,  877683,  877702, 11, 19, 17, }, // Samara
    {  658, 1596,  877719,  877734,  877757, 15, 23, 21, }, // Samoa
    {  658, 1602,       0,  877778,       0,  0, 14,  0, }, // Seychelles
    {  658, 1624,       0,  877792,       0,  0, 24,  0, }, // Singapore
    {  658, 1634,       0,  877816,       0,  0, 21,  0, }, // Solomon
    {  658, 1642,       0,  877837,       0,  0, 18,  0, }, // South_Georgia
    {  658, 1656,       0,  877855,       0,  0, 16,  0, }, // Suriname
    {  658, 1676,       0,  877871,       0,  0, 10,  0, }, // Syowa
    {  658, 1682,       0,  877881,       0,  0, 17,  0, }, // Tahiti
    {  658, 1689,  877898,  877909,  877928, 11, 19, 17, }, // Taipei
    {  658, 1696,       0,  877945,       0,  0, 15,  0, }, // Tajikistan
    {  658, 1724,       0,  877960,       0,  0, 21,  0, }, // Tokelau
    {  658, 1732,  877981,  877996,  878019, 15, 23, 21, }, // Tonga
    {  658, 1738,       0,  878040,       0,  0, 12,  0, }, // Truk
    {  658, 1750,  878052,  878067,  878090, 15, 23, 21, }, // Turkmenistan
    {  658, 1763,       0,  878111,       0,  0, 17,  0, }, // Tuvalu
    {  658, 1777,  878128,  878146,  878172, 18, 26, 24, }, // Uruguay
    {  658, 1792,  878196,  878210,  878232, 14, 22, 20, }, // Uzbekistan
    {  658, 1803,  878252,  878270,  878296, 18, 26, 24, }, // Vanuatu
    {  658, 1811,       0,  878320,       0,  0, 18,  0, }, // Venezuela
    {  658, 1821,  878338,  878354,  878378, 16, 24, 22, }, // Vladivostok
    {  658, 1833,  878400,  878414,  878436, 14, 22, 20, }, // Volgograd
    {  658, 1843,       0,  878456,       0,  0, 11,  0, }, // Vostok
    {  658, 1850,       0,  878467,       0,  0, 18,  0, }, // Wake
    {  658, 1855,       0,  878485,       0,  0, 23,  0, }, // Wallis
    {  658, 1862,  878508,  878521,  878542, 13, 21, 19, }, // Yakutsk
    {  658, 1870,  878561,  878581,  878609, 20, 28, 26, }, // Yekaterinburg
    {  658, 1892,       0,  878635,       0,  0, 10,  0, }, // Yukon
    {  660,    5,       0,  878645,       0,  0, 16,  0, }, // Afghanistan Wolof/Latin/Senegal
    {  660,   17,       0,  878661,       0,  0, 22,  0, }, // Africa_Central
    {  660,   32,       0,  878683,       0,  0, 24,  0, }, // Africa_Eastern
    {  660,   65,       0,  264554,       0,  0, 14,  0, }, // Africa_Southern
    {  660,   81,  878707,  878726,  878757, 19, 31, 21, }, // Africa_Western
    {  660,  107,  878778,  878790,  878814, 12, 24, 22, }, // Alaska
    {  660,  135,  878836,  878848,  878872, 12, 24, 19, }, // Amazon
    {  660,  142,  878891,  878910,  878942, 19, 32, 28, }, // America_Central
    {  660,  158,  878970,  878984,  879012, 14, 28, 25, }, // America_Eastern
    {  660,  174,  879037,  879053,  879081, 16, 28, 25, }, // America_Mountain
    {  660,  191,  879106,  879124,  879154, 18, 30, 27, }, // America_Pacific
    {  660,  214,  879181,  879191,  879213, 10, 22, 17, }, // Apia
    {  660,  232,  879230,  879244,  879267, 14, 23, 21, }, // Arabian
    {  660,  240,  879288,  879303,  879330, 15, 27, 22, }, // Argentina
    {  660,  250,  879352,  879373,  879406, 21, 33, 28, }, // Argentina_Western
    {  660,  268,  879434,  879446,  879470, 12, 24, 19, }, // Armenia
    {  660,  286,  879489,  879508,  879536, 19, 28, 28, }, // Atlantic
    {  660,  295,  879564,  879591,  879618, 27, 27, 31, }, // Australia_Central
    {  660,  313,  879649,  879670,  879709, 21, 39, 37, }, // Australia_CentralWestern
    {  660,  338,  879746,  879767,  879791, 21, 24, 33, }, // Australia_Eastern
    {  660,  356,  879824,  879852,  879885, 28, 33, 31, }, // Australia_Western
    {  660,  374,  879916,  879933,  879950, 17, 17, 24, }, // Azerbaijan
    {  660,  385,  879974,  879986,  880010, 12, 24, 16, }, // Azores
    {  660,  397,  880026,  880042,  880070, 16, 28, 23, }, // Bangladesh
    {  660,  415,       0,  880093,       0,  0, 13,  0, }, // Bhutan
    {  660,  422,       0,  880106,       0,  0, 13,  0, }, // Bolivia
    {  660,  437,  880119,  880131,  880144, 12, 13, 20, }, // Brasilia
    {  660,  454,       0,   66139,       0,  0, 17,  0, }, // Brunei
    {  660,  461,    1679,  880164,  880180, 10, 16, 25, }, // Cape_Verde
    {  660,  478,       0,    1750,       0,  0, 22,  0, }, // Chamorro
    {  660,  487,  880205,    1784,    1805, 13, 21, 21, }, // Chatham
    {  660,  495,  880218,  880228,  880250, 10, 22, 17, }, // Chile
    {  660,  501,  880267,  880278,  880301, 11, 23, 18, }, // China
    {  660,  507,       0,  880319,       0,  0, 17,  0, }, // Christmas
    {  660,  517,       0,  880336,       0,  0, 15,  0, }, // Cocos
    {  660,  523,  880351,  880364,  880389, 13, 25, 22, }, // Colombia
    {  660,  532,  880411,  880425,  880451, 14, 26, 18, }, // Cook
    {  660,  537,  880469,  880479,  880501, 10, 22, 17, }, // Cuba
    {  660,  548,       0,  880518,       0,  0, 11,  0, }, // Davis
    {  660,  564,       0,   14996,       0,  0, 16,  0, }, // DumontDUrville
    {  660,  601,       0,  259419,       0,  0, 14,  0, }, // East_Timor
    {  660,  612,  880529,  880548,  880579, 19, 31, 28, }, // Easter
    {  660,  619,       0,  880607,       0,  0, 13,  0, }, // Ecuador
    {  660,  627,  880620,  880647,  880686, 27, 39, 35, }, // Europe_Central
    {  660,  642,  880721,  880746,  880783, 25, 37, 33, }, // Europe_Eastern
    {  660,  657,       0,  880816,       0,  0, 18,  0, }, // Europe_Further_Eastern
    {  660,  680,  880834,  880863,  880905, 29, 42, 38, }, // Europe_Western
    {  660,  695,  880943,  880956,  880979, 13, 23, 27, }, // Falkland
    {  660,  704,  881006,   15391,  881017, 11,  5, 19, }, // Fiji
    {  660,  709,       0,  259985,       0,  0, 16,  0, }, // French_Guiana
    {  660,  723,       0,  881036,       0,  0, 36,  0, }, // French_Southern
    {  660,  746,       0,  881072,       0,  0, 15,  0, }, // Galapagos
    {  660,  756,       0,  881087,       0,  0, 13,  0, }, // Gambier
    {  660,  764,  881100,  881113,  881126, 13, 13, 17, }, // Georgia
    {  660,  772,       0,  881143,       0,  0, 17,  0, }, // Gilbert_Islands
    {  660,  788,       0,  881160,       0,  0, 21,  0, }, // GMT
    {  660,  830,  881181,  881202,  881235, 21, 33, 28, }, // Greenland_Eastern
    {  660,  848,  881263,  881284,  881317, 21, 33, 28, }, // Greenland_Western
    {  660,  871,       0,  881345,       0,  0, 10,  0, }, // Gulf
    {  660,  876,       0,  881355,       0,  0, 12,  0, }, // Guyana
    {  660,  883,  881367,  881388,  881421, 21, 33, 31, }, // Hawaii_Aleutian
    {  660,  899,  881452,  881467,  881494, 15, 27, 24, }, // Hong_Kong
    {  660,  909,  881518,  881528,  881547, 10, 19, 14, }, // Hovd
    {  660,  914,       0,  881561,       0,  0, 10,  0, }, // India
    {  660,  920,       0,  881571,       0,  0, 16,  0, }, // Indian_Ocean
    {  660,  933,       0,  881587,       0,  0, 15,  0, }, // Indochina
    {  660,  943,       0,  881602,       0,  0, 27,  0, }, // Indonesia_Central
    {  660,  961,       0,  881629,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  660,  979,       0,  881650,       0,  0, 21,  0, }, // Indonesia_Western
    {  660,  997,  881671,  881681,  881700, 10, 19, 20, }, // Iran
    {  660, 1008,  881720,  881732,  881750, 12, 18, 20, }, // Irkutsk
    {  660, 1016,  881770,  881782,  881806, 12, 24, 19, }, // Israel
    {  660, 1023,   28193,  881825,  881836,  5, 11, 18, }, // Japan
    {  660, 1047,       0,  881854,       0,  0, 17,  0, }, // Kazakhstan
    {  660, 1058,       0,  881871,       0,  0, 16,  0, }, // Kazakhstan_Eastern
    {  660, 1077,       0,  881887,       0,  0, 24,  0, }, // Kazakhstan_Western
    {  660, 1106,  881911,  881921,  881943, 10, 22, 20, }, // Korea
    {  660, 1112,       0,  881963,       0,  0, 12,  0, }, // Kosrae
    {  660, 1119,  881975,  881992,  882009, 17, 17, 26, }, // Krasnoyarsk
    {  660, 1151,       0,  882035,       0,  0, 15,  0, }, // Kyrgystan
    {  660, 1175,       0,  882050,       0,  0, 14,  0, }, // Line_Islands
    {  660, 1188,    3833,  882064,    3871, 14, 24, 22, }, // Lord_Howe
    {  660, 1204,  882088,  882102,  882116, 14, 14, 20, }, // Magadan
    {  660, 1219,       0,  882136,       0,  0,  8,  0, }, // Malaysia
    {  660, 1228,       0,  882144,       0,  0, 14,  0, }, // Maldives
    {  660, 1237,       0,  882158,       0,  0, 15,  0, }, // Marquesas
    {  660, 1247,       0,  882173,       0,  0, 18,  0, }, // Marshall_Islands
    {  660, 1264,  882191,  882204,  882229, 13, 25, 20, }, // Mauritius
    {  660, 1274,       0,  882249,       0,  0, 12,  0, }, // Mawson
    {  660, 1281,  882261,  882283,  882318, 22, 35, 31, }, // Mexico_Pacific
    {  660, 1296,  882349,  882361,  882371, 12, 10, 17, }, // Mongolia
    {  660, 1305,  882388,  882400,  882412, 12, 12, 19, }, // Moscow
    {  660, 1312,       0,  882431,       0,  0, 13,  0, }, // Myanmar
    {  660, 1320,       0,  882444,       0,  0, 11,  0, }, // Nauru
    {  660, 1326,       0,  882455,       0,  0, 11,  0, }, // Nepal
    {  660, 1332,  882466,  882485,  882520, 19, 35, 31, }, // New_Caledonia
    {  660, 1346,  882551,  882573,  262165, 22, 34, 16, }, // New_Zealand
    {  660, 1358,  882607,  882624,  262260, 17, 11, 11, }, // Newfoundland
    {  660, 1371,       0,  882635,       0,  0, 10,  0, }, // Niue
    {  660, 1376,  882645,  882662,  882691, 17, 29, 27, }, // Norfolk
    {  660, 1384,  882718,  882737,  882756, 19, 19, 31, }, // Noronha
    {  660, 1406,  882787,  882804,  882821, 17, 17, 26, }, // Novosibirsk
    {  660, 1418,  882847,  882857,  882879, 10, 22, 14, }, // Omsk
    {  660, 1428,  882893,  882907,  882933, 14, 26, 21, }, // Pakistan
    {  660, 1437,       0,  882954,       0,  0, 11,  0, }, // Palau
    {  660, 1443,       0,  262771,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  660, 1460,  882965,  882979,  882993, 14, 14, 18, }, // Paraguay
    {  660, 1469,  883011,  883021,  883040, 10, 19, 14, }, // Peru
    {  660, 1474,  883054,  883070,  883096, 16, 26, 26, }, // Philippines
    {  660, 1486,       0,  883122,       0,  0, 17,  0, }, // Phoenix_Islands
    {  660, 1502,  883139,  249375,  883163, 24, 23, 25, }, // Pierre_Miquelon
    {  660, 1518,       0,  883188,       0,  0, 14,  0, }, // Pitcairn
    {  660, 1527,       0,  883202,       0,  0, 12,  0, }, // Ponape
    {  660, 1534,       0,  883214,       0,  0, 15,  0, }, // Pyongyang
    {  660, 1554,       0,  883229,       0,  0, 11,  0, }, // Reunion
    {  660, 1562,       0,  883240,       0,  0, 12,  0, }, // Rothera
    {  660, 1570,  883252,   69492,  883265, 13,  7, 17, }, // Sakhalin
    {  660, 1596,  883282,     611,  883293, 11, 19, 18, }, // Samoa
    {  660, 1602,       0,  883311,       0,  0, 16,  0, }, // Seychelles
    {  660, 1624,       0,  883327,       0,  0, 27,  0, }, // Singapore
    {  660, 1634,       0,  883354,       0,  0, 17,  0, }, // Solomon
    {  660, 1642,       0,  883371,       0,  0, 14,  0, }, // South_Georgia
    {  660, 1656,       0,  883385,       0,  0, 13,  0, }, // Suriname
    {  660, 1676,       0,  883398,       0,  0, 11,  0, }, // Syowa
    {  660, 1682,       0,  883409,       0,  0, 12,  0, }, // Tahiti
    {  660, 1689,  883421,  883433,  883457, 12, 24, 24, }, // Taipei
    {  660, 1696,       0,  883481,       0,  0, 17,  0, }, // Tajikistan
    {  660, 1724,       0,  883498,       0,  0, 12,  0, }, // Tokelau
    {  660, 1732,  883510,  883521,  883541, 11, 20, 20, }, // Tonga
    {  660, 1738,       0,  883561,       0,  0, 11,  0, }, // Truk
    {  660, 1750,  883572,  878726,  883590, 18, 14, 25, }, // Turkmenistan
    {  660, 1763,       0,  883615,       0,  0, 12,  0, }, // Tuvalu
    {  660, 1777,  883627,  883640,  883662, 13, 22, 17, }, // Uruguay
    {  660, 1792,  883679,  883696,  883725, 17, 29, 24, }, // Uzbekistan
    {  660, 1803,  883749,  883762,  883772, 13, 10, 20, }, // Vanuatu
    {  660, 1811,       0,  883792,       0,  0, 15,  0, }, // Venezuela
    {  660, 1821,  883807,  883824,  883844, 17, 20, 26, }, // Vladivostok
    {  660, 1833,  883870,  883885,  883909, 15, 24, 24, }, // Volgograd
    {  660, 1843,       0,  883933,       0,  0, 12,  0, }, // Vostok
    {  660, 1850,       0,  883945,       0,  0, 14,  0, }, // Wake
    {  660, 1855,       0,    6170,       0,  0, 20,  0, }, // Wallis
    {  660, 1862,  883959,  883972,  883985, 13, 13, 20, }, // Yakutsk
    {  660, 1870,  884005,  884024,  880389, 19, 14, 11, }, // Yekaterinburg
    {  660, 1892,       0,  884038,       0,  0, 11,  0, }, // Yukon
    {  661,    5,       0,      43,       0,  0, 16,  0, }, // Afghanistan Xhosa/Latin/South Africa
    {  661,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central
    {  661,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  661,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  661,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  661,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  661,  135,     284,     295,     315, 11, 20, 18, }, // Amazon
    {  661,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  661,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  661,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  661,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  661,  214,  884049,  884058,  884076,  9, 18, 18, }, // Apia
    {  661,  232,     744,     756,     777, 12, 21, 21, }, // Arabian
    {  661,  240,     798,     812,     835, 14, 23, 21, }, // Argentina
    {  661,  250,     856,     878,     909, 22, 31, 29, }, // Argentina_Western
    {  661,  268,     938,     950,     971, 12, 21, 19, }, // Armenia
    {  661,  286,     990,    1003,    1025, 13, 22, 22, }, // Atlantic
    {  661,  295,  884094,    1070,    1102, 22, 32, 32, }, // Australia_Central
    {  661,  313,    1134,    1165,    1205, 31, 40, 40, }, // Australia_CentralWestern
    {  661,  338,  884116,    1268,    1300, 22, 32, 32, }, // Australia_Eastern
    {  661,  356,  884138,    1355,    1387, 22, 32, 32, }, // Australia_Western
    {  661,  374,    1419,    1434,    1458, 15, 24, 22, }, // Azerbaijan
    {  661,  385,    1480,    1491,    1511, 11, 20, 18, }, // Azores
    {  661,  397,    1529,    1544,    1568, 15, 24, 22, }, // Bangladesh
    {  661,  415,       0,    1590,       0,  0, 11,  0, }, // Bhutan
    {  661,  422,       0,    1601,       0,  0, 12,  0, }, // Bolivia
    {  661,  437,    1613,    1626,    1648, 13, 22, 20, }, // Brasilia
    {  661,  454,       0,  884160,       0,  0, 22,  0, }, // Brunei
    {  661,  461,    1679,    1694,    1718, 15, 24, 22, }, // Cape_Verde
    {  661,  478,       0,    1750,       0,  0, 22,  0, }, // Chamorro
    {  661,  487,    1772,    1784,    1805, 12, 21, 21, }, // Chatham
    {  661,  495,    1826,    1836,    1855, 10, 19, 17, }, // Chile
    {  661,  501,    1872,    1882,    1901, 10, 19, 19, }, // China
    {  661,  507,       0,    1920,       0,  0, 21,  0, }, // Christmas
    {  661,  517,       0,    1941,       0,  0, 18,  0, }, // Cocos
    {  661,  523,    1959,    1972,    1994, 13, 22, 20, }, // Colombia
    {  661,  532,    2014,    2031,  884182, 17, 26, 29, }, // Cook
    {  661,  537,    2081,    2090,    2108,  9, 18, 18, }, // Cuba
    {  661,  548,       0,    2126,       0,  0, 10,  0, }, // Davis
    {  661,  564,       0,  884211,       0,  0, 21,  0, }, // DumontDUrville
    {  661,  601,       0,  884232,       0,  0, 15,  0, }, // East_Timor
    {  661,  612,    2173,    2191,    2218, 18, 27, 25, }, // Easter
    {  661,  619,       0,    2243,       0,  0, 12,  0, }, // Ecuador
    {  661,  627,    2255,    2276,    2306, 21, 30, 28, }, // Europe_Central
    {  661,  642,    2334,    2355,    2385, 21, 30, 28, }, // Europe_Eastern
    {  661,  657,       0,    2413,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {  661,  680,    2442,    2463,    2493, 21, 30, 28, }, // Europe_Western
    {  661,  695,    2521,    2542,    2572, 21, 30, 28, }, // Falkland
    {  661,  704,    2600,    2609,    2627,  9, 18, 16, }, // Fiji
    {  661,  709,       0,    2643,       0,  0, 18,  0, }, // French_Guiana
    {  661,  723,       0,    2661,       0,  0, 32,  0, }, // French_Southern
    {  661,  746,       0,    2693,       0,  0, 14,  0, }, // Galapagos
    {  661,  756,       0,    2707,       0,  0, 12,  0, }, // Gambier
    {  661,  764,    2719,    2731,    2752, 12, 21, 19, }, // Georgia
    {  661,  772,       0,    2771,       0,  0, 20,  0, }, // Gilbert_Islands
    {  661,  788,       0,    2791,       0,  0, 19,  0, }, // GMT
    {  661,  830,    2868,    2887,    2915, 19, 28, 26, }, // Greenland_Eastern
    {  661,  848,    2941,    2960,    2988, 19, 28, 26, }, // Greenland_Western
    {  661,  871,       0,    3032,       0,  0, 18,  0, }, // Gulf
    {  661,  876,       0,    3050,       0,  0, 11,  0, }, // Guyana
    {  661,  883,    3061,    3081,    3110, 20, 29, 29, }, // Hawaii_Aleutian
    {  661,  899,    3139,    3153,    3176, 14, 23, 21, }, // Hong_Kong
    {  661,  909,    3197,    3206,    3224,  9, 18, 16, }, // Hovd
    {  661,  914,       0,    3240,       0,  0, 19,  0, }, // India
    {  661,  920,       0,    3259,       0,  0, 17,  0, }, // Indian_Ocean
    {  661,  933,       0,    3276,       0,  0, 14,  0, }, // Indochina
    {  661,  943,       0,    3290,       0,  0, 22,  0, }, // Indonesia_Central
    {  661,  961,       0,    3312,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  661,  979,       0,    3334,       0,  0, 22,  0, }, // Indonesia_Western
    {  661,  997,    3356,    3365,    3383,  9, 18, 18, }, // Iran
    {  661, 1008,    3401,    3413,    3434, 12, 21, 19, }, // Irkutsk
    {  661, 1016,    3453,    3464,    3484, 11, 20, 20, }, // Israel
    {  661, 1023,    3504,    3514,    3533, 10, 19, 19, }, // Japan
    {  661, 1047,       0,    3610,       0,  0, 15,  0, }, // Kazakhstan
    {  661, 1058,       0,    3625,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  661, 1077,       0,    3645,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  661, 1106,    3665,    3676,    3696, 11, 20, 20, }, // Korea
    {  661, 1112,       0,    3716,       0,  0, 11,  0, }, // Kosrae
    {  661, 1119,    3727,    3743,    3768, 16, 25, 23, }, // Krasnoyarsk
    {  661, 1151,       0,    3791,       0,  0, 15,  0, }, // Kyrgystan
    {  661, 1175,       0,    3816,       0,  0, 17,  0, }, // Line_Islands
    {  661, 1188,    3833,    3847,    3870, 14, 23, 23, }, // Lord_Howe
    {  661, 1204,    3939,    3951,    3972, 12, 21, 19, }, // Magadan
    {  661, 1219,       0,    3991,       0,  0, 13,  0, }, // Malaysia
    {  661, 1228,       0,    4004,       0,  0, 13,  0, }, // Maldives
    {  661, 1237,       0,    4017,       0,  0, 14,  0, }, // Marquesas
    {  661, 1247,       0,    4031,       0,  0, 21,  0, }, // Marshall_Islands
    {  661, 1264,    4052,    4066,    4089, 14, 23, 21, }, // Mauritius
    {  661, 1274,       0,    4110,       0,  0, 11,  0, }, // Mawson
    {  661, 1281,    4121,    4141,    4170, 20, 29, 29, }, // Mexico_Pacific
    {  661, 1296,    4199,    4215,    4240, 16, 25, 23, }, // Mongolia
    {  661, 1305,    4263,    4274,    4294, 11, 20, 18, }, // Moscow
    {  661, 1312,       0,    4312,       0,  0, 12,  0, }, // Myanmar
    {  661, 1320,       0,    4324,       0,  0, 10,  0, }, // Nauru
    {  661, 1326,       0,    4334,       0,  0, 10,  0, }, // Nepal
    {  661, 1332,    4344,    4362,    4389, 18, 27, 25, }, // New_Caledonia
    {  661, 1346,    4414,    4430,    4455, 16, 25, 25, }, // New_Zealand
    {  661, 1358,    4480,    4497,    4523, 17, 26, 26, }, // Newfoundland
    {  661, 1371,       0,    4549,       0,  0,  9,  0, }, // Niue
    {  661, 1376,    4558,    4577,    4605, 19, 28, 28, }, // Norfolk
    {  661, 1384,    4633,    4657,    4690, 24, 33, 31, }, // Noronha
    {  661, 1406,    4750,    4766,    4791, 16, 25, 23, }, // Novosibirsk
    {  661, 1418,    4814,    4823,    4841,  9, 18, 16, }, // Omsk
    {  661, 1428,    4857,    4870,    4892, 13, 22, 20, }, // Pakistan
    {  661, 1437,       0,    4912,       0,  0, 10,  0, }, // Palau
    {  661, 1443,       0,    4922,       0,  0, 21,  0, }, // Papua_New_Guinea
    {  661, 1460,    4943,    4956,    4978, 13, 22, 20, }, // Paraguay
    {  661, 1469,    4998,    5007,    5025,  9, 18, 16, }, // Peru
    {  661, 1474,    5041,    5056,    5080, 15, 24, 22, }, // Philippines
    {  661, 1486,       0,    5102,       0,  0, 20,  0, }, // Phoenix_Islands
    {  661, 1502,    5122,    5148,    5183, 26, 35, 35, }, // Pierre_Miquelon
    {  661, 1518,       0,    5218,       0,  0, 13,  0, }, // Pitcairn
    {  661, 1527,       0,  765394,       0,  0, 11,  0, }, // Ponape
    {  661, 1534,       0,  884247,       0,  0, 14,  0, }, // Pyongyang
    {  661, 1554,       0,    5317,       0,  0, 12,  0, }, // Reunion
    {  661, 1562,       0,    5329,       0,  0, 12,  0, }, // Rothera
    {  661, 1570,    5341,    5354,    5376, 13, 22, 20, }, // Sakhalin
    {  661, 1596,     601,     611,     630, 10, 19, 19, }, // Samoa
    {  661, 1602,       0,    5520,       0,  0, 15,  0, }, // Seychelles
    {  661, 1624,       0,    5535,       0,  0, 23,  0, }, // Singapore
    {  661, 1634,       0,    5558,       0,  0, 20,  0, }, // Solomon
    {  661, 1642,       0,    5578,       0,  0, 18,  0, }, // South_Georgia
    {  661, 1656,       0,    5596,       0,  0, 13,  0, }, // Suriname
    {  661, 1676,       0,    5609,       0,  0, 10,  0, }, // Syowa
    {  661, 1682,       0,    5619,       0,  0, 11,  0, }, // Tahiti
    {  661, 1689,  884261,  884272,  884292, 11, 20, 20, }, // Taipei
    {  661, 1696,       0,    5681,       0,  0, 15,  0, }, // Tajikistan
    {  661, 1724,       0,    5696,       0,  0, 12,  0, }, // Tokelau
    {  661, 1732,    5708,    5718,    5737, 10, 19, 17, }, // Tonga
    {  661, 1738,       0,    5754,       0,  0, 10,  0, }, // Truk
    {  661, 1750,    5764,    5781,    5807, 17, 26, 24, }, // Turkmenistan
    {  661, 1763,       0,    5831,       0,  0, 11,  0, }, // Tuvalu
    {  661, 1777,    5842,    5854,    5875, 12, 21, 19, }, // Uruguay
    {  661, 1792,    5894,    5909,    5933, 15, 24, 22, }, // Uzbekistan
    {  661, 1803,    5955,    5967,    5988, 12, 21, 19, }, // Vanuatu
    {  661, 1811,       0,    6007,       0,  0, 14,  0, }, // Venezuela
    {  661, 1821,    6021,    6037,    6062, 16, 25, 23, }, // Vladivostok
    {  661, 1833,    6085,    6099,    6122, 14, 23, 21, }, // Volgograd
    {  661, 1843,       0,    6143,       0,  0, 11,  0, }, // Vostok
    {  661, 1850,       0,    6154,       0,  0, 16,  0, }, // Wake
    {  661, 1855,       0,    6170,       0,  0, 20,  0, }, // Wallis
    {  661, 1862,    6190,    6202,    6223, 12, 21, 19, }, // Yakutsk
    {  661, 1870,    6242,    6260,    6287, 18, 27, 25, }, // Yekaterinburg
    {  661, 1892,       0,    6312,       0,  0, 10,  0, }, // Yukon
    {  664,    5,       0,  884312,       0,  0, 17,  0, }, // Afghanistan Yoruba/Latin/Nigeria
    {  664,   17,       0,  884329,       0,  0, 19,  0, }, // Africa_Central
    {  664,   32,       0,  884348,       0,  0, 23,  0, }, // Africa_Eastern
    {  664,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  664,   81,  884371,  884395,  884427, 24, 32, 29, }, // Africa_Western
    {  664,  107,  884456,  884469,  884490, 13, 21, 20, }, // Alaska
    {  664,  135,  884510,  884524,  884546, 14, 22, 18, }, // Amazon
    {  664,  142,  884564,  884584,  884610, 20, 26, 28, }, // America_Central
    {  664,  158,  884638,  884657,  884682, 19, 25, 27, }, // America_Eastern
    {  664,  174,  884709,  884718,  884733,  9, 15, 21, }, // America_Mountain
    {  664,  191,  884754,  884769,  884790, 15, 21, 23, }, // America_Pacific
    {  664,  214,  884813,  884823,  884841, 10, 18, 18, }, // Apia
    {  664,  232,  884859,  884871,  884891, 12, 20, 20, }, // Arabian
    {  664,  240,  884911,  884924,  884944, 13, 20, 19, }, // Argentina
    {  664,  250,  884963,  884989,  885023, 26, 34, 30, }, // Argentina_Western
    {  664,  268,  885053,  885066,  885087, 13, 21, 17, }, // Armenia
    {  664,  286,  885104,  885121,  885144, 17, 23, 25, }, // Atlantic
    {  664,  295,  885169,  885197,  885233, 28, 36, 36, }, // Australia_Central
    {  664,  313,  885269,  885308,  885355, 39, 47, 47, }, // Australia_CentralWestern
    {  664,  338,  885402,  885427,  885460, 25, 33, 33, }, // Australia_Eastern
    {  664,  356,  885493,  885519,  885553, 26, 34, 34, }, // Australia_Western
    {  664,  374,  885587,  885603,  885627, 16, 24, 21, }, // Azerbaijan
    {  664,  385,  885648,  885660,  885680, 12, 20, 17, }, // Azores
    {  664,  397,  885697,  885713,  885737, 16, 24, 21, }, // Bangladesh
    {  664,  415,       0,  885758,       0,  0, 12,  0, }, // Bhutan
    {  664,  422,       0,  885770,       0,  0, 12,  0, }, // Bolivia
    {  664,  437,  885782,  885796,  885816, 14, 20, 18, }, // Brasilia
    {  664,  454,       0,  884160,       0,  0, 22,  0, }, // Brunei
    {  664,  461,  885834,  885850,  885874, 16, 24, 22, }, // Cape_Verde
    {  664,  478,       0,  885896,       0,  0, 22,  0, }, // Chamorro
    {  664,  487,  885918,  885930,  885950, 12, 20, 20, }, // Chatham
    {  664,  495,  885970,  885980,  885998, 10, 18, 14, }, // Chile
    {  664,  501,  886012,  886023,  886044, 11, 21, 19, }, // China
    {  664,  507,       0,  886063,       0,  0, 23,  0, }, // Christmas
    {  664,  517,       0,  886086,       0,  0, 24,  0, }, // Cocos
    {  664,  523,  886110,  886123,  886142, 13, 19, 18, }, // Colombia
    {  664,  532,  886160,  886184,  886216, 24, 32, 35, }, // Cook
    {  664,  537,  886251,  886261,  886279, 10, 18, 18, }, // Cuba
    {  664,  548,       0,  886297,       0,  0, 11,  0, }, // Davis
    {  664,  564,       0,  886308,       0,  0, 22,  0, }, // DumontDUrville
    {  664,  601,       0,  886330,       0,  0, 21,  0, }, // East_Timor
    {  664,  612,  886351,  886372,  886396, 21, 24, 23, }, // Easter
    {  664,  619,       0,  886419,       0,  0, 12,  0, }, // Ecuador
    {  664,  627,  886431,  886449,  886473, 18, 24, 23, }, // Europe_Central
    {  664,  642,  886496,  886522,  886554, 26, 32, 31, }, // Europe_Eastern
    {  664,  657,       0,  886585,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {  664,  680,  886614,  886636,  886664, 22, 28, 31, }, // Europe_Western
    {  664,  695,  886695,  886710,  886740, 15, 30, 27, }, // Falkland
    {  664,  704,  886767,  886777,  886795, 10, 18, 15, }, // Fiji
    {  664,  709,       0,  886810,       0,  0, 20,  0, }, // French_Guiana
    {  664,  723,       0,  886830,       0,  0, 32,  0, }, // French_Southern
    {  664,  746,       0,  886862,       0,  0, 13,  0, }, // Galapagos
    {  664,  756,       0,  886875,       0,  0, 12,  0, }, // Gambier
    {  664,  764,  886887,  886900,  886921, 13, 21, 18, }, // Georgia
    {  664,  772,       0,  886939,       0,  0, 27,  0, }, // Gilbert_Islands
    {  664,  788,       0,    2791,       0,  0, 19,  0, }, // GMT
    {  664,  830,  886966,  886991,  887025, 25, 34, 25, }, // Greenland_Eastern
    {  664,  848,  887050,  887076,  887110, 26, 34, 33, }, // Greenland_Western
    {  664,  871,       0,  887143,       0,  0, 18,  0, }, // Gulf
    {  664,  876,       0,  887161,       0,  0, 12,  0, }, // Guyana
    {  664,  883,  887173,  887194,  887223, 21, 29, 29, }, // Hawaii_Aleutian
    {  664,  899,  887252,  887267,  887292, 15, 25, 20, }, // Hong_Kong
    {  664,  909,  887312,  887324,  887346, 12, 22, 17, }, // Hovd
    {  664,  914,       0,  887363,       0,  0, 19,  0, }, // India
    {  664,  920,       0,  887382,       0,  0, 18,  0, }, // Indian_Ocean
    {  664,  933,       0,  887400,       0,  0, 15,  0, }, // Indochina
    {  664,  943,       0,  887415,       0,  0, 30,  0, }, // Indonesia_Central
    {  664,  961,       0,  887445,       0,  0, 25,  0, }, // Indonesia_Eastern
    {  664,  979,       0,  887470,       0,  0, 26,  0, }, // Indonesia_Western
    {  664,  997,  887496,  887507,  887526, 11, 19, 19, }, // Iran
    {  664, 1008,  887545,  887558,  887579, 13, 21, 18, }, // Irkutsk
    {  664, 1016,  887597,  887609,  887629, 12, 20, 20, }, // Israel
    {  664, 1023,  887649,  887660,  887681, 11, 21, 16, }, // Japan
    {  664, 1047,       0,  887697,       0,  0, 16,  0, }, // Kazakhstan
    {  664, 1058,       0,  887713,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  664, 1077,       0,  887740,       0,  0, 28,  0, }, // Kazakhstan_Western
    {  664, 1106,  887768,  887779,  887800, 11, 21, 19, }, // Korea
    {  664, 1112,       0,  887819,       0,  0, 13,  0, }, // Kosrae
    {  664, 1119,  887832,  887851,  887878, 19, 27, 24, }, // Krasnoyarsk
    {  664, 1151,       0,  887902,       0,  0, 19,  0, }, // Kyrgystan
    {  664, 1175,       0,  887921,       0,  0, 24,  0, }, // Line_Islands
    {  664, 1188,  887945,  887960,  887983, 15, 23, 23, }, // Lord_Howe
    {  664, 1204,  888006,  888020,  888042, 14, 22, 19, }, // Magadan
    {  664, 1219,       0,  888061,       0,  0, 14,  0, }, // Malaysia
    {  664, 1228,       0,  888075,       0,  0, 14,  0, }, // Maldives
    {  664, 1237,       0,  888089,       0,  0, 15,  0, }, // Marquesas
    {  664, 1247,       0,  888104,       0,  0, 26,  0, }, // Marshall_Islands
    {  664, 1264,  888130,  888144,  888166, 14, 22, 19, }, // Mauritius
    {  664, 1274,       0,  888185,       0,  0, 12,  0, }, // Mawson
    {  664, 1281,  888197,  888220,  888251, 23, 31, 31, }, // Mexico_Pacific
    {  664, 1296,  888282,  888299,  888326, 17, 27, 22, }, // Mongolia
    {  664, 1305,  888348,  888360,  888380, 12, 20, 17, }, // Moscow
    {  664, 1312,       0,  888397,       0,  0, 17,  0, }, // Myanmar
    {  664, 1320,       0,  888414,       0,  0, 11,  0, }, // Nauru
    {  664, 1326,       0,  888425,       0,  0, 12,  0, }, // Nepal
    {  664, 1332,  888437,  888459,  888489, 22, 30, 27, }, // New_Caledonia
    {  664, 1346,  888516,  888533,  888558, 17, 25, 25, }, // New_Zealand
    {  664, 1358,  888583,  888601,  888627, 18, 26, 26, }, // Newfoundland
    {  664, 1371,       0,  888653,       0,  0, 10,  0, }, // Niue
    {  664, 1376,  888663,  888684,  888713, 21, 29, 29, }, // Norfolk
    {  664, 1384,  888742,  888764,  888792, 22, 28, 29, }, // Noronha
    {  664, 1406,  888821,  888840,  888866, 19, 26, 23, }, // Novosibirsk
    {  664, 1418,  888889,  888902,  888923, 13, 21, 18, }, // Omsk
    {  664, 1428,  888941,  888956,  888979, 15, 23, 20, }, // Pakistan
    {  664, 1437,       0,  888999,       0,  0, 11,  0, }, // Palau
    {  664, 1443,       0,  889010,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  664, 1460,  889032,  889046,  889068, 14, 22, 19, }, // Paraguay
    {  664, 1469,  889087,  889097,  889115, 10, 18, 15, }, // Peru
    {  664, 1474,  889130,  889147,  889172, 17, 25, 22, }, // Philippines
    {  664, 1486,       0,  889194,       0,  0, 26,  0, }, // Phoenix_Islands
    {  664, 1502,  889220,  889243,  889274, 23, 31, 31, }, // Pierre_Miquelon
    {  664, 1518,       0,  889305,       0,  0, 14,  0, }, // Pitcairn
    {  664, 1527,       0,  889319,       0,  0, 12,  0, }, // Ponape
    {  664, 1534,       0,  889331,       0,  0, 16,  0, }, // Pyongyang
    {  664, 1554,       0,  889347,       0,  0, 17,  0, }, // Reunion
    {  664, 1562,       0,  889364,       0,  0, 13,  0, }, // Rothera
    {  664, 1570,  889377,  889391,  889413, 14, 22, 19, }, // Sakhalin
    {  664, 1596,  889432,  889443,  889462, 11, 19, 19, }, // Samoa
    {  664, 1602,       0,  889481,       0,  0, 15,  0, }, // Seychelles
    {  664, 1624,       0,  889496,       0,  0, 23,  0, }, // Singapore
    {  664, 1634,       0,  889519,       0,  0, 26,  0, }, // Solomon
    {  664, 1642,       0,  889545,       0,  0, 17,  0, }, // South_Georgia
    {  664, 1656,       0,  889562,       0,  0, 14,  0, }, // Suriname
    {  664, 1676,       0,  889576,       0,  0, 11,  0, }, // Syowa
    {  664, 1682,       0,  889587,       0,  0, 12,  0, }, // Tahiti
    {  664, 1689,  889599,  889611,  889633, 12, 22, 20, }, // Taipei
    {  664, 1696,       0,  889653,       0,  0, 19,  0, }, // Tajikistan
    {  664, 1724,       0,  889672,       0,  0, 13,  0, }, // Tokelau
    {  664, 1732,  889685,  889696,  889715, 11, 19, 16, }, // Tonga
    {  664, 1738,       0,  889731,       0,  0, 11,  0, }, // Truk
    {  664, 1750,  889742,  889762,  889790, 20, 28, 24, }, // Turkmenistan
    {  664, 1763,       0,  889814,       0,  0, 12,  0, }, // Tuvalu
    {  664, 1777,  889826,  889838,  889859, 12, 21, 17, }, // Uruguay
    {  664, 1792,  889876,  889894,  889920, 18, 26, 23, }, // Uzbekistan
    {  664, 1803,  889943,  889956,  889977, 13, 21, 18, }, // Vanuatu
    {  664, 1811,       0,  889995,       0,  0, 14,  0, }, // Venezuela
    {  664, 1821,  890009,  890029,  890057, 20, 28, 25, }, // Vladivostok
    {  664, 1833,  890082,  890100,  890126, 18, 26, 23, }, // Volgograd
    {  664, 1843,       0,  890149,       0,  0, 12,  0, }, // Vostok
    {  664, 1850,       0,  890161,       0,  0, 18,  0, }, // Wake
    {  664, 1855,       0,  890179,       0,  0, 22,  0, }, // Wallis
    {  664, 1862,  890201,  890214,  890235, 13, 21, 18, }, // Yakutsk
    {  664, 1870,  890253,  890272,  890299, 19, 27, 24, }, // Yekaterinburg
    {  664, 1892,       0,  890323,       0,  0, 13,  0, }, // Yukon
    {  665,   81,  890336,  890360,  890392, 24, 32, 29, }, // Africa_Western Yoruba/Latin/Benin
    {  665,  107,  884456,  890421,  890442, 13, 21, 20, }, // Alaska
    {  665,  135,  890462,  890476,  890498, 14, 22, 18, }, // Amazon
    {  665,  142,  884564,  884584,  890516, 20, 26, 28, }, // America_Central
    {  665,  158,  884638,  884657,  890544, 19, 25, 27, }, // America_Eastern
    {  665,  174,  884709,  884718,  890571,  9, 15, 21, }, // America_Mountain
    {  665,  191,  884754,  884769,  890592, 15, 21, 23, }, // America_Pacific
    {  665,  214,  884813,  890615,  890633, 10, 18, 18, }, // Apia
    {  665,  232,  884859,  890651,  890671, 12, 20, 20, }, // Arabian
    {  665,  240,  890691,  884924,  884944, 13, 20, 19, }, // Argentina
    {  665,  250,  890704,  890730,  890764, 26, 34, 30, }, // Argentina_Western
    {  665,  268,  885053,  890794,  890815, 13, 21, 17, }, // Armenia
    {  665,  286,  885104,  885121,  890832, 17, 23, 25, }, // Atlantic
    {  665,  295,  885169,  890857,  890893, 28, 36, 36, }, // Australia_Central
    {  665,  313,  890929,  890968,  891015, 39, 47, 47, }, // Australia_CentralWestern
    {  665,  338,  885402,  891062,  891095, 25, 33, 33, }, // Australia_Eastern
    {  665,  356,  891128,  891154,  891188, 26, 34, 34, }, // Australia_Western
    {  665,  374,  885587,  891222,  891246, 16, 24, 21, }, // Azerbaijan
    {  665,  385,  891267,  891279,  891299, 12, 20, 17, }, // Azores
    {  665,  397,  885697,  891316,  891340, 16, 24, 21, }, // Bangladesh
    {  665,  461,  885834,  891361,  891385, 16, 24, 22, }, // Cape_Verde
    {  665,  478,       0,  891407,       0,  0, 22,  0, }, // Chamorro
    {  665,  487,  885918,  891429,  891449, 12, 20, 20, }, // Chatham
    {  665,  495,  891469,  891480,  891499, 11, 19, 15, }, // Chile
    {  665,  501,  891514,  891526,  891548, 12, 22, 20, }, // China
    {  665,  517,       0,  891568,       0,  0, 24,  0, }, // Cocos
    {  665,  532,  891592,  891616,  891648, 24, 32, 35, }, // Cook
    {  665,  537,  886251,  891683,  891701, 10, 18, 18, }, // Cuba
    {  665,  627,  886431,  886449,  891719, 18, 24, 23, }, // Europe_Central
    {  665,  642,  886496,  886522,  891742, 26, 32, 31, }, // Europe_Eastern
    {  665,  680,  891773,  891795,  891823, 22, 28, 31, }, // Europe_Western
    {  665,  695,  886695,  891854,  886740, 15, 30, 27, }, // Falkland
    {  665,  704,  886767,  891884,  891902, 10, 18, 15, }, // Fiji
    {  665,  764,  886887,  891917,  891938, 13, 21, 18, }, // Georgia
    {  665,  772,       0,  891956,       0,  0, 27,  0, }, // Gilbert_Islands
    {  665,  830,  886966,  891983,  887025, 25, 34, 25, }, // Greenland_Eastern
    {  665,  848,  892017,  892043,  892077, 26, 34, 33, }, // Greenland_Western
    {  665,  871,       0,  892110,       0,  0, 18,  0, }, // Gulf
    {  665,  883,  887173,  892128,  892157, 21, 29, 29, }, // Hawaii_Aleutian
    {  665,  899,  887252,  892186,  892211, 15, 25, 20, }, // Hong_Kong
    {  665,  909,  887312,  892231,  892253, 12, 22, 17, }, // Hovd
    {  665,  914,       0,  892270,       0,  0, 19,  0, }, // India
    {  665,  979,       0,  892289,       0,  0, 26,  0, }, // Indonesia_Western
    {  665,  997,  887496,  892315,  892334, 11, 19, 19, }, // Iran
    {  665, 1008,  887545,  892353,  892374, 13, 21, 18, }, // Irkutsk
    {  665, 1016,  887597,  892392,  892412, 12, 20, 20, }, // Israel
    {  665, 1023,  887649,  892432,  892453, 11, 21, 16, }, // Japan
    {  665, 1077,       0,  892469,       0,  0, 28,  0, }, // Kazakhstan_Western
    {  665, 1106,  887768,  892497,  892518, 11, 21, 19, }, // Korea
    {  665, 1119,  887832,  892537,  892564, 19, 27, 24, }, // Krasnoyarsk
    {  665, 1175,       0,  892588,       0,  0, 24,  0, }, // Line_Islands
    {  665, 1188,  887945,  892612,  892635, 15, 23, 23, }, // Lord_Howe
    {  665, 1204,  888006,  892658,  892680, 14, 22, 19, }, // Magadan
    {  665, 1247,       0,  892699,       0,  0, 26,  0, }, // Marshall_Islands
    {  665, 1264,  892725,  892741,  892765, 16, 24, 21, }, // Mauritius
    {  665, 1281,  892786,  892810,  892841, 24, 31, 31, }, // Mexico_Pacific
    {  665, 1296,  888282,  892872,  892899, 17, 27, 22, }, // Mongolia
    {  665, 1305,  888348,  892921,  892941, 12, 20, 17, }, // Moscow
    {  665, 1332,  888437,  892958,  892988, 22, 30, 27, }, // New_Caledonia
    {  665, 1346,  888516,  893015,  893040, 17, 25, 25, }, // New_Zealand
    {  665, 1358,  888583,  893065,  893091, 18, 26, 26, }, // Newfoundland
    {  665, 1376,  888663,  893117,  893146, 21, 29, 29, }, // Norfolk
    {  665, 1406,  888821,  893175,  893201, 19, 26, 23, }, // Novosibirsk
    {  665, 1418,  888889,  893224,  893245, 13, 21, 18, }, // Omsk
    {  665, 1428,  888941,  893263,  893286, 15, 23, 20, }, // Pakistan
    {  665, 1460,  889032,  893306,  889068, 14, 22, 19, }, // Paraguay
    {  665, 1469,  889087,  893328,  889115, 10, 18, 15, }, // Peru
    {  665, 1474,  889130,  893346,  893371, 17, 25, 22, }, // Philippines
    {  665, 1486,       0,  893393,       0,  0, 26,  0, }, // Phoenix_Islands
    {  665, 1502,  889220,  893419,  893450, 23, 31, 31, }, // Pierre_Miquelon
    {  665, 1554,       0,  893481,       0,  0, 17,  0, }, // Reunion
    {  665, 1570,  889377,  893498,  893520, 14, 22, 19, }, // Sakhalin
    {  665, 1596,  889432,  893539,  893558, 11, 19, 19, }, // Samoa
    {  665, 1602,       0,  893577,       0,  0, 16,  0, }, // Seychelles
    {  665, 1624,       0,  893593,       0,  0, 23,  0, }, // Singapore
    {  665, 1634,       0,  893616,       0,  0, 26,  0, }, // Solomon
    {  665, 1642,       0,  893642,       0,  0, 17,  0, }, // South_Georgia
    {  665, 1689,  889599,  893659,  893681, 12, 22, 20, }, // Taipei
    {  665, 1732,  889685,  893701,  893720, 11, 19, 16, }, // Tonga
    {  665, 1750,  889742,  893736,  893764, 20, 28, 24, }, // Turkmenistan
    {  665, 1777,  889826,  893788,  889859, 12, 21, 17, }, // Uruguay
    {  665, 1792,  889876,  893809,  893835, 18, 26, 23, }, // Uzbekistan
    {  665, 1803,  889943,  893858,  893879, 13, 21, 18, }, // Vanuatu
    {  665, 1821,  890009,  893897,  893925, 20, 28, 25, }, // Vladivostok
    {  665, 1833,  890082,  893950,  893976, 18, 26, 23, }, // Volgograd
    {  665, 1862,  890201,  893999,  894020, 13, 21, 18, }, // Yakutsk
    {  665, 1870,  890253,  894038,  894065, 19, 27, 24, }, // Yekaterinburg
    {  665, 1892,       0,  894089,       0,  0, 13,  0, }, // Yukon
    {  667,  788,       0,  894102,       0,  0, 31,  0, }, // GMT Zhuang/Latin/China
    {  668,    5,       0,  894133,       0,  0, 26,  0, }, // Afghanistan Zulu/Latin/South Africa
    {  668,   17,       0,  894159,       0,  0, 29,  0, }, // Africa_Central
    {  668,   32,       0,  894188,       0,  0, 31,  0, }, // Africa_Eastern
    {  668,   65,       0,  894219,       0,  0, 40,  0, }, // Africa_Southern
    {  668,   81,  894259,  894292,  894335, 33, 43, 43, }, // Africa_Western
    {  668,  107,  894378,  894399,  894435, 21, 36, 30, }, // Alaska
    {  668,  135,  894465,  894486,  894522, 21, 36, 31, }, // Amazon
    {  668,  142,  894553,  894590,  894642, 37, 52, 46, }, // America_Central
    {  668,  158,  894688,  894722,  894771, 34, 49, 43, }, // America_Eastern
    {  668,  174,  894814,  894852,  894905, 38, 53, 47, }, // America_Mountain
    {  668,  191,  894952,  894989,  895041, 37, 52, 46, }, // America_Pacific
    {  668,  207,  895087,  895104,  895130, 17, 26, 24, }, // Anadyr
    {  668,  214,  895154,  895173,  895202, 19, 29, 28, }, // Apia
    {  668,  232,  895230,  895252,  895284, 22, 32, 29, }, // Arabian
    {  668,  240,  895313,  895337,  895376, 24, 39, 34, }, // Argentina
    {  668,  250,  895410,  895445,  895495, 35, 50, 45, }, // Argentina_Western
    {  668,  268,  895540,  895561,  895593, 21, 32, 30, }, // Armenia
    {  668,  286,  895623,  895646,  895684, 23, 38, 32, }, // Atlantic
    {  668,  295,  895716,  895748,  895791, 32, 43, 42, }, // Australia_Central
    {  668,  313,  895833,  895871,  895919, 38, 48, 47, }, // Australia_CentralWestern
    {  668,  338,  895966,  895998,  896038, 32, 40, 39, }, // Australia_Eastern
    {  668,  356,  896077,  896109,  896152, 32, 43, 42, }, // Australia_Western
    {  668,  374,  896194,  896219,  896254, 25, 35, 33, }, // Azerbaijan
    {  668,  385,  896287,  896308,  896344, 21, 36, 31, }, // Azores
    {  668,  397,  896375,  896400,  896435, 25, 35, 35, }, // Bangladesh
    {  668,  415,       0,  896470,       0,  0, 21,  0, }, // Bhutan
    {  668,  422,       0,  896491,       0,  0, 22,  0, }, // Bolivia
    {  668,  437,  896513,  896536,  896574, 23, 38, 33, }, // Brasilia
    {  668,  454,       0,  896607,       0,  0, 32,  0, }, // Brunei
    {  668,  461,  896639,  896664,  896700, 25, 36, 33, }, // Cape_Verde
    {  668,  478,       0,  896733,       0,  0, 33,  0, }, // Chamorro
    {  668,  487,  896766,  896788,  896820, 22, 32, 31, }, // Chatham
    {  668,  495,  896851,  896871,  896906, 20, 35, 30, }, // Chile
    {  668,  501,  896936,  896956,  896986, 20, 30, 27, }, // China
    {  668,  507,       0,  897013,       0,  0, 31,  0, }, // Christmas
    {  668,  517,       0,  897044,       0,  0, 28,  0, }, // Cocos
    {  668,  523,  897072,  897095,  897133, 23, 38, 33, }, // Colombia
    {  668,  532,  897166,  897193,  897230, 27, 37, 50, }, // Cook
    {  668,  537,  897280,  897299,  897333, 19, 34, 28, }, // Cuba
    {  668,  548,       0,  897361,       0,  0, 20,  0, }, // Davis
    {  668,  564,       0,  897381,       0,  0, 31,  0, }, // DumontDUrville
    {  668,  601,       0,  897412,       0,  0, 25,  0, }, // East_Timor
    {  668,  612,  897437,  897465,  897508, 28, 43, 38, }, // Easter
    {  668,  619,       0,  897546,       0,  0, 22,  0, }, // Ecuador
    {  668,  627,  897568,  897597,  897641, 29, 44, 39, }, // Europe_Central
    {  668,  642,  897680,  897709,  897753, 29, 44, 39, }, // Europe_Eastern
    {  668,  657,       0,  897792,       0,  0, 37,  0, }, // Europe_Further_Eastern
    {  668,  680,  897829,  897858,  897902, 29, 44, 39, }, // Europe_Western
    {  668,  695,  897941,  897972,  898018, 31, 46, 41, }, // Falkland
    {  668,  704,  898059,  898078,  898107, 19, 29, 27, }, // Fiji
    {  668,  709,       0,  898134,       0,  0, 28,  0, }, // French_Guiana
    {  668,  723,       0,  898162,       0,  0, 45,  0, }, // French_Southern
    {  668,  746,       0,  898207,       0,  0, 24,  0, }, // Galapagos
    {  668,  756,       0,  898231,       0,  0, 22,  0, }, // Gambier
    {  668,  764,  898253,  898275,  898307, 22, 32, 30, }, // Georgia
    {  668,  772,       0,  898337,       0,  0, 30,  0, }, // Gilbert_Islands
    {  668,  788,       0,  898367,       0,  0, 29,  0, }, // GMT
    {  668,  830,  898396,  898425,  898469, 29, 44, 38, }, // Greenland_Eastern
    {  668,  848,  898507,  898536,  898580, 29, 44, 39, }, // Greenland_Western
    {  668,  871,       0,  898619,       0,  0, 29,  0, }, // Gulf
    {  668,  876,       0,  898648,       0,  0, 21,  0, }, // Guyana
    {  668,  883,  898669,  898698,  898742, 29, 44, 38, }, // Hawaii_Aleutian
    {  668,  899,  898780,  898804,  898838, 24, 34, 32, }, // Hong_Kong
    {  668,  909,  898870,  898889,  898918, 19, 29, 24, }, // Hovd
    {  668,  914,       0,  898942,       0,  0, 30,  0, }, // India
    {  668,  920,       0,  898972,       0,  0, 27,  0, }, // Indian_Ocean
    {  668,  933,       0,  898999,       0,  0, 24,  0, }, // Indochina
    {  668,  943,       0,  899023,       0,  0, 32,  0, }, // Indonesia_Central
    {  668,  961,       0,  899055,       0,  0, 32,  0, }, // Indonesia_Eastern
    {  668,  979,       0,  899087,       0,  0, 32,  0, }, // Indonesia_Western
    {  668,  997,  899119,  899138,  899167, 19, 29, 28, }, // Iran
    {  668, 1008,  899195,  899217,  899249, 22, 32, 29, }, // Irkutsk
    {  668, 1016,  899278,  899299,  899330, 21, 31, 31, }, // Israel
    {  668, 1023,  899361,  899381,  899411, 20, 30, 27, }, // Japan
    {  668, 1029,  899438,  899473,  899517, 35, 44, 42, }, // Kamchatka
    {  668, 1047,       0,  899559,       0,  0, 24,  0, }, // Kazakhstan
    {  668, 1058,       0,  899583,       0,  0, 39,  0, }, // Kazakhstan_Eastern
    {  668, 1077,       0,  899622,       0,  0, 40,  0, }, // Kazakhstan_Western
    {  668, 1106,  899662,  899682,  899712, 20, 30, 27, }, // Korea
    {  668, 1112,       0,  899739,       0,  0, 21,  0, }, // Kosrae
    {  668, 1119,  899760,  899786,  899822, 26, 36, 33, }, // Krasnoyarsk
    {  668, 1151,       0,  899855,       0,  0, 24,  0, }, // Kyrgystan
    {  668, 1175,       0,  899879,       0,  0, 27,  0, }, // Line_Islands
    {  668, 1188,  899906,  899930,  899964, 24, 34, 33, }, // Lord_Howe
    {  668, 1204,  899997,  900019,  900051, 22, 32, 29, }, // Magadan
    {  668, 1219,       0,  900080,       0,  0, 23,  0, }, // Malaysia
    {  668, 1228,       0,  900103,       0,  0, 23,  0, }, // Maldives
    {  668, 1237,       0,  900126,       0,  0, 24,  0, }, // Marquesas
    {  668, 1247,       0,  900150,       0,  0, 31,  0, }, // Marshall_Islands
    {  668, 1264,  900181,  900205,  900239, 24, 34, 32, }, // Mauritius
    {  668, 1274,       0,  900271,       0,  0, 21,  0, }, // Mawson
    {  668, 1281,  900292,  900322,  900367, 30, 45, 39, }, // Mexico_Pacific
    {  668, 1296,  900406,  900431,  900466, 25, 35, 30, }, // Mongolia
    {  668, 1305,  900496,  900517,  900553, 21, 36, 28, }, // Moscow
    {  668, 1312,       0,  900581,       0,  0, 22,  0, }, // Myanmar
    {  668, 1320,       0,  900603,       0,  0, 20,  0, }, // Nauru
    {  668, 1326,       0,  900623,       0,  0, 20,  0, }, // Nepal
    {  668, 1332,  900643,  900671,  900709, 28, 38, 38, }, // New_Caledonia
    {  668, 1346,  900747,  900773,  900809, 26, 36, 35, }, // New_Zealand
    {  668, 1358,  900844,  900871,  900913, 27, 42, 36, }, // Newfoundland
    {  668, 1371,       0,  900949,       0,  0, 19,  0, }, // Niue
    {  668, 1376,  900968,  900998,  901038, 30, 40, 40, }, // Norfolk
    {  668, 1384,  901078,  901112,  901156, 34, 44, 44, }, // Noronha
    {  668, 1406,  901200,  901226,  901262, 26, 36, 36, }, // Novosibirsk
    {  668, 1418,  901298,  901317,  901346, 19, 29, 29, }, // Omsk
    {  668, 1428,  901375,  901398,  901431, 23, 33, 33, }, // Pakistan
    {  668, 1437,       0,  901464,       0,  0, 20,  0, }, // Palau
    {  668, 1443,       0,  901484,       0,  0, 31,  0, }, // Papua_New_Guinea
    {  668, 1460,  901515,  901538,  901571, 23, 33, 33, }, // Paraguay
    {  668, 1469,  901604,  901623,  901652, 19, 29, 29, }, // Peru
    {  668, 1474,  901681,  901706,  901741, 25, 35, 33, }, // Philippines
    {  668, 1486,       0,  901774,       0,  0, 30,  0, }, // Phoenix_Islands
    {  668, 1502,  901804,  901845,  901900, 41, 55, 50, }, // Pierre_Miquelon
    {  668, 1518,       0,  901950,       0,  0, 23,  0, }, // Pitcairn
    {  668, 1527,       0,  901973,       0,  0, 21,  0, }, // Ponape
    {  668, 1534,       0,  901994,       0,  0, 24,  0, }, // Pyongyang
    {  668, 1554,       0,  902018,       0,  0, 22,  0, }, // Reunion
    {  668, 1562,       0,  902040,       0,  0, 22,  0, }, // Rothera
    {  668, 1570,  902062,  902085,  902118, 23, 33, 30, }, // Sakhalin
    {  668, 1579,  902148,  902165,  902191, 17, 26, 24, }, // Samara
    {  668, 1596,  902215,  902235,  902265, 20, 30, 29, }, // Samoa
    {  668, 1602,       0,  902294,       0,  0, 25,  0, }, // Seychelles
    {  668, 1624,       0,  902319,       0,  0, 34,  0, }, // Singapore
    {  668, 1634,       0,  902353,       0,  0, 30,  0, }, // Solomon
    {  668, 1642,       0,  902383,       0,  0, 28,  0, }, // South_Georgia
    {  668, 1656,       0,  902411,       0,  0, 23,  0, }, // Suriname
    {  668, 1676,       0,  902434,       0,  0, 20,  0, }, // Syowa
    {  668, 1682,       0,  902454,       0,  0, 21,  0, }, // Tahiti
    {  668, 1689,  902475,  902496,  902527, 21, 31, 28, }, // Taipei
    {  668, 1696,       0,  902555,       0,  0, 25,  0, }, // Tajikistan
    {  668, 1724,       0,  902580,       0,  0, 22,  0, }, // Tokelau
    {  668, 1732,  902602,  902622,  902652, 20, 30, 30, }, // Tonga
    {  668, 1738,       0,  902682,       0,  0, 20,  0, }, // Truk
    {  668, 1750,  902702,  902729,  902766, 27, 37, 35, }, // Turkmenistan
    {  668, 1763,       0,  902801,       0,  0, 21,  0, }, // Tuvalu
    {  668, 1777,  902822,  902844,  902881, 22, 37, 32, }, // Uruguay
    {  668, 1792,  902913,  902938,  902973, 25, 35, 33, }, // Uzbekistan
    {  668, 1803,  903006,  903028,  903065, 22, 37, 32, }, // Vanuatu
    {  668, 1811,       0,  903097,       0,  0, 24,  0, }, // Venezuela
    {  668, 1821,  903121,  903147,  903183, 26, 36, 33, }, // Vladivostok
    {  668, 1833,  903216,  903240,  903274, 24, 34, 34, }, // Volgograd
    {  668, 1843,       0,  903308,       0,  0, 21,  0, }, // Vostok
    {  668, 1850,       0,  903329,       0,  0, 26,  0, }, // Wake
    {  668, 1855,       0,  903355,       0,  0, 33,  0, }, // Wallis
    {  668, 1862,  903388,  903410,  903442, 22, 32, 29, }, // Yakutsk
    {  668, 1870,  903471,  903499,  903537, 28, 38, 35, }, // Yekaterinburg
    {  668, 1892,       0,    6312,       0,  0, 10,  0, }, // Yukon
    {  669,    0,  903572,  903583,  903597, 11, 14, 17, }, // Acre Kaingang/Latin/Brazil
    {  669,    5,       0,  903614,       0,  0, 16,  0, }, // Afghanistan
    {  669,   17,       0,  903630,       0,  0, 18,  0, }, // Africa_Central
    {  669,   32,       0,  903648,       0,  0, 19,  0, }, // Africa_Eastern
    {  669,   65,       0,  903667,       0,  0, 17,  0, }, // Africa_Southern
    {  669,   81,  903684,  903703,  903725, 19, 22, 25, }, // Africa_Western
    {  669,  107,  903750,  903764,  903781, 14, 17, 20, }, // Alaska
    {  669,  128,  903801,  903814,  903830, 13, 16, 19, }, // Almaty
    {  669,  135,  903849,  903863,  903880, 14, 17, 20, }, // Amazon
    {  669,  142,  903637,  903900,  903914, 11, 14, 17, }, // America_Central
    {  669,  158,  903931,  903943,  903958, 12, 15, 18, }, // America_Eastern
    {  669,  174,  903976,  903986,  903999, 10, 13, 16, }, // America_Mountain
    {  669,  191,  904015,  904027,  904042, 12, 15, 18, }, // America_Pacific
    {  669,  207,  904060,  904074,  904091, 14, 17, 20, }, // Anadyr
    {  669,  214,  904111,  904123,  904138, 12, 15, 18, }, // Apia
    {  669,  219,  904156,  904168,  904183, 12, 15, 18, }, // Aqtau
    {  669,  225,  904201,  904214,  904230, 13, 16, 19, }, // Aqtobe
    {  669,  232,  904249,  904263,  904280, 14, 17, 20, }, // Arabian
    {  669,  240,  904300,  904315,  904334, 15, 19, 21, }, // Argentina
    {  669,  250,  904355,  904376,  904400, 21, 24, 26, }, // Argentina_Western
    {  669,  268,  904426,  904441,  904459, 15, 18, 21, }, // Armenia
    {  669,  286,  904480,  904495,  904513, 15, 18, 21, }, // Atlantic
    {  669,  295,  904534,  904556,  904581, 22, 25, 28, }, // Australia_Central
    {  669,  313,  904609,  904636,  904667, 27, 31, 34, }, // Australia_CentralWestern
    {  669,  338,  904701,  904724,  904750, 23, 26, 29, }, // Australia_Eastern
    {  669,  356,  904779,  904802,  904828, 23, 26, 29, }, // Australia_Western
    {  669,  374,  904857,  904874,  904894, 17, 20, 23, }, // Azerbaijan
    {  669,  385,  904917,  904931,  904948, 14, 17, 19, }, // Azores
    {  669,  397,  904967,  904984,  905004, 17, 20, 23, }, // Bangladesh
    {  669,  415,       0,  905027,       0,  0, 12,  0, }, // Bhutan
    {  669,  422,       0,  905039,       0,  0, 14,  0, }, // Bolivia
    {  669,  437,  905053,  905068,  905086, 15, 18, 21, }, // Brasilia
    {  669,  454,       0,  905107,       0,  0, 22,  0, }, // Brunei
    {  669,  461,  905129,  905143,  905160, 14, 17, 20, }, // Cape_Verde
    {  669,  478,       0,  905180,       0,  0, 13,  0, }, // Chamorro
    {  669,  487,  905193,  905207,  905224, 14, 17, 20, }, // Chatham
    {  669,  495,  905244,  905255,  905269, 11, 14, 17, }, // Chile
    {  669,  501,  905286,  905297,  905311, 11, 14, 17, }, // China
    {  669,  507,       0,  905328,       0,  0, 25,  0, }, // Christmas
    {  669,  517,       0,  905353,       0,  0, 22,  0, }, // Cocos
    {  669,  523,  905375,  905390,  905408, 15, 18, 21, }, // Colombia
    {  669,  532,  905429,  905449,  905472, 20, 23, 26, }, // Cook
    {  669,  537,  905498,  905509,  905523, 11, 14, 17, }, // Cuba
    {  669,  548,       0,  905540,       0,  0, 13,  0, }, // Davis
    {  669,  564,       0,  905553,       0,  0, 19,  0, }, // DumontDUrville
    {  669,  601,       0,  905572,       0,  0, 18,  0, }, // East_Timor
    {  669,  612,  905590,  905614,  905641, 24, 27, 30, }, // Easter
    {  669,  619,       0,  905671,       0,  0, 15,  0, }, // Ecuador
    {  669,  627,  905686,  905703,  905723, 17, 20, 23, }, // Europe_Central
    {  669,  642,  905746,  905764,  905785, 18, 21, 24, }, // Europe_Eastern
    {  669,  657,       0,  905809,       0,  0, 27,  0, }, // Europe_Further_Eastern
    {  669,  680,  905836,  905854,  905875, 18, 21, 24, }, // Europe_Western
    {  669,  695,  905899,  905922,  905947, 23, 25, 29, }, // Falkland
    {  669,  704,  905976,  905987,  906001, 11, 14, 17, }, // Fiji
    {  669,  709,       0,  906018,       0,  0, 19,  0, }, // French_Guiana
    {  669,  723,       0,  906037,       0,  0, 32,  0, }, // French_Southern
    {  669,  746,       0,  906069,       0,  0, 16,  0, }, // Galapagos
    {  669,  756,       0,  906085,       0,  0, 12,  0, }, // Gambier
    {  669,  764,  906097,  906110,  906126, 13, 16, 19, }, // Georgia
    {  669,  772,       0,  906145,       0,  0, 24,  0, }, // Gilbert_Islands
    {  669,  788,       0,  906169,       0,  0, 26,  0, }, // GMT
    {  669,  830,  906195,  906218,  906244, 23, 26, 29, }, // Greenland_Eastern
    {  669,  848,  906273,  906296,  906322, 23, 26, 29, }, // Greenland_Western
    {  669,  866,       0,  906351,       0,  0, 15,  0, }, // Guam
    {  669,  871,       0,  906366,       0,  0, 12,  0, }, // Gulf
    {  669,  876,       0,  906378,       0,  0, 13,  0, }, // Guyana
    {  669,  883,  906391,  906424,  906460, 33, 36, 39, }, // Hawaii_Aleutian
    {  669,  899,  906499,  906513,  906530, 14, 17, 20, }, // Hong_Kong
    {  669,  909,  906550,  906562,  906577, 12, 15, 18, }, // Hovd
    {  669,  914,       0,  906595,       0,  0, 15,  0, }, // India
    {  669,  920,       0,  906610,       0,  0, 19,  0, }, // Indian_Ocean
    {  669,  933,       0,  906629,       0,  0, 14,  0, }, // Indochina
    {  669,  943,       0,  906643,       0,  0, 19,  0, }, // Indonesia_Central
    {  669,  961,       0,  906662,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  669,  979,       0,  906682,       0,  0, 20,  0, }, // Indonesia_Western
    {  669,  997,  906702,  906712,  906725, 10, 13, 16, }, // Iran
    {  669, 1008,  906741,  906758,  906778, 17, 20, 23, }, // Irkutsk
    {  669, 1016,  906801,  906814,  906830, 13, 16, 19, }, // Israel
    {  669, 1023,  906849,  906860,  906874, 11, 14, 17, }, // Japan
    {  669, 1029,  906891,  906924,  906960, 33, 36, 39, }, // Kamchatka
    {  669, 1058,       0,  906999,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  669, 1077,       0,  907023,       0,  0, 24,  0, }, // Kazakhstan_Western
    {  669, 1106,  907047,  907060,  907076, 13, 16, 19, }, // Korea
    {  669, 1112,       0,  907095,       0,  0, 18,  0, }, // Kosrae
    {  669, 1119,  907113,  907132,  907154, 19, 22, 25, }, // Krasnoyarsk
    {  669, 1151,       0,  907179,       0,  0, 17,  0, }, // Kyrgystan
    {  669, 1161,       0,  907196,       0,  0, 12,  0, }, // Lanka
    {  669, 1175,       0,  907208,       0,  0, 23,  0, }, // Line_Islands
    {  669, 1188,  907231,  907248,  907268, 17, 20, 23, }, // Lord_Howe
    {  669, 1198,  907291,  907303,  907318, 12, 15, 18, }, // Macau
    {  669, 1204,  907336,  907350,  907367, 14, 17, 20, }, // Magadan
    {  669, 1219,       0,  907387,       0,  0, 13,  0, }, // Malaysia
    {  669, 1228,       0,  907400,       0,  0, 23,  0, }, // Maldives
    {  669, 1237,       0,  907423,       0,  0, 18,  0, }, // Marquesas
    {  669, 1247,       0,  907441,       0,  0, 22,  0, }, // Marshall_Islands
    {  669, 1264,  907463,  907476,  907492, 13, 16, 19, }, // Mauritius
    {  669, 1274,       0,  907511,       0,  0, 13,  0, }, // Mawson
    {  669, 1281,  907524,  907546,  907571, 22, 25, 28, }, // Mexico_Pacific
    {  669, 1296,  907599,  907616,  907636, 17, 20, 23, }, // Mongolia
    {  669, 1305,  907659,  907673,  907690, 14, 17, 20, }, // Moscow
    {  669, 1312,       0,  907710,       0,  0, 14,  0, }, // Myanmar
    {  669, 1320,       0,  907724,       0,  0, 13,  0, }, // Nauru
    {  669, 1326,       0,  907737,       0,  0, 12,  0, }, // Nepal
    {  669, 1332,  907749,  907770,  907794, 21, 24, 27, }, // New_Caledonia
    {  669, 1346,  907821,  907840,  907862, 19, 22, 25, }, // New_Zealand
    {  669, 1358,  907887,  907900,  907916, 13, 16, 19, }, // Newfoundland
    {  669, 1371,       0,  907935,       0,  0, 11,  0, }, // Niue
    {  669, 1376,  907946,  907970,  907997, 24, 27, 30, }, // Norfolk
    {  669, 1384,  908027,  908051,  908078, 24, 27, 30, }, // Noronha
    {  669, 1392,       0,  908108,       0,  0, 30,  0, }, // North_Mariana
    {  669, 1406,  908138,  908158,  908181, 20, 23, 29, }, // Novosibirsk
    {  669, 1418,  908210,  908224,  908241, 14, 17, 20, }, // Omsk
    {  669, 1428,  908261,  908277,  908296, 16, 19, 22, }, // Pakistan
    {  669, 1437,       0,  908318,       0,  0, 13,  0, }, // Palau
    {  669, 1443,       0,  908331,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  669, 1460,  908353,  908369,  908388, 16, 19, 22, }, // Paraguay
    {  669, 1469,  908410,  908421,  908435, 11, 14, 17, }, // Peru
    {  669, 1474,  908452,  908467,  908485, 15, 18, 17, }, // Philippines
    {  669, 1486,       0,  908502,       0,  0, 21,  0, }, // Phoenix_Islands
    {  669, 1502,  908523,  908550,  908580, 27, 30, 33, }, // Pierre_Miquelon
    {  669, 1518,       0,  908613,       0,  0, 24,  0, }, // Pitcairn
    {  669, 1527,       0,  908637,       0,  0, 13,  0, }, // Ponape
    {  669, 1534,       0,  908650,       0,  0, 15,  0, }, // Pyongyang
    {  669, 1544,  908665,  908681,  908700, 16, 19, 22, }, // Qyzylorda
    {  669, 1554,       0,  908722,       0,  0, 12,  0, }, // Reunion
    {  669, 1562,       0,  908734,       0,  0, 13,  0, }, // Rothera
    {  669, 1570,  908747,  908762,  908780, 15, 18, 21, }, // Sakhalin
    {  669, 1579,  908801,  908814,  908830, 13, 16, 19, }, // Samara
    {  669, 1596,  908849,  908862,  908878, 13, 16, 19, }, // Samoa
    {  669, 1602,       0,  908897,       0,  0, 16,  0, }, // Seychelles
    {  669, 1624,       0,  908913,       0,  0, 18,  0, }, // Singapore
    {  669, 1634,       0,  908931,       0,  0, 23,  0, }, // Solomon
    {  669, 1642,       0,  908954,       0,  0, 20,  0, }, // South_Georgia
    {  669, 1656,       0,  908974,       0,  0, 15,  0, }, // Suriname
    {  669, 1676,       0,  908989,       0,  0, 12,  0, }, // Syowa
    {  669, 1682,       0,  909001,       0,  0, 12,  0, }, // Tahiti
    {  669, 1689,  909013,  909026,  909042, 13, 16, 19, }, // Taipei
    {  669, 1696,       0,  909061,       0,  0, 18,  0, }, // Tajikistan
    {  669, 1724,       0,  909079,       0,  0, 15,  0, }, // Tokelau
    {  669, 1732,  909094,  909105,  909119, 11, 14, 17, }, // Tonga
    {  669, 1738,       0,  909136,       0,  0, 12,  0, }, // Truk
    {  669, 1750,  909148,  909169,  909193, 21, 24, 27, }, // Turkmenistan
    {  669, 1763,       0,  909220,       0,  0, 13,  0, }, // Tuvalu
    {  669, 1777,  909233,  909248,  909266, 15, 18, 21, }, // Uruguay
    {  669, 1792,  909287,  909306,  909328, 19, 22, 25, }, // Uzbekistan
    {  669, 1803,  909353,  909368,  909386, 15, 18, 21, }, // Vanuatu
    {  669, 1811,       0,  909407,       0,  0, 17,  0, }, // Venezuela
    {  669, 1821,  909424,  909445,  909468, 21, 23, 26, }, // Vladivostok
    {  669, 1833,  909494,  909511,  909531, 17, 20, 23, }, // Volgograd
    {  669, 1843,       0,  909554,       0,  0, 15,  0, }, // Vostok
    {  669, 1850,       0,  909569,       0,  0, 21,  0, }, // Wake
    {  669, 1855,       0,  909590,       0,  0, 24,  0, }, // Wallis
    {  669, 1862,  909614,  909632,  909654, 18, 22, 24, }, // Yakutsk
    {  669, 1870,  909678,  909698,  909721, 20, 23, 26, }, // Yekaterinburg
    {  670,    0,  909747,  909765,  909790, 18, 25, 29, }, // Acre Nheengatu/Latin/Brazil
    {  670,    5,       0,  909819,       0,  0, 22,  0, }, // Afghanistan
    {  670,   17,       0,  909841,       0,  0, 27,  0, }, // Africa_Central
    {  670,   32,       0,  909868,       0,  0, 27,  0, }, // Africa_Eastern
    {  670,   65,       0,  909895,       0,  0, 23,  0, }, // Africa_Southern
    {  670,   81,  909918,  909947,  909982, 29, 35, 39, }, // Africa_Western
    {  670,  107,  910021,  910037,  910057, 16, 20, 27, }, // Alaska
    {  670,  128,  910084,  910099,  910118, 15, 19, 26, }, // Almaty
    {  670,  135,  910144,  910160,  910180, 16, 20, 27, }, // Amazon
    {  670,  142,  910207,  910215,  910241,  8, 26, 30, }, // America_Central
    {  670,  158,  910271,  910286,  910313, 15, 27, 26, }, // America_Eastern
    {  670,  174,  910339,  910352,  910368, 13, 16, 19, }, // America_Mountain
    {  670,  191,  910387,  910402,  910420, 15, 18, 21, }, // America_Pacific
    {  670,  207,  910441,  910455,  910476, 14, 21, 25, }, // Anadyr
    {  670,  214,  910501,  910515,  910536, 14, 21, 25, }, // Apia
    {  670,  219,  910561,  910576,  910598, 15, 22, 26, }, // Aqtau
    {  670,  225,  910624,  910640,  910663, 16, 23, 27, }, // Aqtobe
    {  670,  232,  910690,  910706,  910729, 16, 23, 27, }, // Arabian
    {  670,  240,  910756,  910773,  910797, 17, 24, 28, }, // Argentina
    {  670,  250,  910825,  910854,  910890, 29, 36, 40, }, // Argentina_Western
    {  670,  268,  910930,  910948,  910973, 18, 25, 29, }, // Armenia
    {  670,  286,  911002,  911020,  911045, 18, 25, 28, }, // Atlantic
    {  670,  295,  911073,  911105,  911144, 32, 39, 43, }, // Australia_Central
    {  670,  313,  911187,  911231,  911282, 44, 51, 55, }, // Australia_CentralWestern
    {  670,  338,  911337,  911369,  911408, 32, 39, 43, }, // Australia_Eastern
    {  670,  356,  911451,  911484,  911524, 33, 40, 44, }, // Australia_Western
    {  670,  374,  911568,  911588,  911615, 20, 27, 31, }, // Azerbaijan
    {  670,  385,  911646,  911664,  911689, 18, 25, 29, }, // Azores
    {  670,  397,  911718,  911737,  911763, 19, 26, 30, }, // Bangladesh
    {  670,  415,       0,  911793,       0,  0, 14,  0, }, // Bhutan
    {  670,  422,       0,  911807,       0,  0, 16,  0, }, // Bolivia
    {  670,  437,  911823,  911838,  911856, 15, 18, 21, }, // Brasilia
    {  670,  454,       0,  911877,       0,  0, 26,  0, }, // Brunei
    {  670,  461,  911903,  911924,  911952, 21, 28, 32, }, // Cape_Verde
    {  670,  478,       0,  911984,       0,  0, 15,  0, }, // Chamorro
    {  670,  487,  911999,  912014,  912036, 15, 22, 26, }, // Chatham
    {  670,  495,  912062,  912075,  912095, 13, 20, 24, }, // Chile
    {  670,  501,  912119,  912132,  912152, 13, 20, 24, }, // China
    {  670,  507,       0,  912176,       0,  0, 24,  0, }, // Christmas
    {  670,  517,       0,  912200,       0,  0, 27,  0, }, // Cocos
    {  670,  523,  912227,  912243,  912266, 16, 23, 27, }, // Colombia
    {  670,  532,  912293,  912317,  912348, 24, 31, 42, }, // Cook
    {  670,  537,  912390,  912403,  912423, 13, 20, 24, }, // Cuba
    {  670,  548,       0,  912447,       0,  0, 13,  0, }, // Davis
    {  670,  564,       0,  912460,       0,  0, 25,  0, }, // DumontDUrville
    {  670,  601,       0,  912485,       0,  0, 22,  0, }, // East_Timor
    {  670,  612,  912507,  912532,  912564, 25, 32, 36, }, // Easter
    {  670,  619,       0,  912600,       0,  0, 15,  0, }, // Ecuador
    {  670,  627,  912615,  912641,  912674, 26, 33, 37, }, // Europe_Central
    {  670,  642,  912711,  912737,  912770, 26, 33, 37, }, // Europe_Eastern
    {  670,  657,       0,  912807,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {  670,  680,  912833,  912860,  912894, 27, 34, 38, }, // Europe_Western
    {  670,  695,  912932,  912957,  912989, 25, 32, 36, }, // Falkland
    {  670,  704,  913025,  913038,  913058, 13, 20, 24, }, // Fiji
    {  670,  709,       0,  913082,       0,  0, 26,  0, }, // French_Guiana
    {  670,  723,       0,  913108,       0,  0, 47,  0, }, // French_Southern
    {  670,  746,       0,  913155,       0,  0, 21,  0, }, // Galapagos
    {  670,  756,       0,  913176,       0,  0, 16,  0, }, // Gambier
    {  670,  764,  913192,  913208,  913231, 16, 23, 27, }, // Georgia
    {  670,  772,       0,  913258,       0,  0, 25,  0, }, // Gilbert_Islands
    {  670,  788,       0,  913283,       0,  0, 34,  0, }, // GMT
    {  670,  830,  913317,  913348,  913386, 31, 38, 42, }, // Greenland_Eastern
    {  670,  848,  913428,  913460,  913499, 32, 39, 43, }, // Greenland_Western
    {  670,  866,       0,  913542,       0,  0, 20,  0, }, // Guam
    {  670,  871,       0,  913562,       0,  0, 14,  0, }, // Gulf
    {  670,  876,       0,  913576,       0,  0, 15,  0, }, // Guyana
    {  670,  883,  913591,  913631,  913678, 40, 47, 51, }, // Hawaii_Aleutian
    {  670,  899,  913729,  913745,  913768, 16, 23, 27, }, // Hong_Kong
    {  670,  909,  913795,  913810,  913832, 15, 22, 26, }, // Hovd
    {  670,  914,       0,  913858,       0,  0, 20,  0, }, // India
    {  670,  920,       0,  913878,       0,  0, 22,  0, }, // Indian_Ocean
    {  670,  933,       0,  913900,       0,  0, 16,  0, }, // Indochina
    {  670,  943,       0,  913916,       0,  0, 29,  0, }, // Indonesia_Central
    {  670,  961,       0,  913945,       0,  0, 29,  0, }, // Indonesia_Eastern
    {  670,  979,       0,  913974,       0,  0, 30,  0, }, // Indonesia_Western
    {  670,  997,  914004,  914016,  914035, 12, 19, 23, }, // Iran
    {  670, 1008,  914058,  914077,  914103, 19, 26, 30, }, // Irkutsk
    {  670, 1016,  914133,  914150,  914174, 17, 24, 28, }, // Israel
    {  670, 1023,  914202,  914215,  914235, 13, 20, 24, }, // Japan
    {  670, 1029,  914259,  914292,  914332, 33, 40, 44, }, // Kamchatka
    {  670, 1058,       0,  914376,       0,  0, 32,  0, }, // Kazakhstan_Eastern
    {  670, 1077,       0,  914408,       0,  0, 33,  0, }, // Kazakhstan_Western
    {  670, 1106,  914441,  914456,  914478, 15, 22, 26, }, // Korea
    {  670, 1112,       0,  914504,       0,  0, 16,  0, }, // Kosrae
    {  670, 1119,  914520,  914545,  914577, 25, 32, 36, }, // Krasnoyarsk
    {  670, 1151,       0,  914613,       0,  0, 21,  0, }, // Kyrgystan
    {  670, 1161,       0,  914634,       0,  0, 13,  0, }, // Lanka
    {  670, 1175,       0,  914647,       0,  0, 24,  0, }, // Line_Islands
    {  670, 1188,  914671,  914691,  914718, 20, 27, 31, }, // Lord_Howe
    {  670, 1198,  914749,  914763,  914784, 14, 21, 25, }, // Macau
    {  670, 1204,  914809,  914824,  914846, 15, 22, 26, }, // Magadan
    {  670, 1219,       0,  914872,       0,  0, 17,  0, }, // Malaysia
    {  670, 1228,       0,  914889,       0,  0, 30,  0, }, // Maldives
    {  670, 1237,       0,  914919,       0,  0, 19,  0, }, // Marquesas
    {  670, 1247,       0,  914938,       0,  0, 27,  0, }, // Marshall_Islands
    {  670, 1264,  914965,  914983,  915008, 18, 25, 29, }, // Mauritius
    {  670, 1274,       0,  915037,       0,  0, 14,  0, }, // Mawson
    {  670, 1281,  915051,  915077,  915110, 26, 33, 37, }, // Mexico_Pacific
    {  670, 1296,  915147,  915166,  915192, 19, 26, 30, }, // Mongolia
    {  670, 1305,  915222,  915237,  915259, 15, 22, 26, }, // Moscow
    {  670, 1312,       0,  915285,       0,  0, 15,  0, }, // Myanmar
    {  670, 1320,       0,  915300,       0,  0, 14,  0, }, // Nauru
    {  670, 1326,       0,  915314,       0,  0, 14,  0, }, // Nepal
    {  670, 1332,  915328,  915354,  915387, 26, 33, 37, }, // New_Caledonia
    {  670, 1346,  915424,  915448,  915479, 24, 31, 35, }, // New_Zealand
    {  670, 1358,  915514,  915533,  915559, 19, 26, 30, }, // Newfoundland
    {  670, 1371,       0,  915589,       0,  0, 13,  0, }, // Niue
    {  670, 1376,  915602,  915626,  915657, 24, 31, 35, }, // Norfolk
    {  670, 1384,  915692,  915714,  915743, 22, 29, 33, }, // Noronha
    {  670, 1392,       0,  915776,       0,  0, 36,  0, }, // North_Mariana
    {  670, 1406,  915812,  915838,  915871, 26, 33, 37, }, // Novosibirsk
    {  670, 1418,  915908,  915924,  915947, 16, 23, 27, }, // Omsk
    {  670, 1428,  915974,  915993,  916019, 19, 26, 30, }, // Pakistan
    {  670, 1437,       0,  916049,       0,  0, 14,  0, }, // Palau
    {  670, 1443,       0,  916063,       0,  0, 27,  0, }, // Papua_New_Guinea
    {  670, 1460,  916090,  916108,  916133, 18, 25, 28, }, // Paraguay
    {  670, 1469,  916161,  916174,  916194, 13, 20, 24, }, // Peru
    {  670, 1474,  916218,  916235,  916259, 17, 24, 28, }, // Philippines
    {  670, 1486,       0,  916287,       0,  0, 15,  0, }, // Phoenix_Islands
    {  670, 1502,  916302,  916333,  916371, 31, 38, 42, }, // Pierre_Miquelon
    {  670, 1518,       0,  916413,       0,  0, 17,  0, }, // Pitcairn
    {  670, 1527,       0,  916430,       0,  0, 15,  0, }, // Ponape
    {  670, 1534,       0,  916445,       0,  0, 17,  0, }, // Pyongyang
    {  670, 1544,  916462,  916479,  916503, 17, 24, 28, }, // Qyzylorda
    {  670, 1554,       0,  916531,       0,  0, 22,  0, }, // Reunion
    {  670, 1562,       0,  916553,       0,  0, 15,  0, }, // Rothera
    {  670, 1570,  916568,  916585,  916609, 17, 24, 28, }, // Sakhalin
    {  670, 1579,  916637,  916652,  916674, 15, 22, 26, }, // Samara
    {  670, 1596,  916700,  916715,  916737, 15, 22, 26, }, // Samoa
    {  670, 1602,       0,  916763,       0,  0, 16,  0, }, // Seychelles
    {  670, 1624,       0,  916779,       0,  0, 24,  0, }, // Singapore
    {  670, 1634,       0,  916803,       0,  0, 27,  0, }, // Solomon
    {  670, 1642,       0,  916830,       0,  0, 23,  0, }, // South_Georgia
    {  670, 1656,       0,  916853,       0,  0, 17,  0, }, // Suriname
    {  670, 1676,       0,  916870,       0,  0, 15,  0, }, // Syowa
    {  670, 1682,       0,  916885,       0,  0, 14,  0, }, // Tahiti
    {  670, 1689,  916899,  916914,  916936, 15, 22, 26, }, // Taipei
    {  670, 1696,       0,  916962,       0,  0, 21,  0, }, // Tajikistan
    {  670, 1724,       0,  916983,       0,  0, 16,  0, }, // Tokelau
    {  670, 1732,  916999,  917012,  917032, 13, 20, 24, }, // Tonga
    {  670, 1738,       0,  917056,       0,  0, 14,  0, }, // Truk
    {  670, 1750,  917070,  917094,  917125, 24, 31, 35, }, // Turkmenistan
    {  670, 1763,       0,  917160,       0,  0, 15,  0, }, // Tuvalu
    {  670, 1777,  917175,  917192,  917216, 17, 24, 28, }, // Uruguay
    {  670, 1792,  917244,  917269,  917301, 25, 32, 36, }, // Uzbekistan
    {  670, 1803,  917337,  917353,  917376, 16, 23, 27, }, // Vanuatu
    {  670, 1811,       0,  917403,       0,  0, 18,  0, }, // Venezuela
    {  670, 1821,  917421,  917444,  917474, 23, 30, 34, }, // Vladivostok
    {  670, 1833,  917508,  917528,  917555, 20, 27, 31, }, // Volgograd
    {  670, 1843,       0,  917586,       0,  0, 17,  0, }, // Vostok
    {  670, 1850,       0,  917603,       0,  0, 14,  0, }, // Wake
    {  670, 1855,       0,  917617,       0,  0, 25,  0, }, // Wallis
    {  670, 1862,  917642,  917661,  917687, 19, 26, 30, }, // Yakutsk
    {  670, 1870,  917717,  917737,  917764, 20, 27, 31, }, // Yekaterinburg
    {  671,  415,       0,  917795,       0,  0, 14,  0, }, // Bhutan Nheengatu/Latin/Colombia
    {  671, 1502,  917809,  917840,  917878, 31, 38, 42, }, // Pierre_Miquelon
    {  672,  415,       0,  917795,       0,  0, 14,  0, }, // Bhutan Nheengatu/Latin/Venezuela
    {  672, 1502,  917809,  917840,  917878, 31, 38, 42, }, // Pierre_Miquelon
    {  673,  788,       0,   91115,       0,  0, 17,  0, }, // GMT Haryanvi/Devanagari/India
    {  674,    5,       0,  917920,       0,  0, 18,  0, }, // Afghanistan Northern Frisian/Latin/Germany
    {  674,   17,       0,  917938,       0,  0, 21,  0, }, // Africa_Central
    {  674,   32,       0,  917959,       0,  0, 17,  0, }, // Africa_Eastern
    {  674,   65,       0,  917976,       0,  0, 29,  0, }, // Africa_Southern
    {  674,   81,  918005,  918023,  918050, 18, 27, 23, }, // Africa_Western
    {  674,  107,  918073,  918084,  918104, 11, 20, 16, }, // Alaska
    {  674,  135,  918120,  918133,  918155, 13, 22, 18, }, // Amazon
    {  674,  142,  918173,  918196,  918228, 23, 32, 28, }, // America_Central
    {  674,  158,  918256,  918275,  918303, 19, 28, 24, }, // America_Eastern
    {  674,  174,  918327,  918347,  918376, 20, 29, 25, }, // America_Mountain
    {  674,  191,  918401,  918423,  918455, 22, 32, 28, }, // America_Pacific
    {  674,  214,  918483,  918492,  918510,  9, 18, 14, }, // Apia
    {  674,  232,  918524,  918537,  918559, 13, 22, 18, }, // Arabian
    {  674,  240,  918577,  918594,  918620, 17, 26, 22, }, // Argentina
    {  674,  250,  918642,  918665,  918697, 23, 32, 28, }, // Argentina_Western
    {  674,  268,  918725,  918739,  918762, 14, 23, 19, }, // Armenia
    {  674,  286,  918781,  918804,  918836, 23, 32, 28, }, // Atlantic
    {  674,  295,  918864,  918889,  918923, 25, 34, 30, }, // Australia_Central
    {  674,  313,  918953,  918983,  919022, 30, 39, 35, }, // Australia_CentralWestern
    {  674,  338,  919057,  919078,  919108, 21, 30, 26, }, // Australia_Eastern
    {  674,  356,  919134,  919156,  919187, 22, 31, 27, }, // Australia_Western
    {  674,  374,  919214,  919233,  919261, 19, 28, 24, }, // Azerbaijan
    {  674,  385,  919285,  919298,  919320, 13, 22, 18, }, // Azores
    {  674,  397,  919338,  919354,  919379, 16, 25, 21, }, // Bangladesh
    {  674,  415,       0,  919400,       0,  0, 13,  0, }, // Bhutan
    {  674,  422,       0,  919413,       0,  0, 14,  0, }, // Bolivia
    {  674,  437,  919427,  919442,  919466, 15, 24, 20, }, // Brasilia
    {  674,  454,       0,  919486,       0,  0, 22,  0, }, // Brunei
    {  674,  461,  919508,  919522,  919545, 14, 23, 20, }, // Cape_Verde
    {  674,  478,       0,  919565,       0,  0, 13,  0, }, // Chamorro
    {  674,  487,  919578,  919590,  919611, 12, 21, 17, }, // Chatham
    {  674,  495,  919628,  919639,  919659, 11, 20, 16, }, // Chile
    {  674,  501,  919675,  919687,  919708, 12, 21, 17, }, // China
    {  674,  507,       0,  919725,       0,  0, 14,  0, }, // Christmas
    {  674,  517,       0,  919739,       0,  0, 18,  0, }, // Cocos
    {  674,  523,  919757,  919771,  919794, 14, 23, 19, }, // Colombia
    {  674,  532,  919813,  919830,  919856, 17, 26, 29, }, // Cook
    {  674,  537,  919885,  919895,  919914, 10, 19, 15, }, // Cuba
    {  674,  548,       0,  919929,       0,  0, 10,  0, }, // Davis
    {  674,  564,       0,  919939,       0,  0, 21,  0, }, // DumontDUrville
    {  674,  601,       0,  919960,       0,  0, 16,  0, }, // East_Timor
    {  674,  612,  919976,  919992,  920017, 16, 25, 21, }, // Easter
    {  674,  619,       0,  920038,       0,  0, 13,  0, }, // Ecuador
    {  674,  627,  920051,  920070,  920098, 19, 28, 24, }, // Europe_Central
    {  674,  642,  920122,  920140,  920167, 18, 27, 23, }, // Europe_Eastern
    {  674,  657,       0,  920190,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {  674,  680,  920216,  920235,  920263, 19, 28, 24, }, // Europe_Western
    {  674,  695,  920287,  920307,  920336, 20, 29, 25, }, // Falkland
    {  674,  704,  920361,  920373,  920394, 12, 21, 17, }, // Fiji
    {  674,  709,       0,  920411,       0,  0, 23,  0, }, // French_Guiana
    {  674,  723,       0,  920434,       0,  0, 32,  0, }, // French_Southern
    {  674,  746,       0,  920466,       0,  0, 14,  0, }, // Galapagos
    {  674,  756,       0,  920480,       0,  0, 12,  0, }, // Gambier
    {  674,  764,  920492,  920505,  920527, 13, 22, 18, }, // Georgia
    {  674,  772,       0,  920545,       0,  0, 20,  0, }, // Gilbert_Islands
    {  674,  788,       0,  920565,       0,  0, 20,  0, }, // GMT
    {  674,  830,  920585,  920603,  920630, 18, 27, 23, }, // Greenland_Eastern
    {  674,  848,  920653,  920672,  920700, 19, 28, 24, }, // Greenland_Western
    {  674,  871,       0,  920724,       0,  0,  9,  0, }, // Gulf
    {  674,  876,       0,  920733,       0,  0, 12,  0, }, // Guyana
    {  674,  883,  920745,  920764,  920792, 19, 28, 24, }, // Hawaii_Aleutian
    {  674,  899,  920816,  920829,  920851, 13, 22, 18, }, // Hong_Kong
    {  674,  909,  920869,  920879,  920898, 10, 19, 15, }, // Hovd
    {  674,  914,       0,  920913,       0,  0, 11,  0, }, // India
    {  674,  920,       0,  920924,       0,  0, 19,  0, }, // Indian_Ocean
    {  674,  933,       0,  920943,       0,  0, 16,  0, }, // Indochina
    {  674,  943,       0,  920959,       0,  0, 25,  0, }, // Indonesia_Central
    {  674,  961,       0,  920984,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  674,  979,       0,  921005,       0,  0, 22,  0, }, // Indonesia_Western
    {  674,  997,  921027,  921037,  921056, 10, 19, 15, }, // Iran
    {  674, 1008,  921071,  921083,  921104, 12, 21, 17, }, // Irkutsk
    {  674, 1016,  921121,  921132,  921152, 11, 20, 16, }, // Israel
    {  674, 1023,  921168,  921179,  921199, 11, 20, 16, }, // Japan
    {  674, 1047,       0,  921215,       0,  0, 16,  0, }, // Kazakhstan
    {  674, 1058,       0,  921231,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {  674, 1077,       0,  921252,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  674, 1106,  921274,  921284,  921303, 10, 19, 15, }, // Korea
    {  674, 1112,       0,  921318,       0,  0, 11,  0, }, // Kosrae
    {  674, 1119,  921329,  921345,  921370, 16, 25, 21, }, // Krasnoyarsk
    {  674, 1151,       0,  921391,       0,  0, 15,  0, }, // Kyrgystan
    {  674, 1175,       0,  921406,       0,  0, 17,  0, }, // Line_Islands
    {  674, 1188,  921423,  921437,  921460, 14, 23, 19, }, // Lord_Howe
    {  674, 1204,  921479,  921491,  921512, 12, 21, 17, }, // Magadan
    {  674, 1219,       0,  921529,       0,  0, 13,  0, }, // Malaysia
    {  674, 1228,       0,  921542,       0,  0, 15,  0, }, // Maldives
    {  674, 1237,       0,  921557,       0,  0, 14,  0, }, // Marquesas
    {  674, 1247,       0,  921571,       0,  0, 21,  0, }, // Marshall_Islands
    {  674, 1264,  921592,  921606,  921629, 14, 23, 19, }, // Mauritius
    {  674, 1274,       0,  921648,       0,  0, 11,  0, }, // Mawson
    {  674, 1281,  921659,  921680,  921710, 21, 30, 26, }, // Mexico_Pacific
    {  674, 1296,  921736,  921752,  921777, 16, 25, 21, }, // Mongolia
    {  674, 1305,  921798,  921809,  921829, 11, 20, 16, }, // Moscow
    {  674, 1312,       0,  921845,       0,  0, 13,  0, }, // Myanmar
    {  674, 1320,       0,  921858,       0,  0, 10,  0, }, // Nauru
    {  674, 1326,       0,  921868,       0,  0, 12,  0, }, // Nepal
    {  674, 1332,  921880,  921899,  921927, 19, 28, 24, }, // New_Caledonia
    {  674, 1346,  921951,  921965,  921988, 14, 23, 19, }, // New_Zealand
    {  674, 1358,  922007,  922022,  922046, 15, 24, 20, }, // Newfoundland
    {  674, 1371,       0,  922066,       0,  0,  9,  0, }, // Niue
    {  674, 1376,  922075,  922093,  922120, 18, 27, 23, }, // Norfolk
    {  674, 1384,  922143,  922167,  922200, 24, 33, 29, }, // Noronha
    {  674, 1406,  922229,  922245,  922270, 16, 25, 21, }, // Novosibirsk
    {  674, 1418,  922291,  922300,  922318,  9, 18, 14, }, // Omsk
    {  674, 1428,  922332,  922346,  922369, 14, 23, 19, }, // Pakistan
    {  674, 1437,       0,  922388,       0,  0, 10,  0, }, // Palau
    {  674, 1443,       0,  922398,       0,  0, 20,  0, }, // Papua_New_Guinea
    {  674, 1460,  922418,  922431,  922453, 13, 22, 18, }, // Paraguay
    {  674, 1469,  922471,  922481,  922500, 10, 19, 15, }, // Peru
    {  674, 1474,  922515,  922530,  922554, 15, 24, 20, }, // Philippines
    {  674, 1486,       0,  922574,       0,  0, 20,  0, }, // Phoenix_Islands
    {  674, 1502,  922594,  922621,  922657, 27, 36, 32, }, // Pierre_Miquelon
    {  674, 1518,       0,  922689,       0,  0, 21,  0, }, // Pitcairn
    {  674, 1527,       0,  922710,       0,  0, 11,  0, }, // Ponape
    {  674, 1534,       0,  922721,       0,  0, 14,  0, }, // Pyongyang
    {  674, 1554,       0,  922735,       0,  0, 12,  0, }, // Reunion
    {  674, 1562,       0,  922747,       0,  0, 12,  0, }, // Rothera
    {  674, 1570,  922759,  922772,  922794, 13, 22, 18, }, // Sakhalin
    {  674, 1596,  922812,  922822,  922841, 10, 19, 15, }, // Samoa
    {  674, 1602,       0,  922856,       0,  0, 14,  0, }, // Seychelles
    {  674, 1624,       0,  922870,       0,  0, 14,  0, }, // Singapore
    {  674, 1634,       0,  922884,       0,  0, 15,  0, }, // Solomon
    {  674, 1642,       0,  922899,       0,  0, 17,  0, }, // South_Georgia
    {  674, 1656,       0,  922916,       0,  0, 13,  0, }, // Suriname
    {  674, 1676,       0,  922929,       0,  0, 10,  0, }, // Syowa
    {  674, 1682,       0,  922939,       0,  0, 11,  0, }, // Tahiti
    {  674, 1689,  922950,  922961,  922981, 11, 20, 16, }, // Taipei
    {  674, 1696,       0,  922997,       0,  0, 17,  0, }, // Tajikistan
    {  674, 1724,       0,  923014,       0,  0, 12,  0, }, // Tokelau
    {  674, 1732,  923026,  923036,  923055, 10, 19, 15, }, // Tonga
    {  674, 1738,       0,  923070,       0,  0, 10,  0, }, // Truk
    {  674, 1750,  923080,  923099,  923127, 19, 28, 24, }, // Turkmenistan
    {  674, 1763,       0,  923151,       0,  0, 12,  0, }, // Tuvalu
    {  674, 1777,  923163,  923175,  923196, 12, 21, 17, }, // Uruguay
    {  674, 1792,  923213,  923229,  923254, 16, 25, 21, }, // Uzbekistan
    {  674, 1803,  923275,  923287,  923308, 12, 21, 17, }, // Vanuatu
    {  674, 1811,       0,  923325,       0,  0, 15,  0, }, // Venezuela
    {  674, 1821,  923340,  923356,  923381, 16, 25, 21, }, // Vladivostok
    {  674, 1833,  923402,  923416,  923439, 14, 23, 19, }, // Volgograd
    {  674, 1843,       0,  923458,       0,  0, 11,  0, }, // Vostok
    {  674, 1850,       0,  923469,       0,  0, 15,  0, }, // Wake
    {  674, 1855,       0,  923484,       0,  0, 21,  0, }, // Wallis
    {  674, 1862,  923505,  923517,  923538, 12, 21, 17, }, // Yakutsk
    {  674, 1870,  923555,  923573,  923600, 18, 27, 23, }, // Yekaterinburg
    {  674, 1892,       0,  923623,       0,  0, 10,  0, }, // Yukon
    {  675,  788,       0,  923633,       0,  0, 18,  0, }, // GMT Rajasthani/Devanagari/India
    {  677,  788,       0,  923651,       0,  0, 16,  0, }, // GMT Toki Pona/Latin/world
    {  678,  788,       0,  923667,       0,  0, 15,  0, }, // GMT Pijin/Latin/Solomon Islands
    {  680,  107,  923682,  923696,  923719, 14, 23, 21, }, // Alaska Baluchi/Arabic/Pakistan
    {  680,  135,  923740,  923753,  923775, 13, 22, 20, }, // Amazon
    {  680,  142,  923795,  923814,  923842, 19, 28, 26, }, // America_Central
    {  680,  158,  923868,  923890,  923921, 22, 31, 29, }, // America_Eastern
    {  680,  174,  923950,  923972,  924003, 22, 31, 29, }, // America_Mountain
    {  680,  191,  924032,  924054,  924085, 22, 31, 29, }, // America_Pacific
    {  680,  240,  924114,  924129,  924153, 15, 24, 22, }, // Argentina
    {  680,  250,  924175,  924197,  924228, 22, 31, 29, }, // Argentina_Western
    {  680,  295,  924257,  924279,  924310, 22, 31, 29, }, // Australia_Central
    {  680,  313,  924339,  924367,  924404, 28, 37, 35, }, // Australia_CentralWestern
    {  680,  338,  924439,  924464,  924498, 25, 34, 32, }, // Australia_Eastern
    {  680,  356,  924344,  924372,  924409, 23, 32, 30, }, // Australia_Western
    {  680,  437,  924530,  924543,  924565, 13, 22, 20, }, // Brasilia
    {  680,  627,  924585,  924602,  924628, 17, 26, 24, }, // Europe_Central
    {  680,  642,  924652,  924672,  924701, 20, 29, 27, }, // Europe_Eastern
    {  680,  657,       0,  924728,       0,  0, 36,  0, }, // Europe_Further_Eastern
    {  680,  680,  924764,  924782,  924809, 18, 27, 25, }, // Europe_Western
    {  680,  788,       0,  924834,       0,  0, 17,  0, }, // GMT
    {  680,  883,  924851,  924864,  924886, 13, 22, 20, }, // Hawaii_Aleutian
    {  680,  943,       0,  924906,       0,  0, 31,  0, }, // Indonesia_Central
    {  680,  961,       0,  924937,       0,  0, 34,  0, }, // Indonesia_Eastern
    {  680,  979,       0,  924971,       0,  0, 32,  0, }, // Indonesia_Western
    {  680, 1008,  925003,  925018,  925042, 15, 24, 22, }, // Irkutsk
    {  680, 1058,       0,  925064,       0,  0, 33,  0, }, // Kazakhstan_Eastern
    {  680, 1077,       0,  925097,       0,  0, 31,  0, }, // Kazakhstan_Western
    {  680, 1119,  925128,  925147,  925175, 19, 28, 26, }, // Krasnoyarsk
    {  680, 1188,  925201,  925226,  925260, 25, 34, 32, }, // Lord_Howe
    {  680, 1204,  925292,  925306,  925329, 14, 23, 21, }, // Magadan
    {  680, 1281,  925350,  925373,  925405, 23, 32, 30, }, // Mexico_Pacific
    {  680, 1305,  925435,  925447,  925468, 12, 21, 19, }, // Moscow
    {  680, 1358,  925487,  925508,  925538, 21, 30, 28, }, // Newfoundland
    {  680, 1384,  925566,  925580,  925603, 14, 23, 21, }, // Noronha
    {  680, 1406,  925624,  925642,  925669, 18, 27, 25, }, // Novosibirsk
    {  680, 1418,  925694,  925706,  925727, 12, 21, 19, }, // Omsk
    {  680, 1821,  925746,  925764,  925791, 18, 27, 25, }, // Vladivostok
    {  680, 1862,  925816,  925830,  925853, 14, 23, 21, }, // Yakutsk
    {  680, 1870,  925874,  925893,  925921, 19, 28, 26, }, // Yekaterinburg
    {  681,    5,       0,  925947,       0,  0, 17,  0, }, // Afghanistan Baluchi/Latin/Pakistan
    {  681,   17,       0,  925964,       0,  0, 21,  0, }, // Africa_Central
    {  681,   32,       0,  925985,       0,  0, 23,  0, }, // Africa_Eastern
    {  681,   65,       0,  926008,       0,  0, 30,  0, }, // Africa_Southern
    {  681,   81,  926038,  926059,  926087, 21, 28, 29, }, // Africa_Western
    {  681,  107,  926116,  926129,  926149, 13, 20, 21, }, // Alaska
    {  681,  135,  926170,  926183,  926203, 13, 20, 21, }, // Amazon
    {  681,  142,  926224,  926245,  926273, 21, 28, 29, }, // America_Central
    {  681,  158,  926302,  926325,  926355, 23, 30, 31, }, // America_Eastern
    {  681,  174,  926386,  926409,  926439, 23, 30, 31, }, // America_Mountain
    {  681,  191,  926470,  926492,  926521, 22, 29, 30, }, // America_Pacific
    {  681,  214,  926551,  926562,  926580, 11, 18, 17, }, // Apia
    {  681,  232,  926597,  926607,  926624, 10, 17, 16, }, // Arabian
    {  681,  240,  926640,  926656,  926679, 16, 23, 24, }, // Argentina
    {  681,  250,  926703,  926727,  926758, 24, 31, 32, }, // Argentina_Western
    {  681,  268,  926790,  926804,  926825, 14, 21, 22, }, // Armenia
    {  681,  286,  926847,  926861,  926882, 14, 21, 20, }, // Atlantic
    {  681,  295,  926902,  926925,  926955, 23, 30, 31, }, // Australia_Central
    {  681,  313,  926986,  927016,  927053, 30, 37, 38, }, // Australia_CentralWestern
    {  681,  338,  927091,  927116,  927148, 25, 32, 33, }, // Australia_Eastern
    {  681,  356,  927181,  927204,  927234, 23, 30, 31, }, // Australia_Western
    {  681,  374,  927265,  927282,  927306, 17, 24, 25, }, // Azerbaijan
    {  681,  385,  927331,  927344,  927364, 13, 20, 21, }, // Azores
    {  681,  397,  927385,  927403,  927428, 18, 25, 26, }, // Bangladesh
    {  681,  415,       0,  927454,       0,  0, 13,  0, }, // Bhutan
    {  681,  422,       0,  927467,       0,  0, 14,  0, }, // Bolivia
    {  681,  437,  927481,  927494,  927514, 13, 20, 21, }, // Brasilia
    {  681,  454,       0,  927535,       0,  0, 13,  0, }, // Brunei
    {  681,  461,  927548,  927564,  927587, 16, 23, 24, }, // Cape_Verde
    {  681,  478,       0,  927611,       0,  0, 15,  0, }, // Chamorro
    {  681,  487,  927626,  927637,  927655, 11, 18, 17, }, // Chatham
    {  681,  495,  927672,  927685,  927705, 13, 20, 21, }, // Chile
    {  681,  501,  927726,  927737,  927755, 11, 18, 17, }, // China
    {  681,  507,       0,  927772,       0,  0, 21,  0, }, // Christmas
    {  681,  517,       0,  927793,       0,  0, 19,  0, }, // Cocos
    {  681,  523,  927812,  927827,  927849, 15, 22, 23, }, // Colombia
    {  681,  532,  927872,  927889,  927913, 17, 24, 29, }, // Cook
    {  681,  537,  927942,  927954,  927973, 12, 19, 19, }, // Cuba
    {  681,  548,       0,  927992,       0,  0, 12,  0, }, // Davis
    {  681,  564,       0,  928004,       0,  0, 19,  0, }, // DumontDUrville
    {  681,  601,       0,  928023,       0,  0, 22,  0, }, // East_Timor
    {  681,  612,  928045,  928065,  928092, 20, 27, 28, }, // Easter
    {  681,  619,       0,  928120,       0,  0, 14,  0, }, // Ecuador
    {  681,  627,  928134,  928154,  928181, 20, 27, 28, }, // Europe_Central
    {  681,  642,  928209,  928231,  928260, 22, 29, 30, }, // Europe_Eastern
    {  681,  657,       0,  928290,       0,  0, 38,  0, }, // Europe_Further_Eastern
    {  681,  680,  928328,  928348,  928375, 20, 27, 28, }, // Europe_Western
    {  681,  695,  928403,  928425,  928454, 22, 29, 30, }, // Falkland
    {  681,  704,  928484,  928495,  928513, 11, 18, 19, }, // Fiji
    {  681,  709,       0,  928532,       0,  0, 20,  0, }, // French_Guiana
    {  681,  723,       0,  928552,       0,  0, 35,  0, }, // French_Southern
    {  681,  746,       0,  928587,       0,  0, 16,  0, }, // Galapagos
    {  681,  756,       0,  928603,       0,  0, 13,  0, }, // Gambier
    {  681,  764,  928616,  928629,  928649, 13, 20, 21, }, // Georgia
    {  681,  772,       0,  928670,       0,  0, 21,  0, }, // Gilbert_Islands
    {  681,  788,       0,  928691,       0,  0, 17,  0, }, // GMT
    {  681,  830,  928708,  928733,  928765, 25, 32, 33, }, // Greenland_Eastern
    {  681,  848,  928798,  928821,  928851, 23, 30, 29, }, // Greenland_Western
    {  681,  871,       0,  928880,       0,  0, 20,  0, }, // Gulf
    {  681,  876,       0,  928900,       0,  0, 13,  0, }, // Guyana
    {  681,  883,  928913,  928932,  928958, 19, 26, 27, }, // Hawaii_Aleutian
    {  681,  899,  928985,  929001,  929024, 16, 23, 24, }, // Hong_Kong
    {  681,  909,  929048,  929059,  929077, 11, 18, 19, }, // Hovd
    {  681,  914,       0,  929096,       0,  0, 18,  0, }, // India
    {  681,  920,       0,  929114,       0,  0, 16,  0, }, // Indian_Ocean
    {  681,  933,       0,  929130,       0,  0, 16,  0, }, // Indochina
    {  681,  943,       0,  929146,       0,  0, 33,  0, }, // Indonesia_Central
    {  681,  961,       0,  929179,       0,  0, 35,  0, }, // Indonesia_Eastern
    {  681,  979,       0,  929214,       0,  0, 33,  0, }, // Indonesia_Western
    {  681,  997,  929247,  929258,  929276, 11, 18, 17, }, // Iran
    {  681, 1008,  929293,  929306,  929326, 13, 20, 21, }, // Irkutsk
    {  681, 1016,  929347,  929360,  929380, 13, 20, 19, }, // Israel
    {  681, 1023,  929399,  929411,  929430, 12, 19, 18, }, // Japan
    {  681, 1047,       0,  929448,       0,  0, 17,  0, }, // Kazakhstan
    {  681, 1058,       0,  929465,       0,  0, 34,  0, }, // Kazakhstan_Eastern
    {  681, 1077,       0,  929499,       0,  0, 32,  0, }, // Kazakhstan_Western
    {  681, 1106,  929531,  929543,  929562, 12, 19, 18, }, // Korea
    {  681, 1112,       0,  929580,       0,  0, 13,  0, }, // Kosrae
    {  681, 1119,  929593,  929610,  929634, 17, 24, 25, }, // Krasnoyarsk
    {  681, 1151,       0,  929659,       0,  0, 18,  0, }, // Kyrgystan
    {  681, 1175,       0,  929677,       0,  0, 18,  0, }, // Line_Islands
    {  681, 1188,  929695,  929720,  929752, 25, 32, 33, }, // Lord_Howe
    {  681, 1204,  929785,  929799,  929820, 14, 21, 22, }, // Magadan
    {  681, 1219,       0,  929842,       0,  0, 15,  0, }, // Malaysia
    {  681, 1228,       0,  929857,       0,  0, 13,  0, }, // Maldives
    {  681, 1237,       0,  929870,       0,  0, 15,  0, }, // Marquesas
    {  681, 1247,       0,  929885,       0,  0, 21,  0, }, // Marshall_Islands
    {  681, 1264,  929906,  929922,  929945, 16, 23, 24, }, // Mauritius
    {  681, 1274,       0,  929969,       0,  0, 13,  0, }, // Mawson
    {  681, 1281,  929982,  930005,  930035, 23, 30, 31, }, // Mexico_Pacific
    {  681, 1296,  930066,  930082,  930105, 16, 23, 24, }, // Mongolia
    {  681, 1305,  930129,  930141,  930160, 12, 19, 20, }, // Moscow
    {  681, 1312,       0,  930180,       0,  0, 14,  0, }, // Myanmar
    {  681, 1320,       0,  930194,       0,  0, 12,  0, }, // Nauru
    {  681, 1326,       0,  930206,       0,  0, 12,  0, }, // Nepal
    {  681, 1332,  930218,  930238,  930265, 20, 27, 28, }, // New_Caledonia
    {  681, 1346,  930293,  930310,  930334, 17, 24, 23, }, // New_Zealand
    {  681, 1358,  930357,  930375,  930400, 18, 25, 26, }, // Newfoundland
    {  681, 1371,       0,  930426,       0,  0, 10,  0, }, // Niue
    {  681, 1376,  930436,  930457,  930485, 21, 28, 27, }, // Norfolk
    {  681, 1384,  930512,  930525,  930545, 13, 20, 21, }, // Noronha
    {  681, 1406,  930566,  930583,  930607, 17, 24, 25, }, // Novosibirsk
    {  681, 1418,  930632,  930643,  930661, 11, 18, 19, }, // Omsk
    {  681, 1428,  930680,  930695,  930717, 15, 22, 23, }, // Pakistan
    {  681, 1437,       0,  930740,       0,  0, 12,  0, }, // Palau
    {  681, 1443,       0,  930752,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  681, 1460,  930774,  930789,  930811, 15, 22, 23, }, // Paraguay
    {  681, 1469,  930834,  930845,  930863, 11, 18, 19, }, // Peru
    {  681, 1474,  930882,  930895,  930915, 13, 20, 21, }, // Philippines
    {  681, 1486,       0,  930936,       0,  0, 22,  0, }, // Phoenix_Islands
    {  681, 1502,  930958,  930982,  931012, 24, 30, 30, }, // Pierre_Miquelon
    {  681, 1518,       0,  931042,       0,  0, 15,  0, }, // Pitcairn
    {  681, 1527,       0,  931057,       0,  0, 12,  0, }, // Ponape
    {  681, 1534,       0,  931069,       0,  0, 16,  0, }, // Pyongyang
    {  681, 1554,       0,  931085,       0,  0, 15,  0, }, // Reunion
    {  681, 1562,       0,  931100,       0,  0, 14,  0, }, // Rothera
    {  681, 1570,  931114,  931129,  931151, 15, 22, 23, }, // Sakhalin
    {  681, 1596,  931174,  931187,  931207, 13, 20, 19, }, // Samoa
    {  681, 1602,       0,  931226,       0,  0, 15,  0, }, // Seychelles
    {  681, 1624,       0,  931241,       0,  0, 22,  0, }, // Singapore
    {  681, 1634,       0,  931263,       0,  0, 21,  0, }, // Solomon
    {  681, 1642,       0,  931284,       0,  0, 21,  0, }, // South_Georgia
    {  681, 1656,       0,  931305,       0,  0, 15,  0, }, // Suriname
    {  681, 1676,       0,  931320,       0,  0, 12,  0, }, // Syowa
    {  681, 1682,       0,  931332,       0,  0, 13,  0, }, // Tahiti
    {  681, 1689,  931345,  931357,  931376, 12, 19, 18, }, // Taipei
    {  681, 1696,       0,  931394,       0,  0, 17,  0, }, // Tajikistan
    {  681, 1724,       0,  931411,       0,  0, 14,  0, }, // Tokelau
    {  681, 1732,  931425,  931437,  931456, 12, 19, 20, }, // Tonga
    {  681, 1738,       0,  931476,       0,  0, 11,  0, }, // Truk
    {  681, 1750,  931487,  931506,  931532, 19, 26, 27, }, // Turkmenistan
    {  681, 1763,       0,  931559,       0,  0, 13,  0, }, // Tuvalu
    {  681, 1777,  931572,  931587,  931609, 15, 22, 23, }, // Uruguay
    {  681, 1792,  931632,  931649,  931673, 17, 24, 25, }, // Uzbekistan
    {  681, 1803,  931698,  931712,  931733, 14, 21, 22, }, // Vanuatu
    {  681, 1811,       0,  931755,       0,  0, 16,  0, }, // Venezuela
    {  681, 1821,  931771,  931790,  931816, 19, 26, 27, }, // Vladivostok
    {  681, 1833,  931843,  931859,  931882, 16, 23, 24, }, // Volgograd
    {  681, 1843,       0,  931906,       0,  0, 13,  0, }, // Vostok
    {  681, 1850,       0,  931919,       0,  0, 18,  0, }, // Wake
    {  681, 1855,       0,  931937,       0,  0, 22,  0, }, // Wallis
    {  681, 1862,  931959,  931972,  931992, 13, 20, 21, }, // Yakutsk
    {  681, 1870,  932013,  932032,  932058, 19, 26, 27, }, // Yekaterinburg
    {  681, 1892,       0,  932085,       0,  0, 10,  0, }, // Yukon
    {  682,    5,       0,  932095,       0,  0, 19,  0, }, // Afghanistan Ligurian/Latin/Italy
    {  682,   17,       0,  932114,       0,  0, 22,  0, }, // Africa_Central
    {  682,   32,       0,  932136,       0,  0, 25,  0, }, // Africa_Eastern
    {  682,   65,       0,  932161,       0,  0, 26,  0, }, // Africa_Southern
    {  682,   81,  932187,  932212,  932246, 25, 34, 32, }, // Africa_Western
    {  682,  107,  932278,  932292,  932315, 14, 23, 21, }, // Alaska
    {  682,  135,  932336,  932353,  932379, 17, 26, 24, }, // Amazon
    {  682,  142,  932403,  932427,  932460, 24, 33, 31, }, // America_Central
    {  682,  158,  932491,  932518,  932554, 27, 36, 34, }, // America_Eastern
    {  682,  174,  932588,  932611,  932643, 23, 32, 30, }, // America_Mountain
    {  682,  191,  932673,  932701,  932738, 28, 37, 35, }, // America_Pacific
    {  682,  214,  932773,  932783,  932802, 10, 19, 17, }, // Apia
    {  682,  232,  932819,  932833,  932856, 14, 23, 21, }, // Arabian
    {  682,  240,  932877,  932894,  932920, 17, 26, 24, }, // Argentina
    {  682,  250,  932944,  932972,  933009, 28, 37, 35, }, // Argentina_Western
    {  682,  268,  933044,  933059,  933083, 15, 24, 22, }, // Armenia
    {  682,  286,  933105,  933136,  933176, 31, 40, 38, }, // Atlantic
    {  682,  295,  933214,  933239,  933273, 25, 34, 32, }, // Australia_Central
    {  682,  313,  933305,  933338,  933380, 33, 42, 40, }, // Australia_CentralWestern
    {  682,  338,  933420,  933448,  933485, 28, 37, 35, }, // Australia_Eastern
    {  682,  356,  933520,  933548,  933585, 28, 37, 35, }, // Australia_Western
    {  682,  374,  933620,  933639,  933667, 19, 28, 26, }, // Azerbaijan
    {  682,  385,  933693,  933704,  933724, 11, 20, 18, }, // Azores
    {  682,  397,  933742,  933758,  933783, 16, 25, 23, }, // Bangladesh
    {  682,  415,       0,  933806,       0,  0, 12,  0, }, // Bhutan
    {  682,  422,       0,  933818,       0,  0, 13,  0, }, // Bolivia
    {  682,  437,  933831,  933845,  933868, 14, 23, 21, }, // Brasilia
    {  682,  454,       0,  933889,       0,  0, 23,  0, }, // Brunei
    {  682,  461,  933912,  933929,  933955, 17, 26, 24, }, // Cape_Verde
    {  682,  478,       0,  933979,       0,  0, 14,  0, }, // Chamorro
    {  682,  487,  933993,  934006,  934028, 13, 22, 20, }, // Chatham
    {  682,  495,  934048,  934058,  934077, 10, 19, 17, }, // Chile
    {  682,  501,  934094,  934104,  934123, 10, 19, 17, }, // China
    {  682,  507,       0,  934140,       0,  0, 22,  0, }, // Christmas
    {  682,  517,       0,  934162,       0,  0, 16,  0, }, // Cocos
    {  682,  523,  934178,  934192,  934215, 14, 23, 21, }, // Colombia
    {  682,  532,  934236,  934251,  934275, 15, 24, 22, }, // Cook
    {  682,  537,  934297,  934308,  934328, 11, 20, 18, }, // Cuba
    {  682,  548,       0,  934346,       0,  0, 11,  0, }, // Davis
    {  682,  564,       0,  934357,       0,  0, 22,  0, }, // DumontDUrville
    {  682,  601,       0,  934379,       0,  0, 15,  0, }, // East_Timor
    {  682,  612,  934394,  934416,  934447, 22, 31, 29, }, // Easter
    {  682,  619,       0,  934476,       0,  0, 15,  0, }, // Ecuador
    {  682,  627,  934491,  934513,  934544, 22, 31, 29, }, // Europe_Central
    {  682,  642,  934573,  934598,  934632, 25, 34, 32, }, // Europe_Eastern
    {  682,  657,       0,  934664,       0,  0, 17,  0, }, // Europe_Further_Eastern
    {  682,  680,  934681,  934706,  934740, 25, 34, 32, }, // Europe_Western
    {  682,  695,  934772,  934790,  934817, 18, 27, 25, }, // Falkland
    {  682,  704,  934842,  934852,  934871, 10, 19, 17, }, // Fiji
    {  682,  709,       0,  934888,       0,  0, 22,  0, }, // French_Guiana
    {  682,  723,       0,  934910,       0,  0, 42,  0, }, // French_Southern
    {  682,  746,       0,  934952,       0,  0, 15,  0, }, // Galapagos
    {  682,  756,       0,  934967,       0,  0, 13,  0, }, // Gambier
    {  682,  764,  934980,  934993,  935015, 13, 22, 20, }, // Georgia
    {  682,  772,       0,  935035,       0,  0, 18,  0, }, // Gilbert_Islands
    {  682,  788,       0,  935053,       0,  0, 27,  0, }, // GMT
    {  682,  830,  935080,  935108,  935145, 28, 37, 35, }, // Greenland_Eastern
    {  682,  848,  935180,  935208,  935245, 28, 37, 35, }, // Greenland_Western
    {  682,  871,       0,  935280,       0,  0, 20,  0, }, // Gulf
    {  682,  876,       0,  934888,       0,  0, 12,  0, }, // Guyana
    {  682,  883,  935300,  935321,  935351, 21, 30, 28, }, // Hawaii_Aleutian
    {  682,  899,  935379,  935394,  935418, 15, 24, 22, }, // Hong_Kong
    {  682,  909,  935440,  935450,  935469, 10, 19, 17, }, // Hovd
    {  682,  914,       0,  935486,       0,  0, 13,  0, }, // India
    {  682,  920,       0,  935499,       0,  0, 21,  0, }, // Indian_Ocean
    {  682,  933,       0,  935520,       0,  0, 16,  0, }, // Indochina
    {  682,  943,       0,  935536,       0,  0, 25,  0, }, // Indonesia_Central
    {  682,  961,       0,  935561,       0,  0, 28,  0, }, // Indonesia_Eastern
    {  682,  979,       0,  935589,       0,  0, 28,  0, }, // Indonesia_Western
    {  682,  997,  935617,  935629,  935650, 12, 21, 19, }, // Iran
    {  682, 1008,  935669,  935682,  935704, 13, 22, 20, }, // Irkutsk
    {  682, 1016,  935724,  935736,  935757, 12, 21, 19, }, // Israel
    {  682, 1023,  935776,  935789,  935811, 13, 22, 20, }, // Japan
    {  682, 1058,       0,  935831,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  682, 1077,       0,  935858,       0,  0, 27,  0, }, // Kazakhstan_Western
    {  682, 1106,  935885,  935896,  935916, 11, 20, 18, }, // Korea
    {  682, 1112,       0,  935934,       0,  0, 12,  0, }, // Kosrae
    {  682, 1119,  935946,  935963,  935989, 17, 26, 24, }, // Krasnoyarsk
    {  682, 1151,       0,  936013,       0,  0, 18,  0, }, // Kyrgystan
    {  682, 1175,       0,  936031,       0,  0, 19,  0, }, // Line_Islands
    {  682, 1188,  936050,  936065,  936089, 15, 24, 22, }, // Lord_Howe
    {  682, 1204,  936111,  936124,  936146, 13, 22, 20, }, // Magadan
    {  682, 1219,       0,  936166,       0,  0, 13,  0, }, // Malaysia
    {  682, 1228,       0,  936179,       0,  0, 13,  0, }, // Maldives
    {  682, 1237,       0,  936192,       0,  0, 15,  0, }, // Marquesas
    {  682, 1247,       0,  936207,       0,  0, 19,  0, }, // Marshall_Islands
    {  682, 1264,  936226,  936241,  936265, 15, 24, 22, }, // Mauritius
    {  682, 1274,       0,  936287,       0,  0, 12,  0, }, // Mawson
    {  682, 1281,  936299,  936323,  936356, 24, 33, 31, }, // Mexico_Pacific
    {  682, 1296,  936387,  936402,  936426, 15, 24, 22, }, // Mongolia
    {  682, 1305,  936448,  936459,  936479, 11, 20, 18, }, // Moscow
    {  682, 1312,       0,  936497,       0,  0, 14,  0, }, // Myanmar
    {  682, 1320,       0,  936511,       0,  0, 11,  0, }, // Nauru
    {  682, 1326,       0,  936522,       0,  0, 11,  0, }, // Nepal
    {  682, 1332,  936533,  936554,  936584, 21, 30, 28, }, // New_Caledonia
    {  682, 1346,  936612,  936631,  936659, 19, 28, 26, }, // New_Zealand
    {  682, 1358,  936685,  936700,  936724, 15, 24, 22, }, // Newfoundland
    {  682, 1371,       0,  936746,       0,  0, 10,  0, }, // Niue
    {  682, 1376,  936756,  936776,  936805, 20, 29, 27, }, // Norfolk
    {  682, 1384,  936832,  936857,  936891, 25, 34, 32, }, // Noronha
    {  682, 1406,  936923,  936940,  936966, 17, 26, 24, }, // Novosibirsk
    {  682, 1418,  936990,  937000,  937019, 10, 19, 17, }, // Omsk
    {  682, 1428,  937036,  937050,  937073, 14, 23, 21, }, // Pakistan
    {  682, 1437,       0,  937094,       0,  0, 11,  0, }, // Palau
    {  682, 1443,       0,  937105,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  682, 1460,  937129,  937143,  937166, 14, 23, 21, }, // Paraguay
    {  682, 1469,  937187,  937197,  937216, 10, 19, 17, }, // Peru
    {  682, 1474,  937233,  937247,  937270, 14, 23, 21, }, // Philippines
    {  682, 1486,       0,  937291,       0,  0, 18,  0, }, // Phoenix_Islands
    {  682, 1502,  937309,  937332,  937364, 23, 32, 30, }, // Pierre_Miquelon
    {  682, 1518,       0,  937394,       0,  0, 14,  0, }, // Pitcairn
    {  682, 1527,       0,  937408,       0,  0, 13,  0, }, // Ponape
    {  682, 1534,       0,  937421,       0,  0, 15,  0, }, // Pyongyang
    {  682, 1554,       0,  937436,       0,  0, 13,  0, }, // Reunion
    {  682, 1562,       0,  937449,       0,  0, 13,  0, }, // Rothera
    {  682, 1570,  937462,  937476,  937499, 14, 23, 21, }, // Sakhalin
    {  682, 1596,  937520,  937531,  937551, 11, 20, 18, }, // Samoa
    {  682, 1602,       0,  937569,       0,  0, 16,  0, }, // Seychelles
    {  682, 1624,       0,  937585,       0,  0, 14,  0, }, // Singapore
    {  682, 1634,       0,  937599,       0,  0, 18,  0, }, // Solomon
    {  682, 1642,       0,  937617,       0,  0, 20,  0, }, // South_Georgia
    {  682, 1656,       0,  937637,       0,  0, 14,  0, }, // Suriname
    {  682, 1676,       0,  937651,       0,  0, 11,  0, }, // Syowa
    {  682, 1682,       0,  937662,       0,  0, 12,  0, }, // Tahiti
    {  682, 1689,  937674,  937686,  937707, 12, 21, 19, }, // Taipei
    {  682, 1696,       0,  937726,       0,  0, 16,  0, }, // Tajikistan
    {  682, 1724,       0,  937742,       0,  0, 13,  0, }, // Tokelau
    {  682, 1732,  937755,  937766,  937786, 11, 20, 18, }, // Tonga
    {  682, 1738,       0,  937804,       0,  0, 11,  0, }, // Truk
    {  682, 1750,  937815,  937833,  937860, 18, 27, 25, }, // Turkmenistan
    {  682, 1763,       0,  937885,       0,  0, 12,  0, }, // Tuvalu
    {  682, 1777,  937897,  937912,  937936, 15, 24, 22, }, // Uruguay
    {  682, 1792,  937958,  937976,  938003, 18, 27, 25, }, // Uzbekistan
    {  682, 1803,  938028,  938041,  938063, 13, 22, 20, }, // Vanuatu
    {  682, 1811,       0,  938083,       0,  0, 16,  0, }, // Venezuela
    {  682, 1821,  938099,  938116,  938142, 17, 26, 24, }, // Vladivostok
    {  682, 1833,  938166,  938181,  938205, 15, 24, 22, }, // Volgograd
    {  682, 1843,       0,  938227,       0,  0, 12,  0, }, // Vostok
    {  682, 1850,       0,  938239,       0,  0, 20,  0, }, // Wake
    {  682, 1855,       0,  938259,       0,  0, 21,  0, }, // Wallis
    {  682, 1862,  938280,  938293,  938315, 13, 22, 20, }, // Yakutsk
    {  682, 1870,  938335,  938352,  938378, 17, 26, 24, }, // Yekaterinburg
    {  682, 1892,       0,  938402,       0,  0, 11,  0, }, // Yukon
    {  685,    5,       0,  938413,       0,  0, 16,  0, }, // Afghanistan Torwali/Arabic/Pakistan
    {  685,   17,       0,  834330,       0,  0, 16,  0, }, // Africa_Central
    {  685,   32,       0,  834346,       0,  0, 17,  0, }, // Africa_Eastern
    {  685,   65,       0,  834363,       0,  0, 25,  0, }, // Africa_Southern
    {  685,   81,  834388,  834405,  834430, 17, 25, 21, }, // Africa_Western
    {  685,  107,  834451,  834462,  834482, 11, 20, 19, }, // Alaska
    {  685,  135,  834501,  938429,  938450, 11, 21, 26, }, // Amazon
    {  685,  142,  834559,  834569,  834588, 10, 19, 18, }, // America_Central
    {  685,  158,  834606,  834617,  834637, 11, 20, 19, }, // America_Eastern
    {  685,  174,  834656,  834668,  834689, 12, 21, 20, }, // America_Mountain
    {  685,  191,  834709,  834719,  834738, 10, 19, 18, }, // America_Pacific
    {  685,  214,  834799,  834809,  834827, 10, 18, 18, }, // Apia
    {  685,  232,  938476,  938486,  834872, 10, 17, 16, }, // Arabian
    {  685,  240,  938503,  938518,  938540, 15, 22, 28, }, // Argentina
    {  685,  250,  938568,  938589,  938617, 21, 28, 34, }, // Argentina_Western
    {  685,  268,  938651,  938665,  938686, 14, 21, 27, }, // Armenia
    {  685,  286,  835098,  835110,  835131, 12, 21, 20, }, // Atlantic
    {  685,  295,  835151,  835170,  835198, 19, 28, 27, }, // Australia_Central
    {  685,  313,  835225,  835251,  835286, 26, 35, 34, }, // Australia_CentralWestern
    {  685,  338,  835320,  835340,  835369, 20, 29, 28, }, // Australia_Eastern
    {  685,  356,  835397,  835418,  835446, 20, 28, 28, }, // Australia_Western
    {  685,  374,  938713,  938730,  938754, 17, 24, 30, }, // Azerbaijan
    {  685,  385,  938784,  938797,  938817, 13, 20, 26, }, // Azores
    {  685,  397,  938843,  938859,  938882, 16, 23, 29, }, // Bangladesh
    {  685,  415,       0,  938911,       0,  0, 13,  0, }, // Bhutan
    {  685,  422,       0,  938924,       0,  0, 14,  0, }, // Bolivia
    {  685,  437,  835699,  835712,  835734, 13, 22, 17, }, // Brasilia
    {  685,  454,       0,  835751,       0,  0, 21,  0, }, // Brunei
    {  685,  461,  835772,  835785,  835806, 13, 21, 17, }, // Cape_Verde
    {  685,  478,       0,  835823,       0,  0, 19,  0, }, // Chamorro
    {  685,  487,  835842,  835852,  835871, 10, 19, 18, }, // Chatham
    {  685,  495,  938938,  938948,  938965, 10, 17, 23, }, // Chile
    {  685,  501,  938988,  835949,  835965, 10, 16, 17, }, // China
    {  685,  507,       0,  835982,       0,  0, 17,  0, }, // Christmas
    {  685,  517,       0,  835999,       0,  0, 18,  0, }, // Cocos
    {  685,  523,  836017,  938998,  939019, 12, 21, 27, }, // Colombia
    {  685,  532,  836077,  836092,  836115, 15, 23, 23, }, // Cook
    {  685,  537,  836138,  836148,  836167, 10, 19, 18, }, // Cuba
    {  685,  548,       0,  836185,       0,  0,  9,  0, }, // Davis
    {  685,  564,       0,  836194,       0,  0, 21,  0, }, // DumontDUrville
    {  685,  601,       0,  836215,       0,  0, 16,  0, }, // East_Timor
    {  685,  612,  939046,  939065,  939091, 19, 26, 32, }, // Easter
    {  685,  619,       0,  939123,       0,  0, 15,  0, }, // Ecuador
    {  685,  627,  939138,  939153,  939176, 15, 23, 29, }, // Europe_Central
    {  685,  642,  939205,  939222,  939246, 17, 24, 30, }, // Europe_Eastern
    {  685,  657,       0,  939276,       0,  0, 20,  0, }, // Europe_Further_Eastern
    {  685,  680,  939296,  939313,  939337, 17, 24, 30, }, // Europe_Western
    {  685,  695,  939367,  939390,  939420, 23, 30, 36, }, // Falkland
    {  685,  704,  836641,  836649,  836665,  8, 16, 12, }, // Fiji
    {  685,  709,       0,  939456,       0,  0, 18,  0, }, // French_Guiana
    {  685,  723,       0,  939474,       0,  0, 28,  0, }, // French_Southern
    {  685,  746,       0,  939502,       0,  0, 16,  0, }, // Galapagos
    {  685,  756,       0,  836740,       0,  0, 12,  0, }, // Gambier
    {  685,  764,  939518,  939531,  939551, 13, 20, 26, }, // Georgia
    {  685,  772,       0,  836811,       0,  0, 18,  0, }, // Gilbert_Islands
    {  685,  788,       0,  939577,       0,  0, 18,  0, }, // GMT
    {  685,  830,  836847,  836867,  939595, 20, 29, 35, }, // Greenland_Eastern
    {  685,  848,  836931,  836951,  939630, 20, 29, 35, }, // Greenland_Western
    {  685,  871,       0,  939665,       0,  0, 18,  0, }, // Gulf
    {  685,  876,       0,  939462,       0,  0, 12,  0, }, // Guyana
    {  685,  883,  837033,  837052,  837080, 19, 28, 27, }, // Hawaii_Aleutian
    {  685,  899,  939683,  939697,  939719, 14, 22, 18, }, // Hong_Kong
    {  685,  909,  837125,  837134,  837151,  9, 17, 13, }, // Hovd
    {  685,  914,       0,  939737,       0,  0, 22,  0, }, // India
    {  685,  920,       0,  837186,       0,  0, 12,  0, }, // Indian_Ocean
    {  685,  933,       0,  837198,       0,  0, 12,  0, }, // Indochina
    {  685,  943,       0,  837210,       0,  0, 19,  0, }, // Indonesia_Central
    {  685,  961,       0,  837229,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  685,  979,       0,  837249,       0,  0, 20,  0, }, // Indonesia_Western
    {  685,  997,  939759,  939771,  837300, 12, 19, 18, }, // Iran
    {  685, 1008,  837318,  837329,  837348, 11, 19, 15, }, // Irkutsk
    {  685, 1016,  939790,  939804,  837398, 14, 21, 20, }, // Israel
    {  685, 1023,  837418,  837428,  837446, 10, 18, 18, }, // Japan
    {  685, 1058,       0,  939825,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {  685, 1077,       0,  939846,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  685, 1106,  837612,  837622,  837640, 10, 18, 18, }, // Korea
    {  685, 1112,       0,  837658,       0,  0, 10,  0, }, // Kosrae
    {  685, 1119,  837668,  837684,  837707, 16, 23, 20, }, // Krasnoyarsk
    {  685, 1151,       0,  939867,       0,  0, 14,  0, }, // Kyrgystan
    {  685, 1175,       0,  837741,       0,  0, 17,  0, }, // Line_Islands
    {  685, 1188,  837758,  837772,  837795, 14, 23, 22, }, // Lord_Howe
    {  685, 1204,  837817,  837828,  837848, 11, 20, 15, }, // Magadan
    {  685, 1219,       0,  837863,       0,  0, 11,  0, }, // Malaysia
    {  685, 1228,       0,  939881,       0,  0, 13,  0, }, // Maldives
    {  685, 1237,       0,  837887,       0,  0, 12,  0, }, // Marquesas
    {  685, 1247,       0,  837899,       0,  0, 18,  0, }, // Marshall_Islands
    {  685, 1264,  837917,  837928,  837947, 11, 19, 15, }, // Mauritius
    {  685, 1274,       0,  837962,       0,  0, 10,  0, }, // Mawson
    {  685, 1281,  837972,  837990,  838017, 18, 27, 26, }, // Mexico_Pacific
    {  685, 1296,  838043,  838059,  838083, 16, 24, 20, }, // Mongolia
    {  685, 1305,  838103,  838113,  838132, 10, 19, 14, }, // Moscow
    {  685, 1312,       0,  838146,       0,  0, 12,  0, }, // Myanmar
    {  685, 1320,       0,  838158,       0,  0, 10,  0, }, // Nauru
    {  685, 1326,       0,  939894,       0,  0, 12,  0, }, // Nepal
    {  685, 1332,  838180,  838198,  838224, 18, 26, 22, }, // New_Caledonia
    {  685, 1346,  939906,  939923,  838287, 17, 24, 23, }, // New_Zealand
    {  685, 1358,  414925,  838310,  414971, 19, 28, 27, }, // Newfoundland
    {  685, 1371,       0,  838338,       0,  0,  9,  0, }, // Niue
    {  685, 1376,  939947,  939967,  939994, 20, 27, 33, }, // Norfolk
    {  685, 1384,  940027,  940051,  838474, 24, 31, 27, }, // Noronha
    {  685, 1406,  838501,  838516,  838539, 15, 23, 19, }, // Novosibirsk
    {  685, 1418,  838558,  838568,  838586, 10, 18, 14, }, // Omsk
    {  685, 1428,  940082,  940096,  940117, 14, 21, 27, }, // Pakistan
    {  685, 1437,       0,  838662,       0,  0,  9,  0, }, // Palau
    {  685, 1443,       0,  838671,       0,  0, 18,  0, }, // Papua_New_Guinea
    {  685, 1460,  940144,  940159,  940181, 15, 22, 28, }, // Paraguay
    {  685, 1469,  940209,  940220,  940238, 11, 18, 24, }, // Peru
    {  685, 1474,  838807,  838818,  838837, 11, 19, 15, }, // Philippines
    {  685, 1486,       0,  838852,       0,  0, 18,  0, }, // Phoenix_Islands
    {  685, 1502,  940262,  940287,  940321, 25, 34, 33, }, // Pierre_Miquelon
    {  685, 1518,       0,  838965,       0,  0, 12,  0, }, // Pitcairn
    {  685, 1527,       0,  838977,       0,  0, 11,  0, }, // Ponape
    {  685, 1534,       0,  940354,       0,  0, 14,  0, }, // Pyongyang
    {  685, 1554,       0,  839002,       0,  0, 13,  0, }, // Reunion
    {  685, 1562,       0,  940368,       0,  0, 14,  0, }, // Rothera
    {  685, 1570,  839029,  839040,  839059, 11, 19, 15, }, // Sakhalin
    {  685, 1596,  839117,  839127,  839145, 10, 18, 18, }, // Samoa
    {  685, 1602,       0,  839163,       0,  0, 11,  0, }, // Seychelles
    {  685, 1624,       0,  839174,       0,  0, 20,  0, }, // Singapore
    {  685, 1634,       0,  839194,       0,  0, 18,  0, }, // Solomon
    {  685, 1642,       0,  839212,       0,  0, 17,  0, }, // South_Georgia
    {  685, 1656,       0,  940382,       0,  0, 14,  0, }, // Suriname
    {  685, 1676,       0,  839243,       0,  0, 10,  0, }, // Syowa
    {  685, 1682,       0,  839253,       0,  0, 11,  0, }, // Tahiti
    {  685, 1689,  839264,  839278,  839301, 14, 23, 21, }, // Taipei
    {  685, 1696,       0,  940396,       0,  0, 15,  0, }, // Tajikistan
    {  685, 1724,       0,  839337,       0,  0, 12,  0, }, // Tokelau
    {  685, 1732,  839349,  839359,  839377, 10, 18, 14, }, // Tonga
    {  685, 1738,       0,  839391,       0,  0,  8,  0, }, // Truk
    {  685, 1750,  940411,  940428,  940452, 17, 24, 30, }, // Turkmenistan
    {  685, 1763,       0,  839470,       0,  0, 10,  0, }, // Tuvalu
    {  685, 1777,  940482,  940497,  940519, 15, 22, 28, }, // Uruguay
    {  685, 1792,  940547,  940562,  940584, 15, 22, 28, }, // Uzbekistan
    {  685, 1803,  839610,  839622,  839642, 12, 20, 16, }, // Vanuatu
    {  685, 1811,       0,  940612,       0,  0, 16,  0, }, // Venezuela
    {  685, 1821,  839674,  839690,  839714, 16, 24, 20, }, // Vladivostok
    {  685, 1833,  839734,  839748,  839771, 14, 23, 18, }, // Volgograd
    {  685, 1843,       0,  940628,       0,  0, 13,  0, }, // Vostok
    {  685, 1850,       0,  839802,       0,  0, 15,  0, }, // Wake
    {  685, 1855,       0,  839817,       0,  0, 20,  0, }, // Wallis
    {  685, 1862,  839837,  839848,  839868, 11, 20, 15, }, // Yakutsk
    {  685, 1870,  839883,  839899,  839924, 16, 25, 20, }, // Yekaterinburg
    {  685, 1892,       0,  416728,       0,  0, 10,  0, }, // Yukon
    {  686,    5,       0,  940641,       0,  0, 22,  0, }, // Afghanistan Anii/Latin/Benin
    {  686,   17,       0,  940663,       0,  0, 31,  0, }, // Africa_Central
    {  686,   32,       0,  940694,       0,  0, 33,  0, }, // Africa_Eastern
    {  686,   65,       0,  940727,       0,  0, 36,  0, }, // Africa_Southern
    {  686,   81,  940763,  940800,  940844, 37, 44, 46, }, // Africa_Western
    {  686,  107,  940890,  940909,  940935, 19, 26, 28, }, // Alaska
    {  686,  135,  940963,  940981,  941006, 18, 25, 27, }, // Amazon
    {  686,  142,  941033,  941075,  941124, 42, 49, 51, }, // America_Central
    {  686,  158,  941175,  941218,  941268, 43, 50, 52, }, // America_Eastern
    {  686,  174,  941320,  941359,  941405, 39, 46, 48, }, // America_Mountain
    {  686,  191,  941453,  941483,  941520, 30, 37, 39, }, // America_Pacific
    {  686,  214,  941559,  941576,  941600, 17, 24, 26, }, // Apia
    {  686,  232,  941626,  941651,  941683, 25, 32, 34, }, // Arabian
    {  686,  240,  941717,  941737,  941764, 20, 27, 29, }, // Argentina
    {  686,  250,  941793,  941827,  941868, 34, 41, 43, }, // Argentina_Western
    {  686,  268,  941911,  941930,  941956, 19, 26, 28, }, // Armenia
    {  686,  286,  941984,  942015,  942053, 31, 38, 40, }, // Atlantic
    {  686,  295,  942093,  942124,  942162, 31, 38, 40, }, // Australia_Central
    {  686,  313,  942202,  942247,  942299, 45, 52, 54, }, // Australia_CentralWestern
    {  686,  338,  942353,  942385,  942424, 32, 39, 41, }, // Australia_Eastern
    {  686,  356,  942465,  942502,  942546, 37, 44, 46, }, // Australia_Western
    {  686,  374,  942592,  942614,  942643, 22, 29, 31, }, // Azerbaijan
    {  686,  385,  942674,  942691,  942715, 17, 24, 26, }, // Azores
    {  686,  397,  942741,  942765,  942796, 24, 31, 33, }, // Bangladesh
    {  686,  415,       0,  942829,       0,  0, 17,  0, }, // Bhutan
    {  686,  422,       0,  942846,       0,  0, 20,  0, }, // Bolivia
    {  686,  437,  942866,  942887,  942915, 21, 28, 30, }, // Brasilia
    {  686,  454,       0,  942945,       0,  0, 29,  0, }, // Brunei
    {  686,  461,  942974,  942993,  943019, 19, 26, 28, }, // Cape_Verde
    {  686,  478,       0,  943047,       0,  0, 20,  0, }, // Chamorro
    {  686,  487,  943067,  943085,  943110, 18, 25, 27, }, // Chatham
    {  686,  495,  943137,  943155,  943180, 18, 25, 27, }, // Chile
    {  686,  501,  943207,  943224,  943248, 17, 24, 26, }, // China
    {  686,  507,       0,  943274,       0,  0, 30,  0, }, // Christmas
    {  686,  517,       0,  943304,       0,  0, 32,  0, }, // Cocos
    {  686,  523,  943336,  943356,  943383, 20, 27, 29, }, // Colombia
    {  686,  532,  943412,  943442,  943479, 30, 37, 39, }, // Cook
    {  686,  537,  943518,  943535,  943559, 17, 24, 26, }, // Cuba
    {  686,  548,       0,  943585,       0,  0, 17,  0, }, // Davis
    {  686,  564,       0,  943602,       0,  0, 24,  0, }, // DumontDUrville
    {  686,  601,       0,  943626,       0,  0, 27,  0, }, // East_Timor
    {  686,  612,  943653,  943684,  943721, 31, 37, 39, }, // Easter
    {  686,  619,       0,  943760,       0,  0, 21,  0, }, // Ecuador
    {  686,  627,  943781,  943812,  943850, 31, 38, 40, }, // Europe_Central
    {  686,  642,  943890,  943923,  943963, 33, 40, 42, }, // Europe_Eastern
    {  686,  657,       0,  944005,       0,  0, 44,  0, }, // Europe_Further_Eastern
    {  686,  680,  944049,  944086,  944130, 37, 44, 46, }, // Europe_Western
    {  686,  695,  944176,  944209,  944249, 33, 40, 42, }, // Falkland
    {  686,  704,  944291,  944307,  944330, 16, 23, 25, }, // Fiji
    {  686,  709,       0,  944355,       0,  0, 34,  0, }, // French_Guiana
    {  686,  723,       0,  944389,       0,  0, 49,  0, }, // French_Southern
    {  686,  746,       0,  944438,       0,  0, 21,  0, }, // Galapagos
    {  686,  756,       0,  944459,       0,  0, 20,  0, }, // Gambier
    {  686,  764,  944479,  944498,  944524, 19, 26, 28, }, // Georgia
    {  686,  772,       0,  944552,       0,  0, 33,  0, }, // Gilbert_Islands
    {  686,  788,       0,  944585,       0,  0, 21,  0, }, // GMT
    {  686,  830,  944606,  944636,  944673, 30, 37, 39, }, // Greenland_Eastern
    {  686,  848,  944712,  944746,  944787, 34, 41, 43, }, // Greenland_Western
    {  686,  866,       0,  944830,       0,  0, 17,  0, }, // Guam
    {  686,  871,       0,  944847,       0,  0, 16,  0, }, // Gulf
    {  686,  876,       0,  944863,       0,  0, 19,  0, }, // Guyana
    {  686,  883,  944882,  944911,  944947, 29, 36, 38, }, // Hawaii_Aleutian
    {  686,  899,  944985,  945004,  945030, 19, 26, 28, }, // Hong_Kong
    {  686,  909,  945058,  945075,  945099, 17, 24, 26, }, // Hovd
    {  686,  914,       0,  945125,       0,  0, 18,  0, }, // India
    {  686,  920,       0,  945143,       0,  0, 26,  0, }, // Indian_Ocean
    {  686,  933,       0,  945169,       0,  0, 21,  0, }, // Indochina
    {  686,  943,       0,  945190,       0,  0, 31,  0, }, // Indonesia_Central
    {  686,  961,       0,  945221,       0,  0, 32,  0, }, // Indonesia_Eastern
    {  686,  979,       0,  945253,       0,  0, 37,  0, }, // Indonesia_Western
    {  686,  997,  945290,  945306,  945329, 16, 23, 25, }, // Iran
    {  686, 1008,  945354,  945371,  945395, 17, 24, 26, }, // Irkutsk
    {  686, 1016,  945421,  945441,  945468, 20, 27, 29, }, // Israel
    {  686, 1023,  945497,  945514,  945538, 17, 24, 26, }, // Japan
    {  686, 1047,       0,  945564,       0,  0, 20,  0, }, // Kazakhstan
    {  686, 1058,       0,  945584,       0,  0, 30,  0, }, // Kazakhstan_Eastern
    {  686, 1077,       0,  945614,       0,  0, 34,  0, }, // Kazakhstan_Western
    {  686, 1106,  945648,  945665,  945689, 17, 24, 26, }, // Korea
    {  686, 1112,       0,  945715,       0,  0, 18,  0, }, // Kosrae
    {  686, 1119,  945733,  945755,  945784, 22, 29, 31, }, // Krasnoyarsk
    {  686, 1151,       0,  945815,       0,  0, 21,  0, }, // Kyrgystan
    {  686, 1161,       0,  945836,       0,  0, 23,  0, }, // Lanka
    {  686, 1175,       0,  945859,       0,  0, 30,  0, }, // Line_Islands
    {  686, 1188,  945889,  945909,  945936, 20, 27, 29, }, // Lord_Howe
    {  686, 1198,  945965,  945984,  946010, 19, 26, 27, }, // Macau
    {  686, 1204,  946037,  946056,  946082, 19, 26, 28, }, // Magadan
    {  686, 1219,       0,  946110,       0,  0, 19,  0, }, // Malaysia
    {  686, 1228,       0,  946129,       0,  0, 20,  0, }, // Maldives
    {  686, 1237,       0,  946149,       0,  0, 20,  0, }, // Marquesas
    {  686, 1247,       0,  946169,       0,  0, 33,  0, }, // Marshall_Islands
    {  686, 1264,  946202,  946220,  946245, 18, 25, 27, }, // Mauritius
    {  686, 1274,       0,  946272,       0,  0, 17,  0, }, // Mawson
    {  686, 1281,  946289,  946317,  946352, 28, 35, 37, }, // Mexico_Pacific
    {  686, 1296,  946389,  946411,  946440, 22, 29, 31, }, // Mongolia
    {  686, 1305,  946471,  946489,  946514, 18, 25, 27, }, // Moscow
    {  686, 1312,       0,  946541,       0,  0, 20,  0, }, // Myanmar
    {  686, 1320,       0,  946561,       0,  0, 18,  0, }, // Nauru
    {  686, 1326,       0,  946579,       0,  0, 18,  0, }, // Nepal
    {  686, 1332,  946597,  946624,  946658, 27, 34, 36, }, // New_Caledonia
    {  686, 1346,  946694,  946718,  946749, 24, 31, 33, }, // New_Zealand
    {  686, 1358,  946782,  946809,  946843, 27, 34, 36, }, // Newfoundland
    {  686, 1371,       0,  946879,       0,  0, 18,  0, }, // Niue
    {  686, 1376,  946897,  946929,  946968, 32, 39, 41, }, // Norfolk
    {  686, 1384,  947009,  947040,  947078, 31, 38, 40, }, // Noronha
    {  686, 1392,       0,  947118,       0,  0, 46,  0, }, // North_Mariana
    {  686, 1406,  947164,  947186,  947215, 22, 29, 31, }, // Novosibirsk
    {  686, 1418,  947246,  947264,  947289, 18, 25, 27, }, // Omsk
    {  686, 1428,  947316,  947336,  947363, 20, 27, 29, }, // Pakistan
    {  686, 1437,       0,  947392,       0,  0, 19,  0, }, // Palau
    {  686, 1443,       0,  947411,       0,  0, 32,  0, }, // Papua_New_Guinea
    {  686, 1460,  947443,  947464,  947492, 21, 28, 30, }, // Paraguay
    {  686, 1469,  947522,  947539,  947563, 17, 24, 26, }, // Peru
    {  686, 1474,  947589,  947609,  947636, 20, 27, 29, }, // Philippines
    {  686, 1486,       0,  947665,       0,  0, 33,  0, }, // Phoenix_Islands
    {  686, 1502,  947698,  947731,  947771, 33, 40, 42, }, // Pierre_Miquelon
    {  686, 1518,       0,  947813,       0,  0, 33,  0, }, // Pitcairn
    {  686, 1527,       0,  947846,       0,  0, 19,  0, }, // Ponape
    {  686, 1534,       0,  947865,       0,  0, 20,  0, }, // Pyongyang
    {  686, 1554,       0,  947885,       0,  0, 20,  0, }, // Reunion
    {  686, 1562,       0,  947905,       0,  0, 19,  0, }, // Rothera
    {  686, 1570,  947924,  947943,  947969, 19, 26, 28, }, // Sakhalin
    {  686, 1596,  947997,  948015,  948040, 18, 25, 27, }, // Samoa
    {  686, 1602,       0,  948067,       0,  0, 18,  0, }, // Seychelles
    {  686, 1624,       0,  948085,       0,  0, 21,  0, }, // Singapore
    {  686, 1634,       0,  948106,       0,  0, 34,  0, }, // Solomon
    {  686, 1642,       0,  948140,       0,  0, 29,  0, }, // South_Georgia
    {  686, 1656,       0,  948169,       0,  0, 19,  0, }, // Suriname
    {  686, 1676,       0,  948188,       0,  0, 19,  0, }, // Syowa
    {  686, 1682,       0,  948207,       0,  0, 19,  0, }, // Tahiti
    {  686, 1689,  948226,  948244,  948269, 18, 25, 27, }, // Taipei
    {  686, 1696,       0,  948296,       0,  0, 22,  0, }, // Tajikistan
    {  686, 1724,       0,  948318,       0,  0, 19,  0, }, // Tokelau
    {  686, 1732,  948337,  948354,  948378, 17, 24, 26, }, // Tonga
    {  686, 1738,       0,  948404,       0,  0, 16,  0, }, // Truk
    {  686, 1750,  948420,  948444,  948475, 24, 31, 33, }, // Turkmenistan
    {  686, 1763,       0,  948508,       0,  0, 18,  0, }, // Tuvalu
    {  686, 1777,  948526,  948546,  948573, 20, 27, 29, }, // Uruguay
    {  686, 1792,  948602,  948625,  948655, 23, 30, 32, }, // Uzbekistan
    {  686, 1803,  948687,  948707,  948734, 20, 27, 29, }, // Vanuatu
    {  686, 1811,       0,  948763,       0,  0, 23,  0, }, // Venezuela
    {  686, 1821,  948786,  948809,  948839, 23, 30, 32, }, // Vladivostok
    {  686, 1833,  948871,  948893,  948922, 22, 29, 31, }, // Volgograd
    {  686, 1843,       0,  948953,       0,  0, 18,  0, }, // Vostok
    {  686, 1850,       0,  948971,       0,  0, 30,  0, }, // Wake
    {  686, 1855,       0,  949001,       0,  0, 27,  0, }, // Wallis
    {  686, 1862,  949028,  949045,  949069, 17, 24, 26, }, // Yakutsk
    {  686, 1870,  949095,  949119,  949150, 24, 31, 33, }, // Yekaterinburg
    {  686, 1892,       0,  949183,       0,  0, 17,  0, }, // Yukon
    {  687,    5,       0,  949200,       0,  0, 20,  0, }, // Afghanistan Kangri/Devanagari/India
    {  687,   17,       0,  949220,       0,  0, 20,  0, }, // Africa_Central
    {  687,   32,       0,  949240,       0,  0, 21,  0, }, // Africa_Eastern
    {  687,   65,       0,  949261,       0,  0, 24,  0, }, // Africa_Southern
    {  687,   81,  949285,  949307,  949334, 22, 27, 33, }, // Africa_Western
    {  687,  107,  949367,  949381,  949401, 14, 20, 35, }, // Alaska
    {  687,  135,  949436,  949450,  949469, 14, 19, 25, }, // Amazon
    {  687,  142,  949494,  949523,  949557, 29, 34, 49, }, // America_Central
    {  687,  158,  949606,  949629,  949660, 23, 31, 46, }, // America_Eastern
    {  687,  174,  949706,  949733,  949766, 27, 33, 48, }, // America_Mountain
    {  687,  191,  949814,  949844,  949879, 30, 35, 50, }, // America_Pacific
    {  687,  214,  949929,  949940,  949956, 11, 16, 31, }, // Apia
    {  687,  232,  949987,  949997,  950012, 10, 15, 30, }, // Arabian
    {  687,  240,  950042,  950059,  950081, 17, 22, 28, }, // Argentina
    {  687,  250,  950109,  950134,  950164, 25, 30, 36, }, // Argentina_Western
    {  687,  268,  950200,  950216,  950237, 16, 21, 27, }, // Armenia
    {  687,  286,  950264,  950279,  950299, 15, 20, 35, }, // Atlantic
    {  687,  295,  950334,  950358,  950394, 24, 36, 51, }, // Australia_Central
    {  687,  313,  950445,  950482,  950523, 37, 41, 54, }, // Australia_CentralWestern
    {  687,  338,  950577,  950601,  950632, 24, 31, 46, }, // Australia_Eastern
    {  687,  356,  950678,  950704,  950735, 26, 31, 46, }, // Australia_Western
    {  687,  374,  950781,  950797,  950818, 16, 21, 27, }, // Azerbaijan
    {  687,  385,  950845,  950859,  950878, 14, 19, 25, }, // Azores
    {  687,  397,  950903,  950921,  950944, 18, 23, 29, }, // Bangladesh
    {  687,  415,       0,  950973,       0,  0, 13,  0, }, // Bhutan
    {  687,  422,       0,  950986,       0,  0, 15,  0, }, // Bolivia
    {  687,  437,  951001,  951019,  951042, 18, 23, 29, }, // Brasilia
    {  687,  454,       0,  951071,       0,  0, 24,  0, }, // Brunei
    {  687,  461,  951095,  951110,  951130, 15, 20, 26, }, // Cape_Verde
    {  687,  478,       0,  951156,       0,  0, 18,  0, }, // Chamorro
    {  687,  487,  951174,  951185,  951201, 11, 16, 31, }, // Chatham
    {  687,  495,  951232,  951243,  951259, 11, 16, 22, }, // Chile
    {  687,  501,  951281,  951291,  951306, 10, 15, 30, }, // China
    {  687,  507,       0,  951336,       0,  0, 19,  0, }, // Christmas
    {  687,  517,       0,  951355,       0,  0, 28,  0, }, // Cocos
    {  687,  523,  951383,  951398,  951418, 15, 20, 26, }, // Colombia
    {  687,  532,  951444,  951470,  951501, 26, 31, 43, }, // Cook
    {  687,  537,  951544,  951557,  951575, 13, 18, 33, }, // Cuba
    {  687,  548,       0,  951608,       0,  0, 12,  0, }, // Davis
    {  687,  564,       0,  951620,       0,  0, 26,  0, }, // DumontDUrville
    {  687,  601,       0,  951646,       0,  0, 19,  0, }, // East_Timor
    {  687,  612,  951665,  951682,  951704, 17, 22, 28, }, // Easter
    {  687,  619,       0,  951732,       0,  0, 15,  0, }, // Ecuador
    {  687,  627,  951747,  951766,  951790, 19, 24, 30, }, // Europe_Central
    {  687,  642,  951820,  951839,  951863, 19, 24, 30, }, // Europe_Eastern
    {  687,  657,       0,  951893,       0,  0, 24,  0, }, // Europe_Further_Eastern
    {  687,  680,  951917,  951938,  951964, 21, 26, 32, }, // Europe_Western
    {  687,  695,  951996,  952027,  952063, 31, 36, 42, }, // Falkland
    {  687,  704,  952105,  952117,  952134, 12, 17, 23, }, // Fiji
    {  687,  709,       0,  952157,       0,  0, 21,  0, }, // French_Guiana
    {  687,  723,       0,  952178,       0,  0, 39,  0, }, // French_Southern
    {  687,  746,       0,  952217,       0,  0, 16,  0, }, // Galapagos
    {  687,  756,       0,  952233,       0,  0, 14,  0, }, // Gambier
    {  687,  764,  952247,  952262,  952282, 15, 20, 26, }, // Georgia
    {  687,  772,       0,  952308,       0,  0, 31,  0, }, // Gilbert_Islands
    {  687,  788,       0,  952339,       0,  0, 16,  0, }, // GMT
    {  687,  830,  952355,  952374,  952401, 19, 27, 33, }, // Greenland_Eastern
    {  687,  848,  952434,  952455,  952484, 21, 29, 35, }, // Greenland_Western
    {  687,  871,       0,  952519,       0,  0, 17,  0, }, // Gulf
    {  687,  876,       0,  952165,       0,  0, 13,  0, }, // Guyana
    {  687,  883,  952536,  952555,  952579, 19, 24, 39, }, // Hawaii_Aleutian
    {  687,  899,  952618,  952634,  952655, 16, 21, 27, }, // Hong_Kong
    {  687,  909,  952682,  952694,  952711, 12, 17, 23, }, // Hovd
    {  687,  914,       0,  952734,       0,  0, 17,  0, }, // India
    {  687,  920,       0,  952751,       0,  0, 19,  0, }, // Indian_Ocean
    {  687,  933,       0,  952770,       0,  0, 16,  0, }, // Indochina
    {  687,  943,       0,  952786,       0,  0, 23,  0, }, // Indonesia_Central
    {  687,  961,       0,  952809,       0,  0, 23,  0, }, // Indonesia_Eastern
    {  687,  979,       0,  952832,       0,  0, 25,  0, }, // Indonesia_Western
    {  687,  997,  952857,  952868,  952884, 11, 16, 31, }, // Iran
    {  687, 1008,  952915,  952932,  952954, 17, 22, 28, }, // Irkutsk
    {  687, 1016,  952982,  952996,  953015, 14, 19, 34, }, // Israel
    {  687, 1023,  953049,  953062,  953080, 13, 18, 33, }, // Japan
    {  687, 1058,       0,  953113,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {  687, 1077,       0,  953135,       0,  0, 24,  0, }, // Kazakhstan_Western
    {  687, 1106,  953159,  953173,  953192, 14, 19, 34, }, // Korea
    {  687, 1112,       0,  953226,       0,  0, 13,  0, }, // Kosrae
    {  687, 1119,  953239,  953261,  953288, 22, 27, 33, }, // Krasnoyarsk
    {  687, 1151,       0,  953321,       0,  0, 19,  0, }, // Kyrgystan
    {  687, 1175,       0,  953340,       0,  0, 27,  0, }, // Line_Islands
    {  687, 1188,  953367,  953384,  953406, 17, 22, 37, }, // Lord_Howe
    {  687, 1204,  953443,  953457,  953476, 14, 19, 25, }, // Magadan
    {  687, 1219,       0,  953501,       0,  0, 14,  0, }, // Malaysia
    {  687, 1228,       0,  953515,       0,  0, 13,  0, }, // Maldives
    {  687, 1237,       0,  953528,       0,  0, 15,  0, }, // Marquesas
    {  687, 1247,       0,  953543,       0,  0, 29,  0, }, // Marshall_Islands
    {  687, 1264,  953572,  953585,  953603, 13, 18, 24, }, // Mauritius
    {  687, 1274,       0,  953627,       0,  0, 13,  0, }, // Mawson
    {  687, 1281,  953640,  953663,  953691, 23, 28, 43, }, // Mexico_Pacific
    {  687, 1296,  953734,  953750,  953771, 16, 21, 27, }, // Mongolia
    {  687, 1305,  953798,  953811,  953829, 13, 18, 24, }, // Moscow
    {  687, 1312,       0,  953853,       0,  0, 15,  0, }, // Myanmar
    {  687, 1320,       0,  953868,       0,  0, 11,  0, }, // Nauru
    {  687, 1326,       0,  953879,       0,  0, 12,  0, }, // Nepal
    {  687, 1332,  953891,  953913,  953940, 22, 27, 33, }, // New_Caledonia
    {  687, 1346,  953973,  953991,  954014, 18, 23, 38, }, // New_Zealand
    {  687, 1358,  954052,  954073,  954099, 21, 26, 41, }, // Newfoundland
    {  687, 1371,       0,  954140,       0,  0, 11,  0, }, // Niue
    {  687, 1376,  954151,  954170,  954194, 19, 24, 39, }, // Norfolk
    {  687, 1384,  954233,  954263,  954298, 30, 35, 41, }, // Noronha
    {  687, 1406,  954339,  954359,  954384, 20, 25, 31, }, // Novosibirsk
    {  687, 1418,  954415,  954428,  954446, 13, 18, 24, }, // Omsk
    {  687, 1428,  954470,  954487,  954509, 17, 22, 28, }, // Pakistan
    {  687, 1437,       0,  954537,       0,  0, 11,  0, }, // Palau
    {  687, 1443,       0,  954548,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  687, 1460,  954570,  954585,  954605, 15, 20, 26, }, // Paraguay
    {  687, 1469,  954631,  954642,  954658, 11, 16, 22, }, // Peru
    {  687, 1474,  954680,  954695,  954715, 15, 20, 26, }, // Philippines
    {  687, 1486,       0,  954741,       0,  0, 31,  0, }, // Phoenix_Islands
    {  687, 1502,  954772,  954803,  954839, 31, 36, 51, }, // Pierre_Miquelon
    {  687, 1518,       0,  954890,       0,  0, 15,  0, }, // Pitcairn
    {  687, 1527,       0,  954905,       0,  0, 13,  0, }, // Ponape
    {  687, 1534,       0,  954918,       0,  0, 17,  0, }, // Pyongyang
    {  687, 1554,       0,  954935,       0,  0, 15,  0, }, // Reunion
    {  687, 1562,       0,  954950,       0,  0, 13,  0, }, // Rothera
    {  687, 1570,  954963,  954976,  954994, 13, 18, 24, }, // Sakhalin
    {  687, 1596,  955018,  955029,  955045, 11, 16, 31, }, // Samoa
    {  687, 1602,       0,  955076,       0,  0, 14,  0, }, // Seychelles
    {  687, 1624,       0,  955090,       0,  0, 16,  0, }, // Singapore
    {  687, 1634,       0,  955106,       0,  0, 29,  0, }, // Solomon
    {  687, 1642,       0,  955135,       0,  0, 19,  0, }, // South_Georgia
    {  687, 1656,       0,  955154,       0,  0, 15,  0, }, // Suriname
    {  687, 1676,       0,  955169,       0,  0, 13,  0, }, // Syowa
    {  687, 1682,       0,  955182,       0,  0, 13,  0, }, // Tahiti
    {  687, 1689,  955195,  955207,  955224, 12, 17, 32, }, // Taipei
    {  687, 1696,       0,  955256,       0,  0, 19,  0, }, // Tajikistan
    {  687, 1724,       0,  955275,       0,  0, 14,  0, }, // Tokelau
    {  687, 1732,  955289,  955301,  955318, 12, 17, 23, }, // Tonga
    {  687, 1738,       0,  955341,       0,  0, 10,  0, }, // Truk
    {  687, 1750,  955351,  955372,  955399, 21, 27, 33, }, // Turkmenistan
    {  687, 1763,       0,  955432,       0,  0, 13,  0, }, // Tuvalu
    {  687, 1777,  955445,  955459,  955478, 14, 19, 25, }, // Uruguay
    {  687, 1792,  955503,  955523,  955548, 20, 25, 31, }, // Uzbekistan
    {  687, 1803,  955579,  955592,  955610, 13, 18, 24, }, // Vanuatu
    {  687, 1811,       0,  955634,       0,  0, 17,  0, }, // Venezuela
    {  687, 1821,  955651,  955671,  955696, 20, 25, 31, }, // Vladivostok
    {  687, 1833,  955727,  955745,  955768, 18, 23, 29, }, // Volgograd
    {  687, 1843,       0,  955657,       0,  0, 14,  0, }, // Vostok
    {  687, 1850,       0,  955797,       0,  0, 15,  0, }, // Wake
    {  687, 1855,       0,  955812,       0,  0, 26,  0, }, // Wallis
    {  687, 1862,  955838,  955854,  955875, 16, 21, 27, }, // Yakutsk
    {  687, 1870,  955902,  955922,  955947, 20, 25, 31, }, // Yekaterinburg
    {  687, 1892,       0,  955978,       0,  0,  9,  0, }, // Yukon
    {  688,    5,       0,  955987,       0,  0, 20,  0, }, // Afghanistan Venetian/Latin/Italy
    {  688,   17,       0,  956007,       0,  0, 24,  0, }, // Africa_Central
    {  688,   32,       0,  956031,       0,  0, 25,  0, }, // Africa_Eastern
    {  688,   65,       0,  956056,       0,  0, 27,  0, }, // Africa_Southern
    {  688,   81,  956083,  956109,  956143, 26, 34, 33, }, // Africa_Western
    {  688,  107,  956176,  956191,  956214, 15, 23, 22, }, // Alaska
    {  688,  135,  956236,  956253,  956278, 17, 25, 24, }, // Amazon
    {  688,  142,  956302,  956332,  956370, 30, 38, 37, }, // America_Central
    {  688,  158,  956407,  956438,  956477, 31, 39, 38, }, // America_Eastern
    {  688,  174,  956515,  956551,  956595, 36, 44, 43, }, // America_Mountain
    {  688,  191,  956638,  956655,  956680, 17, 25, 24, }, // America_Pacific
    {  688,  214,  655035,  956704,  956723, 11, 19, 18, }, // Apia
    {  688,  232,  956741,  956756,  956779, 15, 23, 22, }, // Arabian
    {  688,  240,  956801,  956819,  956845, 18, 26, 25, }, // Argentina
    {  688,  250,  956870,  956899,  956936, 29, 37, 36, }, // Argentina_Western
    {  688,  268,  956972,  956988,  957012, 16, 24, 23, }, // Armenia
    {  688,  286,  957035,  957053,  957079, 18, 26, 25, }, // Atlantic
    {  688,  295,  957104,  957131,  957166, 27, 35, 34, }, // Australia_Central
    {  688,  313,  957200,  957236,  957280, 36, 44, 43, }, // Australia_CentralWestern
    {  688,  338,  957323,  957351,  957387, 28, 36, 35, }, // Australia_Eastern
    {  688,  356,  957422,  957451,  957488, 29, 37, 36, }, // Australia_Western
    {  688,  374,  957524,  957543,  957570, 19, 27, 26, }, // Azerbaijan
    {  688,  385,  957596,  957611,  957634, 15, 23, 22, }, // Azores
    {  688,  397,  957656,  957675,  957702, 19, 27, 26, }, // Bangladesh
    {  688,  415,       0,  957728,       0,  0, 14,  0, }, // Bhutan
    {  688,  422,       0,  957742,       0,  0, 17,  0, }, // Bolivia
    {  688,  437,  957759,  957774,  957797, 15, 23, 22, }, // Brasilia
    {  688,  454,       0,  957819,       0,  0, 15,  0, }, // Brunei
    {  688,  461,  957834,  957850,  957874, 16, 24, 23, }, // Cape_Verde
    {  688,  478,       0,  957897,       0,  0, 14,  0, }, // Chamorro
    {  688,  487,  957911,  957933,  957963, 22, 30, 29, }, // Chatham
    {  688,  495,  957992,  958005,  958026, 13, 21, 20, }, // Chile
    {  688,  501,  958046,  958060,  958082, 14, 22, 21, }, // China
    {  688,  507,       0,  958103,       0,  0, 24,  0, }, // Christmas
    {  688,  517,       0,  958127,       0,  0, 21,  0, }, // Cocos
    {  688,  523,  958148,  958166,  958192, 18, 26, 25, }, // Colombia
    {  688,  532,  958217,  958237,  958265, 20, 28, 27, }, // Cook
    {  688,  537,  656845,  958292,  958311, 11, 19, 18, }, // Cuba
    {  688,  548,       0,  656895,       0,  0, 12,  0, }, // Davis
    {  688,  564,       0,  958329,       0,  0, 23,  0, }, // DumontDUrville
    {  688,  601,       0,  958352,       0,  0, 18,  0, }, // East_Timor
    {  688,  612,  958370,  958394,  958426, 24, 32, 31, }, // Easter
    {  688,  619,       0,  958457,       0,  0, 16,  0, }, // Ecuador
    {  688,  627,  958473,  958497,  958529, 24, 32, 31, }, // Europe_Central
    {  688,  642,  958560,  958585,  958618, 25, 33, 32, }, // Europe_Eastern
    {  688,  657,       0,  958650,       0,  0, 30,  0, }, // Europe_Further_Eastern
    {  688,  680,  958680,  958706,  958740, 26, 34, 33, }, // Europe_Western
    {  688,  695,  958773,  958796,  958827, 23, 31, 30, }, // Falkland
    {  688,  704,  958857,  958877,  958905, 20, 28, 27, }, // Fiji
    {  688,  709,       0,  958932,       0,  0, 25,  0, }, // French_Guiana
    {  688,  723,       0,  958957,       0,  0, 49,  0, }, // French_Southern
    {  688,  746,       0,  959006,       0,  0, 19,  0, }, // Galapagos
    {  688,  756,       0,  959025,       0,  0, 23,  0, }, // Gambier
    {  688,  764,  959048,  959064,  959088, 16, 24, 23, }, // Georgia
    {  688,  772,       0,  959111,       0,  0, 23,  0, }, // Gilbert_Islands
    {  688,  788,       0,  959134,       0,  0, 30,  0, }, // GMT
    {  688,  830,  959164,  959194,  959232, 30, 38, 37, }, // Greenland_Eastern
    {  688,  848,  959269,  959300,  959339, 31, 39, 38, }, // Greenland_Western
    {  688,  871,       0,  959377,       0,  0, 14,  0, }, // Gulf
    {  688,  876,       0,  958932,       0,  0, 16,  0, }, // Guyana
    {  688,  883,  959391,  959414,  959445, 23, 31, 30, }, // Hawaii_Aleutian
    {  688,  899,  658178,  959475,  959499, 16, 24, 23, }, // Hong_Kong
    {  688,  909,  658243,  959522,  959541, 11, 19, 18, }, // Hovd
    {  688,  914,       0,  959559,       0,  0, 14,  0, }, // India
    {  688,  920,       0,  959573,       0,  0, 22,  0, }, // Indian_Ocean
    {  688,  933,       0,  959595,       0,  0, 17,  0, }, // Indochina
    {  688,  943,       0,  959612,       0,  0, 27,  0, }, // Indonesia_Central
    {  688,  961,       0,  959639,       0,  0, 28,  0, }, // Indonesia_Eastern
    {  688,  979,       0,  959667,       0,  0, 29,  0, }, // Indonesia_Western
    {  688,  997,  959696,  959709,  959730, 13, 21, 20, }, // Iran
    {  688, 1008,  658501,  959750,  959772, 14, 22, 21, }, // Irkutsk
    {  688, 1016,  959793,  959807,  959829, 14, 22, 21, }, // Israel
    {  688, 1023,  959850,  959864,  959886, 14, 22, 21, }, // Japan
    {  688, 1047,       0,  959907,       0,  0, 19,  0, }, // Kazakhstan
    {  688, 1058,       0,  959926,       0,  0, 29,  0, }, // Kazakhstan_Eastern
    {  688, 1077,       0,  959955,       0,  0, 30,  0, }, // Kazakhstan_Western
    {  688, 1106,  959985,  960000,  960023, 15, 23, 22, }, // Korea
    {  688, 1112,       0,  960045,       0,  0, 21,  0, }, // Kosrae
    {  688, 1119,  658913,  960066,  960092, 18, 26, 25, }, // Krasnoyarsk
    {  688, 1151,       0,  960117,       0,  0, 21,  0, }, // Kyrgystan
    {  688, 1175,       0,  960138,       0,  0, 27,  0, }, // Line_Islands
    {  688, 1188,  960165,  960189,  960221, 24, 32, 31, }, // Lord_Howe
    {  688, 1204,  659165,  960252,  960274, 14, 22, 21, }, // Magadan
    {  688, 1219,       0,  960295,       0,  0, 17,  0, }, // Malaysia
    {  688, 1228,       0,  960312,       0,  0, 23,  0, }, // Maldives
    {  688, 1237,       0,  960335,       0,  0, 24,  0, }, // Marquesas
    {  688, 1247,       0,  960359,       0,  0, 24,  0, }, // Marshall_Islands
    {  688, 1264,  960383,  960407,  960439, 24, 32, 31, }, // Mauritius
    {  688, 1274,       0,  659386,       0,  0, 13,  0, }, // Mawson
    {  688, 1281,  960470,  960499,  960536, 29, 37, 36, }, // Mexico_Pacific
    {  688, 1296,  960572,  960589,  960614, 17, 25, 24, }, // Mongolia
    {  688, 1305,  659571,  960638,  960658, 12, 20, 19, }, // Moscow
    {  688, 1312,       0,  960677,       0,  0, 25,  0, }, // Myanmar
    {  688, 1320,       0,  960702,       0,  0, 20,  0, }, // Nauru
    {  688, 1326,       0,  960722,       0,  0, 14,  0, }, // Nepal
    {  688, 1332,  960736,  960760,  960792, 24, 32, 31, }, // New_Caledonia
    {  688, 1346,  960823,  960845,  960875, 22, 30, 29, }, // New_Zealand
    {  688, 1358,  960904,  960919,  960942, 15, 23, 22, }, // Newfoundland
    {  688, 1371,       0,  960964,       0,  0, 19,  0, }, // Niue
    {  688, 1376,  960983,  961005,  961035, 22, 30, 29, }, // Norfolk
    {  688, 1384,  961064,  961090,  961124, 26, 34, 33, }, // Noronha
    {  688, 1406,  660139,  961157,  961183, 18, 26, 25, }, // Novosibirsk
    {  688, 1418,  660210,  961208,  961227, 11, 19, 18, }, // Omsk
    {  688, 1428,  961245,  961262,  961287, 17, 25, 24, }, // Pakistan
    {  688, 1437,       0,  961311,       0,  0, 21,  0, }, // Palau
    {  688, 1443,       0,  961332,       0,  0, 27,  0, }, // Papua_New_Guinea
    {  688, 1460,  961359,  961376,  961401, 17, 25, 24, }, // Paraguay
    {  688, 1469,  961425,  961438,  961459, 13, 21, 20, }, // Peru
    {  688, 1474,  961479,  961503,  961535, 24, 32, 31, }, // Philippines
    {  688, 1486,       0,  961566,       0,  0, 22,  0, }, // Phoenix_Islands
    {  688, 1502,  961588,  961614,  961648, 26, 34, 33, }, // Pierre_Miquelon
    {  688, 1518,       0,  961681,       0,  0, 24,  0, }, // Pitcairn
    {  688, 1527,       0,  961705,       0,  0, 21,  0, }, // Ponape
    {  688, 1534,       0,  660748,       0,  0, 16,  0, }, // Pyongyang
    {  688, 1554,       0,  961726,       0,  0, 22,  0, }, // Reunion
    {  688, 1562,       0,  660847,       0,  0, 14,  0, }, // Rothera
    {  688, 1570,  961748,  961762,  961784, 14, 22, 21, }, // Sakhalin
    {  688, 1596,  961805,  961826,  961855, 21, 29, 28, }, // Samoa
    {  688, 1602,       0,  961883,       0,  0, 29,  0, }, // Seychelles
    {  688, 1624,       0,  661065,       0,  0, 16,  0, }, // Singapore
    {  688, 1634,       0,  961912,       0,  0, 23,  0, }, // Solomon
    {  688, 1642,       0,  961935,       0,  0, 26,  0, }, // South_Georgia
    {  688, 1656,       0,  961961,       0,  0, 17,  0, }, // Suriname
    {  688, 1676,       0,  661152,       0,  0, 12,  0, }, // Syowa
    {  688, 1682,       0,  961978,       0,  0, 20,  0, }, // Tahiti
    {  688, 1689,  961998,  962011,  962032, 13, 21, 20, }, // Taipei
    {  688, 1696,       0,  962052,       0,  0, 19,  0, }, // Tajikistan
    {  688, 1724,       0,  962071,       0,  0, 23,  0, }, // Tokelau
    {  688, 1732,  962094,  962115,  962144, 21, 29, 28, }, // Tonga
    {  688, 1738,       0,  962172,       0,  0, 14,  0, }, // Truk
    {  688, 1750,  962186,  962207,  962236, 21, 29, 28, }, // Turkmenistan
    {  688, 1763,       0,  962264,       0,  0, 22,  0, }, // Tuvalu
    {  688, 1777,  962286,  962302,  962326, 16, 24, 23, }, // Uruguay
    {  688, 1792,  962349,  962368,  962395, 19, 27, 26, }, // Uzbekistan
    {  688, 1803,  962421,  962444,  962475, 23, 31, 30, }, // Vanuatu
    {  688, 1811,       0,  962505,       0,  0, 18,  0, }, // Venezuela
    {  688, 1821,  661673,  962523,  962549, 18, 26, 25, }, // Vladivostok
    {  688, 1833,  962574,  962591,  962616, 17, 25, 24, }, // Volgograd
    {  688, 1843,       0,  661809,       0,  0, 13,  0, }, // Vostok
    {  688, 1850,       0,  962640,       0,  0, 19,  0, }, // Wake
    {  688, 1855,       0,  962659,       0,  0, 31,  0, }, // Wallis
    {  688, 1862,  661866,  962690,  962712, 14, 22, 21, }, // Yakutsk
    {  688, 1870,  962733,  962753,  962781, 20, 28, 27, }, // Yekaterinburg
    {  688, 1892,       0,  962808,       0,  0, 14,  0, }, // Yukon
    {  689,    5,       0,  962822,       0,  0, 16,  0, }, // Afghanistan Kuvi/Latin/India
    {  689,   17,       0,  962838,       0,  0, 19,  0, }, // Africa_Central
    {  689,   32,       0,  962857,       0,  0, 21,  0, }, // Africa_Eastern
    {  689,   65,       0,  962878,       0,  0, 17,  0, }, // Africa_Southern
    {  689,   81,  962895,  962917,  962948, 22, 31, 28, }, // Africa_Western
    {  689,  107,  962976,  962987,  963006, 11, 19, 19, }, // Alaska
    {  689,  135,  963025,  963036,  963055, 11, 19, 21, }, // Amazon
    {  689,  142,  963076,  963087,  963106, 11, 19, 19, }, // America_Central
    {  689,  158,  963125,  963138,  963159, 13, 21, 21, }, // America_Eastern
    {  689,  174,  963180,  963190,  963208, 10, 18, 18, }, // America_Mountain
    {  689,  191,  963226,  963238,  963258, 12, 20, 20, }, // America_Pacific
    {  689,  214,  963278,  963287,  963304,  9, 17, 16, }, // Apia
    {  689,  232,  963320,  963332,  963352, 12, 20, 19, }, // Arabian
    {  689,  240,  963371,  963385,  963407, 14, 22, 25, }, // Argentina
    {  689,  250,  963432,  963457,  963490, 25, 33, 31, }, // Argentina_Western
    {  689,  268,  963521,  963533,  963553, 12, 20, 18, }, // Armenia
    {  689,  286,  963571,  963584,  963605, 13, 21, 20, }, // Atlantic
    {  689,  295,  963625,  963645,  963675, 20, 30, 29, }, // Australia_Central
    {  689,  313,  963704,  963737,  963778, 33, 41, 40, }, // Australia_CentralWestern
    {  689,  338,  963818,  963840,  963872, 22, 32, 32, }, // Australia_Eastern
    {  689,  356,  963904,  963928,  963962, 24, 34, 33, }, // Australia_Western
    {  689,  374,  963995,  964010,  964033, 15, 23, 21, }, // Azerbaijan
    {  689,  385,  964054,  964065,  964084, 11, 19, 17, }, // Azores
    {  689,  397,  964101,  964115,  964137, 14, 22, 20, }, // Bangladesh
    {  689,  415,       0,  964157,       0,  0, 10,  0, }, // Bhutan
    {  689,  422,       0,  964167,       0,  0, 11,  0, }, // Bolivia
    {  689,  437,  964178,  964191,  964212, 13, 21, 19, }, // Brasilia
    {  689,  454,       0,  964231,       0,  0, 22,  0, }, // Brunei
    {  689,  461,  964253,  964264,  964283, 11, 19, 17, }, // Cape_Verde
    {  689,  478,       0,  964300,       0,  0, 18,  0, }, // Chamorro
    {  689,  487,  964318,  964329,  964348, 11, 19, 18, }, // Chatham
    {  689,  495,  964366,  964375,  964392,  9, 17, 15, }, // Chile
    {  689,  501,  964407,  964415,  964431,  8, 16, 15, }, // China
    {  689,  507,       0,  964446,       0,  0, 16,  0, }, // Christmas
    {  689,  517,       0,  964462,       0,  0, 14,  0, }, // Cocos
    {  689,  523,  964476,  964489,  964510, 13, 21, 19, }, // Colombia
    {  689,  532,  964529,  964541,  964561, 12, 20, 22, }, // Cook
    {  689,  537,  964583,  964592,  964609,  9, 17, 16, }, // Cuba
    {  689,  548,       0,  964625,       0,  0, 10,  0, }, // Davis
    {  689,  564,       0,  964635,       0,  0, 21,  0, }, // DumontDUrville
    {  689,  601,       0,  964656,       0,  0, 19,  0, }, // East_Timor
    {  689,  612,  964675,  964688,  964709, 13, 21, 19, }, // Easter
    {  689,  619,       0,  964728,       0,  0, 12,  0, }, // Ecuador
    {  689,  627,  964740,  964761,  964790, 21, 29, 27, }, // Europe_Central
    {  689,  642,  964817,  964839,  964869, 22, 30, 28, }, // Europe_Eastern
    {  689,  657,       0,  964897,       0,  0, 24,  0, }, // Europe_Further_Eastern
    {  689,  680,  964921,  964946,  964979, 25, 33, 31, }, // Europe_Western
    {  689,  695,  965010,  965030,  965058, 20, 28, 26, }, // Falkland
    {  689,  704,  965084,  965093,  965110,  9, 17, 15, }, // Fiji
    {  689,  709,       0,  965125,       0,  0, 17,  0, }, // French_Guiana
    {  689,  723,       0,  965142,       0,  0, 31,  0, }, // French_Southern
    {  689,  746,       0,  965173,       0,  0, 13,  0, }, // Galapagos
    {  689,  756,       0,  965186,       0,  0, 12,  0, }, // Gambier
    {  689,  764,  965198,  965209,  965228, 11, 19, 17, }, // Georgia
    {  689,  772,       0,  965245,       0,  0, 15,  0, }, // Gilbert_Islands
    {  689,  788,       0,  965260,       0,  0, 16,  0, }, // GMT
    {  689,  830,  965276,  965297,  965326, 21, 29, 27, }, // Greenland_Eastern
    {  689,  848,  965353,  965376,  965407, 23, 31, 29, }, // Greenland_Western
    {  689,  871,       0,  965436,       0,  0, 17,  0, }, // Gulf
    {  689,  876,       0,  965132,       0,  0, 10,  0, }, // Guyana
    {  689,  883,  965453,  965472,  965499, 19, 27, 26, }, // Hawaii_Aleutian
    {  689,  899,  965525,  965539,  965561, 14, 22, 20, }, // Hong_Kong
    {  689,  909,  965581,  965590,  965607,  9, 17, 15, }, // Hovd
    {  689,  914,       0,  965622,       0,  0, 18,  0, }, // India
    {  689,  920,       0,  965640,       0,  0, 15,  0, }, // Indian_Ocean
    {  689,  933,       0,  965655,       0,  0, 13,  0, }, // Indochina
    {  689,  943,       0,  965668,       0,  0, 20,  0, }, // Indonesia_Central
    {  689,  961,       0,  965688,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  689,  979,       0,  965709,       0,  0, 23,  0, }, // Indonesia_Western
    {  689,  997,  965732,  965741,  965758,  9, 17, 16, }, // Iran
    {  689, 1008,  965774,  965787,  965808, 13, 21, 19, }, // Irkutsk
    {  689, 1016,  965827,  965839,  965859, 12, 20, 19, }, // Israel
    {  689, 1023,  965878,  965888,  965906, 10, 18, 17, }, // Japan
    {  689, 1047,       0,  965923,       0,  0, 14,  0, }, // Kazakhstan
    {  689, 1058,       0,  965937,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  689, 1077,       0,  965960,       0,  0, 24,  0, }, // Kazakhstan_Western
    {  689, 1106,  965984,  965995,  966014, 11, 19, 18, }, // Korea
    {  689, 1112,       0,  966032,       0,  0, 11,  0, }, // Kosrae
    {  689, 1119,  966043,  966058,  966081, 15, 23, 21, }, // Krasnoyarsk
    {  689, 1151,       0,  966102,       0,  0, 15,  0, }, // Kyrgystan
    {  689, 1175,       0,  966117,       0,  0, 13,  0, }, // Line_Islands
    {  689, 1188,  966130,  966143,  966164, 13, 21, 20, }, // Lord_Howe
    {  689, 1204,  966184,  966196,  966216, 12, 20, 19, }, // Magadan
    {  689, 1219,       0,  966235,       0,  0, 12,  0, }, // Malaysia
    {  689, 1228,       0,  966247,       0,  0, 11,  0, }, // Maldives
    {  689, 1237,       0,  966258,       0,  0, 12,  0, }, // Marquesas
    {  689, 1247,       0,  966270,       0,  0, 16,  0, }, // Marshall_Islands
    {  689, 1264,  966286,  966298,  966317, 12, 19, 18, }, // Mauritius
    {  689, 1274,       0,  966335,       0,  0, 11,  0, }, // Mawson
    {  689, 1281,  966346,  966367,  966396, 21, 29, 28, }, // Mexico_Pacific
    {  689, 1296,  966424,  966438,  966460, 14, 22, 20, }, // Mongolia
    {  689, 1305,  966480,  966490,  966508, 10, 18, 16, }, // Moscow
    {  689, 1312,       0,  966524,       0,  0, 13,  0, }, // Myanmar
    {  689, 1320,       0,  966537,       0,  0, 10,  0, }, // Nauru
    {  689, 1326,       0,  966547,       0,  0, 10,  0, }, // Nepal
    {  689, 1332,  966557,  966575,  966601, 18, 26, 24, }, // New_Caledonia
    {  689, 1346,  966625,  966639,  966661, 14, 22, 21, }, // New_Zealand
    {  689, 1358,  966682,  966700,  966726, 18, 26, 25, }, // Newfoundland
    {  689, 1371,       0,  966751,       0,  0,  8,  0, }, // Niue
    {  689, 1376,  966759,  966774,  966797, 15, 23, 22, }, // Norfolk
    {  689, 1384,  966819,  966842,  966873, 23, 31, 29, }, // Noronha
    {  689, 1406,  966902,  966918,  966942, 16, 24, 22, }, // Novosibirsk
    {  689, 1418,  966964,  966974,  966992, 10, 18, 16, }, // Omsk
    {  689, 1428,  967008,  967021,  967042, 13, 21, 19, }, // Pakistan
    {  689, 1437,       0,  967061,       0,  0, 10,  0, }, // Palau
    {  689, 1443,       0,  967071,       0,  0, 20,  0, }, // Papua_New_Guinea
    {  689, 1460,  967091,  967104,  967125, 13, 21, 19, }, // Paraguay
    {  689, 1469,  967144,  967153,  967170,  9, 17, 15, }, // Peru
    {  689, 1474,  967185,  967197,  967217, 12, 20, 18, }, // Philippines
    {  689, 1486,       0,  967235,       0,  0, 19,  0, }, // Phoenix_Islands
    {  689, 1502,  967254,  967282,  967318, 28, 36, 35, }, // Pierre_Miquelon
    {  689, 1518,       0,  967353,       0,  0, 13,  0, }, // Pitcairn
    {  689, 1527,       0,  967366,       0,  0, 11,  0, }, // Ponape
    {  689, 1534,       0,  967377,       0,  0, 15,  0, }, // Pyongyang
    {  689, 1554,       0,  967392,       0,  0, 12,  0, }, // Reunion
    {  689, 1562,       0,  967404,       0,  0, 11,  0, }, // Rothera
    {  689, 1570,  967415,  967427,  967447, 12, 20, 18, }, // Sakhalin
    {  689, 1596,  967465,  967476,  967495, 11, 19, 13, }, // Samoa
    {  689, 1602,       0,  967508,       0,  0, 11,  0, }, // Seychelles
    {  689, 1624,       0,  967519,       0,  0, 21,  0, }, // Singapore
    {  689, 1634,       0,  967540,       0,  0, 19,  0, }, // Solomon
    {  689, 1642,       0,  967559,       0,  0, 17,  0, }, // South_Georgia
    {  689, 1656,       0,  967576,       0,  0, 12,  0, }, // Suriname
    {  689, 1676,       0,  967588,       0,  0,  9,  0, }, // Syowa
    {  689, 1682,       0,  967597,       0,  0, 11,  0, }, // Tahiti
    {  689, 1689,  967608,  967618,  967636, 10, 18, 12, }, // Taipei
    {  689, 1696,       0,  967648,       0,  0, 15,  0, }, // Tajikistan
    {  689, 1724,       0,  967663,       0,  0, 12,  0, }, // Tokelau
    {  689, 1732,  967675,  967685,  967703, 10, 18, 16, }, // Tonga
    {  689, 1738,       0,  967719,       0,  0,  8,  0, }, // Truk
    {  689, 1750,  967727,  967744,  967768, 17, 24, 23, }, // Turkmenistan
    {  689, 1763,       0,  967791,       0,  0, 11,  0, }, // Tuvalu
    {  689, 1777,  967802,  967813,  967831, 11, 18, 17, }, // Uruguay
    {  689, 1792,  967848,  967863,  967886, 15, 23, 20, }, // Uzbekistan
    {  689, 1803,  967906,  967918,  967938, 12, 20, 18, }, // Vanuatu
    {  689, 1811,       0,  967956,       0,  0, 14,  0, }, // Venezuela
    {  689, 1821,  967970,  967986,  968010, 16, 24, 22, }, // Vladivostok
    {  689, 1833,  968032,  968046,  968068, 14, 22, 20, }, // Volgograd
    {  689, 1843,       0,  967975,       0,  0, 11,  0, }, // Vostok
    {  689, 1850,       0,  968088,       0,  0, 14,  0, }, // Wake
    {  689, 1855,       0,  968102,       0,  0, 16,  0, }, // Wallis
    {  689, 1862,  968118,  968131,  968152, 13, 21, 19, }, // Yakutsk
    {  689, 1870,  968171,  968189,  968215, 18, 26, 24, }, // Yekaterinburg
    {  690,    5,       0,  968239,       0,  0, 16,  0, }, // Afghanistan Kuvi/Devanagari/India
    {  690,   17,       0,  968255,       0,  0, 19,  0, }, // Africa_Central
    {  690,   32,       0,  968274,       0,  0, 23,  0, }, // Africa_Eastern
    {  690,   65,       0,  968297,       0,  0, 27,  0, }, // Africa_Southern
    {  690,   81,  968324,  968343,  968370, 19, 27, 30, }, // Africa_Western
    {  690,  107,  968400,  968412,  968433, 12, 21, 20, }, // Alaska
    {  690,  135,  968453,  968464,  968483, 11, 19, 22, }, // Amazon
    {  690,  142,  968505,  968516,  968535, 11, 19, 18, }, // America_Central
    {  690,  158,  968553,  968568,  968591, 15, 23, 22, }, // America_Eastern
    {  690,  174,  968613,  968624,  968643, 11, 19, 18, }, // America_Mountain
    {  690,  191,  968661,  968673,  968693, 12, 20, 19, }, // America_Pacific
    {  690,  214,  968712,  968721,  968738,  9, 17, 16, }, // Apia
    {  690,  232,  968754,  968762,  968778,  8, 16, 15, }, // Arabian
    {  690,  240,  968793,  968808,  968831, 15, 23, 26, }, // Argentina
    {  690,  250,  968857,  968885,  968921, 28, 36, 39, }, // Argentina_Western
    {  690,  268,  968960,  968974,  968996, 14, 22, 25, }, // Armenia
    {  690,  286,  969021,  969034,  969055, 13, 21, 20, }, // Atlantic
    {  690,  295,  969075,  969100,  969134, 25, 34, 33, }, // Australia_Central
    {  690,  313,  969167,  969206,  969253, 39, 47, 46, }, // Australia_CentralWestern
    {  690,  338,  969299,  969326,  969364, 27, 38, 37, }, // Australia_Eastern
    {  690,  356,  969401,  969430,  969469, 29, 39, 38, }, // Australia_Western
    {  690,  374,  969507,  969521,  969543, 14, 22, 25, }, // Azerbaijan
    {  690,  385,  969568,  969580,  969600, 12, 20, 23, }, // Azores
    {  690,  397,  969623,  969638,  969661, 15, 23, 26, }, // Bangladesh
    {  690,  415,       0,  969687,       0,  0, 10,  0, }, // Bhutan
    {  690,  422,       0,  969697,       0,  0, 13,  0, }, // Bolivia
    {  690,  437,  969710,  969726,  969750, 16, 24, 27, }, // Brasilia
    {  690,  454,       0,  969777,       0,  0, 23,  0, }, // Brunei
    {  690,  461,  969800,  969813,  969834, 13, 21, 24, }, // Cape_Verde
    {  690,  478,       0,  969858,       0,  0, 19,  0, }, // Chamorro
    {  690,  487,  969877,  969889,  969909, 12, 20, 19, }, // Chatham
    {  690,  495,  969928,  969937,  969954,  9, 17, 20, }, // Chile
    {  690,  501,  969974,  969982,  969998,  8, 16, 15, }, // China
    {  690,  507,       0,  970013,       0,  0, 18,  0, }, // Christmas
    {  690,  517,       0,  970031,       0,  0, 16,  0, }, // Cocos
    {  690,  523,  970047,  970060,  970081, 13, 21, 24, }, // Colombia
    {  690,  532,  970105,  970119,  970141, 14, 22, 29, }, // Cook
    {  690,  537,  970170,  970181,  970200, 11, 19, 18, }, // Cuba
    {  690,  548,       0,  970218,       0,  0, 10,  0, }, // Davis
    {  690,  564,       0,  970228,       0,  0, 24,  0, }, // DumontDUrville
    {  690,  601,       0,  970252,       0,  0, 21,  0, }, // East_Timor
    {  690,  612,  970273,  970289,  970313, 16, 24, 27, }, // Easter
    {  690,  619,       0,  970340,       0,  0, 13,  0, }, // Ecuador
    {  690,  627,  970353,  970373,  970401, 20, 28, 31, }, // Europe_Central
    {  690,  642,  970432,  970456,  970488, 24, 32, 35, }, // Europe_Eastern
    {  690,  657,       0,  970523,       0,  0, 28,  0, }, // Europe_Further_Eastern
    {  690,  680,  970551,  970577,  970611, 26, 34, 37, }, // Europe_Western
    {  690,  695,  970648,  970668,  970696, 20, 28, 31, }, // Falkland
    {  690,  704,  970727,  970737,  970755, 10, 18, 21, }, // Fiji
    {  690,  709,       0,  970776,       0,  0, 19,  0, }, // French_Guiana
    {  690,  723,       0,  970795,       0,  0, 35,  0, }, // French_Southern
    {  690,  746,       0,  970830,       0,  0, 19,  0, }, // Galapagos
    {  690,  756,       0,  970849,       0,  0, 15,  0, }, // Gambier
    {  690,  764,  970864,  970877,  970898, 13, 21, 24, }, // Georgia
    {  690,  772,       0,  970922,       0,  0, 19,  0, }, // Gilbert_Islands
    {  690,  788,       0,  970941,       0,  0, 17,  0, }, // GMT
    {  690,  830,  970958,  970983,  971016, 25, 33, 36, }, // Greenland_Eastern
    {  690,  848,  971052,  971079,  971114, 27, 35, 38, }, // Greenland_Western
    {  690,  871,       0,  971152,       0,  0, 17,  0, }, // Gulf
    {  690,  876,       0,  970784,       0,  0, 11,  0, }, // Guyana
    {  690,  883,  971169,  971187,  971213, 18, 26, 25, }, // Hawaii_Aleutian
    {  690,  899,  971238,  971252,  971274, 14, 22, 25, }, // Hong_Kong
    {  690,  909,  971299,  971309,  971327, 10, 18, 21, }, // Hovd
    {  690,  914,       0,  971348,       0,  0, 19,  0, }, // India
    {  690,  920,       0,  971367,       0,  0, 24,  0, }, // Indian_Ocean
    {  690,  933,       0,  971391,       0,  0, 14,  0, }, // Indochina
    {  690,  943,       0,  971405,       0,  0, 20,  0, }, // Indonesia_Central
    {  690,  961,       0,  971425,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  690,  979,       0,  971451,       0,  0, 28,  0, }, // Indonesia_Western
    {  690,  997,  971479,  971488,  971505,  9, 17, 17, }, // Iran
    {  690, 1008,  971522,  971537,  971560, 15, 23, 26, }, // Irkutsk
    {  690, 1016,  971586,  971598,  971618, 12, 20, 19, }, // Israel
    {  690, 1023,  971637,  971647,  971665, 10, 18, 17, }, // Japan
    {  690, 1047,       0,  971682,       0,  0, 15,  0, }, // Kazakhstan
    {  690, 1058,       0,  971697,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  690, 1077,       0,  971723,       0,  0, 28,  0, }, // Kazakhstan_Western
    {  690, 1106,  971751,  971764,  971785, 13, 21, 20, }, // Korea
    {  690, 1112,       0,  971805,       0,  0, 11,  0, }, // Kosrae
    {  690, 1119,  971816,  971836,  971864, 20, 28, 31, }, // Krasnoyarsk
    {  690, 1151,       0,  971895,       0,  0, 17,  0, }, // Kyrgystan
    {  690, 1175,       0,  971912,       0,  0, 15,  0, }, // Line_Islands
    {  690, 1188,  971927,  971942,  971965, 15, 23, 22, }, // Lord_Howe
    {  690, 1204,  971987,  971999,  972019, 12, 20, 23, }, // Magadan
    {  690, 1219,       0,  972042,       0,  0, 12,  0, }, // Malaysia
    {  690, 1228,       0,  972054,       0,  0, 11,  0, }, // Maldives
    {  690, 1237,       0,  972065,       0,  0, 13,  0, }, // Marquesas
    {  690, 1247,       0,  972078,       0,  0, 17,  0, }, // Marshall_Islands
    {  690, 1264,  972095,  972106,  972125, 11, 19, 22, }, // Mauritius
    {  690, 1274,       0,  972147,       0,  0, 11,  0, }, // Mawson
    {  690, 1281,  972158,  972181,  972212, 23, 31, 30, }, // Mexico_Pacific
    {  690, 1296,  972242,  972256,  972278, 14, 22, 25, }, // Mongolia
    {  690, 1305,  972303,  972314,  972333, 11, 19, 22, }, // Moscow
    {  690, 1312,       0,  972355,       0,  0, 13,  0, }, // Myanmar
    {  690, 1320,       0,  972368,       0,  0, 10,  0, }, // Nauru
    {  690, 1326,       0,  972378,       0,  0, 10,  0, }, // Nepal
    {  690, 1332,  972388,  972408,  972436, 20, 28, 31, }, // New_Caledonia
    {  690, 1346,  972467,  972483,  972507, 16, 24, 23, }, // New_Zealand
    {  690, 1358,  972530,  972549,  972576, 19, 27, 26, }, // Newfoundland
    {  690, 1371,       0,  972602,       0,  0,  9,  0, }, // Niue
    {  690, 1376,  972611,  972629,  972655, 18, 26, 25, }, // Norfolk
    {  690, 1384,  972680,  972708,  972744, 28, 36, 39, }, // Noronha
    {  690, 1406,  972783,  972801,  972827, 18, 26, 29, }, // Novosibirsk
    {  690, 1418,  972856,  972867,  972886, 11, 19, 22, }, // Omsk
    {  690, 1428,  972908,  972922,  972944, 14, 22, 25, }, // Pakistan
    {  690, 1437,       0,  972969,       0,  0,  9,  0, }, // Palau
    {  690, 1443,       0,  972978,       0,  0, 20,  0, }, // Papua_New_Guinea
    {  690, 1460,  972998,  973011,  973032, 13, 21, 24, }, // Paraguay
    {  690, 1469,  973056,  973065,  973082,  9, 17, 20, }, // Peru
    {  690, 1474,  973102,  973115,  973136, 13, 21, 24, }, // Philippines
    {  690, 1486,       0,  973160,       0,  0, 19,  0, }, // Phoenix_Islands
    {  690, 1502,  973179,  973209,  973247, 30, 38, 37, }, // Pierre_Miquelon
    {  690, 1518,       0,  973284,       0,  0, 13,  0, }, // Pitcairn
    {  690, 1527,       0,  973297,       0,  0, 11,  0, }, // Ponape
    {  690, 1534,       0,  973308,       0,  0, 15,  0, }, // Pyongyang
    {  690, 1554,       0,  973323,       0,  0, 13,  0, }, // Reunion
    {  690, 1562,       0,  973336,       0,  0, 11,  0, }, // Rothera
    {  690, 1570,  973347,  973358,  973377, 11, 19, 22, }, // Sakhalin
    {  690, 1596,  973399,  973408,  973425,  9, 17, 16, }, // Samoa
    {  690, 1602,       0,  973441,       0,  0, 12,  0, }, // Seychelles
    {  690, 1624,       0,  973453,       0,  0, 13,  0, }, // Singapore
    {  690, 1634,       0,  973466,       0,  0, 17,  0, }, // Solomon
    {  690, 1642,       0,  973483,       0,  0, 21,  0, }, // South_Georgia
    {  690, 1656,       0,  973504,       0,  0, 12,  0, }, // Suriname
    {  690, 1676,       0,  973516,       0,  0, 11,  0, }, // Syowa
    {  690, 1682,       0,  973527,       0,  0, 11,  0, }, // Tahiti
    {  690, 1689,  973538,  973548,  973566, 10, 18, 17, }, // Taipei
    {  690, 1696,       0,  973583,       0,  0, 16,  0, }, // Tajikistan
    {  690, 1724,       0,  973599,       0,  0, 12,  0, }, // Tokelau
    {  690, 1732,  973611,  973621,  973639, 10, 18, 21, }, // Tonga
    {  690, 1738,       0,  973660,       0,  0,  8,  0, }, // Truk
    {  690, 1750,  973668,  973687,  973714, 19, 27, 30, }, // Turkmenistan
    {  690, 1763,       0,  973744,       0,  0, 11,  0, }, // Tuvalu
    {  690, 1777,  973755,  973767,  973787, 12, 20, 23, }, // Uruguay
    {  690, 1792,  973810,  973828,  973854, 18, 26, 29, }, // Uzbekistan
    {  690, 1803,  973883,  973894,  973913, 11, 19, 22, }, // Vanuatu
    {  690, 1811,       0,  973935,       0,  0, 15,  0, }, // Venezuela
    {  690, 1821,  973950,  973968,  973994, 18, 26, 29, }, // Vladivostok
    {  690, 1833,  974023,  974039,  974063, 16, 24, 27, }, // Volgograd
    {  690, 1843,       0,  973956,       0,  0, 12,  0, }, // Vostok
    {  690, 1850,       0,  974090,       0,  0, 14,  0, }, // Wake
    {  690, 1855,       0,  974104,       0,  0, 25,  0, }, // Wallis
    {  690, 1862,  974129,  974143,  974165, 14, 22, 25, }, // Yakutsk
    {  690, 1870,  974190,  974208,  974234, 18, 26, 29, }, // Yekaterinburg
    {  691,    5,       0,  974263,       0,  0, 16,  0, }, // Afghanistan Kuvi/Odia/India
    {  691,   17,       0,  974279,       0,  0, 19,  0, }, // Africa_Central
    {  691,   32,       0,  974298,       0,  0, 22,  0, }, // Africa_Eastern
    {  691,   65,       0,  974320,       0,  0, 25,  0, }, // Africa_Southern
    {  691,   81,  974345,  974371,  974405, 26, 34, 32, }, // Africa_Western
    {  691,  107,  974437,  974449,  974469, 12, 20, 20, }, // Alaska
    {  691,  135,  974489,  974499,  974517, 10, 18, 16, }, // Amazon
    {  691,  142,  974533,  974544,  974563, 11, 19, 19, }, // America_Central
    {  691,  158,  974582,  974602,  974630, 20, 28, 28, }, // America_Eastern
    {  691,  174,  974658,  974668,  974686, 10, 18, 18, }, // America_Mountain
    {  691,  191,  974704,  974717,  974738, 13, 21, 21, }, // America_Pacific
    {  691,  214,  974759,  974768,  974785,  9, 17, 17, }, // Apia
    {  691,  232,  974802,  974814,  974834, 12, 20, 19, }, // Arabian
    {  691,  240,  974853,  974868,  974891, 15, 23, 21, }, // Argentina
    {  691,  250,  974912,  974942,  974980, 30, 38, 36, }, // Argentina_Western
    {  691,  268,  975016,  975029,  975050, 13, 21, 19, }, // Armenia
    {  691,  286,  975069,  975084,  975107, 15, 23, 22, }, // Atlantic
    {  691,  295,  975129,  975151,  975183, 22, 32, 31, }, // Australia_Central
    {  691,  313,  975214,  975251,  975296, 37, 45, 44, }, // Australia_CentralWestern
    {  691,  338,  975340,  975364,  975398, 24, 34, 33, }, // Australia_Eastern
    {  691,  356,  975431,  975459,  975497, 28, 38, 37, }, // Australia_Western
    {  691,  374,  975534,  975549,  975572, 15, 23, 21, }, // Azerbaijan
    {  691,  385,  975593,  975603,  975621, 10, 18, 16, }, // Azores
    {  691,  397,  975637,  975653,  975677, 16, 24, 22, }, // Bangladesh
    {  691,  415,       0,  975699,       0,  0, 10,  0, }, // Bhutan
    {  691,  422,       0,  975709,       0,  0, 10,  0, }, // Bolivia
    {  691,  437,  975719,  975734,  975754, 15, 20, 21, }, // Brasilia
    {  691,  454,       0,  975775,       0,  0, 22,  0, }, // Brunei
    {  691,  461,  975797,  975810,  975831, 13, 21, 19, }, // Cape_Verde
    {  691,  478,       0,  975850,       0,  0, 16,  0, }, // Chamorro
    {  691,  487,  975866,  975876,  975894, 10, 18, 18, }, // Chatham
    {  691,  495,  975912,  975921,  975938,  9, 17, 15, }, // Chile
    {  691,  501,  975953,  975961,  975977,  8, 16, 15, }, // China
    {  691,  507,       0,  975992,       0,  0, 17,  0, }, // Christmas
    {  691,  517,       0,  976009,       0,  0, 12,  0, }, // Cocos
    {  691,  523,  976021,  976034,  976055, 13, 21, 19, }, // Colombia
    {  691,  532,  976074,  976086,  976106, 12, 20, 24, }, // Cook
    {  691,  537,  976130,  976141,  976160, 11, 19, 19, }, // Cuba
    {  691,  548,       0,  976179,       0,  0, 11,  0, }, // Davis
    {  691,  564,       0,  976190,       0,  0, 24,  0, }, // DumontDUrville
    {  691,  601,       0,  976214,       0,  0, 18,  0, }, // East_Timor
    {  691,  612,  976232,  976246,  976268, 14, 22, 20, }, // Easter
    {  691,  619,       0,  976288,       0,  0, 12,  0, }, // Ecuador
    {  691,  627,  976300,  976321,  976350, 21, 29, 27, }, // Europe_Central
    {  691,  642,  976377,  976398,  976428, 21, 30, 28, }, // Europe_Eastern
    {  691,  657,       0,  976456,       0,  0, 23,  0, }, // Europe_Further_Eastern
    {  691,  680,  976479,  976503,  976537, 24, 34, 30, }, // Europe_Western
    {  691,  695,  976567,  976584,  976611, 17, 27, 25, }, // Falkland
    {  691,  704,  976636,  976645,  976662,  9, 17, 15, }, // Fiji
    {  691,  709,       0,  976677,       0,  0, 20,  0, }, // French_Guiana
    {  691,  723,       0,  976697,       0,  0, 32,  0, }, // French_Southern
    {  691,  746,       0,  976729,       0,  0, 14,  0, }, // Galapagos
    {  691,  756,       0,  976743,       0,  0, 12,  0, }, // Gambier
    {  691,  764,  976755,  976767,  976787, 12, 20, 17, }, // Georgia
    {  691,  772,       0,  976804,       0,  0, 21,  0, }, // Gilbert_Islands
    {  691,  788,       0,  976825,       0,  0, 17,  0, }, // GMT
    {  691,  830,  976842,  976867,  976900, 25, 33, 31, }, // Greenland_Eastern
    {  691,  848,  976931,  976960,  976997, 29, 37, 35, }, // Greenland_Western
    {  691,  871,       0,  977032,       0,  0, 17,  0, }, // Gulf
    {  691,  876,       0,  976686,       0,  0, 11,  0, }, // Guyana
    {  691,  883,  977049,  977068,  977095, 19, 27, 27, }, // Hawaii_Aleutian
    {  691,  899,  977122,  977136,  977158, 14, 22, 20, }, // Hong_Kong
    {  691,  909,  977178,  977186,  977202,  8, 16, 14, }, // Hovd
    {  691,  914,       0,  977216,       0,  0, 17,  0, }, // India
    {  691,  920,       0,  977233,       0,  0, 23,  0, }, // Indian_Ocean
    {  691,  933,       0,  977256,       0,  0, 13,  0, }, // Indochina
    {  691,  943,       0,  977269,       0,  0, 21,  0, }, // Indonesia_Central
    {  691,  961,       0,  977290,       0,  0, 23,  0, }, // Indonesia_Eastern
    {  691,  979,       0,  977313,       0,  0, 26,  0, }, // Indonesia_Western
    {  691,  997,  977339,  977348,  977365,  9, 17, 18, }, // Iran
    {  691, 1008,  977383,  977396,  977417, 13, 21, 19, }, // Irkutsk
    {  691, 1016,  977436,  977447,  977466, 11, 19, 18, }, // Israel
    {  691, 1023,  977484,  977494,  977512, 10, 18, 17, }, // Japan
    {  691, 1047,       0,  977529,       0,  0, 15,  0, }, // Kazakhstan
    {  691, 1058,       0,  977544,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  691, 1077,       0,  977568,       0,  0, 27,  0, }, // Kazakhstan_Western
    {  691, 1106,  977595,  977605,  977623, 10, 18, 17, }, // Korea
    {  691, 1112,       0,  977640,       0,  0, 10,  0, }, // Kosrae
    {  691, 1119,  977650,  977669,  977696, 19, 27, 24, }, // Krasnoyarsk
    {  691, 1151,       0,  977720,       0,  0, 16,  0, }, // Kyrgystan
    {  691, 1175,       0,  977736,       0,  0, 15,  0, }, // Line_Islands
    {  691, 1188,  977751,  977764,  977785, 13, 21, 21, }, // Lord_Howe
    {  691, 1204,  977806,  977822,  977841, 16, 19, 18, }, // Magadan
    {  691, 1219,       0,  977859,       0,  0, 13,  0, }, // Malaysia
    {  691, 1228,       0,  977872,       0,  0, 14,  0, }, // Maldives
    {  691, 1237,       0,  977886,       0,  0, 12,  0, }, // Marquesas
    {  691, 1247,       0,  977898,       0,  0, 16,  0, }, // Marshall_Islands
    {  691, 1264,  977914,  977925,  977944, 11, 19, 17, }, // Mauritius
    {  691, 1274,       0,  977961,       0,  0, 10,  0, }, // Mawson
    {  691, 1281,  977971,  977993,  978023, 22, 30, 29, }, // Mexico_Pacific
    {  691, 1296,  978052,  978065,  978086, 13, 21, 19, }, // Mongolia
    {  691, 1305,  978105,  978114,  978131,  9, 17, 15, }, // Moscow
    {  691, 1312,       0,  978146,       0,  0, 13,  0, }, // Myanmar
    {  691, 1320,       0,  978159,       0,  0,  9,  0, }, // Nauru
    {  691, 1326,       0,  978168,       0,  0, 10,  0, }, // Nepal
    {  691, 1332,  978178,  978197,  978224, 19, 27, 25, }, // New_Caledonia
    {  691, 1346,  978249,  978265,  978289, 16, 24, 23, }, // New_Zealand
    {  691, 1358,  978312,  978332,  978360, 20, 28, 27, }, // Newfoundland
    {  691, 1371,       0,  978387,       0,  0,  9,  0, }, // Niue
    {  691, 1376,  978396,  978411,  978434, 15, 23, 22, }, // Norfolk
    {  691, 1384,  978456,  978477,  978506, 21, 29, 26, }, // Noronha
    {  691, 1406,  978532,  978548,  978572, 16, 24, 20, }, // Novosibirsk
    {  691, 1418,  978592,  978602,  978620, 10, 18, 16, }, // Omsk
    {  691, 1428,  978636,  978650,  978672, 14, 22, 20, }, // Pakistan
    {  691, 1437,       0,  978692,       0,  0,  9,  0, }, // Palau
    {  691, 1443,       0,  978701,       0,  0, 21,  0, }, // Papua_New_Guinea
    {  691, 1460,  978722,  978735,  978756, 13, 21, 19, }, // Paraguay
    {  691, 1469,  978775,  978784,  978801,  9, 17, 15, }, // Peru
    {  691, 1474,  978816,  978828,  978848, 12, 20, 18, }, // Philippines
    {  691, 1486,       0,  978866,       0,  0, 18,  0, }, // Phoenix_Islands
    {  691, 1502,  978884,  978915,  978954, 31, 39, 38, }, // Pierre_Miquelon
    {  691, 1518,       0,  978992,       0,  0, 12,  0, }, // Pitcairn
    {  691, 1527,       0,  979004,       0,  0, 10,  0, }, // Ponape
    {  691, 1534,       0,  979014,       0,  0, 16,  0, }, // Pyongyang
    {  691, 1554,       0,  979030,       0,  0, 13,  0, }, // Reunion
    {  691, 1562,       0,  979043,       0,  0, 10,  0, }, // Rothera
    {  691, 1570,  979053,  979064,  979083, 11, 19, 17, }, // Sakhalin
    {  691, 1596,  979100,  979109,  979126,  9, 17, 16, }, // Samoa
    {  691, 1602,       0,  979142,       0,  0, 12,  0, }, // Seychelles
    {  691, 1624,       0,  979154,       0,  0, 22,  0, }, // Singapore
    {  691, 1634,       0,  979176,       0,  0, 15,  0, }, // Solomon
    {  691, 1642,       0,  979191,       0,  0, 17,  0, }, // South_Georgia
    {  691, 1656,       0,  979208,       0,  0, 12,  0, }, // Suriname
    {  691, 1676,       0,  979220,       0,  0,  8,  0, }, // Syowa
    {  691, 1682,       0,  979228,       0,  0, 11,  0, }, // Tahiti
    {  691, 1689,  979239,  979249,  979266, 10, 17, 17, }, // Taipei
    {  691, 1696,       0,  979283,       0,  0, 16,  0, }, // Tajikistan
    {  691, 1724,       0,  979299,       0,  0, 11,  0, }, // Tokelau
    {  691, 1732,  979310,  979320,  979338, 10, 18, 16, }, // Tonga
    {  691, 1738,       0,  979354,       0,  0,  8,  0, }, // Truk
    {  691, 1750,  979362,  979381,  979408, 19, 27, 25, }, // Turkmenistan
    {  691, 1763,       0,  979433,       0,  0, 11,  0, }, // Tuvalu
    {  691, 1777,  979444,  979456,  979476, 12, 20, 18, }, // Uruguay
    {  691, 1792,  979494,  979511,  979536, 17, 25, 23, }, // Uzbekistan
    {  691, 1803,  979559,  979570,  979589, 11, 19, 17, }, // Vanuatu
    {  691, 1811,       0,  979606,       0,  0, 14,  0, }, // Venezuela
    {  691, 1821,  979620,  979636,  979660, 16, 24, 22, }, // Vladivostok
    {  691, 1833,  979682,  979696,  979718, 14, 22, 20, }, // Volgograd
    {  691, 1843,       0,  979626,       0,  0, 10,  0, }, // Vostok
    {  691, 1850,       0,  979738,       0,  0, 12,  0, }, // Wake
    {  691, 1855,       0,  979750,       0,  0, 22,  0, }, // Wallis
    {  691, 1862,  979772,  979785,  979806, 13, 21, 19, }, // Yakutsk
    {  691, 1870,  979825,  979843,  979869, 18, 26, 24, }, // Yekaterinburg
    {  692,    5,       0,  780430,       0,  0, 17,  0, }, // Afghanistan Kuvi/Telugu/India
    {  692,   17,       0,  780447,       0,  0, 21,  0, }, // Africa_Central
    {  692,   32,       0,  780468,       0,  0, 19,  0, }, // Africa_Eastern
    {  692,   65,       0,  780487,       0,  0, 29,  0, }, // Africa_Southern
    {  692,   81,  780516,  780535,  780564, 19, 29, 25, }, // Africa_Western
    {  692,  107,  780589,  780601,  780623, 12, 22, 26, }, // Alaska
    {  692,  135,  780701,  780713,  780735, 12, 22, 18, }, // Amazon
    {  692,  142,  780753,  780763,  780783, 10, 20, 24, }, // America_Central
    {  692,  158,  780807,  780818,  780839, 11, 21, 25, }, // America_Eastern
    {  692,  174,  780864,  780878,  780902, 14, 24, 28, }, // America_Mountain
    {  692,  191,  780930,  780942,  780964, 12, 22, 26, }, // America_Pacific
    {  692,  214,  781045,  781055,  781075, 10, 20, 15, }, // Apia
    {  692,  232,  781194,  781207,  781230, 13, 23, 27, }, // Arabian
    {  692,  240,  781257,  781272,  781297, 15, 25, 21, }, // Argentina
    {  692,  250,  781318,  781340,  781372, 22, 32, 28, }, // Argentina_Western
    {  692,  268,  781400,  781414,  781438, 14, 24, 20, }, // Armenia
    {  692,  286,  781458,  781473,  781498, 15, 25, 29, }, // Atlantic
    {  692,  295,  781527,  781549,  781581, 22, 32, 36, }, // Australia_Central
    {  692,  313,  781617,  781646,  781686, 29, 40, 43, }, // Australia_CentralWestern
    {  692,  338,  781729,  781752,  781786, 23, 34, 38, }, // Australia_Eastern
    {  692,  356,  781824,  781652,  781847, 23, 34, 38, }, // Australia_Western
    {  692,  374,  781885,  781900,  781925, 15, 25, 21, }, // Azerbaijan
    {  692,  385,  781946,  781958,  781980, 12, 22, 18, }, // Azores
    {  692,  397,  781998,  782013,  782038, 15, 25, 21, }, // Bangladesh
    {  692,  415,       0,  782059,       0,  0, 11,  0, }, // Bhutan
    {  692,  422,       0,  782070,       0,  0, 13,  0, }, // Bolivia
    {  692,  437,  782083,  782098,  782123, 15, 25, 21, }, // Brasilia
    {  692,  454,       0,  782144,       0,  0, 20,  0, }, // Brunei
    {  692,  461,  782164,  782180,  782206, 16, 26, 22, }, // Cape_Verde
    {  692,  478,       0,  782228,       0,  0, 22,  0, }, // Chamorro
    {  692,  487,  782250,  782260,  782280, 10, 20, 24, }, // Chatham
    {  692,  495,  782304,  782313,  782332,  9, 19, 15, }, // Chile
    {  692,  501,  782347,  782356,  782375,  9, 19, 23, }, // China
    {  692,  507,       0,  782398,       0,  0, 19,  0, }, // Christmas
    {  692,  517,       0,  782417,       0,  0, 17,  0, }, // Cocos
    {  692,  523,  782434,  782447,  782470, 13, 23, 19, }, // Colombia
    {  692,  532,  782489,  782504,  782529, 15, 25, 26, }, // Cook
    {  692,  537,  782555,  782566,  782587, 11, 21, 25, }, // Cuba
    {  692,  548,       0,  782612,       0,  0, 11,  0, }, // Davis
    {  692,  564,       0,  782623,       0,  0, 27,  0, }, // DumontDUrville
    {  692,  601,       0,  782650,       0,  0, 18,  0, }, // East_Timor
    {  692,  612,  782668,  782684,  782710, 16, 26, 22, }, // Easter
    {  692,  619,       0,  782732,       0,  0, 13,  0, }, // Ecuador
    {  692,  627,  782745,  782768,  782801, 23, 33, 29, }, // Europe_Central
    {  692,  642,  782830,  782851,  782882, 21, 31, 27, }, // Europe_Eastern
    {  692,  657,       0,  782909,       0,  0, 27,  0, }, // Europe_Further_Eastern
    {  692,  680,  782936,  782957,  782988, 21, 31, 27, }, // Europe_Western
    {  692,  695,  783015,  783038,  783071, 23, 33, 29, }, // Falkland
    {  692,  704,  783100,  783109,  783128,  9, 19, 15, }, // Fiji
    {  692,  709,       0,  783143,       0,  0, 18,  0, }, // French_Guiana
    {  692,  723,       0,  783161,       0,  0, 38,  0, }, // French_Southern
    {  692,  746,       0,  783199,       0,  0, 15,  0, }, // Galapagos
    {  692,  756,       0,  783214,       0,  0, 13,  0, }, // Gambier
    {  692,  764,  783227,  783240,  783263, 13, 23, 19, }, // Georgia
    {  692,  772,       0,  783282,       0,  0, 20,  0, }, // Gilbert_Islands
    {  692,  788,       0,  783302,       0,  0, 21,  0, }, // GMT
    {  692,  830,  783323,  783349,  783385, 26, 36, 32, }, // Greenland_Eastern
    {  692,  848,  783417,  783443,  783479, 26, 36, 32, }, // Greenland_Western
    {  692,  871,       0,  783532,       0,  0, 20,  0, }, // Gulf
    {  692,  876,       0,  783151,       0,  0, 10,  0, }, // Guyana
    {  692,  883,  783552,  783573,  783604, 21, 31, 35, }, // Hawaii_Aleutian
    {  692,  899,  783639,  783652,  783675, 13, 23, 19, }, // Hong_Kong
    {  692,  909,  783694,  783705,  783726, 11, 21, 17, }, // Hovd
    {  692,  914,       0,  783743,       0,  0, 22,  0, }, // India
    {  692,  920,       0,  783765,       0,  0, 21,  0, }, // Indian_Ocean
    {  692,  933,       0,  783786,       0,  0, 13,  0, }, // Indochina
    {  692,  943,       0,  783799,       0,  0, 24,  0, }, // Indonesia_Central
    {  692,  961,       0,  783823,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  692,  979,       0,  783845,       0,  0, 22,  0, }, // Indonesia_Western
    {  692,  997,  783867,  783877,  783897, 10, 20, 24, }, // Iran
    {  692, 1008,  783921,  783937,  783963, 16, 26, 22, }, // Irkutsk
    {  692, 1016,  783985,  783999,  784023, 14, 24, 28, }, // Israel
    {  692, 1023,  784051,  784061,  784081, 10, 20, 24, }, // Japan
    {  692, 1047,       0,  784251,       0,  0, 16,  0, }, // Kazakhstan
    {  692, 1058,       0,  784244,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  692, 1077,       0,  784267,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  692, 1106,  784289,  784301,  784323, 12, 22, 26, }, // Korea
    {  692, 1112,       0,  784349,       0,  0, 13,  0, }, // Kosrae
    {  692, 1119,  784362,  784383,  784414, 21, 31, 27, }, // Krasnoyarsk
    {  692, 1151,       0,  784441,       0,  0, 17,  0, }, // Kyrgystan
    {  692, 1175,       0,  784467,       0,  0, 15,  0, }, // Line_Islands
    {  692, 1188,  784482,  784498,  784524, 16, 26, 21, }, // Lord_Howe
    {  692, 1204,  784591,  784602,  784623, 11, 21, 17, }, // Magadan
    {  692, 1219,       0,  784640,       0,  0, 12,  0, }, // Malaysia
    {  692, 1228,       0,  784652,       0,  0, 14,  0, }, // Maldives
    {  692, 1237,       0,  784666,       0,  0, 17,  0, }, // Marquesas
    {  692, 1247,       0,  784683,       0,  0, 18,  0, }, // Marshall_Islands
    {  692, 1264,  784701,  784713,  784735, 12, 22, 18, }, // Mauritius
    {  692, 1274,       0,  784753,       0,  0, 10,  0, }, // Mawson
    {  692, 1281,  784763,  784785,  784817, 22, 32, 36, }, // Mexico_Pacific
    {  692, 1296,  784853,  784868,  784893, 15, 25, 21, }, // Mongolia
    {  692, 1305,  784914,  784925,  784946, 11, 21, 17, }, // Moscow
    {  692, 1312,       0,  784963,       0,  0, 13,  0, }, // Myanmar
    {  692, 1320,       0,  784976,       0,  0,  9,  0, }, // Nauru
    {  692, 1326,       0,  784985,       0,  0, 11,  0, }, // Nepal
    {  692, 1332,  784996,  785016,  785046, 20, 30, 26, }, // New_Caledonia
    {  692, 1346,  785072,  785090,  785118, 18, 28, 32, }, // New_Zealand
    {  692, 1358,  785150,  785172,  785204, 22, 32, 36, }, // Newfoundland
    {  692, 1371,       0,  785240,       0,  0,  9,  0, }, // Niue
    {  692, 1376,  785249,  785267,  785295, 18, 28, 23, }, // Norfolk
    {  692, 1384,  785318,  785344,  785380, 26, 36, 32, }, // Noronha
    {  692, 1406,  785437,  785456,  785485, 19, 29, 25, }, // Novosibirsk
    {  692, 1418,  785510,  785522,  785544, 12, 22, 18, }, // Omsk
    {  692, 1428,  785562,  785577,  785602, 15, 25, 21, }, // Pakistan
    {  692, 1437,       0,  785623,       0,  0, 11,  0, }, // Palau
    {  692, 1443,       0,  785634,       0,  0, 23,  0, }, // Papua_New_Guinea
    {  692, 1460,  785657,  785669,  785691, 12, 22, 18, }, // Paraguay
    {  692, 1469,  785709,  785718,  785737,  9, 19, 15, }, // Peru
    {  692, 1474,  785752,  785767,  785792, 15, 25, 21, }, // Philippines
    {  692, 1486,       0,  785813,       0,  0, 19,  0, }, // Phoenix_Islands
    {  692, 1502,  785832,  785868,  785914, 36, 46, 49, }, // Pierre_Miquelon
    {  692, 1518,       0,  785963,       0,  0, 16,  0, }, // Pitcairn
    {  692, 1527,       0,  785979,       0,  0, 11,  0, }, // Ponape
    {  692, 1534,       0,  785990,       0,  0, 15,  0, }, // Pyongyang
    {  692, 1554,       0,  786066,       0,  0, 14,  0, }, // Reunion
    {  692, 1562,       0,  786080,       0,  0, 11,  0, }, // Rothera
    {  692, 1570,  786091,  786103,  786125, 12, 22, 18, }, // Sakhalin
    {  692, 1596,  786189,  786199,  786219, 10, 20, 24, }, // Samoa
    {  692, 1602,       0,  786243,       0,  0, 13,  0, }, // Seychelles
    {  692, 1624,       0,  786256,       0,  0, 23,  0, }, // Singapore
    {  692, 1634,       0,  786279,       0,  0, 17,  0, }, // Solomon
    {  692, 1642,       0,  786296,       0,  0, 20,  0, }, // South_Georgia
    {  692, 1656,       0,  786316,       0,  0, 13,  0, }, // Suriname
    {  692, 1676,       0,  786329,       0,  0, 11,  0, }, // Syowa
    {  692, 1682,       0,  786340,       0,  0, 10,  0, }, // Tahiti
    {  692, 1689,  786350,  786359,  786378,  9, 19, 23, }, // Taipei
    {  692, 1696,       0,  786401,       0,  0, 16,  0, }, // Tajikistan
    {  692, 1724,       0,  786417,       0,  0, 13,  0, }, // Tokelau
    {  692, 1732,  786430,  786440,  786460, 10, 20, 16, }, // Tonga
    {  692, 1738,       0,  786476,       0,  0,  8,  0, }, // Truk
    {  692, 1750,  786484,  786506,  786538, 22, 32, 28, }, // Turkmenistan
    {  692, 1763,       0,  786566,       0,  0, 11,  0, }, // Tuvalu
    {  692, 1777,  786577,  786589,  786611, 12, 22, 18, }, // Uruguay
    {  692, 1792,  786629,  786647,  786675, 18, 28, 24, }, // Uzbekistan
    {  692, 1803,  786699,  786709,  786729, 10, 20, 16, }, // Vanuatu
    {  692, 1811,       0,  786745,       0,  0, 13,  0, }, // Venezuela
    {  692, 1821,  786758,  786777,  786806, 19, 29, 25, }, // Vladivostok
    {  692, 1833,  786831,  786848,  786875, 17, 27, 23, }, // Volgograd
    {  692, 1843,       0,  786764,       0,  0, 13,  0, }, // Vostok
    {  692, 1850,       0,  786898,       0,  0, 14,  0, }, // Wake
    {  692, 1855,       0,  786912,       0,  0, 24,  0, }, // Wallis
    {  692, 1862,  786936,  786951,  786976, 15, 25, 21, }, // Yakutsk
    {  692, 1870,  786997,  787017,  787047, 20, 30, 26, }, // Yekaterinburg
    {  695,  788,       0,  979893,       0,  0, 20,  0, }, // GMT Swampy Cree/Canadian Aboriginal/Canada
    {  696,    0,       0,       0,       0,  0,  0,  0, } // Terminal row
}; // Metazone long name table

// Sorted by locale index, then meta key
static inline constexpr LocaleMetaZoneShortNames localeMetaZoneShortNameTable[] = {
    // locInd, metaKey, (generic, standard, DST){ind, sz}
    {    0,  107,    0,    3,    7,  3,  4,  4, }, // Alaska C/AnyScript/AnyTerritory
    {    0,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {    0,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {    0,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {    0,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {    0,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {    0,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {    0,  883,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {    5,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central Afrikaans/Latin/South Africa
    {    5,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {    5,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {    5,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {   14,  871,    0,   82,    0,  0,  3,  0, }, // Gulf Arabic/Arabic/Egypt
    {   44,  914,    0,   85,    0,  0,  6,  0, }, // India Assamese/Bangla/India
    {   45,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Asturian/Latin/Spain
    {   45,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {   45,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {   45,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {   45,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {   45,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {   45,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {   45,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {   45,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {   45,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {   45,  883,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {   56,  914,    0,  112,    0,  0,  3,  0, }, // India Bangla/Bangla/Bangladesh
    {   60,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Basque/Latin/Spain
    {   60,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {   60,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {   60,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {   60,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {   60,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {   60,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {   60,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {   60,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {   60,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {   60,  883,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {   66,  914,    0,  112,    0,  0,  3,  0, }, // India Bodo/Devanagari/India
    {   67,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Bosnian/Latin/Bosnia and Herzegovina
    {   67,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {   67,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {   67,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {   76,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Catalan/Latin/Spain
    {   76,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {   76,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {   76,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {   84,  914,    0,  112,    0,  0,  3,  0, }, // India Chakma/Chakma/Bangladesh
    {   87,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Cherokee/Cherokee/United States
    {   87,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {   87,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {   87,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {   87,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {   87,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {   87,  883,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {   95, 1219,    0,  115,    0,  0,  3,  0, }, // Malaysia Chinese/Simplified Han/Singapore
    {   95, 1624,    0,  118,    0,  0,  3,  0, }, // Singapore
    {   96,  142,   11,   13,   16,  2,  3,  3, }, // America_Central Chinese/Traditional Han/Hong Kong
    {   96,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {   96,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {   96,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {   97,  142,   11,   13,   16,  2,  3,  3, }, // America_Central Chinese/Traditional Han/Macao
    {   97,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {   97,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {   97,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {   99,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Chinese/Traditional Han/Taiwan
    {   99,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {   99,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {   99,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {   99,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {   99,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {   99,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {   99,  883,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {  101,    0,  121,  121,  129,  8,  8,  9, }, // Acre Chuvash/Cyrillic/Russia
    {  101,    5,    0,  138,    0,  0,  3,  0, }, // Afghanistan
    {  101,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central
    {  101,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  101,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  101,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  101,  107,    0,    3,    7,  3,  4,  4, }, // Alaska
    {  101,  128,  141,  141,  145,  4,  4,  8, }, // Almaty
    {  101,  135,  153,  153,  156,  3,  3,  4, }, // Amazon
    {  101,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  101,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  101,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  101,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  101,  207,  160,  160,  164,  4,  4,  5, }, // Anadyr
    {  101,  214,  169,  169,  183, 14, 14, 18, }, // Apia
    {  101,  219,  201,  201,  211, 10, 10, 14, }, // Aqtau
    {  101,  225,  225,  225,  229,  4,  4,  8, }, // Aqtobe
    {  101,  232,  237,  237,  247, 10, 10, 10, }, // Arabian
    {  101,  240,  257,  257,  260,  3,  3,  4, }, // Argentina
    {  101,  250,  264,  264,  268,  4,  4,  5, }, // Argentina_Western
    {  101,  268,  273,  273,  284, 11, 11, 12, }, // Armenia
    {  101,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  101,  295,  129,  129,  296,  4,  4,  4, }, // Australia_Central
    {  101,  313,  300,  300,  305,  5,  5,  5, }, // Australia_CentralWestern
    {  101,  338,  310,  310,  314,  4,  4,  4, }, // Australia_Eastern
    {  101,  356,  318,  318,  322,  4,  4,  4, }, // Australia_Western
    {  101,  374,  326,  326,  329,  3,  3,  4, }, // Azerbaijan
    {  101,  385,  333,  333,  337,  4,  4,  5, }, // Azores
    {  101,  397,  342,  342,  356, 14, 14, 14, }, // Bangladesh
    {  101,  415,    0,  370,    0,  0,  3,  0, }, // Bhutan
    {  101,  422,    0,  373,    0,  0,  3,  0, }, // Bolivia
    {  101,  437,  376,  376,  379,  3,  3,  4, }, // Brasilia
    {  101,  454,    0,  383,    0,  0,  3,  0, }, // Brunei
    {  101,  461,  386,  386,  389,  3,  3,  4, }, // Cape_Verde
    {  101,  472,    0,  393,    0,  0,  4,  0, }, // Casey
    {  101,  478,    0,  397,    0,  0,  4,  0, }, // Chamorro
    {  101,  487,  401,  401,  406,  5,  5,  5, }, // Chatham
    {  101,  495,  411,  411,  414,  3,  3,  4, }, // Chile
    {  101,  501,  418,  418,  427,  9,  9, 13, }, // China
    {  101,  507,    0,  440,    0,  0,  3,  0, }, // Christmas
    {  101,  517,    0,  443,    0,  0,  3,  0, }, // Cocos
    {  101,  523,  446,  446,  449,  3,  3,  4, }, // Colombia
    {  101,  532,  453,  453,  456,  3,  3,  7, }, // Cook
    {  101,  537,  463,  463,  471,  8,  8,  8, }, // Cuba
    {  101,  548,    0,  479,    0,  0,  4,  0, }, // Davis
    {  101,  564,    0,  483,    0,  0,  4,  0, }, // DumontDUrville
    {  101,  601,    0,  487,    0,  0,  3,  0, }, // East_Timor
    {  101,  612,  490,  490,  494,  4,  4,  5, }, // Easter
    {  101,  619,    0,  499,    0,  0, 11,  0, }, // Ecuador
    {  101,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  101,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  101,  657,    0,  510,    0,  0,  4,  0, }, // Europe_Further_Eastern
    {  101,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  101,  695,  514,  514,  517,  3,  3,  4, }, // Falkland
    {  101,  704,  521,  521,  524,  3,  3, 10, }, // Fiji
    {  101,  709,    0,  534,    0,  0,  3,  0, }, // French_Guiana
    {  101,  723,    0,  537,    0,  0,  4,  0, }, // French_Southern
    {  101,  746,    0,  541,    0,  0,  4,  0, }, // Galapagos
    {  101,  756,    0,  545,    0,  0,  4,  0, }, // Gambier
    {  101,  764,  549,  549,  552,  3,  3,  4, }, // Georgia
    {  101,  772,    0,  556,    0,  0,  3,  0, }, // Gilbert_Islands
    {  101,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  101,  802,  559,  559,  563,  4,  4,  5, }, // Greenland
    {  101,  830,  568,  568,  571,  3,  3,  4, }, // Greenland_Eastern
    {  101,  848,  575,  575,  578,  3,  3,  4, }, // Greenland_Western
    {  101,  866,    0,  582,    0,  0,  8,  0, }, // Guam
    {  101,  871,    0,  590,    0,  0,  8,  0, }, // Gulf
    {  101,  876,    0,  598,    0,  0,  3,  0, }, // Guyana
    {  101,  883,  601,  601,  604,  3,  3,  3, }, // Hawaii_Aleutian
    {  101,  899,  607,  607,  610,  3,  3,  7, }, // Hong_Kong
    {  101,  909,  617,  617,  621,  4,  4,  5, }, // Hovd
    {  101,  914,    0,  626,    0,  0, 10,  0, }, // India
    {  101,  920,    0,  636,    0,  0,  3,  0, }, // Indian_Ocean
    {  101,  933,    0,  639,    0,  0,  3,  0, }, // Indochina
    {  101,  943,    0,  642,    0,  0,  3,  0, }, // Indonesia_Central
    {  101,  961,    0,  645,    0,  0,  3,  0, }, // Indonesia_Eastern
    {  101,  979,    0,  648,    0,  0,  3,  0, }, // Indonesia_Western
    {  101,  997,  651,  651,  655,  4,  4,  8, }, // Iran
    {  101, 1008,  663,  663,  667,  4,  4,  5, }, // Irkutsk
    {  101, 1016,  672,  672,  682, 10, 10,  3, }, // Israel
    {  101, 1023,  685,  685,  688,  3,  3,  7, }, // Japan
    {  101, 1029,  695,  699,  704,  4,  5,  5, }, // Kamchatka
    {  101, 1047,    0,  709,    0,  0, 13,  0, }, // Kazakhstan
    {  101, 1058,    0,  722,    0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  101, 1077,    0,  746,    0,  0, 18,  0, }, // Kazakhstan_Western
    {  101, 1106,    4,    4,  764,  3,  3,  7, }, // Korea
    {  101, 1112,    0,  771,    0,  0,  4,  0, }, // Kosrae
    {  101, 1119,  775,  775,  779,  4,  4,  5, }, // Krasnoyarsk
    {  101, 1151,    0,  784,    0,  0,  3,  0, }, // Kyrgystan
    {  101, 1161,    0,  787,    0,  0,  4,  0, }, // Lanka
    {  101, 1175,    0,  791,    0,  0,  4,  0, }, // Line_Islands
    {  101, 1188,  795,  795,  799,  4,  4, 11, }, // Lord_Howe
    {  101, 1198,  810,  810,  819,  9,  9, 13, }, // Macau
    {  101, 1204,  832,  832,  836,  4,  4,  5, }, // Magadan
    {  101, 1219,    0,  115,    0,  0,  3,  0, }, // Malaysia
    {  101, 1228,    0,  841,    0,  0,  3,  0, }, // Maldives
    {  101, 1237,    0,  844,    0,  0,  4,  0, }, // Marquesas
    {  101, 1247,    0,  848,    0,  0,  3,  0, }, // Marshall_Islands
    {  101, 1264,  851,  851,  854,  3,  3,  4, }, // Mauritius
    {  101, 1274,    0,  858,    0,  0,  4,  0, }, // Mawson
    {  101, 1281,  862,  862,  866,  4,  4,  4, }, // Mexico_Pacific
    {  101, 1296,  870,  870,  874,  4,  4,  5, }, // Mongolia
    {  101, 1305,  879,  882,  885,  3,  3,  3, }, // Moscow
    {  101, 1312,    0,  888,    0,  0,  3,  0, }, // Myanmar
    {  101, 1320,    0,  891,    0,  0,  3,  0, }, // Nauru
    {  101, 1326,    0,  894,    0,  0,  3,  0, }, // Nepal
    {  101, 1332,  897,  897,  900,  3,  3, 17, }, // New_Caledonia
    {  101, 1346,  917,  917,  921,  4,  4,  4, }, // New_Zealand
    {  101, 1358,  560,  560,  900,  3,  3,  3, }, // Newfoundland
    {  101, 1371,    0,  925,    0,  0,  3,  0, }, // Niue
    {  101, 1376,  928,  928,  931,  3,  3,  4, }, // Norfolk
    {  101, 1384,  935,  935,  938,  3,  3,  4, }, // Noronha
    {  101, 1392,    0,  942,    0,  0,  8,  0, }, // North_Mariana
    {  101, 1406,  950,  950,  954,  4,  4,  5, }, // Novosibirsk
    {  101, 1418,  959,  959,  963,  4,  4,  5, }, // Omsk
    {  101, 1428,  968,  968,  971,  3,  3,  7, }, // Pakistan
    {  101, 1437,    0,  978,    0,  0,  3,  0, }, // Palau
    {  101, 1443,    0,  981,    0,  0,  3,  0, }, // Papua_New_Guinea
    {  101, 1460,  984,  984,  987,  3,  3,  4, }, // Paraguay
    {  101, 1469,  695,  695,  991,  3,  3,  8, }, // Peru
    {  101, 1474,  999,  999, 1013, 14, 14, 18, }, // Philippines
    {  101, 1486,    0, 1031,    0,  0,  4,  0, }, // Phoenix_Islands
    {  101, 1502, 1035, 1035, 1039,  4,  4,  4, }, // Pierre_Miquelon
    {  101, 1518,    0, 1043,    0,  0, 12,  0, }, // Pitcairn
    {  101, 1527,    0, 1055,    0,  0,  4,  0, }, // Ponape
    {  101, 1534,    0, 1059,    0,  0,  9,  0, }, // Pyongyang
    {  101, 1544, 1068, 1068, 1072,  4,  4,  8, }, // Qyzylorda
    {  101, 1554,    0, 1080,    0,  0,  3,  0, }, // Reunion
    {  101, 1562,    0, 1083,    0,  0,  4,  0, }, // Rothera
    {  101, 1570, 1087, 1087, 1091,  4,  4,  5, }, // Sakhalin
    {  101, 1579, 1096, 1096, 1100,  4,  4,  5, }, // Samara
    {  101, 1596,  169,  169, 1105,  9,  9, 13, }, // Samoa
    {  101, 1602,    0, 1118,    0,  0,  3,  0, }, // Seychelles
    {  101, 1624,    0,  118,    0,  0,  3,  0, }, // Singapore
    {  101, 1634,    0, 1121,    0,  0,  3,  0, }, // Solomon
    {  101, 1642,    0,   82,    0,  0,  3,  0, }, // South_Georgia
    {  101, 1656,    0, 1124,    0,  0,  3,  0, }, // Suriname
    {  101, 1676,    0, 1127,    0,  0,  4,  0, }, // Syowa
    {  101, 1682,    0, 1131,    0,  0,  4,  0, }, // Tahiti
    {  101, 1689, 1135, 1135, 1138,  3,  3,  7, }, // Taipei
    {  101, 1696,    0, 1145,    0,  0,  3,  0, }, // Tajikistan
    {  101, 1724,    0, 1148,    0,  0,  3,  0, }, // Tokelau
    {  101, 1732, 1151, 1151, 1154,  3,  3,  4, }, // Tonga
    {  101, 1738,    0, 1158,    0,  0,  4,  0, }, // Truk
    {  101, 1750,   26,   26, 1162,  3,  3,  7, }, // Turkmenistan
    {  101, 1763,    0, 1169,    0,  0,  3,  0, }, // Tuvalu
    {  101, 1777, 1172, 1172, 1175,  3,  3,  4, }, // Uruguay
    {  101, 1792, 1179, 1179, 1182,  3,  3,  7, }, // Uzbekistan
    {  101, 1803, 1189, 1189, 1192,  3,  3,  7, }, // Vanuatu
    {  101, 1811,    0, 1199,    0,  0,  5,  0, }, // Venezuela
    {  101, 1821, 1204, 1204, 1208,  4,  4,  5, }, // Vladivostok
    {  101, 1833, 1213, 1213, 1217,  4,  4,  5, }, // Volgograd
    {  101, 1843,    0, 1222,    0,  0,  4,  0, }, // Vostok
    {  101, 1850,    0, 1226,    0,  0,  4,  0, }, // Wake
    {  101, 1855,    0, 1230,    0,  0,  3,  0, }, // Wallis
    {  101, 1862, 1233, 1233, 1237,  4,  4,  5, }, // Yakutsk
    {  101, 1870, 1242, 1242, 1246,  4,  4,  5, }, // Yekaterinburg
    {  101, 1892,    0,  988,    0,  0,  3,  0, }, // Yukon
    {  102,  627, 1251, 1251, 1254,  3,  3,  4, }, // Europe_Central Colognian/Latin/Germany
    {  102,  642, 1258, 1258, 1261,  3,  3,  4, }, // Europe_Eastern
    {  102,  680, 1265, 1265, 1268,  3,  3,  4, }, // Europe_Western
    {  104,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Cornish/Latin/United Kingdom
    {  104,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  104,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  104,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  106,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Croatian/Latin/Croatia
    {  106,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  106,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  106,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  108,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Czech/Latin/Czechia
    {  108,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  108,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  108,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  108,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  108,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  108,  627, 1272, 1272, 1275,  3,  3,  4, }, // Europe_Central
    {  109,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Danish/Latin/Denmark
    {  109,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  109,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  114,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Dutch/Latin/Netherlands
    {  114,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  114,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  114,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  114,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  114,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  114,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  114,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  114,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  114,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  114,  883,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {  120, 1656,    0, 1124,    0,  0,  3,  0, }, // Suriname Dutch/Latin/Suriname
    {  121,  415,    0,  370,    0,  0,  2,  0, }, // Bhutan Dzongkha/Tibetan/Bhutan
    {  123,  107,    0,    3,    7,  3,  4,  4, }, // Alaska English/Latin/United States
    {  123,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  123,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  123,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  123,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  123,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  123,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  123,  883,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {  126,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Anguilla
    {  126,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  126,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  126,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  126,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  126,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  126,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  127,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Antigua and Barbuda
    {  127,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  127,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  127,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  127,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  127,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  127,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  128,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Australia
    {  128,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  128,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  128,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  128,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  128,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  128,  295,  121,  129,  296,  3,  4,  4, }, // Australia_Central
    {  128,  313, 1282,  300,  305,  4,  5,  5, }, // Australia_CentralWestern
    {  128,  338, 1286,  310,  314,  3,  4,  4, }, // Australia_Eastern
    {  128,  356,  859,  318,  322,  3,  4,  4, }, // Australia_Western
    {  128,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  128, 1188, 1289,  795, 1292,  3,  4,  4, }, // Lord_Howe
    {  128, 1346, 1296,  917,  921,  3,  4,  4, }, // New_Zealand
    {  129,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Austria
    {  129,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  129,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  129,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  129,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  129,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  129,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  129,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  129,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  129,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  130,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Bahamas
    {  130,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  130,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  130,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  130,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  130,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  130,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  131,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Barbados
    {  131,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  131,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  131,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  131,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  131,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  131,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  132,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Belgium
    {  132,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  132,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  132,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  132,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  132,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  132,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  132,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  132,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  132,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  133,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Belize
    {  133,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  133,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  133,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  133,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  133,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  133,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  134,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Bermuda
    {  134,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  134,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  134,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  134,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  134,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  134,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  135,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Botswana
    {  135,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  135,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  135,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  135,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  135,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  135,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  135,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  135,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  135,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  135,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  136,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/British Indian Ocean Territory
    {  136,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  136,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  136,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  136,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  136,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  136,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  137,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/British Virgin Islands
    {  137,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  137,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  137,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  137,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  137,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  137,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  139,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Cameroon
    {  139,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  139,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  139,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  139,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  139,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  139,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  139,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  139,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  139,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  139,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  140,    5,    0,  138,    0,  0,  3,  0, }, // Afghanistan English/Latin/Canada
    {  140,  107,    0,    3,    7,  3,  4,  4, }, // Alaska
    {  140,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  140,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  140,  240,  257,    0,    0,  3,  0,  0, }, // Argentina
    {  140,  313, 1282,  300,  305,  4,  5,  5, }, // Australia_CentralWestern
    {  140,  338, 1286,  310,  314,  3,  4,  4, }, // Australia_Eastern
    {  140,  356,    0,  318,  322,  0,  4,  4, }, // Australia_Western
    {  140,  397,    0,  342,    0,  0,  3,  0, }, // Bangladesh
    {  140,  415,    0,  370,    0,  0,  3,  0, }, // Bhutan
    {  140,  437,  376,    0,  379,  3,  0,  4, }, // Brasilia
    {  140,  454,    0,  383,    0,  0,  3,  0, }, // Brunei
    {  140,  487,    0,  401,  406,  0,  5,  5, }, // Chatham
    {  140,  507,    0,  440,    0,  0,  3,  0, }, // Christmas
    {  140,  517,    0,  443,    0,  0,  3,  0, }, // Cocos
    {  140,  523,    0,    0,  449,  0,  0,  4, }, // Colombia
    {  140,  601,    0,  487,    0,  0,  3,  0, }, // East_Timor
    {  140,  612,    0,  490,  494,  0,  4,  5, }, // Easter
    {  140,  619,    0,  499,    0,  0,  3,  0, }, // Ecuador
    {  140,  695,  514,    0,  517,  3,  0,  4, }, // Falkland
    {  140,  746,    0,  541,    0,  0,  4,  0, }, // Galapagos
    {  140,  830,  568,    0,    0,  3,  0,  0, }, // Greenland_Eastern
    {  140,  871,    0, 1299,    0,  0,  9,  0, }, // Gulf
    {  140,  876,    0,  598,    0,  0,  3,  0, }, // Guyana
    {  140,  914,    0,  112,    0,  0,  3,  0, }, // India
    {  140,  933,    0,  639,    0,  0,  3,  0, }, // Indochina
    {  140,  943,    0, 1308,    0,  0,  4,  0, }, // Indonesia_Central
    {  140,  961,    0,  648,    0,  0,  3,  0, }, // Indonesia_Eastern
    {  140,  979,    0, 1312,    0,  0,  3,  0, }, // Indonesia_Western
    {  140,  997,    0,  651, 1315,  0,  4,  4, }, // Iran
    {  140, 1219,    0,  115,    0,  0,  3,  0, }, // Malaysia
    {  140, 1228,    0,  841,    0,  0,  3,  0, }, // Maldives
    {  140, 1326,    0,  894,    0,  0,  3,  0, }, // Nepal
    {  140, 1358,  384,  560,  900,  2,  3,  3, }, // Newfoundland
    {  140, 1384,  935,    0,    0,  3,  0,  0, }, // Noronha
    {  140, 1428,    0,  968,    0,  0,  3,  0, }, // Pakistan
    {  140, 1460,  984,    0,  987,  3,  0,  4, }, // Paraguay
    {  140, 1469,  695,    0,    0,  3,  0,  0, }, // Peru
    {  140, 1502, 1319, 1035, 1039,  3,  4,  4, }, // Pierre_Miquelon
    {  140, 1777,    0, 1172, 1175,  0,  3,  4, }, // Uruguay
    {  140, 1811,    0, 1322,    0,  0,  3,  0, }, // Venezuela
    {  141,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Cayman Islands
    {  141,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  141,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  141,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  141,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  141,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  141,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  142,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Christmas Island
    {  142,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  142,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  142,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  142,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  142,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  142,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  143,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Cocos Islands
    {  143,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  143,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  143,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  143,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  143,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  143,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  144,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Cook Islands
    {  144,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  144,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  144,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  144,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  144,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  144,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  145,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Cyprus
    {  145,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  145,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  145,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  145,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  145,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  145,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  146,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Czechia
    {  146,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  146,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  146,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  146,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  146,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  146,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  146,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  146,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  146,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  147,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Denmark
    {  147,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  147,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  147,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  147,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  147,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  147,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  147,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  147,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  147,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  148,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Diego Garcia
    {  148,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  148,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  148,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  148,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  148,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  148,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  149,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Dominica
    {  149,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  149,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  149,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  149,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  149,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  149,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  150,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Eritrea
    {  150,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  150,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  150,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  150,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  150,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  150,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  150,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  150,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  150,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  150,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  151,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Eswatini
    {  151,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  151,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  151,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  151,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  151,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  151,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  151,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  151,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  151,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  151,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  152,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Europe
    {  152,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  152,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  152,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  152,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  152,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  152,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  152,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  152,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  152,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  153,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Falkland Islands
    {  153,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  153,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  153,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  153,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  153,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  153,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  154,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Fiji
    {  154,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  154,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  154,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  154,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  154,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  154,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  155,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Finland
    {  155,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  155,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  155,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  155,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  155,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  155,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  155,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  155,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  155,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  156,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/France
    {  156,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  156,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  156,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  156,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  156,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  156,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  156,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  156,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  156,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  157,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Gambia
    {  157,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  157,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  157,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  157,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  157,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  157,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  157,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  157,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  157,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  157,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  158,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Germany
    {  158,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  158,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  158,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  158,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  158,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  158,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  158,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  158,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  158,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  159,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Ghana
    {  159,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  159,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  159,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  159,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  159,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  159,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  159,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  159,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  159,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  159,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  160,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Gibraltar
    {  160,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  160,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  160,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  160,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  160,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  160,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  161,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Grenada
    {  161,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  161,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  161,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  161,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  161,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  161,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  162,  478,    0,  397,    0,  0,  4,  0, }, // Chamorro English/Latin/Guam
    {  163,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Guernsey
    {  163,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  163,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  163,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  163,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  163,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  163,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  164,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Guyana
    {  164,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  164,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  164,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  164,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  164,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  164,  876,    0,  598,    0,  0,  3,  0, }, // Guyana
    {  164,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  165,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Hong Kong
    {  165,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  165,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  165,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  165,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  165,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  165,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  165,  899,  607,  607, 1325,  3,  3,  4, }, // Hong_Kong
    {  166,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Hungary
    {  166,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  166,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  166,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  166,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  166,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  166,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  166,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  166,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  166,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  167,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/India
    {  167,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  167,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  167,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  167,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  167,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  167,  871,    0,   82,    0,  0,  3,  0, }, // Gulf
    {  167,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  167,  914,    0,  112,    0,  0,  3,  0, }, // India
    {  168,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Indonesia
    {  168,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  168,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  168,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  168,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  168,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  168,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  168,  943,    0, 1308,    0,  0,  4,  0, }, // Indonesia_Central
    {  168,  961,    0,  648,    0,  0,  3,  0, }, // Indonesia_Eastern
    {  168,  979,    0, 1312,    0,  0,  3,  0, }, // Indonesia_Western
    {  169,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Ireland
    {  169,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  169,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  169,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  169,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  169,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  169,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  170,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Isle of Man
    {  170,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  170,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  170,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  170,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  170,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  170,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  171,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Israel
    {  171,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  171,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  171,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  171,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  171,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  171,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  172,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Italy
    {  172,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  172,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  172,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  172,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  172,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  172,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  172,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  172,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  172,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  173,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Jamaica
    {  173,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  173,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  173,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  173,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  173,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  173,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  174,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Jersey
    {  174,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  174,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  174,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  174,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  174,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  174,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  175,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Kenya
    {  175,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  175,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  175,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  175,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  175,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  175,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  175,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  175,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  175,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  175,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  176,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Kiribati
    {  176,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  176,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  176,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  176,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  176,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  176,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  177,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Lesotho
    {  177,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  177,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  177,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  177,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  177,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  177,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  177,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  177,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  177,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  177,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  178,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Liberia
    {  178,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  178,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  178,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  178,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  178,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  178,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  178,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  178,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  178,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  178,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  179,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Macao
    {  179,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  179,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  179,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  179,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  179,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  179,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  179,  899,  607,  607, 1325,  3,  3,  4, }, // Hong_Kong
    {  179, 1198,   29,   29,   32,  3,  3,  3, }, // Macau
    {  180,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Madagascar
    {  180,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  180,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  180,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  180,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  180,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  180,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  180,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  180,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  180,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  180,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  181,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Malawi
    {  181,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  181,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  181,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  181,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  181,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  181,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  181,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  181,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  181,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  181,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  182,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Malaysia
    {  182,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  182,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  182,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  182,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  182,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  182,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  182, 1219,    0,  115,    0,  0,  3,  0, }, // Malaysia
    {  183,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Maldives
    {  183,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  183,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  183,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  183,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  183,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  183,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  184,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Malta
    {  184,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  184,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  184,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  184,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  184,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  184,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  185,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Marshall Islands
    {  185,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  185,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  185,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  185,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  185,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  185,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  186,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Mauritius
    {  186,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  186,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  186,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  186,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  186,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  186,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  186,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  186,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  186,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  186,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  187,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Micronesia
    {  187,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  187,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  187,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  187,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  187,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  187,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  188,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Montserrat
    {  188,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  188,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  188,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  188,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  188,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  188,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  189,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Namibia
    {  189,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  189,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  189,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  189,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  189,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  189,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  189,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  189,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  189,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  189,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  190,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Nauru
    {  190,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  190,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  190,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  190,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  190,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  190,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  191,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Netherlands
    {  191,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  191,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  191,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  191,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  191,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  191,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  191,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  191,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  191,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  192,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/New Zealand
    {  192,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  192,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  192,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  192,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  192,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  192,  295,  121,  129,  296,  3,  4,  4, }, // Australia_Central
    {  192,  313, 1282,  300,  305,  4,  5,  5, }, // Australia_CentralWestern
    {  192,  338, 1286,  310,  314,  3,  4,  4, }, // Australia_Eastern
    {  192,  356,  859,  318,  322,  3,  4,  4, }, // Australia_Western
    {  192,  487, 1329,  401,  406,  4,  5,  5, }, // Chatham
    {  192,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  192, 1188, 1289,  795, 1292,  3,  4,  4, }, // Lord_Howe
    {  192, 1346, 1296,  917,  921,  3,  4,  4, }, // New_Zealand
    {  193,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Nigeria
    {  193,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  193,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  193,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  193,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  193,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  193,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  193,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  193,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  193,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  193,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  194,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Niue
    {  194,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  194,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  194,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  194,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  194,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  194,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  195,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Norfolk Island
    {  195,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  195,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  195,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  195,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  195,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  195,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  196,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Northern Mariana Islands
    {  196,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  196,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  196,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  196,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  196,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  196,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  197,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Norway
    {  197,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  197,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  197,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  197,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  197,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  197,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  197,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  197,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  197,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  198,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Pakistan
    {  198,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  198,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  198,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  198,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  198,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  198,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  199,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Palau
    {  199,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  199,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  199,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  199,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  199,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  199,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  200,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Papua New Guinea
    {  200,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  200,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  200,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  200,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  200,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  200,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  202,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Pitcairn
    {  202,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  202,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  202,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  202,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  202,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  202,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  203,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Poland
    {  203,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  203,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  203,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  203,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  203,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  203,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  203,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  203,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  203,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  204,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Portugal
    {  204,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  204,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  204,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  204,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  204,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  204,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  204,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  204,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  204,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  206,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Romania
    {  206,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  206,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  206,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  206,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  206,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  206,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  206,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  206,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  206,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  207,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Rwanda
    {  207,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  207,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  207,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  207,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  207,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  207,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  207,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  207,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  207,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  207,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  208,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Saint Helena
    {  208,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  208,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  208,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  208,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  208,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  208,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  209,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Saint Kitts and Nevis
    {  209,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  209,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  209,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  209,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  209,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  209,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  210,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Saint Lucia
    {  210,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  210,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  210,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  210,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  210,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  210,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  211,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Saint Vincent and Grenadines
    {  211,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  211,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  211,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  211,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  211,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  211,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  212,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Samoa
    {  212,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  212,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  212,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  212,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  212,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  212,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  213,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Seychelles
    {  213,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  213,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  213,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  213,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  213,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  213,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  214,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Sierra Leone
    {  214,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  214,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  214,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  214,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  214,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  214,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  214,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  214,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  214,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  214,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  215,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Singapore
    {  215,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  215,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  215,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  215,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  215,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  215,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  215, 1219,    0,  115,    0,  0,  3,  0, }, // Malaysia
    {  215, 1624,    0,  118,    0,  0,  3,  0, }, // Singapore
    {  216,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Sint Maarten
    {  216,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  216,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  216,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  216,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  216,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  216,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  217,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Slovakia
    {  217,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  217,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  217,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  217,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  217,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  217,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  217,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  217,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  217,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  218,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Slovenia
    {  218,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  218,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  218,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  218,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  218,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  218,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  218,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  218,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  218,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  219,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Solomon Islands
    {  219,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  219,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  219,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  219,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  219,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  219,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  220,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/South Africa
    {  220,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  220,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  220,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  220,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  220,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  220,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  220,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  220,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  220,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  220,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  221,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/South Georgia and South Sandwich Islands
    {  221,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  221,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  221,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  221,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  221,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  221,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  222,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/South Sudan
    {  222,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  222,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  222,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  222,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  222,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  222,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  222,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  222,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  222,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  222,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  223,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Spain
    {  223,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  223,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  223,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  223,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  223,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  223,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  223,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  223,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  223,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  224,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Sudan
    {  224,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  224,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  224,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  224,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  224,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  224,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  224,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  224,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  224,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  224,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  225,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Sweden
    {  225,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  225,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  225,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  225,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  225,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  225,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  225,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  225,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  225,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  226,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Switzerland
    {  226,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  226,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  226,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  226,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  226,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  226,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  226,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  226,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  226,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  227,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Tanzania
    {  227,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  227,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  227,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  227,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  227,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  227,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  227,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  227,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  227,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  227,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  228,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Tokelau
    {  228,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  228,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  228,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  228,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  228,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  228,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  229,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Tonga
    {  229,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  229,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  229,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  229,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  229,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  229,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  230,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Trinidad and Tobago
    {  230,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  230,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  230,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  230,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  230,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  230,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  231,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Turks and Caicos Islands
    {  231,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  231,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  231,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  231,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  231,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  231,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  232,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Tuvalu
    {  232,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  232,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  232,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  232,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  232,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  232,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  233,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Uganda
    {  233,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  233,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  233,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  233,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  233,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  233,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  233,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  233,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  233,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  233,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  234,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/United Arab Emirates
    {  234,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  234,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  234,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  234,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  234,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  234,  871,    0,   82,    0,  0,  3,  0, }, // Gulf
    {  234,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  235,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/United Kingdom
    {  235,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  235,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  235,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  235,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  235,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  235,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  235,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  235,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  235,  871,    0,   82,    0,  0,  3,  0, }, // Gulf
    {  235,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  238,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Vanuatu
    {  238,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  238,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  238,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  238,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  238,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  238,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  239,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/world
    {  239,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  239,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  239,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  239,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  239,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  239,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  240,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Zambia
    {  240,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  240,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  240,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  240,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  240,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  240,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  240,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  240,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  240,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  240,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  241,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Zimbabwe
    {  241,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  241,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  241,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  241,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  241,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  241,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  241,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  241,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  241,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  241,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  246,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central Ewe/Latin/Ghana
    {  246,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  246,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  246,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  253,  107, 1333, 1336, 1340,  3,  4,  4, }, // Alaska French/Latin/France
    {  253,  142, 1344, 1346, 1349,  2,  3,  3, }, // America_Central
    {  253,  158, 1340, 1352, 1355,  2,  3,  3, }, // America_Eastern
    {  253,  174, 1358, 1360, 1363,  2,  3,  3, }, // America_Mountain
    {  253,  191, 1366, 1368, 1371,  2,  3,  3, }, // America_Pacific
    {  253,  286,   54, 1336, 1340,  2,  3,  3, }, // Atlantic
    {  253,  537, 1374, 1377, 1381,  3,  4,  4, }, // Cuba
    {  253,  642,   98,  101, 1385,  3,  4,  4, }, // Europe_Eastern
    {  253,  680,  105,  108, 1389,  3,  4,  4, }, // Europe_Western
    {  253,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  253,  830, 1393, 1396, 1400,  3,  4,  4, }, // Greenland_Eastern
    {  253,  848, 1404, 1407, 1411,  3,  4,  4, }, // Greenland_Western
    {  253,  883, 1415, 1418, 1422,  3,  4,  4, }, // Hawaii_Aleutian
    {  253, 1281, 1426, 1430, 1435,  4,  5,  5, }, // Mexico_Pacific
    {  253, 1358, 1440, 1443, 1447,  3,  4,  4, }, // Newfoundland
    {  253, 1502, 1426, 1430, 1435,  3,  4,  4, }, // Pierre_Miquelon
    {  260,  107, 1333, 1336, 1340,  3,  4,  4, }, // Alaska French/Latin/Canada
    {  260,  142, 1344, 1346, 1451,  2,  3,  3, }, // America_Central
    {  260,  158, 1340, 1352, 1454,  2,  3,  3, }, // America_Eastern
    {  260,  174, 1358, 1360, 1457,  2,  3,  3, }, // America_Mountain
    {  260,  191, 1366, 1368, 1460,  2,  3,  3, }, // America_Pacific
    {  260,  286,   54, 1336, 1340,  2,  3,  3, }, // Atlantic
    {  260,  537, 1374, 1377, 1381,  3,  4,  4, }, // Cuba
    {  260,  642,   98,  101, 1385,  3,  4,  4, }, // Europe_Eastern
    {  260,  680,  105,  108, 1389,  3,  4,  4, }, // Europe_Western
    {  260,  830, 1393, 1396, 1400,  3,  4,  4, }, // Greenland_Eastern
    {  260,  848, 1404, 1407, 1411,  3,  4,  4, }, // Greenland_Western
    {  260,  883, 1415, 1418, 1422,  3,  4,  4, }, // Hawaii_Aleutian
    {  260, 1281, 1426, 1430, 1435,  4,  5,  5, }, // Mexico_Pacific
    {  260, 1358,  849, 1443,   54,  2,  3,  3, }, // Newfoundland
    {  260, 1502, 1426, 1430, 1435,  3,  4,  4, }, // Pierre_Miquelon
    {  268,  709,    0,  534,    0,  0,  3,  0, }, // French_Guiana French/Latin/French Guiana
    {  299,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Friulian/Latin/Italy
    {  299,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  299,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  299,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  324,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Gaelic/Latin/United Kingdom
    {  324,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  324,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  324,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  324,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  324,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  324,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  324,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  324,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  324,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  324,  883,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {  326,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Galician/Latin/Spain
    {  326,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  326,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  326,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  331,  627, 1251, 1251, 1254,  3,  3,  4, }, // Europe_Central German/Latin/Germany
    {  331,  642, 1258, 1258, 1261,  3,  3,  4, }, // Europe_Eastern
    {  331,  680, 1265, 1265, 1268,  3,  3,  4, }, // Europe_Western
    {  338,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Greek/Greek/Greece
    {  338,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  338,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  341,  914,    0,  112,    0,  0,  3,  0, }, // India Gujarati/Gujarati/India
    {  343,  107, 1333, 1336, 1340,  3,  4,  4, }, // Alaska Haitian/Latin/Haiti
    {  343,  142, 1344, 1346, 1349,  2,  3,  3, }, // America_Central
    {  343,  158, 1340, 1352, 1355,  2,  3,  3, }, // America_Eastern
    {  343,  174, 1358, 1360, 1363,  2,  3,  3, }, // America_Mountain
    {  343,  191, 1366, 1368, 1371,  2,  3,  3, }, // America_Pacific
    {  343,  286,   54, 1336, 1340,  2,  3,  3, }, // Atlantic
    {  343,  537, 1374, 1377, 1381,  3,  4,  4, }, // Cuba
    {  343,  642,   98,  101, 1385,  3,  4,  4, }, // Europe_Eastern
    {  343,  680,  105,  108, 1389,  3,  4,  4, }, // Europe_Western
    {  343,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  343,  830, 1393, 1396, 1400,  3,  4,  4, }, // Greenland_Eastern
    {  343,  848, 1404, 1407, 1411,  3,  4,  4, }, // Greenland_Western
    {  343,  883, 1415, 1418, 1422,  3,  4,  4, }, // Hawaii_Aleutian
    {  343, 1281, 1426, 1430, 1435,  4,  5,  5, }, // Mexico_Pacific
    {  343, 1358, 1440, 1443, 1447,  3,  4,  4, }, // Newfoundland
    {  343, 1502, 1426, 1430, 1435,  3,  4,  4, }, // Pierre_Miquelon
    {  349,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Hawaiian/Latin/United States
    {  349,  883,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {  351,  914,    0,  112,    0,  0,  3,  0, }, // India Hindi/Devanagari/India
    {  352,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska Hindi/Latin/India
    {  352,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  352,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  352,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  352,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  352,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  352,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  352,  871,    0,   82,    0,  0,  3,  0, }, // Gulf
    {  352,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  352,  914,    0,  112,    0,  0,  3,  0, }, // India
    {  353,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Hungarian/Latin/Hungary
    {  353,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  353,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  353,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  358,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Indonesian/Latin/Indonesia
    {  358,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  358,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  358,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  358,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  358,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  358,  537, 1463, 1472, 1482,  9, 10, 10, }, // Cuba
    {  358,  830,  568,  571, 1492,  3,  4,  4, }, // Greenland_Eastern
    {  358,  883,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {  358,  943,    0, 1308,    0,  0,  4,  0, }, // Indonesia_Central
    {  358,  961,    0,  648,    0,  0,  3,  0, }, // Indonesia_Eastern
    {  358,  979,    0, 1312,    0,  0,  3,  0, }, // Indonesia_Western
    {  358, 1358,  384,  560,  900,  2,  3,  3, }, // Newfoundland
    {  358, 1502, 1319, 1035, 1039,  3,  4,  4, }, // Pierre_Miquelon
    {  363,  191, 1496, 1499, 1503,  3,  4,  4, }, // America_Pacific Irish/Latin/Ireland
    {  363,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  363,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  363,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  363,  788,    0,  832,    0,  0,  3,  0, }, // GMT
    {  365,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Italian/Latin/Italy
    {  365,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  365,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  369, 1023, 1279,  685, 1507,  3,  3,  3, }, // Japan Japanese/Japanese/Japan
    {  379,  914,    0,  112,    0,  0,  3,  0, }, // India Kannada/Kannada/India
    {  382,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern Kazakh/Cyrillic/Kazakhstan
    {  383,  128, 1510, 1510, 1516,  6,  6, 13, }, // Almaty Kazakh/Arabic/China
    {  383,  219, 1529, 1529, 1534,  5,  5, 12, }, // Aqtau
    {  383,  225, 1546, 1546, 1552,  6,  6, 13, }, // Aqtobe
    {  383, 1047,    0, 1565,    0,  0,  9,  0, }, // Kazakhstan
    {  383, 1058,    0, 1574,    0,  0, 15,  0, }, // Kazakhstan_Eastern
    {  383, 1077,    0, 1589,    0,  0, 15,  0, }, // Kazakhstan_Western
    {  383, 1151,    0, 1604,    0,  0, 10,  0, }, // Kyrgystan
    {  383, 1544, 1614, 1614, 1623,  9,  9, 16, }, // Qyzylorda
    {  389,  914,    0,  112,    0,  0,  3,  0, }, // India Konkani/Devanagari/India
    {  398,  107, 1087, 1639, 1643,  3,  4,  4, }, // Alaska Kurdish/Latin/Turkey
    {  398,  142, 1647, 1649, 1652,  2,  3,  3, }, // America_Central
    {  398,  158, 1124, 1655, 1658,  2,  3,  3, }, // America_Eastern
    {  398,  174, 1661, 1663, 1666,  2,  3,  3, }, // America_Mountain
    {  398,  191, 1669, 1671, 1674,  2,  3,  3, }, // America_Pacific
    {  398,  286,   71, 1639, 1643,  2,  3,  3, }, // Atlantic
    {  398,  537,  880, 1677, 1680,  2,  3,  3, }, // Cuba
    {  398,  830, 1683, 1686, 1690,  3,  4,  4, }, // Greenland_Eastern
    {  398,  848, 1694, 1698, 1703,  4,  5,  5, }, // Greenland_Western
    {  398,  883, 1708, 1712, 1716,  4,  4,  4, }, // Hawaii_Aleutian
    {  398, 1358, 1720, 1723, 1727,  3,  4,  4, }, // Newfoundland
    {  398, 1892,    0, 1127,    0,  0,  2,  0, }, // Yukon
    {  405,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Latvian/Latin/Latvia
    {  405,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  405,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  412,  627, 1251, 1251, 1254,  3,  3,  4, }, // Europe_Central Lower Sorbian/Latin/Germany
    {  412,  642, 1258, 1258, 1261,  3,  3,  4, }, // Europe_Eastern
    {  412,  680, 1265, 1265, 1268,  3,  3,  4, }, // Europe_Western
    {  421,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Macedonian/Cyrillic/Macedonia
    {  421,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  421,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  427,  914,    0,  112,    0,  0,  3,  0, }, // India Malayalam/Malayalam/India
    {  428,  788,    0,   51,    0,  0,  3,  0, }, // GMT Malay/Latin/Malaysia
    {  428, 1219,    0,  115,    0,  0,  3,  0, }, // Malaysia
    {  428, 1624,    0,  118,    0,  0,  3,  0, }, // Singapore
    {  434,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Maltese/Latin/Malta
    {  434,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  434,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  434,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  437,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Manx/Latin/Isle of Man
    {  437,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  437,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  437,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  440,  914,    0,  112,    0,  0,  3,  0, }, // India Marathi/Devanagari/India
    {  456,  914,    0,  112,    0,  0,  3,  0, }, // India Nepali/Devanagari/India
    {  463,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Northern Sami/Latin/Norway
    {  463,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  463,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  463,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  464,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Northern Sami/Latin/Finland
    {  464,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  464,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  464,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  468,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Norwegian Bokmal/Latin/Norway
    {  468,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  468,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  468,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  468,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  468,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  468,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  468,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  468,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  468,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  468,  883,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {  470,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Norwegian Nynorsk/Latin/Norway
    {  470,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  470,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  470,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  470,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  470,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  470,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  470,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  470,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  470,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  470,  883,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {  476,  788,    0,   51,    0,  0,  3,  0, }, // GMT Odia/Odia/India
    {  476,  914,    0,  112,    0,  0,  3,  0, }, // India
    {  477,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Oromo/Latin/Ethiopia
    {  477,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  477,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  477,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  477,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  477,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  477,  883,   54,   57,   61,  3,  3,  4, }, // Hawaii_Aleutian
    {  488,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Polish/Latin/Poland
    {  488,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  488,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  489,    0,  121,  121,  129,  3,  3,  4, }, // Acre Portuguese/Latin/Brazil
    {  489,  135,  153,  153,  156,  3,  3,  4, }, // Amazon
    {  489,  437,  376,  376,  379,  3,  3,  4, }, // Brasilia
    {  490,    0, 1279, 1279, 1279,  3,  3,  3, }, // Acre Portuguese/Latin/Angola
    {  490,  135, 1279, 1279, 1279,  3,  3,  3, }, // Amazon
    {  490,  385, 1279, 1279, 1279,  3,  3,  3, }, // Azores
    {  490,  437, 1279, 1279, 1279,  3,  3,  3, }, // Brasilia
    {  490,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central
    {  490,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  490,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  491,    0, 1279, 1279, 1279,  3,  3,  3, }, // Acre Portuguese/Latin/Cape Verde
    {  491,  135, 1279, 1279, 1279,  3,  3,  3, }, // Amazon
    {  491,  385,  333, 1279, 1279,  4,  3,  3, }, // Azores
    {  491,  437, 1279, 1279, 1279,  3,  3,  3, }, // Brasilia
    {  491,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central
    {  491,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  491,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  492,    0, 1279, 1279, 1279,  3,  3,  3, }, // Acre Portuguese/Latin/Equatorial Guinea
    {  492,  135, 1279, 1279, 1279,  3,  3,  3, }, // Amazon
    {  492,  385,  333,  333,  337,  4,  4,  5, }, // Azores
    {  492,  437, 1279, 1279, 1279,  3,  3,  3, }, // Brasilia
    {  492,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  492,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  492,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  493,    0, 1279, 1279, 1279,  3,  3,  3, }, // Acre Portuguese/Latin/Guinea-Bissau
    {  493,  135, 1279, 1279, 1279,  3,  3,  3, }, // Amazon
    {  493,  385, 1279, 1279, 1279,  3,  3,  3, }, // Azores
    {  493,  437, 1279, 1279, 1279,  3,  3,  3, }, // Brasilia
    {  493,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central
    {  493,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  493,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  494,    0, 1279, 1279, 1279,  3,  3,  3, }, // Acre Portuguese/Latin/Luxembourg
    {  494,  135, 1279, 1279, 1279,  3,  3,  3, }, // Amazon
    {  494,  385,  333,  333,  337,  4,  4,  5, }, // Azores
    {  494,  437, 1279, 1279, 1279,  3,  3,  3, }, // Brasilia
    {  494,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  494,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  494,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  495,    0, 1279, 1279, 1279,  3,  3,  3, }, // Acre Portuguese/Latin/Macao
    {  495,  135, 1279, 1279, 1279,  3,  3,  3, }, // Amazon
    {  495,  385, 1279, 1279, 1279,  3,  3,  3, }, // Azores
    {  495,  437, 1279, 1279, 1279,  3,  3,  3, }, // Brasilia
    {  495,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central
    {  495,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  495,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  496,    0, 1279, 1279, 1279,  3,  3,  3, }, // Acre Portuguese/Latin/Mozambique
    {  496,  135, 1279, 1279, 1279,  3,  3,  3, }, // Amazon
    {  496,  385, 1279, 1279, 1279,  3,  3,  3, }, // Azores
    {  496,  437, 1279, 1279, 1279,  3,  3,  3, }, // Brasilia
    {  496,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central
    {  496,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  496,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  497,    0, 1279, 1279, 1279,  3,  3,  3, }, // Acre Portuguese/Latin/Portugal
    {  497,  135, 1279, 1279, 1279,  3,  3,  3, }, // Amazon
    {  497,  385,  333,  333,  337,  4,  4,  5, }, // Azores
    {  497,  437, 1279, 1279, 1279,  3,  3,  3, }, // Brasilia
    {  497,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  497,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  497,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  498,    0, 1279, 1279, 1279,  3,  3,  3, }, // Acre Portuguese/Latin/Sao Tome and Principe
    {  498,  135, 1279, 1279, 1279,  3,  3,  3, }, // Amazon
    {  498,  385, 1279, 1279, 1279,  3,  3,  3, }, // Azores
    {  498,  437, 1279, 1279, 1279,  3,  3,  3, }, // Brasilia
    {  498,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central
    {  498,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  498,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  499,    0, 1279, 1279, 1279,  3,  3,  3, }, // Acre Portuguese/Latin/Switzerland
    {  499,  135, 1279, 1279, 1279,  3,  3,  3, }, // Amazon
    {  499,  385,  333,  333,  337,  4,  4,  5, }, // Azores
    {  499,  437, 1279, 1279, 1279,  3,  3,  3, }, // Brasilia
    {  499,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  499,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  499,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  500,    0, 1279, 1279, 1279,  3,  3,  3, }, // Acre Portuguese/Latin/Timor-Leste
    {  500,  135, 1279, 1279, 1279,  3,  3,  3, }, // Amazon
    {  500,  385, 1279, 1279, 1279,  3,  3,  3, }, // Azores
    {  500,  437, 1279, 1279, 1279,  3,  3,  3, }, // Brasilia
    {  500,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central
    {  500,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  500,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  502,  914,    0,  112,    0,  0,  3,  0, }, // India Punjabi/Gurmukhi/India
    {  504, 1469,  695,  695, 1731,  3,  3,  4, }, // Peru Quechua/Latin/Peru
    {  505,  422,    0,  373,    0,  0,  3,  0, }, // Bolivia Quechua/Latin/Bolivia
    {  505, 1469, 1279, 1279, 1279,  3,  3,  3, }, // Peru
    {  506,  619,    0,  499,    0,  0,  3,  0, }, // Ecuador Quechua/Latin/Ecuador
    {  506, 1469, 1279, 1279, 1279,  3,  3,  3, }, // Peru
    {  507,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Romanian/Latin/Romania
    {  507,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  507,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  507,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  527,  107, 1735, 1738, 1742,  3,  4,  4, }, // Alaska Sardinian/Latin/Italy
    {  527,  142,  335, 1746, 1214,  2,  3,  3, }, // America_Central
    {  527,  158, 1749, 1751, 1754,  2,  3,  3, }, // America_Eastern
    {  527,  174, 1757, 1760, 1764,  3,  4,  4, }, // America_Mountain
    {  527,  191, 1768, 1770, 1773,  2,  3,  3, }, // America_Pacific
    {  527,  286, 1735, 1738, 1742,  2,  3,  3, }, // Atlantic
    {  527,  788,    0, 1776,    0,  0,  3,  0, }, // GMT
    {  529,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Serbian/Cyrillic/Serbia
    {  529,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  529,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  529,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  530,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Serbian/Cyrillic/Bosnia and Herzegovina
    {  530,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  530,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  530,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  533,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Serbian/Latin/Bosnia and Herzegovina
    {  533,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  533,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  533,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  534,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Serbian/Latin/Kosovo
    {  534,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  534,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  534,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  535,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Serbian/Latin/Montenegro
    {  535,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  535,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  535,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  536,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Serbian/Latin/Serbia
    {  536,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  536,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  536,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  540,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Sicilian/Latin/Italy
    {  540,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  540,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  540,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  540,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  540,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  540,  537, 1779, 1782, 1786,  3,  4,  4, }, // Cuba
    {  540,  883,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {  542,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Silesian/Latin/Poland
    {  542,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  542,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  547,  627, 1272, 1272, 1275,  3,  3,  4, }, // Europe_Central Slovak/Latin/Slovakia
    {  550,  883,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian Somali/Latin/Somalia
    {  561,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Spanish/Latin/Spain
    {  561,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  561,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  561,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  562,  240,  257,  257,  260,  3,  3,  4, }, // Argentina Spanish/Latin/Argentina
    {  562,  250,  264,  264,  268,  4,  4,  5, }, // Argentina_Western
    {  562,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central
    {  562,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  562,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  562,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  563,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/Belize
    {  563,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  563,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  563,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  564,  422,    0,  373,    0,  0,  3,  0, }, // Bolivia Spanish/Latin/Bolivia
    {  564,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central
    {  564,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  564,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  564,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  565,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/Brazil
    {  565,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  565,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  565,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  568,  495,  411,  411,  414,  3,  3,  4, }, // Chile Spanish/Latin/Chile
    {  568,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central
    {  568,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  568,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  568,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  569,  523,  446,  446,  449,  3,  3,  4, }, // Colombia Spanish/Latin/Colombia
    {  569,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central
    {  569,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  569,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  569,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  570,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/Costa Rica
    {  570,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  570,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  570,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  571,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/Cuba
    {  571,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  571,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  571,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  572,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/Dominican Republic
    {  572,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  572,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  572,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  573,  619,    0,  499,    0,  0,  3,  0, }, // Ecuador Spanish/Latin/Ecuador
    {  573,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central
    {  573,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  573,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  573,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  574,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/El Salvador
    {  574,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  574,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  574,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  576,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/Guatemala
    {  576,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  576,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  576,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  577,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/Honduras
    {  577,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  577,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  577,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  578,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/Latin America
    {  578,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  578,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  578,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  579,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/Mexico
    {  579,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  579,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  579,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  580,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/Nicaragua
    {  580,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  580,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  580,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  581,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/Panama
    {  581,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  581,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  581,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  582,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/Paraguay
    {  582,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  582,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  582,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  583,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/Peru
    {  583,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  583,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  583,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  583, 1469,  695,  695, 1731,  3,  3,  4, }, // Peru
    {  585,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/Puerto Rico
    {  585,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  585,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  585,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  586,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Spanish/Latin/United States
    {  586,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  586,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  586,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  586,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  586,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  586,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central
    {  586,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  586,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  586,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  586,  883,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {  587,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/Uruguay
    {  587,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  587,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  587,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  587, 1777, 1172, 1172, 1175,  3,  3,  4, }, // Uruguay
    {  588,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/Venezuela
    {  588,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  588,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  588,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  588, 1811,    0, 1322,    0,  0,  3,  0, }, // Venezuela
    {  597,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Swedish/Latin/Sweden
    {  597,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  597,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  597,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  600,  627, 1251, 1251, 1254,  3,  3,  4, }, // Europe_Central Swiss German/Latin/Switzerland
    {  600,  642, 1258, 1258, 1261,  3,  3,  4, }, // Europe_Eastern
    {  600,  680, 1265, 1265, 1268,  3,  3,  4, }, // Europe_Western
    {  610,  914,    0,  112,    0,  0,  3,  0, }, // India Tamil/Tamil/India
    {  611,  914,    0, 1279,    0,  0,  3,  0, }, // India Tamil/Tamil/Malaysia
    {  611, 1219,    0,  115,    0,  0,  3,  0, }, // Malaysia
    {  611, 1624,    0,  118,    0,  0,  3,  0, }, // Singapore
    {  612,  914,    0, 1279,    0,  0,  3,  0, }, // India Tamil/Tamil/Singapore
    {  612, 1219,    0,  115,    0,  0,  3,  0, }, // Malaysia
    {  612, 1624,    0,  118,    0,  0,  3,  0, }, // Singapore
    {  614,  142, 1790, 1793, 1797,  3,  4,  5, }, // America_Central Taroko/Latin/Taiwan
    {  614,  158, 1802, 1805, 1809,  3,  4,  5, }, // America_Eastern
    {  614,  174, 1814, 1817, 1821,  3,  4,  4, }, // America_Mountain
    {  614,  191, 1507, 1825, 1829,  2,  4,  5, }, // America_Pacific
    {  614,  286, 1834, 1837, 1841,  3,  4,  5, }, // Atlantic
    {  614,  627, 1790, 1790, 1846,  2,  2,  4, }, // Europe_Central
    {  614,  642,  522,  522, 1850,  2,  2,  4, }, // Europe_Eastern
    {  614,  680,  685,  685, 1854,  2,  2,  4, }, // Europe_Western
    {  614,  788,    0, 1858,    0,  0,  3,  0, }, // GMT
    {  617,  914,    0,  112,    0,  0,  3,  0, }, // India Telugu/Telugu/India
    {  633, 1750,   26,   26,   28,  3,  3,  4, }, // Turkmenistan Turkmen/Latin/Turkmenistan
    {  636,  627, 1251, 1251, 1254,  3,  3,  4, }, // Europe_Central Upper Sorbian/Latin/Germany
    {  636,  642, 1258, 1258, 1261,  3,  3,  4, }, // Europe_Eastern
    {  636,  680, 1265, 1265, 1268,  3,  3,  4, }, // Europe_Western
    {  646,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Vietnamese/Latin/Vietnam
    {  646,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  646,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  646,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  646,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  646,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  646,  883,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {  650,  627, 1251, 1251, 1254,  3,  3,  4, }, // Europe_Central Walser/Latin/Switzerland
    {  650,  642, 1258, 1258, 1261,  3,  3,  4, }, // Europe_Eastern
    {  650,  680, 1265, 1265, 1268,  3,  3,  4, }, // Europe_Western
    {  652,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Welsh/Latin/United Kingdom
    {  652,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  652,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  652,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  658,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Western Frisian/Latin/Netherlands
    {  658,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  658,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  669,    0,  121,  121,  129,  3,  3,  4, }, // Acre Kaingang/Latin/Brazil
    {  669,  135,  153,  153,  156,  3,  3,  4, }, // Amazon
    {  669,  437,  376,  376,  379,  3,  3,  4, }, // Brasilia
    {  670,    0,  121,  121,  129,  3,  3,  4, }, // Acre Nheengatu/Latin/Brazil
    {  670,  135,  153,  153,  156,  3,  3,  4, }, // Amazon
    {  670,  437,  376,  376,  379,  3,  3,  4, }, // Brasilia
    {  686,  788,    0, 1861,    0,  0,  2,  0, }, // GMT Anii/Latin/Benin
    {  696,    0,    0,    0,    0,  0,  0,  0, } // Terminal Row
}; // Metazone short name table

// Indexing matches that of locale_data in qlocale_data_p.h
static inline constexpr LocaleZoneData localeZoneData[] = {
    // LOCALE_TAGS(lng,scp,ter) xct1st, zn1st, ml1st, ms1st, (+hr, -hr, gmt, flbk, rgen, rstd, rdst){ind,sz}
    { LOCALE_TAGS(  1,  0,  0)     0,     0,     0,     0,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // C/AnyScript/AnyTerritory
    { LOCALE_TAGS(  2, 27, 90)    50,   158,     8,     4,    0,    6,    0,    0,   39,   47,   66,  6,  6,  5,  7,  8, 19, 17, }, // Abkhazian/Cyrillic/Georgia
    { LOCALE_TAGS(  3, 66, 77)   466,   314,     8,     7,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Afar/Latin/Ethiopia
    { LOCALE_TAGS(  3, 66, 67)   508,   314,     8,     8,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Afar/Latin/Djibouti
    { LOCALE_TAGS(  3, 66, 74)   508,   314,     8,     8,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Afar/Latin/Eritrea
    { LOCALE_TAGS(  4, 66,216)   508,   314,     8,     8,    0,    6,    0,    0,   97,  103,  118,  6,  6,  5,  7,  6, 15, 12, }, // Afrikaans/Latin/South Africa
    { LOCALE_TAGS(  4, 66,162)   622,   461,    12,    11,    0,    6,    0,    0,   97,  103,  118,  6,  6,  5,  7,  6, 15, 12, }, // Afrikaans/Latin/Namibia
    { LOCALE_TAGS(  5, 66, 40)   622,   461,    12,    11,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Aghem/Latin/Cameroon
    { LOCALE_TAGS(  6, 66, 92)   664,   461,    12,    12,    0,    6,    0,    0,  130,  137,  152,  6,  6,  5,  7,  7, 15, 11, }, // Akan/Latin/Ghana
    { LOCALE_TAGS(  8, 66, 40)   898,   605,    12,    15,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Akoose/Latin/Cameroon
    { LOCALE_TAGS(  9, 66,  3)   940,   605,    12,    16,    0,    6,    0,    0,  163,  170,  187,  6,  6,  5,  7,  7, 17, 14, }, // Albanian/Latin/Albania
    { LOCALE_TAGS(  9, 66,126)  1230,   762,    12,    19,    0,    6,    0,    0,  163,  170,  187,  6,  6,  5,  7,  7, 17, 14, }, // Albanian/Latin/Kosovo
    { LOCALE_TAGS(  9, 66,140)  1230,   762,    12,    19,    0,    6,    0,    0,  163,  170,  187,  6,  6,  5,  7,  7, 17, 14, }, // Albanian/Latin/Macedonia
    { LOCALE_TAGS( 11, 33, 77)  1230,   762,    12,    19,   12,   17,    5,    0,  201,  207,  218,  5,  5,  8,  7,  6, 11, 15, }, // Amharic/Ethiopic/Ethiopia
    { LOCALE_TAGS( 14,  4, 71)  1647,   909,    12,    22,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Egypt
    { LOCALE_TAGS( 14,  4,  4)  2064,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Algeria
    { LOCALE_TAGS( 14,  4, 19)  2064,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Bahrain
    { LOCALE_TAGS( 14,  4, 48)  2064,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Chad
    { LOCALE_TAGS( 14,  4, 55)  2064,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Comoros
    { LOCALE_TAGS( 14,  4, 67)  2064,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Djibouti
    { LOCALE_TAGS( 14,  4, 74)  2064,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Eritrea
    { LOCALE_TAGS( 14,  4,113)  2064,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Iraq
    { LOCALE_TAGS( 14,  4,116)  2064,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Israel
    { LOCALE_TAGS( 14,  4,122)  2064,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Jordan
    { LOCALE_TAGS( 14,  4,127)  2064,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Kuwait
    { LOCALE_TAGS( 14,  4,132)  2064,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Lebanon
    { LOCALE_TAGS( 14,  4,135)  2064,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Libya
    { LOCALE_TAGS( 14,  4,149)  2064,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Mauritania
    { LOCALE_TAGS( 14,  4,159)  2064,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Morocco
    { LOCALE_TAGS( 14,  4,176)  2064,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Oman
    { LOCALE_TAGS( 14,  4,180)  2064,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Palestinian Territories
    { LOCALE_TAGS( 14,  4,190)  2064,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Qatar
    { LOCALE_TAGS( 14,  4,205)  2064,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Saudi Arabia
    { LOCALE_TAGS( 14,  4,215)  2064,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Somalia
    { LOCALE_TAGS( 14,  4,219)  2064,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/South Sudan
    { LOCALE_TAGS( 14,  4,222)  2064,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Sudan
    { LOCALE_TAGS( 14,  4,227)  2064,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Syria
    { LOCALE_TAGS( 14,  4,238)  2064,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Tunisia
    { LOCALE_TAGS( 14,  4,245)  2064,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/United Arab Emirates
    { LOCALE_TAGS( 14,  4,257)  2064,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Western Sahara
    { LOCALE_TAGS( 14,  4,258)  2064,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/world
    { LOCALE_TAGS( 14,  4,259)  2064,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Yemen
    { LOCALE_TAGS( 15, 66,220)  2064,  1058,    13,    25,    0,    6,    0,    0,  271,  281,  300,  6,  6,  5,  7, 10, 19, 20, }, // Aragonese/Latin/Spain
    { LOCALE_TAGS( 17,  5, 12)  2106,  1067,    13,    26,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Armenian/Armenian/Armenia
    { LOCALE_TAGS( 18,  9,110)  2523,  1211,    13,    29,    0,    6,    0,    0,  320,  327,  343,  6,  6,  5,  7,  7, 16, 19, }, // Assamese/Bangla/India
    { LOCALE_TAGS( 19, 66,220)  2939,  1355,    14,    32,    0,    6,    0,    0,  271,  362,  381,  6,  6,  5,  7, 10, 19, 19, }, // Asturian/Latin/Spain
    { LOCALE_TAGS( 20, 66,230)  3080,  1510,    25,    36,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Asu/Latin/Tanzania
    { LOCALE_TAGS( 21, 66,169)  3122,  1510,    25,    37,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Atsam/Latin/Nigeria
    { LOCALE_TAGS( 25, 66, 17)  3164,  1510,    25,    38,    0,    6,    0,    0,  400,  408,  425,  6,  6,  5,  7,  8, 17, 12, }, // Azerbaijani/Latin/Azerbaijan
    { LOCALE_TAGS( 25,  4,112)  3470,  1655,    25,    41,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Azerbaijani/Arabic/Iran
    { LOCALE_TAGS( 25,  4,113)  3507,  1655,    25,    42,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Azerbaijani/Arabic/Iraq
    { LOCALE_TAGS( 25,  4,239)  3507,  1655,    25,    42,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Azerbaijani/Arabic/Turkey
    { LOCALE_TAGS( 25, 27, 17)  3507,  1655,    25,    42,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Azerbaijani/Cyrillic/Azerbaijan
    { LOCALE_TAGS( 26, 66, 40)  3544,  1655,    25,    43,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Bafia/Latin/Cameroon
    { LOCALE_TAGS( 28, 66,145)  3586,  1655,    25,    44,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Bambara/Latin/Mali
    { LOCALE_TAGS( 28, 90,145)  3628,  1655,    25,    45,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Bambara/Nko/Mali
    { LOCALE_TAGS( 30,  9, 20)  3628,  1655,    25,    45,    0,    6,   21,    0,  320,  437,  449,  6,  6,  6,  7,  7, 12, 15, }, // Bangla/Bangla/Bangladesh
    { LOCALE_TAGS( 30,  9,110)  4045,  1811,    26,    48,    0,    6,   21,    0,  320,  437,  449,  6,  6,  6,  7,  7, 12, 15, }, // Bangla/Bangla/India
    { LOCALE_TAGS( 31, 66, 40)  4045,  1811,    26,    48,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Basaa/Latin/Cameroon
    { LOCALE_TAGS( 32, 27,193)  4087,  1811,    26,    49,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Bashkir/Cyrillic/Russia
    { LOCALE_TAGS( 33, 66,220)  4129,  1811,    26,    50,    0,   22,    0,    0,  464,  479,  504,  6,  6,  5,  7, 15, 25, 16, }, // Basque/Latin/Spain
    { LOCALE_TAGS( 35, 27, 22)  4276,  1968,    37,    54,    0,    6,    0,    0,  520,  527,  545,  6,  6,  5,  7,  7, 18, 13, }, // Belarusian/Cyrillic/Belarus
    { LOCALE_TAGS( 36, 66,260)  4693,  2112,    37,    57,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Bemba/Latin/Zambia
    { LOCALE_TAGS( 37, 66,230)  4735,  2112,    37,    58,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Bena/Latin/Tanzania
    { LOCALE_TAGS( 38, 29,110)  4777,  2112,    37,    59,    0,    6,    0,    0,  558,  565,  576,  6,  6,  5,  7,  7, 11, 13, }, // Bhojpuri/Devanagari/India
    { LOCALE_TAGS( 40, 33, 74)  4819,  2113,    37,    60,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Blin/Ethiopic/Eritrea
    { LOCALE_TAGS( 41, 29,110)  4861,  2113,    37,    61,    0,    6,   27,    0,  589,  594,  607,  6,  6, 11,  7,  5, 13, 17, }, // Bodo/Devanagari/India
    { LOCALE_TAGS( 42, 66, 29)  5278,  2268,    38,    64,    0,   28,   21,    0,    0,  624,  646,  6,  7,  6,  7,  2, 22, 18, }, // Bosnian/Latin/Bosnia and Herzegovina
    { LOCALE_TAGS( 42, 27, 29)  5431,  2424,    42,    67,    0,    6,    0,    0,  664,   83,   90,  6,  6,  5,  7,  9,  7,  7, }, // Bosnian/Cyrillic/Bosnia and Herzegovina
    { LOCALE_TAGS( 43, 66, 84)  5847,  2580,    42,    70,    0,    6,    0,    0,  673,  679,  693,  6,  6,  5,  7,  6, 14, 11, }, // Breton/Latin/France
    { LOCALE_TAGS( 45, 27, 36)  5969,  2728,    42,    73,    0,    6,   38,    0,    0,  704,  725,  6,  6,  9,  7,  2, 21, 23, }, // Bulgarian/Cyrillic/Bulgaria
    { LOCALE_TAGS( 46, 86,161)  6386,  2875,    42,    76,    0,    6,    0,    0,  748,  757,  771,  6,  6,  5,  7,  9, 14, 22, }, // Burmese/Myanmar/Myanmar
    { LOCALE_TAGS( 47,137,107)  6803,  3021,    42,    79,    0,    6,    0,    0,  793,  797,  803,  6,  6,  5,  7,  4,  6,  6, }, // Cantonese/Traditional Han/Hong Kong
    { LOCALE_TAGS( 47,118, 50)  7220,  3178,    42,    82,    0,    6,    0,    0,  809,   83,   90,  6,  6,  5,  7,  4,  7,  7, }, // Cantonese/Simplified Han/China
    { LOCALE_TAGS( 47,137, 50)  7482,  3335,    42,    85,    0,    6,    0,    0,  793,  797,  803,  6,  6,  5,  7,  4,  6,  6, }, // Cantonese/Traditional Han/China
    { LOCALE_TAGS( 47,137,139)  7482,  3335,    42,    85,    0,    6,    0,    0,  793,  797,  803,  6,  6,  5,  7,  4,  6,  6, }, // Cantonese/Traditional Han/Macao
    { LOCALE_TAGS( 48, 66,220)  7482,  3335,    42,    85,    0,    6,    0,    0,  813,  824,  842,  6,  6,  5,  7, 11, 18, 16, }, // Catalan/Latin/Spain
    { LOCALE_TAGS( 48, 66,  6)  7627,  3484,    46,    88,    0,    6,    0,    0,  813,  824,  842,  6,  6,  5,  7, 11, 18, 16, }, // Catalan/Latin/Andorra
    { LOCALE_TAGS( 48, 66, 84)  7627,  3484,    46,    88,    0,    6,    0,    0,  813,  824,  842,  6,  6,  5,  7, 11, 18, 16, }, // Catalan/Latin/France
    { LOCALE_TAGS( 48, 66,117)  7627,  3484,    46,    88,    0,    6,    0,    0,  813,  824,  842,  6,  6,  5,  7, 11, 18, 16, }, // Catalan/Latin/Italy
    { LOCALE_TAGS( 49, 66,185)  7627,  3484,    46,    88,    0,    6,   21,    7,  858,  868,  891,  6,  6,  6,  5, 10, 23, 23, }, // Cebuano/Latin/Philippines
    { LOCALE_TAGS( 50, 66,159)  7670,  3628,    46,    91,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Central Atlas Tamazight/Latin/Morocco
    { LOCALE_TAGS( 51,  4,113)  7712,  3628,    46,    92,    0,    6,   47,    0,  914,  923,  936,  6,  6,  9,  7,  9, 13, 14, }, // Central Kurdish/Arabic/Iraq
    { LOCALE_TAGS( 51,  4,112)  7754,  3628,    46,    93,    0,    6,   47,    0,  914,  923,  936,  6,  6,  9,  7,  9, 13, 14, }, // Central Kurdish/Arabic/Iran
    { LOCALE_TAGS( 52, 21, 20)  7754,  3628,    46,    93,    0,    6,   21,    0,  950,  965,  995,  6,  6,  6,  7, 15, 30, 38, }, // Chakma/Chakma/Bangladesh
    { LOCALE_TAGS( 52, 21,110)  8166,  3781,    47,    96,    0,    6,   21,    0,  950,  965,  995,  6,  6,  6,  7, 15, 30, 38, }, // Chakma/Chakma/India
    { LOCALE_TAGS( 54, 27,193)  8166,  3781,    47,    96,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Chechen/Cyrillic/Russia
    { LOCALE_TAGS( 55, 23,248)  8582,  3923,    47,    99,    0,    6,    0,    0, 1033, 1041, 1055,  6,  6,  5,  7,  8, 14, 14, }, // Cherokee/Cherokee/United States
    { LOCALE_TAGS( 56, 66,248)  8999,  4067,    54,   103,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Chickasaw/Latin/United States
    { LOCALE_TAGS( 57, 66,243)  9041,  4067,    54,   104,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Chiga/Latin/Uganda
    { LOCALE_TAGS( 58,118, 50)  9083,  4067,    54,   105,    0,    6,    0,   12,  809, 1069, 1075,  6,  6,  5,  6,  4,  6,  6, }, // Chinese/Simplified Han/China
    { LOCALE_TAGS( 58, 66, 50)  9500,  4224,    54,   108,    0,    6,    0,   12,  809, 1069, 1075,  6,  6,  5,  6,  4,  6,  6, }, // Chinese/Latin/China
    { LOCALE_TAGS( 58,118,107)  9500,  4224,    54,   108,    0,    6,    0,   12,  809, 1069, 1075,  6,  6,  5,  6,  4,  6,  6, }, // Chinese/Simplified Han/Hong Kong
    { LOCALE_TAGS( 58,118,139)  9500,  4224,    54,   108,    0,    6,    0,   12,  809, 1069, 1075,  6,  6,  5,  6,  4,  6,  6, }, // Chinese/Simplified Han/Macao
    { LOCALE_TAGS( 58,118,143)  9500,  4224,    54,   108,    0,    6,    0,   12,  809, 1069, 1075,  6,  6,  5,  6,  4,  6,  6, }, // Chinese/Simplified Han/Malaysia
    { LOCALE_TAGS( 58,118,210)  9500,  4224,    54,   108,    0,    6,    0,   12,  809, 1069, 1075,  6,  6,  5,  6,  4,  6,  6, }, // Chinese/Simplified Han/Singapore
    { LOCALE_TAGS( 58,137,107)  9501,  4226,    56,   108,    0,    6,    0,   12,  793,  797,  803,  6,  6,  5,  6,  4,  6,  6, }, // Chinese/Traditional Han/Hong Kong
    { LOCALE_TAGS( 58,137,139)  9631,  4259,    60,   108,    0,    6,    0,   12,  793,  797,  803,  6,  6,  5,  6,  4,  6,  6, }, // Chinese/Traditional Han/Macao
    { LOCALE_TAGS( 58,137,143)  9761,  4292,    64,   108,    0,    6,    0,   12,  793,   83,   90,  6,  6,  5,  6,  4,  7,  7, }, // Chinese/Traditional Han/Malaysia
    { LOCALE_TAGS( 58,137,228)  9761,  4292,    64,   108,    0,    6,    0,   12,  793,   83,   90,  6,  6,  5,  6,  4,  7,  7, }, // Chinese/Traditional Han/Taiwan
    { LOCALE_TAGS( 59, 27,193) 10095,  4449,    72,   111,    0,    6,    0,    0, 1081, 1092, 1111,  6,  6,  5,  7, 11, 19, 19, }, // Church/Cyrillic/Russia
    { LOCALE_TAGS( 60, 27,193) 10165,  4473,    72,   112,    0,    6,    0,    0,    0, 1130, 1150,  6,  6,  5,  7,  2, 20, 17, }, // Chuvash/Cyrillic/Russia
    { LOCALE_TAGS( 61, 66, 91) 10581,  4631,   230,   116,    0,    6,    0,    0, 1167, 1178, 1199,  6,  6,  5,  7, 11, 21, 17, }, // Colognian/Latin/Germany
    { LOCALE_TAGS( 62, 24, 71) 10688,  4645,   233,   119,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Coptic/Coptic/Egypt
    { LOCALE_TAGS( 63, 66,246) 10730,  4645,   233,   120,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Cornish/Latin/United Kingdom
    { LOCALE_TAGS( 64, 66, 84) 10772,  4645,   237,   121,    0,    6,   56,    0, 1216, 1224, 1239,  6,  6,  5,  7,  8, 15, 17, }, // Corsican/Latin/France
    { LOCALE_TAGS( 66, 66, 60) 10814,  4646,   237,   122,    0,   28,    0,    0,    0,  624,  646,  6,  7,  5,  7,  2, 22, 18, }, // Croatian/Latin/Croatia
    { LOCALE_TAGS( 66, 66, 29) 10947,  4803,   241,   125,    0,   28,    0,    0,    0,  624,  646,  6,  7,  5,  7,  2, 22, 18, }, // Croatian/Latin/Bosnia and Herzegovina
    { LOCALE_TAGS( 67, 66, 64) 10947,  4803,   241,   125,   35,   40,    0,    0, 1256,   83,   90,  5,  5,  5,  7, 15,  7,  7, }, // Czech/Latin/Czechia
    { LOCALE_TAGS( 68, 66, 65) 11116,  4960,   248,   129,   45,   51,    0,    0, 1271, 1277, 1289,  6,  6,  5,  7,  6, 12, 12, }, // Danish/Latin/Denmark
    { LOCALE_TAGS( 68, 66, 95) 11206,  5116,   251,   132,   45,   51,    0,    0, 1271, 1277, 1289,  6,  6,  5,  7,  6, 12, 12, }, // Danish/Latin/Greenland
    { LOCALE_TAGS( 69,132,144) 11206,  5116,   251,   132,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Divehi/Thaana/Maldives
    { LOCALE_TAGS( 70, 29,110) 11248,  5116,   251,   133,    0,    6,    0,    0, 1301, 1308, 1320,  6,  6,  5,  7,  7, 12, 14, }, // Dogri/Devanagari/India
    { LOCALE_TAGS( 71, 66, 40) 11290,  5125,   251,   136,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Duala/Latin/Cameroon
    { LOCALE_TAGS( 72, 66,165) 11332,  5125,   251,   137,    0,    6,    0,    0, 1334, 1344, 1363,  6,  6,  5,  7, 10, 19, 15, }, // Dutch/Latin/Netherlands
    { LOCALE_TAGS( 72, 66, 13) 11448,  5282,   262,   141,    0,    6,    0,    0, 1334, 1344, 1363,  6,  6,  5,  7, 10, 19, 15, }, // Dutch/Latin/Aruba
    { LOCALE_TAGS( 72, 66, 23) 11448,  5282,   262,   141,    0,    6,    0,    0, 1334, 1344, 1363,  6,  6,  5,  7, 10, 19, 15, }, // Dutch/Latin/Belgium
    { LOCALE_TAGS( 72, 66, 44) 11448,  5282,   262,   141,    0,    6,    0,    0, 1334, 1344, 1363,  6,  6,  5,  7, 10, 19, 15, }, // Dutch/Latin/Caribbean Netherlands
    { LOCALE_TAGS( 72, 66, 62) 11448,  5282,   262,   141,    0,    6,    0,    0, 1334, 1344, 1363,  6,  6,  5,  7, 10, 19, 15, }, // Dutch/Latin/Curacao
    { LOCALE_TAGS( 72, 66,211) 11448,  5282,   262,   141,    0,    6,    0,    0, 1334, 1344, 1363,  6,  6,  5,  7, 10, 19, 15, }, // Dutch/Latin/Sint Maarten
    { LOCALE_TAGS( 72, 66,223) 11448,  5282,   262,   141,    0,    6,    0,    0, 1334, 1344, 1363,  6,  6,  5,  7, 10, 19, 15, }, // Dutch/Latin/Suriname
    { LOCALE_TAGS( 73,134, 27) 11448,  5283,   263,   141,    0,    6,   61,   18, 1378,   83,   90,  6,  6, 12,  9, 10,  7,  7, }, // Dzongkha/Tibetan/Bhutan
    { LOCALE_TAGS( 74, 66,124) 11637,  5366,   264,   144,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Embu/Latin/Kenya
    { LOCALE_TAGS( 75, 66,248) 11679,  5366,   264,   145,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/United States
    { LOCALE_TAGS( 75, 28,248) 11729,  5524,   272,   149,    0,    6,   73,    0, 1388,   83,   90,  6,  6,  9,  7,  9,  7,  7, }, // English/Deseret/United States
    { LOCALE_TAGS( 75, 66,  5) 11758,  5532,   272,   150,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/American Samoa
    { LOCALE_TAGS( 75, 66,  8) 11758,  5532,   272,   150,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Anguilla
    { LOCALE_TAGS( 75, 66, 10) 11767,  5540,   279,   151,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Antigua and Barbuda
    { LOCALE_TAGS( 75, 66, 15) 11776,  5548,   286,   152,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Australia
    { LOCALE_TAGS( 75, 66, 16) 11785,  5567,   299,   153,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Austria
    { LOCALE_TAGS( 75, 66, 18) 11794,  5578,   309,   154,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Bahamas
    { LOCALE_TAGS( 75, 66, 21) 11803,  5586,   316,   155,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Barbados
    { LOCALE_TAGS( 75, 66, 23) 11812,  5594,   323,   156,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Belgium
    { LOCALE_TAGS( 75, 66, 24) 11821,  5605,   333,   157,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Belize
    { LOCALE_TAGS( 75, 66, 26) 11830,  5613,   340,   158,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Bermuda
    { LOCALE_TAGS( 75, 66, 30) 11839,  5621,   347,   159,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Botswana
    { LOCALE_TAGS( 75, 66, 33) 11848,  5633,   358,   160,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/British Indian Ocean Territory
    { LOCALE_TAGS( 75, 66, 34) 11857,  5641,   365,   161,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/British Virgin Islands
    { LOCALE_TAGS( 75, 66, 38) 11866,  5649,   372,   162,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Burundi
    { LOCALE_TAGS( 75, 66, 40) 11866,  5649,   372,   162,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Cameroon
    { LOCALE_TAGS( 75, 66, 41) 11875,  5661,   383,   163,    0,    6,    0,    0,    0,    7, 1397,  6,  6,  5,  7,  7, 16, 23, }, // English/Latin/Canada
    { LOCALE_TAGS( 75, 66, 45) 11883,  5705,   424,   163,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Cayman Islands
    { LOCALE_TAGS( 75, 66, 51) 11892,  5713,   431,   164,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Christmas Island
    { LOCALE_TAGS( 75, 66, 53) 11901,  5721,   438,   165,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Cocos Islands
    { LOCALE_TAGS( 75, 66, 58) 11910,  5729,   445,   166,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Cook Islands
    { LOCALE_TAGS( 75, 66, 63) 11919,  5737,   452,   167,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Cyprus
    { LOCALE_TAGS( 75, 66, 64) 11928,  5745,   459,   168,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Czechia
    { LOCALE_TAGS( 75, 66, 65) 11937,  5756,   469,   169,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Denmark
    { LOCALE_TAGS( 75, 66, 66) 11946,  5767,   479,   170,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Diego Garcia
    { LOCALE_TAGS( 75, 66, 68) 11955,  5775,   486,   171,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Dominica
    { LOCALE_TAGS( 75, 66, 74) 11964,  5783,   493,   172,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Eritrea
    { LOCALE_TAGS( 75, 66, 76) 11973,  5795,   504,   173,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Eswatini
    { LOCALE_TAGS( 75, 66, 78) 11982,  5807,   515,   174,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Europe
    { LOCALE_TAGS( 75, 66, 80) 11991,  5818,   525,   175,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Falkland Islands
    { LOCALE_TAGS( 75, 66, 82) 12000,  5826,   532,   176,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Fiji
    { LOCALE_TAGS( 75, 66, 83) 12009,  5834,   539,   177,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Finland
    { LOCALE_TAGS( 75, 66, 84) 12018,  5845,   549,   178,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/France
    { LOCALE_TAGS( 75, 66, 89) 12027,  5856,   559,   179,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Gambia
    { LOCALE_TAGS( 75, 66, 91) 12036,  5868,   570,   180,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Germany
    { LOCALE_TAGS( 75, 66, 92) 12045,  5879,   580,   181,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Ghana
    { LOCALE_TAGS( 75, 66, 93) 12054,  5891,   591,   182,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Gibraltar
    { LOCALE_TAGS( 75, 66, 96) 12063,  5899,   598,   183,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Grenada
    { LOCALE_TAGS( 75, 66, 98) 12072,  5907,   605,   184,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Guam
    { LOCALE_TAGS( 75, 66,100) 12072,  5908,   606,   184,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Guernsey
    { LOCALE_TAGS( 75, 66,103) 12081,  5916,   613,   185,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Guyana
    { LOCALE_TAGS( 75, 66,107) 12090,  5925,   621,   186,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Hong Kong
    { LOCALE_TAGS( 75, 66,108) 12099,  5934,   629,   187,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Hungary
    { LOCALE_TAGS( 75, 66,110) 12108,  5945,   639,   188,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/India
    { LOCALE_TAGS( 75, 66,111) 12119,  5956,   648,   189,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Indonesia
    { LOCALE_TAGS( 75, 66,114) 12128,  5967,   658,   190,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Ireland
    { LOCALE_TAGS( 75, 66,115) 12137,  5975,   665,   192,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Isle of Man
    { LOCALE_TAGS( 75, 66,116) 12146,  5983,   672,   193,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Israel
    { LOCALE_TAGS( 75, 66,117) 12155,  5991,   679,   194,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Italy
    { LOCALE_TAGS( 75, 66,119) 12164,  6002,   689,   195,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Jamaica
    { LOCALE_TAGS( 75, 66,121) 12173,  6010,   696,   196,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Jersey
    { LOCALE_TAGS( 75, 66,124) 12182,  6018,   703,   197,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Kenya
    { LOCALE_TAGS( 75, 66,125) 12191,  6030,   714,   198,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Kiribati
    { LOCALE_TAGS( 75, 66,133) 12200,  6038,   721,   199,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Lesotho
    { LOCALE_TAGS( 75, 66,134) 12209,  6050,   732,   200,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Liberia
    { LOCALE_TAGS( 75, 66,139) 12218,  6062,   743,   201,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Macao
    { LOCALE_TAGS( 75, 66,141) 12227,  6072,   752,   202,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Madagascar
    { LOCALE_TAGS( 75, 66,142) 12236,  6084,   763,   203,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Malawi
    { LOCALE_TAGS( 75, 66,143) 12245,  6096,   774,   204,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Malaysia
    { LOCALE_TAGS( 75, 66,144) 12254,  6105,   782,   205,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Maldives
    { LOCALE_TAGS( 75, 66,146) 12263,  6113,   789,   206,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Malta
    { LOCALE_TAGS( 75, 66,147) 12272,  6121,   796,   207,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Marshall Islands
    { LOCALE_TAGS( 75, 66,150) 12272,  6128,   803,   208,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Mauritius
    { LOCALE_TAGS( 75, 66,153) 12281,  6140,   814,   209,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Micronesia
    { LOCALE_TAGS( 75, 66,158) 12290,  6148,   821,   210,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Montserrat
    { LOCALE_TAGS( 75, 66,162) 12299,  6156,   828,   211,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Namibia
    { LOCALE_TAGS( 75, 66,163) 12308,  6168,   839,   212,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Nauru
    { LOCALE_TAGS( 75, 66,165) 12317,  6176,   846,   213,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Netherlands
    { LOCALE_TAGS( 75, 66,167) 12326,  6187,   856,   214,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/New Zealand
    { LOCALE_TAGS( 75, 66,169) 12335,  6202,   870,   215,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Nigeria
    { LOCALE_TAGS( 75, 66,171) 12344,  6214,   881,   216,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Niue
    { LOCALE_TAGS( 75, 66,172) 12353,  6222,   888,   217,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Norfolk Island
    { LOCALE_TAGS( 75, 66,173) 12362,  6230,   895,   218,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Northern Mariana Islands
    { LOCALE_TAGS( 75, 66,175) 12362,  6237,   902,   219,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Norway
    { LOCALE_TAGS( 75, 66,178) 12371,  6248,   912,   220,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Pakistan
    { LOCALE_TAGS( 75, 66,179) 12380,  6256,   919,   221,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Palau
    { LOCALE_TAGS( 75, 66,182) 12389,  6264,   926,   222,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Papua New Guinea
    { LOCALE_TAGS( 75, 66,185) 12398,  6272,   933,   223,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Philippines
    { LOCALE_TAGS( 75, 66,186) 12398,  6272,   933,   223,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Pitcairn
    { LOCALE_TAGS( 75, 66,187) 12407,  6280,   940,   224,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Poland
    { LOCALE_TAGS( 75, 66,188) 12416,  6291,   950,   225,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Portugal
    { LOCALE_TAGS( 75, 66,189) 12425,  6302,   960,   226,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Puerto Rico
    { LOCALE_TAGS( 75, 66,192) 12425,  6302,   960,   226,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Romania
    { LOCALE_TAGS( 75, 66,194) 12434,  6313,   970,   227,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Rwanda
    { LOCALE_TAGS( 75, 66,196) 12443,  6325,   981,   228,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Saint Helena
    { LOCALE_TAGS( 75, 66,197) 12452,  6333,   988,   229,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Saint Kitts and Nevis
    { LOCALE_TAGS( 75, 66,198) 12461,  6341,   995,   230,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Saint Lucia
    { LOCALE_TAGS( 75, 66,201) 12470,  6349,  1002,   231,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Saint Vincent and Grenadines
    { LOCALE_TAGS( 75, 66,202) 12479,  6357,  1009,   232,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Samoa
    { LOCALE_TAGS( 75, 66,208) 12488,  6365,  1016,   233,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Seychelles
    { LOCALE_TAGS( 75, 66,209) 12497,  6373,  1023,   234,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Sierra Leone
    { LOCALE_TAGS( 75, 66,210) 12506,  6385,  1034,   235,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Singapore
    { LOCALE_TAGS( 75, 66,211) 12515,  6395,  1043,   236,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Sint Maarten
    { LOCALE_TAGS( 75, 66,212) 12524,  6403,  1050,   237,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Slovakia
    { LOCALE_TAGS( 75, 66,213) 12533,  6414,  1060,   238,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Slovenia
    { LOCALE_TAGS( 75, 66,214) 12542,  6425,  1070,   239,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Solomon Islands
    { LOCALE_TAGS( 75, 66,216) 12551,  6433,  1077,   240,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/South Africa
    { LOCALE_TAGS( 75, 66,217) 12560,  6445,  1088,   241,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/South Georgia and South Sandwich Islands
    { LOCALE_TAGS( 75, 66,219) 12569,  6453,  1095,   242,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/South Sudan
    { LOCALE_TAGS( 75, 66,220) 12578,  6465,  1106,   243,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Spain
    { LOCALE_TAGS( 75, 66,222) 12587,  6476,  1116,   244,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Sudan
    { LOCALE_TAGS( 75, 66,225) 12596,  6488,  1127,   245,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Sweden
    { LOCALE_TAGS( 75, 66,226) 12605,  6499,  1137,   246,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Switzerland
    { LOCALE_TAGS( 75, 66,230) 12614,  6510,  1147,   247,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Tanzania
    { LOCALE_TAGS( 75, 66,234) 12623,  6522,  1158,   248,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Tokelau
    { LOCALE_TAGS( 75, 66,235) 12632,  6530,  1165,   249,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Tonga
    { LOCALE_TAGS( 75, 66,236) 12641,  6538,  1172,   250,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Trinidad and Tobago
    { LOCALE_TAGS( 75, 66,241) 12650,  6546,  1179,   251,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Turks and Caicos Islands
    { LOCALE_TAGS( 75, 66,242) 12659,  6554,  1186,   252,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Tuvalu
    { LOCALE_TAGS( 75, 66,243) 12668,  6562,  1193,   253,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Uganda
    { LOCALE_TAGS( 75, 66,245) 12677,  6574,  1204,   254,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/United Arab Emirates
    { LOCALE_TAGS( 75, 66,246) 12677,  6582,  1212,   255,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/United Kingdom
    { LOCALE_TAGS( 75, 66,247) 12689,  6594,  1223,   257,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/United States Outlying Islands
    { LOCALE_TAGS( 75, 66,249) 12689,  6594,  1223,   257,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/United States Virgin Islands
    { LOCALE_TAGS( 75, 66,252) 12689,  6594,  1223,   257,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Vanuatu
    { LOCALE_TAGS( 75, 66,258) 12698,  6602,  1230,   258,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/world
    { LOCALE_TAGS( 75, 66,260) 12707,  6610,  1237,   259,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Zambia
    { LOCALE_TAGS( 75, 66,261) 12716,  6622,  1248,   260,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Zimbabwe
    { LOCALE_TAGS( 75,115,246) 12725,  6634,  1259,   261,    0,    6,   82,    0, 1420,   83,   90,  6,  6,  9,  7,  9,  7,  7, }, // English/Shavian/United Kingdom
    { LOCALE_TAGS( 76, 27,193) 12726,  6634,  1259,   262,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Erzya/Cyrillic/Russia
    { LOCALE_TAGS( 77, 66,258) 12835,  6655,  1259,   263,    0,    6,   56,    0, 1429, 1440, 1457,  6,  6,  5,  7, 11, 17, 18, }, // Esperanto/Latin/world
    { LOCALE_TAGS( 78, 66, 75) 13158,  6799,  1259,   266,    0,   57,   21,    0, 1475,   83,   90,  6,  6,  6,  7,  4,  7,  7, }, // Estonian/Latin/Estonia
    { LOCALE_TAGS( 79, 66, 92) 13301,  6957,  1259,   269,    0,    6,   91,    0, 1479, 1486, 1500,  6,  6,  6,  7,  7, 14, 16, }, // Ewe/Latin/Ghana
    { LOCALE_TAGS( 79, 66,233) 13345,  7107,  1263,   272,    0,    6,   91,    0, 1479, 1486, 1500,  6,  6,  6,  7,  7, 14, 16, }, // Ewe/Latin/Togo
    { LOCALE_TAGS( 80, 66, 40) 13345,  7107,  1263,   272,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Ewondo/Latin/Cameroon
    { LOCALE_TAGS( 81, 66, 81) 13387,  7107,  1263,   273,    0,    6,    0,    0, 1516, 1522, 1535,  6,  6,  5,  7,  6, 13, 12, }, // Faroese/Latin/Faroe Islands
    { LOCALE_TAGS( 81, 66, 65) 13464,  7251,  1263,   276,    0,    6,    0,    0, 1516, 1522, 1535,  6,  6,  5,  7,  6, 13, 12, }, // Faroese/Latin/Denmark
    { LOCALE_TAGS( 83, 66,185) 13464,  7251,  1263,   276,    0,    6,    0,    0,  858, 1547, 1569,  6,  6,  5,  7, 10, 22, 19, }, // Filipino/Latin/Philippines
    { LOCALE_TAGS( 84, 66, 83) 13512,  7398,  1263,   279,   63,   68,   56,    0, 1588, 1603, 1620,  5,  5,  5,  7, 15, 17, 13, }, // Finnish/Latin/Finland
    { LOCALE_TAGS( 85, 66, 84) 13655,  7555,  1263,   282,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/France
    { LOCALE_TAGS( 85, 66,  4) 13811,  7712,  1279,   286,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Algeria
    { LOCALE_TAGS( 85, 66, 23) 13811,  7712,  1279,   286,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Belgium
    { LOCALE_TAGS( 85, 66, 25) 13811,  7712,  1279,   286,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Benin
    { LOCALE_TAGS( 85, 66, 37) 13811,  7712,  1279,   286,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Burkina Faso
    { LOCALE_TAGS( 85, 66, 38) 13811,  7712,  1279,   286,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Burundi
    { LOCALE_TAGS( 85, 66, 40) 13811,  7712,  1279,   286,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Cameroon
    { LOCALE_TAGS( 85, 66, 41) 13811,  7712,  1279,   286,    0,   57,   56,    0, 1633, 1678, 1696,  6,  6,  5,  7, 10, 18, 18, }, // French/Latin/Canada
    { LOCALE_TAGS( 85, 66, 46) 13831,  7802,  1294,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Central African Republic
    { LOCALE_TAGS( 85, 66, 48) 13831,  7802,  1294,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Chad
    { LOCALE_TAGS( 85, 66, 55) 13831,  7802,  1294,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Comoros
    { LOCALE_TAGS( 85, 66, 56) 13831,  7802,  1294,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Congo - Brazzaville
    { LOCALE_TAGS( 85, 66, 57) 13831,  7802,  1294,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Congo - Kinshasa
    { LOCALE_TAGS( 85, 66, 67) 13831,  7802,  1294,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Djibouti
    { LOCALE_TAGS( 85, 66, 73) 13831,  7802,  1294,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Equatorial Guinea
    { LOCALE_TAGS( 85, 66, 85) 13831,  7802,  1294,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/French Guiana
    { LOCALE_TAGS( 85, 66, 86) 13831,  7803,  1295,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/French Polynesia
    { LOCALE_TAGS( 85, 66, 88) 13831,  7803,  1295,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Gabon
    { LOCALE_TAGS( 85, 66, 97) 13831,  7803,  1295,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Guadeloupe
    { LOCALE_TAGS( 85, 66,102) 13831,  7803,  1295,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Guinea
    { LOCALE_TAGS( 85, 66,104) 13831,  7803,  1295,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Haiti
    { LOCALE_TAGS( 85, 66,118) 13831,  7803,  1295,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Ivory Coast
    { LOCALE_TAGS( 85, 66,138) 13831,  7803,  1295,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Luxembourg
    { LOCALE_TAGS( 85, 66,141) 13831,  7803,  1295,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Madagascar
    { LOCALE_TAGS( 85, 66,145) 13831,  7803,  1295,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Mali
    { LOCALE_TAGS( 85, 66,148) 13831,  7803,  1295,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Martinique
    { LOCALE_TAGS( 85, 66,149) 13831,  7803,  1295,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Mauritania
    { LOCALE_TAGS( 85, 66,150) 13831,  7803,  1295,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Mauritius
    { LOCALE_TAGS( 85, 66,151) 13831,  7803,  1295,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Mayotte
    { LOCALE_TAGS( 85, 66,155) 13831,  7803,  1295,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Monaco
    { LOCALE_TAGS( 85, 66,159) 13831,  7803,  1295,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Morocco
    { LOCALE_TAGS( 85, 66,166) 13831,  7803,  1295,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/New Caledonia
    { LOCALE_TAGS( 85, 66,170) 13831,  7803,  1295,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Niger
    { LOCALE_TAGS( 85, 66,191) 13831,  7803,  1295,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Reunion
    { LOCALE_TAGS( 85, 66,194) 13831,  7803,  1295,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Rwanda
    { LOCALE_TAGS( 85, 66,195) 13831,  7803,  1295,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Saint Barthelemy
    { LOCALE_TAGS( 85, 66,199) 13831,  7803,  1295,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Saint Martin
    { LOCALE_TAGS( 85, 66,200) 13831,  7803,  1295,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Saint Pierre and Miquelon
    { LOCALE_TAGS( 85, 66,206) 13831,  7803,  1295,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Senegal
    { LOCALE_TAGS( 85, 66,208) 13831,  7803,  1295,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Seychelles
    { LOCALE_TAGS( 85, 66,226) 13831,  7803,  1295,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Switzerland
    { LOCALE_TAGS( 85, 66,227) 13831,  7803,  1295,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Syria
    { LOCALE_TAGS( 85, 66,233) 13831,  7803,  1295,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Togo
    { LOCALE_TAGS( 85, 66,238) 13831,  7803,  1295,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Tunisia
    { LOCALE_TAGS( 85, 66,252) 13831,  7803,  1295,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Vanuatu
    { LOCALE_TAGS( 85, 66,256) 13831,  7803,  1295,   288,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Wallis and Futuna
    { LOCALE_TAGS( 86, 66,117) 13831,  7803,  1295,   288,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Friulian/Latin/Italy
    { LOCALE_TAGS( 87, 66,206) 13886,  7807,  1299,   289,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Senegal
    { LOCALE_TAGS( 87,  1, 37) 13928,  7807,  1299,   290,    0,    6,   97,    0, 1714, 1731, 1767,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Burkina Faso
    { LOCALE_TAGS( 87,  1, 40) 13928,  7807,  1299,   290,    0,    6,   97,    0, 1714, 1731, 1767,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Cameroon
    { LOCALE_TAGS( 87,  1, 89) 13928,  7807,  1299,   290,    0,    6,   97,    0, 1714, 1731, 1767,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Gambia
    { LOCALE_TAGS( 87,  1, 92) 13928,  7807,  1299,   290,    0,    6,   97,    0, 1714, 1731, 1767,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Ghana
    { LOCALE_TAGS( 87,  1,101) 13928,  7807,  1299,   290,    0,    6,   97,    0, 1714, 1731, 1767,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Guinea-Bissau
    { LOCALE_TAGS( 87,  1,102) 13928,  7807,  1299,   290,    0,    6,   97,    0, 1714, 1731, 1767,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Guinea
    { LOCALE_TAGS( 87,  1,134) 14343,  7958,  1299,   294,    0,    6,   97,    0, 1714, 1731, 1767,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Liberia
    { LOCALE_TAGS( 87,  1,149) 14343,  7958,  1299,   294,    0,    6,   97,    0, 1714, 1731, 1767,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Mauritania
    { LOCALE_TAGS( 87,  1,169) 14343,  7958,  1299,   294,    0,    6,   97,    0, 1714, 1731, 1767,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Nigeria
    { LOCALE_TAGS( 87,  1,170) 14343,  7958,  1299,   294,    0,    6,   97,    0, 1714, 1731, 1767,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Niger
    { LOCALE_TAGS( 87,  1,206) 14343,  7958,  1299,   294,    0,    6,   97,    0, 1714, 1731, 1767,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Senegal
    { LOCALE_TAGS( 87,  1,209) 14343,  7958,  1299,   294,    0,    6,   97,    0, 1714, 1731, 1767,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Sierra Leone
    { LOCALE_TAGS( 87, 66, 37) 14343,  7958,  1299,   294,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Burkina Faso
    { LOCALE_TAGS( 87, 66, 40) 14343,  7958,  1299,   294,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Cameroon
    { LOCALE_TAGS( 87, 66, 89) 14343,  7958,  1299,   294,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Gambia
    { LOCALE_TAGS( 87, 66, 92) 14343,  7958,  1299,   294,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Ghana
    { LOCALE_TAGS( 87, 66,101) 14343,  7958,  1299,   294,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Guinea-Bissau
    { LOCALE_TAGS( 87, 66,102) 14343,  7958,  1299,   294,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Guinea
    { LOCALE_TAGS( 87, 66,134) 14343,  7958,  1299,   294,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Liberia
    { LOCALE_TAGS( 87, 66,149) 14343,  7958,  1299,   294,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Mauritania
    { LOCALE_TAGS( 87, 66,169) 14343,  7958,  1299,   294,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Nigeria
    { LOCALE_TAGS( 87, 66,170) 14343,  7958,  1299,   294,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Niger
    { LOCALE_TAGS( 87, 66,209) 14343,  7958,  1299,   294,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Sierra Leone
    { LOCALE_TAGS( 88, 66,246) 14343,  7958,  1299,   294,    0,    6,    0,    0,    0, 1795, 1805,  6,  6,  5,  7,  2, 10, 18, }, // Gaelic/Latin/United Kingdom
    { LOCALE_TAGS( 89, 66, 92) 14497,  8116,  1310,   298,    0,    6,    0,    0, 1823, 1828, 1846,  6,  6,  5,  7,  5, 18, 20, }, // Ga/Latin/Ghana
    { LOCALE_TAGS( 90, 66,220) 14602,  8141,  1310,   301,    0,    6,    0,    0, 1866, 1877, 1897,  6,  6,  5,  7, 11, 20, 20, }, // Galician/Latin/Spain
    { LOCALE_TAGS( 91, 66,243) 14775,  8289,  1314,   304,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Ganda/Latin/Uganda
    { LOCALE_TAGS( 92, 33, 77) 14817,  8289,  1314,   305,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Geez/Ethiopic/Ethiopia
    { LOCALE_TAGS( 92, 33, 74) 14859,  8289,  1314,   306,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Geez/Ethiopic/Eritrea
    { LOCALE_TAGS( 93, 35, 90) 14859,  8289,  1314,   306,    0,    6,    0,    0, 1917, 1924, 1942,  6,  6,  5,  7,  7, 18, 15, }, // Georgian/Georgian/Georgia
    { LOCALE_TAGS( 94, 66, 91) 15276,  8433,  1314,   309,    0,    6,    0,    0, 1957, 1970, 1985,  6,  6,  5,  7, 13, 15, 15, }, // German/Latin/Germany
    { LOCALE_TAGS( 94, 66, 16) 15406,  8590,  1317,   312,    0,    6,    0,    0, 1957, 1970, 1985,  6,  6,  5,  7, 13, 15, 15, }, // German/Latin/Austria
    { LOCALE_TAGS( 94, 66, 23) 15406,  8590,  1317,   312,    0,    6,    0,    0, 1957, 1970, 1985,  6,  6,  5,  7, 13, 15, 15, }, // German/Latin/Belgium
    { LOCALE_TAGS( 94, 66,117) 15406,  8590,  1317,   312,    0,    6,    0,    0, 1957, 1970, 1985,  6,  6,  5,  7, 13, 15, 15, }, // German/Latin/Italy
    { LOCALE_TAGS( 94, 66,136) 15406,  8590,  1317,   312,    0,    6,    0,    0, 1957, 1970, 1985,  6,  6,  5,  7, 13, 15, 15, }, // German/Latin/Liechtenstein
    { LOCALE_TAGS( 94, 66,138) 15406,  8590,  1317,   312,    0,    6,    0,    0, 1957, 1970, 1985,  6,  6,  5,  7, 13, 15, 15, }, // German/Latin/Luxembourg
    { LOCALE_TAGS( 94, 66,226) 15406,  8590,  1317,   312,    0,    6,    0,    0, 1957, 1970, 1985,  6,  6,  5,  7, 13, 15, 15, }, // German/Latin/Switzerland
    { LOCALE_TAGS( 96, 39, 94) 15409,  8592,  1317,   312,    0,    6,    0,   27, 2000, 2008, 2026,  6,  6,  5,  9,  8, 18, 15, }, // Greek/Greek/Greece
    { LOCALE_TAGS( 96, 39, 63) 15826,  8742,  1320,   315,    0,    6,    0,   27, 2000, 2008, 2026,  6,  6,  5,  9,  8, 18, 15, }, // Greek/Greek/Cyprus
    { LOCALE_TAGS( 97, 66,183) 15826,  8742,  1320,   315,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Guarani/Latin/Paraguay
    { LOCALE_TAGS( 98, 40,110) 15868,  8746,  1320,   316,    0,    6,    0,    0, 2041, 2047, 2058,  6,  6,  5,  7,  6, 11, 11, }, // Gujarati/Gujarati/India
    { LOCALE_TAGS( 99, 66,124) 16285,  8902,  1321,   319,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Gusii/Latin/Kenya
    { LOCALE_TAGS(100, 66,104) 16327,  8902,  1321,   320,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // Haitian/Latin/Haiti
    { LOCALE_TAGS(101, 66,169) 16483,  9059,  1337,   324,    0,    6,    0,    0, 2069, 2078, 2100,  6,  6,  5,  7,  9, 22, 15, }, // Hausa/Latin/Nigeria
    { LOCALE_TAGS(101,  4,169) 16525,  9203,  1337,   327,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Hausa/Arabic/Nigeria
    { LOCALE_TAGS(101,  4,222) 16528,  9203,  1337,   328,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Hausa/Arabic/Sudan
    { LOCALE_TAGS(101, 66, 92) 16531,  9203,  1337,   329,    0,    6,    0,    0, 2069, 2078, 2100,  6,  6,  5,  7,  9, 22, 15, }, // Hausa/Latin/Ghana
    { LOCALE_TAGS(101, 66,170) 16531,  9203,  1337,   329,    0,    6,    0,    0, 2069, 2078, 2100,  6,  6,  5,  7,  9, 22, 15, }, // Hausa/Latin/Niger
    { LOCALE_TAGS(102, 66,248) 16531,  9203,  1337,   329,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Hawaiian/Latin/United States
    { LOCALE_TAGS(103, 47,116) 16573,  9203,  1339,   331,   73,   80,  105,    0, 2115, 2122, 2136,  7,  7,  6,  7,  7, 14, 13, }, // Hebrew/Hebrew/Israel
    { LOCALE_TAGS(105, 29,110) 16990,  9357,  1339,   334,    0,    6,    0,    0, 2149,  565,  576,  6,  6,  5,  7,  6, 11, 13, }, // Hindi/Devanagari/India
    { LOCALE_TAGS(105, 66,110) 17407,  9504,  1340,   338,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // Hindi/Latin/India
    { LOCALE_TAGS(107, 66,108) 17464,  9662,  1350,   342,    0,    6,    0,    0, 2155, 2161, 2171,  6,  6,  5,  7,  6, 10, 12, }, // Hungarian/Latin/Hungary
    { LOCALE_TAGS(108, 66,109) 17652,  9819,  1354,   345,    0,    6,    0,    0,    0, 2183, 2198,  6,  6,  5,  7,  2, 15, 14, }, // Icelandic/Latin/Iceland
    { LOCALE_TAGS(109, 66,258) 17791,  9966,  1354,   348,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Ido/Latin/world
    { LOCALE_TAGS(110, 66,169) 17833,  9966,  1354,   349,    0,    6,    0,    0, 2212, 2218, 2231,  6,  6,  5,  7,  6, 13, 10, }, // Igbo/Latin/Nigeria
    { LOCALE_TAGS(111, 66, 83) 17876, 10110,  1354,   352,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Inari Sami/Latin/Finland
    { LOCALE_TAGS(112, 66,111) 17918, 10110,  1354,   353,   45,   51,    0,    0, 2241, 2249, 2265,  6,  6,  5,  7,  8, 16, 20, }, // Indonesian/Latin/Indonesia
    { LOCALE_TAGS(114, 66,258) 17996, 10267,  1368,   356,    0,    6,    0,    0, 2285, 2295, 2312,  6,  6,  5,  7, 10, 17, 17, }, // Interlingua/Latin/world
    { LOCALE_TAGS(115, 66, 75) 18073, 10411,  1368,   359,    0,    6,  111,    0, 2329, 2341, 2362,  6,  6,  5,  7, 12, 21, 20, }, // Interlingue/Latin/Estonia
    { LOCALE_TAGS(116, 18, 41) 18192, 10442,  1368,   362,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Inuktitut/Canadian Aboriginal/Canada
    { LOCALE_TAGS(116, 66, 41) 18234, 10442,  1368,   363,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Inuktitut/Latin/Canada
    { LOCALE_TAGS(118, 66,114) 18234, 10442,  1368,   363,    0,    6,  116,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Irish/Latin/Ireland
    { LOCALE_TAGS(118, 66,246) 18405, 10599,  1373,   366,    0,    6,  116,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Irish/Latin/United Kingdom
    { LOCALE_TAGS(119, 66,117) 18405, 10599,  1373,   366,    0,    6,    0,    0, 2382, 2388, 2404,  6,  6,  5,  7,  6, 16, 14, }, // Italian/Latin/Italy
    { LOCALE_TAGS(119, 66,203) 18528, 10747,  1376,   369,    0,    6,    0,    0, 2382, 2388, 2404,  6,  6,  5,  7,  6, 16, 14, }, // Italian/Latin/San Marino
    { LOCALE_TAGS(119, 66,226) 18528, 10747,  1376,   369,    0,    6,    0,    0, 2382, 2388, 2404,  6,  6,  5,  7,  6, 16, 14, }, // Italian/Latin/Switzerland
    { LOCALE_TAGS(119, 66,253) 18528, 10747,  1376,   369,    0,    6,    0,    0, 2382, 2388, 2404,  6,  6,  5,  7,  6, 16, 14, }, // Italian/Latin/Vatican City
    { LOCALE_TAGS(120, 53,120) 18528, 10747,  1376,   369,    0,    6,    0,   12,  793,  797, 2418,  6,  6,  5,  6,  4,  5,  5, }, // Japanese/Japanese/Japan
    { LOCALE_TAGS(121, 66,111) 18945, 10904,  1377,   372,    0,    6,    0,    0, 2423, 2431, 2447,  6,  6,  5,  7,  8, 16, 15, }, // Javanese/Latin/Indonesia
    { LOCALE_TAGS(122, 66,169) 19035, 11048,  1377,   375,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Jju/Latin/Nigeria
    { LOCALE_TAGS(123, 66,206) 19077, 11048,  1377,   376,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Jola-Fonyi/Latin/Senegal
    { LOCALE_TAGS(124, 66, 43) 19119, 11048,  1377,   377,    0,    6,    0,    0, 2462, 2471, 2487,  6,  6,  5,  7,  9, 16, 18, }, // Kabuverdianu/Latin/Cape Verde
    { LOCALE_TAGS(125, 66,  4) 19192, 11075,  1377,   380,    0,    6,  121,    0, 2505, 2513, 2528,  6,  6,  6,  7,  8, 15, 16, }, // Kabyle/Latin/Algeria
    { LOCALE_TAGS(126, 66, 40) 19426, 11217,  1377,   383,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kako/Latin/Cameroon
    { LOCALE_TAGS(127, 66, 95) 19468, 11217,  1377,   384,    0,   57,    0,   36,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kalaallisut/Latin/Greenland
    { LOCALE_TAGS(128, 66,124) 19511, 11217,  1377,   385,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kalenjin/Latin/Kenya
    { LOCALE_TAGS(129, 66,124) 19553, 11217,  1377,   386,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kamba/Latin/Kenya
    { LOCALE_TAGS(130, 56,110) 19595, 11217,  1377,   387,    0,    6,    0,    0, 2544, 2550, 2565,  6,  6,  5,  7,  6, 15, 11, }, // Kannada/Kannada/India
    { LOCALE_TAGS(132,  4,110) 20012, 11373,  1378,   390,    0,    6,    0,    0, 2576, 2583, 2597,  6,  6,  5,  7,  7, 14, 15, }, // Kashmiri/Arabic/India
    { LOCALE_TAGS(132, 29,110) 20426, 11527,  1378,   393,    0,    6,  127,    0, 2612, 2617, 2628,  6,  6, 11,  7,  5, 11, 13, }, // Kashmiri/Devanagari/India
    { LOCALE_TAGS(133, 27,123) 20467, 11536,  1378,   394,    0,    6,    0,    0, 2641, 2650, 2670,  6,  6,  5,  7,  9, 20, 15, }, // Kazakh/Cyrillic/Kazakhstan
    { LOCALE_TAGS(133,  4, 50) 20884, 11680,  1379,   397,    0,    6,    0,    0, 2685, 2694, 2714,  6,  6,  5,  7,  9, 20, 15, }, // Kazakh/Arabic/China
    { LOCALE_TAGS(134, 66, 40) 21300, 11828,  1387,   400,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kenyang/Latin/Cameroon
    { LOCALE_TAGS(135, 60, 39) 21342, 11828,  1387,   401,    0,    6,  138,    0, 2729, 2740, 2760,  6,  6, 11,  7, 11, 20, 20, }, // Khmer/Khmer/Cambodia
    { LOCALE_TAGS(136, 66, 99) 21759, 11972,  1387,   404,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kiche/Latin/Guatemala
    { LOCALE_TAGS(137, 66,124) 21801, 11972,  1387,   405,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kikuyu/Latin/Kenya
    { LOCALE_TAGS(138, 66,194) 21843, 11972,  1387,   406,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kinyarwanda/Latin/Rwanda
    { LOCALE_TAGS(141, 29,110) 21885, 11973,  1387,   407,    0,    6,    0,    0, 2780, 2786, 2801,  6,  6,  5,  7,  6, 15, 13, }, // Konkani/Devanagari/India
    { LOCALE_TAGS(141, 66,110) 22301, 12117,  1388,   410,    0,    6,    0,    0, 2814, 2821, 2836,  6,  6,  5,  7,  7, 15, 26, }, // Konkani/Latin/India
    { LOCALE_TAGS(142, 63,218) 22343, 12118,  1388,   411,    0,    6,    0,   43, 2862, 2867, 2873,  6,  6,  5,  6,  5,  6,  9, }, // Korean/Korean/South Korea
    { LOCALE_TAGS(142, 63, 50) 22760, 12275,  1388,   414,    0,    6,    0,   43, 2862, 2867, 2873,  6,  6,  5,  6,  5,  6,  9, }, // Korean/Korean/China
    { LOCALE_TAGS(142, 63,174) 22760, 12275,  1388,   414,    0,    6,    0,   43, 2862, 2867, 2873,  6,  6,  5,  6,  5,  6,  9, }, // Korean/Korean/North Korea
    { LOCALE_TAGS(144, 66,145) 22760, 12276,  1388,   414,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Koyraboro Senni/Latin/Mali
    { LOCALE_TAGS(145, 66,145) 22802, 12276,  1388,   415,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Koyra Chiini/Latin/Mali
    { LOCALE_TAGS(146, 66,134) 22844, 12276,  1388,   416,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kpelle/Latin/Liberia
    { LOCALE_TAGS(146, 66,102) 22886, 12276,  1388,   417,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kpelle/Latin/Guinea
    { LOCALE_TAGS(148, 66,239) 22886, 12276,  1388,   417,    0,    6,    0,    0, 2882, 2894, 2917,  6,  6,  5,  7, 12, 23, 22, }, // Kurdish/Latin/Turkey
    { LOCALE_TAGS(149, 66, 40) 23053, 12420,  1400,   420,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kwasio/Latin/Cameroon
    { LOCALE_TAGS(150, 27,128) 23095, 12420,  1400,   421,    0,    6,    0,    0, 2939,   83,   90,  6,  6,  5,  7, 11,  7,  7, }, // Kyrgyz/Cyrillic/Kyrgyzstan
    { LOCALE_TAGS(151, 66,248) 23512, 12564,  1400,   424,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Lakota/Latin/United States
    { LOCALE_TAGS(152, 66,230) 23554, 12564,  1400,   425,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Langi/Latin/Tanzania
    { LOCALE_TAGS(153, 65,129) 23596, 12564,  1400,   426,    0,    6,    0,    0, 2950, 2957, 2972,  6,  6,  5,  7,  7, 15, 14, }, // Lao/Lao/Laos
    { LOCALE_TAGS(154, 66,253) 24013, 12718,  1400,   429,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Latin/Latin/Vatican City
    { LOCALE_TAGS(155, 66,131) 24055, 12718,  1400,   430,    0,    6,    0,   36, 2986, 3001, 3020,  6,  6,  5,  7, 15, 19, 17, }, // Latvian/Latin/Latvia
    { LOCALE_TAGS(158, 66, 57) 24422, 12865,  1403,   433,    0,    6,    0,    0, 3037,   83,   90,  6,  6,  5,  7, 12,  7,  7, }, // Lingala/Latin/Congo - Kinshasa
    { LOCALE_TAGS(158, 66,  7) 24464, 12870,  1403,   434,    0,    6,    0,    0, 3037,   83,   90,  6,  6,  5,  7, 12,  7,  7, }, // Lingala/Latin/Angola
    { LOCALE_TAGS(158, 66, 46) 24464, 12870,  1403,   434,    0,    6,    0,    0, 3037,   83,   90,  6,  6,  5,  7, 12,  7,  7, }, // Lingala/Latin/Central African Republic
    { LOCALE_TAGS(158, 66, 56) 24464, 12870,  1403,   434,    0,    6,    0,    0, 3037,   83,   90,  6,  6,  5,  7, 12,  7,  7, }, // Lingala/Latin/Congo - Brazzaville
    { LOCALE_TAGS(160, 66,137) 24464, 12870,  1403,   434,    0,   57,    0,    0, 3049, 3059, 3076,  6,  6,  5,  7, 10, 17, 18, }, // Lithuanian/Latin/Lithuania
    { LOCALE_TAGS(161, 66,258) 24851, 13027,  1403,   437,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Lojban/Latin/world
    { LOCALE_TAGS(162, 66, 91) 24893, 13029,  1403,   438,    0,    6,    0,    0, 3094, 3109, 3122,  6,  6,  5,  7, 15, 13, 15, }, // Lower Sorbian/Latin/Germany
    { LOCALE_TAGS(163, 66, 91) 25014, 13173,  1406,   441,   45,   51,   56,    0, 3137, 3144, 3159,  6,  6,  5,  7,  7, 15, 13, }, // Low German/Latin/Germany
    { LOCALE_TAGS(163, 66,165) 25070, 13201,  1406,   442,   45,   51,   56,    0, 3137, 3144, 3159,  6,  6,  5,  7,  7, 15, 13, }, // Low German/Latin/Netherlands
    { LOCALE_TAGS(164, 66, 57) 25070, 13201,  1406,   442,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Luba-Katanga/Latin/Congo - Kinshasa
    { LOCALE_TAGS(165, 66,225) 25112, 13201,  1406,   443,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Lule Sami/Latin/Sweden
    { LOCALE_TAGS(165, 66,175) 25154, 13201,  1406,   444,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Lule Sami/Latin/Norway
    { LOCALE_TAGS(166, 66,124) 25154, 13201,  1406,   444,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Luo/Latin/Kenya
    { LOCALE_TAGS(167, 66,138) 25196, 13201,  1406,   445,    0,    6,    0,    0, 3172, 3179, 3192,  6,  6,  5,  7,  7, 13, 13, }, // Luxembourgish/Latin/Luxembourg
    { LOCALE_TAGS(168, 66,124) 25306, 13346,  1406,   448,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Luyia/Latin/Kenya
    { LOCALE_TAGS(169, 27,140) 25348, 13346,  1406,   449,    0,    6,    0,    0, 3205,   83,   90,  6,  6,  5,  7, 11,  7,  7, }, // Macedonian/Cyrillic/Macedonia
    { LOCALE_TAGS(170, 66,230) 25765, 13493,  1409,   452,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Machame/Latin/Tanzania
    { LOCALE_TAGS(171, 29,110) 25807, 13493,  1409,   453,    0,    6,  149,    0, 2149,  565,  576,  6,  6,  8,  7,  6, 11, 13, }, // Maithili/Devanagari/India
    { LOCALE_TAGS(172, 66,160) 26185, 13604,  1409,   454,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Makhuwa-Meetto/Latin/Mozambique
    { LOCALE_TAGS(173, 66,230) 26227, 13604,  1409,   455,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Makonde/Latin/Tanzania
    { LOCALE_TAGS(174, 66,141) 26269, 13604,  1409,   456,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Malagasy/Latin/Madagascar
    { LOCALE_TAGS(175, 74,110) 26311, 13604,  1409,   457,    0,    6,  157,    0, 3216, 3223, 3243,  6,  6,  9,  7,  7, 20, 16, }, // Malayalam/Malayalam/India
    { LOCALE_TAGS(176, 66,143) 26728, 13760,  1410,   460,    0,    6,    0,    0, 2241, 3259, 3274,  6,  6,  5,  7,  8, 15, 14, }, // Malay/Latin/Malaysia
    { LOCALE_TAGS(176,  4, 35) 26780, 13914,  1413,   463,    0,    6,    0,    0, 3288,   83,   90,  6,  6,  5,  7,  7,  7,  7, }, // Malay/Arabic/Brunei
    { LOCALE_TAGS(176,  4,143) 26787, 13921,  1413,   464,    0,    6,    0,    0, 3288,   83,   90,  6,  6,  5,  7,  7,  7,  7, }, // Malay/Arabic/Malaysia
    { LOCALE_TAGS(176, 66, 35) 26794, 13928,  1413,   465,    0,    6,    0,    0, 2241, 3259, 3274,  6,  6,  5,  7,  8, 15, 14, }, // Malay/Latin/Brunei
    { LOCALE_TAGS(176, 66,111) 26794, 13928,  1413,   465,   45,   51,    0,    0, 2241, 3259, 3274,  6,  6,  5,  7,  8, 15, 14, }, // Malay/Latin/Indonesia
    { LOCALE_TAGS(176, 66,210) 26794, 13928,  1413,   465,    0,    6,    0,    0, 2241, 3259, 3274,  6,  6,  5,  7,  8, 15, 14, }, // Malay/Latin/Singapore
    { LOCALE_TAGS(177, 66,146) 26794, 13928,  1413,   465,    0,    6,    0,    0, 3295, 3305,   90,  6,  6,  5,  7, 10, 15,  7, }, // Maltese/Latin/Malta
    { LOCALE_TAGS(179,  9,110) 26866, 13930,  1417,   466,    0,    6,  166,    0, 3320, 3327, 3350,  6,  6, 11,  7,  7, 23, 19, }, // Manipuri/Bangla/India
    { LOCALE_TAGS(179, 78,110) 26908, 13939,  1417,   467,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Manipuri/Meitei Mayek/India
    { LOCALE_TAGS(180, 66,115) 26908, 13939,  1417,   468,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Manx/Latin/Isle of Man
    { LOCALE_TAGS(181, 66,167) 26950, 13939,  1421,   469,    0,    6,    0,    0, 3369, 3374, 3389,  6,  6,  5,  7,  5, 15, 12, }, // Maori/Latin/New Zealand
    { LOCALE_TAGS(182, 66, 49) 27093, 14083,  1421,   472,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Mapuche/Latin/Chile
    { LOCALE_TAGS(183, 29,110) 27135, 14083,  1421,   473,    0,    6,  177,    0, 2780, 3401, 3414,  6,  6,  7,  7,  6, 13, 18, }, // Marathi/Devanagari/India
    { LOCALE_TAGS(185, 66,124) 27552, 14239,  1422,   476,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Masai/Latin/Kenya
    { LOCALE_TAGS(185, 66,230) 27594, 14239,  1422,   477,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Masai/Latin/Tanzania
    { LOCALE_TAGS(186,  4,112) 27594, 14239,  1422,   477,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Mazanderani/Arabic/Iran
    { LOCALE_TAGS(188, 66,124) 27636, 14239,  1422,   478,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Meru/Latin/Kenya
    { LOCALE_TAGS(189, 66, 40) 27678, 14239,  1422,   479,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Meta/Latin/Cameroon
    { LOCALE_TAGS(190, 66, 41) 27720, 14239,  1422,   480,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Mohawk/Latin/Canada
    { LOCALE_TAGS(191, 27,156) 27762, 14239,  1422,   481,    0,    6,    0,    0, 3432, 3440, 3457,  6,  6,  5,  7,  8, 17, 13, }, // Mongolian/Cyrillic/Mongolia
    { LOCALE_TAGS(191, 83, 50) 28179, 14383,  1422,   484,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Mongolian/Mongolian/China
    { LOCALE_TAGS(191, 83,156) 28221, 14383,  1422,   485,    0,    6,   21,    0, 3470, 3476, 3494,  6,  6,  6,  7,  6, 18, 15, }, // Mongolian/Mongolian/Mongolia
    { LOCALE_TAGS(192, 66,150) 28223, 14392,  1422,   486,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Morisyen/Latin/Mauritius
    { LOCALE_TAGS(193, 66, 40) 28265, 14392,  1422,   487,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Mundang/Latin/Cameroon
    { LOCALE_TAGS(194, 66,248) 28307, 14392,  1422,   488,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Muscogee/Latin/United States
    { LOCALE_TAGS(195, 66,162) 28349, 14392,  1422,   489,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Nama/Latin/Namibia
    { LOCALE_TAGS(197, 66,248) 28391, 14392,  1422,   490,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Navajo/Latin/United States
    { LOCALE_TAGS(199, 29,164) 28433, 14392,  1422,   491,    0,    6,    0,    0, 2149, 3509, 3516,  6,  6,  5,  7,  6,  7,  7, }, // Nepali/Devanagari/Nepal
    { LOCALE_TAGS(199, 29,110) 28850, 14536,  1422,   494,    0,    6,    0,    0, 2149, 3509, 3516,  6,  6,  5,  7,  6,  7,  7, }, // Nepali/Devanagari/India
    { LOCALE_TAGS(201, 66, 40) 28850, 14537,  1423,   494,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Ngiemboon/Latin/Cameroon
    { LOCALE_TAGS(202, 66, 40) 28892, 14537,  1423,   495,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Ngomba/Latin/Cameroon
    { LOCALE_TAGS(203, 66,169) 28934, 14537,  1423,   496,    0,    6,    0,    0, 3523, 3530, 3543,  6,  6,  5,  7,  7, 13, 14, }, // Nigerian Pidgin/Latin/Nigeria
    { LOCALE_TAGS(204, 90,102) 29336, 14681,  1423,   499,    0,    6,  184,    0, 3557, 3565, 3565,  6,  6,  6,  7,  8, 20, 16, }, // Nko/Nko/Guinea
    { LOCALE_TAGS(205,  4,112) 29752, 14825,  1423,   502,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Northern Luri/Arabic/Iran
    { LOCALE_TAGS(205,  4,113) 29794, 14826,  1423,   503,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Northern Luri/Arabic/Iraq
    { LOCALE_TAGS(206, 66,175) 29794, 14826,  1423,   503,    0,   57,   56,   36, 3585, 3592, 3606,  6,  6,  5,  7,  7, 14, 13, }, // Northern Sami/Latin/Norway
    { LOCALE_TAGS(206, 66, 83) 29837, 14831,  1427,   504,    0,    6,   91,   36, 3585, 3619, 3631,  6,  6,  6,  7,  7, 12, 13, }, // Northern Sami/Latin/Finland
    { LOCALE_TAGS(206, 66,225) 29879, 14973,  1431,   507,    0,   57,   56,   36, 3585, 3592, 3606,  6,  6,  5,  7,  7, 14, 13, }, // Northern Sami/Latin/Sweden
    { LOCALE_TAGS(207, 66,216) 29879, 14973,  1431,   507,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Northern Sotho/Latin/South Africa
    { LOCALE_TAGS(208, 66,261) 29921, 14974,  1431,   508,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // North Ndebele/Latin/Zimbabwe
    { LOCALE_TAGS(209, 66,175) 29963, 14974,  1431,   509,    0,    6,    0,    0, 3644, 3659, 3673,  6,  6,  5,  7, 15, 14, 14, }, // Norwegian Bokmal/Latin/Norway
    { LOCALE_TAGS(209, 66,224) 30082, 15131,  1442,   513,    0,    6,    0,    0, 3644, 3659, 3673,  6,  6,  5,  7, 15, 14, 14, }, // Norwegian Bokmal/Latin/Svalbard and Jan Mayen
    { LOCALE_TAGS(210, 66,175) 30082, 15131,  1442,   513,    0,    6,    0,    0, 3644, 3659, 3687,  6,  6,  5,  7, 15, 14, 14, }, // Norwegian Nynorsk/Latin/Norway
    { LOCALE_TAGS(211, 66,219) 30202, 15288,  1453,   517,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Nuer/Latin/South Sudan
    { LOCALE_TAGS(212, 66,142) 30244, 15288,  1453,   518,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Nyanja/Latin/Malawi
    { LOCALE_TAGS(213, 66,243) 30286, 15288,  1453,   519,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Nyankole/Latin/Uganda
    { LOCALE_TAGS(214, 66, 84) 30328, 15288,  1453,   520,    0,    6,   56,    0,  272, 3701, 3719,  6,  6,  5,  7,  9, 18, 16, }, // Occitan/Latin/France
    { LOCALE_TAGS(214, 66,220) 30399, 15431,  1453,   523,    0,    6,   56,    0,  272, 3735, 3756,  6,  6,  5,  7,  9, 21, 19, }, // Occitan/Latin/Spain
    { LOCALE_TAGS(215, 91,110) 30524, 15526,  1453,   526,    0,    6,    0,    0, 3775, 3781, 3795,  6,  6,  5,  7,  6, 14, 14, }, // Odia/Odia/India
    { LOCALE_TAGS(220, 66, 77) 30941, 15681,  1455,   529,    0,    6,    0,    0, 3809, 3820, 3844,  6,  6,  5,  7, 11, 24, 18, }, // Oromo/Latin/Ethiopia
    { LOCALE_TAGS(220, 66,124) 30983, 15828,  1462,   532,    0,    6,    0,    0, 3809, 3820, 3844,  6,  6,  5,  7, 11, 24, 18, }, // Oromo/Latin/Kenya
    { LOCALE_TAGS(221,101,248) 30983, 15828,  1462,   532,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Osage/Osage/United States
    { LOCALE_TAGS(222, 27, 90) 31025, 15828,  1462,   533,    0,    6,    0,    0, 3862,   83,   90,  6,  6,  5,  7,  9,  7,  7, }, // Ossetic/Cyrillic/Georgia
    { LOCALE_TAGS(222, 27,193) 31025, 15828,  1462,   533,    0,    6,    0,    0, 3862,   83,   90,  6,  6,  5,  7,  9,  7,  7, }, // Ossetic/Cyrillic/Russia
    { LOCALE_TAGS(226, 66, 62) 31070, 15834,  1462,   534,    0,    6,    0,    0, 3871,   83,   90,  6,  6,  5,  7,  9,  7,  7, }, // Papiamento/Latin/Curacao
    { LOCALE_TAGS(226, 66, 13) 31120, 15841,  1462,   535,    0,    6,    0,    0, 3871,   83,   90,  6,  6,  5,  7,  9,  7,  7, }, // Papiamento/Latin/Aruba
    { LOCALE_TAGS(227,  4,  1) 31120, 15841,  1462,   535,    0,    6,    0,    0, 3880, 3891, 3904,  6,  6,  5,  7, 11, 13, 14, }, // Pashto/Arabic/Afghanistan
    { LOCALE_TAGS(227,  4,178) 31537, 15985,  1462,   538,    0,    6,    0,    0, 3880, 3891, 3904,  6,  6,  5,  7, 11, 13, 14, }, // Pashto/Arabic/Pakistan
    { LOCALE_TAGS(228,  4,112) 31539, 16010,  1462,   538,   73,   87,  190,    0, 3918, 3924, 3935,  7,  7, 10,  7,  6, 11, 15, }, // Persian/Arabic/Iran
    { LOCALE_TAGS(228,  4,  1) 31956, 16163,  1462,   541,   73,   87,  190,    0, 3918, 3924, 3935,  7,  7, 10,  7,  6, 11, 15, }, // Persian/Arabic/Afghanistan
    { LOCALE_TAGS(230, 66,187) 31956, 16163,  1462,   541,    0,    6,    0,    0, 3950, 3958, 3979,  6,  6,  5,  7,  8, 21, 15, }, // Polish/Latin/Poland
    { LOCALE_TAGS(231, 66, 32) 32159, 16314,  1465,   544,    0,    6,    0,    0, 3994, 4004, 4022,  6,  6,  5,  7, 10, 18, 20, }, // Portuguese/Latin/Brazil
    { LOCALE_TAGS(231, 66,  7) 32321, 16470,  1468,   547,    0,    6,    0,    0,  271, 4042, 4042,  6,  6,  5,  7, 10, 17, 17, }, // Portuguese/Latin/Angola
    { LOCALE_TAGS(231, 66, 43) 32389, 16626,  1475,   550,    0,    6,    0,    0,  271, 4042, 4042,  6,  6,  5,  7, 10, 17, 17, }, // Portuguese/Latin/Cape Verde
    { LOCALE_TAGS(231, 66, 73) 32457, 16782,  1482,   553,    0,    6,    0,    0,  271, 4042, 4042,  6,  6,  5,  7, 10, 17, 17, }, // Portuguese/Latin/Equatorial Guinea
    { LOCALE_TAGS(231, 66,101) 32525, 16938,  1489,   556,    0,    6,    0,    0,  271, 4042, 4042,  6,  6,  5,  7, 10, 17, 17, }, // Portuguese/Latin/Guinea-Bissau
    { LOCALE_TAGS(231, 66,138) 32593, 17094,  1496,   559,    0,    6,    0,    0,  271, 4042, 4042,  6,  6,  5,  7, 10, 17, 17, }, // Portuguese/Latin/Luxembourg
    { LOCALE_TAGS(231, 66,139) 32661, 17250,  1503,   562,    0,    6,    0,    0,  271, 4042, 4042,  6,  6,  5,  7, 10, 17, 17, }, // Portuguese/Latin/Macao
    { LOCALE_TAGS(231, 66,160) 32729, 17406,  1510,   565,    0,    6,    0,    0,  271, 4042, 4042,  6,  6,  5,  7, 10, 17, 17, }, // Portuguese/Latin/Mozambique
    { LOCALE_TAGS(231, 66,188) 32797, 17562,  1517,   568,    0,    6,    0,    0,  271, 4042, 4042,  6,  6,  5,  7, 10, 17, 17, }, // Portuguese/Latin/Portugal
    { LOCALE_TAGS(231, 66,204) 32865, 17718,  1524,   571,    0,    6,    0,    0,  271, 4042, 4042,  6,  6,  5,  7, 10, 17, 17, }, // Portuguese/Latin/Sao Tome and Principe
    { LOCALE_TAGS(231, 66,226) 32933, 17874,  1531,   574,    0,    6,    0,    0,  271, 4042, 4042,  6,  6,  5,  7, 10, 17, 17, }, // Portuguese/Latin/Switzerland
    { LOCALE_TAGS(231, 66,232) 33001, 18030,  1538,   577,    0,    6,    0,    0,  271, 4042, 4042,  6,  6,  5,  7, 10, 17, 17, }, // Portuguese/Latin/Timor-Leste
    { LOCALE_TAGS(232, 66,187) 33069, 18186,  1545,   580,    0,    6,    0,    0, 4059, 4068, 4084,  6,  6,  5,  7,  9, 16, 16, }, // Prussian/Latin/Poland
    { LOCALE_TAGS(233, 41,110) 33111, 18195,  1545,   581,    0,    6,    0,    0, 4100, 4107, 4120,  6,  6,  5,  7,  7, 13, 15, }, // Punjabi/Gurmukhi/India
    { LOCALE_TAGS(233,  4,178) 33528, 18348,  1546,   584,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Punjabi/Arabic/Pakistan
    { LOCALE_TAGS(234, 66,184) 33570, 18348,  1546,   585,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Quechua/Latin/Peru
    { LOCALE_TAGS(234, 66, 28) 33686, 18492,  1547,   588,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Quechua/Latin/Bolivia
    { LOCALE_TAGS(234, 66, 70) 33686, 18494,  1549,   588,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Quechua/Latin/Ecuador
    { LOCALE_TAGS(235, 66,192) 33686, 18496,  1551,   588,    0,    6,    0,    0, 4135, 4145, 4164,  6,  6,  5,  7, 10, 19, 18, }, // Romanian/Latin/Romania
    { LOCALE_TAGS(235, 66,154) 33799, 18647,  1555,   591,    0,    6,    0,    0, 4135, 4145, 4164,  6,  6,  5,  7, 10, 19, 18, }, // Romanian/Latin/Moldova
    { LOCALE_TAGS(236, 66,226) 33799, 18647,  1555,   591,    0,    6,    0,    0, 4182, 4190, 4205,  6,  6,  5,  7,  8, 15, 16, }, // Romansh/Latin/Switzerland
    { LOCALE_TAGS(237, 66,230) 33939, 18656,  1555,   593,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Rombo/Latin/Tanzania
    { LOCALE_TAGS(238, 66, 38) 33981, 18656,  1555,   594,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Rundi/Latin/Burundi
    { LOCALE_TAGS(239, 27,193) 34023, 18656,  1555,   595,    0,    6,    0,    0,    0, 4221, 4242,  6,  6,  5,  7,  2, 21, 16, }, // Russian/Cyrillic/Russia
    { LOCALE_TAGS(239, 27, 22) 34440, 18813,  1555,   598,    0,    6,    0,    0,    0, 4221, 4242,  6,  6,  5,  7,  2, 21, 16, }, // Russian/Cyrillic/Belarus
    { LOCALE_TAGS(239, 27,123) 34440, 18813,  1555,   598,    0,    6,    0,    0,    0, 4221, 4242,  6,  6,  5,  7,  2, 21, 16, }, // Russian/Cyrillic/Kazakhstan
    { LOCALE_TAGS(239, 27,128) 34440, 18813,  1555,   598,    0,    6,    0,    0,    0, 4221, 4242,  6,  6,  5,  7,  2, 21, 16, }, // Russian/Cyrillic/Kyrgyzstan
    { LOCALE_TAGS(239, 27,154) 34440, 18813,  1555,   598,    0,    6,    0,    0,    0, 4221, 4242,  6,  6,  5,  7,  2, 21, 16, }, // Russian/Cyrillic/Moldova
    { LOCALE_TAGS(239, 27,244) 34440, 18813,  1555,   598,    0,    6,    0,    0,    0, 4221, 4242,  6,  6,  5,  7,  2, 21, 16, }, // Russian/Cyrillic/Ukraine
    { LOCALE_TAGS(240, 66,230) 34440, 18813,  1555,   598,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Rwa/Latin/Tanzania
    { LOCALE_TAGS(241, 66, 74) 34482, 18813,  1555,   599,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Saho/Latin/Eritrea
    { LOCALE_TAGS(242, 27,193) 34524, 18813,  1555,   600,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Sakha/Cyrillic/Russia
    { LOCALE_TAGS(243, 66,124) 34599, 18841,  1555,   601,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Samburu/Latin/Kenya
    { LOCALE_TAGS(245, 66, 46) 34641, 18841,  1555,   602,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Sango/Latin/Central African Republic
    { LOCALE_TAGS(246, 66,230) 34683, 18841,  1555,   603,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Sangu/Latin/Tanzania
    { LOCALE_TAGS(247, 29,110) 34725, 18841,  1555,   604,    0,    6,  200,    0, 4258, 4265, 4278,  6,  6, 12,  7,  7, 13, 11, }, // Sanskrit/Devanagari/India
    { LOCALE_TAGS(248, 93,110) 34767, 18850,  1555,   605,    0,    6,  212,    0, 4289, 4296, 4309,  6,  6,  8,  7,  7, 13, 15, }, // Santali/Ol Chiki/India
    { LOCALE_TAGS(248, 29,110) 34888, 18871,  1555,   606,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Santali/Devanagari/India
    { LOCALE_TAGS(249, 66,117) 34913, 18871,  1555,   607,    0,    6,    0,    0, 2382, 4324, 2404,  6,  6,  5,  7,  6, 17, 14, }, // Sardinian/Latin/Italy
    { LOCALE_TAGS(251, 66,160) 35024, 19028,  1562,   611,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Sena/Latin/Mozambique
    { LOCALE_TAGS(252, 27,207) 35066, 19028,  1562,   612,    0,    6,    0,    0,    0, 4341, 4361,  6,  6,  5,  7,  2, 20, 15, }, // Serbian/Cyrillic/Serbia
    { LOCALE_TAGS(252, 27, 29) 35483, 19184,  1566,   615,    0,    6,    0,    0,    0, 4376, 4398,  6,  6,  5,  7,  2, 22, 17, }, // Serbian/Cyrillic/Bosnia and Herzegovina
    { LOCALE_TAGS(252, 27,126) 35506, 19326,  1570,   618,    0,    6,    0,    0,    0, 4341, 4361,  6,  6,  5,  7,  2, 20, 15, }, // Serbian/Cyrillic/Kosovo
    { LOCALE_TAGS(252, 27,157) 35506, 19326,  1570,   618,    0,    6,    0,    0,    0, 4341, 4361,  6,  6,  5,  7,  2, 20, 15, }, // Serbian/Cyrillic/Montenegro
    { LOCALE_TAGS(252, 66, 29) 35506, 19326,  1570,   618,    0,    6,    0,    0,    0,  624, 4415,  6,  6,  5,  7,  2, 22, 19, }, // Serbian/Latin/Bosnia and Herzegovina
    { LOCALE_TAGS(252, 66,126) 35923, 19482,  1574,   621,    0,    6,    0,    0,    0, 4434, 4454,  6,  6,  5,  7,  2, 20, 16, }, // Serbian/Latin/Kosovo
    { LOCALE_TAGS(252, 66,157) 36340, 19638,  1578,   624,    0,    6,    0,    0,    0, 4434, 4454,  6,  6,  5,  7,  2, 20, 16, }, // Serbian/Latin/Montenegro
    { LOCALE_TAGS(252, 66,207) 36757, 19794,  1582,   627,    0,    6,    0,    0,    0, 4434, 4454,  6,  6,  5,  7,  2, 20, 16, }, // Serbian/Latin/Serbia
    { LOCALE_TAGS(253, 66,230) 37174, 19950,  1586,   630,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Shambala/Latin/Tanzania
    { LOCALE_TAGS(254, 66,261) 37216, 19950,  1586,   631,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Shona/Latin/Zimbabwe
    { LOCALE_TAGS(255,141, 50) 37258, 19950,  1586,   632,    0,    6,  220,   12, 4470, 4474, 4480,  6,  6,  8,  6,  4,  6,  6, }, // Sichuan Yi/Yi/China
    { LOCALE_TAGS(256, 66,117) 37300, 19951,  1586,   633,    0,    6,    0,    0,    0, 4486, 4501,  6,  6,  5,  7,  2, 15, 15, }, // Sicilian/Latin/Italy
    { LOCALE_TAGS(257, 66, 77) 37424, 20096,  1594,   636,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Sidamo/Latin/Ethiopia
    { LOCALE_TAGS(258, 66,187) 37466, 20096,  1594,   637,    0,    6,    0,    0, 3950, 4516, 4538,  6,  6,  5,  7,  8, 22, 16, }, // Silesian/Latin/Poland
    { LOCALE_TAGS(259,  4,178) 37677, 20218,  1597,   640,    0,    6,    0,    0, 4554, 4560, 4573,  6,  6,  5,  7,  6, 13, 15, }, // Sindhi/Arabic/Pakistan
    { LOCALE_TAGS(259, 29,110) 38094, 20362,  1597,   643,    0,    6,  149,    0, 4588, 4597, 4613,  6,  6,  8,  7,  9, 16, 16, }, // Sindhi/Devanagari/India
    { LOCALE_TAGS(260,119,221) 38136, 20371,  1597,   644,   45,   51,  228,    0, 4629, 4637, 4651,  6,  6, 10,  7,  8, 14, 17, }, // Sinhala/Sinhala/Sri Lanka
    { LOCALE_TAGS(261, 66, 83) 38553, 20516,  1597,   647,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Skolt Sami/Latin/Finland
    { LOCALE_TAGS(262, 66,212) 38595, 20516,  1597,   648,    0,    6,    0,    0, 1256,   83,   90,  6,  6,  5,  7, 15,  7,  7, }, // Slovak/Latin/Slovakia
    { LOCALE_TAGS(263, 66,213) 38768, 20673,  1598,   652,   45,   51,   21,    0, 4668, 4674, 4691,  6,  6,  6,  7,  6, 17, 14, }, // Slovenian/Latin/Slovenia
    { LOCALE_TAGS(264, 66,243) 38919, 20820,  1598,   655,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Soga/Latin/Uganda
    { LOCALE_TAGS(265, 66,215) 38961, 20820,  1598,   656,    0,    6,    0,    0, 4705, 4715, 4739,  6,  6,  5,  7, 10, 24, 23, }, // Somali/Latin/Somalia
    { LOCALE_TAGS(265, 66, 67) 39322, 20972,  1599,   659,    0,    6,    0,    0, 4705, 4715, 4739,  6,  6,  5,  7, 10, 24, 23, }, // Somali/Latin/Djibouti
    { LOCALE_TAGS(265, 66, 77) 39322, 20972,  1599,   659,    0,    6,    0,    0, 4705, 4715, 4739,  6,  6,  5,  7, 10, 24, 23, }, // Somali/Latin/Ethiopia
    { LOCALE_TAGS(265, 66,124) 39322, 20972,  1599,   659,    0,    6,    0,    0, 4705, 4715, 4739,  6,  6,  5,  7, 10, 24, 23, }, // Somali/Latin/Kenya
    { LOCALE_TAGS(266,  4,112) 39322, 20972,  1599,   659,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Southern Kurdish/Arabic/Iran
    { LOCALE_TAGS(266,  4,113) 39364, 20972,  1599,   660,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Southern Kurdish/Arabic/Iraq
    { LOCALE_TAGS(267, 66,225) 39364, 20972,  1599,   660,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Southern Sami/Latin/Sweden
    { LOCALE_TAGS(267, 66,175) 39406, 20972,  1599,   661,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Southern Sami/Latin/Norway
    { LOCALE_TAGS(268, 66,216) 39406, 20972,  1599,   661,    0,    6,    0,    0, 4762, 4769, 4787,  6,  6,  5,  7,  7, 18, 21, }, // Southern Sotho/Latin/South Africa
    { LOCALE_TAGS(268, 66,133) 39448, 20973,  1599,   662,    0,    6,    0,    0, 4762, 4769, 4787,  6,  6,  5,  7,  7, 18, 21, }, // Southern Sotho/Latin/Lesotho
    { LOCALE_TAGS(269, 66,216) 39448, 20973,  1599,   662,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // South Ndebele/Latin/South Africa
    { LOCALE_TAGS(270, 66,220) 39490, 20973,  1599,   663,    0,    6,    0,    0, 2285, 4808, 4830,  6,  6,  5,  7, 10, 22, 23, }, // Spanish/Latin/Spain
    { LOCALE_TAGS(270, 66, 11) 39690, 21128,  1603,   666,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Argentina
    { LOCALE_TAGS(270, 66, 24) 39701, 21149,  1609,   668,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Belize
    { LOCALE_TAGS(270, 66, 28) 39712, 21168,  1613,   670,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Bolivia
    { LOCALE_TAGS(270, 66, 32) 39723, 21188,  1618,   672,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Brazil
    { LOCALE_TAGS(270, 66, 42) 39734, 21207,  1622,   674,    0,    6,    0,    0, 2285, 4808, 4830,  6,  6,  5,  7, 10, 22, 23, }, // Spanish/Latin/Canary Islands
    { LOCALE_TAGS(270, 66, 47) 39734, 21207,  1622,   674,    0,    6,    0,    0, 2285, 4808, 4830,  6,  6,  5,  7, 10, 22, 23, }, // Spanish/Latin/Ceuta and Melilla
    { LOCALE_TAGS(270, 66, 49) 39734, 21207,  1622,   674,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Chile
    { LOCALE_TAGS(270, 66, 54) 39745, 21227,  1627,   676,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Colombia
    { LOCALE_TAGS(270, 66, 59) 39756, 21247,  1632,   678,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Costa Rica
    { LOCALE_TAGS(270, 66, 61) 39767, 21266,  1636,   680,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Cuba
    { LOCALE_TAGS(270, 66, 69) 39778, 21285,  1640,   682,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Dominican Republic
    { LOCALE_TAGS(270, 66, 70) 39789, 21304,  1644,   684,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Ecuador
    { LOCALE_TAGS(270, 66, 72) 39800, 21324,  1649,   686,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/El Salvador
    { LOCALE_TAGS(270, 66, 73) 39811, 21343,  1653,   688,    0,    6,    0,    0, 2285, 4808, 4830,  6,  6,  5,  7, 10, 22, 23, }, // Spanish/Latin/Equatorial Guinea
    { LOCALE_TAGS(270, 66, 99) 39811, 21343,  1653,   688,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Guatemala
    { LOCALE_TAGS(270, 66,106) 39822, 21362,  1657,   690,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Honduras
    { LOCALE_TAGS(270, 66,130) 39833, 21381,  1661,   692,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Latin America
    { LOCALE_TAGS(270, 66,152) 39844, 21400,  1665,   694,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Mexico
    { LOCALE_TAGS(270, 66,168) 39860, 21415,  1669,   696,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Nicaragua
    { LOCALE_TAGS(270, 66,181) 39871, 21434,  1673,   698,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Panama
    { LOCALE_TAGS(270, 66,183) 39882, 21453,  1677,   700,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Paraguay
    { LOCALE_TAGS(270, 66,184) 39893, 21472,  1681,   702,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Peru
    { LOCALE_TAGS(270, 66,185) 39904, 21492,  1686,   704,    0,    6,    0,    0, 2285, 4808, 4830,  6,  6,  5,  7, 10, 22, 23, }, // Spanish/Latin/Philippines
    { LOCALE_TAGS(270, 66,189) 39904, 21492,  1686,   704,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Puerto Rico
    { LOCALE_TAGS(270, 66,248) 39915, 21511,  1690,   706,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/United States
    { LOCALE_TAGS(270, 66,250) 39931, 21530,  1701,   709,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Uruguay
    { LOCALE_TAGS(270, 66,254) 39942, 21550,  1706,   711,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Venezuela
    { LOCALE_TAGS(271,135,159) 39953, 21570,  1711,   713,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Standard Moroccan Tamazight/Tifinagh/Morocco
    { LOCALE_TAGS(272, 66,111) 39995, 21570,  1711,   714,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Sundanese/Latin/Indonesia
    { LOCALE_TAGS(273, 66,230) 40037, 21581,  1711,   715,    0,    6,   21,    0, 4892, 4901, 4921,  6,  6,  6,  7,  9, 20, 19, }, // Swahili/Latin/Tanzania
    { LOCALE_TAGS(273, 66, 57) 40084, 21728,  1711,   718,    0,    6,   21,    0, 4892, 4901, 4921,  6,  6,  6,  7,  9, 20, 19, }, // Swahili/Latin/Congo - Kinshasa
    { LOCALE_TAGS(273, 66,124) 40084, 21728,  1711,   718,    0,    6,   21,    0, 4892, 4901, 4921,  6,  6,  6,  7,  9, 20, 19, }, // Swahili/Latin/Kenya
    { LOCALE_TAGS(273, 66,243) 40104, 21793,  1711,   719,    0,    6,   21,    0, 4892, 4901, 4921,  6,  6,  6,  7,  9, 20, 19, }, // Swahili/Latin/Uganda
    { LOCALE_TAGS(274, 66,216) 40104, 21793,  1711,   719,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Swati/Latin/South Africa
    { LOCALE_TAGS(274, 66, 76) 40146, 21793,  1711,   720,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Swati/Latin/Eswatini
    { LOCALE_TAGS(275, 66,225) 40146, 21793,  1711,   720,    0,   57,    0,    0, 4940, 4945, 4959,  6,  6,  5,  7,  5, 14, 14, }, // Swedish/Latin/Sweden
    { LOCALE_TAGS(275, 66,  2) 40272, 21950,  1715,   724,    0,   57,    0,    0, 4940, 4945, 4959,  6,  6,  5,  7,  5, 14, 14, }, // Swedish/Latin/Aland Islands
    { LOCALE_TAGS(275, 66, 83) 40272, 21950,  1715,   724,    0,   57,    0,    0, 4940, 4945, 4959,  6,  6,  5,  7,  5, 14, 14, }, // Swedish/Latin/Finland
    { LOCALE_TAGS(276, 66,226) 40272, 21950,  1715,   724,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Swiss German/Latin/Switzerland
    { LOCALE_TAGS(276, 66, 84) 40381, 21964,  1718,   725,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Swiss German/Latin/France
    { LOCALE_TAGS(276, 66,136) 40381, 21964,  1718,   725,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Swiss German/Latin/Liechtenstein
    { LOCALE_TAGS(277,123,113) 40381, 21964,  1718,   725,    0,    6,    0,    0, 4973, 4980, 4997,  6,  6,  5,  7,  7, 17, 17, }, // Syriac/Syriac/Iraq
    { LOCALE_TAGS(277,123,227) 40797, 22120,  1718,   728,    0,    6,    0,    0, 4973, 4980, 4997,  6,  6,  5,  7,  7, 17, 17, }, // Syriac/Syriac/Syria
    { LOCALE_TAGS(278,135,159) 40797, 22120,  1718,   728,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tachelhit/Tifinagh/Morocco
    { LOCALE_TAGS(278, 66,159) 40839, 22120,  1718,   729,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tachelhit/Latin/Morocco
    { LOCALE_TAGS(280,127,255) 40839, 22120,  1718,   729,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tai Dam/Tai Viet/Vietnam
    { LOCALE_TAGS(281, 66,124) 40881, 22120,  1718,   730,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Taita/Latin/Kenya
    { LOCALE_TAGS(282, 27,229) 40923, 22120,  1718,   731,    0,    6,    0,    0, 5014, 5022, 5041,  6,  6,  5,  7,  8, 19, 16, }, // Tajik/Cyrillic/Tajikistan
    { LOCALE_TAGS(283,129,110) 41339, 22264,  1718,   734,    0,    6,    0,    0, 5057, 5065, 5081,  6,  6,  5,  7,  8, 16, 15, }, // Tamil/Tamil/India
    { LOCALE_TAGS(283,129,143) 41756, 22420,  1719,   737,    0,    6,    0,    0, 5057, 5065, 5081,  6,  6,  5,  7,  8, 16, 15, }, // Tamil/Tamil/Malaysia
    { LOCALE_TAGS(283,129,210) 41756, 22423,  1722,   737,    0,    6,    0,    0, 5057, 5065, 5081,  6,  6,  5,  7,  8, 16, 15, }, // Tamil/Tamil/Singapore
    { LOCALE_TAGS(283,129,221) 41756, 22426,  1725,   737,    0,    6,    0,    0, 5057, 5065, 5081,  6,  6,  5,  7,  8, 16, 15, }, // Tamil/Tamil/Sri Lanka
    { LOCALE_TAGS(284, 66,228) 41756, 22426,  1725,   737,    0,    6,  238,    0, 5096,   83,   90,  6,  6,  5,  7,  8,  7,  7, }, // Taroko/Latin/Taiwan
    { LOCALE_TAGS(285, 66,170) 41809, 22435,  1734,   738,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tasawaq/Latin/Niger
    { LOCALE_TAGS(286, 27,193) 41851, 22435,  1734,   739,    0,    6,    0,    0, 5104, 5113, 5129,  6,  6,  5,  7,  9, 16, 15, }, // Tatar/Cyrillic/Russia
    { LOCALE_TAGS(287,131,110) 42266, 22583,  1734,   742,    0,    6,    0,    0, 5144, 5151, 5168,  6,  6,  5,  7,  7, 17, 21, }, // Telugu/Telugu/India
    { LOCALE_TAGS(288, 66,243) 42683, 22739,  1735,   745,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Teso/Latin/Uganda
    { LOCALE_TAGS(288, 66,124) 42725, 22739,  1735,   746,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Teso/Latin/Kenya
    { LOCALE_TAGS(289,133,231) 42725, 22739,  1735,   746,    0,    6,    0,    0, 5189, 5195, 5208,  6,  6,  5,  7,  6, 13, 12, }, // Thai/Thai/Thailand
    { LOCALE_TAGS(290,134, 50) 43142, 22896,  1735,   749,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tibetan/Tibetan/China
    { LOCALE_TAGS(290,134,110) 43184, 22896,  1735,   750,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tibetan/Tibetan/India
    { LOCALE_TAGS(291, 33, 74) 43184, 22896,  1735,   750,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tigre/Ethiopic/Eritrea
    { LOCALE_TAGS(292, 33, 77) 43226, 22896,  1735,   751,    0,    6,    0,    0, 5220, 5225, 5241,  6,  6,  5,  7,  5, 16, 13, }, // Tigrinya/Ethiopic/Ethiopia
    { LOCALE_TAGS(292, 33, 74) 43643, 23041,  1735,   754,    0,    6,    0,    0, 5220, 5225, 5241,  6,  6,  5,  7,  5, 16, 13, }, // Tigrinya/Ethiopic/Eritrea
    { LOCALE_TAGS(294, 66,182) 43643, 23041,  1735,   754,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tok Pisin/Latin/Papua New Guinea
    { LOCALE_TAGS(295, 66,235) 43685, 23041,  1735,   755,    0,    6,    0,    0, 5254, 5262, 5277,  6,  6,  5,  7,  8, 15, 14, }, // Tongan/Latin/Tonga
    { LOCALE_TAGS(296, 66,216) 43768, 23198,  1735,   759,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tsonga/Latin/South Africa
    { LOCALE_TAGS(297, 66,216) 43810, 23198,  1735,   760,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tswana/Latin/South Africa
    { LOCALE_TAGS(297, 66, 30) 43852, 23199,  1735,   761,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tswana/Latin/Botswana
    { LOCALE_TAGS(298, 66,239) 43852, 23199,  1735,   761,    0,    6,    0,    0, 5291, 5299, 5316,  6,  6,  5,  7,  8, 17, 12, }, // Turkish/Latin/Turkey
    { LOCALE_TAGS(298, 66, 63) 44000, 23356,  1735,   764,    0,    6,    0,    0, 5291, 5299, 5316,  6,  6,  5,  7,  8, 17, 12, }, // Turkish/Latin/Cyprus
    { LOCALE_TAGS(299, 66,240) 44000, 23356,  1735,   764,    0,    6,    0,    0, 5328, 5336, 5353,  6,  6,  5,  7,  8, 17, 16, }, // Turkmen/Latin/Turkmenistan
    { LOCALE_TAGS(301, 66,169) 44314, 23503,  1736,   767,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tyap/Latin/Nigeria
    { LOCALE_TAGS(303, 27,244) 44356, 23503,  1736,   768,    0,    6,    0,    0,  538, 5369, 5389,  6,  6,  5,  7,  7, 20, 15, }, // Ukrainian/Cyrillic/Ukraine
    { LOCALE_TAGS(304, 66, 91) 44773, 23654,  1736,   771,    0,    6,    0,    0, 5404, 5419, 5432,  6,  6,  5,  7, 15, 13, 12, }, // Upper Sorbian/Latin/Germany
    { LOCALE_TAGS(305,  4,178) 44893, 23798,  1739,   774,    0,    6,   21,    0, 4554, 5444, 5457,  6,  6,  6,  7,  6, 13, 15, }, // Urdu/Arabic/Pakistan
    { LOCALE_TAGS(305,  4,110) 45310, 23945,  1739,   777,    0,    6,    0,    0, 4554, 5444, 5472,  6,  6,  5,  7,  6, 13, 12, }, // Urdu/Arabic/India
    { LOCALE_TAGS(306,  4, 50) 45318, 23988,  1739,   777,    0,    6,    0,    0, 5484, 5492, 5510,  6,  6,  5,  7,  8, 18, 15, }, // Uyghur/Arabic/China
    { LOCALE_TAGS(307, 66,251) 45360, 24140,  1739,   780,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Uzbek/Latin/Uzbekistan
    { LOCALE_TAGS(307,  4,  1) 45649, 24284,  1739,   783,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Uzbek/Arabic/Afghanistan
    { LOCALE_TAGS(307, 27,251) 45690, 24285,  1739,   784,    0,    6,    0,    0, 5525, 5533, 5550,  6,  6,  5,  7,  8, 17, 17, }, // Uzbek/Cyrillic/Uzbekistan
    { LOCALE_TAGS(308,139,134) 45730, 24423,  1739,   787,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Vai/Vai/Liberia
    { LOCALE_TAGS(308, 66,134) 45772, 24423,  1739,   788,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Vai/Latin/Liberia
    { LOCALE_TAGS(309, 66,216) 45772, 24423,  1739,   788,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Venda/Latin/South Africa
    { LOCALE_TAGS(310, 66,255) 45814, 24423,  1739,   789,    0,    6,    0,    0, 5567, 5573, 5585,  6,  6,  5,  7,  6, 12, 13, }, // Vietnamese/Latin/Vietnam
    { LOCALE_TAGS(311, 66,258) 45869, 24580,  1746,   793,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Volapuk/Latin/world
    { LOCALE_TAGS(312, 66,230) 45911, 24580,  1746,   794,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Vunjo/Latin/Tanzania
    { LOCALE_TAGS(313, 66, 23) 45953, 24580,  1746,   795,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Walloon/Latin/Belgium
    { LOCALE_TAGS(314, 66,226) 45995, 24580,  1746,   796,    0,    6,    0,    0, 5598,   83,   90,  6,  6,  5,  7,  6,  7,  7, }, // Walser/Latin/Switzerland
    { LOCALE_TAGS(315, 66, 15) 46076, 24584,  1749,   797,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Warlpiri/Latin/Australia
    { LOCALE_TAGS(316, 66,246) 46118, 24584,  1749,   798,    0,    6,    0,    0, 5604, 5612, 5628,  6,  6,  5,  7,  8, 16, 12, }, // Welsh/Latin/United Kingdom
    { LOCALE_TAGS(317,  4,178) 46193, 24729,  1753,   801,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Western Balochi/Arabic/Pakistan
    { LOCALE_TAGS(317,  4,  1) 46274, 24734,  1753,   802,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Western Balochi/Arabic/Afghanistan
    { LOCALE_TAGS(317,  4,112) 46274, 24734,  1753,   802,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Western Balochi/Arabic/Iran
    { LOCALE_TAGS(317,  4,176) 46274, 24734,  1753,   802,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Western Balochi/Arabic/Oman
    { LOCALE_TAGS(317,  4,245) 46274, 24734,  1753,   802,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Western Balochi/Arabic/United Arab Emirates
    { LOCALE_TAGS(318, 66,165) 46274, 24734,  1753,   802,    0,    6,    0,    0, 5640, 5647, 5663,  6,  6,  5,  7,  7, 16, 12, }, // Western Frisian/Latin/Netherlands
    { LOCALE_TAGS(319, 33, 77) 46394, 24890,  1756,   805,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Wolaytta/Ethiopic/Ethiopia
    { LOCALE_TAGS(320, 66,206) 46436, 24890,  1756,   806,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Wolof/Latin/Senegal
    { LOCALE_TAGS(321, 66,216) 46478, 25034,  1756,   809,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // Xhosa/Latin/South Africa
    { LOCALE_TAGS(322, 66, 40) 46521, 25178,  1756,   812,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Yangben/Latin/Cameroon
    { LOCALE_TAGS(323, 47,244) 46563, 25178,  1756,   813,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Yiddish/Hebrew/Ukraine
    { LOCALE_TAGS(324, 66,169) 46608, 25178,  1756,   814,    0,    6,  243,    0, 5675, 5682, 5696,  6,  6,  5,  7,  7, 14, 16, }, // Yoruba/Latin/Nigeria
    { LOCALE_TAGS(324, 66, 25) 46728, 25322,  1756,   817,    0,    6,  243,    0, 5675, 5682, 5712,  6,  6,  5,  7,  7, 14, 16, }, // Yoruba/Latin/Benin
    { LOCALE_TAGS(325, 66,170) 46749, 25410,  1756,   820,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Zarma/Latin/Niger
    { LOCALE_TAGS(326, 66, 50) 46791, 25410,  1756,   821,    0,    6,    0,    0, 5728, 5738, 5758,  6,  6,  5,  7, 10, 20, 17, }, // Zhuang/Latin/China
    { LOCALE_TAGS(327, 66,216) 46833, 25411,  1756,   822,    0,    6,    0,    0, 5775, 5792, 5814,  6,  6,  5,  7, 17, 22, 21, }, // Zulu/Latin/South Africa
    { LOCALE_TAGS(328, 66, 32) 47248, 25558,  1756,   825,    0,    6,    0,    0, 5835, 5845, 5854,  6,  6,  5,  7, 10,  9, 18, }, // Kaingang/Latin/Brazil
    { LOCALE_TAGS(329, 66, 32) 47659, 25712,  1759,   828,    0,    6,    0,    0, 5872, 5883, 5902,  6,  6,  5,  7, 11, 19, 23, }, // Nheengatu/Latin/Brazil
    { LOCALE_TAGS(329, 66, 54) 48052, 25866,  1762,   831,    0,    6,    0,    0, 5872, 5883, 5902,  6,  6,  5,  7, 11, 19, 23, }, // Nheengatu/Latin/Colombia
    { LOCALE_TAGS(329, 66,254) 48053, 25868,  1762,   831,    0,    6,    0,    0, 5872, 5883, 5902,  6,  6,  5,  7, 11, 19, 23, }, // Nheengatu/Latin/Venezuela
    { LOCALE_TAGS(330, 29,110) 48054, 25870,  1762,   831,    0,    6,    0,    0, 2149, 5925,   90,  6,  6,  5,  7,  6, 22,  7, }, // Haryanvi/Devanagari/India
    { LOCALE_TAGS(331, 66, 91) 48096, 25871,  1762,   832,    0,    6,    0,    0, 5947, 5954, 5970,  6,  6,  5,  7,  7, 16, 12, }, // Northern Frisian/Latin/Germany
    { LOCALE_TAGS(332, 29,110) 48220, 26015,  1762,   835,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Rajasthani/Devanagari/India
    { LOCALE_TAGS(333, 27,193) 48262, 26016,  1762,   836,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Moksha/Cyrillic/Russia
    { LOCALE_TAGS(334, 66,258) 48304, 26016,  1762,   837,    0,    6,  248,    0, 5982, 5982, 5993,  6,  6, 11,  7, 11, 11, 21, }, // Toki Pona/Latin/world
    { LOCALE_TAGS(335, 66,214) 48346, 26017,  1762,   838,    0,    6,   21,    0, 6014, 6021, 6036,  6,  6,  6,  7,  7, 15, 14, }, // Pijin/Latin/Solomon Islands
    { LOCALE_TAGS(336, 66,169) 48388, 26018,  1762,   839,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Obolo/Latin/Nigeria
    { LOCALE_TAGS(337,  4,178) 48430, 26018,  1762,   840,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Baluchi/Arabic/Pakistan
    { LOCALE_TAGS(337, 66,178) 48472, 26055,  1762,   841,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Baluchi/Latin/Pakistan
    { LOCALE_TAGS(338, 66,117) 48749, 26199,  1762,   844,    0,   57,   56,    0, 6050, 6056, 6071,  6,  6,  5,  7,  6, 15, 13, }, // Ligurian/Latin/Italy
    { LOCALE_TAGS(339,142,161) 48851, 26342,  1762,   847,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Rohingya/Hanifi/Myanmar
    { LOCALE_TAGS(339,142, 20) 48893, 26342,  1762,   848,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Rohingya/Hanifi/Bangladesh
    { LOCALE_TAGS(340,  4,178) 48893, 26342,  1762,   848,    0,    6,    0,    0, 6084, 6090, 6103,  6,  6,  5,  7,  6, 13,  7, }, // Torwali/Arabic/Pakistan
    { LOCALE_TAGS(341, 66, 25) 49309, 26485,  1762,   851,    0,    6,  259,    0, 6110, 6124, 6145,  6,  6,  5,  7, 14, 21, 23, }, // Anii/Latin/Benin
    { LOCALE_TAGS(342, 29,110) 49713, 26633,  1763,   854,    0,    6,  264,    0, 6168, 6174, 6185,  6,  6, 15,  7,  6, 11, 26, }, // Kangri/Devanagari/India
    { LOCALE_TAGS(343, 66,117) 50129, 26776,  1763,   857,    0,    6,   56,    0, 2382, 6211, 6225,  6,  6,  5,  7,  6, 14, 13, }, // Venetian/Latin/Italy
    { LOCALE_TAGS(344, 66,110) 50360, 26920,  1763,   860,    0,    6,    0,    0, 6238, 6245, 6260,  6,  6,  5,  7,  7, 15, 14, }, // Kuvi/Latin/India
    { LOCALE_TAGS(344, 29,110) 50776, 27063,  1763,   863,    0,    6,    0,    0, 6274, 6281, 6295,  6,  6,  5,  7,  7, 14, 15, }, // Kuvi/Devanagari/India
    { LOCALE_TAGS(344, 91,110) 51191, 27206,  1763,   864,    0,    6,    0,    0, 6310, 6317, 6332,  6,  6,  5,  7,  7, 15, 14, }, // Kuvi/Odia/India
    { LOCALE_TAGS(344,131,110) 51606, 27349,  1763,   867,    0,    6,    0,    0, 5144, 5151, 5168,  6,  6,  5,  7,  7, 17, 21, }, // Kuvi/Telugu/India
    { LOCALE_TAGS(345, 27,251) 52021, 27492,  1763,   870,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kara-Kalpak/Cyrillic/Uzbekistan
    { LOCALE_TAGS(345, 66,251) 52063, 27492,  1763,   871,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kara-Kalpak/Latin/Uzbekistan
    { LOCALE_TAGS(346, 18, 41) 52063, 27492,  1763,   871,    0,    6,    0,    0, 6346, 6355, 6372,  6,  6,  5,  7,  9, 17, 14, }, // Swampy Cree/Canadian Aboriginal/Canada
    { LOCALE_TAGS(  0,  0,  0) 52110, 27493,  1763,   872,   94,   94,  279,   49, 6386, 6386, 6386,  0,  0,  0,  0,  0,  0,  0, } // Terminal row
}; // Locale/zone data

static inline constexpr char16_t hourFormatTable[] = {
0x2b, 0x48, 0x48, 0x3a, 0x6d, 0x6d, 0x2d, 0x48, 0x48, 0x3a, 0x6d, 0x6d,
0x2b, 0x48, 0x48, 0x6d, 0x6d, 0x2d, 0x48, 0x48, 0x6d, 0x6d, 0x2013, 0x48,
0x48, 0x3a, 0x6d, 0x6d, 0x20, 0x2d, 0x48, 0x48, 0x3a, 0x6d, 0x6d, 0x2b,
0x48, 0x3a, 0x6d, 0x6d, 0x2d, 0x48, 0x3a, 0x6d, 0x6d, 0x2b, 0x48, 0x48,
0x2e, 0x6d, 0x6d, 0x2d, 0x48, 0x48, 0x2e, 0x6d, 0x6d, 0x2212, 0x48, 0x48,
0x3a, 0x6d, 0x6d, 0x2b, 0x48, 0x2e, 0x6d, 0x6d, 0x2d, 0x48, 0x2e, 0x6d,
0x6d, 0x200e, 0x2b, 0x48, 0x48, 0x3a, 0x6d, 0x6d, 0x2d, 0x48, 0x48, 0x3a,
0x6d, 0x6d, 0x200e, 0x200e, 0x2212, 0x48, 0x48, 0x3a, 0x6d, 0x6d
};

static inline constexpr char16_t gmtFormatTable[] = {
0x47, 0x4d, 0x54, 0x25, 0x30, 0x1302, 0x20, 0x12a4, 0x121d, 0x20, 0x1272, 0x25,
0x30, 0x63a, 0x631, 0x64a, 0x646, 0x62a, 0x634, 0x25, 0x30, 0x47, 0x4d, 0x54,
0x20, 0x25, 0x30, 0x91c, 0x93f, 0x2e, 0x90f, 0x92e, 0x2e, 0x924, 0x93f, 0x20,
0x25, 0x30, 0x413, 0x440, 0x438, 0x43d, 0x443, 0x438, 0x447, 0x25, 0x30, 0x6af,
0x631, 0x6cc, 0x646, 0x6cc, 0x686, 0x20, 0x25, 0x30, 0x55, 0x54, 0x43, 0x25,
0x30, 0xf47, 0xf72, 0xf0b, 0xf68, 0xf7a, 0xf58, 0xf0b, 0xf4f, 0xf72, 0xf0b, 0x25,
0x30, 0xd801, 0xdc18, 0xd801, 0xdc23, 0xd801, 0xdc13, 0x20, 0x25, 0x30, 0xb7, 0xd801,
0xdc5c, 0xd801, 0xdc65, 0xd801, 0xdc51, 0x25, 0x30, 0x25, 0x30, 0x20, 0x47, 0x4d,
0x54, 0xd83a, 0xdd11, 0xd83a, 0xdd16, 0xd83a, 0xdd18, 0x25, 0x30, 0x47, 0x4d, 0x54,
0x25, 0x30, 0x200e, 0x54, 0x4d, 0x47, 0x25, 0x30, 0x4d, 0x41, 0x47, 0x25,
0x30, 0x4b, 0x4c, 0x47, 0x20, 0x25, 0x30, 0x91c, 0x940, 0x20, 0x90f, 0x92e,
0x20, 0x91f, 0x940, 0x20, 0x25, 0x30, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f,
0x1780, 0x179b, 0x20, 0x25, 0x30, 0x91c, 0x940, 0x90f, 0x92e, 0x91f, 0x940, 0x25,
0x30, 0xd1c, 0xd3f, 0xd0e, 0xd02, 0xd1f, 0xd3f, 0x20, 0x25, 0x30, 0x99c, 0x9bf,
0x20, 0x98f, 0x9ae, 0x20, 0x99f, 0x9bf, 0x20, 0x25, 0x30, 0x5b, 0x47, 0x4d,
0x54, 0x5d, 0x25, 0x30, 0x7dc, 0x7ed, 0x7d5, 0x7d6, 0x25, 0x30, 0x25, 0x30,
0x20, 0x6af, 0x631, 0x6cc, 0x646, 0x648, 0x6cc, 0x686, 0x91c, 0x940, 0x2e, 0x90f,
0x92e, 0x2e, 0x91f, 0x940, 0x2e, 0x20, 0x25, 0x30, 0x1c61, 0x1c64, 0x1c6e, 0x1c62,
0x1c74, 0x1c64, 0x25, 0x30, 0xa2e7, 0xa0c5, 0xa395, 0xa3e6, 0xa12e, 0xa209, 0x25, 0x30,
0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd2, 0xdb8, 0xdc0, 0xdda, 0x25, 0x30, 0x4a, 0x51,
0x47, 0x25, 0x30, 0x57, 0x41, 0x54, 0x25, 0x30, 0x74, 0x65, 0x6e, 0x70,
0x6f, 0x20, 0x55, 0x54, 0x43, 0x25, 0x30, 0x25, 0x30, 0x20, 0x47, 0x6b,
0x917, 0x94d, 0x930, 0x940, 0x970, 0x20, 0x92e, 0x948, 0x970, 0x20, 0x91f, 0x948,
0x970, 0x25, 0x30
};

static inline constexpr char16_t regionFormatTable[] = {
0x25, 0x30, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x25, 0x30, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x25,
0x30, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x25, 0x30, 0x20, 0x410, 0x430, 0x43c, 0x4ad, 0x430, 0x25,
0x30, 0x20, 0x410, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9,
0x20, 0x410, 0x430, 0x43c, 0x4ad, 0x430, 0x25, 0x30, 0x20, 0x410, 0x525, 0x445,
0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x430, 0x43c, 0x4ad, 0x430, 0x25,
0x30, 0x20, 0x28, 0x2b, 0x30, 0x29, 0x25, 0x30, 0x20, 0x28, 0x2b, 0x31,
0x29, 0x25, 0x30, 0x2d, 0x74, 0x79, 0x64, 0x25, 0x30, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x25, 0x30,
0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x42, 0x65,
0x72, 0x25b, 0x20, 0x25, 0x30, 0x42, 0x65, 0x72, 0x25b, 0x20, 0x73, 0x75,
0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x25, 0x30, 0x41, 0x77, 0x69, 0x61,
0x20, 0x62, 0x65, 0x72, 0x20, 0x25, 0x30, 0x4f, 0x72, 0x61, 0x3a, 0x20,
0x25, 0x30, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x65, 0x3a, 0x20, 0x25, 0x30, 0x4f, 0x72, 0x61, 0x20, 0x76,
0x65, 0x72, 0x6f, 0x72, 0x65, 0x3a, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20,
0x1230, 0x12d3, 0x1275, 0x25, 0x30, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230,
0x12d3, 0x1275, 0x25, 0x30, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1265, 0x122d, 0x1203,
0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x25,
0x30, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x25, 0x30, 0x20, 0x627, 0x644,
0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x25, 0x30,
0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x25, 0x30, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x25, 0x30, 0x20, 0x28, 0x2b, 0x30, 0x29, 0x20, 0x9ae,
0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x25, 0x30, 0x20, 0x28, 0x2b,
0x31, 0x29, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64,
0x65, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x25, 0x30, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x25, 0x30, 0x20, 0x59, 0x61, 0x79, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x25, 0x30, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x25, 0x30, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be,
0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x25, 0x30, 0x20, 0x61,
0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x25,
0x30, 0x20, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61,
0x25, 0x30, 0x20, 0x28, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x29, 0x427, 0x430, 0x441, 0x3a, 0x20, 0x25, 0x30, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430,
0x441, 0x3a, 0x20, 0x25, 0x30, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447,
0x430, 0x441, 0x3a, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x25, 0x30, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x25, 0x30, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e,
0x92f, 0x25, 0x30, 0x20, 0x938, 0x92e, 0x25, 0x30, 0x20, 0x925, 0x93e, 0x916,
0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x25, 0x30, 0x20, 0x938, 0x93e,
0x928, 0x928, 0x93f, 0x20, 0x938, 0x94b, 0x930, 0x93e, 0x902, 0x20, 0x938, 0x92e,
0x25, 0x30, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x25, 0x30,
0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x3a, 0x20, 0x25,
0x30, 0x65, 0x75, 0x72, 0x20, 0x25, 0x30, 0x65, 0x75, 0x72, 0x20, 0x63,
0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x25, 0x30, 0x65, 0x75, 0x72,
0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x2013,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x25, 0x30, 0x20, 0x2013, 0x20, 0x43b, 0x44f,
0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x25, 0x30, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x25, 0x30, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x25, 0x30, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e,
0x102e, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x25, 0x30, 0x6642, 0x9593, 0x25, 0x30, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x25,
0x30, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x25, 0x30, 0x65f6, 0x95f4, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x3a, 0x20, 0x25, 0x30, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x2c, 0x20,
0x25, 0x30, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x2c, 0x20, 0x25, 0x30, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x25, 0x30, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x25, 0x30, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20,
0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x25, 0x30, 0x628, 0x6d5, 0x6a9, 0x627, 0x62a, 0x6cc, 0x20, 0x25, 0x30, 0x6a9,
0x627, 0x62a, 0x6cc, 0x20, 0x25, 0x30, 0x20, 0x641, 0x6d5, 0x631, 0x645, 0x6cc,
0x6a9, 0x627, 0x62a, 0x6cc, 0x20, 0x25, 0x30, 0x20, 0x647, 0x627, 0x648, 0x6cc,
0x646, 0x6d5, 0x25, 0x30, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0x25, 0x30, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd34, 0x25,
0x30, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1d, 0xd804, 0xdd2a, 0xd804, 0xdd0c,
0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd33, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804,
0xdd34, 0x25, 0x30, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x25, 0x30, 0x20,
0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x25,
0x30, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x25, 0x30, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x25, 0x30, 0x590f, 0x4ee4, 0x65f6,
0x95f4, 0x25, 0x30, 0x20, 0x28, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x29,
0x25, 0x30, 0x20, 0x28, 0x437, 0x438, 0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x29, 0x25, 0x30, 0x20, 0x28, 0x43b,
0x463, 0x301, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c,
0x467, 0x29, 0x25, 0x30, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x25, 0x30,
0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x5a, 0x69, 0x63, 0x6b, 0x20, 0x76, 0x75, 0x6e, 0x20,
0x25, 0x30, 0x53, 0x63, 0x68, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x74, 0x74,
0x7a, 0x69, 0x63, 0x6b, 0x20, 0x76, 0x75, 0x6e, 0x20, 0x25, 0x30, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x63, 0x6b, 0x20, 0x76, 0x75,
0x6e, 0x20, 0x25, 0x30, 0x6f, 0x72, 0x61, 0x20, 0x3a, 0x20, 0x25, 0x30,
0x25, 0x30, 0x20, 0x28, 0x6f, 0x72, 0x61, 0x20, 0x75, 0x73, 0x75, 0x61,
0x6c, 0x65, 0x29, 0x25, 0x30, 0x20, 0x28, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x61, 0x74, 0x65, 0x29, 0x10d, 0x61, 0x73, 0x6f,
0x76, 0xe9, 0x20, 0x70, 0xe1, 0x73, 0x6d, 0x6f, 0x20, 0x25, 0x30, 0x25,
0x30, 0x2d, 0x74, 0x69, 0x64, 0x25, 0x30, 0x2d, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x25, 0x30, 0x2d, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x25, 0x30, 0x20, 0x938, 0x92e, 0x93e, 0x902,
0x25, 0x30, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x93e, 0x902,
0x25, 0x30, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e,
0x93e, 0x902, 0x74, 0x69, 0x6a, 0x64, 0x20, 0x69, 0x6e, 0x20, 0x25, 0x30,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a,
0x64, 0x20, 0x69, 0x6e, 0x20, 0x25, 0x30, 0x7a, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x6a, 0x64, 0x20, 0x69, 0x6e, 0x20, 0x25, 0x30, 0x25, 0x30,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf0d, 0x25, 0x30, 0x20, 0xd801,
0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0x25, 0x30, 0x20, 0x44, 0x61, 0x79, 0x6c,
0x69, 0x67, 0x68, 0x74, 0x20, 0x53, 0x61, 0x76, 0x69, 0x6e, 0x67, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x25, 0x30, 0x20, 0xd801, 0xdc51, 0xd801, 0xdc72, 0xd801,
0xdc65, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x25, 0x30, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30, 0x28,
0x25, 0x30, 0x29, 0x25, 0x30, 0x20, 0x67, 0x61, 0x6d, 0x65, 0x25, 0x30,
0x20, 0x67, 0x61, 0x6d, 0x65, 0x20, 0x256, 0x6f, 0x256, 0x6f, 0x65, 0x61,
0x25, 0x30, 0x20, 0x14b, 0x6b, 0x65, 0x6b, 0x65, 0x6d, 0x65, 0x263, 0x65,
0x79, 0x69, 0x263, 0x69, 0x25, 0x30, 0x20, 0x74, 0xed, 0xf0, 0x25, 0x30,
0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x25,
0x30, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x25, 0x30, 0x44, 0x61, 0x79,
0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x6e,
0x67, 0x20, 0x25, 0x30, 0x61, 0x69, 0x6b, 0x61, 0x76, 0x79, 0xf6, 0x68,
0x79, 0x6b, 0x65, 0x3a, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x28, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x29,
0x25, 0x30, 0x20, 0x28, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61,
0x29, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x3a, 0x20, 0x25, 0x30, 0x25,
0x30, 0x20, 0x28, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x29, 0x25, 0x30, 0x20, 0x28, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x29, 0x25, 0x30,
0x20, 0x28, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x29, 0x25, 0x30, 0x20, 0x28, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x29, 0x25, 0x30,
0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x25, 0x30, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a,
0xdd23, 0xd83a, 0xdd2b, 0x25, 0x30, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd36, 0xd83a, 0xdd22,
0xd83a, 0xdd25, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd15, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x42, 0x75, 0x6e, 0x2d, 0xe0,
0x6d, 0x3a, 0x20, 0x25, 0x30, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x3a, 0x20, 0x25, 0x30, 0x25,
0x30, 0x20, 0x42, 0x65, 0x25, 0x30, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b,
0x20, 0x46, 0x25b, 0x69, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x25, 0x30,
0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73, 0x61,
0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x3a, 0x20, 0x25, 0x30, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x3a, 0x20, 0x25,
0x30, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe1, 0x6e, 0x20, 0x64, 0x65, 0x3a, 0x20, 0x25, 0x30, 0x10d3, 0x10e0, 0x10dd,
0x3a, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3,
0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x25, 0x30,
0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x25, 0x30, 0x20, 0x28, 0x4f, 0x72, 0x74, 0x73, 0x7a, 0x65, 0x69,
0x74, 0x29, 0x25, 0x30, 0x20, 0x28, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0x65, 0x69, 0x74, 0x29, 0x25, 0x30, 0x20, 0x28, 0x53, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x29, 0x38f, 0x3c1, 0x3b1, 0x20,
0x28, 0x25, 0x30, 0x29, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x28, 0x25, 0x30, 0x29, 0x398, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x28, 0x25, 0x30,
0x29, 0x25, 0x30, 0x20, 0xab8, 0xaae, 0xaaf, 0x25, 0x30, 0x20, 0xaae, 0xabe,
0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0x25, 0x30, 0x20, 0xaa6, 0xabf, 0xab5,
0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0x25, 0x30, 0x20, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x25, 0x30, 0x20, 0x44, 0x61, 0x69, 0x64, 0x61, 0x69, 0x74,
0x61, 0x63, 0x63, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x25, 0x30, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52,
0x61, 0x6e, 0x61, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x25, 0x30, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x25, 0x30, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x25, 0x30, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5,
0x29, 0x25, 0x30, 0x20, 0x938, 0x92e, 0x92f, 0x25, 0x30, 0x20, 0x69, 0x64,
0x151, 0x25, 0x30, 0x20, 0x7a, 0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151, 0x25,
0x30, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x25,
0x30, 0x20, 0x28, 0x73, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d,
0x69, 0x29, 0x25, 0x30, 0x20, 0x28, 0x73, 0x75, 0x6d, 0x61, 0x72, 0x74,
0xed, 0x6d, 0x69, 0x29, 0x4f, 0x67, 0x65, 0x20, 0x25, 0x30, 0x4f, 0x67,
0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x25, 0x30, 0x4f,
0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x25, 0x30, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x25, 0x30, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x25, 0x30, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e,
0x61, 0x73, 0x20, 0x25, 0x30, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x25, 0x30, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x25,
0x30, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x20, 0x25,
0x30, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30, 0x74, 0xe9,
0x6d, 0x70, 0x6f, 0x72, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61, 0x6c,
0x20, 0x64, 0x65, 0x20, 0x25, 0x30, 0x4f, 0x72, 0x61, 0x20, 0x25, 0x30,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x3a, 0x20, 0x25, 0x30, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61,
0x6c, 0x65, 0x3a, 0x20, 0x25, 0x30, 0x25, 0x30, 0x590f, 0x6642, 0x9593, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x25, 0x30, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x25, 0x30, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20,
0x25, 0x30, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x25, 0x30, 0x4f,
0x72, 0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69,
0x20, 0x25, 0x30, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x65,
0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x25, 0x30, 0x41, 0x6b, 0x75,
0x64, 0x3a, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x25, 0x30, 0x20, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75,
0x25, 0x30, 0x20, 0xcb8, 0xcae, 0xcaf, 0x25, 0x30, 0x20, 0xcaa, 0xccd, 0xcb0,
0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0x25, 0x30, 0x20,
0xca6, 0xcbf, 0xca8, 0xca6, 0x20, 0xcb8, 0xcae, 0xcaf, 0x25, 0x30, 0x20, 0x648,
0x64e, 0x6a9, 0x6be, 0x25, 0x30, 0x20, 0x645, 0x639, 0x6cc, 0x672, 0x631, 0x6cc,
0x20, 0x648, 0x64e, 0x6a9, 0x6be, 0x25, 0x30, 0x20, 0x688, 0x6d2, 0x20, 0x644,
0x627, 0x626, 0x679, 0x20, 0x648, 0x64e, 0x6a9, 0x6be, 0x25, 0x30, 0x20, 0x935,
0x916, 0x25, 0x30, 0x20, 0x92e, 0x92f, 0x949, 0x930, 0x940, 0x20, 0x935, 0x916,
0x25, 0x30, 0x20, 0x921, 0x947, 0x20, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x935,
0x916, 0x25, 0x30, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x25, 0x30,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x25, 0x30, 0x20, 0x436, 0x430, 0x437,
0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x25, 0x30, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x25, 0x30, 0x20, 0x633, 0x62a, 0x627,
0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x25, 0x30, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x200b, 0x20, 0x25, 0x30, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f,
0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x20, 0x200b, 0x25, 0x30,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2, 0x1784,
0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x935,
0x947, 0x933, 0x25, 0x30, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f,
0x924, 0x20, 0x935, 0x947, 0x933, 0x25, 0x30, 0x20, 0x921, 0x947, 0x932, 0x93e,
0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x25, 0x30, 0x20, 0x56, 0x65, 0x6c,
0x6c, 0x25, 0x30, 0x20, 0x50, 0x72, 0x6f, 0x6d, 0x61, 0x6e, 0x6e, 0x20,
0x56, 0x65, 0x6c, 0x6c, 0x25, 0x30, 0x20, 0x44, 0x69, 0x73, 0x2d, 0x75,
0x7a, 0x76, 0x61, 0x64, 0x64, 0x20, 0x76, 0x61, 0x63, 0x68, 0x6f, 0x76,
0x70, 0x20, 0x56, 0x65, 0x6c, 0x6c, 0x25, 0x30, 0x20, 0xc2dc, 0xac04, 0x25,
0x30, 0x20, 0xd45c, 0xc900, 0xc2dc, 0x25, 0x30, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c,
0xc900, 0xc2dc, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x25, 0x30, 0x28, 0x79,
0x29, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x25, 0x30, 0x28, 0x79, 0x29,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e,
0xea, 0x20, 0x79, 0x61, 0x20, 0x25, 0x30, 0x28, 0x79, 0x29, 0xea, 0x25,
0x30, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0xec0, 0xea7,
0xea5, 0xeb2, 0x20, 0x25, 0x30, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94,
0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0x20, 0x25, 0x30, 0xec0, 0xea7, 0xea5, 0xeb2,
0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0xeb1, 0xe99, 0x20, 0x25, 0x30, 0x4c, 0x61,
0x69, 0x6b, 0x61, 0x20, 0x6a, 0x6f, 0x73, 0x6c, 0x61, 0x3a, 0x20, 0x25,
0x30, 0x25, 0x30, 0x3a, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x61, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x25, 0x30, 0x3a, 0x20,
0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x4e, 0x67, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x79, 0x61, 0x20, 0x25,
0x30, 0x4c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x3a, 0x20, 0x25, 0x30, 0x17d,
0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x3a, 0x20, 0x25, 0x30, 0x56, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x3a, 0x20, 0x25, 0x30, 0x43, 0x61,
0x73, 0x6f, 0x77, 0x65, 0x20, 0x70, 0x61, 0x73, 0x6d, 0x6f, 0x20, 0x25,
0x30, 0x25, 0x30, 0x20, 0x7a, 0x79, 0x6d, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x25, 0x30, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x25, 0x30, 0x2d, 0x54, 0x69, 0x65, 0x74,
0x25, 0x30, 0x2d, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74,
0x69, 0x65, 0x74, 0x25, 0x30, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x65, 0x74, 0x25, 0x30, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x25,
0x30, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74,
0x25, 0x30, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69,
0x74, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x25, 0x30,
0x25, 0x30, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0x25, 0x30, 0x20, 0xd38, 0xd4d,
0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0x25, 0x30, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d,
0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x25, 0x30, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x25, 0x30,
0x648, 0x642, 0x62a, 0x648, 0x20, 0x25, 0x30, 0x126, 0x69, 0x6e, 0x20, 0x74,
0x61, 0x2019, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x126, 0x69, 0x6e, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x25, 0x30, 0x20, 0x99f,
0x9be, 0x987, 0x9ae, 0x25, 0x30, 0x20, 0x28, 0x2b, 0x30, 0x29, 0x20, 0x9b7,
0x9cd, 0x99f, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9b0, 0x9cd, 0x9a6, 0x20, 0x99f, 0x9be,
0x987, 0x9ae, 0x25, 0x30, 0x20, 0x28, 0x2b, 0x31, 0x29, 0x20, 0x9a6, 0x9c7,
0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x25, 0x30, 0x20,
0x57, 0x101, 0x25, 0x30, 0x20, 0x57, 0x101, 0x20, 0x41, 0x72, 0x6f, 0x77,
0x68, 0x101, 0x6e, 0x75, 0x69, 0x25, 0x30, 0x20, 0x57, 0x101, 0x20, 0x41,
0x77, 0x61, 0x74, 0x65, 0x61, 0x25, 0x30, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x25, 0x30, 0x20, 0x938, 0x942, 0x930,
0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933,
0x25, 0x30, 0x2d, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x25, 0x30, 0x2d, 0x43d,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430,
0x433, 0x25, 0x30, 0x2d, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446,
0x430, 0x433, 0x25, 0x30, 0x20, 0x1834, 0x1820, 0x182d, 0x25, 0x30, 0x20, 0x1830,
0x1832, 0x180b, 0x1820, 0x1828, 0x180b, 0x1833, 0x1820, 0x1837, 0x1833, 0x180b, 0x20, 0x1834,
0x1820, 0x182d, 0x25, 0x30, 0x20, 0x1835, 0x1823, 0x1828, 0x20, 0x180e, 0x180e, 0x180e,
0x1824, 0x20, 0x1834, 0x1820, 0x182d, 0x25, 0x30, 0x20, 0x28, 0x2b, 0x966, 0x29,
0x25, 0x30, 0x20, 0x28, 0x2b, 0x967, 0x29, 0x25, 0x30, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x25, 0x30, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x25, 0x30, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x25, 0x30, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x25, 0x30, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x2d, 0x7e6,
0x7df, 0x7cd, 0x7e1, 0x7ca, 0x7f2, 0x7d3, 0x7ca, 0x7df, 0x7cc, 0x25, 0x30, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x25, 0x30, 0x20, 0x64, 0xe1, 0x62, 0xe1, 0x6c,
0x61, 0x161, 0xe1, 0x69, 0x67, 0x69, 0x25, 0x30, 0x20, 0x67, 0x65, 0x61,
0x73, 0x73, 0x69, 0xe1, 0x69, 0x67, 0x69, 0x25, 0x30, 0x20, 0x64, 0xe1,
0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x25, 0x30, 0x20, 0x67, 0x65,
0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x74, 0x69, 0x64, 0x73,
0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x25, 0x30, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x2013, 0x20, 0x25,
0x30, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x2013,
0x20, 0x25, 0x30, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x20, 0x2013, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x28, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x29, 0x25,
0x30, 0x20, 0x28, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x29, 0x6f, 0x72, 0x61, 0x72, 0x69, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30,
0x6f, 0x72, 0x61, 0x72, 0x69, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0xb38, 0xb2e,
0xb5f, 0x25, 0x30, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0x25, 0x30, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x25, 0x30, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x69,
0x69, 0x20, 0x25, 0x30, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x47, 0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x25, 0x30, 0x25, 0x30,
0x20, 0x440, 0x4d5, 0x441, 0x442, 0x4d5, 0x433, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x25, 0x30, 0x62f, 0x20, 0x25, 0x30, 0x20, 0x67e, 0x647, 0x20,
0x648, 0x62e, 0x62a, 0x25, 0x30, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc,
0x20, 0x648, 0x62e, 0x62a, 0x25, 0x30, 0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648,
0x631, 0x681, 0x20, 0x648, 0x62e, 0x62a, 0x648, 0x642, 0x62a, 0x20, 0x25, 0x30,
0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x25, 0x30, 0x648,
0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20,
0x25, 0x30, 0x63, 0x7a, 0x61, 0x73, 0x3a, 0x20, 0x25, 0x30, 0x25, 0x30,
0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x25, 0x30, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x25, 0x30, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x3a, 0x20,
0x25, 0x30, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x3a, 0x20, 0x25, 0x30, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x25, 0x30, 0x4b, 0x65, 0x72, 0x64, 0x101, 0x3a, 0x20, 0x25, 0x30,
0x5a, 0x113, 0x69, 0x6d, 0x61, 0x73, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101,
0x3a, 0x20, 0x25, 0x30, 0x44, 0x61, 0x67, 0x67, 0x61, 0x73, 0x20, 0x6b,
0x65, 0x72, 0x64, 0x101, 0x3a, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0x25, 0x30, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0x25, 0x30, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e,
0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x25, 0x30, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x25, 0x30,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x25, 0x30, 0x74, 0x65, 0x6d, 0x70, 0x3a, 0x20,
0x25, 0x30, 0x74, 0x65, 0x6d, 0x70, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x3a, 0x20, 0x25, 0x30, 0x74, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61,
0x20, 0x73, 0x74, 0x61, 0x64, 0x3a, 0x20, 0x25, 0x30, 0x25, 0x30, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x25, 0x30, 0x2c, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x25, 0x30,
0x20, 0x938, 0x92e, 0x92f, 0x3a, 0x25, 0x30, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x938, 0x92e, 0x92f, 0x903, 0x25, 0x30, 0x20, 0x905, 0x92f, 0x93e,
0x92e, 0x938, 0x92e, 0x92f, 0x903, 0x25, 0x30, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a,
0x25, 0x30, 0x20, 0x1c62, 0x1c5f, 0x1c71, 0x1c5a, 0x1c60, 0x20, 0x1c5a, 0x1c60, 0x1c5b,
0x1c5a, 0x25, 0x30, 0x20, 0x1c6b, 0x1c64, 0x1c71, 0x1c75, 0x1c6e, 0x1c72, 0x1c5f, 0x20,
0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x3a, 0x20, 0x25, 0x30, 0x25, 0x30, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x25, 0x30, 0x2c, 0x20, 0x43b, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x25, 0x30, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x25, 0x30, 0x2c, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x25,
0x30, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x25, 0x30, 0x2c, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x25, 0x30, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x25, 0x30, 0xa12e, 0xa209, 0x25, 0x30,
0xa395, 0xa3e6, 0xa12e, 0xa209, 0x25, 0x30, 0xa0c5, 0xa3b8, 0xa12e, 0xa209, 0x25, 0x30,
0x20, 0x28, 0x75, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69,
0x29, 0x25, 0x30, 0x20, 0x28, 0x75, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67,
0x61, 0x6c, 0x69, 0x29, 0x25, 0x30, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x29, 0x25, 0x30, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x25, 0x30, 0x20, 0x648, 0x642, 0x62a,
0x25, 0x30, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642,
0x62a, 0x25, 0x30, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x25, 0x30, 0x20, 0x935, 0x915, 0x93c, 0x94d, 0x924,
0x941, 0x25, 0x30, 0x20, 0x92e, 0x907, 0x92f, 0x93e, 0x930, 0x940, 0x20, 0x935,
0x915, 0x93c, 0x94d, 0x924, 0x941, 0x25, 0x30, 0x20, 0x926, 0x940, 0x902, 0x952,
0x939, 0x20, 0x91c, 0x94b, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x25, 0x30, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0x25, 0x30, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8,
0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0x25, 0x30, 0x20, 0xdaf, 0xdd2,
0xdc0, 0xdcf, 0xd86, 0xdbd, 0xddd, 0xd9a, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0x25, 0x30, 0x20, 0x10d, 0x61, 0x73, 0x25, 0x30, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x25,
0x30, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x25, 0x30, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69,
0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x25, 0x30, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61,
0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x25, 0x30, 0x25, 0x30,
0x20, 0x4e, 0x61, 0x6b, 0x6f, 0x25, 0x30, 0x20, 0x4e, 0x61, 0x6b, 0x6f,
0x20, 0x79, 0x61, 0x20, 0x54, 0x6c, 0x77, 0x61, 0x65, 0x6c, 0x6f, 0x25,
0x30, 0x20, 0x4e, 0x61, 0x6b, 0x6f, 0x20, 0x79, 0x61, 0x20, 0x4d, 0x6f,
0x74, 0x73, 0x68, 0x65, 0x68, 0x61, 0x72, 0x65, 0x68, 0x6f, 0x72, 0x61,
0x72, 0x69, 0x6f, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x25, 0x30, 0x68, 0x6f, 0x72, 0x61, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x25, 0x30, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x25, 0x30, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x77, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x25,
0x30, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61,
0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x25, 0x30, 0x25, 0x30, 0x74, 0x69,
0x64, 0x25, 0x30, 0x20, 0x28, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x29, 0x25, 0x30, 0x20, 0x28, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x29, 0x725, 0x715, 0x722, 0x710, 0x20, 0x25, 0x30,
0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d,
0x72c, 0x710, 0x20, 0x25, 0x30, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712,
0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x25, 0x30, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x25, 0x30, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x25,
0x30, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d,
0x430, 0x438, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0x25, 0x30, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0x25, 0x30, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca,
0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0x4a, 0x69, 0x6b, 0x61,
0x6e, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x25, 0x30, 0x20, 0x433, 0x430, 0x434, 0x4d9, 0x442, 0x438, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x25, 0x30, 0x20, 0x497, 0x4d9, 0x439, 0x433,
0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x25, 0x30, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0x25, 0x30, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e,
0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0x25, 0x30, 0x20, 0xc2a,
0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xe40, 0xe27, 0xe25, 0xe32, 0x25, 0x30, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0x25, 0x30,
0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0x25, 0x30,
0x130d, 0x12d8, 0x20, 0x25, 0x30, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x130a, 0x12dc, 0x20, 0x25, 0x30, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x25, 0x30, 0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x130d, 0x12d8, 0x54, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x25, 0x30, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x25,
0x30, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x25, 0x30, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x25, 0x30, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x25, 0x30, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x25, 0x30, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x25, 0x30, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x447, 0x430, 0x441, 0x3a, 0x20, 0x25, 0x30,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438,
0x439, 0x447, 0x430, 0x441, 0x3a, 0x20, 0x25, 0x30, 0x2c, 0x20, 0x43b, 0x456,
0x442, 0x43d, 0x456, 0x439, 0x10d, 0x61, 0x73, 0x6f, 0x77, 0x65, 0x20, 0x70,
0x61, 0x73, 0x6d, 0x6f, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x7a, 0x79,
0x6d, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x25, 0x30, 0x20, 0x6c,
0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x25, 0x30, 0x20, 0x645,
0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x25, 0x30, 0x20,
0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645,
0x25, 0x30, 0x20, 0x62f, 0x646, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a,
0x25, 0x30, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x25, 0x30, 0x20, 0x626,
0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x25, 0x30, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x25, 0x30, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x25, 0x30, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x25, 0x30, 0x20, 0x43a, 0x443, 0x43d,
0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x47,
0x69, 0x1edd, 0x20, 0x25, 0x30, 0x47, 0x69, 0x1edd, 0x20, 0x63, 0x68, 0x75,
0x1ea9, 0x6e, 0x20, 0x25, 0x30, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61,
0x20, 0x68, 0xe8, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x7a, 0x69, 0x74,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x25, 0x30, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x25, 0x30,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x25, 0x30,
0x25, 0x30, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x69, 0x64, 0x20, 0x25, 0x30, 0x5a,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x20, 0x25, 0x30, 0xcc,
0x67, 0x62, 0xe0, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0xcc, 0x6c, 0xe0,
0x6e, 0xe0, 0x20, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x25, 0x30, 0x20, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x6f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd,
0x25, 0x30, 0x20, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x6f, 0x6a, 0xfa,
0x6d, 0x254, 0x6d, 0x254, 0x25, 0x30, 0x20, 0x53, 0x69, 0x7a, 0x67, 0x65,
0x6e, 0x68, 0x25, 0x30, 0x20, 0x42, 0x79, 0x61, 0x75, 0x68, 0x63, 0x75,
0x6e, 0x6a, 0x20, 0x53, 0x69, 0x7a, 0x67, 0x65, 0x6e, 0x68, 0x25, 0x30,
0x20, 0x59, 0x61, 0x6c, 0x69, 0x6e, 0x67, 0x20, 0x53, 0x69, 0x7a, 0x67,
0x65, 0x6e, 0x68, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x25, 0x30, 0x25, 0x30, 0x20, 0x69,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69,
0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x25, 0x30, 0x20, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d,
0x69, 0x6e, 0x69, 0xd3, 0x72, 0x61, 0x20, 0x6b, 0x61, 0x72, 0x20, 0x25,
0x30, 0xd3, 0x72, 0x61, 0x20, 0x70, 0x1ebd, 0x20, 0x25, 0x30, 0x50, 0x72,
0x1ef9, 0x67, 0x20, 0x6b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x6b, 0x61,
0x72, 0x20, 0x25, 0x30, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x20, 0x25, 0x30, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x3a, 0x20, 0x25, 0x30, 0x4b, 0x75,
0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x3a, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20,
0x28, 0x2b, 0x30, 0x29, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x923, 0x94d, 0x921,
0x930, 0x94d, 0x921, 0x20, 0x938, 0x92e, 0x92f, 0x25, 0x30, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x25, 0x30, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x25, 0x30, 0x20, 0x53, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x74, 0x65, 0x6e, 0x70, 0x6f, 0x20,
0x70, 0x69, 0x20, 0x25, 0x30, 0x74, 0x65, 0x6e, 0x70, 0x6f, 0x20, 0x73,
0x65, 0x6c, 0x69, 0x20, 0x73, 0x75, 0x6e, 0x6f, 0x20, 0x70, 0x69, 0x20,
0x25, 0x30, 0x25, 0x30, 0x20, 0x54, 0x61, 0x65, 0x6d, 0x25, 0x30, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x64, 0x20, 0x54, 0x61, 0x65, 0x6d,
0x25, 0x30, 0x20, 0x44, 0x65, 0x6c, 0x61, 0x65, 0x74, 0x20, 0x54, 0x61,
0x65, 0x6d, 0x6f, 0x61, 0x3a, 0x20, 0x25, 0x30, 0x6f, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x3a, 0x20, 0x25, 0x30, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x3a, 0x20, 0x25, 0x30,
0x25, 0x30, 0x20, 0x648, 0x64e, 0x62e, 0x25, 0x30, 0x20, 0x645, 0x639, 0x6cc,
0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x25, 0x30, 0x20, 0x62f, 0x6be,
0x627, 0x62a, 0x25, 0x30, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x25, 0x30, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65,
0x69, 0x25, 0x30, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61,
0x25, 0x30, 0x20, 0x91f, 0x948, 0x92e, 0x25, 0x30, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x91f, 0x948, 0x92e, 0x25, 0x30, 0x20, 0x927, 0x94d, 0x92f, 0x93e,
0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x4f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x25, 0x30, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x25, 0x30, 0x25, 0x30,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x25, 0x30, 0x20, 0x6d, 0x101, 0x6e, 0x101,
0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x25, 0x30, 0x20, 0x1e0d,
0x61, 0x79, 0x6c, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x25, 0x30,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x25, 0x30, 0x20, 0x921, 0x947, 0x932, 0x93e,
0x907, 0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x25, 0x30, 0x20, 0x92e, 0x93e,
0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x25, 0x30,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0x25, 0x30, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e,
0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0x25, 0x30, 0x20, 0xb21,
0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0x25, 0x30,
0x20, 0x1401, 0x1403, 0x1422, 0x1438, 0x14c2, 0x1420, 0x25, 0x30, 0x20, 0x142f, 0x152d,
0x146f, 0x1426, 0x1456, 0x140f, 0x1423, 0x20, 0x1401, 0x1403, 0x1422, 0x1438, 0x14c2, 0x1420,
0x25, 0x30, 0x20, 0x146e, 0x14ef, 0x1473, 0x1424, 0x20, 0x1401, 0x1403, 0x1422, 0x1438,
0x14c2, 0x1420
};

static inline constexpr char16_t fallbackFormatTable[] = {
0x25, 0x31, 0x20, 0x28, 0x25, 0x30, 0x29, 0x25, 0x31, 0x20, 0x25, 0x30,
0x25, 0x31, 0xff08, 0x25, 0x30, 0xff09, 0x25, 0x31, 0xf0d, 0x20, 0x28, 0x25,
0x30, 0xf0b, 0x29, 0x5b, 0x25, 0x31, 0x20, 0x28, 0x25, 0x30, 0x29, 0x5d,
0x25, 0x30, 0x20, 0x28, 0x25, 0x31, 0x29, 0x25, 0x31, 0x28, 0x25, 0x30,
0x29
};

static inline constexpr char16_t exemplarCityTable[] = {
0x41, 0x73, 0x6d, 0x61, 0x72, 0x61, 0x53, 0xe3, 0x6f, 0x20, 0x54, 0x6f,
0x6d, 0xe9, 0x41, 0x73, 0x75, 0x6e, 0x63, 0x69, 0xf3, 0x6e, 0x41, 0x74,
0x69, 0x6b, 0x6f, 0x6b, 0x61, 0x6e, 0x42, 0x61, 0x68, 0xed, 0x61, 0x20,
0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x61, 0x73, 0x43,
0x61, 0x6e, 0x63, 0xfa, 0x6e, 0x43, 0x69, 0x75, 0x64, 0x61, 0x64, 0x20,
0x4a, 0x75, 0xe1, 0x72, 0x65, 0x7a, 0x43, 0x75, 0x72, 0x61, 0xe7, 0x61,
0x6f, 0x4b, 0x6e, 0x6f, 0x78, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61,
0x6e, 0x61, 0x4d, 0x61, 0x72, 0x65, 0x6e, 0x67, 0x6f, 0x2c, 0x20, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x50, 0x65, 0x74, 0x65, 0x72, 0x73,
0x62, 0x75, 0x72, 0x67, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e,
0x61, 0x54, 0x65, 0x6c, 0x6c, 0x20, 0x43, 0x69, 0x74, 0x79, 0x2c, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x56, 0x65, 0x76, 0x61, 0x79,
0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x56, 0x69, 0x6e,
0x63, 0x65, 0x6e, 0x6e, 0x65, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69,
0x61, 0x6e, 0x61, 0x57, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x63, 0x2c, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x4d, 0x6f, 0x6e, 0x74, 0x69,
0x63, 0x65, 0x6c, 0x6c, 0x6f, 0x2c, 0x20, 0x4b, 0x65, 0x6e, 0x74, 0x75,
0x63, 0x6b, 0x79, 0x4c, 0x6f, 0x77, 0x65, 0x72, 0x20, 0x50, 0x72, 0x69,
0x6e, 0x63, 0x65, 0x2019, 0x73, 0x20, 0x51, 0x75, 0x61, 0x72, 0x74, 0x65,
0x72, 0x4d, 0xe9, 0x72, 0x69, 0x64, 0x61, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x4e, 0x6f,
0x72, 0x74, 0x68, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x43, 0x65,
0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61,
0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x75, 0x75, 0x6b, 0x49, 0x74, 0x74,
0x6f, 0x71, 0x71, 0x6f, 0x72, 0x74, 0x6f, 0x6f, 0x72, 0x6d, 0x69, 0x69,
0x74, 0x53, 0x74, 0x2e, 0x20, 0x42, 0x61, 0x72, 0x74, 0x68, 0xe9, 0x6c,
0x65, 0x6d, 0x79, 0x53, 0x74, 0x2e, 0x20, 0x4a, 0x6f, 0x68, 0x6e, 0x2019,
0x73, 0x53, 0x74, 0x2e, 0x20, 0x4b, 0x69, 0x74, 0x74, 0x73, 0x53, 0x74,
0x2e, 0x20, 0x4c, 0x75, 0x63, 0x69, 0x61, 0x53, 0x74, 0x2e, 0x20, 0x54,
0x68, 0x6f, 0x6d, 0x61, 0x73, 0x53, 0x74, 0x2e, 0x20, 0x56, 0x69, 0x6e,
0x63, 0x65, 0x6e, 0x74, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64,
0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x4d, 0x61, 0x63, 0x71,
0x75, 0x61, 0x72, 0x69, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64,
0x48, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x20,
0x43, 0x69, 0x74, 0x79, 0x4b, 0x61, 0x74, 0x68, 0x6d, 0x61, 0x6e, 0x64,
0x75, 0x4b, 0x6f, 0x6c, 0x6b, 0x61, 0x74, 0x61, 0x4d, 0x61, 0x63, 0x61,
0x6f, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x79, 0x59, 0x61, 0x6e,
0x67, 0x6f, 0x6e, 0x46, 0x61, 0x72, 0x6f, 0x65, 0x53, 0x74, 0x2e, 0x20,
0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4b, 0x79,
0x69, 0x76, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x52, 0xe9, 0x75, 0x6e, 0x69,
0x6f, 0x6e, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x45, 0x61, 0x73, 0x74, 0x65,
0x72, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x50, 0x6f,
0x68, 0x6e, 0x70, 0x65, 0x69, 0x57, 0x61, 0x6b, 0x65, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x410, 0x431, 0x438, 0x45f, 0x430, 0x43d, 0x410, 0x43a,
0x43a, 0x440, 0x430, 0x410, 0x434, 0x434, 0x438, 0x441, 0x2d, 0x410, 0x431, 0x435,
0x431, 0x430, 0x410, 0x43b, 0x436, 0x438, 0x440, 0x410, 0x441, 0x43c, 0x435, 0x440,
0x430, 0x411, 0x430, 0x43c, 0x430, 0x43a, 0x43e, 0x411, 0x430, 0x43d, 0x433, 0x438,
0x411, 0x430, 0x43d, 0x436, 0x443, 0x43b, 0x411, 0x438, 0x441, 0x430, 0x443, 0x411,
0x43b, 0x430, 0x43d, 0x442, 0x430, 0x438, 0x440, 0x411, 0x440, 0x430, 0x437, 0x437,
0x430, 0x432, 0x438, 0x43b, 0x411, 0x443, 0x436, 0x443, 0x43c, 0x431, 0x443, 0x440,
0x430, 0x41a, 0x430, 0x438, 0x440, 0x41a, 0x430, 0x441, 0x430, 0x431, 0x43b, 0x430,
0x43d, 0x43a, 0x430, 0x421, 0x435, 0x443, 0x442, 0x430, 0x41a, 0x43e, 0x43d, 0x430,
0x43a, 0x440, 0x438, 0x414, 0x430, 0x43a, 0x430, 0x440, 0x414, 0x430, 0x440, 0x2d,
0x435, 0x441, 0x2d, 0x421, 0x430, 0x43b, 0x430, 0x43c, 0x40f, 0x438, 0x431, 0x443,
0x442, 0x438, 0x414, 0x443, 0x430, 0x43b, 0x430, 0x415, 0x43b, 0x2d, 0x410, 0x438,
0x443, 0x43d, 0x424, 0x440, 0x438, 0x442, 0x430, 0x443, 0x43d, 0x413, 0x430, 0x431,
0x43e, 0x440, 0x43e, 0x43d, 0x435, 0x4b2, 0x430, 0x440, 0x430, 0x440, 0x435, 0x418,
0x43e, 0x4b3, 0x430, 0x43d, 0x43d, 0x435, 0x441, 0x431, 0x443, 0x440, 0x433, 0x40f,
0x443, 0x431, 0x430, 0x41a, 0x430, 0x43c, 0x43f, 0x430, 0x43b, 0x430, 0x425, 0x430,
0x440, 0x442, 0x443, 0x43c, 0x41a, 0x438, 0x433, 0x430, 0x43b, 0x438, 0x41a, 0x438,
0x43d, 0x448, 0x430, 0x441, 0x430, 0x41b, 0x430, 0x433, 0x43e, 0x441, 0x41b, 0x438,
0x431, 0x440, 0x435, 0x432, 0x438, 0x43b, 0x41b, 0x43e, 0x43c, 0x435, 0x41b, 0x443,
0x430, 0x43d, 0x434, 0x430, 0x41b, 0x443, 0x431, 0x443, 0x43c, 0x431, 0x430, 0x448,
0x438, 0x41b, 0x443, 0x441, 0x430, 0x43a, 0x430, 0x41c, 0x430, 0x43b, 0x430, 0x431,
0x43e, 0x41c, 0x430, 0x43f, 0x443, 0x442, 0x443, 0x41c, 0x430, 0x441, 0x435, 0x440,
0x443, 0x41c, 0x431, 0x430, 0x431, 0x430, 0x43d, 0x435, 0x41c, 0x43e, 0x433, 0x430,
0x434, 0x438, 0x448, 0x43e, 0x41c, 0x43e, 0x43d, 0x440, 0x43e, 0x432, 0x438, 0x430,
0x41d, 0x430, 0x438, 0x440, 0x43e, 0x431, 0x438, 0x41d, 0x45f, 0x430, 0x43c, 0x435,
0x43d, 0x430, 0x41d, 0x438, 0x430, 0x43c, 0x435, 0x438, 0x41d, 0x443, 0x430, 0x43a,
0x448, 0x43e, 0x442, 0x423, 0x430, 0x433, 0x430, 0x434, 0x443, 0x433, 0x443, 0x41f,
0x43e, 0x440, 0x442, 0x43e, 0x2d, 0x41d, 0x43e, 0x432, 0x43e, 0x421, 0x430, 0x43d,
0x2d, 0x422, 0x43e, 0x43c, 0x435, 0x422, 0x440, 0x438, 0x43f, 0x43e, 0x43b, 0x438,
0x422, 0x443, 0x43d, 0x438, 0x441, 0x412, 0x438, 0x43d, 0x434, 0x445, 0x443, 0x43a,
0x410, 0x434, 0x430, 0x43a, 0x410, 0x43d, 0x43a, 0x43e, 0x440, 0x438, 0x45f, 0x410,
0x43d, 0x433, 0x438, 0x43b, 0x438, 0x430, 0x410, 0x43d, 0x442, 0x438, 0x433, 0x443,
0x430, 0x410, 0x440, 0x430, 0x433, 0x443, 0x430, 0x438, 0x43d, 0x430, 0x411, 0x443,
0x435, 0x43d, 0x43e, 0x441, 0x2d, 0x410, 0x438, 0x440, 0x435, 0x441, 0x41a, 0x430,
0x442, 0x430, 0x43c, 0x430, 0x440, 0x43a, 0x430, 0x41a, 0x43e, 0x440, 0x434, 0x43e,
0x432, 0x430, 0x416, 0x443, 0x436, 0x443, 0x438, 0x41b, 0x430, 0x2d, 0x420, 0x438,
0x43e, 0x445, 0x430, 0x41c, 0x435, 0x43d, 0x434, 0x43e, 0x441, 0x430, 0x420, 0x438,
0x43e, 0x2d, 0x413, 0x430, 0x43b, 0x435, 0x433, 0x43e, 0x441, 0x421, 0x430, 0x43b,
0x442, 0x430, 0x421, 0x430, 0x43d, 0x2d, 0x425, 0x443, 0x430, 0x43d, 0x421, 0x430,
0x43d, 0x2d, 0x41b, 0x443, 0x438, 0x441, 0x422, 0x443, 0x43a, 0x443, 0x43c, 0x430,
0x43d, 0x423, 0x448, 0x443, 0x430, 0x438, 0x430, 0x410, 0x440, 0x443, 0x431, 0x430,
0x410, 0x441, 0x443, 0x43d, 0x441, 0x43e, 0x43d, 0x41a, 0x43e, 0x440, 0x430, 0x43b,
0x2d, 0x425, 0x430, 0x440, 0x431, 0x43e, 0x440, 0x411, 0x430, 0x438, 0x430, 0x411,
0x430, 0x438, 0x430, 0x2d, 0x434, 0x435, 0x2d, 0x411, 0x430, 0x43d, 0x434, 0x435,
0x440, 0x430, 0x441, 0x411, 0x430, 0x440, 0x431, 0x430, 0x434, 0x43e, 0x441, 0x411,
0x435, 0x43b, 0x435, 0x43d, 0x411, 0x435, 0x43b, 0x438, 0x437, 0x411, 0x43b, 0x430,
0x43d, 0x43a, 0x2d, 0x421, 0x430, 0x431, 0x43b, 0x43e, 0x43d, 0x411, 0x43e, 0x430,
0x2d, 0x412, 0x438, 0x441, 0x442, 0x430, 0x411, 0x43e, 0x433, 0x43e, 0x442, 0x430,
0x411, 0x43e, 0x438, 0x441, 0x435, 0x41a, 0x435, 0x438, 0x43c, 0x431, 0x440, 0x438,
0x45f, 0x2d, 0x411, 0x435, 0x438, 0x41a, 0x430, 0x43c, 0x43f, 0x443, 0x2d, 0x413,
0x440, 0x430, 0x43d, 0x434, 0x438, 0x41a, 0x430, 0x43d, 0x43a, 0x443, 0x43d, 0x41a,
0x430, 0x440, 0x430, 0x43a, 0x430, 0x441, 0x41a, 0x430, 0x438, 0x435, 0x43d, 0x43d,
0x430, 0x410, 0x434, 0x433, 0x44c, 0x44b, 0x43b, 0x431, 0x436, 0x44c, 0x430, 0x445,
0x430, 0x49b, 0x4d9, 0x430, 0x20, 0x41a, 0x430, 0x438, 0x43c, 0x430, 0x43d, 0x427,
0x438, 0x43a, 0x430, 0x433, 0x43e, 0x427, 0x438, 0x443, 0x430, 0x443, 0x430, 0x41a,
0x43e, 0x441, 0x442, 0x430, 0x2d, 0x420, 0x438, 0x43a, 0x430, 0x41a, 0x440, 0x435,
0x441, 0x442, 0x43e, 0x43d, 0x41a, 0x443, 0x438, 0x430, 0x431, 0x430, 0x41a, 0x438,
0x443, 0x440, 0x430, 0x441, 0x430, 0x43e, 0x414, 0x435, 0x43d, 0x43c, 0x430, 0x440,
0x43a, 0x441, 0x445, 0x430, 0x432, 0x43d, 0x414, 0x43e, 0x443, 0x441, 0x43e, 0x43d,
0x414, 0x43e, 0x443, 0x441, 0x43e, 0x43d, 0x2d, 0x41a, 0x440, 0x438, 0x43a, 0x414,
0x435, 0x43d, 0x432, 0x435, 0x440, 0x414, 0x435, 0x442, 0x440, 0x43e, 0x438, 0x442,
0x414, 0x43e, 0x43c, 0x438, 0x43d, 0x438, 0x43a, 0x430, 0x415, 0x434, 0x43c, 0x43e,
0x43d, 0x442, 0x43e, 0x43d, 0x415, 0x438, 0x440, 0x443, 0x43d, 0x435, 0x43f, 0x435,
0x421, 0x430, 0x43b, 0x432, 0x430, 0x434, 0x43e, 0x440, 0x424, 0x43e, 0x440, 0x442,
0x20, 0x41d, 0x435, 0x43b, 0x441, 0x43e, 0x43d, 0x424, 0x43e, 0x440, 0x442, 0x430,
0x43b, 0x435, 0x437, 0x430, 0x413, 0x43b, 0x435, 0x438, 0x441, 0x2d, 0x411, 0x435,
0x438, 0x413, 0x443, 0x441, 0x2d, 0x411, 0x435, 0x438, 0x413, 0x440, 0x430, 0x43d,
0x434, 0x2d, 0x422, 0x435, 0x440, 0x43a, 0x413, 0x440, 0x435, 0x43d, 0x430, 0x434,
0x430, 0x413, 0x432, 0x430, 0x434, 0x435, 0x43b, 0x443, 0x43f, 0x430, 0x413, 0x432,
0x430, 0x442, 0x435, 0x43c, 0x430, 0x43b, 0x430, 0x413, 0x443, 0x430, 0x438, 0x430,
0x43a, 0x438, 0x43b, 0x413, 0x430, 0x438, 0x430, 0x43d, 0x430, 0x4b2, 0x430, 0x43b,
0x438, 0x444, 0x430, 0x43a, 0x441, 0x413, 0x430, 0x432, 0x430, 0x43d, 0x430, 0x415,
0x440, 0x43c, 0x43e, 0x441, 0x438, 0x43b, 0x43e, 0x418, 0x43d, 0x434, 0x438, 0x430,
0x43d, 0x430, 0x43f, 0x43e, 0x43b, 0x438, 0x441, 0x41d, 0x43e, 0x43a, 0x441, 0x2c,
0x20, 0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d, 0x430, 0x41c, 0x430, 0x440, 0x435,
0x43d, 0x433, 0x43e, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d, 0x430,
0x41f, 0x438, 0x442, 0x435, 0x440, 0x441, 0x431, 0x435, 0x440, 0x433, 0x2c, 0x20,
0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d, 0x430, 0x422, 0x435, 0x43b, 0x43b, 0x2d,
0x421, 0x438, 0x442, 0x438, 0x412, 0x435, 0x432, 0x435, 0x438, 0x2c, 0x20, 0x418,
0x43d, 0x434, 0x438, 0x430, 0x43d, 0x430, 0x412, 0x438, 0x43d, 0x441, 0x435, 0x43d,
0x43d, 0x435, 0x441, 0x423, 0x438, 0x43d, 0x430, 0x43c, 0x430, 0x43a, 0x418, 0x43d,
0x443, 0x432, 0x438, 0x43a, 0x418, 0x43a, 0x430, 0x43b, 0x443, 0x438, 0x442, 0x418,
0x430, 0x43c, 0x430, 0x438, 0x43a, 0x430, 0x40f, 0x443, 0x43d, 0x43e, 0x41b, 0x443,
0x438, 0x441, 0x432, 0x438, 0x43b, 0x43b, 0x41c, 0x43e, 0x43d, 0x442, 0x438, 0x441,
0x435, 0x43b, 0x43b, 0x43e, 0x2c, 0x20, 0x41a, 0x435, 0x43d, 0x442, 0x443, 0x43a,
0x43a, 0x438, 0x41a, 0x440, 0x430, 0x43b, 0x435, 0x43d, 0x434, 0x435, 0x438, 0x43a,
0x41b, 0x430, 0x2d, 0x41f, 0x430, 0x441, 0x41b, 0x438, 0x43c, 0x430, 0x41b, 0x43e,
0x441, 0x2d, 0x410, 0x43d, 0x45f, 0x435, 0x43b, 0x435, 0x441, 0x41b, 0x43e, 0x443,
0x435, 0x440, 0x2d, 0x41f, 0x440, 0x438, 0x43d, 0x441, 0x2d, 0x41a, 0x443, 0x43e,
0x442, 0x435, 0x440, 0x41c, 0x430, 0x441, 0x435, 0x438, 0x43e, 0x41c, 0x430, 0x43d,
0x430, 0x433, 0x443, 0x430, 0x41c, 0x430, 0x43d, 0x430, 0x443, 0x441, 0x41c, 0x430,
0x440, 0x438, 0x433, 0x43e, 0x41c, 0x430, 0x440, 0x442, 0x438, 0x43d, 0x438, 0x43a,
0x430, 0x41c, 0x430, 0x442, 0x430, 0x43c, 0x43e, 0x440, 0x43e, 0x441, 0x41c, 0x430,
0x441, 0x430, 0x442, 0x43b, 0x430, 0x43d, 0x41c, 0x435, 0x43d, 0x43e, 0x43c, 0x438,
0x43d, 0x438, 0x41c, 0x435, 0x440, 0x438, 0x434, 0x430, 0x41c, 0x435, 0x442, 0x43b,
0x430, 0x43a, 0x430, 0x442, 0x43b, 0x430, 0x41c, 0x435, 0x445, 0x438, 0x43a, 0x43e,
0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x41c, 0x43e, 0x43d, 0x43a, 0x442,
0x43e, 0x43d, 0x41c, 0x43e, 0x43d, 0x442, 0x435, 0x440, 0x440, 0x435, 0x438, 0x41c,
0x43e, 0x43d, 0x442, 0x435, 0x432, 0x438, 0x434, 0x435, 0x43e, 0x41c, 0x43e, 0x43d,
0x442, 0x441, 0x435, 0x440, 0x440, 0x430, 0x442, 0x41d, 0x430, 0x441, 0x441, 0x430,
0x443, 0x41d, 0x438, 0x443, 0x2d, 0x418, 0x43e, 0x440, 0x43a, 0x41d, 0x43e, 0x43c,
0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x438, 0x430, 0x411, 0x43e, 0x438, 0x43b, 0x430,
0x2c, 0x20, 0x410, 0x4a9, 0x430, 0x434, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x414,
0x430, 0x43a, 0x43e, 0x442, 0x430, 0x410, 0x433, 0x4d9, 0x4ad, 0x430, 0x2c, 0x20,
0x410, 0x4a9, 0x430, 0x434, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x414, 0x430, 0x43a,
0x43e, 0x442, 0x430, 0x41d, 0x438, 0x443, 0x2d, 0x421, 0x435, 0x438, 0x43b, 0x435,
0x43c, 0x2c, 0x20, 0x410, 0x4a9, 0x430, 0x434, 0x430, 0x442, 0x4d9, 0x438, 0x20,
0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41d, 0x443, 0x443, 0x43a, 0x41e, 0x445,
0x438, 0x43d, 0x430, 0x433, 0x430, 0x41f, 0x430, 0x43d, 0x430, 0x43c, 0x430, 0x41f,
0x430, 0x440, 0x430, 0x43c, 0x430, 0x440, 0x438, 0x431, 0x43e, 0x424, 0x438, 0x43d,
0x438, 0x43a, 0x441, 0x41f, 0x43e, 0x440, 0x442, 0x2d, 0x43e, 0x2d, 0x41f, 0x440,
0x435, 0x43d, 0x441, 0x41f, 0x43e, 0x440, 0x442, 0x2d, 0x43e, 0x444, 0x2d, 0x421,
0x43f, 0x435, 0x438, 0x43d, 0x41f, 0x43e, 0x440, 0x442, 0x443, 0x2d, 0x412, 0x435,
0x43b, 0x438, 0x443, 0x41f, 0x443, 0x435, 0x440, 0x442, 0x43e, 0x2d, 0x420, 0x438,
0x43a, 0x43e, 0x41f, 0x443, 0x43d, 0x442, 0x430, 0x2d, 0x410, 0x440, 0x435, 0x43d,
0x430, 0x441, 0x420, 0x430, 0x43d, 0x43a, 0x438, 0x43d, 0x2d, 0x418, 0x43d, 0x43b,
0x435, 0x442, 0x420, 0x435, 0x441, 0x438, 0x444, 0x438, 0x420, 0x435, 0x45f, 0x430,
0x438, 0x43d, 0x430, 0x420, 0x435, 0x437, 0x43e, 0x43b, 0x438, 0x443, 0x442, 0x420,
0x438, 0x443, 0x2d, 0x411, 0x440, 0x430, 0x43d, 0x43a, 0x443, 0x421, 0x430, 0x43d,
0x442, 0x430, 0x440, 0x435, 0x43d, 0x421, 0x430, 0x43d, 0x442, 0x438, 0x430, 0x433,
0x43e, 0x421, 0x430, 0x43d, 0x442, 0x43e, 0x2d, 0x414, 0x43e, 0x43c, 0x438, 0x43d,
0x433, 0x43e, 0x421, 0x430, 0x43d, 0x2d, 0x41f, 0x430, 0x443, 0x43b, 0x443, 0x421,
0x43a, 0x43e, 0x440, 0x441, 0x431, 0x438, 0x441, 0x443, 0x43d, 0x43d, 0x421, 0x438,
0x442, 0x43a, 0x430, 0x421, 0x435, 0x43d, 0x2d, 0x411, 0x430, 0x440, 0x442, 0x435,
0x43b, 0x435, 0x43c, 0x438, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x40f, 0x43e, 0x43d,
0x441, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41a, 0x438, 0x442, 0x441, 0x421, 0x435,
0x43d, 0x442, 0x2d, 0x41b, 0x438, 0x443, 0x441, 0x438, 0x430, 0x421, 0x435, 0x43d,
0x442, 0x2d, 0x422, 0x43e, 0x43c, 0x430, 0x441, 0x421, 0x435, 0x43d, 0x442, 0x2d,
0x412, 0x438, 0x43d, 0x441, 0x435, 0x43d, 0x442, 0x421, 0x432, 0x438, 0x444, 0x442,
0x2d, 0x41a, 0x435, 0x440, 0x440, 0x435, 0x43d, 0x442, 0x422, 0x435, 0x433, 0x443,
0x441, 0x438, 0x433, 0x430, 0x43b, 0x43f, 0x430, 0x422, 0x443, 0x43b, 0x435, 0x422,
0x438, 0x445, 0x443, 0x430, 0x43d, 0x430, 0x422, 0x43e, 0x440, 0x43e, 0x43d, 0x442,
0x43e, 0x422, 0x43e, 0x440, 0x442, 0x43e, 0x43b, 0x430, 0x412, 0x430, 0x43d, 0x43a,
0x443, 0x432, 0x435, 0x440, 0x423, 0x430, 0x438, 0x442, 0x445, 0x43e, 0x440, 0x441,
0x412, 0x438, 0x43d, 0x43d, 0x438, 0x43f, 0x435, 0x433, 0x418, 0x430, 0x43a, 0x443,
0x442, 0x430, 0x442, 0x41a, 0x435, 0x438, 0x441, 0x438, 0x414, 0x435, 0x438, 0x432,
0x438, 0x441, 0x414, 0x438, 0x443, 0x43c, 0x43e, 0x43d, 0x2d, 0x434, 0x2019, 0x418,
0x443, 0x440, 0x432, 0x438, 0x43b, 0x41c, 0x430, 0x43a, 0x43a, 0x443, 0x43e, 0x440,
0x438, 0x41c, 0x43e, 0x443, 0x441, 0x43e, 0x43d, 0x41c, 0x430, 0x43a, 0x2d, 0x41c,
0x435, 0x440, 0x434, 0x43e, 0x41f, 0x430, 0x43b, 0x43c, 0x435, 0x440, 0x420, 0x43e,
0x442, 0x435, 0x440, 0x430, 0x421, 0x438, 0x43e, 0x432, 0x430, 0x422, 0x440, 0x43e,
0x43b, 0x43b, 0x410, 0x43c, 0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430, 0x440, 0x430,
0x41b, 0x43e, 0x43d, 0x433, 0x438, 0x440, 0x410, 0x434, 0x435, 0x43d, 0x410, 0x43b,
0x43c, 0x430, 0x442, 0x44b, 0x410, 0x43c, 0x43c, 0x430, 0x43d, 0x410, 0x43d, 0x430,
0x434, 0x44b, 0x440, 0x410, 0x43a, 0x442, 0x430, 0x443, 0x410, 0x43a, 0x442, 0x43e,
0x431, 0x435, 0x410, 0x448, 0x445, 0x430, 0x431, 0x430, 0x434, 0x410, 0x442, 0x44b,
0x440, 0x430, 0x443, 0x411, 0x430, 0x433, 0x434, 0x430, 0x434, 0x411, 0x430, 0x445,
0x440, 0x435, 0x438, 0x43d, 0x411, 0x430, 0x43a, 0x443, 0x411, 0x430, 0x43d, 0x433,
0x43a, 0x43e, 0x43a, 0x411, 0x430, 0x440, 0x43d, 0x430, 0x443, 0x43b, 0x411, 0x435,
0x438, 0x440, 0x443, 0x442, 0x411, 0x438, 0x448, 0x43a, 0x435, 0x43a, 0x411, 0x440,
0x443, 0x43d, 0x435, 0x438, 0x427, 0x438, 0x442, 0x430, 0x41a, 0x43e, 0x43b, 0x43e,
0x43c, 0x431, 0x43e, 0x414, 0x430, 0x43c, 0x430, 0x441, 0x43a, 0x414, 0x430, 0x43a,
0x43a, 0x430, 0x414, 0x438, 0x43b, 0x438, 0x414, 0x443, 0x431, 0x430, 0x438, 0x414,
0x443, 0x448, 0x430, 0x43d, 0x431, 0x435, 0x424, 0x430, 0x43c, 0x430, 0x433, 0x443,
0x441, 0x442, 0x430, 0x413, 0x430, 0x437, 0x430, 0x425, 0x435, 0x432, 0x440, 0x43e,
0x43d, 0x425, 0x43e, 0x448, 0x438, 0x43c, 0x438, 0x43d, 0x4b2, 0x43e, 0x43d, 0x43a,
0x43e, 0x43d, 0x433, 0x425, 0x43e, 0x432, 0x434, 0x418, 0x440, 0x43a, 0x443, 0x442,
0x441, 0x43a, 0x40f, 0x430, 0x43a, 0x430, 0x440, 0x442, 0x430, 0x40f, 0x430, 0x438,
0x430, 0x43f, 0x443, 0x440, 0x430, 0x418, 0x435, 0x440, 0x443, 0x441, 0x430, 0x43b,
0x438, 0x43c, 0x41a, 0x430, 0x431, 0x443, 0x43b, 0x41f, 0x435, 0x442, 0x440, 0x43e,
0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x2d, 0x41a, 0x430, 0x43c,
0x447, 0x430, 0x442, 0x43a, 0x430, 0x442, 0x4d9, 0x438, 0x41a, 0x430, 0x440, 0x430,
0x447, 0x438, 0x41a, 0x430, 0x442, 0x43c, 0x430, 0x43d, 0x434, 0x443, 0x425, 0x430,
0x43d, 0x434, 0x44b, 0x433, 0x430, 0x41a, 0x430, 0x43b, 0x43a, 0x443, 0x442, 0x442,
0x430, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x438, 0x430, 0x440, 0x441, 0x43a,
0x41a, 0x443, 0x430, 0x43b, 0x430, 0x2d, 0x41b, 0x443, 0x43c, 0x43f, 0x443, 0x440,
0x41a, 0x443, 0x447, 0x438, 0x43d, 0x433, 0x41a, 0x443, 0x432, 0x435, 0x438, 0x442,
0x41c, 0x430, 0x43a, 0x430, 0x43e, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d,
0x41c, 0x430, 0x43a, 0x430, 0x441, 0x430, 0x440, 0x41c, 0x430, 0x43d, 0x438, 0x43b,
0x430, 0x41c, 0x430, 0x441, 0x43a, 0x430, 0x442, 0x41d, 0x438, 0x43a, 0x43e, 0x441,
0x438, 0x430, 0x41d, 0x43e, 0x432, 0x43e, 0x43a, 0x443, 0x437, 0x43d, 0x435, 0x446,
0x43a, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a,
0x41e, 0x43c, 0x441, 0x43a, 0x423, 0x440, 0x430, 0x43b, 0x441, 0x43a, 0x41f, 0x43d,
0x43e, 0x43c, 0x43f, 0x435, 0x43d, 0x41f, 0x43e, 0x43d, 0x442, 0x438, 0x430, 0x43d,
0x430, 0x43a, 0x41f, 0x445, 0x435, 0x43d, 0x438, 0x430, 0x43d, 0x41a, 0x430, 0x442,
0x430, 0x440, 0x41a, 0x43e, 0x441, 0x442, 0x430, 0x43d, 0x430, 0x438, 0x41a, 0x44b,
0x437, 0x44b, 0x43b, 0x43e, 0x440, 0x434, 0x430, 0x415, 0x440, 0x2d, 0x420, 0x438,
0x430, 0x434, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x20, 0x421, 0x430, 0x445, 0x430,
0x43b, 0x438, 0x43d, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x43a, 0x430, 0x43d, 0x434,
0x421, 0x435, 0x443, 0x43b, 0x428, 0x430, 0x43d, 0x445, 0x430, 0x438, 0x421, 0x438,
0x43d, 0x433, 0x430, 0x43f, 0x443, 0x440, 0x421, 0x440, 0x435, 0x434, 0x43d, 0x435,
0x43a, 0x43e, 0x43b, 0x44b, 0x43c, 0x441, 0x43a, 0x422, 0x430, 0x438, 0x431, 0x435,
0x438, 0x422, 0x430, 0x448, 0x43a, 0x435, 0x43d, 0x442, 0x49a, 0x430, 0x440, 0x4ad,
0x422, 0x435, 0x433, 0x435, 0x440, 0x430, 0x43d, 0x422, 0x445, 0x438, 0x43c, 0x43f,
0x445, 0x443, 0x422, 0x43e, 0x43a, 0x438, 0x43e, 0x422, 0x43e, 0x43c, 0x441, 0x43a,
0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x423, 0x440,
0x443, 0x43c, 0x447, 0x438, 0x423, 0x441, 0x442, 0x2d, 0x41d, 0x435, 0x440, 0x430,
0x412, 0x435, 0x43d, 0x442, 0x438, 0x430, 0x43d, 0x412, 0x43b, 0x430, 0x434, 0x438,
0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x418, 0x430, 0x43a, 0x443, 0x442, 0x441,
0x43a, 0x418, 0x430, 0x43d, 0x433, 0x43e, 0x43d, 0x415, 0x43a, 0x430, 0x442, 0x435,
0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x415, 0x440, 0x435, 0x432, 0x430,
0x43d, 0x410, 0x437, 0x43e, 0x440, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x434, 0x2d,
0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x411, 0x435, 0x440, 0x43c, 0x443, 0x434, 0x442,
0x4d9, 0x438, 0x20, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x41a,
0x430, 0x43d, 0x430, 0x440, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x434, 0x2d, 0x445,
0x430, 0x49b, 0x4d9, 0x430, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440,
0x434, 0x435, 0x424, 0x430, 0x440, 0x435, 0x440, 0x442, 0x4d9, 0x438, 0x20, 0x430,
0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x41c, 0x430, 0x434, 0x435, 0x438,
0x440, 0x430, 0x420, 0x435, 0x438, 0x43a, 0x438, 0x430, 0x432, 0x438, 0x43a, 0x410,
0x43b, 0x430, 0x434, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x413, 0x435, 0x43e, 0x440,
0x433, 0x438, 0x430, 0x418, 0x525, 0x448, 0x44c, 0x43e, 0x443, 0x20, 0x415, 0x43b,
0x435, 0x43d, 0x430, 0x20, 0x43b, 0x430, 0x434, 0x433, 0x44c, 0x44b, 0x43b, 0x431,
0x436, 0x44c, 0x430, 0x445, 0x430, 0x421, 0x442, 0x435, 0x43d, 0x43b, 0x438, 0x410,
0x434, 0x435, 0x43b, 0x430, 0x438, 0x434, 0x430, 0x411, 0x440, 0x438, 0x441, 0x431,
0x435, 0x43d, 0x411, 0x440, 0x43e, 0x43a, 0x435, 0x43d, 0x2d, 0x425, 0x438, 0x43b,
0x43b, 0x414, 0x430, 0x440, 0x432, 0x438, 0x43d, 0x418, 0x443, 0x43a, 0x43b, 0x430,
0x425, 0x43e, 0x431, 0x430, 0x440, 0x442, 0x41b, 0x438, 0x43d, 0x434, 0x435, 0x43c,
0x430, 0x43d, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x425, 0x430, 0x443, 0x41c, 0x435,
0x43b, 0x431, 0x443, 0x440, 0x43d, 0x41f, 0x435, 0x440, 0x442, 0x421, 0x438, 0x434,
0x43d, 0x435, 0x438, 0x410, 0x43c, 0x441, 0x442, 0x435, 0x440, 0x434, 0x430, 0x43c,
0x410, 0x43d, 0x434, 0x43e, 0x440, 0x440, 0x430, 0x410, 0x441, 0x442, 0x440, 0x430,
0x445, 0x430, 0x43d, 0x410, 0x444, 0x438, 0x43d, 0x411, 0x435, 0x43b, 0x433, 0x440,
0x430, 0x434, 0x411, 0x435, 0x440, 0x43b, 0x438, 0x43d, 0x411, 0x440, 0x430, 0x442,
0x438, 0x441, 0x43b, 0x430, 0x432, 0x430, 0x411, 0x440, 0x438, 0x443, 0x441, 0x441,
0x435, 0x43b, 0x411, 0x443, 0x445, 0x430, 0x440, 0x435, 0x441, 0x442, 0x411, 0x443,
0x434, 0x430, 0x43f, 0x435, 0x448, 0x442, 0x411, 0x438, 0x443, 0x437, 0x438, 0x43d,
0x433, 0x435, 0x43d, 0x2d, 0x425, 0x44b, 0x445, 0x44c, 0x442, 0x4d9, 0x438, 0x2d,
0x420, 0x435, 0x438, 0x43d, 0x435, 0x41a, 0x438, 0x448, 0x438, 0x43d, 0x435, 0x432,
0x41a, 0x43e, 0x43f, 0x435, 0x43d, 0x4b3, 0x430, 0x433, 0x435, 0x43d, 0x414, 0x443,
0x431, 0x43b, 0x438, 0x43d, 0x413, 0x438, 0x431, 0x440, 0x430, 0x43b, 0x442, 0x430,
0x440, 0x413, 0x435, 0x440, 0x43d, 0x441, 0x438, 0x425, 0x435, 0x43b, 0x441, 0x438,
0x43d, 0x43a, 0x438, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x20, 0x41c, 0x435, 0x43d,
0x421, 0x442, 0x430, 0x43c, 0x431, 0x443, 0x43b, 0x40f, 0x435, 0x440, 0x441, 0x438,
0x41a, 0x430, 0x43b, 0x438, 0x43d, 0x438, 0x43d, 0x433, 0x440, 0x430, 0x434, 0x41a,
0x438, 0x440, 0x43e, 0x432, 0x41a, 0x438, 0x435, 0x432, 0x41b, 0x438, 0x441, 0x441,
0x430, 0x431, 0x43e, 0x43d, 0x41b, 0x438, 0x443, 0x431, 0x43b, 0x438, 0x430, 0x43d,
0x430, 0x41b, 0x43e, 0x43d, 0x434, 0x43e, 0x43d, 0x41b, 0x438, 0x443, 0x43a, 0x441,
0x435, 0x43c, 0x431, 0x443, 0x440, 0x433, 0x41c, 0x430, 0x434, 0x440, 0x438, 0x434,
0x41c, 0x430, 0x43b, 0x442, 0x430, 0x41c, 0x430, 0x440, 0x438, 0x435, 0x445, 0x430,
0x43c, 0x43d, 0x41c, 0x438, 0x43d, 0x441, 0x43a, 0x41c, 0x43e, 0x43d, 0x430, 0x43a,
0x43e, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x41e, 0x441, 0x43b, 0x43e, 0x41f,
0x430, 0x440, 0x438, 0x436, 0x41f, 0x43e, 0x434, 0x433, 0x43e, 0x440, 0x438, 0x446,
0x430, 0x41f, 0x440, 0x430, 0x433, 0x430, 0x420, 0x438, 0x433, 0x430, 0x420, 0x438,
0x43c, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x430, 0x421, 0x430, 0x43d, 0x2d, 0x41c,
0x430, 0x440, 0x438, 0x43d, 0x43e, 0x421, 0x430, 0x440, 0x430, 0x435, 0x432, 0x43e,
0x421, 0x430, 0x440, 0x430, 0x442, 0x43e, 0x432, 0x421, 0x438, 0x43c, 0x444, 0x435,
0x440, 0x43e, 0x43f, 0x43e, 0x43b, 0x421, 0x43a, 0x43e, 0x43f, 0x435, 0x421, 0x43e,
0x444, 0x438, 0x430, 0x421, 0x442, 0x43e, 0x43a, 0x4b3, 0x43e, 0x43b, 0x43c, 0x422,
0x430, 0x43b, 0x43b, 0x438, 0x43d, 0x422, 0x438, 0x440, 0x430, 0x43d, 0x430, 0x423,
0x43b, 0x438, 0x430, 0x43d, 0x43e, 0x432, 0x441, 0x43a, 0x412, 0x430, 0x434, 0x443,
0x446, 0x412, 0x430, 0x442, 0x438, 0x43a, 0x430, 0x43d, 0x412, 0x435, 0x43d, 0x430,
0x412, 0x438, 0x43b, 0x43d, 0x438, 0x443, 0x441, 0x412, 0x43e, 0x43b, 0x433, 0x43e,
0x433, 0x440, 0x430, 0x434, 0x412, 0x430, 0x440, 0x448, 0x430, 0x432, 0x430, 0x417,
0x430, 0x433, 0x440, 0x435, 0x431, 0x426, 0x438, 0x443, 0x440, 0x438, 0x445, 0x410,
0x43d, 0x442, 0x430, 0x43d, 0x430, 0x43d, 0x430, 0x440, 0x438, 0x432, 0x443, 0x427,
0x430, 0x433, 0x43e, 0x441, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x20, 0x49a, 0x44c,
0x44b, 0x440, 0x441, 0x430, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x442, 0x4d9, 0x438,
0x20, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x41a, 0x43e, 0x43c,
0x43e, 0x440, 0x41a, 0x435, 0x440, 0x433, 0x435, 0x43b, 0x435, 0x43d, 0x41c, 0x430,
0x435, 0x41c, 0x430, 0x43b, 0x434, 0x438, 0x432, 0x41c, 0x430, 0x432, 0x440, 0x438,
0x43a, 0x438, 0x41c, 0x430, 0x438, 0x43e, 0x442, 0x442, 0x430, 0x420, 0x435, 0x438,
0x443, 0x43d, 0x43e, 0x43d, 0x410, 0x43f, 0x438, 0x430, 0x41e, 0x43a, 0x43b, 0x435,
0x43d, 0x434, 0x411, 0x443, 0x433, 0x435, 0x43d, 0x432, 0x438, 0x43b, 0x427, 0x430,
0x442, 0x435, 0x43c, 0x422, 0x440, 0x443, 0x43a, 0x430, 0x434, 0x2d, 0x445, 0x430,
0x20, 0x41f, 0x430, 0x441, 0x445, 0x438, 0x415, 0x444, 0x430, 0x442, 0x435, 0x424,
0x430, 0x43a, 0x430, 0x43e, 0x444, 0x43e, 0x424, 0x438, 0x45f, 0x438, 0x424, 0x443,
0x43d, 0x430, 0x444, 0x443, 0x442, 0x438, 0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430,
0x433, 0x43e, 0x441, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x434, 0x2d, 0x445, 0x430,
0x49b, 0x4d9, 0x430, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x20,
0x413, 0x430, 0x43c, 0x431, 0x435, 0x413, 0x443, 0x430, 0x434, 0x430, 0x43b, 0x43a,
0x430, 0x43d, 0x430, 0x43b, 0x413, 0x443, 0x430, 0x43c, 0x41a, 0x430, 0x43d, 0x442,
0x43e, 0x43d, 0x41a, 0x438, 0x440, 0x438, 0x442, 0x438, 0x43c, 0x430, 0x442, 0x438,
0x41a, 0x43e, 0x441, 0x440, 0x430, 0x435, 0x41a, 0x432, 0x430, 0x45f, 0x430, 0x43b,
0x435, 0x438, 0x43d, 0x41c, 0x430, 0x45f, 0x443, 0x440, 0x43e, 0x41c, 0x430, 0x440,
0x43a, 0x438, 0x437, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x434, 0x2d, 0x445, 0x430,
0x49b, 0x4d9, 0x430, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x20,
0x41c, 0x438, 0x434, 0x443, 0x435, 0x438, 0x41d, 0x430, 0x443, 0x440, 0x443, 0x41d,
0x438, 0x443, 0x435, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x41d, 0x443,
0x43c, 0x435, 0x430, 0x41f, 0x430, 0x433, 0x43e, 0x2d, 0x41f, 0x430, 0x433, 0x43e,
0x41f, 0x430, 0x43b, 0x430, 0x443, 0x41f, 0x438, 0x442, 0x43a, 0x435, 0x440, 0x43d,
0x41f, 0x43e, 0x43d, 0x43f, 0x435, 0x438, 0x41f, 0x43e, 0x440, 0x442, 0x2d, 0x41c,
0x43e, 0x440, 0x441, 0x431, 0x438, 0x420, 0x430, 0x440, 0x43e, 0x442, 0x43e, 0x43d,
0x433, 0x430, 0x421, 0x430, 0x438, 0x43f, 0x430, 0x43d, 0x422, 0x430, 0x438, 0x442,
0x438, 0x422, 0x430, 0x440, 0x430, 0x432, 0x430, 0x422, 0x43e, 0x43d, 0x433, 0x430,
0x442, 0x430, 0x43f, 0x443, 0x423, 0x435, 0x438, 0x43a, 0x423, 0x43e, 0x43b, 0x43b,
0x438, 0x441, 0x41, 0x64, 0x64, 0x69, 0x73, 0x20, 0x41, 0x62, 0x65, 0x62,
0x61, 0x4b, 0x61, 0xef, 0x72, 0x6f, 0x44, 0x6a, 0x69, 0x62, 0x6f, 0x65,
0x74, 0x69, 0x4b, 0x61, 0x72, 0x74, 0x6f, 0x65, 0x6d, 0x4d, 0x6f, 0x67,
0x61, 0x64, 0x69, 0x73, 0x6a, 0x6f, 0x65, 0x42, 0x61, 0x68, 0x69, 0x61,
0x20, 0x42, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x61, 0x73, 0x43, 0x61, 0x6d,
0x62, 0x72, 0x69, 0x64, 0x67, 0x65, 0x62, 0x61, 0x61, 0x69, 0x4b, 0x61,
0x61, 0x69, 0x6d, 0x61, 0x6e, 0x47, 0x6c, 0x61, 0x63, 0x65, 0x62, 0x61,
0x61, 0x69, 0x47, 0x6f, 0x6f, 0x73, 0x65, 0x62, 0x61, 0x61, 0x69, 0x4a,
0x61, 0x6d, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b,
0x6f, 0x73, 0x74, 0x61, 0x64, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c,
0x20, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74,
0x61, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x4e, 0x6f, 0x6f,
0x72, 0x64, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77,
0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x4e, 0x6f, 0x6f, 0x72,
0x64, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x53, 0x69, 0x6e, 0x74,
0x20, 0x42, 0x61, 0x72, 0x74, 0x68, 0x6f, 0x6c, 0x6f, 0x6d, 0x65, 0x75,
0x73, 0x53, 0x69, 0x6e, 0x74, 0x20, 0x4a, 0x6f, 0x68, 0x6e, 0x2019, 0x73,
0x53, 0x69, 0x6e, 0x74, 0x20, 0x4b, 0x69, 0x74, 0x74, 0x73, 0x53, 0x69,
0x6e, 0x74, 0x20, 0x4c, 0x75, 0x63, 0x69, 0x61, 0x53, 0x69, 0x6e, 0x74,
0x20, 0x54, 0x68, 0x6f, 0x6d, 0x61, 0x73, 0x53, 0x69, 0x6e, 0x74, 0x20,
0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x74, 0x57, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x41, 0x73, 0x6a, 0x63, 0x68, 0x61, 0x62, 0x61, 0x64, 0x42, 0x61,
0x67, 0x64, 0x61, 0x64, 0x42, 0x61, 0x68, 0x72, 0x65, 0x69, 0x6e, 0x42,
0x61, 0x6b, 0x6f, 0x65, 0x42, 0x65, 0x69, 0x72, 0x6f, 0x65, 0x74, 0x42,
0x69, 0x73, 0x6a, 0x6b, 0x65, 0x6b, 0x42, 0x72, 0x6f, 0x65, 0x6e, 0x65,
0x69, 0x44, 0x61, 0x6d, 0x61, 0x73, 0x6b, 0x75, 0x73, 0x44, 0x6f, 0x65,
0x62, 0x61, 0x69, 0x48, 0x6f, 0x20, 0x54, 0x73, 0x6a, 0x69, 0x20, 0x4d,
0x69, 0x6e, 0x68, 0x73, 0x74, 0x61, 0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b,
0x6f, 0x6e, 0x67, 0x49, 0x72, 0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b, 0x44,
0x6a, 0x61, 0x6b, 0x61, 0x72, 0x74, 0x61, 0x4b, 0x61, 0x62, 0x6f, 0x65,
0x6c, 0x4b, 0x61, 0x6d, 0x74, 0x73, 0x6a, 0x61, 0x74, 0x6b, 0x61, 0x4b,
0x61, 0x72, 0x61, 0x74, 0x73, 0x6a, 0x69, 0x4b, 0x61, 0x74, 0x6d, 0x61,
0x6e, 0x64, 0x6f, 0x65, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61,
0x72, 0x73, 0x6b, 0x4b, 0x6f, 0x65, 0x61, 0x6c, 0x61, 0x2d, 0x4c, 0x6f,
0x65, 0x6d, 0x70, 0x6f, 0x65, 0x72, 0x4b, 0x6f, 0x65, 0x77, 0x65, 0x69,
0x74, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x4d, 0x75, 0x73, 0x6b, 0x61, 0x74,
0x4b, 0x61, 0x74, 0x61, 0x72, 0x52, 0x69, 0x61, 0x61, 0x64, 0x53, 0x65,
0x6f, 0x65, 0x6c, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x65, 0x72,
0x54, 0x6f, 0x6b, 0x69, 0x6f, 0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x4a, 0x61, 0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b,
0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x41, 0x73, 0x6f, 0x72, 0x65, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x69,
0x65, 0x4b, 0x61, 0x61, 0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x53,
0x75, 0x69, 0x64, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0xeb, 0x53,
0x69, 0x6e, 0x74, 0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x41, 0x74,
0x68, 0x65, 0x6e, 0x65, 0x42, 0x65, 0x72, 0x6c, 0x79, 0x6e, 0x42, 0x72,
0x75, 0x73, 0x73, 0x65, 0x6c, 0x42, 0x6f, 0x65, 0x6b, 0x61, 0x72, 0x65,
0x73, 0x74, 0x42, 0x6f, 0x65, 0x64, 0x61, 0x70, 0x65, 0x73, 0x74, 0x4b,
0x6f, 0x70, 0x65, 0x6e, 0x68, 0x61, 0x67, 0x65, 0x6e, 0x45, 0x69, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x4d, 0x61, 0x6e, 0x4b, 0x69, 0xeb, 0x66, 0x4c,
0x69, 0x73, 0x73, 0x61, 0x62, 0x6f, 0x6e, 0x4c, 0x6f, 0x6e, 0x64, 0x65,
0x6e, 0x4c, 0x75, 0x78, 0x65, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x4d, 0x6f,
0x73, 0x6b, 0x6f, 0x75, 0x50, 0x61, 0x72, 0x79, 0x73, 0x50, 0x72, 0x61,
0x61, 0x67, 0x53, 0x61, 0x72, 0x61, 0x74, 0x6f, 0x66, 0x56, 0x61, 0x74,
0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x74, 0x61, 0x64, 0x57, 0x65, 0x6e,
0x65, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x57, 0x61,
0x72, 0x73, 0x6b, 0x6f, 0x75, 0x5a, 0xfc, 0x72, 0x69, 0x63, 0x68, 0x4b,
0x6f, 0x6b, 0x6f, 0x73, 0x43, 0x6f, 0x6d, 0x6f, 0x72, 0x65, 0x4d, 0x61,
0x6c, 0x65, 0x64, 0x69, 0x76, 0x65, 0x50, 0x61, 0x61, 0x73, 0x46, 0x69,
0x64, 0x6a, 0x69, 0x48, 0x6f, 0x6e, 0x6f, 0x6c, 0x75, 0x6c, 0x75, 0x45,
0x6e, 0x64, 0x65, 0x72, 0x62, 0x75, 0x72, 0x79, 0x4e, 0x6f, 0x75, 0x6d,
0xe9, 0x61, 0x4d, 0x61, 0x74, 0x61, 0x2d, 0x55, 0x74, 0x75, 0x41, 0x62,
0x69, 0x67, 0x79, 0x61, 0x6e, 0x41, 0x6b, 0x72, 0x61, 0x61, 0x186, 0x6c,
0x67, 0x79, 0x65, 0x73, 0x65, 0x42, 0x61, 0x6e, 0x64, 0x77, 0x75, 0x75,
0x42, 0x69, 0x73, 0x61, 0x77, 0x42, 0x6c, 0x61, 0x6e, 0x74, 0x61, 0x69,
0x42, 0x75, 0x64, 0x77, 0x75, 0x6d, 0x62, 0x75, 0x72, 0x61, 0x4b, 0x61,
0x69, 0x72, 0x6f, 0x4b, 0x61, 0x73, 0x61, 0x62, 0x6c, 0x61, 0x6e, 0x6b,
0x61, 0x4b, 0x79, 0x75, 0x74, 0x61, 0x4b, 0x254, 0x6e, 0x61, 0x6b, 0x72,
0x69, 0x44, 0x61, 0x6b, 0x61, 0x61, 0x47, 0x79, 0x69, 0x62, 0x75, 0x75,
0x74, 0x69, 0x44, 0x77, 0x75, 0x62, 0x61, 0x4b, 0x69, 0x6e, 0x68, 0x79,
0x61, 0x61, 0x68, 0x79, 0x61, 0x4c, 0x65, 0x67, 0x254, 0x73, 0x4e, 0x67,
0x79, 0x61, 0x6d, 0x65, 0x6e, 0x61, 0x57, 0x61, 0x67, 0x61, 0x64, 0x75,
0x67, 0x75, 0x41, 0x6e, 0x6b, 0x254, 0x72, 0x61, 0x67, 0x79, 0x69, 0x41,
0x6e, 0x67, 0x75, 0x69, 0x6c, 0x61, 0x4b, 0x61, 0x74, 0x61, 0x6d, 0x61,
0x61, 0x6b, 0x61, 0x4b, 0x254, 0x254, 0x64, 0x254, 0x62, 0x61, 0x44, 0x77,
0x75, 0x64, 0x77, 0x75, 0x69, 0x4c, 0x61, 0x20, 0x52, 0x69, 0x6f, 0x67,
0x79, 0x61, 0x4d, 0x25b, 0x6e, 0x64, 0x254, 0x73, 0x61, 0x52, 0x69, 0x6f,
0x20, 0x47, 0x61, 0x6c, 0x6c, 0x65, 0x67, 0x254, 0x73, 0x53, 0x61, 0x6e,
0x20, 0x44, 0x77, 0x75, 0x61, 0x6e, 0x54, 0x75, 0x6b, 0x75, 0x6d, 0x61,
0x6e, 0x42, 0x61, 0x61, 0x62, 0x61, 0x64, 0x6f, 0x73, 0x42, 0x25b, 0x6c,
0x25b, 0x6d, 0x42, 0x25b, 0x6c, 0x69, 0x73, 0x69, 0x42, 0x6c, 0x61, 0x6e,
0x63, 0x2d, 0x53, 0x61, 0x62, 0x6c, 0x254, 0x6e, 0x42, 0x254, 0x69, 0x73,
0x69, 0x4b, 0x61, 0x6d, 0x62, 0x72, 0x69, 0x67, 0x79, 0x69, 0x20, 0x42,
0x61, 0x79, 0x4b, 0x61, 0x6d, 0x70, 0x6f, 0x20, 0x47, 0x72, 0x61, 0x6e,
0x64, 0x65, 0x4b, 0x61, 0x72, 0x61, 0x6b, 0x61, 0x73, 0x4b, 0x61, 0x79,
0x69, 0x69, 0x6e, 0x69, 0x4b, 0x65, 0x6d, 0x61, 0x6e, 0x66, 0x6f, 0x4b,
0x79, 0x69, 0x6b, 0x61, 0x67, 0x6f, 0x4b, 0x79, 0x69, 0x68, 0x75, 0x61,
0x68, 0x75, 0x61, 0x4b, 0x254, 0x73, 0x74, 0x61, 0x20, 0x52, 0x69, 0x6b,
0x61, 0x4b, 0x72, 0x25b, 0x73, 0x74, 0x6f, 0x6e, 0x4b, 0x75, 0x69, 0x61,
0x62, 0x61, 0x4b, 0x75, 0x72, 0x75, 0x6b, 0x61, 0x77, 0x44, 0x254, 0x254,
0x73, 0x6f, 0x6e, 0x44, 0x254, 0x254, 0x73, 0x6f, 0x6e, 0x20, 0x4b, 0x72,
0x65, 0x65, 0x6b, 0x44, 0x25b, 0x6e, 0x76, 0x61, 0x44, 0x65, 0x74, 0x72,
0x254, 0x69, 0x74, 0x44, 0x254, 0x6d, 0x65, 0x6e, 0x65, 0x6b, 0x61, 0x45,
0x64, 0x6d, 0x254, 0x6e, 0x74, 0x6f, 0x6e, 0x45, 0x6c, 0x20, 0x53, 0x61,
0x6c, 0x76, 0x61, 0x64, 0x254, 0x254, 0x46, 0x254, 0x74, 0x20, 0x4e, 0x25b,
0x6c, 0x73, 0x6f, 0x6e, 0x46, 0x254, 0x254, 0x74, 0x61, 0x6c, 0x25b, 0x73,
0x61, 0x47, 0x75, 0x75, 0x73, 0x20, 0x42, 0x61, 0x79, 0x47, 0x72, 0x61,
0x6e, 0x64, 0x20, 0x54, 0x75, 0x6b, 0x47, 0x75, 0x77, 0x61, 0x64, 0x65,
0x6c, 0x75, 0x70, 0x47, 0x75, 0x77, 0x61, 0x74, 0x65, 0x6d, 0x61, 0x6c,
0x61, 0x47, 0x61, 0x79, 0x61, 0x6b, 0x77, 0x75, 0x69, 0x6c, 0x47, 0x61,
0x79, 0x61, 0x6e, 0x61, 0x48, 0x25b, 0x6d, 0x254, 0x73, 0x69, 0x6c, 0x6f,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x70, 0x254, 0x6c, 0x69, 0x73,
0x50, 0x69, 0x74, 0x25b, 0x73, 0x62, 0x25b, 0x67, 0x79, 0x65, 0x2c, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x54, 0x65, 0x6c, 0x6c, 0x20,
0x53, 0x69, 0x74, 0x69, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e,
0x61, 0x57, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x6b, 0x2c, 0x20, 0x49, 0x6e,
0x64, 0x69, 0x61, 0x6e, 0x61, 0x49, 0x6b, 0x61, 0x6c, 0x75, 0x69, 0x74,
0x47, 0x79, 0x61, 0x6d, 0x65, 0x6b, 0x61, 0x4d, 0x254, 0x6e, 0x74, 0x69,
0x73, 0x25b, 0x6c, 0x6f, 0x2c, 0x20, 0x4b, 0x25b, 0x6e, 0x74, 0x25b, 0x6b,
0x69, 0x4b, 0x72, 0x61, 0x6c, 0x25b, 0x6e, 0x67, 0x79, 0x69, 0x6b, 0x4c,
0x254, 0x73, 0x20, 0x41, 0x6e, 0x67, 0x79, 0x25b, 0x6c, 0x69, 0x73, 0x4c,
0x6f, 0x77, 0x61, 0x20, 0x50, 0x72, 0x69, 0x6e, 0x73, 0x65, 0x20, 0x4b,
0x254, 0x74, 0x61, 0x4d, 0x61, 0x6b, 0x65, 0x69, 0x6f, 0x4d, 0x61, 0x72,
0x69, 0x67, 0x254, 0x74, 0x4d, 0x61, 0x72, 0x74, 0x69, 0x6e, 0x69, 0x6b,
0x65, 0x4d, 0x61, 0x74, 0x61, 0x6d, 0x254, 0x72, 0x254, 0x73, 0x6f, 0x4d,
0x61, 0x73, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x4d, 0x25b, 0x6e, 0x254, 0x6d,
0x69, 0x6e, 0x65, 0x65, 0x4d, 0x25b, 0x74, 0x6c, 0x61, 0x6b, 0x61, 0x74,
0x6c, 0x61, 0x4d, 0x25b, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x20, 0x53, 0x69,
0x74, 0x69, 0x4d, 0x254, 0x6e, 0x6b, 0x74, 0x69, 0x6e, 0x4d, 0x254, 0x6e,
0x74, 0x69, 0x72, 0x69, 0x69, 0x4d, 0x61, 0x6e, 0x74, 0x73, 0x65, 0x72,
0x61, 0x74, 0x4e, 0x65, 0x77, 0x20, 0x59, 0x254, 0x6b, 0x42, 0x65, 0x75,
0x6c, 0x61, 0x2c, 0x20, 0x4e, 0x254, 0x66, 0x20, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x53, 0x25b, 0x6e, 0x74, 0x61, 0x2c, 0x20, 0x4e, 0x254, 0x66,
0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53,
0x61, 0x6c, 0x25b, 0x6d, 0x2c, 0x20, 0x4e, 0x254, 0x66, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x4f, 0x67, 0x79, 0x69, 0x6e, 0x61, 0x67, 0x61,
0x50, 0x61, 0x72, 0x61, 0x6d, 0x61, 0x72, 0x69, 0x62, 0x254, 0x46, 0x69,
0x6e, 0x69, 0x73, 0x6b, 0x53, 0x70, 0x61, 0x69, 0x6e, 0x20, 0x50, 0x254,
0x254, 0x74, 0x6f, 0x50, 0x254, 0x254, 0x74, 0x6f, 0x20, 0x56, 0x65, 0x6c,
0x68, 0x6f, 0x50, 0x75, 0x25b, 0x74, 0x6f, 0x20, 0x52, 0x69, 0x6b, 0x6f,
0x52, 0x25b, 0x73, 0x69, 0x66, 0x25b, 0x52, 0x25b, 0x67, 0x79, 0x69, 0x6e,
0x61, 0x52, 0x25b, 0x73, 0x254, 0x6c, 0x75, 0x74, 0x52, 0x69, 0x6f, 0x20,
0x42, 0x72, 0x61, 0x6e, 0x6b, 0x6f, 0x59, 0x69, 0x74, 0x6f, 0x6b, 0x254,
0x74, 0x75, 0x254, 0x6d, 0x65, 0x74, 0x65, 0x53, 0x74, 0x2e, 0x20, 0x42,
0x61, 0x61, 0x74, 0x69, 0x6c, 0x65, 0x6d, 0x69, 0x53, 0x77, 0x69, 0x66,
0x74, 0x20, 0x4b, 0x25b, 0x72, 0x25b, 0x6e, 0x74, 0x54, 0x65, 0x67, 0x75,
0x73, 0x69, 0x67, 0x61, 0x6c, 0x70, 0x61, 0x54, 0x69, 0x64, 0x77, 0x75,
0x61, 0x6e, 0x61, 0x54, 0x254, 0x254, 0x74, 0x6f, 0x6c, 0x61, 0x57, 0x69,
0x6e, 0x69, 0x70, 0x25b, 0x67, 0x4b, 0x61, 0x73, 0x69, 0x4d, 0x61, 0x6b,
0x61, 0x61, 0x72, 0x69, 0x4d, 0x254, 0x254, 0x73, 0x6f, 0x6e, 0x50, 0x61,
0x61, 0x6d, 0x61, 0x52, 0x6f, 0x74, 0x65, 0x72, 0x61, 0x54, 0x72, 0x254,
0x254, 0x56, 0x254, 0x73, 0x74, 0x254, 0x6b, 0x41, 0x61, 0x6d, 0x61, 0x74,
0x69, 0x41, 0x6d, 0x61, 0x6e, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x41, 0x6b,
0x74, 0x6f, 0x70, 0x25b, 0x42, 0x61, 0x6e, 0x6b, 0x254, 0x6b, 0x42, 0x25b,
0x25b, 0x72, 0x75, 0x74, 0x4b, 0x79, 0x69, 0x74, 0x61, 0x4b, 0x6f, 0x6c,
0x6f, 0x6d, 0x62, 0x6f, 0x44, 0x61, 0x6d, 0x61, 0x73, 0x6b, 0x254, 0x73,
0x6f, 0x48, 0x25b, 0x62, 0x72, 0x254, 0x6e, 0x48, 0x6f, 0x20, 0x4b, 0x79,
0x69, 0x20, 0x4d, 0x69, 0x6e, 0x47, 0x79, 0x61, 0x6b, 0x61, 0x61, 0x74,
0x61, 0x47, 0x79, 0x61, 0x79, 0x61, 0x70, 0x75, 0x72, 0x61, 0x59, 0x65,
0x72, 0x75, 0x73, 0x61, 0x6c, 0x65, 0x6d, 0x4b, 0x61, 0x6d, 0x6b, 0x79,
0x61, 0x74, 0x6b, 0x61, 0x4b, 0x61, 0x72, 0x61, 0x6b, 0x79, 0x69, 0x4b,
0x61, 0x74, 0x6d, 0x61, 0x6e, 0x64, 0x75, 0x4b, 0x254, 0x254, 0x6b, 0x61,
0x74, 0x61, 0x4b, 0x75, 0x6b, 0x79, 0x69, 0x6e, 0x4d, 0x61, 0x6b, 0x61,
0x73, 0x61, 0x4e, 0x69, 0x6b, 0x6f, 0x73, 0x69, 0x61, 0x53, 0x69, 0x6e,
0x67, 0x61, 0x70, 0x254, 0x54, 0x25b, 0x25b, 0x72, 0x61, 0x6e, 0x54, 0x69,
0x6d, 0x70, 0x68, 0x75, 0x59, 0x75, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74,
0x61, 0x59, 0x75, 0x72, 0x79, 0x6d, 0x6b, 0x69, 0x56, 0x69, 0x65, 0x6e,
0x68, 0x79, 0x69, 0x61, 0x6e, 0x65, 0x59, 0x25b, 0x6b, 0x61, 0x74, 0x25b,
0x72, 0x69, 0x6e, 0x62, 0x25b, 0x67, 0x42, 0x25b, 0x6d, 0x75, 0x64, 0x61,
0x4b, 0x65, 0x70, 0x76, 0x25b, 0x64, 0x65, 0x52, 0x25b, 0x25b, 0x6b, 0x67,
0x79, 0x61, 0x76, 0x69, 0x6b, 0x47, 0x79, 0x254, 0x254, 0x67, 0x79, 0x69,
0x61, 0x20, 0x41, 0x6e, 0x61, 0x61, 0x66, 0x6f, 0x254, 0x53, 0x74, 0x61,
0x6e, 0x6c, 0x69, 0x42, 0x72, 0x254, 0x6b, 0x65, 0x6e, 0x20, 0x48, 0x69,
0x6c, 0x6c, 0x44, 0x61, 0x61, 0x77, 0x69, 0x6e, 0x45, 0x75, 0x6b, 0x6c,
0x61, 0x48, 0x254, 0x62, 0x61, 0x74, 0x4c, 0x254, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x4d, 0x25b, 0x25b, 0x62, 0x254, 0x6e, 0x50, 0x25b, 0x25b, 0x74,
0x53, 0x69, 0x64, 0x6e, 0x69, 0x41, 0x6d, 0x73, 0x74, 0x61, 0x64, 0x61,
0x6d, 0x41, 0x6e, 0x64, 0x254, 0x72, 0x61, 0x41, 0x74, 0x65, 0x6e, 0x65,
0x42, 0x25b, 0x6c, 0x67, 0x72, 0x61, 0x64, 0x65, 0x42, 0x25b, 0x25b, 0x6c,
0x69, 0x6e, 0x42, 0x72, 0x25b, 0x73, 0x25b, 0x6c, 0x73, 0x65, 0x42, 0x75,
0x6b, 0x79, 0x61, 0x72, 0x25b, 0x73, 0x74, 0x42, 0x75, 0x64, 0x61, 0x70,
0x25b, 0x73, 0x68, 0x42, 0x75, 0x73, 0x69, 0x6e, 0x67, 0x79, 0x65, 0x4b,
0x79, 0x69, 0x73, 0x69, 0x6e, 0x61, 0x75, 0x4b, 0x254, 0x70, 0x25b, 0x68,
0x61, 0x6e, 0x67, 0x25b, 0x6e, 0x65, 0x44, 0x254, 0x62, 0x6c, 0x69, 0x6e,
0x47, 0x79, 0x65, 0x62, 0x72, 0x6f, 0x74, 0x61, 0x48, 0x25b, 0x6c, 0x73,
0x69, 0x6e, 0x6b, 0x69, 0x4a, 0x25b, 0x25b, 0x73, 0x69, 0x4c, 0x69, 0x73,
0x62, 0x254, 0x6e, 0x4c, 0x64, 0x77, 0x75, 0x62, 0x64, 0x77, 0x61, 0x6e,
0x61, 0x4c, 0x254, 0x6e, 0x64, 0x254, 0x6e, 0x4c, 0x25b, 0x73, 0x65, 0x6d,
0x62, 0x25b, 0x67, 0x4d, 0x254, 0x254, 0x74, 0x61, 0x4d, 0x6f, 0x6e, 0x61,
0x6b, 0x6f, 0x4d, 0x254, 0x73, 0x6b, 0x6f, 0x50, 0x6f, 0x64, 0x67, 0x6f,
0x72, 0x69, 0x6b, 0x61, 0x52, 0x6f, 0x6d, 0x61, 0x53, 0x61, 0x72, 0x61,
0x67, 0x79, 0x65, 0x76, 0x6f, 0x53, 0x6b, 0x254, 0x70, 0x67, 0x79, 0x65,
0x53, 0x254, 0x66, 0x69, 0x61, 0x53, 0x74, 0x254, 0x6b, 0x68, 0x254, 0x6d,
0x56, 0x61, 0x74, 0x69, 0x6b, 0x61, 0x6e, 0x56, 0x65, 0x25b, 0x6e, 0x61,
0x57, 0x254, 0x254, 0x73, 0x254, 0x254, 0x5a, 0x75, 0x72, 0x65, 0x6b, 0x79,
0x65, 0x4b, 0x79, 0x61, 0x67, 0x254, 0x73, 0x42, 0x75, 0x72, 0x6f, 0x6e,
0x79, 0x61, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x6f, 0x4b, 0x254, 0x6d, 0x254,
0x72, 0x254, 0x4b, 0x25b, 0x67, 0x75, 0x65, 0x6c, 0x25b, 0x6e, 0x4d, 0x254,
0x72, 0x69, 0x68, 0x79, 0x69, 0x254, 0x73, 0x6f, 0x4d, 0x61, 0x79, 0x6f,
0x74, 0x65, 0x41, 0x75, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x4b, 0x79, 0x61,
0x74, 0x61, 0x6d, 0x4b, 0x79, 0x75, 0x75, 0x6b, 0x45, 0x61, 0x73, 0x74,
0x61, 0x46, 0x69, 0x67, 0x79, 0x69, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61,
0x67, 0x254, 0x73, 0x47, 0x75, 0x61, 0x64, 0x61, 0x61, 0x6b, 0x61, 0x6e,
0x61, 0x61, 0x4b, 0x77, 0x61, 0x67, 0x79, 0x61, 0x6c, 0x65, 0x65, 0x6e,
0x65, 0x4d, 0x61, 0x67, 0x79, 0x75, 0x72, 0x6f, 0x4d, 0x61, 0x61, 0x6b,
0x65, 0x73, 0x61, 0x73, 0x65, 0x4e, 0x254, 0x254, 0x66, 0x254, 0x6b, 0x50,
0x69, 0x74, 0x6b, 0x61, 0x69, 0x72, 0x6e, 0x65, 0x50, 0x254, 0x6e, 0x70,
0x65, 0x69, 0x50, 0x254, 0x74, 0x20, 0x4d, 0x6f, 0x72, 0x25b, 0x73, 0x62,
0x69, 0x41, 0x62, 0x69, 0x78, 0x68, 0x61, 0x6e, 0x41, 0x64, 0x69, 0x73,
0x2d, 0x41, 0x62, 0x61, 0x62, 0xeb, 0x41, 0x6c, 0x67, 0x6a, 0x65, 0x72,
0x41, 0x73, 0x6d, 0x61, 0x72, 0xeb, 0x42, 0x61, 0x6e, 0x78, 0x68, 0x75,
0x6c, 0x42, 0x69, 0x73, 0x61, 0x75, 0x42, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x72, 0xeb, 0x42, 0x72, 0x61, 0x7a, 0x61, 0x76, 0x69, 0x6c, 0x6c, 0xeb,
0x42, 0x75, 0x78, 0x68, 0x75, 0x6d, 0x62, 0x75, 0x72, 0xeb, 0x4b, 0x61,
0x6a, 0x72, 0x6f, 0x4b, 0x61, 0x7a, 0x61, 0x62, 0x6c, 0x61, 0x6e, 0x6b,
0xeb, 0x54, 0x68, 0x65, 0x75, 0x74, 0x61, 0x4b, 0x6f, 0x6e, 0x61, 0x6b,
0x72, 0x69, 0x44, 0x61, 0x72, 0x2d, 0x65, 0x73, 0x2d, 0x53, 0x61, 0x6c,
0x61, 0x6d, 0x58, 0x68, 0x69, 0x62, 0x75, 0x74, 0x69, 0x45, 0x6c, 0x20,
0x41, 0x6a, 0x75, 0x6e, 0x46, 0x72, 0x69, 0x74, 0x61, 0x75, 0x6e, 0x4a,
0x6f, 0x68, 0x61, 0x6e, 0x65, 0x73, 0x62, 0x75, 0x72, 0x67, 0x58, 0x68,
0x75, 0x62, 0x61, 0x4b, 0x61, 0x72, 0x74, 0x75, 0x6d, 0x4c, 0x69, 0x62,
0x72, 0x65, 0x76, 0x69, 0x6c, 0xeb, 0x4e, 0x69, 0x61, 0x6d, 0x65, 0x6a,
0x4e, 0x6f, 0x75, 0x61, 0x6b, 0x6f, 0x74, 0x55, 0x61, 0x67, 0xeb, 0x64,
0x75, 0x67, 0x75, 0x53, 0x61, 0x6f, 0x2d, 0x54, 0x6f, 0x6d, 0x65, 0x54,
0x75, 0x6e, 0x69, 0x7a, 0x56, 0x69, 0x6e, 0x74, 0x2019, 0x68, 0x75, 0x6b,
0x41, 0x6e, 0x6b, 0x6f, 0x72, 0x65, 0x6a, 0x78, 0x68, 0x41, 0x6e, 0x67,
0x75, 0x69, 0x6c, 0xeb, 0x41, 0x72, 0x61, 0x67, 0x75, 0x61, 0x6a, 0x61,
0x6e, 0x61, 0x42, 0x75, 0x65, 0x6e, 0x6f, 0x73, 0x2d, 0x41, 0x6a, 0x72,
0x65, 0x73, 0x4b, 0x61, 0x74, 0x61, 0x6d, 0x61, 0x72, 0x6b, 0x61, 0x4b,
0x6f, 0x72, 0x64, 0x6f, 0x62, 0x61, 0x48, 0x75, 0x68, 0x75, 0x69, 0x4c,
0x61, 0x20, 0x52, 0x69, 0x6f, 0x68, 0x61, 0x52, 0x69, 0x6f, 0x2d, 0x47,
0x61, 0x6c, 0x65, 0x67, 0x6f, 0x73, 0x53, 0x61, 0x6c, 0x74, 0xeb, 0x53,
0x61, 0x6e, 0x2d, 0x48, 0x75, 0x61, 0x6e, 0x53, 0x68, 0xeb, 0x6e, 0x2d,
0x4c, 0x75, 0x69, 0x73, 0x55, 0x73, 0x68, 0x75, 0x61, 0x6a, 0x61, 0x41,
0x72, 0x75, 0x62, 0xeb, 0x41, 0x73, 0x75, 0x6e, 0x73, 0x69, 0x6f, 0x6e,
0x42, 0x61, 0x68, 0x69, 0x61, 0x2d, 0x42, 0x61, 0x6e, 0x64, 0x65, 0x72,
0x61, 0x73, 0x42, 0x65, 0x6c, 0x69, 0x7a, 0xeb, 0x42, 0x6c, 0x61, 0x6e,
0x6b, 0x2d, 0x53, 0x61, 0x62, 0x6c, 0x6f, 0x6e, 0x42, 0x6f, 0x61, 0x2d,
0x56, 0x69, 0x73, 0x74, 0x61, 0x42, 0x6f, 0x67, 0x6f, 0x74, 0xeb, 0x42,
0x6f, 0x69, 0x7a, 0xeb, 0x47, 0x6a, 0x69, 0x72, 0x69, 0x20, 0x69, 0x20,
0x4b, 0x65, 0x6d, 0x62, 0x72, 0x69, 0x78, 0x68, 0x69, 0x74, 0x4b, 0x61,
0x6d, 0x70, 0x6f, 0x2d, 0x47, 0x72, 0x61, 0x6e, 0x64, 0x65, 0x4b, 0x61,
0x6e, 0x6b, 0x75, 0x6e, 0x4b, 0x61, 0x6a, 0x65, 0x6e, 0xeb, 0x4b, 0x61,
0x6a, 0x6d, 0x61, 0x6e, 0xc7, 0x69, 0x6b, 0x61, 0x67, 0x6f, 0xc7, 0x69,
0x68, 0x61, 0x68, 0x75, 0x61, 0x53, 0x69, 0x75, 0x64, 0x61, 0x64, 0x2d,
0x48, 0x75, 0x61, 0x72, 0x65, 0x7a, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x2d,
0x52, 0x69, 0x6b, 0xeb, 0x4b, 0x72, 0x65, 0x73, 0x74, 0x6f, 0x6e, 0x4b,
0x75, 0x6a, 0x61, 0x62, 0x61, 0x4b, 0x75, 0x72, 0x61, 0x73, 0x61, 0x6f,
0x44, 0x65, 0x6e, 0x6d, 0x61, 0x72, 0x6b, 0x73, 0x68, 0x61, 0x76, 0xeb,
0x6e, 0x44, 0x6f, 0x75, 0x73, 0x6f, 0x6e, 0x47, 0x6a, 0x69, 0x72, 0x69,
0x20, 0x69, 0x20, 0x44, 0x6f, 0x75, 0x73, 0x6f, 0x6e, 0x69, 0x74, 0x44,
0x6f, 0x6d, 0x69, 0x6e, 0x69, 0x6b, 0xeb, 0x45, 0x6a, 0x72, 0x75, 0x6e,
0x65, 0x70, 0x53, 0x61, 0x6c, 0x76, 0x61, 0x64, 0x6f, 0x72, 0x46, 0x6f,
0x72, 0x74, 0x2d, 0x4e, 0x65, 0x6c, 0x73, 0x6f, 0x6e, 0x47, 0x6a, 0x69,
0x72, 0x69, 0x20, 0x69, 0x20, 0x4e, 0x67, 0x72, 0x69, 0x72, 0xeb, 0x47,
0x6a, 0x69, 0x72, 0x69, 0x20, 0x69, 0x20, 0x50, 0x61, 0x74, 0xeb, 0x73,
0x54, 0x75, 0x72, 0x6b, 0x75, 0x20, 0x69, 0x20, 0x4d, 0x61, 0x64, 0x68,
0x47, 0x72, 0x61, 0x6e, 0x61, 0x64, 0xeb, 0x47, 0x75, 0x61, 0x64, 0x65,
0x6c, 0x75, 0x70, 0xeb, 0x47, 0x75, 0x61, 0x74, 0x65, 0x6d, 0x61, 0x6c,
0xeb, 0x47, 0x75, 0x61, 0x6a, 0x61, 0x6b, 0x75, 0x69, 0x6c, 0x47, 0x75,
0x61, 0x6a, 0x61, 0x6e, 0xeb, 0x48, 0x61, 0x6c, 0x69, 0x66, 0x61, 0x6b,
0x73, 0x48, 0x61, 0x76, 0x61, 0x6e, 0xeb, 0x48, 0x65, 0x72, 0x6d, 0x6f,
0x73, 0x69, 0x6a, 0x6f, 0x4b, 0x6e, 0x6f, 0x6b, 0x73, 0x2c, 0x20, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x50, 0x65, 0x74, 0xeb, 0x72, 0x73,
0x62, 0x75, 0x72, 0x67, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e,
0x61, 0x54, 0x65, 0x6c, 0x6c, 0x2d, 0x53, 0x69, 0x74, 0x69, 0x2c, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x56, 0x65, 0x76, 0xeb, 0x6a,
0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x56, 0x69, 0x6e,
0x63, 0x65, 0x6e, 0x65, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61,
0x6e, 0x61, 0x55, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x6b, 0x2c, 0x20, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x58, 0x68, 0x61, 0x6d, 0x61, 0x6a,
0x6b, 0xeb, 0x58, 0x68, 0x75, 0x6e, 0x6f, 0x75, 0x4c, 0x75, 0x69, 0x7a,
0x76, 0x69, 0x6c, 0xeb, 0x4d, 0x6f, 0x6e, 0x74, 0x69, 0xe7, 0x65, 0x6c,
0x6f, 0x2c, 0x20, 0x4b, 0x65, 0x6e, 0x74, 0x61, 0x6b, 0x69, 0x4c, 0x69,
0x6d, 0xeb, 0x4c, 0x6f, 0x73, 0x20, 0x41, 0x6e, 0x78, 0x68, 0x65, 0x6c,
0x6f, 0x73, 0x4c, 0x6f, 0x75, 0x65, 0x72, 0x2d, 0x50, 0x72, 0x69, 0x6e,
0x73, 0x69, 0x73, 0x2d, 0x4b, 0x75, 0x6f, 0x72, 0x74, 0xeb, 0x72, 0x4d,
0x61, 0x6b, 0x65, 0x6a, 0x6f, 0x4d, 0x61, 0x72, 0x74, 0x69, 0x6e, 0x69,
0x6b, 0xeb, 0x4d, 0x65, 0x6e, 0x6f, 0x6d, 0x69, 0x6e, 0x69, 0x4d, 0x65,
0x72, 0x69, 0x64, 0x61, 0x51, 0x79, 0x74, 0x65, 0x74, 0x69, 0x20, 0x69,
0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0xeb, 0x73, 0x4d, 0x69, 0x6b,
0x65, 0x6c, 0x6f, 0x6e, 0x4d, 0x6f, 0x6e, 0x6b, 0x74, 0x6f, 0x6e, 0x4d,
0x6f, 0x6e, 0x74, 0x65, 0x72, 0x72, 0x65, 0x6a, 0x4d, 0x6f, 0x6e, 0x74,
0x73, 0x65, 0x72, 0x61, 0x74, 0x4e, 0x61, 0x73, 0x61, 0x6f, 0x4e, 0x6a,
0x75, 0x2d, 0x4a, 0x6f, 0x72, 0x6b, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x6a,
0x61, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x20, 0x65, 0x20, 0x56, 0x65, 0x72, 0x69, 0x75, 0x74, 0x51,
0x65, 0x6e, 0x64, 0xeb, 0x72, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74,
0x61, 0x20, 0x65, 0x20, 0x56, 0x65, 0x72, 0x69, 0x75, 0x74, 0x4e, 0x6a,
0x75, 0x2d, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x20, 0x65, 0x20, 0x56, 0x65, 0x72, 0x69, 0x75, 0x74,
0x4e, 0x6a, 0x75, 0x6b, 0x4f, 0x68, 0x69, 0x6e, 0x61, 0x67, 0x61, 0x46,
0x65, 0x6e, 0x69, 0x6b, 0x73, 0x50, 0x6f, 0x72, 0x74, 0x2d, 0x6f, 0x2d,
0x50, 0x72, 0x69, 0x6e, 0x73, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x2d, 0x56,
0x65, 0x6c, 0x68, 0x6f, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x2d, 0x52, 0x69,
0x6b, 0x6f, 0x50, 0x75, 0x6e, 0x74, 0x61, 0x2d, 0x41, 0x72, 0x65, 0x6e,
0x61, 0x73, 0x47, 0x72, 0x79, 0x6b, 0x61, 0x20, 0x49, 0x6e, 0x6c, 0x65,
0x74, 0x52, 0x65, 0x78, 0x68, 0x69, 0x6e, 0x61, 0x52, 0x69, 0x6f, 0x2d,
0x42, 0x72, 0x61, 0x6e, 0x6b, 0x6f, 0x53, 0x61, 0x6e, 0x74, 0x6f, 0x2d,
0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x67, 0x6f, 0x53, 0x61, 0x6f, 0x2d, 0x50,
0x61, 0x75, 0x6c, 0x6f, 0x49, 0x74, 0x6f, 0x6b, 0x6f, 0x72, 0x74, 0x75,
0x72, 0x6d, 0x69, 0x74, 0x53, 0x65, 0x6e, 0x2d, 0x42, 0x61, 0x72, 0x74,
0x65, 0x6c, 0x65, 0x6d, 0x69, 0x53, 0x68, 0xeb, 0x6e, 0x2d, 0x47, 0x6a,
0x6f, 0x6e, 0x53, 0x68, 0xeb, 0x6e, 0x2d, 0x4b, 0x69, 0x74, 0x73, 0x53,
0x68, 0xeb, 0x6e, 0x2d, 0x4c, 0x75, 0xe7, 0x69, 0x61, 0x53, 0x68, 0xeb,
0x6e, 0x2d, 0x54, 0x6f, 0x6d, 0x61, 0x73, 0x53, 0x68, 0xeb, 0x6e, 0x2d,
0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x74, 0x52, 0x72, 0x79, 0x6d, 0x61,
0x20, 0x65, 0x20, 0x53, 0x68, 0x70, 0x65, 0x6a, 0x74, 0xeb, 0x44, 0x68,
0x75, 0x6c, 0x65, 0x54, 0x69, 0x68, 0x75, 0x61, 0x6e, 0x61, 0x54, 0x6f,
0x72, 0x74, 0x6f, 0x6c, 0xeb, 0x56, 0x61, 0x6e, 0x6b, 0x75, 0x76, 0x65,
0x72, 0x55, 0x61, 0x6a, 0x74, 0x2019, 0x68, 0x6f, 0x72, 0x73, 0x55, 0x69,
0x6e, 0x69, 0x70, 0x65, 0x67, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x61, 0x74,
0x4b, 0x65, 0x6a, 0x73, 0x69, 0x44, 0x65, 0x6a, 0x76, 0x69, 0x73, 0x44,
0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69,
0x6c, 0xeb, 0x4d, 0x65, 0x6b, 0x75, 0x61, 0x72, 0x69, 0x4d, 0x61, 0x75,
0x73, 0x6f, 0x6e, 0x4d, 0x65, 0x6b, 0x6d, 0x75, 0x72, 0x64, 0x6f, 0x52,
0x6f, 0x64, 0x68, 0x65, 0x72, 0xeb, 0x53, 0x6a, 0x6f, 0x75, 0x61, 0x4c,
0x6f, 0x6e, 0x67, 0x2019, 0x6a, 0xeb, 0x62, 0x6a, 0x65, 0x6e, 0x41, 0x6c,
0x6d, 0x61, 0x74, 0x69, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x41, 0x6b,
0x74, 0x6f, 0x62, 0x65, 0x41, 0x74, 0x69, 0x72, 0x61, 0x75, 0x42, 0x61,
0x68, 0x72, 0x65, 0x6a, 0x6e, 0x42, 0x65, 0x6a, 0x72, 0x75, 0x74, 0xc7,
0x69, 0x74, 0x61, 0x46, 0x61, 0x6d, 0x61, 0x67, 0x75, 0x73, 0x74, 0xeb,
0x48, 0x6f, 0x2d, 0xc7, 0x69, 0x2d, 0x4d, 0x69, 0x6e, 0x48, 0x6f, 0x6e,
0x67, 0x2d, 0x4b, 0x6f, 0x6e, 0x67, 0x58, 0x68, 0x61, 0x6b, 0x61, 0x72,
0x74, 0xeb, 0x58, 0x68, 0x61, 0x6a, 0x61, 0x70, 0x75, 0x72, 0xeb, 0x4b,
0x61, 0x6d, 0xe7, 0x61, 0x74, 0x6b, 0xeb, 0x4b, 0x61, 0x72, 0x61, 0xe7,
0x69, 0x4b, 0x61, 0x6e, 0x64, 0x69, 0x67, 0xeb, 0x4b, 0x61, 0x6c, 0x6b,
0x75, 0x74, 0xeb, 0x4b, 0x75, 0x61, 0x6c, 0x61, 0x2d, 0x4c, 0x75, 0x6d,
0x70, 0x75, 0x72, 0x4b, 0x75, 0xe7, 0x69, 0x6e, 0x67, 0x4b, 0x75, 0x76,
0x61, 0x6a, 0x74, 0x4d, 0x61, 0x6b, 0x61, 0x6f, 0x4d, 0x61, 0x6b, 0x61,
0x73, 0x61, 0x72, 0x4d, 0x61, 0x6e, 0x69, 0x6c, 0xeb, 0x4e, 0x69, 0x6b,
0x6f, 0x7a, 0x69, 0x61, 0x50, 0x6e, 0x6f, 0x6d, 0x2d, 0x50, 0x65, 0x6e,
0x50, 0x65, 0x6e, 0x69, 0x61, 0x6e, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x6e,
0x61, 0x6a, 0x4b, 0x69, 0x7a, 0x69, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x52,
0x69, 0x61, 0x64, 0x53, 0x61, 0x6b, 0x61, 0x6c, 0x69, 0x6e, 0x53, 0x65,
0x75, 0x6c, 0x53, 0x68, 0x61, 0x6e, 0x67, 0x61, 0x69, 0x53, 0x69, 0x6e,
0x67, 0x61, 0x70, 0x6f, 0x72, 0x53, 0x72, 0x65, 0x64, 0x6e, 0x65, 0x6b,
0x6f, 0x6c, 0x69, 0x6d, 0x73, 0x6b, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a,
0x54, 0x62, 0x69, 0x6c, 0x69, 0x73, 0x54, 0x65, 0x68, 0x65, 0x72, 0x61,
0x6e, 0x54, 0x68, 0x69, 0x6d, 0x70, 0x75, 0x55, 0x6c, 0x61, 0x6e, 0x62,
0x61, 0x74, 0x61, 0x72, 0x56, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x61, 0x6e,
0xeb, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x52, 0x61, 0x6e, 0x67,
0x75, 0x6e, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x4a, 0x65, 0x72, 0x65, 0x76, 0x61, 0x6e, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x42, 0x65, 0x72, 0x6d, 0x75, 0x64, 0x65, 0x4b, 0x65, 0x70,
0x69, 0x20, 0x69, 0x20, 0x47, 0x6a, 0x65, 0x6c, 0x62, 0xeb, 0x72, 0x52,
0x65, 0x69, 0x6b, 0x6a, 0x61, 0x76, 0x69, 0x6b, 0x58, 0x68, 0x6f, 0x72,
0x78, 0x68, 0x61, 0x20, 0x65, 0x20, 0x4a, 0x75, 0x67, 0x75, 0x74, 0x53,
0x68, 0xeb, 0x6e, 0x2d, 0x45, 0x6c, 0x65, 0x6e, 0xeb, 0x53, 0x74, 0x65,
0x6e, 0x6c, 0x69, 0x41, 0x64, 0x65, 0x6c, 0x61, 0x6a, 0x64, 0x65, 0x42,
0x72, 0x69, 0x73, 0x62, 0x65, 0x6a, 0x6e, 0x42, 0x72, 0x6f, 0x6b, 0xeb,
0x6e, 0x2d, 0x48, 0x69, 0x6c, 0x6c, 0x44, 0x61, 0x72, 0x76, 0x69, 0x6e,
0x4c, 0x69, 0x6e, 0x64, 0xeb, 0x6d, 0x65, 0x6e, 0x4c, 0x6f, 0x72, 0x64,
0x2d, 0x48, 0x6f, 0x75, 0x69, 0x4d, 0x65, 0x6c, 0x62, 0x75, 0x72, 0x6e,
0x50, 0xeb, 0x72, 0x74, 0x68, 0x53, 0x69, 0x64, 0x6e, 0x65, 0x6a, 0x41,
0x6e, 0x64, 0x6f, 0x72, 0x72, 0xeb, 0x41, 0x73, 0x74, 0x72, 0x61, 0x6b,
0x61, 0x6e, 0x41, 0x74, 0x68, 0x69, 0x6e, 0xeb, 0x42, 0x65, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x42, 0x72, 0x61, 0x74, 0x69, 0x73, 0x6c, 0x61, 0x76,
0xeb, 0x42, 0x72, 0x75, 0x6b, 0x73, 0x65, 0x6c, 0x42, 0x75, 0x6b, 0x75,
0x72, 0x65, 0x73, 0x68, 0x74, 0x4b, 0x69, 0x73, 0x68, 0x69, 0x6e, 0x65,
0x75, 0x47, 0x6a, 0x69, 0x62, 0x72, 0x61, 0x6c, 0x74, 0x61, 0x72, 0x47,
0x65, 0x72, 0x6e, 0x73, 0x65, 0x6a, 0x49, 0x73, 0x68, 0x75, 0x6c, 0x6c,
0x69, 0x20, 0x69, 0x20, 0x4d, 0x61, 0x6e, 0x69, 0x74, 0x53, 0x74, 0x61,
0x6d, 0x62, 0x6f, 0x6c, 0x6c, 0x58, 0x68, 0x65, 0x72, 0x73, 0x65, 0x6a,
0x4b, 0x69, 0x65, 0x76, 0x4c, 0x69, 0x73, 0x62, 0x6f, 0x6e, 0xeb, 0x4c,
0x75, 0x62, 0x6a, 0x61, 0x6e, 0xeb, 0x4c, 0x6f, 0x6e, 0x64, 0xeb, 0x72,
0x4c, 0x75, 0x6b, 0x73, 0x65, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x4d, 0x61,
0x6c, 0x74, 0xeb, 0x4d, 0x6f, 0x73, 0x6b, 0xeb, 0x50, 0x6f, 0x64, 0x67,
0x6f, 0x72, 0x69, 0x63, 0xeb, 0x50, 0x72, 0x61, 0x67, 0xeb, 0x52, 0x69,
0x67, 0xeb, 0x52, 0x6f, 0x6d, 0xeb, 0x53, 0x61, 0x6e, 0x2d, 0x4d, 0x61,
0x72, 0x69, 0x6e, 0x6f, 0x53, 0x61, 0x72, 0x61, 0x6a, 0x65, 0x76, 0xeb,
0x53, 0x68, 0x6b, 0x75, 0x70, 0x53, 0x6f, 0x66, 0x6a, 0x65, 0x53, 0x74,
0x6f, 0x6b, 0x68, 0x6f, 0x6c, 0x6d, 0x54, 0x61, 0x6c, 0x69, 0x6e, 0x54,
0x69, 0x72, 0x61, 0x6e, 0xeb, 0x55, 0x6c, 0x6a, 0x61, 0x6e, 0x6f, 0x76,
0x73, 0x6b, 0x56, 0x6a, 0x65, 0x6e, 0xeb, 0x56, 0x61, 0x72, 0x73, 0x68,
0x61, 0x76, 0xeb, 0x5a, 0x79, 0x72, 0x69, 0x68, 0xc7, 0x61, 0x67, 0x6f,
0x73, 0x4b, 0x72, 0x69, 0x73, 0x68, 0x74, 0x6c, 0x69, 0x6e, 0x64, 0x6a,
0x65, 0x4b, 0x6f, 0x6d, 0x6f, 0x72, 0x65, 0x4d, 0x61, 0x6c, 0x64, 0x69,
0x76, 0x65, 0x4d, 0x61, 0x6a, 0x6f, 0x74, 0xeb, 0x4f, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x42, 0x75, 0x6e, 0x67, 0x61, 0x6e, 0x76, 0x69, 0x6c, 0xeb,
0xc7, 0x61, 0x74, 0x6d, 0x61, 0x6e, 0xc7, 0x75, 0x6b, 0x50, 0x61, 0x73,
0x68, 0x6b, 0xeb, 0x46, 0x69, 0x78, 0x68, 0x69, 0x47, 0x75, 0x61, 0x64,
0x61, 0x6c, 0x6b, 0x61, 0x6e, 0x61, 0x6c, 0x4b, 0x69, 0x72, 0x69, 0x74,
0x69, 0x6d, 0x61, 0x74, 0x4b, 0x6f, 0x73, 0x72, 0x65, 0x4b, 0x75, 0x61,
0x78, 0x68, 0x61, 0x6c, 0x65, 0x69, 0x6e, 0x4d, 0x61, 0x68, 0x75, 0x72,
0x6f, 0x4d, 0x61, 0x72, 0x6b, 0x65, 0x7a, 0x4d, 0x69, 0x64, 0x75, 0x65,
0x6a, 0x50, 0x61, 0x67, 0x6f, 0x2d, 0x50, 0x61, 0x67, 0x6f, 0x50, 0x69,
0x74, 0x6b, 0x65, 0x72, 0x6e, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x69,
0x50, 0x6f, 0x72, 0x74, 0x2d, 0x4d, 0x6f, 0x72, 0x65, 0x73, 0x62, 0x69,
0x52, 0x61, 0x72, 0x6f, 0x74, 0x6f, 0x6e, 0x67, 0xeb, 0x54, 0x61, 0x72,
0x61, 0x76, 0xeb, 0x55, 0x65, 0x6a, 0x6b, 0x55, 0x6f, 0x6c, 0x6c, 0x69,
0x73, 0x12a0, 0x1262, 0x1303, 0x1295, 0x12a0, 0x12ad, 0x122b, 0x12a0, 0x12f2, 0x1235, 0x20,
0x12a0, 0x1260, 0x1263, 0x12a0, 0x120d, 0x1300, 0x122d, 0x1235, 0x12a0, 0x1235, 0x1218, 0x122b,
0x1263, 0x121b, 0x12ae, 0x1263, 0x1295, 0x1309, 0x12a2, 0x1263, 0x1295, 0x1301, 0x120d, 0x1262,
0x1233, 0x12a6, 0x1265, 0x120b, 0x1295, 0x1273, 0x12e8, 0x122d, 0x1265, 0x122b, 0x12db, 0x126a,
0x120d, 0x1261, 0x1301, 0x121d, 0x1265, 0x122b, 0x12ab, 0x12ed, 0x122e, 0x12ab, 0x12db, 0x1265,
0x120b, 0x1295, 0x12ab, 0x1232, 0x12a1, 0x1273, 0x12ae, 0x1293, 0x12ad, 0x122c, 0x12f3, 0x12ab,
0x122d, 0x12f3, 0x122c, 0x20, 0x1230, 0x120b, 0x121d, 0x1305, 0x1261, 0x1272, 0x12f1, 0x12a0,
0x120b, 0x12a4, 0x120d, 0x20, 0x12a0, 0x12e9, 0x12a0, 0x1295, 0x134d, 0x122a, 0x1273, 0x12cd,
0x1295, 0x130b, 0x1266, 0x122e, 0x1295, 0x1203, 0x122b, 0x122c, 0x1306, 0x1203, 0x1295, 0x1235,
0x1260, 0x122d, 0x130d, 0x1301, 0x1263, 0x12ab, 0x121d, 0x1353, 0x120b, 0x12ab, 0x122d, 0x1271,
0x121d, 0x12aa, 0x130b, 0x120a, 0x12aa, 0x1295, 0x123b, 0x1233, 0x120c, 0x130e, 0x1235, 0x120a,
0x1260, 0x122d, 0x126a, 0x120d, 0x120e, 0x121c, 0x1209, 0x12a0, 0x1295, 0x12f3, 0x1209, 0x1261,
0x121d, 0x1263, 0x123a, 0x1209, 0x1233, 0x12ab, 0x121b, 0x120b, 0x1261, 0x121b, 0x1351, 0x1271,
0x121b, 0x1234, 0x1229, 0x121d, 0x1263, 0x1263, 0x1294, 0x121e, 0x1243, 0x12f2, 0x1239, 0x121e,
0x1295, 0x122e, 0x126a, 0x12eb, 0x1293, 0x12ed, 0x122e, 0x1262, 0x1295, 0x1303, 0x121c, 0x1293,
0x1292, 0x12eb, 0x121c, 0x12ed, 0x1291, 0x12a0, 0x12ad, 0x127e, 0x1275, 0x12a1, 0x130b, 0x12f1,
0x1309, 0x1356, 0x122d, 0x1276, 0x20, 0x2d, 0x20, 0x1296, 0x126e, 0x1233, 0x12a6, 0x20,
0x1276, 0x121c, 0x1275, 0x122a, 0x1356, 0x120a, 0x1271, 0x1292, 0x12dd, 0x12ca, 0x1295, 0x12f5,
0x1206, 0x12ad, 0x12a0, 0x12f3, 0x12ad, 0x12a0, 0x1295, 0x12ae, 0x122b, 0x1305, 0x12a0, 0x1295,
0x1309, 0x12ed, 0x120b, 0x12a0, 0x1295, 0x1272, 0x1313, 0x12a0, 0x122b, 0x1309, 0x12e8, 0x1293,
0x1266, 0x1290, 0x1235, 0x20, 0x12a0, 0x12ed, 0x1228, 0x1235, 0x12ab, 0x1273, 0x121b, 0x122d,
0x12ab, 0x12ae, 0x122d, 0x12f6, 0x1263, 0x1301, 0x1301, 0x12ed, 0x120b, 0x20, 0x122a, 0x12a6,
0x1303, 0x121c, 0x1295, 0x12f6, 0x12db, 0x122a, 0x12ee, 0x20, 0x130b, 0x120c, 0x130e, 0x1235,
0x1233, 0x120d, 0x1273, 0x1233, 0x1295, 0x20, 0x1301, 0x12a0, 0x1295, 0x1233, 0x1295, 0x20,
0x120a, 0x12cd, 0x1235, 0x1271, 0x12a9, 0x121b, 0x1295, 0x12a1, 0x1239, 0x12a0, 0x12a2, 0x12a0,
0x1229, 0x1263, 0x12a0, 0x1231, 0x1295, 0x1232, 0x12ee, 0x1295, 0x12a0, 0x1272, 0x12ae, 0x12ab,
0x1295, 0x1263, 0x1202, 0x12a0, 0x1263, 0x1202, 0x12eb, 0x20, 0x1263, 0x1295, 0x12f0, 0x122b,
0x1235, 0x1263, 0x122d, 0x1264, 0x12f6, 0x1235, 0x1264, 0x1208, 0x121d, 0x1264, 0x120a, 0x12dd,
0x1265, 0x120b, 0x1295, 0x12ad, 0x2d, 0x20, 0x1233, 0x1265, 0x120e, 0x1295, 0x1266, 0x12a0,
0x20, 0x126a, 0x1235, 0x1273, 0x1266, 0x130e, 0x1273, 0x1266, 0x12ed, 0x12dd, 0x12ab, 0x121d,
0x1265, 0x122a, 0x1305, 0x20, 0x1264, 0x12ed, 0x12ab, 0x121d, 0x1356, 0x20, 0x130d, 0x122b,
0x1295, 0x12f4, 0x12ab, 0x1295, 0x12a9, 0x1295, 0x12ab, 0x122b, 0x12ab, 0x1235, 0x12ab, 0x12ed,
0x1295, 0x12a4, 0x12ab, 0x12ed, 0x121b, 0x1295, 0x127a, 0x12ab, 0x130e, 0x127a, 0x1201, 0x12cb,
0x12cd, 0x12a0, 0x1233, 0x12ee, 0x12f3, 0x12f5, 0x20, 0x1201, 0x12cb, 0x122c, 0x12dd, 0x12ae,
0x1235, 0x1273, 0x122a, 0x12ab, 0x12ad, 0x1228, 0x1235, 0x1270, 0x1295, 0x12a9, 0x12e8, 0x12a0,
0x1263, 0x12a9, 0x122b, 0x1233, 0x12ce, 0x12f3, 0x1295, 0x121b, 0x122d, 0x12ad, 0x123b, 0x126d,
0x1295, 0x12f3, 0x12cd, 0x1230, 0x1295, 0x12f3, 0x12cd, 0x1230, 0x1295, 0x20, 0x12ad, 0x122c,
0x12ad, 0x12f4, 0x1295, 0x1268, 0x122d, 0x12f2, 0x1275, 0x122e, 0x12ed, 0x1275, 0x12f6, 0x121c,
0x1292, 0x12ab, 0x12a4, 0x12f5, 0x1218, 0x1295, 0x1270, 0x1295, 0x12a2, 0x1229, 0x1294, 0x1355,
0x12a4, 0x120d, 0x1233, 0x120d, 0x126b, 0x12f6, 0x122d, 0x134e, 0x122d, 0x1275, 0x20, 0x1294,
0x120d, 0x1230, 0x1295, 0x134e, 0x122d, 0x1273, 0x120c, 0x12db, 0x130d, 0x120c, 0x1235, 0x20,
0x1264, 0x12ed, 0x1309, 0x12dd, 0x20, 0x1264, 0x12ed, 0x130d, 0x122b, 0x1295, 0x12f5, 0x20,
0x1270, 0x122d, 0x12ad, 0x130d, 0x122c, 0x1293, 0x12f3, 0x1315, 0x12f3, 0x1209, 0x1355, 0x1313,
0x1272, 0x121b, 0x120b, 0x1309, 0x12eb, 0x12a9, 0x12ed, 0x120d, 0x1309, 0x12eb, 0x1293, 0x1203,
0x120a, 0x134b, 0x12ad, 0x1235, 0x1203, 0x126b, 0x1293, 0x12b8, 0x122d, 0x121e, 0x12da, 0x120e,
0x12a2, 0x1295, 0x12f2, 0x12eb, 0x1293, 0x1356, 0x120a, 0x1235, 0x1296, 0x12ad, 0x1235, 0x2c,
0x20, 0x12a2, 0x1295, 0x12f2, 0x12eb, 0x1293, 0x121b, 0x122c, 0x1295, 0x130e, 0x2c, 0x20,
0x12a2, 0x1295, 0x12f2, 0x12eb, 0x1293, 0x1352, 0x1270, 0x122d, 0x1235, 0x1260, 0x122d, 0x130d,
0x2c, 0x20, 0x12a2, 0x1295, 0x12f2, 0x12eb, 0x1293, 0x1274, 0x120d, 0x20, 0x12a8, 0x1270,
0x121b, 0x2c, 0x20, 0x12a2, 0x1295, 0x12f2, 0x12eb, 0x1293, 0x126a, 0x126b, 0x12ed, 0x2c,
0x20, 0x12a2, 0x1295, 0x12f2, 0x12eb, 0x1293, 0x126a, 0x1295, 0x127c, 0x1295, 0x1235, 0x2c,
0x20, 0x12a2, 0x1295, 0x12f2, 0x12eb, 0x1293, 0x12ca, 0x1293, 0x121b, 0x12ad, 0x2c, 0x20,
0x12a2, 0x1295, 0x12f2, 0x12eb, 0x1293, 0x12a2, 0x1291, 0x126a, 0x12ad, 0x12a2, 0x12b3, 0x120a,
0x12cd, 0x1275, 0x1303, 0x121b, 0x12ed, 0x12ab, 0x1301, 0x1292, 0x12e9, 0x120a, 0x12cd, 0x1235,
0x126a, 0x120d, 0x121e, 0x1295, 0x1272, 0x1234, 0x120e, 0x2c, 0x20, 0x12aa, 0x1295, 0x1273,
0x12aa, 0x12ad, 0x122b, 0x1208, 0x1295, 0x12f2, 0x12ed, 0x12ad, 0x120b, 0x20, 0x1353, 0x12dd,
0x120a, 0x121b, 0x120e, 0x1235, 0x20, 0x12a0, 0x1295, 0x1300, 0x1208, 0x1235, 0x12e8, 0x1273,
0x127d, 0x129b, 0x12cd, 0x20, 0x120d, 0x12d1, 0x120d, 0x20, 0x1229, 0x1265, 0x121c, 0x1232,
0x12a6, 0x121b, 0x1293, 0x1309, 0x12a0, 0x121b, 0x1293, 0x12a1, 0x1235, 0x121b, 0x122d, 0x130e,
0x1275, 0x121b, 0x122d, 0x1272, 0x1292, 0x12ad, 0x121b, 0x1273, 0x121e, 0x122e, 0x1235, 0x121b,
0x12db, 0x1275, 0x120b, 0x1295, 0x121c, 0x1296, 0x121a, 0x1292, 0x121c, 0x122a, 0x12f3, 0x1218,
0x1275, 0x120b, 0x12ab, 0x1275, 0x120b, 0x121c, 0x12ad, 0x1232, 0x12ae, 0x20, 0x12a8, 0x1270,
0x121b, 0x121a, 0x12ae, 0x12cd, 0x120e, 0x1295, 0x121e, 0x1295, 0x12ad, 0x1276, 0x1295, 0x121e,
0x1295, 0x1270, 0x122d, 0x122c, 0x12ed, 0x121e, 0x1295, 0x1274, 0x126a, 0x12f5, 0x12ee, 0x121e,
0x1295, 0x1275, 0x1234, 0x122b, 0x1275, 0x1293, 0x1233, 0x12cd, 0x1292, 0x12cd, 0x12ee, 0x122d,
0x12ad, 0x1296, 0x121c, 0x1296, 0x122e, 0x129b, 0x1264, 0x12a1, 0x120b, 0x1205, 0x2c, 0x20,
0x1230, 0x121c, 0x1295, 0x20, 0x12f3, 0x12ae, 0x1273, 0x1218, 0x12ab, 0x12a8, 0x1208, 0x129b,
0x2c, 0x20, 0x1230, 0x121c, 0x1295, 0x20, 0x12f3, 0x12ae, 0x1273, 0x12a0, 0x12f2, 0x1235,
0x20, 0x1233, 0x120c, 0x121d, 0x2c, 0x20, 0x1230, 0x121c, 0x1295, 0x20, 0x12f3, 0x12ae,
0x1273, 0x130b, 0x12f5, 0x1273, 0x1265, 0x12a6, 0x12ea, 0x1293, 0x130b, 0x1353, 0x1293, 0x121b,
0x1353, 0x122b, 0x121b, 0x122a, 0x1266, 0x134a, 0x1292, 0x12ad, 0x1235, 0x1356, 0x122d, 0x1275,
0x20, 0x12a6, 0x20, 0x1355, 0x122a, 0x1295, 0x1235, 0x12e8, 0x12a5, 0x1235, 0x1354, 0x1295,
0x20, 0x12c8, 0x12f0, 0x1265, 0x1354, 0x1275, 0x122e, 0x20, 0x126c, 0x120d, 0x1206, 0x1356,
0x122d, 0x1276, 0x122a, 0x12ae, 0x1351, 0x1295, 0x1273, 0x20, 0x12a0, 0x122c, 0x1293, 0x1235,
0x122b, 0x1295, 0x12aa, 0x1295, 0x20, 0x12a2, 0x1295, 0x120c, 0x1275, 0x1228, 0x1232, 0x134d,
0x1228, 0x1302, 0x1293, 0x122a, 0x12de, 0x120a, 0x12e9, 0x1275, 0x122a, 0x12ee, 0x20, 0x1265,
0x122b, 0x1295, 0x12ae, 0x1233, 0x1295, 0x1273, 0x122c, 0x121d, 0x1233, 0x1295, 0x1272, 0x12eb,
0x130e, 0x1233, 0x1295, 0x1276, 0x20, 0x12f6, 0x121a, 0x1295, 0x130e, 0x1233, 0x12a6, 0x20,
0x1356, 0x120e, 0x1235, 0x12ae, 0x122d, 0x1235, 0x1263, 0x12ed, 0x1230, 0x1295, 0x12f5, 0x1232,
0x1275, 0x12ab, 0x1245, 0x12f5, 0x1235, 0x1275, 0x20, 0x1264, 0x122d, 0x1270, 0x120e, 0x121c,
0x1245, 0x12f1, 0x1235, 0x20, 0x12ee, 0x1210, 0x1295, 0x1235, 0x1245, 0x12f1, 0x1235, 0x20,
0x12aa, 0x1275, 0x1235, 0x1245, 0x12f5, 0x1235, 0x1275, 0x20, 0x1209, 0x1232, 0x12eb, 0x1245,
0x12f1, 0x1235, 0x20, 0x1276, 0x121b, 0x1235, 0x1245, 0x12f1, 0x1235, 0x20, 0x126a, 0x1295,
0x1230, 0x1295, 0x1275, 0x12e8, 0x1210, 0x12cb, 0x120b, 0x20, 0x1308, 0x1295, 0x12d8, 0x1265,
0x1274, 0x1309, 0x1232, 0x130b, 0x120d, 0x1353, 0x1271, 0x120c, 0x1272, 0x1301, 0x12a0, 0x1293,
0x1276, 0x122e, 0x1295, 0x1276, 0x1276, 0x122d, 0x1276, 0x120b, 0x126b, 0x1295, 0x12ae, 0x1268,
0x122d, 0x128b, 0x12ed, 0x1275, 0x1206, 0x122d, 0x1235, 0x12ca, 0x1292, 0x1354, 0x130d, 0x12eb,
0x12a9, 0x1273, 0x1275, 0x12ab, 0x12da, 0x12ed, 0x12f3, 0x126a, 0x1235, 0x12f0, 0x121e, 0x1295,
0x1275, 0x20, 0x12f2, 0x12a1, 0x122d, 0x126a, 0x120d, 0x121b, 0x12a8, 0x122a, 0x1293, 0x12cd,
0x1230, 0x1295, 0x121b, 0x12ad, 0x1218, 0x122d, 0x12f6, 0x1353, 0x120d, 0x1218, 0x122d, 0x122e,
0x1274, 0x122b, 0x1235, 0x12ee, 0x12cb, 0x1275, 0x122e, 0x120d, 0x126d, 0x1235, 0x1276, 0x12ad,
0x120e, 0x1295, 0x130d, 0x12ed, 0x122d, 0x1263, 0x12e8, 0x1295, 0x12a4, 0x12f0, 0x1295, 0x12a0,
0x120d, 0x121b, 0x1275, 0x12ed, 0x12a0, 0x121b, 0x1295, 0x12a0, 0x1293, 0x12f5, 0x12ed, 0x122d,
0x12a0, 0x12a9, 0x1273, 0x12a1, 0x12a0, 0x12a9, 0x1276, 0x1264, 0x12a0, 0x123d, 0x130b, 0x1263,
0x1275, 0x12a0, 0x1275, 0x12ed, 0x122b, 0x12a1, 0x1263, 0x130d, 0x12f3, 0x12f5, 0x1263, 0x1205,
0x122c, 0x1295, 0x1263, 0x1295, 0x12ae, 0x12ad, 0x1263, 0x122d, 0x1293, 0x12a1, 0x120d, 0x1264,
0x1229, 0x1275, 0x1262, 0x123d, 0x12ac, 0x12ad, 0x1265, 0x1229, 0x1293, 0x12ed, 0x127a, 0x1273,
0x12ae, 0x120e, 0x121d, 0x1266, 0x12f0, 0x121b, 0x1235, 0x1246, 0x12f2, 0x120a, 0x12f1, 0x1263,
0x12ed, 0x12f0, 0x123b, 0x1295, 0x1264, 0x134b, 0x121b, 0x1309, 0x1235, 0x1273, 0x130b, 0x12db,
0x12ac, 0x1265, 0x122e, 0x1295, 0x1206, 0x20, 0x127a, 0x20, 0x121a, 0x1295, 0x1205, 0x20,
0x12a8, 0x1270, 0x121b, 0x1206, 0x1295, 0x130d, 0x20, 0x12ae, 0x1295, 0x130d, 0x1206, 0x126d,
0x12f5, 0x12a2, 0x122d, 0x12a9, 0x1275, 0x1235, 0x12ad, 0x1303, 0x12ab, 0x122d, 0x1273, 0x1303,
0x12eb, 0x1351, 0x122b, 0x12a5, 0x12e8, 0x1229, 0x1233, 0x120c, 0x121d, 0x12ab, 0x1261, 0x120d,
0x12ab, 0x121d, 0x127b, 0x1275, 0x12ab, 0x12ab, 0x122b, 0x127a, 0x12ab, 0x1275, 0x121b, 0x1295,
0x12f1, 0x12ab, 0x1295, 0x12f2, 0x130b, 0x12ae, 0x120d, 0x12ab, 0x1273, 0x12ad, 0x122b, 0x1235,
0x1296, 0x12eb, 0x122d, 0x1235, 0x12ad, 0x12a9, 0x12cb, 0x120b, 0x20, 0x120b, 0x121d, 0x1351,
0x122d, 0x12a9, 0x127a, 0x1295, 0x130d, 0x12a9, 0x12cc, 0x1275, 0x121b, 0x12ab, 0x12a1, 0x121b,
0x130b, 0x12f3, 0x1295, 0x121b, 0x12ab, 0x1233, 0x122d, 0x121b, 0x1292, 0x120b, 0x1219, 0x1235,
0x12ab, 0x1275, 0x1292, 0x12ae, 0x1232, 0x12a0, 0x1296, 0x126e, 0x12a9, 0x1275, 0x12dd, 0x1294,
0x12ad, 0x1296, 0x126e, 0x1232, 0x1262, 0x122a, 0x1235, 0x12ad, 0x12a6, 0x121d, 0x1235, 0x12ad,
0x12a6, 0x122b, 0x120d, 0x134d, 0x1296, 0x121d, 0x20, 0x1354, 0x1295, 0x1205, 0x1356, 0x1295,
0x1272, 0x12a0, 0x1293, 0x12ad, 0x1355, 0x12ee, 0x1295, 0x130d, 0x12eb, 0x1295, 0x130d, 0x12b3,
0x1273, 0x122d, 0x12ae, 0x1235, 0x1273, 0x1293, 0x12ed, 0x12a9, 0x12ed, 0x12da, 0x120e, 0x122d,
0x12f3, 0x122a, 0x12eb, 0x12f5, 0x1233, 0x12ad, 0x1203, 0x120a, 0x1295, 0x1233, 0x121b, 0x122d,
0x12ab, 0x1295, 0x12f5, 0x1234, 0x12a6, 0x120d, 0x123b, 0x1295, 0x130b, 0x12ed, 0x1232, 0x1295,
0x130b, 0x1356, 0x122d, 0x1235, 0x122c, 0x12f5, 0x1294, 0x1235, 0x12ae, 0x120d, 0x121d, 0x1235,
0x12ad, 0x1273, 0x12ed, 0x1353, 0x12ed, 0x1273, 0x123d, 0x12ac, 0x1295, 0x1275, 0x1275, 0x1265,
0x120a, 0x1232, 0x1274, 0x1205, 0x122b, 0x1295, 0x1272, 0x121d, 0x1349, 0x1276, 0x12aa, 0x12ee,
0x1276, 0x121d, 0x1235, 0x12ad, 0x12a1, 0x120b, 0x12a0, 0x1295, 0x1263, 0x12a0, 0x1273, 0x122d,
0x12a1, 0x1229, 0x121d, 0x12aa, 0x12a1, 0x1235, 0x1275, 0x2d, 0x1294, 0x122b, 0x126c, 0x1295,
0x1272, 0x12a0, 0x1295, 0x126d, 0x120b, 0x12f2, 0x126e, 0x1235, 0x1276, 0x12ad, 0x12eb, 0x12a9,
0x1275, 0x1235, 0x12ad, 0x12eb, 0x1295, 0x130e, 0x1295, 0x12e8, 0x12ab, 0x1270, 0x122a, 0x1295,
0x1260, 0x122d, 0x130d, 0x12ed, 0x122c, 0x126b, 0x1295, 0x12a0, 0x12de, 0x1228, 0x1235, 0x1264,
0x122d, 0x1219, 0x12f3, 0x12ab, 0x1293, 0x122a, 0x12ac, 0x1355, 0x20, 0x126c, 0x122d, 0x12f0,
0x134b, 0x122e, 0x12a5, 0x121b, 0x12f4, 0x122b, 0x122c, 0x12ed, 0x12ad, 0x1303, 0x126a, 0x12ad,
0x12f0, 0x1261, 0x1265, 0x20, 0x1306, 0x122d, 0x1302, 0x12eb, 0x1245, 0x12f5, 0x1235, 0x1275,
0x20, 0x1204, 0x1208, 0x1293, 0x1235, 0x1273, 0x1295, 0x120c, 0x12ed, 0x12a0, 0x12f4, 0x120c,
0x12a5, 0x12f5, 0x1265, 0x122a, 0x1235, 0x1264, 0x1295, 0x1265, 0x122e, 0x12ad, 0x1295, 0x20,
0x1202, 0x120d, 0x12f3, 0x122d, 0x12ca, 0x1295, 0x12a1, 0x12ad, 0x120b, 0x1206, 0x1263, 0x122d,
0x1275, 0x120a, 0x1295, 0x12f5, 0x121b, 0x1295, 0x120e, 0x122d, 0x12f5, 0x20, 0x1206, 0x12ca,
0x121c, 0x120d, 0x1266, 0x122d, 0x1295, 0x1350, 0x122d, 0x12dd, 0x1232, 0x12f5, 0x1292, 0x12a0,
0x121d, 0x1235, 0x1270, 0x122d, 0x12f3, 0x121d, 0x12a0, 0x1295, 0x12f6, 0x122b, 0x12a0, 0x1235,
0x1275, 0x122b, 0x12ad, 0x1203, 0x1295, 0x12a0, 0x1274, 0x1295, 0x1235, 0x1264, 0x120d, 0x130d,
0x122c, 0x12f5, 0x1260, 0x122d, 0x120a, 0x1295, 0x1265, 0x122b, 0x1272, 0x1235, 0x120b, 0x126b,
0x1265, 0x122b, 0x1230, 0x120d, 0x1235, 0x1261, 0x12ab, 0x122c, 0x1235, 0x1275, 0x1261, 0x12f3,
0x1354, 0x1235, 0x1275, 0x1261, 0x1232, 0x1295, 0x1308, 0x1295, 0x127a, 0x1235, 0x1293, 0x12a1,
0x12ae, 0x1350, 0x1295, 0x1203, 0x1308, 0x1295, 0x12f0, 0x1265, 0x120a, 0x1295, 0x1302, 0x1265,
0x122b, 0x120d, 0x1273, 0x122d, 0x1309, 0x122d, 0x1290, 0x1232, 0x1204, 0x120d, 0x1232, 0x1295,
0x12aa, 0x12a0, 0x12ed, 0x1235, 0x120d, 0x20, 0x12a6, 0x134d, 0x20, 0x121b, 0x1295, 0x12a2,
0x1235, 0x1273, 0x1295, 0x1261, 0x120d, 0x1300, 0x122d, 0x1232, 0x12ab, 0x120a, 0x1292, 0x1295,
0x130d, 0x122b, 0x12f5, 0x12aa, 0x122e, 0x126d, 0x12aa, 0x12e8, 0x126d, 0x120a, 0x12dd, 0x1260,
0x1295, 0x120d, 0x1301, 0x1265, 0x120d, 0x1303, 0x1293, 0x1208, 0x1295, 0x12f0, 0x1295, 0x1209,
0x12ad, 0x1230, 0x121d, 0x1260, 0x122d, 0x130d, 0x121b, 0x12f5, 0x122a, 0x12f5, 0x121b, 0x120d,
0x1273, 0x121c, 0x122a, 0x1203, 0x121d, 0x1295, 0x121a, 0x1295, 0x1235, 0x12ad, 0x121e, 0x1293,
0x12ae, 0x121e, 0x1235, 0x12ae, 0x12a6, 0x1235, 0x120e, 0x1353, 0x122a, 0x1235, 0x1356, 0x12f5,
0x130e, 0x122a, 0x12ab, 0x1355, 0x122b, 0x130d, 0x122a, 0x130b, 0x122e, 0x121d, 0x1233, 0x121b,
0x122b, 0x1233, 0x1295, 0x20, 0x121b, 0x122a, 0x1296, 0x1233, 0x122a, 0x12e8, 0x126e, 0x1233,
0x122b, 0x1276, 0x126d, 0x1232, 0x121d, 0x1348, 0x122e, 0x1356, 0x120d, 0x1235, 0x12ae, 0x1355,
0x12e8, 0x1236, 0x134a, 0x12eb, 0x1235, 0x1276, 0x12ad, 0x1206, 0x120d, 0x121d, 0x1273, 0x120a,
0x1295, 0x1274, 0x122b, 0x1295, 0x12a1, 0x120d, 0x12eb, 0x1296, 0x126d, 0x1235, 0x12ad, 0x126b,
0x12f1, 0x12dd, 0x126b, 0x1272, 0x12ab, 0x1295, 0x126a, 0x12e8, 0x1293, 0x126a, 0x120a, 0x1290,
0x1235, 0x126e, 0x120d, 0x130e, 0x122b, 0x12f5, 0x12cb, 0x122d, 0x1236, 0x12cd, 0x12db, 0x130d,
0x122c, 0x1265, 0x12d9, 0x122a, 0x12ad, 0x12a0, 0x1295, 0x1273, 0x1293, 0x1293, 0x122a, 0x126e,
0x127b, 0x130e, 0x1235, 0x1308, 0x1293, 0x12ae, 0x12ae, 0x1235, 0x12ae, 0x121e, 0x122e, 0x12ac,
0x122d, 0x1309, 0x1208, 0x1295, 0x121b, 0x1204, 0x121b, 0x120d, 0x12f2, 0x1268, 0x121e, 0x122a,
0x123d, 0x12e8, 0x1235, 0x121b, 0x12ee, 0x1274, 0x122c, 0x12e9, 0x1292, 0x12e8, 0x1295, 0x12a0,
0x1352, 0x12a0, 0x12a6, 0x12ad, 0x120b, 0x1295, 0x12f5, 0x1266, 0x130c, 0x12ed, 0x1295, 0x126a,
0x120d, 0x127b, 0x1273, 0x121d, 0x1279, 0x12ad, 0x134b, 0x1232, 0x12ab, 0x12a2, 0x134b, 0x1274,
0x134b, 0x12ab, 0x12a6, 0x134e, 0x134a, 0x1302, 0x1348, 0x1293, 0x1349, 0x1272, 0x130b, 0x120b,
0x1353, 0x130e, 0x1235, 0x130b, 0x121d, 0x1262, 0x12e8, 0x122d, 0x1309, 0x12cb, 0x12f3, 0x120d,
0x12ab, 0x1293, 0x120d, 0x1309, 0x12a0, 0x121d, 0x1206, 0x1296, 0x1209, 0x1209, 0x12a2, 0x1295,
0x12f0, 0x122d, 0x1260, 0x122a, 0x12aa, 0x122a, 0x1272, 0x121b, 0x1272, 0x12ae, 0x1235, 0x122c,
0x12ad, 0x12cb, 0x1303, 0x120a, 0x1295, 0x121b, 0x1301, 0x1229, 0x121b, 0x122d, 0x12ad, 0x12cc,
0x1233, 0x1235, 0x121a, 0x12f5, 0x12c8, 0x12ed, 0x1293, 0x12cd, 0x1229, 0x1292, 0x12cc, 0x1296,
0x122d, 0x134e, 0x120d, 0x12ad, 0x1293, 0x12a6, 0x121a, 0x12a0, 0x1353, 0x130e, 0x20, 0x1353,
0x130e, 0x1353, 0x120b, 0x12cd, 0x1352, 0x1275, 0x12a8, 0x12ed, 0x122d, 0x1295, 0x1356, 0x1205,
0x1295, 0x1354, 0x1356, 0x122d, 0x1275, 0x20, 0x121e, 0x122c, 0x1235, 0x1263, 0x12ed, 0x122b,
0x122e, 0x1276, 0x1295, 0x130b, 0x1234, 0x12ed, 0x1353, 0x1295, 0x1273, 0x1202, 0x1272, 0x1273,
0x122b, 0x12cb, 0x1276, 0x1295, 0x130b, 0x1273, 0x1351, 0x12cb, 0x12ac, 0x12cb, 0x120a, 0x1235,
0x623, 0x628, 0x64a, 0x62f, 0x62c, 0x627, 0x646, 0x623, 0x643, 0x631, 0x627, 0x623,
0x62f, 0x64a, 0x633, 0x20, 0x623, 0x628, 0x627, 0x628, 0x627, 0x627, 0x644, 0x62c,
0x632, 0x627, 0x626, 0x631, 0x623, 0x633, 0x645, 0x631, 0x629, 0x628, 0x627, 0x645,
0x627, 0x643, 0x648, 0x628, 0x627, 0x646, 0x63a, 0x648, 0x64a, 0x628, 0x627, 0x646,
0x62c, 0x648, 0x644, 0x628, 0x64a, 0x633, 0x627, 0x648, 0x628, 0x644, 0x627, 0x646,
0x62a, 0x627, 0x64a, 0x631, 0x628, 0x631, 0x627, 0x632, 0x627, 0x641, 0x64a, 0x644,
0x628, 0x648, 0x62c, 0x648, 0x645, 0x628, 0x648, 0x631, 0x627, 0x627, 0x644, 0x642,
0x627, 0x647, 0x631, 0x629, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x20, 0x627, 0x644,
0x628, 0x64a, 0x636, 0x627, 0x621, 0x633, 0x64a, 0x62a, 0x627, 0x643, 0x648, 0x646,
0x627, 0x643, 0x631, 0x64a, 0x62f, 0x627, 0x643, 0x627, 0x631, 0x62f, 0x627, 0x631,
0x20, 0x627, 0x644, 0x633, 0x644, 0x627, 0x645, 0x62c, 0x64a, 0x628, 0x648, 0x62a,
0x64a, 0x62f, 0x648, 0x627, 0x644, 0x627, 0x627, 0x644, 0x639, 0x64a, 0x648, 0x646,
0x641, 0x631, 0x64a, 0x20, 0x62a, 0x627, 0x648, 0x646, 0x63a, 0x627, 0x628, 0x648,
0x631, 0x648, 0x646, 0x647, 0x631, 0x627, 0x631, 0x64a, 0x62c, 0x648, 0x647, 0x627,
0x646, 0x633, 0x628, 0x631, 0x63a, 0x62c, 0x648, 0x628, 0x627, 0x643, 0x627, 0x645,
0x628, 0x627, 0x644, 0x627, 0x627, 0x644, 0x62e, 0x631, 0x637, 0x648, 0x645, 0x643,
0x64a, 0x63a, 0x627, 0x644, 0x64a, 0x643, 0x64a, 0x646, 0x634, 0x627, 0x633, 0x627,
0x644, 0x627, 0x63a, 0x648, 0x633, 0x644, 0x64a, 0x628, 0x631, 0x641, 0x64a, 0x644,
0x644, 0x648, 0x645, 0x64a, 0x644, 0x648, 0x627, 0x646, 0x62f, 0x627, 0x644, 0x648,
0x645, 0x628, 0x628, 0x627, 0x634, 0x627, 0x644, 0x648, 0x633, 0x627, 0x643, 0x627,
0x645, 0x627, 0x644, 0x627, 0x628, 0x648, 0x645, 0x627, 0x628, 0x648, 0x62a, 0x648,
0x645, 0x627, 0x633, 0x64a, 0x631, 0x648, 0x645, 0x628, 0x627, 0x628, 0x627, 0x646,
0x64a, 0x645, 0x642, 0x62f, 0x64a, 0x634, 0x64a, 0x648, 0x645, 0x648, 0x646, 0x631,
0x648, 0x641, 0x64a, 0x627, 0x646, 0x64a, 0x631, 0x648, 0x628, 0x64a, 0x646, 0x62c,
0x627, 0x645, 0x64a, 0x646, 0x627, 0x646, 0x64a, 0x627, 0x645, 0x64a, 0x646, 0x648,
0x627, 0x643, 0x634, 0x648, 0x637, 0x648, 0x627, 0x63a, 0x627, 0x62f, 0x648, 0x63a,
0x648, 0x628, 0x648, 0x631, 0x62a, 0x648, 0x20, 0x646, 0x648, 0x641, 0x648, 0x633,
0x627, 0x648, 0x20, 0x62a, 0x648, 0x645, 0x64a, 0x637, 0x631, 0x627, 0x628, 0x644,
0x633, 0x62a, 0x648, 0x646, 0x633, 0x648, 0x64a, 0x646, 0x62f, 0x647, 0x648, 0x643,
0x623, 0x62f, 0x627, 0x643, 0x623, 0x646, 0x634, 0x648, 0x631, 0x627, 0x62c, 0x623,
0x646, 0x63a, 0x648, 0x64a, 0x644, 0x627, 0x623, 0x646, 0x62a, 0x64a, 0x63a, 0x648,
0x627, 0x623, 0x631, 0x648, 0x62c, 0x648, 0x627, 0x646, 0x64a, 0x627, 0x628, 0x648,
0x64a, 0x646, 0x648, 0x633, 0x20, 0x623, 0x64a, 0x631, 0x633, 0x643, 0x627, 0x62a,
0x627, 0x645, 0x627, 0x631, 0x643, 0x627, 0x643, 0x648, 0x631, 0x62f, 0x648, 0x628,
0x627, 0x62c, 0x648, 0x62c, 0x648, 0x644, 0x627, 0x20, 0x631, 0x64a, 0x648, 0x62c,
0x627, 0x645, 0x64a, 0x646, 0x62f, 0x648, 0x632, 0x627, 0x631, 0x64a, 0x648, 0x20,
0x62c, 0x627, 0x644, 0x64a, 0x64a, 0x648, 0x633, 0x633, 0x627, 0x644, 0x637, 0x627,
0x633, 0x627, 0x646, 0x20, 0x62e, 0x648, 0x627, 0x646, 0x633, 0x627, 0x646, 0x20,
0x644, 0x648, 0x64a, 0x633, 0x62a, 0x627, 0x643, 0x645, 0x627, 0x646, 0x623, 0x634,
0x648, 0x627, 0x623, 0x631, 0x648, 0x628, 0x627, 0x623, 0x633, 0x648, 0x646, 0x633,
0x64a, 0x648, 0x646, 0x643, 0x648, 0x631, 0x627, 0x644, 0x20, 0x647, 0x627, 0x631,
0x628, 0x631, 0x628, 0x627, 0x647, 0x64a, 0x627, 0x628, 0x627, 0x647, 0x64a, 0x627,
0x20, 0x628, 0x627, 0x646, 0x62f, 0x64a, 0x631, 0x627, 0x633, 0x628, 0x631, 0x628,
0x627, 0x62f, 0x648, 0x633, 0x628, 0x644, 0x645, 0x628, 0x644, 0x64a, 0x632, 0x628,
0x644, 0x627, 0x646, 0x643, 0x2d, 0x633, 0x627, 0x628, 0x644, 0x648, 0x646, 0x628,
0x627, 0x648, 0x20, 0x641, 0x64a, 0x633, 0x62a, 0x627, 0x628, 0x648, 0x63a, 0x648,
0x62a, 0x627, 0x628, 0x648, 0x64a, 0x633, 0x643, 0x627, 0x645, 0x628, 0x631, 0x62f,
0x64a, 0x62c, 0x20, 0x628, 0x627, 0x64a, 0x643, 0x648, 0x645, 0x628, 0x648, 0x20,
0x62c, 0x631, 0x627, 0x646, 0x62f, 0x643, 0x627, 0x646, 0x643, 0x648, 0x646, 0x643,
0x627, 0x631, 0x627, 0x643, 0x627, 0x633, 0x643, 0x627, 0x64a, 0x64a, 0x646, 0x643,
0x627, 0x64a, 0x645, 0x627, 0x646, 0x634, 0x64a, 0x643, 0x627, 0x63a, 0x648, 0x62a,
0x634, 0x64a, 0x648, 0x627, 0x648, 0x627, 0x633, 0x64a, 0x648, 0x62f, 0x627, 0x62f,
0x20, 0x62e, 0x648, 0x627, 0x631, 0x64a, 0x632, 0x643, 0x648, 0x633, 0x62a, 0x627,
0x631, 0x64a, 0x643, 0x627, 0x643, 0x631, 0x64a, 0x633, 0x62a, 0x648, 0x646, 0x643,
0x64a, 0x627, 0x628, 0x627, 0x643, 0x648, 0x631, 0x627, 0x633, 0x627, 0x648, 0x62f,
0x627, 0x646, 0x645, 0x631, 0x643, 0x20, 0x634, 0x627, 0x641, 0x646, 0x62f, 0x627,
0x648, 0x633, 0x627, 0x646, 0x62f, 0x627, 0x648, 0x633, 0x646, 0x20, 0x643, 0x631,
0x64a, 0x643, 0x62f, 0x646, 0x641, 0x631, 0x62f, 0x64a, 0x62a, 0x631, 0x648, 0x64a,
0x62a, 0x62f, 0x648, 0x645, 0x64a, 0x646, 0x64a, 0x643, 0x627, 0x627, 0x64a, 0x62f,
0x645, 0x648, 0x646, 0x62a, 0x648, 0x646, 0x627, 0x64a, 0x631, 0x648, 0x646, 0x628,
0x64a, 0x627, 0x644, 0x633, 0x644, 0x641, 0x627, 0x62f, 0x648, 0x631, 0x641, 0x648,
0x631, 0x62a, 0x20, 0x646, 0x64a, 0x644, 0x633, 0x648, 0x646, 0x641, 0x648, 0x631,
0x62a, 0x627, 0x644, 0x64a, 0x632, 0x627, 0x62c, 0x644, 0x627, 0x633, 0x20, 0x628,
0x627, 0x64a, 0x62c, 0x648, 0x633, 0x20, 0x628, 0x627, 0x64a, 0x63a, 0x631, 0x627,
0x646, 0x62f, 0x20, 0x62a, 0x631, 0x643, 0x63a, 0x631, 0x64a, 0x646, 0x627, 0x62f,
0x627, 0x63a, 0x648, 0x627, 0x62f, 0x644, 0x648, 0x628, 0x63a, 0x648, 0x627, 0x62a,
0x64a, 0x645, 0x627, 0x644, 0x627, 0x63a, 0x648, 0x627, 0x64a, 0x627, 0x643, 0x648,
0x64a, 0x644, 0x63a, 0x64a, 0x627, 0x646, 0x627, 0x647, 0x627, 0x644, 0x64a, 0x641,
0x627, 0x643, 0x633, 0x647, 0x627, 0x641, 0x627, 0x646, 0x627, 0x647, 0x64a, 0x631,
0x645, 0x648, 0x633, 0x64a, 0x644, 0x648, 0x625, 0x646, 0x62f, 0x64a, 0x627, 0x646,
0x627, 0x628, 0x648, 0x644, 0x64a, 0x633, 0x643, 0x648, 0x646, 0x643, 0x633, 0x645,
0x627, 0x631, 0x646, 0x62c, 0x648, 0x628, 0x64a, 0x62a, 0x631, 0x633, 0x628, 0x631,
0x63a, 0x645, 0x62f, 0x64a, 0x646, 0x629, 0x20, 0x62a, 0x644, 0x60c, 0x20, 0x625,
0x646, 0x62f, 0x64a, 0x627, 0x646, 0x627, 0x641, 0x64a, 0x641, 0x627, 0x64a, 0x641,
0x64a, 0x646, 0x633, 0x64a, 0x646, 0x633, 0x648, 0x64a, 0x646, 0x627, 0x645, 0x627,
0x643, 0x627, 0x64a, 0x646, 0x648, 0x641, 0x64a, 0x643, 0x627, 0x643, 0x648, 0x64a,
0x644, 0x62a, 0x62c, 0x627, 0x645, 0x627, 0x64a, 0x643, 0x627, 0x62c, 0x648, 0x646,
0x64a, 0x644, 0x648, 0x64a, 0x633, 0x20, 0x641, 0x64a, 0x644, 0x645, 0x648, 0x646,
0x62a, 0x64a, 0x633, 0x64a, 0x644, 0x648, 0x643, 0x631, 0x627, 0x644, 0x646, 0x62f,
0x64a, 0x643, 0x644, 0x627, 0x20, 0x628, 0x627, 0x632, 0x644, 0x64a, 0x645, 0x627,
0x644, 0x648, 0x633, 0x20, 0x627, 0x646, 0x62c, 0x644, 0x648, 0x633, 0x62d, 0x64a,
0x20, 0x627, 0x644, 0x623, 0x645, 0x64a, 0x631, 0x20, 0x627, 0x644, 0x633, 0x641,
0x644, 0x64a, 0x645, 0x627, 0x634, 0x64a, 0x648, 0x645, 0x627, 0x646, 0x627, 0x63a,
0x648, 0x627, 0x645, 0x627, 0x646, 0x627, 0x648, 0x633, 0x645, 0x627, 0x631, 0x64a,
0x63a, 0x648, 0x62a, 0x627, 0x644, 0x645, 0x627, 0x631, 0x62a, 0x64a, 0x646, 0x64a,
0x643, 0x645, 0x627, 0x62a, 0x627, 0x645, 0x648, 0x631, 0x648, 0x633, 0x645, 0x627,
0x632, 0x627, 0x62a, 0x644, 0x627, 0x646, 0x645, 0x64a, 0x646, 0x648, 0x645, 0x64a,
0x646, 0x64a, 0x645, 0x64a, 0x631, 0x64a, 0x62f, 0x627, 0x645, 0x64a, 0x62a, 0x644,
0x627, 0x643, 0x627, 0x62a, 0x644, 0x627, 0x645, 0x643, 0x633, 0x64a, 0x643, 0x648,
0x20, 0x633, 0x64a, 0x62a, 0x64a, 0x645, 0x643, 0x648, 0x64a, 0x644, 0x648, 0x646,
0x648, 0x64a, 0x646, 0x643, 0x62a, 0x648, 0x646, 0x645, 0x648, 0x646, 0x62a, 0x64a,
0x631, 0x64a, 0x645, 0x648, 0x646, 0x62a, 0x641, 0x64a, 0x62f, 0x64a, 0x648, 0x645,
0x648, 0x646, 0x62a, 0x633, 0x64a, 0x631, 0x627, 0x62a, 0x646, 0x627, 0x633, 0x648,
0x646, 0x64a, 0x648, 0x64a, 0x648, 0x631, 0x643, 0x646, 0x648, 0x631, 0x648, 0x646,
0x627, 0x647, 0x628, 0x64a, 0x648, 0x644, 0x627, 0x60c, 0x20, 0x62f, 0x627, 0x643,
0x648, 0x62a, 0x627, 0x20, 0x627, 0x644, 0x634, 0x645, 0x627, 0x644, 0x64a, 0x629,
0x633, 0x646, 0x62a, 0x631, 0x646, 0x64a, 0x648, 0x20, 0x633, 0x627, 0x644, 0x64a,
0x645, 0x63a, 0x648, 0x62f, 0x62b, 0x627, 0x628, 0x623, 0x648, 0x62c, 0x64a, 0x646,
0x627, 0x62c, 0x627, 0x628, 0x646, 0x645, 0x627, 0x628, 0x627, 0x631, 0x627, 0x645,
0x627, 0x631, 0x64a, 0x628, 0x648, 0x641, 0x64a, 0x646, 0x643, 0x633, 0x628, 0x648,
0x631, 0x62a, 0x20, 0x623, 0x648, 0x20, 0x628, 0x631, 0x646, 0x633, 0x628, 0x648,
0x631, 0x62a, 0x20, 0x623, 0x648, 0x641, 0x20, 0x633, 0x628, 0x64a, 0x646, 0x628,
0x648, 0x631, 0x62a, 0x648, 0x20, 0x641, 0x64a, 0x644, 0x648, 0x628, 0x648, 0x631,
0x62a, 0x648, 0x631, 0x64a, 0x643, 0x648, 0x628, 0x648, 0x646, 0x62a, 0x627, 0x20,
0x623, 0x631, 0x64a, 0x646, 0x627, 0x632, 0x631, 0x627, 0x646, 0x643, 0x646, 0x20,
0x627, 0x646, 0x644, 0x62a, 0x631, 0x64a, 0x633, 0x64a, 0x641, 0x631, 0x64a, 0x62c,
0x64a, 0x646, 0x627, 0x631, 0x64a, 0x632, 0x648, 0x644, 0x648, 0x62a, 0x631, 0x64a,
0x648, 0x628, 0x631, 0x627, 0x646, 0x643, 0x648, 0x633, 0x627, 0x646, 0x62a, 0x627,
0x631, 0x64a, 0x645, 0x633, 0x627, 0x646, 0x62a, 0x64a, 0x627, 0x63a, 0x648, 0x633,
0x627, 0x646, 0x62a, 0x648, 0x20, 0x62f, 0x648, 0x645, 0x64a, 0x646, 0x63a, 0x648,
0x633, 0x627, 0x648, 0x20, 0x628, 0x627, 0x648, 0x644, 0x648, 0x633, 0x643, 0x648,
0x631, 0x633, 0x628, 0x64a, 0x633, 0x646, 0x62f, 0x633, 0x64a, 0x62a, 0x643, 0x627,
0x633, 0x627, 0x646, 0x62a, 0x20, 0x628, 0x627, 0x631, 0x62a, 0x64a, 0x644, 0x64a,
0x645, 0x64a, 0x633, 0x627, 0x646, 0x62a, 0x20, 0x62c, 0x648, 0x646, 0x633, 0x633,
0x627, 0x646, 0x62a, 0x20, 0x643, 0x64a, 0x62a, 0x633, 0x633, 0x627, 0x646, 0x62a,
0x20, 0x644, 0x648, 0x634, 0x64a, 0x627, 0x633, 0x627, 0x646, 0x62a, 0x20, 0x62a,
0x648, 0x645, 0x627, 0x633, 0x633, 0x627, 0x646, 0x62a, 0x20, 0x641, 0x646, 0x633,
0x646, 0x62a, 0x633, 0x648, 0x641, 0x62a, 0x20, 0x643, 0x627, 0x631, 0x646, 0x62a,
0x62a, 0x64a, 0x63a, 0x648, 0x633, 0x64a, 0x63a, 0x627, 0x644, 0x628, 0x627, 0x62b,
0x64a, 0x644, 0x62a, 0x64a, 0x62e, 0x648, 0x627, 0x646, 0x627, 0x62a, 0x648, 0x631,
0x648, 0x646, 0x62a, 0x648, 0x62a, 0x648, 0x631, 0x62a, 0x648, 0x644, 0x627, 0x641,
0x627, 0x646, 0x643, 0x648, 0x641, 0x631, 0x648, 0x627, 0x64a, 0x62a, 0x20, 0x647,
0x648, 0x631, 0x633, 0x648, 0x64a, 0x646, 0x64a, 0x628, 0x64a, 0x62c, 0x64a, 0x627,
0x643, 0x648, 0x62a, 0x627, 0x62a, 0x643, 0x627, 0x633, 0x627, 0x64a, 0x62f, 0x627,
0x641, 0x64a, 0x632, 0x62f, 0x64a, 0x20, 0x645, 0x648, 0x646, 0x62a, 0x20, 0x62f,
0x648, 0x20, 0x631, 0x648, 0x641, 0x64a, 0x644, 0x645, 0x627, 0x643, 0x648, 0x627,
0x631, 0x64a, 0x645, 0x627, 0x648, 0x633, 0x648, 0x646, 0x645, 0x627, 0x643, 0x20,
0x645, 0x648, 0x631, 0x62f, 0x648, 0x628, 0x627, 0x644, 0x645, 0x64a, 0x631, 0x631,
0x648, 0x62b, 0x64a, 0x631, 0x627, 0x633, 0x627, 0x64a, 0x648, 0x648, 0x627, 0x62a,
0x631, 0x648, 0x644, 0x641, 0x648, 0x633, 0x62a, 0x648, 0x643, 0x644, 0x648, 0x646,
0x62c, 0x64a, 0x631, 0x628, 0x64a, 0x646, 0x639, 0x62f, 0x646, 0x623, 0x644, 0x645,
0x627, 0x62a, 0x64a, 0x639, 0x645, 0x651, 0x627, 0x646, 0x623, 0x646, 0x62f, 0x64a,
0x631, 0x623, 0x643, 0x62a, 0x627, 0x648, 0x623, 0x643, 0x62a, 0x648, 0x628, 0x639,
0x634, 0x642, 0x20, 0x622, 0x628, 0x627, 0x62f, 0x623, 0x62a, 0x64a, 0x631, 0x627,
0x648, 0x628, 0x63a, 0x62f, 0x627, 0x62f, 0x627, 0x644, 0x628, 0x62d, 0x631, 0x64a,
0x646, 0x628, 0x627, 0x646, 0x643, 0x648, 0x643, 0x628, 0x627, 0x631, 0x646, 0x627,
0x648, 0x644, 0x628, 0x64a, 0x631, 0x648, 0x62a, 0x628, 0x634, 0x643, 0x64a, 0x643,
0x628, 0x631, 0x648, 0x646, 0x627, 0x64a, 0x62a, 0x634, 0x64a, 0x62a, 0x627, 0x643,
0x648, 0x644, 0x648, 0x645, 0x628, 0x648, 0x62f, 0x645, 0x634, 0x642, 0x62f, 0x64a,
0x644, 0x64a, 0x62f, 0x628, 0x64a, 0x62f, 0x648, 0x634, 0x627, 0x646, 0x628, 0x64a,
0x641, 0x627, 0x645, 0x627, 0x63a, 0x648, 0x633, 0x62a, 0x627, 0x63a, 0x632, 0x629,
0x647, 0x64a, 0x628, 0x631, 0x648, 0x646, 0x20, 0x28, 0x645, 0x62f, 0x64a, 0x646,
0x629, 0x20, 0x627, 0x644, 0x62e, 0x644, 0x64a, 0x644, 0x29, 0x645, 0x62f, 0x64a,
0x646, 0x629, 0x20, 0x647, 0x648, 0x20, 0x62a, 0x634, 0x64a, 0x20, 0x645, 0x646,
0x629, 0x647, 0x648, 0x646, 0x63a, 0x20, 0x643, 0x648, 0x646, 0x63a, 0x647, 0x648,
0x641, 0x62f, 0x627, 0x64a, 0x631, 0x643, 0x64a, 0x62a, 0x633, 0x643, 0x62c, 0x627,
0x643, 0x631, 0x62a, 0x627, 0x62c, 0x627, 0x64a, 0x627, 0x628, 0x64a, 0x648, 0x631,
0x627, 0x627, 0x644, 0x642, 0x62f, 0x633, 0x643, 0x627, 0x628, 0x648, 0x644, 0x643,
0x627, 0x645, 0x62a, 0x634, 0x627, 0x62a, 0x643, 0x627, 0x643, 0x631, 0x627, 0x62a,
0x634, 0x64a, 0x643, 0x627, 0x62a, 0x645, 0x627, 0x646, 0x62f, 0x648, 0x62e, 0x627,
0x646, 0x62f, 0x64a, 0x62c, 0x627, 0x643, 0x627, 0x644, 0x643, 0x62a, 0x627, 0x643,
0x631, 0x627, 0x633, 0x646, 0x648, 0x64a, 0x627, 0x631, 0x633, 0x643, 0x643, 0x648,
0x627, 0x644, 0x627, 0x20, 0x644, 0x627, 0x645, 0x628, 0x648, 0x631, 0x643, 0x64a,
0x634, 0x64a, 0x646, 0x62c, 0x627, 0x644, 0x643, 0x648, 0x64a, 0x62a, 0x645, 0x627,
0x643, 0x627, 0x648, 0x645, 0x62c, 0x627, 0x62f, 0x646, 0x645, 0x627, 0x643, 0x633,
0x627, 0x631, 0x645, 0x627, 0x646, 0x64a, 0x644, 0x627, 0x645, 0x633, 0x642, 0x637,
0x646, 0x64a, 0x642, 0x648, 0x633, 0x64a, 0x627, 0x646, 0x648, 0x641, 0x648, 0x643,
0x648, 0x632, 0x646, 0x62a, 0x633, 0x643, 0x646, 0x648, 0x641, 0x648, 0x633, 0x628,
0x64a, 0x631, 0x633, 0x643, 0x623, 0x648, 0x645, 0x633, 0x643, 0x623, 0x648, 0x631,
0x627, 0x644, 0x628, 0x646, 0x648, 0x645, 0x20, 0x628, 0x646, 0x647, 0x628, 0x648,
0x646, 0x62a, 0x64a, 0x627, 0x646, 0x643, 0x628, 0x64a, 0x648, 0x646, 0x63a, 0x20,
0x64a, 0x627, 0x646, 0x63a, 0x642, 0x637, 0x631, 0x642, 0x648, 0x633, 0x62a, 0x627,
0x646, 0x627, 0x64a, 0x643, 0x64a, 0x632, 0x64a, 0x644, 0x648, 0x631, 0x62f, 0x627,
0x627, 0x644, 0x631, 0x64a, 0x627, 0x636, 0x633, 0x643, 0x627, 0x644, 0x64a, 0x646,
0x633, 0x645, 0x631, 0x642, 0x646, 0x62f, 0x633, 0x648, 0x644, 0x634, 0x646, 0x63a,
0x647, 0x627, 0x64a, 0x633, 0x646, 0x63a, 0x627, 0x641, 0x648, 0x631, 0x629, 0x633,
0x631, 0x64a, 0x62f, 0x646, 0x643, 0x648, 0x644, 0x64a, 0x645, 0x633, 0x643, 0x62a,
0x627, 0x64a, 0x628, 0x64a, 0x647, 0x637, 0x634, 0x642, 0x646, 0x62f, 0x62a, 0x628,
0x644, 0x64a, 0x633, 0x64a, 0x637, 0x647, 0x631, 0x627, 0x646, 0x62a, 0x64a, 0x645,
0x641, 0x648, 0x637, 0x648, 0x643, 0x64a, 0x648, 0x62a, 0x648, 0x645, 0x633, 0x643,
0x622, 0x644, 0x627, 0x646, 0x628, 0x627, 0x62a, 0x627, 0x631, 0x623, 0x631, 0x648,
0x645, 0x643, 0x64a, 0x623, 0x648, 0x633, 0x62a, 0x20, 0x646, 0x64a, 0x631, 0x627,
0x641, 0x64a, 0x627, 0x646, 0x62a, 0x64a, 0x627, 0x646, 0x641, 0x644, 0x627, 0x62f,
0x64a, 0x641, 0x648, 0x633, 0x62a, 0x643, 0x64a, 0x627, 0x643, 0x62a, 0x633, 0x643,
0x631, 0x627, 0x646, 0x63a, 0x648, 0x646, 0x64a, 0x643, 0x627, 0x62a, 0x631, 0x646,
0x628, 0x64a, 0x631, 0x62c, 0x64a, 0x631, 0x64a, 0x641, 0x627, 0x646, 0x623, 0x632,
0x648, 0x631, 0x633, 0x628, 0x631, 0x645, 0x648, 0x62f, 0x627, 0x643, 0x646, 0x627,
0x631, 0x64a, 0x627, 0x644, 0x631, 0x623, 0x633, 0x20, 0x627, 0x644, 0x623, 0x62e,
0x636, 0x631, 0x641, 0x627, 0x631, 0x648, 0x645, 0x627, 0x62f, 0x64a, 0x631, 0x627,
0x631, 0x64a, 0x643, 0x64a, 0x627, 0x641, 0x64a, 0x643, 0x62c, 0x648, 0x631, 0x62c,
0x64a, 0x627, 0x20, 0x627, 0x644, 0x62c, 0x646, 0x648, 0x628, 0x64a, 0x629, 0x633,
0x627, 0x646, 0x62a, 0x20, 0x647, 0x64a, 0x644, 0x64a, 0x646, 0x627, 0x627, 0x633,
0x62a, 0x627, 0x646, 0x644, 0x64a, 0x623, 0x62f, 0x64a, 0x644, 0x64a, 0x62f, 0x628,
0x631, 0x633, 0x64a, 0x628, 0x627, 0x646, 0x628, 0x631, 0x648, 0x643, 0x646, 0x20,
0x647, 0x64a, 0x644, 0x62f, 0x627, 0x631, 0x648, 0x646, 0x623, 0x648, 0x643, 0x644,
0x627, 0x647, 0x648, 0x628, 0x627, 0x631, 0x62a, 0x644, 0x64a, 0x646, 0x62f, 0x645,
0x627, 0x646, 0x644, 0x648, 0x631, 0x62f, 0x20, 0x647, 0x627, 0x648, 0x645, 0x64a,
0x644, 0x628, 0x648, 0x631, 0x646, 0x628, 0x631, 0x62b, 0x627, 0x633, 0x64a, 0x62f,
0x646, 0x64a, 0x623, 0x645, 0x633, 0x62a, 0x631, 0x62f, 0x627, 0x645, 0x623, 0x646,
0x62f, 0x648, 0x631, 0x627, 0x623, 0x633, 0x62a, 0x631, 0x627, 0x62e, 0x627, 0x646,
0x623, 0x62b, 0x64a, 0x646, 0x627, 0x628, 0x644, 0x63a, 0x631, 0x627, 0x62f, 0x628,
0x631, 0x644, 0x64a, 0x646, 0x628, 0x631, 0x627, 0x62a, 0x64a, 0x633, 0x644, 0x627,
0x641, 0x627, 0x628, 0x631, 0x648, 0x643, 0x633, 0x644, 0x628, 0x648, 0x62e, 0x627,
0x631, 0x633, 0x62a, 0x628, 0x648, 0x62f, 0x627, 0x628, 0x633, 0x62a, 0x628, 0x648,
0x633, 0x646, 0x63a, 0x646, 0x62a, 0x634, 0x64a, 0x633, 0x64a, 0x646, 0x627, 0x648,
0x643, 0x648, 0x628, 0x646, 0x647, 0x627, 0x63a, 0x646, 0x62f, 0x628, 0x644, 0x646,
0x62c, 0x628, 0x644, 0x20, 0x637, 0x627, 0x631, 0x642, 0x63a, 0x64a, 0x631, 0x646,
0x632, 0x64a, 0x647, 0x644, 0x633, 0x646, 0x643, 0x64a, 0x62c, 0x632, 0x64a, 0x631,
0x629, 0x20, 0x645, 0x627, 0x646, 0x625, 0x633, 0x637, 0x646, 0x628, 0x648, 0x644,
0x62c, 0x64a, 0x631, 0x633, 0x64a, 0x643, 0x627, 0x644, 0x64a, 0x646, 0x62c, 0x631,
0x627, 0x62f, 0x643, 0x64a, 0x631, 0x648, 0x641, 0x643, 0x64a, 0x64a, 0x641, 0x644,
0x634, 0x628, 0x648, 0x646, 0x629, 0x644, 0x64a, 0x648, 0x628, 0x644, 0x64a, 0x627,
0x646, 0x627, 0x644, 0x646, 0x62f, 0x646, 0x644, 0x648, 0x643, 0x633, 0x645, 0x628,
0x648, 0x631, 0x63a, 0x645, 0x62f, 0x631, 0x64a, 0x62f, 0x645, 0x627, 0x644, 0x637,
0x629, 0x645, 0x627, 0x631, 0x64a, 0x647, 0x627, 0x645, 0x646, 0x645, 0x64a, 0x646,
0x633, 0x643, 0x645, 0x648, 0x646, 0x627, 0x643, 0x648, 0x645, 0x648, 0x633, 0x643,
0x648, 0x623, 0x648, 0x633, 0x644, 0x648, 0x628, 0x627, 0x631, 0x64a, 0x633, 0x628,
0x648, 0x62f, 0x63a, 0x648, 0x631, 0x64a, 0x643, 0x627, 0x628, 0x631, 0x627, 0x63a,
0x631, 0x64a, 0x63a, 0x627, 0x633, 0x645, 0x631, 0x627, 0x621, 0x633, 0x627, 0x646,
0x20, 0x645, 0x627, 0x631, 0x64a, 0x646, 0x648, 0x633, 0x631, 0x627, 0x64a, 0x64a,
0x641, 0x648, 0x633, 0x627, 0x631, 0x627, 0x62a, 0x648, 0x641, 0x633, 0x64a, 0x645,
0x641, 0x631, 0x648, 0x628, 0x648, 0x644, 0x633, 0x643, 0x648, 0x628, 0x64a, 0x635,
0x648, 0x641, 0x64a, 0x627, 0x633, 0x62a, 0x648, 0x643, 0x647, 0x648, 0x644, 0x645,
0x62a, 0x627, 0x644, 0x64a, 0x646, 0x62a, 0x64a, 0x631, 0x627, 0x646, 0x627, 0x623,
0x648, 0x644, 0x64a, 0x627, 0x646, 0x648, 0x641, 0x633, 0x643, 0x641, 0x627, 0x62f,
0x648, 0x632, 0x627, 0x644, 0x641, 0x627, 0x62a, 0x64a, 0x643, 0x627, 0x646, 0x641,
0x64a, 0x64a, 0x646, 0x627, 0x641, 0x64a, 0x644, 0x646, 0x64a, 0x648, 0x633, 0x641,
0x648, 0x644, 0x648, 0x62c, 0x631, 0x627, 0x62f, 0x648, 0x627, 0x631, 0x633, 0x648,
0x632, 0x63a, 0x631, 0x628, 0x632, 0x64a, 0x648, 0x631, 0x62e, 0x623, 0x646, 0x62a,
0x627, 0x646, 0x627, 0x646, 0x627, 0x631, 0x64a, 0x641, 0x648, 0x62a, 0x634, 0x627,
0x63a, 0x648, 0x633, 0x643, 0x631, 0x64a, 0x633, 0x645, 0x627, 0x633, 0x643, 0x648,
0x643, 0x648, 0x633, 0x62c, 0x632, 0x631, 0x20, 0x627, 0x644, 0x642, 0x645, 0x631,
0x643, 0x64a, 0x631, 0x63a, 0x648, 0x64a, 0x644, 0x64a, 0x646, 0x645, 0x627, 0x647,
0x64a, 0x627, 0x644, 0x645, 0x627, 0x644, 0x62f, 0x64a, 0x641, 0x645, 0x648, 0x631,
0x64a, 0x634, 0x64a, 0x648, 0x633, 0x645, 0x627, 0x64a, 0x648, 0x62a, 0x631, 0x64a,
0x648, 0x646, 0x64a, 0x648, 0x646, 0x623, 0x628, 0x64a, 0x627, 0x623, 0x648, 0x643,
0x644, 0x627, 0x646, 0x62f, 0x628, 0x648, 0x63a, 0x627, 0x646, 0x641, 0x64a, 0x644,
0x62a, 0x634, 0x627, 0x62b, 0x627, 0x645, 0x627, 0x633, 0x62a, 0x631, 0x625, 0x64a,
0x641, 0x627, 0x62a, 0x641, 0x627, 0x643, 0x627, 0x648, 0x641, 0x648, 0x641, 0x64a,
0x62c, 0x64a, 0x641, 0x648, 0x646, 0x627, 0x641, 0x648, 0x62a, 0x64a, 0x62c, 0x644,
0x627, 0x628, 0x627, 0x62c, 0x648, 0x633, 0x62c, 0x627, 0x645, 0x628, 0x64a, 0x631,
0x63a, 0x648, 0x627, 0x62f, 0x627, 0x644, 0x643, 0x627, 0x646, 0x627, 0x644, 0x647,
0x648, 0x646, 0x648, 0x644, 0x648, 0x644, 0x648, 0x627, 0x646, 0x62f, 0x631, 0x628,
0x64a, 0x631, 0x62c, 0x643, 0x64a, 0x631, 0x64a, 0x62a, 0x64a, 0x20, 0x645, 0x627,
0x62a, 0x64a, 0x643, 0x648, 0x633, 0x631, 0x627, 0x643, 0x648, 0x627, 0x62c, 0x627,
0x644, 0x64a, 0x646, 0x645, 0x627, 0x62c, 0x648, 0x631, 0x648, 0x645, 0x627, 0x631,
0x643, 0x64a, 0x633, 0x627, 0x633, 0x645, 0x64a, 0x62f, 0x648, 0x627, 0x64a, 0x646,
0x627, 0x648, 0x631, 0x648, 0x646, 0x648, 0x631, 0x641, 0x648, 0x644, 0x643, 0x646,
0x648, 0x645, 0x64a, 0x627, 0x628, 0x627, 0x63a, 0x648, 0x20, 0x628, 0x627, 0x63a,
0x648, 0x628, 0x627, 0x644, 0x627, 0x648, 0x628, 0x64a, 0x62a, 0x643, 0x64a, 0x631,
0x646, 0x628, 0x627, 0x646, 0x627, 0x628, 0x628, 0x648, 0x631, 0x20, 0x645, 0x648,
0x631, 0x633, 0x628, 0x64a, 0x631, 0x627, 0x631, 0x648, 0x62a, 0x648, 0x646, 0x63a,
0x627, 0x633, 0x627, 0x64a, 0x628, 0x627, 0x646, 0x62a, 0x627, 0x647, 0x64a, 0x62a,
0x64a, 0x62a, 0x627, 0x631, 0x627, 0x648, 0x627, 0x62a, 0x648, 0x646, 0x63a, 0x627,
0x62a, 0x627, 0x628, 0x648, 0x648, 0x627, 0x644, 0x64a, 0x633, 0x531, 0x562, 0x56b,
0x57b, 0x561, 0x576, 0x531, 0x584, 0x580, 0x561, 0x531, 0x564, 0x56b, 0x57d, 0x20,
0x531, 0x562, 0x565, 0x562, 0x561, 0x531, 0x56c, 0x56a, 0x56b, 0x580, 0x531, 0x57d,
0x574, 0x565, 0x580, 0x561, 0x532, 0x561, 0x574, 0x561, 0x56f, 0x578, 0x532, 0x561,
0x576, 0x563, 0x56b, 0x532, 0x561, 0x576, 0x56a, 0x578, 0x582, 0x56c, 0x532, 0x56b,
0x57d, 0x561, 0x578, 0x582, 0x532, 0x56c, 0x561, 0x576, 0x569, 0x561, 0x575, 0x580,
0x532, 0x580, 0x561, 0x566, 0x561, 0x57e, 0x56b, 0x56c, 0x532, 0x578, 0x582, 0x56a,
0x578, 0x582, 0x574, 0x562, 0x578, 0x582, 0x580, 0x561, 0x53f, 0x561, 0x570, 0x56b,
0x580, 0x565, 0x53f, 0x561, 0x57d, 0x561, 0x562, 0x56c, 0x561, 0x576, 0x56f, 0x561,
0x54d, 0x565, 0x578, 0x582, 0x57f, 0x561, 0x53f, 0x578, 0x576, 0x561, 0x56f, 0x580,
0x56b, 0x534, 0x561, 0x584, 0x561, 0x580, 0x534, 0x561, 0x580, 0x2d, 0x537, 0x57d,
0x2d, 0x54d, 0x561, 0x56c, 0x561, 0x574, 0x54b, 0x56b, 0x562, 0x578, 0x582, 0x569,
0x56b, 0x534, 0x578, 0x582, 0x561, 0x56c, 0x561, 0x537, 0x56c, 0x20, 0x531, 0x575,
0x578, 0x582, 0x576, 0x556, 0x580, 0x56b, 0x569, 0x561, 0x578, 0x582, 0x576, 0x533,
0x561, 0x562, 0x578, 0x580, 0x578, 0x576, 0x565, 0x540, 0x561, 0x580, 0x561, 0x580,
0x565, 0x545, 0x578, 0x570, 0x561, 0x576, 0x565, 0x57d, 0x562, 0x578, 0x582, 0x580,
0x563, 0x54b, 0x578, 0x582, 0x562, 0x561, 0x53f, 0x561, 0x574, 0x57a, 0x561, 0x56c,
0x561, 0x53d, 0x561, 0x580, 0x569, 0x578, 0x582, 0x574, 0x53f, 0x56b, 0x563, 0x561,
0x56c, 0x56b, 0x53f, 0x56b, 0x576, 0x577, 0x561, 0x57d, 0x561, 0x53c, 0x561, 0x563,
0x578, 0x57d, 0x53c, 0x56b, 0x562, 0x580, 0x587, 0x56b, 0x56c, 0x53c, 0x578, 0x574,
0x565, 0x53c, 0x578, 0x582, 0x561, 0x576, 0x564, 0x561, 0x53c, 0x578, 0x582, 0x562,
0x578, 0x582, 0x574, 0x562, 0x561, 0x577, 0x56b, 0x53c, 0x578, 0x582, 0x57d, 0x561,
0x56f, 0x561, 0x544, 0x561, 0x56c, 0x561, 0x562, 0x578, 0x544, 0x561, 0x57a, 0x578,
0x582, 0x57f, 0x578, 0x582, 0x544, 0x561, 0x57d, 0x565, 0x580, 0x578, 0x582, 0x544,
0x562, 0x561, 0x562, 0x561, 0x576, 0x565, 0x544, 0x578, 0x563, 0x561, 0x564, 0x56b,
0x577, 0x578, 0x544, 0x578, 0x576, 0x580, 0x578, 0x57e, 0x56b, 0x561, 0x546, 0x561,
0x575, 0x580, 0x578, 0x562, 0x56b, 0x546, 0x57b, 0x561, 0x574, 0x565, 0x576, 0x561,
0x546, 0x56b, 0x561, 0x574, 0x565, 0x575, 0x546, 0x578, 0x582, 0x561, 0x56f, 0x577,
0x578, 0x57f, 0x548, 0x582, 0x561, 0x563, 0x561, 0x564, 0x578, 0x582, 0x563, 0x578,
0x582, 0x54a, 0x578, 0x580, 0x57f, 0x578, 0x20, 0x546, 0x578, 0x57e, 0x578, 0x54d,
0x561, 0x576, 0x20, 0x54f, 0x578, 0x574, 0x565, 0x54f, 0x580, 0x56b, 0x57a, 0x578,
0x56c, 0x56b, 0x539, 0x578, 0x582, 0x576, 0x56b, 0x57d, 0x54e, 0x56b, 0x576, 0x564,
0x570, 0x578, 0x582, 0x56f, 0x531, 0x564, 0x561, 0x56f, 0x20, 0x56f, 0x572, 0x566,
0x56b, 0x531, 0x576, 0x584, 0x578, 0x580, 0x56b, 0x57b, 0x531, 0x576, 0x563, 0x578,
0x582, 0x56b, 0x56c, 0x561, 0x531, 0x576, 0x57f, 0x56b, 0x563, 0x578, 0x582, 0x561,
0x531, 0x580, 0x561, 0x563, 0x578, 0x582, 0x561, 0x56b, 0x576, 0x561, 0x532, 0x578,
0x582, 0x565, 0x576, 0x578, 0x57d, 0x20, 0x531, 0x575, 0x580, 0x565, 0x57d, 0x53f,
0x561, 0x57f, 0x561, 0x574, 0x561, 0x580, 0x56f, 0x561, 0x53f, 0x578, 0x580, 0x564,
0x578, 0x57e, 0x561, 0x53a, 0x578, 0x582, 0x56a, 0x578, 0x582, 0x575, 0x53c, 0x561,
0x20, 0x54c, 0x56b, 0x578, 0x56d, 0x561, 0x544, 0x565, 0x576, 0x564, 0x578, 0x57d,
0x561, 0x54c, 0x56b, 0x578, 0x20, 0x533, 0x561, 0x56c, 0x575, 0x565, 0x563, 0x578,
0x57d, 0x54d, 0x561, 0x56c, 0x57f, 0x561, 0x54d, 0x561, 0x576, 0x20, 0x53d, 0x578,
0x582, 0x561, 0x576, 0x54d, 0x561, 0x576, 0x20, 0x53c, 0x578, 0x582, 0x56b, 0x57d,
0x54f, 0x578, 0x582, 0x56f, 0x578, 0x582, 0x574, 0x561, 0x576, 0x548, 0x582, 0x577,
0x578, 0x582, 0x561, 0x575, 0x561, 0x531, 0x580, 0x578, 0x582, 0x562, 0x561, 0x531,
0x57d, 0x578, 0x582, 0x576, 0x57d, 0x575, 0x578, 0x576, 0x531, 0x57f, 0x56b, 0x56f,
0x578, 0x56f, 0x561, 0x576, 0x532, 0x561, 0x56b, 0x575, 0x561, 0x532, 0x561, 0x56b,
0x575, 0x561, 0x20, 0x532, 0x561, 0x576, 0x564, 0x565, 0x580, 0x561, 0x57d, 0x532,
0x561, 0x580, 0x562, 0x561, 0x564, 0x578, 0x57d, 0x532, 0x565, 0x56c, 0x565, 0x574,
0x532, 0x565, 0x56c, 0x56b, 0x566, 0x532, 0x56c, 0x561, 0x576, 0x56f, 0x2d, 0x54d,
0x561, 0x562, 0x56c, 0x578, 0x576, 0x532, 0x578, 0x561, 0x20, 0x54e, 0x56b, 0x57d,
0x57f, 0x561, 0x532, 0x578, 0x563, 0x578, 0x57f, 0x561, 0x532, 0x578, 0x575, 0x57d,
0x565, 0x554, 0x565, 0x574, 0x562, 0x580, 0x56b, 0x57b, 0x20, 0x532, 0x565, 0x575,
0x53f, 0x561, 0x574, 0x57a, 0x578, 0x582, 0x20, 0x533, 0x580, 0x561, 0x576, 0x564,
0x56b, 0x53f, 0x561, 0x576, 0x56f, 0x578, 0x582, 0x576, 0x53f, 0x561, 0x580, 0x561,
0x56f, 0x561, 0x57d, 0x53f, 0x561, 0x575, 0x565, 0x576, 0x53f, 0x561, 0x575, 0x574,
0x561, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x549, 0x56b,
0x56f, 0x561, 0x563, 0x578, 0x549, 0x56b, 0x578, 0x582, 0x561, 0x578, 0x582, 0x561,
0x54d, 0x575, 0x578, 0x582, 0x564, 0x561, 0x564, 0x20, 0x53d, 0x578, 0x582, 0x561,
0x580, 0x565, 0x57d, 0x53f, 0x578, 0x57d, 0x57f, 0x561, 0x20, 0x54c, 0x56b, 0x56f,
0x561, 0x554, 0x580, 0x565, 0x57d, 0x569, 0x578, 0x576, 0x53f, 0x578, 0x582, 0x575,
0x561, 0x562, 0x561, 0x53f, 0x575, 0x578, 0x582, 0x580, 0x561, 0x57d, 0x561, 0x578,
0x534, 0x565, 0x576, 0x574, 0x561, 0x580, 0x584, 0x57d, 0x570, 0x561, 0x57e, 0x576,
0x534, 0x578, 0x578, 0x582, 0x57d, 0x578, 0x576, 0x534, 0x578, 0x578, 0x582, 0x57d,
0x578, 0x576, 0x20, 0x554, 0x580, 0x56b, 0x584, 0x534, 0x565, 0x576, 0x57e, 0x565,
0x580, 0x534, 0x565, 0x57f, 0x580, 0x578, 0x575, 0x569, 0x534, 0x578, 0x574, 0x56b,
0x576, 0x56b, 0x56f, 0x561, 0x537, 0x564, 0x574, 0x578, 0x576, 0x57f, 0x578, 0x576,
0x537, 0x575, 0x580, 0x578, 0x582, 0x576, 0x565, 0x57a, 0x565, 0x54d, 0x561, 0x56c,
0x57e, 0x561, 0x564, 0x578, 0x580, 0x556, 0x578, 0x580, 0x57f, 0x20, 0x546, 0x565,
0x56c, 0x57d, 0x578, 0x576, 0x556, 0x578, 0x580, 0x57f, 0x561, 0x56c, 0x565, 0x566,
0x561, 0x533, 0x56c, 0x565, 0x575, 0x57d, 0x20, 0x532, 0x565, 0x575, 0x533, 0x578,
0x582, 0x57d, 0x20, 0x532, 0x565, 0x575, 0x533, 0x580, 0x561, 0x576, 0x564, 0x20,
0x54f, 0x575, 0x578, 0x580, 0x584, 0x533, 0x580, 0x565, 0x576, 0x561, 0x564, 0x561,
0x533, 0x57e, 0x561, 0x564, 0x565, 0x56c, 0x578, 0x582, 0x57a, 0x561, 0x533, 0x57e,
0x561, 0x57f, 0x565, 0x574, 0x561, 0x56c, 0x561, 0x533, 0x578, 0x582, 0x561, 0x575,
0x561, 0x56f, 0x56b, 0x56c, 0x533, 0x561, 0x575, 0x561, 0x576, 0x561, 0x540, 0x561,
0x56c, 0x56b, 0x586, 0x561, 0x584, 0x57d, 0x540, 0x561, 0x57e, 0x561, 0x576, 0x561,
0x537, 0x580, 0x574, 0x578, 0x57d, 0x56b, 0x56c, 0x575, 0x578, 0x53b, 0x576, 0x564,
0x56b, 0x561, 0x576, 0x561, 0x57a, 0x578, 0x56c, 0x56b, 0x57d, 0x546, 0x578, 0x584,
0x57d, 0x2c, 0x20, 0x53b, 0x576, 0x564, 0x56b, 0x561, 0x576, 0x561, 0x544, 0x561,
0x580, 0x565, 0x576, 0x563, 0x578, 0x2c, 0x20, 0x53b, 0x576, 0x564, 0x56b, 0x561,
0x576, 0x561, 0x54a, 0x565, 0x57f, 0x565, 0x580, 0x57d, 0x562, 0x578, 0x582, 0x580,
0x563, 0x2c, 0x20, 0x53b, 0x576, 0x564, 0x56b, 0x561, 0x576, 0x561, 0x539, 0x565,
0x56c, 0x20, 0x54d, 0x56b, 0x569, 0x56b, 0x2c, 0x20, 0x53b, 0x576, 0x564, 0x56b,
0x561, 0x576, 0x561, 0x54e, 0x56b, 0x57e, 0x565, 0x575, 0x2c, 0x20, 0x53b, 0x576,
0x564, 0x56b, 0x561, 0x576, 0x561, 0x54e, 0x56b, 0x576, 0x57d, 0x565, 0x576, 0x57d,
0x2c, 0x20, 0x53b, 0x576, 0x564, 0x56b, 0x561, 0x576, 0x561, 0x54e, 0x56b, 0x576,
0x561, 0x574, 0x561, 0x56f, 0x2c, 0x20, 0x53b, 0x576, 0x564, 0x56b, 0x561, 0x576,
0x561, 0x53b, 0x576, 0x578, 0x582, 0x57e, 0x56b, 0x56f, 0x53b, 0x56f, 0x561, 0x56c,
0x578, 0x582, 0x56b, 0x57f, 0x543, 0x561, 0x574, 0x561, 0x575, 0x56f, 0x561, 0x54b,
0x578, 0x582, 0x576, 0x578, 0x53c, 0x578, 0x582, 0x56b, 0x57d, 0x57e, 0x56b, 0x56c,
0x544, 0x578, 0x576, 0x57f, 0x56b, 0x57d, 0x565, 0x56c, 0x578, 0x2c, 0x20, 0x53f,
0x565, 0x576, 0x57f, 0x578, 0x582, 0x56f, 0x56b, 0x53f, 0x580, 0x561, 0x56c, 0x565,
0x576, 0x564, 0x565, 0x575, 0x56f, 0x53c, 0x561, 0x20, 0x54a, 0x561, 0x57d, 0x53c,
0x56b, 0x574, 0x561, 0x53c, 0x578, 0x57d, 0x20, 0x531, 0x576, 0x57b, 0x565, 0x56c,
0x565, 0x57d, 0x53c, 0x578, 0x578, 0x582, 0x565, 0x580, 0x20, 0x54a, 0x580, 0x56b,
0x576, 0x57d, 0x565, 0x57d, 0x20, 0x554, 0x57e, 0x578, 0x580, 0x569, 0x565, 0x580,
0x544, 0x561, 0x57d, 0x565, 0x575, 0x578, 0x544, 0x561, 0x576, 0x561, 0x563, 0x578,
0x582, 0x561, 0x544, 0x561, 0x576, 0x561, 0x578, 0x582, 0x57d, 0x544, 0x561, 0x580,
0x56b, 0x563, 0x578, 0x544, 0x561, 0x580, 0x57f, 0x56b, 0x576, 0x56b, 0x56f, 0x561,
0x544, 0x561, 0x57f, 0x561, 0x574, 0x578, 0x580, 0x578, 0x57d, 0x544, 0x561, 0x566,
0x561, 0x569, 0x56c, 0x561, 0x576, 0x544, 0x565, 0x576, 0x578, 0x574, 0x56b, 0x576,
0x56b, 0x544, 0x565, 0x580, 0x56b, 0x564, 0x561, 0x544, 0x565, 0x57f, 0x56c, 0x561,
0x56f, 0x561, 0x57f, 0x56c, 0x561, 0x544, 0x565, 0x56d, 0x56b, 0x56f, 0x578, 0x544,
0x56b, 0x584, 0x565, 0x56c, 0x578, 0x576, 0x544, 0x578, 0x576, 0x56f, 0x57f, 0x578,
0x576, 0x544, 0x578, 0x576, 0x57f, 0x565, 0x57c, 0x565, 0x575, 0x544, 0x578, 0x576,
0x57f, 0x565, 0x57e, 0x56b, 0x564, 0x565, 0x578, 0x544, 0x578, 0x576, 0x57d, 0x565,
0x57c, 0x561, 0x57f, 0x546, 0x561, 0x57d, 0x561, 0x578, 0x582, 0x546, 0x575, 0x578,
0x582, 0x20, 0x545, 0x578, 0x580, 0x584, 0x546, 0x578, 0x574, 0x546, 0x578, 0x580,
0x578, 0x576, 0x575, 0x561, 0x532, 0x578, 0x575, 0x56c, 0x561, 0x2c, 0x20, 0x540,
0x575, 0x578, 0x582, 0x57d, 0x56b, 0x57d, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x534,
0x561, 0x56f, 0x578, 0x57f, 0x561, 0x540, 0x575, 0x578, 0x582, 0x57d, 0x56b, 0x57d,
0x561, 0x575, 0x56b, 0x576, 0x20, 0x534, 0x561, 0x56f, 0x578, 0x57f, 0x561, 0x20,
0x2d, 0x20, 0x53f, 0x565, 0x576, 0x57f, 0x580, 0x578, 0x576, 0x546, 0x575, 0x578,
0x582, 0x20, 0x54d, 0x561, 0x56c, 0x565, 0x574, 0x2c, 0x20, 0x540, 0x575, 0x578,
0x582, 0x57d, 0x56b, 0x57d, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x534, 0x561, 0x56f,
0x578, 0x57f, 0x561, 0x546, 0x578, 0x582, 0x578, 0x582, 0x56f, 0x555, 0x56d, 0x56b,
0x576, 0x561, 0x563, 0x561, 0x54a, 0x561, 0x576, 0x561, 0x574, 0x561, 0x54a, 0x561,
0x580, 0x561, 0x574, 0x561, 0x580, 0x56b, 0x562, 0x578, 0x556, 0x56b, 0x576, 0x56b,
0x584, 0x57d, 0x54a, 0x578, 0x580, 0x57f, 0x2d, 0x585, 0x2d, 0x54a, 0x580, 0x565,
0x576, 0x57d, 0x54a, 0x578, 0x580, 0x57f, 0x2d, 0x585, 0x586, 0x2d, 0x54d, 0x583,
0x565, 0x575, 0x576, 0x54a, 0x578, 0x580, 0x57f, 0x578, 0x582, 0x20, 0x54e, 0x565,
0x56c, 0x575, 0x578, 0x582, 0x54a, 0x578, 0x582, 0x565, 0x580, 0x57f, 0x578, 0x20,
0x54c, 0x56b, 0x56f, 0x578, 0x54a, 0x578, 0x582, 0x576, 0x57f, 0x561, 0x20, 0x531,
0x580, 0x565, 0x576, 0x561, 0x57d, 0x54c, 0x561, 0x576, 0x56f, 0x56b, 0x576, 0x20,
0x53b, 0x576, 0x56c, 0x565, 0x569, 0x54c, 0x565, 0x57d, 0x56b, 0x586, 0x56b, 0x54c,
0x565, 0x57b, 0x561, 0x575, 0x576, 0x561, 0x54c, 0x565, 0x566, 0x578, 0x56c, 0x575,
0x578, 0x582, 0x57f, 0x54c, 0x56b, 0x578, 0x582, 0x20, 0x532, 0x580, 0x561, 0x576,
0x56f, 0x578, 0x582, 0x54d, 0x561, 0x576, 0x57f, 0x561, 0x580, 0x565, 0x574, 0x54d,
0x561, 0x576, 0x57f, 0x575, 0x561, 0x563, 0x578, 0x54d, 0x561, 0x576, 0x57f, 0x578,
0x20, 0x534, 0x578, 0x574, 0x56b, 0x576, 0x563, 0x578, 0x54d, 0x561, 0x576, 0x20,
0x54a, 0x561, 0x578, 0x582, 0x56c, 0x578, 0x582, 0x54d, 0x56f, 0x578, 0x580, 0x57d,
0x562, 0x56b, 0x57d, 0x578, 0x582, 0x576, 0x54d, 0x56b, 0x57f, 0x56f, 0x561, 0x54d,
0x565, 0x576, 0x20, 0x532, 0x561, 0x580, 0x57f, 0x565, 0x56c, 0x574, 0x56b, 0x54d,
0x565, 0x576, 0x569, 0x20, 0x54b, 0x578, 0x576, 0x57d, 0x54d, 0x565, 0x576, 0x569,
0x20, 0x554, 0x56b, 0x57f, 0x57d, 0x54d, 0x565, 0x576, 0x569, 0x20, 0x53c, 0x575,
0x578, 0x582, 0x57d, 0x56b, 0x561, 0x54d, 0x565, 0x576, 0x569, 0x20, 0x539, 0x578,
0x574, 0x561, 0x57d, 0x54d, 0x565, 0x576, 0x569, 0x20, 0x54e, 0x56b, 0x576, 0x57d,
0x565, 0x576, 0x569, 0x54d, 0x57e, 0x56b, 0x586, 0x569, 0x20, 0x554, 0x561, 0x580,
0x565, 0x576, 0x569, 0x54f, 0x565, 0x563, 0x578, 0x582, 0x57d, 0x56b, 0x563, 0x561,
0x56c, 0x57a, 0x561, 0x54f, 0x578, 0x582, 0x56c, 0x565, 0x54f, 0x56b, 0x56d, 0x578,
0x582, 0x561, 0x576, 0x561, 0x54f, 0x578, 0x580, 0x578, 0x576, 0x57f, 0x578, 0x539,
0x578, 0x580, 0x569, 0x578, 0x56c, 0x561, 0x54e, 0x561, 0x576, 0x56f, 0x578, 0x582,
0x57e, 0x565, 0x580, 0x548, 0x582, 0x561, 0x575, 0x569, 0x570, 0x578, 0x580, 0x57d,
0x54e, 0x56b, 0x576, 0x56b, 0x57a, 0x565, 0x563, 0x545, 0x561, 0x56f, 0x578, 0x582,
0x57f, 0x561, 0x57f, 0x554, 0x565, 0x575, 0x57d, 0x56b, 0x534, 0x565, 0x575, 0x57e,
0x56b, 0x57d, 0x534, 0x575, 0x578, 0x582, 0x574, 0x578, 0x576, 0x20, 0x564, 0x2019,
0x545, 0x578, 0x582, 0x580, 0x57e, 0x56b, 0x56c, 0x544, 0x561, 0x56f, 0x56f, 0x578,
0x582, 0x578, 0x580, 0x56b, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x544, 0x578, 0x578,
0x582, 0x57d, 0x578, 0x576, 0x544, 0x561, 0x56f, 0x2d, 0x544, 0x565, 0x580, 0x564,
0x578, 0x54a, 0x561, 0x56c, 0x574, 0x565, 0x580, 0x54c, 0x578, 0x57f, 0x565, 0x580,
0x561, 0x54d, 0x575, 0x578, 0x57e, 0x561, 0x54f, 0x580, 0x578, 0x56c, 0x56c, 0x54e,
0x578, 0x57d, 0x57f, 0x578, 0x56f, 0x53c, 0x578, 0x576, 0x563, 0x575, 0x56b, 0x580,
0x531, 0x564, 0x565, 0x576, 0x531, 0x56c, 0x574, 0x561, 0x569, 0x56b, 0x531, 0x574,
0x574, 0x561, 0x576, 0x531, 0x576, 0x561, 0x564, 0x56b, 0x580, 0x531, 0x56f, 0x57f,
0x561, 0x578, 0x582, 0x531, 0x56f, 0x57f, 0x578, 0x562, 0x565, 0x531, 0x577, 0x56d,
0x561, 0x562, 0x561, 0x564, 0x531, 0x57f, 0x56b, 0x580, 0x561, 0x578, 0x582, 0x532,
0x561, 0x572, 0x564, 0x561, 0x564, 0x532, 0x561, 0x570, 0x580, 0x565, 0x575, 0x576,
0x532, 0x561, 0x584, 0x578, 0x582, 0x532, 0x561, 0x576, 0x563, 0x56f, 0x578, 0x56f,
0x532, 0x561, 0x57c, 0x576, 0x561, 0x578, 0x582, 0x56c, 0x532, 0x565, 0x575, 0x580,
0x578, 0x582, 0x569, 0x532, 0x56b, 0x577, 0x584, 0x565, 0x56f, 0x532, 0x580, 0x578,
0x582, 0x576, 0x565, 0x575, 0x549, 0x56b, 0x57f, 0x561, 0x53f, 0x578, 0x56c, 0x578,
0x574, 0x562, 0x578, 0x534, 0x561, 0x574, 0x561, 0x57d, 0x56f, 0x578, 0x57d, 0x534,
0x561, 0x584, 0x584, 0x561, 0x534, 0x56b, 0x56c, 0x56b, 0x534, 0x578, 0x582, 0x562,
0x561, 0x575, 0x534, 0x578, 0x582, 0x577, 0x561, 0x576, 0x562, 0x565, 0x556, 0x561,
0x574, 0x561, 0x563, 0x578, 0x582, 0x57d, 0x57f, 0x561, 0x533, 0x561, 0x566, 0x561,
0x540, 0x565, 0x562, 0x580, 0x578, 0x576, 0x540, 0x578, 0x577, 0x56b, 0x574, 0x56b,
0x576, 0x540, 0x578, 0x576, 0x56f, 0x578, 0x576, 0x563, 0x540, 0x578, 0x57e, 0x564,
0x53b, 0x580, 0x56f, 0x578, 0x582, 0x57f, 0x57d, 0x56f, 0x54b, 0x561, 0x56f, 0x561,
0x580, 0x57f, 0x561, 0x54b, 0x561, 0x575, 0x57a, 0x578, 0x582, 0x580, 0x561, 0x535,
0x580, 0x578, 0x582, 0x57d, 0x561, 0x572, 0x565, 0x574, 0x554, 0x561, 0x562, 0x578,
0x582, 0x56c, 0x53f, 0x561, 0x574, 0x579, 0x561, 0x57f, 0x56f, 0x561, 0x53f, 0x561,
0x580, 0x561, 0x579, 0x56b, 0x53f, 0x561, 0x57f, 0x574, 0x561, 0x576, 0x564, 0x578,
0x582, 0x53d, 0x561, 0x576, 0x564, 0x56b, 0x563, 0x561, 0x53f, 0x561, 0x56c, 0x56f,
0x578, 0x582, 0x57f, 0x561, 0x53f, 0x580, 0x561, 0x57d, 0x576, 0x578, 0x575, 0x561,
0x580, 0x57d, 0x56f, 0x53f, 0x578, 0x582, 0x561, 0x56c, 0x561, 0x20, 0x53c, 0x578,
0x582, 0x574, 0x57a, 0x578, 0x582, 0x580, 0x53f, 0x578, 0x582, 0x579, 0x56b, 0x576,
0x563, 0x554, 0x578, 0x582, 0x57e, 0x565, 0x575, 0x569, 0x544, 0x561, 0x56f, 0x561,
0x578, 0x544, 0x561, 0x563, 0x561, 0x564, 0x561, 0x576, 0x544, 0x561, 0x56f, 0x561,
0x57d, 0x561, 0x580, 0x544, 0x561, 0x576, 0x56b, 0x56c, 0x561, 0x544, 0x561, 0x57d,
0x56f, 0x561, 0x57f, 0x546, 0x56b, 0x56f, 0x578, 0x57d, 0x56b, 0x561, 0x546, 0x578,
0x57e, 0x578, 0x56f, 0x578, 0x582, 0x566, 0x576, 0x565, 0x581, 0x56f, 0x546, 0x578,
0x57e, 0x578, 0x57d, 0x56b, 0x562, 0x56b, 0x580, 0x57d, 0x56f, 0x555, 0x574, 0x57d,
0x56f, 0x548, 0x582, 0x580, 0x561, 0x56c, 0x57d, 0x56f, 0x54a, 0x576, 0x578, 0x574,
0x57a, 0x565, 0x576, 0x54a, 0x578, 0x576, 0x57f, 0x56b, 0x561, 0x576, 0x561, 0x56f,
0x553, 0x56d, 0x565, 0x576, 0x575, 0x561, 0x576, 0x53f, 0x561, 0x57f, 0x561, 0x580,
0x53f, 0x578, 0x57d, 0x57f, 0x561, 0x576, 0x561, 0x575, 0x53f, 0x56b, 0x566, 0x56b,
0x56c, 0x578, 0x580, 0x564, 0x561, 0x537, 0x580, 0x20, 0x54c, 0x56b, 0x561, 0x564,
0x54d, 0x561, 0x56d, 0x561, 0x56c, 0x56b, 0x576, 0x54d, 0x561, 0x574, 0x561, 0x580,
0x572, 0x561, 0x576, 0x564, 0x54d, 0x565, 0x578, 0x582, 0x56c, 0x547, 0x561, 0x576,
0x570, 0x561, 0x575, 0x54d, 0x56b, 0x576, 0x563, 0x561, 0x57a, 0x578, 0x582, 0x580,
0x54d, 0x580, 0x565, 0x564, 0x576, 0x565, 0x56f, 0x578, 0x56c, 0x56b, 0x574, 0x57d,
0x56f, 0x539, 0x561, 0x575, 0x57a, 0x565, 0x575, 0x54f, 0x561, 0x577, 0x584, 0x565,
0x576, 0x564, 0x539, 0x562, 0x56b, 0x56c, 0x56b, 0x57d, 0x56b, 0x539, 0x565, 0x570,
0x580, 0x561, 0x576, 0x54f, 0x570, 0x56b, 0x574, 0x57a, 0x570, 0x578, 0x582, 0x54f,
0x578, 0x56f, 0x56b, 0x578, 0x54f, 0x578, 0x574, 0x57d, 0x56f, 0x548, 0x582, 0x56c,
0x561, 0x576, 0x20, 0x532, 0x561, 0x57f, 0x578, 0x580, 0x548, 0x582, 0x580, 0x578,
0x582, 0x574, 0x579, 0x56b, 0x548, 0x582, 0x57d, 0x57f, 0x2d, 0x546, 0x565, 0x580,
0x561, 0x54e, 0x575, 0x565, 0x576, 0x57f, 0x575, 0x561, 0x576, 0x54e, 0x56c, 0x561,
0x564, 0x56b, 0x57e, 0x578, 0x57d, 0x57f, 0x578, 0x56f, 0x545, 0x561, 0x56f, 0x578,
0x582, 0x57f, 0x57d, 0x56f, 0x54c, 0x561, 0x576, 0x563, 0x578, 0x582, 0x576, 0x535,
0x56f, 0x561, 0x57f, 0x565, 0x580, 0x56b, 0x576, 0x562, 0x578, 0x582, 0x580, 0x563,
0x535, 0x580, 0x587, 0x561, 0x576, 0x531, 0x566, 0x578, 0x580, 0x575, 0x561, 0x576,
0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x532, 0x565, 0x580, 0x574,
0x578, 0x582, 0x564, 0x576, 0x565, 0x580, 0x53f, 0x561, 0x576, 0x561, 0x580, 0x575,
0x561, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x53f, 0x561,
0x562, 0x578, 0x20, 0x54e, 0x565, 0x580, 0x564, 0x565, 0x556, 0x561, 0x580, 0x565,
0x580, 0x575, 0x561, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580,
0x544, 0x561, 0x564, 0x565, 0x575, 0x580, 0x561, 0x54c, 0x565, 0x575, 0x56f, 0x575,
0x561, 0x57e, 0x56b, 0x56f, 0x540, 0x561, 0x580, 0x561, 0x57e, 0x561, 0x575, 0x56b,
0x576, 0x20, 0x54b, 0x578, 0x580, 0x57b, 0x56b, 0x561, 0x54d, 0x578, 0x582, 0x580,
0x562, 0x20, 0x540, 0x565, 0x572, 0x56b, 0x576, 0x565, 0x56b, 0x20, 0x56f, 0x572,
0x566, 0x56b, 0x54d, 0x569, 0x565, 0x576, 0x56c, 0x56b, 0x531, 0x564, 0x565, 0x56c,
0x561, 0x56b, 0x564, 0x561, 0x532, 0x580, 0x56b, 0x57d, 0x562, 0x565, 0x576, 0x532,
0x580, 0x578, 0x584, 0x565, 0x576, 0x20, 0x540, 0x56b, 0x56c, 0x534, 0x561, 0x580,
0x57e, 0x56b, 0x576, 0x545, 0x578, 0x582, 0x56f, 0x56c, 0x561, 0x540, 0x578, 0x562,
0x561, 0x580, 0x57f, 0x53c, 0x56b, 0x576, 0x564, 0x565, 0x574, 0x561, 0x576, 0x53c,
0x578, 0x580, 0x564, 0x20, 0x540, 0x561, 0x578, 0x582, 0x20, 0x56f, 0x572, 0x566,
0x56b, 0x544, 0x565, 0x56c, 0x562, 0x578, 0x582, 0x580, 0x576, 0x54a, 0x565, 0x580,
0x569, 0x54d, 0x56b, 0x564, 0x576, 0x565, 0x575, 0x531, 0x574, 0x57d, 0x57f, 0x565,
0x580, 0x564, 0x561, 0x574, 0x531, 0x576, 0x564, 0x578, 0x580, 0x580, 0x561, 0x531,
0x57d, 0x57f, 0x580, 0x561, 0x56d, 0x561, 0x576, 0x531, 0x569, 0x565, 0x576, 0x584,
0x532, 0x565, 0x56c, 0x563, 0x580, 0x561, 0x564, 0x532, 0x565, 0x57c, 0x56c, 0x56b,
0x576, 0x532, 0x580, 0x561, 0x57f, 0x56b, 0x57d, 0x56c, 0x561, 0x57e, 0x561, 0x532,
0x580, 0x575, 0x578, 0x582, 0x57d, 0x565, 0x56c, 0x532, 0x578, 0x582, 0x56d, 0x561,
0x580, 0x565, 0x57d, 0x57f, 0x532, 0x578, 0x582, 0x564, 0x561, 0x57a, 0x565, 0x577,
0x57f, 0x532, 0x575, 0x578, 0x582, 0x566, 0x56b, 0x576, 0x563, 0x565, 0x576, 0x554,
0x56b, 0x577, 0x576, 0x587, 0x53f, 0x578, 0x57a, 0x565, 0x576, 0x570, 0x561, 0x563,
0x565, 0x576, 0x534, 0x578, 0x582, 0x562, 0x56c, 0x56b, 0x576, 0x54b, 0x56b, 0x562,
0x580, 0x561, 0x56c, 0x569, 0x561, 0x580, 0x533, 0x565, 0x580, 0x576, 0x57d, 0x56b,
0x540, 0x565, 0x56c, 0x57d, 0x56b, 0x576, 0x56f, 0x56b, 0x544, 0x565, 0x576, 0x20,
0x56f, 0x572, 0x566, 0x56b, 0x54d, 0x57f, 0x561, 0x574, 0x562, 0x578, 0x582, 0x56c,
0x54b, 0x565, 0x580, 0x57d, 0x56b, 0x53f, 0x561, 0x56c, 0x56b, 0x576, 0x56b, 0x576,
0x563, 0x580, 0x561, 0x564, 0x53f, 0x56b, 0x580, 0x578, 0x57e, 0x53f, 0x56b, 0x587,
0x53c, 0x56b, 0x57d, 0x561, 0x562, 0x578, 0x576, 0x53c, 0x575, 0x578, 0x582, 0x562,
0x56c, 0x575, 0x561, 0x576, 0x561, 0x53c, 0x578, 0x576, 0x564, 0x578, 0x576, 0x53c,
0x575, 0x578, 0x582, 0x584, 0x57d, 0x565, 0x574, 0x562, 0x578, 0x582, 0x580, 0x563,
0x544, 0x561, 0x564, 0x580, 0x56b, 0x564, 0x544, 0x561, 0x56c, 0x569, 0x561, 0x544,
0x561, 0x580, 0x56b, 0x565, 0x570, 0x561, 0x574, 0x576, 0x544, 0x56b, 0x576, 0x57d,
0x56f, 0x544, 0x578, 0x576, 0x561, 0x56f, 0x578, 0x544, 0x578, 0x57d, 0x56f, 0x57e,
0x561, 0x555, 0x57d, 0x56c, 0x578, 0x553, 0x561, 0x580, 0x56b, 0x566, 0x54a, 0x578,
0x564, 0x563, 0x578, 0x580, 0x56b, 0x581, 0x561, 0x54a, 0x580, 0x561, 0x570, 0x561,
0x54c, 0x56b, 0x563, 0x561, 0x540, 0x57c, 0x578, 0x574, 0x54d, 0x561, 0x574, 0x561,
0x580, 0x561, 0x54d, 0x561, 0x576, 0x20, 0x544, 0x561, 0x580, 0x56b, 0x576, 0x578,
0x54d, 0x561, 0x580, 0x561, 0x587, 0x578, 0x54d, 0x561, 0x580, 0x561, 0x57f, 0x578,
0x57e, 0x54d, 0x56b, 0x574, 0x586, 0x565, 0x580, 0x578, 0x57a, 0x578, 0x56c, 0x54d,
0x56f, 0x578, 0x57a, 0x575, 0x565, 0x54d, 0x578, 0x586, 0x56b, 0x561, 0x54d, 0x57f,
0x578, 0x56f, 0x570, 0x578, 0x56c, 0x574, 0x54f, 0x561, 0x56c, 0x56c, 0x56b, 0x576,
0x54f, 0x56b, 0x580, 0x561, 0x576, 0x561, 0x548, 0x582, 0x56c, 0x575, 0x561, 0x576,
0x578, 0x57e, 0x57d, 0x56f, 0x54e, 0x561, 0x564, 0x578, 0x582, 0x581, 0x54e, 0x561,
0x57f, 0x56b, 0x56f, 0x561, 0x576, 0x54e, 0x56b, 0x565, 0x576, 0x576, 0x561, 0x54e,
0x56b, 0x56c, 0x576, 0x575, 0x578, 0x582, 0x57d, 0x54e, 0x578, 0x56c, 0x563, 0x578,
0x563, 0x580, 0x561, 0x564, 0x54e, 0x561, 0x580, 0x577, 0x561, 0x57e, 0x561, 0x536,
0x561, 0x563, 0x580, 0x565, 0x562, 0x551, 0x575, 0x578, 0x582, 0x580, 0x56b, 0x56d,
0x531, 0x576, 0x57f, 0x561, 0x576, 0x561, 0x576, 0x561, 0x580, 0x56b, 0x57e, 0x578,
0x582, 0x549, 0x561, 0x563, 0x578, 0x57d, 0x54d, 0x578, 0x582, 0x580, 0x562, 0x20,
0x53e, 0x576, 0x576, 0x564, 0x575, 0x561, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b,
0x53f, 0x578, 0x56f, 0x578, 0x57d, 0x575, 0x561, 0x576, 0x20, 0x56f, 0x572, 0x566,
0x56b, 0x576, 0x565, 0x580, 0x53f, 0x578, 0x574, 0x578, 0x580, 0x575, 0x561, 0x576,
0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x53f, 0x565, 0x580, 0x563,
0x565, 0x56c, 0x565, 0x576, 0x544, 0x561, 0x567, 0x544, 0x561, 0x56c, 0x564, 0x56b,
0x57e, 0x576, 0x565, 0x580, 0x544, 0x561, 0x57e, 0x580, 0x56b, 0x56f, 0x56b, 0x578,
0x57d, 0x544, 0x561, 0x575, 0x578, 0x569, 0x54c, 0x565, 0x575, 0x578, 0x582, 0x576,
0x56b, 0x578, 0x576, 0x531, 0x57a, 0x56b, 0x561, 0x555, 0x584, 0x56c, 0x565, 0x576,
0x564, 0x532, 0x578, 0x582, 0x563, 0x565, 0x576, 0x57e, 0x56b, 0x56c, 0x549, 0x561,
0x569, 0x565, 0x574, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x549, 0x578, 0x582, 0x578,
0x582, 0x56f, 0x536, 0x561, 0x57f, 0x56f, 0x56b, 0x20, 0x56f, 0x572, 0x566, 0x56b,
0x537, 0x586, 0x561, 0x57f, 0x565, 0x556, 0x561, 0x56f, 0x561, 0x578, 0x586, 0x578,
0x556, 0x56b, 0x57b, 0x56b, 0x556, 0x578, 0x582, 0x576, 0x561, 0x586, 0x578, 0x582,
0x57f, 0x56b, 0x533, 0x561, 0x56c, 0x561, 0x57a, 0x561, 0x563, 0x578, 0x57d, 0x575,
0x561, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x533, 0x561,
0x574, 0x562, 0x575, 0x565, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580,
0x533, 0x578, 0x582, 0x561, 0x564, 0x561, 0x56c, 0x56f, 0x561, 0x576, 0x561, 0x56c,
0x533, 0x578, 0x582, 0x561, 0x574, 0x540, 0x578, 0x576, 0x578, 0x56c, 0x578, 0x582,
0x56c, 0x578, 0x582, 0x537, 0x576, 0x564, 0x565, 0x580, 0x562, 0x565, 0x580, 0x56b,
0x20, 0x56f, 0x572, 0x566, 0x56b, 0x53f, 0x56b, 0x580, 0x56b, 0x57f, 0x56b, 0x574,
0x561, 0x57f, 0x56b, 0x53f, 0x578, 0x57d, 0x580, 0x561, 0x565, 0x554, 0x57e, 0x561,
0x57b, 0x561, 0x56c, 0x565, 0x575, 0x576, 0x544, 0x561, 0x57b, 0x578, 0x582, 0x580,
0x578, 0x544, 0x561, 0x580, 0x56f, 0x56b, 0x566, 0x575, 0x561, 0x576, 0x20, 0x56f,
0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x544, 0x56b, 0x564, 0x578, 0x582, 0x565,
0x575, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x546, 0x561, 0x578, 0x582, 0x580, 0x578,
0x582, 0x546, 0x56b, 0x578, 0x582, 0x565, 0x546, 0x578, 0x580, 0x586, 0x578, 0x56c,
0x56f, 0x546, 0x578, 0x582, 0x574, 0x565, 0x561, 0x54a, 0x561, 0x563, 0x578, 0x20,
0x54a, 0x561, 0x563, 0x578, 0x54a, 0x561, 0x56c, 0x561, 0x578, 0x582, 0x54a, 0x56b,
0x57f, 0x56f, 0x565, 0x57c, 0x576, 0x54a, 0x578, 0x576, 0x57a, 0x565, 0x56b, 0x54a,
0x578, 0x580, 0x57f, 0x20, 0x544, 0x578, 0x580, 0x57d, 0x562, 0x56b, 0x54c, 0x561,
0x580, 0x578, 0x57f, 0x578, 0x576, 0x563, 0x561, 0x54d, 0x561, 0x575, 0x57a, 0x561,
0x576, 0x539, 0x561, 0x56b, 0x569, 0x56b, 0x54f, 0x561, 0x580, 0x561, 0x57e, 0x561,
0x54f, 0x578, 0x576, 0x563, 0x561, 0x57f, 0x561, 0x57a, 0x578, 0x582, 0x548, 0x582,
0x565, 0x575, 0x584, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x548, 0x582, 0x578, 0x56c,
0x56b, 0x57d, 0x986, 0x9ac, 0x9bf, 0x9a1, 0x99c, 0x9be, 0x9a8, 0x98f, 0x995, 0x9cd,
0x9f0, 0x9be, 0x986, 0x9a6, 0x9cd, 0x9a6, 0x9bf, 0x99b, 0x20, 0x986, 0x9ac, 0x9be,
0x9ac, 0x9be, 0x986, 0x9b2, 0x99c, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x9cd, 0x99b,
0x986, 0x9b8, 0x9cd, 0x9ae, 0x9be, 0x9f0, 0x9be, 0x9ac, 0x9be, 0x9ae, 0x9be, 0x995,
0x9cb, 0x9ac, 0x9be, 0x982, 0x997, 0x9c1, 0x987, 0x9ac, 0x9c7, 0x99e, 0x9cd, 0x99c,
0x9c1, 0x9b2, 0x9ac, 0x9bf, 0x99b, 0x9be, 0x989, 0x9ac, 0x9cd, 0x9b2, 0x9be, 0x9a3,
0x9cd, 0x99f, 0x9be, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x9ac, 0x9cd, 0x9f0, 0x9be, 0x99c,
0x9be, 0x9ad, 0x9bf, 0x9b2, 0x9c7, 0x9ac, 0x9c1, 0x99c, 0x9c1, 0x9ae, 0x9cd, 0x9ac,
0x9c1, 0x9f0, 0x9be, 0x995, 0x9be, 0x987, 0x9f0, 0x9cb, 0x995, 0x9be, 0x99b, 0x9be,
0x9ac, 0x9cd, 0x9b2, 0x9be, 0x982, 0x995, 0x9be, 0x99a, 0x9c7, 0x989, 0x99f, 0x9be,
0x995, 0x9cb, 0x9a8, 0x9be, 0x995, 0x9cd, 0x9f0, 0x9c0, 0x9a1, 0x9be, 0x995, 0x9be,
0x9f0, 0x9a1, 0x9be, 0x9f0, 0x20, 0x98f, 0x99b, 0x20, 0x99b, 0x9be, 0x9b2, 0x9be,
0x9ae, 0x99c, 0x9bf, 0x9ac, 0x9c1, 0x99f, 0x9bf, 0x9a1, 0x9c1, 0x9f1, 0x9be, 0x9b2,
0x9be, 0x98f, 0x9b2, 0x20, 0x986, 0x987, 0x9af, 0x9bc, 0x9c1, 0x9a8, 0x9ab, 0x9cd,
0x9f0, 0x9bf, 0x99f, 0x9be, 0x989, 0x9a8, 0x997, 0x9be, 0x9ac, 0x9cb, 0x9f0, 0x9cb,
0x9a3, 0x9b9, 0x9be, 0x9f0, 0x9be, 0x9f0, 0x9c7, 0x99c, 0x9cb, 0x9b9, 0x9be, 0x9a8,
0x9cd, 0x9b8, 0x9ac, 0x9be, 0x9f0, 0x9cd, 0x997, 0x99c, 0x9c1, 0x9ac, 0x9be, 0x995,
0x9be, 0x9ae, 0x9cd, 0x9aa, 0x9be, 0x9b2, 0x9be, 0x996, 0x9be, 0x9f0, 0x9cd, 0x99f,
0x9c1, 0x9ae, 0x995, 0x9bf, 0x997, 0x9be, 0x9b2, 0x9c0, 0x995, 0x9bf, 0x982, 0x99a,
0x9be, 0x99b, 0x9be, 0x9b2, 0x9be, 0x997, 0x9cb, 0x99b, 0x9b2, 0x9bf, 0x9ac, 0x9cd,
0x9f0, 0x9bf, 0x9ad, 0x9bf, 0x9b2, 0x9c7, 0x9b2, 0x9cb, 0x9ae, 0x9b2, 0x9c1, 0x9f1,
0x9be, 0x9a3, 0x9cd, 0x9a1, 0x9be, 0x9b2, 0x9c1, 0x9ac, 0x9c1, 0x9ae, 0x9cd, 0x9ac,
0x9be, 0x99a, 0x9bf, 0x9b2, 0x9c1, 0x99b, 0x9be, 0x995, 0x9be, 0x9ae, 0x9be, 0x9b2,
0x9be, 0x9ac, 0x9cb, 0x9ae, 0x9be, 0x9aa, 0x9c1, 0x99f, 0x9cb, 0x9ae, 0x9be, 0x99b,
0x9c7, 0x9f0, 0x9c1, 0x98f, 0x9ae, 0x9cd, 0x9ac, 0x9be, 0x9ac, 0x9c7, 0x9a8, 0x9ae,
0x9cb, 0x997, 0x9be, 0x9a6, 0x9bf, 0x99a, 0x9c1, 0x9ae, 0x9cb, 0x9a8, 0x9f0, 0x9cb,
0x9ad, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x987, 0x9f0, 0x9cb, 0x9ac, 0x9bf,
0x9a8, 0x9c7, 0x9a6, 0x99c, 0x9be, 0x9ae, 0x9c7, 0x9a8, 0x9be, 0x9a8, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x9ae, 0x9c0, 0x9a8, 0x9cc, 0x986, 0x995, 0x99a, 0x9cb, 0x99f, 0x994,
0x997, 0x9be, 0x9a1, 0x9cb, 0x997, 0x9cb, 0x9aa, 0x9cb, 0x9f0, 0x9cd, 0x99f, 0x9cb,
0x2d, 0x9a8, 0x9cb, 0x9ad, 0x9cb, 0x99a, 0x9be, 0x993, 0x20, 0x99f, 0x9cb, 0x9ae,
0x9c7, 0x9a4, 0x9cd, 0x9f0, 0x9bf, 0x9aa, 0x9cb, 0x9b2, 0x9bf, 0x99f, 0x9c1, 0x9a8,
0x9bf, 0x99b, 0x9f1, 0x9bf, 0x9a3, 0x9cd, 0x9a1, 0x9b9, 0x9cb, 0x995, 0x986, 0x9a1,
0x9be, 0x995, 0x98f, 0x982, 0x995, 0x9cb, 0x9f0, 0x9c7, 0x99c, 0x98f, 0x9a8, 0x997,
0x9c1, 0x987, 0x9b2, 0x9be, 0x98f, 0x9a3, 0x9cd, 0x99f, 0x9bf, 0x997, 0x9c1, 0x9f1,
0x9be, 0x986, 0x9f0, 0x9be, 0x997, 0x9be, 0x9af, 0x9bc, 0x9c7, 0x9a8, 0x9be, 0x9ac,
0x9c1, 0x9a8, 0x99b, 0x20, 0x986, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x9cd, 0x99b, 0x995,
0x9be, 0x99f, 0x9be, 0x9ae, 0x9be, 0x9f0, 0x9cd, 0x995, 0x9be, 0x995, 0x9cb, 0x9f0,
0x9cd, 0x9a1, 0x9cb, 0x9ac, 0x9be, 0x99c, 0x9c1, 0x99c, 0x9c1, 0x987, 0x9b2, 0x9be,
0x20, 0x9f0, 0x9bf, 0x993, 0x99c, 0x9be, 0x9ae, 0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x9cb,
0x99c, 0x9be, 0x9f0, 0x9bf, 0x985, 0x2019, 0x20, 0x997, 0x9be, 0x9b2, 0x9c7, 0x997,
0x9cb, 0x99b, 0x99b, 0x9be, 0x9b2, 0x9cd, 0x99f, 0x9be, 0x99b, 0x9c7, 0x9a8, 0x20,
0x99c, 0x9c1, 0x9f1, 0x9be, 0x9a8, 0x99b, 0x9c7, 0x9a8, 0x20, 0x9b2, 0x9c1, 0x987,
0x99a, 0x99f, 0x9c1, 0x995, 0x9c1, 0x9ae, 0x9a8, 0x989, 0x99a, 0x9c1, 0x986, 0x987,
0x9af, 0x9bc, 0x9be, 0x986, 0x9f0, 0x9c1, 0x9ac, 0x9be, 0x986, 0x99b, 0x9c1, 0x9a8,
0x99a, 0x9bf, 0x9af, 0x9bc, 0x9a8, 0x986, 0x99f, 0x9bf, 0x995, 0x9cb, 0x995, 0x9be,
0x9a8, 0x9ac, 0x9be, 0x9b9, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9ac, 0x9be, 0x9b9, 0x9bf,
0x9af, 0x9bc, 0x9be, 0x20, 0x9ac, 0x9be, 0x9a8, 0x9cd, 0x9a6, 0x9c7, 0x9f0, 0x9be,
0x99b, 0x9ac, 0x9be, 0x9f0, 0x9cd, 0x9ac, 0x9be, 0x9a1, 0x9cb, 0x99a, 0x9ac, 0x9c7,
0x9b2, 0x9c7, 0x9ae, 0x9ac, 0x9c7, 0x9b2, 0x9bf, 0x99c, 0x9ac, 0x9cd, 0x9b2, 0x9be,
0x982, 0x995, 0x2d, 0x99b, 0x9c7, 0x9ac, 0x9b2, 0x9cb, 0x9a8, 0x9ac, 0x9cb, 0x9f1,
0x9be, 0x20, 0x9ad, 0x9bf, 0x9b7, 0x9cd, 0x99f, 0x9be, 0x9ac, 0x9cb, 0x997, 0x9cb,
0x99f, 0x9be, 0x9ac, 0x2019, 0x987, 0x99c, 0x9c7, 0x995, 0x9c7, 0x9ae, 0x9ac, 0x9cd,
0x9f0, 0x9bf, 0x99c, 0x20, 0x989, 0x9aa, 0x9b8, 0x9be, 0x997, 0x9f0, 0x995, 0x9c7,
0x9ae, 0x9cd, 0x9aa, 0x9cb, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c7, 0x9a3, 0x9cd, 0x9a1,
0x9c7, 0x995, 0x9c7, 0x9a8, 0x995, 0x9c1, 0x9a8, 0x995, 0x9be, 0x9f0, 0x9be, 0x995,
0x9be, 0x99b, 0x995, 0x9c7, 0x9af, 0x9bc, 0x9c7, 0x9a8, 0x995, 0x9c7, 0x987, 0x9ae,
0x9c7, 0x9a8, 0x99a, 0x9bf, 0x995, 0x9be, 0x997, 0x9cb, 0x99a, 0x9bf, 0x9b9, 0x9c1,
0x986, 0x9b9, 0x9c1, 0x9f1, 0x9be, 0x99a, 0x9bf, 0x989, 0x9a1, 0x9c7, 0x2018, 0x9a1,
0x20, 0x99c, 0x9f0, 0x9c7, 0x99c, 0x995, 0x9cb, 0x9b7, 0x9cd, 0x99f, 0x9be, 0x20,
0x9f0, 0x9bf, 0x995, 0x9be, 0x995, 0x9cd, 0x9f0, 0x9c7, 0x9b7, 0x9cd, 0x99f, 0x9a8,
0x995, 0x9c1, 0x987, 0x986, 0x9ac, 0x9be, 0x995, 0x9c1, 0x9f0, 0x9be, 0x995, 0x9be,
0x993, 0x9a1, 0x9c7, 0x9a8, 0x9ae, 0x9be, 0x9f0, 0x9cd, 0x995, 0x9b6, 0x9cd, 0x9ac,
0x9a8, 0x9a1, 0x9be, 0x993, 0x99b, 0x9a8, 0x9a1, 0x9be, 0x993, 0x99b, 0x9a8, 0x20,
0x995, 0x9cd, 0x9f0, 0x9c0, 0x995, 0x9a1, 0x9c7, 0x9a8, 0x9ad, 0x9be, 0x9f0, 0x9a1,
0x9c7, 0x99f, 0x9cd, 0x9f0, 0x9cb, 0x987, 0x99f, 0x9a1, 0x9ae, 0x9bf, 0x9a8, 0x9bf,
0x995, 0x9be, 0x98f, 0x9a1, 0x9ae, 0x9a3, 0x9cd, 0x99f, 0x9a8, 0x987, 0x9f0, 0x9c1,
0x9a8, 0x9c7, 0x9aa, 0x9c7, 0x98f, 0x9b2, 0x20, 0x99b, 0x9c7, 0x9b2, 0x9ad, 0x9c7,
0x9a1, 0x2019, 0x9f0, 0x9ab, 0x2019, 0x9f0, 0x9cd, 0x99f, 0x20, 0x9a8, 0x9c7, 0x9b2,
0x99b, 0x9a8, 0x9ab, 0x9cb, 0x9f0, 0x9cd, 0x99f, 0x9be, 0x9b2, 0x9c7, 0x99c, 0x9be,
0x997, 0x9cd, 0x9b2, 0x9c7, 0x99a, 0x20, 0x989, 0x9aa, 0x9b8, 0x9be, 0x997, 0x9f0,
0x997, 0x9c1, 0x99b, 0x20, 0x989, 0x9aa, 0x9b8, 0x9be, 0x997, 0x9f0, 0x997, 0x9cd,
0x9f0, 0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x20, 0x99f, 0x9be, 0x9f0, 0x9cd, 0x995, 0x997,
0x9cd, 0x9f0, 0x9c7, 0x9a8, 0x9be, 0x9a1, 0x9be, 0x997, 0x9c1, 0x9f1, 0x9be, 0x9a1,
0x9c7, 0x9b2, 0x9c1, 0x9aa, 0x997, 0x9c1, 0x9f1, 0x9be, 0x99f, 0x9c7, 0x9ae, 0x9be,
0x9b2, 0x9be, 0x997, 0x9be, 0x9af, 0x9bc, 0x9be, 0x995, 0x9c1, 0x987, 0x9b2, 0x997,
0x9be, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x9b9, 0x9c7, 0x9b2, 0x9bf, 0x9ab, 0x9c7,
0x995, 0x9cd, 0x9b8, 0x9b9, 0x9be, 0x9ad, 0x9be, 0x9a8, 0x9be, 0x9b9, 0x9be, 0x9f0,
0x9cd, 0x9ae, 0x9cb, 0x99b, 0x9bf, 0x9b2, 0x9cd, 0x9b2, 0x9cb, 0x987, 0x9a3, 0x9cd,
0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x9aa, 0x2019, 0x9b2, 0x9bf, 0x99b,
0x9a8, 0x995, 0x9cd, 0x9b8, 0x2c, 0x20, 0x987, 0x9a3, 0x9cd, 0x9a1, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x9a8, 0x9be, 0x9ae, 0x9be, 0x9f0, 0x9c7, 0x982, 0x997, 0x9cb, 0x2c,
0x20, 0x987, 0x9a3, 0x9cd, 0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x9aa,
0x9bf, 0x99f, 0x9be, 0x9f0, 0x9cd, 0x99b, 0x9ac, 0x9be, 0x9f0, 0x9cd, 0x997, 0x2c,
0x20, 0x987, 0x9a3, 0x9cd, 0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x99f,
0x9c7, 0x9b2, 0x20, 0x99a, 0x9bf, 0x99f, 0x9c0, 0x2c, 0x20, 0x987, 0x9a3, 0x9cd,
0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x9ad, 0x9c7, 0x9ad, 0x9c7, 0x2c,
0x20, 0x987, 0x9a3, 0x9cd, 0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x9ad,
0x9bf, 0x9a8, 0x99a, 0x9c7, 0x9a8, 0x9c7, 0x99b, 0x2c, 0x20, 0x987, 0x9a3, 0x9cd,
0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x989, 0x987, 0x9a8, 0x9be, 0x9ae,
0x9c7, 0x995, 0x2c, 0x20, 0x987, 0x9a3, 0x9cd, 0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x9a8, 0x9be, 0x987, 0x9a8, 0x9c1, 0x9ad, 0x9bf, 0x995, 0x987, 0x995, 0x9be, 0x9b2,
0x9c1, 0x987, 0x99f, 0x99c, 0x9be, 0x9ae, 0x9be, 0x987, 0x995, 0x9be, 0x99c, 0x9c1,
0x9a8, 0x9c7, 0x989, 0x9b2, 0x9c1, 0x987, 0x99b, 0x9ad, 0x9bf, 0x9b2, 0x9c7, 0x9ae,
0x9a3, 0x9cd, 0x99f, 0x9bf, 0x99a, 0x9c7, 0x9b2, 0x9cb, 0x2c, 0x20, 0x995, 0x9c7,
0x9a3, 0x9cd, 0x99f, 0x9c1, 0x995, 0x9c0, 0x995, 0x9cd, 0x9f0, 0x9be, 0x9b2, 0x9c7,
0x9a3, 0x9cd, 0x9a1, 0x9bf, 0x99c, 0x9bf, 0x995, 0x9b2, 0x9be, 0x20, 0x9aa, 0x9be,
0x99c, 0x9b2, 0x9bf, 0x9ae, 0x9be, 0x9b2, 0x99b, 0x20, 0x98f, 0x99e, 0x9cd, 0x99c,
0x9c7, 0x9b2, 0x99b, 0x9cd, 0x9b2, 0x2019, 0x9f1, 0x9be, 0x9f0, 0x20, 0x9aa, 0x9cd,
0x9f0, 0x9bf, 0x9a8, 0x9cd, 0x9b8, 0x9c7, 0x99b, 0x20, 0x995, 0x9cb, 0x9f1, 0x9be,
0x9f0, 0x9cd, 0x99f, 0x9be, 0x9f0, 0x9ae, 0x9c7, 0x99a, 0x9bf, 0x993, 0x9ae, 0x9be,
0x9a8, 0x9be, 0x997, 0x9c1, 0x9f1, 0x9be, 0x9ae, 0x9be, 0x9a8, 0x9be, 0x99b, 0x9ae,
0x9c7, 0x9f0, 0x9bf, 0x997, 0x9cb, 0x99f, 0x9ae, 0x9be, 0x9f0, 0x9cd, 0x99f, 0x9bf,
0x9a8, 0x9bf, 0x995, 0x9ae, 0x9be, 0x99f, 0x9be, 0x9ae, 0x9cb, 0x9f0, 0x9cb, 0x99b,
0x9ae, 0x9be, 0x99c, 0x9be, 0x99f, 0x9cd, 0x200c, 0x9b2, 0x9be, 0x9a8, 0x9ae, 0x9c7,
0x9a8, 0x9cb, 0x9ae, 0x9bf, 0x9a8, 0x9c0, 0x9ae, 0x9c7, 0x9f0, 0x9bf, 0x9a1, 0x9be,
0x9ae, 0x9c7, 0x99f, 0x9b2, 0x9be, 0x995, 0x9be, 0x99f, 0x9cd, 0x200c, 0x9b2, 0x9be,
0x9ae, 0x9c7, 0x995, 0x9cd, 0x9b8, 0x9bf, 0x995, 0x9cb, 0x20, 0x99a, 0x9bf, 0x99f,
0x9c0, 0x9ae, 0x9bf, 0x995, 0x9bf, 0x989, 0x9f1, 0x9c7, 0x9b2, 0x9a8, 0x9ae, 0x9a8,
0x9cd, 0x200c, 0x995, 0x99f, 0x9a8, 0x9ae, 0x9a3, 0x9cd, 0x99f, 0x9c7, 0x9f0, 0x9c7,
0x9f0, 0x9c0, 0x9ae, 0x9a3, 0x9cd, 0x99f, 0x9c7, 0x9ad, 0x9bf, 0x9a1, 0x9bf, 0x985,
0x2019, 0x9ae, 0x9a3, 0x9cd, 0x99f, 0x99b, 0x9c7, 0x9f0, 0x9be, 0x99f, 0x9a8, 0x9be,
0x99b, 0x9be, 0x989, 0x9a8, 0x9bf, 0x989, 0x20, 0x987, 0x9af, 0x9bc, 0x9f0, 0x9cd,
0x995, 0x9a8, 0x9cb, 0x9f0, 0x9cb, 0x9a8, 0x9cd, 0x200c, 0x9b9, 0x9be, 0x9ac, 0x9c7,
0x989, 0x9b2, 0x9be, 0x9b9, 0x2c, 0x20, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9f0, 0x20,
0x9a1, 0x9be, 0x995, 0x9cb, 0x99f, 0x9be, 0x99a, 0x9c7, 0x9a3, 0x9cd, 0x99f, 0x9be,
0x9f0, 0x2c, 0x20, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9f0, 0x20, 0x9a1, 0x9be, 0x995,
0x9cb, 0x99f, 0x9be, 0x9a8, 0x9bf, 0x989, 0x20, 0x99b, 0x9be, 0x9b2, 0x9c7, 0x9ae,
0x2c, 0x20, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9f0, 0x20, 0x9a1, 0x9be, 0x995, 0x9cb,
0x99f, 0x9be, 0x9a8, 0x9c1, 0x995, 0x985, 0x2019, 0x99c, 0x9bf, 0x9a8, 0x9be, 0x997,
0x9be, 0x9aa, 0x9be, 0x9a8, 0x9be, 0x9ae, 0x9be, 0x9aa, 0x9be, 0x9f0, 0x9be, 0x9ae,
0x9be, 0x9f0, 0x9bf, 0x9ac, 0x9cb, 0x9ab, 0x9bf, 0x9a8, 0x9bf, 0x995, 0x9cd, 0x9b8,
0x9aa, 0x2019, 0x9f0, 0x9cd, 0x99f, 0x2d, 0x994, 0x2d, 0x9aa, 0x9cd, 0x9f0, 0x9bf,
0x9a8, 0x9cd, 0x9b8, 0x9aa, 0x2019, 0x9f0, 0x9cd, 0x99f, 0x20, 0x985, 0x9f1, 0x20,
0x9b8, 0x9cd, 0x9aa, 0x9c7, 0x987, 0x9a8, 0x9aa, 0x9cb, 0x9f0, 0x9cd, 0x99f, 0x9cb,
0x20, 0x9ad, 0x9c7, 0x9b2, 0x9cd, 0x200c, 0x9b9, 0x9cb, 0x9aa, 0x9c1, 0x9f1, 0x9c7,
0x9f0, 0x9cd, 0x99f, 0x9cb, 0x20, 0x9f0, 0x9bf, 0x995, 0x9cb, 0x9aa, 0x9c1, 0x9a3,
0x9cd, 0x99f, 0x9be, 0x20, 0x98f, 0x9f0, 0x9bf, 0x9a8, 0x9be, 0x99b, 0x9f0, 0x9c7,
0x982, 0x995, 0x9bf, 0x9a8, 0x20, 0x987, 0x9a8, 0x9b2, 0x9c7, 0x99f, 0x9f0, 0x9c7,
0x99a, 0x9be, 0x987, 0x9ab, 0x9f0, 0x9c7, 0x99c, 0x9bf, 0x9a8, 0x9be, 0x9f0, 0x9bf,
0x99c, 0x2019, 0x9b2, 0x9bf, 0x989, 0x99f, 0x9f0, 0x9bf, 0x985, 0x2019, 0x20, 0x9ac,
0x9cd, 0x9f0, 0x9be, 0x982, 0x995, 0x9cb, 0x99b, 0x9c7, 0x9a3, 0x9cd, 0x99f, 0x9be,
0x9f0, 0x9c7, 0x9ae, 0x99b, 0x9c7, 0x9a3, 0x9cd, 0x99f, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x997, 0x9cb, 0x99b, 0x9be, 0x9a3, 0x9cd, 0x99f, 0x9cb, 0x20, 0x9a1, 0x9cb, 0x9ae,
0x9bf, 0x982, 0x997, 0x9cb, 0x99b, 0x9be, 0x993, 0x20, 0x9aa, 0x9be, 0x989, 0x9b2,
0x9cb, 0x987, 0x99f, 0x9cb, 0x995, 0x9cb, 0x9f0, 0x9cd, 0x99f, 0x9cb, 0x9f0, 0x9ae,
0x9bf, 0x99f, 0x99b, 0x9bf, 0x99f, 0x9cd, 0x200c, 0x200c, 0x995, 0x9be, 0x99b, 0x9c7,
0x987, 0x9a3, 0x9cd, 0x99f, 0x20, 0x9ac, 0x9be, 0x9f0, 0x9cd, 0x9a5, 0x9b2, 0x9c7,
0x9ae, 0x9c7, 0x99b, 0x9c7, 0x987, 0x9a3, 0x9cd, 0x99f, 0x20, 0x99c, 0x9a8, 0x99a,
0x9cd, 0x99b, 0x9c7, 0x987, 0x9a3, 0x9cd, 0x99f, 0x20, 0x995, 0x9bf, 0x99f, 0x99b,
0x99b, 0x9c7, 0x987, 0x9a3, 0x9cd, 0x99f, 0x20, 0x9b2, 0x9c1, 0x99a, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x99b, 0x9c7, 0x987, 0x9a3, 0x9cd, 0x99f, 0x20, 0x9a5, 0x9ae, 0x9be,
0x99b, 0x99b, 0x9c7, 0x987, 0x9a3, 0x9cd, 0x99f, 0x20, 0x9ad, 0x9bf, 0x9a8, 0x99a,
0x9c7, 0x9a3, 0x9cd, 0x99f, 0x99b, 0x9c1, 0x987, 0x9ab, 0x9cd, 0x99f, 0x20, 0x995,
0x9be, 0x9f0, 0x9c7, 0x9a3, 0x9cd, 0x99f, 0x99f, 0x9c7, 0x997, 0x9c1, 0x99a, 0x9bf,
0x997, 0x9be, 0x9b2, 0x9cd, 0x9aa, 0x9be, 0x9a5, 0x9cd, 0x9af, 0x9c1, 0x9b2, 0x9c7,
0x9a4, 0x9bf, 0x99c, 0x9c1, 0x9f1, 0x9be, 0x9a8, 0x9be, 0x99f, 0x2019, 0x9f0, 0x2019,
0x9a3, 0x9cd, 0x99f, 0x9cb, 0x99f, 0x9cb, 0x9f0, 0x9cd, 0x99f, 0x9cb, 0x9b2, 0x9be,
0x9ad, 0x9c7, 0x9a8, 0x995, 0x9c1, 0x9ad, 0x9c7, 0x9f0, 0x9b9, 0x9cb, 0x9f1, 0x9be,
0x987, 0x99f, 0x9b9, 0x9f0, 0x9cd, 0x99a, 0x9f1, 0x9bf, 0x9a8, 0x9bf, 0x9aa, 0x9c7,
0x997, 0x9af, 0x9bc, 0x9be, 0x995, 0x9c1, 0x99f, 0x9be, 0x99f, 0x995, 0x9c7, 0x99b,
0x9c0, 0x9a1, 0x9c7, 0x9ad, 0x9bf, 0x99b, 0x9a1, 0x9c1, 0x9ae, 0x9cb, 0x9a3, 0x9cd,
0x99f, 0x20, 0x9a1, 0x9bf, 0x20, 0x986, 0x9f0, 0x9cd, 0x9ad, 0x9bf, 0x9b2, 0x9f0,
0x9ae, 0x9c7, 0x995, 0x9cd, 0x200c, 0x995, 0x9c1, 0x9f1, 0x9c7, 0x9f0, 0x9c0, 0x9ae,
0x9be, 0x989, 0x99b, 0x9a8, 0x9ae, 0x9c7, 0x995, 0x9ae, 0x9c1, 0x9f0, 0x9cd, 0x9a1,
0x9c1, 0x9aa, 0x9be, 0x9ae, 0x9f0, 0x9f0, 0x9cb, 0x9a5, 0x9c7, 0x9f0, 0x9be, 0x9b8,
0x9cd, 0x9ac, 0x9cb, 0x9f1, 0x9be, 0x99f, 0x9cd, 0x9f0, 0x9cb, 0x9b2, 0x9ad, 0x9cb,
0x9b7, 0x9cd, 0x99f, 0x995, 0x9b2, 0x982, 0x997, 0x9c7, 0x987, 0x9f0, 0x9ac, 0x9be,
0x9af, 0x9bc, 0x9c7, 0x9a8, 0x986, 0x9a6, 0x9c7, 0x9a8, 0x986, 0x9b2, 0x9ae, 0x9be,
0x99f, 0x9bf, 0x986, 0x9ae, 0x9cd, 0x9ae, 0x9be, 0x9a8, 0x986, 0x9a8, 0x9be, 0x9a1,
0x9bf, 0x9f0, 0x98f, 0x995, 0x9cd, 0x9af, 0x9cb, 0x99f, 0x9cd, 0x9af, 0x9be, 0x993,
0x98f, 0x995, 0x9cd, 0x9af, 0x9cb, 0x99f, 0x9ac, 0x986, 0x9b6, 0x9cd, 0x9ac, 0x9cd,
0x997, 0x9be, 0x200c, 0x9ac, 0x9be, 0x99f, 0x98f, 0x99f, 0x9f0, 0x9be, 0x989, 0x9ac,
0x9be, 0x997, 0x9a6, 0x9be, 0x9a6, 0x9ac, 0x9be, 0x9b9, 0x9f0, 0x9c7, 0x987, 0x9a8,
0x9ac, 0x9c7, 0x982, 0x995, 0x995, 0x9ac, 0x9be, 0x9f0, 0x9cd, 0x9a8, 0x9c1, 0x9b2,
0x9ac, 0x9c7, 0x987, 0x9f0, 0x9c1, 0x99f, 0x9ac, 0x9bf, 0x9b6, 0x9cd, 0x9ac, 0x9cd,
0x200c, 0x995, 0x9c7, 0x995, 0x9ac, 0x9cd, 0x9f0, 0x9c1, 0x9a8, 0x9c7, 0x987, 0x99a,
0x9bf, 0x99f, 0x9be, 0x995, 0x9b2, 0x9ae, 0x9cd, 0x9ac, 0x9cb, 0x9a1, 0x9be, 0x9ae,
0x9be, 0x9b8, 0x9cd, 0x995, 0x9be, 0x99b, 0x9a2, 0x9be, 0x995, 0x9be, 0x9a1, 0x9bf,
0x9b2, 0x9bf, 0x9a1, 0x9c1, 0x9ac, 0x9be, 0x987, 0x9a1, 0x9c1, 0x99b, 0x9be, 0x9a8,
0x9ac, 0x9c7, 0x9ab, 0x9be, 0x9ae, 0x9be, 0x997, 0x9c1, 0x9b8, 0x9cd, 0x9a4, 0x9be,
0x997, 0x9be, 0x99c, 0x9be, 0x9b9, 0x9c7, 0x9ac, 0x9cd, 0x9f0, 0x9cb, 0x9a8, 0x9b9,
0x9cb, 0x20, 0x99a, 0x9bf, 0x20, 0x9ae, 0x9bf, 0x9a8, 0x20, 0x99a, 0x9bf, 0x99f,
0x9c0, 0x9b9, 0x982, 0x20, 0x995, 0x982, 0x9b9, 0x9cb, 0x9ad, 0x9cd, 0x200c, 0x9a1,
0x987, 0x9f0, 0x9cd, 0x995, 0x9c1, 0x9a4, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x99c, 0x9be,
0x995, 0x9be, 0x9f0, 0x9cd, 0x99f, 0x9be, 0x99c, 0x9be, 0x9af, 0x9bc, 0x9be, 0x9aa,
0x9c1, 0x9f0, 0x9be, 0x99c, 0x9c7, 0x9f0, 0x9c1, 0x99c, 0x9be, 0x9b2, 0x9c7, 0x9ae,
0x995, 0x9be, 0x9ac, 0x9c1, 0x9b2, 0x995, 0x9be, 0x9ae, 0x99a, 0x9a4, 0x9cd, 0x995,
0x9be, 0x995, 0x9f0, 0x9be, 0x99a, 0x9c0, 0x995, 0x9be, 0x9a0, 0x9ae, 0x9be, 0x9a3,
0x9cd, 0x9a1, 0x9c1, 0x996, 0x9be, 0x9a3, 0x9cd, 0x9a1, 0x9bf, 0x997, 0x9be, 0x995,
0x9b2, 0x995, 0x9be, 0x9a4, 0x9be, 0x995, 0x9cd, 0x9f0, 0x9be, 0x99b, 0x9a8, 0x9cb,
0x9af, 0x9bc, 0x9be, 0x9f0, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x995, 0x9c1, 0x9f1, 0x9be,
0x9b2, 0x9be, 0x9b2, 0x9c1, 0x9ae, 0x9aa, 0x9c1, 0x9f0, 0x995, 0x9c1, 0x99a, 0x9bf,
0x982, 0x995, 0x9c1, 0x9f1, 0x9c7, 0x987, 0x99f, 0x9ae, 0x9be, 0x995, 0x9be, 0x993,
0x9ae, 0x9be, 0x997, 0x9be, 0x9a6, 0x9be, 0x9a8, 0x9ae, 0x9be, 0x995, 0x9be, 0x99b,
0x9be, 0x9f0, 0x9ae, 0x9be, 0x9a8, 0x9bf, 0x9b2, 0x9be, 0x9ae, 0x9c1, 0x9b8, 0x9cd,
0x995, 0x9c7, 0x99f, 0x9a8, 0x9bf, 0x995, 0x9cb, 0x99b, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x9a8, 0x9cb, 0x9ad, 0x9cb, 0x995, 0x9c1, 0x99c, 0x9a8, 0x9c7, 0x9a4, 0x9cd, 0x9b8,
0x9cd, 0x995, 0x9a8, 0x9cb, 0x9ad, 0x9cb, 0x99b, 0x9bf, 0x9ac, 0x9bf, 0x9f0, 0x9cd,
0x9b8, 0x9cd, 0x995, 0x993, 0x9ae, 0x9b8, 0x9cd, 0x995, 0x985, 0x2019, 0x9f0, 0x9c7,
0x9b2, 0x9a8, 0x9cb, 0x9ae, 0x20, 0x9aa, 0x9c7, 0x9a8, 0x9cd, 0x200c, 0x9b9, 0x9aa,
0x9cb, 0x9a3, 0x9cd, 0x99f, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x982, 0x995, 0x9aa, 0x9cd,
0x9af, 0x982, 0x9af, 0x9bc, 0x9be, 0x982, 0x995, 0x9be, 0x99f, 0x9be, 0x9f0, 0x995,
0x2019, 0x9b7, 0x9cd, 0x99f, 0x9c7, 0x9a8, 0x9c0, 0x995, 0x9c7, 0x99c, 0x9bf, 0x9b2,
0x9cb, 0x9f0, 0x9cd, 0x9a1, 0x9be, 0x9f0, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a7, 0x99b,
0x9be, 0x996, 0x9be, 0x9b2, 0x9bf, 0x9a8, 0x99b, 0x9be, 0x9ae, 0x9be, 0x9f0, 0x995,
0x9be, 0x9a3, 0x9cd, 0x9a1, 0x99b, 0x9bf, 0x989, 0x9b2, 0x99b, 0x9be, 0x982, 0x9b9,
0x9be, 0x987, 0x99b, 0x9bf, 0x982, 0x997, 0x9be, 0x9aa, 0x9c1, 0x9f0, 0x9b8, 0x9cd,
0x9f0, 0x9c7, 0x9a1, 0x9a8, 0x9bf, 0x995, 0x9cb, 0x9b2, 0x9bf, 0x9ae, 0x9b8, 0x9cd,
0x995, 0x99f, 0x9be, 0x987, 0x9aa, 0x9c7, 0x987, 0x9a4, 0x9be, 0x99b, 0x995, 0x9c7,
0x9a3, 0x9cd, 0x99f, 0x99f, 0x9bf, 0x9ac, 0x9b2, 0x9bf, 0x99a, 0x9bf, 0x9a4, 0x9c7,
0x9b9, 0x9f0, 0x9be, 0x9a8, 0x9a5, 0x9bf, 0x9ae, 0x9cd, 0x9ab, 0x9c1, 0x99f, 0x995,
0x9bf, 0x985, 0x2019, 0x9a4, 0x9cb, 0x9ae, 0x9cd, 0x200c, 0x9b8, 0x9cd, 0x995, 0x989,
0x9b2, 0x9be, 0x9a8, 0x9ac, 0x9be, 0x99f, 0x9be, 0x9f0, 0x989, 0x9f0, 0x9c1, 0x9ae,
0x9cd, 0x200c, 0x995, 0x9bf, 0x989, 0x9b7, 0x9cd, 0x99f, 0x2d, 0x9a8, 0x9c7, 0x9f0,
0x9be, 0x9ad, 0x9bf, 0x9af, 0x9bc, 0x9c7, 0x9a3, 0x9cd, 0x99f, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x9a8, 0x9ad, 0x9cd, 0x9b2, 0x9be, 0x9a1, 0x9bf, 0x9ad, 0x9cb, 0x9b7, 0x9cd,
0x99f, 0x995, 0x9af, 0x9bc, 0x9be, 0x995, 0x9c1, 0x9a4, 0x9cd, 0x9b8, 0x9cd, 0x995,
0x9af, 0x9bc, 0x9be, 0x999, 0x9cb, 0x9a8, 0x9af, 0x9bc, 0x9c7, 0x995, 0x9be, 0x99f,
0x9c7, 0x9f0, 0x9bf, 0x9a3, 0x9ac, 0x9be, 0x9f0, 0x9cd, 0x997, 0x9af, 0x9bc, 0x9c7,
0x9f0, 0x9c7, 0x9ad, 0x9be, 0x9a8, 0x98f, 0x99c, 0x9cb, 0x9f0, 0x9c7, 0x99b, 0x9ac,
0x9be, 0x9f0, 0x9cd, 0x9ae, 0x9c1, 0x9a1, 0x9be, 0x995, 0x9c7, 0x9a8, 0x9c7, 0x9f0,
0x9c0, 0x995, 0x9c7, 0x9aa, 0x20, 0x9ad, 0x9be, 0x9f0, 0x9cd, 0x9a6, 0x9c7, 0x9ab,
0x9be, 0x9f0, 0x9cb, 0x9ae, 0x9be, 0x9a1, 0x9c7, 0x9f0, 0x9be, 0x9f0, 0x9c7, 0x987,
0x995, 0x99c, 0x9be, 0x9ad, 0x9bf, 0x995, 0x9a6, 0x995, 0x9cd, 0x9b7, 0x9bf, 0x9a3,
0x20, 0x99c, 0x9f0, 0x9cd, 0x99c, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x99b, 0x9c7, 0x987,
0x9a3, 0x9cd, 0x99f, 0x20, 0x9b9, 0x9c7, 0x9b2, 0x9c7, 0x9a8, 0x9be, 0x9b7, 0x9cd,
0x99f, 0x9c7, 0x9a8, 0x9cd, 0x200c, 0x9b2, 0x9c0, 0x98f, 0x9a1, 0x9bf, 0x9b2, 0x9c7,
0x987, 0x9a1, 0x9ac, 0x9cd, 0x9f0, 0x9bf, 0x99a, 0x9ac, 0x9c7, 0x9a8, 0x9ac, 0x9cd,
0x9f0, 0x9cb, 0x995, 0x9c7, 0x9a8, 0x20, 0x9b9, 0x9bf, 0x9b2, 0x9a1, 0x9be, 0x9f0,
0x989, 0x987, 0x9a8, 0x987, 0x989, 0x995, 0x9cd, 0x9b2, 0x9be, 0x9b9, 0x9cb, 0x9ac,
0x9be, 0x9f0, 0x9cd, 0x99f, 0x9b2, 0x9bf, 0x9a3, 0x9cd, 0x9a1, 0x9ae, 0x9be, 0x9a8,
0x9b2, 0x9f0, 0x9cd, 0x9a1, 0x20, 0x9b9, 0x9be, 0x993, 0x9ae, 0x9c7, 0x9b2, 0x9ac,
0x2019, 0x9f0, 0x9cd, 0x9a3, 0x9aa, 0x9be, 0x9f0, 0x9cd, 0x9a5, 0x99a, 0x9bf, 0x9a1,
0x9a8, 0x9c0, 0x986, 0x9ae, 0x9b7, 0x9cd, 0x99f, 0x9be, 0x9f0, 0x9a1, 0x9be, 0x9ae,
0x986, 0x9a8, 0x9cd, 0x9a6, 0x9cb, 0x9f0, 0x9be, 0x986, 0x9b7, 0x9cd, 0x99f, 0x9cd,
0x9f0, 0x9be, 0x996, 0x9be, 0x9a8, 0x98f, 0x9a5, 0x9c7, 0x9a8, 0x9cd, 0x9b8, 0x9ac,
0x9c7, 0x9b2, 0x997, 0x9cd, 0x9f0, 0x9c7, 0x9a1, 0x9ac, 0x9be, 0x9f0, 0x9cd, 0x9b2,
0x9bf, 0x9a8, 0x9ac, 0x9cd, 0x9f0, 0x9be, 0x99f, 0x9bf, 0x99b, 0x9b2, 0x9be, 0x9ad,
0x9be, 0x9ac, 0x9cd, 0x9f0, 0x9c1, 0x99b, 0x9b2, 0x9c7, 0x99b, 0x9ac, 0x9c1, 0x99a,
0x9be, 0x9f0, 0x9c7, 0x9b7, 0x9cd, 0x99f, 0x9ac, 0x9c1, 0x9a1, 0x9be, 0x9aa, 0x9c7,
0x9b7, 0x9cd, 0x99f, 0x9ac, 0x9c1, 0x99b, 0x9bf, 0x9a8, 0x99c, 0x9c7, 0x9a8, 0x99a,
0x9bf, 0x99b, 0x9bf, 0x9a8, 0x9be, 0x989, 0x995, 0x9cb, 0x9aa, 0x9c7, 0x9a8, 0x9b9,
0x9be, 0x997, 0x9c7, 0x9a8, 0x9a1, 0x9be, 0x9ac, 0x9b2, 0x9bf, 0x9a8, 0x99c, 0x9bf,
0x9ac, 0x9cd, 0x9f0, 0x9be, 0x9b2, 0x9cd, 0x99f, 0x9f0, 0x997, 0x9cb, 0x9f0, 0x9c7,
0x9a8, 0x99a, 0x9bf, 0x9b9, 0x9c7, 0x9b2, 0x99b, 0x9bf, 0x982, 0x995, 0x9bf, 0x986,
0x987, 0x9b2, 0x20, 0x985, 0x9ab, 0x20, 0x9ae, 0x9c7, 0x9a8, 0x987, 0x9b8, 0x9cd,
0x9a4, 0x9be, 0x9a8, 0x9ac, 0x9c1, 0x9b2, 0x99c, 0x9be, 0x9f0, 0x9cd, 0x99a, 0x9bf,
0x995, 0x9be, 0x9b2, 0x9bf, 0x9a8, 0x9bf, 0x9a8, 0x997, 0x9cd, 0x9f0, 0x9be, 0x9a1,
0x995, 0x9bf, 0x9f0, 0x9cb, 0x9ad, 0x995, 0x9bf, 0x9ad, 0x9b2, 0x9bf, 0x99b, 0x9ac,
0x9a8, 0x9b2, 0x9cd, 0x9af, 0x9c1, 0x9ac, 0x9c7, 0x9b2, 0x99c, 0x9be, 0x9a8, 0x9be,
0x9b2, 0x9a3, 0x9cd, 0x9a1, 0x9a8, 0x9b2, 0x9be, 0x995, 0x9cd, 0x9b8, 0x9c7, 0x9ae,
0x9ac, 0x9be, 0x9f0, 0x9cd, 0x997, 0x9ae, 0x9be, 0x9a6, 0x9cd, 0x9f0, 0x9bf, 0x9a6,
0x9ae, 0x9be, 0x9b2, 0x9cd, 0x99f, 0x9be, 0x9ae, 0x9be, 0x9f0, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x9b9, 0x9c7, 0x9ae, 0x9ae, 0x9bf, 0x982, 0x9b8, 0x9cd, 0x995, 0x9ae, 0x9cb,
0x9a8, 0x9be, 0x995, 0x9cb, 0x9ae, 0x9b8, 0x9cd, 0x995, 0x9cb, 0x993, 0x9b8, 0x9cd,
0x9b2, 0x9cb, 0x9aa, 0x9c7, 0x9f0, 0x9bf, 0x99b, 0x9aa, 0x9cb, 0x9a1, 0x997, 0x9cb,
0x9f0, 0x9bf, 0x995, 0x9be, 0x9aa, 0x9cd, 0x9f0, 0x9be, 0x997, 0x9f0, 0x9bf, 0x997,
0x9be, 0x99b, 0x9be, 0x9ae, 0x9be, 0x9f0, 0x9be, 0x99a, 0x9be, 0x9a8, 0x20, 0x9ae,
0x9be, 0x9f0, 0x9bf, 0x9a8, 0x9cb, 0x99b, 0x9be, 0x9f0, 0x9be, 0x99c, 0x9c7, 0x9ad,
0x9cb, 0x99b, 0x9be, 0x9f0, 0x9be, 0x99f, 0x9cb, 0x9ad, 0x99b, 0x9bf, 0x9ae, 0x9cd,
0x9ab, 0x9c7, 0x9f0, 0x9cb, 0x9aa, 0x9cb, 0x9b2, 0x9b8, 0x9cd, 0x995, 0x9cb, 0x9aa,
0x9cd, 0x99c, 0x9c7, 0x99b, 0x9cb, 0x9ab, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9b7, 0x9cd,
0x99f, 0x995, 0x9b9, 0x9cb, 0x9ae, 0x9a4, 0x9c7, 0x9b2, 0x9bf, 0x9a8, 0x99f, 0x9be,
0x987, 0x9f0, 0x9c7, 0x9a8, 0x989, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cb, 0x9ad,
0x9cd, 0x200c, 0x9b8, 0x9cd, 0x995, 0x9ad, 0x9be, 0x9a6, 0x9c1, 0x99c, 0x9ad, 0x9c7,
0x99f, 0x9bf, 0x995, 0x9be, 0x9a8, 0x9ad, 0x9bf, 0x9af, 0x9bc, 0x9c7, 0x9a8, 0x9be,
0x9ad, 0x9bf, 0x9b2, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x99b, 0x9ad, 0x9cb, 0x9b2,
0x9cd, 0x997, 0x9cb, 0x997, 0x9cd, 0x9f0, 0x9be, 0x9a1, 0x9f1, 0x9be, 0x9f0, 0x99b,
0x9be, 0x993, 0x99c, 0x9be, 0x997, 0x9cd, 0x9f0, 0x9c7, 0x9ac, 0x99c, 0x9c1, 0x9f0,
0x9bf, 0x996, 0x98f, 0x9a3, 0x9cd, 0x99f, 0x9be, 0x9a8, 0x9be, 0x9a8, 0x9be, 0x9f0,
0x9bf, 0x9ad, 0x9cb, 0x99a, 0x9be, 0x997, 0x9cb, 0x99b, 0x996, 0x9cd, 0x9f0, 0x9c0,
0x9b7, 0x9cd, 0x99f, 0x9ae, 0x9be, 0x99b, 0x995, 0x9cb, 0x995, 0x9cb, 0x99b, 0x995,
0x9cb, 0x9ae, 0x9cb, 0x9f0, 0x9cb, 0x995, 0x9c7, 0x9f0, 0x997, 0x9c1, 0x9b2, 0x9c7,
0x9a8, 0x9ae, 0x9be, 0x9b9, 0x9c7, 0x9ae, 0x9be, 0x9b2, 0x9a6, 0x9cd, 0x9ac, 0x9c0,
0x9aa, 0x9ae, 0x9f0, 0x9bf, 0x99b, 0x9be, 0x99b, 0x9ae, 0x9be, 0x9af, 0x9bc, 0x9cb,
0x99f, 0x9cd, 0x99f, 0x9c7, 0x9f0, 0x9bf, 0x987, 0x989, 0x9a8, 0x9bf, 0x9af, 0x9bc,
0x9a8, 0x986, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x985, 0x995, 0x9b2, 0x9c7, 0x9a3,
0x9cd, 0x9a1, 0x9ac, 0x9cb, 0x997, 0x9c7, 0x987, 0x9a8, 0x9ad, 0x9bf, 0x9b2, 0x9c7,
0x99a, 0x9be, 0x9a5, 0x9be, 0x9ae, 0x99a, 0x9cd, 0x99a, 0x9c1, 0x995, 0x987, 0x9b7,
0x9cd, 0x99f, 0x9be, 0x9f0, 0x98f, 0x9ab, 0x9c7, 0x99f, 0x9ab, 0x9be, 0x995, 0x9be,
0x993, 0x9ab, 0x9cb, 0x9ab, 0x9bf, 0x99c, 0x9bf, 0x9ab, 0x9c1, 0x9a8, 0x9be, 0x9ab,
0x9c1, 0x99f, 0x9bf, 0x997, 0x9be, 0x9b2, 0x9be, 0x9aa, 0x9be, 0x997, 0x9cb, 0x99b,
0x997, 0x9c7, 0x9ae, 0x9cd, 0x9ac, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x997, 0x9c1,
0x9f1, 0x9be, 0x9a1, 0x9b2, 0x995, 0x9be, 0x9a8, 0x9be, 0x9b2, 0x98f, 0x9a3, 0x9cd,
0x9a1, 0x9f0, 0x9ac, 0x9be, 0x9f0, 0x9c0, 0x995, 0x9bf, 0x9f0, 0x9bf, 0x99f, 0x9bf,
0x9ae, 0x9be, 0x99f, 0x9bf, 0x995, 0x9cb, 0x99b, 0x9f0, 0x9be, 0x9af, 0x9bc, 0x9c7,
0x995, 0x9cb, 0x9f1, 0x9be, 0x99c, 0x9be, 0x9b2, 0x9bf, 0x9a8, 0x9ae, 0x9be, 0x99c,
0x9c1, 0x9f0, 0x9cb, 0x9ae, 0x9be, 0x9f0, 0x9cd, 0x995, 0x99b, 0x9be, 0x99b, 0x9ae,
0x9bf, 0x9a1, 0x9f1, 0x9c7, 0x9a8, 0x9be, 0x9f0, 0x9c1, 0x9a8, 0x9bf, 0x9af, 0x9bc,
0x9cb, 0x9a8, 0x2019, 0x9f0, 0x9ab, 0x2019, 0x995, 0x9a8, 0x989, 0x9ae, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x9aa, 0x9be, 0x997, 0x9cb, 0x20, 0x9aa, 0x9be, 0x997, 0x9cb, 0x9aa,
0x9be, 0x9b2, 0x9be, 0x989, 0x9aa, 0x9bf, 0x99f, 0x995, 0x9c7, 0x987, 0x9f0, 0x9cd,
0x9a3, 0x9aa, 0x9cb, 0x9a8, 0x9aa, 0x9c7, 0x987, 0x9aa, 0x2019, 0x9f0, 0x9cd, 0x99f,
0x20, 0x9ae, 0x9cb, 0x9f0, 0x9c7, 0x99b, 0x9cd, 0x200c, 0x9ac, 0x9bf, 0x9f0, 0x9be,
0x9f0, 0x9cb, 0x99f, 0x9cb, 0x999, 0x9cd, 0x997, 0x9be, 0x99b, 0x9be, 0x987, 0x9aa,
0x9be, 0x9a8, 0x9a4, 0x9be, 0x9b9, 0x9bf, 0x9a4, 0x9bf, 0x9a4, 0x9be, 0x9f0, 0x9be,
0x9f1, 0x9be, 0x99f, 0x982, 0x997, 0x9be, 0x99f, 0x9be, 0x9aa, 0x9c1, 0x9f1, 0x9c7,
0x995, 0x9f1, 0x9be, 0x9b2, 0x9bf, 0x99b, 0x41, 0x64, 0xed, 0x73, 0x20, 0x41,
0x62, 0x65, 0x62, 0x61, 0x41, 0x72, 0x78, 0x65, 0x6c, 0x45, 0x6c, 0x20,
0x43, 0x61, 0x69, 0x72, 0x75, 0x58, 0x69, 0x62, 0x75, 0x74, 0x69, 0x4a,
0x6f, 0x68, 0x61, 0x6e, 0x6e, 0x65, 0x73, 0x62, 0x75, 0x72, 0x67, 0x75,
0x1e24, 0x61, 0x72, 0x74, 0x75, 0x6d, 0x4c, 0x6f, 0x6d, 0xe9, 0x4d, 0x6f,
0x67, 0x61, 0x64, 0x69, 0x78, 0x75, 0x58, 0x61, 0x6d, 0x65, 0x6e, 0x61,
0x4e, 0x69, 0x61, 0x6d, 0xe9, 0x69, 0x4e, 0x75, 0x61, 0x6b, 0x78, 0x6f,
0x74, 0x55, 0x61, 0x67, 0x61, 0x64, 0x75, 0x67, 0xfa, 0x53, 0x61, 0x6e,
0x74, 0x75, 0x20, 0x54, 0x6f, 0x6d, 0xe9, 0x54, 0x72, 0xed, 0x70, 0x6f,
0x6c, 0x69, 0x54, 0xfa, 0x6e, 0x65, 0x7a, 0x43, 0xf3, 0x72, 0x64, 0x6f,
0x62, 0x61, 0x54, 0x75, 0x63, 0x75, 0x6d, 0xe1, 0x6e, 0x42, 0x61, 0x68,
0xed, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x61, 0x73, 0x42,
0x6f, 0x67, 0x6f, 0x74, 0xe1, 0x43, 0x61, 0x69, 0x6d, 0xe1, 0x6e, 0x47,
0x72, 0x61, 0x6e, 0x61, 0x64, 0x61, 0x47, 0x75, 0x61, 0x64, 0x61, 0x6c,
0x75, 0x70, 0x65, 0x4c, 0x61, 0x20, 0x48, 0x61, 0x62, 0x61, 0x6e, 0x61,
0x58, 0x61, 0x6d, 0x61, 0x69, 0x63, 0x61, 0x4c, 0x61, 0x20, 0x4d, 0x61,
0x72, 0x74, 0x69, 0x6e, 0x69, 0x63, 0x61, 0x43, 0x69, 0x75, 0x64, 0xe1,
0x20, 0x64, 0x65, 0x20, 0x4d, 0xe9, 0x78, 0x69, 0x63, 0x75, 0x4e, 0x75,
0x65, 0x76, 0x61, 0x20, 0x59, 0x6f, 0x72, 0x6b, 0x42, 0x65, 0x75, 0x6c,
0x61, 0x68, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x43, 0x65, 0x6e, 0x74,
0x65, 0x72, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x4e, 0x75, 0x65, 0x76,
0x61, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x74,
0x65, 0x50, 0x61, 0x6e, 0x61, 0x6d, 0xe1, 0x50, 0x75, 0x65, 0x72, 0x74,
0x75, 0x20, 0x50, 0x72, 0xed, 0x6e, 0x63, 0x69, 0x70, 0x65, 0x50, 0x75,
0x65, 0x72, 0x74, 0x75, 0x20, 0x45, 0x73, 0x70, 0x61, 0xf1, 0x61, 0x50,
0x75, 0x65, 0x72, 0x74, 0x75, 0x20, 0x52, 0x69, 0x63, 0x75, 0x53, 0x61,
0x6e, 0x74, 0x75, 0x20, 0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x67, 0x75, 0x54,
0xf3, 0x72, 0x74, 0x6f, 0x6c, 0x61, 0x41, 0x6d, 0xe1, 0x6e, 0x42, 0x61,
0x67, 0x64, 0x61, 0x67, 0x42, 0x61, 0x1e25, 0x72, 0xe9, 0x69, 0x6e, 0x42,
0x61, 0x6b, 0xfa, 0x42, 0x72, 0x75, 0x6e, 0xe9, 0x69, 0x44, 0x61, 0x6d,
0x61, 0x73, 0x63, 0x75, 0x44, 0x75, 0x78, 0x61, 0x6e, 0x62, 0xe9, 0x48,
0x65, 0x62, 0x72, 0xf3, 0x6e, 0x43, 0x69, 0x75, 0x64, 0xe1, 0x20, 0x64,
0x65, 0x20, 0x48, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e,
0x68, 0x1e24, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x58, 0x61,
0x6b, 0x61, 0x72, 0x74, 0x61, 0x58, 0x61, 0x79, 0x61, 0x70, 0x75, 0x72,
0x61, 0x58, 0x65, 0x72, 0x75, 0x73, 0x61, 0x6c, 0xe9, 0x6e, 0x4b, 0x61,
0x74, 0x6d, 0x61, 0x6e, 0x64, 0xfa, 0x43, 0x61, 0x6c, 0x63, 0x75, 0x74,
0x61, 0x4d, 0x61, 0x63, 0xe1, 0x75, 0x4d, 0x61, 0x67, 0x61, 0x64, 0xe1,
0x6e, 0x4d, 0x61, 0x73, 0x63, 0x61, 0x74, 0x65, 0x50, 0x79, 0x6f, 0x6e,
0x67, 0x20, 0x59, 0x61, 0x6e, 0x67, 0x52, 0x69, 0x79, 0x61, 0x64, 0x53,
0x61, 0x78, 0x61, 0x6c, 0xed, 0x6e, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x63,
0x61, 0x6e, 0x64, 0x61, 0x53, 0x65, 0xfa, 0x6c, 0x53, 0x69, 0x6e, 0x67,
0x61, 0x70, 0x75, 0x72, 0x54, 0x65, 0x1e25, 0x72, 0xe1, 0x6e, 0x54, 0x6f,
0x6b, 0x69, 0x75, 0x55, 0x6c, 0xe1, 0x6e, 0x20, 0x42, 0xe1, 0x74, 0x6f,
0x72, 0x56, 0x69, 0x65, 0x6e, 0x74, 0x69, 0xe1, 0x6e, 0x52, 0x61, 0x6e,
0x67, 0xfa, 0x6e, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6d,
0x62, 0x75, 0x72, 0x67, 0x75, 0x4c, 0x65, 0x73, 0x20, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x73, 0x43, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x65, 0x73, 0x43,
0x61, 0x62, 0x75, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x49, 0x73, 0x6c,
0x6c, 0x65, 0x73, 0x20, 0x46, 0x65, 0x72, 0x6f, 0x65, 0x52, 0x65, 0x69,
0x6b, 0x69, 0x61, 0x76, 0x69, 0x6b, 0x58, 0x65, 0x6f, 0x72, 0x78, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x53, 0x75, 0x72, 0x53, 0x61, 0x6e, 0x74,
0x61, 0x20, 0x4c, 0x65, 0x6e, 0x61, 0x41, 0x64, 0x65, 0x6c, 0x61, 0x69,
0x64, 0x61, 0x41, 0x73, 0x74, 0x72, 0x61, 0x63, 0xe1, 0x6e, 0x41, 0x74,
0x65, 0x6e, 0x65, 0x73, 0x42, 0x65, 0x6c, 0x67, 0x72, 0xe1, 0x75, 0x42,
0x65, 0x72, 0x6c, 0xed, 0x6e, 0x42, 0x72, 0x75, 0x78, 0x65, 0x6c, 0x65,
0x73, 0x42, 0x75, 0x63, 0x61, 0x72, 0x65, 0x73, 0x74, 0x43, 0x6f, 0x70,
0x65, 0x6e, 0x68, 0x61, 0x67, 0x75, 0x65, 0x44, 0x75, 0x62, 0x6c, 0xed,
0x6e, 0x58, 0x69, 0x62, 0x72, 0x61, 0x6c, 0x74, 0x61, 0x72, 0x1e24, 0xe9,
0x6c, 0x73, 0x69, 0x6e, 0x6b, 0x69, 0x49, 0x73, 0x6c, 0x6c, 0x61, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x61, 0x6e, 0x4b, 0x61, 0x6c, 0x69, 0x6e, 0x69,
0x6e, 0x67, 0x72, 0xe1, 0x75, 0x4c, 0x69, 0x73, 0x62, 0x6f, 0x61, 0x4c,
0x69, 0x75, 0x62, 0x6c, 0x69, 0x61, 0x6e, 0x61, 0x4c, 0x6f, 0x6e, 0x64,
0x72, 0x65, 0x73, 0x4c, 0x75, 0x78, 0x65, 0x6d, 0x62, 0x75, 0x72, 0x67,
0x75, 0x4d, 0xf3, 0x6e, 0x61, 0x63, 0x75, 0x4d, 0x6f, 0x73, 0x63, 0xfa,
0x50, 0x61, 0x72, 0xed, 0x73, 0x50, 0x72, 0x61, 0x67, 0x61, 0x53, 0x61,
0x6e, 0x20, 0x4d, 0x61, 0x72, 0xed, 0x6e, 0x53, 0x6f, 0x66, 0xed, 0x61,
0x45, 0x73, 0x74, 0x6f, 0x63, 0x6f, 0x6c, 0x6d, 0x75, 0x54, 0x69, 0x72,
0x61, 0x6e, 0x61, 0x45, 0x6c, 0x20, 0x56, 0x61, 0x74, 0x69, 0x63, 0x61,
0x6e, 0x75, 0x56, 0x69, 0x65, 0x6e, 0x61, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0xe1, 0x75, 0x56, 0x61, 0x72, 0x73, 0x6f, 0x76, 0x69, 0x61,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x75, 0x52, 0x65, 0x75, 0x6e,
0x69, 0xf3, 0x6e, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x46, 0x69, 0x78,
0x69, 0x47, 0x61, 0x6c, 0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x41, 0x62,
0x69, 0x63, 0x61, 0x6e, 0x41, 0x6b, 0x6b, 0x72, 0x61, 0x18f, 0x64, 0x64,
0x69, 0x73, 0x20, 0x18f, 0x62, 0x259, 0x62, 0x259, 0x18f, 0x6c, 0x63, 0x259,
0x7a, 0x61, 0x69, 0x72, 0x18f, 0x73, 0x6d, 0x259, 0x72, 0x259, 0x42, 0x61,
0x6e, 0x71, 0x75, 0x69, 0x42, 0x61, 0x6e, 0x63, 0x75, 0x6c, 0x42, 0x72,
0x61, 0x7a, 0x7a, 0x61, 0x76, 0x69, 0x6c, 0x51, 0x61, 0x68, 0x69, 0x72,
0x259, 0x53, 0x65, 0x75, 0x74, 0x61, 0x44, 0x61, 0x72, 0x20, 0x65, 0x73,
0x20, 0x53, 0x61, 0x6c, 0x61, 0x6d, 0x43, 0x69, 0x62, 0x75, 0x74, 0x69,
0x44, 0x75, 0x61, 0x6c, 0x61, 0x18f, 0x6c, 0x20, 0x18f, 0x79, 0x75, 0x6e,
0x51, 0x61, 0x62, 0x6f, 0x72, 0x6f, 0x6e, 0x59, 0x6f, 0x68, 0x61, 0x6e,
0x65, 0x73, 0x62, 0x75, 0x72, 0x71, 0x58, 0x61, 0x72, 0x74, 0x75, 0x6d,
0x4b, 0x69, 0x71, 0x61, 0x6c, 0x69, 0x4b, 0x69, 0x6e, 0x15f, 0x61, 0x73,
0x61, 0x4c, 0x61, 0x71, 0x6f, 0x73, 0x4c, 0x75, 0x62, 0x75, 0x6d, 0x62,
0x61, 0x15f, 0x69, 0x4d, 0x6f, 0x71, 0x61, 0x64, 0x69, 0x15f, 0x75, 0x4e,
0x63, 0x61, 0x6d, 0x65, 0x6e, 0x61, 0x4e, 0x6f, 0x75, 0x61, 0x6b, 0x15f,
0x6f, 0x74, 0x55, 0x71, 0x61, 0x64, 0x75, 0x71, 0x75, 0x53, 0x61, 0x6e,
0x20, 0x54, 0x6f, 0x6d, 0x56, 0x69, 0x6e, 0x64, 0x68, 0x75, 0x6b, 0x41,
0x6e, 0x6b, 0x6f, 0x72, 0x61, 0x63, 0x41, 0x6e, 0x67, 0x69, 0x6c, 0x79,
0x61, 0x41, 0x6e, 0x74, 0x69, 0x71, 0x75, 0x61, 0x41, 0x72, 0x61, 0x67,
0x75, 0x61, 0x79, 0x6e, 0x61, 0x42, 0x75, 0x65, 0x6e, 0x6f, 0x73, 0x20,
0x41, 0x79, 0x72, 0x65, 0x73, 0x52, 0x69, 0x6f, 0x78, 0x61, 0x52, 0x69,
0x6f, 0x20, 0x51, 0x61, 0x6c, 0x79, 0x65, 0x71, 0x6f, 0x73, 0x53, 0x61,
0x6e, 0x20, 0x58, 0x75, 0x61, 0x6e, 0x55, 0x15f, 0x75, 0x61, 0x79, 0x61,
0x42, 0x61, 0x68, 0x69, 0x79, 0x61, 0x42, 0x6f, 0x71, 0x6f, 0x74, 0x61,
0x42, 0x6f, 0x79, 0x73, 0x65, 0x4b, 0x65, 0x6d, 0x62, 0x72, 0x69, 0x63,
0x20, 0x4b, 0xf6, 0x72, 0x66, 0x259, 0x7a, 0x69, 0x4b, 0x61, 0x6d, 0x70,
0x6f, 0x20, 0x51, 0x72, 0x61, 0x6e, 0x64, 0x65, 0x4b, 0x61, 0x79, 0x65,
0x6e, 0x4b, 0x61, 0x79, 0x6d, 0x61, 0x6e, 0xc7, 0x69, 0x6b, 0x61, 0x71,
0x6f, 0xc7, 0x69, 0x68, 0x75, 0x61, 0x68, 0x75, 0x61, 0x4b, 0x6f, 0x73,
0x74, 0x61, 0x20, 0x52, 0x69, 0x6b, 0x61, 0x4b, 0x75, 0x79, 0x61, 0x62,
0x61, 0x44, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x6b, 0x15f, 0x61, 0x76, 0x6e,
0x44, 0x6f, 0x75, 0x73, 0x6f, 0x6e, 0x20, 0x4b, 0x72, 0x69, 0x6b, 0x44,
0x65, 0x74, 0x72, 0x6f, 0x79, 0x74, 0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x69,
0x6b, 0x61, 0x45, 0x64, 0x6d, 0x6f, 0x6e, 0x64, 0x74, 0x6f, 0x6e, 0x130,
0x72, 0x75, 0x6e, 0x65, 0x70, 0x65, 0x51, 0x6c, 0x65, 0x79, 0x73, 0x20,
0x4b, 0xf6, 0x72, 0x66, 0x259, 0x7a, 0x69, 0x51, 0x75, 0x7a, 0x20, 0x4b,
0xf6, 0x72, 0x66, 0x259, 0x7a, 0x69, 0x51, 0x72, 0x61, 0x6e, 0x64, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x51, 0x72, 0x65, 0x6e, 0x61, 0x64, 0x61, 0x51,
0x76, 0x61, 0x64, 0x65, 0x6c, 0x75, 0x70, 0x61, 0x51, 0x76, 0x61, 0x74,
0x65, 0x6d, 0x61, 0x6c, 0x61, 0x51, 0x75, 0x61, 0x79, 0x61, 0x6b, 0x69,
0x6c, 0x51, 0x61, 0x79, 0x61, 0x6e, 0x61, 0x48, 0x65, 0x72, 0x6d, 0x6f,
0x73, 0x69, 0x6c, 0x6f, 0x130, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x70,
0x6f, 0x6c, 0x69, 0x73, 0x4e, 0x6f, 0x6b, 0x73, 0x4d, 0x61, 0x72, 0x65,
0x6e, 0x71, 0x6f, 0x50, 0x69, 0x74, 0x65, 0x72, 0x73, 0x62, 0x75, 0x72,
0x71, 0x56, 0x69, 0x76, 0x65, 0x79, 0x56, 0x69, 0x6e, 0x73, 0x65, 0x6e,
0x56, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x6b, 0x130, 0x6e, 0x75, 0x76, 0x69,
0x6b, 0x130, 0x71, 0x61, 0x6c, 0x75, 0x69, 0x74, 0x59, 0x61, 0x6d, 0x61,
0x79, 0x6b, 0x61, 0x43, 0x75, 0x6e, 0x6f, 0x4c, 0x75, 0x69, 0x73, 0x76,
0x69, 0x6c, 0x4d, 0x6f, 0x6e, 0x74, 0x69, 0xe7, 0x65, 0x6c, 0x6c, 0x6f,
0x4b, 0x72, 0x61, 0x6c, 0x65, 0x6e, 0x64, 0x69, 0x6b, 0x4c, 0x61, 0x20,
0x50, 0x61, 0x73, 0x4c, 0x6f, 0x73, 0x20, 0x41, 0x6e, 0x63, 0x65, 0x6c,
0x65, 0x73, 0x41, 0x15f, 0x61, 0x11f, 0x131, 0x20, 0x50, 0x72, 0x69, 0x6e,
0x73, 0x20, 0x4b, 0x76, 0x61, 0x72, 0x74, 0x61, 0x6c, 0x131, 0x4d, 0x61,
0x73, 0x65, 0x79, 0x6f, 0x4d, 0x61, 0x6e, 0x61, 0x71, 0x75, 0x61, 0x4d,
0x61, 0x72, 0x69, 0x71, 0x6f, 0x74, 0x4d, 0x61, 0x7a, 0x61, 0x6c, 0x74,
0x61, 0x6e, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x6f, 0x4d, 0x6f, 0x6e, 0x74,
0x65, 0x72, 0x65, 0x79, 0x4d, 0x6f, 0x6e, 0x73, 0x65, 0x72, 0x61, 0x74,
0x4e, 0x79, 0x75, 0x20, 0x59, 0x6f, 0x72, 0x6b, 0x4e, 0x6f, 0x6d, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x79, 0x61, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68,
0x2c, 0x20, 0x15e, 0x69, 0x6d, 0x61, 0x6c, 0x69, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x4d, 0x259, 0x72, 0x6b, 0x259, 0x7a, 0x2c, 0x20, 0x15e,
0x69, 0x6d, 0x61, 0x6c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e,
0x79, 0x75, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x4f, 0x63, 0x69, 0x6e,
0x61, 0x71, 0x61, 0x130, 0x73, 0x70, 0x61, 0x6e, 0x20, 0x4c, 0x69, 0x6d,
0x61, 0x6e, 0x131, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x20, 0x56, 0x65, 0x6c,
0x79, 0x6f, 0x50, 0x75, 0x65, 0x72, 0x74, 0x6f, 0x20, 0x52, 0x69, 0x6b,
0x6f, 0x52, 0x61, 0x6e, 0x6b, 0x69, 0x6e, 0x20, 0x47, 0x69, 0x72, 0x69,
0x15f, 0x69, 0x52, 0x65, 0x73, 0x69, 0x66, 0x52, 0x65, 0x63, 0x69, 0x6e,
0x61, 0x52, 0x65, 0x7a, 0x6f, 0x6c, 0x79, 0x75, 0x74, 0x53, 0x61, 0x6e,
0x74, 0x79, 0x61, 0x71, 0x6f, 0x53, 0x61, 0x6e, 0x74, 0x6f, 0x20, 0x44,
0x6f, 0x6d, 0x69, 0x6e, 0x71, 0x6f, 0x53, 0x61, 0x6e, 0x20, 0x50, 0x61,
0x75, 0x6c, 0x6f, 0x53, 0x6b, 0x6f, 0x72, 0x65, 0x73, 0x62, 0x69, 0x73,
0x75, 0x6e, 0x64, 0x53, 0x65, 0x6e, 0x74, 0x2d, 0x42, 0x61, 0x72, 0x74,
0x65, 0x6c, 0x65, 0x6d, 0x69, 0x53, 0x65, 0x6e, 0x74, 0x20, 0x43, 0x6f,
0x6e, 0x73, 0x53, 0x61, 0x6e, 0x20, 0x4b, 0x69, 0x74, 0x73, 0x53, 0x61,
0x6e, 0x20, 0x4c, 0xfc, 0x73, 0x69, 0x79, 0x61, 0x53, 0x61, 0x6e, 0x20,
0x54, 0x6f, 0x6d, 0x61, 0x73, 0x53, 0x61, 0x6e, 0x20, 0x56, 0x69, 0x6e,
0x73, 0x65, 0x6e, 0x74, 0x53, 0x76, 0x69, 0x66, 0x74, 0x20, 0x4b, 0x75,
0x72, 0x65, 0x6e, 0x74, 0x54, 0x65, 0x71, 0x75, 0x73, 0x69, 0x71, 0x61,
0x6c, 0x70, 0x61, 0x54, 0x75, 0x6c, 0x54, 0x69, 0x78, 0x75, 0x61, 0x6e,
0x61, 0x55, 0x61, 0x79, 0x74, 0x68, 0x6f, 0x72, 0x73, 0x56, 0x69, 0x6e,
0x6e, 0x69, 0x70, 0x65, 0x71, 0x4b, 0x65, 0x79, 0x73, 0x69, 0x44, 0x65,
0x79, 0x76, 0x69, 0x73, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x20, 0x64,
0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x4d, 0x61, 0x6b, 0x75, 0x61, 0x72,
0x69, 0x4d, 0x6f, 0x75, 0x73, 0x6f, 0x6e, 0x4d, 0x61, 0x6b, 0x20, 0x4d,
0x75, 0x72, 0x64, 0x6f, 0x53, 0x79, 0x6f, 0x76, 0x61, 0x4c, 0x6f, 0x6e,
0x71, 0x79, 0x69, 0x72, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x131, 0x41, 0x6e,
0x61, 0x64, 0x131, 0x72, 0x41, 0x15f, 0x71, 0x61, 0x62, 0x61, 0x74, 0x41,
0x74, 0x131, 0x72, 0x61, 0x75, 0x42, 0x61, 0x11f, 0x64, 0x61, 0x64, 0x42,
0x259, 0x68, 0x72, 0x65, 0x79, 0x6e, 0x42, 0x61, 0x6b, 0x131, 0x42, 0x61,
0x6e, 0x71, 0x6b, 0x6f, 0x6b, 0x42, 0x65, 0x79, 0x72, 0x75, 0x74, 0x42,
0x69, 0x15f, 0x6b, 0x65, 0x6b, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x79, 0x44,
0x259, 0x6d, 0x259, 0x15f, 0x71, 0x44, 0x259, 0x6b, 0x6b, 0x259, 0x44, 0x75,
0x62, 0x61, 0x79, 0x44, 0xfc, 0x15f, 0x259, 0x6e, 0x62, 0x259, 0x46, 0x61,
0x6d, 0x61, 0x71, 0x75, 0x73, 0x74, 0x61, 0x51, 0x259, 0x7a, 0x61, 0x48,
0x6f, 0x20, 0x15e, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x48, 0x6f, 0x6e, 0x71,
0x20, 0x4b, 0x6f, 0x6e, 0x71, 0x130, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x43, 0x61, 0x6b, 0x61, 0x72, 0x74, 0x61, 0x43, 0x61, 0x79, 0x61, 0x70,
0x75, 0x72, 0x61, 0x59, 0x65, 0x72, 0x75, 0x73, 0x259, 0x6c, 0x69, 0x6d,
0x4b, 0x61, 0x62, 0x69, 0x6c, 0x4b, 0x61, 0x6d, 0xe7, 0x61, 0x74, 0x6b,
0x61, 0x58, 0x61, 0x6e, 0x64, 0x131, 0x71, 0x61, 0x4b, 0x259, 0x6c, 0x6b,
0x259, 0x74, 0x259, 0x4b, 0x75, 0xe7, 0x69, 0x6e, 0x71, 0x4b, 0xfc, 0x76,
0x65, 0x79, 0x74, 0x4d, 0x61, 0x71, 0x61, 0x64, 0x61, 0x6e, 0x4e, 0x69,
0x6b, 0x6f, 0x73, 0x69, 0x79, 0x61, 0x50, 0x6e, 0x6f, 0x6d, 0x20, 0x50,
0x65, 0x6e, 0x50, 0x78, 0x65, 0x6e, 0x79, 0x61, 0x6e, 0x51, 0x131, 0x7a,
0x131, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x53, 0x61, 0x78, 0x61, 0x6c, 0x69,
0x6e, 0x53, 0x259, 0x6d, 0x259, 0x72, 0x71, 0x259, 0x6e, 0x64, 0x15e, 0x61,
0x6e, 0x78, 0x61, 0x79, 0x53, 0x69, 0x6e, 0x71, 0x61, 0x70, 0x75, 0x72,
0x53, 0x72, 0x65, 0x64, 0x6e, 0x65, 0x6b, 0x6f, 0x6c, 0x131, 0x6d, 0x73,
0x6b, 0x54, 0x61, 0x79, 0x62, 0x65, 0x79, 0x44, 0x61, 0x15f, 0x6b, 0x259,
0x6e, 0x64, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x55,
0x72, 0x75, 0x6d, 0xe7, 0x69, 0x56, 0x79, 0x65, 0x6e, 0x74, 0x79, 0x61,
0x6e, 0x52, 0x61, 0x6e, 0x71, 0x75, 0x6e, 0x59, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x71, 0x42, 0x65, 0x72, 0x6d,
0x75, 0x64, 0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x4b, 0x61,
0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x46, 0x61, 0x72, 0x65,
0x72, 0x4d, 0x61, 0x64, 0x65, 0x79, 0x72, 0x61, 0x52, 0x65, 0x79, 0x6b,
0x79, 0x61, 0x76, 0x69, 0x6b, 0x43, 0x259, 0x6e, 0x75, 0x62, 0x69, 0x20,
0x43, 0x6f, 0x72, 0x63, 0x69, 0x79, 0x61, 0x4d, 0xfc, 0x71, 0x259, 0x64,
0x64, 0x259, 0x73, 0x20, 0x59, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x42, 0x72,
0x69, 0x73, 0x62, 0x65, 0x79, 0x6e, 0x59, 0x75, 0x6b, 0x6c, 0x61, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x61, 0x75, 0x50, 0x65, 0x72, 0x74, 0x53,
0x69, 0x64, 0x6e, 0x65, 0x79, 0x48, 0x259, 0x15f, 0x74, 0x259, 0x72, 0x78,
0x61, 0x6e, 0x41, 0x66, 0x69, 0x6e, 0x61, 0x42, 0x65, 0x6c, 0x71, 0x72,
0x61, 0x64, 0x42, 0x72, 0xfc, 0x73, 0x73, 0x65, 0x6c, 0x42, 0x75, 0x78,
0x61, 0x72, 0x65, 0x73, 0x74, 0x42, 0x75, 0x64, 0x61, 0x70, 0x65, 0x15f,
0x74, 0x4b, 0x69, 0x15f, 0x69, 0x6e, 0x79, 0x6f, 0x76, 0x43, 0x259, 0x62,
0x259, 0x6c, 0x69, 0x2d, 0x54, 0x61, 0x72, 0x69, 0x71, 0x47, 0x65, 0x72,
0x6e, 0x7a, 0x65, 0x79, 0x4d, 0x65, 0x6e, 0x20, 0x41, 0x64, 0x61, 0x73,
0x131, 0x130, 0x73, 0x74, 0x61, 0x6e, 0x62, 0x75, 0x6c, 0x43, 0x65, 0x72,
0x73, 0x69, 0x4b, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x6e, 0x71, 0x72, 0x61,
0x64, 0x4b, 0x69, 0x79, 0x65, 0x76, 0x4c, 0x79, 0x75, 0x62, 0x6c, 0x79,
0x61, 0x6e, 0x61, 0x4c, 0xfc, 0x6b, 0x73, 0x65, 0x6d, 0x62, 0x75, 0x72,
0x71, 0x4d, 0x61, 0x72, 0x69, 0x68, 0x61, 0x6d, 0x4d, 0x6f, 0x73, 0x6b,
0x76, 0x61, 0x50, 0x6f, 0x64, 0x71, 0x6f, 0x72, 0x69, 0x74, 0x73, 0x61,
0x50, 0x72, 0x61, 0x71, 0x61, 0x52, 0x69, 0x71, 0x61, 0x53, 0x61, 0x72,
0x61, 0x79, 0x65, 0x76, 0x6f, 0x53, 0x6b, 0x6f, 0x70, 0x79, 0x65, 0x53,
0x6f, 0x66, 0x69, 0x79, 0x61, 0x54, 0x61, 0x6c, 0x6c, 0x69, 0x6e, 0x56,
0x61, 0x64, 0x75, 0x74, 0x73, 0x56, 0x79, 0x61, 0x6e, 0x61, 0x56, 0x69,
0x6c, 0x6e, 0xfc, 0x73, 0x56, 0x6f, 0x6c, 0x71, 0x6f, 0x71, 0x72, 0x61,
0x64, 0x56, 0x61, 0x72, 0x15f, 0x61, 0x76, 0x61, 0x5a, 0x61, 0x71, 0x72,
0x65, 0x62, 0x53, 0xfc, 0x72, 0x69, 0x78, 0xc7, 0x61, 0x71, 0x6f, 0x73,
0x4d, 0x69, 0x6c, 0x61, 0x64, 0x4b, 0x6f, 0x6d, 0x6f, 0x72, 0x6f, 0x4b,
0x65, 0x72, 0x67, 0x65, 0x6c, 0x65, 0x6e, 0x4d, 0x61, 0x76, 0x72, 0x69,
0x6b, 0x69, 0x4f, 0x6b, 0x6c, 0x259, 0x6e, 0x64, 0x42, 0x75, 0x71, 0x61,
0x6e, 0x76, 0x69, 0x6c, 0xc7, 0x61, 0x74, 0x61, 0x6d, 0xc7, 0x75, 0x75,
0x6b, 0x50, 0x61, 0x73, 0x78, 0x61, 0x46, 0x69, 0x63, 0x69, 0x51, 0x61,
0x6c, 0x61, 0x70, 0x61, 0x71, 0x6f, 0x73, 0x51, 0x61, 0x6d, 0x62, 0x69,
0x79, 0x65, 0x72, 0x51, 0x75, 0x61, 0x64, 0x61, 0x6c, 0x6b, 0x61, 0x6e,
0x61, 0x6c, 0x51, 0x75, 0x61, 0x6d, 0x45, 0x6e, 0x64, 0x65, 0x72, 0x62,
0xf6, 0x72, 0x69, 0x4b, 0x69, 0x72, 0x69, 0x6d, 0x61, 0x74, 0x69, 0x4b,
0x6f, 0x73, 0x72, 0x61, 0x79, 0x65, 0x4b, 0x76, 0x61, 0x6a, 0x61, 0x6c,
0x65, 0x79, 0x6e, 0x4d, 0x61, 0x63, 0x75, 0x72, 0x6f, 0x4d, 0x61, 0x72,
0x6b, 0x65, 0x73, 0x61, 0x73, 0x4d, 0x69, 0x64, 0x76, 0x65, 0x79, 0x50,
0x61, 0x71, 0x6f, 0x20, 0x50, 0x61, 0x71, 0x6f, 0x50, 0x6f, 0x72, 0x74,
0x20, 0x4d, 0x6f, 0x72, 0x65, 0x73, 0x62, 0x69, 0x52, 0x61, 0x72, 0x6f,
0x74, 0x6f, 0x6e, 0x71, 0x61, 0x53, 0x61, 0x79, 0x70, 0x61, 0x6e, 0x54,
0x61, 0x72, 0x61, 0x76, 0x61, 0x54, 0x6f, 0x6e, 0x71, 0x61, 0x70, 0x61,
0x74, 0x75, 0x56, 0x65, 0x79, 0x6b, 0x986, 0x9ac, 0x9bf, 0x9a6, 0x99c, 0x9be,
0x9a8, 0x986, 0x995, 0x9cd, 0x9b0, 0x9be, 0x986, 0x9a6, 0x9cd, 0x9a6, 0x9bf, 0x9b8,
0x20, 0x986, 0x9ac, 0x9be, 0x9ac, 0x9be, 0x986, 0x9b2, 0x99c, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x9b0, 0x9cd, 0x9b8, 0x985, 0x9cd, 0x9af, 0x9be, 0x9b8, 0x9ae, 0x9be, 0x9b0,
0x9be, 0x9b9, 0x9c1, 0x9ac, 0x9be, 0x9ae, 0x9cd, 0x9af, 0x9be, 0x995, 0x9cb, 0x9ac,
0x9be, 0x999, 0x9cd, 0x997, 0x9c1, 0x987, 0x9ac, 0x9be, 0x99e, 0x9cd, 0x99c, 0x9c1,
0x9b2, 0x9ac, 0x9bf, 0x9b8, 0x9cb, 0x989, 0x9ac, 0x9cd, 0x9b2, 0x9cd, 0x9af, 0x9be,
0x9a8, 0x99f, 0x9be, 0x9af, 0x9bc, 0x9c7, 0x9b0, 0x9ac, 0x9cd, 0x9b0, 0x9be, 0x99c,
0x9be, 0x9ad, 0x9bf, 0x9b2, 0x9bf, 0x9ac, 0x9c1, 0x99c, 0x9c1, 0x9ae, 0x9ac, 0x9c1,
0x9b0, 0x9b9, 0x9c1, 0x995, 0x9be, 0x9af, 0x9bc, 0x9b0, 0x9cb, 0x995, 0x9be, 0x9b8,
0x9be, 0x9ac, 0x9cd, 0x9b2, 0x9be, 0x999, 0x9cd, 0x995, 0x9be, 0x9b8, 0x9c7, 0x989,
0x99f, 0x9be, 0x995, 0x9a8, 0x9be, 0x995, 0x9cd, 0x9b0, 0x9bf, 0x9a1, 0x9be, 0x995,
0x9be, 0x9b0, 0x9a6, 0x9be, 0x9b0, 0x20, 0x98f, 0x9b8, 0x20, 0x9b8, 0x9be, 0x9b2,
0x9be, 0x9ae, 0x9a1, 0x9cb, 0x9af, 0x9bc, 0x9be, 0x9b2, 0x9be, 0x98f, 0x9b2, 0x20,
0x986, 0x9b9, 0x987, 0x989, 0x9a8, 0x9ab, 0x9cd, 0x9b0, 0x9c0, 0x99f, 0x9be, 0x989,
0x9a8, 0x997, 0x9cd, 0x9af, 0x9be, 0x9ac, 0x9c1, 0x9b0, 0x9a8, 0x9bf, 0x9b9, 0x9be,
0x9b0, 0x9be, 0x9b0, 0x9c7, 0x99c, 0x9cb, 0x9b9, 0x9be, 0x9a8, 0x9c7, 0x9b8, 0x9ac,
0x9be, 0x9b0, 0x9cd, 0x997, 0x995, 0x9be, 0x9ae, 0x9aa, 0x9be, 0x9b2, 0x9be, 0x996,
0x9be, 0x9b0, 0x9cd, 0x9a4, 0x9c1, 0x9ae, 0x995, 0x9bf, 0x997, 0x9be, 0x9b2, 0x9bf,
0x995, 0x9bf, 0x9a8, 0x9b6, 0x9be, 0x9b8, 0x9be, 0x9b2, 0x9be, 0x997, 0x9cb, 0x9b8,
0x9b2, 0x9bf, 0x9ac, 0x9cd, 0x9b0, 0x9c1, 0x9ad, 0x9bf, 0x9b2, 0x9b2, 0x9cb, 0x9ae,
0x9c7, 0x9b2, 0x9cb, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x9be, 0x9b2, 0x9c1,
0x9ac, 0x9c1, 0x9ae, 0x9cd, 0x9ac, 0x9be, 0x9b6, 0x9bf, 0x9b2, 0x9c1, 0x9b8, 0x9be,
0x995, 0x9be, 0x9ae, 0x9be, 0x9aa, 0x9c1, 0x9a4, 0x9cb, 0x9ae, 0x9be, 0x9b9, 0x9b8,
0x9c1, 0x9b0, 0x9c1, 0x985, 0x9ae, 0x9ac, 0x9be, 0x9ac, 0x9be, 0x9a8, 0x9ae, 0x9be,
0x993, 0x997, 0x9be, 0x9a6, 0x9bf, 0x9b8, 0x9c1, 0x9ae, 0x9a8, 0x9b0, 0x9cb, 0x9ad,
0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x987, 0x9b0, 0x9cb, 0x9ac, 0x9bf, 0x98f,
0x9a8, 0x99c, 0x9ae, 0x9c7, 0x9a8, 0x9be, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9ae,
0x9c7, 0x9a8, 0x9cb, 0x9af, 0x9bc, 0x9be, 0x995, 0x9b6, 0x99f, 0x993, 0x9af, 0x9bc,
0x9be, 0x9b9, 0x997, 0x9c1, 0x9a1, 0x9cb, 0x997, 0x9c1, 0x9aa, 0x9cb, 0x9b0, 0x9cd,
0x99f, 0x9cb, 0x2d, 0x9a8, 0x9cb, 0x9ad, 0x9cb, 0x9b8, 0x9be, 0x993, 0x20, 0x99f,
0x9cb, 0x9ae, 0x9a4, 0x9cd, 0x9b0, 0x9bf, 0x9aa, 0x9cb, 0x9b2, 0x9bf, 0x99f, 0x9bf,
0x989, 0x9a8, 0x9bf, 0x9b8, 0x989, 0x987, 0x9a8, 0x9a7, 0x9cb, 0x995, 0x986, 0x9a1,
0x995, 0x98f, 0x9a8, 0x995, 0x9cb, 0x9b0, 0x9c7, 0x99c, 0x985, 0x9cd, 0x9af, 0x9be,
0x999, 0x9cd, 0x997, 0x9c1, 0x987, 0x9b2, 0x9be, 0x985, 0x9cd, 0x9af, 0x9be, 0x9a8,
0x9cd, 0x99f, 0x9bf, 0x997, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x986, 0x9b0, 0x9be, 0x997,
0x9c1, 0x9af, 0x9bc, 0x9be, 0x987, 0x9a8, 0x9be, 0x9ac, 0x9c1, 0x9af, 0x9bc, 0x9c7,
0x9a8, 0x9cb, 0x9b8, 0x20, 0x986, 0x9af, 0x9bc, 0x9c7, 0x9b0, 0x9c7, 0x9b8, 0x995,
0x9cd, 0x9af, 0x9be, 0x99f, 0x9be, 0x9ae, 0x9be, 0x9b0, 0x9cd, 0x995, 0x9be, 0x995,
0x9b0, 0x9cd, 0x9a1, 0x9cb, 0x9ac, 0x9be, 0x9b2, 0x9be, 0x20, 0x9b0, 0x9bf, 0x993,
0x9b9, 0x9be, 0x9ae, 0x9c7, 0x9a8, 0x9cd, 0x9a1, 0x9cb, 0x99c, 0x9be, 0x9b0, 0x9bf,
0x993, 0x20, 0x997, 0x9be, 0x9af, 0x9bc, 0x9c7, 0x997, 0x9cb, 0x9b8, 0x9b8, 0x9cd,
0x9af, 0x9be, 0x9b2, 0x99f, 0x9be, 0x9b8, 0x9be, 0x9a8, 0x20, 0x99c, 0x9c1, 0x9af,
0x9bc, 0x9be, 0x9a8, 0x9b8, 0x9be, 0x9a8, 0x20, 0x9b2, 0x9c1, 0x987, 0x9b8, 0x99f,
0x9c1, 0x995, 0x9c1, 0x9ae, 0x9be, 0x9a8, 0x989, 0x9b6, 0x9c1, 0x9af, 0x9bc, 0x9be,
0x987, 0x9af, 0x9bc, 0x9be, 0x98f, 0x9b0, 0x9c1, 0x9ac, 0x9be, 0x986, 0x9b8, 0x9c1,
0x9a8, 0x9b8, 0x9bf, 0x9af, 0x9bc, 0x9a8, 0x9ac, 0x9be, 0x9b9, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x20, 0x9ac, 0x9be, 0x9a8, 0x9cd, 0x9a6, 0x9c7, 0x9b0, 0x9be, 0x9b8, 0x9ac,
0x9be, 0x9b0, 0x9cd, 0x9ac, 0x9be, 0x9a1, 0x9cb, 0x99c, 0x9ac, 0x9cd, 0x9b2, 0x9be,
0x999, 0x9cd, 0x995, 0x2d, 0x9b8, 0x9be, 0x9ac, 0x9cd, 0x9b2, 0x9cb, 0x9a8, 0x9ac,
0x9cb, 0x9af, 0x9bc, 0x9be, 0x20, 0x9ad, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9ac,
0x9af, 0x9bc, 0x99c, 0x9bf, 0x995, 0x9c7, 0x9ae, 0x9ac, 0x9cd, 0x9b0, 0x9bf, 0x99c,
0x20, 0x9ac, 0x9c7, 0x995, 0x9be, 0x9ae, 0x9cd, 0x9aa, 0x9cb, 0x20, 0x997, 0x9cd,
0x9b0, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x9c7, 0x995, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x995,
0x9c1, 0x9a8, 0x995, 0x9cd, 0x9af, 0x9be, 0x9b0, 0x9be, 0x995, 0x9be, 0x9b8, 0x995,
0x9be, 0x9b9, 0x9c7, 0x9a8, 0x995, 0x9be, 0x9ae, 0x9c7, 0x9a8, 0x9b6, 0x9bf, 0x995,
0x9be, 0x997, 0x9cb, 0x99a, 0x9bf, 0x9b9, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x9b9, 0x9c1,
0x9af, 0x9bc, 0x9be, 0x9b8, 0x9bf, 0x989, 0x9a6, 0x9be, 0x9a6, 0x20, 0x99c, 0x9c1,
0x9af, 0x9bc, 0x9be, 0x9b0, 0x9c7, 0x99c, 0x995, 0x9cb, 0x9b8, 0x9cd, 0x99f, 0x9be,
0x9b0, 0x9bf, 0x995, 0x9be, 0x995, 0x9cd, 0x9b0, 0x9bf, 0x9b8, 0x9cd, 0x99f, 0x9be,
0x9a8, 0x995, 0x9c1, 0x987, 0x9af, 0x9bc, 0x9be, 0x9ac, 0x9be, 0x995, 0x9c1, 0x9b0,
0x9be, 0x9b8, 0x9be, 0x993, 0x9a1, 0x9be, 0x9a8, 0x9ae, 0x9be, 0x9b0, 0x9cd, 0x995,
0x9b6, 0x9cd, 0x9af, 0x9be, 0x9ad, 0x9a8, 0x9a1, 0x9b8, 0x9cb, 0x9a8, 0x9a1, 0x9b8,
0x9cb, 0x9a8, 0x20, 0x995, 0x9cd, 0x9b0, 0x9bf, 0x995, 0x9a1, 0x9c7, 0x9a8, 0x9ad,
0x9be, 0x9b0, 0x9a1, 0x9c7, 0x99f, 0x9cd, 0x9b0, 0x9cb, 0x987, 0x99f, 0x9a1, 0x9cb,
0x9ae, 0x9bf, 0x9a8, 0x9bf, 0x995, 0x9be, 0x98f, 0x9a1, 0x9ae, 0x9a8, 0x9cd, 0x99f,
0x9cb, 0x9a8, 0x986, 0x987, 0x9b0, 0x9c1, 0x9a8, 0x9c7, 0x9aa, 0x9c7, 0x98f, 0x9b2,
0x20, 0x9b8, 0x9be, 0x9b2, 0x9ad, 0x9be, 0x9a6, 0x9cb, 0x9b0, 0x9ab, 0x9cb, 0x9b0,
0x9cd, 0x99f, 0x20, 0x9a8, 0x9c7, 0x9b2, 0x9b8, 0x9a8, 0x9ab, 0x9cb, 0x9b0, 0x9cd,
0x99f, 0x9be, 0x9b2, 0x9c7, 0x99c, 0x9be, 0x997, 0x9cd, 0x9b2, 0x9be, 0x9b8, 0x20,
0x9ac, 0x9c7, 0x997, 0x9c1, 0x9b8, 0x20, 0x9ac, 0x9c7, 0x997, 0x9cd, 0x9b0, 0x9cd,
0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x20, 0x9a4, 0x9c1, 0x9b0, 0x9cd, 0x995, 0x997,
0x9cd, 0x9b0, 0x9c7, 0x9a8, 0x9be, 0x9a1, 0x9be, 0x997, 0x9c1, 0x9af, 0x9bc, 0x9be,
0x9a6, 0x9c7, 0x9b2, 0x9cb, 0x9aa, 0x997, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x9a4, 0x9c7,
0x9ae, 0x9be, 0x9b2, 0x9be, 0x997, 0x9cb, 0x9af, 0x9bc, 0x9be, 0x987, 0x9af, 0x9bc,
0x9be, 0x995, 0x9bf, 0x9b2, 0x9b9, 0x9cd, 0x9af, 0x9be, 0x9b2, 0x9bf, 0x9ab, 0x9cd,
0x9af, 0x9be, 0x995, 0x9cd, 0x9b8, 0x9b9, 0x9be, 0x9b0, 0x9ae, 0x9cb, 0x9b8, 0x9bf,
0x9b2, 0x9cd, 0x9b2, 0x9cb, 0x987, 0x9a8, 0x9cd, 0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x9a8, 0x9be, 0x9aa, 0x9cb, 0x9b2, 0x9bf, 0x9b8, 0x9a8, 0x995, 0x9cd, 0x9b8, 0x2c,
0x20, 0x987, 0x9a8, 0x9cd, 0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x9ae,
0x9be, 0x9b0, 0x9c7, 0x999, 0x9cd, 0x997, 0x9cb, 0x2c, 0x20, 0x987, 0x9a8, 0x9cd,
0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x9aa, 0x9bf, 0x99f, 0x9be, 0x9b0,
0x9b0, 0x9cd, 0x9b8, 0x9ac, 0x9be, 0x9b0, 0x9cd, 0x997, 0x2c, 0x20, 0x987, 0x9a8,
0x9cd, 0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x99f, 0x9c7, 0x9b2, 0x9b8,
0x9bf, 0x99f, 0x9bf, 0x2c, 0x20, 0x987, 0x9a8, 0x9cd, 0x9a1, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x9a8, 0x9be, 0x9ad, 0x9c7, 0x9ad, 0x9c7, 0x9af, 0x9bc, 0x2c, 0x20, 0x987,
0x9a8, 0x9cd, 0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x9ad, 0x9bf, 0x9a8,
0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x9a8, 0x9c7, 0x9b8, 0x2c, 0x20, 0x987, 0x9a8, 0x9cd,
0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x989, 0x987, 0x9a8, 0x9be, 0x9ae,
0x9cd, 0x9af, 0x9be, 0x995, 0x2c, 0x20, 0x987, 0x9a8, 0x9cd, 0x9a1, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x9a8, 0x9be, 0x987, 0x9a8, 0x9c1, 0x9ad, 0x9cd, 0x9af, 0x9be, 0x995,
0x987, 0x995, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x9b2, 0x9c1, 0x987, 0x99f, 0x99c, 0x9c1,
0x9a8, 0x9cb, 0x9b2, 0x9c1, 0x987, 0x9b8, 0x9ad, 0x9bf, 0x9b2, 0x9ae, 0x9a8, 0x9cd,
0x99f, 0x9bf, 0x99a, 0x9c7, 0x9b2, 0x9cb, 0x2c, 0x20, 0x995, 0x9c7, 0x9a8, 0x9cd,
0x99f, 0x9be, 0x995, 0x9bf, 0x995, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x9c7, 0x9a8, 0x9cd,
0x9a1, 0x9be, 0x99c, 0x9bf, 0x995, 0x9b2, 0x9b8, 0x20, 0x985, 0x9cd, 0x9af, 0x9be,
0x99e, 0x9cd, 0x99c, 0x9c7, 0x9b2, 0x9c7, 0x9b8, 0x9b2, 0x9cb, 0x9af, 0x9bc, 0x9be,
0x9b0, 0x20, 0x9aa, 0x9cd, 0x9b0, 0x9bf, 0x9a8, 0x9cd, 0x9b8, 0x9c7, 0x9b8, 0x20,
0x995, 0x9cb, 0x9af, 0x9bc, 0x9be, 0x9b0, 0x9cd, 0x99f, 0x9be, 0x9b0, 0x9ae, 0x9be,
0x9b8, 0x9c7, 0x9af, 0x9bc, 0x9cb, 0x9ae, 0x9be, 0x9a8, 0x9be, 0x997, 0x9c1, 0x9af,
0x9bc, 0x9be, 0x9ae, 0x9be, 0x9a8, 0x9be, 0x989, 0x9b8, 0x9ae, 0x9be, 0x9b0, 0x9bf,
0x997, 0x9cb, 0x9ae, 0x9be, 0x9b0, 0x99f, 0x9bf, 0x9a8, 0x9bf, 0x995, 0x9ae, 0x9be,
0x9a4, 0x9be, 0x9ae, 0x9cb, 0x9b0, 0x9b8, 0x9ae, 0x9be, 0x99c, 0x9be, 0x99f, 0x9b2,
0x9be, 0x9a8, 0x9ae, 0x9c7, 0x9a8, 0x9cb, 0x9ae, 0x9bf, 0x9a8, 0x9bf, 0x9ae, 0x9c7,
0x9b0, 0x9bf, 0x9a1, 0x9be, 0x9ae, 0x9c7, 0x99f, 0x9b2, 0x9be, 0x995, 0x9be, 0x99f,
0x9b2, 0x9be, 0x9ae, 0x9c7, 0x995, 0x9cd, 0x9b8, 0x9bf, 0x995, 0x9cb, 0x20, 0x9b8,
0x9bf, 0x99f, 0x9bf, 0x9ae, 0x9bf, 0x995, 0x9c1, 0x9b2, 0x9a8, 0x9ae, 0x999, 0x9cd,
0x995, 0x99f, 0x9cb, 0x9a8, 0x9ae, 0x9a8, 0x9cd, 0x99f, 0x9c7, 0x9b0, 0x9bf, 0x9ae,
0x9a8, 0x9cd, 0x99f, 0x9c7, 0x9ad, 0x9bf, 0x9a1, 0x9bf, 0x993, 0x9ae, 0x9a8, 0x9cd,
0x9a4, 0x9b8, 0x9c7, 0x9b0, 0x9be, 0x9a4, 0x9a8, 0x9be, 0x9b8, 0x9be, 0x989, 0x9a8,
0x9bf, 0x989, 0x987, 0x9af, 0x9bc, 0x9b0, 0x9cd, 0x995, 0x9a8, 0x9b0, 0x9cb, 0x9a8,
0x9cd, 0x200c, 0x9b9, 0x9be, 0x9ac, 0x9c7, 0x989, 0x9b2, 0x9be, 0x2c, 0x20, 0x989,
0x9a4, 0x9cd, 0x9a4, 0x9b0, 0x20, 0x9a1, 0x9be, 0x995, 0x9cb, 0x99f, 0x9be, 0x9ae,
0x9a7, 0x9cd, 0x9af, 0x2c, 0x20, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9b0, 0x20, 0x9a1,
0x9be, 0x995, 0x9cb, 0x99f, 0x9be, 0x9a8, 0x9bf, 0x989, 0x20, 0x9b8, 0x9be, 0x9b2,
0x9c7, 0x9ae, 0x2c, 0x20, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9b0, 0x20, 0x9a1, 0x9be,
0x995, 0x9cb, 0x99f, 0x9be, 0x993, 0x99c, 0x9bf, 0x9a8, 0x9be, 0x997, 0x9be, 0x9aa,
0x9cd, 0x9af, 0x9be, 0x9b0, 0x9be, 0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x9ac, 0x9cb, 0x9aa,
0x9cb, 0x9b0, 0x9cd, 0x99f, 0x2d, 0x985, 0x9b9, 0x2d, 0x9aa, 0x9cd, 0x9b0, 0x9bf,
0x9a8, 0x9cd, 0x9b8, 0x9aa, 0x9cb, 0x9b0, 0x9cd, 0x99f, 0x20, 0x985, 0x9ab, 0x20,
0x9b8, 0x9cd, 0x9aa, 0x9c7, 0x9a8, 0x9aa, 0x9cb, 0x9b0, 0x9cd, 0x9a4, 0x9cb, 0x20,
0x9ad, 0x9c7, 0x9b2, 0x9cd, 0x200c, 0x9b9, 0x9cb, 0x9aa, 0x9c1, 0x9af, 0x9bc, 0x9c7,
0x9b0, 0x9cd, 0x9a4, 0x9cb, 0x20, 0x9b0, 0x9bf, 0x995, 0x9cb, 0x9aa, 0x9c1, 0x9a8,
0x9cd, 0x99f, 0x9be, 0x20, 0x986, 0x9b0, 0x9c7, 0x9a8, 0x9be, 0x9b8, 0x9b0, 0x200d,
0x9cd, 0x9af, 0x9be, 0x999, 0x9cd, 0x995, 0x9bf, 0x9a8, 0x20, 0x987, 0x9a8, 0x9b2,
0x9c7, 0x99f, 0x9b0, 0x9c7, 0x9b8, 0x9bf, 0x9ab, 0x9c7, 0x9b0, 0x9c7, 0x99c, 0x9bf,
0x9a8, 0x9be, 0x9b0, 0x9c7, 0x9b8, 0x9cb, 0x9b2, 0x9c1, 0x99f, 0x9b0, 0x9bf, 0x993,
0x20, 0x9ac, 0x9cd, 0x9b0, 0x9be, 0x999, 0x9cd, 0x995, 0x9cb, 0x9b8, 0x9c7, 0x9a8,
0x99f, 0x9c1, 0x9b0, 0x9c7, 0x9ae, 0x9b8, 0x9be, 0x9a8, 0x9cd, 0x99f, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x997, 0x9cb, 0x9b8, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x99f, 0x9cb,
0x20, 0x9a1, 0x9cb, 0x9ae, 0x9bf, 0x982, 0x997, 0x9cb, 0x9b8, 0x9be, 0x993, 0x20,
0x9aa, 0x9be, 0x989, 0x9b2, 0x9cb, 0x987, 0x99f, 0x9cd, 0x99f, 0x995, 0x9c1, 0x9af,
0x9bc, 0x9cb, 0x9b0, 0x99f, 0x9c1, 0x9b0, 0x9ae, 0x9bf, 0x99f, 0x9b6, 0x9bf, 0x99f,
0x995, 0x9be, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x2e, 0x9ac, 0x9be, 0x9b0, 0x9cd,
0x9a5, 0x9c7, 0x9b2, 0x9c7, 0x9ae, 0x9bf, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x20,
0x99c, 0x9a8, 0x9cd, 0x9b8, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x2e, 0x20, 0x995,
0x9bf, 0x99f, 0x9b8, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x2e, 0x20, 0x9b2, 0x9c1,
0x9b8, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x2e, 0x20,
0x9a5, 0x9ae, 0x9be, 0x9b8, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x2e, 0x20, 0x9ad,
0x9bf, 0x9a8, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x9b8, 0x9c1, 0x987, 0x9ab, 0x9cd,
0x99f, 0x20, 0x995, 0x9be, 0x9b0, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x9a4, 0x9c7, 0x997,
0x9c1, 0x9b8, 0x9bf, 0x997, 0x9be, 0x9b2, 0x9aa, 0x9be, 0x9a5, 0x9c1, 0x9b2, 0x9bf,
0x9a4, 0x9bf, 0x99c, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x99f, 0x9cb, 0x9b0,
0x9a8, 0x9cd, 0x99f, 0x9cb, 0x99f, 0x9b0, 0x99f, 0x9cb, 0x9b2, 0x9be, 0x9ad, 0x9cd,
0x9af, 0x9be, 0x999, 0x9cd, 0x995, 0x9c1, 0x9ad, 0x9be, 0x9b0, 0x9b9, 0x9cb, 0x9af,
0x9bc, 0x9be, 0x987, 0x99f, 0x9b9, 0x9b0, 0x9cd, 0x9b8, 0x989, 0x987, 0x9a8, 0x9bf,
0x9aa, 0x9c7, 0x997, 0x987, 0x9af, 0x9bc, 0x9be, 0x995, 0x9c1, 0x99f, 0x9be, 0x99f,
0x995, 0x9c7, 0x987, 0x9b8, 0x9bf, 0x9a1, 0x9c7, 0x9ad, 0x9bf, 0x9b8, 0x9a1, 0x9c1,
0x9ae, 0x9a8, 0x9cd, 0x99f, 0x20, 0x9a1, 0x9bf, 0x2019, 0x989, 0x9b0, 0x9ad, 0x9bf,
0x9b2, 0x9ae, 0x9cd, 0x9af, 0x9be, 0x995, 0x995, 0x9c1, 0x9af, 0x9bc, 0x9cd, 0x9af,
0x9be, 0x9b0, 0x9bf, 0x9ae, 0x9b8, 0x9cb, 0x9a8, 0x9ae, 0x9cd, 0x9af, 0x9be, 0x995,
0x9ae, 0x9c1, 0x9b0, 0x9cd, 0x9a1, 0x9cb, 0x9aa, 0x9be, 0x9b2, 0x9ae, 0x9be, 0x9b0,
0x9b0, 0x9a5, 0x9c7, 0x9b0, 0x9be, 0x9b8, 0x9bf, 0x993, 0x9af, 0x9bc, 0x9be, 0x99f,
0x9cd, 0x9b0, 0x9b2, 0x9ad, 0x9b8, 0x9cd, 0x99f, 0x9cb, 0x995, 0x9b2, 0x99e, 0x9cd,
0x99c, 0x9bf, 0x9af, 0x9bc, 0x9ac, 0x9bf, 0x9af, 0x9bc, 0x9c7, 0x981, 0x9a8, 0x986,
0x9b9, 0x9a6, 0x9c7, 0x9a8, 0x985, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9be, 0x9a1, 0x9bf,
0x9b0, 0x986, 0x995, 0x99f, 0x9be, 0x989, 0x986, 0x995, 0x99f, 0x9cb, 0x9ac, 0x9c7,
0x986, 0x9b6, 0x997, 0x9be, 0x9ac, 0x9be, 0x9a4, 0x985, 0x9a4, 0x9bf, 0x9b0, 0x9be,
0x989, 0x9ac, 0x9be, 0x9b9, 0x9be, 0x9b0, 0x9bf, 0x9a8, 0x9ac, 0x9cd, 0x9af, 0x9be,
0x982, 0x995, 0x995, 0x9ac, 0x9be, 0x9b0, 0x9cd, 0x9a8, 0x9cc, 0x9b2, 0x9ac, 0x9c7,
0x987, 0x9b0, 0x9c1, 0x99f, 0x9ac, 0x9bf, 0x9b6, 0x995, 0x9c7, 0x995, 0x9ac, 0x9cd,
0x9b0, 0x9c1, 0x9a8, 0x9c7, 0x987, 0x99a, 0x9bf, 0x9a4, 0x9be, 0x9a6, 0x9be, 0x9ae,
0x9be, 0x9b8, 0x9cd, 0x995, 0x9be, 0x9b8, 0x9a6, 0x9bf, 0x9b2, 0x9bf, 0x9a6, 0x9c1,
0x9ac, 0x9be, 0x987, 0x9a6, 0x9c1, 0x9b6, 0x9be, 0x9a8, 0x9ac, 0x9c7, 0x9ab, 0x9be,
0x9ae, 0x9be, 0x997, 0x9be, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9b9, 0x9c7, 0x9ac, 0x9cd,
0x9b0, 0x9cb, 0x9a8, 0x9b9, 0x9cb, 0x20, 0x99a, 0x9bf, 0x20, 0x9ae, 0x9bf, 0x9a8,
0x20, 0x9b6, 0x9b9, 0x9b0, 0x9b9, 0x9cb, 0x9ad, 0x9cd, 0x9a1, 0x987, 0x9b0, 0x995,
0x9c1, 0x99f, 0x9b8, 0x9cd, 0x995, 0x99c, 0x9be, 0x995, 0x9be, 0x9b0, 0x9cd, 0x9a4,
0x9be, 0x99c, 0x9af, 0x9bc, 0x9be, 0x9aa, 0x9c1, 0x9b0, 0x9be, 0x99c, 0x9c7, 0x9b0,
0x9c1, 0x99c, 0x9be, 0x9b2, 0x9c7, 0x9ae, 0x995, 0x9be, 0x9ae, 0x99a, 0x9be, 0x99f,
0x995, 0x9be, 0x995, 0x9b0, 0x9be, 0x99a, 0x9bf, 0x995, 0x9be, 0x9a0, 0x9ae, 0x9be,
0x9a8, 0x9cd, 0x9a1, 0x9c1, 0x996, 0x9be, 0x9a8, 0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x997, 0x9be, 0x995, 0x9cb, 0x9b2, 0x995, 0x9be, 0x9a4, 0x9be, 0x995, 0x9cd, 0x9b0,
0x9be, 0x9b8, 0x9a8, 0x9cb, 0x987, 0x9af, 0x9bc, 0x9be, 0x9b0, 0x9cd, 0x9b8, 0x9cd,
0x995, 0x995, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x9b2, 0x9be, 0x9b2, 0x9be, 0x9ae, 0x9aa,
0x9c1, 0x9b0, 0x995, 0x9c1, 0x9af, 0x9bc, 0x9c7, 0x9a4, 0x9ae, 0x9cd, 0x9af, 0x9be,
0x995, 0x9be, 0x993, 0x9ae, 0x9cd, 0x9af, 0x9be, 0x997, 0x9be, 0x9a1, 0x9be, 0x9a8,
0x9ae, 0x9be, 0x995, 0x9be, 0x9b8, 0x9b8, 0x9be, 0x9b0, 0x9ae, 0x9cd, 0x9af, 0x9be,
0x9a8, 0x9bf, 0x9b2, 0x9be, 0x9ae, 0x9be, 0x9b8, 0x995, 0x99f, 0x9a8, 0x9bf, 0x995,
0x9cb, 0x9b8, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9ad, 0x995, 0x9c1, 0x9af, 0x9bc,
0x9c7, 0x9a4, 0x9b8, 0x9cd, 0x9a8, 0x995, 0x9a8, 0x9ad, 0x9cb, 0x9b8, 0x9bf, 0x9ac,
0x9bf, 0x9b0, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x993, 0x9ae, 0x9cd, 0x9b8, 0x9cd, 0x995,
0x993, 0x9b0, 0x9be, 0x9b2, 0x9a8, 0x9ae, 0x20, 0x9aa, 0x9c7, 0x9a8, 0x9aa, 0x9a8,
0x9cd, 0x99f, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9cd, 0x9af, 0x9be, 0x995, 0x9aa,
0x9bf, 0x9af, 0x9bc, 0x982, 0x987, 0x9af, 0x9bc, 0x982, 0x995, 0x9be, 0x9a4, 0x9be,
0x9b0, 0x995, 0x9cb, 0x9b8, 0x9cd, 0x99f, 0x9be, 0x9a8, 0x9af, 0x9bc, 0x995, 0x9bf,
0x99c, 0x9bf, 0x9b2, 0x9b0, 0x9cd, 0x9a1, 0x9be, 0x9b0, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x9a7, 0x9b8, 0x9be, 0x996, 0x9be, 0x9b2, 0x9bf, 0x9a8, 0x9b8, 0x9ae, 0x9b0, 0x996,
0x9a8, 0x9cd, 0x9a6, 0x9b8, 0x9bf, 0x993, 0x9b2, 0x9b8, 0x9be, 0x982, 0x9b9, 0x9be,
0x987, 0x9b8, 0x9bf, 0x999, 0x9cd, 0x997, 0x9be, 0x9aa, 0x9c1, 0x9b0, 0x9b8, 0x9cd,
0x9b0, 0x9c7, 0x9a6, 0x9a8, 0x9c7, 0x995, 0x9cb, 0x9b2, 0x9af, 0x9bc, 0x9ae, 0x9b8,
0x9cd, 0x995, 0x9a4, 0x9be, 0x987, 0x9aa, 0x9c7, 0x987, 0x9a4, 0x9be, 0x9b8, 0x996,
0x9a8, 0x9cd, 0x9a6, 0x9b8, 0x9bf, 0x9ac, 0x9bf, 0x9b2, 0x9bf, 0x9b6, 0x9bf, 0x9a4,
0x9c7, 0x9b9, 0x9c7, 0x9b0, 0x9be, 0x9a8, 0x99f, 0x9cb, 0x995, 0x9bf, 0x993, 0x9a4,
0x9cb, 0x9ae, 0x9b8, 0x9cd, 0x995, 0x989, 0x9b2, 0x9be, 0x9a8, 0x9ac, 0x9be, 0x9a4,
0x9be, 0x9b0, 0x989, 0x9b0, 0x9c1, 0x9ae, 0x995, 0x9bf, 0x989, 0x9b8, 0x9cd, 0x9a4,
0x2d, 0x20, 0x9a8, 0x9c7, 0x9b0, 0x9be, 0x9ad, 0x9bf, 0x9af, 0x9bc, 0x9c7, 0x9a8,
0x9a4, 0x9be, 0x9af, 0x9bc, 0x9c7, 0x9a8, 0x9ad, 0x9cd, 0x9b2, 0x9be, 0x9a6, 0x9bf,
0x9ad, 0x9b8, 0x9cd, 0x9a4, 0x9cb, 0x995, 0x987, 0x9af, 0x9bc, 0x9be, 0x995, 0x9c1,
0x99f, 0x9b8, 0x9cd, 0x995, 0x9b0, 0x9c7, 0x999, 0x9cd, 0x997, 0x9c1, 0x9a8, 0x987,
0x9af, 0x9bc, 0x9c7, 0x995, 0x9be, 0x99f, 0x9c7, 0x9b0, 0x9bf, 0x9a8, 0x9ac, 0x9be,
0x9b0, 0x9cd, 0x997, 0x987, 0x9af, 0x9bc, 0x9c7, 0x9b0, 0x9be, 0x9ad, 0x9be, 0x9a8,
0x986, 0x99c, 0x9cb, 0x9b0, 0x9c7, 0x9b8, 0x9ac, 0x9be, 0x9b0, 0x9ae, 0x9c1, 0x9a1,
0x9be, 0x995, 0x9be, 0x9a8, 0x9c7, 0x9b0, 0x9bf, 0x995, 0x9c7, 0x9aa, 0x20, 0x9ad,
0x9be, 0x9b0, 0x9cd, 0x9a6, 0x9ab, 0x9cd, 0x9af, 0x9be, 0x9b0, 0x9cb, 0x9ae, 0x9be,
0x9a1, 0x9c7, 0x987, 0x9b0, 0x9be, 0x9b0, 0x9bf, 0x995, 0x99c, 0x9be, 0x9ad, 0x9bf,
0x995, 0x9a6, 0x995, 0x9cd, 0x9b7, 0x9bf, 0x9a3, 0x20, 0x99c, 0x9b0, 0x9cd, 0x99c,
0x9bf, 0x9af, 0x9bc, 0x9be, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x2e, 0x20, 0x9b9,
0x9c7, 0x9b2, 0x9c7, 0x9a8, 0x9be, 0x9b8, 0x9cd, 0x99f, 0x9be, 0x9a8, 0x9b2, 0x9c0,
0x98f, 0x9cd, 0x9af, 0x9be, 0x9a1, 0x9c7, 0x9b2, 0x9c7, 0x987, 0x9a1, 0x9ac, 0x9cd,
0x9b0, 0x9bf, 0x9b8, 0x9ac, 0x9c7, 0x9a8, 0x9ac, 0x9cd, 0x9b0, 0x9cb, 0x995, 0x9c7,
0x9a8, 0x20, 0x9b9, 0x9bf, 0x9b2, 0x9a1, 0x9be, 0x9b0, 0x989, 0x987, 0x9a8, 0x9b9,
0x9cb, 0x9ac, 0x9be, 0x9b0, 0x9cd, 0x99f, 0x9b2, 0x9bf, 0x9a8, 0x9a1, 0x9c7, 0x9ae,
0x9cd, 0x9af, 0x9be, 0x9a8, 0x9b2, 0x9b0, 0x9cd, 0x9a1, 0x20, 0x9b9, 0x9be, 0x993,
0x9ae, 0x9c7, 0x9b2, 0x9ac, 0x9cb, 0x9b0, 0x9cd, 0x9a8, 0x9aa, 0x9be, 0x9b0, 0x9cd,
0x9a5, 0x9b8, 0x9bf, 0x9a1, 0x9a8, 0x9bf, 0x986, 0x9ae, 0x9b8, 0x9cd, 0x99f, 0x9be,
0x9b0, 0x9a1, 0x9be, 0x9ae, 0x985, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x9cb,
0x9b0, 0x9be, 0x986, 0x9b8, 0x99f, 0x9cd, 0x9b0, 0x9be, 0x996, 0x9be, 0x9a8, 0x9ac,
0x9c7, 0x9b2, 0x997, 0x9cd, 0x9b0, 0x9c7, 0x9a1, 0x9ac, 0x9be, 0x9b0, 0x9cd, 0x9b2,
0x9bf, 0x9a8, 0x9ac, 0x9cd, 0x9b0, 0x9be, 0x9a4, 0x9bf, 0x9b8, 0x9cd, 0x9b2, 0x9be,
0x9ad, 0x9be, 0x9ac, 0x9cd, 0x9b0, 0x9be, 0x9b8, 0x9c7, 0x9b2, 0x9b8, 0x9ac, 0x9c1,
0x996, 0x9be, 0x9b0, 0x9c7, 0x9b8, 0x9cd, 0x99f, 0x9ac, 0x9c1, 0x9a1, 0x9be, 0x9aa,
0x9c7, 0x9b8, 0x9cd, 0x99f, 0x9ac, 0x9c1, 0x9b8, 0x9bf, 0x9a8, 0x997, 0x9c7, 0x9a8,
0x995, 0x9bf, 0x9b8, 0x9bf, 0x9a8, 0x9be, 0x9b9, 0x9c1, 0x995, 0x9cb, 0x9aa, 0x9c7,
0x9a8, 0x9b9, 0x9c7, 0x997, 0x9c7, 0x9a8, 0x99c, 0x9bf, 0x9ac, 0x9cd, 0x9b0, 0x9be,
0x9b2, 0x9cd, 0x99f, 0x9be, 0x9b0, 0x997, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x9b0, 0x9cd,
0x9a8, 0x9b8, 0x9bf, 0x9b9, 0x9c7, 0x9b2, 0x9b8, 0x9bf, 0x999, 0x9cd, 0x995, 0x9bf,
0x986, 0x987, 0x9b2, 0x20, 0x985, 0x9ab, 0x20, 0x9ae, 0x9cd, 0x9af, 0x9be, 0x9a8,
0x99c, 0x9be, 0x9b0, 0x9cd, 0x9b8, 0x9bf, 0x995, 0x9be, 0x9b2, 0x9bf, 0x9a8, 0x9bf,
0x999, 0x997, 0x9cd, 0x9b0, 0x9be, 0x9a1, 0x995, 0x9bf, 0x9b0, 0x9cb, 0x9ad, 0x995,
0x9bf, 0x9af, 0x9bc, 0x9c7, 0x9ad, 0x9b2, 0x9bf, 0x9b8, 0x9ac, 0x9a8, 0x9b2, 0x9c1,
0x9ac, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x9b2, 0x9a8, 0x9cd, 0x9a1,
0x9a8, 0x9b2, 0x9c1, 0x995, 0x9cd, 0x9b8, 0x9c7, 0x9ae, 0x9ac, 0x9be, 0x9b0, 0x9cd,
0x997, 0x9ae, 0x9be, 0x9a6, 0x9cd, 0x9b0, 0x9bf, 0x9a6, 0x9ae, 0x9b0, 0x9bf, 0x9af,
0x9bc, 0x9c7, 0x9b9, 0x9be, 0x9ae, 0x9c7, 0x9a8, 0x9ae, 0x9bf, 0x9a8, 0x9cd, 0x9b8,
0x995, 0x985, 0x9b8, 0x9b2, 0x9cb, 0x9aa, 0x9cd, 0x9af, 0x9be, 0x9b0, 0x9bf, 0x9b8,
0x9aa, 0x9a1, 0x997, 0x9b0, 0x9bf, 0x9a4, 0x9cd, 0x9b8, 0x9be, 0x9aa, 0x9cd, 0x9b0,
0x9be, 0x997, 0x9b0, 0x9bf, 0x997, 0x9be, 0x9b8, 0x9be, 0x9ae, 0x9be, 0x9b0, 0x9be,
0x9b8, 0x9be, 0x9a8, 0x20, 0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x9a8, 0x9cb, 0x9b8, 0x9be,
0x9b0, 0x9be, 0x99c, 0x9bf, 0x9ad, 0x9cb, 0x9b8, 0x9be, 0x9b0, 0x9be, 0x99f, 0x9cb,
0x9ad, 0x9b8, 0x9bf, 0x9ae, 0x9ab, 0x9c7, 0x9b0, 0x9cb, 0x9aa, 0x9cb, 0x9b2, 0x9b8,
0x9cd, 0x995, 0x9aa, 0x9af, 0x9bc, 0x9c7, 0x9b8, 0x9cb, 0x9ab, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x9b8, 0x9cd, 0x99f, 0x995, 0x9b9, 0x9cb, 0x9ae, 0x9a4, 0x9be, 0x9b9, 0x9b2,
0x9bf, 0x9a8, 0x9a4, 0x9bf, 0x9b0, 0x9be, 0x9a8, 0x9be, 0x989, 0x9b2, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x9a8, 0x9cb, 0x9ad, 0x9b8, 0x9cd, 0x995, 0x9ad, 0x9be, 0x99f, 0x9bf,
0x995, 0x9be, 0x9a8, 0x9ad, 0x9bf, 0x9b2, 0x9a8, 0x9bf, 0x993, 0x9b8, 0x9ad, 0x9cb,
0x9b2, 0x9cd, 0x997, 0x9cb, 0x997, 0x9cd, 0x9b0, 0x9be, 0x9a6, 0x993, 0x9af, 0x9bc,
0x9be, 0x9b0, 0x9b6, 0x99c, 0x9be, 0x997, 0x9cd, 0x9b0, 0x9c7, 0x9ac, 0x99c, 0x9c1,
0x9b0, 0x9bf, 0x996, 0x986, 0x9a8, 0x9cd, 0x9a4, 0x9c1, 0x9a8, 0x9be, 0x9a8, 0x9be,
0x9b0, 0x9bf, 0x9ad, 0x9cb, 0x99b, 0x9be, 0x997, 0x9cb, 0x9b8, 0x995, 0x9cd, 0x9b0,
0x9bf, 0x9b8, 0x9ae, 0x9be, 0x9b8, 0x995, 0x9cb, 0x995, 0x9cb, 0x9b8, 0x995, 0x9cb,
0x9ae, 0x9cb, 0x9b0, 0x9cb, 0x995, 0x9be, 0x9b0, 0x9cd, 0x997, 0x9c1, 0x9b2, 0x9c7,
0x9a8, 0x9ae, 0x9b0, 0x9bf, 0x9b6, 0x9be, 0x9b8, 0x9ae, 0x9be, 0x9af, 0x9bc, 0x9cb,
0x9a4, 0x9cb, 0x9b0, 0x9bf, 0x987, 0x989, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9a8, 0x985,
0x995, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x9ac, 0x9c1, 0x997, 0x9c7,
0x9a8, 0x9ad, 0x9bf, 0x9b2, 0x9c7, 0x99a, 0x9cd, 0x9af, 0x9be, 0x9a0, 0x9be, 0x9ae,
0x987, 0x9b8, 0x9cd, 0x99f, 0x9be, 0x9b0, 0x987, 0x9ab, 0x9be, 0x9a4, 0x9c7, 0x9ab,
0x9cd, 0x9af, 0x9be, 0x995, 0x9be, 0x993, 0x9ab, 0x9cb, 0x997, 0x9cd, 0x9af, 0x9be,
0x9b2, 0x9be, 0x9aa, 0x9cd, 0x9af, 0x9be, 0x997, 0x9cb, 0x9b8, 0x997, 0x9be, 0x9ae,
0x9cd, 0x9ac, 0x9bf, 0x9af, 0x9bc, 0x9c7, 0x9b0, 0x997, 0x9cb, 0x9af, 0x9bc, 0x9be,
0x9a6, 0x9be, 0x9b2, 0x995, 0x9c1, 0x9a8, 0x9be, 0x9b2, 0x9b9, 0x9a8, 0x9cb, 0x9b2,
0x9c1, 0x9b2, 0x9c1, 0x98f, 0x9a8, 0x9cd, 0x9a1, 0x9be, 0x9b0, 0x9ac, 0x9be, 0x9b0,
0x9bf, 0x995, 0x9bf, 0x9b0, 0x9bf, 0x9a4, 0x9bf, 0x9ae, 0x9be, 0x9a4, 0x9bf, 0x995,
0x9cb, 0x9b8, 0x9b0, 0x9be, 0x9af, 0x9bc, 0x995, 0x9cb, 0x9af, 0x9bc, 0x9be, 0x99c,
0x9be, 0x9b2, 0x9c7, 0x987, 0x9a8, 0x9ae, 0x9be, 0x99c, 0x9c1, 0x9b0, 0x9cb, 0x9ae,
0x9be, 0x9b0, 0x9cd, 0x995, 0x9c7, 0x9b8, 0x9be, 0x9b8, 0x9ae, 0x9bf, 0x9a1, 0x993,
0x9af, 0x9bc, 0x9c7, 0x9a8, 0x9be, 0x989, 0x9b0, 0x9c1, 0x9a8, 0x9bf, 0x989, 0x9af,
0x9bc, 0x9bf, 0x9a8, 0x9b0, 0x9ab, 0x995, 0x9a8, 0x9c1, 0x9ae, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x9aa, 0x9cd, 0x9af, 0x9be, 0x997, 0x9cb, 0x20, 0x9aa, 0x9cd, 0x9af, 0x9be,
0x997, 0x9cb, 0x9aa, 0x9bf, 0x99f, 0x995, 0x9c7, 0x9af, 0x9bc, 0x9be, 0x9b0, 0x9cd,
0x9a8, 0x9aa, 0x9cb, 0x9a8, 0x9be, 0x9aa, 0x9c7, 0x9aa, 0x9cb, 0x9b0, 0x9cd, 0x99f,
0x20, 0x9ae, 0x9cc, 0x9b0, 0x99c, 0x9ac, 0x9bf, 0x9b0, 0x9be, 0x9b0, 0x989, 0x9b9,
0x9a4, 0x9c1, 0x999, 0x9cd, 0x997, 0x9be, 0x9b8, 0x9be, 0x987, 0x9aa, 0x9be, 0x9a8,
0x99f, 0x9be, 0x9b0, 0x9be, 0x993, 0x9af, 0x9bc, 0x9be, 0x99f, 0x9cb, 0x999, 0x9cd,
0x997, 0x9be, 0x99f, 0x9be, 0x9aa, 0x9c1, 0x993, 0x9af, 0x9bc, 0x9c7, 0x995, 0x993,
0x9b2, 0x9bf, 0x9b8, 0x41, 0x6c, 0x6a, 0x65, 0x72, 0x42, 0x61, 0x6e, 0x67,
0x69, 0x4b, 0x6f, 0x6e, 0x61, 0x6b, 0x72, 0x79, 0x44, 0x6a, 0x69, 0x62,
0x75, 0x74, 0x69, 0x41, 0x61, 0x69, 0x75, 0x6e, 0x4b, 0x68, 0x61, 0x72,
0x74, 0x75, 0x6d, 0x4d, 0x75, 0x71, 0x64, 0x69, 0x73, 0x68, 0x6f, 0x4e,
0x2019, 0x64, 0x6a, 0x61, 0x6d, 0x65, 0x6e, 0x61, 0x4e, 0x69, 0x61, 0x6d,
0x65, 0x69, 0x55, 0x61, 0x67, 0x61, 0x64, 0x75, 0x67, 0x75, 0x50, 0x6f,
0x72, 0x74, 0x6f, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x41, 0x69, 0x6e, 0x67,
0x69, 0x72, 0x61, 0x41, 0x72, 0x61, 0x67, 0x75, 0x61, 0xed, 0x6e, 0x61,
0x4b, 0x61, 0x69, 0x6d, 0x61, 0x6e, 0x43, 0x75, 0x69, 0x61, 0x62, 0xe1,
0x45, 0x69, 0x72, 0x75, 0x6e, 0x65, 0x70, 0xe9, 0x4d, 0x61, 0x63, 0x65,
0x69, 0xf3, 0x4d, 0x61, 0x72, 0x74, 0x69, 0x6e, 0x69, 0x6b, 0x61, 0x4d,
0x61, 0x7a, 0x61, 0x74, 0x6c, 0xe1, 0x6e, 0x4d, 0x65, 0x78, 0x69, 0x6b,
0x6f, 0x20, 0x48, 0x69, 0x72, 0x69, 0x61, 0x4d, 0x69, 0x6b, 0x65, 0x6c,
0x75, 0x6e, 0x65, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x49,
0x70, 0x61, 0x72, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x43, 0x65,
0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x49, 0x70, 0x61, 0x72, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c,
0x65, 0x6d, 0x2c, 0x20, 0x49, 0x70, 0x61, 0x72, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x50, 0x6f, 0x72, 0x74, 0x2d, 0x6f, 0x66, 0x2d, 0x53,
0x70, 0x61, 0x69, 0x6e, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x72, 0xe9, 0x6d,
0x53, 0xe3, 0x6f, 0x20, 0x50, 0x61, 0x75, 0x6c, 0x6f, 0x53, 0x61, 0x69,
0x6e, 0x74, 0x2d, 0x42, 0x61, 0x72, 0x74, 0x68, 0xe9, 0x6c, 0x65, 0x6d,
0x79, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x4a, 0x6f, 0x68, 0x6e, 0x2019,
0x73, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x4b, 0x69, 0x74, 0x74, 0x73,
0x53, 0x61, 0x6e, 0x74, 0x61, 0x20, 0x4c, 0x75, 0x7a, 0x69, 0x61, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x2d, 0x54, 0x68, 0x6f, 0x6d, 0x61, 0x73, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x20, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x74,
0x51, 0x61, 0x61, 0x6e, 0x61, 0x61, 0x71, 0x41, 0x73, 0x67, 0x61, 0x62,
0x61, 0x74, 0x42, 0x69, 0x78, 0x6b, 0x65, 0x6b, 0x54, 0x78, 0x69, 0x74,
0x61, 0x44, 0x61, 0x6d, 0x61, 0x73, 0x6b, 0x6f, 0x44, 0x75, 0x78, 0x61,
0x6e, 0x62, 0x65, 0x4b, 0x68, 0x6f, 0x76, 0x64, 0x4b, 0x61, 0x6d, 0x74,
0x78, 0x61, 0x74, 0x6b, 0x61, 0x4b, 0x61, 0x72, 0x61, 0x74, 0x78, 0x69,
0x4b, 0x68, 0x61, 0x6e, 0x64, 0x69, 0x67, 0x61, 0x4b, 0x61, 0x6c, 0x6b,
0x75, 0x74, 0x61, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x69, 0x61, 0x72,
0x73, 0x6b, 0x4d, 0x61, 0x73, 0x6b, 0x61, 0x74, 0x50, 0x69, 0x6f, 0x6e,
0x67, 0x69, 0x61, 0x6e, 0x67, 0x4b, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72,
0x64, 0x61, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x6b, 0x61, 0x6e, 0x64, 0x61,
0x54, 0x61, 0x78, 0x6b, 0x65, 0x6e, 0x74, 0x55, 0x6c, 0x61, 0x6e, 0x20,
0x42, 0x61, 0x74, 0x6f, 0x72, 0xdc, 0x72, 0xfc, 0x6d, 0x71, 0x69, 0x56,
0x69, 0x65, 0x6e, 0x74, 0x69, 0x61, 0x6e, 0x45, 0x72, 0x65, 0x76, 0x61,
0x6e, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x61, 0x6b, 0x4b, 0x61, 0x6e, 0x61,
0x72, 0x69, 0x61, 0x6b, 0x43, 0x61, 0x62, 0x6f, 0x20, 0x56, 0x65, 0x72,
0x64, 0x65, 0x48, 0x65, 0x67, 0x6f, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f,
0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x6b, 0x53, 0x61, 0x6e,
0x74, 0x61, 0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x41, 0x74, 0x65,
0x6e, 0x61, 0x73, 0x42, 0x65, 0x6c, 0x67, 0x72, 0x61, 0x64, 0x42, 0x72,
0x75, 0x73, 0x65, 0x6c, 0x61, 0x42, 0x75, 0x6b, 0x61, 0x72, 0x65, 0x73,
0x74, 0x42, 0xfc, 0x73, 0x69, 0x6e, 0x67, 0x65, 0x6e, 0x47, 0x75, 0x65,
0x72, 0x6e, 0x65, 0x73, 0x65, 0x79, 0x4d, 0x61, 0x6e, 0x20, 0x75, 0x68,
0x61, 0x72, 0x74, 0x65, 0x61, 0x4c, 0x75, 0x78, 0x65, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x6f, 0x4d, 0x61, 0x64, 0x72, 0x69, 0x6c, 0x4d, 0x6f, 0x73,
0x6b, 0x75, 0x45, 0x72, 0x72, 0x6f, 0x6d, 0x61, 0x55, 0x6c, 0x69, 0x61,
0x6e, 0x6f, 0x76, 0x73, 0x6b, 0x56, 0x61, 0x74, 0x69, 0x6b, 0x61, 0x6e,
0x6f, 0x20, 0x48, 0x69, 0x72, 0x69, 0x61, 0x4d, 0x61, 0x68, 0xe9, 0x4d,
0x61, 0x6c, 0x64, 0x69, 0x76, 0x61, 0x6b, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x7a, 0x69, 0x6f, 0x50, 0x61, 0x7a, 0x6b, 0x6f, 0x20, 0x75, 0x68, 0x61,
0x72, 0x74, 0x65, 0x61, 0xc9, 0x66, 0x61, 0x74, 0xe9, 0x47, 0x61, 0x6c,
0x61, 0x70, 0x61, 0x67, 0x6f, 0x61, 0x6b, 0x4d, 0x61, 0x72, 0x6b, 0x65,
0x73, 0x61, 0x6b, 0x410, 0x431, 0x456, 0x434, 0x436, 0x430, 0x43d, 0x410, 0x43a,
0x440, 0x430, 0x410, 0x434, 0x44b, 0x441, 0x2d, 0x410, 0x431, 0x435, 0x431, 0x430,
0x410, 0x43b, 0x436, 0x44b, 0x440, 0x410, 0x441, 0x43c, 0x430, 0x440, 0x430, 0x411,
0x430, 0x43d, 0x433, 0x456, 0x411, 0x430, 0x43d, 0x434, 0x436, 0x443, 0x43b, 0x411,
0x456, 0x441, 0x430, 0x443, 0x411, 0x43b, 0x430, 0x43d, 0x442, 0x430, 0x439, 0x440,
0x411, 0x440, 0x430, 0x437, 0x430, 0x432, 0x456, 0x43b, 0x44c, 0x41a, 0x430, 0x456,
0x440, 0x41a, 0x43e, 0x43d, 0x430, 0x43a, 0x440, 0x44b, 0x414, 0x430, 0x440, 0x2d,
0x44d, 0x441, 0x2d, 0x421, 0x430, 0x43b, 0x430, 0x43c, 0x414, 0x436, 0x44b, 0x431,
0x443, 0x446, 0x456, 0x42d, 0x43b, 0x44c, 0x2d, 0x410, 0x44e, 0x43d, 0x424, 0x440,
0x44b, 0x442, 0x430, 0x45e, 0x43d, 0x413, 0x430, 0x431, 0x430, 0x440, 0x43e, 0x43d,
0x44d, 0x425, 0x430, 0x440, 0x430, 0x440, 0x44d, 0x42f, 0x433, 0x430, 0x43d, 0x44d,
0x441, 0x431, 0x443, 0x440, 0x433, 0x414, 0x436, 0x443, 0x431, 0x430, 0x41a, 0x456,
0x433, 0x430, 0x43b, 0x456, 0x41a, 0x456, 0x43d, 0x448, 0x430, 0x441, 0x430, 0x41b,
0x430, 0x433, 0x430, 0x441, 0x41b, 0x456, 0x431, 0x440, 0x44d, 0x432, 0x456, 0x43b,
0x44c, 0x41b, 0x430, 0x43c, 0x44d, 0x41b, 0x443, 0x431, 0x443, 0x43c, 0x431, 0x430,
0x448, 0x44b, 0x41c, 0x430, 0x43b, 0x430, 0x431, 0x430, 0x41c, 0x431, 0x430, 0x431,
0x430, 0x43d, 0x44d, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x44b, 0x448, 0x430, 0x41c,
0x430, 0x43d, 0x440, 0x43e, 0x432, 0x456, 0x44f, 0x41d, 0x430, 0x439, 0x440, 0x43e,
0x431, 0x456, 0x41d, 0x434, 0x436, 0x430, 0x43c, 0x435, 0x43d, 0x430, 0x41d, 0x456,
0x44f, 0x43c, 0x435, 0x439, 0x41f, 0x43e, 0x440, 0x442, 0x430, 0x2d, 0x41d, 0x43e,
0x432, 0x430, 0x421, 0x430, 0x43d, 0x2d, 0x422, 0x430, 0x43c, 0x44d, 0x422, 0x440,
0x44b, 0x43f, 0x430, 0x43b, 0x456, 0x422, 0x443, 0x43d, 0x456, 0x441, 0x412, 0x456,
0x43d, 0x434, 0x445, 0x443, 0x43a, 0x410, 0x43d, 0x43a, 0x430, 0x440, 0x44b, 0x434,
0x436, 0x410, 0x43d, 0x433, 0x456, 0x43b, 0x44c, 0x44f, 0x410, 0x43d, 0x442, 0x44b,
0x433, 0x443, 0x430, 0x410, 0x440, 0x430, 0x433, 0x443, 0x430, 0x456, 0x43d, 0x430,
0x411, 0x443, 0x44d, 0x43d, 0x430, 0x441, 0x2d, 0x410, 0x439, 0x440, 0x44d, 0x441,
0x41a, 0x43e, 0x440, 0x434, 0x430, 0x432, 0x430, 0x416, 0x443, 0x436, 0x443, 0x439,
0x41b, 0x430, 0x2d, 0x420, 0x44b, 0x451, 0x445, 0x430, 0x420, 0x44b, 0x430, 0x2d,
0x413, 0x430, 0x43b, 0x44c, 0x435, 0x433, 0x430, 0x441, 0x421, 0x430, 0x43b, 0x44c,
0x442, 0x430, 0x421, 0x430, 0x43d, 0x2d, 0x41b, 0x443, 0x456, 0x441, 0x423, 0x448,
0x443, 0x430, 0x44f, 0x410, 0x441, 0x443, 0x43d, 0x441, 0x44c, 0x451, 0x43d, 0x410,
0x442, 0x44b, 0x43a, 0x43e, 0x43a, 0x430, 0x43d, 0x411, 0x430, 0x456, 0x44f, 0x411,
0x430, 0x456, 0x44f, 0x2d, 0x434, 0x44d, 0x2d, 0x411, 0x430, 0x43d, 0x434, 0x44d,
0x440, 0x430, 0x441, 0x411, 0x430, 0x440, 0x431, 0x430, 0x434, 0x430, 0x441, 0x411,
0x435, 0x43b, 0x456, 0x437, 0x411, 0x43e, 0x430, 0x2d, 0x412, 0x456, 0x441, 0x442,
0x430, 0x411, 0x430, 0x433, 0x430, 0x442, 0x430, 0x411, 0x43e, 0x439, 0x441, 0x44d,
0x41a, 0x435, 0x43c, 0x431, 0x440, 0x44b, 0x434, 0x436, 0x2d, 0x411, 0x435, 0x439,
0x41a, 0x430, 0x43c, 0x43f, 0x443, 0x2d, 0x413, 0x440, 0x430, 0x43d, 0x434, 0x44b,
0x41a, 0x430, 0x435, 0x43d, 0x430, 0x41a, 0x430, 0x439, 0x43c, 0x430, 0x43d, 0x430,
0x432, 0x44b, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x44b, 0x427, 0x44b,
0x43a, 0x430, 0x433, 0x430, 0x427, 0x44b, 0x45e, 0x430, 0x45e, 0x430, 0x421, 0x44c,
0x44e, 0x434, 0x430, 0x434, 0x2d, 0x425, 0x443, 0x430, 0x440, 0x44d, 0x441, 0x41a,
0x43e, 0x441, 0x442, 0x430, 0x2d, 0x420, 0x44b, 0x43a, 0x430, 0x41a, 0x440, 0x44d,
0x441, 0x442, 0x430, 0x43d, 0x41a, 0x443, 0x44f, 0x431, 0x430, 0x41a, 0x44e, 0x440,
0x430, 0x441, 0x430, 0x430, 0x414, 0x430, 0x43d, 0x43c, 0x430, 0x440, 0x43a, 0x441,
0x445, 0x430, 0x45e, 0x43d, 0x414, 0x43e, 0x45e, 0x441, 0x430, 0x43d, 0x414, 0x43e,
0x441, 0x430, 0x43d, 0x2d, 0x41a, 0x440, 0x44b, 0x43a, 0x414, 0x44d, 0x43d, 0x432,
0x435, 0x440, 0x414, 0x44d, 0x442, 0x440, 0x43e, 0x439, 0x442, 0x414, 0x430, 0x43c,
0x456, 0x43d, 0x456, 0x43a, 0x430, 0x42d, 0x434, 0x43c, 0x430, 0x43d, 0x442, 0x430,
0x43d, 0x42d, 0x439, 0x440, 0x443, 0x43d, 0x44d, 0x43f, 0x435, 0x421, 0x430, 0x43b,
0x44c, 0x432, 0x430, 0x434, 0x43e, 0x440, 0x424, 0x43e, 0x440, 0x442, 0x2d, 0x41d,
0x435, 0x43b, 0x44c, 0x441, 0x430, 0x43d, 0x424, 0x430, 0x440, 0x442, 0x430, 0x43b,
0x435, 0x437, 0x430, 0x413, 0x43b, 0x44d, 0x439, 0x441, 0x2d, 0x411, 0x44d, 0x439,
0x413, 0x443, 0x441, 0x2d, 0x411, 0x44d, 0x439, 0x413, 0x440, 0x430, 0x43d, 0x434,
0x2d, 0x426, 0x451, 0x440, 0x43a, 0x413, 0x440, 0x44d, 0x43d, 0x430, 0x434, 0x430,
0x413, 0x432, 0x430, 0x434, 0x44d, 0x43b, 0x443, 0x43f, 0x430, 0x413, 0x432, 0x430,
0x442, 0x44d, 0x43c, 0x430, 0x43b, 0x430, 0x413, 0x443, 0x430, 0x44f, 0x43a, 0x456,
0x43b, 0x44c, 0x413, 0x430, 0x44f, 0x43d, 0x430, 0x413, 0x430, 0x43b, 0x456, 0x444,
0x430, 0x43a, 0x441, 0x42d, 0x440, 0x43c, 0x430, 0x441, 0x456, 0x43b, 0x44c, 0x44f,
0x406, 0x43d, 0x434, 0x44b, 0x44f, 0x43d, 0x430, 0x43f, 0x430, 0x43b, 0x456, 0x441,
0x41d, 0x43e, 0x43a, 0x441, 0x2c, 0x20, 0x406, 0x43d, 0x434, 0x44b, 0x44f, 0x43d,
0x430, 0x41c, 0x430, 0x440, 0x44d, 0x43d, 0x433, 0x430, 0x2c, 0x20, 0x406, 0x43d,
0x434, 0x44b, 0x44f, 0x43d, 0x430, 0x41f, 0x456, 0x442, 0x44d, 0x440, 0x441, 0x431,
0x435, 0x440, 0x433, 0x2c, 0x20, 0x406, 0x43d, 0x434, 0x44b, 0x44f, 0x43d, 0x430,
0x422, 0x44d, 0x43b, 0x20, 0x421, 0x456, 0x446, 0x456, 0x2c, 0x20, 0x406, 0x43d,
0x434, 0x44b, 0x44f, 0x43d, 0x430, 0x412, 0x456, 0x432, 0x456, 0x2c, 0x20, 0x406,
0x43d, 0x434, 0x44b, 0x44f, 0x43d, 0x430, 0x412, 0x456, 0x43d, 0x441, 0x435, 0x43d,
0x441, 0x2c, 0x20, 0x406, 0x43d, 0x434, 0x44b, 0x44f, 0x43d, 0x430, 0x423, 0x456,
0x43d, 0x430, 0x43c, 0x430, 0x43a, 0x2c, 0x20, 0x406, 0x43d, 0x434, 0x44b, 0x44f,
0x43d, 0x430, 0x406, 0x43d, 0x443, 0x432, 0x456, 0x43a, 0x406, 0x43a, 0x430, 0x43b,
0x443, 0x456, 0x442, 0x42f, 0x43c, 0x430, 0x439, 0x43a, 0x430, 0x414, 0x436, 0x443,
0x43d, 0x430, 0x41b, 0x443, 0x456, 0x441, 0x432, 0x456, 0x43b, 0x41c, 0x430, 0x43d,
0x442, 0x44b, 0x441, 0x435, 0x43b, 0x430, 0x2c, 0x20, 0x41a, 0x435, 0x43d, 0x442,
0x443, 0x43a, 0x456, 0x41a, 0x440, 0x430, 0x43b, 0x435, 0x43d, 0x434, 0x44d, 0x439,
0x43a, 0x41b, 0x456, 0x43c, 0x430, 0x41b, 0x43e, 0x441, 0x2d, 0x410, 0x43d, 0x434,
0x436, 0x44d, 0x43b, 0x435, 0x441, 0x41b, 0x43e, 0x45e, 0x44d, 0x440, 0x20, 0x41f,
0x440, 0x44b, 0x43d, 0x441, 0x456, 0x437, 0x20, 0x41a, 0x432, 0x43e, 0x442, 0x44d,
0x440, 0x41c, 0x430, 0x441, 0x435, 0x451, 0x41c, 0x430, 0x43d, 0x430, 0x45e, 0x441,
0x41c, 0x430, 0x440, 0x44b, 0x433, 0x43e, 0x41c, 0x430, 0x440, 0x446, 0x456, 0x43d,
0x456, 0x43a, 0x430, 0x41c, 0x430, 0x442, 0x430, 0x43c, 0x43e, 0x440, 0x430, 0x441,
0x41c, 0x435, 0x43d, 0x43e, 0x43c, 0x456, 0x43d, 0x456, 0x41c, 0x435, 0x440, 0x44b,
0x434, 0x430, 0x41c, 0x435, 0x445, 0x456, 0x43a, 0x430, 0x41c, 0x456, 0x43a, 0x435,
0x43b, 0x43e, 0x43d, 0x41c, 0x43e, 0x43d, 0x43a, 0x442, 0x430, 0x43d, 0x41c, 0x430,
0x43d, 0x442, 0x44d, 0x440, 0x44d, 0x439, 0x41c, 0x430, 0x43d, 0x442, 0x44d, 0x432,
0x456, 0x434, 0x44d, 0x430, 0x41c, 0x430, 0x43d, 0x442, 0x441, 0x435, 0x440, 0x430,
0x442, 0x41d, 0x430, 0x441, 0x430, 0x45e, 0x41d, 0x44c, 0x44e, 0x2d, 0x401, 0x440,
0x43a, 0x41d, 0x430, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x411, 0x2019, 0x44e, 0x43b,
0x430, 0x2c, 0x20, 0x41f, 0x430, 0x45e, 0x43d, 0x43e, 0x447, 0x43d, 0x430, 0x44f,
0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x421, 0x435, 0x43d, 0x442, 0x44d,
0x440, 0x2c, 0x20, 0x41f, 0x430, 0x45e, 0x43d, 0x43e, 0x447, 0x43d, 0x430, 0x44f,
0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41d, 0x44c, 0x44e, 0x2d, 0x421,
0x435, 0x439, 0x43b, 0x435, 0x43c, 0x2c, 0x20, 0x41f, 0x430, 0x45e, 0x43d, 0x43e,
0x447, 0x43d, 0x430, 0x44f, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41d,
0x443, 0x43a, 0x410, 0x445, 0x456, 0x43d, 0x430, 0x433, 0x430, 0x41f, 0x430, 0x440,
0x430, 0x43c, 0x430, 0x440, 0x44b, 0x431, 0x430, 0x424, 0x456, 0x43d, 0x456, 0x43a,
0x441, 0x41f, 0x43e, 0x440, 0x442, 0x2d, 0x43e, 0x2d, 0x41f, 0x440, 0x44d, 0x43d,
0x441, 0x41f, 0x43e, 0x440, 0x442, 0x2d, 0x43e, 0x444, 0x2d, 0x421, 0x43f, 0x435,
0x439, 0x43d, 0x41f, 0x43e, 0x440, 0x442, 0x443, 0x2d, 0x412, 0x435, 0x43b, 0x44c,
0x44e, 0x41f, 0x443, 0x44d, 0x440, 0x442, 0x430, 0x2d, 0x420, 0x44b, 0x43a, 0x430,
0x41f, 0x443, 0x43d, 0x442, 0x430, 0x2d, 0x410, 0x440, 0x44d, 0x43d, 0x430, 0x441,
0x420, 0x430, 0x43d, 0x43a, 0x456, 0x43d, 0x2d, 0x406, 0x43d, 0x43b, 0x435, 0x442,
0x420, 0x44d, 0x441, 0x456, 0x444, 0x456, 0x420, 0x44d, 0x434, 0x436, 0x430, 0x439,
0x43d, 0x430, 0x420, 0x44d, 0x437, 0x430, 0x43b, 0x44c, 0x44e, 0x442, 0x420, 0x44b,
0x443, 0x2d, 0x411, 0x440, 0x430, 0x43d, 0x43a, 0x443, 0x421, 0x430, 0x43d, 0x442,
0x430, 0x440, 0x44d, 0x43d, 0x421, 0x430, 0x43d, 0x442, 0x2019, 0x44f, 0x433, 0x430,
0x421, 0x430, 0x43d, 0x442, 0x430, 0x2d, 0x414, 0x430, 0x43c, 0x456, 0x43d, 0x433,
0x430, 0x421, 0x430, 0x43d, 0x2d, 0x41f, 0x430, 0x45e, 0x43b, 0x443, 0x406, 0x442,
0x430, 0x43a, 0x430, 0x440, 0x442, 0x430, 0x430, 0x440, 0x43c, 0x456, 0x442, 0x421,
0x456, 0x442, 0x43a, 0x430, 0x421, 0x435, 0x43d, 0x2d, 0x411, 0x430, 0x440, 0x442,
0x44d, 0x43b, 0x44c, 0x43c, 0x456, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x414, 0x436,
0x43e, 0x43d, 0x441, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41a, 0x456, 0x442, 0x441,
0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41b, 0x44e, 0x441, 0x456, 0x44f, 0x421, 0x435,
0x43d, 0x442, 0x2d, 0x412, 0x456, 0x43d, 0x441, 0x435, 0x43d, 0x442, 0x421, 0x432,
0x456, 0x444, 0x442, 0x2d, 0x41a, 0x435, 0x440, 0x44d, 0x43d, 0x442, 0x422, 0x44d,
0x433, 0x443, 0x441, 0x456, 0x433, 0x430, 0x43b, 0x44c, 0x43f, 0x430, 0x422, 0x44b,
0x445, 0x443, 0x430, 0x43d, 0x430, 0x422, 0x430, 0x440, 0x43e, 0x43d, 0x442, 0x430,
0x422, 0x430, 0x440, 0x442, 0x43e, 0x43b, 0x430, 0x423, 0x430, 0x439, 0x442, 0x445,
0x43e, 0x440, 0x441, 0x412, 0x456, 0x43d, 0x456, 0x43f, 0x435, 0x433, 0x42f, 0x43a,
0x443, 0x442, 0x430, 0x442, 0x41a, 0x44d, 0x439, 0x441, 0x456, 0x414, 0x44d, 0x439,
0x432, 0x456, 0x441, 0x414, 0x437, 0x44e, 0x43c, 0x43e, 0x43d, 0x2d, 0x414, 0x437,
0x44e, 0x440, 0x432, 0x456, 0x43b, 0x44c, 0x41c, 0x430, 0x43a, 0x443, 0x43e, 0x440,
0x44b, 0x41c, 0x43e, 0x45e, 0x441, 0x430, 0x43d, 0x41c, 0x430, 0x43a, 0x2d, 0x41c,
0x435, 0x440, 0x434, 0x430, 0x420, 0x43e, 0x442, 0x44d, 0x440, 0x430, 0x421, 0x451,
0x432, 0x430, 0x412, 0x430, 0x441, 0x442, 0x43e, 0x43a, 0x41b, 0x43e, 0x43d, 0x433,
0x439, 0x456, 0x440, 0x410, 0x434, 0x44d, 0x43d, 0x410, 0x43c, 0x430, 0x43d, 0x20,
0x28, 0x433, 0x43e, 0x440, 0x430, 0x434, 0x29, 0x410, 0x43a, 0x442, 0x430, 0x431,
0x44d, 0x410, 0x448, 0x433, 0x430, 0x431, 0x430, 0x442, 0x411, 0x430, 0x445, 0x440,
0x44d, 0x439, 0x43d, 0x411, 0x435, 0x439, 0x440, 0x443, 0x442, 0x411, 0x456, 0x448,
0x43a, 0x435, 0x43a, 0x411, 0x440, 0x443, 0x43d, 0x435, 0x439, 0x427, 0x44b, 0x442,
0x430, 0x41a, 0x430, 0x43b, 0x43e, 0x43c, 0x431, 0x430, 0x414, 0x44b, 0x43b, 0x456,
0x414, 0x443, 0x431, 0x430, 0x439, 0x414, 0x443, 0x448, 0x430, 0x43d, 0x431, 0x44d,
0x425, 0x435, 0x45e, 0x440, 0x43e, 0x43d, 0x425, 0x430, 0x448, 0x44b, 0x43c, 0x456,
0x43d, 0x413, 0x430, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x425, 0x43e, 0x45e, 0x434,
0x406, 0x440, 0x43a, 0x443, 0x446, 0x43a, 0x414, 0x436, 0x430, 0x43a, 0x430, 0x440,
0x442, 0x430, 0x414, 0x436, 0x430, 0x44f, 0x43f, 0x443, 0x440, 0x430, 0x406, 0x435,
0x440, 0x443, 0x441, 0x430, 0x43b, 0x456, 0x43c, 0x41a, 0x430, 0x440, 0x430, 0x447,
0x44b, 0x41a, 0x430, 0x43b, 0x44c, 0x43a, 0x443, 0x442, 0x430, 0x41a, 0x440, 0x430,
0x441, 0x43d, 0x430, 0x44f, 0x440, 0x441, 0x43a, 0x41a, 0x443, 0x447, 0x44b, 0x43d,
0x433, 0x41a, 0x443, 0x432, 0x435, 0x439, 0x442, 0x41c, 0x430, 0x43a, 0x430, 0x430,
0x41c, 0x430, 0x43d, 0x456, 0x43b, 0x430, 0x41d, 0x456, 0x43a, 0x430, 0x441, 0x456,
0x44f, 0x41d, 0x43e, 0x432, 0x430, 0x43a, 0x443, 0x437, 0x43d, 0x435, 0x446, 0x43a,
0x41d, 0x43e, 0x432, 0x430, 0x441, 0x456, 0x431, 0x456, 0x440, 0x441, 0x43a, 0x423,
0x440, 0x430, 0x43b, 0x44c, 0x441, 0x43a, 0x41f, 0x43d, 0x430, 0x43c, 0x43f, 0x435,
0x43d, 0x44c, 0x41f, 0x430, 0x43d, 0x442, 0x44b, 0x44f, 0x43d, 0x430, 0x43a, 0x41f,
0x445, 0x435, 0x43d, 0x44c, 0x44f, 0x43d, 0x41a, 0x443, 0x441, 0x442, 0x430, 0x43d,
0x430, 0x439, 0x41a, 0x437, 0x44b, 0x43b, 0x2d, 0x410, 0x440, 0x434, 0x430, 0x42d,
0x440, 0x2d, 0x420, 0x44b, 0x44f, 0x434, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x456,
0x43d, 0x428, 0x430, 0x43d, 0x445, 0x430, 0x439, 0x421, 0x456, 0x43d, 0x433, 0x430,
0x43f, 0x443, 0x440, 0x421, 0x44f, 0x440, 0x44d, 0x434, 0x43d, 0x435, 0x43a, 0x430,
0x43b, 0x44b, 0x43c, 0x441, 0x43a, 0x422, 0x430, 0x439, 0x431, 0x44d, 0x439, 0x422,
0x431, 0x456, 0x43b, 0x456, 0x441, 0x456, 0x422, 0x44d, 0x433, 0x435, 0x440, 0x430,
0x43d, 0x422, 0x445, 0x456, 0x43c, 0x43f, 0x445, 0x443, 0x422, 0x43e, 0x43a, 0x456,
0x430, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x430, 0x440, 0x423,
0x440, 0x443, 0x43c, 0x447, 0x44b, 0x423, 0x441, 0x446, 0x44c, 0x2d, 0x41d, 0x435,
0x440, 0x430, 0x412, 0x2019, 0x435, 0x43d, 0x446, 0x44c, 0x44f, 0x43d, 0x423, 0x43b,
0x430, 0x434, 0x437, 0x456, 0x432, 0x430, 0x441, 0x442, 0x43e, 0x43a, 0x42f, 0x43a,
0x443, 0x446, 0x43a, 0x420, 0x430, 0x43d, 0x433, 0x443, 0x43d, 0x415, 0x43a, 0x430,
0x446, 0x44f, 0x440, 0x44b, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x415, 0x440, 0x44d,
0x432, 0x430, 0x43d, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x43a, 0x456, 0x44f, 0x20,
0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x44b, 0x411, 0x435, 0x440, 0x43c, 0x443,
0x434, 0x441, 0x43a, 0x456, 0x44f, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432,
0x44b, 0x41a, 0x430, 0x43d, 0x430, 0x440, 0x441, 0x43a, 0x456, 0x44f, 0x20, 0x430,
0x441, 0x442, 0x440, 0x430, 0x432, 0x44b, 0x41a, 0x430, 0x431, 0x430, 0x2d, 0x412,
0x435, 0x440, 0x434, 0x44d, 0x424, 0x430, 0x440, 0x44d, 0x440, 0x441, 0x43a, 0x456,
0x44f, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x44b, 0x41c, 0x430, 0x434,
0x44d, 0x439, 0x440, 0x430, 0x420, 0x44d, 0x439, 0x43a, 0x2019, 0x44f, 0x432, 0x456,
0x43a, 0x41f, 0x430, 0x45e, 0x434, 0x43d, 0x451, 0x432, 0x430, 0x44f, 0x20, 0x413,
0x435, 0x43e, 0x440, 0x433, 0x456, 0x44f, 0x412, 0x43e, 0x441, 0x442, 0x440, 0x430,
0x45e, 0x20, 0x421, 0x432, 0x44f, 0x442, 0x43e, 0x439, 0x20, 0x410, 0x43b, 0x435,
0x43d, 0x44b, 0x41f, 0x43e, 0x440, 0x442, 0x2d, 0x421, 0x442, 0x44d, 0x43d, 0x43b,
0x456, 0x410, 0x434, 0x44d, 0x43b, 0x430, 0x456, 0x434, 0x430, 0x411, 0x440, 0x44b,
0x441, 0x431, 0x435, 0x43d, 0x411, 0x440, 0x43e, 0x43a, 0x435, 0x43d, 0x2d, 0x425,
0x456, 0x43b, 0x414, 0x430, 0x440, 0x432, 0x456, 0x43d, 0x42e, 0x43a, 0x43b, 0x430,
0x41b, 0x456, 0x43d, 0x434, 0x44d, 0x43c, 0x430, 0x43d, 0x41c, 0x435, 0x43b, 0x44c,
0x431, 0x443, 0x440, 0x43d, 0x421, 0x456, 0x434, 0x43d, 0x435, 0x439, 0x410, 0x43c,
0x441, 0x442, 0x44d, 0x440, 0x434, 0x430, 0x43c, 0x410, 0x43d, 0x434, 0x43e, 0x440,
0x430, 0x410, 0x441, 0x442, 0x440, 0x430, 0x445, 0x430, 0x43d, 0x44c, 0x410, 0x444,
0x456, 0x43d, 0x44b, 0x411, 0x44f, 0x43b, 0x433, 0x440, 0x430, 0x434, 0x411, 0x435,
0x440, 0x43b, 0x456, 0x43d, 0x411, 0x440, 0x430, 0x446, 0x456, 0x441, 0x43b, 0x430,
0x432, 0x430, 0x411, 0x440, 0x443, 0x441, 0x435, 0x43b, 0x44c, 0x411, 0x443, 0x445,
0x430, 0x440, 0x44d, 0x441, 0x442, 0x411, 0x44e, 0x437, 0x456, 0x43d, 0x433, 0x435,
0x43d, 0x41a, 0x456, 0x448, 0x44b, 0x43d, 0x451, 0x45e, 0x41a, 0x430, 0x43f, 0x435,
0x43d, 0x433, 0x430, 0x433, 0x435, 0x43d, 0x414, 0x443, 0x431, 0x43b, 0x456, 0x43d,
0x413, 0x456, 0x431, 0x440, 0x430, 0x43b, 0x442, 0x430, 0x440, 0x413, 0x435, 0x440,
0x43d, 0x441, 0x456, 0x425, 0x435, 0x43b, 0x44c, 0x441, 0x456, 0x43d, 0x43a, 0x456,
0x412, 0x43e, 0x441, 0x442, 0x440, 0x430, 0x45e, 0x20, 0x41c, 0x44d, 0x43d, 0x414,
0x436, 0x44d, 0x440, 0x441, 0x456, 0x41a, 0x430, 0x43b, 0x456, 0x43d, 0x456, 0x43d,
0x433, 0x440, 0x430, 0x434, 0x41a, 0x456, 0x440, 0x430, 0x45e, 0x41a, 0x456, 0x435,
0x45e, 0x41b, 0x456, 0x441, 0x430, 0x431, 0x43e, 0x43d, 0x41b, 0x44e, 0x431, 0x43b,
0x44f, 0x43d, 0x430, 0x41b, 0x43e, 0x43d, 0x434, 0x430, 0x43d, 0x41b, 0x44e, 0x43a,
0x441, 0x435, 0x43c, 0x431, 0x443, 0x440, 0x433, 0x41c, 0x430, 0x434, 0x440, 0x44b,
0x434, 0x41c, 0x430, 0x43b, 0x44c, 0x442, 0x430, 0x41c, 0x430, 0x440, 0x44b, 0x44f,
0x43d, 0x445, 0x430, 0x43c, 0x456, 0x43d, 0x430, 0x41c, 0x456, 0x43d, 0x441, 0x43a,
0x41c, 0x430, 0x43d, 0x430, 0x43a, 0x430, 0x41c, 0x430, 0x441, 0x43a, 0x432, 0x430,
0x41e, 0x441, 0x43b, 0x430, 0x41f, 0x430, 0x440, 0x44b, 0x436, 0x41f, 0x430, 0x434,
0x433, 0x43e, 0x440, 0x44b, 0x446, 0x430, 0x420, 0x44b, 0x433, 0x430, 0x420, 0x44b,
0x43c, 0x421, 0x430, 0x43d, 0x2d, 0x41c, 0x430, 0x440, 0x44b, 0x43d, 0x430, 0x421,
0x430, 0x440, 0x430, 0x435, 0x432, 0x430, 0x421, 0x430, 0x440, 0x430, 0x442, 0x430,
0x45e, 0x421, 0x456, 0x43c, 0x444, 0x435, 0x440, 0x43e, 0x43f, 0x430, 0x43b, 0x44c,
0x421, 0x43a, 0x43e, 0x43f, 0x2019, 0x435, 0x421, 0x430, 0x444, 0x456, 0x44f, 0x421,
0x442, 0x430, 0x43a, 0x433, 0x43e, 0x43b, 0x44c, 0x43c, 0x422, 0x430, 0x43b, 0x456,
0x43d, 0x422, 0x44b, 0x440, 0x430, 0x43d, 0x430, 0x423, 0x43b, 0x44c, 0x44f, 0x43d,
0x430, 0x45e, 0x441, 0x43a, 0x412, 0x430, 0x442, 0x44b, 0x43a, 0x430, 0x43d, 0x412,
0x456, 0x43b, 0x44c, 0x43d, 0x44e, 0x441, 0x412, 0x430, 0x43b, 0x433, 0x430, 0x433,
0x440, 0x430, 0x434, 0x417, 0x430, 0x433, 0x440, 0x430, 0x431, 0x426, 0x44e, 0x440,
0x44b, 0x445, 0x410, 0x43d, 0x442, 0x430, 0x43d, 0x430, 0x43d, 0x430, 0x440, 0x44b,
0x432, 0x443, 0x427, 0x430, 0x433, 0x430, 0x441, 0x412, 0x43e, 0x441, 0x442, 0x440,
0x430, 0x45e, 0x20, 0x41a, 0x430, 0x43b, 0x44f, 0x434, 0x41a, 0x430, 0x43a, 0x43e,
0x441, 0x430, 0x432, 0x44b, 0x44f, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432,
0x44b, 0x41a, 0x430, 0x43c, 0x43e, 0x440, 0x441, 0x43a, 0x456, 0x44f, 0x20, 0x430,
0x441, 0x442, 0x440, 0x430, 0x432, 0x44b, 0x41c, 0x430, 0x44d, 0x41c, 0x430, 0x43b,
0x44c, 0x434, 0x44b, 0x432, 0x44b, 0x41c, 0x430, 0x45e, 0x440, 0x44b, 0x43a, 0x456,
0x439, 0x41c, 0x430, 0x451, 0x442, 0x430, 0x420, 0x44d, 0x44e, 0x43d, 0x44c, 0x451,
0x43d, 0x410, 0x43f, 0x456, 0x44f, 0x411, 0x443, 0x433, 0x435, 0x43d, 0x432, 0x456,
0x43b, 0x44c, 0x427, 0x430, 0x442, 0x44d, 0x43c, 0x412, 0x44f, 0x43b, 0x456, 0x43a,
0x430, 0x434, 0x43d, 0x44f, 0x20, 0x432, 0x43e, 0x441, 0x442, 0x440, 0x430, 0x45e,
0x42d, 0x444, 0x430, 0x442, 0x44d, 0x424, 0x430, 0x43a, 0x430, 0x43e, 0x444, 0x430,
0x424, 0x456, 0x434, 0x436, 0x44b, 0x424, 0x443, 0x43d, 0x430, 0x444, 0x443, 0x446,
0x456, 0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433, 0x430, 0x441, 0x410, 0x441,
0x442, 0x440, 0x430, 0x432, 0x44b, 0x20, 0x413, 0x430, 0x43c, 0x431, 0x2019, 0x435,
0x413, 0x430, 0x43d, 0x430, 0x43b, 0x443, 0x43b, 0x443, 0x42d, 0x43d, 0x434, 0x44d,
0x440, 0x431, 0x435, 0x440, 0x44b, 0x41a, 0x456, 0x440, 0x44b, 0x446, 0x456, 0x43c,
0x430, 0x446, 0x456, 0x41a, 0x443, 0x441, 0x430, 0x456, 0x435, 0x41a, 0x432, 0x430,
0x434, 0x436, 0x430, 0x43b, 0x435, 0x439, 0x43d, 0x41c, 0x430, 0x434, 0x436, 0x443,
0x440, 0x430, 0x41c, 0x430, 0x440, 0x43a, 0x456, 0x437, 0x441, 0x43a, 0x456, 0x44f,
0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x44b, 0x41c, 0x456, 0x434, 0x443,
0x44d, 0x439, 0x41d, 0x456, 0x443, 0x44d, 0x41d, 0x43e, 0x440, 0x444, 0x430, 0x43b,
0x43a, 0x41f, 0x430, 0x433, 0x430, 0x2d, 0x41f, 0x430, 0x433, 0x430, 0x41f, 0x456,
0x442, 0x43a, 0x44d, 0x440, 0x43d, 0x41f, 0x430, 0x43d, 0x43f, 0x435, 0x456, 0x41f,
0x43e, 0x440, 0x442, 0x2d, 0x41c, 0x43e, 0x440, 0x441, 0x431, 0x456, 0x420, 0x430,
0x440, 0x430, 0x442, 0x43e, 0x43d, 0x433, 0x430, 0x421, 0x430, 0x439, 0x43f, 0x430,
0x43d, 0x422, 0x430, 0x456, 0x446, 0x456, 0x422, 0x430, 0x43d, 0x433, 0x430, 0x442,
0x430, 0x43f, 0x443, 0x423, 0x44d, 0x439, 0x43a, 0x423, 0x43e, 0x43b, 0x456, 0x441,
0x905, 0x92c, 0x93f, 0x926, 0x91c, 0x93e, 0x928, 0x906, 0x915, 0x94d, 0x930, 0x93e,
0x90f, 0x926, 0x94d, 0x926, 0x93f, 0x938, 0x20, 0x906, 0x92c, 0x93e, 0x92c, 0x93e,
0x906, 0x932, 0x94d, 0x91c, 0x940, 0x90f, 0x930, 0x94d, 0x938, 0x906, 0x938, 0x94d,
0x92e, 0x93e, 0x930, 0x93e, 0x92c, 0x93e, 0x92e, 0x93e, 0x915, 0x2019, 0x92c, 0x93e,
0x902, 0x917, 0x942, 0x908, 0x92c, 0x928, 0x94d, 0x91c, 0x941, 0x932, 0x92c, 0x93f,
0x938, 0x94d, 0x938, 0x93e, 0x909, 0x92c, 0x94d, 0x932, 0x93e, 0x928, 0x94d, 0x91f,
0x93e, 0x908, 0x930, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x94d, 0x91c, 0x93e, 0x92d,
0x940, 0x932, 0x92c, 0x941, 0x91c, 0x941, 0x92e, 0x94d, 0x92c, 0x941, 0x930, 0x93e,
0x915, 0x93e, 0x92f, 0x930, 0x2019, 0x915, 0x93e, 0x938, 0x93e, 0x92c, 0x94d, 0x932,
0x93e, 0x902, 0x915, 0x93e, 0x938, 0x947, 0x909, 0x924, 0x93e, 0x915, 0x928, 0x93e,
0x915, 0x94d, 0x930, 0x940, 0x921, 0x93e, 0x915, 0x93e, 0x930, 0x926, 0x93e, 0x930,
0x20, 0x90f, 0x938, 0x20, 0x938, 0x932, 0x93e, 0x92e, 0x91c, 0x93f, 0x92c, 0x94c,
0x91f, 0x940, 0x926, 0x941, 0x906, 0x932, 0x93e, 0x90f, 0x932, 0x20, 0x906, 0x907,
0x90a, 0x928, 0x92b, 0x94d, 0x930, 0x940, 0x91f, 0x93e, 0x909, 0x928, 0x917, 0x948,
0x92c, 0x94b, 0x930, 0x94b, 0x928, 0x939, 0x93e, 0x930, 0x93e, 0x930, 0x947, 0x91c,
0x939, 0x93e, 0x928, 0x94d, 0x938, 0x92c, 0x93e, 0x930, 0x94d, 0x917, 0x91c, 0x941,
0x92c, 0x93e, 0x915, 0x93e, 0x92e, 0x94d, 0x92a, 0x93e, 0x932, 0x93e, 0x916, 0x93e,
0x930, 0x94d, 0x924, 0x942, 0x92e, 0x915, 0x93f, 0x917, 0x93e, 0x932, 0x93f, 0x915,
0x93f, 0x928, 0x94d, 0x936, 0x93e, 0x938, 0x93e, 0x932, 0x93e, 0x917, 0x2019, 0x938,
0x932, 0x93f, 0x92c, 0x94d, 0x930, 0x947, 0x92d, 0x93f, 0x932, 0x932, 0x92e, 0x947,
0x932, 0x941, 0x906, 0x928, 0x94d, 0x921, 0x93e, 0x932, 0x941, 0x92c, 0x941, 0x92e,
0x94d, 0x92c, 0x93e, 0x936, 0x940, 0x932, 0x941, 0x938, 0x93e, 0x915, 0x93e, 0x92e,
0x93e, 0x932, 0x93e, 0x92c, 0x2019, 0x92e, 0x93e, 0x92a, 0x941, 0x91f, 0x2019, 0x92e,
0x93e, 0x938, 0x947, 0x930, 0x942, 0x92e, 0x94d, 0x92c, 0x93e, 0x92c, 0x93e, 0x928,
0x947, 0x92e, 0x917, 0x93e, 0x921, 0x93f, 0x936, 0x941, 0x92e, 0x928, 0x94d, 0x930,
0x2019, 0x92d, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x908, 0x930, 0x94c, 0x92c, 0x940,
0x928, 0x94d, 0x91c, 0x93e, 0x92e, 0x947, 0x928, 0x93e, 0x928, 0x940, 0x906, 0x92e,
0x947, 0x928, 0x94d, 0x935, 0x93e, 0x915, 0x936, 0x2019, 0x924, 0x90a, 0x917, 0x93e,
0x921, 0x941, 0x917, 0x941, 0x92a, 0x930, 0x94d, 0x91f, 0x2019, 0x2d, 0x928, 0x92d,
0x2019, 0x938, 0x93e, 0x935, 0x20, 0x91f, 0x92e, 0x947, 0x924, 0x94d, 0x930, 0x93f,
0x92a, 0x2019, 0x932, 0x940, 0x91f, 0x94d, 0x92f, 0x942, 0x928, 0x93f, 0x938, 0x935,
0x93f, 0x928, 0x94d, 0x926, 0x939, 0x2019, 0x915, 0x90f, 0x921, 0x947, 0x915, 0x90f,
0x902, 0x915, 0x930, 0x947, 0x91c, 0x90f, 0x902, 0x917, 0x94d, 0x935, 0x940, 0x932,
0x93e, 0x90f, 0x928, 0x94d, 0x91f, 0x93f, 0x917, 0x941, 0x935, 0x93e, 0x906, 0x930,
0x93e, 0x917, 0x941, 0x906, 0x908, 0x928, 0x93e, 0x92c, 0x941, 0x90f, 0x928, 0x2019,
0x938, 0x20, 0x90f, 0x930, 0x947, 0x938, 0x915, 0x93e, 0x91f, 0x93e, 0x92e, 0x93e,
0x930, 0x94d, 0x915, 0x93e, 0x915, 0x930, 0x94d, 0x921, 0x92c, 0x93e, 0x91c, 0x941,
0x91c, 0x941, 0x908, 0x932, 0x93e, 0x20, 0x930, 0x93f, 0x905, 0x91c, 0x93e, 0x92e,
0x947, 0x928, 0x94d, 0x921, 0x2019, 0x91c, 0x93e, 0x930, 0x93f, 0x905, 0x20, 0x917,
0x93e, 0x932, 0x947, 0x917, 0x938, 0x938, 0x93e, 0x932, 0x94d, 0x91f, 0x93e, 0x938,
0x93e, 0x928, 0x20, 0x939, 0x941, 0x935, 0x93e, 0x928, 0x938, 0x93e, 0x928, 0x20,
0x932, 0x941, 0x908, 0x938, 0x91f, 0x941, 0x915, 0x941, 0x92e, 0x93e, 0x928, 0x909,
0x936, 0x942, 0x906, 0x92f, 0x93f, 0x92f, 0x93e, 0x906, 0x930, 0x942, 0x92c, 0x93e,
0x906, 0x938, 0x941, 0x928, 0x938, 0x940, 0x905, 0x928, 0x906, 0x924, 0x93f, 0x915,
0x915, 0x93e, 0x928, 0x92c, 0x93e, 0x939, 0x93f, 0x92f, 0x93e, 0x92c, 0x93e, 0x939,
0x93f, 0x92f, 0x93e, 0x20, 0x92c, 0x93e, 0x928, 0x94d, 0x926, 0x947, 0x930, 0x93e,
0x938, 0x92c, 0x93e, 0x930, 0x94d, 0x92c, 0x93e, 0x921, 0x2019, 0x938, 0x92c, 0x947,
0x932, 0x947, 0x92e, 0x92c, 0x947, 0x932, 0x93f, 0x91c, 0x92c, 0x94d, 0x932, 0x93e,
0x902, 0x915, 0x2d, 0x20, 0x938, 0x93e, 0x92c, 0x94d, 0x932, 0x928, 0x92c, 0x906,
0x20, 0x92d, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x92c, 0x917, 0x2019, 0x91f, 0x93e,
0x92c, 0x92f, 0x938, 0x93f, 0x915, 0x947, 0x92e, 0x94d, 0x92c, 0x94d, 0x930, 0x93f,
0x91c, 0x20, 0x917, 0x93e, 0x925, 0x94b, 0x928, 0x915, 0x947, 0x92e, 0x94d, 0x92a,
0x20, 0x917, 0x94d, 0x930, 0x93e, 0x928, 0x94d, 0x921, 0x915, 0x93e, 0x902, 0x915,
0x941, 0x928, 0x915, 0x93e, 0x930, 0x93e, 0x915, 0x93e, 0x938, 0x915, 0x93e, 0x92f,
0x947, 0x928, 0x915, 0x948, 0x92e, 0x947, 0x928, 0x936, 0x93f, 0x915, 0x93e, 0x917,
0x2019, 0x91a, 0x941, 0x939, 0x92d, 0x93e, 0x92d, 0x93e, 0x915, 0x938, 0x94d, 0x91f,
0x93e, 0x20, 0x930, 0x93f, 0x915, 0x93e, 0x915, 0x94d, 0x930, 0x947, 0x938, 0x94d,
0x91f, 0x928, 0x915, 0x941, 0x908, 0x906, 0x92c, 0x93e, 0x915, 0x942, 0x930, 0x93e,
0x938, 0x93e, 0x935, 0x921, 0x93e, 0x928, 0x92e, 0x93e, 0x930, 0x94d, 0x915, 0x938,
0x94d, 0x92d, 0x93e, 0x928, 0x921, 0x93e, 0x935, 0x938, 0x2019, 0x928, 0x921, 0x93e,
0x935, 0x938, 0x2019, 0x928, 0x20, 0x915, 0x94d, 0x930, 0x940, 0x915, 0x921, 0x947,
0x928, 0x92d, 0x94b, 0x930, 0x921, 0x947, 0x91f, 0x94d, 0x930, 0x2019, 0x908, 0x924,
0x921, 0x92e, 0x93f, 0x928, 0x93f, 0x915, 0x93e, 0x90f, 0x921, 0x92e, 0x928, 0x94d,
0x91f, 0x928, 0x90f, 0x908, 0x930, 0x941, 0x928, 0x947, 0x92a, 0x947, 0x90f, 0x932,
0x20, 0x938, 0x93e, 0x932, 0x94d, 0x92d, 0x93e, 0x921, 0x930, 0x92b, 0x930, 0x94d,
0x924, 0x20, 0x928, 0x947, 0x932, 0x938, 0x928, 0x92b, 0x930, 0x94d, 0x91f, 0x93e,
0x932, 0x947, 0x91c, 0x93e, 0x917, 0x94d, 0x932, 0x947, 0x938, 0x20, 0x92c, 0x947,
0x907, 0x917, 0x942, 0x91c, 0x20, 0x92c, 0x947, 0x907, 0x917, 0x94d, 0x930, 0x947,
0x923, 0x94d, 0x921, 0x20, 0x924, 0x93e, 0x930, 0x94d, 0x915, 0x917, 0x94d, 0x930,
0x947, 0x928, 0x93e, 0x921, 0x93e, 0x917, 0x94d, 0x935, 0x93e, 0x926, 0x947, 0x932,
0x942, 0x92a, 0x917, 0x94d, 0x935, 0x93e, 0x91f, 0x947, 0x92e, 0x93e, 0x932, 0x93e,
0x917, 0x941, 0x906, 0x92f, 0x93e, 0x915, 0x93f, 0x932, 0x917, 0x941, 0x92f, 0x93e,
0x928, 0x93e, 0x939, 0x947, 0x932, 0x93f, 0x92b, 0x947, 0x915, 0x94d, 0x938, 0x939,
0x93e, 0x92d, 0x93e, 0x928, 0x93e, 0x939, 0x93e, 0x930, 0x94d, 0x92e, 0x2019, 0x938,
0x93f, 0x932, 0x94d, 0x932, 0x2019, 0x907, 0x923, 0x94d, 0x921, 0x93f, 0x92f, 0x93e,
0x928, 0x93e, 0x92a, 0x932, 0x93f, 0x938, 0x928, 0x915, 0x94d, 0x938, 0x2c, 0x20,
0x907, 0x923, 0x94d, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x92e, 0x93e, 0x930,
0x947, 0x919, 0x2019, 0x2c, 0x20, 0x907, 0x923, 0x94d, 0x921, 0x93f, 0x92f, 0x93e,
0x928, 0x93e, 0x92a, 0x93f, 0x924, 0x93e, 0x930, 0x94d, 0x938, 0x92c, 0x93e, 0x930,
0x94d, 0x917, 0x2c, 0x20, 0x907, 0x923, 0x94d, 0x921, 0x93f, 0x92f, 0x93e, 0x928,
0x93e, 0x924, 0x947, 0x932, 0x20, 0x928, 0x94b, 0x917, 0x94b, 0x930, 0x2c, 0x20,
0x907, 0x923, 0x94d, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x92d, 0x947, 0x92d,
0x948, 0x2c, 0x20, 0x907, 0x923, 0x94d, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e,
0x92d, 0x93e, 0x928, 0x938, 0x947, 0x928, 0x94d, 0x928, 0x947, 0x938, 0x2c, 0x20,
0x907, 0x923, 0x94d, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x935, 0x93f, 0x928,
0x93e, 0x92e, 0x947, 0x915, 0x2c, 0x20, 0x907, 0x923, 0x94d, 0x921, 0x93f, 0x92f,
0x93e, 0x928, 0x93e, 0x907, 0x928, 0x942, 0x935, 0x93f, 0x915, 0x907, 0x915, 0x93e,
0x932, 0x941, 0x908, 0x924, 0x91c, 0x93e, 0x92e, 0x93e, 0x908, 0x915, 0x93e, 0x91c,
0x941, 0x928, 0x94b, 0x932, 0x941, 0x908, 0x935, 0x940, 0x932, 0x92e, 0x928, 0x94d,
0x91f, 0x93f, 0x91b, 0x947, 0x932, 0x2019, 0x2c, 0x20, 0x915, 0x947, 0x928, 0x94d,
0x91f, 0x93e, 0x915, 0x940, 0x915, 0x94d, 0x930, 0x93e, 0x932, 0x947, 0x923, 0x91c,
0x940, 0x915, 0x932, 0x93e, 0x20, 0x92a, 0x93e, 0x91c, 0x932, 0x940, 0x92e, 0x93e,
0x932, 0x938, 0x20, 0x90f, 0x928, 0x94d, 0x91c, 0x947, 0x932, 0x94b, 0x938, 0x932,
0x935, 0x93e, 0x930, 0x20, 0x92a, 0x94d, 0x930, 0x93f, 0x928, 0x94d, 0x938, 0x94b,
0x938, 0x20, 0x915, 0x94d, 0x935, 0x93e, 0x930, 0x94d, 0x924, 0x93e, 0x930, 0x92e,
0x93e, 0x91a, 0x947, 0x908, 0x913, 0x92e, 0x928, 0x93e, 0x917, 0x941, 0x906, 0x92e,
0x928, 0x93e, 0x909, 0x938, 0x92e, 0x947, 0x930, 0x93f, 0x917, 0x924, 0x92e, 0x93e,
0x930, 0x94d, 0x91f, 0x940, 0x928, 0x940, 0x915, 0x92e, 0x947, 0x924, 0x93e, 0x92e,
0x930, 0x938, 0x92e, 0x93e, 0x91c, 0x93e, 0x91f, 0x932, 0x93e, 0x928, 0x92e, 0x947,
0x928, 0x2019, 0x92e, 0x940, 0x928, 0x940, 0x92e, 0x947, 0x930, 0x93f, 0x921, 0x93e,
0x92e, 0x947, 0x924, 0x94d, 0x932, 0x93e, 0x915, 0x93e, 0x924, 0x94d, 0x932, 0x93e,
0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x2019, 0x20, 0x938, 0x93f, 0x924,
0x940, 0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932, 0x2019, 0x928, 0x92e, 0x915,
0x94d, 0x91f, 0x2019, 0x928, 0x92e, 0x928, 0x94d, 0x91f, 0x947, 0x930, 0x94d, 0x930,
0x947, 0x92e, 0x928, 0x94d, 0x91f, 0x947, 0x92d, 0x93f, 0x921, 0x93f, 0x905, 0x92e,
0x928, 0x938, 0x947, 0x930, 0x94d, 0x930, 0x93e, 0x924, 0x928, 0x93e, 0x938, 0x94d,
0x938, 0x93e, 0x909, 0x928, 0x93f, 0x909, 0x20, 0x92f, 0x930, 0x94d, 0x915, 0x928,
0x930, 0x2019, 0x928, 0x94d, 0x939, 0x93e, 0x92c, 0x947, 0x90a, 0x932, 0x93e, 0x939,
0x2c, 0x20, 0x938, 0x93e, 0x939, 0x93e, 0x20, 0x921, 0x93e, 0x915, 0x2019, 0x924,
0x93e, 0x938, 0x947, 0x928, 0x94d, 0x91f, 0x94b, 0x930, 0x2c, 0x20, 0x938, 0x93e,
0x939, 0x93e, 0x20, 0x921, 0x93e, 0x915, 0x2019, 0x924, 0x93e, 0x928, 0x93f, 0x909,
0x20, 0x938, 0x93e, 0x932, 0x947, 0x92e, 0x2c, 0x20, 0x938, 0x93e, 0x939, 0x93e,
0x20, 0x921, 0x93e, 0x915, 0x2019, 0x924, 0x93e, 0x928, 0x942, 0x915, 0x906, 0x91c,
0x93f, 0x928, 0x93e, 0x917, 0x93e, 0x92a, 0x93e, 0x928, 0x93e, 0x92e, 0x93e, 0x92a,
0x93e, 0x930, 0x93e, 0x92e, 0x93e, 0x930, 0x93f, 0x92c, 0x2019, 0x92b, 0x940, 0x928,
0x93f, 0x915, 0x94d, 0x938, 0x92a, 0x930, 0x94d, 0x91f, 0x2d, 0x906, 0x90a, 0x2d,
0x92a, 0x94d, 0x930, 0x93f, 0x928, 0x94d, 0x938, 0x92a, 0x930, 0x94d, 0x91f, 0x20,
0x905, 0x92b, 0x20, 0x938, 0x94d, 0x92a, 0x948, 0x928, 0x92a, 0x930, 0x94d, 0x91f,
0x2019, 0x20, 0x92d, 0x947, 0x932, 0x94d, 0x939, 0x2019, 0x92a, 0x941, 0x90f, 0x930,
0x94d, 0x91f, 0x2019, 0x20, 0x930, 0x93f, 0x915, 0x2019, 0x92a, 0x941, 0x928, 0x94d,
0x91f, 0x93e, 0x20, 0x906, 0x930, 0x947, 0x928, 0x93e, 0x938, 0x930, 0x947, 0x928,
0x915, 0x93f, 0x928, 0x20, 0x907, 0x928, 0x932, 0x947, 0x91f, 0x930, 0x947, 0x938,
0x93f, 0x92b, 0x947, 0x930, 0x947, 0x91c, 0x93f, 0x928, 0x93e, 0x930, 0x947, 0x91c,
0x932, 0x94d, 0x92f, 0x941, 0x924, 0x930, 0x93f, 0x905, 0x20, 0x92c, 0x94d, 0x930,
0x93e, 0x928, 0x94d, 0x915, 0x2019, 0x938, 0x93e, 0x928, 0x94d, 0x924, 0x93e, 0x930,
0x947, 0x92e, 0x938, 0x93e, 0x928, 0x94d, 0x924, 0x93f, 0x906, 0x917, 0x2019, 0x938,
0x93e, 0x928, 0x94d, 0x924, 0x94b, 0x20, 0x921, 0x92e, 0x93f, 0x917, 0x2019, 0x938,
0x93e, 0x935, 0x20, 0x92a, 0x93e, 0x935, 0x932, 0x2019, 0x908, 0x924, 0x94d, 0x924,
0x2019, 0x915, 0x94d, 0x935, 0x930, 0x94d, 0x924, 0x942, 0x92e, 0x940, 0x938, 0x93f,
0x924, 0x94d, 0x915, 0x93e, 0x938, 0x948, 0x928, 0x94d, 0x91f, 0x20, 0x92c, 0x93e,
0x930, 0x94d, 0x925, 0x947, 0x932, 0x947, 0x92e, 0x940, 0x938, 0x948, 0x928, 0x94d,
0x91f, 0x20, 0x91c, 0x928, 0x94d, 0x938, 0x938, 0x948, 0x928, 0x94d, 0x91f, 0x20,
0x915, 0x940, 0x924, 0x94d, 0x938, 0x938, 0x948, 0x928, 0x94d, 0x91f, 0x20, 0x932,
0x941, 0x938, 0x93f, 0x92f, 0x93e, 0x938, 0x948, 0x928, 0x94d, 0x91f, 0x20, 0x925,
0x92e, 0x93e, 0x938, 0x938, 0x948, 0x928, 0x94d, 0x91f, 0x20, 0x92d, 0x93f, 0x928,
0x94d, 0x938, 0x947, 0x928, 0x94d, 0x91f, 0x938, 0x941, 0x908, 0x92b, 0x924, 0x20,
0x915, 0x93e, 0x930, 0x94d, 0x930, 0x947, 0x928, 0x94d, 0x924, 0x924, 0x947, 0x917,
0x941, 0x938, 0x93f, 0x917, 0x93e, 0x932, 0x94d, 0x92a, 0x93e, 0x925, 0x94d, 0x92f,
0x942, 0x932, 0x91f, 0x93f, 0x939, 0x941, 0x906, 0x928, 0x93e, 0x91f, 0x930, 0x928,
0x94d, 0x91f, 0x2019, 0x91f, 0x930, 0x94d, 0x924, 0x2019, 0x932, 0x93e, 0x92d, 0x947,
0x928, 0x915, 0x942, 0x92d, 0x930, 0x935, 0x94d, 0x939, 0x93e, 0x908, 0x924, 0x939,
0x930, 0x94d, 0x938, 0x935, 0x940, 0x928, 0x940, 0x92a, 0x947, 0x917, 0x92f, 0x93e,
0x915, 0x941, 0x924, 0x93e, 0x924, 0x915, 0x947, 0x938, 0x940, 0x921, 0x947, 0x92d,
0x93f, 0x938, 0x921, 0x941, 0x92e, 0x928, 0x94d, 0x924, 0x2d, 0x921, 0x940, 0x906,
0x930, 0x94d, 0x92d, 0x940, 0x932, 0x92e, 0x93e, 0x915, 0x94d, 0x935, 0x93e, 0x930,
0x940, 0x92e, 0x94c, 0x938, 0x928, 0x92e, 0x947, 0x915, 0x92e, 0x93e, 0x930, 0x94d,
0x921, 0x2019, 0x92a, 0x93e, 0x932, 0x92e, 0x93e, 0x930, 0x930, 0x925, 0x947, 0x930,
0x93e, 0x938, 0x940, 0x905, 0x935, 0x93e, 0x924, 0x94d, 0x930, 0x2019, 0x932, 0x92d,
0x938, 0x94d, 0x924, 0x2019, 0x915, 0x932, 0x902, 0x92f, 0x93e, 0x930, 0x92c, 0x94d,
0x92f, 0x947, 0x928, 0x90f, 0x921, 0x947, 0x928, 0x906, 0x932, 0x94d, 0x92e, 0x93e,
0x91f, 0x940, 0x906, 0x92e, 0x94d, 0x92e, 0x93e, 0x928, 0x906, 0x928, 0x93e, 0x921,
0x940, 0x930, 0x905, 0x915, 0x94d, 0x924, 0x93e, 0x909, 0x906, 0x915, 0x94d, 0x924,
0x2019, 0x92c, 0x947, 0x905, 0x936, 0x94d, 0x917, 0x93e, 0x92c, 0x93e, 0x924, 0x906,
0x924, 0x940, 0x930, 0x93e, 0x909, 0x92c, 0x93e, 0x918, 0x926, 0x93e, 0x926, 0x92c,
0x93e, 0x939, 0x930, 0x948, 0x928, 0x92c, 0x948, 0x902, 0x917, 0x915, 0x949, 0x915,
0x92c, 0x93e, 0x930, 0x94d, 0x928, 0x94c, 0x932, 0x92c, 0x948, 0x930, 0x941, 0x924,
0x92c, 0x93f, 0x936, 0x94d, 0x915, 0x947, 0x915, 0x92c, 0x94d, 0x930, 0x941, 0x928,
0x907, 0x91a, 0x940, 0x924, 0x93e, 0x915, 0x932, 0x92e, 0x94d, 0x92c, 0x2019, 0x926,
0x93e, 0x92e, 0x93e, 0x938, 0x94d, 0x915, 0x93e, 0x938, 0x927, 0x93e, 0x915, 0x93e,
0x921, 0x93f, 0x932, 0x940, 0x926, 0x941, 0x92c, 0x93e, 0x908, 0x926, 0x941, 0x936,
0x93e, 0x928, 0x94d, 0x92c, 0x947, 0x92b, 0x93e, 0x92e, 0x93e, 0x917, 0x941, 0x938,
0x94d, 0x924, 0x93e, 0x917, 0x93e, 0x91c, 0x93e, 0x939, 0x947, 0x92c, 0x94d, 0x930,
0x2019, 0x928, 0x938, 0x93e, 0x907, 0x917, 0x949, 0x928, 0x939, 0x902, 0x915, 0x902,
0x939, 0x2019, 0x92d, 0x94d, 0x926, 0x908, 0x930, 0x94d, 0x915, 0x941, 0x924, 0x94d,
0x938, 0x94d, 0x915, 0x91c, 0x93e, 0x915, 0x93e, 0x930, 0x94d, 0x924, 0x93e, 0x91c,
0x92f, 0x93e, 0x92a, 0x941, 0x930, 0x93e, 0x91c, 0x947, 0x930, 0x941, 0x91c, 0x93e,
0x932, 0x947, 0x92e, 0x915, 0x93e, 0x92c, 0x942, 0x932, 0x915, 0x93e, 0x92e, 0x91a,
0x93e, 0x91f, 0x915, 0x93e, 0x915, 0x930, 0x93e, 0x91a, 0x940, 0x915, 0x93e, 0x920,
0x92e, 0x93e, 0x928, 0x94d, 0x921, 0x941, 0x916, 0x93e, 0x928, 0x94d, 0x926, 0x940,
0x917, 0x93e, 0x915, 0x932, 0x915, 0x93e, 0x924, 0x93e, 0x915, 0x94d, 0x930, 0x93e,
0x938, 0x94d, 0x928, 0x2019, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x915,
0x94d, 0x935, 0x93e, 0x932, 0x93e, 0x20, 0x932, 0x92e, 0x94d, 0x92a, 0x941, 0x930,
0x915, 0x942, 0x91a, 0x93f, 0x902, 0x917, 0x915, 0x941, 0x935, 0x948, 0x924, 0x92e,
0x93e, 0x915, 0x93e, 0x90a, 0x92e, 0x93e, 0x917, 0x93e, 0x926, 0x93e, 0x928, 0x92e,
0x93e, 0x915, 0x93e, 0x938, 0x94d, 0x938, 0x93e, 0x930, 0x92e, 0x928, 0x940, 0x932,
0x93e, 0x92e, 0x93e, 0x938, 0x94d, 0x915, 0x947, 0x91f, 0x928, 0x93f, 0x915, 0x2019,
0x938, 0x93f, 0x92f, 0x93e, 0x928, 0x92d, 0x2019, 0x915, 0x941, 0x91c, 0x94d, 0x928,
0x947, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x928, 0x92d, 0x2019, 0x938, 0x93f, 0x92c,
0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x905, 0x92e, 0x94d, 0x938, 0x94d, 0x915,
0x905, 0x930, 0x93e, 0x932, 0x928, 0x949, 0x92e, 0x20, 0x92a, 0x947, 0x928, 0x92a,
0x94b, 0x902, 0x924, 0x93f, 0x906, 0x928, 0x93e, 0x915, 0x92a, 0x94d, 0x92f, 0x902,
0x92f, 0x93e, 0x902, 0x915, 0x93e, 0x924, 0x93e, 0x930, 0x915, 0x938, 0x94d, 0x924,
0x93e, 0x928, 0x948, 0x915, 0x940, 0x91c, 0x940, 0x932, 0x930, 0x94d, 0x921, 0x93e,
0x930, 0x93f, 0x92f, 0x93e, 0x927, 0x938, 0x93e, 0x916, 0x93e, 0x932, 0x93f, 0x928,
0x938, 0x92e, 0x930, 0x915, 0x93e, 0x928, 0x94d, 0x926, 0x938, 0x2019, 0x909, 0x932,
0x936, 0x93e, 0x902, 0x918, 0x93e, 0x908, 0x938, 0x93f, 0x902, 0x917, 0x93e, 0x92a,
0x941, 0x930, 0x938, 0x94d, 0x930, 0x947, 0x926, 0x928, 0x947, 0x915, 0x2019, 0x932,
0x93f, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x924, 0x93e, 0x908, 0x92a, 0x947, 0x907,
0x924, 0x93e, 0x936, 0x915, 0x947, 0x928, 0x94d, 0x924, 0x924, 0x93f, 0x92c, 0x93f,
0x932, 0x93f, 0x938, 0x940, 0x924, 0x947, 0x939, 0x930, 0x93e, 0x928, 0x925, 0x93f,
0x92e, 0x94d, 0x92b, 0x941, 0x91f, 0x2019, 0x915, 0x94d, 0x92f, 0x2019, 0x924, 0x92e,
0x94d, 0x938, 0x94d, 0x915, 0x909, 0x932, 0x93e, 0x928, 0x92c, 0x93e, 0x924, 0x93e,
0x930, 0x909, 0x930, 0x941, 0x92e, 0x915, 0x940, 0x906, 0x938, 0x94d, 0x924, 0x2d,
0x928, 0x947, 0x930, 0x93e, 0x935, 0x93f, 0x92f, 0x902, 0x938, 0x93f, 0x92f, 0x93e,
0x928, 0x92d, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x92d, 0x938, 0x94d, 0x924, 0x2019,
0x915, 0x92f, 0x93e, 0x915, 0x942, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x930, 0x902,
0x917, 0x942, 0x928, 0x92f, 0x947, 0x915, 0x93e, 0x924, 0x947, 0x930, 0x940, 0x928,
0x93e, 0x92c, 0x941, 0x930, 0x94d, 0x917, 0x92f, 0x947, 0x930, 0x947, 0x92d, 0x93e,
0x928, 0x906, 0x91c, 0x2019, 0x930, 0x94d, 0x938, 0x92c, 0x93e, 0x930, 0x94d, 0x92e,
0x941, 0x921, 0x93e, 0x915, 0x947, 0x928, 0x947, 0x930, 0x940, 0x915, 0x947, 0x92a,
0x20, 0x92d, 0x947, 0x930, 0x94d, 0x926, 0x947, 0x92b, 0x947, 0x930, 0x94c, 0x92e,
0x93e, 0x921, 0x948, 0x930, 0x93e, 0x930, 0x940, 0x915, 0x92f, 0x93e, 0x92d, 0x93f,
0x915, 0x916, 0x94b, 0x932, 0x93e, 0x20, 0x91c, 0x930, 0x94d, 0x91c, 0x93f, 0x92f,
0x93e, 0x938, 0x948, 0x928, 0x94d, 0x91f, 0x20, 0x939, 0x947, 0x932, 0x947, 0x928,
0x93e, 0x938, 0x94d, 0x91f, 0x947, 0x928, 0x932, 0x940, 0x90f, 0x921, 0x947, 0x932,
0x948, 0x921, 0x92c, 0x94d, 0x930, 0x940, 0x938, 0x92c, 0x947, 0x928, 0x92c, 0x94d,
0x930, 0x2019, 0x915, 0x947, 0x928, 0x20, 0x939, 0x93f, 0x932, 0x921, 0x93e, 0x930,
0x94d, 0x935, 0x940, 0x928, 0x92f, 0x942, 0x915, 0x94d, 0x932, 0x93e, 0x939, 0x92c,
0x93e, 0x930, 0x94d, 0x91f, 0x932, 0x93f, 0x923, 0x94d, 0x921, 0x92e, 0x947, 0x928,
0x932, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x93e, 0x935, 0x93f, 0x92e, 0x947, 0x932,
0x92c, 0x94c, 0x930, 0x94d, 0x928, 0x92a, 0x94b, 0x930, 0x94d, 0x925, 0x938, 0x93f,
0x921, 0x928, 0x940, 0x910, 0x92e, 0x94d, 0x938, 0x94d, 0x91f, 0x930, 0x921, 0x93e,
0x92e, 0x906, 0x928, 0x94d, 0x921, 0x2019, 0x930, 0x93e, 0x906, 0x938, 0x94d, 0x924,
0x94d, 0x930, 0x93e, 0x916, 0x93e, 0x928, 0x90f, 0x925, 0x947, 0x928, 0x94d, 0x938,
0x92c, 0x947, 0x932, 0x917, 0x94d, 0x930, 0x947, 0x921, 0x92c, 0x93e, 0x930, 0x94d,
0x932, 0x93f, 0x928, 0x92c, 0x94d, 0x930, 0x93e, 0x924, 0x93f, 0x938, 0x94d, 0x932,
0x93e, 0x92d, 0x93e, 0x92c, 0x94d, 0x930, 0x941, 0x938, 0x94d, 0x938, 0x947, 0x932,
0x94d, 0x938, 0x92c, 0x941, 0x915, 0x93e, 0x930, 0x947, 0x938, 0x94d, 0x91f, 0x92c,
0x941, 0x926, 0x93e, 0x92a, 0x947, 0x938, 0x94d, 0x91f, 0x92c, 0x941, 0x938, 0x93f,
0x902, 0x917, 0x947, 0x928, 0x91a, 0x93f, 0x938, 0x940, 0x928, 0x94c, 0x915, 0x92a,
0x947, 0x928, 0x939, 0x947, 0x917, 0x947, 0x928, 0x921, 0x93e, 0x92c, 0x932, 0x93f,
0x928, 0x91c, 0x93f, 0x92c, 0x94d, 0x930, 0x93e, 0x932, 0x94d, 0x91f, 0x93e, 0x930,
0x917, 0x93e, 0x930, 0x94d, 0x928, 0x938, 0x940, 0x939, 0x947, 0x932, 0x94d, 0x938,
0x93f, 0x902, 0x915, 0x940, 0x906, 0x92f, 0x94b, 0x932, 0x20, 0x905, 0x92b, 0x20,
0x92e, 0x947, 0x928, 0x907, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x92c, 0x941, 0x932,
0x91c, 0x93e, 0x930, 0x94d, 0x938, 0x940, 0x915, 0x93e, 0x932, 0x93f, 0x928, 0x940,
0x902, 0x917, 0x94d, 0x930, 0x93e, 0x926, 0x915, 0x93f, 0x930, 0x2019, 0x92d, 0x915,
0x940, 0x92d, 0x932, 0x93f, 0x938, 0x94d, 0x92c, 0x928, 0x91c, 0x941, 0x92c, 0x94d,
0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x93e, 0x932, 0x928, 0x94d, 0x926, 0x928, 0x932,
0x93e, 0x915, 0x94d, 0x91c, 0x947, 0x92e, 0x94d, 0x92c, 0x93e, 0x930, 0x94d, 0x917,
0x92e, 0x93e, 0x926, 0x94d, 0x930, 0x93f, 0x926, 0x92e, 0x93e, 0x932, 0x94d, 0x91f,
0x93e, 0x92e, 0x93e, 0x930, 0x948, 0x939, 0x93e, 0x92e, 0x92e, 0x93f, 0x928, 0x94d,
0x938, 0x94d, 0x915, 0x92e, 0x928, 0x93e, 0x915, 0x2019, 0x92e, 0x938, 0x94d, 0x915,
0x94c, 0x905, 0x938, 0x94d, 0x932, 0x2019, 0x92a, 0x947, 0x930, 0x93f, 0x938, 0x92a,
0x926, 0x917, 0x2019, 0x930, 0x93f, 0x915, 0x93e, 0x92a, 0x94d, 0x930, 0x947, 0x917,
0x930, 0x93f, 0x917, 0x93e, 0x930, 0x2019, 0x92e, 0x938, 0x93e, 0x92e, 0x93e, 0x930,
0x93e, 0x938, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x930, 0x93f, 0x928, 0x2019, 0x938,
0x93e, 0x930, 0x93e, 0x91c, 0x947, 0x92d, 0x2019, 0x938, 0x93e, 0x930, 0x93e, 0x924,
0x2019, 0x92d, 0x938, 0x93f, 0x92e, 0x94d, 0x92b, 0x947, 0x930, 0x2019, 0x92a, 0x2019,
0x932, 0x938, 0x94d, 0x915, 0x2019, 0x92a, 0x94d, 0x91c, 0x947, 0x938, 0x92b, 0x93f,
0x92f, 0x93e, 0x938, 0x94d, 0x91f, 0x2019, 0x915, 0x939, 0x2019, 0x92e, 0x91f, 0x947,
0x932, 0x93f, 0x928, 0x91f, 0x93f, 0x930, 0x947, 0x928, 0x90a, 0x932, 0x94d, 0x92f,
0x93e, 0x928, 0x2019, 0x92d, 0x94d, 0x938, 0x94d, 0x915, 0x92d, 0x93e, 0x921, 0x941,
0x91c, 0x92d, 0x947, 0x91f, 0x93f, 0x915, 0x93e, 0x928, 0x92d, 0x93f, 0x92f, 0x947,
0x928, 0x94d, 0x928, 0x93e, 0x935, 0x93f, 0x932, 0x94d, 0x928, 0x93f, 0x909, 0x938,
0x92d, 0x932, 0x94d, 0x917, 0x2019, 0x917, 0x94d, 0x930, 0x947, 0x926, 0x935, 0x93e,
0x930, 0x938, 0x94c, 0x91c, 0x93e, 0x917, 0x94d, 0x930, 0x947, 0x92c, 0x91c, 0x941,
0x930, 0x93f, 0x916, 0x90f, 0x928, 0x94d, 0x91f, 0x93e, 0x928, 0x93e, 0x928, 0x93e,
0x930, 0x93f, 0x92d, 0x2019, 0x91a, 0x93e, 0x917, 0x2019, 0x938, 0x916, 0x94d, 0x930,
0x940, 0x938, 0x94d, 0x91f, 0x92e, 0x93e, 0x938, 0x20, 0x926, 0x94d, 0x935, 0x940,
0x92a, 0x915, 0x915, 0x2019, 0x938, 0x915, 0x92e, 0x2019, 0x930, 0x2019, 0x915, 0x947,
0x930, 0x94d, 0x917, 0x947, 0x932, 0x947, 0x928, 0x92e, 0x93e, 0x939, 0x947, 0x92e,
0x93e, 0x932, 0x926, 0x940, 0x92d, 0x94d, 0x938, 0x92e, 0x930, 0x93f, 0x936, 0x93f,
0x92f, 0x93e, 0x938, 0x92e, 0x93e, 0x92f, 0x2019, 0x924, 0x94d, 0x924, 0x947, 0x930,
0x947, 0x92f, 0x942, 0x928, 0x93f, 0x92f, 0x2019, 0x928, 0x906, 0x92a, 0x93f, 0x92f,
0x93e, 0x914, 0x915, 0x932, 0x948, 0x923, 0x94d, 0x921, 0x92c, 0x941, 0x917, 0x948,
0x928, 0x92d, 0x940, 0x932, 0x91a, 0x93e, 0x925, 0x93e, 0x92e, 0x91a, 0x942, 0x915,
0x938, 0x93e, 0x928, 0x91c, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x90f, 0x92b, 0x93e,
0x91f, 0x947, 0x92b, 0x93e, 0x915, 0x93e, 0x905, 0x92b, 0x2019, 0x92b, 0x93f, 0x91c,
0x940, 0x92b, 0x941, 0x928, 0x93e, 0x92b, 0x942, 0x91f, 0x940, 0x917, 0x93e, 0x932,
0x93e, 0x92a, 0x93e, 0x917, 0x2019, 0x938, 0x917, 0x93e, 0x92e, 0x94d, 0x92c, 0x93f,
0x92f, 0x947, 0x930, 0x917, 0x94d, 0x935, 0x93e, 0x921, 0x93e, 0x932, 0x915, 0x947,
0x928, 0x947, 0x932, 0x917, 0x941, 0x935, 0x93e, 0x92e, 0x939, 0x928, 0x932, 0x941,
0x932, 0x941, 0x90f, 0x928, 0x94d, 0x921, 0x93e, 0x930, 0x92c, 0x93e, 0x930, 0x940,
0x915, 0x93f, 0x930, 0x940, 0x924, 0x93f, 0x92e, 0x93e, 0x924, 0x93f, 0x915, 0x938,
0x94d, 0x930, 0x947, 0x915, 0x94d, 0x935, 0x93e, 0x91c, 0x93e, 0x932, 0x948, 0x928,
0x92e, 0x93e, 0x91c, 0x942, 0x930, 0x2019, 0x92e, 0x93e, 0x930, 0x94d, 0x915, 0x947,
0x938, 0x93e, 0x938, 0x92e, 0x940, 0x921, 0x935, 0x947, 0x928, 0x93e, 0x90a, 0x930,
0x941, 0x928, 0x940, 0x90a, 0x90f, 0x928, 0x930, 0x92b, 0x2019, 0x915, 0x928, 0x942,
0x92e, 0x93f, 0x92f, 0x93e, 0x92a, 0x93e, 0x917, 0x2019, 0x20, 0x92a, 0x93e, 0x917,
0x2019, 0x92a, 0x93e, 0x932, 0x93e, 0x909, 0x92a, 0x93f, 0x91f, 0x915, 0x948, 0x930,
0x94d, 0x928, 0x92a, 0x928, 0x93e, 0x92a, 0x947, 0x92a, 0x930, 0x94d, 0x91f, 0x20,
0x92e, 0x930, 0x94d, 0x938, 0x92c, 0x940, 0x930, 0x93e, 0x930, 0x2019, 0x91f, 0x902,
0x917, 0x93e, 0x938, 0x93e, 0x907, 0x92a, 0x93e, 0x928, 0x91f, 0x93e, 0x939, 0x93f,
0x91f, 0x940, 0x924, 0x93e, 0x930, 0x93e, 0x935, 0x93e, 0x91f, 0x902, 0x917, 0x93e,
0x91f, 0x93e, 0x92a, 0x941, 0x935, 0x947, 0x915, 0x935, 0x93e, 0x932, 0x93f, 0x938,
0x4b, 0x61, 0x7a, 0x61, 0x62, 0x6c, 0x61, 0x6e, 0x6b, 0x61, 0x44, 0x17e,
0x69, 0x62, 0x75, 0x74, 0x69, 0x4d, 0x6f, 0x67, 0x61, 0x64, 0x69, 0x161,
0x41, 0x6e, 0x67, 0x76, 0x69, 0x6c, 0x61, 0x41, 0x6e, 0x74, 0x69, 0x67,
0x76, 0x61, 0x43, 0x61, 0x6e, 0x63, 0x75, 0x6e, 0x4b, 0x6f, 0x73, 0x74,
0x61, 0x72, 0x69, 0x6b, 0x61, 0x47, 0x76, 0x61, 0x64, 0x61, 0x6c, 0x75,
0x70, 0x65, 0x47, 0x76, 0x61, 0x74, 0x65, 0x6d, 0x61, 0x6c, 0x61, 0x4a,
0x61, 0x6d, 0x61, 0x6a, 0x6b, 0x61, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68,
0x2c, 0x20, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x61, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c,
0x20, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x61, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65,
0x6d, 0x2c, 0x20, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x61, 0x20,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x72,
0x69, 0x6b, 0x6f, 0x41, 0x6b, 0x75, 0x74, 0x6f, 0x62, 0x65, 0x41, 0x161,
0x68, 0x61, 0x62, 0x61, 0x64, 0x41, 0x74, 0x69, 0x72, 0x61, 0x6a, 0x42,
0x69, 0x161, 0x6b, 0x65, 0x6b, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x6a, 0x69,
0x44, 0x75, 0x161, 0x61, 0x6e, 0x62, 0x65, 0x48, 0x6f, 0x20, 0x160, 0x69,
0x20, 0x4d, 0x69, 0x6e, 0x44, 0x17e, 0x61, 0x6b, 0x61, 0x72, 0x74, 0x61,
0x44, 0x17e, 0x61, 0x6a, 0x61, 0x70, 0x75, 0x72, 0x61, 0x4a, 0x65, 0x72,
0x75, 0x7a, 0x61, 0x6c, 0x65, 0x6d, 0x4b, 0x61, 0x6d, 0x10d, 0x61, 0x74,
0x6b, 0x61, 0x4b, 0x61, 0x72, 0x61, 0x10d, 0x69, 0x48, 0x61, 0x6e, 0x64,
0x69, 0x67, 0x61, 0x4b, 0x75, 0x10d, 0x69, 0x6e, 0x67, 0x4d, 0x61, 0x6b,
0x61, 0x75, 0x4e, 0x69, 0x6b, 0x6f, 0x7a, 0x69, 0x6a, 0x61, 0x4e, 0x6f,
0x76, 0x6f, 0x6b, 0x75, 0x7a, 0x6e, 0x6a, 0x65, 0x63, 0x6b, 0x50, 0x6a,
0x6f, 0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x67, 0x52, 0x69, 0x6a, 0x61, 0x64,
0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x160, 0x61, 0x6e, 0x67, 0x61,
0x6a, 0x54, 0x61, 0x161, 0x6b, 0x65, 0x6e, 0x74, 0x55, 0x72, 0x75, 0x6d,
0x10d, 0x69, 0x56, 0x69, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6a, 0x61, 0x6e,
0x41, 0x7a, 0x6f, 0x72, 0x69, 0x52, 0x65, 0x6a, 0x6b, 0x6a, 0x61, 0x76,
0x69, 0x6b, 0x53, 0x76, 0x65, 0x74, 0x61, 0x20, 0x48, 0x65, 0x6c, 0x65,
0x6e, 0x61, 0x41, 0x6e, 0x64, 0x6f, 0x72, 0x61, 0x41, 0x73, 0x74, 0x72,
0x61, 0x68, 0x61, 0x6e, 0x41, 0x74, 0x69, 0x6e, 0x61, 0x42, 0x72, 0x69,
0x73, 0x65, 0x6c, 0x42, 0x75, 0x6b, 0x75, 0x72, 0x65, 0x161, 0x74, 0x42,
0x75, 0x64, 0x69, 0x6d, 0x70, 0x65, 0x161, 0x74, 0x61, 0x4b, 0x69, 0x161,
0x69, 0x6e, 0x6a, 0x65, 0x76, 0x44, 0x61, 0x62, 0x6c, 0x69, 0x6e, 0x47,
0x65, 0x72, 0x6e, 0x7a, 0x69, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x6f, 0x20,
0x4d, 0x61, 0x6e, 0x4b, 0x61, 0x6c, 0x69, 0x6e, 0x6a, 0x69, 0x6e, 0x67,
0x72, 0x61, 0x64, 0x4b, 0x69, 0x6a, 0x65, 0x76, 0x4c, 0x69, 0x73, 0x61,
0x62, 0x6f, 0x6e, 0x50, 0x61, 0x72, 0x69, 0x7a, 0x52, 0x69, 0x6d, 0x53,
0x69, 0x6d, 0x66, 0x65, 0x72, 0x6f, 0x70, 0x6f, 0x6c, 0x6a, 0x53, 0x6b,
0x6f, 0x70, 0x6c, 0x6a, 0x65, 0x53, 0x6f, 0x66, 0x69, 0x6a, 0x61, 0x160,
0x74, 0x6f, 0x6b, 0x68, 0x6f, 0x6c, 0x6d, 0x42, 0x65, 0x10d, 0x56, 0x61,
0x72, 0x161, 0x61, 0x76, 0x61, 0x43, 0x69, 0x72, 0x69, 0x68, 0x42, 0x6f,
0x17e, 0x69, 0x107, 0x6e, 0x6f, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x6f,
0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x6f, 0x76, 0x61, 0x20, 0x6f, 0x73, 0x74,
0x72, 0x76, 0x61, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x69, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x63, 0x69, 0x6a, 0x75, 0x73, 0x46, 0x69, 0x64, 0x17e,
0x69, 0x56, 0x61, 0x6c, 0x69, 0x73, 0x410, 0x434, 0x438, 0x441, 0x20, 0x410,
0x431, 0x435, 0x431, 0x430, 0x411, 0x430, 0x43d, 0x433, 0x443, 0x438, 0x411, 0x438,
0x441, 0x430, 0x43e, 0x411, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x440, 0x411, 0x440,
0x430, 0x437, 0x430, 0x432, 0x438, 0x43b, 0x411, 0x443, 0x45f, 0x443, 0x43c, 0x431,
0x443, 0x440, 0x430, 0x41a, 0x430, 0x438, 0x440, 0x43e, 0x41a, 0x430, 0x437, 0x430,
0x431, 0x43b, 0x430, 0x43d, 0x43a, 0x430, 0x415, 0x43b, 0x20, 0x410, 0x458, 0x443,
0x43d, 0x425, 0x430, 0x440, 0x430, 0x440, 0x435, 0x408, 0x43e, 0x445, 0x430, 0x43d,
0x435, 0x441, 0x431, 0x443, 0x440, 0x433, 0x41a, 0x430, 0x440, 0x442, 0x443, 0x43c,
0x41b, 0x443, 0x43c, 0x443, 0x43c, 0x431, 0x430, 0x448, 0x438, 0x41c, 0x430, 0x43f,
0x443, 0x442, 0x43e, 0x41c, 0x43e, 0x43d, 0x440, 0x43e, 0x432, 0x438, 0x458, 0x430,
0x41d, 0x430, 0x458, 0x440, 0x43e, 0x431, 0x438, 0x41d, 0x438, 0x458, 0x430, 0x43c,
0x435, 0x458, 0x41f, 0x43e, 0x440, 0x442, 0x43e, 0x20, 0x41d, 0x43e, 0x432, 0x43e,
0x421, 0x430, 0x43e, 0x20, 0x422, 0x43e, 0x43c, 0x435, 0x415, 0x43d, 0x43a, 0x43e,
0x440, 0x438, 0x45f, 0x410, 0x43d, 0x433, 0x432, 0x438, 0x43b, 0x430, 0x410, 0x43d,
0x442, 0x438, 0x433, 0x432, 0x430, 0x410, 0x440, 0x430, 0x433, 0x432, 0x430, 0x458,
0x430, 0x43d, 0x430, 0x411, 0x443, 0x435, 0x43d, 0x43e, 0x441, 0x20, 0x410, 0x438,
0x440, 0x435, 0x441, 0x41a, 0x43e, 0x440, 0x434, 0x43e, 0x431, 0x430, 0x425, 0x443,
0x445, 0x443, 0x458, 0x41b, 0x430, 0x20, 0x420, 0x438, 0x43e, 0x445, 0x430, 0x41c,
0x435, 0x43d, 0x434, 0x43e, 0x437, 0x430, 0x420, 0x438, 0x43e, 0x20, 0x413, 0x430,
0x459, 0x435, 0x433, 0x43e, 0x441, 0x421, 0x430, 0x43d, 0x20, 0x425, 0x443, 0x430,
0x43d, 0x421, 0x430, 0x43d, 0x20, 0x41b, 0x443, 0x438, 0x423, 0x448, 0x443, 0x430,
0x438, 0x458, 0x430, 0x410, 0x441, 0x443, 0x43d, 0x441, 0x438, 0x43e, 0x43d, 0x410,
0x442, 0x438, 0x43a, 0x43e, 0x43a, 0x430, 0x43d, 0x411, 0x430, 0x438, 0x458, 0x430,
0x411, 0x430, 0x438, 0x458, 0x430, 0x20, 0x411, 0x430, 0x43d, 0x434, 0x435, 0x440,
0x430, 0x441, 0x411, 0x435, 0x43b, 0x435, 0x43c, 0x411, 0x435, 0x43b, 0x438, 0x437,
0x435, 0x411, 0x43b, 0x430, 0x43d, 0x43a, 0x2d, 0x421, 0x435, 0x458, 0x431, 0x43b,
0x43e, 0x43d, 0x411, 0x43e, 0x430, 0x20, 0x412, 0x438, 0x441, 0x442, 0x430, 0x411,
0x43e, 0x458, 0x437, 0x438, 0x41a, 0x435, 0x43c, 0x431, 0x440, 0x438, 0x45f, 0x20,
0x411, 0x435, 0x458, 0x41a, 0x430, 0x43c, 0x43f, 0x43e, 0x20, 0x413, 0x440, 0x430,
0x43d, 0x434, 0x435, 0x41a, 0x430, 0x458, 0x435, 0x43d, 0x41a, 0x430, 0x458, 0x43c,
0x430, 0x43d, 0x441, 0x43a, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430,
0x427, 0x438, 0x432, 0x430, 0x432, 0x430, 0x41a, 0x43e, 0x441, 0x442, 0x430, 0x440,
0x438, 0x43a, 0x430, 0x41a, 0x443, 0x440, 0x430, 0x441, 0x430, 0x43e, 0x414, 0x430,
0x43d, 0x43c, 0x430, 0x440, 0x43a, 0x441, 0x445, 0x430, 0x433, 0x435, 0x43d, 0x414,
0x43e, 0x441, 0x43e, 0x43d, 0x414, 0x43e, 0x441, 0x43e, 0x43d, 0x20, 0x41a, 0x440,
0x438, 0x43a, 0x413, 0x43b, 0x435, 0x458, 0x441, 0x20, 0x411, 0x435, 0x458, 0x413,
0x443, 0x441, 0x20, 0x411, 0x435, 0x458, 0x413, 0x440, 0x430, 0x43d, 0x434, 0x20,
0x422, 0x443, 0x440, 0x43a, 0x413, 0x432, 0x430, 0x434, 0x430, 0x43b, 0x443, 0x43f,
0x435, 0x413, 0x432, 0x430, 0x458, 0x430, 0x43a, 0x438, 0x43b, 0x413, 0x443, 0x430,
0x43d, 0x430, 0x425, 0x430, 0x43b, 0x438, 0x444, 0x430, 0x43a, 0x441, 0x425, 0x430,
0x432, 0x430, 0x43d, 0x430, 0x425, 0x435, 0x440, 0x43c, 0x43e, 0x441, 0x438, 0x459,
0x43e, 0x41a, 0x43d, 0x43e, 0x43a, 0x441, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438,
0x458, 0x430, 0x43d, 0x430, 0x41c, 0x430, 0x440, 0x435, 0x43d, 0x433, 0x43e, 0x2c,
0x20, 0x418, 0x43d, 0x434, 0x438, 0x458, 0x430, 0x43d, 0x430, 0x41f, 0x435, 0x442,
0x435, 0x440, 0x441, 0x431, 0x443, 0x440, 0x433, 0x2c, 0x20, 0x418, 0x43d, 0x434,
0x438, 0x458, 0x430, 0x43d, 0x430, 0x422, 0x435, 0x43b, 0x20, 0x421, 0x438, 0x442,
0x438, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x458, 0x430, 0x43d, 0x430, 0x412,
0x435, 0x432, 0x430, 0x458, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x458, 0x430,
0x43d, 0x430, 0x412, 0x438, 0x43d, 0x446, 0x435, 0x43d, 0x435, 0x441, 0x2c, 0x20,
0x418, 0x43d, 0x434, 0x438, 0x458, 0x430, 0x43d, 0x430, 0x412, 0x438, 0x43d, 0x430,
0x43c, 0x430, 0x43a, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x458, 0x430, 0x43d,
0x430, 0x418, 0x43a, 0x432, 0x430, 0x43b, 0x443, 0x438, 0x442, 0x408, 0x430, 0x43c,
0x430, 0x458, 0x43a, 0x430, 0x416, 0x443, 0x43d, 0x43e, 0x41c, 0x43e, 0x43d, 0x442,
0x438, 0x447, 0x435, 0x43b, 0x43e, 0x2c, 0x20, 0x41a, 0x435, 0x43d, 0x442, 0x430,
0x43a, 0x438, 0x41a, 0x440, 0x435, 0x43b, 0x435, 0x43d, 0x434, 0x438, 0x458, 0x43a,
0x41b, 0x430, 0x20, 0x41f, 0x430, 0x437, 0x41b, 0x43e, 0x441, 0x20, 0x410, 0x43d,
0x452, 0x435, 0x43b, 0x435, 0x441, 0x41b, 0x43e, 0x443, 0x435, 0x440, 0x20, 0x41f,
0x440, 0x438, 0x43d, 0x441, 0x20, 0x41a, 0x432, 0x43e, 0x440, 0x442, 0x435, 0x440,
0x41c, 0x430, 0x441, 0x435, 0x458, 0x43e, 0x41c, 0x430, 0x43d, 0x430, 0x433, 0x432,
0x430, 0x41c, 0x430, 0x437, 0x430, 0x442, 0x43b, 0x430, 0x43d, 0x41c, 0x435, 0x43a,
0x441, 0x438, 0x43a, 0x43e, 0x20, 0x421, 0x438, 0x442, 0x438, 0x41c, 0x43e, 0x43d,
0x442, 0x435, 0x440, 0x435, 0x458, 0x41c, 0x43e, 0x43d, 0x442, 0x441, 0x435, 0x440,
0x430, 0x442, 0x41d, 0x430, 0x441, 0x430, 0x443, 0x40a, 0x443, 0x458, 0x43e, 0x440,
0x43a, 0x41d, 0x43e, 0x440, 0x43e, 0x45a, 0x430, 0x411, 0x438, 0x458, 0x443, 0x43b,
0x430, 0x2c, 0x20, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x414,
0x430, 0x43a, 0x43e, 0x442, 0x430, 0x426, 0x435, 0x43d, 0x442, 0x430, 0x440, 0x2c,
0x20, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x414, 0x430, 0x43a,
0x43e, 0x442, 0x430, 0x41d, 0x43e, 0x432, 0x438, 0x20, 0x421, 0x430, 0x43b, 0x435,
0x43c, 0x2c, 0x20, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x414,
0x430, 0x43a, 0x43e, 0x442, 0x430, 0x424, 0x435, 0x43d, 0x438, 0x43a, 0x441, 0x41f,
0x43e, 0x440, 0x442, 0x20, 0x43e, 0x444, 0x20, 0x421, 0x43f, 0x435, 0x458, 0x43d,
0x41f, 0x43e, 0x440, 0x442, 0x43e, 0x20, 0x412, 0x435, 0x459, 0x43e, 0x41f, 0x43e,
0x440, 0x442, 0x43e, 0x440, 0x438, 0x43a, 0x43e, 0x41f, 0x443, 0x43d, 0x442, 0x430,
0x20, 0x410, 0x440, 0x435, 0x43d, 0x430, 0x441, 0x420, 0x430, 0x43d, 0x43a, 0x438,
0x43d, 0x20, 0x418, 0x43d, 0x43b, 0x435, 0x442, 0x420, 0x435, 0x441, 0x438, 0x444,
0x435, 0x420, 0x435, 0x433, 0x438, 0x43d, 0x430, 0x420, 0x435, 0x441, 0x43e, 0x43b,
0x443, 0x442, 0x420, 0x438, 0x43e, 0x20, 0x411, 0x440, 0x430, 0x43d, 0x43a, 0x43e,
0x421, 0x430, 0x43d, 0x442, 0x430, 0x440, 0x435, 0x43c, 0x421, 0x430, 0x43d, 0x442,
0x438, 0x458, 0x430, 0x433, 0x43e, 0x421, 0x430, 0x43d, 0x442, 0x43e, 0x20, 0x414,
0x43e, 0x43c, 0x438, 0x43d, 0x433, 0x43e, 0x421, 0x430, 0x43e, 0x20, 0x41f, 0x430,
0x43e, 0x43b, 0x43e, 0x418, 0x442, 0x43e, 0x43a, 0x43e, 0x440, 0x442, 0x43e, 0x440,
0x43c, 0x438, 0x442, 0x421, 0x432, 0x2e, 0x20, 0x411, 0x430, 0x440, 0x442, 0x43e,
0x43b, 0x43e, 0x43c, 0x65, 0x458, 0x421, 0x432, 0x2e, 0x20, 0x40f, 0x43e, 0x43d,
0x421, 0x435, 0x43d, 0x442, 0x20, 0x41a, 0x438, 0x442, 0x441, 0x421, 0x432, 0x2e,
0x20, 0x41b, 0x443, 0x446, 0x438, 0x458, 0x430, 0x421, 0x432, 0x2e, 0x20, 0x422,
0x43e, 0x43c, 0x430, 0x421, 0x435, 0x43d, 0x442, 0x20, 0x412, 0x438, 0x43d, 0x441,
0x435, 0x43d, 0x442, 0x421, 0x432, 0x438, 0x444, 0x442, 0x20, 0x41a, 0x443, 0x440,
0x435, 0x43d, 0x442, 0x412, 0x430, 0x458, 0x442, 0x445, 0x43e, 0x440, 0x441, 0x412,
0x438, 0x43d, 0x438, 0x43f, 0x435, 0x433, 0x408, 0x430, 0x43a, 0x443, 0x442, 0x430,
0x442, 0x41a, 0x430, 0x441, 0x435, 0x458, 0x414, 0x435, 0x458, 0x432, 0x438, 0x441,
0x414, 0x438, 0x43c, 0x43e, 0x43d, 0x20, 0x434, 0x2019, 0x423, 0x440, 0x432, 0x438,
0x43b, 0x41c, 0x435, 0x43a, 0x432, 0x43e, 0x440, 0x438, 0x41c, 0x43e, 0x441, 0x43e,
0x43d, 0x41c, 0x430, 0x43a, 0x43c, 0x443, 0x440, 0x434, 0x43e, 0x428, 0x43e, 0x432,
0x430, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x41b, 0x43e, 0x43d, 0x433, 0x458,
0x435, 0x440, 0x431, 0x458, 0x435, 0x43d, 0x410, 0x43b, 0x43c, 0x430, 0x442, 0x438,
0x410, 0x43d, 0x430, 0x434, 0x438, 0x440, 0x410, 0x43a, 0x443, 0x442, 0x43e, 0x431,
0x435, 0x410, 0x442, 0x438, 0x440, 0x430, 0x458, 0x411, 0x430, 0x43d, 0x43a, 0x43e,
0x43a, 0x411, 0x435, 0x458, 0x440, 0x443, 0x442, 0x411, 0x440, 0x443, 0x43d, 0x435,
0x458, 0x438, 0x425, 0x435, 0x431, 0x440, 0x43e, 0x43d, 0x425, 0x43e, 0x20, 0x428,
0x438, 0x20, 0x41c, 0x438, 0x43d, 0x425, 0x43e, 0x43d, 0x433, 0x20, 0x41a, 0x43e,
0x43d, 0x433, 0x418, 0x440, 0x43a, 0x443, 0x446, 0x43a, 0x40f, 0x430, 0x458, 0x430,
0x43f, 0x443, 0x440, 0x430, 0x408, 0x435, 0x440, 0x443, 0x441, 0x430, 0x43b, 0x438,
0x43c, 0x425, 0x430, 0x43d, 0x434, 0x438, 0x433, 0x430, 0x41a, 0x430, 0x43b, 0x43a,
0x443, 0x442, 0x430, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x458, 0x430, 0x440,
0x441, 0x43a, 0x41a, 0x443, 0x430, 0x43b, 0x430, 0x20, 0x41b, 0x443, 0x43c, 0x43f,
0x443, 0x440, 0x41a, 0x443, 0x432, 0x430, 0x458, 0x442, 0x41c, 0x430, 0x43a, 0x430,
0x443, 0x41c, 0x443, 0x441, 0x43a, 0x430, 0x442, 0x41d, 0x438, 0x43a, 0x43e, 0x437,
0x438, 0x458, 0x430, 0x41d, 0x43e, 0x432, 0x43e, 0x43a, 0x443, 0x437, 0x45a, 0x435,
0x446, 0x43a, 0x41e, 0x440, 0x430, 0x43b, 0x41f, 0x43d, 0x43e, 0x43c, 0x20, 0x41f,
0x435, 0x43d, 0x41f, 0x458, 0x43e, 0x43d, 0x433, 0x458, 0x430, 0x43d, 0x433, 0x41a,
0x43e, 0x441, 0x442, 0x430, 0x43d, 0x430, 0x458, 0x41a, 0x438, 0x437, 0x438, 0x43b,
0x43e, 0x440, 0x434, 0x430, 0x420, 0x438, 0x458, 0x430, 0x434, 0x428, 0x430, 0x43d,
0x433, 0x430, 0x458, 0x421, 0x440, 0x435, 0x434, 0x45a, 0x435, 0x43a, 0x43e, 0x43b,
0x438, 0x43c, 0x441, 0x43a, 0x422, 0x430, 0x458, 0x43f, 0x435, 0x458, 0x422, 0x431,
0x438, 0x43b, 0x438, 0x441, 0x438, 0x422, 0x435, 0x445, 0x435, 0x440, 0x430, 0x43d,
0x422, 0x438, 0x43c, 0x43f, 0x443, 0x423, 0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430,
0x442, 0x43e, 0x440, 0x423, 0x440, 0x443, 0x43c, 0x43a, 0x432, 0x438, 0x412, 0x438,
0x458, 0x435, 0x43d, 0x442, 0x438, 0x458, 0x430, 0x43d, 0x408, 0x430, 0x43a, 0x443,
0x442, 0x441, 0x43a, 0x408, 0x435, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d,
0x431, 0x443, 0x440, 0x433, 0x408, 0x435, 0x440, 0x435, 0x432, 0x430, 0x43d, 0x410,
0x437, 0x43e, 0x440, 0x438, 0x411, 0x435, 0x440, 0x43c, 0x443, 0x434, 0x430, 0x41a,
0x430, 0x43d, 0x430, 0x440, 0x441, 0x43a, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440,
0x432, 0x430, 0x417, 0x435, 0x43b, 0x435, 0x43d, 0x43e, 0x440, 0x442, 0x441, 0x43a,
0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x424, 0x430, 0x440, 0x441,
0x43a, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x420, 0x435, 0x458,
0x43a, 0x458, 0x430, 0x432, 0x438, 0x43a, 0x408, 0x443, 0x436, 0x43d, 0x430, 0x20,
0x40f, 0x43e, 0x440, 0x45f, 0x438, 0x458, 0x430, 0x421, 0x432, 0x2e, 0x20, 0x408,
0x435, 0x43b, 0x435, 0x43d, 0x430, 0x411, 0x440, 0x438, 0x437, 0x431, 0x435, 0x458,
0x43d, 0x411, 0x440, 0x43e, 0x43a, 0x435, 0x43d, 0x20, 0x425, 0x438, 0x43b, 0x41b,
0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x432, 0x421, 0x438, 0x434, 0x43d, 0x435,
0x458, 0x410, 0x442, 0x438, 0x43d, 0x430, 0x411, 0x435, 0x43e, 0x433, 0x440, 0x430,
0x434, 0x411, 0x440, 0x438, 0x441, 0x435, 0x43b, 0x411, 0x443, 0x43a, 0x443, 0x440,
0x435, 0x448, 0x442, 0x411, 0x443, 0x434, 0x438, 0x43c, 0x43f, 0x435, 0x448, 0x442,
0x430, 0x411, 0x443, 0x441, 0x438, 0x43d, 0x433, 0x435, 0x43d, 0x41a, 0x438, 0x448,
0x438, 0x45a, 0x435, 0x432, 0x41a, 0x43e, 0x43f, 0x435, 0x43d, 0x445, 0x430, 0x433,
0x435, 0x43d, 0x414, 0x430, 0x431, 0x43b, 0x438, 0x43d, 0x413, 0x435, 0x440, 0x43d,
0x437, 0x438, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x43e, 0x20, 0x41c, 0x435, 0x43d,
0x418, 0x441, 0x442, 0x430, 0x43d, 0x431, 0x443, 0x43b, 0x41a, 0x430, 0x43b, 0x438,
0x45a, 0x438, 0x43d, 0x433, 0x440, 0x430, 0x434, 0x41a, 0x438, 0x458, 0x435, 0x432,
0x41b, 0x438, 0x441, 0x430, 0x431, 0x43e, 0x43d, 0x409, 0x443, 0x431, 0x459, 0x430,
0x43d, 0x430, 0x41b, 0x443, 0x43a, 0x441, 0x435, 0x43c, 0x431, 0x443, 0x440, 0x433,
0x41c, 0x430, 0x440, 0x438, 0x445, 0x430, 0x43c, 0x43d, 0x41f, 0x430, 0x440, 0x438,
0x437, 0x421, 0x430, 0x43d, 0x20, 0x41c, 0x430, 0x440, 0x438, 0x43d, 0x43e, 0x421,
0x430, 0x440, 0x430, 0x458, 0x435, 0x432, 0x43e, 0x421, 0x438, 0x43c, 0x444, 0x435,
0x440, 0x43e, 0x43f, 0x43e, 0x459, 0x421, 0x43a, 0x43e, 0x43f, 0x459, 0x435, 0x421,
0x43e, 0x444, 0x438, 0x458, 0x430, 0x421, 0x442, 0x43e, 0x43a, 0x445, 0x43e, 0x43b,
0x43c, 0x422, 0x430, 0x43b, 0x438, 0x43d, 0x423, 0x459, 0x430, 0x43d, 0x43e, 0x432,
0x441, 0x43a, 0x412, 0x430, 0x434, 0x443, 0x437, 0x411, 0x435, 0x447, 0x412, 0x438,
0x459, 0x43d, 0x443, 0x441, 0x426, 0x438, 0x440, 0x438, 0x445, 0x410, 0x43d, 0x442,
0x430, 0x43d, 0x430, 0x43d, 0x430, 0x440, 0x438, 0x432, 0x43e, 0x411, 0x43e, 0x436,
0x438, 0x45b, 0x41a, 0x43e, 0x43c, 0x43e, 0x440, 0x43e, 0x41c, 0x430, 0x445, 0x435,
0x41c, 0x430, 0x43b, 0x434, 0x438, 0x432, 0x438, 0x41c, 0x430, 0x443, 0x440, 0x438,
0x446, 0x438, 0x458, 0x443, 0x441, 0x41c, 0x430, 0x458, 0x43e, 0x442, 0x420, 0x435,
0x443, 0x43d, 0x438, 0x43e, 0x43d, 0x410, 0x43f, 0x438, 0x458, 0x430, 0x41e, 0x43a,
0x43b, 0x430, 0x43d, 0x434, 0x41a, 0x430, 0x442, 0x445, 0x430, 0x43c, 0x423, 0x441,
0x43a, 0x440, 0x448, 0x45a, 0x435, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x43e,
0x413, 0x430, 0x43c, 0x431, 0x438, 0x458, 0x435, 0x413, 0x432, 0x430, 0x434, 0x430,
0x43b, 0x43a, 0x430, 0x43d, 0x430, 0x43b, 0x425, 0x43e, 0x43d, 0x43e, 0x43b, 0x443,
0x43b, 0x443, 0x415, 0x43d, 0x434, 0x435, 0x440, 0x431, 0x435, 0x440, 0x438, 0x41a,
0x43e, 0x448, 0x440, 0x435, 0x41a, 0x432, 0x430, 0x45f, 0x430, 0x43b, 0x435, 0x458,
0x438, 0x43d, 0x41c, 0x430, 0x458, 0x443, 0x440, 0x43e, 0x41c, 0x438, 0x434, 0x432,
0x435, 0x458, 0x41f, 0x430, 0x433, 0x43e, 0x20, 0x41f, 0x430, 0x433, 0x43e, 0x41f,
0x43e, 0x43d, 0x430, 0x43f, 0x435, 0x41f, 0x43e, 0x440, 0x442, 0x20, 0x41c, 0x43e,
0x440, 0x437, 0x431, 0x438, 0x421, 0x430, 0x458, 0x43f, 0x430, 0x43d, 0x422, 0x430,
0x445, 0x438, 0x442, 0x438, 0x412, 0x435, 0x458, 0x43a, 0x412, 0x430, 0x43b, 0x438,
0x441, 0x41, 0x64, 0x69, 0x73, 0x20, 0x41, 0x62, 0x65, 0x62, 0x61, 0x4b,
0x61, 0x65, 0x72, 0x6f, 0x44, 0x61, 0x72, 0x20, 0x65, 0x6c, 0x20, 0x42,
0x65, 0x69, 0x64, 0x61, 0x20, 0x28, 0x43, 0x61, 0x73, 0x61, 0x62, 0x6c,
0x61, 0x6e, 0x63, 0x61, 0x29, 0x4c, 0x61, 0x2bc, 0x59, 0x6f, 0x75, 0x6e,
0x4e, 0x2bc, 0x44, 0x6a, 0x61, 0x6d, 0x65, 0x6e, 0x61, 0x54, 0x61, 0x72,
0x61, 0x62, 0x75, 0x6c, 0x75, 0x73, 0x20, 0x28, 0x54, 0x72, 0x69, 0x70,
0x6f, 0x6c, 0x69, 0x29, 0x42, 0x65, 0x6c, 0xe9, 0x6d, 0x47, 0x77, 0x61,
0x64, 0x65, 0x6c, 0x6f, 0x75, 0x70, 0x4b, 0xea, 0x72, 0x20, 0x56, 0x65,
0x63, 0x2bc, 0x68, 0x69, 0x6b, 0x6f, 0x4e, 0x75, 0x75, 0x6b, 0x20, 0x28,
0x47, 0x6f, 0x64, 0x74, 0x68, 0xe5, 0x62, 0x29, 0x53, 0x61, 0x69, 0x6e,
0x74, 0x20, 0x42, 0x61, 0x72, 0x74, 0x68, 0xe9, 0x6c, 0x65, 0x6d, 0x79,
0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x4a, 0x6f, 0x68, 0x6e, 0x2bc, 0x73,
0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x4c, 0x75, 0x63, 0x69, 0x61, 0x53,
0x61, 0x6e, 0x74, 0x20, 0x56, 0x69, 0x73, 0x61, 0x6e, 0x74, 0x51, 0xe2,
0x6e, 0xe2, 0x71, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x2bc, 0x42, 0x61,
0x6b, 0x6f, 0x75, 0x42, 0x61, 0x79, 0x72, 0x75, 0x74, 0x54, 0x63, 0x68,
0x69, 0x74, 0x61, 0x4b, 0x6f, 0x6c, 0x61, 0x6d, 0x62, 0x61, 0x4b, 0xea,
0x72, 0x20, 0x48, 0xf4, 0x2d, 0x43, 0x68, 0x69, 0x2d, 0x4d, 0x69, 0x6e,
0x68, 0x4b, 0x61, 0x62, 0x6f, 0x75, 0x6c, 0x4b, 0x61, 0x6d, 0x74, 0x63,
0x68, 0x61, 0x74, 0x6b, 0x61, 0x43, 0x61, 0x6c, 0x63, 0x75, 0x74, 0x74,
0x61, 0x4b, 0x6f, 0x77, 0x65, 0x69, 0x74, 0x4d, 0x61, 0x73, 0x71, 0x61,
0x74, 0x4c, 0x65, 0x76, 0x6b, 0x6f, 0x73, 0xed, 0x61, 0x50, 0x68, 0x6e,
0x75, 0x6d, 0x20, 0x50, 0xe9, 0x6e, 0x68, 0x50, 0x2bc, 0x79, 0x6f, 0x6e,
0x67, 0x79, 0x61, 0x6e, 0x67, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f,
0x75, 0x72, 0x54, 0x6f, 0x73, 0x68, 0x6b, 0x65, 0x6e, 0x74, 0x56, 0x69,
0x61, 0x6e, 0x67, 0x63, 0x68, 0x61, 0x6e, 0x59, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x6f, 0x75, 0x72, 0x67, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x7a, 0x42, 0x65, 0x72, 0x6d, 0x75, 0x64, 0x65, 0x7a, 0x4b,
0x61, 0x6e, 0x61, 0x72, 0x69, 0x65, 0x7a, 0x4b, 0x61, 0x62, 0x20, 0x47,
0x6c, 0x61, 0x73, 0x46, 0x61, 0x65, 0x72, 0x6f, 0x52, 0x65, 0x79, 0x6b,
0x6a, 0x61, 0x76, 0xed, 0x6b, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61,
0x20, 0x61, 0x72, 0x20, 0x53, 0x75, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20,
0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x44, 0x75, 0x6c, 0x65, 0x6e, 0x6e,
0x4a, 0x69, 0x62, 0x72, 0x61, 0x6c, 0x74, 0x61, 0x72, 0x47, 0x77, 0x65,
0x72, 0x6e, 0x65, 0x6e, 0x65, 0x7a, 0x4d, 0x61, 0x6e, 0x61, 0x76, 0x4a,
0x65, 0x72, 0x7a, 0x65, 0x6e, 0x65, 0x7a, 0x4c, 0x6f, 0x6e, 0x64, 0x72,
0x65, 0x7a, 0x4c, 0x75, 0x6b, 0x73, 0x65, 0x6d, 0x62, 0x6f, 0x75, 0x72,
0x67, 0x4d, 0x61, 0x72, 0x6a, 0x65, 0x68, 0x61, 0x6d, 0x6e, 0x4d, 0x65,
0x6e, 0x73, 0x6b, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x50, 0x72, 0x61,
0x68, 0x61, 0x4b, 0x6f, 0x6b, 0x6f, 0x7a, 0x4b, 0x6f, 0x6d, 0x6f, 0x72,
0x65, 0x7a, 0x4b, 0x65, 0x72, 0x67, 0x65, 0x6c, 0x65, 0x6e, 0x6e, 0x4d,
0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x7a, 0x4d, 0x6f, 0x72, 0x69, 0x73,
0x52, 0x65, 0xfc, 0x6e, 0x69, 0x6f, 0x6e, 0x45, 0x6e, 0x65, 0x7a, 0x20,
0x50, 0x61, 0x73, 0x6b, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x7a, 0x410, 0x431,
0x438, 0x434, 0x436, 0x430, 0x43d, 0x411, 0x43b, 0x430, 0x43d, 0x442, 0x430, 0x439,
0x44a, 0x440, 0x41a, 0x430, 0x439, 0x440, 0x43e, 0x414, 0x430, 0x440, 0x20, 0x435,
0x441, 0x20, 0x421, 0x430, 0x43b, 0x430, 0x43c, 0x414, 0x436, 0x438, 0x431, 0x443,
0x442, 0x438, 0x415, 0x43b, 0x20, 0x410, 0x44e, 0x43d, 0x424, 0x440, 0x438, 0x439,
0x442, 0x430, 0x443, 0x43d, 0x419, 0x43e, 0x445, 0x430, 0x43d, 0x435, 0x441, 0x431,
0x443, 0x440, 0x433, 0x41c, 0x43e, 0x433, 0x430, 0x434, 0x438, 0x448, 0x443, 0x41c,
0x43e, 0x43d, 0x440, 0x43e, 0x432, 0x438, 0x44f, 0x41d, 0x430, 0x439, 0x440, 0x43e,
0x431, 0x438, 0x41d, 0x438, 0x430, 0x43c, 0x435, 0x439, 0x410, 0x43d, 0x43a, 0x44a,
0x440, 0x438, 0x434, 0x436, 0x410, 0x43d, 0x433, 0x443, 0x438, 0x43b, 0x430, 0x410,
0x440, 0x430, 0x433, 0x443, 0x430, 0x439, 0x43d, 0x430, 0x411, 0x443, 0x435, 0x43d,
0x43e, 0x441, 0x20, 0x410, 0x439, 0x440, 0x435, 0x441, 0x425, 0x443, 0x445, 0x443,
0x439, 0x420, 0x438, 0x43e, 0x20, 0x413, 0x430, 0x43b, 0x435, 0x433, 0x43e, 0x441,
0x421, 0x430, 0x43d, 0x20, 0x41b, 0x443, 0x438, 0x441, 0x411, 0x430, 0x438, 0x44f,
0x411, 0x430, 0x438, 0x44f, 0x20, 0x434, 0x435, 0x20, 0x411, 0x430, 0x43d, 0x434,
0x435, 0x440, 0x430, 0x441, 0x411, 0x43b, 0x430, 0x43d, 0x2d, 0x421, 0x430, 0x431,
0x43b, 0x43e, 0x43d, 0x411, 0x43e, 0x439, 0x441, 0x438, 0x41a, 0x435, 0x439, 0x43c,
0x431, 0x440, 0x438, 0x434, 0x436, 0x20, 0x411, 0x435, 0x439, 0x41a, 0x430, 0x439,
0x435, 0x43d, 0x41a, 0x430, 0x439, 0x43c, 0x430, 0x43d, 0x43e, 0x432, 0x438, 0x20,
0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x421, 0x438, 0x443, 0x434, 0x430,
0x434, 0x20, 0x425, 0x443, 0x430, 0x440, 0x435, 0x441, 0x41a, 0x43e, 0x441, 0x442,
0x430, 0x20, 0x420, 0x438, 0x43a, 0x430, 0x41a, 0x440, 0x435, 0x441, 0x442, 0x44a,
0x43d, 0x427, 0x443, 0x44f, 0x431, 0x430, 0x41a, 0x44e, 0x440, 0x430, 0x441, 0x430,
0x43e, 0x414, 0x430, 0x43d, 0x43c, 0x430, 0x440, 0x43a, 0x441, 0x445, 0x430, 0x432,
0x43d, 0x414, 0x43e, 0x443, 0x441, 0x44a, 0x43d, 0x414, 0x43e, 0x443, 0x441, 0x44a,
0x43d, 0x20, 0x41a, 0x440, 0x438, 0x439, 0x43a, 0x414, 0x435, 0x43d, 0x432, 0x44a,
0x440, 0x414, 0x435, 0x442, 0x440, 0x43e, 0x439, 0x442, 0x415, 0x434, 0x43c, 0x44a,
0x43d, 0x442, 0x44a, 0x43d, 0x415, 0x439, 0x440, 0x443, 0x43d, 0x435, 0x43f, 0x435,
0x424, 0x43e, 0x440, 0x442, 0x20, 0x41d, 0x435, 0x43b, 0x441, 0x44a, 0x43d, 0x413,
0x43b, 0x435, 0x439, 0x441, 0x20, 0x411, 0x435, 0x439, 0x413, 0x443, 0x441, 0x20,
0x411, 0x435, 0x439, 0x413, 0x440, 0x430, 0x43d, 0x434, 0x20, 0x422, 0x44a, 0x440,
0x43a, 0x413, 0x443, 0x430, 0x44f, 0x43a, 0x438, 0x43b, 0x415, 0x440, 0x43c, 0x43e,
0x441, 0x438, 0x43b, 0x44c, 0x43e, 0x41f, 0x438, 0x442, 0x44a, 0x440, 0x441, 0x431,
0x44a, 0x440, 0x433, 0x412, 0x438, 0x432, 0x438, 0x412, 0x438, 0x43d, 0x441, 0x435,
0x43d, 0x441, 0x414, 0x436, 0x443, 0x43d, 0x43e, 0x41a, 0x440, 0x430, 0x43b, 0x435,
0x43d, 0x434, 0x435, 0x439, 0x43a, 0x41b, 0x430, 0x20, 0x41f, 0x430, 0x441, 0x41b,
0x43e, 0x441, 0x20, 0x410, 0x43d, 0x434, 0x436, 0x435, 0x43b, 0x438, 0x441, 0x41b,
0x43e, 0x443, 0x44a, 0x440, 0x20, 0x43f, 0x440, 0x438, 0x43d, 0x441, 0x435, 0x441,
0x20, 0x43a, 0x443, 0x43e, 0x442, 0x44a, 0x440, 0x41c, 0x430, 0x441, 0x435, 0x439,
0x43e, 0x41c, 0x43e, 0x43d, 0x442, 0x435, 0x440, 0x435, 0x439, 0x41d, 0x44e, 0x20,
0x419, 0x43e, 0x440, 0x43a, 0x41d, 0x43e, 0x443, 0x43c, 0x41d, 0x43e, 0x440, 0x43e,
0x43d, 0x44f, 0x411, 0x44e, 0x43b, 0x430, 0x421, 0x435, 0x43d, 0x442, 0x44a, 0x440,
0x41d, 0x44e, 0x20, 0x421, 0x435, 0x439, 0x43b, 0x44a, 0x43c, 0x41f, 0x43e, 0x440,
0x442, 0x20, 0x44a, 0x444, 0x20, 0x421, 0x43f, 0x435, 0x439, 0x43d, 0x41f, 0x43e,
0x440, 0x442, 0x43e, 0x20, 0x412, 0x435, 0x43b, 0x44c, 0x43e, 0x41f, 0x443, 0x435,
0x440, 0x442, 0x43e, 0x20, 0x420, 0x438, 0x43a, 0x43e, 0x420, 0x438, 0x434, 0x436,
0x430, 0x439, 0x43d, 0x430, 0x420, 0x435, 0x437, 0x43e, 0x43b, 0x44e, 0x442, 0x421,
0x430, 0x43d, 0x442, 0x44f, 0x433, 0x43e, 0x421, 0x430, 0x43e, 0x20, 0x41f, 0x430,
0x443, 0x43b, 0x43e, 0x421, 0x433, 0x43e, 0x440, 0x441, 0x431, 0x438, 0x441, 0x43e,
0x43d, 0x421, 0x435, 0x43d, 0x20, 0x411, 0x430, 0x440, 0x442, 0x435, 0x43b, 0x435,
0x43c, 0x438, 0x421, 0x435, 0x439, 0x43d, 0x442, 0x20, 0x414, 0x436, 0x43e, 0x43d,
0x441, 0x421, 0x435, 0x439, 0x43d, 0x442, 0x20, 0x41a, 0x438, 0x442, 0x441, 0x421,
0x435, 0x439, 0x43d, 0x442, 0x20, 0x41b, 0x443, 0x441, 0x438, 0x44f, 0x421, 0x435,
0x439, 0x43d, 0x442, 0x20, 0x422, 0x43e, 0x43c, 0x430, 0x441, 0x421, 0x435, 0x439,
0x43d, 0x442, 0x20, 0x412, 0x438, 0x43d, 0x441, 0x44a, 0x43d, 0x442, 0x421, 0x443,
0x438, 0x444, 0x442, 0x20, 0x41a, 0x44a, 0x440, 0x435, 0x43d, 0x442, 0x412, 0x430,
0x43d, 0x43a, 0x443, 0x432, 0x44a, 0x440, 0x423, 0x438, 0x43d, 0x438, 0x43f, 0x435,
0x433, 0x41a, 0x435, 0x439, 0x441, 0x438, 0x414, 0x435, 0x439, 0x432, 0x438, 0x441,
0x414, 0x44e, 0x43c, 0x43e, 0x43d, 0x20, 0x414, 0x44e, 0x440, 0x432, 0x438, 0x43b,
0x41c, 0x43e, 0x443, 0x441, 0x44a, 0x43d, 0x41c, 0x430, 0x43a, 0x43c, 0x44a, 0x440,
0x434, 0x43e, 0x41f, 0x430, 0x43b, 0x43c, 0x44a, 0x440, 0x41b, 0x43e, 0x43d, 0x433,
0x438, 0x440, 0x431, 0x44e, 0x435, 0x43d, 0x410, 0x442, 0x44a, 0x440, 0x430, 0x443,
0x411, 0x430, 0x445, 0x440, 0x435, 0x439, 0x43d, 0x425, 0x43e, 0x43d, 0x43a, 0x43e,
0x43d, 0x433, 0x419, 0x435, 0x440, 0x443, 0x441, 0x430, 0x43b, 0x438, 0x43c, 0x41a,
0x43e, 0x43b, 0x43a, 0x430, 0x442, 0x430, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e,
0x44f, 0x440, 0x441, 0x43a, 0x41d, 0x438, 0x43a, 0x43e, 0x437, 0x438, 0x44f, 0x410,
0x440, 0x430, 0x43b, 0x41f, 0x445, 0x435, 0x43d, 0x44f, 0x43d, 0x41a, 0x43e, 0x441,
0x442, 0x430, 0x43d, 0x430, 0x439, 0x41a, 0x44a, 0x437, 0x44a, 0x43b, 0x43e, 0x440,
0x434, 0x430, 0x420, 0x438, 0x44f, 0x434, 0x421, 0x440, 0x435, 0x434, 0x43d, 0x435,
0x43a, 0x43e, 0x43b, 0x438, 0x43c, 0x441, 0x43a, 0x422, 0x430, 0x439, 0x43f, 0x435,
0x412, 0x438, 0x435, 0x43d, 0x442, 0x44f, 0x43d, 0x42f, 0x43a, 0x443, 0x442, 0x441,
0x43a, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x438, 0x411, 0x435, 0x440, 0x43c, 0x443, 0x434, 0x441, 0x43a,
0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x41a, 0x430, 0x43d,
0x430, 0x440, 0x441, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432,
0x438, 0x41a, 0x430, 0x431, 0x43e, 0x20, 0x412, 0x435, 0x440, 0x434, 0x435, 0x424,
0x430, 0x440, 0x44c, 0x43e, 0x440, 0x441, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x438, 0x41c, 0x430, 0x434, 0x435, 0x439, 0x440, 0x430, 0x420,
0x435, 0x439, 0x43a, 0x44f, 0x432, 0x438, 0x43a, 0x42e, 0x436, 0x43d, 0x430, 0x20,
0x414, 0x436, 0x43e, 0x440, 0x434, 0x436, 0x438, 0x44f, 0x421, 0x432, 0x435, 0x442,
0x430, 0x20, 0x415, 0x43b, 0x435, 0x43d, 0x430, 0x411, 0x440, 0x438, 0x437, 0x431,
0x435, 0x439, 0x43d, 0x411, 0x440, 0x43e, 0x443, 0x43a, 0x44a, 0x43d, 0x20, 0x425,
0x438, 0x43b, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x430, 0x443, 0x41c, 0x435,
0x43b, 0x431, 0x44a, 0x440, 0x43d, 0x41f, 0x44a, 0x440, 0x442, 0x421, 0x438, 0x434,
0x43d, 0x438, 0x411, 0x440, 0x44e, 0x43a, 0x441, 0x435, 0x43b, 0x411, 0x443, 0x43a,
0x443, 0x440, 0x435, 0x449, 0x411, 0x443, 0x434, 0x430, 0x43f, 0x435, 0x449, 0x430,
0x411, 0x44e, 0x437, 0x438, 0x43d, 0x433, 0x435, 0x43d, 0x414, 0x44a, 0x431, 0x43b,
0x438, 0x43d, 0x413, 0x44a, 0x440, 0x43d, 0x437, 0x438, 0x425, 0x435, 0x43b, 0x437,
0x438, 0x43d, 0x43a, 0x438, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x20, 0x41c,
0x430, 0x43d, 0x414, 0x436, 0x44a, 0x440, 0x441, 0x438, 0x421, 0x43a, 0x43e, 0x43f,
0x438, 0x435, 0x421, 0x43e, 0x444, 0x438, 0x44f, 0x423, 0x43b, 0x44f, 0x43d, 0x43e,
0x432, 0x441, 0x43a, 0x412, 0x438, 0x435, 0x43d, 0x430, 0x412, 0x438, 0x43b, 0x43d,
0x44e, 0x441, 0x426, 0x44e, 0x440, 0x438, 0x445, 0x420, 0x43e, 0x436, 0x434, 0x435,
0x441, 0x442, 0x432, 0x43e, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x43e, 0x432, 0x438,
0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x41a, 0x43e, 0x43c, 0x43e,
0x440, 0x441, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438,
0x41c, 0x430, 0x432, 0x440, 0x438, 0x446, 0x438, 0x439, 0x41c, 0x430, 0x439, 0x43e,
0x442, 0x420, 0x435, 0x44e, 0x43d, 0x438, 0x43e, 0x43d, 0x410, 0x43f, 0x438, 0x44f,
0x427, 0x430, 0x442, 0x430, 0x43c, 0x427, 0x443, 0x44e, 0x43a, 0x412, 0x435, 0x43b,
0x438, 0x43a, 0x434, 0x435, 0x43d, 0x441, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x424, 0x438, 0x434, 0x436, 0x438, 0x413, 0x430, 0x43c, 0x431,
0x438, 0x435, 0x415, 0x43d, 0x434, 0x44a, 0x440, 0x431, 0x44a, 0x440, 0x438, 0x41a,
0x43e, 0x448, 0x440, 0x430, 0x439, 0x41a, 0x443, 0x430, 0x434, 0x436, 0x430, 0x43b,
0x438, 0x43d, 0x41c, 0x430, 0x434, 0x436, 0x443, 0x440, 0x43e, 0x41c, 0x430, 0x440,
0x43a, 0x438, 0x437, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432,
0x438, 0x41c, 0x438, 0x434, 0x443, 0x435, 0x439, 0x41f, 0x43e, 0x43d, 0x43f, 0x435,
0x439, 0x41f, 0x43e, 0x440, 0x442, 0x20, 0x41c, 0x43e, 0x440, 0x441, 0x431, 0x438,
0x422, 0x430, 0x440, 0x430, 0x443, 0x430, 0x423, 0x435, 0x439, 0x43a, 0x423, 0x43e,
0x43b, 0x438, 0x441, 0x1021, 0x102c, 0x1018, 0x102e, 0x1002, 0x103b, 0x1014, 0x103a, 0x1021,
0x1000, 0x103a, 0x1000, 0x101b, 0x102c, 0x1021, 0x102c, 0x1038, 0x1012, 0x1005, 0x103a, 0x20,
0x1021, 0x1018, 0x102c, 0x1018, 0x102c, 0x1021, 0x101a, 0x103a, 0x101c, 0x103a, 0x1002, 0x103b,
0x102e, 0x1038, 0x101b, 0x102e, 0x1038, 0x101a, 0x102c, 0x1038, 0x1021, 0x102c, 0x1038, 0x1005,
0x103a, 0x1019, 0x102c, 0x1038, 0x101b, 0x102c, 0x1018, 0x102c, 0x1019, 0x102c, 0x1000, 0x102d,
0x102f, 0x1018, 0x102c, 0x1014, 0x103a, 0x1002, 0x102e, 0x1038, 0x1018, 0x1014, 0x103a, 0x1002,
0x103b, 0x102f, 0x1038, 0x101c, 0x103a, 0x1018, 0x102e, 0x1005, 0x102c, 0x1021, 0x102d, 0x102f,
0x1018, 0x101c, 0x1014, 0x103a, 0x1010, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x101b, 0x1032,
0x101c, 0x103a, 0x1018, 0x101b, 0x102c, 0x1038, 0x1007, 0x102c, 0x1017, 0x102e, 0x1038, 0x101c,
0x103a, 0x1018, 0x1030, 0x1002, 0x103b, 0x103d, 0x1014, 0x103a, 0x1018, 0x1030, 0x1038, 0x101b,
0x102c, 0x1000, 0x102d, 0x102f, 0x1004, 0x103a, 0x101b, 0x102d, 0x102f, 0x1000, 0x102c, 0x1006,
0x102c, 0x1018, 0x101c, 0x1014, 0x103a, 0x1000, 0x102c, 0x1006, 0x102e, 0x101b, 0x1030, 0x1038,
0x1010, 0x102c, 0x1000, 0x102d, 0x102f, 0x1014, 0x102c, 0x1000, 0x101b, 0x102e, 0x1038, 0x1012,
0x1000, 0x103b, 0x1000, 0x102c, 0x1038, 0x1012, 0x102b, 0x101b, 0x1000, 0x103a, 0x1005, 0x103a,
0x1006, 0x102c, 0x101c, 0x1019, 0x103a, 0x1002, 0x103b, 0x102e, 0x1018, 0x1030, 0x1010, 0x102e,
0x1012, 0x102d, 0x102f, 0x1021, 0x1030, 0x1021, 0x102c, 0x101c, 0x102c, 0x1021, 0x101a, 0x103a,
0x101c, 0x103a, 0x1021, 0x102c, 0x101a, 0x103d, 0x1014, 0x103a, 0x1038, 0x1016, 0x101b, 0x102e,
0x1038, 0x1010, 0x1031, 0x102c, 0x1004, 0x103a, 0x1038, 0x1014, 0x103a, 0x1002, 0x102b, 0x1018,
0x102c, 0x101b, 0x103d, 0x1014, 0x103a, 0x1038, 0x1014, 0x102e, 0x101f, 0x102c, 0x101b, 0x102c,
0x1038, 0x101b, 0x1032, 0x101a, 0x103a, 0x1002, 0x103b, 0x102d, 0x102f, 0x101f, 0x1014, 0x103a,
0x1038, 0x1014, 0x1000, 0x103a, 0x1005, 0x1018, 0x1010, 0x103a, 0x1002, 0x103b, 0x102f, 0x1018,
0x102c, 0x1000, 0x1019, 0x103a, 0x1015, 0x102b, 0x101c, 0x102c, 0x1001, 0x102b, 0x1010, 0x102d,
0x102f, 0x1021, 0x1014, 0x103a, 0x1000, 0x102e, 0x1002, 0x102b, 0x1038, 0x101c, 0x102e, 0x1000,
0x1004, 0x103a, 0x101b, 0x103e, 0x102c, 0x1005, 0x102c, 0x101c, 0x102c, 0x1002, 0x102d, 0x102f,
0x1005, 0x103a, 0x101c, 0x102e, 0x1017, 0x101b, 0x102c, 0x1017, 0x102e, 0x1038, 0x101c, 0x103a,
0x101c, 0x102d, 0x102f, 0x1019, 0x102e, 0x101c, 0x1030, 0x101d, 0x1019, 0x103a, 0x1012, 0x102b,
0x101c, 0x1030, 0x1018, 0x103d, 0x1014, 0x103a, 0x1038, 0x1018, 0x102c, 0x101b, 0x103e, 0x102e,
0x101c, 0x1030, 0x1005, 0x102c, 0x1000, 0x102c, 0x1019, 0x102c, 0x101c, 0x102c, 0x1018, 0x102d,
0x102f, 0x1019, 0x102c, 0x1015, 0x1030, 0x1038, 0x1010, 0x102d, 0x102f, 0x1019, 0x102c, 0x1006,
0x1030, 0x101b, 0x1030, 0x1038, 0x1018, 0x102c, 0x1038, 0x1018, 0x102c, 0x1014, 0x103a, 0x1038,
0x1019, 0x1031, 0x102c, 0x103a, 0x1002, 0x102b, 0x1012, 0x102e, 0x1038, 0x101e, 0x103b, 0x102f,
0x1038, 0x1019, 0x103d, 0x1014, 0x103a, 0x101b, 0x102d, 0x102f, 0x1038, 0x1017, 0x102e, 0x1038,
0x101a, 0x102c, 0x1038, 0x1014, 0x102d, 0x102f, 0x1004, 0x103a, 0x101b, 0x102d, 0x102f, 0x1018,
0x102e, 0x1021, 0x1004, 0x103a, 0x1002, 0x103b, 0x102c, 0x1019, 0x102e, 0x1014, 0x102c, 0x100a,
0x102c, 0x1019, 0x1032, 0x101a, 0x103a, 0x1014, 0x102d, 0x102f, 0x1021, 0x102c, 0x1037, 0x1001,
0x103a, 0x101b, 0x103e, 0x1031, 0x102c, 0x1037, 0x1010, 0x103a, 0x101d, 0x102b, 0x1002, 0x102b,
0x1012, 0x1030, 0x1038, 0x1002, 0x1030, 0x1015, 0x102d, 0x102f, 0x1010, 0x102d, 0x102f, 0x2d,
0x1014, 0x102d, 0x102f, 0x1017, 0x102d, 0x102f, 0x1006, 0x1031, 0x102c, 0x1004, 0x103a, 0x1010,
0x1030, 0x1019, 0x1031, 0x1038, 0x1011, 0x101b, 0x102e, 0x1015, 0x102d, 0x102f, 0x101c, 0x102e,
0x1010, 0x1030, 0x1014, 0x102e, 0x1005, 0x103a, 0x1017, 0x1004, 0x103a, 0x1038, 0x101f, 0x1030,
0x1038, 0x1001, 0x103a, 0x1021, 0x102c, 0x1012, 0x1001, 0x103b, 0x1021, 0x1014, 0x103a, 0x1000,
0x102c, 0x101b, 0x1031, 0x1037, 0x1002, 0x103b, 0x103a, 0x1021, 0x1014, 0x103a, 0x1002, 0x102e,
0x101c, 0x102c, 0x1021, 0x1014, 0x103a, 0x1010, 0x102e, 0x1002, 0x103d, 0x102b, 0x1021, 0x102c,
0x101b, 0x102c, 0x1002, 0x103d, 0x102b, 0x1021, 0x102e, 0x1014, 0x102c, 0x1017, 0x103b, 0x1030,
0x1014, 0x102d, 0x102f, 0x1021, 0x1031, 0x1038, 0x101b, 0x102d, 0x1005, 0x103a, 0x1000, 0x102c,
0x1010, 0x102c, 0x1019, 0x102c, 0x101b, 0x1000, 0x102c, 0x1000, 0x102d, 0x102f, 0x1012, 0x102d,
0x102f, 0x1018, 0x102c, 0x1002, 0x103b, 0x102f, 0x1002, 0x103b, 0x1031, 0x101c, 0x102c, 0x20,
0x101b, 0x102e, 0x101a, 0x102d, 0x102f, 0x101f, 0x102c, 0x1019, 0x1014, 0x103a, 0x1012, 0x102d,
0x102f, 0x1007, 0x102c, 0x101b, 0x102e, 0x101a, 0x102d, 0x102f, 0x20, 0x1002, 0x102b, 0x101c,
0x102e, 0x1002, 0x102d, 0x102f, 0x1005, 0x103a, 0x1006, 0x102c, 0x101c, 0x103a, 0x1010, 0x102c,
0x1006, 0x1014, 0x103a, 0x20, 0x1002, 0x103d, 0x1019, 0x103a, 0x1006, 0x1014, 0x103a, 0x20,
0x101c, 0x1030, 0x101d, 0x102e, 0x1005, 0x103a, 0x1010, 0x1030, 0x1000, 0x1030, 0x1019, 0x1014,
0x103a, 0x1025, 0x1006, 0x103d, 0x102c, 0x1021, 0x102e, 0x1021, 0x102c, 0x1021, 0x102c, 0x101b,
0x1030, 0x1038, 0x1017, 0x102c, 0x1021, 0x102c, 0x1006, 0x1030, 0x1005, 0x102e, 0x1021, 0x103d,
0x1014, 0x103a, 0x1038, 0x1021, 0x102c, 0x1010, 0x102e, 0x1000, 0x102d, 0x102f, 0x1000, 0x1014,
0x103a, 0x1018, 0x102c, 0x101f, 0x102e, 0x1021, 0x102c, 0x1018, 0x102c, 0x101f, 0x102e, 0x1021,
0x102c, 0x20, 0x1018, 0x1014, 0x103a, 0x1012, 0x101b, 0x1000, 0x103a, 0x1005, 0x103a, 0x1018,
0x102c, 0x1018, 0x1031, 0x1038, 0x1012, 0x102d, 0x102f, 0x1038, 0x1005, 0x103a, 0x1018, 0x102e,
0x101c, 0x1004, 0x103a, 0x1019, 0x103a, 0x1018, 0x101c, 0x102d, 0x1007, 0x103a, 0x1018, 0x101c,
0x103d, 0x1014, 0x103a, 0x1001, 0x103a, 0x2d, 0x1005, 0x102c, 0x1018, 0x101c, 0x103d, 0x1014,
0x103a, 0x1018, 0x102d, 0x102f, 0x1021, 0x102c, 0x1017, 0x102e, 0x1005, 0x103a, 0x1010, 0x102c,
0x1018, 0x102d, 0x102f, 0x1002, 0x102d, 0x102f, 0x1010, 0x102c, 0x1017, 0x103d, 0x102d, 0x102f,
0x1000, 0x103a, 0x1005, 0x102e, 0x1000, 0x102d, 0x1014, 0x103a, 0x1038, 0x1018, 0x101b, 0x1005,
0x103a, 0x1001, 0x103b, 0x103a, 0x20, 0x1018, 0x1031, 0x1038, 0x1000, 0x102d, 0x1019, 0x103a,
0x1015, 0x102d, 0x102f, 0x20, 0x1002, 0x101b, 0x1014, 0x103a, 0x1012, 0x102e, 0x1000, 0x1014,
0x103a, 0x1001, 0x1014, 0x103a, 0x1000, 0x102c, 0x101b, 0x102c, 0x1000, 0x102c, 0x1005, 0x103a,
0x1000, 0x1031, 0x100a, 0x1004, 0x103a, 0x1014, 0x103a, 0x1000, 0x1031, 0x1019, 0x1014, 0x103a,
0x1001, 0x103b, 0x102e, 0x1000, 0x102c, 0x1000, 0x102d, 0x102f, 0x1001, 0x103b, 0x102e, 0x101f,
0x1030, 0x1021, 0x102c, 0x101f, 0x1030, 0x1021, 0x102c, 0x1005, 0x102e, 0x101a, 0x1030, 0x1012,
0x1010, 0x103a, 0x1005, 0x103a, 0x20, 0x101f, 0x103d, 0x102c, 0x101b, 0x1000, 0x103a, 0x1005,
0x103a, 0x1000, 0x102d, 0x102f, 0x1037, 0x1005, 0x1010, 0x102c, 0x101b, 0x102e, 0x1000, 0x102c,
0x1000, 0x101b, 0x1005, 0x103a, 0x1005, 0x1010, 0x103d, 0x1014, 0x103a, 0x1000, 0x103d, 0x102e,
0x101b, 0x102c, 0x1018, 0x102c, 0x1000, 0x103b, 0x1030, 0x101b, 0x1031, 0x1038, 0x1000, 0x102d,
0x102f, 0x1038, 0x1012, 0x1014, 0x103a, 0x1019, 0x1010, 0x103a, 0x101b, 0x103e, 0x103a, 0x1017,
0x102c, 0x1014, 0x103a, 0x1012, 0x1031, 0x102b, 0x103a, 0x1006, 0x1014, 0x103a, 0x1012, 0x1031,
0x102b, 0x103a, 0x1006, 0x1014, 0x103a, 0x20, 0x1001, 0x101b, 0x102d, 0x1001, 0x103a, 0x1012,
0x1004, 0x103a, 0x1014, 0x103a, 0x1017, 0x102c, 0x1012, 0x1000, 0x103a, 0x1011, 0x101b, 0x103d,
0x102d, 0x102f, 0x1000, 0x103a, 0x1012, 0x102d, 0x102f, 0x1019, 0x102e, 0x1014, 0x102e, 0x1000,
0x102c, 0x1021, 0x1000, 0x103a, 0x1012, 0x103a, 0x1019, 0x103d, 0x1014, 0x103a, 0x1010, 0x1014,
0x103a, 0x1021, 0x102e, 0x101b, 0x1030, 0x1014, 0x102e, 0x1015, 0x102e, 0x1021, 0x101a, 0x103a,
0x101c, 0x103a, 0x1006, 0x102c, 0x1017, 0x1031, 0x1012, 0x102d, 0x102f, 0x1016, 0x102d, 0x102f,
0x1037, 0x1010, 0x103a, 0x20, 0x1014, 0x101a, 0x103a, 0x101c, 0x103a, 0x1006, 0x1004, 0x103a,
0x1016, 0x102d, 0x102f, 0x1037, 0x1010, 0x103a, 0x1010, 0x102c, 0x101c, 0x102e, 0x1007, 0x102c,
0x1002, 0x101c, 0x1032, 0x1005, 0x103a, 0x1018, 0x1031, 0x1038, 0x1002, 0x1030, 0x1038, 0x1005,
0x103a, 0x1018, 0x1031, 0x1038, 0x1002, 0x101b, 0x1014, 0x103a, 0x1012, 0x103a, 0x20, 0x1010,
0x1001, 0x103a, 0x1002, 0x101b, 0x102e, 0x1014, 0x1031, 0x1012, 0x102b, 0x1002, 0x103d, 0x102b,
0x1012, 0x102e, 0x101c, 0x102f, 0x1015, 0x103a, 0x1002, 0x103d, 0x102b, 0x1010, 0x102e, 0x1019,
0x102c, 0x101c, 0x102c, 0x1002, 0x103d, 0x102b, 0x101b, 0x102c, 0x1000, 0x103d, 0x102e, 0x1038,
0x101c, 0x103a, 0x1002, 0x102d, 0x102f, 0x1004, 0x103a, 0x101a, 0x102c, 0x1014, 0x102c, 0x101f,
0x101c, 0x102e, 0x1016, 0x1000, 0x103a, 0x1005, 0x103a, 0x101f, 0x102c, 0x1017, 0x102c, 0x1014,
0x102c, 0x101f, 0x102c, 0x1019, 0x102d, 0x102f, 0x1005, 0x103a, 0x1005, 0x102e, 0x101c, 0x102d,
0x102f, 0x1021, 0x1004, 0x103a, 0x1012, 0x102e, 0x101a, 0x102c, 0x1038, 0x1014, 0x102c, 0x1038,
0x20, 0x1015, 0x102d, 0x102f, 0x101c, 0x102d, 0x1005, 0x103a, 0x1014, 0x1031, 0x102c, 0x1000,
0x103a, 0x1001, 0x103a, 0x1005, 0x103a, 0x104a, 0x20, 0x1021, 0x1004, 0x103a, 0x1012, 0x102e,
0x101a, 0x102c, 0x1038, 0x1014, 0x102c, 0x1038, 0x1019, 0x102c, 0x101b, 0x1014, 0x103a, 0x1002,
0x102d, 0x102f, 0x104a, 0x20, 0x1021, 0x1004, 0x103a, 0x1012, 0x102e, 0x101a, 0x102c, 0x1038,
0x1014, 0x102c, 0x1038, 0x1015, 0x102e, 0x1010, 0x102c, 0x1005, 0x103a, 0x1018, 0x1010, 0x103a,
0x1001, 0x103a, 0x104a, 0x20, 0x1021, 0x1004, 0x103a, 0x1012, 0x102e, 0x101a, 0x102c, 0x1038,
0x1014, 0x102c, 0x1038, 0x1010, 0x1032, 0x101c, 0x103a, 0x1005, 0x102e, 0x1038, 0x1010, 0x102e,
0x1038, 0x104a, 0x20, 0x1021, 0x1004, 0x103a, 0x1012, 0x102e, 0x101a, 0x102c, 0x1038, 0x1014,
0x102c, 0x1038, 0x1017, 0x102e, 0x1017, 0x1032, 0x104a, 0x20, 0x1021, 0x1004, 0x103a, 0x1012,
0x102e, 0x101a, 0x102c, 0x1038, 0x1014, 0x102c, 0x1038, 0x1017, 0x1004, 0x103a, 0x1006, 0x1004,
0x1037, 0x103a, 0x1005, 0x103a, 0x104a, 0x20, 0x1021, 0x1004, 0x103a, 0x1012, 0x102e, 0x101a,
0x102c, 0x1038, 0x1014, 0x102c, 0x1038, 0x101d, 0x102e, 0x1014, 0x102c, 0x1019, 0x1000, 0x103a,
0x1001, 0x103a, 0x104a, 0x20, 0x1021, 0x1004, 0x103a, 0x1012, 0x102e, 0x101a, 0x102c, 0x1038,
0x1014, 0x102c, 0x1038, 0x1021, 0x102e, 0x1014, 0x102f, 0x1017, 0x102e, 0x1001, 0x103a, 0x1021,
0x102e, 0x1000, 0x102c, 0x101c, 0x1030, 0x1021, 0x102e, 0x1010, 0x103a, 0x1002, 0x103b, 0x1019,
0x1031, 0x1000, 0x102c, 0x1002, 0x103b, 0x102f, 0x1014, 0x102d, 0x102f, 0x1021, 0x1031, 0x102c,
0x101c, 0x1030, 0x101d, 0x102e, 0x1017, 0x102e, 0x1038, 0x101c, 0x103a, 0x1019, 0x103d, 0x1014,
0x103a, 0x1010, 0x102e, 0x1001, 0x103b, 0x101a, 0x103a, 0x101c, 0x102d, 0x102f, 0x104a, 0x20,
0x1000, 0x1004, 0x103a, 0x1010, 0x1015, 0x103a, 0x1000, 0x102e, 0x1001, 0x101b, 0x102c, 0x101c,
0x1032, 0x1014, 0x103a, 0x1038, 0x1012, 0x1005, 0x103a, 0x1001, 0x103b, 0x103a, 0x101c, 0x102c,
0x1015, 0x102b, 0x1007, 0x103a, 0x101c, 0x102e, 0x1019, 0x102c, 0x101c, 0x1031, 0x102c, 0x1037,
0x1005, 0x103a, 0x1021, 0x102d, 0x1014, 0x103a, 0x1002, 0x103b, 0x101a, 0x103a, 0x101c, 0x102d,
0x1005, 0x103a, 0x101c, 0x102d, 0x102f, 0x1021, 0x102c, 0x1015, 0x101b, 0x1004, 0x1037, 0x103a,
0x1005, 0x103a, 0x20, 0x1000, 0x103d, 0x102c, 0x1010, 0x102c, 0x1019, 0x102c, 0x1006, 0x1032,
0x101e, 0x103d, 0x102c, 0x1019, 0x102c, 0x1014, 0x102c, 0x1002, 0x103d, 0x102b, 0x1019, 0x102c,
0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x1005, 0x103a, 0x1019, 0x102c, 0x101b, 0x102e, 0x1002,
0x1031, 0x102b, 0x1037, 0x1019, 0x102c, 0x1010, 0x102e, 0x1014, 0x102e, 0x1002, 0x103a, 0x1019,
0x102c, 0x1010, 0x102c, 0x1019, 0x102d, 0x102f, 0x101b, 0x102d, 0x102f, 0x1005, 0x103a, 0x1019,
0x102c, 0x1007, 0x1010, 0x103a, 0x101c, 0x1014, 0x103a, 0x1019, 0x102e, 0x1014, 0x102d, 0x102f,
0x1019, 0x102e, 0x1014, 0x102e, 0x1038, 0x1019, 0x102e, 0x101b, 0x102e, 0x1012, 0x102b, 0x1019,
0x1000, 0x103a, 0x1010, 0x103a, 0x101c, 0x102c, 0x1000, 0x1000, 0x103a, 0x101c, 0x102c, 0x1019,
0x1000, 0x103a, 0x1000, 0x1006, 0x102e, 0x1000, 0x102d, 0x102f, 0x20, 0x1005, 0x102e, 0x1038,
0x1010, 0x102e, 0x1038, 0x1019, 0x102e, 0x1000, 0x103d, 0x102e, 0x101c, 0x103d, 0x1014, 0x103a,
0x1019, 0x103d, 0x1014, 0x103a, 0x1001, 0x103a, 0x1010, 0x103d, 0x1014, 0x103a, 0x1019, 0x103d,
0x1014, 0x103a, 0x1010, 0x1032, 0x101b, 0x1031, 0x1038, 0x1019, 0x103d, 0x1014, 0x103a, 0x1010,
0x102e, 0x1017, 0x102e, 0x1012, 0x102e, 0x1021, 0x102d, 0x102f, 0x1019, 0x103d, 0x1014, 0x1037,
0x103a, 0x28, 0x1010, 0x103a, 0x29, 0x1006, 0x1031, 0x1038, 0x101b, 0x1010, 0x103a, 0x1014,
0x102c, 0x1037, 0x1006, 0x1031, 0x102c, 0x103a, 0x1014, 0x101a, 0x1030, 0x1038, 0x101a, 0x1031,
0x102c, 0x1000, 0x103a, 0x1014, 0x102d, 0x102f, 0x101b, 0x102d, 0x102f, 0x100a, 0x102c, 0x1017,
0x103c, 0x1030, 0x101c, 0x102c, 0x104a, 0x20, 0x1019, 0x103c, 0x1031, 0x102c, 0x1000, 0x103a,
0x1012, 0x102b, 0x1000, 0x102d, 0x102f, 0x1010, 0x102c, 0x1005, 0x1004, 0x103a, 0x1010, 0x102c,
0x104a, 0x20, 0x1019, 0x103c, 0x1031, 0x102c, 0x1000, 0x103a, 0x1012, 0x102b, 0x1000, 0x102d,
0x102f, 0x1010, 0x102c, 0x1014, 0x101a, 0x1030, 0x1038, 0x1006, 0x1031, 0x1038, 0x101c, 0x1019,
0x103a, 0x104a, 0x20, 0x1019, 0x103c, 0x1031, 0x102c, 0x1000, 0x103a, 0x1012, 0x102b, 0x1000,
0x102d, 0x102f, 0x1010, 0x102c, 0x1014, 0x102f, 0x1001, 0x103a, 0x1021, 0x102d, 0x102f, 0x1001,
0x102e, 0x1014, 0x1002, 0x102b, 0x1015, 0x1014, 0x102c, 0x1038, 0x1019, 0x102c, 0x1038, 0x1015,
0x102b, 0x101b, 0x102c, 0x1019, 0x102c, 0x101b, 0x102e, 0x1018, 0x102d, 0x102f, 0x1016, 0x102e,
0x1038, 0x1014, 0x1005, 0x103a, 0x1015, 0x102d, 0x102f, 0x1037, 0x1010, 0x103a, 0x2d, 0x1021,
0x102d, 0x102f, 0x2d, 0x1015, 0x101b, 0x1004, 0x1037, 0x103a, 0x1005, 0x103a, 0x1015, 0x102d,
0x102f, 0x1037, 0x1010, 0x103a, 0x20, 0x1021, 0x1031, 0x102c, 0x1037, 0x1016, 0x103a, 0x20,
0x1005, 0x1015, 0x102d, 0x1014, 0x103a, 0x1015, 0x102d, 0x102f, 0x1037, 0x1010, 0x103a, 0x1010,
0x102d, 0x102f, 0x20, 0x1017, 0x1032, 0x101c, 0x102e, 0x101a, 0x102d, 0x102f, 0x1015, 0x1031,
0x102b, 0x103a, 0x1010, 0x102d, 0x102f, 0x101b, 0x102e, 0x1000, 0x102d, 0x102f, 0x1015, 0x103d,
0x1014, 0x103a, 0x1010, 0x102c, 0x20, 0x1021, 0x101b, 0x102e, 0x1014, 0x102c, 0x1005, 0x103a,
0x101b, 0x1014, 0x103a, 0x1000, 0x1004, 0x103a, 0x20, 0x1021, 0x1004, 0x103a, 0x1038, 0x101c,
0x1000, 0x103a, 0x101f, 0x1031, 0x1005, 0x102e, 0x1016, 0x102e, 0x101c, 0x103a, 0x101b, 0x101a,
0x103a, 0x1002, 0x103b, 0x102e, 0x1014, 0x102c, 0x101b, 0x102e, 0x1006, 0x102d, 0x102f, 0x101c,
0x102f, 0x1015, 0x103a, 0x28, 0x1010, 0x103a, 0x29, 0x101b, 0x102e, 0x101a, 0x102d, 0x102f,
0x20, 0x1018, 0x101b, 0x1014, 0x103a, 0x1000, 0x102d, 0x102f, 0x1006, 0x1014, 0x103a, 0x1010,
0x102c, 0x101b, 0x1019, 0x103a, 0x1006, 0x1014, 0x103a, 0x1010, 0x102e, 0x1021, 0x102c, 0x1002,
0x102d, 0x102f, 0x1006, 0x1014, 0x103a, 0x1010, 0x102d, 0x102f, 0x20, 0x1012, 0x102d, 0x102f,
0x1019, 0x1004, 0x103a, 0x1038, 0x1002, 0x102d, 0x102f, 0x1006, 0x1031, 0x102c, 0x103a, 0x1015,
0x102d, 0x102f, 0x101c, 0x102d, 0x102f, 0x1021, 0x1005, 0x103a, 0x1010, 0x102c, 0x1000, 0x102c,
0x20, 0x1010, 0x102d, 0x102f, 0x1038, 0x1019, 0x102d, 0x101b, 0x1000, 0x103a, 0x1010, 0x103a,
0x1005, 0x1005, 0x103a, 0x1000, 0x102c, 0x1005, 0x102d, 0x1014, 0x1037, 0x103a, 0x1018, 0x102c,
0x101e, 0x101a, 0x103a, 0x101c, 0x1019, 0x102e, 0x1005, 0x102d, 0x1014, 0x1037, 0x103a, 0x1002,
0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x1005, 0x102d, 0x1014, 0x1037, 0x103a, 0x1000, 0x102d,
0x1005, 0x103a, 0x1005, 0x102d, 0x1014, 0x1037, 0x103a, 0x101c, 0x1030, 0x1005, 0x102e, 0x101a,
0x102c, 0x1005, 0x102d, 0x1014, 0x1037, 0x103a, 0x101e, 0x1031, 0x102c, 0x1019, 0x1010, 0x103a,
0x1005, 0x103a, 0x1005, 0x102d, 0x1014, 0x1037, 0x103a, 0x1017, 0x1004, 0x103a, 0x1038, 0x1006,
0x1004, 0x1037, 0x103a, 0x1005, 0x103d, 0x1010, 0x103a, 0x1016, 0x103a, 0x1010, 0x103a, 0x20,
0x1000, 0x102c, 0x1038, 0x101b, 0x1004, 0x1037, 0x103a, 0x1010, 0x102e, 0x1002, 0x1030, 0x1005,
0x102e, 0x1002, 0x101c, 0x103a, 0x1015, 0x102b, 0x101e, 0x102f, 0x101c, 0x102e, 0x1010, 0x102e,
0x1002, 0x103d, 0x102b, 0x1014, 0x102c, 0x1010, 0x102d, 0x102f, 0x101b, 0x1014, 0x103a, 0x1010,
0x102d, 0x102f, 0x1010, 0x1031, 0x102c, 0x1010, 0x102d, 0x102f, 0x101c, 0x102c, 0x1017, 0x1014,
0x103a, 0x1000, 0x1030, 0x1038, 0x1017, 0x102c, 0x1038, 0x101d, 0x102d, 0x102f, 0x1000, 0x103a,
0x28, 0x1010, 0x103a, 0x29, 0x101f, 0x102d, 0x102f, 0x1005, 0x103a, 0x101d, 0x102e, 0x1014,
0x102e, 0x1017, 0x1000, 0x103a, 0x1002, 0x103a, 0x101b, 0x102c, 0x1000, 0x102f, 0x1010, 0x1010,
0x103a, 0x1000, 0x1031, 0x1005, 0x102e, 0x1012, 0x1031, 0x1038, 0x1017, 0x1005, 0x103a, 0x1012,
0x1030, 0x1038, 0x1019, 0x1031, 0x102c, 0x1037, 0x1010, 0x103a, 0x20, 0x1012, 0x102b, 0x101b,
0x103a, 0x1017, 0x102e, 0x1038, 0x101c, 0x103a, 0x1019, 0x1000, 0x103a, 0x1000, 0x103d, 0x101a,
0x103a, 0x101b, 0x102e, 0x1019, 0x1031, 0x102c, 0x103a, 0x1005, 0x103d, 0x1014, 0x103a, 0x1019,
0x1000, 0x103a, 0x1019, 0x1030, 0x1012, 0x102d, 0x102f, 0x1038, 0x1015, 0x102b, 0x101c, 0x103a,
0x1019, 0x102c, 0x101b, 0x102d, 0x102f, 0x101e, 0x102e, 0x101b, 0x102c, 0x101b, 0x103e, 0x102d,
0x102f, 0x1038, 0x101d, 0x102b, 0x1011, 0x101b, 0x102d, 0x102f, 0x101c, 0x103a, 0x1017, 0x102d,
0x102f, 0x1005, 0x103a, 0x1010, 0x102d, 0x102f, 0x1001, 0x103a, 0x101c, 0x1031, 0x102c, 0x1004,
0x103a, 0x101b, 0x102e, 0x101a, 0x102c, 0x1038, 0x1018, 0x101b, 0x1036, 0x1021, 0x102c, 0x1012,
0x1004, 0x103a, 0x1021, 0x1031, 0x102c, 0x103a, 0x1019, 0x102c, 0x1010, 0x102e, 0x1021, 0x102c,
0x1019, 0x102c, 0x1014, 0x103a, 0x1038, 0x1021, 0x1014, 0x103a, 0x1021, 0x102c, 0x1012, 0x102e,
0x1021, 0x102c, 0x1021, 0x1000, 0x103a, 0x1010, 0x102c, 0x1025, 0x102f, 0x1021, 0x102c, 0x1001,
0x103b, 0x1010, 0x1030, 0x1038, 0x1018, 0x102e, 0x1021, 0x102c, 0x101b, 0x103e, 0x103a, 0x1002,
0x102b, 0x1018, 0x102c, 0x1012, 0x103a, 0x1021, 0x102c, 0x1010, 0x102e, 0x101b, 0x1021, 0x1030,
0x1018, 0x1002, 0x1039, 0x1002, 0x1012, 0x1000, 0x103a, 0x1018, 0x102c, 0x101b, 0x102d, 0x1014,
0x103a, 0x1038, 0x1018, 0x102c, 0x1000, 0x1030, 0x1018, 0x1014, 0x103a, 0x1000, 0x1031, 0x102c,
0x1000, 0x103a, 0x1018, 0x101b, 0x103a, 0x1014, 0x102c, 0x1021, 0x1030, 0x1018, 0x1031, 0x101b,
0x103d, 0x1010, 0x103a, 0x1018, 0x102e, 0x101b, 0x103e, 0x103a, 0x1001, 0x1000, 0x103a, 0x1018,
0x101b, 0x1030, 0x1014, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x1001, 0x103b, 0x102e, 0x1010,
0x102c, 0x1000, 0x102d, 0x102f, 0x101c, 0x1036, 0x1018, 0x102d, 0x102f, 0x1012, 0x1019, 0x102c,
0x1038, 0x1005, 0x1000, 0x1015, 0x103a, 0x1012, 0x1000, 0x103a, 0x1000, 0x102c, 0x1012, 0x1005,
0x103a, 0x101c, 0x102e, 0x1012, 0x1030, 0x1018, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x1012,
0x1030, 0x101b, 0x103e, 0x1014, 0x103a, 0x1038, 0x1018, 0x1032, 0x1016, 0x102c, 0x1019, 0x102c,
0x1002, 0x1030, 0x1005, 0x1010, 0x102c, 0x1002, 0x102b, 0x1007, 0x102c, 0x101f, 0x102e, 0x1018,
0x101b, 0x103d, 0x1014, 0x103a, 0x101f, 0x102d, 0x102f, 0x1001, 0x103b, 0x102e, 0x1019, 0x1004,
0x103a, 0x1038, 0x1005, 0x102e, 0x1038, 0x1010, 0x102e, 0x1038, 0x101f, 0x1031, 0x102c, 0x1004,
0x103a, 0x1000, 0x1031, 0x102c, 0x1004, 0x103a, 0x101f, 0x102d, 0x102f, 0x1038, 0x1017, 0x103b,
0x1021, 0x102e, 0x101b, 0x1030, 0x1001, 0x1030, 0x1010, 0x103a, 0x1002, 0x103b, 0x1000, 0x102c,
0x1010, 0x102c, 0x1002, 0x103b, 0x102c, 0x101b, 0x102c, 0x1015, 0x1030, 0x101b, 0x102c, 0x1002,
0x103b, 0x1031, 0x101b, 0x102f, 0x1006, 0x101c, 0x1004, 0x103a, 0x1000, 0x102c, 0x1018, 0x1030,
0x1038, 0x101c, 0x103a, 0x1001, 0x1019, 0x103a, 0x1001, 0x103b, 0x102c, 0x1037, 0x1001, 0x102b,
0x1000, 0x101b, 0x102c, 0x1001, 0x103b, 0x102d, 0x1001, 0x1010, 0x103a, 0x1010, 0x1019, 0x1014,
0x103a, 0x1012, 0x1030, 0x1001, 0x1014, 0x103a, 0x1012, 0x102d, 0x102f, 0x1004, 0x103a, 0x1002,
0x102b, 0x1000, 0x102d, 0x102f, 0x101c, 0x103b, 0x1000, 0x1010, 0x103a, 0x1010, 0x102c, 0x1038,
0x1001, 0x101b, 0x102c, 0x1037, 0x1005, 0x103a, 0x1014, 0x102d, 0x102f, 0x101b, 0x102c, 0x1005,
0x103a, 0x1000, 0x103d, 0x102c, 0x101c, 0x102c, 0x101c, 0x1019, 0x103a, 0x1015, 0x1030, 0x1000,
0x1030, 0x1001, 0x103b, 0x1004, 0x103a, 0x1000, 0x1030, 0x101d, 0x102d, 0x1010, 0x103a, 0x1019,
0x1000, 0x102c, 0x1021, 0x102d, 0x102f, 0x1019, 0x102c, 0x1002, 0x102b, 0x1012, 0x1014, 0x103a,
0x1038, 0x1019, 0x1001, 0x1000, 0x103a, 0x1005, 0x103a, 0x1006, 0x102c, 0x1019, 0x1014, 0x102e,
0x101c, 0x102c, 0x1019, 0x1010, 0x103a, 0x1005, 0x103a, 0x1000, 0x1000, 0x103a, 0x1010, 0x103a,
0x1014, 0x102e, 0x1000, 0x102d, 0x102f, 0x101b, 0x103e, 0x102c, 0x1038, 0x1014, 0x102d, 0x102f,
0x1017, 0x102d, 0x102f, 0x1001, 0x1030, 0x1007, 0x103a, 0x1014, 0x1000, 0x103a, 0x1005, 0x103a,
0x1014, 0x102d, 0x102f, 0x1017, 0x102d, 0x102f, 0x1005, 0x1032, 0x1018, 0x102e, 0x1021, 0x1032,
0x101a, 0x103a, 0x1005, 0x103a, 0x1021, 0x103d, 0x1019, 0x103a, 0x1038, 0x1005, 0x103a, 0x1001,
0x103a, 0x1021, 0x1031, 0x102c, 0x103a, 0x101b, 0x1032, 0x101c, 0x103a, 0x1016, 0x1014, 0x103d,
0x1019, 0x103a, 0x1015, 0x1004, 0x103a, 0x1015, 0x103d, 0x1014, 0x103a, 0x1010, 0x102e, 0x1021,
0x102c, 0x1038, 0x1014, 0x102c, 0x1037, 0x1001, 0x103a, 0x1015, 0x103c, 0x102f, 0x1036, 0x101a,
0x1014, 0x103a, 0x1038, 0x1000, 0x1031, 0x102c, 0x1037, 0x1005, 0x103a, 0x1010, 0x1014, 0x1031,
0x1000, 0x102e, 0x1007, 0x101c, 0x1031, 0x102c, 0x103a, 0x1012, 0x102b, 0x101b, 0x102e, 0x101a,
0x102c, 0x1038, 0x1012, 0x103a, 0x1006, 0x102c, 0x1001, 0x102b, 0x101c, 0x1004, 0x103a, 0x1006,
0x1019, 0x103a, 0x1038, 0x1019, 0x102c, 0x1001, 0x1014, 0x103a, 0x1038, 0x1012, 0x103a, 0x1006,
0x102d, 0x102f, 0x1038, 0x101c, 0x103a, 0x101b, 0x103e, 0x1014, 0x103a, 0x101f, 0x102d, 0x102f,
0x1004, 0x103a, 0x1038, 0x1005, 0x1004, 0x103a, 0x1039, 0x1000, 0x102c, 0x1015, 0x1030, 0x1006,
0x101b, 0x1005, 0x103a, 0x1014, 0x1005, 0x103a, 0x1000, 0x102c, 0x101c, 0x1004, 0x103a, 0x1019,
0x103a, 0x1005, 0x103a, 0x1010, 0x102d, 0x102f, 0x1004, 0x103a, 0x1015, 0x1031, 0x1010, 0x102c,
0x101b, 0x103e, 0x103a, 0x1000, 0x1032, 0x1014, 0x1037, 0x103a, 0x1010, 0x1018, 0x102e, 0x1038,
0x101c, 0x102e, 0x1005, 0x102e, 0x1010, 0x102e, 0x101f, 0x102e, 0x101b, 0x1014, 0x103a, 0x1010,
0x1004, 0x103a, 0x1016, 0x1030, 0x1038, 0x1010, 0x102d, 0x102f, 0x1000, 0x103b, 0x102d, 0x102f,
0x1010, 0x103d, 0x1014, 0x103a, 0x1019, 0x103a, 0x1005, 0x103a, 0x1001, 0x103a, 0x1025, 0x101c,
0x1014, 0x103a, 0x1018, 0x102c, 0x1010, 0x1031, 0x102c, 0x1021, 0x1030, 0x101b, 0x102f, 0x1019,
0x103a, 0x1001, 0x103b, 0x102e, 0x1021, 0x1030, 0x1038, 0x1005, 0x103a, 0x20, 0x1014, 0x102e,
0x101b, 0x102c, 0x1038, 0x1017, 0x102e, 0x101a, 0x1004, 0x103a, 0x1000, 0x103b, 0x1014, 0x103a,
0x1038, 0x1017, 0x101c, 0x102c, 0x1012, 0x102e, 0x1017, 0x1031, 0x102c, 0x1037, 0x1005, 0x1010,
0x1031, 0x102c, 0x1037, 0x1001, 0x103a, 0x101a, 0x1030, 0x1001, 0x1030, 0x1038, 0x1010, 0x103a,
0x1005, 0x103a, 0x101b, 0x1014, 0x103a, 0x1000, 0x102f, 0x1014, 0x103a, 0x101b, 0x101a, 0x103a,
0x1001, 0x102b, 0x1038, 0x1010, 0x102e, 0x101b, 0x1004, 0x103a, 0x1018, 0x102c, 0x1038, 0x1001,
0x103a, 0x101b, 0x1032, 0x101a, 0x103a, 0x101b, 0x1031, 0x1017, 0x1014, 0x103a, 0x1038, 0x1021,
0x1031, 0x1007, 0x102d, 0x102f, 0x1038, 0x101b, 0x102e, 0x1038, 0x1005, 0x103a, 0x1018, 0x102c,
0x1019, 0x103c, 0x1030, 0x1012, 0x102b, 0x1000, 0x1014, 0x1031, 0x101b, 0x102e, 0x1000, 0x102d,
0x1010, 0x103a, 0x1015, 0x103a, 0x20, 0x1017, 0x102c, 0x1012, 0x102e, 0x1016, 0x102b, 0x101b,
0x102d, 0x102f, 0x1019, 0x1012, 0x102e, 0x1038, 0x101b, 0x102c, 0x101b, 0x1031, 0x1038, 0x1000,
0x102e, 0x1017, 0x1005, 0x103a, 0x1001, 0x103a, 0x1010, 0x1031, 0x102c, 0x1004, 0x103a, 0x1002,
0x103b, 0x1031, 0x102c, 0x103a, 0x1002, 0x103b, 0x102e, 0x101a, 0x102c, 0x1005, 0x102d, 0x1014,
0x1037, 0x103a, 0x101f, 0x101a, 0x103a, 0x101c, 0x101a, 0x103a, 0x1014, 0x102c, 0x1005, 0x1010,
0x1014, 0x103a, 0x101c, 0x1031, 0x1021, 0x1014, 0x1039, 0x1012, 0x102e, 0x101c, 0x102d, 0x1010,
0x103a, 0x1012, 0x103a, 0x1018, 0x101b, 0x1005, 0x1039, 0x1005, 0x1018, 0x102d, 0x1014, 0x103a,
0x1038, 0x1018, 0x101b, 0x102d, 0x102f, 0x1000, 0x1004, 0x103a, 0x101f, 0x102e, 0x1038, 0x101c,
0x103a, 0x1012, 0x102b, 0x101d, 0x1004, 0x103a, 0x101a, 0x1030, 0x1038, 0x1001, 0x101c, 0x102c,
0x101f, 0x102d, 0x102f, 0x1038, 0x1018, 0x102c, 0x1038, 0x1010, 0x103a, 0x101c, 0x1004, 0x103a,
0x1038, 0x1012, 0x102e, 0x1019, 0x1014, 0x103a, 0x1038, 0x101c, 0x1031, 0x102c, 0x1037, 0x1012,
0x103a, 0x20, 0x101f, 0x1031, 0x102c, 0x1004, 0x103a, 0x1019, 0x1032, 0x101c, 0x103a, 0x1018,
0x102f, 0x1014, 0x103a, 0x1038, 0x1015, 0x102b, 0x1038, 0x101e, 0x103a, 0x1006, 0x1005, 0x103a,
0x1012, 0x1014, 0x102e, 0x1021, 0x1019, 0x103a, 0x1005, 0x1010, 0x102c, 0x1012, 0x1019, 0x103a,
0x1021, 0x1014, 0x103a, 0x1012, 0x102d, 0x102f, 0x101b, 0x102c, 0x1021, 0x102c, 0x1038, 0x1005,
0x103a, 0x1010, 0x101b, 0x1001, 0x1014, 0x103a, 0x1038, 0x1021, 0x1031, 0x101e, 0x1004, 0x103a,
0x1018, 0x1032, 0x101c, 0x103a, 0x1002, 0x101b, 0x102d, 0x1010, 0x103a, 0x1018, 0x102c, 0x101c,
0x1004, 0x103a, 0x1018, 0x101b, 0x102c, 0x1010, 0x102e, 0x1038, 0x1005, 0x103a, 0x101c, 0x102c,
0x1038, 0x1017, 0x102c, 0x1018, 0x101b, 0x1015, 0x103a, 0x1006, 0x1032, 0x101c, 0x103a, 0x1018,
0x1030, 0x1001, 0x102b, 0x101b, 0x1000, 0x103a, 0x1005, 0x103a, 0x1018, 0x1030, 0x1012, 0x102b,
0x1015, 0x1000, 0x103a, 0x1005, 0x103a, 0x1018, 0x1030, 0x101b, 0x103e, 0x1004, 0x103a, 0x1002,
0x103b, 0x1004, 0x103a, 0x1001, 0x103b, 0x102e, 0x101b, 0x103e, 0x102e, 0x1014, 0x102c, 0x1038,
0x1021, 0x1030, 0x1000, 0x102d, 0x102f, 0x1015, 0x1004, 0x103a, 0x101f, 0x1031, 0x1002, 0x1004,
0x103a, 0x1012, 0x1010, 0x103a, 0x1018, 0x101c, 0x1004, 0x103a, 0x1002, 0x103b, 0x102e, 0x1018,
0x101b, 0x1031, 0x102c, 0x103a, 0x101c, 0x103a, 0x1010, 0x102c, 0x1002, 0x103d, 0x1014, 0x103a,
0x1038, 0x1007, 0x102e, 0x101f, 0x1032, 0x101c, 0x103a, 0x1005, 0x1004, 0x103a, 0x1000, 0x102e,
0x1019, 0x1014, 0x103a, 0x1038, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x1021, 0x1005,
0x1039, 0x1005, 0x1010, 0x1014, 0x103a, 0x1018, 0x1030, 0x101c, 0x103a, 0x1002, 0x103b, 0x102c,
0x1005, 0x102e, 0x1001, 0x102b, 0x101c, 0x102e, 0x1014, 0x1004, 0x103a, 0x1002, 0x101b, 0x1000,
0x103a, 0x1001, 0x102e, 0x101b, 0x102d, 0x102f, 0x1017, 0x103a, 0x1001, 0x102e, 0x1038, 0x1021,
0x1000, 0x103a, 0x1016, 0x103a, 0x101c, 0x1005, 0x1039, 0x1005, 0x1018, 0x103d, 0x1014, 0x103a,
0x1038, 0x101c, 0x1030, 0x1018, 0x101c, 0x102e, 0x1021, 0x102c, 0x1038, 0x1014, 0x102c, 0x1038,
0x101c, 0x1014, 0x103a, 0x1012, 0x1014, 0x103a, 0x101c, 0x1030, 0x1007, 0x1004, 0x103a, 0x1018,
0x1010, 0x103a, 0x1019, 0x1012, 0x101b, 0x1005, 0x103a, 0x1019, 0x1031, 0x102c, 0x103a, 0x101c,
0x103a, 0x1010, 0x102c, 0x1019, 0x101b, 0x102e, 0x1021, 0x102c, 0x20, 0x101f, 0x102c, 0x1038,
0x1019, 0x1014, 0x103a, 0x1019, 0x1004, 0x103a, 0x1038, 0x1005, 0x1001, 0x103a, 0x1019, 0x102d,
0x102f, 0x1014, 0x102c, 0x1000, 0x102d, 0x102f, 0x1019, 0x1031, 0x102c, 0x103a, 0x1005, 0x1000,
0x102d, 0x102f, 0x1021, 0x1031, 0x102c, 0x103a, 0x1005, 0x101c, 0x102d, 0x102f, 0x1015, 0x1032,
0x101b, 0x1005, 0x103a, 0x1015, 0x1031, 0x102b, 0x1037, 0x1002, 0x102d, 0x102f, 0x101b, 0x102e,
0x1000, 0x102c, 0x1015, 0x101b, 0x1000, 0x103a, 0x1002, 0x103a, 0x101b, 0x102e, 0x1002, 0x102b,
0x101b, 0x1031, 0x102c, 0x1019, 0x1006, 0x102c, 0x1019, 0x102c, 0x1038, 0x101b, 0x102c, 0x1006,
0x1014, 0x103a, 0x1019, 0x101b, 0x102e, 0x1038, 0x1014, 0x102d, 0x102f, 0x1006, 0x102c, 0x101b,
0x102c, 0x101a, 0x1031, 0x1017, 0x102d, 0x102f, 0x1006, 0x102c, 0x101b, 0x102c, 0x1010, 0x102d,
0x102f, 0x1037, 0x1016, 0x103a, 0x1005, 0x1004, 0x103a, 0x1016, 0x102c, 0x101b, 0x102d, 0x102f,
0x1038, 0x1016, 0x102d, 0x102f, 0x1005, 0x1000, 0x1031, 0x102c, 0x1037, 0x1015, 0x103a, 0x101b,
0x102c, 0x1006, 0x102d, 0x102f, 0x1016, 0x102e, 0x1021, 0x102c, 0x1005, 0x1010, 0x1031, 0x102c,
0x1037, 0x101f, 0x102f, 0x1019, 0x103a, 0x1038, 0x1011, 0x102c, 0x1038, 0x101c, 0x1004, 0x103a,
0x1010, 0x102e, 0x101b, 0x102c, 0x1014, 0x102e, 0x1021, 0x1030, 0x101c, 0x102e, 0x101a, 0x102c,
0x1014, 0x102d, 0x102f, 0x1005, 0x1000, 0x1005, 0x103a, 0x1016, 0x103a, 0x1017, 0x102c, 0x1012,
0x1030, 0x1038, 0x1005, 0x103a, 0x1017, 0x102c, 0x1010, 0x102e, 0x1000, 0x1014, 0x103a, 0x1017,
0x102e, 0x101a, 0x1004, 0x103a, 0x1014, 0x102c, 0x1017, 0x102e, 0x1038, 0x101c, 0x103a, 0x1014,
0x102e, 0x1021, 0x102d, 0x102f, 0x1038, 0x1005, 0x103a, 0x1017, 0x102d, 0x102f, 0x101c, 0x103a,
0x1002, 0x102d, 0x102f, 0x1002, 0x101b, 0x1000, 0x103a, 0x101d, 0x102b, 0x1006, 0x1031, 0x102c,
0x1007, 0x102c, 0x1002, 0x103a, 0x1002, 0x101b, 0x1000, 0x103a, 0x1018, 0x103a, 0x1007, 0x1030,
0x1038, 0x101b, 0x1005, 0x103a, 0x1001, 0x103b, 0x103a, 0x1021, 0x1014, 0x103a, 0x1010, 0x102c,
0x1014, 0x102c, 0x1014, 0x102c, 0x101b, 0x102e, 0x1038, 0x1018, 0x102d, 0x102f, 0x1001, 0x103b,
0x102c, 0x1002, 0x102d, 0x102f, 0x1005, 0x103a, 0x1001, 0x101b, 0x1005, 0x103a, 0x1005, 0x1019,
0x1010, 0x103a, 0x1000, 0x102d, 0x102f, 0x1000, 0x102d, 0x102f, 0x1038, 0x1005, 0x103a, 0x1000,
0x102d, 0x102f, 0x1019, 0x102d, 0x102f, 0x101b, 0x102d, 0x102f, 0x1001, 0x102b, 0x1002, 0x102b,
0x101c, 0x1014, 0x103a, 0x1019, 0x102c, 0x101f, 0x102e, 0x1019, 0x1031, 0x102c, 0x103a, 0x101c,
0x1012, 0x102d, 0x102f, 0x1000, 0x103a, 0x1019, 0x1031, 0x102c, 0x101b, 0x1005, 0x103a, 0x101b,
0x103e, 0x1019, 0x102c, 0x101a, 0x102d, 0x102f, 0x1010, 0x1032, 0x101b, 0x102e, 0x101a, 0x1030,
0x1014, 0x102e, 0x101a, 0x1014, 0x103a, 0x1021, 0x102c, 0x1038, 0x1015, 0x102e, 0x1021, 0x102c,
0x1038, 0x1021, 0x1031, 0x102c, 0x1037, 0x1000, 0x101c, 0x1014, 0x103a, 0x1018, 0x1030, 0x1002,
0x1014, 0x103a, 0x1017, 0x102e, 0x1038, 0x101c, 0x102e, 0x1038, 0x101a, 0x102c, 0x1038, 0x1001,
0x103b, 0x102c, 0x1038, 0x101e, 0x1019, 0x103a, 0x1001, 0x103b, 0x102f, 0x1001, 0x103a, 0x1021,
0x102e, 0x1005, 0x1010, 0x102c, 0x1021, 0x102e, 0x1016, 0x102c, 0x1010, 0x102e, 0x1016, 0x102c,
0x1001, 0x102b, 0x1021, 0x102d, 0x102f, 0x1016, 0x102d, 0x102f, 0x1016, 0x102e, 0x1002, 0x103b,
0x102e, 0x1016, 0x1030, 0x1014, 0x102c, 0x1016, 0x1030, 0x1010, 0x102e, 0x1002, 0x102b, 0x101c,
0x102c, 0x1015, 0x102b, 0x1000, 0x1015, 0x103a, 0x1005, 0x103a, 0x1002, 0x1019, 0x103a, 0x1018,
0x102e, 0x101a, 0x102c, 0x1002, 0x103d, 0x102b, 0x1012, 0x102b, 0x1000, 0x1014, 0x1032, 0x101c,
0x103a, 0x1002, 0x1030, 0x1021, 0x1019, 0x103a, 0x101f, 0x102d, 0x102f, 0x1014, 0x102d, 0x102f,
0x101c, 0x1030, 0x101c, 0x1030, 0x1021, 0x1014, 0x103a, 0x1012, 0x102b, 0x1018, 0x1030, 0x101b,
0x102e, 0x1001, 0x101b, 0x102d, 0x1012, 0x102e, 0x1019, 0x1010, 0x102e, 0x1001, 0x102d, 0x102f,
0x1005, 0x103a, 0x101b, 0x102d, 0x102f, 0x1004, 0x103a, 0x1001, 0x103d, 0x102c, 0x1002, 0x103b,
0x102c, 0x101c, 0x102d, 0x1014, 0x103a, 0x1019, 0x102c, 0x1002, 0x103b, 0x1030, 0x1038, 0x101b,
0x102d, 0x102f, 0x1019, 0x102c, 0x1001, 0x1031, 0x1038, 0x1021, 0x1015, 0x103a, 0x1005, 0x103a,
0x1019, 0x1005, 0x103a, 0x1012, 0x103a, 0x101d, 0x1031, 0x1038, 0x1014, 0x102c, 0x1025, 0x1030,
0x101b, 0x1030, 0x1014, 0x102e, 0x1026, 0x1038, 0x1021, 0x1031, 0x1014, 0x1031, 0x102c, 0x103a,
0x1016, 0x1031, 0x102c, 0x1037, 0x1001, 0x103a, 0x1014, 0x1030, 0x1019, 0x101a, 0x103a, 0x1021,
0x102c, 0x1015, 0x102b, 0x1002, 0x102d, 0x102f, 0x1015, 0x102b, 0x1002, 0x102d, 0x102f, 0x1015,
0x101c, 0x102c, 0x1021, 0x102d, 0x102f, 0x1015, 0x102b, 0x1010, 0x103a, 0x1000, 0x101a, 0x103a,
0x101b, 0x1004, 0x103a, 0x1016, 0x102d, 0x102f, 0x1014, 0x102c, 0x1016, 0x1032, 0x1021, 0x103a,
0x1016, 0x102d, 0x102f, 0x1037, 0x1010, 0x103a, 0x20, 0x1019, 0x102d, 0x102f, 0x101b, 0x1000,
0x103a, 0x1005, 0x103a, 0x1018, 0x102e, 0x101b, 0x102c, 0x101b, 0x102d, 0x102f, 0x1010, 0x103d,
0x1014, 0x103a, 0x1038, 0x1002, 0x102b, 0x1006, 0x102d, 0x102f, 0x1004, 0x103a, 0x1015, 0x1014,
0x103a, 0x1010, 0x101f, 0x102e, 0x1010, 0x102e, 0x1010, 0x102c, 0x101b, 0x102c, 0x101d, 0x102b,
0x1010, 0x103d, 0x1014, 0x103a, 0x1002, 0x102b, 0x1010, 0x102c, 0x1015, 0x102f, 0x101d, 0x102d,
0x1010, 0x103a, 0x1001, 0x103a, 0x101d, 0x1031, 0x102b, 0x101c, 0x102e, 0x1005, 0x103a, 0x963f,
0x6bd4, 0x8b93, 0x963f, 0x514b, 0x62c9, 0x963f, 0x8fea, 0x65af, 0x963f, 0x8c9d, 0x5df4, 0x963f,
0x723e, 0x53ca, 0x723e, 0x963f, 0x65af, 0x746a, 0x62c9, 0x5df4, 0x99ac, 0x79d1, 0x73ed, 0x5409,
0x73ed, 0x7af9, 0x6bd4, 0x7d39, 0x5e03, 0x862d, 0x592a, 0x723e, 0x5e03, 0x62c9, 0x67f4, 0x7dad,
0x723e, 0x5e03, 0x677e, 0x5e03, 0x62c9, 0x958b, 0x7f85, 0x5361, 0x85a9, 0x5e03, 0x862d, 0x5361,
0x4f11, 0x9054, 0x67ef, 0x90a3, 0x514b, 0x91cc, 0x9054, 0x5580, 0x723e, 0x6c99, 0x862d, 0x6e2f,
0x5409, 0x5e03, 0x5730, 0x675c, 0x963f, 0x62c9, 0x963f, 0x5c24, 0x6069, 0x81ea, 0x7531, 0x57ce,
0x5609, 0x67cf, 0x9686, 0x91cc, 0x54c8, 0x62c9, 0x96f7, 0x7d04, 0x7ff0, 0x5c3c, 0x65af, 0x5821,
0x6731, 0x5df4, 0x574e, 0x5e15, 0x62c9, 0x5580, 0x571f, 0x7a46, 0x57fa, 0x52a0, 0x5229, 0x91d1,
0x590f, 0x6c99, 0x62c9, 0x54e5, 0x65af, 0x81ea, 0x7531, 0x5e02, 0x6d1b, 0x7f8e, 0x7f85, 0x5b89,
0x9054, 0x76e7, 0x672c, 0x5df4, 0x5e0c, 0x8def, 0x6c99, 0x5361, 0x99ac, 0x62c9, 0x535a, 0x99ac,
0x666e, 0x6258, 0x99ac, 0x8cfd, 0x9b6f, 0x58a8, 0x5df4, 0x672c, 0x6469, 0x52a0, 0x8fea, 0x4f11,
0x8499, 0x7f85, 0x7dad, 0x4e9e, 0x5948, 0x6d1b, 0x6bd4, 0x6069, 0x8cc8, 0x6885, 0x7d0d, 0x5c3c,
0x4e9e, 0x7f8e, 0x8afe, 0x514b, 0x5c11, 0x74e6, 0x52a0, 0x675c, 0x53e4, 0x6ce2, 0x591a, 0x8afe,
0x4f5b, 0x8056, 0x591a, 0x7f8e, 0x7684, 0x9ece, 0x6ce2, 0x91cc, 0x7a81, 0x5c3c, 0x65af, 0x6eab,
0x5f97, 0x548c, 0x514b, 0x827e, 0x9054, 0x514b, 0x5b89, 0x514b, 0x62c9, 0x6cbb, 0x5b89, 0x5409,
0x62c9, 0x5b89, 0x5730, 0x5361, 0x963f, 0x62c9, 0x572d, 0x90a3, 0x5e03, 0x5b9c, 0x8afe, 0x65af,
0x827e, 0x5229, 0x65af, 0x5361, 0x5854, 0x99ac, 0x5361, 0x54e5, 0x591a, 0x83ef, 0x80e1, 0x80e1,
0x4f0a, 0x62c9, 0x7565, 0x54c8, 0x9580, 0x591a, 0x85a9, 0x91cc, 0x5967, 0x52a0, 0x8036, 0x6208,
0x65af, 0x85a9, 0x723e, 0x5854, 0x8056, 0x80e1, 0x5b89, 0x8056, 0x8def, 0x6613, 0x5410, 0x5eab,
0x66fc, 0x70cf, 0x65af, 0x61f7, 0x4e9e, 0x963f, 0x8def, 0x5df4, 0x4e9e, 0x677e, 0x68ee, 0x963f,
0x8482, 0x79d1, 0x80af, 0x5df4, 0x4f0a, 0x963f, 0x5df4, 0x4f0a, 0x4e9e, 0x73ed, 0x5fb7, 0x62c9,
0x65af, 0x5df4, 0x8c9d, 0x591a, 0x8c9d, 0x502b, 0x8c9d, 0x91cc, 0x65af, 0x767d, 0x6717, 0x85a9,
0x5e03, 0x9686, 0x4fdd, 0x7dad, 0x65af, 0x5854, 0x6ce2, 0x54e5, 0x5927, 0x6ce2, 0x5915, 0x528d,
0x6a4b, 0x7063, 0x683c, 0x862d, 0x5834, 0x574e, 0x6606, 0x5361, 0x62c9, 0x5361, 0x65af, 0x958b,
0x96f2, 0x958b, 0x66fc, 0x7fa4, 0x5cf6, 0x829d, 0x52a0, 0x54e5, 0x5947, 0x83ef, 0x83ef, 0x83ef,
0x96f7, 0x65af, 0x57ce, 0x54e5, 0x65af, 0x5927, 0x9ece, 0x52a0, 0x514b, 0x96f7, 0x65af, 0x9813,
0x53e4, 0x96c5, 0x5df4, 0x5eab, 0x62c9, 0x7d22, 0x4e39, 0x99ac, 0x6c99, 0x6587, 0x9053, 0x68ee,
0x9053, 0x68ee, 0x514b, 0x91cc, 0x514b, 0x4e39, 0x4f5b, 0x5e95, 0x7279, 0x5f8b, 0x591a, 0x660e,
0x5c3c, 0x52a0, 0x827e, 0x5fb7, 0x8499, 0x541e, 0x827e, 0x9b6f, 0x5167, 0x4f69, 0x85a9, 0x723e,
0x74e6, 0x591a, 0x7d0d, 0x723e, 0x905c, 0x5821, 0x798f, 0x5854, 0x529b, 0x838e, 0x683c, 0x96f7,
0x65af, 0x8c9d, 0x9d5d, 0x7063, 0x5927, 0x7279, 0x514b, 0x5cf6, 0x683c, 0x745e, 0x7d0d, 0x9054,
0x74dc, 0x5730, 0x6d1b, 0x666e, 0x74dc, 0x5730, 0x99ac, 0x62c9, 0x74dc, 0x4e9e, 0x57fa, 0x723e,
0x84cb, 0x4e9e, 0x90a3, 0x54c8, 0x91cc, 0x6cd5, 0x514b, 0x65af, 0x54c8, 0x74e6, 0x90a3, 0x57c3,
0x83ab, 0x897f, 0x7d04, 0x5370, 0x7b2c, 0x5b89, 0x90a3, 0x6ce2, 0x91cc, 0x65af, 0x5370, 0x7b2c,
0x5b89, 0x90a3, 0x5dde, 0x8afe, 0x514b, 0x65af, 0x5370, 0x7b2c, 0x5b89, 0x90a3, 0x5dde, 0x99ac,
0x502b, 0x54e5, 0x5370, 0x7b2c, 0x5b89, 0x90a3, 0x5dde, 0x5f7c, 0x5f97, 0x5821, 0x5370, 0x7b2c,
0x5b89, 0x90a3, 0x5dde, 0x6cf0, 0x723e, 0x57ce, 0x5370, 0x7b2c, 0x5b89, 0x90a3, 0x5dde, 0x7dad,
0x5a01, 0x5370, 0x7b2c, 0x5b89, 0x90a3, 0x5dde, 0x6eab, 0x68ee, 0x65af, 0x5370, 0x7b2c, 0x5b89,
0x90a3, 0x5dde, 0x5a01, 0x7d0d, 0x99ac, 0x514b, 0x4f0a, 0x5974, 0x7dad, 0x514b, 0x4f0a, 0x9b41,
0x7279, 0x7259, 0x8cb7, 0x52a0, 0x6731, 0x8afe, 0x8def, 0x6613, 0x65af, 0x7dad, 0x723e, 0x80af,
0x5854, 0x57fa, 0x5dde, 0x8499, 0x5730, 0x537b, 0x7f85, 0x514b, 0x62c9, 0x502b, 0x4ee3, 0x514b,
0x62c9, 0x5df4, 0x65af, 0x5229, 0x99ac, 0x6d1b, 0x6749, 0x78ef, 0x4e0b, 0x592a, 0x5b50, 0x5340,
0x99ac, 0x745f, 0x6b50, 0x99ac, 0x62ff, 0x74dc, 0x746a, 0x7459, 0x65af, 0x99ac, 0x91cc, 0x6208,
0x7279, 0x99ac, 0x4e01, 0x5c3c, 0x514b, 0x99ac, 0x5854, 0x83ab, 0x7f85, 0x65af, 0x99ac, 0x85a9,
0x7279, 0x862d, 0x7f8e, 0x8afe, 0x7c73, 0x5c3c, 0x6885, 0x91cc, 0x9054, 0x6885, 0x7279, 0x62c9,
0x5361, 0x7279, 0x62c9, 0x58a8, 0x897f, 0x54e5, 0x5e02, 0x5bc6, 0x555f, 0x5d19, 0x8499, 0x514b,
0x9813, 0x8499, 0x7279, 0x745e, 0x8499, 0x7279, 0x7dad, 0x591a, 0x8499, 0x54f2, 0x81d8, 0x62ff,
0x7d22, 0x7d10, 0x7d04, 0x8afe, 0x59c6, 0x8afe, 0x502b, 0x54c8, 0x5317, 0x9054, 0x79d1, 0x4ed6,
0x5dde, 0x5e03, 0x7531, 0x62c9, 0x5317, 0x9054, 0x79d1, 0x4ed6, 0x5dde, 0x4e2d, 0x5fc3, 0x5317,
0x9054, 0x79d1, 0x4ed6, 0x5dde, 0x7d10, 0x6c99, 0x502b, 0x52aa, 0x514b, 0x5967, 0x5e0c, 0x7d0d,
0x52a0, 0x5df4, 0x62ff, 0x99ac, 0x5df4, 0x62c9, 0x99ac, 0x5229, 0x6ce2, 0x9cf3, 0x51f0, 0x57ce,
0x592a, 0x5b50, 0x6e2f, 0x897f, 0x73ed, 0x7259, 0x6e2f, 0x7dad, 0x7559, 0x6e2f, 0x6ce2, 0x591a,
0x9ece, 0x5404, 0x84ec, 0x5854, 0x963f, 0x96f7, 0x7d0d, 0x65af, 0x862d, 0x4eca, 0x7063, 0x96f7,
0x897f, 0x975e, 0x91cc, 0x8cc8, 0x7d0d, 0x7f85, 0x65af, 0x9b6f, 0x7279, 0x91cc, 0x7d04, 0x5e03,
0x862d, 0x8056, 0x5854, 0x502b, 0x8056, 0x5730, 0x7259, 0x54e5, 0x8056, 0x591a, 0x660e, 0x54e5,
0x8056, 0x4fdd, 0x7f85, 0x4f0a, 0x6258, 0x79d1, 0x723e, 0x6258, 0x7c73, 0x7279, 0x932b, 0x7279,
0x5361, 0x8056, 0x5df4, 0x6258, 0x6d1b, 0x7e46, 0x5cf6, 0x8056, 0x7d04, 0x7ff0, 0x8056, 0x57fa,
0x8328, 0x8056, 0x9732, 0x897f, 0x4e9e, 0x8056, 0x6258, 0x99ac, 0x65af, 0x8056, 0x6587, 0x68ee,
0x65af, 0x5a01, 0x592b, 0x7279, 0x5361, 0x502b, 0x7279, 0x5fb7, 0x53e4, 0x65af, 0x52a0, 0x5df4,
0x675c, 0x91cc, 0x63d0, 0x83ef, 0x7d0d, 0x591a, 0x502b, 0x591a, 0x6258, 0x723e, 0x6258, 0x62c9,
0x6eab, 0x54e5, 0x83ef, 0x61f7, 0x7279, 0x970d, 0x65af, 0x6eab, 0x5c3c, 0x4f2f, 0x96c5, 0x5eab,
0x5854, 0x51f1, 0x897f, 0x6234, 0x7dad, 0x65af, 0x675c, 0x8499, 0x675c, 0x6bd4, 0x723e, 0x9ea5,
0x89ba, 0x7406, 0x83ab, 0x68ee, 0x9ea5, 0x514b, 0x9ed8, 0x591a, 0x5e15, 0x9ea5, 0x7f85, 0x745f,
0x62c9, 0x662d, 0x548c, 0x57fa, 0x5730, 0x7dbd, 0x723e, 0x6c83, 0x65af, 0x6258, 0x514b, 0x9686,
0x610f, 0x8033, 0x62dc, 0x6069, 0x4e9e, 0x4e01, 0x963f, 0x62c9, 0x6728, 0x5716, 0x5b89, 0x66fc,
0x963f, 0x90a3, 0x5e95, 0x963f, 0x514b, 0x5957, 0x963f, 0x514b, 0x6258, 0x6bd4, 0x963f, 0x4ec0,
0x54c8, 0x5df4, 0x7279, 0x963f, 0x7279, 0x52de, 0x5df4, 0x683c, 0x9054, 0x5df4, 0x6797, 0x66fc,
0x8c37, 0x5df4, 0x723e, 0x7459, 0x723e, 0x8c9d, 0x9b6f, 0x7279, 0x6bd4, 0x4ec0, 0x51f1, 0x514b,
0x6c76, 0x840a, 0x8d64, 0x5854, 0x53ef, 0x502b, 0x5761, 0x5927, 0x99ac, 0x58eb, 0x9769, 0x9054,
0x5361, 0x5e1d, 0x529b, 0x675c, 0x62dc, 0x675c, 0x6851, 0x8c9d, 0x6cd5, 0x99ac, 0x53e4, 0x65af,
0x5854, 0x52a0, 0x85a9, 0x8d6b, 0x5e03, 0x9686, 0x80e1, 0x5fd7, 0x660e, 0x5e02, 0x4e2d, 0x83ef,
0x4eba, 0x6c11, 0x5171, 0x548c, 0x570b, 0x9999, 0x6e2f, 0x7279, 0x5225, 0x884c, 0x653f, 0x5340,
0x79d1, 0x5e03, 0x591a, 0x4f0a, 0x723e, 0x5eab, 0x6b21, 0x514b, 0x96c5, 0x52a0, 0x9054, 0x52a0,
0x4e9e, 0x5e03, 0x62c9, 0x8036, 0x8def, 0x6492, 0x51b7, 0x5580, 0x5e03, 0x723e, 0x582a, 0x5bdf,
0x52a0, 0x5580, 0x62c9, 0x86a9, 0x52a0, 0x5fb7, 0x6eff, 0x90fd, 0x582a, 0x5730, 0x52a0, 0x52a0,
0x723e, 0x5404, 0x7b54, 0x514b, 0x62c9, 0x65af, 0x8afe, 0x4e9e, 0x723e, 0x65af, 0x514b, 0x5409,
0x9686, 0x5761, 0x53e4, 0x6649, 0x79d1, 0x5a01, 0x7279, 0x4e2d, 0x83ef, 0x4eba, 0x6c11, 0x5171,
0x548c, 0x570b, 0x6fb3, 0x9580, 0x7279, 0x5225, 0x884c, 0x653f, 0x5340, 0x99ac, 0x52a0, 0x4e39,
0x99ac, 0x5361, 0x6c99, 0x723e, 0x99ac, 0x5c3c, 0x62c9, 0x99ac, 0x65af, 0x958b, 0x7279, 0x5c3c,
0x53e4, 0x897f, 0x4e9e, 0x65b0, 0x5eab, 0x8332, 0x6d85, 0x8328, 0x514b, 0x65b0, 0x897f, 0x4f2f,
0x5229, 0x4e9e, 0x9102, 0x6728, 0x65af, 0x514b, 0x70cf, 0x62c9, 0x723e, 0x91d1, 0x908a, 0x5764,
0x7538, 0x5e73, 0x58e4, 0x5361, 0x9054, 0x79d1, 0x65af, 0x5854, 0x5948, 0x514b, 0x5b5c, 0x52d2,
0x5967, 0x723e, 0x9054, 0x5229, 0x96c5, 0x5fb7, 0x5eab, 0x9801, 0x5cf6, 0x6492, 0x99ac, 0x723e,
0x7f55, 0x9996, 0x723e, 0x4e0a, 0x6d77, 0x65b0, 0x52a0, 0x5761, 0x4e2d, 0x79d1, 0x96f7, 0x59c6,
0x65af, 0x514b, 0x53f0, 0x5317, 0x5854, 0x4ec0, 0x5e72, 0x7b2c, 0x6bd4, 0x5229, 0x65af, 0x5fb7,
0x9ed1, 0x862d, 0x5ef7, 0x5e03, 0x6771, 0x4eac, 0x6258, 0x6728, 0x65af, 0x514b, 0x70cf, 0x862d,
0x5df4, 0x6258, 0x70cf, 0x9b6f, 0x6728, 0x9f4a, 0x70cf, 0x65af, 0x5167, 0x62c9, 0x6c38, 0x73cd,
0x6d77, 0x53c3, 0x5d34, 0x96c5, 0x5eab, 0x6b21, 0x514b, 0x4ef0, 0x5149, 0x8449, 0x5361, 0x6377,
0x6797, 0x5821, 0x8449, 0x91cc, 0x6eab, 0x4e9e, 0x901f, 0x723e, 0x7fa4, 0x5cf6, 0x767e, 0x6155,
0x9054, 0x52a0, 0x7d0d, 0x5229, 0x7dad, 0x5fb7, 0x89d2, 0x6cd5, 0x7f85, 0x7fa4, 0x5cf6, 0x99ac,
0x5f97, 0x62c9, 0x7fa4, 0x5cf6, 0x96f7, 0x514b, 0x96c5, 0x7dad, 0x514b, 0x5357, 0x55ac, 0x6cbb,
0x4e9e, 0x8056, 0x8d6b, 0x52d2, 0x62ff, 0x5cf6, 0x53f2, 0x5766, 0x5229, 0x963f, 0x5f97, 0x96f7,
0x5fb7, 0x5e03, 0x5229, 0x65af, 0x73ed, 0x5e03, 0x7f85, 0x80af, 0x5e0c, 0x723e, 0x9054, 0x723e,
0x6587, 0x5c24, 0x514b, 0x62c9, 0x8377, 0x5df4, 0x7279, 0x6797, 0x5fb7, 0x66fc, 0x8c6a, 0x52f3,
0x7235, 0x5cf6, 0x58a8, 0x723e, 0x672c, 0x4f2f, 0x65af, 0x96ea, 0x68a8, 0x963f, 0x59c6, 0x65af,
0x7279, 0x4e39, 0x5b89, 0x9053, 0x723e, 0x963f, 0x65af, 0x7279, 0x62c9, 0x7f55, 0x96c5, 0x5178,
0x8c9d, 0x723e, 0x683c, 0x52d2, 0x67cf, 0x6797, 0x5e03, 0x62c9, 0x63d0, 0x65af, 0x62c9, 0x74e6,
0x5e03, 0x9b6f, 0x585e, 0x723e, 0x5e03, 0x52a0, 0x52d2, 0x65af, 0x7279, 0x5e03, 0x9054, 0x4f69,
0x65af, 0x5e03, 0x8f9b, 0x6839, 0x5947, 0x897f, 0x7459, 0x54e5, 0x672c, 0x54c8, 0x6839, 0x90fd,
0x67cf, 0x6797, 0x76f4, 0x5e03, 0x7f85, 0x9640, 0x6839, 0x606f, 0x5cf6, 0x8d6b, 0x723e, 0x8f9b,
0x57fa, 0x66fc, 0x5cf6, 0x4f0a, 0x65af, 0x5766, 0x5821, 0x6fa4, 0x897f, 0x5cf6, 0x52a0, 0x91cc,
0x5be7, 0x683c, 0x52d2, 0x57fa, 0x6d1b, 0x592b, 0x57fa, 0x8f14, 0x91cc, 0x65af, 0x672c, 0x76e7,
0x6bd4, 0x5b89, 0x7d0d, 0x502b, 0x6566, 0x76e7, 0x68ee, 0x5821, 0x99ac, 0x5fb7, 0x91cc, 0x99ac,
0x723e, 0x4ed6, 0x746a, 0x9e97, 0x6e2f, 0x660e, 0x65af, 0x514b, 0x6469, 0x7d0d, 0x54e5, 0x83ab,
0x65af, 0x79d1, 0x5967, 0x65af, 0x9678, 0x5df4, 0x9ece, 0x6ce2, 0x591a, 0x91cc, 0x5bdf, 0x5e03,
0x62c9, 0x683c, 0x91cc, 0x52a0, 0x7f85, 0x99ac, 0x6c99, 0x99ac, 0x62c9, 0x8056, 0x99ac, 0x5229,
0x8afe, 0x585e, 0x62c9, 0x8036, 0x4f5b, 0x85a9, 0x62c9, 0x6258, 0x592b, 0x8f9b, 0x975e, 0x6d1b,
0x6d66, 0x53f2, 0x9ad8, 0x6bd4, 0x8036, 0x7d22, 0x83f2, 0x4e9e, 0x65af, 0x5fb7, 0x54e5, 0x723e,
0x6469, 0x5854, 0x6797, 0x5730, 0x62c9, 0x90a3, 0x70cf, 0x91cc, 0x63da, 0x8afe, 0x592b, 0x65af,
0x514b, 0x74e6, 0x90fd, 0x8332, 0x68b5, 0x8482, 0x5ca1, 0x7dad, 0x4e5f, 0x7d0d, 0x7dad, 0x723e,
0x7d10, 0x65af, 0x4f0f, 0x723e, 0x52a0, 0x683c, 0x52d2, 0x83ef, 0x6c99, 0x672d, 0x683c, 0x745e,
0x5e03, 0x8607, 0x9ece, 0x4e16, 0x5b89, 0x5854, 0x90a3, 0x90a3, 0x5229, 0x4f5b, 0x67e5, 0x6208,
0x65af, 0x8056, 0x8a95, 0x5cf6, 0x79d1, 0x79d1, 0x65af, 0x7fa4, 0x5cf6, 0x79d1, 0x6469, 0x7f85,
0x7fa4, 0x5cf6, 0x51f1, 0x723e, 0x84cb, 0x6717, 0x5cf6, 0x99ac, 0x57c3, 0x5cf6, 0x99ac, 0x723e,
0x5730, 0x592b, 0x6a21, 0x91cc, 0x897f, 0x65af, 0x99ac, 0x7d04, 0x7279, 0x5cf6, 0x7559, 0x5c3c,
0x65fa, 0x5cf6, 0x963f, 0x76ae, 0x4e9e, 0x5967, 0x514b, 0x862d, 0x5e03, 0x5e72, 0x7dad, 0x723e,
0x67e5, 0x5766, 0x695a, 0x514b, 0x5fa9, 0x6d3b, 0x5cf6, 0x57c3, 0x6cd5, 0x7279, 0x6cd5, 0x8003,
0x798f, 0x6590, 0x6fdf, 0x5bcc, 0x90a3, 0x5bcc, 0x63d0, 0x52a0, 0x62c9, 0x5df4, 0x54e5, 0x7fa4,
0x5cf6, 0x7518, 0x6bd4, 0x723e, 0x7fa4, 0x5cf6, 0x74dc, 0x9054, 0x5eb7, 0x7d0d, 0x723e, 0x5cf6,
0x95dc, 0x5cf6, 0x6a80, 0x9999, 0x5c71, 0x6069, 0x5f97, 0x4f2f, 0x7406, 0x5cf6, 0x57fa, 0x91cc,
0x5730, 0x99ac, 0x5730, 0x5cf6, 0x79d1, 0x65af, 0x745e, 0x74dc, 0x52a0, 0x6797, 0x5cf6, 0x99ac,
0x6731, 0x8afe, 0x99ac, 0x53ef, 0x85a9, 0x65af, 0x5cf6, 0x4e2d, 0x9014, 0x5cf6, 0x8afe, 0x9b6f,
0x7d10, 0x57c3, 0x5cf6, 0x8afe, 0x798f, 0x514b, 0x8afe, 0x7f8e, 0x4e9e, 0x5df4, 0x54e5, 0x5df4,
0x54e5, 0x5e1b, 0x7409, 0x76ae, 0x7279, 0x80af, 0x7fa4, 0x5cf6, 0x6ce2, 0x7d0d, 0x4f69, 0x83ab,
0x58eb, 0x6bd4, 0x6e2f, 0x62c9, 0x7f85, 0x6e6f, 0x52a0, 0x585e, 0x73ed, 0x5927, 0x6eaa, 0x5730,
0x5854, 0x62c9, 0x74e6, 0x6771, 0x52a0, 0x5854, 0x5e03, 0x5cf6, 0x5a01, 0x514b, 0x74e6, 0x5229,
0x65af, 0x963f, 0x6bd4, 0x8ba9, 0x963f, 0x8fea, 0x65af, 0x963f, 0x8d1d, 0x5df4, 0x963f, 0x5c14,
0x53ca, 0x5c14, 0x963f, 0x65af, 0x739b, 0x62c9, 0x5df4, 0x9a6c, 0x79d1, 0x6bd4, 0x7ecd, 0x5e03,
0x5170, 0x592a, 0x5c14, 0x5e03, 0x62c9, 0x67f4, 0x7ef4, 0x5c14, 0x5f00, 0x7f57, 0x5361, 0x8428,
0x5e03, 0x5170, 0x5361, 0x4f11, 0x8fbe, 0x8fbe, 0x5580, 0x5c14, 0x6c99, 0x5170, 0x6e2f, 0x7ea6,
0x7ff0, 0x5c3c, 0x65af, 0x5821, 0x7f57, 0x5b89, 0x8fbe, 0x5362, 0x672c, 0x5df4, 0x5e0c, 0x9a6c,
0x62c9, 0x535a, 0x9a6c, 0x666e, 0x6258, 0x9a6c, 0x8d5b, 0x9c81, 0x8499, 0x7f57, 0x7ef4, 0x4e9a,
0x6069, 0x8d3e, 0x6885, 0x7eb3, 0x5c3c, 0x4e9a, 0x7f8e, 0x8bfa, 0x514b, 0x5c11, 0x6ce2, 0x591a,
0x8bfa, 0x4f5b, 0x5723, 0x591a, 0x7f8e, 0x6e29, 0x5f97, 0x548c, 0x514b, 0x827e, 0x8fbe, 0x514b,
0x5e03, 0x5b9c, 0x8bfa, 0x65af, 0x827e, 0x5229, 0x65af, 0x5361, 0x5854, 0x9a6c, 0x5361, 0x54e5,
0x591a, 0x534e, 0x95e8, 0x591a, 0x8428, 0x91cc, 0x5965, 0x52a0, 0x8036, 0x6208, 0x65af, 0x8428,
0x5c14, 0x5854, 0x5723, 0x80e1, 0x5b89, 0x5723, 0x8def, 0x6613, 0x5410, 0x5e93, 0x66fc, 0x4e4c,
0x65af, 0x6000, 0x4e9a, 0x4e9a, 0x677e, 0x68ee, 0x5df4, 0x4f0a, 0x4e9a, 0x73ed, 0x5fb7, 0x62c9,
0x65af, 0x5df4, 0x8d1d, 0x591a, 0x8d1d, 0x4f26, 0x8d1d, 0x91cc, 0x65af, 0x767d, 0x6717, 0x8428,
0x5e03, 0x9686, 0x4fdd, 0x7ef4, 0x65af, 0x5854, 0x5251, 0x6865, 0x6e7e, 0x683c, 0x5170, 0x573a,
0x5f00, 0x4e91, 0x5f00, 0x66fc, 0x7fa4, 0x5c9b, 0x5947, 0x534e, 0x534e, 0x534e, 0x96f7, 0x65af,
0x57ce, 0x514b, 0x96f7, 0x65af, 0x987f, 0x5e93, 0x62c9, 0x7d22, 0x4e39, 0x9a6c, 0x6c99, 0x6587,
0x827e, 0x9c81, 0x5185, 0x4f69, 0x8428, 0x5c14, 0x74e6, 0x591a, 0x7eb3, 0x5c14, 0x900a, 0x5821,
0x683c, 0x96f7, 0x65af, 0x8d1d, 0x9e45, 0x6e7e, 0x5927, 0x7279, 0x514b, 0x5c9b, 0x683c, 0x745e,
0x7eb3, 0x8fbe, 0x74dc, 0x5730, 0x9a6c, 0x62c9, 0x74dc, 0x4e9a, 0x57fa, 0x5c14, 0x76d6, 0x4e9a,
0x90a3, 0x57c3, 0x83ab, 0x897f, 0x7ea6, 0x5370, 0x7b2c, 0x5b89, 0x90a3, 0x5dde, 0x8bfa, 0x514b,
0x65af, 0x5370, 0x7b2c, 0x5b89, 0x90a3, 0x5dde, 0x9a6c, 0x4f26, 0x54e5, 0x5370, 0x7b2c, 0x5b89,
0x90a3, 0x5dde, 0x6cf0, 0x5c14, 0x57ce, 0x5370, 0x7b2c, 0x5b89, 0x90a3, 0x5dde, 0x7ef4, 0x5a01,
0x5370, 0x7b2c, 0x5b89, 0x90a3, 0x5dde, 0x6e29, 0x68ee, 0x65af, 0x5370, 0x7b2c, 0x5b89, 0x90a3,
0x5dde, 0x5a01, 0x7eb3, 0x9a6c, 0x514b, 0x4f0a, 0x5974, 0x7ef4, 0x514b, 0x7259, 0x4e70, 0x52a0,
0x6731, 0x8bfa, 0x8def, 0x6613, 0x65af, 0x7ef4, 0x5c14, 0x80af, 0x5854, 0x57fa, 0x5dde, 0x8499,
0x5730, 0x5374, 0x7f57, 0x514b, 0x62c9, 0x4f26, 0x4ee3, 0x514b, 0x5229, 0x9a6c, 0x6d1b, 0x6749,
0x77f6, 0x4e0b, 0x592a, 0x5b50, 0x533a, 0x9a6c, 0x745f, 0x6b27, 0x9a6c, 0x62ff, 0x74dc, 0x739b,
0x7459, 0x65af, 0x9a6c, 0x91cc, 0x6208, 0x7279, 0x9a6c, 0x4e01, 0x5c3c, 0x514b, 0x9a6c, 0x5854,
0x83ab, 0x7f57, 0x65af, 0x9a6c, 0x8428, 0x7279, 0x5170, 0x7f8e, 0x8bfa, 0x7c73, 0x5c3c, 0x6885,
0x91cc, 0x8fbe, 0x5bc6, 0x542f, 0x4ed1, 0x8499, 0x514b, 0x987f, 0x8499, 0x7279, 0x7ef4, 0x591a,
0x8499, 0x54f2, 0x814a, 0x7ebd, 0x7ea6, 0x8bfa, 0x59c6, 0x8bfa, 0x4f26, 0x54c8, 0x5317, 0x8fbe,
0x79d1, 0x4ed6, 0x5dde, 0x5e03, 0x7531, 0x62c9, 0x5317, 0x8fbe, 0x79d1, 0x4ed6, 0x5dde, 0x4e2d,
0x5fc3, 0x5317, 0x8fbe, 0x79d1, 0x4ed6, 0x5dde, 0x7ebd, 0x6c99, 0x4f26, 0x5965, 0x5e0c, 0x7eb3,
0x52a0, 0x5df4, 0x62ff, 0x9a6c, 0x5df4, 0x62c9, 0x9a6c, 0x5229, 0x6ce2, 0x51e4, 0x51f0, 0x57ce,
0x7ef4, 0x7559, 0x6e2f, 0x84ec, 0x5854, 0x963f, 0x96f7, 0x7eb3, 0x65af, 0x5170, 0x4eca, 0x6e7e,
0x91cc, 0x8d3e, 0x7eb3, 0x7f57, 0x65af, 0x9c81, 0x7279, 0x91cc, 0x7ea6, 0x5e03, 0x5170, 0x5723,
0x5854, 0x4f26, 0x5723, 0x5730, 0x7259, 0x54e5, 0x5723, 0x591a, 0x660e, 0x54e5, 0x5723, 0x4fdd,
0x7f57, 0x4f0a, 0x6258, 0x79d1, 0x5c14, 0x6258, 0x7c73, 0x7279, 0x9521, 0x7279, 0x5361, 0x5723,
0x5df4, 0x6258, 0x6d1b, 0x7f2a, 0x5c9b, 0x5723, 0x7ea6, 0x7ff0, 0x5723, 0x57fa, 0x8328, 0x5723,
0x9732, 0x897f, 0x4e9a, 0x5723, 0x6258, 0x9a6c, 0x65af, 0x5723, 0x6587, 0x68ee, 0x65af, 0x5a01,
0x592b, 0x7279, 0x5361, 0x4f26, 0x7279, 0x63d0, 0x534e, 0x7eb3, 0x591a, 0x4f26, 0x591a, 0x6258,
0x5c14, 0x6258, 0x62c9, 0x6e29, 0x54e5, 0x534e, 0x6000, 0x7279, 0x970d, 0x65af, 0x6e29, 0x5c3c,
0x4f2f, 0x96c5, 0x5e93, 0x5854, 0x51ef, 0x897f, 0x6234, 0x7ef4, 0x65af, 0x675c, 0x8499, 0x675c,
0x6bd4, 0x5c14, 0x9ea6, 0x89c9, 0x7406, 0x9ea6, 0x514b, 0x9ed8, 0x591a, 0x5e15, 0x9ea6, 0x7f57,
0x745f, 0x62c9, 0x7ef0, 0x5c14, 0x4e9a, 0x4e01, 0x963f, 0x62c9, 0x6728, 0x56fe, 0x963f, 0x7279,
0x52b3, 0x5df4, 0x683c, 0x8fbe, 0x5df4, 0x5e93, 0x5df4, 0x5c14, 0x7459, 0x5c14, 0x8d1d, 0x9c81,
0x7279, 0x6bd4, 0x4ec0, 0x51ef, 0x514b, 0x6c76, 0x83b1, 0x53ef, 0x4f26, 0x5761, 0x5927, 0x9a6c,
0x58eb, 0x9769, 0x8fbe, 0x5361, 0x675c, 0x6851, 0x8d1d, 0x6cd5, 0x9a6c, 0x53e4, 0x65af, 0x5854,
0x52a0, 0x8428, 0x4e2d, 0x534e, 0x4eba, 0x6c11, 0x5171, 0x548c, 0x56fd, 0x9999, 0x6e2f, 0x7279,
0x522b, 0x884c, 0x653f, 0x533a, 0x4f0a, 0x5c14, 0x5e93, 0x6b21, 0x514b, 0x96c5, 0x52a0, 0x8fbe,
0x52a0, 0x4e9a, 0x5e03, 0x62c9, 0x5580, 0x5e03, 0x5c14, 0x52a0, 0x5fb7, 0x6ee1, 0x90fd, 0x52a0,
0x5c14, 0x5404, 0x7b54, 0x514b, 0x62c9, 0x65af, 0x8bfa, 0x4e9a, 0x5c14, 0x65af, 0x514b, 0x53e4,
0x664b, 0x4e2d, 0x534e, 0x4eba, 0x6c11, 0x5171, 0x548c, 0x56fd, 0x6fb3, 0x95e8, 0x7279, 0x522b,
0x884c, 0x653f, 0x533a, 0x9a6c, 0x52a0, 0x4e39, 0x9a6c, 0x5361, 0x6c99, 0x5c14, 0x9a6c, 0x5c3c,
0x62c9, 0x9a6c, 0x65af, 0x5f00, 0x7279, 0x5c3c, 0x53e4, 0x897f, 0x4e9a, 0x65b0, 0x5e93, 0x5179,
0x6d85, 0x8328, 0x514b, 0x65b0, 0x897f, 0x4f2f, 0x5229, 0x4e9a, 0x4e4c, 0x62c9, 0x5c14, 0x91d1,
0x8fb9, 0x5361, 0x8fbe, 0x514b, 0x5b5c, 0x52d2, 0x5965, 0x5c14, 0x8fbe, 0x5e93, 0x9875, 0x5c9b,
0x6492, 0x9a6c, 0x5c14, 0x7f55, 0x9996, 0x5c14, 0x5fb7, 0x9ed1, 0x5170, 0x4e1c, 0x4eac, 0x4e4c,
0x5170, 0x5df4, 0x6258, 0x4e4c, 0x9c81, 0x6728, 0x9f50, 0x4e4c, 0x65af, 0x5185, 0x62c9, 0x6d77,
0x53c2, 0x5d34, 0x96c5, 0x5e93, 0x6b21, 0x514b, 0x53f6, 0x5361, 0x6377, 0x6797, 0x5821, 0x53f6,
0x91cc, 0x6e29, 0x4e9a, 0x901f, 0x5c14, 0x7fa4, 0x5c9b, 0x767e, 0x6155, 0x8fbe, 0x52a0, 0x7eb3,
0x5229, 0x7ef4, 0x5fb7, 0x89d2, 0x6cd5, 0x7f57, 0x7fa4, 0x5c9b, 0x9a6c, 0x5f97, 0x62c9, 0x7fa4,
0x5c9b, 0x96f7, 0x514b, 0x96c5, 0x7ef4, 0x514b, 0x5357, 0x4e54, 0x6cbb, 0x4e9a, 0x5723, 0x8d6b,
0x52d2, 0x62ff, 0x5c9b, 0x5e03, 0x7f57, 0x80af, 0x5e0c, 0x5c14, 0x8fbe, 0x5c14, 0x6587, 0x8c6a,
0x52cb, 0x7235, 0x5c9b, 0x58a8, 0x5c14, 0x672c, 0x5b89, 0x9053, 0x5c14, 0x8d1d, 0x5c14, 0x683c,
0x52d2, 0x5e03, 0x9c81, 0x585e, 0x5c14, 0x5e03, 0x8fbe, 0x4f69, 0x65af, 0x76f4, 0x5e03, 0x7f57,
0x9640, 0x6839, 0x606f, 0x5c9b, 0x8d6b, 0x5c14, 0x8f9b, 0x57fa, 0x66fc, 0x5c9b, 0x6cfd, 0x897f,
0x5c9b, 0x52a0, 0x91cc, 0x5b81, 0x683c, 0x52d2, 0x57fa, 0x8f85, 0x5362, 0x6bd4, 0x5b89, 0x7eb3,
0x4f26, 0x6566, 0x5362, 0x68ee, 0x5821, 0x9a6c, 0x5fb7, 0x91cc, 0x9a6c, 0x5c14, 0x4ed6, 0x739b,
0x4e3d, 0x6e2f, 0x6469, 0x7eb3, 0x54e5, 0x5965, 0x65af, 0x9646, 0x7f57, 0x9a6c, 0x6c99, 0x9a6c,
0x62c9, 0x5723, 0x9a6c, 0x5229, 0x8bfa, 0x8428, 0x62c9, 0x6258, 0x592b, 0x7d22, 0x83f2, 0x4e9a,
0x65af, 0x5fb7, 0x54e5, 0x5c14, 0x6469, 0x4e4c, 0x91cc, 0x626c, 0x8bfa, 0x592b, 0x65af, 0x514b,
0x74e6, 0x90fd, 0x5179, 0x68b5, 0x8482, 0x5188, 0x7ef4, 0x4e5f, 0x7eb3, 0x7ef4, 0x5c14, 0x7ebd,
0x65af, 0x4f0f, 0x5c14, 0x52a0, 0x683c, 0x52d2, 0x534e, 0x6c99, 0x82cf, 0x9ece, 0x4e16, 0x5723,
0x8bde, 0x5c9b, 0x79d1, 0x79d1, 0x65af, 0x7fa4, 0x5c9b, 0x79d1, 0x6469, 0x7f57, 0x7fa4, 0x5c9b,
0x51ef, 0x5c14, 0x76d6, 0x6717, 0x5c9b, 0x9a6c, 0x57c3, 0x5c9b, 0x9a6c, 0x5c14, 0x5730, 0x592b,
0x9a6c, 0x7ea6, 0x7279, 0x5c9b, 0x7559, 0x5c3c, 0x65fa, 0x5c9b, 0x963f, 0x76ae, 0x4e9a, 0x5965,
0x514b, 0x5170, 0x5e03, 0x5e72, 0x7ef4, 0x5c14, 0x590d, 0x6d3b, 0x5c9b, 0x6590, 0x6d4e, 0x52a0,
0x62c9, 0x5df4, 0x54e5, 0x7fa4, 0x5c9b, 0x7518, 0x6bd4, 0x5c14, 0x7fa4, 0x5c9b, 0x74dc, 0x8fbe,
0x5eb7, 0x7eb3, 0x5c14, 0x5c9b, 0x5173, 0x5c9b, 0x6069, 0x5f97, 0x4f2f, 0x7406, 0x5c9b, 0x57fa,
0x91cc, 0x5730, 0x9a6c, 0x5730, 0x5c9b, 0x74dc, 0x52a0, 0x6797, 0x5c9b, 0x9a6c, 0x6731, 0x8bfa,
0x9a6c, 0x53ef, 0x8428, 0x65af, 0x5c9b, 0x4e2d, 0x9014, 0x5c9b, 0x8bfa, 0x9c81, 0x7ebd, 0x57c3,
0x5c9b, 0x8bfa, 0x798f, 0x514b, 0x8bfa, 0x7f8e, 0x4e9a, 0x76ae, 0x7279, 0x80af, 0x7fa4, 0x5c9b,
0x6ce2, 0x7eb3, 0x4f69, 0x62c9, 0x7f57, 0x6c64, 0x52a0, 0x4e1c, 0x52a0, 0x5854, 0x5e03, 0x5c9b,
0x41, 0x6c, 0x67, 0x65, 0x72, 0x43, 0x61, 0x69, 0x72, 0x65, 0x2c, 0x20,
0x65, 0x6c, 0x61, 0x6c, 0x2d, 0x41, 0x61, 0x69, 0x75, 0x6e, 0x4d, 0x6f,
0x67, 0x61, 0x64, 0x69, 0x73, 0x63, 0x69, 0x6f, 0x4d, 0x6f, 0x6e, 0x72,
0xf2, 0x76, 0x69, 0x61, 0x4e, 0x2019, 0x44, 0x6a, 0x61, 0x6d, 0x65, 0x6e,
0x61, 0x52, 0xed, 0x6f, 0x20, 0x47, 0x61, 0x6c, 0x6c, 0x65, 0x67, 0x6f,
0x73, 0x42, 0x6c, 0x61, 0x6e, 0x63, 0x20, 0x53, 0x61, 0x62, 0x6c, 0x6f,
0x6e, 0x42, 0x6f, 0x67, 0x6f, 0x74, 0xe0, 0x43, 0x61, 0x69, 0x65, 0x6e,
0x61, 0x43, 0x61, 0x69, 0x6d, 0x61, 0x6e, 0x43, 0x69, 0x75, 0x74, 0x61,
0x74, 0x20, 0x64, 0x65, 0x20, 0x4d, 0xe8, 0x78, 0x69, 0x63, 0x4e, 0x6f,
0x76, 0x61, 0x20, 0x59, 0x6f, 0x72, 0x6b, 0x42, 0x65, 0x75, 0x6c, 0x61,
0x68, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72,
0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x4e, 0x6f, 0x72, 0x64, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c,
0x65, 0x6d, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x50, 0x61, 0x6e, 0x61, 0x6d,
0xe0, 0x53, 0x63, 0x6f, 0x72, 0x65, 0x73, 0x62, 0x79, 0x73, 0x75, 0x6e,
0x64, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x54, 0x68, 0x6f, 0x6d, 0x61,
0x73, 0x41, 0x6e, 0xe0, 0x64, 0x69, 0x72, 0x41, 0x71, 0x74, 0x61, 0xfd,
0x41, 0x71, 0x74, 0xf3, 0x62, 0x65, 0x41, 0x15f, 0x67, 0x61, 0x62, 0x61,
0x74, 0x41, 0x74, 0x79, 0x72, 0x61, 0xfd, 0x42, 0x61, 0x72, 0x6e, 0x61,
0xfc, 0x6c, 0x54, 0x78, 0x69, 0x74, 0xe0, 0x44, 0x75, 0x69, 0x78, 0x61,
0x6e, 0x62, 0xe9, 0x48, 0x1ed3, 0x20, 0x43, 0x68, 0xed, 0x20, 0x4d, 0x69,
0x6e, 0x68, 0x4a, 0x61, 0x69, 0x70, 0x75, 0x72, 0x4e, 0x69, 0x63, 0xf2,
0x73, 0x69, 0x61, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x51, 0x6f, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x69, 0x53,
0x61, 0x6b, 0x68, 0x61, 0x6c, 0xed, 0x6e, 0x53, 0x65, 0xfc, 0x6c, 0x54,
0x61, 0x69, 0x78, 0x6b, 0x65, 0x6e, 0x74, 0x54, 0xf2, 0x71, 0x75, 0x69,
0x6f, 0xdc, 0x72, 0xfc, 0x6d, 0x63, 0x68, 0x69, 0x55, 0x73, 0x74, 0x2019,
0x2d, 0x4e, 0x65, 0x72, 0x61, 0x49, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x49, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x41, 0xe7, 0x6f, 0x72, 0x65, 0x73, 0x42, 0x65, 0x72, 0x6d, 0x75,
0x64, 0x65, 0x73, 0x49, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x61, 0x6e,
0xe0, 0x72, 0x69, 0x65, 0x73, 0x43, 0x61, 0x70, 0x20, 0x56, 0x65, 0x72,
0x64, 0x49, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x46, 0xe8, 0x72, 0x6f, 0x65,
0x47, 0x65, 0xf2, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x53, 0x75, 0x64, 0x41, 0x73, 0x74, 0x72, 0x61, 0x63, 0x61, 0x6e, 0x42,
0x72, 0x75, 0x73, 0x73, 0x65, 0x6c, 0xb7, 0x6c, 0x65, 0x73, 0x43, 0x6f,
0x70, 0x65, 0x6e, 0x68, 0x61, 0x67, 0x75, 0x65, 0x6e, 0x48, 0xe8, 0x6c,
0x73, 0x69, 0x6e, 0x6b, 0x69, 0x4b, 0xed, 0x72, 0x6f, 0x76, 0x4b, 0xed,
0x69, 0x76, 0x4d, 0xf2, 0x6e, 0x61, 0x63, 0x6f, 0x4d, 0x6f, 0x73, 0x63,
0x6f, 0x75, 0x53, 0x61, 0x72, 0xe0, 0x74, 0x6f, 0x76, 0x53, 0x69, 0x6d,
0x66, 0x65, 0x72, 0xf2, 0x70, 0x6f, 0x6c, 0x55, 0x6c, 0x69, 0xe0, 0x6e,
0x6f, 0x76, 0x73, 0x6b, 0x56, 0x61, 0x74, 0x69, 0x63, 0xe0, 0x56, 0xed,
0x6c, 0x6e, 0x69, 0x75, 0x73, 0x56, 0x61, 0x72, 0x73, 0xf2, 0x76, 0x69,
0x61, 0x5a, 0xfa, 0x72, 0x69, 0x63, 0x49, 0x6c, 0x6c, 0x61, 0x20, 0x64,
0x65, 0x20, 0x50, 0x61, 0x73, 0x71, 0x75, 0x61, 0x4d, 0x61, 0x72, 0x71,
0x75, 0x65, 0x73, 0x65, 0x73, 0x54, 0x61, 0x68, 0x69, 0x74, 0xed, 0x53,
0x69, 0x79, 0x75, 0x64, 0x61, 0x64, 0x20, 0x73, 0x61, 0x20, 0x48, 0x6f,
0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0xd804, 0xdd03, 0xd804,
0xdd1d, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd03, 0xd804,
0xdd13, 0xd804, 0xdd33, 0xd804, 0xdd13, 0xd804, 0xdd28, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd1d, 0xd804, 0xdd1d, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd34,
0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd25,
0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd25, 0xd804, 0xdd34,
0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd1d, 0xd804, 0xdd1f, 0xd804, 0xdd33,
0xd804, 0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd0b, 0xd804, 0xdd34,
0xd804, 0xdd09, 0xd804, 0xdd2d, 0xd804, 0xdd2a, 0xd804, 0xdd1d, 0xd804, 0xdd1a, 0xd804, 0xdd0e,
0xd804, 0xdd2a, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd25, 0xd804, 0xdd03,
0xd804, 0xdd2a, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd0e, 0xd804, 0xdd34,
0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2a,
0xd804, 0xdd0e, 0xd804, 0xdd2a, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2a,
0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd26, 0xd804, 0xdd2a, 0xd804, 0xdd07, 0xd804, 0xdd20,
0xd804, 0xdd34, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd07, 0xd804, 0xdd25, 0xd804, 0xdd1d,
0xd804, 0xdd33, 0xd804, 0xdd23, 0xd804, 0xdd01, 0xd804, 0xdd07, 0xd804, 0xdd25, 0xd804, 0xdd2c,
0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd11, 0xd804, 0xdd07, 0xd804, 0xdd27, 0xd804, 0xdd1a,
0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd13, 0xd804, 0xdd07,
0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd25, 0xd804, 0xdd23,
0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd1d, 0xd804, 0xdd2a,
0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd13, 0xd804, 0xdd2e, 0xd804, 0xdd20, 0xd804, 0xdd23,
0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd26, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1c, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804,
0xdd11, 0xd804, 0xdd05, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804,
0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1d, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804,
0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd26, 0xd804, 0xdd22, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804,
0xdd0e, 0xd804, 0xdd2e, 0xd804, 0xdd26, 0xd804, 0xdd1a, 0xd804, 0xdd2c, 0xd804, 0xdd25, 0xd804,
0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd34, 0xd804,
0xdd0e, 0xd804, 0xdd2a, 0xd804, 0xdd1d, 0xd804, 0xdd07, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804,
0xdd1b, 0xd804, 0xdd23, 0xd804, 0xdd08, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd2a, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd09, 0xd804,
0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd25, 0xd804, 0xdd25, 0xd804, 0xdd23, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd0c, 0xd804,
0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804,
0xdd2a, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804,
0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd20, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd23, 0xd804, 0xdd2a, 0xd804, 0xdd1d, 0xd804,
0xdd2a, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804,
0xdd23, 0xd804, 0xdd2a, 0xd804, 0xdd25, 0xd804, 0xdd07, 0xd804, 0xdd1f, 0xd804, 0xdd23, 0xd804,
0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd16, 0xd804,
0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd26, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804,
0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804,
0xdd1d, 0xd804, 0xdd1d, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd2e, 0xd804,
0xdd09, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd2a, 0xd804, 0xdd1f, 0xd804,
0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1e, 0xd804,
0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd2d, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804,
0xdd1d, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd0e, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd1a, 0xd804,
0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804,
0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd27, 0xd804, 0xdd11, 0xd804,
0xdd34, 0xd804, 0xdd24, 0xd804, 0xdd26, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804,
0xdd13, 0xd804, 0xdd2e, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1b, 0xd804, 0xdd2e, 0xd804,
0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd2e, 0x2d, 0xd804, 0xdd1a, 0xd804, 0xdd2e,
0xd804, 0xdd1e, 0xd804, 0xdd2e, 0xd804, 0xdd25, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0x20, 0xd804,
0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd33, 0xd804,
0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1b, 0xd804, 0xdd2e, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804,
0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd05, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804,
0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd19, 0xd804, 0xdd2e, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd03, 0xd804, 0xdd13, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804,
0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd33, 0xd804, 0xdd03, 0xd804, 0xdd0b, 0xd804,
0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804,
0xdd03, 0xd804, 0xdd33, 0xd804, 0xdd03, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804,
0xdd28, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd09, 0xd804, 0xdd03, 0xd804,
0xdd22, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd2d, 0xd804, 0xdd1a, 0xd804,
0xdd1d, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804,
0xdd0c, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd22,
0xd804, 0xdd2c, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd20,
0xd804, 0xdd11, 0xd804, 0xdd1f, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd07,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd2e, 0xd804, 0xdd1d,
0xd804, 0xdd26, 0xd804, 0xdd2a, 0xd804, 0xdd26, 0xd804, 0xdd2a, 0xd804, 0xdd03, 0xd804, 0xdd28,
0xd804, 0xdd23, 0x20, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804,
0xdd26, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804,
0xdd2e, 0xd804, 0xdd0e, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0x20,
0xd804, 0xdd09, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd0c,
0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd23, 0xd804, 0xdd34,
0xd804, 0xdd11, 0xd804, 0xdd25, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd26, 0xd804,
0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd23, 0xd804, 0xdd2d, 0xd804, 0xdd2a, 0xd804, 0xdd0c, 0xd804, 0xdd34,
0xd804, 0xdd11, 0xd804, 0xdd2a, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd1f, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd05, 0xd804, 0xdd25, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd2d,
0xd804, 0xdd20, 0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd1d,
0xd804, 0xdd03, 0xd804, 0xdd25, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd25,
0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd23, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd26, 0xd804,
0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804,
0xdd26, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1d, 0xd804, 0xdd26, 0xd804, 0xdd28, 0xd804,
0xdd20, 0x20, 0xd804, 0xdd1d, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd2c,
0xd804, 0xdd22, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd34,
0xd804, 0xdd1d, 0xd804, 0xdd13, 0xd804, 0xdd2e, 0xd804, 0xdd0e, 0xd804, 0xdd34, 0xd804, 0xdd1d,
0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1d,
0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd0e, 0xd804, 0xdd34, 0xd804, 0xdd1d,
0xd804, 0xdd33, 0xd804, 0xdd23, 0xd804, 0xdd01, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x2d, 0xd804,
0xdd25, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd31, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd0c,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd09, 0xd804, 0xdd2e,
0xd804, 0xdd11, 0xd804, 0xdd1d, 0xd804, 0xdd27, 0xd804, 0xdd20, 0xd804, 0xdd27, 0xd804, 0xdd0e,
0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd2c, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1d,
0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd0e, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd1d, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1b, 0xd804,
0xdd2e, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd13, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd07, 0xd804, 0xdd0c, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd26, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd28,
0xd804, 0xdd07, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd0c, 0xd804, 0xdd28, 0xd804, 0xdd26,
0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd26, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd07,
0xd804, 0xdd2e, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd22, 0xd804, 0xdd28,
0xd804, 0xdd07, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd0c,
0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2d,
0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd1d, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd22,
0xd804, 0xdd25, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd13, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd1f, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd25,
0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1e, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd13, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd13, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1e, 0xd804,
0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd2c, 0xd804, 0xdd11, 0xd804, 0xdd33, 0xd804,
0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804,
0xdd13, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804,
0xdd07, 0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd13, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804,
0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd2d, 0xd804, 0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804,
0xdd2c, 0xd804, 0xdd1b, 0xd804, 0xdd2c, 0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd25, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1e, 0xd804, 0xdd18,
0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1c, 0xd804, 0xdd2e, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1a, 0xd804, 0xdd2c, 0xd804,
0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd1c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804,
0xdd0e, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd23, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd1d, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd0c, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd1d, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804,
0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd16, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd13,
0xd804, 0xdd09, 0xd804, 0xdd31, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd2e,
0xd804, 0xdd1b, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd31, 0xd804, 0xdd16, 0xd804, 0xdd2c,
0xd804, 0xdd1f, 0xd804, 0xdd23, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd20, 0xd804, 0xdd03,
0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd34,
0xd804, 0xdd09, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd26, 0xd804, 0xdd33, 0xd804, 0xdd20,
0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd1c, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd26, 0xd804, 0xdd1e, 0xd804, 0xdd1a,
0xd804, 0xdd26, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd2e, 0xd804, 0xdd25,
0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd03,
0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd28, 0xd804, 0xdd20,
0xd804, 0xdd1a, 0xd804, 0xdd1b, 0xd804, 0xdd2e, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd0c,
0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd25,
0xd804, 0xdd34, 0x2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd13, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd1f, 0xd804, 0xdd22,
0xd804, 0xdd2c, 0xd804, 0xdd0b, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0x2c, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd28,
0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd1b, 0xd804, 0xdd28, 0xd804, 0xdd11, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd34,
0xd804, 0xdd09, 0xd804, 0xdd34, 0x2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd11,
0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd11,
0xd804, 0xdd28, 0x2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd13, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd1e, 0xd804, 0xdd2c,
0xd804, 0xdd1e, 0xd804, 0xdd2c, 0xd804, 0xdd20, 0xd804, 0xdd34, 0x2c, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd28, 0xd804, 0xdd20,
0xd804, 0xdd1a, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd25,
0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd2c, 0xd804, 0xdd0c,
0xd804, 0xdd34, 0x2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd13, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd03, 0xd804, 0xdd2a,
0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd1f, 0xd804, 0xdd33, 0xd804, 0xdd20,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd03,
0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2a, 0xd804, 0xdd1e, 0xd804, 0xdd33, 0xd804, 0xdd20,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd31,
0xd804, 0xdd23, 0xd804, 0xdd2a, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd11, 0xd804, 0xdd34,
0xd804, 0xdd0e, 0xd804, 0xdd1f, 0xd804, 0xdd2d, 0xd804, 0xdd07, 0xd804, 0xdd0e, 0xd804, 0xdd2a,
0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd23, 0xd804, 0xdd2d, 0xd804, 0xdd2a, 0xd804, 0xdd25,
0xd804, 0xdd34, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1f,
0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd0c,
0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0x2c, 0x20, 0xd804, 0xdd07, 0xd804, 0xdd2c,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd07,
0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd23, 0x20, 0xd804, 0xdd1b, 0xd804, 0xdd0e, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804,
0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd0e,
0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd23,
0xd804, 0xdd2e, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1b, 0xd804,
0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804,
0xdd2c, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd20,
0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1f,
0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd20, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd1a,
0xd804, 0xdd09, 0xd804, 0xdd31, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd03, 0xd804, 0xdd2a,
0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd09,
0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd28,
0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd16,
0xd804, 0xdd1f, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd0c, 0xd804, 0xdd34,
0xd804, 0xdd1f, 0xd804, 0xdd0e, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd1f,
0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd22,
0xd804, 0xdd28, 0xd804, 0xdd13, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd24, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2c,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd2e,
0x20, 0xd804, 0xdd25, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd2a, 0xd804, 0xdd23, 0xd804,
0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd0b, 0xd804,
0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804,
0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804,
0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804,
0xdd2c, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd13, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804,
0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804,
0xdd1a, 0xd804, 0xdd25, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804,
0xdd05, 0xd804, 0xdd2a, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804,
0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804,
0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd26, 0xd804, 0xdd1d, 0xd804, 0xdd2c, 0xd804, 0xdd03, 0xd804,
0xdd2a, 0xd804, 0xdd23, 0x2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd16, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd13,
0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd11, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd16,
0xd804, 0xdd34, 0xd804, 0xdd19, 0xd804, 0xdd33, 0xd804, 0xdd20, 0x2c, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd2a, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd2e, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd13, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd11, 0xd804,
0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0x20, 0xd804, 0xdd25, 0xd804, 0xdd23,
0xd804, 0xdd2c, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0x2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd2a,
0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd13, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd11, 0xd804, 0xdd1a, 0xd804,
0xdd2a, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd0e, 0xd804,
0xdd28, 0xd804, 0xdd0b, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd1b, 0xd804, 0xdd1a, 0xd804,
0xdd1f, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd1f, 0xd804,
0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd1c, 0xd804,
0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804,
0xdd34, 0xd804, 0xdd1b, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804,
0xdd34, 0x2d, 0xd804, 0xdd26, 0xd804, 0xdd27, 0x2d, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804,
0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804,
0xdd1b, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd25, 0xd804,
0xdd33, 0xd804, 0xdd1b, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1b, 0xd804,
0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd1e,
0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd26, 0xd804, 0xdd2e, 0xd804, 0xdd1b,
0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd2e, 0x20, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804,
0xdd22, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd0b, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804,
0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd22,
0xd804, 0xdd2c, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd1c, 0xd804, 0xdd2c, 0xd804, 0xdd22,
0xd804, 0xdd2c, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd22, 0xd804, 0xdd2c,
0xd804, 0xdd25, 0xd804, 0xdd2e, 0xd804, 0xdd23, 0xd804, 0xdd2a, 0xd804, 0xdd11, 0xd804, 0xdd34,
0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd1d, 0xd804,
0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd0b, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804,
0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd2a, 0xd804,
0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804,
0xdd25, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804,
0xdd2e, 0x20, 0xd804, 0xdd13, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd28, 0xd804, 0xdd01,
0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd25, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0x20, 0xd804,
0xdd1b, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804,
0xdd28, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804,
0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804,
0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd28, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804,
0xdd28, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x2e, 0x20, 0xd804, 0xdd1d, 0xd804,
0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd17, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804,
0xdd1f, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd11, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd0e, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd11, 0xd804, 0xdd34, 0x2e, 0x20, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd11,
0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x2e, 0x20, 0xd804, 0xdd23, 0xd804, 0xdd2a,
0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd17, 0xd804, 0xdd27, 0xd804,
0xdd1f, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11,
0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd2d, 0xd804, 0xdd2a, 0xd804, 0xdd1c, 0xd804, 0xdd34,
0xd804, 0xdd11, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd07, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804,
0xdd1a, 0xd804, 0xdd33, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd2c, 0xd804,
0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd09, 0xd804, 0xdd23, 0xd804,
0xdd34, 0xd804, 0xdd1b, 0xd804, 0xdd17, 0xd804, 0xdd2a, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804,
0xdd16, 0xd804, 0xdd28, 0xd804, 0xdd0e, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804,
0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd11, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804,
0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd23, 0xd804, 0xdd1e, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd1e, 0xd804, 0xdd22, 0xd804,
0xdd34, 0xd804, 0xdd26, 0xd804, 0xdd20, 0xd804, 0xdd2d, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804,
0xdd26, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804,
0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804,
0xdd1b, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804,
0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd11, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804,
0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd06, 0xd804, 0xdd2c, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804,
0xdd13, 0xd804, 0xdd2c, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804,
0xdd13, 0xd804, 0xdd2a, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd11, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd13, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a,
0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd34,
0xd804, 0xdd1f, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd05,
0xd804, 0xdd20, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1f,
0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1f,
0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd2a,
0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd2e, 0xd804, 0xdd1b, 0xd804, 0xdd23,
0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd22, 0xd804, 0xdd27,
0xd804, 0xdd17, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd24,
0xd804, 0xdd11, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd34,
0xd804, 0xdd1e, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd2e,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd01, 0xd804, 0xdd09,
0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd28,
0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd26,
0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd03,
0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd03,
0xd804, 0xdd1f, 0xd804, 0xdd33, 0xd804, 0xdd26, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd03, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd13, 0xd804, 0xdd28,
0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd11,
0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd03, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd11,
0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd2c, 0xd804, 0xdd03, 0xd804, 0xdd0c, 0xd804, 0xdd34,
0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1d, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd1d,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd1d,
0xd804, 0xdd26, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd01, 0xd804, 0xdd07, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1a,
0xd804, 0xdd2f, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2c, 0xd804, 0xdd2d,
0xd804, 0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd28,
0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd2c,
0xd804, 0xdd2d, 0xd804, 0xdd0c, 0xd804, 0xdd28, 0xd804, 0xdd16, 0xd804, 0xdd07, 0xd804, 0xdd27,
0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd33, 0xd804, 0xdd1d, 0xd804, 0xdd27,
0xd804, 0xdd27, 0xd804, 0xdd31, 0xd804, 0xdd31, 0xd804, 0xdd18, 0xd804, 0xdd1f, 0xd804, 0xdd0c,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd14, 0xd804, 0xdd07,
0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2a,
0xd804, 0xdd1d, 0xd804, 0xdd2d, 0xd804, 0xdd18, 0xd804, 0xdd2a, 0xd804, 0xdd25, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd0e, 0xd804, 0xdd26,
0xd804, 0xdd2c, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd26, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd0c, 0xd804, 0xdd28, 0x20,
0xd804, 0xdd1f, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd25, 0xd804,
0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd26, 0xd804,
0xdd27, 0xd804, 0xdd01, 0x20, 0xd804, 0xdd07, 0xd804, 0xdd27, 0xd804, 0xdd01, 0xd804, 0xdd26,
0xd804, 0xdd2e, 0xd804, 0xdd1e, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34, 0xd804, 0xdd03,
0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd11,
0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd0e,
0xd804, 0xdd07, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd0e, 0xd804, 0xdd27,
0xd804, 0xdd20, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd0e, 0xd804, 0xdd2c,
0xd804, 0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd0e, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804, 0xdd1f,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd1d, 0xd804, 0xdd2a, 0xd804, 0xdd23, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd0c, 0xd804, 0xdd11, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd07, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd0c, 0xd804, 0xdd28,
0xd804, 0xdd07, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd2a, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd2a, 0xd804, 0xdd08, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd09, 0xd804, 0xdd07, 0xd804, 0xdd27,
0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd18, 0xd804, 0xdd07, 0xd804, 0xdd33,
0xd804, 0xdd22, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd20,
0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd23, 0xd804, 0xdd23, 0xd804, 0xdd1f,
0xd804, 0xdd34, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd2a, 0xd804, 0xdd0c, 0xd804, 0xdd28, 0xd804, 0xdd01, 0xd804, 0xdd07, 0xd804, 0xdd2a,
0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd33,
0xd804, 0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd33,
0xd804, 0xdd20, 0xd804, 0xdd09, 0xd804, 0xdd13, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1f,
0xd804, 0xdd07, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd22, 0xd804, 0xdd34,
0xd804, 0xdd1f, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd23,
0xd804, 0xdd1f, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd16, 0xd804, 0xdd34,
0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd25, 0xd804, 0xdd28,
0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd1e, 0xd804, 0xdd2e, 0xd804, 0xdd07,
0xd804, 0xdd2a, 0xd804, 0xdd0e, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd2c, 0xd804, 0xdd11,
0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd1a,
0xd804, 0xdd27, 0xd804, 0xdd1e, 0xd804, 0xdd2e, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd1d,
0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd25,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd22,
0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd1b, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1b, 0xd804,
0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804,
0xdd1a, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd1b, 0xd804,
0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd27, 0xd804, 0xdd01, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28,
0xd804, 0xdd20, 0xd804, 0xdd27, 0xd804, 0xdd01, 0xd804, 0xdd07, 0xd804, 0xdd16, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd23,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd22, 0xd804, 0xdd28,
0xd804, 0xdd20, 0xd804, 0xdd19, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd08, 0xd804, 0xdd23,
0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd27, 0xd804, 0xdd1f,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd08, 0xd804, 0xdd27, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd03,
0xd804, 0xdd2e, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd01, 0xd804, 0xdd0a,
0xd804, 0xdd2d, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd0b, 0xd804, 0xdd34, 0xd804, 0xdd09,
0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd33,
0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd13, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd2c,
0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd20, 0xd804, 0xdd34,
0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd2d, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd06, 0xd804, 0xdd2c,
0xd804, 0xdd16, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd08, 0xd804, 0xdd27, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd2a, 0xd804, 0xdd1d,
0xd804, 0xdd2a, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd16,
0xd804, 0xdd2c, 0xd804, 0xdd26, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd17, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1c, 0xd804, 0xdd2a,
0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2e,
0xd804, 0xdd16, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd23, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd16, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd03,
0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd28, 0xd804, 0xdd05, 0xd804, 0xdd2a, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0x20, 0xd804, 0xdd1a, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd1e, 0xd804,
0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1e, 0xd804,
0xdd33, 0xd804, 0xdd23, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1e, 0xd804, 0xdd27, 0xd804,
0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804,
0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd0b, 0xd804,
0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804,
0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd11, 0xd804, 0xdd2c, 0xd804,
0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804,
0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804,
0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd1e, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804,
0xdd0e, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804,
0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd2a, 0xd804, 0xdd13, 0xd804,
0xdd07, 0xd804, 0xdd1a, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804,
0xdd2c, 0xd804, 0xdd1b, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd22, 0xd804, 0xdd34,
0xd804, 0xdd13, 0xd804, 0xdd2c, 0xd804, 0xdd1c, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd22,
0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd13, 0xd804, 0xdd2c, 0xd804, 0xdd2d, 0xd804, 0xdd22,
0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd1e,
0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd27, 0xd804, 0xdd09,
0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd0e, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd25, 0xd804,
0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd26,
0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd25, 0xd804, 0xdd33,
0xd804, 0xdd11, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd03,
0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd13, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd2c,
0xd804, 0xdd2d, 0xd804, 0xdd13, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd22,
0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2c, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd07,
0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd26, 0xd804, 0xdd28, 0xd804,
0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804,
0xdd2a, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804,
0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd23, 0xd804,
0xdd26, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804,
0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804,
0xdd2c, 0xd804, 0xdd1f, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd26, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd23,
0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd1b, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd17, 0xd804, 0xdd27, 0xd804, 0xdd25,
0xd804, 0xdd28, 0xd804, 0xdd13, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd03,
0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd33,
0xd804, 0xdd03, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd2e, 0xd804, 0xdd22,
0xd804, 0xdd03, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd33, 0xd804, 0xdd22,
0xd804, 0xdd08, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd17,
0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd1d,
0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22,
0xd804, 0xdd2c, 0xd804, 0xdd13, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd34,
0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd33,
0xd804, 0xdd22, 0xd804, 0xdd16, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd33, 0xd804, 0xdd23,
0xd804, 0xdd1e, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd25, 0xd804, 0xdd2c,
0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2a,
0xd804, 0xdd08, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11,
0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2a, 0xd804, 0xdd13, 0xd804, 0xdd1b, 0xd804, 0xdd2c,
0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2a,
0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd2c,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd28,
0xd804, 0xdd1a, 0xd804, 0xdd26, 0xd804, 0xdd2a, 0xd804, 0xdd0e, 0xd804, 0xdd2e, 0xd804, 0xdd1b,
0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd26, 0xd804, 0xdd2c, 0xd804, 0xdd09,
0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd1b, 0xd804, 0xdd34,
0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd28,
0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd11,
0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd2c,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd26, 0xd804, 0xdd2c,
0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd01, 0xd804, 0xdd07,
0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2d, 0xd804, 0xdd23, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd33,
0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd0c,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2a,
0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd25,
0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd28,
0xd804, 0xdd0b, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd13,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd1e,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd1e,
0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd1d,
0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd2a, 0xd804, 0xdd1d,
0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd23,
0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd27, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd2a, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd25,
0xd804, 0xdd2c, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd34,
0xd804, 0xdd09, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd13, 0xd804, 0xdd33, 0xd804, 0xdd22,
0xd804, 0xdd28, 0xd804, 0xdd13, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd23,
0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd28,
0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd26, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd25,
0xd804, 0xdd33, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd2e, 0xd804, 0xdd1a,
0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd34,
0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd22,
0xd804, 0xdd28, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd13,
0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd07,
0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd09, 0xd804, 0xdd34, 0xd804, 0xdd22,
0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd1f, 0xd804, 0xdd22,
0xd804, 0xdd25, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804,
0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd25, 0xd804, 0xdd22, 0xd804,
0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd1e, 0xd804, 0xdd2e, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804,
0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1c, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804,
0xdd1b, 0xd804, 0xdd2e, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd33, 0xd804,
0xdd07, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd34, 0xd804, 0xdd21, 0xd804, 0xdd2c, 0xd804,
0xdd25, 0xd804, 0xdd2e, 0xd804, 0xdd1c, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd25, 0xd804,
0xdd33, 0xd804, 0xdd11, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd26, 0xd804,
0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd26, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd1a, 0xd804,
0xdd05, 0xd804, 0xdd2a, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804,
0xdd2e, 0xd804, 0xdd1e, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd1e, 0xd804, 0xdd18, 0xd804, 0xdd2a, 0xd804, 0xdd0e, 0xd804, 0xdd34, 0xd804,
0xdd1e, 0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804,
0xdd20, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd1e, 0xd804, 0xdd2e, 0xd804, 0xdd23, 0xd804,
0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804,
0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd24, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804,
0xdd27, 0xd804, 0xdd0e, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804,
0xdd1b, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804,
0xdd08, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd1a, 0xd804, 0xdd1a, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1e, 0xd804, 0xdd2e, 0xd804,
0xdd0d, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd21, 0xd804,
0xdd2a, 0xd804, 0xdd25, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd0e,
0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd18,
0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd07,
0xd804, 0xdd2e, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd1f,
0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd07, 0xd804, 0xdd22, 0xd804, 0xdd34,
0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd1f, 0xd804, 0xdd26, 0xd804, 0xdd2c, 0xd804, 0xdd1f, 0xd804, 0xdd23, 0xd804, 0xdd34,
0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1b, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd1f,
0xd804, 0xdd20, 0xd804, 0xdd2e, 0xd804, 0xdd16, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd28,
0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd05, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd28,
0xd804, 0xdd20, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd1b,
0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13,
0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2c, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804, 0xdd0c,
0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd17, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd11,
0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd03,
0xd804, 0xdd28, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd22, 0xd804, 0xdd34,
0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1c, 0xd804, 0xdd16, 0xd804, 0xdd2c, 0xd804, 0xdd1c,
0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd1c,
0xd804, 0xdd2e, 0xd804, 0xdd1c, 0xd804, 0xdd28, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd1c,
0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd1c, 0xd804, 0xdd2a, 0xd804, 0xdd11, 0xd804, 0xdd28,
0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd23, 0xd804, 0xdd1b, 0xd804, 0xdd33,
0xd804, 0xdd20, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd09,
0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd2c,
0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd20, 0xd804, 0xdd18,
0xd804, 0xdd23, 0xd804, 0xdd18, 0xd804, 0xdd07, 0xd804, 0xdd1a, 0xd804, 0xdd23, 0xd804, 0xdd34,
0xd804, 0xdd09, 0xd804, 0xdd31, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd26, 0xd804, 0xdd27,
0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd23, 0xd804, 0xdd2a, 0xd804, 0xdd23, 0xd804, 0xdd2a,
0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd28,
0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd16, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd16,
0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd22,
0xd804, 0xdd20, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd31, 0xd804, 0xdd0e, 0xd804, 0xdd23,
0xd804, 0xdd2c, 0xd804, 0xdd2d, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd0e,
0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd22, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd2c, 0xd804, 0xdd25, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd1f,
0xd804, 0xdd28, 0xd804, 0xdd13, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd20,
0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2a,
0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd28,
0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1c, 0xd804, 0xdd2e,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd2a, 0xd804, 0xdd1f, 0xd804, 0xdd28,
0xd804, 0xdd20, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd09, 0xd804, 0xdd2e,
0x20, 0xd804, 0xdd1c, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804,
0xdd1b, 0xd804, 0xdd23, 0xd804, 0xdd05, 0xd804, 0xdd2a, 0xd804, 0xdd1b, 0xd804, 0xdd28, 0xd804,
0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2c, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804,
0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1b, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804,
0xdd1b, 0xd804, 0xdd2c, 0xd804, 0xdd1b, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804,
0xdd11, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd2f, 0xd804, 0xdd22, 0xd804, 0xdd2c,
0xd804, 0xdd0e, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd22,
0xd804, 0xdd27, 0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd01, 0xd804, 0xdd09, 0xd804, 0xdd25,
0xd804, 0xdd2d, 0xd804, 0xdd1b, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd26,
0xd804, 0xdd28, 0xd804, 0xdd16, 0xd804, 0xdd28, 0xd804, 0xdd11, 0xd804, 0xdd22, 0xd804, 0xdd24,
0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd01, 0xd804, 0xdd09, 0xd804, 0xdd11, 0xd804, 0xdd1b,
0xd804, 0xdd2a, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd0c,
0xd804, 0xdd34, 0x411, 0x440, 0x430, 0x437, 0x437, 0x430, 0x432, 0x438, 0x43b, 0x44c,
0x41a, 0x44a, 0x430, 0x445, 0x4c0, 0x438, 0x440, 0x430, 0x419, 0x43e, 0x445, 0x430,
0x43d, 0x43d, 0x435, 0x441, 0x431, 0x443, 0x440, 0x433, 0x41b, 0x438, 0x431, 0x440,
0x435, 0x432, 0x438, 0x43b, 0x44c, 0x410, 0x43d, 0x43a, 0x43e, 0x440, 0x438, 0x434,
0x436, 0x410, 0x43d, 0x433, 0x438, 0x43b, 0x44c, 0x44f, 0x411, 0x443, 0x44d, 0x43d,
0x43e, 0x441, 0x2d, 0x410, 0x439, 0x440, 0x435, 0x441, 0x420, 0x438, 0x43e, 0x2d,
0x413, 0x430, 0x43b, 0x44c, 0x435, 0x433, 0x43e, 0x441, 0x410, 0x441, 0x443, 0x43d,
0x441, 0x44c, 0x43e, 0x43d, 0x411, 0x430, 0x438, 0x44f, 0x2d, 0x434, 0x435, 0x2d,
0x411, 0x430, 0x43d, 0x434, 0x435, 0x440, 0x430, 0x441, 0x411, 0x43e, 0x439, 0x441,
0x435, 0x41a, 0x435, 0x439, 0x43c, 0x431, 0x440, 0x438, 0x434, 0x436, 0x2d, 0x411,
0x435, 0x439, 0x41a, 0x430, 0x439, 0x435, 0x43d, 0x43d, 0x430, 0x42d, 0x434, 0x43c,
0x43e, 0x43d, 0x442, 0x43e, 0x43d, 0x42d, 0x439, 0x440, 0x443, 0x43d, 0x435, 0x43f,
0x435, 0x424, 0x43e, 0x440, 0x442, 0x20, 0x41d, 0x435, 0x43b, 0x44c, 0x441, 0x43e,
0x43d, 0x413, 0x43b, 0x435, 0x439, 0x441, 0x2d, 0x411, 0x435, 0x439, 0x413, 0x443,
0x441, 0x2d, 0x411, 0x435, 0x439, 0x413, 0x443, 0x430, 0x44f, 0x43a, 0x438, 0x43b,
0x44c, 0x413, 0x430, 0x439, 0x430, 0x43d, 0x430, 0x413, 0x430, 0x43b, 0x438, 0x444,
0x430, 0x43a, 0x441, 0x42d, 0x440, 0x43c, 0x43e, 0x441, 0x438, 0x43b, 0x44c, 0x43e,
0x412, 0x438, 0x432, 0x44d, 0x439, 0x41b, 0x43e, 0x441, 0x2d, 0x410, 0x43d, 0x434,
0x436, 0x435, 0x43b, 0x435, 0x441, 0x41b, 0x43e, 0x443, 0x44d, 0x440, 0x2d, 0x41f,
0x440, 0x438, 0x43d, 0x441, 0x435, 0x441, 0x2d, 0x41a, 0x443, 0x43e, 0x440, 0x442,
0x435, 0x440, 0x41c, 0x43e, 0x43d, 0x442, 0x435, 0x440, 0x440, 0x435, 0x439, 0x41c,
0x43e, 0x43d, 0x441, 0x435, 0x440, 0x440, 0x430, 0x442, 0x41d, 0x44c, 0x44e, 0x2d,
0x419, 0x43e, 0x440, 0x43a, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x445, 0x430, 0x411,
0x43e, 0x439, 0x43b, 0x430, 0x2c, 0x20, 0x41a, 0x44a, 0x438, 0x43b, 0x431, 0x430,
0x441, 0x435, 0x434, 0x430, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x42e,
0x43a, 0x44a, 0x2c, 0x20, 0x41a, 0x44a, 0x438, 0x43b, 0x431, 0x430, 0x441, 0x435,
0x434, 0x430, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41d, 0x44c, 0x44e,
0x2d, 0x421, 0x430, 0x43b, 0x435, 0x43c, 0x413, 0x43e, 0x442, 0x445, 0x43e, 0x431,
0x41f, 0x443, 0x44d, 0x440, 0x442, 0x43e, 0x2d, 0x420, 0x438, 0x43a, 0x43e, 0x420,
0x435, 0x434, 0x436, 0x430, 0x439, 0x43d, 0x430, 0x421, 0x430, 0x43d, 0x442, 0x44c,
0x44f, 0x433, 0x43e, 0x421, 0x435, 0x43d, 0x2d, 0x411, 0x430, 0x440, 0x442, 0x435,
0x43b, 0x44c, 0x43c, 0x438, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41b, 0x44e, 0x441,
0x438, 0x421, 0x432, 0x438, 0x444, 0x442, 0x2d, 0x41a, 0x430, 0x440, 0x435, 0x43d,
0x442, 0x422, 0x435, 0x433, 0x443, 0x441, 0x438, 0x433, 0x430, 0x43b, 0x44c, 0x43f,
0x430, 0x414, 0x44e, 0x43c, 0x43e, 0x43d, 0x2d, 0x434, 0x2019, 0x42e, 0x440, 0x432,
0x438, 0x43b, 0x44c, 0x422, 0x440, 0x43e, 0x43b, 0x43b, 0x44c, 0x41b, 0x43e, 0x43d,
0x433, 0x439, 0x438, 0x440, 0x410, 0x43b, 0x43c, 0x430, 0x2d, 0x410, 0x442, 0x430,
0x4c0, 0x430, 0x43c, 0x43c, 0x430, 0x43d, 0x410, 0x43d, 0x430, 0x434, 0x44b, 0x440,
0x44c, 0x410, 0x442, 0x438, 0x440, 0x430, 0x443, 0x411, 0x430, 0x433, 0x4c0, 0x434,
0x430, 0x434, 0x411, 0x430, 0x43a, 0x43e, 0x445, 0x414, 0x438, 0x43c, 0x430, 0x448,
0x43a, 0x44a, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x414, 0x436, 0x430,
0x439, 0x43f, 0x443, 0x440, 0x41a, 0x44a, 0x443, 0x434, 0x441, 0x2d, 0x413, 0x4c0,
0x430, 0x43b, 0x430, 0x41f, 0x435, 0x442, 0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b,
0x43e, 0x432, 0x441, 0x43a, 0x2d, 0x41a, 0x430, 0x43c, 0x447, 0x430, 0x442, 0x441,
0x43a, 0x438, 0x41a, 0x430, 0x43b, 0x44c, 0x43a, 0x443, 0x442, 0x442, 0x430, 0x41f,
0x43d, 0x43e, 0x43c, 0x43f, 0x435, 0x43d, 0x44c, 0x42d, 0x440, 0x2d, 0x420, 0x438,
0x44f, 0x434, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20, 0x433, 0x4c0,
0x2d, 0x435, 0x422, 0x435, 0x445, 0x4c0, 0x440, 0x430, 0x43d, 0x423, 0x441, 0x442,
0x44c, 0x2d, 0x41d, 0x435, 0x440, 0x430, 0x412, 0x44c, 0x435, 0x43d, 0x442, 0x44c,
0x44f, 0x43d, 0x410, 0x437, 0x43e, 0x440, 0x430, 0x43d, 0x20, 0x433, 0x4c0, 0x430,
0x439, 0x440, 0x435, 0x43d, 0x430, 0x448, 0x411, 0x435, 0x440, 0x43c, 0x443, 0x434,
0x430, 0x448, 0x41a, 0x430, 0x43d, 0x430, 0x440, 0x430, 0x43d, 0x20, 0x433, 0x4c0,
0x430, 0x439, 0x440, 0x435, 0x43d, 0x430, 0x448, 0x424, 0x430, 0x440, 0x435, 0x440,
0x438, 0x439, 0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439, 0x440, 0x435, 0x43d, 0x430,
0x448, 0x41c, 0x430, 0x434, 0x435, 0x439, 0x440, 0x430, 0x2c, 0x20, 0x433, 0x4c0,
0x2d, 0x435, 0x420, 0x435, 0x439, 0x43a, 0x44c, 0x44f, 0x432, 0x438, 0x43a, 0x41a,
0x44a, 0x438, 0x43b, 0x431, 0x430, 0x20, 0x414, 0x436, 0x43e, 0x440, 0x434, 0x436,
0x438, 0x415, 0x437, 0x430, 0x447, 0x443, 0x20, 0x415, 0x43b, 0x435, 0x43d, 0x438,
0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439, 0x440, 0x435, 0x421, 0x442, 0x44d, 0x43d,
0x43b, 0x438, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x425, 0x430, 0x443, 0x2c, 0x20,
0x433, 0x4c0, 0x2d, 0x435, 0x421, 0x438, 0x434, 0x43d, 0x435, 0x439, 0x410, 0x448,
0x442, 0x430, 0x440, 0x43a, 0x445, 0x43d, 0x435, 0x410, 0x444, 0x438, 0x43d, 0x430,
0x411, 0x440, 0x44e, 0x441, 0x441, 0x435, 0x43b, 0x44c, 0x411, 0x44e, 0x437, 0x438,
0x43d, 0x433, 0x435, 0x43d, 0x2d, 0x430, 0x43c, 0x2d, 0x425, 0x43e, 0x445, 0x440,
0x430, 0x439, 0x43d, 0x41a, 0x43e, 0x43f, 0x435, 0x43d, 0x433, 0x430, 0x433, 0x435,
0x43d, 0x425, 0x435, 0x43b, 0x44c, 0x441, 0x438, 0x43d, 0x43a, 0x438, 0x41c, 0x44d,
0x43d, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x435, 0x414, 0x436, 0x435, 0x440, 0x441,
0x438, 0x421, 0x438, 0x43c, 0x444, 0x435, 0x440, 0x43e, 0x43f, 0x43e, 0x43b, 0x44c,
0x421, 0x43a, 0x43e, 0x43f, 0x44c, 0x435, 0x421, 0x442, 0x43e, 0x43a, 0x433, 0x43e,
0x43b, 0x44c, 0x43c, 0x423, 0x43b, 0x44c, 0x44f, 0x43d, 0x43e, 0x432, 0x441, 0x43a,
0x412, 0x438, 0x43b, 0x44c, 0x43d, 0x44e, 0x441, 0x4c0, 0x438, 0x439, 0x441, 0x430,
0x20, 0x43f, 0x430, 0x439, 0x445, 0x430, 0x43c, 0x430, 0x440, 0x20, 0x432, 0x438,
0x43d, 0x447, 0x443, 0x20, 0x434, 0x435, 0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439,
0x440, 0x435, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x438, 0x439, 0x43d, 0x2c, 0x20,
0x433, 0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x41a, 0x43e, 0x43c, 0x43e, 0x440, 0x438,
0x439, 0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439, 0x440, 0x435, 0x43d, 0x430, 0x448,
0x41c, 0x430, 0x43b, 0x44c, 0x434, 0x438, 0x432, 0x430, 0x448, 0x41c, 0x430, 0x439,
0x43e, 0x440, 0x43a, 0x430, 0x420, 0x435, 0x44e, 0x43d, 0x44c, 0x43e, 0x43d, 0x411,
0x443, 0x433, 0x435, 0x43d, 0x432, 0x438, 0x43b, 0x44c, 0x2c, 0x20, 0x433, 0x4c0,
0x2d, 0x435, 0x427, 0x430, 0x442, 0x435, 0x43c, 0x2c, 0x20, 0x433, 0x4c0, 0x2d,
0x435, 0x422, 0x440, 0x443, 0x43a, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d, 0x430,
0x448, 0x41c, 0x430, 0x440, 0x445, 0x438, 0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439,
0x440, 0x435, 0x43d, 0x430, 0x448, 0x42d, 0x444, 0x430, 0x442, 0x435, 0x413, 0x430,
0x43b, 0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441, 0x430, 0x43d, 0x20, 0x433, 0x4c0,
0x430, 0x439, 0x440, 0x435, 0x43d, 0x430, 0x448, 0x413, 0x430, 0x43c, 0x431, 0x44c,
0x435, 0x43d, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x413, 0x43e,
0x43d, 0x43e, 0x43b, 0x443, 0x43b, 0x443, 0x42d, 0x43d, 0x434, 0x435, 0x440, 0x431,
0x435, 0x440, 0x438, 0x43d, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x435, 0x41c, 0x430,
0x440, 0x43a, 0x438, 0x437, 0x430, 0x43d, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d,
0x430, 0x448, 0x41c, 0x438, 0x434, 0x443, 0x44d, 0x439, 0x2c, 0x20, 0x433, 0x4c0,
0x2d, 0x43d, 0x430, 0x448, 0x41d, 0x438, 0x443, 0x44d, 0x41f, 0x43e, 0x43d, 0x430,
0x43f, 0x435, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x422, 0x430,
0x438, 0x442, 0x438, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x423,
0x44d, 0x439, 0x43a, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x435, 0x13a0, 0x13c8, 0x13e3,
0x13c2, 0x13a0, 0x13c6, 0x13a1, 0x13d7, 0x13cd, 0x20, 0x13a0, 0x13c6, 0x13c6, 0x13a0, 0x13b5,
0x13e5, 0x13b5, 0x13a0, 0x13cf, 0x13b9, 0x13b3, 0x13c6, 0x13b9, 0x13aa, 0x13c7, 0x13c2, 0x13ab,
0x13eb, 0x13c6, 0x13c2, 0x13e7, 0x13b5, 0x13c7, 0x13cc, 0x13eb, 0x13c6, 0x13d8, 0x13b4, 0x13c6,
0x13cc, 0x13e9, 0x13b5, 0x13ca, 0x13e7, 0x13bb, 0x13ca, 0x13b3, 0x13a7, 0x13ef, 0x13b6, 0x13a4,
0x13c1, 0x13a6, 0x20, 0x13a6, 0x13b5, 0x13e6, 0x13d5, 0x13d1, 0x13d4, 0x13aa, 0x13be, 0x13c8,
0x13d3, 0x13a7, 0x13e9, 0x13d3, 0x20, 0x13a1, 0x13cf, 0x20, 0x13cc, 0x13b3, 0x13bb, 0x13e5,
0x13ca, 0x13d7, 0x13d9, 0x13a4, 0x13e9, 0x13b3, 0x13a1, 0x13b5, 0x20, 0x13a0, 0x13f1, 0x13f3,
0x13c2, 0x13a0, 0x13ce, 0x13c7, 0x20, 0x13a6, 0x13da, 0x13b2, 0x13a6, 0x13c9, 0x13b6, 0x13c1,
0x13ad, 0x13b3, 0x13b4, 0x13e6, 0x13ad, 0x13c1, 0x13cd, 0x13ca, 0x13a6, 0x13e7, 0x13c6, 0x13a7,
0x13bb, 0x13c6, 0x13b3, 0x13a7, 0x13da, 0x13bb, 0x13a9, 0x13a6, 0x13b5, 0x13a8, 0x13c2, 0x13dd,
0x13cc, 0x13b3, 0x13aa, 0x13cf, 0x13b5, 0x13c7, 0x13eb, 0x13b5, 0x13b6, 0x13ba, 0x13b7, 0x13e9,
0x13c2, 0x13d3, 0x13b7, 0x13ca, 0x13c6, 0x13cf, 0x13b5, 0x13cc, 0x13a7, 0x13b9, 0x13b3, 0x13c9,
0x13b9, 0x13ca, 0x13d9, 0x13b9, 0x13ce, 0x13b5, 0x13c6, 0x13c7, 0x13c1, 0x13bc, 0x13a6, 0x13d7,
0x13e1, 0x13bc, 0x13c2, 0x13b6, 0x13eb, 0x13ef, 0x13be, 0x13f1, 0x13b6, 0x13c8, 0x13be, 0x13e3,
0x13ba, 0x13be, 0x13c2, 0x13ef, 0x13ba, 0x13be, 0x13ec, 0x13e3, 0x13d8, 0x13a3, 0x13e9, 0x13a6,
0x13da, 0x13ab, 0x13e5, 0x13f3, 0x13d7, 0x13d4, 0x13b3, 0x13d7, 0x13cd, 0x13d7, 0x2d, 0x13c3,
0x13ec, 0x13cc, 0x13a3, 0x13d9, 0x13ba, 0x13df, 0x13c9, 0x13b5, 0x13da, 0x13c2, 0x13cf, 0x13ea,
0x13c4, 0x13d7, 0x13b0, 0x13a9, 0x13a0, 0x13d3, 0x13a7, 0x13a0, 0x13c2, 0x13aa, 0x13b4, 0x13e5,
0x13a0, 0x13c2, 0x13a9, 0x13b3, 0x13a4, 0x13ea, 0x13d8, 0x13a0, 0x13b3, 0x13ab, 0x13e9, 0x13f1,
0x13be, 0x13a3, 0x13cd, 0x13d3, 0x20, 0x13a4, 0x13c3, 0x13b4, 0x13a7, 0x13d4, 0x13b9, 0x13a7,
0x13aa, 0x13d9, 0x13c6, 0x13e7, 0x13e7, 0x13eb, 0x13b3, 0x20, 0x13b5, 0x13f2, 0x13ad, 0x13ba,
0x13be, 0x13d9, 0x13cc, 0x13a6, 0x13f0, 0x13aa, 0x13cf, 0x20, 0x13a4, 0x13ea, 0x13f4, 0x13cc,
0x13b3, 0x13d4, 0x13cc, 0x13c2, 0x20, 0x13e9, 0x13c2, 0x13a4, 0x13d3, 0x13c5, 0x13d7, 0x20,
0x13b7, 0x13eb, 0x13cf, 0x13da, 0x13ab, 0x13b9, 0x13c2, 0x13a4, 0x13d1, 0x13e9, 0x13ef, 0x13a0,
0x13b7, 0x13c6, 0x13a0, 0x13d1, 0x13cf, 0x13f2, 0x13c5, 0x13a0, 0x13d8, 0x13aa, 0x13a6, 0x13c2,
0x13c6, 0x13af, 0x13ef, 0x13c6, 0x13af, 0x13ef, 0x20, 0x13c6, 0x13c2, 0x13d5, 0x13b3, 0x13cf,
0x13c6, 0x13c7, 0x13d9, 0x13cd, 0x13c7, 0x13b4, 0x13bb, 0x13c7, 0x13b5, 0x13cd, 0x13dd, 0x13c2,
0x2d, 0x13cc, 0x13e0, 0x13c2, 0x13a3, 0x13cd, 0x13d3, 0x20, 0x13a0, 0x13aa, 0x13b5, 0x13f0,
0x13d7, 0x13c9, 0x13aa, 0x13d4, 0x13c9, 0x13f1, 0x13cf, 0x13a8, 0x13bb, 0x13c8, 0x13e5, 0x20,
0x13a1, 0x13c9, 0x13c4, 0x13b8, 0x13d7, 0x13a4, 0x13d4, 0x13be, 0x20, 0x13e0, 0x13a8, 0x13cf,
0x13a8, 0x13c2, 0x13ab, 0x13c2, 0x13a7, 0x13b3, 0x13a7, 0x13cf, 0x13a7, 0x13f0, 0x13c2, 0x13a8,
0x13a2, 0x13b9, 0x13c2, 0x13e5, 0x13a7, 0x13a9, 0x13e5, 0x13e9, 0x13e9, 0x13cf, 0x13f3, 0x13d3,
0x20, 0x13c6, 0x13a1, 0x13cf, 0x13aa, 0x13cd, 0x13d3, 0x13b5, 0x13a7, 0x13de, 0x13cd, 0x13d4,
0x13c2, 0x13e7, 0x13eb, 0x13c6, 0x13ab, 0x13b3, 0x13a8, 0x13a3, 0x13d5, 0x13be, 0x13b9, 0x13a7,
0x13cc, 0x13e9, 0x13c2, 0x13d3, 0x13cc, 0x13c2, 0x13d3, 0x13cc, 0x13c2, 0x20, 0x13a4, 0x13ea,
0x13f4, 0x13a6, 0x13b8, 0x13b3, 0x13d7, 0x20, 0x13a6, 0x13da, 0x13b2, 0x13d7, 0x13e0, 0x13d8,
0x13d9, 0x13bb, 0x13c2, 0x13a7, 0x13a1, 0x13d7, 0x13b9, 0x13c2, 0x13d4, 0x13c2, 0x13a1, 0x13b7,
0x13c1, 0x13c7, 0x13a1, 0x13b5, 0x20, 0x13cc, 0x13b5, 0x13c6, 0x13d9, 0x13b5, 0x13d7, 0x13d0,
0x13f4, 0x20, 0x13c1, 0x13b5, 0x13cc, 0x13c2, 0x13ec, 0x13d4, 0x13b4, 0x13ce, 0x13de, 0x13cf,
0x20, 0x13a1, 0x13c9, 0x13c4, 0x13b8, 0x13d7, 0x13cc, 0x13cc, 0x20, 0x13a1, 0x13c9, 0x13c4,
0x13b8, 0x13d7, 0x13c4, 0x13ac, 0x13eb, 0x13f3, 0x13d2, 0x20, 0x13ac, 0x13be, 0x13cb, 0x13be,
0x13d3, 0x13e9, 0x13d3, 0x13b7, 0x13c7, 0x13e9, 0x13d4, 0x13b9, 0x13b3, 0x13c6, 0x13ef, 0x13a9,
0x13b5, 0x13a6, 0x13ef, 0x13be, 0x13ad, 0x13b5, 0x13e9, 0x13a7, 0x13cf, 0x13ad, 0x13e9, 0x13be,
0x13ae, 0x13bc, 0x13cf, 0x13b6, 0x13a2, 0x13c2, 0x13d7, 0x13a0, 0x13c2, 0x13a0, 0x13c9, 0x13b5,
0x13cf, 0x13c3, 0x13c8, 0x13cf, 0x2c, 0x20, 0x13a2, 0x13c2, 0x13d7, 0x13f0, 0x13be, 0x13b9,
0x13b4, 0x13c2, 0x13aa, 0x2c, 0x20, 0x13a2, 0x13c2, 0x13d7, 0x13f0, 0x13be, 0x13c8, 0x13d3,
0x13c8, 0x13a6, 0x2c, 0x20, 0x13a2, 0x13c2, 0x13d7, 0x13f0, 0x13be, 0x13d6, 0x13b5, 0x20,
0x13a6, 0x13da, 0x13b2, 0x2c, 0x20, 0x13a2, 0x13c2, 0x13d7, 0x13f0, 0x13be, 0x13ea, 0x13ea,
0x2c, 0x20, 0x13a2, 0x13c2, 0x13d7, 0x13f0, 0x13be, 0x13eb, 0x13c2, 0x13ce, 0x13c1, 0x13cf,
0x2c, 0x20, 0x13a2, 0x13c2, 0x13d7, 0x13f0, 0x13be, 0x13eb, 0x13be, 0x13b9, 0x13a9, 0x2c,
0x20, 0x13a2, 0x13c2, 0x13d7, 0x13f0, 0x13be, 0x13a2, 0x13c4, 0x13eb, 0x13a9, 0x13a2, 0x13c6,
0x13b7, 0x13f1, 0x13d8, 0x13e3, 0x13ba, 0x13a2, 0x13a7, 0x13e7, 0x13c3, 0x13b7, 0x13eb, 0x13eb,
0x13b5, 0x13b9, 0x13c2, 0x13d4, 0x13ce, 0x13b6, 0x2c, 0x20, 0x13ac, 0x13c5, 0x13d3, 0x13a9,
0x13c6, 0x13b4, 0x13c2, 0x13d7, 0x13d9, 0x13af, 0x13be, 0x13cd, 0x13a9, 0x20, 0x13d7, 0x13c2,
0x13a7, 0x13bf, 0x13e9, 0x13d7, 0x13d9, 0x13af, 0x13a1, 0x13b3, 0x13d7, 0x20, 0x13d7, 0x13dc,
0x20, 0x13a4, 0x13ac, 0x13eb, 0x13f3, 0x13af, 0x20, 0x13a9, 0x13c4, 0x13d9, 0x13d7, 0x13b9,
0x13ce, 0x13f2, 0x13b9, 0x13be, 0x13c6, 0x13b9, 0x13be, 0x13a4, 0x13cf, 0x13b9, 0x13b5, 0x13a6,
0x13d8, 0x13b9, 0x13d8, 0x13c2, 0x13c7, 0x13b9, 0x13d4, 0x13bc, 0x13b6, 0x13cf, 0x13b9, 0x13cc,
0x13dd, 0x13c2, 0x13ba, 0x13c3, 0x13bb, 0x13c2, 0x13ba, 0x13b5, 0x13d3, 0x13ba, 0x13dd, 0x13a7,
0x13dd, 0x13a0, 0x13c2, 0x13cd, 0x13c6, 0x13c2, 0x20, 0x13a6, 0x13da, 0x13b2, 0x13bb, 0x13a8,
0x13b6, 0x13c2, 0x13b9, 0x13be, 0x13d4, 0x13c2, 0x13bc, 0x13d6, 0x13b5, 0x13bc, 0x13c2, 0x13d6,
0x13eb, 0x13d5, 0x13f2, 0x13b9, 0x13c2, 0x13d8, 0x13cc, 0x13b3, 0x13d7, 0x13be, 0x13cc, 0x13eb,
0x13c4, 0x13ef, 0x13a9, 0x13c3, 0x13ba, 0x13c3, 0x13b6, 0x13be, 0x13ad, 0x13c7, 0x13f3, 0x13b3,
0x2c, 0x20, 0x13e7, 0x13f4, 0x13e2, 0x20, 0x13d3, 0x13aa, 0x13d4, 0x13a0, 0x13f0, 0x13df,
0x2c, 0x20, 0x13e7, 0x13f4, 0x13e2, 0x20, 0x13d3, 0x13aa, 0x13d4, 0x13a2, 0x13e4, 0x20,
0x13ce, 0x13b4, 0x13bb, 0x2c, 0x20, 0x13e7, 0x13f4, 0x13e2, 0x20, 0x13d3, 0x13aa, 0x13d4,
0x13c4, 0x13a9, 0x13a3, 0x13e5, 0x13be, 0x13a6, 0x13c6, 0x13be, 0x13b9, 0x13c6, 0x13b3, 0x13b9,
0x13b4, 0x13c9, 0x13e7, 0x13b4, 0x13af, 0x13cc, 0x13c5, 0x13af, 0x13e5, 0x13f3, 0x13d7, 0x13d4,
0x13b3, 0x13d7, 0x13cd, 0x13d7, 0x2d, 0x13be, 0x13bf, 0x2d, 0x13a4, 0x13ac, 0x13eb, 0x13f3,
0x13af, 0x13e5, 0x13f3, 0x13d7, 0x13d4, 0x13b3, 0x13d7, 0x13cd, 0x13d7, 0x20, 0x13cd, 0x13c6,
0x13c2, 0x13a8, 0x13cd, 0x13db, 0x13a4, 0x13ea, 0x13d8, 0x20, 0x13e5, 0x13f3, 0x13d7, 0x13d4,
0x13b3, 0x13d7, 0x13cd, 0x13d7, 0x13c7, 0x13a1, 0x13d9, 0x20, 0x13b5, 0x13a2, 0x13aa, 0x13ca,
0x13d4, 0x20, 0x13a1, 0x13eb, 0x13be, 0x13cd, 0x13b4, 0x13c2, 0x13a9, 0x13c2, 0x20, 0x13a2,
0x13c2, 0x13b4, 0x13d8, 0x13b4, 0x13cf, 0x13ea, 0x13b4, 0x13a9, 0x13be, 0x13b4, 0x13d0, 0x13b7,
0x13d8, 0x13a4, 0x13c1, 0x13a6, 0x20, 0x13a4, 0x13ea, 0x13f4, 0x13cc, 0x13c2, 0x13d4, 0x13b4,
0x13bb, 0x13cc, 0x13c2, 0x13d8, 0x13ef, 0x13aa, 0x13cc, 0x13c2, 0x13d9, 0x20, 0x13d9, 0x13bb,
0x13aa, 0x13cc, 0x13a3, 0x20, 0x13c6, 0x13b6, 0x13a2, 0x13d9, 0x13aa, 0x13d9, 0x13bb, 0x13d8,
0x13cf, 0x13d8, 0x13a7, 0x13a4, 0x13d3, 0x13c5, 0x13d8, 0x20, 0x13c6, 0x13b5, 0x13de, 0x13b4,
0x13bb, 0x13a4, 0x13d3, 0x13c5, 0x13d8, 0x20, 0x13e3, 0x13c2, 0x20, 0x13a4, 0x13e4, 0x13b5,
0x13a4, 0x13d3, 0x13c5, 0x13d8, 0x20, 0x13a9, 0x13d8, 0x13cf, 0x13a4, 0x13d3, 0x13c5, 0x13d8,
0x20, 0x13b7, 0x13cf, 0x13ef, 0x13a4, 0x13d3, 0x13c5, 0x13d8, 0x20, 0x13d9, 0x13b9, 0x13cf,
0x13a4, 0x13d3, 0x13c5, 0x13d8, 0x20, 0x13eb, 0x13c2, 0x13ce, 0x13d8, 0x13a0, 0x13ef, 0x13c4,
0x13b5, 0x20, 0x13a4, 0x13c3, 0x13b4, 0x13d6, 0x13ab, 0x13cf, 0x13a6, 0x13b5, 0x13c6, 0x13e1,
0x13b5, 0x13d8, 0x13f3, 0x13e9, 0x13be, 0x13d9, 0x13b3, 0x13be, 0x13d9, 0x13d9, 0x13d9, 0x13b3,
0x13ea, 0x13c2, 0x13ab, 0x13ea, 0x13b5, 0x13a4, 0x13c1, 0x13a6, 0x20, 0x13d0, 0x13c8, 0x13b5,
0x13eb, 0x13c2, 0x13c7, 0x13a9, 0x13ef, 0x13ab, 0x13d4, 0x13d8, 0x13d5, 0x13eb, 0x13cf, 0x13da,
0x13bc, 0x13be, 0x13d8, 0x2d, 0x13d7, 0x2019, 0x13a4, 0x13b5, 0x13eb, 0x13b5, 0x13b9, 0x13c7,
0x13b5, 0x13b9, 0x13cc, 0x13c2, 0x13bb, 0x13a6, 0x13bd, 0x13d9, 0x13c6, 0x13b5, 0x13ba, 0x13b5,
0x13b3, 0x13de, 0x13b3, 0x13cf, 0x13f2, 0x13e9, 0x13e0, 0x13b5, 0x13ec, 0x13cd, 0x13d9, 0x13a7,
0x13a6, 0x13c5, 0x13af, 0x13d3, 0x20, 0x13a4, 0x13d5, 0x13d8, 0x13f4, 0x13cc, 0x13d7, 0x13d2,
0x13a2, 0x20, 0x13a6, 0x13da, 0x13b2, 0x13a1, 0x13d5, 0x13c2, 0x13a0, 0x13b5, 0x13b9, 0x13d8,
0x13a0, 0x13b9, 0x13c2, 0x13a0, 0x13be, 0x13d7, 0x13b5, 0x13a0, 0x13a6, 0x13d4, 0x13eb, 0x13a0,
0x13a6, 0x13d9, 0x13c7, 0x13a0, 0x13cd, 0x13a6, 0x13c6, 0x13d8, 0x13a0, 0x13d8, 0x13c6, 0x13a4,
0x13c6, 0x13a9, 0x13d3, 0x13d7, 0x13c6, 0x13ad, 0x13b4, 0x13a2, 0x13c2, 0x13c7, 0x13c2, 0x13a9,
0x13aa, 0x13a9, 0x13c6, 0x13be, 0x13a3, 0x13b5, 0x13c7, 0x13b7, 0x13d8, 0x13c7, 0x13cd, 0x13a8,
0x13a9, 0x13ca, 0x13be, 0x13a2, 0x13e5, 0x13d4, 0x13aa, 0x13b6, 0x13bb, 0x13c9, 0x13d3, 0x13b9,
0x13cd, 0x13ac, 0x13cf, 0x13da, 0x13c6, 0x13f1, 0x13da, 0x13dd, 0x13be, 0x13c7, 0x13c6, 0x13b9,
0x13ab, 0x13cd, 0x13d4, 0x13a6, 0x13cc, 0x13ae, 0x13c9, 0x13c2, 0x13b0, 0x20, 0x13e5, 0x20,
0x13bb, 0x13c2, 0x20, 0x13a6, 0x13da, 0x13b2, 0x13b0, 0x13c2, 0x13a9, 0x20, 0x13aa, 0x13c2,
0x13a9, 0x13b0, 0x13e9, 0x13d7, 0x13a2, 0x13ab, 0x13e5, 0x13a7, 0x13e3, 0x13a7, 0x13d4, 0x13e3,
0x13ef, 0x13cb, 0x13b3, 0x13e4, 0x13b7, 0x13cc, 0x13b4, 0x13bb, 0x13a7, 0x13ca, 0x13b5, 0x13a7,
0x13bb, 0x13e3, 0x13a7, 0x13a7, 0x13b3, 0x13e5, 0x13a7, 0x13d8, 0x13b9, 0x13c2, 0x13da, 0x13a7,
0x13c2, 0x13d7, 0x13a6, 0x13aa, 0x13b5, 0x13a7, 0x13d4, 0x13dd, 0x13cd, 0x13c3, 0x13ef, 0x13cd,
0x13a7, 0x13ab, 0x13e9, 0x13b3, 0x20, 0x13b8, 0x13bb, 0x13cb, 0x13ab, 0x13e5, 0x13c2, 0x13a6,
0x13ab, 0x13ea, 0x13d8, 0x13b9, 0x13a7, 0x13a4, 0x13b9, 0x13a6, 0x13d3, 0x13c2, 0x13b9, 0x13a7,
0x13cc, 0x13b5, 0x13b9, 0x13c2, 0x13b3, 0x13bd, 0x13cd, 0x13a6, 0x13d8, 0x13c2, 0x13aa, 0x13cf,
0x13ef, 0x13c3, 0x13ec, 0x13ab, 0x13c1, 0x13d6, 0x13cd, 0x13a7, 0x13c3, 0x13ec, 0x13cf, 0x13c8,
0x13cd, 0x13a7, 0x13a3, 0x13bb, 0x13cd, 0x13a7, 0x13a3, 0x13b3, 0x13b5, 0x13bf, 0x13bb, 0x20,
0x13c7, 0x13c2, 0x13c6, 0x13c2, 0x13d8, 0x13a0, 0x13be, 0x13a9, 0x13c8, 0x13ef, 0x13c2, 0x13a9,
0x13f0, 0x13c2, 0x13a9, 0x13a7, 0x13d4, 0x13b5, 0x13a7, 0x13cd, 0x13d4, 0x13c1, 0x13a9, 0x13cf,
0x13b6, 0x13b3, 0x13d3, 0x13b5, 0x13ef, 0x13d7, 0x13cc, 0x13a7, 0x13b5, 0x13c2, 0x13cc, 0x13b9,
0x13a7, 0x13c2, 0x13d7, 0x13d0, 0x13b5, 0x13ce, 0x13c2, 0x13a6, 0x13ad, 0x13f1, 0x13cf, 0x13c2,
0x13a6, 0x13c9, 0x13b5, 0x13cd, 0x13b4, 0x13d7, 0x13c1, 0x13aa, 0x13b5, 0x13bb, 0x13cd, 0x13a7,
0x13d4, 0x13f1, 0x13c7, 0x13d4, 0x13cf, 0x13a8, 0x13c2, 0x13d8, 0x13d8, 0x13c8, 0x13b5, 0x13cf,
0x13d6, 0x13b3, 0x13c2, 0x13de, 0x13bb, 0x13e1, 0x13d9, 0x13a9, 0x13f2, 0x13d9, 0x13bb, 0x13cd,
0x13a7, 0x13a4, 0x13b3, 0x13c2, 0x13c6, 0x13d4, 0x13a4, 0x13b7, 0x13bb, 0x13a9, 0x13a4, 0x13cd,
0x13d4, 0x2d, 0x13c1, 0x13b3, 0x13eb, 0x13f0, 0x13c2, 0x13d8, 0x13ef, 0x13c1, 0x13ed, 0x13b3,
0x13d7, 0x13ec, 0x13cd, 0x13d9, 0x13a9, 0x13ef, 0x13ab, 0x13e5, 0x13a7, 0x13b3, 0x13c2, 0x13ab,
0x13c2, 0x13f0, 0x13a7, 0x13d6, 0x13b5, 0x13c2, 0x13ca, 0x13a6, 0x13f0, 0x13b4, 0x13ea, 0x13c2,
0x13a0, 0x13d0, 0x13b4, 0x13cf, 0x13c6, 0x13ca, 0x13d3, 0x13e5, 0x13cd, 0x13c6, 0x13a2, 0x13ac,
0x13be, 0x13d5, 0x13be, 0x20, 0x13a2, 0x13e4, 0x13f3, 0x13cd, 0x13d7, 0x13ea, 0x13b6, 0x13b9,
0x13d5, 0x13b3, 0x13b4, 0x13e3, 0x13eb, 0x13a9, 0x13e7, 0x13a6, 0x13be, 0x13ee, 0x20, 0x13e3,
0x13a0, 0x13e5, 0x13a2, 0x13a4, 0x13d3, 0x13c5, 0x13d8, 0x20, 0x13ae, 0x13b5, 0x13be, 0x13cd,
0x13d5, 0x13c2, 0x13b5, 0x13a1, 0x13d5, 0x13b4, 0x13d7, 0x13c7, 0x13cd, 0x13c7, 0x13c2, 0x13a4,
0x13f2, 0x13e8, 0x13af, 0x20, 0x13a6, 0x13da, 0x13cf, 0x13d3, 0x13e9, 0x13c2, 0x13f3, 0x13dd,
0x13b0, 0x13c6, 0x13d8, 0x13b4, 0x13be, 0x13d5, 0x13b9, 0x13c2, 0x13a4, 0x13ac, 0x13eb, 0x13f3,
0x13af, 0x20, 0x13ad, 0x13eb, 0x13ba, 0x13b5, 0x13c9, 0x13c1, 0x13c7, 0x13b5, 0x13dd, 0x13cf,
0x13d7, 0x13c2, 0x13a0, 0x13bb, 0x13cd, 0x13d5, 0x13d3, 0x13bb, 0x13a0, 0x13c2, 0x13d9, 0x13e9,
0x13a0, 0x13dd, 0x13a7, 0x13c2, 0x13a0, 0x13d6, 0x13c2, 0x13cf, 0x13c7, 0x13b5, 0x13c7, 0x13d7,
0x13c7, 0x13b5, 0x13c2, 0x13c6, 0x13d8, 0x13cd, 0x13b3, 0x13e9, 0x13cb, 0x13ce, 0x13b5, 0x13cf,
0x13c7, 0x13e3, 0x13b4, 0x13cd, 0x13d7, 0x13ca, 0x13d3, 0x13c7, 0x13cd, 0x13d7, 0x13ca, 0x13cf,
0x13c2, 0x13a8, 0x13c2, 0x13e5, 0x13cf, 0x13be, 0x13eb, 0x13aa, 0x13c7, 0x13c2, 0x13ae, 0x13a8,
0x13c2, 0x13db, 0x13b5, 0x13c2, 0x13e5, 0x13c6, 0x13b5, 0x13d3, 0x13ac, 0x13c2, 0x13cf, 0x13ae,
0x13b5, 0x13cf, 0x13c2, 0x13a9, 0x13a4, 0x13cd, 0x13d7, 0x13a4, 0x13a6, 0x13da, 0x13db, 0x20,
0x13be, 0x13bf, 0x20, 0x13a0, 0x13cd, 0x13a6, 0x13ef, 0x13a2, 0x13cd, 0x13d4, 0x13c2, 0x13ca,
0x13b5, 0x13e8, 0x13b5, 0x13cf, 0x13a7, 0x13b5, 0x13c2, 0x13c6, 0x13d7, 0x13a9, 0x13b6, 0x13e9,
0x13a9, 0x13eb, 0x13b5, 0x13cf, 0x13c9, 0x13c2, 0x13e7, 0x13e3, 0x13be, 0x13b8, 0x13c2, 0x13d3,
0x13c2, 0x13b8, 0x13a7, 0x13ce, 0x13cb, 0x13a9, 0x13b9, 0x13df, 0x13d7, 0x13b9, 0x13b5, 0x13d4,
0x13ba, 0x13b5, 0x13ad, 0x13bb, 0x13ba, 0x13c2, 0x13cd, 0x13a9, 0x13bc, 0x13be, 0x13aa, 0x13b9,
0x13cd, 0x13a6, 0x13eb, 0x13a3, 0x13cf, 0x13b6, 0x13c7, 0x13eb, 0x13cf, 0x13c9, 0x13aa, 0x13b5,
0x13a7, 0x13cc, 0x13b9, 0x13b3, 0x13b9, 0x13b5, 0x13c3, 0x13cc, 0x13b3, 0x13e4, 0x13ec, 0x13cc,
0x13c6, 0x13d9, 0x13a5, 0x13cf, 0x13bb, 0x13ea, 0x13b6, 0x13c9, 0x13b5, 0x13cd, 0x13aa, 0x13e4,
0x13d0, 0x13df, 0x13a0, 0x13cd, 0x13d3, 0x13a9, 0x13b0, 0x13bb, 0x13d4, 0x13b5, 0x13c2, 0x13d8,
0x13b3, 0x13be, 0x13a4, 0x13b5, 0x13ef, 0x13c3, 0x13ec, 0x13cd, 0x13a7, 0x13e9, 0x13da, 0x13cf,
0x13a0, 0x13e5, 0x13b3, 0x13c1, 0x13e0, 0x13eb, 0x13f0, 0x13be, 0x13eb, 0x13b5, 0x13c2, 0x13f4,
0x13cf, 0x13ec, 0x13b6, 0x13aa, 0x13dd, 0x13d7, 0x13d3, 0x13bf, 0x13e9, 0x20, 0x13a4, 0x13aa,
0x13b2, 0x13a9, 0x13cc, 0x13c7, 0x13c8, 0x13d1, 0x13b5, 0x13e5, 0x13a0, 0x13c2, 0x13d4, 0x13be,
0x13be, 0x13b5, 0x13ec, 0x13e3, 0x13aa, 0x13cf, 0x13d3, 0x13c2, 0x13cd, 0x13d3, 0x13f2, 0x13af,
0x13b2, 0x13aa, 0x13aa, 0x13cd, 0x13aa, 0x13bc, 0x13b3, 0x13ac, 0x13b5, 0x13ab, 0x13f0, 0x13b4,
0x13c2, 0x13b9, 0x13ae, 0x13b9, 0x13b5, 0x13d7, 0x13eb, 0x13cd, 0x13bc, 0x13b5, 0x13cf, 0x13a5,
0x13cd, 0x13ba, 0x13ef, 0x13d6, 0x13b4, 0x13f3, 0x13c2, 0x13a0, 0x13c2, 0x13a0, 0x13c8, 0x13a0,
0x13a0, 0x13a6, 0x13b3, 0x13be, 0x13d7, 0x13ca, 0x13a8, 0x13c2, 0x13eb, 0x13b5, 0x13e3, 0x13dd,
0x13bb, 0x13e7, 0x13a9, 0x13e5, 0x13cc, 0x13d5, 0x13b4, 0x13af, 0x13cc, 0x13c5, 0x13a1, 0x13e9,
0x13d6, 0x13e9, 0x13a7, 0x13a3, 0x13ec, 0x13eb, 0x13e5, 0x13e1, 0x13be, 0x13e1, 0x13d8, 0x13a1,
0x13c6, 0x20, 0x13d3, 0x13a6, 0x13cf, 0x20, 0x13a4, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x13a6,
0x13bb, 0x13c7, 0x13b5, 0x13c6, 0x13d3, 0x13b5, 0x13a7, 0x13be, 0x13b5, 0x13c6, 0x13bb, 0x13ad,
0x13c3, 0x13b7, 0x13b7, 0x13a1, 0x13c2, 0x13c7, 0x13b5, 0x13a9, 0x13b5, 0x13d8, 0x13b9, 0x13d8,
0x13aa, 0x13cd, 0x13b4, 0x13c6, 0x13e3, 0x13b4, 0x13a2, 0x13c2, 0x13b9, 0x13e7, 0x13b6, 0x13b9,
0x13b5, 0x13a8, 0x13cc, 0x13cf, 0x13a0, 0x13f0, 0x13df, 0x13f4, 0x13da, 0x13c3, 0x13a4, 0x13b7,
0x13c3, 0x13b5, 0x13ec, 0x13b5, 0x13a9, 0x13c3, 0x13a4, 0x13ba, 0x13a0, 0x13c6, 0x13aa, 0x20,
0x13c6, 0x13aa, 0x13c6, 0x13b4, 0x13a0, 0x13eb, 0x13c8, 0x13a7, 0x13b5, 0x13c2, 0x13a4, 0x13c9,
0x13c2, 0x13c7, 0x13e5, 0x13f3, 0x13d7, 0x13d4, 0x13b3, 0x13d7, 0x13cd, 0x13d7, 0x20, 0x13bc,
0x13b5, 0x13cd, 0x13c8, 0x13b3, 0x13b6, 0x13d9, 0x13c2, 0x13a6, 0x13cc, 0x13f1, 0x13c6, 0x13c2,
0x13d4, 0x13af, 0x13d8, 0x13d4, 0x13b3, 0x13e9, 0x13d9, 0x13be, 0x13a6, 0x13d4, 0x13ca, 0x13a4,
0x13f0, 0x13e8, 0x13e9, 0x13b5, 0x13cd, 0x4e9a, 0x7684, 0x65af, 0x4e9a, 0x8d1d, 0x5df4, 0x963f,
0x65af, 0x9a6c, 0x62c9, 0x73ed, 0x73e0, 0x5c14, 0x5e03, 0x743c, 0x5e03, 0x62c9, 0x79d1, 0x7eb3,
0x514b, 0x91cc, 0x8fbe, 0x7d2f, 0x65af, 0x8428, 0x62c9, 0x59c6, 0x5409, 0x5e03, 0x63d0, 0x5f17,
0x91cc, 0x6566, 0x54c8, 0x535a, 0x7f57, 0x5185, 0x7ea6, 0x7ff0, 0x5185, 0x65af, 0x5821, 0x91d1,
0x6c99, 0x8428, 0x62c9, 0x5404, 0x65af, 0x5229, 0x4f2f, 0x7ef4, 0x5c14, 0x5362, 0x8428, 0x5361,
0x9a6c, 0x585e, 0x5362, 0x59c6, 0x5df4, 0x5df4, 0x7eb3, 0x6469, 0x52a0, 0x8fea, 0x6c99, 0x5185,
0x7f57, 0x6bd5, 0x52aa, 0x74e6, 0x514b, 0x8096, 0x7279, 0x6ce2, 0x591a, 0x8bfa, 0x4f0f, 0x57c3,
0x8fbe, 0x514b, 0x5b89, 0x514b, 0x96f7, 0x5947, 0x5b89, 0x572d, 0x62c9, 0x5b89, 0x63d0, 0x74dc,
0x963f, 0x62c9, 0x74dc, 0x4f0a, 0x7eb3, 0x79d1, 0x5c14, 0x591a, 0x74e6, 0x62c9, 0x91cc, 0x5965,
0x54c8, 0x5723, 0x8def, 0x6613, 0x65af, 0x56fe, 0x5e93, 0x66fc, 0x963f, 0x9c81, 0x5df4, 0x5df4,
0x5df4, 0x591a, 0x65af, 0x4f2f, 0x5229, 0x5179, 0x5e03, 0x5170, 0x514b, 0x8428, 0x5e03, 0x9686,
0x535a, 0x963f, 0x7ef4, 0x65af, 0x5854, 0x535a, 0x4f0a, 0x897f, 0x5927, 0x574e, 0x666e, 0x52a0,
0x62c9, 0x52a0, 0x65af, 0x5361, 0x5bb4, 0x5947, 0x74e6, 0x74e6, 0x54e5, 0x65af, 0x8fbe, 0x9ece,
0x52a0, 0x5e93, 0x4e9a, 0x5df4, 0x591a, 0x7c73, 0x5c3c, 0x52a0, 0x57c3, 0x5fb7, 0x8499, 0x987f,
0x4f9d, 0x4f26, 0x5c3c, 0x8d1d, 0x798f, 0x5854, 0x96f7, 0x8428, 0x683c, 0x83b1, 0x65af, 0x8d1d,
0x53e4, 0x65af, 0x6e7e, 0x683c, 0x6797, 0x7eb3, 0x8fbe, 0x74dc, 0x5fb7, 0x7f57, 0x666e, 0x5371,
0x5730, 0x9a6c, 0x62c9, 0x572d, 0x4e9a, 0x90a3, 0x54c8, 0x5229, 0x6cd5, 0x514b, 0x65af, 0x5370,
0x7b2c, 0x5b89, 0x7eb3, 0x6ce2, 0x5229, 0x65af, 0x5370, 0x7b2c, 0x5b89, 0x7eb3, 0x5dde, 0x8bfa,
0x514b, 0x65af, 0x5370, 0x7b2c, 0x5b89, 0x7eb3, 0x5dde, 0x9a6c, 0x4f26, 0x6208, 0x5370, 0x7b2c,
0x5b89, 0x7eb3, 0x5dde, 0x5f7c, 0x5f97, 0x65af, 0x5821, 0x5370, 0x7b2c, 0x5b89, 0x7eb3, 0x5dde,
0x7279, 0x5c14, 0x57ce, 0x5370, 0x7b2c, 0x5b89, 0x7eb3, 0x5dde, 0x7ef4, 0x7ef4, 0x5e02, 0x5370,
0x7b2c, 0x5b89, 0x7eb3, 0x5dde, 0x6e29, 0x68ee, 0x65af, 0x5370, 0x7b2c, 0x5b89, 0x7eb3, 0x5dde,
0x5a01, 0x7eb3, 0x9a6c, 0x514b, 0x4f0a, 0x52aa, 0x7ef4, 0x514b, 0x80af, 0x5854, 0x57fa, 0x5dde,
0x8499, 0x8482, 0x585e, 0x6d1b, 0x9a6c, 0x585e, 0x7ea6, 0x9a6c, 0x90a3, 0x74dc, 0x9a6c, 0x7459,
0x65af, 0x9a6c, 0x63d0, 0x5c3c, 0x514b, 0x6885, 0x8bfa, 0x7c73, 0x5c3c, 0x58a8, 0x897f, 0x54e5,
0x57ce, 0x5bc6, 0x514b, 0x9686, 0x8499, 0x7279, 0x96f7, 0x8499, 0x5f97, 0x7ef4, 0x7684, 0x4e9a,
0x8499, 0x7279, 0x585e, 0x62c9, 0x7279, 0x62ff, 0x9a9a, 0x6d1b, 0x7f57, 0x5c3c, 0x4e9a, 0x5317,
0x8fbe, 0x79d1, 0x4ed6, 0x5dde, 0x6bd4, 0x5c24, 0x62c9, 0x5317, 0x8fbe, 0x79d1, 0x4ed6, 0x5dde,
0x7533, 0x7279, 0x5317, 0x8fbe, 0x79d1, 0x4ed6, 0x5dde, 0x65b0, 0x585e, 0x52d2, 0x59c6, 0x5e15,
0x62c9, 0x9a6c, 0x91cc, 0x535a, 0x6ce2, 0x591a, 0x97e6, 0x67f3, 0x7d2f, 0x897f, 0x8153, 0x96f7,
0x7d22, 0x5362, 0x7279, 0x91cc, 0x5965, 0x5e03, 0x90ce, 0x5e93, 0x5723, 0x5730, 0x4e9a, 0x54e5,
0x5723, 0x591a, 0x660e, 0x5404, 0x65af, 0x79d1, 0x5217, 0x65af, 0x6bd4, 0x6851, 0x5fb7, 0x5723,
0x5df4, 0x6cf0, 0x52d2, 0x7c73, 0x5c9b, 0x5723, 0x7ea6, 0x7ff0, 0x65af, 0x5723, 0x5362, 0x897f,
0x4e9a, 0x5723, 0x6587, 0x68ee, 0x7279, 0x7279, 0x53e4, 0x897f, 0x52a0, 0x5c14, 0x5df4, 0x56fe,
0x52d2, 0x8482, 0x534e, 0x7eb3, 0x4e9a, 0x5e93, 0x5854, 0x7279, 0x5361, 0x585e, 0x8fea, 0x8499,
0xb7, 0x8fea, 0x7ef4, 0x5c14, 0x9ea6, 0x683c, 0x7406, 0x5e15, 0x5c14, 0x9ed8, 0x7279, 0x7f57,
0x5c14, 0x6717, 0x4f0a, 0x5c14, 0x57ce, 0x963f, 0x7eb3, 0x5fb7, 0x5c14, 0x963f, 0x514b, 0x6258,
0x522b, 0x963f, 0x4ec0, 0x54c8, 0x5df4, 0x5fb7, 0x6587, 0x83b1, 0x79d1, 0x4f26, 0x5761, 0x8fea,
0x62dc, 0x675c, 0x5c1a, 0x522b, 0x52a0, 0x6c99, 0x5e0c, 0x4f2f, 0x4f26, 0x4f0a, 0x5c14, 0x5e93,
0x8328, 0x514b, 0x67e5, 0x4e9a, 0x666e, 0x62c9, 0x5361, 0x62c9, 0x5947, 0x6c49, 0x5fb7, 0x52a0,
0x671b, 0x52a0, 0x9521, 0x9a6c, 0x65af, 0x5580, 0x7279, 0x5c3c, 0x79d1, 0x897f, 0x4e9a, 0x5361,
0x5854, 0x5c14, 0x5e93, 0x65af, 0x5854, 0x5948, 0x514b, 0x5b5c, 0x6d1b, 0x5c14, 0x8fbe, 0x5229,
0x96c5, 0x5f97, 0x8428, 0x54c8, 0x6797, 0x4e07, 0x8c61, 0x7b26, 0x62c9, 0x8fea, 0x6c83, 0x65af,
0x6258, 0x514b, 0x96c5, 0x5e93, 0x8328, 0x514b, 0x53f6, 0x5361, 0x6377, 0x7433, 0x5821, 0x57c3,
0x91cc, 0x6e29, 0x767e, 0x6155, 0x5927, 0x52a0, 0x90a3, 0x5229, 0x4f5b, 0x5f97, 0x89d2, 0x9a6c,
0x5fb7, 0x62c9, 0x96f7, 0x514b, 0x96c5, 0x672a, 0x514b, 0x65af, 0x5766, 0x5229, 0x963f, 0x5fb7,
0x83b1, 0x5fb7, 0x5e03, 0x91cc, 0x65af, 0x73ed, 0x970d, 0x5df4, 0x7279, 0x73c0, 0x65af, 0x6089,
0x5c3c, 0x8d1d, 0x5c14, 0x683c, 0x83b1, 0x5fb7, 0x5e03, 0x62c9, 0x8fea, 0x65af, 0x62c9, 0x53d1,
0x57fa, 0x5e0c, 0x8bb7, 0x4e4c, 0x6839, 0x897f, 0x5c9b, 0x9a6c, 0x6069, 0x5c9b, 0x4f0a, 0x65af,
0x5766, 0x5e03, 0x5c14, 0x5362, 0x5e03, 0x5c14, 0x96c5, 0x90a3, 0x9a6c, 0x8033, 0x4ed6, 0x6ce2,
0x5fb7, 0x6208, 0x91cc, 0x5bdf, 0x8428, 0x9a6c, 0x62c9, 0x5723, 0x9a6c, 0x529b, 0x8bfa, 0x8428,
0x62c9, 0x70ed, 0x7a9d, 0x8f9b, 0x83f2, 0x7f57, 0x6ce2, 0x5c14, 0x65af, 0x79d1, 0x666e, 0x91cc,
0x7d22, 0x975e, 0x4e9a, 0x74e6, 0x675c, 0x5179, 0x8428, 0x683c, 0x52d2, 0x5e03, 0x53ef, 0x53ef,
0x65af, 0x9a6c, 0x5c14, 0x4ee3, 0x592b, 0x6bdb, 0x91cc, 0x6c42, 0x65af, 0x7559, 0x5c3c, 0x6c6a,
0x67e5, 0x5854, 0x59c6, 0x7279, 0x9c81, 0x514b, 0x7fa4, 0x5c9b, 0x590d, 0x6d3b, 0x8282, 0x5c9b,
0x5bcc, 0x7eb3, 0x5bcc, 0x63d0, 0x79d1, 0x9686, 0x7fa4, 0x5c9b, 0x74dc, 0x8fbe, 0x5c14, 0x5361,
0x7eb3, 0x5c14, 0x6069, 0x5fb7, 0x4f2f, 0x91cc, 0x5e93, 0x8d5b, 0x57c3, 0x5938, 0x8d3e, 0x6797,
0x9a6c, 0x6731, 0x7f57, 0x9a6c, 0x514b, 0x8428, 0x65af, 0x7459, 0x9c81, 0x52aa, 0x7f8e, 0x963f,
0x5e15, 0x679c, 0x5e15, 0x679c, 0x5e15, 0x52b3, 0x76ae, 0x7279, 0x51ef, 0x6069, 0x6ce2, 0x7eb3,
0x4f69, 0x5c9b, 0x83ab, 0x5c14, 0x5179, 0x6bd4, 0x6e2f, 0x5854, 0x5e0c, 0x63d0, 0x65af, 0x8003,
0x65af, 0x4f2f, 0x677e, 0x5fb7, 0x963f, 0x6bd4, 0x8d0a, 0x963f, 0x65af, 0x99ac, 0x62c9, 0x73ed,
0x57fa, 0x79d1, 0x7d0d, 0x514b, 0x91cc, 0x9054, 0x7d2f, 0x65af, 0x85a9, 0x62c9, 0x59c6, 0x7d04,
0x7ff0, 0x5167, 0x65af, 0x5821, 0x91d1, 0x6c99, 0x85a9, 0x5229, 0x5e03, 0x7dad, 0x76e7, 0x5b89,
0x9054, 0x76e7, 0x85a9, 0x5361, 0x99ac, 0x585e, 0x9b6f, 0x5167, 0x7f85, 0x7562, 0x6eab, 0x7279,
0x548c, 0x514b, 0x57c3, 0x9054, 0x514b, 0x963f, 0x62c9, 0x74dc, 0x4f0a, 0x7d0d, 0x79d1, 0x723e,
0x591a, 0x74e6, 0x62c9, 0x91cc, 0x5967, 0x54c8, 0x963f, 0x9b6f, 0x5df4, 0x963f, 0x677e, 0x68ee,
0x5df4, 0x5e0c, 0x96c5, 0x5df4, 0x4f0a, 0x4e9e, 0x5fb7, 0x73ed, 0x5fb7, 0x62c9, 0x65af, 0x4f2f,
0x5229, 0x8332, 0x535a, 0x963f, 0x7dad, 0x65af, 0x5854, 0x829d, 0x5a03, 0x5a03, 0x54e5, 0x65af,
0x9054, 0x9ece, 0x52a0, 0x5eab, 0x4e9e, 0x5df4, 0x9053, 0x68ee, 0x7063, 0x611b, 0x6c11, 0x9813,
0x798f, 0x5854, 0x840a, 0x85a9, 0x683c, 0x840a, 0x65af, 0x7063, 0x683c, 0x6797, 0x7d0d, 0x9054,
0x74dc, 0x5fb7, 0x7f85, 0x666e, 0x5371, 0x5730, 0x99ac, 0x62c9, 0x572d, 0x4e9e, 0x90a3, 0x590f,
0x7063, 0x62ff, 0x5370, 0x7b2c, 0x5b89, 0x7d0d, 0x6ce2, 0x5229, 0x65af, 0x5370, 0x7b2c, 0x5b89,
0x7d0d, 0x5dde, 0x8afe, 0x514b, 0x65af, 0x5370, 0x7b2c, 0x5b89, 0x7d0d, 0x5dde, 0x99ac, 0x502b,
0x54e5, 0x5370, 0x7b2c, 0x5b89, 0x7d0d, 0x5dde, 0x5f7c, 0x5f97, 0x5821, 0x5370, 0x7b2c, 0x5b89,
0x7d0d, 0x5dde, 0x7279, 0x723e, 0x57ce, 0x5370, 0x7b2c, 0x5b89, 0x7d0d, 0x5dde, 0x97cb, 0x97cb,
0x5370, 0x7b2c, 0x5b89, 0x7d0d, 0x5dde, 0x6eab, 0x68ee, 0x65af, 0x5370, 0x7b2c, 0x5b89, 0x7d0d,
0x5dde, 0x5a01, 0x7d0d, 0x99ac, 0x514b, 0x4f0a, 0x52aa, 0x7dad, 0x514b, 0x8def, 0x6613, 0x7dad,
0x723e, 0x99ac, 0x585e, 0x7d04, 0x99ac, 0x90a3, 0x74dc, 0x99ac, 0x63d0, 0x5c3c, 0x514b, 0x6885,
0x8afe, 0x7c73, 0x5c3c, 0x8499, 0x5f97, 0x7dad, 0x7684, 0x4e9e, 0x8499, 0x585e, 0x62c9, 0x7279,
0x5cf6, 0x62ff, 0x9a37, 0x8afe, 0x7f85, 0x5c3c, 0x4e9e, 0x5317, 0x9054, 0x79d1, 0x4ed6, 0x5dde,
0x6bd4, 0x5c24, 0x62c9, 0x5317, 0x9054, 0x79d1, 0x4ed6, 0x5dde, 0x4e2d, 0x5fc3, 0x5e02, 0x5317,
0x9054, 0x79d1, 0x4ed6, 0x5dde, 0x65b0, 0x85a9, 0x52d2, 0x59c6, 0x97cb, 0x67f3, 0x6e2f, 0x91cc,
0x7d04, 0x5e03, 0x862d, 0x79d1, 0x8056, 0x5730, 0x4e9e, 0x54e5, 0x8056, 0x591a, 0x660e, 0x5404,
0x8056, 0x5df4, 0x6cf0, 0x52d2, 0x7c73, 0x8056, 0x7d04, 0x7ff0, 0x65af, 0x8056, 0x76e7, 0x897f,
0x4e9e, 0x8056, 0x6587, 0x68ee, 0x7279, 0x7279, 0x53e4, 0x897f, 0x52a0, 0x723e, 0x5df4, 0x5716,
0x52d2, 0x8482, 0x83ef, 0x7d0d, 0x767d, 0x99ac, 0x5e02, 0x4e9e, 0x5eab, 0x5854, 0x7279, 0x51f1,
0x897f, 0x7ad9, 0x6234, 0x7dad, 0x65af, 0x7ad9, 0x675c, 0x8499, 0x8fea, 0x7dad, 0x723e, 0x7ad9,
0x9ea5, 0x5938, 0x91cc, 0x83ab, 0x68ee, 0x7ad9, 0x9ea5, 0x514b, 0x9ed8, 0x591a, 0x7ad9, 0x5e15,
0x723e, 0x9ed8, 0x7f85, 0x745f, 0x62c9, 0x7ad9, 0x662d, 0x548c, 0x7ad9, 0x7279, 0x7f85, 0x723e,
0x7ad9, 0x6717, 0x4f0a, 0x723e, 0x57ce, 0x963f, 0x7d0d, 0x5fb7, 0x723e, 0x6bd4, 0x65af, 0x51f1,
0x514b, 0x79d1, 0x502b, 0x5761, 0x675c, 0x5c1a, 0x5225, 0x5e0c, 0x4f2f, 0x4f96, 0x4f0a, 0x723e,
0x5eab, 0x8328, 0x514b, 0x67e5, 0x96c5, 0x666e, 0x62c9, 0x932b, 0x6c5f, 0x99ac, 0x65af, 0x5580,
0x7279, 0x5c3c, 0x79d1, 0x897f, 0x4e9e, 0x5361, 0x5854, 0x723e, 0x96c5, 0x5eab, 0x8328, 0x514b,
0x8449, 0x5361, 0x6377, 0x7433, 0x5821, 0x99ac, 0x5fb7, 0x62c9, 0x5cf6, 0x53f2, 0x4e39, 0x5229,
0x963f, 0x5fb7, 0x840a, 0x5fb7, 0x5e03, 0x91cc, 0x65af, 0x672c, 0x8377, 0x4f2f, 0x7279, 0x8c9d,
0x723e, 0x683c, 0x840a, 0x5fb7, 0x4f2f, 0x62c9, 0x7b2c, 0x65af, 0x62c9, 0x74e6, 0x57fa, 0x5e0c,
0x8a25, 0x70cf, 0x6839, 0x897f, 0x5cf6, 0x99ac, 0x6069, 0x5cf6, 0x76e7, 0x5e03, 0x723e, 0x96c5,
0x90a3, 0x99ac, 0x8033, 0x4ed6, 0x85a9, 0x62c9, 0x71b1, 0x7aa9, 0x83ef, 0x675c, 0x8332, 0x85a9,
0x683c, 0x52d2, 0x5e03, 0x67e5, 0x6208, 0x65af, 0x7fa4, 0x5cf6, 0x53ef, 0x53ef, 0x65af, 0x7fa4,
0x5cf6, 0x51f1, 0x723e, 0x84cb, 0x6717, 0x7fa4, 0x5cf6, 0x99ac, 0x723e, 0x4ee3, 0x592b, 0x6bdb,
0x91cc, 0x88d8, 0x65af, 0x5e03, 0x5e72, 0x7dad, 0x723e, 0x5cf6, 0x6cd5, 0x8003, 0x798f, 0x74b0,
0x7901, 0x74dc, 0x9054, 0x723e, 0x5361, 0x7d0d, 0x723e, 0x5cf6, 0x6069, 0x5fb7, 0x4f2f, 0x91cc,
0x5cf6, 0x79d1, 0x65af, 0x96f7, 0x74dc, 0x52a0, 0x6797, 0x74b0, 0x7901, 0x99ac, 0x4e45, 0x7f85,
0x99ac, 0x514b, 0x85a9, 0x65af, 0x7fa4, 0x5cf6, 0x7459, 0x9b6f, 0x76ae, 0x7279, 0x5eb7, 0x7fa4,
0x5cf6, 0x6e6f, 0x52a0, 0x5854, 0x5e03, 0x5cf6, 0x5a01, 0x514b, 0x5cf6, 0x5b89, 0x594e, 0x62c9,
0x8377, 0x5c6c, 0x963f, 0x9b6f, 0x5df4, 0x591a, 0x7c73, 0x5c3c, 0x514b, 0x5eab, 0x65af, 0x5854,
0x5948, 0x57fa, 0x897f, 0x7d10, 0x5b89, 0x5854, 0x90a3, 0x90a3, 0x5229, 0x5f17, 0x430, 0x486,
0x43b, 0x43c, 0x430, 0x442, 0x44b, 0x300, 0x430, 0x486, 0x43d, 0x430, 0x301, 0x434,
0x44b, 0x440, 0x44c, 0x430, 0x486, 0x43a, 0x442, 0x430, 0x301, 0xa64b, 0x430, 0x486,
0x43a, 0x442, 0x44e, 0x301, 0x431, 0x438, 0x43d, 0x441, 0x43a, 0x44a, 0x431, 0x438,
0x448, 0x43a, 0x435, 0x301, 0x43a, 0x44a, 0x447, 0x438, 0x442, 0x430, 0x300, 0x438,
0x486, 0x440, 0x43a, 0xa64b, 0x301, 0x442, 0x441, 0x43a, 0x44a, 0x43f, 0x435, 0x442,
0x440, 0x43e, 0x43f, 0x430, 0x301, 0x475, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x44a,
0x5f, 0x43a, 0x430, 0x43c, 0x447, 0x430, 0x301, 0x442, 0x441, 0x43a, 0x457, 0x439,
0x445, 0x430, 0x301, 0x43d, 0x434, 0x44b, 0x433, 0x430, 0x43a, 0x440, 0x430, 0x441,
0x43d, 0x43e, 0x467, 0x301, 0x440, 0x441, 0x43a, 0x44a, 0x43c, 0x430, 0x433, 0x430,
0x434, 0x430, 0x301, 0x43d, 0x44a, 0x43d, 0x43e, 0x432, 0x43e, 0x43a, 0xa64b, 0x437,
0x43d, 0x435, 0x301, 0x446, 0x43a, 0x44a, 0x43d, 0x43e, 0x432, 0x43e, 0x441, 0x438,
0x431, 0x438, 0x301, 0x440, 0x441, 0x43a, 0x44a, 0x47b, 0x486, 0x301, 0x43c, 0x441,
0x43a, 0x44a, 0x1c82, 0x443, 0x486, 0x440, 0x430, 0x301, 0x43b, 0x44c, 0x441, 0x43a,
0x44a, 0x43a, 0x44b, 0x437, 0x44b, 0x43b, 0x47b, 0x440, 0x434, 0x430, 0x300, 0x441,
0x430, 0x445, 0x430, 0x43b, 0x438, 0x301, 0x43d, 0x44a, 0x441, 0x440, 0x435, 0x434,
0x43d, 0x435, 0x43a, 0x43e, 0x43b, 0x44b, 0x301, 0x43c, 0x441, 0x43a, 0x44a, 0x1c82,
0x443, 0x486, 0x301, 0x441, 0x442, 0x44c, 0x5f, 0x43d, 0x435, 0x301, 0x440, 0x430,
0x432, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x301, 0x43a,
0x44a, 0xa657, 0x486, 0x43a, 0xa64b, 0x301, 0x442, 0x441, 0x43a, 0x44a, 0x454, 0x486,
0x43a, 0x430, 0x442, 0x435, 0x440, 0x457, 0x43d, 0x431, 0xa64b, 0x301, 0x440, 0x433,
0x44a, 0x43a, 0x430, 0x43b, 0x438, 0x43d, 0x438, 0x43d, 0x433, 0x440, 0x430, 0x301,
0x434, 0x44a, 0x43a, 0x456, 0x301, 0x435, 0x432, 0x44a, 0x43c, 0x438, 0x301, 0x43d,
0x441, 0x43a, 0x44a, 0x43c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x300, 0x441, 0x430,
0x43c, 0x430, 0x301, 0x440, 0x430, 0x441, 0x477, 0x43c, 0x444, 0x435, 0x440, 0xa64b,
0x301, 0x43f, 0x43e, 0x43b, 0x44c, 0x432, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440,
0x430, 0x301, 0x434, 0x44a, 0x410, 0x441, 0x43c, 0x44d, 0x440, 0x430, 0x421, 0x430,
0x43d, 0x2d, 0x422, 0x43e, 0x43c, 0x435, 0x20, 0x421, 0x430, 0x43d, 0x2d, 0x422,
0x43e, 0x43c, 0x435, 0x421, 0x430, 0x43d, 0x2d, 0x41b, 0x443, 0x438, 0x441, 0x20,
0x421, 0x430, 0x43d, 0x2d, 0x41b, 0x443, 0x438, 0x441, 0x41a, 0x430, 0x439, 0x43c,
0x430, 0x43d, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43c,
0x421, 0x44c, 0x44e, 0x434, 0x430, 0x434, 0x2d, 0x425, 0x443, 0x430, 0x440, 0x435,
0x441, 0x422, 0x435, 0x43b, 0x43b, 0x2d, 0x421, 0x438, 0x442, 0x438, 0x2c, 0x20,
0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d, 0x430, 0x412, 0x435, 0x432, 0x435, 0x439,
0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d, 0x430, 0x412, 0x438, 0x43d,
0x441, 0x435, 0x43d, 0x43d, 0x435, 0x441, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438,
0x430, 0x43d, 0x430, 0x423, 0x438, 0x43d, 0x430, 0x43c, 0x430, 0x43a, 0x2c, 0x20,
0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d, 0x430, 0x41b, 0x43e, 0x443, 0x44d, 0x440,
0x2d, 0x41f, 0x440, 0x438, 0x43d, 0x441, 0x2d, 0x41a, 0x443, 0x43e, 0x442, 0x435,
0x440, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x411, 0x43e, 0x439, 0x43b,
0x430, 0x2c, 0x20, 0x4aa, 0x443, 0x440, 0x4ab, 0x4d7, 0x440, 0x20, 0x414, 0x430,
0x43a, 0x43e, 0x442, 0x430, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x2c, 0x20, 0x4aa,
0x443, 0x440, 0x4ab, 0x4d7, 0x440, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430,
0x41d, 0x44c, 0x44e, 0x2d, 0x421, 0x435, 0x439, 0x43b, 0x435, 0x43c, 0x2c, 0x20,
0x4aa, 0x443, 0x440, 0x4ab, 0x4d7, 0x440, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442,
0x430, 0x420, 0x435, 0x437, 0x43e, 0x43b, 0x44c, 0x44e, 0x442, 0x421, 0x430, 0x43d,
0x2d, 0x41f, 0x430, 0x443, 0x43b, 0x443, 0x20, 0x421, 0x430, 0x43d, 0x2d, 0x41f,
0x430, 0x443, 0x43b, 0x443, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41b, 0x44e, 0x441,
0x438, 0x44f, 0x414, 0x44e, 0x43c, 0x43e, 0x43d, 0x2d, 0x434, 0x2019, 0x42e, 0x440,
0x432, 0x438, 0x43b, 0x44c, 0x20, 0x414, 0x44e, 0x43c, 0x43e, 0x43d, 0x2d, 0x434,
0x2019, 0x42e, 0x440, 0x432, 0x438, 0x43b, 0x44c, 0x421, 0x430, 0x445, 0x430, 0x43b,
0x438, 0x43d, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x42f, 0x43d, 0x433,
0x43e, 0x43d, 0x410, 0x437, 0x43e, 0x440, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432,
0x4d7, 0x441, 0x435, 0x43c, 0x411, 0x435, 0x440, 0x43c, 0x443, 0x434, 0x20, 0x443,
0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43c, 0x41a, 0x430, 0x43d, 0x430,
0x440, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43c, 0x424,
0x430, 0x440, 0x435, 0x440, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441,
0x435, 0x43c, 0x41a, 0x4d1, 0x43d, 0x442, 0x4d1, 0x440, 0x20, 0x413, 0x435, 0x43e,
0x440, 0x433, 0x438, 0x421, 0x4d1, 0x432, 0x430, 0x43f, 0x43b, 0x4d1, 0x20, 0x415,
0x43b, 0x435, 0x43d, 0x430, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x410,
0x444, 0x438, 0x43d, 0x441, 0x435, 0x43c, 0x41c, 0x44d, 0x43d, 0x20, 0x443, 0x442,
0x440, 0x430, 0x432, 0x4d7, 0x41c, 0x443, 0x441, 0x43a, 0x430, 0x432, 0x420, 0x430,
0x448, 0x442, 0x430, 0x432, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x41a,
0x43e, 0x43a, 0x43e, 0x441, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441,
0x435, 0x43c, 0x41a, 0x43e, 0x43c, 0x43e, 0x440, 0x441, 0x435, 0x43c, 0x41c, 0x430,
0x43b, 0x44c, 0x434, 0x438, 0x432, 0x441, 0x435, 0x43c, 0x41c, 0x430, 0x432, 0x440,
0x438, 0x43a, 0x438, 0x439, 0x41c, 0x430, 0x439, 0x43e, 0x442, 0x442, 0x430, 0x41c,
0x4d1, 0x43d, 0x43a, 0x443, 0x43d, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7,
0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441, 0x20, 0x443, 0x442,
0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43c, 0x413, 0x430, 0x43c, 0x431, 0x44c,
0x435, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43c, 0x41c,
0x430, 0x440, 0x43a, 0x438, 0x437, 0x430, 0x441, 0x20, 0x443, 0x442, 0x440, 0x430,
0x432, 0x4d7, 0x441, 0x435, 0x43c, 0x41f, 0x438, 0x442, 0x43a, 0x44d, 0x440, 0x43d,
0x4b, 0x61, 0x69, 0x6a, 0x72, 0x6f, 0x4c, 0x61, 0x61, 0x6a, 0x6f, 0x73,
0x57, 0x69, 0x6e, 0x64, 0x68, 0x75, 0x6b, 0x4b, 0x6f, 0xdf, 0x74, 0x61,
0x72, 0x69, 0x63, 0x6b, 0x61, 0x4a, 0x75, 0x61, 0x79, 0x61, 0x71, 0x75,
0x69, 0x6c, 0x4b, 0x6e, 0x6f, 0x78, 0x20, 0x65, 0x6e, 0x20, 0x49, 0x6e,
0x64, 0x69, 0x61, 0x6e, 0x61, 0x4d, 0x61, 0x72, 0x65, 0x6e, 0x67, 0x6f,
0x20, 0x65, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x50,
0x65, 0x74, 0x65, 0x72, 0x73, 0x62, 0x75, 0x72, 0x67, 0x20, 0x65, 0x6e,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x54, 0x65, 0x6c, 0x6c,
0x20, 0x43, 0x69, 0x74, 0x79, 0x20, 0x65, 0x6e, 0x20, 0x49, 0x6e, 0x64,
0x69, 0x61, 0x6e, 0x61, 0x56, 0x65, 0x76, 0x61, 0x79, 0x20, 0x65, 0x6e,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x56, 0x69, 0x6e, 0x63,
0x65, 0x6e, 0x6e, 0x65, 0x73, 0x20, 0x65, 0x6e, 0x20, 0x49, 0x6e, 0x64,
0x69, 0x61, 0x6e, 0x61, 0x57, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x63, 0x20,
0x65, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x4d, 0x6f,
0x6e, 0x74, 0x69, 0x63, 0x65, 0x6c, 0x6c, 0x6f, 0x20, 0x65, 0x6e, 0x20,
0x4b, 0x65, 0x6e, 0x74, 0x75, 0x63, 0x6b, 0x79, 0x53, 0x63, 0x68, 0x74,
0x61, 0x64, 0x74, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x6f, 0x42, 0x65,
0x75, 0x6c, 0x61, 0x68, 0x20, 0x65, 0x6e, 0x20, 0x4e, 0x6f, 0x6f, 0x64,
0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x5a, 0x65, 0x6e, 0x74, 0x65,
0x72, 0x20, 0x65, 0x6e, 0x20, 0x4e, 0x6f, 0x6f, 0x64, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x75, 0x20, 0x53, 0x61, 0x6c, 0x65,
0x6d, 0x20, 0x65, 0x6e, 0x20, 0x4e, 0x6f, 0x6f, 0x64, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x5a, 0x69, 0x6e, 0x74, 0x20, 0x42, 0x61, 0x72,
0x74, 0x68, 0xe9, 0x6c, 0x65, 0x6d, 0x79, 0x5a, 0x69, 0x6e, 0x74, 0x20,
0x4a, 0x6f, 0x68, 0x6e, 0x2019, 0x73, 0x5a, 0x69, 0x6e, 0x74, 0x20, 0x4b,
0x69, 0x74, 0x74, 0x73, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x20, 0x4c, 0x75,
0x74, 0x73, 0x63, 0x68, 0x69, 0x69, 0x6a, 0x61, 0x5a, 0x69, 0x6e, 0x74,
0x20, 0x54, 0x68, 0x6f, 0x6d, 0x61, 0x73, 0x5a, 0x69, 0x6e, 0x74, 0x20,
0x56, 0x69, 0x6e, 0x74, 0x73, 0x63, 0x68, 0xe4, 0x6e, 0x74, 0x44, 0x75,
0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c,
0x6c, 0x65, 0x2d, 0x53, 0x63, 0x68, 0x74, 0x61, 0x7a, 0x6a, 0x6f, 0x68,
0x6e, 0x53, 0x63, 0x68, 0x6f, 0x6f, 0x77, 0x61, 0x2d, 0x53, 0x63, 0x68,
0x74, 0x61, 0x7a, 0x6a, 0x6f, 0x68, 0x6e, 0x20, 0x6f, 0x70, 0x20, 0x64,
0xe4, 0x20, 0x45, 0x6e, 0x73, 0x65, 0x6c, 0x20, 0x4f, 0x6e, 0x6a, 0x75,
0x6c, 0x41, 0x71, 0x74, 0xf6, 0x62, 0x65, 0x41, 0x73, 0x73, 0x68, 0x67,
0x61, 0x62, 0x61, 0x74, 0x42, 0x69, 0x73, 0x63, 0x68, 0x6b, 0x65, 0x6b,
0x44, 0x75, 0x73, 0x63, 0x68, 0x61, 0x6e, 0x62, 0x65, 0x4a, 0x61, 0x61,
0x73, 0x61, 0x53, 0x69, 0x67, 0x6f, 0x6e, 0x67, 0x20, 0x28, 0x48, 0x6f,
0x2d, 0x54, 0x73, 0x63, 0x68, 0x69, 0x2d, 0x4d, 0x69, 0x6e, 0x68, 0x2d,
0x53, 0x63, 0x68, 0x74, 0x61, 0x64, 0x74, 0x29, 0x4b, 0x61, 0x6d, 0x73,
0x63, 0x68, 0x61, 0x74, 0x74, 0x6b, 0x61, 0x4e, 0x69, 0x6b, 0x6f, 0x73,
0x69, 0x6a, 0x61, 0x50, 0x6e, 0x6f, 0x6d, 0x20, 0x50, 0x65, 0x6e, 0x68,
0x51, 0x79, 0x73, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x53, 0x63, 0x68,
0x61, 0x6e, 0x67, 0x68, 0x61, 0x69, 0x53, 0x69, 0x6e, 0x67, 0x6a, 0x61,
0x70, 0x75, 0x75, 0x72, 0x54, 0x61, 0x73, 0x63, 0x68, 0x6b, 0x65, 0x6e,
0x74, 0x54, 0x69, 0x62, 0x6c, 0x69, 0x73, 0x55, 0x72, 0x72, 0x75, 0x6d,
0x70, 0x74, 0x73, 0x63, 0x68, 0x69, 0x52, 0x61, 0x6e, 0x6a, 0x75, 0x6e,
0x45, 0x72, 0x69, 0x77, 0x61, 0x6e, 0x41, 0x7a, 0x6f, 0x6f, 0x72, 0x65,
0x4b, 0x61, 0x6e, 0x6e, 0x61, 0x61, 0x72, 0x65, 0x4b, 0x61, 0x70, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x46, 0xe4, 0x72, 0xf6, 0x72, 0x5a, 0x69,
0x6e, 0x74, 0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x42, 0x65, 0x6c,
0x6a, 0x72, 0x61, 0x64, 0x4b, 0x6f, 0x70, 0x65, 0x6e, 0x68, 0x61, 0x72,
0x65, 0x6e, 0x4a, 0x69, 0x62, 0x72, 0x61, 0x6c, 0x74, 0x61, 0x61, 0x4a,
0xf6, 0xf6, 0x6e, 0x73, 0x65, 0x69, 0x45, 0x6e, 0x73, 0x65, 0x6c, 0x20,
0x4d, 0xe4, 0x6e, 0x49, 0x73, 0x74, 0x61, 0x6d, 0x62, 0x75, 0x6c, 0x4a,
0xf6, 0xf6, 0x73, 0x65, 0x69, 0x4b, 0x61, 0x6c, 0x69, 0x6e, 0x6e, 0x69,
0x6e, 0x6a, 0x72, 0x61, 0x61, 0x64, 0x4c, 0x75, 0x78, 0x65, 0x6d, 0x62,
0x75, 0x72, 0x73, 0x63, 0x68, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75, 0x52,
0x69, 0x69, 0x6a, 0x61, 0x54, 0x69, 0x72, 0x61, 0x61, 0x6e, 0x61, 0x64,
0x65, 0x72, 0x20, 0x56, 0x61, 0x74, 0x69, 0x6b, 0x61, 0x68, 0x6e, 0x57,
0x69, 0x65, 0x6e, 0x57, 0x6f, 0x6c, 0x6a, 0x6f, 0x6a, 0x72, 0x61, 0x64,
0x57, 0x61, 0x72, 0x73, 0x63, 0x68, 0x61, 0x75, 0x53, 0x61, 0x67, 0x72,
0x65, 0x62, 0x5a, 0xfc, 0x72, 0x65, 0x73, 0x63, 0x68, 0x4d, 0x61, 0x6c,
0x6c, 0x64, 0x69, 0x76, 0x65, 0x4a, 0x61, 0x6c, 0x61, 0x70, 0x70, 0x61,
0x6a, 0x6f, 0x73, 0x4a, 0x75, 0x61, 0x6d, 0x48, 0x6f, 0x6e, 0x75, 0x6c,
0x75, 0x6c, 0x75, 0x41, 0x6c, 0x17e, 0x69, 0x72, 0x47, 0x76, 0x61, 0x6a,
0x61, 0x6e, 0x61, 0x43, 0x69, 0x75, 0x64, 0x61, 0x64, 0x20, 0x64, 0x65,
0x20, 0x4d, 0xe9, 0x78, 0x69, 0x63, 0x6f, 0x41, 0x6c, 0x6d, 0x61, 0x20,
0x41, 0x74, 0x61, 0x41, 0x161, 0x67, 0x61, 0x62, 0x61, 0x74, 0x10c, 0x69,
0x74, 0x61, 0x42, 0x65, 0x72, 0x6d, 0x75, 0x64, 0x69, 0x46, 0x65, 0x72,
0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x63, 0x69, 0x4a,
0x75, 0x17e, 0x6e, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x6a,
0x61, 0x42, 0x72, 0x75, 0x78, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x4f, 0x74,
0x6f, 0x6b, 0x20, 0x4d, 0x61, 0x6e, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x17e,
0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6f, 0x74, 0x6f, 0x10d, 0x6a, 0x65,
0x41, 0x62, 0x69, 0x64, 0x17e, 0x61, 0x6e, 0x41, 0x6c, 0x17e, 0xed, 0x72,
0x4b, 0xe1, 0x68, 0x69, 0x72, 0x61, 0x43, 0x68, 0x61, 0x72, 0x74, 0xfa,
0x6d, 0x4d, 0x6f, 0x67, 0x61, 0x64, 0x69, 0x161, 0x75, 0x4e, 0x64, 0x17e,
0x61, 0x6d, 0x65, 0x6e, 0x61, 0x4e, 0x75, 0x61, 0x6b, 0x161, 0x6f, 0x74,
0x74, 0x53, 0x76, 0x61, 0x74, 0xfd, 0x20, 0x54, 0x6f, 0x6d, 0xe1, 0x161,
0x54, 0x72, 0x69, 0x70, 0x6f, 0x6c, 0x69, 0x73, 0x4b, 0x61, 0x6a, 0x6d,
0x61, 0x6e, 0x73, 0x6b, 0xe9, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76,
0x79, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x53, 0x65, 0x76,
0x65, 0x72, 0x6e, 0xed, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x43,
0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x53, 0x65, 0x76, 0x65, 0x72,
0x6e, 0xed, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77,
0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x53, 0x65, 0x76, 0x65,
0x72, 0x6e, 0xed, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x53, 0x76,
0x61, 0x74, 0xfd, 0x20, 0x42, 0x61, 0x72, 0x74, 0x6f, 0x6c, 0x6f, 0x6d,
0x11b, 0x6a, 0x53, 0x76, 0x61, 0x74, 0xfd, 0x20, 0x4b, 0x72, 0x79, 0x161,
0x74, 0x6f, 0x66, 0x53, 0x76, 0x61, 0x74, 0xe1, 0x20, 0x4c, 0x75, 0x63,
0x69, 0x65, 0x53, 0x76, 0x61, 0x74, 0xfd, 0x20, 0x54, 0x6f, 0x6d, 0xe1,
0x161, 0x20, 0x28, 0x4b, 0x61, 0x72, 0x69, 0x62, 0x69, 0x6b, 0x29, 0x53,
0x76, 0x61, 0x74, 0xfd, 0x20, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x63,
0x41, 0x6d, 0x6d, 0xe1, 0x6e, 0x41, 0x161, 0x63, 0x68, 0x61, 0x62, 0x61,
0x64, 0x42, 0x61, 0x67, 0x64, 0xe1, 0x64, 0x42, 0x61, 0x68, 0x72, 0x61,
0x6a, 0x6e, 0x42, 0x65, 0x6a, 0x72, 0xfa, 0x74, 0x44, 0x61, 0x6d, 0x61,
0x161, 0x65, 0x6b, 0x44, 0x68, 0xe1, 0x6b, 0x61, 0x44, 0x75, 0x62, 0x61,
0x6a, 0x48, 0x6f, 0x20, 0x10c, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x6f, 0x76,
0x6f, 0x20, 0x6d, 0x11b, 0x73, 0x74, 0x6f, 0x4a, 0x65, 0x72, 0x75, 0x7a,
0x61, 0x6c, 0xe9, 0x6d, 0x4b, 0xe1, 0x62, 0x75, 0x6c, 0x4b, 0x61, 0x72,
0xe1, 0x10d, 0xed, 0x4b, 0xe1, 0x74, 0x68, 0x6d, 0xe1, 0x6e, 0x64, 0xfa,
0x43, 0x68, 0x61, 0x6e, 0x64, 0x79, 0x67, 0x61, 0x4b, 0x61, 0x6c, 0x6b,
0x61, 0x74, 0x61, 0x4e, 0x69, 0x6b, 0xf3, 0x73, 0x69, 0x65, 0x4e, 0x6f,
0x76, 0x6f, 0x6b, 0x75, 0x7a, 0x6e, 0x11b, 0x63, 0x6b, 0x55, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0x50, 0x68, 0x6e, 0x6f, 0x6d, 0x70, 0x65, 0x6e, 0x68,
0x50, 0x63, 0x68, 0x6a, 0x6f, 0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x67, 0x52,
0x69, 0x6a, 0xe1, 0x64, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x53, 0x6f, 0x75, 0x6c, 0x160, 0x61, 0x6e, 0x67, 0x68, 0x61, 0x6a, 0x53,
0x72, 0x65, 0x64, 0x6e, 0x11b, 0x6b, 0x6f, 0x6c, 0x79, 0x6d, 0x73, 0x6b,
0x54, 0x63, 0x68, 0x61, 0x6a, 0x2d, 0x70, 0x65, 0x6a, 0x54, 0x65, 0x68,
0x65, 0x72, 0xe1, 0x6e, 0x54, 0x68, 0x69, 0x6d, 0x62, 0xfa, 0x55, 0x6c,
0xe1, 0x6e, 0x62, 0xe1, 0x74, 0x61, 0x72, 0x4a, 0x65, 0x6b, 0x61, 0x74,
0x11b, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x41, 0x7a, 0x6f, 0x72,
0x73, 0x6b, 0xe9, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x79, 0x42,
0x65, 0x72, 0x6d, 0x75, 0x64, 0x79, 0x4b, 0x61, 0x6e, 0xe1, 0x72, 0x73,
0x6b, 0xe9, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x79, 0x4b, 0x61,
0x70, 0x76, 0x65, 0x72, 0x64, 0x79, 0x46, 0x61, 0x65, 0x72, 0x73, 0x6b,
0xe9, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x79, 0x4a, 0x69, 0x17e,
0x6e, 0xed, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x53, 0x76,
0x61, 0x74, 0xe1, 0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x41, 0x73,
0x74, 0x72, 0x61, 0x63, 0x68, 0x61, 0x148, 0x41, 0x74, 0x68, 0xe9, 0x6e,
0x79, 0x42, 0x11b, 0x6c, 0x65, 0x68, 0x72, 0x61, 0x64, 0x42, 0x75, 0x6b,
0x75, 0x72, 0x65, 0x161, 0x165, 0x42, 0x75, 0x64, 0x61, 0x70, 0x65, 0x161,
0x165, 0x4b, 0x69, 0x161, 0x69, 0x6e, 0x11b, 0x76, 0x4b, 0x6f, 0x64, 0x61,
0x148, 0x48, 0x65, 0x6c, 0x73, 0x69, 0x6e, 0x6b, 0x79, 0x4f, 0x73, 0x74,
0x72, 0x6f, 0x76, 0x20, 0x4d, 0x61, 0x6e, 0x4b, 0x79, 0x6a, 0x65, 0x76,
0x4c, 0x75, 0x62, 0x6c, 0x61, 0x148, 0x4c, 0x6f, 0x6e, 0x64, 0xfd, 0x6e,
0x4c, 0x75, 0x63, 0x65, 0x6d, 0x62, 0x75, 0x72, 0x6b, 0x50, 0x61, 0x159,
0xed, 0x17e, 0x158, 0xed, 0x6d, 0x53, 0x6f, 0x66, 0x69, 0x65, 0x56, 0x61,
0x74, 0x69, 0x6b, 0xe1, 0x6e, 0x56, 0xed, 0x64, 0x65, 0x148, 0x5a, 0xe1,
0x68, 0x159, 0x65, 0x62, 0x43, 0x75, 0x72, 0x79, 0x63, 0x68, 0x56, 0xe1,
0x6e, 0x6f, 0x10d, 0x6e, 0xed, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76,
0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x6f, 0x76, 0xe9, 0x20, 0x6f, 0x73, 0x74,
0x72, 0x6f, 0x76, 0x79, 0x4b, 0x6f, 0x6d, 0x6f, 0x72, 0x79, 0x4b, 0x65,
0x72, 0x67, 0x75, 0x65, 0x6c, 0x65, 0x6e, 0x6f, 0x76, 0x79, 0x20, 0x6f,
0x73, 0x74, 0x72, 0x6f, 0x76, 0x79, 0x4d, 0x61, 0x6c, 0x65, 0x64, 0x69,
0x76, 0x79, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x75, 0x73, 0x43,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x73, 0x6b, 0xe9, 0x20, 0x6f, 0x73,
0x74, 0x72, 0x6f, 0x76, 0x79, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x73, 0x6b,
0xe9, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x79, 0x56, 0x65, 0x6c,
0x69, 0x6b, 0x6f, 0x6e, 0x6f, 0x10d, 0x6e, 0xed, 0x20, 0x6f, 0x73, 0x74,
0x72, 0x6f, 0x76, 0x47, 0x61, 0x6c, 0x61, 0x70, 0xe1, 0x67, 0x79, 0x47,
0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x6f, 0x76, 0x79, 0x20, 0x6f, 0x73,
0x74, 0x72, 0x6f, 0x76, 0x79, 0x4d, 0x61, 0x72, 0x6b, 0xe9, 0x7a, 0x79,
0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x6f, 0x76, 0x79, 0x20,
0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x79, 0x41, 0x6c, 0x67, 0x69, 0x65,
0x72, 0x43, 0x61, 0x79, 0x6d, 0x61, 0x6e, 0xf8, 0x65, 0x72, 0x6e, 0x65,
0x41, 0x73, 0x6a, 0x6b, 0x68, 0x61, 0x62, 0x61, 0x64, 0x44, 0x75, 0x73,
0x6a, 0x61, 0x6e, 0x62, 0x65, 0x4b, 0x61, 0x6d, 0x74, 0x6a, 0x61, 0x74,
0x6b, 0x61, 0x4e, 0x6f, 0x76, 0x6f, 0x6b, 0x75, 0x73, 0x6e, 0x65, 0x74,
0x73, 0x6b, 0x54, 0x61, 0x73, 0x6a, 0x6b, 0x65, 0x6e, 0x74, 0x52, 0x61,
0x6e, 0x67, 0x6f, 0x6f, 0x6e, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x72, 0x6e,
0x65, 0x44, 0x65, 0x20, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x73, 0x6b,
0x65, 0x20, 0xd8, 0x65, 0x72, 0x46, 0xe6, 0x72, 0xf8, 0x65, 0x72, 0x6e,
0x65, 0x4b, 0xf8, 0x62, 0x65, 0x6e, 0x68, 0x61, 0x76, 0x6e, 0x56, 0x61,
0x74, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x74, 0x57, 0x61, 0x72, 0x73, 0x7a,
0x61, 0x77, 0x61, 0x4a, 0x75, 0x6c, 0x65, 0xf8, 0x65, 0x6e, 0x43, 0x6f,
0x6d, 0x6f, 0x72, 0x65, 0x72, 0x6e, 0x65, 0x4d, 0x61, 0x6c, 0x64, 0x69,
0x76, 0x65, 0x72, 0x6e, 0x65, 0x50, 0xe5, 0x73, 0x6b, 0x65, 0xf8, 0x65,
0x6e, 0x43, 0x61, 0xef, 0x72, 0x6f, 0x4b, 0x68, 0x61, 0x72, 0x74, 0x6f,
0x65, 0x6d, 0x53, 0x61, 0x6f, 0x20, 0x54, 0x6f, 0x6d, 0xe9, 0x42, 0x65,
0x6e, 0x65, 0x64, 0x65, 0x6e, 0x20, 0x50, 0x72, 0x69, 0x6e, 0x73, 0x65,
0x6e, 0x20, 0x4b, 0x77, 0x61, 0x72, 0x74, 0x69, 0x65, 0x72, 0x4d, 0x65,
0x78, 0x69, 0x63, 0x6f, 0x2d, 0x53, 0x74, 0x61, 0x64, 0x41, 0x6c, 0x6d,
0x61, 0x2d, 0x41, 0x74, 0x61, 0x41, 0x74, 0x131, 0x72, 0x61, 0x77, 0x44,
0x6f, 0x65, 0x73, 0x6a, 0x61, 0x6e, 0x62, 0x65, 0x48, 0x6f, 0x20, 0x43,
0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x73, 0x74, 0x61, 0x64, 0x4d,
0x61, 0x6e, 0x69, 0x6c, 0x6c, 0x61, 0x53, 0x6a, 0x61, 0x6e, 0x67, 0x68,
0x61, 0x69, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x65,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x6e, 0x43,
0x61, 0x6e, 0x61, 0x72, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x45, 0x69,
0x6c, 0x61, 0x6e, 0x64, 0x65, 0x6e, 0x4b, 0x61, 0x61, 0x70, 0x76, 0x65,
0x72, 0x64, 0x69, 0xeb, 0x46, 0x61, 0x65, 0x72, 0xf6, 0x65, 0x72, 0x5a,
0x75, 0x69, 0x64, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x53,
0x69, 0x6e, 0x74, 0x2d, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x42, 0x65,
0x6c, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x42, 0x65, 0x72, 0x6c, 0x69, 0x6a,
0x6e, 0x49, 0x73, 0x74, 0x61, 0x6e, 0x62, 0x6f, 0x65, 0x6c, 0x50, 0x61,
0x72, 0x69, 0x6a, 0x73, 0x56, 0x61, 0x74, 0x69, 0x63, 0x61, 0x61, 0x6e,
0x73, 0x74, 0x61, 0x64, 0x57, 0x65, 0x6e, 0x65, 0x6e, 0x43, 0x68, 0x61,
0x67, 0x6f, 0x73, 0x61, 0x72, 0x63, 0x68, 0x69, 0x70, 0x65, 0x6c, 0x43,
0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x65, 0x69, 0x6c, 0x61,
0x6e, 0x64, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e,
0x64, 0x65, 0x6e, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x6e, 0x50,
0x61, 0x61, 0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0xce, 0x6c, 0x65,
0x73, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x4d, 0x61, 0x72,
0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64,
0x65, 0x6e, 0xf68, 0xf7a, 0xf40, 0xf0b, 0xf40, 0xfb2, 0xf68, 0xf0b, 0xf4c, 0xf72,
0xf66, 0xf0b, 0x20, 0xf68, 0xf0b, 0xf56, 0xfb7, 0xf0b, 0xf56, 0xfb7, 0xf68, 0xf63,
0xf0b, 0xf47, 0xf72, 0xf0b, 0xf61, 0xf66, 0xf68, 0xf71, 0xf66, 0xf0b, 0xf58, 0xf0b,
0xf62, 0xf56, 0xfb7, 0xf0b, 0xf58, 0xf0b, 0xf40, 0xf7c, 0xf56, 0xfb7, 0xf44, 0xf0b,
0xf42, 0xf72, 0xf56, 0xfb7, 0xf71, 0xf53, 0xf0b, 0xf47, 0xf74, 0xf63, 0xf56, 0xfb7,
0xf72, 0xf0b, 0xf66, 0xf71, 0xf60, 0xf74, 0xf56, 0xfb7, 0xfb2, 0xf0b, 0xf5b, 0xf0b,
0xf56, 0xf72, 0xf63, 0xf56, 0xfb7, 0xf74, 0xf0b, 0xf47, 0xf74, 0xf58, 0xf0b, 0xf56,
0xfb7, 0xf74, 0xf0b, 0xf62, 0xf40, 0xf71, 0xf60, 0xf72, 0xf0b, 0xf62, 0xf7c, 0xf40,
0xf7c, 0xf0b, 0xf53, 0xf40, 0xf0b, 0xf62, 0xf72, 0xf4c, 0xf71, 0xf0b, 0xf40, 0xf62,
0xf47, 0xf72, 0xf0b, 0xf56, 0xfb7, 0xf74, 0xf0b, 0xf4a, 0xf72, 0xf42, 0xfb7, 0xf0b,
0xf56, 0xfb7, 0xf7c, 0xf0b, 0xf62, 0xf7c, 0xf0b, 0xf53, 0xf7a, 0xf67, 0xf0b, 0xf62,
0xf71, 0xf0b, 0xf62, 0xf7a, 0xf40, 0xf7a, 0xf58, 0xf0b, 0xf54, 0xf0b, 0xf63, 0xf41,
0xf71, 0xf62, 0xf0b, 0xf4a, 0xf74, 0xf58, 0xf40, 0xf72, 0xf0b, 0xf42, 0xfb7, 0xf0b,
0xf63, 0xf72, 0xf40, 0xf72, 0xf53, 0xf0b, 0xf64, 0xf0b, 0xf66, 0xf63, 0xf72, 0xf0b,
0xf56, 0xfb7, 0xfb2, 0xf0b, 0xf56, 0xf72, 0xf63, 0xf63, 0xf7c, 0xf0b, 0xf58, 0xf7a,
0xf63, 0xf74, 0xf0b, 0xf5d, 0xf7a, 0xf53, 0xf0b, 0xf4c, 0xf63, 0xf74, 0xf0b, 0xf66,
0xf71, 0xf0b, 0xf40, 0xf58, 0xf0b, 0xf63, 0xf0b, 0xf56, 0xfb7, 0xf7c, 0xf58, 0xf0b,
0xf54, 0xf74, 0xf0b, 0xf4f, 0xf7c, 0xf58, 0xf0b, 0xf66, 0xf0b, 0xf62, 0xf74, 0xf60,
0xf74, 0xf58, 0xf0b, 0xf56, 0xfb7, 0xf0b, 0xf56, 0xfb7, 0xf0b, 0xf53, 0xf72, 0xf58,
0xf7c, 0xf0b, 0xf42, 0xf71, 0xf0b, 0xf4c, 0xf72, 0xf0b, 0xf64, 0xf74, 0xf58, 0xf71,
0xf53, 0xf0b, 0xf62, 0xf7c, 0xf0b, 0xf56, 0xf72, 0xf0b, 0xf61, 0xf53, 0xf71, 0xf60,
0xf72, 0xf0b, 0xf62, 0xf7c, 0xf0b, 0xf56, 0xf72, 0xf53, 0xf72, 0xf0b, 0xf61, 0xf0b,
0xf58, 0xf7a, 0xf53, 0xf74, 0xf0b, 0xf68, 0xf40, 0xf0b, 0xf64, 0xf60, 0xf7c, 0xf4a,
0xf5d, 0xf0b, 0xf42, 0xf71, 0xf0b, 0xf4c, 0xf74, 0xf0b, 0xf42, 0xf74, 0xf66, 0xf5d,
0xf0b, 0x20, 0xf4a, 0xf7c, 0xf0b, 0xf58, 0xf7a, 0xf4f, 0xfb2, 0xf72, 0xf0b, 0xf54,
0xf7c, 0xf0b, 0xf63, 0xf72, 0xf4a, 0xf74, 0xf0b, 0xf53, 0xf72, 0xf66, 0xf84, 0xf5d,
0xf72, 0xf53, 0xf4c, 0xf0b, 0xf67, 0xf74, 0xf40, 0xf68, 0xf53, 0xf0b, 0xf4a, 0xf72,
0xf0b, 0xf42, 0xf74, 0xf0b, 0xf5d, 0xf0b, 0xf56, 0xfb7, 0xf74, 0xf60, 0xf72, 0xf0b,
0xf53, 0xf7c, 0xf66, 0xf0b, 0x20, 0xf68, 0xf60, 0xf72, 0xf0b, 0xf62, 0xf72, 0xf66,
0xf68, 0xf0b, 0xf66, 0xf74, 0xf53, 0xf0b, 0xf66, 0xf72, 0xf0b, 0xf60, 0xf7c, 0xf53,
0xf68, 0xf0b, 0xf4f, 0xf72, 0xf0b, 0xf40, 0xf7c, 0xf0b, 0xf40, 0xf71, 0xf53, 0xf56,
0xfb7, 0xf62, 0xf0b, 0xf56, 0xf7a, 0xf0b, 0xf4c, 0xf7c, 0xf66, 0xf56, 0xfb7, 0xf7a,
0xf0b, 0xf63, 0xf72, 0xf5b, 0xf56, 0xfb7, 0xf7c, 0xf0b, 0xf42, 0xf0b, 0xf4a, 0xf40,
0xf7a, 0xf58, 0xf0b, 0xf56, 0xf72, 0xf62, 0xf72, 0xf51, 0xf46, 0xf0b, 0x20, 0xf56,
0xf60, 0xf7a, 0xf40, 0xf0b, 0xf62, 0xf0b, 0xf40, 0xf71, 0xf66, 0xf45, 0xf72, 0xf0b,
0xf40, 0xf71, 0xf0b, 0xf42, 0xf7c, 0xf40, 0xf7c, 0xf66, 0xf0b, 0xf4a, 0xf0b, 0xf62,
0xf72, 0xf0b, 0xf40, 0xf51, 0xf60, 0xf74, 0xf0b, 0xf66, 0xf71, 0xf7c, 0xf53, 0xf51,
0xf60, 0xf74, 0xf0b, 0xf66, 0xf71, 0xf7c, 0xf53, 0xf0b, 0x20, 0xf40, 0xf72, 0xf62,
0xf72, 0xf40, 0xf4c, 0xf7c, 0xf0b, 0xf58, 0xf72, 0xf0b, 0xf53, 0xf72, 0xf0b, 0xf40,
0xf68, 0xf7a, 0xf4c, 0xf0b, 0xf58, 0xf71, 0xf7c, 0xf53, 0xf0b, 0xf4a, 0xf7c, 0xf53,
0xf68, 0xf7a, 0xf63, 0xf0b, 0x20, 0xf66, 0xf71, 0xf63, 0xf0b, 0xf56, 0xf0b, 0xf4c,
0xf7c, 0xf42, 0xf71, 0xf74, 0xf66, 0xf72, 0xf0b, 0x20, 0xf56, 0xf60, 0xf7a, 0xf42,
0xfb7, 0xf7c, 0xf0b, 0xf4a, 0xf0b, 0xf58, 0xf0b, 0xf63, 0xf42, 0xfb7, 0xf0b, 0xf61,
0xf0b, 0xf53, 0xf67, 0xf0b, 0xf63, 0xf72, 0xf0b, 0xf55, 0xf7a, 0xf42, 0xf66, 0xf72,
0xf68, 0xf72, 0xf0b, 0xf53, 0xf74, 0xf0b, 0xf5d, 0xf72, 0xf40, 0xf47, 0xf71, 0xf0b,
0xf58, 0xf60, 0xf7a, 0xf0b, 0xf40, 0xf63, 0xf0b, 0xf54, 0xf71, 0xf5b, 0xf84, 0xf63,
0xf72, 0xf0b, 0xf58, 0xf58, 0xf7a, 0xf40, 0xf0b, 0xf66, 0xf72, 0xf0b, 0xf40, 0xf7c,
0xf0b, 0x20, 0xf66, 0xf72, 0xf0b, 0xf4a, 0xf72, 0xf58, 0xf71, 0xf7c, 0xf44, 0xf40,
0xf0b, 0xf4a, 0xf7c, 0xf53, 0xf58, 0xf7c, 0xf53, 0xf0b, 0xf4a, 0xf7a, 0xf0b, 0xf56,
0xfb7, 0xf72, 0xf0b, 0xf4c, 0xf72, 0xf60, 0xf7c, 0xf53, 0xf0b, 0xf66, 0xf60, 0xf7c,
0xf54, 0xf0b, 0xf53, 0xf0b, 0xf58, 0xf54, 0xf0b, 0xf62, 0xf0b, 0xf58, 0xf0b, 0xf62,
0xf72, 0xf0b, 0xf56, 0xfb7, 0xf7c, 0xf62, 0xf7a, 0xf53, 0xf0b, 0xf40, 0xf72, 0xf53,
0xf0b, 0x20, 0xf68, 0xf72, 0xf53, 0xf0b, 0xf63, 0xf7a, 0xf4a, 0xf62, 0xf72, 0xf0b,
0xf47, 0xf72, 0xf0b, 0xf53, 0xf62, 0xf72, 0xf0b, 0xf66, 0xf7c, 0xf0b, 0xf63, 0xf72,
0xf60, 0xf74, 0xf4a, 0xf66, 0xf71, 0xf53, 0xf0b, 0xf4a, 0xf72, 0xf0b, 0xf61, 0xf0b,
0xf42, 0xfb7, 0xf7c, 0xf66, 0xf71, 0xf53, 0xf0b, 0xf4a, 0xf7c, 0xf0b, 0x20, 0xf4c,
0xf7c, 0xf0b, 0xf58, 0xf72, 0xf44, 0xf0b, 0xf42, 0xfb7, 0xf7c, 0xf68, 0xf72, 0xf66,
0xf72, 0xf0b, 0xf4a, 0xf7a, 0xf4a, 0xf0b, 0x20, 0xf47, 0xf71, 0xf7c, 0xf53, 0xf66,
0xf72, 0xf0b, 0xf66, 0xf74, 0xf0b, 0xf68, 0xf72, 0xf55, 0xf4a, 0xf0b, 0xf40, 0xf0b,
0xf62, 0xf7a, 0xf53, 0xf4a, 0xf4a, 0xf7a, 0xf0b, 0xf42, 0xf74, 0xf0b, 0xf66, 0xf72,
0xf0b, 0xf42, 0xf71, 0xf63, 0xf0b, 0xf54, 0xf4f, 0xf72, 0xf60, 0xf74, 0xf0b, 0xf5d,
0xf0b, 0xf53, 0xf4a, 0xf7c, 0xf0b, 0xf62, 0xf7c, 0xf53, 0xf0b, 0xf4a, 0xf7c, 0xf61,
0xf0b, 0xf40, 0xf74, 0xf0b, 0xf4f, 0xf4f, 0xf4c, 0xf7a, 0xf0b, 0xf5d, 0xf72, 0xf66,
0xf0d, 0xf58, 0xf60, 0xf74, 0xf0b, 0xf66, 0xf71, 0xf7c, 0xf53, 0xf55, 0xf71, 0xf63,
0xf0b, 0xf58, 0xf62, 0xf62, 0xf71, 0xf7c, 0xf0b, 0xf50, 0xf7a, 0xf0b, 0xf62, 0xf5d,
0xf7c, 0xf66, 0xf72, 0xf0b, 0xf4a, 0xf7c, 0xf40, 0xf56, 0xfb7, 0xf0b, 0xf62, 0xf7a,
0xf53, 0xf0b, 0xf56, 0xfb7, 0xfb2, 0xf74, 0xf0b, 0xf53, 0xf60, 0xf72, 0xf40, 0xf7c,
0xf0b, 0xf63, 0xf7c, 0xf58, 0xf0b, 0xf56, 0xfb7, 0xf7c, 0xf47, 0xf71, 0xf0b, 0xf40,
0xf62, 0xf0b, 0xf4a, 0xf47, 0xf0b, 0xf61, 0xf0b, 0xf54, 0xf74, 0xf0b, 0xf62, 0xf40,
0xf0b, 0xf56, 0xfb7, 0xf74, 0xf63, 0xf40, 0xf0b, 0xf62, 0xf0b, 0xf45, 0xf72, 0xf40,
0xf71, 0xf4a, 0xf0b, 0xf58, 0xf71, 0xf53, 0xf0b, 0xf4c, 0xf74, 0xf40, 0xf71, 0xf7c,
0xf63, 0xf0b, 0xf40, 0xf0b, 0xf4a, 0xf40, 0xf7c, 0xf0b, 0xf63, 0xf0b, 0x20, 0xf63,
0xf58, 0xf0b, 0xf54, 0xf74, 0xf62, 0xf54, 0xf71, 0xf7c, 0xf53, 0xf0b, 0xf4a, 0xf60,
0xf7a, 0xf0b, 0xf53, 0xf40, 0xf4a, 0xf7a, 0xf0b, 0xf62, 0xf71, 0xf53, 0xf50, 0xf72,
0xf58, 0xf0b, 0xf55, 0xf74, 0xf42, 0xf56, 0xf72, 0xf0b, 0xf61, 0xf71, 0xf53, 0xf0b,
0xf4a, 0xf72, 0xf0b, 0xf61, 0xf71, 0xf53, 0xf62, 0xf44, 0xf0b, 0xf42, 0xfb7, 0xf74,
0xf53, 0xf56, 0xf62, 0xf0b, 0xf58, 0xf74, 0xf0b, 0xf51, 0xf40, 0xf7a, 0xf54, 0xf0b,
0xf56, 0xf71, 0xf4c, 0xf55, 0xf71, 0xf60, 0xf7a, 0xf0b, 0xf62, 0xf7c, 0xf0b, 0xf62,
0xf7a, 0xf0b, 0xf41, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf56, 0xf72, 0xf40, 0xf66, 0xf71,
0xf60, 0xf74, 0xf50, 0xf0b, 0xf47, 0xf7c, 0xf0b, 0xf47, 0xf71, 0xf68, 0xf58, 0xf66,
0xf0b, 0xf4a, 0xf62, 0xf0b, 0xf4c, 0xf58, 0xf68, 0xf71, 0xf53, 0xf0b, 0xf51, 0xf7c,
0xf0b, 0xf62, 0xf68, 0xf7a, 0xf0b, 0xf50, 0xf7a, 0xf53, 0xf66, 0xf0b, 0xf56, 0xfb7,
0xf62, 0xf0b, 0xf63, 0xf72, 0xf53, 0xf56, 0xfb2, 0xf0b, 0xf66, 0xf7a, 0xf63, 0xf66,
0xf56, 0xfb7, 0xf74, 0xf0b, 0xf40, 0xf0b, 0xf62, 0xf7a, 0xf66, 0xf4a, 0xf56, 0xfb7,
0xf74, 0xf0b, 0xf4c, 0xf0b, 0xf54, 0xf7a, 0xf66, 0xf4a, 0xf40, 0xf72, 0xf0b, 0xf64,
0xf72, 0xf0b, 0xf53, 0xf71, 0xf5d, 0xf40, 0xf7c, 0xf0b, 0xf54, 0xf7a, 0xf53, 0xf0b,
0xf67, 0xf7a, 0xf0b, 0xf42, 0xf7a, 0xf53, 0xf4c, 0xf56, 0xf0b, 0xf63, 0xf72, 0xf53,
0xf67, 0xf7a, 0xf63, 0xf0b, 0xf66, 0xf72, 0xf44, 0xf0b, 0xf40, 0xf72, 0xf63, 0xf71,
0xf53, 0xf0b, 0xf4c, 0xf71, 0xf53, 0xf63, 0xf42, 0xf0b, 0xf66, 0xf58, 0xf0b, 0xf56,
0xfb7, 0xf40, 0xf58, 0xf0b, 0xf4c, 0xf7a, 0xf0b, 0xf62, 0xf7a, 0xf4c, 0xf58, 0xf72,
0xf53, 0xf66, 0xf40, 0xf58, 0xf7c, 0xf0b, 0xf53, 0xf0b, 0xf40, 0xf7c, 0xf58, 0xf71,
0xf7c, 0xf66, 0xf0b, 0xf40, 0xf7c, 0xf68, 0xf7c, 0xf66, 0xf0b, 0xf63, 0xf7c, 0xf54,
0xf7a, 0xf0b, 0xf62, 0xf72, 0xf66, 0xf54, 0xfb2, 0xf42, 0xf62, 0xf72, 0xf0b, 0xf42,
0xfb7, 0xf66, 0xf0b, 0xf62, 0xf0b, 0xf61, 0xf7a, 0xf0b, 0xf56, 0xf7c, 0xf66, 0xf7c,
0xf0b, 0xf55, 0xf72, 0xf0b, 0xf61, 0xf68, 0xf72, 0xf66, 0xf0b, 0xf4a, 0xf7c, 0xf40,
0xf0b, 0xf67, 0xf7c, 0xf63, 0xf58, 0xf4a, 0xf71, 0xf0b, 0xf63, 0xf71, 0xf72, 0xf53,
0xf4a, 0xf0b, 0xf62, 0xf0b, 0xf53, 0xf72, 0xf56, 0xf0b, 0xf51, 0xf74, 0xf51, 0xf66,
0xf5d, 0xf72, 0xf0b, 0xf61, 0xf7a, 0xf0b, 0xf53, 0xf56, 0xf72, 0xf63, 0xf0b, 0xf53,
0xf72, 0xf0b, 0xf61, 0xf71, 0xf66, 0xf5d, 0xf62, 0xf0b, 0xf66, 0xf7c, 0xf5b, 0xf74,
0xf0b, 0xf62, 0xf72, 0xf46, 0xf68, 0xf71, 0xf53, 0xf0b, 0xf4f, 0xf0b, 0xf53, 0xf0b,
0xf53, 0xf0b, 0xf51, 0xf72, 0xf0b, 0xf56, 0xf7c, 0xf40, 0xf72, 0xf62, 0xf72, 0xf66,
0xf4a, 0xf0b, 0xf58, 0xf66, 0xf72, 0xf40, 0xf7c, 0xf0b, 0xf40, 0xf7c, 0xf66, 0xf72,
0xf40, 0xf7c, 0xf0b, 0xf58, 0xf7c, 0xf0b, 0xf62, 0xf7c, 0xf40, 0xf72, 0xf62, 0xf0b,
0xf42, 0xf74, 0xf0b, 0xf63, 0xf72, 0xf53, 0xf58, 0xf71, 0xf63, 0xf0b, 0xf51, 0xf72,
0xf56, 0xf66, 0xf58, 0xf7c, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf64, 0xf71, 0xf66, 0xf68,
0xf0b, 0xf54, 0xf72, 0xf0b, 0xf61, 0xf55, 0xf0b, 0xf40, 0xf60, 0xf7c, 0xf0b, 0xf55,
0xf7c, 0xf55, 0xf74, 0xf0b, 0xf53, 0xf0b, 0xf55, 0xf74, 0xf0b, 0xf4a, 0xf72, 0xf42,
0xf7a, 0xf58, 0xf0b, 0xf56, 0xf72, 0xf61, 0xf62, 0xf58, 0xf7a, 0xf0b, 0xf47, 0xf74,
0xf0b, 0xf62, 0xf7c, 0xf58, 0xf72, 0xf4c, 0xf0b, 0xf5d, 0xf7a, 0xf53, 0xf71, 0xf5d,
0xf0b, 0xf62, 0xf74, 0xf54, 0xf0b, 0xf42, 0xf7c, 0xf0b, 0x20, 0xf54, 0xf0b, 0xf42,
0xf7c, 0xf54, 0xf0b, 0xf63, 0xf71, 0xf5d, 0xf62, 0xf71, 0xf0b, 0xf62, 0xf7c, 0xf0b,
0xf0b, 0xf4f, 0xf7c, 0xf44, 0xf0b, 0xf42, 0xf66, 0xf71, 0xf60, 0xf7a, 0xf0b, 0xf55,
0xf71, 0xf53, 0xf4f, 0xf7c, 0xf44, 0xf0b, 0xf42, 0xf0b, 0xf4f, 0xf0b, 0xf54, 0xf74,
0xf5d, 0xf7a, 0xf40, 0xd801, 0xdc08, 0xd801, 0xdc3c, 0xd801, 0xdc30, 0xd801, 0xdc3f, 0xd801,
0xdc01, 0xd801, 0xdc4d, 0xd801, 0xdc3f, 0xd801, 0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc2e, 0xd801,
0xdc3e, 0xd801, 0xdc12, 0xd801, 0xdc31, 0xd801, 0xdc2e, 0xd801, 0xdc46, 0xd801, 0xdc28, 0xd801,
0xdc1f, 0xd801, 0xdc2e, 0xd801, 0xdc3f, 0xd801, 0xdc2a, 0xd801, 0xdc40, 0xd801, 0xdc2c, 0xd801,
0xdc14, 0xd801, 0xdc2f, 0xd801, 0xdc4c, 0xd801, 0xdc42, 0xd801, 0xdc32, 0xd801, 0xdc49, 0xd801,
0xdc14, 0xd801, 0xdc32, 0xd801, 0xdc3b, 0xd801, 0xdc49, 0xd801, 0xdc31, 0xd801, 0xdc2e, 0xd801,
0xdc3b, 0xd801, 0xdc06, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc28, 0xd801, 0xdc32, 0xd801,
0xdc4c, 0xd801, 0xdc30, 0xd801, 0xdc39, 0xd801, 0xdc2c, 0xd801, 0xdc4a, 0xd801, 0xdc32, 0xd801,
0xdc45, 0xd801, 0xdc24, 0xd801, 0xdc2a, 0xd801, 0xdc3f, 0xd801, 0xdc45, 0x2c, 0x20, 0xd801,
0xdc06, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc28, 0xd801, 0xdc30, 0xd801, 0xdc4c, 0xd801,
0xdc32, 0xd801, 0xdc23, 0xd801, 0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc2f, 0xd801, 0xdc4d, 0xd801,
0xdc40, 0xd801, 0xdc2c, 0x2c, 0x20, 0xd801, 0xdc06, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801,
0xdc28, 0xd801, 0xdc30, 0xd801, 0xdc4c, 0xd801, 0xdc32, 0xd801, 0xdc11, 0xd801, 0xdc28, 0xd801,
0xdc3b, 0xd801, 0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc46, 0xd801, 0xdc3a, 0xd801, 0xdc32, 0xd801,
0xdc49, 0xd801, 0xdc40, 0x2c, 0x20, 0xd801, 0xdc06, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801,
0xdc28, 0xd801, 0xdc30, 0xd801, 0xdc4c, 0xd801, 0xdc32, 0xd801, 0xdc13, 0xd801, 0xdc2f, 0xd801,
0xdc4a, 0x20, 0xd801, 0xdc1d, 0xd801, 0xdc2e, 0xd801, 0xdc3c, 0xd801, 0xdc28, 0x2c, 0x20,
0xd801, 0xdc06, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc28, 0xd801, 0xdc30, 0xd801, 0xdc4c,
0xd801, 0xdc32, 0xd801, 0xdc1a, 0xd801, 0xdc2f, 0xd801, 0xdc42, 0xd801, 0xdc29, 0x2c, 0x20,
0xd801, 0xdc06, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc28, 0xd801, 0xdc30, 0xd801, 0xdc4c,
0xd801, 0xdc32, 0xd801, 0xdc1a, 0xd801, 0xdc2e, 0xd801, 0xdc4c, 0xd801, 0xdc45, 0xd801, 0xdc2f,
0xd801, 0xdc4c, 0xd801, 0xdc46, 0x2c, 0x20, 0xd801, 0xdc06, 0xd801, 0xdc4c, 0xd801, 0xdc3c,
0xd801, 0xdc28, 0xd801, 0xdc30, 0xd801, 0xdc4c, 0xd801, 0xdc32, 0xd801, 0xdc0e, 0xd801, 0xdc2e,
0xd801, 0xdc4c, 0xd801, 0xdc32, 0xd801, 0xdc4b, 0xd801, 0xdc30, 0xd801, 0xdc3f, 0x2c, 0x20,
0xd801, 0xdc06, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc28, 0xd801, 0xdc30, 0xd801, 0xdc4c,
0xd801, 0xdc32, 0xd801, 0xdc16, 0xd801, 0xdc2d, 0xd801, 0xdc4c, 0xd801, 0xdc2c, 0xd801, 0xdc22,
0xd801, 0xdc2d, 0xd801, 0xdc36, 0xd801, 0xdc28, 0xd801, 0xdc42, 0xd801, 0xdc2e, 0xd801, 0xdc4a,
0xd801, 0xdc23, 0xd801, 0xdc2a, 0xd801, 0xdc4c, 0xd801, 0xdc3b, 0xd801, 0xdc32, 0xd801, 0xdc45,
0xd801, 0xdc2f, 0xd801, 0xdc4a, 0xd801, 0xdc2c, 0x2c, 0x20, 0xd801, 0xdc17, 0xd801, 0xdc32,
0xd801, 0xdc4c, 0xd801, 0xdc3b, 0xd801, 0xdc32, 0xd801, 0xdc3f, 0xd801, 0xdc28, 0xd801, 0xdc22,
0xd801, 0xdc31, 0xd801, 0xdc45, 0x20, 0xd801, 0xdc08, 0xd801, 0xdc4c, 0xd801, 0xdc3e, 0xd801,
0xdc32, 0xd801, 0xdc4a, 0xd801, 0xdc45, 0xd801, 0xdc23, 0xd801, 0xdc32, 0xd801, 0xdc4c, 0xd801,
0xdc2a, 0xd801, 0xdc4b, 0xd801, 0xdc32, 0xd801, 0xdc4c, 0xd801, 0xdc28, 0xd801, 0xdc24, 0xd801,
0xdc2d, 0x20, 0xd801, 0xdc0f, 0xd801, 0xdc31, 0xd801, 0xdc49, 0xd801, 0xdc3f, 0xd801, 0xdc24,
0xd801, 0xdc2c, 0xd801, 0xdc4b, 0xd801, 0xdc1d, 0xd801, 0xdc2f, 0xd801, 0xdc4c, 0xd801, 0xdc3b,
0xd801, 0xdc32, 0xd801, 0xdc49, 0x2c, 0x20, 0xd801, 0xdc24, 0xd801, 0xdc31, 0xd801, 0xdc49,
0xd801, 0xdc43, 0x20, 0xd801, 0xdc14, 0xd801, 0xdc32, 0xd801, 0xdc3f, 0xd801, 0xdc2c, 0xd801,
0xdc3c, 0xd801, 0xdc32, 0xd801, 0xdc24, 0xd801, 0xdc2d, 0x20, 0xd801, 0xdc1d, 0xd801, 0xdc29,
0xd801, 0xdc4a, 0xd801, 0xdc32, 0xd801, 0xdc4b, 0x2c, 0x20, 0xd801, 0xdc24, 0xd801, 0xdc31,
0xd801, 0xdc49, 0xd801, 0xdc43, 0x20, 0xd801, 0xdc14, 0xd801, 0xdc32, 0xd801, 0xdc3f, 0xd801,
0xdc2c, 0xd801, 0xdc3c, 0xd801, 0xdc32, 0xd801, 0xdc19, 0xd801, 0xdc28, 0xd801, 0xdc4c, 0xd801,
0xdc2e, 0xd801, 0xdc3f, 0xd801, 0xdc45, 0xd801, 0xdc0f, 0xd801, 0xdc30, 0xd801, 0xdc3f, 0xd801,
0xdc2d, 0xd801, 0xdc3b, 0xd801, 0xdc30, 0xd801, 0xdc3b, 0xd801, 0xdc10, 0xd801, 0xdc2a, 0xd801,
0xdc4c, 0xd801, 0xdc32, 0xd801, 0xdc4a, 0xd801, 0xdc2d, 0xd801, 0xdc4a, 0xd801, 0xdc2d, 0xd801,
0xdc23, 0xd801, 0xdc2e, 0xd801, 0xdc3c, 0xd801, 0xdc36, 0xd801, 0xdc29, 0xd801, 0xdc0e, 0xd801,
0xdc29, 0xd801, 0xdc3f, 0x53, 0x74, 0x20, 0x42, 0x61, 0x72, 0x74, 0x68, 0xe9,
0x6c, 0x65, 0x6d, 0x79, 0x53, 0x74, 0x20, 0x4a, 0x6f, 0x68, 0x6e, 0x2019,
0x73, 0x53, 0x74, 0x20, 0x4b, 0x69, 0x74, 0x74, 0x73, 0x53, 0x74, 0x20,
0x4c, 0x75, 0x63, 0x69, 0x61, 0x53, 0x74, 0x20, 0x54, 0x68, 0x6f, 0x6d,
0x61, 0x73, 0x53, 0x74, 0x20, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x74,
0x53, 0x74, 0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x428, 0x438, 0x43a,
0x430, 0x433, 0x43e, 0x41b, 0x43e, 0x43d, 0x433, 0x439, 0x438, 0x440, 0x431, 0x44e,
0x435, 0x43d, 0x41e, 0x43c, 0x441, 0x43a, 0x43e, 0x439, 0x422, 0x430, 0x439, 0x43f,
0x435, 0x439, 0x422, 0x43e, 0x43c, 0x441, 0x43a, 0x43e, 0x439, 0x423, 0x43b, 0x430,
0x43d, 0x2d, 0x431, 0x430, 0x442, 0x43e, 0x440, 0x410, 0x444, 0x438, 0x43d, 0x44b,
0x41c, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x439, 0x41c, 0x43e, 0x441, 0x43a, 0x43e,
0x432, 0x41, 0x62, 0x69, 0x11d, 0x61, 0x6e, 0x6f, 0x41, 0x6b, 0x72, 0x61,
0x6f, 0x41, 0x64, 0x69, 0x73, 0x61, 0x62, 0x65, 0x62, 0x6f, 0x41, 0x6c,
0x11d, 0x65, 0x72, 0x6f, 0x41, 0x73, 0x6d, 0x65, 0x72, 0x6f, 0x42, 0x61,
0x6e, 0x67, 0x6f, 0x42, 0x61, 0x6e, 0x6a, 0x75, 0x6c, 0x6f, 0x42, 0x69,
0x73, 0x61, 0x16d, 0x6f, 0x42, 0x72, 0x61, 0x7a, 0x61, 0x76, 0x69, 0x6c,
0x6f, 0x42, 0x75, 0x135, 0x75, 0x6d, 0x62, 0x75, 0x72, 0x6f, 0x4b, 0x61,
0x7a, 0x61, 0x62, 0x6c, 0x61, 0x6e, 0x6b, 0x6f, 0x43, 0x65, 0x16d, 0x74,
0x6f, 0x4b, 0x6f, 0x6e, 0x61, 0x6b, 0x72, 0x69, 0x6f, 0x44, 0x61, 0x6b,
0x61, 0x72, 0x6f, 0x44, 0x61, 0x72, 0x65, 0x73, 0x61, 0x6c, 0x61, 0x6d,
0x6f, 0x11c, 0x69, 0x62, 0x75, 0x74, 0x69, 0x6f, 0x44, 0x75, 0x61, 0x6c,
0x61, 0x6f, 0x41, 0x6a, 0x75, 0x6e, 0x6f, 0x46, 0x72, 0x69, 0x74, 0x61,
0x16d, 0x6e, 0x6f, 0x47, 0x61, 0x62, 0x6f, 0x72, 0x6f, 0x6e, 0x6f, 0x48,
0x61, 0x72, 0x61, 0x72, 0x65, 0x6f, 0x4a, 0x6f, 0x68, 0x61, 0x6e, 0x65,
0x73, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x11c, 0x75, 0x62, 0x61, 0x6f, 0x4b,
0x61, 0x6d, 0x70, 0x61, 0x6c, 0x6f, 0x124, 0x61, 0x72, 0x74, 0x75, 0x6d,
0x6f, 0x4b, 0x69, 0x67, 0x61, 0x6c, 0x6f, 0x4b, 0x69, 0x6e, 0x15d, 0x61,
0x73, 0x6f, 0x4c, 0x61, 0x67, 0x6f, 0x73, 0x6f, 0x4c, 0x69, 0x62, 0x72,
0x65, 0x76, 0x69, 0x6c, 0x6f, 0x4c, 0x6f, 0x6d, 0x65, 0x6f, 0x4c, 0x75,
0x61, 0x6e, 0x64, 0x6f, 0x4c, 0x75, 0x62, 0x75, 0x6d, 0x62, 0x61, 0x15d,
0x6f, 0x4c, 0x75, 0x73, 0x61, 0x6b, 0x6f, 0x4d, 0x61, 0x73, 0x65, 0x72,
0x75, 0x6f, 0x4d, 0x62, 0x61, 0x62, 0x61, 0x6e, 0x6f, 0x4d, 0x6f, 0x67,
0x61, 0x64, 0x69, 0x15d, 0x6f, 0x4d, 0x6f, 0x6e, 0x72, 0x6f, 0x76, 0x69,
0x6f, 0x4e, 0x61, 0x6a, 0x72, 0x6f, 0x62, 0x69, 0x6f, 0x4e, 0x69, 0x11d,
0x61, 0x6d, 0x65, 0x6e, 0x6f, 0x4e, 0x69, 0x61, 0x6d, 0x65, 0x6a, 0x6f,
0x4e, 0x75, 0x61, 0x6b, 0x15d, 0x6f, 0x74, 0x6f, 0x56, 0x61, 0x67, 0x61,
0x64, 0x75, 0x67, 0x75, 0x6f, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x6e, 0x6f,
0x76, 0x6f, 0x53, 0x61, 0x6e, 0x74, 0x6f, 0x6d, 0x65, 0x6f, 0x54, 0x72,
0x69, 0x70, 0x6f, 0x6c, 0x6f, 0x54, 0x75, 0x6e, 0x69, 0x7a, 0x6f, 0x56,
0x69, 0x6e, 0x64, 0x68, 0x75, 0x6b, 0x6f, 0x41, 0x6e, 0x67, 0x76, 0x69,
0x6c, 0x6f, 0x41, 0x6e, 0x74, 0x69, 0x67, 0x76, 0x6f, 0x42, 0x6f, 0x6e,
0x61, 0x65, 0x72, 0x6f, 0x4b, 0x6f, 0x72, 0x64, 0x6f, 0x62, 0x6f, 0x53,
0x61, 0x6c, 0x74, 0x61, 0x75, 0x72, 0x62, 0x6f, 0x41, 0x72, 0x75, 0x62,
0x6f, 0x41, 0x73, 0x75, 0x6e, 0x63, 0x69, 0x6f, 0x6e, 0x6f, 0x42, 0x61,
0x68, 0x69, 0x6f, 0x42, 0x61, 0x72, 0x62, 0x61, 0x64, 0x6f, 0x42, 0x65,
0x6c, 0x65, 0x6d, 0x6f, 0x42, 0x65, 0x6c, 0x69, 0x7a, 0x6f, 0x42, 0x6f,
0x67, 0x6f, 0x74, 0x6f, 0x4b, 0x65, 0x6d, 0x62, 0x72, 0x69, 0x11d, 0x61,
0x20, 0x47, 0x6f, 0x6c, 0x66, 0x6f, 0x4b, 0x61, 0x6e, 0x6b, 0x75, 0x6e,
0x6f, 0x4b, 0x61, 0x72, 0x61, 0x6b, 0x61, 0x73, 0x6f, 0x4b, 0x61, 0x6a,
0x65, 0x6e, 0x6f, 0x4b, 0x61, 0x6a, 0x6d, 0x61, 0x6e, 0x6f, 0x6a, 0x108,
0x69, 0x6b, 0x61, 0x67, 0x6f, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x72, 0x69,
0x6b, 0x6f, 0x4b, 0x75, 0x72, 0x61, 0x63, 0x61, 0x6f, 0x44, 0x65, 0x6e,
0x76, 0x65, 0x72, 0x6f, 0x44, 0x65, 0x74, 0x72, 0x6f, 0x6a, 0x74, 0x6f,
0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x69, 0x6b, 0x6f, 0x45, 0x64, 0x6d, 0x6f,
0x6e, 0x74, 0x6f, 0x6e, 0x6f, 0x53, 0x61, 0x6c, 0x76, 0x61, 0x64, 0x6f,
0x72, 0x6f, 0x46, 0x6f, 0x72, 0x74, 0x61, 0x6c, 0x65, 0x7a, 0x6f, 0x47,
0x72, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6f, 0x47,
0x72, 0x65, 0x6e, 0x61, 0x64, 0x6f, 0x47, 0x76, 0x61, 0x64, 0x65, 0x6c,
0x75, 0x70, 0x6f, 0x47, 0x76, 0x61, 0x74, 0x65, 0x6d, 0x61, 0x6c, 0x6f,
0x47, 0x75, 0x6a, 0x61, 0x6e, 0x6f, 0x48, 0x61, 0x6c, 0x69, 0x66, 0x61,
0x6b, 0x73, 0x6f, 0x48, 0x61, 0x76, 0x61, 0x6e, 0x6f, 0x49, 0x6e, 0x64,
0x69, 0x61, 0x6e, 0x61, 0x70, 0x6f, 0x6c, 0x6f, 0x2c, 0x20, 0x49, 0x6e,
0x64, 0x69, 0x61, 0x6e, 0x61, 0x6f, 0x4b, 0x6e, 0x6f, 0x78, 0x2c, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x6f, 0x4d, 0x61, 0x72, 0x65,
0x6e, 0x67, 0x6f, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61,
0x6f, 0x50, 0x65, 0x74, 0x65, 0x72, 0x73, 0x62, 0x75, 0x72, 0x67, 0x2c,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x6f, 0x54, 0x65, 0x6c,
0x6c, 0x20, 0x43, 0x69, 0x74, 0x79, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69,
0x61, 0x6e, 0x61, 0x6f, 0x56, 0x65, 0x76, 0x61, 0x79, 0x2c, 0x20, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x6f, 0x56, 0x69, 0x6e, 0x63, 0x65,
0x6e, 0x6e, 0x65, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e,
0x61, 0x6f, 0x57, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x63, 0x2c, 0x20, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x6f, 0x49, 0x6b, 0x61, 0x6c, 0x75,
0x69, 0x74, 0x6f, 0x4a, 0x61, 0x6d, 0x61, 0x6a, 0x6b, 0x6f, 0x4d, 0x6f,
0x6e, 0x74, 0x69, 0x63, 0x65, 0x6c, 0x6c, 0x6f, 0x2c, 0x20, 0x4b, 0x65,
0x6e, 0x74, 0x75, 0x6b, 0x69, 0x6f, 0x4c, 0x69, 0x6d, 0x6f, 0x4c, 0x6f,
0x73, 0x61, 0x6e, 0x11d, 0x65, 0x6c, 0x65, 0x73, 0x6f, 0x4d, 0x61, 0x6e,
0x61, 0x67, 0x76, 0x6f, 0x4d, 0x61, 0x6e, 0x61, 0x16d, 0x73, 0x6f, 0x4d,
0x61, 0x72, 0x69, 0x67, 0x6f, 0x74, 0x6f, 0x4d, 0x61, 0x72, 0x74, 0x69,
0x6e, 0x69, 0x6b, 0x6f, 0x4d, 0x65, 0x72, 0x69, 0x64, 0x6f, 0x4d, 0x65,
0x6b, 0x73, 0x69, 0x6b, 0x75, 0x72, 0x62, 0x6f, 0x4d, 0x69, 0x6b, 0x65,
0x6c, 0x6f, 0x6e, 0x6f, 0x4d, 0x6f, 0x6e, 0x6b, 0x74, 0x6f, 0x6e, 0x6f,
0x4d, 0x6f, 0x6e, 0x74, 0x65, 0x72, 0x65, 0x6a, 0x6f, 0x4d, 0x6f, 0x6e,
0x63, 0x65, 0x72, 0x61, 0x74, 0x6f, 0x4e, 0x61, 0x73, 0x61, 0x16d, 0x6f,
0x4e, 0x6f, 0x76, 0x6a, 0x6f, 0x72, 0x6b, 0x6f, 0x42, 0x65, 0x75, 0x6c,
0x61, 0x68, 0x2c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x61, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x6f, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20,
0x4e, 0x6f, 0x72, 0x64, 0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x6f,
0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x4e,
0x6f, 0x72, 0x64, 0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x6f, 0x4e,
0x75, 0x6b, 0x6f, 0x50, 0x61, 0x6e, 0x61, 0x6d, 0x6f, 0x46, 0x65, 0x6e,
0x69, 0x6b, 0x73, 0x6f, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x70, 0x72, 0x69,
0x6e, 0x63, 0x6f, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x73, 0x70, 0x65, 0x6e,
0x6f, 0x50, 0x75, 0x65, 0x72, 0x74, 0x6f, 0x72, 0x69, 0x6b, 0x6f, 0x53,
0x61, 0x6e, 0x6b, 0x74, 0x61, 0x20, 0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x67,
0x6f, 0x53, 0x61, 0x6e, 0x70, 0x61, 0x16d, 0x6c, 0x6f, 0x53, 0x61, 0x6e,
0x6b, 0x74, 0x61, 0x20, 0x42, 0x61, 0x72, 0x74, 0x6f, 0x6c, 0x6f, 0x6d,
0x65, 0x6f, 0x53, 0x61, 0x6e, 0x6b, 0x74, 0x61, 0x20, 0x4b, 0x72, 0x69,
0x73, 0x74, 0x6f, 0x66, 0x6f, 0x72, 0x6f, 0x53, 0x61, 0x6e, 0x6b, 0x74,
0x61, 0x20, 0x4c, 0x75, 0x63, 0x69, 0x6f, 0x53, 0x61, 0x6e, 0x6b, 0x74,
0x61, 0x20, 0x54, 0x6f, 0x6d, 0x61, 0x73, 0x6f, 0x53, 0x61, 0x6e, 0x6b,
0x74, 0x61, 0x20, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x74, 0x6f, 0x54,
0x65, 0x67, 0x75, 0x63, 0x69, 0x67, 0x61, 0x6c, 0x70, 0x6f, 0x56, 0x61,
0x6e, 0x6b, 0x75, 0x76, 0x65, 0x72, 0x6f, 0x56, 0x69, 0x6e, 0x69, 0x70,
0x65, 0x67, 0x6f, 0x4d, 0x61, 0x6b, 0x76, 0x6f, 0x72, 0x61, 0x20, 0x69,
0x6e, 0x73, 0x75, 0x6c, 0x6f, 0x53, 0x68, 0x6f, 0x77, 0x61, 0x4c, 0x6f,
0x6e, 0x67, 0x6a, 0x65, 0x72, 0x75, 0x72, 0x62, 0x6f, 0x41, 0x64, 0x65,
0x6e, 0x6f, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x6f, 0x41, 0x6d, 0x61, 0x6e,
0x6f, 0x41, 0x15d, 0x125, 0x61, 0x62, 0x61, 0x64, 0x6f, 0x42, 0x61, 0x67,
0x64, 0x61, 0x64, 0x6f, 0x42, 0x61, 0x72, 0x65, 0x6a, 0x6e, 0x6f, 0x42,
0x61, 0x6b, 0x75, 0x6f, 0x42, 0x61, 0x6e, 0x6b, 0x6f, 0x6b, 0x6f, 0x42,
0x65, 0x6a, 0x72, 0x75, 0x74, 0x6f, 0x42, 0x69, 0x15d, 0x6b, 0x65, 0x6b,
0x6f, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x6a, 0x6f, 0x108, 0x69, 0x74, 0x61,
0x44, 0x69, 0x6c, 0x6f, 0x44, 0x75, 0x62, 0x61, 0x6a, 0x75, 0x72, 0x62,
0x6f, 0x44, 0x75, 0x15d, 0x61, 0x6e, 0x62, 0x65, 0x6f, 0x47, 0x61, 0x7a,
0x61, 0x6f, 0x48, 0x65, 0x62, 0x72, 0x6f, 0x6e, 0x6f, 0x48, 0x6f, 0x109,
0x69, 0x6d, 0x69, 0x6e, 0x6f, 0x48, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e, 0x67,
0x6f, 0x124, 0x6f, 0x76, 0x64, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x6f, 0x11c, 0x61, 0x6b, 0x61, 0x72, 0x74, 0x6f, 0x4a, 0x65, 0x72, 0x75,
0x73, 0x61, 0x6c, 0x65, 0x6d, 0x6f, 0x4b, 0x61, 0x62, 0x75, 0x6c, 0x6f,
0x4b, 0x61, 0x6d, 0x109, 0x61, 0x74, 0x6b, 0x6f, 0x4b, 0x61, 0x72, 0x61,
0x109, 0x69, 0x6f, 0x4b, 0x61, 0x74, 0x6d, 0x61, 0x6e, 0x64, 0x75, 0x6f,
0x124, 0x61, 0x6e, 0x64, 0x69, 0x67, 0x61, 0x4b, 0x6f, 0x6c, 0x6b, 0x61,
0x74, 0x6f, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73,
0x6b, 0x6f, 0x4b, 0x75, 0x61, 0x6c, 0x61, 0x6c, 0x75, 0x6d, 0x70, 0x75,
0x72, 0x6f, 0x4b, 0x75, 0x76, 0x61, 0x6a, 0x74, 0x6f, 0x4d, 0x61, 0x6e,
0x69, 0x6c, 0x6f, 0x4d, 0x61, 0x73, 0x6b, 0x61, 0x74, 0x6f, 0x4e, 0x69,
0x6b, 0x6f, 0x7a, 0x69, 0x6f, 0x4e, 0x6f, 0x76, 0x6f, 0x6b, 0x75, 0x7a,
0x6e, 0x65, 0x63, 0x6b, 0x6f, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x6f, 0x4f, 0x6d, 0x73, 0x6b, 0x6f, 0x4f, 0x72,
0x61, 0x6c, 0x6f, 0x50, 0x6e, 0x6f, 0x6d, 0x70, 0x65, 0x6e, 0x6f, 0x50,
0x6a, 0x6f, 0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x67, 0x6f, 0x4b, 0x61, 0x74,
0x61, 0x72, 0x6f, 0x4b, 0x69, 0x7a, 0x69, 0x6c, 0x6f, 0x72, 0x64, 0x6f,
0x52, 0x69, 0x61, 0x64, 0x6f, 0x53, 0x61, 0x125, 0x61, 0x6c, 0x65, 0x6e,
0x6f, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x6b, 0x61, 0x6e, 0x64, 0x6f, 0x53,
0x65, 0x75, 0x6c, 0x6f, 0x15c, 0x61, 0x6e, 0x68, 0x61, 0x6a, 0x6f, 0x53,
0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x6f, 0x54, 0x61, 0x6a, 0x70,
0x65, 0x6f, 0x54, 0x61, 0x15d, 0x6b, 0x65, 0x6e, 0x74, 0x6f, 0x54, 0x62,
0x69, 0x6c, 0x69, 0x73, 0x6f, 0x54, 0x65, 0x68, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x54, 0x69, 0x6d, 0x62, 0x75, 0x6f, 0x55, 0x6c, 0x61, 0x6e, 0x62,
0x61, 0x74, 0x6f, 0x72, 0x6f, 0x55, 0x72, 0x75, 0x6d, 0x109, 0x69, 0x6f,
0x55, 0x73, 0x74, 0x6e, 0x65, 0x72, 0x6f, 0x56, 0x6a, 0x65, 0x6e, 0x74,
0x69, 0x61, 0x6e, 0x6f, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x6f, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f,
0x52, 0x61, 0x6e, 0x67, 0x75, 0x6e, 0x6f, 0x4a, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x45, 0x72, 0x65,
0x76, 0x61, 0x6e, 0x6f, 0x41, 0x63, 0x6f, 0x72, 0x6f, 0x6a, 0x42, 0x65,
0x72, 0x6d, 0x75, 0x64, 0x6f, 0x6a, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x69,
0x6f, 0x6a, 0x4b, 0x61, 0x62, 0x6f, 0x76, 0x65, 0x72, 0x64, 0x6f, 0x46,
0x65, 0x72, 0x6f, 0x6f, 0x6a, 0x4d, 0x61, 0x64, 0x65, 0x6a, 0x72, 0x6f,
0x52, 0x65, 0x6a, 0x6b, 0x6a, 0x61, 0x76, 0x69, 0x6b, 0x6f, 0x53, 0x75,
0x64, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x6f, 0x53, 0x61, 0x6e,
0x6b, 0x74, 0x61, 0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x6f, 0x53, 0x74,
0x61, 0x6e, 0x6c, 0x65, 0x6a, 0x6f, 0x41, 0x64, 0x65, 0x6c, 0x61, 0x6a,
0x64, 0x6f, 0x42, 0x72, 0x69, 0x73, 0x62, 0x61, 0x6e, 0x6f, 0x44, 0x61,
0x72, 0x76, 0x69, 0x6e, 0x6f, 0x48, 0x6f, 0x62, 0x61, 0x72, 0x74, 0x6f,
0x4d, 0x65, 0x6c, 0x62, 0x75, 0x72, 0x6e, 0x6f, 0x50, 0x65, 0x72, 0x74,
0x6f, 0x53, 0x69, 0x64, 0x6e, 0x65, 0x6a, 0x6f, 0x41, 0x6d, 0x73, 0x74,
0x65, 0x72, 0x64, 0x61, 0x6d, 0x6f, 0x41, 0x6e, 0x64, 0x6f, 0x72, 0x6f,
0x41, 0x73, 0x74, 0x72, 0x61, 0x125, 0x61, 0x6e, 0x6f, 0x41, 0x74, 0x65,
0x6e, 0x6f, 0x42, 0x65, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x42, 0x65,
0x72, 0x6c, 0x69, 0x6e, 0x6f, 0x42, 0x72, 0x61, 0x74, 0x69, 0x73, 0x6c,
0x61, 0x76, 0x6f, 0x42, 0x72, 0x75, 0x73, 0x65, 0x6c, 0x6f, 0x42, 0x75,
0x6b, 0x61, 0x72, 0x65, 0x15d, 0x74, 0x6f, 0x42, 0x75, 0x64, 0x61, 0x70,
0x65, 0x15d, 0x74, 0x6f, 0x42, 0xfc, 0x73, 0x69, 0x6e, 0x67, 0x65, 0x6e,
0x20, 0x61, 0x6d, 0x20, 0x48, 0x6f, 0x63, 0x68, 0x72, 0x68, 0x65, 0x69,
0x6e, 0x4b, 0x69, 0x15d, 0x69, 0x6e, 0x65, 0x76, 0x6f, 0x4b, 0x6f, 0x70,
0x65, 0x6e, 0x68, 0x61, 0x67, 0x6f, 0x44, 0x75, 0x62, 0x6c, 0x69, 0x6e,
0x6f, 0x11c, 0x69, 0x62, 0x72, 0x61, 0x6c, 0x74, 0x61, 0x72, 0x6f, 0x47,
0x65, 0x72, 0x6e, 0x65, 0x7a, 0x65, 0x6a, 0x6f, 0x48, 0x65, 0x6c, 0x73,
0x69, 0x6e, 0x6b, 0x6f, 0x4d, 0x61, 0x6e, 0x6b, 0x69, 0x6e, 0x73, 0x75,
0x6c, 0x6f, 0x49, 0x73, 0x74, 0x61, 0x6e, 0x62, 0x75, 0x6c, 0x6f, 0x134,
0x65, 0x72, 0x7a, 0x65, 0x6a, 0x6f, 0x4b, 0x61, 0x6c, 0x69, 0x6e, 0x69,
0x6e, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x4b, 0x69, 0x65, 0x76, 0x6f, 0x4c,
0x69, 0x73, 0x62, 0x6f, 0x6e, 0x6f, 0x4c, 0x6a, 0x75, 0x62, 0x6c, 0x6a,
0x61, 0x6e, 0x6f, 0x4c, 0x6f, 0x6e, 0x64, 0x6f, 0x6e, 0x6f, 0x4c, 0x75,
0x6b, 0x73, 0x65, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x4d, 0x61, 0x64,
0x72, 0x69, 0x64, 0x6f, 0x4d, 0x61, 0x6c, 0x74, 0x6f, 0x4d, 0x69, 0x6e,
0x73, 0x6b, 0x6f, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x6f, 0x50, 0x61, 0x72,
0x69, 0x7a, 0x6f, 0x50, 0x6f, 0x64, 0x67, 0x6f, 0x72, 0x69, 0x63, 0x6f,
0x50, 0x72, 0x61, 0x67, 0x6f, 0x52, 0x69, 0x67, 0x6f, 0x52, 0x6f, 0x6d,
0x6f, 0x53, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x69, 0x6e, 0x6f, 0x53, 0x69,
0x6d, 0x66, 0x65, 0x72, 0x6f, 0x70, 0x6f, 0x6c, 0x6f, 0x53, 0x6b, 0x6f,
0x70, 0x6a, 0x6f, 0x53, 0x6f, 0x66, 0x69, 0x6f, 0x53, 0x74, 0x6f, 0x6b,
0x68, 0x6f, 0x6c, 0x6d, 0x6f, 0x54, 0x61, 0x6c, 0x69, 0x6e, 0x6f, 0x54,
0x69, 0x72, 0x61, 0x6e, 0x6f, 0x55, 0x6c, 0x6a, 0x61, 0x6e, 0x6f, 0x76,
0x73, 0x6b, 0x6f, 0x56, 0x61, 0x64, 0x75, 0x7a, 0x6f, 0x56, 0x69, 0x65,
0x6e, 0x6f, 0x56, 0x69, 0x6c, 0x6e, 0x6f, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x6f, 0x56, 0x61, 0x72, 0x73, 0x6f, 0x76, 0x69,
0x6f, 0x5a, 0x61, 0x67, 0x72, 0x65, 0x62, 0x6f, 0x5a, 0x75, 0x72, 0x69,
0x6b, 0x6f, 0x108, 0x61, 0x67, 0x6f, 0x73, 0x6f, 0x6a, 0x4b, 0x72, 0x69,
0x73, 0x74, 0x6e, 0x61, 0x73, 0x6b, 0x69, 0x6e, 0x73, 0x75, 0x6c, 0x6f,
0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x69, 0x6e, 0x73, 0x75, 0x6c, 0x6f, 0x6a,
0x4b, 0x6f, 0x6d, 0x6f, 0x72, 0x6f, 0x6a, 0x4b, 0x65, 0x72, 0x67, 0x65,
0x6c, 0x65, 0x6e, 0x6f, 0x6a, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x6f,
0x6a, 0x4d, 0x61, 0x16d, 0x72, 0x69, 0x63, 0x69, 0x6f, 0x4d, 0x61, 0x6a,
0x6f, 0x74, 0x6f, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x41, 0x70, 0x69,
0x6f, 0x41, 0x16d, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x6f, 0x108, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x6f, 0x108, 0x75, 0x75, 0x6b, 0x50, 0x61, 0x73, 0x6b,
0x69, 0x6e, 0x73, 0x75, 0x6c, 0x6f, 0x46, 0x69, 0x11d, 0x69, 0x6f, 0x46,
0x75, 0x6e, 0x61, 0x66, 0x75, 0x74, 0x69, 0x6f, 0x47, 0x61, 0x6c, 0x61,
0x70, 0x61, 0x67, 0x6f, 0x6a, 0x47, 0x76, 0x61, 0x6d, 0x6f, 0x4d, 0x61,
0x72, 0x6b, 0x69, 0x7a, 0x69, 0x6e, 0x73, 0x75, 0x6c, 0x6f, 0x6a, 0x4d,
0x69, 0x64, 0x76, 0x65, 0x6a, 0x69, 0x6e, 0x73, 0x75, 0x6c, 0x6f, 0x6a,
0x4e, 0x61, 0x75, 0x72, 0x6f, 0x4e, 0x69, 0x75, 0x6f, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x69, 0x6e, 0x73, 0x75, 0x6c, 0x6f, 0x4e, 0x75,
0x6d, 0x65, 0x6f, 0x50, 0x61, 0x67, 0x6f, 0x70, 0x61, 0x67, 0x6f, 0x50,
0x61, 0x6c, 0x61, 0x16d, 0x6f, 0x50, 0x69, 0x74, 0x6b, 0x61, 0x72, 0x6e,
0x61, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x6f, 0x52, 0x61, 0x72, 0x6f,
0x74, 0x6f, 0x6e, 0x67, 0x6f, 0x53, 0x61, 0x69, 0x70, 0x61, 0x6e, 0x6f,
0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x6f, 0x54, 0x61, 0x72, 0x61, 0x76,
0x6f, 0x56, 0x65, 0x6a, 0x6b, 0x69, 0x6e, 0x73, 0x75, 0x6c, 0x6f, 0x56,
0x61, 0x6c, 0x69, 0x73, 0x6f, 0x41, 0x6c, 0x17e, 0x69, 0x69, 0x72, 0x45,
0x6c, 0x20, 0x41, 0x61, 0x69, 0xfa, 0x6e, 0x48, 0x61, 0x72, 0x74, 0x75,
0x6d, 0x48, 0x61, 0x76, 0x61, 0x6e, 0x6e, 0x61, 0x42, 0x65, 0x75, 0x6c,
0x61, 0x68, 0x2c, 0x20, 0x50, 0xf5, 0x68, 0x6a, 0x61, 0x2d, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20,
0x50, 0xf5, 0x68, 0x6a, 0x61, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x50,
0xf5, 0x68, 0x6a, 0x61, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x41,
0x6c, 0x6d, 0x61, 0x74, 0xf5, 0x41, 0x6e, 0x61, 0x64, 0xf5, 0x72, 0x41,
0x6b, 0x74, 0xf6, 0x62, 0x65, 0x41, 0x74, 0xf5, 0x72, 0x61, 0x75, 0x42,
0x61, 0x6b, 0x75, 0x75, 0x54, 0x161, 0x69, 0x74, 0x61, 0x48, 0xf4, 0x20,
0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x4a, 0x65, 0x72, 0x75,
0x75, 0x73, 0x61, 0x6c, 0x65, 0x6d, 0x6d, 0x4b, 0x61, 0x6d, 0x74, 0x161,
0x61, 0x74, 0x6b, 0x61, 0x48, 0x61, 0x6e, 0x64, 0xf5, 0x67, 0x61, 0x4b,
0x75, 0x76, 0x65, 0x69, 0x74, 0x4d, 0x61, 0x73, 0x71, 0x61, 0x163, 0x4b,
0xf5, 0x7a, 0xf5, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x41, 0x72, 0x2d, 0x52,
0x69, 0x79, 0x101, 0x1e11, 0x53, 0x61, 0x68, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x53, 0x72, 0x65, 0x64, 0x6e, 0x65, 0x6b, 0x6f, 0x6c, 0xf5, 0x6d, 0x73,
0x6b, 0x54, 0x68, 0x62, 0x69, 0x6c, 0x69, 0x73, 0x69, 0x54, 0x14d, 0x6b,
0x79, 0x14d, 0x41, 0x73, 0x73, 0x6f, 0x6f, 0x72, 0x69, 0x64, 0x4b, 0x61,
0x6e, 0x61, 0x61, 0x72, 0x69, 0x20, 0x73, 0x61, 0x61, 0x72, 0x65, 0x64,
0x52, 0x6f, 0x68, 0x65, 0x6e, 0x65, 0x65, 0x6d, 0x65, 0x73, 0x61, 0x61,
0x72, 0x65, 0x64, 0x46, 0xe4, 0xe4, 0x72, 0x69, 0x20, 0x73, 0x61, 0x61,
0x72, 0x65, 0x64, 0x4c, 0xf5, 0x75, 0x6e, 0x61, 0x2d, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x61, 0x41, 0x74, 0x65, 0x65, 0x6e, 0x61, 0x42, 0x65,
0x72, 0x6c, 0x69, 0x69, 0x6e, 0x43, 0x68, 0x69, 0x219, 0x69, 0x6e, 0x103,
0x75, 0x4b, 0x6f, 0x70, 0x65, 0x6e, 0x68, 0x61, 0x61, 0x67, 0x65, 0x6e,
0x48, 0x65, 0x6c, 0x73, 0x69, 0x6e, 0x67, 0x69, 0x4d, 0x61, 0x6e, 0x69,
0x20, 0x73, 0x61, 0x61, 0x72, 0x4b, 0x69, 0x69, 0x65, 0x76, 0x4d, 0x61,
0x61, 0x72, 0x69, 0x61, 0x6e, 0x68, 0x61, 0x6d, 0x69, 0x6e, 0x61, 0x50,
0x61, 0x72, 0x69, 0x69, 0x73, 0x52, 0x69, 0x69, 0x61, 0x52, 0x6f, 0x6f,
0x6d, 0x61, 0x56, 0x69, 0x69, 0x6e, 0x56, 0x61, 0x72, 0x73, 0x73, 0x61,
0x76, 0x69, 0x4a, 0xf5, 0x75, 0x6c, 0x75, 0x73, 0x61, 0x61, 0x72, 0x4b,
0x6f, 0x6f, 0x6b, 0x6f, 0x73, 0x73, 0x61, 0x61, 0x72, 0x65, 0x64, 0x4d,
0x61, 0x6c, 0x64, 0x69, 0x69, 0x76, 0x69, 0x64, 0x4c, 0x69, 0x68, 0x61,
0x76, 0xf5, 0x74, 0x74, 0x65, 0x73, 0x61, 0x61, 0x72, 0x4d, 0x61, 0x72,
0x6b, 0x69, 0x69, 0x73, 0x61, 0x61, 0x72, 0x65, 0x64, 0x42, 0x65, 0x6c,
0x61, 0x75, 0x4b, 0x61, 0x74, 0x68, 0x6d, 0x61, 0x6e, 0x64, 0x75, 0x20,
0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x6d, 0x65, 0x42, 0x65, 0x6c, 0x69, 0x73, 0x42, 0x61, 0x72, 0x65, 0x69,
0x6e, 0x47, 0x61, 0x73, 0x61, 0x4b, 0x75, 0x76, 0x61, 0x69, 0x74, 0x41,
0x7a, 0x6f, 0x72, 0x75, 0x72, 0x6e, 0x61, 0x72, 0x47, 0x72, 0xf8, 0x6e,
0x68, 0xf8, 0x76, 0x64, 0x61, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72,
0x46, 0xf8, 0x72, 0x6f, 0x79, 0x61, 0x72, 0x53, 0x75, 0xf0, 0x75, 0x72,
0x67, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x6f, 0x79, 0x67, 0x67, 0x6a,
0x61, 0x72, 0x4b, 0x65, 0x79, 0x70, 0x6d, 0x61, 0x6e, 0x6e, 0x61, 0x68,
0x61, 0x76, 0x6e, 0x4c, 0x75, 0x6b, 0x73, 0x65, 0x6d, 0x62, 0x6f, 0x72,
0x67, 0x53, 0x74, 0x6f, 0x6b, 0x6b, 0x68, 0xf3, 0x6c, 0x6d, 0x56, 0x61,
0x74, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0xf0, 0x56, 0x61, 0x72, 0x73, 0x6a,
0x61, 0x76, 0x61, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x6f, 0x79, 0x67,
0x67, 0x6a, 0x61, 0x72, 0x4d, 0xf3, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73,
0x4c, 0x75, 0x6e, 0x67, 0x73, 0x6f, 0x64, 0x20, 0x6e, 0x67, 0x20, 0x4a,
0x75, 0xe1, 0x72, 0x65, 0x7a, 0x4c, 0x75, 0x6e, 0x67, 0x73, 0x6f, 0x64,
0x20, 0x6e, 0x67, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f, 0x50, 0x75,
0x77, 0x65, 0x72, 0x74, 0x6f, 0x20, 0x6e, 0x67, 0x20, 0x45, 0x73, 0x70,
0x61, 0x6e, 0x79, 0x61, 0x4d, 0x61, 0x6b, 0x69, 0x70, 0x6f, 0x74, 0x20,
0x6e, 0x61, 0x20, 0x4c, 0x6f, 0x6f, 0x6b, 0x20, 0x6e, 0x67, 0x20, 0x52,
0x61, 0x6e, 0x6b, 0x69, 0x6e, 0x4c, 0x75, 0x6e, 0x67, 0x73, 0x6f, 0x64,
0x20, 0x6e, 0x67, 0x20, 0x48, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d,
0x69, 0x6e, 0x68, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x50,
0x6f, 0x68, 0x6a, 0x6f, 0x69, 0x73, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74,
0x61, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x50, 0x6f, 0x68,
0x6a, 0x6f, 0x69, 0x73, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e,
0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x50, 0x6f,
0x68, 0x6a, 0x6f, 0x69, 0x73, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x53, 0x61, 0x6e, 0x74, 0x69, 0x61, 0x67, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x43, 0x68, 0x69, 0x6c, 0x65, 0x4d, 0x61, 0x63, 0x71, 0x75, 0x61, 0x72,
0x69, 0x65, 0x6e, 0x73, 0x61, 0x61, 0x72, 0x69, 0x41, 0x71, 0x74, 0x61,
0x77, 0x44, 0x61, 0x6d, 0x61, 0x73, 0x6b, 0x6f, 0x73, 0x48, 0x1ed3, 0x20,
0x43, 0x68, 0xed, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x69, 0x6e, 0x20, 0x6b,
0x61, 0x75, 0x70, 0x75, 0x6e, 0x6b, 0x69, 0x48, 0x61, 0x6e, 0x64, 0x79,
0x67, 0x61, 0x4b, 0x61, 0x6c, 0x6b, 0x75, 0x74, 0x74, 0x61, 0x4b, 0x6f,
0x73, 0x74, 0x61, 0x6e, 0x61, 0x69, 0x41, 0x7a, 0x6f, 0x72, 0x69, 0x74,
0x4b, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x61, 0x6e, 0x73, 0x61, 0x61, 0x72,
0x65, 0x74, 0x46, 0xe4, 0x72, 0x73, 0x61, 0x61, 0x72, 0x65, 0x74, 0x45,
0x74, 0x65, 0x6c, 0xe4, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61,
0x42, 0x65, 0x72, 0x6c, 0x69, 0x69, 0x6e, 0x69, 0x42, 0x72, 0x79, 0x73,
0x73, 0x65, 0x6c, 0x43, 0x68, 0x69, 0x15f, 0x69, 0x6e, 0x103, 0x75, 0x4b,
0xf6, 0xf6, 0x70, 0x65, 0x6e, 0x68, 0x61, 0x6d, 0x69, 0x6e, 0x61, 0x4d,
0x61, 0x6e, 0x73, 0x61, 0x61, 0x72, 0x69, 0x4b, 0x69, 0x6f, 0x76, 0x61,
0x4c, 0x6f, 0x6e, 0x74, 0x6f, 0x6f, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76,
0x61, 0x50, 0x61, 0x72, 0x69, 0x69, 0x73, 0x69, 0x52, 0x69, 0x69, 0x6b,
0x61, 0x54, 0x75, 0x6b, 0x68, 0x6f, 0x6c, 0x6d, 0x61, 0x54, 0x61, 0x6c,
0x6c, 0x69, 0x6e, 0x6e, 0x61, 0x56, 0x61, 0x74, 0x69, 0x6b, 0x61, 0x61,
0x6e, 0x69, 0x56, 0x69, 0x6c, 0x6e, 0x61, 0x56, 0x61, 0x72, 0x73, 0x6f,
0x76, 0x61, 0x4a, 0x6f, 0x75, 0x6c, 0x75, 0x73, 0x61, 0x61, 0x72, 0x69,
0x4b, 0x6f, 0x6f, 0x6b, 0x6f, 0x73, 0x73, 0x61, 0x61, 0x72, 0x65, 0x74,
0x4b, 0x6f, 0x6d, 0x6f, 0x72, 0x69, 0x74, 0x4b, 0x65, 0x72, 0x67, 0x75,
0x65, 0x6c, 0x65, 0x6e, 0x73, 0x61, 0x61, 0x72, 0x65, 0x74, 0x4d, 0x61,
0x6c, 0x65, 0x64, 0x69, 0x69, 0x76, 0x69, 0x74, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x73, 0x61, 0x61, 0x72, 0x65, 0x74, 0x50, 0xe4, 0xe4,
0x73, 0x69, 0xe4, 0x69, 0x73, 0x73, 0x61, 0x61, 0x72, 0x69, 0x47, 0x61,
0x6d, 0x62, 0x69, 0x65, 0x72, 0x73, 0x61, 0x61, 0x72, 0x65, 0x74, 0x4d,
0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x73, 0x61, 0x61, 0x72,
0x65, 0x74, 0x4d, 0x69, 0x64, 0x77, 0x61, 0x79, 0x73, 0x61, 0x61, 0x72,
0x65, 0x74, 0x41, 0x64, 0x64, 0x69, 0x73, 0x2d, 0x41, 0x62, 0x65, 0x62,
0x61, 0x4c, 0x65, 0x20, 0x43, 0x61, 0x69, 0x72, 0x65, 0x4c, 0x61, 0xe2,
0x79, 0x6f, 0x75, 0x6e, 0x65, 0x54, 0x72, 0x69, 0x70, 0x6f, 0x6c, 0x69,
0x20, 0x28, 0x4c, 0x69, 0x62, 0x79, 0x65, 0x29, 0x55, 0x73, 0x68, 0x75,
0x61, 0xef, 0x61, 0x42, 0x61, 0x68, 0x69, 0x61, 0x20, 0x64, 0x65, 0x20,
0x42, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x61, 0x73, 0x4c, 0x61, 0x20, 0x42,
0x61, 0x72, 0x62, 0x61, 0x64, 0x65, 0x43, 0x61, 0xef, 0x6d, 0x61, 0x6e,
0x73, 0x44, 0xe9, 0x74, 0x72, 0x6f, 0x69, 0x74, 0x44, 0x6f, 0x6d, 0x69,
0x6e, 0x69, 0x71, 0x75, 0x65, 0x47, 0x72, 0x65, 0x6e, 0x61, 0x64, 0x65,
0x4c, 0x61, 0x20, 0x48, 0x61, 0x76, 0x61, 0x6e, 0x65, 0x4b, 0x6e, 0x6f,
0x78, 0x20, 0x5b, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x5d, 0x4d,
0x61, 0x72, 0x65, 0x6e, 0x67, 0x6f, 0x20, 0x5b, 0x49, 0x6e, 0x64, 0x69,
0x61, 0x6e, 0x61, 0x5d, 0x50, 0x65, 0x74, 0x65, 0x72, 0x73, 0x62, 0x75,
0x72, 0x67, 0x20, 0x5b, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x5d,
0x54, 0x65, 0x6c, 0x6c, 0x20, 0x43, 0x69, 0x74, 0x79, 0x20, 0x5b, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x5d, 0x56, 0x65, 0x76, 0x61, 0x79,
0x20, 0x5b, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x5d, 0x56, 0x69,
0x6e, 0x63, 0x65, 0x6e, 0x6e, 0x65, 0x73, 0x20, 0x5b, 0x49, 0x6e, 0x64,
0x69, 0x61, 0x6e, 0x61, 0x5d, 0x57, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x63,
0x20, 0x5b, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x5d, 0x4a, 0x61,
0x6d, 0x61, 0xef, 0x71, 0x75, 0x65, 0x4d, 0x6f, 0x6e, 0x74, 0x69, 0x63,
0x65, 0x6c, 0x6c, 0x6f, 0x20, 0x5b, 0x4b, 0x65, 0x6e, 0x74, 0x75, 0x63,
0x6b, 0x79, 0x5d, 0x4d, 0x61, 0x6e, 0x61, 0x6f, 0x73, 0x42, 0x65, 0x75,
0x6c, 0x61, 0x68, 0x20, 0x28, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20,
0x64, 0x75, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x29, 0x43, 0x65, 0x6e, 0x74,
0x65, 0x72, 0x20, 0x28, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64,
0x75, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x29, 0x4e, 0x65, 0x77, 0x20, 0x53,
0x61, 0x6c, 0x65, 0x6d, 0x20, 0x28, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x20, 0x64, 0x75, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x29, 0x50, 0x6f, 0x72,
0x74, 0x2d, 0x64, 0x2019, 0x45, 0x73, 0x70, 0x61, 0x67, 0x6e, 0x65, 0x50,
0x6f, 0x72, 0x74, 0x6f, 0x20, 0x52, 0x69, 0x63, 0x6f, 0x53, 0x61, 0x69,
0x6e, 0x74, 0x2d, 0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x67, 0x75, 0x65, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x2d, 0x4a, 0x65, 0x61, 0x6e, 0x20, 0x64, 0x65,
0x20, 0x54, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x4e, 0x65, 0x75, 0x76, 0x65,
0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74,
0x6f, 0x70, 0x68, 0x65, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x65, 0x2d, 0x4c,
0x75, 0x63, 0x69, 0x65, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x56, 0x69,
0x6e, 0x63, 0x65, 0x6e, 0x74, 0x54, 0x68, 0x75, 0x6c, 0xe9, 0x41, 0x6b,
0x74, 0x61, 0x6f, 0x75, 0x41, 0x6b, 0x74, 0x69, 0x6f, 0x75, 0x62, 0x69,
0x6e, 0x73, 0x6b, 0x41, 0x63, 0x68, 0x67, 0x61, 0x62, 0x61, 0x74, 0x41,
0x74, 0x79, 0x72, 0x61, 0x6f, 0x75, 0x42, 0x61, 0x68, 0x72, 0x65, 0xef,
0x6e, 0x42, 0x65, 0x79, 0x72, 0x6f, 0x75, 0x74, 0x68, 0x42, 0x69, 0x63,
0x68, 0x6b, 0x65, 0x6b, 0x44, 0x75, 0x62, 0x61, 0xef, 0x44, 0x6f, 0x75,
0x63, 0x68, 0x61, 0x6e, 0x62, 0xe9, 0x46, 0x61, 0x6d, 0x61, 0x67, 0x6f,
0x75, 0x73, 0x74, 0x65, 0x48, 0xe9, 0x62, 0x72, 0x6f, 0x6e, 0x48, 0xf4,
0x2d, 0x43, 0x68, 0x69, 0x2d, 0x4d, 0x69, 0x6e, 0x68, 0x2d, 0x56, 0x69,
0x6c, 0x6c, 0x65, 0x49, 0x72, 0x6b, 0x6f, 0x75, 0x74, 0x73, 0x6b, 0x4a,
0xe9, 0x72, 0x75, 0x73, 0x61, 0x6c, 0x65, 0x6d, 0x4b, 0x61, 0x74, 0x6d,
0x61, 0x6e, 0x64, 0x6f, 0x75, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0xef,
0x61, 0x72, 0x73, 0x6b, 0x4b, 0x6f, 0x77, 0x65, 0xef, 0x74, 0x4d, 0x61,
0x63, 0x61, 0x73, 0x73, 0x61, 0x72, 0x4d, 0x61, 0x6e, 0x69, 0x6c, 0x6c,
0x65, 0x4e, 0x69, 0x63, 0x6f, 0x73, 0x69, 0x65, 0x4f, 0x75, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x6e, 0x61, 0xef, 0x4b,
0x7a, 0x79, 0x6c, 0x20, 0x4f, 0x72, 0x64, 0x61, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x65, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x63, 0x61,
0x6e, 0x64, 0x65, 0x53, 0xe9, 0x6f, 0x75, 0x6c, 0x54, 0x61, 0x63, 0x68,
0x6b, 0x65, 0x6e, 0x74, 0x54, 0x62, 0x69, 0x6c, 0x69, 0x73, 0x73, 0x69,
0x54, 0xe9, 0x68, 0xe9, 0x72, 0x61, 0x6e, 0x4f, 0x75, 0x6c, 0x61, 0x6e,
0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x49, 0x61, 0x6b, 0x6f, 0x75, 0x74,
0x73, 0x6b, 0x52, 0x61, 0x6e, 0x67, 0x6f, 0x75, 0x6e, 0x45, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x6f, 0x75, 0x72, 0x67, 0xce, 0x6c,
0x65, 0x73, 0x20, 0x43, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x65, 0x73, 0x43,
0x61, 0x70, 0x2d, 0x56, 0x65, 0x72, 0x74, 0xce, 0x6c, 0x65, 0x73, 0x20,
0x46, 0xe9, 0x72, 0x6f, 0xe9, 0x4d, 0x61, 0x64, 0xe8, 0x72, 0x65, 0x47,
0xe9, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x20, 0x64, 0x75, 0x20, 0x53, 0x75,
0x64, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x65, 0x2d, 0x48, 0xe9, 0x6c, 0xe8,
0x6e, 0x65, 0x41, 0x64, 0xe9, 0x6c, 0x61, 0xef, 0x64, 0x65, 0x41, 0x6e,
0x64, 0x6f, 0x72, 0x72, 0x65, 0x41, 0x74, 0x68, 0xe8, 0x6e, 0x65, 0x73,
0xce, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x6e, 0x4c, 0x69,
0x73, 0x62, 0x6f, 0x6e, 0x6e, 0x65, 0x4d, 0x61, 0x6c, 0x74, 0x65, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x2d, 0x4d, 0x61, 0x72, 0x69, 0x6e, 0x4f, 0x75,
0x6c, 0x69, 0x61, 0x6e, 0x6f, 0x76, 0x73, 0x6b, 0x4c, 0x65, 0x20, 0x56,
0x61, 0x74, 0x69, 0x63, 0x61, 0x6e, 0x56, 0x69, 0x65, 0x6e, 0x6e, 0x65,
0x56, 0x61, 0x72, 0x73, 0x6f, 0x76, 0x69, 0x65, 0x43, 0x6f, 0x6d, 0x6f,
0x72, 0x65, 0x73, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x65, 0x4c, 0x61,
0x20, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0xce, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x50, 0xe2, 0x71, 0x75, 0x65, 0x73, 0x4d, 0x61, 0x72,
0x71, 0x75, 0x69, 0x73, 0x65, 0x73, 0x50, 0x61, 0x6c, 0x61, 0x6f, 0x73,
0x4e, 0x64, 0x6a, 0x61, 0x6d, 0x65, 0x6e, 0x61, 0x54, 0x72, 0x69, 0x70,
0x6f, 0x6c, 0x69, 0x20, 0x5b, 0x4c, 0x69, 0x62, 0x79, 0x65, 0x5d, 0x42,
0x61, 0x72, 0x62, 0x61, 0x64, 0x65, 0x20, 0x28, 0x4c, 0x61, 0x29, 0xee,
0x6c, 0x65, 0x73, 0x20, 0x43, 0x61, 0xef, 0x6d, 0x61, 0x6e, 0x73, 0x42,
0x65, 0x75, 0x6c, 0x61, 0x68, 0x20, 0x5b, 0x44, 0x61, 0x6b, 0x6f, 0x74,
0x61, 0x20, 0x64, 0x75, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x5d, 0x43, 0x65,
0x6e, 0x74, 0x65, 0x72, 0x20, 0x5b, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x20, 0x64, 0x75, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x5d, 0x4e, 0x65, 0x77,
0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x20, 0x5b, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x20, 0x64, 0x75, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x5d, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x2d, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6f,
0x70, 0x68, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4e, 0x69, 0xe9, 0x76, 0xe8,
0x73, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x20, 0x64, 0x2019, 0x55, 0x72,
0x76, 0x69, 0x6c, 0x6c, 0x65, 0x44, 0x61, 0x63, 0x63, 0x61, 0x54, 0x68,
0x69, 0x6d, 0x70, 0x68, 0x6f, 0x75, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x43,
0x61, 0x6e, 0x61, 0x72, 0x69, 0x65, 0x73, 0xee, 0x6c, 0x65, 0x73, 0x20,
0x46, 0xe9, 0x72, 0x6f, 0xe9, 0xee, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x61, 0x6e, 0xee, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x50, 0xe2,
0x71, 0x75, 0x65, 0x73, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x47, 0x6e, 0x6f,
0x76, 0x65, 0x20, 0x59, 0x6f, 0x72, 0x6b, 0x53, 0x61, 0x6e, 0x20, 0x50,
0x61, 0x75, 0x6c, 0x69, 0x20, 0x64, 0x61, 0x6c, 0x20, 0x42, 0x72, 0x61,
0x73, 0xee, 0x6c, 0x41, 0x7a, 0x6f, 0x72, 0x69, 0x73, 0x43, 0x61, 0x6e,
0x61, 0x72, 0x69, 0x69, 0x73, 0x42, 0x65, 0x6c, 0x67, 0x72, 0xe2, 0x74,
0x4c, 0x69, 0x73, 0x62, 0x6f, 0x6e, 0x65, 0x4c, 0x75, 0x62, 0x69, 0x61,
0x6e, 0x65, 0x4c, 0x75, 0x73, 0x73, 0x65, 0x6d, 0x62, 0x75, 0x72, 0x63,
0x4d, 0x6f, 0x73, 0x63, 0x68, 0x65, 0x50, 0x72, 0x61, 0x67, 0x68, 0x65,
0x53, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6e, 0x56, 0x69, 0x65,
0x6e, 0x65, 0xd83a, 0xdd00, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd00, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22,
0xd83a, 0xdd00, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd22, 0xd83a, 0xdd26,
0xd83a, 0xdd22, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd00, 0xd83a, 0xdd24, 0xd83a, 0xdd36,
0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd00, 0xd83a, 0xdd27,
0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd04, 0xd83a, 0xdd22,
0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd04,
0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2d, 0xd83a, 0xdd04, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd36, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd04, 0xd83a, 0xdd2d,
0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2e, 0xd83a, 0xdd04, 0xd83a, 0xdd2d,
0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd41, 0xd83a, 0xdd22,
0xd83a, 0xdd3e, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd04, 0xd83a, 0xdd35, 0xd83a, 0xdd36,
0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd22,
0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd11,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd27, 0xd83a, 0xdd22, 0xd83a, 0xdd26, 0xd83a, 0xdd35,
0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd11,
0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd33, 0xd83a, 0xdd2d,
0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd01, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd2a, 0xd83a, 0xdd01, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x2d, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd25, 0xd83a, 0xdd35, 0xd83a, 0xdd14, 0xd83a, 0xdd2d, 0xd83a, 0xdd26, 0xd83a,
0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd01, 0xd83a, 0xdd35, 0xd83a,
0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd02, 0xd83a, 0xdd22, 0xd83a,
0xdd34, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd0a, 0xd83a,
0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd18, 0xd83a,
0xdd22, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd32, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0xd83a, 0xdd14, 0xd83a, 0xdd2e, 0xd83a, 0xdd38, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd14, 0xd83a, 0xdd35, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a,
0xdd25, 0xd83a, 0xdd28, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd1d, 0xd83a,
0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd25, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd27, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a,
0xdd22, 0xd83a, 0xdd02, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd27, 0xd83a, 0xdd02, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd26, 0xd83a, 0xdd2b, 0xd83a,
0xdd2a, 0xd83a, 0xdd3e, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd24, 0xd83a, 0xdd02, 0xd83a,
0xdd2e, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd02, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a,
0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd02, 0xd83a,
0xdd35, 0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a,
0xdd43, 0xd83a, 0xdd2d, 0xd83a, 0xdd02, 0xd83a, 0xdd35, 0xd83a, 0xdd27, 0xd83a, 0xdd22, 0xd83a,
0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a,
0xdd26, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd28, 0xd83a,
0xdd35, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a,
0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd10, 0xd83a, 0xdd4b, 0xd83a, 0xdd04, 0xd83a,
0xdd22, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd2b, 0xd83a,
0xdd03, 0xd83a, 0xdd35, 0xd83a, 0xdd39, 0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd03, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a,
0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd10, 0xd83a, 0xdd22, 0xd83a,
0xdd34, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd10, 0xd83a,
0xdd4b, 0xd83a, 0xdd14, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a,
0xdd22, 0xd83a, 0xdd10, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a,
0xdd2b, 0xd83a, 0xdd10, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd33, 0xd83a, 0xdd43, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd3c, 0xd83a, 0xdd0f, 0xd83a,
0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd34, 0xd83a, 0xdd3a, 0xd83a,
0xdd35, 0xd83a, 0xdd06, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0x2d,
0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd3e, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd05,
0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2e, 0x2d, 0xd83a, 0xdd1a, 0xd83a, 0xdd2e, 0xd83a,
0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd1a, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a,
0xdd28, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a,
0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a, 0xdd27, 0xd83a, 0xdd0f, 0xd83a, 0xdd2d, 0xd83a,
0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd00, 0xd83a,
0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd00, 0xd83a, 0xdd32, 0xd83a, 0xdd27, 0xd83a,
0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd36, 0xd83a, 0xdd35, 0xd83a,
0xdd00, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd00, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd3a, 0xd83a,
0xdd22, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd35, 0xd83a,
0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd04, 0xd83a, 0xdd2d, 0xd83a,
0xdd34, 0xd83a, 0xdd32, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0x2d, 0xd83a, 0xdd09, 0xd83a, 0xdd45,
0xd83a, 0xdd36, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd27,
0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd22,
0xd83a, 0xdd2a, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2e,
0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd14, 0xd83a, 0xdd35, 0xd83a, 0xdd36, 0xd83a, 0xdd35, 0xd83a, 0xdd34, 0xd83a, 0xdd02,
0xd83a, 0xdd22, 0x2d, 0xd83a, 0xdd08, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd2e, 0xd83a,
0xdd38, 0xd83a, 0xdd22, 0xd83a, 0xdd03, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd41, 0xd83a, 0xdd22, 0xd83a, 0xdd08, 0xd83a, 0xdd2d, 0xd83a,
0xdd2e, 0x2d, 0xd83a, 0xdd18, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd3a,
0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd3c,
0xd83a, 0xdd22, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0x2d, 0xd83a, 0xdd1d, 0xd83a,
0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd05, 0xd83a,
0xdd22, 0xd83a, 0xdd32, 0x2d, 0xd83a, 0xdd02, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd2d,
0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd33, 0xd83a, 0xdd35,
0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd13, 0xd83a, 0xdd27,
0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd36, 0xd83a, 0xdd00,
0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd00,
0xd83a, 0xdd27, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd34,
0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd00, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd33,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd04,
0xd83a, 0xdd22, 0xd83a, 0xdd38, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd04,
0xd83a, 0xdd22, 0xd83a, 0xdd38, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0x20, 0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a,
0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd27, 0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a,
0xdd2a, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a,
0xdd27, 0xd83a, 0xdd04, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd25, 0xd83a,
0xdd04, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd41, 0xd83a,
0xdd04, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0x20, 0xd83a, 0xdd05,
0xd83a, 0xdd22, 0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd32,
0xd83a, 0xdd04, 0xd83a, 0xdd2e, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0x2d, 0xd83a, 0xdd1c, 0xd83a,
0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd04, 0xd83a, 0xdd2e, 0xd83a,
0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd04, 0xd83a, 0xdd2e, 0xd83a,
0xdd34, 0xd83a, 0xdd41, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a,
0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a,
0xdd36, 0x20, 0xd83a, 0xdd04, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd22,
0xd83a, 0xdd25, 0xd83a, 0xdd28, 0xd83a, 0xdd2e, 0x2d, 0xd83a, 0xdd18, 0xd83a, 0xdd2a, 0xd83a,
0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a,
0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a,
0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a,
0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd32, 0xd83a, 0xdd15, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd15, 0xd83a, 0xdd0b, 0xd83a, 0xdd31, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a,
0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0x20, 0xd83a, 0xdd08, 0xd83a, 0xdd2d,
0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd35,
0xd83a, 0xdd36, 0xd83a, 0xdd22, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11,
0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd22, 0xd83a, 0xdd31,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd01, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32,
0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd33, 0xd83a, 0xdd43, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd3e, 0xd83a, 0xdd32, 0xd83a, 0xdd01, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd27, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd01, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd27, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0x2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0xd83a, 0xdd33, 0xd83a, 0xdd01, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd3e, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd01, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a, 0xdd2a, 0xd83a,
0xdd2e, 0xd83a, 0xdd34, 0xd83a, 0xdd3c, 0xd83a, 0xdd01, 0xd83a, 0xdd2e, 0xd83a, 0xdd25, 0xd83a,
0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd09, 0xd83a, 0xdd23, 0xd83a, 0xdd25, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a,
0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd09, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a,
0xdd2b, 0xd83a, 0xdd28, 0xd83a, 0xdd2b, 0xd83a, 0xdd09, 0xd83a, 0xdd24, 0x2d, 0xd83a, 0xdd05,
0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd3e, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd0a, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd3c,
0x2d, 0xd83a, 0xdd10, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd27, 0xd83a, 0xdd2e, 0xd83a,
0xdd32, 0x3b, 0xd83a, 0xdd0a, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd3c, 0xd83a, 0xdd22,
0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd41, 0xd83a, 0xdd22, 0xd83a, 0xdd18,
0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd27, 0x2d, 0xd83a, 0xdd04, 0xd83a, 0xdd2b, 0xd83a,
0xdd45, 0xd83a, 0xdd18, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0x2d, 0xd83a, 0xdd04,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd18, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd32,
0xd83a, 0xdd23, 0x2d, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd33, 0xd83a,
0xdd18, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd18, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a,
0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd35, 0xd83a, 0xdd28, 0xd83a, 0xdd46, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd18, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a,
0xdd3c, 0xd83a, 0xdd2b, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a,
0xdd18, 0xd83a, 0xdd35, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a,
0xdd24, 0xd83a, 0xdd18, 0xd83a, 0xdd35, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a,
0xdd2c, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd16, 0xd83a,
0xdd22, 0xd83a, 0xdd3e, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd25, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a,
0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd28, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a,
0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd33, 0xd83a, 0xdd27, 0xd83a,
0xdd35, 0x2c, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a,
0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0x2c, 0x20, 0xd83a,
0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd06, 0xd83a, 0xdd2b, 0xd83a, 0xdd3c, 0xd83a,
0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd04, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd3a, 0x2c, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a,
0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd1a, 0xd83a,
0xdd2b, 0xd83a, 0xdd24, 0x2d, 0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d,
0xd83a, 0xdd45, 0x2c, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d,
0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd1c,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd3e, 0xd83a, 0xdd2b, 0xd83a, 0xdd34, 0x2c, 0x20,
0xd83a, 0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd1c, 0xd83a, 0xdd2d, 0xd83a, 0xdd32,
0xd83a, 0xdd27, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0x2c, 0x20, 0xd83a, 0xdd0b,
0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd0f, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd22,
0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0x2c, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd32,
0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a, 0xdd3e, 0xd83a, 0xdd2d,
0xd83a, 0xdd33, 0xd83a, 0xdd0b, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd35,
0xd83a, 0xdd31, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a, 0xdd14, 0xd83a, 0xdd22, 0xd83a, 0xdd25,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd34, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd14,
0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd02,
0xd83a, 0xdd35, 0xd83a, 0xdd2d, 0xd83a, 0xdd3e, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd03,
0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd37, 0xd83a, 0xdd2b,
0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0x2c, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2b,
0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd2d,
0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd2b,
0xd83a, 0xdd32, 0xd83a, 0xdd36, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd02,
0xd83a, 0xdd22, 0x2d, 0xd83a, 0xdd06, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd41, 0xd83a,
0xdd02, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd02, 0xd83a,
0xdd2e, 0xd83a, 0xdd27, 0x2d, 0xd83a, 0xdd00, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd24,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd02, 0xd83a, 0xdd2e, 0xd83a, 0xdd31,
0xd83a, 0xdd2e, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a,
0xdd27, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2e,
0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd27,
0xd83a, 0xdd2b, 0xd83a, 0xdd34, 0xd83a, 0xdd2e, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd32,
0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd03, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd03, 0xd83a, 0xdd22,
0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd03,
0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd2d,
0xd83a, 0xdd33, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd25,
0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd03,
0xd83a, 0xdd22, 0xd83a, 0xdd41, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd24, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd03, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd2e, 0xd83a, 0xdd25,
0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd03, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a,
0xd83a, 0xdd2d, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd03, 0xd83a, 0xdd2b, 0xd83a, 0xdd3c,
0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd24,
0xd83a, 0xdd22, 0xd83a, 0xdd03, 0xd83a, 0xdd2b, 0xd83a, 0xdd33, 0xd83a, 0xdd27, 0xd83a, 0xdd2d,
0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a,
0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd03, 0xd83a, 0xdd2b, 0xd83a, 0xdd33, 0xd83a,
0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd03, 0xd83a, 0xdd2e, 0xd83a,
0xdd32, 0xd83a, 0xdd33, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a,
0xdd03, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd34, 0xd83a, 0xdd03, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a,
0xdd3c, 0xd83a, 0xdd2b, 0xd83a, 0xdd3e, 0xd83a, 0xdd2d, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a,
0xdd34, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd03, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a,
0xdd3c, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a,
0xdd3c, 0xd83a, 0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd10, 0xd83a, 0xdd2b, 0xd83a, 0xdd31, 0x2d, 0xd83a, 0xdd12, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a,
0xd83a, 0xdd33, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd31, 0xd83a, 0xdd25, 0xd83a, 0xdd35,
0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3f,
0xd83a, 0xdd22, 0xd83a, 0xdd04, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0x2c, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2c, 0x2d, 0xd83a,
0xdd01, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a,
0xdd05, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0x2c,
0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2c, 0x2d, 0xd83a, 0xdd01,
0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd10, 0xd83a, 0xdd2b, 0xd83a, 0xdd31, 0x2d, 0xd83a, 0xdd05, 0xd83a, 0xdd2b, 0xd83a,
0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd25, 0x2c, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a,
0xdd2a, 0xd83a, 0xdd2c, 0x2d, 0xd83a, 0xdd01, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd2e,
0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd10, 0xd83a, 0xdd35, 0xd83a, 0xdd45,
0xd83a, 0xdd33, 0xd83a, 0xdd0c, 0xd83a, 0xdd36, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd22,
0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd06, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd06, 0xd83a, 0xdd22,
0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d,
0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0xd83a, 0xdd0a, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd2d,
0xd83a, 0xdd33, 0xd83a, 0xdd27, 0xd83a, 0xdd06, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd3c,
0x2d, 0xd83a, 0xdd2e, 0x2d, 0xd83a, 0xdd06, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd32,
0xd83a, 0xdd27, 0xd83a, 0xdd06, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd3c, 0x20, 0xd83a,
0xdd2e, 0xd83a, 0xdd2c, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a, 0xdd28, 0xd83a, 0xdd2b,
0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd06, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd3c,
0xd83a, 0xdd2e, 0x2d, 0xd83a, 0xdd1c, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd38, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd06, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd3c, 0x2d,
0xd83a, 0xdd08, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd06,
0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0x2d, 0xd83a, 0xdd00, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd41, 0xd83a, 0xdd08, 0xd83a,
0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0x20, 0xd83a, 0xdd0b,
0xd83a, 0xdd32, 0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd3c, 0xd83a, 0xdd08, 0xd83a, 0xdd2b,
0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd2c, 0xd83a, 0xdd2d, 0xd83a, 0xdd08, 0xd83a, 0xdd2d,
0xd83a, 0xdd3a, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd08,
0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd35, 0xd83a, 0xdd45,
0xd83a, 0xdd3c, 0xd83a, 0xdd08, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd2e, 0x2d, 0xd83a,
0xdd04, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a,
0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd25, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0x2d,
0xd83a, 0xdd01, 0xd83a, 0xdd2e, 0xd83a, 0xdd25, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd3a,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd31,
0x2d, 0xd83a, 0xdd06, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd0b, 0xd83a, 0xdd3c, 0xd83a, 0xdd46, 0xd83a, 0xdd2e, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a,
0xdd2a, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd25, 0xd83a,
0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd3c, 0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a,
0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd05, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0x2d,
0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd2b,
0xd83a, 0xdd25, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd05, 0xd83a, 0xdd2b, 0xd83a, 0xdd32,
0xd83a, 0xdd3c, 0x2d, 0xd83a, 0xdd14, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a,
0xdd27, 0xd83a, 0xdd05, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0x2d, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a, 0xdd46, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd05,
0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0x2d, 0xd83a, 0xdd02, 0xd83a, 0xdd35, 0xd83a,
0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd22, 0xd83a, 0xdd05, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a,
0xdd3c, 0x2d, 0xd83a, 0xdd1a, 0xd83a, 0xdd2e, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd27, 0xd83a, 0xdd05, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0x2d, 0xd83a,
0xdd1c, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd27, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a,
0xdd3c, 0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a, 0xdd2c, 0xd83a, 0xdd3c, 0x2d, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3c,
0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd3a, 0xd83a, 0xdd35, 0xd83a, 0xdd27, 0xd83a, 0xdd2d,
0xd83a, 0xdd3a, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd28, 0xd83a, 0xdd22, 0xd83a, 0xdd1a,
0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd1a, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd22,
0xd83a, 0xdd1a, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3c,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd1a, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd3c,
0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd1c, 0xd83a, 0xdd2b,
0xd83a, 0xdd32, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd3e, 0xd83a, 0xdd2e,
0xd83a, 0xdd0f, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd3c, 0xd83a, 0xdd38, 0xd83a, 0xdd2e,
0xd83a, 0xdd2a, 0xd83a, 0xdd27, 0xd83a, 0xdd2b, 0xd83a, 0xdd0f, 0xd83a, 0xdd2d, 0xd83a, 0xdd32,
0xd83a, 0xdd2d, 0xd83a, 0xdd28, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd3a, 0xd83a, 0xdd12,
0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd3c,
0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd27, 0xd83a, 0xdd2b, 0xd83a, 0xdd01,
0xd83a, 0xdd22, 0xd83a, 0xdd3e, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd01,
0xd83a, 0xdd2d, 0xd83a, 0xdd25, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd35,
0x2d, 0xd83a, 0xdd01, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd3e, 0xd83a, 0xdd2d, 0xd83a,
0xdd24, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd27, 0xd83a,
0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd25, 0xd83a,
0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd06, 0xd83a,
0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd08, 0xd83a, 0xdd2e, 0xd83a, 0xdd3c, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a,
0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd34, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a,
0xdd22, 0xd83a, 0xdd1a, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd24, 0xd83a, 0xdd1c, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a,
0xdd33, 0xd83a, 0xdd02, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd36, 0xd83a, 0xdd2d, 0xd83a,
0xdd2a, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd32, 0xd83a, 0xdd00, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd00, 0xd83a,
0xdd24, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a,
0xdd00, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a,
0xdd00, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd00, 0xd83a, 0xdd33, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd31, 0xd83a, 0xdd35, 0xd83a, 0xdd00, 0xd83a, 0xdd33, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd26, 0xd83a, 0xdd2b, 0xd83a, 0xdd00, 0xd83a, 0xdd27, 0xd83a, 0xdd3a, 0xd83a,
0xdd22, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0xd83a, 0xdd00, 0xd83a,
0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd35, 0xd83a,
0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd3f, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd23, 0xd83a, 0xdd35, 0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd38, 0xd83a, 0xdd2a, 0xd83a,
0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd04, 0xd83a,
0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd24, 0xd83a,
0xdd35, 0xd83a, 0xdd04, 0xd83a, 0xdd2b, 0xd83a, 0xdd34, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a,
0xdd45, 0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0xd83a, 0xdd04, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a,
0xdd33, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd04, 0xd83a,
0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd15, 0xd83a,
0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a,
0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd01, 0xd83a,
0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd39, 0xd83a, 0xdd22, 0xd83a,
0xdd01, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd01, 0xd83a,
0xdd35, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd01, 0xd83a, 0xdd35, 0xd83a,
0xdd27, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd26, 0xd83a, 0xdd2b, 0xd83a, 0xdd0a, 0xd83a,
0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd35, 0xd83a, 0xdd27, 0xd83a,
0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd18, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd41, 0xd83a,
0xdd22, 0xd83a, 0xdd1d, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd24, 0xd83a, 0xdd35, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd38, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0x2d, 0xd83a, 0xdd15, 0xd83a,
0xdd2d, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd16,
0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a,
0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd16, 0xd83a, 0xdd2e, 0xd83a, 0xdd2c, 0xd83a, 0xdd23, 0xd83a,
0xdd35, 0xd83a, 0xdd0b, 0xd83a, 0xdd2a, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd3c, 0xd83a,
0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd14, 0xd83a, 0xdd22, 0xd83a,
0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd14, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd28, 0xd83a, 0xdd35, 0xd83a,
0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd17, 0xd83a, 0xdd35, 0xd83a, 0xdd23, 0xd83a, 0xdd35, 0xd83a,
0xdd27, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a,
0xdd24, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd37, 0xd83a, 0xdd22, 0xd83a,
0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a,
0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd37, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a,
0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd23, 0xd83a, 0xdd35, 0xd83a, 0xdd1d, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a,
0xdd2d, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a,
0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a,
0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd32, 0xd83a, 0xdd2e, 0xd83a, 0xdd34, 0xd83a,
0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd27, 0xd83a, 0xdd35, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a,
0xdd11, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0x2d,
0xd83a, 0xdd02, 0xd83a, 0xdd2e, 0xd83a, 0xdd25, 0xd83a, 0xdd28, 0xd83a, 0xdd35, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd11, 0xd83a, 0xdd35, 0xd83a, 0xdd37, 0xd83a, 0xdd2d, 0xd83a, 0xdd32,
0xd83a, 0xdd11, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd3c,
0xd83a, 0xdd2d, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd31,
0xd83a, 0xdd2e, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd23,
0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22,
0xd83a, 0xdd27, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd2a, 0xd83a, 0xdd03, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd03, 0xd83a, 0xdd35,
0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0xd83a, 0xdd10,
0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd34,
0xd83a, 0xdd22, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd3e, 0xd83a, 0xdd2e, 0xd83a, 0xdd33,
0xd83a, 0xdd35, 0xd83a, 0xdd41, 0xd83a, 0xdd32, 0xd83a, 0xdd2b, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d,
0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd3e,
0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a,
0xd83a, 0xdd27, 0xd83a, 0xdd35, 0xd83a, 0xdd33, 0xd83a, 0xdd0c, 0xd83a, 0xdd25, 0xd83a, 0xdd27,
0xd83a, 0xdd35, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd13, 0xd83a, 0xdd2a, 0xd83a, 0xdd22,
0xd83a, 0xdd24, 0xd83a, 0xdd06, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd2e, 0xd83a, 0xdd25,
0x2d, 0xd83a, 0xdd06, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd06, 0xd83a, 0xdd2e, 0xd83a,
0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd06, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd2e, 0xd83a,
0xdd32, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd17, 0xd83a, 0xdd22, 0xd83a,
0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a,
0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd41, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a,
0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd08, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd23, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd3f, 0xd83a, 0xdd22, 0xd83a,
0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a,
0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd23, 0xd83a, 0xdd35, 0xd83a, 0xdd05, 0xd83a, 0xdd2b, 0xd83a, 0xdd31, 0xd83a, 0xdd35, 0xd83a,
0xdd24, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd38, 0xd83a, 0xdd22, 0xd83a,
0xdd34, 0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a,
0xdd28, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd23, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a,
0xdd2d, 0xd83a, 0xdd25, 0xd83a, 0xdd27, 0xd83a, 0xdd35, 0xd83a, 0xdd1a, 0xd83a, 0xdd22, 0xd83a,
0xdd34, 0xd83a, 0xdd28, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd1a, 0xd83a, 0xdd22, 0xd83a,
0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0xd83a,
0xdd1a, 0xd83a, 0xdd2d, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a,
0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd1a, 0xd83a, 0xdd2b, 0xd83a, 0xdd38, 0xd83a,
0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd1a, 0xd83a,
0xdd2d, 0xd83a, 0xdd25, 0xd83a, 0xdd28, 0xd83a, 0xdd35, 0xd83a, 0xdd1a, 0xd83a, 0xdd2e, 0xd83a,
0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd2e, 0xd83a, 0xdd1a, 0xd83a, 0xdd2e, 0xd83a,
0xdd25, 0xd83a, 0xdd27, 0xd83a, 0xdd35, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd13, 0xd83a,
0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a,
0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd13, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a,
0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd13, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0x2d,
0xd83a, 0xdd10, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd1c, 0xd83a, 0xdd2d,
0xd83a, 0xdd34, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd37, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd32, 0xd83a, 0xdd1c, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd23,
0xd83a, 0xdd2d, 0xd83a, 0xdd3e, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd12, 0xd83a, 0xdd22, 0xd83a, 0xdd33,
0xd83a, 0xdd35, 0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd35,
0xd83a, 0xdd08, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd35, 0xd83a, 0xdd45,
0xd83a, 0xdd32, 0xd83a, 0xdd12, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd3c,
0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd26, 0xd83a, 0xdd35,
0xd83a, 0xdd2a, 0xd83a, 0xdd3a, 0xd83a, 0xdd35, 0xd83a, 0xdd12, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0xd83a, 0xdd3e, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd00,
0xd83a, 0xdd41, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd27,
0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd25, 0xd83a, 0xdd35, 0xd83a, 0xdd45,
0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22,
0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd26,
0xd83a, 0xdd2e, 0x2d, 0xd83a, 0xdd1c, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a,
0xdd2b, 0xd83a, 0xdd0a, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a,
0xdd22, 0xd83a, 0xdd08, 0xd83a, 0xdd2b, 0xd83a, 0xdd34, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a,
0xdd3e, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a,
0xdd31, 0xd83a, 0xdd2c, 0x2d, 0xd83a, 0xdd14, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd36,
0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd05, 0xd83a, 0xdd2b,
0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0x2d, 0xd83a, 0xdd16, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a,
0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a,
0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd00, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a,
0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd04, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd27, 0xd83a,
0xdd2d, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a,
0xdd04, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0x2d,
0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd24, 0xd83a, 0xdd01, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd2a, 0xd83a, 0xdd31, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd13,
0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd16,
0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd3c,
0xd83a, 0xdd35, 0xd83a, 0xdd02, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d,
0xd83a, 0xdd45, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd02,
0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd35, 0x2d, 0xd83a, 0xdd16, 0xd83a,
0xdd2e, 0xd83a, 0xdd31, 0xd83a, 0xdd2b, 0xd83a, 0xdd03, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a,
0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a,
0xdd06, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd05, 0xd83a,
0xdd2d, 0xd83a, 0xdd23, 0xd83a, 0xdd32, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd00, 0xd83a,
0xdd25, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a,
0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd25, 0xd83a, 0xdd00, 0xd83a, 0xdd32, 0xd83a,
0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd00, 0xd83a,
0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd3f, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd00, 0xd83a, 0xdd3c, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a,
0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd04, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd3a, 0xd83a,
0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd23, 0xd83a, 0xdd04, 0xd83a,
0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd04, 0xd83a,
0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a,
0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd3e, 0xd83a, 0xdd22, 0xd83a, 0xdd04, 0xd83a, 0xdd2d, 0xd83a,
0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd27, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a,
0xdd04, 0xd83a, 0xdd35, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a,
0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd04, 0xd83a, 0xdd35, 0xd83a, 0xdd23, 0xd83a,
0xdd22, 0xd83a, 0xdd28, 0xd83a, 0xdd2b, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd04, 0xd83a,
0xdd35, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd36, 0xd83a, 0xdd2b, 0xd83a,
0xdd32, 0xd83a, 0xdd15, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a,
0xdd32, 0xd83a, 0xdd2e, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a,
0xdd28, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd3a, 0xd83a, 0xdd01, 0xd83a, 0xdd35, 0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a,
0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd14, 0xd83a, 0xdd2d, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a,
0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a,
0xdd14, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a,
0xdd2b, 0xd83a, 0xdd34, 0xd83a, 0xdd16, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd27, 0xd83a,
0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd05, 0xd83a, 0xdd35, 0xd83a,
0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x2d, 0xd83a, 0xdd03,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd0b, 0xd83a, 0xdd27, 0xd83a, 0xdd3c,
0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd14,
0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd11,
0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd22,
0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd3e, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd34,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd3e, 0xd83a, 0xdd02, 0xd83a, 0xdd2d, 0xd83a, 0xdd27,
0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd0b, 0xd83a, 0xdd36,
0xd83a, 0xdd35, 0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd36, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd02, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd23,
0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd02, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd27,
0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd3a,
0xd83a, 0xdd35, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a,
0xd83a, 0xdd2d, 0xd83a, 0xdd23, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd3c,
0xd83a, 0xdd22, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd34,
0xd83a, 0xdd22, 0xd83a, 0xdd38, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd25, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd03, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd27,
0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd03, 0xd83a, 0xdd2e, 0xd83a, 0xdd32,
0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd38, 0xd83a, 0xdd03, 0xd83a, 0xdd2e,
0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd0c, 0xd83a, 0xdd27, 0xd83a, 0xdd24,
0xd83a, 0xdd2e, 0xd83a, 0xdd38, 0xd83a, 0xdd06, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d,
0xd83a, 0xdd06, 0xd83a, 0xdd2e, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e,
0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd06, 0xd83a, 0xdd22,
0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd3a, 0xd83a, 0xdd2d, 0xd83a, 0xdd08,
0xd83a, 0xdd2d, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd08, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd25, 0xd83a, 0xdd35, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd22,
0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0x2d, 0xd83a,
0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd2e, 0xd83a,
0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd2b, 0xd83a,
0xdd3e, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a,
0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd3e, 0xd83a, 0xdd05, 0xd83a,
0xdd2d, 0xd83a, 0xdd25, 0xd83a, 0xdd2c, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd28, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd24, 0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd28, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a,
0xdd05, 0xd83a, 0xdd2e, 0xd83a, 0xdd2c, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a,
0xdd05, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a,
0xdd24, 0xd83a, 0xdd25, 0xd83a, 0xdd35, 0xd83a, 0xdd1a, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a,
0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a, 0xdd1a, 0xd83a, 0xdd2d, 0xd83a,
0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd13, 0xd83a, 0xdd24, 0xd83a,
0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd2e, 0xd83a, 0xdd3e, 0xd83a, 0xdd2e, 0xd83a,
0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd1c, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a,
0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd36, 0xd83a, 0xdd35, 0xd83a, 0xdd1c, 0xd83a, 0xdd22, 0xd83a,
0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd1c, 0xd83a,
0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd1c, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a,
0xdd27, 0xd83a, 0xdd1c, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a,
0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd0f, 0xd83a,
0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd27, 0xd83a, 0xdd2e, 0xd83a, 0xdd1f, 0xd83a, 0xdd22, 0xd83a,
0xdd3a, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd1f, 0xd83a,
0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd33, 0xd83a, 0xdd00, 0xd83a, 0xdd32, 0xd83a,
0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a,
0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd3e, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd27, 0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a,
0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd25, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd11, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a,
0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd03, 0xd83a,
0xdd22, 0xd83a, 0xdd38, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a,
0xdd24, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd34, 0xd83a, 0xdd2d, 0xd83a,
0xdd27, 0xd83a, 0xdd03, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a,
0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0xd83a, 0xdd08, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd00, 0xd83a,
0xdd44, 0xd83a, 0xdd28, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd0c, 0xd83a,
0xdd33, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a,
0xdd04, 0xd83a, 0xdd35, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd3e, 0xd83a,
0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd24, 0xd83a, 0xdd15, 0xd83a, 0xdd22, 0xd83a, 0xdd43, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd25, 0xd83a, 0xdd15, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a,
0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd0b, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd09, 0xd83a, 0xdd2c, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0xd83a,
0xdd0a, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2c, 0xd83a,
0xdd2e, 0xd83a, 0xdd0a, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a,
0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd2c, 0xd83a, 0xdd35, 0xd83a,
0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd18, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a,
0xdd22, 0xd83a, 0xdd28, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a,
0xdd18, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a,
0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd18, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a,
0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd18, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd25, 0xd83a, 0xdd09, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a,
0xdd2a, 0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a,
0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a,
0xdd27, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd34, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a,
0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd36, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a,
0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd36, 0xd83a, 0xdd35, 0xd83a,
0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a,
0xdd33, 0xd83a, 0xdd2b, 0xd83a, 0xdd27, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd27, 0xd83a,
0xdd03, 0xd83a, 0xdd2d, 0xd83a, 0xdd23, 0xd83a, 0xdd31, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd10, 0xd83a,
0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd31, 0xd83a, 0xdd2d, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a,
0xdd2a, 0xd83a, 0xdd2c, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a,
0xdd10, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a,
0xdd06, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0x2d, 0xd83a, 0xdd06, 0xd83a, 0xdd22,
0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd06, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd22,
0xd83a, 0xdd31, 0xd83a, 0xdd06, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a, 0xdd33, 0xd83a, 0xdd2d,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a, 0xdd06, 0xd83a, 0xdd2e,
0xd83a, 0xdd25, 0xd83a, 0xdd28, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd06, 0xd83a, 0xdd2e,
0xd83a, 0xdd2a, 0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0x2d, 0xd83a, 0xdd03, 0xd83a, 0xdd2e, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd27, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd08, 0xd83a,
0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a,
0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd28, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd1a, 0xd83a, 0xdd22, 0xd83a, 0xdd38, 0xd83a,
0xdd2d, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd1a, 0xd83a, 0xdd2b, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd1a, 0xd83a,
0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd28, 0xd83a, 0xdd35, 0xd83a, 0xdd0f, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd0f, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd46, 0xd83a,
0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0x41, 0x64, 0x64, 0x69, 0x73, 0x20, 0x41,
0x62, 0xe4, 0x62, 0x61, 0x41, 0x73, 0x6d, 0x61, 0x72, 0xe0, 0x44, 0xe0,
0x72, 0x20, 0x61, 0x73, 0x2d, 0x53, 0x61, 0x6c, 0xe0, 0x6d, 0x44, 0x69,
0x6f, 0x62, 0xf9, 0x74, 0x61, 0x69, 0x64, 0x68, 0x48, 0x61, 0x6e, 0x6e,
0x73, 0x61, 0x62, 0x6f, 0x72, 0x67, 0x68, 0x4e, 0x2019, 0x44, 0x6a, 0x61,
0x6d, 0xe9, 0x6e, 0x61, 0x41, 0x6e, 0x67, 0x75, 0x69, 0x6c, 0x6c, 0x69,
0x61, 0x41, 0x69, 0x6e, 0x74, 0xec, 0x6f, 0x67, 0x61, 0x41, 0x72, 0xf9,
0x62, 0x61, 0x41, 0x2019, 0x20, 0x42, 0x68, 0x65, 0x69, 0x6c, 0xec, 0x73,
0x43, 0x61, 0x69, 0x6d, 0x65, 0x61, 0x6e, 0x43, 0x6f, 0x73, 0x74, 0x61,
0x20, 0x52, 0xec, 0x63, 0x65, 0x61, 0x44, 0x6f, 0x69, 0x6d, 0x69, 0x6e,
0x69, 0x63, 0x65, 0x61, 0x41, 0x6e, 0x20, 0x53, 0x61, 0x6c, 0x62, 0x68,
0x61, 0x64, 0x6f, 0x72, 0x47, 0x6c, 0x61, 0x73, 0x62, 0x61, 0x69, 0x64,
0x68, 0x41, 0x6e, 0x20, 0x54, 0x75, 0x72, 0x63, 0x20, 0x4d, 0x68, 0xf2,
0x72, 0x47, 0x72, 0x65, 0x61, 0x6e, 0xe0, 0x64, 0x61, 0x47, 0x75, 0x61,
0x74, 0x61, 0x6d, 0x61, 0x6c, 0x61, 0x47, 0x75, 0x69, 0x64, 0x68, 0x65,
0xe0, 0x6e, 0x61, 0x49, 0x6e, 0x75, 0x75, 0x76, 0x69, 0x6b, 0x44, 0x69,
0x61, 0x6d, 0x65, 0x75, 0x67, 0x61, 0x4d, 0x61, 0x69, 0x72, 0x74, 0x69,
0x6e, 0x69, 0x63, 0x43, 0x61, 0x74, 0x68, 0x61, 0x69, 0x72, 0x20, 0x4d,
0x68, 0x65, 0x61, 0x67, 0x73, 0x61, 0x67, 0x6f, 0x4d, 0x6f, 0x6e, 0x74,
0x73, 0x61, 0x72, 0x61, 0x74, 0x4e, 0x75, 0x61, 0x64, 0x68, 0x20, 0x45,
0x61, 0x62, 0x68, 0x72, 0x61, 0x63, 0x50, 0x6f, 0x72, 0x74, 0x20, 0x6e,
0x61, 0x20, 0x53, 0x70, 0xe0, 0x69, 0x6e, 0x6e, 0x65, 0x50, 0x6f, 0x72,
0x74, 0x6f, 0x20, 0x52, 0xec, 0x63, 0x65, 0x6f, 0x4b, 0x61, 0x6e, 0x67,
0x69, 0x71, 0x6c, 0x69, 0x6e, 0x69, 0x71, 0x51, 0x61, 0x75, 0x73, 0x75,
0x69, 0x74, 0x74, 0x75, 0x71, 0x4e, 0x61, 0x6f, 0x6d, 0x68, 0x20, 0x43,
0x72, 0xec, 0x73, 0x74, 0x65, 0x61, 0x6e, 0x4e, 0x61, 0x6f, 0x6d, 0x68,
0x20, 0x4c, 0xf9, 0x69, 0x73, 0x65, 0x61, 0x4e, 0x61, 0x6f, 0x6d, 0x68,
0x20, 0x42, 0x68, 0x69, 0x6f, 0x6e, 0x73, 0x61, 0x6e, 0x74, 0x45, 0x69,
0x6c, 0x65, 0x61, 0x6e, 0x20, 0x4d, 0x68, 0x69, 0x63, 0x47, 0x75, 0x61,
0x69, 0x72, 0x65, 0x41, 0x6d, 0x6d, 0x101, 0x6e, 0x42, 0x61, 0x67, 0x68,
0x64, 0x101, 0x64, 0x42, 0x61, 0x63, 0x68, 0x72, 0x61, 0x69, 0x6e, 0x42,
0x72, 0xf9, 0x6e, 0x61, 0x69, 0x67, 0x68, 0x47, 0xe0, 0x73, 0x61, 0x43,
0x61, 0x74, 0x68, 0x61, 0x69, 0x72, 0x20, 0x48, 0x6f, 0x20, 0x43, 0x68,
0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x49, 0x65, 0x72, 0x75, 0x73, 0x61,
0x6c, 0x61, 0x6d, 0x4b, 0x61, 0x72, 0x101, 0x63, 0x12b, 0x43, 0x75, 0x69,
0x62, 0x68, 0xe8, 0x69, 0x74, 0x4d, 0x61, 0x63, 0xe0, 0x74, 0x68, 0x75,
0x50, 0x79, 0x65, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x43, 0x61,
0x74, 0x61, 0x72, 0x51, 0x6f, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x131, 0x53,
0x69, 0x6e, 0x67, 0x65, 0x61, 0x70, 0xf2, 0x72, 0x54, 0x6f, 0x161, 0x6b,
0x65, 0x6e, 0x74, 0x54, 0x2019, 0x62, 0x69, 0x6c, 0x69, 0x73, 0x69, 0x54,
0x65, 0x68, 0x72, 0x101, 0x6e, 0x56, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x43,
0x68, 0x61, 0x6e, 0x50, 0x6f, 0x6e, 0x74, 0x61, 0x20, 0x44, 0x65, 0x6c,
0x67, 0x61, 0x64, 0x61, 0x42, 0x65, 0x61, 0x72, 0x6d, 0xf9, 0x64, 0x61,
0x4e, 0x61, 0x20, 0x68, 0x2d, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61,
0x6e, 0x20, 0x43, 0x61, 0x6e, 0xe0, 0x72, 0x61, 0x63, 0x68, 0x41, 0x6e,
0x20, 0x43, 0x65, 0x61, 0x70, 0x20, 0x55, 0x61, 0x69, 0x6e, 0x65, 0x46,
0xe0, 0x72, 0x6f, 0x53, 0x65, 0xf2, 0x72, 0x73, 0x65, 0x61, 0x20, 0x61,
0x20, 0x44, 0x65, 0x61, 0x73, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x20,
0x4e, 0x61, 0x6f, 0x6d, 0x68, 0x20, 0x45, 0x69, 0x6c, 0x69, 0x64, 0x68,
0x41, 0x6e, 0x20, 0xc0, 0x69, 0x74, 0x68, 0x6e, 0x65, 0x41, 0x2019, 0x20,
0x42, 0x68, 0x72, 0x75, 0x69, 0x73, 0x65, 0x61, 0x6c, 0x42, 0x65, 0x69,
0x72, 0x62, 0x68, 0x42, 0x61, 0x69, 0x6c, 0x65, 0x20, 0xc0, 0x74, 0x68,
0x61, 0x20, 0x43, 0x6c, 0x69, 0x61, 0x74, 0x68, 0x44, 0x69, 0x6f, 0x62,
0x72, 0x61, 0x6c, 0x74, 0x61, 0x72, 0x47, 0x65, 0xe0, 0x72, 0x6e, 0x73,
0x61, 0x69, 0x64, 0x68, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x20, 0x4d,
0x68, 0x61, 0x6e, 0x61, 0x69, 0x6e, 0x6e, 0x44, 0x65, 0xe0, 0x72, 0x73,
0x61, 0x69, 0x64, 0x68, 0x44, 0xf9, 0x6e, 0x20, 0xc8, 0x69, 0x64, 0x65,
0x61, 0x6e, 0x6e, 0x2f, 0x4c, 0x75, 0x6e, 0x6e, 0x61, 0x69, 0x6e, 0x6e,
0x4c, 0x75, 0x67, 0x73, 0x61, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x4d, 0x6f,
0x73, 0x67, 0x6f, 0x50, 0x61, 0x72, 0x61, 0x73, 0x50, 0x72, 0xe0, 0x67,
0x52, 0x12b, 0x67, 0x61, 0x41, 0x6e, 0x20, 0x52, 0xf2, 0x69, 0x6d, 0x68,
0x41, 0x2019, 0x20, 0x42, 0x68, 0x61, 0x74, 0x61, 0x63, 0x61, 0x6e, 0x4e,
0x6f, 0x6c, 0x6c, 0x61, 0x69, 0x67, 0x4e, 0x61, 0x20, 0x68, 0x2d, 0x45,
0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x4d, 0x68, 0x61, 0x6c,
0x61, 0x64, 0x61, 0x69, 0x62, 0x68, 0x4e, 0x61, 0x20, 0x68, 0x2d, 0x45,
0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x4d, 0x68, 0x6f, 0x69,
0x72, 0x69, 0x73, 0x65, 0x61, 0x73, 0x52, 0x61, 0x70, 0x61, 0x20, 0x4e,
0x75, 0x69, 0x46, 0xec, 0x64, 0x69, 0x4d, 0x61, 0x6e, 0x67, 0x61, 0x72,
0x65, 0x76, 0x61, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20,
0x61, 0x2019, 0x20, 0x4d, 0x68, 0x61, 0x72, 0x63, 0x61, 0x69, 0x73, 0x4e,
0x61, 0x62, 0x68, 0x72, 0x75, 0x50, 0x61, 0x6c, 0x61, 0x62, 0x68, 0x50,
0x65, 0x69, 0x74, 0x20, 0x61, 0x2019, 0x20, 0x43, 0x68, 0xe0, 0x69, 0x72,
0x6e, 0x55, 0x61, 0x6c, 0x6c, 0x61, 0x73, 0x4e, 0x6f, 0x75, 0x61, 0x6b,
0x73, 0x68, 0x6f, 0x74, 0x74, 0x41, 0x6e, 0x6b, 0x6f, 0x72, 0x61, 0x67,
0x65, 0x4b, 0x61, 0x6d, 0x62, 0x72, 0x69, 0x64, 0x67, 0x65, 0x20, 0x14a,
0x73, 0x68, 0x254, 0x6e, 0x69, 0x6e, 0x65, 0x20, 0x42, 0x69, 0x62, 0x69,
0x6f, 0x6f, 0x54, 0x73, 0x69, 0x6b, 0x61, 0x67, 0x6f, 0x54, 0x73, 0x69,
0x68, 0x75, 0x61, 0x68, 0x75, 0x61, 0x44, 0x61, 0x77, 0x73, 0x6f, 0x6e,
0x20, 0x4b, 0x70, 0x61, 0x61, 0x6b, 0x70, 0x6f, 0x20, 0x42, 0x69, 0x62,
0x69, 0x6f, 0x6f, 0x4e, 0x65, 0x6c, 0x73, 0x6f, 0x6e, 0x20, 0x4d, 0x254,
0x254, 0x47, 0x6c, 0x61, 0x73, 0x65, 0x20, 0x14a, 0x73, 0x68, 0x254, 0x6e,
0x69, 0x6e, 0x65, 0x20, 0x42, 0x69, 0x62, 0x69, 0x6f, 0x6f, 0x47, 0x6f,
0x6f, 0x73, 0x65, 0x20, 0x14a, 0x73, 0x68, 0x254, 0x6e, 0x69, 0x6e, 0x65,
0x20, 0x42, 0x69, 0x62, 0x69, 0x6f, 0x6f, 0x54, 0x75, 0x72, 0x6b, 0x65,
0x20, 0x57, 0x75, 0x6c, 0x75, 0x48, 0x61, 0x6c, 0x69, 0x66, 0x61, 0x73,
0x4b, 0x6e, 0x6f, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e,
0x61, 0x4f, 0x73, 0x68, 0x65, 0x6b, 0x75, 0x20, 0x4d, 0x61, 0x14b, 0x2c,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x56, 0x69, 0x6e, 0x73,
0x65, 0x6e, 0x6e, 0x65, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61,
0x6e, 0x61, 0x4d, 0x6f, 0x6e, 0x74, 0x69, 0x73, 0x65, 0x6c, 0x6c, 0x6f,
0x2c, 0x20, 0x4b, 0x65, 0x6e, 0x74, 0x75, 0x6b, 0x79, 0x41, 0x62, 0x6c,
0x61, 0x64, 0x65, 0x20, 0x53, 0x68, 0x129, 0x61, 0x20, 0x4e, 0x69, 0x20,
0x59, 0x254, 0x254, 0x20, 0x4a, 0x254, 0x254, 0x20, 0x4d, 0x6c, 0x69, 0x4d,
0x65, 0x7a, 0x69, 0x6b, 0x6f, 0x20, 0x4d, 0x61, 0x14b, 0x4d, 0x61, 0x14b,
0x74, 0x65, 0x14b, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20,
0x4b, 0x6f, 0x6f, 0x79, 0x69, 0x67, 0x62, 0x25b, 0x4e, 0x65, 0x77, 0x20,
0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74,
0x61, 0x20, 0x4b, 0x6f, 0x6f, 0x79, 0x69, 0x67, 0x62, 0x25b, 0x41, 0x62,
0x6c, 0x61, 0x64, 0x65, 0x69, 0x20, 0x41, 0x6c, 0x25b, 0x6a, 0x69, 0x61,
0x64, 0x61, 0x61, 0x6d, 0x254, 0x68, 0x65, 0x53, 0x70, 0x61, 0x69, 0x6e,
0x20, 0x4c, 0x25b, 0x6a, 0x69, 0x61, 0x64, 0x61, 0x61, 0x6d, 0x254, 0x68,
0x65, 0x52, 0x61, 0x6e, 0x6b, 0x69, 0x6e, 0x20, 0x14a, 0x73, 0x68, 0x254,
0x6e, 0x69, 0x6e, 0x65, 0x49, 0x74, 0x74, 0x6f, 0x6b, 0x6f, 0x72, 0x74,
0x6f, 0x6f, 0x72, 0x6d, 0x69, 0x69, 0x74, 0x53, 0x74, 0x2e, 0x20, 0x4c,
0x75, 0x73, 0x69, 0x61, 0x53, 0x74, 0x2e, 0x20, 0x56, 0x69, 0x6e, 0x73,
0x65, 0x6e, 0x74, 0x53, 0x77, 0x69, 0x66, 0x74, 0x20, 0x4b, 0x61, 0x72,
0x25b, 0x6e, 0x74, 0x56, 0x61, 0x6e, 0x6b, 0x6f, 0x75, 0x76, 0x65, 0x72,
0x4f, 0x6b, 0x70, 0x254, 0x14b, 0x254, 0x20, 0x59, 0x25b, 0x14b, 0x4b, 0x61,
0x73, 0x65, 0x79, 0x4d, 0x61, 0x6b, 0x77, 0x61, 0x72, 0x69, 0x65, 0x4d,
0x61, 0x6b, 0x4d, 0x75, 0x72, 0x64, 0x6f, 0x48, 0x6f, 0x20, 0x54, 0x73,
0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x20, 0x4d, 0x61, 0x14b, 0x74, 0x69,
0x61, 0x73, 0x65, 0x4b, 0x61, 0x72, 0x61, 0x74, 0x73, 0x69, 0x4b, 0x75,
0x74, 0x73, 0x69, 0x6e, 0x67, 0x55, 0x72, 0x75, 0x6d, 0x6b, 0x69, 0x14a,
0x6d, 0x65, 0x14b, 0x6d, 0x65, 0x20, 0x4e, 0x69, 0x20, 0x45, 0x6b, 0x75,
0x6d, 0x254, 0x4e, 0x75, 0x14b, 0x74, 0x73, 0x254, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x54, 0x73, 0x61, 0x67, 0x6f, 0x73, 0x4b, 0x72, 0x69, 0x73, 0x6d,
0x61, 0x73, 0x186, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x54, 0x73, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x54, 0x73, 0x75, 0x75, 0x6b, 0x4d, 0x61, 0x72, 0x6b,
0x77, 0x65, 0x73, 0x61, 0x73, 0x41, 0x63, 0x72, 0x61, 0x41, 0x6c, 0x78,
0x65, 0x72, 0x42, 0x61, 0x6d, 0x61, 0x63, 0x6f, 0x4f, 0x20, 0x43, 0x61,
0x69, 0x72, 0x6f, 0x4f, 0x20, 0x41, 0x69, 0xfa, 0x6e, 0x58, 0x6f, 0x68,
0x61, 0x6e, 0x65, 0x73, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x4b, 0x68, 0x61,
0x72, 0x74, 0xfa, 0x6e, 0x4c, 0x75, 0x73, 0x61, 0x63, 0x61, 0x4d, 0x6f,
0x67, 0x61, 0x64, 0x69, 0x78, 0x6f, 0x53, 0x61, 0x6e, 0x20, 0x54, 0x6f,
0x6d, 0xe9, 0x54, 0x75, 0x6e, 0x65, 0x73, 0x42, 0x61, 0xed, 0x61, 0x49,
0x6c, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x61, 0x69, 0x6d, 0xe1, 0x6e, 0x4f,
0x20, 0x53, 0x61, 0x6c, 0x76, 0x61, 0x64, 0x6f, 0x72, 0x47, 0xfc, 0x69,
0x61, 0x6e, 0x61, 0x41, 0x20, 0x48, 0x61, 0x62, 0x61, 0x6e, 0x61, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x6e, 0xe1, 0x70, 0x6f, 0x6c, 0x69, 0x73, 0x41,
0x20, 0x50, 0x61, 0x7a, 0x4f, 0x73, 0x20, 0xc1, 0x6e, 0x78, 0x65, 0x6c,
0x65, 0x73, 0x43, 0x69, 0x64, 0x61, 0x64, 0x65, 0x20, 0x64, 0x65, 0x20,
0x4d, 0xe9, 0x78, 0x69, 0x63, 0x6f, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68,
0x2c, 0x20, 0x44, 0x61, 0x63, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x4e, 0x6f, 0x72, 0x74, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c,
0x20, 0x44, 0x61, 0x63, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4e,
0x6f, 0x72, 0x74, 0x65, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65,
0x6d, 0x2c, 0x20, 0x44, 0x61, 0x63, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x6f,
0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x20,
0x50, 0x72, 0xed, 0x6e, 0x63, 0x69, 0x70, 0x65, 0x50, 0x6f, 0x72, 0x74,
0x6f, 0x20, 0x45, 0x73, 0x70, 0x61, 0xf1, 0x61, 0x52, 0xed, 0x6f, 0x20,
0x42, 0x72, 0x61, 0x6e, 0x63, 0x6f, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x20,
0x4c, 0x75, 0x63, 0xed, 0x61, 0x53, 0x61, 0x6e, 0x20, 0x56, 0x69, 0x63,
0x65, 0x6e, 0x74, 0x65, 0x41, 0x64, 0xe9, 0x6e, 0x41, 0x63, 0x68, 0x6b,
0x68, 0x61, 0x62, 0x61, 0x64, 0x42, 0x61, 0x63, 0xfa, 0x43, 0x68, 0x69,
0x74, 0xe1, 0x44, 0x61, 0x6d, 0x61, 0x73, 0x63, 0x6f, 0x49, 0x61, 0x63,
0x61, 0x72, 0x74, 0x61, 0x43, 0x61, 0x62, 0x75, 0x6c, 0x45, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x49, 0x65,
0x72, 0x65, 0x76, 0xe1, 0x6e, 0x49, 0x6c, 0x6c, 0x61, 0x73, 0x20, 0x42,
0x65, 0x72, 0x6d, 0x75, 0x64, 0x61, 0x73, 0x49, 0x6c, 0x6c, 0x61, 0x73,
0x20, 0x43, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x61, 0x73, 0x52, 0x65, 0x69,
0x71, 0x75, 0x69, 0x61, 0x76, 0x69, 0x6b, 0x58, 0x65, 0x6f, 0x72, 0x78,
0x69, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x53, 0x75, 0x72, 0xc1, 0x6d, 0x73,
0x74, 0x65, 0x72, 0x64, 0x61, 0x6d, 0x41, 0x73, 0x74, 0x72, 0x61, 0x6b,
0xe1, 0x6e, 0x42, 0x72, 0x75, 0x78, 0x65, 0x6c, 0x61, 0x73, 0x48, 0x65,
0x6c, 0x73, 0x69, 0x6e, 0x71, 0x75, 0x69, 0x49, 0x6c, 0x6c, 0x61, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x61, 0x6e, 0x4c, 0x75, 0x78, 0x65, 0x6d, 0x62,
0x75, 0x72, 0x67, 0x6f, 0x4d, 0xf3, 0x6e, 0x61, 0x63, 0x6f, 0x4d, 0x6f,
0x73, 0x63, 0x6f, 0x76, 0x61, 0x53, 0x61, 0x72, 0x61, 0x69, 0x65, 0x76,
0x6f, 0x53, 0x69, 0x6d, 0x66, 0x65, 0x72, 0xf3, 0x70, 0x6f, 0x6c, 0x45,
0x73, 0x74, 0x6f, 0x63, 0x6f, 0x6c, 0x6d, 0x6f, 0x56, 0x61, 0x74, 0x69,
0x63, 0x61, 0x6e, 0x6f, 0x49, 0x6c, 0x6c, 0x61, 0x20, 0x43, 0x68, 0x72,
0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76,
0x61, 0x73, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6f, 0x49, 0x6c,
0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61,
0x49, 0x6c, 0x6c, 0x61, 0x73, 0x20, 0x47, 0x61, 0x6c, 0xe1, 0x70, 0x61,
0x67, 0x6f, 0x73, 0x48, 0x6f, 0x6e, 0x6f, 0x6c, 0x75, 0x6c, 0xfa, 0x10d0,
0x10d1, 0x10d8, 0x10ef, 0x10d0, 0x10dc, 0x10d8, 0x10d0, 0x10d9, 0x10e0, 0x10d0, 0x10d0, 0x10d3,
0x10d8, 0x10e1, 0x2d, 0x10d0, 0x10d1, 0x10d4, 0x10d1, 0x10d0, 0x10d0, 0x10da, 0x10df, 0x10d8,
0x10e0, 0x10d8, 0x10d0, 0x10e1, 0x10db, 0x10d0, 0x10e0, 0x10d0, 0x10d1, 0x10d0, 0x10db, 0x10d0,
0x10d9, 0x10dd, 0x10d1, 0x10d0, 0x10dc, 0x10d2, 0x10d8, 0x10d1, 0x10d0, 0x10dc, 0x10ef, 0x10e3,
0x10da, 0x10d8, 0x10d1, 0x10d8, 0x10e1, 0x10d0, 0x10e3, 0x10d1, 0x10da, 0x10d0, 0x10dc, 0x10e2,
0x10d8, 0x10e0, 0x10d8, 0x10d1, 0x10e0, 0x10d0, 0x10d6, 0x10d0, 0x10d5, 0x10d8, 0x10da, 0x10d8,
0x10d1, 0x10e3, 0x10ef, 0x10e3, 0x10db, 0x10d1, 0x10e3, 0x10e0, 0x10d0, 0x10d9, 0x10d0, 0x10d8,
0x10e0, 0x10dd, 0x10d9, 0x10d0, 0x10e1, 0x10d0, 0x10d1, 0x10da, 0x10d0, 0x10dc, 0x10d9, 0x10d0,
0x10e1, 0x10d4, 0x10e3, 0x10e2, 0x10d0, 0x10d9, 0x10dd, 0x10dc, 0x10d0, 0x10d9, 0x10e0, 0x10d8,
0x10d3, 0x10d0, 0x10d9, 0x10d0, 0x10e0, 0x10d8, 0x10d3, 0x10d0, 0x10e0, 0x2d, 0x10d4, 0x10e1,
0x2d, 0x10e1, 0x10d0, 0x10da, 0x10d0, 0x10db, 0x10d8, 0x10ef, 0x10d8, 0x10d1, 0x10e3, 0x10e2,
0x10d8, 0x10d3, 0x10e3, 0x10d0, 0x10da, 0x10d0, 0x10d4, 0x10da, 0x2d, 0x10d0, 0x10d0, 0x10d8,
0x10e3, 0x10dc, 0x10d8, 0x10e4, 0x10e0, 0x10d8, 0x10e2, 0x10d0, 0x10e3, 0x10dc, 0x10d8, 0x10d2,
0x10d0, 0x10d1, 0x10dd, 0x10e0, 0x10dd, 0x10dc, 0x10d4, 0x10f0, 0x10d0, 0x10e0, 0x10d0, 0x10e0,
0x10d4, 0x10d8, 0x10dd, 0x10f0, 0x10d0, 0x10dc, 0x10d4, 0x10e1, 0x10d1, 0x10e3, 0x10e0, 0x10d2,
0x10d8, 0x10ef, 0x10e3, 0x10d1, 0x10d0, 0x10d9, 0x10d0, 0x10db, 0x10de, 0x10d0, 0x10da, 0x10d0,
0x10ee, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10db, 0x10d8, 0x10d9, 0x10d8, 0x10d2, 0x10d0, 0x10da,
0x10d8, 0x10d9, 0x10d8, 0x10dc, 0x10e8, 0x10d0, 0x10e1, 0x10d0, 0x10da, 0x10d0, 0x10d2, 0x10dd,
0x10e1, 0x10d8, 0x10da, 0x10d8, 0x10d1, 0x10e0, 0x10d4, 0x10d5, 0x10d8, 0x10da, 0x10d8, 0x10da,
0x10dd, 0x10db, 0x10d4, 0x10da, 0x10e3, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10da, 0x10e3, 0x10d1,
0x10e3, 0x10db, 0x10d1, 0x10d0, 0x10e8, 0x10d8, 0x10da, 0x10e3, 0x10e1, 0x10d0, 0x10d9, 0x10d0,
0x10db, 0x10d0, 0x10da, 0x10d0, 0x10d1, 0x10dd, 0x10db, 0x10d0, 0x10de, 0x10e3, 0x10e2, 0x10dd,
0x10db, 0x10d0, 0x10e1, 0x10d4, 0x10e0, 0x10e3, 0x10db, 0x10d1, 0x10d0, 0x10d1, 0x10d0, 0x10dc,
0x10d4, 0x10db, 0x10dd, 0x10d2, 0x10d0, 0x10d3, 0x10d8, 0x10e8, 0x10e3, 0x10db, 0x10dd, 0x10dc,
0x10e0, 0x10dd, 0x10d5, 0x10d8, 0x10d0, 0x10dc, 0x10d0, 0x10d8, 0x10e0, 0x10dd, 0x10d1, 0x10d8,
0x10dc, 0x10ef, 0x10d0, 0x10db, 0x10d4, 0x10dc, 0x10d0, 0x10dc, 0x10d8, 0x10d0, 0x10db, 0x10d4,
0x10d8, 0x10dc, 0x10e3, 0x10d0, 0x10d9, 0x10e8, 0x10dd, 0x10e2, 0x10d8, 0x10e3, 0x10d0, 0x10d2,
0x10d0, 0x10d3, 0x10e3, 0x10d2, 0x10e3, 0x10de, 0x10dd, 0x10e0, 0x10e2, 0x10dd, 0x2d, 0x10dc,
0x10dd, 0x10d5, 0x10dd, 0x10e1, 0x10d4, 0x10dc, 0x2d, 0x10e2, 0x10dd, 0x10db, 0x10d4, 0x10e2,
0x10e0, 0x10d8, 0x10de, 0x10dd, 0x10da, 0x10d8, 0x10e2, 0x10e3, 0x10dc, 0x10d8, 0x10e1, 0x10d8,
0x10d5, 0x10d8, 0x10dc, 0x10d3, 0x10ee, 0x10e3, 0x10d9, 0x10d8, 0x10d0, 0x10d3, 0x10d0, 0x10d9,
0x10d8, 0x10d4, 0x10dc, 0x10e5, 0x10dd, 0x10e0, 0x10d0, 0x10ef, 0x10d8, 0x10d0, 0x10dc, 0x10d2,
0x10d8, 0x10da, 0x10d8, 0x10d0, 0x10d0, 0x10dc, 0x10e2, 0x10d8, 0x10d2, 0x10e3, 0x10d0, 0x10d0,
0x10e0, 0x10d0, 0x10d2, 0x10e3, 0x10d0, 0x10d8, 0x10dc, 0x10d0, 0x10d1, 0x10e3, 0x10d4, 0x10dc,
0x10dd, 0x10e1, 0x2d, 0x10d0, 0x10d8, 0x10e0, 0x10d4, 0x10e1, 0x10d8, 0x10d9, 0x10d0, 0x10e2,
0x10d0, 0x10db, 0x10d0, 0x10e0, 0x10d9, 0x10d0, 0x10d9, 0x10dd, 0x10e0, 0x10d3, 0x10dd, 0x10d1,
0x10d0, 0x10ef, 0x10e3, 0x10ef, 0x10e3, 0x10d8, 0x10da, 0x10d0, 0x20, 0x10e0, 0x10d8, 0x10dd,
0x10ee, 0x10d0, 0x10db, 0x10d4, 0x10dc, 0x10d3, 0x10dd, 0x10d6, 0x10d0, 0x10e0, 0x10d8, 0x10dd,
0x20, 0x10d2, 0x10d0, 0x10da, 0x10d4, 0x10d2, 0x10dd, 0x10e1, 0x10d8, 0x10e1, 0x10d0, 0x10da,
0x10e2, 0x10d0, 0x10e1, 0x10d0, 0x10dc, 0x20, 0x10ee, 0x10e3, 0x10d0, 0x10dc, 0x10d8, 0x10e1,
0x10d0, 0x10dc, 0x2d, 0x10da, 0x10e3, 0x10d8, 0x10e1, 0x10d8, 0x10e2, 0x10e3, 0x10d9, 0x10e3,
0x10db, 0x10d0, 0x10dc, 0x10d8, 0x10e3, 0x10e8, 0x10e3, 0x10d0, 0x10d8, 0x10d0, 0x10d0, 0x10e0,
0x10e3, 0x10d1, 0x10d0, 0x10d0, 0x10e1, 0x10e3, 0x10dc, 0x10e1, 0x10d8, 0x10dd, 0x10dc, 0x10d8,
0x10d0, 0x10e2, 0x10d8, 0x10d9, 0x10dd, 0x10d9, 0x10d0, 0x10dc, 0x10d8, 0x10d1, 0x10d0, 0x10d8,
0x10d0, 0x10d1, 0x10d0, 0x10f0, 0x10d8, 0x10d0, 0x20, 0x10d1, 0x10d0, 0x10dc, 0x10d3, 0x10d4,
0x10e0, 0x10d0, 0x10e1, 0x10d8, 0x10d1, 0x10d0, 0x10e0, 0x10d1, 0x10d0, 0x10d3, 0x10dd, 0x10e1,
0x10d8, 0x10d1, 0x10d4, 0x10da, 0x10d4, 0x10db, 0x10d8, 0x10d1, 0x10d4, 0x10da, 0x10d8, 0x10d6,
0x10d8, 0x10d1, 0x10da, 0x10d0, 0x10dc, 0x2d, 0x10e1, 0x10d0, 0x10d1, 0x10da, 0x10dd, 0x10dc,
0x10d8, 0x10d1, 0x10dd, 0x10d0, 0x20, 0x10d5, 0x10d8, 0x10e1, 0x10e2, 0x10d0, 0x10d1, 0x10dd,
0x10d2, 0x10dd, 0x10e2, 0x10d0, 0x10d1, 0x10e3, 0x10d0, 0x10e1, 0x10d8, 0x10d9, 0x10d4, 0x10db,
0x10d1, 0x10e0, 0x10d8, 0x10ef, 0x20, 0x10d1, 0x10d4, 0x10d8, 0x10d9, 0x10d0, 0x10db, 0x10de,
0x10dd, 0x20, 0x10d2, 0x10e0, 0x10d0, 0x10dc, 0x10d3, 0x10d4, 0x10d9, 0x10d0, 0x10dc, 0x10d9,
0x10e3, 0x10dc, 0x10d8, 0x10d9, 0x10d0, 0x10e0, 0x10d0, 0x10d9, 0x10d0, 0x10e1, 0x10d8, 0x10d9,
0x10d0, 0x10d8, 0x10d4, 0x10dc, 0x10d0, 0x10d9, 0x10d0, 0x10d8, 0x10db, 0x10d0, 0x10dc, 0x10d8,
0x10e9, 0x10d8, 0x10d9, 0x10d0, 0x10d2, 0x10dd, 0x10e9, 0x10d8, 0x10f0, 0x10e3, 0x10d0, 0x10f0,
0x10e3, 0x10d0, 0x10e1, 0x10d8, 0x10e3, 0x10d3, 0x10d0, 0x10d3, 0x2d, 0x10ee, 0x10e3, 0x10d0,
0x10e0, 0x10d4, 0x10e1, 0x10d8, 0x10d9, 0x10dd, 0x10e1, 0x10e2, 0x10d0, 0x2d, 0x10e0, 0x10d8,
0x10d9, 0x10d0, 0x10d9, 0x10e0, 0x10d4, 0x10e1, 0x10e2, 0x10dd, 0x10dc, 0x10d8, 0x10d9, 0x10e3,
0x10d8, 0x10d0, 0x10d1, 0x10d0, 0x10d9, 0x10d8, 0x10e3, 0x10e0, 0x10d0, 0x10e1, 0x10d0, 0x10dd,
0x10d3, 0x10d4, 0x10dc, 0x10db, 0x10d0, 0x10e0, 0x10d9, 0x10e8, 0x10d0, 0x10d5, 0x10dc, 0x10d8,
0x10d3, 0x10dd, 0x10e3, 0x10e1, 0x10dd, 0x10dc, 0x10d8, 0x10d3, 0x10dd, 0x10e3, 0x10e1, 0x10dd,
0x10dc, 0x20, 0x10e5, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10d3, 0x10d4, 0x10dc, 0x10d5, 0x10d4,
0x10e0, 0x10d8, 0x10d3, 0x10d4, 0x10e2, 0x10e0, 0x10dd, 0x10d8, 0x10e2, 0x10d8, 0x10d3, 0x10dd,
0x10db, 0x10d8, 0x10dc, 0x10d8, 0x10d9, 0x10d0, 0x10d4, 0x10d3, 0x10db, 0x10dd, 0x10dc, 0x10e2,
0x10dd, 0x10dc, 0x10d8, 0x10d4, 0x10d8, 0x10e0, 0x10e3, 0x10dc, 0x10d4, 0x10de, 0x10d4, 0x10e1,
0x10d0, 0x10da, 0x10d5, 0x10d0, 0x10d3, 0x10dd, 0x10e0, 0x10d8, 0x10e4, 0x10dd, 0x10e0, 0x10e2,
0x2d, 0x10dc, 0x10d4, 0x10da, 0x10e1, 0x10dd, 0x10dc, 0x10d8, 0x10e4, 0x10dd, 0x10e0, 0x10e2,
0x10d0, 0x10da, 0x10d4, 0x10d6, 0x10d0, 0x10d2, 0x10da, 0x10d4, 0x10d8, 0x10e1, 0x20, 0x10d1,
0x10d4, 0x10d8, 0x10d2, 0x10e3, 0x10d6, 0x20, 0x10d1, 0x10d4, 0x10d8, 0x10d2, 0x10e0, 0x10d0,
0x10dc, 0x10d3, 0x2d, 0x10e2, 0x10e3, 0x10e0, 0x10d9, 0x10d8, 0x10d2, 0x10e0, 0x10d4, 0x10dc,
0x10d0, 0x10d3, 0x10d0, 0x10d2, 0x10d5, 0x10d0, 0x10d3, 0x10d4, 0x10da, 0x10e3, 0x10de, 0x10d0,
0x10d2, 0x10d5, 0x10d0, 0x10e2, 0x10d4, 0x10db, 0x10d0, 0x10da, 0x10d0, 0x10d2, 0x10e3, 0x10d0,
0x10d8, 0x10d0, 0x10d9, 0x10d8, 0x10da, 0x10d8, 0x10d2, 0x10d0, 0x10d8, 0x10d0, 0x10dc, 0x10d0,
0x10f0, 0x10d0, 0x10da, 0x10d8, 0x10e4, 0x10d0, 0x10e5, 0x10e1, 0x10d8, 0x10f0, 0x10d0, 0x10d5,
0x10d0, 0x10dc, 0x10d0, 0x10f0, 0x10d4, 0x10e0, 0x10db, 0x10dd, 0x10e1, 0x10d8, 0x10da, 0x10dd,
0x10d8, 0x10dc, 0x10d3, 0x10d8, 0x10d0, 0x10dc, 0x10d0, 0x10de, 0x10dd, 0x10da, 0x10d8, 0x10e1,
0x10d8, 0x10dc, 0x10dd, 0x10e5, 0x10e1, 0x10d8, 0x2c, 0x20, 0x10d8, 0x10dc, 0x10d3, 0x10d8,
0x10d0, 0x10dc, 0x10d0, 0x10db, 0x10d0, 0x10e0, 0x10d4, 0x10dc, 0x10d2, 0x10dd, 0x2c, 0x20,
0x10d8, 0x10dc, 0x10d3, 0x10d8, 0x10d0, 0x10dc, 0x10d0, 0x10de, 0x10d8, 0x10e2, 0x10d4, 0x10e0,
0x10e1, 0x10d1, 0x10e3, 0x10e0, 0x10d2, 0x10d8, 0x2c, 0x20, 0x10d8, 0x10dc, 0x10d3, 0x10d8,
0x10d0, 0x10dc, 0x10d0, 0x10d7, 0x10d4, 0x10da, 0x20, 0x10e1, 0x10d8, 0x10d7, 0x10d8, 0x2c,
0x20, 0x10d8, 0x10dc, 0x10d3, 0x10d8, 0x10d0, 0x10dc, 0x10d0, 0x10d5, 0x10d8, 0x10d5, 0x10d4,
0x10d8, 0x2c, 0x20, 0x10d8, 0x10dc, 0x10d3, 0x10d8, 0x10d0, 0x10dc, 0x10d0, 0x10d5, 0x10d8,
0x10dc, 0x10e1, 0x10d4, 0x10dc, 0x10d8, 0x2c, 0x20, 0x10d8, 0x10dc, 0x10d3, 0x10d8, 0x10d0,
0x10dc, 0x10d0, 0x10e3, 0x10d8, 0x10dc, 0x10d4, 0x10db, 0x10d0, 0x10d9, 0x10d8, 0x2c, 0x20,
0x10d8, 0x10dc, 0x10d3, 0x10d8, 0x10d0, 0x10dc, 0x10d0, 0x10d8, 0x10dc, 0x10e3, 0x10d5, 0x10d8,
0x10d9, 0x10d8, 0x10d8, 0x10e5, 0x10d0, 0x10da, 0x10e3, 0x10d8, 0x10e2, 0x10d8, 0x10d8, 0x10d0,
0x10db, 0x10d0, 0x10d8, 0x10d9, 0x10d0, 0x10ef, 0x10e3, 0x10dc, 0x10dd, 0x10da, 0x10e3, 0x10d8,
0x10e1, 0x10d5, 0x10d8, 0x10da, 0x10d8, 0x10db, 0x10dd, 0x10dc, 0x10e2, 0x10d8, 0x10e9, 0x10d4,
0x10da, 0x10dd, 0x2c, 0x20, 0x10d9, 0x10d4, 0x10dc, 0x10e2, 0x10e3, 0x10d9, 0x10d8, 0x10d9,
0x10e0, 0x10d0, 0x10da, 0x10d4, 0x10dc, 0x10d3, 0x10d8, 0x10d8, 0x10d9, 0x10d8, 0x10da, 0x10d0,
0x2d, 0x10de, 0x10d0, 0x10d6, 0x10d8, 0x10da, 0x10d8, 0x10db, 0x10d0, 0x10da, 0x10dd, 0x10e1,
0x2d, 0x10d0, 0x10dc, 0x10ef, 0x10d4, 0x10da, 0x10d4, 0x10e1, 0x10d8, 0x10da, 0x10dd, 0x10d4,
0x10e0, 0x20, 0x10de, 0x10e0, 0x10d8, 0x10dc, 0x10ea, 0x20, 0x10d9, 0x10d5, 0x10dd, 0x10e0,
0x10e2, 0x10d4, 0x10e0, 0x10d8, 0x10db, 0x10d0, 0x10e1, 0x10d4, 0x10d8, 0x10dd, 0x10db, 0x10d0,
0x10dc, 0x10d0, 0x10d2, 0x10e3, 0x10d0, 0x10db, 0x10d0, 0x10dc, 0x10d0, 0x10e3, 0x10e1, 0x10d8,
0x10db, 0x10d0, 0x10e0, 0x10d8, 0x10d2, 0x10dd, 0x10db, 0x10d0, 0x10e0, 0x10e2, 0x10d8, 0x10dc,
0x10d8, 0x10d9, 0x10d8, 0x10db, 0x10d0, 0x10e2, 0x10d0, 0x10db, 0x10dd, 0x10e0, 0x10dd, 0x10e1,
0x10d8, 0x10db, 0x10d0, 0x10d6, 0x10d0, 0x10e2, 0x10da, 0x10d0, 0x10dc, 0x10d8, 0x10db, 0x10d4,
0x10dc, 0x10dd, 0x10db, 0x10d8, 0x10dc, 0x10d8, 0x10db, 0x10d4, 0x10e0, 0x10d8, 0x10d3, 0x10d0,
0x10db, 0x10d4, 0x10e2, 0x10da, 0x10d0, 0x10d9, 0x10d0, 0x10e2, 0x10da, 0x10d0, 0x10db, 0x10d4,
0x10ee, 0x10d8, 0x10d9, 0x10dd, 0x10db, 0x10d8, 0x10d9, 0x10d4, 0x10da, 0x10dd, 0x10dc, 0x10d8,
0x10db, 0x10dd, 0x10dc, 0x10e5, 0x10e2, 0x10dd, 0x10dc, 0x10d8, 0x10db, 0x10dd, 0x10dc, 0x10e2,
0x10d4, 0x10e0, 0x10d4, 0x10d8, 0x10db, 0x10dd, 0x10dc, 0x10e2, 0x10d4, 0x10d5, 0x10d8, 0x10d3,
0x10d4, 0x10dd, 0x10db, 0x10dd, 0x10dc, 0x10e1, 0x10d4, 0x10e0, 0x10d0, 0x10e2, 0x10d8, 0x10dc,
0x10d0, 0x10e1, 0x10d0, 0x10e3, 0x10dc, 0x10d8, 0x10e3, 0x2d, 0x10d8, 0x10dd, 0x10e0, 0x10d9,
0x10d8, 0x10dc, 0x10dd, 0x10e0, 0x10dd, 0x10dc, 0x10d8, 0x10d0, 0x10d1, 0x10d8, 0x10e3, 0x10da,
0x10d0, 0x2c, 0x20, 0x10e9, 0x10e0, 0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7, 0x10d8,
0x20, 0x10d3, 0x10d0, 0x10d9, 0x10dd, 0x10e2, 0x10d0, 0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0,
0x10d8, 0x2c, 0x20, 0x10e9, 0x10e0, 0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7, 0x10d8,
0x20, 0x10d3, 0x10d0, 0x10d9, 0x10dd, 0x10e2, 0x10d0, 0x10dc, 0x10d8, 0x10e3, 0x2d, 0x10e1,
0x10d0, 0x10da, 0x10d4, 0x10db, 0x10d8, 0x2c, 0x20, 0x10e9, 0x10e0, 0x10d3, 0x10d8, 0x10da,
0x10dd, 0x10d4, 0x10d7, 0x10d8, 0x20, 0x10d3, 0x10d0, 0x10d9, 0x10dd, 0x10e2, 0x10d0, 0x10d2,
0x10dd, 0x10d3, 0x10d7, 0x10d4, 0x10d1, 0x10d8, 0x10dd, 0x10ee, 0x10d8, 0x10dc, 0x10d0, 0x10d2,
0x10d0, 0x10de, 0x10d0, 0x10dc, 0x10d0, 0x10db, 0x10d0, 0x10de, 0x10d0, 0x10e0, 0x10d0, 0x10db,
0x10d0, 0x10e0, 0x10d8, 0x10d1, 0x10dd, 0x10e4, 0x10d4, 0x10dc, 0x10d8, 0x10e5, 0x10e1, 0x10d8,
0x10de, 0x10dd, 0x10e0, 0x10e2, 0x2d, 0x10d0, 0x2d, 0x10de, 0x10e0, 0x10d8, 0x10dc, 0x10e1,
0x10d8, 0x10de, 0x10dd, 0x10e0, 0x10e2, 0x2d, 0x10dd, 0x10d5, 0x2d, 0x10e1, 0x10de, 0x10d4,
0x10d8, 0x10dc, 0x10d8, 0x10de, 0x10dd, 0x10e0, 0x10e2, 0x10e3, 0x2d, 0x10d5, 0x10d4, 0x10da,
0x10d8, 0x10dd, 0x10de, 0x10e3, 0x10d4, 0x10e0, 0x10e2, 0x10dd, 0x2d, 0x10e0, 0x10d8, 0x10d9,
0x10dd, 0x10de, 0x10e3, 0x10dc, 0x10e2, 0x10d0, 0x2d, 0x10d0, 0x10e0, 0x10d4, 0x10dc, 0x10d0,
0x10e1, 0x10d8, 0x10e0, 0x10d4, 0x10dc, 0x10d9, 0x10d8, 0x10dc, 0x20, 0x10d8, 0x10dc, 0x10da,
0x10d4, 0x10e2, 0x10d8, 0x10e0, 0x10d4, 0x10ea, 0x10d8, 0x10e4, 0x10d4, 0x10e0, 0x10d4, 0x10ef,
0x10d8, 0x10dc, 0x10d0, 0x10e0, 0x10d4, 0x10d6, 0x10dd, 0x10da, 0x10e3, 0x10e2, 0x10d4, 0x10e0,
0x10d8, 0x10dd, 0x20, 0x10d1, 0x10e0, 0x10d0, 0x10dc, 0x10d9, 0x10dd, 0x10e1, 0x10d0, 0x10dc,
0x10e2, 0x10d0, 0x10e0, 0x10d4, 0x10db, 0x10d8, 0x10e1, 0x10d0, 0x10dc, 0x10e2, 0x10d8, 0x10d0,
0x10d2, 0x10dd, 0x10e1, 0x10d0, 0x10dc, 0x10e2, 0x10dd, 0x2d, 0x10d3, 0x10dd, 0x10db, 0x10d8,
0x10dc, 0x10d2, 0x10dd, 0x10e1, 0x10d0, 0x10dc, 0x2d, 0x10de, 0x10d0, 0x10e3, 0x10da, 0x10e3,
0x10e1, 0x10d9, 0x10dd, 0x10e0, 0x10e1, 0x10d1, 0x10d8, 0x10e1, 0x10e3, 0x10dc, 0x10d3, 0x10d8,
0x10e1, 0x10d8, 0x10e2, 0x10d9, 0x10d0, 0x10e1, 0x10d4, 0x10dc, 0x2d, 0x10d1, 0x10d0, 0x10e0,
0x10d7, 0x10d4, 0x10da, 0x10d4, 0x10db, 0x10d8, 0x10e1, 0x10d4, 0x10dc, 0x10e2, 0x2d, 0x10ef,
0x10dd, 0x10dc, 0x10e1, 0x10d8, 0x10e1, 0x10d4, 0x10dc, 0x10e2, 0x2d, 0x10d9, 0x10d8, 0x10e2,
0x10e1, 0x10d8, 0x10e1, 0x10d4, 0x10dc, 0x10e2, 0x2d, 0x10da, 0x10e3, 0x10e1, 0x10d8, 0x10d0,
0x10e1, 0x10d4, 0x10dc, 0x2d, 0x10e2, 0x10dd, 0x10db, 0x10d0, 0x10e1, 0x10d8, 0x10e1, 0x10d4,
0x10dc, 0x2d, 0x10d5, 0x10d8, 0x10dc, 0x10e1, 0x10d4, 0x10dc, 0x10e2, 0x10d8, 0x10e1, 0x10d5,
0x10d8, 0x10e4, 0x10e2, 0x20, 0x10d9, 0x10d0, 0x10e0, 0x10d4, 0x10dc, 0x10e2, 0x10d8, 0x10e2,
0x10d4, 0x10d2, 0x10e3, 0x10e9, 0x10d8, 0x10d2, 0x10d0, 0x10da, 0x10de, 0x10d0, 0x10d7, 0x10e3,
0x10da, 0x10d4, 0x10e2, 0x10d8, 0x10ee, 0x10e3, 0x10d0, 0x10dc, 0x10d0, 0x10e2, 0x10dd, 0x10e0,
0x10dd, 0x10dc, 0x10e2, 0x10dd, 0x10e2, 0x10dd, 0x10e0, 0x10e2, 0x10dd, 0x10da, 0x10d0, 0x10d5,
0x10d0, 0x10dc, 0x10d9, 0x10e3, 0x10d5, 0x10d4, 0x10e0, 0x10d8, 0x10e3, 0x10d0, 0x10d8, 0x10d7,
0x10f0, 0x10dd, 0x10e0, 0x10e1, 0x10d8, 0x10e3, 0x10d8, 0x10dc, 0x10d8, 0x10de, 0x10d4, 0x10d2,
0x10d8, 0x10d8, 0x10d0, 0x10d9, 0x10e3, 0x10e2, 0x10d0, 0x10e2, 0x10d8, 0x10d9, 0x10d4, 0x10d8,
0x10e1, 0x10d8, 0x10d3, 0x10d4, 0x10d5, 0x10d8, 0x10e1, 0x10d8, 0x10d3, 0x10d8, 0x10e3, 0x10db,
0x10dd, 0x10dc, 0x20, 0x10d3, 0x2019, 0x10e3, 0x10e0, 0x10d5, 0x10d8, 0x10da, 0x10d8, 0x10db,
0x10d4, 0x10e5, 0x10d5, 0x10dd, 0x10e0, 0x10d8, 0x10db, 0x10dd, 0x10e3, 0x10e1, 0x10dd, 0x10dc,
0x10d8, 0x10db, 0x10d0, 0x10d9, 0x10db, 0x10e3, 0x10e0, 0x10d3, 0x10dd, 0x10de, 0x10d0, 0x10da,
0x10db, 0x10d4, 0x10e0, 0x10d8, 0x10e0, 0x10dd, 0x10d7, 0x10d4, 0x10e0, 0x10d0, 0x10e1, 0x10d8,
0x10dd, 0x10e3, 0x10d0, 0x10e2, 0x10e0, 0x10dd, 0x10da, 0x10d8, 0x10d5, 0x10dd, 0x10e1, 0x10e2,
0x10dd, 0x10d9, 0x10d8, 0x10da, 0x10dd, 0x10dc, 0x10d2, 0x10d8, 0x10e0, 0x10d1, 0x10d8, 0x10d4,
0x10dc, 0x10d8, 0x10d0, 0x10d3, 0x10d4, 0x10dc, 0x10d8, 0x10d0, 0x10da, 0x10db, 0x10d0, 0x10e2,
0x10d8, 0x10d0, 0x10db, 0x10d0, 0x10dc, 0x10d8, 0x10d0, 0x10dc, 0x10d0, 0x10d3, 0x10d8, 0x10e0,
0x10d8, 0x10d0, 0x10e5, 0x10e2, 0x10d0, 0x10e3, 0x10d0, 0x10e5, 0x10e2, 0x10dd, 0x10d1, 0x10d4,
0x10d0, 0x10e8, 0x10ee, 0x10d0, 0x10d1, 0x10d0, 0x10d3, 0x10d8, 0x10d0, 0x10e2, 0x10d8, 0x10e0,
0x10d0, 0x10e3, 0x10d1, 0x10d0, 0x10e6, 0x10d3, 0x10d0, 0x10d3, 0x10d8, 0x10d1, 0x10d0, 0x10f0,
0x10e0, 0x10d4, 0x10d8, 0x10dc, 0x10d8, 0x10d1, 0x10d0, 0x10e5, 0x10dd, 0x10d1, 0x10d0, 0x10dc,
0x10d2, 0x10d9, 0x10dd, 0x10d9, 0x10d8, 0x10d1, 0x10d0, 0x10e0, 0x10dc, 0x10d0, 0x10e3, 0x10da,
0x10d8, 0x10d1, 0x10d4, 0x10d8, 0x10e0, 0x10e3, 0x10d7, 0x10d8, 0x10d1, 0x10d8, 0x10e8, 0x10d9,
0x10d4, 0x10d9, 0x10d8, 0x10d1, 0x10e0, 0x10e3, 0x10dc, 0x10d4, 0x10d8, 0x10e9, 0x10d8, 0x10e2,
0x10d0, 0x10d9, 0x10dd, 0x10da, 0x10dd, 0x10db, 0x10d1, 0x10dd, 0x10d3, 0x10d0, 0x10db, 0x10d0,
0x10e1, 0x10d9, 0x10d8, 0x10d3, 0x10d8, 0x10da, 0x10d8, 0x10d3, 0x10e3, 0x10d1, 0x10d0, 0x10d8,
0x10d3, 0x10e3, 0x10e8, 0x10d0, 0x10dc, 0x10d1, 0x10d4, 0x10e4, 0x10d0, 0x10db, 0x10d0, 0x10d2,
0x10e3, 0x10e1, 0x10e2, 0x10d0, 0x10d2, 0x10d0, 0x10d6, 0x10d0, 0x10f0, 0x10d4, 0x10d1, 0x10e0,
0x10dd, 0x10dc, 0x10d8, 0x10f0, 0x10dd, 0x10e9, 0x10d8, 0x10db, 0x10d8, 0x10dc, 0x10d8, 0x10f0,
0x10dd, 0x10dc, 0x10d9, 0x10dd, 0x10dc, 0x10d2, 0x10d8, 0x10f0, 0x10dd, 0x10d5, 0x10d3, 0x10d8,
0x10d8, 0x10e0, 0x10d9, 0x10e3, 0x10e2, 0x10e1, 0x10d9, 0x10d8, 0x10ef, 0x10d0, 0x10d9, 0x10d0,
0x10e0, 0x10e2, 0x10d0, 0x10ef, 0x10d0, 0x10d8, 0x10d0, 0x10de, 0x10e3, 0x10e0, 0x10d0, 0x10d8,
0x10d4, 0x10e0, 0x10e3, 0x10e1, 0x10d0, 0x10da, 0x10d8, 0x10db, 0x10d8, 0x10e5, 0x10d0, 0x10d1,
0x10e3, 0x10da, 0x10d8, 0x10d9, 0x10d0, 0x10db, 0x10e9, 0x10d0, 0x10e2, 0x10d9, 0x10d0, 0x10d9,
0x10d0, 0x10e0, 0x10d0, 0x10e9, 0x10d8, 0x10d9, 0x10d0, 0x10e2, 0x10db, 0x10d0, 0x10dc, 0x10d3,
0x10e3, 0x10ee, 0x10d0, 0x10dc, 0x10d3, 0x10d8, 0x10d2, 0x10d0, 0x10d9, 0x10d0, 0x10da, 0x10d9,
0x10e3, 0x10e2, 0x10d0, 0x10d9, 0x10e0, 0x10d0, 0x10e1, 0x10dc, 0x10dd, 0x10d8, 0x10d0, 0x10e0,
0x10e1, 0x10d9, 0x10d8, 0x10d9, 0x10e3, 0x10d0, 0x10da, 0x10d0, 0x2d, 0x10da, 0x10e3, 0x10db,
0x10de, 0x10e3, 0x10e0, 0x10d8, 0x10d9, 0x10e3, 0x10e9, 0x10d8, 0x10dc, 0x10d2, 0x10d8, 0x10e5,
0x10e3, 0x10d5, 0x10d4, 0x10d8, 0x10d7, 0x10d8, 0x10db, 0x10d0, 0x10d9, 0x10d0, 0x10dd, 0x10db,
0x10d0, 0x10d2, 0x10d0, 0x10d3, 0x10d0, 0x10dc, 0x10d8, 0x10db, 0x10d0, 0x10d9, 0x10d0, 0x10e1,
0x10d0, 0x10e0, 0x10d8, 0x10db, 0x10d0, 0x10dc, 0x10d8, 0x10da, 0x10d0, 0x10db, 0x10e3, 0x10e1,
0x10d9, 0x10d0, 0x10e2, 0x10d8, 0x10dc, 0x10d8, 0x10d9, 0x10dd, 0x10e1, 0x10d8, 0x10d0, 0x10dc,
0x10dd, 0x10d5, 0x10dd, 0x10d9, 0x10e3, 0x10d6, 0x10dc, 0x10d4, 0x10ea, 0x10d9, 0x10d8, 0x10dc,
0x10dd, 0x10d5, 0x10dd, 0x10e1, 0x10d8, 0x10d1, 0x10d8, 0x10e0, 0x10e1, 0x10d9, 0x10d8, 0x10dd,
0x10db, 0x10e1, 0x10d9, 0x10d8, 0x10dd, 0x10e0, 0x10d0, 0x10da, 0x10d8, 0x10de, 0x10dc, 0x10dd,
0x10db, 0x10de, 0x10d4, 0x10dc, 0x10d8, 0x10de, 0x10dd, 0x10dc, 0x10e2, 0x10d8, 0x10d0, 0x10dc,
0x10d0, 0x10d9, 0x10d8, 0x10e4, 0x10ee, 0x10d4, 0x10dc, 0x10d8, 0x10d0, 0x10dc, 0x10d8, 0x10d9,
0x10d0, 0x10e2, 0x10d0, 0x10e0, 0x10d8, 0x10d9, 0x10dd, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d0,
0x10d8, 0x10e7, 0x10d8, 0x10d6, 0x10d8, 0x10da, 0x10dd, 0x10e0, 0x10d3, 0x10d0, 0x10d4, 0x10e0,
0x2d, 0x10e0, 0x10d8, 0x10d0, 0x10d3, 0x10d8, 0x10e1, 0x10d0, 0x10ee, 0x10d0, 0x10da, 0x10d8,
0x10dc, 0x10d8, 0x10e1, 0x10d0, 0x10db, 0x10d0, 0x10e0, 0x10e7, 0x10d0, 0x10dc, 0x10d3, 0x10d8,
0x10e1, 0x10d4, 0x10e3, 0x10da, 0x10d8, 0x10e8, 0x10d0, 0x10dc, 0x10ee, 0x10d0, 0x10d8, 0x10e1,
0x10d8, 0x10dc, 0x10d2, 0x10d0, 0x10de, 0x10e3, 0x10e0, 0x10d8, 0x10e1, 0x10e0, 0x10d4, 0x10d3,
0x10dc, 0x10d4, 0x10d9, 0x10dd, 0x10da, 0x10d8, 0x10db, 0x10e1, 0x10d9, 0x10d8, 0x10e2, 0x10d0,
0x10d8, 0x10de, 0x10d4, 0x10d8, 0x10e2, 0x10d0, 0x10e8, 0x10d9, 0x10d4, 0x10dc, 0x10e2, 0x10d8,
0x10d7, 0x10d1, 0x10d8, 0x10da, 0x10d8, 0x10e1, 0x10d8, 0x10d7, 0x10d4, 0x10d8, 0x10e0, 0x10d0,
0x10dc, 0x10d8, 0x10d7, 0x10ee, 0x10d8, 0x10db, 0x10e4, 0x10ee, 0x10e3, 0x10e2, 0x10dd, 0x10d9,
0x10d8, 0x10dd, 0x10e2, 0x10dd, 0x10db, 0x10e1, 0x10d9, 0x10d8, 0x10e3, 0x10da, 0x10d0, 0x10dc,
0x10d1, 0x10d0, 0x10e2, 0x10d0, 0x10e0, 0x10d8, 0x10e3, 0x10e0, 0x10e3, 0x10db, 0x10e5, 0x10d8,
0x10e3, 0x10e1, 0x10e2, 0x2d, 0x10dc, 0x10d4, 0x10e0, 0x10d0, 0x10d5, 0x10d8, 0x10d4, 0x10dc,
0x10e2, 0x10d8, 0x10d0, 0x10dc, 0x10d8, 0x10d5, 0x10da, 0x10d0, 0x10d3, 0x10d8, 0x10d5, 0x10dd,
0x10e1, 0x10e2, 0x10dd, 0x10d9, 0x10d8, 0x10d8, 0x10d0, 0x10d9, 0x10e3, 0x10e2, 0x10e1, 0x10d9,
0x10d8, 0x10e0, 0x10d0, 0x10dc, 0x10d2, 0x10e3, 0x10dc, 0x10d8, 0x10d4, 0x10d9, 0x10d0, 0x10e2,
0x10d4, 0x10e0, 0x10d8, 0x10dc, 0x10d1, 0x10e3, 0x10e0, 0x10d2, 0x10d8, 0x10d4, 0x10e0, 0x10d4,
0x10d5, 0x10d0, 0x10dc, 0x10d8, 0x10d0, 0x10d6, 0x10dd, 0x10e0, 0x10d8, 0x10e1, 0x20, 0x10d9,
0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10d1, 0x10d4, 0x10e0, 0x10db,
0x10e3, 0x10d3, 0x10d0, 0x10d9, 0x10d0, 0x10dc, 0x10d0, 0x10e0, 0x10d8, 0x10d9, 0x10d0, 0x10d1,
0x10dd, 0x2d, 0x10d5, 0x10d4, 0x10e0, 0x10d3, 0x10d4, 0x10e4, 0x10d0, 0x10e0, 0x10d4, 0x10e0,
0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8,
0x10db, 0x10d0, 0x10d3, 0x10d4, 0x10d8, 0x10e0, 0x10d0, 0x10e0, 0x10d4, 0x10d8, 0x10d9, 0x10d8,
0x10d0, 0x10d5, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x10d0, 0x10db, 0x10ee, 0x10e0, 0x10d4, 0x10d7,
0x20, 0x10ef, 0x10dd, 0x10e0, 0x10ef, 0x10d8, 0x10d0, 0x10ec, 0x10db, 0x2e, 0x20, 0x10d4,
0x10da, 0x10d4, 0x10dc, 0x10d4, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da,
0x10d8, 0x10e1, 0x10e2, 0x10d4, 0x10dc, 0x10da, 0x10d8, 0x10d0, 0x10d3, 0x10d4, 0x10da, 0x10d0,
0x10d8, 0x10d3, 0x10d0, 0x10d1, 0x10e0, 0x10d8, 0x10e1, 0x10d1, 0x10d4, 0x10d8, 0x10dc, 0x10d8,
0x10d1, 0x10e0, 0x10dd, 0x10e3, 0x10d9, 0x10d4, 0x10dc, 0x20, 0x10f0, 0x10d8, 0x10da, 0x10d8,
0x10d3, 0x10d0, 0x10e0, 0x10d5, 0x10d8, 0x10dc, 0x10d8, 0x10d4, 0x10e3, 0x10d9, 0x10da, 0x10d0,
0x10f0, 0x10dd, 0x10d1, 0x10d0, 0x10e0, 0x10e2, 0x10d8, 0x10da, 0x10d8, 0x10dc, 0x10d3, 0x10db,
0x10d0, 0x10dc, 0x10d8, 0x10da, 0x10dd, 0x10e0, 0x10d3, 0x20, 0x10f0, 0x10dd, 0x10e3, 0x10d8,
0x10db, 0x10d4, 0x10da, 0x10d1, 0x10e3, 0x10e0, 0x10dc, 0x10d8, 0x10de, 0x10d4, 0x10e0, 0x10d7,
0x10d8, 0x10e1, 0x10d8, 0x10d3, 0x10dc, 0x10d4, 0x10d8, 0x10d0, 0x10db, 0x10e1, 0x10e2, 0x10d4,
0x10e0, 0x10d3, 0x10d0, 0x10db, 0x10d8, 0x10d0, 0x10dc, 0x10d3, 0x10dd, 0x10e0, 0x10d0, 0x10d0,
0x10e1, 0x10e2, 0x10e0, 0x10d0, 0x10ee, 0x10d0, 0x10dc, 0x10d8, 0x10d0, 0x10d7, 0x10d4, 0x10dc,
0x10d8, 0x10d1, 0x10d4, 0x10da, 0x10d2, 0x10e0, 0x10d0, 0x10d3, 0x10d8, 0x10d1, 0x10d4, 0x10e0,
0x10da, 0x10d8, 0x10dc, 0x10d8, 0x10d1, 0x10e0, 0x10d0, 0x10e2, 0x10d8, 0x10e1, 0x10da, 0x10d0,
0x10d5, 0x10d0, 0x10d1, 0x10e0, 0x10d8, 0x10e3, 0x10e1, 0x10d4, 0x10da, 0x10d8, 0x10d1, 0x10e3,
0x10e5, 0x10d0, 0x10e0, 0x10d4, 0x10e1, 0x10e2, 0x10d8, 0x10d1, 0x10e3, 0x10d3, 0x10d0, 0x10de,
0x10d4, 0x10e8, 0x10e2, 0x10d8, 0x10d1, 0x10e3, 0x10e1, 0x10d8, 0x10dc, 0x10d2, 0x10d4, 0x10dc,
0x10d8, 0x10d9, 0x10d8, 0x10e8, 0x10d8, 0x10dc, 0x10d8, 0x10dd, 0x10d5, 0x10d8, 0x10d9, 0x10dd,
0x10de, 0x10d4, 0x10dc, 0x10f0, 0x10d0, 0x10d2, 0x10d4, 0x10dc, 0x10d8, 0x10d3, 0x10e3, 0x10d1,
0x10da, 0x10d8, 0x10dc, 0x10d8, 0x10d2, 0x10d8, 0x10d1, 0x10e0, 0x10d0, 0x10da, 0x10e2, 0x10d0,
0x10e0, 0x10d8, 0x10d2, 0x10d4, 0x10e0, 0x10dc, 0x10e1, 0x10d8, 0x10f0, 0x10d4, 0x10da, 0x10e1,
0x10d8, 0x10dc, 0x10d9, 0x10d8, 0x10db, 0x10d4, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3,
0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x10e2, 0x10d0, 0x10db, 0x10d1, 0x10dd, 0x10da,
0x10d8, 0x10ef, 0x10d4, 0x10e0, 0x10e1, 0x10d8, 0x10d9, 0x10d0, 0x10da, 0x10d8, 0x10dc, 0x10d8,
0x10dc, 0x10d2, 0x10e0, 0x10d0, 0x10d3, 0x10d8, 0x10d9, 0x10d8, 0x10e0, 0x10dd, 0x10d5, 0x10d8,
0x10d9, 0x10d8, 0x10d4, 0x10d5, 0x10d8, 0x10da, 0x10d8, 0x10e1, 0x10d0, 0x10d1, 0x10dd, 0x10dc,
0x10d8, 0x10da, 0x10d8, 0x10e3, 0x10d1, 0x10da, 0x10d8, 0x10d0, 0x10dc, 0x10d0, 0x10da, 0x10dd,
0x10dc, 0x10d3, 0x10dd, 0x10dc, 0x10d8, 0x10da, 0x10e3, 0x10e5, 0x10e1, 0x10d4, 0x10db, 0x10d1,
0x10e3, 0x10e0, 0x10d2, 0x10d8, 0x10db, 0x10d0, 0x10d3, 0x10e0, 0x10d8, 0x10d3, 0x10d8, 0x10db,
0x10d0, 0x10da, 0x10e2, 0x10d0, 0x10db, 0x10d0, 0x10e0, 0x10d8, 0x10f0, 0x10d0, 0x10db, 0x10dc,
0x10d8, 0x10db, 0x10d8, 0x10dc, 0x10e1, 0x10d9, 0x10d8, 0x10db, 0x10dd, 0x10dc, 0x10d0, 0x10d9,
0x10dd, 0x10db, 0x10dd, 0x10e1, 0x10d9, 0x10dd, 0x10d5, 0x10d8, 0x10dd, 0x10e1, 0x10da, 0x10dd,
0x10de, 0x10d0, 0x10e0, 0x10d8, 0x10d6, 0x10d8, 0x10de, 0x10dd, 0x10d3, 0x10d2, 0x10dd, 0x10e0,
0x10d8, 0x10ea, 0x10d0, 0x10de, 0x10e0, 0x10d0, 0x10e6, 0x10d0, 0x10e0, 0x10d8, 0x10d2, 0x10d0,
0x10e0, 0x10dd, 0x10db, 0x10d8, 0x10e1, 0x10d0, 0x10db, 0x10d0, 0x10e0, 0x10d0, 0x10e1, 0x10d0,
0x10dc, 0x2d, 0x10db, 0x10d0, 0x10e0, 0x10d8, 0x10dc, 0x10dd, 0x10e1, 0x10d0, 0x10e0, 0x10d0,
0x10d4, 0x10d5, 0x10dd, 0x10e1, 0x10d0, 0x10e0, 0x10d0, 0x10e2, 0x10dd, 0x10d5, 0x10d8, 0x10e1,
0x10d8, 0x10db, 0x10e4, 0x10d4, 0x10e0, 0x10dd, 0x10de, 0x10dd, 0x10da, 0x10d8, 0x10e1, 0x10d9,
0x10dd, 0x10de, 0x10d8, 0x10d4, 0x10e1, 0x10dd, 0x10e4, 0x10d8, 0x10d0, 0x10e1, 0x10e2, 0x10dd,
0x10d9, 0x10f0, 0x10dd, 0x10da, 0x10db, 0x10d8, 0x10e2, 0x10d0, 0x10da, 0x10d8, 0x10dc, 0x10d8,
0x10e2, 0x10d8, 0x10e0, 0x10d0, 0x10dc, 0x10d0, 0x10e3, 0x10da, 0x10d8, 0x10d0, 0x10dc, 0x10dd,
0x10d5, 0x10e1, 0x10d9, 0x10d8, 0x10d5, 0x10d0, 0x10d3, 0x10e3, 0x10ea, 0x10d8, 0x10d5, 0x10d0,
0x10e2, 0x10d8, 0x10d9, 0x10d0, 0x10dc, 0x10d8, 0x10d5, 0x10d4, 0x10dc, 0x10d0, 0x10d5, 0x10d8,
0x10da, 0x10dc, 0x10d8, 0x10e3, 0x10e1, 0x10d8, 0x10d5, 0x10dd, 0x10da, 0x10d2, 0x10dd, 0x10d2,
0x10e0, 0x10d0, 0x10d3, 0x10d8, 0x10d5, 0x10d0, 0x10e0, 0x10e8, 0x10d0, 0x10d5, 0x10d0, 0x10d6,
0x10d0, 0x10d2, 0x10e0, 0x10d4, 0x10d1, 0x10d8, 0x10ea, 0x10d8, 0x10e3, 0x10e0, 0x10d8, 0x10ee,
0x10d8, 0x10d0, 0x10dc, 0x10e2, 0x10d0, 0x10dc, 0x10d0, 0x10dc, 0x10d0, 0x10e0, 0x10d8, 0x10d5,
0x10e3, 0x10e9, 0x10d0, 0x10d2, 0x10dd, 0x10e1, 0x10d8, 0x10e8, 0x10dd, 0x10d1, 0x10d8, 0x10e1,
0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d8, 0x10e5, 0x10dd, 0x10e5, 0x10dd,
0x10e1, 0x10d8, 0x10d9, 0x10dd, 0x10db, 0x10dd, 0x10e0, 0x10dd, 0x10d9, 0x10d4, 0x10e0, 0x10d2,
0x10d4, 0x10da, 0x10d4, 0x10dc, 0x10d8, 0x10db, 0x10d0, 0x10f0, 0x10d4, 0x10db, 0x10d0, 0x10da,
0x10d3, 0x10d8, 0x10d5, 0x10d4, 0x10d1, 0x10d8, 0x10db, 0x10d0, 0x10d5, 0x10e0, 0x10d8, 0x10d9,
0x10d8, 0x10db, 0x10d0, 0x10d8, 0x10dd, 0x10e2, 0x10d8, 0x10e0, 0x10d4, 0x10d8, 0x10e3, 0x10dc,
0x10d8, 0x10dd, 0x10dc, 0x10d8, 0x10d0, 0x10de, 0x10d8, 0x10d0, 0x10dd, 0x10d9, 0x10da, 0x10d4,
0x10dc, 0x10d3, 0x10d8, 0x10d1, 0x10e3, 0x10d2, 0x10d4, 0x10dc, 0x10d5, 0x10d8, 0x10da, 0x10d8,
0x10e9, 0x10d0, 0x10d7, 0x10d0, 0x10db, 0x10d8, 0x10e9, 0x10e3, 0x10e3, 0x10d9, 0x10d8, 0x10d8,
0x10e1, 0x10d7, 0x10d4, 0x10e0, 0x10d8, 0x10d4, 0x10e4, 0x10d0, 0x10e2, 0x10d4, 0x10e4, 0x10d0,
0x10d9, 0x10d0, 0x10dd, 0x10e4, 0x10dd, 0x10e4, 0x10d8, 0x10ef, 0x10d8, 0x10e4, 0x10e3, 0x10dc,
0x10d0, 0x10e4, 0x10e3, 0x10e2, 0x10d8, 0x10d2, 0x10d0, 0x10da, 0x10d0, 0x10de, 0x10d0, 0x10d2,
0x10dd, 0x10e1, 0x10d8, 0x10d2, 0x10d0, 0x10db, 0x10d1, 0x10d8, 0x10d4, 0x10e0, 0x10d8, 0x10d2,
0x10d5, 0x10d0, 0x10d3, 0x10d0, 0x10da, 0x10d9, 0x10d0, 0x10dc, 0x10d0, 0x10da, 0x10d8, 0x10d2,
0x10e3, 0x10d0, 0x10db, 0x10d8, 0x10f0, 0x10dd, 0x10dc, 0x10dd, 0x10da, 0x10e3, 0x10da, 0x10e3,
0x10d4, 0x10dc, 0x10d3, 0x10d4, 0x10e0, 0x10d1, 0x10e3, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e0,
0x10d8, 0x10e2, 0x10d8, 0x10db, 0x10d0, 0x10e2, 0x10d8, 0x10d9, 0x10dd, 0x10e1, 0x10e0, 0x10d0,
0x10d4, 0x10d9, 0x10d5, 0x10d0, 0x10ef, 0x10d0, 0x10da, 0x10d4, 0x10d8, 0x10dc, 0x10d8, 0x10db,
0x10d0, 0x10ee, 0x10e3, 0x10e0, 0x10dd, 0x10db, 0x10d0, 0x10e0, 0x10d9, 0x10d4, 0x10d6, 0x10d0,
0x10e1, 0x10d8, 0x10db, 0x10d8, 0x10d3, 0x10e3, 0x10d4, 0x10d8, 0x10dc, 0x10d0, 0x10e3, 0x10e0,
0x10e3, 0x10dc, 0x10d8, 0x10e3, 0x10d4, 0x10dc, 0x10dd, 0x10e0, 0x10e4, 0x10dd, 0x10da, 0x10d9,
0x10d8, 0x10dc, 0x10e3, 0x10db, 0x10d4, 0x10d0, 0x10de, 0x10d0, 0x10d2, 0x10dd, 0x2d, 0x10de,
0x10d0, 0x10d2, 0x10dd, 0x10de, 0x10d0, 0x10da, 0x10d0, 0x10e3, 0x10de, 0x10d8, 0x10e2, 0x10d9,
0x10d4, 0x10e0, 0x10dc, 0x10d8, 0x10de, 0x10dd, 0x10dc, 0x10de, 0x10d4, 0x10d8, 0x10de, 0x10dd,
0x10e0, 0x10e2, 0x20, 0x10db, 0x10dd, 0x10e0, 0x10e1, 0x10d1, 0x10d8, 0x10e0, 0x10d0, 0x10e0,
0x10dd, 0x10e2, 0x10dd, 0x10dc, 0x10d2, 0x10d0, 0x10e1, 0x10d0, 0x10d8, 0x10de, 0x10d0, 0x10dc,
0x10d8, 0x10e2, 0x10d0, 0x10d8, 0x10e2, 0x10d8, 0x10e2, 0x10d0, 0x10e0, 0x10d0, 0x10e3, 0x10d0,
0x10e2, 0x10dd, 0x10dc, 0x10d2, 0x10d0, 0x10e2, 0x10d0, 0x10de, 0x10e3, 0x10e3, 0x10d4, 0x10d8,
0x10d9, 0x10d8, 0x10d5, 0x10d0, 0x10da, 0x10d8, 0x10e1, 0x10d8, 0x44, 0x61, 0x72, 0x65,
0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x44, 0x73, 0x63, 0x68, 0x69, 0x62,
0x75, 0x74, 0x69, 0x4d, 0x6f, 0x67, 0x61, 0x64, 0x69, 0x73, 0x63, 0x68,
0x75, 0x4b, 0x61, 0x69, 0x6d, 0x61, 0x6e, 0x69, 0x6e, 0x73, 0x65, 0x6c,
0x6e, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x6f, 0x2d, 0x53, 0x74, 0x61, 0x64,
0x74, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75,
0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x54, 0x73, 0x63, 0x68, 0x69, 0x74,
0x61, 0x48, 0x6f, 0x2d, 0x43, 0x68, 0x69, 0x2d, 0x4d, 0x69, 0x6e, 0x68,
0x2d, 0x53, 0x74, 0x61, 0x64, 0x74, 0x43, 0x68, 0x6f, 0x77, 0x64, 0x4b,
0x61, 0x6d, 0x74, 0x73, 0x63, 0x68, 0x61, 0x74, 0x6b, 0x61, 0x4b, 0x61,
0x72, 0x61, 0x74, 0x73, 0x63, 0x68, 0x69, 0x4e, 0x6f, 0x77, 0x6f, 0x6b,
0x75, 0x7a, 0x6e, 0x65, 0x74, 0x73, 0x6b, 0x4e, 0x6f, 0x77, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x50, 0x6a, 0xf6, 0x6e, 0x67, 0x6a,
0x61, 0x6e, 0x67, 0x54, 0x61, 0x69, 0x70, 0x65, 0x68, 0x54, 0x69, 0x66,
0x6c, 0x69, 0x73, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x65, 0x6e, 0x46, 0xe4,
0x72, 0xf6, 0x65, 0x72, 0x52, 0x65, 0x79, 0x6b, 0xad, 0x6a, 0x61, 0xad,
0x76, 0xed, 0x6b, 0x53, 0xfc, 0x64, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x65, 0x6e, 0x41, 0x73, 0x74, 0x72, 0x61, 0x63, 0x68, 0x61, 0x6e, 0x4b,
0x69, 0x72, 0x6f, 0x77, 0x4b, 0x69, 0x65, 0x77, 0x53, 0x61, 0x72, 0x61,
0x74, 0x6f, 0x77, 0x55, 0x6c, 0x6a, 0x61, 0x6e, 0x6f, 0x77, 0x73, 0x6b,
0x57, 0x65, 0x69, 0x68, 0x6e, 0x61, 0x63, 0x68, 0x74, 0x73, 0x69, 0x6e,
0x73, 0x65, 0x6c, 0x4b, 0x6f, 0x6d, 0x6f, 0x72, 0x65, 0x6e, 0x4d, 0x61,
0x6c, 0x65, 0x64, 0x69, 0x76, 0x65, 0x6e, 0x4f, 0x73, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x73, 0x65, 0x6c, 0x46, 0x69, 0x64, 0x73, 0x63, 0x68, 0x69,
0x53, 0x61, 0x72, 0x61, 0x74, 0x6f, 0x76, 0x391, 0x3bc, 0x3c0, 0x3b9, 0x3c4,
0x3b6, 0x3ac, 0x3bd, 0x386, 0x3ba, 0x3c1, 0x3b1, 0x391, 0x3bd, 0x3c4, 0x3af, 0x3c2,
0x20, 0x391, 0x3bc, 0x3c0, 0x3ad, 0x3bc, 0x3c0, 0x3b1, 0x391, 0x3bb, 0x3b3, 0x3ad,
0x3c1, 0x3b9, 0x391, 0x3c3, 0x3bc, 0x3ac, 0x3c1, 0x3b1, 0x39c, 0x3c0, 0x3b1, 0x3bc,
0x3ac, 0x3ba, 0x3bf, 0x39c, 0x3c0, 0x3b1, 0x3bd, 0x3b3, 0x3ba, 0x3af, 0x39c, 0x3c0,
0x3b1, 0x3bd, 0x3b6, 0x3bf, 0x3cd, 0x3bb, 0x39c, 0x3c0, 0x3b9, 0x3c3, 0x3ac, 0x3bf,
0x3c5, 0x39c, 0x3c0, 0x3bb, 0x3b1, 0x3bd, 0x3c4, 0x3ac, 0x3b9, 0x3c1, 0x39c, 0x3c0,
0x3c1, 0x3b1, 0x3b6, 0x3b1, 0x3b2, 0x3af, 0x3bb, 0x39c, 0x3c0, 0x3bf, 0x3c5, 0x3b6,
0x3bf, 0x3c5, 0x3bc, 0x3c0, 0x3bf, 0x3cd, 0x3c1, 0x3b1, 0x39a, 0x3ac, 0x3b9, 0x3c1,
0x3bf, 0x39a, 0x3b1, 0x3b6, 0x3b1, 0x3bc, 0x3c0, 0x3bb, 0x3ac, 0x3bd, 0x3ba, 0x3b1,
0x398, 0x3ad, 0x3bf, 0x3c5, 0x3c4, 0x3b1, 0x39a, 0x3cc, 0x3bd, 0x3b1, 0x3ba, 0x3c1,
0x3b9, 0x39d, 0x3c4, 0x3b1, 0x3ba, 0x3ac, 0x3c1, 0x39d, 0x3c4, 0x3b1, 0x3c1, 0x20,
0x3b5, 0x3c2, 0x20, 0x3a3, 0x3b1, 0x3bb, 0x3ac, 0x3bc, 0x3a4, 0x3b6, 0x3b9, 0x3bc,
0x3c0, 0x3bf, 0x3c5, 0x3c4, 0x3af, 0x39d, 0x3c4, 0x3bf, 0x3c5, 0x3ac, 0x3bb, 0x3b1,
0x395, 0x3bb, 0x20, 0x391, 0x3b3, 0x3b9, 0x3bf, 0x3cd, 0x3bd, 0x3a6, 0x3c1, 0x3af,
0x3c4, 0x3b1, 0x3bf, 0x3c5, 0x3bd, 0x393, 0x3ba, 0x3b1, 0x3bc, 0x3c0, 0x3bf, 0x3c1,
0x3cc, 0x3bd, 0x3b5, 0x3a7, 0x3b1, 0x3c1, 0x3ac, 0x3c1, 0x3b5, 0x393, 0x3b9, 0x3bf,
0x3c7, 0x3ac, 0x3bd, 0x3b5, 0x3c3, 0x3bc, 0x3c0, 0x3bf, 0x3c5, 0x3c1, 0x3b3, 0x3ba,
0x3a4, 0x3b6, 0x3bf, 0x3cd, 0x3bc, 0x3c0, 0x3b1, 0x39a, 0x3b1, 0x3bc, 0x3c0, 0x3ac,
0x3bb, 0x3b1, 0x3a7, 0x3b1, 0x3c1, 0x3c4, 0x3bf, 0x3cd, 0x3bc, 0x39a, 0x3b9, 0x3b3,
0x3ba, 0x3ac, 0x3bb, 0x3b9, 0x39a, 0x3b9, 0x3bd, 0x3c3, 0x3ac, 0x3c3, 0x3b1, 0x39b,
0x3ac, 0x3b3, 0x3ba, 0x3bf, 0x3c2, 0x39b, 0x3b9, 0x3bc, 0x3c0, 0x3c1, 0x3b5, 0x3b2,
0x3af, 0x3bb, 0x39b, 0x3bf, 0x3bc, 0x3ad, 0x39b, 0x3bf, 0x3c5, 0x3ac, 0x3bd, 0x3c4,
0x3b1, 0x39b, 0x3bf, 0x3c5, 0x3bc, 0x3c0, 0x3bf, 0x3c5, 0x3bc, 0x3c0, 0x3ac, 0x3c3,
0x3b9, 0x39b, 0x3bf, 0x3c5, 0x3b6, 0x3ac, 0x3ba, 0x3b1, 0x39c, 0x3b1, 0x3bb, 0x3ac,
0x3bc, 0x3c0, 0x3bf, 0x39c, 0x3b1, 0x3c0, 0x3bf, 0x3cd, 0x3c4, 0x3bf, 0x39c, 0x3b1,
0x3c3, 0x3ad, 0x3c1, 0x3bf, 0x3c5, 0x39c, 0x3c0, 0x3b1, 0x3bc, 0x3c0, 0x3ac, 0x3bd,
0x3b5, 0x39c, 0x3bf, 0x3b3, 0x3ba, 0x3b1, 0x3bd, 0x3c4, 0x3af, 0x3c3, 0x3bf, 0x3c5,
0x39c, 0x3bf, 0x3bd, 0x3c1, 0x3cc, 0x3b2, 0x3b9, 0x3b1, 0x39d, 0x3b1, 0x3ca, 0x3c1,
0x3cc, 0x3bc, 0x3c0, 0x3b9, 0x39d, 0x3c4, 0x3b6, 0x3b1, 0x3bc, 0x3ad, 0x3bd, 0x3b1,
0x39d, 0x3b9, 0x3b1, 0x3bc, 0x3ad, 0x3b9, 0x39d, 0x3bf, 0x3c5, 0x3b1, 0x3ba, 0x3c3,
0x3cc, 0x3c4, 0x39f, 0x3c5, 0x3b1, 0x3b3, 0x3ba, 0x3b1, 0x3bd, 0x3c4, 0x3bf, 0x3cd,
0x3b3, 0x3ba, 0x3bf, 0x3c5, 0x3a0, 0x3cc, 0x3c1, 0x3c4, 0x3bf, 0x2d, 0x39d, 0x3cc,
0x3b2, 0x3bf, 0x3a3, 0x3ac, 0x3bf, 0x20, 0x3a4, 0x3bf, 0x3bc, 0x3ad, 0x3a4, 0x3c1,
0x3af, 0x3c0, 0x3bf, 0x3bb, 0x3b7, 0x3a4, 0x3cd, 0x3bd, 0x3b9, 0x3b4, 0x3b1, 0x392,
0x3af, 0x3bd, 0x3c4, 0x3c7, 0x3bf, 0x3c5, 0x3ba, 0x386, 0x3bd, 0x3c4, 0x3b1, 0x3ba,
0x386, 0x3bd, 0x3ba, 0x3bf, 0x3c1, 0x3b1, 0x3c4, 0x3b6, 0x391, 0x3bd, 0x3b3, 0x3ba,
0x3bf, 0x3c5, 0x3af, 0x3bb, 0x3b1, 0x391, 0x3bd, 0x3c4, 0x3af, 0x3b3, 0x3ba, 0x3bf,
0x3c5, 0x3b1, 0x391, 0x3c1, 0x3b1, 0x3b3, 0x3ba, 0x3bf, 0x3c5, 0x3ac, 0x3b9, 0x3bd,
0x3b1, 0x39c, 0x3c0, 0x3bf, 0x3c5, 0x3ad, 0x3bd, 0x3bf, 0x3c2, 0x20, 0x386, 0x3b9,
0x3c1, 0x3b5, 0x3c2, 0x39a, 0x3b1, 0x3c4, 0x3b1, 0x3bc, 0x3ac, 0x3c1, 0x3ba, 0x3b1,
0x39a, 0x3cc, 0x3c1, 0x3b4, 0x3bf, 0x3b2, 0x3b1, 0x3a7, 0x3bf, 0x3c5, 0x3c7, 0x3bf,
0x3cd, 0x3b9, 0x39b, 0x3b1, 0x20, 0x3a1, 0x3b9, 0x3cc, 0x3c7, 0x3b1, 0x39c, 0x3b5,
0x3bd, 0x3c4, 0x3cc, 0x3b6, 0x3b1, 0x3a1, 0x3af, 0x3bf, 0x20, 0x393, 0x3ba, 0x3b1,
0x3b3, 0x3b9, 0x3ad, 0x3b3, 0x3ba, 0x3bf, 0x3c2, 0x3a3, 0x3ac, 0x3bb, 0x3c4, 0x3b1,
0x3a3, 0x3b1, 0x3bd, 0x20, 0x3a7, 0x3bf, 0x3c5, 0x3ac, 0x3bd, 0x3a3, 0x3b1, 0x3bd,
0x20, 0x39b, 0x3bf, 0x3cd, 0x3b9, 0x3c2, 0x3a4, 0x3bf, 0x3c5, 0x3ba, 0x3bf, 0x3c5,
0x3bc, 0x3ac, 0x3bd, 0x39f, 0x3c5, 0x3c3, 0x3bf, 0x3c5, 0x3ac, 0x3b9, 0x3b1, 0x391,
0x3c1, 0x3bf, 0x3cd, 0x3bc, 0x3c0, 0x3b1, 0x391, 0x3c3, 0x3bf, 0x3c5, 0x3bd, 0x3c3,
0x3b9, 0x3cc, 0x3bd, 0x391, 0x3c4, 0x3b9, 0x3ba, 0x3cc, 0x3ba, 0x3b1, 0x3bd, 0x39c,
0x3c0, 0x3b1, 0x390, 0x3b1, 0x39c, 0x3c0, 0x3b1, 0x390, 0x3b1, 0x20, 0x3bd, 0x3c4,
0x3b5, 0x20, 0x39c, 0x3c0, 0x3b1, 0x3bd, 0x3c4, 0x3ad, 0x3c1, 0x3b1, 0x3c2, 0x39c,
0x3c0, 0x3b1, 0x3c1, 0x3bc, 0x3c0, 0x3ad, 0x3b9, 0x3bd, 0x3c4, 0x3bf, 0x3c2, 0x39c,
0x3c0, 0x3b5, 0x3bb, 0x3ad, 0x3bc, 0x39c, 0x3c0, 0x3b5, 0x3bb, 0x3af, 0x3b6, 0x39c,
0x3c0, 0x3bb, 0x3b1, 0x3bd, 0x20, 0x3a3, 0x3b1, 0x3bc, 0x3c0, 0x3bb, 0x3cc, 0x3bd,
0x39c, 0x3c0, 0x3cc, 0x3b1, 0x20, 0x392, 0x3af, 0x3c3, 0x3c4, 0x3b1, 0x39c, 0x3c0,
0x3bf, 0x3b3, 0x3ba, 0x3bf, 0x3c4, 0x3ac, 0x39c, 0x3c0, 0x3cc, 0x3b9, 0x3b6, 0x3b9,
0x39a, 0x3ad, 0x3bc, 0x3c0, 0x3c1, 0x3b9, 0x3c4, 0x3b6, 0x20, 0x39c, 0x3c0, 0x3ad,
0x3b9, 0x39a, 0x3ac, 0x3bc, 0x3c0, 0x3bf, 0x20, 0x393, 0x3ba, 0x3c1, 0x3ac, 0x3bd,
0x3c4, 0x3b5, 0x39a, 0x3b1, 0x3bd, 0x3ba, 0x3bf, 0x3cd, 0x3bd, 0x39a, 0x3b1, 0x3c1,
0x3ac, 0x3ba, 0x3b1, 0x3c2, 0x39a, 0x3b1, 0x3b3, 0x3b9, 0x3ad, 0x3bd, 0x39a, 0x3ad,
0x3b9, 0x3bc, 0x3b1, 0x3bd, 0x3a3, 0x3b9, 0x3ba, 0x3ac, 0x3b3, 0x3bf, 0x3a4, 0x3c3,
0x3b9, 0x3bf, 0x3c5, 0x3ac, 0x3bf, 0x3c5, 0x3b1, 0x3a3, 0x3b9, 0x3bf, 0x3c5, 0x3b4,
0x3ac, 0x3b4, 0x20, 0x3a7, 0x3bf, 0x3c5, 0x3ac, 0x3c1, 0x3b5, 0x3c2, 0x39a, 0x3cc,
0x3c3, 0x3c4, 0x3b1, 0x20, 0x3a1, 0x3af, 0x3ba, 0x3b1, 0x39a, 0x3c1, 0x3ad, 0x3c3,
0x3c4, 0x3bf, 0x3bd, 0x39a, 0x3bf, 0x3c5, 0x3b9, 0x3b1, 0x3bc, 0x3c0, 0x3ac, 0x39a,
0x3bf, 0x3c5, 0x3c1, 0x3b1, 0x3c3, 0x3ac, 0x3bf, 0x39d, 0x3c4, 0x3b1, 0x3bd, 0x3bc,
0x3b1, 0x3c1, 0x3ba, 0x3c3, 0x3ac, 0x3b2, 0x3bd, 0x39d, 0x3c4, 0x3cc, 0x3c3, 0x3bf,
0x3bd, 0x39d, 0x3c4, 0x3cc, 0x3c3, 0x3bf, 0x3bd, 0x20, 0x39a, 0x3c1, 0x3b9, 0x3ba,
0x39d, 0x3c4, 0x3ad, 0x3bd, 0x3b2, 0x3b5, 0x3c1, 0x39d, 0x3c4, 0x3b9, 0x3c4, 0x3c1,
0x3cc, 0x3b9, 0x3c4, 0x39d, 0x3c4, 0x3bf, 0x3bc, 0x3af, 0x3bd, 0x3b9, 0x3ba, 0x3b1,
0x388, 0x3bd, 0x3c4, 0x3bc, 0x3bf, 0x3bd, 0x3c4, 0x3bf, 0x3bd, 0x395, 0x3ca, 0x3c1,
0x3bf, 0x3c5, 0x3bd, 0x3b5, 0x3c0, 0x3ad, 0x395, 0x3bb, 0x20, 0x3a3, 0x3b1, 0x3bb,
0x3b2, 0x3b1, 0x3b4, 0x3cc, 0x3c1, 0x3a6, 0x3bf, 0x3c1, 0x3c4, 0x20, 0x39d, 0x3ad,
0x3bb, 0x3c3, 0x3bf, 0x3bd, 0x3a6, 0x3bf, 0x3c1, 0x3c4, 0x3b1, 0x3bb, 0x3ad, 0x3b6,
0x3b1, 0x393, 0x3ba, 0x3bb, 0x3ad, 0x3b9, 0x3c2, 0x20, 0x39c, 0x3c0, 0x3ad, 0x3b9,
0x393, 0x3ba, 0x3bf, 0x3c5, 0x3c2, 0x20, 0x39c, 0x3c0, 0x3ad, 0x3b9, 0x393, 0x3ba,
0x3c1, 0x3b1, 0x3bd, 0x3c4, 0x20, 0x3a4, 0x3bf, 0x3c5, 0x3c1, 0x3ba, 0x393, 0x3c1,
0x3b5, 0x3bd, 0x3ac, 0x3b4, 0x3b1, 0x393, 0x3bf, 0x3c5, 0x3b1, 0x3b4, 0x3b5, 0x3bb,
0x3bf, 0x3cd, 0x3c0, 0x3b7, 0x393, 0x3bf, 0x3c5, 0x3b1, 0x3c4, 0x3b5, 0x3bc, 0x3ac,
0x3bb, 0x3b1, 0x393, 0x3ba, 0x3bf, 0x3c5, 0x3b1, 0x3b3, 0x3b9, 0x3b1, 0x3ba, 0x3af,
0x3bb, 0x393, 0x3bf, 0x3c5, 0x3b9, 0x3ac, 0x3bd, 0x3b1, 0x3a7, 0x3ac, 0x3bb, 0x3b9,
0x3c6, 0x3b1, 0x3be, 0x391, 0x3b2, 0x3ac, 0x3bd, 0x3b1, 0x395, 0x3c1, 0x3bc, 0x3bf,
0x3c3, 0x3af, 0x3b3, 0x3b9, 0x3bf, 0x399, 0x3bd, 0x3c4, 0x3b9, 0x3b1, 0x3bd, 0x3ac,
0x3c0, 0x3bf, 0x3bb, 0x3b9, 0x3c2, 0x39d, 0x3bf, 0x3be, 0x2c, 0x20, 0x399, 0x3bd,
0x3c4, 0x3b9, 0x3ac, 0x3bd, 0x3b1, 0x39c, 0x3b1, 0x3c1, 0x3ad, 0x3bd, 0x3b3, 0x3ba,
0x3bf, 0x2c, 0x20, 0x399, 0x3bd, 0x3c4, 0x3b9, 0x3ac, 0x3bd, 0x3b1, 0x3a0, 0x3af,
0x3c4, 0x3b5, 0x3c1, 0x3c3, 0x3bc, 0x3c0, 0x3b5, 0x3c1, 0x3b3, 0x3ba, 0x2c, 0x20,
0x399, 0x3bd, 0x3c4, 0x3b9, 0x3ac, 0x3bd, 0x3b1, 0x3a4, 0x3b5, 0x3bb, 0x20, 0x3a3,
0x3af, 0x3c4, 0x3b9, 0x2c, 0x20, 0x399, 0x3bd, 0x3c4, 0x3b9, 0x3ac, 0x3bd, 0x3b1,
0x392, 0x3b9, 0x3b2, 0x3ad, 0x3b9, 0x2c, 0x20, 0x399, 0x3bd, 0x3c4, 0x3b9, 0x3ac,
0x3bd, 0x3b1, 0x392, 0x3b1, 0x3bd, 0x3c3, 0x3ad, 0x3bd, 0x2c, 0x20, 0x399, 0x3bd,
0x3c4, 0x3b9, 0x3ac, 0x3bd, 0x3b1, 0x393, 0x3bf, 0x3c5, 0x3af, 0x3bd, 0x3b1, 0x3bc,
0x3b1, 0x3ba, 0x2c, 0x20, 0x399, 0x3bd, 0x3c4, 0x3b9, 0x3ac, 0x3bd, 0x3b1, 0x399,
0x3bd, 0x3bf, 0x3cd, 0x3b2, 0x3b9, 0x3ba, 0x399, 0x3ba, 0x3b1, 0x3bb, 0x3bf, 0x3cd,
0x3b9, 0x3c4, 0x3a4, 0x3b6, 0x3b1, 0x3bc, 0x3ac, 0x3b9, 0x3ba, 0x3b1, 0x3a4, 0x3b6,
0x3bf, 0x3cd, 0x3bd, 0x3bf, 0x39b, 0x3bf, 0x3cd, 0x3b9, 0x3b2, 0x3b9, 0x3bb, 0x39c,
0x3bf, 0x3bd, 0x3c4, 0x3b9, 0x3c4, 0x3c3, 0x3ad, 0x3bb, 0x3bf, 0x2c, 0x20, 0x39a,
0x3b5, 0x3bd, 0x3c4, 0x3ac, 0x3ba, 0x3b9, 0x39a, 0x3c1, 0x3ac, 0x3bb, 0x3b5, 0x3bd,
0x3c4, 0x3b9, 0x3ba, 0x39b, 0x3b1, 0x20, 0x3a0, 0x3b1, 0x3b6, 0x39b, 0x3af, 0x3bc,
0x3b1, 0x39b, 0x3bf, 0x3c2, 0x20, 0x386, 0x3bd, 0x3c4, 0x3b6, 0x3b5, 0x3bb, 0x3b5,
0x3c2, 0x39c, 0x3b1, 0x3c3, 0x3b5, 0x3ca, 0x3cc, 0x39c, 0x3b1, 0x3bd, 0x3ac, 0x3b3,
0x3ba, 0x3bf, 0x3c5, 0x3b1, 0x39c, 0x3b1, 0x3bd, 0x3ac, 0x3bf, 0x3c5, 0x3c2, 0x39c,
0x3b1, 0x3c1, 0x3b9, 0x3b3, 0x3ba, 0x3cc, 0x39c, 0x3b1, 0x3c1, 0x3c4, 0x3b9, 0x3bd,
0x3af, 0x3ba, 0x3b1, 0x39c, 0x3b1, 0x3c4, 0x3b1, 0x3bc, 0x3cc, 0x3c1, 0x3bf, 0x3c2,
0x39c, 0x3b1, 0x3b6, 0x3b1, 0x3c4, 0x3bb, 0x3ac, 0x3bd, 0x39c, 0x3b5, 0x3bd, 0x3bf,
0x3bc, 0x3af, 0x3bd, 0x3b5, 0x39c, 0x3ad, 0x3c1, 0x3b9, 0x3b4, 0x3b1, 0x39c, 0x3b5,
0x3c4, 0x3bb, 0x3b1, 0x3ba, 0x3ac, 0x3c4, 0x3bb, 0x3b1, 0x3a0, 0x3cc, 0x3bb, 0x3b7,
0x20, 0x3c4, 0x3bf, 0x3c5, 0x20, 0x39c, 0x3b5, 0x3be, 0x3b9, 0x3ba, 0x3bf, 0x3cd,
0x39c, 0x3b9, 0x3ba, 0x3b5, 0x3bb, 0x3cc, 0x3bd, 0x39c, 0x3cc, 0x3bd, 0x3ba, 0x3c4,
0x3bf, 0x3bd, 0x39c, 0x3bf, 0x3bd, 0x3c4, 0x3b5, 0x3c1, 0x3ad, 0x3b9, 0x39c, 0x3bf,
0x3bd, 0x3c4, 0x3b5, 0x3b2, 0x3b9, 0x3b4, 0x3ad, 0x3bf, 0x39c, 0x3bf, 0x3bd, 0x3c3,
0x3b5, 0x3c1, 0x3ac, 0x3c4, 0x39d, 0x3b1, 0x3c3, 0x3ac, 0x3bf, 0x3c5, 0x39d, 0x3ad,
0x3b1, 0x20, 0x3a5, 0x3cc, 0x3c1, 0x3ba, 0x3b7, 0x39d, 0x3cc, 0x3bc, 0x3b5, 0x39d,
0x3bf, 0x3c1, 0x3cc, 0x3bd, 0x3b9, 0x3b1, 0x39c, 0x3c0, 0x3ad, 0x3bf, 0x3c5, 0x3bb,
0x3b1, 0x2c, 0x20, 0x392, 0x3cc, 0x3c1, 0x3b5, 0x3b9, 0x3b1, 0x20, 0x39d, 0x3c4,
0x3b1, 0x3ba, 0x3cc, 0x3c4, 0x3b1, 0x3a3, 0x3ad, 0x3bd, 0x3c4, 0x3b5, 0x3c1, 0x2c,
0x20, 0x392, 0x3cc, 0x3c1, 0x3b5, 0x3b9, 0x3b1, 0x20, 0x39d, 0x3c4, 0x3b1, 0x3ba,
0x3cc, 0x3c4, 0x3b1, 0x39d, 0x3b9, 0x3bf, 0x3c5, 0x20, 0x3a3, 0x3ad, 0x3b9, 0x3bb,
0x3b5, 0x3bc, 0x2c, 0x20, 0x392, 0x3cc, 0x3c1, 0x3b5, 0x3b9, 0x3b1, 0x20, 0x39d,
0x3c4, 0x3b1, 0x3ba, 0x3cc, 0x3c4, 0x3b1, 0x39d, 0x3bf, 0x3c5, 0x3bf, 0x3cd, 0x3ba,
0x39f, 0x3c7, 0x3b9, 0x3bd, 0x3ac, 0x3b3, 0x3ba, 0x3b1, 0x3a0, 0x3b1, 0x3bd, 0x3b1,
0x3bc, 0x3ac, 0x3c2, 0x3a0, 0x3b1, 0x3c1, 0x3b1, 0x3bc, 0x3b1, 0x3c1, 0x3af, 0x3bc,
0x3c0, 0x3bf, 0x3a6, 0x3bf, 0x3af, 0x3bd, 0x3b9, 0x3be, 0x3a0, 0x3bf, 0x3c1, 0x3c4,
0x2d, 0x3bf, 0x2d, 0x3a0, 0x3c1, 0x3b5, 0x3bd, 0x3c2, 0x3a0, 0x3bf, 0x3c1, 0x3c4,
0x20, 0x3bf, 0x3c6, 0x20, 0x3a3, 0x3c0, 0x3ad, 0x3b9, 0x3bd, 0x3a0, 0x3cc, 0x3c1,
0x3c4, 0x3bf, 0x20, 0x392, 0x3ad, 0x3bb, 0x3b9, 0x3bf, 0x3a0, 0x3bf, 0x3c5, 0x3ad,
0x3c1, 0x3c4, 0x3bf, 0x20, 0x3a1, 0x3af, 0x3ba, 0x3bf, 0x3a0, 0x3bf, 0x3cd, 0x3bd,
0x3c4, 0x3b1, 0x20, 0x391, 0x3c1, 0x3ad, 0x3bd, 0x3b1, 0x3c2, 0x3a1, 0x3ac, 0x3bd,
0x3ba, 0x3b9, 0x3bd, 0x20, 0x38a, 0x3bd, 0x3bb, 0x3b5, 0x3c4, 0x3a1, 0x3b5, 0x3c3,
0x3af, 0x3c6, 0x3b5, 0x3a1, 0x3b5, 0x3c4, 0x3b6, 0x3ac, 0x3b9, 0x3bd, 0x3b1, 0x3a1,
0x3ad, 0x3b6, 0x3bf, 0x3bb, 0x3bf, 0x3c5, 0x3c4, 0x3a1, 0x3af, 0x3bf, 0x20, 0x39c,
0x3c0, 0x3c1, 0x3ac, 0x3bd, 0x3ba, 0x3bf, 0x3a3, 0x3b1, 0x3bd, 0x3c4, 0x3b1, 0x3c1,
0x3ad, 0x3bc, 0x3a3, 0x3b1, 0x3bd, 0x3c4, 0x3b9, 0x3ac, 0x3b3, 0x3ba, 0x3bf, 0x386,
0x3b3, 0x3b9, 0x3bf, 0x3c2, 0x20, 0x394, 0x3bf, 0x3bc, 0x3af, 0x3bd, 0x3b9, 0x3ba,
0x3bf, 0x3c2, 0x3a3, 0x3ac, 0x3bf, 0x20, 0x3a0, 0x3ac, 0x3bf, 0x3bb, 0x3bf, 0x3a3,
0x3ba, 0x3bf, 0x3c1, 0x3b5, 0x3c3, 0x3bc, 0x3c0, 0x3af, 0x3c3, 0x3bf, 0x3c5, 0x3bd,
0x3c4, 0x3a3, 0x3af, 0x3c4, 0x3ba, 0x3b1, 0x386, 0x3b3, 0x3b9, 0x3bf, 0x3c2, 0x20,
0x392, 0x3b1, 0x3c1, 0x3b8, 0x3bf, 0x3bb, 0x3bf, 0x3bc, 0x3b1, 0x3af, 0x3bf, 0x3c2,
0x3a3, 0x3b5, 0x3bd, 0x20, 0x3a4, 0x3b6, 0x3bf, 0x3bd, 0x3c2, 0x3a3, 0x3b5, 0x3bd,
0x20, 0x39a, 0x3b9, 0x3c4, 0x3c2, 0x391, 0x3b3, 0x3af, 0x3b1, 0x20, 0x39b, 0x3bf,
0x3c5, 0x3ba, 0x3af, 0x3b1, 0x386, 0x3b3, 0x3b9, 0x3bf, 0x3c2, 0x20, 0x398, 0x3c9,
0x3bc, 0x3ac, 0x3c2, 0x386, 0x3b3, 0x3b9, 0x3bf, 0x3c2, 0x20, 0x392, 0x3b9, 0x3ba,
0x3ad, 0x3bd, 0x3c4, 0x3b9, 0x3bf, 0x3c2, 0x3a3, 0x3bf, 0x3c5, 0x3af, 0x3c6, 0x3c4,
0x20, 0x39a, 0x3ac, 0x3c1, 0x3b5, 0x3bd, 0x3c4, 0x3a4, 0x3b5, 0x3b3, 0x3ba, 0x3bf,
0x3c5, 0x3c3, 0x3b9, 0x3b3, 0x3ba, 0x3ac, 0x3bb, 0x3c0, 0x3b1, 0x398, 0x3bf, 0x3cd,
0x3bb, 0x3b7, 0x3a4, 0x3b9, 0x3c7, 0x3bf, 0x3c5, 0x3ac, 0x3bd, 0x3b1, 0x3a4, 0x3bf,
0x3c1, 0x3cc, 0x3bd, 0x3c4, 0x3bf, 0x3a4, 0x3bf, 0x3c1, 0x3c4, 0x3cc, 0x3bb, 0x3b1,
0x392, 0x3b1, 0x3bd, 0x3ba, 0x3bf, 0x3cd, 0x3b2, 0x3b5, 0x3c1, 0x393, 0x3bf, 0x3c5,
0x3ac, 0x3b9, 0x3c4, 0x3c7, 0x3bf, 0x3c1, 0x3c2, 0x393, 0x3bf, 0x3c5, 0x3af, 0x3bd,
0x3b9, 0x3c0, 0x3b5, 0x3b3, 0x3ba, 0x393, 0x3b9, 0x3ac, 0x3ba, 0x3bf, 0x3c5, 0x3c4,
0x3b1, 0x3c4, 0x39a, 0x3ac, 0x3c3, 0x3b5, 0x3ca, 0x39d, 0x3c4, 0x3ad, 0x3b9, 0x3b2,
0x3b9, 0x3c2, 0x39d, 0x3c4, 0x3b9, 0x3bc, 0x3cc, 0x3bd, 0x20, 0x3bd, 0x3c4, 0x2019,
0x20, 0x39f, 0x3c5, 0x3c1, 0x3b2, 0x3af, 0x3bb, 0x39c, 0x3b1, 0x3ba, 0x3bf, 0x3c5,
0x3ac, 0x3c1, 0x3b9, 0x39c, 0x3cc, 0x3c3, 0x3bf, 0x3bd, 0x39c, 0x3b1, 0x3ba, 0x3bc,
0x3ad, 0x3c1, 0x3bd, 0x3c4, 0x3bf, 0x3a0, 0x3ac, 0x3bb, 0x3bc, 0x3b5, 0x3c1, 0x3a1,
0x3cc, 0x3b8, 0x3b5, 0x3c1, 0x3b1, 0x3a3, 0x3af, 0x3bf, 0x3b2, 0x3b1, 0x3a4, 0x3c1,
0x3bf, 0x3bb, 0x392, 0x3cc, 0x3c3, 0x3c4, 0x3bf, 0x3ba, 0x39b, 0x3cc, 0x3bd, 0x3b3,
0x3ba, 0x3b9, 0x3b5, 0x3b1, 0x3c1, 0x3bc, 0x3c0, 0x3b9, 0x3b5, 0x3bd, 0x386, 0x3bd,
0x3c4, 0x3b5, 0x3bd, 0x391, 0x3bb, 0x3bc, 0x3ac, 0x3c4, 0x3b9, 0x391, 0x3bc, 0x3bc,
0x3ac, 0x3bd, 0x391, 0x3bd, 0x3b1, 0x3bd, 0x3c4, 0x3af, 0x3c1, 0x391, 0x3ba, 0x3c4,
0x3ac, 0x3bf, 0x3c5, 0x391, 0x3ba, 0x3c4, 0x3cc, 0x3bc, 0x3c0, 0x3b5, 0x391, 0x3c3,
0x3c7, 0x3b1, 0x3bc, 0x3c0, 0x3ac, 0x3c4, 0x391, 0x3c4, 0x3b9, 0x3c1, 0x3ac, 0x3bf,
0x3c5, 0x392, 0x3b1, 0x3b3, 0x3b4, 0x3ac, 0x3c4, 0x3b7, 0x39c, 0x3c0, 0x3b1, 0x3c7,
0x3c1, 0x3ad, 0x3b9, 0x3bd, 0x39c, 0x3c0, 0x3b1, 0x3ba, 0x3bf, 0x3cd, 0x39c, 0x3c0,
0x3b1, 0x3bd, 0x3b3, 0x3ba, 0x3cc, 0x3ba, 0x39c, 0x3c0, 0x3b1, 0x3c1, 0x3bd, 0x3b1,
0x3bf, 0x3cd, 0x3bb, 0x392, 0x3c5, 0x3c1, 0x3b7, 0x3c4, 0x3cc, 0x3c2, 0x39c, 0x3c0,
0x3b9, 0x3c3, 0x3ba, 0x3ad, 0x3ba, 0x39c, 0x3c0, 0x3c1, 0x3bf, 0x3c5, 0x3bd, 0x3ad,
0x3b9, 0x3a4, 0x3c3, 0x3b9, 0x3c4, 0x3ac, 0x39a, 0x3bf, 0x3bb, 0x3cc, 0x3bc, 0x3c0,
0x3bf, 0x394, 0x3b1, 0x3bc, 0x3b1, 0x3c3, 0x3ba, 0x3cc, 0x3c2, 0x39d, 0x3c4, 0x3ac,
0x3ba, 0x3b1, 0x39d, 0x3c4, 0x3af, 0x3bb, 0x3b9, 0x39d, 0x3c4, 0x3bf, 0x3c5, 0x3bc,
0x3c0, 0x3ac, 0x3b9, 0x39d, 0x3c4, 0x3bf, 0x3c5, 0x3c3, 0x3ac, 0x3bd, 0x3bc, 0x3c0,
0x3b5, 0x391, 0x3bc, 0x3bc, 0x3cc, 0x3c7, 0x3c9, 0x3c3, 0x3c4, 0x3bf, 0x3c2, 0x393,
0x3ac, 0x3b6, 0x3b1, 0x3a7, 0x3b5, 0x3b2, 0x3c1, 0x3ce, 0x3bd, 0x3b1, 0x3a0, 0x3cc,
0x3bb, 0x3b7, 0x20, 0x3a7, 0x3bf, 0x20, 0x3a4, 0x3c3, 0x3b9, 0x20, 0x39c, 0x3b9,
0x3bd, 0x3c7, 0x3a7, 0x3bf, 0x3bd, 0x3b3, 0x3ba, 0x20, 0x39a, 0x3bf, 0x3bd, 0x3b3,
0x3ba, 0x3a7, 0x3bf, 0x3b2, 0x3bd, 0x3c4, 0x399, 0x3c1, 0x3ba, 0x3bf, 0x3cd, 0x3c4,
0x3c3, 0x3ba, 0x3a4, 0x3b6, 0x3b1, 0x3ba, 0x3ac, 0x3c1, 0x3c4, 0x3b1, 0x3a4, 0x3b6,
0x3b1, 0x3b3, 0x3b9, 0x3b1, 0x3c0, 0x3bf, 0x3cd, 0x3c1, 0x3b1, 0x399, 0x3b5, 0x3c1,
0x3bf, 0x3c5, 0x3c3, 0x3b1, 0x3bb, 0x3ae, 0x3bc, 0x39a, 0x3b1, 0x3bc, 0x3c0, 0x3bf,
0x3cd, 0x3bb, 0x39a, 0x3b1, 0x3bc, 0x3c4, 0x3c3, 0x3ac, 0x3c4, 0x3ba, 0x3b1, 0x39a,
0x3b1, 0x3c1, 0x3ac, 0x3c4, 0x3c3, 0x3b9, 0x39a, 0x3b1, 0x3c4, 0x3bc, 0x3b1, 0x3bd,
0x3c4, 0x3bf, 0x3cd, 0x3a7, 0x3b1, 0x3bd, 0x3c4, 0x3af, 0x3b3, 0x3ba, 0x3b1, 0x39a,
0x3b1, 0x3bb, 0x3ba, 0x3bf, 0x3cd, 0x3c4, 0x3b1, 0x39a, 0x3c1, 0x3b1, 0x3c3, 0x3bd,
0x3bf, 0x3b3, 0x3b9, 0x3ac, 0x3c1, 0x3c3, 0x3ba, 0x39a, 0x3bf, 0x3c5, 0x3ac, 0x3bb,
0x3b1, 0x20, 0x39b, 0x3bf, 0x3c5, 0x3bc, 0x3c0, 0x3bf, 0x3cd, 0x3c1, 0x39a, 0x3bf,
0x3c5, 0x3c4, 0x3c3, 0x3af, 0x3bd, 0x3b3, 0x3ba, 0x39a, 0x3bf, 0x3c5, 0x3b2, 0x3ad,
0x3b9, 0x3c4, 0x39c, 0x3b1, 0x3ba, 0x3ac, 0x3bf, 0x39c, 0x3b1, 0x3b3, 0x3ba, 0x3b1,
0x3bd, 0x3c4, 0x3ac, 0x3bd, 0x39c, 0x3b1, 0x3ba, 0x3b1, 0x3c3, 0x3ac, 0x3c1, 0x39c,
0x3b1, 0x3bd, 0x3af, 0x3bb, 0x3b1, 0x39c, 0x3b1, 0x3c3, 0x3ba, 0x3ac, 0x3c4, 0x39b,
0x3b5, 0x3c5, 0x3ba, 0x3c9, 0x3c3, 0x3af, 0x3b1, 0x39d, 0x3bf, 0x3b2, 0x3bf, 0x3ba,
0x3bf, 0x3c5, 0x3b6, 0x3bd, 0x3ad, 0x3c4, 0x3c3, 0x3ba, 0x39d, 0x3bf, 0x3b2, 0x3bf,
0x3c3, 0x3b9, 0x3bc, 0x3c0, 0x3af, 0x3c1, 0x3c3, 0x3ba, 0x39f, 0x3bc, 0x3c3, 0x3ba,
0x39f, 0x3c1, 0x3ac, 0x3bb, 0x3a0, 0x3bd, 0x3bf, 0x3bc, 0x20, 0x3a0, 0x3b5, 0x3bd,
0x3c7, 0x3a0, 0x3cc, 0x3bd, 0x3c4, 0x3b9, 0x3b1, 0x3bd, 0x3b1, 0x3ba, 0x3a0, 0x3b9,
0x3bf, 0x3bd, 0x3b3, 0x3ba, 0x3b3, 0x3b9, 0x3ac, 0x3bd, 0x3b3, 0x3ba, 0x39a, 0x3b1,
0x3c4, 0x3ac, 0x3c1, 0x39a, 0x3bf, 0x3c3, 0x3c4, 0x3b1, 0x3bd, 0x3ac, 0x3b9, 0x39a,
0x3b9, 0x3b6, 0x3b9, 0x3bb, 0x3bf, 0x3c1, 0x3bd, 0x3c4, 0x3ac, 0x3a1, 0x3b9, 0x3ac,
0x3bd, 0x3c4, 0x3a3, 0x3b1, 0x3c7, 0x3b1, 0x3bb, 0x3af, 0x3bd, 0x3b7, 0x3a3, 0x3b1,
0x3bc, 0x3b1, 0x3c1, 0x3ba, 0x3ac, 0x3bd, 0x3b4, 0x3b7, 0x3a3, 0x3b5, 0x3bf, 0x3cd,
0x3bb, 0x3a3, 0x3b1, 0x3bd, 0x3b3, 0x3ba, 0x3ac, 0x3b7, 0x3a3, 0x3b9, 0x3b3, 0x3ba,
0x3b1, 0x3c0, 0x3bf, 0x3cd, 0x3c1, 0x3b7, 0x3a3, 0x3c1, 0x3b5, 0x3bd, 0x3c4, 0x3bd,
0x3b5, 0x3ba, 0x3bf, 0x3bb, 0x3af, 0x3bc, 0x3c3, 0x3ba, 0x3a4, 0x3b1, 0x3ca, 0x3c0,
0x3ad, 0x3b9, 0x3a4, 0x3b1, 0x3c3, 0x3ba, 0x3ad, 0x3bd, 0x3b4, 0x3b7, 0x3a4, 0x3b9,
0x3c6, 0x3bb, 0x3af, 0x3b4, 0x3b1, 0x3a4, 0x3b5, 0x3c7, 0x3b5, 0x3c1, 0x3ac, 0x3bd,
0x3b7, 0x398, 0x3af, 0x3bc, 0x3c6, 0x3bf, 0x3c5, 0x3a4, 0x3cc, 0x3ba, 0x3b9, 0x3bf,
0x3a4, 0x3bf, 0x3bc, 0x3c3, 0x3ba, 0x39f, 0x3c5, 0x3bb, 0x3ac, 0x3bd, 0x20, 0x39c,
0x3c0, 0x3b1, 0x3c4, 0x3cc, 0x3c1, 0x39f, 0x3c5, 0x3c1, 0x3bf, 0x3cd, 0x3bc, 0x3c4,
0x3c3, 0x3b9, 0x39f, 0x3c5, 0x3c3, 0x3c4, 0x2d, 0x39d, 0x3b5, 0x3c1, 0x3ac, 0x392,
0x3b9, 0x3b5, 0x3bd, 0x3c4, 0x3b9, 0x3ac, 0x3bd, 0x392, 0x3bb, 0x3b1, 0x3b4, 0x3b9,
0x3b2, 0x3bf, 0x3c3, 0x3c4, 0x3cc, 0x3ba, 0x393, 0x3b9, 0x3b1, 0x3ba, 0x3bf, 0x3cd,
0x3c4, 0x3c3, 0x3ba, 0x3a1, 0x3b1, 0x3bd, 0x3b3, 0x3ba, 0x3bf, 0x3cd, 0x3bd, 0x391,
0x3b9, 0x3ba, 0x3b1, 0x3c4, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3bf, 0x3cd, 0x3c0, 0x3bf,
0x3bb, 0x3b7, 0x395, 0x3c1, 0x3b5, 0x3b2, 0x3ac, 0x3bd, 0x391, 0x3b6, 0x3cc, 0x3c1,
0x3b5, 0x3c2, 0x392, 0x3b5, 0x3c1, 0x3bc, 0x3bf, 0x3cd, 0x3b4, 0x3b5, 0x3c2, 0x39a,
0x3b1, 0x3bd, 0x3ac, 0x3c1, 0x3b9, 0x3b1, 0x3a0, 0x3c1, 0x3ac, 0x3c3, 0x3b9, 0x3bd,
0x3bf, 0x20, 0x391, 0x3ba, 0x3c1, 0x3c9, 0x3c4, 0x3ae, 0x3c1, 0x3b9, 0x3bf, 0x3a6,
0x3b5, 0x3c1, 0x3cc, 0x3b5, 0x3c2, 0x39c, 0x3b1, 0x3b4, 0x3ad, 0x3c1, 0x3b1, 0x3a1,
0x3ad, 0x3c5, 0x3ba, 0x3b9, 0x3b1, 0x3b2, 0x3b9, 0x3ba, 0x39d, 0x3cc, 0x3c4, 0x3b9,
0x3b1, 0x20, 0x393, 0x3b5, 0x3c9, 0x3c1, 0x3b3, 0x3af, 0x3b1, 0x391, 0x3b3, 0x2e,
0x20, 0x395, 0x3bb, 0x3ad, 0x3bd, 0x3b7, 0x3a3, 0x3c4, 0x3ac, 0x3bd, 0x3bb, 0x3b5,
0x3ca, 0x391, 0x3b4, 0x3b5, 0x3bb, 0x3b1, 0x390, 0x3b4, 0x3b1, 0x39c, 0x3c0, 0x3c1,
0x3b9, 0x3c3, 0x3bc, 0x3c0, 0x3ad, 0x3b9, 0x3bd, 0x39c, 0x3c0, 0x3c1, 0x3cc, 0x3ba,
0x3b5, 0x3bd, 0x20, 0x3a7, 0x3b9, 0x3bb, 0x39d, 0x3c4, 0x3ac, 0x3c1, 0x3b3, 0x3bf,
0x3c5, 0x3b9, 0x3bd, 0x393, 0x3b9, 0x3bf, 0x3cd, 0x3ba, 0x3bb, 0x3b1, 0x3a7, 0x3cc,
0x3bc, 0x3c0, 0x3b1, 0x3c1, 0x3c4, 0x39b, 0x3af, 0x3bd, 0x3c4, 0x3b5, 0x3bc, 0x3b1,
0x3bd, 0x39b, 0x3bf, 0x3c1, 0x3bd, 0x3c4, 0x20, 0x3a7, 0x3ac, 0x3bf, 0x3c5, 0x39c,
0x3b5, 0x3bb, 0x3b2, 0x3bf, 0x3cd, 0x3c1, 0x3bd, 0x3b7, 0x3a0, 0x3b5, 0x3c1, 0x3b8,
0x3a3, 0x3af, 0x3b4, 0x3bd, 0x3b5, 0x3ca, 0x386, 0x3bc, 0x3c3, 0x3c4, 0x3b5, 0x3c1,
0x3bd, 0x3c4, 0x3b1, 0x3bc, 0x391, 0x3bd, 0x3b4, 0x3cc, 0x3c1, 0x3b1, 0x391, 0x3c3,
0x3c4, 0x3c1, 0x3b1, 0x3c7, 0x3ac, 0x3bd, 0x391, 0x3b8, 0x3ae, 0x3bd, 0x3b1, 0x392,
0x3b5, 0x3bb, 0x3b9, 0x3b3, 0x3c1, 0x3ac, 0x3b4, 0x3b9, 0x392, 0x3b5, 0x3c1, 0x3bf,
0x3bb, 0x3af, 0x3bd, 0x3bf, 0x39c, 0x3c0, 0x3c1, 0x3b1, 0x3c4, 0x3b9, 0x3c3, 0x3bb,
0x3ac, 0x3b2, 0x3b1, 0x392, 0x3c1, 0x3c5, 0x3be, 0x3ad, 0x3bb, 0x3bb, 0x3b5, 0x3c2,
0x392, 0x3bf, 0x3c5, 0x3ba, 0x3bf, 0x3c5, 0x3c1, 0x3ad, 0x3c3, 0x3c4, 0x3b9, 0x392,
0x3bf, 0x3c5, 0x3b4, 0x3b1, 0x3c0, 0x3ad, 0x3c3, 0x3c4, 0x3b7, 0x39c, 0x3c0, 0x3af,
0x3c3, 0x3b9, 0x3bd, 0x3b3, 0x3ba, 0x3b5, 0x3bd, 0x39a, 0x3b9, 0x3c3, 0x3b9, 0x3bd,
0x3ac, 0x3bf, 0x3c5, 0x39a, 0x3bf, 0x3c0, 0x3b5, 0x3b3, 0x3c7, 0x3ac, 0x3b3, 0x3b7,
0x394, 0x3bf, 0x3c5, 0x3b2, 0x3bb, 0x3af, 0x3bd, 0x3bf, 0x393, 0x3b9, 0x3b2, 0x3c1,
0x3b1, 0x3bb, 0x3c4, 0x3ac, 0x3c1, 0x393, 0x3ba, 0x3ad, 0x3c1, 0x3bd, 0x3b6, 0x3b9,
0x395, 0x3bb, 0x3c3, 0x3af, 0x3bd, 0x3ba, 0x3b9, 0x39d, 0x3ae, 0x3c3, 0x3bf, 0x3c2,
0x20, 0x3c4, 0x3bf, 0x3c5, 0x20, 0x39c, 0x3b1, 0x3bd, 0x39a, 0x3c9, 0x3bd, 0x3c3,
0x3c4, 0x3b1, 0x3bd, 0x3c4, 0x3b9, 0x3bd, 0x3bf, 0x3cd, 0x3c0, 0x3bf, 0x3bb, 0x3b7,
0x3a4, 0x3b6, 0x3ad, 0x3c1, 0x3c3, 0x3b5, 0x3ca, 0x39a, 0x3b1, 0x3bb, 0x3af, 0x3bd,
0x3b9, 0x3bd, 0x3b3, 0x3ba, 0x3c1, 0x3b1, 0x3bd, 0x3c4, 0x39a, 0x3af, 0x3c1, 0x3bf,
0x3c6, 0x39a, 0x3af, 0x3b5, 0x3b2, 0x3bf, 0x39b, 0x3b9, 0x3c3, 0x3b1, 0x3b2, 0x3cc,
0x3bd, 0x3b1, 0x39b, 0x3b9, 0x3bf, 0x3c5, 0x3bc, 0x3c0, 0x3bb, 0x3b9, 0x3ac, 0x3bd,
0x3b1, 0x39b, 0x3bf, 0x3bd, 0x3b4, 0x3af, 0x3bd, 0x3bf, 0x39b, 0x3bf, 0x3c5, 0x3be,
0x3b5, 0x3bc, 0x3b2, 0x3bf, 0x3cd, 0x3c1, 0x3b3, 0x3bf, 0x39c, 0x3b1, 0x3b4, 0x3c1,
0x3af, 0x3c4, 0x3b7, 0x39c, 0x3ac, 0x3bb, 0x3c4, 0x3b1, 0x39c, 0x3ac, 0x3c1, 0x3b9,
0x3b5, 0x3c7, 0x3b1, 0x3bc, 0x3bd, 0x39c, 0x3b9, 0x3bd, 0x3c3, 0x3ba, 0x39c, 0x3bf,
0x3bd, 0x3b1, 0x3ba, 0x3cc, 0x39c, 0x3cc, 0x3c3, 0x3c7, 0x3b1, 0x38c, 0x3c3, 0x3bb,
0x3bf, 0x3a0, 0x3b1, 0x3c1, 0x3af, 0x3c3, 0x3b9, 0x3a0, 0x3bf, 0x3bd, 0x3c4, 0x3b3,
0x3ba, 0x3cc, 0x3c1, 0x3b9, 0x3c4, 0x3c3, 0x3b1, 0x3a0, 0x3c1, 0x3ac, 0x3b3, 0x3b1,
0x3a1, 0x3af, 0x3b3, 0x3b1, 0x3a1, 0x3ce, 0x3bc, 0x3b7, 0x3a3, 0x3b1, 0x3bc, 0x3ac,
0x3c1, 0x3b1, 0x386, 0x3b3, 0x3b9, 0x3bf, 0x3c2, 0x20, 0x39c, 0x3b1, 0x3c1, 0x3af,
0x3bd, 0x3bf, 0x3c2, 0x3a3, 0x3b1, 0x3c1, 0x3ac, 0x3b3, 0x3b5, 0x3b2, 0x3bf, 0x3a3,
0x3b1, 0x3c1, 0x3ac, 0x3c4, 0x3bf, 0x3c6, 0x3a3, 0x3c5, 0x3bc, 0x3c6, 0x3b5, 0x3c1,
0x3cc, 0x3c0, 0x3bf, 0x3bb, 0x3b7, 0x3a3, 0x3ba, 0x3cc, 0x3c0, 0x3b9, 0x3b1, 0x3a3,
0x3cc, 0x3c6, 0x3b9, 0x3b1, 0x3a3, 0x3c4, 0x3bf, 0x3ba, 0x3c7, 0x3cc, 0x3bb, 0x3bc,
0x3b7, 0x3a4, 0x3b1, 0x3bb, 0x3af, 0x3bd, 0x3a4, 0x3af, 0x3c1, 0x3b1, 0x3bd, 0x3b1,
0x39f, 0x3c5, 0x3bb, 0x3b9, 0x3ac, 0x3bd, 0x3bf, 0x3c6, 0x3c3, 0x3ba, 0x392, 0x3b1,
0x3bd, 0x3c4, 0x3bf, 0x3cd, 0x3b6, 0x392, 0x3b1, 0x3c4, 0x3b9, 0x3ba, 0x3b1, 0x3bd,
0x3cc, 0x392, 0x3b9, 0x3ad, 0x3bd, 0x3bd, 0x3b7, 0x392, 0x3af, 0x3bb, 0x3bd, 0x3b9,
0x3bf, 0x3c5, 0x3c2, 0x392, 0x3cc, 0x3bb, 0x3b3, 0x3ba, 0x3bf, 0x3ba, 0x3c1, 0x3b1,
0x3bd, 0x3c4, 0x392, 0x3b1, 0x3c1, 0x3c3, 0x3bf, 0x3b2, 0x3af, 0x3b1, 0x396, 0x3ac,
0x3b3, 0x3ba, 0x3c1, 0x3b5, 0x3bc, 0x3c0, 0x396, 0x3c5, 0x3c1, 0x3af, 0x3c7, 0x3b7,
0x391, 0x3bd, 0x3c4, 0x3b1, 0x3bd, 0x3b1, 0x3bd, 0x3b1, 0x3c1, 0x3af, 0x3b2, 0x3bf,
0x3a4, 0x3c3, 0x3ac, 0x3b3, 0x3ba, 0x3bf, 0x3c2, 0x39d, 0x3ae, 0x3c3, 0x3bf, 0x3c2,
0x20, 0x3a7, 0x3c1, 0x3b9, 0x3c3, 0x3c4, 0x3bf, 0x3c5, 0x3b3, 0x3ad, 0x3bd, 0x3bd,
0x3c9, 0x3bd, 0x39a, 0x3cc, 0x3ba, 0x3bf, 0x3c2, 0x39a, 0x3bf, 0x3bc, 0x3cc, 0x3c1,
0x3bf, 0x39a, 0x3b5, 0x3c1, 0x3b3, 0x3ba, 0x3b5, 0x3bb, 0x3ad, 0x3bd, 0x39c, 0x3ac,
0x3c7, 0x3b5, 0x39c, 0x3b1, 0x3bb, 0x3b4, 0x3af, 0x3b2, 0x3b5, 0x3c2, 0x39c, 0x3b1,
0x3c5, 0x3c1, 0x3af, 0x3ba, 0x3b9, 0x3bf, 0x3c2, 0x39c, 0x3b1, 0x3b3, 0x3b9, 0x3cc,
0x3c4, 0x3a1, 0x3b5, 0x3ca, 0x3bd, 0x3b9, 0x3cc, 0x3bd, 0x391, 0x3c0, 0x3af, 0x3b1,
0x38c, 0x3ba, 0x3bb, 0x3b1, 0x3bd, 0x3c4, 0x39c, 0x3c0, 0x3bf, 0x3c5, 0x3b3, 0x3ba,
0x3b5, 0x3bd, 0x3b2, 0x3af, 0x3bb, 0x3a4, 0x3c3, 0x3ac, 0x3c4, 0x3b1, 0x3bc, 0x3a4,
0x3c3, 0x3bf, 0x3c5, 0x3ba, 0x39d, 0x3ae, 0x3c3, 0x3bf, 0x3c2, 0x20, 0x3a0, 0x3ac,
0x3c3, 0x3c7, 0x3b1, 0x395, 0x3c6, 0x3ac, 0x3c4, 0x3b5, 0x3a6, 0x3b1, 0x3ba, 0x3b1,
0x3cc, 0x3c6, 0x3bf, 0x3a6, 0x3af, 0x3c4, 0x3b6, 0x3b9, 0x3a6, 0x3bf, 0x3c5, 0x3bd,
0x3b1, 0x3c6, 0x3bf, 0x3cd, 0x3c4, 0x3b9, 0x393, 0x3ba, 0x3b1, 0x3bb, 0x3ac, 0x3c0,
0x3b1, 0x3b3, 0x3ba, 0x3bf, 0x3c2, 0x393, 0x3ba, 0x3ac, 0x3bc, 0x3c0, 0x3b9, 0x3b5,
0x3c1, 0x393, 0x3ba, 0x3bf, 0x3c5, 0x3b1, 0x3bd, 0x3c4, 0x3b1, 0x3bb, 0x3ba, 0x3b1,
0x3bd, 0x3ac, 0x3bb, 0x393, 0x3ba, 0x3bf, 0x3c5, 0x3ac, 0x3bc, 0x3a7, 0x3bf, 0x3bd,
0x3bf, 0x3bb, 0x3bf, 0x3c5, 0x3bb, 0x3bf, 0x3cd, 0x388, 0x3bd, 0x3c4, 0x3b5, 0x3c1,
0x3bc, 0x3c0, 0x3b5, 0x3c1, 0x3b9, 0x39a, 0x3b9, 0x3c1, 0x3b9, 0x3c4, 0x3b9, 0x3bc,
0x3ac, 0x3c4, 0x3b9, 0x39a, 0x3cc, 0x3c3, 0x3c1, 0x3b1, 0x3ca, 0x39a, 0x3bf, 0x3c5,
0x3b1, 0x3c4, 0x3b6, 0x3b1, 0x3bb, 0x3ad, 0x3b9, 0x3bd, 0x39c, 0x3b1, 0x3c4, 0x3b6,
0x3bf, 0x3cd, 0x3c1, 0x3bf, 0x39c, 0x3b1, 0x3c1, 0x3ba, 0x3ad, 0x3b6, 0x3b1, 0x3c2,
0x39c, 0x3af, 0x3bd, 0x3c4, 0x3b3, 0x3bf, 0x3c5, 0x3b5, 0x3ca, 0x39d, 0x3b1, 0x3bf,
0x3cd, 0x3c1, 0x3bf, 0x3c5, 0x39d, 0x3b9, 0x3bf, 0x3cd, 0x3b5, 0x39d, 0x3cc, 0x3c1,
0x3c6, 0x3bf, 0x3bb, 0x3ba, 0x39d, 0x3bf, 0x3c5, 0x3bc, 0x3ad, 0x3b1, 0x3a0, 0x3ac,
0x3b3, 0x3ba, 0x3bf, 0x20, 0x3a0, 0x3ac, 0x3b3, 0x3ba, 0x3bf, 0x3a0, 0x3b1, 0x3bb,
0x3ac, 0x3bf, 0x3c5, 0x3a0, 0x3af, 0x3c4, 0x3ba, 0x3b5, 0x3c1, 0x3bd, 0x3a0, 0x3bf,
0x3bd, 0x3ac, 0x3c0, 0x3b5, 0x3a0, 0x3bf, 0x3c1, 0x3c4, 0x20, 0x39c, 0x3cc, 0x3c1,
0x3b5, 0x3c3, 0x3bc, 0x3c0, 0x3b9, 0x3a1, 0x3b1, 0x3c1, 0x3bf, 0x3c4, 0x3cc, 0x3bd,
0x3b3, 0x3ba, 0x3b1, 0x3a3, 0x3b1, 0x3ca, 0x3c0, 0x3ac, 0x3bd, 0x3a4, 0x3b1, 0x3ca,
0x3c4, 0x3ae, 0x3a4, 0x3b1, 0x3c1, 0x3ac, 0x3bf, 0x3c5, 0x3b1, 0x3a4, 0x3bf, 0x3bd,
0x3b3, 0x3ba, 0x3b1, 0x3c4, 0x3ac, 0x3c0, 0x3bf, 0x3c5, 0x393, 0x3bf, 0x3c5, 0x3ad,
0x3b9, 0x3ba, 0x393, 0x3bf, 0x3c5, 0x3ac, 0x3bb, 0x3b9, 0x3c2, 0xa86, 0xaac, 0xabf,
0xaa6, 0xacd, 0xa9c, 0xabe, 0xaa8, 0xa8d, 0xa95, 0xab0, 0xabe, 0xa8f, 0xaa1, 0xabf,
0xab8, 0x20, 0xa85, 0xaac, 0xabe, 0xaac, 0xabe, 0xa85, 0xab2, 0xacd, 0xa9c, 0xac0,
0xaaf, 0xab0, 0xacd, 0xab8, 0xa85, 0xab8, 0xacd, 0xaae, 0xabe, 0xab0, 0xabe, 0xaac,
0xaae, 0xac7, 0xa95, 0xacb, 0xaac, 0xabe, 0xa82, 0xa97, 0xac1, 0xa88, 0xaac, 0xabe,
0xa82, 0xa9c, 0xac1, 0xab2, 0xaac, 0xabf, 0xab8, 0xabe, 0xa89, 0xaac, 0xacd, 0xab2,
0xac7, 0xa82, 0xa9f, 0xac0, 0xab0, 0xaac, 0xacd, 0xab0, 0xabe, 0xa9d, 0xabe, 0xab5,
0xabf, 0xab2, 0xac7, 0xaac, 0xac1, 0xa9c, 0xaae, 0xacd, 0xaac, 0xac1, 0xab0, 0xabe,
0xa95, 0xac8, 0xab0, 0xacb, 0xa95, 0xabe, 0xab8, 0xabe, 0xaac, 0xacd, 0xab2, 0xabe,
0xa82, 0xa95, 0xabe, 0xa95, 0xacd, 0xab5, 0xac7, 0xa9f, 0xabe, 0xa95, 0xacb, 0xaa8,
0xac5, 0xa95, 0xacd, 0xab0, 0xac0, 0xaa1, 0xa95, 0xabe, 0xab0, 0xaa6, 0xabe, 0xab0,
0x20, 0xa8f, 0xab8, 0x20, 0xab8, 0xab2, 0xabe, 0xaae, 0xa9c, 0xac0, 0xaac, 0xac1,
0xa9f, 0xac0, 0xaa1, 0xacc, 0xa86, 0xab2, 0xabe, 0xa8f, 0xab2, 0x20, 0xa90, 0xa89,
0xaa8, 0xaab, 0xacd, 0xab0, 0xac0, 0xa9f, 0xabe, 0xa89, 0xaa8, 0xa97, 0xac5, 0xaac,
0xacb, 0xab0, 0xacb, 0xaa8, 0xab9, 0xab0, 0xabe, 0xab0, 0xac7, 0xa9c, 0xacb, 0xab9,
0xabe, 0xaa8, 0xacd, 0xab8, 0xaac, 0xab0, 0xacd, 0xa97, 0xa9c, 0xac1, 0xaac, 0xabe,
0xa95, 0xaae, 0xacd, 0xaaa, 0xabe, 0xab2, 0xabe, 0xa96, 0xabe, 0xab0, 0xacd, 0xa9f,
0xac2, 0xaae, 0xa95, 0xabf, 0xa97, 0xabe, 0xab2, 0xac0, 0xa95, 0xabf, 0xa82, 0xab6,
0xabe, 0xab8, 0xabe, 0xab2, 0xabe, 0xa97, 0xacb, 0xab8, 0xab2, 0xabf, 0xaac, 0xacd,
0xab0, 0xac7, 0xab5, 0xabf, 0xab2, 0xac7, 0xab2, 0xacb, 0xaae, 0xab2, 0xacd, 0xaaf,
0xac1, 0xa8f, 0xaa8, 0xacd, 0xaa1, 0xabe, 0xab2, 0xac1, 0xaac, 0xac1, 0xaae, 0xacd,
0xaac, 0xabe, 0xab6, 0xac0, 0xab2, 0xac1, 0xab8, 0xabe, 0xa95, 0xabe, 0xaae, 0xab2,
0xabe, 0xaac, 0xacb, 0xaae, 0xabe, 0xaaa, 0xac1, 0xa9f, 0xacb, 0xaae, 0xab8, 0xac7,
0xab0, 0xac1, 0xa85, 0xaae, 0xacd, 0xaac, 0xaac, 0xabe, 0xaa8, 0xaae, 0xacb, 0xa97,
0xabe, 0xaa1, 0xac0, 0xab6, 0xac1, 0xaae, 0xacb, 0xa82, 0xab0, 0xacb, 0xab5, 0xabf,
0xaaf, 0xabe, 0xaa8, 0xac8, 0xab0, 0xacb, 0xaac, 0xac0, 0xa85, 0xaa8, 0xacd, 0xa9c,
0xaae, 0xac7, 0xaa8, 0xaa8, 0xabf, 0xaaf, 0xabe, 0xaae, 0xac0, 0xaa8, 0xacc, 0xa95,
0xa9a, 0xacb, 0xa9f, 0xa94, 0xa86, 0xa97, 0xabe, 0xaa6, 0xacc, 0xa97, 0xacc, 0xaaa,
0xacb, 0xab0, 0xacd, 0xa9f, 0xacb, 0x2d, 0xaa8, 0xacb, 0xab5, 0xacb, 0xab8, 0xabe,
0xa93, 0x20, 0xa9f, 0xac9, 0xaae, 0xa9f, 0xacd, 0xab0, 0xac0, 0xaaa, 0xacb, 0xab2,
0xac0, 0xa9f, 0xacd, 0xaaf, 0xac1, 0xaa8, 0xabf, 0xab8, 0xab5, 0xabf, 0xa82, 0xaa1,
0xab9, 0xac9, 0xa95, 0xa85, 0xaa6, 0xa95, 0xa8f, 0xaa8, 0xacd, 0xa95, 0xacb, 0xab0,
0xac7, 0xa9c, 0xa8d, 0xa82, 0xa97, 0xacd, 0xab5, 0xabf, 0xab2, 0xabe, 0xa8d, 0xaa8,
0xacd, 0xa9f, 0xabf, 0xa97, 0xac1, 0xa86, 0xa85, 0xab0, 0xabe, 0xa97, 0xac1, 0xa87,
0xaa8, 0xabe, 0xaac, 0xacd, 0xaaf, 0xac1, 0xaa8, 0xab8, 0x20, 0xa8f, 0xaaf, 0xab0,
0xacd, 0xab8, 0xa95, 0xabe, 0xaa4, 0xabe, 0xaae, 0xabe, 0xab0, 0xacd, 0xa95, 0xabe,
0xa95, 0xacb, 0xab0, 0xacd, 0xaa1, 0xacb, 0xaac, 0xabe, 0xab9, 0xac1, 0xab9, 0xac1,
0xa87, 0xab2, 0xabe, 0x20, 0xab0, 0xabf, 0xa93, 0xab9, 0xabe, 0xaae, 0xac7, 0xaa8,
0xacd, 0xaa1, 0xacb, 0xa9d, 0xabe, 0xab0, 0xabf, 0xa93, 0x20, 0xa97, 0xac7, 0xab2,
0xac7, 0xa97, 0xacb, 0xab8, 0xab8, 0xabe, 0xab2, 0xacd, 0xa9f, 0xabe, 0xab8, 0xac7,
0xaa8, 0x20, 0xa9c, 0xac1, 0xa8f, 0xaa8, 0xab8, 0xac7, 0xaa8, 0x20, 0xab2, 0xac1,
0xa87, 0xab8, 0xaa4, 0xac1, 0xa95, 0xac1, 0xaae, 0xabe, 0xaa8, 0xa89, 0xab6, 0xabe,
0xaaf, 0xabe, 0xa85, 0xab0, 0xac1, 0xaac, 0xabe, 0xa8f, 0xab8, 0xaa8, 0xacd, 0xab6,
0xabf, 0xaaf, 0xaa8, 0xa8f, 0xa9f, 0xabf, 0xa95, 0xacb, 0xa95, 0xac7, 0xaa8, 0xaac,
0xabe, 0xab9, 0xabf, 0xaaf, 0xabe, 0xaac, 0xab9, 0xabf, 0xaaf, 0xabe, 0x20, 0xaac,
0xabe, 0xa82, 0xaa6, 0xacd, 0xab0, 0xabe, 0xab8, 0xaac, 0xabe, 0xab0, 0xaac, 0xabe,
0xaa1, 0xacb, 0xab8, 0xaac, 0xac7, 0xab2, 0xac7, 0xaae, 0xaac, 0xac7, 0xab2, 0xac0,
0xa9d, 0xaac, 0xacd, 0xab2, 0xabe, 0xa82, 0xa95, 0x2d, 0xab8, 0xac7, 0xaac, 0xab2,
0xacb, 0xaa8, 0xaac, 0xacb, 0xa86, 0x20, 0xab5, 0xabf, 0xab8, 0xacd, 0xa9f, 0xabe,
0xaac, 0xacb, 0xa97, 0xacb, 0xa9f, 0xabe, 0xaac, 0xacb, 0xa87, 0xa9d, 0xa95, 0xac7,
0xaae, 0xacd, 0xaac, 0xacd, 0xab0, 0xabf, 0xa9c, 0x20, 0xaac, 0xac7, 0xa95, 0xabe,
0xaae, 0xacd, 0xaaa, 0xacb, 0x20, 0xa97, 0xacd, 0xab0, 0xabe, 0xa82, 0xaa1, 0xa95,
0xabe, 0xaa8, 0xacd, 0xa95, 0xac1, 0xaa8, 0xa95, 0xac5, 0xab0, 0xabe, 0xa95, 0xab8,
0xa95, 0xabe, 0xaaf, 0xac7, 0xaa8, 0xac7, 0xa95, 0xac7, 0xaae, 0xac7, 0xaa8, 0xab6,
0xabf, 0xa95, 0xabe, 0xa97, 0xacb, 0xa9a, 0xabf, 0xab9, 0xac1, 0xa86, 0xab9, 0xac1,
0xa86, 0xab8, 0xabf, 0xaaf, 0xac1, 0xaa6, 0xabe, 0xaa6, 0x20, 0xab9, 0xac1, 0xa86,
0xab0, 0xac7, 0xa9d, 0xa95, 0xacb, 0xab8, 0xacd, 0xa9f, 0xabe, 0x20, 0xab0, 0xabf,
0xa95, 0xabe, 0xa95, 0xacd, 0xab0, 0xac7, 0xab8, 0xacd, 0xa9f, 0xaa8, 0xa95, 0xacd,
0xaaf, 0xac1, 0xa8f, 0xaac, 0xabe, 0xa95, 0xac1, 0xab0, 0xabe, 0xa95, 0xabe, 0xa93,
0xaa1, 0xac7, 0xaa8, 0xaae, 0xabe, 0xab0, 0xacd, 0xa95, 0xab6, 0xac9, 0xaa8, 0xaa1,
0xacc, 0xab8, 0xaa8, 0xaa1, 0xacc, 0xab8, 0xaa8, 0x20, 0xa95, 0xacd, 0xab0, 0xabf,
0xa95, 0xaa6, 0xac7, 0xaa8, 0xacd, 0xab5, 0xab0, 0xaa1, 0xac7, 0xa9f, 0xacd, 0xab0,
0xacb, 0xa87, 0xa9f, 0xaa1, 0xacb, 0xaae, 0xabf, 0xaa8, 0xabf, 0xa95, 0xabe, 0xa8f,
0xaa1, 0xaae, 0xac7, 0xaa8, 0xacd, 0xa9f, 0xaa8, 0xa87, 0xab0, 0xac1, 0xaa8, 0xac7,
0xaaa, 0xa8f, 0xab2, 0x20, 0xab8, 0xac7, 0xab2, 0xacd, 0xab5, 0xabe, 0xaa1, 0xacb,
0xab0, 0xaab, 0xacb, 0xab0, 0xacd, 0xa9f, 0x20, 0xaa8, 0xac7, 0xab2, 0xacd, 0xab8,
0xaa8, 0xaab, 0xacb, 0xab0, 0xacd, 0xa9f, 0xabe, 0xab2, 0xac7, 0xa9d, 0xabe, 0xa97,
0xacd, 0xab2, 0xac7, 0xab8, 0x20, 0xaac, 0xac7, 0xa97, 0xac2, 0xab8, 0x20, 0xaac,
0xac7, 0xa97, 0xacd, 0xab0, 0xabe, 0xaa8, 0xacd, 0xaa1, 0x20, 0xa9f, 0xab0, 0xacd,
0xa95, 0xa97, 0xacd, 0xab0, 0xac7, 0xaa8, 0xac7, 0xaa1, 0xabe, 0xa97, 0xacd, 0xab5,
0xabe, 0xaa1, 0xac7, 0xab2, 0xacb, 0xaaa, 0xa97, 0xacd, 0xab5, 0xabe, 0xa9f, 0xac7,
0xaae, 0xabe, 0xab2, 0xabe, 0xa97, 0xac1, 0xaaf, 0xabe, 0xa95, 0xacd, 0xab5, 0xabf,
0xab2, 0xa97, 0xaaf, 0xabe, 0xaa8, 0xabe, 0xab9, 0xac7, 0xab2, 0xabf, 0xaab, 0xac7,
0xa95, 0xacd, 0xab8, 0xab9, 0xab5, 0xabe, 0xaa8, 0xabe, 0xa8f, 0xab0, 0xaae, 0xacb,
0xab8, 0xabf, 0xaaf, 0xacb, 0xa87, 0xaa8, 0xacd, 0xaa1, 0xabf, 0xaaf, 0xabe, 0xaa8,
0xabe, 0xaaa, 0xacb, 0xab2, 0xabf, 0xab8, 0xaa8, 0xacb, 0xa95, 0xacd, 0xab8, 0x2c,
0x20, 0xa87, 0xaa8, 0xacd, 0xaa1, 0xabf, 0xaaf, 0xabe, 0xaa8, 0xabe, 0xaae, 0xac7,
0xab0, 0xac7, 0xa82, 0xa97, 0xacb, 0x2c, 0x20, 0xa87, 0xaa8, 0xacd, 0xaa1, 0xabf,
0xaaf, 0xabe, 0xaa8, 0xabe, 0xaaa, 0xabf, 0xa9f, 0xab0, 0xacd, 0xab8, 0xaac, 0xab0,
0xacd, 0xa97, 0x2c, 0x20, 0xa87, 0xaa8, 0xacd, 0xaa1, 0xabf, 0xaaf, 0xabe, 0xaa8,
0xabe, 0xa9f, 0xac7, 0xab2, 0x20, 0xab8, 0xabf, 0xa9f, 0xac0, 0x2c, 0x20, 0xa87,
0xaa8, 0xacd, 0xaa1, 0xabf, 0xaaf, 0xabe, 0xaa8, 0xabe, 0xab5, 0xac7, 0xab5, 0xac8,
0x2c, 0x20, 0xa87, 0xaa8, 0xacd, 0xaa1, 0xabf, 0xaaf, 0xabe, 0xaa8, 0xabe, 0xab5,
0xabf, 0xaa8, 0xacd, 0xab8, 0xac7, 0xaa8, 0xacd, 0xab8, 0x2c, 0x20, 0xa87, 0xaa8,
0xacd, 0xaa1, 0xabf, 0xaaf, 0xabe, 0xaa8, 0xabe, 0xab5, 0xabf, 0xaa8, 0xabe, 0xaae,
0xac7, 0xa95, 0x2c, 0x20, 0xa87, 0xaa8, 0xacd, 0xaa1, 0xabf, 0xaaf, 0xabe, 0xaa8,
0xabe, 0xa87, 0xaa8, 0xac1, 0xab5, 0xabf, 0xa95, 0xa87, 0xa95, 0xabe, 0xab2, 0xac1,
0xa87, 0xaa4, 0xa9c, 0xaae, 0xac8, 0xa95, 0xabe, 0xa9c, 0xac1, 0xaa8, 0xac7, 0xa87,
0xab2, 0xac2, 0xa87, 0xab5, 0xabf, 0xab2, 0xaae, 0xacb, 0xaa8, 0xacd, 0xa9f, 0xac0,
0xab8, 0xac7, 0xab2, 0xacb, 0x2c, 0x20, 0xa95, 0xac7, 0xaa8, 0xacd, 0xa9f, 0xac1,
0xa95, 0xac0, 0xa95, 0xacd, 0xab0, 0xabe, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0xabf,
0xa95, 0xab2, 0xabe, 0x20, 0xaaa, 0xac7, 0xa9d, 0xab2, 0xabf, 0xaae, 0xabe, 0xab2,
0xacb, 0xab8, 0x20, 0xa8f, 0xa82, 0xa9c, 0xac7, 0xab2, 0xac7, 0xab8, 0xab2, 0xacb,
0xa85, 0xab0, 0x20, 0xaaa, 0xacd, 0xab0, 0xabf, 0xaa8, 0xacd, 0xab8, 0xaa8, 0xac1,
0xa82, 0x20, 0xa95, 0xacd, 0xab5, 0xabe, 0xab0, 0xacd, 0xa9f, 0xab0, 0xaae, 0xac7,
0xab8, 0xabf, 0xa93, 0xaae, 0xabe, 0xaa8, 0xabe, 0xa97, 0xac1, 0xa86, 0xaae, 0xaa8,
0xacc, 0xab8, 0xaae, 0xac7, 0xab0, 0xac0, 0xa97, 0xacb, 0xa9f, 0xaae, 0xabe, 0xab0,
0xacd, 0xa9f, 0xac0, 0xaa8, 0xac0, 0xa95, 0xaae, 0xabe, 0xaa4, 0xabe, 0xaae, 0xacb,
0xab0, 0xacb, 0xab8, 0xaae, 0xa9d, 0xaa4, 0xacd, 0xab2, 0xabe, 0xaa8, 0xaae, 0xac7,
0xaa8, 0xacb, 0xaae, 0xabf, 0xaa8, 0xac0, 0xaae, 0xac7, 0xab0, 0xabf, 0xaa6, 0xabe,
0xaae, 0xac7, 0xa9f, 0xacd, 0xab2, 0xabe, 0xa95, 0xabe, 0xa9f, 0xacd, 0xab2, 0xabe,
0xaae, 0xac7, 0xa95, 0xacd, 0xab8, 0xabf, 0xa95, 0xacb, 0x20, 0xab8, 0xabf, 0xa9f,
0xac0, 0xaae, 0xabf, 0xa95, 0xacd, 0xaaf, 0xac1, 0xab2, 0xacb, 0xaa8, 0xaae, 0xacb,
0xaa8, 0xacd, 0xa95, 0xa9f, 0xaa8, 0xaae, 0xacb, 0xaa8, 0xacd, 0xaa4, 0xacb, 0xab0,
0xac7, 0xaae, 0xac9, 0xa82, 0xa9f, 0xac7, 0xab5, 0xabf, 0xaa1, 0xabf, 0xa93, 0xaae,
0xacb, 0xa82, 0xa9f, 0xab8, 0xac7, 0xab0, 0xabe, 0xaa4, 0xaa8, 0xabe, 0xab8, 0xabe,
0xa89, 0xaa8, 0xacd, 0xaaf, 0xac2, 0xaaf, 0xacb, 0xab0, 0xacd, 0xa95, 0xaa8, 0xacb,
0xab0, 0xacb, 0xaa8, 0xacd, 0xab9, 0xabe, 0xaac, 0xabf, 0xaaf, 0xac1, 0xab2, 0xabe,
0xab9, 0x2c, 0x20, 0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0, 0x20, 0xaa1, 0xac7, 0xa95,
0xacb, 0xa9f, 0xabe, 0xab8, 0xac7, 0xaa8, 0xacd, 0xa9f, 0xab0, 0x2c, 0x20, 0xa89,
0xaa4, 0xacd, 0xaa4, 0xab0, 0x20, 0xaa1, 0xac7, 0xa95, 0xacb, 0xa9f, 0xabe, 0xaa8,
0xacd, 0xaaf, 0xac1, 0x20, 0xab8, 0xac7, 0xab2, 0xaae, 0x2c, 0x20, 0xa89, 0xaa4,
0xacd, 0xaa4, 0xab0, 0x20, 0xaa1, 0xac7, 0xa95, 0xacb, 0xa9f, 0xabe, 0xaa8, 0xac2,
0xa95, 0xa93, 0xa9c, 0xabf, 0xaa8, 0xabe, 0xa97, 0xabe, 0xaaa, 0xaa8, 0xabe, 0xaae,
0xabe, 0xaaa, 0xac7, 0xab0, 0xabe, 0xaae, 0xac7, 0xab0, 0xabf, 0xaac, 0xacb, 0xaab,
0xacb, 0xaa8, 0xabf, 0xa95, 0xacd, 0xab8, 0xaaa, 0xacb, 0xab0, 0xacd, 0xa9f, 0x2d,
0xa93, 0x2d, 0xaaa, 0xacd, 0xab0, 0xabf, 0xaa8, 0xacd, 0xab8, 0xaaa, 0xacb, 0xab0,
0xacd, 0xa9f, 0x20, 0xa91, 0xaab, 0x20, 0xab8, 0xacd, 0xaaa, 0xac7, 0xaa8, 0xaaa,
0xacb, 0xab0, 0xacd, 0xa9f, 0xacb, 0x20, 0xab5, 0xac7, 0xab2, 0xacd, 0xab9, 0xacb,
0xaaa, 0xacd, 0xaaf, 0xac1, 0xa85, 0xab0, 0xacd, 0xa9f, 0xacb, 0x20, 0xab0, 0xabf,
0xa95, 0xacb, 0xaaa, 0xac1, 0xaa8, 0xacd, 0xa9f, 0xabe, 0x20, 0xa8f, 0xab0, 0xac0,
0xaa8, 0xabe, 0xa9d, 0xab0, 0xac7, 0xa82, 0xa95, 0xabf, 0xaa8, 0x20, 0xa87, 0xaa8,
0xacd, 0xab2, 0xac7, 0xa9f, 0xab0, 0xac7, 0xab8, 0xac0, 0xaab, 0xab0, 0xac7, 0xa9c,
0xac0, 0xaa8, 0xabe, 0xab0, 0xac7, 0xa9d, 0xacb, 0xab2, 0xac2, 0xa9f, 0xab0, 0xabf,
0xaaf, 0xacb, 0x20, 0xaac, 0xacd, 0xab0, 0xabe, 0xa82, 0xa95, 0xacb, 0xab8, 0xac7,
0xaa8, 0xacd, 0xaa4, 0xabe, 0xab0, 0xac7, 0xaae, 0xab8, 0xabe, 0xa82, 0xa9f, 0xabf,
0xaaf, 0xabe, 0xa97, 0xacb, 0xab8, 0xac7, 0xa82, 0xa9f, 0xacb, 0x20, 0xaa1, 0xacb,
0xaae, 0xabf, 0xa82, 0xa97, 0xacb, 0xab8, 0xabe, 0xa93, 0x20, 0xaaa, 0xabe, 0xa89,
0xab2, 0xacb, 0xa87, 0xaa4, 0xacb, 0xa95, 0xacb, 0xab0, 0xacd, 0xa9f, 0xacb, 0xab0,
0xacb, 0xaae, 0xabf, 0xa9f, 0xab8, 0xabf, 0xa9f, 0xacd, 0xa95, 0xabe, 0xab8, 0xac7,
0xa82, 0xa9f, 0x20, 0xaac, 0xabe, 0xab0, 0xacd, 0xaa5, 0xac7, 0xab2, 0xac7, 0xaae,
0xac0, 0xab8, 0xac7, 0xa82, 0xa9f, 0x20, 0xa9c, 0xacd, 0xab9, 0xacb, 0xaa8, 0xacd,
0xab8, 0xab8, 0xac7, 0xa82, 0xa9f, 0x20, 0xa95, 0xac0, 0xa9f, 0xacd, 0xab8, 0xab8,
0xac7, 0xa82, 0xa9f, 0x20, 0xab2, 0xac1, 0xab8, 0xabf, 0xaaf, 0xabe, 0xab8, 0xac7,
0xa82, 0xa9f, 0x20, 0xaa5, 0xacb, 0xaae, 0xab8, 0xab8, 0xac7, 0xa82, 0xa9f, 0x20,
0xab5, 0xabf, 0xaa8, 0xacd, 0xab8, 0xac7, 0xaa8, 0xacd, 0xa9f, 0xab8, 0xacd, 0xab5,
0xabf, 0xaab, 0xacd, 0xa9f, 0x20, 0xa95, 0xab0, 0xa82, 0xa9f, 0xaa4, 0xac7, 0xa97,
0xac1, 0xab8, 0xabf, 0xa97, 0xab2, 0xacd, 0xaaa, 0xabe, 0xaa5, 0xac1, 0xab2, 0xac7,
0xaa4, 0xabf, 0xa9c, 0xac1, 0xa86, 0xaa8, 0xabe, 0xa9f, 0xacb, 0xab0, 0xaa8, 0xacd,
0xa9f, 0xacb, 0xa9f, 0xacb, 0xab0, 0xacd, 0xa9f, 0xacb, 0xab2, 0xabe, 0xab5, 0xac7,
0xaa8, 0xacd, 0xa95, 0xac1, 0xab5, 0xab0, 0xab5, 0xacd, 0xab9, 0xabe, 0xa87, 0xa9f,
0xab9, 0xacb, 0xab0, 0xacd, 0xab8, 0xab5, 0xabf, 0xaa8, 0xacd, 0xaa8, 0xabf, 0xaaa,
0xac7, 0xa97, 0xaaf, 0xa95, 0xac1, 0xaa4, 0xaa4, 0xa95, 0xac5, 0xab8, 0xac0, 0xaa1,
0xac7, 0xab5, 0xabf, 0xab8, 0xaa6, 0xac1, 0xaae, 0xacb, 0xaa8, 0xacd, 0xaa4, 0x20,
0xaa6, 0xac0, 0x2018, 0xa89, 0xab0, 0xacd, 0xab5, 0xabf, 0xab2, 0xaae, 0xac7, 0xa95,
0xacd, 0xab5, 0xac7, 0xab0, 0xac0, 0xaae, 0xacb, 0xab8, 0xaa8, 0xaae, 0xac5, 0xa95,
0xaae, 0xab0, 0xacd, 0xaa1, 0xacb, 0xaaa, 0xabe, 0xab2, 0xacd, 0xaae, 0xab0, 0xab0,
0xacb, 0xaa5, 0xac7, 0xab0, 0xabe, 0xab8, 0xacd, 0xaaf, 0xacb, 0xab5, 0xabe, 0xa9f,
0xacd, 0xab0, 0xacb, 0xab2, 0xab5, 0xacb, 0xab8, 0xacd, 0xa9f, 0xac9, 0xa95, 0xab2,
0xacb, 0xa82, 0xa97, 0xa87, 0xaaf, 0xab0, 0xaac, 0xabf, 0xaaf, 0xac7, 0xaa8, 0xa8f,
0xaa6, 0xac7, 0xaa8, 0xa85, 0xab2, 0xacd, 0xaae, 0xabe, 0xa9f, 0xac0, 0xa85, 0xaae,
0xacd, 0xaae, 0xabe, 0xaa8, 0xa85, 0xaa8, 0xaa6, 0xabf, 0xab0, 0xa85, 0xa95, 0xa9f,
0xabe, 0xa89, 0xa8d, 0xa95, 0xacd, 0xa9f, 0xacb, 0xaac, 0xa85, 0xab6, 0xa97, 0xabe,
0xaac, 0xa9f, 0xa85, 0xaa4, 0xacd, 0xaaf, 0xabe, 0xab0, 0xac1, 0xaac, 0xa97, 0xaa6,
0xabe, 0xaa6, 0xaac, 0xac7, 0xab9, 0xab0, 0xac0, 0xaa8, 0xaac, 0xac7, 0xa82, 0xa97,
0xa95, 0xacb, 0xa95, 0xaac, 0xabe, 0xab0, 0xaa8, 0xacc, 0xab2, 0xaac, 0xac8, 0xab0,
0xac1, 0xaa4, 0xaac, 0xabf, 0xab6, 0xa95, 0xac7, 0xa95, 0xaac, 0xacd, 0xab0, 0xac1,
0xaa8, 0xac7, 0xa87, 0xa9a, 0xabf, 0xaa4, 0xabe, 0xa95, 0xacb, 0xab2, 0xa82, 0xaac,
0xacb, 0xaa6, 0xaae, 0xabe, 0xab8, 0xacd, 0xa95, 0xab8, 0xaa2, 0xabe, 0xa95, 0xabe,
0xaa6, 0xabf, 0xab2, 0xabf, 0xaa6, 0xac1, 0xaac, 0xa88, 0xaa6, 0xac1, 0xab6, 0xabe,
0xaae, 0xacd, 0xaac, 0xac7, 0xaab, 0xabe, 0xaae, 0xabe, 0xa97, 0xac1, 0xab8, 0xacd,
0xaa4, 0xabe, 0xa97, 0xabe, 0xa9d, 0xabe, 0xab9, 0xac7, 0xaac, 0xacd, 0xab0, 0xacb,
0xaa8, 0xab9, 0xacb, 0x20, 0xa9a, 0xac0, 0x20, 0xaae, 0xac0, 0xaa8, 0x20, 0xab8,
0xabf, 0xa9f, 0xac0, 0xab9, 0xacb, 0xa82, 0xa97, 0xa95, 0xacb, 0xa82, 0xa97, 0xab9,
0xacb, 0xab5, 0xacd, 0xaa1, 0xa87, 0xab0, 0xacd, 0xa95, 0xac1, 0xaa4, 0xacd, 0xab8,
0xacd, 0xa95, 0xa9c, 0xa95, 0xabe, 0xab0, 0xacd, 0xaa4, 0xabe, 0xa9c, 0xaaf, 0xabe,
0xaaa, 0xac1, 0xab0, 0xabe, 0xa9c, 0xac7, 0xab0, 0xac1, 0xab8, 0xab2, 0xac7, 0xaae,
0xa95, 0xabe, 0xaac, 0xac1, 0xab2, 0xa95, 0xabe, 0xaae, 0xa9a, 0xa9f, 0xacd, 0xa95,
0xabe, 0xa95, 0xab0, 0xabe, 0xa9a, 0xac0, 0xa95, 0xabe, 0xaa0, 0xaae, 0xa82, 0xaa1,
0xac1, 0xa96, 0xa82, 0xaa1, 0xabf, 0xa97, 0xabe, 0xa95, 0xacb, 0xab2, 0xa95, 0xabe,
0xaa4, 0xabe, 0xa95, 0xacd, 0xab0, 0xabf, 0xab8, 0xacd, 0xaa8, 0xacb, 0xaaf, 0xabe,
0xab0, 0xacd, 0xab8, 0xacd, 0xa95, 0xa95, 0xac1, 0xa86, 0xab2, 0xabe, 0x20, 0xab2,
0xac1, 0xaae, 0xacd, 0xaaa, 0xac1, 0xab0, 0xa95, 0xac1, 0xa9a, 0xabf, 0xa82, 0xa97,
0xa95, 0xac1, 0xab5, 0xac8, 0xaa4, 0xaae, 0xa95, 0xabe, 0xa89, 0xaae, 0xac7, 0xa97,
0xabe, 0xaa1, 0xaa8, 0xaae, 0xabe, 0xa95, 0xab8, 0xacd, 0xab8, 0xab0, 0xaae, 0xaa8,
0xabf, 0xab2, 0xabe, 0xaae, 0xab8, 0xacd, 0xa95, 0xaa4, 0xaa8, 0xabf, 0xa95, 0xacb,
0xab8, 0xabf, 0xaaf, 0xabe, 0xaa8, 0xacb, 0xab5, 0xacb, 0xa95, 0xac1, 0xa9d, 0xacd,
0xaa8, 0xac7, 0xaa4, 0xacd, 0xab8, 0xacd, 0xa95, 0xaa8, 0xacb, 0xab5, 0xacb, 0xab8,
0xac0, 0xab0, 0xacd, 0xaac, 0xabf, 0xab0, 0xacd, 0xab8, 0xacd, 0xa95, 0xa93, 0xaae,
0xacd, 0xab8, 0xacd, 0xa95, 0xa93, 0xab0, 0xab2, 0xaab, 0xacd, 0xaa8, 0xacb, 0xaae,
0x20, 0xaaa, 0xac7, 0xaa8, 0xacd, 0xab9, 0xaaa, 0xacb, 0xaa8, 0xacd, 0xa9f, 0xabf,
0xaaf, 0xaa8, 0xac7, 0xa95, 0xaaa, 0xacd, 0xaaf, 0xacb, 0xa82, 0xa97, 0xaaf, 0xabe,
0xa82, 0xa97, 0xa95, 0xaa4, 0xabe, 0xab0, 0xa95, 0xacb, 0xab8, 0xacd, 0xa9f, 0xabe,
0xaa8, 0xac7, 0xa95, 0xabf, 0xa9d, 0xabf, 0xab2, 0xacb, 0xab0, 0xacd, 0xaa1, 0xabe,
0xab0, 0xabf, 0xaaf, 0xabe, 0xaa7, 0xab8, 0xa96, 0xabe, 0xab2, 0xabf, 0xaa8, 0xab8,
0xaae, 0xab0, 0xa95, 0xaa8, 0xacd, 0xaa1, 0xab8, 0xac0, 0xa93, 0xab2, 0xab6, 0xabe,
0xa82, 0xa98, 0xabe, 0xa88, 0xab8, 0xabf, 0xa82, 0xa97, 0xabe, 0xaaa, 0xac1, 0xab0,
0xab8, 0xacd, 0xab0, 0xac7, 0xaa1, 0xacd, 0xaa8, 0xac7, 0xa95, 0xacb, 0xab2, 0xac0,
0xaae, 0xacd, 0xab8, 0xacd, 0xa95, 0xaa4, 0xabe, 0xa87, 0xaaa, 0xac7, 0xa87, 0xaa4,
0xabe, 0xab6, 0xa95, 0xac7, 0xaa8, 0xacd, 0xa9f, 0xaac, 0xabf, 0xab2, 0xabf, 0xab8,
0xabf, 0xaa4, 0xac7, 0xab9, 0xab0, 0xabe, 0xaa8, 0xaa5, 0xac0, 0xaae, 0xacd, 0xaab,
0xac1, 0xa9f, 0xacb, 0xa95, 0xacd, 0xaaf, 0xacb, 0xaa4, 0xacb, 0xaae, 0xab8, 0xacd,
0xa95, 0xa89, 0xab2, 0xabe, 0xa82, 0xaac, 0xabe, 0xaa4, 0xab0, 0xa89, 0xab0, 0xac1,
0xaae, 0xacd, 0xa95, 0xac0, 0xa89, 0xab8, 0xacd, 0xaa4, 0x2d, 0xaa8, 0xac7, 0xab0,
0xabe, 0xab5, 0xabf, 0xaaf, 0xac7, 0xaa8, 0xa9f, 0xabf, 0xaaf, 0xac7, 0xaa8, 0xab5,
0xacd, 0xab2, 0xac7, 0xaa1, 0xabf, 0xab5, 0xacb, 0xab8, 0xacd, 0xa9f, 0xacb, 0xa95,
0xaaf, 0xa95, 0xac1, 0xaa4, 0xacd, 0xab8, 0xa95, 0xab0, 0xa82, 0xa97, 0xac2, 0xaa8,
0xaaf, 0xac7, 0xa95, 0xabe, 0xaa4, 0xac7, 0xab0, 0xabf, 0xaa8, 0xaac, 0xab0, 0xacd,
0xa97, 0xaaf, 0xac7, 0xab0, 0xac7, 0xab5, 0xabe, 0xaa8, 0xa8f, 0xa9d, 0xacb, 0xab0,
0xac7, 0xa9d, 0xaac, 0xab0, 0xacd, 0xaae, 0xac1, 0xaa1, 0xabe, 0xa95, 0xac5, 0xaa8,
0xac7, 0xab0, 0xac0, 0xa95, 0xac5, 0xaaa, 0x20, 0xab5, 0xab0, 0xacd, 0xaa1, 0xac7,
0xaab, 0xac7, 0xab0, 0xacb, 0xaae, 0xaa1, 0xabf, 0xab0, 0xabe, 0xab0, 0xac5, 0xa95,
0xaaf, 0xabe, 0xab5, 0xabf, 0xa95, 0xaa6, 0xa95, 0xacd, 0xab7, 0xabf, 0xaa3, 0x20,
0xa9c, 0xacd, 0xaaf, 0xacb, 0xab0, 0xacd, 0xa9c, 0xabf, 0xaaf, 0xabe, 0xab8, 0xac7,
0xa82, 0xa9f, 0x20, 0xab9, 0xac7, 0xab2, 0xac7, 0xaa8, 0xabe, 0xab8, 0xacd, 0xa9f,
0xac7, 0xaa8, 0xab2, 0xac0, 0xa8f, 0xaa1, 0xabf, 0xab2, 0xac7, 0xa87, 0xaa1, 0xaac,
0xacd, 0xab0, 0xabf, 0xab8, 0xaac, 0xac7, 0xaa8, 0xaac, 0xacd, 0xab0, 0xacb, 0xa95,
0xaa8, 0x20, 0xab9, 0xabf, 0xab2, 0xaa1, 0xabe, 0xab0, 0xacd, 0xab5, 0xabf, 0xaa8,
0xa89, 0xa95, 0xacd, 0xab2, 0xabe, 0xab9, 0xacb, 0xaac, 0xabe, 0xab0, 0xacd, 0xa9f,
0xab2, 0xabf, 0xaa8, 0xacd, 0xaa1, 0xac7, 0xaae, 0xac7, 0xaa8, 0xab2, 0xacb, 0xab0,
0xacd, 0xaa1, 0x20, 0xab9, 0xacb, 0xab5, 0xaae, 0xac7, 0xab2, 0xaac, 0xacb, 0xab0,
0xacd, 0xaa8, 0xaaa, 0xab0, 0xacd, 0xaa5, 0xab8, 0xabf, 0xaa1, 0xaa8, 0xac0, 0xa8d,
0xaae, 0xacd, 0xab8, 0xacd, 0xa9f, 0xab0, 0xaa1, 0xac5, 0xaae, 0xa8d, 0xa82, 0xaa1,
0xacb, 0xab0, 0xabe, 0xa86, 0xab8, 0xacd, 0xa9f, 0xacd, 0xab0, 0xa96, 0xaa8, 0xa8f,
0xaa5, 0xac7, 0xa82, 0xab8, 0xaac, 0xac7, 0xab2, 0xa97, 0xacd, 0xab0, 0xac7, 0xaa1,
0xaac, 0xab0, 0xacd, 0xab2, 0xabf, 0xaa8, 0xaac, 0xacd, 0xab0, 0xac7, 0xa9f, 0xabf,
0xab8, 0xab2, 0xabe, 0xab5, 0xabe, 0xaac, 0xacd, 0xab0, 0xab8, 0xac7, 0xab2, 0xacd,
0xab8, 0xaac, 0xac1, 0xa95, 0xab0, 0xac7, 0xab8, 0xacd, 0xa9f, 0xaac, 0xac1, 0xaa1,
0xabe, 0xaaa, 0xac7, 0xab8, 0xacd, 0xa9f, 0xaac, 0xabf, 0xa9d, 0xac7, 0xaa8, 0xacd,
0xa97, 0xaa8, 0xa9a, 0xabf, 0xab8, 0xac0, 0xaa8, 0xabe, 0xa89, 0xa95, 0xacb, 0xaaa,
0xaa8, 0xab9, 0xac7, 0xa97, 0xaa8, 0xaa1, 0xaac, 0xab2, 0xabf, 0xaa8, 0xa9c, 0xabf,
0xaac, 0xacd, 0xab0, 0xabe, 0xab2, 0xacd, 0xa9f, 0xab0, 0xa97, 0xab0, 0xacd, 0xaa8,
0xa9d, 0xac0, 0xab9, 0xac7, 0xab2, 0xab8, 0xabf, 0xa82, 0xa95, 0xac0, 0xa86, 0xa87,
0xab2, 0x20, 0xa91, 0xaab, 0x20, 0xaae, 0xac7, 0xaa8, 0xa87, 0xab8, 0xacd, 0xaa4,
0xabe, 0xa82, 0xaac, 0xac1, 0xab2, 0xa9c, 0xab0, 0xacd, 0xab8, 0xac0, 0xa95, 0xab2,
0xabf, 0xaa8, 0xabf, 0xaa8, 0xacd, 0xa97, 0xacd, 0xab0, 0xac7, 0xaa1, 0xa95, 0xac0,
0xab0, 0xacb, 0xab5, 0xa95, 0xac8, 0xab5, 0xab2, 0xabf, 0xab8, 0xacd, 0xaac, 0xaa8,
0xab2, 0xac1, 0xaac, 0xacd, 0xab2, 0xabf, 0xaaf, 0xabe, 0xaa8, 0xabe, 0xab2, 0xa82,
0xaa1, 0xaa8, 0xab2, 0xa95, 0xacd, 0xa9d, 0xaae, 0xaac, 0xab0, 0xacd, 0xa97, 0xaae,
0xac7, 0xaa1, 0xacd, 0xab0, 0xabf, 0xaa1, 0xaae, 0xabe, 0xab2, 0xacd, 0xa9f, 0xabe,
0xaae, 0xac7, 0xab0, 0xabf, 0xab9, 0xac7, 0xaae, 0xaae, 0xabf, 0xa82, 0xab8, 0xacd,
0xa95, 0xaae, 0xacb, 0xaa8, 0xabe, 0xa95, 0xacb, 0xaae, 0xacb, 0xab8, 0xacd, 0xa95,
0xacb, 0xa91, 0xab8, 0xacd, 0xab2, 0xacb, 0xaaa, 0xac7, 0xab0, 0xabf, 0xab8, 0xaaa,
0xacb, 0xaa1, 0xa97, 0xacb, 0xab0, 0xabf, 0xa95, 0xabe, 0xaaa, 0xacd, 0xab0, 0xabe,
0xa97, 0xab0, 0xabf, 0xa97, 0xabe, 0xab8, 0xaae, 0xabe, 0xab0, 0xabe, 0xab8, 0xac5,
0xaa8, 0x20, 0xaae, 0xac7, 0xab0, 0xabf, 0xaa8, 0xacb, 0xab8, 0xabe, 0xab0, 0xabe,
0xa9c, 0xac7, 0xab5, 0xacb, 0xab8, 0xabe, 0xab0, 0xabe, 0xa9f, 0xacb, 0xab5, 0xab8,
0xabf, 0xaae, 0xacd, 0xaab, 0xac7, 0xab0, 0xacb, 0xaaa, 0xacb, 0xab2, 0xab8, 0xacd,
0xa95, 0xacb, 0xaaa, 0xaaf, 0xac7, 0xab8, 0xacb, 0xaab, 0xabf, 0xaaf, 0xabe, 0xab8,
0xacd, 0xa9f, 0xacb, 0xa95, 0xab9, 0xac9, 0xaae, 0xaa4, 0xab2, 0xacd, 0xab2, 0xac0,
0xaa8, 0xaa4, 0xabf, 0xab0, 0xabe, 0xaa8, 0xac7, 0xa89, 0xab2, 0xac7, 0xaa8, 0xacb,
0xab5, 0xacd, 0xab8, 0xacd, 0xa95, 0xab5, 0xac5, 0xaa6, 0xac1, 0xa9d, 0xab5, 0xac7,
0xa9f, 0xabf, 0xa95, 0xaa8, 0xab5, 0xabf, 0xaaf, 0xac7, 0xaa8, 0xabe, 0xab5, 0xabf,
0xab2, 0xacd, 0xaa8, 0xabf, 0xa85, 0xab8, 0xab5, 0xacb, 0xab2, 0xacd, 0xa97, 0xacb,
0xa97, 0xacd, 0xab0, 0xac7, 0xaa1, 0xab5, 0xacb, 0xab0, 0xab8, 0xac9, 0xa9d, 0xac7,
0xa97, 0xab0, 0xac7, 0xaac, 0xa9d, 0xac1, 0xab0, 0xac0, 0xa95, 0xa85, 0xa82, 0xaa4,
0xabe, 0xaa8, 0xabe, 0xaa8, 0xabe, 0xab0, 0xabf, 0xab5, 0xacb, 0xa9a, 0xabe, 0xa97,
0xacb, 0xab8, 0xa95, 0xacd, 0xab0, 0xabf, 0xab8, 0xaae, 0xab8, 0xa95, 0xacb, 0xa95,
0xacb, 0xa9d, 0xa95, 0xacb, 0xaae, 0xacb, 0xab0, 0xacb, 0xa95, 0xac7, 0xab0, 0xa97,
0xacd, 0xab5, 0xac7, 0xab2, 0xabf, 0xaa8, 0xaae, 0xabe, 0xab9, 0xac7, 0xaae, 0xabe,
0xab2, 0xaa6, 0xabf, 0xab5, 0xacd, 0xab8, 0xaae, 0xacb, 0xab0, 0xabf, 0xab6, 0xabf,
0xaaf, 0xab8, 0xaae, 0xac7, 0xaaf, 0xacb, 0xa9f, 0xab0, 0xac0, 0xaaf, 0xac1, 0xaa8,
0xabf, 0xaaf, 0xaa8, 0xa8f, 0xaaa, 0xabf, 0xaaf, 0xabe, 0xa91, 0xa95, 0xab2, 0xac7,
0xaa8, 0xacd, 0xaa1, 0xaac, 0xacc, 0xa97, 0xac7, 0xa88, 0xaa8, 0xab5, 0xabf, 0xab2,
0xac7, 0xa9a, 0xaa5, 0xaae, 0xa9a, 0xac1, 0xa95, 0xa87, 0xab8, 0xacd, 0xa9f, 0xab0,
0xa8f, 0xaab, 0xac7, 0xa9f, 0xaab, 0xabe, 0xa95, 0xabe, 0xa93, 0xaab, 0xacb, 0xaab,
0xac0, 0xa9c, 0xac0, 0xaab, 0xac1, 0xaa8, 0xabe, 0xaab, 0xac1, 0xa9f, 0xac0, 0xa97,
0xabe, 0xab2, 0xabe, 0xaaa, 0xac7, 0xa97, 0xacb, 0xab8, 0xa97, 0xac7, 0xaae, 0xacd,
0xaac, 0xabf, 0xaaf, 0xab0, 0xa97, 0xacc, 0xaa1, 0xab2, 0xa95, 0xac5, 0xaa8, 0xab2,
0xa97, 0xacd, 0xab5, 0xabe, 0xaae, 0xab9, 0xacb, 0xaa8, 0xacb, 0xab2, 0xac1, 0xab2,
0xac1, 0xa8f, 0xa82, 0xaa1, 0xab0, 0xacd, 0xaac, 0xab0, 0xac0, 0xa95, 0xabf, 0xab0,
0xabf, 0xaa4, 0xabf, 0xaae, 0xabe, 0xaa4, 0xac0, 0xa95, 0xacb, 0xab8, 0xab0, 0xac7,
0xa95, 0xacd, 0xab5, 0xabe, 0xa9c, 0xabe, 0xab2, 0xac0, 0xaa8, 0xaae, 0xabe, 0xa9c,
0xac1, 0xab0, 0xacb, 0xaae, 0xabe, 0xab0, 0xacd, 0xa95, 0xac0, 0xab8, 0xabe, 0xab8,
0xaae, 0xac0, 0xaa1, 0xab5, 0xac7, 0xaa8, 0xacc, 0xab0, 0xac1, 0xaa8, 0xac0, 0xaaf,
0xac1, 0xaa8, 0xac9, 0xab0, 0xaab, 0xac9, 0xa95, 0xaa8, 0xacc, 0xaae, 0xac0, 0xa86,
0xaaa, 0xac7, 0xa97, 0xacb, 0x20, 0xaaa, 0xac7, 0xa97, 0xacb, 0xaaa, 0xab2, 0xabe,
0xa89, 0xaaa, 0xac0, 0xa9f, 0xa95, 0xac8, 0xab0, 0xacd, 0xaa8, 0xaaa, 0xacb, 0xaa8,
0xacd, 0xaaa, 0xac7, 0xaaa, 0xacb, 0xab0, 0xacd, 0xa9f, 0x20, 0xaae, 0xacb, 0xab0,
0xacd, 0xab8, 0xacd, 0xaac, 0xac0, 0xab0, 0xac7, 0xab0, 0xacb, 0xa9f, 0xacb, 0xa82,
0xa97, 0xabe, 0xab8, 0xabe, 0xa87, 0xaaa, 0xaa8, 0xaa4, 0xabe, 0xab9, 0xac0, 0xaa4,
0xac0, 0xaa4, 0xabe, 0xab0, 0xabe, 0xab5, 0xabe, 0xa9f, 0xac9, 0xa82, 0xa97, 0xabe,
0xa9f, 0xabe, 0xaaa, 0xac1, 0xab5, 0xac7, 0xa95, 0xab5, 0xabe, 0xab2, 0xabf, 0xab8,
0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x41, 0x72, 0x65, 0x77,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x43,
0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x41, 0x72, 0x65, 0x77, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65,
0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x41, 0x72, 0x65,
0x77, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x5d0, 0x5d1, 0x5d9, 0x5d2, 0x5f3, 0x5d0, 0x5df, 0x5d0, 0x5e7, 0x5e8, 0x5d4, 0x5d0,
0x5d3, 0x5d9, 0x5e1, 0x20, 0x5d0, 0x5d1, 0x5d1, 0x5d4, 0x5d0, 0x5dc, 0x5d2, 0x5f3,
0x5d9, 0x5e8, 0x5d0, 0x5e1, 0x5de, 0x5e8, 0x5d4, 0x5d1, 0x5de, 0x5d0, 0x5e7, 0x5d5,
0x5d1, 0x5e0, 0x5d2, 0x5d5, 0x5d0, 0x5d9, 0x5d1, 0x5e0, 0x5d2, 0x5f3, 0x5d5, 0x5dc,
0x5d1, 0x5d9, 0x5e1, 0x5d0, 0x5d5, 0x5d1, 0x5dc, 0x5e0, 0x5d8, 0x5d9, 0x5e8, 0x5d1,
0x5e8, 0x5d6, 0x5d5, 0x5d5, 0x5d9, 0x5dc, 0x5d1, 0x5d5, 0x5d2, 0x5f3, 0x5d5, 0x5de,
0x5d1, 0x5d5, 0x5e8, 0x5d4, 0x5e7, 0x5d4, 0x5d9, 0x5e8, 0x5e7, 0x5d6, 0x5d1, 0x5dc,
0x5e0, 0x5e7, 0x5d4, 0x5e1, 0x5d0, 0x5d5, 0x5d8, 0x5d4, 0x5e7, 0x5d5, 0x5e0, 0x5d0,
0x5e7, 0x5e8, 0x5d9, 0x5d3, 0x5e7, 0x5d0, 0x5e8, 0x5d3, 0x5d0, 0x5e8, 0x20, 0x5d0,
0x2d, 0x5e1, 0x5dc, 0x5d0, 0x5dd, 0x5d2, 0x5f3, 0x5d9, 0x5d1, 0x5d5, 0x5d8, 0x5d9,
0x5d3, 0x5d5, 0x5d0, 0x5dc, 0x5d4, 0x5d0, 0x5dc, 0x20, 0x5e2, 0x5d9, 0x5d5, 0x5df,
0x5e4, 0x5e8, 0x5d9, 0x5d8, 0x5d0, 0x5d5, 0x5df, 0x5d2, 0x5d1, 0x5d5, 0x5e8, 0x5d5,
0x5e0, 0x5d4, 0x5d4, 0x5e8, 0x5d0, 0x5e8, 0x5d4, 0x5d9, 0x5d5, 0x5d4, 0x5e0, 0x5e1,
0x5d1, 0x5d5, 0x5e8, 0x5d2, 0x5d2, 0x5f3, 0x5d5, 0x5d1, 0x5d4, 0x5e7, 0x5de, 0x5e4,
0x5d0, 0x5dc, 0x5d4, 0x5d7, 0x5e8, 0x5d8, 0x5d5, 0x5dd, 0x5e7, 0x5d9, 0x5d2, 0x5dc,
0x5d9, 0x5e7, 0x5d9, 0x5e0, 0x5e9, 0x5e1, 0x5d4, 0x5dc, 0x5d0, 0x5d2, 0x5d5, 0x5e1,
0x5dc, 0x5d9, 0x5d1, 0x5e8, 0x5d5, 0x5d5, 0x5d9, 0x5dc, 0x5dc, 0x5d5, 0x5de, 0x5d4,
0x5dc, 0x5d5, 0x5d0, 0x5e0, 0x5d3, 0x5d4, 0x5dc, 0x5d5, 0x5d1, 0x5d5, 0x5de, 0x5d1,
0x5d0, 0x5e9, 0x5d9, 0x5dc, 0x5d5, 0x5e1, 0x5e7, 0x5d4, 0x5de, 0x5dc, 0x5d1, 0x5d5,
0x5de, 0x5d0, 0x5e4, 0x5d5, 0x5d8, 0x5d5, 0x5de, 0x5e1, 0x5e8, 0x5d5, 0x5de, 0x5d1,
0x5d1, 0x5e0, 0x5d4, 0x5de, 0x5d5, 0x5d2, 0x5d3, 0x5d9, 0x5e9, 0x5d5, 0x5de, 0x5d5,
0x5e0, 0x5e8, 0x5d5, 0x5d1, 0x5d9, 0x5d4, 0x5e0, 0x5d9, 0x5d9, 0x5e8, 0x5d5, 0x5d1,
0x5d9, 0x5e0, 0x5d2, 0x5f3, 0x5de, 0x5e0, 0x5d4, 0x5e0, 0x5d9, 0x5d0, 0x5de, 0x5d9,
0x5d9, 0x5e0, 0x5d5, 0x5d0, 0x5e7, 0x5e6, 0x5f3, 0x5d5, 0x5d8, 0x5d5, 0x5d5, 0x5d0,
0x5d2, 0x5d0, 0x5d3, 0x5d5, 0x5d2, 0x5d5, 0x5e4, 0x5d5, 0x5e8, 0x5d8, 0x5d5, 0x20,
0x5e0, 0x5d5, 0x5d1, 0x5d5, 0x5e1, 0x5d0, 0x5d5, 0x20, 0x5d8, 0x5d5, 0x5de, 0x5d4,
0x5d8, 0x5e8, 0x5d9, 0x5e4, 0x5d5, 0x5dc, 0x5d9, 0x5ea, 0x5d5, 0x5e0, 0x5d9, 0x5e1,
0x5d5, 0x5d9, 0x5e0, 0x5d3, 0x5d4, 0x5d5, 0x5e7, 0x5d0, 0x5d3, 0x5d0, 0x5e7, 0x5d0,
0x5e0, 0x5e7, 0x5d5, 0x5e8, 0x5d2, 0x5f3, 0x5d0, 0x5e0, 0x5d2, 0x5d5, 0x5d5, 0x5d9,
0x5dc, 0x5d4, 0x5d0, 0x5e0, 0x5d8, 0x5d9, 0x5d2, 0x5d5, 0x5d0, 0x5d4, 0x5d0, 0x5e8,
0x5d0, 0x5d2, 0x5d5, 0x5d0, 0x5d9, 0x5e0, 0x5d4, 0x5d1, 0x5d5, 0x5d0, 0x5e0, 0x5d5,
0x5e1, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x5e8, 0x5e1, 0x5e7, 0x5d8, 0x5de, 0x5e8, 0x5e7,
0x5d4, 0x5e7, 0x5d5, 0x5e8, 0x5d3, 0x5d5, 0x5d1, 0x5d4, 0x5d7, 0x5d5, 0x5d7, 0x5d5,
0x5d9, 0x5dc, 0x5d4, 0x20, 0x5e8, 0x5d9, 0x5d5, 0x5d7, 0x5d4, 0x5de, 0x5e0, 0x5d3,
0x5d5, 0x5d6, 0x5d4, 0x5e8, 0x5d9, 0x5d5, 0x20, 0x5d2, 0x5d0, 0x5d9, 0x5d9, 0x5d2,
0x5d5, 0x5e1, 0x5e1, 0x5dc, 0x5d8, 0x5d4, 0x5e1, 0x5df, 0x20, 0x5d7, 0x5d5, 0x5d0,
0x5df, 0x5e1, 0x5df, 0x20, 0x5dc, 0x5d5, 0x5d0, 0x5d9, 0x5e1, 0x5d8, 0x5d5, 0x5e7,
0x5d5, 0x5de, 0x5df, 0x5d0, 0x5d5, 0x5e9, 0x5d5, 0x5d5, 0x5d0, 0x5d9, 0x5d4, 0x5d0,
0x5e8, 0x5d5, 0x5d1, 0x5d4, 0x5d0, 0x5e1, 0x5d5, 0x5e0, 0x5e1, 0x5d9, 0x5d5, 0x5df,
0x5d0, 0x5d8, 0x5d9, 0x5e7, 0x5d5, 0x5e7, 0x5df, 0x5d1, 0x5d0, 0x5d4, 0x5d9, 0x5d4,
0x5d1, 0x5d0, 0x5d4, 0x5d9, 0x5d4, 0x20, 0x5d1, 0x5e0, 0x5d3, 0x5e8, 0x5e1, 0x5d1,
0x5e8, 0x5d1, 0x5d3, 0x5d5, 0x5e1, 0x5d1, 0x5dc, 0x5dd, 0x5d1, 0x5dc, 0x5d9, 0x5d6,
0x5d1, 0x5dc, 0x5d0, 0x5df, 0x2d, 0x5e1, 0x5d1, 0x5dc, 0x5d5, 0x5df, 0x5d1, 0x5d5,
0x5d0, 0x5d4, 0x20, 0x5d5, 0x5d9, 0x5e1, 0x5d8, 0x5d4, 0x5d1, 0x5d5, 0x5d2, 0x5d5,
0x5d8, 0x5d4, 0x5d1, 0x5d5, 0x5d9, 0x5e1, 0x5d9, 0x5e7, 0x5d9, 0x5d9, 0x5de, 0x5d1,
0x5e8, 0x5d9, 0x5d3, 0x5d2, 0x5f3, 0x20, 0x5d1, 0x5d9, 0x5d9, 0x5e7, 0x5de, 0x5e4,
0x5d5, 0x20, 0x5d2, 0x5e8, 0x5e0, 0x5d3, 0x5d4, 0x5e7, 0x5e0, 0x5e7, 0x5d5, 0x5df,
0x5e7, 0x5e8, 0x5d0, 0x5e7, 0x5e1, 0x5e7, 0x5d0, 0x5d9, 0x5d9, 0x5df, 0x5e7, 0x5d9,
0x5d9, 0x5de, 0x5df, 0x5e9, 0x5d9, 0x5e7, 0x5d2, 0x5d5, 0x5e6, 0x5f3, 0x5d9, 0x5d5,
0x5d5, 0x5d0, 0x5d5, 0x5d5, 0x5d4, 0x5e1, 0x5d9, 0x5d5, 0x5d3, 0x5d3, 0x20, 0x5d7,
0x5d5, 0x5d0, 0x5e8, 0x5e1, 0x5e7, 0x5d5, 0x5e1, 0x5d8, 0x5d4, 0x20, 0x5e8, 0x5d9,
0x5e7, 0x5d4, 0x5e7, 0x5e8, 0x5e1, 0x5d8, 0x5d5, 0x5df, 0x5e7, 0x5d5, 0x5d9, 0x5d0,
0x5d1, 0x5d4, 0x5e7, 0x5d5, 0x5e8, 0x5d0, 0x5e1, 0x5d0, 0x5d5, 0x5d3, 0x5e0, 0x5de,
0x5e8, 0x5e7, 0x5e1, 0x5d4, 0x5d5, 0x5d5, 0x5df, 0x5d3, 0x5d5, 0x5e1, 0x5d5, 0x5df,
0x5d3, 0x5d5, 0x5e1, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5e8, 0x5d9, 0x5e7, 0x5d3, 0x5e0,
0x5d5, 0x5d5, 0x5e8, 0x5d3, 0x5d8, 0x5e8, 0x5d5, 0x5d9, 0x5d8, 0x5d3, 0x5d5, 0x5de,
0x5d9, 0x5e0, 0x5d9, 0x5e7, 0x5d4, 0x5d0, 0x5d3, 0x5de, 0x5d5, 0x5e0, 0x5d8, 0x5d5,
0x5df, 0x5d0, 0x5d9, 0x5e8, 0x5d5, 0x5e0, 0x5e4, 0x5d9, 0x5d0, 0x5dc, 0x20, 0x5e1,
0x5dc, 0x5d1, 0x5d3, 0x5d5, 0x5e8, 0x5e4, 0x5d5, 0x5e8, 0x5d8, 0x20, 0x5e0, 0x5dc,
0x5e1, 0x5d5, 0x5df, 0x5e4, 0x5d5, 0x5e8, 0x5d8, 0x5d0, 0x5dc, 0x5d6, 0x5d4, 0x5d2,
0x5dc, 0x5d9, 0x5d9, 0x5e1, 0x20, 0x5d1, 0x5d9, 0x5d9, 0x5d2, 0x5d5, 0x5e1, 0x20,
0x5d1, 0x5d9, 0x5d9, 0x5d2, 0x5e8, 0x5e0, 0x5d3, 0x20, 0x5d8, 0x5d5, 0x5e8, 0x5e7,
0x5d2, 0x5d5, 0x5d0, 0x5d3, 0x5dc, 0x5d5, 0x5e4, 0x5d2, 0x5d5, 0x5d0, 0x5d8, 0x5de,
0x5dc, 0x5d4, 0x5d2, 0x5d5, 0x5d0, 0x5d9, 0x5d0, 0x5e7, 0x5d9, 0x5dc, 0x5d2, 0x5d9,
0x5d0, 0x5e0, 0x5d4, 0x5d4, 0x5dc, 0x5d9, 0x5e4, 0x5e7, 0x5e1, 0x5d4, 0x5d5, 0x5d5,
0x5d0, 0x5e0, 0x5d4, 0x5d4, 0x5e8, 0x5de, 0x5d5, 0x5e1, 0x5d9, 0x5d5, 0x5d0, 0x5d9,
0x5e0, 0x5d3, 0x5d9, 0x5d0, 0x5e0, 0x5e4, 0x5d5, 0x5dc, 0x5d9, 0x5e1, 0x5e0, 0x5d5,
0x5e7, 0x5e1, 0x2c, 0x20, 0x5d0, 0x5d9, 0x5e0, 0x5d3, 0x5d9, 0x5d0, 0x5e0, 0x5d4,
0x5de, 0x5e8, 0x5e0, 0x5d2, 0x5d5, 0x2c, 0x20, 0x5d0, 0x5d9, 0x5e0, 0x5d3, 0x5d9,
0x5d0, 0x5e0, 0x5d4, 0x5e4, 0x5d9, 0x5d8, 0x5e8, 0x5e1, 0x5d1, 0x5d5, 0x5e8, 0x5d2,
0x2c, 0x20, 0x5d0, 0x5d9, 0x5e0, 0x5d3, 0x5d9, 0x5d0, 0x5e0, 0x5d4, 0x5d8, 0x5dc,
0x20, 0x5e1, 0x5d9, 0x5d8, 0x5d9, 0x2c, 0x20, 0x5d0, 0x5d9, 0x5e0, 0x5d3, 0x5d9,
0x5d0, 0x5e0, 0x5d4, 0x5d5, 0x5d9, 0x5d5, 0x5d5, 0x5d0, 0x5d9, 0x5d9, 0x2c, 0x20,
0x5d0, 0x5d9, 0x5e0, 0x5d3, 0x5d9, 0x5d0, 0x5e0, 0x5d4, 0x5d5, 0x5d9, 0x5e0, 0x5e1,
0x5e0, 0x5e1, 0x2c, 0x20, 0x5d0, 0x5d9, 0x5e0, 0x5d3, 0x5d9, 0x5d0, 0x5e0, 0x5d4,
0x5d5, 0x5d9, 0x5e0, 0x5de, 0x5d0, 0x5e7, 0x2c, 0x20, 0x5d0, 0x5d9, 0x5e0, 0x5d3,
0x5d9, 0x5d0, 0x5e0, 0x5d4, 0x5d0, 0x5d9, 0x5e0, 0x5d5, 0x5d5, 0x5d9, 0x5e7, 0x5d0,
0x5d9, 0x5e7, 0x5dc, 0x5d5, 0x5d0, 0x5d9, 0x5d8, 0x5d2, 0x5f3, 0x5de, 0x5d9, 0x5d9,
0x5e7, 0x5d4, 0x5d2, 0x5f3, 0x5d5, 0x5e0, 0x5d5, 0x5dc, 0x5d5, 0x5d0, 0x5d9, 0x5d5,
0x5d5, 0x5d9, 0x5dc, 0x5de, 0x5d5, 0x5e0, 0x5d8, 0x5d9, 0x5e6, 0x5f3, 0x5dc, 0x5d5,
0x2c, 0x20, 0x5e7, 0x5e0, 0x5d8, 0x5d0, 0x5e7, 0x5d9, 0x5e7, 0x5e8, 0x5dc, 0x5e0,
0x5d3, 0x5d9, 0x5d9, 0x5e7, 0x5dc, 0x5d4, 0x20, 0x5e4, 0x5d0, 0x5e1, 0x5dc, 0x5d9,
0x5de, 0x5d4, 0x5dc, 0x5d5, 0x5e1, 0x20, 0x5d0, 0x5e0, 0x5d2, 0x5f3, 0x5dc, 0x5e1,
0x5dc, 0x5d5, 0x5d0, 0x5d5, 0x5d5, 0x5e8, 0x20, 0x5e4, 0x5e8, 0x5d9, 0x5e0, 0x5e1,
0x5e1, 0x20, 0x5e7, 0x5d5, 0x5d5, 0x5e8, 0x5d8, 0x5e8, 0x5de, 0x5e1, 0x5d9, 0x5d9,
0x5d0, 0x5d5, 0x5de, 0x5e0, 0x5d2, 0x5d5, 0x5d0, 0x5d4, 0x5de, 0x5e0, 0x5d0, 0x5d5,
0x5e1, 0x5de, 0x5e8, 0x5d9, 0x5d2, 0x5d5, 0x5de, 0x5e8, 0x5d8, 0x5d9, 0x5e0, 0x5d9,
0x5e7, 0x5de, 0x5d8, 0x5de, 0x5d5, 0x5e8, 0x5d5, 0x5e1, 0x5de, 0x5d6, 0x5d8, 0x5dc,
0x5df, 0x5de, 0x5e0, 0x5d5, 0x5de, 0x5d9, 0x5e0, 0x5d9, 0x5de, 0x5e8, 0x5d9, 0x5d3,
0x5d4, 0x5de, 0x5d8, 0x5dc, 0x5e7, 0x5d8, 0x5dc, 0x5d4, 0x5de, 0x5e7, 0x5e1, 0x5d9,
0x5e7, 0x5d5, 0x20, 0x5e1, 0x5d9, 0x5d8, 0x5d9, 0x5de, 0x5d9, 0x5e7, 0x5dc, 0x5d5,
0x5df, 0x5de, 0x5d5, 0x5e0, 0x5e7, 0x5d8, 0x5d5, 0x5df, 0x5de, 0x5d5, 0x5e0, 0x5d8,
0x5e8, 0x5d9, 0x5d9, 0x5de, 0x5d5, 0x5e0, 0x5d8, 0x5d5, 0x5d5, 0x5d9, 0x5d3, 0x5d0,
0x5d5, 0x5de, 0x5d5, 0x5e0, 0x5e1, 0x5e8, 0x5d0, 0x5d8, 0x5e0, 0x5e1, 0x5d0, 0x5d5,
0x5e0, 0x5d9, 0x5d5, 0x20, 0x5d9, 0x5d5, 0x5e8, 0x5e7, 0x5e0, 0x5d5, 0x5dd, 0x5e0,
0x5d5, 0x5e8, 0x5d5, 0x5e0, 0x5d9, 0x5d4, 0x5d1, 0x5d9, 0x5d5, 0x5dc, 0x5d4, 0x2c,
0x20, 0x5d3, 0x5e7, 0x5d5, 0x5d8, 0x5d4, 0x20, 0x5d4, 0x5e6, 0x5e4, 0x5d5, 0x5e0,
0x5d9, 0x5ea, 0x5e1, 0x5e0, 0x5d8, 0x5e8, 0x2c, 0x20, 0x5d3, 0x5e7, 0x5d5, 0x5d8,
0x5d4, 0x20, 0x5d4, 0x5e6, 0x5e4, 0x5d5, 0x5e0, 0x5d9, 0x5ea, 0x5e0, 0x5d9, 0x5d5,
0x20, 0x5e1, 0x5d9, 0x5d9, 0x5dc, 0x5dd, 0x2c, 0x20, 0x5d3, 0x5e7, 0x5d5, 0x5d8,
0x5d4, 0x20, 0x5d4, 0x5e6, 0x5e4, 0x5d5, 0x5e0, 0x5d9, 0x5ea, 0x5e0, 0x5d5, 0x5d0,
0x5d5, 0x5e7, 0x5d0, 0x5d5, 0x5d2, 0x5f3, 0x5d9, 0x5e0, 0x5d0, 0x5d2, 0x5d4, 0x5e4,
0x5e0, 0x5de, 0x5d4, 0x5e4, 0x5e8, 0x5de, 0x5e8, 0x5d9, 0x5d1, 0x5d5, 0x5e4, 0x5d9,
0x5e0, 0x5d9, 0x5e7, 0x5e1, 0x5e4, 0x5d5, 0x5e8, 0x5d8, 0x20, 0x5d0, 0x5d5, 0x20,
0x5e4, 0x5e8, 0x5d0, 0x5e0, 0x5e1, 0x5e4, 0x5d5, 0x5e8, 0x5d8, 0x20, 0x5d0, 0x5d5,
0x5e3, 0x20, 0x5e1, 0x5e4, 0x5d9, 0x5d9, 0x5df, 0x5e4, 0x5d5, 0x5e8, 0x5d8, 0x5d5,
0x20, 0x5d5, 0x5dc, 0x5d9, 0x5d5, 0x5e4, 0x5d5, 0x5d0, 0x5e8, 0x5d8, 0x5d5, 0x20,
0x5e8, 0x5d9, 0x5e7, 0x5d5, 0x5e4, 0x5d5, 0x5e0, 0x5d8, 0x5d4, 0x20, 0x5d0, 0x5e8,
0x5e0, 0x5e1, 0x5e8, 0x5e0, 0x5e7, 0x5d9, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5e0, 0x5dc,
0x5d8, 0x5e8, 0x5e1, 0x5d9, 0x5e4, 0x5d4, 0x5e8, 0x5d2, 0x5f3, 0x5d9, 0x5e0, 0x5d4,
0x5e8, 0x5d6, 0x5d5, 0x5dc, 0x5d5, 0x5d8, 0x5e8, 0x5d9, 0x5d5, 0x20, 0x5d1, 0x5e8,
0x5e0, 0x5e7, 0x5d5, 0x5e1, 0x5e0, 0x5d8, 0x5e8, 0x5dd, 0x5e1, 0x5e0, 0x5d8, 0x5d9,
0x5d0, 0x5d2, 0x5d5, 0x5e1, 0x5e0, 0x5d8, 0x5d5, 0x20, 0x5d3, 0x5d5, 0x5de, 0x5d9,
0x5e0, 0x5d2, 0x5d5, 0x5e1, 0x5d0, 0x5d5, 0x20, 0x5e4, 0x5d0, 0x5d5, 0x5dc, 0x5d5,
0x5e1, 0x5e7, 0x5d5, 0x5e8, 0x5e1, 0x5d1, 0x5d9, 0x5e1, 0x5d5, 0x5e0, 0x5d3, 0x5e1,
0x5d9, 0x5d8, 0x5e7, 0x5d4, 0x5e1, 0x5e0, 0x5d8, 0x20, 0x5d1, 0x5e8, 0x5ea, 0x5dc,
0x5de, 0x5d9, 0x5e1, 0x5e0, 0x5d8, 0x20, 0x5d2, 0x5f3, 0x5d5, 0x5e0, 0x5e1, 0x5e1,
0x5e0, 0x5d8, 0x20, 0x5e7, 0x5d9, 0x5d8, 0x5e1, 0x5e1, 0x5e0, 0x5d8, 0x20, 0x5dc,
0x5d5, 0x5e1, 0x5d9, 0x5d4, 0x5e1, 0x5e0, 0x5d8, 0x20, 0x5ea, 0x5d5, 0x5de, 0x5d0,
0x5e1, 0x5e1, 0x5e0, 0x5d8, 0x20, 0x5d5, 0x5d9, 0x5e0, 0x5e1, 0x5e0, 0x5d8, 0x5e1,
0x5d5, 0x5d5, 0x5d9, 0x5e4, 0x5d8, 0x20, 0x5e7, 0x5e8, 0x5e0, 0x5d8, 0x5d8, 0x5d2,
0x5d5, 0x5e1, 0x5d9, 0x5d2, 0x5dc, 0x5e4, 0x5d4, 0x5ea, 0x5d5, 0x5dc, 0x5d4, 0x5d8,
0x5d9, 0x5d7, 0x5d5, 0x5d0, 0x5e0, 0x5d4, 0x5d8, 0x5d5, 0x5e8, 0x5d5, 0x5e0, 0x5d8,
0x5d5, 0x5d8, 0x5d5, 0x5e8, 0x5d8, 0x5d5, 0x5dc, 0x5d4, 0x5d5, 0x5e0, 0x5e7, 0x5d5,
0x5d1, 0x5e8, 0x5d5, 0x5d5, 0x5d9, 0x5d9, 0x5d8, 0x5d4, 0x5d5, 0x5e8, 0x5e1, 0x5d5,
0x5d5, 0x5d9, 0x5e0, 0x5d9, 0x5e4, 0x5d2, 0x5d9, 0x5e7, 0x5d5, 0x5d8, 0x5d0, 0x5d8,
0x5e7, 0x5d9, 0x5d9, 0x5e1, 0x5d9, 0x5d3, 0x5d9, 0x5d5, 0x5d5, 0x5d9, 0x5e1, 0x5d3,
0x5d5, 0x5de, 0x5d5, 0x5df, 0x20, 0x5d3, 0x5f3, 0x5d0, 0x5d5, 0x5e8, 0x5d5, 0x5d5,
0x5d9, 0x5dc, 0x5de, 0x5e7, 0x5d5, 0x5d5, 0x5d0, 0x5e8, 0x5d9, 0x5de, 0x5d5, 0x5e1,
0x5d5, 0x5df, 0x5de, 0x5e7, 0x2d, 0x5de, 0x5e8, 0x5d3, 0x5d5, 0x5e4, 0x5d0, 0x5dc,
0x5de, 0x5e8, 0x5e8, 0x5d5, 0x5ea, 0x5f3, 0x5e8, 0x5d4, 0x5e1, 0x5d9, 0x5d9, 0x5d5,
0x5d5, 0x5d4, 0x5d8, 0x5e8, 0x5d5, 0x5dc, 0x5d5, 0x5d5, 0x5e1, 0x5d8, 0x5d5, 0x5e7,
0x5dc, 0x5d5, 0x5e0, 0x5d2, 0x5d9, 0x5d9, 0x5e8, 0x5d1, 0x5d9, 0x5df, 0x5e2, 0x5d3,
0x5df, 0x5d0, 0x5dc, 0x5de, 0x5d0, 0x5d8, 0x5d9, 0x5e2, 0x5de, 0x5d0, 0x5df, 0x5d0,
0x5e0, 0x5d3, 0x5d9, 0x5e8, 0x5d0, 0x5e7, 0x5d8, 0x5d0, 0x5d5, 0x5d0, 0x5e7, 0x5d8,
0x5d5, 0x5d1, 0x5d4, 0x5d0, 0x5e9, 0x5d2, 0x5d1, 0x5d8, 0x5d0, 0x5d8, 0x5d9, 0x5e8,
0x5d0, 0x5d5, 0x5d1, 0x5d2, 0x5d3, 0x5d3, 0x5d1, 0x5d7, 0x5e8, 0x5d9, 0x5d9, 0x5df,
0x5d1, 0x5d0, 0x5e7, 0x5d5, 0x5d1, 0x5e0, 0x5d2, 0x5e7, 0x5d5, 0x5e7, 0x5d1, 0x5e8,
0x5e0, 0x5d0, 0x5d5, 0x5dc, 0x5d1, 0x5d9, 0x5d9, 0x5e8, 0x5d5, 0x5ea, 0x5d1, 0x5d9,
0x5e9, 0x5e7, 0x5e7, 0x5d1, 0x5e8, 0x5d5, 0x5e0, 0x5d9, 0x5d9, 0x5e6, 0x5f3, 0x5d9,
0x5d8, 0x5d4, 0x5e7, 0x5d5, 0x5dc, 0x5d5, 0x5de, 0x5d1, 0x5d5, 0x5d3, 0x5de, 0x5e9,
0x5e7, 0x5d3, 0x5d0, 0x5e7, 0x5d4, 0x5d3, 0x5d9, 0x5dc, 0x5d9, 0x5d3, 0x5d5, 0x5d1,
0x5d0, 0x5d9, 0x5d3, 0x5d5, 0x5e9, 0x5e0, 0x5d1, 0x5d4, 0x5e4, 0x5de, 0x5d2, 0x5d5,
0x5e1, 0x5d8, 0x5d4, 0x5e2, 0x5d6, 0x5d4, 0x5d7, 0x5d1, 0x5e8, 0x5d5, 0x5df, 0x5d4,
0x5d5, 0x20, 0x5e6, 0x5f3, 0x5d9, 0x20, 0x5de, 0x5d9, 0x5df, 0x20, 0x5e1, 0x5d9,
0x5d8, 0x5d9, 0x5d4, 0x5d5, 0x5e0, 0x5d2, 0x20, 0x5e7, 0x5d5, 0x5e0, 0x5d2, 0x5d7,
0x5d5, 0x5d1, 0x5d3, 0x5d0, 0x5d9, 0x5e8, 0x5e7, 0x5d5, 0x5d8, 0x5e1, 0x5e7, 0x5d2,
0x5f3, 0x5e7, 0x5e8, 0x5d8, 0x5d4, 0x5d2, 0x5f3, 0x5d0, 0x5d9, 0x5d0, 0x5e4, 0x5d5,
0x5e8, 0x5d4, 0x5d9, 0x5e8, 0x5d5, 0x5e9, 0x5dc, 0x5d9, 0x5dd, 0x5e7, 0x5d0, 0x5d1,
0x5d5, 0x5dc, 0x5e7, 0x5de, 0x5e6, 0x5f3, 0x5d8, 0x5e7, 0x5d4, 0x5e7, 0x5e8, 0x5d0,
0x5e6, 0x5f3, 0x5d9, 0x5e7, 0x5d8, 0x5de, 0x5e0, 0x5d3, 0x5d5, 0x5d7, 0x5e0, 0x5d3,
0x5d9, 0x5d2, 0x5d4, 0x5e7, 0x5d5, 0x5dc, 0x5e7, 0x5d8, 0x5d4, 0x5e7, 0x5e8, 0x5e1,
0x5e0, 0x5d5, 0x5d9, 0x5d0, 0x5e8, 0x5e1, 0x5e7, 0x5e7, 0x5d5, 0x5d0, 0x5dc, 0x5d4,
0x20, 0x5dc, 0x5d5, 0x5de, 0x5e4, 0x5d5, 0x5e8, 0x5e7, 0x5d5, 0x5e6, 0x5f3, 0x5d9,
0x5e0, 0x5d2, 0x5db, 0x5d5, 0x5d5, 0x5d9, 0x5ea, 0x5de, 0x5e7, 0x5d0, 0x5d5, 0x5de,
0x5d2, 0x5d3, 0x5df, 0x5de, 0x5d0, 0x5e7, 0x5d0, 0x5e1, 0x5d0, 0x5e8, 0x5de, 0x5e0,
0x5d9, 0x5dc, 0x5d4, 0x5de, 0x5d5, 0x5e1, 0x5e7, 0x5d8, 0x5e0, 0x5d9, 0x5e7, 0x5d5,
0x5e1, 0x5d9, 0x5d4, 0x5e0, 0x5d5, 0x5d1, 0x5d5, 0x5e7, 0x5d5, 0x5d6, 0x5e0, 0x5d8,
0x5e1, 0x5e7, 0x5e0, 0x5d5, 0x5d1, 0x5d5, 0x5e1, 0x5d9, 0x5d1, 0x5d9, 0x5e8, 0x5e1,
0x5e7, 0x5d0, 0x5d5, 0x5de, 0x5e1, 0x5e7, 0x5d0, 0x5d5, 0x5e8, 0x5dc, 0x5e4, 0x5e0,
0x5d5, 0x5dd, 0x20, 0x5e4, 0x5df, 0x5e4, 0x5d5, 0x5e0, 0x5d8, 0x5d9, 0x5d0, 0x5e0,
0x5e7, 0x5e4, 0x5d9, 0x5d5, 0x5e0, 0x5d2, 0x5d9, 0x5d0, 0x5e0, 0x5d2, 0x5e7, 0x5d8,
0x5d0, 0x5e8, 0x5e7, 0x5d5, 0x5e1, 0x5d8, 0x5e0, 0x5d0, 0x5d9, 0x5e7, 0x5d9, 0x5d6,
0x5d9, 0x5dc, 0x5d5, 0x5e8, 0x5d3, 0x5d4, 0x5e8, 0x5d9, 0x5d0, 0x5d3, 0x5e1, 0x5d7,
0x5dc, 0x5d9, 0x5df, 0x5e1, 0x5de, 0x5e8, 0x5e7, 0x5e0, 0x5d3, 0x5e1, 0x5d9, 0x5d0,
0x5d5, 0x5dc, 0x5e9, 0x5e0, 0x5d7, 0x5d0, 0x5d9, 0x5e1, 0x5d9, 0x5e0, 0x5d2, 0x5e4,
0x5d5, 0x5e8, 0x5e1, 0x5e8, 0x5d3, 0x5e0, 0x5d9, 0x5d9, 0x5e7, 0x5d5, 0x5dc, 0x5d9,
0x5de, 0x5e1, 0x5e7, 0x5d8, 0x5d0, 0x5d9, 0x5e4, 0x5d9, 0x5d9, 0x5d8, 0x5e9, 0x5e7,
0x5e0, 0x5d8, 0x5d8, 0x5d1, 0x5d9, 0x5dc, 0x5d9, 0x5e1, 0x5d9, 0x5d8, 0x5d4, 0x5e8,
0x5df, 0x5d8, 0x5d4, 0x5d9, 0x5de, 0x5e4, 0x5d4, 0x5d5, 0x5d8, 0x5d5, 0x5e7, 0x5d9,
0x5d5, 0x5d8, 0x5d5, 0x5de, 0x5e1, 0x5e7, 0x5d0, 0x5d5, 0x5dc, 0x5d0, 0x5df, 0x20,
0x5d1, 0x5d0, 0x5d8, 0x5d5, 0x5e8, 0x5d0, 0x5d5, 0x5e8, 0x5d5, 0x5de, 0x5e7, 0x5d9,
0x5d0, 0x5d5, 0x5e1, 0x5d8, 0x2d, 0x5e0, 0x5e8, 0x5d4, 0x5d5, 0x5d9, 0x5d9, 0x5e0,
0x5d8, 0x5d9, 0x5d0, 0x5df, 0x5d5, 0x5dc, 0x5d3, 0x5d9, 0x5d5, 0x5d5, 0x5e1, 0x5d8,
0x5d5, 0x5e7, 0x5d9, 0x5e7, 0x5d5, 0x5d8, 0x5e1, 0x5e7, 0x5e8, 0x5e0, 0x5d2, 0x5d5,
0x5df, 0x5d9, 0x5e7, 0x5d8, 0x5e8, 0x5d9, 0x5e0, 0x5d1, 0x5d5, 0x5e8, 0x5d2, 0x5d9,
0x5e8, 0x5d5, 0x5d5, 0x5d0, 0x5df, 0x5d4, 0x5d0, 0x5d9, 0x5d9, 0x5dd, 0x20, 0x5d4,
0x5d0, 0x5d6, 0x5d5, 0x5e8, 0x5d9, 0x5d9, 0x5dd, 0x5d1, 0x5e8, 0x5de, 0x5d5, 0x5d3,
0x5d4, 0x5d4, 0x5d0, 0x5d9, 0x5d9, 0x5dd, 0x20, 0x5d4, 0x5e7, 0x5e0, 0x5e8, 0x5d9,
0x5d9, 0x5dd, 0x5db, 0x5e3, 0x20, 0x5d5, 0x5e8, 0x5d3, 0x5d4, 0x5e4, 0x5d0, 0x5e8,
0x5d5, 0x5de, 0x5d3, 0x5d9, 0x5d9, 0x5e8, 0x5d4, 0x5e8, 0x5d9, 0x5d9, 0x5e7, 0x5d9,
0x5d0, 0x5d5, 0x5d5, 0x5d9, 0x5e7, 0x5d3, 0x5e8, 0x5d5, 0x5dd, 0x20, 0x5d2, 0x5f3,
0x5d5, 0x5e8, 0x5d2, 0x5f3, 0x5d9, 0x5d4, 0x5e1, 0x5e0, 0x5d8, 0x20, 0x5d4, 0x5dc,
0x5e0, 0x5d4, 0x5e1, 0x5d8, 0x5e0, 0x5dc, 0x5d9, 0x5d0, 0x5d3, 0x5dc, 0x5d9, 0x5d9,
0x5d3, 0x5d1, 0x5e8, 0x5d9, 0x5e1, 0x5d1, 0x5d9, 0x5d9, 0x5df, 0x5d1, 0x5e8, 0x5d5,
0x5e7, 0x5df, 0x20, 0x5d4, 0x5d9, 0x5dc, 0x5d3, 0x5e8, 0x5d5, 0x5d5, 0x5d9, 0x5df,
0x5d9, 0x5d5, 0x5e7, 0x5dc, 0x5d4, 0x5d4, 0x5d5, 0x5d1, 0x5e8, 0x5d8, 0x5dc, 0x5d9,
0x5e0, 0x5d3, 0x5de, 0x5df, 0x5d0, 0x5d9, 0x20, 0x5d4, 0x5dc, 0x5d5, 0x5e8, 0x5d3,
0x20, 0x5d4, 0x5d0, 0x5d5, 0x5de, 0x5dc, 0x5d1, 0x5d5, 0x5e8, 0x5df, 0x5e4, 0x5e8,
0x5ea, 0x5f3, 0x5e1, 0x5d9, 0x5d3, 0x5e0, 0x5d9, 0x5d0, 0x5de, 0x5e1, 0x5d8, 0x5e8,
0x5d3, 0x5dd, 0x5d0, 0x5e0, 0x5d3, 0x5d5, 0x5e8, 0x5d4, 0x5d0, 0x5e1, 0x5d8, 0x5e8,
0x5d7, 0x5df, 0x5d0, 0x5ea, 0x5d5, 0x5e0, 0x5d4, 0x5d1, 0x5dc, 0x5d2, 0x5e8, 0x5d3,
0x5d1, 0x5e8, 0x5dc, 0x5d9, 0x5df, 0x5d1, 0x5e8, 0x5d8, 0x5d9, 0x5e1, 0x5dc, 0x5d1,
0x5d4, 0x5d1, 0x5e8, 0x5d9, 0x5e1, 0x5dc, 0x5d1, 0x5d5, 0x5e7, 0x5e8, 0x5e9, 0x5d8,
0x5d1, 0x5d5, 0x5d3, 0x5e4, 0x5e9, 0x5d8, 0x5d1, 0x5d9, 0x5d6, 0x5d9, 0x5e0, 0x5d2,
0x5df, 0x5e7, 0x5d9, 0x5e9, 0x5d9, 0x5e0, 0x5d1, 0x5e7, 0x5d5, 0x5e4, 0x5e0, 0x5d4,
0x5d2, 0x5df, 0x5d3, 0x5d1, 0x5dc, 0x5d9, 0x5df, 0x5d2, 0x5d9, 0x5d1, 0x5e8, 0x5dc,
0x5d8, 0x5e8, 0x5d2, 0x5e8, 0x5e0, 0x5d6, 0x5d9, 0x5d4, 0x5dc, 0x5e1, 0x5d9, 0x5e0,
0x5e7, 0x5d9, 0x5d4, 0x5d0, 0x5d9, 0x20, 0x5de, 0x5d0, 0x5df, 0x5d0, 0x5d9, 0x5e1,
0x5d8, 0x5e0, 0x5d1, 0x5d5, 0x5dc, 0x5d2, 0x5f3, 0x5e8, 0x5d6, 0x5d9, 0x5e7, 0x5dc,
0x5d9, 0x5e0, 0x5d9, 0x5e0, 0x5d2, 0x5e8, 0x5d3, 0x5e7, 0x5d9, 0x5e8, 0x5d5, 0x5d1,
0x5e7, 0x5d9, 0x5d9, 0x5d1, 0x5dc, 0x5d9, 0x5e1, 0x5d1, 0x5d5, 0x5df, 0x5dc, 0x5d5,
0x5d1, 0x5dc, 0x5d9, 0x5d0, 0x5e0, 0x5d4, 0x5dc, 0x5d5, 0x5e0, 0x5d3, 0x5d5, 0x5df,
0x5dc, 0x5d5, 0x5e7, 0x5e1, 0x5de, 0x5d1, 0x5d5, 0x5e8, 0x5d2, 0x5de, 0x5d3, 0x5e8,
0x5d9, 0x5d3, 0x5de, 0x5dc, 0x5d8, 0x5d4, 0x5de, 0x5e8, 0x5d9, 0x5d9, 0x5d4, 0x5d0,
0x5de, 0x5df, 0x5de, 0x5d9, 0x5e0, 0x5e1, 0x5e7, 0x5de, 0x5d5, 0x5e0, 0x5e7, 0x5d5,
0x5de, 0x5d5, 0x5e1, 0x5e7, 0x5d1, 0x5d4, 0x5d0, 0x5d5, 0x5e1, 0x5dc, 0x5d5, 0x5e4,
0x5e8, 0x5d9, 0x5d6, 0x5e4, 0x5d5, 0x5d3, 0x5d2, 0x5d5, 0x5e8, 0x5d9, 0x5e6, 0x5d4,
0x5e4, 0x5e8, 0x5d0, 0x5d2, 0x5e8, 0x5d9, 0x5d2, 0x5d4, 0x5e8, 0x5d5, 0x5de, 0x5d0,
0x5e1, 0x5df, 0x20, 0x5de, 0x5e8, 0x5d9, 0x5e0, 0x5d5, 0x5e1, 0x5e8, 0x5d9, 0x5d9,
0x5d1, 0x5d5, 0x5e1, 0x5e8, 0x5d0, 0x5d8, 0x5d5, 0x5d1, 0x5e1, 0x5d9, 0x5de, 0x5e4,
0x5e8, 0x5d5, 0x5e4, 0x5d5, 0x5dc, 0x5e1, 0x5e7, 0x5d5, 0x5e4, 0x5d9, 0x5d4, 0x5e1,
0x5d5, 0x5e4, 0x5d9, 0x5d4, 0x5e9, 0x5d8, 0x5d5, 0x5e7, 0x5d4, 0x5d5, 0x5dc, 0x5dd,
0x5d8, 0x5d0, 0x5dc, 0x5d9, 0x5df, 0x5d8, 0x5d9, 0x5e8, 0x5e0, 0x5d4, 0x5d0, 0x5d5,
0x5dc, 0x5d9, 0x5d0, 0x5e0, 0x5d5, 0x5d1, 0x5e1, 0x5e7, 0x5d5, 0x5d0, 0x5d3, 0x5d5,
0x5e5, 0x5d4, 0x5d5, 0x5d5, 0x5ea, 0x5d9, 0x5e7, 0x5df, 0x5d5, 0x5d9, 0x5e0, 0x5d4,
0x5d5, 0x5d9, 0x5dc, 0x5e0, 0x5d4, 0x5d5, 0x5d5, 0x5dc, 0x5d2, 0x5d5, 0x5d2, 0x5e8,
0x5d3, 0x5d5, 0x5e8, 0x5e9, 0x5d4, 0x5d6, 0x5d0, 0x5d2, 0x5e8, 0x5d1, 0x5e6, 0x5d9,
0x5e8, 0x5d9, 0x5da, 0x5d0, 0x5e0, 0x5d8, 0x5e0, 0x5e0, 0x5e8, 0x5d9, 0x5d1, 0x5d5,
0x5e6, 0x5f3, 0x5d0, 0x5d2, 0x5d5, 0x5e1, 0x5d4, 0x5d0, 0x5d9, 0x20, 0x5db, 0x5e8,
0x5d9, 0x5e1, 0x5d8, 0x5de, 0x5e1, 0x5e7, 0x5d5, 0x5e7, 0x5d5, 0x5e1, 0x5e7, 0x5d5,
0x5de, 0x5d5, 0x5e8, 0x5d5, 0x5e7, 0x5e8, 0x5d2, 0x5d5, 0x5d5, 0x5dc, 0x5df, 0x5de,
0x5d4, 0x5d0, 0x5d4, 0x5d0, 0x5d9, 0x5d9, 0x5dd, 0x20, 0x5d4, 0x5de, 0x5dc, 0x5d3,
0x5d9, 0x5d1, 0x5d9, 0x5d9, 0x5dd, 0x5de, 0x5d0, 0x5d5, 0x5e8, 0x5d9, 0x5e6, 0x5d9,
0x5d5, 0x5e1, 0x5de, 0x5d0, 0x5d9, 0x5d5, 0x5d8, 0x5e8, 0x5d0, 0x5d5, 0x5e0, 0x5d9,
0x5d5, 0x5df, 0x5d0, 0x5e4, 0x5d9, 0x5d4, 0x5d0, 0x5d5, 0x5e7, 0x5dc, 0x5e0, 0x5d3,
0x5d1, 0x5d5, 0x5d2, 0x5e0, 0x5d5, 0x5d5, 0x5d9, 0x5dc, 0x5e6, 0x5f3, 0x5d0, 0x5d8,
0x5d4, 0x5d0, 0x5dd, 0x5e6, 0x5f3, 0x5d5, 0x5e7, 0x5d0, 0x5d9, 0x20, 0x5d4, 0x5e4,
0x5e1, 0x5d7, 0x5d0, 0x5d0, 0x5e4, 0x5d8, 0x5d4, 0x5e4, 0x5e7, 0x5d0, 0x5d5, 0x5e4,
0x5d5, 0x5e4, 0x5d9, 0x5d2, 0x5f3, 0x5d9, 0x5e4, 0x5d5, 0x5e0, 0x5e4, 0x5d5, 0x5d8,
0x5d9, 0x5d2, 0x5dc, 0x5e4, 0x5d0, 0x5d2, 0x5d5, 0x5e1, 0x5d0, 0x5d9, 0x5d9, 0x20,
0x5d2, 0x5de, 0x5d1, 0x5d9, 0x5d9, 0x5d4, 0x5d2, 0x5d5, 0x5d5, 0x5d3, 0x5dc, 0x5e7,
0x5e0, 0x5d0, 0x5dc, 0x5d2, 0x5d5, 0x5d0, 0x5dd, 0x5d4, 0x5d5, 0x5e0, 0x5d5, 0x5dc,
0x5d5, 0x5dc, 0x5d5, 0x5d0, 0x5e0, 0x5d3, 0x5e8, 0x5d1, 0x5d5, 0x5e8, 0x5d9, 0x5e7,
0x5d9, 0x5e8, 0x5d9, 0x5d8, 0x5d9, 0x5de, 0x5d0, 0x5d8, 0x5d9, 0x5e7, 0x5d5, 0x5e1,
0x5e8, 0x5d4, 0x5e7, 0x5d5, 0x5d5, 0x5d0, 0x5d2, 0x5f3, 0x5dc, 0x5d9, 0x5d9, 0x5df,
0x5de, 0x5d0, 0x5d2, 0x5f3, 0x5d5, 0x5e8, 0x5d5, 0x5d0, 0x5d9, 0x5d9, 0x20, 0x5de,
0x5e8, 0x5e7, 0x5d9, 0x5d6, 0x5de, 0x5d9, 0x5d3, 0x5d5, 0x5d5, 0x5d9, 0x5d9, 0x5e0,
0x5d0, 0x5d5, 0x5e8, 0x5d5, 0x5e0, 0x5d9, 0x5d5, 0x5d0, 0x5d4, 0x5e0, 0x5d5, 0x5e8,
0x5e4, 0x5d5, 0x5e7, 0x5e0, 0x5d5, 0x5de, 0x5d0, 0x5d4, 0x5e4, 0x5d0, 0x5d2, 0x5d5,
0x20, 0x5e4, 0x5d0, 0x5d2, 0x5d5, 0x5e4, 0x5dc, 0x5d0, 0x5d5, 0x5e4, 0x5d9, 0x5d8,
0x5e7, 0x5e8, 0x5df, 0x5e4, 0x5d5, 0x5e0, 0x5e4, 0x5d9, 0x5d9, 0x5e4, 0x5d5, 0x5e8,
0x5d8, 0x20, 0x5de, 0x5d5, 0x5e8, 0x5e1, 0x5d1, 0x5d9, 0x5e8, 0x5d0, 0x5e8, 0x5d5,
0x5d8, 0x5d5, 0x5e0, 0x5d2, 0x5d4, 0x5e1, 0x5d0, 0x5d9, 0x5e4, 0x5d0, 0x5df, 0x5d8,
0x5d4, 0x5d9, 0x5d8, 0x5d9, 0x5d8, 0x5d0, 0x5e8, 0x5d0, 0x5d5, 0x5d5, 0x5d4, 0x5d8,
0x5d5, 0x5e0, 0x5d2, 0x5d8, 0x5d0, 0x5e4, 0x5d5, 0x5d5, 0x5d9, 0x5d9, 0x5e7, 0x5d5,
0x5d5, 0x5dc, 0x5d9, 0x5e1, 0x90f, 0x915, 0x94d, 0x930, 0x93e, 0x905, 0x926, 0x940,
0x938, 0x20, 0x905, 0x92c, 0x93e, 0x92c, 0x93e, 0x905, 0x932, 0x94d, 0x91c, 0x940,
0x92f, 0x930, 0x94d, 0x938, 0x905, 0x938, 0x94d, 0x92e, 0x93e, 0x930, 0x93e, 0x92c,
0x93e, 0x92e, 0x93e, 0x915, 0x94b, 0x92c, 0x93e, 0x902, 0x917, 0x941, 0x907, 0x92c,
0x948, 0x902, 0x91c, 0x941, 0x932, 0x92c, 0x93f, 0x938, 0x93e, 0x90a, 0x92c, 0x94d,
0x932, 0x93e, 0x902, 0x91f, 0x93e, 0x92f, 0x930, 0x92c, 0x94d, 0x930, 0x93e, 0x91c,
0x93c, 0x93e, 0x935, 0x93f, 0x932, 0x947, 0x92c, 0x941, 0x91c, 0x941, 0x902, 0x92c,
0x942, 0x930, 0x93e, 0x915, 0x93e, 0x92f, 0x930, 0x94b, 0x938, 0x947, 0x909, 0x91f,
0x93e, 0x915, 0x94b, 0x928, 0x93e, 0x915, 0x94d, 0x930, 0x940, 0x921, 0x915, 0x93e,
0x930, 0x926, 0x93e, 0x930, 0x20, 0x905, 0x938, 0x20, 0x938, 0x932, 0x93e, 0x92e,
0x91c, 0x93f, 0x92c, 0x942, 0x924, 0x940, 0x921, 0x942, 0x906, 0x932, 0x93e, 0x905,
0x932, 0x20, 0x906, 0x907, 0x92f, 0x942, 0x928, 0x92b, 0x93c, 0x94d, 0x930, 0x940,
0x91f, 0x93e, 0x909, 0x928, 0x917, 0x93e, 0x92c, 0x94b, 0x930, 0x94b, 0x928, 0x939,
0x930, 0x93e, 0x930, 0x947, 0x91c, 0x94b, 0x939, 0x93e, 0x902, 0x938, 0x92c, 0x930,
0x94d, 0x917, 0x915, 0x902, 0x92a, 0x93e, 0x932, 0x93e, 0x915, 0x93f, 0x917, 0x93e,
0x932, 0x940, 0x915, 0x93f, 0x902, 0x936, 0x93e, 0x938, 0x93e, 0x932, 0x93e, 0x917,
0x94b, 0x938, 0x932, 0x93f, 0x92c, 0x94d, 0x930, 0x947, 0x935, 0x93f, 0x932, 0x947,
0x932, 0x94b, 0x92e, 0x932, 0x941, 0x906, 0x902, 0x921, 0x93e, 0x932, 0x941, 0x92c,
0x941, 0x92e, 0x92c, 0x93e, 0x936, 0x940, 0x92e, 0x932, 0x93e, 0x92c, 0x94b, 0x92e,
0x93e, 0x92a, 0x941, 0x91f, 0x94b, 0x90f, 0x92e, 0x92c, 0x93e, 0x92c, 0x947, 0x928,
0x92e, 0x94b, 0x917, 0x93e, 0x926, 0x93f, 0x936, 0x941, 0x92e, 0x94b, 0x928, 0x930,
0x94b, 0x935, 0x93f, 0x92f, 0x93e, 0x928, 0x948, 0x930, 0x94b, 0x92c, 0x940, 0x928,
0x947, 0x926, 0x94d, 0x91c, 0x93e, 0x92e, 0x940, 0x928, 0x93e, 0x928, 0x93f, 0x92f,
0x93e, 0x92e, 0x940, 0x928, 0x94c, 0x906, 0x915, 0x936, 0x949, 0x91f, 0x914, 0x917,
0x93e, 0x921, 0x94b, 0x917, 0x942, 0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x94b, 0x2d,
0x928, 0x94b, 0x935, 0x94b, 0x938, 0x93e, 0x913, 0x20, 0x91f, 0x94b, 0x92e, 0x924,
0x94d, 0x930, 0x93f, 0x92a, 0x94b, 0x932, 0x940, 0x935, 0x93f, 0x902, 0x921, 0x939,
0x94b, 0x915, 0x905, 0x921, 0x915, 0x90f, 0x902, 0x917, 0x94d, 0x935, 0x93f, 0x932,
0x93e, 0x90f, 0x902, 0x91f, 0x940, 0x917, 0x941, 0x906, 0x906, 0x930, 0x93e, 0x917,
0x94d, 0x935, 0x947, 0x928, 0x93e, 0x92c, 0x94d, 0x92f, 0x942, 0x928, 0x938, 0x20,
0x906, 0x92f, 0x930, 0x938, 0x915, 0x94b, 0x930, 0x94d, 0x921, 0x94b, 0x92c, 0x93e,
0x91c, 0x941, 0x91c, 0x94b, 0x90f, 0x932, 0x93e, 0x20, 0x930, 0x93f, 0x913, 0x91c,
0x93e, 0x92e, 0x947, 0x902, 0x921, 0x94b, 0x91c, 0x93c, 0x93e, 0x930, 0x93f, 0x92f,
0x94b, 0x20, 0x917, 0x93e, 0x932, 0x947, 0x917, 0x94b, 0x938, 0x938, 0x948, 0x928,
0x20, 0x939, 0x94d, 0x935, 0x93e, 0x928, 0x938, 0x948, 0x928, 0x20, 0x932, 0x942,
0x908, 0x91f, 0x94b, 0x915, 0x942, 0x92e, 0x928, 0x909, 0x936, 0x941, 0x906, 0x907,
0x92f, 0x93e, 0x905, 0x930, 0x942, 0x92c, 0x93e, 0x90f, 0x938, 0x928, 0x936, 0x93f,
0x92f, 0x949, 0x928, 0x905, 0x91f, 0x93f, 0x915, 0x94b, 0x915, 0x93e, 0x928, 0x92c,
0x939, 0x93f, 0x92f, 0x93e, 0x92c, 0x947, 0x939, 0x93f, 0x92f, 0x93e, 0x20, 0x92c,
0x93e, 0x902, 0x921, 0x947, 0x930, 0x93e, 0x938, 0x92c, 0x93e, 0x930, 0x92c, 0x93e,
0x921, 0x94b, 0x938, 0x92c, 0x947, 0x932, 0x940, 0x91c, 0x93c, 0x92c, 0x94d, 0x932,
0x93e, 0x902, 0x915, 0x2d, 0x938, 0x947, 0x92c, 0x932, 0x94b, 0x928, 0x92c, 0x94b,
0x906, 0x20, 0x935, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x92c, 0x94b, 0x917, 0x94b,
0x91f, 0x93e, 0x92c, 0x949, 0x907, 0x938, 0x940, 0x915, 0x948, 0x92e, 0x94d, 0x92c,
0x94d, 0x930, 0x93f, 0x91c, 0x20, 0x916, 0x93e, 0x921, 0x93c, 0x940, 0x915, 0x948,
0x902, 0x92a, 0x94b, 0x20, 0x917, 0x94d, 0x930, 0x93e, 0x902, 0x921, 0x947, 0x915,
0x948, 0x928, 0x915, 0x941, 0x928, 0x915, 0x93e, 0x930, 0x93e, 0x915, 0x938, 0x936,
0x93f, 0x915, 0x93e, 0x917, 0x94b, 0x91a, 0x93f, 0x939, 0x941, 0x906, 0x939, 0x941,
0x906, 0x938, 0x94d, 0x92f, 0x941, 0x926, 0x93e, 0x926, 0x20, 0x939, 0x94d, 0x935,
0x93e, 0x930, 0x947, 0x91c, 0x93c, 0x915, 0x94b, 0x938, 0x94d, 0x91f, 0x93e, 0x20,
0x930, 0x93f, 0x915, 0x93e, 0x915, 0x94d, 0x92f, 0x942, 0x906, 0x92c, 0x93e, 0x915,
0x941, 0x930, 0x93e, 0x915, 0x93e, 0x913, 0x921, 0x947, 0x928, 0x92e, 0x93e, 0x930,
0x94d, 0x915, 0x936, 0x949, 0x928, 0x921, 0x949, 0x938, 0x928, 0x921, 0x949, 0x938,
0x928, 0x20, 0x915, 0x94d, 0x930, 0x940, 0x915, 0x921, 0x947, 0x928, 0x935, 0x930,
0x921, 0x947, 0x91f, 0x94d, 0x930, 0x949, 0x92f, 0x91f, 0x921, 0x94b, 0x92e, 0x93f,
0x928, 0x93f, 0x915, 0x93e, 0x90f, 0x921, 0x92e, 0x902, 0x91f, 0x928, 0x905, 0x932,
0x20, 0x938, 0x932, 0x94d, 0x935, 0x93e, 0x921, 0x94b, 0x930, 0x92b, 0x93c, 0x94b,
0x930, 0x94d, 0x91f, 0x20, 0x928, 0x947, 0x932, 0x94d, 0x938, 0x928, 0x92b, 0x93c,
0x94b, 0x930, 0x94d, 0x91f, 0x93e, 0x932, 0x947, 0x91c, 0x93c, 0x93e, 0x917, 0x94d,
0x932, 0x947, 0x938, 0x20, 0x916, 0x93e, 0x921, 0x93c, 0x940, 0x917, 0x942, 0x938,
0x20, 0x916, 0x93e, 0x921, 0x93c, 0x940, 0x917, 0x94d, 0x930, 0x93e, 0x902, 0x921,
0x20, 0x91f, 0x930, 0x94d, 0x915, 0x917, 0x94d, 0x935, 0x93e, 0x921, 0x947, 0x932,
0x94b, 0x92a, 0x917, 0x94d, 0x935, 0x93e, 0x92f, 0x93e, 0x915, 0x940, 0x932, 0x917,
0x92f, 0x93e, 0x928, 0x93e, 0x939, 0x947, 0x932, 0x93f, 0x92b, 0x93c, 0x948, 0x915,
0x94d, 0x938, 0x939, 0x935, 0x93e, 0x928, 0x93e, 0x939, 0x930, 0x94d, 0x92e, 0x94b,
0x938, 0x93f, 0x932, 0x94d, 0x932, 0x94b, 0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e,
0x928, 0x93e, 0x92a, 0x94b, 0x932, 0x93f, 0x938, 0x928, 0x94c, 0x915, 0x94d, 0x938,
0x2c, 0x20, 0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x92e, 0x93e,
0x930, 0x947, 0x902, 0x917, 0x94b, 0x2c, 0x20, 0x907, 0x902, 0x921, 0x93f, 0x92f,
0x93e, 0x928, 0x93e, 0x92a, 0x940, 0x91f, 0x930, 0x94d, 0x938, 0x92c, 0x930, 0x94d,
0x917, 0x2c, 0x20, 0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x91f,
0x947, 0x932, 0x20, 0x938, 0x93f, 0x91f, 0x940, 0x2c, 0x20, 0x907, 0x902, 0x921,
0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x935, 0x947, 0x935, 0x947, 0x2c, 0x20, 0x907,
0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x935, 0x93f, 0x902, 0x938, 0x947,
0x928, 0x947, 0x938, 0x2c, 0x20, 0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928,
0x93e, 0x935, 0x93f, 0x928, 0x93e, 0x92e, 0x947, 0x915, 0x2c, 0x20, 0x907, 0x902,
0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x907, 0x915, 0x93e, 0x932, 0x941, 0x908,
0x91f, 0x91c, 0x92e, 0x948, 0x915, 0x93e, 0x91c, 0x94d, 0x92f, 0x942, 0x928, 0x93e,
0x909, 0x932, 0x941, 0x907, 0x938, 0x935, 0x93f, 0x932, 0x947, 0x92e, 0x94b, 0x902,
0x91f, 0x940, 0x938, 0x947, 0x932, 0x94b, 0x2c, 0x20, 0x915, 0x947, 0x902, 0x91f,
0x915, 0x940, 0x915, 0x94d, 0x930, 0x93e, 0x932, 0x948, 0x902, 0x921, 0x93f, 0x91c,
0x93f, 0x915, 0x932, 0x93e, 0x20, 0x92a, 0x93e, 0x91c, 0x93c, 0x932, 0x949, 0x938,
0x20, 0x90f, 0x902, 0x91c, 0x93f, 0x932, 0x94d, 0x938, 0x932, 0x94b, 0x905, 0x930,
0x20, 0x92a, 0x94d, 0x930, 0x93f, 0x902, 0x938, 0x947, 0x938, 0x20, 0x915, 0x94d,
0x935, 0x93e, 0x930, 0x94d, 0x91f, 0x930, 0x92e, 0x947, 0x938, 0x940, 0x913, 0x92e,
0x93e, 0x928, 0x93e, 0x917, 0x941, 0x906, 0x92e, 0x928, 0x94c, 0x938, 0x92e, 0x948,
0x930, 0x940, 0x917, 0x94b, 0x91f, 0x92e, 0x93e, 0x930, 0x94d, 0x91f, 0x93f, 0x928,
0x93f, 0x915, 0x92e, 0x93e, 0x91f, 0x93e, 0x92e, 0x94b, 0x930, 0x94b, 0x938, 0x92e,
0x93e, 0x91c, 0x93c, 0x93e, 0x91f, 0x932, 0x93e, 0x928, 0x92e, 0x947, 0x928, 0x94b,
0x92e, 0x93f, 0x928, 0x940, 0x92e, 0x947, 0x91f, 0x94d, 0x932, 0x947, 0x915, 0x93e,
0x91f, 0x94d, 0x932, 0x93e, 0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x94b,
0x20, 0x938, 0x93f, 0x91f, 0x940, 0x92e, 0x93f, 0x915, 0x947, 0x932, 0x949, 0x928,
0x92e, 0x94b, 0x902, 0x915, 0x91f, 0x928, 0x92e, 0x94b, 0x902, 0x91f, 0x947, 0x930,
0x947, 0x930, 0x940, 0x92e, 0x94b, 0x902, 0x91f, 0x947, 0x935, 0x940, 0x921, 0x93f,
0x92f, 0x94b, 0x92e, 0x94b, 0x902, 0x91f, 0x938, 0x947, 0x930, 0x93e, 0x924, 0x928,
0x93e, 0x938, 0x93e, 0x909, 0x928, 0x94d, 0x92f, 0x942, 0x92f, 0x949, 0x930, 0x94d,
0x915, 0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x92c, 0x94d, 0x92f,
0x942, 0x932, 0x93e, 0x2c, 0x20, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20,
0x921, 0x915, 0x94b, 0x91f, 0x93e, 0x92e, 0x927, 0x94d, 0x92f, 0x2c, 0x20, 0x909,
0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x926, 0x93e, 0x915, 0x94b, 0x924, 0x93e,
0x928, 0x94d, 0x92f, 0x942, 0x20, 0x938, 0x93e, 0x932, 0x947, 0x92e, 0x2c, 0x20,
0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x921, 0x915, 0x94b, 0x91f, 0x93e,
0x928, 0x941, 0x915, 0x913, 0x916, 0x93e, 0x91c, 0x940, 0x928, 0x93e, 0x917, 0x93e,
0x92a, 0x928, 0x93e, 0x92e, 0x93e, 0x92a, 0x93e, 0x930, 0x93e, 0x92e, 0x93e, 0x930,
0x93f, 0x92c, 0x94b, 0x92b, 0x93c, 0x940, 0x928, 0x93f, 0x915, 0x94d, 0x938, 0x92a,
0x94b, 0x930, 0x94d, 0x91f, 0x2d, 0x911, 0x2d, 0x92a, 0x94d, 0x930, 0x93f, 0x902,
0x938, 0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x20, 0x911, 0x92b, 0x93c, 0x20, 0x938,
0x94d, 0x92a, 0x947, 0x928, 0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x94b, 0x20, 0x935,
0x947, 0x932, 0x94d, 0x939, 0x94b, 0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x94b, 0x20,
0x930, 0x93f, 0x915, 0x94b, 0x92a, 0x941, 0x902, 0x91f, 0x93e, 0x20, 0x90f, 0x930,
0x93f, 0x928, 0x93e, 0x938, 0x930, 0x947, 0x902, 0x915, 0x93f, 0x928, 0x20, 0x907,
0x928, 0x932, 0x947, 0x91f, 0x930, 0x947, 0x938, 0x93e, 0x907, 0x92b, 0x93c, 0x930,
0x93f, 0x938, 0x94b, 0x932, 0x94d, 0x92f, 0x942, 0x91f, 0x930, 0x93f, 0x92f, 0x94b,
0x20, 0x92c, 0x94d, 0x930, 0x93e, 0x902, 0x915, 0x94b, 0x938, 0x948, 0x902, 0x91f,
0x93e, 0x930, 0x947, 0x92e, 0x938, 0x948, 0x902, 0x91f, 0x93f, 0x92f, 0x93e, 0x917,
0x94b, 0x938, 0x947, 0x902, 0x91f, 0x94b, 0x20, 0x921, 0x94b, 0x92e, 0x93f, 0x902,
0x917, 0x94b, 0x938, 0x93e, 0x913, 0x20, 0x92a, 0x93e, 0x909, 0x932, 0x94b, 0x907,
0x91f, 0x94b, 0x915, 0x94b, 0x930, 0x94d, 0x91f, 0x94b, 0x930, 0x92e, 0x93f, 0x91f,
0x938, 0x93f, 0x91f, 0x94d, 0x915, 0x93e, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x92c,
0x93e, 0x930, 0x94d, 0x925, 0x947, 0x932, 0x947, 0x92e, 0x940, 0x938, 0x947, 0x902,
0x91f, 0x20, 0x91c, 0x94b, 0x902, 0x938, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x915,
0x93f, 0x91f, 0x94d, 0x938, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x932, 0x942, 0x938,
0x93f, 0x92f, 0x93e, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x925, 0x949, 0x92e, 0x938,
0x938, 0x947, 0x902, 0x91f, 0x20, 0x935, 0x93f, 0x902, 0x938, 0x947, 0x902, 0x91f,
0x938, 0x94d, 0x935, 0x93f, 0x92b, 0x93c, 0x94d, 0x91f, 0x20, 0x915, 0x930, 0x902,
0x91f, 0x91f, 0x947, 0x917, 0x941, 0x938, 0x93f, 0x917, 0x932, 0x94d, 0x92a, 0x93e,
0x925, 0x94d, 0x92f, 0x942, 0x932, 0x947, 0x924, 0x93f, 0x91c, 0x941, 0x906, 0x928,
0x93e, 0x91f, 0x94b, 0x930, 0x902, 0x91f, 0x94b, 0x91f, 0x94b, 0x930, 0x94d, 0x91f,
0x94b, 0x932, 0x93e, 0x935, 0x948, 0x902, 0x915, 0x942, 0x935, 0x930, 0x935, 0x94d,
0x939, 0x93e, 0x907, 0x91f, 0x939, 0x94b, 0x930, 0x94d, 0x938, 0x935, 0x93f, 0x928,
0x940, 0x92a, 0x947, 0x917, 0x92f, 0x93e, 0x915, 0x942, 0x91f, 0x93e, 0x91f, 0x921,
0x947, 0x935, 0x93f, 0x938, 0x921, 0x94d, 0x92f, 0x942, 0x92e, 0x94b, 0x902, 0x91f,
0x20, 0x921, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x935, 0x93f, 0x932, 0x947, 0x92e,
0x915, 0x94d, 0x935, 0x93e, 0x930, 0x940, 0x92e, 0x949, 0x938, 0x928, 0x92e, 0x948,
0x915, 0x92e, 0x941, 0x930, 0x94d, 0x921, 0x94b, 0x92a, 0x949, 0x92e, 0x930, 0x930,
0x94b, 0x925, 0x947, 0x930, 0x93e, 0x938, 0x94d, 0x92f, 0x94b, 0x935, 0x93e, 0x91f,
0x94d, 0x930, 0x94b, 0x932, 0x935, 0x94b, 0x938, 0x94d, 0x924, 0x94b, 0x915, 0x932,
0x949, 0x928, 0x94d, 0x917, 0x908, 0x92f, 0x930, 0x92c, 0x93e, 0x92f, 0x947, 0x928,
0x906, 0x926, 0x947, 0x928, 0x905, 0x932, 0x94d, 0x92e, 0x93e, 0x91f, 0x940, 0x905,
0x92e, 0x94d, 0x92e, 0x93e, 0x928, 0x905, 0x928, 0x93e, 0x921, 0x93f, 0x930, 0x905,
0x915, 0x94d, 0x924, 0x94b, 0x92c, 0x90f, 0x924, 0x930, 0x93e, 0x909, 0x92c, 0x917,
0x926, 0x93e, 0x926, 0x92c, 0x939, 0x930, 0x940, 0x928, 0x92c, 0x948, 0x902, 0x915,
0x949, 0x915, 0x92c, 0x930, 0x94d, 0x928, 0x94b, 0x932, 0x92c, 0x947, 0x930, 0x941,
0x924, 0x92c, 0x94d, 0x930, 0x942, 0x928, 0x947, 0x908, 0x924, 0x94d, 0x936, 0x93f,
0x924, 0x93e, 0x915, 0x94b, 0x932, 0x902, 0x92c, 0x94b, 0x926, 0x92e, 0x93e, 0x938,
0x94d, 0x915, 0x938, 0x922, 0x93e, 0x915, 0x93e, 0x921, 0x93f, 0x932, 0x93f, 0x926,
0x941, 0x92c, 0x908, 0x926, 0x941, 0x936, 0x93e, 0x902, 0x92c, 0x947, 0x92b, 0x93c,
0x93e, 0x92e, 0x93e, 0x917, 0x941, 0x938, 0x94d, 0x924, 0x93e, 0x917, 0x93e, 0x91c,
0x93c, 0x93e, 0x939, 0x947, 0x92c, 0x94d, 0x930, 0x94b, 0x928, 0x939, 0x94b, 0x20,
0x91a, 0x940, 0x20, 0x92e, 0x93f, 0x928, 0x94d, 0x939, 0x20, 0x938, 0x93f, 0x91f,
0x940, 0x939, 0x93e, 0x901, 0x917, 0x20, 0x915, 0x93e, 0x901, 0x917, 0x939, 0x94b,
0x935, 0x94d, 0x921, 0x907, 0x930, 0x94d, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d,
0x915, 0x91c, 0x915, 0x93e, 0x930, 0x94d, 0x924, 0x93e, 0x92f, 0x930, 0x942, 0x936,
0x932, 0x92e, 0x915, 0x93e, 0x92c, 0x941, 0x932, 0x915, 0x92e, 0x91a, 0x924, 0x94d,
0x915, 0x93e, 0x915, 0x93e, 0x920, 0x92e, 0x93e, 0x902, 0x921, 0x942, 0x916, 0x93e,
0x902, 0x921, 0x93f, 0x917, 0x93e, 0x915, 0x94b, 0x932, 0x915, 0x93e, 0x924, 0x93e,
0x915, 0x94d, 0x930, 0x93e, 0x938, 0x94d, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d,
0x938, 0x94d, 0x915, 0x915, 0x941, 0x906, 0x932, 0x93e, 0x932, 0x902, 0x92a, 0x941,
0x930, 0x92e, 0x915, 0x93e, 0x90a, 0x92e, 0x915, 0x938, 0x94d, 0x938, 0x930, 0x92e,
0x938, 0x94d, 0x915, 0x91f, 0x928, 0x93f, 0x915, 0x94b, 0x938, 0x93f, 0x92f, 0x93e,
0x928, 0x94b, 0x935, 0x94b, 0x915, 0x941, 0x91c, 0x93c, 0x94d, 0x928, 0x947, 0x924,
0x94d, 0x938, 0x94d, 0x915, 0x928, 0x94b, 0x935, 0x94b, 0x938, 0x93f, 0x92c, 0x93f,
0x930, 0x94d, 0x938, 0x94d, 0x915, 0x913, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x913,
0x930, 0x932, 0x928, 0x949, 0x92e, 0x20, 0x92a, 0x947, 0x928, 0x94d, 0x939, 0x92a,
0x94b, 0x902, 0x91f, 0x940, 0x92f, 0x93e, 0x902, 0x915, 0x92a, 0x94d, 0x92f, 0x94b,
0x902, 0x917, 0x92f, 0x93e, 0x902, 0x917, 0x915, 0x924, 0x930, 0x915, 0x94b, 0x938,
0x94d, 0x91f, 0x93e, 0x928, 0x947, 0x915, 0x947, 0x91c, 0x93c, 0x947, 0x932, 0x94b,
0x930, 0x94d, 0x921, 0x93e, 0x930, 0x93f, 0x92f, 0x93e, 0x926, 0x938, 0x916, 0x93e,
0x932, 0x93f, 0x928, 0x938, 0x92e, 0x930, 0x915, 0x902, 0x926, 0x938, 0x93f, 0x913,
0x932, 0x936, 0x902, 0x918, 0x93e, 0x908, 0x938, 0x94d, 0x930, 0x947, 0x926, 0x94d,
0x928, 0x93f, 0x915, 0x94b, 0x932, 0x93f, 0x92e, 0x938, 0x94d, 0x915, 0x924, 0x93e,
0x908, 0x92a, 0x947, 0x908, 0x924, 0x93e, 0x936, 0x915, 0x902, 0x926, 0x91f, 0x92c,
0x93f, 0x932, 0x93f, 0x938, 0x940, 0x925, 0x93f, 0x902, 0x92a, 0x942, 0x91f, 0x94b,
0x915, 0x94d, 0x92f, 0x94b, 0x924, 0x94b, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x909,
0x932, 0x93e, 0x928, 0x92c, 0x93e, 0x924, 0x930, 0x909, 0x930, 0x942, 0x92e, 0x94d,
0x915, 0x940, 0x92f, 0x942, 0x938, 0x94d, 0x91f, 0x2013, 0x928, 0x947, 0x930, 0x93e,
0x935, 0x93f, 0x90f, 0x928, 0x924, 0x93f, 0x92f, 0x93e, 0x928, 0x935, 0x94d, 0x932,
0x93e, 0x926, 0x93f, 0x935, 0x94b, 0x938, 0x94d, 0x924, 0x94b, 0x915, 0x92f, 0x947,
0x915, 0x93e, 0x924, 0x947, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x92f,
0x947, 0x930, 0x947, 0x935, 0x93e, 0x928, 0x905, 0x91c, 0x93c, 0x94b, 0x930, 0x947,
0x938, 0x92c, 0x930, 0x92e, 0x942, 0x921, 0x93e, 0x915, 0x948, 0x928, 0x947, 0x930,
0x940, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x930, 0x94d, 0x921, 0x92b, 0x93c, 0x948,
0x930, 0x94b, 0x92e, 0x921, 0x947, 0x930, 0x93e, 0x930, 0x947, 0x915, 0x94d, 0x92f,
0x93e, 0x935, 0x93f, 0x915, 0x926, 0x915, 0x94d, 0x937, 0x93f, 0x923, 0x20, 0x91c,
0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x938, 0x947, 0x902, 0x91f, 0x20,
0x939, 0x947, 0x932, 0x947, 0x928, 0x93e, 0x938, 0x94d, 0x91f, 0x948, 0x928, 0x932,
0x940, 0x90f, 0x921, 0x93f, 0x932, 0x947, 0x921, 0x92c, 0x94d, 0x930, 0x93f, 0x938,
0x94d, 0x92c, 0x928, 0x92c, 0x94d, 0x930, 0x94b, 0x915, 0x928, 0x20, 0x939, 0x93f,
0x932, 0x921, 0x93e, 0x930, 0x94d, 0x935, 0x93f, 0x928, 0x939, 0x94b, 0x92c, 0x93e,
0x930, 0x94d, 0x91f, 0x932, 0x93f, 0x902, 0x921, 0x947, 0x92e, 0x93e, 0x928, 0x932,
0x949, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x94b, 0x935, 0x947, 0x92e, 0x947, 0x932,
0x92c, 0x94b, 0x930, 0x94d, 0x928, 0x92a, 0x930, 0x94d, 0x925, 0x90f, 0x92e, 0x94d,
0x938, 0x94d, 0x91f, 0x930, 0x94d, 0x921, 0x92e, 0x905, 0x902, 0x921, 0x94b, 0x930,
0x93e, 0x906, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x93e, 0x916, 0x93e, 0x928, 0x90f,
0x925, 0x947, 0x902, 0x938, 0x92c, 0x930, 0x94d, 0x932, 0x93f, 0x928, 0x92c, 0x94d,
0x930, 0x93e, 0x924, 0x93f, 0x938, 0x94d, 0x932, 0x93e, 0x935, 0x93e, 0x92c, 0x94d,
0x930, 0x942, 0x938, 0x947, 0x932, 0x94d, 0x938, 0x92c, 0x941, 0x916, 0x93c, 0x93e,
0x930, 0x947, 0x938, 0x94d, 0x91f, 0x92c, 0x941, 0x921, 0x93e, 0x92a, 0x947, 0x938,
0x94d, 0x91f, 0x92c, 0x94d, 0x92f, 0x942, 0x938, 0x93f, 0x928, 0x91c, 0x947, 0x928,
0x91a, 0x93f, 0x938, 0x940, 0x928, 0x93e, 0x909, 0x915, 0x94b, 0x92a, 0x947, 0x928,
0x939, 0x947, 0x917, 0x928, 0x921, 0x92c, 0x932, 0x93f, 0x928, 0x91c, 0x93f, 0x92c,
0x94d, 0x930, 0x93e, 0x932, 0x94d, 0x91f, 0x930, 0x917, 0x930, 0x94d, 0x928, 0x938,
0x940, 0x939, 0x947, 0x932, 0x938, 0x93f, 0x902, 0x915, 0x940, 0x906, 0x907, 0x932,
0x20, 0x911, 0x92b, 0x93c, 0x20, 0x92e, 0x948, 0x928, 0x907, 0x938, 0x94d, 0x924,
0x93e, 0x902, 0x92c, 0x941, 0x932, 0x91c, 0x930, 0x94d, 0x938, 0x940, 0x915, 0x93e,
0x932, 0x940, 0x928, 0x93f, 0x928, 0x917, 0x94d, 0x930, 0x93e, 0x921, 0x915, 0x93f,
0x930, 0x94b, 0x935, 0x915, 0x940, 0x935, 0x932, 0x94d, 0x92f, 0x942, 0x92c, 0x947,
0x932, 0x91c, 0x93e, 0x928, 0x93e, 0x932, 0x902, 0x926, 0x928, 0x932, 0x915, 0x94d,
0x91c, 0x93c, 0x92e, 0x92c, 0x930, 0x94d, 0x917, 0x92e, 0x948, 0x921, 0x94d, 0x930,
0x93f, 0x921, 0x92e, 0x93e, 0x930, 0x93f, 0x92f, 0x93e, 0x939, 0x948, 0x92e, 0x92e,
0x93f, 0x902, 0x938, 0x94d, 0x915, 0x92e, 0x94b, 0x928, 0x93e, 0x915, 0x94b, 0x92e,
0x949, 0x938, 0x94d, 0x915, 0x94b, 0x913, 0x938, 0x94d, 0x932, 0x94b, 0x92a, 0x94b,
0x921, 0x94d, 0x917, 0x94b, 0x930, 0x93f, 0x915, 0x93e, 0x92a, 0x94d, 0x930, 0x93e,
0x917, 0x930, 0x940, 0x917, 0x93e, 0x938, 0x92e, 0x93e, 0x930, 0x93e, 0x938, 0x948,
0x928, 0x20, 0x92e, 0x93e, 0x930, 0x940, 0x928, 0x94b, 0x938, 0x93e, 0x930, 0x93e,
0x91c, 0x947, 0x935, 0x94b, 0x938, 0x93e, 0x930, 0x93e, 0x924, 0x94b, 0x935, 0x938,
0x93f, 0x92e, 0x94d, 0x92b, 0x93c, 0x947, 0x930, 0x94b, 0x92a, 0x94b, 0x932, 0x938,
0x94d, 0x915, 0x94b, 0x92a, 0x94d, 0x91c, 0x947, 0x938, 0x94b, 0x92b, 0x93c, 0x93f,
0x92f, 0x93e, 0x938, 0x94d, 0x91f, 0x949, 0x915, 0x939, 0x94b, 0x92e, 0x924, 0x947,
0x932, 0x93f, 0x928, 0x91f, 0x93e, 0x907, 0x930, 0x947, 0x928, 0x909, 0x932, 0x94d,
0x92f, 0x93e, 0x928, 0x94b, 0x935, 0x94d, 0x938, 0x94d, 0x915, 0x935, 0x93e, 0x926,
0x941, 0x91c, 0x93c, 0x935, 0x947, 0x91f, 0x93f, 0x915, 0x928, 0x935, 0x93f, 0x90f,
0x928, 0x93e, 0x935, 0x93f, 0x932, 0x94d, 0x928, 0x93f, 0x92f, 0x938, 0x935, 0x94b,
0x932, 0x94d, 0x917, 0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x921, 0x935, 0x949, 0x930,
0x938, 0x949, 0x91c, 0x93c, 0x93e, 0x917, 0x94d, 0x930, 0x947, 0x92c, 0x91c, 0x93c,
0x94d, 0x92f, 0x942, 0x930, 0x93f, 0x916, 0x93c, 0x90f, 0x902, 0x91f, 0x93e, 0x928,
0x93e, 0x928, 0x930, 0x940, 0x935, 0x94b, 0x936, 0x93e, 0x917, 0x94b, 0x938, 0x915,
0x94d, 0x930, 0x93f, 0x938, 0x92e, 0x938, 0x915, 0x94b, 0x915, 0x94b, 0x938, 0x915,
0x94b, 0x92e, 0x94b, 0x930, 0x94b, 0x915, 0x930, 0x917, 0x941, 0x932, 0x947, 0x928,
0x92e, 0x93e, 0x932, 0x926, 0x940, 0x935, 0x92e, 0x949, 0x930, 0x940, 0x936, 0x938,
0x92e, 0x93e, 0x92f, 0x94b, 0x924, 0x94d, 0x924, 0x947, 0x930, 0x940, 0x92f, 0x942,
0x928, 0x93f, 0x92f, 0x928, 0x90f, 0x92a, 0x93f, 0x92f, 0x93e, 0x911, 0x915, 0x932,
0x948, 0x902, 0x921, 0x92c, 0x94b, 0x917, 0x928, 0x935, 0x93f, 0x932, 0x947, 0x91a,
0x948, 0x925, 0x92e, 0x91a, 0x915, 0x908, 0x938, 0x94d, 0x91f, 0x930, 0x90f, 0x92b,
0x93c, 0x947, 0x91f, 0x92b, 0x93c, 0x93e, 0x915, 0x93e, 0x913, 0x92b, 0x93c, 0x94b,
0x92b, 0x93c, 0x93f, 0x91c, 0x940, 0x92b, 0x93c, 0x94d, 0x92f, 0x942, 0x928, 0x93e,
0x92b, 0x93c, 0x941, 0x91f, 0x940, 0x917, 0x947, 0x932, 0x93e, 0x92a, 0x93e, 0x917,
0x94b, 0x938, 0x917, 0x948, 0x902, 0x92c, 0x93f, 0x92f, 0x930, 0x917, 0x94d, 0x935,
0x93e, 0x921, 0x932, 0x915, 0x928, 0x93e, 0x932, 0x939, 0x94b, 0x928, 0x94b, 0x932,
0x941, 0x932, 0x941, 0x90f, 0x902, 0x921, 0x930, 0x92c, 0x930, 0x940, 0x915, 0x94b,
0x938, 0x930, 0x93e, 0x90f, 0x915, 0x94d, 0x935, 0x93e, 0x91c, 0x93c, 0x93e, 0x932,
0x940, 0x928, 0x92e, 0x93e, 0x91c, 0x941, 0x930, 0x94b, 0x92e, 0x93e, 0x930, 0x94d,
0x915, 0x947, 0x938, 0x938, 0x92e, 0x93f, 0x921, 0x935, 0x947, 0x928, 0x94c, 0x930,
0x941, 0x928, 0x940, 0x92f, 0x942, 0x928, 0x949, 0x930, 0x92b, 0x93c, 0x949, 0x915,
0x928, 0x94c, 0x92e, 0x93f, 0x92f, 0x93e, 0x92a, 0x93e, 0x917, 0x94b, 0x20, 0x92a,
0x93e, 0x917, 0x94b, 0x92a, 0x932, 0x93e, 0x90a, 0x92a, 0x94b, 0x928, 0x92a, 0x947,
0x908, 0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x20, 0x92e, 0x94b, 0x930, 0x947, 0x938,
0x94d, 0x92c, 0x940, 0x930, 0x93e, 0x930, 0x94b, 0x91f, 0x94b, 0x902, 0x917, 0x93e,
0x938, 0x93e, 0x92f, 0x92a, 0x93e, 0x928, 0x924, 0x93e, 0x939, 0x93f, 0x924, 0x940,
0x91f, 0x93e, 0x930, 0x93e, 0x935, 0x93e, 0x91f, 0x94b, 0x902, 0x917, 0x93e, 0x91f,
0x93e, 0x92a, 0x942, 0x41, 0x73, 0x6d, 0x65, 0x72, 0x61, 0x53, 0x61, 0x6f,
0x20, 0x54, 0x6f, 0x6d, 0x65, 0x43, 0x6f, 0x72, 0x61, 0x6c, 0x20, 0x48,
0x61, 0x72, 0x62, 0x6f, 0x75, 0x72, 0x43, 0x69, 0x75, 0x64, 0x61, 0x64,
0x20, 0x4a, 0x75, 0x61, 0x72, 0x65, 0x7a, 0x43, 0x75, 0x72, 0x61, 0x63,
0x61, 0x6f, 0x53, 0x74, 0x20, 0x42, 0x61, 0x72, 0x74, 0x68, 0x65, 0x6c,
0x65, 0x6d, 0x79, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x44, 0x55, 0x72,
0x76, 0x69, 0x6c, 0x6c, 0x65, 0x41, 0x71, 0x74, 0x61, 0x75, 0x53, 0x61,
0x69, 0x67, 0x6f, 0x6e, 0x51, 0x6f, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x79,
0x46, 0x61, 0x65, 0x72, 0x6f, 0x65, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f,
0x6e, 0x54, 0x72, 0x75, 0x6b, 0x41, 0x64, 0x64, 0x69, 0x73, 0x7a, 0x2d,
0x41, 0x62, 0x65, 0x62, 0x61, 0x41, 0x6c, 0x67, 0xed, 0x72, 0x41, 0x73,
0x7a, 0x6d, 0x61, 0x72, 0x61, 0x4b, 0x61, 0x69, 0x72, 0xf3, 0x44, 0x61,
0x72, 0x20, 0x65, 0x73, 0x2d, 0x53, 0x61, 0x6c, 0x61, 0x61, 0x6d, 0x44,
0x7a, 0x73, 0x69, 0x62, 0x75, 0x74, 0x69, 0x45, 0x6c, 0x2d, 0x41, 0x6a,
0xfa, 0x6e, 0x4b, 0x61, 0x72, 0x74, 0xfa, 0x6d, 0x4d, 0x61, 0x6c, 0x61,
0x62, 0xf3, 0x54, 0x75, 0x6e, 0x69, 0x73, 0x7a, 0x4b, 0x61, 0x6a, 0x6d,
0xe1, 0x6e, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x4d,
0x65, 0x78, 0x69, 0x6b, 0xf3, 0x76, 0xe1, 0x72, 0x6f, 0x73, 0x42, 0x65,
0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0xc9, 0x73, 0x7a, 0x61, 0x6b, 0x2d,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72,
0x2c, 0x20, 0xc9, 0x73, 0x7a, 0x61, 0x6b, 0x2d, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c,
0x20, 0xc9, 0x73, 0x7a, 0x61, 0x6b, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74,
0x61, 0x56, 0x6f, 0x73, 0x7a, 0x74, 0x6f, 0x6b, 0xc1, 0x64, 0x65, 0x6e,
0x42, 0x69, 0x73, 0x6b, 0x65, 0x6b, 0x43, 0x73, 0x69, 0x74, 0x61, 0x44,
0x61, 0x6d, 0x61, 0x73, 0x7a, 0x6b, 0x75, 0x73, 0x7a, 0x44, 0x61, 0x6b,
0x6b, 0x61, 0x44, 0x75, 0x73, 0x61, 0x6e, 0x62, 0x65, 0x47, 0xe1, 0x7a,
0x61, 0x48, 0x6f, 0x20, 0x53, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x2d,
0x76, 0xe1, 0x72, 0x6f, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x7a,
0x6b, 0x4a, 0x65, 0x72, 0x75, 0x7a, 0x73, 0xe1, 0x6c, 0x65, 0x6d, 0x4b,
0x61, 0x6d, 0x63, 0x73, 0x61, 0x74, 0x6b, 0x61, 0x4b, 0x61, 0x72, 0x61,
0x63, 0x73, 0x69, 0x4b, 0x72, 0x61, 0x73, 0x7a, 0x6e, 0x6f, 0x6a, 0x61,
0x72, 0x73, 0x7a, 0x6b, 0x4b, 0x75, 0x63, 0x73, 0x65, 0x6e, 0x67, 0x4d,
0x61, 0x6b, 0x61, 0xf3, 0x4d, 0x61, 0x73, 0x7a, 0x6b, 0x61, 0x74, 0x4e,
0x6f, 0x76, 0x6f, 0x6b, 0x75, 0x7a, 0x6e, 0x79, 0x65, 0x63, 0x6b, 0x4e,
0x6f, 0x76, 0x6f, 0x73, 0x7a, 0x69, 0x62, 0x69, 0x72, 0x73, 0x7a, 0x6b,
0x4f, 0x6d, 0x73, 0x7a, 0x6b, 0x50, 0x68, 0x65, 0x6e, 0x6a, 0x61, 0x6e,
0x4b, 0x6f, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x61, 0x6a, 0x53, 0x7a, 0x61,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x53, 0x7a, 0x61, 0x6d, 0x61, 0x72, 0x6b,
0x61, 0x6e, 0x64, 0x53, 0x7a, 0xf6, 0x75, 0x6c, 0x53, 0x61, 0x6e, 0x67,
0x68, 0x61, 0x6a, 0x53, 0x7a, 0x69, 0x6e, 0x67, 0x61, 0x70, 0xfa, 0x72,
0x53, 0x7a, 0x72, 0x65, 0x64, 0x6e, 0x65, 0x6b, 0x6f, 0x6c, 0x69, 0x6d,
0x73, 0x7a, 0x6b, 0x54, 0x61, 0x73, 0x6b, 0x65, 0x6e, 0x74, 0x54, 0x62,
0x69, 0x6c, 0x69, 0x73, 0x7a, 0x69, 0x54, 0x69, 0x6d, 0x70, 0x75, 0x54,
0x6f, 0x6b, 0x69, 0xf3, 0x54, 0x6f, 0x6d, 0x73, 0x7a, 0x6b, 0x55, 0x6c,
0xe1, 0x6e, 0x62, 0xe1, 0x74, 0x6f, 0x72, 0xdc, 0x72, 0xfc, 0x6d, 0x63,
0x73, 0x69, 0x55, 0x73, 0x7a, 0x74, 0x79, 0x2d, 0x4e, 0x79, 0x65, 0x72,
0x61, 0x56, 0x6c, 0x61, 0x67, 0x79, 0x69, 0x76, 0x6f, 0x73, 0x7a, 0x74,
0x6f, 0x6b, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x7a, 0x6b, 0x4a, 0x65,
0x6b, 0x61, 0x74, 0x79, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x4a, 0x65, 0x72, 0x65, 0x76, 0xe1, 0x6e, 0x41, 0x7a, 0x6f, 0x72, 0x69,
0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x4b, 0x61, 0x6e,
0xe1, 0x72, 0x69, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b,
0x5a, 0xf6, 0x6c, 0x64, 0x2d, 0x66, 0x6f, 0x6b, 0x69, 0x20, 0x73, 0x7a,
0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x46, 0x65, 0x72, 0xf6, 0x65, 0x72,
0x44, 0xe9, 0x6c, 0x69, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61,
0x53, 0x7a, 0x65, 0x6e, 0x74, 0x20, 0x49, 0x6c, 0x6f, 0x6e, 0x61, 0x41,
0x6d, 0x73, 0x7a, 0x74, 0x65, 0x72, 0x64, 0x61, 0x6d, 0x41, 0x73, 0x7a,
0x74, 0x72, 0x61, 0x68, 0xe1, 0x6e, 0x42, 0x65, 0x6c, 0x67, 0x72, 0xe1,
0x64, 0x50, 0x6f, 0x7a, 0x73, 0x6f, 0x6e, 0x79, 0x42, 0x72, 0xfc, 0x73,
0x73, 0x7a, 0x65, 0x6c, 0x4b, 0x6f, 0x70, 0x70, 0x65, 0x6e, 0x68, 0xe1,
0x67, 0x61, 0x47, 0x69, 0x62, 0x72, 0x61, 0x6c, 0x74, 0xe1, 0x72, 0x4d,
0x61, 0x6e, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x49, 0x73, 0x7a,
0x74, 0x61, 0x6e, 0x62, 0x75, 0x6c, 0x4b, 0x61, 0x6c, 0x69, 0x6e, 0x79,
0x69, 0x6e, 0x67, 0x72, 0xe1, 0x64, 0x4c, 0x69, 0x73, 0x73, 0x7a, 0x61,
0x62, 0x6f, 0x6e, 0x4d, 0xe1, 0x6c, 0x74, 0x61, 0x4d, 0x69, 0x6e, 0x73,
0x7a, 0x6b, 0x4d, 0x6f, 0x73, 0x7a, 0x6b, 0x76, 0x61, 0x50, 0xe1, 0x72,
0x69, 0x7a, 0x73, 0x50, 0x72, 0xe1, 0x67, 0x61, 0x52, 0xf3, 0x6d, 0x61,
0x53, 0x7a, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x53, 0x7a, 0x61, 0x72, 0x61,
0x6a, 0x65, 0x76, 0xf3, 0x53, 0x7a, 0x61, 0x72, 0x61, 0x74, 0x6f, 0x76,
0x53, 0x7a, 0x69, 0x6d, 0x66, 0x65, 0x72, 0x6f, 0x70, 0x6f, 0x6c, 0x53,
0x7a, 0x6b, 0x6f, 0x70, 0x6a, 0x65, 0x53, 0x7a, 0xf3, 0x66, 0x69, 0x61,
0x55, 0x6c, 0x6a, 0x61, 0x6e, 0x6f, 0x76, 0x73, 0x7a, 0x6b, 0x42, 0xe9,
0x63, 0x73, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0xe1, 0x64, 0x56,
0x61, 0x72, 0x73, 0xf3, 0x5a, 0xe1, 0x67, 0x72, 0xe1, 0x62, 0x4b, 0x61,
0x72, 0xe1, 0x63, 0x73, 0x6f, 0x6e, 0x79, 0x2d, 0x73, 0x7a, 0x69, 0x67,
0x65, 0x74, 0x4b, 0xf3, 0x6b, 0x75, 0x73, 0x7a, 0x2d, 0x73, 0x7a, 0x69,
0x67, 0x65, 0x74, 0x4b, 0x6f, 0x6d, 0x6f, 0x72, 0xf3, 0x4d, 0x61, 0x6c,
0x64, 0xed, 0x76, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x2d, 0x73, 0x7a, 0x69, 0x67,
0x65, 0x74, 0x65, 0x6b, 0x48, 0xfa, 0x73, 0x76, 0xe9, 0x74, 0x2d, 0x73,
0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x46, 0x69, 0x64, 0x7a, 0x73,
0x69, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x2d, 0x73,
0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x47, 0x61, 0x6d, 0x62, 0x69,
0x65, 0x72, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x4b,
0x69, 0x72, 0x69, 0x74, 0x69, 0x6d, 0x61, 0x74, 0x69, 0x2d, 0x73, 0x7a,
0x69, 0x67, 0x65, 0x74, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x2d, 0x73,
0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x4b, 0x77, 0x61, 0x6a, 0x61,
0x6c, 0x65, 0x69, 0x6e, 0x2d, 0x7a, 0xe1, 0x74, 0x6f, 0x6e, 0x79, 0x4d,
0x61, 0x6a, 0x75, 0x72, 0x6f, 0x2d, 0x7a, 0xe1, 0x74, 0x6f, 0x6e, 0x79,
0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x2d, 0x73, 0x7a,
0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x4d, 0x69, 0x64, 0x77, 0x61, 0x79,
0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x50, 0x69, 0x74,
0x63, 0x61, 0x69, 0x72, 0x6e, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74,
0x65, 0x6b, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x2d, 0x73, 0x7a, 0x69,
0x67, 0x65, 0x74, 0x65, 0x6b, 0x57, 0x61, 0x6b, 0x65, 0x2d, 0x73, 0x7a,
0x69, 0x67, 0x65, 0x74, 0x41, 0x6c, 0x67, 0x65, 0x69, 0x72, 0x73, 0x62,
0x6f, 0x72, 0x67, 0x42, 0x69, 0x73, 0x73, 0xe1, 0x4b, 0x61, 0xed, 0x72,
0xf3, 0x44, 0x6a, 0x69, 0x62, 0xfa, 0x74, 0xed, 0x4a, 0xf3, 0x68, 0x61,
0x6e, 0x6e, 0x65, 0x73, 0x61, 0x72, 0x62, 0x6f, 0x72, 0x67, 0x53, 0x61,
0xf3, 0x20, 0x54, 0xf3, 0x6d, 0x65, 0x54, 0x72, 0xed, 0x70, 0xf3, 0x6c,
0xed, 0x54, 0xfa, 0x6e, 0x69, 0x73, 0x62, 0x6f, 0x72, 0x67, 0x41, 0x6e,
0x67, 0x76, 0x69, 0x6c, 0x6c, 0x61, 0x41, 0x6e, 0x74, 0xed, 0x67, 0x76,
0x61, 0x41, 0x72, 0xfa, 0x62, 0x61, 0x42, 0x65, 0x6c, 0xed, 0x73, 0x4b,
0x61, 0x6e, 0x6b, 0xfa, 0x6e, 0x43, 0x61, 0x79, 0x6d, 0x61, 0x6e, 0x2d,
0x65, 0x79, 0x6a, 0x61, 0x72, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x72, 0xed,
0x6b, 0x61, 0x44, 0xf3, 0x6d, 0x69, 0x6e, 0xed, 0x6b, 0x61, 0x47, 0x76,
0x61, 0x64, 0x65, 0x6c, 0xfa, 0x70, 0x47, 0x76, 0xe6, 0x6a, 0x61, 0x6e,
0x61, 0x4a, 0x61, 0x6d, 0x61, 0xed, 0x6b, 0x61, 0x4d, 0x61, 0x72, 0x74,
0x69, 0x6e, 0xed, 0x6b, 0x4d, 0x65, 0x78, 0xed, 0x6b, 0xf3, 0x62, 0x6f,
0x72, 0x67, 0x50, 0xfa, 0x65, 0x72, 0x74, 0xf3, 0x20, 0x52, 0xed, 0x6b,
0xf3, 0x53, 0x61, 0x6e, 0x6b, 0x74, 0x69, 0x20, 0x42, 0x61, 0x72, 0x74,
0xf3, 0x6c, 0xf3, 0x6d, 0x65, 0x75, 0x73, 0x61, 0x72, 0x65, 0x79, 0x6a,
0x61, 0x72, 0x53, 0x61, 0x6e, 0x6b, 0x74, 0x69, 0x20, 0x4b, 0x69, 0x74,
0x74, 0x73, 0x53, 0x61, 0x6e, 0x6b, 0x74, 0x69, 0x20, 0x4c, 0xfa, 0x73,
0xed, 0x61, 0x53, 0x61, 0x6e, 0x6b, 0x74, 0x69, 0x20, 0x54, 0x68, 0x6f,
0x6d, 0x61, 0x73, 0x53, 0x61, 0x6e, 0x6b, 0x74, 0x69, 0x20, 0x56, 0x69,
0x6e, 0x73, 0x65, 0x6e, 0x74, 0x54, 0x6f, 0x72, 0x74, 0xf3, 0x6c, 0x61,
0x42, 0x65, 0x69, 0x72, 0xfa, 0x74, 0x42, 0x72, 0xfa, 0x6e, 0x65, 0x69,
0x4b, 0xf3, 0x6c, 0x6f, 0x6d, 0x62, 0xf3, 0x48, 0x6f, 0x20, 0x43, 0x68,
0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x2d, 0x62, 0x6f, 0x72, 0x67, 0x4a,
0x65, 0x72, 0xfa, 0x73, 0x61, 0x6c, 0x65, 0x6d, 0x4b, 0x61, 0x62, 0xfa,
0x6c, 0x4b, 0x61, 0x6c, 0x6b, 0xfa, 0x74, 0x74, 0x61, 0x4b, 0xfa, 0x61,
0x6c, 0x61, 0x20, 0x4c, 0xfa, 0x6d, 0x70, 0xfa, 0x72, 0x4b, 0xfa, 0x76,
0x65, 0x69, 0x74, 0x4e, 0xed, 0x6b, 0xf3, 0x73, 0xed, 0x61, 0x52, 0xed,
0x6a, 0x61, 0x64, 0x53, 0x6a, 0x61, 0x6e, 0x67, 0x68, 0xe6, 0x53, 0x69,
0x6e, 0x67, 0x61, 0x70, 0xfa, 0x72, 0x54, 0xf3, 0x6b, 0xfd, 0xf3, 0xda,
0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x79, 0x6a, 0x61, 0x72, 0x42, 0x65, 0x72, 0x6d, 0xfa, 0x64,
0x61, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0xed, 0x65, 0x79, 0x6a, 0x61, 0x72,
0x47, 0x72, 0xe6, 0x6e, 0x68, 0xf6, 0x66, 0xf0, 0x61, 0x65, 0x79, 0x6a,
0x61, 0x72, 0x46, 0xe6, 0x72, 0x65, 0x79, 0x6a, 0x61, 0x72, 0x53, 0x75,
0xf0, 0x75, 0x72, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0xed, 0x61, 0x53,
0x61, 0x6e, 0x6b, 0x74, 0x69, 0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61,
0x41, 0xfe, 0x65, 0x6e, 0x61, 0x42, 0xfa, 0x6b, 0x61, 0x72, 0x65, 0x73,
0x74, 0x42, 0xfa, 0x64, 0x61, 0x70, 0x65, 0x73, 0x74, 0x4b, 0x61, 0x75,
0x70, 0x6d, 0x61, 0x6e, 0x6e, 0x61, 0x68, 0xf6, 0x66, 0x6e, 0x47, 0xed,
0x62, 0x72, 0x61, 0x6c, 0x74, 0x61, 0x72, 0x4d, 0xf6, 0x6e, 0x49, 0x73,
0x74, 0x61, 0x6e, 0x62, 0xfa, 0x6c, 0x4b, 0xe6, 0x6e, 0x75, 0x67, 0x61,
0x72, 0xf0, 0x75, 0x72, 0x4c, 0x75, 0x6e, 0x64, 0xfa, 0x6e, 0x69, 0x72,
0x4c, 0xfa, 0x78, 0x65, 0x6d, 0x62, 0x6f, 0x72, 0x67, 0x4d, 0x61, 0x64,
0x72, 0xed, 0x64, 0x4d, 0x61, 0x72, 0xed, 0x75, 0x68, 0xf6, 0x66, 0x6e,
0x4d, 0xf3, 0x6e, 0x61, 0x6b, 0xf3, 0x4f, 0x73, 0x6c, 0xf3, 0x53, 0x61,
0x6e, 0x20, 0x4d, 0x61, 0x72, 0xed, 0x6e, 0xf3, 0x53, 0x61, 0x72, 0x61,
0x6a, 0x65, 0x76, 0xf3, 0x53, 0xf3, 0x66, 0xed, 0x61, 0x53, 0x74, 0x6f,
0x6b, 0x6b, 0x68, 0xf3, 0x6c, 0x6d, 0x75, 0x72, 0x54, 0xed, 0x72, 0x61,
0x6e, 0x61, 0x56, 0x61, 0x74, 0xed, 0x6b, 0x61, 0x6e, 0x69, 0xf0, 0x56,
0xed, 0x6e, 0x56, 0x69, 0x6c, 0x6e, 0xed, 0x75, 0x73, 0x56, 0x61, 0x72,
0x73, 0x6a, 0xe1, 0x4a, 0xf3, 0x6c, 0x61, 0x65, 0x79, 0x4b, 0xf3, 0x6b,
0x6f, 0x73, 0x65, 0x79, 0x6a, 0x61, 0x72, 0x4d, 0x61, 0x6c, 0x64, 0xed,
0x76, 0x65, 0x79, 0x6a, 0x61, 0x72, 0x4d, 0xe1, 0x72, 0x69, 0x74, 0xed,
0x75, 0x73, 0x50, 0xe1, 0x73, 0x6b, 0x61, 0x65, 0x79, 0x6a, 0x61, 0x46,
0xed, 0x64, 0x6a, 0xed, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61,
0x73, 0x2d, 0x65, 0x79, 0x6a, 0x61, 0x72, 0x4e, 0xe1, 0x72, 0xfa, 0x50,
0x61, 0x6c, 0xe1, 0x54, 0x61, 0x68, 0xed, 0x74, 0xed, 0x41, 0x6c, 0x6a,
0x69, 0x72, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x20, 0x55, 0x74, 0x61, 0x72, 0x61, 0x43, 0x65,
0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x20, 0x55, 0x74, 0x61, 0x72, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61,
0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20,
0x55, 0x74, 0x61, 0x72, 0x61, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75,
0x72, 0x61, 0x54, 0x61, 0x6e, 0x6a, 0x75, 0x6e, 0x67, 0x20, 0x56, 0x65,
0x72, 0x64, 0x65, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x53,
0x65, 0x6c, 0x61, 0x74, 0x61, 0x6e, 0x41, 0x74, 0x68, 0x65, 0x6e, 0x61,
0x4b, 0x69, 0x73, 0x68, 0x69, 0x6e, 0x65, 0x76, 0x50, 0x75, 0x6c, 0x61,
0x75, 0x20, 0x4d, 0x61, 0x6e, 0x4d, 0x6f, 0x73, 0x6b, 0x77, 0x61, 0x57,
0x61, 0x72, 0x73, 0x61, 0x77, 0x61, 0x4d, 0x61, 0x6c, 0x61, 0x64, 0x65,
0x77, 0x61, 0x43, 0x69, 0x74, 0x61, 0x74, 0x65, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f, 0x4e, 0x6f, 0x76, 0x65, 0x20, 0x59,
0x6f, 0x72, 0x6b, 0x53, 0x61, 0x6e, 0x63, 0x74, 0x65, 0x20, 0x42, 0x61,
0x72, 0x74, 0x68, 0x6f, 0x6c, 0x6f, 0x6d, 0x65, 0x6f, 0x53, 0x61, 0x6e,
0x63, 0x74, 0x65, 0x20, 0x4a, 0x6f, 0x68, 0x61, 0x6e, 0x6e, 0x65, 0x73,
0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x6e, 0x6f, 0x76,
0x61, 0x53, 0x61, 0x6e, 0x63, 0x74, 0x65, 0x20, 0x43, 0x68, 0x72, 0x69,
0x73, 0x74, 0x6f, 0x70, 0x68, 0x6f, 0x72, 0x6f, 0x53, 0x61, 0x6e, 0x63,
0x74, 0x65, 0x20, 0x4c, 0x75, 0x63, 0x69, 0x61, 0x53, 0x61, 0x6e, 0x63,
0x74, 0x65, 0x20, 0x54, 0x68, 0x6f, 0x6d, 0x61, 0x73, 0x53, 0x61, 0x6e,
0x63, 0x74, 0x65, 0x20, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x74, 0x65,
0x43, 0x61, 0x70, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x41, 0x74,
0x68, 0x65, 0x6e, 0x61, 0x73, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x61, 0x6e, 0x4c, 0x69, 0x73, 0x62, 0x6f, 0x6e,
0x61, 0x4e, 0x61, 0x74, 0x61, 0x6c, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74,
0x69, 0x6f, 0x4d, 0x61, 0x79, 0x6f, 0x74, 0x74, 0x61, 0x50, 0x61, 0x73,
0x63, 0x68, 0x61, 0x44, 0x61, 0x72, 0x2d, 0x65, 0x73, 0x2d, 0x53, 0x61,
0x6c, 0x61, 0x61, 0x6d, 0x45, 0x6c, 0x2d, 0x41, 0x61, 0x69, 0x75, 0x6e,
0x53, 0xe3, 0x6f, 0x2d, 0x54, 0x6f, 0x6d, 0xe9, 0x42, 0x75, 0x65, 0x6e,
0x6f, 0x73, 0x2d, 0x41, 0x69, 0x72, 0x65, 0x73, 0x4c, 0x61, 0x2d, 0x52,
0x69, 0x6f, 0x6a, 0x61, 0x52, 0x69, 0x6f, 0x2d, 0x47, 0x61, 0x6c, 0x6c,
0x65, 0x67, 0x6f, 0x73, 0x53, 0x61, 0x6e, 0x74, 0x2d, 0x4a, 0x75, 0x61,
0x6e, 0x53, 0x61, 0x6e, 0x74, 0x2d, 0x4c, 0x75, 0x69, 0x73, 0x62, 0x61,
0x79, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x61,
0x73, 0x43, 0x61, 0x6d, 0x62, 0x72, 0x69, 0x64, 0x67, 0x65, 0x2d, 0x42,
0x61, 0x79, 0x43, 0x61, 0x6d, 0x70, 0x6f, 0x2d, 0x47, 0x72, 0x61, 0x6e,
0x64, 0x65, 0x43, 0x69, 0x75, 0x64, 0x61, 0x64, 0x2d, 0x4a, 0x75, 0x61,
0x72, 0x65, 0x7a, 0x43, 0x6f, 0x73, 0x74, 0x61, 0x2d, 0x52, 0x69, 0x63,
0x61, 0x44, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x2d, 0x43, 0x72, 0x65, 0x65,
0x6b, 0x45, 0x6c, 0x2d, 0x53, 0x61, 0x6c, 0x76, 0x61, 0x64, 0x6f, 0x72,
0x47, 0x6c, 0x61, 0x63, 0x65, 0x2d, 0x42, 0x61, 0x79, 0x47, 0x6f, 0x6f,
0x73, 0x65, 0x2d, 0x42, 0x61, 0x79, 0x47, 0x72, 0x61, 0x6e, 0x64, 0x2d,
0x54, 0x75, 0x72, 0x6b, 0x47, 0x75, 0x61, 0x64, 0x65, 0x6c, 0x75, 0x70,
0x61, 0x54, 0x65, 0x6c, 0x6c, 0x2d, 0x43, 0x69, 0x74, 0x79, 0x2c, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x4c, 0x61, 0x2d, 0x50, 0x61,
0x7a, 0x4c, 0x6f, 0x73, 0x2d, 0x41, 0x6e, 0x67, 0x65, 0x6c, 0x65, 0x73,
0x4c, 0x6f, 0x77, 0x65, 0x72, 0x2d, 0x50, 0x72, 0x69, 0x6e, 0x63, 0x65,
0x2019, 0x73, 0x2d, 0x51, 0x75, 0x61, 0x72, 0x74, 0x65, 0x72, 0x4d, 0x65,
0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x43, 0x69, 0x74, 0xe9, 0x42, 0x65,
0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x2d, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c,
0x20, 0x4e, 0x6f, 0x72, 0x64, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x4e,
0x6f, 0x72, 0x64, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x50, 0x6f,
0x72, 0x74, 0x6f, 0x2d, 0x52, 0x69, 0x63, 0x6f, 0x52, 0x61, 0x6e, 0x6b,
0x69, 0x6e, 0x2d, 0x49, 0x6e, 0x6c, 0x65, 0x74, 0x52, 0x69, 0x6f, 0x2d,
0x42, 0x72, 0x61, 0x6e, 0x63, 0x6f, 0x53, 0x61, 0x6e, 0x2d, 0x44, 0x6f,
0x6d, 0x69, 0x6e, 0x67, 0x6f, 0x53, 0xe3, 0x6f, 0x2d, 0x50, 0x61, 0x75,
0x6c, 0x6f, 0x53, 0x61, 0x6e, 0x74, 0x2d, 0x42, 0x61, 0x72, 0x74, 0x6f,
0x6c, 0x6f, 0x6d, 0x65, 0x6f, 0x53, 0x74, 0x2e, 0x2d, 0x4a, 0x6f, 0x68,
0x6e, 0x2019, 0x73, 0x53, 0x74, 0x2e, 0x2d, 0x4b, 0x69, 0x74, 0x74, 0x73,
0x53, 0x74, 0x2e, 0x2d, 0x4c, 0x75, 0x63, 0x69, 0x61, 0x53, 0x74, 0x2e,
0x2d, 0x54, 0x68, 0x6f, 0x6d, 0x61, 0x73, 0x53, 0x74, 0x2e, 0x2d, 0x56,
0x69, 0x6e, 0x63, 0x65, 0x6e, 0x74, 0x53, 0x77, 0x69, 0x66, 0x74, 0x2d,
0x43, 0x75, 0x72, 0x72, 0x65, 0x6e, 0x74, 0x50, 0x68, 0x6e, 0x6f, 0x6d,
0x2d, 0x50, 0x65, 0x6e, 0x68, 0x43, 0x61, 0x70, 0x2d, 0x56, 0x65, 0x72,
0x64, 0x69, 0x53, 0x75, 0x64, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x53, 0x61, 0x6e, 0x74, 0x2d, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61,
0x42, 0x72, 0x6f, 0x6b, 0x65, 0x6e, 0x2d, 0x48, 0x69, 0x6c, 0x6c, 0x49,
0x6e, 0x73, 0x75, 0x6c, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48, 0x6f,
0x77, 0x65, 0x42, 0x75, 0x63, 0x75, 0x72, 0x65, 0x73, 0x74, 0x69, 0x49,
0x6e, 0x73, 0x75, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x6e, 0x4d,
0x6f, 0x73, 0x63, 0x76, 0x61, 0x50, 0x61, 0x73, 0x63, 0x61, 0x50, 0x6f,
0x72, 0x74, 0x2d, 0x4d, 0x6f, 0x72, 0x65, 0x73, 0x62, 0x79, 0x43, 0x61,
0x74, 0x68, 0x61, 0x69, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x69,
0x6c, 0x67, 0xe9, 0x69, 0x72, 0x65, 0x43, 0x61, 0x69, 0x72, 0x65, 0x6f,
0x43, 0x6f, 0x6e, 0x61, 0x63, 0x72, 0x61, 0xed, 0x44, 0x61, 0x63, 0xe1,
0x72, 0x44, 0xe1, 0x72, 0x61, 0x73, 0x61, 0x6c, 0xe1, 0x6d, 0x4c, 0xe1,
0xfa, 0x69, 0x6e, 0x65, 0x43, 0x61, 0x72, 0x74, 0xfa, 0x6d, 0x4d, 0x61,
0x70, 0xfa, 0x74, 0xf3, 0x4d, 0x6f, 0x67, 0x61, 0x69, 0x64, 0x69, 0x73,
0x69, 0xfa, 0x4e, 0x75, 0x61, 0x63, 0x73, 0x61, 0x74, 0x54, 0x72, 0x69,
0x70, 0x69, 0x6c, 0xed, 0x54, 0xfa, 0x69, 0x6e, 0x69, 0x73, 0x41, 0x6e,
0x67, 0x61, 0xed, 0x6c, 0x65, 0x42, 0x61, 0x72, 0x62, 0x61, 0x64, 0xf3,
0x73, 0x61, 0x6e, 0x20, 0x42, 0x68, 0x65, 0x69, 0x6c, 0xed, 0x73, 0x42,
0x6f, 0x67, 0x61, 0x74, 0xe1, 0x43, 0xf3, 0x73, 0x74, 0x61, 0x20, 0x52,
0xed, 0x63, 0x65, 0x43, 0xfa, 0x72, 0x61, 0x73, 0xf3, 0x61, 0x6e, 0x20,
0x74, 0x53, 0x61, 0x6c, 0x76, 0x61, 0x64, 0xf3, 0x69, 0x72, 0x47, 0x72,
0x65, 0x61, 0x6e, 0xe1, 0x64, 0x61, 0x47, 0x75, 0x61, 0x64, 0x61, 0x6c,
0xfa, 0x69, 0x70, 0x61, 0x6e, 0x20, 0x47, 0x68, 0x75, 0xe1, 0x69, 0x6e,
0x49, 0x61, 0x6d, 0xe1, 0x69, 0x63, 0x65, 0x4c, 0xed, 0x6f, 0x6d, 0x61,
0x43, 0x61, 0x74, 0x68, 0x61, 0x69, 0x72, 0x20, 0x4d, 0x68, 0x65, 0x69,
0x63, 0x73, 0x69, 0x63, 0x65, 0x6f, 0x4e, 0x75, 0x61, 0x2d, 0x45, 0x61,
0x62, 0x68, 0x72, 0x61, 0x63, 0x50, 0xf3, 0x72, 0x74, 0xf3, 0x20, 0x52,
0xed, 0x63, 0x65, 0x53, 0x61, 0x6e, 0x20, 0x43, 0x72, 0xed, 0x6f, 0x73,
0x74, 0xf3, 0x69, 0x72, 0x53, 0x61, 0x6e, 0x20, 0x55, 0x69, 0x6e, 0x73,
0x65, 0x61, 0x6e, 0x6e, 0x49, 0x6e, 0x69, 0x73, 0x20, 0x54, 0x75, 0x69,
0x6c, 0x65, 0x4d, 0x61, 0x63, 0x20, 0x47, 0x75, 0x61, 0x69, 0x72, 0x65,
0xc1, 0x69, 0x64, 0x69, 0x6e, 0x42, 0x61, 0x69, 0x72, 0xe9, 0x69, 0x6e,
0x42, 0x61, 0x6b, 0x69, 0x42, 0xe9, 0x69, 0x72, 0x69, 0xfa, 0x74, 0x42,
0x72, 0xfa, 0x69, 0x6e, 0xe9, 0x61, 0x6e, 0x20, 0x44, 0x61, 0x6d, 0x61,
0x69, 0x73, 0x63, 0x48, 0x65, 0x61, 0x62, 0x72, 0xf3, 0x6e, 0x48, 0x6f,
0x6e, 0x67, 0x20, 0x43, 0x6f, 0x6e, 0x67, 0x49, 0x61, 0x63, 0xe1, 0x72,
0x74, 0x61, 0x49, 0x61, 0x72, 0xfa, 0x73, 0x61, 0x69, 0x6c, 0xe9, 0x69,
0x6d, 0x43, 0x61, 0x62, 0xfa, 0x6c, 0x43, 0x61, 0x6c, 0x63, 0xfa, 0x74,
0x61, 0x43, 0x75, 0xe1, 0x69, 0x74, 0x4d, 0x61, 0x63, 0x61, 0x73, 0x61,
0x72, 0x4d, 0x61, 0x69, 0x6e, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x20, 0x4e,
0x69, 0x6f, 0x63, 0xf3, 0x69, 0x73, 0x53, 0xfa, 0x6c, 0x53, 0x68, 0x61,
0x6e, 0x67, 0x2d, 0x68, 0x61, 0x69, 0x53, 0x69, 0x6e, 0x67, 0x65, 0x61,
0x70, 0xf3, 0x72, 0x54, 0xf3, 0x69, 0x63, 0x65, 0x6f, 0x49, 0x61, 0x63,
0xfa, 0x74, 0x73, 0x63, 0x45, 0x69, 0x72, 0x65, 0x61, 0x76, 0xe1, 0x6e,
0x6e, 0x61, 0x20, 0x68, 0x41, 0x73, 0xf3, 0x69, 0x72, 0x42, 0x65, 0x69,
0x72, 0x6d, 0x69, 0xfa, 0x64, 0x61, 0x6e, 0x61, 0x20, 0x68, 0x4f, 0x69,
0x6c, 0x65, 0xe1, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x61, 0x6e, 0xe1, 0x72,
0x61, 0x63, 0x68, 0x61, 0x52, 0x69, 0x6e, 0x6e, 0x20, 0x56, 0x65, 0x72,
0x64, 0x65, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e, 0x20, 0x46, 0x68,
0x61, 0x72, 0xf3, 0x4d, 0x61, 0x69, 0x64, 0xe9, 0x61, 0x72, 0x61, 0x52,
0xe9, 0x69, 0x63, 0x69, 0x76, 0xed, 0x63, 0x61, 0x6e, 0x20, 0x74, 0x53,
0x65, 0x6f, 0x69, 0x72, 0x73, 0x69, 0x61, 0x20, 0x54, 0x68, 0x65, 0x61,
0x73, 0x53, 0x61, 0x6e, 0x20, 0x48, 0xe9, 0x69, 0x6c, 0x69, 0x6e, 0x41,
0x6d, 0x73, 0x74, 0x61, 0x72, 0x64, 0x61, 0x6d, 0x41, 0x6e, 0x64, 0xf3,
0x72, 0x61, 0x61, 0x6e, 0x20, 0x41, 0x73, 0x74, 0x72, 0x61, 0x63, 0xe1,
0x69, 0x6e, 0x61, 0x6e, 0x20, 0x41, 0x69, 0x74, 0x68, 0x69, 0x6e, 0x42,
0xe9, 0x61, 0x6c, 0x67, 0x72, 0xe1, 0x64, 0x42, 0x65, 0x69, 0x72, 0x6c,
0xed, 0x6e, 0x61, 0x6e, 0x20, 0x42, 0x68, 0x72, 0x61, 0x74, 0x61, 0x73,
0x6c, 0xe1, 0x69, 0x76, 0x61, 0x6e, 0x20, 0x42, 0x68, 0x72, 0x75, 0x69,
0x73, 0xe9, 0x69, 0x6c, 0x42, 0xfa, 0x63, 0x61, 0x69, 0x72, 0x69, 0x73,
0x74, 0x42, 0xfa, 0x64, 0x61, 0x69, 0x70, 0x65, 0x69, 0x73, 0x74, 0x43,
0xed, 0x73, 0x69, 0x6e, 0x65, 0xe1, 0x43, 0xf3, 0x62, 0x61, 0x6e, 0x68,
0xe1, 0x76, 0x61, 0x6e, 0x42, 0x61, 0x69, 0x6c, 0x65, 0x20, 0xc1, 0x74,
0x68, 0x61, 0x20, 0x43, 0x6c, 0x69, 0x61, 0x74, 0x68, 0x47, 0x69, 0x6f,
0x62, 0x72, 0xe1, 0x6c, 0x74, 0x61, 0x72, 0x47, 0x65, 0x61, 0x6e, 0x73,
0x61, 0xed, 0x48, 0x65, 0x69, 0x6c, 0x73, 0x69, 0x6e, 0x63, 0xed, 0x4f,
0x69, 0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x4d, 0x68, 0x61, 0x6e, 0x61, 0x6e,
0x6e, 0x49, 0x6f, 0x73, 0x74, 0x61, 0x6e, 0x62, 0xfa, 0x6c, 0x47, 0x65,
0x69, 0x72, 0x73, 0xed, 0x43, 0xed, 0x76, 0x4c, 0x69, 0x6f, 0x73, 0x70,
0xf3, 0x69, 0x6e, 0x4c, 0x69, 0xfa, 0x69, 0x62, 0x6c, 0x65, 0xe1, 0x6e,
0x61, 0x4c, 0x6f, 0x6e, 0x64, 0x61, 0x69, 0x6e, 0x4c, 0x75, 0x63, 0x73,
0x61, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x4d, 0x61, 0x69, 0x64, 0x72, 0x69,
0x64, 0x4d, 0x69, 0x6f, 0x6e, 0x73, 0x63, 0x4d, 0x6f, 0x6e, 0x61, 0x63,
0xf3, 0x4d, 0x6f, 0x73, 0x63, 0xf3, 0x50, 0xe1, 0x72, 0x61, 0x73, 0x50,
0x6f, 0x64, 0x67, 0x61, 0x69, 0x72, 0xed, 0x74, 0x73, 0x65, 0x52, 0xed,
0x67, 0x65, 0x61, 0x6e, 0x20, 0x52, 0xf3, 0x69, 0x6d, 0x68, 0x53, 0x61,
0x6e, 0x20, 0x4d, 0x61, 0x69, 0x72, 0xed, 0x6e, 0x65, 0x53, 0x61, 0x69,
0x72, 0xe9, 0x61, 0x76, 0xf3, 0x53, 0x63, 0xf3, 0x69, 0x70, 0xe9, 0x53,
0xf3, 0x69, 0x66, 0x69, 0x61, 0x53, 0x74, 0xf3, 0x63, 0xf3, 0x6c, 0x6d,
0x54, 0x61, 0x69, 0x6c, 0x6c, 0x69, 0x6e, 0x6e, 0x54, 0x69, 0x6f, 0x72,
0xe1, 0x6e, 0x61, 0x56, 0x61, 0x64, 0xfa, 0x73, 0x61, 0x6e, 0x20, 0x56,
0x61, 0x74, 0x61, 0x63, 0xe1, 0x69, 0x6e, 0x56, 0x69, 0x6c, 0x6e, 0x69,
0x61, 0x73, 0x56, 0xe1, 0x72, 0x73, 0xe1, 0x53, 0xe1, 0x67, 0x72, 0x61,
0x62, 0x41, 0x6e, 0x74, 0x61, 0x6e, 0x61, 0x6e, 0x61, 0x69, 0x72, 0xed,
0x76, 0x65, 0x6f, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x6e, 0x61,
0x20, 0x4e, 0x6f, 0x6c, 0x6c, 0x61, 0x67, 0x4f, 0x69, 0x6c, 0x65, 0xe1,
0x6e, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x4f, 0x69, 0x6c, 0x65, 0xe1,
0x69, 0x6e, 0x20, 0x43, 0x68, 0x6f, 0x6d, 0xf3, 0x72, 0x61, 0x4f, 0x69,
0x6c, 0x65, 0xe1, 0x69, 0x6e, 0x20, 0x4d, 0x68, 0x61, 0x69, 0x6c, 0x64,
0xed, 0x76, 0x65, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x4d, 0x68,
0x75, 0x69, 0x72, 0xed, 0x73, 0x46, 0x69, 0x64, 0x73, 0xed, 0x6e, 0x61,
0x20, 0x68, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e, 0x20, 0x4d, 0x68,
0x61, 0x72, 0x63, 0x61, 0x73, 0x61, 0x63, 0x68, 0x61, 0x4f, 0x69, 0x6c,
0x65, 0xe1, 0x69, 0x6e, 0x20, 0x4d, 0x69, 0x64, 0x77, 0x61, 0x79, 0x54,
0x61, 0x69, 0x68, 0xed, 0x74, 0xed, 0x56, 0x61, 0x69, 0x6c, 0xed, 0x73,
0x41, 0x6c, 0x67, 0x65, 0x72, 0x69, 0x49, 0x6c, 0x20, 0x43, 0x61, 0x69,
0x72, 0x6f, 0x47, 0x69, 0x62, 0x75, 0x74, 0x69, 0x45, 0x6c, 0x20, 0x41,
0x79, 0x75, 0x6e, 0x47, 0x69, 0x75, 0x62, 0x61, 0x54, 0x75, 0x6e, 0x69,
0x73, 0x69, 0x43, 0x61, 0x69, 0x65, 0x6e, 0x6e, 0x61, 0x47, 0x75, 0x61,
0x64, 0x61, 0x6c, 0x75, 0x70, 0x61, 0x4c, 0x2019, 0x41, 0x76, 0x61, 0x6e,
0x61, 0x47, 0x69, 0x61, 0x6d, 0x61, 0x69, 0x63, 0x61, 0x43, 0x69, 0x74,
0x74, 0xe0, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4d, 0x65, 0x73, 0x73, 0x69,
0x63, 0x6f, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6e, 0x6f, 0x72,
0x64, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x64,
0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6e, 0x6f,
0x72, 0x64, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x72, 0x69, 0x63, 0x6f, 0x53,
0x61, 0x6e, 0x20, 0x50, 0x61, 0x6f, 0x6c, 0x6f, 0x53, 0x61, 0x6e, 0x74,
0x61, 0x20, 0x4c, 0x75, 0x63, 0x69, 0x61, 0x41, 0x6e, 0x61, 0x64, 0x79,
0x72, 0x2019, 0x46, 0x61, 0x6d, 0x61, 0x67, 0x6f, 0x73, 0x74, 0x61, 0x47,
0x69, 0x61, 0x63, 0x61, 0x72, 0x74, 0x61, 0x47, 0x65, 0x72, 0x75, 0x73,
0x61, 0x6c, 0x65, 0x6d, 0x6d, 0x65, 0x41, 0x7a, 0x7a, 0x6f, 0x72, 0x72,
0x65, 0x49, 0x73, 0x6f, 0x6c, 0x65, 0x20, 0x46, 0xe6, 0x72, 0x20, 0xd8,
0x65, 0x72, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x53, 0x75, 0x64, 0x53, 0x61, 0x6e, 0x74, 0x2019, 0x45, 0x6c,
0x65, 0x6e, 0x61, 0x43, 0x6f, 0x70, 0x65, 0x6e, 0x61, 0x67, 0x68, 0x65,
0x6e, 0x47, 0x69, 0x62, 0x69, 0x6c, 0x74, 0x65, 0x72, 0x72, 0x61, 0x49,
0x73, 0x6f, 0x6c, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x61, 0x6e, 0x4c,
0x75, 0x62, 0x69, 0x61, 0x6e, 0x61, 0x4c, 0x6f, 0x6e, 0x64, 0x72, 0x61,
0x4c, 0x75, 0x73, 0x73, 0x65, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x4d,
0x6f, 0x73, 0x63, 0x61, 0x50, 0x61, 0x72, 0x69, 0x67, 0x69, 0x53, 0x69,
0x6e, 0x66, 0x65, 0x72, 0x6f, 0x70, 0x6f, 0x6c, 0x69, 0x53, 0x74, 0x6f,
0x63, 0x63, 0x6f, 0x6c, 0x6d, 0x61, 0x43, 0x69, 0x74, 0x74, 0xe0, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x56, 0x61, 0x74, 0x69, 0x63, 0x61, 0x6e, 0x6f,
0x56, 0x61, 0x72, 0x73, 0x61, 0x76, 0x69, 0x61, 0x5a, 0x61, 0x67, 0x61,
0x62, 0x72, 0x69, 0x61, 0x5a, 0x75, 0x72, 0x69, 0x67, 0x6f, 0x4c, 0x61,
0x20, 0x52, 0x69, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x65, 0x46, 0x69, 0x67,
0x69, 0x4d, 0x61, 0x72, 0x63, 0x68, 0x65, 0x73, 0x69, 0x30a2, 0x30d3, 0x30b8,
0x30e3, 0x30f3, 0x30a2, 0x30af, 0x30e9, 0x30a2, 0x30b8, 0x30b9, 0x30a2, 0x30d9, 0x30d0, 0x30a2,
0x30eb, 0x30b8, 0x30a7, 0x30a2, 0x30b9, 0x30de, 0x30e9, 0x30d0, 0x30de, 0x30b3, 0x30d0, 0x30f3,
0x30ae, 0x30d0, 0x30f3, 0x30b8, 0x30e5, 0x30fc, 0x30eb, 0x30d3, 0x30b5, 0x30a6, 0x30d6, 0x30e9,
0x30f3, 0x30bf, 0x30a4, 0x30e4, 0x30d6, 0x30e9, 0x30b6, 0x30d3, 0x30eb, 0x30d6, 0x30b8, 0x30e5,
0x30f3, 0x30d6, 0x30e9, 0x30ab, 0x30a4, 0x30ed, 0x30ab, 0x30b5, 0x30d6, 0x30e9, 0x30f3, 0x30ab,
0x30bb, 0x30a6, 0x30bf, 0x30b3, 0x30ca, 0x30af, 0x30ea, 0x30c0, 0x30ab, 0x30fc, 0x30eb, 0x30c0,
0x30eb, 0x30a8, 0x30b9, 0x30b5, 0x30e9, 0x30fc, 0x30e0, 0x30b8, 0x30d6, 0x30c1, 0x30c9, 0x30a5,
0x30a2, 0x30e9, 0x30a2, 0x30a4, 0x30a6, 0x30f3, 0x30d5, 0x30ea, 0x30fc, 0x30bf, 0x30a6, 0x30f3,
0x30cf, 0x30dc, 0x30ed, 0x30fc, 0x30cd, 0x30cf, 0x30e9, 0x30ec, 0x30e8, 0x30cf, 0x30cd, 0x30b9,
0x30d6, 0x30eb, 0x30b0, 0x30b8, 0x30e5, 0x30d0, 0x30ab, 0x30f3, 0x30d1, 0x30e9, 0x30cf, 0x30eb,
0x30c4, 0x30fc, 0x30e0, 0x30ad, 0x30ac, 0x30ea, 0x30ad, 0x30f3, 0x30b7, 0x30e3, 0x30b5, 0x30e9,
0x30b4, 0x30b9, 0x30ea, 0x30fc, 0x30d6, 0x30eb, 0x30f4, 0x30a3, 0x30eb, 0x30ed, 0x30e1, 0x30eb,
0x30a2, 0x30f3, 0x30c0, 0x30eb, 0x30d6, 0x30f3, 0x30d0, 0x30b7, 0x30eb, 0x30b5, 0x30ab, 0x30de,
0x30e9, 0x30dc, 0x30de, 0x30d7, 0x30c8, 0x30de, 0x30bb, 0x30eb, 0x30e0, 0x30d0, 0x30d0, 0x30fc,
0x30cd, 0x30e2, 0x30ac, 0x30c7, 0x30a3, 0x30b7, 0x30aa, 0x30e2, 0x30f3, 0x30ed, 0x30d3, 0x30a2,
0x30ca, 0x30a4, 0x30ed, 0x30d3, 0x30f3, 0x30b8, 0x30e3, 0x30e1, 0x30ca, 0x30cb, 0x30a2, 0x30e1,
0x30cc, 0x30a2, 0x30af, 0x30b7, 0x30e7, 0x30c3, 0x30c8, 0x30ef, 0x30ac, 0x30c9, 0x30a5, 0x30b0,
0x30fc, 0x30dd, 0x30eb, 0x30c8, 0x30ce, 0x30dc, 0x30b5, 0x30f3, 0x30c8, 0x30e1, 0x30c8, 0x30ea,
0x30dd, 0x30ea, 0x30c1, 0x30e5, 0x30cb, 0x30b9, 0x30a6, 0x30a3, 0x30f3, 0x30c8, 0x30d5, 0x30c3,
0x30af, 0x30a2, 0x30c0, 0x30c3, 0x30af, 0x30a2, 0x30f3, 0x30ab, 0x30ec, 0x30c3, 0x30b8, 0x30a2,
0x30f3, 0x30ae, 0x30e9, 0x30a2, 0x30f3, 0x30c6, 0x30a3, 0x30b0, 0x30a2, 0x30a2, 0x30e9, 0x30b0,
0x30a1, 0x30a4, 0x30ca, 0x30d6, 0x30a8, 0x30ce, 0x30b9, 0x30a2, 0x30a4, 0x30ec, 0x30b9, 0x30ab,
0x30bf, 0x30de, 0x30eb, 0x30ab, 0x30b3, 0x30eb, 0x30c9, 0x30d0, 0x30d5, 0x30d5, 0x30a4, 0x30e9,
0x30ea, 0x30aa, 0x30cf, 0x30e1, 0x30f3, 0x30c9, 0x30fc, 0x30b5, 0x30ea, 0x30aa, 0x30ac, 0x30b8,
0x30a7, 0x30b4, 0x30b9, 0x30b5, 0x30eb, 0x30bf, 0x30b5, 0x30f3, 0x30d5, 0x30a1, 0x30f3, 0x30b5,
0x30f3, 0x30eb, 0x30a4, 0x30b9, 0x30c8, 0x30a5, 0x30af, 0x30de, 0x30f3, 0x30a6, 0x30b7, 0x30e5,
0x30a2, 0x30a4, 0x30a2, 0x30a2, 0x30eb, 0x30d0, 0x30a2, 0x30b9, 0x30f3, 0x30b7, 0x30aa, 0x30f3,
0x30a2, 0x30c6, 0x30a3, 0x30b3, 0x30ab, 0x30f3, 0x30d0, 0x30a4, 0x30fc, 0x30a2, 0x30d0, 0x30a4,
0x30a2, 0x30d0, 0x30f3, 0x30c7, 0x30e9, 0x30d0, 0x30eb, 0x30d0, 0x30c9, 0x30b9, 0x30d9, 0x30ec,
0x30f3, 0x30d9, 0x30ea, 0x30fc, 0x30ba, 0x30d6, 0x30e9, 0x30f3, 0x30fb, 0x30b5, 0x30d6, 0x30ed,
0x30f3, 0x30dc, 0x30a2, 0x30d3, 0x30b9, 0x30bf, 0x30dc, 0x30b4, 0x30bf, 0x30dc, 0x30a4, 0x30b7,
0x30b1, 0x30f3, 0x30d6, 0x30ea, 0x30c3, 0x30b8, 0x30d9, 0x30a4, 0x30ab, 0x30f3, 0x30dd, 0x30b0,
0x30e9, 0x30f3, 0x30c7, 0x30ab, 0x30f3, 0x30af, 0x30f3, 0x30ab, 0x30e9, 0x30ab, 0x30b9, 0x30ab,
0x30a4, 0x30a8, 0x30f3, 0x30cc, 0x30b1, 0x30a4, 0x30de, 0x30f3, 0x30b7, 0x30ab, 0x30b4, 0x30c1,
0x30ef, 0x30ef, 0x30b7, 0x30a6, 0x30c0, 0x30fc, 0x30fb, 0x30d5, 0x30a2, 0x30ec, 0x30b9, 0x30b3,
0x30b9, 0x30bf, 0x30ea, 0x30ab, 0x30af, 0x30ec, 0x30b9, 0x30c8, 0x30f3, 0x30af, 0x30a4, 0x30a2,
0x30d0, 0x30ad, 0x30e5, 0x30e9, 0x30bd, 0x30fc, 0x30c7, 0x30f3, 0x30de, 0x30fc, 0x30af, 0x30b7,
0x30e3, 0x30a6, 0x30f3, 0x30c9, 0x30fc, 0x30bd, 0x30f3, 0x30c9, 0x30fc, 0x30bd, 0x30f3, 0x30af,
0x30ea, 0x30fc, 0x30af, 0x30c7, 0x30f3, 0x30d0, 0x30fc, 0x30c7, 0x30c8, 0x30ed, 0x30a4, 0x30c8,
0x30c9, 0x30df, 0x30cb, 0x30ab, 0x30a8, 0x30c9, 0x30e2, 0x30f3, 0x30c8, 0x30f3, 0x30a8, 0x30a4,
0x30eb, 0x30cd, 0x30da, 0x30a8, 0x30eb, 0x30b5, 0x30eb, 0x30d0, 0x30c9, 0x30eb, 0x30d5, 0x30a9,
0x30fc, 0x30c8, 0x30cd, 0x30eb, 0x30bd, 0x30f3, 0x30d5, 0x30a9, 0x30eb, 0x30bf, 0x30ec, 0x30b6,
0x30b0, 0x30ec, 0x30fc, 0x30b9, 0x30d9, 0x30a4, 0x30b0, 0x30fc, 0x30b9, 0x30d9, 0x30a4, 0x30b0,
0x30e9, 0x30f3, 0x30c9, 0x30bf, 0x30fc, 0x30af, 0x30b0, 0x30ec, 0x30ca, 0x30c0, 0x30b0, 0x30a2,
0x30c9, 0x30eb, 0x30fc, 0x30d7, 0x30b0, 0x30a2, 0x30c6, 0x30de, 0x30e9, 0x30b0, 0x30a2, 0x30e4,
0x30ad, 0x30eb, 0x30ac, 0x30a4, 0x30a2, 0x30ca, 0x30cf, 0x30ea, 0x30d5, 0x30a1, 0x30af, 0x30b9,
0x30cf, 0x30d0, 0x30ca, 0x30a8, 0x30eb, 0x30e2, 0x30b7, 0x30e8, 0x30a4, 0x30f3, 0x30c7, 0x30a3,
0x30a2, 0x30ca, 0x30dd, 0x30ea, 0x30b9, 0x30a4, 0x30f3, 0x30c7, 0x30a3, 0x30a2, 0x30ca, 0x5dde,
0x30ce, 0x30c3, 0x30af, 0x30b9, 0x30a4, 0x30f3, 0x30c7, 0x30a3, 0x30a2, 0x30ca, 0x5dde, 0x30de,
0x30ec, 0x30f3, 0x30b4, 0x30a4, 0x30f3, 0x30c7, 0x30a3, 0x30a2, 0x30ca, 0x5dde, 0x30d4, 0x30fc,
0x30bf, 0x30fc, 0x30ba, 0x30d0, 0x30fc, 0x30b0, 0x30a4, 0x30f3, 0x30c7, 0x30a3, 0x30a2, 0x30ca,
0x5dde, 0x30c6, 0x30eb, 0x30b7, 0x30c6, 0x30a3, 0x30a4, 0x30f3, 0x30c7, 0x30a3, 0x30a2, 0x30ca,
0x5dde, 0x30d3, 0x30d9, 0x30fc, 0x30a4, 0x30f3, 0x30c7, 0x30a3, 0x30a2, 0x30ca, 0x5dde, 0x30d3,
0x30f3, 0x30bb, 0x30f3, 0x30b9, 0x30a4, 0x30f3, 0x30c7, 0x30a3, 0x30a2, 0x30ca, 0x5dde, 0x30a6,
0x30a3, 0x30ca, 0x30de, 0x30c3, 0x30af, 0x30a4, 0x30cc, 0x30f4, 0x30a3, 0x30af, 0x30a4, 0x30ab,
0x30eb, 0x30a4, 0x30c3, 0x30c8, 0x30b8, 0x30e3, 0x30de, 0x30a4, 0x30ab, 0x30b8, 0x30e5, 0x30ce,
0x30fc, 0x30eb, 0x30a4, 0x30d3, 0x30eb, 0x30b1, 0x30f3, 0x30bf, 0x30c3, 0x30ad, 0x30fc, 0x5dde,
0x30e2, 0x30f3, 0x30c6, 0x30a3, 0x30c1, 0x30a7, 0x30ed, 0x30af, 0x30e9, 0x30ec, 0x30f3, 0x30c0,
0x30a4, 0x30af, 0x30e9, 0x30d1, 0x30b9, 0x30ea, 0x30de, 0x30ed, 0x30b5, 0x30f3, 0x30bc, 0x30eb,
0x30b9, 0x30ed, 0x30fc, 0x30ef, 0x30fc, 0x30fb, 0x30d7, 0x30ea, 0x30f3, 0x30bb, 0x30ba, 0x30fb,
0x30af, 0x30a6, 0x30a9, 0x30fc, 0x30bf, 0x30fc, 0x30de, 0x30bb, 0x30a4, 0x30aa, 0x30de, 0x30ca,
0x30b0, 0x30a2, 0x30de, 0x30ca, 0x30a6, 0x30b9, 0x30de, 0x30ea, 0x30b4, 0x30de, 0x30eb, 0x30c6,
0x30a3, 0x30cb, 0x30fc, 0x30af, 0x30de, 0x30bf, 0x30e2, 0x30ed, 0x30b9, 0x30de, 0x30b5, 0x30c8,
0x30e9, 0x30f3, 0x30e1, 0x30ce, 0x30df, 0x30cb, 0x30fc, 0x30e1, 0x30ea, 0x30c0, 0x30e1, 0x30c8,
0x30e9, 0x30ab, 0x30c8, 0x30e9, 0x30e1, 0x30ad, 0x30b7, 0x30b3, 0x30b7, 0x30c6, 0x30a3, 0x30fc,
0x30df, 0x30af, 0x30ed, 0x30f3, 0x5cf6, 0x30e2, 0x30f3, 0x30af, 0x30c8, 0x30f3, 0x30e2, 0x30f3,
0x30c6, 0x30ec, 0x30a4, 0x30e2, 0x30f3, 0x30c6, 0x30d3, 0x30c7, 0x30aa, 0x30e2, 0x30f3, 0x30c8,
0x30bb, 0x30e9, 0x30c8, 0x30ca, 0x30c3, 0x30bd, 0x30fc, 0x30cb, 0x30e5, 0x30fc, 0x30e8, 0x30fc,
0x30af, 0x30ce, 0x30fc, 0x30e0, 0x30ce, 0x30ed, 0x30fc, 0x30cb, 0x30e3, 0x30ce, 0x30fc, 0x30b9,
0x30c0, 0x30b3, 0x30bf, 0x5dde, 0x30d3, 0x30e5, 0x30fc, 0x30e9, 0x30fc, 0x30ce, 0x30fc, 0x30b9,
0x30c0, 0x30b3, 0x30bf, 0x5dde, 0x30bb, 0x30f3, 0x30bf, 0x30fc, 0x30ce, 0x30fc, 0x30b9, 0x30c0,
0x30b3, 0x30bf, 0x5dde, 0x30cb, 0x30e5, 0x30fc, 0x30bb, 0x30fc, 0x30e9, 0x30e0, 0x30cc, 0x30fc,
0x30af, 0x30aa, 0x30d2, 0x30ca, 0x30ac, 0x30d1, 0x30ca, 0x30de, 0x30d1, 0x30e9, 0x30de, 0x30ea,
0x30dc, 0x30d5, 0x30a7, 0x30cb, 0x30c3, 0x30af, 0x30b9, 0x30dd, 0x30eb, 0x30c8, 0x30fc, 0x30d7,
0x30e9, 0x30f3, 0x30b9, 0x30dd, 0x30fc, 0x30c8, 0x30aa, 0x30d6, 0x30b9, 0x30da, 0x30a4, 0x30f3,
0x30dd, 0x30eb, 0x30c8, 0x30d9, 0x30fc, 0x30ea, 0x30e7, 0x30d7, 0x30a8, 0x30eb, 0x30c8, 0x30ea,
0x30b3, 0x30d7, 0x30f3, 0x30bf, 0x30a2, 0x30ec, 0x30ca, 0x30b9, 0x30e9, 0x30f3, 0x30ad, 0x30f3,
0x30a4, 0x30f3, 0x30ec, 0x30c3, 0x30c8, 0x30ec, 0x30b7, 0x30d5, 0x30a7, 0x30ec, 0x30b8, 0x30e3,
0x30a4, 0x30ca, 0x30ec, 0x30be, 0x30ea, 0x30e5, 0x30fc, 0x30c8, 0x30ea, 0x30aa, 0x30d6, 0x30e9,
0x30f3, 0x30b3, 0x30b5, 0x30f3, 0x30bf, 0x30ec, 0x30e0, 0x30b5, 0x30f3, 0x30c1, 0x30a2, 0x30b4,
0x30b5, 0x30f3, 0x30c8, 0x30c9, 0x30df, 0x30f3, 0x30b4, 0x30b5, 0x30f3, 0x30d1, 0x30a6, 0x30ed,
0x30a4, 0x30c8, 0x30b3, 0x30eb, 0x30c8, 0x30eb, 0x30df, 0x30c3, 0x30c8, 0x30b7, 0x30c8, 0x30ab,
0x30b5, 0x30f3, 0x30fb, 0x30d0, 0x30eb, 0x30c6, 0x30eb, 0x30df, 0x30fc, 0x30bb, 0x30f3, 0x30c8,
0x30b8, 0x30e7, 0x30f3, 0x30ba, 0x30bb, 0x30f3, 0x30c8, 0x30af, 0x30ea, 0x30b9, 0x30c8, 0x30d5,
0x30a1, 0x30fc, 0x30bb, 0x30f3, 0x30c8, 0x30eb, 0x30b7, 0x30a2, 0x30bb, 0x30f3, 0x30c8, 0x30c8,
0x30fc, 0x30de, 0x30b9, 0x30bb, 0x30f3, 0x30c8, 0x30d3, 0x30f3, 0x30bb, 0x30f3, 0x30c8, 0x30b9,
0x30a6, 0x30a3, 0x30d5, 0x30c8, 0x30ab, 0x30ec, 0x30f3, 0x30c8, 0x30c6, 0x30b0, 0x30b7, 0x30ac,
0x30eb, 0x30d1, 0x30c1, 0x30e5, 0x30fc, 0x30ec, 0x30c6, 0x30a3, 0x30d5, 0x30a2, 0x30ca, 0x30c8,
0x30ed, 0x30f3, 0x30c8, 0x30c8, 0x30eb, 0x30c8, 0x30fc, 0x30e9, 0x30d0, 0x30f3, 0x30af, 0x30fc,
0x30d0, 0x30fc, 0x30db, 0x30ef, 0x30a4, 0x30c8, 0x30db, 0x30fc, 0x30b9, 0x30a6, 0x30a3, 0x30cb,
0x30da, 0x30b0, 0x30e4, 0x30af, 0x30bf, 0x30c3, 0x30c8, 0x30b1, 0x30fc, 0x30b7, 0x30fc, 0x57fa,
0x5730, 0x30c7, 0x30fc, 0x30d3, 0x30b9, 0x57fa, 0x5730, 0x30c7, 0x30e5, 0x30e2, 0x30f3, 0x30fb,
0x30c7, 0x30e5, 0x30eb, 0x30f4, 0x30a3, 0x30eb, 0x57fa, 0x5730, 0x30de, 0x30c3, 0x30b3, 0x30ea,
0x30fc, 0x30e2, 0x30fc, 0x30bd, 0x30f3, 0x57fa, 0x5730, 0x30de, 0x30af, 0x30de, 0x30fc, 0x30c9,
0x57fa, 0x5730, 0x30d1, 0x30fc, 0x30de, 0x30fc, 0x57fa, 0x5730, 0x30ed, 0x30bc, 0x30e9, 0x57fa,
0x5730, 0x30c8, 0x30ed, 0x30eb, 0x57fa, 0x5730, 0x30dc, 0x30b9, 0x30c8, 0x30fc, 0x30af, 0x57fa,
0x5730, 0x30ed, 0x30f3, 0x30b0, 0x30a4, 0x30a7, 0x30fc, 0x30eb, 0x30d3, 0x30fc, 0x30f3, 0x30a2,
0x30c7, 0x30f3, 0x30a2, 0x30eb, 0x30de, 0x30c8, 0x30a4, 0x30a2, 0x30f3, 0x30de, 0x30f3, 0x30a2,
0x30ca, 0x30c7, 0x30a3, 0x30ea, 0x30a2, 0x30af, 0x30bf, 0x30a6, 0x30a2, 0x30af, 0x30c8, 0x30d9,
0x30a2, 0x30b7, 0x30ac, 0x30d0, 0x30fc, 0x30c9, 0x30a2, 0x30c6, 0x30a3, 0x30e9, 0x30a6, 0x30d0,
0x30b0, 0x30c0, 0x30c3, 0x30c9, 0x30d0, 0x30fc, 0x30ec, 0x30fc, 0x30f3, 0x30d0, 0x30af, 0x30fc,
0x30d0, 0x30f3, 0x30b3, 0x30af, 0x30d0, 0x30eb, 0x30ca, 0x30a6, 0x30eb, 0x30d9, 0x30a4, 0x30eb,
0x30fc, 0x30c8, 0x30d3, 0x30b7, 0x30e5, 0x30b1, 0x30af, 0x30d6, 0x30eb, 0x30cd, 0x30a4, 0x30c1,
0x30bf, 0x30b3, 0x30ed, 0x30f3, 0x30dc, 0x30c0, 0x30de, 0x30b9, 0x30ab, 0x30b9, 0x30c0, 0x30c3,
0x30ab, 0x30c9, 0x30d0, 0x30a4, 0x30c9, 0x30a5, 0x30b7, 0x30e3, 0x30f3, 0x30d9, 0x30d5, 0x30a1,
0x30de, 0x30b0, 0x30b9, 0x30bf, 0x30ac, 0x30b6, 0x30d8, 0x30d6, 0x30ed, 0x30f3, 0x30db, 0x30fc,
0x30c1, 0x30df, 0x30f3, 0x30db, 0x30d6, 0x30c9, 0x30a4, 0x30eb, 0x30af, 0x30fc, 0x30c4, 0x30af,
0x30b8, 0x30e3, 0x30ab, 0x30eb, 0x30bf, 0x30b8, 0x30e3, 0x30e4, 0x30d7, 0x30e9, 0x30a8, 0x30eb,
0x30b5, 0x30ec, 0x30e0, 0x30ab, 0x30d6, 0x30fc, 0x30eb, 0x30ab, 0x30e0, 0x30c1, 0x30e3, 0x30c3,
0x30ab, 0x30ab, 0x30e9, 0x30c1, 0x30ab, 0x30c8, 0x30de, 0x30f3, 0x30ba, 0x30cf, 0x30f3, 0x30c9,
0x30a5, 0x30a4, 0x30ac, 0x30b3, 0x30eb, 0x30ab, 0x30bf, 0x30af, 0x30e9, 0x30b9, 0x30ce, 0x30e4,
0x30eb, 0x30b9, 0x30af, 0x30af, 0x30a2, 0x30e9, 0x30eb, 0x30f3, 0x30d7, 0x30fc, 0x30eb, 0x30af,
0x30c1, 0x30f3, 0x30af, 0x30a6, 0x30a7, 0x30fc, 0x30c8, 0x30de, 0x30ab, 0x30aa, 0x30de, 0x30ac,
0x30c0, 0x30f3, 0x30de, 0x30ab, 0x30c3, 0x30b5, 0x30eb, 0x30de, 0x30cb, 0x30e9, 0x30de, 0x30b9,
0x30ab, 0x30c3, 0x30c8, 0x30cb, 0x30b3, 0x30b7, 0x30a2, 0x30ce, 0x30f4, 0x30a9, 0x30af, 0x30ba,
0x30cd, 0x30c4, 0x30af, 0x30ce, 0x30f4, 0x30a9, 0x30b7, 0x30d3, 0x30eb, 0x30b9, 0x30af, 0x30aa,
0x30e0, 0x30b9, 0x30af, 0x30aa, 0x30e9, 0x30eb, 0x30d7, 0x30ce, 0x30f3, 0x30da, 0x30f3, 0x30dd,
0x30f3, 0x30c6, 0x30a3, 0x30a2, 0x30ca, 0x30c3, 0x30af, 0x5e73, 0x58cc, 0x30ab, 0x30bf, 0x30fc,
0x30eb, 0x30b3, 0x30b9, 0x30bf, 0x30ca, 0x30a4, 0x30af, 0x30ba, 0x30ed, 0x30eb, 0x30c0, 0x30ea,
0x30e4, 0x30c9, 0x30b5, 0x30cf, 0x30ea, 0x30f3, 0x30b5, 0x30de, 0x30eb, 0x30ab, 0x30f3, 0x30c9,
0x30bd, 0x30a6, 0x30eb, 0x30b7, 0x30f3, 0x30ac, 0x30dd, 0x30fc, 0x30eb, 0x30b9, 0x30ec, 0x30c9,
0x30cd, 0x30b3, 0x30ea, 0x30e0, 0x30b9, 0x30af, 0x30bf, 0x30b7, 0x30b1, 0x30f3, 0x30c8, 0x30c8,
0x30d3, 0x30ea, 0x30b7, 0x30c6, 0x30d8, 0x30e9, 0x30f3, 0x30c6, 0x30a3, 0x30f3, 0x30d7, 0x30fc,
0x30c8, 0x30e0, 0x30b9, 0x30af, 0x30a6, 0x30e9, 0x30f3, 0x30d0, 0x30fc, 0x30c8, 0x30eb, 0x30a6,
0x30eb, 0x30e0, 0x30c1, 0x30a6, 0x30b9, 0x30c1, 0x30cd, 0x30e9, 0x30d3, 0x30a8, 0x30f3, 0x30c1,
0x30e3, 0x30f3, 0x30a6, 0x30e9, 0x30b8, 0x30aa, 0x30b9, 0x30c8, 0x30af, 0x30e4, 0x30af, 0x30fc,
0x30c4, 0x30af, 0x30e4, 0x30f3, 0x30b4, 0x30f3, 0x30a8, 0x30ab, 0x30c6, 0x30ea, 0x30f3, 0x30d6,
0x30eb, 0x30b0, 0x30a8, 0x30ec, 0x30d0, 0x30f3, 0x30a2, 0x30be, 0x30ec, 0x30b9, 0x30d0, 0x30df,
0x30e5, 0x30fc, 0x30c0, 0x30ab, 0x30ca, 0x30ea, 0x30a2, 0x30ab, 0x30fc, 0x30dc, 0x30d9, 0x30eb,
0x30c7, 0x30d5, 0x30a7, 0x30ed, 0x30fc, 0x30de, 0x30c7, 0x30a4, 0x30e9, 0x30ec, 0x30a4, 0x30ad,
0x30e3, 0x30d3, 0x30af, 0x30b5, 0x30a6, 0x30b9, 0x30b8, 0x30e7, 0x30fc, 0x30b8, 0x30a2, 0x30bb,
0x30f3, 0x30c8, 0x30d8, 0x30ec, 0x30ca, 0x30b9, 0x30bf, 0x30f3, 0x30ec, 0x30fc, 0x30a2, 0x30c7,
0x30ec, 0x30fc, 0x30c9, 0x30d6, 0x30ea, 0x30b9, 0x30d9, 0x30f3, 0x30d6, 0x30ed, 0x30fc, 0x30af,
0x30f3, 0x30d2, 0x30eb, 0x30c0, 0x30fc, 0x30a6, 0x30a3, 0x30f3, 0x30e6, 0x30fc, 0x30af, 0x30e9,
0x30db, 0x30d0, 0x30fc, 0x30c8, 0x30ea, 0x30f3, 0x30c7, 0x30de, 0x30f3, 0x30ed, 0x30fc, 0x30c9,
0x30cf, 0x30a6, 0x30e1, 0x30eb, 0x30dc, 0x30eb, 0x30f3, 0x30d1, 0x30fc, 0x30b9, 0x30b7, 0x30c9,
0x30cb, 0x30fc, 0x30a2, 0x30e0, 0x30b9, 0x30c6, 0x30eb, 0x30c0, 0x30e0, 0x30a2, 0x30f3, 0x30c9,
0x30e9, 0x30a2, 0x30b9, 0x30c8, 0x30e9, 0x30cf, 0x30f3, 0x30a2, 0x30c6, 0x30cd, 0x30d9, 0x30aa,
0x30b0, 0x30e9, 0x30fc, 0x30c9, 0x30d9, 0x30eb, 0x30ea, 0x30f3, 0x30d6, 0x30e9, 0x30c1, 0x30b9,
0x30e9, 0x30d0, 0x30d6, 0x30ea, 0x30e5, 0x30c3, 0x30bb, 0x30eb, 0x30d6, 0x30ab, 0x30ec, 0x30b9,
0x30c8, 0x30d6, 0x30c0, 0x30da, 0x30b9, 0x30c8, 0x30d3, 0x30e5, 0x30fc, 0x30b8, 0x30f3, 0x30b2,
0x30f3, 0x30ad, 0x30b7, 0x30ca, 0x30a6, 0x30b3, 0x30da, 0x30f3, 0x30cf, 0x30fc, 0x30b2, 0x30f3,
0x30c0, 0x30d6, 0x30ea, 0x30f3, 0x30b8, 0x30d6, 0x30e9, 0x30eb, 0x30bf, 0x30eb, 0x30ac, 0x30fc,
0x30f3, 0x30b8, 0x30fc, 0x30d8, 0x30eb, 0x30b7, 0x30f3, 0x30ad, 0x30de, 0x30f3, 0x5cf6, 0x30a4,
0x30b9, 0x30bf, 0x30f3, 0x30d6, 0x30fc, 0x30eb, 0x30b8, 0x30e3, 0x30fc, 0x30b8, 0x30fc, 0x30ab,
0x30ea, 0x30fc, 0x30cb, 0x30f3, 0x30b0, 0x30e9, 0x30fc, 0x30c9, 0x30ad, 0x30fc, 0x30ed, 0x30d5,
0x30ad, 0x30fc, 0x30a6, 0x30ea, 0x30b9, 0x30dc, 0x30f3, 0x30ea, 0x30e5, 0x30d6, 0x30ea, 0x30e3,
0x30ca, 0x30ed, 0x30f3, 0x30c9, 0x30f3, 0x30eb, 0x30af, 0x30bb, 0x30f3, 0x30d6, 0x30eb, 0x30af,
0x30de, 0x30c9, 0x30ea, 0x30fc, 0x30c9, 0x30de, 0x30eb, 0x30bf, 0x30de, 0x30ea, 0x30a8, 0x30cf,
0x30e0, 0x30f3, 0x30df, 0x30f3, 0x30b9, 0x30af, 0x30e2, 0x30ca, 0x30b3, 0x30e2, 0x30b9, 0x30af,
0x30ef, 0x30aa, 0x30b9, 0x30ed, 0x30d1, 0x30ea, 0x30dd, 0x30c9, 0x30b4, 0x30ea, 0x30c4, 0x30a1,
0x30d7, 0x30e9, 0x30cf, 0x30ea, 0x30ac, 0x30b5, 0x30de, 0x30e9, 0x30b5, 0x30f3, 0x30de, 0x30ea,
0x30ce, 0x30b5, 0x30e9, 0x30a8, 0x30dc, 0x30b5, 0x30e9, 0x30c8, 0x30d5, 0x30b7, 0x30f3, 0x30d5,
0x30a7, 0x30ed, 0x30dd, 0x30ea, 0x30b9, 0x30b3, 0x30d4, 0x30a8, 0x30bd, 0x30d5, 0x30a3, 0x30a2,
0x30b9, 0x30c8, 0x30c3, 0x30af, 0x30db, 0x30eb, 0x30e0, 0x30bf, 0x30ea, 0x30f3, 0x30c6, 0x30a3,
0x30e9, 0x30ca, 0x30a6, 0x30ea, 0x30e4, 0x30ce, 0x30d5, 0x30b9, 0x30af, 0x30d5, 0x30a1, 0x30c9,
0x30a5, 0x30fc, 0x30c4, 0x30d0, 0x30c1, 0x30ab, 0x30f3, 0x30a6, 0x30a3, 0x30fc, 0x30f3, 0x30f4,
0x30a3, 0x30ea, 0x30cb, 0x30e5, 0x30b9, 0x30dc, 0x30eb, 0x30b4, 0x30b0, 0x30e9, 0x30fc, 0x30c9,
0x30ef, 0x30eb, 0x30b7, 0x30e3, 0x30ef, 0x30b6, 0x30b0, 0x30ec, 0x30d6, 0x30c1, 0x30e5, 0x30fc,
0x30ea, 0x30c3, 0x30d2, 0x30a2, 0x30f3, 0x30bf, 0x30ca, 0x30ca, 0x30ea, 0x30dc, 0x30c1, 0x30e3,
0x30b4, 0x30b9, 0x30af, 0x30ea, 0x30b9, 0x30de, 0x30b9, 0x5cf6, 0x30b3, 0x30b3, 0x30b9, 0x8af8,
0x5cf6, 0x30b3, 0x30e2, 0x30ed, 0x30b1, 0x30eb, 0x30b2, 0x30ec, 0x30f3, 0x8af8, 0x5cf6, 0x30de,
0x30d8, 0x30e2, 0x30eb, 0x30c7, 0x30a3, 0x30d6, 0x30e2, 0x30fc, 0x30ea, 0x30b7, 0x30e3, 0x30b9,
0x30de, 0x30e8, 0x30c3, 0x30c8, 0x30ec, 0x30e6, 0x30cb, 0x30aa, 0x30f3, 0x30a2, 0x30d4, 0x30a2,
0x30aa, 0x30fc, 0x30af, 0x30e9, 0x30f3, 0x30c9, 0x30d6, 0x30fc, 0x30b2, 0x30f3, 0x30d3, 0x30eb,
0x30c1, 0x30e3, 0x30bf, 0x30e0, 0x30c1, 0x30e5, 0x30fc, 0x30af, 0x30a4, 0x30fc, 0x30b9, 0x30bf,
0x30fc, 0x5cf6, 0x30a8, 0x30d5, 0x30a7, 0x30c6, 0x5cf6, 0x30d5, 0x30a1, 0x30ab, 0x30aa, 0x30d5,
0x30a9, 0x30d5, 0x30a3, 0x30b8, 0x30fc, 0x30d5, 0x30ca, 0x30d5, 0x30c6, 0x30a3, 0x30ac, 0x30e9,
0x30d1, 0x30b4, 0x30b9, 0x30ac, 0x30f3, 0x30d3, 0x30a8, 0x8af8, 0x5cf6, 0x30ac, 0x30c0, 0x30eb,
0x30ab, 0x30ca, 0x30eb, 0x30b0, 0x30a2, 0x30e0, 0x30db, 0x30ce, 0x30eb, 0x30eb, 0x30a8, 0x30f3,
0x30c0, 0x30fc, 0x30d9, 0x30ea, 0x30fc, 0x5cf6, 0x30ad, 0x30ea, 0x30b9, 0x30a3, 0x30de, 0x30b9,
0x30a3, 0x5cf6, 0x30b3, 0x30b9, 0x30e9, 0x30a8, 0x30af, 0x30a7, 0x30bc, 0x30ea, 0x30f3, 0x30de,
0x30b8, 0x30e5, 0x30ed, 0x30de, 0x30eb, 0x30ad, 0x30fc, 0x30ba, 0x30df, 0x30c3, 0x30c9, 0x30a6,
0x30a7, 0x30fc, 0x5cf6, 0x30cb, 0x30a6, 0x30a8, 0x30ce, 0x30fc, 0x30d5, 0x30a9, 0x30fc, 0x30af,
0x5cf6, 0x30cc, 0x30e1, 0x30a2, 0x30d1, 0x30b4, 0x30d1, 0x30b4, 0x30d1, 0x30e9, 0x30aa, 0x30d4,
0x30c8, 0x30b1, 0x30a2, 0x30f3, 0x8af8, 0x5cf6, 0x30dd, 0x30f3, 0x30da, 0x30a4, 0x5cf6, 0x30dd,
0x30fc, 0x30c8, 0x30e2, 0x30ec, 0x30b9, 0x30d3, 0x30fc, 0x30e9, 0x30ed, 0x30c8, 0x30f3, 0x30ac,
0x30b5, 0x30a4, 0x30d1, 0x30f3, 0x30bf, 0x30d2, 0x30c1, 0x30bf, 0x30e9, 0x30ef, 0x30c8, 0x30f3,
0x30ac, 0x30bf, 0x30d7, 0x30a6, 0x30a7, 0x30fc, 0x30af, 0x5cf6, 0x30a6, 0x30a9, 0x30ea, 0x30b9,
0x8af8, 0x5cf6, 0x4d, 0x65, 0x6e, 0x64, 0x6f, 0x73, 0x61, 0x42, 0x65, 0x6c,
0x69, 0x73, 0x65, 0x54, 0x65, 0x6c, 0x75, 0x6b, 0x20, 0x43, 0x61, 0x6d,
0x62, 0x72, 0x69, 0x64, 0x67, 0x65, 0x4b, 0x61, 0x79, 0x65, 0x6e, 0x6e,
0x65, 0x43, 0x69, 0x75, 0x64, 0x61, 0x64, 0x20, 0x4a, 0x75, 0xe1, 0x72,
0x65, 0x73, 0x42, 0x65, 0x6e, 0x74, 0x65, 0x6e, 0x67, 0x20, 0x4e, 0x65,
0x6c, 0x73, 0x6f, 0x6e, 0x54, 0x65, 0x6c, 0x75, 0x6b, 0x20, 0x47, 0x6c,
0x61, 0x63, 0x65, 0x54, 0x65, 0x6c, 0x75, 0x6b, 0x20, 0x47, 0x6f, 0x6f,
0x73, 0x65, 0x4b, 0x75, 0x74, 0x68, 0x6f, 0x20, 0x4d, 0x65, 0x6b, 0x73,
0x69, 0x6b, 0x6f, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x20, 0x5b, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x5d, 0x54, 0x65,
0x6e, 0x67, 0x61, 0x68, 0x20, 0x5b, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x20, 0x4c, 0x6f, 0x72, 0x5d, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x20, 0x41,
0x6e, 0x79, 0x61, 0x72, 0x20, 0x5b, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x20, 0x4c, 0x6f, 0x72, 0x5d, 0x50, 0x61, 0x6c, 0x61, 0x62, 0x75, 0x68,
0x61, 0x6e, 0x20, 0x53, 0x70, 0x61, 0x6e, 0x79, 0x6f, 0x6c, 0x53, 0x61,
0x6e, 0x74, 0x61, 0x20, 0x42, 0x61, 0x72, 0x74, 0x68, 0x65, 0x6c, 0x65,
0x6d, 0x79, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x20, 0x4a, 0x6f, 0x68, 0x6e,
0x53, 0x61, 0x6e, 0x74, 0x61, 0x20, 0x4b, 0x69, 0x74, 0x74, 0x73, 0x53,
0x61, 0x6e, 0x74, 0x61, 0x20, 0x54, 0x68, 0x6f, 0x6d, 0x61, 0x73, 0x53,
0x61, 0x6e, 0x74, 0x61, 0x20, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x74,
0x41, 0x72, 0x75, 0x73, 0x20, 0x42, 0x61, 0x6e, 0x74, 0x65, 0x72, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x4b, 0x69, 0x64, 0x75, 0x6c,
0x50, 0x75, 0x6c, 0x6f, 0x20, 0x4d, 0x61, 0x6e, 0x4c, 0x75, 0x6b, 0x73,
0x65, 0x6d, 0x62, 0x75, 0x72, 0x6b, 0x4b, 0x68, 0x61, 0x67, 0x6f, 0x73,
0x50, 0x61, 0x73, 0x6b, 0x61, 0x68, 0x50, 0x65, 0x6c, 0x61, 0x62, 0x75,
0x68, 0x61, 0x6e, 0x20, 0x4d, 0x6f, 0x72, 0x65, 0x73, 0x62, 0x79, 0x41,
0x64, 0x61, 0x6b, 0x69, 0x41, 0x6e, 0x6b, 0x6f, 0x72, 0x61, 0x6a, 0x69,
0x41, 0x6e, 0x67, 0x69, 0x6c, 0x61, 0x42, 0x61, 0xed, 0x61, 0x20, 0x64,
0x69, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x61, 0x73, 0x42, 0x61,
0x72, 0x62, 0x61, 0x64, 0x75, 0x73, 0x42, 0x65, 0x6c, 0x69, 0x7a, 0x69,
0x58, 0x69, 0x6b, 0x61, 0x67, 0x75, 0x58, 0x69, 0x75, 0xe1, 0x75, 0x61,
0x4b, 0x72, 0xe9, 0x73, 0x74, 0x6f, 0x6e, 0x4b, 0x75, 0x72, 0x61, 0x73,
0x61, 0x75, 0x47, 0x75, 0x61, 0x64, 0x61, 0x6c, 0x75, 0x70, 0x69, 0x47,
0x75, 0x61, 0x74, 0x69, 0x6d, 0x61, 0x6c, 0x61, 0x4d, 0x61, 0x6e, 0xe1,
0x67, 0x75, 0x61, 0x53, 0x69, 0x64, 0x61, 0x64, 0x69, 0x20, 0x64, 0x69,
0x20, 0x4d, 0xe9, 0x78, 0x69, 0x6b, 0x75, 0x4e, 0x6f, 0x76, 0x61, 0x20,
0x49, 0x6f, 0x72, 0x6b, 0x69, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c,
0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4e,
0x6f, 0x72, 0x74, 0x69, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x6f,
0x72, 0x74, 0x69, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d,
0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x69, 0x20,
0x4e, 0x6f, 0x72, 0x74, 0x69, 0x50, 0x6f, 0x72, 0x74, 0x75, 0x20, 0x64,
0x69, 0x20, 0x53, 0x70, 0x61, 0x6e, 0x68, 0x61, 0x50, 0x6f, 0x72, 0x74,
0x75, 0x20, 0x52, 0x69, 0x6b, 0x75, 0x53, 0x61, 0x6e, 0x74, 0x75, 0x20,
0x44, 0x75, 0x6d, 0x69, 0x6e, 0x67, 0x75, 0x53, 0x61, 0x6e, 0x20, 0x42,
0x61, 0x72, 0x74, 0x6f, 0x6c, 0x6f, 0x6d, 0x65, 0x75, 0x53, 0x61, 0x6e,
0x20, 0x4b, 0x72, 0x69, 0x73, 0x74, 0x6f, 0x76, 0x61, 0x6e, 0x53, 0x61,
0x6e, 0x74, 0x61, 0x20, 0x4c, 0xfa, 0x73, 0x69, 0x61, 0x53, 0x61, 0x6e,
0x20, 0x42, 0x69, 0x73, 0x65, 0x6e, 0x74, 0x69, 0x54, 0x6f, 0x72, 0x6f,
0x6e, 0x74, 0x75, 0x55, 0xed, 0x6e, 0x69, 0x70, 0x65, 0x67, 0x41, 0x62,
0x69, 0x1e7, 0x61, 0x6e, 0x41, 0x6b, 0x1e5b, 0x61, 0x41, 0x64, 0x69, 0x73,
0x20, 0x41, 0x62, 0x61, 0x62, 0x61, 0x4c, 0x65, 0x7a, 0x7a, 0x61, 0x79,
0x65, 0x72, 0x42, 0x61, 0x6d, 0x61, 0x6b, 0x75, 0x42, 0x72, 0x61, 0x1e93,
0x61, 0x76, 0x69, 0x6c, 0x4c, 0x71, 0x61, 0x68, 0x69, 0x72, 0x61, 0x4b,
0x61, 0x1e93, 0x61, 0x62, 0x6c, 0x61, 0x6e, 0x6b, 0x61, 0x53, 0x65, 0x62,
0x74, 0x61, 0x4b, 0x75, 0x6e, 0x61, 0x6b, 0x72, 0x69, 0x44, 0x61, 0x72,
0x20, 0x53, 0x61, 0x6c, 0x61, 0x6d, 0x1e6, 0x69, 0x62, 0x75, 0x74, 0x69,
0x44, 0x77, 0x61, 0x6c, 0x61, 0x4c, 0x65, 0x25b, 0x79, 0x75, 0x6e, 0x46,
0x72, 0x69, 0x1e6d, 0x61, 0x77, 0x6e, 0x47, 0x61, 0x62, 0x75, 0x72, 0x75,
0x6e, 0x48, 0x61, 0x72, 0x61, 0x72, 0x69, 0x4a, 0x75, 0x68, 0x61, 0x6e,
0x69, 0x73, 0x62, 0x75, 0x72, 0x67, 0x58, 0x61, 0x1e5b, 0x74, 0x75, 0x6d,
0x4b, 0x69, 0x6e, 0x63, 0x61, 0x1e63, 0x61, 0x4c, 0x61, 0x67, 0x75, 0x73,
0x4c, 0x69, 0x62, 0x65, 0x72, 0x76, 0x69, 0x6c, 0x4c, 0x75, 0x6d, 0x69,
0x4c, 0x77, 0x61, 0x6e, 0x64, 0x61, 0x4c, 0x75, 0x62, 0x75, 0x6d, 0x62,
0x61, 0x63, 0x69, 0x4c, 0x75, 0x7a, 0x61, 0x6b, 0x61, 0x4d, 0x61, 0x6c,
0x61, 0x62, 0x75, 0x4d, 0x61, 0x70, 0x75, 0x1e6d, 0x75, 0x4d, 0x61, 0x73,
0x69, 0x72, 0x75, 0x4d, 0x75, 0x71, 0x61, 0x64, 0x69, 0x63, 0x75, 0x4d,
0x75, 0x6e, 0x1e5b, 0x75, 0x76, 0x79, 0x61, 0x4e, 0x61, 0x79, 0x72, 0x75,
0x62, 0x69, 0x4e, 0x1e7, 0x61, 0x6d, 0x69, 0x6e, 0x61, 0x4e, 0x79, 0x61,
0x6d, 0x65, 0x79, 0x4e, 0x77, 0x61, 0x71, 0x63, 0x75, 0x1e6d, 0x50, 0x75,
0x1e5b, 0x74, 0x75, 0x20, 0x4e, 0x75, 0x76, 0x75, 0x53, 0x61, 0x77, 0x20,
0x1e6c, 0x75, 0x6d, 0x69, 0x54, 0x72, 0x61, 0x62, 0x6c, 0x75, 0x73, 0x57,
0x69, 0x6e, 0x64, 0x68, 0x65, 0x6b, 0x41, 0x6e, 0x6b, 0x75, 0x72, 0x61,
0x6a, 0x41, 0x6e, 0x74, 0x69, 0x67, 0x61, 0x4b, 0x75, 0x72, 0x64, 0x75,
0x62, 0x61, 0x4d, 0x69, 0x6e, 0x64, 0x75, 0x1e93, 0x61, 0x53, 0x61, 0x6e,
0x20, 0x58, 0x77, 0x61, 0x6e, 0x53, 0x61, 0x6e, 0x20, 0x4c, 0x77, 0x69,
0x73, 0x55, 0x63, 0x77, 0x61, 0x79, 0x61, 0x41, 0x73, 0x75, 0x6e, 0x73,
0x79, 0x75, 0x6e, 0x41, 0x74, 0x69, 0x6b, 0x75, 0x6b, 0x61, 0x6e, 0x42,
0x61, 0x68, 0x79, 0x61, 0x42, 0x61, 0x68, 0x79, 0x61, 0x20, 0x42, 0x61,
0x6e, 0x64, 0x69, 0x72, 0x61, 0x73, 0x42, 0x69, 0x6c, 0x69, 0x7a, 0x42,
0x6c, 0x75, 0x6e, 0x2d, 0x53, 0x61, 0x62, 0x6c, 0x75, 0x6e, 0x42, 0x75,
0x67, 0x75, 0x1e6d, 0x61, 0x42, 0x77, 0x61, 0x1e93, 0x4b, 0x61, 0x6d, 0x62,
0x72, 0x69, 0x1e7, 0x20, 0x42, 0x61, 0x79, 0x4b, 0x75, 0x6d, 0x70, 0x75,
0x20, 0x47, 0x72, 0x61, 0x6e, 0x64, 0x69, 0x4b, 0x61, 0x79, 0x61, 0x6e,
0x43, 0x69, 0x6b, 0x61, 0x67, 0x75, 0x43, 0x69, 0x77, 0x61, 0x77, 0x61,
0x4b, 0x75, 0x73, 0x74, 0x61, 0x20, 0x52, 0x69, 0x6b, 0x61, 0x4b, 0x72,
0x69, 0x73, 0x74, 0x75, 0x6e, 0x4b, 0x75, 0x72, 0x61, 0x10d, 0x61, 0x77,
0x44, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x6b, 0x63, 0x61, 0x76, 0x65, 0x6e,
0x44, 0x61, 0x77, 0x73, 0x75, 0x6e, 0x44, 0x61, 0x77, 0x73, 0x75, 0x6e,
0x20, 0x4b, 0x72, 0x69, 0x6b, 0x44, 0x69, 0x74, 0x72, 0x77, 0x61, 0x44,
0x75, 0x6d, 0x69, 0x6e, 0x69, 0x6b, 0x49, 0x64, 0x6d, 0x75, 0x6e, 0x74,
0x75, 0x6e, 0x53, 0x61, 0x6c, 0x76, 0x61, 0x64, 0x75, 0x72, 0x46, 0x75,
0x1e5b, 0x74, 0x20, 0x4e, 0x69, 0x6c, 0x1e63, 0x75, 0x6e, 0x46, 0x75, 0x72,
0x1e6d, 0x61, 0x6c, 0x69, 0x7a, 0x61, 0x47, 0x72, 0x75, 0x6e, 0x61, 0x64,
0x61, 0x47, 0x77, 0x61, 0x64, 0x61, 0x6c, 0x75, 0x70, 0x69, 0x47, 0x77,
0x61, 0x74, 0x69, 0x6d, 0x61, 0x6c, 0x61, 0x47, 0x61, 0x79, 0x61, 0x6b,
0x69, 0x6c, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x4a, 0x61, 0x6d, 0x61, 0x79,
0x6b, 0x61, 0x4d, 0x61, 0x6e, 0x61, 0x67, 0x61, 0x4d, 0x61, 0x72, 0x69,
0x67, 0x75, 0x1e6d, 0x4e, 0x75, 0x1e5b, 0x75, 0x6e, 0x68, 0x61, 0x50, 0x61,
0x72, 0x61, 0x6d, 0x61, 0x72, 0x69, 0x62, 0x75, 0x50, 0x75, 0x72, 0x74,
0x75, 0x20, 0x56, 0x69, 0x6c, 0x68, 0x75, 0x52, 0x69, 0x73, 0x69, 0x66,
0x52, 0x79, 0x75, 0x20, 0x42, 0x72, 0x61, 0x6e, 0x6b, 0x75, 0x53, 0x61,
0x6e, 0x74, 0x79, 0x61, 0x67, 0x75, 0x53, 0x61, 0x77, 0x20, 0x50, 0x61,
0x77, 0x6c, 0x75, 0x4d, 0x61, 0x6b, 0x61, 0x72, 0x69, 0x56, 0x75, 0x73,
0x1e6d, 0x75, 0x6b, 0x4c, 0x75, 0x6e, 0x67, 0x79, 0x65, 0x72, 0x62, 0x79,
0x65, 0x6e, 0x190, 0x61, 0x64, 0x65, 0x6e, 0x190, 0x65, 0x6d, 0x6d, 0x61,
0x6e, 0x42, 0x65, 0x263, 0x64, 0x61, 0x64, 0x42, 0x61, 0x1e25, 0x72, 0x69,
0x6e, 0x42, 0x61, 0x6e, 0x67, 0x6b, 0x75, 0x6b, 0x42, 0x72, 0x75, 0x6e,
0x61, 0x79, 0x4b, 0x75, 0x6c, 0x75, 0x6d, 0x62, 0x75, 0x194, 0x65, 0x7a,
0x7a, 0x61, 0x48, 0x65, 0x62, 0x72, 0x75, 0x6e, 0x48, 0x75, 0x6e, 0x67,
0x20, 0x4b, 0x75, 0x6e, 0x67, 0x48, 0x75, 0x76, 0x64, 0x4a, 0x69, 0x72,
0x75, 0x73, 0x61, 0x6c, 0x65, 0x6d, 0x4b, 0x61, 0x72, 0x61, 0x63, 0x69,
0x4b, 0x75, 0x6c, 0x6b, 0x61, 0x74, 0x61, 0x4b, 0x77, 0x61, 0x6c, 0x61,
0x20, 0x4c, 0x75, 0x6d, 0x70, 0x75, 0x72, 0x4b, 0x77, 0x69, 0x74, 0x4d,
0x61, 0x6b, 0x61, 0x77, 0x4d, 0x75, 0x73, 0x71, 0x61, 0x1e6d, 0x4e, 0x69,
0x6b, 0x75, 0x73, 0x79, 0x61, 0x50, 0x79, 0x75, 0x6e, 0x67, 0x79, 0x75,
0x6e, 0x67, 0x51, 0x61, 0x1e6d, 0x61, 0x72, 0x4b, 0x75, 0x73, 0x74, 0x61,
0x6e, 0x61, 0x79, 0x52, 0x69, 0x79, 0x61, 0x1e0d, 0x53, 0x69, 0x79, 0x75,
0x6c, 0x43, 0x61, 0x6e, 0x67, 0x61, 0x79, 0x53, 0x61, 0x6e, 0x67, 0x61,
0x70, 0x75, 0x72, 0x54, 0x61, 0x79, 0x70, 0x61, 0x79, 0x54, 0x75, 0x6b,
0x79, 0x75, 0x59, 0x65, 0x72, 0x76, 0x61, 0x6e, 0x41, 0x1e93, 0x75, 0x72,
0x69, 0x73, 0x42, 0x69, 0x72, 0x6d, 0x75, 0x64, 0x54, 0x69, 0x67, 0x7a,
0x69, 0x72, 0x69, 0x6e, 0x20, 0x6e, 0x20, 0x4b, 0x61, 0x6e, 0x61, 0x72,
0x69, 0x4b, 0x61, 0x70, 0x20, 0x56, 0x69, 0x72, 0x46, 0x61, 0x72, 0x77,
0x69, 0x4d, 0x61, 0x64, 0x69, 0x72, 0x61, 0x4a, 0x79, 0x75, 0x72, 0x6a,
0x79, 0x61, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x1e93, 0x75, 0x6c, 0x53, 0x61,
0x6e, 0x74, 0x20, 0x49, 0x6c, 0x69, 0x6e, 0x61, 0x41, 0x64, 0x69, 0x6c,
0x61, 0x79, 0x69, 0x64, 0x4d, 0x61, 0x6c, 0x62, 0x75, 0x72, 0x6e, 0x41,
0x6d, 0x65, 0x73, 0x74, 0x69, 0x72, 0x64, 0x61, 0x6d, 0x41, 0x6e, 0x64,
0x75, 0x72, 0x61, 0x42, 0x69, 0x6c, 0x67, 0x72, 0x61, 0x64, 0x4b, 0x75,
0x70, 0x65, 0x6e, 0x68, 0x61, 0x67, 0x65, 0x6e, 0x44, 0x75, 0x62, 0x6c,
0x61, 0x6e, 0x4a, 0x69, 0x62, 0x72, 0x61, 0x6c, 0x1e6d, 0x61, 0x72, 0x47,
0x69, 0x72, 0x6e, 0x69, 0x7a, 0x65, 0x79, 0x48, 0x69, 0x6c, 0x73, 0x69,
0x6e, 0x6b, 0x69, 0x54, 0x69, 0x67, 0x7a, 0x69, 0x72, 0x74, 0x20, 0x6e,
0x20, 0x4d, 0x61, 0x6e, 0x49, 0x73, 0x1e6d, 0x61, 0x6e, 0x62, 0x75, 0x6c,
0x4a, 0x69, 0x72, 0x7a, 0x65, 0x79, 0x4b, 0x79, 0x65, 0x76, 0x4c, 0x69,
0x7a, 0x62, 0x75, 0x6e, 0x4c, 0x75, 0x6e, 0x64, 0x75, 0x6e, 0x4d, 0x61,
0x72, 0x69, 0x68, 0x61, 0x6d, 0x6e, 0x4d, 0x75, 0x6e, 0x61, 0x6b, 0x75,
0x4d, 0x75, 0x73, 0x6b, 0x75, 0x55, 0x1e63, 0x6c, 0x75, 0x52, 0x75, 0x6d,
0x53, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6e, 0x75, 0x53, 0x61,
0x72, 0x61, 0x6a, 0x69, 0x76, 0x75, 0x53, 0x6f, 0x66, 0x79, 0x61, 0x53,
0x1e6d, 0x75, 0x6b, 0x68, 0x75, 0x6c, 0x6d, 0x56, 0x79, 0x65, 0x6e, 0x6e,
0x61, 0x56, 0x69, 0x6c, 0x6e, 0x79, 0x75, 0x73, 0x5a, 0x75, 0x72, 0x69,
0x78, 0x41, 0x6e, 0x74, 0x61, 0x6e, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x76,
0x75, 0x43, 0x61, 0x67, 0x75, 0x73, 0x4b, 0x75, 0x6b, 0x75, 0x4b, 0x75,
0x6d, 0x75, 0x1e5b, 0x4b, 0x65, 0x72, 0x67, 0x6c, 0x65, 0x6e, 0x4d, 0x61,
0x68, 0x69, 0x4d, 0x75, 0x72, 0x69, 0x73, 0x4d, 0x61, 0x79, 0x75, 0x1e6d,
0x52, 0x69, 0x79, 0x75, 0x6e, 0x79, 0x75, 0x6e, 0x41, 0x70, 0x79, 0x61,
0x55, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x43, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x75, 0x73, 0x47, 0x61, 0x6d,
0x62, 0x79, 0x65, 0x72, 0x4e, 0x69, 0x79, 0x75, 0x4e, 0x75, 0x72, 0x66,
0x75, 0x6c, 0x6b, 0xc85, 0xcac, 0xcbf, 0xca6, 0xccd, 0x200c, 0xc9c, 0xca8, 0xccd,
0xc85, 0xc95, 0xccd, 0xcb0, 0xcbe, 0xc86, 0xca1, 0xcbf, 0xcb8, 0xccd, 0x20, 0xc85,
0xcac, 0xcbe, 0xcac, 0xcbe, 0xc85, 0xcb2, 0xccd, 0xc9c, 0xcc0, 0xcb0, 0xccd, 0xcb8,
0xccd, 0xc85, 0xcb8, 0xccd, 0xcae, 0xcbe, 0xcb0, 0xcbe, 0xcac, 0xcae, 0xcbe, 0xc95,
0xcca, 0xcac, 0xcbe, 0xc82, 0xc97, 0xcbf, 0xcac, 0xc82, 0xc9c, 0xcc1, 0xcb2, 0xccd,
0xcac, 0xcbf, 0xcb8, 0xccd, 0xcb8, 0xcbe, 0xcb5, 0xccd, 0xcac, 0xccd, 0xcb2, 0xccd,
0xcaf, 0xcbe, 0xc82, 0xc9f, 0xcc8, 0xcb0, 0xccd, 0xcac, 0xccd, 0xcb0, 0xcc6, 0xc9c,
0xcbe, 0xcb5, 0xcc0, 0xcb2, 0xccd, 0xcac, 0xcc1, 0xc9c, 0xcc1, 0xc82, 0xcac, 0xcc1,
0xcb0, 0xcbe, 0xc95, 0xcc8, 0xcb0, 0xccb, 0xc95, 0xccd, 0xcaf, 0xcbe, 0xcb8, 0xcbe,
0xcac, 0xccd, 0xcb2, 0xcbe, 0xc82, 0xc95, 0xcbe, 0xcb8, 0xcc6, 0xcaf, 0xcc1, 0xc9f,
0xcbe, 0xc95, 0xcca, 0xca8, 0xcbe, 0xc95, 0xccd, 0xcb0, 0xcbf, 0xca1, 0xcbe, 0xc95,
0xcb0, 0xccd, 0xca1, 0xcbe, 0xcb0, 0xccd, 0x20, 0xc8e, 0xcb8, 0xccd, 0x20, 0xcb8,
0xcb2, 0xcbe, 0xcae, 0xccd, 0xc9c, 0xcbf, 0xcac, 0xcc2, 0xc9f, 0xcbf, 0xca6, 0xccc,
0xcb2, 0xcbe, 0xc8e, 0xcb2, 0xccd, 0x20, 0xc94, 0xca8, 0xccd, 0xcab, 0xccd, 0xcb0,
0xcc0, 0xc9f, 0xccc, 0xca8, 0xccd, 0xc97, 0xccd, 0xcaf, 0xcbe, 0xcac, 0xcb0, 0xccb,
0xca8, 0xcc6, 0xcb9, 0xcb0, 0xcbe, 0xcb0, 0xcc6, 0xc9c, 0xccb, 0xcb9, 0xcbe, 0xca8,
0xccd, 0xcb8, 0xccd, 0x200c, 0xcac, 0xcb0, 0xccd, 0xc97, 0xccd, 0xc9c, 0xcc1, 0xcac,
0xcbe, 0xc95, 0xc82, 0xcaa, 0xcbe, 0xcb2, 0xcbe, 0xc96, 0xcb0, 0xccd, 0xc9f, 0xccb,
0xcae, 0xccd, 0xc95, 0xcbf, 0xc97, 0xcbe, 0xcb2, 0xcbf, 0xc95, 0xcbf, 0xca8, 0xccd,
0xcb7, 0xcbe, 0xcb8, 0xcbe, 0xcb2, 0xcbe, 0xc97, 0xccb, 0xcb8, 0xccd, 0xcb2, 0xcbf,
0xcac, 0xccd, 0xcb0, 0xcc6, 0xcb5, 0xcc0, 0xcb2, 0xccd, 0xcb2, 0xccb, 0xcae, 0xccd,
0xcb2, 0xcc1, 0xcb5, 0xcbe, 0xc82, 0xca1, 0xcbe, 0xcb2, 0xcc1, 0xcac, 0xcc1, 0xc82,
0xcac, 0xcbe, 0xcb6, 0xcbf, 0xcb2, 0xcc1, 0xcb8, 0xcbe, 0xc95, 0xcbe, 0xcae, 0xcb2,
0xcbe, 0xcac, 0xccb, 0xcae, 0xccd, 0xcaf, 0xcbe, 0xcaa, 0xcc1, 0xc9f, 0xcca, 0xcae,
0xcb8, 0xcc7, 0xcb0, 0xcc1, 0xc85, 0xc82, 0xcac, 0xcac, 0xcbe, 0xca8, 0xcc6, 0xcae,
0xcca, 0xc97, 0xcbe, 0xca6, 0xcbf, 0xcb6, 0xcc1, 0xcae, 0xcca, 0xca8, 0xccd, 0xcb0,
0xccb, 0xcb5, 0xcbf, 0xcaf, 0xcbe, 0xca8, 0xcc8, 0xcb0, 0xccb, 0xcac, 0xcbf, 0xca1,
0xccd, 0xc9c, 0xcae, 0xcc7, 0xca8, 0xcbe, 0xca8, 0xcbf, 0xcaf, 0xcbe, 0xcae, 0xcc6,
0xca8, 0xccc, 0xc95, 0xccd, 0x200c, 0xc9a, 0xccb, 0xc9f, 0xccd, 0xc9f, 0xccd, 0xc94,
0xc97, 0xcbe, 0xca1, 0xccb, 0xc97, 0xcc1, 0xcaa, 0xccb, 0xcb0, 0xccd, 0xc9f, 0xccb,
0x2d, 0xca8, 0xccb, 0xcb5, 0xccb, 0xcb8, 0xcbe, 0xcb5, 0xccd, 0x20, 0xc9f, 0xcca,
0xcae, 0xcbf, 0xc9f, 0xccd, 0xcb0, 0xcc8, 0xcaa, 0xccb, 0xcb2, 0xcbf, 0xc9f, 0xcc1,
0xca8, 0xcc0, 0xcb8, 0xccd, 0xcb5, 0xcbf, 0xc82, 0xca1, 0xccd, 0xcb9, 0xccb, 0xc95,
0xccd, 0xc85, 0xca1, 0xc95, 0xccd, 0x200c, 0x200c, 0xc86, 0xc82, 0xc95, 0xcb0, 0xccb,
0xc9c, 0xccd, 0xc86, 0xc82, 0xc97, 0xccd, 0xcb5, 0xcbf, 0xcb2, 0xccd, 0xcb2, 0xcbe,
0xc86, 0xc82, 0xc9f, 0xcbf, 0xc97, 0xcc1, 0xcb5, 0xcbe, 0xc85, 0xcb0, 0xcbe, 0xc97,
0xcc1, 0xcaf, 0xcbe, 0xca8, 0xcbe, 0xcac, 0xccd, 0xcaf, 0xcc2, 0xca8, 0xcb8, 0xccd,
0x20, 0xc90, 0xcb0, 0xcbf, 0xcb8, 0xccd, 0xc95, 0xc9f, 0xcae, 0xcbe, 0xcb0, 0xccd,
0xc95, 0xc95, 0xcca, 0xcb0, 0xccd, 0xca1, 0xcca, 0xcac, 0xcbe, 0xc9c, 0xcc1, 0xc9c,
0xcc8, 0xcb2, 0xcbe, 0x20, 0xcb0, 0xcbf, 0xcaf, 0xccb, 0xc9c, 0xcbe, 0xcae, 0xcc6,
0xc82, 0xca1, 0xcca, 0xc9c, 0xcbe, 0xcb0, 0xcbf, 0xcaf, 0xccb, 0x20, 0xc97, 0xcb2,
0xccd, 0xcb2, 0xcc6, 0xc97, 0xcca, 0xcb8, 0xccd, 0xcb8, 0xcbe, 0xcb2, 0xccd, 0xc9f,
0xcbe, 0xcb8, 0xccd, 0xcaf, 0xcbe, 0xca8, 0xccd, 0x20, 0xc9c, 0xcc1, 0xcb5, 0xcbe,
0xca8, 0xccd, 0xcb8, 0xccd, 0xcaf, 0xcbe, 0xca8, 0xccd, 0x20, 0xcb2, 0xcc2, 0xcaf,
0xcc0, 0xcb8, 0xccd, 0xc9f, 0xcc1, 0xc95, 0xcc1, 0xcae, 0xcbe, 0xca8, 0xccd, 0xc89,
0xcb6, 0xccd, 0xcb5, 0xcbe, 0xcaf, 0xc85, 0xcb0, 0xcc1, 0xcac, 0xcbe, 0xc85, 0xcb8,
0xcc1, 0xca8, 0xccd, 0xcb8, 0xcbf, 0xcaf, 0xcbe, 0xca8, 0xccd, 0xc85, 0xc9f, 0xcbf,
0xc95, 0xcca, 0xc95, 0xcbe, 0xca8, 0xccd, 0xcac, 0xcb9, 0xcc0, 0xcaf, 0xcbe, 0xcac,
0xcbe, 0xcb9, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcac, 0xc82, 0xca1, 0xcc7, 0xcb0, 0xcbe,
0xcb8, 0xccd, 0xcac, 0xcbe, 0xcb0, 0xccd, 0xcac, 0xca1, 0xccb, 0xcb8, 0xccd, 0xcac,
0xcc7, 0xcb2, 0xcc6, 0xc82, 0xcac, 0xcc6, 0xcb2, 0xcc0, 0xc9c, 0xccd, 0xcac, 0xccd,
0xcb2, 0xcbe, 0xc82, 0xc95, 0xccd, 0x2d, 0xcb8, 0xcbe, 0xcac, 0xccd, 0xcb2, 0xca8,
0xccd, 0xcac, 0xccb, 0xcb5, 0xcbe, 0x20, 0xcb5, 0xcbf, 0xcb8, 0xccd, 0xc9f, 0xcac,
0xcca, 0xc97, 0xccb, 0xc9f, 0xcac, 0xccd, 0xcb5, 0xcbe, 0xcb8, 0xcbf, 0xc95, 0xcc7,
0xcae, 0xccd, 0x200c, 0xcac, 0xccd, 0xcb0, 0xcbf, 0xca1, 0xccd, 0xc9c, 0xccd, 0x20,
0xcac, 0xcc7, 0xc95, 0xcbe, 0xc82, 0xcaa, 0xcca, 0x20, 0xc97, 0xccd, 0xcb0, 0xcbe,
0xc82, 0xca1, 0xcc6, 0xc95, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xc95, 0xca8, 0xccd, 0xc95,
0xccd, 0xcaf, 0xcbe, 0xcb0, 0xc95, 0xcbe, 0xcb8, 0xccd, 0xc95, 0xcc6, 0xcaf, 0xcc6,
0xca8, 0xcbf, 0xc95, 0xcc7, 0xcae, 0xca8, 0xccd, 0xc9a, 0xcbf, 0xc95, 0xcbe, 0xc97,
0xccb, 0xc9a, 0xcbf, 0xcb5, 0xcbe, 0xcb5, 0xcb8, 0xcbf, 0xcaf, 0xcc1, 0xca1, 0xcbe,
0xca1, 0xccd, 0x20, 0xcb5, 0xcbe, 0xcb0, 0xcc6, 0xc9d, 0xccd, 0xc95, 0xccb, 0xcb8,
0xccd, 0xc9f, 0xcbe, 0x20, 0xcb0, 0xcbf, 0xc95, 0xcbe, 0xc95, 0xccd, 0xcb0, 0xcc6,
0xcb8, 0xccd, 0xc9f, 0xca8, 0xccd, 0xc95, 0xccd, 0xcaf, 0xcc2, 0xc87, 0xcac, 0xcbe,
0xc95, 0xcc1, 0xcb0, 0xcbe, 0xc95, 0xcbe, 0xcb5, 0xccd, 0xca1, 0xca8, 0xccd, 0x200c,
0xcae, 0xcbe, 0xcb0, 0xccd, 0xc95, 0xccd, 0x200c, 0xcb6, 0xccd, 0xcb5, 0xcbe, 0xca8,
0xccd, 0xca1, 0xcbe, 0xcb8, 0xca8, 0xccd, 0xca1, 0xcbe, 0xcb8, 0xca8, 0xccd, 0x20,
0xc95, 0xccd, 0xcb0, 0xcc0, 0xc95, 0xccd, 0xca1, 0xcc6, 0xca8, 0xccd, 0xcb5, 0xcb0,
0xccd, 0xca1, 0xcc6, 0xc9f, 0xccd, 0xcb0, 0xcbe, 0xcaf, 0xccd, 0xc9f, 0xccd, 0xca1,
0xcca, 0xcae, 0xcbf, 0xca8, 0xcbf, 0xc95, 0xcbe, 0xc8e, 0xca1, 0xccd, 0xcae, 0xc82,
0xc9f, 0xca8, 0xccd, 0x200c, 0xc87, 0xcb0, 0xcc1, 0xca8, 0xcbf, 0xcaa, 0xcc6, 0xc8e,
0xcb2, 0xccd, 0x20, 0xcb8, 0xcbe, 0xcb2, 0xccd, 0xcb5, 0xca1, 0xccb, 0xcb0, 0xccd,
0xcab, 0xccb, 0xcb0, 0xccd, 0xc9f, 0xccd, 0x20, 0xca8, 0xcc6, 0xcb2, 0xccd, 0xcb8,
0xca8, 0xccd, 0xcab, 0xcca, 0xcb0, 0xccd, 0xc9f, 0xcb2, 0xcc6, 0xc9c, 0xcbe, 0xc97,
0xccd, 0xcb2, 0xcc7, 0xcb8, 0xccd, 0x20, 0xcac, 0xcc7, 0xc97, 0xcc2, 0xcb8, 0xccd,
0x20, 0xcac, 0xcc7, 0xc97, 0xccd, 0xcb0, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd,
0x20, 0xc9f, 0xcb0, 0xccd, 0xc95, 0xccd, 0xc97, 0xccd, 0xcb0, 0xcc6, 0xca8, 0xcbe,
0xca1, 0xc97, 0xccd, 0xcb5, 0xcbe, 0xca1, 0xcc6, 0xcb2, 0xccb, 0xcaa, 0xccd, 0xc97,
0xccd, 0xcb5, 0xcbe, 0xc9f, 0xcc6, 0xcae, 0xcbe, 0xcb2, 0xcbe, 0xc97, 0xcc1, 0xcaf,
0xcbe, 0xc95, 0xccd, 0xcb5, 0xcbf, 0xcb2, 0xccd, 0xc97, 0xcaf, 0xcbe, 0xca8, 0xcbe,
0xcb9, 0xccd, 0xcaf, 0xcbe, 0xcb2, 0xcbf, 0xcab, 0xcc6, 0xc95, 0xccd, 0xcb8, 0xccd,
0xcb9, 0xcb5, 0xcbe, 0xca8, 0xcbe, 0xcb9, 0xcb0, 0xccd, 0xcae, 0xcca, 0xcb8, 0xcbf,
0xcb2, 0xccd, 0xcb2, 0xccb, 0xc87, 0xc82, 0xca1, 0xcbf, 0xcaf, 0xcbe, 0xca8, 0xcbe,
0xcaa, 0xcca, 0xcb2, 0xcbf, 0xcb8, 0xccd, 0xca8, 0xcbe, 0xc95, 0xccd, 0xcb8, 0xccd,
0x2c, 0x20, 0xc87, 0xc82, 0xca1, 0xcbf, 0xcaf, 0xcbe, 0xca8, 0xcbe, 0xcae, 0xcb0,
0xcc6, 0xc82, 0xc97, 0xcca, 0x2c, 0x20, 0xc87, 0xc82, 0xca1, 0xcbf, 0xcaf, 0xcbe,
0xca8, 0xcbe, 0xcaa, 0xcc0, 0xc9f, 0xcb0, 0xccd, 0xcb8, 0xccd, 0x200c, 0xcac, 0xcb0,
0xccd, 0xc97, 0xccd, 0x2c, 0x20, 0xc87, 0xc82, 0xca1, 0xcbf, 0xcaf, 0xcbe, 0xca8,
0xcbe, 0xc9f, 0xcc6, 0xcb2, 0xccd, 0x20, 0xcb8, 0xcbf, 0xc9f, 0xcbf, 0x2c, 0x20,
0xc87, 0xc82, 0xca1, 0xcbf, 0xcaf, 0xcbe, 0xca8, 0xcbe, 0xcb5, 0xcc7, 0xcb5, 0xcbe,
0xcaf, 0xccd, 0x2c, 0x20, 0xc87, 0xc82, 0xca1, 0xcbf, 0xcaf, 0xcbe, 0xca8, 0xcbe,
0xcb5, 0xcbf, 0xca8, 0xccd, 0x200c, 0xcb8, 0xcc6, 0xca8, 0xccd, 0xca8, 0xcc0, 0xcb8,
0xccd, 0x2c, 0x20, 0xc87, 0xc82, 0xca1, 0xcbf, 0xcaf, 0xcbe, 0xca8, 0xcbe, 0xcb5,
0xcbf, 0xca8, 0xcbe, 0xcae, 0xccd, 0xcaf, 0xcbe, 0xc95, 0xccd, 0x2c, 0x20, 0xc87,
0xc82, 0xca1, 0xcbf, 0xcaf, 0xcbe, 0xca8, 0xcbe, 0xc87, 0xca8, 0xcc1, 0xcb5, 0xcbf,
0xc95, 0xccd, 0xc88, 0xc95, 0xccd, 0xcb5, 0xcbe, 0xcb2, 0xcbf, 0xc9f, 0xccd, 0xc9c,
0xcae, 0xcc8, 0xc95, 0xcbe, 0xc9c, 0xcc1, 0xca8, 0xcc7, 0xcb5, 0xccd, 0xcb2, 0xcc2,
0xcaf, 0xcc0, 0xcb8, 0xccd, 0x200c, 0xcb5, 0xcbf, 0xcb2, 0xccd, 0xcb2, 0xcc6, 0xcae,
0xcca, 0xc82, 0xc9f, 0xcbf, 0xcb8, 0xcc6, 0xcb2, 0xccd, 0xcb2, 0xccb, 0x2c, 0x20,
0xc95, 0xcc6, 0xc82, 0xc9f, 0xcc1, 0xc95, 0xcbf, 0xc95, 0xccd, 0xcb0, 0xcc6, 0xcb2,
0xcc6, 0xc82, 0xca1, 0xcbf, 0xc9c, 0xccd, 0xc95, 0xccd, 0xcb2, 0xcbe, 0x20, 0xcaa,
0xcc6, 0xc9c, 0xccd, 0xcb2, 0xcbf, 0xcae, 0xcbe, 0xcb2, 0xcbe, 0xcb8, 0xccd, 0x20,
0xc8f, 0xc82, 0xc9c, 0xcb2, 0xcc0, 0xcb8, 0xccd, 0xcb2, 0xccb, 0xcb5, 0xcb0, 0xccd,
0x20, 0xcaa, 0xccd, 0xcb0, 0xcbf, 0xca8, 0xccd, 0xcb8, 0xcb8, 0xccd, 0x20, 0xc95,
0xccd, 0xcb5, 0xcbe, 0xcb0, 0xccd, 0xc9f, 0xcb0, 0xccd, 0xcae, 0xcc6, 0xcb8, 0xcc0,
0xcaf, 0xcca, 0xcae, 0xca8, 0xcbe, 0xc97, 0xcc1, 0xcb5, 0xcbe, 0xcae, 0xca8, 0xcbe,
0xcb5, 0xccd, 0xcb8, 0xccd, 0xcae, 0xcb0, 0xcbf, 0xc97, 0xcca, 0xcae, 0xcbe, 0xcb0,
0xccd, 0xc9f, 0xcbf, 0xca8, 0xcbf, 0xc95, 0xccd, 0xcae, 0xc9f, 0xcbe, 0xcae, 0xccb,
0xcb0, 0xcb8, 0xccd, 0xcae, 0xc9c, 0xc9f, 0xccd, 0xcb2, 0xcbe, 0xca8, 0xccd, 0xcae,
0xcc6, 0xca8, 0xcca, 0xcae, 0xcbf, 0xca8, 0xcc0, 0xcae, 0xcc6, 0xcb0, 0xcbf, 0xca6,
0xcbe, 0xcae, 0xcc6, 0xc9f, 0xccd, 0xcb2, 0xcbe, 0xc95, 0xcbe, 0xc9f, 0xccd, 0xcb2,
0xcbe, 0xcae, 0xcc6, 0xc95, 0xccd, 0xcb8, 0xcbf, 0xc95, 0xccb, 0x20, 0xca8, 0xc97,
0xcb0, 0xcae, 0xcbf, 0xc95, 0xcc6, 0xcb2, 0xcbe, 0xca8, 0xccd, 0xcae, 0xcbe, 0xc82,
0xc95, 0xccd, 0xc9f, 0xca8, 0xccd, 0xcae, 0xcbe, 0xc82, 0xc9f, 0xcc6, 0xcb0, 0xccd,
0xcb0, 0xcbf, 0xcae, 0xcbe, 0xc82, 0xc9f, 0xcbf, 0xcb5, 0xcc0, 0xca1, 0xcbf, 0xcaf,
0xcca, 0xcae, 0xcbe, 0xc82, 0xc9f, 0xccd, 0x200c, 0xcb8, 0xcc6, 0xcb0, 0xcc7, 0xc9f,
0xccd, 0xca8, 0xcb8, 0xccd, 0xcb8, 0xccc, 0xcb5, 0xccd, 0xca8, 0xccd, 0xcaf, 0xcc2,
0xcaf, 0xcbe, 0xcb0, 0xccd, 0xc95, 0xccd, 0xca8, 0xccb, 0xcae, 0xccd, 0xca8, 0xcca,
0xcb0, 0xcca, 0xca8, 0xccd, 0xcb9, 0xcbe, 0xcac, 0xccd, 0xcaf, 0xcc2, 0xcb2, 0xcbe,
0x2c, 0x20, 0xc89, 0xca4, 0xccd, 0xca4, 0xcb0, 0x20, 0xca1, 0xc95, 0xcca, 0xc9f,
0xcb8, 0xcc6, 0xc82, 0xc9f, 0xcb0, 0xccd, 0x2c, 0x20, 0xc89, 0xca4, 0xccd, 0xca4,
0xcb0, 0x20, 0xca1, 0xc95, 0xcca, 0xc9f, 0xca8, 0xccd, 0xcaf, 0xcc2, 0x20, 0xcb8,
0xcb2, 0xcc7, 0xc82, 0x2c, 0x20, 0xc89, 0xca4, 0xccd, 0xca4, 0xcb0, 0x20, 0xca1,
0xc95, 0xcca, 0xc9f, 0xca8, 0xcc2, 0xc95, 0xccd, 0xc93, 0xc9c, 0xcbf, 0xca8, 0xcbe,
0xc97, 0xcbe, 0xcaa, 0xca8, 0xcbe, 0xcae, 0xcbe, 0xcaa, 0xcb0, 0xcae, 0xcbe, 0xcb0,
0xcbf, 0xcac, 0xccb, 0xcab, 0xcbf, 0xca8, 0xcbf, 0xc95, 0xccd, 0xcb8, 0xccd, 0xcaa,
0xccb, 0xcb0, 0xccd, 0xc9f, 0xccd, 0x2d, 0xc92, 0x2d, 0xcaa, 0xccd, 0xcb0, 0xcbf,
0xca8, 0xccd, 0xcb8, 0xccd, 0xcaa, 0xccb, 0xcb0, 0xccd, 0xc9f, 0xccd, 0x20, 0xc86,
0xcab, 0xccd, 0x20, 0xcb8, 0xccd, 0xcaa, 0xcc7, 0xca8, 0xccd, 0xcaa, 0xccb, 0xcb0,
0xccd, 0xc9f, 0xcca, 0x20, 0xcb5, 0xcc6, 0xcb2, 0xccd, 0xcb9, 0xccb, 0xcaa, 0xccd,
0xcaf, 0xcc1, 0xcb0, 0xccd, 0xc9f, 0xccb, 0x20, 0xcb0, 0xcbf, 0xc95, 0xcca, 0xcaa,
0xcc1, 0xc82, 0xca4, 0xcbe, 0x20, 0xc85, 0xcb0, 0xcc7, 0xca8, 0xcbe, 0xcb8, 0xccd,
0xcb0, 0xcbe, 0xc82, 0xc95, 0xcbf, 0xca8, 0xccd, 0x20, 0xc87, 0xca8, 0xccd, 0x200c,
0xcb2, 0xcc6, 0xc9f, 0xccd, 0xcb0, 0xcc6, 0xcb8, 0xcbf, 0xcab, 0xcbf, 0xcb0, 0xcc6,
0xc9c, 0xcbf, 0xca8, 0xcbe, 0xcb0, 0xcc6, 0xcb8, 0xcca, 0xcb2, 0xccd, 0xcaf, 0xcc2,
0xc9f, 0xccd, 0xcb0, 0xcbf, 0xcaf, 0xccb, 0x20, 0xcac, 0xccd, 0xcb0, 0xccd, 0xcaf,
0xcbe, 0xc82, 0xc95, 0xcca, 0xcb8, 0xcbe, 0xc82, 0xc9f, 0xcb0, 0xcc6, 0xcae, 0xccd,
0xcb8, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xc9f, 0xcbf, 0xcaf, 0xcbe, 0xc97, 0xcca, 0xcb8,
0xccd, 0xcaf, 0xcbe, 0xc82, 0xc9f, 0xccb, 0x20, 0xca1, 0xcca, 0xcae, 0xcbf, 0xc82,
0xc97, 0xcca, 0xcb8, 0xcbe, 0xcb5, 0xccd, 0x20, 0xcaa, 0xcbe, 0xcb2, 0xcca, 0xc87,
0xc9f, 0xccd, 0xc9f, 0xcca, 0xc95, 0xccd, 0xc95, 0xcca, 0xcb0, 0xccd, 0xc9f, 0xcc2,
0xcae, 0xcbf, 0xcaf, 0xcc8, 0xc9f, 0xccd, 0xcb8, 0xcbf, 0xca4, 0xccd, 0xc95, 0xcbe,
0xcb8, 0xcc7, 0xc82, 0xc9f, 0xccd, 0x20, 0xcac, 0xcbe, 0xcb0, 0xccd, 0xca4, 0xcc6,
0xcb2, 0xcc6, 0xcae, 0xcbf, 0xcb8, 0xcc7, 0xc82, 0xc9f, 0xccd, 0x20, 0xc9c, 0xcbe,
0xca8, 0xccd, 0xcb8, 0xccd, 0xcb8, 0xcc7, 0xc82, 0xc9f, 0xccd, 0x20, 0xc95, 0xcbf,
0xc9f, 0xccd, 0xcb8, 0xccd, 0xcb8, 0xcc7, 0xc82, 0xc9f, 0xccd, 0x20, 0xcb2, 0xcc2,
0xcb8, 0xcbf, 0xcaf, 0xcbe, 0xcb8, 0xcc7, 0xc82, 0xc9f, 0xccd, 0x20, 0xca5, 0xcbe,
0xcae, 0xcb8, 0xccd, 0xcb8, 0xcc7, 0xc82, 0xc9f, 0xccd, 0x20, 0xcb5, 0xcbf, 0xca8,
0xccd, 0xcb8, 0xcc6, 0xc82, 0xc9f, 0xccd, 0xcb8, 0xccd, 0xcb5, 0xcbf, 0xcab, 0xccd,
0xc9f, 0xccd, 0x20, 0xc95, 0xcb0, 0xcc6, 0xc82, 0xc9f, 0xccd, 0xca4, 0xcc6, 0xc97,
0xcc1, 0xcb8, 0xcbf, 0xc97, 0xcb2, 0xccd, 0xcaa, 0xcbe, 0xca5, 0xcc2, 0xcb2, 0xcc6,
0xca4, 0xcbf, 0xc9c, 0xccd, 0xcb5, 0xcbe, 0xca8, 0xcbe, 0xc9f, 0xcca, 0xcb0, 0xcca,
0xc82, 0xc9f, 0xccb, 0xc9f, 0xcbe, 0xcb0, 0xccd, 0xc9f, 0xcca, 0xcb2, 0xcbe, 0xcb5,
0xccd, 0xcaf, 0xcbe, 0xc82, 0xc95, 0xcca, 0xcb5, 0xcb0, 0xccd, 0x200c, 0xcb5, 0xcc8,
0xc9f, 0xccd, 0x200c, 0xcb9, 0xcbe, 0xcb0, 0xccd, 0xcb8, 0xccd, 0xcb5, 0xcbf, 0xca8,
0xccd, 0xca8, 0xcbf, 0xcaa, 0xcc6, 0xc97, 0xccd, 0xcaf, 0xcbe, 0xc95, 0xcc1, 0xc9f,
0xcbe, 0xc9f, 0xccd, 0xc95, 0xcc7, 0xcb8, 0xcbf, 0xca1, 0xcc7, 0xcb5, 0xcc0, 0xcb8,
0xccd, 0xca1, 0xcc1, 0xcae, 0xc82, 0xc9f, 0xccd, 0x20, 0xca1, 0xcbf, 0x20, 0xc89,
0xcb0, 0xccd, 0xcb5, 0xcbf, 0xcb2, 0xcc6, 0xcae, 0xccd, 0xcaf, 0xcbe, 0xc95, 0xccd,
0xcb5, 0xcc6, 0xcb0, 0xcbf, 0xcae, 0xcbe, 0xcb5, 0xccd, 0xcb8, 0xca8, 0xccd, 0xcae,
0xccd, 0xcaf, 0xcbe, 0xc95, 0xccd, 0x200d, 0xcae, 0xcc1, 0xcb0, 0xccd, 0xca1, 0xcca,
0xcaa, 0xcbe, 0xcb2, 0xccd, 0xcae, 0xcb0, 0xccd, 0xcb0, 0xcca, 0xca5, 0xcc7, 0xcb0,
0xcbe, 0xcb8, 0xccd, 0xcaf, 0xccb, 0xcb5, 0xcbe, 0xc9f, 0xccd, 0xcb0, 0xcbe, 0xcb2,
0xccd, 0xcb5, 0xccb, 0xcb8, 0xccd, 0xc9f, 0xccb, 0xc95, 0xccd, 0xcb2, 0xcbe, 0xc82,
0xc97, 0xccd, 0xcaf, 0xcc0, 0xcb0, 0xccd, 0xcac, 0xcc6, 0xca8, 0xccd, 0xc8f, 0xca1,
0xca8, 0xccd, 0xc85, 0xcb2, 0xccd, 0xcae, 0xcbe, 0xc9f, 0xcbf, 0xc85, 0xcae, 0xccd,
0xcae, 0xcbe, 0xca8, 0xccd, 0xc85, 0xca8, 0xca6, 0xccd, 0xcaf, 0xcb0, 0xccd, 0xc85,
0xc95, 0xccd, 0xca4, 0xcbe, 0xcb5, 0xccd, 0xc85, 0xc95, 0xccd, 0xc9f, 0xccb, 0xcac,
0xcc6, 0xc85, 0xcb6, 0xccd, 0xc97, 0xcac, 0xcbe, 0xca4, 0xccd, 0xc85, 0xc9f, 0xccd,
0xcb0, 0xcbe, 0xcb5, 0xccd, 0xcac, 0xcbe, 0xc97, 0xccd, 0xca6, 0xcbe, 0xca6, 0xccd,
0xcac, 0xcb9, 0xccd, 0xcb0, 0xcc7, 0xca8, 0xccd, 0xcac, 0xc95, 0xcc1, 0xcac, 0xccd,
0xcaf, 0xcbe, 0xc82, 0xc95, 0xcbe, 0xc95, 0xccd, 0xcac, 0xcbe, 0xcb0, 0xccd, 0xca8,
0xccc, 0xcb2, 0xccd, 0xcac, 0xcc0, 0xcb0, 0xc9f, 0xccd, 0xcac, 0xcbf, 0xcb6, 0xccd,
0xc95, 0xcc6, 0xc95, 0xccd, 0xcac, 0xccd, 0xcb0, 0xcc2, 0xca8, 0xcbf, 0xc9a, 0xcbf,
0xc9f, 0xc95, 0xcca, 0xcb2, 0xc82, 0xcac, 0xcca, 0xca1, 0xcae, 0xcbe, 0xcb8, 0xccd,
0xc95, 0xcb8, 0xccd, 0xca2, 0xcbe, 0xc95, 0xcbe, 0xca6, 0xcbf, 0xcb2, 0xcbf, 0xca6,
0xcc1, 0xcac, 0xcc8, 0xca6, 0xcc1, 0xcb6, 0xcbe, 0xc82, 0xcac, 0xcc6, 0xcab, 0xcae,
0xcbe, 0xc97, 0xcc1, 0xcb8, 0xccd, 0xca4, 0xcbe, 0xc97, 0xcbe, 0xc9c, 0xcbe, 0xcb9,
0xcc6, 0xcac, 0xccd, 0xcb0, 0xcbe, 0xca8, 0xccd, 0xcb9, 0xcca, 0x20, 0xc9a, 0xcbf,
0x20, 0xcae, 0xcbf, 0xca8, 0xccd, 0x20, 0xcb8, 0xcbf, 0xc9f, 0xcbf, 0xcb9, 0xcbe,
0xc82, 0xc97, 0xccd, 0x20, 0xc95, 0xcbe, 0xc82, 0xc97, 0xccd, 0xcb9, 0xccb, 0xcb5,
0xccd, 0xca1, 0xccd, 0xc87, 0xcb0, 0xc95, 0xcc1, 0xc9f, 0xccd, 0xcb8, 0xccd, 0xc95,
0xccd, 0xc9c, 0xc95, 0xcbe, 0xcb0, 0xccd, 0xca4, 0xcbe, 0xc9c, 0xcaf, 0xcaa, 0xcc1,
0xcb0, 0xc9c, 0xcc6, 0xcb0, 0xcc1, 0xcb8, 0xcb2, 0xcc6, 0xcae, 0xccd, 0xc95, 0xcbe,
0xcac, 0xcc2, 0xcb2, 0xccd, 0xc95, 0xc82, 0xc9a, 0xc9f, 0xccd, 0xc95, 0xccd, 0xc95,
0xcb0, 0xcbe, 0xc9a, 0xcbf, 0xc95, 0xca0, 0xccd, 0xcae, 0xc82, 0xca1, 0xcc1, 0xc96,
0xc82, 0xca1, 0xccd, 0xcaf, 0xcbe, 0xc97, 0xcbe, 0xc95, 0xcca, 0xcb2, 0xccd, 0xc95,
0xca4, 0xccd, 0xca4, 0xcbe, 0xc95, 0xccd, 0xcb0, 0xcbe, 0xcb8, 0xccd, 0xca8, 0xcca,
0xcaf, 0xcbe, 0xcb0, 0xccd, 0xcb8, 0xccd, 0xc95, 0xccd, 0xc95, 0xccc, 0xcb2, 0xcbe,
0x20, 0xcb2, 0xc82, 0xcaa, 0xcc1, 0xcb0, 0xc95, 0xcc1, 0xc9a, 0xcbf, 0xc82, 0xc97,
0xccd, 0xc95, 0xcc1, 0xcb5, 0xcc8, 0xca4, 0xccd, 0xcae, 0xc95, 0xcbe, 0xcb5, 0xcc1,
0xcae, 0xc97, 0xca6, 0xca8, 0xccd, 0x200c, 0xcae, 0xcc6, 0xc95, 0xcbe, 0xcb8, 0xccd,
0xcb8, 0xcb0, 0xccd, 0xcae, 0xca8, 0xcbf, 0xcb2, 0xcbe, 0xcae, 0xcb8, 0xccd, 0xc95,
0xc9f, 0xccd, 0xca8, 0xcbf, 0xc95, 0xccb, 0xcb8, 0xcbf, 0xcaf, 0xcbe, 0xca8, 0xccb,
0xcb5, 0xccb, 0xc95, 0xcc1, 0xc9c, 0xcc6, 0xca8, 0xcca, 0xcb5, 0xccb, 0xcb8, 0xcbf,
0xcac, 0xcbf, 0xcb8, 0xccd, 0xc95, 0xccd, 0xc92, 0xcae, 0xccd, 0xcb8, 0xccd, 0xc95,
0xccd, 0xc92, 0xcb0, 0xcb2, 0xccd, 0xca8, 0xccb, 0xcae, 0xccd, 0x20, 0xcaa, 0xcc6,
0xca8, 0xccd, 0xcaa, 0xcca, 0xc82, 0xc9f, 0xcbf, 0xcaf, 0xcbe, 0xca8, 0xc95, 0xccd,
0xcaa, 0xccd, 0xcaf, 0xcca, 0xc82, 0xc97, 0xccd, 0x200d, 0xcaf, 0xcbe, 0xc82, 0xc97,
0xccd, 0xc96, 0xca4, 0xcbe, 0xcb0, 0xccd, 0xc95, 0xcca, 0xcb8, 0xccd, 0xc9f, 0xca8,
0xcaf, 0xccd, 0xc95, 0xcbf, 0xc9c, 0xcc8, 0xcb2, 0xccb, 0xcb0, 0xccd, 0xca6, 0xcb0,
0xcbf, 0xcaf, 0xcbe, 0xca6, 0xccd, 0xcb8, 0xc96, 0xcbe, 0xcb2, 0xcbf, 0xca8, 0xccd,
0xcb8, 0xcae, 0xcb0, 0xc96, 0xc82, 0xca1, 0xcb8, 0xcbf, 0xcaf, 0xccb, 0xcb2, 0xccd,
0xcb6, 0xcbe, 0xc82, 0xc98, 0xcc8, 0xcb8, 0xcbf, 0xc82, 0xc97, 0xcaa, 0xcc1, 0xcb0,
0xccd, 0xcb8, 0xccd, 0xcb0, 0xcc7, 0xca6, 0xccd, 0xca8, 0xcc7, 0xc95, 0xccb, 0xcb2,
0xcc0, 0xcae, 0xcb8, 0xccd, 0xc95, 0xca4, 0xcc8, 0xcaa, 0xcc6, 0xca4, 0xcbe, 0xcb6,
0xccd, 0xc95, 0xcc6, 0xc82, 0xc9f, 0xccd, 0xc9f, 0xcbf, 0xcac, 0xcbf, 0xcb2, 0xcbf,
0xcb8, 0xcbf, 0xca4, 0xcc6, 0xcb9, 0xccd, 0xcb0, 0xcbe, 0xca8, 0xccd, 0xca5, 0xcbf,
0xc82, 0xcaa, 0xcc1, 0xc9f, 0xccb, 0xc95, 0xcbf, 0xcaf, 0xcca, 0xc9f, 0xccb, 0xcae,
0xccd, 0x200c, 0xcb8, 0xccd, 0xc95, 0xccd, 0xc89, 0xcb2, 0xcbe, 0xca8, 0xccd, 0x20,
0xcac, 0xca4, 0xcca, 0xcb0, 0xccd, 0xc89, 0xcb0, 0xcc1, 0xcae, 0xccd, 0x200d, 0xc95,
0xcbf, 0xc89, 0xcb8, 0xccd, 0xca4, 0xccd, 0x2d, 0xca8, 0xcc6, 0xcb0, 0xcbe, 0xcb5,
0xcbf, 0xcaf, 0xcc6, 0xc82, 0xc9f, 0xcbf, 0xcaf, 0xcbe, 0xca8, 0xccd, 0xcb5, 0xccd,
0xcb2, 0xcbe, 0xca6, 0xcbf, 0xcb5, 0xcca, 0xcb8, 0xccd, 0xc9f, 0xcbe, 0xc95, 0xccd,
0x200c, 0xcaf, 0xc95, 0xcc1, 0xc9f, 0xccd, 0xcb8, 0xccd, 0xc95, 0xccd, 0xcaf, 0xcbe,
0xc82, 0xc97, 0xcca, 0xca8, 0xccd, 0xc8e, 0xc95, 0xccd, 0xcaf, 0xcbe, 0xc9f, 0xcc6,
0xcb0, 0xcbf, 0xca8, 0xccd, 0xcac, 0xcb0, 0xccd, 0xc97, 0xccd, 0xcaf, 0xcc6, 0xcb0,
0xcc6, 0xcb5, 0xcbe, 0xca8, 0xccd, 0xc85, 0xc9d, 0xccb, 0xcb0, 0xcbf, 0xcb8, 0xccd,
0xcac, 0xcb0, 0xccd, 0xcae, 0xcc1, 0xca1, 0xcbe, 0xc95, 0xccd, 0xcaf, 0xcbe, 0xca8,
0xcb0, 0xcbf, 0xc95, 0xcc7, 0xcaa, 0xccd, 0x20, 0xcb5, 0xcc6, 0xcb0, 0xccd, 0xca1,
0xccd, 0xcab, 0xccd, 0xcaf, 0xcbe, 0xcb0, 0xcca, 0xcae, 0xcc6, 0xca1, 0xcbf, 0xcb0,
0xcbe, 0xcb0, 0xcc8, 0xc95, 0xccd, 0xc9c, 0xcbe, 0xcb5, 0xcbf, 0xc95, 0xccd, 0xca6,
0xc95, 0xccd, 0xcb7, 0xcbf, 0xca3, 0x20, 0xc9c, 0xcbe, 0xcb0, 0xccd, 0xc9c, 0xcbf,
0xcaf, 0xcbe, 0xcb8, 0xcc7, 0xc82, 0xc9f, 0xccd, 0x20, 0xcb9, 0xcc6, 0xcb2, 0xcc6,
0xca8, 0xcbe, 0xcb8, 0xccd, 0xc9f, 0xccd, 0xcaf, 0xcbe, 0xca8, 0xccd, 0xcb2, 0xcbf,
0xc85, 0xca1, 0xcbf, 0xcb2, 0xcc7, 0xca1, 0xccd, 0xcac, 0xccd, 0xcb0, 0xcbf, 0xcb8,
0xccd, 0xcac, 0xcc7, 0xca8, 0xccd, 0x200c, 0xcac, 0xccd, 0xcb0, 0xcca, 0xc95, 0xca8,
0xccd, 0x20, 0xcb9, 0xcbf, 0xcb2, 0xccd, 0xca1, 0xcbe, 0xcb0, 0xccd, 0xcb5, 0xcbf,
0xca8, 0xccd, 0xcaf, 0xcc1, 0xc95, 0xccd, 0xcb2, 0xcbe, 0xcb9, 0xccb, 0xcac, 0xcbe,
0xcb0, 0xccd, 0xc9f, 0xccd, 0x200c, 0xcb2, 0xcbf, 0xc82, 0xca1, 0xcbf, 0xcae, 0xcbe,
0xca8, 0xccd, 0xcb2, 0xcbe, 0xcb0, 0xccd, 0xca1, 0xccd, 0x20, 0xcb9, 0xccb, 0xcb5,
0xccd, 0xcae, 0xcc6, 0xcb2, 0xccd, 0xcac, 0xcb0, 0xccd, 0xca8, 0xccd, 0xcaa, 0xcb0,
0xccd, 0xca5, 0xccd, 0xcb8, 0xcbf, 0xca1, 0xccd, 0xca8, 0xcbf, 0xc86, 0xcae, 0xccd,
0x200c, 0xcb8, 0xccd, 0xc9f, 0xcc6, 0xcb0, 0xccd, 0x200c, 0xca1, 0xccd, 0xcaf, 0xcbe,
0xc82, 0xc85, 0xc82, 0xca1, 0xccb, 0xcb0, 0xcbe, 0xc86, 0xcb8, 0xccd, 0xc9f, 0xccd,
0xcb0, 0xcbe, 0xc96, 0xcbe, 0xca8, 0xccd, 0xc85, 0xca5, 0xcc6, 0xca8, 0xccd, 0xcb8,
0xccd, 0xcac, 0xcc6, 0xcb2, 0xccd, 0x200c, 0xc97, 0xccd, 0xcb0, 0xcc7, 0xca1, 0xccd,
0xcac, 0xcb0, 0xccd, 0xcb2, 0xcbf, 0xca8, 0xccd, 0xcac, 0xccd, 0xcb0, 0xcbe, 0xc9f,
0xcbf, 0xcb8, 0xccd, 0xcb2, 0xcbe, 0xcb5, 0xcac, 0xccd, 0xcb0, 0xcb8, 0xccd, 0xcb8,
0xcc6, 0xcb2, 0xccd, 0xcb8, 0xccd, 0xcac, 0xcc1, 0xc95, 0xcbe, 0xcb0, 0xcc6, 0xcb8,
0xccd, 0xc9f, 0xccd, 0xcac, 0xcc1, 0xca1, 0xcbe, 0xcaa, 0xcc6, 0xcb8, 0xccd, 0xc9f,
0xccd, 0xcac, 0xcc1, 0xcb8, 0xcbf, 0xc82, 0xc9c, 0xcc6, 0xca8, 0xccd, 0xc9a, 0xcbf,
0xcb8, 0xcbf, 0xca8, 0xcbe, 0xcb5, 0xccd, 0xc95, 0xccb, 0xcaa, 0xcc6, 0xca8, 0xccd,
0x200c, 0xcb9, 0xcc7, 0xc97, 0xca8, 0xccd, 0xca1, 0xcc1, 0xcac, 0xccd, 0xcb2, 0xcbf,
0xca8, 0xccd, 0xc9c, 0xcbf, 0xcac, 0xccd, 0xcb0, 0xcbe, 0xcb2, 0xccd, 0xc9f, 0xcb0,
0xccd, 0xc97, 0xccd, 0xcaf, 0xcc1, 0xcb0, 0xccd, 0xca8, 0xccd, 0x200d, 0xcb8, 0xcc6,
0xcb9, 0xcc6, 0xcb2, 0xccd, 0xcb8, 0xcbf, 0xc82, 0xc95, 0xcbf, 0xc90, 0xcb2, 0xccd,
0x20, 0xc86, 0xcab, 0xccd, 0x20, 0xcae, 0xccd, 0xcaf, 0xcbe, 0xca8, 0xccd, 0xc87,
0xcb8, 0xccd, 0xca4, 0xcbe, 0xca8, 0xccd, 0x200c, 0xcac, 0xcc1, 0xcb2, 0xccd, 0xc9c,
0xcc6, 0xcb0, 0xccd, 0xcb8, 0xcbf, 0xc95, 0xcb2, 0xcbf, 0xca8, 0xcbf, 0xca8, 0xccd,
0x200c, 0xc97, 0xccd, 0xcb0, 0xcbe, 0xca1, 0xccd, 0xc95, 0xcbf, 0xcb0, 0xcbe, 0xcb5,
0xccd, 0xc95, 0xcbf, 0xcb5, 0xccd, 0xcb2, 0xcbf, 0xcb8, 0xccd, 0xcac, 0xca8, 0xccd,
0xcb2, 0xccd, 0xcaf, 0xcc2, 0xcac, 0xccd, 0xcb2, 0xccd, 0xcaf, 0xcbe, 0xca8, 0xcbe,
0xcb2, 0xc82, 0xca1, 0xca8, 0xccd, 0xcb2, 0xc95, 0xccd, 0xcb8, 0xcc6, 0xc82, 0xcac,
0xcb0, 0xccd, 0xc97, 0xccd, 0xcae, 0xccd, 0xcaf, 0xcbe, 0xca1, 0xccd, 0xcb0, 0xcbf,
0xca1, 0xccd, 0xcae, 0xcbe, 0xcb2, 0xccd, 0xc9f, 0xcbe, 0xcae, 0xcb0, 0xcbf, 0xcb9,
0xcae, 0xccd, 0xca8, 0xccd, 0xcae, 0xcbf, 0xca8, 0xccd, 0x200c, 0xcb8, 0xccd, 0xc95,
0xccd, 0xcae, 0xcca, 0xca8, 0xcbe, 0xc95, 0xcca, 0xcae, 0xcbe, 0xcb8, 0xccd, 0xc95,
0xccb, 0xc93, 0xcb8, 0xccd, 0xcb2, 0xccb, 0xcaa, 0xccd, 0xcaf, 0xcbe, 0xcb0, 0xcbf,
0xcb8, 0xccd, 0xcaa, 0xcca, 0xca1, 0xccd, 0xc97, 0xcca, 0xcb0, 0xcbf, 0xc95, 0xcaa,
0xccd, 0xcb0, 0xcbe, 0xc97, 0xccd, 0xcb5, 0xcc6, 0xcb0, 0xcbf, 0xc97, 0xcbe, 0xcb0,
0xccb, 0xcae, 0xccd, 0xcb8, 0xccd, 0xcaf, 0xcbe, 0xca8, 0xccd, 0x20, 0xcae, 0xcb0,
0xcc0, 0xca8, 0xcca, 0xcb8, 0xcb0, 0xcaf, 0xcc7, 0xcb5, 0xcca, 0xcb8, 0xcb0, 0xcbe,
0xc9f, 0xccb, 0xcb5, 0xccd, 0xcb8, 0xcbf, 0xcae, 0xccd, 0xcab, 0xcc6, 0xcb0, 0xccb,
0xcaa, 0xccb, 0xcb2, 0xccd, 0xcb8, 0xccd, 0xc95, 0xcbe, 0xcaa, 0xcbf, 0xcaf, 0xcbe,
0xcb8, 0xccb, 0xcab, 0xcbf, 0xcaf, 0xcbe, 0xcb8, 0xccd, 0xc9f, 0xcbe, 0xc95, 0xccd,
0x200d, 0xcb9, 0xccb, 0xcae, 0xccd, 0xc9f, 0xccd, 0xcaf, 0xcbe, 0xcb2, 0xcbf, 0xca8,
0xccd, 0xc9f, 0xcbf, 0xcb0, 0xcbe, 0xca8, 0xcc6, 0xc89, 0xcb2, 0xccd, 0xcaf, 0xcbe,
0xca8, 0xcca, 0xcb5, 0xcb8, 0xccd, 0xc95, 0xccd, 0xcb5, 0xca1, 0xcc2, 0xc9c, 0xccd,
0xcb5, 0xccd, 0xcaf, 0xcbe, 0xc9f, 0xcbf, 0xc95, 0xca8, 0xccd, 0xcb5, 0xcbf, 0xcaf,
0xcc6, 0xca8, 0xccd, 0xca8, 0xcbe, 0xcb5, 0xcbf, 0xcb2, 0xccd, 0xca8, 0xcbf, 0xcaf,
0xcb8, 0xccd, 0xcb5, 0xcca, 0xcb2, 0xccd, 0xc97, 0xcca, 0xc97, 0xccd, 0xcb0, 0xcbe,
0xca1, 0xccd, 0xcb5, 0xcbe, 0xcb0, 0xccd, 0xcb8, 0xcbe, 0xc9c, 0xcbe, 0xc97, 0xccd,
0xcb0, 0xcc6, 0xcac, 0xccd, 0x200c, 0xc9c, 0xccd, 0xcaf, 0xcc2, 0xcb0, 0xcbf, 0xc95,
0xccd, 0xc85, 0xc82, 0xc9f, 0xcbe, 0xca8, 0xca8, 0xcbe, 0xcb0, 0xcbf, 0xcb5, 0xcca,
0xc9a, 0xcbe, 0xc97, 0xcca, 0xcb8, 0xccd, 0xc95, 0xccd, 0xcb0, 0xcbf, 0xcb8, 0xccd,
0xcae, 0xcb8, 0xccd, 0xc95, 0xcca, 0xc95, 0xccb, 0xcb8, 0xccd, 0xc95, 0xcca, 0xcae,
0xccb, 0xcb0, 0xcca, 0xc95, 0xcc6, 0xcb0, 0xccd, 0xc97, 0xccd, 0xcaf, 0xcc1, 0xcb2,
0xcc6, 0xca8, 0xccd, 0xcae, 0xcbe, 0xcb9, 0xcc6, 0xcae, 0xcbe, 0xcb2, 0xccd, 0xca1,
0xcc0, 0xcb5, 0xccd, 0xcb8, 0xccd, 0xcae, 0xcbe, 0xcb0, 0xcbf, 0xcb7, 0xcb8, 0xccd,
0xcae, 0xcaf, 0xcca, 0xc9f, 0xccd, 0xc9f, 0xcc6, 0xcb0, 0xcc0, 0xcaf, 0xcc2, 0xca8,
0xcbf, 0xcaf, 0xca8, 0xccd, 0xc85, 0xcaa, 0xcbf, 0xcaf, 0xcbe, 0xc86, 0xc95, 0xccd,
0x20, 0xcb2, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd, 0xcac, 0xccc, 0xc97, 0xcc6,
0xca8, 0xccd, 0x200d, 0xcb5, 0xcbf, 0xcb2, 0xccd, 0xcb2, 0xcc6, 0xc9a, 0xca5, 0xcbe,
0xcae, 0xccd, 0xc9a, 0xcc1, 0xc95, 0xccd, 0xc88, 0xcb8, 0xccd, 0xc9f, 0xcb0, 0xccd,
0xc87, 0xcab, 0xcc7, 0xc9f, 0xccd, 0xcab, 0xc95, 0xccb, 0xcab, 0xccb, 0xcab, 0xcbf,
0xc9c, 0xcbf, 0xcab, 0xcc1, 0xca8, 0xcbe, 0xcab, 0xcc1, 0xc9f, 0xcbf, 0xc97, 0xcb2,
0xcbe, 0xcaa, 0xc97, 0xccb, 0xcb8, 0xccd, 0xc97, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xcac,
0xcbf, 0xcaf, 0xcb0, 0xccd, 0xc97, 0xccd, 0xcb5, 0xcbe, 0xca1, 0xcb2, 0xccd, 0xc95,
0xcc6, 0xca8, 0xcbe, 0xcb2, 0xccd, 0xc97, 0xccd, 0xcb5, 0xcbe, 0xcae, 0xccd, 0xcb9,
0xcca, 0xca8, 0xcb2, 0xcc1, 0xcb2, 0xcc1, 0xc8e, 0xc82, 0xca1, 0xcb0, 0xccd, 0xcac,
0xcb0, 0xcbf, 0xc95, 0xcbf, 0xcb0, 0xcbf, 0xca4, 0xcbf, 0xcae, 0xca4, 0xcbf, 0xc95,
0xccb, 0xcb8, 0xccd, 0xcb0, 0xcc6, 0xc95, 0xccd, 0xcb5, 0xcbe, 0xc9c, 0xcb2, 0xcc7,
0xca8, 0xccd, 0xcae, 0xc9c, 0xcc1, 0xcb0, 0xcca, 0xcae, 0xcbe, 0xcb0, 0xccd, 0xc95,
0xccd, 0xcaf, 0xcc2, 0xcb8, 0xcb8, 0xccd, 0xcae, 0xcbf, 0xca1, 0xccd, 0x200c, 0xcb5,
0xcc7, 0xca8, 0xccc, 0xcb0, 0xcc1, 0xca8, 0xcbf, 0xcaf, 0xcc1, 0xca8, 0xcbe, 0xcb0,
0xccd, 0xcab, 0xccb, 0xc95, 0xccd, 0xca8, 0xccd, 0xcaf, 0xcc2, 0xcae, 0xcbf, 0xcaf,
0xcbe, 0xcaa, 0xcbe, 0xc97, 0xcca, 0x20, 0xcaa, 0xcbe, 0xc97, 0xcca, 0xcaa, 0xcbe,
0xcb2, 0xcbe, 0xcb5, 0xccd, 0xcaa, 0xcbf, 0xc9f, 0xccd, 0x200c, 0xc95, 0xcc8, 0xcb0,
0xccd, 0xca8, 0xccd, 0xcaa, 0xccb, 0xca8, 0xccd, 0x200c, 0xcaa, 0xcc6, 0xcaa, 0xccb,
0xcb0, 0xccd, 0xc9f, 0xccd, 0x20, 0xcae, 0xccb, 0xcb0, 0xcbf, 0xcb8, 0xccd, 0x200c,
0xcac, 0xcc8, 0xcb0, 0xcb0, 0xccb, 0xc9f, 0xccb, 0xc82, 0xc97, 0xcbe, 0xcb8, 0xcc8,
0xcaa, 0xca8, 0xccd, 0xca4, 0xcb9, 0xcbf, 0xca4, 0xcbf, 0xca4, 0xcb0, 0xcbe, 0xcb5,
0xcbe, 0xc9f, 0xcca, 0xc82, 0xc97, 0xcbe, 0xc9f, 0xcbe, 0xcaa, 0xcc1, 0xcb5, 0xcc7,
0xc95, 0xccd, 0xcb5, 0xccd, 0xcaf, 0xcbe, 0xcb2, 0xcbf, 0xcb8, 0xccd, 0x639, 0x627,
0x628, 0x650, 0x62f, 0x62c, 0x627, 0x646, 0x627, 0x620, 0x6a9, 0x631, 0x627, 0x639,
0x62f, 0x6cc, 0x633, 0x20, 0x627, 0x628, 0x627, 0x628, 0x627, 0x627, 0x64e, 0x644,
0x62c, 0x6cc, 0x656, 0x631, 0x650, 0x6cc, 0x627, 0x627, 0x64e, 0x633, 0x645, 0x6cc,
0x631, 0x627, 0x628, 0x645, 0x627, 0x6a9, 0x648, 0x628, 0x620, 0x646, 0x6af, 0x648,
0x657, 0x6cc, 0x6cc, 0x628, 0x64e, 0x646, 0x62c, 0x648, 0x657, 0x644, 0x628, 0x650,
0x633, 0x627, 0x624, 0x6ba, 0x628, 0x644, 0x627, 0x646, 0x62a, 0x64e, 0x6cc, 0x631,
0x628, 0x631, 0x627, 0x632, 0x627, 0x648, 0x650, 0x644, 0x628, 0x64f, 0x62c, 0x64f,
0x645, 0x628, 0x64f, 0x631, 0x627, 0x6a9, 0x64e, 0x6cc, 0x631, 0x648, 0x6a9, 0x620,
0x633, 0x627, 0x628, 0x644, 0x620, 0x646, 0x6a9, 0x627, 0x6a9, 0x6cc, 0x648, 0x657,
0x679, 0x627, 0x6a9, 0x648, 0x646, 0x627, 0x6a9, 0x631, 0x6cc, 0x62f, 0x64e, 0x6a9,
0x627, 0x631, 0x62f, 0x627, 0x631, 0x627, 0x644, 0x633, 0x644, 0x627, 0x645, 0x688,
0x650, 0x62c, 0x6cc, 0x628, 0x648, 0x679, 0x6cc, 0x688, 0x648, 0x648, 0x644, 0x627,
0x627, 0x644, 0x20, 0x639, 0x6cc, 0x648, 0x646, 0x641, 0x631, 0x6cc, 0x20, 0x679,
0x627, 0x648, 0x64f, 0x646, 0x6af, 0x620, 0x628, 0x648, 0x631, 0x648, 0x646, 0x6c1,
0x64e, 0x631, 0x627, 0x631, 0x6cc, 0x6d2, 0x62c, 0x648, 0x6c1, 0x627, 0x646, 0x633,
0x628, 0x631, 0x6af, 0x6a9, 0x64e, 0x645, 0x67e, 0x627, 0x644, 0x627, 0x6a9, 0x6be,
0x627, 0x631, 0x62a, 0x648, 0x645, 0x6a9, 0x650, 0x6af, 0x627, 0x644, 0x6cc, 0x6a9,
0x650, 0x646, 0x634, 0x627, 0x633, 0x627, 0x644, 0x6cc, 0x6af, 0x648, 0x633, 0x644,
0x650, 0x628, 0x64e, 0x631, 0x648, 0x650, 0x644, 0x644, 0x6cc, 0x648, 0x627, 0x646,
0x688, 0x627, 0x644, 0x648, 0x628, 0x645, 0x20, 0x628, 0x627, 0x634, 0x6cc, 0x644,
0x648, 0x633, 0x627, 0x6a9, 0x627, 0x645, 0x64e, 0x67e, 0x648, 0x657, 0x679, 0x648,
0x645, 0x64e, 0x633, 0x6cc, 0x631, 0x648, 0x657, 0x645, 0x627, 0x628, 0x6cc, 0x646,
0x645, 0x648, 0x6af, 0x627, 0x62f, 0x650, 0x634, 0x648, 0x657, 0x645, 0x648, 0x646,
0x631, 0x648, 0x648, 0x650, 0x6cc, 0x627, 0x646, 0x6cc, 0x631, 0x648, 0x628, 0x6cc,
0x62c, 0x64e, 0x645, 0x6cc, 0x656, 0x646, 0x627, 0x646, 0x64e, 0x6cc, 0x645, 0x6cc,
0x6d2, 0x646, 0x648, 0x648, 0x627, 0x6a9, 0x686, 0x6be, 0x648, 0x62a, 0x627, 0x648,
0x622, 0x6af, 0x62f, 0x648, 0x6af, 0x648, 0x67e, 0x648, 0x631, 0x679, 0x648, 0x20,
0x646, 0x648, 0x648, 0x648, 0x633, 0x627, 0x648, 0x20, 0x679, 0x648, 0x645, 0x62a,
0x631, 0x67e, 0x648, 0x644, 0x6cc, 0x679, 0x648, 0x657, 0x646, 0x650, 0x633, 0x648,
0x650, 0x646, 0x688, 0x6c1, 0x648, 0x6a9, 0x627, 0x62f, 0x6a9, 0x627, 0x64e, 0x646,
0x6a9, 0x648, 0x631, 0x627, 0x62c, 0x627, 0x620, 0x646, 0x6af, 0x650, 0x648, 0x644,
0x627, 0x627, 0x620, 0x646, 0x679, 0x650, 0x6af, 0x64f, 0x648, 0x627, 0x627, 0x620,
0x631, 0x6cc, 0x6af, 0x64f, 0x648, 0x6cc, 0x646, 0x627, 0x628, 0x6cc, 0x648, 0x657,
0x646, 0x64e, 0x633, 0x20, 0x622, 0x6cc, 0x631, 0x64e, 0x633, 0x6a9, 0x6cc, 0x679,
0x627, 0x645, 0x627, 0x631, 0x6a9, 0x627, 0x6a9, 0x648, 0x631, 0x691, 0x648, 0x628,
0x627, 0x62c, 0x64f, 0x62c, 0x648, 0x6cc, 0x6d2, 0x644, 0x627, 0x20, 0x631, 0x650,
0x6cc, 0x648, 0x62c, 0x627, 0x645, 0x620, 0x646, 0x691, 0x648, 0x632, 0x627, 0x631,
0x650, 0x6cc, 0x648, 0x20, 0x6af, 0x627, 0x644, 0x6cc, 0x6af, 0x648, 0x633, 0x633,
0x627, 0x644, 0x679, 0x627, 0x633, 0x6cc, 0x646, 0x20, 0x62c, 0x64f, 0x648, 0x627,
0x6ba, 0x633, 0x6cc, 0x646, 0x20, 0x644, 0x648, 0x657, 0x6cc, 0x650, 0x633, 0x679,
0x648, 0x6a9, 0x648, 0x657, 0x645, 0x64e, 0x646, 0x627, 0x64f, 0x634, 0x64f, 0x648,
0x657, 0x627, 0x6cc, 0x6cc, 0x627, 0x627, 0x64e, 0x631, 0x648, 0x657, 0x628, 0x627,
0x627, 0x64e, 0x633, 0x64f, 0x646, 0x686, 0x650, 0x6cc, 0x64e, 0x646, 0x627, 0x679,
0x6cc, 0x20, 0x6a9, 0x648, 0x6a9, 0x646, 0x64d, 0x628, 0x64e, 0x6c1, 0x650, 0x6cc,
0x627, 0x628, 0x627, 0x6c1, 0x6cc, 0x627, 0x20, 0x628, 0x646, 0x62f, 0x627, 0x631,
0x633, 0x628, 0x627, 0x631, 0x628, 0x627, 0x688, 0x648, 0x633, 0x628, 0x620, 0x644,
0x620, 0x645, 0x628, 0x620, 0x644, 0x6cc, 0x656, 0x632, 0x628, 0x644, 0x627, 0x646,
0x6a9, 0x20, 0x633, 0x620, 0x628, 0x644, 0x64e, 0x646, 0x628, 0x648, 0x622, 0x20,
0x648, 0x650, 0x633, 0x679, 0x627, 0x628, 0x648, 0x6af, 0x648, 0x679, 0x627, 0x628,
0x648, 0x6cc, 0x650, 0x633, 0x6a9, 0x6cc, 0x645, 0x628, 0x631, 0x650, 0x62c, 0x20,
0x62e, 0x644, 0x6cc, 0x656, 0x62c, 0x6a9, 0x620, 0x645, 0x67e, 0x648, 0x20, 0x6af,
0x631, 0x6cc, 0x646, 0x691, 0x6a9, 0x6cc, 0x646, 0x6a9, 0x64e, 0x646, 0x6a9, 0x6cc,
0x631, 0x64e, 0x6a9, 0x64e, 0x633, 0x6a9, 0x64e, 0x6cc, 0x6cc, 0x646, 0x6a9, 0x6cc,
0x645, 0x64e, 0x646, 0x634, 0x650, 0x6a9, 0x627, 0x6af, 0x648, 0x686, 0x650, 0x6c1,
0x64f, 0x648, 0x627, 0x20, 0x6c1, 0x64f, 0x648, 0x627, 0x6a9, 0x648, 0x633, 0x679,
0x627, 0x20, 0x631, 0x6cc, 0x656, 0x6a9, 0x627, 0x6a9, 0x631, 0x6cc, 0x633, 0x679,
0x646, 0x6a9, 0x648, 0x657, 0x6cc, 0x627, 0x628, 0x627, 0x6a9, 0x6cc, 0x648, 0x657,
0x631, 0x627, 0x6a9, 0x627, 0x627, 0x648, 0x688, 0x646, 0x645, 0x627, 0x631, 0x6a9,
0x20, 0x634, 0x627, 0x648, 0x646, 0x688, 0x627, 0x648, 0x633, 0x646, 0x688, 0x648,
0x627, 0x633, 0x64e, 0x646, 0x20, 0x6a9, 0x631, 0x6cc, 0x656, 0x6a9, 0x688, 0x6cc,
0x646, 0x648, 0x64e, 0x631, 0x688, 0x6cc, 0x679, 0x631, 0x627, 0x6cc, 0x650, 0x679,
0x688, 0x648, 0x645, 0x650, 0x646, 0x650, 0x6a9, 0x627, 0x627, 0x620, 0x688, 0x645,
0x64e, 0x646, 0x679, 0x64e, 0x646, 0x627, 0x6cc, 0x656, 0x631, 0x648, 0x657, 0x646,
0x6cc, 0x67e, 0x627, 0x6cc, 0x644, 0x20, 0x633, 0x64e, 0x644, 0x648, 0x6cc, 0x62f,
0x64e, 0x631, 0x641, 0x648, 0x631, 0x679, 0x20, 0x646, 0x6cc, 0x644, 0x633, 0x646,
0x641, 0x648, 0x631, 0x679, 0x6cc, 0x644, 0x6cc, 0x632, 0x627, 0x6af, 0x644, 0x6cc,
0x633, 0x20, 0x62e, 0x644, 0x6cc, 0x656, 0x62c, 0x6af, 0x648, 0x657, 0x633, 0x20,
0x62e, 0x644, 0x6cc, 0x656, 0x62c, 0x6af, 0x631, 0x6cc, 0x646, 0x688, 0x20, 0x62a,
0x654, 0x631, 0x6a9, 0x6af, 0x631, 0x6cc, 0x646, 0x627, 0x688, 0x627, 0x6af, 0x648,
0x691, 0x644, 0x648, 0x67e, 0x6af, 0x648, 0x627, 0x679, 0x6cc, 0x645, 0x627, 0x644,
0x627, 0x6af, 0x648, 0x627, 0x6cc, 0x627, 0x6a9, 0x650, 0x648, 0x64e, 0x644, 0x6af,
0x64f, 0x6cc, 0x627, 0x646, 0x627, 0x62d, 0x6cc, 0x644, 0x650, 0x641, 0x620, 0x6a9,
0x633, 0x62d, 0x648, 0x627, 0x646, 0x627, 0x6c2, 0x631, 0x645, 0x648, 0x633, 0x650,
0x644, 0x648, 0x627, 0x650, 0x646, 0x688, 0x6cc, 0x64e, 0x646, 0x20, 0x67e, 0x648,
0x644, 0x650, 0x633, 0x646, 0x648, 0x6a9, 0x633, 0x645, 0x6cc, 0x631, 0x646, 0x6af,
0x648, 0x60c, 0x20, 0x627, 0x646, 0x688, 0x6cc, 0x627, 0x646, 0x627, 0x67e, 0x650,
0x679, 0x633, 0x20, 0x628, 0x654, 0x631, 0x6af, 0x60c, 0x20, 0x627, 0x646, 0x688,
0x6cc, 0x627, 0x646, 0x627, 0x679, 0x6cc, 0x644, 0x20, 0x633, 0x650, 0x679, 0x6cc,
0x60c, 0x20, 0x627, 0x646, 0x688, 0x6cc, 0x627, 0x646, 0x627, 0x648, 0x6cc, 0x648,
0x6cc, 0x6d2, 0x60c, 0x20, 0x627, 0x646, 0x688, 0x6cc, 0x627, 0x646, 0x627, 0x648,
0x650, 0x646, 0x633, 0x6cc, 0x646, 0x6cc, 0x633, 0x60c, 0x20, 0x627, 0x646, 0x688,
0x6cc, 0x627, 0x646, 0x627, 0x648, 0x650, 0x646, 0x6cc, 0x645, 0x6cc, 0x6a9, 0x60c,
0x20, 0x627, 0x646, 0x688, 0x6cc, 0x627, 0x646, 0x627, 0x627, 0x650, 0x646, 0x648,
0x657, 0x648, 0x650, 0x6a9, 0x627, 0x650, 0x642, 0x627, 0x644, 0x6cc, 0x648, 0x657,
0x6cc, 0x650, 0x62a, 0x62c, 0x64e, 0x645, 0x64e, 0x6cc, 0x6a9, 0x627, 0x62c, 0x648,
0x657, 0x646, 0x6cc, 0x644, 0x648, 0x657, 0x6cc, 0x650, 0x633, 0x20, 0x648, 0x650,
0x644, 0x645, 0x648, 0x646, 0x679, 0x650, 0x633, 0x6cc, 0x644, 0x648, 0x60c, 0x20,
0x6a9, 0x6cc, 0x646, 0x679, 0x648, 0x6a9, 0x6cc, 0x6a9, 0x631, 0x627, 0x644, 0x6cc,
0x646, 0x688, 0x650, 0x6a9, 0x644, 0x627, 0x20, 0x67e, 0x627, 0x632, 0x644, 0x650,
0x645, 0x627, 0x644, 0x627, 0x633, 0x20, 0x627, 0x6cc, 0x646, 0x62c, 0x655, 0x644,
0x632, 0x644, 0x648, 0x648, 0x631, 0x20, 0x67e, 0x631, 0x646, 0x633, 0x20, 0x6a9,
0x648, 0x627, 0x631, 0x679, 0x631, 0x645, 0x6cc, 0x633, 0x650, 0x6cc, 0x648, 0x648,
0x645, 0x64e, 0x646, 0x627, 0x6af, 0x648, 0x627, 0x645, 0x64e, 0x646, 0x648, 0x633,
0x645, 0x6cc, 0x631, 0x6cc, 0x6af, 0x648, 0x679, 0x645, 0x627, 0x631, 0x679, 0x650,
0x646, 0x650, 0x6a9, 0x645, 0x6cc, 0x679, 0x627, 0x645, 0x648, 0x631, 0x648, 0x633,
0x645, 0x64e, 0x632, 0x64e, 0x679, 0x644, 0x627, 0x646, 0x645, 0x6cc, 0x646, 0x648,
0x645, 0x650, 0x646, 0x6cc, 0x645, 0x6cc, 0x631, 0x650, 0x688, 0x627, 0x645, 0x6cc,
0x679, 0x644, 0x627, 0x20, 0x6a9, 0x627, 0x679, 0x644, 0x627, 0x645, 0x6cc, 0x6a9,
0x633, 0x650, 0x6a9, 0x648, 0x20, 0x633, 0x650, 0x679, 0x6cc, 0x645, 0x650, 0x6a9,
0x6cc, 0x656, 0x644, 0x64e, 0x646, 0x645, 0x648, 0x646, 0x6a9, 0x679, 0x655, 0x646,
0x645, 0x648, 0x679, 0x6cc, 0x631, 0x6cc, 0x645, 0x648, 0x646, 0x679, 0x6cc, 0x648,
0x650, 0x688, 0x6cc, 0x648, 0x645, 0x648, 0x646, 0x698, 0x6cc, 0x631, 0x627, 0x62a,
0x646, 0x633, 0x627, 0x624, 0x6ba, 0x646, 0x650, 0x648, 0x20, 0x6cc, 0x627, 0x631,
0x6a9, 0x646, 0x648, 0x631, 0x648, 0x646, 0x6c1, 0x627, 0x628, 0x6cc, 0x648, 0x644,
0x627, 0x6c1, 0x60c, 0x20, 0x634, 0x64f, 0x645, 0x627, 0x644, 0x6cc, 0x20, 0x688,
0x6cc, 0x6a9, 0x648, 0x679, 0x627, 0x645, 0x64e, 0x631, 0x6a9, 0x632, 0x6cc, 0x20,
0x62c, 0x646, 0x648, 0x657, 0x628, 0x6cc, 0x20, 0x688, 0x6a9, 0x648, 0x679, 0x627,
0x646, 0x648, 0x648, 0x20, 0x633, 0x6cc, 0x644, 0x655, 0x645, 0x60c, 0x20, 0x634,
0x64f, 0x645, 0x627, 0x644, 0x6cc, 0x20, 0x688, 0x6cc, 0x6a9, 0x648, 0x679, 0x627,
0x646, 0x648, 0x657, 0x6a9, 0x627, 0x648, 0x62c, 0x6cc, 0x20, 0x646, 0x627, 0x6af,
0x627, 0x67e, 0x64e, 0x646, 0x627, 0x645, 0x627, 0x67e, 0x64e, 0x631, 0x627, 0x645,
0x627, 0x631, 0x650, 0x628, 0x648, 0x67e, 0x6be, 0x650, 0x646, 0x650, 0x6a9, 0x633,
0x67e, 0x648, 0x679, 0x20, 0x622, 0x624, 0x6ba, 0x20, 0x67e, 0x631, 0x650, 0x646,
0x633, 0x67e, 0x648, 0x679, 0x20, 0x622, 0x641, 0x20, 0x633, 0x67e, 0x6cc, 0x646,
0x67e, 0x648, 0x679, 0x648, 0x20, 0x648, 0x620, 0x644, 0x6c1, 0x648, 0x67e, 0x6cc,
0x656, 0x679, 0x648, 0x20, 0x631, 0x650, 0x6a9, 0x648, 0x67e, 0x648, 0x646, 0x679,
0x627, 0x20, 0x627, 0x631, 0x6cc, 0x646, 0x627, 0x633, 0x631, 0x6cc, 0x646, 0x6a9,
0x650, 0x646, 0x20, 0x627, 0x650, 0x646, 0x644, 0x620, 0x679, 0x631, 0x6cc, 0x633,
0x6cc, 0x641, 0x631, 0x620, 0x62c, 0x6cc, 0x656, 0x646, 0x627, 0x631, 0x650, 0x633,
0x648, 0x644, 0x6cc, 0x648, 0x657, 0x679, 0x631, 0x650, 0x6cc, 0x648, 0x20, 0x628,
0x631, 0x627, 0x646, 0x6a9, 0x648, 0x633, 0x627, 0x646, 0x62a, 0x627, 0x631, 0x6cc,
0x645, 0x633, 0x6cc, 0x646, 0x679, 0x6cc, 0x627, 0x6af, 0x648, 0x633, 0x620, 0x646,
0x679, 0x648, 0x20, 0x691, 0x648, 0x645, 0x650, 0x646, 0x6af, 0x648, 0x633, 0x627,
0x624, 0x20, 0x67e, 0x627, 0x644, 0x648, 0x633, 0x6a9, 0x648, 0x631, 0x655, 0x633,
0x628, 0x627, 0x6d2, 0x633, 0x64e, 0x646, 0x691, 0x633, 0x650, 0x679, 0x6a9, 0x627,
0x633, 0x6cc, 0x646, 0x679, 0x20, 0x628, 0x627, 0x631, 0x62a, 0x6be, 0x6cc, 0x644,
0x645, 0x6cc, 0x633, 0x620, 0x646, 0x679, 0x20, 0x62c, 0x648, 0x646, 0x633, 0x633,
0x6cc, 0x646, 0x679, 0x20, 0x6a9, 0x650, 0x679, 0x633, 0x633, 0x620, 0x646, 0x679,
0x20, 0x644, 0x648, 0x657, 0x633, 0x650, 0x6cc, 0x627, 0x633, 0x6cc, 0x646, 0x679,
0x20, 0x62a, 0x6be, 0x627, 0x645, 0x64e, 0x633, 0x633, 0x6cc, 0x646, 0x679, 0x20,
0x648, 0x650, 0x646, 0x633, 0x6cc, 0x646, 0x679, 0x633, 0x655, 0x648, 0x650, 0x641,
0x679, 0x20, 0x6a9, 0x64e, 0x631, 0x64e, 0x646, 0x679, 0x679, 0x6cc, 0x6af, 0x648,
0x633, 0x6cc, 0x20, 0x6af, 0x627, 0x644, 0x67e, 0x627, 0x62a, 0x6be, 0x6cc, 0x648,
0x657, 0x644, 0x6d2, 0x62a, 0x650, 0x62c, 0x64f, 0x648, 0x627, 0x646, 0x627, 0x679,
0x648, 0x631, 0x648, 0x646, 0x679, 0x648, 0x679, 0x627, 0x631, 0x679, 0x648, 0x644,
0x627, 0x648, 0x620, 0x646, 0x6a9, 0x648, 0x648, 0x64e, 0x631, 0x648, 0x627, 0x6cc,
0x650, 0x679, 0x20, 0x6c1, 0x627, 0x631, 0x655, 0x633, 0x648, 0x650, 0x646, 0x650,
0x67e, 0x620, 0x6af, 0x6cc, 0x6a9, 0x648, 0x657, 0x62a, 0x627, 0x62a, 0x6a9, 0x6cc,
0x633, 0x6cc, 0x688, 0x6cc, 0x648, 0x650, 0x633, 0x688, 0x64f, 0x645, 0x648, 0x646,
0x679, 0x20, 0x688, 0x20, 0x627, 0x64e, 0x631, 0x648, 0x650, 0x644, 0x645, 0x627,
0x633, 0x64e, 0x646, 0x645, 0x620, 0x6a9, 0x20, 0x645, 0x672, 0x691, 0x648, 0x67e,
0x627, 0x645, 0x64e, 0x631, 0x631, 0x648, 0x62a, 0x6be, 0x6cc, 0x631, 0x627, 0x633,
0x6cc, 0x648, 0x648, 0x627, 0x648, 0x648, 0x633, 0x62a, 0x648, 0x6a9, 0x644, 0x648,
0x646, 0x6af, 0x20, 0x6cc, 0x6cc, 0x626, 0x631, 0x20, 0x628, 0x626, 0x6cc, 0x646,
0x627, 0x6cc, 0x688, 0x655, 0x646, 0x627, 0x64e, 0x644, 0x645, 0x627, 0x679, 0x6cc,
0x627, 0x64e, 0x645, 0x627, 0x646, 0x627, 0x64e, 0x646, 0x64e, 0x62f, 0x6cc, 0x631,
0x627, 0x6a9, 0x679, 0x648, 0x627, 0x64e, 0x642, 0x679, 0x648, 0x628, 0x627, 0x64e,
0x634, 0x6af, 0x64e, 0x628, 0x64e, 0x62a, 0x627, 0x679, 0x6cc, 0x631, 0x648, 0x628,
0x620, 0x6c1, 0x631, 0x6cc, 0x656, 0x646, 0x628, 0x627, 0x642, 0x648, 0x657, 0x628,
0x6cc, 0x646, 0x6af, 0x20, 0x6a9, 0x627, 0x6a9, 0x628, 0x631, 0x646, 0x648, 0x644,
0x628, 0x6cc, 0x631, 0x679, 0x628, 0x650, 0x634, 0x6a9, 0x6cc, 0x6a9, 0x628, 0x631,
0x648, 0x657, 0x646, 0x64e, 0x6d2, 0x686, 0x6cc, 0x679, 0x627, 0x6a9, 0x648, 0x644,
0x64e, 0x645, 0x628, 0x648, 0x62f, 0x64e, 0x645, 0x64e, 0x633, 0x6a9, 0x64e, 0x633,
0x688, 0x6be, 0x627, 0x6a9, 0x627, 0x62f, 0x650, 0x644, 0x6cc, 0x62f, 0x64f, 0x628,
0x626, 0x6cc, 0x62f, 0x64f, 0x634, 0x627, 0x646, 0x628, 0x6cc, 0x6d2, 0x641, 0x627,
0x645, 0x627, 0x20, 0x6af, 0x648, 0x633, 0x679, 0x627, 0x63a, 0x632, 0x6c1, 0x6c1,
0x6cc, 0x628, 0x631, 0x648, 0x646, 0x633, 0x6cc, 0x6af, 0x64e, 0x646, 0x6c1, 0x627,
0x646, 0x6af, 0x20, 0x6a9, 0x627, 0x646, 0x6af, 0x62d, 0x648, 0x648, 0x691, 0x627,
0x650, 0x631, 0x6a9, 0x64f, 0x633, 0x6a9, 0x62c, 0x6a9, 0x627, 0x631, 0x62a, 0x627,
0x62c, 0x64e, 0x6cc, 0x627, 0x67e, 0x648, 0x657, 0x631, 0x627, 0x6cc, 0x631, 0x648,
0x634, 0x644, 0x645, 0x642, 0x627, 0x628, 0x64f, 0x644, 0x6a9, 0x64e, 0x645, 0x686,
0x6be, 0x679, 0x6a9, 0x627, 0x6a9, 0x64e, 0x631, 0x627, 0x686, 0x6cc, 0x6a9, 0x627,
0x679, 0x6be, 0x645, 0x64e, 0x646, 0x688, 0x648, 0x657, 0x6a9, 0x6be, 0x627, 0x646,
0x688, 0x6cc, 0x6af, 0x627, 0x6a9, 0x64e, 0x644, 0x655, 0x6a9, 0x64e, 0x62a, 0x627,
0x6a9, 0x631, 0x646, 0x633, 0x646, 0x648, 0x6cc, 0x627, 0x631, 0x633, 0x6a9, 0x6a9,
0x648, 0x644, 0x627, 0x644, 0x64e, 0x645, 0x67e, 0x648, 0x657, 0x631, 0x6a9, 0x64f,
0x686, 0x650, 0x646, 0x6af, 0x6a9, 0x64f, 0x648, 0x6cc, 0x62a, 0x645, 0x6a9, 0x648,
0x645, 0x64e, 0x6af, 0x627, 0x62f, 0x64e, 0x646, 0x645, 0x64e, 0x6a9, 0x64e, 0x633,
0x64e, 0x631, 0x645, 0x64e, 0x646, 0x6cc, 0x656, 0x644, 0x627, 0x646, 0x650, 0x6a9,
0x648, 0x633, 0x650, 0x6cc, 0x627, 0x646, 0x648, 0x648, 0x648, 0x6a9, 0x64f, 0x632,
0x646, 0x6cc, 0x679, 0x633, 0x6a9, 0x646, 0x648, 0x648, 0x648, 0x633, 0x650, 0x628,
0x650, 0x631, 0x633, 0x6a9, 0x627, 0x648, 0x645, 0x633, 0x6a9, 0x627, 0x648, 0x631,
0x64e, 0x644, 0x646, 0x648, 0x645, 0x20, 0x67e, 0x620, 0x646, 0x6c1, 0x6c1, 0x67e,
0x648, 0x646, 0x62a, 0x650, 0x639, 0x627, 0x646, 0x6a9, 0x67e, 0x6cc, 0x648, 0x646,
0x6af, 0x6cc, 0x627, 0x646, 0x6af, 0x6a9, 0x648, 0x633, 0x679, 0x627, 0x646, 0x6d2,
0x642, 0x620, 0x632, 0x620, 0x644, 0x648, 0x691, 0x627, 0x631, 0x6cc, 0x627, 0x636,
0x633, 0x64e, 0x6a9, 0x6be, 0x627, 0x644, 0x650, 0x646, 0x633, 0x64e, 0x645, 0x64e,
0x631, 0x6a9, 0x64e, 0x646, 0x62f, 0x633, 0x6cc, 0x648, 0x644, 0x634, 0x627, 0x646,
0x6af, 0x6d2, 0x633, 0x650, 0x646, 0x6af, 0x627, 0x67e, 0x648, 0x631, 0x633, 0x631,
0x6cc, 0x688, 0x646, 0x6cc, 0x6a9, 0x648, 0x644, 0x6cc, 0x645, 0x633, 0x6a9, 0x62a,
0x64e, 0x6cc, 0x67e, 0x6cc, 0x6d2, 0x62a, 0x627, 0x634, 0x6a9, 0x6cc, 0x646, 0x679,
0x628, 0x650, 0x644, 0x650, 0x633, 0x6cc, 0x62a, 0x620, 0x6c1, 0x631, 0x627, 0x646,
0x62a, 0x6be, 0x650, 0x645, 0x67e, 0x648, 0x657, 0x679, 0x648, 0x6a9, 0x6cc, 0x648,
0x679, 0x648, 0x645, 0x633, 0x6a9, 0x627, 0x648, 0x644, 0x646, 0x20, 0x628, 0x627,
0x679, 0x631, 0x627, 0x64f, 0x631, 0x648, 0x645, 0x642, 0x6cc, 0x627, 0x648, 0x633,
0x679, 0x2d, 0x646, 0x6cc, 0x631, 0x627, 0x648, 0x650, 0x6cc, 0x64e, 0x646, 0x62a,
0x650, 0x6cc, 0x6cc, 0x646, 0x644, 0x627, 0x62f, 0x650, 0x648, 0x648, 0x633, 0x62a,
0x648, 0x6a9, 0x6cc, 0x6a9, 0x648, 0x633, 0x6a9, 0x631, 0x646, 0x6af, 0x648, 0x657,
0x646, 0x6cc, 0x64e, 0x6a9, 0x627, 0x62a, 0x650, 0x631, 0x650, 0x646, 0x628, 0x654,
0x631, 0x6af, 0x6cc, 0x6d2, 0x631, 0x6cc, 0x648, 0x64e, 0x646, 0x627, 0x64e, 0x632,
0x648, 0x631, 0x64e, 0x633, 0x628, 0x631, 0x645, 0x648, 0x657, 0x691, 0x627, 0x6a9,
0x620, 0x646, 0x64e, 0x631, 0x6cc, 0x6a9, 0x6cc, 0x67e, 0x20, 0x624, 0x631, 0x691,
0x6d2, 0x641, 0x620, 0x631, 0x648, 0x645, 0x64e, 0x688, 0x6cc, 0x656, 0x631, 0x627,
0x631, 0x620, 0x6a9, 0x6cc, 0x627, 0x648, 0x650, 0x6a9, 0x62c, 0x646, 0x648, 0x628,
0x6cc, 0x20, 0x62c, 0x627, 0x631, 0x62c, 0x6cc, 0x627, 0x633, 0x6cc, 0x646, 0x679,
0x20, 0x6c1, 0x6cc, 0x644, 0x650, 0x646, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x644,
0x6cc, 0x627, 0x620, 0x691, 0x650, 0x644, 0x6cc, 0x691, 0x628, 0x631, 0x633, 0x628,
0x6cc, 0x646, 0x628, 0x631, 0x648, 0x6a9, 0x655, 0x646, 0x20, 0x6c1, 0x650, 0x644,
0x688, 0x627, 0x631, 0x648, 0x650, 0x646, 0x6cc, 0x648, 0x657, 0x6a9, 0x644, 0x627,
0x62d, 0x6c4, 0x628, 0x672, 0x679, 0x644, 0x650, 0x646, 0x691, 0x6cc, 0x645, 0x627,
0x646, 0x644, 0x639, 0x627, 0x691, 0x20, 0x6c1, 0x648, 0x648, 0x6d2, 0x645, 0x620,
0x644, 0x628, 0x639, 0x627, 0x631, 0x646, 0x67e, 0x654, 0x631, 0x62a, 0x6be, 0x633,
0x650, 0x691, 0x646, 0x6cc, 0x627, 0x6cc, 0x645, 0x633, 0x679, 0x64e, 0x631, 0x688,
0x6cc, 0x645, 0x627, 0x64e, 0x646, 0x688, 0x648, 0x631, 0x627, 0x627, 0x6cc, 0x633,
0x679, 0x631, 0x627, 0x20, 0x6a9, 0x6be, 0x627, 0x646, 0x627, 0x620, 0x62a, 0x6be,
0x655, 0x646, 0x633, 0x628, 0x6cc, 0x644, 0x6af, 0x631, 0x6cc, 0x688, 0x628, 0x654,
0x631, 0x644, 0x650, 0x646, 0x628, 0x631, 0x6cc, 0x679, 0x6cc, 0x633, 0x644, 0x627,
0x648, 0x627, 0x628, 0x631, 0x633, 0x655, 0x644, 0x633, 0x628, 0x64e, 0x686, 0x627,
0x631, 0x6cc, 0x633, 0x679, 0x628, 0x64f, 0x688, 0x627, 0x67e, 0x6cc, 0x633, 0x679,
0x628, 0x648, 0x633, 0x650, 0x646, 0x6af, 0x6cc, 0x646, 0x686, 0x650, 0x633, 0x6cc,
0x656, 0x646, 0x648, 0x6a9, 0x648, 0x67e, 0x646, 0x20, 0x6c1, 0x6cc, 0x6af, 0x646,
0x688, 0x64e, 0x628, 0x644, 0x650, 0x646, 0x6af, 0x650, 0x628, 0x631, 0x627, 0x644,
0x679, 0x64e, 0x631, 0x6af, 0x648, 0x6cc, 0x631, 0x646, 0x633, 0x6d2, 0x62d, 0x620,
0x644, 0x633, 0x650, 0x646, 0x6a9, 0x6cc, 0x622, 0x626, 0x644, 0x20, 0x622, 0x641,
0x20, 0x645, 0x6cc, 0x646, 0x627, 0x650, 0x633, 0x62a, 0x627, 0x646, 0x628, 0x64f,
0x644, 0x62c, 0x631, 0x633, 0x6cc, 0x6a9, 0x64e, 0x644, 0x650, 0x646, 0x627, 0x650,
0x646, 0x6af, 0x631, 0x64e, 0x62f, 0x6a9, 0x6cc, 0x631, 0x648, 0x6a9, 0x6cc, 0x656,
0x648, 0x644, 0x650, 0x633, 0x628, 0x64e, 0x646, 0x644, 0x650, 0x6cc, 0x648, 0x657,
0x628, 0x20, 0x644, 0x650, 0x6cc, 0x627, 0x646, 0x627, 0x644, 0x64e, 0x646, 0x62f,
0x64e, 0x646, 0x644, 0x64e, 0x6a9, 0x632, 0x655, 0x645, 0x628, 0x654, 0x631, 0x6af,
0x645, 0x6cc, 0x688, 0x631, 0x6cc, 0x688, 0x645, 0x627, 0x644, 0x679, 0x627, 0x645,
0x6cc, 0x631, 0x6cc, 0x20, 0x6c1, 0x6cc, 0x645, 0x645, 0x650, 0x646, 0x633, 0x6a9,
0x645, 0x648, 0x646, 0x627, 0x6a9, 0x648, 0x645, 0x627, 0x633, 0x6a9, 0x648, 0x627,
0x648, 0x633, 0x644, 0x648, 0x67e, 0x6cc, 0x631, 0x650, 0x633, 0x67e, 0x648, 0x688,
0x6af, 0x648, 0x631, 0x6cc, 0x6a9, 0x627, 0x67e, 0x631, 0x627, 0x6af, 0x631, 0x650,
0x6af, 0x627, 0x633, 0x64e, 0x645, 0x627, 0x631, 0x627, 0x633, 0x6cc, 0x646, 0x20,
0x645, 0x631, 0x6cc, 0x646, 0x648, 0x633, 0x627, 0x631, 0x627, 0x20, 0x62c, 0x6cc,
0x648, 0x648, 0x633, 0x627, 0x631, 0x627, 0x679, 0x648, 0x633, 0x650, 0x645, 0x641,
0x6cc, 0x631, 0x648, 0x67e, 0x648, 0x644, 0x633, 0x650, 0x6a9, 0x648, 0x67e, 0x6cc,
0x6d2, 0x633, 0x648, 0x641, 0x650, 0x6cc, 0x627, 0x633, 0x679, 0x627, 0x6a9, 0x20,
0x6c1, 0x648, 0x644, 0x645, 0x679, 0x620, 0x644, 0x650, 0x646, 0x679, 0x650, 0x631,
0x6cc, 0x646, 0x627, 0x648, 0x644, 0x6cc, 0x627, 0x646, 0x648, 0x648, 0x633, 0x6a9,
0x648, 0x627, 0x62f, 0x64f, 0x632, 0x648, 0x6cc, 0x679, 0x6cc, 0x6a9, 0x646, 0x648,
0x650, 0x6cc, 0x64e, 0x646, 0x646, 0x627, 0x648, 0x650, 0x644, 0x650, 0x646, 0x650,
0x6cc, 0x64e, 0x633, 0x648, 0x648, 0x644, 0x6af, 0x648, 0x6af, 0x631, 0x64e, 0x62f,
0x648, 0x627, 0x631, 0x633, 0x627, 0x632, 0x6af, 0x631, 0x6cc, 0x628, 0x632, 0x6cc,
0x648, 0x657, 0x631, 0x650, 0x6a9, 0x627, 0x620, 0x646, 0x679, 0x646, 0x627, 0x646,
0x631, 0x650, 0x648, 0x648, 0x686, 0x627, 0x6af, 0x648, 0x633, 0x6a9, 0x631, 0x650,
0x633, 0x645, 0x64e, 0x633, 0x6a9, 0x648, 0x6a9, 0x648, 0x633, 0x6a9, 0x648, 0x645,
0x648, 0x631, 0x648, 0x6a9, 0x6cc, 0x631, 0x6af, 0x648, 0x644, 0x650, 0x646, 0x645,
0x627, 0x6c1, 0x6cc, 0x6d2, 0x645, 0x627, 0x644, 0x62f, 0x6cc, 0x656, 0x648, 0x632,
0x645, 0x648, 0x631, 0x650, 0x634, 0x6cc, 0x633, 0x645, 0x6cc, 0x648, 0x679, 0x631,
0x650, 0x6cc, 0x648, 0x657, 0x646, 0x6cc, 0x64e, 0x646, 0x622, 0x67e, 0x650, 0x6cc,
0x627, 0x622, 0x6a9, 0x644, 0x6cc, 0x646, 0x688, 0x686, 0x64e, 0x62a, 0x6be, 0x64e,
0x645, 0x679, 0x631, 0x6a9, 0x627, 0x6cc, 0x656, 0x633, 0x679, 0x64e, 0x631, 0x627,
0x6cc, 0x641, 0x627, 0x62a, 0x6cc, 0x6d2, 0x641, 0x64e, 0x6a9, 0x64e, 0x648, 0x641,
0x648, 0x641, 0x650, 0x62c, 0x6cc, 0x641, 0x64f, 0x646, 0x627, 0x641, 0x648, 0x657,
0x62a, 0x6cc, 0x6af, 0x620, 0x644, 0x64e, 0x67e, 0x6af, 0x648, 0x633, 0x6af, 0x6cc,
0x645, 0x628, 0x650, 0x6cc, 0x64e, 0x631, 0x6af, 0x64f, 0x648, 0x627, 0x62f, 0x64e,
0x644, 0x686, 0x64e, 0x646, 0x64e, 0x644, 0x6c1, 0x648, 0x646, 0x648, 0x644, 0x648,
0x20, 0x644, 0x648, 0x627, 0x620, 0x646, 0x691, 0x631, 0x628, 0x6cc, 0x631, 0x6cc,
0x6a9, 0x650, 0x631, 0x650, 0x62a, 0x650, 0x645, 0x627, 0x62a, 0x6cc, 0x6a9, 0x648,
0x633, 0x631, 0x627, 0x6d2, 0x6a9, 0x64f, 0x648, 0x62c, 0x644, 0x6cc, 0x646, 0x645,
0x64e, 0x62c, 0x648, 0x657, 0x631, 0x648, 0x645, 0x627, 0x631, 0x6a9, 0x6cc, 0x633,
0x64e, 0x633, 0x645, 0x650, 0x688, 0x648, 0x6cc, 0x6d2, 0x646, 0x627, 0x648, 0x631,
0x648, 0x657, 0x646, 0x648, 0x631, 0x641, 0x648, 0x6a9, 0x646, 0x648, 0x645, 0x6cc,
0x6cc, 0x67e, 0x6cc, 0x6af, 0x648, 0x20, 0x67e, 0x6cc, 0x6af, 0x648, 0x67e, 0x64e,
0x644, 0x627, 0x648, 0x67e, 0x650, 0x679, 0x6a9, 0x6cc, 0x631, 0x64e, 0x646, 0x67e,
0x648, 0x646, 0x627, 0x67e, 0x6d2, 0x67e, 0x648, 0x679, 0x20, 0x645, 0x648, 0x631,
0x633, 0x628, 0x6cc, 0x631, 0x627, 0x631, 0x648, 0x679, 0x648, 0x646, 0x6af, 0x627,
0x633, 0x64e, 0x6cc, 0x67e, 0x627, 0x646, 0x62a, 0x64e, 0x6c1, 0x650, 0x62a, 0x6cc,
0x679, 0x64e, 0x631, 0x627, 0x648, 0x627, 0x62a, 0x648, 0x646, 0x6af, 0x627, 0x62a,
0x64e, 0x67e, 0x648, 0x657, 0x648, 0x6cc, 0x6a9, 0x648, 0x6cc, 0x644, 0x650, 0x633,
0x423, 0x448, 0x443, 0x430, 0x439, 0x44f, 0x41a, 0x435, 0x43c, 0x431, 0x440, 0x438,
0x434, 0x436, 0x2d, 0x411, 0x435, 0x439, 0x41a, 0x430, 0x439, 0x43c, 0x430, 0x43d,
0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x414, 0x43e, 0x441,
0x43e, 0x43d, 0x2d, 0x41a, 0x440, 0x438, 0x43a, 0x424, 0x43e, 0x440, 0x442, 0x2d,
0x41d, 0x435, 0x43b, 0x44c, 0x441, 0x43e, 0x43d, 0x412, 0x438, 0x432, 0x435, 0x439,
0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d, 0x430, 0x411, 0x43e, 0x439,
0x43b, 0x430, 0x2c, 0x20, 0x421, 0x43e, 0x43b, 0x442, 0x4af, 0x441, 0x442, 0x456,
0x43a, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x421, 0x435, 0x43d, 0x442,
0x435, 0x440, 0x2c, 0x20, 0x421, 0x43e, 0x43b, 0x442, 0x4af, 0x441, 0x442, 0x456,
0x43a, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41d, 0x44c, 0x44e, 0x20,
0x421, 0x435, 0x439, 0x43b, 0x435, 0x43c, 0x2c, 0x20, 0x421, 0x43e, 0x43b, 0x442,
0x4af, 0x441, 0x442, 0x456, 0x43a, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430,
0x418, 0x43b, 0x43b, 0x43e, 0x43a, 0x43a, 0x43e, 0x440, 0x442, 0x43e, 0x43e, 0x440,
0x43c, 0x438, 0x443, 0x442, 0x421, 0x443, 0x438, 0x444, 0x442, 0x2d, 0x41a, 0x430,
0x440, 0x440, 0x435, 0x43d, 0x442, 0x414, 0x44d, 0x439, 0x432, 0x438, 0x441, 0x410,
0x49b, 0x442, 0x430, 0x443, 0x410, 0x49b, 0x442, 0x4e9, 0x431, 0x435, 0x411, 0x430,
0x493, 0x434, 0x430, 0x442, 0x411, 0x430, 0x440, 0x43d, 0x430, 0x443, 0x44b, 0x43b,
0x41d, 0x438, 0x43a, 0x43e, 0x441, 0x438, 0x44f, 0x41d, 0x43e, 0x432, 0x43e, 0x441,
0x456, 0x431, 0x456, 0x440, 0x41e, 0x43c, 0x431, 0x44b, 0x49a, 0x43e, 0x441, 0x442,
0x430, 0x43d, 0x430, 0x439, 0x49a, 0x44b, 0x437, 0x44b, 0x43b, 0x43e, 0x440, 0x434,
0x430, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x49b, 0x430, 0x43d, 0x434, 0x4b0, 0x43b,
0x430, 0x43d, 0x431, 0x430, 0x442, 0x44b, 0x440, 0x4ae, 0x440, 0x456, 0x43c, 0x448,
0x456, 0x410, 0x437, 0x43e, 0x440, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430,
0x440, 0x44b, 0x411, 0x435, 0x440, 0x43c, 0x443, 0x434, 0x20, 0x430, 0x440, 0x430,
0x43b, 0x434, 0x430, 0x440, 0x44b, 0x41a, 0x430, 0x43d, 0x430, 0x440, 0x20, 0x430,
0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x424, 0x430, 0x440, 0x435, 0x440,
0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x41e, 0x4a3, 0x442,
0x4af, 0x441, 0x442, 0x456, 0x43a, 0x20, 0x413, 0x435, 0x43e, 0x440, 0x433, 0x438,
0x44f, 0x4d8, 0x443, 0x43b, 0x438, 0x435, 0x20, 0x415, 0x43b, 0x435, 0x43d, 0x430,
0x20, 0x430, 0x440, 0x430, 0x43b, 0x44b, 0x41c, 0x44d, 0x43d, 0x20, 0x430, 0x440,
0x430, 0x43b, 0x44b, 0x421, 0x442, 0x430, 0x43c, 0x431, 0x4b1, 0x43b, 0x41c, 0x4d9,
0x441, 0x43a, 0x435, 0x443, 0x420, 0x43e, 0x436, 0x434, 0x435, 0x441, 0x442, 0x432,
0x43e, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x44b, 0x41a, 0x43e, 0x43a, 0x43e, 0x441,
0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x41a, 0x43e, 0x43c,
0x43e, 0x440, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x41c,
0x430, 0x43b, 0x44c, 0x434, 0x438, 0x432, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434,
0x430, 0x440, 0x44b, 0x41f, 0x430, 0x441, 0x445, 0x430, 0x20, 0x430, 0x440, 0x430,
0x43b, 0x44b, 0x41a, 0x443, 0x441, 0x430, 0x438, 0x435, 0x41c, 0x430, 0x440, 0x43a,
0x438, 0x437, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x627,
0x628, 0x64a, 0x62f, 0x62c, 0x627, 0x646, 0x627, 0x643, 0x643, 0x631, 0x627, 0x627,
0x62f, 0x62f, 0x64a, 0x633, 0x2d, 0x627, 0x628, 0x6d5, 0x628, 0x627, 0x627, 0x644,
0x62c, 0x64a, 0x631, 0x627, 0x633, 0x645, 0x627, 0x631, 0x627, 0x628, 0x627, 0x646,
0x6af, 0x64a, 0x628, 0x627, 0x646, 0x62c, 0x6cb, 0x644, 0x628, 0x64a, 0x633, 0x627,
0x6cb, 0x628, 0x631, 0x627, 0x632, 0x632, 0x627, 0x6c6, 0x64a, 0x644, 0x628, 0x6cb,
0x62c, 0x6cb, 0x645, 0x628, 0x6cb, 0x631, 0x627, 0x643, 0x627, 0x64a, 0x631, 0x643,
0x627, 0x633, 0x627, 0x628, 0x644, 0x627, 0x646, 0x643, 0x627, 0x633, 0x6d5, 0x6cb,
0x62a, 0x627, 0x62f, 0x627, 0x631, 0x2d, 0x6d5, 0x633, 0x2d, 0x633, 0x627, 0x644,
0x627, 0x645, 0x62c, 0x64a, 0x628, 0x6cb, 0x62a, 0x64a, 0x62f, 0x6cb, 0x627, 0x644,
0x627, 0x6d5, 0x644, 0x2d, 0x627, 0x64a, 0x6cb, 0x646, 0x641, 0x631, 0x64a, 0x62a,
0x627, 0x6cb, 0x646, 0x6af, 0x627, 0x628, 0x648, 0x631, 0x648, 0x646, 0x6d5, 0x62d,
0x627, 0x631, 0x627, 0x631, 0x6d5, 0x64a, 0x648, 0x62d, 0x627, 0x646, 0x646, 0x6d5,
0x633, 0x628, 0x6cb, 0x631, 0x6af, 0x62c, 0x6cb, 0x628, 0x627, 0x643, 0x627, 0x645,
0x67e, 0x627, 0x644, 0x627, 0x62d, 0x627, 0x631, 0x62a, 0x6cb, 0x645, 0x643, 0x64a,
0x6af, 0x627, 0x644, 0x64a, 0x644, 0x627, 0x6af, 0x648, 0x633, 0x644, 0x64a, 0x628,
0x631, 0x6d5, 0x6c6, 0x64a, 0x644, 0x644, 0x648, 0x645, 0x6d5, 0x644, 0x6cb, 0x627,
0x646, 0x62f, 0x627, 0x644, 0x6cb, 0x628, 0x6cb, 0x645, 0x628, 0x627, 0x634, 0x64a,
0x644, 0x6cb, 0x633, 0x627, 0x643, 0x627, 0x645, 0x627, 0x67e, 0x6cb, 0x62a, 0x6cb,
0x645, 0x627, 0x633, 0x6d5, 0x631, 0x6cb, 0x645, 0x628, 0x627, 0x628, 0x6d5, 0x646,
0x6d5, 0x645, 0x648, 0x6af, 0x627, 0x62f, 0x64a, 0x634, 0x6cb, 0x645, 0x648, 0x646,
0x631, 0x648, 0x6c6, 0x64a, 0x627, 0x646, 0x627, 0x64a, 0x631, 0x648, 0x628, 0x64a,
0x646, 0x62f, 0x62c, 0x627, 0x645, 0x6d5, 0x646, 0x627, 0x646, 0x64a, 0x627, 0x645,
0x6d5, 0x64a, 0x646, 0x6cb, 0x627, 0x643, 0x634, 0x648, 0x62a, 0x6cb, 0x627, 0x6af,
0x627, 0x62f, 0x6cb, 0x6af, 0x6cb, 0x67e, 0x648, 0x631, 0x62a, 0x648, 0x2d, 0x646,
0x648, 0x6c6, 0x648, 0x633, 0x627, 0x646, 0x2d, 0x62a, 0x648, 0x645, 0x6d5, 0x62a,
0x631, 0x64a, 0x67e, 0x648, 0x644, 0x64a, 0x62a, 0x6cb, 0x646, 0x64a, 0x633, 0x6c6,
0x64a, 0x646, 0x62f, 0x62d, 0x6cb, 0x643, 0x627, 0x62f, 0x627, 0x643, 0x627, 0x646,
0x643, 0x648, 0x631, 0x64a, 0x62c, 0x627, 0x646, 0x6af, 0x64a, 0x644, 0x64a, 0x627,
0x627, 0x646, 0x62a, 0x64a, 0x6af, 0x6cb, 0x627, 0x627, 0x631, 0x627, 0x6af, 0x6cb,
0x627, 0x64a, 0x646, 0x627, 0x628, 0x6cb, 0x6d5, 0x646, 0x648, 0x633, 0x2d, 0x627,
0x64a, 0x631, 0x648, 0x633, 0x62c, 0x6cb, 0x62c, 0x6cb, 0x64a, 0x644, 0x627, 0x2d,
0x631, 0x64a, 0x648, 0x62d, 0x627, 0x645, 0x6d5, 0x646, 0x62f, 0x648, 0x632, 0x627,
0x631, 0x64a, 0x648, 0x2d, 0x6af, 0x627, 0x644, 0x6d5, 0x6af, 0x648, 0x633, 0x633,
0x627, 0x644, 0x62a, 0x627, 0x633, 0x627, 0x646, 0x2d, 0x62d, 0x6cb, 0x627, 0x646,
0x633, 0x627, 0x646, 0x2d, 0x644, 0x6cb, 0x64a, 0x633, 0x62a, 0x6cb, 0x643, 0x6cb,
0x645, 0x627, 0x646, 0x6cb, 0x634, 0x6cb, 0x627, 0x64a, 0x627, 0x627, 0x631, 0x6cb,
0x628, 0x627, 0x627, 0x633, 0x6cb, 0x646, 0x633, 0x64a, 0x648, 0x646, 0x627, 0x62a,
0x64a, 0x643, 0x648, 0x643, 0x627, 0x646, 0x628, 0x627, 0x64a, 0x627, 0x628, 0x627,
0x64a, 0x627, 0x2d, 0x62f, 0x6d5, 0x2d, 0x628, 0x627, 0x646, 0x62f, 0x6d5, 0x631,
0x627, 0x633, 0x628, 0x627, 0x631, 0x628, 0x627, 0x62f, 0x648, 0x633, 0x628, 0x6d5,
0x644, 0x6d5, 0x645, 0x628, 0x6d5, 0x644, 0x64a, 0x632, 0x628, 0x627, 0x648, 0x2d,
0x6c6, 0x64a, 0x633, 0x62a, 0x627, 0x628, 0x648, 0x6af, 0x648, 0x62a, 0x627, 0x628,
0x648, 0x64a, 0x633, 0x6d5, 0x643, 0x6d5, 0x645, 0x628, 0x631, 0x64a, 0x62c, 0x2d,
0x628, 0x6d5, 0x64a, 0x643, 0x627, 0x645, 0x67e, 0x6cb, 0x2d, 0x6af, 0x631, 0x627,
0x646, 0x62f, 0x6d5, 0x643, 0x627, 0x646, 0x643, 0x6cb, 0x646, 0x643, 0x627, 0x64a,
0x6d5, 0x646, 0x646, 0x627, 0x643, 0x627, 0x64a, 0x645, 0x627, 0x646, 0x20, 0x627,
0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x686, 0x64a, 0x643, 0x627, 0x6af,
0x648, 0x686, 0x64a, 0x6cb, 0x627, 0x6cb, 0x627, 0x633, 0x64a, 0x6cb, 0x62f, 0x627,
0x62f, 0x2d, 0x62d, 0x6cb, 0x627, 0x631, 0x6d5, 0x633, 0x643, 0x648, 0x633, 0x62a,
0x627, 0x2d, 0x631, 0x64a, 0x643, 0x627, 0x643, 0x631, 0x6d5, 0x633, 0x62a, 0x648,
0x646, 0x643, 0x6cb, 0x64a, 0x627, 0x628, 0x627, 0x643, 0x64a, 0x6cb, 0x631, 0x627,
0x633, 0x627, 0x648, 0x62f, 0x627, 0x646, 0x645, 0x627, 0x631, 0x643, 0x633, 0x62d,
0x627, 0x6c6, 0x646, 0x62f, 0x648, 0x6cb, 0x633, 0x648, 0x646, 0x62f, 0x648, 0x6cb,
0x633, 0x648, 0x646, 0x2d, 0x643, 0x631, 0x64a, 0x643, 0x62f, 0x6d5, 0x646, 0x6c6,
0x6d5, 0x631, 0x62f, 0x6d5, 0x62a, 0x631, 0x648, 0x64a, 0x62f, 0x6d5, 0x62f, 0x645,
0x648, 0x646, 0x62a, 0x648, 0x646, 0x6d5, 0x64a, 0x631, 0x6cb, 0x646, 0x6d5, 0x67e,
0x6d5, 0x633, 0x627, 0x644, 0x6c6, 0x627, 0x62f, 0x648, 0x631, 0x641, 0x648, 0x631,
0x62a, 0x2d, 0x646, 0x6d5, 0x644, 0x633, 0x648, 0x646, 0x641, 0x648, 0x631, 0x62a,
0x627, 0x644, 0x6d5, 0x632, 0x627, 0x6af, 0x644, 0x6d5, 0x64a, 0x633, 0x2d, 0x628,
0x6d5, 0x64a, 0x6af, 0x6cb, 0x633, 0x2d, 0x628, 0x6d5, 0x64a, 0x6af, 0x631, 0x627,
0x646, 0x62f, 0x2d, 0x62a, 0x6d5, 0x631, 0x643, 0x6af, 0x631, 0x6d5, 0x646, 0x627,
0x62f, 0x627, 0x6af, 0x6c6, 0x627, 0x62f, 0x6d5, 0x644, 0x6c7, 0x67e, 0x627, 0x6af,
0x6c6, 0x627, 0x62a, 0x6d5, 0x645, 0x627, 0x644, 0x627, 0x6af, 0x6cb, 0x627, 0x64a,
0x627, 0x643, 0x64a, 0x644, 0x6af, 0x627, 0x64a, 0x627, 0x646, 0x627, 0x6af, 0x627,
0x644, 0x64a, 0x641, 0x627, 0x643, 0x633, 0x6af, 0x627, 0x6c6, 0x627, 0x646, 0x627,
0x6d5, 0x631, 0x645, 0x648, 0x633, 0x64a, 0x644, 0x648, 0x64a, 0x646, 0x62f, 0x64a,
0x627, 0x646, 0x627, 0x67e, 0x648, 0x644, 0x64a, 0x633, 0x646, 0x648, 0x643, 0x633,
0x60c, 0x20, 0x64a, 0x646, 0x62f, 0x64a, 0x627, 0x646, 0x627, 0x645, 0x627, 0x631,
0x6d5, 0x646, 0x6af, 0x648, 0x60c, 0x20, 0x64a, 0x646, 0x62f, 0x64a, 0x627, 0x646,
0x627, 0x67e, 0x64a, 0x62a, 0x6d5, 0x631, 0x633, 0x628, 0x6d5, 0x631, 0x6af, 0x60c,
0x20, 0x64a, 0x646, 0x62f, 0x64a, 0x627, 0x646, 0x627, 0x62a, 0x6d5, 0x644, 0x644,
0x2d, 0x633, 0x64a, 0x62a, 0x64a, 0x60c, 0x20, 0x64a, 0x646, 0x62f, 0x64a, 0x627,
0x646, 0x627, 0x6c6, 0x64a, 0x6c6, 0x6d5, 0x64a, 0x60c, 0x20, 0x64a, 0x646, 0x62f,
0x64a, 0x627, 0x646, 0x627, 0x6c6, 0x64a, 0x646, 0x633, 0x6d5, 0x646, 0x646, 0x6d5,
0x633, 0x60c, 0x20, 0x64a, 0x646, 0x62f, 0x64a, 0x627, 0x646, 0x627, 0x6cb, 0x64a,
0x646, 0x627, 0x645, 0x627, 0x643, 0x60c, 0x20, 0x64a, 0x646, 0x62f, 0x64a, 0x627,
0x646, 0x627, 0x64a, 0x646, 0x6cb, 0x6c6, 0x64a, 0x643, 0x64a, 0x643, 0x627, 0x644,
0x6cb, 0x64a, 0x62a, 0x64a, 0x627, 0x645, 0x627, 0x64a, 0x643, 0x627, 0x62c, 0x6cb,
0x646, 0x648, 0x644, 0x6cb, 0x64a, 0x633, 0x6c6, 0x64a, 0x644, 0x645, 0x648, 0x646,
0x62a, 0x64a, 0x633, 0x6d5, 0x644, 0x644, 0x648, 0x60c, 0x20, 0x643, 0x6d5, 0x646,
0x62a, 0x6cb, 0x643, 0x643, 0x64a, 0x643, 0x631, 0x627, 0x644, 0x6d5, 0x646, 0x62f,
0x6d5, 0x64a, 0x643, 0x644, 0x627, 0x2d, 0x67e, 0x627, 0x633, 0x644, 0x648, 0x633,
0x2d, 0x627, 0x646, 0x62c, 0x6d5, 0x644, 0x6d5, 0x633, 0x644, 0x648, 0x6cb, 0x6d5,
0x631, 0x2d, 0x67e, 0x631, 0x64a, 0x646, 0x633, 0x2d, 0x643, 0x6cb, 0x648, 0x62a,
0x6d5, 0x631, 0x645, 0x627, 0x633, 0x6d5, 0x64a, 0x648, 0x645, 0x627, 0x646, 0x627,
0x6af, 0x6cb, 0x627, 0x645, 0x627, 0x646, 0x627, 0x6cb, 0x633, 0x645, 0x627, 0x631,
0x64a, 0x6af, 0x648, 0x645, 0x627, 0x631, 0x62a, 0x64a, 0x646, 0x64a, 0x643, 0x627,
0x645, 0x627, 0x633, 0x627, 0x62a, 0x644, 0x627, 0x646, 0x645, 0x6d5, 0x646, 0x648,
0x645, 0x64a, 0x646, 0x64a, 0x645, 0x6d5, 0x631, 0x64a, 0x62f, 0x627, 0x645, 0x6d5,
0x62a, 0x644, 0x627, 0x643, 0x627, 0x62a, 0x644, 0x627, 0x645, 0x6d5, 0x62d, 0x64a,
0x643, 0x648, 0x645, 0x64a, 0x643, 0x6d5, 0x644, 0x648, 0x646, 0x645, 0x648, 0x646,
0x643, 0x62a, 0x648, 0x646, 0x645, 0x648, 0x646, 0x62a, 0x6d5, 0x631, 0x631, 0x6d5,
0x64a, 0x645, 0x648, 0x646, 0x62a, 0x6d5, 0x6c6, 0x64a, 0x62f, 0x6d5, 0x648, 0x645,
0x648, 0x646, 0x62a, 0x633, 0x6d5, 0x631, 0x631, 0x627, 0x62a, 0x646, 0x627, 0x633,
0x633, 0x627, 0x6cb, 0x646, 0x64a, 0x6cb, 0x2d, 0x64a, 0x648, 0x631, 0x643, 0x646,
0x648, 0x631, 0x648, 0x646, 0x64a, 0x627, 0x628, 0x648, 0x64a, 0x644, 0x627, 0x60c,
0x20, 0x633, 0x648, 0x644, 0x62a, 0x6c7, 0x633, 0x62a, 0x649, 0x643, 0x20, 0x62f,
0x627, 0x643, 0x648, 0x62a, 0x627, 0x633, 0x6d5, 0x646, 0x62a, 0x6d5, 0x631, 0x60c,
0x20, 0x633, 0x648, 0x644, 0x62a, 0x6c7, 0x633, 0x62a, 0x649, 0x643, 0x20, 0x62f,
0x627, 0x643, 0x648, 0x62a, 0x627, 0x646, 0x64a, 0x6cb, 0x2d, 0x633, 0x6d5, 0x64a,
0x644, 0x6d5, 0x645, 0x60c, 0x20, 0x633, 0x648, 0x644, 0x62a, 0x6c7, 0x633, 0x62a,
0x649, 0x643, 0x20, 0x62f, 0x627, 0x643, 0x648, 0x62a, 0x627, 0x646, 0x6cb, 0x6cb,
0x643, 0x648, 0x62d, 0x64a, 0x646, 0x627, 0x6af, 0x627, 0x67e, 0x627, 0x646, 0x627,
0x645, 0x627, 0x67e, 0x627, 0x631, 0x627, 0x645, 0x627, 0x631, 0x64a, 0x628, 0x648,
0x641, 0x64a, 0x646, 0x64a, 0x643, 0x633, 0x67e, 0x648, 0x631, 0x62a, 0x2d, 0x648,
0x641, 0x2d, 0x67e, 0x631, 0x6d5, 0x646, 0x633, 0x67e, 0x648, 0x631, 0x62a, 0x2d,
0x648, 0x641, 0x2d, 0x633, 0x67e, 0x627, 0x64a, 0x646, 0x67e, 0x648, 0x631, 0x62a,
0x648, 0x2d, 0x6c6, 0x6d5, 0x644, 0x64a, 0x6cb, 0x67e, 0x6cb, 0x6d5, 0x631, 0x62a,
0x648, 0x2d, 0x631, 0x64a, 0x643, 0x648, 0x67e, 0x6cb, 0x646, 0x62a, 0x627, 0x2d,
0x627, 0x631, 0x6d5, 0x646, 0x627, 0x633, 0x631, 0x627, 0x646, 0x643, 0x64a, 0x646,
0x2d, 0x64a, 0x646, 0x644, 0x6d5, 0x62a, 0x631, 0x6d5, 0x633, 0x64a, 0x641, 0x64a,
0x631, 0x6d5, 0x62c, 0x627, 0x64a, 0x646, 0x627, 0x631, 0x6d5, 0x632, 0x648, 0x644,
0x64a, 0x6cb, 0x62a, 0x631, 0x64a, 0x6cb, 0x2d, 0x628, 0x631, 0x627, 0x646, 0x643,
0x6cb, 0x633, 0x627, 0x646, 0x62a, 0x627, 0x631, 0x6d5, 0x646, 0x633, 0x627, 0x646,
0x62a, 0x64a, 0x627, 0x6af, 0x648, 0x633, 0x627, 0x646, 0x62a, 0x648, 0x2d, 0x62f,
0x648, 0x645, 0x64a, 0x646, 0x6af, 0x648, 0x633, 0x627, 0x646, 0x2d, 0x67e, 0x6cb,
0x627, 0x644, 0x6cb, 0x64a, 0x644, 0x644, 0x648, 0x643, 0x643, 0x648, 0x631, 0x62a,
0x648, 0x648, 0x631, 0x645, 0x64a, 0x6cb, 0x62a, 0x633, 0x6d5, 0x646, 0x2d, 0x628,
0x627, 0x631, 0x62a, 0x6d5, 0x644, 0x6d5, 0x645, 0x64a, 0x633, 0x6d5, 0x646, 0x62a,
0x2d, 0x62c, 0x648, 0x646, 0x633, 0x633, 0x6d5, 0x646, 0x62a, 0x2d, 0x643, 0x64a,
0x62a, 0x633, 0x633, 0x6d5, 0x646, 0x62a, 0x2d, 0x644, 0x64a, 0x6cb, 0x633, 0x64a,
0x627, 0x633, 0x6d5, 0x646, 0x62a, 0x2d, 0x62a, 0x648, 0x645, 0x627, 0x633, 0x633,
0x6d5, 0x646, 0x62a, 0x2d, 0x6c6, 0x64a, 0x646, 0x633, 0x6d5, 0x646, 0x62a, 0x633,
0x6cb, 0x64a, 0x627, 0x62a, 0x2d, 0x643, 0x627, 0x631, 0x631, 0x6d5, 0x646, 0x62a,
0x62a, 0x6d5, 0x6af, 0x6cb, 0x633, 0x64a, 0x6af, 0x627, 0x644, 0x67e, 0x627, 0x62a,
0x6cb, 0x644, 0x6d5, 0x62a, 0x64a, 0x62d, 0x6cb, 0x627, 0x646, 0x627, 0x6c6, 0x627,
0x646, 0x643, 0x6cb, 0x6c6, 0x6d5, 0x631, 0x6cb, 0x627, 0x64a, 0x62a, 0x62d, 0x648,
0x631, 0x633, 0x6c6, 0x64a, 0x646, 0x646, 0x64a, 0x67e, 0x6d5, 0x6af, 0x64a, 0x627,
0x643, 0x6cb, 0x62a, 0x627, 0x62a, 0x643, 0x6d5, 0x64a, 0x633, 0x64a, 0x62f, 0x6d5,
0x64a, 0x6c6, 0x64a, 0x633, 0x62f, 0x64a, 0x6cb, 0x645, 0x648, 0x646, 0x2d, 0x62f,
0x64a, 0x6cb, 0x631, 0x6c6, 0x64a, 0x644, 0x645, 0x627, 0x643, 0x643, 0x6cb, 0x648,
0x631, 0x64a, 0x645, 0x648, 0x6cb, 0x633, 0x648, 0x646, 0x645, 0x627, 0x643, 0x2d,
0x645, 0x6d5, 0x631, 0x62f, 0x648, 0x67e, 0x627, 0x644, 0x645, 0x6d5, 0x631, 0x631,
0x648, 0x62a, 0x6d5, 0x631, 0x627, 0x633, 0x6d5, 0x6c6, 0x627, 0x6c6, 0x648, 0x633,
0x62a, 0x648, 0x643, 0x644, 0x648, 0x646, 0x6af, 0x64a, 0x631, 0x627, 0x62f, 0x6d5,
0x646, 0x627, 0x644, 0x645, 0x627, 0x62a, 0x649, 0x627, 0x645, 0x645, 0x627, 0x646,
0x627, 0x646, 0x627, 0x62f, 0x649, 0x631, 0x627, 0x642, 0x62a, 0x627, 0x6cb, 0x627,
0x642, 0x62a, 0x648, 0x628, 0x6d5, 0x627, 0x634, 0x62d, 0x627, 0x628, 0x627, 0x62a,
0x627, 0x62a, 0x649, 0x631, 0x627, 0x6cb, 0x628, 0x627, 0x6af, 0x62f, 0x627, 0x62a,
0x628, 0x627, 0x62d, 0x631, 0x6d5, 0x64a, 0x646, 0x628, 0x627, 0x643, 0x6cb, 0x628,
0x627, 0x6ad, 0x643, 0x648, 0x643, 0x628, 0x627, 0x631, 0x646, 0x627, 0x6cb, 0x649,
0x644, 0x628, 0x6d5, 0x64a, 0x631, 0x6cb, 0x62a, 0x628, 0x649, 0x634, 0x643, 0x6d5,
0x643, 0x628, 0x631, 0x6cb, 0x646, 0x6d5, 0x64a, 0x686, 0x64a, 0x62a, 0x627, 0x62f,
0x627, 0x645, 0x627, 0x633, 0x643, 0x62f, 0x627, 0x643, 0x643, 0x627, 0x62f, 0x6cb,
0x628, 0x627, 0x64a, 0x62f, 0x6cb, 0x634, 0x627, 0x646, 0x628, 0x6d5, 0x641, 0x627,
0x645, 0x627, 0x6af, 0x6cb, 0x633, 0x62a, 0x627, 0x6af, 0x627, 0x632, 0x627, 0x62d,
0x6d5, 0x6c6, 0x631, 0x648, 0x646, 0x62d, 0x648, 0x634, 0x64a, 0x645, 0x64a, 0x646,
0x62d, 0x648, 0x6ad, 0x643, 0x648, 0x6ad, 0x62d, 0x648, 0x6c6, 0x62f, 0x64a, 0x631,
0x643, 0x6cb, 0x62a, 0x633, 0x643, 0x62c, 0x627, 0x643, 0x627, 0x631, 0x62a, 0x627,
0x62c, 0x627, 0x64a, 0x627, 0x67e, 0x6cb, 0x631, 0x627, 0x64a, 0x6d5, 0x631, 0x6cb,
0x633, 0x627, 0x644, 0x64a, 0x645, 0x643, 0x627, 0x628, 0x6cb, 0x644, 0x643, 0x627,
0x645, 0x686, 0x627, 0x62a, 0x643, 0x627, 0x643, 0x627, 0x631, 0x627, 0x686, 0x627,
0x643, 0x627, 0x62a, 0x645, 0x627, 0x646, 0x62f, 0x6cb, 0x62d, 0x627, 0x646, 0x62f,
0x649, 0x6af, 0x627, 0x643, 0x627, 0x644, 0x643, 0x6cb, 0x62a, 0x627, 0x643, 0x6cb,
0x627, 0x644, 0x627, 0x2d, 0x644, 0x6cb, 0x645, 0x67e, 0x6cb, 0x631, 0x643, 0x6cb,
0x686, 0x64a, 0x6ad, 0x643, 0x6cb, 0x6c6, 0x6d5, 0x64a, 0x62a, 0x645, 0x627, 0x6af,
0x627, 0x62f, 0x627, 0x646, 0x645, 0x627, 0x643, 0x627, 0x633, 0x627, 0x631, 0x645,
0x627, 0x633, 0x643, 0x627, 0x62a, 0x646, 0x64a, 0x643, 0x648, 0x633, 0x64a, 0x627,
0x646, 0x648, 0x6c6, 0x648, 0x643, 0x6cb, 0x632, 0x646, 0x6d5, 0x62a, 0x633, 0x643,
0x62c, 0x627, 0x6ad, 0x627, 0x633, 0x649, 0x628, 0x649, 0x631, 0x648, 0x645, 0x628,
0x649, 0x67e, 0x646, 0x648, 0x645, 0x67e, 0x6d5, 0x646, 0x67e, 0x648, 0x646, 0x62a,
0x64a, 0x627, 0x646, 0x627, 0x643, 0x67e, 0x62d, 0x6d5, 0x646, 0x64a, 0x627, 0x646,
0x643, 0x627, 0x62a, 0x627, 0x631, 0x642, 0x649, 0x632, 0x649, 0x644, 0x648, 0x631,
0x62f, 0x627, 0x6d5, 0x631, 0x2d, 0x631, 0x64a, 0x627, 0x62f, 0x633, 0x627, 0x62d,
0x627, 0x644, 0x64a, 0x646, 0x633, 0x627, 0x645, 0x627, 0x631, 0x642, 0x627, 0x646,
0x62f, 0x633, 0x6d5, 0x6cb, 0x644, 0x634, 0x627, 0x646, 0x62d, 0x627, 0x64a, 0x633,
0x64a, 0x646, 0x6af, 0x627, 0x67e, 0x6cb, 0x631, 0x633, 0x631, 0x6d5, 0x62f, 0x646,
0x6d5, 0x643, 0x648, 0x644, 0x649, 0x645, 0x633, 0x643, 0x62a, 0x627, 0x64a, 0x628,
0x6d5, 0x64a, 0x62a, 0x627, 0x634, 0x643, 0x6d5, 0x646, 0x62a, 0x62a, 0x628, 0x64a,
0x644, 0x64a, 0x633, 0x64a, 0x62a, 0x6d5, 0x6af, 0x6d5, 0x631, 0x627, 0x646, 0x62a,
0x62d, 0x64a, 0x645, 0x67e, 0x62d, 0x6cb, 0x62a, 0x648, 0x643, 0x64a, 0x648, 0x62a,
0x6cb, 0x645, 0x6d5, 0x646, 0x6c7, 0x644, 0x627, 0x646, 0x628, 0x627, 0x62a, 0x649,
0x631, 0x621, 0x6c7, 0x631, 0x649, 0x645, 0x634, 0x649, 0x6cb, 0x633, 0x62a, 0x2d,
0x646, 0x6d5, 0x631, 0x627, 0x6c6, 0x6d5, 0x646, 0x62a, 0x64a, 0x627, 0x646, 0x6c6,
0x644, 0x627, 0x62f, 0x64a, 0x6c6, 0x627, 0x633, 0x62a, 0x648, 0x643, 0x64a, 0x627,
0x643, 0x6cb, 0x62a, 0x633, 0x643, 0x64a, 0x627, 0x646, 0x6af, 0x648, 0x646, 0x6d5,
0x643, 0x627, 0x62a, 0x6d5, 0x631, 0x64a, 0x646, 0x628, 0x6cb, 0x631, 0x6af, 0x64a,
0x6d5, 0x631, 0x6d5, 0x6c6, 0x627, 0x646, 0x627, 0x632, 0x648, 0x631, 0x20, 0x627,
0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x628, 0x6d5, 0x631, 0x645, 0x6cb,
0x62f, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x643, 0x627,
0x646, 0x627, 0x631, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649,
0x643, 0x627, 0x628, 0x648, 0x2d, 0x6c6, 0x6d5, 0x631, 0x62f, 0x6d5, 0x641, 0x627,
0x631, 0x6d5, 0x631, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649,
0x645, 0x627, 0x62f, 0x6d5, 0x64a, 0x631, 0x627, 0x631, 0x6d5, 0x64a, 0x643, 0x64a,
0x627, 0x6c6, 0x64a, 0x643, 0x648, 0x6ad, 0x62a, 0x6c7, 0x633, 0x62a, 0x649, 0x643,
0x20, 0x6af, 0x6d5, 0x648, 0x631, 0x6af, 0x64a, 0x627, 0x627, 0x6cb, 0x644, 0x64a,
0x6d5, 0x20, 0x6d5, 0x644, 0x6d5, 0x646, 0x627, 0x20, 0x627, 0x631, 0x627, 0x644,
0x649, 0x633, 0x62a, 0x6d5, 0x646, 0x644, 0x64a, 0x627, 0x62f, 0x6d5, 0x644, 0x627,
0x64a, 0x62f, 0x627, 0x628, 0x631, 0x64a, 0x633, 0x628, 0x6d5, 0x646, 0x628, 0x631,
0x648, 0x643, 0x6d5, 0x646, 0x2d, 0x62d, 0x64a, 0x644, 0x62f, 0x627, 0x631, 0x6c6,
0x64a, 0x646, 0x64a, 0x6cb, 0x643, 0x644, 0x627, 0x62d, 0x648, 0x628, 0x627, 0x631,
0x62a, 0x644, 0x64a, 0x646, 0x62f, 0x6d5, 0x645, 0x627, 0x646, 0x644, 0x648, 0x631,
0x62f, 0x2d, 0x62d, 0x627, 0x6cb, 0x645, 0x6d5, 0x644, 0x628, 0x6cb, 0x631, 0x646,
0x67e, 0x6d5, 0x631, 0x62a, 0x633, 0x64a, 0x62f, 0x646, 0x6d5, 0x64a, 0x627, 0x645,
0x633, 0x62a, 0x6d5, 0x631, 0x62f, 0x627, 0x645, 0x627, 0x646, 0x62f, 0x648, 0x631,
0x631, 0x627, 0x627, 0x633, 0x62a, 0x631, 0x627, 0x62d, 0x627, 0x646, 0x627, 0x641,
0x64a, 0x646, 0x627, 0x628, 0x6d5, 0x644, 0x6af, 0x631, 0x627, 0x62f, 0x628, 0x6d5,
0x631, 0x644, 0x64a, 0x646, 0x628, 0x631, 0x627, 0x62a, 0x64a, 0x633, 0x644, 0x627,
0x6c6, 0x627, 0x628, 0x631, 0x64a, 0x6cb, 0x633, 0x6d5, 0x644, 0x628, 0x6cb, 0x62d,
0x627, 0x631, 0x6d5, 0x633, 0x62a, 0x628, 0x6cb, 0x62f, 0x627, 0x67e, 0x6d5, 0x634,
0x62a, 0x628, 0x64a, 0x6cb, 0x632, 0x64a, 0x646, 0x6af, 0x6d5, 0x646, 0x643, 0x64a,
0x634, 0x64a, 0x646, 0x6d5, 0x6c6, 0x643, 0x648, 0x67e, 0x6d5, 0x646, 0x6af, 0x627,
0x6af, 0x6d5, 0x646, 0x62f, 0x6cb, 0x628, 0x644, 0x64a, 0x646, 0x6af, 0x64a, 0x628,
0x631, 0x627, 0x644, 0x62a, 0x627, 0x644, 0x6af, 0x6d5, 0x631, 0x646, 0x633, 0x64a,
0x62d, 0x6d5, 0x644, 0x633, 0x64a, 0x646, 0x643, 0x64a, 0x645, 0x6d5, 0x646, 0x20,
0x627, 0x631, 0x627, 0x644, 0x649, 0x633, 0x62a, 0x627, 0x645, 0x628, 0x6c7, 0x644,
0x62c, 0x6d5, 0x631, 0x633, 0x6d5, 0x64a, 0x643, 0x627, 0x644, 0x64a, 0x646, 0x64a,
0x646, 0x6af, 0x631, 0x627, 0x62f, 0x643, 0x64a, 0x631, 0x648, 0x6c6, 0x643, 0x64a,
0x6d5, 0x6c6, 0x644, 0x64a, 0x633, 0x627, 0x628, 0x648, 0x646, 0x644, 0x64a, 0x6cb,
0x628, 0x644, 0x64a, 0x627, 0x646, 0x627, 0x644, 0x648, 0x646, 0x62f, 0x648, 0x646,
0x644, 0x64a, 0x6cb, 0x643, 0x633, 0x6d5, 0x645, 0x628, 0x6cb, 0x631, 0x6af, 0x645,
0x627, 0x62f, 0x631, 0x64a, 0x62f, 0x645, 0x627, 0x644, 0x62a, 0x627, 0x645, 0x627,
0x631, 0x64a, 0x6d5, 0x62d, 0x627, 0x645, 0x646, 0x645, 0x627, 0x633, 0x643, 0x6d5,
0x6cb, 0x67e, 0x627, 0x631, 0x64a, 0x62c, 0x67e, 0x648, 0x62f, 0x6af, 0x648, 0x631,
0x64a, 0x633, 0x627, 0x67e, 0x631, 0x627, 0x6af, 0x627, 0x631, 0x64a, 0x6af, 0x627,
0x633, 0x627, 0x645, 0x627, 0x631, 0x627, 0x633, 0x627, 0x646, 0x2d, 0x645, 0x627,
0x631, 0x64a, 0x646, 0x648, 0x633, 0x627, 0x631, 0x627, 0x6d5, 0x6c6, 0x648, 0x633,
0x627, 0x631, 0x649, 0x62a, 0x627, 0x6cb, 0x633, 0x64a, 0x645, 0x641, 0x6d5, 0x631,
0x648, 0x644, 0x633, 0x643, 0x648, 0x67e, 0x64a, 0x6d5, 0x633, 0x648, 0x641, 0x64a,
0x627, 0x633, 0x62a, 0x648, 0x643, 0x6af, 0x648, 0x644, 0x649, 0x645, 0x62a, 0x627,
0x644, 0x644, 0x64a, 0x646, 0x6cb, 0x644, 0x64a, 0x627, 0x646, 0x648, 0x6c6, 0x633,
0x643, 0x6c6, 0x627, 0x62f, 0x6cb, 0x633, 0x6c6, 0x627, 0x62a, 0x64a, 0x643, 0x627,
0x646, 0x6c6, 0x6d5, 0x646, 0x627, 0x6c6, 0x64a, 0x646, 0x64a, 0x6cb, 0x633, 0x6c6,
0x648, 0x644, 0x6af, 0x648, 0x6af, 0x631, 0x627, 0x62f, 0x6c6, 0x627, 0x631, 0x634,
0x627, 0x6c6, 0x627, 0x632, 0x627, 0x6af, 0x631, 0x6d5, 0x628, 0x633, 0x64a, 0x6cb,
0x631, 0x64a, 0x62d, 0x627, 0x646, 0x62a, 0x627, 0x646, 0x627, 0x646, 0x627, 0x631,
0x64a, 0x6c6, 0x6cb, 0x643, 0x631, 0x64a, 0x633, 0x62a, 0x645, 0x627, 0x633, 0x20,
0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x643, 0x648, 0x643, 0x648,
0x633, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x643, 0x648,
0x645, 0x648, 0x631, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649,
0x643, 0x6d5, 0x631, 0x6af, 0x6d5, 0x644, 0x6d5, 0x646, 0x645, 0x627, 0x6d5, 0x645,
0x627, 0x644, 0x62f, 0x64a, 0x6c6, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627,
0x631, 0x649, 0x645, 0x627, 0x6c6, 0x631, 0x64a, 0x643, 0x64a, 0x645, 0x627, 0x64a,
0x648, 0x62a, 0x62a, 0x627, 0x631, 0x6d5, 0x64a, 0x6cb, 0x646, 0x64a, 0x648, 0x646,
0x627, 0x67e, 0x64a, 0x627, 0x648, 0x643, 0x644, 0x6d5, 0x646, 0x62f, 0x628, 0x6cb,
0x6af, 0x6d5, 0x646, 0x6c6, 0x64a, 0x644, 0x686, 0x627, 0x62a, 0x6d5, 0x645, 0x62a,
0x631, 0x6cb, 0x643, 0x67e, 0x627, 0x633, 0x62d, 0x627, 0x20, 0x627, 0x631, 0x627,
0x644, 0x649, 0x6d5, 0x641, 0x627, 0x62a, 0x6d5, 0x641, 0x6cb, 0x646, 0x627, 0x641,
0x6cb, 0x62a, 0x64a, 0x6af, 0x627, 0x644, 0x627, 0x67e, 0x627, 0x6af, 0x648, 0x633,
0x6af, 0x627, 0x645, 0x628, 0x64a, 0x6d5, 0x6af, 0x6cb, 0x627, 0x62f, 0x627, 0x644,
0x643, 0x627, 0x646, 0x627, 0x644, 0x643, 0x627, 0x646, 0x62a, 0x648, 0x646, 0x643,
0x64a, 0x631, 0x64a, 0x62a, 0x64a, 0x645, 0x627, 0x62a, 0x64a, 0x643, 0x6cb, 0x633,
0x627, 0x64a, 0x6d5, 0x643, 0x6c6, 0x627, 0x62c, 0x627, 0x644, 0x6d5, 0x64a, 0x649,
0x646, 0x645, 0x627, 0x62c, 0x6cb, 0x631, 0x648, 0x645, 0x627, 0x631, 0x643, 0x64a,
0x632, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x645, 0x64a,
0x62f, 0x6cb, 0x6d5, 0x64a, 0x646, 0x627, 0x6cb, 0x631, 0x6cb, 0x646, 0x64a, 0x6cb,
0x6d5, 0x646, 0x6cb, 0x645, 0x6d5, 0x627, 0x67e, 0x627, 0x6af, 0x648, 0x2d, 0x67e,
0x627, 0x6af, 0x648, 0x67e, 0x627, 0x644, 0x627, 0x6cb, 0x67e, 0x64a, 0x62a, 0x643,
0x6d5, 0x631, 0x646, 0x67e, 0x648, 0x646, 0x627, 0x67e, 0x6d5, 0x67e, 0x648, 0x631,
0x62a, 0x2d, 0x645, 0x648, 0x631, 0x633, 0x628, 0x64a, 0x631, 0x627, 0x631, 0x648,
0x62a, 0x648, 0x646, 0x6af, 0x627, 0x633, 0x627, 0x64a, 0x67e, 0x627, 0x646, 0x62a,
0x627, 0x64a, 0x62a, 0x64a, 0x62a, 0x627, 0x631, 0x627, 0x6cb, 0x627, 0x62a, 0x648,
0x646, 0x6af, 0x627, 0x62a, 0x627, 0x67e, 0x6cb, 0x6cb, 0x6d5, 0x64a, 0x643, 0x6cb,
0x648, 0x644, 0x644, 0x64a, 0x633, 0x17a2, 0x17b6, 0x1794, 0x17ca, 0x17b8, 0x178a, 0x17d2,
0x1799, 0x17b6, 0x1793, 0x17a2, 0x17b6, 0x1780, 0x17d2, 0x179a, 0x17b6, 0x17a2, 0x17b6, 0x178c,
0x17b8, 0x179f, 0x17a2, 0x17b6, 0x1794, 0x17c1, 0x1794, 0x17b6, 0x17a2, 0x17b6, 0x179b, 0x17cb,
0x17a0, 0x17d2, 0x179f, 0x17c1, 0x17a2, 0x17b6, 0x179f, 0x17d2, 0x1798, 0x17b6, 0x179a, 0x17c9,
0x17b6, 0x1794, 0x17b6, 0x1798, 0x17c9, 0x17b6, 0x1780, 0x17bc, 0x1794, 0x1784, 0x17cb, 0x17a0,
0x17d2, 0x1782, 0x17b8, 0x1794, 0x1784, 0x17cb, 0x17a0, 0x17d2, 0x179f, 0x17ca, 0x17bb, 0x179b,
0x1794, 0x17ca, 0x17b8, 0x179f, 0x17d2, 0x179f, 0x17bc, 0x1794, 0x17d2, 0x179b, 0x17b6, 0x17c6,
0x1784, 0x1791, 0x17b8, 0x179a, 0x17c9, 0x17c1, 0x1794, 0x17d2, 0x179a, 0x17b6, 0x179f, 0x17d2,
0x17a0, 0x17b6, 0x179c, 0x17b8, 0x179b, 0x1794, 0x17ca, 0x17bc, 0x1799, 0x17c9, 0x17b6, 0x17c6,
0x1794, 0x17ca, 0x17bc, 0x179a, 0x17c9, 0x17b6, 0x1782, 0x17c2, 0x179a, 0x1780, 0x17b6, 0x179f,
0x17b6, 0x1794, 0x17d2, 0x179b, 0x17b6, 0x17c6, 0x1784, 0x1780, 0x17b6, 0x1788, 0x17c0, 0x178f,
0x17b6, 0x1780, 0x17bc, 0x178e, 0x17b6, 0x1782, 0x17d2, 0x179a, 0x17b8, 0x178a, 0x17b6, 0x1780,
0x17b6, 0x178a, 0x17b6, 0x17a2, 0x17ca, 0x17b8, 0x179f, 0x17b6, 0x17a1, 0x17b6, 0x17c6, 0x1787,
0x17b8, 0x1794, 0x17ca, 0x17bc, 0x1791, 0x17b8, 0x178c, 0x17bc, 0x17a2, 0x17ca, 0x17bc, 0x17a1,
0x17b6, 0x17a2, 0x17c2, 0x179b, 0x200b, 0x17a2, 0x17ca, 0x17bb, 0x1799, 0x1789, 0x17c0, 0x1793,
0x17a0, 0x17d2, 0x179c, 0x17d2, 0x179a, 0x17b8, 0x1790, 0x17c4, 0x1793, 0x17a0, 0x17d2, 0x1782,
0x17b6, 0x1794, 0x17bc, 0x179a, 0x17c9, 0x17bc, 0x1793, 0x17a0, 0x17b6, 0x179a, 0x17c9, 0x17b6,
0x179a, 0x17c9, 0x17c1, 0x1787, 0x17bc, 0x17a0, 0x17b6, 0x1793, 0x178e, 0x17c1, 0x1794, 0x17ca,
0x17ba, 0x17a0, 0x17d2, 0x1782, 0x1787, 0x17bc, 0x1794, 0x17b6, 0x1780, 0x17c6, 0x1794, 0x17c9,
0x17b6, 0x17a1, 0x17b6, 0x1781, 0x17b6, 0x1791, 0x17bb, 0x17c6, 0x1782, 0x17b8, 0x17a0, 0x17d2,
0x1782, 0x17b6, 0x179b, 0x17b8, 0x1782, 0x17b8, 0x1793, 0x179f, 0x17d2, 0x17a0, 0x17b6, 0x179f,
0x17b6, 0x17a1, 0x17b6, 0x17a0, 0x17d2, 0x1782, 0x17bc, 0x179f, 0x179b, 0x17b8, 0x1794, 0x17d2,
0x179a, 0x17b8, 0x179c, 0x17b8, 0x179b, 0x17a1, 0x17bc, 0x1798, 0x17c9, 0x17c1, 0x179b, 0x17bc,
0x17a2, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179b, 0x17bc, 0x1794, 0x17ca, 0x17bb, 0x1798, 0x1794,
0x17b6, 0x179f, 0x17ca, 0x17b8, 0x179b, 0x17bc, 0x179f, 0x17b6, 0x1780, 0x17b6, 0x1798, 0x17c9,
0x17b6, 0x17a1, 0x17b6, 0x1794, 0x17bc, 0x1798, 0x17c9, 0x17b6, 0x1796, 0x17bb, 0x1799, 0x178f,
0x17bc, 0x1798, 0x17c9, 0x17b6, 0x179f, 0x17c1, 0x179a, 0x17bb, 0x1799, 0x1794, 0x17b6, 0x1794,
0x17b6, 0x1793, 0x1798, 0x17c9, 0x17bc, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x178c, 0x17b8, 0x179f,
0x17d2, 0x1799, 0x17bc, 0x1798, 0x17c9, 0x17bb, 0x1784, 0x179a, 0x17c9, 0x17bc, 0x179c, 0x17b8,
0x1799, 0x17c9, 0x17b6, 0x178e, 0x17c3, 0x179a, 0x17c9, 0x17bc, 0x1794, 0x17ca, 0x17b8, 0x178a,
0x17d2, 0x1799, 0x17b6, 0x1798, 0x17c1, 0x178e, 0x17b6, 0x1793, 0x17b8, 0x1789, 0x17c9, 0x17b6,
0x1798, 0x17c9, 0x17c1, 0x178e, 0x17bd, 0x1780, 0x179f, 0x17bb, 0x178f, 0x17a2, 0x17c5, 0x17a0,
0x17d2, 0x1782, 0x17b6, 0x178c, 0x17bc, 0x17a0, 0x17d2, 0x1782, 0x17bc, 0x1796, 0x17d0, 0x179a,
0x178f, 0x17bc, 0x178e, 0x17bc, 0x179c, 0x17c9, 0x17bc, 0x179f, 0x17c5, 0x178f, 0x17bc, 0x1798,
0x17c9, 0x17c1, 0x1791, 0x17d2, 0x179a, 0x17b8, 0x1794, 0x17c9, 0x17bc, 0x179b, 0x17b8, 0x1791,
0x17bb, 0x1799, 0x1793, 0x17b8, 0x179f, 0x179c, 0x17b8, 0x1793, 0x178a, 0x17bb, 0x1780, 0x17a2,
0x17b6, 0x178a, 0x17b6, 0x1780, 0x17a2, 0x1784, 0x17cb, 0x1785, 0x17bc, 0x179a, 0x17c9, 0x17b6,
0x1780, 0x17a2, 0x1784, 0x17cb, 0x17a0, 0x17d2, 0x1782, 0x17b8, 0x17a1, 0x17b6, 0x17a2, 0x1784,
0x17cb, 0x1791, 0x17b8, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x17a2, 0x17b6, 0x179a, 0x17c9, 0x17b6,
0x17a0, 0x17d2, 0x1782, 0x17bb, 0x1799, 0x178e, 0x17b6, 0x1794, 0x17ca, 0x17bb, 0x1799, 0x178e,
0x17bc, 0x179f, 0x17ca, 0x17c2, 0x1780, 0x17b6, 0x178f, 0x17b6, 0x1798, 0x17c9, 0x17b6, 0x1780,
0x17b6, 0x1780, 0x17bc, 0x178a, 0x17bc, 0x1794, 0x17b6, 0x179f, 0x17bc, 0x179f, 0x17bb, 0x1799,
0x17a1, 0x17b6, 0x179a, 0x17b8, 0x1799, 0x17c9, 0x17bc, 0x17a0, 0x17b6, 0x1798, 0x17c9, 0x17c2,
0x1793, 0x178c, 0x17bc, 0x17a0, 0x17d2, 0x179f, 0x17b6, 0x179a, 0x17b8, 0x1799, 0x17c9, 0x17bc,
0x17a0, 0x17d2, 0x1782, 0x17b6, 0x17a1, 0x17c1, 0x17a0, 0x17d2, 0x1782, 0x17bc, 0x179f, 0x179f,
0x17b6, 0x179b, 0x17cb, 0x178f, 0x17b6, 0x179f, 0x17b6, 0x17c6, 0x1784, 0x17a0, 0x17d2, 0x179f,
0x1784, 0x17cb, 0x179f, 0x17b6, 0x1793, 0x17cb, 0x179b, 0x17bc, 0x17a2, 0x17ca, 0x17b8, 0x179f,
0x1791, 0x17bc, 0x1782, 0x17bc, 0x1798, 0x17c9, 0x1784, 0x17cb, 0x17a2, 0x17ca, 0x17bb, 0x1799,
0x179f, 0x17bc, 0x17a2, 0x17c3, 0x17a2, 0x17b6, 0x179a, 0x17c9, 0x17bc, 0x1794, 0x17b6, 0x17a2,
0x17b6, 0x179f, 0x1784, 0x17cb, 0x179f, 0x17d2, 0x1799, 0x17bb, 0x1784, 0x17a2, 0x17b6, 0x1791,
0x17b8, 0x1780, 0x17bc, 0x1780, 0x17b6, 0x1793, 0x1794, 0x17b6, 0x17a0, 0x17c0, 0x1794, 0x17b6,
0x17a0, 0x17c0, 0x1794, 0x1784, 0x17cb, 0x178a, 0x17c1, 0x179a, 0x17c9, 0x17b6, 0x179f, 0x1794,
0x17b6, 0x1794, 0x17b6, 0x178a, 0x17bb, 0x179f, 0x1794, 0x17c1, 0x17a1, 0x17b9, 0x1798, 0x1794,
0x17c1, 0x179b, 0x17b8, 0x1794, 0x17d2, 0x179b, 0x1784, 0x17cb, 0x200b, 0x179f, 0x17b6, 0x1794,
0x17cb, 0x17a1, 0x17bb, 0x1784, 0x1794, 0x17c5, 0x200b, 0x179c, 0x17b8, 0x179f, 0x17d2, 0x178f,
0x17b6, 0x1794, 0x17bc, 0x17a0, 0x17d2, 0x1782, 0x17c4, 0x178f, 0x17b6, 0x1794, 0x17ca, 0x17bc,
0x179f, 0x17ca, 0x17b8, 0x1781, 0x17c1, 0x1798, 0x1794, 0x17d2, 0x179a, 0x17b8, 0x178a, 0x1794,
0x17c1, 0x1781, 0x17c1, 0x1798, 0x1794, 0x17c9, 0x17bc, 0x17a0, 0x17d2, 0x1782, 0x17d2, 0x179a,
0x1784, 0x17cb, 0x200b, 0x1781, 0x17b6, 0x1793, 0x1781, 0x17bb, 0x1793, 0x1780, 0x17b6, 0x179a,
0x17c9, 0x17b6, 0x1780, 0x17b6, 0x179f, 0x1780, 0x17b6, 0x1799, 0x17c9, 0x17c1, 0x1793, 0x1780,
0x17c3, 0x1798, 0x17c2, 0x1793, 0x179f, 0x17ca, 0x17b8, 0x1780, 0x17b6, 0x17a0, 0x17d2, 0x1782,
0x17c4, 0x1787, 0x17b8, 0x17a0, 0x17bd, 0x17a0, 0x17bd, 0x179f, 0x17ca, 0x17b8, 0x17a2, 0x17ca,
0x17bc, 0x178a, 0x17b6, 0x178a, 0x20, 0x17a0, 0x17ca, 0x17bc, 0x17a2, 0x17b6, 0x179a, 0x17c9,
0x17c1, 0x179f, 0x1780, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17b6, 0x179a, 0x17b8, 0x1780, 0x17b6,
0x1780, 0x17d2, 0x179a, 0x17c1, 0x179f, 0x17d2, 0x178f, 0x17bb, 0x1793, 0x1782, 0x17bb, 0x1799,
0x17a2, 0x17b6, 0x1794, 0x17b6, 0x1780, 0x17bc, 0x179a, 0x17c9, 0x17b6, 0x1780, 0x17c5, 0x178a,
0x17b6, 0x1793, 0x1798, 0x17c9, 0x17b6, 0x1780, 0x179f, 0x17d2, 0x17a0, 0x17b6, 0x179c, 0x17b7,
0x1793, 0x178a, 0x17b6, 0x179c, 0x179f, 0x17bb, 0x1793, 0x178a, 0x17b6, 0x179c, 0x179f, 0x17bb,
0x1793, 0x200b, 0x1780, 0x17d2, 0x179a, 0x17b7, 0x1780, 0x178a, 0x17c2, 0x1793, 0x179c, 0x17be,
0x178a, 0x17c1, 0x178f, 0x17d2, 0x179a, 0x17bc, 0x17a2, 0x17ca, 0x17b8, 0x178f, 0x178a, 0x17bc,
0x1798, 0x17b8, 0x1793, 0x17b8, 0x1780, 0x17a2, 0x17c2, 0x178f, 0x1798, 0x17c9, 0x17bb, 0x1784,
0x178f, 0x17bb, 0x1784, 0x17a2, 0x17ca, 0x17b8, 0x179a, 0x17c9, 0x17bb, 0x1799, 0x178e, 0x17c1,
0x1797, 0x17b8, 0x17a2, 0x17c2, 0x179b, 0x179f, 0x17b6, 0x179b, 0x17cb, 0x179c, 0x17c9, 0x17b6,
0x178c, 0x17d0, 0x179a, 0x17a0, 0x17d2, 0x179c, 0x178f, 0x178e, 0x17c1, 0x179b, 0x179f, 0x17b6,
0x1793, 0x17cb, 0x17a0, 0x17d2, 0x179c, 0x17d0, 0x179a, 0x178f, 0x17b6, 0x17a1, 0x17c1, 0x17a0,
0x17d2, 0x179f, 0x17b6, 0x179a, 0x1780, 0x17d2, 0x179b, 0x17b6, 0x179f, 0x200b, 0x1794, 0x17c1,
0x1780, 0x17bc, 0x179f, 0x17c1, 0x1794, 0x17c1, 0x17a0, 0x17d2, 0x1782, 0x17d2, 0x179a, 0x17c1,
0x1793, 0x200b, 0x1791, 0x17bc, 0x1780, 0x17a0, 0x17d2, 0x1782, 0x17d2, 0x179a, 0x17be, 0x178e,
0x17b6, 0x178a, 0x17a0, 0x17d2, 0x1782, 0x17c4, 0x178a, 0x17c1, 0x179b, 0x17bc, 0x1794, 0x17c9,
0x17c1, 0x1780, 0x17d2, 0x179c, 0x17b6, 0x178f, 0x17c1, 0x1798, 0x17c9, 0x17b6, 0x17a1, 0x17b6,
0x17a0, 0x17d2, 0x1782, 0x17bb, 0x1799, 0x17c9, 0x17b6, 0x1782, 0x17b8, 0x179b, 0x17a0, 0x17d2,
0x1782, 0x17b8, 0x1799, 0x17c9, 0x17b6, 0x1793, 0x17a0, 0x17b6, 0x179b, 0x17b8, 0x17a0, 0x17d2,
0x179c, 0x17b6, 0x1780, 0x17a1, 0x17b6, 0x17a0, 0x17b6, 0x179c, 0x17c9, 0x17b6, 0x1793, 0x17a0,
0x17ca, 0x17c2, 0x1798, 0x17c9, 0x17bc, 0x179f, 0x17ca, 0x17b8, 0x17a1, 0x17bc, 0x17a5, 0x178e,
0x17d2, 0x178c, 0x17b6, 0x178e, 0x17b6, 0x1794, 0x17c9, 0x17bc, 0x179b, 0x17b8, 0x179f, 0x178e,
0x17bb, 0x1780, 0x20, 0x17a5, 0x178e, 0x17d2, 0x178c, 0x17b6, 0x178e, 0x17b6, 0x1798, 0x17c9,
0x17b6, 0x179a, 0x17c1, 0x1793, 0x1780, 0x17bc, 0x20, 0x17a5, 0x178e, 0x17d2, 0x178c, 0x17b6,
0x178e, 0x17b6, 0x1797, 0x17b8, 0x1792, 0x17be, 0x200b, 0x1794, 0x17ca, 0x17b9, 0x1780, 0x20,
0x17a5, 0x178e, 0x17d2, 0x178c, 0x17b6, 0x178e, 0x17b6, 0x1791, 0x17b8, 0x1780, 0x17d2, 0x179a,
0x17bb, 0x1784, 0x1790, 0x17c2, 0x179b, 0x20, 0x17a5, 0x178e, 0x17d2, 0x178c, 0x17b6, 0x178e,
0x17b6, 0x179c, 0x17c1, 0x179c, 0x17c9, 0x17c3, 0x20, 0x17a5, 0x178e, 0x17d2, 0x178c, 0x17b6,
0x1799, 0x17c9, 0x17b6, 0x178e, 0x17b6, 0x179c, 0x17b7, 0x1793, 0x179f, 0x17c1, 0x178e, 0x17c1,
0x179f, 0x20, 0x17a5, 0x178e, 0x17d2, 0x178c, 0x17b6, 0x178e, 0x17b6, 0x179c, 0x17b8, 0x178e,
0x17b6, 0x1798, 0x17c9, 0x17b6, 0x1780, 0x17cb, 0x20, 0x17a5, 0x178e, 0x17d2, 0x178c, 0x17b6,
0x178e, 0x17b6, 0x17a2, 0x17ca, 0x17b8, 0x1793, 0x17bc, 0x179c, 0x17b8, 0x1780, 0x17a2, 0x17ca,
0x17b8, 0x1780, 0x17b6, 0x17a1, 0x17bc, 0x17a2, 0x17ca, 0x17b8, 0x178f, 0x17a0, 0x17d2, 0x179f,
0x17b6, 0x1798, 0x17c9, 0x17b6, 0x17a2, 0x17ca, 0x17b8, 0x1780, 0x17a0, 0x17d2, 0x1799, 0x17bc,
0x1793, 0x17c0, 0x179b, 0x17bc, 0x17a2, 0x17ca, 0x17b8, 0x179c, 0x17b8, 0x179b, 0x1798, 0x17c9,
0x1784, 0x1791, 0x17b8, 0x179f, 0x17c1, 0x17a1, 0x17bc, 0x20, 0x1780, 0x17c1, 0x1793, 0x1791,
0x17bc, 0x1782, 0x17b8, 0x1780, 0x17d2, 0x179a, 0x17b6, 0x179b, 0x17c2, 0x1793, 0x178c, 0x17b8,
0x1780, 0x17a1, 0x17b6, 0x1794, 0x17c9, 0x17b6, 0x179f, 0x179b, 0x17b8, 0x1798, 0x17c9, 0x17b6,
0x17a1, 0x17bc, 0x17a2, 0x1784, 0x17cb, 0x17a0, 0x17d2, 0x1782, 0x17b7, 0x17a1, 0x17c1, 0x179f,
0x17a1, 0x17bc, 0x179c, 0x17be, 0x200b, 0x1796, 0x17d2, 0x179a, 0x17b8, 0x1793, 0x200b, 0x17a0,
0x17d2, 0x1782, 0x17d0, 0x179a, 0x1791, 0x17d0, 0x179a, 0x1798, 0x17c9, 0x17b6, 0x1780, 0x179f,
0x17c1, 0x17a2, 0x17bc, 0x1798, 0x17c9, 0x17b6, 0x178e, 0x17b6, 0x17a0, 0x17d2, 0x1782, 0x17b6,
0x1798, 0x17c9, 0x17b6, 0x178e, 0x17b6, 0x17a2, 0x17ca, 0x17bc, 0x179f, 0x1798, 0x17c9, 0x17b6,
0x179a, 0x17b8, 0x17a0, 0x17d2, 0x1782, 0x17bb, 0x178f, 0x1798, 0x17c9, 0x17b6, 0x1791, 0x17b8,
0x1793, 0x17b8, 0x1780, 0x1798, 0x17c9, 0x17b6, 0x178f, 0x17b6, 0x1798, 0x17c9, 0x17bc, 0x179a,
0x17c9, 0x17bc, 0x179f, 0x1798, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x179f, 0x17b6, 0x178f, 0x17a1,
0x1784, 0x17cb, 0x1798, 0x17c9, 0x17c1, 0x178e, 0x17bc, 0x1798, 0x17b8, 0x1793, 0x17b8, 0x1798,
0x17c9, 0x17c1, 0x179a, 0x17b8, 0x178a, 0x17b6, 0x1798, 0x17c1, 0x178f, 0x17d2, 0x179b, 0x17b6,
0x20, 0x1780, 0x17b6, 0x178f, 0x17d2, 0x179b, 0x17b6, 0x1791, 0x17b8, 0x1780, 0x17d2, 0x179a,
0x17bb, 0x1784, 0x1798, 0x17c9, 0x17b7, 0x1785, 0x179f, 0x17ca, 0x17b8, 0x1780, 0x17bc, 0x1798,
0x17b8, 0x17a0, 0x17d2, 0x1782, 0x17b8, 0x17a1, 0x17bb, 0x1784, 0x1798, 0x17c9, 0x17bb, 0x1784,
0x178f, 0x17bb, 0x1793, 0x1798, 0x17c9, 0x17bb, 0x1793, 0x178f, 0x17c1, 0x179a, 0x17c9, 0x17c1,
0x1798, 0x17c9, 0x17bb, 0x1784, 0x178f, 0x17c1, 0x179c, 0x17b8, 0x178a, 0x17c1, 0x17a2, 0x17bc,
0x1798, 0x17c9, 0x17bb, 0x1784, 0x179f, 0x17ca, 0x17c2, 0x179a, 0x17c9, 0x17b6, 0x178f, 0x200b,
0x178e, 0x17b6, 0x179f, 0x17d2, 0x179f, 0x17bc, 0x1789, 0x17bc, 0x179c, 0x1799, 0x17c9, 0x1780,
0x178e, 0x17bc, 0x179a, 0x17c9, 0x17bb, 0x1793, 0x1789, 0x17c9, 0x17b6, 0x1794, 0x17ca, 0x17ba,
0x17a1, 0x17b6, 0x20, 0x178a, 0x17b6, 0x1780, 0x17bc, 0x178f, 0x17b6, 0x200b, 0x1781, 0x17b6,
0x1784, 0x1787, 0x17be, 0x1784, 0x179f, 0x17b7, 0x1793, 0x1792, 0x17ba, 0x20, 0x178a, 0x17b6,
0x1780, 0x17bc, 0x178f, 0x17b6, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x1787, 0x17be, 0x1784,
0x1789, 0x17bc, 0x179c, 0x200b, 0x179f, 0x17b6, 0x17a1, 0x17b9, 0x1798, 0x20, 0x178a, 0x17b6,
0x1780, 0x17bc, 0x178f, 0x17b6, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x1787, 0x17be, 0x1784,
0x1793, 0x17bc, 0x1780, 0x17a2, 0x17bc, 0x1787, 0x17b8, 0x178e, 0x17b6, 0x17a0, 0x17d2, 0x1780,
0x17b6, 0x1794, 0x17c9, 0x17b6, 0x178e, 0x17b6, 0x1798, 0x17c9, 0x17b6, 0x1794, 0x17c9, 0x17b6,
0x179a, 0x17c9, 0x17b6, 0x1798, 0x17c9, 0x17b6, 0x179a, 0x17b8, 0x1794, 0x17bc, 0x1795, 0x17bc,
0x1793, 0x17b8, 0x1780, 0x1796, 0x17d0, 0x179a, 0x17a2, 0x17bc, 0x1794, 0x17d2, 0x179a, 0x17c9,
0x17b6, 0x17c6, 0x1784, 0x1796, 0x17d0, 0x179a, 0x17a2, 0x17c1, 0x179f, 0x17d2, 0x1794, 0x17c9,
0x17b6, 0x1789, 0x1796, 0x17d0, 0x179a, 0x178f, 0x17bc, 0x200b, 0x179c, 0x17b7, 0x179b, 0x17a0,
0x17bc, 0x1796, 0x17d0, 0x179a, 0x178f, 0x17bc, 0x179a, 0x17b8, 0x1780, 0x17bc, 0x1796, 0x17bb,
0x1793, 0x178f, 0x17b6, 0x20, 0x17a2, 0x17b6, 0x179a, 0x17c9, 0x17c1, 0x178e, 0x17b6, 0x179a,
0x17c9, 0x17b6, 0x1793, 0x17cb, 0x1783, 0x17b8, 0x1793, 0x200b, 0x17a2, 0x17ca, 0x17b8, 0x1793,
0x17a1, 0x17b7, 0x178f, 0x179a, 0x17c9, 0x17c1, 0x179f, 0x17ca, 0x17b8, 0x17a0, 0x17d2, 0x179c,
0x17b8, 0x179a, 0x17c9, 0x17c1, 0x17a0, 0x17d2, 0x1782, 0x17b8, 0x178e, 0x17b6, 0x179a, 0x17c9,
0x17c1, 0x200b, 0x179f, 0x17ca, 0x17bc, 0x17a1, 0x17bc, 0x178f, 0x179a, 0x17b8, 0x1799, 0x17c9,
0x17bc, 0x1794, 0x17d2, 0x179a, 0x17b6, 0x1793, 0x1780, 0x17bc, 0x179f, 0x17b6, 0x1793, 0x17cb,
0x178f, 0x17b6, 0x179a, 0x17b9, 0x1798, 0x179f, 0x17b6, 0x1793, 0x17cb, 0x1791, 0x17b8, 0x17a2,
0x17c1, 0x17a0, 0x17d2, 0x1782, 0x17c4, 0x179f, 0x17b6, 0x1793, 0x17cb, 0x178a, 0x17bc, 0x1798,
0x17c9, 0x17b6, 0x17c6, 0x1784, 0x179f, 0x17c5, 0x200b, 0x1794, 0x17c9, 0x17bc, 0x17a1, 0x17bc,
0x17a2, 0x17ca, 0x17b8, 0x178f, 0x17bc, 0x1782, 0x17d0, 0x179a, 0x178f, 0x17bc, 0x1798, 0x17b8,
0x178f, 0x179f, 0x17ca, 0x17b8, 0x178f, 0x1780, 0x17b6, 0x179f, 0x17b6, 0x17c6, 0x1784, 0x200b,
0x1794, 0x17b6, 0x1792, 0x17b8, 0x179b, 0x17b8, 0x1798, 0x17c9, 0x17b8, 0x179f, 0x17b6, 0x17c6,
0x1784, 0x200b, 0x1785, 0x1793, 0x179f, 0x17b6, 0x17c6, 0x1784, 0x200b, 0x1783, 0x17b8, 0x178f,
0x179f, 0x17b6, 0x17c6, 0x1784, 0x200b, 0x179b, 0x17bc, 0x179f, 0x17c0, 0x179f, 0x17b6, 0x17c6,
0x1784, 0x200b, 0x178f, 0x17bc, 0x1798, 0x17c9, 0x17b6, 0x179f, 0x17cb, 0x179f, 0x17b6, 0x17c6,
0x1784, 0x200b, 0x179c, 0x17b8, 0x1793, 0x179f, 0x17c1, 0x1793, 0x179f, 0x17d2, 0x179c, 0x17b8,
0x178f, 0x200b, 0x1781, 0x17ba, 0x179a, 0x17b7, 0x1793, 0x178f, 0x17c1, 0x17a0, 0x17d2, 0x1782,
0x17bc, 0x179f, 0x17ca, 0x17b8, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x179b, 0x17cb, 0x1794, 0x17c9,
0x17b6, 0x1792, 0x17bb, 0x17a1, 0x17c1, 0x1791, 0x17b8, 0x1799, 0x17d2, 0x1799, 0x17bc, 0x17a2,
0x17b6, 0x178e, 0x17b6, 0x178f, 0x17bc, 0x179a, 0x17c9, 0x17bb, 0x1793, 0x178f, 0x17bc, 0x178f,
0x17bc, 0x178f, 0x17bc, 0x17a1, 0x17b6, 0x179c, 0x17c9, 0x17b6, 0x1793, 0x17cb, 0x1780, 0x17bc,
0x179c, 0x17be, 0x179c, 0x17c9, 0x17b6, 0x1799, 0x17a0, 0x179f, 0x179c, 0x17b8, 0x1793, 0x17b8,
0x1797, 0x17b7, 0x1780, 0x1799, 0x17c9, 0x17b6, 0x1782, 0x17bc, 0x178f, 0x17b6, 0x178f, 0x1780,
0x17b6, 0x179f, 0x17b8, 0x178a, 0x17b6, 0x179c, 0x17b8, 0x179f, 0x178c, 0x17bb, 0x1799, 0x1798,
0x17c9, 0x17bb, 0x1784, 0x178a, 0x17ba, 0x17a2, 0x17ca, 0x17bb, 0x1799, 0x179c, 0x17b8, 0x179b,
0x1798, 0x17c9, 0x17b6, 0x1780, 0x1781, 0x17d2, 0x179c, 0x17b6, 0x179a, 0x17b8, 0x1798, 0x17c9,
0x17c5, 0x179f, 0x17b6, 0x1793, 0x17cb, 0x1798, 0x17c9, 0x17b6, 0x1780, 0x17cb, 0x1798, 0x17bb,
0x1799, 0x178a, 0x17bc, 0x1795, 0x17b6, 0x179b, 0x17cb, 0x1798, 0x17d0, 0x179a, 0x179a, 0x17c9,
0x17bc, 0x1792, 0x17b8, 0x179a, 0x17c9, 0x17b6, 0x179f, 0x17ca, 0x17b8, 0x1799, 0x17c9, 0x17bc,
0x179c, 0x17c9, 0x17b6, 0x178f, 0x17d2, 0x179a, 0x17bc, 0x179b, 0x179c, 0x17c9, 0x17bc, 0x179f,
0x17d2, 0x178f, 0x17bb, 0x1780, 0x17a1, 0x17bb, 0x1784, 0x200b, 0x1799, 0x17c9, 0x17b6, 0x200b,
0x1794, 0x17ca, 0x17b8, 0x1799, 0x17c1, 0x1793, 0x17a2, 0x17b6, 0x178a, 0x17c2, 0x1793, 0x17a2,
0x17b6, 0x179b, 0x17cb, 0x1798, 0x17c9, 0x17b6, 0x1791, 0x17b8, 0x17a2, 0x17b6, 0x1798, 0x17c9,
0x17b6, 0x1793, 0x17cb, 0x17a2, 0x17b6, 0x178e, 0x17b6, 0x178c, 0x17b8, 0x17a2, 0x17b6, 0x1780,
0x1791, 0x17bc, 0x17a2, 0x17b6, 0x1780, 0x1791, 0x17bc, 0x1794, 0x17ca, 0x17b8, 0x17a2, 0x17b6,
0x179f, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x1794, 0x17b6, 0x178f, 0x17a2, 0x17b6, 0x1791, 0x17b8,
0x179a, 0x17c9, 0x17bc, 0x1794, 0x17b6, 0x1780, 0x178a, 0x17b6, 0x178a, 0x1794, 0x17b6, 0x179a,
0x17c9, 0x17c2, 0x1793, 0x1794, 0x17b6, 0x1782, 0x17bc, 0x1794, 0x17b6, 0x1784, 0x1780, 0x1780,
0x1794, 0x17b6, 0x1793, 0x17bc, 0x179b, 0x1794, 0x17c1, 0x179a, 0x17bc, 0x178f, 0x1794, 0x17ca,
0x17b8, 0x179f, 0x17d2, 0x1780, 0x17c1, 0x1780, 0x1796, 0x17d2, 0x179a, 0x17bb, 0x1799, 0x178e,
0x17c1, 0x1788, 0x17b8, 0x178f, 0x17b6, 0x1780, 0x17bc, 0x17a1, 0x17bb, 0x17c6, 0x1794, 0x17bc,
0x178a, 0x17b6, 0x1798, 0x17c9, 0x17b6, 0x179f, 0x178a, 0x17b6, 0x1780, 0x17d2, 0x1780, 0x17b6,
0x178a, 0x17c1, 0x179b, 0x17b8, 0x178c, 0x17bb, 0x1799, 0x1794, 0x17c3, 0x178c, 0x17bc, 0x179f,
0x17d2, 0x17a0, 0x17b6, 0x1793, 0x17cb, 0x1794, 0x17c1, 0x17a0, 0x17d2, 0x179c, 0x17c9, 0x17b6,
0x1798, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x1782, 0x17bb, 0x179f, 0x17d2, 0x178f, 0x17b6, 0x17a0,
0x17c1, 0x1794, 0x179a, 0x17bb, 0x1793, 0x17a0, 0x17bc, 0x1787, 0x17b8, 0x1798, 0x17b8, 0x1789,
0x17a0, 0x17bb, 0x1784, 0x1780, 0x17bb, 0x1784, 0x17a0, 0x17bc, 0x179c, 0x17a2, 0x17c0, 0x179a,
0x1782, 0x17bb, 0x178f, 0x179f, 0x17d2, 0x1780, 0x17b7, 0x17cd, 0x17a0, 0x17d2, 0x179f, 0x17b6,
0x1780, 0x17b6, 0x179a, 0x178f, 0x17b6, 0x1785, 0x17b6, 0x1799, 0x17c9, 0x17b6, 0x1797, 0x17bc,
0x179a, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x179f, 0x17ca, 0x17c1, 0x179a, 0x17bb, 0x1799, 0x179f,
0x17b6, 0x17a1, 0x17b7, 0x1798, 0x1780, 0x17b6, 0x1794, 0x17ca, 0x17bb, 0x179b, 0x1780, 0x17b6,
0x17c6, 0x1786, 0x17b6, 0x178f, 0x1780, 0x17b6, 0x1780, 0x17b6, 0x179a, 0x17c9, 0x17b6, 0x1787,
0x17b8, 0x1780, 0x17b6, 0x178f, 0x17cb, 0x1798, 0x17c9, 0x17b6, 0x1793, 0x17cb, 0x178c, 0x17bc,
0x1781, 0x17b6, 0x1793, 0x17cb, 0x178c, 0x17b8, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x1780, 0x17bb,
0x179b, 0x1780, 0x17b6, 0x178f, 0x17b6, 0x1780, 0x17d2, 0x179a, 0x17b6, 0x179f, 0x1793, 0x17bb,
0x1799, 0x17c9, 0x17b6, 0x1780, 0x1780, 0x17bc, 0x17a1, 0x17b6, 0x17a1, 0x17b6, 0x17c6, 0x1796,
0x17bd, 0x179a, 0x1783, 0x17ba, 0x178f, 0x1785, 0x17b8, 0x1784, 0x1780, 0x17bc, 0x179c, 0x17c9,
0x17c2, 0x178f, 0x1798, 0x17c9, 0x17b6, 0x1780, 0x17b6, 0x179c, 0x1798, 0x17c9, 0x17b6, 0x17a0,
0x17d2, 0x1782, 0x17b6, 0x178a, 0x17b6, 0x1793, 0x1798, 0x17c9, 0x17b6, 0x1780, 0x17b6, 0x179f,
0x179f, 0x17b6, 0x179a, 0x1798, 0x17c9, 0x17b6, 0x1793, 0x17b8, 0x179b, 0x1798, 0x17c9, 0x17b6,
0x179f, 0x17d2, 0x1780, 0x17b6, 0x178f, 0x1793, 0x17b8, 0x1780, 0x17bc, 0x179f, 0x17ca, 0x17b8,
0x178e, 0x17bc, 0x179c, 0x17c9, 0x17bc, 0x1783, 0x17bc, 0x179f, 0x178e, 0x17c1, 0x178f, 0x179f,
0x17d2, 0x1780, 0x17b7, 0x17cd, 0x178e, 0x17bc, 0x179c, 0x17c9, 0x17bc, 0x179f, 0x17ca, 0x17b8,
0x1794, 0x17ca, 0x17ba, 0x1780, 0x17a2, 0x17bc, 0x1798, 0x179f, 0x17d2, 0x1780, 0x17b7, 0x17cd,
0x17a2, 0x17bc, 0x179a, 0x17c9, 0x17b6, 0x179b, 0x17cb, 0x1797, 0x17d2, 0x1793, 0x17c6, 0x1796,
0x17c1, 0x1789, 0x1794, 0x17c9, 0x17bb, 0x1793, 0x1791, 0x17b8, 0x17a2, 0x17b6, 0x178e, 0x17b6,
0x1780, 0x17cb, 0x1796, 0x17d2, 0x1799, 0x17bb, 0x1784, 0x1799, 0x17c9, 0x17b6, 0x1784, 0x1780,
0x17bc, 0x179f, 0x17d2, 0x178a, 0x17c1, 0x178e, 0x17c1, 0x1782, 0x17b8, 0x17a0, 0x17d2, 0x179f,
0x17ca, 0x17b8, 0x17a1, 0x17bc, 0x178a, 0x17b6, 0x179a, 0x17b8, 0x1799, 0x17c9, 0x17b6, 0x178a,
0x179f, 0x17b6, 0x1781, 0x17b6, 0x179b, 0x17b8, 0x1793, 0x179f, 0x17b6, 0x1798, 0x17c9, 0x17b6,
0x1780, 0x17b6, 0x1793, 0x17cb, 0x179f, 0x17c1, 0x17a2, 0x17ca, 0x17bc, 0x179b, 0x179f, 0x17c0,
0x1784, 0x17a0, 0x17c3, 0x179f, 0x17b7, 0x1784, 0x17d2, 0x17a0, 0x1794, 0x17bc, 0x179a, 0x17b8,
0x179f, 0x17d2, 0x179a, 0x17b8, 0x178f, 0x1793, 0x17b8, 0x1780, 0x17bc, 0x179b, 0x17b8, 0x1798,
0x178f, 0x17c3, 0x1794, 0x17c9, 0x17b7, 0x178f, 0x17b6, 0x179f, 0x17d2, 0x1780, 0x17b7, 0x1793,
0x1794, 0x17ca, 0x17b8, 0x179b, 0x17b8, 0x179f, 0x17ca, 0x17b8, 0x178f, 0x17c1, 0x17a0, 0x17c1,
0x179a, 0x17c9, 0x1784, 0x17cb, 0x1792, 0x17b8, 0x1798, 0x1797, 0x17bc, 0x178f, 0x17bc, 0x1780,
0x17d2, 0x1799, 0x17bc, 0x1790, 0x1798, 0x179f, 0x17d2, 0x1780, 0x17b7, 0x17cd, 0x17a2, 0x17ca,
0x17bc, 0x17a1, 0x1784, 0x17cb, 0x1794, 0x17b6, 0x1791, 0x17d0, 0x179a, 0x17a2, 0x17ca, 0x17bc,
0x179a, 0x17b6, 0x17c6, 0x17a0, 0x17d2, 0x1782, 0x17b8, 0x17a2, 0x17bc, 0x179f, 0x17d2, 0x178f,
0x178e, 0x17c1, 0x179a, 0x17c9, 0x17b6, 0x179c, 0x17c0, 0x1784, 0x1785, 0x1793, 0x17d2, 0x1791,
0x1793, 0x17cd, 0x179c, 0x17c9, 0x17d2, 0x179b, 0x17b6, 0x178a, 0x17b8, 0x179c, 0x17c9, 0x17bc,
0x179f, 0x17d2, 0x178f, 0x17bb, 0x1780, 0x1799, 0x17c9, 0x17b6, 0x1782, 0x17bb, 0x178f, 0x179f,
0x17d2, 0x1780, 0x17b7, 0x17cd, 0x179a, 0x17c9, 0x1784, 0x17cb, 0x17a0, 0x17d2, 0x1782, 0x17bc,
0x1793, 0x1799, 0x17c9, 0x17c1, 0x1780, 0x17b6, 0x1792, 0x17ba, 0x179a, 0x17b7, 0x1793, 0x1794,
0x17ca, 0x17be, 0x1780, 0x1799, 0x17c9, 0x17c1, 0x179a, 0x17c9, 0x17c1, 0x179c, 0x17c9, 0x17b6,
0x1793, 0x17cb, 0x17a2, 0x17b6, 0x179f, 0x17bc, 0x179a, 0x17c9, 0x17c1, 0x179f, 0x1794, 0x17c9,
0x17ba, 0x1798, 0x17bb, 0x1799, 0x178a, 0x17b6, 0x1780, 0x17b6, 0x178e, 0x17b6, 0x179a, 0x17b8,
0x1780, 0x17b6, 0x1794, 0x17cb, 0x179c, 0x17c2, 0x179a, 0x17a0, 0x17d2, 0x179c, 0x17b6, 0x179a,
0x17c9, 0x17bc, 0x1798, 0x17c9, 0x17b6, 0x178a, 0x17c0, 0x179a, 0x17c9, 0x17b6, 0x179a, 0x17c9,
0x17c3, 0x1780, 0x17d2, 0x1799, 0x17b6, 0x179c, 0x17b7, 0x1780, 0x17a0, 0x17d2, 0x179f, 0x1780,
0x17a0, 0x17d2, 0x179f, 0x17ca, 0x17b8, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x178f, 0x17d2,
0x1794, 0x17bc, 0x1784, 0x179f, 0x17b6, 0x17c6, 0x1784, 0x17a0, 0x17c1, 0x17a1, 0x17c1, 0x178e,
0x17b6, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x179b, 0x17b8, 0x17a2, 0x17b6, 0x178a, 0x17c1,
0x17a1, 0x17c2, 0x178f, 0x1794, 0x17d2, 0x179a, 0x17b8, 0x179f, 0x1794, 0x17ca, 0x17c2, 0x1793,
0x1794, 0x17d2, 0x179a, 0x17bc, 0x1780, 0x1781, 0x17b7, 0x1793, 0x17a0, 0x17b8, 0x179b, 0x178a,
0x17b6, 0x179c, 0x17b8, 0x1793, 0x17a2, 0x17ca, 0x17bb, 0x1799, 0x1780, 0x17d2, 0x179b, 0x17b6,
0x17a0, 0x17bc, 0x1794, 0x17b6, 0x178f, 0x179b, 0x17b8, 0x1793, 0x178c, 0x17b8, 0x1798, 0x17c9,
0x17c2, 0x1793, 0x17a1, 0x178f, 0x17a0, 0x17bc, 0x179c, 0x17c9, 0x17c1, 0x1798, 0x17c9, 0x17c1,
0x179b, 0x1794, 0x17ca, 0x1793, 0x1797, 0x17ba, 0x1792, 0x179f, 0x17ca, 0x17b8, 0x178a, 0x1793,
0x17b8, 0x17a2, 0x17b6, 0x17c6, 0x179f, 0x17d2, 0x1791, 0x17c2, 0x178a, 0x17b6, 0x17c6, 0x17a2,
0x1784, 0x17cb, 0x178a, 0x17bc, 0x179a, 0x17c9, 0x17b6, 0x17a2, 0x17b6, 0x179f, 0x17d2, 0x178f,
0x17d2, 0x179a, 0x17b6, 0x1781, 0x17b6, 0x1793, 0x17cb, 0x17a2, 0x17b6, 0x1790, 0x17c2, 0x1793,
0x1794, 0x17c2, 0x179b, 0x1780, 0x17d2, 0x179a, 0x17b6, 0x178a, 0x1794, 0x17c2, 0x179a, 0x17a1,
0x17b6, 0x17c6, 0x1784, 0x1794, 0x17d2, 0x179a, 0x17b6, 0x1791, 0x17b8, 0x179f, 0x17d2, 0x179b,
0x17b6, 0x179c, 0x17c9, 0x17b6, 0x1796, 0x17d2, 0x179a, 0x17bb, 0x1785, 0x179f, 0x17c2, 0x179b,
0x1794, 0x17ca, 0x17bb, 0x1799, 0x1780, 0x17b6, 0x179a, 0x17c9, 0x17c1, 0x179f, 0x1794, 0x17ca,
0x17bb, 0x1799, 0x178a, 0x17b6, 0x1794, 0x17c9, 0x17c2, 0x179f, 0x1794, 0x17ca, 0x17bc, 0x179f,
0x17ca, 0x17b8, 0x1784, 0x17a0, 0x17d2, 0x1782, 0x17c2, 0x1793, 0x1787, 0x17b8, 0x179f, 0x17ca,
0x17b8, 0x178e, 0x17bc, 0x1780, 0x17bc, 0x1794, 0x17c9, 0x17b7, 0x1793, 0x17a0, 0x17b6, 0x1780,
0x178c, 0x17bb, 0x1799, 0x1794, 0x17d2, 0x179b, 0x17b6, 0x17c6, 0x1784, 0x17a0, 0x17d2, 0x179f,
0x17ca, 0x17b8, 0x1794, 0x17d2, 0x179a, 0x17b6, 0x179b, 0x17cb, 0x178f, 0x17b6, 0x17a0, 0x17d2,
0x1782, 0x17c1, 0x1793, 0x179f, 0x17ca, 0x17b8, 0x17a0, 0x17c2, 0x179b, 0x179f, 0x17ca, 0x17b7,
0x1793, 0x1782, 0x17b8, 0x17a2, 0x17c2, 0x179b, 0x17a2, 0x17bb, 0x17a0, 0x17d2, 0x179c, 0x1798,
0x17c9, 0x17c2, 0x1793, 0x17a2, 0x17ca, 0x17b8, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x1794,
0x17ca, 0x17bb, 0x179b, 0x1787, 0x17c1, 0x179f, 0x17ca, 0x17b8, 0x1780, 0x17b6, 0x179b, 0x17b8,
0x1793, 0x17b8, 0x1784, 0x17a0, 0x17d2, 0x179a, 0x17d2, 0x1782, 0x17b6, 0x178a, 0x1782, 0x17b8,
0x179a, 0x17c9, 0x17bc, 0x179c, 0x1780, 0x17c0, 0x179c, 0x179b, 0x17b8, 0x179f, 0x1794, 0x17c4,
0x1793, 0x1787, 0x17bc, 0x1794, 0x17a0, 0x17d2, 0x179f, 0x17b6, 0x178e, 0x17b6, 0x17a1, 0x17bb,
0x1784, 0x178a, 0x17cd, 0x179b, 0x17bb, 0x1785, 0x179f, 0x17c6, 0x1794, 0x17bd, 0x1798, 0x17c9,
0x17b6, 0x178c, 0x17d2, 0x179a, 0x17b8, 0x178a, 0x1798, 0x17c9, 0x17b6, 0x179b, 0x17cb, 0x178f,
0x17cd, 0x1798, 0x17c9, 0x17b6, 0x179a, 0x17c0, 0x17a0, 0x17b6, 0x17c6, 0x1798, 0x17b8, 0x1793,
0x179f, 0x17d2, 0x1780, 0x17b7, 0x17cd, 0x1798, 0x17c9, 0x17bc, 0x178e, 0x17b6, 0x1780, 0x17bc,
0x1798, 0x17bc, 0x179f, 0x17d2, 0x1782, 0x17bc, 0x17a2, 0x17bc, 0x179f, 0x17d2, 0x179b, 0x17bc,
0x1794, 0x17c9, 0x17b6, 0x179a, 0x17b8, 0x179f, 0x1794, 0x17c9, 0x17bc, 0x178a, 0x17a0, 0x17d2,
0x1782, 0x17c4, 0x179a, 0x17b8, 0x1780, 0x17b6, 0x1794, 0x17d2, 0x179a, 0x17b6, 0x1780, 0x179a,
0x17b8, 0x17a0, 0x17d2, 0x1780, 0x17b6, 0x179f, 0x17b6, 0x1798, 0x17c9, 0x17b6, 0x179a, 0x17c9,
0x17b6, 0x179f, 0x17b6, 0x17c6, 0x1784, 0x1798, 0x17c9, 0x17b6, 0x179a, 0x17c9, 0x17b6, 0x17c6,
0x1784, 0x179f, 0x17b6, 0x179a, 0x17c9, 0x17b6, 0x1799, 0x17c9, 0x17c1, 0x179c, 0x17c9, 0x17bc,
0x179f, 0x17b6, 0x179a, 0x17c9, 0x17b6, 0x178f, 0x17b6, 0x179c, 0x179f, 0x17ca, 0x17b8, 0x1798,
0x17a0, 0x17d2, 0x179c, 0x17be, 0x179a, 0x17c9, 0x17bb, 0x1794, 0x17c9, 0x17bc, 0x179b, 0x179f,
0x17d2, 0x1780, 0x17bb, 0x1794, 0x1787, 0x17c1, 0x179f, 0x17bc, 0x17a0, 0x17ca, 0x17d2, 0x179c,
0x17b8, 0x1799, 0x17c9, 0x17b6, 0x179f, 0x17d2, 0x178f, 0x17bb, 0x1780, 0x1781, 0x17bb, 0x179b,
0x178f, 0x17b6, 0x179b, 0x17b7, 0x1793, 0x1791, 0x17b8, 0x179a, 0x17c9, 0x17b6, 0x178e, 0x17c1,
0x17a2, 0x17bb, 0x179b, 0x1799, 0x17c9, 0x17b6, 0x178e, 0x17bc, 0x179c, 0x179f, 0x17d2, 0x1780,
0x17b7, 0x17cd, 0x179c, 0x17c9, 0x17b6, 0x178c, 0x17bc, 0x179f, 0x179c, 0x17c9, 0x17b6, 0x1791,
0x17b8, 0x1780, 0x1784, 0x17cb, 0x179c, 0x17b8, 0x1799, 0x17c2, 0x1793, 0x179c, 0x17b8, 0x179b,
0x1793, 0x17b8, 0x1789, 0x17bc, 0x179f, 0x179c, 0x17c9, 0x17bb, 0x179b, 0x17a0, 0x17d2, 0x1782,
0x17c4, 0x17a0, 0x17d2, 0x1782, 0x17d2, 0x179a, 0x17b6, 0x178a, 0x179c, 0x17c9, 0x17b6, 0x179f,
0x17bc, 0x179c, 0x17b8, 0x179f, 0x17b6, 0x1782, 0x17d2, 0x179a, 0x17c2, 0x1794, 0x17a0, 0x17ca,
0x17d2, 0x179f, 0x17bc, 0x179a, 0x17b8, 0x1785, 0x17a2, 0x1784, 0x17cb, 0x178f, 0x17b6, 0x178e,
0x17b6, 0x178e, 0x17b6, 0x179a, 0x17b8, 0x179c, 0x1782, 0x17d2, 0x179a, 0x17b8, 0x179f, 0x17d2,
0x1798, 0x17b6, 0x179f, 0x1780, 0x17bc, 0x1780, 0x17bc, 0x179f, 0x1780, 0x17bc, 0x1798, 0x17d0,
0x179a, 0x1783, 0x17ba, 0x17a0, 0x17d2, 0x1782, 0x17bb, 0x1799, 0x17a1, 0x17c2, 0x1793, 0x1798,
0x17c9, 0x17b6, 0x17a0, 0x17c1, 0x1798, 0x17c9, 0x17b6, 0x179b, 0x17cb, 0x178c, 0x17b8, 0x179c,
0x1798, 0x17c9, 0x17bc, 0x179a, 0x17b8, 0x179f, 0x1798, 0x17c9, 0x17b6, 0x1799, 0x17bb, 0x178f,
0x179a, 0x17c1, 0x17a2, 0x17ca, 0x17bb, 0x1799, 0x17c9, 0x17bb, 0x1784, 0x17a2, 0x17b6, 0x1796,
0x17b8, 0x17a2, 0x1780, 0x17a1, 0x17c2, 0x1793, 0x1794, 0x17bc, 0x17a0, 0x17d2, 0x1782, 0x17c2,
0x1793, 0x179c, 0x17b8, 0x179b, 0x1785, 0x17b6, 0x1790, 0x17b6, 0x17c6, 0x1788, 0x17bc, 0x17a2,
0x17bb, 0x1780, 0x17a2, 0x17ca, 0x17b8, 0x179f, 0x17d2, 0x1791, 0x17d0, 0x179a, 0x17a2, 0x17ca,
0x17b8, 0x17a0, 0x17d2, 0x179c, 0x17b6, 0x178f, 0x17c1, 0x17a0, 0x17d2, 0x179c, 0x17b6, 0x1780,
0x17c5, 0x17a0, 0x17d2, 0x179c, 0x17bc, 0x17a0, 0x17d2, 0x179c, 0x17b8, 0x1787, 0x17b8, 0x17a0,
0x17ca, 0x17d2, 0x179c, 0x17bc, 0x178e, 0x17b6, 0x17a0, 0x17ca, 0x17d2, 0x179c, 0x17bc, 0x1791,
0x17b8, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x17a1, 0x17b6, 0x1794, 0x17c9, 0x17b6, 0x17a0, 0x17d2,
0x1782, 0x17c4, 0x179f, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x17c6, 0x1794, 0x17c0, 0x17a0, 0x17d2,
0x1782, 0x17b6, 0x178a, 0x17b6, 0x179b, 0x17cb, 0x1781, 0x17b6, 0x178e, 0x17b6, 0x179b, 0x17cb,
0x17a0, 0x17bc, 0x178e, 0x17bc, 0x179b, 0x17bc, 0x179b, 0x17bc, 0x17a2, 0x17ca, 0x17b8, 0x1793,
0x178a, 0x17ba, 0x1794, 0x17ca, 0x17bc, 0x179a, 0x17b8, 0x1782, 0x17b7, 0x179a, 0x17b8, 0x1791,
0x17b8, 0x1798, 0x17c9, 0x17b6, 0x1791, 0x17b7, 0x1780, 0x17bc, 0x179f, 0x17d2, 0x179a, 0x17c2,
0x1780, 0x17d2, 0x179c, 0x17b6, 0x17a0, 0x17d2, 0x179f, 0x17b6, 0x179b, 0x17c0, 0x1793, 0x1798,
0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x179f, 0x17bc, 0x179a, 0x17c9, 0x17bc, 0x1798, 0x17c9, 0x17b6,
0x17a0, 0x17d2, 0x1782, 0x17b8, 0x179f, 0x17b6, 0x179f, 0x1798, 0x17b8, 0x178f, 0x179c, 0x17c9,
0x17c1, 0x178e, 0x17bc, 0x179a, 0x17bc, 0x1793, 0x17b8, 0x179c, 0x17c9, 0x17c1, 0x178e, 0x17d0,
0x179a, 0x17a0, 0x17d2, 0x179c, 0x17bb, 0x1780, 0x178e, 0x17bc, 0x1798, 0x17c0, 0x1794, 0x17c9,
0x17b6, 0x17a0, 0x17d2, 0x1782, 0x17c4, 0x20, 0x1794, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x1782,
0x17c4, 0x1794, 0x17c9, 0x17b6, 0x17a1, 0x17c5, 0x1797, 0x17b8, 0x178f, 0x1781, 0x17c3, 0x179a,
0x17c9, 0x17b7, 0x1793, 0x1795, 0x17bb, 0x1793, 0x1794, 0x17c9, 0x17b8, 0x1796, 0x17d0, 0x179a,
0x1798, 0x17c9, 0x17bc, 0x179a, 0x17c1, 0x179f, 0x1794, 0x17ca, 0x17b8, 0x179a, 0x17c9, 0x17b6,
0x179a, 0x17c9, 0x17bc, 0x178f, 0x17bb, 0x1784, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x179f, 0x17c3,
0x1794, 0x17c9, 0x17b6, 0x1793, 0x178f, 0x17b6, 0x17a0, 0x17b8, 0x1791, 0x17b8, 0x178f, 0x17b6,
0x179a, 0x17c9, 0x17b6, 0x179c, 0x17c9, 0x17b6, 0x178f, 0x17bb, 0x1784, 0x17a0, 0x17d2, 0x1782,
0x17b6, 0x178f, 0x17b6, 0x1796, 0x17bc, 0x179c, 0x17c9, 0x17b6, 0x1782, 0x17b8, 0x179c, 0x17c9,
0x17b6, 0x179b, 0x17b8, 0x179f, 0x906, 0x915, 0x930, 0x93e, 0x906, 0x926, 0x940, 0x938,
0x20, 0x905, 0x92c, 0x93e, 0x92c, 0x93e, 0x906, 0x932, 0x94d, 0x91c, 0x93f, 0x92f,
0x93e, 0x930, 0x94d, 0x938, 0x92c, 0x93e, 0x902, 0x917, 0x940, 0x92c, 0x93e, 0x928,
0x91c, 0x941, 0x932, 0x92c, 0x93f, 0x938, 0x93e, 0x909, 0x92c, 0x94d, 0x932, 0x93e,
0x928, 0x91f, 0x93e, 0x92f, 0x930, 0x92c, 0x94d, 0x930, 0x93e, 0x91d, 0x93e, 0x935,
0x93f, 0x932, 0x947, 0x92c, 0x941, 0x91c, 0x941, 0x92e, 0x92c, 0x941, 0x930, 0x93e,
0x915, 0x94b, 0x928, 0x93e, 0x915, 0x94d, 0x930, 0x93e, 0x926, 0x93e, 0x930, 0x20,
0x907, 0x20, 0x938, 0x932, 0x93e, 0x92e, 0x921, 0x94c, 0x906, 0x932, 0x93e, 0x905,
0x932, 0x20, 0x906, 0x92f, 0x941, 0x928, 0x92b, 0x94d, 0x930, 0x93f, 0x91f, 0x93e,
0x935, 0x928, 0x91c, 0x94b, 0x939, 0x93e, 0x928, 0x94d, 0x938, 0x92c, 0x930, 0x94d,
0x917, 0x916, 0x93e, 0x930, 0x94d, 0x91f, 0x94b, 0x92e, 0x915, 0x93f, 0x928, 0x936,
0x93e, 0x938, 0x93e, 0x92e, 0x93e, 0x938, 0x947, 0x930, 0x941, 0x92e, 0x94b, 0x917,
0x93e, 0x926, 0x93f, 0x936, 0x942, 0x928, 0x93e, 0x92f, 0x930, 0x94b, 0x92c, 0x940,
0x90f, 0x928, 0x91c, 0x93e, 0x92e, 0x947, 0x928, 0x93e, 0x928, 0x935, 0x93e, 0x915,
0x938, 0x941, 0x924, 0x913, 0x917, 0x93e, 0x926, 0x94c, 0x917, 0x94b, 0x91f, 0x941,
0x928, 0x93f, 0x938, 0x905, 0x921, 0x93e, 0x915, 0x910, 0x902, 0x915, 0x930, 0x91c,
0x905, 0x902, 0x917, 0x941, 0x932, 0x93e, 0x90f, 0x901, 0x91f, 0x93f, 0x917, 0x941,
0x906, 0x905, 0x930, 0x917, 0x941, 0x92f, 0x947, 0x928, 0x93e, 0x92c, 0x94d, 0x92f,
0x942, 0x928, 0x938, 0x20, 0x906, 0x92f, 0x930, 0x94d, 0x938, 0x915, 0x93e, 0x91f,
0x93e, 0x92e, 0x93e, 0x930, 0x915, 0x93e, 0x915, 0x94b, 0x930, 0x921, 0x94b, 0x92c,
0x93e, 0x91c, 0x941, 0x91c, 0x947, 0x92f, 0x932, 0x93e, 0x20, 0x930, 0x93f, 0x92f,
0x94b, 0x91c, 0x93e, 0x92e, 0x947, 0x928, 0x94d, 0x921, 0x94b, 0x91d, 0x93e, 0x930,
0x93f, 0x92f, 0x94b, 0x20, 0x917, 0x932, 0x947, 0x917, 0x949, 0x938, 0x938, 0x93e,
0x928, 0x20, 0x91c, 0x941, 0x906, 0x928, 0x91f, 0x941, 0x915, 0x941, 0x92e, 0x945,
0x928, 0x905, 0x930, 0x941, 0x92c, 0x93e, 0x905, 0x938, 0x902, 0x938, 0x93f, 0x92f,
0x928, 0x905, 0x924, 0x93f, 0x915, 0x94b, 0x915, 0x949, 0x928, 0x92c, 0x939, 0x93f,
0x92f, 0x93e, 0x20, 0x92c, 0x902, 0x926, 0x947, 0x930, 0x938, 0x92c, 0x947, 0x932,
0x93f, 0x91d, 0x92c, 0x94d, 0x932, 0x93e, 0x902, 0x915, 0x2d, 0x938, 0x93e, 0x92c,
0x94d, 0x932, 0x94b, 0x928, 0x92c, 0x93e, 0x913, 0x20, 0x935, 0x93f, 0x938, 0x94d,
0x91f, 0x93e, 0x92c, 0x94b, 0x92f, 0x938, 0x915, 0x947, 0x902, 0x92c, 0x94d, 0x930,
0x93f, 0x91c, 0x20, 0x92c, 0x947, 0x915, 0x945, 0x92e, 0x94d, 0x92a, 0x94b, 0x20,
0x917, 0x94d, 0x930, 0x93e, 0x902, 0x921, 0x915, 0x93e, 0x928, 0x915, 0x941, 0x928,
0x915, 0x930, 0x93e, 0x915, 0x938, 0x915, 0x947, 0x907, 0x928, 0x938, 0x93f, 0x909,
0x926, 0x93e, 0x926, 0x20, 0x91c, 0x941, 0x906, 0x930, 0x947, 0x91c, 0x915, 0x94b,
0x938, 0x94d, 0x924, 0x93e, 0x20, 0x930, 0x93f, 0x915, 0x93e, 0x915, 0x94d, 0x930,
0x947, 0x938, 0x94d, 0x91f, 0x94b, 0x928, 0x915, 0x941, 0x92f, 0x92f, 0x92c, 0x93e,
0x921, 0x947, 0x928, 0x94d, 0x92e, 0x93e, 0x930, 0x94d, 0x915, 0x936, 0x949, 0x928,
0x921, 0x94c, 0x938, 0x928, 0x921, 0x94c, 0x938, 0x928, 0x20, 0x915, 0x94d, 0x930,
0x93f, 0x915, 0x921, 0x947, 0x928, 0x94d, 0x935, 0x947, 0x930, 0x921, 0x947, 0x91f,
0x94d, 0x930, 0x949, 0x908, 0x91f, 0x921, 0x94b, 0x92e, 0x93f, 0x928, 0x940, 0x915,
0x93e, 0x90d, 0x921, 0x92e, 0x94b, 0x902, 0x91f, 0x928, 0x90f, 0x932, 0x20, 0x938,
0x93e, 0x932, 0x94d, 0x935, 0x93e, 0x921, 0x94b, 0x930, 0x92b, 0x94b, 0x930, 0x94d,
0x91f, 0x20, 0x928, 0x947, 0x932, 0x94d, 0x938, 0x928, 0x92b, 0x94b, 0x930, 0x94d,
0x91f, 0x93e, 0x932, 0x947, 0x91d, 0x93e, 0x917, 0x942, 0x938, 0x20, 0x92c, 0x947,
0x917, 0x94d, 0x930, 0x901, 0x921, 0x20, 0x924, 0x941, 0x930, 0x94d, 0x915, 0x917,
0x94d, 0x930, 0x947, 0x928, 0x945, 0x921, 0x93e, 0x917, 0x94d, 0x935, 0x93e, 0x92f,
0x947, 0x915, 0x93f, 0x932, 0x939, 0x947, 0x932, 0x93f, 0x92b, 0x945, 0x915, 0x94d,
0x938, 0x939, 0x930, 0x94d, 0x92e, 0x94b, 0x938, 0x93f, 0x932, 0x94b, 0x907, 0x902,
0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x92a, 0x94b, 0x932, 0x940, 0x938, 0x928,
0x949, 0x915, 0x94d, 0x938, 0x2c, 0x20, 0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e,
0x928, 0x93e, 0x92a, 0x93f, 0x91f, 0x930, 0x94d, 0x938, 0x92c, 0x930, 0x94d, 0x917,
0x2c, 0x20, 0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x935, 0x947,
0x935, 0x92f, 0x2c, 0x20, 0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e,
0x935, 0x93f, 0x902, 0x938, 0x947, 0x928, 0x938, 0x2c, 0x20, 0x907, 0x902, 0x921,
0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x935, 0x93f, 0x928, 0x93e, 0x92e, 0x945, 0x915,
0x2c, 0x20, 0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x907, 0x928,
0x941, 0x935, 0x93f, 0x915, 0x907, 0x915, 0x93e, 0x932, 0x941, 0x92f, 0x940, 0x91f,
0x91c, 0x942, 0x928, 0x94b, 0x932, 0x941, 0x908, 0x91c, 0x935, 0x93f, 0x932, 0x947,
0x92e, 0x94b, 0x902, 0x91f, 0x940, 0x938, 0x945, 0x932, 0x94b, 0x2c, 0x20, 0x915,
0x947, 0x902, 0x91f, 0x941, 0x915, 0x93e, 0x92f, 0x915, 0x94d, 0x930, 0x93e, 0x932,
0x947, 0x928, 0x921, 0x940, 0x915, 0x932, 0x93e, 0x20, 0x92a, 0x93e, 0x91d, 0x932,
0x93f, 0x92e, 0x93e, 0x932, 0x949, 0x938, 0x20, 0x90f, 0x902, 0x91c, 0x932, 0x940,
0x938, 0x932, 0x94b, 0x935, 0x930, 0x20, 0x92a, 0x94d, 0x930, 0x93f, 0x902, 0x938,
0x947, 0x938, 0x20, 0x915, 0x94d, 0x935, 0x93e, 0x930, 0x94d, 0x91f, 0x930, 0x92e,
0x93e, 0x938, 0x93f, 0x92f, 0x94b, 0x92e, 0x945, 0x928, 0x93e, 0x917, 0x94d, 0x935,
0x93e, 0x92e, 0x947, 0x930, 0x940, 0x917, 0x949, 0x91f, 0x92e, 0x93e, 0x91d, 0x93e,
0x91f, 0x94d, 0x932, 0x93e, 0x928, 0x92e, 0x947, 0x91f, 0x94d, 0x932, 0x93e, 0x915,
0x93e, 0x91f, 0x94d, 0x932, 0x93e, 0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915,
0x94b, 0x20, 0x936, 0x93e, 0x930, 0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932,
0x94b, 0x928, 0x92e, 0x94b, 0x902, 0x915, 0x94d, 0x91f, 0x949, 0x928, 0x92e, 0x94b,
0x902, 0x91f, 0x947, 0x930, 0x940, 0x92e, 0x94b, 0x902, 0x91f, 0x947, 0x935, 0x94d,
0x939, 0x93f, 0x921, 0x93f, 0x92f, 0x94b, 0x92e, 0x949, 0x928, 0x94d, 0x91f, 0x938,
0x947, 0x930, 0x93e, 0x91f, 0x928, 0x938, 0x93e, 0x90a, 0x928, 0x94d, 0x92f, 0x941,
0x20, 0x92f, 0x949, 0x930, 0x94d, 0x915, 0x928, 0x94b, 0x92e, 0x947, 0x92c, 0x94d,
0x92f, 0x942, 0x932, 0x94d, 0x932, 0x93e, 0x939, 0x2c, 0x20, 0x909, 0x924, 0x94d,
0x924, 0x930, 0x20, 0x921, 0x93e, 0x915, 0x94b, 0x91f, 0x93e, 0x92e, 0x927, 0x94d,
0x92f, 0x2c, 0x20, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x921, 0x93e, 0x915,
0x94b, 0x91f, 0x93e, 0x928, 0x94d, 0x92f, 0x941, 0x20, 0x938, 0x93e, 0x932, 0x947,
0x92e, 0x2c, 0x20, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x921, 0x93e, 0x915,
0x94b, 0x91f, 0x93e, 0x913, 0x91c, 0x93f, 0x928, 0x93e, 0x917, 0x93e, 0x92a, 0x930,
0x93e, 0x92e, 0x93e, 0x930, 0x93f, 0x92c, 0x94b, 0x92b, 0x93f, 0x928, 0x93f, 0x915,
0x94d, 0x938, 0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x2d, 0x914, 0x2d, 0x92a, 0x94d,
0x930, 0x93f, 0x928, 0x94d, 0x938, 0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x20, 0x911,
0x92b, 0x20, 0x938, 0x94d, 0x92a, 0x947, 0x928, 0x92a, 0x93f, 0x930, 0x94d, 0x91f,
0x94b, 0x20, 0x930, 0x93f, 0x915, 0x94b, 0x92a, 0x941, 0x902, 0x91f, 0x93e, 0x20,
0x905, 0x930, 0x947, 0x928, 0x93e, 0x938, 0x930, 0x901, 0x915, 0x93f, 0x928, 0x20,
0x907, 0x928, 0x932, 0x945, 0x91f, 0x930, 0x947, 0x938, 0x940, 0x92b, 0x930, 0x947,
0x91c, 0x940, 0x928, 0x93e, 0x930, 0x93f, 0x938, 0x94b, 0x932, 0x94d, 0x92f, 0x941,
0x91f, 0x938, 0x93e, 0x902, 0x924, 0x93e, 0x930, 0x947, 0x92e, 0x938, 0x901, 0x91f,
0x93f, 0x92f, 0x93e, 0x917, 0x94b, 0x938, 0x93e, 0x902, 0x924, 0x94b, 0x20, 0x921,
0x94b, 0x92e, 0x93f, 0x902, 0x917, 0x94b, 0x938, 0x93e, 0x913, 0x20, 0x92a, 0x93e,
0x935, 0x932, 0x94b, 0x907, 0x91f, 0x94b, 0x915, 0x949, 0x930, 0x91f, 0x949, 0x930,
0x94d, 0x92e, 0x940, 0x91f, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x91c, 0x949, 0x928,
0x94d, 0x938, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x932, 0x941, 0x938, 0x940, 0x92f,
0x93e, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x935, 0x93f, 0x928, 0x94d, 0x938, 0x947,
0x902, 0x91f, 0x938, 0x94d, 0x935, 0x93f, 0x92b, 0x94d, 0x91f, 0x20, 0x915, 0x930,
0x902, 0x91f, 0x924, 0x947, 0x917, 0x941, 0x938, 0x93f, 0x917, 0x932, 0x94d, 0x92a,
0x93e, 0x925, 0x941, 0x932, 0x947, 0x91f, 0x949, 0x930, 0x902, 0x91f, 0x94b, 0x91f,
0x94b, 0x930, 0x91f, 0x94b, 0x932, 0x93e, 0x935, 0x94d, 0x939, 0x901, 0x915, 0x941,
0x935, 0x94d, 0x939, 0x930, 0x935, 0x94d, 0x939, 0x93e, 0x92f, 0x91f, 0x939, 0x949,
0x930, 0x94d, 0x938, 0x92f, 0x93e, 0x915, 0x941, 0x91f, 0x93e, 0x91f, 0x921, 0x947,
0x935, 0x94d, 0x939, 0x940, 0x938, 0x926, 0x94d, 0x92f, 0x941, 0x92e, 0x93e, 0x901,
0x20, 0x926, 0x942, 0x930, 0x935, 0x940, 0x92f, 0x92e, 0x945, 0x915, 0x94d, 0x935,
0x947, 0x930, 0x940, 0x92e, 0x945, 0x915, 0x92e, 0x941, 0x930, 0x94d, 0x926, 0x94b,
0x92a, 0x93e, 0x932, 0x94d, 0x92e, 0x947, 0x930, 0x932, 0x94b, 0x902, 0x917, 0x92f,
0x947, 0x905, 0x930, 0x92c, 0x94d, 0x92f, 0x947, 0x928, 0x90d, 0x921, 0x928, 0x905,
0x928, 0x93e, 0x926, 0x93f, 0x930, 0x90d, 0x915, 0x94d, 0x91f, 0x94b, 0x905, 0x915,
0x94d, 0x91f, 0x94b, 0x92c, 0x947, 0x905, 0x936, 0x917, 0x93e, 0x92c, 0x93e, 0x91f,
0x905, 0x924, 0x94d, 0x92f, 0x93e, 0x930, 0x94b, 0x92c, 0x947, 0x939, 0x930, 0x947,
0x928, 0x92c, 0x901, 0x915, 0x949, 0x915, 0x92c, 0x930, 0x928, 0x94c, 0x932, 0x92c,
0x947, 0x930, 0x942, 0x924, 0x91a, 0x93f, 0x91f, 0x93e, 0x926, 0x92e, 0x938, 0x94d,
0x915, 0x938, 0x926, 0x93f, 0x932, 0x940, 0x926, 0x941, 0x92c, 0x92f, 0x92b, 0x92e,
0x93e, 0x917, 0x941, 0x938, 0x94d, 0x924, 0x93e, 0x917, 0x93e, 0x91d, 0x93e, 0x939,
0x947, 0x92c, 0x94d, 0x930, 0x949, 0x928, 0x939, 0x94b, 0x935, 0x94d, 0x939, 0x921,
0x907, 0x930, 0x94d, 0x915, 0x941, 0x91f, 0x938, 0x94d, 0x915, 0x91c, 0x93e, 0x92f,
0x93e, 0x92a, 0x941, 0x930, 0x93e, 0x91c, 0x947, 0x930, 0x941, 0x938, 0x93e, 0x932,
0x947, 0x92e, 0x915, 0x93e, 0x92e, 0x91a, 0x93e, 0x91f, 0x94d, 0x915, 0x93e, 0x916,
0x93e, 0x93c, 0x902, 0x921, 0x93f, 0x917, 0x93e, 0x915, 0x941, 0x906, 0x932, 0x93e,
0x932, 0x92e, 0x94d, 0x92a, 0x942, 0x930, 0x915, 0x941, 0x91a, 0x940, 0x902, 0x917,
0x915, 0x941, 0x935, 0x947, 0x924, 0x92e, 0x915, 0x93e, 0x935, 0x92e, 0x917, 0x93e,
0x926, 0x93e, 0x928, 0x92e, 0x915, 0x93e, 0x938, 0x94d, 0x938, 0x93e, 0x930, 0x92e,
0x928, 0x93f, 0x932, 0x93e, 0x928, 0x94b, 0x935, 0x94b, 0x915, 0x941, 0x91d, 0x928,
0x947, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x92b, 0x94d, 0x928, 0x94b, 0x92e, 0x20,
0x92a, 0x947, 0x928, 0x94d, 0x939, 0x92a, 0x94b, 0x923, 0x94d, 0x91f, 0x940, 0x905,
0x928, 0x93e, 0x915, 0x915, 0x924, 0x93e, 0x930, 0x915, 0x94b, 0x938, 0x94d, 0x91f,
0x93e, 0x928, 0x93e, 0x92f, 0x915, 0x94d, 0x92f, 0x94d, 0x91c, 0x94d, 0x92f, 0x932,
0x949, 0x930, 0x921, 0x93e, 0x938, 0x916, 0x932, 0x93f, 0x928, 0x936, 0x93e, 0x902,
0x918, 0x93e, 0x92f, 0x938, 0x93f, 0x902, 0x917, 0x93e, 0x92a, 0x942, 0x930, 0x938,
0x94d, 0x930, 0x947, 0x926, 0x928, 0x947, 0x915, 0x949, 0x932, 0x92f, 0x94d, 0x92e,
0x938, 0x94d, 0x915, 0x924, 0x948, 0x92a, 0x947, 0x908, 0x924, 0x93e, 0x936, 0x94d,
0x915, 0x902, 0x926, 0x924, 0x941, 0x92c, 0x932, 0x940, 0x938, 0x940, 0x925, 0x93f,
0x92e, 0x94d, 0x92b, 0x942, 0x924, 0x94b, 0x915, 0x94d, 0x92f, 0x94b, 0x91f, 0x949,
0x92e, 0x938, 0x94d, 0x915, 0x909, 0x932, 0x93e, 0x928, 0x92c, 0x924, 0x93e, 0x930,
0x909, 0x930, 0x941, 0x92e, 0x94d, 0x915, 0x940, 0x909, 0x938, 0x94d, 0x924, 0x20,
0x928, 0x947, 0x930, 0x93e, 0x935, 0x94d, 0x939, 0x93f, 0x90f, 0x928, 0x94d, 0x91f,
0x93f, 0x92f, 0x928, 0x92f, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x92f,
0x93e, 0x928, 0x917, 0x949, 0x928, 0x905, 0x91d, 0x94b, 0x930, 0x947, 0x938, 0x92c,
0x930, 0x94d, 0x92e, 0x941, 0x921, 0x93e, 0x915, 0x945, 0x928, 0x930, 0x940, 0x915,
0x947, 0x92a, 0x20, 0x935, 0x930, 0x94d, 0x926, 0x947, 0x92b, 0x948, 0x930, 0x94b,
0x92e, 0x926, 0x93f, 0x930, 0x93e, 0x930, 0x93f, 0x915, 0x94d, 0x91c, 0x947, 0x935,
0x93f, 0x915, 0x938, 0x947, 0x902, 0x2e, 0x20, 0x939, 0x947, 0x932, 0x947, 0x928,
0x93e, 0x938, 0x94d, 0x91f, 0x947, 0x928, 0x94d, 0x932, 0x947, 0x92c, 0x94d, 0x930,
0x93f, 0x938, 0x94d, 0x92c, 0x947, 0x928, 0x909, 0x915, 0x94d, 0x932, 0x93e, 0x932,
0x93f, 0x921, 0x947, 0x92e, 0x945, 0x928, 0x92e, 0x947, 0x932, 0x92c, 0x930, 0x94d,
0x928, 0x90d, 0x92e, 0x94d, 0x938, 0x94d, 0x91f, 0x930, 0x921, 0x945, 0x92e, 0x906,
0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x916, 0x928, 0x905, 0x925, 0x947, 0x928, 0x94d,
0x938, 0x92c, 0x94d, 0x930, 0x93e, 0x91f, 0x93f, 0x938, 0x94d, 0x932, 0x93e, 0x935,
0x94d, 0x939, 0x93e, 0x92c, 0x94d, 0x930, 0x941, 0x938, 0x947, 0x932, 0x94d, 0x938,
0x92c, 0x941, 0x916, 0x93e, 0x930, 0x947, 0x938, 0x94d, 0x91f, 0x92c, 0x941, 0x938,
0x93f, 0x902, 0x91c, 0x947, 0x928, 0x91a, 0x93f, 0x938, 0x940, 0x928, 0x93e, 0x90a,
0x915, 0x94b, 0x92a, 0x928, 0x939, 0x947, 0x917, 0x928, 0x921, 0x92c, 0x94d, 0x932,
0x93f, 0x928, 0x907, 0x938, 0x932, 0x947, 0x20, 0x911, 0x92b, 0x20, 0x92e, 0x945,
0x928, 0x907, 0x938, 0x94d, 0x924, 0x902, 0x92c, 0x942, 0x932, 0x915, 0x93e, 0x932,
0x93f, 0x928, 0x93f, 0x928, 0x917, 0x94d, 0x930, 0x93e, 0x921, 0x91c, 0x941, 0x92c,
0x94d, 0x932, 0x91c, 0x93e, 0x928, 0x93e, 0x932, 0x902, 0x921, 0x928, 0x932, 0x915,
0x94d, 0x938, 0x947, 0x92e, 0x92c, 0x930, 0x94d, 0x917, 0x92e, 0x93e, 0x926, 0x94d,
0x930, 0x940, 0x926, 0x92e, 0x945, 0x930, 0x940, 0x939, 0x93e, 0x92e, 0x94d, 0x928,
0x92e, 0x94b, 0x928, 0x945, 0x915, 0x94b, 0x92a, 0x945, 0x930, 0x93f, 0x938, 0x92a,
0x949, 0x921, 0x917, 0x94b, 0x930, 0x93f, 0x915, 0x93e, 0x938, 0x945, 0x928, 0x20,
0x92e, 0x93e, 0x930, 0x940, 0x928, 0x94b, 0x938, 0x93e, 0x930, 0x93e, 0x91c, 0x93f,
0x935, 0x94b, 0x938, 0x93e, 0x930, 0x93e, 0x91f, 0x94b, 0x935, 0x938, 0x93f, 0x92e,
0x92b, 0x930, 0x94b, 0x92a, 0x94b, 0x932, 0x938, 0x94b, 0x92b, 0x93f, 0x92f, 0x93e,
0x91f, 0x945, 0x932, 0x93f, 0x928, 0x924, 0x93f, 0x930, 0x93e, 0x928, 0x93e, 0x935,
0x921, 0x941, 0x91c, 0x935, 0x945, 0x91f, 0x93f, 0x915, 0x928, 0x935, 0x93f, 0x92f,
0x947, 0x928, 0x93e, 0x935, 0x94d, 0x939, 0x94b, 0x932, 0x94d, 0x917, 0x94b, 0x917,
0x94d, 0x930, 0x93e, 0x921, 0x935, 0x93e, 0x930, 0x938, 0x949, 0x91d, 0x93e, 0x917,
0x94d, 0x930, 0x947, 0x92c, 0x91d, 0x941, 0x930, 0x93f, 0x91a, 0x905, 0x902, 0x91f,
0x93e, 0x928, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x935, 0x94b, 0x91a, 0x93e, 0x917,
0x94b, 0x938, 0x915, 0x947, 0x930, 0x917, 0x941, 0x932, 0x947, 0x928, 0x92e, 0x949,
0x930, 0x93f, 0x936, 0x938, 0x92e, 0x947, 0x92f, 0x94b, 0x91f, 0x930, 0x93f, 0x92f,
0x941, 0x928, 0x93f, 0x92f, 0x928, 0x905, 0x92a, 0x93f, 0x92f, 0x93e, 0x911, 0x915,
0x932, 0x901, 0x921, 0x92c, 0x94b, 0x917, 0x947, 0x928, 0x935, 0x93f, 0x932, 0x947,
0x91a, 0x945, 0x925, 0x92e, 0x91a, 0x941, 0x915, 0x907, 0x92b, 0x947, 0x91f, 0x92b,
0x93e, 0x915, 0x93e, 0x913, 0x92b, 0x94b, 0x92b, 0x941, 0x928, 0x93e, 0x92b, 0x941,
0x91f, 0x940, 0x917, 0x93e, 0x932, 0x93e, 0x92a, 0x93e, 0x917, 0x94b, 0x938, 0x917,
0x93e, 0x92e, 0x94d, 0x92c, 0x93f, 0x92f, 0x930, 0x917, 0x94d, 0x935, 0x93e, 0x926,
0x93e, 0x932, 0x915, 0x945, 0x928, 0x93e, 0x932, 0x907, 0x902, 0x921, 0x930, 0x92c,
0x930, 0x940, 0x915, 0x93f, 0x930, 0x93f, 0x924, 0x93f, 0x92e, 0x93e, 0x924, 0x940,
0x915, 0x94b, 0x938, 0x930, 0x947, 0x915, 0x94d, 0x935, 0x93e, 0x91c, 0x932, 0x947,
0x928, 0x92e, 0x93e, 0x930, 0x94d, 0x915, 0x93f, 0x938, 0x938, 0x928, 0x93e, 0x935,
0x930, 0x942, 0x928, 0x949, 0x930, 0x92b, 0x94b, 0x915, 0x928, 0x942, 0x92e, 0x947,
0x906, 0x92a, 0x93f, 0x91f, 0x915, 0x945, 0x930, 0x928, 0x92a, 0x94b, 0x939, 0x94d,
0x928, 0x92a, 0x947, 0x908, 0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x20, 0x92e, 0x949,
0x930, 0x947, 0x938, 0x94d, 0x92c, 0x940, 0x924, 0x93e, 0x930, 0x935, 0x93e, 0x935,
0x948, 0x915, 0xc544, 0xbe44, 0xc7a5, 0xc544, 0xd06c, 0xb77c, 0xc544, 0xb514, 0xc2a4, 0xc544,
0xbc14, 0xbc14, 0xc54c, 0xc81c, 0xc544, 0xc2a4, 0xba54, 0xb77c, 0xbc14, 0xb9c8, 0xcf54, 0xbc29,
0xae30, 0xbc18, 0xc904, 0xbe44, 0xc0ac, 0xc6b0, 0xbe14, 0xb79c, 0xd0c0, 0xc774, 0xc5b4, 0xbe0c,
0xb77c, 0xc790, 0xbe4c, 0xbd80, 0xc90c, 0xbd80, 0xb77c, 0xce74, 0xc774, 0xb85c, 0xce74, 0xc0ac,
0xbe14, 0xb791, 0xce74, 0xc138, 0xc6b0, 0xd0c0, 0xcf54, 0xb098, 0xd06c, 0xb9ac, 0xb2e4, 0xce74,
0xb974, 0xb2e4, 0xb974, 0xc5d0, 0xc2a4, 0xc0b4, 0xb78c, 0xc9c0, 0xbd80, 0xd2f0, 0xb450, 0xc54c,
0xb77c, 0xc5d8, 0xc544, 0xc774, 0xc6b4, 0xd504, 0xb9ac, 0xd0c0, 0xc6b4, 0xac00, 0xbcf4, 0xb85c,
0xb124, 0xd558, 0xb77c, 0xb808, 0xc694, 0xd558, 0xb124, 0xc2a4, 0xbc84, 0xadf8, 0xc8fc, 0xbc14,
0xce84, 0xd314, 0xb77c, 0xce74, 0xb974, 0xd23c, 0xd0a4, 0xac08, 0xb9ac, 0xd0a8, 0xc0e4, 0xc0ac,
0xb77c, 0xace0, 0xc2a4, 0xb9ac, 0xbe0c, 0xb974, 0xbe4c, 0xb85c, 0xba54, 0xb8e8, 0xc548, 0xb2e4,
0xb8e8, 0xbd90, 0xbc14, 0xc2dc, 0xb8e8, 0xc0ac, 0xce74, 0xb9d0, 0xb77c, 0xbcf4, 0xb9c8, 0xd478,
0xd1a0, 0xb9c8, 0xc138, 0xb8e8, 0xc74c, 0xbc14, 0xbc14, 0xb124, 0xbaa8, 0xac00, 0xb514, 0xc288,
0xbaac, 0xb85c, 0xbe44, 0xc544, 0xb098, 0xc774, 0xb85c, 0xbe44, 0xc5d4, 0xc790, 0xba54, 0xb098,
0xb2c8, 0xc544, 0xba54, 0xb204, 0xc545, 0xc1fc, 0xd2b8, 0xc640, 0xac00, 0xb450, 0xad6c, 0xd3ec,
0xb974, 0xd1a0, 0xb178, 0xbcf4, 0xc0c1, 0xd22c, 0xba54, 0xd2b8, 0xb9ac, 0xd3f4, 0xb9ac, 0xd280,
0xb2c8, 0xc2a4, 0xbe48, 0xd2b8, 0xd6c4, 0xd06c, 0xc5d0, 0xc774, 0xb2e5, 0xc575, 0xcee4, 0xb9ac,
0xc9c0, 0xc559, 0xadc8, 0xb77c, 0xc548, 0xd2f0, 0xacfc, 0xc544, 0xb77c, 0xacfc, 0xc774, 0xb098,
0xbd80, 0xc5d0, 0xb178, 0xc2a4, 0x20, 0xc544, 0xc774, 0xb808, 0xc2a4, 0xce74, 0xd0c0, 0xb9c8,
0xb974, 0xce74, 0xcf54, 0xb974, 0xb3c4, 0xbc14, 0xd6c4, 0xd6c4, 0xc774, 0xb77c, 0x20, 0xb9ac,
0xc624, 0xd558, 0xba58, 0xb3c4, 0xc0ac, 0xb9ac, 0xc624, 0x20, 0xac00, 0xc608, 0xace0, 0xc2a4,
0xc0b4, 0xd0c0, 0xc0b0, 0xd6c4, 0xc548, 0xc0b0, 0xb8e8, 0xc774, 0xc2a4, 0xd22c, 0xcfe0, 0xb9cc,
0xc6b0, 0xc218, 0xc544, 0xc774, 0xc544, 0xc544, 0xb8e8, 0xbc14, 0xc544, 0xc21c, 0xc2dc, 0xc628,
0xcf54, 0xb784, 0xd558, 0xbc84, 0xbc14, 0xd788, 0xc544, 0xbc14, 0xc774, 0xc544, 0x20, 0xbc18,
0xb370, 0xb77c, 0xc2a4, 0xbc14, 0xbca0, 0xc774, 0xb3c4, 0xc2a4, 0xbca8, 0xb818, 0xbca8, 0xb9ac,
0xc988, 0xbe14, 0xb791, 0x20, 0xc0ac, 0xbe14, 0xb871, 0xbcf4, 0xc544, 0x20, 0xbe44, 0xc2a4,
0xd0c0, 0xbcf4, 0xace0, 0xd0c0, 0xbcf4, 0xc774, 0xc2dc, 0xcf00, 0xc784, 0xbe0c, 0xb9ac, 0xc9c0,
0x20, 0xbca0, 0xc774, 0xce84, 0xd3ec, 0x20, 0xadf8, 0xb780, 0xb370, 0xce78, 0xcfe4, 0xce74,
0xb77c, 0xce74, 0xc2a4, 0xce74, 0xc60c, 0xcf00, 0xc774, 0xb9e8, 0xc2dc, 0xce74, 0xace0, 0xce58,
0xc640, 0xc640, 0xc2dc, 0xc6b0, 0xb2e4, 0xb4dc, 0xd6c4, 0xc544, 0xb808, 0xc2a4, 0xcf54, 0xc2a4,
0xd0c0, 0xb9ac, 0xce74, 0xd06c, 0xb808, 0xc2a4, 0xd1a4, 0xcfe0, 0xc774, 0xc544, 0xbc14, 0xd034,
0xb77c, 0xc18c, 0xb374, 0xb9c8, 0xd06c, 0xc0e4, 0xbe10, 0xb3c4, 0xc2a8, 0xb3c4, 0xc2a8, 0xd06c,
0xb9ad, 0xb374, 0xbc84, 0xb514, 0xd2b8, 0xb85c, 0xc774, 0xd2b8, 0xb3c4, 0xbbf8, 0xb2c8, 0xce74,
0xc5d0, 0xb4dc, 0xba3c, 0xd134, 0xc544, 0xc774, 0xb8e8, 0xb124, 0xd398, 0xc5d8, 0xc0b4, 0xbc14,
0xb3c4, 0xb974, 0xd3ec, 0xd2b8, 0x20, 0xb12c, 0xc2a8, 0xd3ec, 0xb974, 0xd0c8, 0xb808, 0xc790,
0xae00, 0xb77c, 0xc2a4, 0xbca0, 0xc774, 0xad6c, 0xc988, 0xbca0, 0xc774, 0xadf8, 0xb79c, 0xb4dc,
0x20, 0xd130, 0xd06c, 0xadf8, 0xb808, 0xb098, 0xb2e4, 0xacfc, 0xb4e4, 0xb8e8, 0xd504, 0xacfc,
0xd14c, 0xb9d0, 0xb77c, 0xacfc, 0xc57c, 0xd0ac, 0xac00, 0xc774, 0xc544, 0xb098, 0xd57c, 0xb9ac,
0xd329, 0xc2a4, 0xd558, 0xbc14, 0xb098, 0xc5d0, 0xb974, 0xbaa8, 0xc2dc, 0xc694, 0xc778, 0xb514,
0xc560, 0xb098, 0xd3f4, 0xb9ac, 0xc2a4, 0xc778, 0xb514, 0xc560, 0xb098, 0xc8fc, 0x2c, 0x20,
0xb179, 0xc2a4, 0xc778, 0xb514, 0xc560, 0xb098, 0xc8fc, 0x2c, 0x20, 0xba38, 0xb81d, 0xace0,
0xc778, 0xb514, 0xc560, 0xb098, 0xc8fc, 0x2c, 0x20, 0xd53c, 0xce20, 0xbc84, 0xadf8, 0xc778,
0xb514, 0xc560, 0xb098, 0xc8fc, 0x2c, 0x20, 0xd154, 0xc2dc, 0xd2f0, 0xc778, 0xb514, 0xc560,
0xb098, 0xc8fc, 0x2c, 0x20, 0xbe44, 0xbe44, 0xc778, 0xb514, 0xc560, 0xb098, 0xc8fc, 0x2c,
0x20, 0xbe48, 0xc13c, 0xc2a4, 0xc778, 0xb514, 0xc560, 0xb098, 0xc8fc, 0x2c, 0x20, 0xc704,
0xb108, 0xb9e5, 0xc774, 0xb204, 0xbe45, 0xc774, 0xce7c, 0xb8e8, 0xc774, 0xd2b8, 0xc790, 0xba54,
0xc774, 0xce74, 0xc8fc, 0xb178, 0xb8e8, 0xc774, 0xbe4c, 0xcf04, 0xd130, 0xd0a4, 0xc8fc, 0x2c,
0x20, 0xbaac, 0xd2f0, 0xccbc, 0xb85c, 0xd06c, 0xb77c, 0xb80c, 0xb514, 0xc988, 0xd06c, 0xb77c,
0xd30c, 0xc2a4, 0xb9ac, 0xb9c8, 0xb85c, 0xc2a4, 0xc564, 0xc824, 0xb808, 0xc2a4, 0xb85c, 0xc6cc,
0x20, 0xd504, 0xb9b0, 0xc2a4, 0x20, 0xcffc, 0xd130, 0xb9c8, 0xc138, 0xc774, 0xc624, 0xb9c8,
0xb098, 0xacfc, 0xb9c8, 0xb098, 0xc6b0, 0xc2a4, 0xb9c8, 0xb9ac, 0xacf3, 0xb9c8, 0xd2f0, 0xb2c8,
0xd06c, 0xb9c8, 0xd0c0, 0xbaa8, 0xb85c, 0xc2a4, 0xb9c8, 0xc0ac, 0xd2c0, 0xb780, 0xba54, 0xb178,
0xbbf8, 0xb2c8, 0xba54, 0xb9ac, 0xb2e4, 0xba54, 0xd2c0, 0xb77c, 0xce74, 0xd2c0, 0xb77c, 0xba55,
0xc2dc, 0xcf54, 0x20, 0xc2dc, 0xd2f0, 0xbbf8, 0xd074, 0xb871, 0xbabd, 0xd134, 0xbaac, 0xd14c,
0xb808, 0xc774, 0xbaac, 0xd14c, 0xbe44, 0xb370, 0xc624, 0xbaac, 0xc138, 0xb77c, 0xd2b8, 0xb098,
0xc18c, 0xb274, 0xc695, 0xb188, 0xb178, 0xb871, 0xc57c, 0xb178, 0xc2a4, 0xb2e4, 0xcf54, 0xd0c0,
0xc8fc, 0x2c, 0x20, 0xbca0, 0xb77c, 0xc911, 0xbd80, 0x2c, 0x20, 0xb178, 0xc2a4, 0xb2e4,
0xcf54, 0xd0c0, 0xb178, 0xc2a4, 0xb2e4, 0xcf54, 0xd0c0, 0xc8fc, 0x2c, 0x20, 0xb274, 0xc0b4,
0xb818, 0xace0, 0xb4dc, 0xd638, 0xd504, 0xc624, 0xd788, 0xb098, 0xac00, 0xd30c, 0xb098, 0xb9c8,
0xd30c, 0xb77c, 0xb9c8, 0xb9ac, 0xbcf4, 0xd53c, 0xb2c9, 0xc2a4, 0xd3ec, 0xb974, 0xd1a0, 0xd504,
0xb7ad, 0xc2a4, 0xd3ec, 0xd2b8, 0xc624, 0xbe0c, 0xc2a4, 0xd398, 0xc778, 0xd3ec, 0xb974, 0xd1a0,
0xbca8, 0xb8cc, 0xd478, 0xc5d0, 0xb974, 0xd1a0, 0xb9ac, 0xcf54, 0xd47c, 0xd0c0, 0xc544, 0xb808,
0xb098, 0xc2a4, 0xb7ad, 0xd0b9, 0x20, 0xc778, 0xb81b, 0xb808, 0xc2dc, 0xd398, 0xb9ac, 0xc790,
0xc774, 0xb098, 0xb9ac, 0xc878, 0xb8e8, 0xd2b8, 0xd788, 0xc6b0, 0x20, 0xbe0c, 0xb791, 0xcfe0,
0xc0b0, 0xd0c0, 0xb818, 0xc0b0, 0xd2f0, 0xc544, 0xace0, 0xc0b0, 0xd1a0, 0xb3c4, 0xbc0d, 0xace0,
0xc0c1, 0xd30c, 0xc6b8, 0xb8e8, 0xc2a4, 0xcf54, 0xb808, 0xc2a4, 0xbc14, 0xc774, 0xc120, 0xb4dc,
0xc2ef, 0xce74, 0xc0dd, 0xbc14, 0xb974, 0xd154, 0xb808, 0xbbf8, 0xc138, 0xc778, 0xd2b8, 0xc874,
0xc2a4, 0xc138, 0xc778, 0xd2b8, 0xd0a4, 0xce20, 0xc138, 0xc778, 0xd2b8, 0xb8e8, 0xc2dc, 0xc544,
0xc138, 0xc778, 0xd2b8, 0xd1a0, 0xb9c8, 0xc2a4, 0xc138, 0xc778, 0xd2b8, 0xbe48, 0xc13c, 0xd2b8,
0xc2a4, 0xc704, 0xd504, 0xd2b8, 0xcee4, 0xb7f0, 0xd2b8, 0xd14c, 0xad6c, 0xc2dc, 0xac08, 0xd30c,
0xd234, 0xb808, 0xd2f0, 0xd6c4, 0xc544, 0xb098, 0xd1a0, 0xb860, 0xd1a0, 0xd1a0, 0xb974, 0xd1a8,
0xb77c, 0xbca4, 0xcfe0, 0xbc84, 0xd654, 0xc774, 0xd2b8, 0xd638, 0xc2a4, 0xc704, 0xb2c8, 0xd399,
0xc57c, 0xcfe0, 0xd0c0, 0xd2b8, 0xcf00, 0xc774, 0xc2dc, 0xb370, 0xc774, 0xbe44, 0xc2a4, 0xb4a4,
0xbabd, 0x20, 0xb4a4, 0xb974, 0xbe4c, 0xb9e5, 0xcffc, 0xb9ac, 0xbaa8, 0xc2a8, 0xb9e5, 0xba38,
0xb3c4, 0xd30c, 0xba38, 0xb85c, 0xb370, 0xb77c, 0xc1fc, 0xc640, 0xd2b8, 0xb864, 0xbcf4, 0xc2a4,
0xd1a0, 0xd06c, 0xb871, 0xc774, 0xc5b4, 0xbe44, 0xc5d4, 0xc544, 0xb374, 0xc54c, 0xb9c8, 0xd2f0,
0xc554, 0xb9cc, 0xc544, 0xb098, 0xb514, 0xb9ac, 0xc544, 0xd06c, 0xd0c0, 0xc6b0, 0xc545, 0xd1a0,
0xbe0c, 0xc544, 0xc288, 0xd558, 0xbc14, 0xd2b8, 0xc544, 0xd2f0, 0xb77c, 0xc6b0, 0xbc14, 0xadf8,
0xb2e4, 0xb4dc, 0xbc14, 0xb808, 0xc778, 0xbc14, 0xcfe0, 0xbc29, 0xcf55, 0xbc14, 0xb974, 0xb098,
0xc6b8, 0xbca0, 0xc774, 0xb8e8, 0xd2b8, 0xbe44, 0xc288, 0xcf00, 0xd06c, 0xbe0c, 0xb8e8, 0xb098,
0xc774, 0xce58, 0xd0c0, 0xcf5c, 0xb86c, 0xbcf4, 0xb2e4, 0xb9c8, 0xc2a4, 0xcfe0, 0xc2a4, 0xb51c,
0xb9ac, 0xb450, 0xbc14, 0xc774, 0xb450, 0xc0e8, 0xbca0, 0xd30c, 0xb9c8, 0xad6c, 0xc2a4, 0xd0c0,
0xac00, 0xc790, 0xd5e4, 0xbe0c, 0xb860, 0xc0ac, 0xc774, 0xacf5, 0xd64d, 0xcf69, 0xd638, 0xbe0c,
0xb4dc, 0xc774, 0xb974, 0xcfe0, 0xce20, 0xd06c, 0xc790, 0xce74, 0xb974, 0xd0c0, 0xc790, 0xc57c,
0xd478, 0xb77c, 0xc608, 0xb8e8, 0xc0b4, 0xb818, 0xce74, 0xbd88, 0xce84, 0xcc28, 0xce74, 0xce74,
0xb77c, 0xce58, 0xce74, 0xd2b8, 0xb9cc, 0xb450, 0xd55c, 0xb514, 0xac00, 0xcf5c, 0xce74, 0xd0c0,
0xd06c, 0xb77c, 0xc2a4, 0xb178, 0xc57c, 0xb974, 0xc2a4, 0xd06c, 0xcfe0, 0xc54c, 0xb77c, 0xb8f8,
0xd478, 0xb974, 0xcfe0, 0xce6d, 0xcfe0, 0xc6e8, 0xc774, 0xd2b8, 0xb9c8, 0xce74, 0xc624, 0xb9c8,
0xac00, 0xb2e8, 0xb9c8, 0xce74, 0xc0ac, 0xb974, 0xb9c8, 0xb2d0, 0xb77c, 0xbb34, 0xc2a4, 0xce74,
0xd2b8, 0xb2c8, 0xcf54, 0xc2dc, 0xc544, 0xb178, 0xbcf4, 0xcfe0, 0xc988, 0xb124, 0xce20, 0xd06c,
0xb178, 0xbcf4, 0xc2dc, 0xbe44, 0xb974, 0xc2a4, 0xd06c, 0xc634, 0xc2a4, 0xd06c, 0xc624, 0xb784,
0xd504, 0xb188, 0xd39c, 0xd3f0, 0xd2f0, 0xc544, 0xb098, 0xd06c, 0xd3c9, 0xc591, 0xce74, 0xd0c0,
0xb974, 0xcf54, 0xc2a4, 0xd0c0, 0xb098, 0xc774, 0xd0a4, 0xc9c8, 0xb85c, 0xb974, 0xb2e4, 0xb9ac,
0xc57c, 0xb4dc, 0xc0ac, 0xd560, 0xb9b0, 0xc0ac, 0xb9c8, 0xb974, 0xce78, 0xd2b8, 0xc11c, 0xc6b8,
0xc0c1, 0xd558, 0xc774, 0xc2f1, 0xac00, 0xd3ec, 0xb974, 0xc2a4, 0xb808, 0xb4dc, 0xb124, 0xcf5c,
0xb9bc, 0xc2a4, 0xd06c, 0xd0c0, 0xc774, 0xbca0, 0xc774, 0xd0c0, 0xc288, 0xcf04, 0xd2b8, 0xd2b8,
0xbe4c, 0xb9ac, 0xc2dc, 0xd14c, 0xd5e4, 0xb780, 0xd300, 0xbd80, 0xb3c4, 0xcfc4, 0xd1b0, 0xc2a4,
0xd06c, 0xc6b8, 0xb780, 0xbc14, 0xd1a0, 0xb974, 0xc6b0, 0xb8e8, 0xbb34, 0xce58, 0xc6b0, 0xc2a4,
0xd2f0, 0xb124, 0xb77c, 0xbe44, 0xc5d4, 0xd2f0, 0xc548, 0xbe14, 0xb77c, 0xb514, 0xbcf4, 0xc2a4,
0xd1a0, 0xd06c, 0xc57c, 0xcfe0, 0xce20, 0xd06c, 0xb791, 0xad70, 0xc608, 0xce74, 0xd14c, 0xb9b0,
0xbd80, 0xb974, 0xd06c, 0xc608, 0xb808, 0xbc18, 0xc544, 0xc870, 0xb808, 0xc2a4, 0xbc84, 0xbba4,
0xb2e4, 0xce74, 0xb098, 0xb9ac, 0xc544, 0x20, 0xc81c, 0xb3c4, 0xce74, 0xbcf4, 0x20, 0xbca0,
0xb974, 0xb370, 0xd398, 0xb85c, 0x20, 0xc81c, 0xb3c4, 0xb9c8, 0xb370, 0xc774, 0xb77c, 0xb808,
0xc774, 0xceac, 0xbe44, 0xd06c, 0xc0ac, 0xc6b0, 0xc2a4, 0xc870, 0xc9c0, 0xc544, 0xc138, 0xc778,
0xd2b8, 0x20, 0xd5ec, 0xb808, 0xb098, 0xc2a4, 0xd0e0, 0xb9ac, 0xc560, 0xb4e4, 0xb808, 0xc774,
0xb4dc, 0xbe0c, 0xb9ac, 0xc2a4, 0xbca0, 0xc778, 0xbe0c, 0xb85c, 0xcee8, 0xd790, 0xb2e4, 0xc708,
0xc720, 0xd074, 0xb77c, 0xd638, 0xbc14, 0xd2b8, 0xb9b0, 0xb370, 0xb9cc, 0xb85c, 0xb4dc, 0x20,
0xd558, 0xc6b0, 0xba5c, 0xbc84, 0xb978, 0xd37c, 0xc2a4, 0xc2dc, 0xb4dc, 0xb2c8, 0xc554, 0xc2a4,
0xd14c, 0xb974, 0xb2f4, 0xc548, 0xb3c4, 0xb77c, 0xc544, 0xc2a4, 0xd2b8, 0xb77c, 0xd55c, 0xc544,
0xd14c, 0xb124, 0xbca0, 0xc624, 0xadf8, 0xb77c, 0xb4dc, 0xbca0, 0xb97c, 0xb9b0, 0xbe0c, 0xb77c,
0xd2f0, 0xc2ac, 0xb77c, 0xbc14, 0xbe0c, 0xb93c, 0xc140, 0xbd80, 0xcfe0, 0xb808, 0xc288, 0xd2f0,
0xbd80, 0xb2e4, 0xd398, 0xc2a4, 0xd2b8, 0xbdd4, 0xc9c0, 0xac90, 0xd0a4, 0xc2dc, 0xb098, 0xc6b0,
0xcf54, 0xd39c, 0xd558, 0xac90, 0xb354, 0xbe14, 0xb9b0, 0xc9c0, 0xbe0c, 0xb864, 0xd130, 0xac74,
0xc9c0, 0xd5ec, 0xc2f1, 0xd0a4, 0xb9e8, 0xc12c, 0xc774, 0xc2a4, 0xd0c4, 0xbd88, 0xc800, 0xc9c0,
0xce7c, 0xb9ac, 0xb2cc, 0xadf8, 0xb77c, 0xb4dc, 0xd0a4, 0xb85c, 0xd504, 0xd0a4, 0xc608, 0xd504,
0xb9ac, 0xc2a4, 0xbcf8, 0xb958, 0xbe14, 0xb7b4, 0xb098, 0xb7f0, 0xb358, 0xb8e9, 0xc148, 0xbd80,
0xb974, 0xd06c, 0xb9c8, 0xb4dc, 0xb9ac, 0xb4dc, 0xbab0, 0xd0c0, 0xb9c8, 0xb9ac, 0xc5d0, 0xd568,
0xbbfc, 0xc2a4, 0xd06c, 0xbaa8, 0xb098, 0xcf54, 0xbaa8, 0xc2a4, 0xd06c, 0xbc14, 0xc624, 0xc2ac,
0xb85c, 0xd30c, 0xb9ac, 0xd3ec, 0xb4dc, 0xace0, 0xb9ac, 0xcc28, 0xd504, 0xb77c, 0xd558, 0xb9ac,
0xac00, 0xb85c, 0xb9c8, 0xc0ac, 0xb9c8, 0xb77c, 0xc0b0, 0xb9c8, 0xb9ac, 0xb178, 0xc0ac, 0xb77c,
0xc608, 0xbcf4, 0xc0ac, 0xb77c, 0xd1a0, 0xd504, 0xc2ec, 0xd398, 0xb85c, 0xd3f4, 0xc2a4, 0xcf54,
0xd398, 0xc18c, 0xd53c, 0xc544, 0xc2a4, 0xd1a1, 0xd640, 0xb984, 0xd0c8, 0xb9b0, 0xd2f0, 0xb77c,
0xb098, 0xc6b8, 0xb9ac, 0xc57c, 0xb178, 0xd504, 0xc2a4, 0xd06c, 0xd30c, 0xb450, 0xce20, 0xbc14,
0xd2f0, 0xce78, 0xbe44, 0xc5d4, 0xb098, 0xbe4c, 0xb2c8, 0xc6b0, 0xc2a4, 0xbcfc, 0xace0, 0xadf8,
0xb77c, 0xd2b8, 0xbc14, 0xb974, 0xc0e4, 0xbc14, 0xc790, 0xadf8, 0xb808, 0xbe0c, 0xcde8, 0xb9ac,
0xd788, 0xc548, 0xd0c0, 0xb098, 0xb098, 0xb9ac, 0xbcf4, 0xcc28, 0xace0, 0xc2a4, 0xd06c, 0xb9ac,
0xc2a4, 0xb9c8, 0xc2a4, 0xcf54, 0xcf54, 0xc2a4, 0xcf54, 0xbaa8, 0xb85c, 0xcf00, 0xb974, 0xcf08,
0xb80c, 0xb9c8, 0xd5e4, 0xbab0, 0xb514, 0xbe0c, 0xbaa8, 0xb9ac, 0xc154, 0xc2a4, 0xba54, 0xc694,
0xd2b8, 0xb808, 0xc704, 0xb2c8, 0xc639, 0xc544, 0xd53c, 0xc544, 0xc624, 0xd074, 0xb79c, 0xb4dc,
0xbd80, 0xac31, 0xbe4c, 0xcc44, 0xd140, 0xd2b8, 0xb8e8, 0xd06c, 0xc774, 0xc2a4, 0xd130, 0x20,
0xc12c, 0xc5d0, 0xd30c, 0xd14c, 0xd30c, 0xce74, 0xc624, 0xd478, 0xd53c, 0xc9c0, 0xd478, 0xb098,
0xd478, 0xd2f0, 0xac08, 0xb77c, 0xd30c, 0xace0, 0xc2a4, 0xac10, 0xbe44, 0xc5b4, 0xacfc, 0xb2ec,
0xce74, 0xb0a0, 0xad0c, 0xd638, 0xb180, 0xb8f0, 0xb8e8, 0xc5d4, 0xb354, 0xbca0, 0xb9ac, 0xd0a4,
0xb9ac, 0xd2f0, 0xb9c8, 0xd2f0, 0xcf54, 0xc2a4, 0xb808, 0xcf70, 0xc798, 0xb80c, 0xb9c8, 0xc8fc,
0xb85c, 0xb9c8, 0xd018, 0xc0ac, 0xc2a4, 0xbbf8, 0xb4dc, 0xc6e8, 0xc774, 0xb098, 0xc6b0, 0xb8e8,
0xb2c8, 0xc6b0, 0xc5d0, 0xb178, 0xd37d, 0xb204, 0xba54, 0xc544, 0xd30c, 0xace0, 0xd30c, 0xace0,
0xd314, 0xb77c, 0xc6b0, 0xd54f, 0xcf00, 0xc5b8, 0xd3ec, 0xb098, 0xd398, 0xd3ec, 0xd2b8, 0xbaa8,
0xb974, 0xc988, 0xbe44, 0xb77c, 0xb85c, 0xd1b5, 0xac00, 0xc0ac, 0xc774, 0xd310, 0xd0c0, 0xd788,
0xd2f0, 0xd0c0, 0xb77c, 0xc640, 0xd1b5, 0xac00, 0xd0c0, 0xd478, 0xc6e8, 0xc774, 0xd06c, 0xc6d4,
0xb9ac, 0xc2a4, 0x43, 0x65, 0x7a, 0x61, 0x79, 0xee, 0x72, 0x51, 0x61, 0x68,
0xee, 0x72, 0x65, 0x53, 0x65, 0x70, 0x74, 0x65, 0x4b, 0x6f, 0x6e, 0x61,
0x6b, 0x72, 0xee, 0x44, 0x61, 0x72, 0x75, 0x73, 0x65, 0x6c, 0x61, 0x6d,
0x43, 0x69, 0x62, 0xfb, 0x74, 0xee, 0x58, 0x61, 0x72, 0x74, 0xfb, 0x6d,
0x4b, 0xee, 0x67, 0x61, 0x6c, 0xee, 0x4b, 0xee, 0x6e, 0x15f, 0x61, 0x73,
0x61, 0x4c, 0xee, 0x62, 0x72, 0x65, 0x76, 0xee, 0x6c, 0x4d, 0x61, 0x73,
0x65, 0x72, 0xfb, 0x4d, 0x6f, 0x67, 0x61, 0x64, 0xee, 0x15f, 0xfb, 0x4e,
0x61, 0xee, 0x72, 0x6f, 0x62, 0xee, 0x54, 0x72, 0x61, 0x62, 0x6c, 0xfb,
0x73, 0x54, 0xfb, 0x6e, 0x69, 0x73, 0x41, 0x72, 0xfb, 0x62, 0x61, 0x42,
0x61, 0x68, 0xee, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x61,
0x73, 0x42, 0x65, 0x6c, 0xee, 0x7a, 0x65, 0x43, 0x61, 0x6e, 0x63, 0xfb,
0x6e, 0x43, 0x69, 0xfb, 0x64, 0x61, 0x64, 0x20, 0x4a, 0x75, 0x61, 0x72,
0x65, 0x7a, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x20, 0x52, 0xee, 0x6b, 0x61,
0x44, 0x6f, 0x6d, 0xee, 0x6e, 0xee, 0x6b, 0x61, 0x4a, 0x61, 0x6d, 0x61,
0xee, 0x6b, 0x61, 0x4d, 0x65, 0x72, 0xee, 0x64, 0x61, 0x42, 0x65, 0xfb,
0x6c, 0x61, 0x68, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x79,
0x61, 0x20, 0x42, 0x61, 0x6b, 0x75, 0x72, 0x43, 0x65, 0x6e, 0x74, 0x65,
0x72, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x79, 0x61, 0x20,
0x42, 0x61, 0x6b, 0x75, 0x72, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c,
0x65, 0x6d, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x79, 0x61,
0x20, 0x42, 0x61, 0x6b, 0x75, 0x72, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x20,
0x52, 0xee, 0x6b, 0x6f, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x42, 0x61,
0x72, 0x74, 0x68, 0x65, 0x6c, 0x65, 0x6d, 0x79, 0x44, 0x61, 0x76, 0xee,
0x73, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0xee, 0x45, 0x6d, 0x61, 0x6e, 0x45,
0x15f, 0x71, 0x61, 0x62, 0x61, 0x74, 0x41, 0x74, 0xee, 0x72, 0x61, 0x76,
0x42, 0x65, 0x78, 0x64, 0x61, 0x42, 0x65, 0x68, 0x72, 0x65, 0x79, 0x6e,
0x42, 0x61, 0x6b, 0xfb, 0x42, 0x65, 0x79, 0x72, 0xfb, 0x74, 0x42, 0xee,
0x15f, 0x6b, 0x65, 0x6b, 0x42, 0x72, 0xfb, 0x6e, 0x65, 0x79, 0xc7, 0xee,
0x74, 0x61, 0x15e, 0x61, 0x6d, 0x44, 0xee, 0x6c, 0xee, 0x44, 0xfb, 0x62,
0x61, 0xee, 0x44, 0x75, 0x15f, 0x65, 0x6e, 0x62, 0x65, 0x58, 0x65, 0x7a,
0x65, 0x42, 0x61, 0x6a, 0x61, 0x72, 0xea, 0x20, 0x48, 0x6f, 0x20, 0x43,
0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0xea, 0x43, 0x61, 0x79, 0x61,
0x70, 0xfb, 0x72, 0x61, 0x51, 0x75, 0x64, 0x73, 0x4b, 0x61, 0x62, 0xfb,
0x6c, 0x4b, 0x61, 0x72, 0x61, 0xe7, 0xee, 0x4b, 0x61, 0x74, 0x6d, 0x61,
0x6e, 0x64, 0xfb, 0x58, 0x61, 0x6e, 0x64, 0xee, 0x67, 0x61, 0x4b, 0xfb,
0x61, 0x6c, 0x61, 0x20, 0x4c, 0xfb, 0x6d, 0x70, 0xfb, 0x72, 0x4b, 0xfb,
0xe7, 0xee, 0x6e, 0x67, 0x4b, 0x75, 0x77, 0x65, 0x79, 0x74, 0x4d, 0x61,
0x6e, 0xee, 0x6c, 0x61, 0x4c, 0x65, 0x66, 0x6b, 0x6f, 0x15f, 0x65, 0x51,
0x65, 0x74, 0x65, 0x72, 0x51, 0x69, 0x7a, 0x69, 0x6c, 0x6f, 0x72, 0x64,
0x61, 0x53, 0x61, 0x78, 0x61, 0x6c, 0xee, 0x6e, 0x53, 0x65, 0x6d, 0x65,
0x72, 0x6b, 0x61, 0x6e, 0x64, 0x53, 0x65, 0xfb, 0x6c, 0x15e, 0x61, 0x6e,
0x67, 0x68, 0x61, 0x79, 0x53, 0xee, 0x6e, 0x67, 0x61, 0x70, 0xfb, 0x72,
0x54, 0x61, 0xee, 0x70, 0x65, 0xee, 0x54, 0x61, 0x15f, 0x6b, 0x65, 0x6e,
0x74, 0x54, 0x69, 0x66, 0x6c, 0xee, 0x73, 0xdb, 0x6c, 0x61, 0x6e, 0x62,
0x61, 0x74, 0x61, 0x72, 0xdb, 0x72, 0xfb, 0x6d, 0xe7, 0xee, 0xdb, 0x73,
0x74, 0x2d, 0x4e, 0x65, 0x72, 0x61, 0x52, 0x65, 0x77, 0x61, 0x6e, 0x47,
0x69, 0x72, 0x61, 0x76, 0xea, 0x6e, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0xea,
0x42, 0x65, 0x72, 0x6d, 0xfb, 0x64, 0x61, 0x47, 0x69, 0x72, 0x61, 0x76,
0xea, 0x6e, 0x20, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x79, 0x61, 0x79, 0xea,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x79, 0x61, 0x20, 0x42, 0x61,
0x15f, 0xfb, 0x72, 0x53, 0xee, 0x64, 0x6e, 0x65, 0x79, 0x41, 0x73, 0x74,
0x72, 0x61, 0x78, 0x61, 0x6e, 0x41, 0x74, 0xee, 0x6e, 0x61, 0x42, 0x65,
0x72, 0x6c, 0xee, 0x6e, 0x42, 0x75, 0x6b, 0x72, 0x65, 0x15f, 0x42, 0xfb,
0x64, 0x61, 0x70, 0x65, 0x15f, 0x74, 0x42, 0xfb, 0x73, 0xee, 0x6e, 0x67,
0x65, 0x6e, 0x4b, 0x69, 0x15f, 0xee, 0x6e, 0x65, 0x77, 0x44, 0xfb, 0x62,
0x6c, 0xee, 0x6e, 0x43, 0x65, 0x62, 0x65, 0x6c, 0xee, 0x74, 0x61, 0x72,
0x69, 0x71, 0x48, 0x65, 0x6c, 0x73, 0xee, 0x6e, 0x6b, 0xee, 0x47, 0x69,
0x72, 0x61, 0x76, 0x61, 0x20, 0x4d, 0x61, 0x6e, 0xea, 0x53, 0x74, 0x65,
0x6e, 0x62, 0x6f, 0x6c, 0x4b, 0x61, 0x6c, 0xee, 0x6e, 0xee, 0x6e, 0x67,
0x72, 0x61, 0x64, 0x4b, 0xee, 0x72, 0x6f, 0x76, 0x4b, 0xee, 0x65, 0x76,
0x4c, 0xee, 0x7a, 0x62, 0x6f, 0x6e, 0x4c, 0x75, 0x6b, 0x73, 0x65, 0x6d,
0x62, 0xfb, 0x72, 0x67, 0x4d, 0x61, 0x64, 0x72, 0xee, 0x64, 0x4d, 0xee,
0x6e, 0x73, 0x6b, 0x50, 0x61, 0x72, 0xee, 0x73, 0x50, 0x6f, 0x64, 0x67,
0x6f, 0x72, 0xee, 0x6b, 0x61, 0x52, 0xee, 0x67, 0x61, 0x53, 0x61, 0x6e,
0x20, 0x4d, 0x61, 0x72, 0xee, 0x6e, 0x6f, 0x53, 0x61, 0x72, 0x61, 0x79,
0x62, 0x6f, 0x73, 0x6e, 0x61, 0x55, 0x73, 0x6b, 0x75, 0x70, 0x54, 0x61,
0x6c, 0xee, 0x6e, 0x54, 0xee, 0x72, 0x61, 0x6e, 0x56, 0x61, 0x74, 0xee,
0x6b, 0x61, 0x6e, 0x56, 0x69, 0x79, 0x61, 0x6e, 0x61, 0x56, 0xee, 0x6c,
0x6e, 0xfb, 0x73, 0x57, 0x61, 0x72, 0x15f, 0x6f, 0x76, 0x61, 0x5a, 0xfb,
0x72, 0xee, 0x68, 0x4d, 0x61, 0x6c, 0x64, 0xee, 0x76, 0x46, 0xee, 0x6a,
0xee, 0x54, 0x61, 0x68, 0xee, 0x74, 0xee, 0x411, 0x430, 0x433, 0x438, 0x411,
0x440, 0x430, 0x437, 0x437, 0x430, 0x432, 0x438, 0x43b, 0x43b, 0x414, 0x430, 0x440,
0x20, 0x44d, 0x441, 0x20, 0x421, 0x430, 0x43b, 0x430, 0x430, 0x43c, 0x416, 0x438,
0x431, 0x443, 0x442, 0x438, 0x42d, 0x43b, 0x20, 0x410, 0x439, 0x443, 0x43d, 0x416,
0x443, 0x431, 0x430, 0x41b, 0x438, 0x431, 0x440, 0x435, 0x432, 0x438, 0x43b, 0x43b,
0x41d, 0x436, 0x430, 0x43c, 0x435, 0x43d, 0x430, 0x41d, 0x439, 0x430, 0x43c, 0x435,
0x423, 0x438, 0x43d, 0x434, 0x445, 0x443, 0x43a, 0x410, 0x43d, 0x43a, 0x43e, 0x440,
0x438, 0x436, 0x420, 0x438, 0x43e, 0x20, 0x413, 0x430, 0x43b, 0x43b, 0x435, 0x433,
0x43e, 0x441, 0x423, 0x448, 0x443, 0x430, 0x439, 0x430, 0x411, 0x430, 0x439, 0x438,
0x430, 0x411, 0x430, 0x438, 0x439, 0x430, 0x20, 0x411, 0x430, 0x43d, 0x434, 0x435,
0x440, 0x430, 0x441, 0x41a, 0x44d, 0x43c, 0x431, 0x440, 0x438, 0x436, 0x20, 0x411,
0x435, 0x439, 0x427, 0x438, 0x445, 0x443, 0x430, 0x445, 0x443, 0x430, 0x41a, 0x443,
0x439, 0x430, 0x431, 0x430, 0x414, 0x43e, 0x443, 0x441, 0x43e, 0x43d, 0x20, 0x413,
0x440, 0x435, 0x43a, 0x42d, 0x43b, 0x20, 0x421, 0x430, 0x43b, 0x432, 0x430, 0x434,
0x43e, 0x440, 0x413, 0x440, 0x430, 0x43d, 0x434, 0x20, 0x422, 0x4af, 0x440, 0x43a,
0x413, 0x432, 0x430, 0x434, 0x435, 0x43b, 0x443, 0x43f, 0x435, 0x413, 0x443, 0x430,
0x439, 0x430, 0x43a, 0x438, 0x43b, 0x413, 0x443, 0x439, 0x430, 0x43d, 0x430, 0x42d,
0x440, 0x43c, 0x43e, 0x441, 0x438, 0x43b, 0x43b, 0x43e, 0x41f, 0x438, 0x442, 0x435,
0x440, 0x441, 0x431, 0x443, 0x440, 0x433, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438,
0x430, 0x43d, 0x430, 0x422, 0x435, 0x43b, 0x43b, 0x20, 0x421, 0x438, 0x442, 0x438,
0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d, 0x430, 0x412, 0x438, 0x43d,
0x441, 0x435, 0x43d, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d, 0x430,
0x41b, 0x443, 0x438, 0x437, 0x432, 0x438, 0x43b, 0x43b, 0x41c, 0x43e, 0x43d, 0x442,
0x438, 0x447, 0x435, 0x43b, 0x43b, 0x43e, 0x2c, 0x20, 0x41a, 0x435, 0x43d, 0x442,
0x443, 0x43a, 0x43a, 0x438, 0x41b, 0x43e, 0x441, 0x2d, 0x410, 0x43d, 0x436, 0x435,
0x43b, 0x435, 0x441, 0x41b, 0x43e, 0x443, 0x44d, 0x440, 0x20, 0x41f, 0x440, 0x438,
0x43d, 0x441, 0x435, 0x441, 0x20, 0x41a, 0x443, 0x430, 0x442, 0x435, 0x41c, 0x435,
0x445, 0x438, 0x43a, 0x43e, 0x20, 0x448, 0x430, 0x430, 0x440, 0x44b, 0x41c, 0x43e,
0x43d, 0x441, 0x435, 0x440, 0x430, 0x442, 0x411, 0x435, 0x443, 0x43b, 0x430, 0x2c,
0x20, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a, 0x20, 0x414, 0x430, 0x43a, 0x43e,
0x442, 0x430, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a, 0x20, 0x414, 0x430, 0x43a,
0x43e, 0x442, 0x430, 0x43d, 0x44b, 0x43d, 0x20, 0x431, 0x43e, 0x440, 0x431, 0x43e,
0x440, 0x443, 0x41d, 0x44c, 0x44e, 0x2d, 0x421, 0x430, 0x43b, 0x435, 0x43c, 0x2c,
0x20, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a, 0x20, 0x414, 0x430, 0x43a, 0x43e,
0x442, 0x430, 0x41f, 0x43e, 0x440, 0x442, 0x20, 0x43e, 0x444, 0x20, 0x421, 0x43f,
0x435, 0x439, 0x43d, 0x41f, 0x43e, 0x440, 0x442, 0x43e, 0x20, 0x412, 0x435, 0x43b,
0x438, 0x43e, 0x420, 0x44d, 0x43d, 0x43a, 0x438, 0x43d, 0x20, 0x418, 0x43d, 0x43b,
0x435, 0x442, 0x420, 0x435, 0x437, 0x43e, 0x43b, 0x443, 0x442, 0x418, 0x442, 0x442,
0x43e, 0x43a, 0x43a, 0x43e, 0x440, 0x442, 0x43e, 0x43e, 0x440, 0x43c, 0x438, 0x438,
0x442, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x411, 0x430, 0x440, 0x442, 0x435, 0x43b,
0x435, 0x43c, 0x438, 0x421, 0x435, 0x43d, 0x442, 0x20, 0x416, 0x43e, 0x43d, 0x441,
0x421, 0x430, 0x43d, 0x442, 0x430, 0x20, 0x41b, 0x44e, 0x447, 0x438, 0x44f, 0x421,
0x432, 0x438, 0x444, 0x442, 0x20, 0x41a, 0x430, 0x440, 0x440, 0x435, 0x43d, 0x442,
0x423, 0x438, 0x43d, 0x43d, 0x438, 0x43f, 0x435, 0x433, 0x414, 0x44d, 0x432, 0x438,
0x441, 0x414, 0x44e, 0x43c, 0x43e, 0x43d, 0x20, 0x434, 0x2d, 0x423, 0x440, 0x432,
0x438, 0x43b, 0x41c, 0x430, 0x43a, 0x41c, 0x451, 0x440, 0x434, 0x43e, 0x421, 0x430,
0x43e, 0x430, 0x41b, 0x43e, 0x43d, 0x433, 0x439, 0x435, 0x440, 0x431, 0x438, 0x435,
0x43d, 0x416, 0x430, 0x43a, 0x430, 0x440, 0x442, 0x430, 0x416, 0x430, 0x439, 0x430,
0x43f, 0x443, 0x440, 0x430, 0x41a, 0x430, 0x43d, 0x434, 0x44b, 0x433, 0x430, 0x41a,
0x443, 0x447, 0x438, 0x4a3, 0x41c, 0x430, 0x43a, 0x430, 0x441, 0x441, 0x430, 0x440,
0x420, 0x438, 0x439, 0x430, 0x434, 0x4ae, 0x440, 0x4af, 0x43c, 0x447, 0x4af, 0x412,
0x438, 0x435, 0x43d, 0x442, 0x438, 0x430, 0x43d, 0x41a, 0x430, 0x43d, 0x430, 0x440,
0x438, 0x41a, 0x430, 0x43f, 0x435, 0x20, 0x412, 0x435, 0x440, 0x434, 0x435, 0x422,
0x4af, 0x448, 0x442, 0x4af, 0x43a, 0x20, 0x416, 0x43e, 0x440, 0x436, 0x438, 0x44f,
0x421, 0x435, 0x43d, 0x442, 0x20, 0x415, 0x43b, 0x435, 0x43d, 0x430, 0x411, 0x440,
0x43e, 0x443, 0x43a, 0x435, 0x43d, 0x20, 0x425, 0x438, 0x43b, 0x411, 0x443, 0x437,
0x438, 0x43d, 0x433, 0x435, 0x43d, 0x416, 0x435, 0x440, 0x441, 0x438, 0x421, 0x43a,
0x43e, 0x43f, 0x439, 0x435, 0x424, 0x430, 0x434, 0x443, 0x446, 0x41a, 0x440, 0x438,
0x441, 0x43c, 0x430, 0x441, 0x41e, 0x43a, 0x43b, 0x44d, 0x43d, 0x434, 0x427, 0x443,
0x443, 0x43a, 0x418, 0x441, 0x442, 0x435, 0x440, 0x424, 0x438, 0x436, 0x438, 0x41a,
0x43e, 0x437, 0x440, 0x430, 0x435, 0x41a, 0x443, 0x430, 0x436, 0x430, 0x43b, 0x435,
0x439, 0x43d, 0x41c, 0x430, 0x436, 0x443, 0x440, 0x43e, 0x41c, 0x430, 0x440, 0x43a,
0x435, 0x437, 0x430, 0x441, 0x41f, 0x43e, 0x440, 0x442, 0x20, 0x41c, 0x43e, 0x44d,
0x440, 0x441, 0x431, 0x438, 0x420, 0x430, 0x433, 0x43e, 0x442, 0x43e, 0x43d, 0x433,
0x430, 0xead, 0xeb2, 0xe9a, 0xeb4, 0xe94, 0xec1, 0xe88, 0xe99, 0xead, 0xeb1, 0xe81,
0xe84, 0xea3, 0xeb2, 0xec1, 0xead, 0xe94, 0xe94, 0xeb4, 0xeaa, 0x20, 0xead, 0xeb0,
0xe9a, 0xeb2, 0xe9a, 0xeb2, 0xec1, 0xead, 0xea7, 0xec0, 0xe88, 0xe8d, 0xead, 0xeb1,
0xeaa, 0xea1, 0xeb2, 0xea3, 0xeb2, 0xe9a, 0xeb2, 0xea1, 0xeb2, 0xec2, 0xe81, 0xe9a,
0xeb1, 0xe87, 0xe81, 0xeb8, 0xe8d, 0xe9a, 0xeb2, 0xe99, 0xe88, 0xeb9, 0xe9a, 0xeb4,
0xeaa, 0xec0, 0xe8a, 0xebb, 0xeb2, 0xec1, 0xe9a, 0xea5, 0xe99, 0xec4, 0xe97, 0xea3,
0xecc, 0xe9a, 0xea3, 0xeb2, 0xe8a, 0xeb2, 0xea7, 0xeb4, 0xea7, 0xe9a, 0xeb9, 0xe88,
0xeb9, 0xea1, 0xe9a, 0xeb9, 0xea3, 0xeb2, 0xec4, 0xe84, 0xec2, 0xea3, 0xe84, 0xeb2,
0xe8a, 0xeb2, 0xe9a, 0xea5, 0xeb1, 0xe87, 0xe81, 0xeb2, 0xe8a, 0xeb5, 0xea7, 0xe95,
0xeb2, 0xec2, 0xe84, 0xe99, 0xeb2, 0xe84, 0xea3, 0xeb5, 0xe94, 0xeb2, 0xe81, 0xeb2,
0xe94, 0xeb2, 0xec0, 0xead, 0xeaa, 0xeaa, 0xeb0, 0xea5, 0xeb2, 0xea1, 0xe88, 0xeb5,
0xe9a, 0xeb9, 0xe95, 0xeb4, 0xe94, 0xeb9, 0xead, 0xeb2, 0xea5, 0xeb2, 0xec0, 0xead,
0xea7, 0x20, 0xead, 0xeb2, 0xea2, 0xeb9, 0xe99, 0xe9f, 0xea3, 0xeb5, 0xe97, 0xeb2,
0xea7, 0xe81, 0xeb2, 0xe9a, 0xecd, 0xec2, 0xea3, 0xe99, 0xeae, 0xeb2, 0xea3, 0xeb2,
0xec0, 0xea3, 0xec2, 0xe88, 0xeae, 0xeb1, 0xe99, 0xec0, 0xe99, 0xe94, 0xeaa, 0xec0,
0xe9a, 0xeb5, 0xe81, 0xe88, 0xeb9, 0xe9a, 0xeb2, 0xe84, 0xeb3, 0xe9b, 0xeb2, 0xea5,
0xeb2, 0xe84, 0xeb2, 0xe97, 0xeb9, 0xea1, 0xe84, 0xeb5, 0xe81, 0xeb2, 0xea5, 0xeb5,
0xe81, 0xeb4, 0xe99, 0xe8a, 0xeb2, 0xe8a, 0xeb2, 0xea5, 0xeb2, 0xec2, 0xe81, 0xeaa,
0xea5, 0xeb4, 0xec0, 0xe9a, 0xebc, 0xeb5, 0xea7, 0xeb4, 0xea7, 0xec2, 0xea5, 0xec0,
0xea1, 0xea5, 0xea7, 0xe99, 0xe94, 0xeb2, 0xea5, 0xeb9, 0xe9a, 0xeb3, 0xe9a, 0xeb2,
0xe8a, 0xeb4, 0xea5, 0xeb9, 0xe8a, 0xeb2, 0xe81, 0xeb2, 0xea1, 0xeb2, 0xea5, 0xeb2,
0xec2, 0xe9a, 0xea1, 0xeb2, 0xe9b, 0xeb9, 0xec2, 0xe95, 0xea1, 0xeb2, 0xec0, 0xe8a,
0xea3, 0xeb9, 0xead, 0xeb3, 0xe9a, 0xeb2, 0xe9a, 0xeb2, 0xec0, 0xe99, 0xec2, 0xea1,
0xe81, 0xeb2, 0xe94, 0xeb5, 0xe8a, 0xeb9, 0xea1, 0xead, 0xe99, 0xec2, 0xea3, 0xec0,
0xea7, 0xe8d, 0xec4, 0xe99, 0xec2, 0xea3, 0xe9a, 0xeb5, 0xec0, 0xead, 0xe99, 0xe88,
0xeb2, 0xec0, 0xea1, 0xe99, 0xec8, 0xeb2, 0xe99, 0xeb5, 0xead, 0xeb2, 0xec0, 0xea1,
0xe99, 0xeb9, 0xec1, 0xead, 0xe81, 0xe8a, 0xead, 0xe94, 0xead, 0xeb9, 0xe81, 0xeb2,
0xe94, 0xeb9, 0xe81, 0xeb9, 0xe9b, 0xecd, 0xec2, 0xe95, 0x2d, 0xec2, 0xe99, 0xec2,
0xea7, 0xe8a, 0xeb2, 0xea7, 0xec2, 0xe95, 0xec0, 0xea1, 0xe97, 0xea3, 0xeb4, 0xec2,
0xe9e, 0xea5, 0xeb4, 0xe95, 0xeb9, 0xe99, 0xeb4, 0xeaa, 0xea7, 0xeb5, 0xe99, 0xeae,
0xeb9, 0xe94, 0xead, 0xeb2, 0xec1, 0xe94, 0xe81, 0xec1, 0xead, 0xe99, 0xec0, 0xe84,
0xeb5, 0xec0, 0xea3, 0xe81, 0xec1, 0xead, 0xe99, 0xe81, 0xeb4, 0xe99, 0xea5, 0xeb2,
0xec1, 0xead, 0xe99, 0xe97, 0xeb4, 0xe81, 0xebb, 0xea7, 0xead, 0xeb2, 0xea3, 0xeb2,
0xe81, 0xebb, 0xea7, 0xe99, 0xeb2, 0xe9a, 0xebb, 0xea7, 0xec2, 0xe99, 0xeaa, 0x20,
0xec4, 0xead, 0xec0, 0xea3, 0xeaa, 0xe84, 0xeb2, 0xe95, 0xeb2, 0xea1, 0xeb2, 0xe81,
0xeb2, 0xe84, 0xecd, 0xec2, 0xe94, 0xe9a, 0xeb2, 0xe88, 0xeb9, 0xe88, 0xeb8, 0xe8d,
0xea5, 0xeb2, 0x20, 0xea3, 0xeb4, 0xec2, 0xead, 0xe88, 0xeb2, 0xec0, 0xea1, 0xeb1,
0xe99, 0xec2, 0xe94, 0xe8a, 0xec8, 0xeb2, 0xea3, 0xeb4, 0xec2, 0xe81, 0x20, 0xec1,
0xe81, 0xea7, 0xec0, 0xea5, 0xe81, 0xead, 0xeaa, 0xe8a, 0xeb2, 0xe99, 0xe95, 0xeb2,
0xec1, 0xe8a, 0xe99, 0xeae, 0xea7, 0xe99, 0xec1, 0xe8a, 0xe99, 0xea5, 0xeb8, 0xe8d,
0xeaa, 0xecc, 0xe95, 0xeb9, 0xe84, 0xeb9, 0xec1, 0xea1, 0xe99, 0xead, 0xeb9, 0xe8a,
0xeb9, 0xec0, 0xead, 0xe8d, 0xead, 0xeb2, 0xea3, 0xeb9, 0xe9a, 0xeb2, 0xead, 0xeb0,
0xe8a, 0xeb8, 0xe99, 0xe8a, 0xeb4, 0xead, 0xead, 0xe87, 0xead, 0xeb2, 0xe97, 0xeb4,
0xec2, 0xe84, 0xe84, 0xeb2, 0xe99, 0xe9a, 0xeb2, 0xec0, 0xea2, 0xe8d, 0xe9a, 0xeb2,
0xec0, 0xeae, 0xe8d, 0x20, 0xec1, 0xe9a, 0xe99, 0xec0, 0xe94, 0xeb5, 0xea3, 0xeb2,
0xeaa, 0xe9a, 0xeb2, 0xec0, 0xe9a, 0xe94, 0xead, 0xeaa, 0xe9a, 0xeb5, 0xec0, 0xea5,
0xea1, 0xec0, 0xe9a, 0xea5, 0xeb5, 0xe8a, 0xe9a, 0xea5, 0xeb2, 0xe99, 0x2d, 0xe8a,
0xeb2, 0xe9a, 0xea5, 0xead, 0xe99, 0xe9a, 0xebb, 0xea7, 0x20, 0xea7, 0xeb4, 0xeaa,
0xe95, 0xeb2, 0xec2, 0xe9a, 0xec2, 0xe81, 0xe95, 0xeb2, 0xe9a, 0xead, 0xe8d, 0xe8a,
0xecc, 0xec1, 0xe84, 0xea1, 0xe9a, 0xea3, 0xeb4, 0xe94, 0x20, 0xec0, 0xe9a, 0xe81,
0xeb1, 0xe87, 0xe9b, 0xeb9, 0xea3, 0xeb1, 0xe87, 0xe88, 0xeb5, 0xec1, 0xe84, 0xe99,
0xe81, 0xeb8, 0xe99, 0xe84, 0xeb2, 0xea3, 0xeb2, 0xe81, 0xeb1, 0xeaa, 0xe84, 0xeb2,
0xec0, 0xea2, 0xe99, 0xec0, 0xe84, 0xec1, 0xea1, 0xe99, 0xe8a, 0xeb4, 0xe84, 0xeb2,
0xec2, 0xe81, 0xe8a, 0xeb4, 0xea7, 0xeb2, 0xea7, 0xeb2, 0xe8a, 0xeb5, 0xead, 0xeb9,
0xe94, 0xeb1, 0xe94, 0x20, 0xe84, 0xebb, 0xea7, 0xec0, 0xea3, 0xe8a, 0xe84, 0xead,
0xeaa, 0xe95, 0xeb2, 0xea3, 0xeb4, 0xe81, 0xeb2, 0xec0, 0xe84, 0xea3, 0xeaa, 0xe95,
0xeb1, 0xe99, 0xe81, 0xeb8, 0xea2, 0xeb2, 0xe9a, 0xeb2, 0xe81, 0xeb7, 0xea3, 0xeb2,
0xec0, 0xe8a, 0xebb, 0xeb2, 0xec0, 0xe94, 0xe99, 0xea1, 0xeb2, 0xe81, 0xec1, 0xe8a,
0xe99, 0xe94, 0xecd, 0xeaa, 0xeb1, 0xe99, 0xe94, 0xecd, 0xeaa, 0xeb1, 0xe99, 0x20,
0xe84, 0xea3, 0xeb5, 0xe81, 0xec0, 0xe94, 0xeb1, 0xe99, 0xec0, 0xea7, 0xeb5, 0xe94,
0xeb5, 0xe97, 0xea3, 0xead, 0xe8d, 0xec2, 0xe94, 0xea1, 0xeb4, 0xe99, 0xeb4, 0xe81,
0xeb2, 0xec0, 0xead, 0xe94, 0xea1, 0xead, 0xe99, 0xe95, 0xead, 0xe99, 0xec0, 0xead,
0xea3, 0xeb8, 0xec0, 0xe99, 0xe9b, 0xeb5, 0xec0, 0xead, 0xea7, 0x20, 0xe8a, 0xeb2,
0xea7, 0xeb2, 0xe94, 0xecd, 0xe9f, 0xead, 0xe94, 0x20, 0xec0, 0xe99, 0xea7, 0xeaa,
0xeb1, 0xe99, 0xe9f, 0xecd, 0xe95, 0xeb2, 0xec0, 0xea5, 0xe8a, 0xeb2, 0xec0, 0xe81,
0xea5, 0xe94, 0xec0, 0xe9a, 0xe81, 0xeb9, 0xeaa, 0xec0, 0xe9a, 0xec1, 0xe81, 0xea3,
0xe99, 0x20, 0xec0, 0xe97, 0xeb5, 0xe81, 0xec0, 0xe81, 0xea3, 0xe99, 0xeb2, 0xe94,
0xeb2, 0xe81, 0xeb2, 0xea7, 0xec0, 0xe94, 0xea5, 0xeb9, 0xe9a, 0xe81, 0xebb, 0xea7,
0xec0, 0xe95, 0xea1, 0xeb2, 0xea5, 0xeb2, 0xe81, 0xebb, 0xea7, 0xea2, 0xeb2, 0xe81,
0xeb4, 0xea5, 0xe81, 0xeb9, 0xea2, 0xeb2, 0xe99, 0xeb2, 0xeae, 0xeb2, 0xea5, 0xeb4,
0xec1, 0xe9f, 0xeb1, 0xe81, 0xeae, 0xeb2, 0xea7, 0xeb2, 0xe99, 0xeb2, 0xec0, 0xead,
0xec2, 0xea1, 0xe8a, 0xeb4, 0xec2, 0xea2, 0xead, 0xeb4, 0xe99, 0xe94, 0xeb5, 0xec0,
0xead, 0xe99, 0xeb2, 0xec2, 0xe9e, 0xea5, 0xeb4, 0xeaa, 0xe99, 0xecb, 0xead, 0xe81,
0x2c, 0x20, 0xead, 0xeb4, 0xe99, 0xec0, 0xe94, 0xe8d, 0xe99, 0xeb2, 0xea1, 0xeb2,
0xec0, 0xea3, 0xe99, 0xec2, 0xe81, 0x2c, 0x20, 0xead, 0xeb4, 0xe99, 0xec0, 0xe94,
0xe8d, 0xe99, 0xeb2, 0xe9b, 0xeb5, 0xec0, 0xe95, 0xeb5, 0xeaa, 0xec0, 0xe9b, 0xeb5,
0xe81, 0x2c, 0x20, 0xead, 0xeb4, 0xe99, 0xec0, 0xe94, 0xe8d, 0xe99, 0xeb2, 0xec0,
0xe97, 0xea7, 0x20, 0xe8a, 0xeb4, 0xe95, 0xeb5, 0x2c, 0x20, 0xead, 0xeb4, 0xe99,
0xec0, 0xe94, 0xe8d, 0xe99, 0xeb2, 0xea7, 0xeb5, 0xec0, 0xea7, 0x2c, 0x20, 0xead,
0xeb4, 0xe99, 0xec0, 0xe94, 0xe8d, 0xe99, 0xeb2, 0xea7, 0xeb4, 0xe99, 0xec0, 0xe8a,
0xe99, 0xec0, 0xe99, 0xeaa, 0x2c, 0x20, 0xead, 0xeb4, 0xe99, 0xec0, 0xe94, 0xe8d,
0xe99, 0xeb2, 0xea7, 0xeb4, 0xe99, 0xeb2, 0xec1, 0xea1, 0xeb1, 0xe81, 0x2c, 0x20,
0xead, 0xeb4, 0xe99, 0xec0, 0xe94, 0xe8d, 0xe99, 0xeb2, 0xead, 0xeb5, 0xe99, 0xeb9,
0xea7, 0xeb4, 0xe81, 0xead, 0xeb5, 0xe81, 0xebb, 0xea7, 0xea5, 0xeb4, 0xe94, 0xe88,
0xeb2, 0xec4, 0xea1, 0xe81, 0xeb2, 0xe88, 0xeb9, 0xec2, 0xe99, 0xeab, 0xea5, 0xeb8,
0xe8d, 0xea7, 0xeb4, 0xea7, 0xea1, 0xead, 0xe99, 0xe95, 0xeb5, 0xec0, 0xe8a, 0xea7,
0xec2, 0xea5, 0x2c, 0x20, 0xec0, 0xe84, 0xe99, 0xe97, 0xeb1, 0xe81, 0xe81, 0xeb5,
0xe84, 0xea3, 0xeb2, 0xec0, 0xea5, 0xe99, 0xe94, 0xeb4, 0xe81, 0xea5, 0xeb2, 0xe9b,
0xeb2, 0xeaa, 0xea5, 0xeb5, 0xea1, 0xeb2, 0xea5, 0xead, 0xeaa, 0xec1, 0xead, 0xe99,
0xec0, 0xe88, 0xea5, 0xeb5, 0xeaa, 0xec2, 0xea5, 0xec0, 0xea7, 0xeb5, 0x20, 0xe9e,
0xea3, 0xeb4, 0xe99, 0xe8a, 0xecc, 0x20, 0xe84, 0xea7, 0xe94, 0xec0, 0xe95, 0xeb5,
0xea1, 0xeb2, 0xec0, 0xe8a, 0xec2, 0xead, 0xea1, 0xeb2, 0xe99, 0xeb2, 0xe81, 0xebb,
0xea7, 0xea1, 0xeb2, 0xec0, 0xe99, 0xebb, 0xeb2, 0xeaa, 0xecc, 0xea1, 0xeb2, 0xea3,
0xeb5, 0xec2, 0xe81, 0xe94, 0xea1, 0xeb2, 0xe97, 0xeb5, 0xe99, 0xeb4, 0xe81, 0xea1,
0xeb2, 0xe95, 0xeb2, 0xec2, 0xea1, 0xea3, 0xead, 0xeaa, 0xea1, 0xeb2, 0xe8a, 0xeb2,
0xe94, 0xea5, 0xeb2, 0xe99, 0xec0, 0xea1, 0xec2, 0xe99, 0xea1, 0xeb4, 0xe99, 0xeb5,
0xec0, 0xea1, 0xeb5, 0xea3, 0xeb4, 0xe94, 0xeb2, 0xec0, 0xea1, 0xeb1, 0xe94, 0xea5,
0xeb2, 0xe81, 0xeb1, 0xe94, 0xea5, 0xeb2, 0xec0, 0xea1, 0xeb1, 0xe81, 0xe8a, 0xeb4,
0xec2, 0xe81, 0x20, 0xe8a, 0xeb4, 0xe95, 0xeb5, 0xea1, 0xeb4, 0xe81, 0xebb, 0xea7,
0xea5, 0xead, 0xe99, 0xea1, 0xead, 0xe99, 0xe95, 0xeb1, 0xe99, 0xea1, 0xead, 0xe99,
0xec0, 0xe95, 0xeb5, 0xec0, 0xea3, 0xe8d, 0xea1, 0xead, 0xe99, 0xec0, 0xe95, 0xea7,
0xeb4, 0xec0, 0xe94, 0xec2, 0xead, 0xea1, 0xead, 0xe99, 0xec0, 0xe8a, 0xeb5, 0xea3,
0xeb1, 0xe94, 0xec1, 0xe99, 0xeaa, 0xe8a, 0xecd, 0xe99, 0xeb4, 0xea7, 0xea2, 0xead,
0xe81, 0xe99, 0xead, 0xea1, 0xe99, 0xecd, 0xea3, 0xead, 0xe99, 0xeae, 0xeb2, 0xec0,
0xe9a, 0xeb5, 0xea5, 0xeb2, 0x2c, 0x20, 0xe99, 0xead, 0xe94, 0x20, 0xe94, 0xeb2,
0xec2, 0xe81, 0xe95, 0xeb2, 0xec0, 0xe8a, 0xeb1, 0xe99, 0xec0, 0xe88, 0xeb5, 0x2c,
0x20, 0xe99, 0xead, 0xe94, 0x20, 0xe94, 0xeb2, 0xec2, 0xe81, 0xe95, 0xeb2, 0xe99,
0xeb4, 0xea7, 0x20, 0xe8a, 0xeb2, 0xec0, 0xea5, 0xea1, 0x2c, 0x20, 0xe99, 0xead,
0xe94, 0x20, 0xe94, 0xeb2, 0xec2, 0xe81, 0xe95, 0xeb2, 0xe99, 0xeb9, 0xe81, 0xec2,
0xead, 0xe88, 0xeb4, 0xe99, 0xeb2, 0xe81, 0xeb2, 0xe9e, 0xeb2, 0xe99, 0xeb2, 0xea1,
0xeb2, 0xe9e, 0xeb2, 0xea3, 0xeb2, 0xea1, 0xeb2, 0xea3, 0xeb4, 0xec2, 0xe9a, 0xe9f,
0xeb4, 0xe99, 0xeb4, 0xe81, 0xe9b, 0xecd, 0xec2, 0xe95, 0xec1, 0xe9b, 0xea3, 0xe87,
0xe9e, 0xead, 0xe94, 0xead, 0xead, 0xe9a, 0xeaa, 0xeb0, 0xec0, 0xe9b, 0xe99, 0xe9b,
0xecd, 0xe95, 0xeb9, 0x20, 0xec0, 0xea7, 0xea7, 0xea2, 0xeb9, 0xec0, 0xe9b, 0xeb5,
0xec2, 0xe97, 0xea3, 0xeb4, 0xec2, 0xe81, 0xe9e, 0xeb8, 0xe99, 0xe97, 0xeb2, 0x20,
0xead, 0xeb0, 0xea3, 0xeb5, 0xe99, 0xeb2, 0xeaa, 0xecc, 0xec1, 0xea3, 0xe99, 0xe81,
0xeb4, 0xe99, 0x20, 0xead, 0xeb4, 0xe99, 0xec0, 0xea5, 0xeb1, 0xe94, 0xec0, 0xea3,
0xe8a, 0xeb4, 0xec0, 0xe9f, 0xec0, 0xea3, 0xe88, 0xeb4, 0xe99, 0xeb2, 0xec0, 0xea3,
0xec2, 0xe8a, 0xea5, 0xeb8, 0xe94, 0xea3, 0xeb4, 0xec2, 0xead, 0x20, 0xe9a, 0xea3,
0xeb1, 0xe99, 0xec2, 0xe81, 0xe8a, 0xeb1, 0xe99, 0xe95, 0xeb2, 0xec0, 0xea3, 0xea1,
0xe8a, 0xeb1, 0xe99, 0xe95, 0xeb5, 0xead, 0xeb2, 0xec2, 0xe81, 0xe8a, 0xeb2, 0xe99,
0xec2, 0xe95, 0xec2, 0xe94, 0xea1, 0xeb4, 0xe99, 0xec2, 0xe81, 0xec0, 0xe8a, 0xebb,
0xeb2, 0x20, 0xec0, 0xe9b, 0xebb, 0xeb2, 0xec2, 0xea5, 0xead, 0xeb4, 0xec2, 0xe95,
0xe84, 0xecd, 0xe97, 0xebb, 0xea7, 0xea1, 0xeb4, 0xe94, 0xe8a, 0xeb4, 0xe94, 0xe81,
0xeb2, 0xec0, 0xe8a, 0xe99, 0xe9a, 0xeb2, 0xec0, 0xe97, 0xec0, 0xea5, 0xea1, 0xeb5,
0xec0, 0xe8a, 0xe99, 0x20, 0xe88, 0xead, 0xe99, 0xec0, 0xe8a, 0xe99, 0x20, 0xe84,
0xeb4, 0xe94, 0xeaa, 0xecc, 0xec0, 0xe8a, 0xe99, 0x20, 0xea5, 0xeb9, 0xec0, 0xe8a,
0xe8d, 0xec0, 0xe8a, 0xe99, 0x20, 0xec2, 0xe97, 0xea1, 0xeb1, 0xeaa, 0xec0, 0xe8a,
0xe99, 0x20, 0xea7, 0xeb4, 0xe99, 0xec0, 0xe8a, 0xe99, 0xeaa, 0xeb0, 0xea7, 0xeb4,
0xe9f, 0x20, 0xec0, 0xe84, 0xeb5, 0xec0, 0xea3, 0xe99, 0xec0, 0xe95, 0xe81, 0xeb9,
0xe8a, 0xeb5, 0xe81, 0xeb2, 0xe99, 0xe9b, 0xeb2, 0xe97, 0xeb9, 0xec0, 0xea5, 0xe97,
0xeb5, 0xe88, 0xebb, 0xea7, 0xe99, 0xeb2, 0xec2, 0xe97, 0xea3, 0xead, 0xe99, 0xec2,
0xe95, 0xe97, 0xecd, 0xec2, 0xe95, 0xea5, 0xeb2, 0xec1, 0xea7, 0xe99, 0xe84, 0xeb9,
0xec0, 0xea7, 0xeb5, 0xec4, 0xea7, 0xe97, 0xecc, 0xeae, 0xead, 0xeaa, 0xea7, 0xeb4,
0xe99, 0xe99, 0xeb5, 0xec0, 0xe9e, 0xe81, 0xea2, 0xeb2, 0xe84, 0xeb9, 0xe95, 0xeb1,
0xe94, 0xec0, 0xe84, 0xe8a, 0xeb5, 0xe94, 0xeb2, 0xea7, 0xeb5, 0xeaa, 0xe94, 0xeb9,
0xea1, 0xead, 0xe99, 0xe94, 0xeb5, 0xe8d, 0xeb9, 0xea7, 0xeb5, 0xea7, 0xeaa, 0xecc,
0xec1, 0xea1, 0xeb1, 0xe81, 0xe84, 0xebb, 0xea7, 0xea3, 0xeb5, 0xec0, 0xea1, 0xebb,
0xeb2, 0xe8a, 0xeb1, 0xe99, 0xec1, 0xea1, 0xeb1, 0xe81, 0xea1, 0xebb, 0xea7, 0xec2,
0xe94, 0xe9e, 0xeb2, 0xea1, 0xec0, 0xea1, 0xeb5, 0xec2, 0xea3, 0xec0, 0xe97, 0xea3,
0xeb2, 0xec0, 0xe8a, 0xe8d, 0xea7, 0xeb2, 0xec2, 0xe97, 0xea3, 0xea5, 0xecc, 0xea7,
0xecd, 0xeaa, 0xeb0, 0xe95, 0xead, 0xe81, 0xea5, 0xead, 0xe87, 0xec0, 0xea2, 0xe8d,
0xe9a, 0xebd, 0xe99, 0xec0, 0xead, 0xec0, 0xe94, 0xeb1, 0xe99, 0xead, 0xecd, 0xea1,
0xeb2, 0xe95, 0xeb5, 0xead, 0xeb3, 0xea1, 0xeb2, 0xe99, 0xead, 0xeb2, 0xe99, 0xeb2,
0xe94, 0xeb5, 0xead, 0xeb1, 0xe94, 0xe95, 0xeb2, 0xead, 0xeb9, 0xead, 0xeb1, 0xe94,
0xec2, 0xe97, 0xe9a, 0xeb5, 0xead, 0xeb2, 0xe94, 0xe8a, 0xecc, 0xe81, 0xeb2, 0xe9a,
0xeb1, 0xe94, 0xead, 0xeb1, 0xe94, 0xe97, 0xeb5, 0xec0, 0xea3, 0xebb, 0xeb2, 0xec1,
0xe9a, 0xe81, 0xec1, 0xe94, 0xe94, 0xe9a, 0xeb2, 0xec0, 0xea3, 0xe99, 0xe9a, 0xeb2,
0xe81, 0xeb9, 0xe9a, 0xeb2, 0xe87, 0xe81, 0xead, 0xe81, 0xe9a, 0xeb2, 0xe99, 0xebb,
0xea7, 0xec0, 0xe9a, 0xea3, 0xeb8, 0xe94, 0xe9a, 0xeb4, 0xe94, 0xe8a, 0xecc, 0xec0,
0xe84, 0xe81, 0xe9a, 0xea3, 0xeb9, 0xec4, 0xe99, 0xe8a, 0xeb5, 0xe95, 0xec8, 0xeb2,
0xec2, 0xe84, 0xea5, 0xeb3, 0xec2, 0xe9a, 0xe94, 0xeb2, 0xea1, 0xeb2, 0xeaa, 0xe84,
0xeb1, 0xeaa, 0xe94, 0xeb2, 0xeab, 0xecc, 0xe81, 0xeb2, 0xe94, 0xeb5, 0xea5, 0xeb5,
0xe94, 0xeb9, 0xec4, 0xe9a, 0xe94, 0xeb9, 0xe8a, 0xeb2, 0xe99, 0xec0, 0xe9a, 0xe9f,
0xeb2, 0xea1, 0xeb2, 0xe81, 0xeb1, 0xeaa, 0xe97, 0xeb2, 0xe81, 0xeb2, 0xe8a, 0xeb2,
0xec0, 0xeae, 0xe9a, 0xea3, 0xead, 0xe99, 0xec2, 0xeae, 0xe88, 0xeb4, 0xea1, 0xeb4,
0xe99, 0xeae, 0xec8, 0xead, 0xe87, 0xe81, 0xebb, 0xe87, 0xeae, 0xead, 0xe9a, 0xead,
0xeb5, 0xe84, 0xeb8, 0xeaa, 0xe84, 0xecc, 0xe88, 0xeb2, 0xe81, 0xeb2, 0xe95, 0xeb2,
0xe88, 0xeb2, 0xea2, 0xeb2, 0xe9b, 0xeb9, 0xea3, 0xeb0, 0xec0, 0xea2, 0xea3, 0xeb9,
0xe8a, 0xeb2, 0xec0, 0xea5, 0xea1, 0xe84, 0xeb2, 0xe9a, 0xeb9, 0xe84, 0xeb2, 0xea1,
0xe8a, 0xeb2, 0xe94, 0xe81, 0xeb2, 0xe81, 0xeb2, 0xea3, 0xeb2, 0xe88, 0xeb5, 0xe84,
0xeb1, 0xe94, 0xea1, 0xeb1, 0xe99, 0xe94, 0xeb9, 0xec1, 0xe84, 0xe99, 0xe94, 0xeb5,
0xe81, 0xeb2, 0xec2, 0xe81, 0xea5, 0xe81, 0xeb2, 0xe95, 0xeb2, 0xe84, 0xea3, 0xeb1,
0xeaa, 0xec2, 0xe99, 0xea2, 0xeb2, 0xeaa, 0xe84, 0xecc, 0xe81, 0xebb, 0xea7, 0xea5,
0xeb2, 0xea5, 0xeb3, 0xec0, 0xe9b, 0xeb5, 0xe81, 0xeb9, 0xe8a, 0xeb5, 0xe87, 0xe84,
0xeb9, 0xec0, 0xea7, 0xe94, 0xea1, 0xeb2, 0xec0, 0xe81, 0xebb, 0xeb2, 0xea1, 0xeb2,
0xe81, 0xeb2, 0xec1, 0xe94, 0xe99, 0xea1, 0xeb2, 0xe81, 0xeb2, 0xeaa, 0xe8a, 0xeb2,
0xea1, 0xeb2, 0xe99, 0xeb5, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xeaa, 0xe84, 0xeb1, 0xe94,
0xe99, 0xeb4, 0xec2, 0xe84, 0xec0, 0xe8a, 0xe8d, 0xec2, 0xe99, 0xec2, 0xea7, 0xe84,
0xeb8, 0xeaa, 0xec0, 0xe99, 0xeb1, 0xe94, 0xec2, 0xe99, 0xec2, 0xea7, 0xe8a, 0xeb5,
0xe9a, 0xeb5, 0xeaa, 0xe84, 0xecc, 0xead, 0xead, 0xea1, 0xeaa, 0xe84, 0xecc, 0xead,
0xecd, 0xea3, 0xecd, 0xe9e, 0xeb0, 0xe99, 0xebb, 0xea1, 0xec0, 0xe9b, 0xeb1, 0xe99,
0xe9e, 0xead, 0xe99, 0xec0, 0xe97, 0xe8d, 0xe99, 0xeb1, 0xe81, 0xe9b, 0xebd, 0xe87,
0xea2, 0xeb2, 0xe87, 0xe81, 0xeb2, 0xe95, 0xeb2, 0xea3, 0xecc, 0xe84, 0xead, 0xeaa,
0xe95, 0xeb2, 0xec0, 0xe99, 0xec4, 0xe84, 0xe8a, 0xeb5, 0xea5, 0xecd, 0xe94, 0xeb2,
0xea3, 0xeb5, 0xea2, 0xeb2, 0xe94, 0xe8a, 0xeb2, 0xe84, 0xeb2, 0xea5, 0xeb4, 0xe99,
0xe8a, 0xeb2, 0xea1, 0xeb2, 0xec1, 0xe84, 0xe99, 0xec2, 0xe8a, 0xea5, 0xecc, 0xe8a,
0xec8, 0xebd, 0xe87, 0xec4, 0xeae, 0xec9, 0xeaa, 0xeb4, 0xe87, 0xe81, 0xeb0, 0xec2,
0xe9b, 0xeaa, 0xea3, 0xeb4, 0xea5, 0xec0, 0xe99, 0xeb1, 0xe81, 0xec2, 0xe81, 0xea5,
0xeb5, 0xe94, 0xec4, 0xe97, 0xec0, 0xe9b, 0xe97, 0xeb2, 0xe94, 0xeaa, 0xecc, 0xec0,
0xe84, 0xe99, 0xe97, 0xeb4, 0xe9a, 0xeb5, 0xea5, 0xeb5, 0xe8a, 0xeb5, 0xec0, 0xe95,
0xea3, 0xeb2, 0xe99, 0xe97, 0xeb4, 0xea1, 0xe9e, 0xeb9, 0xec2, 0xe95, 0xe81, 0xebd,
0xea7, 0xe97, 0xead, 0xea1, 0xeaa, 0xe81, 0xecc, 0xead, 0xeb9, 0xea5, 0xeb2, 0xe99,
0xe9a, 0xeb2, 0xe95, 0xeb2, 0xea3, 0xecc, 0xead, 0xeb9, 0xea3, 0xeb8, 0xea1, 0xe8a,
0xeb5, 0xead, 0xeb8, 0xeaa, 0x20, 0xec0, 0xe99, 0xea3, 0xeb2, 0xea7, 0xebd, 0xe87,
0xe88, 0xeb1, 0xe99, 0xea7, 0xeb5, 0xea5, 0xeb2, 0xe94, 0xeb4, 0xec2, 0xea7, 0xeaa,
0xe95, 0xead, 0xe81, 0xea2, 0xeb2, 0xe84, 0xeb8, 0xe97, 0xe8a, 0xecc, 0xea2, 0xeb2,
0xe87, 0xe81, 0xead, 0xe99, 0xec0, 0xea2, 0xe84, 0xeb2, 0xec0, 0xe95, 0xeb5, 0xea3,
0xeb4, 0xe99, 0xec0, 0xe9a, 0xeb5, 0xe81, 0xec0, 0xe8d, 0xec0, 0xea3, 0xea7, 0xeb2,
0xe99, 0xead, 0xeb2, 0x200b, 0xec2, 0xe8a, 0x200b, 0xec0, 0xea3, 0x200b, 0xeaa, 0xec0,
0xe9a, 0xeb5, 0xea1, 0xeb4, 0xea7, 0xe94, 0xeb2, 0xe84, 0xeb2, 0xe99, 0xeb2, 0xea3,
0xeb5, 0xec0, 0xe84, 0xe9a, 0xec0, 0xea7, 0xeb5, 0xe94, 0xec1, 0xe9f, 0xec2, 0xea3,
0xea1, 0xeb2, 0xec0, 0xe94, 0xea3, 0xeb2, 0xec0, 0xea3, 0xe81, 0xea2, 0xeb2, 0xea7,
0xeb4, 0xe81, 0xec0, 0xe8a, 0xebb, 0xeb2, 0xe88, 0xecd, 0xec0, 0xe88, 0xe8d, 0xec0,
0xe8a, 0xe99, 0x20, 0xec0, 0xeae, 0xec0, 0xea5, 0xe99, 0xeb2, 0xeaa, 0xeb0, 0xec1,
0xe95, 0xe99, 0xec0, 0xea5, 0xec0, 0xead, 0xe94, 0xeb5, 0xec1, 0xea5, 0xe94, 0xe9a,
0xea3, 0xeb4, 0xeaa, 0xec0, 0xe9a, 0xe99, 0xec2, 0xe9a, 0xea3, 0xe81, 0xec0, 0xe84,
0xe99, 0x20, 0xeae, 0xeb4, 0xea7, 0xe94, 0xeb2, 0xea7, 0xeb4, 0xe99, 0xea2, 0xeb9,
0xe84, 0xea5, 0xeb2, 0xec2, 0xeae, 0xe9a, 0xeb2, 0xe94, 0xea5, 0xeb4, 0xe99, 0xe94,
0xeb5, 0xec1, 0xea1, 0xe99, 0xec2, 0xea5, 0xe94, 0xec2, 0xeae, 0xea7, 0xeb5, 0xec0,
0xea1, 0xea7, 0xec0, 0xe9a, 0xeb4, 0xe99, 0xec0, 0xe9e, 0xeb5, 0xe94, 0xe8a, 0xeb4,
0xe94, 0xe99, 0xeb5, 0xead, 0xeb2, 0xea1, 0xeaa, 0xec0, 0xe95, 0xeb5, 0xe94, 0xeb3,
0xead, 0xeb4, 0xe99, 0xe94, 0xecd, 0xea3, 0xeb2, 0xead, 0xeb2, 0xeaa, 0xe95, 0xea3,
0xeb2, 0xe84, 0xeb2, 0xe99, 0xec0, 0xead, 0xec0, 0xe97, 0xe99, 0xeaa, 0xecc, 0xec0,
0xe9a, 0xea7, 0xec0, 0xe81, 0xea3, 0xe94, 0xec0, 0xe9a, 0xeb5, 0xea5, 0xeb4, 0xe99,
0xe9a, 0xea3, 0xeb2, 0xe97, 0xeb4, 0xeaa, 0xea5, 0xeb2, 0xea7, 0xeb2, 0xe9a, 0xea3,
0xeb1, 0xeaa, 0xec0, 0xe8a, 0xea7, 0xe9a, 0xeb9, 0xe84, 0xeb2, 0xec0, 0xea3, 0xeaa,
0xe95, 0xecc, 0xe9a, 0xeb9, 0xe94, 0xeb2, 0xec0, 0xe9b, 0xeaa, 0xe95, 0xecc, 0xe9a,
0xeb1, 0xe94, 0xe8a, 0xeb4, 0xe99, 0xec0, 0xe81, 0xe99, 0xe84, 0xeb5, 0xe8a, 0xeb5,
0xec0, 0xe99, 0xebb, 0xeb2, 0xec2, 0xe84, 0xec0, 0xe9b, 0xe99, 0xec0, 0xeae, 0xec0,
0xe81, 0xe99, 0xe94, 0xeb1, 0xe9a, 0xe9a, 0xea5, 0xeb4, 0xe99, 0xe81, 0xeb4, 0xe9a,
0xea3, 0xead, 0xea5, 0xe97, 0xeb2, 0xec0, 0xe81, 0xeb5, 0xe99, 0xe8a, 0xeb5, 0xec0,
0xeae, 0xea7, 0xe8a, 0xeb4, 0xe99, 0xe81, 0xeb4, 0xec0, 0xe81, 0xeb2, 0xeb0, 0xec1,
0xea1, 0xe99, 0xead, 0xeb4, 0xeaa, 0xe95, 0xeb1, 0xe99, 0xe9a, 0xeb9, 0xea5, 0xecc,
0xec0, 0xe88, 0xeb5, 0xe8a, 0xeb5, 0xe84, 0xeb2, 0xea5, 0xeb4, 0xe99, 0xeb4, 0xe99,
0xe81, 0xea3, 0xeb2, 0xe94, 0xe81, 0xeb4, 0xec2, 0xea3, 0xe9a, 0xe82, 0xebd, 0xe9a,
0xea5, 0xeb4, 0xeaa, 0xe9a, 0xead, 0xe99, 0xea5, 0xeb9, 0xe9a, 0xea5, 0xeb5, 0xe8d,
0xeb2, 0xe99, 0xeb2, 0xea5, 0xead, 0xe99, 0xe94, 0xead, 0xe99, 0xea5, 0xeb1, 0xe81,
0xec0, 0xe8a, 0xeb1, 0xea1, 0xec0, 0xe9a, 0xeb5, 0xe81, 0xea1, 0xeb2, 0xe94, 0xea3,
0xeb4, 0xe94, 0xea1, 0xead, 0xea5, 0xe95, 0xeb2, 0xea1, 0xeb2, 0xea3, 0xeb5, 0xec1,
0xeae, 0xea1, 0xe99, 0xecc, 0xea1, 0xeb4, 0xe99, 0xeaa, 0xe81, 0xecc, 0xec2, 0xea1,
0xe99, 0xeb2, 0xec2, 0xe84, 0xea1, 0xead, 0xeaa, 0xec2, 0xe84, 0xead, 0xead, 0xeaa,
0xec2, 0xea5, 0xe9b, 0xeb2, 0xea3, 0xeb5, 0xeaa, 0xe9e, 0xead, 0xe94, 0xe81, 0xecd,
0xea3, 0xeb5, 0xe81, 0xeb2, 0xe9b, 0xea3, 0xeb2, 0xe81, 0xe8a, 0xeb2, 0xea1, 0xeb2,
0xea3, 0xeb2, 0xe8a, 0xeb2, 0xe99, 0xea1, 0xeb2, 0xea3, 0xeb5, 0xec2, 0xe99, 0xe8a,
0xeb2, 0xea3, 0xeb2, 0xec0, 0xea2, 0xec2, 0xea7, 0xe8a, 0xeb2, 0xea3, 0xeb2, 0xe97,
0xead, 0xe9a, 0xe8a, 0xeb5, 0xea1, 0xec0, 0xe9f, 0xec2, 0xea5, 0xec2, 0xe9b, 0xeaa,
0xeb0, 0xec2, 0xe81, 0xec0, 0xe9b, 0xe8d, 0xec2, 0xe8a, 0xec0, 0xe9f, 0xe8d, 0xeaa,
0xeb0, 0xe95, 0xead, 0xe81, 0xec2, 0xeae, 0xea1, 0xe97, 0xeb2, 0xea5, 0xea5, 0xeb4,
0xe99, 0xe99, 0xecc, 0xe97, 0xeb4, 0xea3, 0xeb2, 0xe99, 0xead, 0xeb9, 0xea5, 0xeb5,
0xead, 0xeb2, 0xe99, 0xead, 0xe9a, 0xeaa, 0xe84, 0xecc, 0xea7, 0xeb2, 0xe94, 0xeb2,
0xe8a, 0xea7, 0xeb2, 0xe95, 0xeb4, 0xe81, 0xeb1, 0xe99, 0xea7, 0xebd, 0xe99, 0xe99,
0xeb2, 0xea7, 0xeb4, 0xea5, 0xe99, 0xeb4, 0xead, 0xeb8, 0xeaa, 0xea7, 0xead, 0xea5,
0xec2, 0xe81, 0xe81, 0xea3, 0xeb2, 0xe94, 0xea7, 0xecd, 0xe8a, 0xecd, 0xe8a, 0xeb2,
0xec0, 0xe81, 0xea3, 0xe9a, 0xe8a, 0xeb9, 0xea3, 0xeb4, 0xe81, 0xead, 0xeb1, 0xe99,
0xe95, 0xeb2, 0xe99, 0xeb2, 0xe99, 0xeb2, 0xea3, 0xeb4, 0xec2, 0xea7, 0xe8a, 0xeb2,
0xec2, 0xe81, 0xeaa, 0xe84, 0xea3, 0xeb4, 0xe94, 0xeaa, 0xeb0, 0xea1, 0xeb2, 0xe94,
0xec2, 0xe84, 0xec2, 0xe84, 0xeaa, 0xec2, 0xe84, 0xec2, 0xea1, 0xec2, 0xea3, 0xec1,
0xe81, 0xec0, 0xe81, 0xeb5, 0xec0, 0xea5, 0xe99, 0xea1, 0xeb2, 0xec0, 0xeae, 0xea1,
0xeb1, 0xea5, 0xe94, 0xeb5, 0xe9f, 0xec0, 0xea1, 0xebb, 0xeb2, 0xea3, 0xeb4, 0xe97,
0xebd, 0xeaa, 0xea1, 0xeb2, 0xea2, 0xeb1, 0xead, 0xe94, 0xec0, 0xe95, 0xec0, 0xea3,
0xead, 0xeb9, 0xe99, 0xeb4, 0xea2, 0xebb, 0xe87, 0xec0, 0xead, 0xe9b, 0xeb5, 0xead,
0xeb2, 0xead, 0xeb1, 0xe81, 0xec1, 0xea5, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe95,
0xeb2, 0xea1, 0xec0, 0xe82, 0xe94, 0xe9a, 0xeb9, 0xe99, 0xe81, 0xebd, 0xe99, 0xea7,
0xeb4, 0xea7, 0xec1, 0xe8a, 0xec1, 0xe97, 0xea1, 0xe88, 0xeb1, 0xe81, 0xec0, 0xead,
0xeaa, 0xec0, 0xe95, 0xeb5, 0xec0, 0xead, 0xe9f, 0xeb2, 0xec0, 0xe95, 0xe9f, 0xeb2,
0xe81, 0xeb2, 0xea7, 0xec2, 0xe9f, 0xe9f, 0xeb9, 0xe88, 0xeb4, 0xe9f, 0xeb9, 0xe99,
0xeb0, 0xe9f, 0xeb9, 0xe95, 0xeb4, 0xe81, 0xeb2, 0xea5, 0xeb2, 0xe9b, 0xeb2, 0xe81,
0xead, 0xeaa, 0xec1, 0xe81, 0xea1, 0xec0, 0xe9a, 0xe8d, 0xe81, 0xebb, 0xea7, 0xe94,
0xeb1, 0xea5, 0xe84, 0xeb0, 0xec1, 0xe99, 0xea5, 0xe81, 0xea7, 0xea1, 0xec2, 0xeae,
0xec2, 0xe99, 0xea5, 0xeb9, 0xea5, 0xeb9, 0xec0, 0xead, 0xeb1, 0xe99, 0xec0, 0xe94,
0xeb5, 0xec0, 0xe9a, 0xeb5, 0xea5, 0xeb5, 0xe84, 0xeb5, 0xea3, 0xeb4, 0xe97, 0xeb5,
0xea1, 0xeb2, 0xe95, 0xeb5, 0xe84, 0xecd, 0xeaa, 0xec1, 0xea3, 0xe84, 0xea7, 0xeb2,
0xe88, 0xeb2, 0xec0, 0xea5, 0xe99, 0xea1, 0xeb2, 0xe88, 0xeb9, 0xec2, 0xea3, 0xea1,
0xeb2, 0xe84, 0xeb4, 0xea7, 0xe8a, 0xeb2, 0xeaa, 0xea1, 0xeb4, 0xe94, 0xec0, 0xea7,
0xe99, 0xeb2, 0xead, 0xeb9, 0xea3, 0xeb9, 0xe99, 0xeb5, 0xead, 0xeb9, 0xec0, 0xead,
0xe99, 0xecd, 0xe9f, 0xead, 0xea5, 0xecc, 0xe81, 0xe99, 0xeb9, 0xec0, 0xea1, 0xead,
0xeb2, 0xe9b, 0xeb2, 0xec2, 0xe81, 0xe9b, 0xeb2, 0xec2, 0xe81, 0xe9b, 0xeb2, 0xec0,
0xea5, 0xebb, 0xeb2, 0xe9e, 0xeb4, 0xe94, 0xec1, 0xe84, 0xe99, 0xe9b, 0xebb, 0xe99,
0xec0, 0xe9b, 0xeb5, 0xe8d, 0xe9e, 0xead, 0xe94, 0x20, 0xea1, 0xecd, 0xec0, 0xea3,
0xeaa, 0xe9a, 0xeb5, 0xea3, 0xeb2, 0xec2, 0xea3, 0xe95, 0xead, 0xe87, 0xe81, 0xeb2,
0xec4, 0xe8a, 0xe9b, 0xeb2, 0xe99, 0xe97, 0xeb2, 0xeae, 0xeb5, 0xe95, 0xeb4, 0xe95,
0xeb2, 0xea3, 0xeb2, 0xea7, 0xeb2, 0xe95, 0xead, 0xe87, 0xe81, 0xeb2, 0xe95, 0xeb2,
0xe9b, 0xeb9, 0xec0, 0xea7, 0xe81, 0xea7, 0xeb2, 0xea5, 0xeb4, 0xe94, 0x41, 0x62,
0x69, 0x64, 0x17e, 0x61, 0x6e, 0x61, 0x41, 0x64, 0x69, 0x73, 0x61, 0x62,
0x65, 0x62, 0x61, 0x41, 0x6c, 0x17e, 0x12b, 0x72, 0x61, 0x42, 0x61, 0x6e,
0x64, 0x17e, 0x75, 0x6c, 0x61, 0x42, 0x69, 0x73, 0x61, 0x76, 0x61, 0x42,
0x6c, 0x61, 0x6e, 0x74, 0x61, 0x69, 0x72, 0x61, 0x42, 0x72, 0x61, 0x7a,
0x61, 0x76, 0x69, 0x6c, 0x61, 0x42, 0x75, 0x17e, 0x75, 0x6d, 0x62, 0x75,
0x72, 0x61, 0x4b, 0x61, 0x69, 0x72, 0x61, 0x44, 0x61, 0x6b, 0x61, 0x72,
0x61, 0x44, 0x101, 0x72, 0x65, 0x73, 0x61, 0x6c, 0x101, 0x6d, 0x61, 0x44,
0x17e, 0x69, 0x62, 0x75, 0x74, 0x69, 0x6a, 0x61, 0x41, 0x6a, 0x16b, 0x6e,
0x61, 0x46, 0x72, 0x12b, 0x74, 0x61, 0x75, 0x6e, 0x61, 0x4a, 0x6f, 0x68,
0x61, 0x6e, 0x6e, 0x65, 0x73, 0x62, 0x75, 0x72, 0x67, 0x61, 0x44, 0x17e,
0x16b, 0x62, 0x61, 0x48, 0x61, 0x72, 0x74, 0x16b, 0x6d, 0x61, 0x4b, 0x69,
0x6e, 0x161, 0x61, 0x73, 0x61, 0x4c, 0x61, 0x67, 0x6f, 0x73, 0x61, 0x4c,
0x69, 0x62, 0x72, 0x65, 0x76, 0x69, 0x6c, 0x61, 0x4c, 0x75, 0x62, 0x75,
0x6d, 0x62, 0x61, 0x161, 0x69, 0x4d, 0x61, 0x70, 0x75, 0x74, 0x75, 0x4d,
0x6f, 0x67, 0x61, 0x64, 0x12b, 0x161, 0x6f, 0x4d, 0x6f, 0x6e, 0x72, 0x6f,
0x76, 0x69, 0x6a, 0x61, 0x4e, 0x69, 0x61, 0x6d, 0x65, 0x6a, 0x61, 0x4e,
0x75, 0x61, 0x6b, 0x161, 0x6f, 0x74, 0x61, 0x54, 0x72, 0x69, 0x70, 0x6f,
0x6c, 0x65, 0x54, 0x75, 0x6e, 0x69, 0x73, 0x61, 0x56, 0x69, 0x6e, 0x64,
0x68, 0x75, 0x6b, 0x61, 0x41, 0x64, 0x61, 0x6b, 0x61, 0x41, 0x6e, 0x6b,
0x75, 0x72, 0x101, 0x17e, 0x61, 0x41, 0x6e, 0x67, 0x69, 0x6c, 0x6a, 0x61,
0x41, 0x72, 0x61, 0x67, 0x76, 0x61, 0x69, 0x6e, 0x61, 0x42, 0x75, 0x65,
0x6e, 0x6f, 0x73, 0x61, 0x69, 0x72, 0x65, 0x73, 0x61, 0x4b, 0x6f, 0x72,
0x64, 0x6f, 0x76, 0x61, 0x48, 0x75, 0x68, 0x75, 0x6a, 0x61, 0x4c, 0x61,
0x72, 0x6a, 0x6f, 0x68, 0x61, 0x52, 0x69, 0x6f, 0x67, 0x61, 0x6c, 0x6a,
0x65, 0x67, 0x6f, 0x73, 0x61, 0x53, 0x61, 0x6e, 0x68, 0x75, 0x61, 0x6e,
0x61, 0x53, 0x61, 0x6e, 0x6c, 0x75, 0x69, 0x73, 0x61, 0x54, 0x75, 0x6b,
0x75, 0x6d, 0x61, 0x6e, 0x61, 0x55, 0x161, 0x75, 0x61, 0x6a, 0x61, 0x41,
0x73, 0x75, 0x6e, 0x73, 0x6a, 0x6f, 0x6e, 0x61, 0x41, 0x74, 0x69, 0x6b,
0x6f, 0x6b, 0x61, 0x6e, 0x61, 0x42, 0x61, 0x69, 0x6a, 0x61, 0x42, 0x61,
0x6a, 0x61, 0x62, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x61, 0x73, 0x61, 0x42,
0x61, 0x72, 0x62, 0x61, 0x64, 0x6f, 0x73, 0x61, 0x42, 0x65, 0x6c, 0x65,
0x6e, 0x61, 0x42, 0x65, 0x6c, 0x69, 0x7a, 0x61, 0x42, 0x6c, 0x61, 0x6e,
0x73, 0x61, 0x62, 0x6c, 0x6f, 0x6e, 0x61, 0x42, 0x6f, 0x61, 0x76, 0x69,
0x73, 0x74, 0x61, 0x42, 0x6f, 0x69, 0x73, 0x69, 0x73, 0x69, 0x74, 0x69,
0x6a, 0x61, 0x4b, 0x65, 0x6d, 0x62, 0x72, 0x69, 0x64, 0x17e, 0x62, 0x65,
0x6a, 0x61, 0x4b, 0x61, 0x6d, 0x70, 0x75, 0x67, 0x72, 0x61, 0x6e, 0x64,
0x69, 0x4b, 0x61, 0x6e, 0x6b, 0x75, 0x6e, 0x61, 0x4b, 0x61, 0x72, 0x61,
0x6b, 0x61, 0x73, 0x61, 0x4b, 0x61, 0x6a, 0x65, 0x6e, 0x6e, 0x61, 0x4b,
0x61, 0x69, 0x6d, 0x61, 0x6e, 0x75, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x73,
0x10c, 0x69, 0x6b, 0x101, 0x67, 0x61, 0x10c, 0x69, 0x76, 0x61, 0x76, 0x61,
0x48, 0x75, 0x61, 0x72, 0x65, 0x73, 0x61, 0x4b, 0x72, 0x65, 0x73, 0x74,
0x6f, 0x6e, 0x61, 0x4b, 0x69, 0x72, 0x61, 0x73, 0x61, 0x6f, 0x44, 0x65,
0x6e, 0x6d, 0x101, 0x72, 0x6b, 0x161, 0x61, 0x76, 0x6e, 0x61, 0x44, 0x6f,
0x75, 0x73, 0x6f, 0x6e, 0x61, 0x44, 0x6f, 0x75, 0x73, 0x6f, 0x6e, 0x6b,
0x72, 0x12b, 0x6b, 0x61, 0x44, 0x65, 0x6e, 0x76, 0x65, 0x72, 0x61, 0x44,
0x65, 0x74, 0x72, 0x6f, 0x69, 0x74, 0x61, 0x45, 0x64, 0x6d, 0x6f, 0x6e,
0x74, 0x6f, 0x6e, 0x61, 0x53, 0x61, 0x6c, 0x76, 0x61, 0x64, 0x6f, 0x72,
0x61, 0x46, 0x6f, 0x72, 0x74, 0x6e, 0x65, 0x6c, 0x73, 0x6f, 0x6e, 0x61,
0x47, 0x6c, 0x65, 0x69, 0x73, 0x62, 0x65, 0x6a, 0x61, 0x47, 0x16b, 0x73,
0x62, 0x65, 0x6a, 0x61, 0x47, 0x72, 0x61, 0x6e, 0x64, 0x74, 0x6b, 0x113,
0x72, 0x6b, 0x61, 0x47, 0x72, 0x65, 0x6e, 0x101, 0x64, 0x61, 0x47, 0x76,
0x61, 0x64, 0x65, 0x6c, 0x75, 0x70, 0x61, 0x47, 0x76, 0x61, 0x6a, 0x61,
0x6b, 0x69, 0x6c, 0x61, 0x47, 0x61, 0x6a, 0x101, 0x6e, 0x61, 0x48, 0x65,
0x6c, 0x69, 0x66, 0x65, 0x6b, 0x73, 0x61, 0x45, 0x72, 0x6d, 0x6f, 0x73,
0x69, 0x6c, 0x6a, 0x6f, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x70,
0x6f, 0x6c, 0x69, 0x73, 0x61, 0x4e, 0x6f, 0x6b, 0x73, 0x61, 0x2c, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x101, 0x6e, 0x61, 0x4d, 0x61, 0x72, 0x65, 0x6e,
0x67, 0x6f, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x101, 0x6e, 0x61, 0x50,
0x12b, 0x74, 0x65, 0x72, 0x73, 0x62, 0x113, 0x72, 0x67, 0x61, 0x2c, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x101, 0x6e, 0x61, 0x54, 0x65, 0x6c, 0x73, 0x69,
0x74, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x101, 0x6e,
0x61, 0x56, 0x12b, 0x76, 0x65, 0x6a, 0x61, 0x2c, 0x20, 0x49, 0x6e, 0x64,
0x69, 0x101, 0x6e, 0x61, 0x56, 0x69, 0x6e, 0x73, 0x65, 0x6e, 0x73, 0x61,
0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x101, 0x6e, 0x61, 0x56, 0x69, 0x6e,
0x61, 0x6d, 0x61, 0x6b, 0x61, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x101,
0x6e, 0x61, 0x49, 0x6e, 0x75, 0x76, 0x69, 0x6b, 0x61, 0x49, 0x6b, 0x61,
0x6c, 0x75, 0x69, 0x74, 0x61, 0x44, 0x17e, 0x75, 0x6e, 0x6f, 0x4c, 0x16b,
0x69, 0x76, 0x69, 0x6c, 0x61, 0x4d, 0x6f, 0x6e, 0x74, 0x69, 0x10d, 0x65,
0x6c, 0x6f, 0x2c, 0x20, 0x4b, 0x65, 0x6e, 0x74, 0x75, 0x6b, 0x69, 0x4b,
0x72, 0x101, 0x6c, 0x65, 0x6e, 0x64, 0x65, 0x69, 0x6b, 0x61, 0x4c, 0x61,
0x70, 0x61, 0x73, 0x61, 0x4c, 0x6f, 0x73, 0x61, 0x6e, 0x64, 0x17e, 0x65,
0x6c, 0x6f, 0x73, 0x61, 0x4c, 0x6f, 0x75, 0x65, 0x72, 0x70, 0x72, 0x69,
0x6e, 0x73, 0x65, 0x73, 0x6b, 0x76, 0x6f, 0x74, 0x65, 0x72, 0x61, 0x4d,
0x61, 0x73, 0x65, 0x6a, 0x6f, 0x4d, 0x61, 0x6e, 0x61, 0x67, 0x76, 0x61,
0x4d, 0x61, 0x6e, 0x61, 0x75, 0x73, 0x61, 0x4d, 0x65, 0x72, 0x69, 0x67,
0x6f, 0x74, 0x61, 0x4d, 0x61, 0x74, 0x61, 0x6d, 0x6f, 0x72, 0x6f, 0x73,
0x61, 0x4d, 0x61, 0x73, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x61, 0x4d, 0x65,
0x6e, 0x6f, 0x6d, 0x69, 0x6e, 0x12b, 0x4d, 0x65, 0x68, 0x69, 0x6b, 0x6f,
0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x61, 0x4d, 0x6f, 0x6e, 0x6b,
0x74, 0x6f, 0x6e, 0x61, 0x4d, 0x6f, 0x6e, 0x74, 0x65, 0x72, 0x72, 0x65,
0x6a, 0x61, 0x4d, 0x6f, 0x6e, 0x74, 0x73, 0x65, 0x72, 0x72, 0x61, 0x74,
0x61, 0x4e, 0x61, 0x73, 0x6f, 0x145, 0x75, 0x6a, 0x6f, 0x72, 0x6b, 0x61,
0x4e, 0x6f, 0x6d, 0x61, 0x4e, 0x6f, 0x72, 0x6f, 0x146, 0x61, 0x42, 0x6a,
0x75, 0x6c, 0x61, 0x2c, 0x20, 0x5a, 0x69, 0x65, 0x6d, 0x65, 0x13c, 0x64,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x53, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x61,
0x2c, 0x20, 0x5a, 0x69, 0x65, 0x6d, 0x65, 0x13c, 0x64, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x145, 0x16b, 0x73, 0x65, 0x69, 0x6c, 0x65, 0x6d, 0x61, 0x2c,
0x20, 0x5a, 0x69, 0x65, 0x6d, 0x65, 0x13c, 0x64, 0x61, 0x6b, 0x6f, 0x74,
0x61, 0x4e, 0x16b, 0x6b, 0x61, 0x46, 0x12b, 0x6e, 0x69, 0x6b, 0x73, 0x61,
0x50, 0x6f, 0x72, 0x74, 0x6f, 0x70, 0x72, 0x65, 0x6e, 0x73, 0x61, 0x50,
0x6f, 0x72, 0x74, 0x6f, 0x66, 0x73, 0x70, 0x65, 0x69, 0x6e, 0x61, 0x50,
0x6f, 0x72, 0x74, 0x75, 0x76, 0x65, 0x13c, 0x75, 0x50, 0x75, 0x6e, 0x74,
0x61, 0x61, 0x72, 0x65, 0x6e, 0x61, 0x73, 0x61, 0x52, 0x61, 0x6e, 0x6b,
0x69, 0x6e, 0x69, 0x6e, 0x6c, 0x65, 0x74, 0x61, 0x52, 0x65, 0x73, 0x69,
0x66, 0x69, 0x52, 0x69, 0x64, 0x17e, 0x61, 0x69, 0x6e, 0x61, 0x52, 0x65,
0x7a, 0x6f, 0x6c, 0x16b, 0x74, 0x61, 0x52, 0x69, 0x6f, 0x62, 0x72, 0x61,
0x6e, 0x6b, 0x6f, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x65, 0x6e, 0x61,
0x53, 0x61, 0x6e, 0x74, 0x6a, 0x61, 0x67, 0x6f, 0x53, 0x61, 0x6e, 0x74,
0x6f, 0x64, 0x6f, 0x6d, 0x69, 0x6e, 0x67, 0x6f, 0x53, 0x61, 0x6e, 0x70,
0x61, 0x75, 0x6c, 0x75, 0x49, 0x74, 0x6f, 0x6b, 0x6f, 0x72, 0x74, 0x6f,
0x72, 0x6d, 0x69, 0x74, 0x61, 0x53, 0x65, 0x6e, 0x62, 0x61, 0x72, 0x74,
0x65, 0x6c, 0x6d, 0x12b, 0x53, 0x65, 0x6e, 0x74, 0x64, 0x17e, 0x6f, 0x6e,
0x73, 0x61, 0x53, 0x65, 0x6e, 0x74, 0x6b, 0x69, 0x74, 0x73, 0x61, 0x53,
0x65, 0x6e, 0x74, 0x6c, 0x16b, 0x73, 0x69, 0x6a, 0x61, 0x53, 0x65, 0x6e,
0x74, 0x6f, 0x6d, 0x61, 0x73, 0x61, 0x53, 0x65, 0x6e, 0x74, 0x76, 0x69,
0x6e, 0x73, 0x65, 0x6e, 0x74, 0x61, 0x53, 0x76, 0x69, 0x66, 0x74, 0x6b,
0x61, 0x72, 0x65, 0x6e, 0x74, 0x61, 0x54, 0x75, 0x6c, 0x65, 0x56, 0x61,
0x6e, 0x6b, 0x16b, 0x76, 0x65, 0x72, 0x61, 0x56, 0x61, 0x69, 0x74, 0x68,
0x6f, 0x72, 0x73, 0x61, 0x56, 0x69, 0x6e, 0x69, 0x70, 0x65, 0x67, 0x61,
0x4a, 0x61, 0x6b, 0x75, 0x74, 0x61, 0x74, 0x61, 0x4b, 0x65, 0x69, 0x73,
0x69, 0x44, 0x65, 0x69, 0x76, 0x69, 0x73, 0x61, 0x44, 0x69, 0x6d, 0x6f,
0x6e, 0x64, 0x69, 0x72, 0x76, 0x69, 0x6c, 0x61, 0x4d, 0x61, 0x6b, 0x76,
0x6f, 0x72, 0x69, 0x4d, 0x6f, 0x73, 0x6f, 0x6e, 0x61, 0x4d, 0x61, 0x6b,
0x6d, 0x65, 0x72, 0x64, 0x6f, 0x50, 0x101, 0x6c, 0x6d, 0x65, 0x72, 0x61,
0x160, 0x6f, 0x76, 0x61, 0x54, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x56, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x61, 0x4c, 0x6f, 0x6e, 0x67, 0x6a, 0x113, 0x72,
0x62, 0x12b, 0x65, 0x6e, 0x65, 0x41, 0x64, 0x65, 0x6e, 0x61, 0x41, 0x6d,
0x6d, 0x101, 0x6e, 0x61, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x61, 0x41,
0x6b, 0x74, 0x65, 0x62, 0x65, 0x41, 0x161, 0x67, 0x61, 0x62, 0x61, 0x74,
0x61, 0x42, 0x61, 0x67, 0x64, 0x101, 0x64, 0x65, 0x42, 0x61, 0x68, 0x72,
0x65, 0x69, 0x6e, 0x61, 0x42, 0x61, 0x6e, 0x67, 0x6b, 0x6f, 0x6b, 0x61,
0x42, 0x61, 0x72, 0x6e, 0x61, 0x75, 0x6c, 0x61, 0x42, 0x65, 0x69, 0x72,
0x16b, 0x74, 0x61, 0x42, 0x69, 0x161, 0x6b, 0x65, 0x6b, 0x61, 0x42, 0x72,
0x75, 0x6e, 0x65, 0x6a, 0x61, 0x44, 0x61, 0x6d, 0x61, 0x73, 0x6b, 0x61,
0x44, 0x75, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x48, 0x65, 0x62, 0x72, 0x6f,
0x6e, 0x61, 0x48, 0x6f, 0x161, 0x69, 0x6d, 0x69, 0x6e, 0x61, 0x48, 0x6f,
0x6e, 0x6b, 0x6f, 0x6e, 0x67, 0x61, 0x48, 0x6f, 0x76, 0x64, 0x61, 0x49,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x61, 0x4a, 0x65, 0x72, 0x75, 0x7a,
0x61, 0x6c, 0x65, 0x6d, 0x65, 0x4b, 0x61, 0x62, 0x75, 0x6c, 0x61, 0x4b,
0x61, 0x72, 0x101, 0x10d, 0x69, 0x4b, 0x61, 0x6c, 0x6b, 0x101, 0x74, 0x61,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x61,
0x4b, 0x75, 0x61, 0x6c, 0x61, 0x6c, 0x75, 0x6d, 0x70, 0x75, 0x72, 0x61,
0x4b, 0x75, 0x10d, 0x69, 0x6e, 0x61, 0x4b, 0x75, 0x76, 0x65, 0x69, 0x74,
0x61, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x61, 0x4d, 0x61, 0x6b,
0x61, 0x73, 0x61, 0x72, 0x61, 0x4d, 0x61, 0x73, 0x6b, 0x61, 0x74, 0x61,
0x4e, 0x6f, 0x76, 0x6f, 0x6b, 0x75, 0x7a, 0x146, 0x65, 0x63, 0x6b, 0x61,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x61,
0x4f, 0x6d, 0x73, 0x6b, 0x61, 0x4f, 0x72, 0x61, 0x6c, 0x61, 0x50, 0x6e,
0x6f, 0x6d, 0x70, 0x65, 0x146, 0x61, 0x50, 0x6f, 0x6e, 0x74, 0x69, 0x61,
0x6e, 0x61, 0x6b, 0x61, 0x50, 0x68, 0x65, 0x6e, 0x6a, 0x61, 0x6e, 0x61,
0x4b, 0x61, 0x74, 0x61, 0x72, 0x61, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x6e,
0x61, 0x6a, 0x61, 0x52, 0x69, 0x6a, 0x101, 0x64, 0x61, 0x53, 0x61, 0x68,
0x61, 0x6c, 0x12b, 0x6e, 0x61, 0x53, 0x65, 0x75, 0x6c, 0x61, 0x160, 0x61,
0x6e, 0x68, 0x61, 0x6a, 0x61, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x16b,
0x72, 0x61, 0x53, 0x72, 0x65, 0x64, 0x146, 0x65, 0x6b, 0x6f, 0x6c, 0x69,
0x6d, 0x73, 0x6b, 0x61, 0x54, 0x61, 0x69, 0x62, 0x65, 0x69, 0x54, 0x61,
0x161, 0x6b, 0x65, 0x6e, 0x74, 0x61, 0x54, 0x65, 0x68, 0x65, 0x72, 0x101,
0x6e, 0x61, 0x54, 0x6f, 0x6b, 0x69, 0x6a, 0x61, 0x54, 0x6f, 0x6d, 0x73,
0x6b, 0x61, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x61,
0x55, 0x73, 0x74, 0x6a, 0x146, 0x65, 0x72, 0x61, 0x56, 0x6a, 0x65, 0x6e,
0x10d, 0x61, 0x6e, 0x61, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x61, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x61,
0x52, 0x61, 0x6e, 0x67, 0x75, 0x6e, 0x61, 0x4a, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x61, 0x45, 0x72, 0x65,
0x76, 0x101, 0x6e, 0x61, 0x41, 0x7a, 0x6f, 0x72, 0x75, 0x20, 0x73, 0x61,
0x6c, 0x61, 0x73, 0x4b, 0x61, 0x6e, 0x101, 0x72, 0x69, 0x6a, 0x75, 0x20,
0x73, 0x61, 0x6c, 0x61, 0x73, 0x4b, 0x61, 0x62, 0x6f, 0x76, 0x65, 0x72,
0x64, 0x65, 0x46, 0x113, 0x72, 0x75, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x73,
0x52, 0x65, 0x69, 0x6b, 0x6a, 0x61, 0x76, 0x69, 0x6b, 0x61, 0x44, 0x69,
0x65, 0x6e, 0x76, 0x69, 0x64, 0x64, 0x17e, 0x6f, 0x72, 0x64, 0x17e, 0x69,
0x6a, 0x61, 0x53, 0x76, 0x2e, 0x48, 0x65, 0x6c, 0x113, 0x6e, 0x61, 0x73,
0x20, 0x73, 0x61, 0x6c, 0x61, 0x42, 0x72, 0x69, 0x73, 0x62, 0x65, 0x6e,
0x61, 0x42, 0x72, 0x6f, 0x6b, 0x65, 0x6e, 0x68, 0x69, 0x6c, 0x61, 0x44,
0x101, 0x72, 0x76, 0x69, 0x6e, 0x61, 0x4a, 0x75, 0x6b, 0x6c, 0x61, 0x48,
0x6f, 0x62, 0x101, 0x72, 0x74, 0x61, 0x4c, 0x69, 0x6e, 0x64, 0x65, 0x6d,
0x61, 0x6e, 0x61, 0x4c, 0x6f, 0x72, 0x64, 0x68, 0x61, 0x76, 0x61, 0x4d,
0x65, 0x6c, 0x62, 0x75, 0x72, 0x6e, 0x61, 0x50, 0x113, 0x72, 0x74, 0x61,
0x53, 0x69, 0x64, 0x6e, 0x65, 0x6a, 0x61, 0x41, 0x6d, 0x73, 0x74, 0x65,
0x72, 0x64, 0x61, 0x6d, 0x61, 0x41, 0x73, 0x74, 0x72, 0x61, 0x68, 0x61,
0x146, 0x61, 0x41, 0x74, 0x113, 0x6e, 0x61, 0x73, 0x42, 0x65, 0x6c, 0x67,
0x72, 0x61, 0x64, 0x61, 0x42, 0x65, 0x72, 0x6c, 0x12b, 0x6e, 0x65, 0x42,
0x72, 0x69, 0x73, 0x65, 0x6c, 0x65, 0x42, 0x75, 0x6b, 0x61, 0x72, 0x65,
0x73, 0x74, 0x65, 0x42, 0x75, 0x64, 0x61, 0x70, 0x65, 0x161, 0x74, 0x61,
0x42, 0x12b, 0x7a, 0x69, 0x6e, 0x67, 0x65, 0x6e, 0x65, 0x4b, 0x69, 0x161,
0x69, 0x146, 0x65, 0x76, 0x61, 0x4b, 0x6f, 0x70, 0x65, 0x6e, 0x68, 0x101,
0x67, 0x65, 0x6e, 0x61, 0x44, 0x75, 0x62, 0x6c, 0x69, 0x6e, 0x61, 0x47,
0x69, 0x62, 0x72, 0x61, 0x6c, 0x74, 0x101, 0x72, 0x73, 0x47, 0x113, 0x72,
0x6e, 0x73, 0x69, 0x6a, 0x61, 0x4d, 0x65, 0x6e, 0x61, 0x73, 0x20, 0x73,
0x61, 0x6c, 0x61, 0x53, 0x74, 0x61, 0x6d, 0x62, 0x75, 0x6c, 0x61, 0x44,
0x17e, 0x113, 0x72, 0x73, 0x69, 0x6a, 0x61, 0x4b, 0x61, 0x13c, 0x69, 0x146,
0x69, 0x6e, 0x67, 0x72, 0x61, 0x64, 0x61, 0x4b, 0x69, 0x72, 0x6f, 0x76,
0x61, 0x4b, 0x69, 0x6a, 0x65, 0x76, 0x61, 0x4c, 0x69, 0x73, 0x61, 0x62,
0x6f, 0x6e, 0x61, 0x13b, 0x75, 0x62, 0x13c, 0x61, 0x6e, 0x61, 0x4c, 0x6f,
0x6e, 0x64, 0x6f, 0x6e, 0x61, 0x4c, 0x75, 0x6b, 0x73, 0x65, 0x6d, 0x62,
0x75, 0x72, 0x67, 0x61, 0x4d, 0x61, 0x64, 0x72, 0x69, 0x64, 0x65, 0x4d,
0x61, 0x72, 0x69, 0x65, 0x68, 0x61, 0x6d, 0x6e, 0x61, 0x4d, 0x69, 0x6e,
0x73, 0x6b, 0x61, 0x4d, 0x61, 0x73, 0x6b, 0x61, 0x76, 0x61, 0x50, 0x61,
0x72, 0x12b, 0x7a, 0x65, 0x50, 0x72, 0x101, 0x67, 0x61, 0x53, 0x61, 0x6e,
0x6d, 0x61, 0x72, 0x12b, 0x6e, 0x6f, 0x53, 0x61, 0x72, 0x61, 0x6a, 0x65,
0x76, 0x61, 0x53, 0x61, 0x72, 0x61, 0x74, 0x6f, 0x76, 0x61, 0x53, 0x69,
0x6d, 0x66, 0x65, 0x72, 0x6f, 0x70, 0x6f, 0x6c, 0x65, 0x53, 0x74, 0x6f,
0x6b, 0x68, 0x6f, 0x6c, 0x6d, 0x61, 0x54, 0x61, 0x6c, 0x6c, 0x69, 0x6e,
0x61, 0x54, 0x69, 0x72, 0x101, 0x6e, 0x61, 0x55, 0x13c, 0x6a, 0x61, 0x6e,
0x6f, 0x76, 0x73, 0x6b, 0x61, 0x56, 0x61, 0x64, 0x75, 0x63, 0x61, 0x56,
0x61, 0x74, 0x69, 0x6b, 0x101, 0x6e, 0x73, 0x56, 0x12b, 0x6e, 0x65, 0x56,
0x69, 0x13c, 0x146, 0x61, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x61, 0x5a, 0x61, 0x67, 0x72, 0x65, 0x62, 0x61, 0x43, 0x12b, 0x72,
0x69, 0x68, 0x65, 0x10c, 0x61, 0x67, 0x6f, 0x73, 0x61, 0x5a, 0x69, 0x65,
0x6d, 0x73, 0x76, 0x113, 0x74, 0x6b, 0x75, 0x20, 0x73, 0x61, 0x6c, 0x61,
0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x75, 0x20, 0x28, 0x4b, 0x12b, 0x6c, 0x69,
0x6e, 0x67, 0x61, 0x29, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x4b, 0x6f, 0x6d,
0x6f, 0x72, 0x61, 0x73, 0x4b, 0x65, 0x72, 0x67, 0x65, 0x6c, 0x113, 0x6e,
0x61, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x4d, 0x61, 0x65, 0x4d, 0x61, 0x6c,
0x64, 0x12b, 0x76, 0x69, 0x6a, 0x61, 0x4d, 0x61, 0x75, 0x72, 0x12b, 0x63,
0x69, 0x6a, 0x61, 0x4d, 0x61, 0x6a, 0x6f, 0x74, 0x61, 0x52, 0x65, 0x69,
0x6e, 0x6a, 0x6f, 0x6e, 0x61, 0x41, 0x70, 0x69, 0x6a, 0x61, 0x4f, 0x6b,
0x6c, 0x65, 0x6e, 0x64, 0x61, 0x42, 0x75, 0x67, 0x65, 0x6e, 0x76, 0x69,
0x6c, 0x61, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x10c, 0x65, 0x74, 0x65, 0x6d,
0x61, 0x10c, 0x16b, 0x6b, 0x61, 0x4c, 0x69, 0x65, 0x6c, 0x64, 0x69, 0x65,
0x6e, 0x75, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x47, 0x61, 0x6c, 0x61, 0x70,
0x61, 0x67, 0x75, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x73, 0x47, 0x61, 0x6d,
0x62, 0x6a, 0x113, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x73, 0x47, 0x76, 0x61,
0x64, 0x61, 0x6c, 0x6b, 0x61, 0x6e, 0x61, 0x6c, 0x61, 0x47, 0x75, 0x61,
0x6d, 0x61, 0x45, 0x6e, 0x64, 0x65, 0x72, 0x62, 0x65, 0x72, 0x69, 0x6a,
0x61, 0x4b, 0x69, 0x72, 0x69, 0x73, 0x69, 0x6d, 0x61, 0x73, 0x69, 0x4b,
0x6f, 0x73, 0x72, 0x61, 0x6a, 0x61, 0x4b, 0x76, 0x61, 0x64, 0x17e, 0x61,
0x6c, 0x65, 0x69, 0x6e, 0x61, 0x4d, 0x61, 0x64, 0x17e, 0x75, 0x72, 0x6f,
0x4d, 0x61, 0x72, 0x137, 0x12b, 0x7a, 0x61, 0x20, 0x73, 0x61, 0x6c, 0x61,
0x73, 0x4d, 0x69, 0x64, 0x76, 0x65, 0x6a, 0x73, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x61, 0x4e, 0x75, 0x6d, 0x65, 0x61, 0x50, 0x69, 0x74,
0x6b, 0x113, 0x72, 0x6e, 0x61, 0x50, 0x6f, 0x6e, 0x70, 0x65, 0x6a, 0x61,
0x50, 0x6f, 0x72, 0x74, 0x6d, 0x6f, 0x72, 0x73, 0x62, 0x69, 0x53, 0x61,
0x69, 0x70, 0x61, 0x6e, 0x61, 0x54, 0x61, 0x69, 0x74, 0x69, 0x56, 0x65,
0x69, 0x6b, 0x61, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x56, 0x6f, 0x6c, 0x69,
0x73, 0x61, 0x41, 0x62, 0x69, 0x64, 0x17e, 0x61, 0x6e, 0x61, 0x73, 0x41,
0x6c, 0x17e, 0x79, 0x72, 0x61, 0x73, 0x42, 0x61, 0x6d, 0x61, 0x6b, 0x61,
0x73, 0x42, 0x61, 0x6e, 0x67, 0x69, 0x73, 0x42, 0x61, 0x6e, 0x64, 0x17e,
0x75, 0x6c, 0x69, 0x73, 0x42, 0x6c, 0x61, 0x6e, 0x74, 0x61, 0x69, 0x72,
0x61, 0x73, 0x42, 0x72, 0x61, 0x7a, 0x61, 0x76, 0x69, 0x6c, 0x69, 0x73,
0x42, 0x75, 0x17e, 0x75, 0x6d, 0x62, 0x16b, 0x72, 0x61, 0x4b, 0x61, 0x69,
0x72, 0x61, 0x73, 0x4b, 0x6f, 0x6e, 0x61, 0x6b, 0x72, 0x69, 0x73, 0x44,
0x61, 0x6b, 0x61, 0x72, 0x61, 0x73, 0x44, 0x61, 0x72, 0x20, 0x65, 0x73,
0x20, 0x53, 0x61, 0x6c, 0x61, 0x6d, 0x61, 0x73, 0x44, 0x17e, 0x69, 0x62,
0x75, 0x74, 0x69, 0x73, 0x61, 0x6c, 0x20, 0x41, 0x6a, 0x16b, 0x6e, 0x61,
0x73, 0x46, 0x72, 0x69, 0x74, 0x61, 0x75, 0x6e, 0x61, 0x73, 0x47, 0x61,
0x62, 0x6f, 0x72, 0x6f, 0x6e, 0x61, 0x73, 0x48, 0x61, 0x72, 0x61, 0x72,
0x117, 0x4a, 0x6f, 0x68, 0x61, 0x6e, 0x65, 0x73, 0x62, 0x75, 0x72, 0x67,
0x61, 0x73, 0x44, 0x17e, 0x75, 0x62, 0x61, 0x43, 0x68, 0x61, 0x72, 0x74,
0x75, 0x6d, 0x61, 0x73, 0x4b, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x73, 0x4c,
0x61, 0x67, 0x6f, 0x73, 0x61, 0x73, 0x4c, 0x69, 0x62, 0x72, 0x65, 0x76,
0x69, 0x6c, 0x69, 0x73, 0x4c, 0x6f, 0x6d, 0x117, 0x4c, 0x75, 0x62, 0x75,
0x6d, 0x62, 0x61, 0x161, 0x69, 0x73, 0x4d, 0x61, 0x6c, 0x61, 0x62, 0x61,
0x73, 0x4d, 0x61, 0x70, 0x75, 0x74, 0x61, 0x73, 0x4d, 0x62, 0x61, 0x62,
0x61, 0x6e, 0x117, 0x4d, 0x6f, 0x67, 0x61, 0x64, 0x69, 0x161, 0x61, 0x73,
0x4e, 0x61, 0x69, 0x72, 0x6f, 0x62, 0x69, 0x73, 0x4e, 0x69, 0x61, 0x6d,
0x117, 0x6a, 0x75, 0x73, 0x4e, 0x75, 0x61, 0x6b, 0x161, 0x6f, 0x74, 0x61,
0x73, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x73,
0x53, 0x61, 0x6e, 0x20, 0x54, 0x6f, 0x6d, 0x117, 0x54, 0x75, 0x6e, 0x69,
0x73, 0x61, 0x73, 0x56, 0x69, 0x6e, 0x64, 0x68, 0x75, 0x6b, 0x61, 0x73,
0x45, 0x69, 0x64, 0x61, 0x6b, 0x61, 0x73, 0x41, 0x6e, 0x6b, 0x6f, 0x72,
0x69, 0x64, 0x17e, 0x61, 0x73, 0x41, 0x6e, 0x67, 0x69, 0x6c, 0x69, 0x6a,
0x61, 0x41, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x61, 0x6e, 0x61, 0x42,
0x75, 0x65, 0x6e, 0x6f, 0x73, 0x20, 0x41, 0x69, 0x72, 0x117, 0x73, 0x43,
0x68, 0x75, 0x63, 0x68, 0x75, 0x6a, 0x75, 0x73, 0x4c, 0x61, 0x20, 0x52,
0x69, 0x6f, 0x63, 0x68, 0x61, 0x52, 0x69, 0x6f, 0x20, 0x47, 0x61, 0x6c,
0x6a, 0x65, 0x67, 0x6f, 0x73, 0x61, 0x73, 0x53, 0x61, 0x6c, 0x74, 0x61,
0x73, 0x53, 0x61, 0x6e, 0x20, 0x43, 0x68, 0x75, 0x61, 0x6e, 0x61, 0x73,
0x53, 0x65, 0x6e, 0x74, 0x20, 0x4c, 0x75, 0x69, 0x73, 0x61, 0x73, 0x54,
0x75, 0x6b, 0x75, 0x6d, 0x61, 0x6e, 0x61, 0x73, 0x41, 0x73, 0x75, 0x6e,
0x73, 0x6a, 0x6f, 0x6e, 0x61, 0x73, 0x41, 0x74, 0x69, 0x6b, 0x6f, 0x6b,
0x61, 0x6e, 0x61, 0x73, 0x42, 0x61, 0x68, 0x69, 0x61, 0x20, 0x42, 0x61,
0x6e, 0x64, 0x65, 0x72, 0x61, 0x73, 0x61, 0x73, 0x42, 0x61, 0x72, 0x62,
0x61, 0x64, 0x6f, 0x73, 0x61, 0x73, 0x42, 0x65, 0x6c, 0x65, 0x6e, 0x61,
0x73, 0x42, 0x65, 0x6c, 0x69, 0x7a, 0x61, 0x73, 0x42, 0x6c, 0x61, 0x6e,
0x10d, 0x20, 0x53, 0x61, 0x62, 0x6c, 0x6f, 0x6e, 0x61, 0x73, 0x42, 0x75,
0x61, 0x20, 0x56, 0x69, 0x73, 0x74, 0x61, 0x4b, 0x65, 0x69, 0x6d, 0x62,
0x72, 0x69, 0x64, 0x17e, 0x20, 0x42, 0x117, 0x6a, 0x75, 0x73, 0x4b, 0x61,
0x6d, 0x70, 0x6f, 0x20, 0x47, 0x72, 0x61, 0x6e, 0x64, 0x117, 0x4b, 0x61,
0x6e, 0x6b, 0x16b, 0x6e, 0x61, 0x73, 0x4b, 0x61, 0x72, 0x61, 0x6b, 0x61,
0x73, 0x61, 0x73, 0x4b, 0x61, 0x6a, 0x65, 0x6e, 0x61, 0x73, 0x4b, 0x61,
0x69, 0x6d, 0x61, 0x6e, 0x61, 0x73, 0x10c, 0x69, 0x6b, 0x61, 0x67, 0x61,
0x10c, 0x69, 0x68, 0x75, 0x61, 0x68, 0x75, 0x61, 0x53, 0x69, 0x75, 0x64,
0x61, 0x64, 0x20, 0x43, 0x68, 0x75, 0x61, 0x72, 0x65, 0x73, 0x61, 0x73,
0x4b, 0x72, 0x65, 0x73, 0x74, 0x6f, 0x6e, 0x61, 0x73, 0x4b, 0x69, 0x75,
0x72, 0x61, 0x73, 0x61, 0x6f, 0x44, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x6b,
0x73, 0x68, 0x61, 0x75, 0x6e, 0x61, 0x73, 0x44, 0x6f, 0x73, 0x6f, 0x6e,
0x61, 0x73, 0x44, 0x6f, 0x73, 0x6f, 0x6e, 0x20, 0x4b, 0x72, 0x69, 0x6b,
0x61, 0x73, 0x44, 0x65, 0x6e, 0x76, 0x65, 0x72, 0x69, 0x73, 0x44, 0x65,
0x74, 0x72, 0x6f, 0x69, 0x74, 0x61, 0x73, 0x45, 0x64, 0x6d, 0x6f, 0x6e,
0x74, 0x6f, 0x6e, 0x61, 0x73, 0x45, 0x69, 0x72, 0x75, 0x6e, 0x65, 0x70,
0x117, 0x53, 0x61, 0x6c, 0x76, 0x61, 0x64, 0x6f, 0x72, 0x61, 0x73, 0x46,
0x6f, 0x72, 0x74, 0x20, 0x4e, 0x65, 0x6c, 0x73, 0x6f, 0x6e, 0x61, 0x73,
0x47, 0x6c, 0x65, 0x69, 0x73, 0x20, 0x42, 0x117, 0x6a, 0x75, 0x73, 0x47,
0x75, 0x73, 0x20, 0x42, 0x117, 0x6a, 0x75, 0x73, 0x47, 0x72, 0x61, 0x6e,
0x64, 0x20, 0x54, 0x65, 0x72, 0x6b, 0x61, 0x73, 0x47, 0x76, 0x61, 0x64,
0x61, 0x6c, 0x75, 0x70, 0x117, 0x47, 0x76, 0x61, 0x6a, 0x61, 0x6b, 0x69,
0x6c, 0x69, 0x73, 0x48, 0x61, 0x6c, 0x69, 0x66, 0x61, 0x6b, 0x73, 0x61,
0x73, 0x48, 0x65, 0x72, 0x6d, 0x6f, 0x73, 0x69, 0x6c, 0x6a, 0x61, 0x73,
0x4e, 0x6f, 0x6b, 0x73, 0x61, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69,
0x61, 0x6e, 0x61, 0x4d, 0x61, 0x72, 0x65, 0x6e, 0x67, 0x61, 0x73, 0x2c,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x50, 0x69, 0x74, 0x65,
0x72, 0x73, 0x62, 0x65, 0x72, 0x67, 0x61, 0x73, 0x2c, 0x20, 0x49, 0x6e,
0x64, 0x69, 0x61, 0x6e, 0x61, 0x54, 0x65, 0x6c, 0x20, 0x53, 0x69, 0x74,
0x69, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x56,
0x69, 0x76, 0x69, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e,
0x61, 0x56, 0x69, 0x6e, 0x73, 0x65, 0x6e, 0x61, 0x73, 0x2c, 0x20, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x56, 0x69, 0x6e, 0x61, 0x6d, 0x61,
0x6b, 0x61, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61,
0x49, 0x6e, 0x75, 0x76, 0x69, 0x6b, 0x61, 0x73, 0x49, 0x6b, 0x61, 0x6c,
0x75, 0x69, 0x74, 0x61, 0x73, 0x44, 0x17e, 0x75, 0x6e, 0x61, 0x73, 0x4c,
0x75, 0x69, 0x73, 0x76, 0x69, 0x6c, 0x69, 0x73, 0x4d, 0x6f, 0x6e, 0x74,
0x69, 0x73, 0x65, 0x6c, 0x61, 0x73, 0x2c, 0x20, 0x4b, 0x65, 0x6e, 0x74,
0x75, 0x6b, 0x69, 0x73, 0x4b, 0x72, 0x61, 0x6c, 0x65, 0x6e, 0x64, 0x65,
0x69, 0x6b, 0x61, 0x73, 0x4c, 0x61, 0x20, 0x50, 0x61, 0x73, 0x61, 0x73,
0x4c, 0x6f, 0x73, 0x20, 0x41, 0x6e, 0x64, 0x17e, 0x65, 0x6c, 0x61, 0x73,
0x17d, 0x65, 0x6d, 0x75, 0x74, 0x69, 0x6e, 0x69, 0x73, 0x20, 0x50, 0x72,
0x69, 0x6e, 0x73, 0x65, 0x73, 0x20, 0x4b, 0x76, 0x6f, 0x72, 0x74, 0x65,
0x72, 0x69, 0x73, 0x4d, 0x61, 0x6e, 0x61, 0x75, 0x73, 0x61, 0x73, 0x4d,
0x61, 0x74, 0x61, 0x6d, 0x6f, 0x72, 0x6f, 0x73, 0x61, 0x73, 0x4d, 0x61,
0x73, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x61, 0x73, 0x4d, 0x65, 0x6e, 0x6f,
0x6d, 0x69, 0x6e, 0x69, 0x73, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61,
0x73, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x61, 0x73, 0x4d, 0x6f,
0x6e, 0x6b, 0x74, 0x6f, 0x6e, 0x61, 0x73, 0x4d, 0x6f, 0x6e, 0x74, 0x65,
0x72, 0x117, 0x6a, 0x75, 0x73, 0x4d, 0x6f, 0x6e, 0x74, 0x65, 0x76, 0x69,
0x64, 0x117, 0x6a, 0x61, 0x73, 0x4d, 0x6f, 0x6e, 0x74, 0x73, 0x65, 0x72,
0x61, 0x74, 0x61, 0x73, 0x4e, 0x61, 0x73, 0x61, 0x75, 0x4e, 0x69, 0x75,
0x6a, 0x6f, 0x72, 0x6b, 0x61, 0x73, 0x4e, 0x6f, 0x6d, 0x61, 0x73, 0x42,
0x6a, 0x75, 0x6c, 0x61, 0x2c, 0x20, 0x160, 0x69, 0x61, 0x75, 0x72, 0x117,
0x73, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x53, 0x65, 0x6e, 0x74,
0x65, 0x72, 0x69, 0x73, 0x2c, 0x20, 0x160, 0x69, 0x61, 0x75, 0x72, 0x117,
0x73, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x69, 0x75, 0x20,
0x53, 0x65, 0x69, 0x6c, 0x65, 0x6d, 0x61, 0x73, 0x2c, 0x20, 0x160, 0x69,
0x61, 0x75, 0x72, 0x117, 0x73, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x4e, 0x16b, 0x6b, 0x61, 0x73, 0x4f, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x67,
0x61, 0x50, 0x61, 0x72, 0x61, 0x6d, 0x61, 0x72, 0x69, 0x62, 0x61, 0x73,
0x46, 0x69, 0x6e, 0x69, 0x6b, 0x73, 0x61, 0x73, 0x50, 0x6f, 0x72, 0x74,
0x20, 0x6f, 0x20, 0x50, 0x72, 0x65, 0x6e, 0x73, 0x61, 0x73, 0x50, 0x6f,
0x72, 0x74, 0x20, 0x6f, 0x66, 0x20, 0x53, 0x70, 0x65, 0x69, 0x6e, 0x61,
0x73, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x20, 0x56, 0x65, 0x6c, 0x6a, 0x61,
0x73, 0x50, 0x75, 0x65, 0x72, 0x74, 0x6f, 0x20, 0x52, 0x69, 0x6b, 0x61,
0x73, 0x50, 0x75, 0x6e, 0x74, 0x61, 0x20, 0x41, 0x72, 0x65, 0x6e, 0x61,
0x73, 0x61, 0x73, 0x52, 0x61, 0x6e, 0x6b, 0x69, 0x6e, 0x20, 0x49, 0x6e,
0x6c, 0x65, 0x74, 0x61, 0x73, 0x52, 0x65, 0x73, 0x69, 0x66, 0x117, 0x52,
0x65, 0x64, 0x17e, 0x69, 0x6e, 0x61, 0x52, 0x65, 0x73, 0x6f, 0x6c, 0x75,
0x74, 0x61, 0x73, 0x52, 0x69, 0x6f, 0x20, 0x42, 0x72, 0x61, 0x6e, 0x6b,
0x61, 0x73, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x65, 0x6e, 0x61, 0x73,
0x53, 0x61, 0x6e, 0x74, 0x6a, 0x61, 0x67, 0x61, 0x73, 0x53, 0x61, 0x6e,
0x74, 0x6f, 0x20, 0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x67, 0x61, 0x73, 0x53,
0x61, 0x6e, 0x20, 0x50, 0x61, 0x75, 0x6c, 0x61, 0x73, 0x49, 0x74, 0x6f,
0x6b, 0x6f, 0x72, 0x74, 0x6f, 0x72, 0x6d, 0x69, 0x74, 0x61, 0x73, 0x53,
0x65, 0x6e, 0x20, 0x42, 0x61, 0x72, 0x74, 0x65, 0x6c, 0x65, 0x6d, 0x69,
0x53, 0x65, 0x6e, 0x74, 0x20, 0x44, 0x17e, 0x6f, 0x6e, 0x73, 0x61, 0x73,
0x53, 0x65, 0x6e, 0x74, 0x20, 0x4b, 0x69, 0x74, 0x73, 0x61, 0x73, 0x53,
0x65, 0x6e, 0x74, 0x20, 0x4c, 0x75, 0x73, 0x69, 0x6a, 0x61, 0x53, 0x65,
0x6e, 0x74, 0x20, 0x54, 0x6f, 0x6d, 0x61, 0x73, 0x61, 0x73, 0x53, 0x65,
0x6e, 0x74, 0x20, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x74, 0x61, 0x73,
0x53, 0x76, 0x69, 0x66, 0x74, 0x20, 0x4b, 0x61, 0x72, 0x65, 0x6e, 0x74,
0x61, 0x73, 0x4b, 0x61, 0x6e, 0x61, 0x6b, 0x61, 0x73, 0x54, 0x69, 0x63,
0x68, 0x75, 0x61, 0x6e, 0x61, 0x54, 0x6f, 0x72, 0x6f, 0x6e, 0x74, 0x61,
0x73, 0x56, 0x61, 0x6e, 0x6b, 0x75, 0x76, 0x65, 0x72, 0x69, 0x73, 0x56,
0x61, 0x69, 0x74, 0x68, 0x6f, 0x72, 0x73, 0x61, 0x73, 0x56, 0x69, 0x6e,
0x69, 0x70, 0x65, 0x67, 0x61, 0x73, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x61,
0x74, 0x61, 0x73, 0x4b, 0x65, 0x69, 0x73, 0x69, 0x73, 0x44, 0x65, 0x69,
0x76, 0x69, 0x73, 0x61, 0x73, 0x44, 0x69, 0x75, 0x6d, 0x6f, 0x6e, 0x61,
0x73, 0x20, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x69, 0x73, 0x4d,
0x61, 0x6b, 0x76, 0x6f, 0x72, 0x69, 0x73, 0x4d, 0x6f, 0x73, 0x6f, 0x6e,
0x61, 0x73, 0x4d, 0x61, 0x6b, 0x6d, 0x65, 0x72, 0x64, 0x61, 0x73, 0x50,
0x61, 0x6c, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x53, 0x69, 0x6f, 0x76, 0x61,
0x54, 0x72, 0x6f, 0x6c, 0x69, 0x73, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x61, 0x73, 0x4c, 0x6f, 0x6e, 0x67, 0x6a, 0x79, 0x72, 0x62, 0x69, 0x65,
0x6e, 0x61, 0x73, 0x41, 0x64, 0x65, 0x6e, 0x61, 0x73, 0x41, 0x6d, 0x61,
0x6e, 0x61, 0x73, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x69, 0x73, 0x41,
0x6b, 0x74, 0x69, 0x75, 0x62, 0x69, 0x6e, 0x73, 0x6b, 0x61, 0x73, 0x41,
0x161, 0x63, 0x68, 0x61, 0x62, 0x61, 0x64, 0x61, 0x73, 0x42, 0x61, 0x67,
0x64, 0x61, 0x64, 0x61, 0x73, 0x42, 0x61, 0x68, 0x72, 0x65, 0x69, 0x6e,
0x61, 0x73, 0x42, 0x61, 0x6e, 0x6b, 0x6f, 0x6b, 0x61, 0x73, 0x42, 0x61,
0x72, 0x6e, 0x61, 0x75, 0x6c, 0x61, 0x73, 0x42, 0x65, 0x69, 0x72, 0x75,
0x74, 0x61, 0x73, 0x42, 0x69, 0x161, 0x6b, 0x65, 0x6b, 0x61, 0x73, 0x42,
0x72, 0x75, 0x6e, 0x117, 0x6a, 0x75, 0x73, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d,
0x62, 0x61, 0x73, 0x44, 0x61, 0x6d, 0x61, 0x73, 0x6b, 0x61, 0x73, 0x44,
0x69, 0x6c, 0x69, 0x73, 0x44, 0x75, 0x62, 0x61, 0x6a, 0x75, 0x73, 0x44,
0x75, 0x161, 0x61, 0x6e, 0x62, 0x117, 0x47, 0x61, 0x7a, 0x6f, 0x73, 0x20,
0x72, 0x75, 0x6f, 0x17e, 0x61, 0x73, 0x48, 0x65, 0x62, 0x72, 0x6f, 0x6e,
0x61, 0x73, 0x48, 0x6f, 0x161, 0x69, 0x6d, 0x69, 0x6e, 0x61, 0x73, 0x48,
0x6f, 0x6e, 0x6b, 0x6f, 0x6e, 0x67, 0x61, 0x73, 0x48, 0x6f, 0x76, 0x64,
0x61, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x61, 0x73, 0x4a,
0x65, 0x72, 0x75, 0x7a, 0x61, 0x6c, 0x117, 0x4b, 0x61, 0x62, 0x75, 0x6c,
0x61, 0x73, 0x4b, 0x61, 0x6d, 0x10d, 0x69, 0x61, 0x74, 0x6b, 0x61, 0x4b,
0x61, 0x72, 0x61, 0x10d, 0x69, 0x73, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x6a, 0x61, 0x72, 0x73, 0x6b, 0x61, 0x73, 0x4b, 0x76, 0x61, 0x6c, 0x61,
0x20, 0x4c, 0x75, 0x6d, 0x70, 0x16b, 0x72, 0x61, 0x73, 0x4b, 0x75, 0x10d,
0x69, 0x6e, 0x67, 0x61, 0x73, 0x4b, 0x75, 0x76, 0x65, 0x69, 0x74, 0x61,
0x73, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x61, 0x73, 0x4d, 0x61,
0x6b, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x4d, 0x61, 0x73, 0x6b, 0x61,
0x74, 0x61, 0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x6b, 0x75, 0x7a, 0x6e, 0x65,
0x63, 0x6b, 0x61, 0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x61, 0x73, 0x4f, 0x6d, 0x73, 0x6b, 0x61, 0x73, 0x55,
0x72, 0x61, 0x6c, 0x73, 0x6b, 0x61, 0x73, 0x50, 0x6e, 0x6f, 0x6d, 0x70,
0x65, 0x6e, 0x69, 0x73, 0x50, 0x6f, 0x6e, 0x74, 0x69, 0x61, 0x6e, 0x61,
0x6b, 0x61, 0x73, 0x50, 0x63, 0x68, 0x65, 0x6e, 0x6a, 0x61, 0x6e, 0x61,
0x73, 0x4b, 0x61, 0x74, 0x61, 0x72, 0x61, 0x73, 0x4b, 0x6f, 0x73, 0x74,
0x61, 0x6e, 0x61, 0x6a, 0x75, 0x73, 0x4b, 0x7a, 0x79, 0x6c, 0x2d, 0x4f,
0x72, 0x64, 0x61, 0x52, 0x69, 0x6a, 0x61, 0x64, 0x61, 0x73, 0x53, 0x61,
0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x61, 0x73, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x6b, 0x61, 0x6e, 0x64, 0x61, 0x73, 0x53, 0x65, 0x75, 0x6c, 0x61,
0x73, 0x160, 0x61, 0x6e, 0x63, 0x68, 0x61, 0x6a, 0x75, 0x73, 0x53, 0x69,
0x6e, 0x67, 0x61, 0x70, 0x16b, 0x72, 0x61, 0x73, 0x53, 0x72, 0x65, 0x64,
0x6e, 0x65, 0x6b, 0x6f, 0x6c, 0x79, 0x6d, 0x73, 0x6b, 0x61, 0x73, 0x54,
0x61, 0x69, 0x70, 0x117, 0x6a, 0x75, 0x73, 0x54, 0x61, 0x161, 0x6b, 0x65,
0x6e, 0x74, 0x61, 0x73, 0x54, 0x62, 0x69, 0x6c, 0x69, 0x73, 0x69, 0x73,
0x54, 0x65, 0x68, 0x65, 0x72, 0x61, 0x6e, 0x61, 0x73, 0x54, 0x6f, 0x6b,
0x69, 0x6a, 0x61, 0x73, 0x54, 0x6f, 0x6d, 0x73, 0x6b, 0x61, 0x73, 0x55,
0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x61, 0x73, 0x55,
0x73, 0x74, 0x20, 0x4e, 0x65, 0x72, 0x61, 0x56, 0x69, 0x65, 0x6e, 0x74,
0x69, 0x61, 0x6e, 0x61, 0x73, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x61, 0x73, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x61, 0x73, 0x52, 0x61, 0x6e, 0x67, 0x16b, 0x6e, 0x61, 0x73, 0x4a,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x61, 0x73, 0x4a, 0x65, 0x72, 0x65, 0x76, 0x61, 0x6e, 0x61, 0x73, 0x41,
0x7a, 0x6f, 0x72, 0x61, 0x69, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x173, 0x20,
0x73, 0x61, 0x6c, 0x6f, 0x73, 0x17d, 0x61, 0x6c, 0x69, 0x61, 0x73, 0x69,
0x73, 0x20, 0x4b, 0x79, 0x161, 0x75, 0x6c, 0x79, 0x73, 0x46, 0x61, 0x72,
0x65, 0x72, 0x61, 0x69, 0x52, 0x65, 0x69, 0x6b, 0x6a, 0x61, 0x76, 0x69,
0x6b, 0x61, 0x73, 0x50, 0x69, 0x65, 0x74, 0x173, 0x20, 0x44, 0x17e, 0x6f,
0x72, 0x64, 0x17e, 0x69, 0x6a, 0x61, 0x53, 0x65, 0x6e, 0x74, 0x20, 0x48,
0x65, 0x6c, 0x65, 0x6e, 0x61, 0x53, 0x74, 0x65, 0x6e, 0x6c, 0x69, 0x73,
0x41, 0x64, 0x65, 0x6c, 0x61, 0x69, 0x64, 0x117, 0x42, 0x72, 0x69, 0x73,
0x62, 0x61, 0x6e, 0x61, 0x73, 0x42, 0x72, 0x6f, 0x6b, 0x65, 0x6e, 0x20,
0x48, 0x69, 0x6c, 0x69, 0x73, 0x44, 0x61, 0x72, 0x76, 0x69, 0x6e, 0x61,
0x73, 0x48, 0x6f, 0x62, 0x61, 0x72, 0x74, 0x61, 0x73, 0x4c, 0x69, 0x6e,
0x64, 0x65, 0x6d, 0x61, 0x6e, 0x61, 0x73, 0x4c, 0x6f, 0x72, 0x64, 0x6f,
0x20, 0x48, 0x61, 0x75, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x4d, 0x65, 0x6c,
0x62, 0x75, 0x72, 0x6e, 0x61, 0x73, 0x50, 0x65, 0x72, 0x74, 0x61, 0x73,
0x53, 0x69, 0x64, 0x6e, 0x117, 0x6a, 0x75, 0x73, 0x41, 0x6d, 0x73, 0x74,
0x65, 0x72, 0x64, 0x61, 0x6d, 0x61, 0x73, 0x41, 0x73, 0x74, 0x72, 0x61,
0x63, 0x68, 0x61, 0x6e, 0x117, 0x41, 0x74, 0x117, 0x6e, 0x61, 0x69, 0x42,
0x65, 0x6c, 0x67, 0x72, 0x61, 0x64, 0x61, 0x73, 0x42, 0x65, 0x72, 0x6c,
0x79, 0x6e, 0x61, 0x73, 0x42, 0x72, 0x69, 0x75, 0x73, 0x65, 0x6c, 0x69,
0x73, 0x42, 0x75, 0x6b, 0x61, 0x72, 0x65, 0x161, 0x74, 0x61, 0x73, 0x42,
0x75, 0x64, 0x61, 0x70, 0x65, 0x161, 0x74, 0x61, 0x73, 0x42, 0x69, 0x75,
0x73, 0x69, 0x6e, 0x67, 0x65, 0x6e, 0x61, 0x73, 0x4b, 0x69, 0x161, 0x69,
0x6e, 0x69, 0x6f, 0x76, 0x61, 0x73, 0x4b, 0x6f, 0x70, 0x65, 0x6e, 0x68,
0x61, 0x67, 0x61, 0x44, 0x75, 0x62, 0x6c, 0x69, 0x6e, 0x61, 0x73, 0x47,
0x69, 0x62, 0x72, 0x61, 0x6c, 0x74, 0x61, 0x72, 0x61, 0x73, 0x47, 0x65,
0x72, 0x6e, 0x73, 0x69, 0x73, 0x48, 0x65, 0x6c, 0x73, 0x69, 0x6e, 0x6b,
0x69, 0x73, 0x4d, 0x65, 0x6e, 0x6f, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x53,
0x74, 0x61, 0x6d, 0x62, 0x75, 0x6c, 0x61, 0x73, 0x44, 0x17e, 0x65, 0x72,
0x73, 0x69, 0x73, 0x4b, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x6e, 0x67, 0x72,
0x61, 0x64, 0x61, 0x73, 0x4b, 0x69, 0x72, 0x6f, 0x76, 0x61, 0x73, 0x4b,
0x69, 0x6a, 0x65, 0x76, 0x61, 0x73, 0x4c, 0x6f, 0x6e, 0x64, 0x6f, 0x6e,
0x61, 0x73, 0x4c, 0x69, 0x75, 0x6b, 0x73, 0x65, 0x6d, 0x62, 0x75, 0x72,
0x67, 0x61, 0x73, 0x4d, 0x61, 0x64, 0x72, 0x69, 0x64, 0x61, 0x73, 0x4d,
0x61, 0x72, 0x69, 0x61, 0x6e, 0x68, 0x61, 0x6d, 0x69, 0x6e, 0x61, 0x4d,
0x69, 0x6e, 0x73, 0x6b, 0x61, 0x73, 0x4d, 0x6f, 0x6e, 0x61, 0x6b, 0x61,
0x73, 0x4d, 0x61, 0x73, 0x6b, 0x76, 0x61, 0x4f, 0x73, 0x6c, 0x61, 0x73,
0x50, 0x61, 0x72, 0x79, 0x17e, 0x69, 0x75, 0x73, 0x52, 0x79, 0x67, 0x61,
0x53, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6e, 0x61, 0x73, 0x53,
0x61, 0x72, 0x61, 0x6a, 0x65, 0x76, 0x61, 0x73, 0x53, 0x61, 0x72, 0x61,
0x74, 0x6f, 0x76, 0x61, 0x73, 0x53, 0x69, 0x6d, 0x66, 0x65, 0x72, 0x6f,
0x70, 0x6f, 0x6c, 0x69, 0x73, 0x53, 0x6b, 0x6f, 0x70, 0x6a, 0x117, 0x53,
0x74, 0x6f, 0x6b, 0x68, 0x6f, 0x6c, 0x6d, 0x61, 0x73, 0x54, 0x61, 0x6c,
0x69, 0x6e, 0x61, 0x73, 0x55, 0x6c, 0x6a, 0x61, 0x6e, 0x6f, 0x76, 0x73,
0x6b, 0x61, 0x73, 0x56, 0x61, 0x64, 0x75, 0x63, 0x61, 0x73, 0x56, 0x61,
0x74, 0x69, 0x6b, 0x61, 0x6e, 0x61, 0x73, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x61, 0x73, 0x56, 0x61, 0x72, 0x161, 0x75, 0x76,
0x61, 0x5a, 0x61, 0x67, 0x72, 0x65, 0x62, 0x61, 0x73, 0x43, 0x69, 0x75,
0x72, 0x69, 0x63, 0x68, 0x61, 0x73, 0x41, 0x6e, 0x74, 0x61, 0x6e, 0x61,
0x6e, 0x61, 0x72, 0x79, 0x76, 0x61, 0x73, 0x10c, 0x61, 0x67, 0x6f, 0x73,
0x61, 0x73, 0x4b, 0x61, 0x6c, 0x117, 0x64, 0x173, 0x20, 0x53, 0x61, 0x6c,
0x61, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x173, 0x20, 0x73, 0x61, 0x6c, 0x61,
0x4b, 0x65, 0x72, 0x67, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x73, 0x4d, 0x61,
0x68, 0x117, 0x4d, 0x61, 0x6c, 0x64, 0x79, 0x76, 0x61, 0x69, 0x4d, 0x61,
0x6a, 0x6f, 0x74, 0x61, 0x73, 0x52, 0x65, 0x75, 0x6e, 0x6a, 0x6f, 0x6e,
0x61, 0x73, 0x4f, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x73, 0x42, 0x75,
0x67, 0x65, 0x6e, 0x76, 0x69, 0x6c, 0x69, 0x73, 0x10c, 0x61, 0x74, 0x61,
0x6d, 0x61, 0x73, 0x10c, 0x75, 0x6b, 0x61, 0x73, 0x56, 0x65, 0x6c, 0x79,
0x6b, 0x173, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x45, 0x66, 0x61, 0x74, 0x117,
0x46, 0x61, 0x6b, 0x61, 0x6f, 0x66, 0x61, 0x73, 0x46, 0x69, 0x64, 0x17e,
0x69, 0x73, 0x46, 0x75, 0x6e, 0x61, 0x66, 0x75, 0x74, 0x69, 0x73, 0x47,
0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x61, 0x69, 0x47, 0x61, 0x6d, 0x62,
0x79, 0x72, 0x61, 0x73, 0x47, 0x76, 0x61, 0x64, 0x61, 0x6c, 0x6b, 0x61,
0x6e, 0x61, 0x6c, 0x69, 0x73, 0x47, 0x75, 0x61, 0x6d, 0x61, 0x73, 0x45,
0x6e, 0x64, 0x65, 0x72, 0x62, 0x75, 0x72, 0x69, 0x73, 0x4b, 0x69, 0x72,
0x69, 0x74, 0x69, 0x6d, 0x61, 0x74, 0x69, 0x73, 0x4b, 0x6f, 0x73, 0x72,
0x61, 0x6a, 0x117, 0x4b, 0x76, 0x61, 0x64, 0x17e, 0x61, 0x6c, 0x65, 0x69,
0x6e, 0x61, 0x73, 0x4d, 0x61, 0x64, 0x17e, 0x16b, 0x72, 0x61, 0x73, 0x4d,
0x61, 0x72, 0x6b, 0x69, 0x7a, 0x6f, 0x20, 0x73, 0x61, 0x6c, 0x6f, 0x73,
0x4d, 0x69, 0x64, 0x76, 0x117, 0x6a, 0x75, 0x73, 0x4e, 0x69, 0x75, 0x6a,
0x117, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x61, 0x73, 0x4e, 0x75,
0x6d, 0x117, 0x6a, 0x61, 0x50, 0x61, 0x67, 0x6f, 0x20, 0x50, 0x61, 0x67,
0x61, 0x73, 0x50, 0x69, 0x74, 0x6b, 0x65, 0x72, 0x6e, 0x6f, 0x20, 0x73,
0x61, 0x6c, 0x61, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x117, 0x50, 0x6f, 0x72,
0x74, 0x20, 0x4d, 0x6f, 0x72, 0x73, 0x62, 0x69, 0x73, 0x53, 0x61, 0x69,
0x70, 0x61, 0x6e, 0x61, 0x73, 0x54, 0x61, 0x69, 0x74, 0x69, 0x73, 0x56,
0x65, 0x69, 0x6b, 0x6f, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x56, 0x6f, 0x6c,
0x69, 0x73, 0x61, 0x73, 0x4b, 0x61, 0x69, 0x6d, 0x61, 0x6e, 0x69, 0x73,
0x6b, 0x65, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x4d, 0x65, 0x78, 0x69, 0x6b,
0x6f, 0x2d, 0x6d, 0x11b, 0x73, 0x74, 0x6f, 0x53, 0x74, 0x2e, 0x42, 0x61,
0x72, 0x74, 0x68, 0xe9, 0x6c, 0x65, 0x6d, 0x79, 0x44, 0x75, 0x6d, 0x6f,
0x6e, 0x74, 0x20, 0x44, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65,
0x48, 0x6f, 0x2d, 0x43, 0x68, 0x69, 0x2d, 0x4d, 0x69, 0x6e, 0x68, 0x2d,
0x6d, 0x11b, 0x73, 0x74, 0x6f, 0x50, 0x6f, 0x72, 0x74, 0x20, 0x4e, 0x75,
0x6d, 0x62, 0x61, 0x79, 0x4e, 0x6f, 0x77, 0x6f, 0x6b, 0x75, 0x7a, 0x6e,
0x6a, 0x65, 0x74, 0x73, 0x6b, 0x4a, 0x65, 0x72, 0x65, 0x77, 0x61, 0x6e,
0x41, 0x63, 0x6f, 0x72, 0x79, 0x46, 0xe4, 0x72, 0xf6, 0x6a, 0x65, 0x50,
0xf3, 0x64, 0x70, 0x6f, 0x142, 0x64, 0x6e, 0x6a, 0x6f, 0x77, 0x61, 0x20,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x61, 0x42, 0x11b, 0x142,
0x6f, 0x67, 0x72, 0x6f, 0x64, 0x4b, 0x69, 0x161, 0x69, 0x6e, 0x61, 0x75,
0x57, 0x69, 0x6c, 0x6e, 0x61, 0x57, 0x61, 0x72, 0x161, 0x61, 0x77, 0x61,
0x47, 0xf3, 0x64, 0x6f, 0x77, 0x6e, 0x65, 0x20, 0x6b, 0x75, 0x70, 0x79,
0x4d, 0x61, 0x6c, 0x65, 0x64, 0x69, 0x77, 0x79, 0x4a, 0x61, 0x74, 0x161,
0x6f, 0x77, 0x6e, 0x65, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x4b, 0x61, 0x6d,
0x73, 0x63, 0x68, 0x61, 0x74, 0x6b, 0x61, 0x4a, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x6f, 0x72, 0x67, 0x4b, 0x61, 0x69, 0x6d,
0x61, 0x6e, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x4a, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x65, 0x72, 0x67, 0x45,
0x72, 0x69, 0x76, 0x61, 0x6e, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x65, 0x73,
0x63, 0x68, 0x20, 0x49, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x42, 0x72,
0xe9, 0x69, 0x73, 0x73, 0x65, 0x6c, 0x4b, 0x69, 0x73, 0x63, 0x68, 0x69,
0x6e, 0x61, 0x75, 0x4c, 0xeb, 0x74, 0x7a, 0x65, 0x62, 0x75, 0x65, 0x72,
0x67, 0x52, 0x6f, 0x75, 0x6d, 0x5a, 0xfc, 0x72, 0x65, 0x63, 0x68, 0x43,
0x68, 0x72, 0xeb, 0x73, 0x63, 0x68, 0x74, 0x64, 0x61, 0x67, 0x73, 0x69,
0x6e, 0x73, 0x65, 0x6c, 0x4f, 0x75, 0x73, 0x63, 0x68, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x73, 0x65, 0x6c, 0x411, 0x430, 0x43d, 0x45f, 0x443, 0x43b, 0x411,
0x43b, 0x430, 0x43d, 0x442, 0x430, 0x458, 0x440, 0x41b, 0x438, 0x431, 0x440, 0x432,
0x438, 0x43b, 0x421, 0x430, 0x443, 0x43d, 0x20, 0x422, 0x43e, 0x43c, 0x435, 0x410,
0x43d, 0x442, 0x438, 0x433, 0x430, 0x423, 0x448, 0x443, 0x430, 0x458, 0x430, 0x41a,
0x43e, 0x440, 0x430, 0x43b, 0x20, 0x425, 0x430, 0x440, 0x431, 0x43e, 0x440, 0x411,
0x430, 0x445, 0x438, 0x430, 0x411, 0x430, 0x445, 0x438, 0x458, 0x430, 0x20, 0x434,
0x435, 0x20, 0x411, 0x430, 0x43d, 0x434, 0x435, 0x440, 0x430, 0x441, 0x41a, 0x430,
0x458, 0x435, 0x43d, 0x430, 0x41a, 0x430, 0x458, 0x43c, 0x430, 0x43d, 0x441, 0x43a,
0x438, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x421, 0x438, 0x443,
0x434, 0x430, 0x434, 0x20, 0x425, 0x443, 0x430, 0x440, 0x435, 0x437, 0x41a, 0x443,
0x458, 0x430, 0x431, 0x430, 0x414, 0x430, 0x43d, 0x43c, 0x430, 0x440, 0x43a, 0x448,
0x430, 0x43d, 0x415, 0x43b, 0x20, 0x421, 0x430, 0x43b, 0x432, 0x430, 0x434, 0x43e,
0x440, 0x413, 0x432, 0x430, 0x458, 0x430, 0x43d, 0x430, 0x415, 0x440, 0x43c, 0x43e,
0x441, 0x438, 0x459, 0x43e, 0x418, 0x43d, 0x434, 0x438, 0x458, 0x430, 0x43d, 0x430,
0x43f, 0x43e, 0x43b, 0x438, 0x441, 0x41d, 0x43e, 0x43a, 0x441, 0x2c, 0x20, 0x418,
0x43d, 0x434, 0x438, 0x458, 0x430, 0x43d, 0x430, 0x41f, 0x438, 0x442, 0x435, 0x440,
0x441, 0x431, 0x443, 0x440, 0x433, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x458,
0x430, 0x43d, 0x430, 0x412, 0x438, 0x432, 0x435, 0x458, 0x2c, 0x20, 0x418, 0x43d,
0x434, 0x438, 0x458, 0x430, 0x43d, 0x430, 0x412, 0x435, 0x43d, 0x441, 0x435, 0x43d,
0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x458, 0x430, 0x43d, 0x430, 0x41a, 0x440,
0x430, 0x43b, 0x435, 0x43d, 0x434, 0x430, 0x458, 0x43a, 0x41b, 0x43e, 0x441, 0x20,
0x410, 0x43d, 0x45f, 0x435, 0x43b, 0x435, 0x441, 0x414, 0x43e, 0x43b, 0x435, 0x43d,
0x20, 0x41f, 0x440, 0x438, 0x43d, 0x441, 0x20, 0x43a, 0x432, 0x43e, 0x440, 0x442,
0x440, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x430, 0x43d, 0x411, 0x458, 0x443, 0x43b,
0x430, 0x2c, 0x20, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x414,
0x430, 0x43a, 0x43e, 0x442, 0x430, 0x40a, 0x443, 0x20, 0x421, 0x430, 0x43b, 0x435,
0x43c, 0x2c, 0x20, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x414,
0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41f, 0x43e, 0x440, 0x442, 0x20, 0x43e, 0x20,
0x41f, 0x440, 0x435, 0x43d, 0x441, 0x41f, 0x43e, 0x440, 0x442, 0x20, 0x43e, 0x432,
0x20, 0x421, 0x43f, 0x435, 0x458, 0x43d, 0x420, 0x435, 0x43d, 0x43a, 0x438, 0x43d,
0x20, 0x418, 0x43d, 0x43b, 0x435, 0x442, 0x420, 0x435, 0x45f, 0x430, 0x458, 0x43d,
0x430, 0x421, 0x435, 0x43d, 0x442, 0x20, 0x411, 0x430, 0x440, 0x442, 0x43e, 0x43b,
0x43e, 0x43c, 0x435, 0x458, 0x421, 0x435, 0x43d, 0x442, 0x20, 0x40f, 0x43e, 0x43d,
0x441, 0x421, 0x432, 0x435, 0x442, 0x438, 0x20, 0x41a, 0x438, 0x442, 0x441, 0x421,
0x435, 0x43d, 0x442, 0x20, 0x41b, 0x443, 0x441, 0x438, 0x458, 0x430, 0x421, 0x435,
0x43d, 0x442, 0x20, 0x422, 0x43e, 0x43c, 0x430, 0x441, 0x41a, 0x435, 0x458, 0x441,
0x438, 0x414, 0x438, 0x43c, 0x43e, 0x43d, 0x20, 0x414, 0x438, 0x440, 0x432, 0x438,
0x43b, 0x41c, 0x430, 0x43a, 0x432, 0x43e, 0x440, 0x438, 0x421, 0x430, 0x458, 0x43e,
0x432, 0x430, 0x41b, 0x43e, 0x43d, 0x433, 0x458, 0x435, 0x440, 0x431, 0x438, 0x458,
0x435, 0x43d, 0x410, 0x448, 0x43a, 0x430, 0x431, 0x430, 0x434, 0x415, 0x440, 0x443,
0x441, 0x430, 0x43b, 0x438, 0x43c, 0x41f, 0x43e, 0x43d, 0x442, 0x438, 0x458, 0x430,
0x43d, 0x430, 0x43a, 0x421, 0x430, 0x43a, 0x430, 0x43b, 0x438, 0x43d, 0x412, 0x438,
0x435, 0x43d, 0x442, 0x438, 0x458, 0x430, 0x43d, 0x410, 0x437, 0x43e, 0x440, 0x441,
0x43a, 0x438, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x411, 0x435,
0x440, 0x43c, 0x443, 0x434, 0x438, 0x41a, 0x430, 0x43d, 0x430, 0x440, 0x441, 0x43a,
0x438, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x424, 0x430, 0x440,
0x441, 0x43a, 0x438, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x408,
0x443, 0x43a, 0x43b, 0x430, 0x411, 0x438, 0x437, 0x438, 0x43d, 0x433, 0x435, 0x43d,
0x41a, 0x438, 0x448, 0x438, 0x43d, 0x430, 0x443, 0x41e, 0x441, 0x442, 0x440, 0x43e,
0x432, 0x20, 0x41c, 0x430, 0x43d, 0x40f, 0x435, 0x440, 0x437, 0x438, 0x41c, 0x430,
0x440, 0x438, 0x458, 0x435, 0x445, 0x430, 0x43c, 0x435, 0x43d, 0x421, 0x43a, 0x43e,
0x43f, 0x458, 0x435, 0x423, 0x43b, 0x458, 0x430, 0x43d, 0x43e, 0x432, 0x441, 0x43a,
0x412, 0x438, 0x43b, 0x43d, 0x443, 0x441, 0x411, 0x43e, 0x436, 0x438, 0x45c, 0x435,
0x43d, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x41a, 0x43e, 0x43a, 0x43e,
0x441, 0x43e, 0x432, 0x438, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438,
0x41c, 0x430, 0x432, 0x440, 0x438, 0x446, 0x438, 0x443, 0x441, 0x420, 0x435, 0x458,
0x443, 0x43d, 0x438, 0x43e, 0x43d, 0x411, 0x443, 0x433, 0x430, 0x43d, 0x432, 0x438,
0x43b, 0x427, 0x443, 0x43a, 0x412, 0x435, 0x43b, 0x438, 0x433, 0x434, 0x435, 0x43d,
0x441, 0x43a, 0x438, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x413, 0x432,
0x430, 0x43c, 0x41a, 0x432, 0x430, 0x45f, 0x430, 0x43b, 0x435, 0x458, 0x43d, 0x41c,
0x430, 0x440, 0x43a, 0x435, 0x441, 0x43a, 0x438, 0x20, 0x41e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x438, 0x41f, 0x438, 0x442, 0x43a, 0x435, 0x440, 0x43d, 0x441, 0x43a,
0x438, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x41f, 0x43e, 0x43d,
0x43f, 0x435, 0x458, 0x905, 0x92c, 0x93f, 0x91c, 0x93e, 0x928, 0x905, 0x915, 0x930,
0x93e, 0x905, 0x926, 0x93f, 0x938, 0x20, 0x905, 0x92c, 0x93e, 0x92c, 0x93e, 0x905,
0x938, 0x92e, 0x93e, 0x930, 0x93e, 0x92c, 0x92e, 0x93e, 0x915, 0x94b, 0x92c, 0x902,
0x917, 0x941, 0x908, 0x92c, 0x902, 0x91c, 0x941, 0x932, 0x92c, 0x94d, 0x932, 0x947,
0x902, 0x91f, 0x93e, 0x92f, 0x930, 0x92c, 0x94d, 0x930, 0x947, 0x91c, 0x93e, 0x935,
0x93f, 0x932, 0x915, 0x93e, 0x939, 0x93f, 0x930, 0x93e, 0x938, 0x93f, 0x909, 0x924,
0x93e, 0x921, 0x93e, 0x930, 0x947, 0x20, 0x938, 0x932, 0x93e, 0x92e, 0x921, 0x93e,
0x909, 0x932, 0x93e, 0x90f, 0x932, 0x20, 0x910, 0x92f, 0x942, 0x928, 0x917, 0x947,
0x92c, 0x94b, 0x930, 0x94b, 0x928, 0x91c, 0x94b, 0x939, 0x93e, 0x928, 0x93f, 0x938,
0x92c, 0x930, 0x94d, 0x917, 0x915, 0x92e, 0x92a, 0x93e, 0x932, 0x93e, 0x916, 0x93e,
0x930, 0x924, 0x94c, 0x92e, 0x92e, 0x92a, 0x942, 0x924, 0x94b, 0x92e, 0x938, 0x948,
0x930, 0x942, 0x92e, 0x947, 0x902, 0x92c, 0x93e, 0x92c, 0x93e, 0x928, 0x947, 0x928,
0x947, 0x91c, 0x947, 0x92e, 0x940, 0x928, 0x93e, 0x928, 0x93f, 0x92f, 0x93e, 0x92e,
0x947, 0x92f, 0x928, 0x94c, 0x915, 0x91a, 0x94b, 0x91f, 0x914, 0x917, 0x93e, 0x921,
0x941, 0x917, 0x942, 0x92c, 0x93f, 0x902, 0x921, 0x94b, 0x90f, 0x915, 0x905, 0x926,
0x915, 0x90f, 0x902, 0x915, 0x94b, 0x930, 0x947, 0x91c, 0x90f, 0x902, 0x91f, 0x93f,
0x917, 0x941, 0x906, 0x905, 0x930, 0x93e, 0x917, 0x941, 0x90f, 0x928, 0x93e, 0x91c,
0x941, 0x91c, 0x941, 0x92f, 0x92e, 0x947, 0x902, 0x921, 0x94b, 0x91c, 0x93e, 0x930,
0x93f, 0x92f, 0x94b, 0x20, 0x917, 0x948, 0x932, 0x917, 0x938, 0x938, 0x93e, 0x932,
0x91f, 0x93e, 0x938, 0x928, 0x20, 0x91c, 0x941, 0x906, 0x928, 0x938, 0x928, 0x20,
0x932, 0x942, 0x908, 0x938, 0x909, 0x936, 0x941, 0x92f, 0x93e, 0x92f, 0x93e, 0x905,
0x938, 0x941, 0x928, 0x936, 0x93f, 0x92f, 0x928, 0x905, 0x91f, 0x93f, 0x915, 0x94b,
0x915, 0x928, 0x92c, 0x939, 0x93f, 0x92f, 0x93e, 0x20, 0x92c, 0x902, 0x921, 0x947,
0x930, 0x93e, 0x938, 0x92c, 0x93e, 0x930, 0x92c, 0x921, 0x94b, 0x938, 0x92c, 0x948,
0x932, 0x948, 0x92e, 0x92c, 0x94d, 0x932, 0x948, 0x902, 0x915, 0x2d, 0x938, 0x92c,
0x932, 0x94b, 0x928, 0x92c, 0x94b, 0x906, 0x20, 0x92c, 0x93f, 0x938, 0x94d, 0x91f,
0x93e, 0x92c, 0x94b, 0x92f, 0x91c, 0x940, 0x915, 0x948, 0x92e, 0x94d, 0x92c, 0x94d,
0x930, 0x940, 0x91c, 0x20, 0x92c, 0x947, 0x915, 0x947, 0x92e, 0x94d, 0x92a, 0x94b,
0x20, 0x917, 0x94d, 0x930, 0x947, 0x928, 0x921, 0x93e, 0x915, 0x941, 0x92f, 0x93e,
0x92c, 0x93e, 0x915, 0x94d, 0x92f, 0x942, 0x930, 0x93e, 0x938, 0x93e, 0x913, 0x926,
0x92e, 0x930, 0x94d, 0x915, 0x936, 0x928, 0x921, 0x93e, 0x909, 0x938, 0x928, 0x921,
0x93e, 0x909, 0x938, 0x928, 0x20, 0x915, 0x94d, 0x930, 0x940, 0x915, 0x921, 0x947,
0x928, 0x92c, 0x930, 0x921, 0x947, 0x91f, 0x94d, 0x930, 0x93e, 0x92f, 0x91f, 0x90f,
0x921, 0x92e, 0x94b, 0x902, 0x91f, 0x94b, 0x928, 0x908, 0x930, 0x942, 0x928, 0x947,
0x92a, 0x947, 0x92b, 0x94b, 0x930, 0x94d, 0x91f, 0x20, 0x928, 0x947, 0x932, 0x938,
0x928, 0x92b, 0x94b, 0x930, 0x94d, 0x91f, 0x93e, 0x932, 0x947, 0x91c, 0x93e, 0x917,
0x94d, 0x935, 0x93e, 0x921, 0x947, 0x932, 0x942, 0x92a, 0x917, 0x941, 0x906, 0x92f,
0x93e, 0x915, 0x94d, 0x935, 0x93f, 0x932, 0x939, 0x930, 0x92e, 0x94b, 0x938, 0x93f,
0x932, 0x94b, 0x928, 0x94b, 0x915, 0x94d, 0x938, 0x2c, 0x20, 0x907, 0x902, 0x921,
0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x92e, 0x947, 0x930, 0x948, 0x902, 0x917, 0x94b,
0x2c, 0x20, 0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x92c, 0x940,
0x92c, 0x947, 0x2c, 0x20, 0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e,
0x935, 0x93f, 0x928, 0x938, 0x947, 0x902, 0x938, 0x2c, 0x20, 0x907, 0x902, 0x921,
0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x92c, 0x93f, 0x928, 0x93e, 0x92e, 0x948, 0x915,
0x2c, 0x20, 0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x907, 0x928,
0x941, 0x92c, 0x93f, 0x915, 0x907, 0x915, 0x93e, 0x932, 0x935, 0x93f, 0x91f, 0x91c,
0x941, 0x928, 0x947, 0x909, 0x92e, 0x94b, 0x902, 0x91f, 0x940, 0x91a, 0x947, 0x932,
0x94b, 0x2c, 0x20, 0x915, 0x947, 0x902, 0x91f, 0x941, 0x915, 0x940, 0x915, 0x947,
0x932, 0x947, 0x902, 0x921, 0x93f, 0x91c, 0x94d, 0x915, 0x932, 0x93e, 0x938, 0x20,
0x90f, 0x902, 0x91c, 0x93f, 0x932, 0x93f, 0x938, 0x932, 0x94b, 0x905, 0x930, 0x20,
0x92a, 0x94d, 0x930, 0x93f, 0x902, 0x938, 0x947, 0x938, 0x20, 0x915, 0x94d, 0x935,
0x93e, 0x91f, 0x930, 0x92e, 0x938, 0x940, 0x92f, 0x94b, 0x92e, 0x93e, 0x930, 0x940,
0x917, 0x94c, 0x91f, 0x92e, 0x91c, 0x93e, 0x91f, 0x932, 0x93e, 0x928, 0x92e, 0x948,
0x928, 0x94b, 0x92e, 0x93f, 0x928, 0x940, 0x92e, 0x948, 0x930, 0x93f, 0x921, 0x93e,
0x92e, 0x948, 0x91f, 0x932, 0x93e, 0x915, 0x93e, 0x91f, 0x932, 0x93e, 0x92e, 0x948,
0x915, 0x94d, 0x938, 0x93f, 0x915, 0x94b, 0x20, 0x938, 0x93f, 0x91f, 0x940, 0x92e,
0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932, 0x93e, 0x928, 0x92e, 0x93e, 0x928, 0x94d,
0x915, 0x91f, 0x93e, 0x928, 0x92e, 0x93e, 0x928, 0x91f, 0x947, 0x948, 0x930, 0x940,
0x92e, 0x94b, 0x902, 0x91f, 0x948, 0x935, 0x93f, 0x921, 0x93f, 0x92f, 0x94b, 0x928,
0x938, 0x93e, 0x909, 0x928, 0x94d, 0x92f, 0x942, 0x92f, 0x93e, 0x930, 0x94d, 0x915,
0x92c, 0x93f, 0x92f, 0x942, 0x932, 0x93e, 0x2c, 0x20, 0x909, 0x924, 0x94d, 0x924,
0x930, 0x20, 0x921, 0x915, 0x94b, 0x91f, 0x93e, 0x938, 0x947, 0x902, 0x91f, 0x930,
0x2c, 0x20, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x921, 0x915, 0x94b, 0x91f,
0x93e, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x938, 0x932, 0x947, 0x92e, 0x2c, 0x20,
0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x921, 0x915, 0x94b, 0x91f, 0x93e, 0x92a,
0x94b, 0x930, 0x94d, 0x91f, 0x20, 0x906, 0x909, 0x20, 0x92a, 0x94d, 0x930, 0x93f,
0x902, 0x938, 0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x20, 0x906, 0x92b, 0x20, 0x938,
0x94d, 0x92a, 0x947, 0x928, 0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x94b, 0x20, 0x92c,
0x947, 0x932, 0x94d, 0x939, 0x94b, 0x92a, 0x941, 0x902, 0x91f, 0x93e, 0x20, 0x905,
0x930, 0x947, 0x928, 0x93e, 0x91c, 0x930, 0x93f, 0x938, 0x940, 0x92b, 0x930, 0x948,
0x917, 0x93f, 0x928, 0x93e, 0x930, 0x947, 0x938, 0x93e, 0x932, 0x942, 0x924, 0x930,
0x93f, 0x92f, 0x94b, 0x20, 0x92c, 0x94d, 0x930, 0x947, 0x902, 0x915, 0x94b, 0x938,
0x947, 0x902, 0x91f, 0x94b, 0x20, 0x921, 0x94b, 0x92e, 0x93f, 0x928, 0x917, 0x94b,
0x907, 0x91f, 0x94b, 0x915, 0x94b, 0x930, 0x94d, 0x91f, 0x942, 0x930, 0x92e, 0x93f,
0x91f, 0x938, 0x93f, 0x924, 0x915, 0x93e, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x92c,
0x930, 0x94d, 0x925, 0x947, 0x932, 0x92e, 0x940, 0x938, 0x947, 0x902, 0x91f, 0x20,
0x91c, 0x93e, 0x928, 0x94d, 0x938, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x932, 0x942,
0x936, 0x93f, 0x92f, 0x93e, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x925, 0x94b, 0x92e,
0x938, 0x938, 0x94d, 0x935, 0x93f, 0x92b, 0x94d, 0x91f, 0x20, 0x915, 0x930, 0x947,
0x902, 0x91f, 0x924, 0x941, 0x932, 0x947, 0x92c, 0x948, 0x902, 0x915, 0x941, 0x92c,
0x930, 0x935, 0x93e, 0x907, 0x91f, 0x939, 0x93e, 0x930, 0x94d, 0x938, 0x92c, 0x93f,
0x928, 0x93f, 0x92a, 0x948, 0x917, 0x92f, 0x915, 0x941, 0x91f, 0x93e, 0x91f, 0x92e,
0x948, 0x915, 0x94d, 0x935, 0x948, 0x930, 0x940, 0x932, 0x93e, 0x928, 0x94d, 0x917,
0x93f, 0x92f, 0x930, 0x92c, 0x947, 0x928, 0x905, 0x926, 0x928, 0x905, 0x932, 0x94d,
0x92e, 0x93e, 0x924, 0x940, 0x90f, 0x928, 0x93e, 0x921, 0x93f, 0x930, 0x905, 0x924,
0x93f, 0x930, 0x93e, 0x909, 0x92c, 0x948, 0x902, 0x915, 0x93e, 0x915, 0x91a, 0x93f,
0x924, 0x93e, 0x915, 0x94b, 0x932, 0x92e, 0x94d, 0x92c, 0x94b, 0x926, 0x92e, 0x93f,
0x936, 0x94d, 0x915, 0x939, 0x948, 0x92c, 0x94d, 0x930, 0x93e, 0x928, 0x939, 0x93e,
0x902, 0x917, 0x20, 0x915, 0x93e, 0x902, 0x917, 0x939, 0x94b, 0x92c, 0x94d, 0x926,
0x907, 0x930, 0x915, 0x941, 0x938, 0x94d, 0x915, 0x91c, 0x947, 0x930, 0x942, 0x938,
0x947, 0x932, 0x92e, 0x915, 0x92e, 0x91a, 0x91f, 0x915, 0x93e, 0x916, 0x93e, 0x902,
0x927, 0x94d, 0x92f, 0x917, 0x93e, 0x915, 0x94d, 0x930, 0x947, 0x938, 0x94d, 0x928,
0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x915, 0x941, 0x906, 0x932,
0x93e, 0x932, 0x92e, 0x94d, 0x92a, 0x941, 0x930, 0x915, 0x941, 0x91a, 0x93f, 0x902,
0x917, 0x92e, 0x915, 0x93e, 0x938, 0x93e, 0x930, 0x928, 0x93f, 0x915, 0x94b, 0x936,
0x93f, 0x92f, 0x93e, 0x915, 0x94b, 0x92c, 0x94b, 0x915, 0x941, 0x91c, 0x928, 0x947,
0x91f, 0x94d, 0x938, 0x928, 0x94b, 0x92c, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930,
0x938, 0x94d, 0x915, 0x913, 0x92e, 0x938, 0x94d, 0x915, 0x913, 0x930, 0x93e, 0x932,
0x92a, 0x94b, 0x902, 0x91f, 0x93f, 0x92f, 0x93e, 0x902, 0x915, 0x915, 0x94b, 0x938,
0x94d, 0x924, 0x93e, 0x928, 0x947, 0x915, 0x93f, 0x91c, 0x93f, 0x932, 0x94b, 0x930,
0x94d, 0x921, 0x93e, 0x938, 0x93e, 0x915, 0x932, 0x93f, 0x928, 0x938, 0x93f, 0x92f,
0x94b, 0x932, 0x936, 0x94d, 0x930, 0x947, 0x921, 0x928, 0x947, 0x915, 0x94b, 0x932,
0x93f, 0x92e, 0x93f, 0x938, 0x94d, 0x915, 0x924, 0x93e, 0x907, 0x92a, 0x947, 0x908,
0x91f, 0x92c, 0x93f, 0x932, 0x93f, 0x938, 0x93f, 0x925, 0x93f, 0x92e, 0x94d, 0x92a,
0x942, 0x91f, 0x94b, 0x92e, 0x94d, 0x938, 0x909, 0x932, 0x928, 0x92c, 0x91f, 0x94b,
0x930, 0x909, 0x930, 0x942, 0x92e, 0x915, 0x940, 0x909, 0x938, 0x94d, 0x924, 0x2d,
0x928, 0x947, 0x930, 0x93e, 0x92c, 0x93f, 0x92f, 0x902, 0x91f, 0x93f, 0x92f, 0x93e,
0x928, 0x92c, 0x94d, 0x932, 0x93e, 0x921, 0x93f, 0x92c, 0x94b, 0x938, 0x94d, 0x924,
0x94b, 0x915, 0x92f, 0x941, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x92f,
0x93e, 0x902, 0x917, 0x941, 0x928, 0x92f, 0x948, 0x915, 0x947, 0x91f, 0x947, 0x930,
0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x92f, 0x948, 0x930, 0x947, 0x92c, 0x928,
0x90f, 0x91c, 0x94b, 0x930, 0x94d, 0x938, 0x915, 0x947, 0x92a, 0x20, 0x92c, 0x930,
0x94d, 0x921, 0x92b, 0x930, 0x93e, 0x913, 0x92e, 0x926, 0x948, 0x930, 0x93e, 0x930,
0x93f, 0x915, 0x91c, 0x947, 0x935, 0x93f, 0x915, 0x926, 0x915, 0x94d, 0x937, 0x93f,
0x923, 0x20, 0x91c, 0x93e, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x92c, 0x94d,
0x930, 0x93f, 0x938, 0x92c, 0x947, 0x928, 0x92c, 0x94d, 0x930, 0x94b, 0x915, 0x947,
0x928, 0x20, 0x939, 0x93f, 0x932, 0x921, 0x93e, 0x930, 0x94d, 0x92c, 0x93f, 0x928,
0x907, 0x92f, 0x942, 0x915, 0x932, 0x93e, 0x939, 0x94b, 0x92c, 0x930, 0x94d, 0x91f,
0x932, 0x93f, 0x902, 0x921, 0x92e, 0x948, 0x928, 0x932, 0x93e, 0x930, 0x94d, 0x921,
0x20, 0x939, 0x94b, 0x92c, 0x947, 0x90f, 0x92e, 0x94d, 0x938, 0x91f, 0x930, 0x94d,
0x921, 0x92e, 0x90f, 0x902, 0x921, 0x94b, 0x930, 0x93e, 0x905, 0x938, 0x94d, 0x91f,
0x94d, 0x930, 0x93e, 0x915, 0x93e, 0x928, 0x92c, 0x93e, 0x930, 0x93f, 0x924, 0x938,
0x94d, 0x932, 0x93e, 0x935, 0x93e, 0x92c, 0x94d, 0x930, 0x938, 0x947, 0x932, 0x94d,
0x938, 0x92c, 0x938, 0x93f, 0x902, 0x917, 0x947, 0x928, 0x91a, 0x93f, 0x938, 0x93f,
0x928, 0x93e, 0x90a, 0x915, 0x94b, 0x92a, 0x947, 0x928, 0x939, 0x947, 0x917, 0x947,
0x928, 0x917, 0x930, 0x94d, 0x928, 0x91c, 0x940, 0x939, 0x948, 0x932, 0x938, 0x93f,
0x902, 0x915, 0x940, 0x906, 0x907, 0x91c, 0x932, 0x20, 0x906, 0x92b, 0x20, 0x92e,
0x948, 0x928, 0x907, 0x938, 0x94d, 0x924, 0x93e, 0x92e, 0x94d, 0x92c, 0x941, 0x932,
0x915, 0x932, 0x93f, 0x928, 0x93f, 0x928, 0x917, 0x94d, 0x930, 0x947, 0x921, 0x932,
0x93f, 0x938, 0x92c, 0x928, 0x932, 0x941, 0x92c, 0x932, 0x93f, 0x92f, 0x93e, 0x928,
0x93e, 0x932, 0x917, 0x94d, 0x91c, 0x92e, 0x92c, 0x930, 0x94d, 0x917, 0x92e, 0x947,
0x921, 0x94d, 0x930, 0x93f, 0x921, 0x92e, 0x93e, 0x932, 0x91f, 0x93e, 0x92e, 0x947,
0x930, 0x93f, 0x92f, 0x947, 0x939, 0x92e, 0x92e, 0x93f, 0x928, 0x94d, 0x938, 0x915,
0x92e, 0x94b, 0x928, 0x947, 0x915, 0x94b, 0x92e, 0x93e, 0x938, 0x94d, 0x915, 0x94b,
0x92a, 0x94b, 0x921, 0x917, 0x94b, 0x930, 0x93f, 0x936, 0x93e, 0x938, 0x948, 0x928,
0x20, 0x92e, 0x930, 0x940, 0x928, 0x94b, 0x938, 0x930, 0x93e, 0x91c, 0x947, 0x92c,
0x94b, 0x938, 0x93e, 0x930, 0x93e, 0x924, 0x94b, 0x92c, 0x938, 0x93f, 0x92e, 0x94d,
0x92b, 0x947, 0x930, 0x94b, 0x92a, 0x94b, 0x932, 0x938, 0x94d, 0x915, 0x94b, 0x92a,
0x92f, 0x947, 0x938, 0x94d, 0x91f, 0x93e, 0x915, 0x939, 0x94b, 0x92e, 0x91f, 0x93e,
0x932, 0x93f, 0x928, 0x91f, 0x93f, 0x930, 0x93e, 0x928, 0x947, 0x909, 0x932, 0x94d,
0x92f, 0x947, 0x928, 0x94b, 0x92c, 0x94b, 0x938, 0x94d, 0x915, 0x92c, 0x947, 0x926,
0x941, 0x91c, 0x92c, 0x947, 0x91f, 0x93f, 0x915, 0x928, 0x92c, 0x93f, 0x90f, 0x928,
0x93e, 0x92c, 0x93f, 0x932, 0x928, 0x93f, 0x909, 0x938, 0x92c, 0x94b, 0x932, 0x917,
0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x926, 0x92c, 0x93e, 0x930, 0x938, 0x93e, 0x91c,
0x917, 0x930, 0x948, 0x92c, 0x91c, 0x94d, 0x92f, 0x942, 0x930, 0x93f, 0x916, 0x905,
0x902, 0x924, 0x93e, 0x928, 0x93e, 0x928, 0x947, 0x930, 0x93f, 0x935, 0x94b, 0x91a,
0x917, 0x94b, 0x938, 0x915, 0x948, 0x930, 0x917, 0x941, 0x90f, 0x932, 0x947, 0x928,
0x92e, 0x93e, 0x930, 0x940, 0x936, 0x938, 0x92e, 0x92f, 0x94b, 0x924, 0x947, 0x930,
0x93f, 0x92f, 0x942, 0x928, 0x93f, 0x92f, 0x928, 0x906, 0x915, 0x932, 0x948, 0x902,
0x921, 0x91a, 0x947, 0x925, 0x92e, 0x917, 0x948, 0x932, 0x93e, 0x92a, 0x948, 0x917,
0x94b, 0x938, 0x928, 0x93e, 0x930, 0x92b, 0x93e, 0x915, 0xd05, 0xd2c, 0xd3f, 0xd26,
0xd4d, 0x200c, 0xd1c, 0xd3e, 0xd7b, 0x200c, 0xd06, 0xd15, 0xd4d, 0xd30, 0xd05, 0xd21,
0xd3f, 0xd38, 0xd4d, 0x20, 0xd05, 0xd2c, 0xd3e, 0xd2c, 0xd05, 0xd7e, 0xd1c, 0xd3f,
0xd2f, 0xd47, 0xd34, 0xd4d, 0x200c, 0xd38, 0xd4d, 0xd05, 0xd38, 0xd4d, 0x200d, 0xd2e,
0xd3e, 0xd30, 0xd2c, 0xd2e, 0xd3e, 0xd15, 0xd4b, 0xd2c, 0xd02, 0xd17, 0xd41, 0xd2f,
0xd3f, 0xd2c, 0xd1e, 0xd4d, 0xd1a, 0xd41, 0xd7d, 0xd2c, 0xd3f, 0xd38, 0xd4d, 0xd38,
0xd3e, 0xd35, 0xd41, 0xd2c, 0xd4d, 0xd32, 0xd3e, 0xd23, 0xd4d, 0xd1f, 0xd46, 0xd2f,
0xd7c, 0x200c, 0xd2c, 0xd4d, 0xd30, 0xd3e, 0xd38, 0xd35, 0xd3f, 0xd7d, 0xd2c, 0xd41,
0xd1c, 0xd41, 0xd02, 0x200c, 0xd2c, 0xd41, 0xd30, 0xd15, 0xd46, 0xd2f, 0xd4d, 0x200c,
0xd31, 0xd4b, 0xd15, 0xd3e, 0xd38, 0xd2c, 0xd4d, 0xd32, 0xd3e, 0xd19, 0xd4d, 0xd15,
0xd15, 0xd4d, 0xd2f, 0xd42, 0xd1f, 0xd4d, 0xd1f, 0xd15, 0xd4a, 0xd23, 0xd3e, 0xd15,
0xd4d, 0xd30, 0xd3f, 0xd21, 0xd3e, 0xd15, 0xd4d, 0xd15, 0xd7c, 0x200c, 0xd26, 0xd3e,
0xd7c, 0x20, 0xd0e, 0xd38, 0xd4d, 0x20, 0xd38, 0xd32, 0xd3e, 0xd02, 0xd26, 0xd3f,
0xd1c, 0xd3f, 0xd2c, 0xd57, 0xd1f, 0xd4d, 0xd1f, 0xd3f, 0xd21, 0xd57, 0xd32, 0xd0e,
0xd7d, 0x200c, 0x20, 0xd10, 0xd2f, 0xd41, 0xd7b, 0x200c, 0xd2b, 0xd4d, 0xd30, 0xd40,
0xd1f, 0xd57, 0xd7a, 0xd17, 0xd3e, 0xd2c, 0xd31, 0xd4b, 0xd7a, 0xd39, 0xd30, 0xd3e,
0xd30, 0xd46, 0xd1c, 0xd4b, 0xd39, 0xd28, 0xd4d, 0xd28, 0xd3e, 0xd38, 0xd4d, 0xd2c,
0xd7c, 0x200c, 0xd17, 0xd4d, 0xd1c, 0xd41, 0xd2c, 0xd15, 0xd2e, 0xd4d, 0xd2a, 0xd3e,
0xd32, 0xd16, 0xd3e, 0xd7c, 0x200c, 0xd24, 0xd4c, 0xd02, 0xd15, 0xd3f, 0xd17, 0xd3e,
0xd32, 0xd3f, 0xd15, 0xd3f, 0xd7b, 0xd37, 0xd3e, 0xd38, 0xd32, 0xd3e, 0xd17, 0xd4b,
0xd38, 0xd4d, 0xd32, 0xd3f, 0xd2c, 0xd4d, 0xd30, 0xd46, 0xd35, 0xd3f, 0xd32, 0xd4d,
0xd32, 0xd32, 0xd4b, 0xd02, 0xd32, 0xd41, 0xd35, 0xd3e, 0xd7b, 0xd21, 0xd32, 0xd42,
0xd2c, 0xd41, 0xd02, 0xd2c, 0xd3e, 0xd37, 0xd3f, 0xd32, 0xd41, 0xd38, 0xd3e, 0xd15,
0xd2e, 0xd32, 0xd3e, 0xd2c, 0xd4b, 0xd2e, 0xd3e, 0xd2a, 0xd4d, 0xd2f, 0xd41, 0xd1f,
0xd4d, 0xd1f, 0xd4b, 0xd2e, 0xd38, 0xd47, 0xd31, 0xd41, 0xd2e, 0xd2c, 0xd3e, 0xd2c,
0xd46, 0xd2f, 0xd4d, 0x200c, 0xd7b, 0x200c, 0xd2e, 0xd4a, 0xd17, 0xd3e, 0xd26, 0xd3f,
0xd37, 0xd41, 0xd2e, 0xd7a, 0x200c, 0xd31, 0xd4b, 0xd35, 0xd3f, 0xd2f, 0xd28, 0xd2f,
0xd4d, 0x200c, 0xd31, 0xd4b, 0xd2c, 0xd3f, 0xd1c, 0xd2e, 0xd46, 0xd28, 0xd28, 0xd3f,
0xd2f, 0xd3e, 0xd2e, 0xd3f, 0xd28, 0xd57, 0xd35, 0xd3e, 0xd15, 0xd4d, 0x200c, 0xd37,
0xd4b, 0xd1f, 0xd4d, 0xd1f, 0xd4d, 0xd14, 0xd17, 0xd3e, 0xd26, 0xd57, 0xd17, 0xd57,
0xd2a, 0xd4b, 0xd7c, 0x200c, 0xd1f, 0xd4d, 0xd1f, 0xd4b, 0x2d, 0xd28, 0xd4b, 0xd35,
0xd4b, 0xd38, 0xd3e, 0xd35, 0xd4b, 0x20, 0xd1f, 0xd4b, 0xd02, 0x200c, 0xd1f, 0xd4d,
0xd30, 0xd3f, 0xd2a, 0xd4a, 0xd33, 0xd3f, 0xd1f, 0xd4d, 0xd2f, 0xd42, 0xd23, 0xd3f,
0xd38, 0xd4d, 0xd35, 0xd3f, 0xd7b, 0xd21, 0xd4d, 0x200c, 0xd39, 0xd4b, 0xd15, 0xd4d,
0xd05, 0xd21, 0xd3e, 0xd15, 0xd4d, 0xd06, 0xd19, 0xd4d, 0xd15, 0xd31, 0xd47, 0xd1c,
0xd4d, 0xd06, 0xd7b, 0xd17, 0xd4d, 0xd35, 0xd3f, 0xd32, 0xd4d, 0xd32, 0xd06, 0xd7b,
0xd31, 0xd3f, 0xd17, 0xd4d, 0xd35, 0xd05, 0xd31, 0xd17, 0xd4d, 0xd35, 0xd48, 0xd28,
0xd2c, 0xd4d, 0xd2f, 0xd42, 0xd23, 0xd38, 0xd4d, 0x20, 0xd10, 0xd31, 0xd3f, 0xd38,
0xd4d, 0x200d, 0xd15, 0xd4d, 0xd2f, 0xd3e, 0xd31, 0xd4d, 0xd31, 0xd2e, 0xd3e, 0xd7c,
0xd15, 0xd4d, 0xd15, 0xd15, 0xd4b, 0xd7c, 0xd21, 0xd4b, 0xd2c, 0xd1c, 0xd41, 0xd1c,
0xd41, 0xd2f, 0xd4d, 0xd32, 0xd3e, 0x20, 0xd31, 0xd3f, 0xd2f, 0xd4b, 0xd1c, 0xd2e,
0xd46, 0xd7b, 0xd21, 0xd4b, 0xd38, 0xd31, 0xd3f, 0xd2f, 0xd4b, 0x20, 0xd17, 0xd4d,
0xd2f, 0xd3e, 0xd32, 0xd17, 0xd4b, 0xd38, 0xd4d, 0xd38, 0xd3e, 0xd7d, 0xd1f, 0xd4d,
0xd1f, 0xd38, 0xd3e, 0xd7b, 0x20, 0xd1c, 0xd41, 0xd35, 0xd3e, 0xd7b, 0xd38, 0xd3e,
0xd7b, 0x20, 0xd32, 0xd42, 0xd2f, 0xd3f, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd41,
0xd15, 0xd41, 0xd2e, 0xd3e, 0xd7b, 0xd09, 0xd37, 0xd3f, 0xd2f, 0xd05, 0xd31, 0xd42,
0xd2c, 0xd05, 0xd38, 0xd7b, 0x200c, 0xd37, 0xd4d, 0xd2f, 0xd7b, 0x200c, 0xd0f, 0xd31,
0xd4d, 0xd31, 0xd3f, 0xd15, 0xd4b, 0xd15, 0xd4d, 0xd15, 0xd7a, 0xd2c, 0xd39, 0xd3f,
0xd2f, 0xd2c, 0xd39, 0xd3f, 0xd2f, 0x20, 0xd2c, 0xd7b, 0xd21, 0xd3e, 0xd30, 0xd38,
0xd4d, 0xd2c, 0xd3e, 0xd7c, 0xd2c, 0xd21, 0xd4b, 0xd38, 0xd4d, 0xd2c, 0xd46, 0xd32,
0xd02, 0xd2c, 0xd46, 0xd32, 0xd40, 0xd38, 0xd4d, 0xd2c, 0xd4d, 0xd32, 0xd3e, 0xd19,
0xd4d, 0xd15, 0xd4d, 0x20, 0xd38, 0xd3e, 0xd2c, 0xd4d, 0xd32, 0xd4b, 0xd7a, 0xd2c,
0xd4b, 0xd35, 0x20, 0xd35, 0xd3f, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd2c, 0xd4a,
0xd17, 0xd4b, 0xd1f, 0xd4d, 0xd1f, 0xd2c, 0xd4a, 0xd2f, 0xd4d, 0xd38, 0xd3f, 0xd15,
0xd47, 0xd02, 0xd2c, 0xd4d, 0xd30, 0xd3f, 0xd21, 0xd4d, 0x200c, 0xd1c, 0xd4d, 0x20,
0xd2c, 0xd47, 0xd15, 0xd4d, 0xd2f, 0xd3e, 0xd2e, 0xd4d, 0xd2a, 0xd4b, 0x20, 0xd17,
0xd4d, 0xd30, 0xd3e, 0xd7b, 0xd21, 0xd46, 0xd15, 0xd3e, 0xd7b, 0xd15, 0xd42, 0xd7a,
0xd15, 0xd30, 0xd3e, 0xd15, 0xd4d, 0xd15, 0xd38, 0xd4d, 0xd15, 0xd2f, 0xd40, 0xd7b,
0x200c, 0xd15, 0xd47, 0xd2e, 0xd3e, 0xd7b, 0xd1a, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd3e,
0xd17, 0xd4b, 0xd1a, 0xd3f, 0xd39, 0xd4d, 0xd35, 0xd3e, 0xd39, 0xd38, 0xd3f, 0xd2f,
0xd41, 0xd21, 0xd3e, 0xd21, 0xd4d, 0x20, 0xd39, 0xd41, 0xd35, 0xd3e, 0xd30, 0xd38,
0xd4d, 0xd15, 0xd4b, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0x20, 0xd31, 0xd3f, 0xd15,
0xd4d, 0xd15, 0xd15, 0xd4d, 0xd30, 0xd46, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd7a,
0xd15, 0xd41, 0xd2f, 0xd4d, 0x200c, 0xd2c, 0xd15, 0xd41, 0xd31, 0xd3e, 0xd15, 0xd4d,
0xd15, 0xd3e, 0xd35, 0xd4b, 0xd21, 0xd3e, 0xd7b, 0xd2e, 0xd3e, 0xd7c, 0xd15, 0xd4d,
0xd15, 0xd4d, 0xd37, 0xd3e, 0xd35, 0xd4d, 0xd7b, 0xd21, 0xd4b, 0xd35, 0xd4d, 0xd38,
0xd7a, 0xd21, 0xd4b, 0xd35, 0xd4d, 0xd38, 0xd7a, 0x20, 0xd15, 0xd4d, 0xd30, 0xd40,
0xd15, 0xd4d, 0xd15, 0xd4d, 0xd21, 0xd46, 0xd7b, 0x200c, 0xd35, 0xd7c, 0xd21, 0xd46,
0xd1f, 0xd4d, 0xd30, 0xd4b, 0xd2f, 0xd3f, 0xd31, 0xd4d, 0xd31, 0xd4d, 0xd21, 0xd4a,
0xd2e, 0xd3f, 0xd28, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd0e, 0xd21, 0xd4d, 0xd2e, 0xd4b,
0xd7a, 0xd1f, 0xd7a, 0xd2f, 0xd46, 0xd31, 0xd41, 0xd28, 0xd40, 0xd2a, 0xd4d, 0xd2a,
0xd46, 0xd0e, 0xd7d, 0x20, 0xd38, 0xd3e, 0xd7d, 0x200c, 0xd35, 0xd26, 0xd4b, 0xd7c,
0xd2b, 0xd4b, 0xd7c, 0xd1f, 0xd4d, 0xd1f, 0xd4d, 0x20, 0xd28, 0xd46, 0xd7d, 0xd38,
0xd7a, 0xd2b, 0xd4b, 0xd7c, 0xd1f, 0xd4d, 0xd1f, 0xd32, 0xd40, 0xd38, 0xd17, 0xd4d,
0xd32, 0xd46, 0xd2f, 0xd4d, 0xd38, 0xd4d, 0x20, 0xd2c, 0xd47, 0xd17, 0xd42, 0xd38,
0xd4d, 0x20, 0xd2c, 0xd47, 0xd17, 0xd4d, 0xd30, 0xd3e, 0xd7b, 0xd21, 0xd4d, 0x20,
0xd1f, 0xd7c, 0xd15, 0xd4d, 0xd15, 0xd4d, 0xd17, 0xd4d, 0xd30, 0xd28, 0xd47, 0xd21,
0xd17, 0xd4d, 0xd35, 0xd3e, 0xd21, 0xd32, 0xd42, 0xd2a, 0xd4d, 0xd2a, 0xd4d, 0xd17,
0xd4d, 0xd35, 0xd3e, 0xd1f, 0xd4d, 0xd1f, 0xd3f, 0xd2e, 0xd3e, 0xd32, 0xd17, 0xd41,
0xd35, 0xd3e, 0xd2f, 0xd15, 0xd4d, 0xd35, 0xd3f, 0xd7d, 0xd17, 0xd2f, 0xd3e, 0xd28,
0xd39, 0xd3e, 0xd32, 0xd3f, 0xd2b, 0xd3e, 0xd15, 0xd4d, 0xd38, 0xd4d, 0xd39, 0xd35,
0xd3e, 0xd28, 0xd39, 0xd46, 0xd7c, 0xd2e, 0xd4b, 0xd38, 0xd3f, 0xd32, 0xd4d, 0xd32,
0xd4b, 0xd07, 0xd7b, 0xd21, 0xd4d, 0xd2f, 0xd3e, 0xd28, 0xd3e, 0xd2a, 0xd4b, 0xd32,
0xd40, 0xd38, 0xd4d, 0xd28, 0xd4b, 0xd15, 0xd4d, 0xd38, 0xd4d, 0x2c, 0x20, 0xd07,
0xd7b, 0xd21, 0xd4d, 0xd2f, 0xd3e, 0xd28, 0xd2e, 0xd30, 0xd46, 0xd19, 0xd4d, 0xd15,
0xd4b, 0x2c, 0x20, 0xd07, 0xd7b, 0xd21, 0xd4d, 0xd2f, 0xd3e, 0xd28, 0xd2a, 0xd40,
0xd31, 0xd4d, 0xd31, 0xd47, 0xd34, 0xd4d, 0xd38, 0xd4d, 0xd2c, 0xd7c, 0xd17, 0xd4d,
0x2c, 0x20, 0xd07, 0xd7b, 0xd21, 0xd4d, 0xd2f, 0xd3e, 0xd28, 0xd31, 0xd4d, 0xd31,
0xd46, 0xd7d, 0x20, 0xd38, 0xd3f, 0xd31, 0xd4d, 0xd31, 0xd3f, 0x2c, 0x20, 0xd07,
0xd7b, 0xd21, 0xd4d, 0xd2f, 0xd3e, 0xd28, 0xd35, 0xd3f, 0xd35, 0xd46, 0xd2f, 0xd4d,
0x2c, 0x20, 0xd07, 0xd7b, 0xd21, 0xd4d, 0xd2f, 0xd3e, 0xd28, 0xd35, 0xd3f, 0xd7b,
0xd38, 0xd46, 0xd7b, 0xd38, 0xd4d, 0x2c, 0x20, 0xd07, 0xd7b, 0xd21, 0xd4d, 0xd2f,
0xd3e, 0xd28, 0xd35, 0xd3f, 0xd28, 0xd3e, 0xd2e, 0xd3e, 0xd15, 0xd4d, 0x2c, 0x20,
0xd07, 0xd7b, 0xd21, 0xd4d, 0xd2f, 0xd3e, 0xd28, 0xd07, 0xd28, 0xd41, 0xd35, 0xd3f,
0xd15, 0xd4d, 0xd07, 0xd16, 0xd3e, 0xd32, 0xd3f, 0xd24, 0xd4d, 0xd1c, 0xd2e, 0xd48,
0xd15, 0xd4d, 0xd15, 0xd1c, 0xd42, 0xd28, 0xd4b, 0xd32, 0xd42, 0xd2f, 0xd3f, 0xd38,
0xd4d, 0x200c, 0xd35, 0xd3f, 0xd32, 0xd4d, 0xd32, 0xd46, 0xd2e, 0xd4b, 0xd23, 0xd4d,
0xd1f, 0xd3f, 0xd38, 0xd46, 0xd32, 0xd4d, 0xd32, 0xd4b, 0x2c, 0x20, 0xd15, 0xd46,
0xd28, 0xd4d, 0xd31, 0xd15, 0xd4d, 0xd15, 0xd3f, 0xd15, 0xd3e, 0xd7c, 0xd32, 0xd7b,
0xd21, 0xd3f, 0xd1c, 0xd46, 0xd15, 0xd4d, 0xd15, 0xd4d, 0xd32, 0xd3e, 0x20, 0xd2a,
0xd3e, 0xd38, 0xd4d, 0xd32, 0xd3f, 0xd2e, 0xd32, 0xd4b, 0xd38, 0xd4d, 0x20, 0xd0e,
0xd2f, 0xd4d, 0xd1e, 0xd4d, 0xd1a, 0xd32, 0xd38, 0xd4d, 0xd32, 0xd4b, 0xd35, 0xd7c,
0x20, 0xd2a, 0xd4d, 0xd30, 0xd3f, 0xd7b, 0xd38, 0xd38, 0xd4d, 0x20, 0xd15, 0xd4d,
0xd35, 0xd3e, 0xd7c, 0xd1f, 0xd4d, 0xd1f, 0xd7c, 0xd2e, 0xd3e, 0xd38, 0xd3f, 0xd2f,
0xd4b, 0xd2e, 0xd28, 0xd3e, 0xd17, 0xd4d, 0xd35, 0xd2e, 0xd28, 0xd57, 0xd38, 0xd4d,
0xd2e, 0xd3e, 0xd30, 0xd3f, 0xd17, 0xd4b, 0xd2e, 0xd3e, 0xd7c, 0xd1f, 0xd4d, 0xd1f,
0xd3f, 0xd28, 0xd3f, 0xd15, 0xd4d, 0xd2e, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd2e, 0xd4a,
0xd31, 0xd4b, 0xd38, 0xd4d, 0xd2e, 0xd38, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x200c, 0xd32,
0xd3e, 0xd7b, 0xd2e, 0xd46, 0xd28, 0xd4b, 0xd2e, 0xd3f, 0xd28, 0xd40, 0xd2e, 0xd46,
0xd30, 0xd3f, 0xd21, 0xd2e, 0xd46, 0xd21, 0xd4d, 0x200c, 0xd32, 0xd15, 0xd1f, 0xd4d,
0x200c, 0xd32, 0xd46, 0xd2e, 0xd46, 0xd15, 0xd4d, 0xd38, 0xd3f, 0xd15, 0xd4d, 0xd15,
0xd4b, 0x20, 0xd38, 0xd3f, 0xd31, 0xd4d, 0xd31, 0xd3f, 0xd2e, 0xd3f, 0xd15, 0xd4d,
0xd35, 0xd32, 0xd7b, 0xd2e, 0xd4b, 0xd02, 0xd17, 0xd4d, 0x200c, 0xd1f, 0xd7b, 0xd2e,
0xd4b, 0xd23, 0xd4d, 0xd1f, 0xd46, 0xd31, 0xd3f, 0xd2e, 0xd4a, 0xd23, 0xd4d, 0xd1f,
0xd46, 0x200c, 0xd35, 0xd40, 0xd21, 0xd3f, 0xd2f, 0xd4b, 0xd2e, 0xd4a, 0xd23, 0xd4d,
0xd1f, 0xd46, 0xd38, 0xd30, 0xd24, 0xd4d, 0xd24, 0xd4d, 0xd28, 0xd3e, 0xd38, 0xd57,
0xd28, 0xd4d, 0xd2f, 0xd42, 0xd2f, 0xd4b, 0xd7c, 0xd15, 0xd4d, 0xd15, 0xd4d, 0xd28,
0xd4b, 0xd02, 0xd28, 0xd4a, 0xd31, 0xd4b, 0xd28, 0xd2c, 0xd4d, 0xd2f, 0xd42, 0xd32,
0x2c, 0x20, 0xd35, 0xd1f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd21, 0xd46, 0xd15,
0xd4d, 0xd15, 0xd4b, 0xd1f, 0xd4d, 0xd1f, 0xd38, 0xd46, 0xd28, 0xd4d, 0xd31, 0xd7c,
0x2c, 0x20, 0xd35, 0xd1f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd21, 0xd46, 0xd15,
0xd4d, 0xd15, 0xd4b, 0xd1f, 0xd4d, 0xd1f, 0xd28, 0xd4d, 0xd2f, 0xd42, 0x20, 0xd38,
0xd46, 0xd2f, 0xd4d, 0x200c, 0xd32, 0xd02, 0x2c, 0x20, 0xd35, 0xd1f, 0xd15, 0xd4d,
0xd15, 0xd7b, 0x20, 0xd21, 0xd46, 0xd15, 0xd4d, 0xd15, 0xd4b, 0xd1f, 0xd4d, 0xd1f,
0xd28, 0xd42, 0xd15, 0xd4d, 0xd15, 0xd4d, 0xd12, 0xd1c, 0xd3f, 0xd7b, 0xd17, 0xd2a,
0xd28, 0xd3e, 0xd2e, 0xd2a, 0xd30, 0xd3e, 0xd2e, 0xd3e, 0xd30, 0xd3f, 0xd2c, 0xd4b,
0xd2b, 0xd40, 0xd28, 0xd3f, 0xd15, 0xd4d, 0xd38, 0xd4d, 0xd2a, 0xd4b, 0xd7c, 0xd1f,
0xd4d, 0xd1f, 0xd4b, 0xd2a, 0xd4d, 0xd30, 0xd3f, 0xd7b, 0xd38, 0xd4d, 0xd2a, 0xd4b,
0xd7c, 0x200c, 0xd1f, 0xd4d, 0xd1f, 0xd4d, 0x20, 0xd13, 0xd2b, 0xd4d, 0x20, 0xd38,
0xd4d, 0x200c, 0xd2a, 0xd46, 0xd2f, 0xd3f, 0xd7b, 0x200c, 0xd2a, 0xd4b, 0xd7c, 0xd1f,
0xd4d, 0xd1f, 0xd4b, 0x20, 0xd35, 0xd46, 0xd32, 0xd4d, 0xd32, 0xd4b, 0xd2a, 0xd4d,
0xd2f, 0xd42, 0xd7c, 0xd1f, 0xd4d, 0xd1f, 0xd4b, 0x20, 0xd31, 0xd3f, 0xd15, 0xd4d,
0xd15, 0xd4b, 0xd2a, 0xd41, 0xd28, 0xd4d, 0xd31, 0x20, 0xd05, 0xd30, 0xd40, 0xd28,
0xd38, 0xd4d, 0xd31, 0xd3e, 0xd19, 0xd4d, 0xd15, 0xd3f, 0xd7b, 0x20, 0xd07, 0xd7b,
0xd32, 0xd46, 0xd31, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd46, 0xd38, 0xd40, 0xd2b, 0xd46,
0xd31, 0xd3f, 0xd1c, 0xd40, 0xd28, 0xd31, 0xd46, 0xd38, 0xd32, 0xd4d, 0xd2f, 0xd42,
0xd1f, 0xd4d, 0xd1f, 0xd4d, 0xd31, 0xd3f, 0xd2f, 0xd4b, 0x20, 0xd2c, 0xd4d, 0xd30,
0xd3e, 0xd19, 0xd4d, 0xd15, 0xd4b, 0xd38, 0xd3e, 0xd28, 0xd4d, 0xd31, 0xd31, 0xd46,
0xd02, 0xd38, 0xd3e, 0xd28, 0xd4d, 0xd31, 0xd3f, 0xd2f, 0xd3e, 0xd17, 0xd4b, 0xd38,
0xd3e, 0xd28, 0xd4d, 0xd24, 0xd4b, 0x20, 0xd21, 0xd4b, 0xd2e, 0xd3f, 0xd02, 0xd17,
0xd4b, 0xd38, 0xd3e, 0xd35, 0xd4b, 0xd2a, 0xd4b, 0xd33, 0xd4b, 0xd07, 0xd31, 0xd4d,
0xd31, 0xd4d, 0xd35, 0xd3e, 0xd16, 0xd4d, 0xd35, 0xd3e, 0xd7c, 0xd1f, 0xd42, 0xd7c,
0xd2e, 0xd3f, 0xd31, 0xd4d, 0xd31, 0xd4d, 0xd38, 0xd3f, 0xd31, 0xd4d, 0xd31, 0xd4d,
0x200c, 0xd15, 0xd3e, 0xd38, 0xd46, 0xd28, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd2c, 0xd7c,
0xd24, 0xd4d, 0xd24, 0xd32, 0xd46, 0xd2e, 0xd3f, 0xd38, 0xd46, 0xd28, 0xd4d, 0xd31,
0xd4d, 0x20, 0xd1c, 0xd4b, 0xd7a, 0xd38, 0xd4d, 0xd38, 0xd46, 0xd28, 0xd4d, 0xd31,
0xd4d, 0x20, 0xd15, 0xd3f, 0xd31, 0xd4d, 0xd31, 0xd4d, 0xd38, 0xd4d, 0xd38, 0xd46,
0xd28, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd32, 0xd42, 0xd38, 0xd3f, 0xd2f, 0xd38, 0xd46,
0xd28, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd24, 0xd4b, 0xd2e, 0xd38, 0xd4d, 0xd38, 0xd46,
0xd28, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd35, 0xd3f, 0xd7b, 0xd38, 0xd46, 0xd28, 0xd4d,
0xd31, 0xd4d, 0xd38, 0xd4d, 0xd35, 0xd3f, 0xd2b, 0xd4d, 0x200c, 0xd31, 0xd4d, 0xd31,
0xd4d, 0x20, 0xd15, 0xd31, 0xd28, 0xd4d, 0xd31, 0xd4d, 0xd1f, 0xd46, 0xd17, 0xd42,
0xd38, 0xd3f, 0xd17, 0xd3e, 0xd7d, 0xd2a, 0xd24, 0xd42, 0xd33, 0xd3f, 0xd24, 0xd3f,
0xd2f, 0xd41, 0xd35, 0xd3e, 0xd28, 0xd1f, 0xd4a, 0xd31, 0xd28, 0xd4d, 0xd31, 0xd4b,
0xd1f, 0xd4b, 0xd7c, 0x200c, 0xd1f, 0xd4d, 0xd1f, 0xd4b, 0xd33, 0xd35, 0xd3e, 0xd7b,
0x200c, 0xd15, 0xd42, 0xd35, 0xd7c, 0xd35, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0xd39,
0xd4b, 0xd34, 0xd4d, 0xd38, 0xd4d, 0xd35, 0xd3f, 0xd28, 0xd4d, 0xd28, 0xd3f, 0xd2a,
0xd46, 0xd17, 0xd4d, 0xd2f, 0xd3e, 0xd15, 0xd41, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd31,
0xd4d, 0xd31, 0xd4d, 0xd21, 0xd46, 0xd2f, 0xd4d, 0x200c, 0xd35, 0xd3f, 0xd38, 0xd4d,
0xd21, 0xd4d, 0xd2f, 0xd42, 0xd2e, 0xd23, 0xd4d, 0xd1f, 0xd4d, 0x20, 0xd21, 0xd3f,
0x20, 0xd2f, 0xd41, 0xd7c, 0xd35, 0xd3f, 0xd7d, 0xd2e, 0xd15, 0xd4d, 0xd35, 0xd2f,
0xd31, 0xd3f, 0xd2e, 0xd3e, 0xd35, 0xd4d, 0xd38, 0xd7a, 0xd2e, 0xd3e, 0xd15, 0xd4d,
0xd2e, 0xd7c, 0xd21, 0xd4b, 0xd2a, 0xd3e, 0xd2e, 0xd7c, 0xd31, 0xd4a, 0xd24, 0xd40,
0xd31, 0xd38, 0xd4d, 0xd2f, 0xd4b, 0xd35, 0xd1f, 0xd4d, 0xd30, 0xd4b, 0xd7e, 0xd35,
0xd4b, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd4b, 0xd15, 0xd4d, 0xd32, 0xd02, 0xd17,
0xd4d, 0x200d, 0xd2f, 0xd46, 0xd7c, 0xd2c, 0xd3f, 0xd7b, 0xd0f, 0xd26, 0xd46, 0xd7b,
0xd05, 0xd7d, 0xd2e, 0xd3e, 0xd1f, 0xd4d, 0xd1f, 0xd3f, 0xd05, 0xd2e, 0xd4d, 0xd2e,
0xd3e, 0xd7b, 0x200c, 0xd05, 0xd28, 0xd3e, 0xd21, 0xd3f, 0xd7c, 0xd05, 0xd15, 0xd4d,
0xd24, 0xd57, 0xd05, 0xd16, 0xd4d, 0x200c, 0xd24, 0xd4b, 0xd2c, 0xd4d, 0xd06, 0xd37,
0xd4d, 0x200c, 0xd17, 0xd3e, 0xd2c, 0xd1f, 0xd4d, 0xd1f, 0xd4d, 0xd05, 0xd31, 0xd4d,
0xd31, 0xd3f, 0xd31, 0xd4b, 0xd2c, 0xd3e, 0xd17, 0xd4d, 0x200c, 0xd26, 0xd3e, 0xd26,
0xd4d, 0xd2c, 0xd39, 0xd4d, 0xd31, 0xd3f, 0xd7b, 0xd2c, 0xd3e, 0xd15, 0xd4d, 0xd15,
0xd41, 0xd2c, 0xd3e, 0xd19, 0xd4d, 0xd15, 0xd4b, 0xd15, 0xd4d, 0xd15, 0xd4d, 0xd2c,
0xd7c, 0xd23, 0xd4b, 0xd7d, 0xd2c, 0xd46, 0xd2f, 0xd4d, 0x200c, 0xd31, 0xd42, 0xd1f,
0xd4d, 0xd1f, 0xd4d, 0xd2c, 0xd3f, 0xd37, 0xd4d, 0x200c, 0xd15, 0xd47, 0xd15, 0xd4d,
0xd2c, 0xd4d, 0xd30, 0xd42, 0xd23, 0xd48, 0xd1a, 0xd40, 0xd31, 0xd4d, 0xd31, 0xd15,
0xd4a, 0xd33, 0xd02, 0x200c, 0xd2c, 0xd4b, 0xd26, 0xd2e, 0xd3e, 0xd38, 0xd4d, 0xd15,
0xd38, 0xd4d, 0xd27, 0xd3e, 0xd15, 0xd4d, 0xd15, 0xd26, 0xd3f, 0xd32, 0xd3f, 0xd26,
0xd41, 0xd2c, 0xd3e, 0xd2f, 0xd4d, 0xd26, 0xd41, 0xd37, 0xd7b, 0x200c, 0xd2c, 0xd46,
0xd2b, 0xd3e, 0xd2e, 0xd17, 0xd41, 0xd38, 0xd4d, 0x200c, 0xd31, 0xd4d, 0xd31, 0xd17,
0xd3e, 0xd38, 0xd39, 0xd46, 0xd2c, 0xd4d, 0x200c, 0xd31, 0xd4b, 0xd7a, 0xd39, 0xd4b,
0x20, 0xd1a, 0xd3f, 0x20, 0xd2e, 0xd3f, 0xd7b, 0x20, 0xd38, 0xd3f, 0xd31, 0xd4d,
0xd31, 0xd3f, 0xd39, 0xd4b, 0xd19, 0xd4d, 0xd15, 0xd4b, 0xd02, 0xd17, 0xd4d, 0xd39,
0xd4b, 0xd21, 0xd4d, 0xd07, 0xd7c, 0xd15, 0xd38, 0xd4d, 0xd15, 0xd4d, 0xd1c, 0xd15,
0xd4d, 0xd15, 0xd3e, 0xd7c, 0xd24, 0xd4d, 0xd24, 0xd1c, 0xd2f, 0xd2a, 0xd41, 0xd30,
0xd1c, 0xd46, 0xd31, 0xd41, 0xd38, 0xd32, 0xd47, 0xd02, 0xd15, 0xd3e, 0xd2c, 0xd42,
0xd7e, 0xd15, 0xd3e, 0xd02, 0xd1a, 0xd1f, 0xd4d, 0xd15, 0xd15, 0xd31, 0xd3e, 0xd1a,
0xd4d, 0xd1a, 0xd3f, 0xd15, 0xd3e, 0xd20, 0xd4d, 0x200c, 0xd2e, 0xd23, 0xd4d, 0xd21,
0xd41, 0xd15, 0xd3e, 0xd7b, 0xd21, 0xd3f, 0xd17, 0xd15, 0xd4a, 0xd7d, 0x200c, 0xd15,
0xd4d, 0xd15, 0xd24, 0xd4d, 0xd24, 0xd15, 0xd4d, 0xd30, 0xd3e, 0xd38, 0xd4d, 0xd28,
0xd4b, 0xd2f, 0xd3e, 0xd38, 0xd4d, 0xd15, 0xd4d, 0xd15, 0xd4d, 0xd35, 0xd3e, 0xd32,
0xd32, 0xd02, 0x200c, 0xd2a, 0xd42, 0xd7c, 0x200c, 0x200c, 0xd15, 0xd41, 0xd1a, 0xd3f,
0xd19, 0xd4d, 0xd15, 0xd41, 0xd35, 0xd48, 0xd24, 0xd4d, 0xd24, 0xd4d, 0xd2e, 0xd15,
0xd4d, 0xd15, 0xd3e, 0xd35, 0xd41, 0xd2e, 0xd17, 0xd21, 0xd3e, 0xd7b, 0xd2e, 0xd15,
0xd38, 0xd4d, 0xd38, 0xd7c, 0xd2e, 0xd28, 0xd3f, 0xd32, 0xd2e, 0xd38, 0xd4d, 0xd15,
0xd31, 0xd4d, 0xd31, 0xd4d, 0xd28, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd4b, 0xd37, 0xd4d,
0xd2f, 0xd28, 0xd4b, 0xd35, 0xd4b, 0xd15, 0xd41, 0xd38, 0xd46, 0xd7b, 0xd38, 0xd4d,
0x200c, 0xd15, 0xd4d, 0xd28, 0xd4a, 0xd35, 0xd4b, 0xd38, 0xd3f, 0xd2c, 0xd3f, 0xd7c,
0xd38, 0xd4d, 0xd15, 0xd4d, 0xd12, 0xd02, 0xd38, 0xd4d, 0xd15, 0xd4d, 0xd13, 0xd31,
0xd7d, 0xd2b, 0xd46, 0xd28, 0xd4b, 0xd02, 0x20, 0xd2a, 0xd46, 0xd7b, 0xd2a, 0xd4a,
0xd28, 0xd4d, 0xd31, 0xd3f, 0xd2f, 0xd3e, 0xd28, 0xd15, 0xd4d, 0xd2a, 0xd4d, 0xd2f,
0xd4b, 0xd02, 0xd17, 0xd4d, 0x200c, 0xd2f, 0xd3e, 0xd02, 0xd17, 0xd4d, 0xd16, 0xd24,
0xd4d, 0xd24, 0xd7c, 0xd15, 0xd4b, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd28, 0xd47,
0xd16, 0xd3f, 0xd38, 0xd3f, 0xd32, 0xd4b, 0xd7c, 0xd21, 0xd31, 0xd3f, 0xd2f, 0xd3e,
0xd26, 0xd4d, 0xd38, 0xd16, 0xd3e, 0xd32, 0xd3f, 0xd7b, 0xd38, 0xd2e, 0xd7c, 0xd15,
0xd4d, 0xd15, 0xd28, 0xd4d, 0xd26, 0xd4d, 0xd38, 0xd4b, 0xd7e, 0xd37, 0xd3e, 0xd19,
0xd4d, 0x200c, 0xd39, 0xd3e, 0xd2f, 0xd3f, 0xd38, 0xd3f, 0xd02, 0xd17, 0xd2a, 0xd4d,
0xd2a, 0xd42, 0xd7c, 0xd38, 0xd4d, 0xd30, 0xd3f, 0xd21, 0xd4d, 0x200c, 0xd28, 0xd3f,
0xd15, 0xd4b, 0xd33, 0xd3f, 0xd02, 0xd38, 0xd4d, 0xd15, 0xd4d, 0xd24, 0xd3e, 0xd2f,
0xd4d, 0x200c, 0xd2a, 0xd46, 0xd2f, 0xd4d, 0xd24, 0xd3e, 0xd37, 0xd4d, 0x200c, 0xd15,
0xd4d, 0xd15, 0xd28, 0xd4d, 0xd31, 0xd4d, 0xd24, 0xd3f, 0xd2c, 0xd3f, 0xd32, 0xd3f,
0xd38, 0xd3f, 0xd1f, 0xd46, 0xd39, 0xd4d, 0x200c, 0xd31, 0xd3e, 0xd7b, 0x200c, 0xd24,
0xd3f, 0xd02, 0xd2b, 0xd41, 0xd1f, 0xd4b, 0xd15, 0xd4d, 0xd15, 0xd3f, 0xd2f, 0xd4b,
0xd1f, 0xd4b, 0xd02, 0xd38, 0xd4d, 0xd15, 0xd4d, 0xd09, 0xd32, 0xd3e, 0xd7b, 0xd2c,
0xd3e, 0xd24, 0xd4d, 0xd24, 0xd7c, 0xd09, 0xd31, 0xd41, 0xd02, 0xd16, 0xd3f, 0xd2f,
0xd41, 0xd38, 0xd4d, 0x2d, 0xd28, 0xd47, 0xd30, 0xd35, 0xd46, 0xd28, 0xd4d, 0xd31,
0xd3f, 0xd2f, 0xd3e, 0xd7b, 0xd35, 0xd4d, 0xd33, 0xd3e, 0xd21, 0xd3f, 0xd35, 0xd4b,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd4b, 0xd15, 0xd4d, 0xd2f, 0xd3e, 0xd15, 0xd4d,
0xd15, 0xd31, 0xd4d, 0xd31, 0xd4d, 0xd38, 0xd4d, 0x200c, 0xd15, 0xd4d, 0xd31, 0xd19,
0xd4d, 0xd15, 0xd42, 0xd7a, 0x200c, 0xd2f, 0xd3e, 0xd15, 0xd3e, 0xd31, 0xd4d, 0xd31,
0xd46, 0xd31, 0xd3f, 0xd7b, 0xd2c, 0xd7c, 0xd17, 0xd4d, 0xd2f, 0xd47, 0xd30, 0xd35,
0xd7b, 0x200c, 0xd05, 0xd38, 0xd4b, 0xd31, 0xd38, 0xd4d, 0xd2c, 0xd7c, 0xd2e, 0xd41,
0xd21, 0xd15, 0xd4d, 0xd2f, 0xd3e, 0xd28, 0xd31, 0xd3f, 0xd15, 0xd47, 0xd2a, 0xd4d,
0x20, 0xd35, 0xd46, 0xd7c, 0xd26, 0xd46, 0xd2b, 0xd46, 0xd31, 0xd4b, 0xd2e, 0xd21,
0xd46, 0xd2f, 0xd4d, 0xd31, 0xd31, 0xd47, 0xd2f, 0xd4d, 0x200c, 0xd1c, 0xd3e, 0xd35,
0xd3f, 0xd15, 0xd4d, 0xd26, 0xd15, 0xd4d, 0xd37, 0xd3f, 0xd23, 0x20, 0xd1c, 0xd4b,
0xd7c, 0xd1c, 0xd3f, 0xd2f, 0xd38, 0xd46, 0xd28, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd39,
0xd46, 0xd32, 0xd46, 0xd28, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0x200c,
0xd32, 0xd3f, 0xd05, 0xd21, 0xd3f, 0xd32, 0xd46, 0xd2f, 0xd4d, 0x200c, 0xd21, 0xd4d,
0xd2c, 0xd4d, 0xd30, 0xd3f, 0xd38, 0xd4d, 0x200c, 0xd2c, 0xd46, 0xd2f, 0xd3f, 0xd7b,
0xd2c, 0xd4d, 0xd30, 0xd4b, 0xd15, 0xd4d, 0xd15, 0xd7a, 0x20, 0xd39, 0xd3f, 0xd7d,
0xd21, 0xd3e, 0xd7c, 0xd35, 0xd3f, 0xd7b, 0xd2f, 0xd42, 0xd15, 0xd4d, 0xd32, 0xd39,
0xd4a, 0xd2c, 0xd3e, 0xd7c, 0xd1f, 0xd4d, 0xd1f, 0xd4d, 0xd32, 0xd3f, 0xd7b, 0xd21,
0xd46, 0xd2e, 0xd3e, 0xd7b, 0xd32, 0xd4b, 0xd21, 0xd4d, 0x20, 0xd39, 0xd4b, 0xd35,
0xd4d, 0xd2e, 0xd46, 0xd7d, 0xd2c, 0xd7a, 0xd2a, 0xd46, 0xd7c, 0xd24, 0xd4d, 0xd24,
0xd4d, 0xd38, 0xd3f, 0xd21, 0xd4d, 0xd28, 0xd3f, 0xd06, 0xd02, 0x200c, 0xd38, 0xd4d,
0xd31, 0xd4d, 0xd31, 0xd7c, 0x200c, 0xd21, 0xd3e, 0xd02, 0xd05, 0xd23, 0xd4d, 0xd1f,
0xd4b, 0xd31, 0xd05, 0xd38, 0xd4d, 0x200c, 0xd1f, 0xd4d, 0xd30, 0xd16, 0xd3e, 0xd7b,
0xd0f, 0xd24, 0xd7b, 0x200c, 0xd38, 0xd4d, 0xd2c, 0xd46, 0xd7d, 0xd17, 0xd4d, 0xd30,
0xd47, 0xd21, 0xd4d, 0xd2c, 0xd46, 0xd7c, 0x200c, 0xd32, 0xd3f, 0xd7b, 0x200c, 0xd2c,
0xd4d, 0xd30, 0xd3e, 0xd1f, 0xd4d, 0xd1f, 0xd3f, 0xd38, 0xd4d, 0x200d, 0xd32, 0xd3e,
0xd35, 0xd2c, 0xd4d, 0xd30, 0xd38, 0xd4d, 0xd38, 0xd7d, 0x200c, 0xd38, 0xd4d, 0xd2c,
0xd41, 0xd1a, 0xd4d, 0xd1a, 0xd3e, 0xd31, 0xd46, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd4d, 0xd2c, 0xd41, 0xd21, 0xd3e, 0xd2a, 0xd46, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd4d, 0xd2c, 0xd41, 0xd38, 0xd3f, 0xd7b, 0xd1c, 0xd7b, 0xd1a, 0xd3f, 0xd38, 0xd3f,
0xd28, 0xd3e, 0xd35, 0xd41, 0xd15, 0xd4b, 0xd2a, 0xd4d, 0xd2a, 0xd7b, 0x200c, 0xd39,
0xd47, 0xd17, 0xd7b, 0x200c, 0xd21, 0xd2c, 0xd4d, 0xd32, 0xd3f, 0xd7b, 0xd1c, 0xd3f,
0xd2c, 0xd4d, 0xd30, 0xd3e, 0xd7e, 0xd1f, 0xd4d, 0xd1f, 0xd7c, 0xd17, 0xd47, 0xd7a,
0xd38, 0xd47, 0xd39, 0xd46, 0xd7d, 0x200c, 0xd38, 0xd3f, 0xd19, 0xd4d, 0xd15, 0xd3f,
0xd10, 0xd7d, 0x200c, 0x20, 0xd13, 0xd2b, 0xd4d, 0x20, 0xd2e, 0xd3e, 0xd7b, 0x200c,
0xd07, 0xd38, 0xd4d, 0xd24, 0xd3e, 0xd02, 0x200c, 0xd2c, 0xd41, 0xd7e, 0x200c, 0xd1c,
0xd47, 0xd34, 0xd4d, 0x200c, 0xd38, 0xd3f, 0xd15, 0xd32, 0xd3f, 0xd28, 0xd3f, 0xd7b,
0xd17, 0xd4d, 0xd30, 0xd3e, 0xd21, 0xd4d, 0xd15, 0xd3f, 0xd31, 0xd4b, 0xd15, 0xd40,
0xd35, 0xd4d, 0xd32, 0xd3f, 0xd38, 0xd4d, 0x200c, 0xd2c, 0xd7a, 0x200c, 0xd32, 0xd41,
0xd2c, 0xd4d, 0x200d, 0xd32, 0xd3f, 0xd2f, 0xd3e, 0xd28, 0xd32, 0xd23, 0xd4d, 0xd1f,
0xd7b, 0x200c, 0xd32, 0xd15, 0xd4d, 0x200c, 0xd38, 0xd02, 0xd2c, 0xd7c, 0xd17, 0xd4d,
0xd2e, 0xd3e, 0xd21, 0xd4d, 0xd30, 0xd3f, 0xd21, 0xd4d, 0xd2e, 0xd3e, 0xd7e, 0xd1f,
0xd4d, 0xd1f, 0xd2e, 0xd30, 0xd3f, 0xd2f, 0xd3e, 0xd39, 0xd3e, 0xd2e, 0xd7b, 0xd2e,
0xd3f, 0xd7b, 0x200c, 0xd38, 0xd4d, 0xd15, 0xd4d, 0xd2e, 0xd4a, 0xd23, 0xd3e, 0xd15,
0xd4d, 0xd15, 0xd4b, 0xd2e, 0xd4b, 0xd38, 0xd4d, 0xd15, 0xd4b, 0xd13, 0xd38, 0xd4d,
0xd32, 0xd4b, 0xd2a, 0xd3e, 0xd30, 0xd40, 0xd38, 0xd4d, 0xd2a, 0xd4a, 0xd21, 0xd4d,
0x200d, 0xd17, 0xd4a, 0xd31, 0xd3f, 0xd38, 0xd2a, 0xd4d, 0xd30, 0xd3e, 0xd17, 0xd4d,
0xd31, 0xd4b, 0xd02, 0xd38, 0xd2e, 0xd3e, 0xd31, 0xd38, 0xd3e, 0xd7b, 0x20, 0xd2e,
0xd3e, 0xd30, 0xd3f, 0xd28, 0xd4b, 0xd38, 0xd30, 0xd2f, 0xd47, 0xd35, 0xd4b, 0xd38,
0xd30, 0xd3e, 0xd31, 0xd4d, 0xd31, 0xd4b, 0xd35, 0xd4d, 0xd38, 0xd3f, 0xd02, 0xd2b,
0xd46, 0xd31, 0xd4b, 0xd2a, 0xd4b, 0xd7e, 0xd38, 0xd4d, 0xd15, 0xd4b, 0xd2a, 0xd4d,
0xd2a, 0xd3f, 0xd2f, 0xd46, 0xd38, 0xd4b, 0xd2b, 0xd3f, 0xd2f, 0xd38, 0xd4d, 0xd31,
0xd4d, 0xd31, 0xd4b, 0xd15, 0xd4d, 0xd15, 0xd4d, 0xd39, 0xd4b, 0xd02, 0xd1f, 0xd3e,
0xd32, 0xd3f, 0xd7b, 0x200c, 0xd1f, 0xd3f, 0xd30, 0xd3e, 0xd28, 0xd46, 0xd09, 0xd32,
0xd4d, 0xd32, 0xd4d, 0xd2f, 0xd3e, 0xd28, 0xd4b, 0xd35, 0xd4d, 0x200c, 0xd38, 0xd4d,
0x200c, 0xd15, 0xd4d, 0xd35, 0xd3e, 0xd26, 0xd41, 0xd38, 0xd4d, 0xd35, 0xd24, 0xd4d,
0xd24, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd3e, 0xd7b, 0xd35, 0xd3f, 0xd2f, 0xd28, 0xd4d,
0xd28, 0xd35, 0xd3f, 0xd7d, 0x200c, 0xd28, 0xd3f, 0xd2f, 0xd38, 0xd4d, 0xd35, 0xd4b,
0xd7e, 0xd17, 0xd4b, 0xd17, 0xd4d, 0xd30, 0xd3e, 0xd21, 0xd4d, 0xd35, 0xd3e, 0xd7c,
0xd38, 0xd4b, 0xd38, 0xd3e, 0xd15, 0xd4d, 0xd30, 0xd46, 0xd2c, 0xd4d, 0xd38, 0xd42,
0xd31, 0xd3f, 0xd1a, 0xd4d, 0xd1a, 0xd4d, 0xd05, 0xd7b, 0xd31, 0xd3e, 0xd28, 0xd28,
0xd31, 0xd3f, 0xd35, 0xd4b, 0xd1a, 0xd3e, 0xd17, 0xd4b, 0xd38, 0xd4d, 0xd15, 0xd4d,
0xd30, 0xd3f, 0xd38, 0xd4d, 0xd24, 0xd41, 0xd2e, 0xd38, 0xd4d, 0xd15, 0xd4b, 0xd15,
0xd4d, 0xd15, 0xd4b, 0xd38, 0xd4d, 0xd15, 0xd4a, 0xd2e, 0xd4b, 0xd31, 0xd4b, 0xd15,
0xd46, 0xd7c, 0xd17, 0xd41, 0xd32, 0xd46, 0xd7b, 0xd2e, 0xd3e, 0xd39, 0xd3f, 0xd2e,
0xd3e, 0xd32, 0xd3f, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd4d, 0xd2e, 0xd57, 0xd31,
0xd40, 0xd37, 0xd4d, 0xd2f, 0xd38, 0xd4d, 0xd2e, 0xd2f, 0xd4b, 0xd1f, 0xd4d, 0xd1f,
0xd3f, 0xd31, 0xd40, 0xd2f, 0xd42, 0xd23, 0xd3f, 0xd2f, 0xd7b, 0xd06, 0xd2a, 0xd3f,
0xd2f, 0xd13, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x200c, 0xd32, 0xd3e, 0xd28, 0xd4d, 0xd31,
0xd4d, 0xd2c, 0xd4b, 0xd17, 0xd7a, 0xd35, 0xd3f, 0xd32, 0xd4d, 0xd32, 0xd46, 0xd1a,
0xd3e, 0xd24, 0xd4d, 0xd24, 0xd02, 0xd1a, 0xd15, 0xd4d, 0xd08, 0xd38, 0xd4d, 0xd31,
0xd4d, 0xd31, 0xd7c, 0xd07, 0xd2b, 0xd3e, 0xd24, 0xd46, 0xd2b, 0xd15, 0xd4d, 0xd15,
0xd3e, 0xd35, 0xd4b, 0xd2b, 0xd4b, 0xd2b, 0xd3f, 0xd1c, 0xd3f, 0xd2b, 0xd41, 0xd23,
0xd3e, 0xd2b, 0xd41, 0xd1f, 0xd4d, 0xd1f, 0xd3f, 0xd17, 0xd3e, 0xd32, 0xd2a, 0xd4d,
0xd2a, 0xd3e, 0xd17, 0xd4b, 0xd38, 0xd4d, 0xd17, 0xd3e, 0xd2e, 0xd4d, 0xd2a, 0xd3f,
0xd2f, 0xd7c, 0xd17, 0xd4d, 0xd35, 0xd3e, 0xd21, 0xd7d, 0xd15, 0xd28, 0xd3e, 0xd7d,
0xd17, 0xd4d, 0xd35, 0xd3e, 0xd02, 0xd39, 0xd4b, 0xd23, 0xd32, 0xd42, 0xd32, 0xd41,
0xd0e, 0xd7b, 0xd21, 0xd2c, 0xd31, 0xd3f, 0xd15, 0xd3f, 0xd30, 0xd3f, 0xd31, 0xd4d,
0xd31, 0xd3f, 0xd2e, 0xd3e, 0xd31, 0xd4d, 0xd31, 0xd3f, 0xd15, 0xd4a, 0xd38, 0xd4d,
0xd30, 0xd47, 0xd15, 0xd4d, 0xd35, 0xd3e, 0xd1c, 0xd32, 0xd46, 0xd2f, 0xd4d, 0xd7b,
0xd2e, 0xd1c, 0xd42, 0xd31, 0xd4b, 0xd2e, 0xd3e, 0xd7c, 0xd15, 0xd4d, 0xd2f, 0xd41,
0xd38, 0xd3e, 0xd38, 0xd4d, 0xd2e, 0xd3f, 0xd21, 0xd4d, 0x200c, 0xd35, 0xd47, 0xd28,
0xd57, 0xd31, 0xd41, 0xd28, 0xd3f, 0xd2f, 0xd41, 0xd28, 0xd4b, 0xd7c, 0x200c, 0xd2b,
0xd4b, 0xd15, 0xd4d, 0xd15, 0xd4d, 0xd28, 0xd4b, 0xd2e, 0xd3f, 0xd2f, 0xd2a, 0xd3e,
0xd17, 0xd4b, 0x20, 0xd2a, 0xd3e, 0xd17, 0xd4b, 0xd2a, 0xd32, 0xd3e, 0xd35, 0xd41,
0xd2a, 0xd3f, 0xd31, 0xd4d, 0xd31, 0xd4d, 0xd15, 0xd2f, 0xd3f, 0xd7b, 0x200c, 0xd2a,
0xd4b, 0xd7a, 0xd2a, 0xd46, 0xd2a, 0xd4b, 0xd7c, 0xd1f, 0xd4d, 0xd1f, 0xd4d, 0x20,
0xd2e, 0xd4b, 0xd34, 0xd4d, 0x200c, 0xd38, 0xd4d, 0xd2c, 0xd48, 0xd31, 0xd3e, 0xd30,
0xd4b, 0xd1f, 0xd4b, 0xd02, 0xd17, 0xd38, 0xd46, 0xd2f, 0xd4d, 0x200c, 0xd2a, 0xd4d,
0xd2a, 0xd3e, 0xd7b, 0x200c, 0xd24, 0xd3e, 0xd39, 0xd3f, 0xd24, 0xd3f, 0xd24, 0xd30,
0xd3e, 0xd35, 0xd1f, 0xd4b, 0xd02, 0xd17, 0xd3e, 0xd1f, 0xd3e, 0xd2a, 0xd41, 0xd35,
0xd46, 0xd2f, 0xd4d, 0xd15, 0xd4d, 0xd35, 0xd3e, 0xd32, 0xd4d, 0xd32, 0xd3f, 0xd38,
0xd4d, 0x4b, 0x61, 0x68, 0x65, 0x72, 0x61, 0x68, 0x44, 0x61, 0x6d, 0x73,
0x79, 0x69, 0x6b, 0x42, 0x61, 0x69, 0x74, 0x75, 0x6c, 0x6d, 0x75, 0x71,
0x61, 0x64, 0x64, 0x69, 0x73, 0x633, 0x64a, 0x6a0, 0x627, 0x6a4, 0x648, 0x631,
0x627, 0x6c, 0x2d, 0x41, 0x6c, 0x121, 0x69, 0x65, 0x72, 0x69, 0x6c, 0x2d,
0x47, 0x77, 0x61, 0x74, 0x65, 0x6d, 0x61, 0x6c, 0x61, 0x69, 0x6c, 0x2d,
0x120, 0x61, 0x6d, 0x61, 0x6a, 0x6b, 0x61, 0x120, 0x65, 0x72, 0x75, 0x73,
0x61, 0x6c, 0x65, 0x6d, 0x6d, 0x69, 0x6c, 0x2d, 0x42, 0x65, 0x6c, 0x74,
0x20, 0x74, 0x61, 0x6c, 0x2d, 0x4b, 0x75, 0x77, 0x61, 0x6a, 0x74, 0x69,
0x6c, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x74, 0x61,
0x6e, 0x2d, 0x4e, 0x6f, 0x66, 0x73, 0x69, 0x6e, 0x68, 0x61, 0x72, 0x41,
0x74, 0x65, 0x6e, 0x69, 0x42, 0x72, 0x75, 0x73, 0x73, 0x65, 0x6c, 0x6c,
0x120, 0x69, 0x62, 0x69, 0x6c, 0x74, 0xe0, 0x4c, 0x69, 0x17c, 0x62, 0x6f,
0x6e, 0x61, 0x69, 0x6c, 0x2d, 0x4c, 0x75, 0x73, 0x73, 0x65, 0x6d, 0x62,
0x75, 0x72, 0x67, 0x75, 0x56, 0x61, 0x6c, 0x6c, 0x65, 0x74, 0x74, 0x61,
0x50, 0x61, 0x72, 0x69, 0x121, 0x69, 0x52, 0x75, 0x6d, 0x61, 0x53, 0x74,
0x6f, 0x6b, 0x6b, 0x6f, 0x6c, 0x6d, 0x61, 0x69, 0x6c, 0x2d, 0x42, 0x65,
0x6c, 0x74, 0x20, 0x74, 0x61, 0x6c, 0x2d, 0x56, 0x61, 0x74, 0x69, 0x6b,
0x61, 0x6e, 0x56, 0x6a, 0x65, 0x6e, 0x6e, 0x61, 0x56, 0x61, 0x72, 0x73,
0x61, 0x76, 0x6a, 0x61, 0x69, 0x6c, 0x2d, 0x4d, 0x61, 0x6c, 0x64, 0x69,
0x76, 0x69, 0x4e, 0x67, 0x101, 0x69, 0x72, 0x6f, 0x70, 0x69, 0x54, 0x69,
0x72, 0x69, 0x70, 0x6f, 0x72, 0x12b, 0x54, 0x16b, 0x6e, 0x12b, 0x68, 0x69,
0x54, 0x65, 0x20, 0x4d, 0x6f, 0x74, 0x75, 0x20, 0x4e, 0x65, 0x68, 0x65,
0x50, 0x61, 0x70, 0x61, 0x74, 0x6f, 0x68, 0x65, 0x50, 0x113, 0x72, 0x69,
0x68, 0x69, 0x4b, 0x65, 0x6d, 0x75, 0x72, 0x65, 0x74, 0x69, 0x20, 0x50,
0x65, 0x69, 0x4b, 0x101, 0x6d, 0x61, 0x6e, 0x61, 0x48, 0x69, 0x6b, 0x101,
0x6b, 0x6f, 0x4b, 0x6f, 0x74, 0x6f, 0x20, 0x52, 0x69, 0x6b, 0x61, 0x54,
0x6f, 0x6d, 0x69, 0x6e, 0x69, 0x6b, 0x61, 0x57, 0x68, 0x61, 0x6b, 0x61,
0x6f, 0x72, 0x61, 0x4b, 0x75, 0x69, 0x68, 0x69, 0x20, 0x50, 0x65, 0x69,
0x54, 0x101, 0x6b, 0x6f, 0x72, 0x75, 0x20, 0x4e, 0x75, 0x69, 0x4b, 0x65,
0x72, 0x65, 0x6e, 0x61, 0x74, 0x61, 0x4b, 0x75, 0x61, 0x74, 0x61, 0x6d,
0x101, 0x72, 0x61, 0x48, 0x61, 0x77, 0x68, 0x101, 0x6e, 0x61, 0x48, 0x65,
0x6d, 0x65, 0x69, 0x6b, 0x61, 0x4e, 0x67, 0x101, 0x20, 0x41, 0x6e, 0x61,
0x68, 0x65, 0x72, 0x61, 0x4d, 0x101, 0x74, 0x69, 0x6e, 0x69, 0x6b, 0x69,
0x4d, 0x113, 0x68, 0x69, 0x6b, 0x6f, 0x20, 0x54, 0x101, 0x6f, 0x6e, 0x65,
0x6e, 0x75, 0x69, 0x54, 0x65, 0x20, 0x100, 0x70, 0x6f, 0x72, 0x6f, 0x20,
0x4e, 0x75, 0x69, 0x50, 0x65, 0x74, 0x61, 0x20, 0x52, 0x69, 0x6b, 0x6f,
0x48, 0x61, 0x74, 0x6f, 0x20, 0x48, 0x6f, 0x6e, 0x65, 0x48, 0x61, 0x74,
0x6f, 0x20, 0x52, 0x75, 0x69, 0x68, 0x69, 0x61, 0x48, 0x61, 0x74, 0x6f,
0x20, 0x54, 0x61, 0x6d, 0x61, 0x74, 0x69, 0x48, 0x61, 0x74, 0x6f, 0x20,
0x57, 0x113, 0x6e, 0x65, 0x74, 0x69, 0x54, 0x101, 0x72, 0x6f, 0x74, 0x6f,
0x54, 0x65, 0x20, 0x57, 0x68, 0x61, 0x6e, 0x67, 0x61, 0x2d, 0x61, 0x2d,
0x4b, 0x69, 0x77, 0x61, 0x52, 0x113, 0x77, 0x65, 0x74, 0x69, 0x4d, 0x61,
0x6b, 0x6f, 0x61, 0x72, 0x65, 0x50, 0x101, 0x6b, 0x61, 0x74, 0x61, 0x74,
0x61, 0x50, 0x101, 0x72, 0x65, 0x69, 0x6e, 0x61, 0x50, 0x61, 0x6e, 0x67,
0x61, 0x6b, 0x6f, 0x6b, 0x6f, 0x50, 0x6f, 0x72, 0x6f, 0x6e, 0x61, 0x69,
0x54, 0x75, 0x70, 0x61, 0x65, 0x4b, 0x101, 0x68, 0x61, 0x48, 0x6f, 0x6e,
0x67, 0x69, 0x70, 0x75, 0x61, 0x54, 0x69, 0x61, 0x6b, 0x101, 0x74, 0x61,
0x48, 0x69, 0x72, 0x75, 0x68, 0x101, 0x72, 0x61, 0x6d, 0x61, 0x4b, 0x61,
0x74, 0x61, 0x6d, 0x61, 0x72, 0x16b, 0x4b, 0x75, 0x61, 0x72, 0x61, 0x20,
0x52, 0x16b, 0x70, 0x61, 0x4b, 0x16b, 0x77, 0x65, 0x69, 0x74, 0x69, 0x4d,
0x61, 0x6e, 0x69, 0x72, 0x61, 0x50, 0x65, 0x6e, 0x6f, 0x6d, 0x61, 0x20,
0x50, 0x65, 0x6e, 0x61, 0x4b, 0x61, 0x74, 0x101, 0x52, 0x69, 0x61, 0x74,
0x61, 0x48, 0x6f, 0x75, 0x72, 0x61, 0x48, 0x61, 0x6e, 0x67, 0x61, 0x68,
0x61, 0x69, 0x48, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x61, 0x54, 0x65,
0x72, 0x101, 0x6e, 0x61, 0x54, 0x14d, 0x6b, 0x69, 0x6f, 0x50, 0x101, 0x6d,
0x75, 0x72, 0x61, 0x54, 0x65, 0x20, 0x4b, 0x16b, 0x72, 0x61, 0x65, 0x20,
0x4d, 0x61, 0x74, 0x6f, 0x6d, 0x61, 0x74, 0x6f, 0x48, 0x14d, 0x72, 0x69,
0x61, 0x20, 0x6b, 0x69, 0x20, 0x54, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67,
0x61, 0x48, 0x61, 0x74, 0x6f, 0x20, 0x48, 0x113, 0x72, 0x65, 0x6e, 0x61,
0x41, 0x74, 0x69, 0x72, 0x65, 0x69, 0x72, 0x61, 0x50, 0x69, 0x72, 0x69,
0x70, 0x61, 0x6e, 0x65, 0x54, 0x101, 0x77, 0x69, 0x6e, 0x69, 0x48, 0x6f,
0x70, 0x61, 0x74, 0x101, 0x6f, 0x6e, 0x65, 0x50, 0x6f, 0x69, 0x70, 0x69,
0x72, 0x69, 0x70, 0x69, 0x50, 0x113, 0x74, 0x69, 0x61, 0x50, 0x6f, 0x69,
0x68, 0x101, 0x6b, 0x65, 0x6e, 0x61, 0x50, 0x101, 0x70, 0x75, 0x6e, 0x69,
0x101, 0x6d, 0x69, 0x74, 0x61, 0x41, 0x6e, 0x61, 0x74, 0x14d, 0x72, 0x61,
0x100, 0x74, 0x65, 0x6e, 0x65, 0x50, 0x65, 0x61, 0x72, 0x12b, 0x6e, 0x69,
0x50, 0x61, 0x72, 0x75, 0x68, 0x69, 0x50, 0x75, 0x74, 0x61, 0x70, 0x113,
0x68, 0x69, 0x4b, 0x6f, 0x70, 0x65, 0x68, 0x65, 0x69, 0x6b, 0x61, 0x6e,
0x61, 0x54, 0x61, 0x70, 0x75, 0x72, 0x69, 0x6e, 0x69, 0x48, 0x113, 0x72,
0x69, 0x6b, 0x69, 0x54, 0x65, 0x20, 0x4d, 0x6f, 0x75, 0x74, 0x65, 0x72,
0x65, 0x20, 0x4d, 0x61, 0x6e, 0x61, 0x49, 0x74, 0x61, 0x70, 0x16b, 0x72,
0x75, 0x52, 0x12b, 0x70, 0x65, 0x6e, 0x65, 0x52, 0x101, 0x6e, 0x61, 0x6e,
0x61, 0x52, 0x61, 0x6b, 0x61, 0x70, 0x75, 0x14d, 0x4d, 0x101, 0x74, 0x69,
0x72, 0x69, 0x4d, 0x101, 0x72, 0x61, 0x74, 0x61, 0x4d, 0x6f, 0x6e, 0x101,
0x6b, 0x6f, 0x4d, 0x6f, 0x68, 0x69, 0x6b, 0x61, 0x75, 0x14c, 0x68, 0x6f,
0x72, 0x6f, 0x50, 0x61, 0x72, 0x101, 0x6b, 0x61, 0x52, 0x14d, 0x6d, 0x61,
0x48, 0x61, 0x6e, 0x61, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6e, 0x6f, 0x54,
0x6f, 0x6b, 0x6f, 0x6f, 0x6d, 0x6f, 0x54, 0x65, 0x20, 0x50, 0x6f, 0x68,
0x6f, 0x2d, 0x6f, 0x2d, 0x50, 0x69, 0x74, 0x61, 0x57, 0x68, 0x69, 0x65,
0x6e, 0x61, 0x48, 0x16b, 0x72, 0x69, 0x6b, 0x61, 0x4b, 0x69, 0x72, 0x69,
0x68, 0x69, 0x6d, 0x65, 0x74, 0x65, 0x4b, 0x6f, 0x6b, 0x6f, 0x68, 0x69,
0x4d, 0x101, 0x72, 0x61, 0x74, 0x69, 0x72, 0x69, 0x4d, 0x61, 0x72, 0x69,
0x68, 0x69, 0x100, 0x70, 0x69, 0x61, 0x54, 0x101, 0x6d, 0x61, 0x6b, 0x69,
0x20, 0x4d, 0x61, 0x6b, 0x61, 0x75, 0x72, 0x61, 0x75, 0x52, 0x113, 0x6b,
0x6f, 0x68, 0x75, 0x57, 0x68, 0x12b, 0x74, 0x12b, 0x4b, 0x75, 0x61, 0x6d,
0x61, 0x4e, 0x14d, 0x70, 0x6f, 0x6b, 0x65, 0x4e, 0x16b, 0x6d, 0x65, 0x61,
0x50, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x50, 0x61, 0x6e, 0x67, 0x6f, 0x50,
0x101, 0x72, 0x61, 0x75, 0x50, 0x6f, 0x74, 0x61, 0x20, 0x4d, 0x6f, 0x61,
0x68, 0x69, 0x70, 0x69, 0x57, 0x101, 0x72, 0x69, 0x68, 0x69, 0x905, 0x92c,
0x93f, 0x926, 0x94d, 0x91c, 0x93e, 0x928, 0x905, 0x200d, 0x945, 0x915, 0x94d, 0x930,
0x93e, 0x906, 0x926, 0x93f, 0x938, 0x20, 0x905, 0x92c, 0x93e, 0x92c, 0x93e, 0x905,
0x932, 0x94d, 0x91c, 0x93f, 0x905, 0x930, 0x94d, 0x938, 0x90f, 0x938, 0x94d, 0x92e,
0x93e, 0x930, 0x93e, 0x92c, 0x93e, 0x92e, 0x915, 0x94b, 0x92c, 0x93e, 0x902, 0x917,
0x941, 0x908, 0x92c, 0x94d, 0x932, 0x901, 0x91f, 0x93e, 0x92f, 0x930, 0x92c, 0x94d,
0x930, 0x93e, 0x91d, 0x93e, 0x935, 0x94d, 0x939, 0x93f, 0x932, 0x947, 0x92c, 0x941,
0x91c, 0x941, 0x902, 0x92c, 0x941, 0x930, 0x93e, 0x915, 0x948, 0x930, 0x94b, 0x915,
0x945, 0x938, 0x93e, 0x92c, 0x94d, 0x932, 0x93e, 0x928, 0x94d, 0x915, 0x93e, 0x915,
0x94b, 0x928, 0x93e, 0x915, 0x94d, 0x930, 0x93f, 0x921, 0x915, 0x930, 0x926, 0x93e,
0x930, 0x20, 0x90f, 0x20, 0x938, 0x932, 0x93e, 0x92e, 0x926, 0x94b, 0x909, 0x906,
0x932, 0x93e, 0x90f, 0x932, 0x20, 0x910, 0x909, 0x928, 0x916, 0x93e, 0x930, 0x94d,
0x91f, 0x941, 0x92e, 0x915, 0x940, 0x917, 0x93e, 0x932, 0x940, 0x932, 0x93f, 0x92c,
0x930, 0x935, 0x94d, 0x939, 0x93f, 0x932, 0x932, 0x941, 0x92c, 0x902, 0x92c, 0x93e,
0x936, 0x940, 0x92e, 0x938, 0x947, 0x930, 0x941, 0x905, 0x902, 0x92c, 0x93e, 0x92c,
0x93e, 0x928, 0x947, 0x92e, 0x94b, 0x928, 0x930, 0x94b, 0x935, 0x94d, 0x939, 0x93f,
0x92f, 0x93e, 0x907, 0x902, 0x91c, 0x93e, 0x92e, 0x947, 0x928, 0x93e, 0x928, 0x941,
0x935, 0x93e, 0x915, 0x938, 0x941, 0x924, 0x935, 0x93e, 0x917, 0x921, 0x942, 0x917,
0x942, 0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x94b, 0x2d, 0x928, 0x94b, 0x935, 0x94d,
0x939, 0x94b, 0x938, 0x93e, 0x913, 0x20, 0x924, 0x94b, 0x92e, 0x947, 0x91f, 0x92f,
0x942, 0x928, 0x93f, 0x938, 0x935, 0x93f, 0x902, 0x921, 0x939, 0x94b, 0x90f, 0x915,
0x905, 0x921, 0x945, 0x915, 0x905, 0x901, 0x915, 0x930, 0x947, 0x91c, 0x905, 0x901,
0x917, 0x94d, 0x935, 0x93f, 0x932, 0x93e, 0x905, 0x901, 0x91f, 0x93f, 0x917, 0x94d,
0x935, 0x93e, 0x905, 0x930, 0x93e, 0x917, 0x93e, 0x92f, 0x928, 0x93e, 0x92c, 0x94d,
0x92f, 0x941, 0x928, 0x94b, 0x938, 0x20, 0x906, 0x92f, 0x930, 0x94d, 0x938, 0x915,
0x945, 0x91f, 0x93e, 0x92e, 0x93e, 0x930, 0x94d, 0x915, 0x93e, 0x915, 0x949, 0x930,
0x94d, 0x921, 0x94b, 0x92c, 0x93e, 0x92e, 0x947, 0x902, 0x926, 0x94b, 0x91d, 0x93e,
0x930, 0x93f, 0x92f, 0x94b, 0x20, 0x917, 0x945, 0x932, 0x947, 0x917, 0x949, 0x938,
0x938, 0x949, 0x932, 0x94d, 0x924, 0x93e, 0x938, 0x93e, 0x928, 0x20, 0x932, 0x94d,
0x92f, 0x941, 0x907, 0x938, 0x909, 0x938, 0x94d, 0x935, 0x93e, 0x907, 0x92f, 0x93e,
0x906, 0x938, 0x941, 0x928, 0x94d, 0x938, 0x93f, 0x92f, 0x94b, 0x928, 0x905, 0x945,
0x91f, 0x93f, 0x915, 0x94b, 0x915, 0x928, 0x92c, 0x93e, 0x939, 0x93f, 0x92f, 0x93e,
0x20, 0x92c, 0x93e, 0x902, 0x926, 0x947, 0x930, 0x93e, 0x938, 0x92c, 0x93e, 0x930,
0x94d, 0x92c, 0x93e, 0x921, 0x94b, 0x938, 0x92c, 0x947, 0x932, 0x93f, 0x91d, 0x947,
0x92c, 0x94d, 0x932, 0x93e, 0x902, 0x915, 0x20, 0x938, 0x945, 0x92c, 0x932, 0x94b,
0x928, 0x92c, 0x94b, 0x906, 0x20, 0x935, 0x94d, 0x939, 0x93f, 0x938, 0x94d, 0x91f,
0x93e, 0x92c, 0x94b, 0x907, 0x938, 0x940, 0x915, 0x947, 0x902, 0x92c, 0x94d, 0x930,
0x93f, 0x91c, 0x20, 0x909, 0x92a, 0x938, 0x93e, 0x917, 0x930, 0x915, 0x945, 0x92e,
0x94d, 0x92a, 0x94b, 0x20, 0x917, 0x94d, 0x930, 0x901, 0x921, 0x947, 0x915, 0x930,
0x93e, 0x915, 0x93e, 0x938, 0x915, 0x93e, 0x92f, 0x947, 0x928, 0x947, 0x915, 0x947,
0x92e, 0x928, 0x938, 0x93f, 0x909, 0x926, 0x93e, 0x926, 0x20, 0x939, 0x941, 0x906,
0x930, 0x947, 0x91d, 0x915, 0x94d, 0x92f, 0x941, 0x930, 0x93e, 0x938, 0x93e, 0x913,
0x921, 0x947, 0x928, 0x94d, 0x935, 0x94d, 0x939, 0x930, 0x921, 0x947, 0x91f, 0x94d,
0x930, 0x949, 0x907, 0x91f, 0x90f, 0x921, 0x92e, 0x93e, 0x901, 0x91f, 0x928, 0x92f,
0x942, 0x930, 0x941, 0x928, 0x940, 0x92a, 0x947, 0x917, 0x94d, 0x932, 0x947, 0x938,
0x20, 0x909, 0x92a, 0x938, 0x93e, 0x917, 0x930, 0x917, 0x942, 0x938, 0x20, 0x909,
0x92a, 0x938, 0x93e, 0x917, 0x930, 0x917, 0x94d, 0x930, 0x901, 0x921, 0x20, 0x91f,
0x930, 0x94d, 0x915, 0x917, 0x94d, 0x930, 0x947, 0x928, 0x947, 0x921, 0x93e, 0x917,
0x94d, 0x935, 0x93e, 0x921, 0x947, 0x932, 0x94b, 0x909, 0x92a, 0x947, 0x917, 0x92f,
0x93e, 0x915, 0x94d, 0x935, 0x93f, 0x932, 0x939, 0x945, 0x932, 0x93f, 0x92b, 0x945,
0x915, 0x94d, 0x938, 0x935, 0x947, 0x935, 0x93e, 0x92f, 0x2d, 0x907, 0x902, 0x921,
0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x935, 0x93f, 0x928, 0x92e, 0x945, 0x915, 0x2c,
0x20, 0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x907, 0x915, 0x93e,
0x932, 0x941, 0x907, 0x924, 0x91c, 0x94d, 0x92f, 0x942, 0x928, 0x94c, 0x932, 0x94d,
0x92f, 0x941, 0x907, 0x938, 0x935, 0x94d, 0x939, 0x93f, 0x932, 0x92e, 0x93e, 0x901,
0x91f, 0x93f, 0x938, 0x947, 0x932, 0x94b, 0x2c, 0x20, 0x915, 0x947, 0x902, 0x91f,
0x941, 0x915, 0x940, 0x915, 0x94d, 0x930, 0x93e, 0x932, 0x947, 0x902, 0x926, 0x93f,
0x91c, 0x93f, 0x915, 0x932, 0x949, 0x938, 0x20, 0x90f, 0x902, 0x91c, 0x947, 0x932,
0x93f, 0x938, 0x932, 0x94b, 0x905, 0x930, 0x20, 0x92a, 0x94d, 0x930, 0x93f, 0x928,
0x94d, 0x938, 0x938, 0x20, 0x915, 0x94d, 0x935, 0x93e, 0x930, 0x94d, 0x91f, 0x930,
0x92e, 0x947, 0x938, 0x947, 0x907, 0x913, 0x92e, 0x947, 0x930, 0x940, 0x917, 0x94b,
0x91f, 0x92e, 0x93e, 0x91d, 0x93e, 0x924, 0x932, 0x93e, 0x928, 0x92e, 0x947, 0x924,
0x932, 0x93e, 0x915, 0x924, 0x932, 0x93e, 0x92e, 0x93e, 0x901, 0x915, 0x91f, 0x928,
0x92e, 0x949, 0x928, 0x94d, 0x91f, 0x947, 0x930, 0x947, 0x92e, 0x94b, 0x928, 0x94d,
0x91f, 0x947, 0x935, 0x94d, 0x939, 0x93f, 0x921, 0x93f, 0x92f, 0x94b, 0x92e, 0x93e,
0x901, 0x928, 0x94d, 0x91f, 0x938, 0x947, 0x930, 0x93e, 0x924, 0x92c, 0x94d, 0x92f,
0x941, 0x932, 0x93e, 0x939, 0x2c, 0x20, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20,
0x921, 0x915, 0x94b, 0x91f, 0x93e, 0x92e, 0x927, 0x94d, 0x92f, 0x2c, 0x20, 0x909,
0x924, 0x94d, 0x924, 0x930, 0x20, 0x921, 0x915, 0x94b, 0x91f, 0x93e, 0x928, 0x94d,
0x92f, 0x942, 0x20, 0x938, 0x93e, 0x932, 0x947, 0x92e, 0x2c, 0x20, 0x909, 0x924,
0x94d, 0x924, 0x930, 0x20, 0x921, 0x915, 0x94b, 0x91f, 0x93e, 0x92a, 0x93e, 0x930,
0x92e, 0x930, 0x940, 0x92c, 0x94b, 0x92b, 0x949, 0x928, 0x93f, 0x915, 0x94d, 0x938,
0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x2d, 0x914, 0x2d, 0x92a, 0x94d, 0x930, 0x93f,
0x902, 0x938, 0x92a, 0x94d, 0x92f, 0x941, 0x90f, 0x930, 0x94d, 0x924, 0x94b, 0x20,
0x930, 0x93f, 0x915, 0x94b, 0x92a, 0x941, 0x902, 0x924, 0x93e, 0x20, 0x905, 0x930,
0x940, 0x928, 0x93e, 0x938, 0x930, 0x945, 0x928, 0x915, 0x93f, 0x928, 0x20, 0x907,
0x928, 0x932, 0x947, 0x91f, 0x930, 0x947, 0x91c, 0x94b, 0x932, 0x94d, 0x92f, 0x942,
0x91f, 0x938, 0x901, 0x91f, 0x93e, 0x930, 0x947, 0x92e, 0x938, 0x945, 0x928, 0x94d,
0x91f, 0x93f, 0x90f, 0x917, 0x94b, 0x938, 0x945, 0x928, 0x94d, 0x91f, 0x94b, 0x20,
0x921, 0x94b, 0x92e, 0x93f, 0x902, 0x917, 0x94b, 0x938, 0x93f, 0x91f, 0x915, 0x93e,
0x938, 0x947, 0x902, 0x91f, 0x20, 0x932, 0x941, 0x938, 0x93f, 0x92f, 0x93e, 0x91f,
0x947, 0x917, 0x941, 0x938, 0x93f, 0x917, 0x93e, 0x932, 0x94d, 0x92a, 0x93e, 0x91f,
0x94b, 0x930, 0x94b, 0x902, 0x91f, 0x94b, 0x935, 0x94d, 0x939, 0x945, 0x928, 0x915,
0x941, 0x935, 0x94d, 0x939, 0x930, 0x935, 0x94d, 0x939, 0x93e, 0x907, 0x91f, 0x939,
0x949, 0x930, 0x94d, 0x938, 0x92f, 0x915, 0x941, 0x924, 0x93e, 0x924, 0x915, 0x945,
0x938, 0x947, 0x921, 0x947, 0x935, 0x94d, 0x939, 0x93f, 0x938, 0x921, 0x94d, 0x92f,
0x941, 0x92e, 0x949, 0x928, 0x94d, 0x91f, 0x20, 0x921, 0x94d, 0x92f, 0x941, 0x930,
0x94d, 0x935, 0x93f, 0x932, 0x92e, 0x945, 0x915, 0x94d, 0x935, 0x93e, 0x92f, 0x930,
0x92e, 0x945, 0x915, 0x94d, 0x92e, 0x941, 0x930, 0x921, 0x94b, 0x92a, 0x93e, 0x92e,
0x947, 0x930, 0x935, 0x94b, 0x938, 0x94d, 0x91f, 0x94b, 0x915, 0x932, 0x93e, 0x901,
0x917, 0x907, 0x92f, 0x930, 0x92c, 0x940, 0x92f, 0x947, 0x928, 0x90f, 0x928, 0x93e,
0x921, 0x940, 0x92f, 0x930, 0x905, 0x200d, 0x945, 0x915, 0x94d, 0x91f, 0x94c, 0x905,
0x200d, 0x945, 0x915, 0x94d, 0x91f, 0x94b, 0x92c, 0x947, 0x905, 0x924, 0x93f, 0x930,
0x949, 0x92c, 0x939, 0x93e, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x928, 0x94c,
0x932, 0x92c, 0x94d, 0x930, 0x941, 0x928, 0x947, 0x908, 0x92b, 0x945, 0x92e, 0x917,
0x941, 0x938, 0x94d, 0x924, 0x93e, 0x939, 0x94b, 0x20, 0x91a, 0x93f, 0x20, 0x92e,
0x93f, 0x928, 0x94d, 0x939, 0x20, 0x936, 0x939, 0x930, 0x939, 0x93e, 0x901, 0x917,
0x915, 0x93e, 0x901, 0x917, 0x91c, 0x947, 0x930, 0x941, 0x938, 0x932, 0x947, 0x92e,
0x915, 0x945, 0x92e, 0x91a, 0x93e, 0x91f, 0x915, 0x93e, 0x916, 0x902, 0x926, 0x93f,
0x917, 0x93e, 0x915, 0x94d, 0x935, 0x93e, 0x932, 0x93e, 0x932, 0x902, 0x92a, 0x942,
0x930, 0x92e, 0x947, 0x917, 0x93e, 0x921, 0x928, 0x92e, 0x915, 0x938, 0x94d, 0x938,
0x93e, 0x930, 0x928, 0x94b, 0x935, 0x94b, 0x915, 0x941, 0x91d, 0x94d, 0x928, 0x947,
0x924, 0x94d, 0x938, 0x94d, 0x915, 0x92a, 0x94d, 0x928, 0x94b, 0x92e, 0x20, 0x92a,
0x947, 0x928, 0x94d, 0x939, 0x92a, 0x93e, 0x901, 0x91f, 0x93f, 0x92f, 0x93e, 0x928,
0x93e, 0x915, 0x915, 0x93f, 0x91d, 0x940, 0x932, 0x94b, 0x930, 0x94d, 0x921, 0x93e,
0x938, 0x947, 0x909, 0x932, 0x938, 0x94d, 0x930, 0x947, 0x926, 0x928, 0x947, 0x915,
0x94b, 0x932, 0x940, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x925, 0x93f, 0x902, 0x92b,
0x942, 0x91f, 0x94b, 0x915, 0x93f, 0x92f, 0x94b, 0x924, 0x94b, 0x92e, 0x938, 0x915,
0x935, 0x94d, 0x939, 0x93f, 0x90f, 0x928, 0x94d, 0x91f, 0x93e, 0x907, 0x928, 0x935,
0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x935, 0x94b, 0x938, 0x94d, 0x91f, 0x94b, 0x915,
0x92f, 0x947, 0x915, 0x94d, 0x924, 0x947, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d,
0x917, 0x92f, 0x947, 0x930, 0x947, 0x935, 0x928, 0x915, 0x947, 0x92a, 0x20, 0x935,
0x94d, 0x939, 0x930, 0x94d, 0x921, 0x947, 0x92b, 0x945, 0x930, 0x94b, 0x92e, 0x921,
0x940, 0x92f, 0x930, 0x93e, 0x930, 0x947, 0x92f, 0x915, 0x94d, 0x91c, 0x93e, 0x935,
0x93f, 0x915, 0x938, 0x947, 0x902, 0x91f, 0x2e, 0x20, 0x939, 0x947, 0x932, 0x947,
0x928, 0x93e, 0x938, 0x94d, 0x91f, 0x945, 0x928, 0x932, 0x947, 0x90f, 0x921, 0x932,
0x947, 0x921, 0x932, 0x93f, 0x902, 0x921, 0x947, 0x92e, 0x928, 0x905, 0x200d, 0x945,
0x92e, 0x938, 0x94d, 0x91f, 0x930, 0x921, 0x945, 0x92e, 0x905, 0x901, 0x921, 0x94b,
0x930, 0x93e, 0x92c, 0x94d, 0x930, 0x93e, 0x924, 0x93f, 0x938, 0x94d, 0x932, 0x93e,
0x935, 0x94d, 0x939, 0x93e, 0x91a, 0x93f, 0x938, 0x93f, 0x928, 0x94c, 0x917, 0x94d,
0x935, 0x947, 0x930, 0x94d, 0x928, 0x938, 0x947, 0x906, 0x92f, 0x932, 0x20, 0x911,
0x92b, 0x20, 0x92e, 0x945, 0x928, 0x915, 0x932, 0x93f, 0x928, 0x93f, 0x928, 0x917,
0x94d, 0x930, 0x93e, 0x921, 0x932, 0x941, 0x92c, 0x94d, 0x932, 0x93f, 0x92f, 0x93e,
0x928, 0x93e, 0x932, 0x915, 0x94d, 0x91d, 0x947, 0x902, 0x92c, 0x930, 0x94d, 0x917,
0x92e, 0x930, 0x93f, 0x92f, 0x947, 0x939, 0x93e, 0x92e, 0x947, 0x928, 0x911, 0x938,
0x94d, 0x932, 0x94b, 0x938, 0x945, 0x928, 0x20, 0x92e, 0x930, 0x93f, 0x928, 0x94b,
0x938, 0x93e, 0x930, 0x93e, 0x91c, 0x947, 0x935, 0x94d, 0x939, 0x94b, 0x938, 0x93f,
0x92e, 0x94d, 0x92b, 0x930, 0x94b, 0x92a, 0x94b, 0x932, 0x938, 0x94d, 0x915, 0x949,
0x92a, 0x94d, 0x91c, 0x947, 0x924, 0x93e, 0x932, 0x93f, 0x928, 0x935, 0x921, 0x942,
0x91d, 0x935, 0x94d, 0x939, 0x945, 0x91f, 0x93f, 0x915, 0x928, 0x935, 0x94d, 0x939,
0x93f, 0x90f, 0x928, 0x94d, 0x928, 0x93e, 0x935, 0x93f, 0x932, 0x928, 0x93f, 0x92f,
0x938, 0x935, 0x949, 0x930, 0x94d, 0x938, 0x93e, 0x91d, 0x945, 0x917, 0x94d, 0x930,
0x947, 0x92c, 0x91d, 0x941, 0x930, 0x93f, 0x915, 0x916, 0x94d, 0x930, 0x93f, 0x938,
0x92e, 0x938, 0x915, 0x930, 0x917, 0x94d, 0x92f, 0x942, 0x932, 0x947, 0x928, 0x92e,
0x93e, 0x92f, 0x94b, 0x91f, 0x94d, 0x91f, 0x947, 0x911, 0x915, 0x932, 0x902, 0x921,
0x92c, 0x949, 0x917, 0x945, 0x928, 0x935, 0x94d, 0x939, 0x93f, 0x932, 0x91a, 0x945,
0x91f, 0x939, 0x945, 0x92e, 0x92b, 0x941, 0x928, 0x93e, 0x92b, 0x941, 0x924, 0x940,
0x917, 0x945, 0x932, 0x93e, 0x92a, 0x93e, 0x917, 0x94b, 0x938, 0x917, 0x945, 0x92e,
0x94d, 0x92c, 0x93f, 0x92f, 0x930, 0x939, 0x94b, 0x928, 0x94b, 0x932, 0x941, 0x932,
0x942, 0x915, 0x94b, 0x936, 0x93e, 0x92f, 0x915, 0x94d, 0x935, 0x93e, 0x91c, 0x93e,
0x932, 0x947, 0x908, 0x928, 0x92e, 0x91c, 0x941, 0x930, 0x94b, 0x92e, 0x93e, 0x930,
0x915, 0x94d, 0x935, 0x93f, 0x938, 0x93e, 0x938, 0x928, 0x909, 0x930, 0x941, 0x928,
0x94d, 0x92f, 0x941, 0x90f, 0x928, 0x94c, 0x92e, 0x93f, 0x906, 0x92a, 0x945, 0x917,
0x94b, 0x20, 0x92a, 0x945, 0x917, 0x94b, 0x92a, 0x93f, 0x91f, 0x915, 0x947, 0x930,
0x94d, 0x928, 0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x20, 0x92e, 0x94b, 0x930, 0x947,
0x938, 0x94d, 0x92c, 0x947, 0x938, 0x948, 0x92a, 0x93e, 0x928, 0x410, 0x431, 0x438,
0x436, 0x430, 0x43d, 0x410, 0x434, 0x434, 0x438, 0x441, 0x2d, 0x41, 0x431, 0x435,
0x431, 0x430, 0x411, 0x430, 0x43d, 0x433, 0x443, 0x439, 0x411, 0x438, 0x441, 0x441,
0x430, 0x443, 0x410, 0x43d, 0x43a, 0x43e, 0x440, 0x430, 0x436, 0x410, 0x440, 0x430,
0x433, 0x443, 0x430, 0x44f, 0x43d, 0x430, 0x411, 0x430, 0x439, 0x44f, 0x411, 0x430,
0x445, 0x44c, 0x44f, 0x20, 0x411, 0x430, 0x43d, 0x434, 0x435, 0x440, 0x430, 0x441,
0x41a, 0x44d, 0x43c, 0x431, 0x440, 0x438, 0x436, 0x438, 0x439, 0x43d, 0x20, 0x431,
0x443, 0x43b, 0x430, 0x43d, 0x41a, 0x430, 0x43c, 0x43f, 0x43e, 0x2d, 0x413, 0x440,
0x430, 0x43d, 0x434, 0x435, 0x421, 0x44c, 0x44e, 0x434, 0x430, 0x434, 0x20, 0x425,
0x443, 0x430, 0x440, 0x435, 0x441, 0x414, 0x430, 0x43d, 0x43c, 0x430, 0x440, 0x43a,
0x448, 0x430, 0x432, 0x43d, 0x414, 0x43e, 0x443, 0x441, 0x43e, 0x43d, 0x20, 0x41a,
0x440, 0x438, 0x43a, 0x42d, 0x43b, 0x44c, 0x20, 0x421, 0x430, 0x43b, 0x44c, 0x432,
0x430, 0x434, 0x43e, 0x440, 0x424, 0x43e, 0x440, 0x442, 0x20, 0x41d, 0x44d, 0x43b,
0x44c, 0x441, 0x43e, 0x43d, 0x413, 0x43b, 0x435, 0x439, 0x441, 0x20, 0x431, 0x443,
0x43b, 0x430, 0x43d, 0x413, 0x443, 0x443, 0x441, 0x20, 0x431, 0x443, 0x43b, 0x430,
0x43d, 0x425, 0x435, 0x440, 0x43c, 0x43e, 0x441, 0x438, 0x43b, 0x43b, 0x43e, 0x422,
0x435, 0x43b, 0x20, 0x421, 0x438, 0x442, 0x438, 0x2c, 0x20, 0x418, 0x43d, 0x434,
0x438, 0x430, 0x43d, 0x430, 0x412, 0x438, 0x43d, 0x441, 0x435, 0x43d, 0x435, 0x441,
0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d, 0x430, 0x412, 0x438, 0x43d,
0x430, 0x43c, 0x430, 0x43a, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d,
0x430, 0x41c, 0x43e, 0x43d, 0x442, 0x438, 0x447, 0x435, 0x43b, 0x43b, 0x43e, 0x2c,
0x20, 0x41a, 0x435, 0x43d, 0x442, 0x430, 0x43a, 0x438, 0x41a, 0x440, 0x430, 0x43b,
0x435, 0x43d, 0x434, 0x438, 0x43a, 0x41b, 0x43e, 0x432, 0x435, 0x440, 0x20, 0x41f,
0x440, 0x438, 0x43d, 0x441, 0x435, 0x441, 0x20, 0x41a, 0x432, 0x430, 0x440, 0x442,
0x435, 0x440, 0x41c, 0x430, 0x440, 0x438, 0x433, 0x43e, 0x442, 0x41d, 0x43e, 0x440,
0x43e, 0x43d, 0x430, 0x411, 0x438, 0x43b, 0x430, 0x2c, 0x20, 0x425, 0x43e, 0x439,
0x434, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x422, 0x4e9, 0x432, 0x2c,
0x20, 0x425, 0x43e, 0x439, 0x434, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430,
0x41d, 0x44c, 0x44e, 0x2d, 0x421, 0x430, 0x43b, 0x435, 0x43c, 0x2c, 0x20, 0x425,
0x43e, 0x439, 0x434, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41d, 0x4af,
0x4af, 0x43a, 0x41e, 0x436, 0x438, 0x43d, 0x430, 0x433, 0x430, 0x41f, 0x43e, 0x440,
0x442, 0x2d, 0x43e, 0x2d, 0x41f, 0x440, 0x438, 0x43d, 0x441, 0x418, 0x441, 0x43f,
0x430, 0x43d, 0x438, 0x439, 0x20, 0x431, 0x43e, 0x43e, 0x43c, 0x442, 0x41f, 0x43e,
0x440, 0x442, 0x43e, 0x2d, 0x412, 0x435, 0x43b, 0x44c, 0x44e, 0x420, 0x438, 0x43e,
0x2d, 0x411, 0x440, 0x430, 0x43d, 0x43a, 0x43e, 0x421, 0x435, 0x43d, 0x442, 0x2d,
0x411, 0x430, 0x440, 0x442, 0x435, 0x43b, 0x44c, 0x43c, 0x438, 0x421, 0x435, 0x43d,
0x442, 0x2d, 0x416, 0x43e, 0x43d, 0x441, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41a,
0x438, 0x442, 0x442, 0x441, 0x421, 0x435, 0x43d, 0x442, 0x20, 0x41b, 0x44e, 0x441,
0x438, 0x430, 0x423, 0x430, 0x439, 0x442, 0x445, 0x43e, 0x440, 0x437, 0x41a, 0x44d,
0x441, 0x435, 0x439, 0x414, 0x44e, 0x43c, 0x43e, 0x43d, 0x20, 0x434, 0x2019, 0x42e,
0x440, 0x432, 0x438, 0x43b, 0x44c, 0x41c, 0x430, 0x43a, 0x2d, 0x41c, 0x4e9, 0x440,
0x434, 0x43e, 0x41b, 0x43e, 0x43d, 0x433, 0x438, 0x440, 0x431, 0x430, 0x439, 0x435,
0x43d, 0x410, 0x43a, 0x442, 0x4e9, 0x431, 0x435, 0x425, 0x43e, 0x20, 0x428, 0x438,
0x20, 0x41c, 0x438, 0x43d, 0x20, 0x445, 0x43e, 0x442, 0x42d, 0x440, 0x445, 0x4af,
0x4af, 0x416, 0x430, 0x439, 0x43f, 0x443, 0x440, 0x41f, 0x451, 0x43d, 0x44c, 0x44f,
0x43d, 0x421, 0x4e9, 0x4af, 0x43b, 0x423, 0x43b, 0x430, 0x430, 0x43d, 0x431, 0x430,
0x430, 0x442, 0x430, 0x440, 0x4ae, 0x440, 0x4af, 0x43c, 0x447, 0x438, 0x4e8, 0x43c,
0x43d, 0x4e9, 0x434, 0x20, 0x416, 0x43e, 0x440, 0x436, 0x438, 0x430, 0x421, 0x435,
0x43d, 0x442, 0x20, 0x425, 0x435, 0x43b, 0x435, 0x43d, 0x430, 0x411, 0x440, 0x43e,
0x43a, 0x435, 0x43d, 0x20, 0x425, 0x438, 0x43b, 0x43b, 0x415, 0x443, 0x43a, 0x43b,
0x430, 0x41b, 0x438, 0x43d, 0x434, 0x435, 0x43c, 0x44d, 0x43d, 0x41b, 0x43e, 0x440,
0x434, 0x20, 0x425, 0x43e, 0x443, 0x41f, 0x435, 0x440, 0x441, 0x41a, 0x438, 0x448,
0x438, 0x43d, 0x451, 0x432, 0x41c, 0x44d, 0x43d, 0x20, 0x410, 0x440, 0x430, 0x43b,
0x41b, 0x438, 0x441, 0x431, 0x43e, 0x43d, 0x41f, 0x430, 0x440, 0x438, 0x441, 0x420,
0x43e, 0x43c, 0x41e, 0x443, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x427, 0x4af, 0x4af,
0x43a, 0x413, 0x430, 0x43c, 0x431, 0x44c, 0x435, 0x440, 0x41a, 0x43e, 0x441, 0x440,
0x44d, 0x41a, 0x432, 0x430, 0x436, 0x430, 0x43b, 0x435, 0x439, 0x43d, 0x423, 0x44d,
0x43a, 0x182c, 0x1823, 0x182a, 0x1833, 0x1823, 0x1823, 0x182f, 0x1820, 0x182d, 0x1820, 0x1828,
0x182a, 0x1820, 0x182d, 0x1820, 0x1833, 0x1823, 0x1837, 0x905, 0x935, 0x93f, 0x921, 0x94d,
0x91c, 0x93e, 0x928, 0x905, 0x915, 0x94d, 0x930, 0x93e, 0x90f, 0x921, 0x94d, 0x921,
0x93f, 0x938, 0x20, 0x906, 0x92c, 0x93e, 0x92c, 0x93e, 0x905, 0x932, 0x94d, 0x91c,
0x93f, 0x92f, 0x930, 0x94d, 0x938, 0x92c, 0x94b, 0x92e, 0x93e, 0x915, 0x94b, 0x92c,
0x93e, 0x919, 0x94d, 0x917, 0x941, 0x935, 0x940, 0x92c, 0x94d, 0x932, 0x93e, 0x928,
0x94d, 0x91f, 0x93e, 0x92f, 0x930, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x93e, 0x92d,
0x93f, 0x932, 0x94d, 0x932, 0x947, 0x915, 0x93e, 0x907, 0x930, 0x94b, 0x915, 0x93e,
0x938, 0x93e, 0x92c, 0x94d, 0x932, 0x93e, 0x928, 0x94d, 0x915, 0x93e, 0x921, 0x93e,
0x930, 0x20, 0x90f, 0x938, 0x94d, 0x20, 0x938, 0x932, 0x93e, 0x92e, 0x91c, 0x93f,
0x92c, 0x94c, 0x902, 0x91f, 0x940, 0x921, 0x94b, 0x909, 0x906, 0x932, 0x93e, 0x90f,
0x932, 0x94d, 0x20, 0x906, 0x907, 0x92f, 0x941, 0x928, 0x92b, 0x94d, 0x930, 0x93f,
0x91f, 0x93e, 0x909, 0x928, 0x917, 0x93e, 0x935, 0x94b, 0x930, 0x94b, 0x928, 0x91c,
0x94b, 0x939, 0x93e, 0x928, 0x947, 0x938, 0x935, 0x930, 0x94d, 0x917, 0x916, 0x93e,
0x930, 0x94d, 0x91f, 0x909, 0x92e, 0x932, 0x93f, 0x92c, 0x94d, 0x930, 0x947, 0x92d,
0x93f, 0x932, 0x94d, 0x932, 0x947, 0x932, 0x941, 0x935, 0x93e, 0x928, 0x94d, 0x921,
0x93e, 0x932, 0x941, 0x92c, 0x941, 0x92e, 0x94d, 0x92c, 0x93e, 0x938, 0x940, 0x92e,
0x93e, 0x932, 0x93e, 0x92c, 0x94b, 0x92e, 0x93e, 0x92a, 0x941, 0x924, 0x94b, 0x92e,
0x94b, 0x917, 0x93e, 0x921, 0x93f, 0x936, 0x941, 0x92e, 0x94b, 0x928, 0x94d, 0x930,
0x94b, 0x92d, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x907, 0x930, 0x94b, 0x92c, 0x940,
0x90f, 0x928, 0x94d, 0x200c, 0x91c, 0x93e, 0x92e, 0x947, 0x928, 0x93e, 0x928, 0x93e,
0x92f, 0x92e, 0x947, 0x928, 0x94b, 0x909, 0x906, 0x915, 0x91a, 0x94b, 0x91f, 0x914,
0x906, 0x917, 0x93e, 0x921, 0x94c, 0x917, 0x94c, 0x92a, 0x94b, 0x930, 0x94d, 0x91f,
0x94b, 0x2d, 0x928, 0x94b, 0x92d, 0x94b, 0x938, 0x93e, 0x913, 0x20, 0x91f, 0x94b,
0x92e, 0x947, 0x935, 0x93f, 0x928, 0x94d, 0x921, 0x939, 0x94b, 0x90f, 0x915, 0x906,
0x921, 0x93e, 0x915, 0x90f, 0x919, 0x94d, 0x915, 0x94b, 0x930, 0x947, 0x91c, 0x90f,
0x919, 0x94d, 0x917, 0x94d, 0x935, 0x93f, 0x932, 0x94d, 0x932, 0x93e, 0x906, 0x930,
0x93e, 0x917, 0x941, 0x935, 0x93e, 0x928, 0x93e, 0x92c, 0x94d, 0x92f, 0x928, 0x947,
0x936, 0x20, 0x906, 0x92f, 0x930, 0x94d, 0x938, 0x915, 0x93e, 0x924, 0x93e, 0x92e,
0x93e, 0x930, 0x94d, 0x915, 0x93e, 0x915, 0x94b, 0x930, 0x921, 0x94b, 0x935, 0x93e,
0x92e, 0x947, 0x928, 0x94d, 0x921, 0x94b, 0x91c, 0x93e, 0x930, 0x93f, 0x92f, 0x94b,
0x20, 0x917, 0x94d, 0x92f, 0x93e, 0x932, 0x947, 0x917, 0x94b, 0x938, 0x938, 0x93e,
0x928, 0x20, 0x91c, 0x941, 0x935, 0x93e, 0x928, 0x938, 0x93e, 0x928, 0x20, 0x932,
0x941, 0x907, 0x938, 0x909, 0x936, 0x941, 0x906, 0x907, 0x906, 0x905, 0x938, 0x928,
0x94d, 0x938, 0x93f, 0x92f, 0x928, 0x90f, 0x91f, 0x93f, 0x915, 0x94b, 0x915, 0x93e,
0x928, 0x92c, 0x93e, 0x939, 0x93f, 0x92f, 0x93e, 0x20, 0x92c, 0x93e, 0x928, 0x94d,
0x921, 0x947, 0x930, 0x93e, 0x936, 0x92c, 0x94d, 0x932, 0x93e, 0x928, 0x94d, 0x915,
0x2d, 0x938, 0x93e, 0x92c, 0x94d, 0x932, 0x94b, 0x928, 0x92c, 0x94b, 0x906, 0x20,
0x92d, 0x93f, 0x937, 0x94d, 0x91f, 0x93e, 0x92c, 0x94b, 0x907, 0x91c, 0x915, 0x94d,
0x92f, 0x93e, 0x92e, 0x94d, 0x92c, 0x94d, 0x930, 0x93f, 0x91c, 0x20, 0x92c, 0x947,
0x915, 0x94d, 0x92f, 0x93e, 0x92e, 0x94d, 0x92a, 0x94b, 0x20, 0x917, 0x94d, 0x930,
0x93e, 0x928, 0x94d, 0x921, 0x947, 0x915, 0x93e, 0x92f, 0x947, 0x928, 0x94d, 0x928,
0x947, 0x915, 0x947, 0x92e, 0x94d, 0x92f, 0x93e, 0x928, 0x91a, 0x93f, 0x939, 0x941,
0x935, 0x93e, 0x939, 0x941, 0x935, 0x93e, 0x91c, 0x941, 0x906, 0x930, 0x947, 0x91c,
0x20, 0x938, 0x939, 0x930, 0x915, 0x94b, 0x937, 0x94d, 0x91f, 0x93e, 0x20, 0x930,
0x93f, 0x915, 0x93e, 0x915, 0x94d, 0x92f, 0x941, 0x907, 0x906, 0x92c, 0x93e, 0x921,
0x93e, 0x928, 0x94d, 0x92e, 0x93e, 0x930, 0x94d, 0x915, 0x936, 0x93e, 0x92d, 0x928,
0x921, 0x938, 0x928, 0x921, 0x938, 0x928, 0x20, 0x915, 0x94d, 0x930, 0x93f, 0x915,
0x921, 0x947, 0x928, 0x94d, 0x92d, 0x930, 0x921, 0x93f, 0x91f, 0x94d, 0x930, 0x94b,
0x907, 0x91f, 0x90f, 0x921, 0x94d, 0x92e, 0x94b, 0x928, 0x94d, 0x91f, 0x928, 0x906,
0x907, 0x930, 0x928, 0x947, 0x92a, 0x947, 0x90f, 0x932, 0x94d, 0x20, 0x938, 0x93e,
0x932, 0x94d, 0x92d, 0x93e, 0x921, 0x94b, 0x930, 0x917, 0x941, 0x91c, 0x20, 0x92c,
0x947, 0x917, 0x94d, 0x930, 0x93e, 0x928, 0x94d, 0x921, 0x20, 0x91f, 0x930, 0x94d,
0x915, 0x917, 0x94d, 0x935, 0x93e, 0x921, 0x947, 0x932, 0x941, 0x92a, 0x917, 0x941,
0x92f, 0x93e, 0x915, 0x94d, 0x935, 0x93f, 0x932, 0x939, 0x94d, 0x92f, 0x93e, 0x932,
0x93f, 0x92b, 0x94d, 0x92f, 0x93e, 0x915, 0x94d, 0x938, 0x939, 0x92d, 0x93e, 0x928,
0x93e, 0x939, 0x947, 0x930, 0x94d, 0x92e, 0x94b, 0x938, 0x93f, 0x932, 0x94d, 0x932,
0x94b, 0x907, 0x928, 0x94d, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x92a, 0x94b,
0x932, 0x93f, 0x938, 0x928, 0x94b, 0x915, 0x94d, 0x938, 0x20, 0x907, 0x928, 0x94d,
0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x92e, 0x93e, 0x930, 0x947, 0x928, 0x94d,
0x917, 0x94b, 0x2c, 0x20, 0x907, 0x928, 0x94d, 0x921, 0x93f, 0x92f, 0x93e, 0x928,
0x93e, 0x92a, 0x93f, 0x91f, 0x930, 0x94d, 0x938, 0x92c, 0x930, 0x94d, 0x917, 0x2c,
0x20, 0x907, 0x928, 0x94d, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x91f, 0x947,
0x932, 0x20, 0x938, 0x93f, 0x91f, 0x940, 0x2c, 0x20, 0x907, 0x928, 0x94d, 0x921,
0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x92d, 0x947, 0x92d, 0x947, 0x2c, 0x20, 0x907,
0x928, 0x94d, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x92d, 0x93f, 0x928, 0x94d,
0x938, 0x947, 0x928, 0x94d, 0x938, 0x935, 0x93f, 0x928, 0x93e, 0x92e, 0x93e, 0x915,
0x2c, 0x20, 0x907, 0x928, 0x94d, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x907,
0x928, 0x941, 0x92d, 0x93f, 0x915, 0x907, 0x915, 0x94d, 0x935, 0x93e, 0x932, 0x941,
0x907, 0x91f, 0x91c, 0x92e, 0x93e, 0x907, 0x915, 0x93e, 0x91c, 0x941, 0x928, 0x93f,
0x909, 0x932, 0x941, 0x907, 0x938, 0x92d, 0x93f, 0x932, 0x94d, 0x932, 0x947, 0x92e,
0x94b, 0x928, 0x94d, 0x91f, 0x93f, 0x938, 0x947, 0x932, 0x94d, 0x932, 0x94b, 0x2c,
0x915, 0x947, 0x928, 0x94d, 0x91f, 0x915, 0x940, 0x915, 0x93e, 0x932, 0x947, 0x928,
0x94d, 0x926, 0x947, 0x91c, 0x93f, 0x915, 0x932, 0x938, 0x94d, 0x20, 0x90f, 0x928,
0x94d, 0x91c, 0x947, 0x932, 0x938, 0x932, 0x94b, 0x905, 0x930, 0x20, 0x92a, 0x94d,
0x930, 0x93f, 0x928, 0x94d, 0x938, 0x94d, 0x938, 0x20, 0x915, 0x94d, 0x935, 0x93e,
0x930, 0x94d, 0x91f, 0x930, 0x92e, 0x93e, 0x938, 0x947, 0x907, 0x913, 0x92e, 0x93e,
0x928, 0x93e, 0x917, 0x941, 0x935, 0x93e, 0x92e, 0x93e, 0x928, 0x93e, 0x909, 0x938,
0x92e, 0x94d, 0x92f, 0x93e, 0x930, 0x93f, 0x917, 0x91f, 0x92e, 0x93e, 0x91f, 0x94d,
0x924, 0x93e, 0x92e, 0x94b, 0x930, 0x94b, 0x938, 0x94d, 0x92e, 0x93e, 0x91c, 0x93e,
0x91f, 0x94d, 0x932, 0x93e, 0x928, 0x92e, 0x947, 0x91f, 0x94d, 0x932, 0x93e, 0x915,
0x94d, 0x91f, 0x932, 0x93e, 0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x93f, 0x932, 0x928,
0x92e, 0x94b, 0x928, 0x94d, 0x915, 0x91f, 0x928, 0x92e, 0x94b, 0x928, 0x94d, 0x91f,
0x947, 0x930, 0x94d, 0x930, 0x947, 0x92e, 0x94b, 0x928, 0x94d, 0x91f, 0x947, 0x92d,
0x93f, 0x921, 0x93f, 0x92f, 0x94b, 0x92e, 0x928, 0x94d, 0x91f, 0x938, 0x947, 0x930,
0x94d, 0x930, 0x93e, 0x91f, 0x928, 0x94d, 0x92f, 0x941, 0x92f, 0x94b, 0x930, 0x94d,
0x915, 0x92c, 0x947, 0x909, 0x932, 0x93e, 0x2c, 0x20, 0x909, 0x924, 0x94d, 0x924,
0x930, 0x20, 0x921, 0x93e, 0x915, 0x94b, 0x91f, 0x93e, 0x909, 0x924, 0x94d, 0x924,
0x930, 0x20, 0x921, 0x93e, 0x915, 0x94b, 0x91f, 0x93e, 0x2c, 0x20, 0x915, 0x947,
0x928, 0x94d, 0x926, 0x94d, 0x930, 0x928, 0x92f, 0x93e, 0x901, 0x20, 0x938, 0x93e,
0x932, 0x947, 0x92e, 0x2c, 0x20, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x921,
0x93e, 0x915, 0x94b, 0x91f, 0x93e, 0x92a, 0x93e, 0x930, 0x93e, 0x92e, 0x93e, 0x930,
0x93f, 0x935, 0x94b, 0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x2d, 0x905, 0x909, 0x2d,
0x92a, 0x94d, 0x930, 0x93f, 0x928, 0x94d, 0x938, 0x92a, 0x94b, 0x930, 0x94d, 0x91f,
0x20, 0x905, 0x92b, 0x94d, 0x20, 0x938, 0x94d, 0x92a, 0x947, 0x928, 0x92a, 0x94b,
0x930, 0x94d, 0x91f, 0x94b, 0x20, 0x92d, 0x947, 0x932, 0x94d, 0x939, 0x94b, 0x92a,
0x94d, 0x92f, 0x941, 0x930, 0x94d, 0x91f, 0x94b, 0x20, 0x930, 0x93f, 0x915, 0x94b,
0x92a, 0x941, 0x928, 0x94d, 0x91f, 0x93e, 0x20, 0x905, 0x930, 0x93f, 0x928, 0x93e,
0x91c, 0x930, 0x93e, 0x928, 0x94d, 0x915, 0x93f, 0x928, 0x20, 0x907, 0x928, 0x94d,
0x932, 0x947, 0x91f, 0x930, 0x93f, 0x938, 0x93e, 0x907, 0x92b, 0x930, 0x93f, 0x91c,
0x94b, 0x932, 0x941, 0x91f, 0x930, 0x93f, 0x92f, 0x94b, 0x20, 0x92c, 0x94d, 0x930,
0x93e, 0x928, 0x94d, 0x915, 0x94b, 0x938, 0x93e, 0x928, 0x94d, 0x91f, 0x93e, 0x930,
0x947, 0x92e, 0x938, 0x93e, 0x928, 0x94d, 0x91f, 0x93f, 0x906, 0x917, 0x94b, 0x938,
0x93e, 0x928, 0x94d, 0x91f, 0x94b, 0x20, 0x921, 0x94b, 0x92e, 0x93f, 0x919, 0x94d,
0x917, 0x94b, 0x908, 0x91f, 0x94d, 0x91f, 0x94b, 0x915, 0x94d, 0x915, 0x94b, 0x930,
0x91f, 0x942, 0x930, 0x94d, 0x92e, 0x93f, 0x91f, 0x938, 0x947, 0x928, 0x94d, 0x91f,
0x20, 0x92c, 0x93e, 0x930, 0x94d, 0x925, 0x947, 0x932, 0x947, 0x92e, 0x940, 0x938,
0x947, 0x928, 0x94d, 0x91f, 0x20, 0x91c, 0x94b, 0x928, 0x94d, 0x938, 0x938, 0x947,
0x928, 0x94d, 0x91f, 0x94d, 0x20, 0x915, 0x93f, 0x91f, 0x94d, 0x938, 0x938, 0x947,
0x928, 0x94d, 0x91f, 0x20, 0x932, 0x941, 0x938, 0x93f, 0x92f, 0x93e, 0x938, 0x947,
0x928, 0x94d, 0x91f, 0x20, 0x925, 0x94b, 0x92e, 0x938, 0x938, 0x947, 0x928, 0x94d,
0x91f, 0x20, 0x92d, 0x93f, 0x928, 0x94d, 0x938, 0x947, 0x928, 0x94d, 0x91f, 0x938,
0x94d, 0x935, 0x93f, 0x92b, 0x94d, 0x91f, 0x20, 0x915, 0x930, 0x947, 0x928, 0x94d,
0x91f, 0x91f, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x91f, 0x94b, 0x91f, 0x93e, 0x930,
0x94d, 0x91f, 0x94b, 0x932, 0x93e, 0x92d, 0x94d, 0x92f, 0x93e, 0x928, 0x915, 0x94b,
0x92d, 0x930, 0x939, 0x94d, 0x935, 0x93e, 0x907, 0x91f, 0x939, 0x930, 0x94d, 0x938,
0x935, 0x93f, 0x928, 0x94d, 0x928, 0x93f, 0x92a, 0x947, 0x917, 0x915, 0x947, 0x91c,
0x947, 0x926, 0x941, 0x92e, 0x94b, 0x928, 0x94d, 0x91f, 0x20, 0x921, 0x93f, 0x20,
0x909, 0x930, 0x94d, 0x92d, 0x947, 0x932, 0x94d, 0x932, 0x947, 0x92e, 0x915, 0x94d,
0x935, 0x93e, 0x930, 0x93f, 0x908, 0x92e, 0x93e, 0x909, 0x938, 0x928, 0x92e, 0x93e,
0x915, 0x92e, 0x941, 0x930, 0x94d, 0x921, 0x94b, 0x938, 0x93f, 0x913, 0x906, 0x92d,
0x93e, 0x938, 0x94d, 0x91f, 0x94b, 0x915, 0x932, 0x919, 0x92f, 0x93f, 0x905, 0x930,
0x92c, 0x93e, 0x907, 0x90f, 0x928, 0x906, 0x928, 0x93e, 0x921, 0x93f, 0x92f, 0x930,
0x906, 0x915, 0x94d, 0x91f, 0x93e, 0x909, 0x906, 0x915, 0x94d, 0x91f, 0x94b, 0x92c,
0x905, 0x938, 0x94d, 0x915, 0x93e, 0x92c, 0x93e, 0x91f, 0x905, 0x91f, 0x93f, 0x930,
0x93e, 0x909, 0x92c, 0x939, 0x930, 0x93e, 0x908, 0x928, 0x92c, 0x948, 0x902, 0x915,
0x915, 0x92c, 0x947, 0x908, 0x930, 0x941, 0x91f, 0x92c, 0x93f, 0x938, 0x915, 0x947,
0x915, 0x94d, 0x92c, 0x94d, 0x930, 0x941, 0x928, 0x93e, 0x907, 0x926, 0x93e, 0x92e,
0x93e, 0x938, 0x94d, 0x915, 0x938, 0x94d, 0x926, 0x93f, 0x932, 0x94d, 0x932, 0x940,
0x926, 0x941, 0x92c, 0x939, 0x940, 0x926, 0x938, 0x94d, 0x938, 0x93e, 0x928, 0x94d,
0x92c, 0x947, 0x939, 0x93f, 0x92c, 0x94d, 0x930, 0x94b, 0x928, 0x939, 0x94b, 0x20,
0x91a, 0x940, 0x20, 0x92e, 0x93f, 0x928, 0x94d, 0x939, 0x20, 0x936, 0x939, 0x930,
0x939, 0x919, 0x915, 0x919, 0x939, 0x94b, 0x92d, 0x94d, 0x921, 0x915, 0x93e, 0x92e,
0x91a, 0x91f, 0x94d, 0x915, 0x93e, 0x915, 0x93e, 0x920, 0x92e, 0x93e, 0x923, 0x94d,
0x921, 0x94c, 0x902, 0x916, 0x93e, 0x928, 0x94d, 0x926, 0x93f, 0x917, 0x93e, 0x915,
0x941, 0x906, 0x20, 0x932, 0x93e, 0x92e, 0x94d, 0x92a, 0x941, 0x930, 0x915, 0x941,
0x91a, 0x93f, 0x919, 0x92e, 0x915, 0x93e, 0x909, 0x92e, 0x93e, 0x917, 0x93e, 0x921,
0x93e, 0x928, 0x92e, 0x938, 0x94d, 0x915, 0x94d, 0x92f, 0x93e, 0x91f, 0x928, 0x947,
0x92d, 0x94b, 0x915, 0x941, 0x91c, 0x928, 0x947, 0x938, 0x94d, 0x915, 0x928, 0x94b,
0x92c, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x92b,
0x947, 0x928, 0x94b, 0x92e, 0x20, 0x92b, 0x947, 0x928, 0x92a, 0x94b, 0x928, 0x94d,
0x91f, 0x93f, 0x906, 0x928, 0x93e, 0x915, 0x92a, 0x94d, 0x92f, 0x94b, 0x919, 0x92f,
0x93e, 0x919, 0x915, 0x938, 0x94d, 0x91f, 0x93e, 0x928, 0x947, 0x938, 0x92e, 0x93e,
0x930, 0x915, 0x923, 0x94d, 0x921, 0x938, 0x93e, 0x928, 0x94d, 0x918, 0x93e, 0x908,
0x938, 0x94d, 0x930, 0x947, 0x926, 0x94d, 0x928, 0x93f, 0x915, 0x94b, 0x932, 0x93f,
0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x924, 0x93e, 0x938, 0x94d, 0x915, 0x947, 0x928,
0x94d, 0x91f, 0x924, 0x947, 0x939, 0x947, 0x930, 0x93e, 0x928, 0x925, 0x93f, 0x92e,
0x94d, 0x92a, 0x941, 0x91f, 0x94b, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x909, 0x932,
0x93e, 0x928, 0x92c, 0x91f, 0x93e, 0x930, 0x909, 0x938, 0x94d, 0x91f, 0x2d, 0x928,
0x947, 0x930, 0x93e, 0x92d, 0x93f, 0x92f, 0x928, 0x94d, 0x924, 0x93f, 0x928, 0x92d,
0x94d, 0x932, 0x93e, 0x921, 0x93f, 0x92d, 0x93e, 0x938, 0x94d, 0x91f, 0x94b, 0x915,
0x92f, 0x93e, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x930, 0x93e, 0x928,
0x94d, 0x917, 0x941, 0x928, 0x90f, 0x915, 0x91f, 0x930, 0x93f, 0x928, 0x92c, 0x941,
0x930, 0x94d, 0x917, 0x906, 0x91c, 0x94b, 0x930, 0x94d, 0x938, 0x915, 0x94d, 0x92f,
0x93e, 0x928, 0x93e, 0x930, 0x940, 0x915, 0x947, 0x92a, 0x20, 0x92d, 0x930, 0x94d,
0x921, 0x947, 0x92b, 0x93e, 0x930, 0x94b, 0x907, 0x92e, 0x93e, 0x921, 0x947, 0x907,
0x930, 0x93e, 0x930, 0x947, 0x915, 0x94d, 0x91c, 0x93e, 0x92d, 0x93f, 0x915, 0x926,
0x915, 0x94d, 0x937, 0x93f, 0x923, 0x20, 0x91c, 0x930, 0x94d, 0x91c, 0x93f, 0x92f,
0x93e, 0x938, 0x947, 0x928, 0x94d, 0x91f, 0x20, 0x939, 0x947, 0x932, 0x947, 0x928,
0x93e, 0x938, 0x94d, 0x91f, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x932, 0x940, 0x90f,
0x921, 0x947, 0x932, 0x947, 0x921, 0x907, 0x92f, 0x941, 0x915, 0x94d, 0x932, 0x93e,
0x939, 0x94b, 0x92d, 0x93e, 0x930, 0x94d, 0x91f, 0x932, 0x93f, 0x928, 0x94d, 0x921,
0x947, 0x92e, 0x94d, 0x92f, 0x93e, 0x928, 0x932, 0x930, 0x94d, 0x921, 0x20, 0x939,
0x94b, 0x935, 0x947, 0x92e, 0x947, 0x932, 0x94d, 0x92c, 0x930, 0x94d, 0x928, 0x938,
0x93f, 0x921, 0x94d, 0x928, 0x940, 0x90f, 0x92e, 0x94d, 0x938, 0x94d, 0x91f, 0x94d,
0x930, 0x94d, 0x921, 0x92e, 0x906, 0x928, 0x94d, 0x921, 0x94b, 0x930, 0x94d, 0x930,
0x93e, 0x905, 0x938, 0x94d, 0x924, 0x94d, 0x930, 0x916, 0x93e, 0x928, 0x92c, 0x94d,
0x930, 0x93e, 0x91f, 0x93f, 0x938, 0x94d, 0x932, 0x93e, 0x92d, 0x93e, 0x935, 0x941,
0x91a, 0x93e, 0x930, 0x947, 0x938, 0x94d, 0x91f, 0x92c, 0x941, 0x938, 0x93f, 0x928,
0x94d, 0x928, 0x917, 0x928, 0x91a, 0x93f, 0x938, 0x93f, 0x928, 0x93e, 0x909, 0x915,
0x94b, 0x92a, 0x947, 0x928, 0x939, 0x94d, 0x92f, 0x93e, 0x917, 0x928, 0x917, 0x941,
0x90f, 0x930, 0x94d, 0x928, 0x938, 0x947, 0x939, 0x947, 0x932, 0x94d, 0x938, 0x93f,
0x928, 0x94d, 0x915, 0x940, 0x907, 0x91c, 0x932, 0x20, 0x905, 0x92b, 0x94d, 0x20,
0x92e, 0x94d, 0x92f, 0x93e, 0x928, 0x908, 0x938, 0x94d, 0x91f, 0x93e, 0x928, 0x92c,
0x941, 0x932, 0x915, 0x93e, 0x932, 0x93f, 0x928, 0x93f, 0x928, 0x917, 0x94d, 0x930,
0x93e, 0x926, 0x915, 0x93f, 0x930, 0x94b, 0x92d, 0x915, 0x93f, 0x92d, 0x932, 0x93f,
0x938, 0x94d, 0x92c, 0x94b, 0x928, 0x932, 0x91c, 0x941, 0x92c, 0x93f, 0x91c, 0x93e,
0x928, 0x93e, 0x932, 0x923, 0x94d, 0x921, 0x928, 0x932, 0x915, 0x94d, 0x91c, 0x947,
0x92e, 0x94d, 0x935, 0x930, 0x94d, 0x917, 0x92e, 0x94d, 0x92f, 0x93e, 0x921, 0x94d,
0x930, 0x93f, 0x921, 0x92e, 0x94d, 0x92f, 0x93e, 0x930, 0x93f, 0x939, 0x94d, 0x92f,
0x93e, 0x92e, 0x94d, 0x928, 0x92e, 0x938, 0x94d, 0x915, 0x94b, 0x92a, 0x921, 0x94d,
0x917, 0x94b, 0x930, 0x93f, 0x915, 0x93e, 0x938, 0x93e, 0x928, 0x20, 0x92e, 0x93e,
0x930, 0x93f, 0x928, 0x94b, 0x938, 0x93e, 0x930, 0x93e, 0x91c, 0x947, 0x92d, 0x94b,
0x938, 0x93e, 0x930, 0x93e, 0x91f, 0x94b, 0x92d, 0x938, 0x94d, 0x915, 0x94b, 0x92a,
0x91c, 0x947, 0x938, 0x94d, 0x91f, 0x915, 0x939, 0x94b, 0x932, 0x94d, 0x92e, 0x924,
0x93e, 0x932, 0x94d, 0x932, 0x93f, 0x928, 0x909, 0x932, 0x94d, 0x92f, 0x93e, 0x928,
0x94b, 0x92d, 0x94d, 0x938, 0x94d, 0x915, 0x92d, 0x94d, 0x92f, 0x93e, 0x91f, 0x93f,
0x915, 0x928, 0x92d, 0x93f, 0x92f, 0x928, 0x93e, 0x92d, 0x93f, 0x932, 0x94d, 0x928,
0x93f, 0x905, 0x938, 0x92d, 0x94b, 0x932, 0x94d, 0x917, 0x94b, 0x917, 0x94d, 0x930,
0x93e, 0x926, 0x935, 0x93e, 0x930, 0x938, 0x905, 0x91c, 0x941, 0x930, 0x93f, 0x915,
0x905, 0x928, 0x94d, 0x91f, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x92d, 0x94b, 0x916,
0x94d, 0x930, 0x93f, 0x938, 0x94d, 0x91f, 0x92e, 0x938, 0x915, 0x947, 0x930, 0x94d,
0x917, 0x941, 0x90f, 0x932, 0x947, 0x928, 0x92e, 0x93e, 0x932, 0x94d, 0x926, 0x93f,
0x92d, 0x94d, 0x938, 0x92e, 0x909, 0x930, 0x93f, 0x91f, 0x93f, 0x905, 0x938, 0x905,
0x915, 0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x92c, 0x941, 0x917, 0x947,
0x928, 0x92d, 0x93f, 0x932, 0x94d, 0x932, 0x947, 0x907, 0x938, 0x94d, 0x91f, 0x930,
0x908, 0x92b, 0x93e, 0x924, 0x947, 0x92b, 0x941, 0x928, 0x93e, 0x92b, 0x941, 0x924,
0x93f, 0x917, 0x932, 0x93e, 0x92a, 0x93e, 0x917, 0x94b, 0x938, 0x917, 0x94d, 0x92f,
0x93e, 0x92e, 0x94d, 0x92c, 0x93f, 0x92f, 0x930, 0x917, 0x941, 0x905, 0x921, 0x93e,
0x932, 0x915, 0x928, 0x93e, 0x932, 0x90f, 0x928, 0x94d, 0x921, 0x930, 0x92c, 0x930,
0x940, 0x915, 0x93f, 0x930, 0x93f, 0x924, 0x93f, 0x92e, 0x93e, 0x91f, 0x940, 0x915,
0x94b, 0x938, 0x94d, 0x930, 0x93e, 0x90f, 0x915, 0x94d, 0x935, 0x93e, 0x91c, 0x93e,
0x932, 0x947, 0x907, 0x928, 0x928, 0x93e, 0x909, 0x930, 0x941, 0x928, 0x93f, 0x909,
0x908, 0x928, 0x930, 0x92b, 0x94b, 0x932, 0x94d, 0x915, 0x928, 0x94b, 0x909, 0x92e,
0x947, 0x905, 0x92a, 0x932, 0x93e, 0x909, 0x92a, 0x93f, 0x924, 0x915, 0x93e, 0x908,
0x930, 0x928, 0x92a, 0x94b, 0x928, 0x93e, 0x92a, 0x947, 0x930, 0x93e, 0x913, 0x924,
0x94b, 0x902, 0x917, 0x93e, 0x938, 0x93e, 0x908, 0x92a, 0x928, 0x924, 0x930, 0x935,
0x93e, 0x91f, 0x902, 0x917, 0x93e, 0x924, 0x92a, 0x941, 0xc1, 0x62, 0x69, 0x6a,
0x61, 0x6e, 0x41, 0x6b, 0x72, 0xe1, 0x41, 0x64, 0xed, 0x20, 0x41, 0x62,
0xe1, 0x62, 0x61, 0x41, 0x6c, 0x6a, 0xed, 0x1eb9, 0x7a, 0x41, 0x73, 0x6d,
0xe1, 0x72, 0x61, 0x42, 0x61, 0x6d, 0xe1, 0x6b, 0x6f, 0x42, 0x61, 0x6e,
0x67, 0xfa, 0x69, 0x42, 0x6c, 0x61, 0x6e, 0x74, 0xe1, 0x79, 0x61, 0x42,
0x72, 0xe1, 0x7a, 0x61, 0x76, 0x69, 0x6c, 0x42, 0x75, 0x6a, 0x75, 0x6d,
0x62, 0xfa, 0x72, 0x61, 0x4b, 0x61, 0xed, 0x72, 0x6f, 0x4b, 0x61, 0x73,
0x61, 0x62, 0x6c, 0xe1, 0x6e, 0x6b, 0x61, 0x53, 0x1eb9, 0xfa, 0x74, 0x61,
0x4b, 0x1ecd, 0x6e, 0xe1, 0x6b, 0x72, 0x69, 0x44, 0x61, 0x6b, 0xe1, 0x72,
0x44, 0x61, 0x72, 0x20, 0x1eb9, 0x301, 0x73, 0x20, 0x53, 0x61, 0x6c, 0x61,
0x6d, 0x4a, 0x69, 0x62, 0xfa, 0x74, 0x69, 0x44, 0x75, 0xe1, 0x6c, 0x61,
0x1eb8, 0x6c, 0x20, 0x41, 0x69, 0x75, 0x6e, 0x46, 0x72, 0xed, 0x74, 0x61,
0x75, 0x6e, 0x48, 0xe1, 0x62, 0x6f, 0x72, 0xf3, 0x6e, 0x1eb9, 0x48, 0x61,
0x72, 0xe1, 0x72, 0x1eb9, 0x4a, 0x6f, 0x68, 0xe1, 0x6e, 0xed, 0x73, 0x62,
0x1ecd, 0x67, 0x4a, 0xfa, 0x62, 0x61, 0x4b, 0x61, 0x6d, 0x70, 0xe1, 0x6c,
0x61, 0x4b, 0x69, 0x67, 0xe1, 0x6c, 0x69, 0x4b, 0x69, 0x6e, 0x73, 0x68,
0xe1, 0x73, 0x68, 0x61, 0x4c, 0xe9, 0x67, 0x6f, 0x73, 0x4c, 0xed, 0x62,
0x72, 0x1eb9, 0x76, 0x69, 0x6c, 0x4c, 0xf3, 0x6d, 0x1eb9, 0x4c, 0x75, 0xe1,
0x6e, 0x64, 0x61, 0x4c, 0x75, 0x62, 0x75, 0x6d, 0x62, 0xe1, 0x73, 0x68,
0x69, 0x4c, 0x75, 0x73, 0xe1, 0x6b, 0x61, 0x4d, 0x61, 0x6c, 0xe1, 0x62,
0x6f, 0x4d, 0x61, 0x70, 0xfa, 0x74, 0x6f, 0x4d, 0x61, 0x73, 0x1eb9, 0x301,
0x72, 0x75, 0x4d, 0x62, 0x61, 0x62, 0xe1, 0x6e, 0x1eb9, 0x4d, 0x1ecd, 0x67,
0xe1, 0x64, 0xed, 0x73, 0x68, 0x75, 0x4d, 0x6f, 0x6e, 0x72, 0xf3, 0x76,
0x69, 0x61, 0x4e, 0x61, 0xed, 0x72, 0xf3, 0x62, 0x69, 0x4e, 0x6a, 0x61,
0x6d, 0x1eb9, 0x301, 0x6e, 0x61, 0x4e, 0x69, 0xe1, 0x6d, 0x65, 0x4e, 0x6f,
0x75, 0xe1, 0x6b, 0x73, 0x68, 0x1ecd, 0x74, 0x4f, 0x75, 0x61, 0x67, 0x61,
0x64, 0xfa, 0x67, 0x75, 0x50, 0x1ecd, 0x74, 0x6f, 0x2d, 0x4e, 0xf3, 0x76,
0x6f, 0x53, 0x61, 0x6f, 0x20, 0x54, 0xf3, 0x6d, 0x1eb9, 0x54, 0x72, 0xed,
0x70, 0x1ecd, 0x301, 0x6c, 0x69, 0x57, 0xed, 0x6e, 0x64, 0x68, 0x6f, 0x6b,
0xc1, 0x64, 0x61, 0x6b, 0xc1, 0x6e, 0x6b, 0x1ecd, 0x301, 0x72, 0x65, 0x6a,
0x41, 0x6e, 0x67, 0x77, 0xed, 0x6c, 0x61, 0x41, 0x6e, 0x74, 0xed, 0x67,
0x77, 0x75, 0x61, 0x41, 0x72, 0x61, 0x67, 0x77, 0x75, 0x61, 0xed, 0x6e,
0x61, 0x42, 0x75, 0x1eb9, 0x6e, 0x6f, 0x73, 0x20, 0x41, 0xed, 0x72, 0x1eb9,
0x73, 0x4b, 0x61, 0x74, 0x61, 0x6d, 0xe1, 0x6b, 0x61, 0x4b, 0xf3, 0x72,
0x64, 0x6f, 0x62, 0x61, 0x4c, 0x61, 0x20, 0x52, 0x69, 0xf3, 0x6b, 0x68,
0x61, 0x4d, 0x1eb9, 0x6e, 0x64, 0xf3, 0x7a, 0x61, 0x52, 0x69, 0xf3, 0x20,
0x47, 0x61, 0x6c, 0x1eb9, 0x301, 0x67, 0x1ecd, 0x73, 0x53, 0xe1, 0x6c, 0x74,
0x61, 0x53, 0xe1, 0x6e, 0x20, 0x48, 0x77, 0xe1, 0x6e, 0x53, 0xe1, 0x6e,
0x20, 0x4c, 0x75, 0x69, 0x73, 0x54, 0xfa, 0x6b, 0xfa, 0x6d, 0x61, 0x6e,
0x55, 0x73, 0x75, 0xe1, 0x79, 0x61, 0x41, 0x73, 0x75, 0x6e, 0x73, 0x69,
0x1ecd, 0x6e, 0x41, 0x74, 0xed, 0x6b, 0xf3, 0x6b, 0x61, 0x6e, 0x42, 0x61,
0x68, 0xed, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x1eb9, 0x301, 0x72, 0x61,
0x73, 0x42, 0x61, 0x72, 0x62, 0xe9, 0x64, 0x1ecd, 0x73, 0x42, 0x1eb9, 0x6c,
0x1eb9, 0x6d, 0x42, 0x1eb9, 0x6c, 0x69, 0x7a, 0x42, 0x6c, 0x61, 0x6e, 0x6b,
0x2d, 0x53, 0xe1, 0x62, 0x6c, 0x1ecd, 0x6e, 0x42, 0xf3, 0x61, 0x20, 0x56,
0xed, 0x73, 0x74, 0x61, 0x42, 0x1ecd, 0xed, 0x73, 0x69, 0x4b, 0xe9, 0x6d,
0x62, 0x72, 0xed, 0x6a, 0x20, 0x42, 0xe8, 0x4b, 0x61, 0x6d, 0x70, 0xf3,
0x20, 0x47, 0x72, 0xe1, 0x6e, 0x64, 0x1eb9, 0x4b, 0x61, 0x72, 0xe1, 0x6b,
0x61, 0x73, 0x4b, 0x61, 0x79, 0x1eb9, 0x6e, 0x4b, 0xe9, 0x6d, 0x61, 0x6e,
0x43, 0x68, 0x69, 0x6b, 0xe1, 0x67, 0x6f, 0x43, 0x68, 0x69, 0x77, 0x75,
0xe1, 0x77, 0x75, 0x61, 0x4b, 0xf3, 0x73, 0x74, 0x61, 0x20, 0x52, 0xed,
0x6b, 0x61, 0x4b, 0x72, 0x1eb9, 0x301, 0x73, 0x74, 0x1ecd, 0x6e, 0x4b, 0xfa,
0x79, 0xe1, 0x62, 0x61, 0x61, 0x4b, 0x69, 0x75, 0x72, 0xe1, 0x73, 0x61,
0x6f, 0x44, 0x61, 0x6e, 0x6d, 0xe1, 0x6b, 0x73, 0x68, 0xe1, 0x76, 0x75,
0x6e, 0x44, 0x1ecd, 0x301, 0x73, 0x1ecd, 0x6e, 0x44, 0x1ecd, 0x301, 0x73, 0x1ecd,
0x6e, 0x20, 0x4b, 0x72, 0x69, 0x6b, 0x44, 0x1eb9, 0x301, 0x6e, 0x76, 0x61,
0x44, 0x69, 0x74, 0x72, 0x1ecd, 0x69, 0x74, 0x44, 0x1ecd, 0x6d, 0xed, 0x6e,
0xed, 0x6b, 0x61, 0x1eb8, 0x301, 0x64, 0x6d, 0x1ecd, 0x301, 0x6e, 0x74, 0x1ecd,
0x6e, 0x1eb8, 0x69, 0x72, 0x75, 0x6e, 0x1eb9, 0x70, 0x1eb9, 0x45, 0x6c, 0x20,
0x53, 0xe1, 0x6c, 0x76, 0xe1, 0x64, 0x1ecd, 0x46, 0x1ecd, 0x74, 0x20, 0x4e,
0x1eb9, 0x301, 0x6c, 0x73, 0x6f, 0x6e, 0x46, 0x1ecd, 0x74, 0x61, 0x6c, 0x1eb9,
0x301, 0x7a, 0x61, 0x47, 0x6c, 0xe1, 0x73, 0x20, 0x42, 0xe8, 0x47, 0xfa,
0x7a, 0x20, 0x42, 0xe8, 0x47, 0x72, 0xe1, 0x6e, 0x64, 0x20, 0x54, 0x1ecd,
0x6b, 0x47, 0x72, 0x1eb9, 0x6e, 0xe9, 0x64, 0x61, 0x47, 0x75, 0x61, 0x64,
0x61, 0x6c, 0xfa, 0x70, 0x1eb9, 0x47, 0x75, 0xe1, 0x74, 0x1eb9, 0x6d, 0xe1,
0x6c, 0x61, 0x47, 0x75, 0xe1, 0x79, 0x61, 0x6b, 0x69, 0x6c, 0x47, 0x61,
0x79, 0xe1, 0x6e, 0x61, 0x48, 0xe1, 0x6c, 0xed, 0x66, 0x61, 0x6b, 0x73,
0x48, 0x61, 0x76, 0xe1, 0x6e, 0x61, 0x48, 0x1eb9, 0x301, 0x6d, 0xf3, 0x73,
0xed, 0x6c, 0x6f, 0x49, 0x6e, 0x64, 0x69, 0xe1, 0x6e, 0xe1, 0x70, 0x1ecd,
0x301, 0x6c, 0x69, 0x73, 0x4e, 0x1ecd, 0x6b, 0x73, 0x2c, 0x20, 0x49, 0x6e,
0x64, 0x69, 0xe1, 0x6e, 0x61, 0x4d, 0x61, 0x72, 0x1eb9, 0x301, 0x6e, 0x67,
0x6f, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0xe1, 0x6e, 0x61, 0x50, 0xed,
0x74, 0xe1, 0x73, 0x62, 0x1ecd, 0x67, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69,
0xe1, 0x6e, 0x61, 0x54, 0x1eb9, 0x301, 0x6c, 0x20, 0x53, 0xed, 0x74, 0x69,
0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0xe1, 0x6e, 0x61, 0x56, 0x1eb9, 0x301,
0x76, 0x65, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0xe1, 0x6e, 0x61, 0x56,
0xed, 0x6e, 0x73, 0x1eb9, 0x6e, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0xe1,
0x6e, 0x61, 0x57, 0xed, 0x6e, 0xe1, 0x6d, 0x61, 0x6b, 0x2c, 0x20, 0x49,
0x6e, 0x64, 0x69, 0xe1, 0x6e, 0x61, 0x49, 0x6e, 0xfa, 0x76, 0x69, 0x6b,
0x49, 0x6b, 0xe1, 0x6c, 0x75, 0x69, 0x74, 0x4a, 0xfa, 0x6e, 0x6f, 0x4c,
0x75, 0xed, 0x76, 0x69, 0x6c, 0x4d, 0x1ecd, 0x6e, 0x74, 0x1eb9, 0x63, 0x68,
0x1eb9, 0x301, 0x6c, 0x6f, 0x2c, 0x20, 0x4b, 0x1eb9, 0x301, 0x6e, 0x74, 0x1ecd,
0x301, 0x6b, 0x69, 0x4b, 0x72, 0xe1, 0x6c, 0x1eb9, 0x301, 0x6e, 0x64, 0x69,
0x6a, 0x6b, 0x4c, 0xed, 0x6d, 0x61, 0x4c, 0x1ecd, 0x73, 0x20, 0xc1, 0x6e,
0x6a, 0x1eb9, 0x301, 0x6c, 0x69, 0x73, 0x4c, 0xf3, 0x77, 0xe1, 0x20, 0x50,
0x72, 0xed, 0x6e, 0x73, 0x20, 0x49, 0x6d, 0x20, 0x4b, 0x77, 0x1ecd, 0x301,
0x74, 0x61, 0x4d, 0x61, 0x73, 0x1eb9, 0x301, 0x69, 0x6f, 0x4d, 0x61, 0x6e,
0xe1, 0x75, 0x73, 0x4d, 0xe1, 0x72, 0xed, 0x67, 0x1ecd, 0x74, 0x4d, 0x61,
0x74, 0xed, 0x6e, 0x69, 0x6b, 0x4d, 0xe1, 0x74, 0x61, 0x6d, 0xf3, 0x72,
0x6f, 0x73, 0x4d, 0x61, 0x7a, 0xe1, 0x74, 0x6c, 0x61, 0x6e, 0x4d, 0x1eb9,
0x6e, 0x1ecd, 0x301, 0x6d, 0x69, 0x6e, 0x69, 0x69, 0x4d, 0x1eb9, 0x301, 0x72,
0xed, 0x64, 0x61, 0x4d, 0x1eb9, 0x74, 0x6c, 0x61, 0x6b, 0xe1, 0x74, 0x6c,
0x61, 0x4d, 0x1eb9, 0x301, 0x6b, 0x73, 0xed, 0x6b, 0xf3, 0x20, 0x53, 0xed,
0x74, 0x69, 0x4d, 0xed, 0x6b, 0x1eb9, 0x6c, 0x1ecd, 0x6e, 0x4d, 0x1ecd, 0x301,
0x6e, 0x6b, 0x74, 0x1ecd, 0x6e, 0x4d, 0x1ecd, 0x6e, 0x74, 0x1eb9, 0x72, 0x1eb9,
0x1eb9, 0x4d, 0x1ecd, 0x6e, 0x74, 0x1eb9, 0x76, 0x69, 0x64, 0x1eb9, 0x6f, 0x4d,
0x1ecd, 0x6e, 0x74, 0x73, 0x1eb9, 0x72, 0x61, 0x74, 0x4e, 0xe1, 0x73, 0x73,
0x1ecd, 0x75, 0x4e, 0x69, 0xfa, 0x20, 0x59, 0x1ecd, 0x6b, 0x4e, 0x6f, 0x6f,
0x6d, 0x4e, 0x1ecd, 0x72, 0xf3, 0x6e, 0x69, 0x61, 0x42, 0x69, 0xfa, 0x6c,
0x61, 0x2c, 0x20, 0x4e, 0x1ecd, 0x301, 0x74, 0x20, 0x44, 0x61, 0x6b, 0xf3,
0x74, 0x61, 0x53, 0x1eb9, 0x301, 0x6e, 0x74, 0x61, 0x2c, 0x20, 0x4e, 0x1ecd,
0x301, 0x74, 0x20, 0x44, 0x61, 0x6b, 0xf3, 0x74, 0x61, 0x4e, 0x69, 0xfa,
0x20, 0x53, 0xe9, 0x6c, 0x1eb9, 0x6d, 0x2c, 0x20, 0x4e, 0x1ecd, 0x301, 0x74,
0x20, 0x44, 0x61, 0x6b, 0xf3, 0x74, 0x61, 0x4f, 0x6b, 0x68, 0x69, 0x6e,
0xe1, 0x67, 0x61, 0x50, 0xe1, 0x6e, 0xe1, 0x6d, 0x61, 0x50, 0xe1, 0x72,
0xe1, 0x6d, 0x61, 0x72, 0x69, 0x62, 0x6f, 0x46, 0xed, 0x6e, 0x69, 0x6b,
0x73, 0x50, 0x1ecd, 0x74, 0x2d, 0x6f, 0x2d, 0x50, 0x72, 0x69, 0x6e, 0x73,
0x50, 0x1ecd, 0x301, 0x74, 0x20, 0x1ecd, 0x66, 0x20, 0x53, 0x70, 0x65, 0x6e,
0x50, 0x1ecd, 0x74, 0x6f, 0x20, 0x56, 0x1eb9, 0x301, 0x6c, 0x68, 0x6f, 0x50,
0x75, 0x1eb9, 0x301, 0x72, 0x74, 0x6f, 0x20, 0x52, 0xed, 0x6b, 0x1ecd, 0x50,
0xfa, 0x6e, 0x74, 0xe1, 0x20, 0x41, 0x72, 0x1eb9, 0x301, 0x6e, 0x61, 0x73,
0x52, 0xe1, 0x6e, 0x6b, 0xed, 0x6e, 0x20, 0xcd, 0x6e, 0x6c, 0x1eb9, 0x74,
0x52, 0x1eb9, 0x73, 0xed, 0x66, 0x1eb9, 0x52, 0x1eb9, 0x6a, 0xed, 0x6e, 0x61,
0x52, 0x1eb9, 0x301, 0x7a, 0xf3, 0x6c, 0x75, 0x74, 0x52, 0x69, 0xf3, 0x20,
0x42, 0x72, 0xe1, 0x6e, 0x6b, 0x6f, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x72,
0x1eb9, 0x6d, 0x53, 0x61, 0x6e, 0x74, 0x69, 0xe1, 0x67, 0x6f, 0x53, 0xe1,
0x6e, 0x74, 0xf3, 0x20, 0x44, 0x6f, 0x6d, 0xed, 0x6e, 0x67, 0x6f, 0x53,
0x61, 0x6f, 0x20, 0x50, 0x61, 0xfa, 0x6c, 0x6f, 0x49, 0x74, 0xf3, 0x6b,
0x6f, 0x74, 0xfa, 0x72, 0x6d, 0x69, 0x74, 0x53, 0x65, 0x6e, 0x74, 0x20,
0x42, 0x61, 0x74, 0x1eb9, 0x6c, 0x1eb9, 0x301, 0x6d, 0x69, 0x53, 0x65, 0x6e,
0x74, 0x20, 0x4a, 0x1ecd, 0x6e, 0x53, 0x65, 0x6e, 0x74, 0x20, 0x4c, 0xfa,
0x73, 0x68, 0x69, 0x61, 0x53, 0x65, 0x6e, 0x74, 0x20, 0x54, 0x1ecd, 0x6d,
0x1ecd, 0x73, 0x53, 0x65, 0x6e, 0x74, 0x20, 0x56, 0xed, 0x6e, 0x73, 0x1eb9,
0x6e, 0x74, 0x53, 0x77, 0xed, 0x66, 0x74, 0x20, 0x4b, 0x1ecd, 0x301, 0x72,
0x1eb9, 0x6e, 0x74, 0x54, 0x1eb9, 0x67, 0xfa, 0x73, 0x69, 0x67, 0xe1, 0x6c,
0x70, 0x61, 0x54, 0xfa, 0x6c, 0x69, 0x54, 0x69, 0x6b, 0x68, 0x75, 0xe1,
0x6e, 0x61, 0x54, 0x6f, 0x72, 0xf3, 0x6e, 0x74, 0x6f, 0x54, 0x1ecd, 0x74,
0xf3, 0x6c, 0x61, 0x56, 0x61, 0x6e, 0x6b, 0xfa, 0x76, 0x61, 0x57, 0x61,
0xed, 0x74, 0x68, 0x1ecd, 0x73, 0x57, 0xed, 0x6e, 0xed, 0x70, 0x1eb9, 0x67,
0x59, 0x61, 0x6b, 0xfa, 0x74, 0x61, 0x74, 0x4b, 0xe9, 0x73, 0x69, 0x44,
0xe9, 0x76, 0x69, 0x73, 0x44, 0x69, 0xfa, 0x6d, 0x1ecd, 0x301, 0x6e, 0x2d,
0x64, 0x2019, 0x55, 0x76, 0x69, 0x6c, 0x4d, 0x61, 0x6b, 0x77, 0x75, 0xe9,
0xed, 0x4d, 0x1ecd, 0x301, 0x73, 0x1ecd, 0x6e, 0x4d, 0x63, 0x4d, 0x1ecd, 0x301,
0x64, 0x6f, 0x50, 0xe1, 0x6d, 0x61, 0x52, 0x6f, 0x74, 0x1eb9, 0x301, 0x72,
0x61, 0x53, 0x69, 0xf3, 0x77, 0x61, 0x56, 0x1ecd, 0x301, 0x73, 0x74, 0x1ecd,
0x6b, 0x4c, 0x1ecd, 0x6e, 0x67, 0x79, 0x1eb9, 0x301, 0x61, 0x62, 0x69, 0x1eb9,
0x6e, 0xc9, 0x64, 0x1eb9, 0x6e, 0xc1, 0x6c, 0x6d, 0xe1, 0x74, 0x69, 0xc1,
0x6e, 0x61, 0x64, 0x69, 0x61, 0x72, 0x41, 0x6b, 0x74, 0xe1, 0x75, 0x41,
0x6b, 0x74, 0xf3, 0x62, 0x1eb9, 0xc1, 0x73, 0x68, 0x67, 0x61, 0x62, 0x61,
0x74, 0xc1, 0x74, 0xed, 0x72, 0x61, 0x75, 0x42, 0xe1, 0x67, 0x64, 0x61,
0x64, 0x42, 0x61, 0x68, 0x72, 0xe9, 0x6e, 0x42, 0xe1, 0x6b, 0x75, 0x42,
0xe1, 0x6e, 0x6b, 0x1ecd, 0x6b, 0x42, 0xe1, 0x72, 0x6e, 0xe1, 0x75, 0x6c,
0x42, 0x1eb9, 0x72, 0x75, 0x74, 0x42, 0x69, 0x73, 0x68, 0x6b, 0x1eb9, 0x6b,
0x42, 0x72, 0x75, 0x6e, 0x1eb9, 0x69, 0x43, 0x68, 0xed, 0x74, 0xe1, 0x4b,
0x6f, 0x6c, 0xf3, 0x6d, 0x62, 0x6f, 0x44, 0x61, 0x6d, 0xe1, 0x73, 0x6b,
0x1ecd, 0x73, 0x44, 0xe1, 0x6b, 0x61, 0x44, 0xed, 0x6c, 0x69, 0x44, 0x75,
0x73, 0x68, 0xe1, 0x6e, 0x62, 0x1eb9, 0x46, 0x61, 0x67, 0xfa, 0x73, 0x74,
0x61, 0x48, 0x1eb9, 0x301, 0x62, 0x72, 0x1ecd, 0x6e, 0x48, 0xf3, 0x20, 0x43,
0x68, 0x69, 0x20, 0x4d, 0xed, 0x6e, 0x20, 0x53, 0xed, 0x74, 0x69, 0x48,
0x1ecd, 0x6e, 0x67, 0x20, 0x4b, 0x1ecd, 0x6e, 0x67, 0x49, 0x72, 0x6b, 0xfa,
0x74, 0x73, 0x6b, 0x4a, 0x61, 0x6b, 0xe1, 0x74, 0x61, 0x4a, 0x61, 0x79,
0x61, 0x70, 0xfa, 0x72, 0x61, 0x4a, 0x1eb9, 0x72, 0xfa, 0x73, 0xe1, 0x6c,
0x1eb9, 0x6d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0xe1, 0x74, 0x6b, 0xe9, 0x4b,
0x61, 0x72, 0xe1, 0x63, 0x68, 0x69, 0x4b, 0x61, 0x74, 0x6d, 0xe1, 0x6e,
0x64, 0x75, 0x4b, 0x61, 0x6e, 0x64, 0xed, 0x67, 0x61, 0x4b, 0x6f, 0x6c,
0x6b, 0xe1, 0x74, 0x61, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x73, 0x6b, 0x4b, 0x75, 0xe1, 0x6c, 0x61, 0x20, 0x4c, 0xfa, 0x6d, 0x70,
0x1ecd, 0x4b, 0x75, 0x77, 0x65, 0x74, 0x4d, 0x61, 0x6b, 0xe1, 0x6f, 0x4d,
0xe1, 0x67, 0xe1, 0x64, 0x61, 0x6e, 0x4d, 0x61, 0x6b, 0xe1, 0x73, 0x61,
0x4d, 0x61, 0x6e, 0xed, 0x6c, 0x61, 0x4d, 0xfa, 0x73, 0x6b, 0x61, 0x74,
0x4e, 0x6f, 0x76, 0x6f, 0x6b, 0x75, 0x7a, 0x6e, 0x1eb9, 0x301, 0x73, 0x6b,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x73, 0x6b, 0x1ecc, 0x6d,
0x73, 0x6b, 0x1ecc, 0x301, 0x72, 0x61, 0x6c, 0x46, 0x6e, 0x1ecd, 0x301, 0x6d,
0x20, 0x50, 0x1eb9, 0x6e, 0x50, 0x1ecd, 0x6e, 0x74, 0x69, 0xe1, 0x6e, 0x61,
0x6b, 0x50, 0x69, 0x1ecd, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x4b, 0xe1,
0x74, 0x61, 0x4b, 0x6f, 0x73, 0x74, 0xe1, 0x6e, 0x61, 0x69, 0x4b, 0xed,
0x7a, 0x69, 0x6c, 0x1ecd, 0x301, 0x64, 0x61, 0x52, 0x69, 0x79, 0xe1, 0x64,
0x53, 0xe1, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x53, 0xe1, 0x6d, 0xe1,
0x6b, 0x61, 0x6e, 0x64, 0x53, 0x6f, 0x6c, 0x53, 0x68, 0xe1, 0x6e, 0x67,
0x68, 0x61, 0x69, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x1ecd, 0x53, 0x72,
0x1eb9, 0x301, 0x64, 0x6e, 0x1eb9, 0x6b, 0x6f, 0x6c, 0x69, 0x6d, 0x73, 0x6b,
0x54, 0x61, 0x69, 0x70, 0x1eb9, 0x69, 0x54, 0xe1, 0x73, 0x68, 0x6b, 0x1eb9,
0x6e, 0x74, 0x54, 0x69, 0x62, 0x6c, 0xed, 0x73, 0x69, 0x54, 0x1eb9, 0x72,
0x61, 0x6e, 0x54, 0xed, 0x6d, 0x66, 0x75, 0x54, 0xf3, 0x6b, 0x79, 0x6f,
0x55, 0x6c, 0x61, 0x6e, 0x62, 0xe1, 0x74, 0x61, 0x59, 0x75, 0x72, 0x1ecd,
0x301, 0x6d, 0x6b, 0x69, 0x55, 0x73, 0x74, 0x2d, 0x4e, 0x1eb9, 0x301, 0x72,
0x61, 0x56, 0x69, 0x1eb9, 0x301, 0x6e, 0x74, 0x69, 0x1eb9, 0x6e, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x1ecd, 0x73, 0x74, 0x1ecd, 0x6b, 0x59, 0xe9, 0x6b,
0xfa, 0x74, 0x73, 0x6b, 0x59, 0x61, 0x6e, 0x67, 0x1ecd, 0x6e, 0x59, 0x1eb9,
0x6b, 0x65, 0x74, 0x1eb9, 0x72, 0xed, 0x6e, 0x62, 0x75, 0x67, 0x59, 0x1eb9,
0x72, 0x1eb9, 0x301, 0x76, 0x61, 0x6e, 0x41, 0x7a, 0x1ecd, 0x7a, 0x42, 0x1eb9,
0x6d, 0x69, 0xfa, 0x64, 0x61, 0x4b, 0x65, 0x6e, 0x65, 0x72, 0xed, 0x4b,
0xe9, 0x70, 0x20, 0x56, 0x1eb9, 0x64, 0x46, 0xe1, 0x72, 0xf3, 0x69, 0x73,
0x4d, 0x61, 0x64, 0xed, 0x72, 0x61, 0x52, 0x1eb9, 0x301, 0x6b, 0x6a, 0xe1,
0x76, 0x69, 0x6b, 0x53, 0x61, 0xfa, 0x74, 0x20, 0x4a, 0x1ecd, 0x301, 0x6a,
0x69, 0x61, 0x53, 0x65, 0x6e, 0x74, 0x20, 0x48, 0x1eb9, 0x6c, 0x1eb9, 0x301,
0x6e, 0x61, 0x53, 0x74, 0xe1, 0x6e, 0x6c, 0x69, 0x41, 0x64, 0x6c, 0x65,
0x69, 0x64, 0x42, 0x72, 0xed, 0x73, 0x62, 0x65, 0x6e, 0x42, 0x72, 0xf3,
0x6b, 0xfa, 0x6e, 0x20, 0x48, 0x69, 0x6c, 0x44, 0xe1, 0x72, 0x77, 0x69,
0x6e, 0x59, 0xfa, 0x6b, 0x6c, 0x61, 0x48, 0xf3, 0x62, 0x61, 0x74, 0x4c,
0xed, 0x6e, 0x64, 0x1eb9, 0x6d, 0x61, 0x6e, 0x4c, 0x1ecd, 0x64, 0x20, 0x48,
0x61, 0xfa, 0x4d, 0x1eb9, 0x301, 0x6c, 0x62, 0x1ecd, 0x6e, 0x50, 0x1eb9, 0x72,
0x74, 0x53, 0xed, 0x64, 0x6e, 0x69, 0xc1, 0x6d, 0x73, 0x74, 0xe1, 0x64,
0x61, 0x6d, 0x41, 0x6e, 0x64, 0x1ecd, 0x301, 0x72, 0x61, 0xc1, 0x73, 0x74,
0x72, 0x61, 0x68, 0xe1, 0x6e, 0xc1, 0x74, 0x1eb9, 0x6e, 0x73, 0x42, 0x1eb9,
0x301, 0x6c, 0x67, 0x72, 0xe9, 0x64, 0x42, 0x1eb9, 0x6c, 0x69, 0x6e, 0x42,
0x72, 0x61, 0x74, 0xed, 0x73, 0x6c, 0xe1, 0x76, 0x61, 0x42, 0x72, 0xfa,
0x73, 0x75, 0x6c, 0x73, 0x42, 0xfa, 0x6b, 0xe1, 0x72, 0x1eb9, 0x73, 0x74,
0x42, 0xfa, 0x64, 0xe1, 0x70, 0x1eb9, 0x73, 0x74, 0x42, 0x75, 0x73, 0xed,
0x6e, 0x67, 0x1eb9, 0x6e, 0x43, 0x68, 0x69, 0x73, 0x69, 0x6e, 0xe1, 0x75,
0x4b, 0x1ecd, 0x70, 0x1eb9, 0x6e, 0x68, 0xe1, 0x67, 0x1eb9, 0x6e, 0x44, 0x1ecd,
0x301, 0x62, 0x6c, 0x69, 0x6e, 0x4a, 0x69, 0x62, 0x72, 0x1ecd, 0x301, 0x6c,
0x74, 0x61, 0x47, 0x75, 0x1eb9, 0x6e, 0x7a, 0x69, 0x48, 0x1eb9, 0x6c, 0x73,
0xed, 0x6e, 0x6b, 0x69, 0x41, 0xed, 0x6c, 0x20, 0x1ecd, 0x66, 0x20, 0x4d,
0x61, 0x6e, 0xcd, 0x73, 0x74, 0xe1, 0x6e, 0x62, 0x75, 0x6c, 0x4a, 0x1eb9,
0x301, 0x73, 0x69, 0x4b, 0x61, 0x6c, 0xed, 0x6e, 0xed, 0x6e, 0x67, 0x72,
0x61, 0x64, 0x4b, 0x69, 0x72, 0x1ecd, 0x76, 0x4b, 0x69, 0x1eb9, 0x76, 0x4c,
0xed, 0x73, 0x62, 0x1ecd, 0x6e, 0x4c, 0x75, 0x62, 0x6c, 0x69, 0xe1, 0x6e,
0x61, 0x4c, 0x1ecd, 0x301, 0x6e, 0x64, 0x1ecd, 0x6e, 0x4c, 0x1ecd, 0x301, 0x6b,
0x73, 0x1eb9, 0x301, 0x6d, 0x62, 0x1ecd, 0x67, 0x4d, 0x1ecd, 0x301, 0x6c, 0x74,
0x61, 0x4d, 0x61, 0x72, 0xed, 0x61, 0x68, 0xe1, 0x6d, 0x6e, 0x4d, 0x1ecd,
0x301, 0x6e, 0xe1, 0x6b, 0x6f, 0x4d, 0x1ecd, 0x301, 0x73, 0x6b, 0x6f, 0x1ecc,
0x301, 0x73, 0x6c, 0x6f, 0x50, 0xe1, 0x72, 0x69, 0x73, 0x50, 0x1ecd, 0x301,
0x6a, 0xf3, 0x72, 0xed, 0x6b, 0x61, 0x52, 0xed, 0x67, 0x61, 0x53, 0x61,
0x6d, 0xe1, 0x72, 0x61, 0x53, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0xed,
0x6e, 0x6f, 0x53, 0x61, 0x72, 0x61, 0x79, 0x1eb9, 0x301, 0x76, 0x6f, 0x53,
0xe1, 0x72, 0xe1, 0x74, 0x6f, 0x76, 0x53, 0xed, 0x6d, 0x66, 0x1eb9, 0x72,
0xf3, 0x70, 0x6f, 0x6c, 0x53, 0x6b, 0x1ecd, 0x301, 0x70, 0x79, 0x1eb9, 0x53,
0x74, 0x1ecd, 0x301, 0x6b, 0x68, 0x1ecd, 0x6d, 0x54, 0xe1, 0x6c, 0x69, 0x6e,
0x54, 0x69, 0x72, 0xe1, 0x6e, 0x1eb9, 0x55, 0x6c, 0x69, 0xe1, 0x6e, 0x1ecd,
0x76, 0x73, 0x6b, 0x56, 0xe1, 0x74, 0xed, 0x6b, 0x61, 0x6e, 0x56, 0x69,
0x1eb9, 0x301, 0x6e, 0x61, 0x56, 0x6f, 0x6c, 0x76, 0xf3, 0x67, 0x72, 0x61,
0x64, 0x57, 0x1ecd, 0x301, 0x73, 0x1ecd, 0x5a, 0xe1, 0x67, 0x72, 0x1eb9, 0x62,
0x5a, 0xfa, 0x72, 0x69, 0x6b, 0x41, 0x6e, 0x74, 0xe1, 0x6e, 0xe1, 0x6e,
0x61, 0x72, 0xed, 0x76, 0x6f, 0x43, 0x68, 0xe1, 0x67, 0x1ecd, 0x73, 0x4b,
0x72, 0xed, 0x73, 0x6d, 0x61, 0x73, 0x4b, 0x1ecd, 0x301, 0x6d, 0x1ecd, 0x301,
0x72, 0x6f, 0x73, 0x4b, 0x1eb9, 0x301, 0x72, 0x67, 0xfa, 0x6c, 0x1eb9, 0x6e,
0x4d, 0x61, 0x68, 0x1eb9, 0x301, 0x4d, 0x1ecd, 0x301, 0x6c, 0x64, 0x69, 0x76,
0x73, 0x4d, 0x1ecd, 0x72, 0xed, 0x73, 0x68, 0x1ecd, 0x73, 0x4d, 0x65, 0x79,
0x1ecd, 0x74, 0x52, 0x69, 0x79, 0xfa, 0x6e, 0x69, 0x1ecd, 0x6e, 0xc1, 0x70,
0x69, 0x61, 0x1ecc, 0x301, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x42, 0x75, 0x67,
0x65, 0x6e, 0x76, 0xed, 0x6c, 0x69, 0x61, 0x43, 0x68, 0xe1, 0x74, 0x61,
0x6d, 0x43, 0x68, 0x75, 0x6b, 0xcd, 0x73, 0x74, 0x61, 0x1eb8, 0x66, 0xe1,
0x74, 0x1eb9, 0x46, 0x61, 0x6b, 0xe1, 0xf3, 0x66, 0x6f, 0x46, 0xed, 0x6a,
0x69, 0x46, 0x75, 0x6e, 0x61, 0x66, 0xfa, 0x74, 0x69, 0x47, 0x61, 0x6c,
0xe1, 0x70, 0xe1, 0x67, 0x1ecd, 0x73, 0x47, 0xe1, 0x6d, 0x62, 0x69, 0x1eb9,
0x72, 0x47, 0x75, 0xe1, 0x64, 0xe1, 0x6c, 0x6b, 0x61, 0x6e, 0x61, 0x6c,
0x1eb8, 0x301, 0x6e, 0x64, 0xe1, 0x62, 0x1eb9, 0x301, 0x72, 0x69, 0x4b, 0x72,
0x69, 0x74, 0xed, 0x6d, 0xe1, 0x74, 0x69, 0x4b, 0x1ecd, 0x301, 0x73, 0x72,
0x61, 0x1eb9, 0x4b, 0x77, 0xe1, 0x6a, 0x61, 0x6c, 0x1eb9, 0x6e, 0x4d, 0x61,
0x6a, 0xfa, 0x72, 0x6f, 0x4d, 0x61, 0x6b, 0x77, 0x1eb9, 0x301, 0x73, 0x61,
0x73, 0x4d, 0xed, 0x64, 0x77, 0xe8, 0x4e, 0x61, 0xfa, 0x72, 0x75, 0x4e,
0x1ecd, 0x301, 0x72, 0x66, 0x1ecd, 0x301, 0x6c, 0x6b, 0x4e, 0xfa, 0x6d, 0x1eb9,
0x301, 0x61, 0x50, 0xe1, 0x67, 0x6f, 0x20, 0x50, 0xe1, 0x67, 0x6f, 0x50,
0x61, 0x6c, 0xe1, 0x75, 0x50, 0xed, 0x74, 0x6b, 0x61, 0x6e, 0x50, 0x1ecd,
0x6e, 0x70, 0x1eb9, 0x301, 0x69, 0x50, 0x1ecd, 0x74, 0x20, 0x4d, 0x1ecd, 0x72,
0x1eb9, 0x301, 0x73, 0x62, 0x69, 0x52, 0x61, 0x72, 0x61, 0x74, 0xf3, 0x6e,
0x67, 0x61, 0x54, 0x61, 0x68, 0xed, 0x74, 0x69, 0x54, 0x61, 0x72, 0xe1,
0x77, 0x61, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x74, 0xe1, 0x70, 0x75, 0x57,
0x65, 0x6b, 0x57, 0xe1, 0x6c, 0x69, 0x7ca, 0x7d3, 0x7cc, 0x7d6, 0x7ca, 0x7f2,
0x7ec, 0x7ca, 0x7de, 0x7d9, 0x7ca, 0x7eb, 0x7ca, 0x7ec, 0x7d8, 0x7cc, 0x7eb, 0x20,
0x7db, 0x7ca, 0x7ec, 0x7d3, 0x7cb, 0x7d3, 0x7ca, 0x7eb, 0x7ca, 0x7df, 0x7d1, 0x7d6,
0x7ed, 0x7cb, 0x7ca, 0x7db, 0x7d1, 0x7e1, 0x7d9, 0x7ca, 0x7eb, 0x7d3, 0x7e1, 0x7ca,
0x7ec, 0x7de, 0x7d0, 0x7eb, 0x7d3, 0x7ca, 0x7f2, 0x7dc, 0x7ed, 0x7cc, 0x7eb, 0x7d3,
0x7ca, 0x7f2, 0x7d6, 0x7ce, 0x7df, 0x7d3, 0x7cc, 0x7db, 0x7ca, 0x7e5, 0x7cf, 0x7eb,
0x7d3, 0x7d1, 0x7df, 0x7ca, 0x7f2, 0x7d5, 0x7cc, 0x7ef, 0x7d9, 0x7d3, 0x7d9, 0x7d6,
0x7ed, 0x7ca, 0x7dd, 0x7ed, 0x7cc, 0x7df, 0x7d3, 0x7ce, 0x7d6, 0x7ed, 0x7ce, 0x7f2,
0x7d3, 0x7ce, 0x7d9, 0x7ca, 0x7eb, 0x7de, 0x7cd, 0x7ef, 0x7d9, 0x7de, 0x7db, 0x7ca,
0x7d3, 0x7ce, 0x7df, 0x7ca, 0x7f2, 0x7de, 0x7ca, 0x7eb, 0x7db, 0x7cb, 0x7e5, 0x7ce,
0x7d5, 0x7ca, 0x7eb, 0x7de, 0x7d0, 0x7e3, 0x7ca, 0x7de, 0x7d9, 0x7cc, 0x7eb, 0x7d8,
0x7ca, 0x7ec, 0x7de, 0x7ca, 0x7ef, 0x7d9, 0x7ce, 0x7eb, 0x7d8, 0x7ca, 0x7ef, 0x7d9,
0x2d, 0x7db, 0x7ca, 0x7ec, 0x7df, 0x7ca, 0x7ef, 0x7e1, 0x7d6, 0x7cc, 0x7d3, 0x7ce,
0x7d5, 0x7cc, 0x7eb, 0x7d8, 0x7ce, 0x7e5, 0x7df, 0x7ca, 0x7eb, 0x7df, 0x7ca, 0x7ef,
0x7e6, 0x7ce, 0x7e3, 0x7dd, 0x7d9, 0x7cc, 0x7d5, 0x7d0, 0x7f2, 0x7ec, 0x7dc, 0x7ed,
0x7ca, 0x7d3, 0x7cf, 0x7d9, 0x7d0, 0x7e3, 0x7e4, 0x7d9, 0x7ca, 0x7d9, 0x7cb, 0x7eb,
0x7d6, 0x7ed, 0x7ce, 0x7e5, 0x7ca, 0x7e3, 0x7cd, 0x7db, 0x7d1, 0x7d3, 0x7ce, 0x7ef,
0x7d9, 0x7d6, 0x7ce, 0x7d3, 0x7ca, 0x7eb, 0x7de, 0x7ca, 0x7f2, 0x7d4, 0x7df, 0x7ca,
0x7eb, 0x7de, 0x7ca, 0x7d9, 0x7d1, 0x7d5, 0x7ce, 0x7e1, 0x7de, 0x7cc, 0x7dc, 0x7ed,
0x7ca, 0x7df, 0x7cc, 0x7eb, 0x7de, 0x7cc, 0x7f2, 0x7db, 0x7ca, 0x7db, 0x7ca, 0x7eb,
0x7df, 0x7cb, 0x7dc, 0x7ed, 0x7d0, 0x7db, 0x7df, 0x7cc, 0x7d3, 0x7d9, 0x7cb, 0x7dd,
0x7ed, 0x7cc, 0x7df, 0x7df, 0x7cf, 0x7e1, 0x7cb, 0x7eb, 0x7df, 0x7ce, 0x7e5, 0x7ca,
0x7f2, 0x7d8, 0x7ca, 0x7eb, 0x7df, 0x7ce, 0x7d3, 0x7ce, 0x7f2, 0x7d3, 0x7ca, 0x7db,
0x7cc, 0x7eb, 0x7df, 0x7ce, 0x7db, 0x7de, 0x7ca, 0x7eb, 0x7e1, 0x7df, 0x7ca, 0x7d3,
0x7cf, 0x7eb, 0x7e1, 0x7ca, 0x7d4, 0x7ce, 0x7d5, 0x7cf, 0x7eb, 0x7e1, 0x7ca, 0x7db,
0x7cb, 0x7d9, 0x7ce, 0x7eb, 0x7d2, 0x7d3, 0x7ca, 0x7d3, 0x7ca, 0x7e3, 0x7e1, 0x7cf,
0x7dc, 0x7ed, 0x7ca, 0x7d8, 0x7cc, 0x7db, 0x7cf, 0x7eb, 0x7e1, 0x7cf, 0x7ec, 0x7d9,
0x7cf, 0x7dd, 0x7ed, 0x7cc, 0x7e6, 0x7ca, 0x7eb, 0x7e3, 0x7ca, 0x7e6, 0x7d9, 0x7cf,
0x7d3, 0x7cc, 0x7eb, 0x7d2, 0x7d6, 0x7ca, 0x7e1, 0x7cb, 0x7e3, 0x7ca, 0x7eb, 0x7e2,
0x7ca, 0x7e1, 0x7cb, 0x7eb, 0x7e3, 0x7ce, 0x7e5, 0x7ca, 0x7de, 0x7d1, 0x7db, 0x7d0,
0x7d5, 0x7e5, 0x7dc, 0x7ed, 0x7ca, 0x7ec, 0x7d8, 0x7dc, 0x7ed, 0x7ce, 0x7eb, 0x7d4,
0x7d0, 0x7d9, 0x7d1, 0x7d5, 0x7cf, 0x7eb, 0x2d, 0x7e3, 0x7dd, 0x7ed, 0x7cf, 0x7eb,
0x7db, 0x7ca, 0x7cf, 0x2d, 0x7d5, 0x7cf, 0x7e1, 0x7cb, 0x7eb, 0x7d5, 0x7d9, 0x7cc,
0x7d4, 0x7df, 0x7cc, 0x7eb, 0x20, 0x28, 0x20, 0x7df, 0x7d3, 0x7cc, 0x7eb, 0x20,
0x29, 0x7d5, 0x7ce, 0x7e3, 0x7cc, 0x7db, 0x7e5, 0x7cc, 0x7f2, 0x7d8, 0x7d0, 0x7de,
0x7ca, 0x7d8, 0x7ca, 0x7de, 0x7ca, 0x7f2, 0x7de, 0x7cf, 0x7d9, 0x7ca, 0x7d6, 0x7ed,
0x7ca, 0x7f2, 0x7dc, 0x7ed, 0x7cc, 0x7df, 0x7ca, 0x7eb, 0x7ca, 0x7f2, 0x7d5, 0x7cc,
0x7dc, 0x7ed, 0x7ca, 0x7eb, 0x7ca, 0x7d9, 0x7ca, 0x7dc, 0x7ed, 0x7ce, 0x7e5, 0x7ca,
0x7e6, 0x7cc, 0x7e3, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x7d8, 0x7ca, 0x7d3, 0x7cc, 0x7e6, 0x7e3, 0x7cf, 0x7db, 0x7cd, 0x7d9, 0x7de, 0x7d5,
0x7ca, 0x7e1, 0x7ca, 0x7d9, 0x7d1, 0x7de, 0x7ca, 0x7eb, 0x7de, 0x7d0, 0x7d9, 0x7d1,
0x7d8, 0x7cf, 0x7d3, 0x7ca, 0x7eb, 0x7d6, 0x7ce, 0x7d6, 0x7ce, 0x7e6, 0x7cc, 0x7eb,
0x7df, 0x7ca, 0x20, 0x7d9, 0x7cc, 0x7e6, 0x7cf, 0x7d6, 0x7ca, 0x7eb, 0x7e1, 0x7ca,
0x7f2, 0x7d8, 0x7cf, 0x7d6, 0x7ed, 0x7ca, 0x7d9, 0x7cc, 0x7e6, 0x7cf, 0x7eb, 0x20,
0x7dc, 0x7ed, 0x7ca, 0x7df, 0x7df, 0x7cb, 0x7dc, 0x7ed, 0x7d0, 0x7db, 0x7db, 0x7ca,
0x7df, 0x7d1, 0x7d5, 0x7ca, 0x7eb, 0x7db, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d6, 0x7ce,
0x7e5, 0x7ca, 0x7f2, 0x7eb, 0x7db, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7df, 0x7ce, 0x7e5,
0x7cc, 0x7db, 0x7d5, 0x7de, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7eb, 0x7ce, 0x7db, 0x7d1,
0x7e5, 0x7e6, 0x7ca, 0x7eb, 0x7ca, 0x7d9, 0x7ce, 0x7d3, 0x7ca, 0x7eb, 0x7ca, 0x7db,
0x7ce, 0x7f2, 0x7db, 0x7cc, 0x7e6, 0x7d0, 0x7f2, 0x7eb, 0x7ca, 0x7d5, 0x7cc, 0x7de,
0x7cf, 0x7de, 0x7ca, 0x7f2, 0x7eb, 0x7d3, 0x7ca, 0x7e4, 0x7cc, 0x7e6, 0x7ca, 0x7eb,
0x7d3, 0x7ca, 0x7e4, 0x7cc, 0x7e6, 0x7ca, 0x20, 0x7d8, 0x7cb, 0x7ec, 0x20, 0x7d3,
0x7ca, 0x7f2, 0x7d8, 0x7cb, 0x7d9, 0x7ca, 0x7eb, 0x7d3, 0x7ca, 0x7d9, 0x7d1, 0x7d3,
0x7d8, 0x7d3, 0x7df, 0x7cb, 0x7e1, 0x7d3, 0x7cb, 0x7df, 0x7cb, 0x7cc, 0x7d6, 0x7ed,
0x7ca, 0x7eb, 0x7d3, 0x7d1, 0x7df, 0x7ca, 0x7f2, 0x7db, 0x2d, 0x7db, 0x7ca, 0x7d3,
0x7d1, 0x7df, 0x7d0, 0x7f2, 0x7eb, 0x7d3, 0x7cf, 0x7e5, 0x7ca, 0x7eb, 0x20, 0x7dd,
0x7ed, 0x7cc, 0x7db, 0x7d1, 0x7d5, 0x7ca, 0x7eb, 0x7d3, 0x7dc, 0x7ed, 0x7cf, 0x7d5,
0x7ca, 0x7eb, 0x7d3, 0x7ce, 0x7e5, 0x7ca, 0x7db, 0x7de, 0x7ca, 0x7f2, 0x7d3, 0x7d9,
0x7cc, 0x7d8, 0x7d6, 0x20, 0x7d3, 0x7ca, 0x7e6, 0x7de, 0x7ca, 0x7f2, 0x7d4, 0x7cf,
0x7eb, 0x20, 0x7dc, 0x7ed, 0x7d1, 0x7d9, 0x7ca, 0x7f2, 0x7d8, 0x7de, 0x7ca, 0x7f2,
0x7de, 0x7ce, 0x7f2, 0x7eb, 0x7de, 0x7d9, 0x7ca, 0x7de, 0x7ca, 0x7db, 0x7de, 0x7ca,
0x7e6, 0x7d1, 0x7e3, 0x7cb, 0x7eb, 0x7de, 0x7ca, 0x7e6, 0x7cc, 0x7e1, 0x7ca, 0x7f2,
0x7eb, 0x7db, 0x7cc, 0x7de, 0x7ca, 0x7dc, 0x7ed, 0x7cf, 0x7eb, 0x7db, 0x7cc, 0x7e4,
0x7ce, 0x7e5, 0x7ca, 0x7e4, 0x7ce, 0x7eb, 0x7db, 0x7cc, 0x7e6, 0x7ce, 0x7d8, 0x7ca,
0x7d8, 0x20, 0x7d6, 0x7ce, 0x7e5, 0x7ca, 0x7d9, 0x7cb, 0x7eb, 0x7de, 0x7cf, 0x7db,
0x7d1, 0x7d5, 0x7ca, 0x7d9, 0x7cc, 0x7de, 0x7ca, 0x7eb, 0x7de, 0x7d9, 0x7cc, 0x7db,
0x7d1, 0x7d5, 0x7d0, 0x7f2, 0x7eb, 0x7de, 0x7ce, 0x7e6, 0x7cc, 0x7e6, 0x7d3, 0x7ca,
0x7eb, 0x7de, 0x7ce, 0x7d9, 0x7db, 0x7ca, 0x7e5, 0x7cf, 0x7eb, 0x7d8, 0x7ca, 0x7f2,
0x7e1, 0x7ca, 0x7d9, 0x7de, 0x7db, 0x7d1, 0x7e4, 0x7ca, 0x7dd, 0x7ed, 0x7e3, 0x7d8,
0x7ca, 0x7e5, 0x7ce, 0x7db, 0x7d0, 0x7f2, 0x7eb, 0x7d8, 0x7ca, 0x7e5, 0x7ce, 0x7db,
0x7d0, 0x7f2, 0x7eb, 0x20, 0x7de, 0x7d9, 0x7cc, 0x7de, 0x7d8, 0x7ca, 0x7f2, 0x7dd,
0x7ed, 0x7cd, 0x7d9, 0x7d8, 0x7cb, 0x7d5, 0x7d9, 0x7cf, 0x7e5, 0x7ca, 0x7d8, 0x7cf,
0x7e1, 0x7cb, 0x7e3, 0x7cc, 0x7de, 0x7cb, 0x7d8, 0x7d1, 0x7e1, 0x7d0, 0x7f2, 0x7d5,
0x7d0, 0x7f2, 0x7eb, 0x7cb, 0x7d9, 0x7ce, 0x7e3, 0x7d4, 0x7cb, 0x7db, 0x7ca, 0x7df,
0x7d1, 0x7dd, 0x7ed, 0x7ca, 0x7d8, 0x7d0, 0x7d9, 0x7dd, 0x7d0, 0x7d9, 0x7d5, 0x20,
0x7e3, 0x7cb, 0x7df, 0x7d1, 0x7db, 0x7d0, 0x7f2, 0x7eb, 0x7dd, 0x7d0, 0x7d9, 0x7d1,
0x7d5, 0x7ca, 0x7df, 0x7cb, 0x7d6, 0x7ed, 0x7ca, 0x7eb, 0x7dc, 0x7ed, 0x7d1, 0x7df,
0x7ca, 0x7db, 0x20, 0x7d5, 0x7cc, 0x7f2, 0x7dc, 0x7ed, 0x7cf, 0x7db, 0x7cb, 0x7eb,
0x20, 0x7d5, 0x7cc, 0x7f2, 0x7dc, 0x7ed, 0x7d1, 0x7d9, 0x7ca, 0x7f2, 0x7d8, 0x20,
0x7d5, 0x7ce, 0x7d9, 0x7de, 0x7dc, 0x7ed, 0x7d9, 0x7cb, 0x7e3, 0x7d8, 0x7ca, 0x7eb,
0x7dc, 0x7ed, 0x7d1, 0x7e5, 0x7ca, 0x7d8, 0x7cc, 0x7df, 0x7ce, 0x7d4, 0x7dc, 0x7ed,
0x7ca, 0x7ec, 0x7d5, 0x7cc, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7df, 0x7ca, 0x7eb, 0x7dc,
0x7ed, 0x7ce, 0x7e5, 0x7e6, 0x7ca, 0x7de, 0x7cc, 0x7df, 0x7dc, 0x7ed, 0x7ce, 0x7e6,
0x7e3, 0x7ca, 0x7eb, 0x7e4, 0x7ca, 0x7df, 0x7cc, 0x7dd, 0x7ca, 0x7de, 0x7db, 0x7e4,
0x7dd, 0x7ed, 0x7ca, 0x7e3, 0x7ca, 0x7eb, 0x7e4, 0x7cb, 0x7d9, 0x7d1, 0x7e1, 0x7cf,
0x7db, 0x7cc, 0x7df, 0x7cf, 0x7eb, 0x7cc, 0x7f2, 0x7d8, 0x7cc, 0x7e6, 0x7ca, 0x7e3,
0x7ca, 0x7d4, 0x7cf, 0x7df, 0x7cc, 0x7db, 0x7de, 0x7d1, 0x7e3, 0x7cf, 0x7de, 0x7db,
0x20, 0x5b, 0x7cc, 0x7f2, 0x7d8, 0x7cc, 0x7e6, 0x7e3, 0x7ca, 0x7eb, 0x5d, 0x7e1,
0x7ca, 0x7d9, 0x7cc, 0x7f2, 0x7dc, 0x7ed, 0x7cf, 0x7eb, 0x20, 0x5b, 0x7cc, 0x7f2,
0x7d8, 0x7cc, 0x7e6, 0x7e3, 0x7ca, 0x7eb, 0x5d, 0x7d4, 0x7cb, 0x7d5, 0x7cd, 0x7d9,
0x7db, 0x7d1, 0x7d3, 0x7ce, 0x7d9, 0x7dc, 0x7ed, 0x20, 0x5b, 0x7cc, 0x7f2, 0x7d8,
0x7cc, 0x7e6, 0x7e3, 0x7ca, 0x7eb, 0x5d, 0x7d5, 0x7cd, 0x7df, 0x20, 0x7db, 0x7d5,
0x7cc, 0x7eb, 0x20, 0x5b, 0x7cc, 0x7f2, 0x7d8, 0x7cc, 0x7e6, 0x7e3, 0x7ca, 0x7eb,
0x5d, 0x7dd, 0x7ed, 0x7cb, 0x7dd, 0x7ed, 0x7ca, 0x7e6, 0x20, 0x5b, 0x7cc, 0x7f2,
0x7d8, 0x7cc, 0x7e6, 0x7e3, 0x7ca, 0x7eb, 0x5d, 0x7dd, 0x7ed, 0x7cd, 0x7f2, 0x7db,
0x7ca, 0x7f2, 0x7eb, 0x20, 0x5b, 0x7cc, 0x7f2, 0x7d8, 0x7cc, 0x7e6, 0x7e3, 0x7ca,
0x7eb, 0x5d, 0x7e5, 0x7cc, 0x7e3, 0x7e1, 0x7ca, 0x7de, 0x20, 0x5b, 0x7cc, 0x7f2,
0x7d8, 0x7cc, 0x7e6, 0x7e3, 0x7ca, 0x7eb, 0x5d, 0x7cc, 0x7e3, 0x7ce, 0x7dd, 0x7ed,
0x7cc, 0x7de, 0x7cc, 0x7de, 0x7ca, 0x7df, 0x7ce, 0x7e6, 0x7cc, 0x7d5, 0x7d6, 0x7e1,
0x7ca, 0x7e6, 0x7cc, 0x7de, 0x7ca, 0x7eb, 0x7d6, 0x7ce, 0x7e3, 0x7cb, 0x7e5, 0x7cf,
0x7eb, 0x7df, 0x7ce, 0x7e6, 0x7cc, 0x7db, 0x7dd, 0x7ed, 0x7cc, 0x7df, 0x7e1, 0x7d0,
0x7f2, 0x7d5, 0x7cc, 0x7db, 0x7cb, 0x7df, 0x7cf, 0x7ec, 0x20, 0x5b, 0x7de, 0x7ca,
0x7f2, 0x7d5, 0x7ce, 0x7de, 0x7cc, 0x7eb, 0x5d, 0x7de, 0x7d1, 0x7d9, 0x7ca, 0x7df,
0x7ca, 0x7f2, 0x7d8, 0x7cc, 0x7d6, 0x7de, 0x7df, 0x7ca, 0x20, 0x7d4, 0x7ca, 0x7d6,
0x7ed, 0x7df, 0x7cc, 0x7e1, 0x7ca, 0x7eb, 0x7df, 0x7d0, 0x7db, 0x20, 0x7ca, 0x7f2,
0x7d6, 0x7ed, 0x7cb, 0x7df, 0x7cd, 0x7db, 0x7df, 0x7cf, 0x7e5, 0x7cd, 0x20, 0x7d4,
0x7d1, 0x7d9, 0x7cd, 0x7f2, 0x7db, 0x20, 0x7de, 0x7ca, 0x7d9, 0x7d1, 0x7d5, 0x7cb,
0x7eb, 0x7e1, 0x7ca, 0x7db, 0x7cb, 0x7e6, 0x7cf, 0x7eb, 0x7e1, 0x7e3, 0x7ca, 0x7dc,
0x7ed, 0x7ce, 0x7e5, 0x7ca, 0x7eb, 0x7e1, 0x7e3, 0x7ca, 0x7e5, 0x7d0, 0x7db, 0x7e1,
0x7ca, 0x7d9, 0x7cc, 0x7dc, 0x7ed, 0x7d0, 0x7d5, 0x7e1, 0x7ca, 0x7d9, 0x7d1, 0x7d5,
0x7e3, 0x7cc, 0x7de, 0x7e1, 0x7d5, 0x7ca, 0x7e1, 0x7d9, 0x7cf, 0x7db, 0x7e1, 0x7d6,
0x7ed, 0x7ca, 0x7d5, 0x7d1, 0x7df, 0x7ca, 0x7f2, 0x7eb, 0x7e1, 0x7cb, 0x7e3, 0x7cf,
0x7e1, 0x7cc, 0x7f2, 0x7eb, 0x7e1, 0x7cb, 0x7d9, 0x7cc, 0x7d8, 0x7ca, 0x7eb, 0x7e1,
0x7cb, 0x7d5, 0x7d1, 0x7df, 0x7de, 0x7ca, 0x7e6, 0x7d1, 0x7df, 0x7ca, 0x7eb, 0x7e1,
0x7cd, 0x7de, 0x7db, 0x7cc, 0x7de, 0x7cf, 0x7eb, 0x7e1, 0x7cc, 0x7de, 0x7cb, 0x7df,
0x7d0, 0x7f2, 0x7eb, 0x7e1, 0x7d0, 0x7f2, 0x7de, 0x7d1, 0x7d5, 0x7d0, 0x7f2, 0x7eb,
0x7e1, 0x7d0, 0x7f2, 0x7d5, 0x7cb, 0x7da, 0x7cb, 0x7e6, 0x7e1, 0x7d0, 0x7f2, 0x7d5,
0x7cb, 0x7dd, 0x7ed, 0x7cc, 0x7d8, 0x7cb, 0x7e5, 0x7cf, 0x7eb, 0x7e1, 0x7d0, 0x7f2,
0x7d5, 0x7db, 0x7d1, 0x7da, 0x7ca, 0x7d5, 0x7e3, 0x7ca, 0x7db, 0x7ca, 0x7e5, 0x7cf,
0x7eb, 0x7e3, 0x7cb, 0x7e5, 0x20, 0x7e6, 0x7d0, 0x7d9, 0x7de, 0x7e3, 0x7cf, 0x7e1,
0x7cb, 0x7eb, 0x7e3, 0x7d0, 0x7d9, 0x7d0, 0x7f2, 0x7e4, 0x7ca, 0x7f2, 0x7eb, 0x7d3,
0x7cb, 0x7e6, 0x7ce, 0x7df, 0x7ca, 0x7eb, 0x20, 0x28, 0x7de, 0x7d0, 0x7f0, 0x7d8,
0x7ce, 0x7ee, 0x20, 0x7d8, 0x7ca, 0x7de, 0x7cf, 0x7d5, 0x7ca, 0x7eb, 0x29, 0x7db,
0x7ca, 0x7f2, 0x7d5, 0x7d9, 0x7cc, 0x7eb, 0x20, 0x28, 0x7de, 0x7d0, 0x7f0, 0x7d8,
0x7ce, 0x7ee, 0x20, 0x7d8, 0x7ca, 0x7de, 0x7cf, 0x7d5, 0x7ca, 0x7eb, 0x29, 0x7e3,
0x7cb, 0x7e5, 0x20, 0x7db, 0x7ca, 0x7df, 0x7cb, 0x7e1, 0x20, 0x28, 0x7de, 0x7d0,
0x7f0, 0x7d8, 0x7ce, 0x7ee, 0x20, 0x7d8, 0x7ca, 0x7de, 0x7cf, 0x7d5, 0x7ca, 0x7eb,
0x29, 0x7e3, 0x7ce, 0x7ef, 0x7de, 0x7cf, 0x7d6, 0x7cc, 0x7e3, 0x7dc, 0x7ed, 0x7ca,
0x7eb, 0x7d4, 0x7e3, 0x7ca, 0x7e1, 0x7ca, 0x7ec, 0x7d4, 0x7d9, 0x7ca, 0x7e1, 0x7ca,
0x7d9, 0x7cc, 0x7d3, 0x7cf, 0x7eb, 0x7dd, 0x7cf, 0x7cb, 0x7e3, 0x7cc, 0x7de, 0x7db,
0x7d4, 0x7d0, 0x7d9, 0x7d5, 0x2d, 0x7cf, 0x2d, 0x7d4, 0x7d1, 0x7d9, 0x7cd, 0x7f2,
0x7db, 0x7d4, 0x7d0, 0x7d9, 0x7d5, 0x20, 0x7ca, 0x7db, 0x7d1, 0x7d4, 0x7ca, 0x7f2,
0x7eb, 0x7d4, 0x7d0, 0x7d9, 0x7d1, 0x7d5, 0x7cf, 0x7eb, 0x20, 0x7dd, 0x7ed, 0x7cb,
0x7df, 0x7cf, 0x7eb, 0x7d4, 0x7d0, 0x7d9, 0x7d5, 0x2d, 0x7cf, 0x2d, 0x7d9, 0x7cc,
0x7de, 0x7cf, 0x7eb, 0x7d4, 0x7ce, 0x7f2, 0x7d5, 0x7ca, 0x7eb, 0x20, 0x7ca, 0x7d9,
0x7cb, 0x7e3, 0x7ca, 0x7eb, 0x7d9, 0x7ca, 0x7f2, 0x7de, 0x7cc, 0x7f2, 0x7eb, 0x20,
0x7cc, 0x7f2, 0x7df, 0x7cd, 0x7d5, 0x7d9, 0x7cb, 0x7db, 0x7cc, 0x7dd, 0x7d9, 0x7cb,
0x7d6, 0x7ed, 0x7cc, 0x7e3, 0x7ca, 0x7eb, 0x7d9, 0x7cb, 0x7d6, 0x7ed, 0x7cf, 0x7df,
0x7ce, 0x7d5, 0x7d9, 0x7cc, 0x7e6, 0x7cf, 0x7eb, 0x20, 0x7d3, 0x7d1, 0x7d9, 0x7ca,
0x7f2, 0x7de, 0x7cf, 0x7eb, 0x7db, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d9, 0x7cd, 0x7e1,
0x7db, 0x7ca, 0x7f2, 0x7d5, 0x7cc, 0x7e6, 0x7ca, 0x7dc, 0x7ed, 0x7cf, 0x7eb, 0x7db,
0x7ca, 0x7f2, 0x7d5, 0x7cf, 0x2d, 0x7d8, 0x7cf, 0x7e1, 0x7cc, 0x7f2, 0x7dc, 0x7ed,
0x7db, 0x7ca, 0x7f2, 0x7d4, 0x7ca, 0x7e5, 0x7df, 0x7cf, 0x7eb, 0x7cc, 0x7d5, 0x7d5,
0x7cf, 0x7de, 0x7de, 0x7cf, 0x7d9, 0x7d1, 0x7d5, 0x7cf, 0x7d9, 0x7d1, 0x7df, 0x7dc,
0x7d5, 0x7db, 0x7cc, 0x7d5, 0x7d1, 0x7de, 0x7ca, 0x7eb, 0x7db, 0x7cd, 0x7f2, 0x2d,
0x7d3, 0x7ca, 0x7d9, 0x7d5, 0x7d1, 0x7e4, 0x7df, 0x7cb, 0x7e1, 0x7cc, 0x7eb, 0x7db,
0x7cd, 0x7f2, 0x2d, 0x7d6, 0x7ed, 0x7ca, 0x7f2, 0x20, 0x7d5, 0x7cb, 0x20, 0x7d5,
0x7cd, 0x7da, 0x2d, 0x7e3, 0x7cb, 0x7dd, 0x7ed, 0x7db, 0x7cd, 0x7f2, 0x2d, 0x7de,
0x7d9, 0x7cc, 0x7db, 0x7d1, 0x7d5, 0x7d0, 0x7dd, 0x7db, 0x7cd, 0x2d, 0x7df, 0x7ce,
0x7e5, 0x7cc, 0x7db, 0x7db, 0x7cd, 0x7f2, 0x2d, 0x7d5, 0x7cf, 0x7e1, 0x7ca, 0x7db,
0x7db, 0x7cd, 0x7f2, 0x2d, 0x7dd, 0x7ed, 0x7cd, 0x7f2, 0x7db, 0x7ca, 0x7f2, 0x7eb,
0x7db, 0x7e5, 0x7cc, 0x7dd, 0x20, 0x7de, 0x7ce, 0x7d9, 0x7ca, 0x7f2, 0x7eb, 0x7d5,
0x7cb, 0x7dc, 0x7ed, 0x7ce, 0x7db, 0x7cc, 0x7dc, 0x7ed, 0x7ca, 0x7df, 0x7d1, 0x7d4,
0x7ca, 0x7eb, 0x7d5, 0x7ce, 0x7df, 0x7cb, 0x7eb, 0x7d5, 0x7cc, 0x7d6, 0x7ce, 0x7e5,
0x7e3, 0x7ca, 0x7eb, 0x7d5, 0x7cf, 0x7d9, 0x7cf, 0x7f2, 0x7d5, 0x7cf, 0x7eb, 0x7d5,
0x7cf, 0x7d9, 0x7d1, 0x7d5, 0x7cf, 0x7df, 0x7ca, 0x7eb, 0x7dd, 0x7ed, 0x7ca, 0x7f2,
0x7de, 0x7ce, 0x7dd, 0x7ed, 0x7cd, 0x7d9, 0x7e5, 0x7cc, 0x7d5, 0x7cb, 0x7e4, 0x7d0,
0x7d9, 0x7db, 0x7e5, 0x7cc, 0x7f2, 0x7e3, 0x7cc, 0x7d4, 0x7cb, 0x7dc, 0x7ed, 0x7e6,
0x7ca, 0x7de, 0x7ce, 0x7d5, 0x7ca, 0x7eb, 0x7de, 0x7ca, 0x7db, 0x7cb, 0x7e6, 0x7cc,
0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d8, 0x7ca,
0x7dd, 0x7ed, 0x7cc, 0x7db, 0x7d8, 0x7ce, 0x7e1, 0x7d0, 0x7f2, 0x2d, 0x7ce, 0x7d9,
0x7d1, 0x7dd, 0x7ed, 0x7cc, 0x7df, 0x7e1, 0x7de, 0x7ca, 0x7d9, 0x7cc, 0x7eb, 0x7e1,
0x7ca, 0x7e5, 0x7d1, 0x7db, 0x7d0, 0x7f2, 0x7eb, 0x7e1, 0x7de, 0x7d1, 0x7e1, 0x7ce,
0x7d9, 0x7d1, 0x7d8, 0x7cf, 0x7eb, 0x7d4, 0x7ca, 0x7df, 0x7d1, 0x7e1, 0x7cd, 0x7d9,
0x7d9, 0x7cf, 0x7d5, 0x7cb, 0x7d9, 0x7ca, 0x7eb, 0x7db, 0x7cf, 0x7e5, 0x7ca, 0x7eb,
0x7d5, 0x7d9, 0x7cf, 0x7df, 0x7dd, 0x7ed, 0x7cf, 0x7db, 0x7d1, 0x7d5, 0x7cf, 0x7de,
0x7df, 0x7d0, 0x7f2, 0x7dc, 0x7ed, 0x7cc, 0x7cb, 0x7e6, 0x7ca, 0x7d9, 0x7d1, 0x7d3,
0x7cc, 0x7e6, 0x7cd, 0x7f2, 0x7eb, 0x7ca, 0x7d8, 0x7cd, 0x7f2, 0x7eb, 0x7ca, 0x7df,
0x7d1, 0x7e1, 0x7ca, 0x20, 0x7ca, 0x7d5, 0x7ca, 0x7eb, 0x7ca, 0x7e1, 0x7e1, 0x7ca,
0x7f2, 0x7eb, 0x7ca, 0x7e3, 0x7ca, 0x7d8, 0x7cc, 0x7d9, 0x7ca, 0x7de, 0x7d1, 0x7d5,
0x7ca, 0x7e5, 0x7ce, 0x7eb, 0x7ca, 0x7de, 0x7d1, 0x7d5, 0x7cf, 0x7d3, 0x7cc, 0x7f2,
0x7db, 0x7de, 0x7ca, 0x7db, 0x7d1, 0x7dc, 0x7ed, 0x7ca, 0x7d3, 0x7ca, 0x7d5, 0x7ca,
0x7d5, 0x7cc, 0x7d9, 0x7ca, 0x7ce, 0x7eb, 0x7d3, 0x7ca, 0x7dc, 0x7ed, 0x7d1, 0x7d8,
0x7ca, 0x7d8, 0x7d3, 0x7e4, 0x7ca, 0x7d9, 0x7cd, 0x7f2, 0x7eb, 0x7d3, 0x7ca, 0x7de,
0x7ce, 0x7eb, 0x7d3, 0x7ca, 0x7f2, 0x7de, 0x7d0, 0x7de, 0x7d3, 0x7ca, 0x7d9, 0x7d1,
0x7e3, 0x7d0, 0x7df, 0x7d3, 0x7cb, 0x7e6, 0x7cc, 0x7d9, 0x7ce, 0x7d5, 0x7d3, 0x7cc,
0x7d7, 0x7cd, 0x7de, 0x7d3, 0x7d9, 0x7ce, 0x7e3, 0x7cb, 0x7e6, 0x7cc, 0x7eb, 0x7d7,
0x7cc, 0x7d5, 0x7ca, 0x7eb, 0x7de, 0x7cf, 0x7df, 0x7cf, 0x7f2, 0x7d3, 0x7cf, 0x7eb,
0x7d8, 0x7e1, 0x7ca, 0x7db, 0x7d8, 0x7de, 0x7ca, 0x7eb, 0x7d8, 0x7df, 0x7cc, 0x7eb,
0x7d8, 0x7ce, 0x7ec, 0x7d3, 0x7ca, 0x7e6, 0x7cc, 0x7ec, 0x7d8, 0x7ce, 0x7db, 0x7ca,
0x7f2, 0x7d3, 0x7cb, 0x7eb, 0x7dd, 0x7e1, 0x7ca, 0x7dc, 0x7ed, 0x7ce, 0x7db, 0x7d1,
0x7d5, 0x7ca, 0x7eb, 0x7dc, 0x7ed, 0x7ca, 0x7d6, 0x7ed, 0x7ca, 0x7eb, 0x7e4, 0x7cb,
0x7d3, 0x7d1, 0x7d9, 0x7d0, 0x7f2, 0x7eb, 0x7e4, 0x7d0, 0x7ef, 0x2d, 0x7d7, 0x7cc,
0x2d, 0x7e1, 0x7cc, 0x7f2, 0x2d, 0x7d8, 0x7ce, 0x7f1, 0x7e4, 0x7d0, 0x7f2, 0x7dc,
0x7ed, 0x20, 0x7de, 0x7d0, 0x7f2, 0x7dc, 0x7ed, 0x7e4, 0x7cf, 0x7dd, 0x7ed, 0x7d8,
0x7cc, 0x7d9, 0x7d1, 0x7de, 0x7ce, 0x7d5, 0x7d1, 0x7d8, 0x7de, 0x7cc, 0x7eb, 0x7d6,
0x7de, 0x7ca, 0x7d9, 0x7d1, 0x7d5, 0x7ca, 0x7eb, 0x7d6, 0x7e6, 0x7ca, 0x7d4, 0x7ce,
0x7d9, 0x7ca, 0x7eb, 0x7d6, 0x7ed, 0x7cb, 0x7d9, 0x7ce, 0x7d6, 0x7ed, 0x7ca, 0x7df,
0x7cd, 0x7e1, 0x7de, 0x7ca, 0x7d3, 0x7ce, 0x7df, 0x7de, 0x7ca, 0x7f2, 0x7e6, 0x7d1,
0x7d7, 0x7ca, 0x7d5, 0x7d1, 0x7de, 0x7cc, 0x7eb, 0x7de, 0x7d9, 0x7ca, 0x7db, 0x7cc,
0x7eb, 0x7de, 0x7ca, 0x7d5, 0x7d1, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ce, 0x7eb, 0x7de,
0x7ed, 0x7ca, 0x7f2, 0x7d8, 0x7cc, 0x7dc, 0x7ed, 0x7ca, 0x7eb, 0x7de, 0x7ca, 0x7df,
0x7d1, 0x7de, 0x7ce, 0x7d5, 0x7ca, 0x7eb, 0x7de, 0x7d9, 0x7ca, 0x7db, 0x7d1, 0x7df,
0x7cf, 0x7e6, 0x7d1, 0x7e6, 0x7ca, 0x7d9, 0x7d1, 0x7db, 0x7de, 0x7cc, 0x7eb, 0x7de,
0x7ce, 0x7e5, 0x7df, 0x7ca, 0x20, 0x7df, 0x7ce, 0x7f2, 0x7d4, 0x7ce, 0x7d9, 0x7de,
0x7ce, 0x7d7, 0x7cc, 0x7f2, 0x7dc, 0x7ed, 0x7de, 0x7ce, 0x7e5, 0x7cd, 0x7e6, 0x7cc,
0x7d5, 0x7e1, 0x7de, 0x7ca, 0x7e5, 0x7cf, 0x7eb, 0x7e1, 0x7dc, 0x7ed, 0x7ca, 0x7d8,
0x7ca, 0x7f2, 0x7eb, 0x7e1, 0x7de, 0x7ca, 0x7db, 0x7ca, 0x7d9, 0x7e1, 0x7ca, 0x7e3,
0x7cc, 0x7df, 0x7e1, 0x7ca, 0x7db, 0x7d1, 0x7de, 0x7ca, 0x7d5, 0x7e3, 0x7cc, 0x7de,
0x7cf, 0x7db, 0x7cc, 0x7e6, 0x7ca, 0x7eb, 0x7e3, 0x7dd, 0x7ed, 0x7cf, 0x7de, 0x7ce,
0x7d6, 0x7ed, 0x7e3, 0x7cd, 0x7d5, 0x7d1, 0x7db, 0x7de, 0x7cc, 0x7eb, 0x7e3, 0x7dd,
0x7ed, 0x7cf, 0x7de, 0x7db, 0x7d3, 0x7cc, 0x7d9, 0x7d1, 0x7db, 0x7de, 0x7cc, 0x7eb,
0x7cf, 0x7e1, 0x7d1, 0x7db, 0x7de, 0x7cf, 0x7d9, 0x7ca, 0x7df, 0x7d4, 0x7d1, 0x7e3,
0x7d0, 0x7f2, 0x7eb, 0x20, 0x7d4, 0x7cd, 0x7f2, 0x7eb, 0x7d4, 0x7d0, 0x7f2, 0x7d5,
0x7cc, 0x7e6, 0x7ca, 0x7e3, 0x7ca, 0x7de, 0x7d4, 0x7cc, 0x7e6, 0x7d0, 0x7f2, 0x7dc,
0x7ed, 0x7e6, 0x7ca, 0x7f2, 0x7dc, 0x7ed, 0x7de, 0x7d5, 0x7ca, 0x7d9, 0x7de, 0x7cf,
0x7db, 0x7d1, 0x7d5, 0x7e3, 0x7ca, 0x7e6, 0x7de, 0x7d6, 0x7ed, 0x7cc, 0x7df, 0x7cf,
0x7d9, 0x7d1, 0x7d8, 0x7ca, 0x7eb, 0x7d9, 0x7cc, 0x7e6, 0x7ca, 0x7d8, 0x7db, 0x7de,
0x7ca, 0x7df, 0x7cc, 0x7e3, 0x7db, 0x7e1, 0x7ca, 0x7d9, 0x7d1, 0x7de, 0x7ca, 0x7f2,
0x7d8, 0x7db, 0x7cb, 0x7ce, 0x7df, 0x7db, 0x7ca, 0x7f2, 0x7dc, 0x7ed, 0x7ca, 0x7e6,
0x7cc, 0x7eb, 0x7db, 0x7cd, 0x7f2, 0x7dc, 0x7ed, 0x7ca, 0x7d4, 0x7ce, 0x7d9, 0x7db,
0x7d9, 0x7cb, 0x7d8, 0x7d1, 0x7e3, 0x7cf, 0x7de, 0x7d0, 0x7df, 0x7cc, 0x7e1, 0x7d1,
0x7db, 0x7de, 0x7cc, 0x7eb, 0x7d5, 0x7ca, 0x7e6, 0x7cc, 0x7d4, 0x7cb, 0x7eb, 0x7d5,
0x7ca, 0x7db, 0x7d7, 0x7d1, 0x7de, 0x7cd, 0x7f2, 0x7d5, 0x7d5, 0x7d3, 0x7cc, 0x7df,
0x7db, 0x7cc, 0x7eb, 0x7d5, 0x7e4, 0x7cb, 0x7d9, 0x7ca, 0x7f2, 0x7eb, 0x7d5, 0x7cc,
0x7f2, 0x7d4, 0x7ce, 0x7eb, 0x7d5, 0x7cf, 0x7de, 0x7cc, 0x7e6, 0x7cf, 0x7eb, 0x7d5,
0x7cf, 0x7e1, 0x7d1, 0x7db, 0x7de, 0x7ce, 0x7df, 0x7ca, 0x7f2, 0x7d3, 0x7ca, 0x7d5,
0x7d0, 0x7d9, 0x7ce, 0x7d9, 0x7ce, 0x7f2, 0x7de, 0x7cc, 0x7eb, 0x7ce, 0x7db, 0x7d5,
0x2d, 0x7e3, 0x7cb, 0x7d9, 0x7ca, 0x7eb, 0x7dd, 0x7ed, 0x7cc, 0x7e6, 0x7cd, 0x7d5,
0x7cc, 0x7e3, 0x7ca, 0x7e1, 0x7dd, 0x7ed, 0x7d1, 0x7df, 0x7ca, 0x7d8, 0x7cc, 0x7dd,
0x7ed, 0x7cf, 0x7db, 0x7d1, 0x7d5, 0x7d0, 0x7de, 0x7e6, 0x7ca, 0x7de, 0x7ce, 0x7d5,
0x7d1, 0x7db, 0x7de, 0x7d9, 0x7ca, 0x7f2, 0x7dc, 0x7ed, 0x7ce, 0x7f2, 0x7eb, 0x7e6,
0x7cb, 0x7de, 0x7ca, 0x7d5, 0x7cb, 0x7d9, 0x7cc, 0x7f2, 0x7d3, 0x7ce, 0x7d9, 0x7dc,
0x7ed, 0x7e6, 0x7d9, 0x7cb, 0x7dd, 0x7ed, 0x7ca, 0x7f2, 0x7eb, 0x7ca, 0x7d7, 0x7ed,
0x7cf, 0x7d9, 0x7cd, 0x7db, 0x7d3, 0x7cd, 0x7d9, 0x7f8, 0x7e1, 0x7ce, 0x7d8, 0x7cd,
0x7db, 0x7de, 0x7e3, 0x7ca, 0x7d9, 0x7cc, 0x7eb, 0x7dc, 0x7e6, 0x7cb, 0x7de, 0x7ce,
0x7f2, 0x7eb, 0x2d, 0x7dd, 0x7d9, 0x7cc, 0x7db, 0x7cc, 0x7eb, 0x7dd, 0x7ca, 0x7d9,
0x7cf, 0x7cb, 0x7eb, 0x7e1, 0x7ca, 0x7d8, 0x7cd, 0x7d9, 0x7d9, 0x7cb, 0x7e6, 0x7d1,
0x7de, 0x7cc, 0x7e6, 0x7ca, 0x7dd, 0x7ed, 0x7cd, 0x7de, 0x7de, 0x7d0, 0x7f0, 0x7d8,
0x7ce, 0x7ee, 0x20, 0x7d6, 0x7ed, 0x7d0, 0x7d9, 0x7d1, 0x7d6, 0x7ed, 0x7cc, 0x7df,
0x7ca, 0x7eb, 0x7db, 0x7cd, 0x7f2, 0x7d5, 0x20, 0x7e4, 0x7cb, 0x7df, 0x7cd, 0x7e3,
0x7db, 0x7d1, 0x7d5, 0x7ca, 0x7f2, 0x7df, 0x7cb, 0x7e6, 0x7ca, 0x7d8, 0x7cb, 0x7df,
0x7ca, 0x7e6, 0x7d8, 0x7d3, 0x7d1, 0x7d9, 0x7cc, 0x7db, 0x7d1, 0x7d3, 0x7ca, 0x7f2,
0x7eb, 0x7d3, 0x7d1, 0x7d9, 0x7cf, 0x7de, 0x7cd, 0x7f2, 0x7eb, 0x20, 0x7e4, 0x7cc,
0x7df, 0x7d8, 0x7ca, 0x7d9, 0x7d1, 0x7e5, 0x7cc, 0x7f2, 0x7eb, 0x7cb, 0x7de, 0x7d1,
0x7df, 0x7ca, 0x7eb, 0x7e4, 0x7cf, 0x7d3, 0x7ca, 0x7d9, 0x7d5, 0x7df, 0x7cc, 0x7f2,
0x7d8, 0x7cb, 0x7e1, 0x7ca, 0x7f2, 0x7eb, 0x7df, 0x7d0, 0x7d9, 0x7d8, 0x20, 0x7e4,
0x7cf, 0x7e5, 0x7cb, 0x7eb, 0x7e1, 0x7cb, 0x7df, 0x7d1, 0x7d3, 0x7ce, 0x7d9, 0x7e3,
0x7d4, 0x7cd, 0x7d9, 0x7d5, 0x7db, 0x7cc, 0x7d8, 0x7d1, 0x7e3, 0x7cb, 0x7eb, 0x7ca,
0x7e1, 0x7db, 0x7d1, 0x7d5, 0x7cd, 0x7d9, 0x7d1, 0x7d8, 0x7ca, 0x7e1, 0x7ca, 0x7f2,
0x7d8, 0x7cf, 0x7d9, 0x7cb, 0x7eb, 0x7ca, 0x7db, 0x7d1, 0x7d5, 0x7d9, 0x7ca, 0x7de,
0x7d1, 0x7e4, 0x7ca, 0x7f2, 0x7eb, 0x7ca, 0x7d5, 0x7cd, 0x7e3, 0x7d3, 0x7cd, 0x7df,
0x7d1, 0x7dc, 0x7ed, 0x7d9, 0x7ca, 0x7d8, 0x7d3, 0x7cd, 0x7d9, 0x7d1, 0x7df, 0x7cd,
0x7f2, 0x7eb, 0x7d3, 0x7d1, 0x7d9, 0x7ca, 0x7d5, 0x7cc, 0x7db, 0x7d1, 0x7df, 0x7dd,
0x7ed, 0x7ca, 0x7eb, 0x7d3, 0x7d1, 0x7d9, 0x7ce, 0x7de, 0x7d1, 0x7db, 0x7cd, 0x7df,
0x7d3, 0x7ce, 0x7de, 0x7ca, 0x7d9, 0x7cd, 0x7db, 0x7d3, 0x7ce, 0x7d8, 0x7ca, 0x7d3,
0x7cd, 0x7db, 0x7d5, 0x7d3, 0x7ce, 0x7db, 0x7cd, 0x7f2, 0x7dc, 0x7ed, 0x7cd, 0x7f2,
0x7eb, 0x7d7, 0x7db, 0x7cc, 0x7e3, 0x7ca, 0x7e5, 0x7de, 0x7cf, 0x7d4, 0x7cb, 0x7e3,
0x7ca, 0x7dc, 0x7ed, 0x7d8, 0x7ce, 0x7d3, 0x7d1, 0x7df, 0x7cd, 0x7eb, 0x7d6, 0x7ed,
0x7cc, 0x7d3, 0x7d9, 0x7ca, 0x7df, 0x7d1, 0x7d5, 0x7ca, 0x7eb, 0x7dc, 0x7ed, 0x7cd,
0x7d9, 0x7d1, 0x7e3, 0x7db, 0x7cb, 0x7e6, 0x7e4, 0x7cd, 0x7df, 0x7d1, 0x7db, 0x7cc,
0x7f2, 0x7de, 0x7cc, 0x7eb, 0x7e1, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7cc, 0x7f2,
0x7cc, 0x7db, 0x7d1, 0x7d5, 0x7ca, 0x7f2, 0x7d3, 0x7ce, 0x7df, 0x7d6, 0x7cb, 0x7d9,
0x7d1, 0x7db, 0x7cc, 0x7eb, 0x7de, 0x7ca, 0x7df, 0x7cc, 0x7e3, 0x7cc, 0x7f2, 0x7dc,
0x7ed, 0x7d9, 0x7ca, 0x7d8, 0x7de, 0x7cc, 0x7d9, 0x7d0, 0x7dd, 0x7ed, 0x7de, 0x7cc,
0x7e6, 0x7cd, 0x7dd, 0x7ed, 0x7df, 0x7cc, 0x7db, 0x7d1, 0x7d3, 0x7d0, 0x7f2, 0x7eb,
0x7df, 0x7d1, 0x7d6, 0x7ce, 0x7d3, 0x7d1, 0x7d6, 0x7e3, 0x7ca, 0x7eb, 0x7df, 0x7d0,
0x7f2, 0x7d8, 0x7d9, 0x7cd, 0x7db, 0x7df, 0x7ce, 0x7de, 0x7d1, 0x7db, 0x7ca, 0x7f2,
0x7d3, 0x7ce, 0x7d9, 0x7dc, 0x7ed, 0x7e1, 0x7ca, 0x7d8, 0x7d9, 0x7cc, 0x7d8, 0x7e1,
0x7ca, 0x7df, 0x7d1, 0x7d5, 0x7e1, 0x7ca, 0x7d9, 0x7cc, 0x7e4, 0x7ca, 0x7e1, 0x7e1,
0x7cc, 0x7f2, 0x7db, 0x7de, 0x7e1, 0x7cf, 0x7e3, 0x7ca, 0x7de, 0x7cf, 0x7eb, 0x7e1,
0x7cf, 0x7db, 0x7d1, 0x7de, 0x7ce, 0x7eb, 0x7cf, 0x7db, 0x7d1, 0x7df, 0x7cf, 0x7eb,
0x7d4, 0x7ca, 0x7ec, 0x7d9, 0x7cc, 0x7eb, 0x7d4, 0x7cf, 0x7d8, 0x7dc, 0x7ed, 0x7e2,
0x7d9, 0x7cc, 0x7de, 0x7ca, 0x7eb, 0x7d4, 0x7d9, 0x7ca, 0x7dc, 0x7ed, 0x7d9, 0x7cc,
0x7dc, 0x7ed, 0x7ca, 0x7eb, 0x7d9, 0x7d0, 0x7e1, 0x7db, 0x7e1, 0x7ca, 0x7d9, 0x7ca,
0x7eb, 0x7db, 0x7cd, 0x7f2, 0x2d, 0x7e1, 0x7ca, 0x7d9, 0x7cc, 0x7e6, 0x7cf, 0x7eb,
0x7db, 0x7d9, 0x7ca, 0x7d6, 0x7ed, 0x7cb, 0x7dd, 0x7ed, 0x7cf, 0x7eb, 0x7db, 0x7d9,
0x7ca, 0x7d5, 0x7d0, 0x7dd, 0x7ed, 0x7db, 0x7cc, 0x7f2, 0x7dd, 0x7cb, 0x7d9, 0x7d4,
0x7cf, 0x7df, 0x7db, 0x7d1, 0x7de, 0x7cf, 0x7d4, 0x7d1, 0x7d6, 0x7cb, 0x7eb, 0x7db,
0x7cf, 0x7dd, 0x7cc, 0x7e6, 0x7ca, 0x7eb, 0x7db, 0x7d1, 0x7d5, 0x7cf, 0x7de, 0x7d0,
0x7df, 0x7e1, 0x7d5, 0x7ca, 0x7df, 0x7cc, 0x7f2, 0x7e3, 0x7d5, 0x7cc, 0x7d9, 0x7e3,
0x7ca, 0x7eb, 0x7ce, 0x7df, 0x7cc, 0x7e6, 0x7ca, 0x7e3, 0x7cf, 0x7dd, 0x7ed, 0x7d1,
0x7db, 0x7de, 0x7dd, 0x7ed, 0x7ee, 0x7ca, 0x7d8, 0x7ce, 0x7d6, 0x7ed, 0x7dd, 0x7ed,
0x7ca, 0x7d5, 0x7cc, 0x7de, 0x7ca, 0x7f2, 0x7eb, 0x7dd, 0x7ed, 0x7cc, 0x7e6, 0x7cd,
0x7e3, 0x7dd, 0x7ed, 0x7cc, 0x7df, 0x7d1, 0x7e3, 0x7cc, 0x7e6, 0x7ce, 0x7db, 0x7dd,
0x7ed, 0x7cf, 0x7df, 0x7dc, 0x7ed, 0x7cf, 0x7dc, 0x7ed, 0x7d9, 0x7ca, 0x7d8, 0x7dd,
0x7ed, 0x7ca, 0x7d9, 0x7d1, 0x7db, 0x7cf, 0x7dd, 0x7ed, 0x7cc, 0x7eb, 0x7d7, 0x7ed,
0x7ca, 0x7dc, 0x7ed, 0x7d9, 0x7cd, 0x7d3, 0x7d6, 0x7ed, 0x7ce, 0x7d9, 0x7cc, 0x7de,
0x7ca, 0x7f2, 0x7d5, 0x7e3, 0x7ca, 0x7e3, 0x7ca, 0x7d9, 0x7cc, 0x7dd, 0x7ed, 0x7cf,
0x7eb, 0x7db, 0x7ca, 0x7dc, 0x7ed, 0x7d0, 0x7db, 0x7de, 0x7d9, 0x7cc, 0x7db, 0x7d1,
0x7e1, 0x7ca, 0x7db, 0x7de, 0x7cf, 0x7de, 0x7cf, 0x7db, 0x7de, 0x7d0, 0x7e1, 0x7d0,
0x7ef, 0x7d9, 0x7de, 0x7cd, 0x7d9, 0x7d1, 0x7dc, 0x7ed, 0x7cb, 0x7df, 0x7cd, 0x7f2,
0x7eb, 0x7e1, 0x7ca, 0x7e4, 0x7cb, 0x7eb, 0x7e1, 0x7ca, 0x7df, 0x7d1, 0x7d8, 0x7cc,
0x7dd, 0x7ed, 0x7e1, 0x7cf, 0x7d9, 0x7cc, 0x7db, 0x7e1, 0x7ca, 0x7e6, 0x7d0, 0x7d5,
0x7d9, 0x7cb, 0x7e3, 0x7cc, 0x7e6, 0x7d0, 0x7f2, 0x7eb, 0x7ca, 0x7d4, 0x7cc, 0x7e6,
0x7ca, 0x7eb, 0x7cf, 0x7de, 0x7d1, 0x7df, 0x7ca, 0x7f2, 0x7d8, 0x7d3, 0x7ce, 0x7dc,
0x7ed, 0x7cd, 0x7f2, 0x7dd, 0x7ed, 0x7cc, 0x7df, 0x7d7, 0x7d5, 0x7ca, 0x7e1, 0x7d7,
0x7ce, 0x7ef, 0x7de, 0x7cd, 0x7db, 0x7d1, 0x7d5, 0x7cd, 0x7d9, 0x7cb, 0x7dd, 0x7ca,
0x7d5, 0x7cb, 0x7eb, 0x7dd, 0x7de, 0x7ca, 0x7cf, 0x7dd, 0x7cf, 0x7eb, 0x7dd, 0x7d6,
0x7cc, 0x7eb, 0x7dd, 0x7ce, 0x7e3, 0x7ca, 0x7dd, 0x7ce, 0x7d5, 0x7cc, 0x7eb, 0x7dc,
0x7ed, 0x7df, 0x7ca, 0x7d4, 0x7ca, 0x7dc, 0x7ed, 0x7d0, 0x7db, 0x7dc, 0x7ed, 0x7ca,
0x7f2, 0x7d3, 0x7cc, 0x7e6, 0x7cb, 0x7d9, 0x7dc, 0x7ed, 0x7ce, 0x7e5, 0x7d8, 0x7ca,
0x7df, 0x7d1, 0x7de, 0x7e3, 0x7ca, 0x7df, 0x7dc, 0x7ed, 0x7ce, 0x7e5, 0x7ca, 0x7f2,
0x7eb, 0x7de, 0x7ca, 0x7f2, 0x7d5, 0x7d0, 0x7f2, 0x7eb, 0x7de, 0x7d9, 0x7cc, 0x7d5,
0x7cc, 0x7e1, 0x7ca, 0x7d5, 0x7cc, 0x7eb, 0x7de, 0x7d0, 0x7db, 0x2e, 0x7d9, 0x7ca,
0x7eb, 0x7de, 0x7d1, 0x7e5, 0x7d6, 0x7ca, 0x7df, 0x7cb, 0x7e6, 0x7cc, 0x7f2, 0x7eb,
0x7e1, 0x7ca, 0x7d6, 0x7ce, 0x7d9, 0x7cf, 0x7eb, 0x7e1, 0x7ca, 0x7d9, 0x7de, 0x7cc,
0x7db, 0x7e1, 0x7cc, 0x7d8, 0x7d1, 0x7e5, 0x7ca, 0x7e6, 0x7e3, 0x7ca, 0x7e5, 0x7d9,
0x7ce, 0x7eb, 0x7e3, 0x7cc, 0x7e6, 0x7ce, 0x7cb, 0x7eb, 0x7e3, 0x7d0, 0x7d9, 0x7d1,
0x7dd, 0x7d0, 0x7df, 0x7de, 0x7e3, 0x7ce, 0x7e1, 0x7cb, 0x7e6, 0x7ca, 0x7eb, 0x7d4,
0x7ca, 0x7dc, 0x7ed, 0x7cf, 0x7eb, 0x20, 0x7e1, 0x7cf, 0x7dc, 0x7ed, 0x7cf, 0x7eb,
0x7d4, 0x7df, 0x7ca, 0x7cf, 0x7db, 0x7d4, 0x7cc, 0x7d5, 0x7de, 0x7cd, 0x7d9, 0x7e3,
0x7dd, 0x7cf, 0x7f2, 0x7d4, 0x7cb, 0x7e6, 0x7cc, 0x7eb, 0x7e1, 0x7cf, 0x7d9, 0x7cd,
0x7db, 0x7d1, 0x7d3, 0x7cc, 0x7eb, 0x20, 0x7d8, 0x7ca, 0x7f2, 0x7de, 0x7ca, 0x7f2,
0x7d9, 0x7ca, 0x7d9, 0x7cf, 0x7d5, 0x7cf, 0x7f2, 0x7dc, 0x7ed, 0x7ca, 0x7eb, 0x7db,
0x7cd, 0x7f2, 0x7d4, 0x7ca, 0x7f2, 0x7eb, 0x7d5, 0x7ca, 0x7e4, 0x7d5, 0x7cc, 0x7eb,
0x7d5, 0x7d9, 0x7ca, 0x7e5, 0x7ca, 0x7eb, 0x7d5, 0x7cf, 0x7f2, 0x7dc, 0x7ed, 0x7d5,
0x7ca, 0x7d4, 0x7ce, 0x7eb, 0x7e5, 0x7ca, 0x7de, 0x7cb, 0x7eb, 0x7e5, 0x7ca, 0x7df,
0x7df, 0x7cc, 0x7db, 0x44, 0xe1, 0x6e, 0x6d, 0xe1, 0x72, 0x6b, 0x6b, 0x75,
0x68, 0xe1, 0x6d, 0x6d, 0x61, 0x6e, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68,
0x2c, 0x20, 0x44, 0x61, 0x76, 0x76, 0x69, 0x2d, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x47, 0x75, 0x6f, 0x76, 0x64, 0x64, 0xe1, 0x161, 0x2c, 0x20,
0x44, 0x61, 0x76, 0x76, 0x69, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44,
0x61, 0x76, 0x76, 0x69, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x49,
0x72, 0x6b, 0x75, 0x63, 0x6b, 0x4e, 0x6f, 0x76, 0x6f, 0x6b, 0x75, 0x73,
0x6e, 0x65, 0x63, 0x6b, 0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74,
0x6f, 0x72, 0x4a, 0x61, 0x6b, 0x75, 0x63, 0x6b, 0x41, 0x7a, 0x6f, 0x72,
0x61, 0x74, 0x4b, 0x61, 0x6e, 0xe1, 0x72, 0x69, 0x61, 0x4c, 0x75, 0x6c,
0x6c, 0x69, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x48, 0x65,
0x6c, 0x73, 0x73, 0x65, 0x74, 0x4d, 0x61, 0x6e, 0x73, 0x75, 0x6f, 0x6c,
0x75, 0x4a, 0x75, 0x6f, 0x76, 0x6c, 0x61, 0x73, 0x75, 0x6f, 0x6c, 0x75,
0x4d, 0x61, 0x6c, 0x65, 0x64, 0x69, 0x69, 0x76, 0x76, 0x61, 0x74, 0x4d,
0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x61, 0x74, 0x43, 0x61,
0x79, 0x6d, 0x61, 0x6e, 0xf8, 0x79, 0x65, 0x6e, 0x65, 0x4d, 0x65, 0x78,
0x69, 0x63, 0x6f, 0x20, 0x62, 0x79, 0x54, 0x73, 0x6a, 0x69, 0x74, 0x61,
0x48, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x2d,
0x62, 0x79, 0x65, 0x6e, 0x4a, 0x61, 0x6a, 0x61, 0x70, 0x75, 0x72, 0x61,
0x41, 0x73, 0x6f, 0x72, 0x65, 0x6e, 0x65, 0x4b, 0x61, 0x6e, 0x61, 0x72,
0x69, 0xf8, 0x79, 0x65, 0x6e, 0x65, 0x4b, 0x61, 0x70, 0x70, 0x20, 0x56,
0x65, 0x72, 0x64, 0x65, 0x46, 0xe6, 0x72, 0xf8, 0x79, 0x65, 0x6e, 0x65,
0x53, 0xf8, 0x72, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x42,
0x75, 0x63, 0x75, 0x72, 0x65, 0x219, 0x74, 0x69, 0x48, 0x65, 0x6c, 0x73,
0x69, 0x6e, 0x67, 0x66, 0x6f, 0x72, 0x73, 0x56, 0x61, 0x74, 0x69, 0x6b,
0x61, 0x6e, 0x73, 0x74, 0x61, 0x74, 0x65, 0x6e, 0x43, 0x68, 0x72, 0x69,
0x73, 0x74, 0x6d, 0x61, 0x73, 0xf8, 0x79, 0x61, 0x4b, 0x6f, 0x6b, 0x6f,
0x73, 0xf8, 0x79, 0x65, 0x6e, 0x65, 0x4b, 0x6f, 0x6d, 0x6f, 0x72, 0x65,
0x6e, 0x65, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x6e, 0x65, 0x50,
0xe5, 0x73, 0x6b, 0x65, 0xf8, 0x79, 0x61, 0x47, 0x61, 0x6c, 0xe1, 0x70,
0x61, 0x67, 0x6f, 0x73, 0xf8, 0x79, 0x65, 0x6e, 0x65, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0xf8, 0x79, 0x61, 0x43, 0x61, 0x79, 0x6d, 0x61,
0x6e, 0xf8, 0x79, 0x61, 0x6e, 0x65, 0x43, 0x75, 0x69, 0x61, 0x62, 0x61,
0x41, 0x73, 0x6a, 0x67, 0x61, 0x62, 0x61, 0x74, 0x59, 0x61, 0x6e, 0x67,
0xf4, 0x6e, 0x41, 0x73, 0x6f, 0x72, 0x61, 0x6e, 0x65, 0x4b, 0x61, 0x6e,
0x61, 0x72, 0x69, 0xf8, 0x79, 0x61, 0x6e, 0x65, 0x46, 0xe6, 0x72, 0xf8,
0x79, 0x61, 0x6e, 0x65, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0xf8, 0x79, 0x61,
0x6e, 0x65, 0x4b, 0x6f, 0x6d, 0x6f, 0x72, 0x61, 0x6e, 0x65, 0x4d, 0x61,
0x6c, 0x64, 0x69, 0x76, 0x61, 0x6e, 0x65, 0x47, 0x61, 0x6c, 0xe1, 0x70,
0x61, 0x67, 0x6f, 0x73, 0xf8, 0x79, 0x61, 0x6e, 0x65, 0x41, 0x72, 0x67,
0x69, 0xe8, 0x72, 0x73, 0x4a, 0x69, 0x62, 0x6f, 0x74, 0x69, 0x44, 0x6f,
0x61, 0x6c, 0x61, 0x42, 0x61, 0x6e, 0x67, 0x6b, 0xf2, 0x6b, 0x4b, 0x61,
0x62, 0x6f, 0x6c, 0x53, 0x61, 0x6a, 0x61, 0x6c, 0x69, 0x6e, 0x45, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x41, 0xe7,
0xf2, 0x72, 0x65, 0x73, 0x41, 0x64, 0x65, 0x6c, 0x61, 0xef, 0x64, 0x61,
0x41, 0x6e, 0x64, 0xf2, 0x72, 0x72, 0x61, 0x42, 0x72, 0x75, 0x73, 0x73,
0xe8, 0x6c, 0x61, 0x73, 0x42, 0x75, 0x63, 0x61, 0x72, 0xe8, 0x73, 0x74,
0x42, 0x75, 0x64, 0x61, 0x70, 0xe8, 0x73, 0x74, 0x4c, 0x75, 0x78, 0x65,
0x6d, 0x62, 0x6f, 0x72, 0x67, 0x4d, 0xf3, 0x6e, 0x65, 0x67, 0x75, 0x65,
0x4d, 0x6f, 0x73, 0x63, 0xf2, 0x75, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e,
0x65, 0x73, 0x41, 0x62, 0x69, 0x79, 0x61, 0x6e, 0x41, 0x72, 0x67, 0x65,
0x6c, 0x45, 0x74, 0x68, 0x20, 0x43, 0x61, 0x69, 0x72, 0x6f, 0x59, 0x69,
0x62, 0x75, 0x74, 0x69, 0x4a, 0x61, 0x72, 0x74, 0x75, 0x6d, 0x59, 0x61,
0x6d, 0x65, 0x6e, 0x61, 0x4e, 0x75, 0x61, 0x6b, 0x63, 0x68, 0x6f, 0x74,
0x53, 0x61, 0x6e, 0x74, 0x20, 0x54, 0x6f, 0x6d, 0xe9, 0x43, 0xf2, 0x64,
0x6f, 0x62, 0x61, 0x42, 0x65, 0x6c, 0xe9, 0x6e, 0x42, 0x65, 0x6c, 0x69,
0x63, 0x65, 0x43, 0x61, 0x79, 0x65, 0x6e, 0x61, 0x43, 0xf2, 0x73, 0x74,
0x61, 0x20, 0x52, 0x69, 0x63, 0x61, 0x47, 0x72, 0x61, 0x6e, 0x20, 0x54,
0x75, 0x72, 0x63, 0x61, 0x45, 0x72, 0x61, 0x20, 0x48, 0x61, 0x76, 0x61,
0x6e, 0x61, 0x4d, 0x6d, 0x4d, 0x61, 0x6e, 0x61, 0x6f, 0x73, 0x43, 0x69,
0x75, 0x74, 0x61, 0x74, 0x20, 0x64, 0x65, 0x20, 0x4d, 0xe9, 0x78, 0x69,
0x63, 0x4e, 0xf2, 0x76, 0x61, 0x20, 0x59, 0x6f, 0x72, 0x6b, 0x42, 0x65,
0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x20, 0x64, 0x65, 0x74, 0x68, 0x20, 0x4e, 0xf2, 0x72, 0x64, 0x43, 0x65,
0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x20, 0x64, 0x65, 0x74, 0x68, 0x20, 0x4e, 0xf2, 0x72, 0x64, 0x4e, 0x65,
0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x20, 0x64, 0x65, 0x74, 0x68, 0x20, 0x4e, 0xf2, 0x72,
0x64, 0x50, 0x75, 0x65, 0x72, 0x74, 0x6f, 0x20, 0x50, 0x72, 0xed, 0x6e,
0x63, 0x69, 0x70, 0x65, 0x50, 0x75, 0x65, 0x72, 0x74, 0x6f, 0x20, 0x45,
0x73, 0x70, 0x61, 0x6e, 0x68, 0x61, 0x53, 0x61, 0x6f, 0x20, 0x70, 0x61,
0x75, 0x6c, 0x6f, 0x53, 0x61, 0x6e, 0x74, 0x20, 0x42, 0x61, 0x72, 0x74,
0x6f, 0x6c, 0x6f, 0x6d, 0xe9, 0x53, 0x61, 0x6e, 0x74, 0x20, 0x4a, 0x6f,
0x61, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x6e,
0xf2, 0x76, 0x61, 0x53, 0x61, 0x6e, 0x74, 0x20, 0x43, 0x72, 0x69, 0x73,
0x74, 0xf2, 0x62, 0x61, 0x6c, 0x53, 0x61, 0x6e, 0x74, 0x20, 0x56, 0x69,
0x63, 0x65, 0x6e, 0x74, 0x54, 0xf2, 0x72, 0x74, 0x6f, 0x6c, 0x61, 0x41,
0x6e, 0xe1, 0x64, 0x79, 0x72, 0x41, 0x73, 0x6a, 0x61, 0x62, 0x61, 0x64,
0x42, 0x61, 0x72, 0xe9, 0x69, 0x6e, 0x42, 0x61, 0x72, 0x6e, 0x61, 0xfa,
0x6c, 0x44, 0x75, 0x73, 0x61, 0x6d, 0x62, 0xe9, 0x43, 0x69, 0x75, 0x74,
0x61, 0x74, 0x20, 0x48, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69,
0x6e, 0x68, 0x4a, 0x65, 0x72, 0x75, 0x73, 0x61, 0x6c, 0x65, 0x6e, 0x53,
0x61, 0x6a, 0x61, 0x6c, 0xed, 0x6e, 0x53, 0x65, 0x6f, 0x6c, 0x54, 0x69,
0x66, 0x6c, 0x69, 0x65, 0x73, 0x45, 0x72, 0x65, 0x76, 0xe1, 0x6e, 0x49,
0x73, 0x6c, 0x65, 0x73, 0x20, 0x46, 0x65, 0x72, 0x6f, 0x65, 0x47, 0x65,
0xf2, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64, 0x65, 0x74, 0x68, 0x20, 0x53,
0x75, 0x64, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x20, 0x45, 0x6c, 0x65, 0x6e,
0x61, 0x53, 0xed, 0x64, 0x6e, 0x65, 0x79, 0x49, 0x73, 0x6c, 0x61, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x61, 0x6e, 0x45, 0x73, 0x74, 0x61, 0x6d, 0x62,
0x75, 0x6c, 0x4c, 0xf2, 0x6e, 0x64, 0x72, 0x65, 0x73, 0x53, 0x61, 0x6e,
0x74, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6e, 0x6f, 0x53, 0x61, 0x72, 0xe1,
0x74, 0x6f, 0x76, 0x53, 0x6b, 0x6f, 0x70, 0x69, 0x65, 0x55, 0x6c, 0x69,
0xe1, 0x6e, 0x6f, 0x76, 0x73, 0x6b, 0x45, 0x74, 0x68, 0x20, 0x56, 0x61,
0x74, 0x69, 0x63, 0x61, 0x6e, 0x5a, 0xfa, 0x72, 0x69, 0x63, 0x68, 0x4e,
0x61, 0x64, 0x61, 0x75, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20,
0x50, 0x61, 0x73, 0x71, 0x75, 0x61, 0x46, 0x69, 0x79, 0x69, 0xb06, 0xb2c,
0xb3f, 0xb26, 0xb1c, 0xb3e, 0xb28, 0xb06, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb06, 0xb26,
0xb3f, 0xb38, 0xb4d, 0x200c, 0x20, 0xb06, 0xb2c, 0xb3e, 0xb2c, 0xb3e, 0xb05, 0xb32,
0xb1c, 0xb3f, 0xb5f, 0xb30, 0xb4d, 0xb38, 0xb05, 0xb38, 0xb2e, 0xb30, 0xb3e, 0xb2c,
0xb3e, 0xb2e, 0xb3e, 0xb15, 0xb4b, 0xb2c, 0xb3e, 0xb19, 0xb4d, 0xb17, 0xb41, 0xb07,
0xb2c, 0xb3e, 0xb1e, 0xb4d, 0xb1c, 0xb41, 0xb32, 0xb2c, 0xb3f, 0xb38, 0xb3e, 0xb09,
0xb2c, 0xb4d, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb1f, 0xb3e, 0xb5f, 0xb3e, 0xb30, 0xb4d,
0x200c, 0xb2c, 0xb4d, 0xb30, 0xb3e, 0xb1c, 0xb3e, 0xb2d, 0xb3f, 0xb32, 0xb4d, 0xb32,
0xb47, 0xb2c, 0xb41, 0xb1c, 0xb41, 0xb2e, 0xb4d, 0xb2c, 0xb41, 0xb30, 0xb3e, 0xb15,
0xb3e, 0xb07, 0xb30, 0xb4b, 0xb15, 0xb3e, 0xb38, 0xb3e, 0xb2c, 0xb4d, 0xb32, 0xb3e,
0xb19, 0xb4d, 0xb15, 0xb3e, 0xb38, 0xb47, 0xb09, 0xb1f, 0xb3e, 0xb15, 0xb4b, 0xb28,
0xb3e, 0xb15, 0xb4d, 0xb30, 0xb3f, 0xb21, 0xb15, 0xb3e, 0xb30, 0xb21, 0xb30, 0xb4d,
0x200c, 0x20, 0xb07, 0xb38, 0xb4d, 0x200c, 0x20, 0xb38, 0xb3e, 0xb32, 0xb3e, 0xb2e,
0xb1c, 0xb3f, 0xb2c, 0xb4b, 0xb1f, 0xb3f, 0xb21, 0xb09, 0xb32, 0xb3e, 0xb0f, 0xb32,
0xb4d, 0x200c, 0x20, 0xb0f, 0xb5f, 0xb28, 0xb4d, 0xb2b, 0xb4d, 0xb30, 0xb3f, 0xb1f,
0xb3e, 0xb09, 0xb28, 0xb4d, 0x200c, 0xb17, 0xb3e, 0xb2c, 0xb4b, 0xb30, 0xb4d, 0xb23,
0xb4d, 0xb23, 0xb39, 0xb30, 0xb3e, 0xb30, 0xb47, 0xb1c, 0xb4b, 0xb39, 0xb3e, 0xb28,
0xb4d, 0xb38, 0xb2c, 0xb30, 0xb4d, 0xb17, 0xb1c, 0xb41, 0xb2c, 0xb3e, 0xb15, 0xb3e,
0xb2e, 0xb4d, 0xb2a, 0xb3e, 0xb32, 0xb3e, 0xb16, 0xb30, 0xb1f, 0xb4c, 0xb2e, 0xb4d,
0x200c, 0xb15, 0xb3f, 0xb17, 0xb3e, 0xb32, 0xb3f, 0xb15, 0xb3f, 0xb28, 0xb38, 0xb3e,
0xb38, 0xb4d, 0x200c, 0xb32, 0xb3e, 0xb17, 0xb4b, 0xb38, 0xb4d, 0x200c, 0xb32, 0xb3f,
0xb2c, 0xb4d, 0xb30, 0xb47, 0xb2d, 0xb3f, 0xb32, 0xb4d, 0xb32, 0xb47, 0xb32, 0xb4b,
0xb2e, 0xb4d, 0x200c, 0xb32, 0xb41, 0xb06, 0xb23, 0xb4d, 0xb21, 0xb3e, 0xb32, 0xb41,
0xb2c, 0xb41, 0xb2e, 0xb4d, 0xb2c, 0xb3e, 0xb36, 0xb3f, 0xb32, 0xb41, 0xb38, 0xb3e,
0xb15, 0xb3e, 0xb2e, 0xb3e, 0xb32, 0xb3e, 0xb2c, 0xb4b, 0xb2e, 0xb3e, 0xb2a, 0xb41,
0xb24, 0xb4b, 0xb2e, 0xb47, 0xb38, 0xb47, 0xb30, 0xb41, 0xb2c, 0xb3e, 0xb2c, 0xb47,
0xb28, 0xb4d, 0x200c, 0xb2e, 0xb4b, 0xb17, 0xb3e, 0xb26, 0xb3f, 0xb36, 0xb41, 0xb2e,
0xb28, 0xb30, 0xb4b, 0xb2d, 0xb3f, 0xb06, 0xb28, 0xb3e, 0xb07, 0xb30, 0xb4b, 0xb2c,
0xb3f, 0xb1c, 0xb3e, 0xb2e, 0xb47, 0xb28, 0xb3e, 0xb28, 0xb3f, 0xb06, 0xb2e, 0xb3f,
0xb28, 0xb4c, 0xb15, 0xb3e, 0xb1a, 0xb4b, 0xb1f, 0xb05, 0xb09, 0xb17, 0xb3e, 0xb21,
0xb09, 0xb17, 0xb09, 0xb2a, 0xb4b, 0xb30, 0xb4d, 0xb1f, 0xb4b, 0x2d, 0xb28, 0xb4b,
0xb2d, 0xb4b, 0xb38, 0xb3e, 0xb13, 0x20, 0xb1f, 0xb4b, 0xb2e, 0xb47, 0xb24, 0xb4d,
0xb30, 0xb3f, 0xb2a, 0xb4b, 0xb32, 0xb3f, 0xb1f, 0xb41, 0xb28, 0xb3f, 0xb38, 0xb4d,
0x200c, 0xb71, 0xb3f, 0xb23, 0xb4d, 0xb21, 0xb39, 0xb4b, 0xb0f, 0xb15, 0xb4d, 0xb06,
0xb21, 0xb3e, 0xb15, 0xb4d, 0xb06, 0xb19, 0xb4d, 0xb15, 0xb30, 0xb47, 0xb1c, 0xb4d,
0xb06, 0xb19, 0xb4d, 0xb17, 0xb41, 0xb07, 0xb32, 0xb3e, 0xb06, 0xb23, 0xb4d, 0xb1f,
0xb3f, 0xb17, 0xb41, 0xb06, 0xb06, 0xb30, 0xb3e, 0xb17, 0xb41, 0xb06, 0xb28, 0xb3e,
0xb2c, 0xb41, 0xb0f, 0xb28, 0xb38, 0xb4d, 0x200c, 0x20, 0xb0f, 0xb30, 0xb3f, 0xb38,
0xb4d, 0x200c, 0xb15, 0xb3e, 0xb1f, 0xb3e, 0xb2e, 0xb3e, 0xb30, 0xb4d, 0xb15, 0xb3e,
0xb15, 0xb4b, 0xb30, 0xb4d, 0xb21, 0xb4b, 0xb2c, 0xb3e, 0xb1c, 0xb41, 0xb1c, 0xb41,
0xb07, 0xb32, 0xb3e, 0x20, 0xb30, 0xb3f, 0xb13, 0xb1c, 0xb3e, 0xb2e, 0xb47, 0xb23,
0xb4d, 0xb21, 0xb4b, 0xb1c, 0xb3e, 0xb30, 0xb3f, 0xb13, 0x20, 0xb17, 0xb3e, 0xb32,
0xb4d, 0xb32, 0xb47, 0xb17, 0xb38, 0xb4d, 0x200c, 0xb38, 0xb32, 0xb4d, 0xb1f, 0xb3e,
0xb38, 0xb3e, 0xb28, 0xb4d, 0x200c, 0x20, 0xb1c, 0xb41, 0xb06, 0xb28, 0xb38, 0xb3e,
0xb28, 0xb4d, 0x200c, 0x20, 0xb32, 0xb41, 0xb07, 0xb38, 0xb4d, 0x200c, 0xb1f, 0xb41,
0xb15, 0xb41, 0xb2e, 0xb3e, 0xb28, 0xb4d, 0x200c, 0xb09, 0xb36, 0xb41, 0xb5f, 0xb3e,
0xb06, 0xb30, 0xb41, 0xb2c, 0xb3e, 0xb06, 0xb38, 0xb28, 0xb38, 0xb3f, 0xb05, 0xb28,
0xb4d, 0x200c, 0xb06, 0xb1f, 0xb3f, 0xb15, 0xb4b, 0xb15, 0xb3e, 0xb28, 0xb4d, 0xb2c,
0xb3e, 0xb39, 0xb3f, 0xb06, 0xb2c, 0xb3e, 0xb39, 0xb3f, 0xb06, 0x20, 0xb2c, 0xb4d,
0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb47, 0xb30, 0xb3e, 0xb38, 0xb4d, 0xb2c, 0xb3e,
0xb30, 0xb4d, 0xb2c, 0xb3e, 0xb21, 0xb4b, 0xb38, 0xb4d, 0xb2c, 0xb47, 0xb32, 0xb47,
0xb2e, 0xb2c, 0xb47, 0xb32, 0xb3f, 0xb1c, 0xb47, 0xb2c, 0xb4d, 0xb32, 0xb3e, 0xb19,
0xb4d, 0xb15, 0x2d, 0xb38, 0xb3e, 0xb2c, 0xb32, 0xb4b, 0xb28, 0xb4d, 0xb2c, 0xb4b,
0xb06, 0x20, 0xb2d, 0xb3f, 0xb37, 0xb4d, 0xb1f, 0xb3e, 0xb2c, 0xb4b, 0xb17, 0xb4b,
0xb1f, 0xb3e, 0xb2c, 0xb4b, 0xb07, 0xb38, 0xb47, 0xb15, 0xb47, 0xb2e, 0xb4d, 0xb35,
0xb4d, 0xb30, 0xb3f, 0xb1c, 0xb4d, 0x20, 0xb2c, 0xb47, 0xb15, 0xb3e, 0xb2e, 0xb4d,
0xb2a, 0xb4b, 0x20, 0xb17, 0xb4d, 0xb30, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb47, 0xb15,
0xb3e, 0xb28, 0xb15, 0xb41, 0xb28, 0xb4d, 0xb15, 0xb3e, 0xb30, 0xb15, 0xb3e, 0xb38,
0xb4d, 0x200c, 0xb15, 0xb47, 0xb5f, 0xb47, 0xb28, 0xb4d, 0xb28, 0xb3f, 0xb15, 0xb3e,
0xb5f, 0xb2e, 0xb4d, 0xb5f, 0xb3e, 0xb28, 0xb4d, 0xb1a, 0xb3f, 0xb15, 0xb3e, 0xb17,
0xb4b, 0xb1a, 0xb3f, 0xb39, 0xb41, 0xb06, 0xb39, 0xb41, 0xb06, 0xb38, 0xb3f, 0xb09,
0xb21, 0xb3e, 0xb21, 0xb4d, 0x20, 0xb1c, 0xb41, 0xb06, 0xb30, 0xb47, 0xb1c, 0xb4d,
0xb15, 0xb4b, 0xb37, 0xb4d, 0xb1f, 0xb3e, 0x20, 0xb30, 0xb3f, 0xb15, 0xb3e, 0xb15,
0xb4d, 0xb30, 0xb47, 0xb37, 0xb4d, 0xb1f, 0xb4b, 0xb28, 0xb4d, 0xb15, 0xb41, 0xb07,
0xb2c, 0xb3e, 0xb15, 0xb41, 0xb30, 0xb3e, 0xb15, 0xb4b, 0xb21, 0xb3e, 0xb28, 0xb2e,
0xb3e, 0xb30, 0xb4d, 0xb15, 0xb38, 0xb3e, 0xb2d, 0xb28, 0xb4d, 0xb21, 0xb38, 0xb28,
0xb4d, 0x200c, 0xb21, 0xb71, 0xb38, 0xb28, 0xb4d, 0x20, 0xb15, 0xb4d, 0xb30, 0xb3f,
0xb15, 0xb4d, 0xb21, 0xb47, 0xb28, 0xb2d, 0xb3f, 0xb30, 0xb4d, 0xb21, 0xb47, 0xb1f,
0xb4d, 0xb30, 0xb4b, 0xb07, 0xb1f, 0xb4d, 0xb21, 0xb4b, 0xb2e, 0xb3f, 0xb28, 0xb3f,
0xb15, 0xb3e, 0xb0f, 0xb21, 0xb2e, 0xb4b, 0xb28, 0xb1f, 0xb4b, 0xb28, 0xb4d, 0xb07,
0xb30, 0xb41, 0xb28, 0xb47, 0xb2a, 0xb47, 0xb0f, 0xb32, 0xb4d, 0x20, 0xb38, 0xb3e,
0xb32, 0xb2d, 0xb3e, 0xb21, 0xb4b, 0xb30, 0xb4d, 0xb2b, 0xb4b, 0xb30, 0xb4d, 0xb1f,
0xb4d, 0x20, 0xb28, 0xb47, 0xb32, 0xb38, 0xb28, 0xb4d, 0xb2b, 0xb4b, 0xb30, 0xb4d,
0xb1f, 0xb47, 0xb32, 0xb47, 0xb1c, 0xb3e, 0xb17, 0xb4d, 0xb32, 0xb3e, 0xb38, 0xb47,
0x20, 0xb2c, 0xb47, 0xb17, 0xb41, 0xb38, 0xb4d, 0x20, 0xb2c, 0xb47, 0xb17, 0xb4d,
0xb30, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb4d, 0x20, 0xb1f, 0xb30, 0xb4d, 0xb15, 0xb4d,
0xb17, 0xb4d, 0xb30, 0xb47, 0xb28, 0xb3e, 0xb21, 0xb3e, 0xb17, 0xb41, 0xb06, 0xb21,
0xb47, 0xb32, 0xb4b, 0xb2a, 0xb4d, 0x200c, 0xb17, 0xb41, 0xb06, 0xb1f, 0xb47, 0xb2e,
0xb3e, 0xb32, 0xb3e, 0xb17, 0xb41, 0xb5f, 0xb3e, 0xb15, 0xb4d, 0xb71, 0xb3f, 0xb32,
0xb17, 0xb41, 0xb0f, 0xb28, 0xb3e, 0xb39, 0xb3e, 0xb32, 0xb3f, 0xb2b, 0xb4d, 0xb5f,
0xb3e, 0xb15, 0xb4d, 0xb38, 0xb4d, 0xb39, 0xb3e, 0xb2d, 0xb3e, 0xb28, 0xb3e, 0xb39,
0xb47, 0xb30, 0xb2e, 0xb4b, 0xb38, 0xb3f, 0xb32, 0xb4b, 0xb07, 0xb23, 0xb4d, 0xb21,
0xb3f, 0xb06, 0xb28, 0xb3e, 0xb2a, 0xb4b, 0xb32, 0xb3f, 0xb38, 0xb4d, 0xb15, 0xb4d,
0xb28, 0xb4b, 0xb15, 0xb4d, 0xb38, 0x2c, 0x20, 0xb07, 0xb23, 0xb4d, 0xb21, 0xb3f,
0xb06, 0xb28, 0xb3e, 0xb2e, 0xb3e, 0xb30, 0xb47, 0xb28, 0xb17, 0xb4b, 0x2c, 0x20,
0xb07, 0xb23, 0xb4d, 0xb21, 0xb3f, 0xb06, 0xb28, 0xb3e, 0xb2a, 0xb47, 0xb1f, 0xb47,
0xb30, 0xb4d, 0xb38, 0xb2c, 0xb30, 0xb4d, 0xb17, 0xb4d, 0x2c, 0x20, 0xb07, 0xb23,
0xb4d, 0xb21, 0xb3f, 0xb06, 0xb28, 0xb3e, 0xb1f, 0xb47, 0xb32, 0xb4d, 0x20, 0xb38,
0xb3f, 0xb1f, 0xb3f, 0x2c, 0x20, 0xb07, 0xb23, 0xb4d, 0xb21, 0xb3f, 0xb06, 0xb28,
0xb3e, 0xb2d, 0xb47, 0xb2d, 0xb3e, 0xb5f, 0x2c, 0x20, 0xb07, 0xb23, 0xb4d, 0xb21,
0xb3f, 0xb06, 0xb28, 0xb3e, 0xb2d, 0xb3f, 0xb28, 0xb38, 0xb47, 0xb28, 0xb4d, 0xb28,
0xb47, 0xb38, 0xb4d, 0x2c, 0x20, 0xb07, 0xb23, 0xb4d, 0xb21, 0xb3f, 0xb06, 0xb28,
0xb3e, 0xb71, 0xb3f, 0xb28, 0xb3e, 0xb2e, 0xb3e, 0xb15, 0xb4d, 0x2c, 0x20, 0xb07,
0xb23, 0xb4d, 0xb21, 0xb3f, 0xb06, 0xb28, 0xb3e, 0xb07, 0xb28, 0xb41, 0xb2d, 0xb3f,
0xb15, 0xb4d, 0xb07, 0xb15, 0xb4d, 0xb35, 0xb3e, 0xb32, 0xb41, 0xb07, 0xb1f, 0xb4d,
0xb1c, 0xb3e, 0xb2e, 0xb3e, 0xb07, 0xb15, 0xb3e, 0xb1c, 0xb41, 0xb28, 0xb47, 0xb06,
0xb09, 0xb32, 0xb41, 0xb07, 0xb2d, 0xb3f, 0xb32, 0xb4d, 0xb32, 0xb47, 0xb2e, 0xb4b,
0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb38, 0xb47, 0xb32, 0xb4b, 0x2c, 0x20, 0xb15, 0xb47,
0xb23, 0xb4d, 0xb1f, 0xb09, 0xb15, 0xb3f, 0xb15, 0xb47, 0xb15, 0xb4d, 0xb30, 0xb3e,
0xb32, 0xb47, 0xb23, 0xb4d, 0xb21, 0xb1c, 0xb3f, 0xb15, 0xb32, 0xb3e, 0x20, 0xb2a,
0xb3e, 0xb1c, 0xb4d, 0x200c, 0xb32, 0xb3f, 0xb2e, 0xb3e, 0xb32, 0xb38, 0xb4d, 0x20,
0xb0f, 0xb1e, 0xb4d, 0xb1c, 0xb47, 0xb32, 0xb47, 0xb38, 0xb4d, 0xb28, 0xb3f, 0xb2e,
0xb4d, 0xb28, 0x20, 0xb2a, 0xb4d, 0xb30, 0xb3f, 0xb28, 0xb4d, 0xb38, 0x2019, 0xb30,
0x20, 0xb15, 0xb4d, 0xb35, 0xb3e, 0xb1f, 0xb30, 0xb4d, 0xb2e, 0xb3e, 0xb38, 0xb3f,
0xb13, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb17, 0xb41, 0xb06, 0xb2e, 0xb3e, 0xb28, 0xb3e,
0xb09, 0xb38, 0xb4d, 0x200c, 0xb2e, 0xb3e, 0xb30, 0xb3f, 0xb17, 0xb4b, 0xb1f, 0xb4d,
0xb2e, 0xb3e, 0xb30, 0xb4d, 0xb1f, 0xb3f, 0xb28, 0xb3f, 0xb15, 0xb4d, 0xb5f, 0xb41,
0xb2e, 0xb3e, 0xb1f, 0xb3e, 0xb2e, 0xb4b, 0xb30, 0xb4b, 0xb38, 0xb4d, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb1f, 0xb32, 0xb3e, 0xb28, 0xb4d, 0xb2e, 0xb47, 0xb28, 0xb4b, 0xb2e,
0xb3f, 0xb28, 0xb3f, 0xb2e, 0xb47, 0xb30, 0xb3f, 0xb21, 0xb3e, 0xb2e, 0xb3e, 0xb1f,
0xb32, 0xb3e, 0xb15, 0xb3e, 0xb1f, 0xb32, 0xb3e, 0xb2e, 0xb47, 0xb15, 0xb4d, 0xb38,
0xb3f, 0xb15, 0xb4b, 0x20, 0xb38, 0xb3f, 0xb1f, 0xb3f, 0xb2e, 0xb3f, 0xb15, 0xb4d,
0xb35, 0xb47, 0xb32, 0xb4b, 0xb28, 0xb4d, 0xb2e, 0xb3e, 0xb28, 0xb15, 0xb1f, 0xb4b,
0xb28, 0xb4d, 0xb2e, 0xb4b, 0xb28, 0xb1f, 0xb47, 0xb30, 0xb3f, 0xb0f, 0xb2e, 0xb23,
0xb4d, 0xb1f, 0xb47, 0xb2d, 0xb3f, 0xb21, 0xb3f, 0xb13, 0xb2e, 0xb4b, 0xb28, 0xb1f,
0xb38, 0xb47, 0xb30, 0xb30, 0xb3e, 0xb1f, 0xb4d, 0xb28, 0xb3e, 0xb38, 0xb3e, 0xb09,
0xb28, 0xb4d, 0xb5f, 0xb41, 0x20, 0xb5f, 0xb4b, 0xb30, 0xb4d, 0xb15, 0xb4d, 0xb28,
0xb4b, 0xb2e, 0xb47, 0xb28, 0xb4b, 0xb30, 0xb4b, 0xb28, 0xb4d, 0xb28, 0xb3e, 0xb2c,
0xb47, 0xb09, 0xb32, 0xb3e, 0xb39, 0x2c, 0x20, 0xb09, 0xb24, 0xb4d, 0xb24, 0xb30,
0x20, 0xb21, 0xb3e, 0xb15, 0xb4b, 0xb1f, 0xb3e, 0xb15, 0xb47, 0xb28, 0xb4d, 0xb26,
0xb4d, 0xb30, 0x2c, 0x20, 0xb09, 0xb24, 0xb4d, 0xb24, 0xb30, 0x20, 0xb21, 0xb3e,
0xb15, 0xb4b, 0xb1f, 0xb3e, 0xb28, 0xb4d, 0xb5f, 0xb41, 0x20, 0xb38, 0xb3e, 0xb32,
0xb47, 0xb2e, 0xb4d, 0x2c, 0x20, 0xb09, 0xb24, 0xb4d, 0xb24, 0xb30, 0x20, 0xb21,
0xb3e, 0xb15, 0xb4b, 0xb1f, 0xb3e, 0xb28, 0xb41, 0xb09, 0xb15, 0xb4d, 0xb13, 0xb1c,
0xb3f, 0xb28, 0xb3e, 0xb17, 0xb3e, 0xb2a, 0xb3e, 0xb28, 0xb3e, 0xb2e, 0xb3e, 0xb2a,
0xb3e, 0xb30, 0xb3e, 0xb2e, 0xb3e, 0xb30, 0xb3f, 0xb2c, 0xb4b, 0xb2b, 0xb4b, 0xb07,
0xb28, 0xb3f, 0xb15, 0xb4d, 0xb38, 0xb2a, 0xb4b, 0xb30, 0xb4d, 0xb1f, 0xb4d, 0x2d,
0xb0f, 0xb5f, 0xb41, 0x2d, 0xb2a, 0xb4d, 0xb30, 0xb3f, 0xb28, 0xb4d, 0xb38, 0xb2a,
0xb4b, 0xb30, 0xb4d, 0xb1f, 0xb4d, 0x20, 0xb05, 0xb2b, 0xb4d, 0x20, 0xb38, 0xb4d,
0xb2a, 0xb47, 0xb28, 0xb4d, 0xb2a, 0xb4b, 0xb30, 0xb4d, 0xb1f, 0xb4b, 0x20, 0xb2d,
0xb47, 0xb32, 0xb4b, 0xb2a, 0xb41, 0xb0f, 0xb30, 0xb4d, 0xb24, 0xb4b, 0x20, 0xb30,
0xb3f, 0xb15, 0xb4b, 0xb2a, 0xb41, 0xb23, 0xb4d, 0xb1f, 0xb3e, 0x20, 0xb0f, 0xb30,
0xb3f, 0xb28, 0xb3e, 0xb38, 0xb4d, 0x200c, 0xb30, 0xb3e, 0xb28, 0xb15, 0xb3f, 0xb28,
0xb4d, 0x20, 0xb07, 0xb28, 0xb32, 0xb47, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb38, 0xb3f,
0xb2b, 0xb3f, 0xb30, 0xb47, 0xb17, 0xb3f, 0xb28, 0xb3e, 0xb30, 0xb3f, 0xb38, 0xb4b,
0xb32, 0xb41, 0xb1f, 0xb47, 0xb30, 0xb3f, 0xb13, 0x20, 0xb2c, 0xb4d, 0xb30, 0xb3e,
0xb19, 0xb4d, 0xb15, 0xb4b, 0xb38, 0xb3e, 0xb23, 0xb4d, 0xb1f, 0xb3e, 0xb30, 0xb47,
0xb2e, 0xb4d, 0x200c, 0xb38, 0xb3e, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb06, 0xb17, 0xb4b,
0xb38, 0xb3e, 0xb23, 0xb4d, 0xb1f, 0xb4b, 0x20, 0xb21, 0xb4b, 0xb2e, 0xb3f, 0xb19,
0xb4d, 0xb17, 0xb4b, 0xb38, 0xb3e, 0xb13, 0x20, 0xb2a, 0xb3e, 0xb13, 0xb32, 0xb4b,
0xb07, 0xb1f, 0xb4d, 0xb1f, 0xb4b, 0xb15, 0xb4d, 0xb35, 0xb4b, 0xb30, 0xb1f, 0xb4b,
0xb30, 0xb2e, 0xb3f, 0xb1f, 0xb4d, 0xb38, 0xb3f, 0xb1f, 0xb15, 0xb3e, 0xb38, 0xb47,
0xb23, 0xb4d, 0xb1f, 0xb4d, 0x2e, 0x20, 0xb2c, 0xb3e, 0xb30, 0xb4d, 0xb25, 0xb47,
0xb32, 0xb47, 0xb2e, 0xb3f, 0xb38, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb4d, 0x2e, 0x20,
0xb1c, 0xb28, 0xb38, 0xb4d, 0xb38, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb4d, 0x20, 0xb15,
0xb3f, 0xb1f, 0xb4d, 0xb38, 0xb4d, 0xb38, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb4d, 0x2e,
0x20, 0xb32, 0xb41, 0xb38, 0xb3f, 0xb06, 0xb38, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb4d,
0x2e, 0x20, 0xb25, 0xb4b, 0xb2e, 0xb3e, 0xb38, 0xb4d, 0xb38, 0xb47, 0xb23, 0xb4d,
0xb1f, 0xb4d, 0x2e, 0x20, 0xb2d, 0xb3f, 0xb28, 0xb38, 0xb47, 0xb23, 0xb4d, 0xb1f,
0xb4d, 0xb38, 0xb41, 0xb07, 0xb2b, 0xb4d, 0xb1f, 0xb4d, 0x20, 0xb15, 0xb30, 0xb47,
0xb23, 0xb4d, 0xb1f, 0xb4d, 0xb1f, 0xb47, 0xb17, 0xb41, 0xb38, 0xb3f, 0xb17, 0xb3e,
0xb32, 0xb2a, 0xb3e, 0xb25, 0xb41, 0xb32, 0xb47, 0xb24, 0xb3f, 0xb1c, 0xb41, 0xb06,
0xb28, 0xb3e, 0xb1f, 0xb4b, 0xb30, 0xb4b, 0xb23, 0xb4d, 0xb1f, 0xb4b, 0xb1f, 0xb4b,
0xb30, 0xb1f, 0xb4b, 0xb32, 0xb3e, 0xb2d, 0xb3e, 0xb19, 0xb4d, 0xb15, 0xb41, 0xb2d,
0xb30, 0xb4d, 0xb39, 0xb4d, 0xb35, 0xb3e, 0xb07, 0xb1f, 0xb39, 0xb30, 0xb4d, 0xb38,
0xb4d, 0xb71, 0xb3f, 0xb28, 0xb3f, 0xb2a, 0xb47, 0xb17, 0xb4d, 0xb5f, 0xb3e, 0xb15,
0xb41, 0xb1f, 0xb3e, 0xb1f, 0xb4d, 0xb21, 0xb47, 0xb2d, 0xb3f, 0xb38, 0xb4d, 0x200c,
0xb21, 0xb41, 0xb2e, 0xb4b, 0xb23, 0xb4d, 0xb1f, 0x20, 0xb21, 0xb3f, 0x2018, 0xb09,
0xb30, 0xb2d, 0xb3f, 0xb32, 0xb4d, 0xb32, 0xb47, 0xb2e, 0xb3e, 0xb15, 0xb4d, 0xb71,
0xb47, 0xb30, 0xb40, 0xb2e, 0xb3e, 0xb71, 0xb38, 0xb28, 0xb4d, 0xb2e, 0xb4d, 0xb5f,
0xb3e, 0xb15, 0xb4d, 0x200c, 0xb2e, 0xb41, 0xb30, 0xb4d, 0xb21, 0xb4b, 0xb2a, 0xb3e,
0xb32, 0xb2e, 0xb30, 0xb4d, 0x200c, 0xb30, 0xb4b, 0xb25, 0xb47, 0xb30, 0xb3e, 0xb38,
0xb4b, 0xb71, 0xb3e, 0xb1f, 0xb4d, 0xb30, 0xb4b, 0xb32, 0xb4d, 0xb2d, 0xb4b, 0xb37,
0xb4d, 0xb1f, 0xb4b, 0xb15, 0xb4d, 0x200c, 0xb32, 0xb19, 0xb4d, 0xb17, 0xb5f, 0xb47,
0xb06, 0xb30, 0xb2c, 0xb5f, 0xb47, 0xb28, 0xb4d, 0xb0f, 0xb21, 0xb47, 0xb28, 0xb4d,
0x200c, 0xb05, 0xb32, 0xb2e, 0xb3e, 0xb1f, 0xb3f, 0xb05, 0xb2e, 0xb4d, 0xb2e, 0xb3e,
0xb28, 0xb06, 0xb28, 0xb3e, 0xb21, 0xb5f, 0xb3e, 0xb30, 0xb4d, 0xb06, 0xb15, 0xb1f,
0xb3e, 0xb09, 0xb06, 0xb15, 0xb1f, 0xb4b, 0xb2c, 0xb4d, 0x200c, 0xb06, 0xb36, 0xb4d,
0x200c, 0xb17, 0xb3e, 0xb2c, 0xb4b, 0xb1f, 0xb4d, 0x200c, 0xb05, 0xb24, 0xb3f, 0xb30,
0xb3e, 0xb09, 0xb2c, 0xb3e, 0xb17, 0xb26, 0xb3e, 0xb26, 0xb4d, 0x200c, 0xb2c, 0xb3e,
0xb39, 0xb3e, 0xb30, 0xb3f, 0xb28, 0xb4d, 0xb2c, 0xb4d, 0xb5f, 0xb3e, 0xb19, 0xb4d,
0xb17, 0xb15, 0xb15, 0xb4d, 0x200c, 0xb2c, 0xb3e, 0xb30, 0xb28, 0xb3e, 0xb09, 0xb32,
0xb4d, 0xb2c, 0xb40, 0xb30, 0xb41, 0xb1f, 0xb4d, 0x200c, 0xb2c, 0xb3f, 0xb36, 0xb15,
0xb47, 0xb15, 0xb4d, 0x200c, 0xb2c, 0xb4d, 0xb30, 0xb41, 0xb28, 0xb47, 0xb07, 0xb1a,
0xb3f, 0xb1f, 0xb3e, 0xb15, 0xb32, 0xb2e, 0xb4d, 0xb2c, 0xb4b, 0xb21, 0xb3e, 0xb2e,
0xb3e, 0xb38, 0xb15, 0xb38, 0xb4d, 0x200c, 0xb22, 0xb3e, 0xb15, 0xb3e, 0xb26, 0xb3f,
0xb32, 0xb4d, 0xb32, 0xb40, 0xb26, 0xb41, 0xb2c, 0xb3e, 0xb07, 0xb26, 0xb41, 0xb36,
0xb3e, 0xb28, 0xb2c, 0xb47, 0xb2b, 0xb3e, 0xb2e, 0xb3e, 0xb17, 0xb41, 0xb37, 0xb4d,
0xb1f, 0xb3e, 0xb17, 0xb3e, 0xb1c, 0xb3e, 0xb39, 0xb47, 0xb2c, 0xb4d, 0xb30, 0xb28,
0xb4d, 0x200c, 0xb39, 0xb4b, 0x20, 0xb1a, 0xb3f, 0x20, 0xb2e, 0xb3f, 0xb28, 0xb4d,
0x200c, 0x20, 0xb38, 0xb3f, 0xb1f, 0xb3f, 0xb39, 0xb02, 0x20, 0xb15, 0xb02, 0xb39,
0xb4b, 0xb2d, 0xb21, 0xb4d, 0x200c, 0xb07, 0xb30, 0xb15, 0xb41, 0xb1f, 0xb38, 0xb4d,
0xb15, 0xb1c, 0xb3e, 0xb15, 0xb30, 0xb4d, 0xb24, 0xb4d, 0xb24, 0xb3e, 0xb1c, 0xb5f,
0xb2a, 0xb41, 0xb30, 0xb3e, 0xb1c, 0xb47, 0xb30, 0xb41, 0xb1c, 0xb47, 0xb32, 0xb2e,
0xb15, 0xb3e, 0xb2c, 0xb41, 0xb32, 0xb15, 0xb3e, 0xb2e, 0xb1a, 0xb3e, 0xb1f, 0xb15,
0xb3e, 0xb15, 0xb30, 0xb3e, 0xb1a, 0xb3f, 0xb15, 0xb3e, 0xb20, 0xb2e, 0xb3e, 0xb23,
0xb4d, 0xb21, 0xb41, 0xb16, 0xb3e, 0xb28, 0xb21, 0xb4d, 0xb5f, 0xb3e, 0xb17, 0xb3e,
0xb15, 0xb4b, 0xb32, 0xb15, 0xb3e, 0xb24, 0xb3e, 0xb15, 0xb4d, 0xb30, 0xb3e, 0xb38,
0xb28, 0xb4b, 0xb5f, 0xb3e, 0xb30, 0xb38, 0xb4d, 0xb15, 0xb15, 0xb4d, 0xb71, 0xb3e,
0xb32, 0xb3e, 0xb32, 0xb2e, 0xb4d, 0xb2a, 0xb41, 0xb30, 0xb15, 0xb41, 0xb1a, 0xb3f,
0xb02, 0xb15, 0xb41, 0xb0f, 0xb24, 0xb2e, 0xb3e, 0xb15, 0xb3e, 0xb09, 0xb2e, 0xb3e,
0xb17, 0xb3e, 0xb21, 0xb3e, 0xb28, 0xb4d, 0xb2e, 0xb3e, 0xb15, 0xb3e, 0xb38, 0xb3e,
0xb30, 0xb4d, 0x200c, 0xb2e, 0xb3e, 0xb28, 0xb3f, 0xb32, 0xb3e, 0xb2e, 0xb38, 0xb4d,
0xb15, 0xb3e, 0xb1f, 0xb4d, 0x200c, 0xb28, 0xb3f, 0xb15, 0xb4b, 0xb38, 0xb3f, 0xb06,
0xb28, 0xb4b, 0xb2d, 0xb4b, 0xb15, 0xb41, 0xb1c, 0xb28, 0xb47, 0xb1f, 0xb38, 0xb4d,
0xb15, 0xb28, 0xb4b, 0xb2d, 0xb4b, 0xb38, 0xb3f, 0xb2c, 0xb3f, 0xb30, 0xb38, 0xb4d,
0xb15, 0xb13, 0xb2e, 0xb38, 0xb4d, 0xb15, 0xb13, 0xb30, 0xb3e, 0xb32, 0xb4d, 0x200c,
0xb2b, 0xb28, 0xb4b, 0xb2e, 0xb4d, 0x200c, 0x20, 0xb2a, 0xb47, 0xb28, 0xb39, 0xb2a,
0xb4b, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb06, 0xb28, 0xb3e, 0xb15, 0xb4d, 0x200c, 0xb2a,
0xb4b, 0xb5f, 0xb19, 0xb4d, 0xb17, 0xb5f, 0xb3e, 0xb19, 0xb4d, 0xb17, 0xb15, 0xb24,
0xb3e, 0xb30, 0xb4d, 0xb15, 0xb4b, 0xb37, 0xb4d, 0xb1f, 0xb28, 0xb47, 0xb15, 0xb40,
0xb1c, 0xb3f, 0xb32, 0xb4b, 0xb30, 0xb4d, 0xb21, 0xb3e, 0xb30, 0xb3f, 0xb06, 0xb26,
0xb38, 0xb16, 0xb3e, 0xb32, 0xb3f, 0xb28, 0xb4d, 0xb38, 0xb2e, 0xb30, 0xb15, 0xb28,
0xb4d, 0xb26, 0xb38, 0xb3f, 0xb13, 0xb32, 0xb38, 0xb3e, 0xb02, 0xb18, 0xb3e, 0xb07,
0xb38, 0xb3f, 0xb19, 0xb4d, 0xb17, 0xb3e, 0xb2a, 0xb41, 0xb30, 0xb4d, 0x200c, 0xb38,
0xb4d, 0xb30, 0xb47, 0xb21, 0xb28, 0xb47, 0xb15, 0xb4b, 0xb32, 0xb5f, 0xb2e, 0xb38,
0xb4d, 0xb15, 0xb24, 0xb3e, 0xb07, 0xb2a, 0xb47, 0xb07, 0xb24, 0xb3e, 0xb36, 0xb15,
0xb47, 0xb23, 0xb4d, 0xb1f, 0xb1f, 0xb3f, 0xb2c, 0xb3f, 0xb32, 0xb3f, 0xb38, 0xb3f,
0xb24, 0xb47, 0xb39, 0xb47, 0xb30, 0xb3e, 0xb28, 0xb4d, 0xb25, 0xb3f, 0xb2e, 0xb4d,
0xb2a, 0xb41, 0xb1f, 0xb4b, 0xb15, 0xb3f, 0xb13, 0xb1f, 0xb4b, 0xb2e, 0xb38, 0xb4d,
0xb15, 0xb09, 0xb32, 0xb3e, 0xb28, 0xb4d, 0x200c, 0xb2c, 0xb3e, 0xb1f, 0xb30, 0xb4d,
0x200c, 0xb09, 0xb30, 0xb41, 0xb2e, 0xb15, 0xb3f, 0xb5f, 0xb41, 0xb37, 0xb4d, 0xb1f,
0x2d, 0xb28, 0xb47, 0xb30, 0xb3e, 0xb2d, 0xb3f, 0xb0f, 0xb23, 0xb4d, 0xb1f, 0xb3f,
0xb0f, 0xb28, 0xb4d, 0x200c, 0xb2d, 0xb4d, 0xb32, 0xb3e, 0xb21, 0xb3f, 0xb2d, 0xb4b,
0xb37, 0xb4d, 0xb1f, 0xb4b, 0xb15, 0xb4d, 0xb5f, 0xb3e, 0xb15, 0xb41, 0xb1f, 0xb38,
0xb4d, 0xb15, 0xb5f, 0xb3e, 0xb19, 0xb4d, 0xb17, 0xb41, 0xb28, 0xb4d, 0x200c, 0xb5f,
0xb47, 0xb15, 0xb3e, 0xb1f, 0xb47, 0xb30, 0xb3f, 0xb28, 0xb2c, 0xb30, 0xb4d, 0xb17,
0xb4d, 0xb5f, 0xb47, 0xb30, 0xb47, 0xb2d, 0xb3e, 0xb28, 0xb4d, 0xb06, 0xb1c, 0xb4b,
0xb30, 0xb47, 0xb38, 0xb4d, 0xb2c, 0xb30, 0xb4d, 0xb2e, 0xb41, 0xb21, 0xb3e, 0xb15,
0xb3e, 0xb28, 0xb3e, 0xb30, 0xb47, 0xb15, 0xb47, 0xb2a, 0xb4d, 0x200c, 0x20, 0xb2d,
0xb30, 0xb4d, 0xb26, 0xb47, 0xb2b, 0xb3e, 0xb30, 0xb4b, 0xb0f, 0xb2e, 0xb3e, 0xb21,
0xb47, 0xb07, 0xb30, 0xb3e, 0xb30, 0xb47, 0xb5f, 0xb15, 0xb1c, 0xb3e, 0xb2d, 0xb3f,
0xb15, 0xb4d, 0xb26, 0xb15, 0xb4d, 0xb37, 0xb3f, 0xb23, 0x20, 0xb1c, 0xb30, 0xb4d,
0xb1c, 0xb3f, 0xb06, 0xb38, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb4d, 0x200c, 0x20, 0xb39,
0xb47, 0xb32, 0xb3f, 0xb28, 0xb3e, 0xb37, 0xb4d, 0xb1f, 0xb3e, 0xb28, 0xb32, 0xb3f,
0xb06, 0xb21, 0xb3f, 0xb32, 0xb47, 0xb21, 0xb4d, 0x200c, 0xb2c, 0xb4d, 0xb30, 0xb3f,
0xb38, 0xb2c, 0xb47, 0xb28, 0xb4d, 0x200c, 0xb2c, 0xb4d, 0xb30, 0xb4b, 0xb15, 0xb47,
0xb28, 0xb4d, 0x200c, 0x20, 0xb39, 0xb3f, 0xb32, 0xb4d, 0x200c, 0xb21, 0xb3e, 0xb30,
0xb71, 0xb3f, 0xb28, 0xb4d, 0x200c, 0xb5f, 0xb41, 0xb15, 0xb32, 0xb3e, 0xb39, 0xb4b,
0xb2c, 0xb3e, 0xb30, 0xb4d, 0xb1f, 0xb4d, 0x200c, 0xb32, 0xb3f, 0xb23, 0xb4d, 0xb21,
0xb47, 0xb2e, 0xb4d, 0xb5f, 0xb3e, 0xb28, 0xb4d, 0x200c, 0xb32, 0xb30, 0xb4d, 0xb21,
0x20, 0xb39, 0xb4b, 0xb71, 0xb47, 0xb2e, 0xb47, 0xb32, 0xb2c, 0xb4b, 0xb30, 0xb4d,
0xb23, 0xb4d, 0xb23, 0xb2a, 0xb30, 0xb4d, 0xb25, 0xb4d, 0x200c, 0xb38, 0xb3f, 0xb21,
0xb28, 0xb40, 0xb06, 0xb2e, 0xb37, 0xb4d, 0xb1f, 0xb30, 0xb21, 0xb3e, 0xb2e, 0xb4d,
0xb06, 0xb28, 0xb21, 0xb4b, 0xb30, 0xb3e, 0xb06, 0xb37, 0xb4d, 0xb1f, 0xb4d, 0xb30,
0xb3e, 0xb16, 0xb3e, 0xb28, 0xb4d, 0xb0f, 0xb25, 0xb47, 0xb28, 0xb4d, 0xb38, 0xb2c,
0xb47, 0xb32, 0xb17, 0xb4d, 0xb30, 0xb47, 0xb21, 0xb4d, 0x200c, 0xb2c, 0xb30, 0xb4d,
0xb32, 0xb3f, 0xb28, 0xb4d, 0xb2c, 0xb4d, 0xb30, 0xb3e, 0xb1f, 0xb3f, 0xb38, 0xb32,
0xb3e, 0xb2d, 0xb3e, 0xb2c, 0xb4d, 0xb30, 0xb38, 0xb32, 0xb4d, 0xb38, 0xb4d, 0xb2c,
0xb41, 0xb1a, 0xb3e, 0xb30, 0xb47, 0xb37, 0xb4d, 0xb1f, 0xb4d, 0xb2c, 0xb41, 0xb21,
0xb3e, 0xb2a, 0xb47, 0xb37, 0xb4d, 0xb1f, 0xb4d, 0xb2c, 0xb41, 0xb38, 0xb3f, 0xb28,
0xb17, 0xb47, 0xb28, 0xb4d, 0xb1a, 0xb3f, 0xb38, 0xb3f, 0xb28, 0xb4c, 0xb15, 0xb4b,
0xb2a, 0xb47, 0xb28, 0xb39, 0xb3e, 0xb17, 0xb47, 0xb28, 0xb4d, 0xb21, 0xb2c, 0xb32,
0xb3f, 0xb28, 0xb4d, 0xb1c, 0xb3f, 0xb2c, 0xb4d, 0xb30, 0xb3e, 0xb32, 0xb1f, 0xb30,
0xb4d, 0x200c, 0xb17, 0xb41, 0xb0f, 0xb30, 0xb28, 0xb38, 0xb47, 0xb39, 0xb47, 0xb32,
0xb38, 0xb3f, 0xb28, 0xb15, 0xb3f, 0xb07, 0xb38, 0xb32, 0xb47, 0x20, 0xb05, 0xb2b,
0xb4d, 0x20, 0xb2e, 0xb47, 0xb28, 0xb4d, 0xb07, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28,
0xb2c, 0xb41, 0xb32, 0xb1c, 0xb30, 0xb4d, 0xb38, 0xb3f, 0xb15, 0xb3e, 0xb32, 0xb3f,
0xb28, 0xb3f, 0xb28, 0xb17, 0xb4d, 0xb30, 0xb3e, 0xb21, 0xb4d, 0xb15, 0xb3f, 0xb30,
0xb4b, 0xb2d, 0xb15, 0xb3f, 0xb0f, 0xb2d, 0xb4d, 0xb32, 0xb3f, 0xb38, 0xb2c, 0xb4b,
0xb28, 0xb4d, 0xb32, 0xb1c, 0xb41, 0xb2c, 0xb4d, 0xb32, 0xb1c, 0xb3e, 0xb28, 0xb4d,
0xb32, 0xb23, 0xb4d, 0xb21, 0xb28, 0xb4d, 0xb32, 0xb15, 0xb4d, 0xb38, 0xb2e, 0xb2c,
0xb30, 0xb4d, 0xb17, 0xb2e, 0xb3e, 0xb21, 0xb4d, 0xb30, 0xb3f, 0xb21, 0xb4d, 0xb2e,
0xb3e, 0xb32, 0xb1f, 0xb3e, 0xb2e, 0xb3e, 0xb30, 0xb3f, 0xb0f, 0xb39, 0xb3e, 0xb2e,
0xb28, 0xb4d, 0xb2e, 0xb3f, 0xb28, 0xb38, 0xb4d, 0xb15, 0xb2e, 0xb4b, 0xb28, 0xb3e,
0xb15, 0xb4b, 0xb2e, 0xb38, 0xb4d, 0xb15, 0xb4b, 0xb13, 0xb38, 0xb32, 0xb4b, 0xb2a,
0xb47, 0xb30, 0xb3f, 0xb38, 0xb4d, 0xb2a, 0xb4b, 0xb21, 0xb17, 0xb4b, 0xb30, 0xb3f,
0xb15, 0xb3e, 0xb2a, 0xb4d, 0xb30, 0xb3e, 0xb17, 0xb4d, 0xb30, 0xb3f, 0xb17, 0xb3e,
0xb30, 0xb4b, 0xb2e, 0xb4d, 0xb38, 0xb3e, 0xb2e, 0xb3e, 0xb30, 0xb3e, 0xb38, 0xb3e,
0xb28, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb30, 0xb3f, 0xb28, 0xb4b, 0xb38, 0xb3e, 0xb30,
0xb3e, 0xb1c, 0xb47, 0xb2d, 0xb4b, 0xb38, 0xb3e, 0xb30, 0xb3e, 0xb1f, 0xb4b, 0xb2d,
0xb4d, 0xb38, 0xb3f, 0xb2e, 0xb2b, 0xb47, 0xb30, 0xb4b, 0xb2a, 0xb4b, 0xb32, 0xb4d,
0xb38, 0xb4d, 0xb15, 0xb4b, 0xb2a, 0xb1c, 0xb47, 0xb38, 0xb4b, 0xb2b, 0xb3f, 0xb06,
0xb37, 0xb4d, 0xb1f, 0xb15, 0xb39, 0xb4b, 0xb2e, 0xb4d, 0x200c, 0xb1f, 0xb3e, 0xb32,
0xb3f, 0xb28, 0xb4d, 0xb28, 0xb1f, 0xb3f, 0xb30, 0xb3e, 0xb28, 0xb47, 0xb5f, 0xb41,
0xb32, 0xb5f, 0xb3e, 0xb28, 0xb4b, 0xb2d, 0xb38, 0xb4d, 0xb15, 0xb2d, 0xb3e, 0xb21,
0xb41, 0xb1c, 0xb2d, 0xb3e, 0xb1f, 0xb3f, 0xb15, 0xb3e, 0xb28, 0xb4d, 0xb2d, 0xb3f,
0xb0f, 0xb28, 0xb3e, 0xb2d, 0xb3f, 0xb32, 0xb28, 0xb3f, 0xb09, 0xb38, 0xb4d, 0xb2d,
0xb4b, 0xb32, 0xb17, 0xb4b, 0xb17, 0xb4d, 0xb30, 0xb3e, 0xb21, 0xb4d, 0xb71, 0xb3e,
0xb30, 0xb38, 0xb71, 0xb3e, 0xb1c, 0xb3e, 0xb17, 0xb4d, 0xb30, 0xb47, 0xb2c, 0xb4d,
0xb1c, 0xb41, 0xb30, 0xb3f, 0xb15, 0xb4d, 0xb06, 0xb23, 0xb4d, 0xb1f, 0xb3e, 0xb28,
0xb3e, 0xb28, 0xb3e, 0xb30, 0xb3f, 0xb2d, 0xb4b, 0xb1a, 0xb3e, 0xb17, 0xb4b, 0xb38,
0xb4d, 0x200c, 0xb16, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb1f, 0xb2e, 0xb3e, 0xb38,
0xb15, 0xb4b, 0xb15, 0xb4b, 0xb38, 0xb4d, 0x200c, 0xb15, 0xb4b, 0xb2e, 0xb4b, 0xb30,
0xb4b, 0xb15, 0xb47, 0xb30, 0xb41, 0xb17, 0xb47, 0xb32, 0xb47, 0xb28, 0xb2e, 0xb3e,
0xb39, 0xb47, 0xb2e, 0xb3e, 0xb33, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a, 0xb2e, 0xb4c,
0xb30, 0xb3f, 0xb38, 0xb38, 0xb4d, 0xb2e, 0xb3e, 0xb5f, 0xb4b, 0xb1f, 0xb47, 0xb30,
0xb3f, 0xb5f, 0xb41, 0xb28, 0xb3f, 0xb05, 0xb28, 0xb4d, 0x200c, 0xb06, 0xb2a, 0xb3f,
0xb06, 0xb05, 0xb15, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb2c, 0xb4c, 0xb17, 0xb47,
0xb28, 0xb4d, 0x200c, 0xb2d, 0xb3f, 0xb32, 0xb4d, 0xb32, 0xb47, 0xb1a, 0xb3e, 0xb25,
0xb3e, 0xb2e, 0xb4d, 0x200c, 0xb1a, 0xb42, 0xb15, 0xb4d, 0x200c, 0xb07, 0xb37, 0xb4d,
0xb1f, 0xb30, 0xb4d, 0x200c, 0xb07, 0xb2b, 0xb47, 0xb1f, 0xb4d, 0x200c, 0xb2b, 0xb3e,
0xb15, 0xb3e, 0xb13, 0xb2b, 0xb4b, 0xb2b, 0xb3f, 0xb1c, 0xb3f, 0xb2b, 0xb41, 0xb28,
0xb3e, 0xb2b, 0xb41, 0xb1f, 0xb3f, 0xb17, 0xb3e, 0xb32, 0xb3e, 0xb2a, 0xb3e, 0xb17,
0xb4b, 0xb38, 0xb17, 0xb3e, 0xb2e, 0xb4d, 0xb2c, 0xb3f, 0xb5f, 0xb30, 0xb4d, 0x200c,
0xb17, 0xb41, 0xb06, 0xb21, 0xb3e, 0xb32, 0xb15, 0xb3e, 0xb28, 0xb3e, 0xb32, 0xb39,
0xb4b, 0xb28, 0xb4b, 0xb32, 0xb41, 0xb32, 0xb41, 0xb0f, 0xb23, 0xb4d, 0xb21, 0xb47,
0xb30, 0xb2c, 0xb41, 0xb30, 0xb3f, 0xb15, 0xb3f, 0xb30, 0xb3f, 0xb24, 0xb3f, 0xb2e,
0xb3e, 0xb1f, 0xb3f, 0xb15, 0xb4b, 0xb38, 0xb30, 0xb47, 0xb07, 0xb15, 0xb4d, 0xb71,
0xb3e, 0xb1c, 0xb3e, 0xb32, 0xb47, 0xb07, 0xb28, 0xb4d, 0xb2e, 0xb3e, 0xb1c, 0xb41,
0xb30, 0xb4b, 0xb2e, 0xb3e, 0xb30, 0xb4d, 0xb15, 0xb4d, 0xb71, 0xb47, 0xb38, 0xb3e,
0xb38, 0xb4d, 0x200c, 0xb2e, 0xb3f, 0xb21, 0xb4d, 0x200c, 0xb71, 0xb47, 0xb28, 0xb3e,
0xb09, 0xb30, 0xb41, 0xb28, 0xb3f, 0xb5f, 0xb42, 0xb28, 0xb30, 0xb2b, 0xb4b, 0xb15,
0xb4d, 0x200c, 0xb28, 0xb4c, 0xb2e, 0xb3f, 0xb5f, 0xb2a, 0xb3e, 0xb17, 0xb4b, 0x20,
0xb2a, 0xb3e, 0xb17, 0xb4b, 0xb2a, 0xb3e, 0xb32, 0xb3e, 0xb09, 0xb2a, 0xb3f, 0xb1f,
0xb15, 0xb3e, 0xb30, 0xb3f, 0xb28, 0xb4d, 0x200c, 0xb2a, 0xb4b, 0xb39, 0xb28, 0xb2a,
0xb47, 0xb07, 0xb2a, 0xb4b, 0xb30, 0xb4d, 0xb1f, 0xb4d, 0x200c, 0x20, 0xb2e, 0xb4b,
0xb30, 0xb47, 0xb38, 0xb2c, 0xb3f, 0xb30, 0xb3e, 0xb30, 0xb4b, 0xb1f, 0xb4b, 0xb19,
0xb4d, 0xb17, 0xb3e, 0xb38, 0xb3e, 0xb07, 0xb2a, 0xb3e, 0xb28, 0xb4d, 0xb24, 0xb3e,
0xb39, 0xb3f, 0xb24, 0xb3f, 0xb24, 0xb3e, 0xb30, 0xb71, 0xb3e, 0xb1f, 0xb4b, 0xb19,
0xb4d, 0xb17, 0xb3e, 0xb1f, 0xb3e, 0xb2a, 0xb41, 0xb71, 0xb47, 0xb15, 0xb4d, 0xb71,
0xb3e, 0xb32, 0xb3f, 0xb38, 0xb4d, 0x200c, 0x41c, 0x4d5, 0x441, 0x43a, 0x443, 0x44b,
0x4b, 0xf2, 0x72, 0x73, 0x6f, 0x75, 0x52, 0x65, 0x70, 0xfa, 0x62, 0x6c,
0x6b, 0x61, 0x20, 0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x69, 0x63, 0x61, 0x6e,
0x61, 0x48, 0x65, 0x72, 0xfa, 0x73, 0x61, 0x6c, 0xe8, 0x6d, 0x41, 0x7a,
0xf3, 0x72, 0x65, 0x73, 0x4b, 0x61, 0x62, 0x6f, 0x20, 0x56, 0x65, 0x72,
0x64, 0x65, 0x41, 0x74, 0xe9, 0x6e, 0x61, 0x73, 0x4c, 0xf2, 0x6e, 0x64,
0x65, 0x6e, 0x627, 0x628, 0x64a, 0x62c, 0x627, 0x646, 0x627, 0x6a9, 0x631, 0x627,
0x627, 0x62f, 0x64a, 0x633, 0x20, 0x627, 0x628, 0x627, 0x628, 0x627, 0x627, 0x644,
0x62c, 0x64a, 0x64a, 0x631, 0x632, 0x627, 0x633, 0x645, 0x627, 0x631, 0x647, 0x628,
0x627, 0x645, 0x6cc, 0x6a9, 0x648, 0x628, 0x627, 0x646, 0x6ab, 0x648, 0x64a, 0x628,
0x633, 0x627, 0x624, 0x628, 0x644, 0x646, 0x67c, 0x627, 0x64a, 0x631, 0x628, 0x631,
0x627, 0x632, 0x627, 0x648, 0x64a, 0x644, 0x628, 0x62c, 0x648, 0x6ab, 0x648, 0x631,
0x627, 0x642, 0x627, 0x647, 0x631, 0x647, 0x6a9, 0x627, 0x633, 0x627, 0x628, 0x644,
0x627, 0x646, 0x6a9, 0x627, 0x633, 0x64a, 0x648, 0x67c, 0x627, 0x6a9, 0x648, 0x646,
0x6a9, 0x631, 0x64a, 0x689, 0x627, 0x6a9, 0x627, 0x631, 0x62c, 0x628, 0x648, 0x62a,
0x64a, 0x627, 0x644, 0x6cc, 0x648, 0x646, 0x641, 0x631, 0x6cc, 0x67c, 0x648, 0x646,
0x6ab, 0x627, 0x628, 0x631, 0x648, 0x646, 0x647, 0x631, 0x627, 0x631, 0x6d0, 0x62c,
0x648, 0x647, 0x627, 0x646, 0x633, 0x628, 0x631, 0x6ab, 0x6a9, 0x645, 0x67e, 0x627,
0x644, 0x647, 0x62e, 0x631, 0x62a, 0x648, 0x645, 0x6a9, 0x64a, 0x6af, 0x627, 0x644,
0x64a, 0x6a9, 0x64a, 0x646, 0x634, 0x627, 0x633, 0x627, 0x644, 0x6cc, 0x628, 0x631,
0x6cc, 0x644, 0x644, 0x648, 0x648, 0x645, 0x644, 0x648, 0x646, 0x62f, 0x647, 0x644,
0x628, 0x648, 0x628, 0x627, 0x634, 0x64a, 0x645, 0x627, 0x67e, 0x648, 0x67c, 0x648,
0x645, 0x633, 0x64a, 0x631, 0x648, 0x645, 0x628, 0x627, 0x628, 0x627, 0x646, 0x6d0,
0x645, 0x648, 0x6af, 0x62f, 0x64a, 0x634, 0x648, 0x645, 0x648, 0x646, 0x631, 0x648,
0x641, 0x6cc, 0x627, 0x646, 0x62c, 0x627, 0x645, 0x6cc, 0x646, 0x627, 0x646, 0x6cc,
0x645, 0x64a, 0x646, 0x648, 0x6a9, 0x648, 0x686, 0x67c, 0x627, 0x648, 0x6ab, 0x648,
0x689, 0x648, 0x6ab, 0x648, 0x67e, 0x648, 0x631, 0x67c, 0x648, 0x20, 0x2d, 0x20,
0x646, 0x648, 0x648, 0x633, 0x627, 0x648, 0x20, 0x67c, 0x648, 0x645, 0x648, 0x6cc,
0x646, 0x647, 0x648, 0x6a9, 0x627, 0x62f, 0x627, 0x6a9, 0x627, 0x6cc, 0x646, 0x6a9,
0x631, 0x6cc, 0x62c, 0x627, 0x646, 0x6ab, 0x6cc, 0x644, 0x627, 0x627, 0x646, 0x67c,
0x64a, 0x6ab, 0x627, 0x631, 0x6ab, 0x6cc, 0x646, 0x6cc, 0x627, 0x628, 0x64a, 0x646,
0x648, 0x633, 0x20, 0x627, 0x64a, 0x64a, 0x631, 0x632, 0x6a9, 0x67c, 0x627, 0x645,
0x627, 0x631, 0x6a9, 0x627, 0x6a9, 0x648, 0x631, 0x689, 0x648, 0x628, 0x627, 0x62c,
0x648, 0x62c, 0x648, 0x64a, 0x644, 0x627, 0x631, 0x64a, 0x648, 0x62c, 0x627, 0x645,
0x6cc, 0x646, 0x689, 0x648, 0x632, 0x627, 0x631, 0x64a, 0x648, 0x20, 0x6ab, 0x64a,
0x644, 0x64a, 0x6ab, 0x648, 0x633, 0x633, 0x627, 0x644, 0x67c, 0x627, 0x633, 0x627,
0x646, 0x20, 0x62c, 0x648, 0x627, 0x646, 0x633, 0x627, 0x646, 0x20, 0x644, 0x648,
0x6cc, 0x633, 0x67c, 0x64a, 0x6a9, 0x648, 0x648, 0x645, 0x627, 0x648, 0x634, 0x648,
0x627, 0x6cc, 0x627, 0x622, 0x631, 0x648, 0x628, 0x627, 0x627, 0x633, 0x648, 0x646,
0x633, 0x64a, 0x648, 0x646, 0x627, 0x62a, 0x6cc, 0x6a9, 0x648, 0x6a9, 0x646, 0x628,
0x647, 0x6cc, 0x627, 0x628, 0x647, 0x6cc, 0x627, 0x20, 0x628, 0x6cc, 0x646, 0x689,
0x631, 0x627, 0x633, 0x628, 0x644, 0x6cc, 0x632, 0x628, 0x644, 0x627, 0x646, 0x6a9,
0x2d, 0x633, 0x627, 0x628, 0x644, 0x648, 0x646, 0x628, 0x648, 0x627, 0x20, 0x648,
0x6cc, 0x633, 0x67c, 0x627, 0x628, 0x648, 0x6ab, 0x67c, 0x627, 0x628, 0x648, 0x632,
0x6a9, 0x6cc, 0x645, 0x628, 0x631, 0x62c, 0x20, 0x628, 0x64a, 0x6a9, 0x645, 0x67e,
0x648, 0x20, 0x6ab, 0x631, 0x689, 0x6cc, 0x6a9, 0x64a, 0x646, 0x6a9, 0x646, 0x6a9,
0x627, 0x631, 0x627, 0x6a9, 0x627, 0x633, 0x6a9, 0x6cc, 0x6cc, 0x646, 0x6a9, 0x6cc,
0x645, 0x646, 0x634, 0x6cc, 0x6a9, 0x627, 0x6ab, 0x648, 0x686, 0x6be, 0x648, 0x627,
0x6be, 0x648, 0x627, 0x633, 0x6cc, 0x648, 0x62f, 0x627, 0x62f, 0x20, 0x62c, 0x6cc,
0x648, 0x631, 0x6cc, 0x632, 0x6a9, 0x648, 0x633, 0x62a, 0x627, 0x631, 0x6cc, 0x6a9,
0x627, 0x6a9, 0x631, 0x633, 0x67c, 0x648, 0x646, 0x6a9, 0x648, 0x6cc, 0x627, 0x628,
0x627, 0x6a9, 0x648, 0x631, 0x627, 0x6a9, 0x627, 0x624, 0x689, 0x646, 0x645, 0x627,
0x631, 0x6a9, 0x634, 0x627, 0x646, 0x62f, 0x627, 0x633, 0x646, 0x20, 0x6a9, 0x631,
0x6cc, 0x6a9, 0x689, 0x646, 0x648, 0x631, 0x689, 0x627, 0x6cc, 0x67c, 0x631, 0x648,
0x67c, 0x62f, 0x648, 0x645, 0x6cc, 0x646, 0x6cc, 0x6a9, 0x627, 0x627, 0x6cc, 0x689,
0x645, 0x648, 0x646, 0x67c, 0x646, 0x627, 0x6cc, 0x6cc, 0x631, 0x648, 0x646, 0x67e,
0x627, 0x64a, 0x644, 0x20, 0x633, 0x644, 0x648, 0x627, 0x62f, 0x648, 0x631, 0x641,
0x648, 0x631, 0x67c, 0x20, 0x646, 0x6cc, 0x644, 0x633, 0x646, 0x641, 0x648, 0x631,
0x62a, 0x6cc, 0x644, 0x632, 0x627, 0x6ab, 0x6cc, 0x633, 0x633, 0x20, 0x628, 0x64a,
0x6cc, 0x6af, 0x648, 0x632, 0x20, 0x628, 0x64a, 0x644, 0x648, 0x6cc, 0x20, 0x62a,
0x631, 0x6a9, 0x6ab, 0x631, 0x646, 0x627, 0x62f, 0x627, 0x6ab, 0x627, 0x644, 0x689,
0x6cc, 0x67e, 0x6ab, 0x648, 0x627, 0x62a, 0x645, 0x627, 0x644, 0x627, 0x6ab, 0x648,
0x64a, 0x627, 0x6a9, 0x644, 0x6ab, 0x6cc, 0x627, 0x646, 0x627, 0x647, 0x6cc, 0x644,
0x641, 0x6a9, 0x633, 0x647, 0x648, 0x627, 0x646, 0x627, 0x647, 0x631, 0x645, 0x648,
0x633, 0x6cc, 0x644, 0x648, 0x627, 0x646, 0x689, 0x64a, 0x627, 0x646, 0x627, 0x20,
0x67e, 0x648, 0x644, 0x64a, 0x633, 0x646, 0x648, 0x6a9, 0x633, 0x20, 0x627, 0x646,
0x689, 0x64a, 0x627, 0x646, 0x627, 0x645, 0x648, 0x631, 0x64a, 0x646, 0x6ab, 0x648,
0x20, 0x627, 0x646, 0x689, 0x64a, 0x627, 0x646, 0x627, 0x67e, 0x64a, 0x67c, 0x633,
0x628, 0x631, 0x6ab, 0x60c, 0x20, 0x627, 0x646, 0x689, 0x64a, 0x627, 0x646, 0x627,
0x67c, 0x644, 0x20, 0x633, 0x67c, 0x64a, 0x60c, 0x20, 0x627, 0x646, 0x689, 0x64a,
0x627, 0x646, 0x627, 0x648, 0x6cc, 0x648, 0x6cc, 0x60c, 0x20, 0x627, 0x646, 0x689,
0x64a, 0x627, 0x646, 0x627, 0x648, 0x64a, 0x646, 0x633, 0x64a, 0x646, 0x633, 0x60c,
0x20, 0x627, 0x646, 0x689, 0x64a, 0x627, 0x646, 0x627, 0x648, 0x64a, 0x646, 0x627,
0x645, 0x6a9, 0x20, 0x627, 0x646, 0x689, 0x64a, 0x627, 0x646, 0x627, 0x627, 0x646,
0x648, 0x6a9, 0x627, 0x642, 0x644, 0x6cc, 0x67c, 0x62c, 0x645, 0x627, 0x6cc, 0x6a9,
0x647, 0x62c, 0x648, 0x646, 0x648, 0x644, 0x648, 0x626, 0x633, 0x20, 0x648, 0x6cc,
0x644, 0x645, 0x648, 0x646, 0x67c, 0x64a, 0x633, 0x64a, 0x644, 0x648, 0x60c, 0x20,
0x6a9, 0x648, 0x646, 0x67c, 0x6a9, 0x6cc, 0x6a9, 0x644, 0x6cc, 0x646, 0x689, 0x6cc,
0x632, 0x62c, 0x644, 0x6cc, 0x645, 0x627, 0x644, 0x627, 0x633, 0x20, 0x627, 0x6cc,
0x646, 0x62c, 0x644, 0x633, 0x644, 0x648, 0x648, 0x631, 0x20, 0x67e, 0x631, 0x646,
0x633, 0x20, 0x6a9, 0x648, 0x627, 0x631, 0x67c, 0x631, 0x645, 0x627, 0x633, 0x64a,
0x648, 0x645, 0x646, 0x6ab, 0x648, 0x627, 0x645, 0x646, 0x627, 0x648, 0x633, 0x645,
0x64a, 0x631, 0x64a, 0x6ab, 0x627, 0x67c, 0x645, 0x627, 0x631, 0x67c, 0x6cc, 0x646,
0x6cc, 0x6a9, 0x645, 0x64a, 0x67c, 0x627, 0x645, 0x648, 0x631, 0x648, 0x633, 0x645,
0x632, 0x627, 0x62a, 0x644, 0x627, 0x646, 0x645, 0x6cc, 0x646, 0x648, 0x645, 0x6cc,
0x646, 0x645, 0x64a, 0x631, 0x64a, 0x689, 0x627, 0x645, 0x6cc, 0x62a, 0x644, 0x627,
0x6a9, 0x627, 0x67c, 0x644, 0x627, 0x645, 0x6a9, 0x633, 0x64a, 0x6a9, 0x648, 0x20,
0x69a, 0x627, 0x631, 0x645, 0x64a, 0x6a9, 0x648, 0x64a, 0x644, 0x627, 0x646, 0x645,
0x648, 0x646, 0x6a9, 0x67c, 0x648, 0x646, 0x645, 0x646, 0x67c, 0x631, 0x631, 0x64a,
0x645, 0x648, 0x646, 0x67c, 0x20, 0x648, 0x689, 0x6cc, 0x648, 0x645, 0x627, 0x646,
0x67c, 0x6cc, 0x633, 0x6cc, 0x631, 0x62a, 0x646, 0x6cc, 0x633, 0x627, 0x648, 0x646,
0x6cc, 0x648, 0x6cc, 0x627, 0x631, 0x6a9, 0x646, 0x648, 0x631, 0x648, 0x646, 0x647,
0x627, 0x628, 0x64a, 0x648, 0x644, 0x627, 0x60c, 0x20, 0x634, 0x645, 0x627, 0x644,
0x64a, 0x20, 0x689, 0x627, 0x6a9, 0x648, 0x67c, 0x627, 0x645, 0x631, 0x6a9, 0x632,
0x60c, 0x20, 0x634, 0x645, 0x627, 0x644, 0x64a, 0x20, 0x689, 0x627, 0x6a9, 0x648,
0x67c, 0x627, 0x646, 0x648, 0x6cc, 0x20, 0x633, 0x644, 0x6cc, 0x645, 0x60c, 0x20,
0x634, 0x645, 0x627, 0x644, 0x64a, 0x20, 0x62f, 0x627, 0x6a9, 0x648, 0x62a, 0x627,
0x646, 0x648, 0x648, 0x6a9, 0x627, 0x648, 0x62c, 0x646, 0x6ab, 0x627, 0x67e, 0x627,
0x631, 0x627, 0x645, 0x627, 0x631, 0x628, 0x648, 0x641, 0x6cc, 0x646, 0x6a9, 0x633,
0x67e, 0x648, 0x631, 0x67c, 0x20, 0x627, 0x6cc, 0x648, 0x20, 0x2d, 0x20, 0x67e,
0x631, 0x646, 0x633, 0x62f, 0x20, 0x627, 0x633, 0x67e, 0x627, 0x646, 0x6cc, 0x627,
0x20, 0x628, 0x646, 0x62f, 0x631, 0x67e, 0x648, 0x631, 0x67c, 0x648, 0x20, 0x648,
0x6cc, 0x644, 0x647, 0x648, 0x67e, 0x648, 0x631, 0x62a, 0x648, 0x20, 0x631, 0x6cc,
0x6a9, 0x648, 0x67e, 0x646, 0x67c, 0x627, 0x20, 0x622, 0x631, 0x6cc, 0x646, 0x627,
0x633, 0x631, 0x64a, 0x646, 0x6a9, 0x646, 0x20, 0x627, 0x646, 0x644, 0x67c, 0x631,
0x6cc, 0x633, 0x6cc, 0x641, 0x64a, 0x631, 0x6cc, 0x6ab, 0x6cc, 0x646, 0x627, 0x631,
0x64a, 0x633, 0x627, 0x644, 0x648, 0x67c, 0x631, 0x6cc, 0x648, 0x20, 0x628, 0x631,
0x627, 0x646, 0x6a9, 0x648, 0x633, 0x646, 0x627, 0x62a, 0x631, 0x645, 0x633, 0x646,
0x62a, 0x64a, 0x627, 0x6ab, 0x648, 0x633, 0x646, 0x62a, 0x648, 0x20, 0x689, 0x648,
0x645, 0x6cc, 0x646, 0x6ab, 0x648, 0x633, 0x627, 0x648, 0x20, 0x67e, 0x627, 0x648,
0x644, 0x648, 0x627, 0x67c, 0x648, 0x6a9, 0x648, 0x67c, 0x648, 0x631, 0x645, 0x6cc,
0x67c, 0x633, 0x6cc, 0x67c, 0x6a9, 0x627, 0x633, 0x6cc, 0x646, 0x67c, 0x20, 0x628,
0x627, 0x631, 0x67c, 0x6cc, 0x644, 0x6cc, 0x645, 0x633, 0x6cc, 0x646, 0x67c, 0x20,
0x62c, 0x627, 0x646, 0x632, 0x633, 0x6cc, 0x646, 0x67c, 0x20, 0x6a9, 0x67c, 0x633,
0x633, 0x6cc, 0x646, 0x67c, 0x20, 0x644, 0x648, 0x633, 0x6cc, 0x627, 0x633, 0x64a,
0x646, 0x67c, 0x20, 0x62a, 0x647, 0x627, 0x645, 0x633, 0x633, 0x6d0, 0x646, 0x67c,
0x20, 0x648, 0x6cc, 0x633, 0x646, 0x67c, 0x633, 0x648, 0x64a, 0x641, 0x67c, 0x20,
0x6a9, 0x631, 0x646, 0x67c, 0x67c, 0x6ab, 0x633, 0x6cc, 0x6ab, 0x627, 0x644, 0x67e,
0x627, 0x62a, 0x62c, 0x648, 0x627, 0x646, 0x627, 0x67c, 0x648, 0x631, 0x646, 0x67c,
0x648, 0x67c, 0x648, 0x631, 0x62a, 0x648, 0x644, 0x627, 0x648, 0x6cc, 0x646, 0x6a9,
0x648, 0x648, 0x631, 0x648, 0x627, 0x64a, 0x67c, 0x20, 0x647, 0x627, 0x631, 0x633,
0x648, 0x6cc, 0x646, 0x67e, 0x6cc, 0x6ab, 0x64a, 0x627, 0x6a9, 0x648, 0x67c, 0x67c,
0x6a9, 0x6cc, 0x633, 0x64a, 0x689, 0x64a, 0x648, 0x633, 0x689, 0x648, 0x645, 0x648,
0x646, 0x67c, 0x20, 0x689, 0x64a, 0x20, 0x627, 0x631, 0x648, 0x644, 0x645, 0x6a9,
0x648, 0x627, 0x631, 0x64a, 0x645, 0x627, 0x648, 0x633, 0x646, 0x645, 0x6a9, 0x645,
0x631, 0x689, 0x648, 0x67e, 0x627, 0x644, 0x645, 0x631, 0x631, 0x648, 0x62f, 0x631,
0x627, 0x633, 0x6cc, 0x648, 0x627, 0x67c, 0x631, 0x648, 0x644, 0x648, 0x627, 0x633,
0x62a, 0x648, 0x6a9, 0x644, 0x627, 0x646, 0x6ab, 0x64a, 0x631, 0x628, 0x64a, 0x646,
0x627, 0x689, 0x646, 0x627, 0x644, 0x645, 0x627, 0x62a, 0x6cc, 0x627, 0x646, 0x627,
0x689, 0x64a, 0x631, 0x627, 0x6a9, 0x67c, 0x627, 0x648, 0x627, 0x6a9, 0x62a, 0x648,
0x628, 0x627, 0x634, 0x63a, 0x20, 0x622, 0x628, 0x627, 0x62f, 0x627, 0x6d0, 0x67c,
0x631, 0x627, 0x648, 0x628, 0x62d, 0x631, 0x6cc, 0x646, 0x628, 0x646, 0x6a9, 0x627,
0x6a9, 0x628, 0x6cc, 0x631, 0x648, 0x62a, 0x628, 0x634, 0x6a9, 0x6cc, 0x6a9, 0x628,
0x631, 0x648, 0x6cc, 0x646, 0x64a, 0x6a9, 0x648, 0x644, 0x645, 0x628, 0x648, 0x689,
0x647, 0x627, 0x6a9, 0x647, 0x62f, 0x644, 0x64a, 0x62f, 0x648, 0x628, 0x6cc, 0x62f,
0x648, 0x634, 0x646, 0x628, 0x64a, 0x641, 0x627, 0x645, 0x627, 0x6ab, 0x633, 0x62a,
0x627, 0x63a, 0x632, 0x647, 0x647, 0x628, 0x631, 0x648, 0x646, 0x647, 0x648, 0x20,
0x686, 0x64a, 0x20, 0x645, 0x646, 0x20, 0x69a, 0x627, 0x631, 0x647, 0x627, 0x646,
0x6ab, 0x20, 0x6a9, 0x627, 0x646, 0x6ab, 0x647, 0x627, 0x648, 0x689, 0x627, 0x631,
0x6a9, 0x648, 0x67c, 0x633, 0x6a9, 0x62c, 0x627, 0x64a, 0x627, 0x67e, 0x648, 0x631,
0x627, 0x64a, 0x631, 0x648, 0x634, 0x644, 0x645, 0x6a9, 0x627, 0x628, 0x644, 0x6a9,
0x627, 0x645, 0x686, 0x627, 0x62a, 0x6a9, 0x627, 0x6a9, 0x631, 0x627, 0x686, 0x64a,
0x6a9, 0x67c, 0x645, 0x646, 0x689, 0x648, 0x62e, 0x646, 0x62f, 0x64a, 0x6ab, 0x627,
0x6a9, 0x648, 0x644, 0x6a9, 0x62a, 0x647, 0x6a9, 0x631, 0x64a, 0x633, 0x646, 0x648,
0x64a, 0x627, 0x631, 0x633, 0x6a9, 0x6a9, 0x648, 0x644, 0x627, 0x644, 0x645, 0x67e,
0x648, 0x631, 0x6a9, 0x648, 0x686, 0x646, 0x6ab, 0x6a9, 0x648, 0x6d0, 0x62a, 0x645,
0x6a9, 0x627, 0x648, 0x645, 0x6ab, 0x62f, 0x627, 0x646, 0x645, 0x6a9, 0x627, 0x633,
0x627, 0x631, 0x645, 0x646, 0x6cc, 0x644, 0x627, 0x646, 0x6cc, 0x6a9, 0x648, 0x633,
0x6cc, 0x627, 0x646, 0x648, 0x648, 0x6a9, 0x648, 0x632, 0x646, 0x6cc, 0x67c, 0x6a9,
0x646, 0x648, 0x648, 0x633, 0x64a, 0x628, 0x631, 0x633, 0x6a9, 0x627, 0x648, 0x631,
0x644, 0x67e, 0x646, 0x648, 0x645, 0x20, 0x67e, 0x646, 0x67e, 0x648, 0x646, 0x67c,
0x6cc, 0x646, 0x6cc, 0x6a9, 0x67e, 0x64a, 0x627, 0x646, 0x6ab, 0x20, 0x64a, 0x627,
0x646, 0x6ab, 0x6a9, 0x648, 0x633, 0x62a, 0x627, 0x646, 0x6d0, 0x642, 0x64a, 0x632,
0x64a, 0x20, 0x644, 0x648, 0x631, 0x689, 0x627, 0x633, 0x62e, 0x627, 0x644, 0x64a,
0x646, 0x634, 0x646, 0x6af, 0x6be, 0x627, 0x626, 0x6cc, 0x633, 0x6cc, 0x646, 0x6ab,
0x627, 0x67e, 0x648, 0x631, 0x633, 0x631, 0x64a, 0x689, 0x646, 0x64a, 0x6a9, 0x648,
0x644, 0x64a, 0x645, 0x633, 0x6a9, 0x62a, 0x627, 0x64a, 0x67e, 0x64a, 0x62a, 0x627,
0x634, 0x642, 0x646, 0x62f, 0x62a, 0x628, 0x644, 0x6cc, 0x633, 0x64a, 0x62a, 0x647,
0x631, 0x627, 0x646, 0x62a, 0x647, 0x64a, 0x645, 0x641, 0x648, 0x67c, 0x648, 0x6a9,
0x6cc, 0x648, 0x627, 0x648, 0x644, 0x627, 0x646, 0x20, 0x628, 0x627, 0x67c, 0x631,
0x627, 0x648, 0x631, 0x648, 0x645, 0x642, 0x64a, 0x627, 0x648, 0x633, 0x62a, 0x646,
0x64a, 0x631, 0x627, 0x648, 0x64a, 0x646, 0x67c, 0x64a, 0x646, 0x648, 0x644, 0x627,
0x62f, 0x6cc, 0x648, 0x633, 0x62a, 0x627, 0x6a9, 0x64a, 0x627, 0x6a9, 0x648, 0x67c,
0x633, 0x6a9, 0x6cc, 0x627, 0x646, 0x6ab, 0x648, 0x646, 0x64a, 0x6a9, 0x627, 0x67c,
0x64a, 0x631, 0x646, 0x628, 0x631, 0x6ab, 0x64a, 0x631, 0x64a, 0x648, 0x627, 0x646,
0x627, 0x64a, 0x632, 0x648, 0x631, 0x633, 0x6a9, 0x646, 0x627, 0x631, 0x64a, 0x6a9,
0x64a, 0x67e, 0x20, 0x648, 0x631, 0x689, 0x645, 0x62f, 0x64a, 0x631, 0x627, 0x631,
0x64a, 0x6a9, 0x62c, 0x627, 0x648, 0x64a, 0x6a9, 0x633, 0x648, 0x64a, 0x644, 0x64a,
0x20, 0x62c, 0x648, 0x631, 0x62c, 0x64a, 0x627, 0x633, 0x6cc, 0x646, 0x67c, 0x20,
0x647, 0x6cc, 0x644, 0x6cc, 0x646, 0x627, 0x633, 0x67c, 0x646, 0x644, 0x64a, 0x627,
0x689, 0x6cc, 0x644, 0x627, 0x6cc, 0x689, 0x628, 0x631, 0x6cc, 0x633, 0x628, 0x646,
0x628, 0x631, 0x648, 0x6a9, 0x646, 0x20, 0x647, 0x644, 0x689, 0x627, 0x631, 0x648,
0x646, 0x627, 0x64a, 0x648, 0x6a9, 0x644, 0x627, 0x647, 0x648, 0x628, 0x627, 0x631,
0x67c, 0x644, 0x6cc, 0x646, 0x689, 0x627, 0x645, 0x6cc, 0x646, 0x644, 0x627, 0x631,
0x689, 0x20, 0x647, 0x648, 0x64a, 0x645, 0x6cc, 0x644, 0x628, 0x648, 0x631, 0x646,
0x67e, 0x631, 0x62a, 0x633, 0x689, 0x646, 0x64a, 0x627, 0x645, 0x633, 0x62a, 0x631,
0x62f, 0x627, 0x645, 0x627, 0x646, 0x62f, 0x648, 0x631, 0x627, 0x627, 0x633, 0x62a,
0x631, 0x627, 0x20, 0x62e, 0x627, 0x646, 0x627, 0x64a, 0x62a, 0x646, 0x632, 0x628,
0x644, 0x63a, 0x627, 0x62f, 0x628, 0x631, 0x644, 0x646, 0x628, 0x631, 0x627, 0x67c,
0x633, 0x644, 0x627, 0x648, 0x627, 0x628, 0x631, 0x648, 0x633, 0x644, 0x632, 0x628,
0x62e, 0x627, 0x631, 0x633, 0x62a, 0x628, 0x62f, 0x627, 0x67e, 0x633, 0x67c, 0x628,
0x648, 0x633, 0x64a, 0x646, 0x62c, 0x646, 0x686, 0x6cc, 0x633, 0x6cc, 0x646, 0x648,
0x6a9, 0x648, 0x67e, 0x646, 0x20, 0x647, 0x64a, 0x6ab, 0x646, 0x689, 0x628, 0x644,
0x646, 0x62c, 0x628, 0x644, 0x20, 0x627, 0x644, 0x637, 0x627, 0x631, 0x642, 0x6ab,
0x631, 0x646, 0x633, 0x64a, 0x647, 0x6cc, 0x644, 0x633, 0x646, 0x6a9, 0x6cc, 0x62f,
0x20, 0x622, 0x626, 0x644, 0x20, 0x622, 0x641, 0x20, 0x645, 0x6cc, 0x646, 0x627,
0x633, 0x62a, 0x646, 0x628, 0x648, 0x644, 0x6a9, 0x64a, 0x644, 0x646, 0x64a, 0x646,
0x6ab, 0x631, 0x627, 0x689, 0x6a9, 0x6cc, 0x631, 0x648, 0x641, 0x6a9, 0x6cc, 0x641,
0x644, 0x6cc, 0x633, 0x628, 0x648, 0x646, 0x644, 0x648, 0x628, 0x62c, 0x627, 0x646,
0x627, 0x644, 0x648, 0x6ab, 0x632, 0x627, 0x645, 0x628, 0x648, 0x631, 0x6ab, 0x645,
0x6cc, 0x689, 0x631, 0x6cc, 0x689, 0x645, 0x627, 0x644, 0x67c, 0x627, 0x645, 0x64a,
0x631, 0x64a, 0x647, 0x627, 0x645, 0x645, 0x646, 0x633, 0x6a9, 0x67e, 0x627, 0x631,
0x6cc, 0x633, 0x67e, 0x648, 0x689, 0x6ab, 0x648, 0x631, 0x64a, 0x6a9, 0x627, 0x67e,
0x631, 0x627, 0x6ab, 0x633, 0x627, 0x646, 0x20, 0x645, 0x627, 0x631, 0x6cc, 0x646,
0x648, 0x633, 0x64a, 0x631, 0x627, 0x62c, 0x64a, 0x648, 0x627, 0x633, 0x631, 0x627,
0x62a, 0x641, 0x633, 0x6cc, 0x645, 0x641, 0x631, 0x648, 0x67e, 0x648, 0x644, 0x633,
0x6a9, 0x67e, 0x648, 0x6ab, 0x635, 0x648, 0x641, 0x6cc, 0x627, 0x627, 0x633, 0x62a,
0x648, 0x644, 0x648, 0x645, 0x627, 0x644, 0x64a, 0x627, 0x646, 0x648, 0x633, 0x6a9,
0x648, 0x627, 0x689, 0x648, 0x632, 0x648, 0x6cc, 0x67c, 0x6cc, 0x6a9, 0x627, 0x646,
0x648, 0x6cc, 0x627, 0x646, 0x627, 0x648, 0x64a, 0x644, 0x646, 0x64a, 0x648, 0x633,
0x648, 0x627, 0x644, 0x6ab, 0x648, 0x6ab, 0x631, 0x627, 0x689, 0x632, 0x6ab, 0x631,
0x628, 0x632, 0x631, 0x6cc, 0x686, 0x627, 0x646, 0x62a, 0x627, 0x646, 0x646, 0x627,
0x631, 0x64a, 0x648, 0x648, 0x686, 0x627, 0x6ab, 0x648, 0x633, 0x6a9, 0x631, 0x64a,
0x633, 0x645, 0x633, 0x6a9, 0x648, 0x6a9, 0x648, 0x632, 0x6a9, 0x631, 0x6ab, 0x648,
0x644, 0x64a, 0x646, 0x645, 0x627, 0x644, 0x62f, 0x64a, 0x67e, 0x645, 0x627, 0x631,
0x64a, 0x634, 0x64a, 0x633, 0x645, 0x6cc, 0x67c, 0x648, 0x62a, 0x631, 0x64a, 0x20,
0x64a, 0x648, 0x646, 0x64a, 0x646, 0x627, 0x67e, 0x6cc, 0x627, 0x627, 0x6a9, 0x644,
0x646, 0x62f, 0x628, 0x648, 0x6ab, 0x646, 0x20, 0x648, 0x6cc, 0x644, 0x686, 0x627,
0x62a, 0x627, 0x645, 0x686, 0x648, 0x6a9, 0x627, 0x6cc, 0x633, 0x67c, 0x631, 0x639,
0x641, 0x627, 0x62a, 0x641, 0x648, 0x6a9, 0x627, 0x641, 0x648, 0x641, 0x62c, 0x64a,
0x6ab, 0x627, 0x644, 0x67e, 0x6ab, 0x648, 0x633, 0x6ab, 0x64a, 0x645, 0x628, 0x64a,
0x631, 0x6ab, 0x648, 0x627, 0x689, 0x644, 0x6a9, 0x64a, 0x646, 0x627, 0x644, 0x647,
0x6cc, 0x646, 0x648, 0x644, 0x648, 0x644, 0x648, 0x627, 0x646, 0x689, 0x631, 0x628,
0x631, 0x64a, 0x6a9, 0x64a, 0x631, 0x64a, 0x67c, 0x645, 0x627, 0x67c, 0x64a, 0x6a9,
0x648, 0x633, 0x631, 0x627, 0x64a, 0x6a9, 0x648, 0x627, 0x62c, 0x644, 0x64a, 0x646,
0x645, 0x62c, 0x648, 0x631, 0x648, 0x645, 0x627, 0x631, 0x6a9, 0x64a, 0x633, 0x627,
0x633, 0x645, 0x64a, 0x689, 0x648, 0x6cc, 0x646, 0x627, 0x6cc, 0x631, 0x648, 0x646,
0x6cc, 0x648, 0x648, 0x646, 0x648, 0x631, 0x641, 0x6a9, 0x67e, 0x64a, 0x6ab, 0x648,
0x20, 0x67e, 0x64a, 0x6ab, 0x648, 0x67e, 0x644, 0x627, 0x648, 0x67e, 0x64a, 0x67c,
0x64a, 0x6a9, 0x64a, 0x631, 0x646, 0x67e, 0x648, 0x646, 0x67e, 0x64a, 0x67e, 0x648,
0x631, 0x67c, 0x20, 0x645, 0x648, 0x631, 0x633, 0x628, 0x6cc, 0x631, 0x627, 0x631,
0x648, 0x67c, 0x648, 0x646, 0x6ab, 0x627, 0x633, 0x64a, 0x67e, 0x627, 0x646, 0x67c,
0x647, 0x64a, 0x67c, 0x64a, 0x67c, 0x648, 0x646, 0x6ab, 0x627, 0x62a, 0x627, 0x67e,
0x648, 0x648, 0x627, 0x644, 0x633, 0x647, 0x631, 0x627, 0x631, 0x6d2, 0x62f, 0x20,
0x6a9, 0x645, 0x62a, 0x631, 0x20, 0x634, 0x647, 0x632, 0x627, 0x62f, 0x647, 0x20,
0x62f, 0x631, 0x6d2, 0x20, 0x645, 0x6cc, 0x627, 0x634, 0x62a, 0x646, 0x6cd, 0x622,
0x628, 0x6cc, 0x62c, 0x627, 0x646, 0x622, 0x62f, 0x6cc, 0x633, 0x20, 0x622, 0x628,
0x627, 0x628, 0x627, 0x627, 0x644, 0x62c, 0x632, 0x6cc, 0x631, 0x647, 0x627, 0x633,
0x645, 0x631, 0x647, 0x628, 0x627, 0x645, 0x627, 0x6a9, 0x648, 0x628, 0x627, 0x646,
0x6af, 0x6cc, 0x628, 0x6cc, 0x633, 0x627, 0x626, 0x648, 0x628, 0x644, 0x627, 0x646,
0x62a, 0x6cc, 0x631, 0x647, 0x628, 0x631, 0x627, 0x632, 0x648, 0x6cc, 0x644, 0x6a9,
0x627, 0x632, 0x627, 0x628, 0x644, 0x627, 0x646, 0x6a9, 0x627, 0x633, 0x628, 0x62a,
0x647, 0x62f, 0x627, 0x6a9, 0x627, 0x631, 0x62c, 0x6cc, 0x628, 0x648, 0x62a, 0x6cc,
0x62f, 0x648, 0x622, 0x644, 0x627, 0x627, 0x644, 0x639, 0x6cc, 0x648, 0x646, 0x641,
0x631, 0x6cc, 0x200c, 0x62a, 0x627, 0x648, 0x646, 0x647, 0x631, 0x627, 0x631, 0x647,
0x698, 0x648, 0x647, 0x627, 0x646, 0x633, 0x628, 0x648, 0x631, 0x6af, 0x6a9, 0x627,
0x645, 0x67e, 0x627, 0x644, 0x627, 0x62e, 0x627, 0x631, 0x637, 0x648, 0x645, 0x6a9,
0x6cc, 0x6af, 0x627, 0x644, 0x6cc, 0x6a9, 0x6cc, 0x646, 0x634, 0x627, 0x633, 0x627,
0x644, 0x6cc, 0x628, 0x631, 0x648, 0x6cc, 0x644, 0x644, 0x648, 0x645, 0x647, 0x644,
0x648, 0x628, 0x648, 0x645, 0x628, 0x627, 0x634, 0x6cc, 0x644, 0x648, 0x632, 0x627,
0x6a9, 0x627, 0x645, 0x627, 0x67e, 0x648, 0x62a, 0x648, 0x645, 0x627, 0x633, 0x631,
0x648, 0x645, 0x628, 0x627, 0x628, 0x627, 0x646, 0x647, 0x645, 0x648, 0x6af, 0x627,
0x62f, 0x6cc, 0x634, 0x648, 0x645, 0x648, 0x646, 0x631, 0x648, 0x648, 0x6cc, 0x627,
0x646, 0x627, 0x6cc, 0x631, 0x648, 0x628, 0x6cc, 0x627, 0x646, 0x62c, 0x627, 0x645,
0x646, 0x627, 0x646, 0x6cc, 0x627, 0x645, 0x6cc, 0x646, 0x648, 0x622, 0x6a9, 0x634,
0x648, 0x62a, 0x627, 0x648, 0x622, 0x6af, 0x627, 0x62f, 0x648, 0x6af, 0x648, 0x67e,
0x648, 0x631, 0x62a, 0x648, 0x20, 0x646, 0x648, 0x648, 0x648, 0x633, 0x627, 0x626,
0x648, 0x62a, 0x648, 0x645, 0x647, 0x648, 0x6cc, 0x646, 0x62f, 0x647, 0x648, 0x6a9,
0x627, 0x6cc, 0x62f, 0x6a9, 0x627, 0x646, 0x6a9, 0x648, 0x631, 0x6cc, 0x62c, 0x622,
0x646, 0x6af, 0x648, 0x626, 0x6cc, 0x644, 0x627, 0x622, 0x646, 0x62a, 0x6cc, 0x6af,
0x648, 0x627, 0x622, 0x631, 0x627, 0x6af, 0x648, 0x627, 0x6cc, 0x646, 0x627, 0x628,
0x648, 0x626, 0x646, 0x648, 0x633, 0x200c, 0x622, 0x6cc, 0x631, 0x633, 0x6a9, 0x627,
0x62a, 0x627, 0x645, 0x627, 0x631, 0x6a9, 0x627, 0x6a9, 0x648, 0x631, 0x62f, 0x648,
0x648, 0x627, 0x62e, 0x648, 0x62e, 0x648, 0x6cc, 0x644, 0x627, 0x631, 0x6cc, 0x648,
0x62e, 0x627, 0x645, 0x646, 0x62f, 0x648, 0x633, 0x627, 0x631, 0x6cc, 0x648, 0x6af,
0x627, 0x644, 0x6af, 0x648, 0x633, 0x633, 0x646, 0x200c, 0x62e, 0x648, 0x627, 0x646,
0x633, 0x646, 0x200c, 0x644, 0x648, 0x626, 0x6cc, 0x633, 0x62a, 0x648, 0x6a9, 0x648,
0x645, 0x646, 0x627, 0x631, 0x648, 0x628, 0x627, 0x622, 0x633, 0x648, 0x646, 0x633,
0x6cc, 0x648, 0x646, 0x627, 0x62a, 0x6a9, 0x648, 0x6a9, 0x627, 0x646, 0x628, 0x627,
0x6cc, 0x627, 0x628, 0x627, 0x647, 0x6cc, 0x627, 0x20, 0x628, 0x627, 0x646, 0x62f,
0x631, 0x627, 0x633, 0x628, 0x644, 0x627, 0x646, 0x2010, 0x633, 0x627, 0x628, 0x644,
0x648, 0x646, 0x628, 0x648, 0x626, 0x627, 0x648, 0x6cc, 0x633, 0x62a, 0x627, 0x628,
0x648, 0x6cc, 0x633, 0x6cc, 0x6a9, 0x645, 0x628, 0x631, 0x6cc, 0x62c, 0x200c, 0x628,
0x6cc, 0x6a9, 0x645, 0x67e, 0x648, 0x20, 0x6af, 0x631, 0x627, 0x646, 0x62f, 0x647,
0x6a9, 0x627, 0x646, 0x6a9, 0x648, 0x646, 0x634, 0x6cc, 0x6a9, 0x627, 0x6af, 0x648,
0x686, 0x6cc, 0x648, 0x627, 0x648, 0x627, 0x633, 0x6cc, 0x648, 0x62f, 0x627, 0x62f,
0x20, 0x62e, 0x648, 0x627, 0x631, 0x632, 0x6a9, 0x627, 0x633, 0x62a, 0x627, 0x631,
0x6cc, 0x6a9, 0x627, 0x6a9, 0x631, 0x633, 0x62a, 0x648, 0x646, 0x6a9, 0x648, 0x6cc,
0x627, 0x648, 0x627, 0x6a9, 0x648, 0x631, 0x627, 0x633, 0x627, 0x626, 0x648, 0x62f,
0x627, 0x646, 0x645, 0x627, 0x631, 0x6a9, 0x633, 0x200c, 0x647, 0x627, 0x648, 0x646,
0x62f, 0x627, 0x648, 0x633, 0x646, 0x20, 0x6a9, 0x631, 0x6cc, 0x6a9, 0x62f, 0x646,
0x648, 0x631, 0x62f, 0x6cc, 0x62a, 0x631, 0x648, 0x6cc, 0x62a, 0x627, 0x62f, 0x645,
0x648, 0x646, 0x62a, 0x648, 0x646, 0x627, 0x6cc, 0x631, 0x648, 0x646, 0x67e, 0x647,
0x627, 0x644, 0x633, 0x627, 0x644, 0x648, 0x627, 0x62f, 0x648, 0x631, 0x641, 0x648,
0x631, 0x62a, 0x20, 0x646, 0x644, 0x633, 0x648, 0x646, 0x641, 0x648, 0x631, 0x62a,
0x627, 0x644, 0x632, 0x627, 0x6af, 0x644, 0x6cc, 0x633, 0x200c, 0x628, 0x6cc, 0x6af,
0x648, 0x633, 0x200c, 0x628, 0x6cc, 0x6af, 0x631, 0x627, 0x646, 0x62f, 0x20, 0x62a,
0x648, 0x631, 0x6a9, 0x6af, 0x631, 0x646, 0x627, 0x62f, 0x627, 0x6af, 0x648, 0x627,
0x62f, 0x644, 0x648, 0x67e, 0x6af, 0x648, 0x627, 0x62a, 0x645, 0x627, 0x644, 0x627,
0x6af, 0x648, 0x627, 0x6cc, 0x627, 0x6a9, 0x6cc, 0x644, 0x6af, 0x648, 0x6cc, 0x627,
0x646, 0x647, 0x644, 0x6cc, 0x641, 0x6a9, 0x633, 0x647, 0x627, 0x648, 0x627, 0x646,
0x627, 0x627, 0x631, 0x645, 0x648, 0x633, 0x6cc, 0x648, 0x627, 0x6cc, 0x646, 0x62f,
0x6cc, 0x627, 0x646, 0x627, 0x67e, 0x648, 0x644, 0x6cc, 0x633, 0x646, 0x627, 0x6a9,
0x633, 0x60c, 0x20, 0x627, 0x6cc, 0x646, 0x62f, 0x6cc, 0x627, 0x646, 0x627, 0x645,
0x627, 0x631, 0x646, 0x6af, 0x648, 0x60c, 0x20, 0x627, 0x6cc, 0x646, 0x62f, 0x6cc,
0x627, 0x646, 0x627, 0x67e, 0x6cc, 0x62a, 0x631, 0x632, 0x628, 0x631, 0x6af, 0x60c,
0x20, 0x627, 0x6cc, 0x646, 0x62f, 0x6cc, 0x627, 0x646, 0x627, 0x62a, 0x644, 0x200c,
0x633, 0x6cc, 0x62a, 0x6cc, 0x60c, 0x20, 0x627, 0x6cc, 0x646, 0x62f, 0x6cc, 0x627,
0x646, 0x627, 0x648, 0x6cc, 0x648, 0x6cc, 0x60c, 0x20, 0x627, 0x6cc, 0x646, 0x62f,
0x6cc, 0x627, 0x646, 0x627, 0x648, 0x6cc, 0x646, 0x633, 0x646, 0x633, 0x60c, 0x20,
0x627, 0x646, 0x62f, 0x6cc, 0x627, 0x646, 0x627, 0x648, 0x6cc, 0x646, 0x627, 0x645,
0x627, 0x6a9, 0x60c, 0x20, 0x627, 0x6cc, 0x646, 0x62f, 0x6cc, 0x627, 0x646, 0x627,
0x627, 0x6cc, 0x646, 0x648, 0x648, 0x6cc, 0x6a9, 0x627, 0x6cc, 0x6a9, 0x644, 0x648,
0x626, 0x62a, 0x62c, 0x627, 0x645, 0x627, 0x626, 0x6cc, 0x6a9, 0x627, 0x644, 0x648,
0x626, 0x6cc, 0x632, 0x648, 0x6cc, 0x644, 0x645, 0x627, 0x646, 0x62a, 0x6cc, 0x633,
0x644, 0x648, 0x60c, 0x20, 0x6a9, 0x646, 0x62a, 0x627, 0x6a9, 0x6cc, 0x6a9, 0x631,
0x627, 0x644, 0x646, 0x62f, 0x6cc, 0x6a9, 0x644, 0x627, 0x67e, 0x627, 0x632, 0x644,
0x648, 0x633, 0x200c, 0x622, 0x646, 0x62c, 0x644, 0x633, 0x628, 0x62e, 0x634, 0x20,
0x634, 0x627, 0x647, 0x632, 0x627, 0x62f, 0x647, 0x200c, 0x200c, 0x646, 0x634, 0x6cc,
0x646, 0x20, 0x67e, 0x627, 0x6cc, 0x6cc, 0x646, 0x645, 0x627, 0x633, 0x6cc, 0x648,
0x645, 0x627, 0x646, 0x627, 0x6af, 0x648, 0x627, 0x645, 0x627, 0x631, 0x6cc, 0x6af,
0x627, 0x62a, 0x645, 0x627, 0x631, 0x62a, 0x6cc, 0x646, 0x6cc, 0x6a9, 0x645, 0x646,
0x627, 0x645, 0x6cc, 0x646, 0x6cc, 0x645, 0x631, 0x6cc, 0x62f, 0x627, 0x645, 0x62a,
0x627, 0x644, 0x627, 0x6a9, 0x627, 0x62a, 0x644, 0x627, 0x645, 0x6a9, 0x632, 0x6cc,
0x6a9, 0x648, 0x633, 0x6cc, 0x62a, 0x6cc, 0x645, 0x6cc, 0x6a9, 0x644, 0x648, 0x646,
0x645, 0x627, 0x646, 0x6a9, 0x62a, 0x648, 0x646, 0x645, 0x648, 0x646, 0x62a, 0x631,
0x6cc, 0x645, 0x648, 0x646, 0x62a, 0x647, 0x200c, 0x648, 0x6cc, 0x62f, 0x626, 0x648,
0x645, 0x648, 0x646, 0x62a, 0x633, 0x631, 0x627, 0x62a, 0x646, 0x627, 0x633, 0x627,
0x626, 0x648, 0x646, 0x6cc, 0x648, 0x6cc, 0x648, 0x631, 0x6a9, 0x646, 0x648, 0x631,
0x648, 0x646, 0x6cc, 0x627, 0x628, 0x6cc, 0x648, 0x644, 0x627, 0x60c, 0x20, 0x62f,
0x627, 0x6a9, 0x648, 0x62a, 0x627, 0x6cc, 0x20, 0x634, 0x645, 0x627, 0x644, 0x6cc,
0x633, 0x646, 0x62a, 0x631, 0x60c, 0x20, 0x62f, 0x627, 0x6a9, 0x648, 0x62a, 0x627,
0x6cc, 0x20, 0x634, 0x645, 0x627, 0x644, 0x6cc, 0x646, 0x6cc, 0x648, 0x633, 0x627,
0x644, 0x645, 0x60c, 0x20, 0x62f, 0x627, 0x6a9, 0x648, 0x62a, 0x627, 0x6cc, 0x20,
0x634, 0x645, 0x627, 0x644, 0x6cc, 0x627, 0x648, 0x62c, 0x6cc, 0x646, 0x627, 0x6af,
0x627, 0x67e, 0x627, 0x631, 0x627, 0x645, 0x627, 0x631, 0x6cc, 0x628, 0x648, 0x67e,
0x648, 0x631, 0x62a, 0x648, 0x67e, 0x631, 0x646, 0x633, 0x67e, 0x648, 0x631, 0x62a,
0x200c, 0x622, 0x648, 0x627, 0x633, 0x67e, 0x6cc, 0x646, 0x67e, 0x648, 0x631, 0x62a,
0x648, 0x648, 0x644, 0x6cc, 0x648, 0x67e, 0x648, 0x631, 0x62a, 0x648, 0x631, 0x6cc,
0x6a9, 0x648, 0x67e, 0x648, 0x646, 0x62a, 0x627, 0x20, 0x622, 0x631, 0x646, 0x627,
0x633, 0x62e, 0x644, 0x6cc, 0x62c, 0x6a9, 0x20, 0x631, 0x646, 0x6a9, 0x6cc, 0x646,
0x631, 0x6cc, 0x633, 0x6cc, 0x641, 0x6cc, 0x631, 0x62c, 0x627, 0x6cc, 0x646, 0x627,
0x631, 0x632, 0x648, 0x644, 0x648, 0x62a, 0x631, 0x6cc, 0x648, 0x628, 0x631, 0x627,
0x646, 0x6a9, 0x648, 0x633, 0x646, 0x62a, 0x627, 0x631, 0x645, 0x633, 0x627, 0x646,
0x62a, 0x6cc, 0x627, 0x6af, 0x648, 0x633, 0x627, 0x646, 0x62a, 0x648, 0x20, 0x62f,
0x648, 0x645, 0x6cc, 0x646, 0x6af, 0x648, 0x633, 0x627, 0x626, 0x648, 0x67e, 0x627,
0x626, 0x648, 0x644, 0x648, 0x627, 0x633, 0x6a9, 0x648, 0x631, 0x633, 0x628, 0x6cc,
0x633, 0x648, 0x646, 0x62f, 0x633, 0x6cc, 0x62a, 0x6a9, 0x627, 0x633, 0x646, 0x62a,
0x20, 0x628, 0x627, 0x631, 0x62a, 0x644, 0x645, 0x6cc, 0x633, 0x646, 0x62a, 0x20,
0x62c, 0x627, 0x646, 0x633, 0x646, 0x62a, 0x20, 0x6a9, 0x6cc, 0x62a, 0x633, 0x633,
0x646, 0x62a, 0x20, 0x644, 0x648, 0x633, 0x6cc, 0x627, 0x633, 0x646, 0x62a, 0x20,
0x62a, 0x648, 0x645, 0x627, 0x633, 0x633, 0x646, 0x62a, 0x20, 0x648, 0x6cc, 0x646,
0x633, 0x646, 0x62a, 0x633, 0x648, 0x6cc, 0x641, 0x62a, 0x200c, 0x6a9, 0x627, 0x631,
0x646, 0x62a, 0x62a, 0x6af, 0x648, 0x633, 0x6cc, 0x6af, 0x627, 0x644, 0x67e, 0x627,
0x62a, 0x6cc, 0x62e, 0x648, 0x627, 0x646, 0x627, 0x62a, 0x648, 0x631, 0x646, 0x62a,
0x648, 0x648, 0x646, 0x6a9, 0x648, 0x648, 0x631, 0x648, 0x627, 0x6cc, 0x62a, 0x200c,
0x647, 0x648, 0x631, 0x633, 0x648, 0x6cc, 0x646, 0x6cc, 0x67e, 0x6af, 0x6cc, 0x627,
0x6a9, 0x648, 0x62a, 0x627, 0x62a, 0x62f, 0x6cc, 0x648, 0x6cc, 0x633, 0x62f, 0x648,
0x645, 0x648, 0x646, 0x20, 0x62f, 0x648, 0x631, 0x648, 0x6cc, 0x644, 0x645, 0x6a9,
0x648, 0x627, 0x631, 0x6cc, 0x645, 0x6a9, 0x200c, 0x645, 0x648, 0x631, 0x62f, 0x648,
0x631, 0x648, 0x62a, 0x631, 0x627, 0x634, 0x648, 0x648, 0x627, 0x644, 0x627, 0x646,
0x6af, 0x200c, 0x6cc, 0x631, 0x628, 0x6cc, 0x646, 0x622, 0x644, 0x645, 0x627, 0x62a,
0x6cc, 0x639, 0x64e, 0x645, 0x627, 0x646, 0x622, 0x646, 0x627, 0x62f, 0x6cc, 0x631,
0x622, 0x642, 0x62a, 0x627, 0x648, 0x622, 0x642, 0x62a, 0x648, 0x628, 0x647, 0x639,
0x634, 0x642, 0x200c, 0x622, 0x628, 0x627, 0x62f, 0x622, 0x62a, 0x6cc, 0x631, 0x627,
0x648, 0x628, 0x627, 0x646, 0x6a9, 0x648, 0x6a9, 0x628, 0x627, 0x631, 0x646, 0x627,
0x626, 0x648, 0x644, 0x628, 0x6cc, 0x634, 0x6a9, 0x6a9, 0x628, 0x631, 0x648, 0x646,
0x626, 0x6cc, 0x686, 0x6cc, 0x62a, 0x627, 0x6a9, 0x644, 0x645, 0x628, 0x648, 0x62f,
0x6cc, 0x644, 0x6cc, 0x62f, 0x628, 0x6cc, 0x62f, 0x648, 0x634, 0x646, 0x628, 0x647,
0x641, 0x627, 0x645, 0x627, 0x6af, 0x648, 0x633, 0x62a, 0x627, 0x627, 0x644, 0x62e,
0x644, 0x6cc, 0x644, 0x647, 0x648, 0x634, 0x6cc, 0x200c, 0x645, 0x6cc, 0x646, 0x200c,
0x633, 0x6cc, 0x62a, 0x6cc, 0x647, 0x646, 0x6af, 0x200c, 0x6a9, 0x646, 0x6af, 0x62e,
0x648, 0x648, 0x62f, 0x627, 0x6cc, 0x631, 0x6a9, 0x648, 0x62a, 0x633, 0x6a9, 0x62c,
0x627, 0x6a9, 0x627, 0x631, 0x62a, 0x627, 0x62c, 0x627, 0x6cc, 0x627, 0x67e, 0x648,
0x631, 0x627, 0x627, 0x648, 0x631, 0x634, 0x644, 0x6cc, 0x645, 0x6a9, 0x631, 0x627,
0x686, 0x6cc, 0x6a9, 0x627, 0x62a, 0x645, 0x627, 0x646, 0x62f, 0x648, 0x62e, 0x627,
0x646, 0x62f, 0x6cc, 0x6af, 0x627, 0x6a9, 0x644, 0x6a9, 0x62a, 0x647, 0x6a9, 0x631,
0x627, 0x633, 0x646, 0x648, 0x6cc, 0x627, 0x631, 0x633, 0x6a9, 0x6a9, 0x648, 0x627,
0x644, 0x627, 0x644, 0x627, 0x645, 0x67e, 0x648, 0x631, 0x6a9, 0x648, 0x686, 0x6cc,
0x646, 0x6af, 0x6a9, 0x648, 0x6cc, 0x62a, 0x645, 0x627, 0x6a9, 0x627, 0x626, 0x648,
0x645, 0x627, 0x6a9, 0x627, 0x633, 0x627, 0x631, 0x645, 0x627, 0x646, 0x6cc, 0x644,
0x646, 0x6cc, 0x6a9, 0x648, 0x632, 0x6cc, 0x627, 0x646, 0x648, 0x648, 0x648, 0x6a9,
0x648, 0x632, 0x646, 0x62a, 0x633, 0x6a9, 0x646, 0x648, 0x648, 0x633, 0x6cc, 0x628,
0x6cc, 0x631, 0x6cc, 0x633, 0x6a9, 0x627, 0x648, 0x631, 0x627, 0x644, 0x67e, 0x646,
0x648, 0x645, 0x200c, 0x67e, 0x646, 0x67e, 0x648, 0x646, 0x62a, 0x6cc, 0x627, 0x646,
0x627, 0x6a9, 0x67e, 0x6cc, 0x648, 0x646, 0x6af, 0x200c, 0x6cc, 0x627, 0x646, 0x6af,
0x642, 0x648, 0x633, 0x62a, 0x627, 0x646, 0x627, 0x6cc, 0x642, 0x6cc, 0x632, 0x6cc,
0x644, 0x200c, 0x627, 0x648, 0x631, 0x62f, 0x627, 0x633, 0x627, 0x62e, 0x627, 0x644,
0x6cc, 0x646, 0x633, 0x626, 0x648, 0x644, 0x634, 0x627, 0x646, 0x6af, 0x647, 0x627,
0x6cc, 0x633, 0x646, 0x6af, 0x627, 0x67e, 0x648, 0x631, 0x627, 0x633, 0x631, 0x62f,
0x646, 0x6a9, 0x648, 0x644, 0x6cc, 0x645, 0x633, 0x6a9, 0x62a, 0x627, 0x6cc, 0x67e,
0x647, 0x62a, 0x627, 0x634, 0x6a9, 0x646, 0x62f, 0x62a, 0x641, 0x644, 0x6cc, 0x633,
0x62a, 0x6cc, 0x645, 0x641, 0x648, 0x62a, 0x648, 0x6a9, 0x6cc, 0x648, 0x62a, 0x648,
0x645, 0x633, 0x6a9, 0x627, 0x648, 0x644, 0x627, 0x646, 0x200c, 0x628, 0x627, 0x62a,
0x648, 0x631, 0x627, 0x631, 0x648, 0x645, 0x686, 0x6cc, 0x627, 0x648, 0x633, 0x62a,
0x20, 0x646, 0x631, 0x627, 0x648, 0x6cc, 0x646, 0x62a, 0x6cc, 0x627, 0x646, 0x648,
0x644, 0x627, 0x62f, 0x6cc, 0x200c, 0x648, 0x633, 0x62a, 0x648, 0x6a9, 0x6cc, 0x627,
0x6a9, 0x648, 0x62a, 0x633, 0x6a9, 0x6cc, 0x627, 0x646, 0x6af, 0x648, 0x646, 0x6cc,
0x6a9, 0x627, 0x62a, 0x631, 0x6cc, 0x646, 0x628, 0x631, 0x6af, 0x627, 0x6cc, 0x631,
0x648, 0x627, 0x646, 0x622, 0x632, 0x648, 0x631, 0x642, 0x646, 0x627, 0x631, 0x6cc,
0x6a9, 0x6cc, 0x67e, 0x200c, 0x648, 0x631, 0x62f, 0x645, 0x627, 0x62f, 0x6cc, 0x631,
0x627, 0x631, 0x6cc, 0x6a9, 0x6cc, 0x627, 0x648, 0x6cc, 0x6a9, 0x62c, 0x648, 0x631,
0x62c, 0x6cc, 0x627, 0x6cc, 0x20, 0x62c, 0x646, 0x648, 0x628, 0x6cc, 0x633, 0x646,
0x62a, 0x20, 0x647, 0x644, 0x646, 0x627, 0x627, 0x633, 0x62a, 0x627, 0x646, 0x644,
0x6cc, 0x622, 0x62f, 0x644, 0x627, 0x6cc, 0x62f, 0x628, 0x631, 0x6cc, 0x633, 0x628,
0x6cc, 0x646, 0x628, 0x631, 0x648, 0x6a9, 0x646, 0x200c, 0x647, 0x6cc, 0x644, 0x62f,
0x627, 0x631, 0x648, 0x6cc, 0x646, 0x627, 0x648, 0x6a9, 0x644, 0x627, 0x644, 0x6cc,
0x646, 0x62f, 0x645, 0x646, 0x644, 0x631, 0x62f, 0x647, 0x627, 0x648, 0x645, 0x644,
0x628, 0x648, 0x631, 0x646, 0x633, 0x6cc, 0x62f, 0x646, 0x6cc, 0x622, 0x645, 0x633,
0x62a, 0x631, 0x62f, 0x627, 0x645, 0x622, 0x646, 0x62f, 0x648, 0x631, 0x627, 0x622,
0x633, 0x62a, 0x631, 0x627, 0x62e, 0x627, 0x646, 0x622, 0x62a, 0x646, 0x628, 0x644,
0x6af, 0x631, 0x627, 0x62f, 0x628, 0x631, 0x644, 0x6cc, 0x646, 0x628, 0x631, 0x627,
0x62a, 0x6cc, 0x633, 0x644, 0x627, 0x648, 0x627, 0x628, 0x631, 0x648, 0x6a9, 0x633,
0x644, 0x628, 0x648, 0x62f, 0x627, 0x67e, 0x633, 0x62a, 0x628, 0x627, 0x632, 0x646,
0x6af, 0x646, 0x6a9, 0x6cc, 0x634, 0x6cc, 0x646, 0x627, 0x648, 0x6a9, 0x67e, 0x646,
0x647, 0x627, 0x6af, 0x62f, 0x648, 0x628, 0x644, 0x6cc, 0x646, 0x62c, 0x628, 0x644,
0x200c, 0x627, 0x644, 0x637, 0x627, 0x631, 0x642, 0x6af, 0x631, 0x646, 0x632, 0x6cc,
0x647, 0x644, 0x633, 0x6cc, 0x646, 0x6a9, 0x6cc, 0x62c, 0x632, 0x6cc, 0x631, 0x647,
0x654, 0x20, 0x645, 0x646, 0x627, 0x633, 0x62a, 0x627, 0x646, 0x628, 0x648, 0x644,
0x62c, 0x631, 0x632, 0x6cc, 0x6a9, 0x627, 0x644, 0x6cc, 0x646, 0x6cc, 0x646, 0x6af,
0x631, 0x627, 0x62f, 0x644, 0x6cc, 0x648, 0x628, 0x644, 0x6cc, 0x627, 0x646, 0x627,
0x644, 0x648, 0x6a9, 0x632, 0x627, 0x645, 0x628, 0x648, 0x631, 0x6af, 0x645, 0x627,
0x62f, 0x631, 0x6cc, 0x62f, 0x645, 0x627, 0x631, 0x6cc, 0x647, 0x200c, 0x647, 0x627,
0x645, 0x646, 0x645, 0x6cc, 0x646, 0x633, 0x6a9, 0x67e, 0x627, 0x62f, 0x6af, 0x627,
0x631, 0x6cc, 0x62a, 0x633, 0x627, 0x633, 0x627, 0x646, 0x200c, 0x645, 0x627, 0x631,
0x6cc, 0x646, 0x648, 0x633, 0x627, 0x631, 0x627, 0x6cc, 0x648, 0x648, 0x633, 0x6cc,
0x645, 0x641, 0x631, 0x648, 0x67e, 0x644, 0x627, 0x633, 0x6a9, 0x648, 0x67e, 0x6cc,
0x647, 0x635, 0x648, 0x641, 0x6cc, 0x647, 0x627, 0x633, 0x62a, 0x6a9, 0x647, 0x644,
0x645, 0x62a, 0x627, 0x644, 0x6cc, 0x646, 0x62a, 0x6cc, 0x631, 0x627, 0x646, 0x627,
0x627, 0x648, 0x644, 0x6cc, 0x627, 0x646, 0x648, 0x641, 0x633, 0x6a9, 0x641, 0x627,
0x62f, 0x648, 0x62a, 0x633, 0x648, 0x627, 0x62a, 0x6cc, 0x6a9, 0x627, 0x646, 0x648,
0x6cc, 0x644, 0x646, 0x6cc, 0x648, 0x633, 0x648, 0x644, 0x6af, 0x627, 0x6af, 0x631,
0x627, 0x62f, 0x648, 0x631, 0x634, 0x648, 0x632, 0x627, 0x6af, 0x631, 0x628, 0x632,
0x648, 0x631, 0x6cc, 0x62e, 0x622, 0x646, 0x62a, 0x627, 0x646, 0x627, 0x646, 0x627,
0x631, 0x6cc, 0x648, 0x648, 0x634, 0x627, 0x6af, 0x648, 0x633, 0x6a9, 0x631, 0x6cc,
0x633, 0x645, 0x633, 0x6a9, 0x631, 0x6af, 0x648, 0x644, 0x646, 0x645, 0x627, 0x647,
0x647, 0x645, 0x627, 0x644, 0x62f, 0x6cc, 0x648, 0x645, 0x648, 0x631, 0x6cc, 0x633,
0x645, 0x627, 0x6cc, 0x648, 0x62a, 0x631, 0x626, 0x648, 0x646, 0x6cc, 0x648, 0x646,
0x622, 0x67e, 0x6cc, 0x627, 0x627, 0x648, 0x6a9, 0x644, 0x646, 0x62f, 0x628, 0x648,
0x6af, 0x646, 0x648, 0x6cc, 0x644, 0x686, 0x62a, 0x200c, 0x647, 0x627, 0x645, 0x627,
0x6cc, 0x633, 0x62a, 0x631, 0x627, 0x641, 0x627, 0x62a, 0x647, 0x641, 0x627, 0x6a9,
0x627, 0x626, 0x648, 0x641, 0x648, 0x641, 0x6cc, 0x62c, 0x6cc, 0x641, 0x648, 0x646,
0x627, 0x641, 0x648, 0x62a, 0x6cc, 0x6af, 0x627, 0x645, 0x628, 0x6cc, 0x631, 0x6af,
0x648, 0x627, 0x62f, 0x627, 0x644, 0x200c, 0x6a9, 0x627, 0x646, 0x627, 0x644, 0x627,
0x646, 0x62f, 0x631, 0x628, 0x631, 0x6cc, 0x6a9, 0x631, 0x6cc, 0x62a, 0x6cc, 0x645,
0x627, 0x62a, 0x6cc, 0x6a9, 0x648, 0x633, 0x631, 0x627, 0x6cc, 0x6a9, 0x648, 0x627,
0x62c, 0x6cc, 0x644, 0x6cc, 0x646, 0x645, 0x627, 0x631, 0x6a9, 0x648, 0x632, 0x647,
0x645, 0x6cc, 0x62f, 0x648, 0x6cc, 0x646, 0x627, 0x626, 0x648, 0x631, 0x648, 0x646,
0x6cc, 0x648, 0x626, 0x647, 0x646, 0x648, 0x631, 0x641, 0x648, 0x644, 0x6a9, 0x646,
0x648, 0x645, 0x626, 0x627, 0x67e, 0x627, 0x6af, 0x648, 0x67e, 0x627, 0x6af, 0x648,
0x67e, 0x627, 0x644, 0x627, 0x626, 0x648, 0x67e, 0x6cc, 0x62a, 0x200c, 0x6a9, 0x631,
0x646, 0x67e, 0x627, 0x646, 0x67e, 0x6cc, 0x67e, 0x648, 0x631, 0x62a, 0x200c, 0x645,
0x648, 0x631, 0x632, 0x628, 0x6cc, 0x633, 0x627, 0x6cc, 0x67e, 0x627, 0x646, 0x62a,
0x627, 0x647, 0x6cc, 0x62a, 0x6cc, 0x62a, 0x648, 0x646, 0x6af, 0x627, 0x62a, 0x627,
0x67e, 0x648, 0x648, 0x627, 0x644, 0x6cc, 0x633, 0x41, 0x62, 0x69, 0x64, 0x17c,
0x61, 0x6e, 0x42, 0x61, 0x6e, 0x64, 0x17c, 0x75, 0x6c, 0x42, 0x75, 0x17c,
0x75, 0x6d, 0x62, 0x75, 0x72, 0x61, 0x44, 0x17c, 0x69, 0x62, 0x75, 0x74,
0x69, 0x41, 0x6c, 0x2d, 0x55, 0x6a, 0x75, 0x6e, 0x44, 0x17c, 0x75, 0x62,
0x61, 0x4b, 0x69, 0x6e, 0x73, 0x7a, 0x61, 0x73, 0x61, 0x4d, 0x6f, 0x67,
0x61, 0x64, 0x69, 0x73, 0x7a, 0x75, 0x4e, 0x64, 0x17c, 0x61, 0x6d, 0x65,
0x6e, 0x61, 0x4e, 0x61, 0x77, 0x61, 0x6b, 0x73, 0x7a, 0x75, 0x74, 0x54,
0x72, 0x79, 0x70, 0x6f, 0x6c, 0x69, 0x73, 0x4b, 0x61, 0x6a, 0x6d, 0x61,
0x6e, 0x79, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x72, 0x79, 0x6b, 0x61, 0x53,
0x61, 0x6c, 0x77, 0x61, 0x64, 0x6f, 0x72, 0x47, 0x77, 0x61, 0x64, 0x65,
0x6c, 0x75, 0x70, 0x61, 0x47, 0x75, 0x6a, 0x61, 0x6e, 0x61, 0x48, 0x61,
0x77, 0x61, 0x6e, 0x61, 0x4d, 0x61, 0x72, 0x74, 0x79, 0x6e, 0x69, 0x6b,
0x61, 0x4d, 0x65, 0x6b, 0x73, 0x79, 0x6b, 0x20, 0x28, 0x6d, 0x69, 0x61,
0x73, 0x74, 0x6f, 0x29, 0x4e, 0x6f, 0x77, 0x79, 0x20, 0x4a, 0x6f, 0x72,
0x6b, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x20, 0x50, 0xf3, 0x142, 0x6e, 0x6f, 0x63, 0x6e, 0x61,
0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x20, 0x50, 0xf3, 0x142, 0x6e, 0x6f, 0x63, 0x6e, 0x61, 0x4e,
0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x20, 0x50, 0xf3, 0x142, 0x6e, 0x6f, 0x63, 0x6e,
0x61, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x72, 0x79, 0x6b, 0x6f, 0x41, 0x142,
0x6d, 0x61, 0x74, 0x79, 0x41, 0x6b, 0x74, 0x69, 0x75, 0x62, 0x69, 0x144,
0x73, 0x6b, 0x41, 0x73, 0x7a, 0x63, 0x68, 0x61, 0x62, 0x61, 0x64, 0x42,
0x61, 0x72, 0x6e, 0x61, 0x75, 0x142, 0x42, 0x69, 0x73, 0x7a, 0x6b, 0x65,
0x6b, 0x43, 0x7a, 0x79, 0x74, 0x61, 0x44, 0x61, 0x6d, 0x61, 0x73, 0x7a,
0x65, 0x6b, 0x44, 0x75, 0x73, 0x7a, 0x61, 0x6e, 0x62, 0x65, 0x4b, 0x6f,
0x62, 0x64, 0x6f, 0x44, 0x17c, 0x61, 0x6b, 0x61, 0x72, 0x74, 0x61, 0x4a,
0x65, 0x72, 0x6f, 0x7a, 0x6f, 0x6c, 0x69, 0x6d, 0x61, 0x4b, 0x61, 0x6d,
0x63, 0x7a, 0x61, 0x74, 0x6b, 0x61, 0x4b, 0x61, 0x72, 0x61, 0x63, 0x7a,
0x69, 0x4b, 0x75, 0x77, 0x65, 0x6a, 0x74, 0x4e, 0x69, 0x6b, 0x6f, 0x7a,
0x6a, 0x61, 0x4e, 0x6f, 0x77, 0x6f, 0x6b, 0x75, 0x17a, 0x6e, 0x69, 0x65,
0x63, 0x6b, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x79, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x4b, 0x75, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x6a, 0x4b, 0x79, 0x7a,
0x79, 0x142, 0x6f, 0x72, 0x64, 0x61, 0x53, 0x7a, 0x61, 0x6e, 0x67, 0x68,
0x61, 0x6a, 0x53, 0x72, 0x69, 0x65, 0x64, 0x6e, 0x69, 0x65, 0x6b, 0x6f,
0x142, 0x79, 0x6d, 0x73, 0x6b, 0x54, 0x61, 0x73, 0x7a, 0x6b, 0x69, 0x65,
0x6e, 0x74, 0x55, 0x142, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72,
0x55, 0x72, 0x75, 0x6d, 0x63, 0x7a, 0x69, 0x55, 0x73, 0x74, 0x2d, 0x4e,
0x69, 0x65, 0x72, 0x61, 0x57, 0x69, 0x65, 0x6e, 0x74, 0x69, 0x61, 0x6e,
0x57, 0x142, 0x61, 0x64, 0x79, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4a,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x79, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x45, 0x72, 0x79, 0x77, 0x61, 0x144, 0x41, 0x7a, 0x6f, 0x72, 0x79, 0x57,
0x79, 0x73, 0x70, 0x79, 0x20, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x79, 0x6a,
0x73, 0x6b, 0x69, 0x65, 0x52, 0x65, 0x70, 0x75, 0x62, 0x6c, 0x69, 0x6b,
0x61, 0x20, 0x5a, 0x69, 0x65, 0x6c, 0x6f, 0x6e, 0x65, 0x67, 0x6f, 0x20,
0x50, 0x72, 0x7a, 0x79, 0x6c, 0x105, 0x64, 0x6b, 0x61, 0x57, 0x79, 0x73,
0x70, 0x79, 0x20, 0x4f, 0x77, 0x63, 0x7a, 0x65, 0x4d, 0x61, 0x64, 0x65,
0x72, 0x61, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x50, 0x6f,
0x142, 0x75, 0x64, 0x6e, 0x69, 0x6f, 0x77, 0x61, 0x15a, 0x77, 0x69, 0x119,
0x74, 0x61, 0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x41, 0x73, 0x74,
0x72, 0x61, 0x63, 0x68, 0x61, 0x144, 0x41, 0x74, 0x65, 0x6e, 0x79, 0x42,
0x72, 0x61, 0x74, 0x79, 0x73, 0x142, 0x61, 0x77, 0x61, 0x42, 0x72, 0x75,
0x6b, 0x73, 0x65, 0x6c, 0x61, 0x42, 0x75, 0x6b, 0x61, 0x72, 0x65, 0x73,
0x7a, 0x74, 0x42, 0x75, 0x64, 0x61, 0x70, 0x65, 0x73, 0x7a, 0x74, 0x4b,
0x69, 0x73, 0x7a, 0x79, 0x6e, 0x69, 0xf3, 0x77, 0x57, 0x79, 0x73, 0x70,
0x61, 0x20, 0x4d, 0x61, 0x6e, 0x53, 0x74, 0x61, 0x6d, 0x62, 0x75, 0x142,
0x4b, 0x69, 0x6a, 0xf3, 0x77, 0x4c, 0x69, 0x7a, 0x62, 0x6f, 0x6e, 0x61,
0x4c, 0x75, 0x62, 0x6c, 0x61, 0x6e, 0x61, 0x4c, 0x6f, 0x6e, 0x64, 0x79,
0x6e, 0x4d, 0x61, 0x64, 0x72, 0x79, 0x74, 0x4d, 0x69, 0x144, 0x73, 0x6b,
0x50, 0x61, 0x72, 0x79, 0x17c, 0x52, 0x7a, 0x79, 0x6d, 0x53, 0x61, 0x72,
0x61, 0x6a, 0x65, 0x77, 0x6f, 0x53, 0x61, 0x72, 0x61, 0x74, 0xf3, 0x77,
0x53, 0x79, 0x6d, 0x66, 0x65, 0x72, 0x6f, 0x70, 0x6f, 0x6c, 0x53, 0x7a,
0x74, 0x6f, 0x6b, 0x68, 0x6f, 0x6c, 0x6d, 0x57, 0x61, 0x74, 0x79, 0x6b,
0x61, 0x6e, 0x57, 0x69, 0x65, 0x64, 0x65, 0x144, 0x57, 0x69, 0x6c, 0x6e,
0x6f, 0x57, 0x6f, 0x142, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x5a, 0x61,
0x67, 0x72, 0x7a, 0x65, 0x62, 0x5a, 0x75, 0x72, 0x79, 0x63, 0x68, 0x41,
0x6e, 0x74, 0x61, 0x6e, 0x61, 0x6e, 0x61, 0x72, 0x79, 0x77, 0x61, 0x43,
0x7a, 0x61, 0x67, 0x6f, 0x73, 0x57, 0x79, 0x73, 0x70, 0x61, 0x20, 0x42,
0x6f, 0x17c, 0x65, 0x67, 0x6f, 0x20, 0x4e, 0x61, 0x72, 0x6f, 0x64, 0x7a,
0x65, 0x6e, 0x69, 0x61, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0x4b, 0x6f,
0x6b, 0x6f, 0x73, 0x6f, 0x77, 0x65, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20,
0x4b, 0x65, 0x72, 0x67, 0x75, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x4d, 0x61,
0x6a, 0x6f, 0x74, 0x74, 0x61, 0x57, 0x79, 0x73, 0x70, 0x61, 0x20, 0x42,
0x6f, 0x75, 0x67, 0x61, 0x69, 0x6e, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x2019,
0x61, 0x57, 0x79, 0x73, 0x70, 0x61, 0x20, 0x57, 0x69, 0x65, 0x6c, 0x6b,
0x61, 0x6e, 0x6f, 0x63, 0x6e, 0x61, 0x46, 0x69, 0x64, 0x17c, 0x69, 0x57,
0x79, 0x73, 0x70, 0x79, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72,
0x61, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x7a, 0x79, 0x43, 0x6f, 0x6e, 0x61,
0x63, 0x72, 0x69, 0x4a, 0x6f, 0x61, 0x6e, 0x65, 0x73, 0x62, 0x75, 0x72,
0x67, 0x6f, 0x43, 0x61, 0x72, 0x74, 0x75, 0x6d, 0x4d, 0x6f, 0x67, 0x61,
0x64, 0xed, 0x73, 0x63, 0x69, 0x6f, 0x4e, 0x61, 0x69, 0x72, 0xf3, 0x62,
0x69, 0x41, 0x6e, 0x74, 0xed, 0x67, 0x75, 0x61, 0x54, 0x75, 0x63, 0x75,
0x6d, 0xe3, 0x41, 0x73, 0x73, 0x75, 0x6e, 0xe7, 0xe3, 0x6f, 0x47, 0x75,
0x61, 0x69, 0x61, 0x71, 0x75, 0x69, 0x6c, 0x47, 0x75, 0x69, 0x61, 0x6e,
0x61, 0x43, 0x69, 0x64, 0x61, 0x64, 0x65, 0x20, 0x64, 0x6f, 0x20, 0x4d,
0xe9, 0x78, 0x69, 0x63, 0x6f, 0x4d, 0x6f, 0x6e, 0x74, 0x65, 0x76, 0x69,
0x64, 0xe9, 0x75, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4e, 0x6f, 0x72,
0x74, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74,
0x65, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6e, 0x2c, 0x20,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4e, 0x6f,
0x72, 0x74, 0x65, 0x53, 0xe3, 0x6f, 0x20, 0x42, 0x61, 0x72, 0x74, 0x6f,
0x6c, 0x6f, 0x6d, 0x65, 0x75, 0x53, 0xe3, 0x6f, 0x20, 0x43, 0x72, 0x69,
0x73, 0x74, 0xf3, 0x76, 0xe3, 0x6f, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x20,
0x4c, 0xfa, 0x63, 0x69, 0x61, 0x53, 0xe3, 0x6f, 0x20, 0x56, 0x69, 0x63,
0x65, 0x6e, 0x74, 0x65, 0x41, 0x6d, 0xe3, 0x41, 0x73, 0x67, 0x61, 0x62,
0x61, 0x74, 0x65, 0x42, 0x65, 0x69, 0x72, 0x75, 0x74, 0x65, 0x44, 0x75,
0x63, 0x68, 0x61, 0x6d, 0x62, 0x65, 0x43, 0x69, 0x64, 0x61, 0x64, 0x65,
0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d,
0x69, 0x6e, 0x68, 0x4a, 0x61, 0x63, 0x61, 0x72, 0x74, 0x61, 0x4a, 0x65,
0x72, 0x75, 0x73, 0x61, 0x6c, 0xe9, 0x6d, 0x43, 0x61, 0x6c, 0x63, 0x75,
0x74, 0xe1, 0x4e, 0x69, 0x63, 0xf3, 0x73, 0x69, 0x61, 0x52, 0x69, 0x61,
0x64, 0x65, 0x53, 0x61, 0x63, 0x61, 0x6c, 0x69, 0x6e, 0x61, 0x58, 0x61,
0x6e, 0x67, 0x61, 0x69, 0x54, 0x65, 0x65, 0x72, 0xe3, 0x54, 0xf3, 0x71,
0x75, 0x69, 0x6f, 0x52, 0x61, 0x6e, 0x67, 0x75, 0x6d, 0x45, 0x63, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x43, 0x61,
0x6e, 0xe1, 0x72, 0x69, 0x61, 0x73, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20,
0x46, 0x61, 0x72, 0x6f, 0xe9, 0x47, 0x65, 0xf3, 0x72, 0x67, 0x69, 0x61,
0x20, 0x64, 0x6f, 0x20, 0x53, 0x75, 0x6c, 0x41, 0x6d, 0x73, 0x74, 0x65,
0x72, 0x64, 0xe3, 0x41, 0x73, 0x74, 0x72, 0x61, 0x63, 0xe3, 0x42, 0x65,
0x72, 0x6c, 0x69, 0x6d, 0x42, 0x75, 0x63, 0x61, 0x72, 0x65, 0x73, 0x74,
0x65, 0x42, 0x75, 0x64, 0x61, 0x70, 0x65, 0x73, 0x74, 0x65, 0x48, 0x65,
0x6c, 0x73, 0x69, 0x6e, 0x71, 0x75, 0x65, 0x49, 0x6c, 0x68, 0x61, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x61, 0x6e, 0x4d, 0xf4, 0x6e, 0x61, 0x63, 0x6f,
0x53, 0xf3, 0x66, 0x69, 0x61, 0x56, 0x61, 0x72, 0x73, 0xf3, 0x76, 0x69,
0x61, 0x5a, 0x75, 0x72, 0x69, 0x71, 0x75, 0x65, 0x4d, 0x61, 0x75, 0x72,
0xed, 0x63, 0x69, 0x6f, 0x52, 0x65, 0x75, 0x6e, 0x69, 0xe3, 0x6f, 0x43,
0x68, 0x61, 0x74, 0x6e, 0x61, 0x6d, 0x49, 0x6c, 0x68, 0x61, 0x20, 0x64,
0x65, 0x20, 0x50, 0xe1, 0x73, 0x63, 0x6f, 0x61, 0x54, 0x61, 0x72, 0x61,
0x75, 0x61, 0x41, 0x64, 0x69, 0x73, 0x2d, 0x41, 0x62, 0x65, 0x62, 0x61,
0x44, 0x61, 0x63, 0x61, 0x72, 0x4a, 0x69, 0x62, 0x75, 0x74, 0x69, 0x43,
0x61, 0x6d, 0x70, 0x61, 0x6c, 0x61, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x2d,
0x4e, 0x6f, 0x76, 0x6f, 0x41, 0x72, 0x61, 0x67, 0x75, 0x61, 0x69, 0x6e,
0x61, 0x43, 0x61, 0x69, 0x6d, 0xe3, 0x6f, 0x43, 0x75, 0x72, 0x61, 0xe7,
0x61, 0x75, 0x44, 0x6f, 0x6d, 0xed, 0x6e, 0x69, 0x63, 0x61, 0x4d, 0x6f,
0x6e, 0x74, 0x65, 0x76, 0x69, 0x64, 0x65, 0x75, 0x4d, 0x6f, 0x6e, 0x73,
0x65, 0x72, 0x72, 0x61, 0x74, 0x65, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x49,
0x6f, 0x72, 0x71, 0x75, 0x65, 0x50, 0x6f, 0x72, 0x74, 0x2d, 0x61, 0x75,
0x2d, 0x50, 0x72, 0x69, 0x6e, 0x63, 0x65, 0x50, 0x6f, 0x72, 0x74, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x45, 0x73, 0x70, 0x61, 0x6e, 0x68, 0x61, 0x41,
0x64, 0x65, 0x6d, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x42, 0x61, 0x67,
0x64, 0x61, 0x64, 0x65, 0x42, 0x61, 0x72, 0xe9, 0x6d, 0x42, 0x61, 0x6e,
0x67, 0x75, 0x65, 0x63, 0x6f, 0x71, 0x75, 0x65, 0x43, 0x61, 0x72, 0x61,
0x63, 0x68, 0x69, 0x43, 0x61, 0x74, 0x6d, 0x61, 0x6e, 0x64, 0x75, 0x54,
0x61, 0x69, 0x70, 0xe9, 0x54, 0x65, 0x65, 0x72, 0xe3, 0x6f, 0x52, 0x65,
0x69, 0x71, 0x75, 0x69, 0x61, 0x76, 0x69, 0x71, 0x75, 0x65, 0x49, 0x6c,
0x68, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x41, 0x6d, 0x65, 0x73, 0x74, 0x65, 0x72, 0x64, 0xe3,
0x6f, 0x42, 0x75, 0x73, 0x69, 0x6e, 0x67, 0x65, 0x6e, 0x43, 0x6f, 0x70,
0x65, 0x6e, 0x68, 0x61, 0x67, 0x61, 0x48, 0x65, 0x6c, 0x73, 0xed, 0x6e,
0x71, 0x75, 0x69, 0x61, 0x43, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x6e, 0x67,
0x72, 0x61, 0x64, 0x6f, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x76, 0x6f, 0x53,
0xe3, 0x6f, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6e, 0x68, 0x6f, 0x54, 0x61,
0x6c, 0x69, 0x6d, 0x49, 0x6c, 0x68, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4e,
0x61, 0x74, 0x61, 0x6c, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x43, 0x6f,
0x63, 0x6f, 0x73, 0x4d, 0x61, 0x68, 0x65, 0x4d, 0x61, 0x75, 0x72, 0xed,
0x63, 0x69, 0x61, 0x49, 0x6c, 0x68, 0x61, 0x20, 0x64, 0x61, 0x20, 0x50,
0xe1, 0x73, 0x63, 0x6f, 0x61, 0x45, 0x66, 0x61, 0x74, 0x65, 0x49, 0x6c,
0x68, 0x61, 0x73, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e,
0x54, 0x61, 0x72, 0x61, 0x77, 0x61, 0xa05, 0xa2c, 0xa40, 0xa26, 0xa1c, 0xa3e,
0xa28, 0xa05, 0xa71, 0xa15, 0xa30, 0xa3e, 0xa10, 0xa21, 0xa3f, 0xa38, 0x20, 0xa05,
0xa2c, 0xa3e, 0xa2c, 0xa3e, 0xa05, 0xa32, 0xa1c, 0xa40, 0xa05, 0xa30, 0xa38, 0xa05,
0xa38, 0xa2e, 0xa3e, 0xa30, 0xa3e, 0xa2c, 0xa2e, 0xa47, 0xa15, 0xa4b, 0xa2c, 0xa3e,
0xa02, 0xa17, 0xa41, 0xa08, 0xa2c, 0xa3e, 0xa02, 0xa1c, 0xa41, 0xa32, 0xa2c, 0xa3f,
0xa38, 0xa3e, 0xa09, 0xa2c, 0xa32, 0xa70, 0xa1f, 0xa3e, 0xa07, 0xa30, 0xa2c, 0xa4d,
0xa30, 0xa3e, 0xa1c, 0xa3c, 0xa3e, 0xa35, 0xa3f, 0xa32, 0xa47, 0xa2c, 0xa41, 0xa1c,
0xa41, 0xa70, 0xa2c, 0xa41, 0xa30, 0xa3e, 0xa15, 0xa48, 0xa30, 0xa4b, 0xa15, 0xa3e,
0xa38, 0xa3e, 0xa2c, 0xa32, 0xa3e, 0xa02, 0xa15, 0xa3e, 0xa38, 0xa40, 0xa0a, 0xa1f,
0xa3e, 0xa15, 0xa4b, 0xa28, 0xa47, 0xa15, 0xa30, 0xa40, 0xa21, 0xa15, 0xa3e, 0xa30,
0xa26, 0xa3e, 0xa30, 0x20, 0xa0f, 0xa38, 0x20, 0xa38, 0xa32, 0xa3e, 0xa2e, 0xa1c,
0xa3f, 0xa2c, 0xa42, 0xa24, 0xa40, 0xa21, 0xa4c, 0xa06, 0xa32, 0xa3e, 0xa05, 0xa32,
0x20, 0xa05, 0xa2f, 0xa42, 0xa28, 0xa2b, 0xa30, 0xa40, 0xa1f, 0xa3e, 0xa09, 0xa28,
0xa17, 0xa3e, 0xa2c, 0xa4b, 0xa30, 0xa4b, 0xa28, 0xa39, 0xa30, 0xa3e, 0xa30, 0xa47,
0xa1c, 0xa4b, 0xa39, 0xa28, 0xa38, 0xa2c, 0xa30, 0xa17, 0xa1c, 0xa42, 0xa2c, 0xa3e,
0xa15, 0xa70, 0xa2a, 0xa3e, 0xa32, 0xa3e, 0xa16, 0xa3e, 0xa30, 0xa1f, 0xa4c, 0xa2e,
0xa15, 0xa3f, 0xa17, 0xa3e, 0xa32, 0xa40, 0xa15, 0xa3f, 0xa70, 0xa38, 0xa3c, 0xa3e,
0xa38, 0xa3e, 0xa32, 0xa3e, 0xa17, 0xa4b, 0xa38, 0xa32, 0xa3f, 0xa2c, 0xa30, 0xa35,
0xa3f, 0xa32, 0xa47, 0xa32, 0xa4b, 0xa2e, 0xa32, 0xa41, 0xa06, 0xa02, 0xa21, 0xa3e,
0xa32, 0xa41, 0xa2c, 0xa41, 0xa2e, 0xa2c, 0xa3e, 0xa38, 0xa3c, 0xa40, 0xa32, 0xa41,
0xa38, 0xa3e, 0xa15, 0xa3e, 0xa2e, 0xa3e, 0xa32, 0xa3e, 0xa2c, 0xa4b, 0xa2e, 0xa3e,
0xa2a, 0xa41, 0xa1f, 0xa4b, 0xa2e, 0xa38, 0xa47, 0xa30, 0xa42, 0xa0f, 0xa2e, 0xa2c,
0xa3e, 0xa2c, 0xa3e, 0xa28, 0xa47, 0xa2e, 0xa4b, 0xa17, 0xa3e, 0xa26, 0xa3f, 0xa38,
0xa3c, 0xa42, 0xa2e, 0xa4b, 0xa28, 0xa30, 0xa4b, 0xa35, 0xa40, 0xa06, 0xa28, 0xa48,
0xa30, 0xa4b, 0xa2c, 0xa40, 0xa10, 0xa02, 0xa1c, 0xa3e, 0xa2e, 0xa47, 0xa28, 0xa3e,
0xa28, 0xa3f, 0xa06, 0xa2e, 0xa47, 0xa28, 0xa4c, 0xa06, 0xa15, 0xa38, 0xa3c, 0xa3e,
0xa1f, 0xa09, 0xa06, 0xa17, 0xa3e, 0xa21, 0xa42, 0xa17, 0xa42, 0xa2a, 0xa4b, 0xa30,
0xa1f, 0xa4b, 0x2d, 0xa28, 0xa4b, 0xa35, 0xa4b, 0xa38, 0xa3e, 0xa13, 0x20, 0xa1f,
0xa4b, 0xa2e, 0xa24, 0xa4d, 0xa30, 0xa3f, 0xa2a, 0xa4b, 0xa32, 0xa40, 0xa1f, 0xa41,
0xa28, 0xa3f, 0xa38, 0xa35, 0xa3f, 0xa70, 0xa21, 0xa39, 0xa4b, 0xa07, 0xa15, 0xa0f,
0xa21, 0xa15, 0xa10, 0xa02, 0xa15, 0xa30, 0xa47, 0xa1c, 0xa05, 0xa70, 0xa17, 0xa41,
0xa07, 0xa32, 0xa3e, 0xa10, 0xa02, 0xa1f, 0xa40, 0xa17, 0xa41, 0xa06, 0xa06, 0xa30,
0xa17, 0xa41, 0xa06, 0xa07, 0xa28, 0xa3e, 0xa2c, 0xa42, 0xa08, 0xa28, 0xa38, 0x20,
0xa06, 0xa07, 0xa30, 0xa38, 0xa15, 0xa48, 0xa1f, 0xa3e, 0xa2e, 0xa3e, 0xa30, 0xa15,
0xa3e, 0xa15, 0xa4b, 0xa30, 0xa21, 0xa4b, 0xa2c, 0xa3e, 0xa1c, 0xa42, 0xa1c, 0xa41,
0xa0f, 0xa32, 0xa3e, 0x20, 0xa30, 0xa3f, 0xa09, 0xa1c, 0xa3e, 0xa2e, 0xa48, 0xa02,
0xa21, 0xa4b, 0xa1c, 0xa3c, 0xa3e, 0xa30, 0xa3f, 0xa13, 0x20, 0xa17, 0xa48, 0xa32,
0xa47, 0xa17, 0xa4b, 0xa38, 0xa38, 0xa3e, 0xa32, 0xa1f, 0xa3e, 0xa38, 0xa47, 0xa28,
0x20, 0xa1c, 0xa41, 0xa06, 0xa28, 0xa38, 0xa47, 0xa28, 0x20, 0xa32, 0xa42, 0xa08,
0xa38, 0xa1f, 0xa41, 0xa15, 0xa41, 0xa2e, 0xa28, 0xa09, 0xa38, 0xa3c, 0xa35, 0xa3e,
0xa07, 0xa06, 0xa05, 0xa30, 0xa42, 0xa2c, 0xa3e, 0xa10, 0xa38, 0xa41, 0xa70, 0xa15,
0xa40, 0xa05, 0xa28, 0xa10, 0xa1f, 0xa40, 0xa15, 0xa4b, 0xa15, 0xa28, 0xa2c, 0xa3e,
0xa39, 0xa40, 0xa06, 0xa2c, 0xa3e, 0xa39, 0xa40, 0xa06, 0x20, 0xa2c, 0xa3e, 0xa02,
0xa26, 0xa47, 0xa30, 0xa38, 0xa2c, 0xa3e, 0xa30, 0xa2c, 0xa3e, 0xa21, 0xa4b, 0xa38,
0xa2c, 0xa47, 0xa32, 0xa47, 0xa2e, 0xa2c, 0xa47, 0xa32, 0xa40, 0xa1c, 0xa3c, 0xa2c,
0xa32, 0xa48, 0xa02, 0xa15, 0x2d, 0xa38, 0xa48, 0xa2c, 0xa32, 0xa3e, 0xa28, 0xa2c,
0xa4b, 0xa06, 0x20, 0xa35, 0xa3f, 0xa38, 0xa1f, 0xa3e, 0xa2c, 0xa4b, 0xa17, 0xa4b,
0xa1f, 0xa3e, 0xa2c, 0xa4b, 0xa07, 0xa38, 0xa15, 0xa48, 0xa2e, 0xa2c, 0xa4d, 0xa30,
0xa3f, 0xa1c, 0x20, 0xa2c, 0xa47, 0xa15, 0xa48, 0xa02, 0xa2a, 0xa4b, 0x20, 0xa17,
0xa4d, 0xa30, 0xa3e, 0xa02, 0xa21, 0xa47, 0xa15, 0xa48, 0xa28, 0xa15, 0xa41, 0xa28,
0xa15, 0xa48, 0xa30, 0xa3e, 0xa15, 0xa3e, 0xa38, 0xa15, 0xa47, 0xa2f, 0xa47, 0xa28,
0xa47, 0xa15, 0xa47, 0xa2e, 0xa48, 0xa28, 0xa38, 0xa3c, 0xa3f, 0xa15, 0xa3e, 0xa17,
0xa4b, 0xa1a, 0xa3f, 0xa39, 0xa41, 0xa06, 0xa39, 0xa41, 0xa06, 0xa38, 0xa3f, 0xa09,
0xa21, 0xa3e, 0xa21, 0x20, 0xa39, 0xa41, 0xa06, 0xa30, 0xa47, 0xa1c, 0xa3c, 0xa15,
0xa4b, 0xa38, 0xa1f, 0xa3e, 0x20, 0xa30, 0xa40, 0xa15, 0xa3e, 0xa15, 0xa4d, 0xa30,
0xa48, 0xa38, 0xa1f, 0xa28, 0xa15, 0xa2f, 0xa41, 0xa0f, 0xa2c, 0xa3e, 0xa15, 0xa41,
0xa30, 0xa3e, 0xa15, 0xa3e, 0xa13, 0xa21, 0xa48, 0xa28, 0xa2e, 0xa3e, 0xa30, 0xa15,
0xa38, 0xa3c, 0xa4c, 0xa28, 0xa21, 0xa4c, 0xa05, 0xa38, 0xa28, 0xa21, 0xa4c, 0xa05,
0xa38, 0xa28, 0x20, 0xa15, 0xa4d, 0xa30, 0xa40, 0xa15, 0xa21, 0xa47, 0xa28, 0xa35,
0xa30, 0xa21, 0xa3f, 0xa1f, 0xa30, 0xa4b, 0xa07, 0xa1f, 0xa21, 0xa4b, 0xa2e, 0xa40,
0xa28, 0xa3f, 0xa15, 0xa3e, 0xa10, 0xa21, 0xa2e, 0xa4b, 0xa02, 0xa1f, 0xa28, 0xa2f,
0xa41, 0xa30, 0xa42, 0xa28, 0xa40, 0xa2a, 0xa47, 0xa05, 0xa32, 0x20, 0xa38, 0xa32,
0xa35, 0xa3e, 0xa21, 0xa4b, 0xa30, 0xa2b, 0xa4b, 0xa30, 0xa1f, 0x20, 0xa28, 0xa48,
0xa32, 0xa38, 0xa28, 0xa2b, 0xa4b, 0xa30, 0xa1f, 0xa3e, 0xa32, 0xa47, 0xa1c, 0xa3c,
0xa3e, 0xa17, 0xa32, 0xa47, 0xa38, 0x20, 0xa2c, 0xa47, 0xa17, 0xa42, 0xa1c, 0xa3c,
0x20, 0xa2c, 0xa47, 0xa17, 0xa30, 0xa3e, 0xa02, 0xa21, 0x20, 0xa24, 0xa41, 0xa30,
0xa15, 0xa17, 0xa4d, 0xa30, 0xa47, 0xa28, 0xa3e, 0xa21, 0xa3e, 0xa17, 0xa41, 0xa06,
0xa21, 0xa47, 0xa32, 0xa4b, 0xa2a, 0xa17, 0xa41, 0xa06, 0xa1f, 0xa47, 0xa2e, 0xa3e,
0xa32, 0xa3e, 0xa17, 0xa41, 0xa06, 0xa07, 0xa15, 0xa35, 0xa3f, 0xa32, 0xa17, 0xa41,
0xa06, 0xa28, 0xa3e, 0xa39, 0xa48, 0xa32, 0xa40, 0xa2b, 0xa48, 0xa15, 0xa38, 0xa39,
0xa35, 0xa3e, 0xa28, 0xa3e, 0xa39, 0xa30, 0xa2e, 0xa4b, 0xa38, 0xa3f, 0xa71, 0xa32,
0xa4b, 0xa07, 0xa70, 0xa21, 0xa40, 0xa06, 0xa28, 0xa3e, 0xa2a, 0xa4b, 0xa32, 0xa3f,
0xa38, 0xa28, 0xa4b, 0xa15, 0xa38, 0x2c, 0x20, 0xa07, 0xa70, 0xa21, 0xa40, 0xa06,
0xa28, 0xa3e, 0xa2e, 0xa3e, 0xa30, 0xa47, 0xa02, 0xa17, 0xa4b, 0x2c, 0x20, 0xa07,
0xa70, 0xa21, 0xa40, 0xa06, 0xa28, 0xa3e, 0xa2a, 0xa40, 0xa1f, 0xa30, 0xa38, 0xa2c,
0xa30, 0xa17, 0x2c, 0x20, 0xa07, 0xa70, 0xa21, 0xa40, 0xa06, 0xa28, 0xa3e, 0xa1f,
0xa48, 0xa32, 0x20, 0xa38, 0xa3f, 0xa1f, 0xa40, 0x2c, 0x20, 0xa07, 0xa70, 0xa21,
0xa40, 0xa06, 0xa28, 0xa3e, 0xa35, 0xa47, 0xa35, 0xa47, 0x2c, 0x20, 0xa07, 0xa70,
0xa21, 0xa40, 0xa06, 0xa28, 0xa3e, 0xa35, 0xa3f, 0xa70, 0xa38, 0xa47, 0xa28, 0xa47,
0xa38, 0x2c, 0x20, 0xa07, 0xa70, 0xa21, 0xa40, 0xa06, 0xa28, 0xa3e, 0xa35, 0xa3f,
0xa28, 0xa2e, 0xa48, 0xa15, 0x2c, 0x20, 0xa07, 0xa70, 0xa21, 0xa40, 0xa06, 0xa28,
0xa3e, 0xa07, 0xa28, 0xa41, 0xa35, 0xa3f, 0xa15, 0xa07, 0xa15, 0xa3e, 0xa32, 0xa41,
0xa08, 0xa1f, 0xa1c, 0xa2e, 0xa3e, 0xa07, 0xa15, 0xa3e, 0xa1c, 0xa2f, 0xa42, 0xa28,
0xa4b, 0xa32, 0xa41, 0xa08, 0xa38, 0xa35, 0xa3f, 0xa32, 0xa47, 0xa2e, 0xa4b, 0xa02,
0xa1f, 0xa40, 0xa38, 0xa48, 0xa32, 0xa4b, 0x2c, 0x20, 0xa15, 0xa48, 0xa02, 0xa1f,
0xa15, 0xa40, 0xa15, 0xa4d, 0xa30, 0xa3e, 0xa32, 0xa47, 0xa02, 0xa26, 0xa3f, 0xa1c,
0xa15, 0xa32, 0xa3e, 0x20, 0xa2a, 0xa3e, 0xa1c, 0xa3c, 0xa32, 0xa40, 0xa2e, 0xa3e,
0xa32, 0xa3e, 0xa38, 0x20, 0xa10, 0xa02, 0xa1c, 0xa32, 0xa38, 0xa32, 0xa4b, 0xa05,
0xa30, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa3f, 0xa70, 0xa38, 0x2019, 0xa38, 0x20, 0xa15,
0xa35, 0xa3e, 0xa30, 0xa1f, 0xa30, 0xa2e, 0xa48, 0xa38, 0xa40, 0xa13, 0xa2e, 0xa28,
0xa3e, 0xa17, 0xa41, 0xa06, 0xa2e, 0xa28, 0xa4c, 0xa38, 0xa2e, 0xa48, 0xa30, 0xa40,
0xa17, 0xa4b, 0xa1f, 0xa2e, 0xa3e, 0xa30, 0xa1f, 0xa40, 0xa28, 0xa3f, 0xa15, 0xa2e,
0xa3e, 0xa1f, 0xa3e, 0xa2e, 0xa4b, 0xa30, 0xa38, 0xa2e, 0xa1c, 0xa3c, 0xa47, 0xa24,
0xa32, 0xa3e, 0xa28, 0xa2e, 0xa48, 0xa28, 0xa4b, 0xa2e, 0xa3f, 0xa28, 0xa40, 0xa2e,
0xa47, 0xa30, 0xa3f, 0xa21, 0xa3e, 0xa2e, 0xa47, 0xa1f, 0xa32, 0xa3e, 0xa15, 0xa3e,
0xa1f, 0xa32, 0xa3e, 0xa2e, 0xa48, 0xa15, 0xa38, 0xa40, 0xa15, 0xa4b, 0x20, 0xa38,
0xa3f, 0xa1f, 0xa40, 0xa2e, 0xa3f, 0xa15, 0xa47, 0xa32, 0xa28, 0xa2e, 0xa4b, 0xa02,
0xa15, 0xa1f, 0xa28, 0xa2e, 0xa4b, 0xa28, 0xa1f, 0xa47, 0xa30, 0xa40, 0xa2e, 0xa4b,
0xa02, 0xa1f, 0xa47, 0xa35, 0xa40, 0xa21, 0xa40, 0xa13, 0xa2e, 0xa4b, 0xa02, 0xa1f,
0xa38, 0xa47, 0xa30, 0xa3e, 0xa24, 0xa28, 0xa3e, 0xa38, 0xa3e, 0xa13, 0xa28, 0xa3f,
0xa0a, 0x20, 0xa2f, 0xa3e, 0xa30, 0xa15, 0xa28, 0xa4c, 0xa30, 0xa4b, 0xa28, 0xa39,
0xa3e, 0xa2c, 0xa3f, 0xa09, 0xa32, 0xa3e, 0x2c, 0x20, 0xa09, 0xa71, 0xa24, 0xa30,
0xa40, 0x20, 0xa21, 0xa15, 0xa4b, 0xa1f, 0xa3e, 0xa38, 0xa47, 0xa02, 0xa1f, 0xa30,
0x2c, 0x20, 0xa09, 0xa71, 0xa24, 0xa30, 0xa40, 0x20, 0xa21, 0xa15, 0xa4b, 0xa1f,
0xa3e, 0xa28, 0xa3f, 0xa0a, 0x20, 0xa38, 0xa32, 0xa47, 0xa2e, 0x2c, 0x20, 0xa09,
0xa71, 0xa24, 0xa30, 0xa40, 0x20, 0xa21, 0xa15, 0xa4b, 0xa1f, 0xa3e, 0xa28, 0xa42,
0xa15, 0xa13, 0xa1c, 0xa40, 0xa28, 0xa3e, 0xa17, 0xa3e, 0xa2a, 0xa28, 0xa3e, 0xa2e,
0xa3e, 0xa2a, 0xa48, 0xa30, 0xa3e, 0xa2e, 0xa30, 0xa40, 0xa2c, 0xa4b, 0xa2b, 0xa3f,
0xa28, 0xa3f, 0xa15, 0xa38, 0xa2a, 0xa4b, 0xa30, 0xa1f, 0x2d, 0xa14, 0x2d, 0xa2a,
0xa4d, 0xa30, 0xa3f, 0xa70, 0xa38, 0xa2a, 0xa4b, 0xa30, 0xa1f, 0x20, 0xa14, 0xa2b,
0x20, 0xa38, 0xa2a, 0xa47, 0xa28, 0xa2a, 0xa4b, 0xa30, 0xa1f, 0xa4b, 0x20, 0xa35,
0xa47, 0xa32, 0xa4d, 0xa39, 0xa4b, 0xa2a, 0xa3f, 0xa0a, 0xa30, 0xa1f, 0xa4b, 0x20,
0xa30, 0xa40, 0xa15, 0xa4b, 0xa2a, 0xa70, 0xa1f, 0xa3e, 0x20, 0xa05, 0xa30, 0xa47,
0xa28, 0xa38, 0xa30, 0xa48, 0xa02, 0xa15, 0xa3f, 0xa28, 0x20, 0xa07, 0xa28, 0xa32,
0xa48, 0xa1f, 0xa30, 0xa47, 0xa38, 0xa40, 0xa2b, 0xa47, 0xa30, 0xa48, 0xa1c, 0xa40,
0xa28, 0xa3e, 0xa30, 0xa48, 0xa1c, 0xa3c, 0xa4b, 0xa32, 0xa3f, 0xa0a, 0xa1f, 0xa30,
0xa40, 0xa13, 0x20, 0xa2c, 0xa4d, 0xa30, 0xa3e, 0xa02, 0xa15, 0xa4b, 0xa38, 0xa47,
0xa02, 0xa1f, 0xa3e, 0xa30, 0xa2e, 0xa38, 0xa47, 0xa02, 0xa1f, 0xa40, 0xa06, 0xa17,
0xa4b, 0xa38, 0xa47, 0xa02, 0xa1f, 0xa4b, 0x20, 0xa21, 0xa4b, 0xa2e, 0xa3f, 0xa70,
0xa17, 0xa4b, 0xa38, 0xa3e, 0xa13, 0x20, 0xa2a, 0xa4c, 0xa32, 0xa4b, 0xa07, 0xa71,
0xa1f, 0xa4b, 0xa15, 0xa4b, 0xa30, 0xa1f, 0xa42, 0xa30, 0xa2e, 0xa40, 0xa1f, 0xa38,
0xa3f, 0xa1f, 0xa15, 0xa3e, 0xa38, 0xa47, 0xa02, 0xa1f, 0x20, 0xa2c, 0xa3e, 0xa30,
0xa25, 0xa47, 0xa32, 0xa47, 0xa2e, 0xa40, 0xa38, 0xa47, 0xa02, 0xa1f, 0x20, 0xa1c,
0xa4c, 0xa39, 0xa28, 0xa38, 0xa38, 0xa47, 0xa02, 0xa1f, 0x20, 0xa15, 0xa3f, 0xa1f,
0xa38, 0xa38, 0xa47, 0xa02, 0xa1f, 0x20, 0xa32, 0xa41, 0xa38, 0xa40, 0xa06, 0xa38,
0xa47, 0xa02, 0xa1f, 0x20, 0xa25, 0xa4b, 0xa2e, 0xa38, 0xa38, 0xa47, 0xa02, 0xa1f,
0x20, 0xa35, 0xa3f, 0xa28, 0xa38, 0xa47, 0xa02, 0xa1f, 0xa38, 0xa35, 0xa3f, 0xa2b,
0xa1f, 0x20, 0xa15, 0xa30, 0xa70, 0xa1f, 0xa1f, 0xa47, 0xa17, 0xa41, 0xa38, 0xa40,
0xa17, 0xa32, 0xa2a, 0xa3e, 0xa25, 0xa41, 0xa32, 0xa47, 0xa1f, 0xa3f, 0xa1c, 0xa42,
0xa06, 0xa28, 0xa3e, 0xa1f, 0xa4b, 0xa30, 0xa3e, 0xa02, 0xa1f, 0xa4b, 0xa1f, 0xa4b,
0xa30, 0xa1f, 0xa4b, 0xa32, 0xa3e, 0xa35, 0xa48, 0xa28, 0xa15, 0xa42, 0xa35, 0xa30,
0xa35, 0xa3e, 0xa08, 0xa1f, 0xa39, 0xa4c, 0xa30, 0xa38, 0xa35, 0xa3f, 0xa28, 0xa40,
0xa2a, 0xa48, 0xa17, 0xa2f, 0xa15, 0xa41, 0xa24, 0xa24, 0xa21, 0xa47, 0xa35, 0xa3f,
0xa38, 0xa21, 0xa3f, 0xa09, 0xa2e, 0xa4b, 0xa02, 0xa1f, 0x20, 0xa21, 0xa3f, 0xa09,
0xa30, 0xa35, 0xa3f, 0xa32, 0xa47, 0xa2e, 0xa48, 0xa15, 0xa15, 0xa35, 0xa48, 0xa30,
0xa40, 0xa2e, 0xa4c, 0xa38, 0xa28, 0xa2e, 0xa48, 0xa15, 0xa2e, 0xa41, 0xa30, 0xa21,
0xa4b, 0xa2a, 0xa3e, 0xa2e, 0xa30, 0xa30, 0xa4b, 0xa25, 0xa47, 0xa30, 0xa3e, 0xa38,
0xa35, 0xa4b, 0xa2f, 0xa3e, 0xa1f, 0xa30, 0xa4b, 0xa32, 0xa35, 0xa4b, 0xa38, 0xa1f,
0xa4b, 0xa15, 0xa32, 0xa4b, 0xa02, 0xa17, 0xa08, 0xa05, 0xa30, 0xa2c, 0xa3e, 0xa07,
0xa28, 0xa05, 0xa21, 0xa47, 0xa28, 0xa05, 0xa32, 0xa2e, 0xa47, 0xa1f, 0xa40, 0xa05,
0xa2e, 0xa3e, 0xa28, 0xa10, 0xa28, 0xa3e, 0xa21, 0xa3e, 0xa07, 0xa30, 0xa05, 0xa15,
0xa24, 0xa4c, 0xa05, 0xa15, 0xa24, 0xa4b, 0xa2c, 0xa47, 0xa05, 0xa38, 0xa3c, 0xa17,
0xa3e, 0xa2c, 0xa3e, 0xa1f, 0xa0f, 0xa24, 0xa47, 0xa30, 0xa3e, 0xa13, 0xa2c, 0xa17,
0xa26, 0xa3e, 0xa26, 0xa2c, 0xa39, 0xa3f, 0xa30, 0xa40, 0xa28, 0xa2c, 0xa3e, 0xa15,
0xa42, 0xa2c, 0xa48, 0xa02, 0xa15, 0xa3e, 0xa15, 0xa2c, 0xa30, 0xa28, 0xa4c, 0xa32,
0xa2c, 0xa48, 0xa30, 0xa42, 0xa24, 0xa2c, 0xa3f, 0xa38, 0xa3c, 0xa15, 0xa47, 0xa15,
0xa2c, 0xa30, 0xa42, 0xa28, 0xa47, 0xa08, 0xa1a, 0xa3f, 0xa24, 0xa3e, 0xa15, 0xa4b,
0xa32, 0xa70, 0xa2c, 0xa4b, 0xa21, 0xa48, 0xa2e, 0xa38, 0xa15, 0xa38, 0xa22, 0xa3e,
0xa15, 0xa3e, 0xa21, 0xa3f, 0xa32, 0xa40, 0xa26, 0xa41, 0xa2c, 0xa08, 0xa26, 0xa41,
0xa38, 0xa3c, 0xa3e, 0xa02, 0xa2c, 0xa47, 0xa2b, 0xa3e, 0xa2e, 0xa3e, 0xa17, 0xa41,
0xa38, 0xa1f, 0xa3e, 0xa17, 0xa3e, 0xa1c, 0xa3c, 0xa3e, 0xa39, 0xa47, 0xa2c, 0xa30,
0xa3e, 0xa28, 0xa39, 0xa4b, 0x20, 0xa1a, 0xa40, 0x20, 0xa2e, 0xa3f, 0xa28, 0xa4d,
0xa39, 0x20, 0xa38, 0xa3f, 0xa1f, 0xa40, 0xa39, 0xa3e, 0xa02, 0xa17, 0x20, 0xa15,
0xa3e, 0xa02, 0xa17, 0xa39, 0xa4b, 0xa35, 0xa21, 0xa07, 0xa30, 0xa15, 0xa41, 0xa24,
0xa38, 0xa15, 0xa1c, 0xa15, 0xa3e, 0xa30, 0xa24, 0xa3e, 0xa1c, 0xa2f, 0xa3e, 0xa2a,
0xa41, 0xa30, 0xa3e, 0xa1c, 0xa47, 0xa30, 0xa42, 0xa38, 0xa32, 0xa2e, 0xa15, 0xa3e,
0xa2c, 0xa41, 0xa32, 0xa15, 0xa2e, 0xa1a, 0xa1f, 0xa15, 0xa3e, 0xa15, 0xa30, 0xa3e,
0xa1a, 0xa40, 0xa15, 0xa3e, 0xa20, 0xa2e, 0xa3e, 0xa02, 0xa21, 0xa42, 0xa16, 0xa3e,
0xa28, 0xa21, 0xa3f, 0xa17, 0xa3e, 0xa15, 0xa4b, 0xa32, 0xa15, 0xa3e, 0xa24, 0xa3e,
0xa15, 0xa30, 0xa48, 0xa38, 0xa28, 0xa47, 0xa1c, 0xa30, 0xa38, 0xa15, 0xa41, 0xa06,
0xa32, 0xa3e, 0xa32, 0xa70, 0xa2a, 0xa41, 0xa30, 0xa15, 0xa41, 0xa1a, 0xa3f, 0xa70,
0xa17, 0xa15, 0xa41, 0xa35, 0xa48, 0xa24, 0xa2e, 0xa15, 0xa3e, 0xa09, 0xa2e, 0xa48,
0xa17, 0xa47, 0xa21, 0xa28, 0xa2e, 0xa15, 0xa38, 0xa3e, 0xa30, 0xa2e, 0xa28, 0xa40,
0xa32, 0xa3e, 0xa2e, 0xa38, 0xa15, 0xa1f, 0xa28, 0xa3f, 0xa15, 0xa4b, 0xa38, 0xa40,
0xa06, 0xa28, 0xa4b, 0xa35, 0xa4b, 0xa15, 0xa41, 0xa1c, 0xa3c, 0xa28, 0xa47, 0xa1f,
0xa38, 0xa15, 0xa28, 0xa4b, 0xa35, 0xa4b, 0xa38, 0xa3f, 0xa2c, 0xa40, 0xa30, 0xa38,
0xa15, 0xa13, 0xa2e, 0xa38, 0xa15, 0xa13, 0xa30, 0xa32, 0xa2b, 0xa28, 0xa4b, 0xa2e,
0x20, 0xa2a, 0xa47, 0xa28, 0xa39, 0xa2a, 0xa4c, 0xa02, 0xa1f, 0xa40, 0xa06, 0xa28,
0xa3e, 0xa15, 0xa2a, 0xa2f, 0xa4b, 0xa02, 0xa17, 0xa2f, 0xa3e, 0xa02, 0xa17, 0xa15,
0xa24, 0xa30, 0xa15, 0xa4b, 0xa38, 0xa24, 0xa3e, 0xa28, 0xa3e, 0xa0f, 0xa15, 0xa3f,
0xa1c, 0xa3c, 0xa40, 0xa32, 0xa4b, 0xa30, 0xa21, 0xa3e, 0xa30, 0xa3f, 0xa06, 0xa27,
0xa38, 0xa16, 0xa32, 0xa40, 0xa28, 0xa38, 0xa2e, 0xa30, 0xa15, 0xa70, 0xa26, 0xa38,
0xa3f, 0xa09, 0xa32, 0xa38, 0xa3c, 0xa70, 0xa18, 0xa3e, 0xa08, 0xa38, 0xa3f, 0xa70,
0xa17, 0xa3e, 0xa2a, 0xa41, 0xa30, 0xa38, 0xa30, 0xa3f, 0xa21, 0xa28, 0xa40, 0xa15,
0xa4b, 0xa32, 0xa3f, 0xa38, 0xa15, 0xa24, 0xa48, 0xa2a, 0xa08, 0xa24, 0xa3e, 0xa38,
0xa3c, 0xa15, 0xa70, 0xa26, 0xa1f, 0xa2c, 0xa3f, 0xa32, 0xa3f, 0xa38, 0xa40, 0xa24,
0xa47, 0xa39, 0xa30, 0xa3e, 0xa28, 0xa25, 0xa3f, 0xa70, 0xa2b, 0xa42, 0xa1f, 0xa4b,
0xa15, 0xa40, 0xa13, 0xa1f, 0xa4b, 0xa2e, 0xa38, 0xa15, 0xa09, 0xa32, 0xa3e, 0xa28,
0xa2c, 0xa3e, 0xa24, 0xa30, 0xa0a, 0xa30, 0xa42, 0xa2e, 0xa15, 0xa40, 0xa09, 0xa38,
0xa24, 0x2d, 0xa28, 0xa47, 0xa30, 0xa3e, 0xa35, 0xa3e, 0xa0f, 0xa28, 0xa1f, 0xa3f,
0xa06, 0xa28, 0xa47, 0xa35, 0xa32, 0xa3e, 0xa26, 0xa40, 0xa35, 0xa4b, 0xa38, 0xa24,
0xa15, 0xa2f, 0xa15, 0xa41, 0xa24, 0xa38, 0xa15, 0xa30, 0xa70, 0xa17, 0xa42, 0xa28,
0xa2f, 0xa15, 0xa47, 0xa24, 0xa30, 0xa3f, 0xa28, 0xa2c, 0xa30, 0xa17, 0xa2f, 0xa47,
0xa30, 0xa47, 0xa35, 0xa28, 0xa05, 0xa1c, 0xa4b, 0xa30, 0xa47, 0xa38, 0xa2c, 0xa30,
0xa2e, 0xa42, 0xa21, 0xa3e, 0xa15, 0xa47, 0xa28, 0xa47, 0xa30, 0xa40, 0xa15, 0xa47,
0xa2a, 0x20, 0xa35, 0xa30, 0xa21, 0xa2b, 0xa48, 0xa30, 0xa4b, 0xa2e, 0xa21, 0xa40,
0xa05, 0xa30, 0xa3e, 0xa30, 0xa47, 0xa15, 0xa1c, 0xa3e, 0xa35, 0xa3f, 0xa15, 0xa26,
0xa71, 0xa16, 0xa23, 0xa40, 0x20, 0xa1c, 0xa3e, 0xa30, 0xa1c, 0xa40, 0xa06, 0xa38,
0xa47, 0xa02, 0xa1f, 0x20, 0xa39, 0xa47, 0xa32, 0xa47, 0xa28, 0xa3e, 0xa38, 0xa1f,
0xa47, 0xa28, 0xa32, 0xa40, 0xa10, 0xa21, 0xa40, 0xa32, 0xa47, 0xa21, 0xa2c, 0xa4d,
0xa30, 0xa3f, 0xa38, 0xa2c, 0xa47, 0xa28, 0xa2c, 0xa4d, 0xa30, 0xa4b, 0xa15, 0xa28,
0x20, 0xa39, 0xa3f, 0xa32, 0xa21, 0xa3e, 0xa30, 0xa35, 0xa3f, 0xa28, 0xa2f, 0xa42,
0xa15, 0xa32, 0xa3e, 0xa39, 0xa4b, 0xa2c, 0xa3e, 0xa30, 0xa1f, 0xa32, 0xa3f, 0xa70,
0xa21, 0xa47, 0xa2e, 0xa28, 0xa32, 0xa4c, 0xa30, 0xa21, 0x20, 0xa39, 0xa4b, 0xa35,
0xa47, 0xa2e, 0xa48, 0xa32, 0xa2c, 0xa4b, 0xa30, 0xa28, 0xa2a, 0xa30, 0xa25, 0xa38,
0xa3f, 0xa21, 0xa28, 0xa40, 0xa10, 0xa2e, 0xa38, 0xa1f, 0xa30, 0xa21, 0xa2e, 0xa05,
0xa70, 0xa21, 0xa4b, 0xa30, 0xa3e, 0xa06, 0xa38, 0xa1f, 0xa4d, 0xa30, 0xa3e, 0xa16,
0xa3e, 0xa28, 0xa0f, 0xa25, 0xa28, 0xa38, 0xa2c, 0xa48, 0xa32, 0xa17, 0xa4d, 0xa30,
0xa47, 0xa21, 0xa2c, 0xa30, 0xa32, 0xa3f, 0xa28, 0xa2c, 0xa4d, 0xa30, 0xa3e, 0xa1f,
0xa3f, 0xa38, 0xa32, 0xa3e, 0xa35, 0xa3e, 0xa2c, 0xa30, 0xa71, 0xa38, 0xa32, 0xa1c,
0xa2c, 0xa42, 0xa15, 0xa3e, 0xa30, 0xa48, 0xa38, 0xa1f, 0xa2c, 0xa41, 0xa21, 0xa3e,
0xa2a, 0xa48, 0xa38, 0xa1f, 0xa2c, 0xa41, 0xa38, 0xa3f, 0xa70, 0xa1c, 0xa47, 0xa28,
0xa1a, 0xa3f, 0xa38, 0xa3f, 0xa28, 0xa4c, 0xa15, 0xa4b, 0xa2a, 0xa28, 0xa39, 0xa47,
0xa17, 0xa28, 0xa21, 0xa2c, 0xa32, 0xa3f, 0xa28, 0xa1c, 0xa3f, 0xa2c, 0xa30, 0xa3e,
0xa32, 0xa1f, 0xa30, 0xa17, 0xa30, 0xa28, 0xa1c, 0xa40, 0xa39, 0xa47, 0xa32, 0xa38,
0xa3f, 0xa70, 0xa15, 0xa40, 0xa06, 0xa07, 0xa32, 0x20, 0xa06, 0xa2b, 0x20, 0xa2e,
0xa48, 0xa28, 0xa07, 0xa38, 0xa24, 0xa3e, 0xa02, 0xa2c, 0xa41, 0xa32, 0xa1c, 0xa30,
0xa38, 0xa40, 0xa15, 0xa32, 0xa40, 0xa28, 0xa3f, 0xa70, 0xa17, 0xa4d, 0xa30, 0xa47,
0xa21, 0xa15, 0xa40, 0xa30, 0xa4b, 0xa35, 0xa15, 0xa40, 0xa35, 0xa32, 0xa3f, 0xa38,
0xa2c, 0xa28, 0xa32, 0xa1c, 0xa41, 0xa2c, 0xa32, 0xa1c, 0xa3e, 0xa28, 0xa3e, 0xa32,
0xa70, 0xa21, 0xa28, 0xa32, 0xa15, 0xa1c, 0xa3c, 0xa2e, 0xa2c, 0xa30, 0xa17, 0xa2e,
0xa48, 0xa21, 0xa4d, 0xa30, 0xa3f, 0xa21, 0xa2e, 0xa3e, 0xa32, 0xa1f, 0xa3e, 0xa2e,
0xa3e, 0xa30, 0xa40, 0xa0f, 0xa39, 0xa3e, 0xa2e, 0xa47, 0xa28, 0xa2e, 0xa3f, 0xa70,
0xa38, 0xa15, 0xa2e, 0xa4b, 0xa28, 0xa3e, 0xa15, 0xa4b, 0xa2e, 0xa3e, 0xa38, 0xa15,
0xa4b, 0xa13, 0xa38, 0xa32, 0xa4b, 0xa2a, 0xa48, 0xa30, 0xa3f, 0xa38, 0xa2a, 0xa4b,
0xa21, 0xa17, 0xa4b, 0xa30, 0xa40, 0xa15, 0xa3e, 0xa2a, 0xa4d, 0xa30, 0xa3e, 0xa17,
0xa30, 0xa3f, 0xa17, 0xa3e, 0xa38, 0xa48, 0xa28, 0x20, 0xa2e, 0xa30, 0xa40, 0xa28,
0xa4b, 0xa38, 0xa3e, 0xa30, 0xa3e, 0xa1c, 0xa47, 0xa35, 0xa4b, 0xa38, 0xa48, 0xa30,
0xa3e, 0xa1f, 0xa4b, 0xa35, 0xa38, 0xa3f, 0xa2e, 0xa2b, 0xa30, 0xa4b, 0xa2a, 0xa4b,
0xa32, 0xa38, 0xa15, 0xa4b, 0xa2a, 0xa1c, 0xa47, 0xa38, 0xa4b, 0xa2b, 0xa40, 0xa06,
0xa38, 0xa1f, 0xa3e, 0xa15, 0xa39, 0xa4b, 0xa2e, 0xa1f, 0xa71, 0xa32, 0xa3f, 0xa28,
0xa24, 0xa3f, 0xa30, 0xa3e, 0xa28, 0xa47, 0xa2f, 0xa41, 0xa32, 0xa3f, 0xa06, 0xa28,
0xa4b, 0xa38, 0xa15, 0xa35, 0xa3e, 0xa21, 0xa41, 0xa1c, 0xa3c, 0xa35, 0xa48, 0xa1f,
0xa3f, 0xa15, 0xa28, 0xa35, 0xa3f, 0xa06, 0xa28, 0xa3e, 0xa35, 0xa3f, 0xa32, 0xa28,
0xa3f, 0xa05, 0xa38, 0xa35, 0xa4b, 0xa32, 0xa17, 0xa4b, 0xa17, 0xa4d, 0xa30, 0xa47,
0xa21, 0xa35, 0xa3e, 0xa30, 0xa38, 0xa3e, 0xa05, 0xa1c, 0xa3c, 0xa17, 0xa30, 0xa47,
0xa2c, 0xa1c, 0xa3f, 0xa0a, 0xa30, 0xa3f, 0xa16, 0xa05, 0xa70, 0xa1f, 0xa3e, 0xa28,
0xa28, 0xa47, 0xa30, 0xa3f, 0xa35, 0xa4b, 0xa1a, 0xa3e, 0xa17, 0xa4b, 0xa38, 0xa15,
0xa4d, 0xa30, 0xa3f, 0xa38, 0xa2e, 0xa38, 0xa15, 0xa4b, 0xa15, 0xa4b, 0xa1c, 0xa3c,
0xa15, 0xa4b, 0xa2e, 0xa4b, 0xa30, 0xa4b, 0xa15, 0xa30, 0xa17, 0xa2f, 0xa42, 0xa32,
0xa47, 0xa28, 0xa2e, 0xa3e, 0xa39, 0xa47, 0xa2e, 0xa3e, 0xa32, 0xa26, 0xa40, 0xa35,
0xa2e, 0xa4c, 0xa30, 0xa3f, 0xa38, 0xa3c, 0xa38, 0xa2e, 0xa3e, 0xa2f, 0xa4b, 0xa1f,
0xa40, 0xa30, 0xa3f, 0xa2f, 0xa42, 0xa28, 0xa40, 0xa05, 0xa28, 0xa10, 0xa2a, 0xa40,
0xa06, 0xa06, 0xa15, 0xa32, 0xa48, 0xa02, 0xa21, 0xa2c, 0xa4b, 0xa17, 0xa28, 0xa35,
0xa3f, 0xa32, 0xa47, 0xa1a, 0xa48, 0xa25, 0xa2e, 0xa1a, 0xa42, 0xa15, 0xa08, 0xa38,
0xa1f, 0xa30, 0xa07, 0xa2b, 0xa47, 0xa1f, 0xa2b, 0xa15, 0xa3e, 0xa09, 0xa2b, 0xa4b,
0xa2b, 0xa3c, 0xa3f, 0xa1c, 0xa40, 0xa2b, 0xa41, 0xa28, 0xa3e, 0xa2b, 0xa41, 0xa1f,
0xa40, 0xa17, 0xa32, 0xa2a, 0xa47, 0xa17, 0xa4b, 0xa38, 0xa17, 0xa48, 0xa02, 0xa2c,
0xa40, 0xa05, 0xa30, 0xa17, 0xa41, 0xa06, 0xa21, 0xa3e, 0xa15, 0xa47, 0xa28, 0xa32,
0xa39, 0xa4b, 0xa28, 0xa4b, 0xa32, 0xa41, 0xa32, 0xa42, 0xa0f, 0xa02, 0xa21, 0xa30,
0xa2c, 0xa30, 0xa40, 0xa15, 0xa3f, 0xa30, 0xa3f, 0xa24, 0xa3f, 0xa2e, 0xa24, 0xa40,
0xa15, 0xa4b, 0xa38, 0xa4d, 0xa30, 0xa3e, 0xa0f, 0xa15, 0xa35, 0xa3e, 0xa1c, 0xa3e,
0xa32, 0xa40, 0xa28, 0xa2e, 0xa47, 0xa1c, 0xa41, 0xa30, 0xa4b, 0xa2e, 0xa3e, 0xa30,
0xa15, 0xa3f, 0xa38, 0xa3e, 0xa38, 0xa2e, 0xa3f, 0xa21, 0xa35, 0xa47, 0xa28, 0xa3e,
0xa09, 0xa30, 0xa42, 0xa28, 0xa3f, 0xa2f, 0xa42, 0xa28, 0xa4b, 0xa30, 0xa2b, 0xa4c,
0xa15, 0xa28, 0xa4c, 0xa2e, 0xa3f, 0xa06, 0xa2a, 0xa3e, 0xa17, 0xa4b, 0x20, 0xa2a,
0xa3e, 0xa17, 0xa4b, 0xa2a, 0xa32, 0xa3e, 0xa09, 0xa2a, 0xa3f, 0xa1f, 0xa15, 0xa48,
0xa30, 0xa28, 0xa2a, 0xa4b, 0xa39, 0xa28, 0xa2a, 0xa47, 0xa08, 0xa2a, 0xa4b, 0xa30,
0xa1f, 0x20, 0xa2e, 0xa4b, 0xa30, 0xa47, 0xa38, 0xa2c, 0xa40, 0xa30, 0xa3e, 0xa30,
0xa4b, 0xa1f, 0xa4b, 0xa02, 0xa17, 0xa3e, 0xa38, 0xa48, 0xa2a, 0xa3e, 0xa28, 0xa24,
0xa39, 0xa3f, 0xa24, 0xa40, 0xa1f, 0xa3e, 0xa30, 0xa3e, 0xa35, 0xa3e, 0xa1f, 0xa4b,
0xa02, 0xa17, 0xa3e, 0xa1f, 0xa3e, 0xa2a, 0xa42, 0xa35, 0xa47, 0xa15, 0xa35, 0xa3e,
0xa32, 0xa3f, 0xa38, 0x41, 0x62, 0x69, 0x79, 0xe1, 0x6e, 0x45, 0x6c, 0x20,
0x43, 0x61, 0x69, 0x72, 0x6f, 0x4a, 0x6f, 0x68, 0x61, 0x6e, 0x6e, 0x65,
0x73, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x54, 0x75, 0x6e, 0x65, 0x7a, 0x43,
0x75, 0x72, 0x61, 0x7a, 0x61, 0x6f, 0x43, 0x69, 0x75, 0x64, 0x61, 0x64,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f, 0x50, 0x75,
0x65, 0x72, 0x74, 0x6f, 0x20, 0x45, 0x73, 0x70, 0x61, 0xf1, 0x61, 0x53,
0x61, 0x6e, 0x20, 0x42, 0x61, 0x72, 0x74, 0x6f, 0x6c, 0x6f, 0x6d, 0xe9,
0x53, 0x61, 0x6e, 0x20, 0x4a, 0x75, 0x61, 0x6e, 0x20, 0x64, 0x65, 0x20,
0x54, 0x65, 0x72, 0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x53, 0x61, 0x6e,
0x20, 0x43, 0x72, 0x69, 0x73, 0x74, 0x6f, 0x62, 0x61, 0x6c, 0x53, 0x61,
0x6e, 0x74, 0x6f, 0x20, 0x54, 0x6f, 0x6d, 0x61, 0x73, 0x59, 0x61, 0x6b,
0x61, 0x72, 0x74, 0x61, 0x50, 0x69, 0x6f, 0x6e, 0x79, 0x61, 0x6e, 0x67,
0x4b, 0x6f, 0x73, 0x74, 0x61, 0x6e, 0xe1, 0x69, 0x53, 0x68, 0x61, 0x6e,
0x67, 0xe1, 0x69, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x53, 0x75, 0x72, 0x42, 0x72, 0x75, 0x73, 0x65, 0x6c,
0x61, 0x73, 0x45, 0x6c, 0x20, 0x56, 0x61, 0x74, 0x69, 0x63, 0x61, 0x6e,
0x6f, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x43,
0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x4e, 0x65, 0x77,
0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x41, 0x219,
0x67, 0x61, 0x62, 0x61, 0x74, 0x41, 0x74, 0xee, 0x72, 0x61, 0x75, 0x42,
0x69, 0x219, 0x6b, 0x65, 0x6b, 0x44, 0x75, 0x219, 0x61, 0x6e, 0x62, 0x65,
0x48, 0x6f, 0x20, 0x218, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x49, 0x72, 0x6b,
0x75, 0x21b, 0x6b, 0x49, 0x65, 0x72, 0x75, 0x73, 0x61, 0x6c, 0x69, 0x6d,
0x4b, 0x61, 0x6d, 0x63, 0x69, 0x61, 0x74, 0x6b, 0x61, 0x4b, 0x75, 0x77,
0x65, 0x69, 0x74, 0x4e, 0x6f, 0x76, 0x6f, 0x6b, 0x75, 0x7a, 0x6e, 0x65,
0x21b, 0x6b, 0x50, 0x68, 0x65, 0x6e, 0x69, 0x61, 0x6e, 0x54, 0x61, 0x219,
0x6b, 0x65, 0x6e, 0x74, 0x49, 0x61, 0x6b, 0x75, 0x21b, 0x6b, 0x43, 0x61,
0x6e, 0x61, 0x72, 0x65, 0x43, 0x61, 0x70, 0x75, 0x6c, 0x20, 0x56, 0x65,
0x72, 0x64, 0x65, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64,
0x65, 0x20, 0x53, 0x75, 0x64, 0x53, 0x66, 0x2e, 0x20, 0x45, 0x6c, 0x65,
0x6e, 0x61, 0x42, 0x75, 0x64, 0x61, 0x70, 0x65, 0x73, 0x74, 0x61, 0x49,
0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x4d, 0x61, 0x6e, 0x50, 0x6f, 0x64,
0x67, 0x6f, 0x72, 0x69, 0x21b, 0x61, 0x56, 0x61, 0x72, 0x219, 0x6f, 0x76,
0x69, 0x61, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x50, 0x61, 0x219,
0x74, 0x65, 0x6c, 0x75, 0x69, 0x4d, 0x61, 0x72, 0x63, 0x68, 0x69, 0x7a,
0x65, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x50, 0x69, 0x74, 0x63,
0x61, 0x69, 0x72, 0x6e, 0x41, 0x6c, 0x61, 0x73, 0x63, 0x61, 0x54, 0x68,
0x65, 0x20, 0x56, 0x61, 0x6c, 0x6c, 0x65, 0x79, 0x53, 0x61, 0x6e, 0x20,
0x53, 0x61, 0x6c, 0x76, 0x61, 0x64, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x20,
0x4a, 0x75, 0x6a, 0x75, 0x79, 0x53, 0x61, 0x6e, 0x20, 0x4a, 0x75, 0x61,
0x6e, 0x2c, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x69,
0x61, 0x4f, 0x72, 0x61, 0x6e, 0x6a, 0x65, 0x73, 0x74, 0x61, 0x64, 0x42,
0x65, 0x6c, 0x6d, 0x6f, 0x70, 0x61, 0x6e, 0x49, 0x6e, 0x73, 0x6c, 0x61,
0x73, 0x20, 0x43, 0x61, 0x79, 0x6d, 0x61, 0x6e, 0x43, 0x6f, 0x63, 0x6b,
0x62, 0x75, 0x72, 0x6e, 0x20, 0x54, 0x6f, 0x77, 0x6e, 0x42, 0x61, 0x73,
0x73, 0x65, 0x2d, 0x54, 0x65, 0x72, 0x72, 0x65, 0x43, 0x69, 0x74, 0x61,
0x64, 0x20, 0x64, 0x61, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x42,
0x72, 0x61, 0x64, 0x65, 0x73, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x28, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x29, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x20, 0x28, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c,
0x65, 0x6d, 0x29, 0x52, 0x6f, 0x61, 0x64, 0x20, 0x54, 0x6f, 0x77, 0x6e,
0x4d, 0x61, 0x63, 0x20, 0x4d, 0x75, 0x72, 0x64, 0x6f, 0x42, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x53, 0x65, 0x72, 0x69, 0x20, 0x42, 0x65, 0x67,
0x61, 0x77, 0x61, 0x6e, 0x43, 0x69, 0x74, 0x61, 0x64, 0x20, 0x64, 0x61,
0x20, 0x48, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68,
0x41, 0x7a, 0x6f, 0x72, 0x61, 0x73, 0x49, 0x6e, 0x73, 0x6c, 0x61, 0x73,
0x20, 0x43, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x61, 0x73, 0x49, 0x6e, 0x73,
0x6c, 0x61, 0x73, 0x20, 0x46, 0x65, 0x72, 0x6f, 0x65, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x61, 0x20, 0x64, 0x61, 0x6c, 0x20, 0x53, 0x69, 0x64,
0x53, 0x6f, 0x6e, 0x74, 0x67, 0x2019, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x50,
0x6f, 0x72, 0x74, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x6c, 0x65, 0x79, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x65, 0x74, 0x65, 0x72, 0x20, 0x50,
0x6f, 0x72, 0x74, 0x44, 0x6f, 0x75, 0x67, 0x6c, 0x61, 0x73, 0x53, 0x61,
0x69, 0x6e, 0x74, 0x20, 0x48, 0x65, 0x6c, 0x69, 0x65, 0x72, 0x4d, 0x6f,
0x73, 0x63, 0x61, 0x75, 0x54, 0x75, 0x72, 0x69, 0x74, 0x67, 0x46, 0x6c,
0x79, 0x69, 0x6e, 0x67, 0x20, 0x46, 0x69, 0x73, 0x68, 0x20, 0x43, 0x6f,
0x76, 0x65, 0x57, 0x65, 0x73, 0x74, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x43, 0x6f, 0x6d, 0x6f, 0x72, 0x61, 0x73, 0x57, 0x65, 0x6e, 0x6f,
0x49, 0x6e, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x61, 0x20, 0x50, 0x61, 0x73,
0x63, 0x61, 0x52, 0x69, 0x6b, 0x69, 0x74, 0x65, 0x61, 0x48, 0x6f, 0x6e,
0x69, 0x61, 0x72, 0x61, 0x48, 0x61, 0x67, 0xe5, 0x74, 0xf1, 0x61, 0x54,
0x6f, 0x66, 0x6f, 0x6c, 0x59, 0x61, 0x72, 0x65, 0x6e, 0x41, 0x6c, 0x6f,
0x66, 0x69, 0x4b, 0x69, 0x6e, 0x67, 0x73, 0x74, 0x6f, 0x6e, 0x4d, 0x65,
0x6c, 0x65, 0x6b, 0x6f, 0x6b, 0x50, 0x61, 0x6c, 0x69, 0x6b, 0x69, 0x72,
0x53, 0x6f, 0x75, 0x74, 0x68, 0x20, 0x54, 0x61, 0x72, 0x61, 0x77, 0x61,
0x4e, 0x75, 0x6b, 0x75, 0x2bb, 0x61, 0x6c, 0x6f, 0x66, 0x61, 0x4d, 0x61,
0x74, 0x101, 0x2bb, 0x75, 0x74, 0x75, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432,
0x430, 0x20, 0x41a, 0x430, 0x439, 0x43c, 0x430, 0x43d, 0x411, 0x43e, 0x439, 0x43b,
0x430, 0x2c, 0x20, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x44f, 0x20,
0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x2c,
0x20, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x44f, 0x20, 0x414, 0x430,
0x43a, 0x43e, 0x442, 0x430, 0x41d, 0x44c, 0x44e, 0x2d, 0x421, 0x435, 0x439, 0x43b,
0x435, 0x43c, 0x2c, 0x20, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x44f,
0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41f, 0x435, 0x442, 0x440, 0x43e,
0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x2d, 0x41a, 0x430, 0x43c,
0x447, 0x430, 0x442, 0x441, 0x43a, 0x438, 0x439, 0x43e, 0x2d, 0x432, 0x20, 0x421,
0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x43a,
0x438, 0x435, 0x20, 0x43e, 0x2d, 0x432, 0x430, 0x411, 0x435, 0x440, 0x43c, 0x443,
0x434, 0x441, 0x43a, 0x438, 0x435, 0x20, 0x43e, 0x2d, 0x432, 0x430, 0x41a, 0x430,
0x43d, 0x430, 0x440, 0x441, 0x43a, 0x438, 0x435, 0x20, 0x43e, 0x2d, 0x432, 0x430,
0x424, 0x430, 0x440, 0x435, 0x440, 0x441, 0x43a, 0x438, 0x435, 0x20, 0x43e, 0x2d,
0x432, 0x430, 0x42e, 0x436, 0x43d, 0x430, 0x44f, 0x20, 0x413, 0x435, 0x43e, 0x440,
0x433, 0x438, 0x44f, 0x43e, 0x2d, 0x432, 0x20, 0x421, 0x432, 0x44f, 0x442, 0x43e,
0x439, 0x20, 0x415, 0x43b, 0x435, 0x43d, 0x44b, 0x411, 0x44e, 0x437, 0x438, 0x43d,
0x433, 0x435, 0x43d, 0x2d, 0x43d, 0x430, 0x2d, 0x412, 0x435, 0x440, 0x445, 0x43d,
0x435, 0x43c, 0x2d, 0x420, 0x435, 0x439, 0x43d, 0x435, 0x43e, 0x2d, 0x432, 0x20,
0x41c, 0x44d, 0x43d, 0x43e, 0x2d, 0x432, 0x20, 0x420, 0x43e, 0x436, 0x434, 0x435,
0x441, 0x442, 0x432, 0x430, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x43e, 0x432, 0x44b,
0x435, 0x20, 0x43e, 0x2d, 0x432, 0x430, 0x41a, 0x43e, 0x43c, 0x43e, 0x440, 0x44b,
0x41c, 0x430, 0x43b, 0x44c, 0x434, 0x438, 0x432, 0x44b, 0x43e, 0x2d, 0x432, 0x20,
0x41f, 0x430, 0x441, 0x445, 0x438, 0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433,
0x43e, 0x441, 0x441, 0x43a, 0x438, 0x435, 0x20, 0x43e, 0x2d, 0x432, 0x430, 0x43e,
0x2d, 0x432, 0x430, 0x20, 0x413, 0x430, 0x43c, 0x431, 0x44c, 0x435, 0x43e, 0x2d,
0x432, 0x20, 0x42d, 0x43d, 0x434, 0x435, 0x440, 0x431, 0x435, 0x440, 0x438, 0x41c,
0x430, 0x440, 0x43a, 0x438, 0x437, 0x441, 0x43a, 0x438, 0x435, 0x20, 0x43e, 0x2d,
0x432, 0x430, 0x43e, 0x2d, 0x432, 0x430, 0x20, 0x41c, 0x438, 0x434, 0x443, 0x44d,
0x439, 0x410, 0x43d, 0x430, 0x430, 0x434, 0x44b, 0x440, 0x410, 0x441, 0x445, 0x430,
0x431, 0x430, 0x430, 0x442, 0x411, 0x430, 0x433, 0x434, 0x430, 0x430, 0x434, 0x411,
0x430, 0x43a, 0x443, 0x443, 0x427, 0x438, 0x442, 0x430, 0x430, 0x414, 0x430, 0x43c,
0x430, 0x430, 0x441, 0x414, 0x443, 0x431, 0x430, 0x430, 0x439, 0x425, 0x43e, 0x20,
0x428, 0x438, 0x20, 0x41c, 0x438, 0x43d, 0x44c, 0x418, 0x440, 0x43a, 0x443, 0x442,
0x441, 0x43a, 0x430, 0x439, 0x41a, 0x430, 0x431, 0x443, 0x443, 0x43b, 0x41a, 0x430,
0x43c, 0x447, 0x430, 0x430, 0x442, 0x43a, 0x430, 0x425, 0x430, 0x430, 0x43d, 0x434,
0x44b, 0x433, 0x430, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441,
0x43a, 0x430, 0x439, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x430, 0x43d, 0x41d,
0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x430, 0x439,
0x41e, 0x43c, 0x441, 0x43a, 0x430, 0x439, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438,
0x438, 0x43d, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x43a, 0x430, 0x430, 0x43d, 0x41e,
0x440, 0x442, 0x43e, 0x20, 0x425, 0x430, 0x43b, 0x44b, 0x43c, 0x430, 0x423, 0x43b,
0x430, 0x43d, 0x20, 0x411, 0x430, 0x430, 0x442, 0x430, 0x440, 0x423, 0x440, 0x443,
0x43c, 0x447, 0x443, 0x423, 0x443, 0x441, 0x20, 0x41d, 0x44c, 0x430, 0x440, 0x430,
0x414, 0x44c, 0x43e, 0x43a, 0x443, 0x443, 0x441, 0x43a, 0x430, 0x439, 0x415, 0x43a,
0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x443, 0x440, 0x410, 0x430,
0x441, 0x442, 0x440, 0x430, 0x445, 0x430, 0x43d, 0x421, 0x442, 0x430, 0x43c, 0x431,
0x443, 0x443, 0x43b, 0x41c, 0x43e, 0x441, 0x43a, 0x443, 0x431, 0x430, 0x421, 0x430,
0x43c, 0x430, 0x430, 0x440, 0x430, 0x423, 0x43b, 0x44c, 0x44f, 0x43d, 0x43e, 0x432,
0x441, 0x43a, 0x430, 0x439, 0x1c5f, 0x2e, 0x1c75, 0x1c64, 0x1c70, 0x1c61, 0x1c5f, 0x1c71,
0x1c5f, 0x1c70, 0x1c5f, 0x1c60, 0x1c5f, 0x1c71, 0x1c60, 0x1c77, 0x1c5a, 0x1c68, 0x1c6e, 0x1c61,
0x1c7d, 0x1c5f, 0x1c71, 0x1c5c, 0x1c69, 0x1c6d, 0x1c5e, 0x1c5f, 0x1c5f, 0x1c71, 0x1c74, 0x1c64,
0x1c5c, 0x1c69, 0x1c63, 0x1c5f, 0x1c5f, 0x1c68, 0x1c69, 0x1c75, 0x1c5f, 0x1c5f, 0x1c74, 0x1c64,
0x1c60, 0x1c5a, 0x1c60, 0x1c5f, 0x1c71, 0x1c75, 0x1c5f, 0x1c66, 0x1c64, 0x1c6d, 0x1c5f, 0x20,
0x1c75, 0x1c5f, 0x1c71, 0x1c70, 0x1c6e, 0x1c68, 0x1c5f, 0x1c65, 0x1c75, 0x1c5f, 0x1c68, 0x1c75,
0x1c5f, 0x1c70, 0x1c5a, 0x1c65, 0x1c75, 0x1c6e, 0x1c5e, 0x1c64, 0x1c61, 0x1c6e, 0x1c75, 0x1c5e,
0x1c5f, 0x1c71, 0x1c60, 0x1c7c, 0x1c65, 0x1c5f, 0x1c75, 0x1c5e, 0x1c5a, 0x1c71, 0x1c75, 0x1c5a,
0x1c6d, 0x1c64, 0x1c65, 0x1c60, 0x1c6e, 0x1c62, 0x1c75, 0x1c7d, 0x1c68, 0x1c6e, 0x1c61, 0x1c7d,
0x20, 0x1c75, 0x1c6e, 0x1c60, 0x1c5f, 0x1c71, 0x1c60, 0x1c69, 0x1c71, 0x1c60, 0x1c5a, 0x1c62,
0x1c5f, 0x1c71, 0x1c6a, 0x1c64, 0x1c60, 0x1c5f, 0x1c5c, 0x1c5a, 0x1c6a, 0x1c77, 0x1c64, 0x1c66,
0x1c69, 0x1c63, 0x1c5f, 0x1c66, 0x1c69, 0x1c63, 0x1c5f, 0x1c60, 0x1c5a, 0x1c65, 0x1c74, 0x1c5f,
0x20, 0x1c68, 0x1c64, 0x1c60, 0x1c5f, 0x1c60, 0x1c68, 0x1c6e, 0x1c65, 0x1c74, 0x1c5a, 0x1c71,
0x1c60, 0x1c6d, 0x1c69, 0x1c68, 0x1c5f, 0x1c60, 0x1c5f, 0x1c63, 0x1c70, 0x1c5f, 0x1c71, 0x1c62,
0x1c5f, 0x1c68, 0x1c60, 0x1c65, 0x1c5f, 0x1c63, 0x1c71, 0x1c70, 0x1c5f, 0x1c63, 0x1c65, 0x1c5a,
0x1c71, 0x1c70, 0x1c5f, 0x1c63, 0x1c65, 0x1c5a, 0x1c71, 0x20, 0x1c60, 0x1c68, 0x1c64, 0x1c60,
0x1c70, 0x1c6e, 0x1c71, 0x1c63, 0x1c6e, 0x1c68, 0x1c70, 0x1c6e, 0x1c74, 0x1c68, 0x1c5a, 0x1c6d,
0x1c6e, 0x1c74, 0x1c70, 0x1c5a, 0x1c62, 0x1c64, 0x1c71, 0x1c64, 0x1c60, 0x1c5f, 0x1c6e, 0x1c70,
0x1c62, 0x1c5a, 0x1c71, 0x1c74, 0x1c5a, 0x1c71, 0x1c6e, 0x1c5e, 0x20, 0x1c65, 0x1c5f, 0x1c5e,
0x1c63, 0x1c6e, 0x1c70, 0x1c5a, 0x1c68, 0x1c6f, 0x1c77, 0x1c5a, 0x1c74, 0x20, 0x1c71, 0x1c6e,
0x1c5e, 0x1c65, 0x1c5a, 0x1c71, 0x1c5c, 0x1c5e, 0x1c6e, 0x1c65, 0x20, 0x1c75, 0x1c6e, 0x1c5c,
0x1c69, 0x1c65, 0x20, 0x1c75, 0x1c6e, 0x1c5c, 0x1c7d, 0x1c68, 0x1c5f, 0x1c71, 0x1c70, 0x20,
0x1c74, 0x1c69, 0x1c68, 0x1c60, 0x1c5c, 0x1c7d, 0x1c68, 0x1c6e, 0x1c71, 0x1c5f, 0x1c70, 0x1c5f,
0x1c5c, 0x1c69, 0x1c63, 0x1c5f, 0x1c70, 0x1c6e, 0x1c5e, 0x1c5a, 0x1c6f, 0x1c5c, 0x1c69, 0x1c63,
0x1c5f, 0x1c74, 0x1c6e, 0x1c62, 0x1c5f, 0x1c5e, 0x1c5f, 0x1c66, 0x1c5f, 0x1c5e, 0x1c64, 0x1c6f,
0x1c77, 0x1c5f, 0x1c60, 0x1c65, 0x1c66, 0x1c5f, 0x1c63, 0x1c5f, 0x1c71, 0x1c5f, 0x1c66, 0x1c5f,
0x1c79, 0x1c68, 0x1c62, 0x1c5a, 0x1c65, 0x1c64, 0x1c5e, 0x1c5a, 0x1c64, 0x1c71, 0x1c70, 0x1c64,
0x1c6d, 0x1c5f, 0x1c71, 0x1c5f, 0x1c6f, 0x1c5a, 0x1c5e, 0x1c64, 0x1c65, 0x1c71, 0x1c5a, 0x1c60,
0x1c65, 0x2c, 0x20, 0x1c64, 0x1c71, 0x1c70, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x1c5f, 0x1c62,
0x1c6e, 0x1c68, 0x1c6e, 0x1c5d, 0x1c5c, 0x1c5a, 0x2c, 0x20, 0x1c64, 0x1c71, 0x1c70, 0x1c64,
0x1c6d, 0x1c5f, 0x1c71, 0x1c5f, 0x1c6f, 0x1c64, 0x1c74, 0x1c5a, 0x1c68, 0x1c65, 0x1c75, 0x1c5a,
0x1c68, 0x1c5c, 0x1c7d, 0x2c, 0x20, 0x1c64, 0x1c71, 0x1c70, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71,
0x1c5f, 0x1c74, 0x1c5a, 0x1c5e, 0x20, 0x1c65, 0x1c64, 0x1c74, 0x1c6d, 0x2c, 0x20, 0x1c64,
0x1c71, 0x1c70, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x1c5f, 0x1c75, 0x1c77, 0x1c6e, 0x1c75, 0x1c77,
0x1c5f, 0x1c6d, 0x2c, 0x20, 0x1c64, 0x1c71, 0x1c70, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x1c5f,
0x1c75, 0x1c77, 0x1c64, 0x1c71, 0x1c65, 0x1c6e, 0x1c71, 0x1c65, 0x2c, 0x20, 0x1c64, 0x1c71,
0x1c70, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x1c5f, 0x1c63, 0x1c64, 0x1c71, 0x1c5f, 0x1c62, 0x1c5f,
0x1c60, 0x2c, 0x20, 0x1c64, 0x1c71, 0x1c70, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x1c5f, 0x1c64,
0x1c71, 0x1c69, 0x1c63, 0x1c64, 0x1c60, 0x1c64, 0x1c60, 0x1c5f, 0x1c5e, 0x1c69, 0x1c63, 0x1c64,
0x1c5b, 0x1c61, 0x1c5f, 0x1c62, 0x1c5f, 0x1c6d, 0x1c60, 0x1c5f, 0x1c61, 0x1c69, 0x1c71, 0x1c6e,
0x1c5f, 0x1c79, 0x1c63, 0x1c5e, 0x1c69, 0x1c63, 0x1c64, 0x1c65, 0x1c63, 0x1c64, 0x1c5e, 0x1c6e,
0x1c62, 0x1c5a, 0x1c71, 0x1c74, 0x1c64, 0x1c65, 0x1c6e, 0x1c5e, 0x1c5a, 0x2c, 0x20, 0x1c60,
0x1c6e, 0x1c71, 0x1c74, 0x1c69, 0x1c60, 0x1c6d, 0x1c60, 0x1c68, 0x1c5f, 0x1c5e, 0x1c6e, 0x1c71,
0x1c70, 0x1c64, 0x1c61, 0x1c7d, 0x1c60, 0x1c5a, 0x1c5e, 0x1c6e, 0x1c65, 0x20, 0x1c6e, 0x1c67,
0x1c61, 0x1c6e, 0x1c5e, 0x1c6e, 0x1c65, 0x1c5e, 0x1c5a, 0x1c63, 0x1c5f, 0x1c79, 0x1c68, 0x20,
0x1c6f, 0x1c68, 0x1c64, 0x1c71, 0x1c65, 0x20, 0x1c60, 0x1c63, 0x1c5f, 0x1c74, 0x1c5f, 0x1c79,
0x1c68, 0x1c62, 0x1c5f, 0x1c71, 0x1c5f, 0x1c5c, 0x1c69, 0x1c63, 0x1c5f, 0x1c62, 0x1c5f, 0x1c68,
0x1c64, 0x1c5c, 0x1c5a, 0x1c74, 0x1c62, 0x1c5f, 0x1c68, 0x1c74, 0x1c64, 0x1c71, 0x1c60, 0x1c62,
0x1c5f, 0x1c74, 0x1c5f, 0x1c62, 0x1c5a, 0x1c68, 0x1c5a, 0x1c65, 0x1c62, 0x1c5f, 0x1c61, 0x1c5f,
0x1c74, 0x1c5e, 0x1c5f, 0x1c71, 0x1c62, 0x1c6e, 0x1c71, 0x1c5a, 0x1c62, 0x1c64, 0x1c71, 0x1c6e,
0x1c62, 0x1c6e, 0x1c68, 0x1c64, 0x1c70, 0x1c5f, 0x1c62, 0x1c6e, 0x1c74, 0x1c5e, 0x1c5f, 0x1c60,
0x1c5f, 0x1c74, 0x1c5e, 0x1c5f, 0x1c62, 0x1c6e, 0x1c60, 0x1c65, 0x1c64, 0x1c60, 0x1c5a, 0x20,
0x1c65, 0x1c5a, 0x1c66, 0x1c5a, 0x1c68, 0x1c62, 0x1c64, 0x1c60, 0x1c63, 0x1c6e, 0x1c5e, 0x1c5a,
0x1c71, 0x1c62, 0x1c5a, 0x1c71, 0x1c60, 0x1c74, 0x1c5a, 0x1c71, 0x1c62, 0x1c5a, 0x1c71, 0x1c74,
0x1c6e, 0x1c68, 0x1c6e, 0x1c6d, 0x1c62, 0x1c5a, 0x1c71, 0x1c74, 0x1c65, 0x1c6e, 0x1c68, 0x1c5f,
0x1c74, 0x1c71, 0x1c5f, 0x1c65, 0x1c5f, 0x1c63, 0x1c71, 0x1c64, 0x1c63, 0x20, 0x1c6d, 0x1c5a,
0x1c68, 0x1c60, 0x1c71, 0x1c5a, 0x1c62, 0x1c6e, 0x1c75, 0x1c6e, 0x1c63, 0x1c5e, 0x1c5f, 0x1c66,
0x2c, 0x20, 0x1c6e, 0x1c5b, 0x1c5a, 0x1c62, 0x20, 0x1c70, 0x1c5f, 0x1c60, 0x1c5a, 0x1c74,
0x1c5f, 0x1c65, 0x1c6e, 0x1c71, 0x1c74, 0x1c5a, 0x1c68, 0x2c, 0x20, 0x1c6e, 0x1c5b, 0x1c5a,
0x1c62, 0x20, 0x1c70, 0x1c5f, 0x1c60, 0x1c5a, 0x1c74, 0x1c5f, 0x1c71, 0x1c64, 0x1c63, 0x20,
0x1c65, 0x1c6e, 0x1c5e, 0x1c6e, 0x1c62, 0x2c, 0x20, 0x1c6e, 0x1c5b, 0x1c5a, 0x1c62, 0x20,
0x1c70, 0x1c5f, 0x1c60, 0x1c5a, 0x1c74, 0x1c5f, 0x1c71, 0x1c69, 0x1c60, 0x1c5a, 0x1c61, 0x1c64,
0x1c71, 0x1c5f, 0x1c5c, 0x1c5f, 0x1c6f, 0x1c5f, 0x1c71, 0x1c5f, 0x1c62, 0x1c5f, 0x1c6f, 0x1c77,
0x1c64, 0x1c71, 0x1c64, 0x1c60, 0x1c65, 0x1c6f, 0x1c5a, 0x1c68, 0x1c74, 0x20, 0x1c5f, 0x1c63,
0x20, 0x1c6f, 0x1c68, 0x1c64, 0x1c71, 0x1c65, 0x1c6f, 0x1c5a, 0x1c68, 0x1c74, 0x20, 0x1c5a,
0x1c6f, 0x1c77, 0x20, 0x1c65, 0x1c6f, 0x1c6e, 0x1c71, 0x1c6f, 0x1c6d, 0x1c69, 0x1c68, 0x1c74,
0x1c5a, 0x20, 0x1c68, 0x1c64, 0x1c60, 0x1c5a, 0x1c68, 0x1c5f, 0x1c71, 0x1c60, 0x1c64, 0x1c71,
0x20, 0x1c64, 0x1c71, 0x1c5e, 0x1c6e, 0x1c74, 0x1c68, 0x1c6e, 0x1c5c, 0x1c64, 0x1c6d, 0x1c5f,
0x1c71, 0x1c5f, 0x1c68, 0x1c64, 0x1c65, 0x1c5a, 0x1c5e, 0x1c6d, 0x1c69, 0x1c74, 0x1c65, 0x1c5f,
0x1c71, 0x1c74, 0x1c5a, 0x20, 0x1c70, 0x1c5a, 0x1c62, 0x1c64, 0x1c71, 0x1c5c, 0x1c5a, 0x1c64,
0x1c5b, 0x1c5a, 0x1c60, 0x1c5a, 0x1c5b, 0x1c5a, 0x1c68, 0x1c62, 0x1c64, 0x1c5b, 0x1c65, 0x1c64,
0x1c5b, 0x1c60, 0x1c5f, 0x1c6e, 0x1c65, 0x20, 0x1c74, 0x1c64, 0x20, 0x1c75, 0x1c5f, 0x1c68,
0x1c5b, 0x1c77, 0x1c6e, 0x1c5e, 0x1c62, 0x1c64, 0x1c6e, 0x1c65, 0x20, 0x1c74, 0x1c64, 0x20,
0x1c61, 0x1c5a, 0x1c71, 0x1c65, 0x1c6e, 0x1c65, 0x20, 0x1c74, 0x1c64, 0x20, 0x1c60, 0x1c64,
0x1c74, 0x1c65, 0x1c6e, 0x1c65, 0x20, 0x1c74, 0x1c64, 0x20, 0x1c5e, 0x1c69, 0x1c65, 0x1c64,
0x1c6d, 0x1c5f, 0x1c6e, 0x1c65, 0x20, 0x1c74, 0x1c64, 0x20, 0x1c5b, 0x1c77, 0x1c5a, 0x1c62,
0x1c5f, 0x1c65, 0x1c6e, 0x1c65, 0x20, 0x1c74, 0x1c64, 0x20, 0x1c63, 0x1c64, 0x1c71, 0x1c65,
0x1c6e, 0x1c74, 0x1c65, 0x1c63, 0x1c64, 0x1c6f, 0x1c77, 0x1c74, 0x20, 0x1c60, 0x1c5a, 0x1c68,
0x1c6e, 0x1c71, 0x1c74, 0x1c5b, 0x1c6e, 0x1c5c, 0x1c69, 0x1c6a, 0x1c64, 0x1c5c, 0x1c5f, 0x1c5e,
0x1c6f, 0x1c5f, 0x1c5b, 0x1c77, 0x1c69, 0x1c5e, 0x1c6e, 0x1c74, 0x1c64, 0x1c61, 0x1c69, 0x1c63,
0x1c5f, 0x1c71, 0x1c5f, 0x1c74, 0x1c5a, 0x1c68, 0x1c5a, 0x1c71, 0x1c74, 0x1c5a, 0x1c74, 0x1c5a,
0x1c68, 0x1c74, 0x1c5a, 0x1c5e, 0x1c5f, 0x1c63, 0x1c5f, 0x1c71, 0x1c60, 0x1c5a, 0x1c63, 0x1c5f,
0x1c79, 0x1c68, 0x1c63, 0x1c5f, 0x1c6d, 0x1c64, 0x1c74, 0x1c66, 0x1c5a, 0x1c68, 0x1c65, 0x1c63,
0x1c64, 0x1c71, 0x1c64, 0x1c6f, 0x1c6e, 0x1c5c, 0x1c7d, 0x1c6d, 0x1c5f, 0x1c60, 0x1c69, 0x1c5b,
0x1c5f, 0x1c5b, 0x1c75, 0x1c5f, 0x1c79, 0x1c68, 0x1c62, 0x1c69, 0x1c71, 0x1c70, 0x1c5f, 0x53,
0x75, 0x20, 0x43, 0xe0, 0x69, 0x72, 0x75, 0x4d, 0x6f, 0x67, 0x61, 0x64,
0xec, 0x73, 0x63, 0x69, 0x75, 0x50, 0x6f, 0x72, 0x74, 0x75, 0x2d, 0x4e,
0x6f, 0x75, 0x54, 0x72, 0xec, 0x70, 0x6f, 0x6c, 0x69, 0x54, 0xf9, 0x6e,
0x69, 0x73, 0x69, 0x53, 0x2019, 0x41, 0x76, 0x61, 0x6e, 0x61, 0x54, 0x7a,
0x69, 0x74, 0x61, 0x64, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20,
0x4d, 0xe8, 0x73, 0x73, 0x69, 0x63, 0x75, 0x4e, 0x6f, 0x61, 0x20, 0x59,
0x6f, 0x72, 0x6b, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20,
0x4e, 0x6f, 0x72, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75,
0x20, 0x4e, 0x6f, 0x72, 0x64, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c,
0x65, 0x6d, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x75, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x50, 0xe0, 0x6e,
0x61, 0x6d, 0x61, 0x50, 0x6f, 0x72, 0x74, 0x75, 0x20, 0x64, 0x65, 0x20,
0x49, 0x73, 0x70, 0x61, 0x67, 0x6e, 0x61, 0x53, 0x61, 0x6e, 0x74, 0x75,
0x20, 0x42, 0x61, 0x72, 0x74, 0x6f, 0x6c, 0x6f, 0x6d, 0x65, 0x75, 0x53,
0x61, 0x6e, 0x74, 0x75, 0x20, 0x47, 0x69, 0x75, 0x61, 0x6e, 0x6e, 0x65,
0x53, 0x61, 0x6e, 0x74, 0x75, 0x20, 0x43, 0x72, 0x69, 0x73, 0x74, 0x6f,
0x6c, 0x75, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x20, 0x4c, 0x75, 0x67, 0x68,
0x69, 0x61, 0x53, 0x61, 0x6e, 0x74, 0x75, 0x20, 0x54, 0x6f, 0x6d, 0x6d,
0x61, 0x73, 0x75, 0x53, 0x61, 0x6e, 0x74, 0x75, 0x20, 0x56, 0x69, 0x73,
0x73, 0x65, 0x6e, 0x74, 0x65, 0x54, 0x7a, 0x69, 0x74, 0x61, 0x64, 0x65,
0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d,
0x69, 0x6e, 0x68, 0x43, 0x61, 0x74, 0xe0, 0x72, 0x52, 0x69, 0x79, 0xe0,
0x64, 0x53, 0x65, 0xf9, 0x6c, 0x54, 0x65, 0x68, 0x65, 0x72, 0xe0, 0x6e,
0x55, 0x6c, 0xe0, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x41, 0x7a,
0x6f, 0x72, 0x72, 0x61, 0x73, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20,
0x43, 0x61, 0x6e, 0xe0, 0x72, 0x69, 0x61, 0x73, 0x43, 0x61, 0x62, 0x75,
0x20, 0x42, 0x69, 0x72, 0x64, 0x65, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x73,
0x20, 0x46, 0xf8, 0x72, 0x6f, 0x79, 0x61, 0x72, 0x47, 0x65, 0xf2, 0x72,
0x67, 0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x53, 0x75,
0x64, 0x53, 0x61, 0x6e, 0x74, 0x2019, 0x45, 0x6c, 0x65, 0x6e, 0x65, 0x42,
0x65, 0x6c, 0x67, 0x72, 0x61, 0x64, 0x75, 0x42, 0x65, 0x72, 0x6c, 0x69,
0x6e, 0x75, 0x42, 0xf9, 0x63, 0x61, 0x72, 0x65, 0x73, 0x74, 0x42, 0xf9,
0x64, 0x61, 0x70, 0x65, 0x73, 0x74, 0x44, 0x75, 0x62, 0x6c, 0x69, 0x6e,
0x75, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61,
0x6e, 0xcc, 0x73, 0x74, 0x61, 0x6e, 0x62, 0x75, 0x6c, 0x4d, 0xf2, 0x6e,
0x61, 0x63, 0x75, 0x53, 0x61, 0x6e, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x72,
0x69, 0x6e, 0x75, 0x49, 0x73, 0x74, 0x6f, 0x63, 0x6f, 0x6c, 0x6d, 0x61,
0x54, 0x7a, 0x69, 0x74, 0x61, 0x64, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73,
0x75, 0x20, 0x56, 0x61, 0x74, 0x69, 0x63, 0x61, 0x6e, 0x75, 0x56, 0x61,
0x72, 0x73, 0xe0, 0x76, 0x69, 0x61, 0x5a, 0x61, 0x67, 0xe0, 0x62, 0x72,
0x69, 0x61, 0x5a, 0x75, 0x72, 0x69, 0x67, 0x75, 0x4e, 0x61, 0x74, 0x69,
0x76, 0x69, 0x64, 0x61, 0x64, 0x65, 0x4d, 0x61, 0x75, 0x72, 0xec, 0x74,
0x7a, 0x69, 0x75, 0x73, 0x4d, 0x61, 0x69, 0x6f, 0x74, 0x61, 0x52, 0x65,
0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x65, 0x47, 0x75, 0xe0, 0x6d, 0x4d, 0x61,
0x72, 0x63, 0x68, 0x65, 0x73, 0x61, 0x73, 0x411, 0x443, 0x435, 0x43d, 0x43e,
0x441, 0x20, 0x410, 0x458, 0x440, 0x435, 0x441, 0x41a, 0x43e, 0x440, 0x430, 0x43b,
0x20, 0x425, 0x430, 0x440, 0x431, 0x443, 0x440, 0x41a, 0x430, 0x458, 0x43c, 0x430,
0x43d, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x41b,
0x443, 0x438, 0x432, 0x438, 0x43b, 0x435, 0x41b, 0x43e, 0x443, 0x435, 0x440, 0x20,
0x41f, 0x440, 0x438, 0x43d, 0x441, 0x438, 0x437, 0x20, 0x41a, 0x432, 0x43e, 0x440,
0x442, 0x435, 0x440, 0x413, 0x43e, 0x442, 0x445, 0x430, 0x431, 0x41f, 0x43e, 0x440,
0x442, 0x43e, 0x20, 0x420, 0x438, 0x43a, 0x43e, 0x421, 0x430, 0x43d, 0x442, 0x458,
0x430, 0x433, 0x43e, 0x421, 0x43a, 0x43e, 0x440, 0x435, 0x437, 0x431, 0x438, 0x441,
0x443, 0x43d, 0x434, 0x425, 0x43e, 0x43d, 0x433, 0x43a, 0x43e, 0x43d, 0x433, 0x423,
0x440, 0x443, 0x43c, 0x45b, 0x438, 0x424, 0x430, 0x440, 0x441, 0x43a, 0x430, 0x20,
0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x421, 0x432, 0x435, 0x442, 0x430, 0x20,
0x408, 0x435, 0x43b, 0x435, 0x43d, 0x430, 0x410, 0x434, 0x435, 0x43b, 0x435, 0x458,
0x434, 0x410, 0x441, 0x442, 0x440, 0x430, 0x43a, 0x430, 0x43d, 0x411, 0x438, 0x441,
0x438, 0x43d, 0x433, 0x435, 0x43d, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x43e, 0x20,
0x41c, 0x430, 0x43d, 0x412, 0x438, 0x43b, 0x45a, 0x443, 0x441, 0x412, 0x438, 0x432,
0x438, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x458, 0x430, 0x43d, 0x430, 0x412,
0x438, 0x43d, 0x441, 0x435, 0x43d, 0x441, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438,
0x458, 0x430, 0x43d, 0x430, 0x411, 0x458, 0x443, 0x43b, 0x430, 0x2c, 0x20, 0x421,
0x458, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x414, 0x430, 0x43a, 0x43e,
0x442, 0x430, 0x426, 0x435, 0x43d, 0x442, 0x430, 0x440, 0x2c, 0x20, 0x421, 0x458,
0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442,
0x430, 0x41d, 0x43e, 0x432, 0x438, 0x20, 0x421, 0x430, 0x43b, 0x435, 0x43c, 0x2c,
0x20, 0x421, 0x458, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x414, 0x430,
0x43a, 0x43e, 0x442, 0x430, 0x421, 0x432, 0x438, 0x444, 0x442, 0x20, 0x41a, 0x430,
0x440, 0x435, 0x43d, 0x442, 0x414, 0x438, 0x43c, 0x43e, 0x43d, 0x20, 0x434, 0x2019,
0x418, 0x440, 0x432, 0x438, 0x43b, 0x41b, 0x43e, 0x43d, 0x433, 0x458, 0x438, 0x440,
0x41d, 0x438, 0x458, 0x443, 0x435, 0x42, 0x61, 0x6d, 0x61, 0x6b, 0x6f, 0x42,
0x61, 0x6e, 0x67, 0x75, 0x69, 0x42, 0x61, 0x6e, 0x17e, 0x75, 0x6c, 0x42,
0x69, 0x73, 0x61, 0x6f, 0x42, 0x75, 0x64, 0x17e, 0x75, 0x6d, 0x62, 0x75,
0x72, 0x61, 0x4b, 0x61, 0x6d, 0x70, 0x61, 0x6c, 0x61, 0x4c, 0x75, 0x61,
0x6e, 0x64, 0x61, 0x4c, 0x75, 0x73, 0x61, 0x6b, 0x61, 0x4d, 0x61, 0x6c,
0x61, 0x62, 0x6f, 0x4d, 0x61, 0x70, 0x75, 0x74, 0x6f, 0x4d, 0x62, 0x61,
0x62, 0x61, 0x6e, 0x65, 0x4e, 0x69, 0x6a, 0x61, 0x6d, 0x65, 0x6a, 0x45,
0x6e, 0x6b, 0x6f, 0x72, 0x69, 0x64, 0x17e, 0x42, 0x75, 0x65, 0x6e, 0x6f,
0x73, 0x20, 0x41, 0x6a, 0x72, 0x65, 0x73, 0x17d, 0x75, 0x17e, 0x75, 0x69,
0x53, 0x61, 0x6e, 0x20, 0x48, 0x75, 0x61, 0x6e, 0x53, 0x61, 0x6e, 0x20,
0x4c, 0x75, 0x69, 0x55, 0x161, 0x75, 0x61, 0x69, 0x6a, 0x61, 0x41, 0x72,
0x75, 0x62, 0x61, 0x4b, 0x6f, 0x72, 0x61, 0x6c, 0x20, 0x48, 0x61, 0x72,
0x62, 0x75, 0x72, 0x42, 0x61, 0x69, 0x6a, 0x61, 0x20, 0x42, 0x61, 0x6e,
0x64, 0x65, 0x72, 0x61, 0x73, 0x42, 0x65, 0x6c, 0x69, 0x7a, 0x65, 0x42,
0x6c, 0x61, 0x6e, 0x6b, 0x2d, 0x53, 0x65, 0x6a, 0x62, 0x6c, 0x6f, 0x6e,
0x42, 0x6f, 0x61, 0x20, 0x56, 0x69, 0x73, 0x74, 0x61, 0x42, 0x6f, 0x67,
0x6f, 0x74, 0x61, 0x42, 0x6f, 0x6a, 0x7a, 0x69, 0x4b, 0x65, 0x6d, 0x62,
0x72, 0x69, 0x64, 0x17e, 0x20, 0x42, 0x65, 0x6a, 0x4b, 0x61, 0x6a, 0x6d,
0x61, 0x6e, 0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61,
0x10c, 0x69, 0x6b, 0x61, 0x67, 0x6f, 0x53, 0x69, 0x75, 0x64, 0x61, 0x64,
0x20, 0x48, 0x75, 0x61, 0x72, 0x65, 0x7a, 0x44, 0x61, 0x6e, 0x6d, 0x61,
0x72, 0x6b, 0x73, 0x68, 0x61, 0x67, 0x65, 0x6e, 0x45, 0x69, 0x72, 0x75,
0x6e, 0x65, 0x70, 0x65, 0x46, 0x6f, 0x72, 0x74, 0x61, 0x6c, 0x65, 0x7a,
0x61, 0x47, 0x6c, 0x65, 0x6a, 0x73, 0x20, 0x42, 0x65, 0x6a, 0x47, 0x75,
0x73, 0x20, 0x42, 0x65, 0x6a, 0x47, 0x72, 0x61, 0x6e, 0x64, 0x20, 0x54,
0x75, 0x72, 0x6b, 0x47, 0x72, 0x65, 0x6e, 0x61, 0x64, 0x61, 0x48, 0x65,
0x72, 0x6d, 0x6f, 0x73, 0x69, 0x6c, 0x6a, 0x6f, 0x49, 0x6e, 0x64, 0x69,
0x6a, 0x61, 0x6e, 0x61, 0x70, 0x6f, 0x6c, 0x69, 0x73, 0x4e, 0x6f, 0x6b,
0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x6a, 0x61, 0x6e, 0x61, 0x4d,
0x61, 0x72, 0x65, 0x6e, 0x67, 0x6f, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69,
0x6a, 0x61, 0x6e, 0x61, 0x50, 0x69, 0x74, 0x65, 0x72, 0x73, 0x62, 0x75,
0x72, 0x67, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x6a, 0x61, 0x6e, 0x61,
0x54, 0x65, 0x6c, 0x20, 0x53, 0x69, 0x74, 0x69, 0x2c, 0x20, 0x49, 0x6e,
0x64, 0x69, 0x6a, 0x61, 0x6e, 0x61, 0x56, 0x69, 0x76, 0x69, 0x2c, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x6a, 0x61, 0x6e, 0x61, 0x56, 0x69, 0x6e, 0x73,
0x65, 0x6e, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x6a, 0x61, 0x6e,
0x61, 0x56, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x6b, 0x2c, 0x20, 0x49, 0x6e,
0x64, 0x69, 0x6a, 0x61, 0x6e, 0x61, 0x49, 0x6b, 0x76, 0x61, 0x6c, 0x75,
0x69, 0x74, 0x17d, 0x75, 0x6e, 0x6f, 0x4c, 0x75, 0x69, 0x76, 0x69, 0x6c,
0x4d, 0x6f, 0x6e, 0x74, 0x69, 0x10d, 0x65, 0x6c, 0x6f, 0x2c, 0x20, 0x4b,
0x65, 0x6e, 0x74, 0x61, 0x6b, 0x69, 0x4b, 0x72, 0x61, 0x6c, 0x65, 0x6e,
0x64, 0x61, 0x6a, 0x6b, 0x4c, 0x61, 0x20, 0x50, 0x61, 0x7a, 0x4c, 0x6f,
0x73, 0x20, 0x41, 0x6e, 0x111, 0x65, 0x6c, 0x65, 0x73, 0x4c, 0x6f, 0x75,
0x65, 0x72, 0x20, 0x50, 0x72, 0x69, 0x6e, 0x73, 0x69, 0x7a, 0x20, 0x4b,
0x76, 0x6f, 0x72, 0x74, 0x65, 0x72, 0x4d, 0x61, 0x7a, 0x61, 0x74, 0x6c,
0x61, 0x6e, 0x4d, 0x65, 0x74, 0x6c, 0x61, 0x6b, 0x61, 0x74, 0x6c, 0x61,
0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x20, 0x53, 0x69, 0x74, 0x69,
0x4d, 0x6f, 0x6e, 0x74, 0x65, 0x76, 0x69, 0x64, 0x65, 0x6f, 0x4e, 0x6a,
0x75, 0x6a, 0x6f, 0x72, 0x6b, 0x42, 0x6a, 0x75, 0x6c, 0x61, 0x2c, 0x20,
0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x61, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e, 0x74, 0x61, 0x72, 0x2c, 0x20, 0x53,
0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x4e, 0x6f, 0x76, 0x69, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d,
0x2c, 0x20, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x61, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x47, 0x6f, 0x74, 0x68, 0x61, 0x62, 0x50,
0x61, 0x6e, 0x61, 0x6d, 0x61, 0x50, 0x61, 0x72, 0x61, 0x6d, 0x61, 0x72,
0x69, 0x62, 0x6f, 0x50, 0x6f, 0x72, 0x74, 0x2d, 0x6f, 0x2d, 0x50, 0x72,
0x65, 0x6e, 0x73, 0x50, 0x6f, 0x72, 0x74, 0x20, 0x6f, 0x76, 0x20, 0x53,
0x70, 0x65, 0x6a, 0x6e, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x20, 0x56, 0x65,
0x6c, 0x6a, 0x6f, 0x52, 0x65, 0x73, 0x69, 0x66, 0x65, 0x52, 0x65, 0x64,
0x17e, 0x61, 0x6a, 0x6e, 0x61, 0x52, 0x65, 0x7a, 0x6f, 0x6c, 0x75, 0x74,
0x53, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x65, 0x6d, 0x53, 0x61, 0x6e, 0x74,
0x6f, 0x20, 0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x67, 0x6f, 0x53, 0x61, 0x6f,
0x20, 0x50, 0x61, 0x6f, 0x6c, 0x6f, 0x53, 0x69, 0x74, 0x6b, 0x61, 0x53,
0x76, 0x2e, 0x20, 0x4c, 0x75, 0x63, 0x69, 0x6a, 0x61, 0x53, 0x65, 0x6e,
0x74, 0x20, 0x56, 0x69, 0x6e, 0x73, 0x65, 0x6e, 0x74, 0x54, 0x6f, 0x72,
0x6f, 0x6e, 0x74, 0x6f, 0x54, 0x6f, 0x72, 0x74, 0x6f, 0x6c, 0x61, 0x56,
0x61, 0x6a, 0x74, 0x68, 0x6f, 0x72, 0x73, 0x44, 0x69, 0x6d, 0x6f, 0x6e,
0x20, 0x64, 0x2019, 0x49, 0x72, 0x76, 0x69, 0x6c, 0x4d, 0x61, 0x6b, 0x6d,
0x75, 0x72, 0x64, 0x6f, 0x4c, 0x6f, 0x6e, 0x67, 0x6a, 0x69, 0x72, 0x46,
0x61, 0x6d, 0x61, 0x67, 0x75, 0x73, 0x74, 0x61, 0x4a, 0x65, 0x72, 0x75,
0x73, 0x61, 0x6c, 0x69, 0x6d, 0x4b, 0x75, 0x61, 0x6c, 0x61, 0x20, 0x4c,
0x75, 0x6d, 0x70, 0x75, 0x72, 0x4d, 0x61, 0x6e, 0x69, 0x6c, 0x61, 0x50,
0x6f, 0x6e, 0x74, 0x69, 0x6a, 0x61, 0x6e, 0x61, 0x6b, 0x53, 0x72, 0x65,
0x64, 0x6e, 0x6a, 0x65, 0x6b, 0x6f, 0x6c, 0x69, 0x6d, 0x73, 0x6b, 0x55,
0x72, 0x75, 0x6d, 0x107, 0x69, 0x55, 0x73, 0x74, 0x2d, 0x4e, 0x65, 0x72,
0x61, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x73, 0x6b, 0x61, 0x20, 0x6f, 0x73,
0x74, 0x72, 0x76, 0x61, 0x5a, 0x65, 0x6c, 0x65, 0x6e, 0x6f, 0x72, 0x74,
0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x46, 0x61,
0x72, 0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x4d,
0x61, 0x64, 0x65, 0x69, 0x72, 0x61, 0x4a, 0x75, 0x17e, 0x6e, 0x61, 0x20,
0x44, 0x17e, 0x6f, 0x72, 0x64, 0x17e, 0x69, 0x6a, 0x61, 0x53, 0x76, 0x65,
0x74, 0x61, 0x20, 0x4a, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x41, 0x64, 0x65,
0x6c, 0x65, 0x6a, 0x64, 0x42, 0x72, 0x69, 0x7a, 0x62, 0x65, 0x6a, 0x6e,
0x49, 0x75, 0x6b, 0x6c, 0x61, 0x42, 0x72, 0x61, 0x74, 0x69, 0x73, 0x6c,
0x61, 0x76, 0x61, 0x42, 0x69, 0x73, 0x69, 0x6e, 0x67, 0x65, 0x6e, 0x44,
0x17e, 0x65, 0x72, 0x7a, 0x69, 0x4c, 0x6a, 0x75, 0x62, 0x6c, 0x6a, 0x61,
0x6e, 0x61, 0x4d, 0x61, 0x6c, 0x74, 0x61, 0x4f, 0x73, 0x6c, 0x6f, 0x50,
0x6f, 0x64, 0x67, 0x6f, 0x72, 0x69, 0x63, 0x61, 0x52, 0x69, 0x67, 0x61,
0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x53, 0x61, 0x6e, 0x20, 0x4d, 0x61,
0x72, 0x69, 0x6e, 0x6f, 0x53, 0x61, 0x72, 0x61, 0x6a, 0x65, 0x76, 0x6f,
0x56, 0x69, 0x6c, 0x6e, 0x6a, 0x75, 0x73, 0x41, 0x6e, 0x74, 0x61, 0x6e,
0x61, 0x6e, 0x61, 0x72, 0x69, 0x76, 0x6f, 0x42, 0x75, 0x67, 0x61, 0x6e,
0x76, 0x69, 0x6c, 0x55, 0x73, 0x6b, 0x72, 0x161, 0x6e, 0x6a, 0x65, 0x20,
0x6f, 0x73, 0x74, 0x72, 0x76, 0x6f, 0x46, 0x61, 0x6b, 0x61, 0x6f, 0x66,
0x6f, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x6a, 0x65, 0x4b, 0x6f, 0x161, 0x72,
0x65, 0x4b, 0x76, 0x61, 0x64, 0x17e, 0x61, 0x6c, 0x65, 0x6a, 0x69, 0x6e,
0x4e, 0x61, 0x75, 0x72, 0x75, 0x4e, 0x69, 0x6a, 0x75, 0x65, 0x50, 0x61,
0x67, 0x6f, 0x20, 0x50, 0x61, 0x67, 0x6f, 0x50, 0x6f, 0x72, 0x74, 0x20,
0x4d, 0x6f, 0x72, 0x7a, 0x62, 0x69, 0x52, 0x61, 0x72, 0x6f, 0x74, 0x6f,
0x6e, 0x67, 0x61, 0x53, 0x61, 0x6a, 0x70, 0x61, 0x6e, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x74, 0x61, 0x70, 0x75, 0x56, 0x65, 0x76, 0x61, 0x6a, 0x2c,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x6a, 0x61, 0x6e, 0x61, 0x56, 0x69, 0x6e,
0x63, 0x65, 0x6e, 0x65, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x6a,
0x61, 0x6e, 0x61, 0x4c, 0x75, 0x69, 0x76, 0x69, 0x6c, 0x65, 0x42, 0x69,
0x6a, 0x75, 0x6c, 0x61, 0x2c, 0x20, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e,
0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e, 0x74,
0x61, 0x72, 0x2c, 0x20, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x61, 0x20,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x6f, 0x76, 0x69, 0x20, 0x53,
0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e,
0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x50, 0x6f, 0x72, 0x74,
0x20, 0x6f, 0x66, 0x20, 0x53, 0x70, 0x65, 0x6a, 0x6e, 0x50, 0x6f, 0x72,
0x74, 0x6f, 0x20, 0x52, 0x69, 0x6b, 0x6f, 0x52, 0x65, 0x67, 0x69, 0x6e,
0x61, 0x53, 0x6b, 0x6f, 0x72, 0x65, 0x7a, 0x62, 0x69, 0x73, 0x75, 0x6e,
0x64, 0x53, 0x76, 0x2e, 0x20, 0x42, 0x61, 0x72, 0x74, 0x6f, 0x6c, 0x6f,
0x6d, 0x65, 0x6a, 0x53, 0x76, 0x2e, 0x20, 0x44, 0x17e, 0x6f, 0x6e, 0x53,
0x76, 0x2e, 0x20, 0x54, 0x6f, 0x6d, 0x61, 0x44, 0x69, 0x6d, 0x6f, 0x6e,
0x20, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x4d, 0x65, 0x6b, 0x76,
0x6f, 0x72, 0x69, 0x4c, 0x6f, 0x6e, 0x67, 0x6a, 0x65, 0x72, 0x62, 0x6a,
0x65, 0x6e, 0x4e, 0x69, 0x75, 0x65, 0x43, 0x61, 0x73, 0x61, 0x62, 0x62,
0x6c, 0x61, 0x6e, 0x63, 0x61, 0x43, 0xe8, 0x75, 0x74, 0x61, 0x47, 0x69,
0x62, 0x62, 0x75, 0x74, 0x69, 0x4d, 0x75, 0x67, 0x61, 0x64, 0x69, 0x73,
0x63, 0x69, 0x75, 0x4e, 0x61, 0x69, 0x72, 0x6f, 0x62, 0x62, 0x69, 0x54,
0x72, 0xec, 0x70, 0x75, 0x6c, 0x69, 0x43, 0xf2, 0x72, 0x64, 0x75, 0x62,
0x61, 0x42, 0x69, 0x6c, 0x69, 0x73, 0x69, 0x43, 0x61, 0x6d, 0x70, 0x75,
0x20, 0x52, 0x61, 0x6e, 0x6e, 0x69, 0x43, 0x75, 0x73, 0x74, 0x61, 0x20,
0x52, 0x69, 0x63, 0x61, 0x44, 0x69, 0x74, 0x72, 0x6f, 0x69, 0x74, 0x44,
0x75, 0x6d, 0xec, 0x6e, 0x69, 0x63, 0x61, 0x43, 0x69, 0x74, 0xe0, 0x20,
0x64, 0xfb, 0x20, 0x4d, 0xe8, 0x73, 0x73, 0x69, 0x63, 0x75, 0x4d, 0x75,
0x6e, 0x74, 0x69, 0x72, 0x72, 0x65, 0x69, 0x4d, 0x75, 0x6e, 0x7a, 0x69,
0x72, 0x72, 0x61, 0x74, 0x75, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x4a, 0x6f,
0x72, 0x63, 0x61, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x69, 0x20, 0x54, 0x72, 0x61,
0x6d, 0x75, 0x6e, 0x74, 0x61, 0x6e, 0x61, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x75, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x69,
0x20, 0x54, 0x72, 0x61, 0x6d, 0x75, 0x6e, 0x74, 0x61, 0x6e, 0x61, 0x4e,
0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x69, 0x20, 0x54, 0x72, 0x61, 0x6d,
0x75, 0x6e, 0x74, 0x61, 0x6e, 0x61, 0x50, 0x6f, 0x72, 0x74, 0x75, 0x20,
0x64, 0x69, 0x20, 0x53, 0x70, 0x61, 0x67, 0x6e, 0x61, 0x50, 0x6f, 0x72,
0x74, 0x75, 0x20, 0x52, 0x69, 0x63, 0x75, 0x52, 0x69, 0x67, 0x67, 0x69,
0x6e, 0x61, 0x53, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x75, 0x6c, 0x75, 0x53,
0x61, 0x6e, 0x74, 0x61, 0x20, 0x4c, 0x75, 0x63, 0xec, 0x61, 0x41, 0x6d,
0x6d, 0xe0, 0x6e, 0x43, 0x75, 0x6c, 0x75, 0x6d, 0x6d, 0x75, 0x47, 0x61,
0x7a, 0x7a, 0x61, 0x47, 0x69, 0x72, 0x75, 0x73, 0x61, 0x6c, 0x65, 0x6d,
0x6d, 0x69, 0x4b, 0x75, 0x72, 0x6b, 0x61, 0x74, 0x61, 0x4d, 0x61, 0x73,
0x63, 0x61, 0x74, 0x69, 0x4e, 0x69, 0x63, 0x75, 0x73, 0xec, 0x61, 0x53,
0x61, 0x6d, 0x61, 0x72, 0x63, 0x61, 0x6e, 0x6e, 0x61, 0x53, 0x69, 0x75,
0x6c, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x69, 0x56, 0x6c,
0x61, 0x64, 0xec, 0x76, 0x75, 0x73, 0x74, 0x6f, 0x6b, 0x59, 0x65, 0x6b,
0x61, 0x74, 0x69, 0x72, 0x69, 0x6d, 0x6d, 0x75, 0x72, 0x67, 0x75, 0x41,
0x7a, 0x7a, 0x6f, 0x72, 0x72, 0x69, 0x42, 0x69, 0x72, 0x6d, 0x75, 0x64,
0x61, 0x43, 0x61, 0x70, 0x75, 0x20, 0x56, 0x69, 0x72, 0x64, 0x69, 0x47,
0x69, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64, 0x69, 0x20, 0x73, 0x63,
0x69, 0x72, 0x6f, 0x63, 0x63, 0x75, 0x53, 0x61, 0x6e, 0x74, 0x2019, 0xc8,
0x6c, 0x69, 0x6e, 0x61, 0x41, 0x64, 0x69, 0x6c, 0x61, 0x69, 0x64, 0x69,
0x42, 0x69, 0x72, 0x6c, 0x69, 0x6e, 0x75, 0x42, 0x72, 0x75, 0x73, 0x73,
0x65, 0x6c, 0x69, 0x44, 0x75, 0x62, 0x62, 0x6c, 0x69, 0x6e, 0x75, 0x47,
0x69, 0x62, 0x62, 0x69, 0x72, 0x74, 0x65, 0x72, 0x72, 0x61, 0xcc, 0x73,
0x75, 0x6c, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x61, 0x6e, 0x4c, 0xf2,
0x6e, 0x6e, 0x69, 0x72, 0x61, 0x4c, 0x75, 0x73, 0x73, 0x69, 0x6d, 0x62,
0x75, 0x72, 0x67, 0x75, 0x4d, 0x61, 0x75, 0x74, 0x61, 0x50, 0x61, 0x72,
0x69, 0x67, 0x67, 0x69, 0x53, 0x61, 0x72, 0x61, 0x6a, 0x65, 0x76, 0x75,
0x53, 0x74, 0x75, 0x63, 0x63, 0x6f, 0x72, 0x6d, 0x61, 0x56, 0xf2, 0x6c,
0x67, 0x75, 0x67, 0x72, 0x61, 0x64, 0x5a, 0x61, 0x67, 0x61, 0x62, 0x62,
0x72, 0x69, 0x61, 0x4e, 0x61, 0x74, 0x61, 0x6c, 0x69, 0x4d, 0x61, 0x72,
0x64, 0x69, 0x76, 0x69, 0x47, 0x61, 0x6c, 0xe0, 0x70, 0x61, 0x67, 0x6f,
0x73, 0x4b, 0x14d, 0x6e, 0x61, 0x6b, 0x72, 0x79, 0x4d, 0x79, 0x6e, 0x64,
0x6f, 0x7a, 0x61, 0x41, 0x73, 0x75, 0x144, 0x63, 0x69, 0x6f, 0x6e, 0x4b,
0x61, 0x6a, 0x79, 0x6e, 0x6e, 0x61, 0x44, 0x14d, 0x6d, 0x69, 0x6e, 0x69,
0x6b, 0x61, 0x47, 0x72, 0x79, 0x6e, 0x61, 0x64, 0x61, 0x47, 0x77, 0x61,
0x74, 0x79, 0x6d, 0x61, 0x6c, 0x61, 0x4d, 0x61, 0x63, 0x65, 0x69, 0x14d,
0x4d, 0x79, 0x6e, 0x6f, 0x6d, 0x69, 0x6e, 0x65, 0x65, 0x4d, 0x14d, 0x6e,
0x74, 0x73, 0x65, 0x72, 0x72, 0x61, 0x74, 0x42, 0x65, 0x75, 0x6c, 0x61,
0x68, 0x2c, 0x20, 0x50, 0x14d, 0x142, 0x6e, 0x6f, 0x63, 0x6e, 0x6f, 0x20,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72,
0x2c, 0x20, 0x50, 0x14d, 0x142, 0x6e, 0x6f, 0x63, 0x6e, 0x6f, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c,
0x65, 0x6d, 0x2c, 0x20, 0x50, 0x14d, 0x142, 0x6e, 0x6f, 0x63, 0x6e, 0x6f,
0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x53, 0x61, 0x69, 0x6e, 0x74,
0x2d, 0x42, 0x61, 0x72, 0x74, 0x68, 0xe9, 0x6c, 0x79, 0x6d, 0x79, 0x4c,
0x6f, 0x6e, 0x67, 0x79, 0x65, 0x61, 0x72, 0x62, 0x79, 0x79, 0x6e, 0x41,
0x64, 0x79, 0x6e, 0x4b, 0x6f, 0x6c, 0x14d, 0x6d, 0x62, 0x6f, 0x48, 0x65,
0x62, 0x72, 0x14d, 0x6e, 0x48, 0x14d, 0x6e, 0x67, 0x6b, 0x14d, 0x6e, 0x67,
0xd4, 0x6d, 0x73, 0x6b, 0xd4, 0x72, 0x61, 0x6c, 0x50, 0x6a, 0x14d, 0x6e,
0x67, 0x6a, 0x61, 0x6e, 0x67, 0x54, 0x61, 0x73, 0x7a, 0x6b, 0x79, 0x6e,
0x74, 0x54, 0x14d, 0x6d, 0x73, 0x6b, 0x57, 0x69, 0x79, 0x6e, 0x74, 0x69,
0x61, 0x6e, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0x4b, 0x61, 0x6e, 0x61,
0x72, 0x79, 0x6a, 0x73, 0x6b, 0x65, 0x52, 0x65, 0x70, 0x75, 0x62, 0x6c,
0x69, 0x6b, 0x61, 0x20, 0x5a, 0x69, 0x65, 0x6c, 0x14d, 0x6e, 0x65, 0x67,
0x6f, 0x20, 0x50, 0x72, 0x7a, 0x69, 0x6c, 0x14d, 0x6e, 0x64, 0x6b, 0x61,
0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0xd4, 0x77, 0x63, 0x7a, 0x65, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x50, 0x6f, 0x142, 0x65, 0x64,
0x6e, 0x69, 0x6f, 0x77, 0x6f, 0x15a, 0x77, 0x69, 0x79, 0x6e, 0x74, 0x6f,
0x20, 0x48, 0x65, 0x6c, 0x79, 0x6e, 0x61, 0x41, 0x74, 0x79, 0x6e, 0x79,
0x4b, 0x69, 0x73, 0x7a, 0x79, 0x6e, 0x69, 0x14d, 0x77, 0x4b, 0x6f, 0x70,
0x79, 0x6e, 0x68, 0x61, 0x67, 0x61, 0x4b, 0x69, 0x6a, 0x14d, 0x77, 0x4c,
0x69, 0x7a, 0x62, 0x14d, 0x6e, 0x61, 0x4c, 0x14d, 0x6e, 0x64, 0x79, 0x6e,
0x4c, 0x75, 0x6b, 0x73, 0x79, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x4d, 0x14d,
0x6e, 0x61, 0x6b, 0x6f, 0xd4, 0x73, 0x6c, 0x6f, 0x53, 0x61, 0x72, 0x61,
0x74, 0x14d, 0x77, 0x57, 0x69, 0x65, 0x64, 0x79, 0x144, 0x47, 0x6f, 0x64,
0x6e, 0x69, 0x6f, 0x20, 0x57, 0x79, 0x73, 0x70, 0x61, 0x4b, 0x14d, 0x6d,
0x6f, 0x72, 0x79, 0x4b, 0x65, 0x72, 0x67, 0x75, 0x65, 0x6c, 0x65, 0x6e,
0x6f, 0x77, 0x65, 0x20, 0x57, 0x79, 0x73, 0x70, 0x79, 0x42, 0x6f, 0x75,
0x67, 0x61, 0x69, 0x6e, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x2019, 0x6f, 0x77,
0x61, 0x20, 0x57, 0x79, 0x73, 0x70, 0x61, 0x57, 0x79, 0x73, 0x70, 0x61,
0x20, 0x57, 0x69, 0x65, 0x6c, 0x6b, 0x61, 0x6e, 0x6f, 0x63, 0x6e, 0x6f,
0x627, 0x628, 0x64a, 0x20, 0x62c, 0x627, 0x646, 0x627, 0x64a, 0x6aa, 0x631, 0x627,
0x627, 0x64a, 0x68a, 0x64a, 0x633, 0x20, 0x627, 0x628, 0x628, 0x627, 0x627, 0x644,
0x62c, 0x64a, 0x631, 0x632, 0x627, 0x632, 0x645, 0x627, 0x631, 0x627, 0x628, 0x627,
0x645, 0x627, 0x6aa, 0x648, 0x628, 0x646, 0x6af, 0x64a, 0x628, 0x64a, 0x646, 0x62c,
0x627, 0x644, 0x628, 0x633, 0x627, 0x626, 0x648, 0x628, 0x644, 0x646, 0x67d, 0x627,
0x626, 0x631, 0x628, 0x648, 0x62c, 0x645, 0x628, 0x648, 0x631, 0x627, 0x642, 0x627,
0x626, 0x631, 0x627, 0x6aa, 0x627, 0x633, 0x627, 0x628, 0x644, 0x627, 0x646, 0x6aa,
0x627, 0x633, 0x64a, 0x648, 0x67d, 0x627, 0x6aa, 0x648, 0x646, 0x627, 0x6aa, 0x631,
0x64a, 0x68a, 0x627, 0x6aa, 0x627, 0x631, 0x627, 0x644, 0x20, 0x627, 0x64a, 0x648,
0x646, 0x641, 0x631, 0x64a, 0x20, 0x67d, 0x627, 0x626, 0x648, 0x646, 0x6af, 0x64a,
0x628, 0x648, 0x631, 0x648, 0x646, 0x62c, 0x648, 0x647, 0x627, 0x646, 0x633, 0x628,
0x631, 0x6af, 0x6aa, 0x645, 0x67e, 0x627, 0x644, 0x627, 0x6aa, 0x64a, 0x6af, 0x627,
0x644, 0x64a, 0x6aa, 0x646, 0x634, 0x627, 0x633, 0x627, 0x644, 0x6cc, 0x628, 0x631,
0x648, 0x6cc, 0x644, 0x6cc, 0x644, 0x648, 0x627, 0x646, 0x68a, 0x627, 0x644, 0x648,
0x628, 0x645, 0x628, 0x627, 0x634, 0x6cc, 0x644, 0x648, 0x633, 0x627, 0x6aa, 0x627,
0x645, 0x644, 0x627, 0x628, 0x648, 0x645, 0x628, 0x627, 0x628, 0x64a, 0x646, 0x645,
0x648, 0x63a, 0x627, 0x62f, 0x6cc, 0x634, 0x648, 0x646, 0x648, 0x627, 0x6aa, 0x634,
0x648, 0x637, 0x622, 0x626, 0x648, 0x6af, 0x648, 0x20, 0x68a, 0x627, 0x626, 0x648,
0x6af, 0x648, 0x67e, 0x648, 0x631, 0x67d, 0x648, 0x20, 0x646, 0x648, 0x648, 0x633,
0x627, 0x626, 0x648, 0x20, 0x67d, 0x648, 0x645, 0x67d, 0x631, 0x67e, 0x648, 0x644,
0x64a, 0x62a, 0x64a, 0x648, 0x646, 0x633, 0x648, 0x646, 0x68a, 0x647, 0x648, 0x6aa,
0x627, 0x62f, 0x6aa, 0x627, 0x64a, 0x646, 0x6aa, 0x631, 0x64a, 0x62c, 0x627, 0x646,
0x6af, 0x648, 0x64a, 0x644, 0x627, 0x627, 0x64a, 0x646, 0x67d, 0x64a, 0x6af, 0x648,
0x627, 0x627, 0x631, 0x627, 0x6af, 0x648, 0x6cc, 0x627, 0x646, 0x627, 0x628, 0x64a,
0x648, 0x646, 0x648, 0x633, 0x20, 0x627, 0x64a, 0x626, 0x631, 0x633, 0x6aa, 0x64a,
0x67d, 0x627, 0x20, 0x645, 0x627, 0x631, 0x6aa, 0x627, 0x6aa, 0x627, 0x631, 0x68a,
0x648, 0x628, 0x627, 0x6aa, 0x648, 0x6aa, 0x648, 0x626, 0x64a, 0x644, 0x627, 0x20,
0x631, 0x6cc, 0x626, 0x648, 0x62c, 0x627, 0x645, 0x64a, 0x646, 0x68a, 0x648, 0x632,
0x627, 0x631, 0x6cc, 0x648, 0x20, 0x6af, 0x627, 0x644, 0x6cc, 0x6af, 0x648, 0x633,
0x633, 0x627, 0x644, 0x67d, 0x627, 0x633, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x622,
0x646, 0x633, 0x627, 0x646, 0x20, 0x644, 0x648, 0x626, 0x6cc, 0x633, 0x67d, 0x648,
0x6aa, 0x648, 0x645, 0x64a, 0x646, 0x627, 0x648, 0x634, 0x648, 0x622, 0x626, 0x6cc,
0x627, 0x627, 0x633, 0x627, 0x646, 0x633, 0x6cc, 0x648, 0x646, 0x627, 0x67d, 0x64a,
0x6aa, 0x648, 0x6aa, 0x646, 0x628, 0x627, 0x62d, 0x64a, 0x647, 0x628, 0x627, 0x647,
0x6cc, 0x627, 0x20, 0x628, 0x6cc, 0x646, 0x62f, 0x631, 0x627, 0x633, 0x628, 0x627,
0x631, 0x628, 0x627, 0x68a, 0x648, 0x633, 0x628, 0x6cc, 0x644, 0x645, 0x628, 0x6cc,
0x644, 0x6cc, 0x632, 0x628, 0x644, 0x627, 0x646, 0x6aa, 0x20, 0x633, 0x628, 0x644,
0x648, 0x646, 0x628, 0x627, 0x626, 0x648, 0x20, 0x648, 0x633, 0x62a, 0x627, 0x628,
0x6af, 0x648, 0x67d, 0x627, 0x628, 0x648, 0x626, 0x633, 0x6aa, 0x64a, 0x645, 0x628,
0x631, 0x62c, 0x20, 0x628, 0x64a, 0x6aa, 0x64a, 0x645, 0x67e, 0x648, 0x20, 0x6af,
0x631, 0x627, 0x646, 0x68a, 0x64a, 0x6aa, 0x627, 0x646, 0x6aa, 0x648, 0x646, 0x6aa,
0x631, 0x627, 0x6aa, 0x633, 0x6aa, 0x627, 0x626, 0x64a, 0x646, 0x633, 0x64a, 0x627,
0x645, 0x646, 0x634, 0x6aa, 0x627, 0x6af, 0x648, 0x686, 0x64a, 0x20, 0x647, 0x648,
0x627, 0x20, 0x647, 0x648, 0x627, 0x633, 0x648, 0x68a, 0x64a, 0x68a, 0x20, 0x62c,
0x648, 0x631, 0x64a, 0x632, 0x6aa, 0x648, 0x633, 0x67d, 0x627, 0x20, 0x631, 0x64a,
0x6aa, 0x627, 0x6aa, 0x631, 0x64a, 0x633, 0x67d, 0x646, 0x6aa, 0x648, 0x626, 0x64a,
0x627, 0x628, 0x627, 0x6aa, 0x64a, 0x648, 0x631, 0x627, 0x633, 0x627, 0x626, 0x648,
0x68a, 0x64a, 0x646, 0x645, 0x627, 0x631, 0x6aa, 0x20, 0x634, 0x648, 0x646, 0x68a,
0x627, 0x648, 0x633, 0x646, 0x68a, 0x627, 0x648, 0x633, 0x646, 0x20, 0x6aa, 0x631,
0x64a, 0x6aa, 0x68a, 0x64a, 0x646, 0x648, 0x631, 0x68a, 0x64a, 0x67d, 0x631, 0x627,
0x626, 0x64a, 0x67d, 0x68a, 0x648, 0x645, 0x64a, 0x646, 0x64a, 0x6aa, 0x627, 0x627,
0x64a, 0x68a, 0x645, 0x648, 0x646, 0x67d, 0x646, 0x627, 0x64a, 0x631, 0x648, 0x646,
0x64a, 0x67e, 0x627, 0x64a, 0x644, 0x20, 0x633, 0x644, 0x648, 0x627, 0x68a, 0x648,
0x631, 0x641, 0x648, 0x631, 0x67d, 0x20, 0x646, 0x64a, 0x644, 0x633, 0x646, 0x641,
0x648, 0x631, 0x67d, 0x627, 0x644, 0x64a, 0x632, 0x627, 0x6af, 0x644, 0x64a, 0x633,
0x20, 0x628, 0x64a, 0x6af, 0x631, 0x627, 0x646, 0x68a, 0x20, 0x62a, 0x631, 0x6aa,
0x6af, 0x631, 0x64a, 0x646, 0x627, 0x68a, 0x627, 0x6af, 0x648, 0x627, 0x68a, 0x64a,
0x644, 0x648, 0x67e, 0x6af, 0x648, 0x626, 0x67d, 0x64a, 0x20, 0x645, 0x627, 0x644,
0x627, 0x6af, 0x648, 0x627, 0x64a, 0x627, 0x6aa, 0x64a, 0x644, 0x6af, 0x64a, 0x627,
0x646, 0x627, 0x647, 0x64a, 0x644, 0x64a, 0x20, 0x641, 0x64a, 0x6aa, 0x633, 0x647,
0x631, 0x645, 0x648, 0x633, 0x644, 0x648, 0x627, 0x646, 0x68a, 0x64a, 0x627, 0x646,
0x627, 0x20, 0x67e, 0x648, 0x644, 0x633, 0x646, 0x627, 0x6aa, 0x633, 0x60c, 0x20,
0x627, 0x646, 0x68a, 0x64a, 0x627, 0x646, 0x627, 0x645, 0x631, 0x64a, 0x646, 0x6af,
0x648, 0x60c, 0x20, 0x627, 0x646, 0x68a, 0x64a, 0x627, 0x646, 0x627, 0x67e, 0x64a,
0x67d, 0x631, 0x633, 0x628, 0x631, 0x6af, 0x60c, 0x20, 0x627, 0x646, 0x68a, 0x64a,
0x627, 0x646, 0x627, 0x67d, 0x64a, 0x644, 0x20, 0x633, 0x67d, 0x64a, 0x60c, 0x20,
0x627, 0x646, 0x68a, 0x64a, 0x627, 0x646, 0x627, 0x648, 0x64a, 0x648, 0x64a, 0x60c,
0x20, 0x627, 0x646, 0x68a, 0x64a, 0x627, 0x646, 0x627, 0x648, 0x646, 0x633, 0x6cc,
0x646, 0x6cc, 0x632, 0x60c, 0x20, 0x627, 0x646, 0x68a, 0x6cc, 0x627, 0x646, 0x627,
0x648, 0x64a, 0x646, 0x627, 0x645, 0x64a, 0x6aa, 0x60c, 0x20, 0x627, 0x646, 0x68a,
0x64a, 0x627, 0x646, 0x627, 0x627, 0x646, 0x648, 0x648, 0x6aa, 0x627, 0x6aa, 0x627,
0x644, 0x648, 0x626, 0x67d, 0x62c, 0x645, 0x627, 0x626, 0x6aa, 0x627, 0x62c, 0x648,
0x646, 0x626, 0x648, 0x644, 0x648, 0x626, 0x64a, 0x20, 0x648, 0x64a, 0x644, 0x645,
0x648, 0x646, 0x67d, 0x64a, 0x633, 0x64a, 0x644, 0x648, 0x60c, 0x20, 0x6aa, 0x64a,
0x646, 0x67d, 0x6aa, 0x64a, 0x6aa, 0x631, 0x627, 0x644, 0x64a, 0x646, 0x68a, 0x6aa,
0x644, 0x627, 0x20, 0x67e, 0x632, 0x644, 0x627, 0x633, 0x20, 0x627, 0x64a, 0x646,
0x62c, 0x644, 0x633, 0x644, 0x648, 0x626, 0x631, 0x20, 0x67e, 0x631, 0x646, 0x633,
0x632, 0x20, 0x6aa, 0x648, 0x627, 0x631, 0x67d, 0x631, 0x645, 0x6cc, 0x633, 0x6cc,
0x626, 0x648, 0x645, 0x646, 0x627, 0x626, 0x648, 0x633, 0x645, 0x64a, 0x631, 0x64a,
0x6af, 0x648, 0x67d, 0x645, 0x627, 0x631, 0x67d, 0x64a, 0x646, 0x6aa, 0x645, 0x62a,
0x627, 0x645, 0x648, 0x631, 0x648, 0x633, 0x645, 0x632, 0x627, 0x67d, 0x644, 0x627,
0x646, 0x645, 0x6cc, 0x646, 0x648, 0x645, 0x6cc, 0x646, 0x64a, 0x645, 0x64a, 0x631,
0x64a, 0x68a, 0x627, 0x645, 0x64a, 0x67d, 0x644, 0x627, 0x6aa, 0x627, 0x67d, 0x627,
0x644, 0x627, 0x645, 0x64a, 0x6aa, 0x633, 0x64a, 0x6aa, 0x648, 0x20, 0x634, 0x647,
0x631, 0x645, 0x6cc, 0x6aa, 0x648, 0x626, 0x6cc, 0x644, 0x648, 0x646, 0x645, 0x627,
0x646, 0x6aa, 0x67d, 0x646, 0x645, 0x627, 0x646, 0x67d, 0x64a, 0x631, 0x64a, 0x645,
0x648, 0x646, 0x67d, 0x64a, 0x20, 0x648, 0x64a, 0x68a, 0x64a, 0x648, 0x645, 0x627,
0x646, 0x67d, 0x633, 0x631, 0x64a, 0x67d, 0x646, 0x64a, 0x648, 0x64a, 0x627, 0x631,
0x6aa, 0x646, 0x648, 0x631, 0x627, 0x646, 0x647, 0x64a, 0x627, 0x628, 0x64a, 0x648,
0x644, 0x627, 0x647, 0x60c, 0x20, 0x627, 0x62a, 0x631, 0x20, 0x68a, 0x6aa, 0x648,
0x67d, 0x627, 0x633, 0x64a, 0x646, 0x67d, 0x631, 0x60c, 0x20, 0x627, 0x62a, 0x631,
0x20, 0x68a, 0x6aa, 0x648, 0x67d, 0x627, 0x646, 0x64a, 0x648, 0x20, 0x633, 0x64a,
0x644, 0x645, 0x60c, 0x20, 0x627, 0x62a, 0x631, 0x20, 0x68a, 0x6aa, 0x648, 0x67d,
0x627, 0x646, 0x64a, 0x648, 0x6aa, 0x627, 0x648, 0x6aa, 0x64a, 0x646, 0x627, 0x6af,
0x627, 0x67e, 0x646, 0x627, 0x645, 0x627, 0x67e, 0x64a, 0x631, 0x627, 0x645, 0x64a,
0x631, 0x64a, 0x628, 0x648, 0x641, 0x648, 0x646, 0x64a, 0x6aa, 0x633, 0x67e, 0x648,
0x631, 0x67d, 0x20, 0x627, 0x648, 0x20, 0x67e, 0x631, 0x646, 0x633, 0x627, 0x633,
0x67e, 0x64a, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x67b, 0x64a, 0x67d, 0x67e, 0x648,
0x631, 0x67d, 0x648, 0x20, 0x648, 0x64a, 0x644, 0x647, 0x648, 0x67e, 0x648, 0x631,
0x67d, 0x648, 0x20, 0x631, 0x64a, 0x6aa, 0x648, 0x67e, 0x646, 0x67d, 0x627, 0x20,
0x627, 0x631, 0x64a, 0x646, 0x627, 0x633, 0x631, 0x64a, 0x646, 0x6aa, 0x646, 0x20,
0x627, 0x646, 0x644, 0x64a, 0x67d, 0x647, 0x64a, 0x633, 0x64a, 0x641, 0x64a, 0x631,
0x6cc, 0x62c, 0x6cc, 0x646, 0x627, 0x631, 0x64a, 0x632, 0x648, 0x644, 0x64a, 0x648,
0x67d, 0x631, 0x64a, 0x648, 0x20, 0x628, 0x631, 0x627, 0x646, 0x6aa, 0x648, 0x633,
0x646, 0x67d, 0x627, 0x631, 0x64a, 0x645, 0x633, 0x64a, 0x646, 0x67d, 0x64a, 0x627,
0x6af, 0x648, 0x633, 0x64a, 0x646, 0x67d, 0x648, 0x20, 0x68a, 0x648, 0x645, 0x64a,
0x646, 0x6af, 0x648, 0x633, 0x627, 0x626, 0x648, 0x20, 0x67e, 0x648, 0x644, 0x648,
0x627, 0x67d, 0x648, 0x6aa, 0x648, 0x631, 0x67d, 0x648, 0x645, 0x627, 0x626, 0x67d,
0x633, 0x67d, 0x6aa, 0x627, 0x633, 0x64a, 0x646, 0x67d, 0x20, 0x628, 0x631, 0x67f,
0x644, 0x64a, 0x645, 0x64a, 0x633, 0x64a, 0x646, 0x67d, 0x20, 0x62c, 0x627, 0x646,
0x632, 0x633, 0x64a, 0x646, 0x67d, 0x20, 0x6aa, 0x67d, 0x633, 0x633, 0x64a, 0x646,
0x67d, 0x20, 0x644, 0x648, 0x633, 0x64a, 0x627, 0x633, 0x64a, 0x646, 0x67d, 0x20,
0x67f, 0x627, 0x645, 0x633, 0x633, 0x64a, 0x646, 0x67d, 0x20, 0x648, 0x646, 0x633,
0x64a, 0x646, 0x67d, 0x633, 0x648, 0x626, 0x641, 0x67d, 0x20, 0x6aa, 0x631, 0x646,
0x67d, 0x67d, 0x64a, 0x6af, 0x648, 0x633, 0x64a, 0x6af, 0x644, 0x67e, 0x627, 0x67f,
0x648, 0x644, 0x64a, 0x62a, 0x64a, 0x62c, 0x648, 0x627, 0x646, 0x627, 0x67d, 0x648,
0x631, 0x646, 0x67d, 0x648, 0x67d, 0x648, 0x631, 0x67d, 0x648, 0x644, 0x627, 0x648,
0x64a, 0x646, 0x6aa, 0x648, 0x648, 0x631, 0x648, 0x627, 0x626, 0x64a, 0x67d, 0x20,
0x647, 0x627, 0x626, 0x648, 0x633, 0x648, 0x646, 0x64a, 0x20, 0x67e, 0x64a, 0x6af,
0x64a, 0x627, 0x6aa, 0x648, 0x62a, 0x627, 0x62a, 0x6aa, 0x64a, 0x633, 0x64a, 0x68a,
0x64a, 0x648, 0x633, 0x68a, 0x648, 0x645, 0x627, 0x646, 0x67d, 0x20, 0x62f, 0x631,
0x648, 0x64a, 0x626, 0x644, 0x645, 0x6aa, 0x648, 0x627, 0x626, 0x631, 0x64a, 0x645,
0x648, 0x633, 0x646, 0x645, 0x64a, 0x6aa, 0x20, 0x645, 0x631, 0x68a, 0x648, 0x67e,
0x627, 0x645, 0x631, 0x631, 0x648, 0x68c, 0x631, 0x627, 0x633, 0x64a, 0x648, 0x648,
0x627, 0x67d, 0x631, 0x648, 0x644, 0x648, 0x648, 0x633, 0x62a, 0x648, 0x6aa, 0x644,
0x627, 0x646, 0x6af, 0x20, 0x627, 0x626, 0x64a, 0x631, 0x628, 0x646, 0x627, 0x644,
0x645, 0x627, 0x62a, 0x64a, 0x627, 0x646, 0x64a, 0x62f, 0x631, 0x627, 0x6aa, 0x67d,
0x627, 0x626, 0x648, 0x627, 0x64a, 0x6aa, 0x67d, 0x648, 0x628, 0x622, 0x634, 0x6af,
0x627, 0x628, 0x627, 0x62f, 0x622, 0x62a, 0x64a, 0x631, 0x627, 0x626, 0x648, 0x628,
0x626, 0x646, 0x6aa, 0x627, 0x6aa, 0x628, 0x631, 0x646, 0x644, 0x628, 0x634, 0x6aa,
0x64a, 0x6aa, 0x628, 0x631, 0x648, 0x646, 0x627, 0x626, 0x64a, 0x6aa, 0x648, 0x644,
0x645, 0x628, 0x648, 0x68d, 0x627, 0x6aa, 0x627, 0x62f, 0x628, 0x626, 0x64a, 0x647,
0x648, 0x686, 0x64a, 0x20, 0x645, 0x646, 0x647, 0x627, 0x646, 0x6af, 0x20, 0x6aa,
0x627, 0x646, 0x6af, 0x647, 0x648, 0x648, 0x68a, 0x627, 0x631, 0x6aa, 0x62a, 0x633,
0x6aa, 0x62c, 0x6aa, 0x627, 0x631, 0x62a, 0x627, 0x62c, 0x64a, 0x627, 0x67e, 0x648,
0x631, 0x627, 0x64a, 0x648, 0x631, 0x648, 0x634, 0x644, 0x645, 0x6aa, 0x627, 0x628,
0x644, 0x6aa, 0x645, 0x686, 0x627, 0x633, 0x6aa, 0x64a, 0x6aa, 0x631, 0x627, 0x686,
0x64a, 0x6a9, 0x67d, 0x645, 0x646, 0x68a, 0x648, 0x6a9, 0x646, 0x62f, 0x64a, 0x627,
0x6af, 0x627, 0x6aa, 0x644, 0x6aa, 0x62a, 0x627, 0x6aa, 0x631, 0x64a, 0x633, 0x646,
0x648, 0x64a, 0x627, 0x631, 0x633, 0x6aa, 0x6aa, 0x648, 0x627, 0x644, 0x627, 0x644,
0x645, 0x67e, 0x648, 0x631, 0x6a9, 0x648, 0x686, 0x646, 0x6af, 0x6aa, 0x648, 0x64a,
0x62a, 0x645, 0x6aa, 0x627, 0x626, 0x648, 0x645, 0x6af, 0x627, 0x62f, 0x627, 0x646,
0x645, 0x64a, 0x6aa, 0x627, 0x633, 0x631, 0x645, 0x646, 0x64a, 0x644, 0x627, 0x646,
0x64a, 0x6aa, 0x648, 0x633, 0x64a, 0x627, 0x646, 0x648, 0x648, 0x6aa, 0x632, 0x646,
0x64a, 0x62a, 0x633, 0x6aa, 0x646, 0x648, 0x648, 0x627, 0x633, 0x628, 0x626, 0x64a,
0x631, 0x633, 0x6aa, 0x627, 0x648, 0x645, 0x633, 0x6aa, 0x632, 0x628, 0x627, 0x646,
0x64a, 0x641, 0x646, 0x627, 0x645, 0x20, 0x67e, 0x64a, 0x646, 0x647, 0x67e, 0x648,
0x646, 0x67d, 0x64a, 0x627, 0x646, 0x6aa, 0x634, 0x64a, 0x627, 0x646, 0x6af, 0x20,
0x64a, 0x627, 0x646, 0x6af, 0x6aa, 0x648, 0x67d, 0x627, 0x646, 0x633, 0x64a, 0x6aa,
0x64a, 0x632, 0x644, 0x648, 0x631, 0x68a, 0x627, 0x634, 0x62e, 0x627, 0x644, 0x64a,
0x646, 0x633, 0x64a, 0x648, 0x644, 0x634, 0x646, 0x6af, 0x647, 0x627, 0x626, 0x64a,
0x633, 0x631, 0x64a, 0x62f, 0x64a, 0x6aa, 0x648, 0x644, 0x64a, 0x645, 0x633, 0x6aa,
0x62a, 0x627, 0x626, 0x67e, 0x64a, 0x62a, 0x627, 0x634, 0x642, 0x646, 0x62a, 0x67f,
0x645, 0x641, 0x648, 0x67d, 0x648, 0x6aa, 0x64a, 0x648, 0x62a, 0x645, 0x633, 0x6aa,
0x627, 0x648, 0x644, 0x627, 0x646, 0x20, 0x628, 0x627, 0x62a, 0x631, 0x64a, 0x648,
0x631, 0x645, 0x6aa, 0x64a, 0x627, 0x633, 0x67d, 0x20, 0x646, 0x64a, 0x631, 0x627,
0x648, 0x64a, 0x646, 0x67d, 0x64a, 0x627, 0x646, 0x648, 0x644, 0x627, 0x62f, 0x648,
0x648, 0x633, 0x62a, 0x648, 0x6aa, 0x64a, 0x627, 0x6aa, 0x62a, 0x633, 0x6aa, 0x631,
0x646, 0x6af, 0x648, 0x646, 0x6cc, 0x6aa, 0x627, 0x67d, 0x631, 0x646, 0x628, 0x631,
0x6af, 0x627, 0x632, 0x648, 0x631, 0x632, 0x6aa, 0x64a, 0x646, 0x631, 0x64a, 0x6aa,
0x64a, 0x67e, 0x20, 0x648, 0x631, 0x68a, 0x64a, 0x641, 0x64a, 0x631, 0x648, 0x645,
0x627, 0x68a, 0x64a, 0x631, 0x627, 0x631, 0x64a, 0x6aa, 0x64a, 0x648, 0x6aa, 0x68f,
0x6a9, 0x6bb, 0x20, 0x62c, 0x648, 0x631, 0x62c, 0x64a, 0x627, 0x633, 0x64a, 0x646,
0x67d, 0x20, 0x647, 0x64a, 0x644, 0x64a, 0x646, 0x627, 0x627, 0x633, 0x67d, 0x64a,
0x646, 0x644, 0x64a, 0x627, 0x64a, 0x68a, 0x64a, 0x644, 0x64a, 0x68a, 0x628, 0x631,
0x633, 0x628, 0x64a, 0x646, 0x628, 0x631, 0x648, 0x6aa, 0x646, 0x20, 0x647, 0x644,
0x68a, 0x627, 0x631, 0x648, 0x646, 0x64a, 0x648, 0x6aa, 0x644, 0x627, 0x647, 0x648,
0x628, 0x627, 0x631, 0x67d, 0x644, 0x646, 0x68a, 0x645, 0x64a, 0x646, 0x644, 0x627,
0x631, 0x68a, 0x20, 0x647, 0x627, 0x626, 0x648, 0x67e, 0x631, 0x67f, 0x633, 0x68a,
0x646, 0x64a, 0x627, 0x64a, 0x645, 0x633, 0x67d, 0x631, 0x68a, 0x64a, 0x645, 0x627,
0x64a, 0x67f, 0x646, 0x632, 0x628, 0x631, 0x627, 0x62a, 0x64a, 0x633, 0x644, 0x648,
0x627, 0x628, 0x631, 0x633, 0x644, 0x632, 0x628, 0x62e, 0x627, 0x631, 0x64a, 0x633,
0x67d, 0x628, 0x62f, 0x627, 0x67e, 0x64a, 0x633, 0x67d, 0x628, 0x632, 0x64a, 0x62c,
0x64a, 0x646, 0x686, 0x633, 0x64a, 0x646, 0x627, 0x626, 0x648, 0x6aa, 0x648, 0x67e,
0x646, 0x20, 0x647, 0x64a, 0x6af, 0x646, 0x68a, 0x628, 0x644, 0x646, 0x62c, 0x628,
0x631, 0x627, 0x644, 0x67d, 0x631, 0x6af, 0x631, 0x646, 0x632, 0x64a, 0x647, 0x64a,
0x644, 0x633, 0x646, 0x6aa, 0x64a, 0x622, 0x626, 0x64a, 0x632, 0x644, 0x20, 0x622,
0x641, 0x20, 0x645, 0x64a, 0x646, 0x62c, 0x631, 0x633, 0x64a, 0x6aa, 0x644, 0x64a,
0x646, 0x646, 0x20, 0x6af, 0x631, 0x627, 0x68a, 0x6aa, 0x64a, 0x631, 0x648, 0x641,
0x6aa, 0x650, 0x64a, 0x648, 0x644, 0x633, 0x628, 0x646, 0x644, 0x628, 0x644, 0x64a,
0x627, 0x646, 0x627, 0x644, 0x646, 0x68a, 0x646, 0x644, 0x6af, 0x632, 0x645, 0x628,
0x631, 0x6af, 0x645, 0x64a, 0x68a, 0x631, 0x68a, 0x645, 0x627, 0x644, 0x67d, 0x627,
0x645, 0x646, 0x633, 0x6aa, 0x645, 0x648, 0x646, 0x627, 0x6aa, 0x648, 0x645, 0x627,
0x633, 0x6aa, 0x648, 0x67e, 0x626, 0x631, 0x633, 0x67e, 0x648, 0x68a, 0x6af, 0x648,
0x631, 0x633, 0x64a, 0x627, 0x633, 0x64a, 0x646, 0x20, 0x645, 0x631, 0x64a, 0x646,
0x648, 0x633, 0x631, 0x627, 0x62c, 0x6cc, 0x648, 0x648, 0x633, 0x631, 0x627, 0x62a,
0x648, 0x633, 0x645, 0x641, 0x631, 0x648, 0x67e, 0x648, 0x644, 0x627, 0x633, 0x6aa,
0x648, 0x67e, 0x64a, 0x627, 0x633, 0x67d, 0x627, 0x6aa, 0x20, 0x647, 0x648, 0x645,
0x67d, 0x627, 0x644, 0x646, 0x62a, 0x631, 0x627, 0x646, 0x64a, 0x627, 0x644, 0x64a,
0x627, 0x646, 0x648, 0x633, 0x6aa, 0x648, 0x68a, 0x648, 0x632, 0x648, 0x626, 0x67d,
0x64a, 0x6aa, 0x646, 0x648, 0x64a, 0x644, 0x646, 0x64a, 0x633, 0x648, 0x648, 0x644,
0x6af, 0x648, 0x6af, 0x631, 0x627, 0x62f, 0x627, 0x646, 0x62a, 0x627, 0x646, 0x627,
0x646, 0x627, 0x631, 0x64a, 0x648, 0x648, 0x6aa, 0x631, 0x633, 0x645, 0x633, 0x6aa,
0x648, 0x6aa, 0x648, 0x633, 0x6aa, 0x648, 0x645, 0x648, 0x631, 0x648, 0x6aa, 0x631,
0x6af, 0x64a, 0x648, 0x644, 0x64a, 0x646, 0x645, 0x648, 0x631, 0x64a, 0x634, 0x633,
0x645, 0x64a, 0x627, 0x67d, 0x64a, 0x622, 0x6aa, 0x644, 0x64a, 0x646, 0x68a, 0x628,
0x648, 0x6af, 0x64a, 0x646, 0x20, 0x648, 0x64a, 0x644, 0x64a, 0x627, 0x686, 0x626,
0x67f, 0x645, 0x686, 0x64a, 0x648, 0x6aa, 0x627, 0x64a, 0x633, 0x67d, 0x631, 0x627,
0x641, 0x627, 0x62a, 0x64a, 0x641, 0x6aa, 0x627, 0x626, 0x648, 0x641, 0x648, 0x641,
0x646, 0x627, 0x641, 0x648, 0x67d, 0x64a, 0x6af, 0x627, 0x644, 0x627, 0x67e, 0x627,
0x6af, 0x648, 0x632, 0x6af, 0x64a, 0x645, 0x628, 0x64a, 0x626, 0x631, 0x6af, 0x648,
0x627, 0x68a, 0x644, 0x20, 0x6aa, 0x64a, 0x646, 0x627, 0x644, 0x647, 0x648, 0x646,
0x648, 0x20, 0x644, 0x648, 0x644, 0x648, 0x627, 0x64a, 0x646, 0x68a, 0x631, 0x628,
0x631, 0x64a, 0x6aa, 0x631, 0x64a, 0x67d, 0x645, 0x67d, 0x64a, 0x6aa, 0x648, 0x633,
0x631, 0x627, 0x626, 0x64a, 0x6aa, 0x648, 0x627, 0x62c, 0x644, 0x64a, 0x646, 0x645,
0x631, 0x6aa, 0x64a, 0x648, 0x633, 0x633, 0x645, 0x68a, 0x648, 0x64a, 0x646, 0x627,
0x626, 0x631, 0x648, 0x646, 0x64a, 0x648, 0x648, 0x64a, 0x646, 0x627, 0x631, 0x20,
0x641, 0x648, 0x6aa, 0x646, 0x627, 0x626, 0x648, 0x645, 0x64a, 0x627, 0x67e, 0x627,
0x6af, 0x648, 0x20, 0x67e, 0x627, 0x6af, 0x648, 0x67e, 0x644, 0x627, 0x626, 0x648,
0x67e, 0x67d, 0x6aa, 0x64a, 0x631, 0x646, 0x67e, 0x648, 0x646, 0x67e, 0x6cc, 0x626,
0x64a, 0x67e, 0x648, 0x631, 0x67d, 0x20, 0x645, 0x648, 0x631, 0x633, 0x628, 0x64a,
0x631, 0x627, 0x631, 0x648, 0x67d, 0x648, 0x646, 0x6af, 0x627, 0x633, 0x64a, 0x67e,
0x646, 0x62a, 0x627, 0x647, 0x67d, 0x64a, 0x62a, 0x631, 0x627, 0x648, 0x627, 0x67d,
0x648, 0x646, 0x6af, 0x627, 0x67d, 0x627, 0x67e, 0x648, 0x648, 0x64a, 0x6aa, 0x648,
0x64a, 0x644, 0x633, 0xd85, 0xdb6, 0xdd2, 0xda2, 0xdcf, 0xdb1, 0xdca, 0xd85, 0xd9a,
0xdca, 0xdbb, 0xdcf, 0xd85, 0xda9, 0xdd2, 0xdc3, 0xdca, 0x20, 0xd85, 0xdb6, 0xdcf,
0xdb6, 0xdcf, 0xd87, 0xdbd, 0xdca, 0xda2, 0xdd2, 0xdba, 0xdbb, 0xdca, 0xdc3, 0xdca,
0xd85, 0xdc3, 0xdca, 0xdb8, 0xdcf, 0xdbb, 0xdcf, 0xdb6, 0xdb8, 0xd9a, 0xddc, 0xdb6,
0xdb1, 0xdca, 0xd9c, 0xdd4, 0xdba, 0xdd2, 0xdb6, 0xdb1, 0xdca, 0xda2, 0xdd4, 0xdbd,
0xdca, 0xdb6, 0xdd2, 0xdc3, 0xddd, 0xdb6, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xda7, 0xdba,
0xdbb, 0xdca, 0xdb6, 0xdca, 0x200d, 0xdbb, 0xdc3, 0xdcf, 0xdc0, 0xdd2, 0xdbd, 0xdca,
0xdb6, 0xdd4, 0xda2, 0xdd4, 0xdb8, 0xdca, 0xdb6, 0xdd4, 0xdbb, 0xdcf, 0xd9a, 0xdba,
0xdd2, 0xdbb, 0xddd, 0xd9a, 0xdd0, 0xdc3, 0xdb6, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xd9a,
0xdcf, 0xdc3, 0xdd9, 0xdba, 0xdd4, 0xda7, 0xdcf, 0xd9a, 0xddc, 0xdb1, 0xd9a, 0xdca,
0x200d, 0xdbb, 0xdd2, 0xda9, 0xd9a, 0xdbb, 0xdca, 0xdaf, 0xdcf, 0xdbb, 0xdca, 0x20,
0xd91, 0xdc3, 0xdca, 0x20, 0xdc3, 0xdbd, 0xdcf, 0xdb8, 0xdca, 0xda2, 0xdd2, 0xdb6,
0xdd4, 0xda7, 0xdd2, 0xdaf, 0xdde, 0xd85, 0xdbd, 0xdcf, 0xd91, 0xdbd, 0xdca, 0x20,
0xd86, 0xdba, 0xdd4, 0xdb1, 0xdca, 0xdc6, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xda7, 0xdc0,
0xdd4, 0xdb1, 0xdca, 0xd9c, 0xdd0, 0xdb6, 0xdbb, 0xddd, 0xdb1, 0xdca, 0xdc4, 0xdbb,
0xdcf, 0xdbb, 0xdda, 0xda2, 0xddc, 0xdc4, 0xdb1, 0xdca, 0xdb1, 0xdc3, 0xdca, 0xdb6,
0xdbb, 0xdca, 0xd9c, 0xdca, 0xda2, 0xdd4, 0xdb6, 0xdcf, 0xd9a, 0xdb8, 0xdca, 0xdb4,
0xdcf, 0xdbd, 0xdcf, 0xd9a, 0xdcf, 0xdbb, 0xdca, 0xda7, 0xddc, 0xdb8, 0xdca, 0xd9a,
0xdd2, 0xd9c, 0xdcf, 0xdbd, 0xdd2, 0xd9a, 0xdd2, 0xdb1, 0xdca, 0xdc1, 0xdcf, 0xdc3,
0xdcf, 0xdbd, 0xdcf, 0xd9c, 0xddd, 0xdc3, 0xdca, 0xdbd, 0xdd2, 0xdb6, 0xdca, 0xdbb,
0xdc0, 0xdd2, 0xdbd, 0xdca, 0xdbd, 0xddd, 0xdb8, 0xdca, 0xdbd, 0xdd4, 0xdc0, 0xdb1,
0xdca, 0xda9, 0xdcf, 0xdbd, 0xdd4, 0xdb6, 0xdd4, 0xdb8, 0xdca, 0xdb6, 0xdcf, 0xdc1,
0xdd2, 0xdbd, 0xdd4, 0xdc3, 0xdcf, 0xd9a, 0xdcf, 0xdb8, 0xdbd, 0xdb6, 0xddd, 0xdb8,
0xdb4, 0xdd4, 0xda7, 0xddd, 0xdb8, 0xdc3, 0xdda, 0xdbb, 0xdd4, 0xdb8, 0xdca, 0xdb6,
0xdb6, 0xdcf, 0xdb1, 0xdda, 0xdb8, 0xddc, 0xd9c, 0xda9, 0xdd2, 0xdc2, 0xdd4, 0xdb8,
0xddc, 0xdb1, 0xdca, 0xdbb, 0xddc, 0xdc0, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdba, 0xdd2,
0xdbb, 0xddd, 0xdb6, 0xdd2, 0xdaf, 0xdca, 0xda2, 0xdb8, 0xdd9, 0xdb1, 0xdcf, 0xdb1,
0xdd2, 0xdba, 0xdcf, 0xdb8, 0xdda, 0xdb1, 0xdd4, 0xd86, 0xd9a, 0xdca, 0xda0, 0xddc,
0xda7, 0xdca, 0xd94, 0xd86, 0xd9c, 0xda9, 0xddc, 0xd9c, 0xddc, 0xdb4, 0xddc, 0xdbb,
0xdca, 0xdad, 0xddd, 0x2d, 0xdb1, 0xddc, 0xdc0, 0xddd, 0xdc3, 0xdcf, 0xd95, 0x20,
0xdad, 0xddd, 0xdb8, 0xdca, 0xda7, 0xdca, 0x200d, 0xdbb, 0xdd2, 0xdb4, 0xddc, 0xdbd,
0xdd2, 0xda7, 0xdd2, 0xdba, 0xdd4, 0xdb1, 0xdd2, 0xdc3, 0xdca, 0xdc0, 0xdd2, 0xdb1,
0xdca, 0xda9, 0xdca, 0xdc4, 0xddc, 0xd91, 0xd9a, 0xdca, 0xd87, 0xda9, 0xd9a, 0xdca,
0xd87, 0xdb1, 0xd9a, 0xddc, 0xdbb, 0xdcf, 0xda2, 0xdca, 0xd87, 0xdb1, 0xdca, 0xd9c,
0xdd4, 0xdba, 0xdd2, 0xdbd, 0xdcf, 0xdc0, 0xd87, 0xdb1, 0xdca, 0xda7, 0xdd2, 0xd9c,
0xdd4, 0xdc0, 0xdcf, 0xd87, 0xdbb, 0xd9c, 0xdd4, 0xd85, 0xdba, 0xdd2, 0xdb1, 0xdcf,
0xdb6, 0xdd4, 0xdc0, 0xdb1, 0xddd, 0xdc3, 0xdca, 0x20, 0xd85, 0xdba, 0xdbb, 0xdca,
0xdc3, 0xdca, 0xd9a, 0xda7, 0xdb8, 0xdbb, 0xdca, 0xd9a, 0xdcf, 0xd9a, 0xddc, 0xdbb,
0xdca, 0xdaf, 0xddc, 0xdb6, 0xdcf, 0xda2, 0xdd4, 0xda2, 0xdd4, 0xdba, 0xdd2, 0xdbd,
0xdcf, 0x20, 0xdbb, 0xdd2, 0xda2, 0xddd, 0xdbd, 0xdcf, 0xdb8, 0xdd9, 0xdb1, 0xdca,
0xda9, 0xddd, 0xdc3, 0xdcf, 0xdbb, 0xdd2, 0xdba, 0xddd, 0x20, 0xd9c, 0xdbd, 0xdd9,
0xd9c, 0xddc, 0xdc3, 0xdca, 0xdc3, 0xddc, 0xdbd, 0xdca, 0xda7, 0xdcf, 0xdc3, 0xdd0,
0xdb1, 0xdca, 0x20, 0xda2, 0xdd4, 0xdc0, 0xdcf, 0xdb1, 0xdca, 0xdc3, 0xdd0, 0xdb1,
0xdca, 0x20, 0xdbd, 0xdd4, 0xdc0, 0xdd2, 0xdc3, 0xdca, 0xda7, 0xdd4, 0xd9a, 0xdd4,
0xdb8, 0xdb1, 0xdca, 0xd8b, 0xdc2, 0xdd4, 0xd85, 0xdba, 0xdd2, 0xdba, 0xdcf, 0xd85,
0xdbb, 0xdd6, 0xdb6, 0xdcf, 0xd85, 0xdc3, 0xdd4, 0xdb1, 0xdca, 0xdc3, 0xdd2, 0xdba,
0xddc, 0xdb1, 0xdca, 0xd87, 0xda7, 0xdd2, 0xd9a, 0xddc, 0xd9a, 0xdb1, 0xdca, 0xdb6,
0xdc4, 0xdd2, 0xdba, 0xdcf, 0xdb6, 0xdc4, 0xdd2, 0xdba, 0xdcf, 0x20, 0xdb6, 0xdb1,
0xdca, 0xdaf, 0xdd9, 0xdbb, 0xdcf, 0xdc3, 0xdca, 0xdb6, 0xdcf, 0xdb6, 0xda9, 0xddd,
0xdc3, 0xdca, 0xdb6, 0xdd9, 0xdbd, 0xdd9, 0xdb8, 0xdca, 0xdb6, 0xdd9, 0xdbd, 0xdd3,
0xdc3, 0xdca, 0xdb6, 0xdca, 0xdbd, 0xdd0, 0xdb1, 0xdca, 0xd9a, 0xdca, 0x2d, 0xdc3,
0xdd0, 0xdb6, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xdb6, 0xddc, 0xd86, 0x20, 0xdc0, 0xdd2,
0xdc3, 0xdca, 0xda7, 0xdcf, 0xdb6, 0xddc, 0xd9c, 0xddc, 0xda7, 0xdcf, 0xdb6, 0xddc,
0xdba, 0xdd2, 0xdc3, 0xdd2, 0xd9a, 0xdda, 0xdb8, 0xdca, 0xdb6, 0xdca, 0x200d, 0xdbb,
0xdd2, 0xda2, 0xdca, 0x20, 0xdb6, 0xddc, 0xd9a, 0xdca, 0xd9a, 0xd9a, 0xdb8, 0xdca,
0xdb4, 0xddd, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd1, 0xdb1, 0xdca, 0xda9, 0xdca,
0xd9a, 0xdd0, 0xdb1, 0xdca, 0xd9a, 0xdd4, 0xdb1, 0xdca, 0xd9a, 0xdbb, 0xd9a, 0xdc3,
0xdca, 0xd9a, 0xdda, 0xdba, 0xdd9, 0xdb1, 0xdca, 0xd9a, 0xdda, 0xdb8, 0xdb1, 0xdca,
0xda0, 0xdd2, 0xd9a, 0xdcf, 0xd9c, 0xddd, 0xda0, 0xdd2, 0xdc4, 0xdd4, 0xd86, 0xdc4,
0xdd4, 0xd86, 0xdc3, 0xdd2, 0xdba, 0xdd4, 0xda9, 0xdcf, 0xda9, 0xdca, 0x20, 0xda2,
0xdd4, 0xdc0, 0xdcf, 0xdbb, 0xdd9, 0xdc3, 0xdca, 0xd9a, 0xddc, 0xdc3, 0xdca, 0xda7,
0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xd9a, 0xdca, 0x200d, 0xdbb, 0xdd9, 0xdc3, 0xdca, 0xda7,
0xdb1, 0xdca, 0xd9a, 0xdd4, 0xdba, 0xdd2, 0xd85, 0xdb6, 0xdcf, 0xd9a, 0xdd4, 0xdbb,
0xd9a, 0xdcf, 0xdc0, 0xddd, 0xda9, 0xdd9, 0xdb1, 0xdca, 0xdb8, 0xdcf, 0xdbb, 0xdca,
0xd9a, 0xdca, 0xdc1, 0xdc0, 0xdca, 0xdb1, 0xdca, 0xda9, 0xddd, 0xdc3, 0xdb1, 0xdca,
0xda9, 0xddd, 0xdc3, 0xdb1, 0xdca, 0x20, 0xd9a, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xd9a,
0xdca, 0xda9, 0xdd9, 0xdb1, 0xdca, 0xdc0, 0xdbb, 0xdca, 0xda9, 0xdd9, 0xdad, 0xdca,
0xdbb, 0xdd4, 0xdc0, 0xdcf, 0xda9, 0xddc, 0xdb8, 0xdd2, 0xdb1, 0xdd2, 0xd9a, 0xdcf,
0xd91, 0xda9, 0xdca, 0xdb8, 0xdb1, 0xdca, 0xda7, 0xdb1, 0xdca, 0xd91, 0xdba, 0xdd2,
0xdbb, 0xdd4, 0xdb1, 0xdda, 0xdb4, 0xdca, 0xd91, 0xdbd, 0xdca, 0xdc3, 0xdd0, 0xdbd,
0xdca, 0xdc0, 0xdaf, 0xddd, 0xdbb, 0xdca, 0xdc6, 0xddd, 0xda7, 0xdca, 0x20, 0xdb1,
0xdd9, 0xdbd, 0xdca, 0xdc3, 0xdb1, 0xdca, 0xdc6, 0xddc, 0xdbb, 0xdca, 0xda7, 0xdbd,
0xdda, 0xdc3, 0xdcf, 0xd9c, 0xdca, 0xdbd, 0xdda, 0xdc3, 0xdca, 0x20, 0xdb6, 0xddc,
0xd9a, 0xdca, 0xd9a, 0xd9c, 0xdd6, 0xdc3, 0xdca, 0x20, 0xdb6, 0xddc, 0xd9a, 0xdca,
0xd9a, 0xd9c, 0xdca, 0xdbb, 0xdd1, 0xdb1, 0xdca, 0xda9, 0xdca, 0x20, 0xda7, 0xdbb,
0xdca, 0xd9a, 0xdca, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd0, 0xdb1, 0xda9, 0xdcf, 0xd9c,
0xdca, 0xdc0, 0xdcf, 0xda9, 0xdca, 0xdbd, 0xddd, 0xdb4, 0xdca, 0xd9c, 0xddd, 0xdad,
0xdb8, 0xdcf, 0xdbd, 0xdcf, 0xdc0, 0xd9c, 0xdd4, 0xd85, 0xdba, 0xdcf, 0xd9a, 0xdd4,
0xdba, 0xdd2, 0xdbd, 0xdca, 0xd9c, 0xdba, 0xdb1, 0xdcf, 0xdc0, 0xdc4, 0xdbd, 0xdd2,
0xdc6, 0xdd0, 0xd9a, 0xdca, 0xdc3, 0xdca, 0xdc4, 0xdc0, 0xdcf, 0xdb1, 0xdcf, 0xdc4,
0xdd9, 0xdb8, 0xddc, 0xdc3, 0xdd2, 0xdbd, 0xddd, 0xd89, 0xdb1, 0xdca, 0xda9, 0xdd2,
0xdba, 0xdcf, 0xdb1, 0xdb4, 0xddc, 0xdbd, 0xdd2, 0xdc3, 0xdca, 0xdb1, 0xddc, 0xd9a,
0xdca, 0xdc3, 0xdca, 0x2c, 0x20, 0xd89, 0xdb1, 0xdca, 0xdaf, 0xdd2, 0xdba, 0xdcf,
0xdb1, 0xdcf, 0xdb8, 0xdbb, 0xdd9, 0xdb1, 0xdca, 0xd9c, 0xddd, 0x2c, 0x20, 0xd89,
0xdb1, 0xdca, 0xdaf, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdcf, 0xdb4, 0xdd3, 0xda7, 0xdbb,
0xdca, 0xdc3, 0xdca, 0xdb6, 0xdbb, 0xdca, 0xd9c, 0xdca, 0x2c, 0x20, 0xd89, 0xdb1,
0xdca, 0xdaf, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdcf, 0xda7, 0xdd9, 0xdbd, 0xdca, 0x20,
0xdb1, 0xd9c, 0xdbb, 0xdba, 0x2c, 0x20, 0xd89, 0xdb1, 0xdca, 0xdaf, 0xdd2, 0xdba,
0xdcf, 0xdb1, 0xdcf, 0xdc0, 0xdd9, 0xdc0, 0xdda, 0x2c, 0x20, 0xd89, 0xdb1, 0xdca,
0xdaf, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdcf, 0xdc0, 0xdd2, 0xdb1, 0xdca, 0xdc3, 0xdb1,
0xdc3, 0xdca, 0x2c, 0x20, 0xd89, 0xdb1, 0xdca, 0xdaf, 0xdd2, 0xdba, 0xdcf, 0xdb1,
0xdcf, 0xdc0, 0xdd2, 0xdb1, 0xdb8, 0xdd0, 0xd9a, 0xdca, 0x2c, 0x20, 0xd89, 0xdb1,
0xdca, 0xdaf, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdcf, 0xd89, 0xdb1, 0xdd4, 0xdc0, 0xdd2,
0xd9a, 0xdca, 0xd89, 0xd9a, 0xdcf, 0xdbd, 0xdd4, 0xdba, 0xdd2, 0xda7, 0xdca, 0xda2,
0xdd0, 0xdb8, 0xdd9, 0xdba, 0xdd2, 0xd9a, 0xdcf, 0xda2, 0xdd6, 0xdb1, 0xddd, 0xdbd,
0xdd4, 0xdc0, 0xdd2, 0xdc3, 0xdca, 0xdc0, 0xdd2, 0xdbd, 0xdca, 0xdb8, 0xddc, 0xdb1,
0xdca, 0xda7, 0xdd2, 0xdc3, 0xdd9, 0xdbd, 0xddd, 0x2c, 0x20, 0xd9a, 0xdd9, 0xdb1,
0xdca, 0xda7, 0xd9a, 0xdd2, 0xd9a, 0xdca, 0x200d, 0xdbb, 0xdbd, 0xdd9, 0xdb1, 0xdca,
0xdaf, 0xdd2, 0xda2, 0xdca, 0xd9a, 0xdca, 0xdbd, 0xdcf, 0x20, 0xdb4, 0xdc3, 0xdca,
0xdbd, 0xdd3, 0xdb8, 0xdcf, 0xdbd, 0xddc, 0xdc3, 0xdca, 0x20, 0xd87, 0xdb1, 0xdca,
0xda2, 0xdbd, 0xdd3, 0xdc3, 0xdca, 0xdbd, 0xddd, 0xdc0, 0xdbb, 0xdca, 0x20, 0xdb4,
0xdca, 0x200d, 0xdbb, 0xdd2, 0xdb1, 0xdca, 0xdc3, 0xdc3, 0xdca, 0x20, 0xd9a, 0xdca,
0xdc0, 0xdcf, 0xda7, 0xdbb, 0xdca, 0xdb8, 0xdc3, 0xdda, 0xdba, 0xdd2, 0xdba, 0xddd,
0xdb8, 0xdb1, 0xd9c, 0xdd4, 0xd86, 0xdb8, 0xdd0, 0xdb1, 0xddd, 0xdc3, 0xdca, 0xdb8,
0xdd0, 0xdbb, 0xdd2, 0xd9c, 0xddc, 0xda7, 0xdca, 0xdb8, 0xdcf, 0xda7, 0xdd2, 0xdb1,
0xdd2, 0xd9a, 0xdca, 0xdb8, 0xda7, 0xdb8, 0xddc, 0xdbb, 0xddc, 0xdc3, 0xdca, 0xdb8,
0xdc3, 0xda7, 0xdca, 0xdbd, 0xdcf, 0xdb1, 0xdca, 0xdb8, 0xdd9, 0xdb1, 0xddc, 0xdb8,
0xdd2, 0xdb1, 0xdd3, 0xdb8, 0xdd9, 0xdbb, 0xdd2, 0xda9, 0xdcf, 0xdb8, 0xdd9, 0xda7,
0xdca, 0xdbd, 0xd9a, 0xda7, 0xdca, 0xdbd, 0xdcf, 0xdb8, 0xdd9, 0xd9a, 0xdca, 0xdc3,
0xdd2, 0xd9a, 0xddd, 0x20, 0xdb1, 0xd9c, 0xdbb, 0xdba, 0xdb8, 0xdd2, 0xd9a, 0xdd2,
0xdba, 0xdd4, 0xdbd, 0xdb1, 0xdca, 0xdb8, 0xddc, 0xdb1, 0xdca, 0xd9a, 0xdca, 0xda7,
0xdb1, 0xdca, 0xdb8, 0xddc, 0xdb1, 0xdca, 0xda7, 0xdd9, 0xdbb, 0xdda, 0xdb8, 0xddc,
0xdb1, 0xdca, 0xda7, 0xdd9, 0xdc0, 0xdd2, 0xda9, 0xdd2, 0xdba, 0xddd, 0xdb8, 0xddc,
0xdb1, 0xdca, 0xda7, 0xdca, 0xdc3, 0xdda, 0xdbb, 0xdcf, 0xda7, 0xdca, 0xdb1, 0xdc3,
0xdca, 0xdc3, 0xddd, 0xdb1, 0xdd2, 0xdc0, 0xdca, 0xdba, 0xddd, 0xd9a, 0xdca, 0xdb1,
0xddd, 0xdb8, 0xdca, 0xdb1, 0xddc, 0xdbb, 0xddc, 0xdb1, 0xdca, 0xdc4, 0xdcf, 0xdb6,
0xdd2, 0xdba, 0xdd4, 0xdbd, 0xdcf, 0x2c, 0x20, 0xd8b, 0xdad, 0xdd4, 0xdbb, 0xdd4,
0x20, 0xda9, 0xd9a, 0xddd, 0xda7, 0xdcf, 0xdb8, 0xdb0, 0xdca, 0x200d, 0xdba, 0xdb8,
0x2c, 0x20, 0xd8b, 0xdad, 0xdd4, 0xdbb, 0xdd4, 0x20, 0xda9, 0xd9a, 0xddd, 0xda7,
0xdcf, 0xdb1, 0xdc0, 0x20, 0xdc3, 0xdbd, 0xdd9, 0xdb8, 0xdca, 0xdb8, 0x2c, 0x20,
0xd8b, 0xdad, 0xdd4, 0xdbb, 0xdd4, 0x20, 0xda9, 0xd9a, 0xddd, 0xda7, 0xdcf, 0xdb1,
0xdd6, 0xd9a, 0xdca, 0xd94, 0xda2, 0xdd2, 0xdb1, 0xd9c, 0xdcf, 0xdb4, 0xdd0, 0xdb1,
0xdb8, 0xdcf, 0xdb4, 0xdd0, 0xdbb, 0xdcf, 0xdb8, 0xdbb, 0xdd2, 0xdb6, 0xddd, 0xdc6,
0xdd3, 0xdb1, 0xdd2, 0xd9a, 0xdca, 0xdc3, 0xdca, 0xdb4, 0xddc, 0xdbb, 0xdca, 0xda7,
0xdca, 0x2d, 0xd95, 0x2d, 0xdb4, 0xdca, 0x200d, 0xdbb, 0xdd2, 0xdb1, 0xdca, 0xdc3,
0xdca, 0xdb4, 0xddc, 0xdbb, 0xdca, 0xda7, 0xdca, 0x20, 0xd94, 0xdc6, 0xdca, 0x20,
0xdc3, 0xdca, 0xdb4, 0xdd9, 0xdba, 0xdd2, 0xdb1, 0xdca, 0xdb4, 0xddc, 0xdbb, 0xdca,
0xdad, 0xddd, 0x20, 0xdc0, 0xdd9, 0xdbd, 0xdca, 0xdc4, 0xddd, 0xdb4, 0xdd4, 0xdc0,
0xda7, 0xddc, 0xdbb, 0xdd2, 0xd9a, 0xddd, 0xdc0, 0xdb4, 0xdd4, 0xdb1, 0xdca, 0xda7,
0xdcf, 0x20, 0xd87, 0xdbb, 0xdd2, 0xdb1, 0xdcf, 0xdc3, 0xdca, 0xdbb, 0xdd0, 0xdb1,
0xdca, 0xd9a, 0xdd2, 0xdb1, 0xdca, 0x20, 0xdb4, 0xdd2, 0xdc0, 0xdd2, 0xdc3, 0xdd4,
0xdb8, 0xdbb, 0xdd9, 0xdc3, 0xdd2, 0xdc6, 0xdca, 0xdbb, 0xdd9, 0xda2, 0xdd2, 0xdb1,
0xdcf, 0xdbb, 0xdd9, 0xdc3, 0xddc, 0xdbd, 0xdd4, 0xda7, 0xdca, 0xdbb, 0xdd2, 0xdba,
0xddd, 0x20, 0xdb6, 0xdca, 0x200d, 0xdbb, 0xdb1, 0xdca, 0xd9a, 0xddd, 0xdc3, 0xdb1,
0xdca, 0xda7, 0xdbb, 0xdda, 0xdb8, 0xdca, 0xdc3, 0xdb1, 0xdca, 0xdad, 0xdd2, 0xdba,
0xdcf, 0xd9c, 0xddd, 0xdc3, 0xdd0, 0xdb1, 0xdca, 0xda7, 0xddd, 0x20, 0xda9, 0xddc,
0xdb8, 0xdd2, 0xdb1, 0xdca, 0xd9c, 0xddd, 0xdc3, 0xdcf, 0xdc0, 0xddd, 0x20, 0xdb4,
0xddd, 0xdbd, 0xddc, 0xd89, 0xda7, 0xddc, 0xd9a, 0xddc, 0xdbb, 0xdca, 0xda7, 0xddd,
0xdb8, 0xdd2, 0xda7, 0xdca, 0xdc3, 0xdd2, 0xda7, 0xdca, 0xd9a, 0xdcf, 0xdc1, 0xdcf,
0xdb1, 0xdca, 0xdad, 0x20, 0xdb6, 0xdbb, 0xdca, 0xdad, 0xdbd, 0xdd9, 0xdb8, 0xdd2,
0xdc1, 0xdcf, 0xdb1, 0xdca, 0xdad, 0x20, 0xda2, 0xddd, 0xdb1, 0xdca, 0xd9c, 0xdda,
0xdc1, 0xdcf, 0xdb1, 0xdca, 0xdad, 0x20, 0xd9a, 0xdd2, 0xda7, 0xdca, 0xdc3, 0xdca,
0xdc1, 0xdcf, 0xdb1, 0xdca, 0xdad, 0x20, 0xdbd, 0xdd4, 0xdc3, 0xdd2, 0xdba, 0xdcf,
0xdc1, 0xdcf, 0xdb1, 0xdca, 0xdad, 0x20, 0xdad, 0xddd, 0xdb8, 0xdc3, 0xdca, 0xdc1,
0xdcf, 0xdb1, 0xdca, 0xdad, 0x20, 0xdc0, 0xdd2, 0xdb1, 0xdca, 0xda7, 0xdca, 0xdc3,
0xdb1, 0xdca, 0xdc3, 0xdca, 0xdc0, 0xdd2, 0xdc6, 0xdca, 0xda7, 0xdca, 0x20, 0xd9a,
0xdbb, 0xdb1, 0xdca, 0xda7, 0xdca, 0xda7, 0xdd9, 0xd9c, 0xdd4, 0xdc3, 0xdd2, 0xd9c,
0xdbd, 0xdca, 0xdb4, 0xdcf, 0xdad, 0xdd4, 0xdbd, 0xdda, 0xda7, 0xdd2, 0xda2, 0xdd4,
0xd86, 0xdb1, 0xdcf, 0xda7, 0xddc, 0xdbb, 0xdb1, 0xdca, 0xda7, 0xddd, 0xda7, 0xddc,
0xdbb, 0xdca, 0xda7, 0xddd, 0xdbd, 0xdcf, 0xdc0, 0xdd0, 0xdb1, 0xdca, 0xd9a, 0xdd4,
0xdc0, 0xdbb, 0xdca, 0xdc0, 0xdba, 0xdd2, 0xda7, 0xdca, 0xdc4, 0xddd, 0xdbb, 0xdca,
0xdc3, 0xdca, 0xdc0, 0xdd2, 0xdb1, 0xdd2, 0xdb4, 0xdd9, 0xd9c, 0xdca, 0xdba, 0xd9a,
0xdd4, 0xda7, 0xda7, 0xdca, 0xd9a, 0xdd0, 0xdc3, 0xdda, 0xda9, 0xdda, 0xdc0, 0xdd2,
0xdc3, 0xdca, 0xda9, 0xdd4, 0xdb8, 0xddc, 0xdb1, 0xdca, 0xda7, 0xdca, 0x20, 0xda9,
0xdd4, 0x2018, 0xdbb, 0xdca, 0xdc0, 0xdd2, 0xdbd, 0xdca, 0xdb8, 0xd9a, 0xdca, 0xd9a,
0xdd4, 0xdc0, 0xdbb, 0xdd2, 0xdb8, 0xdc0, 0xdca, 0xdc3, 0xdb1, 0xdca, 0xdb8, 0xdd0,
0xd9a, 0xdca, 0xdb8, 0xdd4, 0xdbb, 0xdca, 0xda9, 0xddd, 0xdb4, 0xdd0, 0xdbd, 0xdca,
0xdb8, 0xdbb, 0xdca, 0xdbb, 0xddc, 0xdad, 0xdd9, 0xdbb, 0xdcf, 0xdc3, 0xdd2, 0xdba,
0xddd, 0xdc0, 0xdcf, 0xda7, 0xdca, 0x200d, 0xdbb, 0xddd, 0xdbd, 0xdca, 0xdc0, 0xddc,
0xdc3, 0xdca, 0xda7, 0xddc, 0xd9a, 0xdca, 0xdbd, 0xddd, 0xdb1, 0xdca, 0xd9c, 0xdca,
0xd89, 0xdba, 0xdbb, 0xdca, 0xdb6, 0xdd2, 0xdba, 0xdd9, 0xdb1, 0xdca, 0xd92, 0xda9,
0xdca, 0xdb1, 0xdca, 0xd85, 0xdbd, 0xdca, 0xdb8, 0xda7, 0xdd2, 0xd85, 0xdb8, 0xdca,
0xdb8, 0xdcf, 0xdb1, 0xdca, 0xd87, 0xdb1, 0xdcf, 0xdaf, 0xdd2, 0xdba, 0xdca, 0xdbb,
0xdca, 0xd85, 0xd9a, 0xdca, 0xda7, 0xddd, 0xd85, 0xd9a, 0xdca, 0xda7, 0xddd, 0xdb6,
0xdca, 0xd85, 0xdc1, 0xdca, 0xd9c, 0xdb6, 0xdd0, 0xda7, 0xdca, 0xd87, 0xda7, 0xdd2,
0xdbb, 0xdc0, 0xdd4, 0xdb6, 0xdd0, 0xd9c, 0xdca, 0xda9, 0xdd1, 0xda9, 0xdca, 0xdb6,
0xdc4, 0xdbb, 0xdda, 0xdb1, 0xdca, 0xdb6, 0xdd0, 0xd82, 0xd9a, 0xddc, 0xd9a, 0xdca,
0xdb6, 0xdbb, 0xdca, 0xdb1, 0xdde, 0xdbd, 0xdca, 0xdb6, 0xdd3, 0xdbb, 0xdd4, 0xda7,
0xdca, 0xdb6, 0xdd2, 0xdc1, 0xdca, 0xd9a, 0xdd9, 0xd9a, 0xdca, 0xdb6, 0xdf2, 0xdb1,
0xdcf, 0xdba, 0xdd2, 0xda0, 0xdd2, 0xda7, 0xdcf, 0xd9a, 0xddc, 0xdc5, 0xdb9, 0xda9,
0xdd0, 0xdb8, 0xdc3, 0xdca, 0xd9a, 0xdc3, 0xdca, 0xda9, 0xd9a, 0xdcf, 0xda9, 0xdd2,
0xdbd, 0xdd2, 0xda9, 0xdd4, 0xdb6, 0xdcf, 0xdba, 0xdd2, 0xda9, 0xdd4, 0xdc1, 0xdb1,
0xdca, 0xdb6, 0xdda, 0xdc6, 0xdd0, 0xdb8, 0xd9c, 0xdd4, 0xdc3, 0xdca, 0xda7, 0xdcf,
0xd9c, 0xdcf, 0xdc3, 0xdcf, 0xdc4, 0xdd9, 0xdb6, 0xdca, 0xdbb, 0xddd, 0xdb1, 0xdca,
0xdc4, 0xddd, 0xda0, 0xdd2, 0x20, 0xdb8, 0xdd2, 0xd82, 0x20, 0xdb1, 0xd9c, 0xdbb,
0xdba, 0xdc4, 0xddc, 0xd82, 0xd9a, 0xddc, 0xd82, 0xdc4, 0xddc, 0xdc0, 0xdca, 0xda9,
0xdca, 0xd89, 0xdbb, 0xd9a, 0xdd4, 0xda7, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0xda2,
0xd9a, 0xdbb, 0xdca, 0xdad, 0xdcf, 0xda2, 0xdba, 0xdb4, 0xdd4, 0xdbb, 0xda2, 0xdd9,
0xdbb, 0xdd4, 0xdc3, 0xdbd, 0xdb8, 0xd9a, 0xdcf, 0xdb6, 0xdd4, 0xdbd, 0xdca, 0xd9a,
0xdb8, 0xdca, 0xda0, 0xda7, 0xdca, 0xd9a, 0xdcf, 0xd9a, 0xdbb, 0xda0, 0xdca, 0xda0,
0xdd2, 0xda0, 0xdd2, 0xd9a, 0xdad, 0xdca, 0xdb8, 0xdb1, 0xdca, 0xda9, 0xdd4, 0xd9b,
0xdb1, 0xdca, 0xda9, 0xdd2, 0xd9c, 0xdcf, 0xd9a, 0xdbd, 0xdca, 0xd9a, 0xda7, 0xdcf,
0xd9a, 0xdca, 0x200d, 0xdbb, 0xdc3, 0xdca, 0xdb1, 0xddd, 0xdba, 0xdcf, 0xdbb, 0xdca,
0xdc3, 0xdca, 0xd9a, 0xdca, 0xd9a, 0xdca, 0xdc0, 0xdcf, 0xdbd, 0xdcf, 0xdbd, 0xdb8,
0xdca, 0xdb4, 0xdd6, 0xdbb, 0xdca, 0xd9a, 0xdd4, 0xda0, 0xdd2, 0xd82, 0xd9a, 0xdd4,
0xdc0, 0xdda, 0xda7, 0xdca, 0xdb8, 0xdd0, 0xd9a, 0xdcf, 0xdc0, 0xdd4, 0xdb8, 0xd9c,
0xda9, 0xdb1, 0xdca, 0xdb8, 0xd9a, 0xdca, 0xdc3, 0xdca, 0xdc3, 0xdcf, 0xdbb, 0xdca,
0xdb8, 0xdd0, 0xdb1, 0xdd2, 0xdbd, 0xdcf, 0xdb8, 0xdc3, 0xdca, 0xd9a, 0xda7, 0xdca,
0xdb1, 0xdd2, 0xd9a, 0xddd, 0xdc3, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xddc, 0xdc0, 0xddc,
0xd9a, 0xdd4, 0xdc3, 0xdca, 0xdb1, 0xdda, 0xda7, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca,
0xdb1, 0xddc, 0xdc0, 0xddc, 0xdc3, 0xdd2, 0xdb6, 0xdbb, 0xdca, 0xdc3, 0xdca, 0xd9a,
0xdca, 0xd94, 0xdb8, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0xd94, 0xdbb, 0xdcf, 0xdbd,
0xdca, 0xdb1, 0xddc, 0xdb8, 0xdca, 0x20, 0xdb4, 0xdd9, 0xdb1, 0xdca, 0xdb4, 0xddc,
0xdb1, 0xdca, 0xda7, 0xdd2, 0xdba, 0xdb1, 0xd9a, 0xdca, 0xdb4, 0xdca, 0xdba, 0xddd,
0xdb1, 0xdca, 0xd9c, 0xdca, 0xdba, 0xdd0, 0xdb1, 0xdca, 0xd9c, 0xdca, 0xd9a, 0xda7,
0xdcf, 0xdbb, 0xdca, 0xd9a, 0xddc, 0xdc3, 0xdca, 0xdad, 0xdcf, 0xdb1, 0xdda, 0xd9a,
0xdca, 0xdba, 0xdd2, 0xdc3, 0xdd2, 0xdbd, 0xddd, 0xdbb, 0xdca, 0xda9, 0xdcf, 0xdbb,
0xdd2, 0xdba, 0xdcf, 0xdaf, 0xdca, 0xdc3, 0xd9a, 0xdca, 0xdc4, 0xdbd, 0xdd2, 0xdb1,
0xdca, 0xdc3, 0xdcf, 0xdb8, 0xdcf, 0xdbb, 0xdca, 0xd9a, 0xdb1, 0xdca, 0xda9, 0xdca,
0xdc3, 0xddd, 0xdbd, 0xdca, 0xdc2, 0xdd0, 0xdb1, 0xdca, 0xdc4, 0xdba, 0xdd2, 0xdc3,
0xdd2, 0xd82, 0xd9c, 0xdb4, 0xdca, 0xdb4, 0xdd6, 0xdbb, 0xdd4, 0xdc0, 0xdc1, 0xdca,
0x200d, 0xdbb, 0xdd9, 0xda9, 0xdca, 0xdb1, 0xdd9, 0xd9a, 0xddd, 0xdbd, 0xdd2, 0xdb8,
0xdca, 0xdc3, 0xd9a, 0xdca, 0xdad, 0xdcf, 0xdba, 0xdd2, 0xdb4, 0xdd9, 0xda7, 0xdc2,
0xdca, 0xd9a, 0xdd9, 0xdb1, 0xdca, 0xda7, 0xdca, 0xda7, 0xdca, 0xdb6, 0xdd2, 0xdbd,
0xdd2, 0xdc3, 0xdd2, 0xda7, 0xdd9, 0xdc4, 0xdd9, 0xdbb, 0xdcf, 0xdb1, 0xdca, 0xdad,
0xdd2, 0xdb8, 0xdca, 0xdb4, 0xdd4, 0xda7, 0xddd, 0xd9a, 0xdd2, 0xdba, 0xddd, 0xda7,
0xddc, 0xdb8, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0xd8b, 0xdbd, 0xdcf, 0xdb1, 0xdca,
0xdb6, 0xdcf, 0xda7, 0xdbb, 0xdca, 0xd8b, 0xdbb, 0xdd4, 0xdb8, 0xdca, 0xd9a, 0xdd2,
0xd8b, 0xdc3, 0xdca, 0xda7, 0xdca, 0x2d, 0xdb1, 0xdd9, 0xdbb, 0xdcf, 0xdc0, 0xdd2,
0xdba, 0xdd0, 0xdb1, 0xdca, 0xda7, 0xdd2, 0xdba, 0xdb1, 0xdca, 0xdc0, 0xdca, 0xdbd,
0xda9, 0xdd2, 0xdc0, 0xddc, 0xdc3, 0xdca, 0xda7, 0xddc, 0xd9a, 0xdca, 0xdba, 0xd9a,
0xdd4, 0xda7, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0xdbb, 0xdd0, 0xd82, 0xd9c, 0xdd4,
0xdb1, 0xdca, 0xdba, 0xdd9, 0xd9a, 0xda7, 0xdbb, 0xdd2, 0xdb1, 0xdca, 0xdb6, 0xdbb,
0xdca, 0xd9c, 0xdca, 0xdba, 0xdd9, 0xdbb, 0xdd9, 0xdc0, 0xdb1, 0xdca, 0xd87, 0xdc3,
0xddc, 0xdbb, 0xdda, 0xdb6, 0xdbb, 0xdca, 0xdb8, 0xdd2, 0xdba, 0xdd4, 0xda9, 0xdcf,
0xd9a, 0xdd0, 0xdb1, 0xdbb, 0xdd2, 0xd9a, 0xdda, 0xdb4, 0xdca, 0x20, 0xdc0, 0xdbb,
0xdca, 0xda9, 0xdca, 0xdc6, 0xdcf, 0xdbb, 0xddd, 0xdb8, 0xda9, 0xdd9, 0xdba, 0xdd2,
0xdbb, 0xdcf, 0xdbb, 0xdd9, 0xdba, 0xdd2, 0xd9a, 0xdca, 0xda2, 0xdcf, 0xdc0, 0xdd2,
0xd9a, 0xdca, 0xdaf, 0xd9a, 0xdd4, 0xdab, 0xdd4, 0x20, 0xda2, 0xddd, 0xdbb, 0xdca,
0xda2, 0xdd2, 0xdba, 0xdcf, 0xdc0, 0xdc1, 0xdcf, 0x2e, 0x20, 0xdc4, 0xdd9, 0xdbd,
0xdda, 0xdb1, 0xdcf, 0xdc3, 0xdca, 0xda7, 0xdd0, 0xdb1, 0xdca, 0xdbd, 0xdd2, 0xd87,
0xda9, 0xdd2, 0xdbd, 0xdda, 0xda9, 0xdca, 0xdb6, 0xdca, 0x200d, 0xdbb, 0xdd2, 0xdc3,
0xdca, 0xdb6, 0xdda, 0xdb1, 0xdca, 0xdb6, 0xdca, 0x200d, 0xdbb, 0xddd, 0xd9a, 0xdb1,
0xdca, 0x20, 0xdc4, 0xdd2, 0xdbd, 0xdca, 0xda9, 0xdcf, 0xdc0, 0xdd2, 0xdb1, 0xdca,
0xd89, 0xdba, 0xdd4, 0xd9a, 0xdca, 0xdbd, 0xdcf, 0xdc4, 0xddd, 0xdb6, 0xdcf, 0xdbb,
0xdca, 0xda7, 0xdca, 0xdbd, 0xdd2, 0xdb1, 0xdca, 0xda9, 0xdda, 0xdb8, 0xdb1, 0xdca,
0xdbd, 0xddd, 0xdbb, 0xdca, 0xda9, 0xdca, 0x20, 0xdc4, 0xddd, 0xdc0, 0xdca, 0xdb8,
0xdd9, 0xdbd, 0xdca, 0xdb6, 0xddd, 0xdbb, 0xdca, 0xdb1, 0xdca, 0xdb4, 0xdbb, 0xdca,
0xdad, 0xdca, 0xdc3, 0xdd2, 0xda9, 0xdca, 0xdb1, 0xdd2, 0xd87, 0xdb8, 0xdca, 0xdc3,
0xdca, 0xda7, 0xdbb, 0xdca, 0xda9, 0xdd1, 0xdb8, 0xdca, 0xd87, 0xdb1, 0xdca, 0xda9,
0xddd, 0xdbb, 0xdcf, 0xdc0, 0xd85, 0xdc3, 0xdca, 0xda7, 0xdca, 0x200d, 0xdbb, 0xd9b,
0xdcf, 0xdb1, 0xdca, 0xd87, 0xdad, 0xdd0, 0xdb1, 0xdca, 0xdc3, 0xdca, 0xdb6, 0xdd9,
0xdbd, 0xdca, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdda, 0xda9, 0xdca, 0xda9, 0xdca, 0xdb6,
0xdbb, 0xdca, 0xdbd, 0xdd2, 0xdb1, 0xdca, 0xdb6, 0xdca, 0x200d, 0xdbb, 0xda7, 0xdd2,
0xdc3, 0xdca, 0xdbd, 0xdc0, 0xdcf, 0xdb6, 0xdca, 0x200d, 0xdbb, 0xdc3, 0xdbd, 0xdca,
0xdc3, 0xdca, 0xdb6, 0xdd4, 0xd9a, 0xdcf, 0xdbb, 0xdd9, 0xdc3, 0xdca, 0xda7, 0xdca,
0xdb6, 0xdd4, 0xda9, 0xdcf, 0xdb4, 0xdd9, 0xdc3, 0xdca, 0xda7, 0xdca, 0xdb6, 0xdd4,
0xdc3, 0xdd2, 0xdb1, 0xdca, 0xda2, 0xdd9, 0xdb1, 0xdca, 0xda0, 0xdd2, 0xdc3, 0xdd2,
0xdb1, 0xddd, 0xd9a, 0xddd, 0xdb4, 0xdb1, 0xdca, 0xdc4, 0xdda, 0xd9c, 0xdb1, 0xdca,
0xda9, 0xdb6, 0xdca, 0xdbd, 0xdd2, 0xdb1, 0xdca, 0xda2, 0xdd2, 0xdb6, 0xdca, 0xdbb,
0xddd, 0xdbd, 0xdca, 0xda7, 0xdcf, 0xd9c, 0xdd4, 0xdc0, 0xdb1, 0xdca, 0xdc3, 0xdda,
0xdc4, 0xdd9, 0xdbd, 0xdca, 0xdc3, 0xdd2, 0xdb1, 0xdca, 0xd9a, 0xdd2, 0xd85, 0xdba,
0xdd2, 0xdbd, 0xdca, 0x20, 0xd94, 0xdc6, 0xdca, 0x20, 0xdb8, 0xdd1, 0xdb1, 0xdca,
0xd89, 0xdc3, 0xdca, 0xdad, 0xdcf, 0xdb1, 0xdca, 0xdb6, 0xdd4, 0xdbd, 0xdca, 0xda2,
0xdbb, 0xdca, 0xdc3, 0xdd2, 0xd9a, 0xdbd, 0xdd2, 0xdb1, 0xdd2, 0xdb1, 0xdca, 0xd9c,
0xdca, 0x200d, 0xdbb, 0xdd1, 0xda9, 0xdca, 0xd9a, 0xdd2, 0xdbb, 0xddc, 0xdc0, 0xdca,
0xd9a, 0xdd2, 0xdc0, 0xdca, 0xdbd, 0xdd2, 0xdc3, 0xdca, 0xdb6, 0xdb1, 0xdca, 0xdbd,
0xdca, 0xda2, 0xdd4, 0xdb6, 0xdd4, 0xdbd, 0xdca, 0xda2, 0xdb1, 0xdcf, 0xdbd, 0xdb1,
0xdca, 0xda9, 0xdb1, 0xdca, 0xdbd, 0xd9a, 0xdca, 0xdc3, 0xdd0, 0xdb8, 0xdca, 0xdb6,
0xdbb, 0xdca, 0xd9c, 0xdca, 0xdb8, 0xdd0, 0xda9, 0xdca, 0xdbb, 0xdd2, 0xda9, 0xdca,
0xdb8, 0xddd, 0xdbd, 0xdca, 0xda7, 0xdcf, 0xdb8, 0xdbb, 0xdd2, 0xdba, 0xdd9, 0xdc4,
0xdb8, 0xdca, 0xdb1, 0xdca, 0xdb8, 0xdd2, 0xdb1, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca,
0xdb8, 0xddc, 0xdb1, 0xdd0, 0xd9a, 0xddd, 0xdb8, 0xddc, 0xdc3, 0xdca, 0xd9a, 0xdc0,
0xdca, 0xd94, 0xdc3, 0xdca, 0xdbd, 0xddd, 0xdb4, 0xdd0, 0xdbb, 0xdd2, 0xdc3, 0xdca,
0xdb4, 0xddc, 0xda9, 0xdca, 0xd9c, 0xddc, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb4, 0xdca,
0x200d, 0xdbb, 0xdd9, 0xd9c, 0xdca, 0xdbb, 0xdd2, 0xd9c, 0xdcf, 0xdbb, 0xddd, 0xdb8,
0xdba, 0xdc3, 0xdb8, 0xdbb, 0xdcf, 0xdc3, 0xdd0, 0xdb1, 0xdca, 0x20, 0xdb8, 0xdd0,
0xdbb, 0xdd2, 0xdb1, 0xddd, 0xdc3, 0xdbb, 0xdcf, 0xda2, 0xdd9, 0xdc0, 0xddd, 0xdc3,
0xdbb, 0xda7, 0xddc, 0xdc0, 0xdca, 0xdc3, 0xdd2, 0xdb8, 0xdca, 0xdc6, 0xdd9, 0xdbb,
0xddd, 0xdb4, 0xddc, 0xdbd, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xddc, 0xdb4, 0xdca, 0xda2,
0xdda, 0xdc3, 0xddc, 0xdc6, 0xdd2, 0xdba, 0xdcf, 0xdc3, 0xdca, 0xda7, 0xddc, 0xd9a,
0xdca, 0xdc4, 0xddd, 0xdb8, 0xdca, 0xda7, 0xdbd, 0xdd2, 0xdb1, 0xdca, 0xda7, 0xdd2,
0xdbb, 0xdcf, 0xdb1, 0xdda, 0xd8b, 0xdbd, 0xdca, 0xdba, 0xdb1, 0xddc, 0xdc0, 0xdca,
0xdc3, 0xdca, 0xd9a, 0xdca, 0xdc0, 0xda9, 0xdd4, 0xdc3, 0xdca, 0xdc0, 0xdad, 0xdd2,
0xd9a, 0xdcf, 0xdb1, 0xdd4, 0xdc0, 0xdc0, 0xdd2, 0xdba, 0xdb1, 0xdcf, 0xdc0, 0xdd2,
0xdbd, 0xdca, 0xdb1, 0xdd2, 0xdba, 0xdd4, 0xdc3, 0xdca, 0xdc0, 0xddc, 0xdbd, 0xdca,
0xd9c, 0xddc, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd1, 0xda9, 0xdca, 0xdc0, 0xdbb, 0xdca,
0xdc3, 0xddd, 0xdc3, 0xd9c, 0xdca, 0xdbb, 0xdd9, 0xdb6, 0xdca, 0xdc3, 0xdd6, 0xdbb,
0xdd2, 0xda0, 0xdca, 0xd87, 0xdb1, 0xdca, 0xda7, 0xdb1, 0xdcf, 0xdb1, 0xdbb, 0xdd2,
0xdc0, 0xddd, 0xda0, 0xdcf, 0xd9c, 0xddc, 0xdc3, 0xdca, 0xd9a, 0xdca, 0x200d, 0xdbb,
0xdd2, 0xdc3, 0xdca, 0xdb8, 0xdc3, 0xdca, 0xd9a, 0xddc, 0xd9a, 0xddc, 0xdc3, 0xdca,
0xd9a, 0xddc, 0xdb8, 0xddc, 0xdbb, 0xddd, 0xd9a, 0xdd9, 0xdbb, 0xdca, 0xd9c, 0xdd4,
0xd91, 0xdbd, 0xdd9, 0xdb1, 0xdca, 0xdb8, 0xdcf, 0xdc4, 0xdda, 0xdb8, 0xdcf, 0xdbd,
0x20, 0xdaf, 0xdd2, 0xdc0, 0xdba, 0xdd2, 0xdb1, 0xdb8, 0xdd4, 0xdbb, 0xdd4, 0xdc3,
0xdd2, 0xdba, 0xdb8, 0xdba, 0xddd, 0xda7, 0xdd2, 0xdbb, 0xdd2, 0xdba, 0xdd4, 0xdb1,
0xdd2, 0xdba, 0xdb1, 0xdca, 0xd87, 0xdb4, 0xdd2, 0xdba, 0xdcf, 0xd95, 0xd9a, 0xdca,
0xdbd, 0xdb1, 0xdca, 0xda9, 0xdca, 0xdb6, 0xddd, 0xd9c, 0xdb1, 0xdca, 0xdc0, 0xdd2,
0xdbd, 0xdcf, 0xda0, 0xdd0, 0xdad, 0xdb8, 0xdca, 0xda0, 0xdd6, 0xd9a, 0xdca, 0xd8a,
0xdc3, 0xdca, 0xda7, 0xdbb, 0xdca, 0xd91, 0xdc6, 0xdcf, 0xdad, 0xdca, 0xdc6, 0xdcf,
0xd9a, 0xdcf, 0xd95, 0xdc6, 0xddd, 0xdc6, 0xdd3, 0xda2, 0xdd2, 0xdc6, 0xdd4, 0xdb1,
0xdc6, 0xdd4, 0xda7, 0xdd2, 0xd9c, 0xdbd, 0xdb4, 0xd9c, 0xddd, 0xdc3, 0xdca, 0xd9c,
0xdb8, 0xdca, 0xdb6, 0xdd2, 0xdba, 0xdd9, 0xdbb, 0xdca, 0xd9c, 0xdd4, 0xdc0, 0xdcf,
0xda9, 0xdbd, 0xdca, 0xd9a, 0xdd0, 0xdb1, 0xdbd, 0xdca, 0xd9c, 0xdd4, 0xdc0, 0xdcf,
0xdb8, 0xdca, 0xdc4, 0xddc, 0xdb1, 0xddc, 0xdbd, 0xdd4, 0xdbd, 0xdd4, 0xd91, 0xdb1,
0xdca, 0xda9, 0xdbb, 0xdca, 0xdb6, 0xdbb, 0xdd2, 0xd9a, 0xdd2, 0xdbb, 0xdd2, 0xdb8,
0xda7, 0xdd2, 0xd9a, 0xddc, 0xdc3, 0xdca, 0xdbb, 0xdda, 0xd9a, 0xdca, 0xdc0, 0xdcf,
0xda2, 0xdbd, 0xdd9, 0xdba, 0xdd2, 0xdb1, 0xdca, 0xdb8, 0xda2, 0xdd4, 0xdbb, 0xddd,
0xdb8, 0xdcf, 0xd9a, 0xdd4, 0xdc0, 0xdd9, 0xdc3, 0xdcf, 0xdc3, 0xdca, 0xdb8, 0xdd2,
0xda9, 0xdca, 0xdc0, 0xdda, 0xdb1, 0xdc0, 0xdd4, 0xdbb, 0xdd4, 0xdb1, 0xddd, 0xdc6,
0xddd, 0xd9a, 0xdca, 0xdb1, 0xddd, 0xdb8, 0xdd2, 0xdba, 0xdcf, 0xdb4, 0xd9c, 0xddd,
0x20, 0xdb4, 0xd9c, 0xddd, 0xdb4, 0xdbd, 0xdcf, 0xdc0, 0xdd4, 0xdb4, 0xdd2, 0xda7,
0xdca, 0xd9a, 0xdd9, 0xdba, 0xdcf, 0xdb1, 0xdca, 0xdb4, 0xddc, 0xdb1, 0xdca, 0xdb4,
0xdd9, 0xdba, 0xdd2, 0xdb4, 0xddc, 0xdbb, 0xdca, 0xda7, 0xdca, 0x20, 0xdb8, 0xddc,
0xdbb, 0xdd9, 0xdc3, 0xdca, 0xdb6, 0xdd2, 0xdbb, 0xdbb, 0xddd, 0xda7, 0xddd, 0xdb1,
0xdca, 0xd9c, 0xdcf, 0xdc3, 0xdba, 0xdd2, 0xdb4, 0xdcf, 0xdb1, 0xdca, 0xda7, 0xdc4,
0xdd2, 0xdad, 0xdd2, 0xda7, 0xdbb, 0xdcf, 0xdc0, 0xdcf, 0xda7, 0xddc, 0xdb1, 0xdca,
0xd9c, 0xda7, 0xdcf, 0xdb4, 0xdd4, 0xdc0, 0xdda, 0xd9a, 0xdca, 0xdc0, 0xdd0, 0xdbd,
0xdd2, 0xdc3, 0xdca, 0x45, 0x6c, 0x2d, 0x41, 0x61, 0x69, 0xfa, 0x6e, 0x4d,
0x6f, 0x67, 0x61, 0x64, 0x69, 0x161, 0x6f, 0x53, 0x76, 0xe4, 0x74, 0xfd,
0x20, 0x54, 0x6f, 0x6d, 0xe1, 0x161, 0x4b, 0x61, 0x6a, 0x6d, 0x61, 0x6e,
0x69, 0x65, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x79, 0x53, 0x61,
0x6c, 0x76, 0xe1, 0x64, 0x6f, 0x72, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68,
0x2c, 0x20, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e, 0xe1, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20,
0x53, 0x65, 0x76, 0x65, 0x72, 0x6e, 0xe1, 0x20, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c,
0x20, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e, 0xe1, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x53, 0x76, 0xe4, 0x74, 0xfd, 0x20, 0x42, 0x61, 0x72,
0x74, 0x6f, 0x6c, 0x6f, 0x6d, 0x65, 0x6a, 0x53, 0x76, 0xe4, 0x74, 0xe1,
0x20, 0x4c, 0x75, 0x63, 0x69, 0x61, 0x53, 0x76, 0x2e, 0x20, 0x54, 0x6f,
0x6d, 0xe1, 0x161, 0x53, 0x76, 0x2e, 0x20, 0x56, 0x69, 0x6e, 0x63, 0x65,
0x6e, 0x74, 0x160, 0xf3, 0x77, 0x61, 0x41, 0x6c, 0x6d, 0x61, 0x2011, 0x41,
0x74, 0x61, 0x48, 0x6f, 0x10d, 0x69, 0x6d, 0x69, 0x6e, 0x6f, 0x76, 0x6f,
0x20, 0x4d, 0x65, 0x73, 0x74, 0x6f, 0x43, 0x68, 0x6f, 0x76, 0x64, 0x4b,
0x61, 0x72, 0xe1, 0x10d, 0x69, 0x4b, 0xe1, 0x74, 0x68, 0x6d, 0x61, 0x6e,
0x64, 0x75, 0x4e, 0x69, 0x6b, 0xf3, 0x7a, 0x69, 0x61, 0x55, 0x72, 0x61,
0x13e, 0x73, 0x6b, 0x50, 0x68, 0x6e, 0x6f, 0x6d, 0x20, 0x50, 0xe9, 0x6e,
0x68, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0xe1, 0x74, 0x61, 0x72, 0x55, 0x73,
0x165, 0x2d, 0x4e, 0x65, 0x72, 0x61, 0x4b, 0x61, 0x6e, 0xe1, 0x72, 0x73,
0x6b, 0x65, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x79, 0x4a, 0x75,
0x17e, 0x6e, 0xe1, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x53,
0x76, 0xe4, 0x74, 0xe1, 0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x41,
0x73, 0x74, 0x72, 0x61, 0x63, 0x68, 0xe1, 0x6e, 0x41, 0x74, 0xe9, 0x6e,
0x79, 0x42, 0x65, 0x6c, 0x65, 0x68, 0x72, 0x61, 0x64, 0x4b, 0x69, 0x161,
0x69, 0x148, 0x6f, 0x76, 0x13d, 0x75, 0x62, 0x13e, 0x61, 0x6e, 0x61, 0x4c,
0x75, 0x78, 0x65, 0x6d, 0x62, 0x75, 0x72, 0x73, 0x6b, 0x6f, 0x50, 0x61,
0x72, 0xed, 0x17e, 0x52, 0xed, 0x6d, 0x55, 0x13e, 0x6a, 0x61, 0x6e, 0x6f,
0x76, 0x73, 0x6b, 0x56, 0x69, 0x65, 0x64, 0x65, 0x148, 0x5a, 0xe1, 0x68,
0x72, 0x65, 0x62, 0x56, 0x69, 0x61, 0x6e, 0x6f, 0x10d, 0x6e, 0xfd, 0x20,
0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x4b, 0x65, 0x72, 0x67, 0x75, 0x65,
0x6c, 0x65, 0x6e, 0x79, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x79, 0x4d,
0x61, 0x75, 0x72, 0xed, 0x63, 0x69, 0x75, 0x73, 0x56, 0x65, 0x13e, 0x6b,
0x6f, 0x6e, 0x6f, 0x10d, 0x6e, 0xfd, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f,
0x76, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x53, 0x65, 0x76,
0x65, 0x72, 0x6e, 0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x43,
0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x53, 0x65, 0x76, 0x65, 0x72,
0x6e, 0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77,
0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x53, 0x65, 0x76, 0x65,
0x72, 0x6e, 0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x53, 0x76,
0x2e, 0x20, 0x4b, 0x72, 0x69, 0x161, 0x74, 0x6f, 0x66, 0x53, 0x76, 0x2e,
0x20, 0x54, 0x6f, 0x6d, 0x61, 0x17e, 0x53, 0x76, 0x2e, 0x20, 0x56, 0x69,
0x6e, 0x63, 0x65, 0x6e, 0x63, 0x69, 0x6a, 0x48, 0x6f, 0x161, 0x69, 0x6d,
0x69, 0x6e, 0x68, 0x4d, 0x75, 0x161, 0x6b, 0x61, 0x74, 0x4b, 0x69, 0x7a,
0x6c, 0x6f, 0x72, 0x64, 0x61, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x73, 0x6b,
0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x69, 0x5a, 0x65, 0x6c, 0x65, 0x6e,
0x6f, 0x72, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x69,
0x46, 0x65, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x69,
0x4a, 0x75, 0x17e, 0x6e, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x53, 0x76, 0x2e, 0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x42,
0x72, 0x75, 0x73, 0x65, 0x6c, 0x6a, 0x4b, 0xf6, 0x62, 0x65, 0x6e, 0x68,
0x61, 0x76, 0x6e, 0x44, 0x75, 0x6e, 0x61, 0x6a, 0x42, 0x6f, 0x17e, 0x69,
0x10d, 0x6e, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x4b, 0x6f, 0x6b, 0x6f,
0x73, 0x6f, 0x76, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x69, 0x56, 0x65,
0x6c, 0x69, 0x6b, 0x6f, 0x6e, 0x6f, 0x10d, 0x6e, 0x69, 0x20, 0x6f, 0x74,
0x6f, 0x6b, 0x41, 0x62, 0x69, 0x64, 0x6a, 0x61, 0x61, 0x6e, 0x41, 0x6c,
0x6a, 0x65, 0x65, 0x72, 0x69, 0x73, 0x42, 0x61, 0x6d, 0x61, 0x61, 0x6b,
0x6f, 0x42, 0x61, 0x61, 0x67, 0x69, 0x42, 0x69, 0x73, 0x61, 0x61, 0x77,
0x42, 0x61, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x72, 0x65, 0x42, 0x61, 0x72,
0x61, 0x73, 0x61, 0x66, 0x69, 0x6c, 0x51, 0x61, 0x61, 0x68, 0x69, 0x72,
0x61, 0x4b, 0x61, 0x73, 0x61, 0x62, 0x61, 0x6c, 0x61, 0x61, 0x6e, 0x6b,
0x61, 0x43, 0x6f, 0x6e, 0x61, 0x6b, 0x72, 0x69, 0x44, 0x61, 0x72, 0x65,
0x73, 0x61, 0x6c, 0x61, 0x61, 0x6d, 0x4a, 0x69, 0x62, 0x75, 0x75, 0x74,
0x69, 0x44, 0x6f, 0x75, 0x61, 0x61, 0x6c, 0x61, 0x45, 0x6c, 0x20, 0x43,
0x65, 0x79, 0x75, 0x75, 0x6e, 0x46, 0x69, 0x72, 0x69, 0x69, 0x74, 0x61,
0x77, 0x6e, 0x47, 0x61, 0x62, 0x72, 0x6f, 0x6f, 0x6e, 0x48, 0x61, 0x72,
0x61, 0x61, 0x72, 0x65, 0x4a, 0x6f, 0x68, 0x61, 0x6e, 0x73, 0x62, 0x61,
0x61, 0x67, 0x4b, 0x61, 0x6d, 0x62, 0x61, 0x61, 0x6c, 0x61, 0x4b, 0x68,
0x61, 0x72, 0x74, 0x75, 0x75, 0x6d, 0x4c, 0x61, 0x61, 0x67, 0x6f, 0x6f,
0x73, 0x4c, 0x69, 0x62, 0x72, 0x65, 0x66, 0x69, 0x6c, 0x4c, 0x6f, 0x6f,
0x6d, 0x4c, 0x75, 0x77, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x4c, 0x75, 0x62,
0x75, 0x6d, 0x62, 0x61, 0x61, 0x73, 0x68, 0x69, 0x4c, 0x75, 0x73, 0x61,
0x61, 0x6b, 0x61, 0x4d, 0x61, 0x62, 0x75, 0x75, 0x74, 0x6f, 0x4d, 0x61,
0x73, 0x65, 0x65, 0x72, 0x6f, 0x4d, 0x61, 0x62, 0x61, 0x62, 0x61, 0x61,
0x6e, 0x65, 0x4d, 0x6f, 0x6e, 0x72, 0x6f, 0x66, 0x69, 0x79, 0x61, 0x4e,
0x61, 0x79, 0x72, 0x6f, 0x6f, 0x62, 0x69, 0x49, 0x6e, 0x6a, 0x61, 0x6d,
0x69, 0x6e, 0x61, 0x4e, 0x6f, 0x6f, 0x6b, 0x6a, 0x6f, 0x74, 0x42, 0x6f,
0x6f, 0x72, 0x74, 0x6f, 0x2d, 0x4e, 0x6f, 0x6f, 0x66, 0x6f, 0x53, 0x61,
0x77, 0x20, 0x54, 0x6f, 0x6f, 0x6d, 0x54, 0x69, 0x72, 0x69, 0x62, 0x6f,
0x6f, 0x6c, 0x69, 0x54, 0x75, 0x75, 0x6e, 0x69, 0x73, 0x57, 0x69, 0x6e,
0x64, 0x68, 0x6f, 0x6f, 0x6b, 0x41, 0x6e, 0x6a, 0x6f, 0x72, 0x61, 0x67,
0x65, 0x41, 0x6e, 0x67, 0x75, 0x77, 0x69, 0x6c, 0x61, 0x41, 0x6e, 0x74,
0x69, 0x67, 0x75, 0x77, 0x61, 0x41, 0x72, 0x61, 0x67, 0x75, 0x77, 0x61,
0x79, 0x6e, 0x61, 0x42, 0x75, 0x77, 0x65, 0x6e, 0x6f, 0x73, 0x20, 0x41,
0x79, 0x72, 0x69, 0x73, 0x4b, 0x61, 0x74, 0x61, 0x6d, 0x61, 0x61, 0x72,
0x6b, 0x61, 0x4b, 0x6f, 0x72, 0x64, 0x6f, 0x6f, 0x62, 0x61, 0x4c, 0x61,
0x20, 0x52, 0x69, 0x79, 0x6f, 0x6a, 0x61, 0x4d, 0x65, 0x65, 0x6e, 0x64,
0x6f, 0x6f, 0x73, 0x61, 0x52, 0x69, 0x79, 0x6f, 0x20, 0x4a, 0x61, 0x6c,
0x65, 0x6a, 0x6f, 0x73, 0x53, 0x61, 0x6e, 0x20, 0x4a, 0x75, 0x77, 0x61,
0x61, 0x6e, 0x53, 0x61, 0x6e, 0x20, 0x4c, 0x75, 0x77, 0x69, 0x73, 0x54,
0x75, 0x6b, 0x75, 0x75, 0x6d, 0x61, 0x61, 0x6e, 0x55, 0x73, 0x68, 0x75,
0x61, 0x61, 0x79, 0x41, 0x73, 0x75, 0x6e, 0x6b, 0x69, 0x79, 0x6f, 0x6e,
0x41, 0x74, 0x69, 0x6b, 0x6f, 0x6b, 0x61, 0x61, 0x6e, 0x42, 0x61, 0x61,
0x68, 0x69, 0x79, 0x61, 0x42, 0x61, 0x68, 0x69, 0x79, 0x61, 0x20, 0x42,
0x61, 0x6e, 0x64, 0x65, 0x72, 0x61, 0x73, 0x42, 0x65, 0x6c, 0x69, 0x69,
0x73, 0x65, 0x42, 0x61, 0x6c, 0x61, 0x6e, 0x6b, 0x2d, 0x53, 0x61, 0x62,
0x6c, 0x6f, 0x6e, 0x42, 0x6f, 0x77, 0x20, 0x46, 0x69, 0x73, 0x74, 0x61,
0x4b, 0x61, 0x6d, 0x62, 0x69, 0x72, 0x69, 0x69, 0x6a, 0x20, 0x42, 0x61,
0x61, 0x79, 0x4b, 0x61, 0x61, 0x6d, 0x62, 0x6f, 0x20, 0x47, 0x61, 0x72,
0x61, 0x6e, 0x64, 0x69, 0x4b, 0x61, 0x61, 0x6e, 0x6b, 0x75, 0x75, 0x6e,
0x4b, 0x61, 0x72, 0x61, 0x6b, 0x61, 0x61, 0x73, 0x4b, 0x61, 0x79, 0x65,
0x65, 0x6e, 0x4b, 0x65, 0x79, 0x6d, 0x61, 0x61, 0x6e, 0x4a, 0x69, 0x6b,
0x61, 0x61, 0x67, 0x6f, 0x4a, 0x69, 0x77, 0x61, 0x61, 0x68, 0x75, 0x75,
0x4d, 0x61, 0x67, 0x61, 0x61, 0x6c, 0x61, 0x64, 0x61, 0x20, 0x4a, 0x75,
0x61, 0x72, 0x65, 0x7a, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x20, 0x52, 0x69,
0x69, 0x6b, 0x61, 0x4b, 0x61, 0x72, 0x65, 0x73, 0x74, 0x6f, 0x6f, 0x6e,
0x4b, 0x75, 0x79, 0x61, 0x61, 0x62, 0x61, 0x4b, 0x75, 0x72, 0x61, 0x6b,
0x6f, 0x6f, 0x77, 0x44, 0x61, 0x61, 0x6e, 0x6d, 0x61, 0x61, 0x6b, 0x73,
0x68, 0x61, 0x61, 0x6e, 0x44, 0x6f, 0x6f, 0x73, 0x61, 0x6e, 0x44, 0x6f,
0x6f, 0x73, 0x61, 0x6e, 0x20, 0x4b, 0x69, 0x72, 0x65, 0x65, 0x6b, 0x44,
0x65, 0x6e, 0x66, 0x61, 0x72, 0x44, 0x65, 0x74, 0x6f, 0x72, 0x6f, 0x79,
0x74, 0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x69, 0x69, 0x6b, 0x61, 0x49, 0x69,
0x72, 0x75, 0x6e, 0x65, 0x65, 0x62, 0x45, 0x6c, 0x20, 0x53, 0x61, 0x6c,
0x66, 0x61, 0x64, 0x6f, 0x6f, 0x72, 0x46, 0x6f, 0x6f, 0x74, 0x20, 0x4e,
0x65, 0x6c, 0x73, 0x6f, 0x6e, 0x46, 0x6f, 0x6f, 0x74, 0x61, 0x6c, 0x65,
0x73, 0x61, 0x47, 0x61, 0x6c, 0x65, 0x79, 0x73, 0x20, 0x42, 0x61, 0x61,
0x79, 0x47, 0x75, 0x75, 0x73, 0x20, 0x42, 0x61, 0x61, 0x79, 0x47, 0x61,
0x72, 0x61, 0x61, 0x6e, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x47, 0x61, 0x72,
0x65, 0x6e, 0x61, 0x61, 0x64, 0x61, 0x47, 0x75, 0x77, 0x61, 0x64, 0x65,
0x6c, 0x75, 0x75, 0x62, 0x47, 0x75, 0x77, 0x61, 0x74, 0x65, 0x6d, 0x61,
0x61, 0x6c, 0x61, 0x47, 0x75, 0x77, 0x61, 0x79, 0x61, 0x71, 0x75, 0x77,
0x69, 0x6c, 0x47, 0x75, 0x79, 0x61, 0x61, 0x6e, 0x61, 0x48, 0x61, 0x6c,
0x69, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x48, 0x61, 0x66, 0x61, 0x61, 0x6e,
0x61, 0x48, 0x61, 0x72, 0x6d, 0x6f, 0x73, 0x69, 0x6c, 0x6f, 0x49, 0x6e,
0x64, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x61, 0x62, 0x6f, 0x6c, 0x69, 0x73,
0x4e, 0x6f, 0x6b, 0x69, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x79,
0x61, 0x61, 0x6e, 0x61, 0x4d, 0x61, 0x72, 0x65, 0x65, 0x6e, 0x67, 0x6f,
0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x61, 0x42,
0x65, 0x74, 0x65, 0x73, 0x62, 0x61, 0x61, 0x67, 0x2c, 0x20, 0x49, 0x6e,
0x64, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x61, 0x54, 0x65, 0x6c, 0x20, 0x53,
0x69, 0x74, 0x69, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x61,
0x6e, 0x61, 0x46, 0x65, 0x65, 0x66, 0x61, 0x61, 0x79, 0x2c, 0x20, 0x49,
0x6e, 0x64, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x61, 0x46, 0x69, 0x6e, 0x73,
0x65, 0x65, 0x6e, 0x65, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x79,
0x61, 0x61, 0x6e, 0x61, 0x57, 0x69, 0x6e, 0x61, 0x61, 0x6d, 0x61, 0x6b,
0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x61, 0x49,
0x6e, 0x75, 0x75, 0x66, 0x69, 0x6b, 0x49, 0x71, 0x61, 0x61, 0x6c, 0x75,
0x75, 0x74, 0x4a, 0x75, 0x6e, 0x69, 0x79, 0x75, 0x75, 0x4c, 0x75, 0x75,
0x73, 0x66, 0x69, 0x6c, 0x65, 0x4d, 0x6f, 0x6e, 0x74, 0x69, 0x73, 0x65,
0x65, 0x6c, 0x6f, 0x2c, 0x20, 0x4b, 0x65, 0x6e, 0x74, 0x61, 0x6b, 0x69,
0x4b, 0x69, 0x72, 0x61, 0x61, 0x6c, 0x65, 0x6e, 0x64, 0x69, 0x6b, 0x4c,
0x61, 0x61, 0x20, 0x42, 0x61, 0x61, 0x73, 0x4c, 0x69, 0x69, 0x6d, 0x61,
0x4c, 0x6f, 0x6f, 0x73, 0x20, 0x41, 0x6e, 0x6a, 0x61, 0x6c, 0x69, 0x73,
0x4c, 0x6f, 0x6f, 0x77, 0x61, 0x20, 0x42, 0x69, 0x72, 0x69, 0x6e, 0x73,
0x65, 0x73, 0x20, 0x4b, 0x75, 0x77, 0x61, 0x61, 0x74, 0x61, 0x4d, 0x61,
0x61, 0x73, 0x69, 0x69, 0x79, 0x6f, 0x4d, 0x61, 0x6e, 0x61, 0x61, 0x67,
0x75, 0x77, 0x61, 0x4d, 0x61, 0x6e, 0x61, 0x61, 0x75, 0x75, 0x73, 0x4d,
0x61, 0x61, 0x72, 0x69, 0x67, 0x6f, 0x74, 0x4d, 0x61, 0x61, 0x72, 0x74,
0x69, 0x6e, 0x69, 0x69, 0x6b, 0x75, 0x79, 0x75, 0x75, 0x4d, 0x61, 0x7a,
0x61, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x4d, 0x65, 0x6e, 0x6f, 0x6f, 0x6d,
0x69, 0x6e, 0x65, 0x65, 0x4d, 0x65, 0x72, 0x69, 0x69, 0x64, 0x61, 0x4d,
0x65, 0x74, 0x6c, 0x61, 0x61, 0x6b, 0x61, 0x74, 0x6c, 0x61, 0x4d, 0x69,
0x69, 0x71, 0x75, 0x75, 0x6c, 0x6f, 0x6e, 0x4d, 0x6f, 0x6f, 0x6e, 0x6b,
0x74, 0x6f, 0x6f, 0x6e, 0x4d, 0x6f, 0x6f, 0x6e, 0x74, 0x65, 0x65, 0x72,
0x65, 0x79, 0x4d, 0x6f, 0x6f, 0x6e, 0x74, 0x61, 0x66, 0x69, 0x69, 0x64,
0x69, 0x79, 0x6f, 0x4d, 0x6f, 0x6f, 0x6e, 0x74, 0x73, 0x65, 0x72, 0x61,
0x61, 0x74, 0x4e, 0x61, 0x73, 0x61, 0x61, 0x77, 0x4e, 0x69, 0x79, 0x75,
0x75, 0x20, 0x59, 0x6f, 0x6f, 0x6b, 0x42, 0x69, 0x79, 0x75, 0x75, 0x6c,
0x61, 0x68, 0x2c, 0x20, 0x57, 0x61, 0x71, 0x6f, 0x6f, 0x79, 0x69, 0x67,
0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x6f, 0x74, 0x61, 0x42, 0x61, 0x72,
0x74, 0x61, 0x6d, 0x61, 0x68, 0x61, 0x2c, 0x20, 0x57, 0x61, 0x71, 0x6f,
0x6f, 0x79, 0x69, 0x67, 0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x6f, 0x74,
0x61, 0x4e, 0x69, 0x79, 0x75, 0x75, 0x20, 0x53, 0x61, 0x61, 0x6c, 0x65,
0x6d, 0x2c, 0x20, 0x57, 0x61, 0x71, 0x6f, 0x6f, 0x79, 0x69, 0x67, 0x61,
0x20, 0x44, 0x61, 0x6b, 0x6f, 0x6f, 0x74, 0x61, 0x4f, 0x6a, 0x69, 0x6e,
0x61, 0x61, 0x67, 0x61, 0x42, 0x61, 0x6e, 0x61, 0x61, 0x6d, 0x61, 0x46,
0x6f, 0x6f, 0x6e, 0x69, 0x6b, 0x69, 0x73, 0x42, 0x6f, 0x6f, 0x72, 0x74,
0x2d, 0x61, 0x77, 0x2d, 0x42, 0x69, 0x72, 0x69, 0x69, 0x6e, 0x73, 0x42,
0x6f, 0x6f, 0x72, 0x74, 0x20, 0x6f, 0x66, 0x20, 0x49, 0x73, 0x62, 0x61,
0x79, 0x6e, 0x42, 0x6f, 0x6f, 0x72, 0x74, 0x61, 0x20, 0x52, 0x69, 0x69,
0x6b, 0x6f, 0x42, 0x75, 0x6e, 0x74, 0x61, 0x20, 0x41, 0x72, 0x69, 0x6e,
0x61, 0x61, 0x73, 0x52, 0x61, 0x61, 0x6e, 0x6b, 0x69, 0x6e, 0x20, 0x49,
0x6e, 0x6c, 0x65, 0x65, 0x74, 0x52, 0x65, 0x63, 0x65, 0x79, 0x66, 0x52,
0x65, 0x6a, 0x69, 0x69, 0x6e, 0x61, 0x52, 0x65, 0x73, 0x6f, 0x6c, 0x75,
0x75, 0x74, 0x52, 0x69, 0x79, 0x6f, 0x20, 0x42, 0x61, 0x72, 0x61, 0x61,
0x6e, 0x6b, 0x6f, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x65, 0x65, 0x6d,
0x53, 0x61, 0x6e, 0x74, 0x69, 0x79, 0x61, 0x61, 0x67, 0x6f, 0x53, 0x61,
0x61, 0x6e, 0x74, 0x6f, 0x20, 0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x67, 0x6f,
0x53, 0x61, 0x61, 0x77, 0x20, 0x42, 0x6f, 0x6f, 0x6c, 0x6f, 0x49, 0x74,
0x6f, 0x71, 0x6f, 0x72, 0x74, 0x6f, 0x6f, 0x6d, 0x69, 0x69, 0x74, 0x53,
0x69, 0x69, 0x74, 0x6b, 0x61, 0x53, 0x74, 0x2e, 0x20, 0x42, 0x61, 0x61,
0x72, 0x74, 0x65, 0x6c, 0x65, 0x6d, 0x69, 0x53, 0x74, 0x2e, 0x20, 0x4a,
0x6f, 0x6f, 0x6e, 0x53, 0x74, 0x2e, 0x20, 0x4b, 0x69, 0x74, 0x69, 0x73,
0x53, 0x74, 0x2e, 0x20, 0x4c, 0x75, 0x73, 0x69, 0x79, 0x61, 0x53, 0x74,
0x2e, 0x20, 0x54, 0x6f, 0x6f, 0x6d, 0x61, 0x73, 0x53, 0x74, 0x2e, 0x20,
0x46, 0x69, 0x6e, 0x73, 0x65, 0x6e, 0x74, 0x49, 0x73, 0x77, 0x69, 0x66,
0x74, 0x20, 0x4b, 0x61, 0x72, 0x65, 0x6e, 0x74, 0x54, 0x65, 0x67, 0x75,
0x73, 0x69, 0x67, 0x61, 0x6c, 0x62, 0x61, 0x54, 0x75, 0x75, 0x6c, 0x54,
0x69, 0x6a, 0x75, 0x77, 0x61, 0x61, 0x6e, 0x61, 0x54, 0x6f, 0x72, 0x74,
0x6f, 0x6f, 0x6c, 0x61, 0x46, 0x61, 0x6e, 0x6b, 0x75, 0x75, 0x66, 0x61,
0x72, 0x57, 0x61, 0x79, 0x74, 0x68, 0x6f, 0x72, 0x61, 0x73, 0x57, 0x69,
0x6e, 0x69, 0x62, 0x65, 0x67, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x61, 0x61,
0x74, 0x4b, 0x61, 0x79, 0x73, 0x65, 0x65, 0x44, 0x61, 0x66, 0x69, 0x73,
0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x20, 0x64, 0x2019, 0x75, 0x72, 0x66,
0x69, 0x6c, 0x65, 0x4d, 0x61, 0x6b, 0x71, 0x75, 0x77, 0x61, 0x72, 0x69,
0x79, 0x42, 0x61, 0x61, 0x6d, 0x61, 0x72, 0x53, 0x69, 0x79, 0x6f, 0x77,
0x61, 0x54, 0x6f, 0x72, 0x6f, 0x6f, 0x6c, 0x46, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x4c, 0x6f, 0x6e, 0x6a, 0x69, 0x72, 0x62, 0x79, 0x65, 0x65, 0x6e,
0x43, 0x61, 0x64, 0x61, 0x6e, 0x41, 0x6d, 0x6d, 0x61, 0x61, 0x6e, 0x41,
0x6e, 0x61, 0x64, 0x69, 0x79, 0x72, 0x41, 0x6b, 0x74, 0x61, 0x77, 0x41,
0x74, 0x69, 0x79, 0x72, 0x61, 0x77, 0x42, 0x61, 0x71, 0x64, 0x61, 0x61,
0x64, 0x42, 0x61, 0x78, 0x72, 0x65, 0x79, 0x6e, 0x42, 0x61, 0x6e, 0x67,
0x6b, 0x6f, 0x6f, 0x6b, 0x42, 0x61, 0x72, 0x6e, 0x61, 0x61, 0x75, 0x75,
0x6c, 0x42, 0x65, 0x79, 0x72, 0x75, 0x75, 0x64, 0x42, 0x75, 0x72, 0x75,
0x6e, 0x65, 0x79, 0x4a, 0x69, 0x69, 0x74, 0x61, 0x44, 0x69, 0x6d, 0x69,
0x73, 0x68, 0x69, 0x71, 0x44, 0x68, 0x61, 0x61, 0x6b, 0x61, 0x51, 0x61,
0x73, 0x61, 0x48, 0x6f, 0x6f, 0x20, 0x4a, 0x69, 0x20, 0x4d, 0x69, 0x68,
0x20, 0x53, 0x69, 0x74, 0x69, 0x48, 0x6f, 0x6f, 0x6e, 0x67, 0x20, 0x4b,
0x6f, 0x6f, 0x6e, 0x67, 0x48, 0x6f, 0x66, 0x75, 0x64, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x69, 0x6b, 0x4a, 0x61, 0x6b, 0x61, 0x61, 0x72, 0x74,
0x61, 0x4a, 0x61, 0x79, 0x61, 0x62, 0x75, 0x72, 0x61, 0x4a, 0x65, 0x65,
0x72, 0x75, 0x73, 0x61, 0x61, 0x6c, 0x65, 0x6d, 0x4b, 0x61, 0x61, 0x62,
0x75, 0x75, 0x6c, 0x4b, 0x61, 0x6d, 0x6b, 0x61, 0x74, 0x6b, 0x61, 0x4b,
0x61, 0x72, 0x61, 0x61, 0x6a, 0x69, 0x4b, 0x68, 0x61, 0x6e, 0x64, 0x69,
0x79, 0x67, 0x61, 0x4b, 0x6f, 0x6c, 0x6b, 0x61, 0x61, 0x74, 0x61, 0x4b,
0x61, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x61,
0x4b, 0x75, 0x61, 0x6c, 0x61, 0x20, 0x4c, 0x61, 0x6d, 0x62, 0x75, 0x75,
0x72, 0x4b, 0x75, 0x6a, 0x69, 0x6e, 0x67, 0x4d, 0x61, 0x6b, 0x6f, 0x77,
0x4d, 0x61, 0x67, 0x65, 0x64, 0x61, 0x6e, 0x4d, 0x61, 0x6e, 0x69, 0x69,
0x6c, 0x61, 0x4e, 0x6f, 0x66, 0x6f, 0x6b, 0x75, 0x73, 0x6e, 0x65, 0x74,
0x73, 0x69, 0x6b, 0x4e, 0x6f, 0x66, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x69, 0x6b, 0x42, 0x65, 0x6e, 0x6f, 0x6d, 0x20, 0x42, 0x65, 0x6e,
0x42, 0x6f, 0x74, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x61, 0x6b, 0x42, 0x6f,
0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x51, 0x61, 0x64, 0x64,
0x61, 0x72, 0x51, 0x69, 0x79, 0x73, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x52,
0x69, 0x79, 0x61, 0x61, 0x64, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x6b, 0x61,
0x61, 0x6e, 0x53, 0x68, 0x61, 0x6e, 0x67, 0x68, 0x61, 0x61, 0x79, 0x53,
0x69, 0x6e, 0x67, 0x61, 0x62, 0x75, 0x75, 0x72, 0x53, 0x61, 0x72, 0x65,
0x64, 0x6e, 0x6f, 0x6b, 0x6f, 0x6c, 0x65, 0x79, 0x6d, 0x69, 0x73, 0x6b,
0x54, 0x65, 0x79, 0x62, 0x65, 0x79, 0x54, 0x6f, 0x73, 0x68, 0x6b, 0x65,
0x65, 0x6e, 0x74, 0x54, 0x69, 0x62, 0x69, 0x6c, 0x69, 0x73, 0x69, 0x54,
0x65, 0x68, 0x72, 0x61, 0x61, 0x6e, 0x46, 0x69, 0x79, 0x61, 0x61, 0x6e,
0x74, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x46, 0x61, 0x6c, 0x61, 0x64, 0x69,
0x66, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x61, 0x61, 0x67, 0x59, 0x65, 0x72, 0x65, 0x66,
0x61, 0x6e, 0x41, 0x73, 0x6f, 0x72, 0x65, 0x73, 0x42, 0x61, 0x72, 0x6d,
0x75, 0x75, 0x64, 0x61, 0x4b, 0x65, 0x79, 0x62, 0x20, 0x46, 0x61, 0x61,
0x72, 0x64, 0x65, 0x46, 0x61, 0x72, 0x6f, 0x77, 0x52, 0x65, 0x79, 0x6b,
0x6a, 0x61, 0x66, 0x69, 0x6b, 0x53, 0x6f, 0x77, 0x74, 0x20, 0x4a, 0x6f,
0x6f, 0x72, 0x6a, 0x69, 0x79, 0x61, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e,
0x6c, 0x65, 0x79, 0x41, 0x64, 0x65, 0x6c, 0x61, 0x79, 0x64, 0x65, 0x42,
0x69, 0x72, 0x69, 0x73, 0x62, 0x61, 0x6e, 0x42, 0x6f, 0x72, 0x6f, 0x6b,
0x65, 0x6e, 0x20, 0x48, 0x69, 0x6c, 0x48, 0x75, 0x62, 0x61, 0x61, 0x72,
0x74, 0x4c, 0x6f, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x4d, 0x65, 0x6c, 0x62,
0x6f, 0x6f, 0x6e, 0x42, 0x65, 0x72, 0x74, 0x41, 0x6d, 0x73, 0x74, 0x65,
0x72, 0x64, 0x61, 0x61, 0x6d, 0x41, 0x6e, 0x64, 0x6f, 0x6f, 0x72, 0x61,
0x41, 0x73, 0x74, 0x61, 0x72, 0x61, 0x6b, 0x68, 0x61, 0x61, 0x6e, 0x41,
0x74, 0x65, 0x6e, 0x73, 0x42, 0x65, 0x6c, 0x67, 0x61, 0x72, 0x61, 0x79,
0x64, 0x68, 0x42, 0x61, 0x72, 0x6c, 0x69, 0x69, 0x6e, 0x42, 0x61, 0x72,
0x61, 0x74, 0x69, 0x73, 0x6c, 0x61, 0x66, 0x61, 0x42, 0x61, 0x72, 0x61,
0x73, 0x61, 0x6c, 0x69, 0x73, 0x42, 0x75, 0x6a, 0x61, 0x72, 0x65, 0x73,
0x74, 0x42, 0x75, 0x64, 0x61, 0x62, 0x65, 0x73, 0x74, 0x42, 0x75, 0x73,
0x69, 0x6e, 0x67, 0x65, 0x65, 0x6e, 0x4a, 0x69, 0x73, 0x69, 0x6e, 0x61,
0x61, 0x77, 0x4b, 0x6f, 0x62, 0x65, 0x6e, 0x68, 0x61, 0x61, 0x67, 0x61,
0x6e, 0x44, 0x68, 0x61, 0x61, 0x62, 0x6c, 0x69, 0x6e, 0x47, 0x65, 0x65,
0x72, 0x6e, 0x69, 0x73, 0x69, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x69, 0x73,
0x6b, 0x69, 0x41, 0x79, 0x6c, 0x65, 0x20, 0x6f, 0x66, 0x20, 0x4d, 0x61,
0x61, 0x6e, 0x49, 0x73, 0x74, 0x61, 0x6e, 0x62, 0x75, 0x75, 0x6c, 0x4a,
0x61, 0x61, 0x72, 0x73, 0x65, 0x79, 0x4b, 0x69, 0x72, 0x6f, 0x6f, 0x66,
0x4b, 0x69, 0x79, 0x65, 0x65, 0x66, 0x4c, 0x75, 0x62, 0x61, 0x6c, 0x61,
0x61, 0x6e, 0x61, 0x4c, 0x61, 0x6e, 0x64, 0x61, 0x6e, 0x4c, 0x75, 0x6b,
0x73, 0x65, 0x6d, 0x62, 0x65, 0x72, 0x67, 0x4d, 0x61, 0x64, 0x72, 0x69,
0x69, 0x64, 0x4d, 0x61, 0x61, 0x72, 0x69, 0x68, 0x61, 0x61, 0x6d, 0x4d,
0x69, 0x6e, 0x69, 0x73, 0x6b, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x77, 0x42,
0x61, 0x61, 0x72, 0x69, 0x69, 0x73, 0x42, 0x6f, 0x64, 0x67, 0x6f, 0x72,
0x69, 0x6b, 0x61, 0x42, 0x61, 0x72, 0x61, 0x61, 0x67, 0x53, 0x61, 0x6e,
0x20, 0x4d, 0x61, 0x72, 0x69, 0x69, 0x6e, 0x6f, 0x53, 0x61, 0x72, 0x61,
0x79, 0x65, 0x65, 0x66, 0x6f, 0x53, 0x61, 0x72, 0x61, 0x74, 0x6f, 0x6f,
0x66, 0x53, 0x69, 0x6d, 0x66, 0x65, 0x72, 0x6f, 0x62, 0x6f, 0x6c, 0x49,
0x73, 0x6b, 0x6f, 0x6f, 0x62, 0x6a, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x6b,
0x68, 0x6f, 0x6f, 0x6d, 0x54, 0x61, 0x61, 0x6c, 0x69, 0x6e, 0x54, 0x69,
0x72, 0x61, 0x61, 0x6e, 0x65, 0x55, 0x6c, 0x79, 0x61, 0x6e, 0x6f, 0x66,
0x69, 0x73, 0x6b, 0x46, 0x61, 0x64, 0x75, 0x75, 0x73, 0x46, 0x61, 0x74,
0x69, 0x6b, 0x61, 0x61, 0x6e, 0x46, 0x69, 0x79, 0x65, 0x65, 0x6e, 0x61,
0x46, 0x69, 0x6e, 0x69, 0x79, 0x75, 0x75, 0x73, 0x46, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x53, 0x61, 0x71, 0x72, 0x69, 0x62, 0x53,
0x75, 0x75, 0x72, 0x69, 0x6b, 0x68, 0x41, 0x6e, 0x74, 0x61, 0x6e, 0x61,
0x6e, 0x61, 0x72, 0x69, 0x66, 0x6f, 0x4a, 0x61, 0x67, 0x6f, 0x73, 0x4b,
0x69, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x4b, 0x65, 0x72, 0x67,
0x61, 0x6c, 0x65, 0x6e, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x66, 0x69, 0x73,
0x4d, 0x6f, 0x72, 0x69, 0x73, 0x68, 0x69, 0x79, 0x61, 0x61, 0x73, 0x4d,
0x61, 0x79, 0x6f, 0x6f, 0x74, 0x65, 0x52, 0x69, 0x79, 0x75, 0x75, 0x6e,
0x69, 0x79, 0x6f, 0x6e, 0x4f, 0x77, 0x6b, 0x6c, 0x61, 0x61, 0x6e, 0x42,
0x6f, 0x6f, 0x67, 0x61, 0x79, 0x6e, 0x66, 0x69, 0x6c, 0x4a, 0x61, 0x74,
0x61, 0x6d, 0x4a, 0x75, 0x75, 0x6b, 0x49, 0x69, 0x73, 0x74, 0x61, 0x72,
0x66, 0x61, 0x6b, 0x6f, 0x66, 0x6f, 0x47, 0x61, 0x6c, 0x61, 0x62, 0x61,
0x67, 0x6f, 0x73, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x79, 0x72, 0x43, 0x75,
0x61, 0x64, 0x61, 0x6c, 0x6b, 0x61, 0x6e, 0x61, 0x6c, 0x47, 0x75, 0x77,
0x61, 0x6d, 0x4b, 0x69, 0x72, 0x69, 0x74, 0x69, 0x6d, 0x61, 0x61, 0x74,
0x69, 0x4b, 0x6f, 0x73, 0x72, 0x69, 0x69, 0x4b, 0x75, 0x77, 0x61, 0x6a,
0x61, 0x6c, 0x65, 0x79, 0x6e, 0x4d, 0x61, 0x6a, 0x72, 0x6f, 0x4d, 0x61,
0x72, 0x71, 0x75, 0x77, 0x65, 0x73, 0x61, 0x73, 0x4e, 0x61, 0x77, 0x72,
0x6f, 0x6f, 0x4e, 0x6f, 0x6f, 0x72, 0x66, 0x65, 0x6b, 0x4e, 0x6f, 0x6f,
0x6d, 0x69, 0x79, 0x61, 0x42, 0x61, 0x67, 0x6f, 0x20, 0x42, 0x61, 0x67,
0x6f, 0x42, 0x61, 0x6c, 0x61, 0x77, 0x42, 0x69, 0x74, 0x6b, 0x61, 0x79,
0x72, 0x6e, 0x42, 0x6f, 0x6e, 0x62, 0x65, 0x79, 0x42, 0x6f, 0x6f, 0x72,
0x74, 0x20, 0x4d, 0x6f, 0x72, 0x65, 0x73, 0x62, 0x69, 0x53, 0x65, 0x79,
0x62, 0x61, 0x6e, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x74, 0x61, 0x62, 0x75,
0x57, 0x61, 0x6c, 0x69, 0x73, 0x42, 0x69, 0x73, 0xe1, 0x75, 0x44, 0x61,
0x72, 0x20, 0x65, 0x73, 0x2d, 0x53, 0x61, 0x6c, 0x61, 0x6d, 0x53, 0x61,
0x6e, 0x74, 0x6f, 0x20, 0x54, 0x6f, 0x6d, 0xe9, 0x4c, 0x6f, 0x73, 0x20,
0xc1, 0x6e, 0x67, 0x65, 0x6c, 0x65, 0x73, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0xf3, 0x6e, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d,
0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x53, 0x61, 0x6e, 0x20, 0x43, 0x72,
0x69, 0x73, 0x74, 0xf3, 0x62, 0x61, 0x6c, 0x44, 0x75, 0x62, 0xe1, 0x69,
0x43, 0x69, 0x75, 0x64, 0x61, 0x64, 0x20, 0x48, 0x6f, 0x20, 0x43, 0x68,
0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x4a, 0x65, 0x72, 0x75, 0x73, 0x61,
0x6c, 0xe9, 0x6e, 0x53, 0x68, 0x61, 0x6e, 0x67, 0x68, 0xe1, 0x69, 0x54,
0x61, 0x69, 0x70, 0xe9, 0x69, 0x55, 0x6c, 0xe1, 0x6e, 0x20, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x59, 0x61, 0x6e, 0x67, 0xf3, 0x6e, 0x20, 0x28, 0x52,
0x61, 0x6e, 0x67, 0xfa, 0x6e, 0x29, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x49, 0x73, 0x6c, 0x61, 0x73,
0x20, 0x46, 0x65, 0x72, 0x6f, 0x65, 0x4e, 0x61, 0x76, 0x69, 0x64, 0x61,
0x64, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73,
0x63, 0x75, 0x61, 0x53, 0x61, 0x69, 0x70, 0xe1, 0x6e, 0x43, 0x6f, 0x6e,
0x61, 0x6b, 0x72, 0x79, 0x46, 0x75, 0x65, 0x72, 0x74, 0x65, 0x20, 0x4e,
0x65, 0x6c, 0x73, 0x6f, 0x6e, 0x4e, 0x61, 0x73, 0xe1, 0x75, 0x53, 0x61,
0x6e, 0x74, 0x6f, 0x20, 0x54, 0x6f, 0x6d, 0xe1, 0x73, 0x4b, 0x68, 0x61,
0x6e, 0x64, 0x79, 0x67, 0x61, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x43,
0x61, 0x6e, 0x61, 0x72, 0x69, 0x61, 0x73, 0x47, 0x75, 0x65, 0x72, 0x6e,
0x73, 0x65, 0x79, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x57, 0x61, 0x6b, 0x65,
0x42, 0x75, 0x79, 0x75, 0x6d, 0x62, 0x75, 0x72, 0x61, 0x52, 0x69, 0x6f,
0x20, 0x42, 0x72, 0x61, 0x6e, 0x63, 0x6f, 0x41, 0x6c, 0x6d, 0x61, 0x74,
0xfd, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0xe9, 0x59, 0x69, 0x62, 0x75, 0x74,
0xed, 0x4e, 0x61, 0x73, 0x73, 0x61, 0x75, 0x41, 0x73, 0x74, 0x72, 0x61,
0x6b, 0x68, 0x61, 0x6e, 0x55, 0x6c, 0x79, 0x61, 0x6e, 0x6f, 0x76, 0x73,
0x6b, 0x4a, 0x69, 0x6a, 0x69, 0x20, 0x6c, 0x61, 0x20, 0x4d, 0x65, 0x78,
0x69, 0x63, 0x6f, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x4b,
0x75, 0x73, 0x69, 0x6e, 0x69, 0x42, 0x61, 0x62, 0x61, 0x64, 0x6f, 0x73,
0x6b, 0x75, 0x72, 0x61, 0x6b, 0x61, 0x6f, 0x45, 0x6c, 0x73, 0x61, 0x6c,
0x76, 0x61, 0x64, 0x6f, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x4b, 0x61, 0x73, 0x6b, 0x61,
0x7a, 0x69, 0x6e, 0x69, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x4b, 0x61, 0x73, 0x6b, 0x61,
0x7a, 0x69, 0x6e, 0x69, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65,
0x6d, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x4b, 0x61,
0x73, 0x6b, 0x61, 0x7a, 0x69, 0x6e, 0x69, 0x42, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x69, 0x20, 0x79, 0x61, 0x20, 0x55, 0x68, 0x69, 0x73, 0x70, 0x61,
0x6e, 0x69, 0x61, 0x50, 0x77, 0x65, 0x74, 0x6f, 0x72, 0x69, 0x6b, 0x6f,
0x4a, 0x69, 0x6a, 0x69, 0x20, 0x6c, 0x61, 0x20, 0x48, 0x6f, 0x20, 0x43,
0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x53, 0x69, 0x6e, 0x67, 0x61,
0x70, 0x6f, 0x6f, 0x4b, 0x72, 0x69, 0x73, 0x6d, 0x61, 0x73, 0x69, 0x43,
0x61, 0x79, 0x6d, 0x61, 0x6e, 0xf6, 0x61, 0x72, 0x6e, 0x61, 0x54, 0x6a,
0x69, 0x74, 0x61, 0x48, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69,
0x6e, 0x68, 0x2d, 0x73, 0x74, 0x61, 0x64, 0x65, 0x6e, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x72, 0x6e, 0x61, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x65,
0xf6, 0x61, 0x72, 0x6e, 0x61, 0x54, 0x6f, 0x72, 0x73, 0x68, 0x61, 0x6d,
0x6e, 0x53, 0x79, 0x64, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x6e,
0x53, 0x61, 0x6e, 0x6b, 0x74, 0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61,
0x4b, 0xf6, 0x70, 0x65, 0x6e, 0x68, 0x61, 0x6d, 0x6e, 0x56, 0x61, 0x74,
0x69, 0x6b, 0x61, 0x6e, 0x65, 0x6e, 0x4a, 0x75, 0x6c, 0xf6, 0x6e, 0x4b,
0x6f, 0x6b, 0x6f, 0x73, 0xf6, 0x61, 0x72, 0x6e, 0x61, 0x4b, 0x6f, 0x6d,
0x6f, 0x72, 0x65, 0x72, 0x6e, 0x61, 0x4b, 0x65, 0x72, 0x67, 0x75, 0x65,
0x6c, 0x65, 0x6e, 0xf6, 0x61, 0x72, 0x6e, 0x61, 0x4d, 0x61, 0x6c, 0x64,
0x69, 0x76, 0x65, 0x72, 0x6e, 0x61, 0x50, 0xe5, 0x73, 0x6b, 0xf6, 0x6e,
0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0xf6, 0x61, 0x72, 0x6e, 0x61,
0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0xf6, 0x61, 0x72,
0x6e, 0x61, 0x4d, 0x69, 0x64, 0x77, 0x61, 0x79, 0xf6, 0x61, 0x72, 0x6e,
0x61, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0xf6, 0x61, 0x72,
0x6e, 0x61, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0xf6, 0x6e, 0x4b, 0x61,
0x69, 0x6d, 0x61, 0x6e, 0x69, 0x6e, 0x73, 0x6c, 0x65, 0x4d, 0x65, 0x78,
0x69, 0x6b, 0x6f, 0x2d, 0x53, 0x63, 0x68, 0x74, 0x61, 0x64, 0x74, 0x57,
0x6f, 0x73, 0x63, 0x68, 0x74, 0x6f, 0x6b, 0x4d, 0x75, 0x73, 0x63, 0x68,
0x6b, 0x61, 0x74, 0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x61, 0x74,
0x6f, 0x72, 0x53, 0xfc, 0xfc, 0x64, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x65, 0x42, 0x75, 0x6b, 0x61, 0x72, 0x65, 0x73, 0x63, 0x68, 0x74,
0x4b, 0x6f, 0x70, 0x65, 0x68, 0x61, 0x67, 0x65, 0x57, 0x69, 0x65, 0x6e,
0x61, 0x63, 0x68, 0x74, 0x73, 0x2d, 0x49, 0x6e, 0x73, 0x6c, 0x65, 0x4f,
0x73, 0x63, 0x68, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x73, 0x6c, 0x65, 0x710,
0x712, 0x715, 0x713, 0x722, 0x710, 0x71f, 0x72a, 0x710, 0x710, 0x715, 0x71d, 0x723,
0x20, 0x710, 0x712, 0x712, 0x710, 0x713, 0x719, 0x710, 0x710, 0x72a, 0x710, 0x723,
0x721, 0x710, 0x72a, 0x710, 0x712, 0x710, 0x721, 0x710, 0x71f, 0x718, 0x712, 0x710,
0x722, 0x713, 0x710, 0x718, 0x71d, 0x712, 0x710, 0x722, 0x713, 0x718, 0x720, 0x712,
0x71d, 0x723, 0x710, 0x718, 0x712, 0x720, 0x722, 0x72c, 0x71d, 0x72a, 0x712, 0x72a,
0x710, 0x719, 0x710, 0x712, 0x71d, 0x720, 0x712, 0x718, 0x713, 0x718, 0x721, 0x712,
0x718, 0x72a, 0x710, 0x729, 0x710, 0x717, 0x716, 0x308, 0x717, 0x71f, 0x710, 0x723,
0x710, 0x712, 0x720, 0x722, 0x71f, 0x710, 0x723, 0x712, 0x72c, 0x710, 0x71f, 0x718,
0x722, 0x710, 0x71f, 0x72a, 0x71d, 0x715, 0x710, 0x71f, 0x710, 0x72a, 0x715, 0x710,
0x72a, 0x20, 0x72b, 0x720, 0x721, 0x710, 0x713, 0x71d, 0x712, 0x718, 0x72c, 0x71d,
0x715, 0x718, 0x710, 0x720, 0x710, 0x710, 0x720, 0x20, 0x725, 0x71d, 0x718, 0x722,
0x726, 0x72a, 0x71d, 0x72c, 0x710, 0x718, 0x722, 0x713, 0x712, 0x718, 0x72a, 0x718,
0x722, 0x717, 0x72a, 0x710, 0x72a, 0x71d, 0x71d, 0x718, 0x717, 0x722, 0x71d, 0x723,
0x712, 0x718, 0x72a, 0x713, 0x713, 0x718, 0x712, 0x710, 0x71f, 0x710, 0x721, 0x726,
0x710, 0x720, 0x710, 0x71a, 0x72a, 0x71b, 0x718, 0x721, 0x71f, 0x71d, 0x713, 0x710,
0x720, 0x71d, 0x71f, 0x71d, 0x722, 0x72b, 0x710, 0x723, 0x710, 0x720, 0x710, 0x713,
0x718, 0x723, 0x720, 0x71d, 0x712, 0x72a, 0x71d, 0x712, 0x71d, 0x720, 0x720, 0x718,
0x721, 0x71d, 0x720, 0x718, 0x710, 0x722, 0x715, 0x710, 0x720, 0x718, 0x712, 0x718,
0x721, 0x712, 0x72b, 0x71d, 0x720, 0x718, 0x723, 0x710, 0x71f, 0x710, 0x721, 0x710,
0x720, 0x710, 0x712, 0x718, 0x721, 0x710, 0x726, 0x718, 0x72c, 0x718, 0x721, 0x710,
0x723, 0x71d, 0x72a, 0x718, 0x721, 0x712, 0x710, 0x712, 0x710, 0x722, 0x71d, 0x721,
0x718, 0x729, 0x715, 0x71d, 0x72b, 0x718, 0x721, 0x718, 0x722, 0x72a, 0x718, 0x712,
0x71d, 0x710, 0x722, 0x71d, 0x72a, 0x718, 0x712, 0x71d, 0x722, 0x713, 0x721, 0x71d,
0x722, 0x710, 0x722, 0x71d, 0x710, 0x721, 0x71d, 0x722, 0x718, 0x710, 0x71f, 0x72b,
0x718, 0x72c, 0x710, 0x718, 0x710, 0x713, 0x710, 0x715, 0x710, 0x718, 0x713, 0x710,
0x718, 0x726, 0x718, 0x72a, 0x72c, 0x718, 0x2d, 0x722, 0x718, 0x712, 0x718, 0x723,
0x710, 0x718, 0x20, 0x72c, 0x718, 0x721, 0x71d, 0x71b, 0x72a, 0x71d, 0x726, 0x718,
0x720, 0x71d, 0x723, 0x72c, 0x718, 0x722, 0x723, 0x718, 0x71d, 0x722, 0x715, 0x717,
0x718, 0x71f, 0x710, 0x715, 0x710, 0x71f, 0x710, 0x722, 0x71f, 0x718, 0x72a, 0x713,
0x710, 0x722, 0x713, 0x718, 0x71d, 0x710, 0x710, 0x722, 0x72c, 0x71d, 0x713, 0x718,
0x710, 0x710, 0x72a, 0x710, 0x713, 0x710, 0x718, 0x71d, 0x722, 0x710, 0x712, 0x718,
0x710, 0x71d, 0x722, 0x718, 0x723, 0x20, 0x725, 0x71d, 0x72a, 0x723, 0x71f, 0x710,
0x72c, 0x710, 0x721, 0x72a, 0x71f, 0x710, 0x71f, 0x718, 0x72a, 0x715, 0x718, 0x712,
0x710, 0x713, 0x718, 0x713, 0x718, 0x71d, 0x720, 0x710, 0x20, 0x72a, 0x71d, 0x718,
0x717, 0x710, 0x721, 0x722, 0x715, 0x718, 0x719, 0x710, 0x72a, 0x71d, 0x718, 0x20,
0x713, 0x71d, 0x713, 0x718, 0x723, 0x723, 0x720, 0x72c, 0x710, 0x723, 0x710, 0x722,
0x20, 0x718, 0x710, 0x722, 0x723, 0x710, 0x722, 0x20, 0x720, 0x718, 0x71d, 0x723,
0x72c, 0x718, 0x71f, 0x718, 0x721, 0x710, 0x722, 0x710, 0x718, 0x72b, 0x718, 0x710,
0x71d, 0x710, 0x710, 0x72a, 0x718, 0x712, 0x710, 0x710, 0x723, 0x718, 0x722, 0x71f,
0x723, 0x71d, 0x718, 0x722, 0x710, 0x72c, 0x71d, 0x71f, 0x718, 0x71f, 0x710, 0x722,
0x712, 0x710, 0x717, 0x71d, 0x710, 0x712, 0x710, 0x717, 0x71d, 0x710, 0x20, 0x712,
0x710, 0x722, 0x71d, 0x72a, 0x723, 0x712, 0x710, 0x72a, 0x712, 0x715, 0x718, 0x723,
0x712, 0x71d, 0x720, 0x71d, 0x721, 0x712, 0x71d, 0x720, 0x71d, 0x719, 0x712, 0x720,
0x710, 0x722, 0x71f, 0x2d, 0x723, 0x710, 0x712, 0x720, 0x718, 0x722, 0x712, 0x718,
0x725, 0x20, 0x712, 0x71d, 0x723, 0x72c, 0x710, 0x712, 0x718, 0x713, 0x718, 0x72c,
0x710, 0x712, 0x718, 0x71d, 0x719, 0x71d, 0x71f, 0x721, 0x712, 0x72a, 0x713, 0x20,
0x712, 0x710, 0x71d, 0x71f, 0x710, 0x721, 0x726, 0x718, 0x20, 0x713, 0x72a, 0x722,
0x715, 0x71d, 0x71f, 0x710, 0x722, 0x71f, 0x718, 0x722, 0x71f, 0x710, 0x72a, 0x710,
0x71f, 0x723, 0x71f, 0x710, 0x71d, 0x710, 0x722, 0x71f, 0x710, 0x71d, 0x721, 0x71d,
0x722, 0x72b, 0x71f, 0x713, 0x718, 0x71f, 0x71d, 0x718, 0x710, 0x718, 0x710, 0x723,
0x71d, 0x718, 0x715, 0x710, 0x715, 0x20, 0x710, 0x718, 0x72a, 0x71d, 0x719, 0x71f,
0x718, 0x723, 0x72c, 0x710, 0x20, 0x72a, 0x71d, 0x71f, 0x710, 0x71f, 0x72a, 0x71d,
0x723, 0x72c, 0x718, 0x722, 0x71f, 0x718, 0x71d, 0x710, 0x712, 0x710, 0x71f, 0x718,
0x72a, 0x710, 0x71f, 0x710, 0x718, 0x715, 0x710, 0x722, 0x721, 0x72a, 0x71f, 0x72b,
0x712, 0x71d, 0x722, 0x715, 0x710, 0x723, 0x718, 0x722, 0x715, 0x710, 0x723, 0x718,
0x722, 0x20, 0x71f, 0x72a, 0x71d, 0x71f, 0x715, 0x722, 0x712, 0x72a, 0x715, 0x71d,
0x72c, 0x72a, 0x718, 0x71d, 0x72c, 0x715, 0x718, 0x721, 0x71d, 0x722, 0x71d, 0x71f,
0x710, 0x710, 0x715, 0x721, 0x718, 0x722, 0x72c, 0x718, 0x722, 0x710, 0x71d, 0x72a,
0x718, 0x722, 0x71d, 0x726, 0x71d, 0x710, 0x720, 0x20, 0x723, 0x720, 0x712, 0x710,
0x715, 0x718, 0x72a, 0x726, 0x718, 0x72a, 0x72c, 0x20, 0x722, 0x71d, 0x720, 0x723,
0x718, 0x722, 0x726, 0x718, 0x72a, 0x72c, 0x710, 0x720, 0x71d, 0x719, 0x710, 0x713,
0x720, 0x71d, 0x723, 0x20, 0x712, 0x710, 0x71d, 0x713, 0x718, 0x723, 0x20, 0x712,
0x710, 0x71d, 0x713, 0x72a, 0x710, 0x722, 0x715, 0x20, 0x72c, 0x718, 0x72a, 0x71f,
0x713, 0x72a, 0x71d, 0x722, 0x715, 0x710, 0x713, 0x718, 0x710, 0x715, 0x710, 0x720,
0x718, 0x726, 0x71d, 0x713, 0x718, 0x710, 0x72c, 0x721, 0x710, 0x720, 0x710, 0x713,
0x718, 0x710, 0x71d, 0x710, 0x729, 0x718, 0x71d, 0x720, 0x713, 0x718, 0x71d, 0x710,
0x722, 0x710, 0x717, 0x720, 0x71d, 0x726, 0x710, 0x71f, 0x723, 0x717, 0x710, 0x712,
0x710, 0x722, 0x710, 0x717, 0x71d, 0x72a, 0x721, 0x718, 0x723, 0x71d, 0x710, 0x710,
0x722, 0x715, 0x71d, 0x710, 0x722, 0x710, 0x726, 0x718, 0x720, 0x71d, 0x723, 0x722,
0x718, 0x71f, 0x723, 0x60c, 0x20, 0x710, 0x722, 0x715, 0x71d, 0x710, 0x722, 0x710,
0x721, 0x72a, 0x71d, 0x722, 0x713, 0x718, 0x60c, 0x20, 0x710, 0x722, 0x715, 0x71d,
0x710, 0x722, 0x710, 0x726, 0x71d, 0x72c, 0x71d, 0x72a, 0x723, 0x712, 0x71d, 0x72a,
0x713, 0x60c, 0x20, 0x710, 0x722, 0x715, 0x71d, 0x710, 0x722, 0x710, 0x721, 0x722,
0x715, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x72c, 0x71d, 0x720, 0x60c, 0x20, 0x710,
0x722, 0x715, 0x71d, 0x710, 0x722, 0x710, 0x712, 0x71d, 0x712, 0x710, 0x71d, 0x60c,
0x20, 0x710, 0x722, 0x715, 0x71d, 0x710, 0x722, 0x710, 0x712, 0x71d, 0x722, 0x723,
0x71d, 0x722, 0x71d, 0x723, 0x60c, 0x20, 0x710, 0x722, 0x715, 0x71d, 0x710, 0x722,
0x710, 0x718, 0x71d, 0x722, 0x710, 0x721, 0x710, 0x71f, 0x60c, 0x20, 0x710, 0x722,
0x715, 0x71d, 0x710, 0x722, 0x710, 0x710, 0x722, 0x718, 0x712, 0x71d, 0x71f, 0x710,
0x71d, 0x729, 0x720, 0x718, 0x71d, 0x72c, 0x713, 0x721, 0x710, 0x71d, 0x71f, 0x710,
0x713, 0x718, 0x722, 0x718, 0x720, 0x718, 0x71d, 0x723, 0x712, 0x71d, 0x720, 0x721,
0x718, 0x722, 0x72c, 0x710, 0x71f, 0x71d, 0x720, 0x718, 0x60c, 0x20, 0x71f, 0x71d,
0x722, 0x72c, 0x710, 0x71f, 0x71d, 0x71f, 0x72a, 0x720, 0x71d, 0x722, 0x713, 0x729,
0x720, 0x710, 0x20, 0x726, 0x710, 0x719, 0x720, 0x71d, 0x721, 0x710, 0x720, 0x718,
0x723, 0x20, 0x710, 0x722, 0x713, 0x720, 0x718, 0x723, 0x72a, 0x718, 0x712, 0x742,
0x725, 0x710, 0x20, 0x715, 0x72b, 0x720, 0x71d, 0x71b, 0x710, 0x20, 0x72c, 0x71a,
0x72c, 0x71d, 0x710, 0x721, 0x710, 0x723, 0x71d, 0x710, 0x718, 0x721, 0x710, 0x722,
0x710, 0x713, 0x718, 0x710, 0x721, 0x710, 0x722, 0x710, 0x718, 0x723, 0x721, 0x72a,
0x71d, 0x713, 0x718, 0x72c, 0x721, 0x710, 0x72a, 0x72c, 0x71d, 0x722, 0x71d, 0x729,
0x721, 0x710, 0x72c, 0x710, 0x721, 0x718, 0x72a, 0x718, 0x723, 0x721, 0x719, 0x71b,
0x720, 0x710, 0x722, 0x721, 0x722, 0x718, 0x721, 0x71d, 0x722, 0x71d, 0x721, 0x72a,
0x71d, 0x715, 0x710, 0x721, 0x71b, 0x720, 0x71f, 0x710, 0x72c, 0x720, 0x710, 0x721,
0x715, 0x71d, 0x722, 0x72c, 0x710, 0x20, 0x715, 0x721, 0x71f, 0x723, 0x71d, 0x71f,
0x718, 0x721, 0x729, 0x718, 0x720, 0x718, 0x722, 0x721, 0x718, 0x722, 0x71f, 0x72c,
0x718, 0x722, 0x721, 0x718, 0x722, 0x71b, 0x72a, 0x710, 0x71d, 0x721, 0x718, 0x722,
0x72c, 0x71d, 0x712, 0x71d, 0x715, 0x71d, 0x718, 0x721, 0x718, 0x722, 0x72c, 0x723,
0x71d, 0x72a, 0x710, 0x72c, 0x722, 0x710, 0x723, 0x710, 0x718, 0x722, 0x71d, 0x718,
0x20, 0x71d, 0x718, 0x72a, 0x71f, 0x722, 0x718, 0x72a, 0x718, 0x722, 0x717, 0x710,
0x712, 0x71d, 0x718, 0x720, 0x710, 0x60c, 0x20, 0x715, 0x710, 0x71f, 0x718, 0x72c,
0x710, 0x20, 0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x723, 0x71d, 0x722,
0x72c, 0x72a, 0x60c, 0x20, 0x715, 0x710, 0x71f, 0x718, 0x72c, 0x710, 0x20, 0x713,
0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x722, 0x71d, 0x718, 0x20, 0x723, 0x710,
0x720, 0x71d, 0x721, 0x60c, 0x200c, 0x20, 0x715, 0x710, 0x71f, 0x718, 0x72c, 0x710,
0x20, 0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x718, 0x713, 0x71d, 0x722,
0x710, 0x713, 0x710, 0x726, 0x722, 0x721, 0x710, 0x726, 0x710, 0x72a, 0x710, 0x721,
0x710, 0x72a, 0x71d, 0x712, 0x718, 0x726, 0x71d, 0x722, 0x71d, 0x71f, 0x723, 0x726,
0x718, 0x72a, 0x72c, 0x20, 0x710, 0x718, 0x20, 0x726, 0x72a, 0x71d, 0x722, 0x723,
0x726, 0x718, 0x72a, 0x72c, 0x20, 0x715, 0x710, 0x723, 0x726, 0x722, 0x71d, 0x710,
0x726, 0x718, 0x72a, 0x72c, 0x718, 0x20, 0x712, 0x71d, 0x720, 0x717, 0x718, 0x726,
0x718, 0x72a, 0x72c, 0x718, 0x20, 0x72a, 0x71d, 0x71f, 0x718, 0x726, 0x718, 0x722,
0x72c, 0x710, 0x20, 0x725, 0x72a, 0x71d, 0x722, 0x723, 0x72a, 0x710, 0x722, 0x71f,
0x722, 0x20, 0x710, 0x722, 0x720, 0x71d, 0x72c, 0x72a, 0x71d, 0x723, 0x71d, 0x726,
0x71d, 0x72a, 0x71d, 0x713, 0x71d, 0x722, 0x710, 0x72a, 0x71d, 0x723, 0x718, 0x720,
0x718, 0x72c, 0x72a, 0x71d, 0x718, 0x20, 0x712, 0x72a, 0x722, 0x71f, 0x718, 0x723,
0x710, 0x722, 0x72c, 0x710, 0x72a, 0x721, 0x723, 0x710, 0x722, 0x72c, 0x71d, 0x710,
0x713, 0x718, 0x723, 0x722, 0x72c, 0x718, 0x20, 0x715, 0x718, 0x721, 0x71d, 0x722,
0x713, 0x718, 0x723, 0x710, 0x718, 0x20, 0x726, 0x710, 0x718, 0x720, 0x718, 0x710,
0x71d, 0x71b, 0x718, 0x729, 0x718, 0x72a, 0x721, 0x71d, 0x72c, 0x723, 0x71d, 0x71b,
0x71f, 0x710, 0x721, 0x72a, 0x20, 0x712, 0x72a, 0x20, 0x72c, 0x718, 0x720, 0x721,
0x71d, 0x721, 0x72a, 0x20, 0x71d, 0x718, 0x71a, 0x722, 0x722, 0x723, 0x710, 0x722,
0x72c, 0x20, 0x71f, 0x72c, 0x723, 0x721, 0x72a, 0x72c, 0x71d, 0x20, 0x720, 0x718,
0x72b, 0x710, 0x721, 0x72a, 0x20, 0x72c, 0x710, 0x718, 0x721, 0x710, 0x721, 0x72a,
0x20, 0x712, 0x722, 0x723, 0x722, 0x72c, 0x722, 0x715, 0x718, 0x72a, 0x72c, 0x710,
0x72c, 0x71d, 0x713, 0x718, 0x723, 0x71d, 0x713, 0x710, 0x720, 0x726, 0x710, 0x72c,
0x71d, 0x710, 0x718, 0x710, 0x722, 0x710, 0x72c, 0x718, 0x72a, 0x718, 0x722, 0x72c,
0x718, 0x72c, 0x718, 0x72a, 0x72c, 0x718, 0x720, 0x710, 0x712, 0x722, 0x71f, 0x718,
0x712, 0x71d, 0x72a, 0x723, 0x718, 0x723, 0x72c, 0x710, 0x20, 0x71a, 0x718, 0x72a,
0x72c, 0x710, 0x718, 0x71d, 0x722, 0x71d, 0x726, 0x713, 0x71d, 0x729, 0x718, 0x72c,
0x710, 0x72c, 0x71f, 0x710, 0x71d, 0x723, 0x71d, 0x715, 0x712, 0x71d, 0x723, 0x715,
0x718, 0x721, 0x718, 0x722, 0x72c, 0x20, 0x715, 0x710, 0x718, 0x72a, 0x712, 0x71d,
0x720, 0x721, 0x710, 0x71f, 0x710, 0x718, 0x72a, 0x71d, 0x721, 0x710, 0x718, 0x723,
0x718, 0x722, 0x721, 0x71f, 0x721, 0x718, 0x72a, 0x715, 0x718, 0x726, 0x710, 0x720,
0x721, 0x71d, 0x72a, 0x72a, 0x718, 0x72c, 0x71d, 0x72a, 0x710, 0x723, 0x71d, 0x718,
0x710, 0x72c, 0x72a, 0x718, 0x720, 0x712, 0x718, 0x723, 0x72c, 0x718, 0x71f, 0x720,
0x718, 0x722, 0x713, 0x71d, 0x725, 0x72a, 0x712, 0x71d, 0x71d, 0x722, 0x725, 0x715,
0x722, 0x710, 0x720, 0x721, 0x710, 0x72c, 0x71d, 0x725, 0x721, 0x710, 0x722, 0x710,
0x722, 0x710, 0x715, 0x71d, 0x72a, 0x710, 0x71f, 0x72c, 0x710, 0x718, 0x710, 0x71f,
0x72c, 0x718, 0x712, 0x725, 0x72b, 0x729, 0x710, 0x712, 0x710, 0x715, 0x710, 0x72c,
0x71d, 0x72a, 0x718, 0x712, 0x713, 0x715, 0x715, 0x712, 0x71a, 0x72a, 0x71d, 0x722,
0x712, 0x710, 0x722, 0x71f, 0x718, 0x71f, 0x712, 0x710, 0x72a, 0x722, 0x710, 0x718,
0x720, 0x712, 0x71d, 0x72a, 0x718, 0x72c, 0x712, 0x71d, 0x72b, 0x71f, 0x71d, 0x71f,
0x712, 0x72a, 0x718, 0x722, 0x710, 0x71d, 0x72c, 0x72b, 0x71d, 0x72c, 0x710, 0x71f,
0x718, 0x720, 0x718, 0x721, 0x712, 0x718, 0x715, 0x72a, 0x721, 0x723, 0x718, 0x729,
0x715, 0x71f, 0x710, 0x715, 0x71d, 0x720, 0x71d, 0x715, 0x718, 0x712, 0x710, 0x71d,
0x715, 0x718, 0x72b, 0x710, 0x722, 0x712, 0x71d, 0x726, 0x710, 0x721, 0x710, 0x713,
0x718, 0x723, 0x72c, 0x710, 0x725, 0x719, 0x710, 0x71a, 0x712, 0x72a, 0x718, 0x722,
0x721, 0x715, 0x71d, 0x722, 0x72c, 0x710, 0x20, 0x715, 0x717, 0x718, 0x20, 0x71f,
0x71d, 0x20, 0x721, 0x71d, 0x722, 0x717, 0x718, 0x722, 0x713, 0x20, 0x71f, 0x718,
0x722, 0x713, 0x717, 0x718, 0x712, 0x715, 0x710, 0x71d, 0x72a, 0x71f, 0x718, 0x72c,
0x723, 0x71f, 0x713, 0x710, 0x71f, 0x710, 0x72a, 0x72c, 0x710, 0x713, 0x710, 0x71d,
0x726, 0x718, 0x72a, 0x710, 0x710, 0x718, 0x72a, 0x72b, 0x720, 0x721, 0x71f, 0x710,
0x712, 0x718, 0x720, 0x71f, 0x710, 0x721, 0x72c, 0x72b, 0x710, 0x72c, 0x71f, 0x710,
0x71f, 0x72a, 0x710, 0x71f, 0x71d, 0x71f, 0x710, 0x72c, 0x721, 0x710, 0x722, 0x715,
0x718, 0x71a, 0x710, 0x722, 0x715, 0x71d, 0x713, 0x710, 0x71f, 0x718, 0x720, 0x71f,
0x72c, 0x710, 0x71f, 0x72a, 0x710, 0x723, 0x722, 0x718, 0x71d, 0x72a, 0x723, 0x71f,
0x71f, 0x718, 0x710, 0x720, 0x710, 0x20, 0x720, 0x718, 0x721, 0x726, 0x718, 0x72a,
0x71f, 0x718, 0x72b, 0x71d, 0x722, 0x713, 0x71f, 0x718, 0x71d, 0x72c, 0x721, 0x710,
0x713, 0x710, 0x715, 0x710, 0x722, 0x721, 0x710, 0x71f, 0x710, 0x723, 0x710, 0x72a,
0x721, 0x710, 0x722, 0x71d, 0x720, 0x710, 0x721, 0x723, 0x729, 0x71b, 0x722, 0x71d,
0x729, 0x718, 0x723, 0x71d, 0x710, 0x722, 0x718, 0x712, 0x718, 0x71f, 0x718, 0x719,
0x722, 0x71d, 0x72c, 0x723, 0x71f, 0x722, 0x718, 0x712, 0x718, 0x723, 0x71d, 0x712,
0x72a, 0x723, 0x71f, 0x710, 0x718, 0x721, 0x723, 0x71f, 0x710, 0x718, 0x72a, 0x710,
0x720, 0x726, 0x722, 0x718, 0x721, 0x20, 0x726, 0x722, 0x726, 0x718, 0x722, 0x72c,
0x71d, 0x710, 0x722, 0x710, 0x71f, 0x726, 0x71d, 0x718, 0x722, 0x713, 0x71d, 0x722,
0x713, 0x729, 0x71b, 0x72a, 0x71f, 0x718, 0x723, 0x72c, 0x710, 0x722, 0x710, 0x71d,
0x71f, 0x71d, 0x719, 0x71d, 0x720, 0x718, 0x72a, 0x715, 0x710, 0x72a, 0x71d, 0x710,
0x715, 0x723, 0x710, 0x71a, 0x710, 0x720, 0x71d, 0x722, 0x723, 0x721, 0x72a, 0x71f,
0x722, 0x715, 0x723, 0x710, 0x718, 0x720, 0x72b, 0x722, 0x713, 0x717, 0x710, 0x71d,
0x71d, 0x723, 0x71d, 0x722, 0x713, 0x710, 0x726, 0x718, 0x72a, 0x723, 0x72a, 0x71d,
0x715, 0x722, 0x71d, 0x71f, 0x718, 0x720, 0x71d, 0x721, 0x723, 0x71f, 0x72c, 0x710,
0x71d, 0x726, 0x710, 0x71d, 0x72c, 0x72b, 0x71f, 0x71d, 0x722, 0x72c, 0x72c, 0x726,
0x720, 0x71d, 0x723, 0x72c, 0x717, 0x72a, 0x722, 0x72c, 0x71d, 0x721, 0x726, 0x718,
0x71b, 0x718, 0x71f, 0x71d, 0x718, 0x72c, 0x718, 0x721, 0x723, 0x71f, 0x710, 0x718,
0x720, 0x710, 0x722, 0x712, 0x710, 0x72c, 0x718, 0x72a, 0x710, 0x718, 0x72a, 0x718,
0x721, 0x71f, 0x71d, 0x710, 0x718, 0x723, 0x72c, 0x2d, 0x722, 0x71d, 0x72a, 0x710,
0x712, 0x71d, 0x710, 0x722, 0x72c, 0x71d, 0x710, 0x722, 0x712, 0x720, 0x710, 0x715,
0x71d, 0x712, 0x718, 0x723, 0x72c, 0x718, 0x71f, 0x71d, 0x710, 0x71f, 0x718, 0x72c,
0x723, 0x71f, 0x71d, 0x722, 0x713, 0x718, 0x722, 0x71d, 0x71f, 0x710, 0x72c, 0x71d,
0x72a, 0x722, 0x712, 0x71d, 0x72a, 0x713, 0x71d, 0x72a, 0x712, 0x710, 0x722, 0x713,
0x719, 0x72a, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x719, 0x718, 0x72a, 0x712, 0x71d,
0x72a, 0x721, 0x71d, 0x718, 0x715, 0x710, 0x71f, 0x710, 0x722, 0x72a, 0x71d, 0x71f,
0x710, 0x726, 0x20, 0x712, 0x71d, 0x72a, 0x715, 0x71d, 0x726, 0x710, 0x72a, 0x718,
0x721, 0x715, 0x710, 0x71d, 0x72a, 0x710, 0x72a, 0x710, 0x71d, 0x71f, 0x712, 0x71d,
0x71f, 0x72c, 0x721, 0x71d, 0x722, 0x20, 0x713, 0x718, 0x72a, 0x713, 0x71d, 0x710,
0x721, 0x72a, 0x72c, 0x71d, 0x20, 0x717, 0x71d, 0x720, 0x71d, 0x722, 0x710, 0x723,
0x72c, 0x710, 0x722, 0x720, 0x71d, 0x712, 0x72a, 0x71d, 0x723, 0x712, 0x710, 0x722,
0x712, 0x72a, 0x718, 0x71f, 0x71d, 0x722, 0x20, 0x717, 0x71d, 0x720, 0x715, 0x72a,
0x718, 0x71d, 0x722, 0x710, 0x718, 0x71f, 0x720, 0x710, 0x717, 0x718, 0x712, 0x72a,
0x72c, 0x720, 0x71d, 0x722, 0x715, 0x721, 0x710, 0x722, 0x720, 0x718, 0x72a, 0x715,
0x20, 0x717, 0x710, 0x718, 0x721, 0x71d, 0x720, 0x712, 0x718, 0x72a, 0x722, 0x726,
0x71d, 0x72a, 0x72c, 0x723, 0x71d, 0x715, 0x722, 0x71d, 0x710, 0x721, 0x723, 0x72c,
0x72a, 0x715, 0x721, 0x710, 0x722, 0x715, 0x718, 0x72a, 0x710, 0x710, 0x723, 0x72c,
0x72a, 0x710, 0x71a, 0x722, 0x710, 0x72c, 0x722, 0x718, 0x723, 0x712, 0x720, 0x713,
0x72a, 0x715, 0x712, 0x72a, 0x720, 0x71d, 0x722, 0x712, 0x72a, 0x72c, 0x71d, 0x723,
0x720, 0x712, 0x710, 0x200f, 0x712, 0x72a, 0x718, 0x71f, 0x723, 0x720, 0x712, 0x718,
0x729, 0x718, 0x72a, 0x72b, 0x71b, 0x712, 0x718, 0x715, 0x726, 0x72b, 0x71b, 0x712,
0x718, 0x723, 0x71d, 0x722, 0x713, 0x722, 0x71f, 0x71d, 0x723, 0x71d, 0x722, 0x710,
0x718, 0x71f, 0x718, 0x726, 0x722, 0x717, 0x710, 0x713, 0x722, 0x715, 0x712, 0x720,
0x722, 0x713, 0x712, 0x72a, 0x720, 0x71b, 0x72a, 0x713, 0x718, 0x72a, 0x722, 0x719,
0x71d, 0x717, 0x720, 0x723, 0x722, 0x71f, 0x71d, 0x713, 0x719, 0x72a, 0x72c, 0x710,
0x20, 0x715, 0x721, 0x710, 0x722, 0x710, 0x723, 0x71b, 0x722, 0x712, 0x718, 0x720,
0x713, 0x71d, 0x72a, 0x719, 0x71d, 0x71f, 0x710, 0x720, 0x71d, 0x722, 0x71d, 0x722,
0x713, 0x72a, 0x710, 0x715, 0x71f, 0x71d, 0x72a, 0x718, 0x712, 0x71f, 0x71d, 0x71d,
0x712, 0x720, 0x72b, 0x712, 0x718, 0x722, 0x710, 0x720, 0x71d, 0x718, 0x712, 0x720,
0x71d, 0x710, 0x722, 0x710, 0x720, 0x718, 0x722, 0x715, 0x718, 0x722, 0x720, 0x718,
0x71f, 0x723, 0x721, 0x712, 0x718, 0x72a, 0x713, 0x721, 0x715, 0x72a, 0x71d, 0x715,
0x721, 0x71d, 0x720, 0x71b, 0x710, 0x721, 0x710, 0x72a, 0x71d, 0x717, 0x710, 0x721,
0x721, 0x71d, 0x722, 0x723, 0x71f, 0x721, 0x718, 0x722, 0x710, 0x71f, 0x718, 0x721,
0x718, 0x723, 0x71f, 0x718, 0x710, 0x718, 0x723, 0x720, 0x718, 0x726, 0x710, 0x72a,
0x71d, 0x723, 0x726, 0x718, 0x715, 0x713, 0x718, 0x72a, 0x71d, 0x71f, 0x710, 0x726,
0x72a, 0x710, 0x713, 0x72a, 0x71d, 0x713, 0x710, 0x72a, 0x718, 0x721, 0x71d, 0x723,
0x722, 0x20, 0x721, 0x72a, 0x71d, 0x722, 0x718, 0x723, 0x72a, 0x710, 0x71d, 0x71d,
0x712, 0x718, 0x723, 0x72a, 0x710, 0x72c, 0x718, 0x712, 0x723, 0x721, 0x726, 0x72a,
0x718, 0x726, 0x720, 0x723, 0x729, 0x718, 0x726, 0x71d, 0x710, 0x723, 0x718, 0x726,
0x71d, 0x710, 0x723, 0x72c, 0x718, 0x71f, 0x717, 0x718, 0x720, 0x721, 0x72c, 0x710,
0x720, 0x71d, 0x722, 0x72c, 0x71d, 0x72a, 0x710, 0x722, 0x710, 0x718, 0x720, 0x71d,
0x722, 0x718, 0x712, 0x723, 0x71f, 0x712, 0x715, 0x718, 0x719, 0x718, 0x710, 0x72c,
0x71d, 0x729, 0x710, 0x722, 0x712, 0x71d, 0x71d, 0x722, 0x710, 0x712, 0x720, 0x722,
0x718, 0x723, 0x712, 0x718, 0x720, 0x713, 0x718, 0x713, 0x72a, 0x710, 0x715, 0x718,
0x710, 0x72a, 0x723, 0x718, 0x719, 0x710, 0x713, 0x72a, 0x712, 0x719, 0x71d, 0x718,
0x72a, 0x71a, 0x710, 0x722, 0x72c, 0x710, 0x722, 0x710, 0x722, 0x72a, 0x71d, 0x712,
0x718, 0x72c, 0x72b, 0x713, 0x718, 0x723, 0x71f, 0x72a, 0x71d, 0x723, 0x72c, 0x721,
0x723, 0x71f, 0x718, 0x71f, 0x718, 0x723, 0x71f, 0x718, 0x721, 0x718, 0x72a, 0x718,
0x71f, 0x71d, 0x72a, 0x713, 0x718, 0x720, 0x71d, 0x722, 0x721, 0x710, 0x717, 0x71d,
0x713, 0x719, 0x72a, 0x72c, 0x710, 0x20, 0x721, 0x710, 0x720, 0x715, 0x71d, 0x712,
0x71d, 0x72c, 0x710, 0x721, 0x718, 0x72a, 0x71d, 0x72b, 0x718, 0x723, 0x721, 0x710,
0x71d, 0x718, 0x72c, 0x72a, 0x71d, 0x718, 0x722, 0x71d, 0x718, 0x722, 0x710, 0x726,
0x71d, 0x710, 0x710, 0x71f, 0x720, 0x710, 0x722, 0x715, 0x712, 0x718, 0x713, 0x710,
0x71d, 0x722, 0x712, 0x71d, 0x720, 0x72c, 0x72b, 0x710, 0x72c, 0x721, 0x72c, 0x72b,
0x718, 0x71f, 0x726, 0x728, 0x71a, 0x710, 0x71d, 0x726, 0x710, 0x72c, 0x726, 0x710,
0x71f, 0x710, 0x718, 0x726, 0x718, 0x726, 0x71d, 0x713, 0x71d, 0x726, 0x718, 0x722,
0x710, 0x726, 0x718, 0x72c, 0x71d, 0x713, 0x710, 0x720, 0x710, 0x726, 0x713, 0x718,
0x723, 0x713, 0x721, 0x712, 0x71d, 0x72a, 0x713, 0x718, 0x710, 0x715, 0x710, 0x720,
0x71f, 0x710, 0x722, 0x710, 0x720, 0x71f, 0x710, 0x722, 0x72c, 0x718, 0x722, 0x71f,
0x71d, 0x72a, 0x71d, 0x721, 0x710, 0x72c, 0x71d, 0x71f, 0x718, 0x723, 0x72a, 0x710,
0x71d, 0x71f, 0x718, 0x710, 0x713, 0x710, 0x720, 0x71d, 0x722, 0x721, 0x710, 0x713,
0x718, 0x72a, 0x718, 0x721, 0x710, 0x72a, 0x71f, 0x710, 0x718, 0x723, 0x710, 0x723,
0x721, 0x71d, 0x715, 0x718, 0x710, 0x71d, 0x722, 0x710, 0x718, 0x72a, 0x718, 0x722,
0x71d, 0x718, 0x71d, 0x722, 0x718, 0x72a, 0x726, 0x720, 0x71f, 0x722, 0x718, 0x721,
0x71d, 0x710, 0x726, 0x710, 0x713, 0x718, 0x20, 0x726, 0x710, 0x713, 0x718, 0x726,
0x720, 0x710, 0x718, 0x726, 0x71d, 0x72c, 0x71f, 0x710, 0x71d, 0x72a, 0x722, 0x726,
0x718, 0x717, 0x722, 0x726, 0x710, 0x71d, 0x726, 0x718, 0x72a, 0x72c, 0x20, 0x721,
0x718, 0x72a, 0x71d, 0x723, 0x712, 0x710, 0x71d, 0x72a, 0x710, 0x72a, 0x718, 0x72c,
0x718, 0x722, 0x713, 0x710, 0x723, 0x710, 0x71d, 0x726, 0x710, 0x722, 0x72c, 0x710,
0x717, 0x71d, 0x72c, 0x71d, 0x72c, 0x710, 0x72a, 0x710, 0x718, 0x710, 0x72c, 0x718,
0x722, 0x713, 0x710, 0x72c, 0x710, 0x726, 0x718, 0x718, 0x710, 0x71d, 0x71f, 0x718,
0x71d, 0x720, 0x71d, 0x723, 0x410, 0x431, 0x438, 0x434, 0x4b7, 0x43e, 0x43d, 0x410,
0x43b, 0x4b7, 0x430, 0x437, 0x43e, 0x438, 0x440, 0x411, 0x430, 0x43d, 0x4b7, 0x443,
0x43b, 0x411, 0x443, 0x4b7, 0x443, 0x43c, 0x431, 0x443, 0x440, 0x430, 0x49a, 0x43e,
0x4b3, 0x438, 0x440, 0x430, 0x414, 0x43e, 0x440, 0x443, 0x441, 0x441, 0x430, 0x43b,
0x43e, 0x43c, 0x4b6, 0x438, 0x431, 0x443, 0x442, 0x438, 0x42d, 0x43b, 0x20, 0x410,
0x438, 0x443, 0x43d, 0x4b6, 0x443, 0x431, 0x430, 0x41d, 0x4b7, 0x430, 0x43c, 0x435,
0x43d, 0x430, 0x410, 0x43d, 0x43a, 0x43e, 0x440, 0x438, 0x447, 0x410, 0x43d, 0x433,
0x438, 0x43b, 0x44f, 0x4b6, 0x443, 0x4b7, 0x443, 0x439, 0x411, 0x430, 0x445, 0x438,
0x44f, 0x411, 0x430, 0x445, 0x438, 0x44f, 0x20, 0x434, 0x435, 0x20, 0x411, 0x430,
0x43d, 0x434, 0x435, 0x440, 0x430, 0x441, 0x411, 0x43e, 0x439, 0x437, 0x41a, 0x435,
0x43c, 0x431, 0x440, 0x438, 0x4b7, 0x20, 0x411, 0x44d, 0x439, 0x421, 0x44e, 0x434,
0x430, 0x434, 0x20, 0x425, 0x443, 0x430, 0x440, 0x435, 0x441, 0x413, 0x43b, 0x44d,
0x439, 0x441, 0x20, 0x411, 0x44d, 0x439, 0x413, 0x443, 0x441, 0x20, 0x411, 0x44d,
0x439, 0x413, 0x440, 0x430, 0x43d, 0x434, 0x20, 0x422, 0x435, 0x440, 0x43a, 0x4b2,
0x430, 0x432, 0x430, 0x43d, 0x430, 0x41f, 0x435, 0x442, 0x435, 0x440, 0x431, 0x443,
0x440, 0x433, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d, 0x430, 0x412,
0x435, 0x432, 0x430, 0x439, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d,
0x430, 0x412, 0x438, 0x43d, 0x441, 0x435, 0x43d, 0x441, 0x2c, 0x20, 0x418, 0x43d,
0x434, 0x438, 0x430, 0x43d, 0x430, 0x4b6, 0x443, 0x43d, 0x43e, 0x41a, 0x440, 0x430,
0x43b, 0x435, 0x43d, 0x434, 0x439, 0x43a, 0x41b, 0x43e, 0x441, 0x2d, 0x410, 0x43d,
0x4b7, 0x435, 0x43b, 0x435, 0x441, 0x41a, 0x432, 0x430, 0x440, 0x442, 0x430, 0x43b,
0x438, 0x20, 0x41f, 0x43e, 0x451, 0x43d, 0x438, 0x20, 0x41f, 0x440, 0x438, 0x43d,
0x441, 0x41d, 0x44e, 0x2d, 0x419, 0x43e, 0x440, 0x43a, 0x411, 0x435, 0x439, 0x43b,
0x430, 0x2c, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x438, 0x20, 0x428,
0x438, 0x43c, 0x43e, 0x43b, 0x4e3, 0x421, 0x435, 0x43d, 0x442, 0x440, 0x2c, 0x20,
0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x438, 0x20, 0x428, 0x438, 0x43c, 0x43e,
0x43b, 0x4e3, 0x41d, 0x44e, 0x20, 0x421, 0x430, 0x43b, 0x435, 0x43c, 0x2c, 0x20,
0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x438, 0x20, 0x428, 0x438, 0x43c, 0x43e,
0x43b, 0x4e3, 0x41f, 0x43e, 0x440, 0x442, 0x438, 0x20, 0x418, 0x441, 0x43f, 0x430,
0x43d, 0x438, 0x44f, 0x41f, 0x43e, 0x440, 0x442, 0x443, 0x20, 0x412, 0x435, 0x43b,
0x445, 0x43e, 0x421, 0x435, 0x43d, 0x442, 0x20, 0x411, 0x430, 0x440, 0x442, 0x435,
0x43b, 0x435, 0x43c, 0x438, 0x421, 0x435, 0x43d, 0x442, 0x20, 0x4b6, 0x43e, 0x43d,
0x441, 0x421, 0x432, 0x438, 0x444, 0x442, 0x2d, 0x41a, 0x430, 0x440, 0x440, 0x435,
0x43d, 0x442, 0x414, 0x44e, 0x43c, 0x43e, 0x43d, 0x20, 0x434, 0x2019, 0x423, 0x440,
0x432, 0x438, 0x43b, 0x43b, 0x41c, 0x430, 0x43a, 0x43a, 0x443, 0x430, 0x440, 0x438,
0x41c, 0x430, 0x432, 0x441, 0x43e, 0x43d, 0x41c, 0x430, 0x43a, 0x43c, 0x435, 0x440,
0x434, 0x43e, 0x41b, 0x43e, 0x43d, 0x433, 0x439, 0x435, 0x440, 0x431, 0x451, 0x43d,
0x410, 0x434, 0x430, 0x43d, 0x410, 0x43b, 0x43c, 0x430, 0x430, 0x442, 0x43e, 0x410,
0x43c, 0x43c, 0x43e, 0x43d, 0x410, 0x448, 0x445, 0x43e, 0x431, 0x43e, 0x434, 0x411,
0x430, 0x433, 0x434, 0x43e, 0x434, 0x411, 0x430, 0x4b3, 0x440, 0x430, 0x439, 0x43d,
0x411, 0x43e, 0x43a, 0x443, 0x414, 0x438, 0x43c, 0x438, 0x448, 0x49b, 0x492, 0x430,
0x437, 0x437, 0x430, 0x4b6, 0x430, 0x43a, 0x430, 0x440, 0x442, 0x430, 0x4b6, 0x430,
0x44f, 0x43f, 0x443, 0x440, 0x430, 0x41a, 0x43e, 0x431, 0x443, 0x43b, 0x41a, 0x443,
0x432, 0x430, 0x439, 0x442, 0x41d, 0x43e, 0x432, 0x43e, 0x43a, 0x443, 0x437, 0x43d,
0x435, 0x442, 0x441, 0x43a, 0x49a, 0x430, 0x442, 0x430, 0x440, 0x49a, 0x438, 0x437,
0x438, 0x43b, 0x43e, 0x440, 0x434, 0x430, 0x420, 0x438, 0x451, 0x437, 0x422, 0x43e,
0x448, 0x43a, 0x435, 0x43d, 0x442, 0x422, 0x435, 0x4b3, 0x440, 0x43e, 0x43d, 0x422,
0x438, 0x43c, 0x444, 0x443, 0x412, 0x44c, 0x435, 0x43d, 0x442, 0x44f, 0x43d, 0x4b6,
0x430, 0x437, 0x438, 0x440, 0x430, 0x4b3, 0x43e, 0x438, 0x20, 0x410, 0x437, 0x43e,
0x440, 0x41a, 0x430, 0x43d, 0x430, 0x440, 0x438, 0x44f, 0x4b6, 0x43e, 0x440, 0x4b7,
0x438, 0x44f, 0x438, 0x20, 0x4b6, 0x430, 0x43d, 0x443, 0x431, 0x4e3, 0x42d, 0x443,
0x43a, 0x43b, 0x430, 0x4b6, 0x430, 0x437, 0x438, 0x440, 0x430, 0x438, 0x20, 0x41c,
0x430, 0x43d, 0x4b6, 0x435, 0x440, 0x441, 0x438, 0x41f, 0x43e, 0x434, 0x433, 0x43e,
0x440, 0x438, 0x442, 0x441, 0x430, 0x421, 0x442, 0x43e, 0x43a, 0x433, 0x43e, 0x43b,
0x43c, 0x421, 0x44e, 0x440, 0x438, 0x445, 0x41a, 0x440, 0x438, 0x441, 0x442, 0x43c,
0x430, 0x441, 0x41a, 0x435, 0x440, 0x433, 0x443, 0x43b, 0x435, 0x43d, 0x411, 0x443,
0x433, 0x435, 0x43d, 0x432, 0x438, 0x43b, 0x43b, 0x424, 0x438, 0x4b7, 0x438, 0x413,
0x430, 0x43c, 0x431, 0x438, 0x440, 0x41c, 0x438, 0x434, 0x432, 0x435, 0x439, 0x412,
0x435, 0x439, 0x43a, 0xb85, 0xbaa, 0xbbf, 0xb9f, 0xbcd, 0xb9c, 0xbbe, 0xba9, 0xbcd,
0xb85, 0xb95, 0xbcd, 0xbb0, 0xbbe, 0xb85, 0xb9f, 0xbbf, 0xbb8, 0xbcd, 0x20, 0xb85,
0xbaa, 0xbbe, 0xbaa, 0xbbe, 0xb85, 0xbb2, 0xbcd, 0xb9c, 0xbbf, 0xbaf, 0xbb0, 0xbcd,
0xbb8, 0xbcd, 0xb85, 0xbb8, 0xbcd, 0xbae, 0xbbe, 0xbb0, 0xbbe, 0xbaa, 0xbae, 0xbbe,
0xb95, 0xbcb, 0xbaa, 0xbbe, 0xb99, 0xbcd, 0xb95, 0xbc1, 0xbaf, 0xbc0, 0xbaa, 0xb9e,
0xbcd, 0xb9a, 0xbc1, 0xbb2, 0xbcd, 0xbaa, 0xbbf, 0xbb8, 0xbbe, 0xbb5, 0xbcd, 0xbaa,
0xbbf, 0xbb3, 0xbbe, 0xba9, 0xbcd, 0xb9f, 0xbc8, 0xbaf, 0xbb0, 0xbcd, 0xbaa, 0xbbf,
0xbb0, 0xbbe, 0xbb8, 0xbbe, 0xbb5, 0xbbf, 0xbb2, 0xbcd, 0xbb2, 0xbbf, 0xbaa, 0xbc1,
0xb9c, 0xbc1, 0xbae, 0xbcd, 0xbaa, 0xbc1, 0xbb0, 0xbbe, 0xb95, 0xbc6, 0xbaf, 0xbcd,
0xbb0, 0xbcb, 0xb95, 0xbbe, 0xbb8, 0xbbe, 0xbaa, 0xbbf, 0xbb3, 0xbbe, 0xba9, 0xbcd,
0xb95, 0xbbe, 0xb9a, 0xbbf, 0xbaf, 0xbc2, 0xb9f, 0xbcd, 0xb9f, 0xbbe, 0xb95, 0xbcb,
0xba9, 0xb95, 0xbcd, 0xbb0, 0xbc7, 0xb9f, 0xbbe, 0xb95, 0xbb0, 0xbcd, 0xba4, 0xbbe,
0xbb0, 0xbcd, 0x20, 0xb8e, 0xbb8, 0xbcd, 0x20, 0xb9a, 0xbb2, 0xbbe, 0xbae, 0xbcd,
0xb9c, 0xbbf, 0xbaa, 0xbcc, 0xb9f, 0xbcd, 0xb9f, 0xbbf, 0xba4, 0xbb5, 0xbc1, 0xbb2,
0xbbe, 0xb8e, 0xbb2, 0xbcd, 0x20, 0xb86, 0xbaf, 0xbc1, 0xba9, 0xbcd, 0xb83, 0xbaa,
0xbcd, 0xbb0, 0xbc0, 0xb9f, 0xbb5, 0xbc1, 0xba9, 0xbcd, 0xb95, 0xbaa, 0xbcb, 0xbb0,
0xbcb, 0xba9, 0xbcd, 0xbb9, 0xbb0, 0xbbe, 0xbb0, 0xbc7, 0xb9c, 0xbcb, 0xbb9, 0xba9,
0xbcd, 0xba9, 0xbb8, 0xbcd, 0xbaa, 0xbc6, 0xbb0, 0xbcd, 0xb95, 0xbcd, 0xb9c, 0xbc1,
0xbaa, 0xbbe, 0xb95, 0xbae, 0xbcd, 0xbaa, 0xbbe, 0xbb2, 0xbbe, 0xb95, 0xbbe, 0xbb0,
0xbcd, 0xb9f, 0xbcb, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xb95, 0xbb2, 0xbbf, 0xb95, 0xbbf,
0xba9, 0xbcd, 0xbb7, 0xb9a, 0xbbe, 0xbb2, 0xbbe, 0xb95, 0xbcb, 0xbb8, 0xbcd, 0xbb2,
0xbbf, 0xbaa, 0xbcd, 0xbb0, 0xbc7, 0xbb5, 0xbbf, 0xbb2, 0xbcd, 0xbb2, 0xbc7, 0xbb2,
0xbcb, 0xbae, 0xbcd, 0xbb2, 0xbc1, 0xbb5, 0xbbe, 0xba9, 0xbcd, 0xb9f, 0xbbe, 0xbb2,
0xbc1, 0xbaa, 0xbc1, 0xbae, 0xbcd, 0xbaa, 0xbbe, 0xbb7, 0xbbf, 0xbb2, 0xbc1, 0xb9a,
0xbbe, 0xb95, 0xbbe, 0xbae, 0xbbe, 0xbb2, 0xbaa, 0xbcb, 0xbae, 0xbaa, 0xbc1, 0xb9f,
0xbcb, 0xbae, 0xb9a, 0xbc7, 0xbb0, 0xbc1, 0xbaa, 0xbaa, 0xbbe, 0xba9, 0xbcd, 0xbae,
0xbca, 0xb95, 0xbbe, 0xba4, 0xbbf, 0xbb7, 0xbc1, 0xbae, 0xbbe, 0xba9, 0xbcd, 0xbb0,
0xbcb, 0xbb5, 0xbbf, 0xbaf, 0xbbe, 0xba8, 0xbc8, 0xbb0, 0xbcb, 0xbaa, 0xbbf, 0xb9c,
0xbae, 0xbc7, 0xba9, 0xbbe, 0xba8, 0xbbf, 0xbaf, 0xbbe, 0xbae, 0xbc7, 0xba8, 0xbcb,
0xbb5, 0xbbe, 0xb95, 0xbcd, 0xb9a, 0xbcb, 0xb9f, 0xbcd, 0xb85, 0xbb5, 0xbc1, 0xb95,
0xb9f, 0xbb5, 0xbcd, 0xb95, 0xbc1, 0xbaa, 0xbcb, 0xbb0, 0xbcd, 0xb9f, 0xbcd, 0xb9f,
0xbcb, 0x2d, 0xba8, 0xbcb, 0xbb5, 0xbcb, 0xb9a, 0xbbe, 0xbb5, 0xbcb, 0x20, 0xb9f,
0xbcb, 0xbae, 0xbc7, 0xba4, 0xbcd, 0xbb0, 0xbbf, 0xbaa, 0xbcb, 0xbb2, 0xbbf, 0xb9f,
0xbc1, 0xba9, 0xbbf, 0xbb8, 0xbcd, 0xbb5, 0xbc8, 0xba3, 0xbcd, 0xb9f, 0xbcd, 0xbb9,
0xbcb, 0xb95, 0xbcd, 0xb85, 0xb9f, 0xb95, 0xbcd, 0xb85, 0xb99, 0xbcd, 0xb95, 0xbcb,
0xbb0, 0xbc7, 0xb9c, 0xbcd, 0xb85, 0xb99, 0xbcd, 0xb95, 0xbc1, 0xbaf, 0xbc1, 0xbb2,
0xbbe, 0xb86, 0xba9, 0xbcd, 0xb9f, 0xbbf, 0xb95, 0xbc1, 0xbb5, 0xbbe, 0xb85, 0xbb0,
0xb95, 0xbc1, 0xbb5, 0xbbe, 0xbaf, 0xbcd, 0xba9, 0xbbe, 0xbaa, 0xbcd, 0xbaf, 0xbc2,
0xba9, 0xbcb, 0xbb8, 0xbcd, 0x20, 0xb8f, 0xbb0, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xb9f,
0xbae, 0xbbe, 0xbb0, 0xbcd, 0xb95, 0xbbe, 0xb95, 0xbbe, 0xbb0, 0xbcd, 0xb9f, 0xbcb,
0xbaa, 0xbbe, 0xb9c, 0xbc2, 0xb9c, 0xbc1, 0xbaf, 0xbcd, 0xbb2, 0xbbe, 0x20, 0xbb0,
0xbbf, 0xbaf, 0xbcb, 0xb9c, 0xbbe, 0xbae, 0xbc6, 0xba9, 0xbcd, 0xb9f, 0xbcb, 0xbb8,
0xbbe, 0xbb0, 0xbbf, 0xbaf, 0xbcb, 0x20, 0xb95, 0xbc7, 0xbb2, 0xbc6, 0xb95, 0xbcb,
0xbb8, 0xbcd, 0xb9a, 0xbbe, 0xbb2, 0xbcd, 0xb9f, 0xbbe, 0xb9a, 0xbbe, 0xba9, 0xbcd,
0x20, 0xb9c, 0xbc1, 0xbb5, 0xbbe, 0xba9, 0xbcd, 0xb9a, 0xbbe, 0xba9, 0xbcd, 0x20,
0xbb2, 0xbc2, 0xbaf, 0xbbf, 0xbb8, 0xbcd, 0xb9f, 0xbc1, 0xb95, 0xbc1, 0xbae, 0xba9,
0xbcd, 0xb89, 0xbb7, 0xbc1, 0xbb5, 0xbbe, 0xbaf, 0xbbe, 0xb85, 0xbb0, 0xbc2, 0xbaa,
0xbbe, 0xb85, 0xbb8, 0xba9, 0xbcd, 0xb9a, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0xb85, 0xb9f,
0xbbf, 0xb95, 0xbcb, 0xb95, 0xba9, 0xbcd, 0xbaa, 0xbbe, 0xbb9, 0xbbf, 0xbaf, 0xbbe,
0xbaa, 0xbb9, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xbaa, 0xba8, 0xbcd, 0xba4, 0xbc7, 0xbb0,
0xbbe, 0xbb8, 0xbcd, 0xbaa, 0xbbe, 0xbb0, 0xbcd, 0xbaa, 0xb9f, 0xbbe, 0xbb8, 0xbcd,
0xbaa, 0xbc6, 0xbb2, 0xbc6, 0xbae, 0xbcd, 0xbaa, 0xbc6, 0xbb2, 0xbbf, 0xbb8, 0xbcd,
0xbaa, 0xbcd, 0xbb2, 0xbbe, 0xb99, 0xbcd, 0xb95, 0xbcd, 0x2d, 0xb9a, 0xbaa, 0xbcd,
0xbb2, 0xbbe, 0xba9, 0xbcd, 0xbaa, 0xbcb, 0xbb5, 0xbbe, 0x20, 0xbb5, 0xbbf, 0xbb8,
0xbcd, 0xb9f, 0xbbe, 0xbaa, 0xbcb, 0xb95, 0xbcb, 0xb9f, 0xbbe, 0xbaa, 0xbcb, 0xbaf,
0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbc7, 0xbae, 0xbcd, 0xbaa, 0xbbf, 0xbb0, 0xbbf, 0xb9f,
0xbcd, 0xb9c, 0xbcd, 0x20, 0xbb5, 0xbb3, 0xbc8, 0xb95, 0xbc1, 0xb9f, 0xbbe, 0xb95,
0xbc7, 0xbae, 0xbcd, 0xbaa, 0xbcb, 0x20, 0xb95, 0xbbf, 0xbb0, 0xbbe, 0xba3, 0xbcd,
0xb9f, 0xbc7, 0xb95, 0xba9, 0xbcd, 0xb95, 0xbc1, 0xba9, 0xbcd, 0xb95, 0xbb0, 0xb95,
0xbbe, 0xbb8, 0xbcd, 0xb95, 0xbc6, 0xbaf, 0xbcd, 0xba9, 0xbcd, 0xb95, 0xbc7, 0xbae,
0xba9, 0xbcd, 0xb9a, 0xbbf, 0xb95, 0xbbe, 0xb95, 0xbcb, 0xb9a, 0xbc1, 0xbb5, 0xbbe,
0xbb5, 0xbbe, 0xb9a, 0xbbf, 0xbaf, 0xbc1, 0xb9f, 0xbbe, 0xb9f, 0xbcd, 0x20, 0xbb5,
0xbbe, 0xbb0, 0xbb8, 0xbcd, 0xb95, 0xbcb, 0xbb8, 0xbcd, 0xb9f, 0xbbe, 0x20, 0xbb0,
0xbbf, 0xb95, 0xbbe, 0xb95, 0xbcd, 0xbb0, 0xbc6, 0xbb8, 0xbcd, 0xb9f, 0xbbe, 0xba9,
0xbcd, 0xb95, 0xbc1, 0xbaf, 0xbbe, 0xbaa, 0xbc7, 0xb95, 0xbcd, 0xbaf, 0xbc2, 0xbb0,
0xbbe, 0xb95, 0xbcb, 0xb9f, 0xbc6, 0xba9, 0xbcd, 0xbae, 0xbbe, 0xbb0, 0xbcd, 0xb95,
0xbcd, 0xbb7, 0xbbe, 0xbb5, 0xbcd, 0xba9, 0xbcd, 0xb9f, 0xbbe, 0xbb5, 0xbcd, 0xb9a,
0xba9, 0xbcd, 0xb9f, 0xbbe, 0xbb5, 0xbcd, 0xb9a, 0xba9, 0xbcd, 0x20, 0xb95, 0xbbf,
0xbb0, 0xbc0, 0xb95, 0xbcd, 0xb9f, 0xbc6, 0xba9, 0xbcd, 0xbb5, 0xbb0, 0xbcd, 0xb9f,
0xbc6, 0xb9f, 0xbcd, 0xbb0, 0xbbe, 0xbaf, 0xbcd, 0xb9f, 0xbcd, 0xb9f, 0xbca, 0xbae,
0xbbf, 0xba9, 0xbbf, 0xb95, 0xbbe, 0xb8e, 0xb9f, 0xbcd, 0xbae, 0xbbe, 0xba9, 0xbcd,
0xb9f, 0xbbe, 0xba9, 0xbcd, 0xb88, 0xbb0, 0xbc1, 0xba9, 0xbc6, 0xbaa, 0xbc7, 0xb8e,
0xbb2, 0xbcd, 0x20, 0xb9a, 0xbbe, 0xbb2, 0xbcd, 0xbb5, 0xb9f, 0xbcb, 0xbb0, 0xbcd,
0xb83, 0xbaa, 0xbcb, 0xbb0, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xba8, 0xbc6, 0xbb2, 0xbcd,
0xb9a, 0xba9, 0xbcd, 0xb83, 0xbaa, 0xbcb, 0xbb0, 0xbcd, 0xb9f, 0xbbe, 0xbb2, 0xbc6,
0xb9a, 0xbbe, 0xb95, 0xbbf, 0xbb2, 0xbc7, 0xbb8, 0xbcd, 0x20, 0xbb5, 0xbb3, 0xbc8,
0xb95, 0xbc1, 0xb9f, 0xbbe, 0xb95, 0xbc2, 0xbb8, 0xbcd, 0x20, 0xbaa, 0xbc7, 0xb95,
0xbbf, 0xbb0, 0xbbe, 0xba3, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xb9f, 0xbb0, 0xbcd, 0xb95,
0xbcd, 0xb95, 0xbbf, 0xbb0, 0xba9, 0xb9f, 0xbbe, 0xb95, 0xbb5, 0xbc1, 0xb9f, 0xbc7,
0xbb2, 0xbc2, 0xbaa, 0xbcd, 0xb95, 0xbb5, 0xbc1, 0xba4, 0xbae, 0xbbe, 0xbb2, 0xbbe,
0xb95, 0xbc1, 0xbb5, 0xbbe, 0xbaf, 0xb95, 0xbbf, 0xbb2, 0xbcd, 0xb95, 0xbaf, 0xbbe,
0xba9, 0xbbe, 0xbb9, 0xbb2, 0xbbf, 0xb83, 0xbaa, 0xbc7, 0xb95, 0xbcd, 0xbb8, 0xbcd,
0xbb9, 0xbb5, 0xbbe, 0xba9, 0xbbe, 0xbb9, 0xbc6, 0xbb0, 0xbcd, 0xbae, 0xbcb, 0xb9a,
0xbbf, 0xbb2, 0xbcb, 0xb87, 0xba3, 0xbcd, 0xb9f, 0xbbf, 0xbaf, 0xbbe, 0xba9, 0xbbe,
0xbaa, 0xbca, 0xbb2, 0xbbf, 0xbb8, 0xbcd, 0xba8, 0xbbe, 0xb95, 0xbcd, 0xbb8, 0xbcd,
0x2c, 0x20, 0xb87, 0xba3, 0xbcd, 0xb9f, 0xbbf, 0xbaf, 0xbbe, 0xba9, 0xbbe, 0xbae,
0xbb0, 0xbc6, 0xba9, 0xbcd, 0xb95, 0xbcb, 0x2c, 0x20, 0xb87, 0xba3, 0xbcd, 0xb9f,
0xbbf, 0xbaf, 0xbbe, 0xba9, 0xbbe, 0xbaa, 0xbc0, 0xb9f, 0xbcd, 0xb9f, 0xbb0, 0xbcd,
0xbb8, 0xbcd, 0xbaa, 0xbc6, 0xbb0, 0xbcd, 0xb95, 0xbcd, 0x2c, 0x20, 0xb87, 0xba3,
0xbcd, 0xb9f, 0xbbf, 0xbaf, 0xbbe, 0xba9, 0xbbe, 0xb9f, 0xbc6, 0xbb2, 0xbcd, 0x20,
0xb9a, 0xbbf, 0xb9f, 0xbcd, 0xb9f, 0xbbf, 0x2c, 0x20, 0xb87, 0xba3, 0xbcd, 0xb9f,
0xbbf, 0xbaf, 0xbbe, 0xba9, 0xbbe, 0xbb5, 0xbc7, 0xbb5, 0xbc7, 0x2c, 0x20, 0xb87,
0xba3, 0xbcd, 0xb9f, 0xbbf, 0xbaf, 0xbbe, 0xba9, 0xbbe, 0xbb5, 0xbbf, 0xba9, 0xbcd,
0xb9a, 0xbc6, 0xba9, 0xbc7, 0xbb8, 0xbcd, 0x2c, 0x20, 0xb87, 0xba3, 0xbcd, 0xb9f,
0xbbf, 0xbaf, 0xbbe, 0xba9, 0xbbe, 0xbb5, 0xbbf, 0xba9, 0xbbe, 0xbae, 0xbc7, 0xb95,
0xbcd, 0x2c, 0x20, 0xb87, 0xba3, 0xbcd, 0xb9f, 0xbbf, 0xbaf, 0xbbe, 0xba9, 0xbbe,
0xb87, 0xba9, 0xbc1, 0xbb5, 0xbbf, 0xb95, 0xbcd, 0xb87, 0xb95, 0xbbe, 0xbb2, 0xbc2,
0xbaf, 0xbbf, 0xba4, 0xbcd, 0xb9c, 0xbae, 0xbc8, 0xb95, 0xbcd, 0xb95, 0xbbe, 0xb9c,
0xbc1, 0xba9, 0xbbf, 0xbaf, 0xbc2, 0xbb2, 0xbc2, 0xbaf, 0xbbf, 0xbb8, 0xbcd, 0xbb5,
0xbbf, 0xbb2, 0xbcd, 0xbb2, 0xbc7, 0xbae, 0xbbe, 0xba9, 0xbcd, 0xb9f, 0xbbf, 0xb9a,
0xbc6, 0xbb2, 0xbcd, 0xbb2, 0xbcb, 0x2c, 0x20, 0xb95, 0xbc6, 0xba9, 0xbcd, 0xb9f,
0xb95, 0xbbf, 0xb95, 0xbbf, 0xbb0, 0xbc6, 0xbb2, 0xba9, 0xbcd, 0xb9f, 0xbbf, 0xb9c,
0xbcd, 0xbb2, 0xbbe, 0x20, 0xbaa, 0xbbe, 0xbb8, 0xbcd, 0xbb2, 0xbbf, 0xbae, 0xbbe,
0xbb2, 0xbbe, 0xbb8, 0xbcd, 0x20, 0xb8f, 0xb9e, 0xbcd, 0xb9a, 0xbb2, 0xbcd, 0xbb8,
0xbcd, 0xbb2, 0xbcb, 0xbaf, 0xbb0, 0xbcd, 0x20, 0xbaa, 0xbbf, 0xbb0, 0xbbf, 0xba9,
0xbcd, 0xbb8, 0xbb8, 0xbcd, 0x20, 0xb95, 0xbc1, 0xbb5, 0xbbe, 0xbb0, 0xbcd, 0xb9f,
0xbcd, 0xb9f, 0xbb0, 0xbcd, 0xbae, 0xbc7, 0xb9a, 0xbbf, 0xbaf, 0xbcb, 0xbae, 0xbbe,
0xba9, 0xbbe, 0xb95, 0xbc1, 0xbb5, 0xbbe, 0xbae, 0xba9, 0xbbe, 0xbb8, 0xbcd, 0xbae,
0xbbe, 0xbb0, 0xbbf, 0xb95, 0xbbe, 0xb9f, 0xbcd, 0xbae, 0xbbe, 0xbb0, 0xbcd, 0xb9f,
0xbcd, 0xb9f, 0xbbf, 0xba9, 0xbbf, 0xb95, 0xbcd, 0xbae, 0xb9f, 0xbae, 0xbcb, 0xbb0,
0xbbe, 0xbb8, 0xbcd, 0xbae, 0xbb8, 0xb9f, 0xbcd, 0xbb2, 0xba9, 0xbcd, 0xbae, 0xbc6,
0xba9, 0xbcb, 0xbae, 0xbbf, 0xba9, 0xbbf, 0xbae, 0xbc6, 0xbb0, 0xbbf, 0xb9f, 0xbbe,
0xbae, 0xbc6, 0xb9f, 0xbcd, 0xbb2, 0xb95, 0xb9f, 0xbcd, 0xbb2, 0xbbe, 0xbae, 0xbc6,
0xb95, 0xbcd, 0xbb8, 0xbbf, 0xb95, 0xbcb, 0x20, 0xba8, 0xb95, 0xbb0, 0xbae, 0xbcd,
0xbae, 0xbbf, 0xb95, 0xbcd, 0xbaf, 0xbc2, 0xbb2, 0xba9, 0xbcd, 0xbae, 0xbbe, 0xb99,
0xbcd, 0xb9f, 0xbbe, 0xba9, 0xbcd, 0xbae, 0xbbe, 0xba9, 0xbcd, 0xb9f, 0xbc6, 0xbb0,
0xbcd, 0xbb0, 0xbc7, 0xbae, 0xbbe, 0xba9, 0xbcd, 0xb9f, 0xbc7, 0xbb5, 0xbc0, 0xb9f,
0xbbf, 0xbaf, 0xbcb, 0xbae, 0xbbe, 0xba9, 0xbcd, 0xb9a, 0xbc6, 0xbb0, 0xbc7, 0xb9f,
0xbcd, 0xba8, 0xb9a, 0xbb5, 0xbcd, 0xba8, 0xbbf, 0xbaf, 0xbc2, 0xbaf, 0xbbe, 0xbb0,
0xbcd, 0xb95, 0xbcd, 0xba8, 0xbcb, 0xbae, 0xbcd, 0xba8, 0xbcb, 0xbb0, 0xba9, 0xbcd,
0xbb9, 0xbbe, 0xbaa, 0xbc6, 0xbb5, 0xbc1, 0xbb2, 0xbbe, 0x2c, 0x20, 0xbb5, 0xb9f,
0xb95, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb9f, 0xb95, 0xbcb, 0xb9f, 0xbcd, 0xb9f, 0xbbe,
0xb9a, 0xbc6, 0xba9, 0xbcd, 0xb9f, 0xbb0, 0xbcd, 0x2c, 0x20, 0xbb5, 0xb9f, 0xb95,
0xbcd, 0xb95, 0xbc1, 0x20, 0xb9f, 0xb95, 0xbcb, 0xb9f, 0xbbe, 0xba8, 0xbbf, 0xbaf,
0xbc2, 0x20, 0xb9a, 0xbb2, 0xbc7, 0xbae, 0xbcd, 0x2c, 0x20, 0xbb5, 0xb9f, 0xb95,
0xbcd, 0xb95, 0xbc1, 0x20, 0xb9f, 0xb95, 0xbcb, 0xb9f, 0xbbe, 0xba8, 0xbc2, 0xb95,
0xbcd, 0xb92, 0xb9c, 0xbbf, 0xba9, 0xb95, 0xbbe, 0xbaa, 0xba9, 0xbbe, 0xbae, 0xbbe,
0xbaa, 0xbb0, 0xbae, 0xbb0, 0xbbf, 0xbaa, 0xbcb, 0xb83, 0xbaa, 0xbcb, 0xba9, 0xbbf,
0xb95, 0xbcd, 0xbb8, 0xbcd, 0xbaa, 0xbcb, 0xbb0, 0xbcd, 0xb9f, 0xbcd, 0x2d, 0xb85,
0xbb5, 0xbcd, 0x2d, 0xbaa, 0xbbf, 0xbb0, 0xbbf, 0xba9, 0xbcd, 0xbb8, 0xbcd, 0xbaa,
0xbcb, 0xbb0, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xb86, 0xb83, 0xbaa, 0xbcd, 0x20, 0xbb8,
0xbcd, 0xbaa, 0xbc6, 0xbaf, 0xbbf, 0xba9, 0xbcd, 0xbaa, 0xbcb, 0xbb0, 0xbcd, 0xb9f,
0xbcd, 0xb9f, 0xbcb, 0x20, 0xbb5, 0xbc6, 0xbb2, 0xbcd, 0xbb9, 0xbcb, 0xbaa, 0xbbf,
0xbaf, 0xbc2, 0xbb0, 0xbcd, 0xb9f, 0xbcb, 0x20, 0xbb0, 0xbbf, 0xb95, 0xbcb, 0xbaa,
0xbc1, 0xba9, 0xbcd, 0xb9f, 0xbbe, 0x20, 0xb85, 0xbb0, 0xbc0, 0xba9, 0xbbe, 0xbb8,
0xbcd, 0xbb0, 0xbbe, 0xba9, 0xbcd, 0xb95, 0xbbf, 0xba9, 0xbcd, 0x20, 0xb87, 0xba9,
0xbcd, 0xbb2, 0xbc6, 0xb9f, 0xbcd, 0xbb0, 0xbc6, 0xbb8, 0xbbf, 0xb83, 0xbaa, 0xbbf,
0xbb0, 0xbc6, 0xb9c, 0xbbf, 0xba9, 0xbbe, 0xbb0, 0xbc6, 0xb9a, 0xbb2, 0xbc2, 0xb9f,
0xbcd, 0xbb0, 0xbbf, 0xbaf, 0xbcb, 0x20, 0xbaa, 0xbbf, 0xbb0, 0xbbe, 0xba9, 0xbcd,
0xb95, 0xbcb, 0xb9a, 0xbbe, 0xba9, 0xbcd, 0xb9f, 0xbb0, 0xbc6, 0xbae, 0xbcd, 0xb9a,
0xbbe, 0xba3, 0xbcd, 0xb9f, 0xbbf, 0xbaf, 0xbbe, 0xb95, 0xbcb, 0xb9a, 0xbbe, 0xba3,
0xbcd, 0xb9f, 0xbcb, 0x20, 0xb9f, 0xbcb, 0xbae, 0xbbf, 0xb99, 0xbcd, 0xb95, 0xbcb,
0xb9a, 0xbbe, 0xbb5, 0xbcb, 0x20, 0xbaa, 0xbb5, 0xbc1, 0xbb2, 0xbcb, 0xb87, 0xb9f,
0xbcb, 0xb95, 0xbcb, 0xbb0, 0xbcd, 0xb9f, 0xbcb, 0xbb0, 0xbcd, 0xbae, 0xbbf, 0xb9f,
0xbcd, 0xb9a, 0xbbf, 0xb9f, 0xbcd, 0xb95, 0xbbe, 0xb9a, 0xbc6, 0xbaf, 0xbbf, 0xba9,
0xbcd, 0xb9f, 0xbcd, 0x20, 0xbaa, 0xbbe, 0xbb0, 0xbcd, 0xba4, 0xbc7, 0xbb2, 0xbc6,
0xbae, 0xbbf, 0xb9a, 0xbc6, 0xbaf, 0xbbf, 0xba9, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xb9c,
0xbbe, 0xba9, 0xbcd, 0xbb8, 0xbcd, 0xb9a, 0xbc6, 0xbaf, 0xbbf, 0xba9, 0xbcd, 0xb9f,
0xbcd, 0x20, 0xb95, 0xbc0, 0xb9f, 0xbcd, 0xbb8, 0xbcd, 0xb9a, 0xbc6, 0xbaf, 0xbbf,
0xba9, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xbb2, 0xbc2, 0xb9a, 0xbbf, 0xbaf, 0xbbe, 0xb9a,
0xbc6, 0xbaf, 0xbbf, 0xba9, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xba4, 0xbbe, 0xbae, 0xbb8,
0xbcd, 0xb9a, 0xbc6, 0xbaf, 0xbbf, 0xba9, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xbb5, 0xbbf,
0xba9, 0xbcd, 0xb9a, 0xbc6, 0xba9, 0xbcd, 0xb9f, 0xbcd, 0xbb8, 0xbcd, 0xbb5, 0xbbf,
0xb83, 0xbaa, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xb95, 0xbb0, 0xba3, 0xbcd, 0xb9f, 0xbcd,
0xba4, 0xbc6, 0xb95, 0xbc1, 0xb9a, 0xbbf, 0xb95, 0xbb2, 0xbcd, 0xbaa, 0xbbe, 0xba4,
0xbc1, 0xbb2, 0xbc7, 0xb9f, 0xbbf, 0xb9c, 0xbc1, 0xbb5, 0xbbe, 0xba9, 0xbbe, 0xb9f,
0xbca, 0xbb0, 0xbca, 0xba9, 0xbcd, 0xb9f, 0xbcb, 0xb9f, 0xbcb, 0xbb0, 0xbcd, 0xb9f,
0xbcb, 0xbb2, 0xbbe, 0xbb5, 0xbbe, 0xba9, 0xbcd, 0xb95, 0xbc2, 0xbb5, 0xbb0, 0xbcd,
0xbb5, 0xbca, 0xbaf, 0xbbf, 0xb9f, 0xbcd, 0xbb9, 0xbbe, 0xbb0, 0xbcd, 0xbb8, 0xbcd,
0xbb5, 0xbbf, 0xba9, 0xbcd, 0xba9, 0xbbf, 0xbaa, 0xbc6, 0xb95, 0xbcd, 0xbaf, 0xb95,
0xbc1, 0xb9f, 0xb9f, 0xbcd, 0xb95, 0xbc7, 0xbb8, 0xbbf, 0xb9f, 0xbc7, 0xbb5, 0xbbf,
0xbb8, 0xbcd, 0xb9f, 0xbc1, 0xbae, 0xbcb, 0xba3, 0xbcd, 0xb9f, 0xbcd, 0x2d, 0xb9f,
0xbbf, 0x20, 0xb89, 0xbb0, 0xbcd, 0xbb5, 0xbbf, 0xbb2, 0xbcd, 0xbb2, 0xbc7, 0xbae,
0xbbe, 0xb95, 0xbcd, 0xb95, 0xbbf, 0xbaf, 0xbc2, 0xbb0, 0xbbf, 0xbae, 0xbbe, 0xb9a,
0xba9, 0xbcd, 0xbae, 0xbc6, 0xb95, 0xbcd, 0xbae, 0xbb0, 0xbcd, 0xb9f, 0xbcb, 0xbaa,
0xbbe, 0xbb2, 0xbcd, 0xbae, 0xbb0, 0xbcd, 0xbb0, 0xbcb, 0xba4, 0xbc7, 0xbb0, 0xbbe,
0xbb8, 0xbcd, 0xbaf, 0xbcb, 0xbb5, 0xbbe, 0xb9f, 0xbcd, 0xbb0, 0xbcb, 0xbb2, 0xbcd,
0xbb5, 0xbcb, 0xbb8, 0xbcd, 0xb9f, 0xbcb, 0xb95, 0xbcd, 0xbb2, 0xbbe, 0xb99, 0xbcd,
0xb87, 0xbaf, 0xbb0, 0xbcd, 0xbaa, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0xb8f, 0xb9f, 0xba9,
0xbcd, 0xb85, 0xbb2, 0xbcd, 0xbae, 0xbbe, 0xba4, 0xbbf, 0xb85, 0xbae, 0xbcd, 0xbae,
0xbbe, 0xba9, 0xbcd, 0xb85, 0xba9, 0xb9f, 0xbc0, 0xbb0, 0xbcd, 0xb85, 0xb95, 0xbcd,
0xba4, 0xbb5, 0xbcd, 0xb85, 0xb95, 0xbcd, 0xb9f, 0xbcb, 0xbaa, 0xbcd, 0xb85, 0xbb7,
0xbcd, 0xb95, 0xbbe, 0xbaa, 0xbbe, 0xba4, 0xbcd, 0xb85, 0xb9f, 0xbbf, 0xbb0, 0xbbe,
0xbaa, 0xbbe, 0xb95, 0xbcd, 0xba4, 0xbbe, 0xba4, 0xbcd, 0xbaa, 0xbb9, 0xbcd, 0xbb0,
0xbc8, 0xba9, 0xbcd, 0xbaa, 0xbbe, 0xb95, 0xbcd, 0xb95, 0xbc2, 0xbaa, 0xbbe, 0xb99,
0xbcd, 0xb95, 0xbbe, 0xb95, 0xbcd, 0xbaa, 0xbbe, 0xbb0, 0xbcd, 0xba9, 0xbbe, 0xbb2,
0xbcd, 0xbaa, 0xbc6, 0xbaf, 0xbcd, 0xbb0, 0xbc2, 0xb9f, 0xbcd, 0xbaa, 0xbbf, 0xbb7,
0xbcd, 0xb95, 0xbc6, 0xb95, 0xbcd, 0xbaa, 0xbc1, 0xbb0, 0xbc1, 0xba9, 0xbc7, 0xb9a,
0xbbf, 0xb9f, 0xbbe, 0xb95, 0xbca, 0xbb4, 0xbc1, 0xbae, 0xbcd, 0xbaa, 0xbc1, 0xb9f,
0xbae, 0xbbe, 0xbb8, 0xbcd, 0xb95, 0xbb8, 0xbcd, 0xb9f, 0xbbe, 0xb95, 0xbcd, 0xb95,
0xbbe, 0xb9f, 0xbbf, 0xbb2, 0xbbf, 0xba4, 0xbc1, 0xbaa, 0xbbe, 0xbaf, 0xbcd, 0xba4,
0xbc1, 0xbb7, 0xba9, 0xbcd, 0xbaa, 0xbc7, 0xb83, 0xbaa, 0xbbe, 0xbae, 0xb95, 0xbc1,
0xbb8, 0xbcd, 0xb9f, 0xbbe, 0xbb9, 0xbc6, 0xbaa, 0xbcd, 0xbb0, 0xbbe, 0xba9, 0xbcd,
0xbb9, 0xbcb, 0x20, 0xb9a, 0xbbf, 0x20, 0xbae, 0xbbf, 0xba9, 0xbcd, 0x20, 0xb9a,
0xbbf, 0xb9f, 0xbcd, 0xb9f, 0xbbf, 0xbb9, 0xbbe, 0xb99, 0xbcd, 0xb95, 0xbbe, 0xb99,
0xbcd, 0xbb9, 0xbcb, 0xbb5, 0xbcd, 0xba4, 0xbcd, 0xb87, 0xbb0, 0xbcd, 0xb95, 0xbc1,
0xb9f, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0xb9c, 0xb95, 0xbbe, 0xbb0, 0xbcd, 0xba4,
0xbcd, 0xba4, 0xbbe, 0xb9c, 0xbc6, 0xbaf, 0xbaa, 0xbc2, 0xbb0, 0xbbe, 0xb9c, 0xbc6,
0xbb0, 0xbc1, 0xb9a, 0xbb2, 0xbc7, 0xbae, 0xbcd, 0xb95, 0xbbe, 0xbaa, 0xbc2, 0xbb2,
0xbcd, 0xb95, 0xbbe, 0xbae, 0xbcd, 0xb9a, 0xb9f, 0xbcd, 0xb95, 0xbbe, 0xb95, 0xbb0,
0xbbe, 0xb9a, 0xbcd, 0xb9a, 0xbbf, 0xb95, 0xbbe, 0xba4, 0xbcd, 0xbae, 0xbbe, 0xba3,
0xbcd, 0xb9f, 0xbc1, 0xb95, 0xbbe, 0xba9, 0xbcd, 0xb9f, 0xbbf, 0xb95, 0xbbe, 0xb95,
0xbca, 0xbb2, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbe, 0xb95, 0xbbf, 0xbb0, 0xbbe,
0xbb8, 0xbcd, 0xba9, 0xbcb, 0xbaf, 0xbbe, 0xbb0, 0xbcd, 0xb95, 0xbcd, 0xbb8, 0xbcd,
0xb95, 0xbcb, 0xbb2, 0xbbe, 0xbb2, 0xbae, 0xbcd, 0xbaa, 0xbc2, 0xbb0, 0xbcd, 0xb95,
0xbc1, 0xb9a, 0xbbf, 0xb99, 0xbcd, 0xb95, 0xbc1, 0xbb5, 0xbc8, 0xba4, 0xbcd, 0xbae,
0xb95, 0xbbe, 0xbb5, 0xbc1, 0xbae, 0xb95, 0xba4, 0xba9, 0xbcd, 0xbae, 0xb95, 0xbcd,
0xb95, 0xbb8, 0xbb0, 0xbcd, 0xbae, 0xba3, 0xbbf, 0xbb2, 0xbbe, 0xbae, 0xbb8, 0xbcd,
0xb95, 0xb9f, 0xbcd, 0xba8, 0xbbf, 0xb95, 0xbcb, 0xb9a, 0xbbf, 0xbaf, 0xbbe, 0xba8,
0xbcb, 0xbb5, 0xbcb, 0xb95, 0xbc1, 0xbb8, 0xbcd, 0xba8, 0xbc6, 0xb9f, 0xbcd, 0xbb8,
0xbcd, 0xb95, 0xbcd, 0xba8, 0xbcb, 0xbb5, 0xbcb, 0xb9a, 0xbc0, 0xbaa, 0xbbf, 0xbb0,
0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0xb93, 0xbae, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd,
0xb93, 0xbb0, 0xbb2, 0xbcd, 0xb83, 0xbaa, 0xbcd, 0xba9, 0xbcb, 0xbae, 0xbcd, 0x20,
0xbaa, 0xbc6, 0xba9, 0xbcd, 0xbaa, 0xbcb, 0xba9, 0xbcd, 0xb9f, 0xbbf, 0xbaf, 0xbbe,
0xba9, 0xbbe, 0xb95, 0xbcd, 0xbaa, 0xbbf, 0xbaf, 0xbbe, 0xb99, 0xbcd, 0xbaf, 0xbbe,
0xb99, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbe, 0xbb0, 0xbcd, 0xb95, 0xbcb, 0xbb8,
0xbcd, 0xb9f, 0xbbe, 0xba9, 0xbc7, 0xb95, 0xbbf, 0xbb8, 0xbbf, 0xbb2, 0xbcb, 0xbb0,
0xbcd, 0xb9f, 0xbbe, 0xbb0, 0xbbf, 0xbaf, 0xbbe, 0xba4, 0xbcd, 0xb9a, 0xb95, 0xbb2,
0xbbf, 0xba9, 0xbcd, 0xb9a, 0xbae, 0xbbe, 0xbb0, 0xbcd, 0xb95, 0xba3, 0xbcd, 0xb9f,
0xbcd, 0xb9a, 0xbbf, 0xbaf, 0xbcb, 0xbb2, 0xbcd, 0xbb7, 0xbbe, 0xb99, 0xbcd, 0xb95,
0xbbe, 0xbaf, 0xbcd, 0xb9a, 0xbbf, 0xb99, 0xbcd, 0xb95, 0xbaa, 0xbcd, 0xbaa, 0xbc2,
0xbb0, 0xbcd, 0xbb8, 0xbcd, 0xbb0, 0xbc6, 0xb9f, 0xbcd, 0xba8, 0xbbf, 0xb95, 0xbcb,
0xbb2, 0xbbf, 0xbae, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0xba4, 0xbbe, 0xbaf, 0xbcd,
0xbaa, 0xbc7, 0xba4, 0xbbe, 0xbb7, 0xbcd, 0xb95, 0xba3, 0xbcd, 0xb9f, 0xbcd, 0xba4,
0xbcd, 0xbaa, 0xbbf, 0xbb2, 0xbbf, 0xb9a, 0xbbf, 0xb9f, 0xbc6, 0xbb9, 0xbcd, 0xbb0,
0xbbe, 0xba9, 0xbcd, 0xba4, 0xbbf, 0xbae, 0xbcd, 0xbaa, 0xbc1, 0xb9f, 0xbcb, 0xb95,
0xbcd, 0xb95, 0xbbf, 0xbaf, 0xbcb, 0xb9f, 0xbbe, 0xbae, 0xbcd, 0xbb8, 0xbcd, 0xb95,
0xbcd, 0xb89, 0xbb2, 0xbbe, 0xba9, 0xbcd, 0xbaa, 0xbbe, 0xb9f, 0xbcd, 0xb9f, 0xbb0,
0xbcd, 0xb89, 0xbb0, 0xbc1, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xb89, 0xbb8, 0xbcd, 0xb9f,
0xbcd, 0x2d, 0xba8, 0xbc7, 0xbb0, 0xbbe, 0xbb5, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0xb9f,
0xbbf, 0xbaf, 0xba9, 0xbcd, 0xbb5, 0xbbf, 0xbb3, 0xbbe, 0xb9f, 0xbbf, 0xbb5, 0xbca,
0xbb8, 0xbcd, 0xba4, 0xbcb, 0xb95, 0xbcd, 0xbaf, 0xb95, 0xbc1, 0xb9f, 0xbcd, 0xbb8,
0xbcd, 0xb95, 0xbcd, 0xbb0, 0xb99, 0xbcd, 0xb95, 0xbc2, 0xba9, 0xbcd, 0xbaf, 0xbc6,
0xb95, 0xbbe, 0xb9f, 0xbbf, 0xbb0, 0xbbf, 0xba9, 0xbcd, 0xbaa, 0xbb0, 0xbcd, 0xb95,
0xbcd, 0xb8f, 0xbb0, 0xbc7, 0xbb5, 0xba9, 0xbcd, 0xb85, 0xb9a, 0xbcb, 0xbb0, 0xbb8,
0xbcd, 0xbaa, 0xbc6, 0xbb0, 0xbcd, 0xbae, 0xbc1, 0xb9f, 0xbbe, 0xb95, 0xbc7, 0xba9,
0xbb0, 0xbbf, 0xb95, 0xbc7, 0xbaa, 0xbcd, 0x20, 0xbb5, 0xbc6, 0xbb0, 0xbcd, 0xb9f,
0xbc7, 0xb83, 0xbaa, 0xbb0, 0xbcb, 0xbae, 0xb9f, 0xbbf, 0xbb0, 0xbbe, 0xbb0, 0xbc7,
0xb95, 0xbcd, 0xb9c, 0xbbe, 0xbb5, 0xbbf, 0xb95, 0xbcd, 0xba4, 0xbc6, 0xbb1, 0xbcd,
0xb95, 0xbc1, 0x20, 0xb9c, 0xbbe, 0xbb0, 0xbcd, 0xb9c, 0xbbf, 0xbaf, 0xbbe, 0xb9a,
0xbc6, 0xbaf, 0xbbf, 0xba9, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xbb9, 0xbc6, 0xbb2, 0xbc6,
0xba9, 0xbbe, 0xbb8, 0xbcd, 0xb9f, 0xbc7, 0xba9, 0xbcd, 0xbb2, 0xbbf, 0xb85, 0xb9f,
0xbbf, 0xbb2, 0xbc6, 0xbaf, 0xbcd, 0xb9f, 0xbcd, 0xbaa, 0xbbf, 0xbb0, 0xbbf, 0xbb8,
0xbcd, 0xbaa, 0xbc7, 0xba9, 0xbcd, 0xbaa, 0xbc1, 0xbb0, 0xbcb, 0xb95, 0xbcd, 0xb95,
0xba9, 0xbcd, 0x20, 0xbb9, 0xbbf, 0xbb2, 0xbcd, 0xb9f, 0xbbe, 0xbb0, 0xbcd, 0xbb5,
0xbbf, 0xba9, 0xbcd, 0xbaf, 0xbc2, 0xb95, 0xbcd, 0xbb2, 0xbbe, 0xbb9, 0xbcb, 0xbaa,
0xbb0, 0xbcd, 0xb9f, 0xbcd, 0xbb2, 0xbbf, 0xba9, 0xbcd, 0xb9f, 0xbc6, 0xbae, 0xba9,
0xbcd, 0xbb2, 0xbbe, 0xbb0, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xbb9, 0xbcb, 0xbb5, 0xbc7,
0xbae, 0xbc6, 0xbb2, 0xbcd, 0xbaa, 0xbcb, 0xbb0, 0xbcd, 0xba9, 0xbcd, 0xbaa, 0xbc6,
0xbb0, 0xbcd, 0xba4, 0xbcd, 0xb9a, 0xbbf, 0xb9f, 0xbcd, 0xba9, 0xbbf, 0xb86, 0xbae,
0xbcd, 0xbb8, 0xbcd, 0xb9f, 0xbcd, 0xbb0, 0xb9f, 0xbbe, 0xbae, 0xbcd, 0xb85, 0xba3,
0xbcd, 0xb9f, 0xbcb, 0xbb0, 0xbbe, 0xb85, 0xbb8, 0xbcd, 0xb9f, 0xbcd, 0xbb0, 0xbbe,
0xb95, 0xbbe, 0xba9, 0xbcd, 0xb8f, 0xba4, 0xba9, 0xbcd, 0xbb8, 0xbcd, 0xbaa, 0xbc6,
0xbb2, 0xbcd, 0xb95, 0xbbf, 0xbb0, 0xbc7, 0xb9f, 0xbcd, 0xbaa, 0xbc6, 0xbb0, 0xbcd,
0xbb2, 0xbbf, 0xba9, 0xbcd, 0xbaa, 0xbbf, 0xbb0, 0xb9f, 0xbbf, 0xbb8, 0xbcd, 0xbb2,
0xbbe, 0xbb5, 0xbbe, 0xbaa, 0xbc1, 0xbb0, 0xbc2, 0xb9a, 0xbb2, 0xbcd, 0xbb8, 0xbcd,
0xbaa, 0xbc1, 0xb9a, 0xbbe, 0xbb0, 0xbc6, 0xbb8, 0xbcd, 0xb9f, 0xbcd, 0xbaa, 0xbc1,
0xb9f, 0xbbe, 0xbaa, 0xbc6, 0xbb8, 0xbcd, 0xb9f, 0xbcd, 0xbaa, 0xb9a, 0xbbf, 0xb99,
0xbcd, 0xb9c, 0xbc6, 0xba9, 0xbcd, 0xb9a, 0xbbf, 0xb9a, 0xbbf, 0xba9, 0xbb5, 0xbcd,
0xb95, 0xbcb, 0xbaa, 0xba9, 0xbcd, 0xbb9, 0xbc7, 0xb95, 0xba9, 0xbcd, 0xb9f, 0xbaa,
0xbcd, 0xbb3, 0xbbf, 0xba9, 0xbcd, 0xb9c, 0xbbf, 0xbaa, 0xbcd, 0xbb0, 0xbbe, 0xbb2,
0xbcd, 0xb9f, 0xbb0, 0xbcd, 0xb95, 0xbb0, 0xbcd, 0xba9, 0xbb8, 0xbc7, 0xbb9, 0xbc6,
0xbb2, 0xbcd, 0xb9a, 0xbbf, 0xb99, 0xbcd, 0xb95, 0xbbf, 0xb90, 0xbb2, 0xbcd, 0x20,
0xb86, 0xb83, 0xbaa, 0xbcd, 0x20, 0xbae, 0xbc7, 0xba9, 0xbcd, 0xb87, 0xbb8, 0xbcd,
0xba4, 0xbbe, 0xba9, 0xbcd, 0xbaa, 0xbc1, 0xbb2, 0xbcd, 0xb9c, 0xbc6, 0xbb0, 0xbcd,
0xb9a, 0xbbf, 0xb95, 0xbb2, 0xbbf, 0xba9, 0xbbf, 0xba9, 0xbcd, 0xb95, 0xbbf, 0xbb0,
0xbbe, 0xb9f, 0xbcd, 0xb95, 0xbbf, 0xbb0, 0xbcb, 0xbb5, 0xbcd, 0xb95, 0xbc0, 0xbb5,
0xbcd, 0xbb2, 0xbbf, 0xbb8, 0xbcd, 0xbaa, 0xba9, 0xbcd, 0xb9c, 0xbc1, 0xbaa, 0xbcd,
0xbb2, 0xbbe, 0xba9, 0xbbe, 0xbb2, 0xba3, 0xbcd, 0xb9f, 0xba9, 0xbcd, 0xbb2, 0xb95,
0xbcd, 0xb9a, 0xbae, 0xbcd, 0xbaa, 0xbb0, 0xbcd, 0xb95, 0xbcd, 0xbae, 0xbc7, 0xb9f,
0xbcd, 0xbb0, 0xbbf, 0xb9f, 0xbcd, 0xbae, 0xbbe, 0xbb2, 0xbcd, 0xb9f, 0xbbe, 0xbae,
0xbb0, 0xbbf, 0xbb9, 0xbae, 0xbcd, 0xbae, 0xbbf, 0xba9, 0xbcd, 0xbb8, 0xbcd, 0xb95,
0xbcd, 0xbae, 0xbca, 0xba9, 0xbbe, 0xb95, 0xbcd, 0xb95, 0xbcb, 0xbae, 0xbbe, 0xbb8,
0xbcd, 0xb95, 0xbcb, 0xb93, 0xbb8, 0xbcd, 0xbb2, 0xbcb, 0xbaa, 0xbbe, 0xbb0, 0xbc0,
0xbb8, 0xbcd, 0xbaa, 0xbcb, 0xb9f, 0xbcd, 0xb95, 0xbcb, 0xbb0, 0xbbf, 0xb95, 0xbbe,
0xbaa, 0xbcd, 0xbb0, 0xbbe, 0xb95, 0xbcd, 0xbb0, 0xbcb, 0xbae, 0xbcd, 0xb9a, 0xbae,
0xbbe, 0xbb0, 0xbbe, 0xb9a, 0xbbe, 0xba9, 0xbcd, 0x20, 0xbae, 0xbb0, 0xbbf, 0xba9,
0xbcb, 0xb9a, 0xbb0, 0xbaf, 0xbc7, 0xbb5, 0xbcb, 0xb9a, 0xbb0, 0xb9f, 0xbcb, 0xbb5,
0xbcd, 0xb9a, 0xbbf, 0xbae, 0xbcd, 0xb83, 0xbaa, 0xbc6, 0xbb0, 0xbcb, 0xbaa, 0xbcb,
0xbb2, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcb, 0xbaa, 0xbcd, 0xb9c, 0xbc7, 0xb9a, 0xbcb,
0xb83, 0xbaa, 0xbbf, 0xbaf, 0xbbe, 0xbb8, 0xbcd, 0xb9f, 0xbbe, 0xb95, 0xbcd, 0xbb9,
0xbcb, 0xbae, 0xbcd, 0xb9f, 0xbb2, 0xbbf, 0xba9, 0xbcd, 0xba4, 0xbbf, 0xbb0, 0xbbe,
0xba9, 0xbc7, 0xb89, 0xbb2, 0xbcd, 0xbaf, 0xbbe, 0xba9, 0xbcb, 0xbb8, 0xbcd, 0xb95,
0xbcd, 0xbb5, 0xba4, 0xbc1, 0xbb8, 0xbcd, 0xbb5, 0xbbe, 0xb9f, 0xbbf, 0xb95, 0xba9,
0xbcd, 0xbb5, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0xba9, 0xbbe, 0xbb5, 0xbbf, 0xbb2, 0xbcd,
0xba9, 0xbbf, 0xbaf, 0xbb8, 0xbcd, 0xbb5, 0xbcb, 0xbb2, 0xbcd, 0xb95, 0xbcb, 0xb95,
0xbbf, 0xbb0, 0xbbe, 0xb9f, 0xbcd, 0xbb5, 0xbbe, 0xbb0, 0xbcd, 0xbb8, 0xbbe, 0xbb8,
0xb95, 0xbcd, 0xbb0, 0xbc6, 0xbaa, 0xbcd, 0xb9c, 0xbc2, 0xbb0, 0xbbf, 0xb9a, 0xbcd,
0xb86, 0xba3, 0xbcd, 0xb9f, 0xba9, 0xba8, 0xbb0, 0xbbf, 0xbb5, 0xbcb, 0xb9a, 0xbbe,
0xb95, 0xbcb, 0xbb8, 0xbcd, 0xb95, 0xbbf, 0xbb1, 0xbbf, 0xbb8, 0xbcd, 0xba4, 0xbc1,
0xbae, 0xbb8, 0xbcd, 0xb95, 0xbcb, 0xb95, 0xbcb, 0xbb8, 0xbcd, 0xb95, 0xbca, 0xbae,
0xbb0, 0xbcb, 0xb95, 0xbc6, 0xbb0, 0xbcd, 0xbaf, 0xbc2, 0xbb2, 0xbc6, 0xba9, 0xbcd,
0xbae, 0xbbe, 0xbb9, 0xbc7, 0xbae, 0xbbe, 0xbb2, 0xba4, 0xbcd, 0xba4, 0xbc0, 0xbb5,
0xbc1, 0xb95, 0xbb3, 0xbcd, 0xbae, 0xbca, 0xbb0, 0xbbf, 0xbb7, 0xbbf, 0xbaf, 0xbb8,
0xbcd, 0xbae, 0xbaf, 0xbcb, 0xb9f, 0xbcd, 0xb9f, 0xbbf, 0xbb0, 0xbc0, 0xbaf, 0xbc2,
0xba9, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0xb85, 0xbaa, 0xbbf, 0xbaf, 0xbbe, 0xb86, 0xb95,
0xbcd, 0xbb2, 0xbbe, 0xba8, 0xbcd, 0xba4, 0xbc1, 0xbaa, 0xbcb, 0xb95, 0xbc6, 0xbaf,
0xbcd, 0xba9, 0xbcd, 0xbb5, 0xbbf, 0xbb2, 0xbcd, 0xbb2, 0xbc7, 0xb9a, 0xba4, 0xbcd,
0xba4, 0xbbe, 0xbae, 0xbcd, 0xb9a, 0xbc1, 0xb95, 0xbcd, 0xb88, 0xbb8, 0xbcd, 0xb9f,
0xbb0, 0xbcd, 0xb88, 0xb83, 0xbaa, 0xbc7, 0xb9f, 0xbcd, 0xb83, 0xbaa, 0xb95, 0xbbe,
0xb83, 0xbaa, 0xbcb, 0xb83, 0xbaa, 0xbbf, 0xb9c, 0xbbf, 0xb83, 0xbaa, 0xbc1, 0xba9,
0xbbe, 0xb83, 0xbaa, 0xbc1, 0xb9f, 0xbbf, 0xb95, 0xbb2, 0xbaa, 0xb95, 0xbcb, 0xbb8,
0xbcd, 0xb95, 0xbc7, 0xbae, 0xbcd, 0xbaa, 0xbbf, 0xbaf, 0xbb0, 0xbcd, 0xb95, 0xbcd,
0xbb5, 0xbbe, 0xb9f, 0xbbe, 0xbb2, 0xbcd, 0xb95, 0xbc7, 0xba9, 0xbb2, 0xbcd, 0xb95,
0xbc1, 0xbb5, 0xbbe, 0xbae, 0xbcd, 0xbb9, 0xbcb, 0xba9, 0xbcb, 0xbb2, 0xbc2, 0xbb2,
0xbc2, 0xb8e, 0xba3, 0xbcd, 0xb9f, 0xbb0, 0xbcd, 0xbaa, 0xbb0, 0xbbf, 0xb95, 0xbbf,
0xbb0, 0xbbf, 0xb9f, 0xbbf, 0xbae, 0xbbe, 0xb9f, 0xbcd, 0xb9f, 0xbbf, 0xb95, 0xbcb,
0xbb8, 0xbcd, 0xbb0, 0xbc7, 0xb95, 0xbcd, 0xbb5, 0xbbe, 0xb9c, 0xbbe, 0xbb2, 0xbc0,
0xbaf, 0xba9, 0xbcd, 0xbae, 0xb9c, 0xbc1, 0xbb0, 0xbcb, 0xbae, 0xbbe, 0xbb0, 0xbcd,
0xb95, 0xbbf, 0xbaf, 0xbc2, 0xb9a, 0xbbe, 0xbb8, 0xbcd, 0xbae, 0xbbf, 0xb9f, 0xbcd,
0xbb5, 0xbc7, 0xba8, 0xbb5, 0xbcd, 0xbb0, 0xbc2, 0xba8, 0xbbf, 0xbaf, 0xbc1, 0xba8,
0xbbe, 0xbb0, 0xbcd, 0xb83, 0xbaa, 0xbcb, 0xb95, 0xbcd, 0xba8, 0xbcb, 0xbb5, 0xbc1,
0xbae, 0xbbf, 0xbaf, 0xbbe, 0xbaa, 0xbc7, 0xb95, 0xbcb, 0x20, 0xbaa, 0xbc7, 0xb95,
0xbcb, 0xbaa, 0xbbe, 0xbb2, 0xbb5, 0xbcd, 0xbaa, 0xbbf, 0xb9f, 0xbcd, 0xb95, 0xbc6,
0xbaf, 0xbcd, 0xbb0, 0xbcd, 0xba9, 0xbcd, 0xb83, 0xbaa, 0xbcb, 0xba9, 0xbcd, 0xbaa,
0xbc6, 0xbaf, 0xbcd, 0xbaa, 0xbcb, 0xbb0, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xbae, 0xbcb,
0xbb0, 0xbcd, 0xbb8, 0xbcd, 0xbaa, 0xbc8, 0xbb0, 0xbb0, 0xbcb, 0xb9f, 0xbcb, 0xb99,
0xbcd, 0xb95, 0xbbe, 0xb9a, 0xbc8, 0xbaa, 0xba9, 0xbcd, 0xba4, 0xbb9, 0xbbf, 0xba4,
0xbbf, 0xba4, 0xbb0, 0xbbe, 0xbb5, 0xbbe, 0xb9f, 0xbcb, 0xba9, 0xbcd, 0xb95, 0xb9f,
0xbaa, 0xbcd, 0xbaa, 0xbc1, 0xbb5, 0xbc7, 0xb95, 0xbcd, 0xbb5, 0xbbe, 0xbb2, 0xbbf,
0xbb8, 0xbcd, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x61, 0x6c, 0x61, 0x6e,
0x67, 0x20, 0x41, 0x6e, 0x6b, 0x72, 0x69, 0x6a, 0x69, 0x4a, 0x69, 0x6b,
0x61, 0x6e, 0x20, 0x61, 0x6c, 0x61, 0x6e, 0x67, 0x20, 0x47, 0x72, 0x61,
0x64, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x61, 0x6c, 0x61, 0x6e, 0x67,
0x20, 0x4a, 0x69, 0x63, 0x69, 0x61, 0x6b, 0x4a, 0x69, 0x6b, 0x61, 0x6e,
0x20, 0x61, 0x6c, 0x61, 0x6e, 0x67, 0x20, 0x54, 0x61, 0x6e, 0x62, 0x6f,
0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x61, 0x6c, 0x61, 0x6e, 0x67, 0x20,
0x49, 0x6e, 0x74, 0x69, 0x61, 0x6e, 0x6e, 0x61, 0x70, 0x6f, 0x72, 0x69,
0x73, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x61, 0x6c, 0x61, 0x6e, 0x67,
0x20, 0x52, 0x6f, 0x73, 0x61, 0x6e, 0x63, 0x69, 0x4a, 0x69, 0x6b, 0x61,
0x6e, 0x20, 0x61, 0x6c, 0x61, 0x6e, 0x67, 0x20, 0x4e, 0x69, 0x75, 0x79,
0x75, 0x65, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x61, 0x6c, 0x61, 0x6e,
0x67, 0x20, 0x42, 0x6f, 0x6e, 0x68, 0x75, 0x61, 0x6e, 0x4a, 0x69, 0x6b,
0x61, 0x6e, 0x20, 0x61, 0x6c, 0x61, 0x6e, 0x67, 0x20, 0x50, 0x75, 0x72,
0x61, 0x6e, 0x6b, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x61, 0x6c, 0x61,
0x6e, 0x67, 0x20, 0x53, 0x6e, 0x70, 0x61, 0x75, 0x72, 0x6f, 0x77, 0x4a,
0x69, 0x6b, 0x61, 0x6e, 0x20, 0x61, 0x6c, 0x61, 0x6e, 0x67, 0x20, 0x48,
0x6f, 0x6e, 0x6f, 0x72, 0x75, 0x72, 0x75, 0x418, 0x434, 0x438, 0x497, 0x430,
0x43d, 0x411, 0x430, 0x43d, 0x497, 0x443, 0x43b, 0x411, 0x4e9, 0x497, 0x4af, 0x43c,
0x431, 0x443, 0x440, 0x430, 0x41a, 0x430, 0x4bb, 0x438, 0x440, 0x4d9, 0x496, 0x443,
0x431, 0x430, 0x4ba, 0x430, 0x440, 0x442, 0x443, 0x43c, 0x41d, 0x497, 0x430, 0x43c,
0x435, 0x43d, 0x41a, 0x435, 0x43c, 0x431, 0x440, 0x438, 0x434, 0x436, 0x20, 0x411,
0x44d, 0x439, 0x41a, 0x430, 0x439, 0x43c, 0x430, 0x43d, 0x20, 0x443, 0x442, 0x440,
0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b, 0x413, 0x43b, 0x435, 0x439, 0x441, 0x20,
0x411, 0x44d, 0x439, 0x41a, 0x430, 0x437, 0x20, 0x431, 0x443, 0x445, 0x442, 0x430,
0x441, 0x44b, 0x422, 0x435, 0x43b, 0x43b, 0x44c, 0x2d, 0x421, 0x438, 0x442, 0x438,
0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d, 0x430, 0x41b, 0x443, 0x438,
0x441, 0x432, 0x438, 0x43b, 0x44c, 0x422, 0x4af, 0x431, 0x4d9, 0x43d, 0x433, 0x435,
0x20, 0x41a, 0x435, 0x43d, 0x4d9, 0x437, 0x20, 0x43a, 0x432, 0x430, 0x440, 0x442,
0x430, 0x43b, 0x44b, 0x41d, 0x43e, 0x43c, 0x435, 0x411, 0x44c, 0x44e, 0x43b, 0x430,
0x2c, 0x20, 0x422, 0x4e9, 0x43d, 0x44c, 0x44f, 0x43a, 0x20, 0x414, 0x430, 0x43a,
0x43e, 0x442, 0x430, 0x4ae, 0x437, 0x4d9, 0x43a, 0x2c, 0x20, 0x422, 0x4e9, 0x43d,
0x44c, 0x44f, 0x43a, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41d, 0x44c,
0x44e, 0x2d, 0x421, 0x430, 0x43b, 0x435, 0x43c, 0x2c, 0x20, 0x422, 0x4e9, 0x43d,
0x44c, 0x44f, 0x43a, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x418, 0x437,
0x433, 0x435, 0x20, 0x412, 0x430, 0x440, 0x444, 0x43e, 0x43b, 0x43e, 0x43c, 0x435,
0x439, 0x418, 0x437, 0x433, 0x435, 0x20, 0x41b, 0x44e, 0x441, 0x438, 0x44f, 0x41b,
0x43e, 0x43d, 0x433, 0x438, 0x435, 0x440, 0x431, 0x438, 0x435, 0x43d, 0x4d8, 0x43c,
0x43c, 0x430, 0x43d, 0x411, 0x4d9, 0x439, 0x440, 0x4af, 0x442, 0x414, 0x4d9, 0x43c,
0x4d9, 0x448, 0x43a, 0x44a, 0x4ba, 0x435, 0x432, 0x440, 0x43e, 0x43d, 0x496, 0x430,
0x43a, 0x430, 0x440, 0x442, 0x430, 0x496, 0x4d9, 0x44f, 0x43f, 0x443, 0x440, 0x430,
0x421, 0x4d9, 0x43c, 0x4d9, 0x440, 0x43a, 0x430, 0x43d, 0x434, 0x422, 0x4d9, 0x4bb,
0x440, 0x430, 0x43d, 0x410, 0x437, 0x43e, 0x440, 0x20, 0x443, 0x442, 0x440, 0x430,
0x443, 0x43b, 0x430, 0x440, 0x44b, 0x411, 0x435, 0x440, 0x43c, 0x443, 0x434, 0x20,
0x443, 0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b, 0x41a, 0x430, 0x43d,
0x430, 0x440, 0x20, 0x443, 0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b,
0x424, 0x430, 0x440, 0x435, 0x440, 0x20, 0x443, 0x442, 0x440, 0x430, 0x443, 0x43b,
0x430, 0x440, 0x44b, 0x41a, 0x4e9, 0x43d, 0x44c, 0x44f, 0x43a, 0x20, 0x414, 0x436,
0x43e, 0x440, 0x434, 0x436, 0x438, 0x44f, 0x418, 0x437, 0x433, 0x435, 0x20, 0x415,
0x43b, 0x435, 0x43d, 0x430, 0x4d8, 0x441, 0x442, 0x435, 0x440, 0x445, 0x430, 0x43d,
0x41c, 0x44d, 0x43d, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x44b, 0x41c, 0x4d9,
0x441, 0x43a, 0x4d9, 0x4af, 0x420, 0x430, 0x448, 0x442, 0x443, 0x430, 0x41c, 0x430,
0x43b, 0x44c, 0x434, 0x438, 0x432, 0x20, 0x443, 0x442, 0x440, 0x430, 0x443, 0x43b,
0x430, 0x440, 0x44b, 0x41f, 0x430, 0x441, 0x445, 0x430, 0x20, 0x443, 0x442, 0x440,
0x430, 0x432, 0x44b, 0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441,
0x20, 0x443, 0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b, 0x41a, 0x43e,
0x441, 0x440, 0x430, 0x44d, 0x41c, 0x430, 0x440, 0x43a, 0x438, 0x437, 0x20, 0x443,
0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b, 0xc05, 0xc2c, 0xc3f, 0xc21,
0xc4d, 0xc1c, 0xc3e, 0xc28, 0xc4d, 0xc05, 0xc15, 0xc4d, 0xc30, 0xc3e, 0xc2f, 0xc3e,
0xc21, 0xc3f, 0xc38, 0xc4d, 0x20, 0xc05, 0xc2c, 0xc3e, 0xc2c, 0xc3e, 0xc05, 0xc32,
0xc4d, 0xc1c, 0xc3f, 0xc2f, 0xc30, 0xc4d, 0xc38, 0xc4d, 0xc05, 0xc38, 0xc4d, 0xc2e,
0xc3e, 0xc30, 0xc3e, 0xc2c, 0xc3e, 0xc2e, 0xc3e, 0xc15, 0xc4b, 0xc2c, 0xc3e, 0xc02,
0xc17, 0xc41, 0xc2f, 0xc4d, 0xc2c, 0xc02, 0xc1c, 0xc42, 0xc32, 0xc4d, 0xc2c, 0xc3f,
0xc38, 0xc4d, 0xc38, 0xc3e, 0xc35, 0xc4d, 0xc2c, 0xc4d, 0xc32, 0xc3e, 0xc28, 0xc4d,
0xc1f, 0xc48, 0xc30, 0xc4d, 0xc2c, 0xc4d, 0xc30, 0xc3e, 0xc1c, 0xc3e, 0xc35, 0xc3f,
0xc32, 0xc4d, 0xc32, 0xc47, 0xc2c, 0xc41, 0xc1c, 0xc2e, 0xc4d, 0xc2c, 0xc41, 0xc30,
0xc3e, 0xc15, 0xc48, 0xc30, 0xc4b, 0xc15, 0xc3e, 0xc38, 0xc3e, 0xc2c, 0xc4d, 0xc32,
0xc3e, 0xc02, 0xc15, 0xc3e, 0xc38, 0xc4d, 0xc2f, 0xc42, 0xc1f, 0xc3e, 0xc15, 0xc4b,
0xc28, 0xc3e, 0xc15, 0xc4d, 0xc30, 0xc40, 0xc21, 0xc15, 0xc3e, 0xc30, 0xc4d, 0xc26,
0xc3e, 0xc30, 0xc4d, 0x20, 0xc0e, 0xc38, 0xc4d, 0x20, 0xc38, 0xc32, 0xc3e, 0xc2e,
0xc4d, 0xc21, 0xc3f, 0xc1c, 0xc4d, 0xc2c, 0xc4c, 0xc1f, 0xc3f, 0xc21, 0xc4c, 0xc32,
0xc3e, 0xc0e, 0xc32, 0xc4d, 0x20, 0xc0e, 0xc2f, 0xc41, 0xc28, 0xc4d, 0xc2b, 0xc4d,
0xc30, 0xc40, 0xc1f, 0xc4c, 0xc28, 0xc4d, 0xc17, 0xc2c, 0xc4b, 0xc30, 0xc4b, 0xc28,
0xc4d, 0xc39, 0xc30, 0xc3e, 0xc30, 0xc47, 0xc1c, 0xc4a, 0xc39, 0xc46, 0xc28, 0xc4d,
0xc38, 0xc4d, 0x200c, 0xc2c, 0xc30, 0xc4d, 0xc17, 0xc4d, 0xc1c, 0xc41, 0xc2c, 0xc3e,
0xc15, 0xc02, 0xc2a, 0xc3e, 0xc32, 0xc3e, 0xc16, 0xc3e, 0xc30, 0xc4d, 0xc1f, 0xc4b,
0xc2e, 0xc4d, 0xc15, 0xc40, 0xc17, 0xc32, 0xc3f, 0xc15, 0xc3f, 0xc28, 0xc4d, 0xc37,
0xc3e, 0xc38, 0xc3e, 0xc32, 0xc3e, 0xc17, 0xc4b, 0xc38, 0xc4d, 0xc32, 0xc46, 0xc2c,
0xc30, 0xc4d, 0xc35, 0xc3f, 0xc32, 0xc4d, 0xc32, 0xc47, 0xc32, 0xc4b, 0xc2e, 0xc4d,
0xc32, 0xc41, 0xc35, 0xc3e, 0xc02, 0xc21, 0xc3e, 0xc32, 0xc41, 0xc2c, 0xc02, 0xc2c,
0xc3e, 0xc37, 0xc3f, 0xc32, 0xc41, 0xc38, 0xc3e, 0xc15, 0xc3e, 0xc2e, 0xc32, 0xc3e,
0xc2c, 0xc4b, 0xc2e, 0xc3e, 0xc2a, 0xc41, 0xc1f, 0xc4b, 0xc2e, 0xc38, 0xc47, 0xc30,
0xc41, 0xc2c, 0xc3e, 0xc2c, 0xc46, 0xc28, 0xc4d, 0xc2e, 0xc4b, 0xc17, 0xc3e, 0xc26,
0xc3f, 0xc37, 0xc41, 0xc2e, 0xc4b, 0xc28, 0xc4d, 0xc30, 0xc4b, 0xc35, 0xc3f, 0xc2f,
0xc3e, 0xc28, 0xc48, 0xc30, 0xc4b, 0xc2c, 0xc40, 0xc21, 0xc4d, 0xc1c, 0xc3e, 0xc2e,
0xc46, 0xc28, 0xc3e, 0xc28, 0xc3f, 0xc2f, 0xc3e, 0xc2e, 0xc47, 0xc28, 0xc4d, 0xc35,
0xc3e, 0xc15, 0xc4d, 0xc37, 0xc4b, 0xc1f, 0xc4d, 0xc14, 0xc17, 0xc3e, 0xc21, 0xc4c,
0xc17, 0xc4b, 0xc35, 0xc4d, 0xc2a, 0xc4b, 0xc30, 0xc4d, 0xc1f, 0xc4b, 0x2d, 0xc28,
0xc4b, 0xc35, 0xc4b, 0xc38, 0xc3e, 0xc35, 0xc4b, 0x20, 0xc1f, 0xc4b, 0xc2e, 0xc4d,
0xc1f, 0xc4d, 0xc30, 0xc3f, 0xc2a, 0xc4b, 0xc32, 0xc3f, 0xc1f, 0xc4d, 0xc2f, 0xc41,
0xc28, 0xc3f, 0xc38, 0xc4d, 0xc35, 0xc3f, 0xc02, 0xc21, 0xc4d, 0xc39, 0xc4b, 0xc15,
0xc4d, 0xc05, 0xc21, 0xc3e, 0xc15, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc15, 0xc30, 0xc47,
0xc1c, 0xc4d, 0xc06, 0xc02, 0xc17, 0xc4d, 0xc35, 0xc3f, 0xc32, 0xc4d, 0xc32, 0xc3e,
0xc06, 0xc02, 0xc1f, 0xc3f, 0xc17, 0xc4d, 0xc35, 0xc3e, 0xc05, 0xc30, 0xc3e, 0xc17,
0xc4d, 0xc35, 0xc47, 0xc2f, 0xc40, 0xc28, 0xc3e, 0xc2c, 0xc4d, 0xc2f, 0xc42, 0xc28,
0xc4b, 0xc38, 0xc4d, 0x20, 0xc0e, 0xc2f, 0xc3f, 0xc30, 0xc4d, 0xc38, 0xc4d, 0xc15,
0xc1f, 0xc2e, 0xc3e, 0xc30, 0xc4d, 0xc15, 0xc3e, 0xc15, 0xc4b, 0xc30, 0xc4d, 0xc21,
0xc4b, 0xc2c, 0xc3e, 0xc1c, 0xc41, 0xc1c, 0xc41, 0xc2f, 0xc4d, 0xc32, 0xc3e, 0x20,
0xc30, 0xc3f, 0xc2f, 0xc4b, 0xc1c, 0xc2e, 0xc46, 0xc02, 0xc21, 0xc4b, 0xc1c, 0xc3e,
0xc30, 0xc3f, 0xc2f, 0xc4b, 0x20, 0xc17, 0xc32, 0xc4d, 0xc32, 0xc47, 0xc17, 0xc4b,
0xc38, 0xc4d, 0xc38, 0xc3e, 0xc32, 0xc4d, 0xc1f, 0xc3e, 0xc36, 0xc3e, 0xc28, 0xc4d,
0x20, 0xc1c, 0xc4d, 0xc2f, 0xc42, 0xc28, 0xc4d, 0xc36, 0xc3e, 0xc28, 0xc4d, 0x20,
0xc32, 0xc42, 0xc2f, 0xc3f, 0xc38, 0xc4d, 0xc1f, 0xc41, 0xc15, 0xc41, 0xc2e, 0xc28,
0xc4d, 0xc09, 0xc37, 0xc4d, 0xc2f, 0xc42, 0xc2f, 0xc05, 0xc30, 0xc41, 0xc2c, 0xc3e,
0xc05, 0xc38, 0xc41, 0xc28, 0xc4d, 0xc38, 0xc3f, 0xc2f, 0xc4b, 0xc28, 0xc4d, 0xc05,
0xc1f, 0xc3f, 0xc15, 0xc4b, 0xc15, 0xc28, 0xc4d, 0xc2c, 0xc39, 0xc3f, 0xc2f, 0xc2c,
0xc39, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc2c, 0xc02, 0xc21, 0xc30, 0xc3e, 0xc38, 0xc4d,
0xc2c, 0xc3e, 0xc30, 0xc4d, 0xc2c, 0xc21, 0xc4b, 0xc38, 0xc4d, 0xc2c, 0xc46, 0xc32,
0xc46, 0xc2e, 0xc4d, 0xc2c, 0xc46, 0xc32, 0xc40, 0xc1c, 0xc4d, 0xc2c, 0xc4d, 0xc32,
0xc3e, 0xc02, 0xc15, 0xc4d, 0x2d, 0xc38, 0xc2c, 0xc4d, 0xc32, 0xc4b, 0xc28, 0xc4d,
0xc2c, 0xc4b, 0xc35, 0xc3e, 0x20, 0xc35, 0xc3f, 0xc38, 0xc4d, 0xc1f, 0xc3e, 0xc2c,
0xc17, 0xc4b, 0xc1f, 0xc3e, 0xc2c, 0xc4a, 0xc2f, 0xc3f, 0xc38, 0xc40, 0xc15, 0xc47,
0xc02, 0xc2c, 0xc4d, 0xc30, 0xc3f, 0xc21, 0xc4d, 0xc1c, 0xc4d, 0x20, 0xc2c, 0xc47,
0xc15, 0xc3e, 0xc02, 0xc2a, 0xc4b, 0x20, 0xc17, 0xc4d, 0xc30, 0xc3e, 0xc02, 0xc21,
0xc4d, 0xc15, 0xc28, 0xc4d, 0x200c, 0xc15, 0xc42, 0xc28, 0xc4d, 0xc15, 0xc3e, 0xc30,
0xc3e, 0xc15, 0xc38, 0xc4d, 0xc15, 0xc2f, 0xc47, 0xc28, 0xc4d, 0xc15, 0xc47, 0xc2e,
0xc3e, 0xc28, 0xc4d, 0xc1a, 0xc3f, 0xc15, 0xc3e, 0xc17, 0xc4b, 0xc1a, 0xc41, 0xc35,
0xc3e, 0xc35, 0xc3e, 0xc38, 0xc3f, 0xc2f, 0xc41, 0xc26, 0xc3e, 0xc26, 0xc4d, 0x20,
0xc39, 0xc4d, 0xc35, 0xc3e, 0xc30, 0xc46, 0xc1c, 0xc4d, 0xc15, 0xc4b, 0xc38, 0xc4d,
0xc1f, 0xc3e, 0x20, 0xc30, 0xc3f, 0xc15, 0xc3e, 0xc15, 0xc4d, 0xc30, 0xc46, 0xc38,
0xc4d, 0xc1f, 0xc28, 0xc4d, 0xc15, 0xc41, 0xc2f, 0xc2c, 0xc3e, 0xc15, 0xc41, 0xc30,
0xc3e, 0xc15, 0xc35, 0xc4b, 0xc21, 0xc46, 0xc28, 0xc4d, 0xc2e, 0xc3e, 0xc30, 0xc4d,
0xc15, 0xc4d, 0x200c, 0xc37, 0xc3e, 0xc28, 0xc4d, 0xc21, 0xc3e, 0xc38, 0xc28, 0xc4d,
0xc21, 0xc3e, 0xc38, 0xc28, 0xc4d, 0x20, 0xc15, 0xc4d, 0xc30, 0xc40, 0xc15, 0xc4d,
0xc21, 0xc46, 0xc28, 0xc4d, 0xc35, 0xc46, 0xc30, 0xc4d, 0xc21, 0xc3f, 0xc1f, 0xc4d,
0xc30, 0xc4b, 0xc2f, 0xc3f, 0xc1f, 0xc4d, 0xc21, 0xc4a, 0xc2e, 0xc3f, 0xc28, 0xc3f,
0xc15, 0xc3e, 0xc0e, 0xc21, 0xc4d, 0xc2e, 0xc4b, 0xc02, 0xc1f, 0xc28, 0xc4d, 0xc07,
0xc30, 0xc41, 0xc28, 0xc46, 0xc2a, 0xc4d, 0xc0e, 0xc32, 0xc4d, 0x20, 0xc38, 0xc3e,
0xc32, 0xc4d, 0xc35, 0xc21, 0xc4b, 0xc30, 0xc4d, 0xc2b, 0xc4b, 0xc30, 0xc4d, 0xc1f,
0xc4d, 0x20, 0xc28, 0xc46, 0xc32, 0xc4d, 0xc38, 0xc28, 0xc4d, 0xc2b, 0xc4b, 0xc30,
0xc4d, 0xc1f, 0xc32, 0xc47, 0xc1c, 0xc3e, 0xc17, 0xc4d, 0xc32, 0xc47, 0xc38, 0xc4d,
0x20, 0xc2c, 0xc47, 0xc17, 0xc42, 0xc38, 0xc4d, 0x20, 0xc2c, 0xc47, 0xc17, 0xc4d,
0xc30, 0xc3e, 0xc02, 0xc21, 0xc4d, 0x20, 0xc1f, 0xc30, 0xc4d, 0xc15, 0xc4d, 0xc17,
0xc4d, 0xc30, 0xc46, 0xc28, 0xc21, 0xc3e, 0xc17, 0xc4d, 0xc35, 0xc3e, 0xc21, 0xc46,
0xc32, 0xc4b, 0xc2a, 0xc4d, 0xc17, 0xc4d, 0xc35, 0xc3e, 0xc1f, 0xc3f, 0xc2e, 0xc3e,
0xc32, 0xc3e, 0xc17, 0xc2f, 0xc3e, 0xc15, 0xc4d, 0xc35, 0xc3f, 0xc32, 0xc4d, 0xc17,
0xc2f, 0xc3e, 0xc28, 0xc3e, 0xc39, 0xc3e, 0xc32, 0xc3f, 0xc2b, 0xc3e, 0xc15, 0xc4d,
0xc38, 0xc4d, 0xc39, 0xc35, 0xc3e, 0xc28, 0xc3e, 0xc39, 0xc46, 0xc30, 0xc4d, 0xc2e,
0xc4b, 0xc38, 0xc3f, 0xc32, 0xc4d, 0xc32, 0xc4b, 0xc07, 0xc02, 0xc21, 0xc3f, 0xc2f,
0xc3e, 0xc28, 0xc2a, 0xc4b, 0xc32, 0xc3f, 0xc38, 0xc4d, 0xc28, 0xc4b, 0xc15, 0xc4d,
0xc38, 0xc4d, 0x2c, 0x20, 0xc07, 0xc02, 0xc21, 0xc3f, 0xc2f, 0xc3e, 0xc28, 0xc2e,
0xc30, 0xc46, 0xc02, 0xc17, 0xc4b, 0x2c, 0x20, 0xc07, 0xc02, 0xc21, 0xc3f, 0xc2f,
0xc3e, 0xc28, 0xc2a, 0xc40, 0xc1f, 0xc30, 0xc4d, 0xc38, 0xc4d, 0x200c, 0xc2c, 0xc30,
0xc4d, 0xc17, 0xc4d, 0x2c, 0x20, 0xc07, 0xc02, 0xc21, 0xc3f, 0xc2f, 0xc3e, 0xc28,
0xc1f, 0xc46, 0xc32, 0xc4d, 0x20, 0xc28, 0xc17, 0xc30, 0xc02, 0x2c, 0x20, 0xc07,
0xc02, 0xc21, 0xc3f, 0xc2f, 0xc3e, 0xc28, 0xc35, 0xc46, 0xc35, 0xc2f, 0xc4d, 0x2c,
0x20, 0xc07, 0xc02, 0xc21, 0xc3f, 0xc2f, 0xc3e, 0xc28, 0xc35, 0xc3f, 0xc28, 0xc4d,
0x200c, 0xc38, 0xc46, 0xc28, 0xc4d, 0xc28, 0xc46, 0xc38, 0xc4d, 0x2c, 0x20, 0xc07,
0xc02, 0xc21, 0xc3f, 0xc2f, 0xc3e, 0xc28, 0xc35, 0xc3f, 0xc28, 0xc3e, 0xc2e, 0xc3e,
0xc15, 0xc4d, 0x2c, 0x20, 0xc07, 0xc02, 0xc21, 0xc3f, 0xc2f, 0xc3e, 0xc28, 0xc07,
0xc28, 0xc41, 0xc35, 0xc3f, 0xc15, 0xc4d, 0xc07, 0xc15, 0xc4d, 0xc35, 0xc3e, 0xc32,
0xc3f, 0xc1f, 0xc4d, 0xc1c, 0xc2e, 0xc48, 0xc15, 0xc3e, 0xc1c, 0xc42, 0xc28, 0xc4b,
0xc32, 0xc42, 0xc2f, 0xc3f, 0xc35, 0xc3f, 0xc32, 0xc4d, 0xc2e, 0xc4b, 0xc02, 0xc1f,
0xc3f, 0xc38, 0xc46, 0xc32, 0xc4d, 0xc32, 0xc4b, 0x2c, 0x20, 0xc15, 0xc46, 0xc28,
0xc4d, 0x200c, 0xc1f, 0xc41, 0xc15, 0xc4d, 0xc15, 0xc40, 0xc15, 0xc4d, 0xc30, 0xc32,
0xc46, 0xc02, 0xc21, 0xc4d, 0xc1c, 0xc3f, 0xc15, 0xc4d, 0xc32, 0xc3e, 0x20, 0xc2a,
0xc3e, 0xc1c, 0xc4d, 0xc32, 0xc3f, 0xc2e, 0xc3e, 0xc32, 0xc3e, 0xc38, 0xc4d, 0x20,
0xc0f, 0xc02, 0xc1c, 0xc32, 0xc4d, 0xc38, 0xc4d, 0xc32, 0xc4b, 0xc2f, 0xc30, 0xc4d,
0x20, 0xc2a, 0xc4d, 0xc30, 0xc3f, 0xc28, 0xc4d, 0xc38, 0xc4d, 0x20, 0xc15, 0xc4d,
0xc35, 0xc3e, 0xc30, 0xc4d, 0xc1f, 0xc30, 0xc4d, 0xc2e, 0xc3e, 0xc38, 0xc3f, 0xc2f,
0xc4b, 0xc2e, 0xc28, 0xc3e, 0xc17, 0xc41, 0xc35, 0xc3e, 0xc2e, 0xc28, 0xc3e, 0xc38,
0xc4d, 0xc2e, 0xc3e, 0xc30, 0xc3f, 0xc17, 0xc4b, 0xc1f, 0xc4d, 0xc2e, 0xc3e, 0xc30,
0xc4d, 0xc1f, 0xc3f, 0xc28, 0xc40, 0xc15, 0xc4d, 0xc2e, 0xc3e, 0xc1f, 0xc2e, 0xc4a,
0xc30, 0xc4b, 0xc38, 0xc4d, 0xc2e, 0xc3e, 0xc38, 0xc1f, 0xc4d, 0x200c, 0xc32, 0xc3e,
0xc28, 0xc4d, 0xc2e, 0xc46, 0xc28, 0xc4b, 0xc2e, 0xc3f, 0xc28, 0xc40, 0xc2e, 0xc46,
0xc30, 0xc3f, 0xc21, 0xc3e, 0xc2e, 0xc46, 0xc1f, 0xc4d, 0xc32, 0xc15, 0xc1f, 0xc4d,
0xc32, 0xc2e, 0xc46, 0xc15, 0xc4d, 0xc38, 0xc3f, 0xc15, 0xc4b, 0x20, 0xc28, 0xc17,
0xc30, 0xc02, 0xc2e, 0xc3f, 0xc15, 0xc46, 0xc32, 0xc3e, 0xc28, 0xc4d, 0xc2e, 0xc4b,
0xc28, 0xc4d, 0x200c, 0xc15, 0xc4d, 0xc1f, 0xc4b, 0xc28, 0xc4d, 0xc2e, 0xc4b, 0xc02,
0xc1f, 0xc46, 0xc30, 0xc4d, 0xc30, 0xc47, 0xc2e, 0xc4b, 0xc02, 0xc1f, 0xc46, 0xc35,
0xc40, 0xc21, 0xc3f, 0xc2f, 0xc4b, 0xc2e, 0xc3e, 0xc02, 0xc1f, 0xc4d, 0xc38, 0xc46,
0xc30, 0xc3e, 0xc1f, 0xc4d, 0xc28, 0xc3e, 0xc38, 0xc4d, 0xc38, 0xc3e, 0xc35, 0xc4d,
0xc28, 0xc4d, 0xc2f, 0xc42, 0xc2f, 0xc3e, 0xc30, 0xc4d, 0xc15, 0xc4d, 0xc28, 0xc4b,
0xc2e, 0xc4d, 0xc28, 0xc30, 0xc4b, 0xc28, 0xc4d, 0xc39, 0xc3e, 0xc2c, 0xc4d, 0xc2f,
0xc41, 0xc32, 0xc3e, 0x2c, 0x20, 0xc09, 0xc24, 0xc4d, 0xc24, 0xc30, 0x20, 0xc21,
0xc15, 0xc4b, 0xc1f, 0xc38, 0xc46, 0xc02, 0xc1f, 0xc30, 0xc4d, 0x2c, 0x20, 0xc09,
0xc24, 0xc4d, 0xc24, 0xc30, 0x20, 0xc21, 0xc15, 0xc4b, 0xc1f, 0xc3e, 0xc28, 0xc4d,
0xc2f, 0xc42, 0x20, 0xc38, 0xc32, 0xc47, 0xc2e, 0xc4d, 0x2c, 0x20, 0xc09, 0xc24,
0xc4d, 0xc24, 0xc30, 0x20, 0xc21, 0xc15, 0xc4b, 0xc1f, 0xc28, 0xc42, 0xc15, 0xc4d,
0xc12, 0xc1c, 0xc3f, 0xc28, 0xc17, 0xc2a, 0xc28, 0xc3e, 0xc2e, 0xc3e, 0xc2a, 0xc30,
0xc3e, 0xc2e, 0xc3e, 0xc30, 0xc3f, 0xc2c, 0xc4b, 0xc2b, 0xc3f, 0xc28, 0xc3f, 0xc15,
0xc4d, 0xc38, 0xc4d, 0xc2a, 0xc4b, 0xc30, 0xc4d, 0xc1f, 0xc4d, 0x2d, 0xc05, 0xc35,
0xc4d, 0x2d, 0xc2a, 0xc4d, 0xc30, 0xc3f, 0xc28, 0xc4d, 0xc38, 0xc4d, 0xc2a, 0xc4b,
0xc30, 0xc4d, 0xc1f, 0xc4d, 0x20, 0xc06, 0xc2b, 0xc4d, 0x20, 0xc38, 0xc4d, 0xc2a,
0xc46, 0xc2f, 0xc3f, 0xc28, 0xc4d, 0xc2a, 0xc4b, 0xc30, 0xc4d, 0xc1f, 0xc4b, 0x20,
0xc35, 0xc46, 0xc32, 0xc4d, 0xc39, 0xc4b, 0xc2a, 0xc4d, 0xc2f, 0xc42, 0xc30, 0xc4d,
0xc1f, 0xc4b, 0x20, 0xc30, 0xc3f, 0xc15, 0xc4b, 0xc2a, 0xc41, 0xc02, 0xc1f, 0xc3e,
0x20, 0xc05, 0xc30, 0xc40, 0xc28, 0xc38, 0xc4d, 0xc30, 0xc28, 0xc4d, 0x200c, 0xc15,
0xc3f, 0xc28, 0xc4d, 0x20, 0xc07, 0xc28, 0xc4d, 0x200c, 0xc32, 0xc46, 0xc1f, 0xc4d,
0xc30, 0xc46, 0xc38, 0xc3f, 0xc2b, 0xc40, 0xc30, 0xc46, 0xc1c, 0xc40, 0xc28, 0xc3e,
0xc30, 0xc3f, 0xc1c, 0xc32, 0xc4d, 0xc2f, 0xc42, 0xc1f, 0xc4d, 0xc30, 0xc3f, 0xc2f,
0xc4b, 0x20, 0xc2c, 0xc4d, 0xc30, 0xc3e, 0xc02, 0xc15, 0xc4b, 0xc38, 0xc3e, 0xc02,
0xc1f, 0xc30, 0xc46, 0xc2e, 0xc4d, 0xc36, 0xc3e, 0xc02, 0xc1f, 0xc3f, 0xc2f, 0xc3e,
0xc17, 0xc4b, 0xc36, 0xc3e, 0xc02, 0xc1f, 0xc4b, 0x20, 0xc21, 0xc4b, 0xc2e, 0xc3f,
0xc02, 0xc17, 0xc4b, 0xc38, 0xc3e, 0xc35, 0xc4b, 0x20, 0xc2a, 0xc3e, 0xc32, 0xc4b,
0xc07, 0xc1f, 0xc4b, 0xc15, 0xc4d, 0xc15, 0xc4b, 0xc30, 0xc4d, 0xc1f, 0xc42, 0xc30,
0xc4d, 0xc2e, 0xc3f, 0xc1f, 0xc4d, 0xc38, 0xc3f, 0xc1f, 0xc4d, 0xc15, 0xc3e, 0xc38,
0xc46, 0xc2f, 0xc3f, 0xc02, 0xc1f, 0xc4d, 0x20, 0xc2c, 0xc30, 0xc4d, 0xc24, 0xc46,
0xc32, 0xc46, 0xc2e, 0xc40, 0xc38, 0xc46, 0xc2f, 0xc3f, 0xc02, 0xc1f, 0xc4d, 0x20,
0xc1c, 0xc3e, 0xc28, 0xc4d, 0xc38, 0xc4d, 0xc38, 0xc46, 0xc2f, 0xc3f, 0xc02, 0xc1f,
0xc4d, 0x20, 0xc15, 0xc3f, 0xc1f, 0xc4d, 0xc38, 0xc4d, 0xc38, 0xc46, 0xc2f, 0xc3f,
0xc02, 0xc1f, 0xc4d, 0x20, 0xc32, 0xc42, 0xc38, 0xc3f, 0xc2f, 0xc3e, 0xc38, 0xc46,
0xc2f, 0xc3f, 0xc02, 0xc1f, 0xc4d, 0x20, 0xc25, 0xc3e, 0xc2e, 0xc38, 0xc4d, 0xc38,
0xc46, 0xc2f, 0xc3f, 0xc02, 0xc1f, 0xc4d, 0x20, 0xc35, 0xc3f, 0xc28, 0xc4d, 0xc38,
0xc46, 0xc02, 0xc1f, 0xc4d, 0xc38, 0xc4d, 0xc35, 0xc3f, 0xc2b, 0xc4d, 0xc1f, 0xc4d,
0x20, 0xc15, 0xc30, 0xc46, 0xc02, 0xc1f, 0xc4d, 0xc24, 0xc46, 0xc17, 0xc41, 0xc38,
0xc3f, 0xc17, 0xc32, 0xc4d, 0xc2a, 0xc3e, 0xc25, 0xc41, 0xc32, 0xc47, 0xc1f, 0xc3f,
0xc1c, 0xc41, 0xc35, 0xc3e, 0xc28, 0xc3e, 0xc1f, 0xc4a, 0xc30, 0xc02, 0xc1f, 0xc4b,
0xc1f, 0xc4b, 0xc30, 0xc4d, 0xc1f, 0xc4b, 0xc32, 0xc3e, 0xc35, 0xc3e, 0xc28, 0xc4d,
0xc15, 0xc42, 0xc35, 0xc30, 0xc4d, 0xc35, 0xc48, 0xc1f, 0xc4d, 0x200c, 0xc39, 0xc3e,
0xc30, 0xc4d, 0xc38, 0xc4d, 0xc35, 0xc3f, 0xc28, 0xc4d, 0xc28, 0xc3f, 0xc2a, 0xc46,
0xc17, 0xc4d, 0xc2f, 0xc15, 0xc41, 0xc1f, 0xc3e, 0xc1f, 0xc4d, 0xc15, 0xc47, 0xc38,
0xc40, 0xc21, 0xc46, 0xc35, 0xc3f, 0xc38, 0xc4d, 0xc21, 0xc4d, 0xc2f, 0xc42, 0xc2e,
0xc3e, 0xc02, 0xc1f, 0xc4d, 0x20, 0xc21, 0xc3f, 0x2019, 0xc09, 0xc30, 0xc4d, 0xc35,
0xc3f, 0xc32, 0xc4d, 0xc32, 0xc47, 0xc2e, 0xc15, 0xc3e, 0xc30, 0xc40, 0xc2e, 0xc3e,
0xc38, 0xc28, 0xc4d, 0xc2e, 0xc46, 0xc15, 0xc4d, 0x200c, 0xc2e, 0xc41, 0xc30, 0xc4d,
0xc21, 0xc4b, 0xc2a, 0xc3e, 0xc32, 0xc4d, 0xc2e, 0xc30, 0xc4d, 0xc30, 0xc4a, 0xc24,
0xc47, 0xc30, 0xc3e, 0xc38, 0xc4d, 0xc2f, 0xc4b, 0xc35, 0xc3e, 0xc1f, 0xc4d, 0xc30,
0xc4b, 0xc32, 0xc4d, 0xc35, 0xc4b, 0xc38, 0xc4d, 0xc1f, 0xc4b, 0xc15, 0xc4d, 0xc32,
0xc3e, 0xc02, 0xc17, 0xc4d, 0x200c, 0xc2f, 0xc3f, 0xc2f, 0xc30, 0xc4d, 0x200c, 0xc2c,
0xc48, 0xc2f, 0xc28, 0xc4d, 0xc0e, 0xc21, 0xc46, 0xc28, 0xc4d, 0xc06, 0xc32, 0xc4d,
0xc2e, 0xc3e, 0xc1f, 0xc3f, 0xc05, 0xc2e, 0xc4d, 0xc2e, 0xc28, 0xc4d, 0xc05, 0xc28,
0xc21, 0xc48, 0xc30, 0xc4d, 0xc05, 0xc15, 0xc4d, 0xc1f, 0xc3e, 0xc35, 0xc4d, 0xc05,
0xc15, 0xc4d, 0xc1f, 0xc4b, 0xc2c, 0xc4d, 0xc2f, 0xc3e, 0xc37, 0xc4d, 0xc17, 0xc3e,
0xc2c, 0xc3e, 0xc1f, 0xc4d, 0xc06, 0xc1f, 0xc3f, 0xc30, 0xc3e, 0xc2c, 0xc3e, 0xc17,
0xc4d, 0xc26, 0xc3e, 0xc26, 0xc4d, 0xc2c, 0xc39, 0xc4d, 0xc30, 0xc46, 0xc2f, 0xc3f,
0xc28, 0xc4d, 0xc2c, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc15, 0xc3e, 0xc15, 0xc4d, 0xc2c,
0xc3e, 0xc30, 0xc4d, 0xc28, 0xc3e, 0xc32, 0xc4d, 0xc2c, 0xc40, 0xc30, 0xc1f, 0xc4d,
0xc2c, 0xc3f, 0xc37, 0xc4d, 0xc15, 0xc46, 0xc15, 0xc4d, 0xc2c, 0xc4d, 0xc30, 0xc42,
0xc28, 0xc48, 0xc1a, 0xc3f, 0xc24, 0xc3e, 0xc15, 0xc4a, 0xc32, 0xc02, 0xc2c, 0xc4b,
0xc21, 0xc2e, 0xc3e, 0xc38, 0xc4d, 0xc15, 0xc38, 0xc4d, 0xc22, 0xc3e, 0xc15, 0xc3e,
0xc21, 0xc3f, 0xc32, 0xc3f, 0xc26, 0xc41, 0xc2c, 0xc3e, 0xc2f, 0xc3f, 0xc21, 0xc41,
0xc37, 0xc28, 0xc4d, 0xc2c, 0xc40, 0xc2b, 0xc3e, 0xc2e, 0xc17, 0xc41, 0xc38, 0xc4d,
0xc1f, 0xc3e, 0xc17, 0xc3e, 0xc1c, 0xc3e, 0xc39, 0xc46, 0xc2c, 0xc4d, 0xc30, 0xc3e,
0xc28, 0xc4d, 0xc39, 0xc4b, 0x20, 0xc1a, 0xc3f, 0x20, 0xc2e, 0xc3f, 0xc28, 0xc4d,
0xc39, 0xc4d, 0x20, 0xc28, 0xc17, 0xc30, 0xc02, 0xc39, 0xc3e, 0xc02, 0xc15, 0xc3e,
0xc02, 0xc17, 0xc4d, 0xc39, 0xc4b, 0xc35, 0xc4d, 0xc21, 0xc4d, 0xc07, 0xc30, 0xc4d,
0xc15, 0xc41, 0xc1f, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0xc1c, 0xc15, 0xc3e, 0xc30,
0xc4d, 0xc24, 0xc3e, 0xc1c, 0xc2f, 0xc2a, 0xc41, 0xc30, 0xc1c, 0xc30, 0xc42, 0xc38,
0xc32, 0xc47, 0xc02, 0xc15, 0xc3e, 0xc2c, 0xc41, 0xc32, 0xc4d, 0xc15, 0xc2e, 0xc4d,
0x200c, 0xc1a, 0xc24, 0xc4d, 0xc15, 0xc3e, 0xc15, 0xc30, 0xc3e, 0xc1a, 0xc40, 0xc16,
0xc3e, 0xc1f, 0xc4d, 0xc2e, 0xc02, 0xc21, 0xc41, 0xc15, 0xc02, 0xc26, 0xc4d, 0xc2f,
0xc3e, 0xc17, 0xc15, 0xc4b, 0xc32, 0xc4d, 0x200c, 0xc15, 0xc24, 0xc3e, 0xc15, 0xc4d,
0xc30, 0xc38, 0xc28, 0xc4b, 0xc2f, 0xc3e, 0xc30, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d,
0xc15, 0xc4c, 0xc32, 0xc3e, 0xc32, 0xc02, 0xc2a, 0xc42, 0xc30, 0xc4d, 0xc15, 0xc41,
0xc1a, 0xc3f, 0xc02, 0xc17, 0xc4d, 0xc15, 0xc41, 0xc35, 0xc48, 0xc1f, 0xc4d, 0xc2e,
0xc15, 0xc3e, 0xc35, 0xc4d, 0xc2e, 0xc17, 0xc21, 0xc3e, 0xc28, 0xc4d, 0xc2e, 0xc15,
0xc3e, 0xc38, 0xc4d, 0xc38, 0xc30, 0xc4d, 0xc2e, 0xc28, 0xc40, 0xc32, 0xc3e, 0xc2e,
0xc38, 0xc4d, 0xc15, 0xc1f, 0xc4d, 0xc28, 0xc3f, 0xc15, 0xc4b, 0xc38, 0xc3f, 0xc2f,
0xc3e, 0xc28, 0xc4a, 0xc35, 0xc4a, 0xc15, 0xc41, 0xc1c, 0xc4d, 0x200c, 0xc28, 0xc46,
0xc1f, 0xc4d, 0x200c, 0xc38, 0xc4d, 0xc15, 0xc4d, 0xc28, 0xc35, 0xc4b, 0xc38, 0xc3f,
0xc2c, 0xc3f, 0xc30, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0xc13, 0xc2e, 0xc4d, 0xc38,
0xc4d, 0xc15, 0xc4d, 0xc13, 0xc30, 0xc32, 0xc4d, 0xc28, 0xc4b, 0xc2e, 0xc4d, 0x200c,
0xc2a, 0xc46, 0xc28, 0xc4d, 0xc39, 0xc4d, 0xc2a, 0xc4a, 0xc28, 0xc4d, 0xc1f, 0xc3f,
0xc2f, 0xc28, 0xc3e, 0xc15, 0xc4d, 0xc2a, 0xc4d, 0xc2f, 0xc4b, 0xc02, 0xc17, 0xc3e,
0xc02, 0xc17, 0xc4d, 0xc16, 0xc24, 0xc3e, 0xc30, 0xc4d, 0xc15, 0xc4b, 0xc38, 0xc4d,
0xc24, 0xc3e, 0xc28, 0xc47, 0xc15, 0xc4d, 0xc35, 0xc3f, 0xc1c, 0xc3f, 0xc32, 0xc4b,
0xc30, 0xc4d, 0xc21, 0xc3e, 0xc30, 0xc3f, 0xc2f, 0xc3e, 0xc27, 0xc4d, 0xc38, 0xc16,
0xc3e, 0xc32, 0xc3f, 0xc28, 0xc4d, 0xc38, 0xc2e, 0xc30, 0xc4d, 0xc15, 0xc3e, 0xc02,
0xc21, 0xc4d, 0xc38, 0xc3f, 0xc2f, 0xc4b, 0xc32, 0xc4d, 0xc37, 0xc3e, 0xc02, 0xc18,
0xc48, 0xc38, 0xc3f, 0xc02, 0xc17, 0xc2a, 0xc42, 0xc30, 0xc4d, 0xc38, 0xc4d, 0xc30,
0xc46, 0xc21, 0xc4d, 0xc28, 0xc46, 0xc15, 0xc4a, 0xc32, 0xc3f, 0xc2e, 0xc4d, 0xc38,
0xc4d, 0xc15, 0xc4d, 0xc24, 0xc48, 0xc2a, 0xc40, 0xc24, 0xc3e, 0xc37, 0xc4d, 0xc15,
0xc46, 0xc02, 0xc1f, 0xc4d, 0xc1f, 0xc3f, 0xc2c, 0xc3f, 0xc32, 0xc3f, 0xc38, 0xc3f,
0xc1f, 0xc46, 0xc39, 0xc4d, 0xc30, 0xc3e, 0xc28, 0xc4d, 0xc25, 0xc3f, 0xc02, 0xc2b,
0xc41, 0xc1f, 0xc4b, 0xc15, 0xc4d, 0xc2f, 0xc4b, 0xc1f, 0xc3e, 0xc2e, 0xc4d, 0xc38,
0xc4d, 0xc15, 0xc4d, 0xc09, 0xc32, 0xc3e, 0xc28, 0xc4d, 0xc2c, 0xc3e, 0xc1f, 0xc30,
0xc4d, 0xc09, 0xc30, 0xc41, 0xc2e, 0xc4d, 0x200c, 0xc15, 0xc40, 0xc05, 0xc38, 0xc4d,
0xc1f, 0xc4d, 0x2d, 0xc28, 0xc46, 0xc30, 0xc3e, 0xc35, 0xc3f, 0xc2f, 0xc28, 0xc4d,
0xc1f, 0xc48, 0xc28, 0xc4d, 0xc35, 0xc4d, 0xc32, 0xc3e, 0xc21, 0xc3f, 0xc35, 0xc4b,
0xc38, 0xc4d, 0xc1f, 0xc4b, 0xc15, 0xc4d, 0xc2f, 0xc15, 0xc41, 0xc1f, 0xc4d, 0xc38,
0xc4d, 0xc15, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc17, 0xc28, 0xc4d, 0xc2f, 0xc46, 0xc15,
0xc1f, 0xc46, 0xc30, 0xc3f, 0xc02, 0xc2c, 0xc30, 0xc4d, 0xc17, 0xc4d, 0xc2f, 0xc46,
0xc30, 0xc46, 0xc35, 0xc28, 0xc4d, 0xc05, 0xc1c, 0xc4b, 0xc30, 0xc4d, 0xc38, 0xc4d,
0xc2c, 0xc46, 0xc30, 0xc4d, 0xc2e, 0xc41, 0xc21, 0xc3e, 0xc15, 0xc46, 0xc28, 0xc30,
0xc40, 0xc15, 0xc47, 0xc2a, 0xc4d, 0x20, 0xc35, 0xc46, 0xc30, 0xc4d, 0xc21, 0xc46,
0xc2b, 0xc3e, 0xc30, 0xc4b, 0xc2e, 0xc26, 0xc48, 0xc30, 0xc3e, 0xc30, 0xc46, 0xc15,
0xc4d, 0xc1c, 0xc3e, 0xc35, 0xc3f, 0xc15, 0xc4d, 0xc26, 0xc15, 0xc4d, 0xc37, 0xc3f,
0xc23, 0x20, 0xc1c, 0xc3e, 0xc30, 0xc4d, 0xc1c, 0xc3f, 0xc2f, 0xc3e, 0xc38, 0xc46,
0xc2f, 0xc3f, 0xc02, 0xc1f, 0xc4d, 0x20, 0xc39, 0xc46, 0xc32, 0xc46, 0xc28, 0xc3e,
0xc38, 0xc4d, 0xc1f, 0xc3e, 0xc28, 0xc4d, 0xc32, 0xc40, 0xc05, 0xc21, 0xc46, 0xc32,
0xc48, 0xc21, 0xc4d, 0xc2c, 0xc4d, 0xc30, 0xc3f, 0xc38, 0xc4d, 0x200c, 0xc2c, 0xc46,
0xc2f, 0xc3f, 0xc28, 0xc4d, 0xc2c, 0xc4d, 0xc30, 0xc4b, 0xc15, 0xc46, 0xc28, 0xc4d,
0x20, 0xc39, 0xc3f, 0xc32, 0xc4d, 0xc21, 0xc3e, 0xc30, 0xc4d, 0xc35, 0xc3f, 0xc28,
0xc4d, 0xc2f, 0xc41, 0xc15, 0xc4d, 0xc32, 0xc3e, 0xc39, 0xc4b, 0xc2c, 0xc30, 0xc4d,
0xc1f, 0xc4d, 0xc32, 0xc3f, 0xc02, 0xc21, 0xc46, 0xc2e, 0xc3e, 0xc28, 0xc4d, 0xc32,
0xc3e, 0xc30, 0xc4d, 0xc21, 0xc4d, 0x20, 0xc39, 0xc4c, 0xc2e, 0xc46, 0xc32, 0xc4d,
0xc2c, 0xc4b, 0xc30, 0xc4d, 0xc28, 0xc4d, 0xc2a, 0xc46, 0xc30, 0xc4d, 0xc24, 0xc4d,
0xc38, 0xc3f, 0xc21, 0xc4d, 0xc28, 0xc40, 0xc06, 0xc2e, 0xc4d, 0x200c, 0xc38, 0xc4d,
0xc1f, 0xc30, 0xc4d, 0x200c, 0xc21, 0xc3e, 0xc2e, 0xc4d, 0xc05, 0xc02, 0xc21, 0xc4b,
0xc30, 0xc3e, 0xc05, 0xc38, 0xc4d, 0xc1f, 0xc4d, 0xc30, 0xc16, 0xc3e, 0xc28, 0xc4d,
0xc0f, 0xc25, 0xc46, 0xc28, 0xc4d, 0xc38, 0xc4d, 0xc2c, 0xc46, 0xc32, 0xc4d, 0x200c,
0xc17, 0xc4d, 0xc30, 0xc47, 0xc21, 0xc4d, 0xc2c, 0xc46, 0xc30, 0xc4d, 0xc32, 0xc3f,
0xc28, 0xc4d, 0xc2c, 0xc4d, 0xc30, 0xc3e, 0xc1f, 0xc3f, 0xc38, 0xc4d, 0xc32, 0xc3e,
0xc35, 0xc3e, 0xc2c, 0xc4d, 0xc30, 0xc38, 0xc4d, 0xc38, 0xc46, 0xc32, 0xc4d, 0xc38,
0xc4d, 0xc2c, 0xc41, 0xc15, 0xc3e, 0xc30, 0xc46, 0xc38, 0xc4d, 0xc1f, 0xc4d, 0xc2c,
0xc41, 0xc21, 0xc3e, 0xc2a, 0xc46, 0xc38, 0xc4d, 0xc1f, 0xc4d, 0xc2c, 0xc38, 0xc3f,
0xc02, 0xc1c, 0xc28, 0xc4d, 0xc1a, 0xc3f, 0xc38, 0xc3f, 0xc28, 0xc3e, 0xc35, 0xc4d,
0xc15, 0xc4b, 0xc2a, 0xc46, 0xc28, 0xc4d, 0xc39, 0xc3e, 0xc17, 0xc28, 0xc4d, 0xc21,
0xc2c, 0xc4d, 0xc32, 0xc3f, 0xc28, 0xc4d, 0xc1c, 0xc3f, 0xc2c, 0xc4d, 0xc30, 0xc3e,
0xc32, 0xc4d, 0xc1f, 0xc30, 0xc4d, 0xc17, 0xc4d, 0xc35, 0xc46, 0xc30, 0xc4d, 0xc28,
0xc4d, 0xc38, 0xc47, 0xc39, 0xc46, 0xc32, 0xc4d, 0xc38, 0xc3f, 0xc02, 0xc15, 0xc3f,
0xc10, 0xc32, 0xc4d, 0x20, 0xc06, 0xc2b, 0xc4d, 0x20, 0xc2e, 0xc47, 0xc28, 0xc4d,
0xc07, 0xc38, 0xc4d, 0xc24, 0xc3e, 0xc02, 0xc2c, 0xc41, 0xc32, 0xc4d, 0xc1c, 0xc46,
0xc30, 0xc4d, 0xc38, 0xc40, 0xc15, 0xc32, 0xc3f, 0xc28, 0xc3f, 0xc28, 0xc4d, 0x200c,
0xc17, 0xc4d, 0xc30, 0xc26, 0xc4d, 0xc15, 0xc3f, 0xc30, 0xc4b, 0xc35, 0xc4d, 0xc15,
0xc40, 0xc35, 0xc4d, 0xc32, 0xc3f, 0xc38, 0xc4d, 0xc2c, 0xc28, 0xc4d, 0xc32, 0xc4d,
0xc2f, 0xc42, 0xc2c, 0xc4d, 0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc28, 0xc3e, 0xc32, 0xc02,
0xc21, 0xc28, 0xc4d, 0xc32, 0xc15, 0xc4d, 0xc38, 0xc46, 0xc02, 0xc2c, 0xc30, 0xc4d,
0xc17, 0xc4d, 0xc2e, 0xc3e, 0xc21, 0xc4d, 0xc30, 0xc3f, 0xc21, 0xc4d, 0xc2e, 0xc3e,
0xc32, 0xc4d, 0xc1f, 0xc3e, 0xc2e, 0xc3e, 0xc30, 0xc40, 0xc2f, 0xc41, 0xc39, 0xc2e,
0xc4d, 0xc2e, 0xc3f, 0xc28, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0xc2e, 0xc4a, 0xc28,
0xc3e, 0xc15, 0xc4b, 0xc2e, 0xc3e, 0xc38, 0xc4d, 0xc15, 0xc4b, 0xc13, 0xc38, 0xc4d,
0xc32, 0xc4b, 0xc2a, 0xc4d, 0xc2f, 0xc3e, 0xc30, 0xc3f, 0xc38, 0xc4d, 0xc2a, 0xc4b,
0xc21, 0xc4d, 0xc17, 0xc4b, 0xc30, 0xc3f, 0xc15, 0xc3e, 0xc2a, 0xc4d, 0xc30, 0xc3e,
0xc17, 0xc4d, 0xc30, 0xc40, 0xc17, 0xc3e, 0xc30, 0xc4b, 0xc2e, 0xc4d, 0xc38, 0xc2e,
0xc3e, 0xc30, 0xc36, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc2e, 0xc3e, 0xc30, 0xc3f, 0xc28,
0xc4b, 0xc38, 0xc30, 0xc3e, 0xc1c, 0xc4b, 0xc35, 0xc4b, 0xc38, 0xc30, 0xc3e, 0xc1f,
0xc35, 0xc4d, 0xc38, 0xc3f, 0xc2e, 0xc4d, 0x200c, 0xc2b, 0xc46, 0xc30, 0xc4b, 0xc2a,
0xc4b, 0xc32, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4b, 0xc2a, 0xc4d, 0x200c, 0xc2f, 0xc47,
0xc38, 0xc4b, 0xc2b, 0xc3f, 0xc2f, 0xc3e, 0xc38, 0xc4d, 0xc1f, 0xc3e, 0xc15, 0xc4d,
0xc39, 0xc4b, 0xc2e, 0xc4d, 0xc24, 0xc3e, 0xc32, 0xc4d, 0xc32, 0xc3f, 0xc28, 0xc4d,
0xc1f, 0xc3f, 0xc30, 0xc47, 0xc28, 0xc4d, 0xc09, 0xc32, 0xc4d, 0xc2f, 0xc28, 0xc4b,
0xc35, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0xc35, 0xc3e, 0xc21, 0xc41, 0xc1c, 0xc4d,
0xc35, 0xc3e, 0xc1f, 0xc3f, 0xc15, 0xc28, 0xc4d, 0xc35, 0xc3f, 0xc2f, 0xc28, 0xc4d,
0xc28, 0xc3e, 0xc35, 0xc3f, 0xc32, 0xc4d, 0xc28, 0xc3f, 0xc2f, 0xc38, 0xc4d, 0xc35,
0xc4b, 0xc32, 0xc4d, 0xc17, 0xc4b, 0xc17, 0xc4d, 0xc30, 0xc3e, 0xc21, 0xc4d, 0xc35,
0xc3e, 0xc30, 0xc4d, 0xc37, 0xc3e, 0xc1c, 0xc3e, 0xc17, 0xc4d, 0xc30, 0xc46, 0xc2c,
0xc4d, 0xc1c, 0xc4d, 0xc2f, 0xc42, 0xc30, 0xc3f, 0xc1a, 0xc4d, 0xc05, 0xc02, 0xc1f,
0xc3e, 0xc28, 0xc3e, 0xc28, 0xc3e, 0xc30, 0xc3f, 0xc35, 0xc4b, 0xc1a, 0xc3e, 0xc17,
0xc4b, 0xc38, 0xc4d, 0xc15, 0xc4d, 0xc30, 0xc3f, 0xc38, 0xc4d, 0xc2e, 0xc38, 0xc4d,
0xc15, 0xc4b, 0xc15, 0xc4b, 0xc38, 0xc4d, 0xc15, 0xc4a, 0xc2e, 0xc4b, 0xc30, 0xc4b,
0xc15, 0xc46, 0xc30, 0xc4d, 0xc17, 0xc4d, 0xc2f, 0xc42, 0xc32, 0xc46, 0xc28, 0xc4d,
0xc2e, 0xc3e, 0xc39, 0xc46, 0xc2e, 0xc3e, 0xc32, 0xc4d, 0xc26, 0xc40, 0xc35, 0xc41,
0xc32, 0xc41, 0xc2e, 0xc3e, 0xc30, 0xc3f, 0xc37, 0xc38, 0xc4d, 0xc2e, 0xc2f, 0xc4b,
0xc1f, 0xc4d, 0xc30, 0xc40, 0xc2f, 0xc42, 0xc28, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0xc0f,
0xc2a, 0xc3f, 0xc2f, 0xc3e, 0xc06, 0xc15, 0xc4d, 0xc32, 0xc3e, 0xc02, 0xc21, 0xc4d,
0xc2c, 0xc4a, 0xc17, 0xc46, 0xc2f, 0xc3f, 0xc28, 0xc4d, 0x200c, 0xc35, 0xc3f, 0xc32,
0xc4d, 0xc32, 0xc47, 0xc1a, 0xc3e, 0xc25, 0xc2e, 0xc4d, 0xc1a, 0xc41, 0xc15, 0xc4d,
0xc08, 0xc38, 0xc4d, 0xc1f, 0xc30, 0xc4d, 0xc07, 0xc2b, 0xc47, 0xc1f, 0xc4d, 0xc2b,
0xc3e, 0xc15, 0xc4b, 0xc2b, 0xc4b, 0xc2b, 0xc40, 0xc1c, 0xc40, 0xc2b, 0xc41, 0xc28,
0xc3e, 0xc2b, 0xc41, 0xc1f, 0xc3f, 0xc17, 0xc3e, 0xc32, 0xc3e, 0xc2a, 0xc3e, 0xc17,
0xc4b, 0xc38, 0xc4d, 0xc17, 0xc3e, 0xc02, 0xc2c, 0xc3f, 0xc2f, 0xc47, 0xc30, 0xc4d,
0xc17, 0xc4d, 0xc35, 0xc3e, 0xc21, 0xc32, 0xc4d, 0xc15, 0xc46, 0xc28, 0xc3e, 0xc32,
0xc4d, 0xc17, 0xc4d, 0xc35, 0xc3e, 0xc2e, 0xc4d, 0xc39, 0xc4b, 0xc28, 0xc4b, 0xc32,
0xc41, 0xc32, 0xc41, 0xc0e, 0xc02, 0xc21, 0xc30, 0xc4d, 0xc2c, 0xc46, 0xc30, 0xc40,
0xc15, 0xc3f, 0xc30, 0xc40, 0xc1f, 0xc3f, 0xc2e, 0xc3e, 0xc1f, 0xc3f, 0xc15, 0xc4b,
0xc38, 0xc4d, 0xc30, 0xc47, 0xc15, 0xc4d, 0xc35, 0xc3e, 0xc1c, 0xc3e, 0xc32, 0xc48,
0xc28, 0xc4d, 0xc2e, 0xc1c, 0xc41, 0xc30, 0xc4b, 0xc2e, 0xc3e, 0xc30, 0xc4d, 0xc15,
0xc4d, 0xc35, 0xc47, 0xc38, 0xc3e, 0xc38, 0xc4d, 0xc2e, 0xc3f, 0xc21, 0xc4d, 0xc35,
0xc47, 0xc28, 0xc4c, 0xc30, 0xc41, 0xc28, 0xc3f, 0xc2f, 0xc42, 0xc28, 0xc4b, 0xc30,
0xc4d, 0xc2b, 0xc4b, 0xc15, 0xc4d, 0xc28, 0xc4c, 0xc2e, 0xc3f, 0xc2f, 0xc3e, 0xc2a,
0xc3e, 0xc17, 0xc4b, 0x20, 0xc2a, 0xc3e, 0xc17, 0xc4b, 0xc2a, 0xc3e, 0xc32, 0xc3e,
0xc35, 0xc4d, 0xc2a, 0xc3f, 0xc1f, 0xc4d, 0x200c, 0xc15, 0xc48, 0xc30, 0xc4d, 0xc28,
0xc4d, 0xc2a, 0xc4b, 0xc28, 0xc4d, 0x200c, 0xc2a, 0xc48, 0xc2a, 0xc4b, 0xc30, 0xc4d,
0xc1f, 0xc4d, 0x20, 0xc2e, 0xc4b, 0xc30, 0xc46, 0xc38, 0xc4d, 0xc2c, 0xc47, 0xc30,
0xc30, 0xc4b, 0xc1f, 0xc4b, 0xc02, 0xc17, 0xc3e, 0xc38, 0xc3e, 0xc2f, 0xc4d, 0xc2a,
0xc3e, 0xc28, 0xc4d, 0xc24, 0xc39, 0xc3f, 0xc24, 0xc3f, 0xc1f, 0xc30, 0xc3e, 0xc35,
0xc3e, 0xc1f, 0xc4b, 0xc02, 0xc17, 0xc3e, 0xc1f, 0xc3e, 0xc2a, 0xc41, 0xc35, 0xc47,
0xc15, 0xc4d, 0xc35, 0xc3e, 0xc32, 0xc4d, 0xc32, 0xc3f, 0xc38, 0xc4d, 0xe2d, 0xe32,
0xe1a, 0xe35, 0xe08, 0xe32, 0xe19, 0xe2d, 0xe31, 0xe01, 0xe01, 0xe23, 0xe32, 0xe41,
0xe2d, 0xe14, 0xe14, 0xe34, 0xe2a, 0xe2d, 0xe32, 0xe1a, 0xe32, 0xe1a, 0xe32, 0xe41,
0xe2d, 0xe25, 0xe40, 0xe08, 0xe35, 0xe22, 0xe23, 0xe4c, 0xe41, 0xe2d, 0xe2a, 0xe21,
0xe32, 0xe23, 0xe32, 0xe1a, 0xe32, 0xe21, 0xe32, 0xe42, 0xe01, 0xe1a, 0xe31, 0xe07,
0xe01, 0xe35, 0xe1a, 0xe31, 0xe19, 0xe08, 0xe39, 0xe25, 0xe1a, 0xe34, 0xe2a, 0xe40,
0xe0b, 0xe32, 0xe41, 0xe1a, 0xe25, 0xe19, 0xe44, 0xe17, 0xe23, 0xe4c, 0xe1a, 0xe23,
0xe32, 0xe0b, 0xe0b, 0xe32, 0xe27, 0xe34, 0xe25, 0xe1a, 0xe39, 0xe08, 0xe38, 0xe21,
0xe1a, 0xe39, 0xe23, 0xe32, 0xe44, 0xe04, 0xe42, 0xe23, 0xe04, 0xe32, 0xe2a, 0xe0b,
0xe32, 0xe1a, 0xe25, 0xe32, 0xe07, 0xe01, 0xe49, 0xe32, 0xe40, 0xe0b, 0xe27, 0xe15,
0xe32, 0xe42, 0xe01, 0xe19, 0xe32, 0xe01, 0xe23, 0xe35, 0xe14, 0xe32, 0xe01, 0xe32,
0xe23, 0xe4c, 0xe14, 0xe32, 0xe23, 0xe4c, 0xe40, 0xe2d, 0xe2a, 0xe0b, 0xe32, 0xe25,
0xe32, 0xe21, 0xe08, 0xe34, 0xe1a, 0xe39, 0xe15, 0xe35, 0xe14, 0xe39, 0xe2d, 0xe32,
0xe25, 0xe32, 0xe40, 0xe2d, 0xe25, 0xe44, 0xe2d, 0xe22, 0xe4c, 0xe2d, 0xe38, 0xe07,
0xe1f, 0xe23, 0xe35, 0xe17, 0xe32, 0xe27, 0xe19, 0xe4c, 0xe01, 0xe32, 0xe42, 0xe1a,
0xe42, 0xe23, 0xe40, 0xe19, 0xe2e, 0xe32, 0xe23, 0xe32, 0xe40, 0xe23, 0xe42, 0xe08,
0xe2e, 0xe31, 0xe19, 0xe40, 0xe19, 0xe2a, 0xe40, 0xe1a, 0xe2d, 0xe23, 0xe4c, 0xe01,
0xe08, 0xe39, 0xe1a, 0xe32, 0xe04, 0xe31, 0xe21, 0xe1e, 0xe32, 0xe25, 0xe32, 0xe04,
0xe32, 0xe23, 0xe4c, 0xe17, 0xe39, 0xe21, 0xe04, 0xe34, 0xe01, 0xe32, 0xe25, 0xe35,
0xe01, 0xe34, 0xe19, 0xe0a, 0xe32, 0xe0b, 0xe32, 0xe25, 0xe32, 0xe01, 0xe2d, 0xe2a,
0xe25, 0xe35, 0xe40, 0xe1a, 0xe23, 0xe2d, 0xe27, 0xe34, 0xe25, 0xe42, 0xe25, 0xe40,
0xe21, 0xe25, 0xe39, 0xe2d, 0xe31, 0xe19, 0xe14, 0xe32, 0xe25, 0xe39, 0xe1a, 0xe31,
0xe21, 0xe1a, 0xe32, 0xe0a, 0xe34, 0xe25, 0xe39, 0xe0b, 0xe32, 0xe01, 0xe32, 0xe21,
0xe32, 0xe25, 0xe32, 0xe42, 0xe1a, 0xe21, 0xe32, 0xe1b, 0xe39, 0xe42, 0xe15, 0xe21,
0xe32, 0xe40, 0xe0b, 0xe23, 0xe39, 0xe2d, 0xe31, 0xe21, 0xe1a, 0xe32, 0xe1a, 0xe32,
0xe40, 0xe19, 0xe42, 0xe21, 0xe01, 0xe32, 0xe14, 0xe34, 0xe0a, 0xe39, 0xe21, 0xe31,
0xe19, 0xe42, 0xe23, 0xe40, 0xe27, 0xe35, 0xe22, 0xe44, 0xe19, 0xe42, 0xe23, 0xe40,
0xe1a, 0xe35, 0xe22, 0xe40, 0xe2d, 0xe47, 0xe19, 0xe08, 0xe32, 0xe40, 0xe21, 0xe19,
0xe32, 0xe19, 0xe35, 0xe2d, 0xe32, 0xe40, 0xe21, 0xe22, 0xe4c, 0xe19, 0xe39, 0xe41,
0xe2d, 0xe01, 0xe0a, 0xe2d, 0xe15, 0xe27, 0xe32, 0xe01, 0xe32, 0xe14, 0xe39, 0xe01,
0xe39, 0xe1b, 0xe2d, 0xe23, 0xe4c, 0xe42, 0xe15, 0x2d, 0xe42, 0xe19, 0xe42, 0xe27,
0xe40, 0xe0b, 0xe32, 0xe15, 0xe39, 0xe40, 0xe21, 0xe15, 0xe23, 0xe35, 0xe42, 0xe1b,
0xe25, 0xe35, 0xe15, 0xe39, 0xe19, 0xe34, 0xe2a, 0xe27, 0xe34, 0xe19, 0xe14, 0xe4c,
0xe2e, 0xe38, 0xe01, 0xe40, 0xe2d, 0xe14, 0xe31, 0xe01, 0xe41, 0xe2d, 0xe07, 0xe40,
0xe04, 0xe2d, 0xe40, 0xe23, 0xe08, 0xe41, 0xe2d, 0xe07, 0xe01, 0xe34, 0xe25, 0xe25,
0xe32, 0xe41, 0xe2d, 0xe19, 0xe15, 0xe34, 0xe01, 0xe32, 0xe2d, 0xe32, 0xe23, 0xe32,
0xe01, 0xe31, 0xe27, 0xe19, 0xe32, 0xe1a, 0xe31, 0xe27, 0xe42, 0xe19, 0xe2a, 0xe44,
0xe2d, 0xe40, 0xe23, 0xe2a, 0xe01, 0xe32, 0xe15, 0xe32, 0xe21, 0xe32, 0xe23, 0xe4c,
0xe01, 0xe32, 0xe04, 0xe2d, 0xe23, 0xe4c, 0xe42, 0xe14, 0xe1a, 0xe32, 0xe08, 0xe39,
0xe08, 0xe34, 0xe27, 0xe25, 0xe32, 0xe23, 0xe34, 0xe42, 0xe2d, 0xe08, 0xe32, 0xe40,
0xe21, 0xe19, 0xe14, 0xe39, 0xe0b, 0xe32, 0xe23, 0xe34, 0xe42, 0xe2d, 0xe01, 0xe32,
0xe25, 0xe40, 0xe25, 0xe01, 0xe2d, 0xe2a, 0xe0b, 0xe31, 0xe25, 0xe15, 0xe32, 0xe0b,
0xe32, 0xe19, 0xe2e, 0xe27, 0xe19, 0xe0b, 0xe31, 0xe19, 0xe25, 0xe39, 0xe2d, 0xe34,
0xe2a, 0xe17, 0xe39, 0xe04, 0xe39, 0xe41, 0xe21, 0xe19, 0xe2d, 0xe39, 0xe0a, 0xe39,
0xe40, 0xe2d, 0xe35, 0xe22, 0xe2d, 0xe32, 0xe23, 0xe39, 0xe1a, 0xe32, 0xe2d, 0xe30,
0xe0b, 0xe38, 0xe19, 0xe0b, 0xe34, 0xe2d, 0xe2d, 0xe07, 0xe04, 0xe2d, 0xe23, 0xe31,
0xe25, 0xe2e, 0xe32, 0xe23, 0xe4c, 0xe40, 0xe1a, 0xe2d, 0xe23, 0xe4c, 0xe1a, 0xe32,
0xe40, 0xe22, 0xe35, 0xe22, 0xe1a, 0xe32, 0xe40, 0xe2d, 0xe35, 0xe22, 0xe1a, 0xe31,
0xe19, 0xe40, 0xe14, 0xe23, 0xe31, 0xe2a, 0xe1a, 0xe32, 0xe23, 0xe4c, 0xe40, 0xe1a,
0xe42, 0xe14, 0xe2a, 0xe40, 0xe1a, 0xe40, 0xe25, 0xe07, 0xe40, 0xe1a, 0xe25, 0xe35,
0xe0b, 0xe1a, 0xe25, 0xe31, 0xe07, 0xe04, 0xe4c, 0x2d, 0xe0b, 0xe32, 0xe1a, 0xe25,
0xe2d, 0xe19, 0xe1a, 0xe31, 0xe27, 0xe27, 0xe35, 0xe0a, 0xe15, 0xe32, 0xe42, 0xe1a,
0xe42, 0xe01, 0xe15, 0xe32, 0xe1a, 0xe2d, 0xe22, 0xe0b, 0xe35, 0xe2d, 0xe48, 0xe32,
0xe27, 0xe41, 0xe04, 0xe21, 0xe1a, 0xe23, 0xe34, 0xe14, 0xe08, 0xe4c, 0xe01, 0xe31,
0xe21, 0xe1b, 0xe39, 0xe01, 0xe23, 0xe31, 0xe19, 0xe14, 0xe35, 0xe41, 0xe04, 0xe19,
0xe04, 0xe38, 0xe19, 0xe04, 0xe32, 0xe23, 0xe32, 0xe04, 0xe31, 0xe2a, 0xe01, 0xe32,
0xe41, 0xe22, 0xe19, 0xe40, 0xe04, 0xe22, 0xe4c, 0xe41, 0xe21, 0xe19, 0xe0a, 0xe34,
0xe04, 0xe32, 0xe42, 0xe01, 0xe0a, 0xe35, 0xe27, 0xe32, 0xe27, 0xe32, 0xe0b, 0xe34,
0xe27, 0xe14, 0xe31, 0xe14, 0xe2e, 0xe31, 0xe27, 0xe40, 0xe23, 0xe0b, 0xe04, 0xe2d,
0xe2a, 0xe15, 0xe32, 0xe23, 0xe34, 0xe01, 0xe32, 0xe40, 0xe04, 0xe23, 0xe2a, 0xe15,
0xe31, 0xe19, 0xe01, 0xe38, 0xe22, 0xe32, 0xe1a, 0xe32, 0xe04, 0xe39, 0xe23, 0xe32,
0xe40, 0xe0b, 0xe32, 0xe14, 0xe32, 0xe19, 0xe21, 0xe32, 0xe23, 0xe4c, 0xe01, 0xe2a,
0xe2e, 0xe32, 0xe27, 0xe19, 0xe4c, 0xe14, 0xe2d, 0xe27, 0xe4c, 0xe2a, 0xe31, 0xe19,
0xe14, 0xe2d, 0xe27, 0xe4c, 0xe2a, 0xe31, 0xe19, 0x20, 0xe04, 0xe23, 0xe35, 0xe01,
0xe40, 0xe14, 0xe19, 0xe40, 0xe27, 0xe2d, 0xe23, 0xe4c, 0xe14, 0xe35, 0xe17, 0xe23,
0xe2d, 0xe22, 0xe15, 0xe4c, 0xe42, 0xe14, 0xe21, 0xe34, 0xe19, 0xe34, 0xe01, 0xe32,
0xe40, 0xe2d, 0xe14, 0xe21, 0xe31, 0xe19, 0xe15, 0xe31, 0xe19, 0xe40, 0xe2d, 0xe23,
0xe39, 0xe40, 0xe19, 0xe40, 0xe1b, 0xe40, 0xe2d, 0xe25, 0xe0b, 0xe31, 0xe25, 0xe27,
0xe32, 0xe14, 0xe2d, 0xe23, 0xe4c, 0xe1f, 0xe2d, 0xe23, 0xe4c, 0xe15, 0xe40, 0xe19,
0xe25, 0xe2a, 0xe31, 0xe19, 0xe1f, 0xe2d, 0xe23, 0xe4c, 0xe15, 0xe32, 0xe40, 0xe25,
0xe0b, 0xe32, 0xe40, 0xe01, 0xe25, 0xe0b, 0xe40, 0xe1a, 0xe22, 0xe4c, 0xe01, 0xe39,
0xe2a, 0xe40, 0xe1a, 0xe22, 0xe4c, 0xe41, 0xe01, 0xe23, 0xe19, 0xe14, 0xe4c, 0xe40,
0xe15, 0xe34, 0xe23, 0xe4c, 0xe01, 0xe40, 0xe01, 0xe23, 0xe19, 0xe32, 0xe14, 0xe32,
0xe01, 0xe27, 0xe32, 0xe40, 0xe14, 0xe2d, 0xe25, 0xe39, 0xe1b, 0xe01, 0xe31, 0xe27,
0xe40, 0xe15, 0xe21, 0xe32, 0xe25, 0xe32, 0xe01, 0xe31, 0xe27, 0xe22, 0xe32, 0xe01,
0xe34, 0xe25, 0xe01, 0xe32, 0xe22, 0xe2d, 0xe32, 0xe19, 0xe32, 0xe41, 0xe2e, 0xe25,
0xe34, 0xe41, 0xe1f, 0xe01, 0xe0b, 0xe4c, 0xe2e, 0xe32, 0xe27, 0xe32, 0xe19, 0xe32,
0xe40, 0xe2d, 0xe23, 0xe4c, 0xe42, 0xe21, 0xe0b, 0xe35, 0xe42, 0xe22, 0xe2d, 0xe34,
0xe19, 0xe40, 0xe14, 0xe35, 0xe22, 0xe41, 0xe19, 0xe42, 0xe1e, 0xe25, 0xe34, 0xe2a,
0xe19, 0xe2d, 0xe01, 0xe0b, 0xe4c, 0x2c, 0x20, 0xe2d, 0xe34, 0xe19, 0xe14, 0xe35,
0xe41, 0xe2d, 0xe19, 0xe32, 0xe21, 0xe32, 0xe40, 0xe23, 0xe07, 0xe42, 0xe01, 0x2c,
0x20, 0xe2d, 0xe34, 0xe19, 0xe14, 0xe35, 0xe41, 0xe2d, 0xe19, 0xe32, 0xe1b, 0xe35,
0xe40, 0xe15, 0xe2d, 0xe23, 0xe4c, 0xe2a, 0xe40, 0xe1a, 0xe34, 0xe23, 0xe4c, 0xe01,
0x2c, 0x20, 0xe2d, 0xe34, 0xe19, 0xe14, 0xe35, 0xe41, 0xe2d, 0xe19, 0xe32, 0xe40,
0xe17, 0xe25, 0xe25, 0xe4c, 0xe0b, 0xe34, 0xe15, 0xe35, 0x2c, 0x20, 0xe2d, 0xe34,
0xe19, 0xe14, 0xe35, 0xe41, 0xe2d, 0xe19, 0xe32, 0xe27, 0xe35, 0xe40, 0xe27, 0xe22,
0xe4c, 0x2c, 0x20, 0xe2d, 0xe34, 0xe19, 0xe14, 0xe35, 0xe41, 0xe2d, 0xe19, 0xe32,
0xe27, 0xe34, 0xe19, 0xe40, 0xe0b, 0xe19, 0xe40, 0xe19, 0xe2a, 0x2c, 0x20, 0xe2d,
0xe34, 0xe19, 0xe14, 0xe35, 0xe41, 0xe2d, 0xe19, 0xe32, 0xe27, 0xe34, 0xe19, 0xe32,
0xe41, 0xe21, 0xe04, 0x2c, 0x20, 0xe2d, 0xe34, 0xe19, 0xe14, 0xe35, 0xe41, 0xe2d,
0xe19, 0xe32, 0xe2d, 0xe34, 0xe19, 0xe39, 0xe27, 0xe34, 0xe01, 0xe2d, 0xe35, 0xe01,
0xe27, 0xe32, 0xe25, 0xe34, 0xe15, 0xe08, 0xe32, 0xe40, 0xe21, 0xe01, 0xe32, 0xe08,
0xe39, 0xe42, 0xe19, 0xe25, 0xe39, 0xe2a, 0xe4c, 0xe27, 0xe34, 0xe25, 0xe25, 0xe4c,
0xe21, 0xe2d, 0xe19, 0xe15, 0xe34, 0xe40, 0xe0b, 0xe25, 0xe42, 0xe25, 0x2c, 0x20,
0xe40, 0xe04, 0xe19, 0xe15, 0xe31, 0xe01, 0xe01, 0xe35, 0xe04, 0xe23, 0xe32, 0xe40,
0xe25, 0xe19, 0xe14, 0xe34, 0xe08, 0xe04, 0xe4c, 0xe25, 0xe32, 0xe1b, 0xe32, 0xe0b,
0xe25, 0xe34, 0xe21, 0xe32, 0xe25, 0xe2d, 0xe2a, 0xe41, 0xe2d, 0xe07, 0xe40, 0xe08,
0xe25, 0xe34, 0xe2a, 0xe42, 0xe25, 0xe40, 0xe27, 0xe2d, 0xe23, 0xe4c, 0xe1e, 0xe23,
0xe34, 0xe19, 0xe0b, 0xe4c, 0x20, 0xe04, 0xe27, 0xe2d, 0xe40, 0xe15, 0xe2d, 0xe23,
0xe4c, 0xe21, 0xe32, 0xe40, 0xe0b, 0xe42, 0xe2d, 0xe21, 0xe32, 0xe19, 0xe32, 0xe01,
0xe31, 0xe27, 0xe21, 0xe32, 0xe40, 0xe19, 0xe32, 0xe2a, 0xe4c, 0xe21, 0xe32, 0xe23,
0xe34, 0xe42, 0xe01, 0xe15, 0xe4c, 0xe21, 0xe32, 0xe23, 0xe4c, 0xe15, 0xe34, 0xe19,
0xe35, 0xe01, 0xe21, 0xe32, 0xe15, 0xe32, 0xe42, 0xe21, 0xe23, 0xe2d, 0xe2a, 0xe21,
0xe32, 0xe0b, 0xe32, 0xe17, 0xe25, 0xe32, 0xe19, 0xe40, 0xe21, 0xe42, 0xe19, 0xe21,
0xe34, 0xe19, 0xe35, 0xe40, 0xe21, 0xe23, 0xe35, 0xe14, 0xe32, 0xe40, 0xe21, 0xe17,
0xe25, 0xe32, 0xe01, 0xe32, 0xe15, 0xe25, 0xe30, 0xe40, 0xe21, 0xe47, 0xe01, 0xe0b,
0xe34, 0xe42, 0xe01, 0xe0b, 0xe34, 0xe15, 0xe35, 0xe21, 0xe35, 0xe40, 0xe01, 0xe2d,
0xe25, 0xe07, 0xe21, 0xe2d, 0xe07, 0xe15, 0xe31, 0xe19, 0xe21, 0xe2d, 0xe19, 0xe40,
0xe15, 0xe23, 0xe4c, 0xe40, 0xe23, 0xe22, 0xe4c, 0xe21, 0xe2d, 0xe19, 0xe40, 0xe15,
0xe27, 0xe34, 0xe40, 0xe14, 0xe42, 0xe2d, 0xe21, 0xe2d, 0xe19, 0xe40, 0xe0b, 0xe2d,
0xe23, 0xe4c, 0xe23, 0xe31, 0xe15, 0xe41, 0xe19, 0xe2a, 0xe0b, 0xe2d, 0xe19, 0xe34,
0xe27, 0xe22, 0xe2d, 0xe23, 0xe4c, 0xe01, 0xe19, 0xe2d, 0xe21, 0xe42, 0xe19, 0xe23,
0xe2d, 0xe19, 0xe2e, 0xe32, 0xe42, 0xe1a, 0xe25, 0xe32, 0xe2b, 0xe4c, 0x2c, 0x20,
0xe19, 0xe2d, 0xe23, 0xe4c, 0xe17, 0xe14, 0xe32, 0xe42, 0xe04, 0xe15, 0xe32, 0xe40,
0xe0b, 0xe19, 0xe40, 0xe15, 0xe2d, 0xe23, 0xe4c, 0x2c, 0x20, 0xe19, 0xe2d, 0xe23,
0xe4c, 0xe17, 0xe14, 0xe32, 0xe42, 0xe04, 0xe15, 0xe32, 0xe19, 0xe34, 0xe27, 0xe40,
0xe0b, 0xe40, 0xe25, 0xe21, 0x2c, 0x20, 0xe19, 0xe2d, 0xe23, 0xe4c, 0xe17, 0xe14,
0xe32, 0xe42, 0xe04, 0xe15, 0xe32, 0xe01, 0xe2d, 0xe14, 0xe41, 0xe18, 0xe1a, 0xe42,
0xe2d, 0xe08, 0xe34, 0xe19, 0xe32, 0xe01, 0xe32, 0xe1b, 0xe32, 0xe19, 0xe32, 0xe21,
0xe32, 0xe1b, 0xe32, 0xe23, 0xe32, 0xe21, 0xe32, 0xe23, 0xe34, 0xe42, 0xe1a, 0xe1f,
0xe34, 0xe19, 0xe34, 0xe01, 0xe0b, 0xe4c, 0xe1b, 0xe2d, 0xe23, 0xe4c, 0xe42, 0xe15,
0xe41, 0xe1b, 0xe23, 0xe07, 0xe0b, 0xe4c, 0xe1e, 0xe2d, 0xe23, 0xe4c, 0xe17, 0xe2d,
0xe2d, 0xe1f, 0xe2a, 0xe40, 0xe1b, 0xe19, 0xe1b, 0xe2d, 0xe23, 0xe4c, 0xe15, 0xe39,
0xe40, 0xe27, 0xe25, 0xe42, 0xe22, 0xe40, 0xe1b, 0xe2d, 0xe42, 0xe15, 0xe23, 0xe34,
0xe42, 0xe01, 0xe1b, 0xe38, 0xe19, 0xe15, 0xe32, 0xe2d, 0xe32, 0xe40, 0xe23, 0xe19,
0xe31, 0xe2a, 0xe41, 0xe23, 0xe07, 0xe01, 0xe34, 0xe19, 0xe2d, 0xe34, 0xe19, 0xe40,
0xe25, 0xe47, 0xe15, 0xe40, 0xe23, 0xe0b, 0xe35, 0xe40, 0xe1f, 0xe23, 0xe34, 0xe44,
0xe08, 0xe19, 0xe32, 0xe40, 0xe23, 0xe42, 0xe0b, 0xe25, 0xe39, 0xe17, 0xe23, 0xe35,
0xe42, 0xe2d, 0xe1a, 0xe23, 0xe31, 0xe07, 0xe42, 0xe01, 0xe0b, 0xe31, 0xe19, 0xe15,
0xe32, 0xe40, 0xe23, 0xe21, 0xe0b, 0xe31, 0xe19, 0xe15, 0xe34, 0xe2d, 0xe32, 0xe42,
0xe01, 0xe0b, 0xe32, 0xe19, 0xe42, 0xe15, 0x20, 0xe42, 0xe14, 0xe21, 0xe34, 0xe07,
0xe42, 0xe01, 0xe40, 0xe0b, 0xe32, 0xe40, 0xe1b, 0xe32, 0xe25, 0xe39, 0xe2a, 0xe01,
0xe2d, 0xe40, 0xe23, 0xe2a, 0xe44, 0xe1a, 0xe0b, 0xe31, 0xe19, 0xe14, 0xe4c, 0xe0b,
0xe34, 0xe15, 0xe01, 0xe32, 0xe40, 0xe0b, 0xe19, 0xe15, 0xe4c, 0xe1a, 0xe32, 0xe23,
0xe4c, 0xe40, 0xe18, 0xe40, 0xe25, 0xe21, 0xe35, 0xe40, 0xe0b, 0xe19, 0xe15, 0xe4c,
0xe08, 0xe2d, 0xe19, 0xe2a, 0xe4c, 0xe40, 0xe0b, 0xe19, 0xe15, 0xe4c, 0xe04, 0xe34,
0xe15, 0xe2a, 0xe4c, 0xe40, 0xe0b, 0xe19, 0xe15, 0xe4c, 0xe25, 0xe39, 0xe40, 0xe0b,
0xe35, 0xe22, 0xe40, 0xe0b, 0xe19, 0xe15, 0xe4c, 0xe42, 0xe18, 0xe21, 0xe31, 0xe2a,
0xe40, 0xe0b, 0xe19, 0xe15, 0xe4c, 0xe27, 0xe34, 0xe19, 0xe40, 0xe0b, 0xe19, 0xe15,
0xe4c, 0xe2a, 0xe27, 0xe34, 0xe1f, 0xe15, 0xe4c, 0xe40, 0xe04, 0xe2d, 0xe23, 0xe4c,
0xe40, 0xe23, 0xe19, 0xe15, 0xe4c, 0xe40, 0xe15, 0xe01, 0xe39, 0xe0b, 0xe34, 0xe01,
0xe31, 0xe25, 0xe1b, 0xe32, 0xe17, 0xe39, 0xe40, 0xe25, 0xe17, 0xe34, 0xe2e, 0xe31,
0xe27, 0xe19, 0xe32, 0xe42, 0xe17, 0xe23, 0xe2d, 0xe19, 0xe42, 0xe15, 0xe15, 0xe2d,
0xe23, 0xe4c, 0xe42, 0xe15, 0xe25, 0xe32, 0xe41, 0xe27, 0xe19, 0xe04, 0xe39, 0xe40,
0xe27, 0xe2d, 0xe23, 0xe4c, 0xe44, 0xe27, 0xe15, 0xe4c, 0xe2e, 0xe2d, 0xe23, 0xe4c,
0xe2a, 0xe27, 0xe34, 0xe19, 0xe19, 0xe34, 0xe40, 0xe1e, 0xe01, 0xe22, 0xe32, 0xe01,
0xe39, 0xe17, 0xe31, 0xe15, 0xe40, 0xe04, 0xe0b, 0xe35, 0xe22, 0xe4c, 0xe40, 0xe14,
0xe27, 0xe34, 0xe2a, 0xe14, 0xe39, 0xe21, 0xe2d, 0xe07, 0xe15, 0xe4c, 0xe14, 0xe39,
0xe23, 0xe4c, 0xe27, 0xe34, 0xe25, 0xe25, 0xe4c, 0xe41, 0xe21, 0xe04, 0xe04, 0xe27,
0xe2d, 0xe23, 0xe35, 0xe21, 0xe2d, 0xe27, 0xe4c, 0xe2a, 0xe31, 0xe19, 0xe41, 0xe21,
0xe04, 0xe21, 0xe31, 0xe27, 0xe42, 0xe14, 0xe1e, 0xe32, 0xe25, 0xe4c, 0xe40, 0xe21,
0xe2d, 0xe23, 0xe4c, 0xe42, 0xe23, 0xe18, 0xe35, 0xe23, 0xe32, 0xe44, 0xe0b, 0xe42,
0xe22, 0xe27, 0xe32, 0xe42, 0xe17, 0xe23, 0xe25, 0xe27, 0xe2d, 0xe2a, 0xe15, 0xe2d,
0xe04, 0xe25, 0xe2d, 0xe07, 0xe40, 0xe22, 0xe35, 0xe22, 0xe23, 0xe4c, 0xe40, 0xe1a,
0xe35, 0xe22, 0xe19, 0xe40, 0xe2d, 0xe40, 0xe14, 0xe19, 0xe2d, 0xe31, 0xe25, 0xe21,
0xe32, 0xe15, 0xe35, 0xe2d, 0xe31, 0xe21, 0xe21, 0xe32, 0xe19, 0xe2d, 0xe32, 0xe19,
0xe32, 0xe14, 0xe35, 0xe23, 0xe4c, 0xe2d, 0xe31, 0xe04, 0xe15, 0xe32, 0xe2d, 0xe39,
0xe2d, 0xe31, 0xe04, 0xe42, 0xe17, 0xe1a, 0xe35, 0xe2d, 0xe32, 0xe0a, 0xe01, 0xe32,
0xe1a, 0xe31, 0xe15, 0xe2d, 0xe17, 0xe35, 0xe23, 0xe32, 0xe27, 0xe41, 0xe1a, 0xe01,
0xe41, 0xe14, 0xe14, 0xe1a, 0xe32, 0xe2b, 0xe4c, 0xe40, 0xe23, 0xe19, 0xe1a, 0xe32,
0xe01, 0xe39, 0xe01, 0xe23, 0xe38, 0xe07, 0xe40, 0xe17, 0xe1e, 0xe1a, 0xe32, 0xe23,
0xe4c, 0xe40, 0xe19, 0xe32, 0xe27, 0xe4c, 0xe40, 0xe1a, 0xe23, 0xe38, 0xe15, 0xe1a,
0xe34, 0xe0a, 0xe40, 0xe04, 0xe01, 0xe1a, 0xe23, 0xe39, 0xe44, 0xe19, 0xe0a, 0xe34,
0xe15, 0xe32, 0xe42, 0xe04, 0xe25, 0xe31, 0xe21, 0xe42, 0xe1a, 0xe14, 0xe32, 0xe21,
0xe31, 0xe2a, 0xe01, 0xe31, 0xe2a, 0xe14, 0xe34, 0xe25, 0xe35, 0xe14, 0xe39, 0xe44,
0xe1a, 0xe14, 0xe39, 0xe0a, 0xe32, 0xe19, 0xe40, 0xe1a, 0xe41, 0xe1f, 0xe21, 0xe32,
0xe01, 0xe38, 0xe2a, 0xe15, 0xe32, 0xe01, 0xe32, 0xe0b, 0xe32, 0xe40, 0xe2e, 0xe1a,
0xe23, 0xe2d, 0xe19, 0xe19, 0xe04, 0xe23, 0xe42, 0xe2e, 0xe08, 0xe34, 0xe21, 0xe34,
0xe19, 0xe2b, 0xe4c, 0xe2e, 0xe48, 0xe2d, 0xe07, 0xe01, 0xe07, 0xe2e, 0xe2d, 0xe1f,
0xe14, 0xe4c, 0xe2d, 0xe35, 0xe23, 0xe4c, 0xe04, 0xe38, 0xe15, 0xe2a, 0xe04, 0xe4c,
0xe08, 0xe32, 0xe01, 0xe32, 0xe23, 0xe4c, 0xe15, 0xe32, 0xe08, 0xe32, 0xe22, 0xe32,
0xe1b, 0xe38, 0xe23, 0xe30, 0xe40, 0xe22, 0xe23, 0xe39, 0xe0b, 0xe32, 0xe40, 0xe25,
0xe21, 0xe04, 0xe32, 0xe1a, 0xe39, 0xe25, 0xe04, 0xe32, 0xe21, 0xe0a, 0xe31, 0xe15,
0xe01, 0xe32, 0xe01, 0xe32, 0xe23, 0xe32, 0xe08, 0xe35, 0xe01, 0xe32, 0xe15, 0xe21,
0xe31, 0xe19, 0xe14, 0xe38, 0xe2e, 0xe31, 0xe19, 0xe14, 0xe37, 0xe22, 0xe01, 0xe32,
0xe42, 0xe01, 0xe25, 0xe01, 0xe32, 0xe15, 0xe32, 0xe04, 0xe23, 0xe31, 0xe2a, 0xe42,
0xe19, 0xe22, 0xe32, 0xe23, 0xe4c, 0xe2a, 0xe01, 0xe4c, 0xe01, 0xe31, 0xe27, 0xe25,
0xe32, 0xe25, 0xe31, 0xe21, 0xe40, 0xe1b, 0xe2d, 0xe23, 0xe4c, 0xe01, 0xe39, 0xe0a,
0xe34, 0xe07, 0xe04, 0xe39, 0xe40, 0xe27, 0xe15, 0xe21, 0xe32, 0xe40, 0xe01, 0xe4a,
0xe32, 0xe21, 0xe32, 0xe01, 0xe32, 0xe14, 0xe32, 0xe19, 0xe21, 0xe32, 0xe01, 0xe31,
0xe2a, 0xe0b, 0xe32, 0xe23, 0xe4c, 0xe21, 0xe30, 0xe19, 0xe34, 0xe25, 0xe32, 0xe21,
0xe31, 0xe2a, 0xe01, 0xe31, 0xe15, 0xe19, 0xe34, 0xe42, 0xe04, 0xe40, 0xe0b, 0xe35,
0xe22, 0xe42, 0xe19, 0xe42, 0xe27, 0xe04, 0xe38, 0xe0b, 0xe40, 0xe19, 0xe15, 0xe2a,
0xe04, 0xe4c, 0xe42, 0xe19, 0xe42, 0xe27, 0xe0b, 0xe34, 0xe1a, 0xe34, 0xe23, 0xe4c,
0xe2a, 0xe01, 0xe4c, 0xe42, 0xe2d, 0xe21, 0xe2a, 0xe01, 0xe4c, 0xe2d, 0xe2d, 0xe23,
0xe31, 0xe25, 0xe1e, 0xe19, 0xe21, 0xe40, 0xe1b, 0xe0d, 0xe1e, 0xe2d, 0xe19, 0xe40,
0xe17, 0xe35, 0xe22, 0xe19, 0xe31, 0xe01, 0xe40, 0xe1b, 0xe35, 0xe22, 0xe07, 0xe22,
0xe32, 0xe07, 0xe01, 0xe32, 0xe15, 0xe32, 0xe23, 0xe4c, 0xe04, 0xe2d, 0xe2a, 0xe15,
0xe32, 0xe40, 0xe19, 0xe22, 0xe4c, 0xe44, 0xe04, 0xe0b, 0xe35, 0xe25, 0xe2d, 0xe23,
0xe4c, 0xe14, 0xe32, 0xe23, 0xe34, 0xe22, 0xe32, 0xe23, 0xe4c, 0xe14, 0xe0b, 0xe32,
0xe04, 0xe32, 0xe25, 0xe34, 0xe19, 0xe0b, 0xe32, 0xe21, 0xe32, 0xe23, 0xe4c, 0xe01,
0xe32, 0xe19, 0xe14, 0xe4c, 0xe40, 0xe0b, 0xe35, 0xe48, 0xe22, 0xe07, 0xe44, 0xe2e,
0xe49, 0xe2a, 0xe34, 0xe07, 0xe04, 0xe42, 0xe1b, 0xe23, 0xe4c, 0xe0b, 0xe40, 0xe23,
0xe14, 0xe40, 0xe19, 0xe04, 0xe42, 0xe04, 0xe25, 0xe34, 0xe21, 0xe2a, 0xe01, 0xe4c,
0xe44, 0xe17, 0xe40, 0xe1b, 0xe17, 0xe32, 0xe0a, 0xe40, 0xe04, 0xe19, 0xe15, 0xe4c,
0xe17, 0xe1a, 0xe34, 0xe25, 0xe34, 0xe0b, 0xe34, 0xe40, 0xe15, 0xe2b, 0xe30, 0xe23,
0xe32, 0xe19, 0xe17, 0xe34, 0xe21, 0xe1e, 0xe39, 0xe42, 0xe15, 0xe40, 0xe01, 0xe35,
0xe22, 0xe27, 0xe15, 0xe2d, 0xe21, 0xe2a, 0xe04, 0xe4c, 0xe2d, 0xe39, 0xe25, 0xe32,
0xe19, 0xe1a, 0xe32, 0xe15, 0xe2d, 0xe23, 0xe4c, 0xe2d, 0xe38, 0xe23, 0xe38, 0xe21,
0xe0a, 0xe35, 0xe2d, 0xe38, 0xe2a, 0xe15, 0xe4c, 0xe40, 0xe19, 0xe23, 0xe32, 0xe40,
0xe27, 0xe35, 0xe22, 0xe07, 0xe08, 0xe31, 0xe19, 0xe17, 0xe19, 0xe4c, 0xe27, 0xe25,
0xe32, 0xe14, 0xe34, 0xe42, 0xe27, 0xe2a, 0xe15, 0xe47, 0xe2d, 0xe01, 0xe22, 0xe32,
0xe04, 0xe38, 0xe15, 0xe2a, 0xe04, 0xe4c, 0xe22, 0xe48, 0xe32, 0xe07, 0xe01, 0xe38,
0xe49, 0xe07, 0xe22, 0xe35, 0xe04, 0xe32, 0xe40, 0xe15, 0xe2d, 0xe23, 0xe34, 0xe19,
0xe40, 0xe1a, 0xe34, 0xe23, 0xe4c, 0xe01, 0xe40, 0xe22, 0xe40, 0xe23, 0xe27, 0xe32,
0xe19, 0xe2d, 0xe30, 0xe42, 0xe0b, 0xe23, 0xe4c, 0xe2a, 0xe40, 0xe1a, 0xe2d, 0xe23,
0xe4c, 0xe21, 0xe34, 0xe27, 0xe14, 0xe32, 0xe04, 0xe30, 0xe40, 0xe19, 0xe23, 0xe35,
0xe40, 0xe04, 0xe1b, 0xe40, 0xe27, 0xe34, 0xe23, 0xe4c, 0xe14, 0xe41, 0xe1f, 0xe42,
0xe23, 0xe21, 0xe32, 0xe40, 0xe14, 0xe23, 0xe32, 0xe40, 0xe23, 0xe04, 0xe22, 0xe32,
0xe27, 0xe34, 0xe01, 0xe40, 0xe0b, 0xe32, 0xe17, 0xe4c, 0x20, 0xe08, 0xe2d, 0xe23,
0xe4c, 0xe40, 0xe08, 0xe35, 0xe22, 0xe40, 0xe0b, 0xe19, 0xe15, 0xe4c, 0xe40, 0xe2e,
0xe40, 0xe25, 0xe19, 0xe32, 0xe2a, 0xe41, 0xe15, 0xe19, 0xe25, 0xe35, 0xe22, 0xe4c,
0xe41, 0xe2d, 0xe14, 0xe34, 0xe40, 0xe25, 0xe14, 0xe1a, 0xe23, 0xe34, 0xe2a, 0xe40,
0xe1a, 0xe19, 0xe42, 0xe1a, 0xe23, 0xe01, 0xe40, 0xe04, 0xe19, 0xe2e, 0xe34, 0xe25,
0xe25, 0xe4c, 0xe14, 0xe32, 0xe23, 0xe4c, 0xe27, 0xe34, 0xe19, 0xe22, 0xe39, 0xe04,
0xe25, 0xe32, 0xe42, 0xe2e, 0xe1a, 0xe32, 0xe23, 0xe4c, 0xe15, 0xe25, 0xe34, 0xe19,
0xe14, 0xe35, 0xe41, 0xe21, 0xe19, 0xe25, 0xe2d, 0xe23, 0xe4c, 0xe14, 0xe42, 0xe2e,
0xe27, 0xe4c, 0xe40, 0xe21, 0xe25, 0xe40, 0xe1a, 0xe34, 0xe23, 0xe4c, 0xe19, 0xe40,
0xe1e, 0xe34, 0xe23, 0xe4c, 0xe17, 0xe0b, 0xe34, 0xe14, 0xe19, 0xe35, 0xe22, 0xe4c,
0xe2d, 0xe31, 0xe21, 0xe2a, 0xe40, 0xe15, 0xe2d, 0xe14, 0xe31, 0xe21, 0xe2d, 0xe31,
0xe19, 0xe14, 0xe2d, 0xe23, 0xe4c, 0xe23, 0xe32, 0xe41, 0xe2d, 0xe2a, 0xe15, 0xe23,
0xe32, 0xe04, 0xe32, 0xe19, 0xe40, 0xe2d, 0xe40, 0xe18, 0xe19, 0xe2a, 0xe4c, 0xe40,
0xe1a, 0xe25, 0xe40, 0xe01, 0xe23, 0xe14, 0xe40, 0xe1a, 0xe2d, 0xe23, 0xe4c, 0xe25,
0xe34, 0xe19, 0xe1a, 0xe23, 0xe32, 0xe15, 0xe34, 0xe2a, 0xe25, 0xe32, 0xe27, 0xe32,
0xe1a, 0xe23, 0xe31, 0xe2a, 0xe40, 0xe0b, 0xe25, 0xe2a, 0xe4c, 0xe1a, 0xe39, 0xe04,
0xe32, 0xe40, 0xe23, 0xe2a, 0xe1a, 0xe39, 0xe14, 0xe32, 0xe40, 0xe1b, 0xe2a, 0xe1a,
0xe38, 0xe2a, 0xe0b, 0xe34, 0xe07, 0xe40, 0xe07, 0xe47, 0xe19, 0xe04, 0xe35, 0xe0a,
0xe35, 0xe40, 0xe19, 0xe32, 0xe42, 0xe04, 0xe40, 0xe1b, 0xe19, 0xe40, 0xe2e, 0xe40,
0xe01, 0xe19, 0xe14, 0xe31, 0xe1a, 0xe1a, 0xe25, 0xe34, 0xe19, 0xe22, 0xe34, 0xe1a,
0xe23, 0xe2d, 0xe25, 0xe15, 0xe32, 0xe23, 0xe4c, 0xe40, 0xe01, 0xe34, 0xe23, 0xe4c,
0xe19, 0xe0b, 0xe35, 0xe22, 0xe4c, 0xe40, 0xe2e, 0xe25, 0xe0b, 0xe34, 0xe07, 0xe01,
0xe34, 0xe40, 0xe01, 0xe32, 0xe30, 0xe41, 0xe21, 0xe19, 0xe2d, 0xe34, 0xe2a, 0xe15,
0xe31, 0xe19, 0xe1a, 0xe39, 0xe25, 0xe40, 0xe08, 0xe2d, 0xe23, 0xe4c, 0xe0b, 0xe35,
0xe22, 0xe4c, 0xe04, 0xe32, 0xe25, 0xe34, 0xe19, 0xe34, 0xe07, 0xe01, 0xe23, 0xe32,
0xe14, 0xe04, 0xe34, 0xe23, 0xe2d, 0xe1f, 0xe40, 0xe04, 0xe35, 0xe22, 0xe1f, 0xe25,
0xe34, 0xe2a, 0xe1a, 0xe2d, 0xe19, 0xe25, 0xe39, 0xe1a, 0xe25, 0xe34, 0xe22, 0xe32,
0xe19, 0xe32, 0xe25, 0xe2d, 0xe19, 0xe14, 0xe2d, 0xe19, 0xe25, 0xe31, 0xe01, 0xe40,
0xe0b, 0xe21, 0xe40, 0xe1a, 0xe34, 0xe23, 0xe4c, 0xe01, 0xe21, 0xe32, 0xe14, 0xe23,
0xe34, 0xe14, 0xe21, 0xe2d, 0xe25, 0xe15, 0xe32, 0xe21, 0xe32, 0xe23, 0xe35, 0xe2e,
0xe32, 0xe21, 0xe19, 0xe4c, 0xe21, 0xe34, 0xe19, 0xe2a, 0xe01, 0xe4c, 0xe42, 0xe21,
0xe19, 0xe32, 0xe42, 0xe01, 0xe21, 0xe2d, 0xe2a, 0xe42, 0xe01, 0xe2d, 0xe2d, 0xe2a,
0xe42, 0xe25, 0xe1b, 0xe32, 0xe23, 0xe35, 0xe2a, 0xe1e, 0xe2d, 0xe14, 0xe01, 0xe2d,
0xe23, 0xe35, 0xe15, 0xe0b, 0xe32, 0xe1b, 0xe23, 0xe32, 0xe01, 0xe0b, 0xe32, 0xe21,
0xe32, 0xe23, 0xe32, 0xe0b, 0xe32, 0xe19, 0xe21, 0xe32, 0xe23, 0xe35, 0xe42, 0xe19,
0xe0b, 0xe32, 0xe23, 0xe32, 0xe40, 0xe22, 0xe42, 0xe27, 0xe0b, 0xe32, 0xe23, 0xe32,
0xe17, 0xe2d, 0xe1f, 0xe0b, 0xe34, 0xe21, 0xe40, 0xe1f, 0xe2d, 0xe42, 0xe23, 0xe42,
0xe1b, 0xe25, 0xe2a, 0xe42, 0xe01, 0xe40, 0xe1b, 0xe35, 0xe22, 0xe42, 0xe0b, 0xe40,
0xe1f, 0xe35, 0xe22, 0xe2a, 0xe15, 0xe2d, 0xe01, 0xe42, 0xe2e, 0xe25, 0xe4c, 0xe21,
0xe17, 0xe32, 0xe25, 0xe25, 0xe34, 0xe19, 0xe19, 0xe4c, 0xe15, 0xe34, 0xe23, 0xe32,
0xe19, 0xe32, 0xe2d, 0xe30, 0xe25, 0xe34, 0xe22, 0xe32, 0xe19, 0xe2d, 0xe1f, 0xe27,
0xe32, 0xe14, 0xe38, 0xe0b, 0xe27, 0xe32, 0xe15, 0xe34, 0xe01, 0xe31, 0xe19, 0xe40,
0xe27, 0xe35, 0xe22, 0xe19, 0xe19, 0xe32, 0xe27, 0xe34, 0xe25, 0xe19, 0xe35, 0xe2d,
0xe38, 0xe2a, 0xe27, 0xe2d, 0xe25, 0xe42, 0xe01, 0xe01, 0xe23, 0xe32, 0xe14, 0xe27,
0xe2d, 0xe23, 0xe4c, 0xe0b, 0xe2d, 0xe0b, 0xe32, 0xe40, 0xe01, 0xe23, 0xe47, 0xe1a,
0xe0b, 0xe39, 0xe23, 0xe34, 0xe04, 0xe2d, 0xe31, 0xe19, 0xe15, 0xe32, 0xe19, 0xe32,
0xe19, 0xe32, 0xe23, 0xe34, 0xe42, 0xe27, 0xe0a, 0xe32, 0xe01, 0xe31, 0xe2a, 0xe04,
0xe23, 0xe34, 0xe2a, 0xe15, 0xe4c, 0xe21, 0xe32, 0xe2a, 0xe42, 0xe04, 0xe42, 0xe04,
0xe2a, 0xe42, 0xe04, 0xe42, 0xe21, 0xe42, 0xe23, 0xe41, 0xe01, 0xe23, 0xe4c, 0xe40,
0xe01, 0xe2d, 0xe25, 0xe2d, 0xe07, 0xe21, 0xe32, 0xe40, 0xe2d, 0xe21, 0xe31, 0xe25,
0xe14, 0xe35, 0xe1f, 0xe2a, 0xe4c, 0xe21, 0xe2d, 0xe23, 0xe34, 0xe40, 0xe0a, 0xe35,
0xe22, 0xe2a, 0xe21, 0xe32, 0xe42, 0xe22, 0xe40, 0xe15, 0xe40, 0xe23, 0xe2d, 0xe39,
0xe19, 0xe35, 0xe22, 0xe07, 0xe2d, 0xe32, 0xe1b, 0xe35, 0xe2d, 0xe32, 0xe42, 0xe2d,
0xe04, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c, 0xe1a, 0xe39, 0xe40, 0xe01, 0xe19, 0xe27,
0xe34, 0xe25, 0xe25, 0xe4c, 0xe41, 0xe0a, 0xe17, 0xe31, 0xe21, 0xe17, 0xe23, 0xe31,
0xe01, 0xe2d, 0xe35, 0xe2a, 0xe40, 0xe15, 0xe2d, 0xe23, 0xe4c, 0xe40, 0xe2d, 0xe1f,
0xe32, 0xe40, 0xe15, 0xe1f, 0xe32, 0xe40, 0xe04, 0xe32, 0xe42, 0xe1f, 0xe1f, 0xe34,
0xe08, 0xe34, 0xe1f, 0xe39, 0xe19, 0xe30, 0xe1f, 0xe39, 0xe15, 0xe35, 0xe01, 0xe32,
0xe25, 0xe32, 0xe1b, 0xe32, 0xe42, 0xe01, 0xe2a, 0xe41, 0xe01, 0xe21, 0xe40, 0xe1a,
0xe35, 0xe22, 0xe23, 0xe4c, 0xe01, 0xe31, 0xe27, 0xe14, 0xe31, 0xe25, 0xe04, 0xe32,
0xe19, 0xe31, 0xe25, 0xe01, 0xe27, 0xe21, 0xe42, 0xe2e, 0xe42, 0xe19, 0xe25, 0xe39,
0xe25, 0xe39, 0xe40, 0xe2d, 0xe19, 0xe40, 0xe14, 0xe2d, 0xe23, 0xe4c, 0xe40, 0xe1a,
0xe2d, 0xe23, 0xe35, 0xe04, 0xe34, 0xe23, 0xe34, 0xe17, 0xe34, 0xe21, 0xe32, 0xe15,
0xe35, 0xe04, 0xe2d, 0xe2a, 0xe44, 0xe23, 0xe04, 0xe27, 0xe32, 0xe08, 0xe32, 0xe40,
0xe25, 0xe19, 0xe21, 0xe32, 0xe08, 0xe39, 0xe42, 0xe23, 0xe21, 0xe32, 0xe23, 0xe4c,
0xe40, 0xe04, 0xe0b, 0xe31, 0xe2a, 0xe21, 0xe34, 0xe14, 0xe40, 0xe27, 0xe22, 0xe4c,
0xe19, 0xe32, 0xe2d, 0xe39, 0xe23, 0xe39, 0xe19, 0xe35, 0xe2d, 0xe39, 0xe40, 0xe2d,
0xe19, 0xe2d, 0xe23, 0xe4c, 0xe1f, 0xe2d, 0xe25, 0xe4c, 0xe01, 0xe19, 0xe39, 0xe40,
0xe21, 0xe2d, 0xe32, 0xe1b, 0xe32, 0xe42, 0xe01, 0x20, 0xe1b, 0xe32, 0xe42, 0xe01,
0xe1b, 0xe32, 0xe40, 0xe25, 0xe32, 0xe1e, 0xe34, 0xe15, 0xe41, 0xe04, 0xe23, 0xe4c,
0xe19, 0xe42, 0xe1b, 0xe19, 0xe32, 0xe40, 0xe1b, 0xe1e, 0xe2d, 0xe23, 0xe4c, 0xe15,
0xe21, 0xe2d, 0xe23, 0xe4c, 0xe2a, 0xe1a, 0xe35, 0xe23, 0xe32, 0xe42, 0xe23, 0xe15,
0xe2d, 0xe07, 0xe01, 0xe32, 0xe44, 0xe0b, 0xe1b, 0xe31, 0xe19, 0xe15, 0xe32, 0xe2e,
0xe35, 0xe15, 0xe35, 0xe15, 0xe32, 0xe23, 0xe30, 0xe27, 0xe32, 0xe15, 0xe2d, 0xe07,
0xe01, 0xe32, 0xe15, 0xe32, 0xe1b, 0xe39, 0xe40, 0xe27, 0xe01, 0xe27, 0xe32, 0xe25,
0xe25, 0xe34, 0xe2a, 0x12a3, 0x1262, 0x1303, 0x1295, 0x12a3, 0x12ad, 0x122b, 0x12a3, 0x12f2,
0x1235, 0x20, 0x12a3, 0x1260, 0x1263, 0x12a3, 0x120d, 0x1300, 0x122d, 0x1235, 0x12a3, 0x1235,
0x1218, 0x122b, 0x1263, 0x1295, 0x130a, 0x1262, 0x1233, 0x12cd, 0x1261, 0x1301, 0x121d, 0x1261,
0x122b, 0x1234, 0x12cd, 0x1273, 0x12ae, 0x1293, 0x12ad, 0x122a, 0x12f3, 0x122d, 0x20, 0x12a4,
0x1235, 0x20, 0x1233, 0x120b, 0x121d, 0x12f1, 0x12cb, 0x120b, 0x12a4, 0x120d, 0x20, 0x12a3,
0x12e9, 0x1295, 0x1203, 0x122b, 0x1228, 0x120a, 0x1265, 0x1228, 0x126a, 0x120d, 0x120e, 0x1218,
0x1209, 0x12cb, 0x1295, 0x12f3, 0x121b, 0x120b, 0x1266, 0x121b, 0x1351, 0x1276, 0x121b, 0x1230,
0x1229, 0x121d, 0x1263, 0x1263, 0x1290, 0x121e, 0x1253, 0x12f5, 0x123e, 0x121e, 0x1295, 0x122e,
0x126d, 0x12eb, 0x1295, 0x1303, 0x1218, 0x1293, 0x1295, 0x12eb, 0x1218, 0x12ed, 0x1295, 0x12cb,
0x12ad, 0x123e, 0x1275, 0x12cb, 0x130b, 0x12f1, 0x1309, 0x1356, 0x122d, 0x1276, 0x20, 0x1296,
0x126e, 0x1233, 0x12a6, 0x20, 0x1276, 0x1218, 0x1271, 0x1292, 0x1235, 0x12a3, 0x12f3, 0x12ad,
0x12a3, 0x1295, 0x12ae, 0x1228, 0x1305, 0x12a3, 0x1295, 0x1312, 0x120b, 0x12a3, 0x1295, 0x1272,
0x1313, 0x12a3, 0x122b, 0x1313, 0x12ed, 0x1293, 0x1265, 0x12c8, 0x1296, 0x1235, 0x20, 0x12a3,
0x12ed, 0x1228, 0x1235, 0x1201, 0x1201, 0x12ed, 0x120b, 0x20, 0x122d, 0x12ee, 0x1203, 0x1218,
0x1295, 0x12f6, 0x12db, 0x122d, 0x12ee, 0x20, 0x130b, 0x12e8, 0x130e, 0x1235, 0x1233, 0x1295,
0x20, 0x1205, 0x12cb, 0x1295, 0x1233, 0x1295, 0x20, 0x120d, 0x12ca, 0x1235, 0x12a1, 0x123d,
0x12cb, 0x12eb, 0x12a3, 0x1229, 0x1263, 0x12a3, 0x1231, 0x1295, 0x1235, 0x12ee, 0x1295, 0x12a3,
0x1272, 0x12ae, 0x12ab, 0x1295, 0x1263, 0x1205, 0x12eb, 0x1263, 0x12a5, 0x12eb, 0x20, 0x12f0,
0x20, 0x1263, 0x1295, 0x12f0, 0x122b, 0x1235, 0x1263, 0x122d, 0x1263, 0x12f6, 0x1235, 0x1260,
0x1208, 0x121d, 0x1260, 0x120a, 0x12dd, 0x1265, 0x120b, 0x1295, 0x12ad, 0x2d, 0x1233, 0x1265,
0x120e, 0x1295, 0x1266, 0x12cb, 0x20, 0x126a, 0x1235, 0x1273, 0x1266, 0x12ed, 0x12da, 0x12ab,
0x121d, 0x1265, 0x122a, 0x1305, 0x20, 0x1260, 0x12ed, 0x12ab, 0x121d, 0x1356, 0x20, 0x130d,
0x122b, 0x1295, 0x12f0, 0x12ab, 0x12e8, 0x1295, 0x127a, 0x12cb, 0x12cb, 0x1232, 0x12e9, 0x12f3,
0x12f5, 0x20, 0x1301, 0x12cb, 0x1228, 0x12dd, 0x12ae, 0x1235, 0x1273, 0x20, 0x122a, 0x12ab,
0x12a9, 0x12eb, 0x1263, 0x12a9, 0x122b, 0x1233, 0x12cd, 0x12f3, 0x12cd, 0x1230, 0x1295, 0x20,
0x12ad, 0x122a, 0x12ad, 0x12f0, 0x1295, 0x1268, 0x122d, 0x12f6, 0x121a, 0x1292, 0x12ab, 0x12a4,
0x12ed, 0x1229, 0x1294, 0x1352, 0x12a4, 0x120d, 0x20, 0x1233, 0x120d, 0x126b, 0x12f6, 0x122d,
0x134e, 0x122d, 0x1275, 0x20, 0x1290, 0x120d, 0x1230, 0x1295, 0x134e, 0x122d, 0x1273, 0x1208,
0x12db, 0x130d, 0x120c, 0x1235, 0x20, 0x1260, 0x12ed, 0x1309, 0x12dd, 0x20, 0x1260, 0x12ed,
0x130d, 0x122b, 0x1295, 0x12f5, 0x20, 0x1271, 0x122d, 0x12ad, 0x130d, 0x1228, 0x1293, 0x12f3,
0x1313, 0x12f0, 0x1209, 0x1355, 0x1313, 0x12eb, 0x12aa, 0x120d, 0x12a4, 0x122d, 0x121e, 0x1235,
0x12ee, 0x12a2, 0x1295, 0x12f5, 0x12eb, 0x1293, 0x1356, 0x120a, 0x1235, 0x1296, 0x12ad, 0x1235,
0x1363, 0x20, 0x12a2, 0x1295, 0x12f5, 0x12eb, 0x1293, 0x121b, 0x1228, 0x1295, 0x130e, 0x1363,
0x20, 0x12a2, 0x1295, 0x12f5, 0x12eb, 0x1293, 0x1352, 0x1270, 0x122d, 0x1235, 0x1260, 0x122d,
0x130d, 0x1363, 0x20, 0x12a2, 0x1295, 0x12f5, 0x12eb, 0x1293, 0x1270, 0x120d, 0x20, 0x1232,
0x1272, 0x1363, 0x20, 0x12a2, 0x1295, 0x12f5, 0x12eb, 0x1293, 0x126a, 0x126a, 0x1363, 0x20,
0x12a2, 0x1295, 0x12f5, 0x12eb, 0x1293, 0x126a, 0x1295, 0x1230, 0x1295, 0x1235, 0x1363, 0x20,
0x12a2, 0x1295, 0x12f5, 0x12eb, 0x1293, 0x12ca, 0x1293, 0x121b, 0x12ad, 0x1363, 0x20, 0x12a2,
0x1295, 0x12f5, 0x12eb, 0x1293, 0x12a2, 0x1243, 0x120d, 0x12ca, 0x1275, 0x1301, 0x1290, 0x12cd,
0x120d, 0x12ca, 0x126a, 0x120d, 0x121e, 0x1295, 0x1272, 0x1278, 0x120e, 0x1363, 0x20, 0x12a8,
0x1295, 0x1273, 0x12aa, 0x12ad, 0x122b, 0x1208, 0x1295, 0x12f3, 0x12ed, 0x12ad, 0x120e, 0x1235,
0x20, 0x12a3, 0x1295, 0x1300, 0x1208, 0x1235, 0x1208, 0x12c8, 0x122d, 0x20, 0x1355, 0x122a,
0x1295, 0x1230, 0x1235, 0x20, 0x12b3, 0x122d, 0x1270, 0x122d, 0x121b, 0x1230, 0x12ee, 0x121b,
0x1293, 0x1313, 0x121b, 0x1293, 0x12cd, 0x1235, 0x121b, 0x122a, 0x130e, 0x1275, 0x1218, 0x122a,
0x12f3, 0x12a8, 0x1270, 0x121b, 0x20, 0x121c, 0x12ad, 0x1232, 0x12ae, 0x121a, 0x12a8, 0x120e,
0x1295, 0x121e, 0x1295, 0x12ad, 0x1270, 0x1295, 0x121e, 0x1295, 0x1270, 0x1228, 0x12ed, 0x121e,
0x1295, 0x1270, 0x126a, 0x12f0, 0x12ee, 0x121e, 0x1295, 0x1275, 0x1230, 0x122b, 0x1275, 0x1292,
0x12cd, 0x20, 0x12ee, 0x122d, 0x12ad, 0x1290, 0x12cd, 0x121d, 0x1296, 0x122e, 0x1295, 0x12eb,
0x1265, 0x12e9, 0x120b, 0x1363, 0x20, 0x1230, 0x121c, 0x1295, 0x20, 0x12f3, 0x12ae, 0x1273,
0x1230, 0x1295, 0x1270, 0x122d, 0x1363, 0x20, 0x1230, 0x121c, 0x1295, 0x20, 0x12f3, 0x12ae,
0x1273, 0x1292, 0x12cd, 0x20, 0x1233, 0x1208, 0x121d, 0x1363, 0x20, 0x1230, 0x121c, 0x1295,
0x20, 0x12f3, 0x12ae, 0x1273, 0x1291, 0x12ad, 0x12a6, 0x1302, 0x1293, 0x130b, 0x1356, 0x122d,
0x1275, 0x2d, 0x12a6, 0x2d, 0x1355, 0x122a, 0x1295, 0x1235, 0x1356, 0x122d, 0x1275, 0x20,
0x12a6, 0x134d, 0x20, 0x1235, 0x1354, 0x1295, 0x1356, 0x122d, 0x1276, 0x20, 0x1268, 0x120d,
0x12ee, 0x1356, 0x122d, 0x1276, 0x20, 0x122a, 0x12ae, 0x1351, 0x1295, 0x1273, 0x20, 0x12a3,
0x1228, 0x1293, 0x1235, 0x122b, 0x1295, 0x12aa, 0x1295, 0x20, 0x12a2, 0x1295, 0x1208, 0x1275,
0x1228, 0x1232, 0x1348, 0x1228, 0x12de, 0x1209, 0x1275, 0x122d, 0x12ee, 0x20, 0x1265, 0x122b,
0x1295, 0x12ae, 0x1233, 0x1295, 0x1273, 0x1228, 0x121d, 0x1233, 0x1295, 0x1275, 0x12eb, 0x130e,
0x1233, 0x12a6, 0x20, 0x1353, 0x12cd, 0x120e, 0x12a2, 0x1276, 0x1246, 0x122d, 0x1276, 0x121a,
0x1275, 0x1245, 0x12f1, 0x1235, 0x20, 0x1263, 0x122d, 0x1270, 0x1208, 0x121a, 0x1245, 0x12f1,
0x1235, 0x20, 0x12ee, 0x1203, 0x1295, 0x1235, 0x1245, 0x12f5, 0x1235, 0x1272, 0x20, 0x1209,
0x1235, 0x12eb, 0x1235, 0x12ca, 0x134d, 0x1275, 0x20, 0x12ab, 0x1228, 0x1295, 0x1275, 0x1270,
0x1309, 0x1232, 0x130b, 0x120d, 0x1353, 0x12d9, 0x120d, 0x1272, 0x1305, 0x12cb, 0x1293, 0x126b,
0x1295, 0x12a9, 0x1268, 0x122d, 0x12cb, 0x12ed, 0x1275, 0x1206, 0x12dd, 0x12ca, 0x1292, 0x1350,
0x130d, 0x12a8, 0x12ed, 0x12da, 0x12f0, 0x126a, 0x1235, 0x12f1, 0x121e, 0x1295, 0x1275, 0x20,
0x12f2, 0x12a1, 0x122d, 0x126a, 0x120d, 0x121b, 0x12b3, 0x122a, 0x121b, 0x12cd, 0x1230, 0x1295,
0x121b, 0x12ad, 0x1219, 0x122d, 0x12f6, 0x122e, 0x12d8, 0x122b, 0x120e, 0x1295, 0x130d, 0x12e8,
0x122d, 0x1263, 0x12e8, 0x1295, 0x12d3, 0x12f0, 0x1295, 0x12a3, 0x120d, 0x121b, 0x1272, 0x12d3,
0x121b, 0x1295, 0x12a3, 0x1293, 0x12f2, 0x122d, 0x12a3, 0x12ad, 0x1273, 0x12cd, 0x12a3, 0x12ad,
0x1276, 0x1260, 0x12a3, 0x123d, 0x130b, 0x1263, 0x1275, 0x12a3, 0x1272, 0x122b, 0x12cd, 0x1263,
0x1255, 0x12f3, 0x12f5, 0x1263, 0x1215, 0x122c, 0x1295, 0x1263, 0x1295, 0x130d, 0x12ae, 0x12ad,
0x1263, 0x122d, 0x1293, 0x12cd, 0x120d, 0x1260, 0x12ed, 0x1229, 0x1275, 0x1265, 0x1229, 0x1290,
0x12ed, 0x12f1, 0x123b, 0x1295, 0x1260, 0x1253, 0x12db, 0x12a8, 0x1270, 0x121b, 0x20, 0x1206,
0x20, 0x127a, 0x20, 0x121a, 0x1295, 0x12b3, 0x120b, 0x20, 0x1209, 0x121d, 0x1351, 0x122d,
0x121b, 0x12ab, 0x12cd, 0x1292, 0x12ae, 0x1235, 0x12eb, 0x1296, 0x126e, 0x12a9, 0x12dd, 0x1290,
0x1275, 0x1235, 0x12ad, 0x1296, 0x126e, 0x1232, 0x1262, 0x122d, 0x1235, 0x12ad, 0x1355, 0x1296,
0x121d, 0x20, 0x1350, 0x1295, 0x1356, 0x1295, 0x1275, 0x12eb, 0x1293, 0x12ad, 0x1250, 0x1320,
0x122d, 0x12aa, 0x12da, 0x120e, 0x122d, 0x12f3, 0x122d, 0x12eb, 0x12f5, 0x1233, 0x12ab, 0x120a,
0x1295, 0x1236, 0x12cd, 0x120d, 0x123b, 0x1295, 0x130d, 0x1203, 0x12ed, 0x1235, 0x122c, 0x12f5,
0x1290, 0x12ae, 0x120a, 0x121d, 0x1235, 0x12ad, 0x1273, 0x12ed, 0x1350, 0x12ed, 0x1273, 0x123d,
0x12a8, 0x1295, 0x1275, 0x1275, 0x1262, 0x120a, 0x1232, 0x1270, 0x1205, 0x122b, 0x1295, 0x1276,
0x12ad, 0x12ee, 0x12a1, 0x120b, 0x1295, 0x20, 0x1263, 0x1276, 0x122d, 0x126d, 0x12e8, 0x1295,
0x1275, 0x12eb, 0x1295, 0x12e8, 0x1228, 0x126b, 0x1295, 0x12a3, 0x12de, 0x1228, 0x1235, 0x1260,
0x122d, 0x1219, 0x12f3, 0x12ac, 0x1355, 0x20, 0x1268, 0x122d, 0x12f0, 0x12f0, 0x1234, 0x1273,
0x1275, 0x20, 0x134b, 0x122e, 0x121b, 0x12f0, 0x12ed, 0x122b, 0x1228, 0x12ed, 0x12ad, 0x12eb,
0x126a, 0x12ad, 0x12f0, 0x1261, 0x1265, 0x20, 0x1306, 0x122d, 0x1305, 0x12eb, 0x1245, 0x12f5,
0x1235, 0x1272, 0x20, 0x1204, 0x1208, 0x1293, 0x1235, 0x1273, 0x1295, 0x120a, 0x12a3, 0x12f0,
0x1208, 0x12ed, 0x12f5, 0x1265, 0x122a, 0x12dd, 0x1264, 0x1295, 0x12e9, 0x12ad, 0x120b, 0x120e,
0x122d, 0x12f5, 0x20, 0x1203, 0x12cd, 0x1218, 0x120d, 0x1260, 0x122d, 0x1295, 0x12a3, 0x121d,
0x1235, 0x1270, 0x122d, 0x12f3, 0x121d, 0x12a3, 0x1295, 0x12f6, 0x122b, 0x12a3, 0x1235, 0x1275,
0x122b, 0x12ab, 0x1295, 0x12a3, 0x1274, 0x1295, 0x1235, 0x1260, 0x120d, 0x130d, 0x122c, 0x12f5,
0x1265, 0x122b, 0x1235, 0x120d, 0x1235, 0x1261, 0x12ab, 0x1228, 0x1235, 0x1275, 0x1261, 0x12f3,
0x1350, 0x1235, 0x1275, 0x12aa, 0x123a, 0x1293, 0x12cd, 0x1308, 0x122d, 0x1295, 0x12da, 0x12a3,
0x12ed, 0x120d, 0x20, 0x12a6, 0x134d, 0x20, 0x121b, 0x1295, 0x1300, 0x122d, 0x12da, 0x12ad,
0x12e8, 0x126d, 0x120d, 0x12e9, 0x1265, 0x120d, 0x12eb, 0x1293, 0x120e, 0x1295, 0x12f0, 0x1295,
0x121b, 0x122a, 0x1203, 0x121d, 0x1295, 0x1356, 0x12f5, 0x130e, 0x122a, 0x133b, 0x1233, 0x122b,
0x12e8, 0x126e, 0x1236, 0x134d, 0x12eb, 0x1272, 0x122b, 0x1293, 0x126d, 0x12e8, 0x1293, 0x126a,
0x120d, 0x1295, 0x12e8, 0x1235, 0x126e, 0x120d, 0x130e, 0x130d, 0x122b, 0x12f5, 0x12cb, 0x122d,
0x1233, 0x12cd, 0x12db, 0x130d, 0x1228, 0x1265, 0x12a3, 0x1295, 0x1273, 0x1293, 0x1293, 0x122a,
0x126e, 0x12ad, 0x122a, 0x1235, 0x121b, 0x1235, 0x12a8, 0x122d, 0x1309, 0x1208, 0x1295, 0x121b,
0x120d, 0x12f2, 0x126d, 0x1235, 0x121b, 0x12cd, 0x122a, 0x1238, 0x1235, 0x121b, 0x12ee, 0x1275,
0x122d, 0x12e9, 0x1295, 0x12e8, 0x1295, 0x12a3, 0x1355, 0x12eb, 0x1261, 0x1308, 0x1295, 0x126a,
0x120d, 0x12f0, 0x1234, 0x1275, 0x20, 0x134b, 0x1232, 0x12ab, 0x12a4, 0x134b, 0x1274, 0x1349,
0x1293, 0x1349, 0x1272, 0x130b, 0x121d, 0x1265, 0x12e8, 0x122d, 0x1313, 0x12f3, 0x120d, 0x12ab,
0x1293, 0x120d, 0x1313, 0x121d, 0x12a4, 0x1295, 0x12f0, 0x122d, 0x1260, 0x122a, 0x121b, 0x1301,
0x122e, 0x121b, 0x122d, 0x12ac, 0x1233, 0x1235, 0x1291, 0x1218, 0x12eb, 0x1352, 0x1275, 0x12a8,
0x122d, 0x1295, 0x1356, 0x1295, 0x1350, 0x12ed, 0x1356, 0x122d, 0x1275, 0x20, 0x121e, 0x122d,
0x1235, 0x1262, 0x1233, 0x12ed, 0x1353, 0x1295, 0x12cc, 0x12ad, 0x4e, 0x69, 0x75, 0x20,
0x2bb, 0x49, 0x6f, 0x6b, 0x65, 0x48, 0x65, 0x70, 0x65, 0x6c, 0x6f, 0x6e,
0x69, 0x53, 0x65, 0x6c, 0x16b, 0x73, 0x61, 0x6c, 0x65, 0x6d, 0x61, 0x41,
0x74, 0x65, 0x6c, 0x61, 0x69, 0x74, 0x65, 0x50, 0x65, 0x6c, 0x69, 0x73,
0x69, 0x70, 0x61, 0x6e, 0x65, 0x4d, 0x65, 0x6c, 0x69, 0x70, 0x6f, 0x61,
0x6e, 0x65, 0x53, 0x65, 0x6e, 0x113, 0x2bb, 0x41, 0x74, 0x65, 0x6e, 0x69,
0x73, 0x69, 0x4c, 0x6f, 0x6e, 0x69, 0x74, 0x6f, 0x6e, 0x69, 0x4d, 0x6f,
0x73, 0x69, 0x6b, 0x6f, 0x75, 0x50, 0x61, 0x6c, 0x65, 0x73, 0x69, 0x4c,
0x6f, 0x6d, 0x61, 0x2bb, 0x41, 0x6f, 0x6b, 0x61, 0x6c, 0x61, 0x6e, 0x69,
0x50, 0x75, 0x6b, 0x61, 0x6e, 0x69, 0x76, 0x69, 0x6c, 0x61, 0x54, 0x16b,
0x6b, 0x65, 0x4c, 0x61, 0x70, 0x61, 0x6e, 0x75, 0x69, 0x46, 0x69, 0x73,
0x69, 0x4b, 0x61, 0x6d, 0x69, 0x70, 0x69, 0x113, 0x4b, 0x75, 0x61, 0x74,
0x101, 0x6b, 0x61, 0x6e, 0x61, 0x6c, 0x69, 0x4b, 0x75, 0x61, 0x6d, 0x69,
0x2bb, 0x45, 0x6e, 0x69, 0x74, 0x69, 0x70, 0x75, 0x6c, 0x12b, 0x4b, 0x69,
0x6c, 0x69, 0x73, 0x69, 0x6d, 0x61, 0x73, 0x69, 0x4b, 0x6f, 0x73, 0x69,
0x6c, 0x61, 0x65, 0x4b, 0x75, 0x61, 0x73, 0x61, 0x6c, 0x65, 0x6e, 0x69,
0x4d, 0x61, 0x73, 0x75, 0x6c, 0x6f, 0x4d, 0x61, 0x6c, 0x69, 0x6b, 0x75,
0x65, 0x73, 0x61, 0x4d, 0x69, 0x74, 0x69, 0x75, 0x61, 0x69, 0x4e, 0x61,
0x75, 0x6c, 0x75, 0x4e, 0x69, 0x75, 0x113, 0x4e, 0x6f, 0x61, 0x66, 0x14d,
0x6b, 0x69, 0x50, 0x61, 0x6e, 0x67, 0x6f, 0x70, 0x61, 0x6e, 0x67, 0x6f,
0x50, 0x69, 0x74, 0x69, 0x6b, 0x65, 0x6e, 0x69, 0x50, 0x6f, 0x6e, 0x61,
0x70, 0x113, 0x54, 0x61, 0x75, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x20, 0x4d,
0x6f, 0x6c, 0x65, 0x73, 0x69, 0x70, 0x69, 0x4c, 0x61, 0x6c, 0x6f, 0x74,
0x6f, 0x6e, 0x67, 0x61, 0x53, 0x61, 0x2bb, 0x69, 0x70, 0x61, 0x6e, 0x69,
0x54, 0x61, 0x68, 0x69, 0x73, 0x69, 0x54, 0x61, 0x6c, 0x61, 0x76, 0x61,
0x55, 0x65, 0x6b, 0x65, 0x2bb, 0x55, 0x76, 0x65, 0x61, 0x43, 0x65, 0x7a,
0x61, 0x79, 0x69, 0x72, 0x4b, 0x61, 0x68, 0x69, 0x72, 0x65, 0x44, 0x61,
0x72, 0xfc, 0x73, 0x73, 0x65, 0x6c, 0x61, 0x6d, 0x4c, 0x61, 0x79, 0x75,
0x6e, 0x43, 0x75, 0x62, 0x61, 0x4d, 0x6f, 0x67, 0x61, 0x64, 0x69, 0x15f,
0x75, 0x54, 0x75, 0x6e, 0x75, 0x73, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68,
0x2c, 0x20, 0x4b, 0x75, 0x7a, 0x65, 0x79, 0x20, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x4d, 0x65, 0x72, 0x6b, 0x65, 0x7a, 0x2c, 0x20, 0x4b, 0x75,
0x7a, 0x65, 0x79, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65,
0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x4b, 0x75, 0x7a,
0x65, 0x79, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x41, 0x6b, 0x74,
0x61, 0x76, 0x41, 0x15f, 0x6b, 0x61, 0x62, 0x61, 0x74, 0x41, 0x74, 0x131,
0x72, 0x61, 0x76, 0x42, 0x61, 0x11f, 0x64, 0x61, 0x74, 0x42, 0x61, 0x68,
0x72, 0x65, 0x79, 0x6e, 0x42, 0x61, 0x6b, 0xfc, 0x44, 0x75, 0x15f, 0x61,
0x6e, 0x62, 0x65, 0x47, 0x61, 0x7a, 0x69, 0x6d, 0x61, 0x11f, 0x75, 0x73,
0x61, 0x47, 0x61, 0x7a, 0x7a, 0x65, 0x45, 0x6c, 0x20, 0x48, 0x61, 0x6c,
0x69, 0x6c, 0x48, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e,
0x68, 0x20, 0x4b, 0x65, 0x6e, 0x74, 0x69, 0x4b, 0x75, 0x64, 0xfc, 0x73,
0x4b, 0x61, 0x6c, 0x6b, 0xfc, 0x74, 0x61, 0x4b, 0x75, 0x76, 0x65, 0x79,
0x74, 0x4c, 0x65, 0x66, 0x6b, 0x6f, 0x15f, 0x61, 0x4b, 0x131, 0x7a, 0x131,
0x6c, 0x6f, 0x72, 0x64, 0x61, 0x54, 0x61, 0x68, 0x72, 0x61, 0x6e, 0x55,
0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x75, 0x72, 0x41, 0x7a, 0x6f,
0x72, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x4b, 0x61, 0x6e,
0x61, 0x72, 0x79, 0x61, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131,
0x4d, 0x61, 0x64, 0x65, 0x69, 0x72, 0x61, 0x20, 0x41, 0x64, 0x61, 0x6c,
0x61, 0x72, 0x131, 0x47, 0xfc, 0x6e, 0x65, 0x79, 0x20, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x61, 0x42, 0x72, 0xfc, 0x6b, 0x73, 0x65, 0x6c, 0x42,
0xfc, 0x6b, 0x72, 0x65, 0x15f, 0x42, 0x75, 0x64, 0x61, 0x70, 0x65, 0x15f,
0x74, 0x65, 0x4b, 0x69, 0x15f, 0x69, 0x6e, 0x65, 0x76, 0x43, 0x65, 0x62,
0x65, 0x6c, 0x69, 0x74, 0x61, 0x72, 0x131, 0x6b, 0x4d, 0x61, 0x6e, 0x20,
0x41, 0x64, 0x61, 0x73, 0x131, 0x4c, 0xfc, 0x6b, 0x73, 0x65, 0x6d, 0x62,
0x75, 0x72, 0x67, 0xdc, 0x73, 0x6b, 0xfc, 0x70, 0x56, 0x61, 0x72, 0x15f,
0x6f, 0x76, 0x61, 0x5a, 0xfc, 0x72, 0x69, 0x68, 0x4d, 0x61, 0x6c, 0x64,
0x69, 0x76, 0x6c, 0x65, 0x72, 0x50, 0x61, 0x73, 0x6b, 0x61, 0x6c, 0x79,
0x61, 0x20, 0x41, 0x64, 0x61, 0x73, 0x131, 0x4d, 0x61, 0x72, 0x6b, 0x69,
0x7a, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x41, 0x62, 0x69,
0x6a, 0x61, 0x6e, 0x42, 0x6c, 0x61, 0x6e, 0x74, 0x61, 0xfd, 0x72, 0x42,
0x72, 0x61, 0x7a, 0x7a, 0x61, 0x77, 0x69, 0x6c, 0x45, 0x6c, 0x2d, 0x41,
0xfd, 0x75, 0x6e, 0xdd, 0x6f, 0x68, 0x61, 0x6e, 0x6e, 0x65, 0x73, 0x62,
0x75, 0x72, 0x67, 0x4c, 0x69, 0x62, 0x72, 0x65, 0x77, 0x69, 0x6c, 0x4d,
0x6f, 0x67, 0x61, 0x64, 0x69, 0x15f, 0x6f, 0x4d, 0x6f, 0x6e, 0x72, 0x6f,
0x77, 0x69, 0xfd, 0x61, 0x4e, 0x61, 0xfd, 0x72, 0x6f, 0x62, 0x69, 0x4a,
0x61, 0x6d, 0x65, 0x6e, 0x61, 0x4e, 0x69, 0x61, 0x6d, 0x65, 0xfd, 0x4e,
0x75, 0x61, 0x6b, 0x15f, 0x6f, 0x74, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x2d,
0x4e, 0x6f, 0x77, 0x6f, 0x53, 0x61, 0x6e, 0x2d, 0x54, 0x6f, 0x6d, 0x65,
0x41, 0x64, 0x61, 0x6b, 0x20, 0x61, 0x64, 0x61, 0x73, 0x79, 0x41, 0x6e,
0x67, 0x69, 0x6c, 0xfd, 0x61, 0x41, 0x72, 0x61, 0x67, 0x75, 0x61, 0xfd,
0x6e, 0x61, 0x42, 0x75, 0x65, 0x6e, 0x6f, 0x73, 0x2d, 0x41, 0xfd, 0x72,
0x65, 0x73, 0x4b, 0x6f, 0x72, 0x64, 0x6f, 0x77, 0x61, 0x17d, 0x75, 0x17e,
0x75, 0xfd, 0x4c, 0x61, 0x2d, 0x52, 0x69, 0x6f, 0x68, 0x61, 0x53, 0x61,
0x6e, 0x2d, 0x4c, 0x75, 0x69, 0x73, 0x55, 0x15f, 0x75, 0x61, 0xfd, 0x61,
0x41, 0x73, 0x75, 0x6e, 0x73, 0xfd, 0x6f, 0x6e, 0x42, 0x61, 0x69, 0xfd,
0x61, 0x42, 0x61, 0x69, 0xfd, 0x61, 0x2d, 0x64, 0x65, 0x2d, 0x42, 0x61,
0x6e, 0x64, 0x65, 0x72, 0x61, 0x73, 0x42, 0x6f, 0x61, 0x2d, 0x57, 0x69,
0x73, 0x74, 0x61, 0x42, 0x6f, 0xfd, 0x73, 0x65, 0x4b, 0x65, 0x6d, 0x62,
0x72, 0x69, 0x6a, 0x2d, 0x42, 0x65, 0xfd, 0x4b, 0x61, 0x6d, 0x70, 0x75,
0x2d, 0x47, 0x72, 0x61, 0x6e, 0x64, 0x69, 0x4b, 0x61, 0xfd, 0x65, 0x6e,
0x6e, 0x61, 0x4b, 0x61, 0xfd, 0x6d, 0x61, 0x6e, 0x20, 0x61, 0x64, 0x61,
0x6c, 0x61, 0x72, 0x79, 0x53, 0xfd, 0x75, 0x64, 0x61, 0x64, 0x2d, 0x48,
0x75, 0x61, 0x72, 0x65, 0x73, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x2d, 0x52,
0x69, 0x6b, 0x61, 0x4b, 0x75, 0xfd, 0x61, 0x62, 0x61, 0x4b, 0xfd, 0x75,
0x72, 0x61, 0x73, 0x61, 0x6f, 0x44, 0x65, 0x6e, 0x6d, 0x61, 0x72, 0x6b,
0x73, 0x68, 0x61, 0x77, 0x6e, 0x44, 0x6f, 0x75, 0x73, 0x6f, 0x6e, 0x2d,
0x4b, 0x72, 0x69, 0x6b, 0x44, 0x65, 0x6e, 0x77, 0x65, 0x72, 0x44, 0x65,
0x74, 0x72, 0x6f, 0xfd, 0x74, 0x45, 0xfd, 0x72, 0x75, 0x6e, 0x65, 0x70,
0x65, 0x47, 0x6c, 0x65, 0xfd, 0x73, 0x2d, 0x42, 0x65, 0xfd, 0x47, 0x75,
0x73, 0x2d, 0x42, 0x65, 0xfd, 0x47, 0x72, 0x61, 0x6e, 0x64, 0x2d, 0x54,
0x65, 0x72, 0x6b, 0x47, 0x75, 0xfd, 0x61, 0x6b, 0x69, 0x6c, 0x47, 0x61,
0xfd, 0x61, 0x6e, 0x61, 0x47, 0x61, 0x6c, 0x69, 0x66, 0x61, 0x6b, 0x73,
0x47, 0x61, 0x77, 0x61, 0x6e, 0x61, 0x45, 0x72, 0x6d, 0x6f, 0x73, 0x69,
0x6c, 0x6f, 0x4e, 0x6f, 0x6b, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69,
0x61, 0x6e, 0x61, 0x57, 0x69, 0x77, 0x65, 0xfd, 0x2c, 0x20, 0x49, 0x6e,
0x64, 0x69, 0x61, 0x6e, 0x61, 0x57, 0x69, 0x6e, 0x73, 0x65, 0x6e, 0x73,
0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x49, 0x6e, 0x75,
0x77, 0x69, 0x6b, 0xdd, 0x61, 0x6d, 0x61, 0xfd, 0x6b, 0x61, 0x4c, 0x75,
0x69, 0x73, 0x77, 0x69, 0x6c, 0x6c, 0x4d, 0x6f, 0x6e, 0x74, 0x69, 0x73,
0x65, 0x6c, 0x6c, 0x6f, 0x2c, 0x20, 0x4b, 0x65, 0x6e, 0x74, 0x75, 0x6b,
0x69, 0x4b, 0x72, 0x61, 0x6c, 0x65, 0x6e, 0x64, 0x65, 0xfd, 0x6b, 0x4c,
0x61, 0x2d, 0x50, 0x61, 0x73, 0x4c, 0x6f, 0x73, 0x2d, 0x41, 0x6e, 0x6a,
0x65, 0x6c, 0x65, 0x73, 0x4c, 0x6f, 0x77, 0x65, 0x72, 0x2d, 0x50, 0x72,
0x69, 0x6e, 0x73, 0x65, 0x73, 0x2d, 0x4b, 0x75, 0x6f, 0x72, 0x74, 0x65,
0x72, 0x4d, 0x61, 0x73, 0x65, 0xfd, 0x6f, 0x4d, 0x6f, 0x6e, 0x74, 0x65,
0x72, 0x72, 0x65, 0xfd, 0x4d, 0x6f, 0x6e, 0x74, 0x65, 0x77, 0x69, 0x64,
0x65, 0x6f, 0x4e, 0xfd, 0x75, 0x2d, 0xdd, 0x6f, 0x72, 0x6b, 0x42, 0x6f,
0xfd, 0x6c, 0x61, 0x2c, 0x20, 0x44, 0x65, 0x6d, 0x69, 0x72, 0x67, 0x61,
0x7a, 0x79, 0x6b, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x53, 0x65,
0x6e, 0x74, 0x72, 0x2c, 0x20, 0x44, 0x65, 0x6d, 0x69, 0x72, 0x67, 0x61,
0x7a, 0x79, 0x6b, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0xfd,
0x75, 0x2d, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44, 0x65, 0x6d,
0x69, 0x72, 0x67, 0x61, 0x7a, 0x79, 0x6b, 0x20, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x50, 0x6f, 0x72, 0x74, 0x2d, 0x6f, 0x66, 0x2d, 0x53, 0x70,
0x65, 0xfd, 0x6e, 0x50, 0x6f, 0x72, 0x74, 0x75, 0x2d, 0x57, 0x65, 0x6c,
0xfd, 0x75, 0x50, 0x75, 0x65, 0x72, 0x74, 0x6f, 0x2d, 0x52, 0x69, 0x6b,
0x6f, 0x52, 0x65, 0x6a, 0x61, 0xfd, 0x6e, 0x61, 0x52, 0x65, 0x7a, 0x6f,
0x6c, 0xfd, 0x75, 0x74, 0x52, 0x69, 0x75, 0x2d, 0x42, 0x72, 0x61, 0x6e,
0x6b, 0x75, 0x53, 0x61, 0x6e, 0x74, 0xfd, 0x61, 0x67, 0x6f, 0x53, 0x61,
0x6e, 0x2d, 0x50, 0x61, 0x75, 0x6c, 0x75, 0x49, 0x6c, 0x6c, 0x6f, 0x6b,
0x6b, 0x6f, 0x72, 0x74, 0x6f, 0x6f, 0x72, 0x6d, 0x69, 0x75, 0x74, 0x53,
0x65, 0x6e, 0x74, 0x2d, 0x4a, 0x6f, 0x6e, 0x73, 0x53, 0x65, 0x6e, 0x74,
0x2d, 0x4b, 0x69, 0x74, 0x73, 0x53, 0x65, 0x6e, 0x74, 0x2d, 0x4c, 0xfd,
0x75, 0x73, 0x69, 0xfd, 0x61, 0x53, 0x65, 0x6e, 0x74, 0x2d, 0x54, 0x6f,
0x6d, 0x61, 0x73, 0x53, 0x65, 0x6e, 0x74, 0x2d, 0x57, 0x69, 0x6e, 0x73,
0x65, 0x6e, 0x74, 0x53, 0x77, 0x69, 0x66, 0x74, 0x2d, 0x4b, 0x61, 0x72,
0x65, 0x6e, 0x74, 0x57, 0x61, 0x6e, 0x6b, 0x75, 0x77, 0x65, 0x72, 0x57,
0x61, 0xfd, 0x74, 0x68, 0x6f, 0x72, 0x73, 0xdd, 0x61, 0x6b, 0x75, 0x74,
0x61, 0x74, 0x4b, 0x65, 0xfd, 0x73, 0x69, 0x44, 0x65, 0xfd, 0x77, 0x69,
0x73, 0x44, 0xfd, 0x75, 0x6d, 0x6f, 0x6e, 0x2d, 0x64, 0x2d, 0xdd, 0x75,
0x72, 0x77, 0x69, 0x6c, 0x4d, 0x61, 0x6b, 0x6b, 0x75, 0x6f, 0x72, 0x69,
0x4d, 0x61, 0x6b, 0x2d, 0x4d, 0x65, 0x72, 0x64, 0x6f, 0x53, 0xfd, 0x6f,
0x77, 0x61, 0x4c, 0x6f, 0x6e, 0x67, 0x69, 0x72, 0x42, 0x61, 0x67, 0x64,
0x61, 0x74, 0x42, 0x61, 0x68, 0x72, 0x65, 0xfd, 0x6e, 0x42, 0x65, 0xfd,
0x72, 0x75, 0x74, 0x42, 0x72, 0x75, 0x6e, 0x65, 0xfd, 0x44, 0x75, 0x62,
0x61, 0xfd, 0x48, 0x65, 0x77, 0x72, 0x6f, 0x6e, 0x48, 0x6f, 0x15f, 0x69,
0x6d, 0x69, 0x6e, 0x47, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e, 0x67, 0x48, 0x6f,
0x77, 0x64, 0x4a, 0x61, 0xfd, 0x61, 0x70, 0x75, 0x72, 0x61, 0x49, 0xfd,
0x65, 0x72, 0x75, 0x73, 0x61, 0x6c, 0x69, 0x6d, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0xfd, 0x61, 0x72, 0x73, 0x6b, 0x4b, 0x75, 0x77, 0x65, 0xfd,
0x74, 0x4e, 0x69, 0x6b, 0x6f, 0x73, 0x69, 0xfd, 0x61, 0x4e, 0x6f, 0x77,
0x6f, 0x6b, 0x75, 0x7a, 0x6e, 0x65, 0x73, 0x6b, 0x50, 0x68, 0x65, 0x6e,
0xfd, 0x61, 0x6e, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x6e, 0x61, 0xfd, 0x47,
0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x45, 0x72, 0x2d, 0x52,
0x69, 0xfd, 0x61, 0x64, 0x15e, 0x61, 0x6e, 0x68, 0x61, 0xfd, 0x54, 0x61,
0xfd, 0x62, 0x65, 0xfd, 0x54, 0xe4, 0x68, 0x72, 0x61, 0x6e, 0x57, 0xfd,
0x65, 0x6e, 0x74, 0xfd, 0x61, 0x6e, 0xdd, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0xdd, 0x61, 0x6e, 0x67, 0x6f, 0x6e, 0xdd, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0xdd, 0x65, 0x72, 0x65,
0x77, 0x61, 0x6e, 0x41, 0x7a, 0x6f, 0x72, 0x20, 0x61, 0x64, 0x61, 0x6c,
0x61, 0x72, 0x79, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x20, 0x61, 0x64, 0x61,
0x6c, 0x61, 0x72, 0x79, 0x4b, 0x61, 0x62, 0x6f, 0x2d, 0x57, 0x65, 0x72,
0x64, 0x65, 0x46, 0x61, 0x72, 0x65, 0x72, 0x20, 0x61, 0x64, 0x61, 0x6c,
0x61, 0x72, 0x79, 0x4d, 0x61, 0x64, 0x65, 0xfd, 0x72, 0x61, 0x20, 0x61,
0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x52, 0x65, 0xfd, 0x6b, 0xfd, 0x61,
0x77, 0x69, 0x6b, 0x47, 0xfc, 0x6e, 0x6f, 0x72, 0x74, 0x61, 0x20, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0xfd, 0x61, 0x4b, 0x65, 0x72, 0x61, 0x6d,
0x61, 0x74, 0x6c, 0x79, 0x20, 0xdd, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x20,
0x61, 0x64, 0x61, 0x73, 0x79, 0xdd, 0x75, 0x6b, 0x6c, 0x61, 0x4c, 0x6f,
0x72, 0x64, 0x2d, 0x48, 0x61, 0x75, 0x53, 0x69, 0x64, 0x6e, 0x65, 0xfd,
0x41, 0x66, 0x69, 0x6e, 0x79, 0x42, 0x72, 0x61, 0x74, 0x69, 0x73, 0x6c,
0x61, 0x77, 0x61, 0x42, 0x72, 0xfd, 0x75, 0x73, 0x73, 0x65, 0x6c, 0x42,
0x75, 0x68, 0x61, 0x72, 0x65, 0x73, 0x74, 0x42, 0xfd, 0x75, 0x7a, 0x69,
0x6e, 0x67, 0x65, 0x6e, 0x4b, 0x69, 0x15f, 0x69, 0x6e, 0xfd, 0x6f, 0x77,
0x4b, 0x6f, 0x70, 0x65, 0x6e, 0x67, 0x61, 0x67, 0x65, 0x6e, 0x4d, 0x65,
0x6e, 0x20, 0x61, 0x64, 0x61, 0x73, 0x79, 0x4a, 0x65, 0x72, 0x73, 0x69,
0x4b, 0x69, 0xfd, 0x65, 0x77, 0x4c, 0xfd, 0x75, 0x62, 0x6c, 0xfd, 0x61,
0x6e, 0x61, 0x4c, 0xfd, 0x75, 0x6b, 0x73, 0x65, 0x6d, 0x62, 0x75, 0x72,
0x67, 0x4d, 0x61, 0x72, 0x69, 0xfd, 0x65, 0x68, 0x61, 0x6d, 0x6e, 0x50,
0x61, 0x72, 0x69, 0x17e, 0x50, 0x6f, 0x64, 0x67, 0x6f, 0x72, 0x69, 0x73,
0x61, 0x53, 0x61, 0x72, 0x61, 0xfd, 0x65, 0x77, 0x6f, 0x53, 0x6b, 0x6f,
0x70, 0xfd, 0x65, 0x53, 0x6f, 0x66, 0x69, 0xfd, 0x61, 0x53, 0x74, 0x6f,
0x6b, 0x67, 0x6f, 0x6c, 0x6d, 0x55, 0x6c, 0xfd, 0x61, 0x6e, 0x6f, 0x77,
0x73, 0x6b, 0x57, 0x61, 0x64, 0x75, 0x7a, 0x57, 0x61, 0x74, 0x69, 0x6b,
0x61, 0x6e, 0x57, 0x65, 0x6e, 0x61, 0x57, 0x69, 0x6c, 0x6e, 0xfd, 0x75,
0x73, 0x57, 0x61, 0x72, 0x15f, 0x61, 0x77, 0x61, 0x53, 0xfd, 0x75, 0x72,
0x69, 0x68, 0x41, 0x6e, 0x74, 0x61, 0x6e, 0x61, 0x6e, 0x61, 0x72, 0x69,
0x77, 0x75, 0x52, 0x6f, 0x17e, 0x64, 0x65, 0x73, 0x74, 0x77, 0x6f, 0x4b,
0x6f, 0x6d, 0x6f, 0x72, 0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79,
0x4d, 0x61, 0xfd, 0x65, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x77, 0x6c, 0x65,
0x72, 0x4d, 0x61, 0x77, 0x72, 0x69, 0x6b, 0x69, 0xfd, 0x4d, 0x61, 0xfd,
0x6f, 0x74, 0x74, 0x61, 0x52, 0x65, 0xfd, 0x75, 0x6e, 0xfd, 0x6f, 0x6e,
0x42, 0x75, 0x67, 0x65, 0x6e, 0x77, 0x69, 0x6c, 0xc7, 0x61, 0x74, 0x65,
0x6d, 0x50, 0x61, 0x73, 0x68, 0x61, 0x20, 0x61, 0x64, 0x61, 0x73, 0x79,
0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x20, 0x61, 0x64,
0x61, 0x6c, 0x61, 0x72, 0x79, 0x47, 0x61, 0x6d, 0x62, 0xfd, 0x65, 0x47,
0x77, 0x61, 0x64, 0x61, 0x6c, 0x6b, 0x61, 0x6e, 0x61, 0x6c, 0x47, 0x6f,
0x6e, 0x6f, 0x6c, 0x75, 0x6c, 0x75, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0xfd,
0x65, 0x4b, 0x77, 0x61, 0x6a, 0x61, 0x6c, 0x65, 0xfd, 0x6e, 0x4d, 0x61,
0x72, 0x6b, 0x69, 0x7a, 0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79,
0x4d, 0x69, 0x64, 0x75, 0x65, 0xfd, 0x50, 0x6f, 0x72, 0x74, 0x2d, 0x4d,
0x6f, 0x72, 0x73, 0x62, 0x69, 0x53, 0x61, 0xfd, 0x70, 0x61, 0x6e, 0x57,
0x65, 0xfd, 0x6b, 0x411, 0x430, 0x43d, 0x491, 0x456, 0x411, 0x440, 0x430, 0x437,
0x437, 0x430, 0x432, 0x456, 0x43b, 0x44c, 0x41a, 0x430, 0x457, 0x440, 0x41a, 0x43e,
0x43d, 0x430, 0x43a, 0x440, 0x456, 0x414, 0x436, 0x456, 0x431, 0x443, 0x442, 0x456,
0x415, 0x43b, 0x44c, 0x2d, 0x410, 0x44e, 0x43d, 0x424, 0x440, 0x456, 0x442, 0x430,
0x443, 0x43d, 0x490, 0x430, 0x431, 0x43e, 0x440, 0x43e, 0x43d, 0x435, 0x419, 0x43e,
0x433, 0x430, 0x43d, 0x43d, 0x435, 0x441, 0x431, 0x443, 0x440, 0x491, 0x41b, 0x430,
0x491, 0x43e, 0x441, 0x41b, 0x456, 0x431, 0x440, 0x435, 0x432, 0x456, 0x43b, 0x44c,
0x41b, 0x443, 0x431, 0x443, 0x43c, 0x431, 0x430, 0x448, 0x456, 0x41c, 0x43e, 0x491,
0x430, 0x434, 0x456, 0x448, 0x43e, 0x41c, 0x43e, 0x43d, 0x440, 0x43e, 0x432, 0x456,
0x44f, 0x41d, 0x456, 0x430, 0x43c, 0x435, 0x439, 0x41d, 0x443, 0x430, 0x43a, 0x448,
0x43e, 0x442, 0x442, 0x423, 0x430, 0x491, 0x430, 0x434, 0x443, 0x491, 0x443, 0x422,
0x440, 0x438, 0x43f, 0x43e, 0x43b, 0x456, 0x412, 0x456, 0x43d, 0x434, 0x433, 0x443,
0x43a, 0x410, 0x43d, 0x491, 0x456, 0x43b, 0x44c, 0x44f, 0x410, 0x43d, 0x442, 0x438,
0x491, 0x443, 0x430, 0x410, 0x440, 0x430, 0x491, 0x443, 0x430, 0x456, 0x43d, 0x430,
0x411, 0x443, 0x435, 0x43d, 0x43e, 0x441, 0x2d, 0x410, 0x439, 0x440, 0x435, 0x441,
0x41b, 0x430, 0x2d, 0x420, 0x456, 0x43e, 0x445, 0x430, 0x420, 0x456, 0x43e, 0x2d,
0x490, 0x430, 0x43b, 0x44c, 0x454, 0x491, 0x43e, 0x441, 0x421, 0x430, 0x43d, 0x2d,
0x41b, 0x443, 0x457, 0x441, 0x410, 0x441, 0x443, 0x43d, 0x441, 0x44c, 0x439, 0x43e,
0x43d, 0x410, 0x442, 0x456, 0x43a, 0x43e, 0x43a, 0x430, 0x43d, 0x411, 0x430, 0x457,
0x44f, 0x20, 0x411, 0x430, 0x43d, 0x434, 0x435, 0x440, 0x430, 0x441, 0x411, 0x43e,
0x491, 0x43e, 0x442, 0x430, 0x41a, 0x435, 0x439, 0x43c, 0x431, 0x440, 0x456, 0x434,
0x436, 0x2d, 0x411, 0x435, 0x439, 0x41a, 0x430, 0x43c, 0x43f, 0x443, 0x2d, 0x490,
0x440, 0x430, 0x43d, 0x434, 0x456, 0x41a, 0x430, 0x454, 0x43d, 0x43d, 0x430, 0x41a,
0x430, 0x439, 0x43c, 0x430, 0x43d, 0x43e, 0x432, 0x456, 0x20, 0x41e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x438, 0x427, 0x456, 0x43a, 0x430, 0x491, 0x43e, 0x427, 0x456,
0x443, 0x430, 0x443, 0x430, 0x421, 0x2019, 0x44e, 0x434, 0x430, 0x434, 0x2d, 0x425,
0x443, 0x430, 0x440, 0x435, 0x441, 0x41a, 0x43e, 0x441, 0x442, 0x430, 0x2d, 0x420,
0x456, 0x43a, 0x430, 0x414, 0x43e, 0x443, 0x441, 0x43e, 0x43d, 0x2d, 0x41a, 0x440,
0x456, 0x43a, 0x414, 0x43e, 0x43c, 0x456, 0x43d, 0x456, 0x43a, 0x430, 0x490, 0x43b,
0x435, 0x439, 0x441, 0x2d, 0x411, 0x435, 0x439, 0x490, 0x443, 0x441, 0x2d, 0x411,
0x435, 0x439, 0x490, 0x440, 0x430, 0x43d, 0x434, 0x2d, 0x422, 0x435, 0x440, 0x43a,
0x490, 0x440, 0x435, 0x43d, 0x430, 0x434, 0x430, 0x490, 0x432, 0x430, 0x434, 0x435,
0x43b, 0x443, 0x43f, 0x430, 0x490, 0x432, 0x430, 0x442, 0x435, 0x43c, 0x430, 0x43b,
0x430, 0x490, 0x443, 0x430, 0x44f, 0x43a, 0x456, 0x43b, 0x44c, 0x490, 0x430, 0x439,
0x430, 0x43d, 0x430, 0x415, 0x440, 0x43c, 0x43e, 0x441, 0x456, 0x43b, 0x44c, 0x439,
0x43e, 0x406, 0x43d, 0x434, 0x456, 0x430, 0x43d, 0x430, 0x43f, 0x43e, 0x43b, 0x456,
0x441, 0x41d, 0x43e, 0x43a, 0x441, 0x2c, 0x20, 0x406, 0x43d, 0x434, 0x456, 0x430,
0x43d, 0x430, 0x41c, 0x430, 0x440, 0x435, 0x43d, 0x433, 0x43e, 0x2c, 0x20, 0x406,
0x43d, 0x434, 0x456, 0x430, 0x43d, 0x430, 0x41f, 0x456, 0x442, 0x435, 0x440, 0x441,
0x431, 0x435, 0x440, 0x433, 0x2c, 0x20, 0x406, 0x43d, 0x434, 0x456, 0x430, 0x43d,
0x430, 0x422, 0x435, 0x43b, 0x43b, 0x2d, 0x421, 0x456, 0x442, 0x456, 0x2c, 0x20,
0x406, 0x43d, 0x434, 0x456, 0x430, 0x43d, 0x430, 0x412, 0x456, 0x432, 0x435, 0x439,
0x2c, 0x20, 0x406, 0x43d, 0x434, 0x456, 0x430, 0x43d, 0x430, 0x412, 0x456, 0x43d,
0x441, 0x435, 0x43d, 0x441, 0x2c, 0x20, 0x406, 0x43d, 0x434, 0x456, 0x430, 0x43d,
0x430, 0x412, 0x456, 0x43d, 0x430, 0x43c, 0x430, 0x43a, 0x2c, 0x20, 0x406, 0x43d,
0x434, 0x456, 0x430, 0x43d, 0x430, 0x406, 0x43a, 0x430, 0x43b, 0x443, 0x457, 0x442,
0x41b, 0x443, 0x457, 0x441, 0x432, 0x456, 0x43b, 0x43b, 0x41c, 0x43e, 0x43d, 0x442,
0x456, 0x441, 0x435, 0x43b, 0x43b, 0x43e, 0x2c, 0x20, 0x41a, 0x435, 0x43d, 0x442,
0x443, 0x43a, 0x43a, 0x456, 0x41b, 0x43e, 0x443, 0x435, 0x440, 0x2d, 0x41f, 0x440,
0x438, 0x43d, 0x441, 0x435, 0x441, 0x2d, 0x41a, 0x432, 0x43e, 0x442, 0x435, 0x440,
0x41c, 0x430, 0x43d, 0x430, 0x491, 0x443, 0x430, 0x41c, 0x430, 0x440, 0x456, 0x491,
0x43e, 0x41c, 0x430, 0x440, 0x442, 0x456, 0x43d, 0x456, 0x43a, 0x430, 0x41c, 0x435,
0x440, 0x456, 0x434, 0x430, 0x41c, 0x435, 0x445, 0x456, 0x43a, 0x43e, 0x41c, 0x43e,
0x43d, 0x442, 0x435, 0x432, 0x456, 0x434, 0x435, 0x43e, 0x411, 0x2bc, 0x44e, 0x43b,
0x430, 0x2c, 0x20, 0x41f, 0x456, 0x432, 0x43d, 0x456, 0x447, 0x43d, 0x430, 0x20,
0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x421, 0x435, 0x43d, 0x442, 0x440, 0x2c,
0x20, 0x41f, 0x456, 0x432, 0x43d, 0x456, 0x447, 0x43d, 0x430, 0x20, 0x414, 0x430,
0x43a, 0x43e, 0x442, 0x430, 0x41d, 0x44c, 0x44e, 0x2d, 0x421, 0x430, 0x43b, 0x435,
0x43c, 0x2c, 0x20, 0x41f, 0x456, 0x432, 0x43d, 0x456, 0x447, 0x43d, 0x430, 0x20,
0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41e, 0x445, 0x456, 0x43d, 0x430, 0x491,
0x430, 0x41f, 0x443, 0x435, 0x440, 0x442, 0x43e, 0x2d, 0x420, 0x456, 0x43a, 0x43e,
0x420, 0x435, 0x43d, 0x43a, 0x456, 0x43d, 0x2d, 0x406, 0x43d, 0x43b, 0x435, 0x442,
0x420, 0x435, 0x441, 0x456, 0x444, 0x456, 0x420, 0x456, 0x43e, 0x2d, 0x411, 0x440,
0x430, 0x43d, 0x43a, 0x43e, 0x421, 0x430, 0x43d, 0x442, 0x44c, 0x44f, 0x491, 0x43e,
0x421, 0x430, 0x43d, 0x442, 0x43e, 0x2d, 0x414, 0x43e, 0x43c, 0x456, 0x43d, 0x491,
0x43e, 0x406, 0x442, 0x442, 0x43e, 0x43a, 0x43a, 0x43e, 0x440, 0x442, 0x43e, 0x43e,
0x440, 0x43c, 0x456, 0x443, 0x442, 0x421, 0x435, 0x43d, 0x2d, 0x411, 0x430, 0x440,
0x442, 0x435, 0x43b, 0x435, 0x43c, 0x456, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41a,
0x456, 0x442, 0x442, 0x441, 0x421, 0x432, 0x456, 0x444, 0x442, 0x2d, 0x41a, 0x430,
0x440, 0x440, 0x435, 0x43d, 0x442, 0x422, 0x435, 0x491, 0x443, 0x441, 0x456, 0x491,
0x430, 0x43b, 0x44c, 0x43f, 0x430, 0x422, 0x456, 0x445, 0x443, 0x430, 0x43d, 0x430,
0x412, 0x430, 0x439, 0x442, 0x433, 0x43e, 0x440, 0x441, 0x412, 0x456, 0x43d, 0x43d,
0x456, 0x43f, 0x435, 0x491, 0x41a, 0x435, 0x439, 0x441, 0x456, 0x414, 0x435, 0x432,
0x456, 0x441, 0x414, 0x44e, 0x43c, 0x43e, 0x43d, 0x2d, 0x434, 0x2bc, 0x42e, 0x440,
0x432, 0x456, 0x43b, 0x44c, 0x41c, 0x430, 0x43a, 0x43a, 0x443, 0x43e, 0x440, 0x456,
0x421, 0x44c, 0x43e, 0x432, 0x430, 0x41b, 0x43e, 0x43d, 0x433, 0x2bc, 0x457, 0x440,
0x414, 0x456, 0x43b, 0x456, 0x425, 0x43e, 0x448, 0x438, 0x43c, 0x456, 0x43d, 0x406,
0x440, 0x43a, 0x443, 0x442, 0x441, 0x44c, 0x43a, 0x404, 0x440, 0x443, 0x441, 0x430,
0x43b, 0x438, 0x43c, 0x41a, 0x430, 0x440, 0x430, 0x447, 0x456, 0x41a, 0x440, 0x430,
0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x44c, 0x43a, 0x41d, 0x456, 0x43a, 0x43e,
0x441, 0x456, 0x44f, 0x41d, 0x43e, 0x432, 0x43e, 0x43a, 0x443, 0x437, 0x43d, 0x435,
0x446, 0x44c, 0x43a, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x456, 0x440,
0x441, 0x44c, 0x43a, 0x41e, 0x43c, 0x441, 0x44c, 0x43a, 0x41f, 0x43e, 0x43d, 0x442,
0x456, 0x430, 0x43d, 0x430, 0x43a, 0x415, 0x440, 0x2d, 0x420, 0x456, 0x44f, 0x434,
0x421, 0x435, 0x440, 0x435, 0x434, 0x43d, 0x44c, 0x43e, 0x43a, 0x43e, 0x43b, 0x438,
0x43c, 0x441, 0x44c, 0x43a, 0x422, 0x430, 0x439, 0x431, 0x435, 0x439, 0x422, 0x43e,
0x43a, 0x456, 0x43e, 0x422, 0x43e, 0x43c, 0x441, 0x44c, 0x43a, 0x423, 0x440, 0x443,
0x43c, 0x447, 0x456, 0x412, 0x2bc, 0x454, 0x43d, 0x442, 0x44c, 0x44f, 0x43d, 0x42f,
0x43a, 0x443, 0x442, 0x441, 0x44c, 0x43a, 0x404, 0x43a, 0x430, 0x442, 0x435, 0x440,
0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x404, 0x440, 0x435, 0x432, 0x430, 0x43d,
0x410, 0x437, 0x43e, 0x440, 0x441, 0x44c, 0x43a, 0x456, 0x20, 0x43e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x438, 0x41a, 0x430, 0x43d, 0x430, 0x440, 0x441, 0x44c, 0x43a,
0x456, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x424, 0x430, 0x440,
0x435, 0x440, 0x441, 0x44c, 0x43a, 0x456, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e,
0x432, 0x438, 0x420, 0x435, 0x439, 0x43a, 0x2bc, 0x44f, 0x432, 0x456, 0x43a, 0x41f,
0x456, 0x432, 0x434, 0x435, 0x43d, 0x43d, 0x430, 0x20, 0x414, 0x436, 0x43e, 0x440,
0x434, 0x436, 0x456, 0x44f, 0x41e, 0x441, 0x442, 0x440, 0x456, 0x432, 0x20, 0x421,
0x432, 0x44f, 0x442, 0x43e, 0x457, 0x20, 0x404, 0x43b, 0x435, 0x43d, 0x438, 0x421,
0x442, 0x435, 0x43d, 0x43b, 0x456, 0x410, 0x434, 0x435, 0x43b, 0x430, 0x457, 0x434,
0x430, 0x411, 0x440, 0x456, 0x441, 0x431, 0x435, 0x43d, 0x411, 0x440, 0x43e, 0x43a,
0x435, 0x43d, 0x2d, 0x425, 0x456, 0x43b, 0x43b, 0x415, 0x432, 0x43a, 0x43b, 0x430,
0x413, 0x43e, 0x431, 0x430, 0x440, 0x442, 0x41b, 0x456, 0x43d, 0x434, 0x435, 0x43c,
0x430, 0x43d, 0x410, 0x444, 0x456, 0x43d, 0x438, 0x41a, 0x438, 0x448, 0x438, 0x43d,
0x456, 0x432, 0x413, 0x435, 0x43b, 0x44c, 0x441, 0x456, 0x43d, 0x43a, 0x456, 0x41e,
0x441, 0x442, 0x440, 0x456, 0x432, 0x20, 0x41c, 0x435, 0x43d, 0x414, 0x436, 0x435,
0x440, 0x441, 0x456, 0x41a, 0x456, 0x440, 0x43e, 0x432, 0x41a, 0x438, 0x457, 0x432,
0x41c, 0x430, 0x440, 0x456, 0x454, 0x433, 0x430, 0x43c, 0x43d, 0x41c, 0x456, 0x43d,
0x441, 0x44c, 0x43a, 0x41f, 0x43e, 0x434, 0x433, 0x43e, 0x440, 0x438, 0x446, 0x44f,
0x421, 0x430, 0x440, 0x430, 0x454, 0x432, 0x43e, 0x421, 0x456, 0x43c, 0x444, 0x435,
0x440, 0x43e, 0x43f, 0x43e, 0x43b, 0x44c, 0x421, 0x43a, 0x43e, 0x43f, 0x2bc, 0x454,
0x421, 0x43e, 0x444, 0x456, 0x44f, 0x422, 0x430, 0x43b, 0x43b, 0x456, 0x43d, 0x43d,
0x423, 0x43b, 0x44c, 0x44f, 0x43d, 0x456, 0x432, 0x441, 0x44c, 0x43a, 0x412, 0x456,
0x434, 0x435, 0x43d, 0x44c, 0x426, 0x44e, 0x440, 0x456, 0x445, 0x410, 0x43d, 0x442,
0x430, 0x43d, 0x430, 0x43d, 0x430, 0x440, 0x456, 0x432, 0x443, 0x427, 0x430, 0x491,
0x43e, 0x441, 0x41e, 0x441, 0x442, 0x440, 0x456, 0x432, 0x20, 0x420, 0x456, 0x437,
0x434, 0x432, 0x430, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x43e, 0x432, 0x456, 0x20,
0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x41a, 0x435, 0x440, 0x491, 0x435,
0x43b, 0x435, 0x43d, 0x41c, 0x430, 0x43b, 0x44c, 0x434, 0x456, 0x432, 0x438, 0x41c,
0x430, 0x432, 0x440, 0x438, 0x43a, 0x456, 0x439, 0x420, 0x435, 0x44e, 0x43d, 0x44c,
0x439, 0x43e, 0x43d, 0x410, 0x43f, 0x456, 0x430, 0x411, 0x443, 0x491, 0x435, 0x43d,
0x432, 0x456, 0x43b, 0x44c, 0x41e, 0x441, 0x442, 0x440, 0x456, 0x432, 0x20, 0x41f,
0x430, 0x441, 0x445, 0x438, 0x424, 0x456, 0x434, 0x436, 0x456, 0x424, 0x443, 0x43d,
0x430, 0x444, 0x443, 0x442, 0x456, 0x490, 0x430, 0x43b, 0x430, 0x43f, 0x430, 0x491,
0x43e, 0x441, 0x413, 0x430, 0x43c, 0x431, 0x2bc, 0x454, 0x415, 0x43d, 0x434, 0x435,
0x440, 0x431, 0x435, 0x440, 0x456, 0x41a, 0x456, 0x440, 0x456, 0x442, 0x456, 0x43c,
0x430, 0x442, 0x456, 0x41c, 0x430, 0x440, 0x43a, 0x456, 0x437, 0x44c, 0x43a, 0x456,
0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x41c, 0x456, 0x434, 0x432,
0x435, 0x439, 0x41d, 0x456, 0x443, 0x435, 0x41f, 0x456, 0x442, 0x43a, 0x435, 0x440,
0x43d, 0x422, 0x430, 0x457, 0x442, 0x456, 0x423, 0x43e, 0x43b, 0x43b, 0x456, 0x441,
0x44, 0x17a, 0x69, 0x62, 0x75, 0x74, 0x69, 0x4b, 0x61, 0x69, 0x6d, 0x61,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x4d, 0x65, 0x78,
0x69, 0x6b, 0x6f, 0x20, 0x6d, 0x11b, 0x73, 0x74, 0x6f, 0x42, 0x65, 0x75,
0x6c, 0x61, 0x68, 0x2c, 0x20, 0x53, 0x65, 0x77, 0x6a, 0x65, 0x72, 0x6e,
0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e, 0x74,
0x65, 0x72, 0x2c, 0x20, 0x53, 0x65, 0x77, 0x6a, 0x65, 0x72, 0x6e, 0x61,
0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53,
0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x53, 0x65, 0x77, 0x6a, 0x65, 0x72,
0x6e, 0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x41, 0x6e, 0x74,
0x61, 0x72, 0x6b, 0x74, 0x69, 0x6b, 0x61, 0x2f, 0x57, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x48, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e,
0x68, 0x6f, 0x77, 0x65, 0x20, 0x6d, 0x11b, 0x73, 0x74, 0x6f, 0x4e, 0x6f,
0x77, 0x6f, 0x6b, 0x75, 0x7a, 0x6e, 0x6a, 0x65, 0x63, 0x6b, 0x4a, 0x65,
0x72, 0x6a, 0x65, 0x77, 0x61, 0x6e, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x69,
0x73, 0x6b, 0x65, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x46, 0xe4, 0x72, 0xf6,
0x73, 0x6b, 0x65, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x4a, 0x75, 0x17e, 0x6e,
0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x61, 0x42,
0x11b, 0x142, 0x6f, 0x68, 0x72, 0xf3, 0x64, 0x4b, 0x69, 0x161, 0x69, 0x6e,
0x6a, 0x6f, 0x77, 0x48, 0x6f, 0x64, 0x6f, 0x77, 0x6e, 0x61, 0x20, 0x6b,
0x75, 0x70, 0x61, 0x4a, 0x75, 0x74, 0x72, 0x6f, 0x77, 0x6e, 0x61, 0x20,
0x6b, 0x75, 0x70, 0x61, 0x46, 0x69, 0x64, 0x17a, 0x69, 0x639, 0x627, 0x628,
0x62f, 0x62c, 0x627, 0x646, 0x627, 0x6a9, 0x651, 0x631, 0x627, 0x627, 0x644, 0x62c,
0x6cc, 0x626, 0x631, 0x633, 0x628, 0x646, 0x6af, 0x648, 0x626, 0x6cc, 0x628, 0x646,
0x62c, 0x648, 0x644, 0x628, 0x644, 0x6cc, 0x646, 0x679, 0x627, 0x626, 0x631, 0x628,
0x631, 0x627, 0x632, 0x627, 0x648, 0x6cc, 0x644, 0x6d2, 0x628, 0x62c, 0x645, 0x628,
0x631, 0x627, 0x642, 0x627, 0x6c1, 0x631, 0x6c1, 0x6a9, 0x6cc, 0x633, 0x627, 0x20,
0x628, 0x644, 0x627, 0x646, 0x6a9, 0x627, 0x633, 0x6cc, 0x648, 0x679, 0x627, 0x6a9,
0x648, 0x646, 0x6a9, 0x631, 0x6cc, 0x688, 0x6a9, 0x627, 0x631, 0x62c, 0x628, 0x648,
0x62a, 0x6cc, 0x688, 0x648, 0x622, 0x644, 0x627, 0x641, 0x631, 0x6cc, 0x20, 0x679,
0x627, 0x624, 0x646, 0x6af, 0x628, 0x631, 0x648, 0x646, 0x6c1, 0x631, 0x627, 0x631,
0x6d2, 0x6a9, 0x6cc, 0x645, 0x67e, 0x627, 0x644, 0x627, 0x6a9, 0x6af, 0x627, 0x644,
0x6cc, 0x6a9, 0x646, 0x634, 0x627, 0x633, 0x627, 0x644, 0x628, 0x631, 0x6d2, 0x20,
0x648, 0x6cc, 0x644, 0x6d2, 0x644, 0x648, 0x627, 0x646, 0x688, 0x627, 0x644, 0x6cc,
0x648, 0x633, 0x627, 0x6a9, 0x627, 0x645, 0x67e, 0x648, 0x679, 0x648, 0x645, 0x633,
0x6cc, 0x631, 0x648, 0x645, 0x628, 0x627, 0x628, 0x6cc, 0x646, 0x627, 0x6cc, 0x646,
0x62c, 0x627, 0x645, 0x6cc, 0x646, 0x627, 0x646, 0x648, 0x627, 0x6a9, 0x634, 0x648,
0x637, 0x627, 0x624, 0x6af, 0x627, 0x688, 0x624, 0x6af, 0x648, 0x648, 0x633, 0x627,
0x624, 0x20, 0x679, 0x648, 0x645, 0x679, 0x631, 0x6cc, 0x67e, 0x648, 0x644, 0x6cc,
0x62a, 0x6cc, 0x648, 0x646, 0x633, 0x648, 0x646, 0x688, 0x6c1, 0x648, 0x6a9, 0x627,
0x646, 0x6af, 0x648, 0x6cc, 0x644, 0x627, 0x627, 0x646, 0x679, 0x6cc, 0x6af, 0x648,
0x627, 0x628, 0x6cc, 0x648, 0x646, 0x633, 0x20, 0x622, 0x626, 0x631, 0x633, 0x6a9,
0x648, 0x631, 0x688, 0x648, 0x628, 0x627, 0x62c, 0x648, 0x62c, 0x648, 0x626, 0x6cc,
0x645, 0x6cc, 0x646, 0x688, 0x648, 0x632, 0x627, 0x633, 0x627, 0x646, 0x20, 0x644,
0x648, 0x626, 0x633, 0x679, 0x648, 0x6a9, 0x648, 0x645, 0x6cc, 0x646, 0x627, 0x633,
0x646, 0x633, 0x6cc, 0x626, 0x646, 0x627, 0x679, 0x6cc, 0x6a9, 0x648, 0x6a9, 0x646,
0x628, 0x6c1, 0x6cc, 0x627, 0x20, 0x628, 0x646, 0x62f, 0x631, 0x627, 0x632, 0x628,
0x6cc, 0x644, 0x6cc, 0x645, 0x628, 0x6cc, 0x644, 0x627, 0x626, 0x632, 0x628, 0x644,
0x627, 0x646, 0x6a9, 0x20, 0x633, 0x628, 0x644, 0x648, 0x646, 0x628, 0x648, 0x622,
0x20, 0x648, 0x633, 0x679, 0x627, 0x628, 0x6af, 0x648, 0x679, 0x627, 0x628, 0x648,
0x627, 0x626, 0x633, 0x6a9, 0x6cc, 0x645, 0x628, 0x631, 0x62c, 0x20, 0x6a9, 0x6cc,
0x20, 0x6a9, 0x6be, 0x627, 0x691, 0x6cc, 0x6a9, 0x6cc, 0x645, 0x67e, 0x648, 0x20,
0x6af, 0x631, 0x6cc, 0x646, 0x688, 0x6a9, 0x646, 0x6a9, 0x6cc, 0x648, 0x646, 0x6a9,
0x631, 0x627, 0x6a9, 0x627, 0x633, 0x6a9, 0x627, 0x626, 0x6cc, 0x646, 0x6a9, 0x6cc,
0x645, 0x6cc, 0x646, 0x634, 0x6a9, 0x627, 0x6af, 0x648, 0x686, 0x6cc, 0x6c1, 0x648,
0x622, 0x6c1, 0x648, 0x622, 0x6a9, 0x648, 0x633, 0x679, 0x627, 0x20, 0x631, 0x6cc,
0x6a9, 0x627, 0x6a9, 0x631, 0x6cc, 0x633, 0x679, 0x648, 0x646, 0x6a9, 0x648, 0x626,
0x6cc, 0x627, 0x628, 0x627, 0x6a9, 0x6cc, 0x648, 0x631, 0x627, 0x6a9, 0x627, 0x624,
0x688, 0x627, 0x624, 0x633, 0x646, 0x688, 0x627, 0x624, 0x633, 0x646, 0x20, 0x6a9,
0x631, 0x6cc, 0x6a9, 0x688, 0x6cc, 0x646, 0x648, 0x631, 0x688, 0x6cc, 0x679, 0x631,
0x627, 0x626, 0x679, 0x688, 0x648, 0x645, 0x646, 0x6cc, 0x6a9, 0x627, 0x627, 0x6cc,
0x688, 0x645, 0x648, 0x646, 0x679, 0x646, 0x627, 0x6cc, 0x631, 0x648, 0x646, 0x6cc,
0x67e, 0x627, 0x644, 0x20, 0x633, 0x644, 0x648, 0x627, 0x688, 0x648, 0x631, 0x641,
0x648, 0x631, 0x679, 0x627, 0x644, 0x6cc, 0x632, 0x627, 0x6af, 0x644, 0x6cc, 0x633,
0x20, 0x6a9, 0x6cc, 0x20, 0x6a9, 0x6be, 0x627, 0x691, 0x6cc, 0x6af, 0x648, 0x633,
0x20, 0x6a9, 0x6cc, 0x20, 0x6a9, 0x6be, 0x627, 0x691, 0x6cc, 0x639, 0x638, 0x6cc,
0x645, 0x20, 0x62a, 0x631, 0x6a9, 0x63a, 0x631, 0x646, 0x627, 0x637, 0x6c1, 0x6af,
0x648, 0x627, 0x688, 0x6cc, 0x644, 0x648, 0x67e, 0x6af, 0x648, 0x627, 0x679, 0x6d2,
0x20, 0x645, 0x627, 0x644, 0x627, 0x6af, 0x648, 0x622, 0x6cc, 0x627, 0x6a9, 0x648,
0x626, 0x644, 0x6af, 0x6cc, 0x627, 0x646, 0x627, 0x6c1, 0x6cc, 0x644, 0x6cc, 0x641,
0x6cc, 0x6a9, 0x633, 0x6c1, 0x648, 0x627, 0x646, 0x627, 0x6c1, 0x631, 0x645, 0x648,
0x633, 0x6cc, 0x644, 0x648, 0x627, 0x646, 0x688, 0x6cc, 0x627, 0x646, 0x627, 0x67e,
0x648, 0x644, 0x633, 0x6a9, 0x646, 0x648, 0x6a9, 0x633, 0x60c, 0x20, 0x627, 0x646,
0x688, 0x6cc, 0x627, 0x646, 0x627, 0x645, 0x631, 0x6cc, 0x646, 0x6af, 0x648, 0x60c,
0x20, 0x627, 0x646, 0x688, 0x6cc, 0x627, 0x646, 0x627, 0x67e, 0x6cc, 0x679, 0x631,
0x632, 0x628, 0x631, 0x6af, 0x60c, 0x20, 0x627, 0x646, 0x688, 0x6cc, 0x627, 0x646,
0x627, 0x679, 0x6cc, 0x644, 0x20, 0x633, 0x679, 0x6cc, 0x60c, 0x20, 0x627, 0x646,
0x688, 0x6cc, 0x627, 0x646, 0x627, 0x648, 0x6cc, 0x648, 0x6d2, 0x60c, 0x20, 0x627,
0x646, 0x688, 0x6cc, 0x627, 0x646, 0x627, 0x648, 0x646, 0x633, 0x6cc, 0x646, 0x6cc,
0x632, 0x60c, 0x20, 0x627, 0x646, 0x688, 0x6cc, 0x627, 0x646, 0x627, 0x648, 0x6cc,
0x646, 0x627, 0x645, 0x6cc, 0x6a9, 0x60c, 0x20, 0x627, 0x646, 0x688, 0x6cc, 0x627,
0x646, 0x627, 0x627, 0x646, 0x648, 0x648, 0x650, 0x6a9, 0x627, 0x6cc, 0x6a9, 0x627,
0x644, 0x648, 0x626, 0x679, 0x62c, 0x645, 0x627, 0x626, 0x6cc, 0x6a9, 0x627, 0x62c,
0x648, 0x646, 0x6cc, 0x626, 0x648, 0x644, 0x648, 0x626, 0x633, 0x20, 0x648, 0x6cc,
0x644, 0x6d2, 0x645, 0x648, 0x646, 0x679, 0x6cc, 0x633, 0x6cc, 0x644, 0x648, 0x60c,
0x20, 0x6a9, 0x6cc, 0x646, 0x679, 0x648, 0x6a9, 0x6cc, 0x6a9, 0x631, 0x627, 0x644,
0x6cc, 0x646, 0x688, 0x6cc, 0x62c, 0x6a9, 0x644, 0x648, 0x648, 0x631, 0x20, 0x67e,
0x631, 0x646, 0x633, 0x633, 0x20, 0x6a9, 0x648, 0x627, 0x631, 0x679, 0x631, 0x645,
0x646, 0x627, 0x6af, 0x648, 0x622, 0x645, 0x646, 0x627, 0x624, 0x633, 0x645, 0x627,
0x631, 0x679, 0x6cc, 0x646, 0x6a9, 0x645, 0x6cc, 0x632, 0x679, 0x644, 0x627, 0x646,
0x645, 0x6cc, 0x646, 0x648, 0x645, 0x6cc, 0x646, 0x6cc, 0x645, 0x6cc, 0x631, 0x6cc,
0x688, 0x627, 0x645, 0x6cc, 0x6a9, 0x633, 0x6cc, 0x6a9, 0x648, 0x20, 0x633, 0x679,
0x6cc, 0x645, 0x6cc, 0x6a9, 0x644, 0x6cc, 0x626, 0x648, 0x646, 0x645, 0x648, 0x646,
0x6a9, 0x679, 0x646, 0x645, 0x648, 0x646, 0x679, 0x6cc, 0x631, 0x6cc, 0x645, 0x648,
0x646, 0x679, 0x6cc, 0x20, 0x648, 0x6cc, 0x688, 0x6cc, 0x648, 0x645, 0x648, 0x646,
0x679, 0x633, 0x6cc, 0x631, 0x627, 0x679, 0x646, 0x6cc, 0x648, 0x20, 0x6cc, 0x627,
0x631, 0x6a9, 0x628, 0x6cc, 0x648, 0x644, 0x627, 0x6c1, 0x60c, 0x20, 0x634, 0x645,
0x627, 0x644, 0x6cc, 0x20, 0x688, 0x6a9, 0x648, 0x679, 0x627, 0x648, 0x633, 0x637,
0x60c, 0x20, 0x634, 0x645, 0x627, 0x644, 0x6cc, 0x20, 0x688, 0x6a9, 0x648, 0x679,
0x627, 0x646, 0x6cc, 0x648, 0x20, 0x633, 0x644, 0x6cc, 0x645, 0x60c, 0x20, 0x634,
0x645, 0x627, 0x644, 0x6cc, 0x20, 0x688, 0x6a9, 0x648, 0x679, 0x627, 0x67e, 0x646,
0x627, 0x645, 0x6c1, 0x67e, 0x631, 0x627, 0x645, 0x627, 0x631, 0x6cc, 0x628, 0x648,
0x67e, 0x648, 0x631, 0x679, 0x20, 0x627, 0x648, 0x20, 0x67e, 0x631, 0x646, 0x633,
0x67e, 0x648, 0x631, 0x679, 0x20, 0x622, 0x641, 0x20, 0x627, 0x633, 0x67e, 0x6cc,
0x646, 0x67e, 0x648, 0x631, 0x679, 0x648, 0x20, 0x648, 0x6cc, 0x644, 0x6c1, 0x648,
0x67e, 0x6cc, 0x648, 0x631, 0x679, 0x648, 0x20, 0x631, 0x6cc, 0x6a9, 0x648, 0x67e,
0x646, 0x679, 0x627, 0x20, 0x627, 0x631, 0x6cc, 0x646, 0x627, 0x633, 0x631, 0x6cc,
0x646, 0x6a9, 0x646, 0x20, 0x627, 0x646, 0x644, 0x6cc, 0x679, 0x631, 0x6cc, 0x633,
0x627, 0x626, 0x641, 0x631, 0x6cc, 0x632, 0x648, 0x644, 0x6cc, 0x648, 0x679, 0x631,
0x6cc, 0x626, 0x648, 0x20, 0x628, 0x631, 0x6cc, 0x646, 0x6a9, 0x648, 0x633, 0x646,
0x679, 0x627, 0x631, 0x6cc, 0x646, 0x633, 0x646, 0x679, 0x6cc, 0x627, 0x6af, 0x648,
0x633, 0x627, 0x646, 0x62a, 0x648, 0x20, 0x688, 0x648, 0x645, 0x646, 0x6af, 0x648,
0x627, 0x633, 0x6a9, 0x648, 0x631, 0x632, 0x20, 0x628, 0x627, 0x626, 0x6cc, 0x20,
0x633, 0x646, 0x688, 0x633, 0x6cc, 0x679, 0x6a9, 0x627, 0x633, 0x6cc, 0x646, 0x679,
0x20, 0x628, 0x631, 0x62a, 0x6be, 0x6cc, 0x644, 0x645, 0x6cc, 0x633, 0x6cc, 0x646,
0x679, 0x20, 0x62c, 0x627, 0x646, 0x632, 0x633, 0x6cc, 0x646, 0x679, 0x20, 0x6a9,
0x679, 0x633, 0x633, 0x6cc, 0x646, 0x679, 0x20, 0x644, 0x648, 0x633, 0x6cc, 0x627,
0x633, 0x6cc, 0x646, 0x679, 0x20, 0x62a, 0x6be, 0x627, 0x645, 0x633, 0x633, 0x6cc,
0x646, 0x679, 0x20, 0x648, 0x646, 0x633, 0x646, 0x679, 0x633, 0x648, 0x650, 0x641,
0x679, 0x20, 0x6a9, 0x631, 0x646, 0x679, 0x679, 0x6cc, 0x6af, 0x648, 0x633, 0x6cc,
0x6af, 0x627, 0x644, 0x67e, 0x6d2, 0x62a, 0x6be, 0x648, 0x644, 0x648, 0x62a, 0x6cc,
0x62c, 0x648, 0x622, 0x646, 0x627, 0x679, 0x648, 0x631, 0x646, 0x679, 0x648, 0x679,
0x648, 0x631, 0x679, 0x648, 0x644, 0x627, 0x648, 0x6c1, 0x627, 0x626, 0x679, 0x20,
0x6c1, 0x627, 0x631, 0x633, 0x648, 0x646, 0x651, 0x6cc, 0x67e, 0x6cc, 0x6af, 0x6cc,
0x6a9, 0x648, 0x679, 0x6cc, 0x679, 0x688, 0x6cc, 0x648, 0x633, 0x688, 0x648, 0x645,
0x648, 0x646, 0x679, 0x20, 0x688, 0x6cc, 0x20, 0x627, 0x631, 0x648, 0x6cc, 0x644,
0x6d2, 0x645, 0x6cc, 0x6a9, 0x648, 0x627, 0x631, 0x6cc, 0x645, 0x627, 0x624, 0x633,
0x646, 0x645, 0x6cc, 0x6a9, 0x20, 0x645, 0x631, 0x688, 0x648, 0x67e, 0x644, 0x645,
0x6cc, 0x631, 0x679, 0x631, 0x648, 0x644, 0x644, 0x627, 0x646, 0x6af, 0x20, 0x627,
0x6cc, 0x626, 0x631, 0x20, 0x628, 0x6cc, 0x646, 0x627, 0x644, 0x645, 0x627, 0x679,
0x6cc, 0x627, 0x645, 0x651, 0x627, 0x646, 0x627, 0x646, 0x6cc, 0x62f, 0x631, 0x627,
0x6a9, 0x62a, 0x627, 0x624, 0x627, 0x6a9, 0x679, 0x648, 0x628, 0x627, 0x634, 0x63a,
0x628, 0x62a, 0x622, 0x62a, 0x6cc, 0x631, 0x627, 0x624, 0x628, 0x631, 0x646, 0x627,
0x644, 0x688, 0x6be, 0x627, 0x6a9, 0x6c1, 0x688, 0x644, 0x6cc, 0x62f, 0x628, 0x626,
0x6cc, 0x62f, 0x648, 0x634, 0x627, 0x646, 0x628, 0x6d2, 0x641, 0x627, 0x645, 0x627,
0x6af, 0x648, 0x633, 0x679, 0x627, 0x6c1, 0x648, 0x20, 0x686, 0x6cc, 0x20, 0x645,
0x646, 0x6c1, 0x20, 0x633, 0x679, 0x6cc, 0x6c1, 0x648, 0x648, 0x627, 0x631, 0x688,
0x627, 0x631, 0x6a9, 0x62a, 0x633, 0x6a9, 0x62c, 0x6a9, 0x627, 0x631, 0x62a, 0x6c1,
0x62c, 0x6d2, 0x20, 0x67e, 0x648, 0x631, 0x6c1, 0x6a9, 0x6cc, 0x645, 0x686, 0x679,
0x6a9, 0x627, 0x6a9, 0x627, 0x679, 0x6be, 0x645, 0x646, 0x688, 0x648, 0x62e, 0x646,
0x62f, 0x6cc, 0x6af, 0x627, 0x6a9, 0x648, 0x644, 0x6a9, 0x627, 0x62a, 0x627, 0x6a9,
0x631, 0x6cc, 0x633, 0x646, 0x648, 0x6cc, 0x627, 0x631, 0x633, 0x6a9, 0x6a9, 0x648,
0x627, 0x644, 0x627, 0x20, 0x644, 0x645, 0x67e, 0x648, 0x631, 0x6a9, 0x6cc, 0x648,
0x686, 0x646, 0x6af, 0x645, 0x6a9, 0x627, 0x624, 0x645, 0x6cc, 0x6af, 0x6cc, 0x62f,
0x646, 0x645, 0x6a9, 0x627, 0x633, 0x631, 0x646, 0x6a9, 0x648, 0x633, 0x6cc, 0x627,
0x646, 0x648, 0x648, 0x648, 0x6a9, 0x6cc, 0x648, 0x632, 0x646, 0x6cc, 0x633, 0x6a9,
0x67e, 0x648, 0x646, 0x679, 0x6cc, 0x627, 0x646, 0x6a9, 0x67e, 0x6cc, 0x648, 0x646,
0x6af, 0x20, 0x6cc, 0x627, 0x646, 0x6af, 0x6a9, 0x648, 0x633, 0x62a, 0x627, 0x646,
0x6d2, 0x6a9, 0x6cc, 0x632, 0x6cc, 0x644, 0x648, 0x631, 0x688, 0x627, 0x633, 0x62e,
0x627, 0x644, 0x6cc, 0x646, 0x633, 0x6cc, 0x626, 0x648, 0x644, 0x633, 0x631, 0x6c1,
0x62f, 0x646, 0x6cc, 0x6a9, 0x648, 0x644, 0x6cc, 0x645, 0x633, 0x6a9, 0x62a, 0x627,
0x626, 0x67e, 0x6d2, 0x637, 0x628, 0x644, 0x6cc, 0x633, 0x6cc, 0x62a, 0x6c1, 0x631,
0x627, 0x646, 0x62a, 0x6be, 0x645, 0x67e, 0x648, 0x679, 0x627, 0x645, 0x633, 0x6a9,
0x627, 0x648, 0x644, 0x627, 0x646, 0x20, 0x628, 0x627, 0x62a, 0x627, 0x631, 0x6cc,
0x648, 0x631, 0x648, 0x645, 0x6a9, 0x6cc, 0x627, 0x648, 0x633, 0x62a, 0x2d, 0x646,
0x6cc, 0x631, 0x627, 0x648, 0x6cc, 0x646, 0x679, 0x6cc, 0x627, 0x646, 0x627, 0x648,
0x644, 0x627, 0x62f, 0x6cc, 0x20, 0x648, 0x648, 0x633, 0x62a, 0x6a9, 0x6cc, 0x6a9,
0x648, 0x62a, 0x633, 0x6a9, 0x6cc, 0x6a9, 0x627, 0x679, 0x6cc, 0x631, 0x650, 0x646,
0x628, 0x631, 0x6af, 0x6cc, 0x631, 0x6cc, 0x648, 0x627, 0x646, 0x627, 0x632, 0x648,
0x631, 0x6cc, 0x633, 0x6a9, 0x6cc, 0x67e, 0x20, 0x648, 0x631, 0x688, 0x6cc, 0x645,
0x688, 0x6cc, 0x626, 0x631, 0x627, 0x631, 0x6cc, 0x6a9, 0x62c, 0x627, 0x648, 0x6a9,
0x633, 0x6cc, 0x646, 0x679, 0x20, 0x6c1, 0x6cc, 0x644, 0x6cc, 0x646, 0x627, 0x627,
0x633, 0x679, 0x6cc, 0x646, 0x644, 0x6d2, 0x627, 0x6cc, 0x688, 0x6cc, 0x644, 0x6cc,
0x688, 0x628, 0x631, 0x648, 0x6a9, 0x646, 0x20, 0x6c1, 0x650, 0x644, 0x688, 0x627,
0x631, 0x648, 0x646, 0x627, 0x6cc, 0x648, 0x6a9, 0x644, 0x627, 0x6c1, 0x648, 0x628,
0x627, 0x631, 0x679, 0x644, 0x650, 0x646, 0x688, 0x645, 0x6cc, 0x646, 0x644, 0x627,
0x631, 0x688, 0x20, 0x6c1, 0x648, 0x648, 0x6d2, 0x67e, 0x631, 0x62a, 0x6be, 0x633,
0x688, 0x646, 0x6cc, 0x627, 0x6cc, 0x645, 0x633, 0x679, 0x631, 0x688, 0x645, 0x627,
0x646, 0x688, 0x648, 0x631, 0x627, 0x627, 0x633, 0x62a, 0x631, 0x627, 0x62e, 0x627,
0x646, 0x627, 0x6cc, 0x62a, 0x6be, 0x646, 0x632, 0x628, 0x631, 0x6cc, 0x679, 0x650,
0x633, 0x644, 0x627, 0x648, 0x627, 0x628, 0x62e, 0x627, 0x631, 0x633, 0x679, 0x628,
0x688, 0x627, 0x67e, 0x633, 0x679, 0x628, 0x632, 0x646, 0x62c, 0x646, 0x686, 0x6cc,
0x633, 0x6cc, 0x646, 0x627, 0x624, 0x688, 0x628, 0x644, 0x646, 0x6c1, 0x6cc, 0x644,
0x633, 0x646, 0x6a9, 0x6cc, 0x6a9, 0x627, 0x644, 0x64a, 0x646, 0x64a, 0x646, 0x63a,
0x631, 0x627, 0x62f, 0x644, 0x6af, 0x698, 0x645, 0x628, 0x631, 0x6af, 0x645, 0x6cc,
0x688, 0x631, 0x688, 0x645, 0x6cc, 0x631, 0x6cc, 0x6c1, 0x627, 0x645, 0x67e, 0x6cc,
0x631, 0x633, 0x67e, 0x648, 0x688, 0x6af, 0x648, 0x631, 0x633, 0x6cc, 0x627, 0x633,
0x627, 0x646, 0x20, 0x645, 0x627, 0x631, 0x6cc, 0x627, 0x646, 0x648, 0x633, 0x6cc,
0x631, 0x627, 0x679, 0x648, 0x627, 0x633, 0x6a9, 0x648, 0x67e, 0x62c, 0x6d2, 0x635,
0x648, 0x641, 0x6cc, 0x6c1, 0x627, 0x633, 0x679, 0x627, 0x6a9, 0x20, 0x6c1, 0x648,
0x645, 0x679, 0x627, 0x644, 0x646, 0x679, 0x6cc, 0x631, 0x627, 0x646, 0x6cc, 0x627,
0x644, 0x6cc, 0x627, 0x646, 0x648, 0x633, 0x6a9, 0x648, 0x6cc, 0x688, 0x648, 0x632,
0x648, 0x627, 0x679, 0x6cc, 0x6a9, 0x646, 0x648, 0x650, 0x644, 0x646, 0x6cc, 0x626,
0x633, 0x632, 0x6cc, 0x6af, 0x631, 0x6cc, 0x628, 0x632, 0x6cc, 0x648, 0x631, 0x62e,
0x627, 0x646, 0x679, 0x627, 0x646, 0x627, 0x646, 0x627, 0x631, 0x6cc, 0x648, 0x648,
0x6a9, 0x631, 0x633, 0x645, 0x633, 0x6a9, 0x631, 0x6af, 0x6cc, 0x648, 0x644, 0x6cc,
0x646, 0x645, 0x627, 0x644, 0x62f, 0x6cc, 0x67e, 0x645, 0x627, 0x631, 0x6cc, 0x634,
0x633, 0x645, 0x627, 0x6cc, 0x648, 0x679, 0x631, 0x6cc, 0x20, 0x6cc, 0x648, 0x646,
0x6cc, 0x646, 0x628, 0x648, 0x6af, 0x6cc, 0x646, 0x648, 0x644, 0x6d2, 0x686, 0x6cc,
0x62a, 0x6be, 0x645, 0x686, 0x6cc, 0x648, 0x6a9, 0x627, 0x6cc, 0x641, 0x6cc, 0x679,
0x641, 0x6a9, 0x627, 0x624, 0x641, 0x648, 0x641, 0x62c, 0x6cc, 0x641, 0x6cc, 0x648,
0x646, 0x627, 0x641, 0x6cc, 0x648, 0x679, 0x6cc, 0x6af, 0x6cc, 0x644, 0x627, 0x67e,
0x6cc, 0x6af, 0x648, 0x633, 0x6af, 0x627, 0x645, 0x628, 0x6cc, 0x626, 0x631, 0x6af,
0x648, 0x627, 0x688, 0x644, 0x20, 0x6a9, 0x6cc, 0x646, 0x627, 0x644, 0x6c1, 0x648,
0x646, 0x648, 0x644, 0x648, 0x644, 0x648, 0x627, 0x6cc, 0x646, 0x688, 0x631, 0x628,
0x631, 0x6cc, 0x6a9, 0x631, 0x6cc, 0x62a, 0x6cc, 0x645, 0x627, 0x679, 0x6cc, 0x6a9,
0x648, 0x633, 0x631, 0x627, 0x626, 0x6cc, 0x645, 0x627, 0x631, 0x6a9, 0x6cc, 0x633,
0x627, 0x633, 0x645, 0x688, 0x648, 0x6d2, 0x646, 0x627, 0x624, 0x631, 0x648, 0x646,
0x624, 0x645, 0x6cc, 0x627, 0x67e, 0x644, 0x627, 0x624, 0x67e, 0x679, 0x6a9, 0x627,
0x626, 0x631, 0x646, 0x67e, 0x648, 0x646, 0x67e, 0x6cc, 0x626, 0x6cc, 0x67e, 0x648,
0x631, 0x679, 0x20, 0x645, 0x648, 0x631, 0x6cc, 0x633, 0x628, 0x6cc, 0x633, 0x627,
0x626, 0x67e, 0x6cc, 0x646, 0x62a, 0x627, 0x6c1, 0x6cc, 0x62a, 0x6cc, 0x679, 0x631,
0x627, 0x648, 0x627, 0x679, 0x648, 0x646, 0x6af, 0x627, 0x679, 0x627, 0x67e, 0x648,
0x648, 0x644, 0x651, 0x6cc, 0x633, 0x6a9, 0x6cc, 0x645, 0x628, 0x631, 0x62c, 0x20,
0x628, 0x6d2, 0x6af, 0x644, 0x6cc, 0x633, 0x20, 0x628, 0x6d2, 0x6af, 0x648, 0x633,
0x20, 0x628, 0x6d2, 0x6af, 0x631, 0x6cc, 0x646, 0x688, 0x20, 0x62a, 0x631, 0x6a9,
0x627, 0x62a, 0x62a, 0x648, 0x642, 0x642, 0x648, 0x631, 0x62a, 0x648, 0x631, 0x645,
0x6cc, 0x62a, 0x62a, 0x6be, 0x648, 0x644, 0x6d2, 0x628, 0x648, 0x688, 0x627, 0x67e,
0x6cc, 0x633, 0x679, 0x633, 0x627, 0x646, 0x2d, 0x62a, 0x648, 0x645, 0x6d0, 0x626,
0x627, 0x633, 0x6c7, 0x646, 0x633, 0x649, 0x64a, 0x648, 0x646, 0x643, 0x6c7, 0x631,
0x627, 0x633, 0x648, 0x633, 0x627, 0x64a, 0x646, 0x649, 0x62a, 0x2d, 0x628, 0x627,
0x631, 0x62a, 0x6be, 0x6d0, 0x644, 0x6d5, 0x645, 0x649, 0x64a, 0x62f, 0x6c7, 0x645,
0x648, 0x646, 0x62a, 0x20, 0x62f, 0x6c7, 0x631, 0x6cb, 0x649, 0x644, 0x644, 0x6d0,
0x62e, 0x648, 0x686, 0x649, 0x645, 0x649, 0x646, 0x20, 0x634, 0x6d5, 0x6be, 0x649,
0x631, 0x649, 0x631, 0x6d0, 0x626, 0x648, 0x646, 0x649, 0x64a, 0x648, 0x646, 0x4a,
0x61, 0x7a, 0x6f, 0x69, 0x72, 0x42, 0x6c, 0x61, 0x6e, 0x74, 0x61, 0x79,
0x72, 0x51, 0x6f, 0x68, 0x69, 0x72, 0x61, 0x44, 0x6f, 0x72, 0x2d, 0x75,
0x73, 0x2d, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x41, 0x6c, 0x2d, 0x41, 0x79,
0x75, 0x6e, 0x58, 0x61, 0x72, 0x61, 0x72, 0x65, 0x59, 0x6f, 0x78, 0x61,
0x6e, 0x6e, 0x65, 0x73, 0x62, 0x75, 0x72, 0x67, 0x4d, 0x6f, 0x67, 0x61,
0x64, 0x69, 0x73, 0x68, 0x6f, 0x4d, 0x6f, 0x6e, 0x72, 0x6f, 0x76, 0x69,
0x79, 0x61, 0x4e, 0x61, 0x79, 0x72, 0x6f, 0x62, 0x69, 0x56, 0x69, 0x6e,
0x64, 0x78, 0x75, 0x6b, 0x41, 0x64, 0x61, 0x6b, 0x20, 0x6f, 0x72, 0x6f,
0x6c, 0x69, 0x41, 0x6e, 0x6b, 0x6f, 0x72, 0x69, 0x6a, 0x42, 0x75, 0x65,
0x6e, 0x6f, 0x73, 0x2d, 0x41, 0x79, 0x72, 0x65, 0x73, 0x4c, 0x61, 0x2d,
0x52, 0x69, 0x6f, 0x78, 0x61, 0x52, 0x69, 0x6f, 0x2d, 0x47, 0x61, 0x6c,
0x79, 0x65, 0x67, 0x6f, 0x73, 0x53, 0x61, 0x6e, 0x2d, 0x58, 0x75, 0x61,
0x6e, 0x55, 0x73, 0x68, 0x75, 0x61, 0x79, 0x61, 0x41, 0x73, 0x75, 0x6e,
0x73, 0x6f, 0x6e, 0x4b, 0x6f, 0x72, 0x61, 0x6c, 0x2d, 0x58, 0x61, 0x72,
0x62, 0x6f, 0x72, 0x42, 0x61, 0x69, 0x79, 0x61, 0x42, 0x61, 0x68, 0x69,
0x79, 0x61, 0x2d, 0x42, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x61, 0x73, 0x4b,
0x65, 0x6d, 0x62, 0x72, 0x69, 0x6a, 0x2d, 0x42, 0x65, 0x79, 0x4b, 0x61,
0x79, 0x65, 0x6e, 0x6e, 0x61, 0x4b, 0x61, 0x79, 0x6d, 0x61, 0x6e, 0x20,
0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x43, 0x68, 0x69, 0x6b,
0x61, 0x67, 0x6f, 0x4b, 0x79, 0x75, 0x72, 0x61, 0x73, 0x61, 0x6f, 0x44,
0x65, 0x6e, 0x6d, 0x61, 0x72, 0x6b, 0x73, 0x78, 0x61, 0x76, 0x6e, 0x45,
0x79, 0x72, 0x75, 0x6e, 0x65, 0x70, 0x65, 0x47, 0x6c, 0x65, 0x79, 0x73,
0x2d, 0x42, 0x65, 0x79, 0x47, 0x75, 0x73, 0x2d, 0x42, 0x65, 0x79, 0x47,
0x75, 0x61, 0x79, 0x61, 0x6b, 0x69, 0x6c, 0x47, 0x61, 0x76, 0x61, 0x6e,
0x61, 0x45, 0x72, 0x6d, 0x6f, 0x73, 0x69, 0x6c, 0x6c, 0x6f, 0x50, 0x69,
0x74, 0x65, 0x72, 0x73, 0x62, 0x65, 0x72, 0x67, 0x2c, 0x20, 0x49, 0x6e,
0x64, 0x69, 0x61, 0x6e, 0x61, 0x56, 0x69, 0x76, 0x65, 0x79, 0x2c, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x56, 0x69, 0x6e, 0x73, 0x65,
0x6e, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x56,
0x69, 0x6e, 0x61, 0x6d, 0x61, 0x6b, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69,
0x61, 0x6e, 0x61, 0x4a, 0x75, 0x6e, 0x6f, 0x4c, 0x75, 0x69, 0x73, 0x76,
0x69, 0x6c, 0x6c, 0x4d, 0x6f, 0x6e, 0x74, 0x69, 0x73, 0x65, 0x6c, 0x6c,
0x6f, 0x2c, 0x20, 0x4b, 0x65, 0x6e, 0x74, 0x75, 0x6b, 0x6b, 0x69, 0x4b,
0x72, 0x61, 0x6c, 0x65, 0x6e, 0x64, 0x65, 0x79, 0x6b, 0x4c, 0x6f, 0x75,
0x65, 0x72, 0x2d, 0x50, 0x72, 0x69, 0x6e, 0x73, 0x65, 0x73, 0x2d, 0x4b,
0x75, 0x6f, 0x72, 0x74, 0x65, 0x72, 0x4e, 0x79, 0x75, 0x2d, 0x59, 0x6f,
0x72, 0x6b, 0x42, 0x6f, 0x79, 0x6c, 0x61, 0x2c, 0x20, 0x53, 0x68, 0x69,
0x6d, 0x6f, 0x6c, 0x69, 0x79, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x4d, 0x61, 0x72, 0x6b, 0x61, 0x7a, 0x2c, 0x20, 0x53, 0x68, 0x69, 0x6d,
0x6f, 0x6c, 0x69, 0x79, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e,
0x79, 0x75, 0x2d, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x53, 0x68,
0x69, 0x6d, 0x6f, 0x6c, 0x69, 0x79, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74,
0x61, 0x47, 0x6f, 0x74, 0x78, 0x6f, 0x62, 0x4f, 0x78, 0x69, 0x6e, 0x61,
0x67, 0x61, 0x50, 0x6f, 0x72, 0x74, 0x2d, 0x6f, 0x66, 0x2d, 0x53, 0x70,
0x65, 0x79, 0x6e, 0x50, 0x6f, 0x72, 0x74, 0x75, 0x2d, 0x56, 0x65, 0x6c,
0x79, 0x75, 0x52, 0x65, 0x6a, 0x61, 0x79, 0x6e, 0x61, 0x53, 0x61, 0x6e,
0x74, 0x79, 0x61, 0x67, 0x6f, 0x49, 0x74, 0x74, 0x6f, 0x6b, 0x6b, 0x6f,
0x72, 0x74, 0x6f, 0x6f, 0x72, 0x6d, 0x69, 0x75, 0x74, 0x53, 0x65, 0x6e,
0x74, 0x2d, 0x4c, 0x79, 0x75, 0x73, 0x69, 0x79, 0x61, 0x53, 0x65, 0x6e,
0x74, 0x2d, 0x56, 0x69, 0x6e, 0x73, 0x65, 0x6e, 0x74, 0x53, 0x76, 0x69,
0x66, 0x74, 0x2d, 0x4b, 0x61, 0x72, 0x72, 0x65, 0x6e, 0x74, 0x55, 0x61,
0x79, 0x74, 0x78, 0x6f, 0x72, 0x73, 0x56, 0x69, 0x6e, 0x6e, 0x69, 0x70,
0x65, 0x67, 0x44, 0x79, 0x75, 0x6d, 0x6f, 0x6e, 0x2d, 0x64, 0x2019, 0x59,
0x75, 0x72, 0x76, 0x69, 0x6c, 0x4c, 0x6f, 0x6e, 0x67, 0x79, 0x69, 0x72,
0x41, 0x64, 0x61, 0x6e, 0x41, 0x6d, 0x6d, 0x6f, 0x6e, 0x4f, 0x71, 0x74,
0x6f, 0x76, 0x4f, 0x71, 0x74, 0x6f, 0x2018, 0x62, 0x61, 0x41, 0x73, 0x68,
0x78, 0x6f, 0x62, 0x6f, 0x64, 0x42, 0x61, 0x67, 0x2018, 0x64, 0x6f, 0x64,
0x42, 0x61, 0x68, 0x72, 0x61, 0x79, 0x6e, 0x42, 0x6f, 0x6b, 0x75, 0x44,
0x61, 0x6d, 0x61, 0x73, 0x68, 0x71, 0x47, 0x2018, 0x61, 0x7a, 0x6f, 0x58,
0x65, 0x76, 0x72, 0x6f, 0x6e, 0x58, 0x6f, 0x73, 0x68, 0x69, 0x6d, 0x69,
0x6e, 0x58, 0x6f, 0x76, 0x64, 0x4a, 0x61, 0x79, 0x70, 0x75, 0x72, 0x51,
0x75, 0x64, 0x64, 0x75, 0x73, 0x51, 0x6f, 0x62, 0x75, 0x6c, 0x58, 0x61,
0x6e, 0x64, 0x69, 0x67, 0x61, 0x51, 0x75, 0x76, 0x61, 0x79, 0x74, 0x51,
0x6f, 0x2bb, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x79, 0x51, 0x69, 0x7a, 0x69,
0x6c, 0x6f, 0x2018, 0x72, 0x64, 0x61, 0x41, 0x72, 0x2d, 0x52, 0x69, 0x79,
0x6f, 0x64, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x71, 0x61, 0x6e, 0x64, 0x53,
0x68, 0x61, 0x6e, 0x78, 0x61, 0x79, 0x54, 0x61, 0x79, 0x70, 0x65, 0x79,
0x54, 0x65, 0x68, 0x72, 0x6f, 0x6e, 0x55, 0x72, 0x75, 0x6d, 0x63, 0x68,
0x69, 0x41, 0x7a, 0x6f, 0x72, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61,
0x72, 0x69, 0x42, 0x65, 0x72, 0x6d, 0x75, 0x64, 0x61, 0x20, 0x6f, 0x72,
0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x20,
0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x4b, 0x61, 0x62, 0x6f,
0x2d, 0x56, 0x65, 0x72, 0x64, 0x65, 0x46, 0x61, 0x72, 0x65, 0x72, 0x20,
0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x4d, 0x61, 0x64, 0x65,
0x79, 0x72, 0x61, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x69, 0x4a, 0x61, 0x6e,
0x75, 0x62, 0x69, 0x79, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x79,
0x61, 0x4d, 0x75, 0x71, 0x61, 0x64, 0x64, 0x61, 0x73, 0x20, 0x59, 0x65,
0x6c, 0x65, 0x6e, 0x61, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x69, 0x42, 0x72,
0x6f, 0x6b, 0x65, 0x6e, 0x2d, 0x58, 0x69, 0x6c, 0x6c, 0x45, 0x76, 0x6b,
0x6c, 0x61, 0x58, 0x6f, 0x62, 0x61, 0x72, 0x74, 0x4c, 0x6f, 0x72, 0x64,
0x2d, 0x58, 0x61, 0x75, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x69, 0x42, 0x72,
0x79, 0x75, 0x73, 0x73, 0x65, 0x6c, 0x42, 0x75, 0x64, 0x61, 0x70, 0x65,
0x73, 0x68, 0x74, 0x42, 0x79, 0x75, 0x7a, 0x69, 0x6e, 0x67, 0x65, 0x6e,
0x4b, 0x69, 0x73, 0x68, 0x69, 0x6e, 0x79, 0x6f, 0x76, 0x58, 0x65, 0x6c,
0x73, 0x69, 0x6e, 0x6b, 0x69, 0x4d, 0x65, 0x6e, 0x20, 0x6f, 0x72, 0x6f,
0x6c, 0x69, 0x4c, 0x79, 0x75, 0x6b, 0x73, 0x65, 0x6d, 0x62, 0x75, 0x72,
0x67, 0x4d, 0x61, 0x72, 0x69, 0x79, 0x65, 0x78, 0x61, 0x6d, 0x6e, 0x50,
0x6f, 0x64, 0x67, 0x6f, 0x72, 0x69, 0x74, 0x73, 0x61, 0x56, 0x65, 0x6e,
0x61, 0x56, 0x61, 0x72, 0x73, 0x68, 0x61, 0x76, 0x61, 0x53, 0x79, 0x75,
0x72, 0x69, 0x78, 0x52, 0x6f, 0x6a, 0x64, 0x65, 0x73, 0x74, 0x76, 0x6f,
0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x69, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x20,
0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x4b, 0x6f, 0x6d, 0x6f,
0x72, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x4d, 0x61,
0x6c, 0x64, 0x69, 0x76, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72,
0x69, 0x4d, 0x61, 0x76, 0x72, 0x69, 0x6b, 0x69, 0x79, 0x4d, 0x61, 0x79,
0x6f, 0x72, 0x6b, 0x61, 0x52, 0x65, 0x79, 0x75, 0x6e, 0x69, 0x6f, 0x6e,
0x43, 0x68, 0x61, 0x74, 0x65, 0x6d, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x69,
0x54, 0x72, 0x75, 0x6b, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72,
0x69, 0x50, 0x61, 0x73, 0x78, 0x61, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x69,
0x47, 0x61, 0x6d, 0x62, 0x79, 0x65, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x69,
0x45, 0x6e, 0x64, 0x65, 0x72, 0x62, 0x65, 0x72, 0x69, 0x20, 0x6f, 0x72,
0x6f, 0x6c, 0x69, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x7a, 0x20, 0x6f, 0x72,
0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x4d, 0x69, 0x64, 0x76, 0x65, 0x79,
0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x50, 0x6f, 0x6e,
0x70, 0x65, 0x69, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x69, 0x54, 0x61, 0x69,
0x74, 0x69, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x69, 0x55, 0x65, 0x79, 0x6b,
0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x69, 0x4c, 0x6f, 0x77, 0x65, 0x72, 0x20,
0x50, 0x72, 0x69, 0x6e, 0x63, 0x65, 0x2bc, 0x73, 0x20, 0x51, 0x75, 0x61,
0x72, 0x74, 0x65, 0x72, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20,
0x42, 0x1eaf, 0x63, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x43, 0x65,
0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x42, 0x1eaf, 0x63, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65,
0x6d, 0x2c, 0x20, 0x42, 0x1eaf, 0x63, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74,
0x61, 0x54, 0x50, 0x20, 0x48, 0x1ed3, 0x20, 0x43, 0x68, 0xed, 0x20, 0x4d,
0x69, 0x6e, 0x68, 0x48, 0x1ed3, 0x6e, 0x67, 0x20, 0x4b, 0xf4, 0x6e, 0x67,
0x4d, 0x61, 0x20, 0x43, 0x61, 0x6f, 0x42, 0xec, 0x6e, 0x68, 0x20, 0x4e,
0x68, 0x1b0, 0x1ee1, 0x6e, 0x67, 0x54, 0x68, 0x1b0, 0x1ee3, 0x6e, 0x67, 0x20,
0x48, 0x1ea3, 0x69, 0x110, 0xe0, 0x69, 0x20, 0x42, 0x1eaf, 0x63, 0x56, 0x69,
0xea, 0x6e, 0x67, 0x20, 0x43, 0x68, 0x103, 0x6e, 0x4e, 0x61, 0x6d, 0x20,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x110, 0x1ea3, 0x6f, 0x20, 0x4d,
0x61, 0x6e, 0x4d, 0xe1, 0x74, 0x2d, 0x78, 0x63, 0x1a1, 0x2d, 0x76, 0x61,
0x44, 0x161, 0x69, 0x62, 0x75, 0x74, 0x69, 0x4b, 0x61, 0x69, 0x6d, 0x61,
0x6e, 0x69, 0x6e, 0x73, 0x6c, 0x61, 0x48, 0x61, 0x77, 0x61, 0x6e, 0x6e,
0x61, 0x52, 0x69, 0x6b, 0x6a, 0x61, 0x77, 0x69, 0x6b, 0x53, 0x69, 0x64,
0x6e, 0x65, 0x69, 0x41, 0x6d, 0x161, 0x74, 0x65, 0x72, 0x64, 0x61, 0x6d,
0x4b, 0x6f, 0x6e, 0x161, 0x74, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x6f, 0x70,
0x65, 0x6c, 0x4b, 0xf6, 0x6e, 0x69, 0x67, 0x73, 0x62, 0xe4, 0x72, 0x67,
0x4c, 0x61, 0x69, 0x62, 0x61, 0x10d, 0x52, 0x65, 0x76, 0x61, 0x6c, 0x57,
0x61, 0x72, 0x161, 0x61, 0x75, 0x5a, 0xfc, 0x72, 0x69, 0x10d, 0x42, 0x61,
0x65, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x61, 0x73, 0x42, 0x61,
0x65, 0x20, 0x43, 0x61, 0x6d, 0x62, 0x72, 0x69, 0x64, 0x67, 0x65, 0x44,
0x69, 0x6e, 0x61, 0x73, 0x20, 0x54, 0x65, 0x6c, 0x6c, 0x2c, 0x20, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x44, 0x69, 0x6e, 0x61, 0x73, 0x20,
0x4d, 0x65, 0x63, 0x73, 0x69, 0x63, 0x6f, 0x45, 0x66, 0x72, 0x6f, 0x67,
0x20, 0x4e, 0x65, 0x77, 0x79, 0x64, 0x64, 0x42, 0x65, 0x75, 0x6c, 0x61,
0x68, 0x2c, 0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65, 0x64, 0x64, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c,
0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65, 0x64, 0x64, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d,
0x2c, 0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65, 0x64, 0x64, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x44, 0x69, 0x6e, 0x61, 0x73, 0x20, 0x48, 0xf4,
0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x4a, 0x65, 0x72,
0x77, 0x73, 0x61, 0x6c, 0x65, 0x6d, 0x59, 0x72, 0x20, 0x59, 0x6e, 0x79,
0x73, 0x6f, 0x65, 0x64, 0x64, 0x20, 0x44, 0x65, 0x64, 0x77, 0x79, 0x64,
0x64, 0x46, 0x66, 0x61, 0x72, 0x6f, 0x44, 0x65, 0x20, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x61, 0x42, 0x72, 0x77, 0x73, 0x65, 0x6c, 0x42, 0x77,
0x63, 0x61, 0x72, 0xe9, 0x73, 0x74, 0x44, 0x75, 0x6c, 0x79, 0x6e, 0x59,
0x6e, 0x79, 0x73, 0x20, 0x79, 0x20, 0x47, 0x61, 0x72, 0x6e, 0x59, 0x6e,
0x79, 0x73, 0x20, 0x4d, 0x61, 0x6e, 0x61, 0x77, 0x43, 0x61, 0x65, 0x72,
0x67, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x6e, 0x69, 0x6e, 0x4c, 0x6c, 0x75,
0x6e, 0x64, 0x61, 0x69, 0x6e, 0x4c, 0x77, 0x63, 0x73, 0x65, 0x6d, 0x62,
0x77, 0x72, 0x67, 0x52, 0x68, 0x75, 0x66, 0x61, 0x69, 0x6e, 0x59, 0x20,
0x46, 0x61, 0x74, 0x69, 0x63, 0x61, 0x6e, 0x46, 0x69, 0x65, 0x6e, 0x6e,
0x61, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x79, 0x20, 0x4e, 0x61, 0x64, 0x6f,
0x6c, 0x69, 0x67, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x79, 0x20, 0x50, 0x61,
0x73, 0x67, 0x627, 0x644, 0x645, 0x627, 0x62a, 0x647, 0x627, 0x642, 0x62a, 0x627,
0x648, 0x627, 0x642, 0x62a, 0x648, 0x628, 0x647, 0x639, 0x634, 0x642, 0x20, 0x622,
0x628, 0x627, 0x62a, 0x628, 0x6cc, 0x634, 0x6a9, 0x6cc, 0x6a9, 0x6a9, 0x648, 0x644,
0x648, 0x645, 0x628, 0x648, 0x688, 0x627, 0x6a9, 0x627, 0x6cc, 0x648, 0x631, 0x634,
0x644, 0x6cc, 0x645, 0x6a9, 0x679, 0x645, 0x646, 0x688, 0x648, 0x6a9, 0x648, 0x6cc,
0x679, 0x642, 0x6cc, 0x632, 0x6cc, 0x644, 0x648, 0x631, 0x62f, 0x627, 0x62a, 0x627,
0x634, 0x6a9, 0x6cc, 0x646, 0x62a, 0x62a, 0x6cc, 0x628, 0x644, 0x6cc, 0x633, 0x6cc,
0x679, 0x6cc, 0x645, 0x67e, 0x648, 0x627, 0x648, 0x631, 0x645, 0x686, 0x6cc, 0x645,
0x627, 0x644, 0x62f, 0x6cc, 0x641, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f, 0x2d,
0x73, 0x74, 0x61, 0x64, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20,
0x4e, 0x6f, 0x61, 0x72, 0x64, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x4e, 0x6f, 0x61, 0x72,
0x64, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77, 0x20,
0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x4e, 0x6f, 0x61, 0x72, 0x64,
0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x50, 0xf4, 0x72, 0x74, 0x6f,
0x20, 0x56, 0x65, 0x6c, 0x68, 0x6f, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x79,
0x73, 0x6b, 0x65, 0x20, 0x45, 0x69, 0x6c, 0x61, 0x6e, 0x6e, 0x65, 0x6e,
0x53, 0xfb, 0x64, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x50,
0x72, 0x61, 0x61, 0x63, 0x68, 0x46, 0x61, 0x74, 0x69, 0x6b, 0x61, 0x61,
0x6e, 0x73, 0x74, 0xea, 0x64, 0x4b, 0x72, 0x79, 0x73, 0x74, 0x65, 0x69,
0x6c, 0xe2, 0x6e, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x65, 0x69, 0x6c, 0x61,
0x6e, 0x6e, 0x65, 0x6e, 0x50, 0x65, 0x61, 0x73, 0x6b, 0x65, 0x65, 0x69,
0x6c, 0xe2, 0x6e, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x65, 0x69,
0x6c, 0x61, 0x6e, 0x6e, 0x65, 0x6e, 0x45, 0x6e, 0x64, 0x65, 0x72, 0x62,
0x75, 0x72, 0x79, 0x2d, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x4d, 0x61, 0x72,
0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x6e,
0x65, 0x6e, 0x5d0, 0x5b7, 0x5d1, 0x5d9, 0x5d3, 0x5d6, 0x5e9, 0x5d0, 0x5b7, 0x5df,
0x5d0, 0x5b7, 0x5e1, 0x5de, 0x5d0, 0x5b7, 0x5e8, 0x5d0, 0x5b7, 0x5d8, 0x5d5, 0x5e0,
0x5d9, 0x5e1, 0x5d5, 0x5d5, 0x5d9, 0x5e0, 0x5d8, 0x5d4, 0x5d5, 0x5e7, 0x5d4, 0x5d0,
0x20, 0x5d8, 0x5e9, 0x5d9, 0x20, 0x5de, 0x5d9, 0x5df, 0x20, 0x5e9, 0x5d8, 0x5d0,
0x5b8, 0x5d8, 0xec, 0x6c, 0xfa, 0x20, 0x41, 0x64, 0xe1, 0x6b, 0xec, 0xec,
0x6c, 0xfa, 0x20, 0x41, 0x6e, 0x6b, 0x1ecd, 0x301, 0x72, 0xe9, 0xe8, 0x6a,
0xec, 0xec, 0x6c, 0xfa, 0x20, 0x41, 0x6e, 0x67, 0xfa, 0xed, 0x6c, 0xe0,
0xec, 0x6c, 0xfa, 0x20, 0x41, 0x6e, 0x74, 0xed, 0x67, 0xf9, 0xe0, 0xec,
0x6c, 0xfa, 0x20, 0x41, 0x72, 0xfa, 0x62, 0xe1, 0xec, 0x6c, 0xfa, 0x20,
0xe0, 0x74, 0x69, 0x6b, 0x1ecd, 0x6b, 0xe0, 0x6e, 0xec, 0x6c, 0xfa, 0x20,
0x42, 0xe1, 0x68, 0xec, 0x20, 0x42, 0xe1, 0x6e, 0x64, 0x1eb9, 0x301, 0x72,
0xe1, 0x73, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x42, 0xe1, 0x62, 0xe1, 0x64,
0x1ecd, 0x301, 0x1ecd, 0x300, 0x73, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x42, 0x1eb9,
0x300, 0x6c, 0xed, 0xec, 0x73, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x42, 0x6c,
0x61, 0x6e, 0x6b, 0x20, 0x53, 0x61, 0x62, 0x75, 0x6c, 0x1ecd, 0x301, 0x1ecd,
0x300, 0x6e, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x42, 0x1ecd, 0x301, 0x69, 0x73,
0xe8, 0xec, 0x6c, 0xfa, 0x20, 0xf2, 0x6b, 0x75, 0x6e, 0x20, 0x6b, 0x61,
0x6d, 0x62, 0xed, 0x72, 0xed, 0xec, 0x6a, 0xec, 0xec, 0x6c, 0xfa, 0x20,
0x4b, 0x61, 0x6e, 0x63, 0x75, 0x6e, 0x69, 0x6c, 0xfa, 0x20, 0x4b, 0x61,
0x79, 0x6d, 0x61, 0x6e, 0xec, 0x6c, 0xfa, 0x20, 0x43, 0x68, 0x69, 0x63,
0x61, 0x67, 0x6f, 0xec, 0x6c, 0xfa, 0x20, 0x1e62, 0x69, 0x68, 0x75, 0x61,
0x68, 0x75, 0x61, 0xec, 0x6c, 0xfa, 0x20, 0x4b, 0x6f, 0x73, 0x74, 0x61,
0x20, 0x52, 0xed, 0x6b, 0xe0, 0xec, 0x6c, 0xfa, 0x20, 0x4b, 0xed, 0x72,
0x1eb9, 0x73, 0x74, 0x1ecd, 0x301, 0x1ecd, 0x300, 0x6e, 0xf9, 0xec, 0x6c, 0xfa,
0x20, 0x4b, 0x75, 0x72, 0x61, 0x1e63, 0x61, 0x6f, 0xec, 0x6c, 0xfa, 0x20,
0x42, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x6b, 0x73, 0x68, 0x61, 0x6e, 0xec,
0x6c, 0xfa, 0x20, 0x44, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0xec, 0x6c, 0xfa,
0x20, 0x44, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x20, 0x43, 0x72, 0x65, 0x65,
0x6b, 0xec, 0x6c, 0xfa, 0x20, 0x44, 0x65, 0x6e, 0x76, 0x65, 0x72, 0xec,
0x6c, 0xfa, 0x20, 0x44, 0x65, 0x74, 0x72, 0x6f, 0x69, 0x74, 0xec, 0x6c,
0xfa, 0x20, 0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x69, 0x6b, 0x61, 0xec, 0x6c,
0xfa, 0x20, 0x45, 0x64, 0x6d, 0x6f, 0x6e, 0x74, 0x6f, 0x6e, 0xec, 0x6c,
0xfa, 0x20, 0x45, 0x6c, 0x20, 0x53, 0x61, 0x76, 0x61, 0x64, 0x6f, 0x72,
0xec, 0x6c, 0xfa, 0x20, 0x46, 0x6f, 0x72, 0x74, 0x20, 0x4e, 0x65, 0x6c,
0x73, 0x6f, 0x6e, 0xec, 0x6c, 0xfa, 0x20, 0x6f, 0x6d, 0x69, 0x20, 0x47,
0x6c, 0x61, 0x63, 0x65, 0xec, 0x6c, 0xfa, 0x20, 0x6f, 0x6d, 0x69, 0x20,
0x47, 0x6f, 0x6f, 0x73, 0xf9, 0xec, 0x6c, 0xfa, 0x20, 0x47, 0x72, 0x61,
0x6e, 0x64, 0x20, 0x54, 0x75, 0x72, 0x6b, 0xec, 0x6c, 0xfa, 0x20, 0x47,
0x72, 0x65, 0x6e, 0x61, 0x64, 0x61, 0xec, 0x6c, 0xfa, 0x20, 0x47, 0x75,
0x61, 0x64, 0x65, 0x6c, 0x6f, 0x75, 0x70, 0x65, 0xec, 0x6c, 0xfa, 0x20,
0x47, 0x75, 0x61, 0x74, 0x65, 0x6d, 0x61, 0x6c, 0x61, 0xec, 0x6c, 0xfa,
0x20, 0x48, 0x61, 0x6c, 0x69, 0x66, 0xe1, 0x73, 0xec, 0xec, 0x6c, 0xfa,
0x20, 0x48, 0x61, 0x76, 0x61, 0x6e, 0x61, 0xec, 0x6c, 0xfa, 0x20, 0x48,
0x65, 0x72, 0x6d, 0x6f, 0x73, 0x69, 0x6c, 0x6f, 0xec, 0x6c, 0xfa, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x70, 0x6f, 0x6c, 0x69, 0x73,
0xec, 0x6c, 0xfa, 0x20, 0x6e, 0x1ecd, 0x301, 0x73, 0xec, 0xec, 0x6c, 0xfa,
0x20, 0x4d, 0x61, 0x72, 0x1eb9, 0x301, 0x6e, 0x67, 0x6f, 0xec, 0x6c, 0xfa,
0x20, 0x50, 0x65, 0x74, 0x65, 0x72, 0x73, 0x62, 0x75, 0x72, 0x67, 0xec,
0x6c, 0xfa, 0x20, 0x54, 0x65, 0x6c, 0x6c, 0x20, 0x43, 0x69, 0x74, 0x79,
0xec, 0x6c, 0xfa, 0x20, 0x56, 0x65, 0x76, 0x61, 0x79, 0xec, 0x6c, 0xfa,
0x20, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x6e, 0x65, 0x73, 0x20, 0x6e,
0xed, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0xec, 0x6c, 0xfa,
0x20, 0x57, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x6b, 0x20, 0x6e, 0xed, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0xec, 0x6c, 0xfa, 0x20, 0x49,
0x6e, 0x75, 0x76, 0x69, 0x6b, 0xec, 0x6c, 0xfa, 0x20, 0x49, 0x71, 0x61,
0x6c, 0x75, 0x69, 0x74, 0xec, 0x6c, 0xfa, 0x20, 0x4a, 0x61, 0x6d, 0x61,
0x69, 0x6b, 0xe0, 0xec, 0x6c, 0xfa, 0x20, 0x4a, 0x75, 0x6e, 0x75, 0xec,
0x6c, 0xfa, 0x20, 0x4c, 0xfa, 0xed, 0x66, 0xed, 0xec, 0x6c, 0xec, 0xec,
0x6c, 0xfa, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x69, 0x73, 0x1eb9, 0x301, 0x6c,
0xf2, 0xec, 0x6c, 0xfa, 0x20, 0x4b, 0xed, 0x72, 0x61, 0x6c, 0x1eb9, 0x301,
0x6e, 0x64, 0xe1, 0x69, 0x6b, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x4c, 0x6f,
0x73, 0x20, 0x41, 0x6e, 0x67, 0x65, 0x6c, 0x65, 0x73, 0xec, 0x6c, 0xfa,
0x20, 0x4c, 0x6f, 0x77, 0x65, 0x72, 0x20, 0x50, 0x72, 0x69, 0x6e, 0x63,
0x65, 0x2019, 0x73, 0x20, 0x51, 0x75, 0x61, 0x72, 0x74, 0x65, 0x72, 0xec,
0x6c, 0xfa, 0x20, 0x4d, 0x61, 0x6e, 0x61, 0x67, 0x75, 0x61, 0xec, 0x6c,
0xfa, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x67, 0x1ecd, 0x301, 0x1ecd, 0x300, 0x74,
0xec, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0xe1, 0x74, 0xed, 0x6e, 0xed, 0x6b,
0xec, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x61, 0x74, 0x61, 0x6d, 0x6f, 0x72,
0x6f, 0x73, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x61, 0x73, 0x61, 0x74,
0x69, 0x61, 0x6e, 0x69, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x65, 0x6e, 0x6f,
0x6d, 0x69, 0x6e, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x65, 0x72, 0x69,
0x64, 0x61, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x65, 0x74, 0x69, 0x6c, 0x61,
0x6b, 0x61, 0x74, 0x69, 0x6c, 0x61, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x1eb9,
0x301, 0x73, 0xed, 0x6b, 0xf2, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x69, 0x6b,
0x75, 0x6c, 0x1ecd, 0x301, 0x6e, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x6f,
0x6e, 0x74, 0x6f, 0x6e, 0x69, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x6f, 0x6e,
0x74, 0x65, 0x72, 0x69, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x6f, 0x6e, 0x73,
0x65, 0x72, 0x61, 0x74, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x4e, 0x61, 0x73,
0x61, 0xf2, 0xec, 0x6c, 0xfa, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x59, 0x6f,
0x72, 0x6b, 0xec, 0x6c, 0xfa, 0x20, 0x4e, 0x6f, 0x6d, 0x69, 0xec, 0x6c,
0xfa, 0x20, 0x42, 0x65, 0x75, 0x6c, 0xe0, 0x20, 0x6e, 0xed, 0x20, 0x4e,
0x6f, 0x72, 0x74, 0x68, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0xec,
0x6c, 0xfa, 0x20, 0x53, 0x65, 0x6e, 0x74, 0x61, 0x20, 0x4e, 0x6f, 0x72,
0x74, 0x68, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0xec, 0x6c, 0xfa,
0x20, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x20, 0x6e,
0x69, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0xec, 0x6c, 0xfa, 0x20, 0x4e, 0x75, 0x75, 0x6b, 0xec, 0x6c,
0xfa, 0x20, 0x4f, 0x6a, 0x69, 0x6e, 0x61, 0x67, 0x61, 0xec, 0x6c, 0xfa,
0x20, 0x50, 0x61, 0x6e, 0x61, 0x6d, 0x61, 0xec, 0x6c, 0xfa, 0x20, 0x46,
0xed, 0x6e, 0xed, 0x73, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x50, 0x6f, 0x72,
0x74, 0x2d, 0x61, 0x75, 0x2d, 0x50, 0x72, 0x69, 0x6e, 0x63, 0x65, 0xec,
0x6c, 0xfa, 0x20, 0x65, 0x74, 0xed, 0x20, 0x6f, 0x6d, 0x69, 0x20, 0x53,
0xed, 0x70, 0xe9, 0xe8, 0x6e, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x50, 0x75,
0x65, 0x72, 0x74, 0x6f, 0x20, 0x52, 0x69, 0x6b, 0x6f, 0xec, 0x6c, 0xfa,
0x20, 0x52, 0x61, 0x6e, 0x6b, 0x69, 0x6e, 0x20, 0x49, 0x6e, 0x6c, 0x65,
0x74, 0xec, 0x6c, 0xfa, 0x20, 0x52, 0x65, 0x67, 0x69, 0x6e, 0x61, 0xec,
0x6c, 0xfa, 0x20, 0x52, 0x65, 0x73, 0x6f, 0x6c, 0xfa, 0x74, 0xec, 0xec,
0x6c, 0xfa, 0x20, 0x53, 0x61, 0x6e, 0x74, 0x6f, 0x20, 0x44, 0x6f, 0x6d,
0x69, 0x67, 0x6f, 0xec, 0x6c, 0xfa, 0x20, 0x49, 0x74, 0x6f, 0x6b, 0x6f,
0x74, 0x6f, 0x6d, 0x69, 0x74, 0x69, 0xec, 0x6c, 0xfa, 0x20, 0x53, 0x69,
0x6b, 0x61, 0xec, 0x6c, 0xfa, 0x20, 0x42, 0x61, 0x74, 0x69, 0x6c, 0x65,
0x6d, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x53, 0x74, 0x20, 0x4a, 0x1ecd, 0x301,
0x1ecd, 0x300, 0x6e, 0xf9, 0xec, 0x6c, 0xfa, 0x20, 0x53, 0x74, 0x20, 0x6b,
0x69, 0x74, 0x69, 0x73, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x53, 0x74, 0x20,
0x4c, 0x75, 0x73, 0x69, 0x61, 0xec, 0x6c, 0xfa, 0x20, 0x53, 0x74, 0x20,
0x54, 0x1ecd, 0x301, 0x6d, 0xe0, 0x73, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x46,
0x69, 0x6e, 0x73, 0x65, 0x6e, 0x74, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x53,
0xfa, 0x66, 0xed, 0x74, 0xf9, 0x20, 0x4b, 0x1ecd, 0x300, 0x72, 0x65, 0x6e,
0x74, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x54, 0x65, 0x67, 0x75, 0x73, 0x69,
0x67, 0x61, 0x70, 0xe0, 0xec, 0x6c, 0xfa, 0x20, 0x54, 0x75, 0x6c, 0xe8,
0xec, 0x6c, 0xfa, 0x20, 0x54, 0xed, 0x6a, 0xfa, 0x61, 0x6e, 0x61, 0xec,
0x6c, 0xfa, 0x20, 0x54, 0x6f, 0x72, 0x6f, 0x6e, 0x74, 0x6f, 0xec, 0x6c,
0xfa, 0x20, 0x54, 0x6f, 0x74, 0x6f, 0x6c, 0x61, 0xec, 0x6c, 0xfa, 0x20,
0x56, 0x61, 0x6e, 0x6b, 0x75, 0x76, 0x61, 0xec, 0x6c, 0xfa, 0x20, 0x57,
0x68, 0x69, 0x74, 0x65, 0x68, 0x6f, 0x73, 0xec, 0xec, 0x6c, 0xfa, 0x20,
0x57, 0x69, 0x6e, 0x69, 0x70, 0x65, 0x67, 0xec, 0xec, 0x6c, 0xfa, 0x20,
0x59, 0x61, 0x6b, 0x75, 0x74, 0x61, 0x49, 0x6c, 0x75, 0x20, 0x48, 0x6f,
0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0xec, 0x6c, 0xfa,
0x20, 0x42, 0x1eb9, 0x300, 0x6d, 0xfa, 0x64, 0xe0, 0xec, 0x6c, 0xfa, 0x20,
0x41, 0x6e, 0x6b, 0x254, 0x301, 0x72, 0xe9, 0xe8, 0x6a, 0xec, 0xec, 0x6c,
0xfa, 0x20, 0xe0, 0x74, 0x69, 0x6b, 0x254, 0x6b, 0xe0, 0x6e, 0xec, 0x6c,
0xfa, 0x20, 0x42, 0xe1, 0x68, 0xec, 0x20, 0x42, 0xe1, 0x6e, 0x64, 0x25b,
0x301, 0x72, 0xe1, 0x73, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x42, 0xe1, 0x62,
0xe1, 0x64, 0x254, 0x301, 0x254, 0x300, 0x73, 0xec, 0xec, 0x6c, 0xfa, 0x20,
0x42, 0x25b, 0x300, 0x6c, 0xed, 0xec, 0x73, 0xec, 0xec, 0x6c, 0xfa, 0x20,
0x42, 0x6c, 0x61, 0x6e, 0x6b, 0x20, 0x53, 0x61, 0x62, 0x75, 0x6c, 0x254,
0x301, 0x254, 0x300, 0x6e, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x42, 0x254, 0x301,
0x69, 0x73, 0xe8, 0xec, 0x6c, 0xfa, 0x20, 0x53, 0x68, 0x69, 0x68, 0x75,
0x61, 0x68, 0x75, 0x61, 0xec, 0x6c, 0xfa, 0x20, 0x4b, 0xed, 0x72, 0x25b,
0x73, 0x74, 0x254, 0x301, 0x254, 0x300, 0x6e, 0xf9, 0xec, 0x6c, 0xfa, 0x20,
0x4b, 0x75, 0x72, 0x61, 0x73, 0x68, 0x61, 0x6f, 0xec, 0x6c, 0xfa, 0x20,
0x6e, 0x254, 0x301, 0x73, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x61, 0x72,
0x25b, 0x301, 0x6e, 0x67, 0x6f, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x6f, 0x6e,
0x74, 0x69, 0x73, 0x25b, 0x301, 0x6c, 0xf2, 0xec, 0x6c, 0xfa, 0x20, 0x4b,
0xed, 0x72, 0x61, 0x6c, 0x25b, 0x301, 0x6e, 0x64, 0xe1, 0x69, 0x6b, 0xec,
0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x67, 0x254, 0x301, 0x254,
0x300, 0x74, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x25b, 0x301, 0x73, 0xed,
0x6b, 0xf2, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x69, 0x6b, 0x75, 0x6c, 0x254,
0x301, 0x6e, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x53, 0x74, 0x20, 0x4a, 0x254,
0x301, 0x254, 0x300, 0x6e, 0xf9, 0xec, 0x6c, 0xfa, 0x20, 0x53, 0x74, 0x20,
0x54, 0x254, 0x301, 0x6d, 0xe0, 0x73, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x53,
0xfa, 0x66, 0xed, 0x74, 0xf9, 0x20, 0x4b, 0x254, 0x300, 0x72, 0x65, 0x6e,
0x74, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x42, 0x25b, 0x300, 0x6d, 0xfa, 0x64,
0xe0, 0x69, 0x2d, 0x41, 0x62, 0x69, 0x64, 0x6a, 0x61, 0x6e, 0x69, 0x2d,
0x41, 0x63, 0x63, 0x72, 0x61, 0x69, 0x2d, 0x41, 0x64, 0x64, 0x69, 0x73,
0x20, 0x41, 0x62, 0x61, 0x62, 0x61, 0x69, 0x2d, 0x41, 0x6c, 0x67, 0x69,
0x65, 0x72, 0x73, 0x69, 0x2d, 0x41, 0x73, 0x6d, 0x61, 0x72, 0x61, 0x69,
0x2d, 0x42, 0x61, 0x6d, 0x61, 0x6b, 0x6f, 0x69, 0x2d, 0x42, 0x61, 0x6e,
0x67, 0x75, 0x69, 0x69, 0x2d, 0x42, 0x61, 0x6e, 0x6a, 0x75, 0x6c, 0x69,
0x2d, 0x42, 0x69, 0x73, 0x73, 0x61, 0x75, 0x69, 0x2d, 0x42, 0x6c, 0x61,
0x6e, 0x74, 0x79, 0x72, 0x65, 0x69, 0x2d, 0x42, 0x72, 0x61, 0x7a, 0x7a,
0x61, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x69, 0x2d, 0x42, 0x75, 0x6a, 0x75,
0x6d, 0x62, 0x75, 0x72, 0x61, 0x69, 0x2d, 0x43, 0x61, 0x69, 0x72, 0x6f,
0x69, 0x2d, 0x43, 0x61, 0x73, 0x61, 0x62, 0x6c, 0x61, 0x6e, 0x63, 0x61,
0x69, 0x2d, 0x43, 0x65, 0x75, 0x74, 0x61, 0x69, 0x2d, 0x43, 0x6f, 0x6e,
0x61, 0x6b, 0x72, 0x79, 0x69, 0x2d, 0x44, 0x61, 0x6b, 0x61, 0x72, 0x69,
0x2d, 0x44, 0x61, 0x72, 0x20, 0x65, 0x73, 0x20, 0x53, 0x61, 0x6c, 0x61,
0x61, 0x6d, 0x69, 0x2d, 0x44, 0x6a, 0x69, 0x62, 0x6f, 0x75, 0x74, 0x69,
0x69, 0x2d, 0x44, 0x6f, 0x75, 0x61, 0x6c, 0x61, 0x69, 0x2d, 0x45, 0x6c,
0x20, 0x41, 0x61, 0x69, 0x75, 0x6e, 0x69, 0x2d, 0x46, 0x72, 0x65, 0x65,
0x74, 0x6f, 0x77, 0x6e, 0x69, 0x2d, 0x47, 0x61, 0x62, 0x6f, 0x72, 0x6f,
0x6e, 0x65, 0x69, 0x2d, 0x48, 0x61, 0x72, 0x61, 0x72, 0x65, 0x69, 0x2d,
0x4a, 0x6f, 0x68, 0x61, 0x6e, 0x6e, 0x65, 0x73, 0x62, 0x75, 0x72, 0x67,
0x69, 0x4a, 0x75, 0x62, 0x61, 0x69, 0x2d, 0x4b, 0x61, 0x6d, 0x70, 0x61,
0x6c, 0x61, 0x69, 0x2d, 0x4b, 0x68, 0x61, 0x72, 0x74, 0x6f, 0x75, 0x6d,
0x69, 0x2d, 0x4b, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x69, 0x2d, 0x4b, 0x69,
0x6e, 0x73, 0x68, 0x61, 0x73, 0x61, 0x69, 0x2d, 0x4c, 0x61, 0x67, 0x6f,
0x73, 0x69, 0x2d, 0x4c, 0x69, 0x62, 0x72, 0x65, 0x76, 0x69, 0x6c, 0x6c,
0x65, 0x69, 0x2d, 0x4c, 0x6f, 0x6d, 0x65, 0x69, 0x2d, 0x4c, 0x75, 0x61,
0x6e, 0x64, 0x61, 0x69, 0x2d, 0x4c, 0x75, 0x62, 0x75, 0x6d, 0x62, 0x61,
0x73, 0x68, 0x69, 0x69, 0x4c, 0x75, 0x73, 0x61, 0x6b, 0x61, 0x69, 0x4d,
0x61, 0x6c, 0x61, 0x62, 0x6f, 0x69, 0x4d, 0x61, 0x70, 0x75, 0x74, 0x6f,
0x69, 0x4d, 0x61, 0x73, 0x65, 0x72, 0x75, 0x69, 0x4d, 0x62, 0x61, 0x62,
0x61, 0x6e, 0x65, 0x69, 0x2d, 0x4d, 0x6f, 0x67, 0x61, 0x64, 0x69, 0x73,
0x68, 0x75, 0x69, 0x2d, 0x4d, 0x6f, 0x6e, 0x72, 0x6f, 0x76, 0x69, 0x61,
0x69, 0x2d, 0x4e, 0x61, 0x69, 0x72, 0x6f, 0x62, 0x69, 0x69, 0x2d, 0x4e,
0x64, 0x6a, 0x61, 0x6d, 0x65, 0x6e, 0x61, 0x69, 0x2d, 0x4e, 0x69, 0x61,
0x6d, 0x65, 0x79, 0x69, 0x2d, 0x4e, 0x6f, 0x75, 0x61, 0x6b, 0x63, 0x68,
0x6f, 0x74, 0x74, 0x69, 0x2d, 0x4f, 0x75, 0x61, 0x67, 0x61, 0x64, 0x6f,
0x75, 0x67, 0x6f, 0x75, 0x69, 0x2d, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x2d,
0x4e, 0x6f, 0x76, 0x6f, 0x69, 0x2d, 0x53, 0xe3, 0x6f, 0x20, 0x54, 0x6f,
0x6d, 0xe9, 0x69, 0x2d, 0x54, 0x72, 0x69, 0x70, 0x6f, 0x6c, 0x69, 0x69,
0x2d, 0x54, 0x75, 0x6e, 0x69, 0x73, 0x69, 0x2d, 0x57, 0x69, 0x6e, 0x64,
0x68, 0x6f, 0x65, 0x6b, 0x69, 0x2d, 0x41, 0x64, 0x61, 0x6b, 0x69, 0x2d,
0x41, 0x6e, 0x63, 0x68, 0x6f, 0x72, 0x61, 0x67, 0x65, 0x69, 0x2d, 0x41,
0x6e, 0x67, 0x75, 0x69, 0x6c, 0x6c, 0x61, 0x69, 0x2d, 0x41, 0x6e, 0x74,
0x69, 0x67, 0x75, 0x61, 0x69, 0x2d, 0x41, 0x72, 0x61, 0x67, 0x75, 0x61,
0x69, 0x6e, 0x61, 0x69, 0x2d, 0x42, 0x75, 0x65, 0x6e, 0x6f, 0x73, 0x20,
0x41, 0x69, 0x72, 0x65, 0x73, 0x69, 0x2d, 0x43, 0x61, 0x74, 0x61, 0x6d,
0x61, 0x72, 0x63, 0x61, 0x69, 0x2d, 0x43, 0x6f, 0x72, 0x64, 0x6f, 0x62,
0x61, 0x69, 0x2d, 0x4a, 0x75, 0x6a, 0x75, 0x79, 0x69, 0x2d, 0x4c, 0x61,
0x20, 0x52, 0x69, 0x6f, 0x6a, 0x61, 0x69, 0x2d, 0x4d, 0x65, 0x6e, 0x64,
0x6f, 0x7a, 0x61, 0x69, 0x2d, 0x52, 0x69, 0x6f, 0x20, 0x47, 0x61, 0x6c,
0x6c, 0x65, 0x67, 0x6f, 0x73, 0x69, 0x2d, 0x53, 0x61, 0x6c, 0x74, 0x61,
0x69, 0x2d, 0x53, 0x61, 0x6e, 0x20, 0x4a, 0x75, 0x61, 0x6e, 0x69, 0x2d,
0x53, 0x61, 0x6e, 0x20, 0x4c, 0x75, 0x69, 0x73, 0x69, 0x2d, 0x54, 0x75,
0x63, 0x75, 0x6d, 0x61, 0x6e, 0x69, 0x2d, 0x55, 0x73, 0x68, 0x75, 0x61,
0x69, 0x61, 0x69, 0x2d, 0x41, 0x72, 0x75, 0x62, 0x61, 0x69, 0x2d, 0x41,
0x73, 0x75, 0x6e, 0x63, 0x69, 0xf3, 0x6e, 0x69, 0x2d, 0x41, 0x74, 0x69,
0x6b, 0x6f, 0x6b, 0x61, 0x6e, 0x69, 0x2d, 0x42, 0x61, 0x68, 0x69, 0x61,
0x69, 0x2d, 0x42, 0x61, 0x68, 0x69, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x64,
0x65, 0x72, 0x61, 0x73, 0x69, 0x2d, 0x42, 0x61, 0x72, 0x62, 0x61, 0x64,
0x6f, 0x73, 0x69, 0x2d, 0x42, 0x65, 0x6c, 0x65, 0x6d, 0x69, 0x2d, 0x42,
0x65, 0x6c, 0x69, 0x7a, 0x65, 0x69, 0x2d, 0x42, 0x6c, 0x61, 0x6e, 0x63,
0x2d, 0x53, 0x61, 0x62, 0x6c, 0x6f, 0x6e, 0x69, 0x2d, 0x42, 0x6f, 0x61,
0x20, 0x56, 0x69, 0x73, 0x74, 0x61, 0x69, 0x2d, 0x42, 0x6f, 0x67, 0x6f,
0x74, 0x61, 0x69, 0x2d, 0x42, 0x6f, 0x69, 0x73, 0x65, 0x69, 0x2d, 0x43,
0x61, 0x6d, 0x62, 0x72, 0x69, 0x64, 0x67, 0x65, 0x20, 0x42, 0x61, 0x79,
0x69, 0x2d, 0x43, 0x61, 0x6d, 0x70, 0x6f, 0x20, 0x47, 0x72, 0x61, 0x6e,
0x64, 0x65, 0x69, 0x2d, 0x43, 0x61, 0x6e, 0x63, 0x75, 0x6e, 0x69, 0x2d,
0x43, 0x61, 0x72, 0x61, 0x63, 0x61, 0x73, 0x69, 0x2d, 0x43, 0x61, 0x79,
0x65, 0x6e, 0x6e, 0x65, 0x69, 0x2d, 0x43, 0x61, 0x79, 0x6d, 0x61, 0x6e,
0x69, 0x2d, 0x43, 0x68, 0x69, 0x63, 0x61, 0x67, 0x6f, 0x69, 0x2d, 0x43,
0x68, 0x69, 0x68, 0x75, 0x61, 0x68, 0x75, 0x61, 0x69, 0x2d, 0x43, 0x6f,
0x73, 0x74, 0x61, 0x20, 0x52, 0x69, 0x63, 0x61, 0x69, 0x2d, 0x43, 0x72,
0x65, 0x73, 0x74, 0x6f, 0x6e, 0x69, 0x2d, 0x43, 0x75, 0x69, 0x61, 0x62,
0x61, 0x69, 0x2d, 0x43, 0x75, 0x72, 0x61, 0xe7, 0x61, 0x6f, 0x69, 0x2d,
0x44, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x6b, 0x73, 0x68, 0x61, 0x76, 0x6e,
0x69, 0x2d, 0x44, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x69, 0x2d, 0x44, 0x61,
0x77, 0x73, 0x6f, 0x6e, 0x20, 0x43, 0x72, 0x65, 0x65, 0x6b, 0x69, 0x2d,
0x44, 0x65, 0x6e, 0x76, 0x65, 0x72, 0x69, 0x2d, 0x44, 0x65, 0x74, 0x72,
0x6f, 0x69, 0x74, 0x69, 0x2d, 0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x69, 0x63,
0x61, 0x69, 0x2d, 0x45, 0x64, 0x6d, 0x6f, 0x6e, 0x74, 0x6f, 0x6e, 0x69,
0x2d, 0x45, 0x69, 0x72, 0x75, 0x6e, 0x65, 0x70, 0x65, 0x69, 0x2d, 0x45,
0x6c, 0x20, 0x53, 0x61, 0x6c, 0x76, 0x61, 0x64, 0x6f, 0x72, 0x69, 0x2d,
0x46, 0x6f, 0x72, 0x74, 0x20, 0x4e, 0x65, 0x6c, 0x73, 0x6f, 0x6e, 0x69,
0x2d, 0x46, 0x6f, 0x72, 0x74, 0x61, 0x6c, 0x65, 0x7a, 0x61, 0x69, 0x2d,
0x47, 0x6c, 0x61, 0x63, 0x65, 0x20, 0x42, 0x61, 0x79, 0x69, 0x2d, 0x47,
0x6f, 0x6f, 0x73, 0x65, 0x20, 0x42, 0x61, 0x79, 0x69, 0x2d, 0x47, 0x72,
0x61, 0x6e, 0x64, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x69, 0x2d, 0x47, 0x72,
0x65, 0x6e, 0x61, 0x64, 0x61, 0x69, 0x2d, 0x47, 0x75, 0x61, 0x64, 0x65,
0x6c, 0x6f, 0x75, 0x70, 0x65, 0x69, 0x2d, 0x47, 0x75, 0x61, 0x74, 0x65,
0x6d, 0x61, 0x6c, 0x61, 0x69, 0x2d, 0x47, 0x75, 0x61, 0x79, 0x61, 0x71,
0x75, 0x69, 0x6c, 0x69, 0x2d, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x69,
0x2d, 0x48, 0x61, 0x6c, 0x69, 0x66, 0x61, 0x78, 0x69, 0x2d, 0x48, 0x61,
0x76, 0x61, 0x6e, 0x61, 0x69, 0x2d, 0x48, 0x65, 0x72, 0x6d, 0x6f, 0x73,
0x69, 0x6c, 0x6c, 0x6f, 0x69, 0x2d, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e,
0x61, 0x70, 0x6f, 0x6c, 0x69, 0x73, 0x69, 0x2d, 0x4b, 0x6e, 0x6f, 0x78,
0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x69, 0x2d, 0x4d,
0x61, 0x72, 0x65, 0x6e, 0x67, 0x6f, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69,
0x61, 0x6e, 0x61, 0x69, 0x2d, 0x50, 0x65, 0x74, 0x65, 0x72, 0x73, 0x62,
0x75, 0x72, 0x67, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61,
0x69, 0x2d, 0x54, 0x65, 0x6c, 0x6c, 0x20, 0x43, 0x69, 0x74, 0x79, 0x2c,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x69, 0x2d, 0x56, 0x65,
0x76, 0x61, 0x79, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61,
0x69, 0x2d, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x6e, 0x65, 0x73, 0x2c,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x69, 0x2d, 0x57, 0x69,
0x6e, 0x61, 0x6d, 0x61, 0x63, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61,
0x6e, 0x61, 0x69, 0x2d, 0x49, 0x6e, 0x75, 0x76, 0x69, 0x6b, 0x69, 0x2d,
0x49, 0x71, 0x61, 0x6c, 0x75, 0x69, 0x74, 0x69, 0x2d, 0x4a, 0x61, 0x6d,
0x61, 0x69, 0x63, 0x61, 0x69, 0x2d, 0x4a, 0x75, 0x6e, 0x65, 0x61, 0x75,
0x69, 0x2d, 0x4c, 0x6f, 0x75, 0x69, 0x73, 0x76, 0x69, 0x6c, 0x6c, 0x65,
0x69, 0x2d, 0x4d, 0x6f, 0x6e, 0x74, 0x69, 0x63, 0x65, 0x6c, 0x6c, 0x6f,
0x2c, 0x20, 0x4b, 0x65, 0x6e, 0x74, 0x75, 0x63, 0x6b, 0x79, 0x69, 0x2d,
0x4b, 0x72, 0x61, 0x6c, 0x65, 0x6e, 0x64, 0x69, 0x6a, 0x6b, 0x69, 0x2d,
0x4c, 0x61, 0x20, 0x50, 0x61, 0x7a, 0x69, 0x2d, 0x4c, 0x69, 0x6d, 0x61,
0x69, 0x2d, 0x4c, 0x6f, 0x73, 0x20, 0x41, 0x6e, 0x67, 0x65, 0x6c, 0x65,
0x73, 0x69, 0x2d, 0x4c, 0x6f, 0x77, 0x65, 0x72, 0x20, 0x50, 0x72, 0x69,
0x6e, 0x63, 0x65, 0x2019, 0x73, 0x20, 0x51, 0x75, 0x61, 0x72, 0x74, 0x65,
0x72, 0x69, 0x2d, 0x4d, 0x61, 0x63, 0x65, 0x69, 0x6f, 0x69, 0x2d, 0x4d,
0x61, 0x6e, 0x61, 0x67, 0x75, 0x61, 0x69, 0x2d, 0x4d, 0x61, 0x6e, 0x61,
0x75, 0x73, 0x69, 0x2d, 0x4d, 0x61, 0x72, 0x69, 0x67, 0x6f, 0x74, 0x69,
0x2d, 0x4d, 0x61, 0x72, 0x74, 0x69, 0x6e, 0x69, 0x71, 0x75, 0x65, 0x69,
0x2d, 0x4d, 0x61, 0x74, 0x61, 0x6d, 0x6f, 0x72, 0x6f, 0x73, 0x69, 0x2d,
0x4d, 0x61, 0x7a, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x69, 0x2d, 0x4d, 0x65,
0x6e, 0x6f, 0x6d, 0x69, 0x6e, 0x65, 0x65, 0x69, 0x2d, 0x4d, 0x65, 0x72,
0x69, 0x64, 0x61, 0x69, 0x2d, 0x4d, 0x65, 0x74, 0x6c, 0x61, 0x6b, 0x61,
0x74, 0x6c, 0x61, 0x69, 0x2d, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f, 0x20,
0x43, 0x69, 0x74, 0x79, 0x69, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x69, 0x2d, 0x4d, 0x6f, 0x6e, 0x63, 0x74, 0x6f, 0x6e, 0x69,
0x2d, 0x4d, 0x6f, 0x6e, 0x74, 0x65, 0x72, 0x72, 0x65, 0x79, 0x69, 0x2d,
0x4d, 0x6f, 0x6e, 0x74, 0x65, 0x76, 0x69, 0x64, 0x65, 0x6f, 0x69, 0x2d,
0x4d, 0x6f, 0x6e, 0x74, 0x73, 0x65, 0x72, 0x72, 0x61, 0x74, 0x69, 0x2d,
0x4e, 0x61, 0x73, 0x73, 0x61, 0x75, 0x69, 0x2d, 0x4e, 0x65, 0x77, 0x20,
0x59, 0x6f, 0x72, 0x6b, 0x69, 0x2d, 0x4e, 0x6f, 0x6d, 0x65, 0x69, 0x2d,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x69, 0x2d, 0x42, 0x65, 0x75,
0x6c, 0x61, 0x68, 0x2c, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x69, 0x2d, 0x43, 0x65, 0x6e, 0x74, 0x65,
0x72, 0x2c, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x69, 0x2d, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c,
0x65, 0x6d, 0x2c, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x69, 0x2d, 0x4e, 0x75, 0x75, 0x6b, 0x69, 0x2d,
0x4f, 0x6a, 0x69, 0x6e, 0x61, 0x67, 0x61, 0x69, 0x2d, 0x50, 0x61, 0x6e,
0x61, 0x6d, 0x61, 0x69, 0x2d, 0x50, 0x61, 0x72, 0x61, 0x6d, 0x61, 0x72,
0x69, 0x62, 0x6f, 0x69, 0x2d, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78,
0x69, 0x2d, 0x50, 0x6f, 0x72, 0x74, 0x2d, 0x61, 0x75, 0x2d, 0x50, 0x72,
0x69, 0x6e, 0x63, 0x65, 0x69, 0x2d, 0x50, 0x6f, 0x72, 0x74, 0x20, 0x6f,
0x66, 0x20, 0x53, 0x70, 0x61, 0x69, 0x6e, 0x69, 0x2d, 0x50, 0x6f, 0x72,
0x74, 0x6f, 0x20, 0x56, 0x65, 0x6c, 0x68, 0x6f, 0x69, 0x2d, 0x50, 0x75,
0x65, 0x72, 0x74, 0x6f, 0x20, 0x52, 0x69, 0x63, 0x6f, 0x69, 0x2d, 0x50,
0x75, 0x6e, 0x74, 0x61, 0x20, 0x41, 0x72, 0x65, 0x6e, 0x61, 0x73, 0x69,
0x2d, 0x52, 0x61, 0x6e, 0x6b, 0x69, 0x6e, 0x20, 0x49, 0x6e, 0x6c, 0x65,
0x74, 0x69, 0x2d, 0x52, 0x65, 0x63, 0x69, 0x66, 0x65, 0x69, 0x2d, 0x52,
0x65, 0x67, 0x69, 0x6e, 0x61, 0x69, 0x2d, 0x52, 0x65, 0x73, 0x6f, 0x6c,
0x75, 0x74, 0x65, 0x69, 0x2d, 0x52, 0x69, 0x6f, 0x20, 0x42, 0x72, 0x61,
0x6e, 0x63, 0x6f, 0x69, 0x2d, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x65,
0x6d, 0x69, 0x2d, 0x53, 0x61, 0x6e, 0x74, 0x69, 0x61, 0x67, 0x6f, 0x69,
0x2d, 0x53, 0x61, 0x6e, 0x74, 0x6f, 0x20, 0x44, 0x6f, 0x6d, 0x69, 0x6e,
0x67, 0x6f, 0x69, 0x2d, 0x53, 0x61, 0x6f, 0x20, 0x50, 0x61, 0x75, 0x6c,
0x6f, 0x69, 0x2d, 0x49, 0x74, 0x74, 0x6f, 0x71, 0x71, 0x6f, 0x72, 0x74,
0x6f, 0x6f, 0x72, 0x6d, 0x69, 0x69, 0x74, 0x69, 0x2d, 0x53, 0x69, 0x74,
0x6b, 0x61, 0x69, 0x2d, 0x53, 0x74, 0x2e, 0x20, 0x42, 0x61, 0x72, 0x74,
0x68, 0xe9, 0x6c, 0x65, 0x6d, 0x79, 0x69, 0x2d, 0x53, 0x74, 0x2e, 0x20,
0x4a, 0x6f, 0x68, 0x6e, 0x2019, 0x73, 0x69, 0x2d, 0x53, 0x74, 0x2e, 0x20,
0x4b, 0x69, 0x74, 0x74, 0x73, 0x49, 0x2d, 0x53, 0x74, 0x2e, 0x20, 0x4c,
0x75, 0x63, 0x69, 0x61, 0x69, 0x2d, 0x53, 0x74, 0x2e, 0x20, 0x54, 0x68,
0x6f, 0x6d, 0x61, 0x73, 0x69, 0x2d, 0x53, 0x74, 0x2e, 0x20, 0x56, 0x69,
0x6e, 0x63, 0x65, 0x6e, 0x74, 0x69, 0x2d, 0x53, 0x77, 0x69, 0x66, 0x74,
0x20, 0x43, 0x75, 0x72, 0x72, 0x65, 0x6e, 0x74, 0x69, 0x2d, 0x54, 0x65,
0x67, 0x75, 0x63, 0x69, 0x67, 0x61, 0x6c, 0x70, 0x61, 0x69, 0x2d, 0x54,
0x68, 0x75, 0x6c, 0x65, 0x69, 0x2d, 0x54, 0x69, 0x6a, 0x75, 0x61, 0x6e,
0x61, 0x69, 0x2d, 0x54, 0x6f, 0x72, 0x6f, 0x6e, 0x74, 0x6f, 0x69, 0x2d,
0x54, 0x6f, 0x72, 0x74, 0x6f, 0x6c, 0x61, 0x69, 0x2d, 0x56, 0x61, 0x6e,
0x63, 0x6f, 0x75, 0x76, 0x65, 0x72, 0x69, 0x2d, 0x57, 0x68, 0x69, 0x74,
0x65, 0x68, 0x6f, 0x72, 0x73, 0x65, 0x69, 0x2d, 0x57, 0x69, 0x6e, 0x6e,
0x69, 0x70, 0x65, 0x67, 0x69, 0x2d, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x61,
0x74, 0x69, 0x2d, 0x43, 0x61, 0x73, 0x65, 0x79, 0x69, 0x2d, 0x44, 0x61,
0x76, 0x69, 0x73, 0x69, 0x2d, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x20,
0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x69, 0x2d, 0x4d,
0x61, 0x63, 0x71, 0x75, 0x61, 0x72, 0x69, 0x65, 0x69, 0x2d, 0x4d, 0x61,
0x77, 0x73, 0x6f, 0x6e, 0x69, 0x2d, 0x4d, 0x63, 0x4d, 0x75, 0x72, 0x64,
0x6f, 0x69, 0x2d, 0x50, 0x61, 0x6c, 0x6d, 0x65, 0x72, 0x69, 0x2d, 0x52,
0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x69, 0x2d, 0x53, 0x79, 0x6f, 0x77,
0x61, 0x69, 0x2d, 0x54, 0x72, 0x6f, 0x6c, 0x6c, 0x69, 0x2d, 0x56, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x69, 0x2d, 0x4c, 0x6f, 0x6e, 0x67, 0x79, 0x65,
0x61, 0x72, 0x62, 0x79, 0x65, 0x6e, 0x69, 0x2d, 0x41, 0x64, 0x65, 0x6e,
0x69, 0x2d, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x69, 0x2d, 0x41, 0x6d,
0x6d, 0x61, 0x6e, 0x69, 0x2d, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x69,
0x2d, 0x41, 0x71, 0x74, 0x61, 0x75, 0x69, 0x2d, 0x41, 0x71, 0x74, 0x6f,
0x62, 0x65, 0x69, 0x2d, 0x41, 0x73, 0x68, 0x67, 0x61, 0x62, 0x61, 0x74,
0x69, 0x2d, 0x42, 0x61, 0x67, 0x68, 0x64, 0x61, 0x64, 0x69, 0x2d, 0x42,
0x61, 0x68, 0x72, 0x61, 0x69, 0x6e, 0x69, 0x2d, 0x42, 0x61, 0x6b, 0x75,
0x69, 0x2d, 0x42, 0x61, 0x6e, 0x67, 0x6b, 0x6f, 0x6b, 0x69, 0x2d, 0x42,
0x61, 0x72, 0x6e, 0x61, 0x75, 0x6c, 0x69, 0x2d, 0x42, 0x65, 0x69, 0x72,
0x75, 0x74, 0x69, 0x2d, 0x42, 0x69, 0x73, 0x68, 0x6b, 0x65, 0x6b, 0x69,
0x2d, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x69, 0x2d, 0x43, 0x68, 0x69,
0x74, 0x61, 0x69, 0x2d, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x6f, 0x69,
0x2d, 0x44, 0x61, 0x6d, 0x61, 0x73, 0x63, 0x75, 0x73, 0x69, 0x2d, 0x44,
0x68, 0x61, 0x6b, 0x61, 0x69, 0x2d, 0x44, 0x69, 0x6c, 0x69, 0x69, 0x2d,
0x44, 0x75, 0x62, 0x61, 0x69, 0x69, 0x2d, 0x44, 0x75, 0x73, 0x68, 0x61,
0x6e, 0x62, 0x65, 0x69, 0x47, 0x61, 0x7a, 0x61, 0x69, 0x2d, 0x48, 0x65,
0x62, 0x72, 0x6f, 0x6e, 0x69, 0x2d, 0x48, 0x6f, 0x20, 0x43, 0x68, 0x69,
0x20, 0x4d, 0x69, 0x6e, 0x68, 0x20, 0x43, 0x69, 0x74, 0x79, 0x69, 0x2d,
0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x69, 0x2d, 0x48,
0x6f, 0x76, 0x64, 0x69, 0x2d, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x69, 0x2d, 0x4a, 0x61, 0x6b, 0x61, 0x72, 0x74, 0x61, 0x69, 0x2d, 0x4a,
0x61, 0x79, 0x61, 0x70, 0x75, 0x72, 0x61, 0x69, 0x2d, 0x4a, 0x65, 0x72,
0x75, 0x73, 0x61, 0x6c, 0x65, 0x6d, 0x69, 0x2d, 0x4b, 0x61, 0x62, 0x75,
0x6c, 0x69, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x6b, 0x61,
0x69, 0x2d, 0x4b, 0x61, 0x72, 0x61, 0x63, 0x68, 0x69, 0x69, 0x2d, 0x4b,
0x61, 0x74, 0x68, 0x6d, 0x61, 0x6e, 0x64, 0x75, 0x69, 0x2d, 0x4b, 0x68,
0x61, 0x6e, 0x64, 0x79, 0x67, 0x61, 0x69, 0x2d, 0x4b, 0x6f, 0x6c, 0x6b,
0x61, 0x74, 0x61, 0x69, 0x2d, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79,
0x61, 0x72, 0x73, 0x6b, 0x69, 0x2d, 0x4b, 0x75, 0x61, 0x6c, 0x61, 0x20,
0x4c, 0x75, 0x6d, 0x70, 0x75, 0x72, 0x69, 0x2d, 0x4b, 0x75, 0x63, 0x68,
0x69, 0x6e, 0x67, 0x69, 0x2d, 0x4b, 0x75, 0x77, 0x61, 0x69, 0x74, 0x69,
0x2d, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x69, 0x2d, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x69, 0x2d, 0x4d, 0x61, 0x6b, 0x61, 0x73, 0x73, 0x61,
0x72, 0x69, 0x2d, 0x4d, 0x61, 0x6e, 0x69, 0x6c, 0x61, 0x69, 0x2d, 0x4d,
0x75, 0x73, 0x63, 0x61, 0x74, 0x69, 0x2d, 0x4e, 0x69, 0x63, 0x6f, 0x73,
0x69, 0x61, 0x69, 0x2d, 0x4e, 0x6f, 0x76, 0x6f, 0x6b, 0x75, 0x7a, 0x6e,
0x65, 0x74, 0x73, 0x6b, 0x69, 0x2d, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x69, 0x2d, 0x4f, 0x6d, 0x73, 0x6b, 0x69,
0x2d, 0x4f, 0x72, 0x61, 0x6c, 0x69, 0x2d, 0x50, 0x68, 0x6e, 0x6f, 0x6d,
0x20, 0x50, 0x65, 0x6e, 0x68, 0x69, 0x2d, 0x50, 0x6f, 0x6e, 0x74, 0x69,
0x61, 0x6e, 0x61, 0x6b, 0x69, 0x2d, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79,
0x61, 0x6e, 0x67, 0x69, 0x2d, 0x51, 0x61, 0x74, 0x61, 0x72, 0x49, 0x2d,
0x4b, 0x6f, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x79, 0x69, 0x2d, 0x51, 0x79,
0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x69, 0x2d, 0x52, 0x69, 0x79,
0x61, 0x64, 0x68, 0x69, 0x2d, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x69, 0x2d, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x6b, 0x61, 0x6e, 0x64,
0x69, 0x2d, 0x53, 0x65, 0x6f, 0x75, 0x6c, 0x69, 0x2d, 0x53, 0x68, 0x61,
0x6e, 0x67, 0x68, 0x61, 0x69, 0x69, 0x2d, 0x53, 0x69, 0x6e, 0x67, 0x61,
0x70, 0x6f, 0x72, 0x65, 0x69, 0x2d, 0x53, 0x72, 0x65, 0x64, 0x6e, 0x65,
0x6b, 0x6f, 0x6c, 0x79, 0x6d, 0x73, 0x6b, 0x69, 0x2d, 0x54, 0x61, 0x69,
0x70, 0x65, 0x69, 0x69, 0x2d, 0x54, 0x61, 0x73, 0x68, 0x6b, 0x65, 0x6e,
0x74, 0x69, 0x2d, 0x54, 0x62, 0x69, 0x6c, 0x69, 0x73, 0x69, 0x69, 0x2d,
0x54, 0x65, 0x68, 0x72, 0x61, 0x6e, 0x69, 0x2d, 0x54, 0x68, 0x69, 0x6d,
0x70, 0x68, 0x75, 0x69, 0x2d, 0x54, 0x6f, 0x6b, 0x79, 0x6f, 0x69, 0x2d,
0x54, 0x6f, 0x6d, 0x73, 0x6b, 0x69, 0x2d, 0x55, 0x6c, 0x61, 0x61, 0x6e,
0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x69, 0x2d, 0x55, 0x72, 0x75, 0x6d,
0x71, 0x69, 0x69, 0x2d, 0x55, 0x73, 0x74, 0x2d, 0x4e, 0x65, 0x72, 0x61,
0x69, 0x2d, 0x56, 0x69, 0x65, 0x6e, 0x74, 0x69, 0x61, 0x6e, 0x65, 0x69,
0x2d, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x69, 0x2d, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x2d, 0x52,
0x61, 0x6e, 0x67, 0x6f, 0x6f, 0x6e, 0x69, 0x2d, 0x59, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x69, 0x2d, 0x59,
0x65, 0x72, 0x65, 0x76, 0x61, 0x6e, 0x69, 0x2d, 0x41, 0x7a, 0x6f, 0x72,
0x65, 0x73, 0x69, 0x2d, 0x42, 0x65, 0x72, 0x6d, 0x75, 0x64, 0x61, 0x69,
0x2d, 0x43, 0x61, 0x6e, 0x61, 0x72, 0x79, 0x69, 0x2d, 0x43, 0x61, 0x70,
0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x69, 0x2d, 0x46, 0x61, 0x72,
0x6f, 0x65, 0x69, 0x2d, 0x4d, 0x61, 0x64, 0x65, 0x69, 0x72, 0x61, 0x69,
0x2d, 0x52, 0x65, 0x79, 0x6b, 0x6a, 0x61, 0x76, 0x69, 0x6b, 0x69, 0x2d,
0x53, 0x6f, 0x75, 0x74, 0x68, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x69, 0x2d, 0x53, 0x74, 0x2e, 0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e,
0x61, 0x69, 0x2d, 0x53, 0x74, 0x61, 0x6e, 0x6c, 0x65, 0x79, 0x69, 0x2d,
0x41, 0x64, 0x65, 0x6c, 0x61, 0x69, 0x64, 0x65, 0x69, 0x2d, 0x42, 0x72,
0x69, 0x73, 0x62, 0x61, 0x6e, 0x65, 0x69, 0x2d, 0x42, 0x72, 0x6f, 0x6b,
0x65, 0x6e, 0x20, 0x48, 0x69, 0x6c, 0x6c, 0x69, 0x2d, 0x44, 0x61, 0x72,
0x77, 0x69, 0x6e, 0x69, 0x2d, 0x45, 0x75, 0x63, 0x6c, 0x61, 0x69, 0x2d,
0x48, 0x6f, 0x62, 0x61, 0x72, 0x74, 0x69, 0x2d, 0x4c, 0x69, 0x6e, 0x64,
0x65, 0x6d, 0x61, 0x6e, 0x69, 0x2d, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x69, 0x2d, 0x4d, 0x65, 0x6c, 0x62, 0x6f, 0x75, 0x72,
0x6e, 0x65, 0x69, 0x2d, 0x50, 0x65, 0x72, 0x74, 0x68, 0x69, 0x2d, 0x53,
0x79, 0x64, 0x6e, 0x65, 0x79, 0x69, 0x2d, 0x41, 0x6d, 0x73, 0x74, 0x65,
0x72, 0x64, 0x61, 0x6d, 0x69, 0x2d, 0x41, 0x6e, 0x64, 0x6f, 0x72, 0x72,
0x61, 0x69, 0x2d, 0x41, 0x73, 0x74, 0x72, 0x61, 0x6b, 0x68, 0x61, 0x6e,
0x69, 0x2d, 0x41, 0x74, 0x68, 0x65, 0x6e, 0x73, 0x69, 0x2d, 0x42, 0x65,
0x6c, 0x67, 0x72, 0x61, 0x64, 0x65, 0x69, 0x2d, 0x42, 0x65, 0x72, 0x6c,
0x69, 0x6e, 0x69, 0x2d, 0x42, 0x72, 0x61, 0x74, 0x69, 0x73, 0x6c, 0x61,
0x76, 0x61, 0x69, 0x2d, 0x42, 0x72, 0x75, 0x73, 0x73, 0x65, 0x6c, 0x73,
0x69, 0x2d, 0x42, 0x75, 0x63, 0x68, 0x61, 0x72, 0x65, 0x73, 0x74, 0x69,
0x2d, 0x42, 0x75, 0x64, 0x61, 0x70, 0x65, 0x73, 0x74, 0x69, 0x2d, 0x42,
0x75, 0x73, 0x69, 0x6e, 0x67, 0x65, 0x6e, 0x69, 0x2d, 0x43, 0x68, 0x69,
0x73, 0x69, 0x6e, 0x61, 0x75, 0x69, 0x2d, 0x43, 0x6f, 0x70, 0x65, 0x6e,
0x68, 0x61, 0x67, 0x65, 0x6e, 0x69, 0x2d, 0x44, 0x75, 0x62, 0x6c, 0x69,
0x6e, 0x69, 0x2d, 0x47, 0x69, 0x62, 0x72, 0x61, 0x6c, 0x74, 0x61, 0x72,
0x69, 0x2d, 0x47, 0x75, 0x65, 0x72, 0x6e, 0x73, 0x65, 0x79, 0x69, 0x2d,
0x48, 0x65, 0x6c, 0x73, 0x69, 0x6e, 0x6b, 0x69, 0x69, 0x2d, 0x49, 0x73,
0x6c, 0x65, 0x20, 0x6f, 0x66, 0x20, 0x4d, 0x61, 0x6e, 0x69, 0x2d, 0x49,
0x73, 0x74, 0x61, 0x6e, 0x62, 0x75, 0x6c, 0x69, 0x2d, 0x4a, 0x65, 0x72,
0x73, 0x65, 0x79, 0x69, 0x2d, 0x4b, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x6e,
0x67, 0x72, 0x61, 0x64, 0x69, 0x2d, 0x4b, 0x69, 0x72, 0x6f, 0x76, 0x69,
0x2d, 0x4b, 0x69, 0x65, 0x76, 0x69, 0x2d, 0x4c, 0x69, 0x73, 0x62, 0x6f,
0x6e, 0x69, 0x2d, 0x4c, 0x6a, 0x75, 0x62, 0x6c, 0x6a, 0x61, 0x6e, 0x61,
0x69, 0x2d, 0x4c, 0x6f, 0x6e, 0x64, 0x6f, 0x6e, 0x69, 0x2d, 0x4c, 0x75,
0x78, 0x65, 0x6d, 0x62, 0x6f, 0x75, 0x72, 0x67, 0x69, 0x2d, 0x4d, 0x61,
0x64, 0x72, 0x69, 0x64, 0x69, 0x2d, 0x4d, 0x61, 0x6c, 0x74, 0x61, 0x69,
0x2d, 0x4d, 0x61, 0x72, 0x69, 0x65, 0x68, 0x61, 0x6d, 0x6e, 0x69, 0x2d,
0x4d, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x2d, 0x4d, 0x6f, 0x6e, 0x61, 0x63,
0x6f, 0x69, 0x2d, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x69, 0x2d, 0x4f,
0x73, 0x6c, 0x6f, 0x69, 0x2d, 0x50, 0x61, 0x72, 0x69, 0x73, 0x69, 0x2d,
0x50, 0x6f, 0x64, 0x67, 0x6f, 0x72, 0x69, 0x63, 0x61, 0x69, 0x2d, 0x50,
0x72, 0x61, 0x67, 0x75, 0x65, 0x69, 0x2d, 0x52, 0x69, 0x67, 0x61, 0x69,
0x2d, 0x52, 0x6f, 0x6d, 0x65, 0x69, 0x2d, 0x53, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x69, 0x2d, 0x53, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6e,
0x6f, 0x69, 0x2d, 0x53, 0x61, 0x72, 0x61, 0x6a, 0x65, 0x76, 0x6f, 0x69,
0x2d, 0x53, 0x61, 0x72, 0x61, 0x74, 0x6f, 0x76, 0x69, 0x2d, 0x53, 0x69,
0x6d, 0x66, 0x65, 0x72, 0x6f, 0x70, 0x6f, 0x6c, 0x69, 0x2d, 0x53, 0x6b,
0x6f, 0x70, 0x6a, 0x65, 0x69, 0x2d, 0x53, 0x6f, 0x66, 0x69, 0x61, 0x69,
0x2d, 0x53, 0x74, 0x6f, 0x63, 0x6b, 0x68, 0x6f, 0x6c, 0x6d, 0x69, 0x2d,
0x54, 0x61, 0x6c, 0x6c, 0x69, 0x6e, 0x6e, 0x69, 0x2d, 0x54, 0x69, 0x72,
0x61, 0x6e, 0x65, 0x69, 0x2d, 0x55, 0x6c, 0x79, 0x61, 0x6e, 0x6f, 0x76,
0x73, 0x6b, 0x69, 0x2d, 0x56, 0x61, 0x64, 0x75, 0x7a, 0x69, 0x2d, 0x56,
0x61, 0x74, 0x69, 0x63, 0x61, 0x6e, 0x69, 0x2d, 0x56, 0x69, 0x65, 0x6e,
0x6e, 0x61, 0x69, 0x2d, 0x56, 0x69, 0x6c, 0x6e, 0x69, 0x75, 0x73, 0x69,
0x2d, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x69, 0x2d,
0x57, 0x61, 0x72, 0x73, 0x61, 0x77, 0x69, 0x2d, 0x5a, 0x61, 0x67, 0x72,
0x65, 0x62, 0x69, 0x2d, 0x5a, 0x75, 0x72, 0x69, 0x63, 0x68, 0x69, 0x2d,
0x41, 0x6e, 0x74, 0x61, 0x6e, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x76, 0x6f,
0x69, 0x2d, 0x43, 0x68, 0x61, 0x67, 0x6f, 0x73, 0x55, 0x6b, 0x68, 0x69,
0x73, 0x69, 0x6d, 0x75, 0x73, 0x69, 0x69, 0x2d, 0x43, 0x6f, 0x63, 0x6f,
0x73, 0x69, 0x2d, 0x43, 0x6f, 0x6d, 0x6f, 0x72, 0x6f, 0x69, 0x2d, 0x4b,
0x65, 0x72, 0x67, 0x75, 0x65, 0x6c, 0x65, 0x6e, 0x69, 0x4d, 0x61, 0x68,
0x65, 0x69, 0x2d, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x73, 0x69,
0x2d, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x69, 0x2d,
0x4d, 0x61, 0x79, 0x6f, 0x74, 0x74, 0x65, 0x69, 0x2d, 0x52, 0xe9, 0x75,
0x6e, 0x69, 0x6f, 0x6e, 0x69, 0x2d, 0x41, 0x70, 0x69, 0x61, 0x69, 0x2d,
0x41, 0x75, 0x63, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x2d, 0x42, 0x6f,
0x75, 0x67, 0x61, 0x69, 0x6e, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x69, 0x2d,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x69, 0x2d, 0x43, 0x68, 0x75,
0x75, 0x6b, 0x69, 0x2d, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x69, 0x2d,
0x45, 0x66, 0x61, 0x74, 0x65, 0x69, 0x2d, 0x46, 0x61, 0x6b, 0x61, 0x6f,
0x66, 0x6f, 0x69, 0x2d, 0x46, 0x69, 0x6a, 0x69, 0x69, 0x2d, 0x46, 0x75,
0x6e, 0x61, 0x66, 0x75, 0x74, 0x69, 0x69, 0x2d, 0x47, 0x61, 0x6c, 0x61,
0x70, 0x61, 0x67, 0x6f, 0x73, 0x69, 0x2d, 0x47, 0x61, 0x6d, 0x62, 0x69,
0x65, 0x72, 0x69, 0x2d, 0x47, 0x75, 0x61, 0x64, 0x61, 0x6c, 0x63, 0x61,
0x6e, 0x61, 0x6c, 0x69, 0x2d, 0x47, 0x75, 0x61, 0x6d, 0x69, 0x2d, 0x48,
0x6f, 0x6e, 0x6f, 0x6c, 0x75, 0x6c, 0x75, 0x69, 0x2d, 0x45, 0x6e, 0x64,
0x65, 0x72, 0x62, 0x75, 0x72, 0x79, 0x69, 0x2d, 0x4b, 0x69, 0x72, 0x69,
0x74, 0x69, 0x6d, 0x61, 0x74, 0x69, 0x69, 0x2d, 0x4b, 0x6f, 0x73, 0x72,
0x61, 0x65, 0x69, 0x2d, 0x4b, 0x77, 0x61, 0x6a, 0x61, 0x6c, 0x65, 0x69,
0x6e, 0x69, 0x2d, 0x4d, 0x61, 0x6a, 0x75, 0x72, 0x6f, 0x69, 0x2d, 0x4d,
0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x69, 0x2d, 0x4d, 0x69,
0x64, 0x77, 0x61, 0x79, 0x69, 0x2d, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x69,
0x2d, 0x4e, 0x69, 0x75, 0x65, 0x69, 0x2d, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x69, 0x2d, 0x4e, 0x6f, 0x75, 0x6d, 0x65, 0x61, 0x69, 0x2d,
0x50, 0x61, 0x67, 0x6f, 0x20, 0x50, 0x61, 0x67, 0x6f, 0x69, 0x2d, 0x50,
0x61, 0x6c, 0x61, 0x75, 0x69, 0x2d, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69,
0x72, 0x6e, 0x69, 0x2d, 0x50, 0x6f, 0x68, 0x6e, 0x70, 0x65, 0x69, 0x69,
0x2d, 0x50, 0x6f, 0x72, 0x74, 0x20, 0x4d, 0x6f, 0x72, 0x65, 0x73, 0x62,
0x79, 0x69, 0x2d, 0x52, 0x61, 0x72, 0x6f, 0x74, 0x6f, 0x6e, 0x67, 0x61,
0x69, 0x2d, 0x53, 0x61, 0x69, 0x70, 0x61, 0x6e, 0x69, 0x2d, 0x54, 0x61,
0x68, 0x69, 0x74, 0x69, 0x69, 0x2d, 0x54, 0x61, 0x72, 0x61, 0x77, 0x61,
0x69, 0x2d, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x74, 0x61, 0x70, 0x75, 0x69,
0x2d, 0x57, 0x61, 0x6b, 0x65, 0x69, 0x2d, 0x57, 0x61, 0x6c, 0x6c, 0x69,
0x73, 0x41, 0x6d, 0x69, 0x6e, 0x6a, 0x61, 0x6e, 0x41, 0x6e, 0x69, 0x73,
0x69, 0x20, 0x41, 0x6d, 0x65, 0x6d, 0x61, 0x41, 0x72, 0x6a, 0xe9, 0x72,
0x41, 0x73, 0x69, 0x6d, 0x1ef9, 0x72, 0x61, 0x4d, 0x61, 0x6d, 0x1ef9, 0x6b,
0x6f, 0x4d, 0x61, 0x67, 0x69, 0x4d, 0x61, 0x6e, 0x6a, 0x75, 0x72, 0x4d,
0x69, 0x73, 0x61, 0x76, 0x4d, 0x72, 0x61, 0x6e, 0x74, 0x79, 0x72, 0x65,
0x4d, 0x72, 0x61, 0x6a, 0x61, 0x76, 0x69, 0x72, 0x65, 0x4d, 0x75, 0x6a,
0x169, 0x6d, 0x75, 0x72, 0x61, 0x128, 0x6e, 0x6b, 0x75, 0x70, 0x72, 0x69,
0x53, 0x65, 0x76, 0x75, 0x74, 0x61, 0x4b, 0x6f, 0x6e, 0x1ef9, 0x6b, 0x72,
0x69, 0x4e, 0x61, 0x6b, 0x61, 0x72, 0x4e, 0x61, 0x72, 0x20, 0x45, 0x6e,
0x68, 0x20, 0x53, 0x61, 0x72, 0x1ef9, 0x6d, 0x4e, 0x68, 0x69, 0x6d, 0x75,
0x74, 0x69, 0x4e, 0x6f, 0x76, 0x61, 0x72, 0x61, 0x45, 0x72, 0x20, 0x41,
0x2019, 0x61, 0x6a, 0x75, 0x6e, 0x1ebc, 0x6d, 0xe3, 0x2d, 0x66, 0x72, 0x69,
0x47, 0x61, 0x6d, 0x6f, 0x72, 0x6f, 0x6e, 0x129, 0x4a, 0x75, 0x76, 0x1ef9,
0x6e, 0x1ebd, 0x6e, 0x68, 0x6d, 0x75, 0x72, 0x67, 0x75, 0x4a, 0x75, 0x6d,
0x61, 0x4b, 0xe3, 0x6d, 0x70, 0x61, 0x72, 0x61, 0x4b, 0x61, 0x72, 0x74,
0x169, 0x6d, 0x4b, 0x69, 0x67, 0x61, 0x72, 0x69, 0x4b, 0x129, 0x73, 0x61,
0x6a, 0x61, 0x52, 0x61, 0x67, 0x6f, 0x52, 0x69, 0x6d, 0x72, 0x65, 0x76,
0x69, 0x72, 0x69, 0x52, 0xf3, 0x6d, 0x1ebd, 0x52, 0x75, 0x76, 0x1ef9, 0x6e,
0x61, 0x52, 0x75, 0x6d, 0x75, 0x6d, 0x61, 0x73, 0x69, 0x52, 0x75, 0x73,
0x61, 0x6b, 0x61, 0x4d, 0x1ef9, 0x72, 0x61, 0x6d, 0x6f, 0x4d, 0x1ef9, 0x70,
0x75, 0x74, 0x75, 0x4d, 0x1ef9, 0x73, 0x65, 0x72, 0x75, 0x4d, 0x129, 0x6d,
0x61, 0x6d, 0x61, 0x6e, 0x1ebd, 0x4d, 0x169, 0x67, 0x61, 0x6e, 0x69, 0x6e,
0x68, 0x73, 0x75, 0x4d, 0xe3, 0x6e, 0x72, 0xf3, 0x76, 0x69, 0x6a, 0x61,
0x4e, 0x1ef9, 0x6a, 0x72, 0xf3, 0x6d, 0x69, 0x4e, 0x69, 0x2019, 0x6e, 0x69,
0x6a, 0x61, 0x6d, 0x1ebd, 0x6e, 0x1ef9, 0x4e, 0x129, 0x61, 0x6d, 0x1ebd, 0x6a,
0x4e, 0xe3, 0x76, 0x61, 0x67, 0x73, 0xf3, 0x74, 0x69, 0x4f, 0x76, 0x61,
0x67, 0x61, 0x6e, 0x6f, 0x67, 0x6f, 0x76, 0x50, 0x6f, 0x72, 0x74, 0x6f,
0x20, 0x54, 0xe3, 0x67, 0x53, 0x1ef9, 0x20, 0x54, 0x75, 0x6d, 0x1ebd, 0x54,
0x72, 0x69, 0x70, 0x6f, 0x72, 0x69, 0x54, 0x75, 0x6e, 0x129, 0x6e, 0x68,
0x56, 0x129, 0x6e, 0x6e, 0x6f, 0xe9, 0x6b, 0x69, 0x41, 0x6e, 0x61, 0x6b,
0x69, 0x1ef8, 0x6b, 0x6f, 0x72, 0x61, 0x67, 0x69, 0x1ef8, 0x67, 0x69, 0x72,
0x61, 0x1ef8, 0x74, 0x69, 0x67, 0x75, 0x76, 0x61, 0x41, 0x72, 0x61, 0x67,
0x75, 0x61, 0x69, 0x6e, 0x1ef9, 0x4d, 0x75, 0x65, 0x6e, 0x169, 0x73, 0x61,
0x69, 0x72, 0x69, 0x4b, 0x61, 0x74, 0x61, 0x6d, 0x1ef9, 0x72, 0x6b, 0x61,
0x4b, 0xf3, 0x72, 0x6e, 0x6f, 0x6d, 0x61, 0x4a, 0x75, 0x6a, 0x75, 0x6a,
0x52, 0x61, 0x20, 0x52, 0x69, 0x6f, 0x68, 0x61, 0x4d, 0x1ebd, 0x6e, 0x6f,
0x73, 0x61, 0x47, 0x61, 0x72, 0x65, 0x67, 0x6f, 0x20, 0x47, 0x6f, 0x6a,
0x53, 0x61, 0x72, 0x74, 0x61, 0x53, 0x1ef9, 0x20, 0x4a, 0x75, 0x76, 0x1ef9,
0x53, 0x1ef9, 0x20, 0x52, 0x75, 0x6a, 0x54, 0x75, 0x6b, 0x75, 0x6d, 0x1ef9,
0x55, 0x73, 0x75, 0x61, 0x6a, 0x61, 0x41, 0x72, 0x75, 0x6d, 0x61, 0x41,
0x73, 0x169, 0x67, 0x73, 0x1ef9, 0x41, 0x74, 0x69, 0x6b, 0x6f, 0x6b, 0x1ef9,
0x6e, 0x4d, 0x61, 0x6a, 0x69, 0x61, 0x4d, 0x61, 0x6a, 0x69, 0x61, 0x20,
0x4d, 0xe1, 0x6e, 0x6e, 0x65, 0x72, 0x61, 0x20, 0x41, 0x67, 0x4d, 0x61,
0x72, 0x6d, 0x61, 0x6e, 0x75, 0x4d, 0x65, 0x72, 0x1ebd, 0x6a, 0x4d, 0x65,
0x72, 0x69, 0x73, 0x65, 0x53, 0x61, 0x6d, 0x72, 0xe3, 0x6e, 0x20, 0x4b,
0x75, 0x70, 0x72, 0x69, 0x56, 0x65, 0x6a, 0x61, 0x20, 0x48, 0xe1, 0x4d,
0x6f, 0x67, 0x6f, 0x74, 0x61, 0x4d, 0x6f, 0x6a, 0x73, 0x65, 0x4b, 0x1ebd,
0x6d, 0x72, 0x69, 0x6e, 0x6a, 0x65, 0x20, 0x4d, 0x65, 0x6a, 0x52, 0x65,
0x20, 0x4d, 0xe1, 0x67, 0x4b, 0x1ef9, 0x6b, 0x169, 0x6e, 0x4b, 0x61, 0x72,
0x61, 0x6b, 0x61, 0x6e, 0x68, 0x4b, 0x61, 0x6a, 0x65, 0x6e, 0x1ef9, 0x4b,
0x61, 0x6a, 0x6d, 0x1ef9, 0x6e, 0x53, 0x69, 0x6b, 0x61, 0x67, 0x75, 0x53,
0x69, 0x68, 0x75, 0x76, 0x61, 0x68, 0x75, 0x76, 0x61, 0x50, 0xe3, 0x6e,
0x129, 0x20, 0x54, 0x169, 0x20, 0x4d, 0xe1, 0x67, 0x4b, 0x72, 0xe9, 0x73,
0x69, 0x74, 0xe3, 0x6e, 0x4b, 0x75, 0x6a, 0x61, 0x6d, 0x61, 0x4b, 0x75,
0x72, 0x61, 0x73, 0x61, 0x76, 0x6f, 0x4e, 0xe1, 0x6e, 0x6d, 0x1ef9, 0x72,
0x6b, 0x69, 0x73, 0x61, 0x76, 0x79, 0x6e, 0x4e, 0x61, 0x76, 0x73, 0xe3,
0x6e, 0x4e, 0x61, 0x75, 0x73, 0xe3, 0x6e, 0x20, 0x4b, 0x72, 0x69, 0x6b,
0x69, 0x4e, 0x65, 0x6e, 0x76, 0x65, 0x72, 0x4e, 0x65, 0x74, 0x6f, 0x72,
0xf3, 0x6a, 0x74, 0x69, 0x4e, 0x6f, 0x6d, 0x129, 0x6e, 0x129, 0x6b, 0x61,
0x45, 0x6e, 0x69, 0x6d, 0xe3, 0x6e, 0x74, 0xe3, 0x45, 0x6a, 0x72, 0x75,
0x6e, 0x1ebd, 0x70, 0xe9, 0xc9, 0x72, 0x20, 0x53, 0x61, 0x72, 0x76, 0x61,
0x6e, 0x6f, 0x72, 0x46, 0xf3, 0x72, 0x74, 0x69, 0x20, 0x4e, 0x1ebd, 0x72,
0x73, 0x75, 0x46, 0x6f, 0x72, 0x74, 0x61, 0x72, 0x65, 0x6a, 0x61, 0x47,
0x72, 0x61, 0x73, 0x65, 0x20, 0x4d, 0x65, 0x6a, 0x47, 0x1ef9, 0x73, 0x6f,
0x20, 0x4d, 0x65, 0x6a, 0x54, 0x75, 0x72, 0x6b, 0x69, 0x20, 0x4d, 0xe1,
0x67, 0x47, 0x72, 0x61, 0x6e, 0x1ef9, 0x6e, 0x61, 0x47, 0x75, 0x76, 0x61,
0x6e, 0x61, 0x72, 0x75, 0x70, 0x65, 0x47, 0x75, 0x72, 0x61, 0x74, 0x65,
0x6d, 0x1ef9, 0x72, 0x61, 0x47, 0x75, 0x76, 0x61, 0x6a, 0x61, 0x6b, 0x69,
0x72, 0x47, 0x75, 0x6a, 0xe1, 0x6e, 0x1ef9, 0x48, 0x61, 0x72, 0x69, 0x66,
0x61, 0x67, 0x48, 0x61, 0x76, 0xe1, 0x6e, 0x1ef9, 0x48, 0xe9, 0x72, 0x6d,
0x6f, 0x73, 0x69, 0x72, 0x6f, 0x128, 0x6e, 0x69, 0x6a, 0x61, 0x6e, 0x1ef9,
0x70, 0x6f, 0x72, 0x69, 0x6e, 0x68, 0x4b, 0x69, 0x6e, 0xe3, 0x67, 0x2c,
0x20, 0x128, 0x6e, 0x69, 0x6a, 0x1ef9, 0x6e, 0x1ef9, 0x4d, 0x1ef9, 0x72, 0x1ebd,
0x67, 0x67, 0x75, 0x2c, 0x20, 0x128, 0x6e, 0x69, 0x6a, 0x1ef9, 0x6e, 0x1ef9,
0x50, 0xe9, 0x74, 0x65, 0x72, 0x6d, 0x75, 0x72, 0x67, 0x69, 0x2c, 0x20,
0x128, 0x6e, 0x69, 0x6a, 0x1ef9, 0x6e, 0x1ef9, 0x54, 0x65, 0x72, 0x20, 0x73,
0x69, 0x74, 0x69, 0x2c, 0x20, 0x128, 0x6e, 0x69, 0x6a, 0x1ef9, 0x6e, 0x1ef9,
0x56, 0x65, 0x76, 0x61, 0x6a, 0x2c, 0x20, 0x128, 0x6e, 0x69, 0x6a, 0x1ef9,
0x6e, 0x1ef9, 0x56, 0x129, 0x73, 0x65, 0x6e, 0x1ebd, 0x6e, 0x68, 0x2c, 0x20,
0x128, 0x6e, 0x69, 0x6a, 0x1ef9, 0x6e, 0x1ef9, 0x56, 0x69, 0x6e, 0x1ef9, 0x6d,
0x1ef9, 0x6b, 0x69, 0x2c, 0x20, 0x128, 0x6e, 0x69, 0x6a, 0x1ef9, 0x6e, 0x1ef9,
0x49, 0x6e, 0x169, 0x76, 0x69, 0x6b, 0x69, 0x49, 0x6b, 0x61, 0x72, 0x75,
0x6a, 0x69, 0x6e, 0x4a, 0x61, 0x6d, 0x1ef9, 0x6a, 0x6b, 0x1ef9, 0x4a, 0x75,
0x6e, 0x1ef9, 0x76, 0x6f, 0x52, 0x75, 0x69, 0x6e, 0x68, 0x76, 0x69, 0x72,
0x69, 0x4d, 0xe3, 0x6e, 0x74, 0x69, 0x73, 0xe9, 0x72, 0x75, 0x2c, 0x20,
0x4b, 0x1ebd, 0x74, 0xe1, 0x6b, 0x69, 0x4b, 0x72, 0x61, 0x72, 0x1ebd, 0x6e,
0x69, 0x6b, 0x65, 0x52, 0x61, 0x20, 0x50, 0x61, 0x73, 0x69, 0x52, 0x69,
0x6d, 0x1ef9, 0x52, 0x6f, 0x6e, 0x68, 0x20, 0x1ef8, 0x6e, 0x6a, 0x69, 0x72,
0x69, 0x52, 0x6f, 0x76, 0x65, 0x72, 0x20, 0x50, 0x72, 0x129, 0x6e, 0x73,
0x69, 0x20, 0x4b, 0x75, 0x76, 0x61, 0x72, 0x74, 0xe9, 0x72, 0x4d, 0x1ef9,
0x73, 0x65, 0x6a, 0xf3, 0x4d, 0x1ef9, 0x6e, 0x1ef9, 0x67, 0x75, 0x76, 0x61,
0x4d, 0x1ef9, 0x6e, 0x1ef9, 0x76, 0x6f, 0x4d, 0x1ef9, 0x72, 0x69, 0x67, 0xf3,
0x74, 0x69, 0x4d, 0x1ef9, 0x72, 0x74, 0x69, 0x6e, 0x129, 0x6b, 0x61, 0x4d,
0xe3, 0x72, 0x6f, 0x2d, 0x61, 0x67, 0x20, 0x54, 0xe3, 0x6e, 0x68, 0x4d,
0x1ef9, 0x6a, 0x61, 0x74, 0x72, 0x1ef9, 0x6e, 0x4d, 0x1ebd, 0x6e, 0xe3, 0x6d,
0x129, 0x6e, 0x129, 0x4d, 0x1ebd, 0x72, 0x69, 0x6e, 0x61, 0x4d, 0x1ebd, 0x74,
0x61, 0x72, 0x61, 0x6b, 0x61, 0x74, 0x61, 0x72, 0x61, 0x1ebc, 0x6d, 0xe3,
0x20, 0x74, 0x1ef9, 0x20, 0x4d, 0x1ebd, 0x73, 0x69, 0x6b, 0x6f, 0x4d, 0x129,
0x6b, 0x65, 0x72, 0x6f, 0x6e, 0x4d, 0xe3, 0x67, 0x74, 0xe3, 0x6e, 0x4d,
0xe3, 0x74, 0x65, 0x68, 0x65, 0x6a, 0x4d, 0xe3, 0x74, 0x69, 0x76, 0x69,
0x6e, 0xe9, 0x76, 0x4d, 0xe3, 0x6e, 0x73, 0x65, 0x72, 0xe1, 0x74, 0x69,
0x4e, 0x1ef9, 0x73, 0x61, 0x76, 0x59, 0x6a, 0xf3, 0x72, 0x6b, 0x69, 0x20,
0x54, 0xe3, 0x67, 0x4a, 0x79, 0x6a, 0x79, 0x46, 0x65, 0x72, 0x6e, 0x1ef9,
0x6e, 0x75, 0x20, 0x4e, 0xe3, 0x72, 0xe3, 0x6a, 0x61, 0x2d, 0x74, 0xe1,
0x4d, 0x65, 0x75, 0x72, 0x61, 0x2c, 0x20, 0x4e, 0x61, 0x6b, 0x6f, 0x74,
0x61, 0x20, 0x4e, 0xe3, 0x72, 0x74, 0x69, 0x53, 0x1ebd, 0x6e, 0x74, 0x65,
0x72, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x4e, 0xe3,
0x72, 0x74, 0x69, 0x4e, 0x129, 0x75, 0x20, 0x53, 0x61, 0x72, 0x65, 0x6e,
0x2c, 0x20, 0x4e, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x4e, 0xe3, 0x72,
0x74, 0x69, 0x4e, 0x169, 0x67, 0x4f, 0x6a, 0x69, 0x6e, 0x1ef9, 0x67, 0x61,
0x50, 0x61, 0x6e, 0x1ef9, 0x6d, 0x1ef9, 0x50, 0x61, 0x72, 0x61, 0x6d, 0x1ef9,
0x72, 0x69, 0x6d, 0x75, 0x46, 0x75, 0x65, 0x6e, 0x129, 0x6e, 0x68, 0x50,
0x6f, 0x72, 0x74, 0x75, 0x20, 0x50, 0x72, 0x129, 0x73, 0x69, 0x70, 0x69,
0x50, 0xf3, 0x72, 0x74, 0x6f, 0x66, 0x69, 0x20, 0x49, 0x6e, 0x68, 0x70,
0x61, 0x6a, 0x69, 0x6e, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x20, 0x4b, 0xf3,
0x66, 0x61, 0x50, 0x169, 0x74, 0x61, 0x20, 0x41, 0x72, 0x65, 0x6e, 0x1ef9,
0x52, 0x1ef9, 0x67, 0x6b, 0x129, 0x67, 0x20, 0x128, 0x72, 0xe9, 0x6e, 0x52,
0x69, 0x73, 0x69, 0x66, 0x69, 0x52, 0x69, 0x6a, 0x69, 0x6e, 0x1ef9, 0x52,
0x69, 0x6a, 0x6f, 0x72, 0x75, 0x74, 0x65, 0x47, 0x6f, 0x6a, 0x20, 0x4b,
0x75, 0x70, 0x72, 0x69, 0x53, 0x1ef9, 0x74, 0x61, 0x72, 0x1ebd, 0x6a, 0x53,
0x1ef9, 0x74, 0x69, 0x6a, 0x61, 0x67, 0x75, 0x53, 0x1ef9, 0x74, 0x75, 0x20,
0x4e, 0x75, 0x6d, 0x129, 0x67, 0x67, 0x75, 0x53, 0x1ef9, 0x20, 0x50, 0x61,
0x76, 0x6f, 0x72, 0x75, 0x49, 0x74, 0x6f, 0x67, 0x6b, 0x6f, 0x72, 0x74,
0x75, 0x72, 0x6d, 0x129, 0x6e, 0x74, 0x69, 0x53, 0x69, 0x6e, 0x6b, 0x61,
0x53, 0x1ef9, 0x20, 0x4d, 0x61, 0x72, 0x74, 0x6f, 0x72, 0x6f, 0x6d, 0x1ebd,
0x53, 0x1ef9, 0x6e, 0x20, 0x4a, 0x6f, 0x6e, 0x68, 0x53, 0x1ef9, 0x20, 0x4b,
0x72, 0x69, 0x74, 0xf3, 0x76, 0x1ef9, 0x53, 0x1ef9, 0x74, 0x61, 0x20, 0x52,
0x75, 0x73, 0x61, 0x53, 0x1ef9, 0x20, 0x54, 0x6f, 0x6d, 0x61, 0x6a, 0x53,
0x1ef9, 0x20, 0x56, 0x69, 0x73, 0x1ebd, 0x74, 0x69, 0x53, 0x75, 0x76, 0x69,
0x66, 0x69, 0x74, 0x69, 0x20, 0x4b, 0x75, 0x72, 0x1ebd, 0x74, 0x69, 0x54,
0x65, 0x67, 0x75, 0x73, 0x69, 0x67, 0x61, 0x72, 0x70, 0x61, 0x54, 0x75,
0x72, 0x65, 0x54, 0x69, 0x76, 0x1ef9, 0x6e, 0x1ef9, 0x54, 0x6f, 0x72, 0xe3,
0x74, 0x6f, 0x54, 0x6f, 0x72, 0x74, 0x6f, 0x72, 0x61, 0x56, 0x1ef9, 0x67,
0x6b, 0x75, 0x76, 0x65, 0x72, 0x4b, 0xe3, 0x76, 0xe3, 0x72, 0x75, 0x20,
0x4b, 0x75, 0x70, 0x72, 0x69, 0x56, 0x129, 0x6e, 0x129, 0x70, 0xe9, 0x67,
0x59, 0x6a, 0x61, 0x6b, 0x75, 0x74, 0x61, 0x74, 0x69, 0x4b, 0x61, 0x73,
0x65, 0x6a, 0x4e, 0x61, 0x76, 0x69, 0x73, 0x69, 0x4d, 0x1ef9, 0x6b, 0x69,
0x6b, 0x75, 0x61, 0x72, 0x69, 0x4d, 0x1ef9, 0x75, 0x73, 0xe3, 0x6e, 0x4d,
0xe9, 0x67, 0x2d, 0x4d, 0x169, 0x72, 0x6e, 0x6f, 0x50, 0x61, 0x72, 0x6d,
0x1ebd, 0x72, 0x54, 0x6f, 0x72, 0xf3, 0x72, 0x56, 0x6f, 0x6e, 0x68, 0x74,
0xf3, 0x6b, 0x69, 0x52, 0x169, 0x67, 0x69, 0x6a, 0xe9, 0x20, 0x41, 0x72,
0x6d, 0x79, 0x6a, 0x1ebd, 0x6e, 0x41, 0x6e, 0x65, 0x6e, 0x41, 0x72, 0x6d,
0x1ef9, 0x74, 0x69, 0x41, 0x6d, 0x1ef9, 0x41, 0x6e, 0x1ef9, 0x6e, 0x68, 0x69,
0x72, 0x41, 0x6b, 0x69, 0x74, 0x61, 0x75, 0x41, 0x6b, 0x69, 0x74, 0x6f,
0x6d, 0x65, 0x41, 0x6e, 0x68, 0x67, 0x61, 0x6d, 0x61, 0x74, 0x69, 0x41,
0x74, 0x69, 0x72, 0x61, 0x76, 0x4d, 0x61, 0x67, 0x69, 0x6e, 0x61, 0x4d,
0x61, 0x72, 0x1ebd, 0x6a, 0x4d, 0x79, 0x67, 0x6b, 0xf3, 0x6b, 0x69, 0x4d,
0x61, 0x72, 0x6e, 0x1ef9, 0x75, 0x72, 0x4d, 0x65, 0x6a, 0x72, 0x75, 0x74,
0x69, 0x4d, 0x69, 0x73, 0x69, 0x6b, 0xe9, 0x6b, 0x69, 0x4d, 0x72, 0x75,
0x6e, 0x1ebd, 0x6a, 0x53, 0x69, 0x74, 0x61, 0x4b, 0x6f, 0x72, 0xe3, 0x6d,
0x6d, 0x75, 0x4e, 0x61, 0x6d, 0x1ef9, 0x6e, 0x68, 0x6b, 0x6f, 0x4e, 0x69,
0x72, 0x69, 0x4e, 0x75, 0x6d, 0x61, 0x6a, 0x4e, 0x75, 0x73, 0xe1, 0x67,
0x6d, 0x65, 0x46, 0x61, 0x6d, 0x1ef9, 0x67, 0x75, 0x6a, 0x74, 0x61, 0x47,
0x61, 0x6a, 0x61, 0x48, 0x65, 0x6d, 0x72, 0x6f, 0x6e, 0x48, 0xe3, 0x67,
0x20, 0x4b, 0xe3, 0x67, 0x48, 0x6f, 0x76, 0x69, 0x6e, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x69, 0x6e, 0x68, 0x6b, 0x69, 0x4a, 0x61, 0x69, 0x61, 0x70,
0x75, 0x72, 0x61, 0x4a, 0x65, 0x72, 0x75, 0x6a, 0x61, 0x72, 0x1ebd, 0x6a,
0x4b, 0x61, 0x6d, 0x75, 0x72, 0x4b, 0x1ef9, 0x6d, 0x73, 0x61, 0x6e, 0x6b,
0x61, 0x4b, 0x61, 0x72, 0x61, 0x73, 0x69, 0x4b, 0x61, 0x74, 0x69, 0x6d,
0x1ef9, 0x6e, 0x6e, 0x75, 0x4b, 0x1ef9, 0x6e, 0x79, 0x67, 0x61, 0x4b, 0x61,
0x72, 0x6b, 0x75, 0x74, 0x61, 0x4b, 0x72, 0x61, 0x6e, 0x68, 0x6e, 0xe3,
0x6a, 0x61, 0x72, 0x6b, 0x69, 0x4b, 0x75, 0x76, 0x61, 0x72, 0x61, 0x20,
0x52, 0x169, 0x70, 0x75, 0x72, 0x4b, 0x75, 0x73, 0x129, 0x67, 0x4b, 0x75,
0x76, 0x61, 0x6a, 0x74, 0x69, 0x4d, 0x1ef9, 0x6b, 0x61, 0x75, 0x4d, 0x1ef9,
0x67, 0x61, 0x6e, 0x61, 0x6e, 0x4d, 0x1ef9, 0x6e, 0x129, 0x72, 0x61, 0x4e,
0x129, 0x6b, 0xf3, 0x6a, 0x69, 0x61, 0x4b, 0x75, 0x6e, 0x68, 0x6e, 0x1ebd,
0x74, 0x69, 0x6e, 0x68, 0x6b, 0x69, 0x20, 0x54, 0xe3, 0x67, 0x53, 0x69,
0x6d, 0x69, 0x72, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x54, 0xe3, 0x67, 0x4f,
0x6d, 0x129, 0x6e, 0x68, 0x6b, 0x69, 0x4f, 0x72, 0x61, 0x72, 0x46, 0x79,
0x6e, 0xe3, 0x67, 0x20, 0x50, 0x1ebd, 0x6a, 0x50, 0xe3, 0x6e, 0x74, 0x69,
0x1ef9, 0x6e, 0x1ef9, 0x6b, 0x69, 0x50, 0x69, 0xe3, 0x67, 0x2d, 0x6a, 0x1ef9,
0x67, 0x4b, 0x6f, 0x6a, 0x74, 0x61, 0x6e, 0x1ef9, 0x6a, 0x4b, 0x79, 0x6a,
0x79, 0x72, 0x6f, 0x72, 0x6e, 0x61, 0x52, 0x69, 0x6a, 0x61, 0x6e, 0x65,
0x53, 0x61, 0x6b, 0x61, 0x72, 0x129, 0x6e, 0x1ef9, 0x53, 0x61, 0x6d, 0x1ef9,
0x72, 0x6b, 0x1ef9, 0x6e, 0x6e, 0x61, 0x53, 0x65, 0x2019, 0x75, 0x72, 0x53,
0x1ef9, 0x67, 0x67, 0x61, 0x6a, 0x53, 0x129, 0x67, 0x61, 0x70, 0x75, 0x72,
0x61, 0x53, 0x69, 0x72, 0x65, 0x6e, 0x1ebd, 0x6b, 0x6f, 0x72, 0x79, 0x6d,
0x73, 0x69, 0x6b, 0x69, 0x54, 0x61, 0x6e, 0x68, 0x6b, 0x1ebd, 0x6e, 0x74,
0x69, 0x54, 0x69, 0x6d, 0x69, 0x72, 0x69, 0x6a, 0x69, 0x54, 0x65, 0x2019,
0x65, 0x72, 0x1ef9, 0x54, 0x129, 0x6d, 0x66, 0x75, 0x54, 0xf3, 0x6b, 0x69,
0x6a, 0x6f, 0x54, 0x6f, 0x6d, 0x129, 0x6e, 0x68, 0x6b, 0x69, 0x55, 0x72,
0x61, 0x6e, 0x20, 0x4d, 0x61, 0x74, 0x6f, 0x72, 0x55, 0x72, 0x169, 0x6d,
0x6b, 0x69, 0x55, 0x6e, 0x68, 0x74, 0x69, 0x2d, 0x4e, 0x1ebd, 0x72, 0x61,
0x56, 0x69, 0x6a, 0x1ebd, 0x74, 0x69, 0x6a, 0x1ef9, 0x6e, 0x129, 0x55, 0x72,
0x61, 0x6e, 0x69, 0x76, 0xf3, 0x6e, 0x68, 0x74, 0xf3, 0x6b, 0x69, 0x59,
0x6a, 0x61, 0x6b, 0x75, 0x74, 0x69, 0x6e, 0x68, 0x6b, 0x69, 0x52, 0x61,
0x6e, 0x67, 0x169, 0x6d, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x129, 0x6e,
0x6d, 0x75, 0x72, 0x67, 0x75, 0x4a, 0x65, 0x72, 0x65, 0x76, 0x1ef9, 0x41,
0x73, 0x6f, 0x72, 0x65, 0x6e, 0x68, 0x4d, 0x65, 0x72, 0x6d, 0x169, 0x6e,
0x61, 0x4b, 0x61, 0x6e, 0x1ef9, 0x72, 0x69, 0x6a, 0x61, 0x20, 0x41, 0x67,
0x50, 0x75, 0x20, 0x54, 0xe1, 0x6e, 0x68, 0x52, 0x65, 0x6a, 0x6b, 0x69,
0x6a, 0x61, 0x76, 0x69, 0x6b, 0x69, 0x4a, 0x69, 0xf3, 0x72, 0x6a, 0x61,
0x20, 0x74, 0x1ef9, 0x20, 0x53, 0x75, 0x72, 0x53, 0x1ef9, 0x74, 0x61, 0x20,
0x45, 0x72, 0x65, 0x6e, 0x1ef9, 0x53, 0x69, 0x74, 0x1ef9, 0x72, 0x65, 0x6a,
0x41, 0x6e, 0x65, 0x72, 0x61, 0x6a, 0x6e, 0x69, 0x4d, 0x72, 0x69, 0x6e,
0x68, 0x6d, 0x61, 0x6e, 0x1ebd, 0x4d, 0x72, 0x75, 0x6b, 0x65, 0x6e, 0x20,
0x48, 0x69, 0x72, 0x4e, 0x61, 0x72, 0x76, 0x129, 0x6e, 0x45, 0x75, 0x6b,
0x72, 0x61, 0x48, 0x6f, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x52, 0x129, 0x6e,
0x65, 0x72, 0x6d, 0x1ef9, 0x6e, 0x52, 0xf3, 0x72, 0x6e, 0x69, 0x20, 0x48,
0x6f, 0x76, 0x65, 0x4d, 0x1ebd, 0x72, 0x6d, 0x75, 0x72, 0x6e, 0x129, 0x50,
0xe9, 0x72, 0x74, 0x69, 0x53, 0x69, 0x6e, 0x69, 0x6e, 0x65, 0x6a, 0x41,
0x6d, 0x129, 0x6e, 0x68, 0x74, 0x65, 0x72, 0x6e, 0xe1, 0x1ef8, 0x6e, 0x6f,
0x68, 0x61, 0x41, 0x6e, 0x68, 0x74, 0x61, 0x6b, 0x1ef9, 0x41, 0x74, 0x65,
0x6e, 0x1ef9, 0x4d, 0x65, 0x72, 0x67, 0x72, 0x61, 0x6e, 0x6f, 0x4d, 0x65,
0x72, 0x2d, 0x72, 0x129, 0x67, 0x4d, 0x72, 0x61, 0x74, 0x69, 0x6e, 0x68,
0x72, 0x61, 0x76, 0x61, 0x4d, 0x72, 0x75, 0x73, 0xe9, 0x72, 0x61, 0x4d,
0x75, 0x6b, 0x61, 0x72, 0xe9, 0x6e, 0x68, 0x74, 0x69, 0x4d, 0x75, 0x6e,
0x61, 0x70, 0x65, 0x6e, 0x68, 0x74, 0x65, 0x4d, 0x79, 0x6a, 0x69, 0x6e,
0x67, 0x65, 0x6e, 0x53, 0x69, 0x6a, 0x69, 0x6e, 0x1ef9, 0x76, 0x6f, 0x4b,
0x6f, 0x70, 0x1ebd, 0x6e, 0x68, 0x1ef9, 0x67, 0x65, 0x4e, 0x75, 0x6d, 0x72,
0x69, 0x6e, 0x4a, 0x69, 0x6d, 0x72, 0x61, 0x74, 0x61, 0x72, 0x47, 0xe9,
0x72, 0x6e, 0x129, 0x73, 0x65, 0x6a, 0x48, 0xe9, 0x72, 0x73, 0x129, 0x67,
0x6b, 0x65, 0x168, 0x6e, 0x2d, 0x67, 0x72, 0xe9, 0x20, 0x47, 0x6f, 0x6a,
0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x49, 0x6e, 0x68, 0x74, 0x61, 0x6d, 0x75,
0x72, 0x4a, 0xe9, 0x72, 0x73, 0x65, 0x6a, 0x4b, 0x61, 0x72, 0x69, 0x6e,
0x129, 0x6e, 0x67, 0x72, 0x61, 0x6e, 0x6f, 0x4b, 0x69, 0x72, 0xf3, 0x76,
0x69, 0x4b, 0x69, 0x6a, 0xe9, 0x76, 0x69, 0x52, 0x69, 0x6e, 0x68, 0x6d,
0x6f, 0x76, 0x61, 0x52, 0x69, 0x6d, 0x72, 0x69, 0x1ef9, 0x6e, 0x1ef9, 0x52,
0xe3, 0x6e, 0x65, 0x72, 0x65, 0x52, 0x75, 0x73, 0x1ebd, 0x67, 0x6d, 0x75,
0x72, 0x67, 0x6f, 0x4d, 0x1ef9, 0x6e, 0x69, 0x72, 0x69, 0x4d, 0x61, 0x72,
0x74, 0x61, 0x4d, 0x1ef9, 0x72, 0x69, 0x65, 0x68, 0x1ef9, 0x6d, 0x4d, 0x129,
0x67, 0x73, 0x69, 0x6b, 0x69, 0x4d, 0xe3, 0x6e, 0x1ef9, 0x6b, 0x6f, 0x4d,
0xe3, 0x6e, 0x68, 0x6b, 0x6f, 0x76, 0xd3, 0x73, 0x69, 0x72, 0x6f, 0x50,
0x61, 0x72, 0x69, 0x6e, 0x68, 0x50, 0x6f, 0x6e, 0x67, 0xf3, 0x72, 0x69,
0x6b, 0x61, 0x52, 0x6f, 0x6d, 0x1ef9, 0x53, 0x61, 0x6d, 0x1ef9, 0x72, 0x61,
0x53, 0x1ef9, 0x20, 0x4d, 0x1ef9, 0x72, 0x129, 0x6e, 0x169, 0x53, 0x61, 0x72,
0x61, 0x6a, 0x65, 0x76, 0x6f, 0x2f, 0x53, 0x61, 0x72, 0x61, 0x6a, 0x65,
0x76, 0x75, 0x53, 0x61, 0x72, 0x61, 0x74, 0xf3, 0x76, 0x69, 0x53, 0x129,
0x67, 0x66, 0x69, 0x72, 0x69, 0x70, 0x6f, 0x72, 0x53, 0x69, 0x6b, 0x6f,
0x70, 0x69, 0x6a, 0x65, 0x53, 0xf3, 0x66, 0x69, 0x6a, 0x61, 0x45, 0x6e,
0x68, 0x74, 0x75, 0x6b, 0x6f, 0x72, 0x6d, 0x169, 0x54, 0x61, 0x72, 0x129,
0x6e, 0x54, 0x69, 0x72, 0x1ef9, 0x6e, 0x1ef9, 0x55, 0x72, 0x69, 0x6a, 0x61,
0x6e, 0xe3, 0x76, 0x69, 0x6e, 0x68, 0x6b, 0x69, 0x56, 0x61, 0x6e, 0x75,
0x6a, 0x56, 0x61, 0x74, 0x69, 0x6b, 0x1ef9, 0x6e, 0x169, 0x56, 0x69, 0x65,
0x6e, 0x1ef9, 0x56, 0x69, 0x72, 0x6e, 0x129, 0x6a, 0x75, 0x73, 0x69, 0x56,
0x6f, 0x72, 0x67, 0x75, 0x67, 0x72, 0x61, 0x6e, 0x6f, 0x56, 0x61, 0x72,
0x73, 0xf3, 0x76, 0x69, 0x6a, 0x61, 0x4a, 0x61, 0x67, 0x72, 0xe9, 0x6d,
0x65, 0x4a, 0x75, 0x72, 0x69, 0x6b, 0x65, 0x1ef8, 0x74, 0x61, 0x6e, 0x1ef9,
0x6e, 0x1ef9, 0x72, 0x69, 0x76, 0x6f, 0x53, 0x61, 0x67, 0x6f, 0x4b, 0x72,
0x69, 0x6e, 0x68, 0x74, 0x69, 0x6d, 0x1ef9, 0x4b, 0xf3, 0x6b, 0x6f, 0x20,
0x41, 0x67, 0x4b, 0x6f, 0x6d, 0xe3, 0x72, 0x65, 0x4b, 0x69, 0x72, 0x67,
0xe9, 0x72, 0x65, 0x6e, 0x4d, 0x1ef9, 0x68, 0xe9, 0x4d, 0x1ef9, 0x72, 0x6e,
0x69, 0x76, 0x61, 0x4d, 0xe3, 0x72, 0x69, 0x73, 0x69, 0x76, 0x4d, 0x1ef9,
0x6a, 0xf3, 0x74, 0x69, 0x52, 0x169, 0x6e, 0x6a, 0x169, 0x76, 0xd3, 0x67,
0x2d, 0x72, 0x1ef9, 0x6e, 0x4d, 0x75, 0x67, 0x1ef9, 0x6e, 0x76, 0x69, 0x72,
0x65, 0x53, 0x61, 0x74, 0x69, 0x6e, 0x1ef9, 0x6d, 0x50, 0x61, 0x73, 0x69,
0x6b, 0x75, 0x76, 0x61, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73,
0x6f, 0x46, 0x61, 0x6b, 0x61, 0x6f, 0x66, 0x75, 0x46, 0x75, 0x6e, 0x1ef9,
0x66, 0x75, 0x74, 0x69, 0x47, 0x61, 0x72, 0x61, 0x20, 0x50, 0x61, 0x67,
0x6f, 0x47, 0x1ef9, 0x6d, 0x6d, 0x69, 0xe9, 0x72, 0x47, 0x75, 0x76, 0x61,
0x6e, 0x61, 0x72, 0x6b, 0x61, 0x6e, 0x1ef9, 0x72, 0x47, 0x75, 0x76, 0x1ef9,
0x6d, 0x48, 0x6f, 0x6e, 0xe3, 0x72, 0x75, 0x72, 0x75, 0x1ebc, 0x6e, 0x6e,
0x65, 0x72, 0x6d, 0x75, 0x72, 0x69, 0x4b, 0x69, 0x72, 0x69, 0x74, 0x69,
0x6d, 0x1ef9, 0x74, 0x69, 0x4b, 0x6f, 0x73, 0x69, 0x72, 0x61, 0x6a, 0x65,
0x4b, 0x75, 0x76, 0x61, 0x6a, 0x61, 0x72, 0x1ebd, 0x6a, 0x4d, 0x1ef9, 0x6a,
0x75, 0x72, 0x6f, 0x4d, 0x1ef9, 0x72, 0x6b, 0x65, 0x6a, 0x61, 0x20, 0x46,
0x61, 0x67, 0x4d, 0x129, 0x6e, 0x6e, 0x75, 0x76, 0x65, 0x6a, 0x4e, 0x1ef9,
0x76, 0x75, 0x72, 0x75, 0x4e, 0x129, 0x76, 0x65, 0x4e, 0xe3, 0x72, 0x66,
0x6f, 0x6b, 0x69, 0x4e, 0xe3, 0x75, 0x6d, 0x1ebd, 0x61, 0x50, 0x61, 0x72,
0x61, 0x76, 0x50, 0x69, 0x74, 0x69, 0x6b, 0x61, 0x69, 0x72, 0x50, 0xf3,
0x72, 0x20, 0x4d, 0xe3, 0x72, 0x65, 0x6e, 0x68, 0x6d, 0x69, 0x52, 0x61,
0x72, 0xe3, 0x74, 0xe3, 0x67, 0x61, 0x53, 0x61, 0x6a, 0x70, 0x1ef9, 0x54,
0x61, 0x6a, 0x74, 0x69, 0x54, 0x61, 0x72, 0x61, 0x75, 0x76, 0x61, 0x54,
0xe3, 0x6e, 0x67, 0x61, 0x74, 0x61, 0x70, 0x75, 0x56, 0x61, 0x6b, 0x69,
0x56, 0x61, 0x72, 0x69, 0x6e, 0x68, 0x41, 0x62, 0x69, 0x64, 0x69, 0x79,
0xe3, 0x41, 0x6b, 0x61, 0x72, 0x61, 0x41, 0x64, 0x69, 0x73, 0x69, 0x20,
0x41, 0x62, 0x65, 0x62, 0x61, 0x41, 0x67, 0x65, 0x75, 0x41, 0x73, 0x69,
0x6d, 0x61, 0x72, 0x61, 0x42, 0xe3, 0x6b, 0x69, 0x42, 0xe3, 0x6a, 0xfa,
0x42, 0x61, 0x72, 0xe3, 0x74, 0x69, 0x72, 0x65, 0x42, 0x61, 0x72, 0x61,
0x7a, 0x61, 0x77, 0x69, 0x72, 0x69, 0x42, 0x75, 0x79, 0x169, 0x62, 0x75,
0x72, 0x61, 0x4b, 0x61, 0x69, 0x72, 0x75, 0x55, 0x6b, 0x61, 0x20, 0x4d,
0x75, 0x72, 0x75, 0x74, 0x129, 0x67, 0x61, 0x4b, 0x75, 0x6e, 0x61, 0x6b,
0x69, 0x72, 0x69, 0x4b, 0x61, 0x74, 0x75, 0x61, 0x77, 0x61, 0x20, 0x72,
0x75, 0x6b, 0x61, 0x44, 0x69, 0x6a, 0x69, 0x62, 0x75, 0x74, 0x69, 0x44,
0x75, 0x77, 0x61, 0x6c, 0x61, 0x45, 0x75, 0x20, 0x41, 0x79, 0x169, 0x46,
0x69, 0x72, 0x65, 0x74, 0x169, 0x47, 0x61, 0x62, 0x75, 0x72, 0x75, 0x6e,
0x69, 0x4a, 0x75, 0x61, 0x6e, 0x65, 0x73, 0x69, 0x62, 0x75, 0x67, 0x75,
0x59, 0x75, 0x62, 0x61, 0x4b, 0xe3, 0x70, 0x61, 0x72, 0x61, 0x4b, 0x61,
0x74, 0x169, 0x4b, 0x129, 0x78, 0x61, 0x73, 0x61, 0x49, 0x70, 0x61, 0x77,
0x61, 0x69, 0x74, 0x61, 0x52, 0x69, 0x62, 0x65, 0x72, 0x65, 0x76, 0x69,
0x6c, 0x69, 0x52, 0x75, 0x6d, 0xe9, 0x52, 0x75, 0xe3, 0x64, 0x61, 0x4c,
0x75, 0x62, 0x169, 0x62, 0x61, 0x78, 0x69, 0x168, 0x62, 0x61, 0x62, 0x61,
0x6e, 0x69, 0x4d, 0x75, 0x67, 0x61, 0x64, 0x69, 0x73, 0x69, 0x6b, 0x75,
0x4d, 0x169, 0x72, 0x75, 0x77, 0x69, 0x79, 0x61, 0x4e, 0x61, 0x69, 0x72,
0x75, 0x62, 0x69, 0x4e, 0x2019, 0x20, 0x44, 0x69, 0x79, 0x61, 0x6d, 0x65,
0x6e, 0x61, 0x4e, 0x6f, 0x77, 0x61, 0x6b, 0x69, 0x78, 0x75, 0x74, 0x69,
0x55, 0x77, 0x61, 0x67, 0x61, 0x64, 0x75, 0x67, 0x75, 0x49, 0x67, 0x61,
0x72, 0x61, 0x70, 0x61, 0x77, 0x61, 0x20, 0x50, 0x69, 0x73, 0x61, 0x73,
0xfa, 0x53, 0xe3, 0x20, 0x54, 0x75, 0x6d, 0xe9, 0x54, 0x69, 0x72, 0x69,
0x70, 0x75, 0x72, 0x69, 0x54, 0x75, 0x6e, 0x69, 0x2d, 0x69, 0x74, 0x61,
0x57, 0x129, 0x64, 0x75, 0x65, 0x6b, 0x69, 0xc3, 0x6b, 0x75, 0x72, 0x61,
0x67, 0x65, 0xc3, 0x67, 0x69, 0x72, 0x61, 0xc3, 0x74, 0x69, 0x67, 0x75,
0x77, 0x61, 0x41, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x69, 0x6e, 0x61,
0x42, 0x75, 0x77, 0x65, 0x6e, 0x75, 0x73, 0x61, 0x69, 0x72, 0x69, 0x4b,
0x61, 0x74, 0x61, 0x6d, 0x61, 0x6b, 0x61, 0x4b, 0xf3, 0x64, 0x75, 0x62,
0x61, 0x59, 0x75, 0x79, 0x75, 0x69, 0x52, 0x61, 0x20, 0x52, 0x69, 0x6f,
0x79, 0x61, 0x4d, 0x1ebd, 0x64, 0x75, 0x73, 0x61, 0x50, 0x61, 0x72, 0x61,
0x6e, 0xe3, 0x20, 0x47, 0x61, 0x72, 0x65, 0x67, 0x75, 0x2d, 0x69, 0x74,
0x61, 0x53, 0x61, 0x75, 0x74, 0x61, 0x53, 0xe3, 0x20, 0x59, 0x75, 0x77,
0xe3, 0x53, 0xe3, 0x20, 0x52, 0x75, 0x69, 0x54, 0x75, 0x6b, 0x75, 0x6d,
0xe3, 0x55, 0x78, 0x75, 0x77, 0x61, 0x79, 0x61, 0x41, 0x73, 0x169, 0x73,
0x69, 0x79, 0xf5, 0x41, 0x74, 0x69, 0x6b, 0x6f, 0x6b, 0xe3, 0x42, 0x61,
0xed, 0x79, 0x61, 0x42, 0xe3, 0x64, 0x65, 0x72, 0x61, 0x2d, 0x69, 0x74,
0x61, 0x20, 0x4b, 0x75, 0x61, 0x72, 0x61, 0x42, 0x61, 0x72, 0x61, 0x62,
0x61, 0x64, 0x75, 0x42, 0x65, 0x6c, 0x1ebd, 0x42, 0x65, 0x72, 0x69, 0x73,
0x65, 0x49, 0x77, 0x69, 0x6b, 0x75, 0xed, 0x20, 0x4d, 0x75, 0x72, 0x75,
0x74, 0x129, 0x67, 0x61, 0x42, 0x75, 0x77, 0x61, 0x20, 0x56, 0x69, 0x73,
0x74, 0x61, 0x42, 0x75, 0x67, 0x75, 0x74, 0xe1, 0x42, 0x75, 0x77, 0x69,
0x73, 0xe9, 0x4b, 0x1ebd, 0x62, 0x69, 0x72, 0x69, 0x79, 0x69, 0x20, 0x4b,
0x75, 0x61, 0x72, 0x61, 0x4b, 0xe3, 0x70, 0x75, 0x20, 0x57, 0x61, 0x73,
0x75, 0x4b, 0xe3, 0x6b, 0x169, 0x4b, 0x61, 0x79, 0x65, 0x6e, 0x61, 0x4b,
0x61, 0x69, 0x6d, 0xe3, 0x53, 0x69, 0x76, 0x61, 0x76, 0x61, 0x4b, 0x75,
0x70, 0xe9, 0x20, 0x52, 0x69, 0x6b, 0x61, 0x4b, 0x65, 0x72, 0x65, 0x73,
0x74, 0x169, 0x44, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x6b, 0x61,
0x73, 0xe3, 0x76, 0x44, 0x61, 0x75, 0x73, 0x169, 0x44, 0x61, 0x75, 0x73,
0x169, 0x20, 0x4b, 0x69, 0x72, 0x69, 0x6b, 0x69, 0x44, 0x1ebd, 0x77, 0x65,
0x72, 0x44, 0x75, 0x6d, 0x69, 0x6e, 0x69, 0x6b, 0x61, 0x45, 0x64, 0x69,
0x6d, 0x169, 0x74, 0x169, 0x45, 0x75, 0x20, 0x53, 0x61, 0x77, 0x61, 0x64,
0x75, 0x46, 0x75, 0x74, 0x69, 0x20, 0x4e, 0x65, 0x75, 0x73, 0x169, 0x46,
0x75, 0x74, 0x61, 0x72, 0x65, 0x73, 0x61, 0x59, 0x72, 0x75, 0x73, 0xe3,
0x67, 0x75, 0x73, 0x75, 0x2d, 0x61, 0x74, 0xe3, 0x20, 0x4b, 0x75, 0x61,
0x72, 0x61, 0x49, 0x70, 0x65, 0x6b, 0x61, 0x77, 0x61, 0x73, 0x75, 0x20,
0x4b, 0x75, 0x61, 0x72, 0x61, 0x57, 0x61, 0x73, 0xfa, 0x20, 0x54, 0x75,
0x72, 0x69, 0x6b, 0x75, 0x47, 0x61, 0x72, 0x61, 0x6e, 0x61, 0x64, 0x61,
0x47, 0x75, 0x61, 0x64, 0x61, 0x72, 0x75, 0x70, 0x69, 0x47, 0x75, 0x61,
0x74, 0x65, 0x6d, 0x61, 0x72, 0x61, 0x47, 0x75, 0x61, 0x79, 0x61, 0x6b,
0x69, 0x75, 0x47, 0x69, 0x79, 0x61, 0x6e, 0x61, 0x48, 0x65, 0x6d, 0x75,
0x73, 0x69, 0x72, 0x75, 0x128, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x70, 0x75,
0x72, 0x69, 0x4b, 0x6e, 0x6f, 0x78, 0x2c, 0x20, 0x128, 0x64, 0x69, 0x61,
0x6e, 0x61, 0x4d, 0x61, 0x72, 0x1ebd, 0x67, 0x75, 0x2c, 0x20, 0x128, 0x64,
0x69, 0x61, 0x6e, 0x61, 0x50, 0x69, 0x74, 0x69, 0x72, 0x69, 0x73, 0x69,
0x62, 0x75, 0x67, 0x69, 0x2c, 0x20, 0x128, 0x64, 0x69, 0x61, 0x6e, 0x61,
0x54, 0x65, 0x6c, 0x6c, 0x20, 0x54, 0x61, 0x77, 0x61, 0x2d, 0x77, 0x61,
0x73, 0xfa, 0x2c, 0x20, 0x128, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x57, 0x65,
0x77, 0x61, 0x69, 0x2c, 0x20, 0x128, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x57,
0x129, 0x73, 0x65, 0x6e, 0x65, 0x2d, 0x69, 0x74, 0x61, 0x2c, 0x20, 0x128,
0x64, 0x69, 0x61, 0x6e, 0x61, 0x57, 0x69, 0x6e, 0xe3, 0x6d, 0x61, 0x6b,
0x69, 0x2c, 0x20, 0x128, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x49, 0x6e, 0x75,
0x77, 0x69, 0x6b, 0x69, 0x49, 0x6b, 0x61, 0x72, 0x75, 0x69, 0x74, 0x69,
0x59, 0x61, 0x6d, 0x61, 0x69, 0x6b, 0x61, 0x59, 0x75, 0x6e, 0x65, 0x61,
0x75, 0x4c, 0x75, 0x77, 0x69, 0x73, 0x69, 0x76, 0x69, 0x72, 0x69, 0x4d,
0x169, 0x74, 0x69, 0x73, 0x65, 0x72, 0x75, 0x2c, 0x20, 0x4b, 0x1ebd, 0x74,
0x75, 0x6b, 0x69, 0x4b, 0x61, 0x72, 0x61, 0x72, 0x1ebd, 0x64, 0x69, 0x6b,
0x69, 0x52, 0x69, 0x6d, 0x61, 0x50, 0x69, 0x72, 0x129, 0x73, 0x69, 0x70,
0x69, 0x20, 0x51, 0x75, 0x61, 0x72, 0x74, 0x65, 0x72, 0x20, 0x55, 0x69,
0x72, 0x70, 0x65, 0x77, 0x61, 0x72, 0x61, 0x4d, 0x61, 0x73, 0x65, 0x79,
0xf3, 0x4d, 0x61, 0x6e, 0xe1, 0x67, 0x75, 0x77, 0x61, 0x4d, 0x61, 0x72,
0x69, 0x67, 0x75, 0x74, 0x69, 0x4d, 0x61, 0x74, 0x69, 0x6e, 0x69, 0x6b,
0x61, 0x4d, 0x6f, 0x72, 0x6f, 0x69, 0x74, 0x61, 0x79, 0x75, 0x6b, 0xe1,
0x4d, 0x61, 0x73, 0x61, 0x74, 0x61, 0x72, 0xe3, 0x4d, 0x65, 0x74, 0x69,
0x72, 0x61, 0x6b, 0x61, 0x74, 0x69, 0x72, 0x61, 0x1ebc, 0x6d, 0xe3, 0x20,
0x74, 0x1ef9, 0x20, 0x4d, 0x1ebd, 0x73, 0x69, 0x6b, 0x75, 0x4d, 0x69, 0x6b,
0x69, 0x72, 0x169, 0x4d, 0x169, 0x6b, 0x69, 0x74, 0x169, 0x4d, 0xf5, 0x74,
0x65, 0x72, 0x65, 0x69, 0x4d, 0x169, 0x74, 0x69, 0x77, 0x69, 0x64, 0x65,
0x77, 0x75, 0x4d, 0xf5, 0x74, 0x69, 0x73, 0x65, 0x72, 0x61, 0x74, 0x69,
0x59, 0x75, 0x72, 0x6b, 0x69, 0x20, 0x50, 0x69, 0x73, 0x61, 0x73, 0xfa,
0x52, 0xe9, 0x72, 0x61, 0x46, 0x65, 0x6e, 0xe3, 0x64, 0x75, 0x20, 0x4e,
0x75, 0x72, 0x75, 0x79, 0xe3, 0x42, 0x65, 0x75, 0x72, 0x2c, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x4e, 0x75, 0x74, 0x69, 0x20, 0x73,
0x75, 0xed, 0x53, 0x1ebd, 0x74, 0x65, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x20, 0x4e, 0x75, 0x74, 0x69, 0x20, 0x73, 0x75, 0xed, 0x50,
0x69, 0x73, 0x61, 0x73, 0xfa, 0x20, 0x53, 0x61, 0x72, 0x1ebd, 0x2c, 0x20,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x4e, 0x75, 0x74, 0x69, 0x20,
0x73, 0x75, 0xed, 0x4e, 0x75, 0x6b, 0x69, 0x4f, 0x79, 0x69, 0x6e, 0x61,
0x67, 0x61, 0x50, 0x61, 0x6e, 0x61, 0x6d, 0xe3, 0x50, 0x75, 0x77, 0x65,
0x6e, 0x69, 0x6b, 0x69, 0x73, 0x69, 0x49, 0x67, 0x61, 0x72, 0x61, 0x70,
0x61, 0x77, 0x61, 0x20, 0x50, 0x69, 0x72, 0x129, 0x73, 0x69, 0x70, 0x69,
0x45, 0x73, 0x69, 0x70, 0xe3, 0x79, 0x61, 0x20, 0x49, 0x67, 0x61, 0x72,
0x61, 0x70, 0x61, 0x77, 0x61, 0x49, 0x67, 0x61, 0x72, 0x61, 0x70, 0x61,
0x77, 0x61, 0x20, 0x4b, 0x75, 0x78, 0x69, 0x6d, 0x61, 0x77, 0x61, 0x72,
0x61, 0x49, 0x67, 0x61, 0x72, 0x61, 0x70, 0x61, 0x77, 0x61, 0x20, 0x52,
0x69, 0x6b, 0x75, 0x50, 0x169, 0x74, 0x61, 0x20, 0x41, 0x72, 0x65, 0x6e,
0x61, 0x2d, 0x69, 0x74, 0x61, 0x52, 0xe3, 0x6b, 0x129, 0x20, 0x55, 0x69,
0x6b, 0x65, 0x73, 0x61, 0x77, 0x61, 0x52, 0x65, 0x6a, 0x129, 0x6e, 0x1ef9,
0x52, 0x65, 0x73, 0x6f, 0x72, 0x75, 0x74, 0x69, 0x50, 0x61, 0x72, 0x61,
0x6e, 0xe3, 0x20, 0x4d, 0x75, 0x72, 0x75, 0x74, 0x129, 0x67, 0x61, 0x53,
0xe3, 0x74, 0x61, 0x72, 0x1ebd, 0x53, 0xe3, 0x74, 0x69, 0x79, 0x61, 0x67,
0x75, 0x53, 0xe3, 0x74, 0x75, 0x20, 0x44, 0x75, 0x6d, 0x129, 0x67, 0x75,
0x53, 0xe3, 0x20, 0x50, 0x61, 0x75, 0x72, 0x75, 0x49, 0x74, 0x75, 0x6b,
0x75, 0x74, 0x75, 0x6d, 0x69, 0x74, 0x69, 0x53, 0x69, 0x74, 0x69, 0x6b,
0x61, 0x53, 0xe3, 0x20, 0x42, 0x61, 0x74, 0x75, 0x6c, 0x75, 0x6d, 0x65,
0x75, 0x53, 0xe3, 0x74, 0x69, 0x20, 0x4a, 0x75, 0x6e, 0x69, 0x53, 0xe3,
0x20, 0x4b, 0x69, 0x72, 0x69, 0x74, 0x75, 0x77, 0xe3, 0x75, 0x53, 0xe3,
0x74, 0x61, 0x20, 0x4c, 0x75, 0x73, 0x69, 0x79, 0x61, 0x53, 0xe3, 0x74,
0x69, 0x20, 0x54, 0x6f, 0x6d, 0xe1, 0x53, 0xe3, 0x20, 0x57, 0x69, 0x73,
0x1ebd, 0x74, 0x69, 0x50, 0x61, 0x72, 0x61, 0x6e, 0xe3, 0x20, 0x50, 0x69,
0x72, 0xe3, 0x74, 0xe3, 0x54, 0x65, 0x67, 0x75, 0x73, 0x69, 0x67, 0x61,
0x75, 0x70, 0x61, 0x54, 0x69, 0x78, 0x75, 0x72, 0x69, 0x54, 0x69, 0x6a,
0x75, 0x76, 0x1ef9, 0x6e, 0x1ef9, 0x54, 0x75, 0x72, 0x169, 0x74, 0x75, 0x54,
0x75, 0x74, 0x75, 0x72, 0x61, 0x56, 0xe3, 0x6b, 0x75, 0x77, 0x65, 0x72,
0x69, 0x4b, 0x61, 0x77, 0x61, 0x72, 0x75, 0x20, 0x4d, 0x75, 0x72, 0x75,
0x74, 0x129, 0x67, 0x61, 0x57, 0x69, 0x6e, 0x129, 0x70, 0x65, 0x67, 0x69,
0x4b, 0x61, 0x73, 0x65, 0x69, 0x44, 0x61, 0x77, 0x69, 0x4d, 0x61, 0x6b,
0x69, 0x6b, 0x75, 0x61, 0x72, 0x69, 0x4d, 0x61, 0x75, 0x73, 0x169, 0x4d,
0x61, 0x6b, 0x69, 0x4d, 0x75, 0x64, 0x75, 0x50, 0x61, 0x75, 0x6d, 0x65,
0x72, 0x65, 0x57, 0x6f, 0x73, 0x69, 0x74, 0x75, 0x6b, 0x69, 0x4c, 0x6f,
0x6e, 0x67, 0x79, 0x65, 0x61, 0x72, 0x20, 0x54, 0x61, 0x77, 0x61, 0x2d,
0x77, 0x61, 0x73, 0xfa, 0x41, 0x64, 0x1ebd, 0x41, 0x75, 0x6d, 0x61, 0x74,
0x69, 0x41, 0x6b, 0x69, 0x74, 0x75, 0x62, 0x69, 0x41, 0x73, 0x69, 0x67,
0x61, 0x62, 0x61, 0x74, 0x69, 0x4d, 0x61, 0x67, 0x6e, 0x61, 0x42, 0x61,
0x72, 0x1ebd, 0x69, 0x4d, 0xe1, 0x67, 0x2d, 0x6b, 0xf3, 0x6b, 0x69, 0x42,
0x69, 0x73, 0x69, 0x6b, 0x65, 0x6b, 0x69, 0x42, 0x75, 0x72, 0x75, 0x6e,
0x65, 0x69, 0x58, 0x69, 0x74, 0x61, 0x4b, 0x75, 0x72, 0x169, 0x62, 0x75,
0x44, 0x61, 0x6d, 0x61, 0x73, 0x69, 0x6b, 0x75, 0x44, 0x69, 0x72, 0x69,
0x44, 0x75, 0x78, 0xe3, 0x62, 0x69, 0x46, 0x61, 0x6d, 0x61, 0x67, 0x75,
0x69, 0x74, 0x61, 0x48, 0x65, 0x62, 0x75, 0x72, 0x169, 0x48, 0x169, 0x67,
0x20, 0x4b, 0x169, 0x67, 0x48, 0x6f, 0x77, 0x69, 0x64, 0x69, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x4a, 0x61, 0x6b, 0x61,
0x74, 0x61, 0x59, 0x61, 0x79, 0x61, 0x70, 0x75, 0x72, 0x61, 0x59, 0x65,
0x72, 0x75, 0x73, 0x61, 0x72, 0x1ebd, 0x69, 0x4b, 0xe3, 0x78, 0x61, 0x74,
0x69, 0x6b, 0x61, 0x4b, 0x61, 0x72, 0x61, 0x78, 0x69, 0x4b, 0x61, 0x74,
0x69, 0x6d, 0xe3, 0x64, 0x75, 0x4b, 0xe3, 0x64, 0x69, 0x67, 0x61, 0x4b,
0x61, 0x75, 0x6b, 0x75, 0x74, 0xe1, 0x4b, 0x61, 0x72, 0x61, 0x73, 0x69,
0x6e, 0x6f, 0x79, 0x61, 0x72, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x4b, 0x75,
0x61, 0x72, 0x61, 0x20, 0x52, 0x169, 0x70, 0x75, 0x72, 0x75, 0x4b, 0x75,
0x78, 0x129, 0x67, 0x69, 0x4d, 0x61, 0x67, 0x61, 0x64, 0xe3, 0x4d, 0x61,
0x73, 0x69, 0x6b, 0x61, 0x74, 0x69, 0x4b, 0x75, 0x73, 0x69, 0x6e, 0x65,
0x74, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x50, 0x69, 0x73, 0x61, 0x73,
0xfa, 0x53, 0x69, 0x62, 0x69, 0x72, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20,
0x50, 0x69, 0x73, 0x61, 0x73, 0xfa, 0x4f, 0x6d, 0x69, 0x73, 0x69, 0x6b,
0x69, 0x52, 0x61, 0x75, 0x72, 0x65, 0x50, 0x69, 0x6e, 0xf5, 0x20, 0x50,
0x1ebd, 0x69, 0x50, 0x169, 0x74, 0x69, 0x61, 0x6e, 0x61, 0x6b, 0x61, 0x50,
0x69, 0x169, 0x67, 0x69, 0xe3, 0x67, 0x69, 0x4b, 0x61, 0x74, 0x61, 0x72,
0x69, 0x4b, 0x6f, 0x73, 0x69, 0x74, 0x61, 0x6e, 0x61, 0x69, 0x4b, 0x69,
0x73, 0x69, 0x72, 0x6f, 0x64, 0x61, 0x53, 0x61, 0x6b, 0x61, 0x72, 0x69,
0x6e, 0x61, 0x53, 0x61, 0x6d, 0x61, 0x6b, 0xe3, 0x64, 0x61, 0x58, 0xe3,
0x67, 0x61, 0x69, 0x58, 0x65, 0x72, 0x65, 0x64, 0x69, 0x6e, 0x65, 0x6b,
0x6f, 0x72, 0x69, 0x6d, 0x69, 0x7a, 0x69, 0x6b, 0x69, 0x54, 0x61, 0x73,
0x69, 0x6b, 0x1ebd, 0x74, 0x69, 0x54, 0x65, 0x79, 0x65, 0x72, 0xe3, 0x54,
0x129, 0x70, 0x75, 0x54, 0x6f, 0x6b, 0x69, 0x79, 0x75, 0x54, 0x6f, 0x6d,
0x69, 0x73, 0x69, 0x6b, 0x69, 0x55, 0x72, 0xe3, 0x20, 0x42, 0x61, 0x74,
0x75, 0x72, 0x75, 0x55, 0x72, 0x169, 0x6b, 0x69, 0x55, 0x73, 0x69, 0x74,
0x69, 0x2d, 0x4e, 0x65, 0x72, 0x61, 0x56, 0x69, 0x1ebd, 0x74, 0x69, 0x61,
0x6e, 0x65, 0x57, 0x61, 0x72, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x69,
0x74, 0x75, 0x6b, 0x69, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x69, 0x73, 0x69,
0x6b, 0x69, 0x52, 0xe3, 0x67, 0x169, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x129, 0x62, 0x75, 0x67, 0x75, 0x59, 0x65, 0x72, 0x65, 0x77, 0xe3, 0x41,
0x73, 0x6f, 0x72, 0x69, 0x2d, 0x69, 0x74, 0x61, 0x42, 0x65, 0x6d, 0x75,
0x64, 0x61, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x79, 0x61, 0x2d, 0x69,
0x74, 0x61, 0x4b, 0x61, 0x62, 0x75, 0x20, 0x53, 0x75, 0x69, 0x6b, 0x69,
0x72, 0x69, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x6d, 0x61, 0x2d, 0x69, 0x74,
0x61, 0x20, 0x46, 0x61, 0x72, 0x75, 0x77, 0x65, 0x52, 0x65, 0x69, 0x6b,
0x69, 0x79, 0x61, 0x77, 0x69, 0x6b, 0x69, 0x47, 0x65, 0x75, 0x67, 0x69,
0x79, 0x61, 0x20, 0x53, 0x75, 0x20, 0x73, 0x75, 0xed, 0x53, 0xe3, 0x74,
0x61, 0x20, 0x45, 0x72, 0x1ebd, 0x6e, 0x61, 0x49, 0x73, 0x69, 0x74, 0xe3,
0x72, 0x65, 0x69, 0x41, 0x64, 0x65, 0x72, 0x61, 0x69, 0x64, 0x69, 0x42,
0x69, 0x72, 0x69, 0x62, 0x61, 0x6e, 0x69, 0x49, 0x77, 0x69, 0x74, 0x65,
0x72, 0x61, 0x20, 0x4d, 0x75, 0x70, 0x65, 0x6e, 0x61, 0xed, 0x72, 0x61,
0x44, 0x61, 0x72, 0x69, 0x77, 0x129, 0x45, 0x75, 0x6b, 0x61, 0x72, 0x61,
0x48, 0x6f, 0x62, 0x61, 0x74, 0x69, 0x52, 0x129, 0x64, 0x65, 0x6d, 0xe3,
0x4c, 0x75, 0x72, 0x75, 0x64, 0x69, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x4d,
0x65, 0x75, 0x62, 0xfa, 0x6e, 0x69, 0x50, 0x65, 0x72, 0x69, 0x74, 0x69,
0x53, 0x69, 0x64, 0x69, 0x6e, 0x65, 0x69, 0x41, 0x6d, 0x69, 0x74, 0x65,
0x72, 0x69, 0x64, 0xe3, 0xc3, 0x64, 0x75, 0x72, 0x61, 0x41, 0x73, 0x69,
0x74, 0x61, 0x72, 0x61, 0x6b, 0xe3, 0x41, 0x74, 0x65, 0x6e, 0x61, 0x2d,
0x69, 0x74, 0x61, 0x42, 0x65, 0x75, 0x67, 0x61, 0x72, 0x61, 0x64, 0x75,
0x42, 0x61, 0x72, 0x61, 0x74, 0x69, 0x73, 0x69, 0x6c, 0x61, 0x77, 0x61,
0x42, 0x75, 0x72, 0x75, 0x78, 0x65, 0x72, 0x61, 0x42, 0x75, 0x6b, 0x61,
0x72, 0x65, 0x74, 0x69, 0x42, 0x75, 0x64, 0x61, 0x70, 0x65, 0x74, 0x69,
0x42, 0x75, 0x73, 0x129, 0x67, 0x65, 0x6e, 0x69, 0x58, 0x69, 0x73, 0x69,
0x6e, 0x61, 0x75, 0x4b, 0x6f, 0x70, 0x1ebd, 0x79, 0x61, 0x67, 0x69, 0x44,
0x75, 0x62, 0x69, 0x72, 0x129, 0x47, 0x69, 0x62, 0x61, 0x72, 0x61, 0x75,
0x74, 0xe1, 0x47, 0x75, 0x77, 0x1ebd, 0x6e, 0x69, 0x73, 0x65, 0x69, 0x48,
0x65, 0x75, 0x73, 0x129, 0x6b, 0x65, 0x4d, 0xe3, 0x20, 0x4b, 0x61, 0x70,
0x75, 0xe3, 0x6d, 0x61, 0x4a, 0x65, 0x72, 0x73, 0x65, 0x69, 0x4b, 0x61,
0x72, 0x69, 0x6e, 0x129, 0x67, 0x61, 0x72, 0x61, 0x64, 0x75, 0x4b, 0x69,
0x72, 0x6f, 0x77, 0x69, 0x4b, 0x69, 0x79, 0x65, 0x77, 0x65, 0x52, 0x69,
0x75, 0x62, 0x69, 0x72, 0x69, 0x61, 0x6e, 0x61, 0x52, 0x75, 0x78, 0x1ebd,
0x62, 0x75, 0x67, 0x75, 0x4d, 0x61, 0x72, 0x69, 0x1ebd, 0x79, 0xe3, 0x4d,
0x129, 0x73, 0x69, 0x6b, 0x69, 0x55, 0x73, 0x69, 0x72, 0x75, 0x50, 0x75,
0x64, 0x69, 0x67, 0x75, 0x72, 0x69, 0x6b, 0x61, 0x53, 0xe3, 0x20, 0x4d,
0x61, 0x72, 0x69, 0x6e, 0x75, 0x53, 0x61, 0x72, 0x61, 0x79, 0x65, 0x77,
0x6f, 0x53, 0x61, 0x72, 0x61, 0x74, 0x75, 0x77, 0x6f, 0x53, 0x129, 0x77,
0x65, 0x72, 0x75, 0x70, 0x75, 0x49, 0x73, 0x69, 0x6b, 0x75, 0x70, 0x69,
0x79, 0x65, 0x53, 0x75, 0x66, 0x69, 0x79, 0x61, 0x55, 0x6c, 0x69, 0x61,
0x6e, 0x75, 0x77, 0x69, 0x6b, 0x69, 0x56, 0x61, 0x64, 0x75, 0x73, 0x69,
0x57, 0x61, 0x74, 0x69, 0x6b, 0x61, 0x6e, 0x75, 0x57, 0x69, 0x79, 0x65,
0x6e, 0x61, 0x56, 0x69, 0x77, 0x69, 0x75, 0x2d, 0x69, 0x74, 0x61, 0x57,
0x75, 0x67, 0x6f, 0x67, 0x61, 0x72, 0x61, 0x64, 0x75, 0x57, 0x61, 0x72,
0x69, 0x73, 0xf3, 0x77, 0x69, 0x79, 0x61, 0x53, 0x61, 0x67, 0x61, 0x72,
0x65, 0x62, 0x69, 0x5a, 0x75, 0x72, 0x69, 0x6b, 0x69, 0xc3, 0x74, 0x61,
0x6e, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x75, 0x58, 0x61, 0x67, 0x75, 0x2d,
0x69, 0x74, 0x61, 0x4b, 0x69, 0x72, 0x69, 0x74, 0x69, 0x6d, 0x61, 0x2d,
0x69, 0x74, 0x61, 0x4b, 0x75, 0x6b, 0x75, 0x2d, 0x69, 0x74, 0x61, 0x4b,
0x75, 0x6d, 0x75, 0x72, 0x65, 0x2d, 0x69, 0x74, 0x61, 0x4b, 0x65, 0x72,
0x67, 0x65, 0x6c, 0x1ebd, 0x4d, 0x61, 0xe9, 0x4d, 0x61, 0x75, 0x64, 0x69,
0x77, 0x61, 0x2d, 0x69, 0x74, 0x61, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x73,
0x69, 0x79, 0x75, 0x4d, 0x61, 0x79, 0x75, 0x74, 0x69, 0x59, 0x75, 0x6d,
0x75, 0x61, 0x74, 0x69, 0x72, 0x69, 0x73, 0x61, 0x77, 0x61, 0x41, 0x70,
0x69, 0x79, 0x61, 0xd3, 0x6b, 0x69, 0x72, 0xe3, 0x64, 0x69, 0x42, 0x75,
0x67, 0x61, 0x129, 0x77, 0x69, 0x72, 0x69, 0x58, 0x61, 0x74, 0x69, 0x6e,
0xe3, 0x50, 0x61, 0x73, 0x69, 0x6b, 0x75, 0x77, 0x61, 0x20, 0x4b, 0x61,
0x70, 0x75, 0xe3, 0x6d, 0x61, 0x45, 0x66, 0x61, 0x74, 0xe9, 0x46, 0x61,
0x6b, 0x61, 0x75, 0x66, 0x75, 0x47, 0x61, 0x72, 0x61, 0x70, 0x61, 0x67,
0x75, 0x2d, 0x69, 0x74, 0x61, 0x47, 0xe3, 0x62, 0x69, 0x65, 0x72, 0x65,
0x47, 0x75, 0x61, 0x64, 0x61, 0x75, 0x6b, 0x61, 0x6e, 0x61, 0x75, 0x47,
0x75, 0x77, 0xe3, 0x48, 0x75, 0x6e, 0x75, 0x72, 0x75, 0x72, 0x75, 0x1ebc,
0x64, 0x65, 0x62, 0x75, 0x72, 0x69, 0x4b, 0x75, 0x73, 0x69, 0x72, 0x61,
0x69, 0x4b, 0x75, 0x61, 0x79, 0x61, 0x72, 0x1ebd, 0x69, 0x4d, 0x61, 0x79,
0x75, 0x72, 0x75, 0x4d, 0x61, 0x6b, 0x65, 0x73, 0x61, 0x2d, 0x69, 0x74,
0x61, 0x4d, 0x69, 0x64, 0x69, 0x77, 0x65, 0x69, 0x4e, 0x69, 0x77, 0x65,
0x4e, 0x75, 0x72, 0x75, 0x66, 0x75, 0x6b, 0x69, 0x4e, 0x75, 0x6d, 0x65,
0x79, 0x61, 0x50, 0x61, 0x67, 0x75, 0x20, 0x50, 0x61, 0x67, 0x75, 0x50,
0x61, 0x72, 0x61, 0x75, 0x49, 0x67, 0x61, 0x72, 0x61, 0x70, 0x61, 0x77,
0x61, 0x20, 0x4d, 0x6f, 0x72, 0x65, 0x73, 0x62, 0x79, 0x52, 0x61, 0x72,
0x75, 0x74, 0x169, 0x67, 0x61, 0x53, 0x61, 0x69, 0x70, 0xe3, 0x54, 0x169,
0x67, 0x61, 0x74, 0x61, 0x70, 0x75, 0x57, 0x61, 0x72, 0x69, 0x2d, 0x69,
0x74, 0x61, 0x53, 0xe3, 0x20, 0x4b, 0x69, 0x72, 0x69, 0x74, 0x75, 0x77,
0x61, 0x6e, 0x41, 0x6e, 0x74, 0x69, 0x69, 0x67, 0x75, 0x61, 0x41, 0x72,
0x75, 0x75, 0x62, 0x61, 0x42, 0x61, 0x72, 0x62, 0x61, 0x61, 0x64, 0x6f,
0x73, 0x44, 0x6f, 0x6d, 0x69, 0x69, 0x6e, 0x69, 0x6b, 0x61, 0x47, 0x72,
0x65, 0x6e, 0x61, 0x61, 0x64, 0x61, 0x47, 0x75, 0x61, 0x74, 0x65, 0x6d,
0x61, 0x61, 0x6c, 0x61, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x20,
0x53, 0x74, 0x65, 0x65, 0x64, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c,
0x20, 0x4e, 0x75, 0x75, 0x72, 0x64, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74,
0x61, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x4e, 0x75, 0x75,
0x72, 0x64, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77,
0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x4e, 0x75, 0x75, 0x72,
0x64, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x41, 0x73, 0x63, 0x68,
0x67, 0x61, 0x62, 0x61, 0x74, 0x48, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x20,
0x4d, 0x69, 0x6e, 0x68, 0x20, 0x53, 0x74, 0x65, 0x65, 0x64, 0x4b, 0x75,
0x74, 0x73, 0x63, 0x68, 0x69, 0x6e, 0x67, 0x4e, 0x6f, 0x77, 0x6f, 0x6b,
0x75, 0x73, 0x6e, 0x65, 0x74, 0x73, 0x6b, 0x4b, 0x61, 0x74, 0x61, 0x61,
0x72, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x75, 0x72, 0x54, 0x61,
0x69, 0x70, 0x65, 0x65, 0x55, 0x72, 0x75, 0x6d, 0x74, 0x73, 0x63, 0x68,
0x69, 0x57, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x61, 0x6e, 0x65, 0x52, 0x61,
0x6e, 0x67, 0x75, 0x75, 0x6e, 0x41, 0x74, 0x73, 0x6f, 0x6f, 0x72, 0x65,
0x6e, 0x42, 0x65, 0x72, 0x6d, 0x75, 0x75, 0x64, 0x61, 0x4b, 0x61, 0x6e,
0x61, 0x61, 0x72, 0x65, 0x6e, 0x4b, 0x61, 0x70, 0x77, 0x65, 0x72, 0x64,
0x65, 0x6e, 0x46, 0xe4, 0x72, 0xf6, 0x65, 0x72, 0x6e, 0x53, 0xfc, 0xfc,
0x64, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x6e, 0x41, 0x74, 0x68,
0x65, 0x65, 0x6e, 0x4b, 0x6f, 0x70, 0x65, 0x6e, 0x68, 0x75, 0x75, 0x77,
0x65, 0x6e, 0x47, 0x69, 0x62, 0x72, 0x61, 0x6c, 0x74, 0x61, 0x61, 0x72,
0x45, 0x69, 0x6c, 0x75, 0x6e, 0x20, 0x4d, 0x61, 0x6e, 0x4c, 0x75, 0x6b,
0x73, 0x65, 0x6d, 0x62, 0x6f, 0x72, 0x69, 0x67, 0x52, 0x69, 0x69, 0x67,
0x61, 0x57, 0x61, 0x74, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x74, 0xe4,
0xe4, 0x64, 0x57, 0x69, 0x69, 0x6e, 0x4a, 0x75, 0x6c, 0x20, 0x45, 0x69,
0x6c, 0x75, 0x6e, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x20, 0x45, 0x69, 0x6c,
0x75, 0x6e, 0x65, 0x6e, 0x4d, 0x61, 0x6c, 0x65, 0x64, 0x69, 0x69, 0x77,
0x65, 0x6e, 0x50, 0x75, 0x61, 0x73, 0x6b, 0x20, 0x45, 0x69, 0x6c, 0x75,
0x6e, 0x65, 0x6e, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x45,
0x69, 0x6c, 0x75, 0x6e, 0xc1, 0x62, 0x65, 0x64, 0x6a, 0xe1, 0x6e, 0x41,
0x6b, 0x6b, 0x72, 0xe1, 0x41, 0x64, 0x64, 0x69, 0x73, 0x20, 0x41, 0x62,
0x61, 0x62, 0xe1, 0x41, 0x73, 0x6d, 0x61, 0x72, 0xe1, 0x42, 0x61, 0x6d,
0x61, 0x6b, 0xf3, 0x42, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x72, 0x61, 0x79,
0x42, 0x72, 0x61, 0x7a, 0x7a, 0x61, 0x77, 0x69, 0x6c, 0x61, 0x79, 0x42,
0x75, 0x6a, 0x6f, 0x6d, 0x62, 0x75, 0x72, 0x61, 0x4b, 0xe1, 0x65, 0x72,
0xf3, 0x4b, 0x61, 0x73, 0x61, 0x62, 0x6c, 0x61, 0x6e, 0x6b, 0xe1, 0x4b,
0x79, 0x75, 0x74, 0xe1, 0x44, 0xe1, 0x72, 0x20, 0x65, 0x73, 0x20, 0x73,
0x61, 0x6c, 0xe1, 0x6d, 0x44, 0x6a, 0x65, 0x62, 0xf3, 0x74, 0x69, 0x44,
0x6f, 0x75, 0xe1, 0x6c, 0xe1, 0x45, 0x6c, 0x20, 0xc1, 0x69, 0x75, 0x6e,
0x50, 0x65, 0x72, 0x69, 0x74, 0x68, 0xe1, 0xf3, 0x6e, 0x47, 0x61, 0x62,
0x6f, 0x72, 0x6f, 0x6e, 0xe9, 0x48, 0x61, 0x72, 0xe1, 0x72, 0xe9, 0x4a,
0x6f, 0x68, 0x61, 0x6e, 0x6e, 0x65, 0x73, 0x62, 0x61, 0x72, 0x67, 0x4a,
0x75, 0x62, 0xe1, 0x4b, 0x65, 0x6e, 0x73, 0x68, 0xe1, 0x73, 0x61, 0x4c,
0x61, 0x67, 0xf3, 0x73, 0x4c, 0x69, 0x62, 0x72, 0x65, 0x77, 0x69, 0x6c,
0x61, 0x79, 0x4c, 0xf3, 0x6d, 0xe9, 0x4c, 0x75, 0x61, 0x6e, 0x64, 0xe1,
0x4c, 0x75, 0x73, 0xe1, 0x6b, 0xe1, 0x4d, 0x61, 0x6c, 0xe1, 0x62, 0xf3,
0x4d, 0x61, 0x70, 0x75, 0x74, 0xf3, 0x4d, 0x61, 0x73, 0xe9, 0x72, 0xf3,
0x4d, 0x62, 0x61, 0x62, 0x61, 0x6e, 0xe9, 0x4d, 0xf3, 0x67, 0xe1, 0x64,
0xe9, 0x73, 0x68, 0xf3, 0x4d, 0x6f, 0x6e, 0x72, 0xf3, 0x77, 0x69, 0xe1,
0x4e, 0xe9, 0x72, 0x75, 0x62, 0x69, 0x4e, 0x64, 0x6a, 0x61, 0x6d, 0x65,
0x6e, 0xe1, 0x4e, 0x69, 0x61, 0x6d, 0xe9, 0x79, 0x50, 0x6f, 0x72, 0x74,
0xf3, 0x2d, 0x4e, 0xf3, 0x77, 0xf3, 0x53, 0xe1, 0xf3, 0x20, 0x54, 0x6f,
0x6d, 0xe9, 0x54, 0x72, 0x69, 0x70, 0xf3, 0x6c, 0x69, 0x57, 0x69, 0x6e,
0x64, 0x6f, 0x65, 0x6b, 0x41, 0x6e, 0x63, 0x68, 0x6f, 0x72, 0x61, 0x67,
0xe9, 0x41, 0x6e, 0x67, 0x75, 0x69, 0x6c, 0x6c, 0xe1, 0x41, 0x6e, 0x74,
0x69, 0x67, 0x75, 0xe1, 0x43, 0x68, 0x61, 0x74, 0x61, 0x6d, 0x61, 0x72,
0x6b, 0xe1, 0x4b, 0x6f, 0x72, 0x74, 0x6f, 0x62, 0x61, 0x4a, 0x6f, 0x6a,
0x75, 0x69, 0x4d, 0x65, 0x6e, 0x64, 0x6f, 0x7a, 0xe1, 0x55, 0x73, 0x68,
0x77, 0xe1, 0x69, 0xe1, 0x41, 0x72, 0x75, 0x62, 0xe1, 0x41, 0x73, 0x75,
0x6e, 0x63, 0x68, 0x69, 0xf3, 0x6e, 0x41, 0x74, 0x69, 0x6b, 0xf3, 0x6b,
0x61, 0x6e, 0x42, 0x61, 0x68, 0xed, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42,
0x61, 0x6e, 0x64, 0x65, 0x72, 0xe1, 0x73, 0x42, 0x6c, 0x61, 0x6e, 0x63,
0x68, 0x2d, 0x53, 0x61, 0x62, 0x6c, 0xf3, 0x6e, 0x42, 0xf3, 0xe1, 0x20,
0x57, 0x69, 0x73, 0x74, 0xe1, 0x4b, 0x61, 0x6d, 0x62, 0x72, 0x65, 0x6a,
0x20, 0x42, 0x61, 0x79, 0x43, 0x68, 0x61, 0x6d, 0x70, 0x6f, 0x20, 0x47,
0x72, 0x61, 0x6e, 0x64, 0x65, 0x43, 0x68, 0x61, 0x6e, 0x63, 0x68, 0x75,
0x6e, 0x43, 0x68, 0x61, 0x72, 0x61, 0x6b, 0xe1, 0x73, 0x43, 0x68, 0x61,
0x79, 0x65, 0x6e, 0x6e, 0x53, 0x68, 0x65, 0x6b, 0xe1, 0x67, 0xf3, 0x43,
0x68, 0x69, 0x68, 0x75, 0x61, 0x68, 0x75, 0xe1, 0x43, 0x68, 0x69, 0x75,
0x64, 0x61, 0x64, 0x20, 0x4a, 0x75, 0xe1, 0x72, 0x65, 0x7a, 0x4b, 0xf3,
0x73, 0x74, 0xe1, 0x20, 0x52, 0x69, 0x6b, 0xe1, 0x4b, 0x72, 0x65, 0x73,
0x74, 0xf3, 0x6e, 0x4b, 0x77, 0x61, 0x69, 0x62, 0xe1, 0x43, 0x68, 0x6f,
0x72, 0xe1, 0x63, 0x68, 0xe1, 0xf3, 0x44, 0x61, 0x6e, 0x6d, 0x61, 0x72,
0x6b, 0x73, 0x68, 0x61, 0x77, 0x6e, 0x44, 0x61, 0x77, 0x73, 0x6f, 0x6e,
0x20, 0x4b, 0x72, 0xe9, 0x6b, 0x44, 0x65, 0x6e, 0x77, 0xe9, 0x72, 0x44,
0x6f, 0x6d, 0x69, 0x6e, 0x69, 0x6b, 0xe1, 0x49, 0x72, 0x75, 0x6e, 0xe9,
0x70, 0xe9, 0x45, 0x6c, 0x20, 0x53, 0x61, 0x6c, 0x77, 0x61, 0x64, 0xf3,
0x72, 0x50, 0x6f, 0x72, 0x74, 0x61, 0x6c, 0x65, 0x7a, 0xe1, 0x47, 0x6c,
0x61, 0x79, 0x73, 0x20, 0x42, 0x61, 0x79, 0x47, 0x72, 0x65, 0x6e, 0x61,
0x64, 0xe1, 0x47, 0x77, 0xe1, 0x64, 0x65, 0x6c, 0xf3, 0x70, 0x47, 0x77,
0xe1, 0x74, 0xe9, 0x6d, 0xe1, 0x6c, 0xe1, 0x47, 0x77, 0xe1, 0x79, 0x61,
0x6b, 0x69, 0x6c, 0x47, 0x75, 0x79, 0xe1, 0x6e, 0x61, 0x48, 0x61, 0x77,
0xe1, 0x6e, 0xe1, 0x49, 0x6e, 0x64, 0x69, 0xe1, 0x6e, 0xe1, 0x70, 0x6f,
0x6c, 0x69, 0x73, 0x4b, 0x6e, 0x6f, 0x6b, 0x73, 0x2c, 0x20, 0x49, 0x6e,
0x64, 0x69, 0xe1, 0x6e, 0xe1, 0x4d, 0x61, 0x72, 0x65, 0x6e, 0x67, 0xf3,
0x20, 0x49, 0x6e, 0x64, 0x69, 0xe1, 0x6e, 0xe1, 0x50, 0x65, 0x74, 0x65,
0x72, 0x73, 0x62, 0x75, 0x72, 0x67, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69,
0xe1, 0x6e, 0xe1, 0x54, 0x65, 0x6c, 0x6c, 0x20, 0x53, 0x69, 0x74, 0x68,
0x69, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0xe1, 0x6e, 0xe1, 0x57, 0x65,
0x77, 0x61, 0x79, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0xe1, 0x6e, 0xe1,
0x57, 0x69, 0x6e, 0x73, 0x65, 0x6e, 0x6e, 0x65, 0x73, 0x2c, 0x20, 0x49,
0x6e, 0x64, 0x69, 0xe1, 0x6e, 0xe1, 0x57, 0x69, 0x6e, 0x61, 0x6d, 0x61,
0x6b, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0xe1, 0x6e, 0xe1, 0x49, 0x6b,
0xe1, 0x6c, 0x77, 0x69, 0x74, 0x4a, 0x61, 0x6d, 0xe1, 0x65, 0x6b, 0xe1,
0x4a, 0x75, 0x6e, 0x69, 0x75, 0x4c, 0x75, 0x69, 0x73, 0x77, 0x69, 0x6c,
0x6c, 0x61, 0x79, 0x4d, 0x6f, 0x6e, 0x74, 0x69, 0x63, 0x68, 0x65, 0x6c,
0x6c, 0xf3, 0x2c, 0x20, 0x4b, 0x65, 0x6e, 0x74, 0x75, 0x6b, 0x69, 0x4c,
0xe1, 0x20, 0x50, 0x61, 0x7a, 0x4c, 0x69, 0x6d, 0xe1, 0x4c, 0xe1, 0x73,
0x20, 0x45, 0x6e, 0x6a, 0x65, 0x6c, 0x65, 0x73, 0x4d, 0x61, 0x63, 0x68,
0x69, 0xf3, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0xf3, 0x20, 0x53, 0x68,
0x61, 0x68, 0x72, 0x4d, 0x65, 0x6b, 0x77, 0x65, 0x6c, 0x6f, 0x6e, 0x4d,
0x6f, 0x6e, 0x74, 0x65, 0x72, 0x72, 0x61, 0x79, 0x4d, 0x6f, 0x6e, 0x74,
0x65, 0x77, 0x69, 0x64, 0x69, 0xf3, 0x4e, 0x79, 0x75, 0x20, 0x59, 0xe1,
0x72, 0x6b, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0xe1, 0x42, 0x65, 0x75,
0x6c, 0x61, 0x68, 0x2c, 0x20, 0x4e, 0xe1, 0x72, 0x74, 0x20, 0x44, 0x61,
0x6b, 0xf3, 0x74, 0xe1, 0x53, 0x61, 0x6e, 0x74, 0x68, 0x61, 0x72, 0x2c,
0x20, 0x4e, 0xe1, 0x72, 0x74, 0x20, 0x44, 0x61, 0x6b, 0xf3, 0x74, 0xe1,
0x4e, 0x79, 0x75, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x4e,
0xe1, 0x72, 0x74, 0x20, 0x44, 0x61, 0x6b, 0xf3, 0x74, 0xe1, 0x50, 0x61,
0x6e, 0xe1, 0x6d, 0xe1, 0x50, 0x61, 0x72, 0x61, 0x6d, 0x61, 0x72, 0x69,
0x62, 0xf3, 0x50, 0xf3, 0x65, 0x6e, 0x69, 0x6b, 0x73, 0x50, 0xf3, 0x72,
0x74, 0x2d, 0x61, 0x75, 0x2d, 0x50, 0x72, 0x65, 0x6e, 0x73, 0x50, 0xf3,
0x72, 0x74, 0x20, 0x45, 0x73, 0x70, 0x69, 0x6e, 0x50, 0x6f, 0x72, 0x74,
0xf3, 0x20, 0x77, 0x65, 0x6c, 0x68, 0xf3, 0x50, 0x75, 0x65, 0x72, 0x74,
0xf3, 0x20, 0x52, 0x69, 0x6b, 0xf3, 0x52, 0x65, 0x63, 0x68, 0x69, 0x70,
0xe9, 0x52, 0x65, 0x73, 0xf3, 0x6c, 0x75, 0x74, 0x52, 0x69, 0xf3, 0x20,
0x42, 0x72, 0x61, 0x6e, 0x6b, 0xf3, 0x53, 0x61, 0x6e, 0x74, 0x69, 0x61,
0x67, 0xf3, 0x53, 0x61, 0x6e, 0x74, 0xf3, 0x20, 0x44, 0x6f, 0x6d, 0x69,
0x6e, 0x67, 0xf3, 0x53, 0xe1, 0xf3, 0x20, 0x50, 0x61, 0x75, 0x6c, 0xf3,
0x49, 0x74, 0x74, 0x6f, 0x6b, 0x6b, 0x6f, 0x72, 0x74, 0x75, 0x72, 0x6d,
0x69, 0x74, 0x53, 0x74, 0x2e, 0x20, 0x4c, 0x75, 0x73, 0x69, 0xe1, 0x53,
0x74, 0x2e, 0x20, 0x54, 0xf3, 0x6d, 0x61, 0x73, 0x53, 0x74, 0x2e, 0x20,
0x57, 0x69, 0x6e, 0x73, 0x65, 0x6e, 0x74, 0x53, 0x77, 0x69, 0x66, 0x74,
0x20, 0x4b, 0x61, 0x72, 0x72, 0x61, 0x6e, 0x74, 0x54, 0x65, 0x67, 0x75,
0x63, 0x68, 0x69, 0x67, 0x61, 0x6c, 0x70, 0xe1, 0x54, 0x75, 0x6c, 0xe9,
0x54, 0x69, 0x6a, 0x75, 0xe1, 0x6e, 0x61, 0x54, 0x6f, 0x72, 0x6f, 0x6e,
0x74, 0xf3, 0x54, 0x6f, 0x72, 0x74, 0x6f, 0x6c, 0xe1, 0x57, 0x61, 0x6e,
0x6b, 0x75, 0x77, 0x61, 0x72, 0x4b, 0xe1, 0x73, 0xe9, 0x44, 0x61, 0x77,
0x69, 0x73, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x20, 0x55, 0x72, 0x77,
0x69, 0x6c, 0x61, 0x4d, 0x61, 0x6b, 0x77, 0xe1, 0x65, 0x72, 0x53, 0x79,
0xf3, 0x77, 0xe1, 0x41, 0x6b, 0x74, 0x75, 0x62, 0xe9, 0x42, 0x61, 0x68,
0x72, 0x65, 0x6e, 0x42, 0x65, 0x6e, 0x67, 0x6b, 0xe1, 0x6b, 0x42, 0xe9,
0x72, 0x75, 0x74, 0x42, 0x72, 0x75, 0x6e, 0xe1, 0x69, 0x4b, 0xf3, 0x6c,
0x61, 0x6d, 0x62, 0xf3, 0x44, 0x61, 0x6d, 0xe1, 0x73, 0x6b, 0x61, 0x73,
0x44, 0x65, 0x68, 0x6c, 0x69, 0x44, 0x61, 0x62, 0x61, 0x69, 0x44, 0x6f,
0x73, 0x68, 0x61, 0x6d, 0x62, 0x65, 0x68, 0x50, 0x61, 0x6d, 0x61, 0x67,
0x75, 0x73, 0x74, 0xe1, 0x48, 0xf3, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d,
0x65, 0x6e, 0x6e, 0x48, 0xe1, 0x6e, 0x67, 0x20, 0x4b, 0xe1, 0x6e, 0x67,
0x48, 0xf3, 0x77, 0x64, 0x4a, 0x61, 0x6b, 0xe1, 0x72, 0x74, 0xe1, 0x55,
0x72, 0x73, 0x68, 0x61, 0x6c, 0x69, 0x6d, 0x4b, 0xe1, 0x62, 0x6f, 0x6c,
0x4b, 0x77, 0xe1, 0x6c, 0xe1, 0x20, 0x4c, 0x61, 0x6d, 0x70, 0x75, 0x72,
0x4b, 0x77, 0x61, 0x79, 0x74, 0x4d, 0x61, 0x6b, 0xe1, 0xf3, 0x4e, 0x69,
0x6b, 0xf3, 0x73, 0x68, 0x69, 0xe1, 0x4f, 0x72, 0xe1, 0x6c, 0x50, 0x79,
0x6f, 0x6e, 0x67, 0x79, 0xe1, 0x6e, 0x67, 0x47, 0x61, 0x74, 0x61, 0x72,
0x4b, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x79, 0x4b, 0x69, 0x7a, 0x69,
0x6c, 0x6f, 0x72, 0x64, 0xe1, 0x52, 0x79, 0xe1, 0x7a, 0x53, 0x65, 0xf3,
0x6c, 0x53, 0x65, 0x6e, 0x67, 0xe1, 0x70, 0x75, 0x72, 0x53, 0x72, 0x65,
0x64, 0x6e, 0x65, 0x6b, 0xf3, 0x6c, 0x69, 0x6d, 0x73, 0x6b, 0x54, 0xe1,
0x69, 0x70, 0x69, 0x54, 0xe1, 0x73, 0x68, 0x6b, 0x61, 0x6e, 0x64, 0x54,
0x65, 0x62, 0x69, 0x6c, 0x69, 0x73, 0x69, 0x54, 0x65, 0x68, 0x72, 0xe1,
0x6e, 0x54, 0xf3, 0x6b, 0x79, 0xf3, 0x57, 0x69, 0x65, 0x6e, 0x74, 0x69,
0xe1, 0x6e, 0xe9, 0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0xf3, 0x73, 0x74,
0x6f, 0x6b, 0x59, 0x65, 0x72, 0x65, 0x77, 0xe1, 0x6e, 0x4b, 0x61, 0x6e,
0x61, 0x72, 0x61, 0x79, 0x4b, 0x61, 0x70, 0x20, 0x57, 0x61, 0x72, 0x64,
0xe9, 0x52, 0x65, 0x79, 0x6b, 0x6a, 0x61, 0x77, 0x69, 0x6b, 0x53, 0x68,
0x65, 0x6d, 0xe1, 0x6c, 0x69, 0x20, 0x4a, 0xe1, 0x72, 0x6a, 0x69, 0xe1,
0x53, 0x74, 0x2e, 0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0xe1, 0x53, 0x74,
0xe1, 0x6e, 0x6c, 0xe9, 0x41, 0x64, 0x69, 0x6c, 0xe9, 0x64, 0x59, 0x75,
0x6b, 0x6c, 0xe1, 0x4d, 0x65, 0x6c, 0x62, 0x6f, 0x72, 0x6e, 0x53, 0x65,
0x64, 0x68, 0x6e, 0x69, 0xc1, 0x74, 0x65, 0x6e, 0x42, 0x72, 0x61, 0x74,
0x69, 0x73, 0x6c, 0x61, 0x77, 0xe1, 0x42, 0x75, 0x6b, 0x68, 0xe1, 0x72,
0x65, 0x73, 0x74, 0x42, 0x75, 0x64, 0xe1, 0x70, 0x65, 0x73, 0x74, 0x4b,
0x6f, 0x70, 0x65, 0x6e, 0x68, 0xe1, 0x67, 0x65, 0x6e, 0x47, 0x75, 0x65,
0x72, 0x6e, 0x73, 0xe9, 0x79, 0x45, 0x73, 0x74, 0x61, 0x6d, 0x62, 0xf3,
0x6c, 0x4a, 0x65, 0x72, 0x73, 0xe9, 0x79, 0x4b, 0x69, 0x72, 0xf3, 0x77,
0x4c, 0x6a, 0x75, 0x62, 0x6c, 0x6a, 0x61, 0x6e, 0xe1, 0x4c, 0x6f, 0x67,
0x7a, 0x65, 0x6d, 0x62, 0x6f, 0x72, 0x67, 0x4d, 0x6f, 0x6e, 0xe1, 0x6b,
0xf3, 0x4d, 0xe1, 0x73, 0x6b, 0xf3, 0xd3, 0x73, 0x6c, 0xf3, 0x50, 0x61,
0x79, 0x72, 0x65, 0x73, 0x50, 0x6f, 0x64, 0x67, 0x6f, 0x72, 0x69, 0x6b,
0xe1, 0x53, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6e, 0xf3, 0x53,
0x61, 0x72, 0x61, 0x6a, 0xe9, 0x77, 0xf3, 0x53, 0x69, 0x6d, 0x70, 0x65,
0x72, 0xf3, 0x70, 0xf3, 0x6c, 0x4b, 0x6f, 0x70, 0x6a, 0xe9, 0x53, 0x6f,
0x66, 0x69, 0xe1, 0x45, 0x73, 0x74, 0x68, 0xe1, 0x6b, 0x68, 0x6f, 0x6c,
0x6d, 0x54, 0x69, 0x72, 0xe1, 0x6e, 0x65, 0x55, 0x6c, 0x79, 0x61, 0x6e,
0xf3, 0x77, 0x73, 0x6b, 0x57, 0x69, 0x65, 0x6e, 0x6e, 0xe1, 0x57, 0x69,
0x6c, 0x6e, 0x69, 0x75, 0x73, 0x57, 0xe1, 0x72, 0x73, 0xe1, 0x5a, 0x79,
0x75, 0x72, 0x65, 0x6b, 0x68, 0x41, 0x6e, 0x74, 0x61, 0x6e, 0x61, 0x6e,
0x61, 0x72, 0x69, 0x77, 0xf3, 0x43, 0x68, 0x61, 0x67, 0xf3, 0x73, 0x4b,
0x72, 0x65, 0x73, 0x6d, 0x65, 0x73, 0x4b, 0x75, 0x6b, 0x75, 0x73, 0x4b,
0x6f, 0x6d, 0xf3, 0x72, 0xf3, 0x4b, 0x61, 0x72, 0x67, 0x77, 0x65, 0x6c,
0x65, 0x6e, 0x4d, 0xe1, 0x6c, 0x64, 0x69, 0x70, 0x4d, 0x75, 0x72, 0x69,
0x73, 0x68, 0x61, 0x73, 0x4d, 0x61, 0x79, 0xf3, 0x74, 0x61, 0x79, 0x52,
0xe9, 0x79, 0x75, 0x6e, 0x69, 0x61, 0x6e, 0x41, 0x70, 0x69, 0xe1, 0x41,
0x77, 0x6b, 0x6c, 0xe1, 0x6e, 0x64, 0x42, 0xf3, 0x67, 0xe1, 0x69, 0x6e,
0x77, 0x69, 0x6c, 0x61, 0x79, 0x49, 0x73, 0x74, 0x68, 0x61, 0x72, 0x45,
0x66, 0xe1, 0x74, 0x69, 0x46, 0x61, 0x6b, 0x61, 0xf3, 0x66, 0xf3, 0x4b,
0xf3, 0x73, 0x72, 0x61, 0x69, 0x4d, 0x61, 0x72, 0x6b, 0x65, 0x73, 0xe1,
0x73, 0x4e, 0x75, 0x72, 0x70, 0x6f, 0x6c, 0x6b, 0x50, 0x69, 0x74, 0x6b,
0x61, 0x72, 0x69, 0x6e, 0x41, 0x72, 0x67, 0xea, 0xc7, 0xe9, 0x75, 0x74,
0x61, 0x4e, 0x61, 0x69, 0x72, 0xf6, 0x62, 0x69, 0x54, 0x75, 0x6e, 0x65,
0x78, 0x69, 0x43, 0xf2, 0x72, 0x64, 0x6f, 0x76, 0x61, 0x43, 0x61, 0x69,
0x65, 0xf1, 0x61, 0x47, 0x75, 0x61, 0x64, 0x61, 0x6c, 0x75, 0x70, 0x70,
0x61, 0xc7, 0x69, 0x74, 0x74, 0xe6, 0x20, 0x64, 0x6f, 0x20, 0x4d, 0x65,
0x73, 0x63, 0x69, 0x63, 0x6f, 0x53, 0x61, 0x6e, 0x20, 0x50, 0x6f, 0x75,
0x6c, 0x6f, 0x53, 0x61, 0x6e, 0x20, 0x42, 0x65, 0x72, 0x74, 0x6f, 0x6d,
0xea, 0x53, 0x61, 0x6e, 0x20, 0x43, 0x72, 0x69, 0x73, 0x74, 0x6f, 0x66,
0x66, 0x61, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x20, 0x4c, 0x75, 0xe7, 0x69,
0x61, 0x53, 0x61, 0x6e, 0x20, 0x56, 0x69, 0xe7, 0x65, 0x6e, 0x73, 0x6f,
0x53, 0x63, 0x69, 0x6e, 0x67, 0x61, 0x70, 0xf4, 0x43, 0x61, 0x6e, 0xe4,
0x69, 0x65, 0x43, 0x61, 0x70, 0x70, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64,
0x65, 0x47, 0x65, 0xf2, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x73, 0x75, 0x64, 0x42, 0x65, 0x6c, 0x67, 0x72, 0x61, 0x64, 0x64, 0x6f,
0x44, 0x6f, 0x62, 0x6c, 0x69, 0x6e, 0x47, 0x69, 0x62, 0x65, 0x72, 0x74,
0xe2, 0x49, 0x73, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x6e,
0x4c, 0x69, 0x73, 0x62, 0x6f, 0xf1, 0x61, 0x4c, 0x75, 0x62, 0x69, 0x61,
0xf1, 0x61, 0x4d, 0x6f, 0x6e, 0x65, 0x67, 0x6f, 0xd2, 0x73, 0x6c, 0x6f,
0x52, 0x6f, 0x6d, 0x6d, 0x61, 0x54, 0x69, 0x72, 0x61, 0xf1, 0x61, 0x43,
0x6f, 0x6d, 0xf6, 0x72, 0x65, 0x4d, 0x61, 0x69, 0xf2, 0x74, 0x74, 0x61,
0x4d, 0x61, 0x72, 0x63, 0x68, 0x65, 0x69, 0x78, 0x69, 0x6a9, 0x6cc, 0x679,
0x627, 0x645, 0x627, 0x631, 0x633, 0x6cc, 0x6a9, 0x6cc, 0x645, 0x628, 0x631, 0x62c,
0x20, 0x633, 0x6cc, 0x20, 0x6a9, 0x6be, 0x627, 0x691, 0x6cc, 0x6a9, 0x631, 0x627,
0x633, 0x6cc, 0x633, 0x633, 0x6cc, 0x626, 0x6cc, 0x646, 0x6af, 0x644, 0x6cc, 0x633,
0x20, 0x633, 0x6cc, 0x20, 0x6a9, 0x6be, 0x627, 0x691, 0x6cc, 0x6af, 0x648, 0x633,
0x20, 0x633, 0x6cc, 0x20, 0x6a9, 0x6be, 0x627, 0x691, 0x6cc, 0x6c1, 0x627, 0x646,
0x6af, 0x20, 0x633, 0x6cc, 0x646, 0x6af, 0x633, 0x6cc, 0x628, 0x644, 0x633, 0x6cc,
0x679, 0x6be, 0x645, 0x646, 0x688, 0x648, 0x6a9, 0x648, 0x644, 0x633, 0x6cc, 0x62a,
0x627, 0x645, 0x633, 0x6cc, 0x624, 0x627, 0x688, 0x6cc, 0x644, 0x627, 0x626, 0x688,
0x41, 0x62, 0x69, 0x6a, 0x61, 0x14b, 0x41, 0x256, 0x69, 0x73, 0x20, 0x41,
0x62, 0x65, 0x62, 0x61, 0x41, 0x6c, 0x6a, 0x25b, 0x25b, 0x72, 0x41, 0x73,
0x6d, 0x61, 0x72, 0x61, 0x61, 0x42, 0x61, 0x6d, 0x61, 0x6b, 0x6f, 0x6f,
0x42, 0x61, 0x14b, 0x67, 0x69, 0x69, 0x42, 0x69, 0x73, 0x6f, 0x6f, 0x42,
0x6c, 0x61, 0x6e, 0x74, 0x69, 0x69, 0x72, 0x42, 0x72, 0x61, 0x73, 0x61,
0x66, 0x69, 0x6c, 0x42, 0x75, 0x6a, 0x75, 0x6d, 0x62, 0x75, 0x72, 0x61,
0x61, 0x4b, 0x61, 0x69, 0x72, 0x6f, 0x6f, 0x4b, 0x61, 0x73, 0x61, 0x62,
0x6c, 0x61, 0x14b, 0x6b, 0x61, 0x61, 0x53, 0x65, 0x79, 0x75, 0x74, 0x61,
0x4b, 0x6f, 0x6e, 0x61, 0x6b, 0x72, 0x69, 0x69, 0x189, 0x61, 0x6b, 0x61,
0x61, 0x72, 0x189, 0x61, 0x72, 0x25b, 0x73, 0x61, 0x6c, 0x61, 0x61, 0x6d,
0x4a, 0x69, 0x62, 0x75, 0x74, 0x69, 0x69, 0x189, 0x75, 0x77, 0x61, 0x6c,
0x61, 0x61, 0x190, 0x6c, 0x20, 0x41, 0x79, 0x75, 0x6e, 0x46, 0x72, 0x69,
0x69, 0x74, 0x61, 0x28a, 0x6e, 0x48, 0x61, 0x72, 0x61, 0x72, 0x65, 0x65,
0x59, 0x6f, 0x68, 0x61, 0x6e, 0x25b, 0x73, 0x62, 0x75, 0x75, 0x72, 0x4a,
0x75, 0x62, 0x61, 0x61, 0x4b, 0x61, 0x6d, 0x70, 0x61, 0x6c, 0x61, 0x61,
0x4b, 0x61, 0x72, 0x74, 0x75, 0x75, 0x6d, 0x4b, 0x69, 0x6e, 0x73, 0x68,
0x61, 0x73, 0x61, 0x61, 0x4c, 0x69, 0x62, 0x72, 0x1dd, 0x66, 0x69, 0x6c,
0x4c, 0x28a, 0x6d, 0x25b, 0x4c, 0x75, 0x77, 0x61, 0x6e, 0x256, 0x61, 0x61,
0x4c, 0x75, 0x73, 0x61, 0x6b, 0x61, 0x61, 0x4d, 0x61, 0x6c, 0x61, 0x62,
0x6f, 0x6f, 0x4d, 0x61, 0x70, 0x75, 0x74, 0x6f, 0x6f, 0x4d, 0x61, 0x73,
0x65, 0x72, 0x75, 0x75, 0x4d, 0x62, 0x61, 0x62, 0x61, 0x6e, 0x65, 0x65,
0x4d, 0x6f, 0x67, 0x61, 0x256, 0x69, 0x73, 0x68, 0x75, 0x75, 0x4d, 0x6f,
0x6e, 0x72, 0x6f, 0x66, 0x69, 0x79, 0x61, 0x61, 0x4e, 0x6a, 0x61, 0x6d,
0x65, 0x6e, 0x61, 0x61, 0x4e, 0x79, 0x25b, 0x6d, 0x25b, 0x4e, 0x75, 0x77,
0x61, 0x6b, 0x63, 0x254, 0x74, 0x57, 0x61, 0x67, 0x61, 0x256, 0x75, 0x67,
0x75, 0x50, 0x254, 0x72, 0x74, 0x6f, 0x20, 0x4e, 0x6f, 0x66, 0x6f, 0x6f,
0x53, 0x61, 0x6f, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x6b, 0x61, 0x6d, 0x70,
0xe1, 0x54, 0x69, 0x6e, 0x69, 0x69, 0x73, 0x57, 0x69, 0x6e, 0x256, 0x68,
0x75, 0x6b, 0x41, 0x256, 0x61, 0x6b, 0x41, 0x14b, 0x6b, 0x254, 0x72, 0x61,
0x61, 0x6a, 0x269, 0x41, 0x14b, 0x67, 0x75, 0x77, 0x69, 0x6c, 0x61, 0x61,
0x41, 0x6e, 0x74, 0x69, 0x67, 0x75, 0x77, 0x61, 0x61, 0x41, 0x72, 0x61,
0x67, 0x75, 0x77, 0x65, 0x65, 0x6e, 0x61, 0x42, 0x75, 0x77, 0x65, 0x6e,
0x254, 0x73, 0x20, 0x41, 0x69, 0x72, 0x25b, 0x73, 0x4b, 0x61, 0x74, 0x61,
0x6d, 0x61, 0x72, 0x6b, 0x61, 0x61, 0x4b, 0x254, 0x72, 0x256, 0x6f, 0x62,
0x61, 0x61, 0x48, 0x75, 0x68, 0x75, 0x79, 0x69, 0x4c, 0x61, 0x20, 0x52,
0x69, 0x79, 0x254, 0x68, 0x61, 0x4d, 0x25b, 0x6e, 0x256, 0x254, 0x73, 0x61,
0x61, 0x52, 0x69, 0x79, 0x6f, 0x20, 0x47, 0x61, 0x6c, 0x65, 0x67, 0x254,
0x73, 0x53, 0x61, 0x6e, 0x20, 0x48, 0x75, 0x77, 0x61, 0x6e, 0x55, 0x73,
0x68, 0x75, 0x77, 0x65, 0x79, 0x61, 0x61, 0x41, 0x72, 0x75, 0x62, 0x61,
0x61, 0x41, 0x73, 0x75, 0x6e, 0x73, 0x69, 0x79, 0x254, 0x6e, 0x42, 0x61,
0x68, 0x69, 0x79, 0x61, 0x61, 0x42, 0x61, 0x68, 0x69, 0x69, 0x79, 0x61,
0x20, 0x256, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x256, 0x65, 0x72, 0x61, 0x61,
0x73, 0x269, 0x42, 0x61, 0x72, 0x62, 0x61, 0x256, 0x254, 0x254, 0x73, 0x269,
0x42, 0x65, 0x6c, 0x25b, 0x6d, 0x42, 0x6c, 0x61, 0x14b, 0x2d, 0x53, 0x61,
0x62, 0x6c, 0x254, 0x14b, 0x42, 0x6f, 0x77, 0x61, 0x20, 0x46, 0x69, 0x73,
0x74, 0x61, 0x61, 0x42, 0x254, 0x77, 0x61, 0x61, 0x73, 0x269, 0x4b, 0x61,
0x6d, 0x62, 0x72, 0x69, 0x69, 0x6a, 0x269, 0x20, 0x42, 0x61, 0x269, 0x43,
0x61, 0x6d, 0x70, 0x6f, 0x20, 0x47, 0x72, 0x61, 0x6e, 0x256, 0x65, 0x65,
0x4b, 0x61, 0x14b, 0x6b, 0x75, 0x6e, 0x4b, 0x61, 0x79, 0x25b, 0x25b, 0x6e,
0x4b, 0x61, 0x79, 0x69, 0x6d, 0x61, 0x61, 0x6e, 0x53, 0x68, 0x69, 0x6b,
0x61, 0x67, 0x6f, 0x6f, 0x53, 0x68, 0x69, 0x77, 0x61, 0x77, 0x61, 0x61,
0x53, 0x69, 0x77, 0x75, 0x256, 0x61, 0x61, 0x256, 0x20, 0x48, 0x75, 0x77,
0x61, 0x72, 0x25b, 0x73, 0x4b, 0x254, 0x73, 0x74, 0x61, 0x20, 0x52, 0x69,
0x6b, 0x61, 0x61, 0x4b, 0x72, 0x25b, 0x73, 0x74, 0x254, 0x6e, 0x4b, 0x75,
0x79, 0x61, 0x62, 0x61, 0x61, 0x4b, 0x75, 0x72, 0x61, 0x73, 0x61, 0x61,
0x77, 0x75, 0x189, 0x61, 0x6e, 0x1dd, 0x6d, 0x61, 0x72, 0x6b, 0x269, 0x68,
0x61, 0x28a, 0x6e, 0x189, 0x254, 0x254, 0x73, 0x1dd, 0x6e, 0x189, 0x254, 0x254,
0x73, 0x1dd, 0x6e, 0x20, 0x4b, 0x72, 0x69, 0x69, 0x6b, 0x189, 0x25b, 0x6e,
0x66, 0x61, 0x189, 0x69, 0x74, 0x72, 0x254, 0x254, 0x269, 0x189, 0x6f, 0x6d,
0x69, 0x6e, 0x69, 0x6b, 0x61, 0x61, 0x190, 0x256, 0x6d, 0x254, 0x6e, 0x74,
0x254, 0x6e, 0x190, 0x269, 0x72, 0x75, 0x6e, 0x65, 0x70, 0x65, 0x65, 0x190,
0x6c, 0x20, 0x53, 0x61, 0x6c, 0x66, 0x61, 0x256, 0x254, 0x254, 0x72, 0x46,
0x254, 0x254, 0x72, 0x20, 0x4e, 0x25b, 0x6c, 0x73, 0x254, 0x6e, 0x46, 0x254,
0x72, 0x74, 0x61, 0x6c, 0x25b, 0x73, 0x61, 0x61, 0x47, 0x6c, 0x61, 0x61,
0x73, 0x20, 0x42, 0x61, 0x269, 0x47, 0x75, 0x75, 0x73, 0x20, 0x42, 0x61,
0x269, 0x47, 0x72, 0x61, 0x6e, 0x256, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x47,
0x72, 0x65, 0x6e, 0x61, 0x61, 0x256, 0x61, 0x47, 0x75, 0x77, 0x61, 0x256,
0x65, 0x6c, 0x75, 0x75, 0x70, 0x75, 0x47, 0x75, 0x77, 0x61, 0x74, 0x65,
0x6d, 0x61, 0x6c, 0x61, 0x61, 0x47, 0x75, 0x77, 0x61, 0x79, 0x61, 0x6b,
0x69, 0x6c, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x61, 0x48, 0x61, 0x6c,
0x69, 0x66, 0x61, 0x61, 0x73, 0x48, 0x61, 0x66, 0x61, 0x6e, 0x61, 0x61,
0x190, 0x72, 0x6d, 0x6f, 0x73, 0x69, 0x6c, 0x6f, 0x6f, 0x196, 0x6e, 0x256,
0x69, 0x79, 0x61, 0x61, 0x6e, 0x61, 0x70, 0x6f, 0x6c, 0x69, 0x4e, 0x254,
0x6b, 0x2c, 0x20, 0x196, 0x6e, 0x256, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x61,
0x4d, 0x61, 0x72, 0x25b, 0x14b, 0x67, 0x6f, 0x2c, 0x20, 0x196, 0x6e, 0x256,
0x69, 0x79, 0x61, 0x61, 0x6e, 0x61, 0x50, 0x65, 0x74, 0x25b, 0x72, 0x73,
0x62, 0x75, 0x75, 0x72, 0x2c, 0x20, 0x196, 0x6e, 0x256, 0x69, 0x79, 0x61,
0x61, 0x6e, 0x61, 0x54, 0x25b, 0x6c, 0x20, 0x53, 0x69, 0x74, 0x69, 0x2c,
0x20, 0x196, 0x6e, 0x256, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x61, 0x46, 0x65,
0x66, 0x25b, 0x269, 0x2c, 0x20, 0x196, 0x6e, 0x256, 0x69, 0x79, 0x61, 0x61,
0x6e, 0x61, 0x46, 0x25b, 0x14b, 0x73, 0x25b, 0x6e, 0x2c, 0x20, 0x196, 0x6e,
0x256, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x61, 0x57, 0x69, 0x6e, 0x61, 0x6d,
0x61, 0x61, 0x6b, 0x2c, 0x20, 0x196, 0x6e, 0x256, 0x69, 0x79, 0x61, 0x61,
0x6e, 0x61, 0x49, 0x6e, 0x75, 0x66, 0x69, 0x6b, 0x49, 0x6b, 0x61, 0x6c,
0x75, 0x77, 0x69, 0x69, 0x74, 0x4a, 0x61, 0x6d, 0x61, 0x269, 0x6b, 0x61,
0x4a, 0x69, 0x6e, 0x6f, 0x6f, 0x4c, 0x75, 0x77, 0x69, 0x66, 0x69, 0x69,
0x6c, 0x4d, 0x254, 0x6e, 0x74, 0x69, 0x73, 0x68, 0x25b, 0x25b, 0x6c, 0x6f,
0x2c, 0x20, 0x4b, 0x25b, 0x6e, 0x74, 0x61, 0x6b, 0x69, 0x4b, 0x72, 0x61,
0x6c, 0x25b, 0x6e, 0x256, 0x61, 0x269, 0x6b, 0x4c, 0x69, 0x6d, 0x61, 0x61,
0x4c, 0x254, 0x73, 0x20, 0x41, 0x6e, 0x6a, 0x25b, 0x6c, 0x25b, 0x25b, 0x73,
0x4c, 0x6f, 0x77, 0x25b, 0x25b, 0x20, 0x50, 0x72, 0x25b, 0x14b, 0x73, 0x20,
0x4b, 0x77, 0x61, 0x74, 0x61, 0x4d, 0x61, 0x73, 0x25b, 0x269, 0x79, 0x6f,
0x6f, 0x4d, 0x61, 0x6e, 0x61, 0x28a, 0x73, 0x4d, 0x61, 0x72, 0x69, 0x67,
0x6f, 0x6f, 0x4d, 0x61, 0x72, 0x74, 0x69, 0x6e, 0x69, 0x69, 0x6b, 0x69,
0x4d, 0x61, 0x74, 0x61, 0x6d, 0x6f, 0x72, 0x6f, 0x6f, 0x73, 0x4d, 0x61,
0x73, 0x61, 0x74, 0x1dd, 0x6c, 0x61, 0x6e, 0x4d, 0x65, 0x72, 0x69, 0x69,
0x256, 0x61, 0x4d, 0x65, 0x74, 0x65, 0x6c, 0x61, 0x6b, 0x61, 0x74, 0x65,
0x6c, 0x61, 0x61, 0x4d, 0x25b, 0x73, 0x69, 0x6b, 0x6f, 0x20, 0x53, 0x69,
0x74, 0x69, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x254, 0x14b, 0x4d, 0x254, 0x14b,
0x74, 0x254, 0x6e, 0x4d, 0x254, 0x6e, 0x74, 0x25b, 0x72, 0x25b, 0x269, 0x4d,
0x254, 0x6e, 0x74, 0x65, 0x66, 0x69, 0x256, 0x65, 0x79, 0x6f, 0x6f, 0x4d,
0x254, 0x6e, 0x73, 0x25b, 0x72, 0x61, 0x61, 0x74, 0x269, 0x4e, 0x69, 0x75,
0x20, 0x59, 0x254, 0x72, 0x6b, 0x4e, 0x6f, 0x72, 0x254, 0x6e, 0x79, 0x61,
0x42, 0x69, 0x75, 0x6c, 0x61, 0x2c, 0x20, 0x4e, 0x254, 0x254, 0x72, 0x20,
0x189, 0x61, 0x6b, 0x6f, 0x6f, 0x74, 0x61, 0x53, 0x25b, 0x6e, 0x74, 0x61,
0x2c, 0x20, 0x4e, 0x254, 0x254, 0x72, 0x20, 0x189, 0x61, 0x6b, 0x6f, 0x6f,
0x74, 0x61, 0x4e, 0x69, 0x75, 0x20, 0x53, 0x61, 0x6c, 0x25b, 0x6d, 0x2c,
0x20, 0x4e, 0x254, 0x254, 0x72, 0x20, 0x189, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x50, 0x61, 0x6e, 0x61, 0x6d, 0x61, 0x61, 0x50, 0x61, 0x72, 0x61, 0x6d,
0x61, 0x72, 0x69, 0x62, 0x6f, 0x6f, 0x46, 0x69, 0x6e, 0x69, 0x69, 0x73,
0x50, 0x254, 0x254, 0x72, 0x20, 0x6f, 0x20, 0x50, 0x72, 0x25b, 0x14b, 0x73,
0x50, 0x254, 0x254, 0x72, 0x20, 0x254, 0x66, 0x20, 0x53, 0x70, 0x25b, 0x25b,
0x6e, 0x50, 0x254, 0x72, 0x74, 0x6f, 0x20, 0x46, 0x65, 0x6c, 0x6f, 0x6f,
0x50, 0x254, 0x72, 0x74, 0x6f, 0x20, 0x52, 0x69, 0x6b, 0x6f, 0x6f, 0x52,
0x61, 0x14b, 0x6b, 0x269, 0x6e, 0x20, 0x196, 0x6e, 0x6c, 0x25b, 0x25b, 0x74,
0x269, 0x52, 0x65, 0x67, 0x69, 0x69, 0x6e, 0x61, 0x52, 0x25b, 0x73, 0x6f,
0x6c, 0x75, 0x75, 0x74, 0x52, 0x69, 0x79, 0x6f, 0x20, 0x42, 0x72, 0x61,
0x14b, 0x6b, 0x6f, 0x6f, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x25b, 0x6d,
0x53, 0x61, 0x6e, 0x74, 0x69, 0x79, 0x61, 0x67, 0x6f, 0x6f, 0x53, 0x61,
0x6e, 0x74, 0x6f, 0x20, 0x189, 0x6f, 0x6d, 0x269, 0x14b, 0x67, 0x6f, 0x53,
0x61, 0x6f, 0x20, 0x50, 0x61, 0x75, 0x6c, 0x6f, 0x6f, 0x49, 0x74, 0x6f,
0x6b, 0x254, 0x72, 0x74, 0x6f, 0x6f, 0x6d, 0x69, 0x69, 0x74, 0x69, 0x53,
0x69, 0x74, 0x6b, 0x61, 0x61, 0x53, 0x25b, 0x14b, 0x2d, 0x42, 0x61, 0x74,
0x6f, 0x6c, 0x6f, 0x6d, 0x61, 0x79, 0x254, 0x53, 0x25b, 0x14b, 0x20, 0x4a,
0x254, 0x254, 0x6e, 0x73, 0x53, 0x25b, 0x14b, 0x20, 0x4b, 0x69, 0x74, 0x73,
0x53, 0x25b, 0x14b, 0x20, 0x4c, 0x75, 0x73, 0x69, 0x79, 0x61, 0x53, 0x25b,
0x14b, 0x20, 0x54, 0x6f, 0x6d, 0x61, 0x61, 0x73, 0x269, 0x53, 0x25b, 0x14b,
0x20, 0x46, 0x269, 0x6e, 0x73, 0x61, 0x14b, 0x53, 0x1dd, 0x77, 0x269, 0x66,
0x74, 0x269, 0x20, 0x4b, 0x1dd, 0x72, 0x25b, 0x25b, 0x6e, 0x74, 0x269, 0x54,
0x75, 0x75, 0x6c, 0x65, 0x54, 0x69, 0x68, 0x75, 0x77, 0x61, 0x6e, 0x61,
0x54, 0x6f, 0x72, 0x254, 0x6e, 0x74, 0x6f, 0x6f, 0x54, 0x254, 0x72, 0x74,
0x254, 0x6c, 0x61, 0x61, 0x46, 0x25b, 0x14b, 0x6b, 0x75, 0x75, 0x66, 0x61,
0x57, 0x61, 0x269, 0x74, 0x68, 0x254, 0x254, 0x73, 0x57, 0x69, 0x6e, 0x69,
0x70, 0x25b, 0x25b, 0x67, 0x4b, 0x65, 0x65, 0x73, 0x69, 0x189, 0x65, 0x66,
0x69, 0x73, 0x189, 0x69, 0x6d, 0x254, 0x6e, 0x20, 0x189, 0x69, 0x72, 0x66,
0x69, 0x6c, 0x4d, 0x61, 0x6b, 0x61, 0x72, 0x69, 0x69, 0x4d, 0x254, 0x73,
0x254, 0x6e, 0x4d, 0x25b, 0x6b, 0x4d, 0x254, 0x72, 0x256, 0x6f, 0x6f, 0x50,
0x61, 0x6c, 0x6d, 0x25b, 0x25b, 0x72, 0x52, 0x6f, 0x74, 0x65, 0x72, 0x61,
0x61, 0x53, 0x69, 0x79, 0x6f, 0x77, 0x61, 0x61, 0x54, 0x72, 0x254, 0x6c,
0x46, 0x254, 0x73, 0x74, 0x254, 0x6b, 0x4c, 0x254, 0x14b, 0x79, 0x69, 0x69,
0x72, 0x62, 0x69, 0x79, 0x25b, 0x6e, 0x41, 0x256, 0x25b, 0x6e, 0x41, 0x6c,
0x6d, 0x61, 0x74, 0x69, 0x69, 0x41, 0x6d, 0x61, 0x61, 0x6e, 0x41, 0x6e,
0x61, 0x256, 0x69, 0x69, 0x72, 0x41, 0x6b, 0x74, 0x61, 0x28a, 0x41, 0x6b,
0x74, 0x6f, 0x62, 0x65, 0x65, 0x41, 0x73, 0x68, 0x67, 0x61, 0x62, 0x61,
0x61, 0x74, 0x41, 0x74, 0x69, 0x72, 0x61, 0x28a, 0x42, 0x61, 0x67, 0x256,
0x61, 0x61, 0x256, 0x42, 0x61, 0x72, 0x25b, 0x25b, 0x6e, 0x42, 0x61, 0x14b,
0x6b, 0x254, 0x6b, 0x42, 0x61, 0x72, 0x6e, 0x61, 0x77, 0x75, 0x6c, 0x42,
0x65, 0x69, 0x72, 0x75, 0x75, 0x74, 0x42, 0x69, 0x73, 0x68, 0x6b, 0x25b,
0x6b, 0x42, 0x72, 0x75, 0x6e, 0x25b, 0x269, 0x53, 0x68, 0x69, 0x74, 0x61,
0x61, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x6f, 0x6f, 0x189, 0x61, 0x6d,
0x61, 0x73, 0x6b, 0x254, 0x254, 0x73, 0x269, 0x189, 0x69, 0x6c, 0x69, 0x69,
0x189, 0x75, 0x62, 0x61, 0x269, 0x189, 0x75, 0x73, 0x68, 0x61, 0x6e, 0x62,
0x65, 0x65, 0x46, 0x61, 0x6d, 0x61, 0x67, 0x75, 0x73, 0x74, 0x61, 0x61,
0x47, 0x61, 0x73, 0x61, 0x61, 0x48, 0x65, 0x62, 0x72, 0x254, 0x6e, 0x48,
0x6f, 0x20, 0x43, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x20, 0x6b, 0x61, 0x4d,
0x70, 0xe1, 0x48, 0x254, 0x14b, 0x20, 0x4b, 0x254, 0x14b, 0x48, 0x254, 0x66,
0x256, 0x1dd, 0x4a, 0x61, 0x6b, 0x61, 0x74, 0x61, 0x61, 0x4a, 0x61, 0x79,
0x61, 0x70, 0x75, 0x72, 0x61, 0x61, 0x59, 0x65, 0x72, 0x75, 0x73, 0x61,
0x6c, 0x25b, 0x6d, 0x4b, 0x61, 0x6d, 0x73, 0x68, 0x61, 0x74, 0x6b, 0x61,
0x61, 0x4b, 0x61, 0x72, 0x61, 0x63, 0x269, 0x4b, 0x61, 0x74, 0x6d, 0x61,
0x6e, 0x256, 0x75, 0x75, 0x4b, 0x61, 0x6e, 0x256, 0x69, 0x67, 0x61, 0x61,
0x4b, 0x254, 0x6c, 0x6b, 0x61, 0x74, 0x61, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x79, 0x61, 0x72, 0x6b, 0x4b, 0x75, 0x77, 0x61, 0x6c, 0x61, 0x20,
0x4c, 0x75, 0x6d, 0x70, 0x75, 0x75, 0x72, 0x4b, 0x75, 0x63, 0x69, 0x14b,
0x4b, 0x6f, 0x77, 0x65, 0x65, 0x74, 0x69, 0x4d, 0x61, 0x6b, 0x61, 0x77,
0x6f, 0x6f, 0x4d, 0x61, 0x67, 0x61, 0x256, 0x61, 0x6e, 0x4d, 0x61, 0x6b,
0x61, 0x73, 0x61, 0x61, 0x72, 0x4d, 0x61, 0x6e, 0x69, 0x6c, 0x61, 0x61,
0x4d, 0x75, 0x73, 0x6b, 0x61, 0x61, 0x74, 0x4e, 0x6f, 0x66, 0x6f, 0x6b,
0x75, 0x73, 0x6e, 0x25b, 0x6b, 0x4e, 0x6f, 0x66, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x6b, 0x186, 0x6d, 0x73, 0x1dd, 0x6b, 0x1dd, 0x4f, 0x72, 0x61,
0x61, 0x6c, 0x4e, 0x254, 0x6d, 0x20, 0x50, 0x25b, 0x6e, 0x50, 0x254, 0x6e,
0x74, 0x69, 0x79, 0x61, 0x6e, 0x61, 0x6b, 0x50, 0x269, 0x79, 0x254, 0x14b,
0x79, 0x61, 0x14b, 0x4b, 0x254, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x269, 0x4b,
0x69, 0x73, 0x69, 0x6c, 0x254, 0x72, 0x256, 0x61, 0x61, 0x52, 0x69, 0x79,
0x61, 0x61, 0x256, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x6b, 0x61, 0x6e, 0x256,
0x53, 0x65, 0x77, 0x75, 0x6c, 0x53, 0x68, 0x61, 0x14b, 0x67, 0x61, 0x269,
0x53, 0x69, 0x14b, 0x67, 0x61, 0x70, 0x75, 0x75, 0x72, 0x53, 0x72, 0x25b,
0x256, 0x6e, 0x25b, 0x6b, 0x254, 0x6c, 0x69, 0x6d, 0x54, 0x61, 0x269, 0x70,
0x65, 0x69, 0x54, 0x61, 0x73, 0x68, 0x6b, 0x25b, 0x6e, 0x74, 0x54, 0x69,
0x62, 0x6c, 0x69, 0x73, 0x69, 0x69, 0x54, 0x65, 0x65, 0x72, 0x61, 0x61,
0x6e, 0x54, 0x69, 0x6d, 0x66, 0x75, 0x54, 0x6f, 0x6b, 0x69, 0x79, 0x6f,
0x6f, 0x54, 0x254, 0x6d, 0x73, 0x1dd, 0x6b, 0x1dd, 0x55, 0x6c, 0x61, 0x6e,
0x62, 0x61, 0x74, 0x254, 0x254, 0x72, 0x55, 0x72, 0x75, 0x6d, 0x6b, 0x69,
0x69, 0x55, 0x73, 0x74, 0x2d, 0x4e, 0x65, 0x72, 0x61, 0x61, 0x46, 0x69,
0x79, 0x65, 0x6e, 0x74, 0x69, 0x79, 0x61, 0x6e, 0x46, 0x6c, 0x61, 0x256,
0x69, 0x66, 0x254, 0x73, 0x74, 0x254, 0x6b, 0x59, 0x61, 0x14b, 0x67, 0x254,
0x6e, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x75, 0x59, 0x65, 0x72, 0x65, 0x66, 0x61, 0x61, 0x6e, 0x41, 0x73, 0x254,
0x254, 0x72, 0x42, 0x25b, 0x72, 0x6d, 0x75, 0x256, 0x61, 0x61, 0x4b, 0x61,
0x6e, 0x61, 0x72, 0x69, 0x69, 0x4b, 0x61, 0x70, 0x66, 0x25b, 0x25b, 0x72,
0x46, 0x61, 0x72, 0x6f, 0x69, 0x4d, 0x61, 0x256, 0x65, 0x69, 0x72, 0x61,
0x61, 0x52, 0x25b, 0x269, 0x6b, 0x79, 0x61, 0x66, 0x69, 0x6b, 0x4a, 0x254,
0x72, 0x6a, 0x69, 0x79, 0x61, 0x20, 0x67, 0x28a, 0x6e, 0x79, 0x269, 0x73,
0x6f, 0x6e, 0x6f, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x6a, 0x61, 0x53,
0x25b, 0x14b, 0x20, 0x45, 0x6c, 0x65, 0x6e, 0x61, 0x61, 0x53, 0x74, 0x61,
0x6e, 0x6c, 0x25b, 0x269, 0x41, 0x256, 0x65, 0x6c, 0x25b, 0x25b, 0x256, 0x42,
0x72, 0x69, 0x73, 0x62, 0x25b, 0x6e, 0x42, 0x72, 0x6f, 0x6b, 0x1dd, 0x6e,
0x20, 0x48, 0x69, 0x6c, 0x189, 0x61, 0x72, 0x66, 0x69, 0x6e, 0x55, 0x6b,
0x6c, 0x61, 0x61, 0x48, 0x6f, 0x62, 0x61, 0x61, 0x4c, 0x69, 0x6e, 0x256,
0x65, 0x6d, 0x61, 0x6e, 0x4c, 0x254, 0x72, 0x256, 0x20, 0x48, 0x6f, 0x6f,
0x4d, 0x25b, 0x6c, 0x62, 0x254, 0x72, 0x6e, 0x50, 0x25b, 0x72, 0x74, 0x53,
0x69, 0x256, 0x6e, 0x65, 0x65, 0x41, 0x6d, 0x73, 0x74, 0x25b, 0x72, 0x256,
0x61, 0x6d, 0x41, 0x6e, 0x256, 0x254, 0x72, 0x61, 0x61, 0x41, 0x73, 0x74,
0x72, 0x61, 0x6b, 0x61, 0x61, 0x6e, 0x41, 0x74, 0x25b, 0x6e, 0x42, 0x25b,
0x6c, 0x67, 0x72, 0x61, 0x61, 0x256, 0x42, 0x25b, 0x72, 0x6c, 0x25b, 0x14b,
0x42, 0x72, 0x61, 0x74, 0x69, 0x73, 0x6c, 0x61, 0x66, 0x61, 0x42, 0x72,
0x69, 0x73, 0x25b, 0x6c, 0x42, 0x75, 0x6b, 0x61, 0x72, 0x25b, 0x73, 0x42,
0x75, 0x256, 0x61, 0x70, 0x25b, 0x73, 0x42, 0x75, 0x73, 0x69, 0x14b, 0x67,
0x25b, 0x6e, 0x53, 0x68, 0x69, 0x73, 0x69, 0x6e, 0x6f, 0x6f, 0x4b, 0x6f,
0x70, 0x65, 0x6e, 0x61, 0x61, 0x67, 0x189, 0x254, 0x62, 0x6c, 0x25b, 0x14b,
0x47, 0x1dd, 0x72, 0x1dd, 0x6e, 0x73, 0x25b, 0x269, 0x48, 0x25b, 0x6c, 0x73,
0x69, 0x14b, 0x6b, 0x69, 0x69, 0x4d, 0x61, 0x6e, 0x20, 0x6b, 0x61, 0x41,
0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x49, 0x73, 0x74,
0x61, 0x6d, 0x62, 0x75, 0x75, 0x6c, 0x4b, 0x61, 0x6c, 0x69, 0x6e, 0x69,
0x14b, 0x67, 0x72, 0x61, 0x61, 0x256, 0x4b, 0x69, 0x72, 0x254, 0x254, 0x66,
0x4b, 0x69, 0x79, 0x25b, 0x66, 0x59, 0x75, 0x62, 0x1dd, 0x6c, 0x79, 0x61,
0x6e, 0x61, 0x4c, 0x254, 0x6e, 0x256, 0x254, 0x6e, 0x4c, 0x75, 0x73, 0x25b,
0x6d, 0x62, 0x75, 0x75, 0x72, 0x4d, 0x61, 0x256, 0x72, 0x69, 0x69, 0x256,
0x4d, 0x61, 0x72, 0x69, 0x79, 0x65, 0x68, 0x61, 0x6d, 0x4d, 0x269, 0x6e,
0x73, 0x4d, 0x6f, 0x6e, 0x61, 0x6b, 0x6f, 0x6f, 0x4d, 0x6f, 0x73, 0x6b,
0x75, 0x75, 0x186, 0x73, 0x6c, 0x6f, 0x6f, 0x50, 0x6f, 0x256, 0x67, 0x6f,
0x72, 0x69, 0x6b, 0x61, 0x61, 0x52, 0x69, 0x67, 0x61, 0x61, 0x53, 0x61,
0x6d, 0x61, 0x72, 0x61, 0x61, 0x53, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x72,
0x69, 0x6e, 0x6f, 0x6f, 0x53, 0x61, 0x72, 0x61, 0x79, 0x65, 0x66, 0x6f,
0x6f, 0x53, 0x61, 0x72, 0x61, 0x74, 0x254, 0x254, 0x66, 0x53, 0x6b, 0x254,
0x70, 0x79, 0x25b, 0x53, 0x74, 0x254, 0x6b, 0x68, 0x254, 0x6c, 0x6d, 0x54,
0x69, 0x72, 0x61, 0x6e, 0x61, 0x61, 0x55, 0x6c, 0x79, 0x61, 0x6e, 0x254,
0x254, 0x66, 0x46, 0x61, 0x256, 0x75, 0x73, 0x46, 0x61, 0x74, 0x69, 0x6b,
0x61, 0x14b, 0x46, 0x69, 0x79, 0x25b, 0x6e, 0x61, 0x46, 0x69, 0x6c, 0x6e,
0x69, 0x79, 0x75, 0x73, 0x46, 0x254, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x61, 0x256, 0x57, 0x61, 0x72, 0x73, 0x61, 0x28a, 0x53, 0x61, 0x67, 0x72,
0x25b, 0x62, 0x53, 0x75, 0x72, 0x69, 0x69, 0x6b, 0x41, 0x6e, 0x74, 0x61,
0x6e, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x66, 0x6f, 0x6f, 0x53, 0x68, 0x61,
0x67, 0x254, 0x73, 0x4e, 0x6f, 0x77, 0x25b, 0x6c, 0x4b, 0x6f, 0x6b, 0x6f,
0x6f, 0x73, 0x4b, 0x6f, 0x6d, 0x6f, 0x72, 0x6f, 0x6f, 0x4b, 0x25b, 0x72,
0x67, 0x65, 0x6c, 0x25b, 0x25b, 0x6e, 0x4d, 0x61, 0x68, 0x65, 0x65, 0x4d,
0x61, 0x6c, 0x256, 0x69, 0x69, 0x66, 0x75, 0x49, 0x6d, 0x6f, 0x72, 0x69,
0x73, 0x4d, 0x61, 0x79, 0x254, 0x254, 0x74, 0x269, 0x52, 0x65, 0x65, 0x6e,
0x69, 0x79, 0x254, 0x14b, 0x186, 0x254, 0x6b, 0x6c, 0x61, 0x6e, 0x256, 0x42,
0x75, 0x67, 0x25b, 0x25b, 0x14b, 0x66, 0x69, 0x6c, 0x53, 0x68, 0x61, 0x74,
0x61, 0x6d, 0x43, 0x75, 0x75, 0x6b, 0x45, 0x66, 0x61, 0x74, 0x65, 0x65,
0x46, 0x61, 0x6b, 0x61, 0x77, 0x6f, 0x66, 0x6f, 0x6f, 0x47, 0x61, 0x6d,
0x62, 0x69, 0x79, 0x65, 0x65, 0x47, 0x75, 0x77, 0x61, 0x256, 0x61, 0x6c,
0x6b, 0x61, 0x6e, 0x61, 0x6c, 0x4b, 0x61, 0x6e, 0x74, 0x254, 0x6e, 0x4b,
0x69, 0x72, 0x69, 0x6d, 0x61, 0x74, 0x69, 0x69, 0x4b, 0x254, 0x73, 0x72,
0x25b, 0x25b, 0x4b, 0x77, 0x61, 0x6a, 0x61, 0x6c, 0x25b, 0x25b, 0x6e, 0x4d,
0x61, 0x6a, 0x75, 0x72, 0x6f, 0x6f, 0x4d, 0x69, 0x256, 0x77, 0x65, 0x65,
0x4e, 0x61, 0x77, 0x75, 0x72, 0x75, 0x4e, 0x69, 0x77, 0x75, 0x79, 0x65,
0x4e, 0x254, 0x72, 0x66, 0x6f, 0x6f, 0x6b, 0x50, 0x61, 0x67, 0x6f, 0x20,
0x50, 0x61, 0x67, 0x6f, 0x6f, 0x50, 0x61, 0x6c, 0x61, 0x77, 0x6f, 0x6f,
0x50, 0x269, 0x74, 0x6b, 0x25b, 0x25b, 0x6e, 0x50, 0x6f, 0x6f, 0x6e, 0x70,
0x65, 0x65, 0x50, 0x254, 0x254, 0x72, 0x20, 0x4d, 0x254, 0x72, 0x25b, 0x73,
0x62, 0x69, 0x69, 0x52, 0x61, 0x72, 0x6f, 0x74, 0x254, 0x14b, 0x67, 0x61,
0x53, 0x61, 0x269, 0x70, 0x61, 0x6e, 0x54, 0x61, 0x72, 0x61, 0x77, 0x61,
0x61, 0x54, 0x254, 0x14b, 0x67, 0x61, 0x74, 0x61, 0x70, 0x75, 0x75, 0x57,
0x65, 0x65, 0x6b, 0x905, 0x932, 0x20, 0x906, 0x907, 0x92f, 0x942, 0x928, 0x3c,
0x92e, 0x91c, 0x93c, 0x93e, 0x91f, 0x932, 0x93e, 0x928, 0x921, 0x92c, 0x908, 0x915,
0x93e, 0x92c, 0x932, 0x926, 0x916, 0x923, 0x940, 0x20, 0x91c, 0x949, 0x930, 0x94d,
0x91c, 0x93f, 0x92f, 0x93e, 0x90f, 0x92e, 0x94d, 0x938, 0x94d, 0x91f, 0x930, 0x921,
0x92e, 0x92a, 0x948, 0x930, 0x938, 0x915, 0x94d, 0x930, 0x93f, 0x938, 0x92e, 0x93f,
0x938, 0x915, 0x948, 0x902, 0x91f, 0x928, 0x92a, 0x93e, 0x917, 0x94b, 0x20, 0x917,
0x902, 0x917, 0x94b, 0x41, 0x6c, 0x6a, 0x65, 0x72, 0x69, 0x41, 0x7a, 0x6d,
0x61, 0x72, 0x61, 0x42, 0x61, 0x6e, 0x67, 0x68, 0xec, 0x42, 0x69, 0x73,
0xe0, 0x6f, 0x42, 0x75, 0x6a, 0x75, 0x6e, 0x62, 0x75, 0x72, 0x61, 0x43,
0x61, 0x7a, 0x61, 0x62, 0x6c, 0x61, 0x6e, 0x63, 0x61, 0x4b, 0x61, 0x6e,
0x70, 0x61, 0x6c, 0x61, 0x4c, 0x75, 0x62, 0x75, 0x6e, 0x62, 0x61, 0x73,
0x68, 0x69, 0x4d, 0x61, 0x7a, 0x65, 0x72, 0x75, 0x4e, 0x62, 0x61, 0x62,
0x61, 0x6e, 0x65, 0x4d, 0x6f, 0x67, 0x61, 0x64, 0x69, 0x73, 0x6f, 0x55,
0x61, 0x67, 0x61, 0x64, 0x75, 0x67, 0xf9, 0xcc, 0x7a, 0x6f, 0x6c, 0x61,
0x20, 0x53, 0x2e, 0x20, 0x54, 0x6f, 0x6d, 0x61, 0x7a, 0x6f, 0x54, 0xf9,
0x6e, 0x65, 0x7a, 0x69, 0x43, 0xf2, 0x72, 0x64, 0x6f, 0x62, 0x61, 0x52,
0x69, 0x6f, 0x20, 0x47, 0xe0, 0x6c, 0x65, 0x67, 0x6f, 0x73, 0x42, 0x61,
0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x65, 0x72,
0x61, 0x73, 0x43, 0x61, 0x6e, 0x70, 0x6f, 0x20, 0x47, 0x72, 0x61, 0x6e,
0x64, 0x65, 0x43, 0x75, 0x69, 0x61, 0x62, 0xe0, 0x44, 0xe8, 0x6e, 0x76,
0x65, 0x72, 0x44, 0x6f, 0x6d, 0xe9, 0x6e, 0x65, 0x67, 0x61, 0x49, 0x6e,
0x64, 0x69, 0x61, 0x6e, 0xe0, 0x70, 0x6f, 0x6c, 0x69, 0x73, 0x4b, 0x6e,
0x6f, 0x78, 0x20, 0x28, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x29,
0x4d, 0x61, 0x72, 0x65, 0x6e, 0x67, 0x6f, 0x20, 0x28, 0x49, 0x6e, 0x64,
0x69, 0x61, 0x6e, 0x61, 0x29, 0x50, 0x65, 0x74, 0x65, 0x72, 0x73, 0x62,
0x75, 0x72, 0x67, 0x20, 0x28, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61,
0x29, 0x54, 0x65, 0x6c, 0x6c, 0x20, 0x43, 0x69, 0x74, 0x79, 0x20, 0x28,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x29, 0x56, 0x65, 0x76, 0x61,
0x79, 0x20, 0x28, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x29, 0x56,
0x69, 0x6e, 0x63, 0x65, 0x6e, 0x6e, 0x65, 0x73, 0x20, 0x28, 0x49, 0x6e,
0x64, 0x69, 0x61, 0x6e, 0x61, 0x29, 0x57, 0x69, 0x6e, 0x61, 0x6d, 0x61,
0x63, 0x20, 0x28, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x29, 0x4a,
0x61, 0x6d, 0xe0, 0x65, 0x67, 0x61, 0x4d, 0x6f, 0x6e, 0x74, 0x69, 0x63,
0x65, 0x6c, 0x6c, 0x6f, 0x20, 0x28, 0x4b, 0x65, 0x6e, 0x74, 0x75, 0x63,
0x6b, 0x79, 0x29, 0x4c, 0x6f, 0x73, 0x20, 0xc0, 0x6e, 0x67, 0x65, 0x6c,
0x65, 0x73, 0x4d, 0x61, 0x72, 0x74, 0x69, 0x6e, 0x69, 0x67, 0x61, 0x53,
0x69, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x4d, 0xe8, 0x73,
0x65, 0x67, 0x6f, 0x4d, 0x69, 0x63, 0x68, 0x65, 0x6c, 0x6f, 0x6e, 0x42,
0x65, 0x75, 0x6c, 0x61, 0x68, 0x20, 0x28, 0x4e, 0x6f, 0x72, 0x64, 0x20,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x29, 0x43, 0x65, 0x6e, 0x74, 0x65,
0x72, 0x20, 0x28, 0x4e, 0x6f, 0x72, 0x64, 0x20, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x29, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d,
0x20, 0x28, 0x4e, 0x6f, 0x72, 0x64, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74,
0x61, 0x29, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x20, 0x50, 0x72, 0xec, 0x6e,
0x73, 0x69, 0x70, 0x65, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x53, 0x70, 0x61, 0x67, 0x6e, 0x61, 0x53, 0x61, 0x6e, 0x20, 0x50,
0x6f, 0x6c, 0x6f, 0x53, 0x2e, 0x20, 0x42, 0x61, 0x72, 0x74, 0x6f, 0x6c,
0x6f, 0x6d, 0xe8, 0x6f, 0x53, 0x2e, 0x20, 0x4a, 0x6f, 0x61, 0x6e, 0x69,
0x53, 0x2e, 0x20, 0x43, 0x72, 0x69, 0x73, 0x74, 0x6f, 0x66, 0x65, 0x72,
0x53, 0x2e, 0x20, 0x4c, 0x75, 0x73, 0xec, 0x61, 0x53, 0x2e, 0x20, 0x56,
0x69, 0x6e, 0x63, 0x65, 0x6e, 0x73, 0x6f, 0xcc, 0x7a, 0x6f, 0x6c, 0x61,
0x20, 0x4d, 0x61, 0x63, 0x71, 0x75, 0x61, 0x72, 0x69, 0x65, 0x41, 0x6e,
0xe0, 0x64, 0x79, 0x72, 0x41, 0x6b, 0x74, 0xe0, 0x75, 0x41, 0x7a, 0x67,
0x61, 0x62, 0x61, 0x64, 0x41, 0x74, 0x79, 0x72, 0xe0, 0x75, 0x43, 0x6f,
0x6c, 0x6f, 0x6e, 0x62, 0x6f, 0x44, 0x75, 0x62, 0xe0, 0x69, 0x44, 0x75,
0x73, 0x61, 0x6e, 0x62, 0xe9, 0x4b, 0x61, 0x74, 0x6d, 0x61, 0x6e, 0x64,
0xf9, 0x48, 0xe0, 0x6e, 0x64, 0x69, 0x67, 0x61, 0x4b, 0x72, 0x61, 0x7a,
0x6e, 0x61, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x4b, 0x75, 0x61, 0x6c, 0x61,
0x20, 0x4c, 0x75, 0x6e, 0x70, 0x75, 0x72, 0x4d, 0x61, 0x63, 0xe0, 0x6f,
0x4e, 0x6f, 0x76, 0x6f, 0x6b, 0x75, 0x7a, 0x6e, 0x69, 0x65, 0x74, 0x73,
0x6b, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x6e, 0xe0, 0x69, 0x53, 0x68, 0x61,
0x6e, 0x67, 0x68, 0xe0, 0x69, 0x5a, 0x72, 0xe9, 0x64, 0x6e, 0x65, 0x6b,
0x6f, 0x6c, 0x69, 0x6d, 0x73, 0x6b, 0x54, 0x69, 0x6e, 0x70, 0x75, 0x55,
0x6c, 0x61, 0x6e, 0x20, 0x42, 0xe0, 0x74, 0x6f, 0x72, 0x55, 0x73, 0x74,
0x2d, 0x47, 0x6e, 0x65, 0x72, 0x61, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x4a, 0xe8, 0x72, 0x65,
0x76, 0x61, 0x6e, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x43, 0x61, 0x6e, 0x61,
0x72, 0x69, 0x65, 0x43, 0x61, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x6f,
0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x46, 0xe0, 0x72, 0x6f, 0x65, 0xcc,
0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x4d, 0x61, 0x64, 0xe8, 0x69, 0x72, 0x61,
0x52, 0x65, 0x6b, 0x69, 0x61, 0x76, 0x69, 0x6b, 0x47, 0x65, 0x6f, 0x72,
0x67, 0x69, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x73, 0x75, 0x64,
0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x53, 0x2e, 0x20, 0x45, 0x6c, 0x65,
0x6e, 0x61, 0x41, 0x64, 0x65, 0x6c, 0xe0, 0x69, 0x64, 0x65, 0x42, 0x72,
0x69, 0x7a, 0x62, 0x61, 0x6e, 0x65, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0xc0, 0x6d, 0x73,
0x74, 0x65, 0x72, 0x64, 0x61, 0x6d, 0x42, 0x72, 0x61, 0x74, 0x69, 0x7a,
0x6c, 0x61, 0x76, 0x61, 0x4a, 0x69, 0x62, 0x69, 0x6c, 0x74, 0x65, 0x72,
0x61, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61,
0x6e, 0x4c, 0x75, 0x62, 0x6c, 0x69, 0x61, 0x6e, 0x61, 0x4c, 0x75, 0x73,
0x65, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x4f, 0x7a, 0x6c, 0x6f, 0x50,
0x61, 0x72, 0x69, 0x6a, 0x69, 0x53, 0x69, 0x6e, 0x66, 0x65, 0x72, 0xf2,
0x70, 0x6f, 0x6c, 0x69, 0x53, 0x74, 0x6f, 0x63, 0x6f, 0x6c, 0x6d, 0x61,
0x55, 0x6c, 0x69, 0xe0, 0x6e, 0x6f, 0x73, 0x6b, 0x56, 0x61, 0x74, 0x65,
0x67, 0x61, 0x6e, 0x43, 0x69, 0x61, 0x67, 0x6f, 0x73, 0xcc, 0x7a, 0x6f,
0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x61, 0x64, 0x61, 0x6c, 0x65,
0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0xcc,
0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x43, 0x6f, 0x6d, 0x6f, 0x72, 0x65, 0xcc,
0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65,
0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x73,
0x69, 0x6f, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x4d, 0x61, 0x69, 0x6f,
0x74, 0x65, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x52, 0x65, 0x75, 0x6e,
0x69, 0x6f, 0x6e, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x42, 0x6f, 0x75,
0x67, 0x61, 0x69, 0x6e, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0xcc, 0x7a, 0x6f,
0x6c, 0x65, 0x20, 0x43, 0x69, 0x61, 0x74, 0x65, 0x6d, 0xcc, 0x7a, 0x6f,
0x6c, 0x65, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0xcc, 0x7a, 0x6f, 0x6c,
0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0xcc,
0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x45, 0x66, 0x61, 0x74, 0x65, 0x41, 0x74,
0x6f, 0x6c, 0x6f, 0x20, 0x46, 0x61, 0x6b, 0x61, 0x6f, 0x66, 0x6f, 0x41,
0x74, 0x6f, 0x6c, 0x6f, 0x20, 0x46, 0x75, 0x6e, 0x61, 0x66, 0x75, 0x74,
0x69, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69,
0x65, 0x72, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x47, 0x75, 0x61, 0x64,
0x61, 0x6c, 0x63, 0x61, 0x6e, 0x61, 0x6c, 0x41, 0x74, 0x6f, 0x6c, 0x6f,
0x20, 0x43, 0x61, 0x6e, 0x74, 0x6f, 0x6e, 0x41, 0x74, 0x6f, 0x6c, 0x6f,
0x20, 0x4b, 0x69, 0x72, 0x69, 0x74, 0x69, 0x6d, 0x61, 0x74, 0x69, 0xcc,
0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x41,
0x74, 0x6f, 0x6c, 0x6f, 0x20, 0x4b, 0x77, 0x61, 0x6a, 0x61, 0x6c, 0x65,
0x69, 0x6e, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x4d, 0x61, 0x72, 0x63,
0x68, 0x65, 0x7a, 0x69, 0x41, 0x74, 0x6f, 0x6c, 0x6f, 0x20, 0x4d, 0x69,
0x64, 0x77, 0x61, 0x79, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x4e, 0x61,
0x75, 0x72, 0x75, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x4e, 0x69, 0x75,
0x65, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x50, 0x61, 0x6c, 0xe0, 0x75, 0xcc, 0x7a, 0x6f, 0x6c, 0x61,
0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0xcc, 0x7a, 0x6f,
0x6c, 0x61, 0x20, 0x50, 0x6f, 0x6e, 0x70, 0xe8, 0x69, 0x50, 0x6f, 0x72,
0x74, 0x6f, 0x20, 0x4d, 0x6f, 0x72, 0x65, 0x73, 0x62, 0x79, 0xcc, 0x7a,
0x6f, 0x6c, 0x61, 0x20, 0x52, 0x61, 0x72, 0x6f, 0x74, 0x6f, 0x6e, 0x67,
0x61, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x53, 0x61, 0x69, 0x70, 0xe0,
0x6e, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x54, 0x61, 0x69, 0x74, 0x69,
0x41, 0x74, 0x6f, 0x6c, 0x6c, 0x6f, 0x20, 0x54, 0x61, 0x72, 0x61, 0x77,
0x61, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x74, 0x61, 0x70, 0x75, 0x41, 0x74, 0x6f, 0x6c, 0x6f, 0x20, 0x57, 0x61,
0x6b, 0x65, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x57, 0x61, 0x6c, 0x6c,
0x69, 0x73, 0x61, 0x62, 0x69, 0x64, 0x6a, 0x61, 0x6e, 0x65, 0x63, 0x72,
0x61, 0x61, 0x1e0d, 0x69, 0x73, 0x20, 0x61, 0x62, 0x61, 0x62, 0x61, 0x61,
0x6c, 0x6a, 0x12b, 0x79, 0x72, 0x73, 0x61, 0x73, 0x6d, 0x61, 0x72, 0x61,
0x62, 0x61, 0x6d, 0x61, 0x6b, 0x6f, 0x62, 0x61, 0x6e, 0x67, 0x75, 0x69,
0x62, 0x61, 0x6e, 0x6a, 0x75, 0x6c, 0x62, 0x69, 0x73, 0x73, 0x61, 0x75,
0x62, 0x6c, 0x61, 0x6e, 0x1e6d, 0x61, 0x65, 0x72, 0x62, 0x72, 0x61, 0x6a,
0x61, 0x76, 0x69, 0x6c, 0x6c, 0x62, 0x75, 0x6a, 0x75, 0x6d, 0x62, 0x75,
0x72, 0x61, 0x6b, 0x61, 0x69, 0x72, 0x6f, 0x6b, 0x61, 0x73, 0x61, 0x62,
0x6c, 0x61, 0x6e, 0x6b, 0x61, 0x73, 0x65, 0x75, 0x1e6d, 0x61, 0x6b, 0x6f,
0x6e, 0x61, 0x6b, 0x72, 0x12b, 0x1e0d, 0x6b, 0x61, 0x72, 0x64, 0x61, 0x72,
0x20, 0x65, 0x73, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x61, 0x6d, 0x64, 0x75,
0x61, 0x6c, 0x61, 0x65, 0x6c, 0x20, 0x61, 0x61, 0x69, 0x79, 0x75, 0x6e,
0x70, 0x72, 0x12b, 0x1e6d, 0x61, 0x75, 0x6e, 0x67, 0x61, 0x62, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x72, 0x61, 0x72, 0x65, 0x6a, 0x6f, 0x68, 0x61, 0x6e,
0x6e, 0x65, 0x73, 0x62, 0x75, 0x72, 0x67, 0x6a, 0x75, 0x62, 0x61, 0x6b,
0x6d, 0x70, 0x61, 0x6c, 0x61, 0x6b, 0x61, 0x72, 0x1e6d, 0x6f, 0x75, 0x6d,
0x6b, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x6b, 0x69, 0x6e, 0x73, 0x61, 0x73,
0x61, 0x6c, 0x61, 0x67, 0x6f, 0x73, 0x6c, 0x69, 0x62, 0x72, 0x65, 0x76,
0x69, 0x6c, 0x6c, 0x6c, 0x75, 0x61, 0x6e, 0x1e0d, 0x61, 0x6c, 0x75, 0x62,
0x75, 0x6d, 0x62, 0x61, 0x73, 0x69, 0x6c, 0x75, 0x73, 0x61, 0x6b, 0x61,
0x6d, 0x61, 0x6c, 0x61, 0x62, 0x6f, 0x6d, 0x61, 0x70, 0x75, 0x1e6d, 0x75,
0x6d, 0x61, 0x73, 0x65, 0x72, 0x75, 0x6d, 0x62, 0x61, 0x62, 0x61, 0x6e,
0x65, 0x6d, 0x6f, 0x67, 0x61, 0x1e0d, 0x69, 0x73, 0x75, 0x6d, 0x6f, 0x6e,
0x72, 0x6f, 0x76, 0x69, 0x61, 0x6e, 0x61, 0x69, 0x72, 0x6f, 0x62, 0x69,
0x6e, 0x6a, 0x61, 0x6d, 0x65, 0x6e, 0x61, 0x6e, 0x69, 0x79, 0x61, 0x6d,
0x65, 0x6e, 0x75, 0x65, 0x6b, 0x73, 0x61, 0x76, 0x61, 0x67, 0x64, 0x75,
0x67, 0x75, 0x70, 0x6f, 0x72, 0x1e6d, 0x6f, 0x2d, 0x1e47, 0x6f, 0x76, 0x6f,
0x73, 0x61, 0x6f, 0x20, 0x1e6d, 0x6f, 0x6d, 0x74, 0x72, 0x69, 0x70, 0x6f,
0x6c, 0x69, 0x1e6d, 0x75, 0x6e, 0x69, 0x73, 0x76, 0x69, 0x6e, 0x64, 0x68,
0x75, 0x6b, 0x61, 0x1e0d, 0x61, 0x6b, 0x61, 0x6e, 0x6b, 0x6f, 0x72, 0x61,
0x6a, 0x65, 0x6e, 0x67, 0x75, 0x69, 0x6c, 0x6c, 0x61, 0x65, 0x6e, 0x1e6d,
0x69, 0x67, 0x75, 0x61, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x69, 0x6e,
0x61, 0x62, 0x75, 0x65, 0x6e, 0x6f, 0x75, 0x73, 0x20, 0x61, 0x69, 0x72,
0x65, 0x73, 0x6b, 0x61, 0x74, 0x61, 0x6d, 0x61, 0x72, 0x6b, 0x61, 0x6b,
0x6f, 0x72, 0x1e0d, 0x61, 0x62, 0x61, 0x68, 0x75, 0x68, 0x75, 0x65, 0x6c,
0x61, 0x20, 0x72, 0x69, 0x6f, 0x6a, 0x61, 0x6d, 0x65, 0x6e, 0x1e0d, 0x6f,
0x6a, 0x61, 0x72, 0x69, 0x79, 0x6f, 0x20, 0x67, 0x61, 0x6c, 0x6c, 0x65,
0x67, 0x6f, 0x73, 0x73, 0x61, 0x6c, 0x1e6d, 0x61, 0x73, 0x61, 0x6e, 0x20,
0x68, 0x75, 0x61, 0x6e, 0x73, 0x61, 0x6e, 0x20, 0x6c, 0x75, 0x69, 0x73,
0x1e6d, 0x75, 0x6b, 0x16b, 0x6d, 0x6e, 0x75, 0x73, 0x76, 0x61, 0x69, 0x79,
0x61, 0x61, 0x72, 0x75, 0x62, 0x61, 0x65, 0x73, 0x75, 0x6e, 0x73, 0x69,
0x6f, 0x6e, 0x65, 0x1e6d, 0x69, 0x6b, 0x6f, 0x6b, 0x65, 0x6e, 0x62, 0x61,
0x68, 0x69, 0x61, 0x62, 0x61, 0x69, 0x61, 0x20, 0x62, 0x61, 0x6e, 0x1e0d,
0x65, 0x72, 0x61, 0x73, 0x62, 0x61, 0x72, 0x62, 0x61, 0x1e0d, 0x6f, 0x73,
0x62, 0x65, 0x6c, 0x65, 0x6d, 0x62, 0x65, 0x6c, 0x69, 0x6a, 0x62, 0x6c,
0x61, 0x6e, 0x6b, 0x2d, 0x73, 0x61, 0x62, 0x6c, 0x6f, 0x6e, 0x62, 0x6f,
0x61, 0x20, 0x76, 0x69, 0x73, 0x74, 0x61, 0x62, 0x6f, 0x67, 0x6f, 0x1e6d,
0x61, 0x62, 0x6f, 0x69, 0x73, 0x12b, 0x6b, 0x65, 0x6d, 0x62, 0x72, 0x69,
0x6a, 0x20, 0x62, 0x65, 0x6b, 0x61, 0x6d, 0x70, 0x6f, 0x20, 0x67, 0x72,
0x61, 0x6e, 0x1e0d, 0x65, 0x6b, 0x61, 0x6e, 0x6b, 0x75, 0x6e, 0x6b, 0x61,
0x72, 0x61, 0x6b, 0x61, 0x73, 0x6b, 0x65, 0x79, 0x65, 0x6e, 0x6b, 0x65,
0x69, 0x6d, 0x61, 0x6e, 0x63, 0x69, 0x6b, 0x61, 0x67, 0x6f, 0x63, 0x68,
0x69, 0x68, 0x75, 0x61, 0x68, 0x75, 0x61, 0x6b, 0x6f, 0x73, 0x1e6d, 0x61,
0x20, 0x72, 0x69, 0x6b, 0x61, 0x6b, 0x72, 0x65, 0x73, 0x1e6d, 0x6f, 0x6e,
0x6b, 0x75, 0x61, 0x61, 0x62, 0x61, 0x6b, 0x79, 0x75, 0x72, 0x61, 0x73,
0x6f, 0x1e0d, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x6b, 0x73, 0x61, 0x76, 0x6e,
0x1e0d, 0x61, 0x6f, 0x73, 0x6e, 0x1e0d, 0x61, 0x6f, 0x73, 0x6f, 0x6e, 0x20,
0x6b, 0x72, 0x12b, 0x6b, 0x1e0d, 0x65, 0x6e, 0x76, 0x65, 0x72, 0x1e0d, 0x65,
0x1e6d, 0x72, 0x6f, 0x69, 0x1e6d, 0x1e0d, 0x6f, 0x6d, 0x69, 0x6e, 0x69, 0x6b,
0x61, 0x65, 0x1e0d, 0x6d, 0x6f, 0x6e, 0x1e6d, 0x6f, 0x6e, 0x65, 0x69, 0x72,
0x75, 0x6e, 0x65, 0x70, 0x65, 0x65, 0x6c, 0x20, 0x73, 0x61, 0x6c, 0x76,
0x61, 0x1e0d, 0x6f, 0x72, 0x70, 0x68, 0x6f, 0x72, 0x74, 0x20, 0x6e, 0x65,
0x6c, 0x73, 0x6e, 0x70, 0x6f, 0x72, 0x74, 0x61, 0x6c, 0x65, 0x6a, 0x61,
0x67, 0x6c, 0x61, 0x73, 0x20, 0x62, 0x61, 0x79, 0x67, 0x6f, 0x6f, 0x73,
0x20, 0x62, 0x65, 0x67, 0x72, 0x61, 0x6e, 0x1e0d, 0x20, 0x1e6d, 0x75, 0x72,
0x6b, 0x67, 0x72, 0x65, 0x6e, 0x61, 0x1e0d, 0x61, 0x67, 0x75, 0x61, 0x1e0d,
0x65, 0x6c, 0x6f, 0x75, 0x70, 0x65, 0x67, 0x75, 0x61, 0x74, 0x65, 0x6d,
0x61, 0x6c, 0x61, 0x67, 0x75, 0x61, 0x6a, 0x61, 0x6b, 0x69, 0x6c, 0x67,
0x75, 0x79, 0x61, 0x6e, 0x61, 0x68, 0x65, 0x6c, 0x69, 0x70, 0x61, 0x6b,
0x73, 0x68, 0x61, 0x76, 0x61, 0x6e, 0x61, 0x69, 0x6e, 0x1e0d, 0x69, 0x61,
0x6e, 0x61, 0x70, 0x6f, 0x6c, 0x69, 0x73, 0x6e, 0x6f, 0x6b, 0x73, 0x2c,
0x20, 0x69, 0x6e, 0x1e0d, 0x69, 0x79, 0x61, 0x6e, 0x61, 0x6d, 0x61, 0x72,
0x65, 0x6e, 0x67, 0x6f, 0x2c, 0x20, 0x69, 0x6e, 0x1e0d, 0x69, 0x79, 0x61,
0x6e, 0x61, 0x70, 0x69, 0x1e6d, 0x74, 0x72, 0x73, 0x62, 0x65, 0x72, 0x67,
0x2c, 0x20, 0x69, 0x6e, 0x1e0d, 0x69, 0x79, 0x61, 0x6e, 0x61, 0x1e6d, 0x65,
0x6c, 0x6c, 0x20, 0x73, 0x69, 0x74, 0x79, 0x2c, 0x20, 0x69, 0x6e, 0x1e0d,
0x69, 0x79, 0x61, 0x6e, 0x61, 0x76, 0x69, 0x76, 0x69, 0x2c, 0x20, 0x69,
0x6e, 0x1e0d, 0x69, 0x79, 0x61, 0x6e, 0x61, 0x76, 0x69, 0x6e, 0x63, 0x65,
0x6e, 0x73, 0x2c, 0x20, 0x69, 0x6e, 0x1e0d, 0x69, 0x79, 0x61, 0x6e, 0x61,
0x76, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x6b, 0x2c, 0x20, 0x69, 0x6e, 0x1e0d,
0x69, 0x79, 0x61, 0x6e, 0x61, 0x69, 0x6e, 0x16b, 0x76, 0x69, 0x6b, 0x69,
0x6b, 0x61, 0x6c, 0x75, 0x69, 0x1e6d, 0x6a, 0x61, 0x6d, 0x61, 0x69, 0x6b,
0x61, 0x6a, 0x75, 0x6e, 0x6f, 0x76, 0x6c, 0x6f, 0x75, 0x69, 0x76, 0x69,
0x6c, 0x6c, 0x65, 0x6d, 0x6f, 0x6e, 0x1e6d, 0x69, 0x73, 0x65, 0x6c, 0x6c,
0x6f, 0x2c, 0x20, 0x6b, 0x65, 0x6e, 0x1e6d, 0x75, 0x6b, 0x12b, 0x6b, 0x72,
0x65, 0x6c, 0x65, 0x6e, 0x1e0d, 0x65, 0x69, 0x6b, 0x6c, 0x61, 0x20, 0x70,
0x61, 0x6a, 0x6c, 0x6f, 0x73, 0x20, 0x61, 0x6e, 0x6a, 0x65, 0x6c, 0x65,
0x73, 0x6c, 0x6f, 0x76, 0x65, 0x72, 0x20, 0x70, 0x72, 0x69, 0x6e, 0x63,
0x2019, 0x73, 0x20, 0x6b, 0x75, 0x76, 0x61, 0x1e6d, 0x61, 0x72, 0x6d, 0x61,
0x73, 0x12b, 0x6f, 0x6d, 0x61, 0x6e, 0x61, 0x67, 0x75, 0x61, 0x6d, 0x61,
0x6e, 0x61, 0x75, 0x73, 0x6d, 0x61, 0x72, 0x69, 0x67, 0x6f, 0x1e6d, 0x6d,
0x61, 0x72, 0x1e6d, 0x69, 0x6e, 0x69, 0x6b, 0x6d, 0x61, 0x1e6d, 0x61, 0x6d,
0x6f, 0x72, 0x6f, 0x73, 0x6d, 0x61, 0x73, 0x61, 0x1e6d, 0x6c, 0x61, 0x6e,
0x6d, 0x65, 0x6e, 0x6f, 0x6d, 0x69, 0x6e, 0x12b, 0x6d, 0x65, 0x72, 0x69,
0x1e0d, 0x61, 0x6d, 0x65, 0x1e6d, 0x6c, 0x61, 0x6b, 0x61, 0x1e6d, 0x6c, 0x61,
0x6d, 0x65, 0x6b, 0x73, 0x69, 0x63, 0x6f, 0x20, 0x73, 0x69, 0x1e6d, 0x79,
0x6d, 0x69, 0x6b, 0x76, 0x69, 0x6c, 0xf5, 0x6d, 0x6f, 0x6e, 0x6b, 0x1e6d,
0x6f, 0x6e, 0x6d, 0x6f, 0x6e, 0x1e6d, 0x65, 0x72, 0x72, 0x65, 0x6d, 0x6f,
0x6e, 0x1e6d, 0x65, 0x76, 0x69, 0x64, 0x69, 0x6f, 0x6d, 0x6f, 0x6e, 0x1e6d,
0x73, 0x65, 0x72, 0x72, 0x65, 0x1e6d, 0x6e, 0x61, 0x73, 0x61, 0x75, 0x6e,
0x69, 0x79, 0x75, 0x20, 0x79, 0x6f, 0x72, 0x6b, 0x6e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x62, 0x69, 0x79, 0x75, 0x6c, 0x61, 0x2c, 0x20, 0x75,
0x74, 0x74, 0x61, 0x72, 0x20, 0x1e0d, 0x61, 0x6b, 0x6f, 0x1e6d, 0x61, 0x6d,
0x61, 0x1e0d, 0x69, 0x6e, 0x12b, 0x20, 0x75, 0x74, 0x74, 0x61, 0x72, 0x20,
0x1e0d, 0x61, 0x6b, 0x6f, 0x1e6d, 0x61, 0x6e, 0x65, 0x75, 0x20, 0x73, 0x61,
0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x75, 0x74, 0x74, 0x61, 0x72, 0x20, 0x1e0d,
0x61, 0x6b, 0x6f, 0x1e6d, 0x61, 0x6e, 0x16b, 0x6b, 0x6f, 0x6a, 0x69, 0x6e,
0x61, 0x67, 0x61, 0x70, 0x61, 0x6e, 0x61, 0x6d, 0x61, 0x70, 0x61, 0x72,
0x61, 0x6d, 0x61, 0x72, 0x69, 0x62, 0x6f, 0x70, 0x69, 0x6e, 0x69, 0x6b,
0x73, 0x70, 0x6f, 0x72, 0x74, 0x2d, 0x61, 0x75, 0x2d, 0x70, 0x72, 0x69,
0x6e, 0x73, 0x65, 0x70, 0x6f, 0x72, 0x1e6d, 0x20, 0x61, 0x70, 0x20, 0x73,
0x70, 0x61, 0x69, 0x6e, 0x70, 0x6f, 0x72, 0x1e6d, 0x70, 0x20, 0x76, 0x65,
0x6c, 0x68, 0x6f, 0x70, 0x75, 0x65, 0x72, 0x74, 0x6f, 0x20, 0x72, 0x69,
0x6b, 0x6f, 0x70, 0x75, 0x1e47, 0x1e6d, 0x61, 0x20, 0x65, 0x72, 0x65, 0x6e,
0x61, 0x73, 0x72, 0x61, 0x6e, 0x6b, 0x69, 0x6e, 0x20, 0x69, 0x6e, 0x6c,
0x65, 0x64, 0x1e6d, 0x72, 0x65, 0x73, 0x69, 0x70, 0x69, 0x72, 0x65, 0x6a,
0x69, 0x6e, 0x61, 0x72, 0x65, 0x6a, 0x61, 0x6c, 0x79, 0x75, 0x1e6d, 0x72,
0x69, 0x6f, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x6b, 0x6f, 0x73, 0x61, 0x74,
0x61, 0x72, 0x69, 0x73, 0x61, 0x6e, 0x1e6d, 0x69, 0x61, 0x67, 0x6f, 0x73,
0x65, 0x6e, 0x74, 0x6f, 0x20, 0x1e0d, 0x6f, 0x6d, 0x69, 0x6e, 0x67, 0x6f,
0x73, 0x61, 0x6f, 0x20, 0x70, 0x61, 0x75, 0x6c, 0x6f, 0x69, 0x1e6d, 0x6f,
0x6b, 0x6f, 0x1e6d, 0x6f, 0x72, 0x6d, 0x69, 0x1e6d, 0x73, 0x69, 0x74, 0x6b,
0x61, 0x73, 0x65, 0x6e, 0x1e6d, 0x20, 0x62, 0x61, 0x72, 0x74, 0x65, 0x6c,
0x65, 0x6d, 0x69, 0x73, 0x65, 0x6e, 0x1e6d, 0x20, 0x6a, 0x6f, 0x6e, 0x73,
0x73, 0x65, 0x6e, 0x74, 0x20, 0x6b, 0x69, 0x1e6d, 0x1e6d, 0x73, 0x73, 0x65,
0x6e, 0x74, 0x20, 0x6c, 0x75, 0x73, 0x69, 0x61, 0x73, 0x65, 0x6e, 0x1e6d,
0x20, 0x74, 0x6f, 0x6d, 0x61, 0x73, 0x73, 0x65, 0x6e, 0x1e6d, 0x20, 0x76,
0x69, 0x6e, 0x73, 0x65, 0x6e, 0x1e6d, 0x73, 0x76, 0x69, 0x70, 0x1e6d, 0x20,
0x6b, 0x75, 0x72, 0x72, 0x65, 0x6e, 0x1e6d, 0x74, 0x65, 0x67, 0x75, 0x74,
0x75, 0x6c, 0x65, 0x74, 0x69, 0x68, 0x76, 0x61, 0x6e, 0x61, 0x1e6d, 0x6f,
0x72, 0x6f, 0x6e, 0x1e6d, 0x6f, 0x1e6d, 0x6f, 0x72, 0x1e6d, 0x6f, 0x6c, 0x61,
0x76, 0x61, 0x6e, 0x6b, 0x16b, 0x76, 0x65, 0x72, 0x76, 0x68, 0x61, 0x69,
0x1e6d, 0x68, 0x6f, 0x72, 0x73, 0x76, 0x69, 0x6e, 0x6e, 0x69, 0x70, 0x65,
0x67, 0x79, 0x61, 0x6b, 0x75, 0x1e6d, 0x61, 0x1e6d, 0x6b, 0x65, 0x73, 0x65,
0x65, 0x1e0d, 0x65, 0x76, 0x69, 0x73, 0x1e0d, 0x79, 0x75, 0x6d, 0x6f, 0x6e,
0x74, 0x20, 0x64, 0x65, 0x20, 0x75, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x6d,
0x65, 0x6b, 0x76, 0x61, 0x72, 0x69, 0x6d, 0x61, 0x76, 0x73, 0x6f, 0x6e,
0x6d, 0x65, 0x6b, 0x20, 0x6d, 0x75, 0x72, 0x1e0d, 0x6f, 0x70, 0x61, 0x6c,
0x6d, 0x65, 0x72, 0x72, 0x6f, 0x74, 0x65, 0x72, 0x61, 0x73, 0x79, 0x6f,
0x76, 0x61, 0x1e6d, 0x72, 0x6f, 0x6c, 0x6c, 0x76, 0x6f, 0x73, 0x1e6d, 0x6f,
0x6b, 0x6c, 0x6e, 0x67, 0x79, 0x61, 0x72, 0x62, 0x79, 0x65, 0x6e, 0x65,
0x1e0d, 0x65, 0x6e, 0x61, 0x6c, 0x6d, 0x61, 0x1e6d, 0x79, 0x61, 0x6d, 0x6d,
0x61, 0x6e, 0x61, 0x6e, 0x61, 0x1e0d, 0x69, 0x72, 0x61, 0x6b, 0x74, 0x61,
0x75, 0x61, 0x61, 0x6b, 0x74, 0x6f, 0x62, 0x65, 0x61, 0x73, 0x67, 0x61,
0x62, 0x61, 0x1e6d, 0x61, 0x74, 0x61, 0x72, 0x61, 0x75, 0x62, 0x61, 0x67,
0x64, 0x61, 0x64, 0x62, 0x61, 0x68, 0x72, 0x61, 0x69, 0x6e, 0x62, 0x61,
0x6b, 0x75, 0x62, 0x61, 0x6e, 0x67, 0x6b, 0x6f, 0x6b, 0x62, 0x61, 0x72,
0x6e, 0x61, 0x75, 0x6c, 0x62, 0x65, 0x69, 0x72, 0x75, 0x74, 0x62, 0x69,
0x73, 0x6b, 0x65, 0x6b, 0x62, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x63, 0x69,
0x74, 0x61, 0x6b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x6f, 0x1e0d, 0x61, 0x6d,
0x61, 0x73, 0x6b, 0x75, 0x73, 0x1e0d, 0x69, 0x6c, 0x69, 0x64, 0x75, 0x62,
0x61, 0x69, 0x64, 0x75, 0x73, 0x61, 0x6d, 0x62, 0x65, 0x70, 0x61, 0x6d,
0x61, 0x67, 0x75, 0x73, 0x74, 0x61, 0x67, 0x61, 0x6a, 0x61, 0x68, 0x65,
0x62, 0x72, 0x6f, 0x6e, 0x68, 0x6f, 0x20, 0x63, 0x69, 0x20, 0x6d, 0x69,
0x6e, 0x68, 0x20, 0x73, 0x69, 0x74, 0x69, 0x68, 0x6f, 0x6e, 0x67, 0x20,
0x6b, 0x6f, 0x6e, 0x67, 0x69, 0x72, 0x6b, 0x75, 0x1e6d, 0x73, 0x6b, 0x6a,
0x61, 0x79, 0x61, 0x70, 0x75, 0x72, 0x61, 0x6a, 0x65, 0x72, 0x75, 0x73,
0x61, 0x6c, 0x65, 0x6d, 0x6b, 0x61, 0x62, 0x75, 0x6c, 0x6b, 0x61, 0x6d,
0x63, 0x61, 0x1e6d, 0x6b, 0x61, 0x6b, 0x61, 0x72, 0x61, 0x63, 0x12b, 0x6b,
0x61, 0x1e6d, 0x6d, 0x61, 0x6e, 0x1e0d, 0x75, 0x6b, 0x61, 0x6e, 0x64, 0x69,
0x67, 0x61, 0x6b, 0x6f, 0x6c, 0x6b, 0x61, 0x74, 0x61, 0x6b, 0x72, 0x61,
0x73, 0x6e, 0x65, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x6b, 0x75, 0x61, 0x6c,
0x61, 0x20, 0x6c, 0x75, 0x6d, 0x70, 0x75, 0x72, 0x6b, 0x75, 0x63, 0x69,
0x6e, 0x67, 0x6b, 0x75, 0x76, 0x61, 0x69, 0x74, 0x6d, 0x6b, 0x61, 0x6f,
0x6d, 0x65, 0x67, 0x61, 0x1e0d, 0x61, 0x6e, 0x6d, 0x61, 0x6b, 0x61, 0x73,
0x61, 0x72, 0x6d, 0x61, 0x6e, 0x12b, 0x6c, 0x61, 0x6d, 0x61, 0x73, 0x6b,
0x61, 0x74, 0x6e, 0x69, 0x6b, 0x6f, 0x73, 0x69, 0x79, 0x61, 0x6e, 0x65,
0x76, 0x6f, 0x6b, 0x75, 0x6a, 0x6e, 0x65, 0x1e6d, 0x73, 0x6b, 0x6e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x70, 0x6e, 0x6f,
0x6d, 0x20, 0x70, 0x65, 0x6e, 0x68, 0x70, 0x6f, 0x6e, 0x1e6d, 0x69, 0x61,
0x6e, 0x61, 0x6b, 0x70, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67,
0x6b, 0x6f, 0x73, 0x1e6d, 0x61, 0x6e, 0x65, 0x6b, 0x69, 0x6a, 0x75, 0x6f,
0x72, 0x64, 0x61, 0x72, 0x69, 0x79, 0x61, 0x64, 0x73, 0x61, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x73, 0x61, 0x6d, 0x61, 0x72, 0x6b, 0x61, 0x6e, 0x64,
0x73, 0x65, 0x6f, 0x6c, 0x73, 0x65, 0x6e, 0x67, 0x61, 0x69, 0x73, 0x69,
0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x73, 0x72, 0x65, 0x1e0d, 0x6e, 0x65,
0x6b, 0x6f, 0x6c, 0x79, 0x6d, 0x73, 0x6b, 0x74, 0x61, 0x69, 0x70, 0x65,
0x69, 0x74, 0x61, 0x73, 0x6b, 0x65, 0x6e, 0x1e6d, 0x74, 0x62, 0x69, 0x6c,
0x69, 0x73, 0x69, 0x74, 0x65, 0x68, 0x72, 0x61, 0x6e, 0x74, 0x69, 0x6d,
0x70, 0x68, 0x75, 0x1e6d, 0x6f, 0x6b, 0x79, 0x6f, 0x1e6d, 0x6f, 0x6d, 0x73,
0x6b, 0x75, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x1e6d, 0x61, 0x72, 0x75, 0x72,
0x75, 0x6d, 0x63, 0x69, 0x79, 0x75, 0x73, 0x74, 0x2d, 0x6e, 0x65, 0x72,
0x61, 0x76, 0x69, 0x65, 0x6e, 0x1e6d, 0x69, 0x61, 0x61, 0x6e, 0x76, 0x6c,
0x61, 0x1e0d, 0x69, 0x76, 0x6f, 0x73, 0x1e6d, 0x6f, 0x6b, 0x79, 0x61, 0x6b,
0x75, 0x1e6d, 0x73, 0x6b, 0x79, 0x61, 0x6e, 0x67, 0x6f, 0x6e, 0x79, 0x69,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x79,
0x65, 0x72, 0x65, 0x76, 0x61, 0x6e, 0x61, 0x6a, 0x6f, 0x72, 0x65, 0x73,
0x62, 0x65, 0x72, 0x6d, 0x16b, 0x64, 0x61, 0x6b, 0x65, 0x6e, 0x65, 0x72,
0x69, 0x6b, 0x65, 0x70, 0x20, 0x76, 0x65, 0x72, 0x1e0d, 0x65, 0x70, 0x65,
0x72, 0x6f, 0x6d, 0x61, 0x1e0d, 0x69, 0x65, 0x72, 0x61, 0x72, 0x65, 0x79,
0x6b, 0x79, 0x61, 0x76, 0x69, 0x6b, 0x64, 0x6b, 0x69, 0x1e47, 0x20, 0x6a,
0x6f, 0x72, 0x6a, 0x69, 0x61, 0x73, 0x65, 0x6e, 0x74, 0x20, 0x68, 0x65,
0x6c, 0x65, 0x6e, 0x61, 0x73, 0x1e6d, 0x61, 0x6e, 0x6c, 0x69, 0x61, 0x65,
0x64, 0x69, 0x6c, 0x65, 0x69, 0x64, 0x62, 0x72, 0x69, 0x73, 0x62, 0x61,
0x6e, 0x65, 0x62, 0x72, 0x6f, 0x6b, 0x65, 0x6e, 0x20, 0x68, 0x69, 0x6c,
0x6c, 0x1e0d, 0x61, 0x72, 0x76, 0x69, 0x6e, 0x79, 0x75, 0x6b, 0x6c, 0x61,
0x68, 0x6f, 0x62, 0x61, 0x72, 0x1e6d, 0x6c, 0x69, 0x6e, 0x1e0d, 0x65, 0x72,
0x6d, 0x61, 0x6e, 0x6c, 0x6f, 0x72, 0x1e0d, 0x20, 0x68, 0x6f, 0x76, 0x65,
0x6d, 0x65, 0x6c, 0x62, 0x6f, 0x72, 0x6e, 0x65, 0x70, 0x65, 0x72, 0x74,
0x73, 0x79, 0x1e0d, 0x6e, 0x12b, 0x61, 0x6d, 0x73, 0x1e6d, 0x65, 0x72, 0x1e0d,
0x61, 0x6d, 0x61, 0x61, 0x1e47, 0x1e0d, 0x6f, 0x72, 0x61, 0x61, 0x61, 0x73,
0x1e6d, 0x72, 0x61, 0x68, 0x61, 0x6e, 0x65, 0x74, 0x65, 0x6e, 0x73, 0x62,
0x65, 0x6c, 0x67, 0x72, 0x61, 0x1e0d, 0x65, 0x62, 0x65, 0x72, 0x6c, 0x69,
0x6e, 0x62, 0x72, 0x61, 0x1e6d, 0x69, 0x73, 0x6c, 0x61, 0x76, 0x61, 0x62,
0x72, 0x75, 0x73, 0x73, 0x65, 0x6c, 0x73, 0x62, 0x75, 0x6b, 0x61, 0x72,
0x65, 0x73, 0x74, 0x62, 0x75, 0x1e0d, 0x61, 0x70, 0x65, 0x73, 0x74, 0x62,
0x75, 0x73, 0x69, 0x6e, 0x67, 0x65, 0x6e, 0x6b, 0x69, 0x73, 0x69, 0x6e,
0x61, 0x75, 0x6b, 0x6f, 0x70, 0x65, 0x6e, 0x68, 0x61, 0x67, 0x65, 0x6e,
0x1e0d, 0x62, 0x6c, 0x69, 0x6e, 0x6a, 0x69, 0x62, 0x72, 0x61, 0x6c, 0x1e6d,
0x61, 0x72, 0x67, 0x65, 0x72, 0x6e, 0x73, 0x69, 0x68, 0x65, 0x6c, 0x73,
0x69, 0x6e, 0x6b, 0x69, 0x61, 0x61, 0x69, 0x6c, 0x20, 0x61, 0x70, 0x20,
0x6d, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x62, 0x75, 0x6c, 0x6a,
0x65, 0x72, 0x73, 0x69, 0x6b, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x6e, 0x67,
0x72, 0x61, 0x1e0d, 0x6b, 0x69, 0x72, 0x6f, 0x76, 0x6b, 0x69, 0x79, 0x76,
0x6c, 0x69, 0x73, 0x62, 0x6f, 0x6e, 0x6c, 0x79, 0x75, 0x62, 0x6c, 0x79,
0x61, 0x6e, 0x61, 0x6c, 0x6e, 0x1e0d, 0x6e, 0x6c, 0x6b, 0x73, 0x65, 0x6d,
0x62, 0x72, 0x67, 0x6d, 0x65, 0x1e0d, 0x72, 0x69, 0x1e0d, 0x6d, 0x61, 0x6c,
0x1e6d, 0x61, 0x6d, 0x61, 0x72, 0x69, 0x65, 0x68, 0x61, 0x6d, 0x6d, 0x69,
0x6e, 0x73, 0x6b, 0x6d, 0x6f, 0x6e, 0x61, 0x6b, 0x6f, 0x6d, 0x6f, 0x73,
0x6b, 0x6f, 0x6f, 0x73, 0x6c, 0x6f, 0x70, 0x61, 0x72, 0x69, 0x73, 0x70,
0x1e0d, 0x67, 0x6f, 0x72, 0x69, 0x74, 0x73, 0x61, 0x70, 0x72, 0x61, 0x67,
0x72, 0x69, 0x67, 0x61, 0x73, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x73, 0x61,
0x6e, 0x20, 0x6d, 0x61, 0x72, 0x69, 0x6e, 0x6f, 0x73, 0x61, 0x72, 0x61,
0x6a, 0x65, 0x76, 0x6f, 0x73, 0x61, 0x72, 0x61, 0x1e6d, 0x6f, 0x76, 0x73,
0x69, 0x6d, 0x70, 0x65, 0x72, 0x6f, 0x70, 0x6f, 0x6c, 0x73, 0x6b, 0x6f,
0x70, 0x69, 0x73, 0x6f, 0x70, 0x69, 0x79, 0x61, 0x73, 0x1e6d, 0x6f, 0x6b,
0x68, 0x6f, 0x6d, 0x74, 0x61, 0x6c, 0x69, 0x6e, 0x1e6d, 0x69, 0x72, 0x61,
0x6e, 0x65, 0x75, 0x6c, 0x79, 0x61, 0x6e, 0x6f, 0x76, 0x73, 0x6b, 0x76,
0x61, 0x1e0d, 0x75, 0x6a, 0x76, 0x61, 0x1e6d, 0x69, 0x6b, 0x61, 0x6e, 0x76,
0x69, 0x65, 0x6e, 0x6e, 0x61, 0x76, 0x69, 0x6c, 0x6e, 0x69, 0x75, 0x73,
0x76, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x1e0d, 0x76, 0x61, 0x72,
0x73, 0x61, 0x74, 0x65, 0x67, 0x75, 0x73, 0x69, 0x67, 0x61, 0x6c, 0x70,
0x61, 0x6a, 0x75, 0x72, 0x69, 0x63, 0x1e6d, 0x61, 0x6e, 0x61, 0x6e, 0x61,
0x72, 0x69, 0x76, 0x63, 0x68, 0x61, 0x67, 0x6f, 0x73, 0x6b, 0x72, 0x69,
0x73, 0x1e6d, 0x6d, 0x61, 0x73, 0x6b, 0x6f, 0x6b, 0x6f, 0x73, 0x6b, 0x6f,
0x6d, 0x6f, 0x72, 0x6f, 0x6b, 0x65, 0x72, 0x67, 0x75, 0x65, 0x6c, 0x65,
0x6e, 0x6d, 0x61, 0x68, 0x65, 0x6d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65,
0x73, 0x6d, 0x61, 0x75, 0x72, 0x69, 0x73, 0x69, 0x75, 0x73, 0x6d, 0x61,
0x79, 0x6f, 0x1e6d, 0x72, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x61, 0x70,
0x69, 0x61, 0x101, 0x6b, 0x6c, 0x61, 0x6e, 0x1e0d, 0x62, 0x6f, 0x75, 0x67,
0x61, 0x6e, 0x76, 0x69, 0x6c, 0x6c, 0x63, 0x65, 0x1e6d, 0x61, 0x6d, 0x63,
0x16b, 0x6b, 0x12b, 0x73, 0x74, 0x65, 0x72, 0x69, 0x70, 0x65, 0x1e6d, 0x65,
0x70, 0x61, 0x6b, 0x61, 0x6f, 0x70, 0x6f, 0x70, 0x69, 0x6a, 0x69, 0x70,
0x75, 0x6e, 0x61, 0x70, 0x75, 0x74, 0x69, 0x67, 0x61, 0x6c, 0x61, 0x70,
0x61, 0x67, 0x6f, 0x73, 0x67, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x67,
0x75, 0x61, 0x1e0d, 0x61, 0x6c, 0x6b, 0x65, 0x6e, 0x61, 0x6c, 0x6b, 0x61,
0x6e, 0x1e6d, 0x6f, 0x6e, 0x6b, 0x69, 0x72, 0x69, 0x74, 0x69, 0x6d, 0x61,
0x74, 0x69, 0x6b, 0x69, 0x73, 0x72, 0x61, 0x65, 0x6b, 0x76, 0x61, 0x6a,
0x61, 0x6c, 0x65, 0x69, 0x6e, 0x6d, 0x61, 0x6a, 0x75, 0x72, 0x6f, 0x6d,
0x61, 0x72, 0x6b, 0x69, 0x73, 0x61, 0x73, 0x6d, 0x69, 0x1e0d, 0x76, 0x61,
0x79, 0x6e, 0x61, 0x75, 0x72, 0x75, 0x6e, 0x69, 0x75, 0x65, 0x6e, 0x6f,
0x72, 0x70, 0x6f, 0x6b, 0x6e, 0x75, 0x6d, 0x69, 0x65, 0x70, 0x61, 0x6e,
0x67, 0x6f, 0x20, 0x70, 0x61, 0x6e, 0x67, 0x6f, 0x70, 0x61, 0x6c, 0x61,
0x75, 0x70, 0x69, 0x1e6d, 0x6b, 0x65, 0x72, 0x6e, 0x70, 0x6f, 0x6e, 0x70,
0x65, 0x69, 0x70, 0x6f, 0x72, 0x1e6d, 0x20, 0x6d, 0x6f, 0x72, 0x65, 0x73,
0x62, 0x69, 0x72, 0x61, 0x72, 0x6f, 0x74, 0x6f, 0x6e, 0x67, 0x61, 0x73,
0x61, 0x69, 0x70, 0x61, 0x6e, 0x1e6d, 0x61, 0x68, 0x69, 0x1e6d, 0x69, 0x74,
0x61, 0x72, 0x61, 0x76, 0x61, 0x1e6d, 0x6f, 0x6e, 0x67, 0x61, 0x1e6d, 0x61,
0x70, 0x75, 0x76, 0x65, 0x6b, 0x76, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x92c,
0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x91c, 0x941, 0x932, 0x92a, 0x93c, 0x94d, 0x930,
0x940, 0x91f, 0x93e, 0x909, 0x928, 0x915, 0x93e, 0x930, 0x94d, 0x924, 0x941, 0x92e,
0x915, 0x93f, 0x902, 0x938, 0x93e, 0x938, 0x93e, 0x932, 0x941, 0x92c, 0x941, 0x92e,
0x92c, 0x93e, 0x938, 0x940, 0x92e, 0x94d, 0x2d, 0x92c, 0x93e, 0x92c, 0x93e, 0x928,
0x947, 0x92e, 0x94b, 0x917, 0x93e, 0x926, 0x93f, 0x938, 0x941, 0x928, 0x93e, 0x907,
0x930, 0x94b, 0x92c, 0x93f, 0x928, 0x941, 0x906, 0x915, 0x91a, 0x949, 0x91f, 0x938,
0x94d, 0x92f, 0x93e, 0x928, 0x20, 0x939, 0x94d, 0x935, 0x93e, 0x928, 0x938, 0x94d,
0x92f, 0x93e, 0x928, 0x20, 0x932, 0x942, 0x924, 0x93f, 0x909, 0x938, 0x941, 0x906,
0x907, 0x92f, 0x93e, 0x90f, 0x938, 0x928, 0x938, 0x93f, 0x92f, 0x949, 0x928, 0x92c,
0x94d, 0x932, 0x93e, 0x902, 0x2d, 0x938, 0x947, 0x92c, 0x932, 0x94b, 0x928, 0x915,
0x947, 0x92e, 0x94d, 0x92c, 0x94d, 0x930, 0x93f, 0x91c, 0x20, 0x92c, 0x947, 0x915,
0x94d, 0x92f, 0x93e, 0x92e, 0x94d, 0x92a, 0x94b, 0x20, 0x917, 0x94d, 0x930, 0x93e,
0x902, 0x921, 0x947, 0x915, 0x94d, 0x92f, 0x93e, 0x928, 0x915, 0x941, 0x928, 0x938,
0x93f, 0x915, 0x93e, 0x917, 0x94b, 0x921, 0x947, 0x928, 0x92e, 0x93e, 0x930, 0x94d,
0x915, 0x938, 0x949, 0x928, 0x924, 0x93f, 0x930, 0x941, 0x928, 0x947, 0x92a, 0x947,
0x92a, 0x93c, 0x94b, 0x930, 0x94d, 0x91f, 0x20, 0x928, 0x947, 0x932, 0x94d, 0x938,
0x928, 0x92a, 0x93c, 0x94b, 0x930, 0x94d, 0x91f, 0x93e, 0x932, 0x947, 0x91c, 0x93c,
0x93e, 0x939, 0x947, 0x932, 0x93f, 0x92a, 0x947, 0x915, 0x94d, 0x938, 0x907, 0x915,
0x93e, 0x932, 0x941, 0x924, 0x93f, 0x91f, 0x915, 0x94d, 0x930, 0x93e, 0x932, 0x947,
0x928, 0x94d, 0x921, 0x93f, 0x91c, 0x93f, 0x915, 0x92e, 0x928, 0x94b, 0x938, 0x92e,
0x947, 0x930, 0x93f, 0x917, 0x94b, 0x91f, 0x92c, 0x94d, 0x92f, 0x942, 0x932, 0x93e,
0x2c, 0x20, 0x909, 0x924, 0x949, 0x930, 0x949, 0x20, 0x921, 0x915, 0x94b, 0x91f,
0x93e, 0x92e, 0x93e, 0x926, 0x93f, 0x928, 0x940, 0x20, 0x909, 0x924, 0x949, 0x930,
0x949, 0x20, 0x921, 0x915, 0x94b, 0x91f, 0x93e, 0x928, 0x94d, 0x92f, 0x942, 0x20,
0x938, 0x93e, 0x932, 0x947, 0x92e, 0x2c, 0x20, 0x909, 0x924, 0x949, 0x930, 0x949,
0x20, 0x921, 0x915, 0x94b, 0x91f, 0x93e, 0x913, 0x915, 0x93e, 0x91c, 0x940, 0x928,
0x93e, 0x917, 0x93e, 0x92a, 0x93c, 0x940, 0x928, 0x93f, 0x915, 0x94d, 0x938, 0x92a,
0x94b, 0x930, 0x94d, 0x91f, 0x20, 0x911, 0x92a, 0x93c, 0x20, 0x938, 0x94d, 0x92a,
0x947, 0x928, 0x930, 0x947, 0x938, 0x93e, 0x907, 0x92a, 0x93c, 0x938, 0x947, 0x928,
0x94d, 0x91f, 0x93e, 0x930, 0x947, 0x92e, 0x938, 0x947, 0x902, 0x924, 0x93f, 0x906,
0x917, 0x94b, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x92c, 0x93e, 0x930, 0x94d, 0x924,
0x947, 0x932, 0x947, 0x92e, 0x93f, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x924, 0x949,
0x92e, 0x938, 0x938, 0x94d, 0x935, 0x93f, 0x92a, 0x93c, 0x94d, 0x91f, 0x20, 0x915,
0x930, 0x902, 0x91f, 0x935, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x915, 0x942, 0x935,
0x930, 0x92e, 0x94d, 0x92f, 0x93e, 0x915, 0x92e, 0x941, 0x930, 0x94d, 0x921, 0x94b,
0x932, 0x949, 0x928, 0x94d, 0x917, 0x92f, 0x930, 0x92c, 0x94d, 0x92f, 0x947, 0x928,
0x905, 0x938, 0x94d, 0x917, 0x93e, 0x92c, 0x93e, 0x924, 0x92c, 0x94d, 0x92f, 0x93e,
0x902, 0x917, 0x915, 0x949, 0x915, 0x92c, 0x93f, 0x938, 0x94d, 0x915, 0x947, 0x915,
0x92c, 0x94d, 0x930, 0x942, 0x928, 0x947, 0x924, 0x93f, 0x924, 0x94d, 0x938, 0x93f,
0x924, 0x93e, 0x926, 0x941, 0x92c, 0x924, 0x93f, 0x926, 0x941, 0x938, 0x93e, 0x902,
0x92c, 0x947, 0x92a, 0x93c, 0x93e, 0x92e, 0x93e, 0x917, 0x941, 0x938, 0x94d, 0x924,
0x93e, 0x92f, 0x947, 0x930, 0x941, 0x938, 0x932, 0x947, 0x92e, 0x915, 0x93e, 0x921,
0x93f, 0x902, 0x917, 0x93e, 0x92a, 0x928, 0x949, 0x92e, 0x20, 0x92a, 0x947, 0x928,
0x94d, 0x939, 0x915, 0x93f, 0x91c, 0x93f, 0x932, 0x949, 0x930, 0x94d, 0x921, 0x93e,
0x938, 0x93e, 0x915, 0x93e, 0x932, 0x93f, 0x928, 0x938, 0x902, 0x918, 0x93e, 0x924,
0x93f, 0x924, 0x93e, 0x924, 0x93f, 0x92a, 0x947, 0x924, 0x93f, 0x924, 0x93e, 0x938,
0x915, 0x902, 0x924, 0x91f, 0x94d, 0x2d, 0x92c, 0x93f, 0x932, 0x93f, 0x938, 0x93f,
0x92f, 0x93e, 0x902, 0x917, 0x949, 0x928, 0x915, 0x94d, 0x92f, 0x93e, 0x928, 0x947,
0x930, 0x940, 0x92a, 0x94d, 0x92f, 0x93e, 0x930, 0x94b, 0x926, 0x93e, 0x945, 0x915,
0x93f, 0x923, 0x93e, 0x945, 0x20, 0x91c, 0x93e, 0x945, 0x930, 0x94d, 0x91c, 0x93f,
0x92f, 0x93e, 0x938, 0x94d, 0x91f, 0x94d, 0x92f, 0x93e, 0x928, 0x932, 0x940, 0x906,
0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x93e, 0x915, 0x93e, 0x928, 0x90f, 0x924, 0x947,
0x928, 0x94d, 0x938, 0x906, 0x907, 0x932, 0x20, 0x911, 0x92a, 0x94d, 0x20, 0x92e,
0x94d, 0x92f, 0x93e, 0x928, 0x938, 0x93f, 0x92e, 0x94d, 0x92a, 0x93c, 0x947, 0x930,
0x94b, 0x92a, 0x94b, 0x932, 0x938, 0x94d, 0x915, 0x94b, 0x92a, 0x94d, 0x92f, 0x947,
0x938, 0x94b, 0x92a, 0x93c, 0x93f, 0x92f, 0x93e, 0x91c, 0x93c, 0x94d, 0x92f, 0x942,
0x930, 0x93f, 0x915, 0x93c, 0x92e, 0x949, 0x930, 0x940, 0x938, 0x938, 0x911, 0x915,
0x932, 0x947, 0x902, 0x921, 0x91a, 0x94d, 0x92f, 0x93e, 0x925, 0x92e, 0x924, 0x93f,
0x938, 0x94d, 0x91f, 0x930, 0x90f, 0x92a, 0x93c, 0x947, 0x91f, 0x92a, 0x93c, 0x93e,
0x915, 0x93e, 0x913, 0x92a, 0x93c, 0x94b, 0x92a, 0x93c, 0x93f, 0x91c, 0x940, 0x92a,
0x93c, 0x94d, 0x92f, 0x942, 0x928, 0x93e, 0x92a, 0x93c, 0x941, 0x91f, 0x940, 0x917,
0x94d, 0x92f, 0x93e, 0x92e, 0x92c, 0x93f, 0x92f, 0x930, 0x915, 0x947, 0x902, 0x91f,
0x928, 0x928, 0x93e, 0x945, 0x909, 0x930, 0x941, 0x928, 0x949, 0x930, 0x92a, 0x93c,
0x949, 0x915, 0x928, 0x949, 0x92e, 0x93f, 0x92f, 0x93e, 0x92a, 0x94b, 0x928, 0x92a,
0x947, 0x924, 0x93f, 0xb0f, 0xb15, 0xb4d, 0xb30, 0xb3e, 0xb2c, 0xb4d, 0xb30, 0xb3e,
0xb1c, 0xb3e, 0xb71, 0xb3f, 0xb32, 0xb4d, 0xb32, 0xb47, 0xb2c, 0xb41, 0xb1c, 0xb42,
0xb2e, 0xb4d, 0xb2c, 0xb41, 0xb30, 0xb3e, 0xb15, 0xb28, 0xb3e, 0xb15, 0xb4d, 0xb30,
0xb3f, 0xb1c, 0xb3f, 0xb2c, 0xb1f, 0xb3f, 0xb2a, 0xb4d, 0xb30, 0xb3f, 0xb1f, 0xb3e,
0xb09, 0xb28, 0xb4d, 0x200c, 0xb17, 0xb3e, 0xb2c, 0xb30, 0xb4d, 0xb23, 0xb4d, 0xb23,
0xb1c, 0xb39, 0xb3e, 0xb28, 0xb4d, 0xb38, 0xb2c, 0xb30, 0xb4d, 0xb17, 0xb15, 0xb30,
0xb1f, 0xb09, 0xb2e, 0xb4d, 0x200c, 0xb32, 0xb3e, 0xb17, 0xb38, 0xb4d, 0x200c, 0xb32,
0xb3f, 0xb2c, 0xb4d, 0xb30, 0xb47, 0xb71, 0xb3f, 0xb32, 0xb4d, 0xb32, 0xb47, 0xb32,
0xb2e, 0xb4d, 0x200c, 0xb2e, 0xb4d, 0x2d, 0xb2c, 0xb3e, 0xb2c, 0xb3e, 0xb28, 0xb47,
0xb2e, 0xb4b, 0xb17, 0xb3e, 0xb21, 0xb3f, 0xb38, 0xb41, 0xb2e, 0xb28, 0xb30, 0xb4b,
0xb2c, 0xb3f, 0xb06, 0xb28, 0xb3e, 0xb07, 0xb30, 0xb2c, 0xb3f, 0xb28, 0xb41, 0xb06,
0xb15, 0xb1a, 0xb1f, 0xb05, 0xb17, 0xb3e, 0xb21, 0xb17, 0xb41, 0xb2a, 0xb30, 0xb4d,
0xb1f, 0x2d, 0xb28, 0xb71, 0xb38, 0xb3e, 0xb05, 0x20, 0xb1f, 0xb2e, 0xb47, 0xb24,
0xb4d, 0xb30, 0xb3f, 0xb2a, 0xb32, 0xb3f, 0xb71, 0xb3f, 0xb23, 0xb4d, 0xb21, 0xb39,
0xb15, 0xb4d, 0xb2c, 0xb41, 0xb0f, 0xb28, 0xb38, 0xb4d, 0x20, 0xb06, 0xb07, 0xb30,
0xb47, 0xb38, 0xb4d, 0xb15, 0xb3e, 0xb3c, 0xb1f, 0xb3e, 0xb2e, 0xb3e, 0xb15, 0xb3e,
0xb01, 0xb15, 0xb4b, 0xb21, 0xb4b, 0xb2c, 0xb3e, 0xb1c, 0xb41, 0xb1c, 0xb4b, 0xb0f,
0xb32, 0xb3e, 0x20, 0xb30, 0xb3f, 0xb4d, 0xb05, 0xb1c, 0xb3e, 0xb2e, 0xb47, 0xb23,
0xb4d, 0xb21, 0xb1c, 0xb3e, 0xb30, 0xb3f, 0xb5f, 0xb4b, 0x20, 0xb17, 0xb3e, 0xb32,
0xb47, 0xb17, 0xb4b, 0xb38, 0xb38, 0xb3e, 0xb32, 0xb4d, 0xb1f, 0xb3e, 0xb38, 0xb3e,
0xb5f, 0xb3e, 0xb28, 0xb4d, 0x20, 0xb71, 0xb3e, 0xb28, 0xb4d, 0xb38, 0xb5f, 0xb3e,
0xb28, 0x20, 0xb32, 0xb41, 0xb07, 0xb38, 0xb1f, 0xb4b, 0xb15, 0xb41, 0xb2e, 0xb28,
0xb09, 0xb38, 0xb41, 0xb06, 0xb07, 0xb5f, 0xb3e, 0xb05, 0xb30, 0xb41, 0xb2c, 0xb3e,
0xb2c, 0xb3e, 0xb30, 0xb2c, 0xb3e, 0xb21, 0xb38, 0xb2c, 0xb4d, 0xb32, 0xb3e, 0xb19,
0xb4d, 0xb15, 0x2d, 0xb38, 0xb3e, 0xb2c, 0xb32, 0xb28, 0xb4d, 0xb2c, 0xb4b, 0xb06,
0x20, 0xb71, 0xb3f, 0xb38, 0xb4d, 0xb1f, 0xb3e, 0xb2c, 0xb17, 0xb1f, 0xb3e, 0xb2c,
0xb07, 0xb38, 0xb47, 0xb15, 0xb3e, 0xb2e, 0xb4d, 0xb2a, 0x20, 0xb17, 0xb4d, 0xb30,
0xb3e, 0xb23, 0xb4d, 0xb21, 0xb47, 0xb38, 0xb3f, 0xb15, 0xb3e, 0xb17, 0xb15, 0xb37,
0xb4d, 0xb1f, 0xb3e, 0x20, 0xb30, 0xb3f, 0xb15, 0xb3e, 0xb15, 0xb4d, 0xb30, 0xb47,
0xb38, 0xb4d, 0x200d, 0xb1f, 0xb28, 0xb21, 0xb47, 0xb28, 0xb71, 0xb3f, 0xb30, 0xb4d,
0xb21, 0xb47, 0xb1f, 0xb4d, 0xb30, 0xb07, 0xb1f, 0xb4d, 0xb21, 0xb2e, 0xb3f, 0xb28,
0xb3f, 0xb15, 0xb3e, 0xb0f, 0xb21, 0xb4d, 0x200d, 0xb2e, 0xb28, 0xb1f, 0xb28, 0xb4d,
0xb0f, 0xb32, 0xb4d, 0x20, 0xb38, 0xb3e, 0xb32, 0xb71, 0xb3e, 0xb21, 0xb4b, 0xb30,
0xb4d, 0xb2a, 0xb30, 0xb4d, 0xb1f, 0x20, 0xb28, 0xb47, 0xb32, 0xb38, 0xb28, 0xb4d,
0xb2a, 0xb30, 0xb4d, 0xb1f, 0xb32, 0xb47, 0xb1c, 0xb3e, 0xb17, 0xb41, 0xb06, 0xb21,
0xb47, 0xb32, 0xb09, 0xb2a, 0xb47, 0xb17, 0xb41, 0xb06, 0xb24, 0xb47, 0xb2e, 0xb3e,
0xb32, 0xb3e, 0xb39, 0xb3e, 0xb71, 0xb28, 0xb3e, 0xb39, 0xb47, 0xb30, 0xb2e, 0xb38,
0xb3f, 0xb32, 0xb4b, 0xb07, 0xb23, 0xb4d, 0xb21, 0xb3f, 0xb06, 0xb28, 0xb3e, 0xb2a,
0xb32, 0xb3f, 0xb38, 0xb4d, 0xb2e, 0xb3e, 0xb30, 0xb47, 0xb28, 0xb17, 0x2c, 0x20,
0xb07, 0xb23, 0xb4d, 0xb21, 0xb3f, 0xb06, 0xb28, 0xb3e, 0xb71, 0xb47, 0xb71, 0xb3e,
0xb5f, 0x2c, 0x20, 0xb07, 0xb23, 0xb4d, 0xb21, 0xb3f, 0xb06, 0xb28, 0xb3e, 0xb07,
0xb28, 0xb41, 0xb71, 0xb3f, 0xb15, 0xb4d, 0xb32, 0xb09, 0xb07, 0xb38, 0xb71, 0xb3f,
0xb32, 0xb4d, 0xb32, 0xb47, 0xb2e, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb38, 0xb47, 0xb32,
0x2c, 0x20, 0xb15, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb09, 0xb15, 0xb3f, 0xb15, 0xb47,
0xb15, 0xb4d, 0xb30, 0xb3e, 0xb32, 0xb47, 0xb23, 0xb4d, 0xb21, 0xb3f, 0xb1c, 0xb3f,
0xb15, 0xb2e, 0xb3e, 0xb38, 0xb3f, 0xb05, 0xb2e, 0xb3e, 0xb30, 0xb3f, 0xb17, 0xb1f,
0xb4d, 0xb2e, 0xb3e, 0xb1f, 0xb3e, 0xb2e, 0xb30, 0xb38, 0xb4d, 0xb2e, 0xb3e, 0xb1c,
0xb3e, 0xb1f, 0xb32, 0xb3e, 0xb28, 0xb4d, 0xb2e, 0xb3f, 0xb15, 0xb4d, 0xb35, 0xb47,
0xb32, 0xb28, 0xb4d, 0xb2e, 0xb3e, 0xb28, 0xb15, 0xb1f, 0xb28, 0xb4d, 0xb2e, 0xb28,
0xb1f, 0xb47, 0xb30, 0xb3f, 0xb0f, 0xb2e, 0xb23, 0xb4d, 0xb1f, 0xb47, 0xb2d, 0xb3f,
0xb21, 0xb3f, 0xb05, 0xb2e, 0xb28, 0xb1f, 0xb38, 0xb47, 0xb30, 0xb30, 0xb3e, 0xb1f,
0xb4d, 0xb28, 0xb4d, 0xb5f, 0xb41, 0x20, 0xb5f, 0xb30, 0xb4d, 0xb15, 0xb4d, 0xb28,
0xb30, 0xb39, 0xb4d, 0xb28, 0xb2c, 0xb47, 0xb09, 0xb32, 0xb3e, 0xb39, 0x2c, 0x20,
0xb09, 0xb24, 0xb4d, 0xb24, 0xb30, 0x20, 0xb21, 0xb3e, 0xb15, 0xb1f, 0xb3e, 0xb15,
0xb47, 0xb28, 0xb4d, 0xb26, 0xb4d, 0xb30, 0x2c, 0x20, 0xb09, 0xb24, 0xb4d, 0xb24,
0xb30, 0x20, 0xb21, 0xb3e, 0xb15, 0xb1f, 0xb3e, 0xb28, 0xb4d, 0xb5f, 0xb41, 0x20,
0xb38, 0xb3e, 0xb32, 0xb47, 0xb2e, 0xb4d, 0x2c, 0x20, 0xb09, 0xb24, 0xb4d, 0xb24,
0xb30, 0x20, 0xb21, 0xb3e, 0xb15, 0xb1f, 0xb3e, 0xb05, 0xb1c, 0xb3f, 0xb28, 0xb3e,
0xb17, 0xb3e, 0xb2a, 0xb07, 0xb28, 0xb3f, 0xb15, 0xb4d, 0xb38, 0xb2a, 0xb30, 0xb4d,
0xb1f, 0x2d, 0xb0f, 0xb5f, 0xb41, 0x2d, 0xb2a, 0xb4d, 0xb30, 0xb3f, 0xb28, 0xb4d,
0x200d, 0xb38, 0xb2a, 0xb30, 0xb4d, 0xb1f, 0xb4d, 0x20, 0xb05, 0xb2b, 0xb4d, 0x20,
0xb38, 0xb4d, 0xb2a, 0xb47, 0xb28, 0xb4d, 0xb2a, 0xb41, 0xb0f, 0xb30, 0xb4d, 0xb24,
0x20, 0xb30, 0xb3f, 0xb15, 0xb30, 0xb47, 0xb38, 0xb3f, 0xb2a, 0xb3f, 0xb30, 0xb3f,
0xb5f, 0x20, 0xb2c, 0xb4d, 0xb30, 0xb3e, 0xb19, 0xb4d, 0xb15, 0xb38, 0xb3e, 0xb28,
0xb4d, 0xb24, 0xb30, 0xb47, 0xb2e, 0xb4d, 0xb38, 0xb3e, 0xb23, 0xb4d, 0xb1f, 0x20,
0xb21, 0xb2e, 0xb3f, 0xb19, 0xb4d, 0xb17, 0xb07, 0xb1f, 0xb4d, 0xb1f, 0xb15, 0xb4d,
0xb35, 0xb30, 0xb1f, 0xb30, 0xb2e, 0xb3f, 0xb1f, 0xb4d, 0xb38, 0xb47, 0xb23, 0xb4d,
0xb1f, 0x20, 0xb2c, 0xb3e, 0xb30, 0xb4d, 0xb24, 0xb47, 0xb32, 0xb47, 0xb2e, 0xb3f,
0xb38, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb4d, 0x20, 0xb15, 0xb3f, 0xb1f, 0xb4d, 0x200d,
0xb38, 0xb38, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb4d, 0x2e, 0x20, 0xb25, 0xb2e, 0xb3e,
0xb38, 0xb4d, 0xb38, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb4d, 0x2e, 0x20, 0xb71, 0xb3f,
0xb28, 0xb38, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb4d, 0xb24, 0xb41, 0xb32, 0xb47, 0xb1f,
0xb30, 0xb23, 0xb4d, 0xb1f, 0xb1f, 0xb30, 0xb1f, 0xb32, 0xb3e, 0xb71, 0xb3e, 0xb19,
0xb4d, 0xb15, 0xb41, 0xb71, 0xb30, 0xb4d, 0xb15, 0xb47, 0xb38, 0xb3f, 0xb21, 0xb47,
0xb71, 0xb3f, 0xb38, 0xb4d, 0xb21, 0xb4d, 0xb5f, 0xb41, 0xb2e, 0xb3e, 0xb23, 0xb4d,
0xb1f, 0x20, 0xb21, 0xb3f, 0x20, 0xb09, 0xb30, 0xb71, 0xb3f, 0xb32, 0xb47, 0xb2e,
0xb15, 0xb4d, 0xb71, 0xb3e, 0xb30, 0xb3f, 0xb2e, 0xb3e, 0xb01, 0xb38, 0xb28, 0xb2e,
0xb4d, 0xb5f, 0xb3e, 0xb15, 0xb2e, 0xb41, 0xb30, 0xb4d, 0xb21, 0xb2a, 0xb3e, 0xb01,
0xb2e, 0xb30, 0xb30, 0xb4b, 0xb24, 0xb47, 0xb30, 0xb3e, 0xb1f, 0xb4b, 0xb32, 0xb4d,
0xb71, 0xb4b, 0xb38, 0xb4d, 0xb24, 0xb15, 0xb4b, 0xb33, 0xb19, 0xb4d, 0xb16, 0xb5f,
0xb3e, 0xb30, 0xb2c, 0xb47, 0xb28, 0xb06, 0xb15, 0xb4d, 0xb1f, 0xb2c, 0xb47, 0xb06,
0xb38, 0xb4d, 0x200d, 0xb17, 0xb3e, 0xb2c, 0xb1f, 0xb4d, 0xb2c, 0xb39, 0xb3e, 0xb30,
0xb47, 0xb28, 0xb2c, 0xb3f, 0xb38, 0xb15, 0xb47, 0xb15, 0xb4d, 0x200c, 0xb26, 0xb41,
0xb38, 0xb3e, 0xb28, 0xb2c, 0xb47, 0xb2a, 0xb3e, 0xb2e, 0xb3e, 0xb17, 0xb41, 0xb38,
0xb4d, 0xb1f, 0xb3e, 0xb39, 0x20, 0xb1a, 0xb3f, 0x20, 0xb2e, 0xb3f, 0xb28, 0xb4d,
0x200c, 0x20, 0xb38, 0xb3f, 0xb1f, 0xb3f, 0xb39, 0xb4b, 0xb71, 0xb21, 0xb4d, 0x200c,
0xb1c, 0xb3e, 0xb15, 0xb30, 0xb4d, 0xb24, 0xb3e, 0xb15, 0xb2c, 0xb41, 0xb32, 0xb4d,
0xb15, 0xb3e, 0xb1f, 0xb2e, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb41, 0xb15, 0xb3e, 0xb28,
0xb21, 0xb4d, 0xb5f, 0xb3e, 0xb17, 0xb3e, 0xb15, 0xb4d, 0xb30, 0xb3e, 0xb38, 0xb28,
0xb5f, 0xb3e, 0xb30, 0xb38, 0xb4d, 0xb15, 0xb28, 0xb3f, 0xb15, 0xb38, 0xb3f, 0xb06,
0xb28, 0xb71, 0xb15, 0xb41, 0xb1c, 0xb28, 0xb47, 0xb1f, 0xb38, 0xb4d, 0xb15, 0xb28,
0xb71, 0xb38, 0xb3f, 0xb2c, 0xb3f, 0xb30, 0xb38, 0xb4d, 0xb15, 0xb05, 0xb30, 0xb3e,
0xb32, 0xb4d, 0x200c, 0xb2a, 0xb28, 0xb2e, 0xb4d, 0x200c, 0x20, 0xb2a, 0xb47, 0xb28,
0xb39, 0xb2a, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb06, 0xb28, 0xb3e, 0xb15, 0xb4d, 0x200c,
0xb15, 0xb37, 0xb4d, 0xb1f, 0xb28, 0xb47, 0xb15, 0xb40, 0xb1c, 0xb3f, 0xb32, 0xb30,
0xb4d, 0xb21, 0xb3e, 0xb38, 0xb15, 0xb3e, 0xb32, 0xb3f, 0xb28, 0xb4d, 0xb38, 0xb3f,
0xb05, 0xb32, 0xb38, 0xb02, 0xb17, 0xb3e, 0xb07, 0xb38, 0xb4d, 0xb30, 0xb47, 0xb21,
0xb28, 0xb47, 0xb15, 0xb32, 0xb5f, 0xb2e, 0xb38, 0xb4d, 0xb15, 0xb24, 0xb3e, 0xb38,
0xb15, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb24, 0xb3f, 0xb2e, 0xb4d, 0xb2a, 0xb41, 0xb1f,
0xb15, 0xb3f, 0xb05, 0xb1f, 0xb2e, 0xb38, 0xb4d, 0xb15, 0xb5f, 0xb41, 0xb38, 0xb4d,
0x200d, 0xb1f, 0x2d, 0xb28, 0xb47, 0xb30, 0xb3e, 0xb71, 0xb3f, 0xb0f, 0xb23, 0xb4d,
0xb1f, 0xb3f, 0xb0f, 0xb28, 0xb4d, 0x200c, 0xb71, 0xb4d, 0xb32, 0xb3e, 0xb21, 0xb3f,
0xb71, 0xb37, 0xb4d, 0xb1f, 0xb4b, 0xb15, 0xb4d, 0xb5f, 0xb47, 0xb30, 0xb47, 0xb2c,
0xb3e, 0xb28, 0xb4d, 0xb06, 0xb1c, 0xb30, 0xb47, 0xb38, 0xb4d, 0xb2c, 0xb30, 0xb2e,
0xb41, 0xb21, 0xb3e, 0xb15, 0xb47, 0xb2a, 0xb4d, 0x200c, 0x20, 0xb71, 0xb30, 0xb4d,
0xb26, 0xb47, 0xb2a, 0xb30, 0xb0f, 0xb30, 0xb47, 0xb15, 0xb4d, 0xb5f, 0xb3e, 0xb2c,
0xb3f, 0xb15, 0xb38, 0xb4d, 0x200d, 0xb1f, 0xb3e, 0xb32, 0xb3f, 0xb28, 0xb0f, 0xb21,
0xb3f, 0xb32, 0xb47, 0xb21, 0xb4d, 0xb2c, 0xb4d, 0xb30, 0xb3f, 0xb38, 0xb2c, 0xb28,
0xb4d, 0xb2c, 0xb4d, 0xb30, 0xb4b, 0xb15, 0xb28, 0x20, 0xb39, 0xb3f, 0xb32, 0xb21,
0xb3e, 0xb30, 0xb4d, 0xb71, 0xb3f, 0xb28, 0xb4d, 0xb32, 0xb3f, 0xb23, 0xb4d, 0xb21,
0xb47, 0xb2e, 0xb3e, 0xb28, 0xb32, 0xb30, 0xb4d, 0x200d, 0xb21, 0x20, 0xb39, 0xb3e,
0xb71, 0xb47, 0xb2e, 0xb47, 0xb32, 0xb2c, 0xb4b, 0xb28, 0xb01, 0xb2a, 0xb30, 0xb4d,
0xb24, 0xb06, 0xb2e, 0xb37, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb21, 0xb3e, 0xb2e,
0xb4d, 0xb06, 0xb23, 0xb4d, 0xb21, 0xb30, 0xb3e, 0xb06, 0xb38, 0xb4d, 0x200d, 0xb1f,
0xb30, 0xb3e, 0xb15, 0xb3e, 0xb28, 0xb0f, 0xb24, 0xb47, 0xb28, 0xb4d, 0xb38, 0xb2c,
0xb47, 0xb32, 0xb17, 0xb4d, 0xb30, 0xb47, 0xb21, 0xb47, 0xb2c, 0xb4d, 0xb30, 0xb3e,
0xb1f, 0xb3f, 0xb38, 0xb32, 0xb3e, 0xb71, 0xb3e, 0xb2c, 0xb4d, 0xb30, 0xb41, 0xb38,
0xb3f, 0xb32, 0xb4d, 0x200d, 0xb38, 0xb2c, 0xb41, 0xb1a, 0xb3e, 0xb30, 0xb47, 0xb38,
0xb4d, 0xb1f, 0xb2c, 0xb41, 0xb21, 0xb3e, 0xb2a, 0xb47, 0xb38, 0xb4d, 0xb1f, 0xb1a,
0xb3f, 0xb38, 0xb3f, 0xb28, 0xb3e, 0xb09, 0xb15, 0xb2a, 0xb47, 0xb28, 0xb39, 0xb3e,
0xb17, 0xb47, 0xb28, 0xb4d, 0xb06, 0xb07, 0xb32, 0xb4d, 0x20, 0xb05, 0xb2a, 0xb4d,
0x20, 0xb2e, 0xb4d, 0xb5f, 0xb3e, 0xb28, 0xb4d, 0xb15, 0xb3f, 0xb30, 0xb71, 0xb2e,
0xb3e, 0xb30, 0xb3f, 0xb5f, 0xb3e, 0xb39, 0xb47, 0xb2e, 0xb2e, 0xb28, 0xb3e, 0xb15,
0xb05, 0xb38, 0xb32, 0xb4b, 0xb2a, 0xb21, 0xb17, 0xb30, 0xb3f, 0xb15, 0xb3e, 0xb30,
0xb2e, 0xb4d, 0xb38, 0xb3e, 0xb30, 0xb3e, 0xb1c, 0xb47, 0xb2c, 0xb38, 0xb3e, 0xb30,
0xb3e, 0xb1f, 0xb71, 0xb4d, 0xb38, 0xb3f, 0xb2e, 0xb2b, 0xb47, 0xb30, 0xb2a, 0xb32,
0xb4d, 0xb38, 0xb4d, 0xb15, 0xb2a, 0xb5f, 0xb47, 0xb38, 0xb2a, 0xb3f, 0xb5f, 0xb3e,
0xb38, 0xb4d, 0xb1f, 0xb15, 0xb4d, 0x20, 0xb39, 0xb2e, 0xb4d, 0x200c, 0xb1f, 0xb3e,
0xb07, 0xb30, 0xb47, 0xb28, 0xb4d, 0xb5f, 0xb41, 0xb32, 0xb5f, 0xb3e, 0xb28, 0xb71,
0xb38, 0xb4d, 0xb15, 0xb2c, 0xb3e, 0xb21, 0xb41, 0xb1c, 0xb71, 0xb3e, 0xb1f, 0xb3f,
0xb15, 0xb3e, 0xb28, 0xb4d, 0xb71, 0xb3f, 0xb0f, 0xb28, 0xb3e, 0xb71, 0xb3f, 0xb32,
0xb28, 0xb3f, 0xb09, 0xb38, 0xb4d, 0xb71, 0xb32, 0xb17, 0xb17, 0xb4d, 0xb30, 0xb3e,
0xb21, 0xb4d, 0xb16, 0xb4d, 0xb30, 0xb40, 0xb38, 0xb4d, 0x200d, 0xb1f, 0x20, 0xb2e,
0xb3e, 0xb38, 0xb15, 0xb15, 0xb38, 0xb4d, 0x200c, 0xb15, 0xb2e, 0xb30, 0xb15, 0xb47,
0xb30, 0xb17, 0xb41, 0xb32, 0xb47, 0xb28, 0xb2e, 0xb30, 0xb3f, 0xb38, 0xb38, 0xb4d,
0xb2e, 0xb3e, 0xb5f, 0xb1f, 0xb47, 0xb2c, 0xb17, 0xb47, 0xb28, 0xb4d, 0x200c, 0xb71,
0xb3f, 0xb32, 0xb4d, 0xb32, 0xb47, 0xb1a, 0xb3e, 0xb24, 0xb3e, 0xb2e, 0xb4d, 0x200c,
0xb2a, 0xb15, 0xb3e, 0xb05, 0xb2a, 0xb2a, 0xb3f, 0xb1c, 0xb3f, 0xb17, 0xb3e, 0xb32,
0xb3e, 0xb2a, 0xb3e, 0xb17, 0xb38, 0xb15, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb28, 0xb28,
0xb30, 0xb2a, 0xb15, 0xb4d, 0x200c, 0xb28, 0xb09, 0xb2e, 0xb3f, 0xb5f, 0xb2a, 0xb39,
0xb28, 0xb2a, 0xb47, 0xb07, 0xb2a, 0xb30, 0xb4d, 0xb1f, 0xb4d, 0x200c, 0x20, 0xb2e,
0xb30, 0xb47, 0xb38, 0xb2c, 0xb3f, 0xb30, 0xb3e, 0xb30, 0xb1f, 0xb19, 0xb4d, 0xb17,
0xb3e, 0xb1f, 0xb19, 0xb4d, 0xb17, 0xb3e, 0xb1f, 0xb3e, 0xb2a, 0xb41, 0xc0e, 0xc02,
0xc17, 0xc4d, 0xc35, 0xc3f, 0xc32, 0xc4d, 0xc32, 0xc3e, 0x3c, 0x65, 0x78, 0x65,
0x6d, 0x70, 0x6c, 0x61, 0x72, 0x43, 0x69, 0x74, 0x79, 0x3e, 0xc05, 0xc21,
0xc46, 0xc32, 0xc48, 0xc21, 0xc4d, 0x3c, 0x2f, 0x65, 0x78, 0x65, 0x6d, 0x70,
0x6c, 0x61, 0x72, 0x43, 0x69, 0x74, 0x79, 0x3e, 0xc15, 0xc3e, 0xc02, 0xc1f,
0xc28, 0xc4d, 0x140a, 0x144e, 0x1426, 0x146f, 0x1472, 0x1423, 0x1401, 0x141f, 0x14aa, 0x1423,
0x1450, 0x1423, 0x1403, 0x14c4, 0x1431, 0x1420, 0x1403, 0x1473, 0x14eb, 0x1405, 0x1403, 0x141f,
0x14f4, 0x1422, 0x1473, 0x141f, 0x1489, 0x1418, 0x1423, 0x1411, 0x14c2, 0x142f, 0x1420
};

static inline constexpr char16_t shortZoneNameTable[] = {
0x55, 0x54, 0x43, 0x48, 0x53, 0x54, 0x48, 0x44, 0x54, 0x41f, 0x422, 0x41a,
0x412, 0x20, 0x28, 0x55, 0x54, 0x43, 0x29, 0x413, 0x430, 0x432, 0x430, 0x439,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x430, 0x432, 0x430, 0x439, 0x20,
0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x2205, 0x2205, 0x2205, 0x49, 0x53, 0x54, 0x42, 0x53, 0x54, 0x54,
0x55, 0x48, 0x54, 0xd83a, 0xdd11, 0xd83a, 0xdd16, 0xd83a, 0xdd0a, 0xd83a, 0xdd11, 0xd83a,
0xdd16, 0xd83a, 0xdd16, 0xd83a, 0xdd11, 0xd83a, 0xdd15, 0xd83a, 0xdd16, 0x54, 0x53, 0xc8,
0x20, 0x28, 0xc8, 0x69, 0x72, 0x69, 0x6e, 0x6e, 0x29, 0x54, 0x53, 0x42,
0x90f, 0x91a, 0x90f, 0x938, 0x91f, 0x940, 0x41, 0x43, 0xc9, 0x41, 0x53, 0x42,
0x53, 0x47, 0x48, 0x54, 0x43, 0x55, 0x4f, 0x49, 0x48, 0x4f, 0x4c, 0x48,
0x645, 0x20, 0x639, 0x20, 0x648, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x55, 0x54, 0x43, 0x48, 0x6f, 0x6e, 0x6f, 0x6c, 0x75, 0x6c, 0x75,
0x74, 0x69, 0x64, 0x48, 0x6f, 0x6e, 0x6f, 0x6c, 0x75, 0x6c, 0x75, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x48, 0x6f, 0x6e, 0x6f,
0x6c, 0x75, 0x6c, 0x75, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x54, 0x4d, 0x54, 0x48, 0x54, 0x54, 0x48, 0x54, 0x4c, 0x47, 0x69,
0x1edd, 0x20, 0x48, 0x53, 0x54, 0x189, 0x41, 0x4b
};

static inline constexpr char16_t longZoneNameTable[] = {
0x43, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x61, 0x74, 0x65, 0x64, 0x20,
0x55, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x49, 0x72, 0x69, 0x73, 0x68, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x42, 0x72, 0x69,
0x74, 0x69, 0x73, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x410, 0x438, 0x440, 0x43b, 0x430, 0x43d, 0x434, 0x20,
0x410, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x410,
0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43d, 0x433, 0x43b, 0x438, 0x430, 0x20, 0x410,
0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x430, 0x43c, 0x4ad,
0x430, 0x47, 0x65, 0x6b, 0x6f, 0xf6, 0x72, 0x64, 0x69, 0x6e, 0x65, 0x65,
0x72, 0x64, 0x65, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x65,
0x6c, 0x65, 0x20, 0x74, 0x79, 0x64, 0x49, 0x65, 0x72, 0x73, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64,
0x42, 0x72, 0x69, 0x74, 0x73, 0x65, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x79, 0x64, 0x41, 0x6d, 0x61, 0x6e, 0x73, 0x61, 0x6e, 0x20, 0x4b,
0x254, 0x64, 0x69, 0x6e, 0x61, 0x74, 0x25b, 0x64, 0x65, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x49, 0x72, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x66, 0x6f, 0x254,
0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x49, 0x6e, 0x67, 0x72, 0x65, 0x73, 0x69, 0x20, 0x41, 0x77, 0x69,
0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4f, 0x72, 0x61, 0x20, 0x75, 0x6e,
0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x65, 0x20, 0x65, 0x20, 0x6b,
0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x75, 0x61, 0x72, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x72, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20,
0x65, 0x20, 0x49, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0xeb, 0x73, 0x4f, 0x72,
0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x62, 0x72, 0x69,
0x74, 0x61, 0x6e, 0x69, 0x6b, 0x65, 0x12e8, 0x1270, 0x1240, 0x1290, 0x1263, 0x1260,
0x1228, 0x20, 0x1201, 0x1208, 0x1308, 0x1265, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a0,
0x12ed, 0x122a, 0x123d, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275,
0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1265, 0x122a, 0x1275, 0x123d, 0x20,
0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320,
0x122d, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x639,
0x627, 0x644, 0x645, 0x64a, 0x20, 0x627, 0x644, 0x645, 0x646, 0x633, 0x642, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x64a, 0x631, 0x644, 0x646, 0x62f, 0x627,
0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x628, 0x631, 0x64a, 0x637, 0x627, 0x646, 0x64a, 0x627, 0x20, 0x627, 0x644,
0x635, 0x64a, 0x641, 0x64a, 0x74, 0x69, 0x65, 0x6d, 0x70, 0x6f, 0x20, 0x75,
0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x20, 0x63, 0x6f, 0x6f,
0x72, 0x64, 0x65, 0x6e, 0x61, 0x64, 0x6f, 0x540, 0x561, 0x574, 0x561, 0x577,
0x56d, 0x561, 0x580, 0x570, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56f, 0x578, 0x578,
0x580, 0x564, 0x56b, 0x576, 0x561, 0x581, 0x57e, 0x561, 0x56e, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x53b, 0x57c, 0x56c, 0x561, 0x576, 0x564, 0x561,
0x56f, 0x561, 0x576, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x532, 0x580, 0x56b, 0x57f,
0x561, 0x576, 0x561, 0x56f, 0x561, 0x576, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561,
0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x9b8,
0x9ae, 0x9a8, 0x9cd, 0x9ac, 0x9bf, 0x9a4, 0x20, 0x9b8, 0x9be, 0x9f0, 0x9cd, 0x9ac,
0x99c, 0x9a8, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x987, 0x9f0,
0x9bf, 0x99a, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac,
0x9cd, 0x9f0, 0x9bf, 0x99f, 0x9bf, 0x99b, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7,
0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x63, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e,
0x61, 0x64, 0x61, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x69, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x73, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67,
0x61, 0x20, 0x62, 0x72, 0x69, 0x74, 0xe1, 0x6e, 0x69, 0x63, 0x61, 0x4b,
0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x61, 0x73, 0x69, 0x79, 0x61, 0x20,
0x65, 0x64, 0x69, 0x6c, 0x6d, 0x69, 0x15f, 0x20, 0xfc, 0x6d, 0x75, 0x6d,
0x64, 0xfc, 0x6e, 0x79, 0x61, 0x20, 0x76, 0x61, 0x78, 0x74, 0x131, 0x130,
0x72, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x59, 0x61, 0x79,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x42, 0x72, 0x69, 0x74, 0x61, 0x6e,
0x69, 0x79, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x9b8, 0x9cd, 0x9a5, 0x9be, 0x9a8, 0x9be, 0x982, 0x995, 0x9bf, 0x9a4, 0x20,
0x986, 0x9a8, 0x9cd, 0x9a4, 0x9b0, 0x9cd, 0x99c, 0x9be, 0x9a4, 0x9bf, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x987, 0x9b0, 0x9bf, 0x9b6, 0x20, 0x9ae, 0x9be,
0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9cd, 0x9b0, 0x9bf, 0x99f,
0x9bf, 0x9b6, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be,
0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x6f, 0x72, 0x64, 0x75,
0x20, 0x75, 0x6e, 0x69, 0x62, 0x65, 0x72, 0x74, 0x73, 0x61, 0x6c, 0x20,
0x6b, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x61, 0x74, 0x75, 0x61, 0x49,
0x72, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61,
0x4c, 0x6f, 0x6e, 0x64, 0x72, 0x65, 0x73, 0x6b, 0x6f, 0x20, 0x75, 0x64,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x423, 0x43d, 0x456,
0x432, 0x435, 0x440, 0x441, 0x430, 0x43b, 0x44c, 0x43d, 0x44b, 0x20, 0x43a, 0x430,
0x430, 0x440, 0x434, 0x44b, 0x43d, 0x430, 0x432, 0x430, 0x43d, 0x44b, 0x20, 0x447,
0x430, 0x441, 0x406, 0x440, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x456, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447,
0x430, 0x441, 0x411, 0x440, 0x44b, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20,
0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x917, 0x94b, 0x930,
0x94b, 0x92c, 0x939, 0x94b, 0x928, 0x93e, 0x92f, 0x20, 0x92e, 0x941, 0x932, 0x941,
0x917, 0x928, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x906, 0x908, 0x930, 0x940, 0x936,
0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92c,
0x94d, 0x930, 0x940, 0x91f, 0x940, 0x936, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932,
0x93e, 0x902, 0x20, 0x938, 0x92e, 0x4b, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e,
0x69, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x73, 0x76, 0x6a, 0x65, 0x74, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x72,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42, 0x72,
0x69, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41a, 0x43e,
0x43e, 0x440, 0x434, 0x438, 0x43d, 0x438, 0x441, 0x430, 0x43d, 0x43e, 0x20, 0x443,
0x43d, 0x438, 0x432, 0x435, 0x440, 0x437, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x441, 0x43a, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x438, 0x442, 0x430, 0x43d,
0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x61, 0x6d, 0x7a, 0x65, 0x72, 0x20, 0x68,
0x6f, 0x6c, 0x6c, 0x76, 0x65, 0x64, 0x65, 0x6c, 0x20, 0x6b, 0x65, 0x6e,
0x75, 0x72, 0x7a, 0x68, 0x69, 0x65, 0x74, 0x65, 0x75, 0x72, 0x20, 0x63,
0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x49, 0x77, 0x65, 0x72, 0x7a,
0x68, 0x6f, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20,
0x42, 0x72, 0x65, 0x69, 0x7a, 0x68, 0x2d, 0x56, 0x65, 0x75, 0x72, 0x41a,
0x43e, 0x43e, 0x440, 0x434, 0x438, 0x43d, 0x438, 0x440, 0x430, 0x43d, 0x43e, 0x20,
0x443, 0x43d, 0x438, 0x432, 0x435, 0x440, 0x441, 0x430, 0x43b, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x43b, 0x430, 0x43d, 0x434, 0x441,
0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x438, 0x442, 0x430,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x100a,
0x103e, 0x102d, 0x1011, 0x102c, 0x1038, 0x101e, 0x100a, 0x1037, 0x103a, 0x20, 0x1000, 0x1019,
0x1039, 0x1018, 0x102c, 0x1037, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x101b, 0x1005,
0x103a, 0x101b, 0x103e, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1017, 0x103c, 0x102d, 0x1010, 0x102d, 0x1014, 0x103a, 0x20,
0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x5354, 0x8abf, 0x4e16, 0x754c, 0x6642, 0x9593, 0x611b, 0x723e, 0x862d, 0x6a19,
0x6e96, 0x6642, 0x9593, 0x82f1, 0x570b, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x534f, 0x8c03, 0x4e16,
0x754c, 0x65f6, 0x95f4, 0x7231, 0x5c14, 0x5170, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x82f1, 0x56fd,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x54, 0x65, 0x6d, 0x70, 0x73, 0x20, 0x75, 0x6e,
0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x20, 0x63, 0x6f, 0x6f, 0x72,
0x64, 0x69, 0x6e, 0x61, 0x74, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x49, 0x72,
0x6c, 0x61, 0x6e, 0x64, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x62, 0x72, 0x69, 0x74, 0xe0, 0x6e,
0x69, 0x63, 0x61, 0xd804, 0xdd18, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804,
0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd1b, 0xd804, 0xdd28, 0xd804, 0xdd16, 0xd804, 0xdd34,
0xd804, 0xdd17, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2d, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd0c, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd11, 0xd804,
0xdd28, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0x418, 0x440, 0x43b, 0x430, 0x43d,
0x434, 0x438, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x411, 0x440, 0x438, 0x442, 0x430, 0x43d, 0x438, 0x43d,
0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445,
0x430, 0x43d, 0x13a2, 0x13a9, 0x13e0, 0x13f1, 0x20, 0x13c2, 0x13a6, 0x13d3, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a8, 0x13b5, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd,
0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c8, 0x13d7, 0x13cf, 0x20, 0x13aa,
0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x4e16, 0x754c, 0x6a19, 0x6e96, 0x6642,
0x9593, 0x432, 0x441, 0x435, 0x43c, 0x456, 0x301, 0x440, 0x43d, 0x43e, 0x435, 0x20,
0x441, 0x477, 0x433, 0x445, 0x440, 0x43e, 0x43d, 0x457, 0x437, 0x438, 0x301, 0x440,
0x43e, 0x432, 0x430, 0x43d, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301,
0x43c, 0x467, 0x41f, 0x4d7, 0x442, 0x4d7, 0x43c, 0x20, 0x442, 0x4d7, 0x43d, 0x447,
0x435, 0x440, 0x438, 0x20, 0x43a, 0x43e, 0x43e, 0x440, 0x434, 0x438, 0x43d, 0x430,
0x446, 0x438, 0x43b, 0x435, 0x43d, 0x435, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x418, 0x440, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x411, 0x440, 0x438, 0x442, 0x430, 0x43d, 0x438, 0x20, 0x4ab, 0x443,
0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x49, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x73, 0x69, 0x6e, 0x67, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x63, 0x6b, 0x4a, 0x72,
0x75, 0xdf, 0x62, 0x72, 0x65, 0x74, 0x61, 0x6e, 0x6e, 0x69, 0x6a, 0x65,
0x20, 0x73, 0x69, 0x6e, 0x67, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0x69, 0x63, 0x6b, 0x54, 0x65, 0x6d, 0x70, 0x75, 0x20, 0x75, 0x6e,
0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x65, 0x20, 0x63, 0x75, 0x75,
0x72, 0x64, 0x69, 0x6e, 0x61, 0x74, 0x75, 0x6b, 0x6f, 0x6f, 0x72, 0x64,
0x69, 0x6e, 0x69, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x73, 0x76, 0x6a, 0x65,
0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x69, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x62, 0x72, 0x69, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x6b, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x6f, 0x76, 0x61, 0x6e, 0xfd,
0x20, 0x73, 0x76, 0x11b, 0x74, 0x6f, 0x76, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x69, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x62, 0x72, 0x69, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x6f, 0x6f, 0x72,
0x64, 0x69, 0x6e, 0x65, 0x72, 0x65, 0x74, 0x20, 0x75, 0x6e, 0x69, 0x76,
0x65, 0x72, 0x73, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x49, 0x72, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x42, 0x72,
0x69, 0x74, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x924, 0x93e, 0x932, 0x92e, 0x947, 0x932, 0x940, 0x20, 0x906,
0x932, 0x92e, 0x940, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x906, 0x92f, 0x930, 0x93f,
0x936, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x92c,
0x94d, 0x930, 0x93f, 0x91f, 0x93f, 0x936, 0x20, 0x917, 0x930, 0x94d, 0x92e, 0x93f,
0x92f, 0x947, 0x902, 0x20, 0x926, 0x93e, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x67,
0x65, 0x63, 0x6f, 0xf6, 0x72, 0x64, 0x69, 0x6e, 0x65, 0x65, 0x72, 0x64,
0x65, 0x20, 0x77, 0x65, 0x72, 0x65, 0x6c, 0x64, 0x74, 0x69, 0x6a, 0x64,
0x49, 0x65, 0x72, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x42, 0x72, 0x69, 0x74, 0x73,
0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0xf68,
0xf71, 0xf0b, 0xf61, 0xf62, 0xf72, 0xf64, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b,
0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb2,
0xf71, 0xf72, 0xf0b, 0xf4a, 0xf72, 0xf64, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b,
0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0x75, 0x6e,
0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x20, 0x6b, 0x75, 0x6e, 0x6f, 0x72, 0x64, 0x69, 0x67, 0x69,
0x74, 0x61, 0x69, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x62, 0x72, 0x69,
0x74, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x4b, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x65, 0x65,
0x72, 0x69, 0x74, 0x75, 0x64, 0x20, 0x6d, 0x61, 0x61, 0x69, 0x6c, 0x6d,
0x61, 0x61, 0x65, 0x67, 0x49, 0x69, 0x72, 0x69, 0x20, 0x73, 0x75, 0x76,
0x65, 0x61, 0x65, 0x67, 0x42, 0x72, 0x69, 0x74, 0x69, 0x20, 0x73, 0x75,
0x76, 0x65, 0x61, 0x65, 0x67, 0x58, 0x65, 0x78, 0x65, 0x6d, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x256, 0x6f, 0x61, 0x6e, 0x79, 0x69,
0x20, 0x6d, 0x65, 0x49, 0x72, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6e,
0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x42, 0x72, 0x69, 0x74, 0x69, 0x73, 0x68, 0x20, 0x64,
0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x73, 0x6b, 0x69,
0x70, 0x61, 0xf0, 0x20, 0x68, 0x65, 0x69, 0x6d, 0x73, 0x74, 0xed, 0xf0,
0xcd, 0x72, 0x73, 0x6b, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20,
0x74, 0xed, 0xf0, 0x42, 0x72, 0x65, 0x74, 0x73, 0x6b, 0x20, 0x73, 0x75,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x49, 0x72, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e,
0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x42, 0x72, 0x69, 0x74, 0x61, 0x69,
0x6e, 0x55, 0x54, 0x43, 0x2d, 0x79, 0x6c, 0x65, 0x69, 0x73, 0x61, 0x69,
0x6b, 0x61, 0x49, 0x72, 0x6c, 0x61, 0x6e, 0x6e, 0x69, 0x6e, 0x20, 0x6b,
0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x42, 0x72, 0x69, 0x74, 0x61,
0x6e, 0x6e, 0x69, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69,
0x6b, 0x61, 0x74, 0x65, 0x6d, 0x70, 0x73, 0x20, 0x75, 0x6e, 0x69, 0x76,
0x65, 0x72, 0x73, 0x65, 0x6c, 0x20, 0x63, 0x6f, 0x6f, 0x72, 0x64, 0x6f,
0x6e, 0x6e, 0xe9, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9,
0x74, 0xe9, 0x20, 0x69, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x69, 0x73,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9,
0x20, 0x62, 0x72, 0x69, 0x74, 0x61, 0x6e, 0x6e, 0x69, 0x71, 0x75, 0x65,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9,
0x65, 0x20, 0x69, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x69, 0x73, 0x65,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9,
0x65, 0x20, 0x62, 0x72, 0x69, 0x74, 0x61, 0x6e, 0x6e, 0x69, 0x71, 0x75,
0x65, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd46,
0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32,
0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd2a, 0xd83a,
0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a,
0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd46, 0xd83a, 0xdd35,
0x20, 0xd83a, 0xdd04, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd33, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xc0, 0x6d, 0x20, 0x55, 0x69,
0x6c, 0x65, 0x2d, 0x63, 0x68, 0x6f, 0x69, 0x74, 0x63, 0x68, 0x65, 0x61,
0x6e, 0x6e, 0x20, 0x43, 0x6f, 0x2d, 0xf2, 0x72, 0x64, 0x61, 0x6e, 0x61,
0x69, 0x63, 0x68, 0x74, 0x65, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20,
0x6e, 0x61, 0x20, 0x68, 0x2d, 0xc8, 0x69, 0x72, 0x65, 0x61, 0x6e, 0x6e,
0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x42, 0x68, 0x72, 0x65, 0x61, 0x74, 0x61, 0x69, 0x6e,
0x6e, 0x42, 0x65, 0x20, 0x4e, 0x69, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x20,
0x4e, 0x69, 0x20, 0x59, 0x254, 0x254, 0x20, 0x4a, 0x65, 0x14b, 0x20, 0x46,
0x25b, 0x25b, 0x20, 0x4b, 0x25b, 0x74, 0x73, 0x75, 0x254, 0x20, 0x4e, 0x69,
0x69, 0x42, 0x65, 0x20, 0x4e, 0x69, 0x20, 0x49, 0x72, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x62, 0x69, 0x69, 0x20, 0x4b, 0x25b, 0x74, 0x73, 0x75, 0x254,
0x20, 0x4e, 0x69, 0x69, 0x42, 0x65, 0x20, 0x4e, 0x69, 0x20, 0x42, 0x72,
0x69, 0x74, 0x61, 0x69, 0x6e, 0x62, 0x69, 0x69, 0x20, 0x4b, 0x25b, 0x74,
0x73, 0x75, 0x254, 0x20, 0x4e, 0x69, 0x69, 0x20, 0x59, 0x25b, 0x20, 0x4c,
0x61, 0x74, 0x73, 0x61, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c,
0x20, 0x63, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x61, 0x64, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x69, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x73, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20,
0x62, 0x72, 0x69, 0x74, 0xe1, 0x6e, 0x69, 0x63, 0x61, 0x10db, 0x10e1, 0x10dd,
0x10e4, 0x10da, 0x10d8, 0x10dd, 0x20, 0x10d9, 0x10dd, 0x10dd, 0x10e0, 0x10d3, 0x10d8, 0x10dc,
0x10d8, 0x10e0, 0x10d4, 0x10d1, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8,
0x10e0, 0x10da, 0x10d0, 0x10dc, 0x10d3, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0,
0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10d1, 0x10e0, 0x10d8, 0x10e2, 0x10d0, 0x10dc, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d6,
0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x4b,
0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x69, 0x65, 0x72, 0x74, 0x65, 0x20,
0x57, 0x65, 0x6c, 0x74, 0x7a, 0x65, 0x69, 0x74, 0x49, 0x72, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65,
0x69, 0x74, 0x42, 0x72, 0x69, 0x74, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x3a3, 0x3c5,
0x3bd, 0x3c4, 0x3bf, 0x3bd, 0x3b9, 0x3c3, 0x3bc, 0x3ad, 0x3bd, 0x3b7, 0x20, 0x3a0,
0x3b1, 0x3b3, 0x3ba, 0x3cc, 0x3c3, 0x3bc, 0x3b9, 0x3b1, 0x20, 0x38f, 0x3c1, 0x3b1,
0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x399, 0x3c1, 0x3bb, 0x3b1, 0x3bd, 0x3b4, 0x3af, 0x3b1, 0x3c2, 0x398,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x392, 0x3c1,
0x3b5, 0x3c4, 0x3b1, 0x3bd, 0x3af, 0x3b1, 0x3c2, 0xab8, 0xa82, 0xa95, 0xab2, 0xabf,
0xaa4, 0x20, 0xaaf, 0xac1, 0xaa8, 0xabf, 0xab5, 0xab0, 0xacd, 0xab8, 0xab2, 0x20,
0xab8, 0xaae, 0xaaf, 0xa86, 0xa88, 0xab0, 0xabf, 0xab6, 0x20, 0xaae, 0xabe, 0xaa8,
0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaac, 0xacd, 0xab0, 0xabf, 0xa9f, 0xabf, 0xab6,
0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf,
0x48, 0x61, 0x64, 0x65, 0x77, 0x61, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x20, 0x6e, 0x61, 0x20, 0x44, 0x75, 0x6e, 0x69, 0x79, 0x61, 0x54,
0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x49, 0x72, 0x69, 0x73, 0x68, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61,
0x20, 0x6e, 0x61, 0x20, 0x42, 0x69, 0x72, 0x74, 0x61, 0x6e, 0x69, 0x79,
0x61, 0x5d6, 0x5de, 0x5df, 0x20, 0x5d0, 0x5d5, 0x5e0, 0x5d9, 0x5d1, 0x5e8, 0x5e1,
0x5dc, 0x5d9, 0x20, 0x5de, 0x5ea, 0x5d5, 0x5d0, 0x5dd, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5e7, 0x5d9, 0x5e5, 0x20, 0x5d0, 0x5d9, 0x5e8, 0x5dc, 0x5e0, 0x5d3, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d9, 0x5e5, 0x20, 0x5d1, 0x5e8, 0x5d9, 0x5d8,
0x5e0, 0x5d9, 0x5d4, 0x938, 0x92e, 0x928, 0x94d, 0x935, 0x93f, 0x924, 0x20, 0x935,
0x948, 0x936, 0x94d, 0x935, 0x93f, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x906, 0x907,
0x930, 0x93f, 0x936, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x92c, 0x94d, 0x930, 0x93f, 0x91f, 0x93f, 0x936, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f,
0x6b, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0xe1, 0x6c, 0x74, 0x20, 0x76,
0x69, 0x6c, 0xe1, 0x67, 0x69, 0x64, 0x151, 0xed, 0x72, 0x20, 0x6e, 0x79,
0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x62, 0x72, 0x69, 0x74, 0x20,
0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x53, 0x61, 0x6d,
0x72, 0xe6, 0x6d, 0x64, 0x75, 0x72, 0x20, 0x61, 0x6c, 0xfe, 0x6a, 0xf3,
0xf0, 0x6c, 0x65, 0x67, 0x75, 0x72, 0x20, 0x74, 0xed, 0x6d, 0x69, 0x53,
0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0xcd,
0x72, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x42, 0x72, 0x65, 0x74, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x4e, 0x68, 0x61, 0x7a, 0x69, 0x20, 0x4f, 0x67, 0x65,
0x20, 0x1ee4, 0x77, 0x61, 0x20, 0x4e, 0x69, 0x69, 0x6c, 0x65, 0x4f, 0x67,
0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x49,
0x72, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b,
0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x42, 0x72, 0x69, 0x74, 0x61,
0x69, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x55, 0x6e, 0x69, 0x76,
0x65, 0x72, 0x73, 0x61, 0x6c, 0x20, 0x54, 0x65, 0x72, 0x6b, 0x6f, 0x6f,
0x72, 0x64, 0x69, 0x6e, 0x61, 0x73, 0x69, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x49, 0x72, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d,
0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x49,
0x6e, 0x67, 0x67, 0x72, 0x69, 0x73, 0x55, 0x6e, 0x69, 0x76, 0x65, 0x72,
0x73, 0x61, 0x6c, 0x20, 0x54, 0x65, 0x6d, 0x70, 0x6f, 0x72, 0x65, 0x20,
0x43, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x61, 0x74, 0x65, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x69, 0x72,
0x6c, 0x61, 0x6e, 0x64, 0x65, 0x73, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x62, 0x72, 0x69, 0x74, 0x61,
0x6e, 0x6e, 0x69, 0x63, 0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61,
0x6c, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x63, 0x6f, 0x6f,
0x72, 0x64, 0x69, 0x6e, 0x61, 0x74, 0x69, 0x72, 0x6c, 0x61, 0x6e, 0x64,
0x65, 0x73, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x62, 0x72, 0x69, 0x74, 0x61,
0x6e, 0x69, 0x63, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61, 0x6c, 0x20,
0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x41, 0x6d, 0x20, 0x55, 0x69, 0x6c,
0xed, 0x6f, 0x63, 0x68, 0x20, 0x4c, 0xe1, 0x72, 0x6e, 0x61, 0x63, 0x68,
0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0xe9, 0x61, 0x6e,
0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x68, 0xc9, 0x69, 0x72, 0x65,
0x61, 0x6e, 0x6e, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x42, 0x72, 0x65, 0x61, 0x74,
0x61, 0x69, 0x6e, 0x65, 0x54, 0x65, 0x6d, 0x70, 0x6f, 0x20, 0x63, 0x6f,
0x6f, 0x72, 0x64, 0x69, 0x6e, 0x61, 0x74, 0x6f, 0x20, 0x75, 0x6e, 0x69,
0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c,
0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x49,
0x72, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x52, 0x65, 0x67,
0x6e, 0x6f, 0x20, 0x55, 0x6e, 0x69, 0x74, 0x6f, 0x5354, 0x5b9a, 0x4e16, 0x754c,
0x6642, 0x30a2, 0x30a4, 0x30eb, 0x30e9, 0x30f3, 0x30c9, 0x6a19, 0x6e96, 0x6642, 0x82f1, 0x56fd,
0x590f, 0x6642, 0x9593, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x55, 0x6e, 0x69,
0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x20, 0x4b, 0x61, 0x6b, 0x6f, 0x6f,
0x72, 0x64, 0x68, 0x69, 0x6e, 0x61, 0x73, 0x69, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x49, 0x72,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x49, 0x6e, 0x67, 0x67, 0x72,
0x69, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x55, 0x6e, 0x69, 0x76, 0x65, 0x72,
0x73, 0x61, 0x6c, 0x20, 0x4b, 0x6f, 0x72, 0x64, 0x65, 0x6e, 0x61, 0x64,
0x75, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20,
0x64, 0x69, 0x20, 0x49, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20, 0x42,
0x72, 0x69, 0x74, 0xe1, 0x6e, 0x69, 0x6b, 0x75, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x61, 0x67, 0x72, 0x61, 0x263, 0x6c, 0x61, 0x6e, 0x20, 0x69, 0x6d,
0x79, 0x75, 0x64, 0x64, 0x73, 0x65, 0x6e, 0x41, 0x67, 0x75, 0x64, 0x20,
0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x49, 0x72,
0x6c, 0x61, 0x6e, 0x64, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75,
0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x42, 0x72, 0x69, 0x74,
0x61, 0x6e, 0x79, 0x61, 0xcb8, 0xc82, 0xc98, 0xc9f, 0xcbf, 0xca4, 0x20, 0xcb8,
0xcbe, 0xcb0, 0xccd, 0xcb5, 0xca4, 0xccd, 0xcb0, 0xcbf, 0xc95, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc90, 0xcb0, 0xcbf, 0xcb7, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe,
0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcac, 0xccd, 0xcb0, 0xcbf, 0xc9f,
0xcbf, 0xcb7, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8,
0xcae, 0xcaf, 0x6a9, 0x648, 0x622, 0x631, 0x688, 0x646, 0x6cc, 0x679, 0x688, 0x20,
0x6cc, 0x648, 0x646, 0x6cc, 0x648, 0x631, 0x633, 0x644, 0x20, 0x648, 0x64e, 0x6a9,
0x6be, 0x627, 0x64e, 0x6cc, 0x631, 0x650, 0x634, 0x20, 0x633, 0x64e, 0x645, 0x64e,
0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x628, 0x631, 0x637, 0x672, 0x646,
0x648, 0x6cc, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x915, 0x94b, 0x911, 0x930, 0x94d, 0x921, 0x928, 0x948, 0x91f, 0x93f,
0x921, 0x20, 0x92f, 0x942, 0x928, 0x935, 0x930, 0x94d, 0x938, 0x932, 0x20, 0x935,
0x916, 0x414, 0x4af, 0x43d, 0x438, 0x435, 0x436, 0x4af, 0x437, 0x456, 0x43b, 0x456,
0x43a, 0x20, 0x4af, 0x439, 0x43b, 0x435, 0x441, 0x442, 0x456, 0x440, 0x456, 0x43b,
0x433, 0x435, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x418, 0x440, 0x43b,
0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x4b0,
0x43b, 0x44b, 0x431, 0x440, 0x438, 0x442, 0x430, 0x43d, 0x438, 0x44f, 0x20, 0x436,
0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x62f,
0x6c7, 0x646, 0x64a, 0x6d5, 0x62c, 0x6c7, 0x632, 0x649, 0x644, 0x649, 0x643, 0x20,
0x6c7, 0x64a, 0x644, 0x6d5, 0x633, 0x62a, 0x649, 0x631, 0x649, 0x644, 0x6af, 0x6d5,
0x646, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x64a, 0x631, 0x644, 0x627, 0x646,
0x62f, 0x64a, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a,
0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6c7, 0x644, 0x649,
0x628, 0x631, 0x64a, 0x62a, 0x627, 0x646, 0x64a, 0x627, 0x20, 0x62c, 0x627, 0x632,
0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x179f, 0x1780, 0x179b, 0x178a, 0x17c2, 0x179b, 0x1798, 0x17b6, 0x1793, 0x1780, 0x17b6,
0x179a, 0x179f, 0x1798, 0x17d2, 0x179a, 0x17bd, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x200b, 0x1793, 0x17c5,
0x200b, 0x17a2, 0x17c0, 0x179a, 0x17a1, 0x1784, 0x17cb, 0x200b, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x200b, 0x200b,
0x1793, 0x17c5, 0x200b, 0x1785, 0x1780, 0x17d2, 0x179a, 0x1797, 0x1796, 0x200b, 0x17a2, 0x1784,
0x17cb, 0x1782, 0x17d2, 0x179b, 0x17c1, 0x179f, 0x938, 0x92e, 0x928, 0x94d, 0x935, 0x93f,
0x924, 0x20, 0x935, 0x948, 0x936, 0x94d, 0x935, 0x93f, 0x915, 0x20, 0x935, 0x947,
0x933, 0x906, 0x92f, 0x930, 0x93f, 0x936, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92c, 0x94d, 0x930, 0x93f, 0x91f,
0x93f, 0x936, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935,
0x947, 0x933, 0xd611, 0xc815, 0x20, 0xc138, 0xacc4, 0xc2dc, 0xc544, 0xc77c, 0xb79c, 0xb4dc,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xc601, 0xad6d, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900,
0xc2dc, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x47, 0x65, 0x72, 0x64, 0xfb,
0x6e, 0xee, 0x20, 0x79, 0x61, 0x20, 0x48, 0x65, 0x76, 0x64, 0x65, 0x6d,
0xee, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0xce, 0x72, 0x6c, 0x61, 0x6e, 0x64,
0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76,
0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x42, 0x72, 0xee, 0x74, 0x61,
0x6e, 0x79, 0x61, 0x79, 0xea, 0x411, 0x438, 0x440, 0x434, 0x438, 0x43a, 0x442,
0x4af, 0x4af, 0x20, 0x434, 0x4af, 0x439, 0x43d, 0x4e9, 0x43b, 0x4af, 0x43a, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x418, 0x440, 0x43b, 0x430, 0x43d, 0x434,
0x438, 0x44f, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x411, 0x440, 0x438, 0x442, 0x430, 0x43d, 0x438, 0x44f,
0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0xec0, 0xea7, 0xea5, 0xeb2, 0xeaa, 0xeb2, 0xe81, 0xebb, 0xe99,
0xec0, 0xe8a, 0xeb5, 0xe87, 0xe9e, 0xeb4, 0xe81, 0xeb1, 0xe94, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2,
0xe99, 0x200b, 0xec4, 0xead, 0x200b, 0xea3, 0xeb4, 0x200b, 0xe8a, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead,
0xe99, 0x200b, 0xead, 0xeb1, 0xe87, 0x200b, 0xe81, 0xeb4, 0xe94, 0x55, 0x6e, 0x69,
0x76, 0x65, 0x72, 0x73, 0x101, 0x6c, 0x61, 0x69, 0x73, 0x20, 0x6b, 0x6f,
0x6f, 0x72, 0x64, 0x69, 0x6e, 0x113, 0x74, 0x61, 0x69, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x12a, 0x72, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x7a,
0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4c,
0x69, 0x65, 0x6c, 0x62, 0x72, 0x69, 0x74, 0x101, 0x6e, 0x69, 0x6a, 0x61,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x70, 0x61, 0x73, 0x61, 0x75, 0x6c, 0x69, 0x6f, 0x20,
0x73, 0x75, 0x64, 0x65, 0x72, 0x69, 0x6e, 0x74, 0x61, 0x73, 0x69, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x69, 0x72, 0x69, 0x6a,
0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x42, 0x72, 0x69, 0x74, 0x61, 0x6e, 0x69,
0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x6b, 0x6f, 0x6f, 0x72, 0x64, 0x69,
0x6e, 0x11b, 0x72, 0x6f, 0x77, 0x61, 0x6e, 0x79, 0x20, 0x73, 0x77, 0x11b,
0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x61, 0x73, 0x49, 0x72, 0x69, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x42, 0x72, 0x69, 0x74, 0x69, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x49, 0x72, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0xe4, 0x69, 0x74, 0x42, 0x72, 0x69, 0x74, 0x65, 0x73, 0x63,
0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74,
0x41a, 0x43e, 0x43e, 0x440, 0x434, 0x438, 0x43d, 0x438, 0x440, 0x430, 0x43d, 0x43e,
0x20, 0x443, 0x43d, 0x438, 0x432, 0x435, 0x440, 0x437, 0x430, 0x43b, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x441, 0x43a, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x438, 0x442, 0x430, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0xd15, 0xd4b, 0xd7c, 0xd21, 0xd3f, 0xd28, 0xd47, 0xd31, 0xd4d, 0xd31, 0xd21,
0xd4d, 0x20, 0xd2f, 0xd42, 0xd23, 0xd3f, 0xd35, 0xd47, 0xd34, 0xd4d, 0x200c, 0xd38,
0xd7d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd10, 0xd31, 0xd3f, 0xd37, 0xd4d, 0x20,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2c, 0xd4d, 0xd30, 0xd3f, 0xd1f, 0xd4d, 0xd1f,
0xd40, 0xd37, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e,
0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x55, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x20,
0x53, 0x65, 0x6c, 0x61, 0x72, 0x61, 0x73, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x49, 0x72, 0x65, 0x6c,
0x61, 0x6e, 0x64, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73,
0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x42, 0x72, 0x69,
0x74, 0x69, 0x73, 0x68, 0x995, 0x9cb, 0x993, 0x9b0, 0x9cd, 0x9a6, 0x9bf, 0x9a8,
0x9c7, 0x99f, 0x9c7, 0x9a6, 0x20, 0x9af, 0x9bc, 0x9c1, 0x9a8, 0x9bf, 0x9ad, 0x9b0,
0x9cd, 0x9b8, 0x9c7, 0x9b2, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x57, 0x101, 0x20,
0x41, 0x6f, 0x6e, 0x75, 0x69, 0x20, 0x4b, 0x14d, 0x74, 0x75, 0x69, 0x74,
0x75, 0x69, 0x57, 0x101, 0x20, 0x41, 0x69, 0x72, 0x69, 0x68, 0x69, 0x20,
0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20,
0x50, 0x69, 0x72, 0x69, 0x74, 0x61, 0x6e, 0x61, 0x20, 0x52, 0x61, 0x75,
0x6d, 0x61, 0x74, 0x69, 0x906, 0x92f, 0x930, 0x93f, 0x936, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92c, 0x94d, 0x930, 0x93f,
0x91f, 0x93f, 0x936, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20,
0x935, 0x947, 0x933, 0x41e, 0x43b, 0x43e, 0x43d, 0x20, 0x443, 0x43b, 0x441, 0x44b,
0x43d, 0x20, 0x437, 0x43e, 0x445, 0x438, 0x446, 0x443, 0x443, 0x43b, 0x430, 0x43b,
0x442, 0x442, 0x430, 0x439, 0x20, 0x446, 0x430, 0x433, 0x418, 0x440, 0x43b, 0x430,
0x43d, 0x434, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x446, 0x430, 0x433, 0x411, 0x440, 0x438, 0x442, 0x430, 0x43d, 0x438,
0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x1823,
0x182f, 0x1820, 0x1828, 0x20, 0x1823, 0x182f, 0x1823, 0x1830, 0x202f, 0x1824, 0x1828, 0x20,
0x1835, 0x1823, 0x182c, 0x1822, 0x1834, 0x1821, 0x182d, 0x1823, 0x182f, 0x1823, 0x182f, 0x1833,
0x1820, 0x1833, 0x1820, 0x1822, 0x20, 0x1834, 0x1820, 0x182d, 0x938, 0x92e, 0x928, 0x94d,
0x935, 0x93f, 0x924, 0x20, 0x935, 0x93f, 0x936, 0x94d, 0x935, 0x20, 0x938, 0x92e,
0x92f, 0x92c, 0x947, 0x932, 0x93e, 0x92f, 0x924, 0x940, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e,
0x92f, 0x41, 0x72, 0xe9, 0x6e, 0x6a, 0x6d, 0x1eb9, 0x6e, 0x74, 0x20, 0x1ecd,
0x66, 0x20, 0x44, 0x69, 0x20, 0x48, 0xf3, 0x6c, 0x20, 0x57, 0x1ecd, 0x6c,
0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x41, 0xed, 0x72, 0xed, 0x73, 0x68,
0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x42,
0x72, 0xed, 0x74, 0xed, 0x73, 0x68, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20,
0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x7d6, 0x7ca,
0x7ef, 0x7d3, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7eb, 0x20,
0x7d8, 0x7cf, 0x7f2, 0x7ec, 0x7d6, 0x7df, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7e3,
0x7cd, 0x7f2, 0x7cc, 0x7d9, 0x7d1, 0x7df, 0x7ca, 0x7f2, 0x7d8, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8,
0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d3, 0x7d9, 0x7cc, 0x7d5, 0x7ca, 0x7f2, 0x7eb, 0x20,
0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8,
0x7ca, 0x6b, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x65, 0x72, 0x65, 0x6a,
0x75, 0x76, 0x76, 0x6f, 0x6e, 0x20, 0x6f, 0x6b, 0x74, 0x61, 0x73, 0x61,
0x161, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x49, 0x72, 0x6c, 0xe1, 0x6e, 0x64,
0x64, 0x61, 0x6c, 0x61, 0x161, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x42, 0x72, 0x69, 0x68, 0x74, 0x61, 0x6c, 0x61, 0x161,
0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x6b,
0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x65, 0x72, 0x74, 0x20, 0x75, 0x6e,
0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x69, 0x72,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x62, 0x72, 0x69, 0x74, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x73, 0x6f,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x62, 0x72, 0x69, 0x74, 0x69,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x6f, 0x72, 0x61, 0x20, 0x63, 0x6f, 0x6f, 0x72, 0x64, 0x6f, 0x6e, 0x61,
0x64, 0x61, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c,
0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x20, 0x69, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x73,
0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75,
0x20, 0x62, 0x72, 0x69, 0x74, 0x61, 0x6e, 0x69, 0x63, 0x61, 0x6f, 0x72,
0x61, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x75, 0x20,
0x63, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x61, 0x64, 0x61, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019,
0x49, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0xb38, 0xb2e, 0xb28, 0xb4d, 0xb71,
0xb3f, 0xb24, 0x20, 0xb38, 0xb3e, 0xb30, 0xb4d, 0xb2c, 0xb1c, 0xb28, 0xb40, 0xb28,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb07, 0xb30, 0xb3f, 0xb36, 0xb4d, 0x200c, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2c, 0xb4d, 0xb30, 0xb3f,
0xb1f, 0xb3f, 0xb36, 0xb4d, 0x200c, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d,
0xb2e, 0xb15, 0xb3e, 0xb33, 0xb40, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x64, 0x69, 0x6c, 0x2d,
0x41, 0x64, 0x64, 0x75, 0x6e, 0x79, 0x61, 0x61, 0x20, 0x51, 0x69, 0x6e,
0x64, 0x61, 0x61, 0x2019, 0x65, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x69, 0x69, 0x20, 0x41, 0x61, 0x79, 0x69, 0x72, 0x69, 0x69, 0x73,
0x68, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f,
0x6e, 0x61, 0x61, 0x20, 0x42, 0x69, 0x72, 0x69, 0x74, 0x69, 0x73, 0x68,
0x647, 0x645, 0x63a, 0x696, 0x649, 0x20, 0x646, 0x693, 0x6cc, 0x648, 0x627, 0x644,
0x20, 0x648, 0x62e, 0x62a, 0x622, 0x64a, 0x631, 0x634, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x628, 0x631, 0x64a, 0x62a, 0x627,
0x646, 0x648, 0x64a, 0x20, 0x62f, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648,
0x62e, 0x62a, 0x632, 0x645, 0x627, 0x646, 0x20, 0x647, 0x645, 0x627, 0x647, 0x646,
0x6af, 0x20, 0x62c, 0x647, 0x627, 0x646, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x639,
0x627, 0x62f, 0x6cc, 0x20, 0x627, 0x6cc, 0x631, 0x644, 0x646, 0x62f, 0x648, 0x642,
0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x628,
0x631, 0x6cc, 0x62a, 0x627, 0x646, 0x6cc, 0x627, 0x75, 0x6e, 0x69, 0x77, 0x65,
0x72, 0x73, 0x61, 0x6c, 0x6e, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x6b, 0x6f, 0x6f, 0x72, 0x64, 0x79, 0x6e, 0x6f, 0x77, 0x61, 0x6e, 0x79,
0x49, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x42, 0x72, 0x79,
0x74, 0x79, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x55, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x20, 0x43,
0x6f, 0x6f, 0x72, 0x64, 0x65, 0x6e, 0x61, 0x64, 0x6f, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x49, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0xea, 0x73, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x42, 0x72, 0x69, 0x74, 0xe2, 0x6e, 0x69, 0x63, 0x6f, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x43, 0x6f, 0x6f, 0x72, 0x64, 0x65, 0x6e, 0x61, 0x64,
0x61, 0x20, 0x55, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x61, 0x20, 0x49, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x42, 0x72, 0x69, 0x74, 0xe2, 0x6e, 0x69, 0x63, 0x61, 0xa15, 0xa4b,
0xa14, 0xa30, 0xa21, 0xa40, 0xa28, 0xa47, 0xa1f, 0xa47, 0xa21, 0x20, 0xa35, 0xa3f,
0xa06, 0xa2a, 0xa15, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa06, 0xa07, 0xa30, 0xa3f,
0xa38, 0xa3c, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa2c, 0xa4d, 0xa30, 0xa3f, 0xa1f, 0xa3f, 0xa38, 0xa3c, 0x20, 0xa17, 0xa30,
0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0x54, 0x69, 0x71, 0x73, 0x69, 0x6d, 0x75, 0x79, 0x75, 0x6e, 0x74, 0x69,
0x6e, 0x20, 0x54, 0x75, 0x70, 0x61, 0x63, 0x68, 0x69, 0x73, 0x71, 0x61,
0x20, 0x48, 0x6f, 0x72, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72,
0x6c, 0x61, 0x6e, 0x64, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x42, 0x72, 0x69, 0x74,
0xe1, 0x6e, 0x69, 0x63, 0x6f, 0x54, 0x69, 0x6d, 0x70, 0x75, 0x6c, 0x20,
0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x20, 0x63, 0x6f,
0x6f, 0x72, 0x64, 0x6f, 0x6e, 0x61, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x49, 0x72, 0x6c,
0x61, 0x6e, 0x64, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x61, 0x72, 0x103, 0x20, 0x62, 0x72, 0x69, 0x74, 0x61, 0x6e, 0x69,
0x63, 0x103, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65,
0x72, 0x73, 0x61, 0x6c, 0x20, 0x63, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e,
0xe0, 0x74, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73, 0x74, 0x61,
0x64, 0x20, 0x62, 0x72, 0x69, 0x74, 0x61, 0x6e, 0x6e, 0x69, 0x63, 0x412,
0x441, 0x435, 0x43c, 0x438, 0x440, 0x43d, 0x43e, 0x435, 0x20, 0x43a, 0x43e, 0x43e,
0x440, 0x434, 0x438, 0x43d, 0x438, 0x440, 0x43e, 0x432, 0x430, 0x43d, 0x43d, 0x43e,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x418, 0x440, 0x43b, 0x430, 0x43d,
0x434, 0x438, 0x44f, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x412, 0x435,
0x43b, 0x438, 0x43a, 0x43e, 0x431, 0x440, 0x438, 0x442, 0x430, 0x43d, 0x438, 0x44f,
0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x938, 0x92e, 0x928, 0x94d, 0x935, 0x93f, 0x924, 0x903, 0x20, 0x935,
0x948, 0x936, 0x94d, 0x935, 0x93f, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x3a, 0x1c60,
0x1c5a, 0x1c68, 0x1c70, 0x1c64, 0x1c71, 0x1c6e, 0x1c74, 0x1c6e, 0x1c70, 0x20, 0x1c6d, 0x1c69,
0x1c71, 0x1c64, 0x1c63, 0x1c5f, 0x1c68, 0x1c65, 0x1c5f, 0x1c5e, 0x20, 0x1c5a, 0x1c60, 0x1c5b,
0x1c5a, 0x54, 0x65, 0x6d, 0x70, 0x75, 0x73, 0x20, 0x63, 0x6f, 0x6f, 0x72,
0x64, 0x69, 0x6e, 0x61, 0x64, 0x75, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65,
0x72, 0x73, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x69, 0x72, 0x6c, 0x61, 0x6e,
0x64, 0x65, 0x73, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x69,
0x61, 0x6c, 0x65, 0x20, 0x62, 0x72, 0x69, 0x74, 0xe0, 0x6e, 0x6e, 0x69,
0x63, 0x61, 0x41a, 0x43e, 0x43e, 0x440, 0x434, 0x438, 0x43d, 0x438, 0x441, 0x430,
0x43d, 0x43e, 0x20, 0x443, 0x43d, 0x438, 0x432, 0x435, 0x440, 0x437, 0x430, 0x43b,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x441, 0x43a,
0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x438, 0x442, 0x430,
0x43d, 0x438, 0x458, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x441, 0x43a, 0x430, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x438, 0x442, 0x430, 0x43d,
0x438, 0x458, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x4b, 0x6f, 0x6f, 0x72, 0x64, 0x69,
0x6e, 0x69, 0x73, 0x61, 0x6e, 0x6f, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65,
0x72, 0x7a, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x49, 0x72, 0x73, 0x6b, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x42, 0x72, 0x69, 0x74, 0x61, 0x6e, 0x69, 0x6a, 0x61,
0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e,
0x69, 0x73, 0x61, 0x6e, 0x6f, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65, 0x72,
0x7a, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x49,
0x72, 0x73, 0x6b, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x42, 0x72,
0x69, 0x74, 0x61, 0x6e, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x55, 0x72, 0x61,
0x20, 0x43, 0x75, 0x72, 0x64, 0x69, 0x6e, 0x61, 0x74, 0x61, 0x20, 0x55,
0x6e, 0x69, 0x76, 0x69, 0x72, 0x73, 0x61, 0x6c, 0x69, 0x55, 0x72, 0x61,
0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x49, 0x72, 0x6c, 0x61,
0x6e, 0x6e, 0x69, 0x73, 0x69, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67,
0x61, 0x6c, 0x69, 0x20, 0x42, 0x72, 0x69, 0x74, 0xe0, 0x6e, 0x6e, 0x69,
0x63, 0x61, 0x75, 0x6e, 0x69, 0x77, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x6e,
0x79, 0x20, 0x6b, 0x6f, 0x6f, 0x72, 0x64, 0x79, 0x6e, 0x6f, 0x77, 0x61,
0x6e, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x49, 0x72, 0x6c, 0x61, 0x6e,
0x64, 0x79, 0x6a, 0x6f, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x42, 0x72, 0x79, 0x74, 0x79, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x6af, 0x68f, 0x64a, 0x644, 0x20, 0x62f, 0x646, 0x64a, 0x627,
0x648, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x626, 0x631, 0x634, 0x20, 0x62c,
0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a,
0x628, 0x631, 0x637, 0x627, 0x646, 0x64a, 0x627, 0x20, 0x62c, 0x64a, 0x20, 0x627,
0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x917, 0x926, 0x93f, 0x952, 0x92f, 0x932, 0x20, 0x906, 0x932, 0x92e, 0x940,
0x20, 0x935, 0x915, 0x94d, 0x924, 0x941, 0xdc3, 0xdb8, 0xd9a, 0xd9a, 0xdca, 0xdc2,
0x20, 0xdc3, 0xdcf, 0xdbb, 0xdca, 0xdc0, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xd85, 0xdba, 0xdbb, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xdad, 0x20, 0xdc3, 0xdb8, 0xdca,
0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xdca, 0x200d, 0xdbb,
0xdd2, 0xdad, 0xdcf, 0xdb1, 0xdca, 0x200d, 0xdba, 0x20, 0xd9c, 0xdd2, 0xdb8, 0xdca,
0xdc4, 0xdcf, 0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0x6b, 0x6f, 0x6f,
0x72, 0x64, 0x69, 0x6e, 0x6f, 0x76, 0x61, 0x6e, 0xfd, 0x20, 0x73, 0x76,
0x65, 0x74, 0x6f, 0x76, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0xed, 0x72, 0x73,
0x6b, 0x79, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x72, 0x69, 0x74, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x75, 0x6e,
0x69, 0x76, 0x65, 0x72, 0x7a, 0x61, 0x6c, 0x6e, 0x69, 0x20, 0x6b, 0x6f,
0x6f, 0x72, 0x64, 0x69, 0x6e, 0x69, 0x72, 0x61, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x49, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x42, 0x72,
0x69, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x49, 0x73, 0x6b, 0x75, 0x2d, 0x78, 0x69, 0x72, 0x61,
0x6e, 0x20, 0x65, 0x65, 0x20, 0x43, 0x61, 0x61, 0x6c, 0x61, 0x6d, 0x6b,
0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61,
0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x41,
0x79, 0x72, 0x69, 0x73, 0x68, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65,
0x65, 0x20, 0x42, 0x69, 0x72, 0x69, 0x74, 0x69, 0x73, 0x68, 0x6b, 0x61,
0x74, 0x69, 0x65, 0x6d, 0x70, 0x6f, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65,
0x72, 0x73, 0x61, 0x6c, 0x20, 0x63, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e,
0x61, 0x64, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6c,
0x61, 0x6e, 0x64, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x62, 0x72, 0x69, 0x74, 0xe1,
0x6e, 0x69, 0x63, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6c,
0x61, 0x6e, 0x64, 0x61, 0x4d, 0x66, 0x75, 0x6d, 0x6f, 0x20, 0x77, 0x61,
0x20, 0x6b, 0x75, 0x72, 0x61, 0x74, 0x69, 0x62, 0x75, 0x20, 0x73, 0x61,
0x61, 0x20, 0x75, 0x6c, 0x69, 0x6d, 0x77, 0x65, 0x6e, 0x67, 0x75, 0x6e,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69,
0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a,
0x61, 0x20, 0x41, 0x79, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20,
0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x55,
0x69, 0x6e, 0x67, 0x65, 0x72, 0x65, 0x7a, 0x61, 0x53, 0x61, 0x61, 0x20,
0x79, 0x61, 0x20, 0x44, 0x75, 0x6e, 0x69, 0x61, 0x6b, 0x6f, 0x6f, 0x72,
0x64, 0x69, 0x6e, 0x65, 0x72, 0x61, 0x64, 0x20, 0x75, 0x6e, 0x69, 0x76,
0x65, 0x72, 0x73, 0x65, 0x6c, 0x6c, 0x20, 0x74, 0x69, 0x64, 0x69, 0x72,
0x6c, 0xe4, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x62, 0x72, 0x69, 0x74, 0x74, 0x69, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x725, 0x715,
0x722, 0x718, 0x72c, 0x710, 0x20, 0x72c, 0x712, 0x720, 0x71d, 0x72c, 0x710, 0x20,
0x721, 0x71b, 0x71f, 0x718, 0x723, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20,
0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710,
0x71d, 0x72a, 0x720, 0x722, 0x715, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b,
0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x712, 0x72a, 0x71d,
0x71b, 0x722, 0x71d, 0x710, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x4b7, 0x430,
0x4b3, 0x43e, 0x43d, 0x438, 0x438, 0x20, 0x4b3, 0x430, 0x43c, 0x43e, 0x4b3, 0x430,
0x43d, 0x433, 0x441, 0x43e, 0x437, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x418,
0x440, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20,
0x411, 0x440, 0x438, 0x442, 0x430, 0x43d, 0x438, 0x44f, 0xb92, 0xbb0, 0xbc1, 0xb99,
0xbcd, 0xb95, 0xbbf, 0xba3, 0xbc8, 0xba8, 0xbcd, 0xba4, 0x20, 0xb9a, 0xbb0, 0xbcd,
0xbb5, 0xba4, 0xbc7, 0xb9a, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb90, 0xbb0,
0xbbf, 0xbb7, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbbf, 0xbb0, 0xbbf, 0xb9f, 0xbcd, 0xb9f,
0xbbf, 0xbb7, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0x411, 0x4e9, 0x442, 0x435, 0x43d, 0x434, 0x4e9, 0x43d, 0x44c, 0x44f,
0x20, 0x43a, 0x438, 0x43b, 0x435, 0x448, 0x442, 0x435, 0x440, 0x435, 0x43b, 0x433,
0x4d9, 0x43d, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x418, 0x440, 0x43b,
0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x411, 0x440, 0x438,
0x442, 0x430, 0x43d, 0x438, 0x44f, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0xc38, 0xc2e, 0xc28, 0xc4d, 0xc35, 0xc2f,
0x20, 0xc38, 0xc3e, 0xc30, 0xc4d, 0xc35, 0xc1c, 0xc28, 0xc40, 0xc28, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc10, 0xc30, 0xc3f, 0xc37, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30,
0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2c,
0xc4d, 0xc30, 0xc3f, 0xc1f, 0xc40, 0xc37, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35,
0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2a, 0xe32,
0xe01, 0xe25, 0xe40, 0xe0a, 0xe34, 0xe07, 0xe1e, 0xe34, 0xe01, 0xe31, 0xe14, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe44, 0xe2d,
0xe23, 0xe4c, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24,
0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe2d, 0xe31, 0xe07, 0xe01, 0xe24, 0xe29,
0x12dd, 0x1270, 0x1233, 0x1290, 0x12e8, 0x20, 0x12a3, 0x12f5, 0x121b, 0x1233, 0x12ca, 0x20,
0x130d, 0x12dc, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12a0, 0x12ed,
0x122a, 0x123d, 0x20, 0x130d, 0x12d8, 0x130d, 0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272,
0x20, 0x1265, 0x122a, 0x1323, 0x1295, 0x12eb, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x65, 0x6d, 0x101, 0x6d, 0x61, 0x6e, 0x69, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x65, 0x61,
0x6c, 0x61, 0x6e, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74,
0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x70, 0x69, 0x6c, 0x69, 0x74, 0x101, 0x6e, 0x69, 0x61, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x45,
0x15f, 0x20, 0x47, 0xfc, 0x64, 0xfc, 0x6d, 0x6c, 0xfc, 0x20, 0x45, 0x76,
0x72, 0x65, 0x6e, 0x73, 0x65, 0x6c, 0x20, 0x5a, 0x61, 0x6d, 0x61, 0x6e,
0x130, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x130, 0x6e,
0x67, 0x69, 0x6c, 0x74, 0x65, 0x72, 0x65, 0x20, 0x59, 0x61, 0x7a, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x55, 0x74, 0x67, 0x61, 0x15f, 0x64, 0x79,
0x72, 0x79, 0x6c, 0xfd, 0x61, 0x6e, 0x20, 0xe4, 0x68, 0x6c, 0x69, 0x75,
0x6d, 0x75, 0x6d, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x49, 0x72, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0xfd, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x42, 0x65, 0xfd,
0x69, 0x6b, 0x20, 0x42, 0x72, 0x69, 0x74, 0x61, 0x6e, 0x69, 0xfd, 0x61,
0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x437, 0x430, 0x20, 0x432, 0x441, 0x435, 0x441, 0x432, 0x456, 0x442,
0x43d, 0x456, 0x43c, 0x20, 0x43a, 0x43e, 0x43e, 0x440, 0x434, 0x438, 0x43d, 0x43e,
0x432, 0x430, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x432, 0x20, 0x406, 0x440, 0x43b, 0x430, 0x43d, 0x434, 0x456,
0x457, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x412, 0x435, 0x43b, 0x438, 0x43a,
0x456, 0x439, 0x20, 0x411, 0x440, 0x438, 0x442, 0x430, 0x43d, 0x456, 0x457, 0x6b,
0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x6f, 0x77, 0x61, 0x6e, 0x79, 0x20,
0x73, 0x77, 0x11b, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x49,
0x72, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x42, 0x72, 0x69, 0x74, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6a9, 0x648, 0x622, 0x631,
0x688, 0x6cc, 0x646, 0x6cc, 0x679, 0x688, 0x20, 0x6cc, 0x648, 0x646, 0x6cc, 0x648,
0x631, 0x633, 0x644, 0x20, 0x679, 0x627, 0x626, 0x645, 0x622, 0x626, 0x631, 0x634,
0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x626, 0x645, 0x628, 0x631, 0x679, 0x634, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679,
0x627, 0x626, 0x645, 0x626, 0x649, 0x631, 0x6d0, 0x644, 0x627, 0x646, 0x62f, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x626, 0x6d5, 0x646, 0x6af, 0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x64a, 0x627, 0x632,
0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x4b, 0x6f, 0x6f,
0x72, 0x64, 0x69, 0x6e, 0x61, 0x74, 0x61, 0x6c, 0x69, 0x20, 0x75, 0x6e,
0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x20, 0x76, 0x61, 0x71, 0x74,
0x49, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x79, 0x6f,
0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x42, 0x72, 0x69,
0x74, 0x61, 0x6e, 0x69, 0x79, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x418, 0x440, 0x43b, 0x430, 0x43d, 0x434,
0x438, 0x44f, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x411, 0x440, 0x438, 0x442, 0x430, 0x43d, 0x438, 0x44f, 0x20, 0x451, 0x437,
0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x47, 0x69, 0x1edd, 0x20,
0x50, 0x68, 0x1ed1, 0x69, 0x20, 0x68, 0x1ee3, 0x70, 0x20, 0x51, 0x75, 0x1ed1,
0x63, 0x20, 0x74, 0x1ebf, 0x47, 0x69, 0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9,
0x6e, 0x20, 0x41, 0x69, 0x2d, 0x6c, 0x65, 0x6e, 0x47, 0x69, 0x1edd, 0x20,
0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x41, 0x6e, 0x68, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x43, 0x79, 0x66, 0x66, 0x72, 0x65, 0x64, 0x6e,
0x69, 0x6f, 0x6c, 0x20, 0x43, 0x79, 0x64, 0x6c, 0x79, 0x6e, 0x6f, 0x6c,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f,
0x6c, 0x20, 0x49, 0x77, 0x65, 0x72, 0x64, 0x64, 0x6f, 0x6e, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x50, 0x72, 0x79, 0x64,
0x61, 0x69, 0x6e, 0x6b, 0x6f, 0xf6, 0x72, 0x64, 0x69, 0x6e, 0x65, 0x61,
0x72, 0x72, 0x65, 0x20, 0x77, 0x72, 0xe2, 0x6c, 0x64, 0x74, 0x69, 0x69,
0x64, 0x49, 0x65, 0x72, 0x73, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x69, 0x64, 0x42, 0x72, 0x69, 0x74, 0x73, 0x65, 0x20,
0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x43, 0x55,
0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x69, 0x6e, 0x69,
0x77, 0x65, 0x6c, 0x73, 0x65, 0x6c, 0x20, 0x79, 0x75, 0xf1, 0x20, 0x62,
0x6f, 0x6f, 0x6c, 0x65, 0x29, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x49,
0x72, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20,
0x65, 0x74, 0x65, 0x20, 0x62, 0x75, 0x20, 0x47, 0x72, 0x61, 0x6e, 0x64,
0x65, 0x20, 0x42, 0x72, 0x65, 0x74, 0x61, 0x67, 0x6e, 0x65, 0xc0, 0x70,
0x61, 0x70, 0x1ecd, 0x300, 0x20, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0,
0x67, 0x62, 0xe1, 0x79, 0xe9, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0,
0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x41, 0x69, 0x72, 0x69, 0x69,
0x73, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0,
0x20, 0x47, 0x1eb9, 0x1eb9, 0x73, 0x69, 0xc0, 0x70, 0x61, 0x70, 0x254, 0x300,
0x20, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x67, 0x62, 0xe1, 0x79,
0xe9, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75,
0x6b, 0xf2, 0x20, 0x41, 0x69, 0x72, 0x69, 0x69, 0x73, 0x69, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0x47, 0x25b, 0x25b,
0x73, 0x69, 0x69, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x6f, 0x6d, 0x68, 0x6c, 0x61, 0x62, 0x61, 0x20, 0x65, 0x73, 0x69,
0x64, 0x69, 0x64, 0x69, 0x79, 0x65, 0x6c, 0x77, 0x65, 0x69, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x49, 0x72, 0x69, 0x73, 0x68, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d,
0x69, 0x6c, 0x65, 0x69, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x42, 0x72, 0x69, 0x74, 0x69, 0x73,
0x68, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x47,
0x61, 0x2d, 0x6b, 0x61, 0x72, 0x20, 0xd3, 0x72, 0x61, 0x20, 0x56, 0x1ebd,
0x6e, 0x68, 0x2d, 0x6b, 0x72, 0xe9, 0x6e, 0x2d, 0x6a, 0x61, 0xd3, 0x72,
0x61, 0x20, 0x50, 0xe3, 0x20, 0x49, 0x72, 0x1ef9, 0x6e, 0x61, 0xd3, 0x72,
0x61, 0x20, 0x4d, 0x72, 0x69, 0x74, 0x1ef9, 0x6e, 0x129, 0x6b, 0x75, 0x20,
0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x20, 0x4d, 0x169, 0x64, 0x69, 0x20, 0x74, 0x75, 0x72, 0x75, 0x73,
0x75, 0x73, 0x61, 0x77, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x64, 0x65, 0x6e,
0x61, 0x64, 0x75, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x20, 0x49, 0x72, 0xe3, 0x64, 0x65,
0x69, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x4b, 0x75,
0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x42, 0x69, 0x72,
0x69, 0x74, 0xe3, 0x6e, 0x69, 0x6b, 0x75, 0x55, 0x66, 0x73, 0x74, 0x65,
0x6d, 0x65, 0x74, 0x20, 0x57, 0xe4, 0xe4, 0x6c, 0x74, 0x74, 0x69, 0x64,
0x6a, 0x49, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x42, 0x72, 0x69, 0x74,
0x69, 0x73, 0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x6a, 0x48, 0x61, 0x6d, 0x64, 0xe1, 0x72, 0xe9, 0x6e, 0x20, 0x4a, 0x61,
0x68, 0xe1, 0x6e, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0xc1, 0x65, 0x72,
0x69, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0x42, 0x61, 0x72, 0x74, 0xe1, 0x6e, 0x69, 0xe1, 0x69, 0x20, 0x67,
0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73,
0x61, 0x6c, 0x65, 0x20, 0x63, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x6f,
0x75, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64,
0x2019, 0x49, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x62, 0x72, 0x69, 0x74, 0x61, 0x6e,
0x6e, 0x69, 0x63, 0x61, 0x189, 0x75, 0x6c, 0x69, 0x6e, 0x79, 0x61, 0x20,
0x61, 0x14b, 0x75, 0x6e, 0x69, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x49, 0x72, 0x6c, 0x61, 0x6e, 0x256,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x47, 0x61, 0x67, 0x25b, 0x25b, 0x73, 0x68, 0x269, 0x74, 0x1dd, 0x6e, 0x61,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x938, 0x92e, 0x928,
0x94d, 0x935, 0x93f, 0x924, 0x20, 0x926, 0x941, 0x928, 0x93f, 0x92f, 0x93e, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x906, 0x907, 0x930, 0x93f, 0x936, 0x20,
0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92c,
0x94d, 0x930, 0x93f, 0x91f, 0x93f, 0x936, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b,
0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x4f, 0x72, 0x61, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61,
0x6c, 0x65, 0x20, 0x63, 0x6f, 0x6f, 0x72, 0x64, 0x65, 0x6e, 0x61, 0x64,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20,
0x69, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x7a, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x62, 0x72, 0x69, 0x74,
0xe0, 0x6e, 0x65, 0x67, 0x61, 0x73, 0x6d, 0x61, 0x6e, 0x69, 0x20, 0x6a,
0x61, 0x67, 0x61, 0x20, 0x70, 0x72, 0x75, 0x74, 0x69, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x61, 0x61, 0x69, 0x72, 0x69, 0x73, 0x20, 0x6d, 0x61, 0x6e,
0x61, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x62, 0x72, 0x69, 0x1e6d,
0x69, 0x73, 0x20, 0x6b, 0x61, 0x72, 0xe3, 0x20, 0x6d, 0x61, 0x73, 0x61,
0x20, 0x62, 0x65, 0x6c, 0x101, 0xb38, 0xb2e, 0xb3e, 0xb28, 0xb3f, 0x20, 0xb1c,
0xb3e, 0xb17, 0xb3e, 0x20, 0xb2a, 0xb4d, 0xb30, 0xb41, 0xb24, 0xb3f, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb06, 0xb07, 0xb30, 0xb3f, 0xb38, 0xb4d, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2c, 0xb4d,
0xb30, 0xb3f, 0xb1f, 0xb3f, 0xb36, 0xb4d, 0x200c, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e,
0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e
};

static inline constexpr char16_t shortMetaZoneNameTable[] = {
0x41, 0x4b, 0x54, 0x41, 0x4b, 0x53, 0x54, 0x41, 0x4b, 0x44, 0x54, 0x43,
0x54, 0x43, 0x53, 0x54, 0x43, 0x44, 0x54, 0x45, 0x54, 0x45, 0x53, 0x54,
0x45, 0x44, 0x54, 0x4d, 0x54, 0x4d, 0x53, 0x54, 0x4d, 0x44, 0x54, 0x50,
0x54, 0x50, 0x53, 0x54, 0x50, 0x44, 0x54, 0x41, 0x54, 0x41, 0x53, 0x54,
0x41, 0x44, 0x54, 0x47, 0x4d, 0x54, 0x48, 0x41, 0x54, 0x48, 0x41, 0x53,
0x54, 0x48, 0x41, 0x44, 0x54, 0x43, 0x41, 0x54, 0x45, 0x41, 0x54, 0x53,
0x41, 0x53, 0x54, 0x57, 0x41, 0x54, 0x57, 0x41, 0x53, 0x54, 0x47, 0x53,
0x54, 0x9ad, 0x9be, 0x2e, 0x20, 0x9b8, 0x2e, 0x43, 0x45, 0x54, 0x43, 0x45,
0x53, 0x54, 0x45, 0x45, 0x54, 0x45, 0x45, 0x53, 0x54, 0x57, 0x45, 0x54,
0x57, 0x45, 0x53, 0x54, 0x49, 0x53, 0x54, 0x4d, 0x59, 0x54, 0x53, 0x47,
0x54, 0x41, 0x43, 0x54, 0x20, 0x41, 0x63, 0x72, 0x65, 0x41, 0x43, 0x53,
0x54, 0x20, 0x41, 0x63, 0x72, 0x65, 0x41, 0x46, 0x54, 0x41, 0x4c, 0x4d,
0x54, 0x41, 0x4c, 0x4d, 0x54, 0x20, 0x44, 0x53, 0x54, 0x41, 0x4d, 0x54,
0x41, 0x4d, 0x53, 0x54, 0x41, 0x4e, 0x41, 0x54, 0x41, 0x4e, 0x41, 0x53,
0x54, 0x53, 0x53, 0x54, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x41,
0x70, 0x69, 0x61, 0x53, 0x53, 0x54, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x20, 0x41, 0x70, 0x69, 0x61, 0x20, 0x44, 0x53, 0x54, 0x41, 0x4c, 0x4d,
0x54, 0x20, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x41, 0x4c, 0x4d, 0x54, 0x20,
0x41, 0x6b, 0x74, 0x61, 0x75, 0x20, 0x44, 0x53, 0x54, 0x41, 0x51, 0x54,
0x54, 0x41, 0x51, 0x54, 0x54, 0x20, 0x44, 0x53, 0x54, 0x41, 0x53, 0x54,
0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x41, 0x44, 0x54, 0x20, 0x41,
0x72, 0x61, 0x62, 0x69, 0x61, 0x41, 0x52, 0x54, 0x41, 0x52, 0x53, 0x54,
0x57, 0x41, 0x52, 0x54, 0x57, 0x41, 0x52, 0x53, 0x54, 0x41, 0x4d, 0x54,
0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x41, 0x4d, 0x53, 0x54,
0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x41, 0x43, 0x44, 0x54,
0x41, 0x43, 0x57, 0x53, 0x54, 0x41, 0x43, 0x57, 0x44, 0x54, 0x41, 0x45,
0x53, 0x54, 0x41, 0x45, 0x44, 0x54, 0x41, 0x57, 0x53, 0x54, 0x41, 0x57,
0x44, 0x54, 0x41, 0x5a, 0x54, 0x41, 0x5a, 0x53, 0x54, 0x41, 0x5a, 0x4f,
0x54, 0x41, 0x5a, 0x4f, 0x53, 0x54, 0x42, 0x53, 0x54, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x42, 0x44, 0x54, 0x20,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x42, 0x54,
0x54, 0x42, 0x4f, 0x54, 0x42, 0x52, 0x54, 0x42, 0x52, 0x53, 0x54, 0x42,
0x4e, 0x54, 0x43, 0x56, 0x54, 0x43, 0x56, 0x53, 0x54, 0x43, 0x41, 0x53,
0x54, 0x43, 0x68, 0x53, 0x54, 0x43, 0x48, 0x41, 0x53, 0x54, 0x43, 0x48,
0x41, 0x44, 0x54, 0x43, 0x4c, 0x54, 0x43, 0x4c, 0x53, 0x54, 0x43, 0x53,
0x54, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x43, 0x53, 0x54, 0x20, 0x43,
0x68, 0x69, 0x6e, 0x61, 0x20, 0x44, 0x53, 0x54, 0x43, 0x58, 0x54, 0x43,
0x43, 0x54, 0x43, 0x4f, 0x54, 0x43, 0x4f, 0x53, 0x54, 0x43, 0x4b, 0x54,
0x43, 0x4b, 0x54, 0x20, 0x44, 0x53, 0x54, 0x43, 0x53, 0x54, 0x20, 0x43,
0x75, 0x62, 0x61, 0x43, 0x44, 0x54, 0x20, 0x43, 0x75, 0x62, 0x61, 0x44,
0x41, 0x56, 0x54, 0x44, 0x44, 0x55, 0x54, 0x54, 0x4c, 0x54, 0x45, 0x41,
0x53, 0x54, 0x45, 0x41, 0x53, 0x53, 0x54, 0x45, 0x43, 0x54, 0x20, 0x45,
0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x46, 0x45, 0x45, 0x54, 0x46, 0x4b,
0x54, 0x46, 0x4b, 0x53, 0x54, 0x46, 0x4a, 0x54, 0x46, 0x4a, 0x54, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x47, 0x46, 0x54, 0x54, 0x41, 0x41,
0x46, 0x47, 0x41, 0x4c, 0x54, 0x47, 0x41, 0x4d, 0x54, 0x47, 0x45, 0x54,
0x47, 0x45, 0x53, 0x54, 0x47, 0x49, 0x54, 0x47, 0x4e, 0x53, 0x54, 0x47,
0x4e, 0x53, 0x53, 0x54, 0x45, 0x47, 0x54, 0x45, 0x47, 0x53, 0x54, 0x57,
0x47, 0x54, 0x57, 0x47, 0x53, 0x54, 0x47, 0x53, 0x54, 0x20, 0x47, 0x75,
0x61, 0x6d, 0x47, 0x53, 0x54, 0x20, 0x47, 0x75, 0x6c, 0x66, 0x47, 0x59,
0x54, 0x48, 0x53, 0x54, 0x48, 0x44, 0x54, 0x48, 0x4b, 0x54, 0x48, 0x4b,
0x54, 0x20, 0x44, 0x53, 0x54, 0x48, 0x4f, 0x56, 0x54, 0x48, 0x4f, 0x56,
0x53, 0x54, 0x49, 0x53, 0x54, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e,
0x49, 0x4f, 0x54, 0x49, 0x43, 0x54, 0x43, 0x49, 0x54, 0x45, 0x49, 0x54,
0x57, 0x49, 0x54, 0x49, 0x52, 0x53, 0x54, 0x49, 0x52, 0x53, 0x54, 0x20,
0x44, 0x53, 0x54, 0x49, 0x52, 0x4b, 0x54, 0x49, 0x52, 0x4b, 0x53, 0x54,
0x49, 0x53, 0x54, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x49, 0x44,
0x54, 0x4a, 0x53, 0x54, 0x4a, 0x53, 0x54, 0x20, 0x44, 0x53, 0x54, 0x50,
0x45, 0x54, 0x54, 0x50, 0x45, 0x54, 0x53, 0x54, 0x50, 0x45, 0x54, 0x44,
0x54, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432,
0x445, 0x447, 0x425, 0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab,
0x20, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432,
0x445, 0x447, 0x410, 0x43d, 0x4d1, 0x4ab, 0x20, 0x41a, 0x430, 0x437, 0x430, 0x445,
0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x445, 0x447, 0x4b, 0x53, 0x54, 0x20,
0x44, 0x53, 0x54, 0x4b, 0x4f, 0x53, 0x54, 0x4b, 0x52, 0x41, 0x54, 0x4b,
0x52, 0x41, 0x53, 0x54, 0x4b, 0x47, 0x54, 0x53, 0x4c, 0x53, 0x54, 0x4c,
0x49, 0x4e, 0x54, 0x4c, 0x48, 0x53, 0x54, 0x4c, 0x48, 0x53, 0x54, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x4d, 0x53, 0x54, 0x20, 0x4d, 0x61,
0x63, 0x61, 0x6f, 0x4d, 0x53, 0x54, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x6f,
0x20, 0x44, 0x53, 0x54, 0x4d, 0x41, 0x47, 0x54, 0x4d, 0x41, 0x47, 0x53,
0x54, 0x4d, 0x56, 0x54, 0x4d, 0x41, 0x52, 0x54, 0x4d, 0x48, 0x54, 0x4d,
0x55, 0x54, 0x4d, 0x55, 0x53, 0x54, 0x4d, 0x41, 0x57, 0x54, 0x50, 0x53,
0x54, 0x4d, 0x50, 0x44, 0x54, 0x4d, 0x55, 0x4c, 0x41, 0x54, 0x55, 0x4c,
0x41, 0x53, 0x54, 0x4d, 0x53, 0x4b, 0x41c, 0x421, 0x41a, 0x4d, 0x53, 0x44,
0x4d, 0x4d, 0x54, 0x4e, 0x52, 0x54, 0x4e, 0x50, 0x54, 0x4e, 0x43, 0x54,
0x4e, 0x44, 0x54, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x69, 0x61, 0x4e, 0x5a, 0x53, 0x54, 0x4e, 0x5a, 0x44,
0x54, 0x4e, 0x55, 0x54, 0x4e, 0x46, 0x54, 0x4e, 0x46, 0x44, 0x54, 0x46,
0x4e, 0x54, 0x46, 0x4e, 0x53, 0x54, 0x43, 0x68, 0x53, 0x54, 0x20, 0x4e,
0x4d, 0x49, 0x4e, 0x4f, 0x56, 0x54, 0x4e, 0x4f, 0x56, 0x53, 0x54, 0x4f,
0x4d, 0x53, 0x54, 0x4f, 0x4d, 0x53, 0x53, 0x54, 0x50, 0x4b, 0x54, 0x50,
0x4b, 0x54, 0x20, 0x44, 0x53, 0x54, 0x50, 0x57, 0x54, 0x50, 0x47, 0x54,
0x50, 0x59, 0x54, 0x50, 0x59, 0x53, 0x54, 0x45, 0x44, 0x54, 0x20, 0x50,
0x65, 0x72, 0x75, 0x50, 0x53, 0x54, 0x20, 0x50, 0x68, 0x69, 0x6c, 0x69,
0x70, 0x70, 0x69, 0x6e, 0x65, 0x50, 0x53, 0x54, 0x20, 0x50, 0x68, 0x69,
0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x65, 0x20, 0x44, 0x53, 0x54, 0x50,
0x48, 0x4f, 0x54, 0x50, 0x4d, 0x53, 0x54, 0x50, 0x4d, 0x44, 0x54, 0x50,
0x53, 0x54, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x50,
0x4f, 0x4e, 0x54, 0x50, 0x59, 0x54, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61,
0x51, 0x59, 0x5a, 0x54, 0x51, 0x59, 0x5a, 0x54, 0x20, 0x44, 0x53, 0x54,
0x52, 0x45, 0x54, 0x52, 0x4f, 0x54, 0x54, 0x53, 0x41, 0x4b, 0x54, 0x53,
0x41, 0x4b, 0x53, 0x54, 0x53, 0x41, 0x4d, 0x54, 0x53, 0x41, 0x4d, 0x53,
0x54, 0x53, 0x53, 0x54, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x44,
0x53, 0x54, 0x53, 0x43, 0x54, 0x53, 0x42, 0x54, 0x53, 0x52, 0x54, 0x53,
0x59, 0x4f, 0x54, 0x54, 0x41, 0x48, 0x54, 0x54, 0x57, 0x54, 0x54, 0x57,
0x54, 0x20, 0x44, 0x53, 0x54, 0x54, 0x4a, 0x54, 0x54, 0x4b, 0x54, 0x54,
0x4f, 0x54, 0x54, 0x4f, 0x53, 0x54, 0x43, 0x48, 0x55, 0x54, 0x54, 0x4d,
0x54, 0x20, 0x44, 0x53, 0x54, 0x54, 0x56, 0x54, 0x55, 0x59, 0x54, 0x55,
0x59, 0x53, 0x54, 0x55, 0x5a, 0x54, 0x55, 0x5a, 0x54, 0x20, 0x44, 0x53,
0x54, 0x56, 0x55, 0x54, 0x56, 0x55, 0x54, 0x20, 0x44, 0x53, 0x54, 0x56,
0x6e, 0x7a, 0x6c, 0x54, 0x56, 0x4c, 0x41, 0x54, 0x56, 0x4c, 0x41, 0x53,
0x54, 0x56, 0x4f, 0x4c, 0x54, 0x56, 0x4f, 0x4c, 0x53, 0x54, 0x56, 0x4f,
0x53, 0x54, 0x57, 0x41, 0x4b, 0x54, 0x57, 0x46, 0x54, 0x59, 0x41, 0x4b,
0x54, 0x59, 0x41, 0x4b, 0x53, 0x54, 0x59, 0x45, 0x4b, 0x54, 0x59, 0x45,
0x4b, 0x53, 0x54, 0x4d, 0x45, 0x5a, 0x4d, 0x45, 0x53, 0x5a, 0x4f, 0x45,
0x5a, 0x4f, 0x45, 0x53, 0x5a, 0x57, 0x45, 0x5a, 0x57, 0x45, 0x53, 0x5a,
0x53, 0x45, 0x10c, 0x53, 0x45, 0x4c, 0x10c, 0x2205, 0x2205, 0x2205, 0x41, 0x43,
0x57, 0x54, 0x41, 0x45, 0x54, 0x4c, 0x48, 0x54, 0x4c, 0x48, 0x44, 0x54,
0x4e, 0x5a, 0x54, 0x47, 0x75, 0x6c, 0x66, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x57, 0x49, 0x54, 0x41, 0x57, 0x49, 0x42, 0x49, 0x52, 0x44, 0x54, 0x50,
0x4d, 0x54, 0x56, 0x45, 0x54, 0x48, 0x4b, 0x53, 0x54, 0x43, 0x48, 0x41,
0x54, 0x48, 0x41, 0x4b, 0x48, 0x4e, 0x41, 0x4b, 0x48, 0x45, 0x41, 0x4b,
0x48, 0x43, 0x48, 0x4e, 0x43, 0x48, 0x45, 0x43, 0x48, 0x4e, 0x45, 0x48,
0x45, 0x45, 0x48, 0x52, 0x48, 0x4e, 0x52, 0x48, 0x45, 0x52, 0x48, 0x50,
0x48, 0x4e, 0x50, 0x48, 0x45, 0x50, 0x48, 0x43, 0x55, 0x48, 0x4e, 0x43,
0x55, 0x48, 0x45, 0x43, 0x55, 0x45, 0x45, 0x44, 0x54, 0x57, 0x45, 0x44,
0x54, 0x48, 0x45, 0x47, 0x48, 0x4e, 0x45, 0x47, 0x48, 0x45, 0x45, 0x47,
0x48, 0x4f, 0x47, 0x48, 0x4e, 0x4f, 0x47, 0x48, 0x45, 0x4f, 0x47, 0x48,
0x48, 0x41, 0x48, 0x4e, 0x48, 0x41, 0x48, 0x45, 0x48, 0x41, 0x48, 0x50,
0x4d, 0x58, 0x48, 0x4e, 0x50, 0x4d, 0x58, 0x48, 0x45, 0x50, 0x4d, 0x58,
0x48, 0x54, 0x4e, 0x48, 0x4e, 0x54, 0x4e, 0x48, 0x45, 0x54, 0x4e, 0x48,
0x41, 0x43, 0x48, 0x41, 0x45, 0x48, 0x41, 0x52, 0x48, 0x41, 0x50, 0x43,
0x54, 0x20, 0x28, 0x4b, 0x75, 0x62, 0x61, 0x29, 0x43, 0x53, 0x54, 0x20,
0x28, 0x4b, 0x75, 0x62, 0x61, 0x29, 0x43, 0x44, 0x54, 0x20, 0x28, 0x4b,
0x75, 0x62, 0x61, 0x29, 0x45, 0x47, 0x44, 0x54, 0x41, 0x41, 0x43, 0x41,
0x43, 0x41, 0x43, 0x41, 0x53, 0x41, 0x43, 0x4a, 0x44, 0x54, 0x627, 0x644,
0x645, 0x627, 0x62a, 0x649, 0x627, 0x644, 0x645, 0x627, 0x62a, 0x649, 0x20, 0x642,
0x627, 0x644, 0x627, 0x633, 0x649, 0x627, 0x642, 0x62a, 0x627, 0x6cb, 0x627, 0x642,
0x62a, 0x627, 0x6cb, 0x20, 0x642, 0x627, 0x644, 0x627, 0x633, 0x649, 0x627, 0x642,
0x62a, 0x648, 0x628, 0x6d5, 0x627, 0x642, 0x62a, 0x648, 0x628, 0x6d5, 0x20, 0x642,
0x627, 0x644, 0x627, 0x633, 0x649, 0x642, 0x627, 0x632, 0x627, 0x642, 0x20, 0x6d5,
0x644, 0x649, 0x634, 0x649, 0x639, 0x649, 0x634, 0x20, 0x642, 0x627, 0x632, 0x627,
0x642, 0x20, 0x6d5, 0x644, 0x649, 0x628, 0x627, 0x62a, 0x649, 0x633, 0x20, 0x642,
0x627, 0x632, 0x627, 0x642, 0x20, 0x6d5, 0x644, 0x649, 0x642, 0x649, 0x631, 0x639,
0x649, 0x632, 0x633, 0x62a, 0x627, 0x646, 0x642, 0x649, 0x632, 0x649, 0x644, 0x648,
0x631, 0x62f, 0x627, 0x642, 0x649, 0x632, 0x649, 0x644, 0x648, 0x631, 0x62f, 0x627,
0x20, 0x642, 0x627, 0x644, 0x627, 0x633, 0x649, 0x53, 0x53, 0x41, 0x4b, 0x53,
0x48, 0x41, 0x4b, 0x53, 0x4e, 0x53, 0x53, 0x4e, 0x53, 0x48, 0x4e, 0x53,
0x53, 0x52, 0x53, 0x48, 0x52, 0x53, 0xc7, 0x53, 0x53, 0xc7, 0x53, 0x48,
0xc7, 0x53, 0x50, 0x53, 0x53, 0x50, 0x53, 0x48, 0x50, 0x53, 0x53, 0x4b,
0x53, 0x48, 0x4b, 0x53, 0x47, 0x52, 0x53, 0x53, 0x47, 0x52, 0x53, 0x48,
0x47, 0x52, 0x53, 0x47, 0x52, 0x4f, 0x53, 0x53, 0x47, 0x52, 0x4f, 0x53,
0x48, 0x47, 0x52, 0x4f, 0x53, 0x48, 0x41, 0x4c, 0x53, 0x53, 0x48, 0x41,
0x53, 0x48, 0x48, 0x41, 0x53, 0x4e, 0x46, 0x53, 0x53, 0x4e, 0x46, 0x53,
0x48, 0x4e, 0x46, 0x50, 0x45, 0x53, 0x54, 0x4f, 0x41, 0x4b, 0x4f, 0x49,
0x41, 0x4b, 0x4f, 0x4c, 0x41, 0x4b, 0x4f, 0x49, 0x54, 0x4f, 0x4f, 0x4f,
0x49, 0x4f, 0x4f, 0x4c, 0x4f, 0x4f, 0x4d, 0x50, 0x4f, 0x49, 0x4d, 0x50,
0x4f, 0x4c, 0x4d, 0x50, 0x4f, 0x50, 0x4f, 0x49, 0x50, 0x4f, 0x4c, 0x50,
0x4f, 0x4d, 0x47, 0x43, 0x75, 0x54, 0x43, 0x75, 0x53, 0x54, 0x43, 0x75,
0x44, 0x54, 0x4a, 0x43, 0x41, 0x53, 0x4a, 0x43, 0x41, 0x4a, 0x43, 0x41,
0x4b, 0x52, 0x4a, 0x54, 0x41, 0x53, 0x4a, 0x54, 0x41, 0x4a, 0x54, 0x41,
0x4b, 0x52, 0x4a, 0x59, 0x41, 0x53, 0x4a, 0x59, 0x41, 0x4a, 0x48, 0x59,
0x41, 0x53, 0x4a, 0x41, 0x44, 0x4a, 0x41, 0x44, 0x4b, 0x52, 0x4a, 0x59,
0x54, 0x53, 0x4a, 0x59, 0x54, 0x4a, 0x59, 0x54, 0x4b, 0x52, 0x4a, 0x43,
0x4b, 0x52, 0x4a, 0x54, 0x4b, 0x52, 0x4a, 0x53, 0x4b, 0x52, 0x4a, 0x51,
0x47, 0x47, 0x6b
};

static inline constexpr char16_t longMetaZoneNameTable[] = {
0x41, 0x63, 0x72, 0x65, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x63, 0x72,
0x65, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x41, 0x63, 0x72, 0x65, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x66, 0x67, 0x68, 0x61,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63,
0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x20, 0x41,
0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53, 0x6f,
0x75, 0x74, 0x68, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x57, 0x65, 0x73, 0x74, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x20, 0x41, 0x66, 0x72,
0x69, 0x63, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x20, 0x41, 0x66,
0x72, 0x69, 0x63, 0x61, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41,
0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67,
0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x6c, 0x6d, 0x61, 0x74,
0x79, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x20, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c,
0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x6f, 0x75,
0x6e, 0x74, 0x61, 0x69, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x6f,
0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x6f, 0x75, 0x6e,
0x74, 0x61, 0x69, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68,
0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69,
0x63, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69,
0x63, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x44,
0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x6e, 0x61, 0x64, 0x79,
0x72, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20,
0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x41, 0x71, 0x74, 0x61, 0x75, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41,
0x71, 0x74, 0x61, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x71, 0x74, 0x61, 0x75, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41,
0x71, 0x74, 0x6f, 0x62, 0x65, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x71,
0x74, 0x6f, 0x62, 0x65, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65,
0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x72, 0x61,
0x62, 0x69, 0x61, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68,
0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x57, 0x65, 0x73,
0x74, 0x65, 0x72, 0x6e, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x63, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x63, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x63, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x6e, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e,
0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68,
0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x6e, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x57, 0x65, 0x73, 0x74,
0x65, 0x72, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x6e, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20,
0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c,
0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x73, 0x74,
0x65, 0x72, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65,
0x72, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x6e, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x44,
0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x57,
0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x57, 0x65,
0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72,
0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61,
0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61,
0x69, 0x6a, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61,
0x69, 0x6a, 0x61, 0x6e, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x73, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x73, 0x68, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69,
0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c,
0x69, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61,
0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43,
0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65,
0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x43, 0x61, 0x73, 0x65, 0x79, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x68,
0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x43, 0x68, 0x69, 0x6c, 0x65, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x68, 0x69, 0x6c, 0x65,
0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x43, 0x68, 0x69, 0x6e, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x68,
0x69, 0x6e, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x20, 0x44,
0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x6f, 0x63,
0x6f, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x6f,
0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x75, 0x62, 0x61, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x43, 0x75, 0x62, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x75, 0x62, 0x61,
0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x44, 0x61, 0x76, 0x69, 0x73, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x20, 0x64, 0x2019, 0x55, 0x72, 0x76,
0x69, 0x6c, 0x6c, 0x65, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x54, 0x69, 0x6d,
0x6f, 0x72, 0x2d, 0x4c, 0x65, 0x73, 0x74, 0x65, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x65,
0x72, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x45, 0x61,
0x73, 0x74, 0x65, 0x72, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x45,
0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x61, 0x6e,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x61, 0x6e, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72,
0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x61, 0x6e, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x45, 0x61, 0x73,
0x74, 0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x61,
0x6e, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x46, 0x75, 0x72, 0x74, 0x68, 0x65, 0x72, 0x2d, 0x65, 0x61, 0x73,
0x74, 0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x61,
0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72,
0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x61, 0x6e, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x57, 0x65, 0x73,
0x74, 0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x61,
0x6e, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x46, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x46, 0x69, 0x6a, 0x69,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x46,
0x69, 0x6a, 0x69, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x46, 0x72, 0x65, 0x6e, 0x63, 0x68, 0x20, 0x47, 0x75,
0x69, 0x61, 0x6e, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x46, 0x72, 0x65,
0x6e, 0x63, 0x68, 0x20, 0x53, 0x6f, 0x75, 0x74, 0x68, 0x65, 0x72, 0x6e,
0x20, 0x26, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x63, 0x74, 0x69, 0x63,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67,
0x6f, 0x73, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x47, 0x61, 0x6d, 0x62, 0x69,
0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x47, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x47, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x47,
0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x77, 0x69, 0x63, 0x68, 0x20, 0x4d, 0x65, 0x61, 0x6e, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x45, 0x61, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x45,
0x61, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x57, 0x65, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74,
0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x57, 0x65, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x47, 0x75, 0x61, 0x6d, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x47, 0x75, 0x6c, 0x66,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x69, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65,
0x75, 0x74, 0x69, 0x61, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67,
0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6e, 0x67, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x48, 0x6f, 0x6e,
0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67,
0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x48, 0x6f, 0x76, 0x64,
0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x49, 0x6e, 0x64, 0x69, 0x61,
0x6e, 0x20, 0x4f, 0x63, 0x65, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x49, 0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x57, 0x65,
0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x73, 0x69, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x49, 0x72, 0x61, 0x6e,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x49,
0x72, 0x61, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x49, 0x73, 0x72,
0x61, 0x65, 0x6c, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x20, 0x44,
0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x4a, 0x61, 0x70, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4a, 0x61,
0x70, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x20, 0x44,
0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x6b, 0x61, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x6b, 0x61, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x6b, 0x61, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4b, 0x61,
0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x45, 0x61, 0x73, 0x74, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x57, 0x65, 0x73,
0x74, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69,
0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4b, 0x6f, 0x73, 0x72,
0x61, 0x65, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4b,
0x79, 0x72, 0x67, 0x79, 0x7a, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x4c, 0x69, 0x6e, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x61,
0x63, 0x61, 0x6f, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x61, 0x6c, 0x61, 0x79,
0x73, 0x69, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x61, 0x6c, 0x64,
0x69, 0x76, 0x65, 0x73, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x61, 0x72,
0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d,
0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x74, 0x69, 0x75, 0x73, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e,
0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63,
0x69, 0x66, 0x69, 0x63, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61,
0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x44, 0x61,
0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x55,
0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74,
0x61, 0x72, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61,
0x74, 0x61, 0x72, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x6f,
0x73, 0x63, 0x6f, 0x77, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69,
0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x65, 0x77,
0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x65,
0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x65, 0x77,
0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e,
0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x4e, 0x69, 0x75, 0x65, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x65, 0x72,
0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x6e, 0x61, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4f,
0x6d, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x50, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20,
0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x50,
0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x50, 0x65, 0x72, 0x75, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x50, 0x65, 0x72, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x50, 0x65, 0x72,
0x75, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x65, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69,
0x6e, 0x65, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69,
0x6e, 0x65, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53, 0x74,
0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d,
0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53,
0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x44, 0x61, 0x79,
0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x50, 0x69,
0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x50,
0x6f, 0x68, 0x6e, 0x70, 0x65, 0x69, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e,
0x6f, 0x72, 0x74, 0x68, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72,
0x64, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64,
0x61, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e,
0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x63, 0x61, 0x6e, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x44,
0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53, 0x6f,
0x75, 0x74, 0x68, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x54, 0x61, 0x69, 0x77, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x54, 0x61, 0x69, 0x77, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x54, 0x61, 0x69,
0x77, 0x61, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x54, 0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x54, 0x6f, 0x6b, 0x65,
0x6c, 0x61, 0x75, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x54, 0x6f, 0x6e, 0x67,
0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x54, 0x75, 0x72,
0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x54,
0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x54,
0x75, 0x76, 0x61, 0x6c, 0x75, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x79, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x79, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x79, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x56, 0x65, 0x6e, 0x65, 0x7a,
0x75, 0x65, 0x6c, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x56,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x57, 0x61,
0x6b, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x26, 0x20, 0x46,
0x75, 0x74, 0x75, 0x6e, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x59, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x59, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x59,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x410, 0x43a,
0x440, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43a, 0x440, 0x438,
0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43a, 0x440, 0x438, 0x20, 0x430, 0x525,
0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x410, 0x444, 0x433, 0x430, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x410, 0x433,
0x4d9, 0x4ad, 0x430, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x444, 0x440, 0x438,
0x43a, 0x430, 0x41c, 0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430, 0x440, 0x430, 0x442,
0x4d9, 0x438, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x410, 0x43b, 0x430,
0x434, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430,
0x41c, 0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9,
0x438, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x41c, 0x440, 0x430, 0x4ad,
0x430, 0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x444,
0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440,
0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20,
0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b,
0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43b,
0x438, 0x430, 0x441, 0x43a, 0x430, 0x410, 0x43b, 0x438, 0x430, 0x441, 0x43a, 0x430,
0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43b, 0x438, 0x430, 0x441, 0x43a,
0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43b, 0x43c, 0x430, 0x2d, 0x410, 0x442,
0x430, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43b, 0x43c, 0x430, 0x2d,
0x410, 0x442, 0x430, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43b, 0x43c, 0x430,
0x2d, 0x410, 0x442, 0x430, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9,
0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43c, 0x430, 0x437, 0x43e,
0x43d, 0x43a, 0x430, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a, 0x430, 0x2c,
0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a,
0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x433, 0x4d9, 0x4ad, 0x430, 0x43d, 0x442,
0x4d9, 0x438, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x410, 0x433,
0x4d9, 0x4ad, 0x430, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x433, 0x4d9,
0x4ad, 0x430, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438,
0x43a, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430, 0x433, 0x44b, 0x43b,
0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438,
0x43a, 0x430, 0x41c, 0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430, 0x440, 0x430, 0x442,
0x4d9, 0x438, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20,
0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430, 0x440,
0x430, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430,
0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x448, 0x44c, 0x445, 0x430, 0x442, 0x4d9, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x20, 0x28, 0x410, 0x4a9, 0x430, 0x434, 0x430,
0x442, 0x4d9, 0x438, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x29,
0x410, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430,
0x448, 0x44c, 0x445, 0x430, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x20, 0x28, 0x410, 0x4a9, 0x430, 0x434, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x29, 0x410, 0x525, 0x445, 0x44b, 0x43d,
0x442, 0x4d9, 0x438, 0x20, 0x430, 0x448, 0x44c, 0x445, 0x430, 0x442, 0x4d9, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x20, 0x28, 0x410, 0x4a9, 0x430, 0x434, 0x430,
0x442, 0x4d9, 0x438, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x29,
0x410, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x4ad, 0x44b, 0x43d, 0x447, 0x442, 0x4d9,
0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43e, 0x43a, 0x435, 0x430,
0x43d, 0x4ad, 0x44b, 0x43d, 0x447, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x410, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x4ad, 0x44b, 0x43d, 0x447, 0x442,
0x4d9, 0x438, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x20, 0x410,
0x43d, 0x430, 0x434, 0x44b, 0x440, 0x20, 0x430, 0x4bf, 0x44b, 0x410, 0x43d, 0x430,
0x434, 0x44b, 0x440, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43d, 0x430, 0x434,
0x44b, 0x440, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43f, 0x438, 0x430, 0x410, 0x43f, 0x438,
0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43f, 0x438, 0x430, 0x2c,
0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x410, 0x43a, 0x442, 0x430, 0x443, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x410, 0x43a, 0x442, 0x430, 0x443, 0x2c, 0x20, 0x430, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x410, 0x43a, 0x442, 0x430, 0x443, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d,
0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43a, 0x442,
0x43e, 0x431, 0x435, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43a, 0x442,
0x43e, 0x431, 0x435, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43a, 0x442, 0x43e,
0x431, 0x435, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x421, 0x430, 0x443, 0x434, 0x442, 0x4d9, 0x438,
0x20, 0x410, 0x440, 0x430, 0x431, 0x441, 0x4ad, 0x430, 0x43d, 0x421, 0x430, 0x443,
0x434, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x440, 0x430, 0x431, 0x441, 0x4ad, 0x430,
0x43d, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x421, 0x430, 0x443, 0x434, 0x442,
0x4d9, 0x438, 0x20, 0x410, 0x440, 0x430, 0x431, 0x441, 0x4ad, 0x430, 0x43d, 0x2c,
0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430,
0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x430,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430,
0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430,
0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442,
0x438, 0x43d, 0x430, 0x41c, 0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430, 0x440,
0x430, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438,
0x43d, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430, 0x4ad,
0x430, 0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x440,
0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445,
0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x415,
0x440, 0x43c, 0x430, 0x43d, 0x442, 0x4d9, 0x44b, 0x43b, 0x430, 0x415, 0x440, 0x43c,
0x430, 0x43d, 0x442, 0x4d9, 0x44b, 0x43b, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x415, 0x440, 0x43c, 0x430, 0x43d, 0x442, 0x4d9, 0x44b, 0x43b, 0x430, 0x2c,
0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x430,
0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x442, 0x43b,
0x430, 0x43d, 0x442, 0x438, 0x43a, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x430, 0x442,
0x4d9, 0x438, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x433, 0x4d9, 0x4ad, 0x430, 0x43d, 0x442,
0x4d9, 0x438, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x430,
0x410, 0x433, 0x4d9, 0x4ad, 0x430, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x410, 0x433, 0x4d9, 0x4ad, 0x430, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x410,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x525,
0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x410, 0x433, 0x4d9, 0x4ad, 0x430, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x430, 0x2c, 0x20, 0x43c, 0x440, 0x430,
0x4ad, 0x430, 0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x433, 0x4d9, 0x4ad, 0x430, 0x43d, 0x442, 0x4d9,
0x438, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x430, 0x2c,
0x20, 0x43c, 0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442,
0x4d9, 0x438, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x433, 0x4d9, 0x4ad, 0x430,
0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x430, 0x2c, 0x20, 0x43c, 0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430,
0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442,
0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430, 0x433,
0x44b, 0x43b, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x430, 0x41c, 0x440, 0x430, 0x433, 0x44b, 0x43b,
0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440,
0x430, 0x433, 0x44b, 0x43b, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x410,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x525,
0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x41c, 0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9,
0x438, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x430, 0x41c,
0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438,
0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x430, 0x2c, 0x20,
0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430,
0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9,
0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x437, 0x435, 0x440, 0x431,
0x430, 0x438, 0x45f, 0x44c, 0x430, 0x43d, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430,
0x438, 0x45f, 0x44c, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410,
0x437, 0x435, 0x440, 0x431, 0x430, 0x438, 0x45f, 0x44c, 0x430, 0x43d, 0x2c, 0x20,
0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x410, 0x437, 0x43e, 0x440, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x434,
0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x410, 0x437, 0x43e, 0x440, 0x442, 0x4d9,
0x438, 0x20, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x2c, 0x20,
0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x437, 0x43e, 0x440, 0x442, 0x4d9, 0x438, 0x20,
0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x2c, 0x20, 0x430, 0x525,
0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x411, 0x430, 0x43d,
0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x2c, 0x20, 0x430,
0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x411, 0x443, 0x442, 0x430, 0x43d, 0x411, 0x43e, 0x43b, 0x438, 0x432, 0x438,
0x430, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x430, 0x411, 0x440, 0x430,
0x437, 0x438, 0x43b, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x411,
0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445,
0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x411,
0x440, 0x443, 0x43d, 0x435, 0x438, 0x2d, 0x414, 0x430, 0x440, 0x443, 0x441, 0x441,
0x430, 0x43b, 0x430, 0x43c, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440,
0x434, 0x435, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435,
0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412,
0x435, 0x440, 0x434, 0x435, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442,
0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41a, 0x435, 0x438, 0x441,
0x438, 0x427, 0x430, 0x43c, 0x43e, 0x440, 0x440, 0x43e, 0x427, 0x430, 0x442, 0x435,
0x43c, 0x427, 0x430, 0x442, 0x435, 0x43c, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x427, 0x430, 0x442, 0x435, 0x43c, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d,
0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x427, 0x438, 0x43b,
0x438, 0x427, 0x438, 0x43b, 0x438, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x427,
0x438, 0x43b, 0x438, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9,
0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41a, 0x438, 0x442, 0x430, 0x438,
0x41a, 0x438, 0x442, 0x430, 0x438, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41a,
0x438, 0x442, 0x430, 0x438, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442,
0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x430, 0x434, 0x2d, 0x445,
0x430, 0x20, 0x49a, 0x44c, 0x44b, 0x440, 0x441, 0x430, 0x41a, 0x43e, 0x43a, 0x43e,
0x441, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9,
0x430, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x430, 0x41a, 0x43e, 0x43b,
0x443, 0x43c, 0x431, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41a,
0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445,
0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41a,
0x443, 0x43a, 0x20, 0x438, 0x434, 0x433, 0x44c, 0x44b, 0x43b, 0x431, 0x436, 0x44c,
0x430, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x41a, 0x443, 0x43a, 0x20, 0x438, 0x434,
0x433, 0x44c, 0x44b, 0x43b, 0x431, 0x436, 0x44c, 0x430, 0x445, 0x430, 0x49b, 0x4d9,
0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41a, 0x443, 0x43a, 0x20, 0x438,
0x434, 0x433, 0x44c, 0x44b, 0x43b, 0x431, 0x436, 0x44c, 0x430, 0x445, 0x430, 0x49b,
0x4d9, 0x430, 0x2c, 0x20, 0x43f, 0x43e, 0x43b, 0x443, 0x430, 0x525, 0x445, 0x44b,
0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41a, 0x443,
0x431, 0x430, 0x41a, 0x443, 0x431, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x41a, 0x443, 0x431, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442,
0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x414, 0x435, 0x438, 0x432,
0x438, 0x441, 0x414, 0x438, 0x443, 0x43c, 0x43e, 0x43d, 0x2d, 0x434, 0x2019, 0x418,
0x443, 0x440, 0x432, 0x438, 0x43b, 0x41c, 0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430,
0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x422, 0x438, 0x43c, 0x43e, 0x440, 0x430,
0x434, 0x2d, 0x445, 0x430, 0x20, 0x41f, 0x430, 0x441, 0x445, 0x438, 0x430, 0x434,
0x2d, 0x445, 0x430, 0x20, 0x41f, 0x430, 0x441, 0x445, 0x438, 0x2c, 0x20, 0x430,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x20, 0x41f, 0x430, 0x441,
0x445, 0x438, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x415, 0x43a, 0x432, 0x430, 0x434, 0x43e,
0x440, 0x410, 0x433, 0x4d9, 0x4ad, 0x430, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x415,
0x432, 0x440, 0x43e, 0x43f, 0x430, 0x410, 0x433, 0x4d9, 0x4ad, 0x430, 0x43d, 0x442,
0x4d9, 0x438, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x2c, 0x20, 0x430,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x410, 0x433, 0x4d9, 0x4ad, 0x430, 0x43d, 0x442, 0x4d9, 0x438,
0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445,
0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c,
0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20,
0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x41c, 0x440, 0x430, 0x433, 0x44b, 0x43b,
0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f,
0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430, 0x433, 0x44b,
0x43b, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x415, 0x432, 0x440, 0x43e,
0x43f, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x43e, 0x441, 0x43a, 0x43e, 0x432,
0x430, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440,
0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20,
0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x41c, 0x440, 0x430, 0x4ad, 0x430, 0x448,
0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x415, 0x432, 0x440, 0x43e,
0x43f, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430, 0x4ad,
0x430, 0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x415, 0x432,
0x440, 0x43e, 0x43f, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442,
0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x424, 0x43e, 0x43b, 0x43a,
0x43b, 0x435, 0x43d, 0x434, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x434, 0x2d, 0x445,
0x430, 0x49b, 0x4d9, 0x430, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434,
0x442, 0x4d9, 0x438, 0x20, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430,
0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435,
0x43d, 0x434, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b,
0x4d9, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x424, 0x438, 0x45f, 0x438, 0x424, 0x438,
0x45f, 0x438, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x424, 0x438, 0x45f, 0x438,
0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x44b, 0x437, 0x442,
0x4d9, 0x438, 0x20, 0x413, 0x432, 0x438, 0x430, 0x43d, 0x430, 0x424, 0x440, 0x430,
0x43d, 0x446, 0x44b, 0x437, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x43b, 0x430, 0x434,
0x430, 0x442, 0x4d9, 0x435, 0x438, 0x20, 0x410, 0x43d, 0x442, 0x430, 0x440, 0x43a,
0x442, 0x438, 0x43a, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x440, 0x4b5, 0x430, 0x43a,
0x44b, 0x440, 0x430, 0x434, 0x433, 0x44c, 0x44b, 0x43b, 0x49b, 0x4d9, 0x430, 0x413,
0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441, 0x442, 0x4d9, 0x438, 0x20,
0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x413, 0x430, 0x43c, 0x431,
0x435, 0x49a, 0x44b, 0x440, 0x4ad, 0x442, 0x4d9, 0x44b, 0x43b, 0x430, 0x49a, 0x44b,
0x440, 0x4ad, 0x442, 0x4d9, 0x44b, 0x43b, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x49a, 0x44b, 0x440, 0x4ad, 0x442, 0x4d9, 0x44b, 0x43b, 0x430, 0x2c, 0x20,
0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x20, 0x413,
0x438, 0x43b, 0x431, 0x435, 0x440, 0x442, 0x430, 0x410, 0x433, 0x440, 0x438, 0x43d,
0x432, 0x438, 0x447, 0x20, 0x418, 0x431, 0x436, 0x44c, 0x430, 0x440, 0x430, 0x442,
0x4d9, 0x443, 0x20, 0x410, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430, 0x433,
0x44b, 0x43b, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x413, 0x440, 0x435,
0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x430, 0x41c, 0x440, 0x430, 0x433, 0x44b,
0x43b, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d,
0x43b, 0x430, 0x43d, 0x434, 0x438, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x438,
0x430, 0x41c, 0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430, 0x440, 0x430, 0x442, 0x4d9,
0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x430,
0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430,
0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430,
0x43d, 0x434, 0x438, 0x430, 0x41c, 0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430,
0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430,
0x43d, 0x434, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440,
0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20,
0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x430, 0x2c, 0x20,
0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x413, 0x443, 0x430, 0x43c, 0x410, 0x45f, 0x44c, 0x430, 0x43c, 0x442,
0x4d9, 0x20, 0x430, 0x4e1, 0x44b, 0x431, 0x436, 0x44c, 0x430, 0x445, 0x430, 0x43b,
0x430, 0x413, 0x430, 0x438, 0x430, 0x43d, 0x430, 0x4b2, 0x430, 0x432, 0x430, 0x438,
0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x4b2, 0x430, 0x432, 0x430, 0x438, 0x2d, 0x430, 0x43b, 0x435,
0x443, 0x442, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x4b2, 0x430,
0x432, 0x430, 0x438, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x442, 0x4d9, 0x438,
0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x4b2, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x4b2, 0x43e,
0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x4b2,
0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b,
0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x425, 0x43e,
0x432, 0x434, 0x425, 0x43e, 0x432, 0x434, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x425, 0x43e, 0x432, 0x434, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442,
0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x418, 0x43d, 0x434, 0x438,
0x430, 0x418, 0x43d, 0x434, 0x438, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x43e,
0x43a, 0x435, 0x430, 0x43d, 0x418, 0x43d, 0x434, 0x43e, 0x43a, 0x438, 0x442, 0x430,
0x438, 0x410, 0x433, 0x4d9, 0x4ad, 0x430, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x418,
0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x430, 0x41c, 0x440, 0x430, 0x433,
0x44b, 0x43b, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x418, 0x43d, 0x434,
0x43e, 0x43d, 0x435, 0x437, 0x438, 0x430, 0x41c, 0x440, 0x430, 0x4ad, 0x430, 0x448,
0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x418, 0x43d, 0x434, 0x43e,
0x43d, 0x435, 0x437, 0x438, 0x430, 0x418, 0x440, 0x430, 0x43d, 0x418, 0x440, 0x430,
0x43d, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x418, 0x440, 0x430, 0x43d, 0x2c,
0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x418, 0x440,
0x43a, 0x443, 0x442, 0x441, 0x43a, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x418,
0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b,
0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x418, 0x437,
0x440, 0x430, 0x438, 0x43b, 0x44c, 0x418, 0x437, 0x440, 0x430, 0x438, 0x43b, 0x44c,
0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x418, 0x437, 0x440, 0x430, 0x438, 0x43b,
0x44c, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x418, 0x430, 0x43f, 0x43e, 0x43d, 0x438, 0x430,
0x418, 0x430, 0x43f, 0x43e, 0x43d, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x418, 0x430, 0x43f, 0x43e, 0x43d, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x525,
0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x41f, 0x435, 0x442, 0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441,
0x43a, 0x2d, 0x41a, 0x430, 0x43c, 0x447, 0x430, 0x442, 0x43a, 0x430, 0x442, 0x4d9,
0x438, 0x41f, 0x435, 0x442, 0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432,
0x441, 0x43a, 0x2d, 0x41a, 0x430, 0x43c, 0x447, 0x430, 0x442, 0x43a, 0x430, 0x442,
0x4d9, 0x438, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41f, 0x435, 0x442, 0x440,
0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x2d, 0x41a, 0x430,
0x43c, 0x447, 0x430, 0x442, 0x43a, 0x430, 0x442, 0x4d9, 0x438, 0x2c, 0x20, 0x430,
0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x41c, 0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430, 0x440, 0x430, 0x442, 0x4d9,
0x438, 0x20, 0x49e, 0x430, 0x437, 0x430, 0x445, 0x441, 0x4ad, 0x430, 0x43d, 0x41c,
0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438,
0x20, 0x49e, 0x430, 0x437, 0x430, 0x445, 0x441, 0x4ad, 0x430, 0x43d, 0x41a, 0x43e,
0x440, 0x435, 0x438, 0x430, 0x41a, 0x43e, 0x440, 0x435, 0x438, 0x430, 0x2c, 0x20,
0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x41a, 0x43e, 0x440, 0x435, 0x438, 0x430, 0x2c, 0x20,
0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x41a, 0x43e, 0x441, 0x440, 0x430, 0x435, 0x41a, 0x440, 0x430, 0x441,
0x43d, 0x43e, 0x438, 0x430, 0x440, 0x441, 0x43a, 0x41a, 0x440, 0x430, 0x441, 0x43d,
0x43e, 0x438, 0x430, 0x440, 0x441, 0x43a, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x438, 0x430, 0x440, 0x441, 0x43a, 0x2c,
0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x41a, 0x438, 0x440, 0x433, 0x438, 0x437, 0x438, 0x430, 0x428,
0x440, 0x438, 0x2d, 0x41b, 0x430, 0x43d, 0x43a, 0x430, 0x430, 0x434, 0x2d, 0x445,
0x430, 0x49b, 0x4d9, 0x430, 0x20, 0x41b, 0x430, 0x438, 0x43d, 0x41b, 0x43e, 0x440,
0x434, 0x2d, 0x4b2, 0x430, 0x443, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x4b2, 0x430,
0x443, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41b, 0x43e, 0x440, 0x434, 0x2d,
0x4b2, 0x430, 0x443, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9,
0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x430, 0x43a, 0x430, 0x43e,
0x41c, 0x430, 0x43a, 0x430, 0x43e, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c,
0x430, 0x43a, 0x430, 0x43e, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442,
0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x430, 0x433, 0x430,
0x434, 0x430, 0x43d, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x2c, 0x20,
0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x2c,
0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x41c, 0x430, 0x43b, 0x430, 0x438, 0x437, 0x438, 0x430, 0x41c,
0x430, 0x43b, 0x44c, 0x434, 0x438, 0x432, 0x41c, 0x430, 0x440, 0x43a, 0x438, 0x437,
0x442, 0x4d9, 0x438, 0x20, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430,
0x41c, 0x430, 0x440, 0x448, 0x430, 0x43b, 0x43b, 0x442, 0x4d9, 0x438, 0x20, 0x410,
0x434, 0x433, 0x44c, 0x44b, 0x43b, 0x431, 0x436, 0x44c, 0x430, 0x445, 0x430, 0x49b,
0x4d9, 0x430, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438, 0x41c, 0x430, 0x432,
0x440, 0x438, 0x43a, 0x438, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x430,
0x432, 0x440, 0x438, 0x43a, 0x438, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d,
0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x43e, 0x443,
0x441, 0x43e, 0x43d, 0x410, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x4ad, 0x44b, 0x43d,
0x447, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x43c, 0x435, 0x43a, 0x441, 0x438, 0x43a,
0x430, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43e, 0x43a,
0x435, 0x430, 0x43d, 0x4ad, 0x44b, 0x43d, 0x447, 0x442, 0x4d9, 0x438, 0x20, 0x430,
0x43c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430, 0x442, 0x4d9, 0x20, 0x430, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x410, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x4ad, 0x44b, 0x43d, 0x447,
0x442, 0x4d9, 0x438, 0x20, 0x430, 0x43c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430,
0x442, 0x4d9, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430,
0x442, 0x43e, 0x440, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e,
0x440, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x423, 0x43b, 0x430, 0x43d, 0x2d,
0x411, 0x430, 0x442, 0x43e, 0x440, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d,
0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x43e, 0x441,
0x43a, 0x432, 0x430, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x2c, 0x20, 0x430,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x2c, 0x20, 0x430,
0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x41c, 0x438, 0x430, 0x43d, 0x43c, 0x430, 0x41d, 0x430, 0x443, 0x440, 0x443,
0x41d, 0x435, 0x43f, 0x430, 0x43b, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d,
0x438, 0x430, 0x20, 0x4bf, 0x44b, 0x446, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e,
0x43d, 0x438, 0x430, 0x20, 0x4bf, 0x44b, 0x446, 0x2c, 0x20, 0x430, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x430, 0x20, 0x4bf,
0x44b, 0x446, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434,
0x438, 0x430, 0x20, 0x4bf, 0x44b, 0x446, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434,
0x438, 0x430, 0x20, 0x4bf, 0x44b, 0x446, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x430, 0x20, 0x4bf, 0x44b, 0x446,
0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x41d, 0x438, 0x443, 0x444, 0x430, 0x443, 0x43d, 0x434,
0x43b, 0x435, 0x43d, 0x434, 0x41d, 0x438, 0x443, 0x444, 0x430, 0x443, 0x43d, 0x434,
0x43b, 0x435, 0x43d, 0x434, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41d, 0x438,
0x443, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x2c, 0x20,
0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x41d, 0x438, 0x443, 0x435, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b,
0x43a, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x2c, 0x20, 0x430, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x2c, 0x20, 0x430,
0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x438,
0x2d, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x438, 0x430, 0x424, 0x435, 0x440, 0x43d,
0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x438, 0x2d, 0x41d, 0x43e, 0x440, 0x43e,
0x43d, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x424, 0x435, 0x440,
0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x438, 0x2d, 0x41d, 0x43e, 0x440,
0x43e, 0x43d, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442,
0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x4a9, 0x430, 0x434,
0x430, 0x442, 0x4d9, 0x438, 0x20, 0x41c, 0x430, 0x440, 0x438, 0x430, 0x43d, 0x442,
0x4d9, 0x438, 0x20, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x41d,
0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x41d, 0x43e,
0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x2c, 0x20, 0x430,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440,
0x441, 0x43a, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41e, 0x43c, 0x441, 0x43a, 0x41e, 0x43c,
0x441, 0x43a, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41e, 0x43c, 0x441, 0x43a,
0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d,
0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20,
0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x41f, 0x430, 0x43b, 0x430, 0x443, 0x41f, 0x430, 0x43f, 0x443, 0x430,
0x20, 0x2014, 0x20, 0x413, 0x432, 0x438, 0x43d, 0x435, 0x438, 0x430, 0x20, 0x4bf,
0x44b, 0x446, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x438, 0x41f, 0x430,
0x440, 0x430, 0x433, 0x432, 0x430, 0x438, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x438, 0x2c, 0x20, 0x430, 0x525,
0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x41f, 0x435, 0x440, 0x443, 0x41f, 0x435, 0x440, 0x443, 0x2c, 0x20, 0x430, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x41f, 0x435, 0x440, 0x443, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b,
0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x424, 0x438,
0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f,
0x438, 0x43d, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x424, 0x438, 0x43b, 0x438,
0x43f, 0x43f, 0x438, 0x43d, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442,
0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x430, 0x434, 0x2d, 0x445,
0x430, 0x49b, 0x4d9, 0x430, 0x20, 0x424, 0x435, 0x43d, 0x438, 0x43a, 0x441, 0x421,
0x435, 0x43d, 0x2d, 0x41f, 0x438, 0x435, 0x440, 0x438, 0x20, 0x41c, 0x438, 0x43a,
0x435, 0x43b, 0x43e, 0x43d, 0x438, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x438, 0x435,
0x440, 0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x438, 0x2c,
0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x438, 0x435,
0x440, 0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x438, 0x2c,
0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x41f, 0x438, 0x442, 0x43a, 0x435, 0x440, 0x43d, 0x41f, 0x43e,
0x43d, 0x43f, 0x435, 0x438, 0x41f, 0x445, 0x435, 0x43d, 0x438, 0x430, 0x43d, 0x41a,
0x44b, 0x437, 0x44b, 0x43b, 0x43e, 0x440, 0x434, 0x430, 0x2a, 0x41a, 0x44b, 0x437,
0x44b, 0x43b, 0x43e, 0x440, 0x434, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x2a, 0x41a, 0x44b, 0x437, 0x44b, 0x43b, 0x43e, 0x440, 0x434, 0x430, 0x2c, 0x20,
0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x2a, 0x420, 0x435, 0x438, 0x443, 0x43d, 0x43e, 0x43d, 0x420, 0x43e,
0x442, 0x435, 0x440, 0x430, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x421,
0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x2c, 0x20, 0x430, 0x525, 0x445,
0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x20, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x430, 0x4bf,
0x44b, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x442, 0x4d9, 0x438, 0x20, 0x430,
0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x2c,
0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x2c, 0x20,
0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x421, 0x435, 0x438, 0x448, 0x435, 0x43b, 0x442, 0x4d9, 0x438, 0x20,
0x430, 0x434, 0x433, 0x44c, 0x44b, 0x43b, 0x431, 0x436, 0x44c, 0x430, 0x445, 0x430,
0x49b, 0x4d9, 0x430, 0x421, 0x438, 0x43d, 0x433, 0x430, 0x43f, 0x443, 0x440, 0x421,
0x43e, 0x43b, 0x43e, 0x43c, 0x43e, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x434,
0x433, 0x44c, 0x44b, 0x43b, 0x431, 0x436, 0x44c, 0x430, 0x445, 0x430, 0x49b, 0x4d9,
0x430, 0x410, 0x43b, 0x430, 0x434, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x413, 0x435,
0x43e, 0x440, 0x433, 0x438, 0x430, 0x421, 0x443, 0x440, 0x438, 0x43d, 0x430, 0x43c,
0x421, 0x438, 0x43e, 0x432, 0x430, 0x422, 0x430, 0x438, 0x442, 0x438, 0x422, 0x430,
0x438, 0x432, 0x430, 0x43d, 0x422, 0x430, 0x438, 0x432, 0x430, 0x43d, 0x2c, 0x20,
0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x422, 0x430, 0x438, 0x432, 0x430, 0x43d, 0x2c, 0x20,
0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x422, 0x430, 0x45f, 0x44c, 0x44b, 0x49b, 0x441, 0x4ad, 0x430, 0x43d,
0x422, 0x43e, 0x43a, 0x435, 0x43b, 0x430, 0x443, 0x422, 0x43e, 0x43d, 0x433, 0x430,
0x422, 0x43e, 0x43d, 0x433, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x422,
0x43e, 0x43d, 0x433, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442,
0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x422, 0x440, 0x443, 0x43a,
0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d,
0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d,
0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435,
0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b,
0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x422, 0x443,
0x432, 0x430, 0x43b, 0x443, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x438, 0x423,
0x440, 0x443, 0x433, 0x432, 0x430, 0x438, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x438, 0x2c, 0x20, 0x430, 0x525, 0x445,
0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x423,
0x437, 0x431, 0x435, 0x49b, 0x44c, 0x438, 0x441, 0x4ad, 0x430, 0x43d, 0x423, 0x437,
0x431, 0x435, 0x49b, 0x44c, 0x438, 0x441, 0x4ad, 0x430, 0x43d, 0x2c, 0x20, 0x430,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x423, 0x437, 0x431, 0x435, 0x49b, 0x44c, 0x438, 0x441, 0x4ad,
0x430, 0x43d, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442,
0x443, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x2c, 0x20, 0x430, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x2c, 0x20, 0x430,
0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x412, 0x435, 0x43d, 0x435, 0x441, 0x443, 0x435, 0x43b, 0x430, 0x412, 0x43b,
0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x412, 0x43b, 0x430,
0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x2c, 0x20, 0x430, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e,
0x43a, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440,
0x430, 0x434, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x2c,
0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440,
0x430, 0x434, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43c, 0x440, 0x430, 0x433, 0x44b,
0x43b, 0x430, 0x440, 0x430, 0x423, 0x435, 0x438, 0x43a, 0x423, 0x43e, 0x43b, 0x43b,
0x438, 0x441, 0x438, 0x20, 0x424, 0x443, 0x442, 0x443, 0x43d, 0x435, 0x438, 0x418,
0x430, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x418, 0x430, 0x43a, 0x443, 0x442, 0x441,
0x43a, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x418, 0x430, 0x43a, 0x443, 0x442,
0x441, 0x43a, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440,
0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440,
0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433,
0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x418, 0x443, 0x43a, 0x43e, 0x43d, 0x41, 0x66, 0x67,
0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x74, 0x79, 0x64, 0x53,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x2d, 0x41, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x2d, 0x74, 0x79, 0x64, 0x4f, 0x6f, 0x73, 0x2d, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x2d, 0x74, 0x79, 0x64, 0x53, 0x75, 0x69, 0x64,
0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x57, 0x65, 0x73, 0x2d,
0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x2d, 0x74, 0x79, 0x64, 0x57, 0x65,
0x73, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x2d, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x57, 0x65, 0x73,
0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x2d, 0x73, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x79, 0x64, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x74,
0x79, 0x64, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79,
0x64, 0x41, 0x6d, 0x61, 0x73, 0x6f, 0x6e, 0x65, 0x2d, 0x74, 0x79, 0x64,
0x41, 0x6d, 0x61, 0x73, 0x6f, 0x6e, 0x65, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x41, 0x6d, 0x61, 0x73,
0x6f, 0x6e, 0x65, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64,
0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x2d, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x65, 0x20, 0x74, 0x79, 0x64, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x2d,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20,
0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4e, 0x6f, 0x6f,
0x72, 0x64, 0x2d, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e,
0x73, 0x65, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x4e, 0x6f, 0x6f,
0x72, 0x64, 0x2d, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e,
0x73, 0x65, 0x20, 0x6f, 0x6f, 0x73, 0x74, 0x65, 0x6c, 0x69, 0x6b, 0x65,
0x20, 0x74, 0x79, 0x64, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x2d, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x6f, 0x6f,
0x73, 0x74, 0x65, 0x6c, 0x69, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4e, 0x6f, 0x6f, 0x72,
0x64, 0x2d, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73,
0x65, 0x20, 0x6f, 0x6f, 0x73, 0x74, 0x65, 0x6c, 0x69, 0x6b, 0x65, 0x20,
0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x4e, 0x6f, 0x6f,
0x72, 0x64, 0x2d, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e,
0x73, 0x65, 0x20, 0x62, 0x65, 0x72, 0x67, 0x74, 0x79, 0x64, 0x4e, 0x6f,
0x6f, 0x72, 0x64, 0x2d, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61,
0x6e, 0x73, 0x65, 0x20, 0x62, 0x65, 0x72, 0x67, 0x2d, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4e, 0x6f, 0x6f,
0x72, 0x64, 0x2d, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e,
0x73, 0x65, 0x20, 0x62, 0x65, 0x72, 0x67, 0x2d, 0x64, 0x61, 0x67, 0x6c,
0x69, 0x67, 0x74, 0x79, 0x64, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x65,
0x73, 0x65, 0x20, 0x74, 0x79, 0x64, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69,
0x65, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x79, 0x64, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x65, 0x73,
0x65, 0x20, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x41,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x2d, 0x74, 0x79, 0x64, 0x41, 0x6e, 0x61,
0x64, 0x79, 0x72, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x79, 0x64, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x2d, 0x73,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x41, 0x70, 0x69, 0x61, 0x2d,
0x74, 0x79, 0x64, 0x41, 0x70, 0x69, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x41, 0x70, 0x69, 0x61,
0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x41, 0x72,
0x61, 0x62, 0x69, 0x65, 0x73, 0x65, 0x20, 0x74, 0x79, 0x64, 0x41, 0x72,
0x61, 0x62, 0x69, 0x65, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x41, 0x72, 0x61, 0x62, 0x69,
0x65, 0x73, 0x65, 0x20, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79,
0x64, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x69, 0xeb, 0x2d,
0x74, 0x79, 0x64, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x69,
0xeb, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x79, 0x64, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x69, 0xeb,
0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x57, 0x65, 0x73,
0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x69, 0xeb, 0x2d,
0x74, 0x79, 0x64, 0x57, 0x65, 0x73, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x69, 0xeb, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x57, 0x65, 0x73, 0x2d, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x69, 0xeb, 0x2d, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x79, 0x64, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0xeb,
0x2d, 0x74, 0x79, 0x64, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0xeb, 0x2d,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64,
0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0xeb, 0x2d, 0x73, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x79, 0x64, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x65,
0x73, 0x65, 0x20, 0x74, 0x79, 0x64, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x65, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x79, 0x64, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x65, 0x73, 0x65, 0x20, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79,
0x64, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x2d, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x73, 0x65, 0x20, 0x74, 0x79,
0x64, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x2d, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x73, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x53, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x65, 0x73, 0x65, 0x20, 0x64, 0x61, 0x67, 0x6c, 0x69,
0x67, 0x74, 0x79, 0x64, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c,
0x2d, 0x77, 0x65, 0x73, 0x74, 0x65, 0x6c, 0x69, 0x6b, 0x65, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x73, 0x65, 0x20, 0x74,
0x79, 0x64, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x2d, 0x77,
0x65, 0x73, 0x74, 0x65, 0x6c, 0x69, 0x6b, 0x65, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x2d, 0x74, 0x79, 0x64, 0x53, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x2d, 0x77, 0x65, 0x73, 0x74, 0x65,
0x6c, 0x69, 0x6b, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x65, 0x73, 0x65, 0x20, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74,
0x79, 0x64, 0x4f, 0x6f, 0x73, 0x74, 0x65, 0x6c, 0x69, 0x6b, 0x65, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x73, 0x65, 0x20,
0x74, 0x79, 0x64, 0x4f, 0x6f, 0x73, 0x74, 0x65, 0x6c, 0x69, 0x6b, 0x65,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x73, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79,
0x64, 0x4f, 0x6f, 0x73, 0x74, 0x65, 0x6c, 0x69, 0x6b, 0x65, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x73, 0x65, 0x20, 0x64,
0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x57, 0x65, 0x73, 0x74,
0x65, 0x6c, 0x69, 0x6b, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x65, 0x73, 0x65, 0x20, 0x74, 0x79, 0x64, 0x57, 0x65, 0x73,
0x74, 0x65, 0x6c, 0x69, 0x6b, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x65, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x57, 0x65, 0x73, 0x74, 0x65,
0x6c, 0x69, 0x6b, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x65, 0x73, 0x65, 0x20, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74,
0x79, 0x64, 0x41, 0x73, 0x65, 0x72, 0x62, 0x65, 0x69, 0x64, 0x6a, 0x61,
0x6e, 0x2d, 0x74, 0x79, 0x64, 0x41, 0x73, 0x65, 0x72, 0x62, 0x65, 0x69,
0x64, 0x6a, 0x61, 0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x79, 0x64, 0x41, 0x73, 0x65, 0x72, 0x62, 0x65, 0x69,
0x64, 0x6a, 0x61, 0x6e, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79,
0x64, 0x41, 0x73, 0x6f, 0x72, 0x65, 0x2d, 0x74, 0x79, 0x64, 0x41, 0x73,
0x6f, 0x72, 0x65, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x79, 0x64, 0x41, 0x73, 0x6f, 0x72, 0x65, 0x2d, 0x73, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x73, 0x6a, 0x2d, 0x74, 0x79, 0x64, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x6a, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x73, 0x6a, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x79, 0x64, 0x42, 0x68, 0x6f, 0x65, 0x74, 0x61, 0x6e, 0x2d, 0x74, 0x79,
0x64, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0xeb, 0x2d, 0x74, 0x79, 0x64,
0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x2d, 0x74, 0x79, 0x64,
0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x2d, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x61, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x79, 0x64, 0x42, 0x72, 0x6f, 0x65, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61,
0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x2d, 0x74, 0x79, 0x64,
0x4b, 0x61, 0x61, 0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x2d, 0x74,
0x79, 0x64, 0x4b, 0x61, 0x61, 0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79,
0x64, 0x4b, 0x61, 0x61, 0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x2d,
0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x43, 0x68, 0x61, 0x6d,
0x6f, 0x72, 0x72, 0x6f, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x79, 0x64, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x2d, 0x74, 0x79, 0x64, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x2d,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x2d, 0x64, 0x61, 0x67, 0x6c,
0x69, 0x67, 0x74, 0x79, 0x64, 0x43, 0x68, 0x69, 0x6c, 0x69, 0x2d, 0x74,
0x79, 0x64, 0x43, 0x68, 0x69, 0x6c, 0x69, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x43, 0x68, 0x69, 0x6c,
0x69, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x43, 0x68,
0x69, 0x6e, 0x61, 0x2d, 0x74, 0x79, 0x64, 0x43, 0x68, 0x69, 0x6e, 0x61,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79,
0x64, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69,
0x67, 0x74, 0x79, 0x64, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61,
0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x74, 0x79, 0x64, 0x4b,
0x6f, 0x6b, 0x6f, 0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x2d,
0x74, 0x79, 0x64, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0xeb, 0x2d,
0x74, 0x79, 0x64, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0xeb, 0x2d,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64,
0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0xeb, 0x2d, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x79, 0x64, 0x43, 0x6f, 0x6f, 0x6b, 0x65, 0x69, 0x6c,
0x61, 0x6e, 0x64, 0x65, 0x2d, 0x74, 0x79, 0x64, 0x43, 0x6f, 0x6f, 0x6b,
0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x43, 0x6f, 0x6f, 0x6b,
0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x2d, 0x68, 0x61, 0x6c, 0x66,
0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x4b, 0x75, 0x62, 0x61,
0x2d, 0x74, 0x79, 0x64, 0x4b, 0x75, 0x62, 0x61, 0x2d, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4b, 0x75, 0x62,
0x61, 0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x44,
0x61, 0x76, 0x69, 0x73, 0x2d, 0x74, 0x79, 0x64, 0x44, 0x75, 0x6d, 0x6f,
0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65,
0x2d, 0x74, 0x79, 0x64, 0x4f, 0x6f, 0x73, 0x2d, 0x54, 0x69, 0x6d, 0x6f,
0x72, 0x2d, 0x74, 0x79, 0x64, 0x50, 0x61, 0x61, 0x73, 0x65, 0x69, 0x6c,
0x61, 0x6e, 0x64, 0x2d, 0x74, 0x79, 0x64, 0x50, 0x61, 0x61, 0x73, 0x65,
0x69, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x50, 0x61, 0x61, 0x73, 0x65, 0x69,
0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79,
0x64, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x2d, 0x74, 0x79, 0x64,
0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x2d, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x73, 0x65, 0x20, 0x74, 0x79, 0x64, 0x53, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x61, 0x6c, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x74, 0x79, 0x64, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x2d,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x73, 0x65, 0x20, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x79, 0x64, 0x4f, 0x6f, 0x73, 0x2d, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x73, 0x65, 0x20, 0x74, 0x79, 0x64, 0x4f, 0x6f, 0x73,
0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x73, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4f, 0x6f,
0x73, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x73, 0x65, 0x20, 0x73,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x56, 0x65, 0x72, 0x64, 0x65,
0x72, 0x2d, 0x6f, 0x6f, 0x73, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x73, 0x65, 0x20, 0x74, 0x79, 0x64, 0x57, 0x65, 0x73, 0x2d, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x73, 0x65, 0x20, 0x74, 0x79, 0x64, 0x57, 0x65,
0x73, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x73, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x57,
0x65, 0x73, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x73, 0x65, 0x20,
0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x2d,
0x74, 0x79, 0x64, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x65,
0x69, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x2d, 0x73,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x46, 0x69, 0x64, 0x6a, 0x69,
0x2d, 0x74, 0x79, 0x64, 0x46, 0x69, 0x64, 0x6a, 0x69, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x46, 0x69,
0x64, 0x6a, 0x69, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64,
0x46, 0x72, 0x61, 0x6e, 0x73, 0x2d, 0x47, 0x75, 0x69, 0x61, 0x6e, 0x61,
0x2d, 0x74, 0x79, 0x64, 0x46, 0x72, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x53,
0x75, 0x69, 0x64, 0x65, 0x72, 0x2d, 0x20, 0x65, 0x6e, 0x20, 0x41, 0x6e,
0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x65, 0x73, 0x65, 0x20, 0x74, 0x79,
0x64, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x2d, 0x74,
0x79, 0x64, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x2d, 0x74, 0x79,
0x64, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0xeb, 0x2d, 0x74, 0x79, 0x64,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0xeb, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x47, 0x65, 0x6f, 0x72,
0x67, 0x69, 0xeb, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64,
0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x65, 0x69, 0x6c, 0x61, 0x6e,
0x64, 0x65, 0x2d, 0x74, 0x79, 0x64, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77,
0x69, 0x63, 0x68, 0x2d, 0x74, 0x79, 0x64, 0x4f, 0x6f, 0x73, 0x2d, 0x47,
0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x74, 0x79, 0x64,
0x4f, 0x6f, 0x73, 0x2d, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x79, 0x64, 0x4f, 0x6f, 0x73, 0x2d, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64,
0x57, 0x65, 0x73, 0x2d, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x2d, 0x74, 0x79, 0x64, 0x57, 0x65, 0x73, 0x2d, 0x47, 0x72, 0x6f,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x57, 0x65, 0x73, 0x2d, 0x47,
0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x79, 0x64, 0x50, 0x65, 0x72, 0x73, 0x69, 0x65, 0x73,
0x65, 0x20, 0x47, 0x6f, 0x6c, 0x66, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x48, 0x61, 0x77, 0x61, 0x69,
0x69, 0x2d, 0x41, 0x6c, 0x65, 0x6f, 0x65, 0x74, 0x65, 0x2d, 0x74, 0x79,
0x64, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x6f,
0x65, 0x74, 0x65, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x79, 0x64, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x6f, 0x65, 0x74, 0x65, 0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69,
0x67, 0x74, 0x79, 0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67,
0x2d, 0x74, 0x79, 0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79,
0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x2d, 0x73, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x48, 0x6f, 0x76, 0x64, 0x2d, 0x74,
0x79, 0x64, 0x48, 0x6f, 0x76, 0x64, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x48, 0x6f, 0x76, 0x64, 0x2d,
0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x49, 0x6e, 0x64, 0x69,
0xeb, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x79, 0x64, 0x49, 0x6e, 0x64, 0x69, 0x65, 0x73, 0x65, 0x20, 0x4f, 0x73,
0x65, 0x61, 0x61, 0x6e, 0x2d, 0x74, 0x79, 0x64, 0x49, 0x6e, 0x64, 0x6f,
0x73, 0x6a, 0x69, 0x6e, 0x61, 0x2d, 0x74, 0x79, 0x64, 0x53, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x61, 0x6c, 0x2d, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x73, 0x69, 0x65, 0x73, 0x65, 0x20, 0x74, 0x79, 0x64, 0x4f, 0x6f, 0x73,
0x2d, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0xeb, 0x2d, 0x74,
0x79, 0x64, 0x57, 0x65, 0x73, 0x2d, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x73, 0x69, 0xeb, 0x2d, 0x74, 0x79, 0x64, 0x49, 0x72, 0x61, 0x6e, 0x2d,
0x74, 0x79, 0x64, 0x49, 0x72, 0x61, 0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x49, 0x72, 0x61, 0x6e,
0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x49, 0x72,
0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b, 0x2d, 0x74, 0x79, 0x64, 0x49, 0x72,
0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x49, 0x72, 0x6b, 0x6f, 0x65,
0x74, 0x73, 0x6b, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64,
0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x2d, 0x74, 0x79, 0x64, 0x49, 0x73,
0x72, 0x61, 0x65, 0x6c, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x79, 0x64, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x2d,
0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x4a, 0x61, 0x70,
0x61, 0x6e, 0x2d, 0x74, 0x79, 0x64, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x2d,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64,
0x4a, 0x61, 0x70, 0x61, 0x6e, 0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67,
0x74, 0x79, 0x64, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c,
0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74,
0x73, 0x6b, 0x69, 0x2d, 0x74, 0x79, 0x64, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x50, 0x65, 0x74, 0x72,
0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61,
0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x2d, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x79, 0x64, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x73, 0x74,
0x61, 0x6e, 0x2d, 0x74, 0x79, 0x64, 0x4f, 0x6f, 0x73, 0x2d, 0x4b, 0x61,
0x7a, 0x61, 0x6b, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x74, 0x79, 0x64, 0x57,
0x65, 0x73, 0x2d, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x73, 0x74, 0x61, 0x6e,
0x2d, 0x74, 0x79, 0x64, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x61, 0x6e, 0x73,
0x65, 0x20, 0x74, 0x79, 0x64, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x61, 0x6e,
0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x74, 0x79, 0x64, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x61, 0x6e, 0x73, 0x65,
0x20, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x4b, 0x6f,
0x73, 0x72, 0x61, 0x65, 0x2d, 0x74, 0x79, 0x64, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2d, 0x74, 0x79, 0x64, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2d, 0x73,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x4b, 0x69, 0x72, 0x67, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x2d, 0x74, 0x79, 0x64, 0x4c, 0x69, 0x6e, 0x65,
0x2d, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x2d, 0x74, 0x79, 0x64,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0x74, 0x79,
0x64, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0x64, 0x61, 0x67,
0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x2d, 0x74, 0x79, 0x64, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79,
0x64, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x2d, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x79, 0x64, 0x4d, 0x61, 0x6c, 0x65, 0x69, 0x73, 0x69,
0xeb, 0x2d, 0x74, 0x79, 0x64, 0x4d, 0x61, 0x6c, 0x65, 0x64, 0x69, 0x76,
0x65, 0x2d, 0x74, 0x79, 0x64, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73,
0x61, 0x73, 0x2d, 0x74, 0x79, 0x64, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61,
0x6c, 0x6c, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x2d, 0x74, 0x79,
0x64, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x2d, 0x74,
0x79, 0x64, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x2d,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x2d, 0x73, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e,
0x2d, 0x74, 0x79, 0x64, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x61,
0x6e, 0x73, 0x65, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x65, 0x73,
0x65, 0x20, 0x74, 0x79, 0x64, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61,
0x61, 0x6e, 0x73, 0x65, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x65,
0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x74, 0x79, 0x64, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x61, 0x6e,
0x73, 0x65, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x65, 0x73, 0x65,
0x20, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x55, 0x6c,
0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x2d, 0x74, 0x79,
0x64, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79,
0x64, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72,
0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x4d, 0x6f, 0x73,
0x6b, 0x6f, 0x75, 0x2d, 0x74, 0x79, 0x64, 0x4d, 0x6f, 0x73, 0x6b, 0x6f,
0x75, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x79, 0x64, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x75, 0x2d, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x79, 0x64, 0x4d, 0x69, 0x61, 0x6e, 0x6d, 0x61, 0x72,
0x2d, 0x74, 0x79, 0x64, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x2d, 0x74, 0x79,
0x64, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x2d, 0x74, 0x79, 0x64, 0x4e, 0x69,
0x65, 0x75, 0x2d, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0xeb,
0x2d, 0x74, 0x79, 0x64, 0x4e, 0x69, 0x65, 0x75, 0x2d, 0x4b, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0xeb, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4e, 0x69, 0x65, 0x75, 0x2d,
0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0xeb, 0x2d, 0x73, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x4e, 0x69, 0x65, 0x75, 0x2d, 0x53,
0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x74, 0x79, 0x64, 0x4e, 0x69,
0x65, 0x75, 0x2d, 0x53, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4e,
0x69, 0x65, 0x75, 0x2d, 0x53, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x2d,
0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x4e, 0x65, 0x77,
0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x74, 0x79,
0x64, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e,
0x64, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x79, 0x64, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64,
0x4e, 0x69, 0x75, 0x65, 0x2d, 0x74, 0x79, 0x64, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x74, 0x79,
0x64, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x65, 0x69, 0x6c, 0x61,
0x6e, 0x64, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x74, 0x79, 0x64, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x65, 0x69,
0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74,
0x79, 0x64, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x74, 0x79,
0x64, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79,
0x64, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x2d, 0x74, 0x79, 0x64, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x79, 0x64, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64,
0x4f, 0x6d, 0x73, 0x6b, 0x2d, 0x74, 0x79, 0x64, 0x4f, 0x6d, 0x73, 0x6b,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79,
0x64, 0x4f, 0x6d, 0x73, 0x6b, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x79, 0x64, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x74,
0x79, 0x64, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x73, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x79, 0x64, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x2d, 0x74, 0x79,
0x64, 0x50, 0x61, 0x70, 0x6f, 0x65, 0x61, 0x2d, 0x4e, 0x69, 0x65, 0x75,
0x2d, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x65, 0x2d, 0x74, 0x79, 0x64, 0x50,
0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x2d, 0x74, 0x79, 0x64, 0x50,
0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x79, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79,
0x64, 0x50, 0x65, 0x72, 0x75, 0x2d, 0x74, 0x79, 0x64, 0x50, 0x65, 0x72,
0x75, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x79, 0x64, 0x50, 0x65, 0x72, 0x75, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x79, 0x64, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x79, 0x6e, 0x73,
0x65, 0x20, 0x74, 0x79, 0x64, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x79,
0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x79, 0x64, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x79, 0x6e,
0x73, 0x65, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x46,
0x65, 0x6e, 0x69, 0x6b, 0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x65,
0x2d, 0x74, 0x79, 0x64, 0x53, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x65, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x2d, 0x74, 0x79, 0x64, 0x53, 0x69, 0x6e, 0x74, 0x2d,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x6e, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x53, 0x69, 0x6e, 0x74, 0x2d,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x6e, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69,
0x67, 0x74, 0x79, 0x64, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e,
0x2d, 0x74, 0x79, 0x64, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x2d, 0x74,
0x79, 0x64, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x2d,
0x74, 0x79, 0x64, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x2d, 0x74,
0x79, 0x64, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x2d, 0x74, 0x79,
0x64, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x74, 0x79,
0x64, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x53, 0x61,
0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x79, 0x64, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d, 0x74, 0x79,
0x64, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x61, 0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x2d, 0x74, 0x79, 0x64, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x74, 0x79, 0x64, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x2d, 0x64, 0x61, 0x67,
0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65,
0x6c, 0x6c, 0x65, 0x2d, 0x74, 0x79, 0x64, 0x53, 0x69, 0x6e, 0x67, 0x61,
0x70, 0x6f, 0x65, 0x72, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x79, 0x64, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e,
0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x2d, 0x74, 0x79, 0x64,
0x53, 0x75, 0x69, 0x64, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0xeb,
0x2d, 0x74, 0x79, 0x64, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65,
0x2d, 0x74, 0x79, 0x64, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x2d, 0x74, 0x79,
0x64, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x2d, 0x74, 0x79, 0x64, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0x2d, 0x74, 0x79, 0x64, 0x54, 0x61, 0x69,
0x70, 0x65, 0x69, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x79, 0x64, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x2d, 0x64,
0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x54, 0x61, 0x64, 0x6a,
0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x74, 0x79, 0x64, 0x54,
0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x2d, 0x74, 0x79, 0x64, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x2d, 0x74, 0x79, 0x64, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79,
0x64, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x79, 0x64, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x2d, 0x74, 0x79, 0x64,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x2d, 0x74, 0x79, 0x64, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x79, 0x64, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x79, 0x64, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x2d, 0x74, 0x79, 0x64,
0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x2d, 0x74, 0x79, 0x64, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x79, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x4f,
0x65, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x74,
0x79, 0x64, 0x4f, 0x65, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x79, 0x64, 0x4f, 0x65, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x2d, 0x74, 0x79, 0x64, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x79, 0x64, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75,
0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x56, 0x65, 0x6e,
0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x2d, 0x74, 0x79, 0x64, 0x57, 0x6c,
0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x74, 0x79,
0x64, 0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79,
0x64, 0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x57, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2d, 0x74, 0x79, 0x64, 0x57, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x57, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x79, 0x64, 0x57, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x74, 0x79, 0x64,
0x57, 0x61, 0x6b, 0x65, 0x2d, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x2d,
0x74, 0x79, 0x64, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x65, 0x6e,
0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x2d, 0x74, 0x79, 0x64, 0x4a,
0x61, 0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b, 0x2d, 0x74, 0x79, 0x64, 0x4a,
0x61, 0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4a, 0x61, 0x6b, 0x6f,
0x65, 0x74, 0x73, 0x6b, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79,
0x64, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x2d, 0x74, 0x79, 0x64, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4a, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x2d, 0x73, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x74,
0x79, 0x64, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x46, 0x69, 0x6e, 0x69, 0x6d, 0x66, 0x69, 0x6e, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x41, 0x70, 0x75, 0x65,
0x65, 0x25b, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x41, 0x6e, 0x61, 0x61, 0x66, 0x6f, 0x254, 0x20, 0x53, 0x75,
0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x41, 0x74, 0x254, 0x65, 0x25b, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x41, 0x74,
0x254, 0x65, 0x25b, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x41,
0x74, 0x254, 0x65, 0x25b, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x53, 0x75, 0x73, 0x75,
0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75,
0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4d, 0x66,
0x69, 0x6e, 0x69, 0x6d, 0x66, 0x69, 0x6e, 0x69, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x4d, 0x66, 0x69, 0x6e, 0x69, 0x6d, 0x66, 0x69, 0x6e, 0x69, 0x20,
0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x4d, 0x66, 0x69, 0x6e, 0x69, 0x6d, 0x66, 0x69, 0x6e, 0x69, 0x20, 0x41,
0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x70, 0x75, 0x65,
0x65, 0x25b, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x41, 0x70, 0x75, 0x65, 0x65, 0x25b, 0x20, 0x41, 0x77,
0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x42, 0x65, 0x70, 0x254, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x42, 0x65, 0x70, 0x254, 0x20, 0x53, 0x75, 0x73,
0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x42, 0x65, 0x70,
0x254, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x50,
0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x50,
0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64,
0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x50, 0x61, 0x73, 0x69, 0x66,
0x69, 0x6b, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x41, 0x70, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x70, 0x69,
0x61, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x41, 0x70, 0x69, 0x61, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x20, 0x53, 0x75,
0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x72,
0x61, 0x62, 0x69, 0x61, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x41, 0x67, 0x79, 0x25b, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x41, 0x67, 0x79, 0x25b, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x41, 0x67, 0x79, 0x25b, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20,
0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x67, 0x79,
0x25b, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x41, 0x74, 0x254, 0x65, 0x65,
0x25b, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x67, 0x79, 0x25b, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x20, 0x41, 0x74, 0x254, 0x65, 0x65, 0x25b, 0x20, 0x53,
0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41,
0x67, 0x79, 0x25b, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x41, 0x74, 0x254,
0x65, 0x65, 0x25b, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x41, 0x61, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x41, 0x61, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x53, 0x75, 0x73,
0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x61, 0x6d,
0x65, 0x6e, 0x69, 0x61, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x20,
0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x20, 0x41, 0x77, 0x69,
0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x186, 0x73, 0x74, 0x72, 0x65, 0x6c,
0x69, 0x61, 0x20, 0x4d, 0x66, 0x69, 0x6e, 0x69, 0x6d, 0x66, 0x69, 0x6e,
0x69, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x186, 0x73, 0x74, 0x72, 0x65, 0x6c,
0x69, 0x61, 0x20, 0x4d, 0x66, 0x69, 0x6e, 0x69, 0x6d, 0x66, 0x69, 0x6e,
0x69, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x186, 0x73, 0x74, 0x72, 0x65, 0x6c, 0x69, 0x61, 0x20, 0x4d,
0x66, 0x69, 0x6e, 0x69, 0x6d, 0x66, 0x69, 0x6e, 0x69, 0x20, 0x41, 0x77,
0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x186, 0x73, 0x74, 0x72, 0x65,
0x6c, 0x69, 0x61, 0x20, 0x4d, 0x66, 0x69, 0x6e, 0x69, 0x6d, 0x66, 0x69,
0x6e, 0x69, 0x20, 0x41, 0x74, 0x254, 0x65, 0x65, 0x25b, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x186, 0x73, 0x74, 0x72, 0x65, 0x6c, 0x69, 0x61, 0x20, 0x4d,
0x66, 0x69, 0x6e, 0x69, 0x6d, 0x66, 0x69, 0x6e, 0x69, 0x20, 0x41, 0x74,
0x254, 0x65, 0x65, 0x25b, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x186, 0x73, 0x74, 0x72, 0x65, 0x6c, 0x69,
0x61, 0x20, 0x4d, 0x66, 0x69, 0x6e, 0x69, 0x6d, 0x66, 0x69, 0x6e, 0x69,
0x20, 0x41, 0x74, 0x254, 0x65, 0x65, 0x25b, 0x20, 0x41, 0x77, 0x69, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x186, 0x73, 0x74, 0x72, 0x65, 0x6c, 0x69,
0x61, 0x20, 0x41, 0x70, 0x75, 0x65, 0x65, 0x25b, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x186, 0x73, 0x74, 0x72, 0x65, 0x6c, 0x69, 0x61, 0x20, 0x41, 0x70,
0x75, 0x65, 0x65, 0x25b, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x186, 0x73, 0x74, 0x72, 0x65, 0x6c, 0x69,
0x61, 0x20, 0x41, 0x70, 0x75, 0x65, 0x65, 0x25b, 0x20, 0x41, 0x77, 0x69,
0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x186, 0x73, 0x74, 0x72, 0x65, 0x6c,
0x69, 0x61, 0x20, 0x41, 0x74, 0x254, 0x65, 0x65, 0x25b, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x186, 0x73, 0x74, 0x72, 0x65, 0x6c, 0x69, 0x61, 0x20, 0x41,
0x74, 0x254, 0x65, 0x65, 0x25b, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75,
0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x186, 0x73, 0x74, 0x72, 0x65, 0x6c,
0x69, 0x61, 0x20, 0x41, 0x74, 0x254, 0x65, 0x65, 0x25b, 0x20, 0x41, 0x77,
0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x73, 0x61, 0x62, 0x65,
0x67, 0x79, 0x61, 0x6e, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x73, 0x61,
0x62, 0x65, 0x67, 0x79, 0x61, 0x6e, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64,
0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x73, 0x61, 0x62, 0x65,
0x67, 0x79, 0x61, 0x6e, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x20, 0x53, 0x75, 0x73, 0x75,
0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x7a, 0x6f, 0x72,
0x65, 0x73, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x25b, 0x68, 0x79, 0x65, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x25b,
0x68, 0x79, 0x65, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x25b,
0x68, 0x79, 0x65, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x42, 0x75, 0x74, 0x61, 0x6e, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x42,
0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x42,
0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x53, 0x75, 0x73,
0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61,
0x72, 0x75, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x4b, 0x65, 0x70, 0x76, 0x25b, 0x64, 0x65, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x4b, 0x65, 0x70, 0x76, 0x25b, 0x64, 0x65, 0x20, 0x53, 0x75, 0x73, 0x75,
0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x65, 0x70, 0x76,
0x25b, 0x64, 0x65, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x4b, 0x79, 0x61, 0x6d, 0x6f, 0x72, 0x6f, 0x20, 0x53, 0x75, 0x73,
0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x79, 0x61,
0x74, 0x61, 0x6d, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x79, 0x61, 0x74,
0x61, 0x6d, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x4b, 0x79, 0x61, 0x74, 0x61, 0x6d, 0x20, 0x41, 0x77,
0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x79, 0x69, 0x6c, 0x69,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x79, 0x69, 0x6c, 0x69, 0x20, 0x53,
0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b,
0x79, 0x69, 0x6c, 0x69, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x4b, 0x79, 0x61, 0x65, 0x6e, 0x61, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x4b, 0x79, 0x61, 0x65, 0x6e, 0x61, 0x20, 0x53, 0x75, 0x73, 0x75,
0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x79, 0x61, 0x65,
0x6e, 0x61, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x42, 0x75, 0x72, 0x6f, 0x6e, 0x79, 0x61, 0x20, 0x41, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x6f, 0x6b, 0x6f, 0x73,
0x6f, 0x20, 0x41, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x53,
0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b,
0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x41, 0x77, 0x69, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x75, 0x6b, 0x20, 0x41, 0x65, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x75, 0x6b, 0x20,
0x41, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64,
0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x75, 0x6b, 0x20, 0x41,
0x65, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x46,
0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x75, 0x62, 0x61, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x4b, 0x75, 0x62, 0x61, 0x20, 0x53, 0x75, 0x73, 0x75,
0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x75, 0x62, 0x61,
0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x44, 0x61,
0x76, 0x69, 0x73, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x44, 0x75, 0x6d, 0x6f,
0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x54, 0x69, 0x6d, 0x254, 0x254, 0x20, 0x41,
0x70, 0x75, 0x65, 0x65, 0x25b, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x45, 0x61,
0x73, 0x74, 0x61, 0x20, 0x41, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x45, 0x61, 0x73, 0x74, 0x61, 0x20, 0x41, 0x65, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x45, 0x61, 0x73, 0x74, 0x61, 0x20, 0x41, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x59, 0x69, 0x6b, 0x75, 0x77, 0x65, 0x64, 0x254, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x59, 0x75, 0x72, 0x6f, 0x70, 0x75, 0x20, 0x4d, 0x66,
0x69, 0x6e, 0x69, 0x6d, 0x66, 0x69, 0x6e, 0x69, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x59, 0x75, 0x72, 0x6f, 0x70, 0x75, 0x20, 0x4d, 0x66, 0x69, 0x6e,
0x69, 0x6d, 0x66, 0x69, 0x6e, 0x69, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64,
0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x59, 0x75, 0x72, 0x6f, 0x70,
0x75, 0x20, 0x4d, 0x66, 0x69, 0x6e, 0x69, 0x6d, 0x66, 0x69, 0x6e, 0x69,
0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x59, 0x75,
0x72, 0x6f, 0x70, 0x75, 0x20, 0x41, 0x70, 0x75, 0x65, 0x65, 0x25b, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x59, 0x75, 0x72, 0x6f, 0x70, 0x75, 0x20, 0x41,
0x70, 0x75, 0x65, 0x65, 0x25b, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75,
0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x59, 0x75, 0x72, 0x6f, 0x70, 0x75,
0x20, 0x41, 0x70, 0x75, 0x65, 0x65, 0x25b, 0x20, 0x41, 0x77, 0x69, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x59, 0x75, 0x72, 0x6f, 0x70, 0x75, 0x20,
0x41, 0x70, 0x75, 0x65, 0x65, 0x25b, 0x20, 0x4e, 0x6f, 0x68, 0x6f, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x59, 0x75, 0x72, 0x6f, 0x70, 0x75, 0x20,
0x41, 0x74, 0x254, 0x65, 0x65, 0x25b, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x59,
0x75, 0x72, 0x6f, 0x70, 0x75, 0x20, 0x41, 0x74, 0x254, 0x65, 0x65, 0x25b,
0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x59, 0x75, 0x72, 0x6f, 0x70, 0x75, 0x20, 0x41, 0x74, 0x254, 0x65,
0x65, 0x25b, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x46, 0x254, 0x6b, 0x6d, 0x61, 0x6e, 0x20, 0x41, 0x65, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x46, 0x254, 0x6b, 0x6d, 0x61, 0x6e,
0x20, 0x41, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53, 0x75, 0x73, 0x75,
0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x46, 0x254, 0x6b, 0x6d,
0x61, 0x6e, 0x20, 0x41, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x41, 0x77,
0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x46, 0x69, 0x67, 0x79, 0x69,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x46, 0x69, 0x67, 0x79, 0x69, 0x20, 0x53,
0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x46,
0x69, 0x67, 0x79, 0x69, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x46, 0x72, 0x25b, 0x6e, 0x6b, 0x79, 0x65, 0x20, 0x47, 0x61,
0x79, 0x61, 0x6e, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x46, 0x72, 0x25b,
0x6e, 0x6b, 0x79, 0x65, 0x20, 0x41, 0x6e, 0x61, 0x61, 0x66, 0x6f, 0x254,
0x20, 0x6e, 0x65, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x61, 0x74, 0x69, 0x6b,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67,
0x254, 0x73, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x47, 0x61, 0x6d, 0x62, 0x69,
0x65, 0x72, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x47, 0x79, 0x254, 0x67, 0x79,
0x65, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x47, 0x79, 0x254, 0x67, 0x79,
0x65, 0x61, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x47, 0x79, 0x254, 0x67, 0x79, 0x65, 0x61, 0x20, 0x41,
0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x47, 0x65, 0x65, 0x62,
0x25b, 0x74, 0x20, 0x41, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x20,
0x4d, 0x65, 0x61, 0x6e, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x41, 0x70, 0x75, 0x65, 0x65,
0x25b, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x41, 0x70, 0x75, 0x65, 0x65, 0x25b, 0x20, 0x53,
0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x41, 0x70, 0x75,
0x65, 0x65, 0x25b, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x41,
0x74, 0x254, 0x65, 0x25b, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x41, 0x74, 0x254, 0x65, 0x25b,
0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x41,
0x74, 0x254, 0x65, 0x25b, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x47, 0x254, 0x254, 0x66, 0x6f, 0x20, 0x53, 0x75, 0x73, 0x75,
0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x48, 0x61, 0x77, 0x61,
0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x20, 0x53, 0x75, 0x73, 0x75,
0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x48, 0x61, 0x77, 0x61,
0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x20,
0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x48, 0x254, 0x6e,
0x6b, 0x254, 0x6e, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x48, 0x254, 0x6e, 0x6b,
0x254, 0x6e, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x48, 0x254, 0x6e, 0x6b, 0x254, 0x6e, 0x20, 0x41, 0x77,
0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x48, 0x6f, 0x76, 0x64, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x53, 0x75, 0x73,
0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x48, 0x6f, 0x76,
0x64, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x50,
0x6f, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x49, 0x6e, 0x64, 0x254, 0x6b, 0x79,
0x69, 0x6e, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x49, 0x6e, 0x64, 0x254,
0x6e, 0x65, 0x68, 0x79, 0x69, 0x61, 0x20, 0x4d, 0x66, 0x69, 0x6e, 0x69,
0x6d, 0x66, 0x69, 0x6e, 0x69, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x49, 0x6e,
0x64, 0x254, 0x6e, 0x65, 0x68, 0x79, 0x69, 0x61, 0x20, 0x41, 0x70, 0x75,
0x65, 0x65, 0x25b, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x49, 0x6e, 0x64, 0x254,
0x6e, 0x65, 0x68, 0x79, 0x69, 0x61, 0x20, 0x41, 0x74, 0x254, 0x65, 0x65,
0x25b, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x53, 0x75, 0x73, 0x75,
0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x49, 0x72, 0x61, 0x6e,
0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75,
0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x49,
0x73, 0x72, 0x61, 0x65, 0x6c, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x49, 0x73,
0x72, 0x61, 0x65, 0x6c, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x20,
0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x47, 0x79, 0x61,
0x70, 0x61, 0x6e, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x47, 0x79, 0x61, 0x70,
0x61, 0x6e, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x47, 0x79, 0x61, 0x70, 0x61, 0x6e, 0x20, 0x41, 0x77,
0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x61, 0x7a, 0x61, 0x6b,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x61, 0x7a,
0x61, 0x6b, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x41, 0x70, 0x75, 0x65, 0x65,
0x25b, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x41, 0x74, 0x254, 0x65, 0x25b, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x20, 0x53, 0x75, 0x73, 0x75,
0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x6f, 0x72, 0x65,
0x61, 0x6e, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x4b, 0x25b, 0x25b, 0x67, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x4c, 0x61, 0x69, 0x20, 0x41, 0x65, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x4c, 0x254, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4c, 0x254, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x4c, 0x254, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20,
0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4d, 0x61,
0x6c, 0x65, 0x68, 0x79, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4d,
0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x73, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x4d, 0x61, 0x6b, 0x65, 0x73, 0x61, 0x73, 0x65, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x4d, 0x61, 0x68, 0x79, 0x61, 0x61, 0x20, 0x41, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4d, 0x254, 0x72, 0x69, 0x68,
0x79, 0x69, 0x254, 0x73, 0x6f, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4d, 0x254,
0x72, 0x69, 0x68, 0x79, 0x69, 0x254, 0x73, 0x6f, 0x20, 0x53, 0x75, 0x73,
0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4d, 0x254, 0x72,
0x69, 0x68, 0x79, 0x69, 0x254, 0x73, 0x6f, 0x20, 0x41, 0x77, 0x69, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x4d, 0x254, 0x254, 0x73, 0x6f, 0x6e, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x4d, 0x25b, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x20,
0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x4d, 0x25b, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x20, 0x50, 0x61, 0x73, 0x69,
0x66, 0x69, 0x6b, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x4d, 0x25b, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x20,
0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x41, 0x77, 0x69, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x59, 0x75, 0x6c, 0x61, 0x6e, 0x62, 0x61,
0x74, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x59, 0x75, 0x6c, 0x61, 0x6e,
0x62, 0x61, 0x74, 0x61, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x59, 0x75, 0x6c, 0x61, 0x6e, 0x62, 0x61,
0x74, 0x61, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x4d, 0x254, 0x73, 0x6b, 0x6f, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4d, 0x254,
0x73, 0x6b, 0x6f, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x4d, 0x254, 0x73, 0x6b, 0x6f, 0x20, 0x41, 0x77,
0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4d, 0x61, 0x79, 0x61, 0x61,
0x6d, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4e, 0x61, 0x75, 0x72, 0x75,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x4e, 0x25b, 0x70, 0x61, 0x6c, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61,
0x20, 0x46, 0x6f, 0x66, 0x6f, 0x72, 0x254, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x46, 0x6f,
0x66, 0x6f, 0x72, 0x254, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x69, 0x61, 0x20, 0x46, 0x6f, 0x66, 0x6f, 0x72, 0x254, 0x20, 0x41, 0x77,
0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x5a, 0x69, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x46, 0x6f, 0x66, 0x6f, 0x72, 0x254, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x5a, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x46, 0x6f, 0x66, 0x6f,
0x72, 0x254, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x5a, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x46, 0x6f,
0x66, 0x6f, 0x72, 0x254, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4e, 0x65, 0x77, 0x66, 0x6f,
0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53, 0x75, 0x73, 0x75,
0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4e, 0x65, 0x77, 0x66,
0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x41, 0x77, 0x69,
0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4e, 0x69, 0x75, 0x65, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x4e, 0x254, 0x254, 0x66, 0x254, 0x6b, 0x20, 0x41, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4e, 0x254, 0x254,
0x66, 0x254, 0x6b, 0x20, 0x41, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53,
0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4e,
0x254, 0x254, 0x66, 0x254, 0x6b, 0x20, 0x41, 0x65, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75,
0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4f, 0x6d, 0x73,
0x6b, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x41,
0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x50, 0x61, 0x6c, 0x61,
0x75, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20,
0x47, 0x69, 0x6e, 0x69, 0x20, 0x46, 0x6f, 0x66, 0x6f, 0x72, 0x254, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x65,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x65, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x65, 0x20, 0x41,
0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x50, 0x65, 0x72, 0x75,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x50, 0x65, 0x72, 0x75, 0x20, 0x53, 0x75,
0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x50, 0x65,
0x72, 0x75, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x20, 0x53, 0x75, 0x73, 0x75,
0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x46, 0x69, 0x6e, 0x69, 0x73, 0x65, 0x20, 0x41, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x53, 0x74, 0x2e, 0x20, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x53, 0x74, 0x2e,
0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64,
0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x53, 0x74, 0x2e, 0x20, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x50, 0x69, 0x74, 0x6b, 0x61, 0x69, 0x72, 0x6e, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x52, 0x6f, 0x74, 0x65, 0x72, 0x61, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x41, 0x77,
0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x53,
0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x53, 0x65, 0x79, 0x68, 0x79, 0x25b, 0x6c, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x254, 0x20, 0x53, 0x75,
0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x53, 0x6f,
0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x20, 0x41, 0x65, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x47, 0x79, 0x254, 0x254, 0x67, 0x79, 0x69,
0x61, 0x20, 0x41, 0x6e, 0x61, 0x61, 0x66, 0x6f, 0x254, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75,
0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69,
0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x54, 0x61,
0x67, 0x79, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75,
0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20,
0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x79, 0x75,
0x75, 0x6b, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x54, 0x25b, 0x6b, 0x6d, 0x25b,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x54,
0x25b, 0x6b, 0x6d, 0x25b, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53,
0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x54,
0x25b, 0x6b, 0x6d, 0x25b, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x41,
0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x54, 0x75, 0x76, 0x61,
0x6c, 0x75, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x59, 0x75, 0x72, 0x75, 0x67,
0x77, 0x61, 0x65, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x59, 0x75, 0x72, 0x75,
0x67, 0x77, 0x61, 0x65, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x59, 0x75, 0x72, 0x75, 0x67, 0x77, 0x61,
0x65, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x55,
0x73, 0x62, 0x25b, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x55, 0x73, 0x62, 0x25b, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x55, 0x73, 0x62, 0x25b, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75,
0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x56, 0x65,
0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x41,
0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x56, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x57, 0x61, 0x6b, 0x65, 0x20,
0x41, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x57,
0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x6e, 0x65, 0x20, 0x46, 0x75, 0x74,
0x75, 0x6e, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x59, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x59, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20,
0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x59, 0x25b, 0x6b,
0x61, 0x74, 0x25b, 0x72, 0x69, 0x6e, 0x62, 0x25b, 0x67, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x59, 0x25b, 0x6b, 0x61, 0x74, 0x25b, 0x72, 0x69, 0x6e, 0x62,
0x25b, 0x67, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x59, 0x25b, 0x6b, 0x61, 0x74, 0x25b, 0x72, 0x69, 0x6e,
0x62, 0x25b, 0x67, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x59, 0x75, 0x6b, 0x254, 0x6e, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x45, 0x6a, 0x6b, 0x72, 0x69, 0x74, 0x20,
0x5b, 0x41, 0x6b, 0x6f, 0x5d, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x45, 0x6a, 0x6b,
0x72, 0x69, 0x74, 0x20, 0x5b, 0x41, 0x6b, 0x6f, 0x5d, 0x4f, 0x72, 0x61,
0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x45, 0x6a,
0x6b, 0x72, 0x69, 0x74, 0x20, 0x5b, 0x41, 0x6b, 0x6f, 0x5d, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x41, 0x66,
0x72, 0x69, 0x6b, 0xeb, 0x73, 0x20, 0x51, 0x65, 0x6e, 0x64, 0x72, 0x6f,
0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x41, 0x66, 0x72, 0x69,
0x6b, 0xeb, 0x73, 0x20, 0x4c, 0x69, 0x6e, 0x64, 0x6f, 0x72, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65,
0x20, 0x65, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0xeb, 0x73, 0x20, 0x4a,
0x75, 0x67, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x41,
0x66, 0x72, 0x69, 0x6b, 0xeb, 0x73, 0x20, 0x50, 0x65, 0x72, 0xeb, 0x6e,
0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x41, 0x66,
0x72, 0x69, 0x6b, 0xeb, 0x73, 0x20, 0x50, 0x65, 0x72, 0xeb, 0x6e, 0x64,
0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72,
0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0xeb,
0x73, 0x20, 0x50, 0x65, 0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0xeb,
0x73, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20,
0x65, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0xeb, 0x73, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x69, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20,
0x65, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x69, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x41, 0x6c,
0x6d, 0x61, 0x74, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x41,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x41,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x76,
0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x41, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x53, 0x48,
0x42, 0x41, 0x2d, 0x73, 0xeb, 0x20, 0x51, 0x65, 0x6e, 0x64, 0x72, 0x6f,
0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x53, 0x48, 0x42, 0x41, 0x2d, 0x73,
0xeb, 0x20, 0x51, 0x65, 0x6e, 0x64, 0x72, 0x6f, 0x72, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x53,
0x48, 0x42, 0x41, 0x2d, 0x73, 0xeb, 0x20, 0x51, 0x65, 0x6e, 0x64, 0x72,
0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x53, 0x48, 0x42,
0x41, 0x2d, 0x73, 0xeb, 0x20, 0x4c, 0x69, 0x6e, 0x64, 0x6f, 0x72, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x65, 0x20, 0x65, 0x20, 0x53, 0x48, 0x42, 0x41, 0x2d, 0x73, 0xeb, 0x20,
0x4c, 0x69, 0x6e, 0x64, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x76,
0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x53, 0x48, 0x42, 0x41,
0x2d, 0x73, 0xeb, 0x20, 0x4c, 0x69, 0x6e, 0x64, 0x6f, 0x72, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x69, 0x74, 0x6f,
0x72, 0x65, 0x76, 0x65, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x6e, 0x65, 0x20, 0x74, 0xeb, 0x20, 0x42, 0x72, 0x65, 0x7a, 0x69, 0x74,
0x20, 0x4d, 0x61, 0x6c, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x54, 0x65,
0x72, 0x72, 0x69, 0x74, 0x6f, 0x72, 0x65, 0x76, 0x65, 0x20, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x20, 0x74, 0xeb, 0x20, 0x42,
0x72, 0x65, 0x7a, 0x69, 0x74, 0x20, 0x4d, 0x61, 0x6c, 0x6f, 0x72, 0x4f,
0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20,
0x54, 0x65, 0x72, 0x72, 0x69, 0x74, 0x6f, 0x72, 0x65, 0x76, 0x65, 0x20,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x20, 0x74, 0xeb,
0x20, 0x42, 0x72, 0x65, 0x7a, 0x69, 0x74, 0x20, 0x4d, 0x61, 0x6c, 0x6f,
0x72, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x69,
0x74, 0x6f, 0x72, 0x65, 0x76, 0x65, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x6e, 0x65, 0x20, 0x74, 0xeb, 0x20, 0x42, 0x72, 0x65, 0x67,
0x75, 0x74, 0x20, 0x74, 0xeb, 0x20, 0x50, 0x61, 0x71, 0xeb, 0x73, 0x6f,
0x72, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x69,
0x74, 0x6f, 0x72, 0x65, 0x76, 0x65, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x6e, 0x65, 0x20, 0x74, 0xeb, 0x20, 0x42, 0x72, 0x65, 0x67,
0x75, 0x74, 0x20, 0x74, 0xeb, 0x20, 0x50, 0x61, 0x71, 0xeb, 0x73, 0x6f,
0x72, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72,
0x65, 0x20, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x69, 0x74, 0x6f, 0x72,
0x65, 0x76, 0x65, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x65, 0x20, 0x74, 0xeb, 0x20, 0x42, 0x72, 0x65, 0x67, 0x75, 0x74, 0x20,
0x74, 0xeb, 0x20, 0x50, 0x61, 0x71, 0xeb, 0x73, 0x6f, 0x72, 0x69, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72,
0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72,
0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65,
0x20, 0x65, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x69, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x73, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20,
0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x76,
0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61,
0x73, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x61, 0x75,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x65, 0x20, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20,
0x41, 0x6b, 0x74, 0x61, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x41, 0x6b, 0x74, 0x6f, 0x62, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x41,
0x6b, 0x74, 0x6f, 0x62, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65,
0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x6f, 0x62,
0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x61, 0x72, 0x61, 0x62, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65,
0x20, 0x61, 0x72, 0x61, 0x62, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65,
0x72, 0x6f, 0x72, 0x65, 0x20, 0x61, 0x72, 0x61, 0x62, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x41, 0x72, 0x67, 0x6a, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x41, 0x72, 0x67, 0x6a, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65,
0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x41, 0x72, 0x67, 0x6a, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x41, 0x72, 0x67, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0xeb, 0x73, 0x20,
0x50, 0x65, 0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65,
0x20, 0x65, 0x20, 0x41, 0x72, 0x67, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0xeb, 0x73, 0x20, 0x50, 0x65, 0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f,
0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65,
0x20, 0x65, 0x20, 0x41, 0x72, 0x67, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0xeb, 0x73, 0x20, 0x50, 0x65, 0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f,
0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f,
0x72, 0x65, 0x20, 0x65, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76,
0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x41, 0x74, 0x6c, 0x61,
0x6e, 0x74, 0x69, 0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0xeb, 0x20, 0x51,
0x65, 0x6e, 0x64, 0x72, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0xeb, 0x20, 0x51, 0x65,
0x6e, 0x64, 0x72, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65,
0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x73, 0xeb, 0x20, 0x51, 0x65, 0x6e, 0x64, 0x72, 0x6f,
0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x73, 0xeb, 0x20, 0x51, 0x65, 0x6e, 0x64, 0x72,
0x6f, 0x72, 0x6f, 0x2d, 0x50, 0x65, 0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d,
0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x73, 0xeb, 0x20, 0x51, 0x65, 0x6e, 0x64, 0x72, 0x6f,
0x72, 0x6f, 0x2d, 0x50, 0x65, 0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f,
0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65,
0x20, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73,
0xeb, 0x20, 0x51, 0x65, 0x6e, 0x64, 0x72, 0x6f, 0x72, 0x6f, 0x2d, 0x50,
0x65, 0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x73, 0xeb, 0x20, 0x4c, 0x69, 0x6e, 0x64, 0x6f, 0x72, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20,
0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0xeb,
0x20, 0x4c, 0x69, 0x6e, 0x64, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0xeb, 0x20, 0x4c, 0x69, 0x6e, 0x64,
0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0xeb, 0x20, 0x50, 0x65, 0x72, 0xeb,
0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0xeb, 0x20, 0x50, 0x65,
0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0xeb, 0x20, 0x50, 0x65, 0x72,
0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x78, 0x68, 0x61,
0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62,
0x61, 0x6a, 0x78, 0x68, 0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x6a, 0x78, 0x68, 0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x76, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65,
0x20, 0x65, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x76, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x76, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x69, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x65, 0x20, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x73, 0x68, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f,
0x72, 0x65, 0x20, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x73, 0x68, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x42,
0x75, 0x74, 0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x73, 0xeb, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65,
0x20, 0x65, 0x20, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x73, 0xeb,
0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65,
0x20, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x73, 0xeb, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x2d, 0x44,
0x75, 0x72, 0x61, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x69, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x4b, 0x65, 0x70, 0x69, 0x74, 0x20, 0x74, 0xeb,
0x20, 0x47, 0x6a, 0x65, 0x6c, 0x62, 0xeb, 0x72, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20,
0x4b, 0x65, 0x70, 0x69, 0x74, 0x20, 0x74, 0xeb, 0x20, 0x47, 0x6a, 0x65,
0x6c, 0x62, 0xeb, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f,
0x72, 0x65, 0x20, 0x65, 0x20, 0x4b, 0x65, 0x70, 0x69, 0x74, 0x20, 0x74,
0xeb, 0x20, 0x47, 0x6a, 0x65, 0x6c, 0x62, 0xeb, 0x72, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x4b, 0x65, 0x6a, 0x73, 0x69, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x4b, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x73, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x4b, 0x61, 0x74, 0x61, 0x6d, 0x69, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x65, 0x20, 0x65, 0x20, 0x4b, 0x61, 0x74, 0x61, 0x6d, 0x69, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20,
0x4b, 0x61, 0x74, 0x61, 0x6d, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x4b, 0x69, 0x6c, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x4b, 0x69,
0x6c, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72,
0x65, 0x20, 0x65, 0x20, 0x4b, 0x69, 0x6c, 0x69, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x4b, 0x69, 0x6e, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20,
0x4b, 0x69, 0x6e, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72,
0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4b, 0x69, 0x6e, 0xeb, 0x73, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6c, 0x6c, 0x69,
0x74, 0x20, 0x74, 0xeb, 0x20, 0x4b, 0x72, 0x69, 0x73, 0x68, 0x74, 0x6c,
0x69, 0x6e, 0x64, 0x6a, 0x65, 0x76, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x49, 0x73, 0x68, 0x75, 0x6a, 0x76, 0x65, 0x20, 0x4b, 0x6f, 0x6b,
0x6f, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4b, 0x6f, 0x6c, 0x75,
0x6d, 0x62, 0x69, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x4b, 0x6f, 0x6c,
0x75, 0x6d, 0x62, 0x69, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65,
0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4b, 0x6f, 0x6c, 0x75, 0x6d,
0x62, 0x69, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x49, 0x73,
0x68, 0x75, 0x6a, 0x76, 0x65, 0x20, 0x4b, 0x75, 0x6b, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65,
0x20, 0x49, 0x73, 0x68, 0x75, 0x6a, 0x76, 0x65, 0x20, 0x4b, 0x75, 0x6b,
0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65,
0x20, 0x49, 0x73, 0x68, 0x75, 0x6a, 0x76, 0x65, 0x20, 0x4b, 0x75, 0x6b,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4b, 0x75, 0x62, 0xeb, 0x73, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65,
0x20, 0x65, 0x20, 0x4b, 0x75, 0x62, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20,
0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4b, 0x75, 0x62,
0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x44, 0x65, 0x6a, 0x76,
0x69, 0x73, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x44, 0x75,
0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c,
0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x54, 0x69, 0x6d, 0x6f,
0x72, 0x69, 0x74, 0x20, 0x4c, 0x69, 0x6e, 0x64, 0x6f, 0x72, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6c, 0x6c, 0x69, 0x74,
0x20, 0x74, 0xeb, 0x20, 0x50, 0x61, 0x73, 0x68, 0x6b, 0xeb, 0x73, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65,
0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6c, 0x6c, 0x69, 0x74, 0x20,
0x74, 0xeb, 0x20, 0x50, 0x61, 0x73, 0x68, 0x6b, 0xeb, 0x73, 0x4f, 0x72,
0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x49,
0x73, 0x68, 0x75, 0x6c, 0x6c, 0x69, 0x74, 0x20, 0x74, 0xeb, 0x20, 0x50,
0x61, 0x73, 0x68, 0x6b, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x45, 0x6b, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x69, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x45, 0x76, 0x72, 0x6f, 0x70, 0xeb, 0x73, 0x20, 0x51,
0x65, 0x6e, 0x64, 0x72, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x45,
0x76, 0x72, 0x6f, 0x70, 0xeb, 0x73, 0x20, 0x51, 0x65, 0x6e, 0x64, 0x72,
0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72,
0x65, 0x20, 0x65, 0x20, 0x45, 0x76, 0x72, 0x6f, 0x70, 0xeb, 0x73, 0x20,
0x51, 0x65, 0x6e, 0x64, 0x72, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x45, 0x76, 0x72, 0x6f, 0x70, 0xeb, 0x73, 0x20, 0x4c, 0x69,
0x6e, 0x64, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x45, 0x76, 0x72,
0x6f, 0x70, 0xeb, 0x73, 0x20, 0x4c, 0x69, 0x6e, 0x64, 0x6f, 0x72, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65,
0x20, 0x45, 0x76, 0x72, 0x6f, 0x70, 0xeb, 0x73, 0x20, 0x4c, 0x69, 0x6e,
0x64, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x45, 0x76,
0x72, 0x6f, 0x70, 0xeb, 0x73, 0x20, 0x73, 0xeb, 0x20, 0x4c, 0x61, 0x72,
0x67, 0xeb, 0x74, 0x20, 0x4c, 0x69, 0x6e, 0x64, 0x6f, 0x72, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x45, 0x76, 0x72, 0x6f, 0x70, 0xeb, 0x73,
0x20, 0x50, 0x65, 0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x65, 0x20, 0x65, 0x20, 0x45, 0x76, 0x72, 0x6f, 0x70, 0xeb, 0x73, 0x20,
0x50, 0x65, 0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20,
0x45, 0x76, 0x72, 0x6f, 0x70, 0xeb, 0x73, 0x20, 0x50, 0x65, 0x72, 0xeb,
0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x49, 0x73, 0x68, 0x75, 0x6a, 0x76, 0x65, 0x20, 0x46, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68,
0x75, 0x6a, 0x76, 0x65, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20,
0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6a, 0x76, 0x65, 0x20, 0x46, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x46, 0x69, 0x78, 0x68, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x46, 0x69,
0x78, 0x68, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f,
0x72, 0x65, 0x20, 0x65, 0x20, 0x46, 0x69, 0x78, 0x68, 0x69, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x47, 0x75, 0x61, 0x6a, 0x61, 0x6e, 0xeb,
0x73, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x7a, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x69, 0x74, 0x6f, 0x72,
0x65, 0x76, 0x65, 0x20, 0x4a, 0x75, 0x67, 0x6f, 0x72, 0x65, 0x20, 0x64,
0x68, 0x65, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x6b,
0x65, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x7a, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f,
0x73, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x47, 0x61, 0x6d,
0x62, 0x69, 0x65, 0x72, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x47, 0x6a, 0x65, 0x6f, 0x72, 0x67, 0x6a, 0x69, 0x73, 0xeb, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20,
0x65, 0x20, 0x47, 0x6a, 0x65, 0x6f, 0x72, 0x67, 0x6a, 0x69, 0x73, 0xeb,
0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65,
0x20, 0x47, 0x6a, 0x65, 0x6f, 0x72, 0x67, 0x6a, 0x69, 0x73, 0xeb, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6a, 0x76, 0x65,
0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x75, 0x69, 0xe7, 0x69, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0xeb, 0x73, 0x20, 0x4c, 0x69, 0x6e, 0x64, 0x6f, 0x72, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65,
0x20, 0x65, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0xeb,
0x73, 0x20, 0x4c, 0x69, 0x6e, 0x64, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x47, 0x72,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0xeb, 0x73, 0x20, 0x4c, 0x69, 0x6e,
0x64, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x47, 0x72,
0xeb, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0xeb, 0x73, 0x20, 0x50, 0x65, 0x72,
0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20,
0x47, 0x72, 0xeb, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0xeb, 0x73, 0x20, 0x50,
0x65, 0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x47,
0x72, 0xeb, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0xeb, 0x73, 0x20, 0x50, 0x65,
0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x47, 0x75, 0x61, 0x6d, 0x69, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x47, 0x6a, 0x69, 0x72, 0x69, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6a, 0x76, 0x65, 0x20, 0x48,
0x61, 0x75, 0x61, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x65, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6a, 0x76, 0x65,
0x20, 0x48, 0x61, 0x75, 0x61, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x69, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72,
0x65, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6a, 0x76, 0x65, 0x20,
0x48, 0x61, 0x75, 0x61, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69,
0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67,
0x2d, 0x4b, 0x6f, 0x6e, 0x67, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x48,
0x6f, 0x6e, 0x67, 0x2d, 0x4b, 0x6f, 0x6e, 0x67, 0x75, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x48,
0x6f, 0x6e, 0x67, 0x2d, 0x4b, 0x6f, 0x6e, 0x67, 0x75, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x69, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20,
0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x48, 0x6f, 0x76,
0x64, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x73,
0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4f, 0x71, 0x65, 0x61, 0x6e,
0x69, 0x74, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6b, 0x69, 0x6e, 0xeb, 0x73,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x7a, 0x69, 0x73, 0xeb, 0x20, 0x51, 0x65, 0x6e, 0x64, 0x72, 0x6f, 0x72,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e,
0x65, 0x7a, 0x69, 0x73, 0xeb, 0x20, 0x4c, 0x69, 0x6e, 0x64, 0x6f, 0x72,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e,
0x65, 0x7a, 0x69, 0x73, 0xeb, 0x20, 0x50, 0x65, 0x72, 0xeb, 0x6e, 0x64,
0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x49,
0x72, 0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x49, 0x72, 0x61,
0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72,
0x65, 0x20, 0x65, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x75,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x65, 0x20, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65,
0x20, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x75, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c,
0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c,
0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65,
0x20, 0x65, 0x20, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x69, 0x73,
0xeb, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x65, 0x20, 0x65, 0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x69, 0x73,
0xeb, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20,
0x65, 0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x69, 0x73, 0xeb, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76,
0x6c, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0xe7, 0x61,
0x74, 0x6b, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72,
0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b,
0x61, 0x6d, 0xe7, 0x61, 0x74, 0x6b, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20,
0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x50, 0x65, 0x74,
0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d,
0x4b, 0x61, 0x6d, 0xe7, 0x61, 0x74, 0x6b, 0xeb, 0x73, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4b, 0x61, 0x7a,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x74, 0x20, 0x4c, 0x69,
0x6e, 0x64, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4b, 0x61,
0x7a, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x74, 0x20, 0x50,
0x65, 0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x4f, 0x72, 0x61,
0x20, 0x6b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x6b, 0x6f,
0x72, 0x65, 0x61, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72,
0x6f, 0x72, 0x65, 0x20, 0x6b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0xeb, 0x73, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a,
0x61, 0x72, 0x73, 0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x75, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x75,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4b, 0x69, 0x72, 0x67, 0x69,
0x7a, 0x69, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4c, 0x61,
0x6e, 0x6b, 0x61, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x49,
0x73, 0x68, 0x75, 0x6a, 0x76, 0x65, 0x20, 0x53, 0x70, 0x6f, 0x72, 0x61,
0x64, 0x69, 0x6b, 0xeb, 0x20, 0x45, 0x6b, 0x75, 0x61, 0x74, 0x6f, 0x72,
0x69, 0x61, 0x6c, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4c, 0x6f,
0x72, 0x64, 0x2d, 0x48, 0x6f, 0x75, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48, 0x6f, 0x75, 0x69, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4c,
0x6f, 0x72, 0x64, 0x2d, 0x48, 0x6f, 0x75, 0x69, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x4d, 0x61, 0x6b, 0x61, 0x6f, 0x73, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65,
0x20, 0x4d, 0x61, 0x6b, 0x61, 0x6f, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x76,
0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x61, 0x6b, 0x61,
0x6f, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65,
0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4d, 0x61,
0x6c, 0x61, 0x6a, 0x7a, 0x69, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x76, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6a, 0x76, 0x65, 0x20,
0x4d, 0x61, 0x72, 0x6b, 0x65, 0x7a, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x49, 0x73, 0x68, 0x75, 0x6a, 0x76, 0x65, 0x20, 0x4d, 0x61, 0x72,
0x73, 0x68, 0x61, 0x6c, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x69, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20,
0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x69,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20,
0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x69,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x73, 0x6f,
0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x54, 0x65, 0x72,
0x72, 0x69, 0x74, 0x6f, 0x72, 0x65, 0x76, 0x65, 0x20, 0x4d, 0x65, 0x6b,
0x73, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x20, 0x74, 0xeb, 0x20, 0x42, 0x72,
0x65, 0x67, 0x75, 0x74, 0x20, 0x74, 0xeb, 0x20, 0x50, 0x61, 0x71, 0xeb,
0x73, 0x6f, 0x72, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x54, 0x65, 0x72,
0x72, 0x69, 0x74, 0x6f, 0x72, 0x65, 0x76, 0x65, 0x20, 0x4d, 0x65, 0x6b,
0x73, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x20, 0x74, 0xeb, 0x20, 0x42, 0x72,
0x65, 0x67, 0x75, 0x74, 0x20, 0x74, 0xeb, 0x20, 0x50, 0x61, 0x71, 0xeb,
0x73, 0x6f, 0x72, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72,
0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x69, 0x74,
0x6f, 0x72, 0x65, 0x76, 0x65, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b,
0x61, 0x6e, 0x65, 0x20, 0x74, 0xeb, 0x20, 0x42, 0x72, 0x65, 0x67, 0x75,
0x74, 0x20, 0x74, 0xeb, 0x20, 0x50, 0x61, 0x71, 0xeb, 0x73, 0x6f, 0x72,
0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x6e,
0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20,
0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x69, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65,
0x20, 0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x69,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x6b, 0xeb,
0x73, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x6b, 0xeb, 0x73, 0x4f,
0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20,
0x4d, 0x6f, 0x73, 0x6b, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x4d, 0x69, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x69, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x73, 0xeb, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x69, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x69, 0x73, 0xeb, 0x20, 0x73, 0xeb, 0x20, 0x52, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65,
0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x73, 0xeb, 0x20,
0x73, 0xeb, 0x20, 0x52, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72,
0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x69, 0x73, 0xeb, 0x20, 0x73, 0xeb, 0x20, 0x52, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0xeb, 0x73,
0x20, 0x73, 0xeb, 0x20, 0x52, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x5a, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0xeb, 0x73, 0x20, 0x73, 0xeb, 0x20, 0x52, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65,
0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0xeb, 0x73, 0x20, 0x73, 0xeb,
0x20, 0x52, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4e, 0x6a, 0x75,
0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x69, 0x74, 0x20,
0x5b, 0x54, 0x6f, 0x6b, 0xeb, 0x73, 0x20, 0x73, 0xeb, 0x20, 0x52, 0x65,
0x5d, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x65, 0x20, 0x65, 0x20, 0x4e, 0x6a, 0x75, 0x66, 0x61, 0x75, 0x6e,
0x64, 0x6c, 0x65, 0x6e, 0x64, 0x69, 0x74, 0x20, 0x5b, 0x54, 0x6f, 0x6b,
0xeb, 0x73, 0x20, 0x73, 0xeb, 0x20, 0x52, 0x65, 0x5d, 0x4f, 0x72, 0x61,
0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4e, 0x6a,
0x75, 0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x69, 0x74,
0x20, 0x5b, 0x54, 0x6f, 0x6b, 0xeb, 0x73, 0x20, 0x73, 0xeb, 0x20, 0x52,
0x65, 0x5d, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4e, 0x69, 0x75, 0x65,
0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75,
0x6c, 0x6c, 0x69, 0x74, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x65, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6c, 0x6c, 0x69, 0x74,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x4f, 0x72, 0x61, 0x20,
0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68,
0x75, 0x6c, 0x6c, 0x69, 0x74, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x2d, 0x64, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x6a, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x2d, 0x64, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x6a, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72,
0x65, 0x20, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x2d, 0x64, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x6a, 0xeb, 0x73,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6a, 0x76,
0x65, 0x20, 0x74, 0xeb, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x6e, 0xeb,
0x73, 0x20, 0x73, 0xeb, 0x20, 0x56, 0x65, 0x72, 0x69, 0x75, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x75, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x75,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x75,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x65, 0x20, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x75, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20,
0x4f, 0x6d, 0x73, 0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20,
0x65, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65,
0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x73, 0xeb,
0x20, 0x73, 0xeb, 0x20, 0x52, 0x65, 0x2d, 0x50, 0x61, 0x70, 0x75, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x56, 0x65, 0x72, 0x6f,
0x72, 0x65, 0x20, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x50, 0x65, 0x72, 0x75,
0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x50, 0x65, 0x72, 0x75, 0x73, 0xeb,
0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65,
0x20, 0x50, 0x65, 0x72, 0x75, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x76, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65,
0x20, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x76,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20,
0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x76, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6a, 0x76,
0x65, 0x20, 0x46, 0x65, 0x6e, 0x69, 0x6b, 0x73, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x53, 0x68, 0xeb, 0x6e, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x20,
0x64, 0x68, 0x65, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65,
0x20, 0x65, 0x20, 0x53, 0x68, 0xeb, 0x6e, 0x2d, 0x50, 0x69, 0x65, 0x72,
0x20, 0x64, 0x68, 0x65, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e,
0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65,
0x20, 0x53, 0x68, 0xeb, 0x6e, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x20, 0x64,
0x68, 0x65, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x50, 0x69, 0x74, 0x6b, 0x65, 0x72, 0x6e, 0x69,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70,
0x65, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x50, 0x65, 0x6e,
0x69, 0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4b,
0x69, 0x7a, 0x69, 0x6c, 0x6f, 0x72, 0x64, 0x69, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65,
0x20, 0x4b, 0x69, 0x7a, 0x69, 0x6c, 0x6f, 0x72, 0x64, 0x69, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20,
0x4b, 0x69, 0x7a, 0x69, 0x6c, 0x6f, 0x72, 0x64, 0x69, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x69,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x52, 0x6f, 0x64, 0x68, 0x65,
0x72, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x53, 0x61, 0x6b,
0x61, 0x6c, 0x69, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x53, 0x61,
0x6b, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76,
0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x61,
0x6c, 0x69, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x53,
0x61, 0x6d, 0x61, 0x72, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x53, 0x61,
0x6d, 0x61, 0x72, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72,
0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0xeb,
0x73, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x73, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x65, 0x20, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x73, 0x4f,
0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x53, 0x65, 0x6a, 0x73, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x76, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f,
0x72, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68,
0x75, 0x6a, 0x76, 0x65, 0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x58, 0x68, 0x6f, 0x72, 0x78, 0x68,
0x61, 0x73, 0x20, 0x74, 0xeb, 0x20, 0x4a, 0x75, 0x67, 0x75, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d,
0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x53, 0x6a, 0x6f, 0x75,
0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x54, 0x61, 0x68, 0x69,
0x74, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x54, 0x61, 0x6a,
0x70, 0x65, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x54, 0x61, 0x6a, 0x70,
0x65, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72,
0x65, 0x20, 0x65, 0x20, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x69, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x54, 0x61, 0x78, 0x68, 0x69, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20,
0x54, 0x6f, 0x6e, 0x67, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65,
0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0xeb,
0x73, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0xc7, 0x75, 0x6b, 0x75, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65,
0x20, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x54,
0x75, 0x76, 0x61, 0x6c, 0x75, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x69, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65,
0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x69, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x65, 0x20, 0x65, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f,
0x72, 0x65, 0x20, 0x65, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x73, 0xeb, 0x4f, 0x72, 0x61,
0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0xeb, 0x73, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x75, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20,
0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x75,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x69, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x69, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x75, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6c, 0x6c, 0x69,
0x74, 0x20, 0x55, 0x65, 0x6a, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x55, 0x6f, 0x6c, 0x6c, 0x69, 0x73, 0x69, 0x74, 0x20, 0x64, 0x68, 0x65,
0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x75, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65,
0x20, 0x65, 0x20, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x75, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65,
0x20, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x75, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x45, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x75, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65,
0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4a, 0x75, 0x6b,
0x6f, 0x6e, 0x69, 0x74, 0x12e8, 0x12a0, 0x134d, 0x130b, 0x1292, 0x1235, 0x1273, 0x1295,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1218, 0x12ab, 0x12a8, 0x1208, 0x129b, 0x12cd, 0x20,
0x12a0, 0x134d, 0x122a, 0x12ab, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x1235, 0x122b,
0x1245, 0x20, 0x12a0, 0x134d, 0x122a, 0x12ab, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12f0,
0x1261, 0x1265, 0x20, 0x12a0, 0x134d, 0x122a, 0x12ab, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x12d5, 0x122b, 0x1265, 0x20, 0x12a0, 0x134d,
0x122a, 0x12ab, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x12d5, 0x122b, 0x1265, 0x20,
0x12a0, 0x134d, 0x122a, 0x12ab, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x121d, 0x12d5, 0x122b, 0x1265, 0x20, 0x12a0, 0x134d, 0x122a, 0x12ab, 0x20,
0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a0, 0x120b, 0x1235,
0x12ab, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8,
0x12a0, 0x120b, 0x1235, 0x12ab, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12e8, 0x1230,
0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x120b, 0x1235,
0x12ab, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x121b, 0x12de, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275,
0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x121b, 0x12de, 0x1295, 0x20,
0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323,
0x1320, 0x122d, 0x12e8, 0x12a0, 0x121b, 0x12de, 0x1295, 0x20, 0x12e8, 0x1240, 0x1295, 0x20,
0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1230, 0x121c,
0x1295, 0x20, 0x12a0, 0x121c, 0x122a, 0x12ab, 0x20, 0x12e8, 0x1218, 0x12ab, 0x12a8, 0x1208,
0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8,
0x1230, 0x121c, 0x1295, 0x20, 0x12a0, 0x121c, 0x122a, 0x12ab, 0x20, 0x12e8, 0x1218, 0x12ab,
0x12a8, 0x1208, 0x129b, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275,
0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1218, 0x12ab, 0x12a8, 0x1208, 0x129b,
0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323,
0x1320, 0x122d, 0x121d, 0x1235, 0x122b, 0x1243, 0x12ca, 0x20, 0x1230, 0x12d3, 0x1275, 0x20,
0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x121d, 0x1235, 0x122b, 0x1243, 0x12ca, 0x20, 0x1218,
0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320,
0x122d, 0x121d, 0x1235, 0x122b, 0x1243, 0x12ca, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1230,
0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1270, 0x122b, 0x122b,
0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8,
0x1270, 0x122b, 0x122b, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12e8, 0x1230, 0x12d3,
0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1270, 0x122b, 0x122b, 0x20,
0x12e8, 0x1240, 0x1295, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d,
0x12e8, 0x1353, 0x1235, 0x134a, 0x12ad, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x1353, 0x1235, 0x134a, 0x12ad, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8,
0x1353, 0x1235, 0x134a, 0x12ad, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275,
0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x1293, 0x12f5, 0x12ed, 0x122d,
0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0,
0x1293, 0x12f2, 0x12ed, 0x122d, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323,
0x1320, 0x122d, 0x12e8, 0x12a0, 0x1293, 0x12f5, 0x12ed, 0x122d, 0x20, 0x12e8, 0x1260, 0x130b,
0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8,
0x12a0, 0x1352, 0x12eb, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a0, 0x1352, 0x12eb, 0x20,
0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a0, 0x1352, 0x12eb,
0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x130a, 0x12dc, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x12d3, 0x1228, 0x1262, 0x12eb, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12d3, 0x1228, 0x1262,
0x12eb, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12d3,
0x1228, 0x1262, 0x12eb, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1265, 0x122d, 0x1203, 0x1295,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a0, 0x122d, 0x1300, 0x1295, 0x1272, 0x1293, 0x20,
0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0,
0x122d, 0x1300, 0x1295, 0x1272, 0x1293, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230,
0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x122d, 0x1300,
0x1295, 0x1272, 0x1293, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20,
0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x122d, 0x1300, 0x1295, 0x1272, 0x1293,
0x20, 0x121d, 0x1235, 0x122b, 0x1243, 0x12ca, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0,
0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20, 0x12a0,
0x122d, 0x1300, 0x1295, 0x1272, 0x1293, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230,
0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x121d, 0x12d5, 0x122b,
0x1263, 0x12ca, 0x20, 0x12a0, 0x122d, 0x1300, 0x1295, 0x1272, 0x1293, 0x20, 0x12e8, 0x1260,
0x130b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8,
0x12a0, 0x122d, 0x1218, 0x1292, 0x12eb, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a0, 0x122d,
0x1218, 0x1292, 0x12eb, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x12a0, 0x122d, 0x1218, 0x1292, 0x12eb, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a0, 0x1275, 0x120b, 0x1295, 0x1272, 0x12ad, 0x20, 0x12e8,
0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x1275,
0x120b, 0x1295, 0x1272, 0x12ad, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12e8, 0x1230,
0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x1275, 0x120b,
0x1295, 0x1272, 0x12ad, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x20,
0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1218, 0x12ab, 0x12a8, 0x1208, 0x129b, 0x12cd,
0x20, 0x12a0, 0x12cd, 0x1235, 0x1275, 0x122b, 0x120a, 0x12eb, 0x20, 0x1230, 0x12d3, 0x1275,
0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x12cd, 0x1235, 0x1275, 0x122b,
0x120a, 0x12eb, 0x20, 0x1218, 0x12ab, 0x12a8, 0x1208, 0x129b, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d,
0x12e8, 0x12a0, 0x12cd, 0x1235, 0x1275, 0x122b, 0x120a, 0x12eb, 0x20, 0x1218, 0x12ab, 0x12a8,
0x1208, 0x129b, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0,
0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x12cd, 0x1235, 0x1275, 0x122b, 0x120a, 0x12eb,
0x20, 0x1218, 0x12ab, 0x12a8, 0x1208, 0x129b, 0x20, 0x121d, 0x1235, 0x122b, 0x1243, 0x12ca,
0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0,
0x12cd, 0x1235, 0x1275, 0x122b, 0x120a, 0x12eb, 0x20, 0x1218, 0x12ab, 0x12a8, 0x1208, 0x129b,
0x20, 0x121d, 0x1235, 0x122b, 0x1243, 0x12ca, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x12cd,
0x1235, 0x1275, 0x122b, 0x120a, 0x12eb, 0x20, 0x1218, 0x12ab, 0x12a8, 0x1208, 0x129b, 0x12cd,
0x20, 0x121d, 0x1235, 0x122b, 0x1245, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1230, 0x12d3,
0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x121d, 0x12d5, 0x122b, 0x1263,
0x12ca, 0x20, 0x12a0, 0x12cd, 0x1235, 0x1275, 0x122b, 0x120a, 0x12eb, 0x20, 0x12e8, 0x1230,
0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x12cd, 0x1235,
0x1275, 0x122b, 0x120a, 0x12eb, 0x20, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20, 0x1218,
0x12f0, 0x1260, 0x129b, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323,
0x1320, 0x122d, 0x12e8, 0x12a0, 0x12cd, 0x1235, 0x1275, 0x122b, 0x120a, 0x12eb, 0x20, 0x121d,
0x12d5, 0x122b, 0x1263, 0x12ca, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275,
0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x121d, 0x1235, 0x122b, 0x1243, 0x12ca,
0x20, 0x12a0, 0x12cd, 0x1235, 0x1275, 0x122b, 0x120a, 0x12eb, 0x20, 0x1230, 0x12d3, 0x1275,
0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x12cd, 0x1235, 0x1275, 0x122b,
0x120a, 0x12eb, 0x20, 0x121d, 0x1235, 0x122b, 0x1243, 0x12ca, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8,
0x12a0, 0x12cd, 0x1235, 0x1275, 0x122b, 0x120a, 0x12eb, 0x20, 0x121d, 0x1235, 0x122b, 0x1243,
0x12ca, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x12d8, 0x122d, 0x1263, 0x1303, 0x1295, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x12a0, 0x12d8, 0x122d, 0x1263, 0x1303, 0x1295, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a0, 0x12d8, 0x122d, 0x1263, 0x1303,
0x1295, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a0,
0x12de, 0x1228, 0x1235, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a0, 0x12de, 0x1228, 0x1235,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a0, 0x12de,
0x1228, 0x1235, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x1263, 0x1295, 0x130d, 0x120b, 0x12f4, 0x123d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1263,
0x1295, 0x130d, 0x120b, 0x12f4, 0x123d, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x1263, 0x1295, 0x130d, 0x120b, 0x12f4, 0x123d, 0x20, 0x12ad, 0x1228,
0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1261, 0x1273, 0x1295, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x1266, 0x120a, 0x126a, 0x12eb, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x1265, 0x122b, 0x12da, 0x120b, 0x12ca, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x1265, 0x122b, 0x1232, 0x120a, 0x12eb, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d,
0x12e8, 0x1265, 0x122b, 0x12da, 0x120b, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3,
0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1265, 0x1229, 0x1294, 0x12ed,
0x20, 0x12f3, 0x1229, 0x1233, 0x120b, 0x121d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12ac,
0x1355, 0x20, 0x1268, 0x122d, 0x12f4, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12ac, 0x1355,
0x20, 0x1268, 0x122d, 0x12f4, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x12ac, 0x1355, 0x20, 0x1268, 0x122d, 0x12f4, 0x20, 0x12ad, 0x1228, 0x121d,
0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x127b, 0x121e, 0x122e, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x127b, 0x1273, 0x121d, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x127b, 0x1273, 0x121d, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x127b, 0x1273, 0x121d, 0x20, 0x12e8, 0x1240, 0x1295, 0x20,
0x1265, 0x122d, 0x1203, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x127a, 0x120a, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x127a, 0x120a, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x127a, 0x120a, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x127b, 0x12ed, 0x1293, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x127b, 0x12ed, 0x1293, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x127b, 0x12ed, 0x1293, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1265, 0x122d, 0x1203,
0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1308, 0x1293, 0x20, 0x12f0, 0x1234, 0x1275,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12ae, 0x12ae, 0x1235, 0x20, 0x12f0, 0x1234, 0x1276,
0x127d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12ae, 0x120e, 0x121d, 0x1262, 0x12eb, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x12ae, 0x120e, 0x121d, 0x1262, 0x12eb, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12ae, 0x120e, 0x121d, 0x1262, 0x12eb,
0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a9, 0x12ad,
0x20, 0x12f0, 0x1234, 0x1276, 0x127d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a9, 0x12ad,
0x20, 0x12f0, 0x1234, 0x1276, 0x127d, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x12a9, 0x12ad, 0x20, 0x12f0, 0x1234, 0x1276, 0x127d, 0x20, 0x130d,
0x121b, 0x123d, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12a9,
0x1263, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a9, 0x1263, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a9, 0x1263, 0x20, 0x12e8, 0x1240, 0x1295,
0x20, 0x1265, 0x122d, 0x1203, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12f4, 0x126a,
0x1235, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12f1, 0x121e, 0x1295, 0x1275, 0x2d, 0x12f1,
0x122d, 0x126a, 0x120d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x1235, 0x122b, 0x1245,
0x20, 0x1272, 0x121e, 0x122d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a2, 0x1235, 0x1270,
0x122d, 0x20, 0x12f0, 0x1234, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a2, 0x1235,
0x1270, 0x122d, 0x20, 0x12f0, 0x1234, 0x1275, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a2, 0x1235, 0x1270, 0x122d, 0x20, 0x12f0, 0x1234, 0x1275,
0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a2, 0x12b3,
0x12f6, 0x122d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1218, 0x12ab, 0x12a8, 0x1208, 0x129b,
0x12cd, 0x20, 0x12a0, 0x12cd, 0x122e, 0x1353, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1218,
0x12ab, 0x12a8, 0x1208, 0x129b, 0x12cd, 0x20, 0x12a0, 0x12cd, 0x122e, 0x1353, 0x20, 0x1218,
0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1218, 0x12ab, 0x12a8, 0x1208,
0x129b, 0x12cd, 0x20, 0x12a0, 0x12cd, 0x122e, 0x1353, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x1235, 0x122b, 0x1243, 0x12ca, 0x20, 0x12a0,
0x12cd, 0x122e, 0x1353, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x1235, 0x122b, 0x1243,
0x12ca, 0x20, 0x12a0, 0x12cd, 0x122e, 0x1353, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x1235, 0x122b, 0x1243, 0x12ca, 0x20, 0x12a0, 0x12cd,
0x122e, 0x1353, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x1229, 0x1245, 0x20, 0x121d, 0x1235, 0x122b, 0x1245, 0x20, 0x12e8, 0x12a0, 0x12cd, 0x122e,
0x1353, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20,
0x12a0, 0x12cd, 0x122e, 0x1353, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x12d5, 0x122b,
0x1263, 0x12ca, 0x20, 0x12a0, 0x12cd, 0x122e, 0x1353, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20, 0x12a0,
0x12cd, 0x122e, 0x1353, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x134b, 0x120d, 0x12ad, 0x120b, 0x1295, 0x12f5, 0x20, 0x12f0, 0x1234, 0x1276, 0x127d,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x134b, 0x120d, 0x12ad, 0x120b, 0x1295, 0x12f5, 0x20,
0x12f0, 0x1234, 0x1276, 0x127d, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x134b, 0x120d, 0x12ad, 0x120b, 0x1295, 0x12f5, 0x20, 0x12f0, 0x1234, 0x1276,
0x127d, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x134a,
0x1302, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x134a, 0x1302, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x134a, 0x1302, 0x20, 0x12ad, 0x1228, 0x121d,
0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1348, 0x1228, 0x1295, 0x1233, 0x12ed, 0x20,
0x1309, 0x12eb, 0x1293, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1348, 0x1228, 0x1295, 0x1233,
0x12ed, 0x20, 0x12f0, 0x1261, 0x1263, 0x12ca, 0x20, 0x12a5, 0x1293, 0x20, 0x12a0, 0x1295,
0x1273, 0x122d, 0x12ad, 0x1272, 0x12ad, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x130b, 0x120b,
0x1353, 0x130e, 0x1235, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x130b, 0x121d, 0x1262, 0x12e8,
0x122d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1302, 0x12ee, 0x122d, 0x1302, 0x12eb, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x1302, 0x12ee, 0x122d, 0x1302, 0x12eb, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1302, 0x12ee, 0x122d, 0x1302, 0x12eb,
0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1302, 0x120d,
0x1260, 0x122d, 0x1275, 0x20, 0x12f0, 0x1234, 0x1276, 0x127d, 0x20, 0x1230, 0x12d3, 0x1275,
0x130d, 0x122a, 0x1295, 0x12ca, 0x127d, 0x20, 0x121b, 0x12d5, 0x12a8, 0x120b, 0x12ca, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x1235, 0x122b, 0x1245, 0x20, 0x130d, 0x122a, 0x1295,
0x120b, 0x1295, 0x12f5, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x1235, 0x122b, 0x1245,
0x20, 0x130d, 0x122a, 0x1295, 0x120b, 0x1295, 0x12f5, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x1235, 0x122b, 0x1245, 0x20, 0x130d, 0x122a,
0x1295, 0x120b, 0x1295, 0x12f5, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x121d, 0x12d5, 0x122b, 0x1265, 0x20, 0x130d, 0x122a, 0x1295, 0x120b, 0x1295,
0x12f5, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x12d5, 0x122b, 0x1265, 0x20, 0x130d,
0x122a, 0x1295, 0x120b, 0x1295, 0x12f5, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x121d, 0x12d5, 0x122b, 0x1265, 0x20, 0x130d, 0x122a, 0x1295, 0x120b,
0x1295, 0x12f5, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x1263, 0x1205, 0x1228, 0x1230, 0x120b, 0x1324, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x1309, 0x12eb, 0x1293, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x1203, 0x12cb, 0x12ed, 0x20, 0x12a0, 0x120c, 0x12a1, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275,
0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1203, 0x12cb, 0x12ed, 0x20, 0x12a0,
0x120c, 0x12a1, 0x1275, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275,
0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1203, 0x12cb, 0x12ed, 0x20, 0x12a0,
0x120c, 0x12a1, 0x1275, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x20,
0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1206, 0x1295, 0x130d, 0x20, 0x12ae, 0x1295,
0x130d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1206, 0x1295, 0x130d, 0x20, 0x12ae, 0x1295,
0x130d, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1206,
0x1295, 0x130d, 0x20, 0x12ae, 0x1295, 0x130d, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x1206, 0x126d, 0x12f5, 0x20, 0x1230, 0x12d3, 0x1275, 0x20,
0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1206, 0x126d, 0x12f5, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320,
0x122d, 0x12e8, 0x1206, 0x126d, 0x12f5, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3,
0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1205, 0x1295, 0x12f5, 0x20,
0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1205, 0x1295, 0x12f5,
0x20, 0x12cd, 0x1245, 0x12eb, 0x1296, 0x1235, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a2,
0x1295, 0x12f6, 0x127b, 0x12ed, 0x1293, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1218, 0x12ab,
0x12a8, 0x1208, 0x129b, 0x12cd, 0x20, 0x12a2, 0x1295, 0x12f6, 0x1294, 0x12e2, 0x12eb, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x1235, 0x122b, 0x1243, 0x12ca, 0x20, 0x12a2, 0x1295,
0x12f6, 0x1294, 0x12e2, 0x12eb, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x12d5, 0x122b,
0x1263, 0x12ca, 0x20, 0x12a2, 0x1295, 0x12f6, 0x1294, 0x12e2, 0x12eb, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x12a2, 0x122b, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a2, 0x122b,
0x1295, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a2,
0x122b, 0x1295, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1265, 0x122d, 0x1203, 0x1295, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a2, 0x122d, 0x12a9, 0x1275, 0x1235, 0x12ad, 0x20, 0x1230,
0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a2, 0x122d, 0x12a9,
0x1275, 0x1235, 0x12ad, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12e8, 0x1230, 0x12d3,
0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12a2, 0x122d, 0x12a9, 0x1275, 0x1235,
0x12ad, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0,
0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a5, 0x1235, 0x122b, 0x12a4, 0x120d, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x12a5, 0x1235, 0x122b, 0x12a4, 0x120d, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a5, 0x1235, 0x122b, 0x12a4, 0x120d, 0x20,
0x12e8, 0x1240, 0x1295, 0x20, 0x1265, 0x122d, 0x1203, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x1303, 0x1353, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1303, 0x1353, 0x1295,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1303, 0x1353,
0x1295, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1265, 0x122d, 0x1203, 0x1295, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x12ab, 0x121d, 0x127b, 0x1275, 0x12ab, 0x20, 0x1230, 0x12d3, 0x1275,
0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1354, 0x1275, 0x122e, 0x1353, 0x126d,
0x120e, 0x1235, 0x12aa, 0x20, 0x2d, 0x20, 0x12ab, 0x121d, 0x127b, 0x1275, 0x1235, 0x12aa,
0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1354,
0x1275, 0x122e, 0x1353, 0x126d, 0x120e, 0x1235, 0x12aa, 0x20, 0x2d, 0x20, 0x12ab, 0x121d,
0x127b, 0x1275, 0x1235, 0x12aa, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275,
0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12ab, 0x12db, 0x12aa, 0x1235, 0x1273, 0x1295,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x1235, 0x122b, 0x1245, 0x20, 0x12ab, 0x12db,
0x12aa, 0x1235, 0x1273, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x12d5, 0x122b,
0x1265, 0x20, 0x12ab, 0x12db, 0x12aa, 0x1235, 0x1273, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x12ae, 0x122a, 0x12eb, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12ae, 0x122a, 0x12eb,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12ae, 0x122a,
0x12eb, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1265, 0x122d, 0x1203, 0x1295, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x12ae, 0x1235, 0x122b, 0x12a4, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x12ad, 0x122b, 0x1235, 0x1296, 0x12eb, 0x122d, 0x1235, 0x12ad, 0x20, 0x1230, 0x12d3, 0x1275,
0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12ad, 0x122b, 0x1235, 0x1296, 0x12ed,
0x12a0, 0x122d, 0x1235, 0x12ad, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3,
0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12ad, 0x122b, 0x1235, 0x1296,
0x12eb, 0x122d, 0x1235, 0x12ad, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275,
0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12aa, 0x122d, 0x130a, 0x1235, 0x1273,
0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x120b, 0x12ed, 0x1295, 0x20, 0x12f0, 0x1234,
0x1276, 0x127d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x120e, 0x122d, 0x12f5, 0x20, 0x1206,
0x12cc, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d,
0x12e8, 0x120e, 0x122d, 0x12f5, 0x20, 0x1206, 0x12cc, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8,
0x120e, 0x122d, 0x12f5, 0x20, 0x1206, 0x12cc, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1230,
0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x121b, 0x130b, 0x12f3,
0x1295, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d,
0x12e8, 0x121b, 0x130b, 0x12f3, 0x1295, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230,
0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x121b, 0x130b, 0x12f3,
0x1295, 0x20, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323,
0x1320, 0x122d, 0x12e8, 0x121b, 0x120c, 0x12ed, 0x12e2, 0x12eb, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x121b, 0x120d, 0x12f2, 0x126d, 0x1235, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121b,
0x122d, 0x12b4, 0x1233, 0x1235, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121b, 0x122d, 0x123b,
0x120d, 0x20, 0x12f0, 0x1234, 0x1276, 0x127d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121b,
0x12cd, 0x122a, 0x123a, 0x12e8, 0x1235, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121b, 0x12cd,
0x122a, 0x123a, 0x12e8, 0x1235, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x121b, 0x12cd, 0x122a, 0x123a, 0x12e8, 0x1235, 0x20, 0x12ad, 0x1228, 0x121d,
0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121b, 0x12cd, 0x1230, 0x1295, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x121c, 0x12ad, 0x1232, 0x12ae, 0x20, 0x1353, 0x1232, 0x134a, 0x12ad,
0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x121c,
0x12ad, 0x1232, 0x12ae, 0x20, 0x1353, 0x1232, 0x134a, 0x12ad, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8,
0x121c, 0x12ad, 0x1232, 0x12ae, 0x20, 0x1353, 0x1232, 0x134a, 0x12ad, 0x20, 0x12e8, 0x1240,
0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8,
0x12a1, 0x120b, 0x1295, 0x20, 0x1263, 0x1276, 0x122d, 0x20, 0x130a, 0x12dc, 0x12e8, 0x12a1,
0x120b, 0x1295, 0x20, 0x1263, 0x1276, 0x122d, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a1, 0x120b,
0x1295, 0x20, 0x1263, 0x1276, 0x122d, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3,
0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x121e, 0x1235, 0x12ae, 0x20,
0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x121e, 0x1235,
0x12ae, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0,
0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x121e, 0x1235, 0x12ae, 0x20, 0x12e8, 0x1260, 0x130b,
0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x121a,
0x12eb, 0x1295, 0x121b, 0x122d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1293, 0x12cd, 0x1229,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1294, 0x1353, 0x120d, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x1292, 0x12cd, 0x20, 0x12ab, 0x120c, 0x12f6, 0x1292, 0x12eb, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x1292, 0x12cd, 0x20, 0x12ab, 0x120c, 0x12f6, 0x1292, 0x12eb, 0x20, 0x1218,
0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1292, 0x12cd, 0x20, 0x12ab,
0x120c, 0x12f6, 0x1292, 0x12eb, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x1292, 0x12cd, 0x20, 0x12da, 0x120b, 0x1295, 0x12f5, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x1292, 0x12cd, 0x20, 0x12da, 0x120b, 0x1295, 0x12f5, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1292, 0x12cd, 0x20, 0x12da, 0x120b,
0x1295, 0x12f5, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1265, 0x122d, 0x1203, 0x1295, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x1292, 0x12cd, 0x134b, 0x12cd, 0x1295, 0x12f5, 0x120b, 0x1295,
0x12f5, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d,
0x12e8, 0x1292, 0x12cd, 0x134b, 0x12cd, 0x1295, 0x12f5, 0x120b, 0x1295, 0x12f5, 0x20, 0x1218,
0x12f0, 0x1260, 0x129b, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323,
0x1320, 0x122d, 0x12e8, 0x1292, 0x12cd, 0x134b, 0x12cd, 0x1295, 0x12f5, 0x120b, 0x1295, 0x12f5,
0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x1292, 0x12e9, 0x12cc, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x1296, 0x122d, 0x134e, 0x120d, 0x12ad, 0x20, 0x12f0, 0x1234, 0x1276, 0x127d, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x1296, 0x122d, 0x134e, 0x120d, 0x12ad, 0x20, 0x12f0, 0x1234, 0x1276,
0x127d, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20,
0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1296, 0x122d, 0x134e, 0x120d, 0x12ad, 0x20,
0x12f0, 0x1234, 0x1276, 0x127d, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x12e8, 0x1230, 0x12d3,
0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1296, 0x122e, 0x1295, 0x1203,
0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1348,
0x122d, 0x1293, 0x1295, 0x12f6, 0x20, 0x12f2, 0x20, 0x1296, 0x122e, 0x1295, 0x127b, 0x20,
0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323,
0x1320, 0x122d, 0x12e8, 0x1348, 0x122d, 0x1293, 0x1295, 0x12f6, 0x20, 0x12f2, 0x20, 0x1296,
0x122e, 0x1295, 0x1203, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275,
0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1296, 0x126e, 0x1232, 0x1265, 0x122a,
0x1235, 0x12ad, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320,
0x122d, 0x12e8, 0x1296, 0x126e, 0x1232, 0x1262, 0x122d, 0x1235, 0x12ad, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320,
0x122d, 0x12e8, 0x1296, 0x126e, 0x1232, 0x1265, 0x122a, 0x1235, 0x12ad, 0x20, 0x12e8, 0x1260,
0x130b, 0x20, 0x1230, 0x12a0, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8,
0x12a6, 0x121d, 0x1235, 0x12ad, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x12a6, 0x121d, 0x1235, 0x12ad, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8,
0x12a6, 0x121d, 0x1235, 0x12ad, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275,
0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1353, 0x12aa, 0x1235, 0x1273, 0x1295,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1353, 0x12aa, 0x1235, 0x1273, 0x1295, 0x20, 0x1218,
0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1353, 0x12aa, 0x1235, 0x1273,
0x1295, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1353,
0x120b, 0x12cd, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1353, 0x1357, 0x20, 0x1292, 0x12cd,
0x20, 0x130a, 0x1292, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1353, 0x122b, 0x1313, 0x12ed,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1353, 0x122b, 0x1313, 0x12ed, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1353, 0x122b, 0x1313, 0x12ed, 0x20,
0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1354, 0x1229, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x1354, 0x1229, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x1354, 0x1229, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x134a, 0x120a, 0x1352, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x134a, 0x120a, 0x1352, 0x1295, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x134a, 0x120a, 0x1352, 0x1295, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x134a, 0x1292, 0x12ad, 0x1235, 0x20, 0x12f0, 0x1234,
0x1276, 0x127d, 0x20, 0x1230, 0x12d3, 0x1275, 0x1245, 0x12f1, 0x1235, 0x20, 0x12e8, 0x1352,
0x12ec, 0x122d, 0x20, 0x12a5, 0x1293, 0x20, 0x121a, 0x12b4, 0x120e, 0x1295, 0x20, 0x1230,
0x12d3, 0x1275, 0x1245, 0x12f1, 0x1235, 0x20, 0x12e8, 0x1352, 0x12ec, 0x122d, 0x20, 0x12a5,
0x1293, 0x20, 0x121a, 0x12b4, 0x120e, 0x1295, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x1230, 0x12d3, 0x1275, 0x1245, 0x12f1, 0x1235, 0x20, 0x12e8, 0x1352, 0x12ec, 0x122d, 0x20,
0x12a5, 0x1293, 0x20, 0x121a, 0x12b4, 0x120e, 0x1295, 0x20, 0x12e8, 0x1240, 0x1295, 0x20,
0x1265, 0x122d, 0x1203, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1352, 0x1275, 0x12ab,
0x12ed, 0x122d, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1356, 0x1293, 0x1354, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x1355, 0x12ee, 0x1295, 0x130d, 0x12eb, 0x1295, 0x130d, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x122c, 0x12e9, 0x1292, 0x12e8, 0x1295, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x122e, 0x1274, 0x122b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1233, 0x12ad,
0x1203, 0x120a, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320,
0x122d, 0x12e8, 0x1233, 0x12ad, 0x1203, 0x120a, 0x1295, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1233,
0x12ad, 0x1203, 0x120a, 0x1295, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275,
0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1233, 0x121b, 0x122b, 0x20, 0x1230,
0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1233, 0x121b, 0x122b,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x1233, 0x121b, 0x122b, 0x20, 0x12e8, 0x1260, 0x130b, 0x20,
0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1233, 0x121e,
0x12cb, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1233, 0x121e, 0x12cb, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1233, 0x121e, 0x12cb, 0x20, 0x12e8,
0x1260, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1234, 0x1238, 0x120d, 0x1235, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x1232, 0x1295, 0x130b, 0x1352, 0x122d, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1230, 0x1208, 0x121e, 0x1295, 0x20,
0x12f0, 0x1234, 0x1276, 0x127d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12f0, 0x1261, 0x1265,
0x20, 0x1302, 0x12ee, 0x122d, 0x1302, 0x12eb, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1231,
0x122a, 0x1293, 0x121d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1232, 0x12ee, 0x12cb, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x1273, 0x1202, 0x1272, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x1273, 0x12ed, 0x1354, 0x12ed, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1273, 0x12ed, 0x1354,
0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1273,
0x12ed, 0x1354, 0x12ed, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1265, 0x122d, 0x1203, 0x1295,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1273, 0x1302, 0x12aa, 0x1235, 0x1273, 0x1295, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x1276, 0x12ac, 0x120b, 0x12cd, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x1276, 0x1295, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1276, 0x1295, 0x130b,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1276, 0x1295,
0x130b, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1279,
0x12ad, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1271, 0x122d, 0x12ad, 0x1218, 0x1292, 0x1235,
0x1273, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1271, 0x122d, 0x12ad, 0x1218, 0x1292,
0x1235, 0x1273, 0x1295, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x1271, 0x122d, 0x12ad, 0x1218, 0x1292, 0x1235, 0x1273, 0x1295, 0x20, 0x12ad, 0x1228,
0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1271, 0x126b, 0x1209, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x12a1, 0x122b, 0x1313, 0x12ed, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x12a1, 0x122b, 0x1313, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x12a1, 0x122b, 0x1313, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a1, 0x12dd, 0x1264, 0x12aa, 0x1235, 0x1273, 0x1295, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a1, 0x12dd, 0x1264, 0x12aa, 0x1235, 0x1273, 0x1295, 0x20,
0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a1, 0x12dd, 0x1264,
0x12aa, 0x1235, 0x1273, 0x1295, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x126b, 0x1297, 0x1271, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x126b, 0x1297,
0x1271, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x126b,
0x1297, 0x1271, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x126c, 0x1294, 0x12dd, 0x12cc, 0x120b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x126d, 0x120b,
0x12f2, 0x126e, 0x1235, 0x1276, 0x12ad, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0,
0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x126a, 0x120b, 0x12f2, 0x126e, 0x1235, 0x1276, 0x12ad,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0,
0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x126d, 0x120b, 0x12f2, 0x126e, 0x1235, 0x1276, 0x12ad,
0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x126e, 0x120d, 0x130e, 0x122b, 0x12f5, 0x20, 0x12e8, 0x1230,
0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x126e, 0x120d, 0x130e,
0x122b, 0x12f5, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20,
0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x126b, 0x120d, 0x130e, 0x122b, 0x12f5, 0x20,
0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320,
0x122d, 0x12e8, 0x126e, 0x1235, 0x1276, 0x12ad, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12cc,
0x12ad, 0x20, 0x12f0, 0x1234, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12cb, 0x120a,
0x1235, 0x20, 0x12a5, 0x1293, 0x20, 0x1349, 0x1271, 0x1293, 0x20, 0x1230, 0x12d3, 0x1275,
0x12eb, 0x12a9, 0x1275, 0x1235, 0x12ad, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0,
0x1246, 0x1323, 0x1320, 0x122d, 0x12eb, 0x12a9, 0x1275, 0x1235, 0x12ad, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d,
0x12e8, 0x12eb, 0x12a9, 0x1275, 0x1235, 0x12ad, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x1230,
0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12e8, 0x12ab, 0x1270,
0x122a, 0x1295, 0x1260, 0x122d, 0x130d, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x12e8, 0x12ab, 0x1270, 0x122a, 0x1295, 0x1260, 0x122d, 0x130d,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x12e8, 0x12ab, 0x1270, 0x122a, 0x1295, 0x1260, 0x122d, 0x130d,
0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323,
0x1320, 0x122d, 0x12e8, 0x12e9, 0x12ae, 0x1295, 0x20, 0x130a, 0x12dc, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x623, 0x641, 0x63a, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x648, 0x633, 0x637, 0x20, 0x623, 0x641,
0x631, 0x64a, 0x642, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x634,
0x631, 0x642, 0x20, 0x623, 0x641, 0x631, 0x64a, 0x642, 0x64a, 0x627, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x646, 0x648, 0x628, 0x20, 0x623, 0x641, 0x631,
0x64a, 0x642, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x631,
0x628, 0x20, 0x623, 0x641, 0x631, 0x64a, 0x642, 0x64a, 0x627, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x623, 0x641, 0x631, 0x64a, 0x642,
0x64a, 0x627, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x623, 0x641, 0x631, 0x64a, 0x642,
0x64a, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x623, 0x644, 0x627, 0x633, 0x643, 0x627, 0x627, 0x644, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x20,
0x644, 0x623, 0x644, 0x627, 0x633, 0x643, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x623, 0x644, 0x627, 0x633, 0x643, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a,
0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x623, 0x645,
0x627, 0x632, 0x648, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644,
0x623, 0x645, 0x627, 0x632, 0x648, 0x646, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645,
0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x623, 0x645, 0x627,
0x632, 0x648, 0x646, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x627, 0x644,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x645, 0x631, 0x643, 0x632,
0x64a, 0x20, 0x644, 0x623, 0x645, 0x631, 0x64a, 0x643, 0x627, 0x20, 0x627, 0x644,
0x634, 0x645, 0x627, 0x644, 0x64a, 0x629, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x20, 0x627, 0x644, 0x645,
0x631, 0x643, 0x632, 0x64a, 0x20, 0x644, 0x623, 0x645, 0x631, 0x64a, 0x643, 0x627,
0x20, 0x627, 0x644, 0x634, 0x645, 0x627, 0x644, 0x64a, 0x629, 0x627, 0x644, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x20,
0x627, 0x644, 0x645, 0x631, 0x643, 0x632, 0x64a, 0x20, 0x644, 0x623, 0x645, 0x631,
0x64a, 0x643, 0x627, 0x20, 0x627, 0x644, 0x634, 0x645, 0x627, 0x644, 0x64a, 0x629,
0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x634, 0x631,
0x642, 0x64a, 0x20, 0x644, 0x623, 0x645, 0x631, 0x64a, 0x643, 0x627, 0x20, 0x627,
0x644, 0x634, 0x645, 0x627, 0x644, 0x64a, 0x629, 0x627, 0x644, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x20, 0x627, 0x644,
0x634, 0x631, 0x642, 0x64a, 0x20, 0x644, 0x623, 0x645, 0x631, 0x64a, 0x643, 0x627,
0x20, 0x627, 0x644, 0x634, 0x645, 0x627, 0x644, 0x64a, 0x629, 0x627, 0x644, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x20,
0x627, 0x644, 0x634, 0x631, 0x642, 0x64a, 0x20, 0x644, 0x623, 0x645, 0x631, 0x64a,
0x643, 0x627, 0x20, 0x627, 0x644, 0x634, 0x645, 0x627, 0x644, 0x64a, 0x629, 0x627,
0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x62c, 0x628, 0x644,
0x64a, 0x20, 0x644, 0x623, 0x645, 0x631, 0x64a, 0x643, 0x627, 0x20, 0x627, 0x644,
0x634, 0x645, 0x627, 0x644, 0x64a, 0x629, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x627, 0x644, 0x62c, 0x628, 0x644, 0x64a, 0x20, 0x627, 0x644, 0x631,
0x633, 0x645, 0x64a, 0x20, 0x644, 0x623, 0x645, 0x631, 0x64a, 0x643, 0x627, 0x20,
0x627, 0x644, 0x634, 0x645, 0x627, 0x644, 0x64a, 0x629, 0x627, 0x644, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x62c, 0x628, 0x644, 0x64a, 0x20, 0x627,
0x644, 0x635, 0x64a, 0x641, 0x64a, 0x20, 0x644, 0x623, 0x645, 0x631, 0x64a, 0x643,
0x627, 0x20, 0x627, 0x644, 0x634, 0x645, 0x627, 0x644, 0x64a, 0x629, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x645, 0x62d, 0x64a, 0x637, 0x20, 0x627,
0x644, 0x647, 0x627, 0x62f, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627,
0x644, 0x645, 0x62d, 0x64a, 0x637, 0x20, 0x627, 0x644, 0x647, 0x627, 0x62f, 0x64a,
0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x627, 0x644, 0x645, 0x62d, 0x64a, 0x637, 0x20, 0x627, 0x644, 0x647, 0x627,
0x62f, 0x64a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x623, 0x646, 0x627, 0x62f, 0x64a, 0x631, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x623, 0x646, 0x627, 0x62f, 0x64a, 0x631, 0x20, 0x627, 0x644,
0x631, 0x633, 0x645, 0x64a, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x20, 0x644, 0x623, 0x646, 0x627, 0x62f,
0x64a, 0x631, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x622, 0x628, 0x64a, 0x627,
0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x631, 0x633,
0x645, 0x64a, 0x20, 0x644, 0x622, 0x628, 0x64a, 0x627, 0x627, 0x644, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x20, 0x644,
0x623, 0x628, 0x64a, 0x627, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x627, 0x644, 0x639, 0x631, 0x628, 0x64a, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x627, 0x644, 0x639, 0x631, 0x628, 0x64a, 0x20, 0x627, 0x644, 0x631,
0x633, 0x645, 0x64a, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627,
0x644, 0x639, 0x631, 0x628, 0x64a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x623, 0x631, 0x62c, 0x646,
0x62a, 0x64a, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x623,
0x631, 0x62c, 0x646, 0x62a, 0x64a, 0x646, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645,
0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x623, 0x631, 0x62c,
0x646, 0x62a, 0x64a, 0x646, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x627, 0x644, 0x623,
0x631, 0x62c, 0x646, 0x62a, 0x64a, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x63a, 0x631, 0x628, 0x20, 0x627, 0x644, 0x623, 0x631, 0x62c, 0x646, 0x62a, 0x64a,
0x646, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x627, 0x644, 0x623, 0x631, 0x62c, 0x646,
0x62a, 0x64a, 0x646, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x623, 0x631, 0x645, 0x64a, 0x646, 0x64a, 0x627, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x631, 0x645, 0x64a, 0x646, 0x64a, 0x627,
0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x623, 0x631, 0x645, 0x64a, 0x646, 0x64a, 0x627, 0x20, 0x627, 0x644, 0x635,
0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x623,
0x637, 0x644, 0x633, 0x64a, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x20, 0x627, 0x644, 0x623, 0x637, 0x644,
0x633, 0x64a, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644,
0x635, 0x64a, 0x641, 0x64a, 0x20, 0x627, 0x644, 0x623, 0x637, 0x644, 0x633, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x648, 0x633, 0x637, 0x20, 0x623, 0x633,
0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x648, 0x633, 0x637, 0x20, 0x623, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627,
0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x648, 0x633, 0x637, 0x20, 0x623, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a,
0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x648, 0x633, 0x637, 0x20, 0x623, 0x633,
0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x63a, 0x631, 0x628, 0x20, 0x648, 0x633, 0x637, 0x20, 0x623, 0x633, 0x62a, 0x631,
0x627, 0x644, 0x64a, 0x627, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x648, 0x633, 0x637,
0x20, 0x623, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x20, 0x627, 0x644,
0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x634, 0x631,
0x642, 0x20, 0x623, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x634, 0x631, 0x642, 0x20, 0x623, 0x633, 0x62a, 0x631,
0x627, 0x644, 0x64a, 0x627, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x634, 0x631, 0x642, 0x20, 0x623, 0x633, 0x62a,
0x631, 0x627, 0x644, 0x64a, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x623, 0x633,
0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x63a, 0x631, 0x628, 0x20, 0x623, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627,
0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x63a, 0x631, 0x628, 0x20, 0x623, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a,
0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x623, 0x630, 0x631, 0x628, 0x64a, 0x62c, 0x627, 0x646, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x623, 0x630, 0x631, 0x628, 0x64a, 0x62c, 0x627, 0x646,
0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x623, 0x630, 0x631, 0x628, 0x64a, 0x62c, 0x627, 0x646, 0x20, 0x627, 0x644,
0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x632,
0x648, 0x631, 0x633, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x632, 0x648,
0x631, 0x633, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x623, 0x632, 0x648, 0x631, 0x633, 0x20, 0x627, 0x644, 0x635,
0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x646, 0x63a,
0x644, 0x627, 0x62f, 0x64a, 0x634, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628,
0x646, 0x63a, 0x644, 0x627, 0x62f, 0x64a, 0x634, 0x20, 0x627, 0x644, 0x631, 0x633,
0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x646, 0x63a, 0x644,
0x627, 0x62f, 0x64a, 0x634, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x648, 0x62a, 0x627, 0x646, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x628, 0x648, 0x644, 0x64a, 0x641, 0x64a, 0x627, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x631, 0x627, 0x632, 0x64a, 0x644, 0x64a,
0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x631, 0x627, 0x632, 0x64a,
0x644, 0x64a, 0x627, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x628, 0x631, 0x627, 0x632, 0x64a, 0x644, 0x64a, 0x627,
0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x628, 0x631, 0x648, 0x646, 0x627, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x627, 0x644, 0x631, 0x623, 0x633, 0x20, 0x627, 0x644, 0x623, 0x62e, 0x636,
0x631, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x631, 0x623, 0x633,
0x20, 0x627, 0x644, 0x623, 0x62e, 0x636, 0x631, 0x20, 0x627, 0x644, 0x631, 0x633,
0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x631, 0x623,
0x633, 0x20, 0x627, 0x644, 0x623, 0x62e, 0x636, 0x631, 0x20, 0x627, 0x644, 0x635,
0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x634, 0x627,
0x645, 0x648, 0x631, 0x648, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x634,
0x627, 0x62a, 0x627, 0x645, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x634,
0x627, 0x62a, 0x627, 0x645, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x634, 0x627, 0x62a, 0x627, 0x645, 0x20,
0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x62a, 0x634, 0x64a, 0x644, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a,
0x634, 0x64a, 0x644, 0x64a, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x634, 0x64a, 0x644, 0x64a, 0x20, 0x627,
0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627,
0x644, 0x635, 0x64a, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644,
0x635, 0x64a, 0x646, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x646, 0x20, 0x627, 0x644,
0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632,
0x631, 0x20, 0x627, 0x644, 0x643, 0x631, 0x64a, 0x633, 0x645, 0x627, 0x633, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x631, 0x20, 0x643, 0x648, 0x643,
0x648, 0x633, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x643, 0x648, 0x644, 0x648,
0x645, 0x628, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x643, 0x648,
0x644, 0x648, 0x645, 0x628, 0x64a, 0x627, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645,
0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x643, 0x648, 0x644, 0x648, 0x645,
0x628, 0x64a, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x631, 0x20, 0x643, 0x648, 0x648, 0x643,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x631, 0x20, 0x643, 0x648,
0x643, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x62c, 0x632, 0x631, 0x20, 0x643, 0x648, 0x643, 0x20, 0x627, 0x644,
0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x643, 0x648,
0x628, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x643, 0x648, 0x628, 0x627,
0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x643, 0x648, 0x628, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62f, 0x627, 0x641, 0x64a, 0x632, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62f, 0x64a, 0x20, 0x645, 0x648, 0x646, 0x62a,
0x20, 0x62f, 0x648, 0x20, 0x631, 0x648, 0x641, 0x64a, 0x644, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x62a, 0x64a, 0x645, 0x648, 0x631, 0x20, 0x627, 0x644, 0x634,
0x631, 0x642, 0x64a, 0x629, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632,
0x64a, 0x631, 0x629, 0x20, 0x627, 0x633, 0x62a, 0x631, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x62c, 0x632, 0x64a, 0x631, 0x629, 0x20, 0x627, 0x633, 0x62a, 0x631,
0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x62c, 0x632, 0x64a, 0x631, 0x629, 0x20, 0x627, 0x633, 0x62a, 0x631, 0x20,
0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x627, 0x644, 0x625, 0x643, 0x648, 0x627, 0x62f, 0x648, 0x631, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x648, 0x633, 0x637, 0x20, 0x623, 0x648, 0x631, 0x648, 0x628,
0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x648, 0x633, 0x637, 0x20, 0x623,
0x648, 0x631, 0x648, 0x628, 0x627, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x648, 0x633, 0x637, 0x20, 0x623, 0x648,
0x631, 0x648, 0x628, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x634, 0x631, 0x642, 0x20, 0x623, 0x648, 0x631,
0x648, 0x628, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x634, 0x631, 0x642,
0x20, 0x623, 0x648, 0x631, 0x648, 0x628, 0x627, 0x20, 0x627, 0x644, 0x631, 0x633,
0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x634, 0x631, 0x642, 0x20,
0x623, 0x648, 0x631, 0x648, 0x628, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641,
0x64a, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x623,
0x648, 0x631, 0x648, 0x628, 0x64a, 0x20, 0x28, 0x623, 0x643, 0x62b, 0x631, 0x20,
0x634, 0x631, 0x642, 0x64b, 0x627, 0x29, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x63a, 0x631, 0x628, 0x20, 0x623, 0x648, 0x631, 0x648, 0x628, 0x627, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x623, 0x648, 0x631, 0x648,
0x628, 0x627, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x623, 0x648, 0x631, 0x648, 0x628,
0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x62c, 0x632, 0x631, 0x20, 0x641, 0x648, 0x643, 0x644, 0x627, 0x646,
0x62f, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x631, 0x20, 0x641,
0x648, 0x643, 0x644, 0x627, 0x646, 0x62f, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645,
0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x631, 0x20, 0x641,
0x648, 0x643, 0x644, 0x627, 0x646, 0x62f, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641,
0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x641, 0x64a, 0x62c, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x641, 0x64a, 0x62c, 0x64a, 0x20, 0x627, 0x644,
0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x641, 0x64a,
0x62c, 0x64a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x63a, 0x648, 0x64a, 0x627, 0x646, 0x627, 0x20, 0x627, 0x644,
0x641, 0x631, 0x646, 0x633, 0x64a, 0x629, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x627, 0x644, 0x645, 0x642, 0x627, 0x637, 0x639, 0x627, 0x62a, 0x20, 0x627, 0x644,
0x641, 0x631, 0x646, 0x633, 0x64a, 0x629, 0x20, 0x627, 0x644, 0x62c, 0x646, 0x648,
0x628, 0x64a, 0x629, 0x20, 0x648, 0x627, 0x644, 0x623, 0x646, 0x62a, 0x627, 0x631,
0x62a, 0x64a, 0x643, 0x64a, 0x629, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a,
0x644, 0x627, 0x628, 0x627, 0x63a, 0x648, 0x633, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x62c, 0x627, 0x645, 0x628, 0x64a, 0x631, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x62c, 0x648, 0x631, 0x62c, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x62c, 0x648, 0x631, 0x62c, 0x64a, 0x627, 0x20, 0x627, 0x644, 0x631, 0x633,
0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x648, 0x631, 0x62c,
0x64a, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x631, 0x20, 0x62c, 0x64a, 0x644, 0x628, 0x631,
0x62a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x631, 0x64a, 0x646, 0x62a,
0x634, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x634, 0x631, 0x642, 0x20, 0x63a,
0x631, 0x64a, 0x646, 0x644, 0x627, 0x646, 0x62f, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x634, 0x631, 0x642, 0x20, 0x63a, 0x631, 0x64a, 0x646, 0x644, 0x627, 0x646,
0x62f, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x634, 0x631, 0x642, 0x20, 0x63a, 0x631, 0x64a, 0x646, 0x644, 0x627,
0x646, 0x62f, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x63a, 0x631, 0x64a, 0x646, 0x644,
0x627, 0x646, 0x62f, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x631, 0x628,
0x20, 0x63a, 0x631, 0x64a, 0x646, 0x644, 0x627, 0x646, 0x62f, 0x20, 0x627, 0x644,
0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x631,
0x628, 0x20, 0x63a, 0x631, 0x64a, 0x646, 0x644, 0x627, 0x646, 0x62f, 0x20, 0x627,
0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a,
0x648, 0x627, 0x645, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x62e,
0x644, 0x64a, 0x62c, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x64a, 0x627,
0x646, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x647, 0x627, 0x648, 0x627,
0x64a, 0x20, 0x623, 0x644, 0x648, 0x62a, 0x64a, 0x627, 0x646, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x647, 0x627, 0x648, 0x627, 0x64a, 0x20, 0x623, 0x644, 0x648,
0x62a, 0x64a, 0x627, 0x646, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x647, 0x627, 0x648, 0x627, 0x64a, 0x20, 0x623,
0x644, 0x648, 0x62a, 0x64a, 0x627, 0x646, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641,
0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x647, 0x648, 0x646, 0x63a, 0x20,
0x643, 0x648, 0x646, 0x63a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x647, 0x648,
0x646, 0x63a, 0x20, 0x643, 0x648, 0x646, 0x63a, 0x20, 0x627, 0x644, 0x631, 0x633,
0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x647, 0x648, 0x646, 0x63a,
0x20, 0x643, 0x648, 0x646, 0x63a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x647, 0x648, 0x641, 0x62f, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x647, 0x648, 0x641, 0x62f, 0x20, 0x627, 0x644, 0x631,
0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x647, 0x648, 0x641,
0x62f, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x627, 0x644, 0x647, 0x646, 0x62f, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x627, 0x644, 0x645, 0x62d, 0x64a, 0x637, 0x20, 0x627, 0x644, 0x647, 0x646,
0x62f, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x647, 0x646,
0x62f, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x646, 0x64a, 0x629, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x648, 0x633, 0x637, 0x20, 0x625, 0x646, 0x62f, 0x648, 0x646,
0x64a, 0x633, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x634, 0x631,
0x642, 0x20, 0x625, 0x646, 0x62f, 0x648, 0x646, 0x64a, 0x633, 0x64a, 0x627, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x625, 0x646, 0x62f,
0x648, 0x646, 0x64a, 0x633, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x625, 0x64a, 0x631, 0x627, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x625,
0x64a, 0x631, 0x627, 0x646, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x625, 0x64a, 0x631, 0x627, 0x646, 0x20, 0x627,
0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x625,
0x631, 0x643, 0x648, 0x62a, 0x633, 0x643, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x625, 0x631, 0x643, 0x648, 0x62a, 0x633, 0x643, 0x20, 0x627, 0x644, 0x631, 0x633,
0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x625, 0x631, 0x643, 0x648,
0x62a, 0x633, 0x643, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x625, 0x633, 0x631, 0x627, 0x626, 0x64a, 0x644, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x625, 0x633, 0x631, 0x627, 0x626, 0x64a, 0x644,
0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x625, 0x633, 0x631, 0x627, 0x626, 0x64a, 0x644, 0x20, 0x627, 0x644, 0x635,
0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x64a,
0x627, 0x628, 0x627, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644,
0x64a, 0x627, 0x628, 0x627, 0x646, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x64a, 0x627, 0x628, 0x627,
0x646, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x643, 0x627, 0x645, 0x634, 0x627, 0x62a, 0x643, 0x627, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x628, 0x64a, 0x62a, 0x631, 0x648, 0x628, 0x627, 0x641,
0x644, 0x648, 0x641, 0x633, 0x643, 0x2d, 0x643, 0x627, 0x645, 0x62a, 0x634, 0x627,
0x62a, 0x633, 0x643, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x64a,
0x62a, 0x631, 0x648, 0x628, 0x627, 0x641, 0x644, 0x648, 0x641, 0x633, 0x643, 0x2d,
0x643, 0x627, 0x645, 0x62a, 0x634, 0x627, 0x62a, 0x633, 0x643, 0x64a, 0x20, 0x627,
0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x643,
0x627, 0x632, 0x627, 0x62e, 0x633, 0x62a, 0x627, 0x646, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x634, 0x631, 0x642, 0x20, 0x643, 0x627, 0x632, 0x627, 0x62e, 0x633,
0x62a, 0x627, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x631, 0x628,
0x20, 0x643, 0x627, 0x632, 0x627, 0x62e, 0x633, 0x62a, 0x627, 0x646, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x643, 0x648, 0x631, 0x64a, 0x627, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x643, 0x648, 0x631, 0x64a, 0x627, 0x20, 0x627, 0x644, 0x631,
0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x643, 0x648, 0x631,
0x64a, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x643, 0x648, 0x633, 0x631, 0x627, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x643, 0x631, 0x627, 0x633, 0x646, 0x648, 0x64a, 0x627, 0x631, 0x633,
0x643, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x643, 0x631, 0x627, 0x633, 0x646,
0x648, 0x64a, 0x627, 0x631, 0x633, 0x643, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645,
0x64a, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x635,
0x64a, 0x641, 0x64a, 0x20, 0x644, 0x643, 0x631, 0x627, 0x633, 0x646, 0x648, 0x64a,
0x627, 0x631, 0x633, 0x643, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x642, 0x64a,
0x631, 0x63a, 0x64a, 0x632, 0x633, 0x62a, 0x627, 0x646, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x62c, 0x632, 0x631, 0x20, 0x644, 0x627, 0x64a, 0x646, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x644, 0x648, 0x631, 0x62f, 0x20, 0x647, 0x627, 0x648,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x644, 0x648, 0x631, 0x62f, 0x20, 0x647,
0x627, 0x648, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x627, 0x644, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x20,
0x644, 0x644, 0x648, 0x631, 0x62f, 0x20, 0x647, 0x627, 0x648, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x645, 0x627, 0x63a, 0x627, 0x62f, 0x627, 0x646, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x645, 0x627, 0x63a, 0x627, 0x62f, 0x627, 0x646, 0x20,
0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x645, 0x627, 0x63a, 0x627, 0x62f, 0x627, 0x646, 0x20, 0x627, 0x644, 0x635, 0x64a,
0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x645, 0x627, 0x644, 0x64a,
0x632, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x631,
0x20, 0x627, 0x644, 0x645, 0x627, 0x644, 0x62f, 0x64a, 0x641, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x645, 0x627, 0x631, 0x643, 0x64a, 0x633, 0x627, 0x633, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x631, 0x20, 0x645, 0x627, 0x631,
0x634, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x645, 0x648, 0x631,
0x64a, 0x634, 0x64a, 0x648, 0x633, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x645,
0x648, 0x631, 0x64a, 0x634, 0x64a, 0x648, 0x633, 0x20, 0x627, 0x644, 0x631, 0x633,
0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x645, 0x648, 0x631, 0x64a,
0x634, 0x64a, 0x648, 0x633, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x645, 0x627, 0x648, 0x633, 0x648, 0x646, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x645, 0x62d, 0x64a, 0x637, 0x20,
0x627, 0x644, 0x647, 0x627, 0x62f, 0x64a, 0x20, 0x644, 0x644, 0x645, 0x643, 0x633,
0x64a, 0x643, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x645, 0x62d,
0x64a, 0x637, 0x20, 0x627, 0x644, 0x647, 0x627, 0x62f, 0x64a, 0x20, 0x627, 0x644,
0x631, 0x633, 0x645, 0x64a, 0x20, 0x644, 0x644, 0x645, 0x643, 0x633, 0x64a, 0x643,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x645, 0x62d, 0x64a, 0x637,
0x20, 0x627, 0x644, 0x647, 0x627, 0x62f, 0x64a, 0x20, 0x627, 0x644, 0x635, 0x64a,
0x641, 0x64a, 0x20, 0x644, 0x644, 0x645, 0x643, 0x633, 0x64a, 0x643, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x623, 0x648, 0x644, 0x627, 0x646, 0x20, 0x628, 0x627,
0x62a, 0x648, 0x631, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x648, 0x644,
0x627, 0x646, 0x20, 0x628, 0x627, 0x62a, 0x648, 0x631, 0x20, 0x627, 0x644, 0x631,
0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x648, 0x644,
0x627, 0x646, 0x20, 0x628, 0x627, 0x62a, 0x648, 0x631, 0x20, 0x627, 0x644, 0x635,
0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x645, 0x648, 0x633,
0x643, 0x648, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x645, 0x648, 0x633, 0x643,
0x648, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x645, 0x648, 0x633, 0x643, 0x648, 0x20, 0x627, 0x644, 0x635, 0x64a,
0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x645, 0x64a, 0x627, 0x646,
0x645, 0x627, 0x631, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x646, 0x627, 0x648,
0x631, 0x648, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x646, 0x64a, 0x628, 0x627,
0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x643, 0x627, 0x644, 0x64a, 0x62f,
0x648, 0x646, 0x64a, 0x627, 0x20, 0x627, 0x644, 0x62c, 0x62f, 0x64a, 0x62f, 0x629,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x643, 0x627, 0x644, 0x64a, 0x62f, 0x648,
0x646, 0x64a, 0x627, 0x20, 0x627, 0x644, 0x62c, 0x62f, 0x64a, 0x62f, 0x629, 0x20,
0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x643, 0x627, 0x644, 0x64a, 0x62f, 0x648, 0x646, 0x64a, 0x627, 0x20, 0x627, 0x644,
0x62c, 0x62f, 0x64a, 0x62f, 0x629, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x646, 0x64a, 0x648, 0x632, 0x64a, 0x644,
0x646, 0x62f, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x646, 0x64a, 0x648,
0x632, 0x64a, 0x644, 0x646, 0x62f, 0x627, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645,
0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x646, 0x64a, 0x648, 0x632, 0x64a,
0x644, 0x646, 0x62f, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x646, 0x64a, 0x648, 0x641, 0x627, 0x648, 0x646,
0x62f, 0x644, 0x627, 0x646, 0x62f, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x646,
0x64a, 0x648, 0x641, 0x627, 0x648, 0x646, 0x62f, 0x644, 0x627, 0x646, 0x62f, 0x20,
0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x646, 0x64a, 0x648, 0x641, 0x627, 0x648, 0x646, 0x62f, 0x644, 0x627, 0x646, 0x62f,
0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x646, 0x64a, 0x648, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c,
0x632, 0x64a, 0x631, 0x629, 0x20, 0x646, 0x648, 0x631, 0x641, 0x648, 0x644, 0x643,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x64a, 0x631, 0x629, 0x20,
0x646, 0x648, 0x631, 0x641, 0x648, 0x644, 0x643, 0x20, 0x627, 0x644, 0x631, 0x633,
0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x64a, 0x631,
0x629, 0x20, 0x646, 0x648, 0x631, 0x641, 0x648, 0x644, 0x643, 0x20, 0x627, 0x644,
0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x641, 0x64a,
0x631, 0x646, 0x627, 0x646, 0x62f, 0x648, 0x20, 0x62f, 0x64a, 0x20, 0x646, 0x648,
0x631, 0x648, 0x646, 0x647, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x641,
0x631, 0x646, 0x627, 0x646, 0x62f, 0x648, 0x20, 0x62f, 0x64a, 0x20, 0x646, 0x648,
0x631, 0x648, 0x646, 0x647, 0x627, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x641, 0x631, 0x646, 0x627, 0x646, 0x62f,
0x648, 0x20, 0x62f, 0x64a, 0x20, 0x646, 0x648, 0x631, 0x648, 0x646, 0x647, 0x627,
0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x62c, 0x632, 0x631, 0x20, 0x645, 0x627, 0x631, 0x64a, 0x627, 0x646, 0x627,
0x20, 0x627, 0x644, 0x634, 0x645, 0x627, 0x644, 0x64a, 0x629, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x646, 0x648, 0x641, 0x648, 0x633, 0x64a, 0x628, 0x64a, 0x631,
0x633, 0x643, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x646, 0x648, 0x641, 0x648,
0x633, 0x64a, 0x628, 0x64a, 0x631, 0x633, 0x643, 0x20, 0x627, 0x644, 0x631, 0x633,
0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x646, 0x648, 0x641, 0x648,
0x633, 0x64a, 0x628, 0x64a, 0x631, 0x633, 0x643, 0x20, 0x627, 0x644, 0x635, 0x64a,
0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x648, 0x645, 0x633,
0x643, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x648, 0x645, 0x633, 0x643,
0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x623, 0x648, 0x645, 0x633, 0x643, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641,
0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x627, 0x643, 0x633, 0x62a,
0x627, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x627, 0x643, 0x633,
0x62a, 0x627, 0x646, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x628, 0x627, 0x643, 0x633, 0x62a, 0x627, 0x646, 0x20,
0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x628, 0x627, 0x644, 0x627, 0x648, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628,
0x627, 0x628, 0x648, 0x627, 0x20, 0x63a, 0x64a, 0x646, 0x64a, 0x627, 0x20, 0x627,
0x644, 0x62c, 0x62f, 0x64a, 0x62f, 0x629, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x628, 0x627, 0x631, 0x627, 0x63a, 0x648, 0x627, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x628, 0x627, 0x631, 0x627, 0x63a, 0x648, 0x627, 0x64a, 0x20, 0x627,
0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628,
0x627, 0x631, 0x627, 0x63a, 0x648, 0x627, 0x64a, 0x20, 0x627, 0x644, 0x635, 0x64a,
0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x64a, 0x631, 0x648,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x64a, 0x631, 0x648, 0x20, 0x627,
0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628,
0x64a, 0x631, 0x648, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x641, 0x64a, 0x644, 0x628, 0x64a, 0x646,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x641, 0x64a, 0x644, 0x628,
0x64a, 0x646, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x627, 0x644, 0x641, 0x64a, 0x644, 0x628, 0x64a, 0x646, 0x20,
0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x62c, 0x632, 0x631, 0x20, 0x641, 0x64a, 0x646, 0x643, 0x633, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x633, 0x627, 0x646, 0x62a, 0x20, 0x628, 0x64a, 0x64a, 0x631,
0x20, 0x648, 0x645, 0x64a, 0x643, 0x648, 0x644, 0x648, 0x646, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x633, 0x627, 0x646, 0x62a, 0x20, 0x628, 0x64a, 0x64a, 0x631,
0x20, 0x648, 0x645, 0x64a, 0x643, 0x648, 0x644, 0x648, 0x646, 0x20, 0x627, 0x644,
0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x633, 0x627,
0x646, 0x62a, 0x20, 0x628, 0x64a, 0x64a, 0x631, 0x20, 0x648, 0x645, 0x64a, 0x643,
0x648, 0x644, 0x648, 0x646, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x64a, 0x62a, 0x643, 0x64a, 0x631, 0x646,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x648, 0x646, 0x627, 0x628, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x64a, 0x648, 0x646, 0x63a, 0x20,
0x64a, 0x627, 0x646, 0x63a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x631, 0x648,
0x64a, 0x646, 0x64a, 0x648, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x631,
0x648, 0x62b, 0x64a, 0x631, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x633,
0x627, 0x62e, 0x627, 0x644, 0x64a, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x633, 0x627, 0x62e, 0x627, 0x644, 0x64a, 0x646, 0x20, 0x627, 0x644, 0x631, 0x633,
0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x633, 0x627, 0x62e, 0x627,
0x644, 0x64a, 0x646, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x633, 0x627, 0x645, 0x627, 0x631, 0x627, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x633, 0x645, 0x627, 0x631, 0x627, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x633, 0x645, 0x627, 0x631, 0x627, 0x20, 0x627, 0x644, 0x635,
0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x633, 0x627, 0x645,
0x648, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x633, 0x627, 0x645, 0x648,
0x627, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x633, 0x627, 0x645, 0x648, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a,
0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x633, 0x64a, 0x634, 0x644,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x633, 0x646, 0x63a, 0x627, 0x641, 0x648,
0x631, 0x629, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x631, 0x20,
0x633, 0x644, 0x64a, 0x645, 0x627, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x62c, 0x646, 0x648, 0x628, 0x20, 0x62c, 0x648, 0x631, 0x62c, 0x64a, 0x627, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x633, 0x648, 0x631, 0x64a, 0x646, 0x627, 0x645,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x633, 0x627, 0x64a, 0x648, 0x648, 0x627,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x627, 0x647, 0x64a, 0x62a, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x627, 0x64a, 0x628, 0x64a, 0x647,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x627, 0x64a, 0x628, 0x64a, 0x647,
0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x62a, 0x627, 0x64a, 0x628, 0x64a, 0x647, 0x20, 0x627, 0x644, 0x635, 0x64a,
0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x637, 0x627, 0x62c, 0x643,
0x633, 0x62a, 0x627, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x648,
0x643, 0x64a, 0x644, 0x627, 0x648, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a,
0x648, 0x646, 0x63a, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x648,
0x646, 0x63a, 0x627, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x648, 0x646, 0x63a, 0x627, 0x20, 0x627, 0x644,
0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x634, 0x648,
0x643, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x631, 0x643, 0x645, 0x627,
0x646, 0x633, 0x62a, 0x627, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a,
0x631, 0x643, 0x645, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x627, 0x644,
0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x631,
0x643, 0x645, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x627, 0x644, 0x635,
0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x648, 0x641,
0x627, 0x644, 0x648, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x648, 0x631,
0x648, 0x63a, 0x648, 0x627, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623,
0x648, 0x631, 0x648, 0x63a, 0x648, 0x627, 0x64a, 0x20, 0x627, 0x644, 0x631, 0x633,
0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x648, 0x631, 0x648,
0x63a, 0x648, 0x627, 0x64a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x648, 0x632, 0x628, 0x643, 0x633, 0x62a,
0x627, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x648, 0x632, 0x628,
0x643, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x648, 0x632, 0x628, 0x643, 0x633,
0x62a, 0x627, 0x646, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x641, 0x627, 0x646, 0x648, 0x627, 0x62a, 0x648, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x641, 0x627, 0x646, 0x648, 0x627, 0x62a, 0x648,
0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x641, 0x627, 0x646, 0x648, 0x627, 0x62a, 0x648, 0x20, 0x627, 0x644, 0x635,
0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x641, 0x646, 0x632,
0x648, 0x64a, 0x644, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x641, 0x644,
0x627, 0x62f, 0x64a, 0x641, 0x648, 0x633, 0x62a, 0x648, 0x643, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x641, 0x644, 0x627, 0x62f, 0x64a, 0x641, 0x648, 0x633, 0x62a,
0x648, 0x643, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x641, 0x644, 0x627, 0x62f, 0x64a, 0x641, 0x648, 0x633, 0x62a,
0x648, 0x643, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x641, 0x648, 0x644, 0x63a, 0x648, 0x63a, 0x631, 0x627, 0x62f,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x641, 0x648, 0x644, 0x63a, 0x648, 0x63a,
0x631, 0x627, 0x62f, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x641, 0x648, 0x644, 0x63a, 0x648, 0x63a, 0x631, 0x627,
0x62f, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x641, 0x648, 0x633, 0x62a, 0x648, 0x643, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x62c, 0x632, 0x64a, 0x631, 0x629, 0x20, 0x648, 0x64a, 0x643, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x648, 0x627, 0x644, 0x64a, 0x633, 0x20, 0x648,
0x20, 0x641, 0x648, 0x62a, 0x648, 0x646, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x64a, 0x627, 0x643, 0x648, 0x62a, 0x633, 0x643, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x64a, 0x627, 0x643, 0x648, 0x62a, 0x633, 0x643, 0x20, 0x627, 0x644,
0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x64a, 0x627,
0x643, 0x648, 0x62a, 0x633, 0x643, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x64a, 0x643, 0x627, 0x62a, 0x631, 0x64a,
0x646, 0x628, 0x648, 0x631, 0x63a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x64a,
0x643, 0x627, 0x62a, 0x631, 0x64a, 0x646, 0x628, 0x648, 0x631, 0x63a, 0x20, 0x627,
0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x64a,
0x643, 0x627, 0x62a, 0x631, 0x64a, 0x646, 0x628, 0x648, 0x631, 0x63a, 0x20, 0x627,
0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x64a,
0x648, 0x643, 0x648, 0x646, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x64, 0x2019,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x2019, 0x6f, 0x20,
0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x20, 0x64, 0x2019, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64,
0x2019, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x4e, 0x6f, 0x72,
0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20,
0x64, 0x2019, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x79,
0x61, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20,
0x64, 0x2019, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x79, 0x61, 0x20, 0x64,
0x2019, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x2019, 0x6f,
0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x79, 0x61, 0x20, 0x64, 0x2019, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x4e,
0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x6f,
0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x64, 0x2019,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x2019, 0x6f, 0x20,
0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x50,
0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x64, 0x2019, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x4e, 0x6f,
0x72, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x50, 0x61,
0x63, 0x69, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x4e, 0x6f, 0x72,
0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63,
0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x20, 0x64, 0x2019,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x6f, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x6f, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x65,
0x73, 0x74, 0x65, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x65, 0x73, 0x74, 0x65, 0x20,
0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x65, 0x73, 0x74, 0x65, 0x20, 0x64, 0x2019, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x75, 0x65, 0x73, 0x74, 0x65, 0x20, 0x64, 0x2019, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x75, 0x65, 0x73, 0x74, 0x65, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x75, 0x65,
0x73, 0x74, 0x65, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x6d, 0x65, 0x72,
0x69, 0x64, 0x69, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x531, 0x586, 0x572, 0x561, 0x576,
0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x53f, 0x565, 0x576, 0x57f, 0x580, 0x578, 0x576, 0x561, 0x56f, 0x561, 0x576,
0x20, 0x531, 0x586, 0x580, 0x56b, 0x56f, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x565, 0x56c, 0x575, 0x561,
0x576, 0x20, 0x531, 0x586, 0x580, 0x56b, 0x56f, 0x561, 0x575, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x540, 0x561, 0x580, 0x561, 0x57e, 0x561,
0x575, 0x56b, 0x576, 0x20, 0x531, 0x586, 0x580, 0x56b, 0x56f, 0x561, 0x575, 0x56b,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x574,
0x57f, 0x575, 0x561, 0x576, 0x20, 0x531, 0x586, 0x580, 0x56b, 0x56f, 0x561, 0x575,
0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587,
0x574, 0x57f, 0x575, 0x561, 0x576, 0x20, 0x531, 0x586, 0x580, 0x56b, 0x56f, 0x561,
0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x574, 0x57f,
0x575, 0x561, 0x576, 0x20, 0x531, 0x586, 0x580, 0x56b, 0x56f, 0x561, 0x575, 0x56b,
0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x56c, 0x575, 0x561, 0x57d, 0x56f, 0x561,
0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x56c,
0x575, 0x561, 0x57d, 0x56f, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576,
0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x531, 0x56c, 0x575, 0x561, 0x57d, 0x56f, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574,
0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x531, 0x574, 0x561, 0x566, 0x578, 0x576, 0x575, 0x561, 0x576, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x574, 0x561, 0x566, 0x578,
0x576, 0x575, 0x561, 0x576, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580,
0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x574, 0x561,
0x566, 0x578, 0x576, 0x575, 0x561, 0x576, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561,
0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f,
0x565, 0x576, 0x57f, 0x580, 0x578, 0x576, 0x561, 0x56f, 0x561, 0x576, 0x20, 0x531,
0x574, 0x565, 0x580, 0x56b, 0x56f, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x53f, 0x565, 0x576, 0x57f, 0x580, 0x578, 0x576, 0x561,
0x56f, 0x561, 0x576, 0x20, 0x531, 0x574, 0x565, 0x580, 0x56b, 0x56f, 0x561, 0x575,
0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x565, 0x576, 0x57f, 0x580, 0x578,
0x576, 0x561, 0x56f, 0x561, 0x576, 0x20, 0x531, 0x574, 0x565, 0x580, 0x56b, 0x56f,
0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x565,
0x56c, 0x575, 0x561, 0x576, 0x20, 0x531, 0x574, 0x565, 0x580, 0x56b, 0x56f, 0x561,
0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580,
0x587, 0x565, 0x56c, 0x575, 0x561, 0x576, 0x20, 0x531, 0x574, 0x565, 0x580, 0x56b,
0x56f, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580,
0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587,
0x565, 0x56c, 0x575, 0x561, 0x576, 0x20, 0x531, 0x574, 0x565, 0x580, 0x56b, 0x56f,
0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53c, 0x565, 0x57c, 0x576,
0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x20, 0x28, 0x531, 0x544, 0x546, 0x29, 0x53c, 0x565, 0x57c, 0x576, 0x561, 0x575,
0x56b, 0x576, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x20, 0x28, 0x531, 0x544, 0x546,
0x29, 0x53c, 0x565, 0x57c, 0x576, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x561, 0x574,
0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x20, 0x28, 0x531, 0x544, 0x546, 0x29, 0x53d, 0x561, 0x572, 0x561,
0x572, 0x585, 0x57e, 0x56f, 0x56b, 0x561, 0x576, 0x578, 0x57d, 0x575, 0x561, 0x576,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53d, 0x561, 0x572, 0x561,
0x572, 0x585, 0x57e, 0x56f, 0x56b, 0x561, 0x576, 0x578, 0x57d, 0x575, 0x561, 0x576,
0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x53d, 0x561, 0x572, 0x561, 0x572, 0x585, 0x57e,
0x56f, 0x56b, 0x561, 0x576, 0x578, 0x57d, 0x575, 0x561, 0x576, 0x20, 0x561, 0x574,
0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x531, 0x57a, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x531, 0x57a, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x57d,
0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x531, 0x57a, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574,
0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x54d, 0x561, 0x578, 0x582, 0x564, 0x575, 0x561, 0x576, 0x20, 0x531,
0x580, 0x561, 0x562, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x54d, 0x561, 0x578, 0x582, 0x564, 0x575, 0x561, 0x576, 0x20,
0x531, 0x580, 0x561, 0x562, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561,
0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x54d, 0x561, 0x578, 0x582, 0x564, 0x575, 0x561, 0x576, 0x20, 0x531, 0x580,
0x561, 0x562, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561,
0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531,
0x580, 0x563, 0x565, 0x576, 0x57f, 0x56b, 0x576, 0x561, 0x575, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x563, 0x565, 0x576, 0x57f,
0x56b, 0x576, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x576, 0x561, 0x564, 0x561,
0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580,
0x563, 0x565, 0x576, 0x57f, 0x56b, 0x576, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574,
0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x531, 0x580, 0x587, 0x574, 0x57f, 0x575, 0x561, 0x576, 0x20, 0x531,
0x580, 0x563, 0x565, 0x576, 0x57f, 0x56b, 0x576, 0x561, 0x575, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x574, 0x57f, 0x575,
0x561, 0x576, 0x20, 0x531, 0x580, 0x563, 0x565, 0x576, 0x57f, 0x56b, 0x576, 0x561,
0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x576, 0x561, 0x564, 0x561, 0x580, 0x57f, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x574, 0x57f,
0x575, 0x561, 0x576, 0x20, 0x531, 0x580, 0x563, 0x565, 0x576, 0x57f, 0x56b, 0x576,
0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x540, 0x561, 0x575, 0x561,
0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x540, 0x561, 0x575, 0x561, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x57d,
0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x540, 0x561, 0x575, 0x561, 0x57d, 0x57f, 0x561, 0x576, 0x56b,
0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x57f, 0x56c, 0x561, 0x576, 0x57f, 0x575,
0x561, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x57f,
0x56c, 0x561, 0x576, 0x57f, 0x575, 0x561, 0x576, 0x20, 0x57d, 0x57f, 0x561, 0x576,
0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x531, 0x57f, 0x56c, 0x561, 0x576, 0x57f, 0x575, 0x561, 0x576, 0x20, 0x561, 0x574,
0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x53f, 0x565, 0x576, 0x57f, 0x580, 0x578, 0x576, 0x561, 0x56f, 0x561,
0x576, 0x20, 0x531, 0x57e, 0x57d, 0x57f, 0x580, 0x561, 0x56c, 0x56b, 0x561, 0x575,
0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x565, 0x576,
0x57f, 0x580, 0x578, 0x576, 0x561, 0x56f, 0x561, 0x576, 0x20, 0x531, 0x57e, 0x57d,
0x57f, 0x580, 0x561, 0x56c, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561,
0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x53f, 0x565, 0x576, 0x57f, 0x580, 0x578, 0x576, 0x561, 0x56f, 0x561, 0x576,
0x20, 0x531, 0x57e, 0x57d, 0x57f, 0x580, 0x561, 0x56c, 0x56b, 0x561, 0x575, 0x56b,
0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x565, 0x576, 0x57f, 0x580, 0x578, 0x576,
0x561, 0x56f, 0x561, 0x576, 0x20, 0x531, 0x57e, 0x57d, 0x57f, 0x580, 0x561, 0x56c,
0x56b, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x580, 0x587, 0x574, 0x57f, 0x575, 0x561,
0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x565, 0x576,
0x57f, 0x580, 0x578, 0x576, 0x561, 0x56f, 0x561, 0x576, 0x20, 0x531, 0x57e, 0x57d,
0x57f, 0x580, 0x561, 0x56c, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x580, 0x587,
0x574, 0x57f, 0x575, 0x561, 0x576, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561,
0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x565,
0x576, 0x57f, 0x580, 0x578, 0x576, 0x561, 0x56f, 0x561, 0x576, 0x20, 0x531, 0x57e,
0x57d, 0x57f, 0x580, 0x561, 0x56c, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x580,
0x587, 0x574, 0x57f, 0x575, 0x561, 0x576, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561,
0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531,
0x580, 0x587, 0x565, 0x56c, 0x575, 0x561, 0x576, 0x20, 0x531, 0x57e, 0x57d, 0x57f,
0x580, 0x561, 0x56c, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x565, 0x56c, 0x575, 0x561, 0x576, 0x20,
0x531, 0x57e, 0x57d, 0x57f, 0x580, 0x561, 0x56c, 0x56b, 0x561, 0x575, 0x56b, 0x20,
0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x565, 0x56c, 0x575, 0x561, 0x576,
0x20, 0x531, 0x57e, 0x57d, 0x57f, 0x580, 0x561, 0x56c, 0x56b, 0x561, 0x575, 0x56b,
0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x574, 0x57f, 0x575, 0x561,
0x576, 0x20, 0x531, 0x57e, 0x57d, 0x57f, 0x580, 0x561, 0x56c, 0x56b, 0x561, 0x575,
0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587,
0x574, 0x57f, 0x575, 0x561, 0x576, 0x20, 0x531, 0x57e, 0x57d, 0x57f, 0x580, 0x561,
0x56c, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561,
0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580,
0x587, 0x574, 0x57f, 0x575, 0x561, 0x576, 0x20, 0x531, 0x57e, 0x57d, 0x57f, 0x580,
0x561, 0x56c, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561,
0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531,
0x564, 0x580, 0x562, 0x565, 0x57b, 0x561, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x531, 0x564, 0x580, 0x562, 0x565, 0x57b, 0x561, 0x576,
0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x564, 0x580, 0x562, 0x565, 0x57b,
0x561, 0x576, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x566, 0x578, 0x580,
0x575, 0x561, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x566, 0x578, 0x580,
0x575, 0x561, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b,
0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x566, 0x578, 0x580, 0x575, 0x561, 0x576,
0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x561, 0x574,
0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x532, 0x561, 0x576, 0x563, 0x56c, 0x561, 0x564, 0x565, 0x577, 0x56b,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x532, 0x561, 0x576, 0x563,
0x56c, 0x561, 0x564, 0x565, 0x577, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564,
0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x532,
0x561, 0x576, 0x563, 0x56c, 0x561, 0x564, 0x565, 0x577, 0x56b, 0x20, 0x561, 0x574,
0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x532, 0x578, 0x582, 0x569, 0x561, 0x576, 0x56b, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x532, 0x578, 0x56c, 0x56b, 0x57e, 0x56b, 0x561,
0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x532, 0x580,
0x561, 0x566, 0x56b, 0x56c, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x532, 0x580, 0x561, 0x566, 0x56b, 0x56c, 0x56b, 0x561,
0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x532, 0x580, 0x561, 0x566, 0x56b,
0x56c, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575,
0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x532, 0x580,
0x578, 0x582, 0x576, 0x565, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x53f, 0x561, 0x562, 0x578, 0x20, 0x54e, 0x565, 0x580, 0x564, 0x565,
0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x561, 0x562,
0x578, 0x20, 0x54e, 0x565, 0x580, 0x564, 0x565, 0x56b, 0x20, 0x57d, 0x57f, 0x561,
0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x53f, 0x561, 0x562, 0x578, 0x20, 0x54e, 0x565, 0x580, 0x564, 0x565, 0x56b,
0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x549, 0x561, 0x574, 0x578, 0x57c, 0x578, 0x575,
0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x549, 0x561, 0x569,
0x565, 0x574, 0x20, 0x56f, 0x572, 0x566, 0x578, 0x582, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x549, 0x561, 0x569, 0x565, 0x574, 0x20, 0x56f, 0x572,
0x566, 0x578, 0x582, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x549, 0x561, 0x569, 0x565,
0x574, 0x20, 0x56f, 0x572, 0x566, 0x578, 0x582, 0x20, 0x561, 0x574, 0x561, 0x57c,
0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x549, 0x56b, 0x56c, 0x56b, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x549, 0x56b, 0x56c, 0x56b, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564,
0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x549,
0x56b, 0x56c, 0x56b, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b,
0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x549, 0x56b, 0x576,
0x561, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x549, 0x56b, 0x576, 0x561, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20,
0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x549, 0x56b, 0x576, 0x561, 0x57d, 0x57f, 0x561, 0x576,
0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54d, 0x578, 0x582, 0x580, 0x562, 0x20,
0x53e, 0x576, 0x576, 0x564, 0x575, 0x561, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x578,
0x582, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x578, 0x56f,
0x578, 0x57d, 0x575, 0x561, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565,
0x580, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x578,
0x56c, 0x578, 0x582, 0x574, 0x562, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x578, 0x56c, 0x578, 0x582, 0x574, 0x562,
0x56b, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580,
0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x578, 0x56c,
0x578, 0x582, 0x574, 0x562, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561,
0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x53f, 0x578, 0x582, 0x56f, 0x56b, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576,
0x565, 0x580, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f,
0x578, 0x582, 0x56f, 0x56b, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580,
0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x578, 0x582, 0x56f, 0x56b, 0x20,
0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x56f, 0x56b, 0x57d,
0x561, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x578, 0x582, 0x562, 0x561, 0x575, 0x56b,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x578, 0x582, 0x562,
0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x578, 0x582, 0x562,
0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x534, 0x565, 0x575, 0x57e,
0x56b, 0x57d, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x534,
0x575, 0x578, 0x582, 0x574, 0x578, 0x576, 0x20, 0x564, 0x2019, 0x545, 0x578, 0x582,
0x580, 0x57e, 0x56b, 0x56c, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x531, 0x580, 0x587, 0x565, 0x56c, 0x575, 0x561, 0x576, 0x20, 0x539, 0x56b,
0x574, 0x578, 0x580, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x536, 0x561, 0x57f, 0x56f, 0x56b, 0x20, 0x56f, 0x572, 0x566, 0x578, 0x582, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x536, 0x561, 0x57f, 0x56f, 0x56b,
0x20, 0x56f, 0x572, 0x566, 0x578, 0x582, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564,
0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x536,
0x561, 0x57f, 0x56f, 0x56b, 0x20, 0x56f, 0x572, 0x566, 0x578, 0x582, 0x20, 0x561,
0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x537, 0x56f, 0x57e, 0x561, 0x564, 0x578, 0x580, 0x56b, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x565, 0x576, 0x57f, 0x580,
0x578, 0x576, 0x561, 0x56f, 0x561, 0x576, 0x20, 0x535, 0x57e, 0x580, 0x578, 0x57a,
0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f,
0x565, 0x576, 0x57f, 0x580, 0x578, 0x576, 0x561, 0x56f, 0x561, 0x576, 0x20, 0x535,
0x57e, 0x580, 0x578, 0x57a, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576,
0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x53f, 0x565, 0x576, 0x57f, 0x580, 0x578, 0x576, 0x561, 0x56f, 0x561, 0x576, 0x20,
0x535, 0x57e, 0x580, 0x578, 0x57a, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561,
0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x531, 0x580, 0x587, 0x565, 0x56c, 0x575, 0x561, 0x576, 0x20, 0x535, 0x57e,
0x580, 0x578, 0x57a, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x531, 0x580, 0x587, 0x565, 0x56c, 0x575, 0x561, 0x576, 0x20, 0x535,
0x57e, 0x580, 0x578, 0x57a, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576,
0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x531, 0x580, 0x587, 0x565, 0x56c, 0x575, 0x561, 0x576, 0x20, 0x535, 0x57e, 0x580,
0x578, 0x57a, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575,
0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x544, 0x56b,
0x576, 0x57d, 0x56f, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x531, 0x580, 0x587, 0x574, 0x57f, 0x575, 0x561, 0x576, 0x20, 0x535, 0x57e, 0x580,
0x578, 0x57a, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x531, 0x580, 0x587, 0x574, 0x57f, 0x575, 0x561, 0x576, 0x20, 0x535, 0x57e,
0x580, 0x578, 0x57a, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564,
0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531,
0x580, 0x587, 0x574, 0x57f, 0x575, 0x561, 0x576, 0x20, 0x535, 0x57e, 0x580, 0x578,
0x57a, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b,
0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x556, 0x578, 0x56c,
0x584, 0x56c, 0x565, 0x576, 0x564, 0x575, 0x561, 0x576, 0x20, 0x56f, 0x572, 0x566,
0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x556, 0x578, 0x56c, 0x584, 0x56c, 0x565, 0x576, 0x564, 0x575, 0x561, 0x576,
0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x57d, 0x57f,
0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x556, 0x578, 0x56c, 0x584, 0x56c, 0x565, 0x576, 0x564, 0x575, 0x561,
0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x561,
0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x556, 0x56b, 0x57b, 0x56b, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x556, 0x56b, 0x57b, 0x56b, 0x56b, 0x20, 0x57d, 0x57f,
0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x556, 0x56b, 0x57b, 0x56b, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c,
0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x556, 0x580, 0x561, 0x576, 0x57d, 0x56b, 0x561, 0x56f, 0x561, 0x576, 0x20, 0x533,
0x57e, 0x56b, 0x561, 0x576, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x556, 0x580, 0x561, 0x576, 0x57d, 0x56b, 0x561, 0x56f, 0x561,
0x576, 0x20, 0x570, 0x561, 0x580, 0x561, 0x57e, 0x561, 0x575, 0x56b, 0x576, 0x20,
0x587, 0x20, 0x561, 0x576, 0x57f, 0x561, 0x580, 0x56f, 0x57f, 0x56b, 0x564, 0x575,
0x561, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x533, 0x561,
0x56c, 0x561, 0x57a, 0x561, 0x563, 0x578, 0x57d, 0x575, 0x561, 0x576, 0x20, 0x56f,
0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x533, 0x561, 0x574, 0x562, 0x575, 0x565, 0x20, 0x56f, 0x572,
0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x54e, 0x580, 0x561, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54e, 0x580, 0x561, 0x57d, 0x57f, 0x561,
0x576, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54e, 0x580, 0x561, 0x57d, 0x57f,
0x561, 0x576, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54b, 0x56b, 0x56c, 0x562,
0x565, 0x580, 0x569, 0x56b, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580,
0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x533, 0x580, 0x56b,
0x576, 0x57e, 0x56b, 0x579, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x531, 0x580, 0x587, 0x565, 0x56c, 0x575, 0x561, 0x576, 0x20, 0x533, 0x580,
0x565, 0x576, 0x56c, 0x561, 0x576, 0x564, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x565, 0x56c, 0x575,
0x561, 0x576, 0x20, 0x533, 0x580, 0x565, 0x576, 0x56c, 0x561, 0x576, 0x564, 0x56b,
0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x565,
0x56c, 0x575, 0x561, 0x576, 0x20, 0x533, 0x580, 0x565, 0x576, 0x56c, 0x561, 0x576,
0x564, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575,
0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580,
0x587, 0x574, 0x57f, 0x575, 0x561, 0x576, 0x20, 0x533, 0x580, 0x565, 0x576, 0x56c,
0x561, 0x576, 0x564, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x574, 0x57f, 0x575, 0x561, 0x576, 0x20,
0x533, 0x580, 0x565, 0x576, 0x56c, 0x561, 0x576, 0x564, 0x56b, 0x561, 0x575, 0x56b,
0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x574, 0x57f, 0x575, 0x561,
0x576, 0x20, 0x533, 0x580, 0x565, 0x576, 0x56c, 0x561, 0x576, 0x564, 0x56b, 0x561,
0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54a, 0x561, 0x580, 0x57d, 0x56b,
0x581, 0x20, 0x56e, 0x578, 0x581, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564,
0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x533,
0x561, 0x575, 0x561, 0x576, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x540, 0x561, 0x57e, 0x561, 0x575, 0x561, 0x576, 0x2d, 0x561,
0x56c, 0x565, 0x578, 0x582, 0x569, 0x575, 0x561, 0x576, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x540, 0x561, 0x57e, 0x561, 0x575, 0x561, 0x576, 0x2d,
0x561, 0x56c, 0x565, 0x578, 0x582, 0x569, 0x575, 0x561, 0x576, 0x20, 0x57d, 0x57f,
0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x540, 0x561, 0x57e, 0x561, 0x575, 0x561, 0x576, 0x2d, 0x561, 0x56c,
0x565, 0x578, 0x582, 0x569, 0x575, 0x561, 0x576, 0x20, 0x561, 0x574, 0x561, 0x57c,
0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x540, 0x578, 0x576, 0x56f, 0x578, 0x576, 0x563, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x540, 0x578, 0x576, 0x56f, 0x578, 0x576, 0x563, 0x56b,
0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x540, 0x578, 0x576, 0x56f, 0x578, 0x576, 0x563,
0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x540, 0x578, 0x57e, 0x564, 0x56b, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x540, 0x578, 0x57e, 0x564, 0x56b,
0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x540, 0x578, 0x57e, 0x564, 0x56b, 0x20, 0x561,
0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x540, 0x576, 0x564, 0x56f, 0x561, 0x57d, 0x57f, 0x561, 0x576,
0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x540, 0x576, 0x564, 0x56f, 0x561, 0x56f,
0x561, 0x576, 0x20, 0x585, 0x57e, 0x56f, 0x56b, 0x561, 0x576, 0x578, 0x57d, 0x56b,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x540, 0x576, 0x564, 0x56f,
0x561, 0x579, 0x56b, 0x576, 0x561, 0x56f, 0x561, 0x576, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x53f, 0x565, 0x576, 0x57f, 0x580, 0x578, 0x576, 0x561,
0x56f, 0x561, 0x576, 0x20, 0x53b, 0x576, 0x564, 0x578, 0x576, 0x565, 0x566, 0x56b,
0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531,
0x580, 0x587, 0x565, 0x56c, 0x575, 0x561, 0x576, 0x20, 0x53b, 0x576, 0x564, 0x578,
0x576, 0x565, 0x566, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x574, 0x57f, 0x575, 0x561, 0x576, 0x20,
0x53b, 0x576, 0x564, 0x578, 0x576, 0x565, 0x566, 0x56b, 0x561, 0x575, 0x56b, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53b, 0x580, 0x561, 0x576, 0x56b,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53b, 0x580, 0x561, 0x576,
0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53b, 0x580, 0x561, 0x576, 0x56b, 0x20,
0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x53b, 0x580, 0x56f, 0x578, 0x582, 0x57f, 0x57d, 0x56f,
0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53b, 0x580, 0x56f,
0x578, 0x582, 0x57f, 0x57d, 0x56f, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564,
0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53b,
0x580, 0x56f, 0x578, 0x582, 0x57f, 0x57d, 0x56f, 0x56b, 0x20, 0x561, 0x574, 0x561,
0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x53b, 0x57d, 0x580, 0x561, 0x575, 0x565, 0x56c, 0x56b, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x53b, 0x57d, 0x580, 0x561, 0x575, 0x565, 0x56c,
0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53b, 0x57d, 0x580, 0x561, 0x575, 0x565,
0x56c, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x543, 0x561, 0x57a, 0x578, 0x576,
0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x543, 0x561, 0x57a, 0x578, 0x576, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f,
0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x543, 0x561, 0x57a, 0x578, 0x576, 0x56b, 0x561, 0x575, 0x56b, 0x20,
0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x542, 0x561, 0x566, 0x561, 0x56d, 0x57d, 0x57f, 0x561,
0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580,
0x587, 0x565, 0x56c, 0x575, 0x561, 0x576, 0x20, 0x542, 0x561, 0x566, 0x561, 0x56d,
0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x531, 0x580, 0x587, 0x574, 0x57f, 0x575, 0x561, 0x576, 0x20, 0x542, 0x561,
0x566, 0x561, 0x56d, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x53f, 0x578, 0x580, 0x565, 0x561, 0x575, 0x56b, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x578, 0x580, 0x565, 0x561,
0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x578, 0x580, 0x565, 0x561,
0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x578, 0x57d, 0x580, 0x561,
0x565, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f,
0x580, 0x561, 0x57d, 0x576, 0x578, 0x575, 0x561, 0x580, 0x57d, 0x56f, 0x56b, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x580, 0x561, 0x57d, 0x576,
0x578, 0x575, 0x561, 0x580, 0x57d, 0x56f, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576,
0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x53f, 0x580, 0x561, 0x57d, 0x576, 0x578, 0x575, 0x561, 0x580, 0x57d, 0x56f, 0x56b,
0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x542, 0x580, 0x572, 0x566, 0x57d, 0x57f, 0x561,
0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53c, 0x561,
0x575, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53c, 0x578, 0x580, 0x564, 0x20,
0x540, 0x561, 0x578, 0x582, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x53c, 0x578, 0x580, 0x564, 0x20, 0x540, 0x561, 0x578, 0x582, 0x56b, 0x20,
0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x53c, 0x578, 0x580, 0x564, 0x20, 0x540, 0x561, 0x578,
0x582, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x544, 0x561, 0x563, 0x561, 0x564,
0x561, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x544,
0x561, 0x563, 0x561, 0x564, 0x561, 0x576, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576,
0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x544, 0x561, 0x563, 0x561, 0x564, 0x561, 0x576, 0x56b, 0x20, 0x561, 0x574, 0x561,
0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x544, 0x561, 0x56c, 0x561, 0x575, 0x566, 0x56b, 0x561, 0x575, 0x56b, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x544, 0x561, 0x56c, 0x564, 0x56b,
0x57e, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x544, 0x561, 0x580, 0x56f, 0x56b, 0x566, 0x575, 0x561, 0x576, 0x20, 0x56f,
0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x544, 0x561, 0x580, 0x577, 0x561, 0x56c, 0x575, 0x561, 0x576,
0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x544, 0x561, 0x57e, 0x580, 0x56b, 0x56f, 0x56b,
0x578, 0x57d, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x544,
0x561, 0x57e, 0x580, 0x56b, 0x56f, 0x56b, 0x578, 0x57d, 0x56b, 0x20, 0x57d, 0x57f,
0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x544, 0x561, 0x57e, 0x580, 0x56b, 0x56f, 0x56b, 0x578, 0x57d, 0x56b,
0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x544, 0x578, 0x578, 0x582, 0x57d, 0x578, 0x576,
0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x544, 0x565, 0x584,
0x57d, 0x56b, 0x56f, 0x561, 0x575, 0x56b, 0x20, 0x56d, 0x561, 0x572, 0x561, 0x572,
0x585, 0x57e, 0x56f, 0x56b, 0x561, 0x576, 0x578, 0x57d, 0x575, 0x561, 0x576, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x544, 0x565, 0x584, 0x57d, 0x56b,
0x56f, 0x561, 0x575, 0x56b, 0x20, 0x56d, 0x561, 0x572, 0x561, 0x572, 0x585, 0x57e,
0x56f, 0x56b, 0x561, 0x576, 0x578, 0x57d, 0x575, 0x561, 0x576, 0x20, 0x57d, 0x57f,
0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x544, 0x565, 0x584, 0x57d, 0x56b, 0x56f, 0x561, 0x575, 0x56b, 0x20,
0x56d, 0x561, 0x572, 0x561, 0x572, 0x585, 0x57e, 0x56f, 0x56b, 0x561, 0x576, 0x578,
0x57d, 0x575, 0x561, 0x576, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b,
0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x548, 0x582, 0x56c,
0x561, 0x576, 0x20, 0x532, 0x561, 0x57f, 0x578, 0x580, 0x56b, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x548, 0x582, 0x56c, 0x561, 0x576, 0x20, 0x532,
0x561, 0x57f, 0x578, 0x580, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561,
0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x548, 0x582,
0x56c, 0x561, 0x576, 0x20, 0x532, 0x561, 0x57f, 0x578, 0x580, 0x56b, 0x20, 0x561,
0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x544, 0x578, 0x57d, 0x56f, 0x57e, 0x561, 0x575, 0x56b, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x544, 0x578, 0x57d, 0x56f, 0x57e,
0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x544, 0x578, 0x57d, 0x56f,
0x57e, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b,
0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x544, 0x575, 0x561,
0x576, 0x574, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x546, 0x561, 0x578, 0x582, 0x580, 0x578, 0x582, 0x56b, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x546, 0x565, 0x57a, 0x561, 0x56c, 0x56b, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x546, 0x578, 0x580, 0x20, 0x53f,
0x561, 0x56c, 0x565, 0x564, 0x578, 0x576, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x546, 0x578, 0x580, 0x20, 0x53f, 0x561,
0x56c, 0x565, 0x564, 0x578, 0x576, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f,
0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x546, 0x578, 0x580, 0x20, 0x53f, 0x561, 0x56c, 0x565, 0x564, 0x578,
0x576, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575,
0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x546, 0x578,
0x580, 0x20, 0x536, 0x565, 0x56c, 0x561, 0x576, 0x564, 0x56b, 0x561, 0x575, 0x56b,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x546, 0x578, 0x580, 0x20,
0x536, 0x565, 0x56c, 0x561, 0x576, 0x564, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x57d,
0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x546, 0x578, 0x580, 0x20, 0x536, 0x565, 0x56c, 0x561, 0x576,
0x564, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575,
0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x546, 0x575,
0x578, 0x582, 0x586, 0x561, 0x578, 0x582, 0x576, 0x564, 0x56c, 0x565, 0x576, 0x564,
0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x546, 0x575, 0x578,
0x582, 0x586, 0x561, 0x578, 0x582, 0x576, 0x564, 0x56c, 0x565, 0x576, 0x564, 0x56b,
0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x546, 0x575, 0x578, 0x582, 0x586, 0x561, 0x578,
0x582, 0x576, 0x564, 0x56c, 0x565, 0x576, 0x564, 0x56b, 0x20, 0x561, 0x574, 0x561,
0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x546, 0x56b, 0x578, 0x582, 0x565, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x546, 0x578, 0x580, 0x586, 0x578, 0x56c, 0x56f, 0x20,
0x56f, 0x572, 0x566, 0x578, 0x582, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x546, 0x578, 0x580, 0x586, 0x578, 0x56c, 0x56f, 0x20, 0x56f, 0x572, 0x566,
0x578, 0x582, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x546, 0x578, 0x580, 0x586, 0x578,
0x56c, 0x56f, 0x20, 0x56f, 0x572, 0x566, 0x578, 0x582, 0x20, 0x561, 0x574, 0x561,
0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x556, 0x565, 0x580, 0x576, 0x561, 0x576, 0x564, 0x578, 0x582, 0x20, 0x564,
0x56b, 0x20, 0x546, 0x578, 0x580, 0x578, 0x576, 0x575, 0x561, 0x575, 0x56b, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x556, 0x565, 0x580, 0x576, 0x561,
0x576, 0x564, 0x578, 0x582, 0x20, 0x564, 0x56b, 0x20, 0x546, 0x578, 0x580, 0x578,
0x576, 0x575, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561,
0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x556, 0x565,
0x580, 0x576, 0x561, 0x576, 0x564, 0x578, 0x582, 0x20, 0x564, 0x56b, 0x20, 0x546,
0x578, 0x580, 0x578, 0x576, 0x575, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561,
0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x546, 0x578, 0x57e, 0x578, 0x57d, 0x56b, 0x562, 0x56b, 0x580, 0x57d, 0x56f,
0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x546, 0x578, 0x57e,
0x578, 0x57d, 0x56b, 0x562, 0x56b, 0x580, 0x57d, 0x56f, 0x56b, 0x20, 0x57d, 0x57f,
0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x546, 0x578, 0x57e, 0x578, 0x57d, 0x56b, 0x562, 0x56b, 0x580, 0x57d,
0x56f, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x555, 0x574, 0x57d, 0x56f, 0x56b,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x555, 0x574, 0x57d, 0x56f,
0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x555, 0x574, 0x57d, 0x56f, 0x56b, 0x20,
0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x54a, 0x561, 0x56f, 0x56b, 0x57d, 0x57f, 0x561, 0x576,
0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54a, 0x561, 0x56f,
0x56b, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564,
0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54a,
0x561, 0x56f, 0x56b, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x561, 0x574, 0x561,
0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x54a, 0x561, 0x56c, 0x561, 0x578, 0x582, 0x575, 0x56b, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x54a, 0x561, 0x57a, 0x578, 0x582, 0x561, 0x20,
0x546, 0x578, 0x580, 0x20, 0x533, 0x57e, 0x56b, 0x576, 0x565, 0x561, 0x575, 0x56b,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54a, 0x561, 0x580, 0x561,
0x563, 0x57e, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x54a, 0x561, 0x580, 0x561, 0x563, 0x57e, 0x561, 0x575, 0x56b, 0x20, 0x57d,
0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x54a, 0x561, 0x580, 0x561, 0x563, 0x57e, 0x561, 0x575, 0x56b,
0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x54a, 0x565, 0x580, 0x578, 0x582, 0x56b, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54a, 0x565, 0x580, 0x578, 0x582,
0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54a, 0x565, 0x580, 0x578, 0x582, 0x56b,
0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x556, 0x56b, 0x56c, 0x56b, 0x57a, 0x56b, 0x576,
0x576, 0x565, 0x580, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x556, 0x56b, 0x56c, 0x56b, 0x57a, 0x56b, 0x576, 0x576, 0x565, 0x580, 0x56b, 0x20,
0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x556, 0x56b, 0x56c, 0x56b, 0x57a, 0x56b, 0x576, 0x576,
0x565, 0x580, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x556, 0x56b, 0x576, 0x56b,
0x584, 0x57d, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54d, 0x565, 0x576, 0x20, 0x54a,
0x56b, 0x565, 0x57c, 0x20, 0x587, 0x20, 0x544, 0x56b, 0x584, 0x565, 0x56c, 0x578,
0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54d, 0x565,
0x576, 0x20, 0x54a, 0x56b, 0x565, 0x57c, 0x20, 0x587, 0x20, 0x544, 0x56b, 0x584,
0x565, 0x56c, 0x578, 0x576, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561,
0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54d, 0x565,
0x576, 0x20, 0x54a, 0x56b, 0x565, 0x57c, 0x20, 0x587, 0x20, 0x544, 0x56b, 0x584,
0x565, 0x56c, 0x578, 0x576, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575,
0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54a, 0x56b,
0x57f, 0x56f, 0x565, 0x57c, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x54a, 0x578, 0x576, 0x561, 0x57a, 0x565, 0x20, 0x56f, 0x572, 0x566,
0x578, 0x582, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x553, 0x56d,
0x565, 0x576, 0x575, 0x561, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x54c, 0x565, 0x575, 0x578, 0x582, 0x576, 0x56b, 0x578, 0x576, 0x56b,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54c, 0x578, 0x57f, 0x565,
0x580, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x54d, 0x561, 0x56d, 0x561, 0x56c, 0x56b, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x54d, 0x561, 0x56d, 0x561, 0x56c, 0x56b, 0x576, 0x56b,
0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x54d, 0x561, 0x56d, 0x561, 0x56c, 0x56b, 0x576,
0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54d, 0x561, 0x574, 0x578, 0x561, 0x575,
0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54d, 0x561, 0x574,
0x578, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580,
0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54d, 0x561, 0x574,
0x578, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b,
0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54d, 0x565, 0x575,
0x577, 0x565, 0x56c, 0x575, 0x561, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576,
0x565, 0x580, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54d,
0x56b, 0x576, 0x563, 0x561, 0x57a, 0x578, 0x582, 0x580, 0x56b, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x54d, 0x578, 0x572, 0x578, 0x574, 0x578, 0x576,
0x56b, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x540, 0x561, 0x580, 0x561, 0x57e, 0x561,
0x575, 0x56b, 0x576, 0x20, 0x54b, 0x578, 0x580, 0x57b, 0x56b, 0x561, 0x575, 0x56b,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54d, 0x578, 0x582, 0x580,
0x56b, 0x576, 0x561, 0x574, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x54d, 0x575, 0x578, 0x57e, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x539, 0x561, 0x56b, 0x569, 0x56b, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x539, 0x561, 0x575, 0x57a, 0x565, 0x575,
0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x539, 0x561, 0x575,
0x57a, 0x565, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580,
0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x539, 0x561, 0x575,
0x57a, 0x565, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b,
0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54f, 0x561, 0x57b,
0x56b, 0x56f, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x54f, 0x578, 0x56f, 0x565, 0x56c, 0x561, 0x578, 0x582, 0x575,
0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54f, 0x578, 0x576,
0x563, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x54f, 0x578, 0x576, 0x563, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576,
0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x54f, 0x578, 0x576, 0x563, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c,
0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x54f, 0x580, 0x578, 0x582, 0x56f, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x539, 0x578, 0x582, 0x580, 0x584, 0x574, 0x565, 0x576, 0x57d, 0x57f,
0x561, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x539,
0x578, 0x582, 0x580, 0x584, 0x574, 0x565, 0x576, 0x57d, 0x57f, 0x561, 0x576, 0x56b,
0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x539, 0x578, 0x582, 0x580, 0x584, 0x574, 0x565,
0x576, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561,
0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54f,
0x578, 0x582, 0x57e, 0x561, 0x56c, 0x578, 0x582, 0x575, 0x56b, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x548, 0x582, 0x580, 0x578, 0x582, 0x563, 0x57e,
0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x548,
0x582, 0x580, 0x578, 0x582, 0x563, 0x57e, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f,
0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x548, 0x582, 0x580, 0x578, 0x582, 0x563, 0x57e, 0x561, 0x575, 0x56b,
0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x548, 0x582, 0x566, 0x562, 0x565, 0x56f, 0x57d,
0x57f, 0x561, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x548, 0x582, 0x566, 0x562, 0x565, 0x56f, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20,
0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x548, 0x582, 0x566, 0x562, 0x565, 0x56f, 0x57d, 0x57f,
0x561, 0x576, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54e, 0x561, 0x576, 0x578,
0x582, 0x561, 0x57f, 0x578, 0x582, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x54e, 0x561, 0x576, 0x578, 0x582, 0x561, 0x57f, 0x578, 0x582,
0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54e, 0x561, 0x576, 0x578, 0x582,
0x561, 0x57f, 0x578, 0x582, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561,
0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54e,
0x565, 0x576, 0x565, 0x57d, 0x578, 0x582, 0x565, 0x56c, 0x561, 0x575, 0x56b, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54e, 0x56c, 0x561, 0x564, 0x56b,
0x57e, 0x578, 0x57d, 0x57f, 0x578, 0x56f, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x54e, 0x56c, 0x561, 0x564, 0x56b, 0x57e, 0x578, 0x57d, 0x57f,
0x578, 0x56f, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54e, 0x56c, 0x561, 0x564,
0x56b, 0x57e, 0x578, 0x57d, 0x57f, 0x578, 0x56f, 0x56b, 0x20, 0x561, 0x574, 0x561,
0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x54e, 0x578, 0x56c, 0x563, 0x578, 0x563, 0x580, 0x561, 0x564, 0x56b, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54e, 0x578, 0x56c, 0x563, 0x578,
0x563, 0x580, 0x561, 0x564, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561,
0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54e, 0x578,
0x56c, 0x563, 0x578, 0x563, 0x580, 0x561, 0x564, 0x56b, 0x20, 0x561, 0x574, 0x561,
0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x54e, 0x578, 0x57d, 0x57f, 0x578, 0x56f, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x548, 0x582, 0x565, 0x575, 0x584, 0x20, 0x56f, 0x572,
0x566, 0x578, 0x582, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x548,
0x582, 0x578, 0x56c, 0x56b, 0x57d, 0x20, 0x587, 0x20, 0x556, 0x578, 0x582, 0x57f,
0x578, 0x582, 0x576, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x545, 0x561, 0x56f, 0x578, 0x582, 0x57f, 0x57d, 0x56f, 0x56b, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x545, 0x561, 0x56f, 0x578, 0x582,
0x57f, 0x57d, 0x56f, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580,
0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x545, 0x561, 0x56f,
0x578, 0x582, 0x57f, 0x57d, 0x56f, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561,
0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x535,
0x56f, 0x561, 0x57f, 0x565, 0x580, 0x56b, 0x576, 0x562, 0x578, 0x582, 0x580, 0x563,
0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x535, 0x56f, 0x561,
0x57f, 0x565, 0x580, 0x56b, 0x576, 0x562, 0x578, 0x582, 0x580, 0x563, 0x56b, 0x20,
0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x535, 0x56f, 0x561, 0x57f, 0x565, 0x580, 0x56b, 0x576,
0x562, 0x578, 0x582, 0x580, 0x563, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561,
0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x545,
0x578, 0x582, 0x56f, 0x578, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x986, 0x9ab, 0x997, 0x9be, 0x9a8, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be,
0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9a7, 0x9cd, 0x9af, 0x20,
0x986, 0x9ab, 0x9cd, 0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9aa, 0x9c2, 0x9ac, 0x20, 0x986, 0x9ab, 0x9cd, 0x9f0, 0x9bf, 0x995, 0x9be,
0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a6, 0x995, 0x9cd, 0x9b7, 0x9bf, 0x9a3,
0x20, 0x986, 0x9ab, 0x9cd, 0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20, 0x9ae, 0x9be,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae,
0x20, 0x986, 0x9ab, 0x9cd, 0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20, 0x986, 0x9ab, 0x9cd,
0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20, 0x986, 0x9ab, 0x9cd,
0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd,
0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986,
0x9b2, 0x9be, 0x9b8, 0x9cd, 0x995, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x986, 0x9b2, 0x9be, 0x9b8, 0x9cd, 0x995, 0x9be, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9b2, 0x9be, 0x9b8, 0x9cd, 0x995, 0x9be,
0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x986, 0x9ae, 0x9be, 0x99c, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x986, 0x9ae, 0x9be, 0x99c, 0x9a8, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x986, 0x9ae, 0x9be, 0x99c, 0x9a8, 0x9f0, 0x20, 0x997, 0x9cd,
0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9f0, 0x20, 0x986, 0x9ae, 0x9c7,
0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20, 0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd,
0x9f0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9a4, 0x9cd,
0x9a4, 0x9f0, 0x20, 0x986, 0x9ae, 0x9c7, 0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20,
0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9f0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9ae,
0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9f0,
0x20, 0x986, 0x9ae, 0x9c7, 0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20, 0x995, 0x9c7,
0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9f0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9a1, 0x9c7, 0x9b2,
0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9a4, 0x9cd, 0x9a4,
0x9f0, 0x20, 0x986, 0x9ae, 0x9c7, 0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20, 0x9aa,
0x9cd, 0x9f0, 0x9be, 0x99a, 0x9cd, 0x9af, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989,
0x9a4, 0x9cd, 0x9a4, 0x9f0, 0x20, 0x986, 0x9ae, 0x9c7, 0x9f0, 0x9bf, 0x995, 0x9be,
0x9f0, 0x20, 0x9aa, 0x9cd, 0x9f0, 0x9be, 0x99a, 0x9cd, 0x9af, 0x20, 0x9ae, 0x9be,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9f0, 0x20,
0x986, 0x9ae, 0x9c7, 0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20, 0x9aa, 0x9cd, 0x9f0,
0x9be, 0x99a, 0x9cd, 0x9af, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9f0, 0x20, 0x986, 0x9ae,
0x9c7, 0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20, 0x9aa, 0x9f0, 0x9cd, 0x9ac, 0x9cd,
0x9ac, 0x9a4, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9a4,
0x9cd, 0x9a4, 0x9f0, 0x20, 0x986, 0x9ae, 0x9c7, 0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0,
0x20, 0x9aa, 0x9f0, 0x9cd, 0x9ac, 0x9cd, 0x9ac, 0x9a4, 0x9c0, 0x9af, 0x9bc, 0x20,
0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9a4, 0x9cd, 0x9a4,
0x9f0, 0x20, 0x986, 0x9ae, 0x9c7, 0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20, 0x9aa,
0x9f0, 0x9cd, 0x9ac, 0x9cd, 0x9ac, 0x9a4, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9a1, 0x9c7,
0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9a4, 0x9cd,
0x9a4, 0x9f0, 0x20, 0x986, 0x9ae, 0x9c7, 0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20,
0x9aa, 0x9cd, 0x9f0, 0x9b6, 0x9be, 0x9a8, 0x9cd, 0x9a4, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9f0, 0x20, 0x986, 0x9ae, 0x9c7, 0x9f0, 0x9bf,
0x995, 0x9be, 0x9f0, 0x20, 0x9aa, 0x9cd, 0x9f0, 0x9b6, 0x9be, 0x9a8, 0x9cd, 0x9a4,
0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9a4, 0x9cd,
0x9a4, 0x9f0, 0x20, 0x986, 0x9ae, 0x9c7, 0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20,
0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986,
0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986,
0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x986, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9a1,
0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9f0,
0x9ac, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9f0, 0x9ac,
0x9c0, 0x9af, 0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x986, 0x9f0, 0x9ac, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987,
0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9f0, 0x9cd, 0x99c, 0x9c7, 0x9a3,
0x9cd, 0x99f, 0x9bf, 0x9a8, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986,
0x9f0, 0x9cd, 0x99c, 0x9c7, 0x9a3, 0x9cd, 0x99f, 0x9bf, 0x9a8, 0x9be, 0x9f0, 0x20,
0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9f0, 0x9cd, 0x99c,
0x9c7, 0x9a3, 0x9cd, 0x99f, 0x9bf, 0x9a8, 0x9be, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0,
0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9aa, 0x9be, 0x9b6, 0x9cd, 0x99a, 0x9be, 0x9a4, 0x9cd, 0x9af, 0x20,
0x986, 0x9f0, 0x9cd, 0x99c, 0x9c7, 0x9a3, 0x9cd, 0x99f, 0x9bf, 0x9a8, 0x9be, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x9b6, 0x9cd, 0x99a, 0x9be, 0x9a4,
0x9cd, 0x9af, 0x20, 0x986, 0x9f0, 0x9cd, 0x99c, 0x9c7, 0x9a3, 0x9cd, 0x99f, 0x9bf,
0x9a8, 0x9be, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9aa, 0x9be, 0x9b6, 0x9cd, 0x99a, 0x9be, 0x9a4, 0x9cd, 0x9af, 0x20, 0x986, 0x9f0,
0x9cd, 0x99c, 0x9c7, 0x9a3, 0x9cd, 0x99f, 0x9bf, 0x9a8, 0x9be, 0x9f0, 0x20, 0x997,
0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9f0, 0x9cd, 0x9ae, 0x9c7, 0x9a8, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9f0, 0x9cd, 0x9ae,
0x9c7, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9f0, 0x9cd, 0x9ae, 0x9c7, 0x9a8, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995,
0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x99f, 0x9b2,
0x9be, 0x9a3, 0x9cd, 0x99f, 0x9bf, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986,
0x99f, 0x9b2, 0x9be, 0x9a3, 0x9cd, 0x99f, 0x9bf, 0x995, 0x20, 0x9ae, 0x9be, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x99f, 0x9b2, 0x9be, 0x9a3, 0x9cd, 0x99f,
0x9bf, 0x995, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9ae, 0x9a7, 0x9cd, 0x9af, 0x20, 0x985, 0x9b7, 0x9cd, 0x99f, 0x9cd,
0x9f0, 0x9c7, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x985, 0x9b7, 0x9cd, 0x99f, 0x9cd, 0x9f0, 0x9c7, 0x9b2, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x9f0, 0x20, 0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9f0, 0x9c0, 0x9af,
0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9b7,
0x9cd, 0x99f, 0x9cd, 0x9f0, 0x9c7, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20,
0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9f0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9a1,
0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9b7,
0x9cd, 0x99f, 0x9cd, 0x9f0, 0x9c7, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20,
0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9f0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9aa,
0x9be, 0x9b6, 0x9cd, 0x99a, 0x9be, 0x9a4, 0x9cd, 0x9af, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x985, 0x9b7, 0x9cd, 0x99f, 0x9cd, 0x9f0, 0x9c7, 0x9b2, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x9f0, 0x20, 0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9f0, 0x9c0, 0x9af,
0x9bc, 0x20, 0x9aa, 0x9be, 0x9b6, 0x9cd, 0x99a, 0x9be, 0x9a4, 0x9cd, 0x9af, 0x20,
0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9b7, 0x9cd, 0x99f,
0x9cd, 0x9f0, 0x9c7, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x995, 0x9c7,
0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9f0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9aa, 0x9be, 0x9b6,
0x9cd, 0x99a, 0x9be, 0x9a4, 0x9cd, 0x9af, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987,
0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cd, 0x9f0, 0x9be, 0x99a, 0x9cd,
0x9af, 0x20, 0x985, 0x9b7, 0x9cd, 0x99f, 0x9cd, 0x9f0, 0x9c7, 0x9b2, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9b7, 0x9cd, 0x99f,
0x9cd, 0x9f0, 0x9c7, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9aa, 0x9cd,
0x9f0, 0x9be, 0x99a, 0x9cd, 0x9af, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x985, 0x9b7, 0x9cd, 0x99f, 0x9cd, 0x9f0, 0x9c7, 0x9b2, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x9f0, 0x20, 0x9aa, 0x9cd, 0x9f0, 0x9be, 0x99a, 0x9cd, 0x9af, 0x20,
0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa,
0x9be, 0x9b6, 0x9cd, 0x99a, 0x9be, 0x9a4, 0x9cd, 0x9af, 0x20, 0x985, 0x9b7, 0x9cd,
0x99f, 0x9cd, 0x9f0, 0x9c7, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x985, 0x9b7, 0x9cd, 0x99f, 0x9cd, 0x9f0, 0x9c7, 0x9b2, 0x9bf,
0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9aa, 0x9be, 0x9b6, 0x9cd, 0x99a, 0x9be, 0x9a4,
0x9cd, 0x9af, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985,
0x9b7, 0x9cd, 0x99f, 0x9cd, 0x9f0, 0x9c7, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0,
0x20, 0x9aa, 0x9be, 0x9b6, 0x9cd, 0x99a, 0x9be, 0x9a4, 0x9cd, 0x9af, 0x20, 0x9a1,
0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x99c,
0x9c7, 0x9f0, 0x9ac, 0x9be, 0x987, 0x99c, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x986, 0x99c, 0x9c7, 0x9f0, 0x9ac, 0x9be, 0x987, 0x99c, 0x9be, 0x9a8,
0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x99c,
0x9c7, 0x9f0, 0x9ac, 0x9be, 0x987, 0x99c, 0x9be, 0x9a8, 0x9f0, 0x20, 0x997, 0x9cd,
0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x986, 0x99c, 0x9cb, 0x9f0, 0x9c7, 0x99b, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x986, 0x99c, 0x9cb, 0x9f0, 0x9c7, 0x99b, 0x9f0, 0x20, 0x9ae, 0x9be,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x99c, 0x9cb, 0x9f0, 0x9c7, 0x99b,
0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2,
0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9be, 0x982, 0x9b2, 0x9be,
0x9a6, 0x9c7, 0x9b6, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9be, 0x982,
0x9b2, 0x9be, 0x9a6, 0x9c7, 0x9b6, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9be, 0x982, 0x9b2, 0x9be, 0x9a6, 0x9c7, 0x9b6, 0x9f0,
0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9c1, 0x99f, 0x9be, 0x9a8, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9b2, 0x9bf, 0x9ad, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9cd, 0x9f0, 0x9be, 0x99c,
0x9bf, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9ac, 0x9cd, 0x9f0, 0x9be, 0x99c, 0x9bf, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0,
0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9cd, 0x9f0,
0x9be, 0x99c, 0x9bf, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x997, 0x9cd,
0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9cd, 0x9f0, 0x9c1, 0x9a8, 0x9c7, 0x987, 0x20, 0x9a1,
0x9be, 0x9f0, 0x9c1, 0x99b, 0x9be, 0x9b2, 0x9c7, 0x9ae, 0x9f0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x995, 0x9c7, 0x9aa, 0x20, 0x9ad, 0x9be, 0x9f0, 0x9cd, 0x9a6, 0x9c7,
0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9c7, 0x9aa, 0x20, 0x9ad, 0x9be,
0x9f0, 0x9cd, 0x9a6, 0x9c7, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x995, 0x9c7, 0x9aa, 0x20, 0x9ad, 0x9be, 0x9f0, 0x9cd, 0x9a6, 0x9c7,
0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2,
0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9be, 0x9ae, 0x9cb, 0x9f0,
0x9cb, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a,
0x9be, 0x9a5, 0x9be, 0x9ae, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9be,
0x9a5, 0x9be, 0x9ae, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x99a, 0x9be, 0x9a5, 0x9be, 0x9ae, 0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be,
0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9bf, 0x9b2, 0x9bf, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9bf, 0x9b2, 0x9bf, 0x9f0, 0x20, 0x9ae,
0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9bf, 0x9b2, 0x9bf, 0x9f0,
0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9c0, 0x9a8, 0x9f0, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x99a, 0x9c0, 0x9a8, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9c0, 0x9a8, 0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2,
0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x996, 0x9cd, 0x9f0, 0x9c0,
0x9b7, 0x9cd, 0x99f, 0x9ae, 0x9be, 0x99b, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa,
0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cb, 0x995, 0x9cb, 0x99b, 0x20,
0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x9f0, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9b2, 0x9ae, 0x9cd, 0x9ac, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9b2, 0x9ae, 0x9cd, 0x9ac,
0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x995, 0x9b2, 0x9ae, 0x9cd, 0x9ac, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0,
0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9c1, 0x995, 0x20, 0x9a6, 0x9cd,
0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x9f0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x995, 0x9c1, 0x995, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa,
0x9c1, 0x99e, 0x9cd, 0x99c, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x995, 0x9c1, 0x995, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa,
0x9c1, 0x99e, 0x9cd, 0x99c, 0x9f0, 0x20, 0x985, 0x9f0, 0x9cd, 0x9a7, 0x20, 0x997,
0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9bf, 0x989, 0x9ac, 0x9be, 0x9f0, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x995, 0x9bf, 0x989, 0x9ac, 0x9be, 0x9f0, 0x20, 0x9ae, 0x9be,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9bf, 0x989, 0x9ac, 0x9be, 0x9f0,
0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9a1, 0x9c7, 0x9ad, 0x9bf, 0x99b, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a1,
0x9c1, 0x9ae, 0x9cb, 0x9a3, 0x9cd, 0x99f, 0x2d, 0x9a1, 0x9bf, 0x20, 0x986, 0x9f0,
0x9cd, 0x9ad, 0x9bf, 0x9b2, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2,
0x9ac, 0x20, 0x9a4, 0x9bf, 0x9ae, 0x9cb, 0x9f0, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x987, 0x9b7, 0x9cd, 0x99f, 0x9be, 0x9f0, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0,
0x9aa, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9b7, 0x9cd, 0x99f, 0x9be,
0x9f0, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9b7, 0x9cd, 0x99f, 0x9be, 0x9f0, 0x20,
0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7,
0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x987, 0x995, 0x9c1, 0x9f1, 0x9c7, 0x9a1, 0x9f0, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9ae, 0x9a7, 0x9cd, 0x9af, 0x20, 0x987, 0x989, 0x9f0, 0x9cb, 0x9aa, 0x9c0,
0x9af, 0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9a7, 0x9cd, 0x9af, 0x20,
0x987, 0x989, 0x9f0, 0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9a7, 0x9cd, 0x9af, 0x20, 0x987, 0x989,
0x9f0, 0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7,
0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9aa, 0x9cd, 0x9f0, 0x9be, 0x99a, 0x9cd, 0x9af, 0x20, 0x987, 0x989, 0x9f0, 0x9cb,
0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cd, 0x9f0,
0x9be, 0x99a, 0x9cd, 0x9af, 0x20, 0x987, 0x989, 0x9f0, 0x9cb, 0x9aa, 0x9c0, 0x9af,
0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cd,
0x9f0, 0x9be, 0x99a, 0x9cd, 0x9af, 0x20, 0x987, 0x989, 0x9f0, 0x9cb, 0x9aa, 0x9c0,
0x9af, 0x9bc, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be,
0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x997, 0x9cd, 0x9f0,
0x2d, 0x9aa, 0x9cd, 0x9f0, 0x9be, 0x99a, 0x9cd, 0x9af, 0x20, 0x987, 0x989, 0x9f0,
0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be,
0x9b6, 0x9cd, 0x99a, 0x9be, 0x9a4, 0x9cd, 0x9af, 0x20, 0x987, 0x989, 0x9f0, 0x9cb,
0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x9b6,
0x9cd, 0x99a, 0x9be, 0x9a4, 0x9cd, 0x9af, 0x20, 0x987, 0x989, 0x9f0, 0x9cb, 0x9aa,
0x9c0, 0x9af, 0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9aa, 0x9be, 0x9b6, 0x9cd, 0x99a, 0x9be, 0x9a4, 0x9cd, 0x9af, 0x20, 0x987, 0x989,
0x9f0, 0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7,
0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9ab, 0x995, 0x9b2, 0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0,
0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9ab, 0x995, 0x9b2, 0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0,
0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x995, 0x9b2, 0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x20,
0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x9f0, 0x20,
0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9bf, 0x99c, 0x9bf, 0x9f0, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9bf, 0x99c, 0x9bf, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9bf, 0x99c, 0x9bf, 0x9f0, 0x20, 0x997,
0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9cd, 0x9f0, 0x9be, 0x9a8, 0x9cd, 0x9b8, 0x20,
0x997, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9a6, 0x995, 0x9cd, 0x9b7, 0x9bf, 0x9a3, 0x20, 0x9ab, 0x9cd, 0x9f0, 0x9be, 0x9a8,
0x9cd, 0x9b8, 0x20, 0x986, 0x9f0, 0x9c1, 0x20, 0x98f, 0x9a3, 0x9cd, 0x99f, 0x9be,
0x9f0, 0x9cd, 0x995, 0x99f, 0x9bf, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x997,
0x9be, 0x9b2, 0x9be, 0x9aa, 0x9be, 0x997, 0x9cb, 0x99b, 0x9f0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x997, 0x9c7, 0x9ae, 0x9cd, 0x9ac, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99c, 0x9f0, 0x9cd, 0x99c, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99c, 0x9f0, 0x9cd, 0x99c, 0x9bf,
0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x99c, 0x9f0, 0x9cd, 0x99c, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x997,
0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x997, 0x9bf, 0x9b2, 0x9ac, 0x9be, 0x9f0, 0x9cd, 0x99f,
0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9a3, 0x989, 0x987,
0x99a, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2,
0x9ac, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9a3, 0x9b2, 0x9c7, 0x9a3, 0x9cd, 0x9a1,
0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9ac, 0x20, 0x997, 0x9cd,
0x9f0, 0x9c0, 0x9a3, 0x9b2, 0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x9f0, 0x20, 0x9ae, 0x9be,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9ac, 0x20, 0x997, 0x9cd,
0x9f0, 0x9c0, 0x9a3, 0x9b2, 0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x9f0, 0x20, 0x997, 0x9cd,
0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20, 0x997, 0x9cd,
0x9f0, 0x9c0, 0x9a3, 0x9b2, 0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x9f0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20, 0x997, 0x9cd, 0x9f0,
0x9c0, 0x9a3, 0x9b2, 0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20,
0x997, 0x9cd, 0x9f0, 0x9c0, 0x9a3, 0x9b2, 0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x9f0, 0x20,
0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9aa, 0x9b8, 0x9be, 0x997, 0x9f0, 0x9c0,
0x9af, 0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x997,
0x9be, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9b9, 0x9be, 0x9f1, 0x9be, 0x987, 0x2d, 0x98f, 0x9b2, 0x9bf, 0x989, 0x9b6, 0x9cd,
0x9ac, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9, 0x9be, 0x9f1, 0x9be,
0x987, 0x2d, 0x98f, 0x9b2, 0x9bf, 0x989, 0x9b6, 0x9cd, 0x9ac, 0x9a8, 0x9f0, 0x20,
0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9, 0x9be, 0x9f1, 0x9be,
0x987, 0x2d, 0x98f, 0x9b2, 0x9bf, 0x989, 0x9b6, 0x9cd, 0x9ac, 0x9a8, 0x9f0, 0x20,
0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9,
0x982, 0x20, 0x995, 0x999, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9, 0x982,
0x20, 0x995, 0x999, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9b9, 0x982, 0x20, 0x995, 0x999, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0,
0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9b9, 0x9cb, 0x9ad, 0x9cd, 0x200c, 0x9a1, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9b9, 0x9cb, 0x9ad, 0x9cd, 0x200c, 0x9a1, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9, 0x9cb, 0x9ad, 0x9cd, 0x200c, 0x9a1, 0x9f0,
0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9be, 0x9f0, 0x9a4, 0x9c0, 0x9af,
0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9be,
0x9f0, 0x9a4, 0x20, 0x9ae, 0x9b9, 0x9be, 0x9b8, 0x9be, 0x997, 0x9f0, 0x9c0, 0x9af,
0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9a3, 0x9cd, 0x9a1, 0x9cb, 0x99a,
0x9be, 0x987, 0x9a8, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9a7,
0x9cd, 0x9af, 0x20, 0x987, 0x9a3, 0x9cd, 0x9a1, 0x9cb, 0x9a8, 0x9c7, 0x99a, 0x9bf,
0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cd, 0x9f0,
0x9be, 0x99a, 0x9cd, 0x9af, 0x20, 0x987, 0x9a3, 0x9cd, 0x9a1, 0x9cb, 0x9a8, 0x9c7,
0x99a, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa,
0x9be, 0x9b6, 0x9cd, 0x99a, 0x9be, 0x9a4, 0x9cd, 0x9af, 0x20, 0x987, 0x9a3, 0x9cd,
0x9a1, 0x9cb, 0x9a8, 0x9c7, 0x99a, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x987, 0x9f0, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x987, 0x9f0, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x987, 0x9f0, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2,
0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9f0, 0x9cd, 0x995,
0x9c1, 0x99f, 0x9b8, 0x9cd, 0x995, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987,
0x9f0, 0x9cd, 0x995, 0x9c1, 0x99f, 0x9b8, 0x9cd, 0x995, 0x9f0, 0x20, 0x9ae, 0x9be,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9f0, 0x9cd, 0x995, 0x9c1, 0x99f,
0x9b8, 0x9cd, 0x995, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae,
0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x99c,
0x9f0, 0x9be, 0x987, 0x9b2, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x99c,
0x9f0, 0x9be, 0x987, 0x9b2, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x987, 0x99c, 0x9f0, 0x9be, 0x987, 0x9b2, 0x9f0, 0x20, 0x9a1, 0x9c7,
0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99c, 0x9be, 0x9aa,
0x9be, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99c, 0x9be, 0x9aa, 0x9be,
0x9a8, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99c,
0x9be, 0x9aa, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9be, 0x99c, 0x9be, 0x996, 0x9b8, 0x9cd,
0x9a4, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9ac,
0x20, 0x995, 0x9be, 0x99c, 0x9be, 0x996, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20,
0x995, 0x9be, 0x99c, 0x9be, 0x996, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x9f0, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cb, 0x9f0, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cb, 0x9f0, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cb,
0x9f0, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987,
0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cb, 0x99b, 0x9f0, 0x9be, 0x9af,
0x9bc, 0x9c7, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cd, 0x9f0, 0x9be, 0x99b,
0x9a8, 0x9cb, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x9f0, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cd, 0x9f0, 0x9be, 0x99b, 0x9a8, 0x9cb, 0x9af,
0x9bc, 0x9be, 0x9f0, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cd, 0x9f0, 0x9be, 0x99b, 0x9a8, 0x9cb,
0x9af, 0x9bc, 0x9be, 0x9f0, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x20, 0x997, 0x9cd, 0x9f0,
0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x995, 0x9bf, 0x9f0, 0x9cd, 0x997, 0x9bf, 0x99c, 0x9b8, 0x9cd, 0x9a4,
0x9be, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b2, 0x9be, 0x987, 0x9a8,
0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b2, 0x9f0, 0x9cd, 0x9a1, 0x20, 0x9b9, 0x9be,
0x993, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b2, 0x9f0, 0x9cd, 0x9a1, 0x20,
0x9b9, 0x9be, 0x993, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9b2, 0x9f0, 0x9cd, 0x9a1, 0x20, 0x9b9, 0x9be, 0x993, 0x9f0, 0x20, 0x9a1,
0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9be,
0x997, 0x9be, 0x9a6, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae,
0x9be, 0x997, 0x9be, 0x9a6, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9be, 0x997, 0x9be, 0x9a6, 0x9be, 0x9a8, 0x9f0,
0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9be, 0x9b2, 0x9af, 0x9bc, 0x9c7,
0x99a, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae,
0x9be, 0x9b2, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9ae, 0x9be, 0x9f0, 0x9cd, 0x995, 0x99b, 0x9c7, 0x99b, 0x9f0, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9be, 0x9f0, 0x9cd, 0x9b6, 0x9cd, 0x9ac, 0x9be, 0x9b2,
0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9f0, 0x9bf, 0x99b, 0x9be, 0x99b, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9f0, 0x9bf, 0x99b, 0x9be, 0x99b, 0x9f0,
0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9f0, 0x9bf,
0x99b, 0x9be, 0x99b, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae,
0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9be,
0x989, 0x99b, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9c7, 0x995,
0x9cd, 0x9b8, 0x9bf, 0x995, 0x9cb, 0x9f0, 0x20, 0x9aa, 0x9cd, 0x9f0, 0x9b6, 0x9be,
0x9a8, 0x9cd, 0x9a4, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9c7, 0x995, 0x9cd,
0x9b8, 0x9bf, 0x995, 0x9cb, 0x9f0, 0x20, 0x9aa, 0x9cd, 0x9f0, 0x9b6, 0x9be, 0x9a8,
0x9cd, 0x9a4, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae,
0x9c7, 0x995, 0x9cd, 0x9b8, 0x9bf, 0x995, 0x9cb, 0x9f0, 0x20, 0x9aa, 0x9cd, 0x9f0,
0x9b6, 0x9be, 0x9a8, 0x9cd, 0x9a4, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9b2, 0x9be, 0x9a8, 0x9ac, 0x9be, 0x99f,
0x9be, 0x9f0, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9b2, 0x9be, 0x9a8,
0x9ac, 0x9be, 0x99f, 0x9be, 0x9f0, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x989, 0x9b2, 0x9be, 0x9a8, 0x9ac, 0x9be, 0x99f, 0x9be, 0x9f0,
0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2,
0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9b8, 0x9cd, 0x995, 0x9cb,
0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9b8, 0x9cd, 0x995, 0x9cb, 0x9f0,
0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9b8, 0x9cd,
0x995, 0x9cb, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995,
0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9cd, 0x9af,
0x9be, 0x9a8, 0x9ae, 0x9be, 0x9f0, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8,
0x9be, 0x989, 0x9f0, 0x9c1, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9c7,
0x9aa, 0x9be, 0x9b2, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989,
0x20, 0x995, 0x9c7, 0x9b2, 0x9bf, 0x9a1, 0x9cb, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x20, 0x995, 0x9c7,
0x9b2, 0x9bf, 0x9a1, 0x9cb, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9ae,
0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x20, 0x995,
0x9c7, 0x9b2, 0x9bf, 0x9a1, 0x9cb, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20,
0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x99c, 0x9bf, 0x9b2, 0x9c7,
0x9a3, 0x9cd, 0x9a1, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989,
0x99c, 0x9bf, 0x9b2, 0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x99c, 0x9bf, 0x9b2, 0x9c7,
0x9a3, 0x9cd, 0x9a1, 0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x9ab, 0x9be, 0x989, 0x9a3, 0x9cd,
0x9a1, 0x9b2, 0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9a8, 0x9bf, 0x989, 0x9ab, 0x9be, 0x989, 0x9a3, 0x9cd, 0x9a1, 0x9b2, 0x9c7, 0x9a3,
0x9cd, 0x9a1, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9a8, 0x9bf, 0x989, 0x9ab, 0x9be, 0x989, 0x9a3, 0x9cd, 0x9a1, 0x9b2, 0x9c7, 0x9a3,
0x9cd, 0x9a1, 0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9c1, 0x9f0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9a8, 0x2019, 0x9f0, 0x9ab, 0x2019, 0x995, 0x20, 0x9a6, 0x9cd, 0x9ac,
0x9c0, 0x9aa, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x2019, 0x9f0, 0x9ab,
0x2019, 0x995, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9f0, 0x20, 0x9ae, 0x9be,
0x9a8, 0x9cd, 0x9af, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x2019, 0x9f0, 0x9ab,
0x2019, 0x995, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9f0, 0x20, 0x997, 0x9cd,
0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9be, 0x9f0, 0x9cd, 0x9a8, 0x9be, 0x9a3, 0x9cd, 0x9a1,
0x9cb, 0x20, 0x9a1, 0x9c7, 0x20, 0x9a8, 0x9cb, 0x9f0, 0x9cb, 0x9a8, 0x9cd, 0x200c,
0x9b9, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9be, 0x9f0, 0x9cd,
0x9a8, 0x9be, 0x9a3, 0x9cd, 0x9a1, 0x9cb, 0x20, 0x9a1, 0x9c7, 0x20, 0x9a8, 0x9cb,
0x9f0, 0x9cb, 0x9a8, 0x9cd, 0x200c, 0x9b9, 0x9be, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9be, 0x9f0, 0x9cd, 0x9a8, 0x9be, 0x9a3,
0x9cd, 0x9a1, 0x9cb, 0x20, 0x9a1, 0x9c7, 0x20, 0x9a8, 0x9cb, 0x9f0, 0x9cb, 0x9a8,
0x9cd, 0x200c, 0x9b9, 0x9be, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd,
0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8,
0x9ad, 0x9cb, 0x99b, 0x9bf, 0x9ac, 0x9bf, 0x9f0, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9ad, 0x9cb, 0x99b, 0x9bf, 0x9ac, 0x9bf,
0x9f0, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9ad, 0x9cb, 0x99b, 0x9bf, 0x9ac, 0x9bf, 0x9f0, 0x9cd,
0x9b8, 0x9cd, 0x995, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae,
0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x993, 0x9ae,
0x9b8, 0x9cd, 0x995, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x993, 0x9ae, 0x9b8,
0x9cd, 0x995, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x993, 0x9ae, 0x9b8, 0x9cd, 0x995, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7,
0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9aa, 0x9be, 0x995, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x995, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8,
0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be,
0x995, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0,
0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9aa, 0x9be, 0x9b2, 0x9be, 0x989, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9aa, 0x9be, 0x9aa, 0x9c1, 0x9f1, 0x9be, 0x20, 0x9a8, 0x9bf, 0x989, 0x20,
0x997, 0x9bf, 0x9a8, 0x9bf, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be,
0x9f0, 0x9be, 0x997, 0x9c1, 0x9f1, 0x9c7, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9aa, 0x9be, 0x9f0, 0x9be, 0x997, 0x9c1, 0x9f1, 0x9c7, 0x9f0, 0x20, 0x9ae, 0x9be,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x9f0, 0x9be, 0x997, 0x9c1,
0x9f1, 0x9c7, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995,
0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c7, 0x9f0,
0x9c1, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c7, 0x9f0, 0x9c1, 0x9f0,
0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c7, 0x9f0,
0x9c1, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be,
0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9bf, 0x9b2, 0x9bf,
0x9aa, 0x9be, 0x987, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9bf,
0x9b2, 0x9bf, 0x9aa, 0x9be, 0x987, 0x9a8, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9bf, 0x9b2, 0x9bf, 0x9aa, 0x9be, 0x987, 0x9a8,
0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2,
0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9bf, 0x9a8, 0x9bf, 0x995,
0x9cd, 0x9b8, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd,
0x99c, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99b, 0x9c7, 0x987, 0x9a3, 0x9cd,
0x99f, 0x20, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9c7, 0x9f0, 0x9c7, 0x20, 0x986, 0x9f0,
0x9c1, 0x20, 0x9ae, 0x9bf, 0x995, 0x9bf, 0x989, 0x9f1, 0x9c7, 0x9b2, 0x9a8, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99b, 0x9c7, 0x987, 0x9a3, 0x9cd, 0x99f, 0x20,
0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9c7, 0x9f0, 0x9c7, 0x20, 0x986, 0x9f0, 0x9c1, 0x20,
0x9ae, 0x9bf, 0x995, 0x9bf, 0x989, 0x9f1, 0x9c7, 0x9b2, 0x9a8, 0x9f0, 0x20, 0x9ae,
0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99b, 0x9c7, 0x987, 0x9a3, 0x9cd,
0x99f, 0x20, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9c7, 0x9f0, 0x9c7, 0x20, 0x986, 0x9f0,
0x9c1, 0x20, 0x9ae, 0x9bf, 0x995, 0x9bf, 0x989, 0x9f1, 0x9c7, 0x9b2, 0x9a8, 0x9f0,
0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9aa, 0x9bf, 0x99f, 0x995, 0x9c7, 0x987, 0x9f0, 0x9cd, 0x9a3, 0x9f0, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cb, 0x9a8, 0x9be, 0x9aa, 0x9c7, 0x9f0, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cd, 0x9af, 0x982, 0x9af, 0x9bc, 0x9be, 0x982, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9f0, 0x9bf, 0x987, 0x989, 0x9a8, 0x9bf, 0x9af,
0x9bc, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9f0, 0x9cb, 0x9a5, 0x9c7,
0x9f0, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99b, 0x9be, 0x996, 0x9be,
0x9b2, 0x9bf, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99b, 0x9be, 0x996,
0x9be, 0x9b2, 0x9bf, 0x9a8, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x99b, 0x9be, 0x996, 0x9be, 0x9b2, 0x9bf, 0x9a8, 0x9f0, 0x20, 0x997,
0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99b, 0x9be, 0x9ae, 0x9cb, 0x9f1, 0x9be, 0x9f0, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99b, 0x9be, 0x9ae, 0x9cb, 0x9f1, 0x9be, 0x9f0, 0x20,
0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99b, 0x9be, 0x9ae, 0x9cb,
0x9f1, 0x9be, 0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x99b, 0x9bf, 0x99a, 0x9bf, 0x9b2, 0x9bf, 0x99b, 0x9f0, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99b, 0x9bf, 0x982, 0x997, 0x9be, 0x9aa, 0x9c1, 0x9f0,
0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9cb,
0x9b2, 0x9cb, 0x9ae, 0x9cb, 0x9a8, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa,
0x9c1, 0x99e, 0x9cd, 0x99c, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a6, 0x995,
0x9cd, 0x9b7, 0x9bf, 0x9a3, 0x20, 0x99c, 0x9f0, 0x9cd, 0x99c, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99b, 0x9c1, 0x9f0, 0x9bf, 0x9a8,
0x9be, 0x9ae, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9cd, 0x9ac, 0x9cb,
0x9f1, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a4, 0x9be, 0x9b9, 0x9bf, 0x9a4,
0x9bf, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99f, 0x9be, 0x987, 0x9aa, 0x9c7,
0x987, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99f, 0x9be, 0x987, 0x9aa, 0x9c7,
0x987, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99f,
0x9be, 0x987, 0x9aa, 0x9c7, 0x987, 0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987,
0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a4, 0x9be, 0x99c, 0x9bf, 0x995, 0x9bf,
0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99f,
0x9cb, 0x995, 0x9c7, 0x9b2, 0x9be, 0x989, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x99f, 0x982, 0x997, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99f, 0x982,
0x997, 0x9be, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x99f, 0x982, 0x997, 0x9be, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd,
0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a,
0x9cd, 0x99a, 0x9c1, 0x995, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a4, 0x9c1,
0x9f0, 0x9cd, 0x995, 0x9ae, 0x9c7, 0x9a8, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8,
0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a4, 0x9c1, 0x9f0, 0x9cd, 0x995, 0x9ae,
0x9c7, 0x9a8, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9ae, 0x9be,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a4, 0x9c1, 0x9f0, 0x9cd, 0x995, 0x9ae,
0x9c7, 0x9a8, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x9f0, 0x20, 0x997, 0x9cd,
0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x99f, 0x9c1, 0x9ad, 0x9be, 0x9b2, 0x9be, 0x989, 0x9f0, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9f0, 0x9c1, 0x997, 0x9c1, 0x9f1, 0x9c7, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9f0, 0x9c1, 0x997, 0x9c1, 0x9f1, 0x9c7,
0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9f0,
0x9c1, 0x997, 0x9c1, 0x9f1, 0x9c7, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7,
0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x989, 0x99c, 0x9ac, 0x9c7, 0x995, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x99c, 0x9ac, 0x9c7, 0x995, 0x9bf, 0x9b8,
0x9cd, 0x9a4, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x989, 0x99c, 0x9ac, 0x9c7, 0x995, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be,
0x9a8, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be,
0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9be, 0x9a8, 0x9be,
0x99f, 0x9c1, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9be, 0x9a8, 0x9be,
0x99f, 0x9c1, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9ad, 0x9be, 0x9a8, 0x9be, 0x99f, 0x9c1, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0,
0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9ad, 0x9c7, 0x9a8, 0x9bf, 0x99c, 0x9c1, 0x9f1, 0x9c7, 0x9b2, 0x9be, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9cd, 0x9b2, 0x9be, 0x9a1, 0x9bf, 0x9ad,
0x9cb, 0x9b7, 0x9cd, 0x99f, 0x9cb, 0x995, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9ad, 0x9cd, 0x9b2, 0x9be, 0x9a1, 0x9bf, 0x9ad, 0x9cb, 0x9b7, 0x9cd, 0x99f, 0x9cb,
0x995, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad,
0x9cd, 0x9b2, 0x9be, 0x9a1, 0x9bf, 0x9ad, 0x9cb, 0x9b7, 0x9cd, 0x99f, 0x9cb, 0x995,
0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2,
0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9cb, 0x9b2, 0x9cd, 0x997,
0x9cb, 0x997, 0x9cd, 0x9f0, 0x9be, 0x9a1, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9ad, 0x9cb, 0x9b2, 0x9cd, 0x997, 0x9cb, 0x997, 0x9cd, 0x9f0, 0x9be, 0x9a1, 0x9f0,
0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9cb, 0x9b2,
0x9cd, 0x997, 0x9cb, 0x997, 0x9cd, 0x9f0, 0x9be, 0x9a1, 0x9f0, 0x20, 0x997, 0x9cd,
0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9cb, 0x9b7, 0x9cd, 0x99f, 0x995, 0x9f0, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9f1, 0x9c7, 0x995, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa,
0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9f1, 0x9be, 0x9b2, 0x9bf, 0x99b, 0x20,
0x986, 0x9f0, 0x9c1, 0x20, 0x9ab, 0x9c1, 0x99f, 0x9c1, 0x9a8, 0x9be, 0x9f0, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9af, 0x9bc, 0x9be, 0x995, 0x9c1, 0x9a4, 0x9cd, 0x9b8,
0x9cd, 0x995, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9af, 0x9bc, 0x9be, 0x995,
0x9c1, 0x9a4, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9af, 0x9bc, 0x9be, 0x995, 0x9c1, 0x9a4, 0x9cd, 0x9b8,
0x9cd, 0x995, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995,
0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9af, 0x9bc, 0x9c7,
0x995, 0x9be, 0x99f, 0x9c7, 0x9f0, 0x9bf, 0x9a3, 0x9ac, 0x9be, 0x9f0, 0x9cd, 0x997,
0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9af, 0x9bc, 0x9c7, 0x995, 0x9be, 0x99f,
0x9c7, 0x9f0, 0x9bf, 0x9a3, 0x9ac, 0x9be, 0x9f0, 0x9cd, 0x997, 0x9f0, 0x20, 0x9ae,
0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9af, 0x9bc, 0x9c7, 0x995, 0x9be,
0x99f, 0x9c7, 0x9f0, 0x9bf, 0x9a3, 0x9ac, 0x9be, 0x9f0, 0x9cd, 0x997, 0x9f0, 0x20,
0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9af, 0x9bc, 0x9c1, 0x995, 0x9cb, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41,
0x63, 0x72, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x41, 0x63, 0x72, 0x65, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61,
0x20, 0x64, 0x2019, 0x41, 0x63, 0x72, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0xe1, 0x6e,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0xc1, 0x66, 0x72, 0x69, 0x63,
0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x53, 0x75, 0x64, 0xe1, 0x66, 0x72, 0x69, 0x63, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0xc1, 0x66, 0x72, 0x69, 0x63,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6f, 0x65, 0x73, 0x74, 0x65, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x2019, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x6f, 0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0xc1,
0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6f, 0x65,
0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69,
0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x6c, 0x6d, 0x61, 0x74,
0x79, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x2019, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61,
0x20, 0x64, 0x2019, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x41, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61,
0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x65, 0x73, 0x74, 0x65, 0x20, 0x6e,
0x6f, 0x72, 0x74, 0x65, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e,
0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x65, 0x73, 0x74, 0x65, 0x20,
0x6e, 0x6f, 0x72, 0x74, 0x65, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61,
0x6e, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69,
0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x65, 0x73, 0x74, 0x65,
0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0x6e, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x73, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0xf1, 0x65, 0x73, 0x20,
0x6e, 0x6f, 0x72, 0x74, 0x65, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61,
0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20,
0x6d, 0x6f, 0x6e, 0x74, 0x61, 0xf1, 0x65, 0x73, 0x20, 0x6e, 0x6f, 0x72,
0x74, 0x65, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x65, 0x73,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x6d, 0x6f, 0x6e,
0x74, 0x61, 0xf1, 0x65, 0x73, 0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69,
0x63, 0x75, 0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x63, 0x61, 0x6e, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50,
0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x75, 0x20, 0x6e, 0x6f, 0x72, 0x74,
0x65, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x75, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x75,
0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0x6e, 0x75, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x6e,
0x61, 0x64, 0x79, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x41, 0x6e, 0x61, 0x64,
0x79, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69,
0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x70, 0x69, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x2019, 0x41, 0x70, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x41,
0x70, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x71,
0x74, 0x61, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x41, 0x71, 0x74, 0x61, 0x75,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67,
0x61, 0x20, 0x64, 0x2019, 0x41, 0x71, 0x74, 0x61, 0x75, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x2019, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019,
0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x41,
0x72, 0x61, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72,
0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x61,
0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x72,
0x78, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x41,
0x72, 0x78, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019,
0x41, 0x72, 0x78, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x20, 0x64, 0x2019, 0x41, 0x72, 0x78, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x20, 0x64, 0x2019, 0x41, 0x72, 0x78, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67,
0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x20, 0x64, 0x2019, 0x41, 0x72, 0x78, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x6d, 0x65, 0x6e,
0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65,
0x67, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x41, 0x74, 0x6c,
0xe1, 0x6e, 0x74, 0x69, 0x63, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x41, 0x74, 0x6c, 0xe1, 0x6e, 0x74, 0x69, 0x63, 0x75, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x41, 0x74, 0x6c, 0xe1, 0x6e, 0x74, 0x69, 0x63, 0x75,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64,
0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6f,
0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6f, 0x65, 0x73, 0x74, 0x65, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61,
0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x6f, 0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72,
0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x65,
0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x6f, 0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6f,
0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61,
0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6f, 0x65,
0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x61, 0x69, 0x78, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019,
0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x78, 0xe1, 0x6e, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20,
0x64, 0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x78, 0xe1, 0x6e,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20,
0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x73, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4c, 0x65, 0x73, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x78, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x78, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x78, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x75, 0x74, 0xe1, 0x6e, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72,
0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62,
0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42,
0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x42, 0x72, 0x75, 0x6e, 0xe9, 0x69, 0x20, 0x44, 0x61,
0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x62, 0x75, 0x20, 0x56, 0x65, 0x72,
0x64, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x62, 0x75, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72,
0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61,
0x62, 0x75, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x73, 0x65, 0x79, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65,
0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69,
0x6c, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67,
0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64,
0x65, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x20, 0x49, 0x73, 0x6c, 0x6c, 0x61, 0x20, 0x43, 0x68, 0x72,
0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x6c, 0x65, 0x73,
0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61,
0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20,
0x49, 0x73, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x6c,
0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x6d, 0x65, 0x64, 0x69, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65,
0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x49, 0x73,
0x6c, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62,
0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43,
0x75, 0x62, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x44,
0x61, 0x76, 0x69, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76,
0x69, 0x6c, 0x6c, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61,
0x20, 0x49, 0x73, 0x6c, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61,
0x73, 0x63, 0x75, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20,
0x49, 0x73, 0x6c, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73,
0x63, 0x75, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e,
0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x49,
0x73, 0x6c, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63,
0x75, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x63, 0x75,
0x61, 0x64, 0x6f, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x45, 0x73, 0x74, 0x65, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x45, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62,
0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x45, 0x73, 0x74,
0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x65, 0x73, 0x74, 0x72, 0x65,
0x6d, 0x75, 0x20, 0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61,
0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73,
0x20, 0x49, 0x73, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73,
0x20, 0x49, 0x73, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61,
0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73,
0x20, 0x49, 0x73, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x46, 0x69, 0x78, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x78,
0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65,
0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x78, 0x69, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x61, 0x20, 0x47, 0x75, 0x79,
0x61, 0x6e, 0x61, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x73, 0x75, 0x72,
0x20, 0x79, 0x20, 0x6c, 0x2019, 0x61, 0x6e, 0x74, 0xe1, 0x72, 0x74, 0x69,
0x63, 0x75, 0x20, 0x66, 0x72, 0x61, 0x6e, 0x63, 0xe9, 0x73, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x61, 0x6c, 0xe1, 0x70, 0x61,
0x67, 0x6f, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47,
0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x58, 0x65, 0x6f, 0x72, 0x78, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x58, 0x65, 0x6f, 0x72, 0x78, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20,
0x58, 0x65, 0x6f, 0x72, 0x78, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x6c, 0x65, 0x73,
0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x6d, 0x65, 0x64, 0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67,
0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20,
0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20,
0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x47, 0x75, 0x61, 0x6d, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x47, 0x6f, 0x6c, 0x66, 0x75, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x65, 0x73, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65,
0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x69, 0x61, 0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x1e24, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x1e24, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e,
0x67, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65,
0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x1e24, 0x6f, 0x6e, 0x67, 0x20, 0x4b,
0x6f, 0x6e, 0x67, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x48,
0x6f, 0x76, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67,
0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4f, 0x63, 0xe9, 0x61, 0x6e,
0x75, 0x20, 0xcd, 0x6e, 0x64, 0x69, 0x63, 0x75, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e,
0x65, 0x73, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e,
0x65, 0x73, 0x69, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x65, 0x73, 0x74,
0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x49, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6f, 0x65,
0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x49, 0x72,
0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x49, 0x72, 0xe1, 0x6e, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20,
0x64, 0x2019, 0x49, 0x72, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x49,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x49,
0x73, 0x72, 0x61, 0x65, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72,
0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x49, 0x73, 0x72,
0x61, 0x65, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x58,
0x61, 0x70, 0xf3, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x58, 0x61, 0x70,
0xf3, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69,
0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x58, 0x61, 0x70, 0xf3, 0x6e,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72,
0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61,
0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76,
0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b,
0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65,
0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70,
0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63,
0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x73, 0x74, 0xe1, 0x6e,
0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x73,
0x74, 0xe1, 0x6e, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f,
0x72, 0x65, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x72, 0x65,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65,
0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61,
0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79,
0x61, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x4b, 0x69, 0x72, 0x67, 0x75, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x61, 0x6e, 0x6b, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20,
0x49, 0x73, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x4c, 0x69, 0x6e, 0x65, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62,
0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x63, 0xe1, 0x75, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x61, 0x63, 0xe1, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x61, 0x63, 0xe1, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x6c, 0x61, 0x73, 0x69, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x65, 0x73, 0x20, 0x4d,
0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75,
0x65, 0x73, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x65, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x4d,
0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x75, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x75,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69,
0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x77,
0x73, 0x6f, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x75, 0x20, 0x64, 0x65, 0x20,
0x4d, 0xe9, 0x78, 0x69, 0x63, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x75, 0x20, 0x64, 0x65, 0x20,
0x4d, 0xe9, 0x78, 0x69, 0x63, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62,
0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x75, 0x20, 0x64, 0x65, 0x20,
0x4d, 0xe9, 0x78, 0x69, 0x63, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x55, 0x6c, 0xe1, 0x6e, 0x20, 0x42, 0xe1, 0x74, 0x6f, 0x72, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x2019, 0x55, 0x6c, 0xe1, 0x6e, 0x20, 0x42, 0xe1, 0x74, 0x6f,
0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65,
0x67, 0x61, 0x20, 0x64, 0x2019, 0x55, 0x6c, 0xe1, 0x6e, 0x20, 0x42, 0xe1,
0x74, 0x6f, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x6f, 0x73, 0x63, 0xfa, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73,
0x63, 0xfa, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69,
0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0xfa,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x79, 0x61, 0x6e,
0x6d, 0x61, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x61, 0x75, 0x72, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x75, 0x65, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x75,
0x65, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65,
0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x75, 0x65, 0x76, 0x61, 0x20,
0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x75, 0x65, 0x76, 0x61, 0x20, 0x5a,
0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x75, 0x65, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x75, 0x65, 0x76, 0x61, 0x20, 0x5a,
0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f,
0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x69, 0x75, 0x65,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x49,
0x73, 0x6c, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x49, 0x73, 0x6c, 0x6c,
0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x20, 0x49, 0x73, 0x6c, 0x6c, 0x61, 0x20, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64,
0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x49, 0x73, 0x6c,
0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x6e, 0x65, 0x73,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x4f, 0x6d, 0x73, 0x6b,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x2019, 0x4f, 0x6d, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019,
0x4f, 0x6d, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x50, 0x61, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe1,
0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65,
0x67, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x71, 0x75, 0x69,
0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x50, 0x61, 0x6c, 0x61, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x50, 0x61, 0x70, 0xfa, 0x61, 0x20, 0x4e, 0x75, 0x65, 0x76, 0x61,
0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0xe1, 0x69,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0xe1, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69,
0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0xe1, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x50, 0x65, 0x72, 0xfa, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50,
0x65, 0x72, 0xfa, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e,
0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x65, 0x72,
0xfa, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6c,
0x69, 0x70, 0x69, 0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x46,
0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x75, 0x20, 0x64, 0x65,
0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x73, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x49, 0x73,
0x6c, 0x6c, 0x65, 0x73, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x69, 0x6e,
0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x79, 0x20, 0x4d,
0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x20, 0x79, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x20, 0x79, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50,
0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61,
0x6e, 0x67, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x51, 0x79,
0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64,
0x65, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x52, 0x65, 0x75, 0x6e, 0x69,
0xf3, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x52, 0x6f,
0x74, 0x68, 0x65, 0x72, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x53, 0x61, 0x78, 0x61, 0x6c, 0xed, 0x6e, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x53, 0x61, 0x78, 0x61, 0x6c, 0xed, 0x6e, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65,
0x20, 0x53, 0x61, 0x78, 0x61, 0x6c, 0xed, 0x6e, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64,
0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4c, 0x65, 0x73, 0x20, 0x53, 0x65, 0x69, 0x78, 0x65, 0x6c, 0x65,
0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70,
0x75, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65,
0x73, 0x20, 0x49, 0x73, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x53, 0x61, 0x6c,
0x6f, 0x6d, 0xf3, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x58, 0x65, 0x6f, 0x72, 0x78, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x53,
0x75, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x53,
0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0xe9, 0x69, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0xe9, 0x69, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20,
0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0xe9, 0x69, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x54, 0x61, 0x78, 0x69, 0x71, 0x75,
0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x54, 0x75, 0x72,
0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54,
0x75, 0x76, 0x61, 0x6c, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0xe1, 0x69, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0xe1, 0x69, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0xe1, 0x69, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x55, 0x7a, 0x62, 0x65,
0x71, 0x75, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x55, 0x7a, 0x62, 0x65, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe1, 0x6e,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x71, 0x75,
0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67,
0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0xe1, 0x75, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0xe1, 0x75, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61,
0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0xe1,
0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x20, 0x49, 0x73, 0x6c, 0x6c, 0x61, 0x20, 0x57, 0x61, 0x6b, 0x65,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x57, 0x61, 0x6c, 0x6c,
0x69, 0x73, 0x20, 0x79, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69,
0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x59, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x75,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20,
0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72,
0x67, 0x75, 0x18f, 0x66, 0x71, 0x61, 0x6e, 0x131, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x259, 0x72, 0x6b, 0x259, 0x7a,
0x69, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x15e, 0x259, 0x72, 0x71, 0x69, 0x20, 0x41, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x43, 0x259, 0x6e, 0x75,
0x62, 0x69, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x51, 0x259, 0x72, 0x62, 0x69, 0x20, 0x41, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x51, 0x259, 0x72,
0x62, 0x69, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x51, 0x259, 0x72, 0x62, 0x69, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x41, 0x6c,
0x79, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x41,
0x6c, 0x79, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x41, 0x6c, 0x79,
0x61, 0x73, 0x6b, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x41,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x15e, 0x69, 0x6d, 0x61, 0x6c, 0x69, 0x20, 0x4d, 0x259,
0x72, 0x6b, 0x259, 0x7a, 0x69, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x69, 0x6d, 0x61, 0x6c,
0x69, 0x20, 0x4d, 0x259, 0x72, 0x6b, 0x259, 0x7a, 0x69, 0x20, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x69, 0x6d, 0x61,
0x6c, 0x69, 0x20, 0x4d, 0x259, 0x72, 0x6b, 0x259, 0x7a, 0x69, 0x20, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x15e, 0x69, 0x6d, 0x61, 0x6c, 0x69, 0x20, 0x15e,
0x259, 0x72, 0x71, 0x69, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x69, 0x6d, 0x61, 0x6c, 0x69,
0x20, 0x15e, 0x259, 0x72, 0x71, 0x69, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x69, 0x6d, 0x61, 0x6c, 0x69, 0x20,
0x15e, 0x259, 0x72, 0x71, 0x69, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e,
0x69, 0x6d, 0x61, 0x6c, 0x69, 0x20, 0x44, 0x61, 0x11f, 0x6c, 0x131, 0x71,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x15e, 0x69, 0x6d, 0x61, 0x6c, 0x69, 0x20, 0x44, 0x61, 0x11f,
0x6c, 0x131, 0x71, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x15e, 0x69, 0x6d, 0x61, 0x6c, 0x69, 0x20, 0x44, 0x61, 0x11f,
0x6c, 0x131, 0x71, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x69, 0x6d,
0x61, 0x6c, 0x69, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x53, 0x61, 0x6b, 0x69, 0x74, 0x20, 0x4f, 0x6b, 0x65, 0x61, 0x6e, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x69, 0x6d, 0x61, 0x6c, 0x69, 0x20,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x53, 0x61, 0x6b, 0x69,
0x74, 0x20, 0x4f, 0x6b, 0x65, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x69,
0x6d, 0x61, 0x6c, 0x69, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x53, 0x61, 0x6b, 0x69, 0x74, 0x20, 0x4f, 0x6b, 0x65, 0x61, 0x6e,
0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x41, 0x70,
0x69, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x41, 0x70, 0x69, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x41, 0x70, 0x69, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x18f, 0x72, 0x259, 0x62, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x18f, 0x72, 0x259, 0x62,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x18f, 0x72, 0x259, 0x62,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x51, 0x259, 0x72, 0x62, 0x69, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x51, 0x259,
0x72, 0x62, 0x69, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x51, 0x259, 0x72, 0x62, 0x69, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x45, 0x72, 0x6d, 0x259, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x45, 0x72, 0x6d,
0x259, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x45, 0x72,
0x6d, 0x259, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x59, 0x61, 0x79,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x6b, 0x20, 0x56, 0x61, 0x78, 0x74, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x56, 0x61, 0x78, 0x74, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x6b, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d,
0x259, 0x72, 0x6b, 0x259, 0x7a, 0x69, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d,
0x259, 0x72, 0x6b, 0x259, 0x7a, 0x69, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x259, 0x72, 0x6b,
0x259, 0x7a, 0x69, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x79, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x4d, 0x259, 0x72, 0x6b, 0x259, 0x7a, 0x69, 0x20, 0x51, 0x259, 0x72, 0x62,
0x69, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x259, 0x72, 0x6b, 0x259, 0x7a,
0x69, 0x20, 0x51, 0x259, 0x72, 0x62, 0x69, 0x20, 0x41, 0x76, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x259, 0x72,
0x6b, 0x259, 0x7a, 0x69, 0x20, 0x51, 0x259, 0x72, 0x62, 0x69, 0x20, 0x41,
0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x59, 0x61,
0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x259, 0x72, 0x71, 0x69,
0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x259, 0x72, 0x71, 0x69, 0x20, 0x41,
0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x15e, 0x259, 0x72, 0x71, 0x69, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x79, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x41, 0x7a, 0x259, 0x72, 0x62, 0x61, 0x79, 0x63, 0x61, 0x6e,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x41, 0x7a, 0x259, 0x72, 0x62, 0x61,
0x79, 0x63, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x41, 0x7a, 0x259, 0x72, 0x62,
0x61, 0x79, 0x63, 0x61, 0x6e, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x41, 0x7a, 0x6f, 0x72, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x41, 0x7a, 0x6f, 0x72, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x41, 0x7a, 0x6f, 0x72,
0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x42, 0x61,
0x6e, 0x71, 0x6c, 0x61, 0x64, 0x65, 0x15f, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x42, 0x61, 0x6e, 0x71, 0x6c, 0x61, 0x64, 0x65, 0x15f, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x42, 0x61, 0x6e, 0x71, 0x6c, 0x61, 0x64, 0x65, 0x15f, 0x20, 0x59,
0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x42, 0x75, 0x74, 0x61,
0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x42, 0x6f, 0x6c, 0x69, 0x76,
0x69, 0x79, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x42, 0x72, 0x61,
0x7a, 0x69, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x59, 0x61,
0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x42, 0x72, 0x75, 0x6e, 0x65,
0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d,
0x20, 0x76, 0x61, 0x78, 0x74, 0x131, 0x4b, 0x61, 0x70, 0x65, 0x20, 0x56,
0x65, 0x72, 0x64, 0x65, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b, 0x61,
0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b,
0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x59, 0x61,
0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0xc7, 0x61, 0x6d, 0x6f, 0x72,
0x72, 0x6f, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0xc7, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0xc7, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0xc7, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x20,
0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0xc7, 0x69, 0x6c,
0x69, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0xc7, 0x69, 0x6c, 0x69, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0xc7, 0x69, 0x6c, 0x69, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0xc7, 0x69, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0xc7, 0x69, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0xc7, 0x69, 0x6e, 0x20, 0x59,
0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x69, 0x6c, 0x61,
0x64, 0x20, 0x41, 0x64, 0x61, 0x73, 0x131, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61,
0x72, 0x131, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b, 0x6f, 0x6c, 0x75,
0x6d, 0x62, 0x69, 0x79, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b,
0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x79, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b,
0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x79, 0x61, 0x20, 0x59, 0x61, 0x79,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b, 0x75, 0x6b, 0x20, 0x41, 0x64,
0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b,
0x75, 0x6b, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x4b, 0x75, 0x6b, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131,
0x20, 0x59, 0x61, 0x72, 0x131, 0x6d, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x4b, 0x75, 0x62, 0x61, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x4b, 0x75, 0x62, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b, 0x75, 0x62,
0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x44,
0x65, 0x76, 0x69, 0x73, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x44, 0xfc,
0x6d, 0x6f, 0x6e, 0x2d, 0x64, 0x2019, 0xdc, 0x72, 0x76, 0x69, 0x6c, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x259, 0x72, 0x71, 0x69, 0x20, 0x54,
0x69, 0x6d, 0x6f, 0x72, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x50, 0x61,
0x73, 0x78, 0x61, 0x20, 0x41, 0x64, 0x61, 0x73, 0x131, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x50, 0x61, 0x73, 0x78, 0x61, 0x20, 0x41, 0x64, 0x61,
0x73, 0x131, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x50, 0x61, 0x73, 0x78, 0x61, 0x20, 0x41,
0x64, 0x61, 0x73, 0x131, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x45, 0x6b, 0x76, 0x61, 0x64, 0x6f, 0x72, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x4d, 0x259, 0x72, 0x6b, 0x259, 0x7a, 0x69, 0x20, 0x41,
0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d,
0x259, 0x72, 0x6b, 0x259, 0x7a, 0x69, 0x20, 0x41, 0x76, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x4d, 0x259, 0x72, 0x6b, 0x259, 0x7a, 0x69, 0x20,
0x41, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x15e, 0x259, 0x72, 0x71, 0x69, 0x20, 0x41, 0x76,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x259,
0x72, 0x71, 0x69, 0x20, 0x41, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x15e, 0x259, 0x72, 0x71, 0x69, 0x20, 0x41, 0x76, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b,
0x259, 0x6e, 0x61, 0x72, 0x20, 0x15e, 0x259, 0x72, 0x71, 0x69, 0x20, 0x41,
0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x51,
0x259, 0x72, 0x62, 0x69, 0x20, 0x41, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x51, 0x259, 0x72, 0x62, 0x69, 0x20, 0x41,
0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x51, 0x259, 0x72, 0x62,
0x69, 0x20, 0x41, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x59, 0x61, 0x79,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x65,
0x6e, 0x64, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x65, 0x6e, 0x64,
0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x46,
0x6f, 0x6c, 0x6b, 0x6c, 0x65, 0x6e, 0x64, 0x20, 0x41, 0x64, 0x61, 0x6c,
0x61, 0x72, 0x131, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x46, 0x69, 0x63, 0x69, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x46,
0x69, 0x63, 0x69, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x46, 0x69, 0x63, 0x69, 0x20, 0x59,
0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x46, 0x72, 0x61, 0x6e,
0x73, 0x131, 0x7a, 0x20, 0x51, 0x76, 0x69, 0x61, 0x6e, 0x61, 0x73, 0x131,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x46, 0x72, 0x61, 0x6e, 0x73, 0x131,
0x7a, 0x20, 0x43, 0x259, 0x6e, 0x75, 0x62, 0x69, 0x20, 0x76, 0x259, 0x20,
0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x6b, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x51, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x71, 0x6f, 0x73,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x51, 0x61, 0x6d, 0x62, 0x69, 0x65,
0x72, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x47, 0x75, 0x72, 0x63, 0xfc,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x47, 0x75,
0x72, 0x63, 0xfc, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x47, 0x75,
0x72, 0x63, 0xfc, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x59, 0x61, 0x79, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74,
0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x51, 0x72, 0x69, 0x6e, 0x76, 0x69, 0xe7, 0x20, 0x4f, 0x72,
0x74, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x259, 0x72, 0x71,
0x69, 0x20, 0x51, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79,
0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x259, 0x72, 0x71, 0x69,
0x20, 0x51, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x15e, 0x259, 0x72, 0x71, 0x69, 0x20, 0x51, 0x72, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x59, 0x61, 0x79,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x51, 0x259, 0x72, 0x62, 0x69, 0x20,
0x51, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x51, 0x259, 0x72, 0x62, 0x69, 0x20, 0x51,
0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x51, 0x259, 0x72, 0x62, 0x69, 0x20, 0x51, 0x72, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x4b, 0xf6, 0x72, 0x66, 0x259, 0x7a, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x51, 0x61, 0x79, 0x61, 0x6e, 0x61, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x48, 0x61, 0x76, 0x61, 0x79, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x48, 0x61, 0x76,
0x61, 0x79, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x48,
0x61, 0x76, 0x61, 0x79, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x20, 0x59,
0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x48, 0x6f, 0x6e, 0x71,
0x20, 0x4b, 0x6f, 0x6e, 0x71, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x48,
0x6f, 0x6e, 0x71, 0x20, 0x4b, 0x6f, 0x6e, 0x71, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x48,
0x6f, 0x6e, 0x71, 0x20, 0x4b, 0x6f, 0x6e, 0x71, 0x20, 0x59, 0x61, 0x79,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x48,
0x6f, 0x76, 0x64, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x48, 0x69, 0x6e, 0x64, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x48, 0x69, 0x6e, 0x64, 0x20, 0x4f, 0x6b, 0x65,
0x61, 0x6e, 0x131, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x48, 0x69, 0x6e,
0x64, 0xe7, 0x69, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x259,
0x72, 0x6b, 0x259, 0x7a, 0x69, 0x20, 0x130, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x7a, 0x69, 0x79, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x259,
0x72, 0x71, 0x69, 0x20, 0x130, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69,
0x79, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x51, 0x259, 0x72, 0x62,
0x69, 0x20, 0x130, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x79, 0x61,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x130, 0x72, 0x61, 0x6e, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x130, 0x72, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x130,
0x72, 0x61, 0x6e, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x130, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x130, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x130, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x59, 0x61, 0x79, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x130, 0x73, 0x72, 0x61, 0x69, 0x6c, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x130, 0x73, 0x72, 0x61, 0x69, 0x6c, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x130, 0x73, 0x72, 0x61, 0x69, 0x6c, 0x20, 0x59, 0x61, 0x79,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x59, 0x61, 0x70, 0x6f, 0x6e, 0x69,
0x79, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x59, 0x61, 0x70, 0x6f,
0x6e, 0x69, 0x79, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x59, 0x61, 0x70, 0x6f, 0x6e,
0x69, 0x79, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x51, 0x61, 0x7a, 0x61, 0x78, 0x131, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x76, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x259, 0x72, 0x71, 0x69, 0x20, 0x51,
0x61, 0x7a, 0x61, 0x78, 0x131, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x51, 0x259, 0x72, 0x62, 0x69, 0x20, 0x51, 0x61, 0x7a,
0x61, 0x78, 0x131, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x4b, 0x6f, 0x72, 0x65, 0x79, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x4b, 0x6f, 0x72, 0x65, 0x79, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b, 0x6f,
0x72, 0x65, 0x79, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x4b, 0x6f, 0x72, 0x73, 0x65, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x79, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x59, 0x61, 0x79, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x51, 0x131, 0x72, 0x11f, 0x131, 0x7a, 0x131,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4c, 0x61,
0x79, 0x6e, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x61, 0x75,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x61, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x61,
0x75, 0x20, 0x59, 0x61, 0x79, 0x20, 0x76, 0x61, 0x78, 0x74, 0x131, 0x4d,
0x61, 0x71, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x4d, 0x61, 0x71, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x61,
0x71, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x4d, 0x61, 0x6c, 0x61, 0x79, 0x7a, 0x69, 0x79, 0x61,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x61, 0x72, 0x6b, 0x65, 0x73,
0x61, 0x73, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x61, 0x72, 0x15f,
0x61, 0x6c, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x4d, 0x61, 0x76, 0x72, 0x69, 0x6b, 0x69, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x61, 0x76, 0x72, 0x69, 0x6b, 0x69,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x4d, 0x61, 0x76, 0x72, 0x69, 0x6b, 0x69, 0x20, 0x59,
0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x6f, 0x75, 0x73,
0x6f, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x65, 0x6b, 0x73,
0x69, 0x6b, 0x61, 0x20, 0x53, 0x61, 0x6b, 0x69, 0x74, 0x20, 0x4f, 0x6b,
0x65, 0x61, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x65, 0x6b,
0x73, 0x69, 0x6b, 0x61, 0x20, 0x53, 0x61, 0x6b, 0x69, 0x74, 0x20, 0x4f,
0x6b, 0x65, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x65, 0x6b, 0x73, 0x69,
0x6b, 0x61, 0x20, 0x53, 0x61, 0x6b, 0x69, 0x74, 0x20, 0x4f, 0x6b, 0x65,
0x61, 0x6e, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72,
0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x6f,
0x73, 0x6b, 0x76, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x6f,
0x73, 0x6b, 0x76, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x6f, 0x73, 0x6b, 0x76,
0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x76, 0x61, 0x78, 0x74, 0x131, 0x4d,
0x79, 0x61, 0x6e, 0x6d, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4e,
0x61, 0x75, 0x72, 0x75, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4e, 0x65,
0x70, 0x61, 0x6c, 0x20, 0x76, 0x61, 0x78, 0x74, 0x131, 0x59, 0x65, 0x6e,
0x69, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x79, 0x61,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x59, 0x65, 0x6e, 0x69, 0x20, 0x4b,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x79, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x59, 0x65, 0x6e, 0x69, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x69, 0x79, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x59, 0x65, 0x6e, 0x69, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x79, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x59, 0x65, 0x6e,
0x69, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x59, 0x65, 0x6e, 0x69, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x79, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x4e, 0x79, 0x75, 0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65,
0x6e, 0x64, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4e, 0x79, 0x75, 0x66,
0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4e,
0x79, 0x75, 0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x20,
0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4e, 0x69, 0x75,
0x65, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x20, 0x41, 0x64, 0x61, 0x73, 0x131, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x41, 0x64,
0x61, 0x73, 0x131, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x20, 0x41, 0x64, 0x61, 0x73, 0x131, 0x20, 0x59, 0x61, 0x79, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x79, 0x61,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x79,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x79, 0x61, 0x20,
0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4f,
0x6d, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x59,
0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x50, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x78, 0x74, 0x131, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x59, 0x65, 0x6e, 0x69,
0x20, 0x51, 0x76, 0x69, 0x6e, 0x65, 0x79, 0x61, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x50, 0x61, 0x72, 0x61, 0x71, 0x76, 0x61, 0x79, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x50, 0x61, 0x72, 0x61, 0x71, 0x76, 0x61, 0x79,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x50, 0x61, 0x72, 0x61, 0x71, 0x76, 0x61, 0x79, 0x20,
0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x50, 0x65, 0x72,
0x75, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x50, 0x65, 0x72, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x50, 0x65, 0x72, 0x75, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70,
0x69, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69,
0x6e, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x46,
0x65, 0x6e, 0x69, 0x6b, 0x73, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72,
0x131, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0xfc, 0x71, 0x259, 0x64,
0x64, 0x259, 0x73, 0x20, 0x50, 0x79, 0x65, 0x72, 0x20, 0x76, 0x259, 0x20,
0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x4d, 0xfc, 0x71, 0x259, 0x64, 0x64, 0x259, 0x73, 0x20, 0x50, 0x79,
0x65, 0x72, 0x20, 0x76, 0x259, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f,
0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x4d, 0xfc, 0x71, 0x259, 0x64, 0x64, 0x259, 0x73,
0x20, 0x50, 0x79, 0x65, 0x72, 0x20, 0x76, 0x259, 0x20, 0x4d, 0x69, 0x6b,
0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x50, 0x69, 0x74, 0x6b, 0x65, 0x72, 0x6e, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x50, 0x78, 0x65, 0x6e, 0x79, 0x61, 0x6e, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x52, 0x65, 0x79, 0x75, 0x6e, 0x79, 0x6f, 0x6e,
0x52, 0x6f, 0x74, 0x65, 0x72, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x53, 0x61, 0x78, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x53, 0x61, 0x78, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x53,
0x61, 0x78, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20, 0x76,
0x61, 0x78, 0x74, 0x131, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x78, 0x74,
0x131, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20, 0x79, 0x61, 0x79, 0x20,
0x76, 0x61, 0x78, 0x74, 0x131, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x53, 0x65, 0x79, 0x15f, 0x65, 0x6c, 0x20, 0x41, 0x64,
0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x53,
0x69, 0x6e, 0x71, 0x61, 0x70, 0x75, 0x72, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x20, 0x41, 0x64, 0x61,
0x6c, 0x61, 0x72, 0x131, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x43, 0x259,
0x6e, 0x75, 0x62, 0x69, 0x20, 0x43, 0x6f, 0x72, 0x63, 0x69, 0x79, 0x61,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61,
0x6d, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x53, 0x79, 0x6f, 0x76, 0x61,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x54, 0x61, 0x79, 0x62, 0x65, 0x79,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x54, 0x61, 0x79, 0x62, 0x65, 0x79,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x54, 0x61, 0x79, 0x62, 0x65, 0x79, 0x20, 0x59, 0x61,
0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x54, 0x61, 0x63, 0x69, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x54,
0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x54, 0x6f, 0x6e, 0x71, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x54,
0x6f, 0x6e, 0x71, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x54, 0x6f, 0x6e, 0x71, 0x61,
0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0xc7, 0x75,
0x75, 0x6b, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x54, 0xfc, 0x72, 0x6b,
0x6d, 0x259, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x54, 0xfc, 0x72, 0x6b, 0x6d, 0x259, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x54, 0xfc, 0x72, 0x6b, 0x6d, 0x259, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x55, 0x72, 0x75, 0x71, 0x76, 0x61, 0x79, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x55, 0x72, 0x75, 0x71, 0x76, 0x61, 0x79, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x55, 0x72, 0x75, 0x71, 0x76, 0x61, 0x79, 0x20, 0x59, 0x61,
0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0xd6, 0x7a, 0x62, 0x259, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0xd6,
0x7a, 0x62, 0x259, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0xd6, 0x7a, 0x62, 0x259, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x59,
0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x56, 0x61, 0x75, 0x6e, 0x61,
0x74, 0x75, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x56, 0x65, 0x6e, 0x65, 0x73, 0x75, 0x65, 0x6c, 0x61, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x59, 0x61,
0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x56, 0x6f, 0x6c, 0x71, 0x6f,
0x71, 0x72, 0x61, 0x64, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x56, 0x6f,
0x6c, 0x71, 0x6f, 0x71, 0x72, 0x61, 0x64, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x56, 0x6f,
0x6c, 0x71, 0x6f, 0x71, 0x72, 0x61, 0x64, 0x20, 0x59, 0x61, 0x79, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x55, 0x65, 0x79, 0x6b, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x55, 0x6f, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x76, 0x259,
0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x59, 0x61, 0x79, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x71, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x71, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x71, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x98f, 0x995, 0x9b0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x98f, 0x995,
0x9b0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x98f,
0x995, 0x9b0, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be,
0x9b2, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9ab, 0x997, 0x9be, 0x9a8, 0x9bf,
0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9a7,
0x9cd, 0x9af, 0x20, 0x986, 0x9ab, 0x9cd, 0x9b0, 0x9bf, 0x995, 0x9be, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x20, 0x986, 0x9ab, 0x9cd,
0x9b0, 0x9bf, 0x995, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a6, 0x995, 0x9cd,
0x9b7, 0x9bf, 0x9a3, 0x20, 0x986, 0x9ab, 0x9cd, 0x9b0, 0x9bf, 0x995, 0x9be, 0x20,
0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd,
0x99a, 0x9bf, 0x9ae, 0x20, 0x986, 0x9ab, 0x9cd, 0x9b0, 0x9bf, 0x995, 0x9be, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20, 0x986,
0x9ab, 0x9cd, 0x9b0, 0x9bf, 0x995, 0x9be, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20, 0x986,
0x9ab, 0x9cd, 0x9b0, 0x9bf, 0x995, 0x9be, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7,
0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x986, 0x9b2, 0x9be, 0x9b8, 0x9cd, 0x995, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x986, 0x9b2, 0x9be, 0x9b8, 0x9cd, 0x995, 0x9be, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9b2, 0x9be, 0x9b8, 0x9cd, 0x995, 0x9be,
0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x986, 0x9b2, 0x9cd, 0x9ae, 0x9be, 0x99f, 0x9bf, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x986, 0x9b2, 0x9cd, 0x9ae, 0x9be, 0x99f, 0x9bf, 0x20, 0x9ae, 0x9be, 0x9a8,
0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9b2, 0x9cd, 0x9ae, 0x9be, 0x99f,
0x9bf, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9ae, 0x9be, 0x99c, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x986, 0x9ae, 0x9be, 0x99c, 0x9a8, 0x20, 0x9ae, 0x9be, 0x9a8,
0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9ae, 0x9be, 0x99c, 0x9a8, 0x20,
0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9b0,
0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9c7, 0x9a8, 0x9cd,
0x9a6, 0x9cd, 0x9b0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9b0, 0x9c0,
0x9af, 0x9bc, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x9be, 0x99e, 0x9cd, 0x99a,
0x9b2, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9b0,
0x9cd, 0x9ac, 0x9be, 0x99e, 0x9cd, 0x99a, 0x9b2, 0x9c7, 0x9b0, 0x20, 0x9ae, 0x9be,
0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac,
0x9be, 0x99e, 0x9cd, 0x99a, 0x9b2, 0x9c7, 0x9b0, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be,
0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x9b0, 0x9cd,
0x9ac, 0x9a4, 0x9cd, 0x9af, 0x20, 0x985, 0x99e, 0x9cd, 0x99a, 0x9b2, 0x9c7, 0x9b0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x9b0, 0x9cd, 0x9ac, 0x9a4, 0x9cd,
0x9af, 0x20, 0x985, 0x99e, 0x9cd, 0x99a, 0x9b2, 0x9c7, 0x9b0, 0x20, 0x9ae, 0x9be,
0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x9b0, 0x9cd, 0x9ac,
0x9a4, 0x9cd, 0x9af, 0x20, 0x985, 0x99e, 0x9cd, 0x99a, 0x9b2, 0x9c7, 0x9b0, 0x20,
0x9a6, 0x9bf, 0x9a8, 0x9c7, 0x9b0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cd,
0x9b0, 0x9b6, 0x9be, 0x9a8, 0x9cd, 0x9a4, 0x20, 0x9ae, 0x9b9, 0x9be, 0x9b8, 0x9be,
0x997, 0x9b0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x985, 0x99e, 0x9cd, 0x99a, 0x9b2, 0x9c7,
0x9b0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cd, 0x9b0, 0x9b6, 0x9be, 0x9a8,
0x9cd, 0x9a4, 0x20, 0x9ae, 0x9b9, 0x9be, 0x9b8, 0x9be, 0x997, 0x9b0, 0x9c0, 0x9af,
0x9bc, 0x20, 0x985, 0x99e, 0x9cd, 0x99a, 0x9b2, 0x9c7, 0x9b0, 0x20, 0x9ae, 0x9be,
0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cd, 0x9b0, 0x9b6, 0x9be,
0x9a8, 0x9cd, 0x9a4, 0x20, 0x9ae, 0x9b9, 0x9be, 0x9b8, 0x9be, 0x997, 0x9b0, 0x9c0,
0x9af, 0x9bc, 0x20, 0x985, 0x99e, 0x9cd, 0x99a, 0x9b2, 0x9c7, 0x9b0, 0x20, 0x9a6,
0x9bf, 0x9a8, 0x9c7, 0x9b0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9a8, 0x9a6,
0x9cd, 0x9af, 0x9cd, 0x9b0, 0x9cd, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9a8,
0x9a6, 0x9cd, 0x9af, 0x9cd, 0x9b0, 0x9cd, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9a8, 0x9a6, 0x9cd, 0x9af, 0x9cd, 0x9b0, 0x9cd,
0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20,
0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9aa, 0x9bf,
0x9af, 0x9bc, 0x9be, 0x20, 0x9a6, 0x9bf, 0x9a8, 0x9c7, 0x9b0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x986, 0x995, 0x99f, 0x9be, 0x993, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x986, 0x995, 0x99f, 0x9be, 0x993, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x986, 0x995, 0x99f, 0x9be, 0x993, 0x20, 0x997, 0x9cd, 0x9b0,
0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x986, 0x995, 0x99f, 0x9cb, 0x9ac, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x995,
0x99f, 0x9cb, 0x9ac, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x986, 0x995, 0x99f, 0x9cb, 0x9ac, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7,
0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9b0,
0x9ac, 0x9bf, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9b0, 0x9ac, 0x9bf, 0x20,
0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9b0, 0x9ac,
0x9bf, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x986, 0x9b0, 0x9cd, 0x99c, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x9bf, 0x9a8,
0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9b0, 0x9cd, 0x99c, 0x9c7, 0x9a8,
0x9cd, 0x99f, 0x9bf, 0x9a8, 0x9be, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x986, 0x9b0, 0x9cd, 0x99c, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x9bf,
0x9a8, 0x9be, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be,
0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a,
0x9bf, 0x9ae, 0x9bf, 0x20, 0x986, 0x9b0, 0x9cd, 0x99c, 0x9c7, 0x9a8, 0x9cd, 0x99f,
0x9bf, 0x9a8, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a,
0x9bf, 0x9ae, 0x9bf, 0x20, 0x986, 0x9b0, 0x9cd, 0x99c, 0x9c7, 0x9a8, 0x99f, 0x9bf,
0x9a8, 0x9be, 0x9b0, 0x20, 0x9aa, 0x9cd, 0x9b0, 0x9ae, 0x9be, 0x9a3, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x9bf, 0x20, 0x986,
0x9b0, 0x9cd, 0x99c, 0x9c7, 0x9a8, 0x99f, 0x9bf, 0x9a8, 0x9be, 0x20, 0x997, 0x9cd,
0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x986, 0x9b0, 0x9cd, 0x9ae, 0x9c7, 0x9a8, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9b0, 0x9cd, 0x9ae, 0x9c7, 0x9a8,
0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x986, 0x9b0, 0x9cd, 0x9ae, 0x9c7, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9a4, 0x9b2, 0x9be, 0x9a8, 0x9cd,
0x9a4, 0x9bf, 0x995, 0x9c7, 0x9b0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9a4,
0x9b2, 0x9be, 0x9a8, 0x9cd, 0x9a4, 0x9bf, 0x995, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9a4, 0x9b2, 0x9be, 0x9a8, 0x9cd, 0x9a4,
0x9bf, 0x995, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9b0, 0x9c0, 0x9af,
0x9bc, 0x20, 0x985, 0x9b8, 0x9cd, 0x99f, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x9c0, 0x9af,
0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9b8, 0x9cd, 0x99f, 0x9cd, 0x9b0,
0x9c7, 0x9b2, 0x9c0, 0x9af, 0x9bc, 0x20, 0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd,
0x9b0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x985, 0x9b8, 0x9cd, 0x99f, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x9c0, 0x9af,
0x9bc, 0x20, 0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9b0, 0x9c0, 0x9af, 0x9bc,
0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x985, 0x9b8, 0x9cd, 0x99f, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x9c0, 0x9af, 0x9bc,
0x20, 0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9b0, 0x9c0, 0x9af, 0x9bc, 0x20,
0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x9bf, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x985, 0x9b8, 0x9cd, 0x99f, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x9c0, 0x9af, 0x9bc, 0x20,
0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9b0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9aa,
0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x9bf, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9b8, 0x9cd, 0x99f, 0x9cd, 0x9b0, 0x9c7, 0x9b2,
0x9c0, 0x9af, 0x9bc, 0x20, 0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9b0, 0x9c0,
0x9af, 0x9bc, 0x20, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x9bf, 0x20, 0x9a6,
0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa,
0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x20, 0x985, 0x9b8, 0x9cd, 0x99f, 0x9cd, 0x9b0, 0x9c7,
0x9b2, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9b8, 0x9cd,
0x99f, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9aa, 0x9c2, 0x9b0,
0x9cd, 0x9ac, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x985, 0x9b8, 0x9cd, 0x99f, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x9c0, 0x9af, 0x9bc, 0x20,
0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb,
0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae,
0x9bf, 0x20, 0x985, 0x9b8, 0x9cd, 0x99f, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x9c0, 0x9af,
0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9b8, 0x9cd, 0x99f, 0x9cd, 0x9b0,
0x9c7, 0x9b2, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae,
0x9bf, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985,
0x9b8, 0x9cd, 0x99f, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9aa,
0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x9bf, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2,
0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x99c, 0x9be, 0x9b0, 0x9ac,
0x9be, 0x987, 0x99c, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x99c,
0x9be, 0x9b0, 0x9ac, 0x9be, 0x987, 0x99c, 0x9be, 0x9a8, 0x20, 0x9ae, 0x9be, 0x9a8,
0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x99c, 0x9be, 0x9b0, 0x9ac, 0x9be,
0x987, 0x99c, 0x9be, 0x9a8, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae,
0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x98f, 0x99c,
0x9cb, 0x9b0, 0x9c7, 0x9b8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x98f, 0x99c, 0x9cb,
0x9b0, 0x9c7, 0x9b8, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x98f, 0x99c, 0x9cb, 0x9b0, 0x9c7, 0x9b8, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0,
0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9ac, 0x9be, 0x982, 0x9b2, 0x9be, 0x9a6, 0x9c7, 0x9b6, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9ac, 0x9be, 0x982, 0x9b2, 0x9be, 0x9a6, 0x9c7, 0x9b6, 0x20, 0x9ae,
0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9be, 0x982, 0x9b2,
0x9be, 0x9a6, 0x9c7, 0x9b6, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae,
0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9c1,
0x99f, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9cb, 0x9b2, 0x9bf,
0x9ad, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9cd,
0x9b0, 0x9be, 0x9b8, 0x9bf, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9ac, 0x9cd, 0x9b0, 0x9be, 0x9b8, 0x9bf, 0x9b2, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac,
0x9cd, 0x9b0, 0x9be, 0x9b8, 0x9bf, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x997,
0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9cd, 0x9b0, 0x9c1, 0x9a8, 0x9c7, 0x987, 0x20,
0x9a6, 0x9be, 0x9b0, 0x9c1, 0x9b8, 0x9b8, 0x9be, 0x9b2, 0x9be, 0x9ae, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x995, 0x9c7, 0x9aa, 0x20, 0x9ad, 0x9be, 0x9b0, 0x9cd, 0x9a6,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9c7, 0x9aa, 0x20, 0x9ad, 0x9be, 0x9b0,
0x9cd, 0x9a6, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x995, 0x9c7, 0x9aa, 0x20, 0x9ad, 0x9be, 0x9b0, 0x9cd, 0x9a6, 0x20, 0x997, 0x9cd,
0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x99a, 0x9be, 0x9ae, 0x9c7, 0x9b0, 0x9cb, 0x20, 0x9ae, 0x9be,
0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9cd, 0x9af, 0x9be, 0x9a5,
0x9be, 0x9ae, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9cd, 0x9af, 0x9be, 0x9a5,
0x9be, 0x9ae, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x99a, 0x9cd, 0x9af, 0x9be, 0x9a5, 0x9be, 0x9ae, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be,
0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9bf, 0x9b2, 0x9bf,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9bf, 0x9b2, 0x9bf, 0x20, 0x9ae, 0x9be,
0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9bf, 0x9b2, 0x9bf, 0x20,
0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x99a, 0x9c0, 0x9a8, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x99a, 0x9c0, 0x9a8, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb,
0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cd, 0x9b0, 0x9bf, 0x9b8, 0x9ae,
0x9be, 0x9b8, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x995, 0x9cb, 0x995, 0x9cb, 0x9b8, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa,
0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cb,
0x9b2, 0x9cb, 0x9ae, 0x9cd, 0x9ac, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x995, 0x9cb, 0x9b2, 0x9cb, 0x9ae, 0x9cd, 0x9ac, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995,
0x9cb, 0x9b2, 0x9cb, 0x9ae, 0x9cd, 0x9ac, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x997,
0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9c1, 0x995, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0,
0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995,
0x9c1, 0x995, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd,
0x99c, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995,
0x9c1, 0x995, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd,
0x99c, 0x20, 0x985, 0x9b0, 0x9cd, 0x9a7, 0x9c7, 0x995, 0x20, 0x997, 0x9cd, 0x9b0,
0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x995, 0x9bf, 0x989, 0x9ac, 0x9be, 0x9b0, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x995, 0x9bf, 0x989, 0x9ac, 0x9be, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9bf, 0x989, 0x9ac, 0x9be, 0x20, 0x9a6, 0x9bf,
0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a1, 0x9c7,
0x9ad, 0x9bf, 0x9b8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a1, 0x9c1, 0x9ae, 0x9a8,
0x9cd, 0x99f, 0x2d, 0x9a6, 0x9cd, 0x9af, 0x2019, 0x989, 0x9b0, 0x9ad, 0x9bf, 0x9b2,
0x9c7, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x20,
0x99f, 0x9bf, 0x9ae, 0x9b0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9b8, 0x9cd,
0x99f, 0x9be, 0x9b0, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x987, 0x9b8, 0x9cd, 0x99f, 0x9be, 0x9b0, 0x20, 0x9a6, 0x9cd, 0x9ac,
0x9c0, 0x9aa, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x987, 0x9b8, 0x9cd, 0x99f, 0x9be, 0x9b0, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa,
0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x995, 0x9c1, 0x9af, 0x9bc, 0x9c7,
0x9a1, 0x9b0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9a7, 0x9cd, 0x9af, 0x20,
0x987, 0x989, 0x9b0, 0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9ae, 0x9a7, 0x9cd, 0x9af, 0x20, 0x987, 0x989, 0x9b0, 0x9cb, 0x9aa, 0x9c0,
0x9af, 0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9ae, 0x9a7, 0x9cd, 0x9af, 0x20, 0x987, 0x989, 0x9b0, 0x9cb, 0x9aa, 0x9c0, 0x9af,
0x9bc, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2,
0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac,
0x20, 0x987, 0x989, 0x9b0, 0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x20, 0x987, 0x989, 0x9b0, 0x9cb,
0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x20, 0x987, 0x989, 0x9b0, 0x9cb,
0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae,
0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cd,
0x9b0, 0x9be, 0x9a8, 0x9cd, 0x9a4, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9aa, 0x9c2, 0x9b0,
0x9cd, 0x9ac, 0x20, 0x987, 0x989, 0x9b0, 0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20, 0x987,
0x989, 0x9b0, 0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20, 0x987, 0x989, 0x9b0, 0x9cb, 0x9aa,
0x9c0, 0x9af, 0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20, 0x987, 0x989, 0x9b0, 0x9cb,
0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae,
0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x995,
0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0,
0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab,
0x995, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x20, 0x9a6, 0x9cd, 0x9ac,
0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x995, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8,
0x9cd, 0x9a1, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd,
0x99c, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2,
0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9bf, 0x99c, 0x9bf, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9bf, 0x99c, 0x9bf, 0x20, 0x9ae, 0x9be, 0x9a8,
0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9bf, 0x99c, 0x9bf, 0x20, 0x997,
0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9b0, 0x9be, 0x9b8, 0x9bf, 0x20, 0x997, 0x9be,
0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9b0,
0x9be, 0x9b8, 0x9bf, 0x20, 0x9a6, 0x995, 0x9cd, 0x9b7, 0x9bf, 0x9a3, 0x20, 0x98f,
0x9ac, 0x982, 0x20, 0x986, 0x9a8, 0x9cd, 0x99f, 0x9be, 0x9b0, 0x9cd, 0x995, 0x99f,
0x9bf, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x997, 0x9be, 0x9b2, 0x9be, 0x9aa,
0x9be, 0x997, 0x9cb, 0x9b8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x997, 0x9cd, 0x9af,
0x9be, 0x9ae, 0x9cd, 0x9ac, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9b0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x99c, 0x9b0, 0x9cd, 0x99c, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x99c, 0x9b0, 0x9cd, 0x99c, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20,
0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99c, 0x9b0, 0x9cd,
0x99c, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd,
0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x997,
0x9bf, 0x9b2, 0x9ac, 0x9be, 0x9b0, 0x9cd, 0x99f, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0,
0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x997,
0x9cd, 0x9b0, 0x9c0, 0x9a8, 0x9bf, 0x99a, 0x20, 0x9ae, 0x9bf, 0x9a8, 0x20, 0x99f,
0x9be, 0x987, 0x9ae, 0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x20, 0x997, 0x9cd, 0x9b0,
0x9c0, 0x9a8, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0,
0x9a8, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x20, 0x9ae, 0x9be, 0x9a8,
0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x20,
0x997, 0x9cd, 0x9b0, 0x9c0, 0x9a8, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1,
0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae,
0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9a8, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd,
0x9a1, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae,
0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9a8, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd,
0x9a1, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa,
0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9a8, 0x9b2,
0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7,
0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x997, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x9ae, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x989, 0x9aa, 0x9b8, 0x9be, 0x997, 0x9b0, 0x9c0, 0x9af, 0x9bc,
0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x997, 0x9c1,
0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9, 0x9be,
0x993, 0x9af, 0x9bc, 0x9be, 0x987, 0x2d, 0x986, 0x9b2, 0x9c7, 0x989, 0x9a4, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9, 0x9be, 0x993, 0x9af, 0x9bc, 0x9be, 0x987, 0x2d,
0x986, 0x9b2, 0x9c7, 0x989, 0x9a4, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9b9, 0x9be, 0x993, 0x9af, 0x9bc, 0x9be, 0x987, 0x2d, 0x986,
0x9b2, 0x9c7, 0x989, 0x9a4, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9, 0x982, 0x20, 0x995, 0x982, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9b9, 0x982, 0x20, 0x995, 0x982, 0x20, 0x9ae, 0x9be, 0x9a8,
0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9, 0x982, 0x20, 0x995, 0x982, 0x20,
0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9, 0x9cb, 0x9ad, 0x9a1, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9b9, 0x9cb, 0x9ad, 0x9a1, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9, 0x9cb, 0x9ad, 0x9a1, 0x20, 0x997, 0x9cd, 0x9b0,
0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9ad, 0x9be, 0x9b0, 0x9a4, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9ae, 0x9be,
0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9be, 0x9b0, 0x9a4, 0x20,
0x9ae, 0x9b9, 0x9be, 0x9b8, 0x9be, 0x997, 0x9b0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x987, 0x9a8, 0x9cd, 0x9a6, 0x9cb, 0x99a, 0x9c0, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9b0, 0x9c0,
0x9af, 0x9bc, 0x20, 0x987, 0x9a8, 0x9cd, 0x9a6, 0x9cb, 0x9a8, 0x9c7, 0x9b6, 0x9bf,
0x9af, 0x9bc, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9b0, 0x9cd,
0x9ac, 0x20, 0x987, 0x9a8, 0x9cd, 0x9a6, 0x9cb, 0x9a8, 0x9c7, 0x9b6, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf,
0x9ae, 0x9c0, 0x20, 0x987, 0x9a8, 0x9cd, 0x9a6, 0x9cb, 0x9a8, 0x9c7, 0x9b6, 0x9bf,
0x9af, 0x9bc, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9b0, 0x9be, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9b0, 0x9be, 0x9a8, 0x20, 0x9ae, 0x9be,
0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9b0, 0x9be, 0x9a8, 0x20,
0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x987, 0x9b0, 0x995, 0x9c1, 0x99f, 0x9b8, 0x9cd, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x987, 0x9b0, 0x995, 0x9c1, 0x99f, 0x9b8, 0x9cd, 0x995, 0x20, 0x9ae, 0x9be,
0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9b0, 0x995, 0x9c1, 0x99f,
0x9b8, 0x9cd, 0x995, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995,
0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x99c, 0x9b0,
0x9be, 0x9af, 0x9bc, 0x9c7, 0x9b2, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x99c,
0x9b0, 0x9be, 0x9af, 0x9bc, 0x9c7, 0x9b2, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x99c, 0x9b0, 0x9be, 0x9af, 0x9bc, 0x9c7, 0x9b2,
0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x99c, 0x9be, 0x9aa, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99c,
0x9be, 0x9aa, 0x9be, 0x9a8, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x99c, 0x9be, 0x9aa, 0x9be, 0x9a8, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be,
0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9bf, 0x9a4, 0x9cd,
0x9b0, 0x9c7, 0x9aa, 0x9cd, 0x9af, 0x9be, 0x9ad, 0x9b2, 0x9b8, 0x9cd, 0x995, 0x2d,
0x20, 0x995, 0x9cd, 0x9af, 0x9be, 0x9ae, 0x99a, 0x9cd, 0x9af, 0x9be, 0x99f, 0x9b8,
0x9cd, 0x995, 0x9bf, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9bf, 0x9a4, 0x9cd,
0x9b0, 0x9c7, 0x9aa, 0x9cd, 0x9af, 0x9be, 0x9ad, 0x9b2, 0x9b8, 0x9cd, 0x995, 0x2d,
0x20, 0x995, 0x9cd, 0x9af, 0x9be, 0x9ae, 0x99a, 0x9cd, 0x9af, 0x9be, 0x99f, 0x9b8,
0x9cd, 0x995, 0x9bf, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9aa, 0x9bf, 0x9a4, 0x9cd, 0x9b0, 0x9c7, 0x9aa, 0x9cd, 0x9af, 0x9be, 0x9ad, 0x9b2,
0x9b8, 0x9cd, 0x995, 0x2d, 0x20, 0x995, 0x9cd, 0x9af, 0x9be, 0x9ae, 0x99a, 0x9cd,
0x9af, 0x9be, 0x99f, 0x9b8, 0x9cd, 0x995, 0x9bf, 0x20, 0x997, 0x9c3, 0x9b7, 0x9cd,
0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995,
0x9be, 0x99c, 0x9be, 0x996, 0x9be, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x20, 0x995, 0x9be, 0x99c,
0x9be, 0x996, 0x9be, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20, 0x995, 0x9be, 0x99c, 0x9be,
0x996, 0x9be, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x995, 0x9cb, 0x9b0, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x995, 0x9cb, 0x9b0, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x20, 0x9ae, 0x9be,
0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cb, 0x9b0, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x9a8, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cb, 0x9b8, 0x9b0, 0x9c7, 0x987, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x995, 0x9cd, 0x9b0, 0x9be, 0x9b8, 0x9a8, 0x9cb, 0x9af, 0x9bc,
0x9be, 0x9b0, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x9bf, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x995, 0x9cd, 0x9b0, 0x9be, 0x9b8, 0x9a8, 0x9cb, 0x9af, 0x9bc, 0x9be, 0x9b0, 0x9cd,
0x9b8, 0x9cd, 0x995, 0x9bf, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x995, 0x9cd, 0x9b0, 0x9be, 0x9b8, 0x9a8, 0x9cb, 0x9af, 0x9bc, 0x9be,
0x9b0, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x9bf, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7,
0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x995, 0x9bf, 0x9b0, 0x997, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9b2, 0x999, 0x9cd, 0x995, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9b2, 0x9be, 0x987, 0x9a8, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa,
0x9c1, 0x99e, 0x9cd, 0x99c, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b2, 0x9b0, 0x9cd,
0x9a1, 0x20, 0x9b9, 0x9be, 0x993, 0x9af, 0x9bc, 0x9c7, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9b2, 0x9b0, 0x9cd, 0x9a1, 0x20, 0x9b9, 0x9be, 0x993, 0x9af, 0x9bc, 0x9c7,
0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9ae, 0x9b8, 0x9af, 0x9bc, 0x9b2, 0x9b0,
0x9cd, 0x9a1, 0x20, 0x9b9, 0x9be, 0x993, 0x9af, 0x9bc, 0x9c7, 0x20, 0x9a6, 0x9bf,
0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9ae, 0x9b8, 0x9af, 0x9bc, 0x9ae, 0x9be,
0x995, 0x9be, 0x993, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9be, 0x995, 0x9be,
0x993, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9be,
0x995, 0x9be, 0x993, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995,
0x9be, 0x9b2, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9cd, 0x9af, 0x9be, 0x997,
0x9be, 0x9a1, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9cd, 0x9af,
0x9be, 0x997, 0x9be, 0x9a1, 0x9be, 0x9a8, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9cd, 0x9af, 0x9be, 0x997, 0x9be, 0x9a1, 0x9be,
0x9a8, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2,
0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9be, 0x9b2, 0x9af, 0x9bc,
0x9c7, 0x9b6, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae,
0x9be, 0x9b2, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9ae, 0x9be, 0x9b0, 0x9cd, 0x995, 0x9c7, 0x9b8, 0x9be, 0x9b8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9ae, 0x9be, 0x9b0, 0x9cd, 0x9b6, 0x9be, 0x9b2, 0x20, 0x9a6, 0x9cd,
0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9ae, 0x9b0, 0x9bf, 0x9b6, 0x9be, 0x9b8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9ae, 0x9b0, 0x9bf, 0x9b6, 0x9be, 0x9b8, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9b0, 0x9bf, 0x9b6, 0x9be, 0x9b8, 0x20, 0x997,
0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9b8, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9ae, 0x9c7, 0x995, 0x9cd, 0x9b8, 0x9bf, 0x995, 0x9be, 0x9a8, 0x20, 0x9aa, 0x9cd,
0x9b0, 0x9b6, 0x9be, 0x9a8, 0x9cd, 0x9a4, 0x20, 0x9ae, 0x9b9, 0x9be, 0x9b8, 0x9be,
0x997, 0x9b0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9c7,
0x995, 0x9cd, 0x9b8, 0x9bf, 0x995, 0x9be, 0x9a8, 0x20, 0x9aa, 0x9cd, 0x9b0, 0x9b6,
0x9be, 0x9a8, 0x9cd, 0x9a4, 0x20, 0x9ae, 0x9b9, 0x9b8, 0x9be, 0x997, 0x9b0, 0x9c0,
0x9af, 0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9ae, 0x9c7, 0x995, 0x9cd, 0x9b8, 0x9bf, 0x995, 0x9be, 0x9a8, 0x20, 0x9aa, 0x9cd,
0x9b0, 0x9b6, 0x9be, 0x9a8, 0x9cd, 0x9a4, 0x20, 0x9ae, 0x9b9, 0x9be, 0x9b8, 0x9be,
0x997, 0x9b0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb,
0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9b2, 0x9be, 0x9a8, 0x20, 0x9ac,
0x9be, 0x9a4, 0x9cb, 0x9b0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9b2, 0x9be,
0x9a8, 0x20, 0x9ac, 0x9be, 0x9a4, 0x9cb, 0x9b0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9b2, 0x9be, 0x9a8, 0x20, 0x9ac, 0x9be,
0x9a4, 0x9cb, 0x9b0, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995,
0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9b8, 0x9cd,
0x995, 0x9cb, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9b8, 0x9cd, 0x995, 0x9cb,
0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9b8,
0x9cd, 0x995, 0x9cb, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995,
0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9be, 0x9af,
0x9bc, 0x9be, 0x9a8, 0x9ae, 0x9be, 0x9b0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8,
0x9be, 0x989, 0x9b0, 0x9c1, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9c7, 0x9aa,
0x9be, 0x9b2, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x20, 0x995,
0x9cd, 0x9af, 0x9be, 0x9b2, 0x9c7, 0x9a1, 0x9cb, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x20, 0x995, 0x9cd, 0x9af,
0x9be, 0x9b2, 0x9c7, 0x9a1, 0x9cb, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x9ae,
0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x20,
0x995, 0x9cd, 0x9af, 0x9be, 0x9b2, 0x9c7, 0x9a1, 0x9cb, 0x9a8, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2,
0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x99c, 0x9bf,
0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9a8, 0x9bf, 0x989, 0x99c, 0x9bf, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1,
0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf,
0x989, 0x99c, 0x9bf, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x20, 0x9a6,
0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8,
0x9bf, 0x989, 0x9ab, 0x9be, 0x989, 0x9a8, 0x9cd, 0x9a1, 0x9b2, 0x9cd, 0x9af, 0x9be,
0x9a8, 0x9cd, 0x9a1, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x9ab,
0x9be, 0x989, 0x9a8, 0x9cd, 0x9a1, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1,
0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf,
0x989, 0x9ab, 0x9be, 0x989, 0x9a8, 0x9cd, 0x9a1, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8,
0x9cd, 0x9a1, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x987, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9a8, 0x9b0, 0x9ab, 0x9cb, 0x995, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9b0, 0x9ab, 0x9cb, 0x995, 0x20, 0x9a6, 0x9cd,
0x9ac, 0x9c0, 0x9aa, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9a8, 0x9b0, 0x9ab, 0x9cb, 0x995, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa,
0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9ab, 0x9be, 0x9b0, 0x9cd, 0x9a8, 0x9be, 0x9a8, 0x9cd, 0x9a6, 0x9cb, 0x20,
0x9a1, 0x9bf, 0x20, 0x9a8, 0x9cb, 0x9b0, 0x9cb, 0x9a8, 0x9b9, 0x9be, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9be, 0x9b0, 0x9cd, 0x9a8, 0x9be, 0x9a8, 0x9cd, 0x9a6,
0x9cb, 0x20, 0x9a1, 0x9bf, 0x20, 0x9a8, 0x9cb, 0x9b0, 0x9cb, 0x9a8, 0x9b9, 0x9be,
0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9be,
0x9b0, 0x9cd, 0x9a8, 0x9be, 0x9a8, 0x9cd, 0x9a6, 0x9cb, 0x20, 0x9a1, 0x9bf, 0x20,
0x9a8, 0x9cb, 0x9b0, 0x9cb, 0x9a8, 0x9b9, 0x9be, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0,
0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9b0, 0x20, 0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x9a8,
0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9cb, 0x9ad, 0x9cb, 0x9b8, 0x9bf, 0x9ac, 0x9bf,
0x9b0, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9cb,
0x9ad, 0x9cb, 0x9b8, 0x9bf, 0x9ac, 0x9bf, 0x9b0, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x20,
0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9cb, 0x9ad,
0x9cb, 0x9b8, 0x9bf, 0x9ac, 0x9bf, 0x9b0, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x20, 0x997,
0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x993, 0x9ae, 0x9b8, 0x9cd, 0x995, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x993, 0x9ae, 0x9b8, 0x9cd, 0x995, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x993, 0x9ae, 0x9b8, 0x9cd, 0x995, 0x20, 0x997,
0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x995, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x995, 0x9bf, 0x9b8, 0x9cd,
0x9a4, 0x9be, 0x9a8, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9aa, 0x9be, 0x995, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x20, 0x997,
0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x9b2, 0x9be, 0x989, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9aa, 0x9be, 0x9aa, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x20, 0x9a8, 0x9bf,
0x989, 0x20, 0x997, 0x9bf, 0x9a8, 0x9bf, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa,
0x9cd, 0x9af, 0x9be, 0x9b0, 0x9be, 0x997, 0x9c1, 0x9af, 0x9bc, 0x9c7, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cd, 0x9af, 0x9be, 0x9b0, 0x9be, 0x997, 0x9c1, 0x9af,
0x9bc, 0x9c7, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9aa, 0x9cd, 0x9af, 0x9be, 0x9b0, 0x9be, 0x997, 0x9c1, 0x9af, 0x9bc, 0x9c7, 0x20,
0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c7, 0x9b0, 0x9c1, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9aa, 0x9c7, 0x9b0, 0x9c1, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c7, 0x9b0, 0x9c1, 0x20, 0x997, 0x9cd, 0x9b0,
0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9ab, 0x9bf, 0x9b2, 0x9bf, 0x9aa, 0x9be, 0x987, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9bf, 0x9b2, 0x9bf, 0x9aa, 0x9be, 0x987, 0x9a8, 0x20,
0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9bf, 0x9b2,
0x9bf, 0x9aa, 0x9be, 0x987, 0x9a8, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd,
0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab,
0x9cb, 0x9a8, 0x9bf, 0x995, 0x9cd, 0x9b8, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa,
0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9c7,
0x9a8, 0x9cd, 0x99f, 0x20, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9c7, 0x9b0, 0x20, 0x993,
0x20, 0x9ae, 0x9bf, 0x995, 0x9c7, 0x9b2, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x20, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9c7, 0x9b0,
0x20, 0x993, 0x20, 0x9ae, 0x9bf, 0x995, 0x9c7, 0x9b2, 0x9a8, 0x20, 0x9ae, 0x9be,
0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x99f,
0x20, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9c7, 0x9b0, 0x20, 0x993, 0x20, 0x9ae, 0x9bf,
0x995, 0x9c7, 0x9b2, 0x9a8, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9bf, 0x99f, 0x995, 0x9c7, 0x9af, 0x9bc,
0x9be, 0x9b0, 0x9cd, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cb, 0x9a8,
0x9be, 0x9aa, 0x9c7, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9bf, 0x9af, 0x9bc,
0x982, 0x987, 0x9af, 0x9bc, 0x9be, 0x982, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995,
0x9bf, 0x99c, 0x9bf, 0x9b2, 0x9cb, 0x9b0, 0x9cd, 0x9a1, 0x9be, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x995, 0x9bf, 0x99c, 0x9bf, 0x9b2, 0x9cb, 0x9b0, 0x9cd, 0x9a1, 0x9be,
0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9bf, 0x99c,
0x9bf, 0x9b2, 0x9cb, 0x9b0, 0x9cd, 0x9a1, 0x9be, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0,
0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b0,
0x9bf, 0x987, 0x989, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9b0, 0x9a5, 0x9c7, 0x9b0, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8,
0x9be, 0x996, 0x9be, 0x9b2, 0x9bf, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8,
0x9be, 0x996, 0x9be, 0x9b2, 0x9bf, 0x9a8, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9be, 0x996, 0x9be, 0x9b2, 0x9bf, 0x9a8, 0x20,
0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9be, 0x9ae, 0x9be, 0x9b0, 0x9be, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9be, 0x9ae, 0x9be, 0x9b0, 0x9be, 0x20, 0x9ae,
0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9be, 0x9ae, 0x9be, 0x9b0,
0x9be, 0x20, 0x997, 0x9c3, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9be, 0x9ae, 0x9cb, 0x9af, 0x9bc, 0x9be,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9be, 0x9ae, 0x9cb, 0x9af, 0x9bc, 0x9be,
0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9be,
0x9ae, 0x9cb, 0x9af, 0x9bc, 0x9be, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb,
0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9c7, 0x9b6, 0x9c7, 0x9b2, 0x9b8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9bf, 0x999, 0x9cd, 0x997, 0x9be, 0x9aa,
0x9c1, 0x9b0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9b8, 0x9b2, 0x9cb, 0x9ae, 0x9a8, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa,
0x9c1, 0x99e, 0x9cd, 0x99c, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a6, 0x995, 0x9cd,
0x9b7, 0x9bf, 0x9a3, 0x20, 0x99c, 0x9b0, 0x9cd, 0x99c, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9c1, 0x9b0, 0x9bf, 0x9a8, 0x9be, 0x9ae,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9be, 0x9af, 0x9bc, 0x9cb, 0x993, 0x9af,
0x9bc, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a4, 0x9be, 0x9b9, 0x9bf, 0x9a4,
0x9bf, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a4, 0x9be, 0x987, 0x9aa, 0x9c7, 0x987,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a4, 0x9be, 0x987, 0x9aa, 0x9c7, 0x987, 0x20,
0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a4, 0x9be, 0x987,
0x9aa, 0x9c7, 0x987, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a4, 0x9be, 0x99c, 0x9be, 0x996, 0x9be, 0x9b8, 0x9cd,
0x9a4, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99f, 0x9cb, 0x995, 0x9c7,
0x9b2, 0x9be, 0x989, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99f, 0x9cb, 0x999, 0x9cd,
0x997, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99f, 0x9cb, 0x999, 0x9cd, 0x997,
0x9be, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99f,
0x9cb, 0x999, 0x9cd, 0x997, 0x9be, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd,
0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a,
0x9c1, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a4, 0x9c1, 0x9b0, 0x9cd, 0x995,
0x9ae, 0x9c7, 0x9a8, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9a4, 0x9c1, 0x9b0, 0x9cd, 0x995, 0x9ae, 0x9c7, 0x9a8, 0x9bf, 0x9b8,
0x9cd, 0x9a4, 0x9be, 0x9a8, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9a4, 0x9c1, 0x9b0, 0x9cd, 0x995, 0x9ae, 0x9c7, 0x9a8, 0x9bf, 0x9b8,
0x9cd, 0x9a4, 0x9be, 0x9a8, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae,
0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99f, 0x9c1,
0x9ad, 0x9be, 0x9b2, 0x9c1, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9b0, 0x9c1,
0x997, 0x9c1, 0x9af, 0x9bc, 0x9c7, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9b0,
0x9c1, 0x997, 0x9c1, 0x9af, 0x9bc, 0x9c7, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9b0, 0x9c1, 0x997, 0x9c1, 0x9af, 0x9bc, 0x9c7,
0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x99c, 0x9ac, 0x9c7, 0x995, 0x9bf,
0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x99c,
0x9ac, 0x9c7, 0x995, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x20, 0x9ae, 0x9be,
0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x99c, 0x9ac, 0x9c7, 0x995,
0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7,
0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9ad, 0x9be, 0x9a8, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x9a4, 0x9c1, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9ad, 0x9be, 0x9a8, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x9a4, 0x9c1, 0x20,
0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9be, 0x9a8,
0x9c1, 0x9af, 0x9bc, 0x9be, 0x9a4, 0x9c1, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7,
0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9ad, 0x9c7, 0x9a8, 0x9c7, 0x99c, 0x9c1, 0x9af, 0x9bc, 0x9c7, 0x9b2, 0x9be, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9cd, 0x9b2, 0x9be, 0x9a6, 0x9bf, 0x9ad, 0x9b8,
0x9cd, 0x9a4, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9cd, 0x9b2, 0x9be,
0x9a6, 0x9bf, 0x9ad, 0x9b8, 0x9cd, 0x9a4, 0x995, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9cd, 0x9b2, 0x9be, 0x9a6, 0x9bf, 0x9ad,
0x9b8, 0x9cd, 0x9a4, 0x995, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae,
0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9b2,
0x997, 0x9cb, 0x997, 0x9cd, 0x9b0, 0x9be, 0x9a1, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9ad, 0x9b2, 0x997, 0x9cb, 0x997, 0x9cd, 0x9b0, 0x9be, 0x9a1, 0x20, 0x9ae, 0x9be,
0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9b2, 0x997, 0x9cb, 0x997,
0x9cd, 0x9b0, 0x9be, 0x9a1, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae,
0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9b8,
0x99f, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x993, 0x9af, 0x9bc, 0x9c7, 0x995,
0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x993,
0x9af, 0x9bc, 0x9be, 0x9b2, 0x9bf, 0x9b8, 0x20, 0x98f, 0x9ac, 0x982, 0x20, 0x9ab,
0x9c1, 0x99f, 0x9c1, 0x9a8, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9af,
0x9bc, 0x9be, 0x995, 0x9c1, 0x99f, 0x9be, 0x9b8, 0x9cd, 0x995, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x987, 0x9af, 0x9bc, 0x9be, 0x995, 0x9c1, 0x99f, 0x9be, 0x9b8, 0x9cd,
0x995, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987,
0x9af, 0x9bc, 0x9be, 0x995, 0x9c1, 0x99f, 0x9be, 0x9b8, 0x9cd, 0x995, 0x20, 0x997,
0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9af, 0x9bc, 0x9c7, 0x995, 0x9be, 0x9a4, 0x9c7,
0x9b0, 0x9bf, 0x9a8, 0x9ac, 0x9c1, 0x9b0, 0x9cd, 0x997, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x987, 0x9af, 0x9bc, 0x9c7, 0x995, 0x9be, 0x9a4, 0x9c7, 0x9b0, 0x9bf, 0x9a8,
0x9ac, 0x9c1, 0x9b0, 0x9cd, 0x997, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x987, 0x9af, 0x9bc, 0x9c7, 0x995, 0x9be, 0x9a4, 0x9c7, 0x9b0,
0x9bf, 0x9a8, 0x9ac, 0x9c1, 0x9b0, 0x9cd, 0x997, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0,
0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x987, 0x989, 0x995, 0x9cb, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x41,
0x63, 0x72, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41,
0x63, 0x72, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x41, 0x63, 0x72,
0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x72, 0x64, 0x69, 0x61, 0x6c,
0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x6b, 0x69, 0x61, 0x6c,
0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x68, 0x65, 0x67, 0x6f, 0x61,
0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x6d, 0x65, 0x6e, 0x64,
0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x6d,
0x65, 0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x72, 0x61, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20,
0x6d, 0x65, 0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f,
0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x72, 0x61, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x75,
0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x6c,
0x6d, 0x61, 0x74, 0x79, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61,
0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x72, 0x61, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x69, 0x61, 0x6b, 0x6f,
0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x49, 0x70, 0x61, 0x72, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x6b, 0x6f, 0x20, 0x65, 0x72, 0x64, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x70, 0x61, 0x72, 0x20,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x72,
0x64, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61,
0x49, 0x70, 0x61, 0x72, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x6b, 0x6f, 0x20, 0x65, 0x72, 0x64, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b,
0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x49, 0x70, 0x61, 0x72, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x6b, 0x6f, 0x20, 0x65, 0x6b, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x70, 0x61, 0x72, 0x20,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x6b,
0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x49,
0x70, 0x61, 0x72, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6b,
0x6f, 0x20, 0x65, 0x6b, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20,
0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49,
0x70, 0x61, 0x72, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6b,
0x6f, 0x20, 0x6d, 0x65, 0x6e, 0x64, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x70, 0x61, 0x72, 0x20,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x6d, 0x65,
0x6e, 0x64, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72,
0x61, 0x49, 0x70, 0x61, 0x72, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x6b, 0x6f, 0x20, 0x6d, 0x65, 0x6e, 0x64, 0x69, 0x61, 0x6c, 0x64,
0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x49, 0x70, 0x61, 0x72, 0x20, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x50, 0x61, 0x7a, 0x69, 0x66, 0x69,
0x6b, 0x6f, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x70,
0x61, 0x72, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f,
0x20, 0x50, 0x61, 0x7a, 0x69, 0x66, 0x69, 0x6b, 0x6f, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x72, 0x61, 0x49, 0x70, 0x61, 0x72, 0x20, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x50, 0x61, 0x7a, 0x69, 0x66, 0x69,
0x6b, 0x6f, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x72, 0x65,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x6e, 0x61, 0x64,
0x79, 0x72, 0x72, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x41, 0x6e,
0x61, 0x64, 0x79, 0x72, 0x72, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x70, 0x69, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x70, 0x69, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x41, 0x70, 0x69, 0x61, 0x6b, 0x6f,
0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x41, 0x6b, 0x74, 0x61, 0x75, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61,
0x41, 0x6b, 0x74, 0x61, 0x75, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x6b, 0x74, 0x6f, 0x62,
0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x6b, 0x74,
0x6f, 0x62, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x41, 0x6b, 0x74,
0x6f, 0x62, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x72, 0x61, 0x62, 0x69,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x41, 0x72, 0x61, 0x62, 0x69,
0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x6b, 0x6f, 0x20, 0x75,
0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6d, 0x65, 0x6e, 0x64,
0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20,
0x6d, 0x65, 0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x72, 0x61, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x20, 0x6d, 0x65, 0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c, 0x64, 0x65,
0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x49, 0x70, 0x61, 0x72, 0x20, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x6b, 0x6f, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49,
0x70, 0x61, 0x72, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6b,
0x6f, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x6f, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x72, 0x61, 0x49, 0x70, 0x61, 0x72, 0x20, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x41, 0x74, 0x6c, 0x61,
0x6e, 0x74, 0x69, 0x6b, 0x6f, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x72, 0x64, 0x69, 0x61,
0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x65,
0x72, 0x64, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72,
0x61, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6b, 0x6f,
0x20, 0x65, 0x72, 0x64, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20,
0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x65,
0x72, 0x64, 0x69, 0x2d, 0x6d, 0x65, 0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c,
0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x72,
0x64, 0x69, 0x2d, 0x6d, 0x65, 0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c, 0x64,
0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x72, 0x64, 0x69, 0x2d,
0x6d, 0x65, 0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f,
0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6b, 0x6f, 0x20,
0x65, 0x6b, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x6b, 0x6f, 0x20, 0x65, 0x6b, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x72, 0x61, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x6b, 0x6f, 0x20, 0x65, 0x6b, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b,
0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6b, 0x6f,
0x20, 0x6d, 0x65, 0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6d, 0x65, 0x6e, 0x64, 0x65,
0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6d,
0x65, 0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20,
0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41,
0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x67, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69,
0x6a, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x75, 0x64,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x72, 0x61, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x65, 0x74, 0x61, 0x6b,
0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x65,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72,
0x61, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x65,
0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x42, 0x72, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x42,
0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72,
0x61, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x6b, 0x6f, 0x20,
0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x42,
0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73,
0x61, 0x6c, 0x61, 0x6d, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x43, 0x61, 0x62, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x43, 0x61, 0x62, 0x6f, 0x20, 0x56,
0x65, 0x72, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x43, 0x61,
0x62, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x75,
0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x43, 0x61,
0x73, 0x65, 0x79, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x43,
0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72,
0x61, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x67, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x67,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x72, 0x61, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x67, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x54, 0x78, 0x69, 0x6c, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x54, 0x78, 0x69, 0x6c, 0x65, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x72, 0x61, 0x54, 0x78, 0x69, 0x6c, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x54, 0x78, 0x69,
0x6e, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x54, 0x78,
0x69, 0x6e, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x54, 0x78, 0x69,
0x6e, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61,
0x73, 0x20, 0x75, 0x68, 0x61, 0x72, 0x74, 0x65, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x20, 0x75, 0x68,
0x61, 0x72, 0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x4b, 0x6f, 0x6c, 0x6f, 0x6e, 0x62, 0x69, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b, 0x6f, 0x6c, 0x6f, 0x6e,
0x62, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x4b, 0x6f, 0x6c,
0x6f, 0x6e, 0x62, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x43, 0x6f, 0x6f, 0x6b, 0x20,
0x75, 0x68, 0x61, 0x72, 0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x75, 0x68,
0x61, 0x72, 0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72,
0x61, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x75, 0x68, 0x61, 0x72, 0x74, 0x65,
0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x20, 0x65, 0x72,
0x64, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x4b, 0x75, 0x62, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x4b, 0x75, 0x62, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61,
0x4b, 0x75, 0x62, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x44, 0x61, 0x76, 0x69, 0x73, 0x65,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x44, 0x75, 0x6d, 0x6f,
0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x45, 0x6b, 0x69, 0x61,
0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x72,
0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x50, 0x61, 0x7a,
0x6b, 0x6f, 0x20, 0x75, 0x68, 0x61, 0x72, 0x74, 0x65, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x50, 0x61, 0x7a, 0x6b, 0x6f, 0x20, 0x75,
0x68, 0x61, 0x72, 0x74, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x50,
0x61, 0x7a, 0x6b, 0x6f, 0x20, 0x75, 0x68, 0x61, 0x72, 0x74, 0x65, 0x6b,
0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x45, 0x6b, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x72, 0x65, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x6b, 0x6f, 0x20, 0x65, 0x72, 0x64, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x6b, 0x6f, 0x20, 0x65, 0x72, 0x64, 0x69, 0x61, 0x6c, 0x64, 0x65,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x6b, 0x6f, 0x20, 0x65, 0x72, 0x64, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b,
0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x6b,
0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x6b,
0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x6b, 0x69, 0x61,
0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x6b,
0x6f, 0x20, 0x65, 0x6b, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x20, 0x75, 0x72,
0x72, 0x75, 0x6e, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x6b, 0x6f, 0x20, 0x6d, 0x65, 0x6e,
0x64, 0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x6b, 0x6f, 0x20,
0x6d, 0x65, 0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x72, 0x61, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x6b, 0x6f,
0x20, 0x6d, 0x65, 0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b,
0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x75, 0x68,
0x61, 0x72, 0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x75, 0x68, 0x61, 0x72, 0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x72, 0x61, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x75, 0x68, 0x61, 0x72, 0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20,
0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x46,
0x69, 0x6a, 0x69, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x46,
0x69, 0x6a, 0x69, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x46, 0x69, 0x6a,
0x69, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x20, 0x46, 0x72,
0x61, 0x6e, 0x74, 0x73, 0x65, 0x73, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x46, 0x72, 0x61, 0x6e, 0x74, 0x7a, 0x69, 0x61, 0x72,
0x65, 0x6e, 0x20, 0x6c, 0x75, 0x72, 0x72, 0x61, 0x6c, 0x64, 0x65, 0x20,
0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x65, 0x74, 0x61, 0x20,
0x61, 0x6e, 0x74, 0x61, 0x72, 0x74, 0x69, 0x6b, 0x6f, 0x65, 0x74, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x74, 0x65, 0x67, 0x69, 0x61,
0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x65, 0x74, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x47, 0x61, 0x6d, 0x62, 0x69,
0x65, 0x72, 0x72, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x20, 0x75,
0x68, 0x61, 0x72, 0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63,
0x68, 0x6b, 0x6f, 0x20, 0x6d, 0x65, 0x72, 0x69, 0x64, 0x69, 0x61, 0x6e,
0x6f, 0x61, 0x72, 0x65, 0x6e, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x47,
0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x6b, 0x6f,
0x20, 0x65, 0x6b, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x6b, 0x69, 0x61, 0x6c, 0x64,
0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x47, 0x72, 0x6f, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x6b, 0x69,
0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6d, 0x65, 0x6e, 0x64,
0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x61, 0x6b, 0x6f, 0x20, 0x6d, 0x65, 0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c,
0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x47, 0x72, 0x6f, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6d, 0x65,
0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x75,
0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x47, 0x75,
0x61, 0x6d, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x47, 0x6f, 0x6c,
0x6b, 0x6f, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x47, 0x75, 0x79, 0x61,
0x6e, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x48, 0x61,
0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61,
0x72, 0x20, 0x75, 0x68, 0x61, 0x72, 0x74, 0x65, 0x65, 0x74, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x48, 0x61, 0x77, 0x61, 0x69,
0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x72, 0x20, 0x75,
0x68, 0x61, 0x72, 0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x72, 0x61, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65,
0x75, 0x74, 0x69, 0x61, 0x72, 0x20, 0x75, 0x68, 0x61, 0x72, 0x74, 0x65,
0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f,
0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x48, 0x6f, 0x6e,
0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x48,
0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x6f, 0x20, 0x75, 0x64,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b, 0x68, 0x6f,
0x76, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b,
0x68, 0x6f, 0x76, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x4b,
0x68, 0x6f, 0x76, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x6e, 0x64, 0x69, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x6e, 0x64, 0x69,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x7a, 0x65, 0x61, 0x6e, 0x6f, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x6e, 0x64, 0x6f, 0x74, 0x78,
0x69, 0x6e, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x65,
0x72, 0x64, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61,
0x6b, 0x6f, 0x20, 0x65, 0x6b, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x73, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6d, 0x65, 0x6e, 0x64, 0x65, 0x62,
0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x49, 0x72, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x49, 0x72, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x49, 0x72,
0x61, 0x6e, 0x67, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x65,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x73,
0x72, 0x61, 0x65, 0x6c, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x49, 0x73,
0x72, 0x61, 0x65, 0x6c, 0x67, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x69,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4a, 0x61, 0x70,
0x6f, 0x6e, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x4a, 0x61,
0x70, 0x6f, 0x6e, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c,
0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74,
0x73, 0x6b, 0x69, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x50, 0x65, 0x74,
0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b,
0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x6b, 0x6f, 0x20,
0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b,
0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x67, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73,
0x74, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x65, 0x6b, 0x69, 0x61, 0x6c, 0x64,
0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b, 0x61, 0x7a,
0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x6d, 0x65,
0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x72, 0x61, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6b, 0x6f, 0x20,
0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b,
0x6f, 0x73, 0x72, 0x61, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x69, 0x61, 0x72, 0x73, 0x6b,
0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x69, 0x61, 0x72, 0x73, 0x6b, 0x65, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x72, 0x61, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x69, 0x61, 0x72,
0x73, 0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b, 0x69, 0x72, 0x67, 0x69, 0x7a, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x4c, 0x69, 0x6e, 0x65, 0x20, 0x75, 0x68, 0x61, 0x72, 0x74, 0x65,
0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4d, 0x61, 0x63, 0x61, 0x6f,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x6b,
0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x67,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x72, 0x61, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x67, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x4d, 0x61, 0x6c, 0x61, 0x79, 0x73, 0x69, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76,
0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4d,
0x61, 0x72, 0x6b, 0x65, 0x73, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c,
0x20, 0x55, 0x68, 0x61, 0x72, 0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x7a,
0x69, 0x6f, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x7a, 0x69, 0x6f, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x7a, 0x69, 0x6f, 0x6b, 0x6f, 0x20, 0x75,
0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4d, 0x61,
0x77, 0x73, 0x6f, 0x6e, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x6f, 0x6b, 0x6f, 0x20, 0x50, 0x61,
0x7a, 0x69, 0x66, 0x69, 0x6b, 0x6f, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x6f, 0x6b, 0x6f, 0x20, 0x50,
0x61, 0x7a, 0x69, 0x66, 0x69, 0x6b, 0x6f, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72,
0x61, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x6f, 0x6b, 0x6f, 0x20, 0x50, 0x61,
0x7a, 0x69, 0x66, 0x69, 0x6b, 0x6f, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x55, 0x6c, 0x61, 0x6e,
0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x72, 0x65, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74,
0x6f, 0x72, 0x72, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x55, 0x6c,
0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x72, 0x65, 0x6b, 0x6f,
0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x4d, 0x6f, 0x73, 0x6b, 0x75, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x4d, 0x6f, 0x73, 0x6b, 0x75, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61,
0x4d, 0x6f, 0x73, 0x6b, 0x75, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4d, 0x79, 0x61, 0x6e, 0x6d,
0x61, 0x72, 0x72, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x4e, 0x61, 0x75, 0x72, 0x75, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20,
0x42, 0x65, 0x72, 0x72, 0x69, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x42,
0x65, 0x72, 0x72, 0x69, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x4b, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x72,
0x69, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x5a, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20,
0x42, 0x65, 0x72, 0x72, 0x69, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x5a, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x42, 0x65,
0x72, 0x72, 0x69, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x5a, 0x65, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x42, 0x65, 0x72, 0x72, 0x69, 0x6b,
0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x54, 0x65, 0x72, 0x6e, 0x75, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x54, 0x65, 0x72, 0x6e, 0x75, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x72, 0x61, 0x54, 0x65, 0x72, 0x6e, 0x75, 0x61, 0x6b, 0x6f, 0x20,
0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4e,
0x69, 0x75, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x75, 0x68, 0x61, 0x72, 0x74,
0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x75, 0x68, 0x61, 0x72,
0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x75, 0x68, 0x61, 0x72, 0x74,
0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x68, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x70, 0x61, 0x72, 0x20,
0x4d, 0x61, 0x72, 0x69, 0x61, 0x6e, 0x61, 0x20, 0x75, 0x68, 0x61, 0x72,
0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x65, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x72, 0x61, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x4f, 0x6d, 0x73, 0x6b, 0x65, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4f, 0x6d, 0x73, 0x6b, 0x65, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x72, 0x61, 0x4f, 0x6d, 0x73, 0x6b, 0x65, 0x6b, 0x6f,
0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x67, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x47,
0x69, 0x6e, 0x65, 0x61, 0x20, 0x42, 0x65, 0x72, 0x72, 0x69, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x69, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x69, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x69, 0x6b, 0x6f, 0x20, 0x75,
0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x50, 0x65,
0x72, 0x75, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x50, 0x65,
0x72, 0x75, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x50, 0x65, 0x72, 0x75,
0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x74, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61,
0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x74, 0x61, 0x6b, 0x6f,
0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x20, 0x75, 0x68, 0x61, 0x72,
0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x20, 0x65, 0x74, 0x61, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x75,
0x6e, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x53, 0x61,
0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65,
0x74, 0x61, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x75, 0x6e, 0x65, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x72, 0x61, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x74, 0x61, 0x20, 0x4d, 0x69,
0x6b, 0x65, 0x6c, 0x75, 0x6e, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x50, 0x69, 0x74, 0x63,
0x61, 0x69, 0x72, 0x6e, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x50, 0x69, 0x6f, 0x6e, 0x67, 0x69, 0x61, 0x6e, 0x67,
0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b, 0x79, 0x7a,
0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x4b, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x72, 0x61, 0x4b, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72,
0x64, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x65,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x52, 0x6f, 0x74, 0x68,
0x65, 0x72, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x53,
0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x67,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x72, 0x61, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x67, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x72, 0x61, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x6b,
0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72,
0x61, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x53, 0x65, 0x79, 0x63,
0x68, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x75, 0x68, 0x61, 0x72, 0x74, 0x65,
0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x53,
0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x72, 0x65, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x72, 0x61, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x20, 0x55,
0x68, 0x61, 0x72, 0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x48, 0x65, 0x67, 0x6f, 0x61, 0x6c, 0x64, 0x65,
0x6b, 0x6f, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x74, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x53, 0x75, 0x72, 0x69,
0x6e, 0x61, 0x6d, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x53,
0x79, 0x6f, 0x77, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x72, 0x61, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x6b, 0x6f,
0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x54, 0x61, 0x64, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x67,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x54, 0x6f, 0x6b, 0x65, 0x6c,
0x61, 0x75, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x65, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x54, 0x75, 0x72, 0x6b, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x54, 0x75, 0x72,
0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x67, 0x6f, 0x20,
0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x54,
0x75, 0x76, 0x61, 0x6c, 0x75, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x69, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x69, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x72, 0x61, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x69,
0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x55, 0x7a, 0x62, 0x65,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61,
0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x67, 0x6f,
0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x72, 0x61, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x6b,
0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x75,
0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x56, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x57,
0x61, 0x6b, 0x65, 0x20, 0x75, 0x68, 0x61, 0x72, 0x74, 0x65, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73,
0x20, 0x65, 0x74, 0x61, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x74, 0x65, 0x67, 0x69, 0x61, 0x4a,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x65, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x72, 0x61, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x4a, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x65, 0x6b,
0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x410, 0x444, 0x433, 0x430, 0x43d, 0x456, 0x441, 0x442, 0x430, 0x43d,
0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x426, 0x44d, 0x43d, 0x442, 0x440,
0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x430, 0x444, 0x440, 0x44b, 0x43a, 0x430, 0x43d,
0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x423, 0x441, 0x445, 0x43e, 0x434,
0x43d, 0x435, 0x430, 0x444, 0x440, 0x44b, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456,
0x20, 0x447, 0x430, 0x441, 0x41f, 0x430, 0x45e, 0x434, 0x43d, 0x451, 0x432, 0x430,
0x430, 0x444, 0x440, 0x44b, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x447,
0x430, 0x441, 0x417, 0x430, 0x445, 0x43e, 0x434, 0x43d, 0x435, 0x430, 0x444, 0x440,
0x44b, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x417,
0x430, 0x445, 0x43e, 0x434, 0x43d, 0x435, 0x430, 0x444, 0x440, 0x44b, 0x43a, 0x430,
0x43d, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x417, 0x430, 0x445, 0x43e, 0x434,
0x43d, 0x435, 0x430, 0x444, 0x440, 0x44b, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456,
0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430,
0x441, 0x20, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x456, 0x421, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x410,
0x43b, 0x44f, 0x441, 0x43a, 0x456, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447,
0x430, 0x441, 0x20, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x456, 0x410, 0x43c, 0x430,
0x437, 0x43e, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x410, 0x43c,
0x430, 0x437, 0x43e, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x410, 0x43c,
0x430, 0x437, 0x43e, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d,
0x456, 0x20, 0x447, 0x430, 0x441, 0x41f, 0x430, 0x45e, 0x43d, 0x43e, 0x447, 0x43d,
0x430, 0x430, 0x43c, 0x435, 0x440, 0x44b, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456,
0x20, 0x446, 0x44d, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x44b, 0x20,
0x447, 0x430, 0x441, 0x41f, 0x430, 0x45e, 0x43d, 0x43e, 0x447, 0x43d, 0x430, 0x430,
0x43c, 0x435, 0x440, 0x44b, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x446,
0x44d, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x44b, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441,
0x41f, 0x430, 0x45e, 0x43d, 0x43e, 0x447, 0x43d, 0x430, 0x430, 0x43c, 0x435, 0x440,
0x44b, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x446, 0x44d, 0x43d, 0x442,
0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x44b, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456,
0x20, 0x447, 0x430, 0x441, 0x41f, 0x430, 0x45e, 0x43d, 0x43e, 0x447, 0x43d, 0x430,
0x430, 0x43c, 0x435, 0x440, 0x44b, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20,
0x45e, 0x441, 0x445, 0x43e, 0x434, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x41f,
0x430, 0x45e, 0x43d, 0x43e, 0x447, 0x43d, 0x430, 0x430, 0x43c, 0x435, 0x440, 0x44b,
0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x45e, 0x441, 0x445, 0x43e, 0x434,
0x43d, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x44b, 0x20, 0x447, 0x430, 0x441, 0x41f, 0x430, 0x45e, 0x43d, 0x43e, 0x447, 0x43d,
0x430, 0x430, 0x43c, 0x435, 0x440, 0x44b, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456,
0x20, 0x45e, 0x441, 0x445, 0x43e, 0x434, 0x43d, 0x456, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x41f, 0x430, 0x45e, 0x43d, 0x43e, 0x447,
0x43d, 0x430, 0x430, 0x43c, 0x435, 0x440, 0x44b, 0x43a, 0x430, 0x43d, 0x441, 0x43a,
0x456, 0x20, 0x433, 0x43e, 0x440, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x41f,
0x430, 0x45e, 0x43d, 0x43e, 0x447, 0x43d, 0x430, 0x430, 0x43c, 0x435, 0x440, 0x44b,
0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x433, 0x43e, 0x440, 0x43d, 0x44b,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20,
0x447, 0x430, 0x441, 0x41f, 0x430, 0x45e, 0x43d, 0x43e, 0x447, 0x43d, 0x430, 0x430,
0x43c, 0x435, 0x440, 0x44b, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x433,
0x43e, 0x440, 0x43d, 0x44b, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447,
0x430, 0x441, 0x426, 0x456, 0x445, 0x430, 0x430, 0x43a, 0x456, 0x44f, 0x43d, 0x441,
0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x426, 0x456, 0x445, 0x430, 0x430, 0x43a,
0x456, 0x44f, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x426, 0x456, 0x445,
0x430, 0x430, 0x43a, 0x456, 0x44f, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x410,
0x43f, 0x456, 0x456, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x410, 0x43f, 0x456, 0x456, 0x41b, 0x435,
0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x410, 0x43f, 0x456, 0x456,
0x427, 0x430, 0x441, 0x20, 0x421, 0x430, 0x443, 0x434, 0x430, 0x45e, 0x441, 0x43a,
0x430, 0x439, 0x20, 0x410, 0x440, 0x430, 0x432, 0x456, 0x456, 0x421, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20,
0x421, 0x430, 0x443, 0x434, 0x430, 0x45e, 0x441, 0x43a, 0x430, 0x439, 0x20, 0x410,
0x440, 0x430, 0x432, 0x456, 0x456, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447,
0x430, 0x441, 0x20, 0x421, 0x430, 0x443, 0x434, 0x430, 0x45e, 0x441, 0x43a, 0x430,
0x439, 0x20, 0x410, 0x440, 0x430, 0x432, 0x456, 0x456, 0x410, 0x440, 0x433, 0x435,
0x43d, 0x446, 0x456, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x410,
0x440, 0x433, 0x435, 0x43d, 0x446, 0x456, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430,
0x441, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x446, 0x456, 0x43d, 0x441, 0x43a, 0x456,
0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430,
0x441, 0x20, 0x417, 0x430, 0x445, 0x43e, 0x434, 0x43d, 0x44f, 0x439, 0x20, 0x410,
0x440, 0x433, 0x435, 0x43d, 0x446, 0x456, 0x43d, 0x44b, 0x421, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x417,
0x430, 0x445, 0x43e, 0x434, 0x43d, 0x44f, 0x439, 0x20, 0x410, 0x440, 0x433, 0x435,
0x43d, 0x446, 0x456, 0x43d, 0x44b, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447,
0x430, 0x441, 0x20, 0x417, 0x430, 0x445, 0x43e, 0x434, 0x43d, 0x44f, 0x439, 0x20,
0x410, 0x440, 0x433, 0x435, 0x43d, 0x446, 0x456, 0x43d, 0x44b, 0x427, 0x430, 0x441,
0x20, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x456, 0x456, 0x421, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x410,
0x440, 0x43c, 0x435, 0x43d, 0x456, 0x456, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20,
0x447, 0x430, 0x441, 0x20, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x456, 0x456, 0x410,
0x442, 0x43b, 0x430, 0x43d, 0x442, 0x44b, 0x447, 0x43d, 0x44b, 0x20, 0x447, 0x430,
0x441, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x44b, 0x447, 0x43d, 0x44b, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447,
0x430, 0x441, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x44b, 0x447, 0x43d, 0x44b,
0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430,
0x441, 0x20, 0x446, 0x44d, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430,
0x439, 0x20, 0x410, 0x45e, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x456, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430,
0x441, 0x20, 0x446, 0x44d, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430,
0x439, 0x20, 0x410, 0x45e, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x456, 0x41b,
0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x446, 0x44d, 0x43d,
0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x439, 0x20, 0x410, 0x45e, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x456, 0x456, 0x426, 0x44d, 0x43d, 0x442, 0x440, 0x430,
0x43b, 0x44c, 0x43d, 0x430, 0x2d, 0x437, 0x430, 0x445, 0x43e, 0x434, 0x43d, 0x456,
0x20, 0x447, 0x430, 0x441, 0x20, 0x410, 0x45e, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x456, 0x456, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b,
0x20, 0x446, 0x44d, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x2d,
0x437, 0x430, 0x445, 0x43e, 0x434, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20,
0x410, 0x45e, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x456, 0x41b, 0x435, 0x442,
0x43d, 0x456, 0x20, 0x446, 0x44d, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d,
0x430, 0x2d, 0x437, 0x430, 0x445, 0x43e, 0x434, 0x43d, 0x456, 0x20, 0x447, 0x430,
0x441, 0x20, 0x410, 0x45e, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x456, 0x427,
0x430, 0x441, 0x20, 0x443, 0x441, 0x445, 0x43e, 0x434, 0x43d, 0x44f, 0x439, 0x20,
0x410, 0x45e, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x456, 0x421, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20,
0x443, 0x441, 0x445, 0x43e, 0x434, 0x43d, 0x44f, 0x439, 0x20, 0x410, 0x45e, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x456, 0x456, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20,
0x447, 0x430, 0x441, 0x20, 0x443, 0x441, 0x445, 0x43e, 0x434, 0x43d, 0x44f, 0x439,
0x20, 0x410, 0x45e, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x456, 0x427, 0x430,
0x441, 0x20, 0x437, 0x430, 0x445, 0x43e, 0x434, 0x43d, 0x44f, 0x439, 0x20, 0x410,
0x45e, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x456, 0x421, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x437,
0x430, 0x445, 0x43e, 0x434, 0x43d, 0x44f, 0x439, 0x20, 0x410, 0x45e, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x456, 0x456, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447,
0x430, 0x441, 0x20, 0x437, 0x430, 0x445, 0x43e, 0x434, 0x43d, 0x44f, 0x439, 0x20,
0x410, 0x45e, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x456, 0x427, 0x430, 0x441,
0x20, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x434, 0x436, 0x430, 0x43d,
0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20,
0x447, 0x430, 0x441, 0x20, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x434,
0x436, 0x430, 0x43d, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430,
0x441, 0x20, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x434, 0x436, 0x430,
0x43d, 0x430, 0x427, 0x430, 0x441, 0x20, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x43a,
0x456, 0x445, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430,
0x441, 0x20, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x43a, 0x456, 0x445, 0x20, 0x430,
0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x41b, 0x435, 0x442, 0x43d, 0x456,
0x20, 0x447, 0x430, 0x441, 0x20, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x43a, 0x456,
0x445, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x427, 0x430,
0x441, 0x20, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x44d, 0x448, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430,
0x441, 0x20, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x44d, 0x448, 0x41b,
0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x411, 0x430, 0x43d,
0x433, 0x43b, 0x430, 0x434, 0x44d, 0x448, 0x427, 0x430, 0x441, 0x20, 0x411, 0x443,
0x442, 0x430, 0x43d, 0x430, 0x411, 0x430, 0x43b, 0x456, 0x432, 0x456, 0x439, 0x441,
0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x411, 0x440, 0x430, 0x437, 0x456, 0x43b,
0x44c, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x411, 0x440, 0x430, 0x437,
0x456, 0x43b, 0x44c, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x411, 0x440, 0x430,
0x437, 0x456, 0x43b, 0x44c, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d,
0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x411, 0x440, 0x443,
0x43d, 0x435, 0x44f, 0x427, 0x430, 0x441, 0x20, 0x41a, 0x430, 0x431, 0x430, 0x2d,
0x412, 0x435, 0x440, 0x434, 0x44d, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x41a, 0x430, 0x431, 0x430,
0x2d, 0x412, 0x435, 0x440, 0x434, 0x44d, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20,
0x447, 0x430, 0x441, 0x20, 0x41a, 0x430, 0x431, 0x430, 0x2d, 0x412, 0x435, 0x440,
0x434, 0x44d, 0x427, 0x430, 0x441, 0x20, 0x427, 0x430, 0x43c, 0x43e, 0x440, 0x430,
0x427, 0x430, 0x441, 0x20, 0x427, 0x430, 0x442, 0x44d, 0x43c, 0x430, 0x421, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441,
0x20, 0x427, 0x430, 0x442, 0x44d, 0x43c, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x456,
0x20, 0x447, 0x430, 0x441, 0x20, 0x427, 0x430, 0x442, 0x44d, 0x43c, 0x430, 0x427,
0x44b, 0x43b, 0x456, 0x439, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427,
0x44b, 0x43b, 0x456, 0x439, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x427, 0x44b,
0x43b, 0x456, 0x439, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456,
0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x41a, 0x456, 0x442, 0x430,
0x44f, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20,
0x447, 0x430, 0x441, 0x20, 0x41a, 0x456, 0x442, 0x430, 0x44f, 0x41b, 0x435, 0x442,
0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x41a, 0x456, 0x442, 0x430, 0x44f,
0x427, 0x430, 0x441, 0x20, 0x432, 0x43e, 0x441, 0x442, 0x440, 0x430, 0x432, 0x430,
0x20, 0x41a, 0x430, 0x43b, 0x44f, 0x434, 0x427, 0x430, 0x441, 0x20, 0x41a, 0x430,
0x43a, 0x43e, 0x441, 0x430, 0x432, 0x44b, 0x445, 0x20, 0x430, 0x441, 0x442, 0x440,
0x430, 0x432, 0x43e, 0x45e, 0x41a, 0x430, 0x43b, 0x443, 0x43c, 0x431, 0x456, 0x439,
0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x41a, 0x430, 0x43b, 0x443, 0x43c,
0x431, 0x456, 0x439, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x41a, 0x430, 0x43b,
0x443, 0x43c, 0x431, 0x456, 0x439, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x430, 0x441,
0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x20, 0x41a, 0x443, 0x43a, 0x430, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430,
0x441, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x20, 0x41a,
0x443, 0x43a, 0x430, 0x41f, 0x430, 0x45e, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20,
0x447, 0x430, 0x441, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e,
0x20, 0x41a, 0x443, 0x43a, 0x430, 0x427, 0x430, 0x441, 0x20, 0x41a, 0x443, 0x431,
0x44b, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20,
0x447, 0x430, 0x441, 0x20, 0x41a, 0x443, 0x431, 0x44b, 0x41b, 0x435, 0x442, 0x43d,
0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x41a, 0x443, 0x431, 0x44b, 0x427, 0x430,
0x441, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x446, 0x44b, 0x456, 0x20, 0x414, 0x44d,
0x439, 0x432, 0x456, 0x441, 0x427, 0x430, 0x441, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x446, 0x44b, 0x456, 0x20, 0x414, 0x437, 0x44e, 0x43c, 0x43e, 0x43d, 0x2d, 0x414,
0x437, 0x44e, 0x440, 0x432, 0x456, 0x43b, 0x44c, 0x427, 0x430, 0x441, 0x20, 0x423,
0x441, 0x445, 0x43e, 0x434, 0x43d, 0x44f, 0x433, 0x430, 0x20, 0x422, 0x44b, 0x43c,
0x43e, 0x440, 0x430, 0x427, 0x430, 0x441, 0x20, 0x432, 0x43e, 0x441, 0x442, 0x440,
0x430, 0x432, 0x430, 0x20, 0x412, 0x44f, 0x43b, 0x456, 0x43a, 0x430, 0x434, 0x43d,
0x44f, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20,
0x447, 0x430, 0x441, 0x20, 0x432, 0x43e, 0x441, 0x442, 0x440, 0x430, 0x432, 0x430,
0x20, 0x412, 0x44f, 0x43b, 0x456, 0x43a, 0x430, 0x434, 0x43d, 0x44f, 0x41b, 0x435,
0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x432, 0x43e, 0x441, 0x442,
0x440, 0x430, 0x432, 0x430, 0x20, 0x412, 0x44f, 0x43b, 0x456, 0x43a, 0x430, 0x434,
0x43d, 0x44f, 0x42d, 0x43a, 0x432, 0x430, 0x434, 0x43e, 0x440, 0x441, 0x43a, 0x456,
0x20, 0x447, 0x430, 0x441, 0x426, 0x44d, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c,
0x43d, 0x430, 0x435, 0x45e, 0x440, 0x430, 0x43f, 0x435, 0x439, 0x441, 0x43a, 0x456,
0x20, 0x447, 0x430, 0x441, 0x426, 0x44d, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c,
0x43d, 0x430, 0x435, 0x45e, 0x440, 0x430, 0x43f, 0x435, 0x439, 0x441, 0x43a, 0x456,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20,
0x447, 0x430, 0x441, 0x426, 0x44d, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d,
0x430, 0x435, 0x45e, 0x440, 0x430, 0x43f, 0x435, 0x439, 0x441, 0x43a, 0x456, 0x20,
0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x423, 0x441, 0x445,
0x43e, 0x434, 0x43d, 0x435, 0x435, 0x45e, 0x440, 0x430, 0x43f, 0x435, 0x439, 0x441,
0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x423, 0x441, 0x445, 0x43e, 0x434, 0x43d,
0x435, 0x435, 0x45e, 0x440, 0x430, 0x43f, 0x435, 0x439, 0x441, 0x43a, 0x456, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447,
0x430, 0x441, 0x423, 0x441, 0x445, 0x43e, 0x434, 0x43d, 0x435, 0x435, 0x45e, 0x440,
0x430, 0x43f, 0x435, 0x439, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d,
0x456, 0x20, 0x447, 0x430, 0x441, 0x414, 0x430, 0x43b, 0x451, 0x43a, 0x430, 0x45e,
0x441, 0x445, 0x43e, 0x434, 0x43d, 0x435, 0x435, 0x45e, 0x440, 0x430, 0x43f, 0x435,
0x439, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x417, 0x430, 0x445, 0x43e,
0x434, 0x43d, 0x435, 0x435, 0x45e, 0x440, 0x430, 0x43f, 0x435, 0x439, 0x441, 0x43a,
0x456, 0x20, 0x447, 0x430, 0x441, 0x417, 0x430, 0x445, 0x43e, 0x434, 0x43d, 0x435,
0x435, 0x45e, 0x440, 0x430, 0x43f, 0x435, 0x439, 0x441, 0x43a, 0x456, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430,
0x441, 0x417, 0x430, 0x445, 0x43e, 0x434, 0x43d, 0x435, 0x435, 0x45e, 0x440, 0x430,
0x43f, 0x435, 0x439, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456,
0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x424, 0x430, 0x43b, 0x43a,
0x43b, 0x435, 0x43d, 0x434, 0x441, 0x43a, 0x456, 0x445, 0x20, 0x430, 0x441, 0x442,
0x440, 0x430, 0x432, 0x43e, 0x45e, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x424, 0x430, 0x43b, 0x43a,
0x43b, 0x435, 0x43d, 0x434, 0x441, 0x43a, 0x456, 0x445, 0x20, 0x430, 0x441, 0x442,
0x440, 0x430, 0x432, 0x43e, 0x45e, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447,
0x430, 0x441, 0x20, 0x424, 0x430, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x441,
0x43a, 0x456, 0x445, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e,
0x427, 0x430, 0x441, 0x20, 0x424, 0x456, 0x434, 0x436, 0x44b, 0x421, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20,
0x424, 0x456, 0x434, 0x436, 0x44b, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447,
0x430, 0x441, 0x20, 0x424, 0x456, 0x434, 0x436, 0x44b, 0x427, 0x430, 0x441, 0x20,
0x424, 0x440, 0x430, 0x43d, 0x446, 0x443, 0x437, 0x441, 0x43a, 0x430, 0x439, 0x20,
0x413, 0x432, 0x456, 0x44f, 0x43d, 0x44b, 0x427, 0x430, 0x441, 0x20, 0x424, 0x440,
0x430, 0x43d, 0x446, 0x443, 0x437, 0x441, 0x43a, 0x430, 0x439, 0x20, 0x43f, 0x430,
0x45e, 0x434, 0x43d, 0x451, 0x432, 0x430, 0x2d, 0x430, 0x43d, 0x442, 0x430, 0x440,
0x43a, 0x442, 0x44b, 0x447, 0x43d, 0x430, 0x439, 0x20, 0x442, 0x44d, 0x440, 0x44b,
0x442, 0x43e, 0x440, 0x44b, 0x456, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x413, 0x430, 0x43b, 0x430,
0x43f, 0x430, 0x433, 0x43e, 0x441, 0x43a, 0x456, 0x445, 0x20, 0x430, 0x441, 0x442,
0x440, 0x430, 0x432, 0x43e, 0x45e, 0x427, 0x430, 0x441, 0x20, 0x430, 0x441, 0x442,
0x440, 0x430, 0x432, 0x43e, 0x45e, 0x20, 0x413, 0x430, 0x43c, 0x431, 0x2019, 0x435,
0x413, 0x440, 0x443, 0x437, 0x456, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430,
0x441, 0x413, 0x440, 0x443, 0x437, 0x456, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430,
0x441, 0x413, 0x440, 0x443, 0x437, 0x456, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x43b,
0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20,
0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x20, 0x413, 0x456, 0x43b,
0x431, 0x435, 0x440, 0x442, 0x430, 0x427, 0x430, 0x441, 0x20, 0x43f, 0x430, 0x20,
0x413, 0x440, 0x44b, 0x43d, 0x432, 0x456, 0x447, 0x44b, 0x427, 0x430, 0x441, 0x20,
0x423, 0x441, 0x445, 0x43e, 0x434, 0x43d, 0x44f, 0x439, 0x20, 0x413, 0x440, 0x44d,
0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x456, 0x421, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x423, 0x441,
0x445, 0x43e, 0x434, 0x43d, 0x44f, 0x439, 0x20, 0x413, 0x440, 0x44d, 0x43d, 0x43b,
0x430, 0x43d, 0x434, 0x44b, 0x456, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447,
0x430, 0x441, 0x20, 0x423, 0x441, 0x445, 0x43e, 0x434, 0x43d, 0x44f, 0x439, 0x20,
0x413, 0x440, 0x44d, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x456, 0x427, 0x430,
0x441, 0x20, 0x417, 0x430, 0x445, 0x43e, 0x434, 0x43d, 0x44f, 0x439, 0x20, 0x413,
0x440, 0x44d, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x456, 0x421, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20,
0x417, 0x430, 0x445, 0x43e, 0x434, 0x43d, 0x44f, 0x439, 0x20, 0x413, 0x440, 0x44d,
0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x456, 0x41b, 0x435, 0x442, 0x43d, 0x456,
0x20, 0x447, 0x430, 0x441, 0x20, 0x417, 0x430, 0x445, 0x43e, 0x434, 0x43d, 0x44f,
0x439, 0x20, 0x413, 0x440, 0x44d, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x456,
0x427, 0x430, 0x441, 0x20, 0x41f, 0x435, 0x440, 0x441, 0x456, 0x434, 0x441, 0x43a,
0x430, 0x433, 0x430, 0x20, 0x437, 0x430, 0x43b, 0x456, 0x432, 0x430, 0x427, 0x430,
0x441, 0x20, 0x413, 0x430, 0x44f, 0x43d, 0x44b, 0x413, 0x430, 0x432, 0x430, 0x439,
0x441, 0x43a, 0x430, 0x2d, 0x410, 0x43b, 0x435, 0x443, 0x446, 0x43a, 0x456, 0x20,
0x447, 0x430, 0x441, 0x413, 0x430, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x430, 0x2d,
0x410, 0x43b, 0x435, 0x443, 0x446, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x413, 0x430,
0x432, 0x430, 0x439, 0x441, 0x43a, 0x430, 0x2d, 0x410, 0x43b, 0x435, 0x443, 0x446,
0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441,
0x427, 0x430, 0x441, 0x20, 0x413, 0x430, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x430,
0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447,
0x430, 0x441, 0x20, 0x413, 0x430, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x430, 0x41b,
0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x413, 0x430, 0x43d,
0x43a, 0x43e, 0x43d, 0x433, 0x430, 0x427, 0x430, 0x441, 0x20, 0x425, 0x43e, 0x45e,
0x434, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b,
0x20, 0x447, 0x430, 0x441, 0x20, 0x425, 0x43e, 0x45e, 0x434, 0x430, 0x41b, 0x435,
0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x425, 0x43e, 0x45e, 0x434,
0x430, 0x427, 0x430, 0x441, 0x20, 0x406, 0x43d, 0x434, 0x44b, 0x456, 0x427, 0x430,
0x441, 0x20, 0x406, 0x43d, 0x434, 0x44b, 0x439, 0x441, 0x43a, 0x430, 0x433, 0x430,
0x20, 0x430, 0x43a, 0x456, 0x44f, 0x43d, 0x430, 0x406, 0x43d, 0x434, 0x430, 0x43a,
0x456, 0x442, 0x430, 0x439, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x426,
0x44d, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x456, 0x43d, 0x434,
0x430, 0x43d, 0x435, 0x437, 0x456, 0x439, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430,
0x441, 0x423, 0x441, 0x445, 0x43e, 0x434, 0x43d, 0x435, 0x456, 0x43d, 0x434, 0x430,
0x43d, 0x435, 0x437, 0x456, 0x439, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441,
0x417, 0x430, 0x445, 0x43e, 0x434, 0x43d, 0x435, 0x456, 0x43d, 0x434, 0x430, 0x43d,
0x435, 0x437, 0x456, 0x439, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x406,
0x440, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x406, 0x440,
0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x406, 0x440, 0x430, 0x43d,
0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430,
0x441, 0x406, 0x440, 0x43a, 0x443, 0x446, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441,
0x406, 0x440, 0x43a, 0x443, 0x446, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x406, 0x440,
0x43a, 0x443, 0x446, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20,
0x447, 0x430, 0x441, 0x406, 0x437, 0x440, 0x430, 0x456, 0x43b, 0x44c, 0x441, 0x43a,
0x456, 0x20, 0x447, 0x430, 0x441, 0x406, 0x437, 0x440, 0x430, 0x456, 0x43b, 0x44c,
0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x406, 0x437, 0x440, 0x430, 0x456, 0x43b,
0x44c, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447,
0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x42f, 0x43f, 0x43e, 0x43d, 0x456, 0x456,
0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447,
0x430, 0x441, 0x20, 0x42f, 0x43f, 0x43e, 0x43d, 0x456, 0x456, 0x41b, 0x435, 0x442,
0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x42f, 0x43f, 0x43e, 0x43d, 0x456,
0x456, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a,
0x456, 0x20, 0x447, 0x430, 0x441, 0x423, 0x441, 0x445, 0x43e, 0x434, 0x43d, 0x435,
0x43a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x456,
0x20, 0x447, 0x430, 0x441, 0x417, 0x430, 0x445, 0x43e, 0x434, 0x43d, 0x435, 0x43a,
0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20,
0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x41a, 0x430, 0x440, 0x44d, 0x456,
0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447,
0x430, 0x441, 0x20, 0x41a, 0x430, 0x440, 0x44d, 0x456, 0x41b, 0x435, 0x442, 0x43d,
0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x41a, 0x430, 0x440, 0x44d, 0x456, 0x427,
0x430, 0x441, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x20,
0x41a, 0x443, 0x441, 0x430, 0x456, 0x435, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x430,
0x44f, 0x440, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x41a, 0x440, 0x430,
0x441, 0x43d, 0x430, 0x44f, 0x440, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x41a,
0x440, 0x430, 0x441, 0x43d, 0x430, 0x44f, 0x440, 0x441, 0x43a, 0x456, 0x20, 0x43b,
0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20,
0x41a, 0x44b, 0x440, 0x433, 0x44b, 0x437, 0x441, 0x442, 0x430, 0x43d, 0x430, 0x427,
0x430, 0x441, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x20,
0x41b, 0x430, 0x439, 0x43d, 0x427, 0x430, 0x441, 0x20, 0x41b, 0x43e, 0x440, 0x434,
0x2d, 0x425, 0x430, 0x443, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x41b, 0x43e, 0x440, 0x434, 0x2d,
0x425, 0x430, 0x443, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441,
0x20, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x425, 0x430, 0x443, 0x41c, 0x430, 0x433,
0x430, 0x434, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x41c,
0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441,
0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x43b,
0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20,
0x41c, 0x430, 0x43b, 0x430, 0x439, 0x437, 0x456, 0x456, 0x427, 0x430, 0x441, 0x20,
0x41c, 0x430, 0x43b, 0x44c, 0x434, 0x44b, 0x432, 0x430, 0x45e, 0x427, 0x430, 0x441,
0x20, 0x41c, 0x430, 0x440, 0x43a, 0x456, 0x437, 0x441, 0x43a, 0x456, 0x445, 0x20,
0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x427, 0x430, 0x441, 0x20,
0x41c, 0x430, 0x440, 0x448, 0x430, 0x43b, 0x430, 0x432, 0x44b, 0x445, 0x20, 0x430,
0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x427, 0x430, 0x441, 0x20, 0x41c,
0x430, 0x45e, 0x440, 0x44b, 0x43a, 0x456, 0x44f, 0x421, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x41c, 0x430,
0x45e, 0x440, 0x44b, 0x43a, 0x456, 0x44f, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20,
0x447, 0x430, 0x441, 0x20, 0x41c, 0x430, 0x45e, 0x440, 0x44b, 0x43a, 0x456, 0x44f,
0x427, 0x430, 0x441, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x446, 0x44b, 0x456, 0x20,
0x41c, 0x43e, 0x45e, 0x441, 0x430, 0x43d, 0x41c, 0x435, 0x43a, 0x441, 0x456, 0x43a,
0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x446, 0x456, 0x445, 0x430, 0x430, 0x43a,
0x456, 0x44f, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x41c, 0x435,
0x43a, 0x441, 0x456, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x446, 0x456,
0x445, 0x430, 0x430, 0x43a, 0x456, 0x44f, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441,
0x41c, 0x435, 0x43a, 0x441, 0x456, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20,
0x446, 0x456, 0x445, 0x430, 0x430, 0x43a, 0x456, 0x44f, 0x43d, 0x441, 0x43a, 0x456,
0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430,
0x441, 0x20, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x430, 0x440,
0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20,
0x447, 0x430, 0x441, 0x20, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442,
0x430, 0x440, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441,
0x20, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x430, 0x440, 0x430,
0x41c, 0x430, 0x441, 0x43a, 0x43e, 0x45e, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430,
0x441, 0x41c, 0x430, 0x441, 0x43a, 0x43e, 0x45e, 0x441, 0x43a, 0x456, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430,
0x441, 0x41c, 0x430, 0x441, 0x43a, 0x43e, 0x45e, 0x441, 0x43a, 0x456, 0x20, 0x43b,
0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20,
0x41c, 0x2019, 0x44f, 0x43d, 0x43c, 0x44b, 0x427, 0x430, 0x441, 0x20, 0x41d, 0x430,
0x443, 0x440, 0x443, 0x41d, 0x435, 0x43f, 0x430, 0x43b, 0x44c, 0x441, 0x43a, 0x456,
0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x41d, 0x43e, 0x432, 0x430,
0x439, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x456, 0x456, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430,
0x441, 0x20, 0x41d, 0x43e, 0x432, 0x430, 0x439, 0x20, 0x41a, 0x430, 0x43b, 0x435,
0x434, 0x43e, 0x43d, 0x456, 0x456, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447,
0x430, 0x441, 0x20, 0x41d, 0x43e, 0x432, 0x430, 0x439, 0x20, 0x41a, 0x430, 0x43b,
0x435, 0x434, 0x43e, 0x43d, 0x456, 0x456, 0x427, 0x430, 0x441, 0x20, 0x41d, 0x43e,
0x432, 0x430, 0x439, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x456,
0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447,
0x430, 0x441, 0x20, 0x41d, 0x43e, 0x432, 0x430, 0x439, 0x20, 0x417, 0x435, 0x43b,
0x430, 0x43d, 0x434, 0x44b, 0x456, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447,
0x430, 0x441, 0x20, 0x41d, 0x43e, 0x432, 0x430, 0x439, 0x20, 0x417, 0x435, 0x43b,
0x430, 0x43d, 0x434, 0x44b, 0x456, 0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x45e, 0x43d,
0x434, 0x43b, 0x435, 0x43d, 0x434, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441,
0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x45e, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434,
0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x45e,
0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x41d,
0x456, 0x443, 0x44d, 0x427, 0x430, 0x441, 0x20, 0x432, 0x43e, 0x441, 0x442, 0x440,
0x430, 0x432, 0x430, 0x20, 0x41d, 0x43e, 0x440, 0x444, 0x430, 0x43b, 0x43a, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430,
0x441, 0x20, 0x432, 0x43e, 0x441, 0x442, 0x440, 0x430, 0x432, 0x430, 0x20, 0x41d,
0x43e, 0x440, 0x444, 0x430, 0x43b, 0x43a, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20,
0x447, 0x430, 0x441, 0x20, 0x432, 0x43e, 0x441, 0x442, 0x440, 0x430, 0x432, 0x430,
0x20, 0x41d, 0x43e, 0x440, 0x444, 0x430, 0x43b, 0x43a, 0x427, 0x430, 0x441, 0x20,
0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x437, 0x456,
0x2d, 0x41d, 0x430, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x421, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x424,
0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x437, 0x456, 0x2d,
0x41d, 0x430, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x41b, 0x435, 0x442, 0x43d, 0x456,
0x20, 0x447, 0x430, 0x441, 0x20, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434,
0x443, 0x2d, 0x434, 0x437, 0x456, 0x2d, 0x41d, 0x430, 0x440, 0x43e, 0x43d, 0x44c,
0x44f, 0x41d, 0x43e, 0x432, 0x430, 0x441, 0x456, 0x431, 0x456, 0x440, 0x441, 0x43a,
0x456, 0x20, 0x447, 0x430, 0x441, 0x41d, 0x43e, 0x432, 0x430, 0x441, 0x456, 0x431,
0x456, 0x440, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x41d, 0x43e, 0x432, 0x430,
0x441, 0x456, 0x431, 0x456, 0x440, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x41e, 0x43c, 0x441, 0x43a, 0x456, 0x20,
0x447, 0x430, 0x441, 0x41e, 0x43c, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x41e,
0x43c, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447,
0x430, 0x441, 0x41f, 0x430, 0x43a, 0x456, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a,
0x456, 0x20, 0x447, 0x430, 0x441, 0x41f, 0x430, 0x43a, 0x456, 0x441, 0x442, 0x430,
0x43d, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x41f, 0x430, 0x43a, 0x456, 0x441,
0x442, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456,
0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x41f, 0x430, 0x43b, 0x430,
0x443, 0x427, 0x430, 0x441, 0x20, 0x41f, 0x430, 0x43f, 0x443, 0x430, 0x2d, 0x41d,
0x43e, 0x432, 0x430, 0x439, 0x20, 0x413, 0x432, 0x456, 0x43d, 0x435, 0x456, 0x427,
0x430, 0x441, 0x20, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x44f, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430,
0x441, 0x20, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x44f, 0x41b, 0x435,
0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x41f, 0x430, 0x440, 0x430,
0x433, 0x432, 0x430, 0x44f, 0x41f, 0x435, 0x440, 0x443, 0x430, 0x43d, 0x441, 0x43a,
0x456, 0x20, 0x447, 0x430, 0x441, 0x41f, 0x435, 0x440, 0x443, 0x430, 0x43d, 0x441,
0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x44b, 0x20, 0x447, 0x430, 0x441, 0x41f, 0x435, 0x440, 0x443, 0x430, 0x43d, 0x441,
0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441,
0x424, 0x456, 0x43b, 0x456, 0x43f, 0x456, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x447,
0x430, 0x441, 0x424, 0x456, 0x43b, 0x456, 0x43f, 0x456, 0x43d, 0x441, 0x43a, 0x456,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20,
0x447, 0x430, 0x441, 0x424, 0x456, 0x43b, 0x456, 0x43f, 0x456, 0x43d, 0x441, 0x43a,
0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427,
0x430, 0x441, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x20,
0x424, 0x435, 0x43d, 0x456, 0x43a, 0x441, 0x427, 0x430, 0x441, 0x20, 0x421, 0x435,
0x43d, 0x2d, 0x41f, 0x2019, 0x435, 0x440, 0x20, 0x456, 0x20, 0x41c, 0x456, 0x43a,
0x435, 0x43b, 0x43e, 0x43d, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x421, 0x435, 0x43d, 0x2d, 0x41f,
0x2019, 0x435, 0x440, 0x20, 0x456, 0x20, 0x41c, 0x456, 0x43a, 0x435, 0x43b, 0x43e,
0x43d, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20,
0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x421, 0x435,
0x43d, 0x2d, 0x41f, 0x2019, 0x435, 0x440, 0x20, 0x456, 0x20, 0x41c, 0x456, 0x43a,
0x435, 0x43b, 0x43e, 0x43d, 0x427, 0x430, 0x441, 0x20, 0x432, 0x43e, 0x441, 0x442,
0x440, 0x430, 0x432, 0x430, 0x20, 0x41f, 0x456, 0x442, 0x43a, 0x44d, 0x440, 0x43d,
0x427, 0x430, 0x441, 0x20, 0x432, 0x43e, 0x441, 0x442, 0x440, 0x430, 0x432, 0x430,
0x20, 0x41f, 0x430, 0x43d, 0x43f, 0x435, 0x456, 0x41f, 0x445, 0x435, 0x43d, 0x44c,
0x44f, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441,
0x20, 0x420, 0x44d, 0x44e, 0x43d, 0x44c, 0x451, 0x43d, 0x430, 0x427, 0x430, 0x441,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x446, 0x44b, 0x456, 0x20, 0x420, 0x43e, 0x442,
0x44d, 0x440, 0x430, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x456, 0x43d, 0x441, 0x43a,
0x456, 0x20, 0x447, 0x430, 0x441, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x456, 0x43d,
0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x456,
0x43d, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447,
0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430,
0x441, 0x20, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x456,
0x20, 0x447, 0x430, 0x441, 0x20, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x427, 0x430,
0x441, 0x20, 0x421, 0x435, 0x439, 0x448, 0x44d, 0x43b, 0x44c, 0x441, 0x43a, 0x456,
0x445, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x421, 0x456,
0x43d, 0x433, 0x430, 0x43f, 0x443, 0x440, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430,
0x441, 0x427, 0x430, 0x441, 0x20, 0x421, 0x430, 0x43b, 0x430, 0x43c, 0x43e, 0x43d,
0x430, 0x432, 0x44b, 0x445, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e,
0x45e, 0x427, 0x430, 0x441, 0x20, 0x41f, 0x430, 0x45e, 0x434, 0x43d, 0x451, 0x432,
0x430, 0x439, 0x20, 0x413, 0x435, 0x43e, 0x440, 0x433, 0x456, 0x456, 0x427, 0x430,
0x441, 0x20, 0x421, 0x443, 0x440, 0x44b, 0x43d, 0x430, 0x43c, 0x430, 0x427, 0x430,
0x441, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x446, 0x44b, 0x456, 0x20, 0x421, 0x451,
0x432, 0x430, 0x427, 0x430, 0x441, 0x20, 0x422, 0x430, 0x456, 0x446, 0x456, 0x427,
0x430, 0x441, 0x20, 0x422, 0x430, 0x439, 0x431, 0x44d, 0x44f, 0x421, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20,
0x422, 0x430, 0x439, 0x431, 0x44d, 0x44f, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20,
0x447, 0x430, 0x441, 0x20, 0x422, 0x430, 0x439, 0x431, 0x44d, 0x44f, 0x427, 0x430,
0x441, 0x20, 0x422, 0x430, 0x434, 0x436, 0x44b, 0x43a, 0x456, 0x441, 0x442, 0x430,
0x43d, 0x430, 0x427, 0x430, 0x441, 0x20, 0x422, 0x430, 0x43a, 0x435, 0x43b, 0x430,
0x443, 0x427, 0x430, 0x441, 0x20, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x421, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441,
0x20, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20,
0x447, 0x430, 0x441, 0x20, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x427, 0x430, 0x441,
0x20, 0x422, 0x440, 0x443, 0x43a, 0x430, 0x427, 0x430, 0x441, 0x20, 0x422, 0x443,
0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x456, 0x441, 0x442, 0x430, 0x43d, 0x430, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430,
0x441, 0x20, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x456, 0x441, 0x442,
0x430, 0x43d, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441,
0x20, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x456, 0x441, 0x442, 0x430,
0x43d, 0x430, 0x427, 0x430, 0x441, 0x20, 0x422, 0x443, 0x432, 0x430, 0x43b, 0x443,
0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x456, 0x20, 0x447,
0x430, 0x441, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x456,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20,
0x447, 0x430, 0x441, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x441, 0x43a,
0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427,
0x430, 0x441, 0x20, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x456, 0x441, 0x442, 0x430,
0x43d, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b,
0x20, 0x447, 0x430, 0x441, 0x20, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x456, 0x441,
0x442, 0x430, 0x43d, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430,
0x441, 0x20, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x456, 0x441, 0x442, 0x430, 0x43d,
0x430, 0x427, 0x430, 0x441, 0x20, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443,
0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447,
0x430, 0x441, 0x20, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x41b, 0x435,
0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x412, 0x430, 0x43d, 0x443,
0x430, 0x442, 0x443, 0x412, 0x435, 0x43d, 0x435, 0x441, 0x443, 0x44d, 0x43b, 0x44c,
0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x423, 0x43b, 0x430, 0x434, 0x437,
0x456, 0x432, 0x430, 0x441, 0x442, 0x43e, 0x446, 0x43a, 0x456, 0x20, 0x447, 0x430,
0x441, 0x423, 0x43b, 0x430, 0x434, 0x437, 0x456, 0x432, 0x430, 0x441, 0x442, 0x43e,
0x446, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x423, 0x43b, 0x430, 0x434, 0x437, 0x456,
0x432, 0x430, 0x441, 0x442, 0x43e, 0x446, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x412, 0x430, 0x43b, 0x433, 0x430, 0x433,
0x440, 0x430, 0x434, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x412, 0x430,
0x43b, 0x433, 0x430, 0x433, 0x440, 0x430, 0x434, 0x441, 0x43a, 0x456, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430,
0x441, 0x412, 0x430, 0x43b, 0x433, 0x430, 0x433, 0x440, 0x430, 0x434, 0x441, 0x43a,
0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427,
0x430, 0x441, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x446, 0x44b, 0x456, 0x20, 0x412,
0x430, 0x441, 0x442, 0x43e, 0x43a, 0x427, 0x430, 0x441, 0x20, 0x432, 0x43e, 0x441,
0x442, 0x440, 0x430, 0x432, 0x430, 0x20, 0x423, 0x44d, 0x439, 0x43a, 0x427, 0x430,
0x441, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x20, 0x423,
0x43e, 0x43b, 0x456, 0x441, 0x20, 0x456, 0x20, 0x424, 0x443, 0x442, 0x443, 0x43d,
0x430, 0x42f, 0x43a, 0x443, 0x446, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x42f,
0x43a, 0x443, 0x446, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x42f, 0x43a, 0x443, 0x446,
0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441,
0x415, 0x43a, 0x430, 0x446, 0x44f, 0x440, 0x44b, 0x43d, 0x431, 0x443, 0x440, 0x433,
0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x415, 0x43a, 0x430, 0x446, 0x44f,
0x440, 0x44b, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x441, 0x43a, 0x456, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430,
0x441, 0x415, 0x43a, 0x430, 0x446, 0x44f, 0x440, 0x44b, 0x43d, 0x431, 0x443, 0x440,
0x433, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447,
0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x42e, 0x43a, 0x430, 0x43d, 0x430, 0x917,
0x94d, 0x930, 0x940, 0x928, 0x935, 0x93f, 0x91a, 0x20, 0x92e, 0x940, 0x928, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x90f, 0x915, 0x94b, 0x930, 0x20, 0x938, 0x92e, 0x906,
0x915, 0x930, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d, 0x921,
0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x906, 0x915, 0x930, 0x20, 0x938, 0x92e, 0x930,
0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x906, 0x92b, 0x917, 0x93e, 0x928, 0x940, 0x938,
0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92e, 0x93f, 0x930, 0x941, 0x20,
0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x20, 0x938, 0x92e, 0x938, 0x93e,
0x928, 0x91c, 0x93e, 0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x20,
0x938, 0x92e, 0x916, 0x94b, 0x932, 0x93e, 0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f,
0x915, 0x93e, 0x928, 0x93f, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f,
0x20, 0x938, 0x92e, 0x938, 0x94b, 0x928, 0x93e, 0x92c, 0x20, 0x906, 0x92b, 0x94d,
0x930, 0x93f, 0x915, 0x93e, 0x928, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x94b, 0x928,
0x93e, 0x92c, 0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x928, 0x93f,
0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938,
0x94b, 0x928, 0x93e, 0x92c, 0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x93e,
0x928, 0x93f, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938,
0x92e, 0x906, 0x932, 0x93e, 0x938, 0x94d, 0x915, 0x93e, 0x928, 0x93f, 0x20, 0x938,
0x92e, 0x906, 0x932, 0x93e, 0x938, 0x94d, 0x915, 0x93e, 0x20, 0x938, 0x94d, 0x91f,
0x947, 0x928, 0x94d, 0x921, 0x93e, 0x930, 0x94d, 0x921, 0x20, 0x938, 0x92e, 0x906,
0x932, 0x93e, 0x938, 0x94d, 0x915, 0x93e, 0x928, 0x93f, 0x20, 0x938, 0x93e, 0x928,
0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x905, 0x932, 0x92e, 0x93e, 0x91f, 0x940,
0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x905, 0x932, 0x92e, 0x93e, 0x91f, 0x940, 0x20,
0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e,
0x908, 0x92e, 0x905, 0x932, 0x92e, 0x93e, 0x91f, 0x940, 0x20, 0x938, 0x92e, 0x930,
0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x906, 0x92e, 0x93e, 0x91c, 0x928, 0x20, 0x938,
0x92e, 0x906, 0x92e, 0x93e, 0x91c, 0x928, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906,
0x931, 0x93f, 0x20, 0x938, 0x92e, 0x906, 0x92e, 0x93e, 0x91c, 0x928, 0x20, 0x926,
0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x939,
0x93e, 0x20, 0x906, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x928, 0x93f, 0x20,
0x92e, 0x93f, 0x930, 0x941, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x939, 0x93e, 0x20,
0x906, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x928, 0x93f, 0x20, 0x92e, 0x93f,
0x930, 0x941, 0x20, 0x92e, 0x93e, 0x928, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930,
0x93f, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x939, 0x93e, 0x20, 0x906, 0x92e, 0x947,
0x930, 0x93f, 0x915, 0x93e, 0x928, 0x93f, 0x20, 0x92e, 0x93f, 0x930, 0x941, 0x20,
0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x939,
0x93e, 0x20, 0x906, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x928, 0x93f, 0x20,
0x938, 0x93e, 0x928, 0x91c, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x938, 0x93e, 0x939, 0x93e, 0x20, 0x906, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e,
0x928, 0x93f, 0x20, 0x938, 0x93e, 0x928, 0x91c, 0x93e, 0x92f, 0x93e, 0x930, 0x93f,
0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938,
0x93e, 0x939, 0x93e, 0x20, 0x906, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x928,
0x93f, 0x20, 0x938, 0x93e, 0x928, 0x91c, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20,
0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x939,
0x93e, 0x20, 0x906, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x928, 0x93f, 0x20,
0x939, 0x93e, 0x91c, 0x94b, 0x92e, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20, 0x938,
0x92e, 0x938, 0x93e, 0x939, 0x93e, 0x20, 0x906, 0x92e, 0x947, 0x930, 0x93f, 0x915,
0x93e, 0x928, 0x93f, 0x20, 0x939, 0x93e, 0x91c, 0x94b, 0x92e, 0x93e, 0x92f, 0x93e,
0x930, 0x93f, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938,
0x92e, 0x938, 0x93e, 0x939, 0x93e, 0x20, 0x906, 0x92e, 0x94b, 0x930, 0x93f, 0x915,
0x93e, 0x928, 0x93f, 0x20, 0x939, 0x93e, 0x91c, 0x94b, 0x92e, 0x93e, 0x92f, 0x93e,
0x930, 0x93f, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x938, 0x93e, 0x939, 0x93e, 0x20, 0x906, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e,
0x928, 0x93f, 0x20, 0x92a, 0x947, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x938,
0x92e, 0x938, 0x93e, 0x939, 0x93e, 0x20, 0x906, 0x92e, 0x947, 0x930, 0x93f, 0x915,
0x93e, 0x928, 0x93f, 0x20, 0x92a, 0x947, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20,
0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x93e,
0x939, 0x93e, 0x20, 0x906, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x928, 0x93f,
0x20, 0x92a, 0x947, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x938, 0x93e, 0x928,
0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x906, 0x928, 0x93e, 0x926, 0x940, 0x930,
0x20, 0x938, 0x92e, 0x905, 0x928, 0x93e, 0x926, 0x940, 0x930, 0x20, 0x938, 0x94d,
0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e,
0x905, 0x928, 0x93e, 0x926, 0x940, 0x930, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f,
0x93e, 0x908, 0x92e, 0x906, 0x92a, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x906,
0x92a, 0x93f, 0x92f, 0x93e, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f,
0x20, 0x938, 0x92e, 0x906, 0x92a, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x93e, 0x928,
0x20, 0x938, 0x92e, 0x905, 0x915, 0x94d, 0x91f, 0x93e, 0x90a, 0x20, 0x938, 0x92e,
0x905, 0x915, 0x93c, 0x94d, 0x91f, 0x93e, 0x90a, 0x20, 0x938, 0x94d, 0x91f, 0x948,
0x902, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x905, 0x915,
0x93c, 0x94d, 0x91f, 0x93e, 0x90a, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e,
0x908, 0x92e, 0x906, 0x915, 0x94d, 0x924, 0x2019, 0x92c, 0x20, 0x938, 0x92e, 0x905,
0x915, 0x93c, 0x94d, 0x91f, 0x94b, 0x92c, 0x947, 0x20, 0x938, 0x94d, 0x91f, 0x948,
0x902, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x905, 0x915,
0x93c, 0x94d, 0x91f, 0x94b, 0x92c, 0x947, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f,
0x93e, 0x908, 0x92e, 0x906, 0x930, 0x93e, 0x92c, 0x940, 0x92f, 0x93e, 0x928, 0x20,
0x938, 0x92e, 0x906, 0x930, 0x93e, 0x92c, 0x940, 0x92f, 0x93e, 0x928, 0x20, 0x925,
0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x906, 0x930, 0x93e,
0x92c, 0x940, 0x92f, 0x93e, 0x928, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f,
0x20, 0x938, 0x92e, 0x906, 0x930, 0x94d, 0x91c, 0x947, 0x928, 0x94d, 0x91f, 0x93f,
0x928, 0x93e, 0x20, 0x938, 0x92e, 0x906, 0x930, 0x94d, 0x91c, 0x947, 0x928, 0x94d,
0x91f, 0x93f, 0x928, 0x93e, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f,
0x20, 0x938, 0x92e, 0x906, 0x930, 0x94d, 0x91c, 0x947, 0x928, 0x94d, 0x91f, 0x93f,
0x928, 0x93e, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938,
0x92e, 0x938, 0x94b, 0x928, 0x93e, 0x92c, 0x20, 0x906, 0x930, 0x94d, 0x91c, 0x947,
0x928, 0x94d, 0x91f, 0x93f, 0x928, 0x93e, 0x20, 0x938, 0x92e, 0x938, 0x94b, 0x928,
0x93e, 0x92c, 0x20, 0x906, 0x930, 0x94d, 0x91c, 0x947, 0x928, 0x94d, 0x91f, 0x93f,
0x928, 0x93e, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938,
0x92e, 0x938, 0x94b, 0x928, 0x93e, 0x92c, 0x20, 0x906, 0x930, 0x94d, 0x91c, 0x947,
0x928, 0x94d, 0x91f, 0x93f, 0x928, 0x93e, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932,
0x93e, 0x902, 0x20, 0x938, 0x92e, 0x906, 0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f,
0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x906, 0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f,
0x92f, 0x93e, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938,
0x92e, 0x906, 0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x926,
0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x906, 0x91f, 0x932,
0x93e, 0x928, 0x94d, 0x91f, 0x93f, 0x915, 0x20, 0x938, 0x92e, 0x906, 0x91f, 0x932,
0x93e, 0x928, 0x94d, 0x91f, 0x93f, 0x915, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906,
0x930, 0x93f, 0x20, 0x938, 0x92e, 0x906, 0x91f, 0x932, 0x93e, 0x928, 0x94d, 0x91f,
0x93f, 0x915, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x92e, 0x93f, 0x930, 0x941, 0x20, 0x905, 0x2019, 0x938, 0x94d, 0x91f, 0x94d, 0x930,
0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x905, 0x2019, 0x938, 0x94d,
0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x928, 0x93f, 0x20, 0x92e,
0x93f, 0x930, 0x941, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x905, 0x2019, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f,
0x92f, 0x93e, 0x20, 0x92e, 0x93f, 0x930, 0x941, 0x20, 0x938, 0x93e, 0x928, 0x93e,
0x930, 0x93f, 0x20, 0x938, 0x92e, 0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947,
0x932, 0x93f, 0x92f, 0x93e, 0x928, 0x93f, 0x20, 0x92e, 0x93f, 0x930, 0x941, 0x20,
0x938, 0x94b, 0x928, 0x93e, 0x92c, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x905,
0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x928, 0x93f,
0x20, 0x92e, 0x93f, 0x930, 0x941, 0x20, 0x938, 0x94b, 0x928, 0x93e, 0x92c, 0x93e,
0x930, 0x93f, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938,
0x92e, 0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e,
0x928, 0x93f, 0x20, 0x92e, 0x93f, 0x930, 0x941, 0x20, 0x938, 0x94b, 0x928, 0x93e,
0x92c, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x938, 0x93e, 0x928, 0x91c, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20,
0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20,
0x938, 0x92e, 0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f,
0x93e, 0x928, 0x93f, 0x20, 0x938, 0x93e, 0x928, 0x91c, 0x93e, 0x92f, 0x93e, 0x930,
0x93f, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x928,
0x93f, 0x20, 0x938, 0x93e, 0x928, 0x91c, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20,
0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x94b, 0x928,
0x93e, 0x92c, 0x93e, 0x930, 0x93f, 0x20, 0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930,
0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x905, 0x938, 0x94d, 0x91f,
0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x928, 0x93f, 0x20, 0x938, 0x94b,
0x928, 0x93e, 0x92c, 0x93e, 0x930, 0x93f, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906,
0x930, 0x93f, 0x20, 0x938, 0x92e, 0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947,
0x932, 0x93f, 0x92f, 0x93e, 0x928, 0x93f, 0x20, 0x938, 0x94b, 0x928, 0x93e, 0x92c,
0x93e, 0x930, 0x93f, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938,
0x92e, 0x906, 0x91c, 0x93e, 0x930, 0x92c, 0x93e, 0x908, 0x91c, 0x93e, 0x928, 0x20,
0x938, 0x92e, 0x906, 0x91c, 0x93e, 0x930, 0x92c, 0x93e, 0x908, 0x91c, 0x93e, 0x928,
0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x906,
0x91c, 0x93e, 0x930, 0x92c, 0x93e, 0x908, 0x91c, 0x93e, 0x928, 0x20, 0x926, 0x948,
0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x906, 0x91c, 0x2019, 0x930,
0x94d, 0x938, 0x928, 0x93f, 0x20, 0x938, 0x92e, 0x906, 0x91c, 0x2019, 0x930, 0x94d,
0x938, 0x928, 0x93f, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x906, 0x91c, 0x2019, 0x930, 0x94d, 0x938, 0x928, 0x93f, 0x20, 0x926,
0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92c, 0x93e, 0x902,
0x932, 0x93e, 0x926, 0x947, 0x936, 0x20, 0x938, 0x92e, 0x92c, 0x93e, 0x902, 0x932,
0x93e, 0x926, 0x947, 0x936, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f,
0x20, 0x938, 0x92e, 0x92c, 0x93e, 0x902, 0x932, 0x93e, 0x926, 0x947, 0x936, 0x20,
0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92d, 0x941,
0x91f, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92c, 0x932, 0x93f, 0x92d, 0x93f, 0x92f,
0x93e, 0x20, 0x938, 0x92e, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x940, 0x932, 0x93f,
0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x940, 0x932,
0x93f, 0x92f, 0x93e, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x940, 0x932, 0x93f, 0x92f, 0x93e,
0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92c,
0x94d, 0x930, 0x941, 0x928, 0x947, 0x908, 0x20, 0x926, 0x930, 0x20, 0x909, 0x938,
0x20, 0x938, 0x932, 0x93e, 0x92e, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921,
0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x915, 0x947, 0x92a, 0x20,
0x92d, 0x947, 0x930, 0x94d, 0x926, 0x947, 0x20, 0x938, 0x92e, 0x915, 0x947, 0x92a,
0x20, 0x92d, 0x947, 0x930, 0x94d, 0x926, 0x947, 0x20, 0x925, 0x93e, 0x916, 0x94b,
0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x915, 0x947, 0x92a, 0x20, 0x92d, 0x947,
0x930, 0x94d, 0x926, 0x947, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902,
0x20, 0x938, 0x92e, 0x91a, 0x93e, 0x92e, 0x2019, 0x930, 0x94d, 0x930, 0x2019, 0x20,
0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x91a, 0x947,
0x925, 0x93e, 0x92e, 0x20, 0x938, 0x92e, 0x91a, 0x947, 0x925, 0x93e, 0x92e, 0x20,
0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x91a, 0x947,
0x925, 0x93e, 0x92e, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938,
0x92e, 0x91a, 0x940, 0x932, 0x947, 0x20, 0x938, 0x92e, 0x91a, 0x940, 0x932, 0x947,
0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x91a,
0x940, 0x932, 0x947, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20,
0x938, 0x92e, 0x91a, 0x93e, 0x908, 0x928, 0x93e, 0x20, 0x938, 0x92e, 0x91a, 0x93e,
0x908, 0x928, 0x93e, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x91a, 0x93e, 0x908, 0x928, 0x93e, 0x20, 0x938, 0x93e, 0x928, 0x93e,
0x930, 0x93f, 0x20, 0x938, 0x92e, 0x916, 0x94d, 0x930, 0x940, 0x938, 0x94d, 0x91f,
0x92e, 0x93e, 0x938, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x938, 0x92e,
0x915, 0x915, 0x2019, 0x938, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x92b, 0x94b,
0x930, 0x928, 0x93f, 0x20, 0x938, 0x92e, 0x915, 0x932, 0x92e, 0x94d, 0x92c, 0x93f,
0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x915, 0x932, 0x92e, 0x94d, 0x92c, 0x93f, 0x92f,
0x93e, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x915, 0x932, 0x92e, 0x94d, 0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x948, 0x91c,
0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x915, 0x941, 0x915, 0x20, 0x926,
0x94d, 0x935, 0x940, 0x92a, 0x92b, 0x94b, 0x930, 0x928, 0x93f, 0x20, 0x938, 0x92e,
0x915, 0x941, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x92b, 0x94b, 0x930,
0x928, 0x93f, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938,
0x92e, 0x915, 0x941, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x92b, 0x94b,
0x930, 0x928, 0x93f, 0x20, 0x916, 0x93e, 0x935, 0x938, 0x947, 0x20, 0x926, 0x948,
0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x915, 0x93f, 0x909, 0x92c,
0x93e, 0x20, 0x938, 0x92e, 0x915, 0x93f, 0x909, 0x92c, 0x93e, 0x20, 0x925, 0x93e,
0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x915, 0x93f, 0x909, 0x92c,
0x93e, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x921,
0x947, 0x92d, 0x93f, 0x938, 0x20, 0x938, 0x92e, 0x921, 0x941, 0x92e, 0x928, 0x94d,
0x924, 0x2d, 0x921, 0x940, 0x906, 0x930, 0x94d, 0x92d, 0x940, 0x932, 0x20, 0x938,
0x92e, 0x908, 0x938, 0x94d, 0x91f, 0x20, 0x91f, 0x940, 0x92e, 0x94b, 0x930, 0x20,
0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e,
0x908, 0x92e, 0x938, 0x93e, 0x928, 0x91c, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20,
0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x928, 0x91c,
0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20,
0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x93e,
0x928, 0x91c, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20, 0x926, 0x94d, 0x935, 0x940,
0x92a, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e,
0x90f, 0x915, 0x941, 0x935, 0x93e, 0x921, 0x930, 0x20, 0x938, 0x92e, 0x92e, 0x93f,
0x930, 0x941, 0x20, 0x92f, 0x941, 0x930, 0x2019, 0x92a, 0x928, 0x93f, 0x20, 0x938,
0x92e, 0x92e, 0x93f, 0x930, 0x941, 0x20, 0x92f, 0x941, 0x930, 0x2019, 0x92a, 0x928,
0x93f, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x92e, 0x93f, 0x930, 0x941, 0x20, 0x92f, 0x941, 0x930, 0x2019, 0x92a, 0x928, 0x93f,
0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x938,
0x93e, 0x928, 0x91c, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20, 0x92f, 0x941, 0x930,
0x2019, 0x92a, 0x928, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x928, 0x91c, 0x93e,
0x92f, 0x93e, 0x930, 0x93f, 0x20, 0x92f, 0x941, 0x930, 0x2019, 0x92a, 0x928, 0x93f,
0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938,
0x93e, 0x928, 0x91c, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20, 0x92f, 0x941, 0x930,
0x2019, 0x92a, 0x928, 0x93f, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902,
0x20, 0x938, 0x92e, 0x917, 0x94b, 0x91c, 0x93e, 0x928, 0x938, 0x93f, 0x928, 0x2d,
0x938, 0x93e, 0x928, 0x91c, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20, 0x92f, 0x941,
0x930, 0x2019, 0x92a, 0x928, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x94b, 0x928, 0x93e,
0x92c, 0x20, 0x92f, 0x941, 0x930, 0x2019, 0x92a, 0x928, 0x93f, 0x20, 0x938, 0x92e,
0x938, 0x94b, 0x928, 0x93e, 0x92c, 0x20, 0x92f, 0x941, 0x930, 0x2019, 0x92a, 0x928,
0x93f, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x938, 0x94b, 0x928, 0x93e, 0x92c, 0x20, 0x92f, 0x941, 0x930, 0x2019, 0x92a, 0x928,
0x93f, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e,
0x92b, 0x915, 0x932, 0x947, 0x923, 0x94d, 0x921, 0x20, 0x926, 0x94d, 0x935, 0x940,
0x92a, 0x92b, 0x94b, 0x930, 0x928, 0x93f, 0x20, 0x938, 0x92e, 0x92b, 0x915, 0x932,
0x947, 0x923, 0x94d, 0x921, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x92b, 0x94b,
0x930, 0x928, 0x93f, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x92b, 0x915, 0x932, 0x947, 0x923, 0x94d, 0x921, 0x20, 0x926, 0x94d,
0x935, 0x940, 0x92a, 0x92b, 0x94b, 0x930, 0x928, 0x93f, 0x20, 0x926, 0x948, 0x91c,
0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92b, 0x940, 0x91c, 0x940, 0x20,
0x938, 0x92e, 0x92b, 0x940, 0x91c, 0x940, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906,
0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92b, 0x940, 0x91c, 0x940, 0x20, 0x926, 0x948,
0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92b, 0x94d, 0x930, 0x947,
0x928, 0x94d, 0x91a, 0x20, 0x917, 0x941, 0x908, 0x92f, 0x93e, 0x928, 0x93e, 0x20,
0x938, 0x92e, 0x92b, 0x94d, 0x930, 0x947, 0x928, 0x94d, 0x91a, 0x20, 0x916, 0x94b,
0x932, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20, 0x906, 0x930, 0x94b, 0x20, 0x90f,
0x928, 0x94d, 0x91f, 0x93e, 0x930, 0x94d, 0x915, 0x91f, 0x93f, 0x915, 0x20, 0x938,
0x92e, 0x917, 0x93e, 0x932, 0x93e, 0x92a, 0x93e, 0x917, 0x2019, 0x938, 0x20, 0x938,
0x92e, 0x917, 0x93e, 0x92e, 0x94d, 0x92c, 0x93f, 0x92f, 0x947, 0x930, 0x20, 0x938,
0x92e, 0x91c, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x91c,
0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906,
0x930, 0x93f, 0x20, 0x938, 0x92e, 0x91c, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e,
0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x917,
0x940, 0x932, 0x92c, 0x93e, 0x930, 0x94d, 0x91f, 0x20, 0x926, 0x94d, 0x935, 0x940,
0x92a, 0x92b, 0x94b, 0x930, 0x928, 0x93f, 0x20, 0x938, 0x92e, 0x917, 0x94d, 0x930,
0x93f, 0x928, 0x935, 0x940, 0x91a, 0x20, 0x917, 0x947, 0x91c, 0x947, 0x930, 0x93e,
0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x928, 0x91c, 0x93e, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x928, 0x932, 0x947, 0x923, 0x94d, 0x921, 0x20, 0x938, 0x92e,
0x938, 0x93e, 0x928, 0x91c, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932,
0x947, 0x923, 0x94d, 0x921, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f,
0x20, 0x938, 0x92e, 0x938, 0x93e, 0x928, 0x921, 0x91c, 0x93e, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x928, 0x932, 0x947, 0x923, 0x94d, 0x921, 0x20, 0x926, 0x948, 0x91c,
0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x938, 0x94b, 0x928, 0x93e, 0x92c,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x947, 0x923, 0x94d, 0x921, 0x20,
0x938, 0x92e, 0x938, 0x94b, 0x928, 0x93e, 0x92c, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x928, 0x932, 0x947, 0x923, 0x94d, 0x921, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906,
0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x947, 0x928, 0x93e, 0x92c, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x928, 0x932, 0x947, 0x923, 0x94d, 0x921, 0x20, 0x926, 0x948,
0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x917, 0x941, 0x906, 0x92e,
0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f,
0x93e, 0x908, 0x92e, 0x917, 0x93e, 0x932, 0x94d, 0x92b, 0x20, 0x925, 0x93e, 0x916,
0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x917, 0x941, 0x92f, 0x93e, 0x928,
0x93e, 0x20, 0x938, 0x92e, 0x939, 0x93e, 0x935, 0x93e, 0x908, 0x2d, 0x90f, 0x932,
0x94d, 0x92f, 0x941, 0x924, 0x93f, 0x92f, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x939,
0x93e, 0x935, 0x93e, 0x908, 0x2d, 0x90f, 0x932, 0x94d, 0x92f, 0x941, 0x924, 0x93f,
0x92f, 0x93e, 0x928, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x939, 0x93e, 0x935, 0x93e, 0x908, 0x2d, 0x90f, 0x932, 0x94d, 0x92f,
0x941, 0x924, 0x93f, 0x92f, 0x93e, 0x928, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930,
0x93f, 0x20, 0x938, 0x92e, 0x939, 0x902, 0x915, 0x902, 0x20, 0x938, 0x92e, 0x939,
0x902, 0x915, 0x902, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x939, 0x902, 0x915, 0x902, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932,
0x93e, 0x902, 0x20, 0x938, 0x92e, 0x939, 0x2019, 0x92d, 0x94d, 0x926, 0x20, 0x938,
0x92e, 0x939, 0x2019, 0x92d, 0x94d, 0x926, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906,
0x930, 0x93f, 0x20, 0x938, 0x92e, 0x939, 0x2019, 0x92d, 0x94d, 0x926, 0x20, 0x926,
0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92d, 0x93e, 0x930,
0x924, 0x93e, 0x930, 0x93f, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f,
0x20, 0x938, 0x92e, 0x92d, 0x93e, 0x930, 0x924, 0x20, 0x932, 0x948, 0x925, 0x94b,
0x92e, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x908, 0x902, 0x921,
0x94b, 0x20, 0x91a, 0x907, 0x928, 0x93e, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902,
0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x908, 0x902, 0x921,
0x94b, 0x928, 0x940, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x94d, 0x91f, 0x948,
0x902, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x908, 0x938,
0x94d, 0x91f, 0x930, 0x94d, 0x928, 0x20, 0x908, 0x902, 0x921, 0x94b, 0x928, 0x940,
0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930,
0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x935, 0x947, 0x938, 0x94d, 0x91f,
0x930, 0x94d, 0x928, 0x20, 0x908, 0x902, 0x921, 0x94b, 0x928, 0x940, 0x936, 0x93f,
0x92f, 0x93e, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d, 0x921,
0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x907, 0x930, 0x93e, 0x928, 0x20, 0x938, 0x92e,
0x907, 0x930, 0x93e, 0x928, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f,
0x20, 0x938, 0x92e, 0x907, 0x930, 0x93e, 0x928, 0x20, 0x938, 0x93e, 0x928, 0x93e,
0x930, 0x93f, 0x20, 0x938, 0x92e, 0x908, 0x930, 0x94d, 0x915, 0x941, 0x924, 0x94d,
0x938, 0x94d, 0x915, 0x20, 0x938, 0x92e, 0x908, 0x930, 0x94d, 0x915, 0x941, 0x924,
0x94d, 0x938, 0x94d, 0x915, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f,
0x20, 0x938, 0x92e, 0x908, 0x930, 0x94d, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d,
0x915, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x902, 0x20, 0x938,
0x92e, 0x907, 0x91c, 0x94d, 0x930, 0x93e, 0x908, 0x932, 0x20, 0x938, 0x92e, 0x907,
0x91c, 0x94d, 0x930, 0x93e, 0x908, 0x932, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906,
0x930, 0x93f, 0x20, 0x938, 0x92e, 0x907, 0x91c, 0x94d, 0x930, 0x93e, 0x908, 0x932,
0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x91c, 0x93e,
0x92a, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x91c, 0x93e, 0x92a, 0x93e, 0x928, 0x20,
0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x91c, 0x93e,
0x92a, 0x93e, 0x928, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938,
0x92e, 0x92a, 0x947, 0x924, 0x94d, 0x930, 0x2019, 0x92a, 0x93e, 0x92d, 0x94d, 0x932,
0x2019, 0x92d, 0x94d, 0x938, 0x94d, 0x915, 0x2d, 0x915, 0x93e, 0x92e, 0x91a, 0x93e,
0x924, 0x94d, 0x938, 0x94d, 0x915, 0x93f, 0x20, 0x938, 0x92e, 0x92a, 0x947, 0x924,
0x94d, 0x930, 0x94b, 0x92a, 0x93e, 0x935, 0x932, 0x94b, 0x938, 0x94d, 0x915, 0x20,
0x915, 0x93e, 0x92e, 0x91a, 0x91f, 0x915, 0x93e, 0x20, 0x938, 0x94d, 0x91f, 0x948,
0x902, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x92a, 0x947,
0x924, 0x94d, 0x930, 0x94b, 0x92a, 0x93e, 0x935, 0x932, 0x94b, 0x938, 0x94d, 0x915,
0x20, 0x915, 0x93e, 0x92e, 0x91a, 0x91f, 0x915, 0x93e, 0x20, 0x938, 0x92e, 0x930,
0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x938, 0x93e, 0x928, 0x91c, 0x93e, 0x20, 0x915,
0x93e, 0x91c, 0x93e, 0x916, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e,
0x938, 0x94b, 0x928, 0x93e, 0x92c, 0x20, 0x915, 0x93e, 0x91c, 0x93e, 0x916, 0x938,
0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x915, 0x2019, 0x930, 0x93f, 0x92f,
0x93e, 0x20, 0x938, 0x92e, 0x915, 0x2019, 0x930, 0x93f, 0x92f, 0x93e, 0x20, 0x925,
0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x915, 0x2019, 0x930,
0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938,
0x92e, 0x915, 0x938, 0x94d, 0x930, 0x947, 0x20, 0x938, 0x92e, 0x915, 0x94d, 0x930,
0x93e, 0x938, 0x94d, 0x928, 0x2019, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915,
0x20, 0x938, 0x92e, 0x915, 0x94d, 0x930, 0x93e, 0x938, 0x94d, 0x928, 0x2019, 0x92f,
0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906,
0x930, 0x93f, 0x20, 0x938, 0x92e, 0x915, 0x94d, 0x930, 0x93e, 0x938, 0x94d, 0x928,
0x2019, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x926, 0x948, 0x91c,
0x94d, 0x932, 0x93e, 0x902, 0x902, 0x20, 0x938, 0x92e, 0x915, 0x940, 0x930, 0x94d,
0x917, 0x940, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x932, 0x902,
0x915, 0x93e, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d, 0x921,
0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x932, 0x93e, 0x908, 0x928, 0x20, 0x926, 0x94d,
0x935, 0x940, 0x92a, 0x92b, 0x94b, 0x930, 0x928, 0x93f, 0x20, 0x938, 0x92e, 0x932,
0x930, 0x94d, 0x921, 0x20, 0x939, 0x93e, 0x935, 0x93f, 0x20, 0x938, 0x92e, 0x932,
0x930, 0x94d, 0x921, 0x20, 0x939, 0x93e, 0x935, 0x93f, 0x20, 0x925, 0x93e, 0x916,
0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x932, 0x930, 0x94d, 0x921, 0x20,
0x939, 0x93e, 0x935, 0x93f, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x92e, 0x93e, 0x915, 0x93e, 0x90a, 0x20, 0x938, 0x92e, 0x92e, 0x93e,
0x915, 0x93e, 0x90a, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d,
0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x92e, 0x93e, 0x915, 0x93e, 0x90a, 0x20,
0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x92e, 0x93e, 0x917, 0x93e,
0x926, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92e, 0x93e, 0x917, 0x93e, 0x926, 0x93e,
0x928, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x92e, 0x93e, 0x917, 0x93e, 0x926, 0x93e, 0x928, 0x20, 0x926, 0x948, 0x91c, 0x94d,
0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92e, 0x932, 0x947, 0x936, 0x93f, 0x92f,
0x93e, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d, 0x921, 0x20,
0x91f, 0x93e, 0x908, 0x92e, 0x92e, 0x93e, 0x932, 0x926, 0x940, 0x92d, 0x94d, 0x938,
0x20, 0x938, 0x92e, 0x92e, 0x93e, 0x930, 0x94d, 0x915, 0x947, 0x938, 0x93e, 0x938,
0x20, 0x938, 0x92e, 0x92e, 0x93e, 0x930, 0x94d, 0x936, 0x947, 0x932, 0x20, 0x926,
0x94d, 0x935, 0x940, 0x92a, 0x92b, 0x94b, 0x930, 0x928, 0x93f, 0x20, 0x938, 0x92e,
0x92e, 0x2019, 0x930, 0x93f, 0x936, 0x93f, 0x92f, 0x93e, 0x938, 0x928, 0x93f, 0x20,
0x938, 0x92e, 0x92e, 0x2019, 0x930, 0x93f, 0x936, 0x93f, 0x92f, 0x93e, 0x938, 0x928,
0x93f, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x92e, 0x2019, 0x930, 0x93f, 0x936, 0x93f, 0x92f, 0x93e, 0x938, 0x928, 0x93f, 0x20,
0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92e, 0x94c,
0x938, 0x928, 0x20, 0x938, 0x92e, 0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915,
0x2019, 0x928, 0x93f, 0x20, 0x92a, 0x947, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20,
0x938, 0x92e, 0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x2019, 0x928, 0x93f,
0x20, 0x92a, 0x947, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x925, 0x93e, 0x916,
0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92e, 0x947, 0x915, 0x94d, 0x938,
0x93f, 0x915, 0x2019, 0x928, 0x93f, 0x20, 0x92a, 0x947, 0x938, 0x93f, 0x92b, 0x93f,
0x915, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x909,
0x932, 0x93e, 0x928, 0x92c, 0x93e, 0x924, 0x93e, 0x930, 0x20, 0x938, 0x92e, 0x909,
0x932, 0x93e, 0x928, 0x92c, 0x93e, 0x924, 0x93e, 0x930, 0x20, 0x925, 0x93e, 0x916,
0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x909, 0x932, 0x93e, 0x928, 0x92c,
0x93e, 0x924, 0x93e, 0x930, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902,
0x20, 0x938, 0x92e, 0x92e, 0x938, 0x94d, 0x915, 0x94c, 0x20, 0x938, 0x92e, 0x92e,
0x938, 0x94d, 0x915, 0x94c, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f,
0x20, 0x938, 0x92e, 0x92e, 0x938, 0x94d, 0x915, 0x94c, 0x20, 0x926, 0x948, 0x91c,
0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92e, 0x94d, 0x92f, 0x93e, 0x928,
0x92e, 0x93e, 0x930, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d,
0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x928, 0x93e, 0x90a, 0x930, 0x941, 0x20,
0x938, 0x92e, 0x928, 0x947, 0x92a, 0x93e, 0x932, 0x20, 0x938, 0x92e, 0x917, 0x94b,
0x926, 0x93e, 0x928, 0x20, 0x915, 0x947, 0x932, 0x947, 0x921, 0x928, 0x93f, 0x92f,
0x93e, 0x20, 0x938, 0x92e, 0x917, 0x94b, 0x926, 0x93e, 0x928, 0x20, 0x915, 0x947,
0x932, 0x947, 0x921, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x925, 0x93e, 0x916, 0x94b,
0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x917, 0x94b, 0x926, 0x93e, 0x928, 0x20,
0x915, 0x947, 0x932, 0x947, 0x921, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x948,
0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x928, 0x93f, 0x909, 0x91c,
0x93f, 0x932, 0x947, 0x923, 0x94d, 0x921, 0x20, 0x938, 0x92e, 0x928, 0x93f, 0x909,
0x91c, 0x93f, 0x932, 0x947, 0x923, 0x94d, 0x921, 0x20, 0x925, 0x93e, 0x916, 0x94b,
0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x928, 0x93f, 0x909, 0x91c, 0x93f, 0x932,
0x947, 0x923, 0x94d, 0x921, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x928, 0x93f, 0x909, 0x92b, 0x93e, 0x909, 0x928, 0x94d, 0x926, 0x932,
0x947, 0x928, 0x94d, 0x926, 0x20, 0x938, 0x92e, 0x928, 0x93f, 0x909, 0x92b, 0x93e,
0x909, 0x928, 0x94d, 0x926, 0x932, 0x947, 0x928, 0x94d, 0x926, 0x20, 0x925, 0x93e,
0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x928, 0x93f, 0x909, 0x92b,
0x93e, 0x909, 0x928, 0x94d, 0x926, 0x932, 0x947, 0x928, 0x94d, 0x926, 0x20, 0x938,
0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x928, 0x940, 0x90a, 0x90f,
0x20, 0x938, 0x92e, 0x928, 0x930, 0x92b, 0x2019, 0x915, 0x20, 0x926, 0x94d, 0x935,
0x940, 0x92a, 0x20, 0x938, 0x92e, 0x928, 0x930, 0x92b, 0x2019, 0x915, 0x20, 0x926,
0x94d, 0x935, 0x940, 0x92a, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f,
0x20, 0x938, 0x92e, 0x928, 0x930, 0x92b, 0x2019, 0x915, 0x20, 0x926, 0x94d, 0x935,
0x940, 0x92a, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x92b, 0x947, 0x930, 0x928, 0x93e, 0x928, 0x94d, 0x926, 0x2019, 0x20, 0x926, 0x947,
0x20, 0x928, 0x930, 0x2019, 0x928, 0x94d, 0x939, 0x93e, 0x20, 0x938, 0x92e, 0x92b,
0x947, 0x930, 0x928, 0x93e, 0x928, 0x94d, 0x926, 0x2019, 0x20, 0x926, 0x947, 0x20,
0x928, 0x930, 0x2019, 0x928, 0x94d, 0x939, 0x93e, 0x20, 0x925, 0x93e, 0x916, 0x94b,
0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92b, 0x947, 0x930, 0x928, 0x93e, 0x928,
0x94d, 0x926, 0x2019, 0x20, 0x926, 0x947, 0x20, 0x928, 0x930, 0x2019, 0x928, 0x94d,
0x939, 0x93e, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938,
0x92e, 0x928, 0x949, 0x930, 0x94d, 0x925, 0x20, 0x92e, 0x93e, 0x930, 0x93f, 0x906,
0x928, 0x93e, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d, 0x921,
0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x928, 0x92d, 0x2019, 0x938, 0x93f, 0x92c, 0x93f,
0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x938, 0x92e, 0x928, 0x92d, 0x2019, 0x938,
0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x925, 0x93e, 0x916,
0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x928, 0x92d, 0x2019, 0x938, 0x93f,
0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x926, 0x948, 0x91c, 0x94d,
0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x905, 0x92e, 0x94d, 0x938, 0x94d, 0x915,
0x20, 0x938, 0x92e, 0x905, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x925, 0x93e,
0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x905, 0x92e, 0x94d, 0x938,
0x94d, 0x915, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938,
0x92e, 0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938,
0x92e, 0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x925,
0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92a, 0x93e, 0x915,
0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932,
0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92a, 0x93e, 0x932, 0x93e, 0x909, 0x20, 0x938,
0x92e, 0x92a, 0x93e, 0x92a, 0x941, 0x906, 0x20, 0x928, 0x93f, 0x909, 0x20, 0x917,
0x93f, 0x928, 0x940, 0x20, 0x938, 0x92e, 0x92a, 0x93e, 0x930, 0x93e, 0x917, 0x941,
0x935, 0x947, 0x20, 0x938, 0x92e, 0x92a, 0x93e, 0x930, 0x93e, 0x917, 0x941, 0x935,
0x947, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x92a, 0x93e, 0x930, 0x93e, 0x917, 0x941, 0x935, 0x947, 0x20, 0x926, 0x948, 0x91c,
0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92a, 0x947, 0x930, 0x941, 0x20,
0x938, 0x92e, 0x92a, 0x947, 0x930, 0x941, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906,
0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92a, 0x947, 0x930, 0x941, 0x20, 0x926, 0x948,
0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92b, 0x93f, 0x932, 0x940,
0x92a, 0x940, 0x928, 0x94d, 0x938, 0x20, 0x938, 0x92e, 0x92b, 0x940, 0x932, 0x940,
0x92a, 0x940, 0x928, 0x94d, 0x938, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921,
0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x92b, 0x940, 0x932, 0x940,
0x92a, 0x940, 0x928, 0x94d, 0x938, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e,
0x908, 0x92e, 0x92b, 0x928, 0x940, 0x915, 0x94d, 0x938, 0x20, 0x926, 0x94d, 0x935,
0x940, 0x92a, 0x92b, 0x94b, 0x930, 0x928, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x948,
0x928, 0x94d, 0x91f, 0x20, 0x92a, 0x93f, 0x92f, 0x947, 0x930, 0x20, 0x906, 0x930,
0x94b, 0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932, 0x928, 0x20, 0x938,
0x92e, 0x938, 0x948, 0x928, 0x94d, 0x91f, 0x20, 0x92a, 0x93f, 0x92f, 0x947, 0x930,
0x20, 0x906, 0x930, 0x94b, 0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932,
0x928, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x938, 0x948, 0x928, 0x94d, 0x91f, 0x20, 0x92a, 0x93f, 0x92f, 0x947, 0x930, 0x20,
0x906, 0x930, 0x94b, 0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932, 0x928,
0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92a, 0x940,
0x91f, 0x915, 0x948, 0x930, 0x94d, 0x928, 0x20, 0x938, 0x92e, 0x92a, 0x928, 0x93e,
0x92a, 0x947, 0x20, 0x938, 0x92e, 0x92a, 0x94d, 0x92f, 0x902, 0x92f, 0x93e, 0x902,
0x20, 0x938, 0x92e, 0x915, 0x93c, 0x940, 0x91c, 0x93c, 0x940, 0x932, 0x94b, 0x930,
0x94d, 0x921, 0x93e, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x915, 0x93c, 0x940, 0x91c,
0x93c, 0x940, 0x932, 0x94b, 0x930, 0x94d, 0x921, 0x93e, 0x20, 0x938, 0x94d, 0x91f,
0x948, 0x902, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x915,
0x93c, 0x940, 0x91c, 0x93c, 0x940, 0x932, 0x94b, 0x930, 0x94d, 0x921, 0x93e, 0x20,
0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x930, 0x93f, 0x92f, 0x942,
0x928, 0x93f, 0x92f, 0x928, 0x20, 0x938, 0x92e, 0x930, 0x925, 0x947, 0x930, 0x93e,
0x20, 0x938, 0x92e, 0x938, 0x93e, 0x916, 0x93e, 0x932, 0x93f, 0x928, 0x20, 0x938,
0x92e, 0x938, 0x93e, 0x916, 0x93e, 0x932, 0x93f, 0x928, 0x20, 0x925, 0x93e, 0x916,
0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x916, 0x93e, 0x932,
0x93f, 0x928, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938,
0x92e, 0x938, 0x92e, 0x93e, 0x930, 0x93e, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x938,
0x92e, 0x93e, 0x930, 0x93e, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930,
0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x938, 0x92e, 0x93e, 0x930, 0x93e,
0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x938, 0x93e, 0x92e,
0x2019, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x92e, 0x2019, 0x935, 0x93e,
0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938,
0x93e, 0x92e, 0x2019, 0x935, 0x93e, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f,
0x20, 0x938, 0x92e, 0x938, 0x947, 0x936, 0x947, 0x932, 0x94d, 0x938, 0x20, 0x938,
0x92e, 0x938, 0x940, 0x902, 0x917, 0x93e, 0x92a, 0x941, 0x930, 0x20, 0x938, 0x94d,
0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e,
0x938, 0x932, 0x2019, 0x92e, 0x928, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x92b,
0x94b, 0x930, 0x928, 0x93f, 0x20, 0x938, 0x92e, 0x916, 0x94b, 0x932, 0x93e, 0x20,
0x91c, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x938, 0x941,
0x930, 0x940, 0x928, 0x93e, 0x92e, 0x20, 0x938, 0x92e, 0x938, 0x940, 0x905, 0x935,
0x93e, 0x20, 0x938, 0x92e, 0x91f, 0x93e, 0x939, 0x93f, 0x91f, 0x940, 0x20, 0x938,
0x92e, 0x924, 0x93e, 0x908, 0x92a, 0x947, 0x907, 0x20, 0x938, 0x92e, 0x924, 0x93e,
0x908, 0x92a, 0x947, 0x907, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f,
0x20, 0x938, 0x92e, 0x924, 0x93e, 0x908, 0x92a, 0x947, 0x907, 0x20, 0x938, 0x93e,
0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x924, 0x93e, 0x91c, 0x93f, 0x915,
0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x91f, 0x915, 0x947,
0x932, 0x94c, 0x20, 0x938, 0x92e, 0x91f, 0x902, 0x917, 0x93e, 0x20, 0x938, 0x92e,
0x91f, 0x902, 0x917, 0x93e, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f,
0x20, 0x938, 0x92e, 0x91f, 0x902, 0x917, 0x93e, 0x20, 0x926, 0x948, 0x91c, 0x94d,
0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x91a, 0x942, 0x915, 0x20, 0x938, 0x92e,
0x924, 0x941, 0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928, 0x940, 0x938, 0x94d, 0x924,
0x93e, 0x928, 0x20, 0x938, 0x92e, 0x924, 0x941, 0x930, 0x94d, 0x915, 0x92e, 0x947,
0x928, 0x940, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x925, 0x93e, 0x916, 0x94b,
0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x924, 0x941, 0x930, 0x94d, 0x915, 0x92e,
0x947, 0x928, 0x940, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x926, 0x948, 0x91c,
0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x924, 0x941, 0x92d, 0x93e, 0x932,
0x941, 0x20, 0x938, 0x92e, 0x909, 0x930, 0x941, 0x917, 0x941, 0x935, 0x947, 0x20,
0x938, 0x92e, 0x909, 0x930, 0x941, 0x917, 0x941, 0x935, 0x947, 0x20, 0x925, 0x93e,
0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x909, 0x930, 0x941, 0x917,
0x941, 0x935, 0x947, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20,
0x938, 0x92e, 0x909, 0x91c, 0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e,
0x928, 0x20, 0x938, 0x92e, 0x909, 0x91c, 0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d,
0x924, 0x93e, 0x928, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x909, 0x91c, 0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e,
0x928, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e,
0x935, 0x93e, 0x928, 0x941, 0x906, 0x91f, 0x941, 0x20, 0x938, 0x92e, 0x935, 0x93e,
0x928, 0x941, 0x906, 0x91f, 0x941, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930,
0x93f, 0x20, 0x938, 0x92e, 0x935, 0x93e, 0x928, 0x941, 0x906, 0x91f, 0x941, 0x20,
0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92d, 0x947,
0x928, 0x947, 0x91c, 0x941, 0x935, 0x947, 0x932, 0x93e, 0x20, 0x938, 0x92e, 0x92d,
0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x92d, 0x938, 0x94d, 0x924, 0x2019, 0x915, 0x20,
0x938, 0x92e, 0x92d, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x92d, 0x938, 0x94d, 0x924,
0x2019, 0x915, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938,
0x92e, 0x92d, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x92d, 0x938, 0x94d, 0x924, 0x2019,
0x915, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e,
0x92d, 0x932, 0x94d, 0x917, 0x2019, 0x917, 0x94d, 0x930, 0x947, 0x926, 0x20, 0x938,
0x92e, 0x92d, 0x932, 0x94d, 0x917, 0x2019, 0x917, 0x94d, 0x930, 0x947, 0x926, 0x20,
0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92d, 0x932,
0x94d, 0x917, 0x2019, 0x917, 0x94d, 0x930, 0x947, 0x926, 0x20, 0x926, 0x948, 0x91c,
0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x935, 0x948, 0x915, 0x20, 0x926,
0x94d, 0x935, 0x940, 0x92a, 0x20, 0x938, 0x92e, 0x935, 0x93e, 0x932, 0x940, 0x938,
0x20, 0x906, 0x930, 0x94b, 0x20, 0x92b, 0x941, 0x924, 0x941, 0x928, 0x93e, 0x20,
0x938, 0x92e, 0x92f, 0x93e, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x93e, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20,
0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92f, 0x93e,
0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x926, 0x948, 0x91c, 0x94d,
0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92f, 0x947, 0x915, 0x93e, 0x924, 0x947,
0x930, 0x940, 0x928, 0x93e, 0x92c, 0x941, 0x930, 0x94d, 0x917, 0x20, 0x938, 0x92e,
0x92f, 0x947, 0x915, 0x93e, 0x924, 0x947, 0x930, 0x940, 0x928, 0x93e, 0x92c, 0x941,
0x930, 0x94d, 0x917, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x92f, 0x947, 0x915, 0x93e, 0x924, 0x947, 0x930, 0x940, 0x928, 0x93e,
0x92c, 0x941, 0x930, 0x94d, 0x917, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e,
0x902, 0x20, 0x938, 0x92e, 0x92f, 0x942, 0x915, 0x928, 0x20, 0x938, 0x92e, 0x41,
0x63, 0x72, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x63, 0x72,
0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x63, 0x72, 0x65, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x72, 0x61, 0x10d, 0x75, 0x6e, 0x61,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x6e, 0x61, 0x41,
0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f,
0x10d, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x75, 0x17e, 0x6e, 0x6f, 0x61,
0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69,
0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a,
0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61,
0x64, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x41, 0x6c, 0x6a, 0x61, 0x73, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x6c, 0x6a, 0x61,
0x73, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x41, 0x6c, 0x6a, 0x61, 0x73, 0x6b, 0x61, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x75, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x75, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x75, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x72, 0x61, 0x10d, 0x75, 0x6e, 0x61, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x6e, 0x61, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76, 0x65,
0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e,
0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76, 0x65,
0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20,
0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x69, 0x73, 0x74, 0x6f,
0x10d, 0x6e, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x6a,
0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d,
0x6b, 0x6f, 0x20, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x6c, 0x61, 0x6e, 0x69, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53,
0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x6c, 0x61, 0x6e, 0x69, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76,
0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f,
0x20, 0x70, 0x6c, 0x61, 0x6e, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69,
0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53,
0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x10d, 0x6b,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76,
0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f,
0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x72, 0x61, 0x10d, 0x75, 0x6e, 0x61, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x6e, 0x61, 0x41, 0x70, 0x69, 0x6a, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x70,
0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x41, 0x70, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x6b,
0x76, 0x74, 0x61, 0x75, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6b,
0x76, 0x74, 0x61, 0x75, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6b, 0x76,
0x74, 0x61, 0x75, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x72,
0x61, 0x10d, 0x75, 0x6e, 0x61, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x6e, 0x61, 0x41, 0x6b, 0x76, 0x74, 0x6f, 0x62, 0x65, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6b, 0x76, 0x74, 0x6f, 0x62, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6b, 0x76, 0x74, 0x6f, 0x62, 0x65,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x72, 0x61, 0x10d, 0x75,
0x6e, 0x61, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x6e,
0x61, 0x41, 0x72, 0x61, 0x62, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x72, 0x61, 0x62, 0x69, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x72,
0x61, 0x62, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x61, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x61, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x61,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76, 0x65,
0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20,
0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e,
0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x61, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x6e, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e,
0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x6e, 0x6f, 0x20, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x6e, 0x6f, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x6e, 0x6f, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f,
0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d,
0x6e, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f,
0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61,
0x64, 0x6e, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61,
0x70, 0x61, 0x64, 0x6e, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x65, 0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65,
0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65, 0x6a, 0x64, 0x17e,
0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x7a, 0x6f, 0x72,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41,
0x7a, 0x6f, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x41, 0x7a, 0x6f, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x161, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x42, 0x75, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69,
0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42, 0x72, 0x61, 0x7a, 0x69,
0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x6a, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x65, 0x6c, 0x65,
0x6e, 0x6f, 0x72, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x5a, 0x65, 0x6c, 0x65, 0x6e, 0x6f, 0x72, 0x74, 0x73,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x65, 0x6c,
0x65, 0x6e, 0x6f, 0x72, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10c,
0x61, 0x6d, 0x6f, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x10c, 0x61, 0x74, 0x61, 0x6d, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10c, 0x61, 0x74, 0x61, 0x6d, 0x73,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10c, 0x61, 0x74,
0x61, 0x6d, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10c, 0x69, 0x6c, 0x65,
0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x10c, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10c, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x73,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x69, 0x6e, 0x65, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x69, 0x6e, 0x65,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x69,
0x6e, 0x65, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x42, 0x6f, 0x17e, 0x69, 0x107,
0x6e, 0x6f, 0x6d, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x75, 0x56, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74,
0x72, 0x76, 0x69, 0x6d, 0x61, 0x20, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x4b,
0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62,
0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61,
0x20, 0x4b, 0x75, 0x6b, 0x6f, 0x76, 0x69, 0x6d, 0x20, 0x6f, 0x73, 0x74,
0x72, 0x76, 0x69, 0x6d, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20,
0x6e, 0x61, 0x20, 0x4b, 0x75, 0x6b, 0x6f, 0x76, 0x69, 0x6d, 0x20, 0x6f,
0x73, 0x74, 0x72, 0x76, 0x69, 0x6d, 0x61, 0x50, 0x6f, 0x6c, 0x75, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x20, 0x6e, 0x61, 0x20, 0x4b, 0x75, 0x6b, 0x6f, 0x76, 0x69, 0x6d,
0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x69, 0x6d, 0x61, 0x4b, 0x75, 0x62,
0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x4b, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x63, 0x65, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73,
0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x63, 0x65, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64,
0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x49, 0x73, 0x74, 0x6f,
0x10d, 0x6e, 0x6f, 0x74, 0x69, 0x6d, 0x6f, 0x72, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x73, 0x6b, 0x72, 0x161,
0x6e, 0x6a, 0x65, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x73, 0x6b, 0x72, 0x161,
0x6e, 0x6a, 0x65, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x73, 0x6b, 0x72, 0x161, 0x6e,
0x6a, 0x65, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x45, 0x6b, 0x76, 0x61, 0x64, 0x6f, 0x72, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x65,
0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x65, 0x76,
0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f,
0x70, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x44, 0x61, 0x6c, 0x65, 0x6b,
0x6f, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f,
0x70, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a,
0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70,
0x61, 0x64, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46, 0x6f,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x20, 0x6e, 0x61, 0x20, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x6a, 0x75,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x46, 0x69,
0x64, 0x17e, 0x69, 0x6a, 0x75, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x6a, 0x73,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x75, 0x73,
0x6b, 0x6f, 0x67, 0x76, 0x61, 0x6a, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x75,
0x73, 0x6b, 0x6f, 0x6a, 0x20, 0x4a, 0x75, 0x17e, 0x6e, 0x6f, 0x6a, 0x20,
0x54, 0x65, 0x72, 0x69, 0x74, 0x6f, 0x72, 0x69, 0x6a, 0x69, 0x20, 0x69,
0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x6b, 0x75, 0x47,
0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x6a,
0x65, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x47, 0x72,
0x75, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x69, 0x6c,
0x62, 0x65, 0x72, 0x74, 0x6f, 0x76, 0x69, 0x6d, 0x20, 0x6f, 0x73, 0x74,
0x72, 0x76, 0x69, 0x6d, 0x61, 0x47, 0x72, 0x69, 0x6e, 0x69, 0x10d, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74,
0x6f, 0x10d, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49,
0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70,
0x61, 0x64, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a,
0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x47, 0x75, 0x61,
0x6d, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x6c, 0x69, 0x76, 0x73,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x47, 0x76, 0x61,
0x6a, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x2d, 0x61,
0x6c, 0x65, 0x75, 0x107, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b,
0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x107, 0x61, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x48, 0x61, 0x76, 0x61, 0x6a,
0x73, 0x6b, 0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x107, 0x61, 0x6e, 0x73,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e,
0x161, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x48,
0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x161, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x161,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x48, 0x6f, 0x76, 0x64, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x48, 0x6f, 0x76, 0x64,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x48, 0x6f,
0x76, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x6e, 0x64, 0x69,
0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x49, 0x6e,
0x64, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x6d, 0x20, 0x6f, 0x6b, 0x65, 0x61,
0x6e, 0x75, 0x49, 0x6e, 0x64, 0x6f, 0x6b, 0x69, 0x6e, 0x65, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x69, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f,
0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x61, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49,
0x72, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x49, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x7a,
0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x73,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x61, 0x70,
0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x65, 0x74, 0x72,
0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61,
0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76,
0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61,
0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x10d, 0x61,
0x74, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x72, 0x61, 0x10d, 0x75, 0x6e, 0x61, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x6e, 0x61, 0x6b, 0x61, 0x7a, 0x61, 0x68, 0x73, 0x74,
0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x6b, 0x61, 0x7a, 0x61,
0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x6b,
0x61, 0x7a, 0x61, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x6f, 0x72, 0x65, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b,
0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20,
0x4f, 0x73, 0x74, 0x72, 0x76, 0x75, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61,
0x65, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61,
0x72, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6b, 0x69, 0x72, 0x67, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61,
0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x69, 0x6d, 0x61, 0x20, 0x4c, 0x61,
0x6a, 0x6e, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61,
0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x75, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x61, 0x75, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e,
0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x75, 0x20, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x61, 0x75, 0x4c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x73, 0x74, 0x72, 0x76, 0x75, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x61, 0x75, 0x4d, 0x61, 0x6b, 0x61, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x4d, 0x61, 0x6b, 0x61, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4d,
0x61, 0x6b, 0x61, 0x6f, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x72, 0x61, 0x10d, 0x75, 0x6e, 0x61, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x6e, 0x61, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x73,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x6c, 0x65, 0x7a, 0x69, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d,
0x61, 0x6c, 0x64, 0x69, 0x76, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20,
0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x69, 0x6d, 0x61, 0x20,
0x4d, 0x61, 0x72, 0x6b, 0x69, 0x7a, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x20, 0x6e, 0x61, 0x20, 0x4d, 0x61, 0x72, 0x161, 0x61, 0x6c, 0x6f,
0x76, 0x69, 0x6d, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x69, 0x6d, 0x61,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x63, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6a, 0x73, 0x6b, 0x6f,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x63, 0x65, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f,
0x6e, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x61,
0x63, 0x69, 0x66, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x10d, 0x6b, 0x6f, 0x20,
0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x10d, 0x6b, 0x6f,
0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x6c, 0x61,
0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f,
0x72, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x6f, 0x73, 0x6b, 0x6f,
0x76, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x69, 0x6a, 0x61, 0x6e, 0x6d, 0x61, 0x72,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x73,
0x74, 0x72, 0x76, 0x75, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x4e, 0x65,
0x70, 0x61, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f,
0x7a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x7a, 0x65, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x7a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6a, 0x75, 0x66, 0x61, 0x75, 0x6e,
0x64, 0x6c, 0x65, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6a, 0x75, 0x66, 0x61, 0x75, 0x6e, 0x64,
0x6c, 0x65, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4e, 0x6a, 0x75, 0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65,
0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76,
0x75, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x161, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x161, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x161, 0x6b, 0x6f,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e,
0x61, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x75, 0x20, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x6a, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e,
0x61, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x75, 0x20, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x6a, 0x61, 0x4c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x6f, 0x73,
0x74, 0x72, 0x76, 0x75, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x6a, 0x61,
0x53, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x61, 0x20, 0x4d, 0x61, 0x72, 0x69,
0x6a, 0x61, 0x6e, 0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76,
0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x4f, 0x6d, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20,
0x4f, 0x73, 0x74, 0x72, 0x76, 0x75, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75,
0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x50,
0x61, 0x70, 0x75, 0x69, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x6a, 0x20, 0x47,
0x76, 0x69, 0x6e, 0x65, 0x6a, 0x69, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76,
0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x61, 0x72, 0x61, 0x67,
0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x65, 0x72,
0x75, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x50, 0x65, 0x72, 0x75, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x65, 0x72, 0x75, 0x61, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x69, 0x6d,
0x61, 0x20, 0x46, 0x69, 0x6e, 0x69, 0x6b, 0x73, 0x56, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76,
0x69, 0x6d, 0x61, 0x20, 0x53, 0x76, 0x65, 0x74, 0x69, 0x20, 0x50, 0x65,
0x74, 0x61, 0x72, 0x20, 0x69, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f,
0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x73, 0x74, 0x72, 0x76, 0x69, 0x6d, 0x61, 0x20, 0x53, 0x76, 0x65, 0x74,
0x69, 0x20, 0x50, 0x65, 0x74, 0x61, 0x72, 0x20, 0x69, 0x20, 0x4d, 0x69,
0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x4c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x73, 0x74, 0x72, 0x76, 0x69, 0x6d, 0x61, 0x20, 0x53, 0x76, 0x65, 0x74,
0x69, 0x20, 0x50, 0x65, 0x74, 0x61, 0x72, 0x20, 0x69, 0x20, 0x4d, 0x69,
0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x69, 0x6d, 0x61,
0x20, 0x50, 0x69, 0x74, 0x6b, 0x65, 0x72, 0x6e, 0x56, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76,
0x75, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x50, 0x6a, 0x6f, 0x6e,
0x67, 0x6a, 0x61, 0x6e, 0x161, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x4b, 0x69, 0x7a, 0x69, 0x6c, 0x6f, 0x72, 0x64, 0x61,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b, 0x69, 0x7a, 0x69, 0x6c, 0x6f,
0x72, 0x64, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b, 0x69, 0x7a, 0x69,
0x6c, 0x6f, 0x72, 0x64, 0x61, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x72, 0x61, 0x10d, 0x75, 0x6e, 0x61, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x6e, 0x61, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x63, 0x65, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x53,
0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x61,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53,
0x61, 0x6d, 0x61, 0x72, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53,
0x61, 0x6d, 0x61, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x61,
0x6d, 0x61, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x72, 0x61, 0x10d, 0x75, 0x6e, 0x61, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x6e, 0x61, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x53, 0x65, 0x6a, 0x161, 0x65, 0x6c, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x69, 0x6e, 0x67, 0x61,
0x70, 0x75, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20,
0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x73, 0x6b, 0x69, 0x6d, 0x20,
0x6f, 0x73, 0x74, 0x72, 0x76, 0x69, 0x6d, 0x61, 0x4a, 0x75, 0x17e, 0x6e,
0x6f, 0x64, 0x17e, 0x6f, 0x72, 0x64, 0x17e, 0x69, 0x6a, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x75, 0x72, 0x69,
0x6e, 0x61, 0x6d, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x63, 0x65, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x54,
0x61, 0x68, 0x69, 0x107, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x54, 0x61,
0x6a, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x74, 0x61, 0x64, 0x17e, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x73, 0x74, 0x72, 0x76, 0x75, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61,
0x75, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10c, 0x75,
0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x74,
0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x74, 0x75,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x74, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x61, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x72, 0x75,
0x67, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x73, 0x6b,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x72, 0x75, 0x67,
0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x75, 0x7a, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x75, 0x7a, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x75, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x61, 0x6e, 0x73,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x56, 0x65, 0x6e, 0x65, 0x63, 0x75, 0x65, 0x6c, 0x61, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x10d, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x10d, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x10d, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x73,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x63, 0x65, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x56, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74,
0x72, 0x76, 0x75, 0x20, 0x56, 0x65, 0x6a, 0x6b, 0x56, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76,
0x69, 0x6d, 0x61, 0x20, 0x56, 0x61, 0x6c, 0x69, 0x73, 0x20, 0x69, 0x20,
0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x161, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x161,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x161, 0x6b, 0x6f,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x4a, 0x75, 0x6b, 0x6f, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x410, 0x43a, 0x440, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43a, 0x440, 0x435, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x410, 0x43a, 0x440, 0x435, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x43d, 0x430, 0x410, 0x432, 0x433, 0x430, 0x43d, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x426,
0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x2d, 0x430, 0x444, 0x440,
0x438, 0x447, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x2d, 0x430, 0x444, 0x440, 0x438,
0x447, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x408,
0x443, 0x436, 0x43d, 0x43e, 0x2d, 0x430, 0x444, 0x440, 0x438, 0x447, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430,
0x434, 0x43d, 0x43e, 0x2d, 0x430, 0x444, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434,
0x43d, 0x43e, 0x2d, 0x430, 0x444, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e,
0x2d, 0x430, 0x444, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x410, 0x459, 0x430, 0x441, 0x43a,
0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x459, 0x430,
0x441, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x459,
0x430, 0x441, 0x43a, 0x430, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x43b, 0x43c, 0x430, 0x442, 0x438,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43b, 0x43c, 0x430, 0x442, 0x438,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43b, 0x43c, 0x430, 0x442, 0x438, 0x20,
0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x410, 0x43c,
0x430, 0x437, 0x43e, 0x43d, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430,
0x43b, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x426,
0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d,
0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x418,
0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41f, 0x43b,
0x430, 0x43d, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x41f, 0x43b, 0x430, 0x43d, 0x438, 0x43d, 0x441, 0x43a, 0x43e,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41f, 0x43b, 0x430, 0x43d, 0x438,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440,
0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x43d, 0x430, 0x41f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x446, 0x438,
0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x41f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x410, 0x43d, 0x430, 0x434,
0x438, 0x440, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43d, 0x430, 0x434,
0x438, 0x440, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43d, 0x430, 0x434, 0x438,
0x440, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443,
0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430,
0x410, 0x43f, 0x438, 0x458, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x410, 0x43f, 0x438, 0x458, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x410, 0x43f, 0x438, 0x458, 0x430, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x43d, 0x430, 0x410, 0x43a, 0x432, 0x430, 0x442, 0x430, 0x443,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43a, 0x432, 0x430, 0x442, 0x430,
0x443, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43a, 0x432, 0x430, 0x442, 0x430,
0x443, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443,
0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430,
0x410, 0x43a, 0x443, 0x442, 0x43e, 0x431, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x410, 0x43a, 0x443, 0x442, 0x43e, 0x431, 0x435, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x410, 0x43a, 0x443, 0x442, 0x43e, 0x431, 0x435, 0x20, 0x43b, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x410, 0x440, 0x430, 0x431, 0x438,
0x458, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x410, 0x440, 0x430, 0x431, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x430, 0x431, 0x438, 0x458, 0x441, 0x43a,
0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443,
0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430,
0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438,
0x43d, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x433,
0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x43d, 0x430, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430,
0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d,
0x430, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d,
0x430, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x410, 0x440,
0x43c, 0x435, 0x43d, 0x438, 0x458, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x458, 0x430, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x458,
0x430, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443,
0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430,
0x410, 0x442, 0x43b, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x441, 0x43a, 0x43e,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x442, 0x43b, 0x430, 0x43d,
0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430,
0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x43d, 0x430, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441,
0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x443, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d,
0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a,
0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x410, 0x443,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x446,
0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x437, 0x430, 0x43f,
0x430, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e,
0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x437,
0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a,
0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20,
0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x438, 0x441, 0x442, 0x43e, 0x447,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x443,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x438,
0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a,
0x43e, 0x20, 0x438, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x410, 0x443, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x437, 0x430, 0x43f,
0x430, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e,
0x20, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458,
0x441, 0x43a, 0x43e, 0x20, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x410, 0x437,
0x435, 0x440, 0x431, 0x435, 0x458, 0x45f, 0x430, 0x43d, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x410, 0x437, 0x435, 0x440, 0x431, 0x435, 0x458, 0x45f,
0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x437, 0x435,
0x440, 0x431, 0x435, 0x458, 0x45f, 0x430, 0x43d, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x410, 0x437, 0x43e, 0x440, 0x438, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x437, 0x43e, 0x440, 0x438,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x437, 0x43e, 0x440, 0x438,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d,
0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x411,
0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435,
0x448, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x411, 0x430, 0x43d, 0x433,
0x43b, 0x430, 0x434, 0x435, 0x448, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x43d, 0x430, 0x411, 0x443, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x411, 0x43e, 0x43b, 0x438, 0x432, 0x438, 0x458,
0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x430,
0x437, 0x438, 0x43b, 0x438, 0x458, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x458, 0x430, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b,
0x438, 0x458, 0x430, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430,
0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x43d, 0x430, 0x411, 0x440, 0x443, 0x43d, 0x435, 0x458, 0x20, 0x414, 0x430, 0x440,
0x443, 0x441, 0x430, 0x43b, 0x430, 0x43c, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x417, 0x435, 0x43b, 0x435, 0x43d, 0x43e, 0x440, 0x442, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x435, 0x43b,
0x435, 0x43d, 0x43e, 0x440, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x417, 0x435, 0x43b, 0x435, 0x43d, 0x43e, 0x440, 0x442, 0x441,
0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447,
0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d,
0x430, 0x427, 0x430, 0x43c, 0x43e, 0x440, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x427, 0x430, 0x442, 0x430, 0x43c, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x427, 0x430, 0x442, 0x430, 0x43c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x427, 0x430, 0x442, 0x430, 0x43c, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x427, 0x438, 0x43b, 0x435,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x427, 0x438, 0x43b, 0x435,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x427, 0x438, 0x43b, 0x435, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41a, 0x438,
0x43d, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x438,
0x43d, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x438, 0x43d,
0x430, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443,
0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430,
0x411, 0x43e, 0x436, 0x438, 0x45b, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440,
0x432, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x43e,
0x43a, 0x43e, 0x441, 0x20, 0x28, 0x41a, 0x435, 0x43b, 0x438, 0x43d, 0x433, 0x29,
0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x458, 0x430,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x43b, 0x443,
0x43c, 0x431, 0x438, 0x458, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x458, 0x430, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41a, 0x443, 0x43a, 0x43e,
0x432, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x443, 0x43a, 0x43e, 0x432, 0x430, 0x20,
0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x41a, 0x443, 0x43a, 0x43e, 0x432, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440,
0x432, 0x430, 0x20, 0x43f, 0x43e, 0x43b, 0x443, 0x2d, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41a, 0x443, 0x431, 0x430, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x443, 0x431, 0x430, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x443, 0x431, 0x430, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x414, 0x435, 0x458, 0x432,
0x438, 0x441, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x414, 0x438,
0x43c, 0x43e, 0x43d, 0x20, 0x434, 0x2019, 0x423, 0x440, 0x432, 0x438, 0x43b, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x422, 0x438, 0x43c, 0x43e, 0x440,
0x2d, 0x41b, 0x435, 0x441, 0x442, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x423, 0x441, 0x43a, 0x440, 0x448, 0x45a, 0x430, 0x20, 0x43e, 0x441,
0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x423, 0x441, 0x43a, 0x440, 0x448, 0x45a, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440,
0x432, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x423, 0x441, 0x43a,
0x440, 0x448, 0x45a, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x415, 0x43a,
0x432, 0x430, 0x434, 0x43e, 0x440, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x421, 0x440, 0x435, 0x434, 0x45a, 0x435, 0x435, 0x432, 0x440, 0x43e, 0x43f,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421,
0x440, 0x435, 0x434, 0x45a, 0x435, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a,
0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x440, 0x435, 0x434,
0x45a, 0x435, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x459,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x418, 0x441, 0x442,
0x43e, 0x447, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x441, 0x442, 0x43e,
0x447, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d,
0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x414, 0x430, 0x459, 0x435,
0x20, 0x438, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e,
0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417,
0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441,
0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f,
0x430, 0x434, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d,
0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x424,
0x43e, 0x43b, 0x43a, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x430, 0x20, 0x41e,
0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x430,
0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a,
0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x424, 0x438, 0x45f, 0x438, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x424, 0x438, 0x45f, 0x438, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x424, 0x438, 0x45f, 0x438, 0x20, 0x459,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x424, 0x440, 0x430,
0x43d, 0x446, 0x443, 0x441, 0x43a, 0x430, 0x20, 0x413, 0x432, 0x430, 0x458, 0x430,
0x43d, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x424, 0x440,
0x430, 0x43d, 0x446, 0x443, 0x441, 0x43a, 0x43e, 0x20, 0x458, 0x443, 0x436, 0x43d,
0x43e, 0x20, 0x438, 0x20, 0x430, 0x43d, 0x442, 0x430, 0x440, 0x43a, 0x442, 0x438,
0x447, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x413,
0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x413, 0x430, 0x43c, 0x431, 0x438, 0x458, 0x435, 0x440,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x413, 0x440, 0x443, 0x437,
0x438, 0x458, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x413,
0x440, 0x443, 0x437, 0x438, 0x458, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x413, 0x440, 0x443, 0x437, 0x438, 0x458, 0x430, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x413, 0x438, 0x43b, 0x431, 0x435,
0x440, 0x442, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x413, 0x440, 0x438, 0x43d, 0x438, 0x447, 0x43a,
0x43e, 0x20, 0x441, 0x440, 0x435, 0x434, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x438, 0x20,
0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x438, 0x20,
0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x438, 0x20, 0x413,
0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d,
0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d,
0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x438,
0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x413, 0x443, 0x430, 0x43c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43b, 0x438, 0x432, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x425, 0x430, 0x432,
0x430, 0x458, 0x441, 0x43a, 0x43e, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x425, 0x430,
0x432, 0x430, 0x458, 0x441, 0x43a, 0x43e, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442,
0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x425, 0x430,
0x432, 0x430, 0x458, 0x441, 0x43a, 0x43e, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442,
0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430,
0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x43d, 0x430, 0x425, 0x43e, 0x43d, 0x433, 0x20, 0x41a, 0x43e, 0x43d, 0x433, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x425, 0x43e, 0x43d, 0x433, 0x20,
0x41a, 0x43e, 0x43d, 0x433, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x425,
0x43e, 0x43d, 0x433, 0x43a, 0x43e, 0x43d, 0x448, 0x43a, 0x43e, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x425, 0x43e, 0x432, 0x434,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x425, 0x43e, 0x432, 0x434,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x425, 0x43e, 0x432, 0x434, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x418, 0x43d,
0x434, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x418, 0x43d, 0x434, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x43e, 0x43a,
0x435, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x418, 0x43d, 0x434, 0x43e, 0x43a, 0x438, 0x43d, 0x430, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430,
0x43b, 0x43d, 0x43e, 0x2d, 0x438, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438,
0x458, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x2d, 0x438, 0x43d, 0x434, 0x43e,
0x43d, 0x435, 0x437, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x2d,
0x438, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x458, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x430, 0x43d,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x430, 0x43d,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x430, 0x43d, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x418, 0x440,
0x43a, 0x443, 0x446, 0x43a, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x418, 0x440, 0x43a, 0x443, 0x446, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x418, 0x440, 0x43a, 0x443, 0x446, 0x43a, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x418, 0x437, 0x440, 0x430, 0x435, 0x43b,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418,
0x437, 0x440, 0x430, 0x435, 0x43b, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x418, 0x437, 0x440, 0x430, 0x435, 0x43b, 0x441, 0x43a, 0x43e,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d,
0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x408,
0x430, 0x43f, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x408, 0x430, 0x43f, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x408, 0x430, 0x43f, 0x430, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447,
0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d,
0x430, 0x41f, 0x435, 0x442, 0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432,
0x441, 0x43a, 0x43e, 0x2d, 0x43a, 0x430, 0x43c, 0x447, 0x430, 0x442, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x435, 0x442, 0x440, 0x43e,
0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x43e, 0x2d, 0x43a, 0x430,
0x43c, 0x447, 0x430, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41f, 0x435, 0x442, 0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441,
0x43a, 0x43e, 0x2d, 0x43a, 0x430, 0x43c, 0x447, 0x430, 0x442, 0x441, 0x43a, 0x43e,
0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d,
0x430, 0x45a, 0x435, 0x20, 0x432, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x418, 0x441,
0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x2d, 0x43a, 0x430, 0x437, 0x430, 0x445, 0x441,
0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x2d, 0x43a, 0x430,
0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x440, 0x435, 0x458, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x43e,
0x440, 0x435, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x41a, 0x43e, 0x440, 0x435, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41a, 0x43e, 0x448, 0x440,
0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x440, 0x430,
0x441, 0x43d, 0x43e, 0x458, 0x430, 0x440, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x458,
0x430, 0x440, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a,
0x440, 0x430, 0x441, 0x43d, 0x43e, 0x458, 0x430, 0x440, 0x441, 0x43a, 0x20, 0x459,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41a, 0x438, 0x440,
0x433, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x428, 0x440, 0x438, 0x20, 0x41b, 0x430, 0x43d, 0x43a, 0x430, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41b, 0x438, 0x43d, 0x435, 0x20, 0x41e, 0x441,
0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x432, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x432,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41b, 0x43e, 0x440, 0x434, 0x20,
0x425, 0x43e, 0x432, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430,
0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x43d, 0x430, 0x41c, 0x430, 0x43a, 0x430, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x41c, 0x430, 0x43a, 0x430, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c,
0x430, 0x43a, 0x430, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440,
0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x435, 0x43c, 0x435,
0x43d, 0x430, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x433, 0x430, 0x434,
0x430, 0x43d, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447,
0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x435, 0x43c, 0x435, 0x43d, 0x430,
0x41c, 0x430, 0x43b, 0x435, 0x437, 0x438, 0x458, 0x430, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x43b, 0x434, 0x438, 0x432, 0x438, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x440, 0x43a, 0x438,
0x437, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x440,
0x448, 0x430, 0x43b, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432,
0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x443,
0x440, 0x438, 0x446, 0x438, 0x458, 0x443, 0x441, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x41c, 0x430, 0x443, 0x440, 0x438, 0x446, 0x438, 0x458, 0x443,
0x441, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x443, 0x440,
0x438, 0x446, 0x438, 0x458, 0x443, 0x441, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41c, 0x43e, 0x441, 0x43e, 0x43d, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x447,
0x43a, 0x43e, 0x20, 0x43f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c, 0x435, 0x43a, 0x441,
0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x447,
0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c, 0x435, 0x43a,
0x441, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43f, 0x430, 0x446, 0x438, 0x444, 0x438,
0x447, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430,
0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x43d, 0x430, 0x423, 0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430, 0x442, 0x43e, 0x440,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x423, 0x43b, 0x430, 0x43d,
0x20, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x423, 0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41c, 0x43e,
0x441, 0x43a, 0x432, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41c, 0x438, 0x458, 0x430, 0x43d, 0x43c,
0x430, 0x440, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d, 0x430,
0x443, 0x440, 0x443, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d,
0x435, 0x43f, 0x430, 0x43b, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x41d, 0x43e, 0x432, 0x430, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d,
0x438, 0x458, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d,
0x43e, 0x432, 0x430, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438,
0x458, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432,
0x430, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x458, 0x430,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d,
0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41d,
0x43e, 0x432, 0x438, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x438, 0x20, 0x417,
0x435, 0x43b, 0x430, 0x43d, 0x434, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x41d, 0x43e, 0x432, 0x438, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x40a, 0x443,
0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x40a, 0x443, 0x444, 0x430, 0x443, 0x43d, 0x434,
0x43b, 0x435, 0x43d, 0x434, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x40a,
0x443, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x459,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41d, 0x438, 0x443,
0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x440,
0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x440, 0x444, 0x43e,
0x43b, 0x43a, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x43e, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20,
0x41e, 0x441, 0x442, 0x440, 0x432, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434,
0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x45a, 0x430, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x424, 0x435, 0x440, 0x43d, 0x430,
0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x45a,
0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x424, 0x435, 0x440, 0x43d,
0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e,
0x45a, 0x430, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447,
0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d,
0x430, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x41c, 0x430, 0x440,
0x438, 0x458, 0x430, 0x43d, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440,
0x432, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x43e,
0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438,
0x440, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d, 0x43e,
0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41e, 0x43c, 0x441, 0x43a,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41e, 0x43c, 0x441, 0x43a,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41e, 0x43c, 0x441, 0x43a, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41f, 0x430,
0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430,
0x43d, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443,
0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430,
0x41f, 0x430, 0x43b, 0x430, 0x443, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x41f, 0x430, 0x43f, 0x443, 0x430, 0x20, 0x41d, 0x43e, 0x432, 0x430, 0x20,
0x413, 0x432, 0x438, 0x43d, 0x435, 0x458, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x458, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x440, 0x430, 0x433,
0x432, 0x430, 0x458, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41f, 0x430,
0x440, 0x430, 0x433, 0x432, 0x430, 0x458, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41f, 0x435, 0x440, 0x443, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x41f, 0x435, 0x440, 0x443, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41f, 0x435, 0x440, 0x443, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x424, 0x438, 0x43b, 0x438, 0x43f,
0x438, 0x43d, 0x438, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x424,
0x438, 0x43b, 0x438, 0x43f, 0x438, 0x43d, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x438, 0x43d, 0x438, 0x20, 0x459,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x424, 0x435, 0x43d,
0x438, 0x43a, 0x441, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x43d, 0x20, 0x41f, 0x458,
0x435, 0x440, 0x20, 0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x43d, 0x20,
0x41f, 0x458, 0x435, 0x440, 0x20, 0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b,
0x43e, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x43d,
0x20, 0x41f, 0x458, 0x435, 0x440, 0x20, 0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435,
0x43b, 0x43e, 0x43d, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430,
0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x435, 0x43c, 0x435, 0x43d,
0x430, 0x41f, 0x438, 0x442, 0x43a, 0x435, 0x440, 0x43d, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41f, 0x43e, 0x43d, 0x430, 0x43f, 0x435, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41f, 0x458, 0x43e, 0x43d, 0x433, 0x458,
0x430, 0x43d, 0x433, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a,
0x438, 0x437, 0x438, 0x43b, 0x43e, 0x440, 0x434, 0x430, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41a, 0x438, 0x437, 0x438, 0x43b, 0x43e, 0x440, 0x434, 0x430, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x41a, 0x438, 0x437, 0x438, 0x43b, 0x43e, 0x440, 0x434,
0x430, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443,
0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430,
0x420, 0x435, 0x438, 0x43d, 0x438, 0x43e, 0x43d, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x420, 0x43e, 0x442, 0x435, 0x440, 0x430, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x445, 0x430,
0x43b, 0x438, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x430,
0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x43d, 0x430, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x430, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x430, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x430, 0x20, 0x43b, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x421, 0x430, 0x43c, 0x43e,
0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x43c,
0x43e, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x43c,
0x43e, 0x430, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447,
0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d,
0x430, 0x421, 0x435, 0x458, 0x448, 0x435, 0x43b, 0x438, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x421, 0x438, 0x43d, 0x433, 0x430, 0x43f, 0x443, 0x440,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x43e, 0x43b, 0x43e, 0x43c,
0x43e, 0x43d, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x408, 0x443, 0x436, 0x43d,
0x430, 0x20, 0x40f, 0x43e, 0x440, 0x45f, 0x438, 0x458, 0x430, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x443, 0x440, 0x438, 0x43d, 0x430, 0x43c,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x428, 0x43e, 0x432, 0x430,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x422, 0x430, 0x445, 0x438,
0x442, 0x438, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x422, 0x430,
0x458, 0x43f, 0x435, 0x458, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x422, 0x430, 0x458, 0x43f, 0x435, 0x458, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x422, 0x430, 0x458, 0x43f, 0x435, 0x458, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x422, 0x430, 0x45f, 0x438, 0x43a, 0x438,
0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x422, 0x43e, 0x43a, 0x435, 0x43b, 0x430, 0x443, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x422, 0x440, 0x443, 0x43a,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x422, 0x443, 0x440, 0x43a,
0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438,
0x441, 0x442, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x422,
0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x422, 0x443,
0x432, 0x430, 0x43b, 0x443, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x458, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x458, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x458,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d,
0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x423,
0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x423, 0x437,
0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x412, 0x430, 0x43d, 0x443, 0x430,
0x442, 0x443, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x412, 0x430,
0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x412, 0x435, 0x43d, 0x435, 0x446, 0x443,
0x435, 0x43b, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x412,
0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432,
0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x412, 0x43e,
0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x412, 0x43e, 0x43b, 0x433, 0x43e,
0x433, 0x440, 0x430, 0x434, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440,
0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x43d, 0x430, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x412, 0x435, 0x458, 0x43a, 0x20, 0x43e, 0x441,
0x442, 0x440, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x412, 0x430, 0x43b, 0x438, 0x441, 0x20, 0x438, 0x20, 0x424, 0x443, 0x442, 0x443,
0x43d, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x408, 0x430, 0x43a, 0x443, 0x442, 0x441, 0x43a,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x408, 0x430, 0x43a, 0x443,
0x442, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x408, 0x430,
0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x43d, 0x430, 0x408, 0x435, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438,
0x43d, 0x431, 0x443, 0x440, 0x433, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x408, 0x435, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443,
0x440, 0x433, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x408, 0x435, 0x43a,
0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x20, 0x459,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x65, 0x75, 0x72,
0x20, 0x41, 0x66, 0x67, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x65, 0x75, 0x72, 0x20, 0x4b, 0x72, 0x65, 0x69, 0x7a, 0x61, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x65, 0x75, 0x72, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x61, 0x72, 0x20, 0x52, 0x65, 0x74, 0x65, 0x72, 0x65, 0x75,
0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x53, 0x75,
0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x65, 0x75, 0x72, 0x20, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f,
0x72, 0x6e, 0xf4, 0x67, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f,
0x61, 0xf1, 0x76, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x61,
0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65, 0x75,
0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4,
0x67, 0x65, 0x75, 0x72, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x65,
0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x41,
0x6c, 0x61, 0x73, 0x6b, 0x61, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1,
0x76, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x65, 0x75, 0x72, 0x20,
0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc,
0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79,
0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x6c, 0x6d,
0x61, 0x74, 0x79, 0x65, 0x75, 0x72, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f,
0x61, 0xf1, 0x76, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x61, 0x6e,
0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x61,
0x72, 0x20, 0x43, 0x2bc, 0x68, 0x72, 0x65, 0x69, 0x7a, 0x65, 0x75, 0x72,
0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x61, 0x72, 0x20,
0x43, 0x2bc, 0x68, 0x72, 0x65, 0x69, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x68,
0x61, 0xf1, 0x76, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x72, 0x65,
0x69, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x61, 0x72, 0x20, 0x52, 0x65, 0x74,
0x65, 0x72, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1,
0x76, 0x20, 0x61, 0x72, 0x20, 0x52, 0x65, 0x74, 0x65, 0x72, 0x65, 0x75,
0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x61, 0x72, 0x20, 0x52, 0x65,
0x74, 0x65, 0x72, 0x65, 0x75, 0x72, 0x20, 0x61, 0x72, 0x20, 0x4d, 0x65,
0x6e, 0x65, 0x7a, 0x69, 0x6f, 0xf9, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc,
0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x61, 0x72, 0x20, 0x4d, 0x65, 0x6e,
0x65, 0x7a, 0x69, 0x6f, 0xf9, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1,
0x76, 0x20, 0x61, 0x72, 0x20, 0x4d, 0x65, 0x6e, 0x65, 0x7a, 0x69, 0x6f,
0xf9, 0x65, 0x75, 0x72, 0x20, 0x61, 0x6e, 0x20, 0x48, 0x61, 0x62, 0x61,
0x73, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1,
0x76, 0x20, 0x61, 0x6e, 0x20, 0x48, 0x61, 0x62, 0x61, 0x73, 0x6b, 0x65,
0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x61, 0x6e, 0x20, 0x48,
0x61, 0x62, 0x61, 0x73, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x41, 0x6e, 0x61,
0x64, 0x79, 0x72, 0x2bc, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f,
0x61, 0xf1, 0x76, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x2bc, 0x65,
0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x6e, 0x61, 0x64,
0x79, 0x72, 0x2bc, 0x65, 0x75, 0x72, 0x20, 0x41, 0x70, 0x69, 0x61, 0x65,
0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x41,
0x70, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20,
0x41, 0x70, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20, 0x41, 0x72, 0x61, 0x62,
0x69, 0x61, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1,
0x76, 0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20,
0x68, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x65,
0x75, 0x72, 0x20, 0x41, 0x72, 0x63, 0x2bc, 0x68, 0x61, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1,
0x76, 0x20, 0x41, 0x72, 0x63, 0x2bc, 0x68, 0x61, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x72,
0x63, 0x2bc, 0x68, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x65, 0x75, 0x72,
0x20, 0x41, 0x72, 0x63, 0x2bc, 0x68, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67,
0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20,
0x41, 0x72, 0x63, 0x2bc, 0x68, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20,
0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65,
0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x72, 0x63, 0x2bc,
0x68, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x61, 0x72, 0x20, 0x43,
0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65, 0x75, 0x72, 0x20, 0x41,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc,
0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x61, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x72,
0x6d, 0x65, 0x6e, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20, 0x61, 0x6e, 0x20,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x20,
0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x61, 0x6e, 0x20, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x20, 0x68,
0x61, 0xf1, 0x76, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x20, 0x4b, 0x72, 0x65, 0x69, 0x7a,
0x61, 0x6f, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x65, 0x75, 0x72,
0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x4b, 0x72, 0x65,
0x69, 0x7a, 0x61, 0x6f, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x65,
0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x4b, 0x72, 0x65, 0x69,
0x7a, 0x61, 0x6f, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x65, 0x75,
0x72, 0x20, 0x4b, 0x72, 0x65, 0x69, 0x7a, 0x61, 0x6f, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f,
0x72, 0x6e, 0xf4, 0x67, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f,
0x61, 0xf1, 0x76, 0x20, 0x4b, 0x72, 0x65, 0x69, 0x7a, 0x61, 0x6f, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc,
0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61,
0xf1, 0x76, 0x20, 0x4b, 0x72, 0x65, 0x69, 0x7a, 0x61, 0x6f, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68,
0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65, 0x75, 0x72, 0x20, 0x41, 0x6f, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x61, 0x72, 0x20, 0x52, 0x65,
0x74, 0x65, 0x72, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61,
0xf1, 0x76, 0x20, 0x41, 0x6f, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x61, 0x72, 0x20, 0x52, 0x65, 0x74, 0x65, 0x72, 0x65, 0x75, 0x72,
0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x6f, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x61, 0x72, 0x20, 0x52, 0x65, 0x74, 0x65, 0x72,
0x65, 0x75, 0x72, 0x20, 0x41, 0x6f, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4,
0x67, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76,
0x20, 0x41, 0x6f, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x61,
0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65, 0x75,
0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x6f, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f,
0x72, 0x6e, 0xf4, 0x67, 0x65, 0x75, 0x72, 0x20, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x61, 0x69, 0x64, 0x6a, 0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x63,
0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62,
0x61, 0x69, 0x64, 0x6a, 0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61,
0xf1, 0x76, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x6a,
0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61,
0xf1, 0x76, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x7a,
0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x61, 0x6e, 0x20,
0x41, 0x7a, 0x6f, 0x72, 0x65, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x20,
0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61,
0xf1, 0x76, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73,
0x68, 0x65, 0x75, 0x72, 0x20, 0x42, 0x68, 0x6f, 0x75, 0x74, 0x61, 0x6e,
0x65, 0x75, 0x72, 0x20, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x65,
0x75, 0x72, 0x20, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x65,
0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x42,
0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20, 0x68,
0x61, 0xf1, 0x76, 0x20, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61,
0x65, 0x75, 0x72, 0x20, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44,
0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x65, 0x75, 0x72,
0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x61, 0x62, 0x2d, 0x47, 0x6c,
0x61, 0x73, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1,
0x76, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x61, 0x62, 0x2d, 0x47,
0x6c, 0x61, 0x73, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20,
0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x61, 0x62, 0x2d, 0x47, 0x6c, 0x61,
0x73, 0x65, 0x75, 0x72, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72,
0x6f, 0x65, 0x75, 0x72, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x65, 0x75, 0x72, 0x20, 0x68,
0x61, 0xf1, 0x76, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x65,
0x75, 0x72, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x65, 0x75, 0x72, 0x20,
0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x43, 0x68, 0x69, 0x6c,
0x65, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x43, 0x68,
0x69, 0x6c, 0x65, 0x65, 0x75, 0x72, 0x20, 0x53, 0x69, 0x6e, 0x61, 0x65,
0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x53,
0x69, 0x6e, 0x61, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20,
0x53, 0x69, 0x6e, 0x61, 0x65, 0x75, 0x72, 0x20, 0x45, 0x6e, 0x65, 0x7a,
0x20, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x65, 0x75,
0x72, 0x20, 0x49, 0x6e, 0x69, 0x7a, 0x69, 0x20, 0x4b, 0x6f, 0x6b, 0x6f,
0x7a, 0x65, 0x75, 0x72, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69,
0x61, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76,
0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x65, 0x75, 0x72,
0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x61, 0x65, 0x75, 0x72, 0x20, 0x49, 0x6e, 0x69, 0x7a, 0x69, 0x20,
0x43, 0x6f, 0x6f, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f,
0x61, 0xf1, 0x76, 0x20, 0x49, 0x6e, 0x69, 0x7a, 0x69, 0x20, 0x43, 0x6f,
0x6f, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x49,
0x6e, 0x69, 0x7a, 0x69, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x65, 0x75, 0x72,
0x20, 0x4b, 0x75, 0x62, 0x61, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68,
0x6f, 0x61, 0xf1, 0x76, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x65, 0x75, 0x72,
0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x65, 0x75,
0x72, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x65, 0x75, 0x72, 0x20, 0x44,
0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69,
0x6c, 0x6c, 0x65, 0x65, 0x75, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72,
0x20, 0x61, 0x72, 0x20, 0x52, 0x65, 0x74, 0x65, 0x72, 0x65, 0x75, 0x72,
0x20, 0x45, 0x6e, 0x65, 0x7a, 0x20, 0x50, 0x61, 0x73, 0x6b, 0x65, 0x75,
0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x45, 0x6e,
0x65, 0x7a, 0x20, 0x50, 0x61, 0x73, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x68,
0x61, 0xf1, 0x76, 0x20, 0x45, 0x6e, 0x65, 0x7a, 0x20, 0x50, 0x61, 0x73,
0x6b, 0x65, 0x75, 0x72, 0x20, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72,
0x65, 0x75, 0x72, 0x20, 0x4b, 0x72, 0x65, 0x69, 0x7a, 0x65, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61,
0xf1, 0x76, 0x20, 0x4b, 0x72, 0x65, 0x69, 0x7a, 0x65, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x4b,
0x72, 0x65, 0x69, 0x7a, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x65, 0x75,
0x72, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x61, 0x72, 0x20,
0x52, 0x65, 0x74, 0x65, 0x72, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68,
0x6f, 0x61, 0xf1, 0x76, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x61, 0x72, 0x20, 0x52, 0x65, 0x74, 0x65, 0x72, 0x65, 0x75, 0x72, 0x20,
0x68, 0x61, 0xf1, 0x76, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x61, 0x72, 0x20, 0x52, 0x65, 0x74, 0x65, 0x72, 0x65, 0x75, 0x72, 0x20,
0x4b, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x6e, 0x67, 0x72, 0x61, 0x64, 0x65,
0x75, 0x72, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x61, 0x72,
0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65, 0x75, 0x72,
0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72,
0x6e, 0xf4, 0x67, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc,
0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65, 0x75, 0x72, 0x20, 0x49, 0x6e,
0x69, 0x7a, 0x69, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20,
0x49, 0x6e, 0x69, 0x7a, 0x69, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x49,
0x6e, 0x69, 0x7a, 0x69, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x65, 0x75, 0x72, 0x20, 0x46, 0x69, 0x64, 0x6a, 0x69, 0x65, 0x75,
0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x46, 0x69,
0x64, 0x6a, 0x69, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20,
0x46, 0x69, 0x64, 0x6a, 0x69, 0x65, 0x75, 0x72, 0x20, 0x47, 0x77, 0x69,
0x61, 0x6e, 0x61, 0x20, 0x63, 0x2bc, 0x68, 0x61, 0x6c, 0x6c, 0x65, 0x75,
0x72, 0x20, 0x44, 0x6f, 0x75, 0x61, 0x72, 0x6f, 0xf9, 0x20, 0x61, 0x6f,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x46, 0x72, 0x61, 0xf1, 0x73, 0x20,
0x68, 0x61, 0x67, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69,
0x6b, 0x61, 0x65, 0x75, 0x72, 0x20, 0x49, 0x6e, 0x69, 0x7a, 0x69, 0x20,
0x47, 0x61, 0x6c, 0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x65, 0x75, 0x72,
0x20, 0x49, 0x6e, 0x69, 0x7a, 0x69, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69,
0x65, 0x72, 0x65, 0x75, 0x72, 0x20, 0x4a, 0x6f, 0x72, 0x6a, 0x69, 0x61,
0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20,
0x4a, 0x6f, 0x72, 0x6a, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61,
0xf1, 0x76, 0x20, 0x4a, 0x6f, 0x72, 0x6a, 0x69, 0x61, 0x65, 0x75, 0x72,
0x20, 0x49, 0x6e, 0x69, 0x7a, 0x69, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65,
0x72, 0x74, 0x41, 0x6d, 0x7a, 0x65, 0x72, 0x20, 0x6b, 0x65, 0x69, 0x74,
0x61, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68,
0x20, 0x28, 0x41, 0x4b, 0x47, 0x29, 0x65, 0x75, 0x72, 0x20, 0x47, 0x72,
0x65, 0x75, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x61, 0x72, 0x20, 0x52,
0x65, 0x74, 0x65, 0x72, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f,
0x61, 0xf1, 0x76, 0x20, 0x47, 0x72, 0x65, 0x75, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x61, 0x72, 0x20, 0x52, 0x65, 0x74, 0x65, 0x72, 0x65, 0x75,
0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x47, 0x72, 0x65, 0x75, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x61, 0x72, 0x20, 0x52, 0x65, 0x74, 0x65,
0x72, 0x65, 0x75, 0x72, 0x20, 0x47, 0x72, 0x65, 0x75, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e,
0xf4, 0x67, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1,
0x76, 0x20, 0x47, 0x72, 0x65, 0x75, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65,
0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x47, 0x72, 0x65, 0x75,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68,
0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68,
0x6f, 0x61, 0xf1, 0x76, 0x20, 0x47, 0x75, 0x61, 0x6d, 0x65, 0x75, 0x72,
0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x61, 0x72, 0x20,
0x50, 0x6c, 0x65, 0x67, 0x2d, 0x6d, 0x6f, 0x72, 0x20, 0x41, 0x72, 0x61,
0x62, 0x2d, 0x68, 0x61, 0x2d, 0x50, 0x65, 0x72, 0x73, 0x65, 0x75, 0x72,
0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x65, 0x75, 0x72, 0x20, 0x48,
0x61, 0x77, 0x61, 0x69, 0x69, 0x20, 0x68, 0x61, 0x67, 0x20, 0x61, 0x6e,
0x20, 0x41, 0x6c, 0x65, 0x6f, 0x75, 0x74, 0x65, 0x64, 0x65, 0x75, 0x72,
0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x20, 0x68, 0x61, 0x67, 0x20, 0x61, 0x6e, 0x20, 0x41,
0x6c, 0x65, 0x6f, 0x75, 0x74, 0x65, 0x64, 0x65, 0x75, 0x72, 0x20, 0x68,
0x61, 0xf1, 0x76, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x20, 0x68,
0x61, 0x67, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x6c, 0x65, 0x6f, 0x75, 0x74,
0x65, 0x64, 0x65, 0x75, 0x72, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b,
0x6f, 0x6e, 0x67, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61,
0xf1, 0x76, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67,
0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x48, 0x6f, 0x6e,
0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x65, 0x75, 0x72, 0x20, 0x48, 0x6f,
0x76, 0x64, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1,
0x76, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61,
0xf1, 0x76, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x65, 0x75, 0x72, 0x20, 0x63,
0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61,
0x65, 0x75, 0x72, 0x20, 0x4d, 0x65, 0x75, 0x72, 0x76, 0x6f, 0x72, 0x20,
0x49, 0x6e, 0x64, 0x65, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x49, 0x6e, 0x64,
0x65, 0x7a, 0x2d, 0x53, 0x69, 0x6e, 0x61, 0x65, 0x75, 0x72, 0x20, 0x4b,
0x72, 0x65, 0x69, 0x7a, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a,
0x69, 0x61, 0x65, 0x75, 0x72, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x7a, 0x69, 0x61, 0x20, 0x61, 0x72, 0x20, 0x52, 0x65, 0x74, 0x65, 0x72,
0x65, 0x75, 0x72, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69,
0x61, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4,
0x67, 0x65, 0x75, 0x72, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x65, 0x75, 0x72,
0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x49, 0x72, 0x61,
0x6e, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x49, 0x72,
0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76,
0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x65, 0x75, 0x72, 0x20,
0x68, 0x61, 0xf1, 0x76, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x65, 0x75, 0x72, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x65, 0x75,
0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x49, 0x73,
0x72, 0x61, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76,
0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x20, 0x4a,
0x61, 0x70, 0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f,
0x61, 0xf1, 0x76, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x65, 0x75, 0x72,
0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x65,
0x75, 0x72, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x61, 0x72, 0x20, 0x52, 0x65, 0x74, 0x65, 0x72, 0x65, 0x75, 0x72,
0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x61,
0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65, 0x75,
0x72, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x65, 0x75, 0x72, 0x20, 0x63,
0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61,
0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x4b, 0x6f, 0x72,
0x65, 0x61, 0x65, 0x75, 0x72, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65,
0x65, 0x75, 0x72, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x73, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61,
0xf1, 0x76, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x65, 0x75,
0x72, 0x20, 0x4b, 0x79, 0x72, 0x67, 0x79, 0x7a, 0x73, 0x74, 0x61, 0x6e,
0x65, 0x75, 0x72, 0x20, 0x53, 0x72, 0x69, 0x20, 0x4c, 0x61, 0x6e, 0x6b,
0x61, 0x65, 0x75, 0x72, 0x20, 0x4c, 0x69, 0x6e, 0x65, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x75, 0x72, 0x20, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc,
0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x65, 0x75, 0x72,
0x20, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc,
0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x65,
0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x4d, 0x61, 0x63, 0x61,
0x75, 0x65, 0x75, 0x72, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x68,
0x61, 0xf1, 0x76, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x65,
0x75, 0x72, 0x20, 0x4d, 0x61, 0x6c, 0x61, 0x79, 0x73, 0x69, 0x61, 0x65,
0x75, 0x72, 0x20, 0x61, 0x72, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76,
0x65, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x49, 0x6e, 0x69, 0x7a, 0x69, 0x20,
0x4d, 0x61, 0x72, 0x6b, 0x69, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x49, 0x6e,
0x69, 0x7a, 0x69, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c,
0x65, 0x75, 0x72, 0x20, 0x4d, 0x6f, 0x72, 0x69, 0x73, 0x65, 0x75, 0x72,
0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x4d, 0x6f, 0x72,
0x69, 0x73, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x4d,
0x6f, 0x72, 0x69, 0x73, 0x65, 0x75, 0x72, 0x20, 0x4d, 0x61, 0x77, 0x73,
0x6f, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x61, 0x6e, 0x20, 0x48, 0x61, 0x62,
0x61, 0x73, 0x6b, 0x20, 0x6d, 0x65, 0x63, 0x2bc, 0x68, 0x69, 0x6b, 0x61,
0x6e, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76,
0x20, 0x61, 0x6e, 0x20, 0x48, 0x61, 0x62, 0x61, 0x73, 0x6b, 0x20, 0x6d,
0x65, 0x63, 0x2bc, 0x68, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x75, 0x72, 0x20,
0x68, 0x61, 0xf1, 0x76, 0x20, 0x61, 0x6e, 0x20, 0x48, 0x61, 0x62, 0x61,
0x73, 0x6b, 0x20, 0x6d, 0x65, 0x63, 0x2bc, 0x68, 0x69, 0x6b, 0x61, 0x6e,
0x65, 0x75, 0x72, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61,
0x74, 0x61, 0x72, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61,
0xf1, 0x76, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74,
0x61, 0x72, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x55,
0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x65, 0x75,
0x72, 0x20, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x65, 0x75, 0x72, 0x20,
0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x4d, 0x6f, 0x73, 0x6b,
0x6f, 0x76, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x4d,
0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x65, 0x75, 0x72, 0x20, 0x4d, 0x79, 0x61,
0x6e, 0x6d, 0x61, 0x72, 0x65, 0x75, 0x72, 0x20, 0x4e, 0x61, 0x75, 0x72,
0x75, 0x65, 0x75, 0x72, 0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x65, 0x75,
0x72, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20,
0x4e, 0x65, 0x76, 0x65, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68,
0x6f, 0x61, 0xf1, 0x76, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x69, 0x61, 0x20, 0x4e, 0x65, 0x76, 0x65, 0x7a, 0x65, 0x75, 0x72, 0x20,
0x68, 0x61, 0xf1, 0x76, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x69, 0x61, 0x20, 0x4e, 0x65, 0x76, 0x65, 0x7a, 0x65, 0x75, 0x72, 0x20,
0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x4e, 0x65, 0x76, 0x65, 0x7a,
0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20,
0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x4e, 0x65, 0x76, 0x65, 0x7a,
0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x5a, 0x65, 0x6c,
0x61, 0x6e, 0x64, 0x2d, 0x4e, 0x65, 0x76, 0x65, 0x7a, 0x65, 0x75, 0x72,
0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e,
0x64, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76,
0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e,
0x64, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x4e, 0x65,
0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x75,
0x72, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x65, 0x75, 0x72, 0x20, 0x45, 0x6e,
0x65, 0x7a, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x65, 0x75,
0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x45, 0x6e,
0x65, 0x7a, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x65, 0x75,
0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x45, 0x6e, 0x65, 0x7a, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc,
0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x68, 0x61, 0x65, 0x75, 0x72, 0x20, 0x4d, 0x61, 0x72,
0x69, 0x61, 0x6e, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x4e, 0x6f, 0x72, 0x7a,
0x68, 0x65, 0x75, 0x72, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f,
0x61, 0xf1, 0x76, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x65,
0x75, 0x72, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x63,
0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x65,
0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x4f, 0x6d, 0x73, 0x6b,
0x65, 0x75, 0x72, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x65, 0x75, 0x72, 0x20,
0x68, 0x61, 0xf1, 0x76, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x65, 0x75, 0x72, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x65, 0x75,
0x72, 0x20, 0x50, 0x61, 0x70, 0x6f, 0x75, 0x61, 0x2d, 0x47, 0x69, 0x6e,
0x65, 0x61, 0x2d, 0x4e, 0x65, 0x76, 0x65, 0x7a, 0x65, 0x75, 0x72, 0x20,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x65, 0x75, 0x72, 0x20,
0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x79, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76,
0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x65, 0x75, 0x72,
0x20, 0x50, 0x65, 0x72, 0x6f, 0x75, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc,
0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x50, 0x65, 0x72, 0x6f, 0x75, 0x65,
0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x50, 0x65, 0x72, 0x6f,
0x75, 0x65, 0x75, 0x72, 0x20, 0x61, 0x72, 0x20, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x65, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68,
0x6f, 0x61, 0xf1, 0x76, 0x20, 0x61, 0x72, 0x20, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x65, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1,
0x76, 0x20, 0x61, 0x72, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e,
0x65, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x49, 0x6e, 0x69, 0x7a, 0x69, 0x20,
0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x65, 0x75, 0x72, 0x20, 0x53,
0x61, 0x6e, 0x74, 0x2d, 0x50, 0xea, 0x72, 0x2d, 0x68, 0x61, 0x2d, 0x4d,
0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc,
0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x53, 0x61, 0x6e, 0x74, 0x2d, 0x50,
0xea, 0x72, 0x2d, 0x68, 0x61, 0x2d, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f,
0x6e, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x53, 0x61,
0x6e, 0x74, 0x2d, 0x50, 0xea, 0x72, 0x2d, 0x68, 0x61, 0x2d, 0x4d, 0x69,
0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x50, 0x69, 0x74,
0x63, 0x61, 0x69, 0x72, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x50, 0x6f, 0x68,
0x6e, 0x70, 0x65, 0x69, 0x65, 0x75, 0x72, 0x20, 0x50, 0x2bc, 0x79, 0x6f,
0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x65, 0x75, 0x72, 0x20, 0x61, 0x72,
0x20, 0x52, 0x65, 0xfc, 0x6e, 0x69, 0x6f, 0x6e, 0x65, 0x75, 0x72, 0x20,
0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x65, 0x75, 0x72, 0x20, 0x53,
0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x63,
0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20,
0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x65, 0x75, 0x72, 0x20,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68,
0x6f, 0x61, 0xf1, 0x76, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x65, 0x75,
0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x65, 0x75, 0x72, 0x20, 0x53, 0x65, 0x63, 0x68, 0x65, 0x6c, 0x65, 0x7a,
0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20,
0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x75, 0x72, 0x65, 0x75, 0x72,
0x20, 0x49, 0x6e, 0x69, 0x7a, 0x69, 0x20, 0x53, 0x61, 0x6c, 0x6f, 0x6d,
0x6f, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x20, 0x61, 0x72, 0x20, 0x53, 0x75, 0x65, 0x75, 0x72, 0x20, 0x53,
0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x75, 0x72, 0x20, 0x53, 0x79,
0x6f, 0x77, 0x61, 0x65, 0x75, 0x72, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74,
0x69, 0x65, 0x75, 0x72, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x65,
0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1,
0x76, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x65, 0x75, 0x72, 0x20,
0x54, 0x61, 0x64, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x65,
0x75, 0x72, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x65, 0x75,
0x72, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x65, 0x75, 0x72, 0x20, 0x63,
0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x65, 0x75, 0x72, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x65,
0x75, 0x72, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61,
0xf1, 0x76, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x65, 0x75, 0x72, 0x20, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x65, 0x75,
0x72, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x65, 0x75, 0x72,
0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x79, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76,
0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x65, 0x75, 0x72, 0x20,
0x4f, 0x75, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x65,
0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x4f,
0x75, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x65, 0x75,
0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x4f, 0x75, 0x7a, 0x62, 0x65,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68,
0x6f, 0x61, 0xf1, 0x76, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75,
0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x65, 0x75, 0x72, 0x20, 0x56, 0x65, 0x6e, 0x65,
0x7a, 0x75, 0x65, 0x6c, 0x61, 0x65, 0x75, 0x72, 0x20, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x65, 0x75, 0x72, 0x20,
0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x68,
0x61, 0xf1, 0x76, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f,
0x61, 0xf1, 0x76, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x65, 0x75, 0x72, 0x20, 0x56,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x57, 0x61, 0x6b,
0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x75, 0x72, 0x20,
0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x68, 0x61, 0x20, 0x46, 0x75,
0x74, 0x75, 0x6e, 0x61, 0x65, 0x75, 0x72, 0x20, 0x59, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61,
0xf1, 0x76, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x65, 0x75,
0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x6f, 0x75, 0x72, 0x67, 0x65, 0x75, 0x72, 0x20,
0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x59, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x6f, 0x75, 0x72, 0x67, 0x65, 0x75,
0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x6f, 0x75, 0x72, 0x67, 0x410, 0x444, 0x433,
0x430, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d,
0x43e, 0x430, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x437, 0x442, 0x43e, 0x447, 0x43d, 0x43e,
0x430, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x42e, 0x436, 0x43d, 0x43e, 0x430, 0x444, 0x440, 0x438,
0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x430, 0x444, 0x440, 0x438, 0x43a,
0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417,
0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x430, 0x444, 0x440, 0x438, 0x43a, 0x430,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f,
0x430, 0x434, 0x43d, 0x43e, 0x430, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43b, 0x44f,
0x441, 0x43a, 0x430, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x20, 0x2013, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x20, 0x2013,
0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43c, 0x430, 0x437, 0x43e,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43c,
0x430, 0x437, 0x43e, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f,
0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x446,
0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435,
0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d,
0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421,
0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a,
0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430,
0x43b, 0x43d, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435,
0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x438, 0x437, 0x442, 0x43e, 0x447, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d,
0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e,
0x20, 0x438, 0x437, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x438, 0x437, 0x442, 0x43e,
0x447, 0x43d, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435,
0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43f, 0x43b, 0x430, 0x43d, 0x438, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435,
0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x43f, 0x43b, 0x430, 0x43d, 0x438, 0x43d, 0x441, 0x43a, 0x43e,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e,
0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20,
0x43f, 0x43b, 0x430, 0x43d, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f,
0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x442,
0x438, 0x445, 0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e,
0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20,
0x442, 0x438, 0x445, 0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441, 0x43a, 0x43e,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e,
0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20,
0x442, 0x438, 0x445, 0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441, 0x43a, 0x43e,
0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43d, 0x430, 0x434, 0x438,
0x440, 0x20, 0x2013, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43d, 0x430, 0x434,
0x438, 0x440, 0x20, 0x2013, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410,
0x43f, 0x438, 0x44f, 0x410, 0x43f, 0x438, 0x44f, 0x20, 0x2013, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x410, 0x43f, 0x438, 0x44f, 0x20, 0x2013, 0x20, 0x43b, 0x44f, 0x442,
0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x410, 0x440, 0x430, 0x431, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x430, 0x431, 0x441, 0x43a, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x430, 0x431, 0x441, 0x43a, 0x43e, 0x20,
0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x436, 0x435, 0x43d, 0x442,
0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410,
0x440, 0x436, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x410, 0x440, 0x436, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f,
0x430, 0x434, 0x43d, 0x43e, 0x430, 0x440, 0x436, 0x435, 0x43d, 0x442, 0x438, 0x43d,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f,
0x430, 0x434, 0x43d, 0x43e, 0x430, 0x440, 0x436, 0x435, 0x43d, 0x442, 0x438, 0x43d,
0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430,
0x434, 0x43d, 0x43e, 0x430, 0x440, 0x436, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x43c,
0x435, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410,
0x440, 0x43c, 0x435, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x410, 0x440, 0x43c, 0x435, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442,
0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c,
0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x430, 0x442,
0x43b, 0x430, 0x43d, 0x442, 0x438, 0x447, 0x435, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x430,
0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x447, 0x435, 0x441, 0x43a, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x430,
0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x447, 0x435, 0x441, 0x43a, 0x43e, 0x20,
0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430,
0x43b, 0x43d, 0x43e, 0x430, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x426, 0x435, 0x43d,
0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x430, 0x432, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x426,
0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x430, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442,
0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f,
0x20, 0x2013, 0x20, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20, 0x446,
0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20,
0x2013, 0x20, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20, 0x446, 0x435,
0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x2013, 0x20,
0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442,
0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x418, 0x437, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x430, 0x432, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x418, 0x437, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x430, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x418, 0x437, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x430, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442,
0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x430, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x430,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x430,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20,
0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430,
0x439, 0x434, 0x436, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x434, 0x436, 0x430,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x437, 0x435,
0x440, 0x431, 0x430, 0x439, 0x434, 0x436, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20,
0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x43a,
0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x410, 0x437, 0x43e,
0x440, 0x441, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438,
0x20, 0x2013, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x437, 0x43e, 0x440, 0x441,
0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x20, 0x2013,
0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411, 0x430, 0x43d, 0x433, 0x43b,
0x430, 0x434, 0x435, 0x448, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x43a, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435,
0x448, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411, 0x443,
0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x411, 0x43e, 0x43b, 0x438, 0x432, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x430, 0x437, 0x438,
0x43b, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x430,
0x437, 0x438, 0x43b, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x411, 0x440, 0x443, 0x43d, 0x435, 0x439, 0x20, 0x414, 0x430, 0x440, 0x443,
0x441, 0x441, 0x430, 0x43b, 0x430, 0x43c, 0x41a, 0x430, 0x431, 0x43e, 0x20, 0x412,
0x435, 0x440, 0x434, 0x435, 0x41a, 0x430, 0x431, 0x43e, 0x20, 0x412, 0x435, 0x440,
0x434, 0x435, 0x20, 0x2013, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x430, 0x431,
0x43e, 0x20, 0x412, 0x435, 0x440, 0x434, 0x435, 0x20, 0x2013, 0x20, 0x43b, 0x44f,
0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x427, 0x430, 0x43c, 0x43e, 0x440, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x430, 0x442, 0x44a, 0x43c, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x430, 0x442, 0x44a,
0x43c, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x430, 0x442,
0x44a, 0x43c, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x427, 0x438, 0x43b, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x427, 0x438, 0x43b, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x427, 0x438, 0x43b, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20,
0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x438, 0x442, 0x430, 0x439, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x438, 0x442, 0x430,
0x439, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x438, 0x442,
0x430, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x20, 0x420, 0x43e, 0x436, 0x434, 0x435,
0x441, 0x442, 0x432, 0x43e, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x43e, 0x432, 0x438,
0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x41a, 0x43e, 0x43b, 0x443,
0x43c, 0x431, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x439, 0x441, 0x43a, 0x43e,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438,
0x439, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x20, 0x41a, 0x443, 0x43a, 0x41e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x438, 0x20, 0x41a, 0x443, 0x43a, 0x20, 0x2013, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x20,
0x41a, 0x443, 0x43a, 0x20, 0x2013, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41a, 0x443, 0x431, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41a, 0x443, 0x431, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41a, 0x443, 0x431, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20,
0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x414, 0x435, 0x439, 0x432, 0x438, 0x441,
0x414, 0x44e, 0x43c, 0x43e, 0x43d, 0x20, 0x414, 0x44e, 0x440, 0x432, 0x438, 0x43b,
0x418, 0x437, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x442, 0x438, 0x43c, 0x43e, 0x440,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x412, 0x435, 0x43b,
0x438, 0x43a, 0x434, 0x435, 0x43d, 0x441, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x412, 0x435, 0x43b, 0x438, 0x43a, 0x434, 0x435, 0x43d, 0x441,
0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x20, 0x2013, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x412, 0x435, 0x43b, 0x438, 0x43a, 0x434, 0x435, 0x43d,
0x441, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x20, 0x2013,
0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x415, 0x43a, 0x432, 0x430, 0x434,
0x43e, 0x440, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x426,
0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e,
0x43f, 0x435, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x435, 0x432, 0x440,
0x43e, 0x43f, 0x435, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x435, 0x432, 0x440,
0x43e, 0x43f, 0x435, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d,
0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x418, 0x437, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x435, 0x432, 0x440,
0x43e, 0x43f, 0x435, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x418, 0x437, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e,
0x43f, 0x435, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418,
0x437, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x435,
0x439, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x414,
0x430, 0x43b, 0x435, 0x447, 0x43d, 0x43e, 0x20, 0x438, 0x437, 0x442, 0x43e, 0x447,
0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x435, 0x439, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d,
0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x435, 0x439, 0x441, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e,
0x435, 0x432, 0x440, 0x43e, 0x43f, 0x435, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x435, 0x432,
0x440, 0x43e, 0x43f, 0x435, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x424, 0x43e, 0x43b, 0x43a,
0x43b, 0x435, 0x43d, 0x434, 0x441, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x438, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x441,
0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x20, 0x2013,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d,
0x434, 0x441, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438,
0x20, 0x2013, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x424, 0x438, 0x434,
0x436, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x424, 0x438, 0x434, 0x436, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x424, 0x438, 0x434, 0x436, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20,
0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x424, 0x440, 0x435, 0x43d, 0x441, 0x43a,
0x430, 0x20, 0x413, 0x432, 0x438, 0x430, 0x43d, 0x430, 0x424, 0x440, 0x435, 0x43d,
0x441, 0x43a, 0x438, 0x20, 0x44e, 0x436, 0x43d, 0x438, 0x20, 0x438, 0x20, 0x430,
0x43d, 0x442, 0x430, 0x440, 0x43a, 0x442, 0x438, 0x447, 0x435, 0x441, 0x43a, 0x438,
0x20, 0x442, 0x435, 0x440, 0x438, 0x442, 0x43e, 0x440, 0x438, 0x438, 0x413, 0x430,
0x43b, 0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x413, 0x430, 0x43c, 0x431, 0x438, 0x435, 0x413, 0x440, 0x443,
0x437, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x413, 0x440, 0x443, 0x437, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x413, 0x440, 0x443, 0x437, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20,
0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432,
0x438, 0x20, 0x413, 0x438, 0x43b, 0x431, 0x44a, 0x440, 0x442, 0x421, 0x440, 0x435,
0x434, 0x43d, 0x43e, 0x20, 0x433, 0x440, 0x438, 0x43d, 0x443, 0x438, 0x447, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x437, 0x442, 0x43e, 0x447,
0x43d, 0x43e, 0x433, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x437, 0x442, 0x43e, 0x447,
0x43d, 0x43e, 0x433, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a,
0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x437, 0x442, 0x43e, 0x447, 0x43d,
0x43e, 0x433, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x43e,
0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434,
0x43d, 0x43e, 0x433, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434,
0x43d, 0x43e, 0x433, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a,
0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d,
0x43e, 0x433, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x43e,
0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x435, 0x440, 0x441, 0x438,
0x439, 0x441, 0x43a, 0x438, 0x20, 0x437, 0x430, 0x43b, 0x438, 0x432, 0x413, 0x430,
0x44f, 0x43d, 0x430, 0x425, 0x430, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x43e, 0x2d,
0x430, 0x43b, 0x435, 0x443, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x425, 0x430, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x43e, 0x2d, 0x430,
0x43b, 0x435, 0x443, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x425, 0x430, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x43e, 0x2d, 0x430, 0x43b, 0x435,
0x443, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x425, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x425, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x441,
0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x425, 0x43e, 0x43d, 0x43a, 0x43e,
0x43d, 0x433, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x425, 0x43e, 0x432, 0x434, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x425, 0x43e, 0x432, 0x434, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x425, 0x43e, 0x432, 0x434, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442,
0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x418, 0x43d, 0x434, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x43d, 0x434, 0x438, 0x439, 0x441, 0x43a,
0x438, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x418, 0x43d, 0x434, 0x43e, 0x43a,
0x438, 0x442, 0x430, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x438, 0x43d,
0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x418, 0x437, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x438,
0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e,
0x438, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x439, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x430, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x430, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x430, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x43a,
0x443, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418,
0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442,
0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x418, 0x437, 0x440, 0x430, 0x435, 0x43b, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x437, 0x440, 0x430, 0x435, 0x43b,
0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x437, 0x440, 0x430,
0x435, 0x43b, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x42f, 0x43f, 0x43e, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x42f, 0x43f, 0x43e, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x42f, 0x43f, 0x43e, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442,
0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41f, 0x435, 0x442, 0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b,
0x43e, 0x432, 0x441, 0x43a, 0x2d, 0x41a, 0x430, 0x43c, 0x447, 0x430, 0x442, 0x441,
0x43a, 0x438, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x435, 0x442, 0x440,
0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x2d, 0x41a, 0x430,
0x43c, 0x447, 0x430, 0x442, 0x441, 0x43a, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41f, 0x435, 0x442, 0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441,
0x43a, 0x2d, 0x41a, 0x430, 0x43c, 0x447, 0x430, 0x442, 0x441, 0x43a, 0x438, 0x20,
0x2013, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x430, 0x437, 0x430,
0x445, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x418, 0x437, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x43a, 0x430, 0x437,
0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x43a, 0x430,
0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x440, 0x435, 0x439, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x440, 0x435, 0x439, 0x441,
0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x440, 0x435, 0x439,
0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x43e,
0x448, 0x440, 0x430, 0x439, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x440, 0x430,
0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x43e,
0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x438, 0x440, 0x433, 0x438,
0x437, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x415, 0x43a, 0x432, 0x430, 0x442, 0x43e, 0x440, 0x438, 0x430, 0x43b,
0x43d, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x41b, 0x43e,
0x440, 0x434, 0x20, 0x425, 0x430, 0x443, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425,
0x430, 0x443, 0x20, 0x2013, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41b, 0x43e, 0x440,
0x434, 0x20, 0x425, 0x430, 0x443, 0x20, 0x2013, 0x20, 0x43b, 0x44f, 0x442, 0x43d,
0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x433,
0x430, 0x434, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d,
0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41c, 0x430, 0x43b, 0x430, 0x439, 0x437, 0x438, 0x439, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x43b, 0x434, 0x438,
0x432, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430,
0x440, 0x43a, 0x438, 0x437, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e,
0x432, 0x438, 0x41c, 0x430, 0x440, 0x448, 0x430, 0x43b, 0x43e, 0x432, 0x438, 0x20,
0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x41c, 0x430, 0x432, 0x440, 0x438,
0x446, 0x438, 0x439, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x446, 0x438, 0x439, 0x20,
0x2013, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x446,
0x438, 0x439, 0x20, 0x2013, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c,
0x43e, 0x443, 0x441, 0x44a, 0x43d, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x442, 0x438, 0x445, 0x43e, 0x43e, 0x43a, 0x435,
0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c,
0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x442,
0x438, 0x445, 0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430, 0x43d,
0x441, 0x43a, 0x43e, 0x20, 0x442, 0x438, 0x445, 0x43e, 0x43e, 0x43a, 0x435, 0x430,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x423,
0x43b, 0x430, 0x43d, 0x431, 0x430, 0x442, 0x43e, 0x440, 0x441, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x423, 0x43b, 0x430, 0x43d, 0x431, 0x430, 0x442,
0x43e, 0x440, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x423, 0x43b,
0x430, 0x43d, 0x431, 0x430, 0x442, 0x43e, 0x440, 0x441, 0x43a, 0x43e, 0x20, 0x43b,
0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x43e, 0x441, 0x43a, 0x43e, 0x432, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x43e, 0x441, 0x43a,
0x43e, 0x432, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x43e,
0x441, 0x43a, 0x43e, 0x432, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d,
0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41c, 0x438, 0x430, 0x43d, 0x43c, 0x430, 0x440, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x435, 0x43f, 0x430, 0x43b, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x43e,
0x43a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x43e, 0x43a, 0x430, 0x43b, 0x435,
0x434, 0x43e, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d,
0x43e, 0x432, 0x43e, 0x43a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x43e,
0x437, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x43e, 0x437, 0x435, 0x43b, 0x430, 0x43d,
0x434, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432,
0x43e, 0x437, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x43e, 0x20, 0x43b,
0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434,
0x43b, 0x435, 0x43d, 0x434, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x41d, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434,
0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x44e, 0x444, 0x430,
0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x441, 0x43a, 0x43e, 0x20, 0x43b,
0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x440,
0x444, 0x43e, 0x43b, 0x43a, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x441, 0x43a, 0x43e, 0x20, 0x43b,
0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434,
0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44f, 0x424,
0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d,
0x43e, 0x440, 0x43e, 0x43d, 0x44f, 0x20, 0x2013, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20,
0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44f, 0x20, 0x2013, 0x20, 0x43b, 0x44f, 0x442,
0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432,
0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441,
0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41e, 0x43c, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41e, 0x43c, 0x441, 0x43a,
0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41e, 0x43c, 0x441, 0x43a, 0x43e, 0x20,
0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442,
0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f,
0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x43f,
0x443, 0x430, 0x20, 0x41d, 0x43e, 0x432, 0x430, 0x20, 0x413, 0x432, 0x438, 0x43d,
0x435, 0x44f, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x440, 0x430, 0x433,
0x432, 0x430, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f,
0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x43b,
0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x435, 0x440, 0x443, 0x430, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x435, 0x440, 0x443,
0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x435,
0x440, 0x443, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d,
0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x438, 0x43d, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x438,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x424, 0x438, 0x43b,
0x438, 0x43f, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d,
0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x20, 0x424, 0x435,
0x43d, 0x438, 0x43a, 0x441, 0x421, 0x435, 0x43d, 0x20, 0x41f, 0x438, 0x435, 0x440,
0x20, 0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x421, 0x435,
0x43d, 0x20, 0x41f, 0x438, 0x435, 0x440, 0x20, 0x438, 0x20, 0x41c, 0x438, 0x43a,
0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x2013, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421,
0x435, 0x43d, 0x20, 0x41f, 0x438, 0x435, 0x440, 0x20, 0x438, 0x20, 0x41c, 0x438,
0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x2013, 0x20, 0x43b, 0x44f, 0x442, 0x43d,
0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41f, 0x445, 0x435, 0x43d, 0x44f, 0x43d, 0x441, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x420, 0x435, 0x44e, 0x43d, 0x438, 0x43e, 0x43d,
0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x445, 0x430, 0x43b,
0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x421, 0x430, 0x43c, 0x430, 0x440, 0x430, 0x20, 0x2013, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x430, 0x20, 0x2013, 0x20, 0x43b, 0x44f,
0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x43c, 0x43e, 0x430,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x43c,
0x43e, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x421, 0x435, 0x439, 0x448, 0x435, 0x43b, 0x438, 0x421, 0x438, 0x43d, 0x433,
0x430, 0x43f, 0x443, 0x440, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x421, 0x43e, 0x43b, 0x43e, 0x43c, 0x43e, 0x43d, 0x43e, 0x432, 0x438, 0x20,
0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x42e, 0x436, 0x43d, 0x430, 0x20,
0x414, 0x436, 0x43e, 0x440, 0x434, 0x436, 0x438, 0x44f, 0x421, 0x443, 0x440, 0x438,
0x43d, 0x430, 0x43c, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x422, 0x430, 0x438, 0x442, 0x44f, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x422, 0x430, 0x439, 0x43f, 0x435, 0x422, 0x430, 0x439, 0x43f,
0x435, 0x20, 0x2013, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x430, 0x439, 0x43f,
0x435, 0x20, 0x2013, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x430,
0x434, 0x436, 0x438, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20,
0x2013, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20,
0x2013, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x443, 0x44e, 0x43a,
0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x443, 0x440,
0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d,
0x438, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442,
0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x423, 0x440, 0x443, 0x433, 0x432,
0x430, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x423, 0x440,
0x443, 0x433, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442,
0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x423, 0x437,
0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442,
0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x2013, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x2013, 0x20,
0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x412, 0x435, 0x43d, 0x435, 0x446, 0x443,
0x435, 0x43b, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x412,
0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x412, 0x43b, 0x430, 0x434, 0x438,
0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e,
0x43a, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x412,
0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x441, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440,
0x430, 0x434, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x412, 0x43e,
0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x441, 0x43a, 0x43e, 0x20, 0x43b,
0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x20,
0x423, 0x435, 0x439, 0x43a, 0x423, 0x43e, 0x43b, 0x438, 0x441, 0x20, 0x438, 0x20,
0x424, 0x443, 0x442, 0x443, 0x43d, 0x430, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x42f, 0x43a, 0x443, 0x442, 0x441,
0x43a, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x42f, 0x43a, 0x443,
0x442, 0x441, 0x43a, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440,
0x433, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x415, 0x43a,
0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x441, 0x43a,
0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440,
0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f,
0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x42e, 0x43a, 0x43e, 0x43d, 0x1021, 0x102c, 0x1016, 0x1002,
0x1014, 0x103a, 0x1014, 0x1005, 0x1039, 0x1005, 0x1010, 0x1014, 0x103a, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101c, 0x101a, 0x103a, 0x1021, 0x102c, 0x1016, 0x101b,
0x102d, 0x1000, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101b, 0x103e,
0x1031, 0x1037, 0x1021, 0x102c, 0x1016, 0x101b, 0x102d, 0x1000, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1010, 0x1031, 0x102c, 0x1004, 0x103a, 0x1021, 0x102c, 0x1016, 0x101b,
0x102d, 0x1000, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1014, 0x1031,
0x102c, 0x1000, 0x103a, 0x1021, 0x102c, 0x1016, 0x101b, 0x102d, 0x1000, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x1021, 0x102c,
0x1016, 0x101b, 0x102d, 0x1000, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x1021, 0x102c,
0x1016, 0x101b, 0x102d, 0x1000, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101c, 0x102c, 0x1005, 0x1000,
0x102c, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101c, 0x102c, 0x1005,
0x1000, 0x102c, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1021, 0x101c, 0x102c, 0x1005, 0x1000, 0x102c, 0x20, 0x1014, 0x103d, 0x1031,
0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1019, 0x1031, 0x1007, 0x102f, 0x1036, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1019, 0x1031, 0x1007, 0x102f, 0x1036, 0x20,
0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021,
0x1019, 0x1031, 0x1007, 0x102f, 0x1036, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e,
0x102e, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101c, 0x101a, 0x103a, 0x1015,
0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021,
0x101c, 0x101a, 0x103a, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x1005, 0x1036, 0x1010,
0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101c, 0x101a, 0x103a,
0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c,
0x101e, 0x102e, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1015,
0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1015, 0x102d, 0x102f,
0x1004, 0x103a, 0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x1005,
0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1010, 0x1031,
0x102c, 0x1004, 0x103a, 0x1010, 0x1014, 0x103a, 0x1038, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1010, 0x1031, 0x102c, 0x1004, 0x103a, 0x1010, 0x1014, 0x103a, 0x1038, 0x1005, 0x1036,
0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1010, 0x1031, 0x102c,
0x1004, 0x103a, 0x1010, 0x1014, 0x103a, 0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c,
0x101e, 0x102e, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1015, 0x1005, 0x102d, 0x1016, 0x102d, 0x1010, 0x103a, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1015, 0x1005, 0x102d, 0x1016, 0x102d, 0x1010, 0x103a, 0x1005, 0x1036, 0x1010,
0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x1005, 0x102d, 0x1016,
0x102d, 0x1010, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x1005,
0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1015,
0x102e, 0x101a, 0x102c, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1015,
0x102e, 0x101a, 0x102c, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1021, 0x1015, 0x102e, 0x101a, 0x102c, 0x20, 0x1014, 0x103d, 0x1031,
0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021,
0x102c, 0x101b, 0x1031, 0x1017, 0x103b, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1021, 0x102c, 0x101b, 0x1031, 0x1017, 0x103b, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c,
0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102c, 0x101b, 0x1031, 0x1017, 0x103b,
0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1021, 0x102c, 0x1002, 0x103b, 0x1004, 0x103a, 0x1010, 0x102e, 0x1038,
0x1014, 0x102c, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102c,
0x1002, 0x103b, 0x1004, 0x103a, 0x1010, 0x102e, 0x1038, 0x1014, 0x102c, 0x1038, 0x20, 0x1005,
0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102c,
0x1002, 0x103b, 0x1004, 0x103a, 0x1010, 0x102e, 0x1038, 0x1014, 0x102c, 0x1038, 0x20, 0x1014,
0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x20, 0x1021, 0x102c, 0x1002, 0x103b, 0x1004,
0x103a, 0x1010, 0x102e, 0x1038, 0x1014, 0x102c, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x20, 0x1021, 0x102c, 0x1002,
0x103b, 0x1004, 0x103a, 0x1010, 0x102e, 0x1038, 0x1014, 0x102c, 0x1038, 0x20, 0x1005, 0x1036,
0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1014, 0x1031,
0x102c, 0x1000, 0x103a, 0x20, 0x1021, 0x102c, 0x1002, 0x103b, 0x1004, 0x103a, 0x1010, 0x102e,
0x1038, 0x1014, 0x102c, 0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102c, 0x1019, 0x1031, 0x1038,
0x1014, 0x102e, 0x1038, 0x101a, 0x102c, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1021, 0x102c, 0x1019, 0x1031, 0x1038, 0x1014, 0x102e, 0x1038, 0x101a, 0x102c, 0x1038,
0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1021, 0x102c, 0x1019, 0x1031, 0x1038, 0x1014, 0x102e, 0x1038, 0x101a, 0x102c, 0x1038, 0x20,
0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1021, 0x1010, 0x1039, 0x1010, 0x101c, 0x1014, 0x103a, 0x1010, 0x1005, 0x103a,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1010, 0x1039, 0x1010, 0x101c,
0x1014, 0x103a, 0x1010, 0x1005, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1010, 0x1039, 0x1010, 0x101c, 0x1014, 0x103a,
0x1010, 0x1005, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20,
0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1029,
0x1005, 0x1010, 0x103c, 0x1031, 0x1038, 0x101c, 0x103b, 0x20, 0x1021, 0x101c, 0x101a, 0x103a,
0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1029, 0x1005, 0x1010, 0x103c, 0x1031, 0x1038, 0x101c, 0x103b, 0x20, 0x1021, 0x101c,
0x101a, 0x103a, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x20, 0x1005, 0x1036, 0x1010,
0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1029, 0x1005, 0x1010, 0x103c,
0x1031, 0x1038, 0x101c, 0x103b, 0x20, 0x1021, 0x101c, 0x101a, 0x103a, 0x1015, 0x102d, 0x102f,
0x1004, 0x103a, 0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101e, 0x103c, 0x1005, 0x1010, 0x103c, 0x1031,
0x1038, 0x101c, 0x103b, 0x102c, 0x1038, 0x20, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a,
0x1021, 0x101c, 0x101a, 0x103a, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101e, 0x103c, 0x1005, 0x1010, 0x103c, 0x1031, 0x1038,
0x101c, 0x103b, 0x102c, 0x1038, 0x20, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x1021,
0x101c, 0x101a, 0x103a, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x20, 0x1005, 0x1036,
0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101e, 0x103c, 0x1005,
0x1010, 0x103c, 0x1031, 0x1038, 0x101c, 0x103b, 0x102c, 0x1038, 0x20, 0x1021, 0x1014, 0x1031,
0x102c, 0x1000, 0x103a, 0x1021, 0x101c, 0x101a, 0x103a, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a,
0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1029, 0x1005, 0x1010,
0x103c, 0x1031, 0x1038, 0x101c, 0x103b, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1029, 0x1005, 0x1010, 0x103c, 0x1031, 0x1038, 0x101c,
0x103b, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1029, 0x1005, 0x1010, 0x103c, 0x1031, 0x1038,
0x101c, 0x103b, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x1029,
0x1005, 0x1010, 0x103c, 0x1031, 0x1038, 0x101c, 0x103b, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x1029, 0x1005, 0x1010, 0x103c,
0x1031, 0x1038, 0x101c, 0x103b, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1007, 0x102c, 0x1018, 0x102d, 0x102f, 0x1004, 0x103a,
0x1002, 0x103b, 0x1014, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021,
0x1007, 0x102c, 0x1018, 0x102d, 0x102f, 0x1004, 0x103a, 0x1002, 0x103b, 0x1014, 0x103a, 0x20,
0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021,
0x1007, 0x102c, 0x1018, 0x102d, 0x102f, 0x1004, 0x103a, 0x1002, 0x103b, 0x1014, 0x103a, 0x20,
0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1021, 0x1031, 0x1007, 0x102d, 0x102f, 0x1038, 0x101b, 0x102e, 0x1038, 0x1005,
0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1031, 0x1007, 0x102d,
0x102f, 0x1038, 0x101b, 0x102e, 0x1038, 0x1005, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031,
0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1031, 0x1007, 0x102d, 0x102f,
0x1038, 0x101b, 0x102e, 0x1038, 0x1005, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c,
0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1018, 0x1004, 0x103a,
0x1039, 0x1002, 0x101c, 0x102c, 0x1038, 0x1012, 0x1031, 0x1037, 0x101b, 0x103e, 0x103a, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1018, 0x1004, 0x103a, 0x1039, 0x1002, 0x101c,
0x102c, 0x1038, 0x1012, 0x1031, 0x1037, 0x101b, 0x103e, 0x103a, 0x20, 0x1005, 0x1036, 0x1010,
0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1018, 0x1004, 0x103a, 0x1039,
0x1002, 0x101c, 0x102c, 0x1038, 0x1012, 0x1031, 0x1037, 0x101b, 0x103e, 0x103a, 0x20, 0x1014,
0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1018, 0x1030, 0x1010, 0x1014, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1018, 0x102d, 0x102f, 0x101c, 0x102e, 0x1038, 0x1018, 0x102e, 0x1038, 0x101a, 0x102c,
0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1018, 0x101b, 0x102c, 0x1007,
0x102e, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1018, 0x101b, 0x102c,
0x1007, 0x102e, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1018, 0x101b, 0x102c, 0x1007, 0x102e, 0x1038, 0x20, 0x1014, 0x103d,
0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1018, 0x101b, 0x1030, 0x1014, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x20, 0x1005, 0x1036,
0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1000, 0x102d, 0x1010,
0x103a, 0x1017, 0x102c, 0x1012, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1000, 0x102d, 0x1010, 0x103a, 0x1017, 0x102c, 0x1012, 0x102e, 0x20, 0x1005, 0x1036, 0x1010,
0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1000, 0x102d, 0x1010, 0x103a,
0x1017, 0x102c, 0x1012, 0x102e, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1001, 0x103b, 0x102c, 0x1019, 0x102d,
0x102f, 0x101b, 0x102d, 0x102f, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1001,
0x103b, 0x102c, 0x1038, 0x101e, 0x1019, 0x103a, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1001, 0x103b, 0x102c, 0x1038, 0x101e, 0x1019, 0x103a, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c,
0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1001, 0x103b, 0x102c, 0x1038, 0x101e, 0x1019,
0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1001, 0x103b, 0x102e, 0x101c, 0x102e, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1001, 0x103b, 0x102e, 0x101c, 0x102e, 0x20, 0x1005, 0x1036,
0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1001, 0x103b, 0x102e,
0x101c, 0x102e, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1010, 0x101b, 0x102f, 0x1010, 0x103a, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1010, 0x101b, 0x102f, 0x1010, 0x103a, 0x20, 0x1005,
0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1010, 0x101b,
0x102f, 0x1010, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1001, 0x101b, 0x1005, 0x103a, 0x1005, 0x1019,
0x1010, 0x103a, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1000, 0x102d, 0x102f, 0x1000, 0x102d, 0x102f, 0x1038, 0x1000, 0x103b,
0x103d, 0x1014, 0x103a, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1000,
0x102d, 0x102f, 0x101c, 0x1036, 0x1018, 0x102e, 0x101a, 0x102c, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1000, 0x102d, 0x102f, 0x101c, 0x1036, 0x1018, 0x102e, 0x101a, 0x102c,
0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1000, 0x102d, 0x102f, 0x101c, 0x1036, 0x1018, 0x102e, 0x101a, 0x102c, 0x20, 0x1014, 0x103d,
0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1000, 0x103d, 0x1010, 0x103a, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x1005, 0x102f,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1000, 0x103d, 0x1010, 0x103a, 0x1000,
0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x1005, 0x102f, 0x20, 0x1005, 0x1036, 0x1010, 0x1031,
0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1000, 0x103d, 0x1010, 0x103a, 0x1000,
0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x1005, 0x102f, 0x20, 0x1014, 0x103d, 0x1031, 0x101b,
0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1000, 0x103b,
0x1030, 0x1038, 0x1018, 0x102c, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1000, 0x103b, 0x1030, 0x1038, 0x1018, 0x102c, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031,
0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1000, 0x103b, 0x1030, 0x1038, 0x1018,
0x102c, 0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1005,
0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1012, 0x1031,
0x1038, 0x1017, 0x1005, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1012,
0x1030, 0x1038, 0x1019, 0x1031, 0x102c, 0x1037, 0x1010, 0x103a, 0x20, 0x1012, 0x102b, 0x101b,
0x103a, 0x1017, 0x102e, 0x1038, 0x101c, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1010, 0x102e, 0x1019, 0x1031, 0x102c, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102e, 0x1005, 0x1010, 0x102c, 0x1000,
0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1021, 0x102e, 0x1005, 0x1010, 0x102c, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x20,
0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021,
0x102e, 0x1005, 0x1010, 0x102c, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x20, 0x1014,
0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1021, 0x102e, 0x1000, 0x103d, 0x1031, 0x1012, 0x1031, 0x102b, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1025, 0x101b, 0x1031, 0x102c, 0x1015, 0x1021, 0x101c, 0x101a,
0x103a, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1025, 0x101b, 0x1031, 0x102c, 0x1015, 0x1021, 0x101c, 0x101a, 0x103a, 0x1015,
0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1025, 0x101b, 0x1031, 0x102c, 0x1015, 0x1021, 0x101c,
0x101a, 0x103a, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x20, 0x1014, 0x103d, 0x1031,
0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021,
0x101b, 0x103e, 0x1031, 0x1037, 0x1025, 0x101b, 0x1031, 0x102c, 0x1015, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1025, 0x101b, 0x1031,
0x102c, 0x1015, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1025, 0x101b, 0x1031, 0x102c, 0x1015,
0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1016, 0x103b, 0x102c, 0x1038,
0x20, 0x1025, 0x101b, 0x1031, 0x102c, 0x1015, 0x1012, 0x1031, 0x101e, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x1025, 0x101b,
0x1031, 0x102c, 0x1015, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1014,
0x1031, 0x102c, 0x1000, 0x103a, 0x1025, 0x101b, 0x1031, 0x102c, 0x1015, 0x20, 0x1005, 0x1036,
0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1014, 0x1031,
0x102c, 0x1000, 0x103a, 0x1025, 0x101b, 0x1031, 0x102c, 0x1015, 0x20, 0x1014, 0x103d, 0x1031,
0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1016,
0x1031, 0x102c, 0x1037, 0x1000, 0x101c, 0x1014, 0x103a, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a,
0x1038, 0x1005, 0x102f, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1016, 0x1031,
0x102c, 0x1037, 0x1000, 0x101c, 0x1014, 0x103a, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a, 0x1038,
0x1005, 0x102f, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1016, 0x1031, 0x102c, 0x1037, 0x1000, 0x101c, 0x1014, 0x103a, 0x1000, 0x103b,
0x103d, 0x1014, 0x103a, 0x1038, 0x1005, 0x102f, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c,
0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1016, 0x102e, 0x1002,
0x103b, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1016, 0x102e, 0x1002,
0x103b, 0x102e, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1016, 0x102e, 0x1002, 0x103b, 0x102e, 0x20, 0x1014, 0x103d, 0x1031, 0x101b,
0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x103c,
0x1004, 0x103a, 0x101e, 0x1005, 0x103a, 0x20, 0x1002, 0x102e, 0x1021, 0x102c, 0x1038, 0x1014,
0x102c, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x103c, 0x1004,
0x103a, 0x101e, 0x1005, 0x103a, 0x1010, 0x1031, 0x102c, 0x1004, 0x103a, 0x1015, 0x102d, 0x102f,
0x1004, 0x103a, 0x1038, 0x1014, 0x103e, 0x1004, 0x1037, 0x103a, 0x20, 0x1021, 0x1014, 0x1039,
0x1010, 0x102c, 0x1010, 0x102d, 0x1010, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1002, 0x102b, 0x101c, 0x102c, 0x1038, 0x1015, 0x102b, 0x1002, 0x102d, 0x102f, 0x1038,
0x1005, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1002, 0x1019, 0x103a,
0x1018, 0x102e, 0x101a, 0x102c, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1002,
0x103b, 0x1031, 0x102c, 0x103a, 0x1002, 0x103b, 0x102e, 0x101a, 0x102c, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1002, 0x103b, 0x1031, 0x102c, 0x103a, 0x1002, 0x103b, 0x102e,
0x101a, 0x102c, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1002, 0x103b, 0x1031, 0x102c, 0x103a, 0x1002, 0x103b, 0x102e, 0x101a, 0x102c,
0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1002, 0x102e, 0x101c, 0x103a, 0x1018, 0x1010, 0x103a, 0x1000, 0x103b,
0x103d, 0x1014, 0x103a, 0x1038, 0x1005, 0x102f, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1002, 0x101b, 0x1004, 0x103a, 0x1038, 0x1014, 0x1005, 0x103a, 0x20, 0x1005, 0x1036,
0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101b, 0x103e,
0x1031, 0x1037, 0x1002, 0x101b, 0x1004, 0x103a, 0x1038, 0x101c, 0x1014, 0x103a, 0x1038, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1002,
0x101b, 0x1004, 0x103a, 0x1038, 0x101c, 0x1014, 0x103a, 0x1038, 0x20, 0x1005, 0x1036, 0x1010,
0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101b, 0x103e, 0x1031,
0x1037, 0x1002, 0x101b, 0x1004, 0x103a, 0x1038, 0x101c, 0x1014, 0x103a, 0x20, 0x1014, 0x103d,
0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x20,
0x1002, 0x101b, 0x1004, 0x103a, 0x1038, 0x101c, 0x1014, 0x103a, 0x1038, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x20, 0x1002,
0x101b, 0x1004, 0x103a, 0x1038, 0x101c, 0x1014, 0x103a, 0x1038, 0x20, 0x1005, 0x1036, 0x1010,
0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1014, 0x1031, 0x102c,
0x1000, 0x103a, 0x20, 0x1002, 0x101b, 0x1004, 0x103a, 0x1038, 0x101c, 0x1014, 0x103a, 0x20,
0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1005, 0x1036, 0x1010, 0x1031,
0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x1004, 0x103a, 0x101c, 0x101a,
0x103a, 0x1000, 0x103d, 0x1031, 0x1037, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1002, 0x102d, 0x102f, 0x1004, 0x103a, 0x101a, 0x102c, 0x1014, 0x102c, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x101f, 0x102c, 0x101d, 0x102d, 0x102f, 0x1004, 0x103a, 0x101a,
0x102e, 0x20, 0x1021, 0x101a, 0x103a, 0x101c, 0x1030, 0x1038, 0x101b, 0x103e, 0x1014, 0x103a,
0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101f, 0x102c, 0x101d, 0x102d,
0x102f, 0x1004, 0x103a, 0x101a, 0x102e, 0x20, 0x1021, 0x101a, 0x103a, 0x101c, 0x1030, 0x1038,
0x101b, 0x103e, 0x1014, 0x103a, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101f, 0x102c, 0x101d, 0x102d, 0x102f, 0x1004, 0x103a,
0x101a, 0x102e, 0x20, 0x1021, 0x101a, 0x103a, 0x101c, 0x1030, 0x1038, 0x101b, 0x103e, 0x1014,
0x103a, 0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1005,
0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101f, 0x1031,
0x102c, 0x1004, 0x103a, 0x1000, 0x1031, 0x102c, 0x1004, 0x103a, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x101f, 0x1031, 0x102c, 0x1004, 0x103a, 0x1000, 0x1031, 0x102c, 0x1004,
0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x101f, 0x1031, 0x102c, 0x1004, 0x103a, 0x1000, 0x1031, 0x102c, 0x1004, 0x103a, 0x20,
0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x101f, 0x102d, 0x102f, 0x1038, 0x1017, 0x103a, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x101f, 0x102d, 0x102f, 0x1038, 0x1017, 0x103a, 0x20, 0x1005, 0x1036,
0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101f, 0x102d, 0x102f,
0x1038, 0x1017, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102d, 0x1014, 0x1039, 0x1012, 0x102d,
0x101a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1021, 0x102d, 0x1014, 0x1039, 0x1012, 0x102d, 0x101a, 0x101e, 0x1019, 0x102f, 0x1012,
0x1039, 0x1012, 0x101b, 0x102c, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021,
0x1004, 0x103a, 0x1012, 0x102d, 0x102f, 0x1001, 0x103b, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038,
0x1014, 0x102c, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101c,
0x101a, 0x103a, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x20, 0x1021, 0x1004, 0x103a,
0x1012, 0x102d, 0x102f, 0x1014, 0x102e, 0x1038, 0x101b, 0x103e, 0x102c, 0x1038, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1015, 0x102d,
0x102f, 0x1004, 0x103a, 0x1038, 0x20, 0x1021, 0x1004, 0x103a, 0x1012, 0x102d, 0x102f, 0x1014,
0x102e, 0x1038, 0x101b, 0x103e, 0x102c, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a,
0x1038, 0x20, 0x1021, 0x1004, 0x103a, 0x1012, 0x102d, 0x102f, 0x1014, 0x102e, 0x1038, 0x101b,
0x103e, 0x102c, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102e,
0x101b, 0x1014, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102e,
0x101b, 0x1014, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1021, 0x102e, 0x101b, 0x1014, 0x103a, 0x20, 0x1014, 0x103d, 0x1031,
0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021,
0x102e, 0x101b, 0x1030, 0x1001, 0x1030, 0x1010, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1021, 0x102e, 0x101b, 0x1030, 0x1001, 0x1030, 0x1010, 0x103a, 0x20, 0x1005,
0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102e,
0x101b, 0x1030, 0x1001, 0x1030, 0x1010, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c,
0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1005, 0x1039,
0x1005, 0x101b, 0x1031, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021,
0x1005, 0x1039, 0x1005, 0x101b, 0x1031, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c,
0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1005, 0x1039, 0x1005, 0x101b, 0x1031,
0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1002, 0x103b, 0x1015, 0x1014, 0x103a, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1002, 0x103b, 0x1015, 0x1014, 0x103a, 0x20, 0x1005, 0x1036,
0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1002, 0x103b, 0x1015,
0x1014, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1000, 0x102c, 0x1007, 0x1000, 0x103a, 0x1005, 0x1010,
0x1014, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101b, 0x103e,
0x1031, 0x1037, 0x1000, 0x102c, 0x1007, 0x1000, 0x103a, 0x1005, 0x1010, 0x1014, 0x103a, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a,
0x1000, 0x102c, 0x1007, 0x1000, 0x103a, 0x1005, 0x1010, 0x1014, 0x103a, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1000, 0x102d, 0x102f, 0x101b, 0x102e, 0x1038, 0x101a, 0x102c,
0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1000, 0x102d, 0x102f, 0x101b,
0x102e, 0x1038, 0x101a, 0x102c, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1000, 0x102d, 0x102f, 0x101b, 0x102e, 0x1038, 0x101a,
0x102c, 0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1001, 0x102d, 0x102f, 0x1005, 0x103a, 0x101b, 0x102d,
0x102f, 0x1004, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1001, 0x101b,
0x102c, 0x1037, 0x1005, 0x103a, 0x1014, 0x102d, 0x102f, 0x101b, 0x102c, 0x1005, 0x103a, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1001, 0x101b, 0x102c, 0x1037, 0x1005, 0x103a,
0x1014, 0x102d, 0x102f, 0x101b, 0x102c, 0x1005, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031,
0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1001, 0x101b, 0x102c, 0x1037, 0x1005,
0x103a, 0x1014, 0x102d, 0x102f, 0x101b, 0x102c, 0x1005, 0x103a, 0x20, 0x1014, 0x103d, 0x1031,
0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1000,
0x102c, 0x1002, 0x103b, 0x1005, 0x1039, 0x1005, 0x1010, 0x1014, 0x103a, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x101e, 0x102e, 0x101b, 0x102d, 0x101c, 0x1004, 0x103a, 0x1039,
0x1000, 0x102c, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101c, 0x102d, 0x102f,
0x1004, 0x103a, 0x1038, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x1005, 0x102f, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101c, 0x1031, 0x102c, 0x1037, 0x1012, 0x103a,
0x101f, 0x1031, 0x102c, 0x1004, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x101c, 0x1031, 0x102c, 0x1037, 0x1012, 0x103a, 0x101f, 0x1031, 0x102c, 0x1004, 0x103a, 0x20,
0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101c,
0x1031, 0x102c, 0x1037, 0x1012, 0x103a, 0x101f, 0x1031, 0x102c, 0x1004, 0x103a, 0x20, 0x1014,
0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1019, 0x1000, 0x102c, 0x1021, 0x102d, 0x102f, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1019, 0x1000, 0x102c, 0x1021, 0x102d, 0x102f, 0x20, 0x1005, 0x1036, 0x1010,
0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x1000, 0x102c, 0x1021,
0x102d, 0x102f, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x102c, 0x1002, 0x102b, 0x1012, 0x1014, 0x103a,
0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x102c, 0x1002, 0x102b,
0x1012, 0x1014, 0x103a, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x102c, 0x1002, 0x102b, 0x1012, 0x1014, 0x103a, 0x1038,
0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1019, 0x101c, 0x1031, 0x1038, 0x101b, 0x103e, 0x102c, 0x1038, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x1031, 0x102c, 0x103a, 0x101c, 0x1012,
0x102d, 0x102f, 0x1000, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019,
0x102c, 0x1001, 0x1031, 0x1038, 0x1021, 0x1015, 0x103a, 0x1005, 0x103a, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x102c, 0x101b, 0x103e, 0x101a, 0x103a, 0x1000, 0x103b,
0x103d, 0x1014, 0x103a, 0x1038, 0x1005, 0x102f, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1019, 0x1031, 0x102c, 0x101b, 0x1005, 0x103a, 0x101b, 0x103e, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x1031, 0x102c, 0x101b, 0x1005, 0x103a, 0x101b, 0x103e,
0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1019, 0x1031, 0x102c, 0x101b, 0x1005, 0x103a, 0x101b, 0x103e, 0x20, 0x1014, 0x103d, 0x1031,
0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019,
0x1031, 0x102c, 0x103a, 0x1005, 0x103d, 0x1014, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1019, 0x1000, 0x1039, 0x1000, 0x1006, 0x102e, 0x1000, 0x1014, 0x103a, 0x20,
0x1015, 0x1005, 0x102d, 0x1016, 0x102d, 0x1010, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1019, 0x1000, 0x1039, 0x1000, 0x1006, 0x102e, 0x1000, 0x1014, 0x103a, 0x20,
0x1015, 0x1005, 0x102d, 0x1016, 0x102d, 0x1010, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031,
0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x1000, 0x1039, 0x1000, 0x1005,
0x102e, 0x1000, 0x1014, 0x103a, 0x20, 0x1015, 0x1005, 0x102d, 0x1016, 0x102d, 0x1010, 0x103a,
0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1005, 0x1036, 0x1010,
0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1025, 0x101c, 0x1014, 0x103a,
0x1018, 0x102c, 0x1010, 0x1031, 0x102c, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1025, 0x101c, 0x1014, 0x103a, 0x1018, 0x102c, 0x1010, 0x1031, 0x102c, 0x20, 0x1005, 0x1036,
0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1025, 0x101c, 0x1014,
0x103a, 0x1018, 0x102c, 0x1010, 0x1031, 0x102c, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c,
0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x1031, 0x102c,
0x103a, 0x1005, 0x1000, 0x102d, 0x102f, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1019, 0x1031, 0x102c, 0x103a, 0x1005, 0x1000, 0x102d, 0x102f, 0x20, 0x1005, 0x1036, 0x1010,
0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x1031, 0x102c, 0x103a,
0x1005, 0x1000, 0x102d, 0x102f, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x103c, 0x1014, 0x103a, 0x1019,
0x102c, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x102c, 0x1025, 0x1030,
0x101b, 0x1030, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x102e, 0x1015,
0x1031, 0x102b, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x101a, 0x1030,
0x1038, 0x1000, 0x101a, 0x103a, 0x101c, 0x102e, 0x1012, 0x102d, 0x102f, 0x1038, 0x1014, 0x102e,
0x1038, 0x101a, 0x102c, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014,
0x101a, 0x1030, 0x1038, 0x1000, 0x102c, 0x101c, 0x102e, 0x1012, 0x102d, 0x102f, 0x1038, 0x1014,
0x102e, 0x1038, 0x101a, 0x102c, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x101a, 0x1030, 0x1038, 0x1000, 0x102c, 0x101c,
0x102e, 0x1012, 0x102d, 0x102f, 0x1038, 0x1014, 0x102e, 0x1038, 0x101a, 0x102c, 0x1038, 0x20,
0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1014, 0x101a, 0x1030, 0x1038, 0x1007, 0x102e, 0x101c, 0x1014, 0x103a, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x101a, 0x1030, 0x1038, 0x1007, 0x102e,
0x101c, 0x1014, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1014, 0x101a, 0x1030, 0x1038, 0x1007, 0x102e, 0x101c, 0x1014, 0x103a,
0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1014, 0x101a, 0x1030, 0x1038, 0x1016, 0x1031, 0x102c, 0x1004, 0x103a,
0x101c, 0x1014, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x101a,
0x1030, 0x1038, 0x1016, 0x1031, 0x102c, 0x1004, 0x103a, 0x101c, 0x1014, 0x103a, 0x20, 0x1005,
0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x101a,
0x1030, 0x1038, 0x1016, 0x1031, 0x102c, 0x1004, 0x103a, 0x101c, 0x1014, 0x103a, 0x20, 0x1014,
0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c,
0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x102e, 0x1025, 0x1030, 0x1021, 0x1031,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x1031, 0x102c, 0x1016, 0x1031,
0x102c, 0x1037, 0x1001, 0x103a, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x1031, 0x102c, 0x1016, 0x1031, 0x102c, 0x1037,
0x1001, 0x103a, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x20, 0x1005, 0x1036, 0x1010,
0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x1031, 0x102c, 0x1016,
0x1031, 0x102c, 0x1037, 0x1001, 0x103a, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x20,
0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1005, 0x1036, 0x1010, 0x1031,
0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1016, 0x102c, 0x1014, 0x1014, 0x103a,
0x1012, 0x102d, 0x102f, 0x1038, 0x1012, 0x102e, 0x1014, 0x102d, 0x102f, 0x1038, 0x101b, 0x102d,
0x102f, 0x1038, 0x100a, 0x102c, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1016,
0x102c, 0x1014, 0x1014, 0x103a, 0x1012, 0x102d, 0x102f, 0x1038, 0x1012, 0x102e, 0x1014, 0x102d,
0x102f, 0x1038, 0x101b, 0x102d, 0x102f, 0x1038, 0x100a, 0x102c, 0x20, 0x1005, 0x1036, 0x1010,
0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1016, 0x102c, 0x1014, 0x1014,
0x103a, 0x1012, 0x102d, 0x102f, 0x1038, 0x1012, 0x102e, 0x1014, 0x102d, 0x102f, 0x1038, 0x101b,
0x102d, 0x102f, 0x1038, 0x100a, 0x102c, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e,
0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x102d, 0x102f, 0x1017,
0x102d, 0x102f, 0x1005, 0x1032, 0x1018, 0x102e, 0x1021, 0x1032, 0x101a, 0x103a, 0x1005, 0x103a,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x102d, 0x102f, 0x1017, 0x102d,
0x102f, 0x1005, 0x1032, 0x1018, 0x102e, 0x1021, 0x1032, 0x101a, 0x103a, 0x1005, 0x103a, 0x20,
0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014,
0x102d, 0x102f, 0x1017, 0x102d, 0x102f, 0x1005, 0x1032, 0x1018, 0x102e, 0x1021, 0x1032, 0x101a,
0x103a, 0x1005, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x103d, 0x1019, 0x103a, 0x1038, 0x1005,
0x103a, 0x1001, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x103d,
0x1019, 0x103a, 0x1038, 0x1005, 0x103a, 0x1001, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031,
0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x103d, 0x1019, 0x103a, 0x1038,
0x1005, 0x103a, 0x1001, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x102b, 0x1000, 0x1005, 0x1039,
0x1005, 0x1010, 0x1014, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015,
0x102b, 0x1000, 0x1005, 0x1039, 0x1005, 0x1010, 0x1014, 0x103a, 0x20, 0x1005, 0x1036, 0x1010,
0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x102b, 0x1000, 0x1005,
0x1039, 0x1005, 0x1010, 0x1014, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e,
0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x101c, 0x102c, 0x1021,
0x102d, 0x102f, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x102b, 0x1015,
0x1030, 0x1021, 0x102c, 0x1014, 0x101a, 0x1030, 0x1038, 0x1002, 0x102e, 0x1014, 0x102e, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x102b, 0x101b, 0x102c, 0x1002, 0x103d,
0x1031, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x102b, 0x101b,
0x102c, 0x1002, 0x103d, 0x1031, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x102b, 0x101b, 0x102c, 0x1002, 0x103d, 0x1031,
0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x102e, 0x101b, 0x1030, 0x1038, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x102e, 0x101b, 0x1030, 0x1038, 0x20, 0x1005, 0x1036,
0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x102e, 0x101b,
0x1030, 0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1016, 0x102d, 0x101c, 0x1005, 0x103a, 0x1015, 0x102d,
0x102f, 0x1004, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1016, 0x102d,
0x101c, 0x1005, 0x103a, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x20, 0x1005, 0x1036, 0x1010,
0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1016, 0x102d, 0x101c, 0x1005,
0x103a, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c,
0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1016, 0x102e, 0x1038,
0x1014, 0x1005, 0x103a, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x1005, 0x102f, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1005, 0x102d, 0x1014, 0x1037, 0x103a, 0x1015,
0x102e, 0x1021, 0x1032, 0x1014, 0x103e, 0x1004, 0x1037, 0x103a, 0x1019, 0x102e, 0x1001, 0x103d,
0x102e, 0x101c, 0x103d, 0x1014, 0x103a, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1005,
0x102d, 0x1014, 0x1037, 0x103a, 0x1015, 0x102e, 0x1021, 0x1032, 0x1014, 0x103e, 0x1004, 0x1037,
0x103a, 0x1019, 0x102e, 0x1001, 0x103d, 0x102e, 0x101c, 0x103d, 0x1014, 0x103a, 0x1005, 0x1036,
0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1005, 0x102d, 0x1014,
0x1037, 0x103a, 0x1015, 0x102e, 0x1021, 0x1032, 0x1014, 0x103e, 0x1004, 0x1037, 0x103a, 0x20,
0x1019, 0x102e, 0x1001, 0x103d, 0x102e, 0x101c, 0x103d, 0x1014, 0x103a, 0x20, 0x1014, 0x103d,
0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x102b, 0x1010, 0x103a, 0x1000, 0x101a, 0x103a,
0x101b, 0x1004, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1016, 0x102d,
0x102f, 0x1014, 0x102c, 0x1016, 0x1032, 0x1021, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1015, 0x103c, 0x102f, 0x1036, 0x101a, 0x1014, 0x103a, 0x1038, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101b, 0x102e, 0x101a, 0x1030, 0x1014, 0x102e, 0x101a,
0x1036, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101b, 0x102d, 0x102f, 0x101e,
0x102e, 0x101b, 0x102c, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1006, 0x102c,
0x1001, 0x102b, 0x101c, 0x1004, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1006, 0x102c, 0x1001, 0x102b, 0x101c, 0x1004, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031,
0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1006, 0x102c, 0x1001, 0x102b, 0x101c,
0x1004, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1006, 0x1019, 0x102d, 0x102f, 0x1021, 0x102c, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1006, 0x1019, 0x102d, 0x102f, 0x1021, 0x102c,
0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1006, 0x1019, 0x102d, 0x102f, 0x1038, 0x1021, 0x102c, 0x1038, 0x20, 0x1014, 0x103d, 0x1031,
0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1006,
0x1031, 0x1038, 0x101b, 0x103e, 0x1032, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1005, 0x1004, 0x103a, 0x1039, 0x1000, 0x102c, 0x1015, 0x1030, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1006, 0x1031, 0x102c, 0x103a, 0x101c, 0x1019, 0x103d, 0x1014, 0x103a,
0x1000, 0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x1005, 0x102f, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1010, 0x1031, 0x102c, 0x1004, 0x103a, 0x1002, 0x103b, 0x1031, 0x102c,
0x103a, 0x1002, 0x103b, 0x102e, 0x101a, 0x102c, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1005, 0x1030, 0x1038, 0x101b, 0x102e, 0x1014, 0x102c, 0x1019, 0x103a, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x101b, 0x103e, 0x102d, 0x102f, 0x101d, 0x102b, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1010, 0x101f, 0x102e, 0x1010, 0x102e, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1011, 0x102d, 0x102f, 0x1004, 0x103a, 0x1015, 0x1031,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1011, 0x102d, 0x102f, 0x1004, 0x103a,
0x1015, 0x1031, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1011, 0x102d, 0x102f, 0x1004, 0x103a, 0x1015, 0x1031, 0x20, 0x1014, 0x103d,
0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1010, 0x102c, 0x1002, 0x103b, 0x1005, 0x103a, 0x1000, 0x1005, 0x1039, 0x1005, 0x1010, 0x1014,
0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1010, 0x102d, 0x102f, 0x1000,
0x102e, 0x101c, 0x102c, 0x1025, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1010,
0x103d, 0x1014, 0x103a, 0x1002, 0x102b, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1010, 0x103d, 0x1014, 0x103a, 0x1002, 0x102b, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c,
0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1010, 0x103d, 0x1014, 0x103a, 0x1002, 0x102b,
0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1001, 0x103b, 0x102f, 0x1001, 0x103a, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1010, 0x102c, 0x1037, 0x1001, 0x103a, 0x1019, 0x1004, 0x103a, 0x1014,
0x1005, 0x1039, 0x1005, 0x1010, 0x1014, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1010, 0x102c, 0x1037, 0x1001, 0x103a, 0x1019, 0x1004, 0x103a, 0x1014, 0x1005, 0x1039,
0x1005, 0x1010, 0x1014, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1010, 0x102c, 0x1037, 0x1001, 0x103a, 0x1019, 0x1004, 0x103a,
0x1014, 0x1005, 0x1039, 0x1005, 0x1010, 0x1014, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b,
0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1010, 0x1030,
0x1017, 0x102c, 0x1038, 0x101c, 0x1030, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1025, 0x101b, 0x102f, 0x1002, 0x103d, 0x1031, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1025, 0x101b, 0x102f, 0x1002, 0x103d, 0x1031, 0x1038, 0x20, 0x1005, 0x1036,
0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1025, 0x101b, 0x102f,
0x1002, 0x103d, 0x1031, 0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1025, 0x1007, 0x1018, 0x1000, 0x103a,
0x1000, 0x1005, 0x1039, 0x1005, 0x1010, 0x1014, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1025, 0x1007, 0x1018, 0x1000, 0x103a, 0x1000, 0x1005, 0x1039, 0x1005, 0x1010,
0x1014, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1025, 0x1007, 0x1018, 0x1000, 0x103a, 0x1000, 0x1005, 0x1039, 0x1005, 0x1010,
0x1014, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1017, 0x1014, 0x103d, 0x102c, 0x1038, 0x1010, 0x1030,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1017, 0x1014, 0x103d, 0x102c, 0x1038,
0x1010, 0x1030, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1017, 0x1014, 0x103d, 0x102c, 0x1038, 0x1010, 0x1030, 0x20, 0x1014, 0x103d,
0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1017, 0x1004, 0x103a, 0x1014, 0x102e, 0x1007, 0x103d, 0x1032, 0x101c, 0x102c, 0x1038, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1017, 0x101c, 0x102c, 0x1012, 0x102e, 0x1017,
0x1031, 0x102c, 0x1037, 0x1005, 0x1010, 0x1031, 0x102c, 0x1037, 0x1001, 0x103a, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1017, 0x101c, 0x102c, 0x1012, 0x102e, 0x1017, 0x1031,
0x102c, 0x1037, 0x1005, 0x1010, 0x1031, 0x102c, 0x1037, 0x1001, 0x103a, 0x20, 0x1005, 0x1036,
0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1017, 0x101c, 0x102c,
0x1012, 0x102e, 0x1017, 0x1031, 0x102c, 0x1037, 0x1005, 0x1010, 0x1031, 0x102c, 0x1037, 0x1001,
0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1017, 0x102d, 0x102f, 0x101c, 0x103a, 0x1002, 0x102d, 0x102f,
0x1002, 0x101b, 0x1000, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1017,
0x102d, 0x102f, 0x101c, 0x103a, 0x1002, 0x102d, 0x102f, 0x1002, 0x101b, 0x1000, 0x103a, 0x20,
0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1017,
0x102d, 0x102f, 0x1002, 0x102d, 0x102f, 0x1002, 0x101b, 0x1000, 0x103a, 0x20, 0x1014, 0x103d,
0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1017, 0x102d, 0x102f, 0x1005, 0x103a, 0x1010, 0x102d, 0x102f, 0x1001, 0x103a, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101d, 0x102d, 0x1010, 0x103a, 0x1001, 0x103a, 0x1000,
0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x101d, 0x1031, 0x102b, 0x101c, 0x102e, 0x1005, 0x103a, 0x1014, 0x103e, 0x1004, 0x1037, 0x103a,
0x20, 0x1016, 0x1030, 0x1000, 0x103b, 0x1030, 0x1014, 0x102c, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x101a, 0x1030, 0x1001, 0x1030, 0x1038, 0x1010, 0x103a, 0x1005, 0x103a,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101a, 0x1030, 0x1001, 0x1030, 0x1038,
0x1010, 0x103a, 0x1005, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x101a, 0x1030, 0x1001, 0x1030, 0x1038, 0x1010, 0x103a, 0x1005,
0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x101b, 0x101a, 0x103a, 0x1001, 0x102b, 0x1038, 0x1010, 0x102e,
0x101b, 0x1004, 0x103a, 0x1018, 0x102c, 0x1038, 0x1001, 0x103a, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x101b, 0x101a, 0x103a, 0x1001, 0x102b, 0x1038, 0x1010, 0x102e, 0x101b,
0x1004, 0x103a, 0x1018, 0x102c, 0x1038, 0x1001, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031,
0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101b, 0x101a, 0x103a, 0x1001, 0x102b,
0x1038, 0x1010, 0x102e, 0x101b, 0x1004, 0x103a, 0x1018, 0x102c, 0x1001, 0x103a, 0x20, 0x1014,
0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x101a, 0x1030, 0x1000, 0x103d, 0x1014, 0x103a, 0x1038, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x827e, 0x514b, 0x6642, 0x9593, 0x827e, 0x514b, 0x6a19, 0x6e96, 0x6642,
0x9593, 0x827e, 0x514b, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x963f, 0x5bcc, 0x6c57, 0x6642, 0x9593,
0x4e2d, 0x975e, 0x6642, 0x9593, 0x6771, 0x975e, 0x6642, 0x9593, 0x5357, 0x975e, 0x6a19, 0x6e96,
0x6642, 0x9593, 0x897f, 0x975e, 0x6642, 0x9593, 0x897f, 0x975e, 0x6a19, 0x6e96, 0x6642, 0x9593,
0x897f, 0x975e, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x963f, 0x62c9, 0x65af, 0x52a0, 0x6642, 0x9593,
0x963f, 0x62c9, 0x65af, 0x52a0, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x963f, 0x62c9, 0x65af, 0x52a0,
0x590f, 0x4ee4, 0x6642, 0x9593, 0x963f, 0x62c9, 0x6728, 0x5716, 0x6642, 0x9593, 0x963f, 0x62c9,
0x6728, 0x5716, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x963f, 0x62c9, 0x6728, 0x5716, 0x590f, 0x4ee4,
0x6642, 0x9593, 0x4e9e, 0x99ac, 0x905c, 0x6642, 0x9593, 0x4e9e, 0x99ac, 0x905c, 0x6a19, 0x6e96,
0x6642, 0x9593, 0x4e9e, 0x99ac, 0x905c, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x4e2d, 0x90e8, 0x6642,
0x9593, 0x4e2d, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x4e2d, 0x90e8, 0x590f, 0x4ee4, 0x6642,
0x9593, 0x6771, 0x90e8, 0x6642, 0x9593, 0x6771, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x6771,
0x90e8, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x5c71, 0x5340, 0x6642, 0x9593, 0x5c71, 0x5340, 0x6a19,
0x6e96, 0x6642, 0x9593, 0x5c71, 0x5340, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x592a, 0x5e73, 0x6d0b,
0x6642, 0x9593, 0x592a, 0x5e73, 0x6d0b, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x592a, 0x5e73, 0x6d0b,
0x590f, 0x4ee4, 0x6642, 0x9593, 0x963f, 0x7d0d, 0x5fb7, 0x723e, 0x6642, 0x9593, 0x963f, 0x90a3,
0x5e95, 0x6cb3, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x963f, 0x90a3, 0x5e95, 0x6cb3, 0x590f, 0x4ee4,
0x6642, 0x9593, 0x963f, 0x76ae, 0x4e9e, 0x6642, 0x9593, 0x963f, 0x76ae, 0x4e9e, 0x6a19, 0x6e96,
0x6642, 0x9593, 0x963f, 0x76ae, 0x4e9e, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x963f, 0x514b, 0x9676,
0x6642, 0x9593, 0x963f, 0x514b, 0x9676, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x963f, 0x514b, 0x9676,
0x590f, 0x4ee4, 0x6642, 0x9593, 0x963f, 0x514b, 0x6258, 0x6bd4, 0x6642, 0x9593, 0x963f, 0x514b,
0x6258, 0x6bd4, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x963f, 0x514b, 0x6258, 0x6bd4, 0x590f, 0x4ee4,
0x6642, 0x9593, 0x963f, 0x62c9, 0x4f2f, 0x6642, 0x9593, 0x963f, 0x62c9, 0x4f2f, 0x6a19, 0x6e96,
0x6642, 0x9593, 0x963f, 0x62c9, 0x4f2f, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x963f, 0x6839, 0x5ef7,
0x6642, 0x9593, 0x963f, 0x6839, 0x5ef7, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x963f, 0x6839, 0x5ef7,
0x590f, 0x4ee4, 0x6642, 0x9593, 0x963f, 0x6839, 0x5ef7, 0x897f, 0x90e8, 0x6642, 0x9593, 0x963f,
0x6839, 0x5ef7, 0x897f, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x963f, 0x6839, 0x5ef7, 0x897f,
0x90e8, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x4e9e, 0x7f8e, 0x5c3c, 0x4e9e, 0x6642, 0x9593, 0x4e9e,
0x7f8e, 0x5c3c, 0x4e9e, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x4e9e, 0x7f8e, 0x5c3c, 0x4e9e, 0x590f,
0x4ee4, 0x6642, 0x9593, 0x5927, 0x897f, 0x6d0b, 0x6642, 0x9593, 0x5927, 0x897f, 0x6d0b, 0x6a19,
0x6e96, 0x6642, 0x9593, 0x5927, 0x897f, 0x6d0b, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x6fb3, 0x6d32,
0x4e2d, 0x90e8, 0x6642, 0x9593, 0x6fb3, 0x6d32, 0x4e2d, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x9593,
0x6fb3, 0x6d32, 0x4e2d, 0x90e8, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x6fb3, 0x6d32, 0x4e2d, 0x897f,
0x90e8, 0x6642, 0x9593, 0x6fb3, 0x6d32, 0x4e2d, 0x897f, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x9593,
0x6fb3, 0x6d32, 0x4e2d, 0x897f, 0x90e8, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x6fb3, 0x6d32, 0x6771,
0x90e8, 0x6642, 0x9593, 0x6fb3, 0x6d32, 0x6771, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x6fb3,
0x6d32, 0x6771, 0x90e8, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x6fb3, 0x6d32, 0x897f, 0x90e8, 0x6642,
0x9593, 0x6fb3, 0x6d32, 0x897f, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x6fb3, 0x6d32, 0x897f,
0x90e8, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x4e9e, 0x585e, 0x62dc, 0x7136, 0x6642, 0x9593, 0x4e9e,
0x585e, 0x62dc, 0x7136, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x4e9e, 0x585e, 0x62dc, 0x7136, 0x590f,
0x4ee4, 0x6642, 0x9593, 0x4e9e, 0x901f, 0x723e, 0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x4e9e, 0x901f,
0x723e, 0x7fa4, 0x5cf6, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x4e9e, 0x901f, 0x723e, 0x7fa4, 0x5cf6,
0x590f, 0x4ee4, 0x6642, 0x9593, 0x5b5f, 0x52a0, 0x62c9, 0x6642, 0x9593, 0x5b5f, 0x52a0, 0x62c9,
0x6a19, 0x6e96, 0x6642, 0x9593, 0x5b5f, 0x52a0, 0x62c9, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x4e0d,
0x4e39, 0x6642, 0x9593, 0x73bb, 0x5229, 0x7dad, 0x4e9e, 0x6642, 0x9593, 0x5df4, 0x897f, 0x5229,
0x4e9e, 0x6642, 0x9593, 0x5df4, 0x897f, 0x5229, 0x4e9e, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x5df4,
0x897f, 0x5229, 0x4e9e, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x6c76, 0x840a, 0x6642, 0x9593, 0x7dad,
0x5fb7, 0x89d2, 0x6642, 0x9593, 0x7dad, 0x5fb7, 0x89d2, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x7dad,
0x5fb7, 0x89d2, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x51f1, 0x897f, 0x7ad9, 0x6642, 0x9593, 0x67e5,
0x83ab, 0x6d1b, 0x6642, 0x9593, 0x67e5, 0x5766, 0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x67e5, 0x5766,
0x7fa4, 0x5cf6, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x67e5, 0x5766, 0x7fa4, 0x5cf6, 0x590f, 0x4ee4,
0x6642, 0x9593, 0x667a, 0x5229, 0x6642, 0x9593, 0x667a, 0x5229, 0x6a19, 0x6e96, 0x6642, 0x9593,
0x667a, 0x5229, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x4e2d, 0x570b, 0x6642, 0x9593, 0x4e2d, 0x570b,
0x6a19, 0x6e96, 0x6642, 0x9593, 0x4e2d, 0x570b, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x8056, 0x8a95,
0x5cf6, 0x6642, 0x9593, 0x79d1, 0x79d1, 0x65af, 0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x54e5, 0x502b,
0x6bd4, 0x4e9e, 0x6642, 0x9593, 0x54e5, 0x502b, 0x6bd4, 0x4e9e, 0x6a19, 0x6e96, 0x6642, 0x9593,
0x54e5, 0x502b, 0x6bd4, 0x4e9e, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x5eab, 0x514b, 0x7fa4, 0x5cf6,
0x6642, 0x9593, 0x5eab, 0x514b, 0x7fa4, 0x5cf6, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x5eab, 0x514b,
0x7fa4, 0x5cf6, 0x534a, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x53e4, 0x5df4, 0x6642, 0x9593, 0x53e4,
0x5df4, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x53e4, 0x5df4, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x6234,
0x7dad, 0x65af, 0x6642, 0x9593, 0x675c, 0x8499, 0x675c, 0x6bd4, 0x723e, 0x6642, 0x9593, 0x6771,
0x5e1d, 0x6c76, 0x6642, 0x9593, 0x5fa9, 0x6d3b, 0x7bc0, 0x5cf6, 0x6642, 0x9593, 0x5fa9, 0x6d3b,
0x7bc0, 0x5cf6, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x5fa9, 0x6d3b, 0x7bc0, 0x5cf6, 0x590f, 0x4ee4,
0x6642, 0x9593, 0x5384, 0x74dc, 0x591a, 0x6642, 0x9593, 0x4e2d, 0x6b50, 0x6642, 0x9593, 0x4e2d,
0x6b50, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x4e2d, 0x6b50, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x6771,
0x6b50, 0x6642, 0x9593, 0x6771, 0x6b50, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x6771, 0x6b50, 0x590f,
0x4ee4, 0x6642, 0x9593, 0x6b50, 0x6d32, 0x9060, 0x6771, 0x6642, 0x9593, 0x897f, 0x6b50, 0x6642,
0x9593, 0x897f, 0x6b50, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x897f, 0x6b50, 0x590f, 0x4ee4, 0x6642,
0x9593, 0x798f, 0x514b, 0x862d, 0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x798f, 0x514b, 0x862d, 0x7fa4,
0x5cf6, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x798f, 0x514b, 0x862d, 0x7fa4, 0x5cf6, 0x590f, 0x4ee4,
0x6642, 0x9593, 0x6590, 0x6fdf, 0x6642, 0x9593, 0x6590, 0x6fdf, 0x6a19, 0x6e96, 0x6642, 0x9593,
0x6590, 0x6fdf, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x6cd5, 0x5c6c, 0x572d, 0x4e9e, 0x90a3, 0x6642,
0x9593, 0x6cd5, 0x570b, 0x5357, 0x65b9, 0x53ca, 0x5357, 0x6975, 0x6642, 0x9593, 0x52a0, 0x62c9,
0x5df4, 0x54e5, 0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x7518, 0x6bd4, 0x723e, 0x7fa4, 0x5cf6, 0x6642,
0x9593, 0x55ac, 0x6cbb, 0x4e9e, 0x6642, 0x9593, 0x55ac, 0x6cbb, 0x4e9e, 0x6a19, 0x6e96, 0x6642,
0x9593, 0x55ac, 0x6cbb, 0x4e9e, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x5409, 0x723e, 0x4f2f, 0x7279,
0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x683c, 0x6797, 0x5a01, 0x6cbb, 0x6a19, 0x6e96, 0x6642, 0x9593,
0x683c, 0x9675, 0x862d, 0x6771, 0x90e8, 0x6642, 0x9593, 0x683c, 0x9675, 0x862d, 0x6771, 0x90e8,
0x6a19, 0x6e96, 0x6642, 0x9593, 0x683c, 0x9675, 0x862d, 0x6771, 0x90e8, 0x590f, 0x4ee4, 0x6642,
0x9593, 0x683c, 0x9675, 0x862d, 0x897f, 0x90e8, 0x6642, 0x9593, 0x683c, 0x9675, 0x862d, 0x897f,
0x90e8, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x683c, 0x9675, 0x862d, 0x897f, 0x90e8, 0x590f, 0x4ee4,
0x6642, 0x9593, 0x95dc, 0x5cf6, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x6ce2, 0x65af, 0x7063, 0x6d77,
0x57df, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x84cb, 0x4e9e, 0x90a3, 0x6642, 0x9593, 0x590f, 0x5a01,
0x5937, 0x2d, 0x963f, 0x7559, 0x7533, 0x6642, 0x9593, 0x590f, 0x5a01, 0x5937, 0x2d, 0x963f,
0x7559, 0x7533, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x590f, 0x5a01, 0x5937, 0x2d, 0x963f, 0x7559,
0x7533, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x9999, 0x6e2f, 0x6642, 0x9593, 0x9999, 0x6e2f, 0x6a19,
0x6e96, 0x6642, 0x9593, 0x9999, 0x6e2f, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x79d1, 0x5e03, 0x591a,
0x6642, 0x9593, 0x79d1, 0x5e03, 0x591a, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x79d1, 0x5e03, 0x591a,
0x590f, 0x4ee4, 0x6642, 0x9593, 0x5370, 0x5ea6, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x5370, 0x5ea6,
0x6d0b, 0x6642, 0x9593, 0x5370, 0x5ea6, 0x652f, 0x90a3, 0x6642, 0x9593, 0x5370, 0x5c3c, 0x4e2d,
0x90e8, 0x6642, 0x9593, 0x5370, 0x5c3c, 0x6771, 0x90e8, 0x6642, 0x9593, 0x5370, 0x5c3c, 0x897f,
0x90e8, 0x6642, 0x9593, 0x4f0a, 0x6717, 0x6642, 0x9593, 0x4f0a, 0x6717, 0x6a19, 0x6e96, 0x6642,
0x9593, 0x4f0a, 0x6717, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x4f0a, 0x723e, 0x5eab, 0x6b21, 0x514b,
0x6642, 0x9593, 0x4f0a, 0x723e, 0x5eab, 0x6b21, 0x514b, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x4f0a,
0x723e, 0x5eab, 0x6b21, 0x514b, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x4ee5, 0x8272, 0x5217, 0x6642,
0x9593, 0x4ee5, 0x8272, 0x5217, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x4ee5, 0x8272, 0x5217, 0x590f,
0x4ee4, 0x6642, 0x9593, 0x65e5, 0x672c, 0x6642, 0x9593, 0x65e5, 0x672c, 0x6a19, 0x6e96, 0x6642,
0x9593, 0x65e5, 0x672c, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x5f7c, 0x5f97, 0x7f85, 0x5df4, 0x752b,
0x6d1b, 0x592b, 0x65af, 0x514b, 0x6642, 0x9593, 0x5f7c, 0x5f97, 0x7f85, 0x5df4, 0x752b, 0x6d1b,
0x592b, 0x65af, 0x514b, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x5f7c, 0x5f97, 0x7f85, 0x5df4, 0x752b,
0x6d1b, 0x592b, 0x65af, 0x514b, 0x65e5, 0x5149, 0x7bc0, 0x7d04, 0x6642, 0x9593, 0x54c8, 0x85a9,
0x514b, 0x6642, 0x9593, 0x6771, 0x54c8, 0x85a9, 0x514b, 0x6642, 0x9593, 0x897f, 0x54c8, 0x85a9,
0x514b, 0x6642, 0x9593, 0x97d3, 0x570b, 0x6642, 0x9593, 0x97d3, 0x570b, 0x6a19, 0x6e96, 0x6642,
0x9593, 0x97d3, 0x570b, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x79d1, 0x65af, 0x745e, 0x6642, 0x9593,
0x514b, 0x62c9, 0x65af, 0x8afe, 0x4e9e, 0x723e, 0x65af, 0x514b, 0x6642, 0x9593, 0x514b, 0x62c9,
0x65af, 0x8afe, 0x4e9e, 0x723e, 0x65af, 0x514b, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x514b, 0x62c9,
0x65af, 0x8afe, 0x4e9e, 0x723e, 0x65af, 0x514b, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x5409, 0x723e,
0x5409, 0x65af, 0x6642, 0x9593, 0x862d, 0x5361, 0x6642, 0x9593, 0x840a, 0x6069, 0x7fa4, 0x5cf6,
0x6642, 0x9593, 0x8c6a, 0x52f3, 0x7235, 0x5cf6, 0x6642, 0x9593, 0x8c6a, 0x52f3, 0x7235, 0x5cf6,
0x6a19, 0x6e96, 0x6642, 0x9593, 0x8c6a, 0x52f3, 0x7235, 0x5cf6, 0x590f, 0x4ee4, 0x6642, 0x9593,
0x6fb3, 0x9580, 0x6642, 0x9593, 0x6fb3, 0x9580, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x6fb3, 0x9580,
0x590f, 0x4ee4, 0x6642, 0x9593, 0x99ac, 0x52a0, 0x4e39, 0x6642, 0x9593, 0x99ac, 0x52a0, 0x4e39,
0x6a19, 0x6e96, 0x6642, 0x9593, 0x99ac, 0x52a0, 0x4e39, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x99ac,
0x4f86, 0x897f, 0x4e9e, 0x6642, 0x9593, 0x99ac, 0x723e, 0x5730, 0x592b, 0x6642, 0x9593, 0x99ac,
0x53ef, 0x85a9, 0x65af, 0x6642, 0x9593, 0x99ac, 0x7d39, 0x723e, 0x7fa4, 0x5cf6, 0x6642, 0x9593,
0x6a21, 0x91cc, 0x897f, 0x65af, 0x6642, 0x9593, 0x6a21, 0x91cc, 0x897f, 0x65af, 0x6a19, 0x6e96,
0x6642, 0x9593, 0x6a21, 0x91cc, 0x897f, 0x65af, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x83ab, 0x68ee,
0x6642, 0x9593, 0x58a8, 0x897f, 0x54e5, 0x592a, 0x5e73, 0x6d0b, 0x6642, 0x9593, 0x58a8, 0x897f,
0x54e5, 0x592a, 0x5e73, 0x6d0b, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x58a8, 0x897f, 0x54e5, 0x592a,
0x5e73, 0x6d0b, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x70cf, 0x862d, 0x5df4, 0x6258, 0x6642, 0x9593,
0x70cf, 0x862d, 0x5df4, 0x6258, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x70cf, 0x862d, 0x5df4, 0x6258,
0x590f, 0x4ee4, 0x6642, 0x9593, 0x83ab, 0x65af, 0x79d1, 0x6642, 0x9593, 0x83ab, 0x65af, 0x79d1,
0x6a19, 0x6e96, 0x6642, 0x9593, 0x83ab, 0x65af, 0x79d1, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x7dec,
0x7538, 0x6642, 0x9593, 0x8afe, 0x9b6f, 0x6642, 0x9593, 0x5c3c, 0x6cca, 0x723e, 0x6642, 0x9593,
0x65b0, 0x5580, 0x91cc, 0x591a, 0x5c3c, 0x4e9e, 0x6642, 0x9593, 0x65b0, 0x5580, 0x91cc, 0x591a,
0x5c3c, 0x4e9e, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x65b0, 0x5580, 0x91cc, 0x591a, 0x5c3c, 0x4e9e,
0x7fa4, 0x5cf6, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x7d10, 0x897f, 0x862d, 0x6642, 0x9593, 0x7d10,
0x897f, 0x862d, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x7d10, 0x897f, 0x862d, 0x590f, 0x4ee4, 0x6642,
0x9593, 0x7d10, 0x82ac, 0x862d, 0x6642, 0x9593, 0x7d10, 0x82ac, 0x862d, 0x6a19, 0x6e96, 0x6642,
0x9593, 0x7d10, 0x82ac, 0x862d, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x7d10, 0x57c3, 0x5cf6, 0x6642,
0x9593, 0x8afe, 0x798f, 0x514b, 0x5cf6, 0x6642, 0x9593, 0x8afe, 0x798f, 0x514b, 0x5cf6, 0x6a19,
0x6e96, 0x6642, 0x9593, 0x8afe, 0x798f, 0x514b, 0x5cf6, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x8cbb,
0x723e, 0x5357, 0x591a, 0x20, 0x2d, 0x20, 0x8fea, 0x8afe, 0x7f85, 0x5c3c, 0x4e9e, 0x6642,
0x9593, 0x8cbb, 0x723e, 0x5357, 0x591a, 0x20, 0x2d, 0x20, 0x8fea, 0x8afe, 0x7f85, 0x5c3c,
0x4e9e, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x8cbb, 0x723e, 0x5357, 0x591a, 0x20, 0x2d, 0x20,
0x8fea, 0x8afe, 0x7f85, 0x5c3c, 0x4e9e, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x5317, 0x99ac, 0x91cc,
0x4e9e, 0x7d0d, 0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x65b0, 0x897f, 0x4f2f, 0x5229, 0x4e9e, 0x6642,
0x9593, 0x65b0, 0x897f, 0x4f2f, 0x5229, 0x4e9e, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x65b0, 0x897f,
0x4f2f, 0x5229, 0x4e9e, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x9102, 0x6728, 0x65af, 0x514b, 0x6642,
0x9593, 0x9102, 0x6728, 0x65af, 0x514b, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x9102, 0x6728, 0x65af,
0x514b, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x5df4, 0x57fa, 0x65af, 0x5766, 0x6642, 0x9593, 0x5df4,
0x57fa, 0x65af, 0x5766, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x5df4, 0x57fa, 0x65af, 0x5766, 0x590f,
0x4ee4, 0x6642, 0x9593, 0x5e1b, 0x7409, 0x6642, 0x9593, 0x5df4, 0x5e03, 0x4e9e, 0x7d10, 0x5e7e,
0x5167, 0x4e9e, 0x6642, 0x9593, 0x5df4, 0x62c9, 0x572d, 0x6642, 0x9593, 0x5df4, 0x62c9, 0x572d,
0x6a19, 0x6e96, 0x6642, 0x9593, 0x5df4, 0x62c9, 0x572d, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x79d8,
0x9b6f, 0x6642, 0x9593, 0x79d8, 0x9b6f, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x79d8, 0x9b6f, 0x590f,
0x4ee4, 0x6642, 0x9593, 0x83f2, 0x5f8b, 0x8cd3, 0x6642, 0x9593, 0x83f2, 0x5f8b, 0x8cd3, 0x6a19,
0x6e96, 0x6642, 0x9593, 0x83f2, 0x5f8b, 0x8cd3, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x9cf3, 0x51f0,
0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x8056, 0x76ae, 0x57c3, 0x723e, 0x548c, 0x5bc6, 0x514b, 0x9686,
0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x8056, 0x76ae, 0x57c3, 0x723e, 0x548c, 0x5bc6, 0x514b, 0x9686,
0x7fa4, 0x5cf6, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x8056, 0x76ae, 0x57c3, 0x723e, 0x548c, 0x5bc6,
0x514b, 0x9686, 0x7fa4, 0x5cf6, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x76ae, 0x7279, 0x80af, 0x6642,
0x9593, 0x6ce2, 0x7d0d, 0x4f69, 0x6642, 0x9593, 0x5e73, 0x58e4, 0x6642, 0x9593, 0x514b, 0x5b5c,
0x52d2, 0x5967, 0x723e, 0x9054, 0x6642, 0x9593, 0x514b, 0x5b5c, 0x52d2, 0x5967, 0x723e, 0x9054,
0x6a19, 0x6e96, 0x6642, 0x9593, 0x514b, 0x5b5c, 0x52d2, 0x5967, 0x723e, 0x9054, 0x590f, 0x4ee4,
0x6642, 0x9593, 0x7559, 0x5c3c, 0x65fa, 0x6642, 0x9593, 0x7f85, 0x745f, 0x62c9, 0x6642, 0x9593,
0x5eab, 0x9801, 0x5cf6, 0x6642, 0x9593, 0x5eab, 0x9801, 0x5cf6, 0x6a19, 0x6e96, 0x6642, 0x9593,
0x5eab, 0x9801, 0x5cf6, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x85a9, 0x99ac, 0x62c9, 0x6642, 0x9593,
0x85a9, 0x99ac, 0x62c9, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x85a9, 0x99ac, 0x62c9, 0x590f, 0x4ee4,
0x6642, 0x9593, 0x85a9, 0x6469, 0x4e9e, 0x6642, 0x9593, 0x85a9, 0x6469, 0x4e9e, 0x6a19, 0x6e96,
0x6642, 0x9593, 0x85a9, 0x6469, 0x4e9e, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x585e, 0x5e2d, 0x723e,
0x6642, 0x9593, 0x65b0, 0x52a0, 0x5761, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x7d22, 0x7f85, 0x9580,
0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x5357, 0x55ac, 0x6cbb, 0x4e9e, 0x6642, 0x9593, 0x8607, 0x5229,
0x5357, 0x6642, 0x9593, 0x662d, 0x548c, 0x57fa, 0x5730, 0x6642, 0x9593, 0x5927, 0x6eaa, 0x5730,
0x6642, 0x9593, 0x53f0, 0x5317, 0x6642, 0x9593, 0x53f0, 0x5317, 0x6a19, 0x6e96, 0x6642, 0x9593,
0x53f0, 0x5317, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x5854, 0x5409, 0x514b, 0x6642, 0x9593, 0x6258,
0x514b, 0x52de, 0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x6771, 0x52a0, 0x6642, 0x9593, 0x6771, 0x52a0,
0x6a19, 0x6e96, 0x6642, 0x9593, 0x6771, 0x52a0, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x695a, 0x514b,
0x5cf6, 0x6642, 0x9593, 0x571f, 0x5eab, 0x66fc, 0x6642, 0x9593, 0x571f, 0x5eab, 0x66fc, 0x6a19,
0x6e96, 0x6642, 0x9593, 0x571f, 0x5eab, 0x66fc, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x5410, 0x74e6,
0x9b6f, 0x6642, 0x9593, 0x70cf, 0x62c9, 0x572d, 0x6642, 0x9593, 0x70cf, 0x62c9, 0x572d, 0x6a19,
0x6e96, 0x6642, 0x9593, 0x70cf, 0x62c9, 0x572d, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x70cf, 0x8332,
0x5225, 0x514b, 0x6642, 0x9593, 0x70cf, 0x8332, 0x5225, 0x514b, 0x6a19, 0x6e96, 0x6642, 0x9593,
0x70cf, 0x8332, 0x5225, 0x514b, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x842c, 0x90a3, 0x675c, 0x6642,
0x9593, 0x842c, 0x90a3, 0x675c, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x842c, 0x90a3, 0x675c, 0x590f,
0x4ee4, 0x6642, 0x9593, 0x59d4, 0x5167, 0x745e, 0x62c9, 0x6642, 0x9593, 0x6d77, 0x53c3, 0x5d34,
0x6642, 0x9593, 0x6d77, 0x53c3, 0x5d34, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x6d77, 0x53c3, 0x5d34,
0x590f, 0x4ee4, 0x6642, 0x9593, 0x4f0f, 0x723e, 0x52a0, 0x683c, 0x52d2, 0x6642, 0x9593, 0x4f0f,
0x723e, 0x52a0, 0x683c, 0x52d2, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x4f0f, 0x723e, 0x52a0, 0x683c,
0x52d2, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x6c83, 0x65af, 0x6258, 0x514b, 0x6642, 0x9593, 0x5a01,
0x514b, 0x5cf6, 0x6642, 0x9593, 0x74e6, 0x5229, 0x65af, 0x548c, 0x5bcc, 0x5716, 0x7d0d, 0x7fa4,
0x5cf6, 0x6642, 0x9593, 0x96c5, 0x5eab, 0x6b21, 0x514b, 0x6642, 0x9593, 0x96c5, 0x5eab, 0x6b21,
0x514b, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x96c5, 0x5eab, 0x6b21, 0x514b, 0x590f, 0x4ee4, 0x6642,
0x9593, 0x8449, 0x5361, 0x6377, 0x7433, 0x5821, 0x6642, 0x9593, 0x8449, 0x5361, 0x6377, 0x7433,
0x5821, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x8449, 0x5361, 0x6377, 0x7433, 0x5821, 0x590f, 0x4ee4,
0x6642, 0x9593, 0x80b2, 0x7a7a, 0x6642, 0x9593, 0x827e, 0x514b, 0x65f6, 0x95f4, 0x827e, 0x514b,
0x6807, 0x51c6, 0x65f6, 0x95f4, 0x827e, 0x514b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x5bcc,
0x6c57, 0x65f6, 0x95f4, 0x4e2d, 0x975e, 0x65f6, 0x95f4, 0x4e1c, 0x975e, 0x65f6, 0x95f4, 0x5357,
0x975e, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x897f, 0x975e, 0x65f6, 0x95f4, 0x897f, 0x975e, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x897f, 0x975e, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x62c9, 0x65af,
0x52a0, 0x65f6, 0x95f4, 0x963f, 0x62c9, 0x65af, 0x52a0, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x963f,
0x62c9, 0x65af, 0x52a0, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x62c9, 0x6728, 0x56fe, 0x65f6,
0x95f4, 0x963f, 0x62c9, 0x6728, 0x56fe, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x963f, 0x62c9, 0x6728,
0x56fe, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4e9a, 0x9a6c, 0x900a, 0x65f6, 0x95f4, 0x4e9a, 0x9a6c,
0x900a, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e9a, 0x9a6c, 0x900a, 0x590f, 0x4ee4, 0x65f6, 0x95f4,
0x4e2d, 0x90e8, 0x65f6, 0x95f4, 0x4e2d, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e2d, 0x90e8,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4e1c, 0x90e8, 0x65f6, 0x95f4, 0x4e1c, 0x90e8, 0x6807, 0x51c6,
0x65f6, 0x95f4, 0x4e1c, 0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5c71, 0x533a, 0x65f6, 0x95f4,
0x5c71, 0x533a, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5c71, 0x533a, 0x590f, 0x4ee4, 0x65f6, 0x95f4,
0x592a, 0x5e73, 0x6d0b, 0x65f6, 0x95f4, 0x592a, 0x5e73, 0x6d0b, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x592a, 0x5e73, 0x6d0b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x7eb3, 0x5fb7, 0x5c14, 0x65f6,
0x95f4, 0x963f, 0x90a3, 0x5e95, 0x6cb3, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x963f, 0x90a3, 0x5e95,
0x6cb3, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x76ae, 0x4e9a, 0x65f6, 0x95f4, 0x963f, 0x76ae,
0x4e9a, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x963f, 0x76ae, 0x4e9a, 0x590f, 0x4ee4, 0x65f6, 0x95f4,
0x963f, 0x514b, 0x9676, 0x65f6, 0x95f4, 0x963f, 0x514b, 0x9676, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x963f, 0x514b, 0x9676, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x514b, 0x6258, 0x6bd4, 0x65f6,
0x95f4, 0x963f, 0x514b, 0x6258, 0x6bd4, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x963f, 0x514b, 0x6258,
0x6bd4, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x62c9, 0x4f2f, 0x65f6, 0x95f4, 0x963f, 0x62c9,
0x4f2f, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x963f, 0x62c9, 0x4f2f, 0x590f, 0x4ee4, 0x65f6, 0x95f4,
0x963f, 0x6839, 0x5ef7, 0x65f6, 0x95f4, 0x963f, 0x6839, 0x5ef7, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x963f, 0x6839, 0x5ef7, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x6839, 0x5ef7, 0x897f, 0x90e8,
0x65f6, 0x95f4, 0x963f, 0x6839, 0x5ef7, 0x897f, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x963f,
0x6839, 0x5ef7, 0x897f, 0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4e9a, 0x7f8e, 0x5c3c, 0x4e9a,
0x65f6, 0x95f4, 0x4e9a, 0x7f8e, 0x5c3c, 0x4e9a, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e9a, 0x7f8e,
0x5c3c, 0x4e9a, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5927, 0x897f, 0x6d0b, 0x65f6, 0x95f4, 0x5927,
0x897f, 0x6d0b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5927, 0x897f, 0x6d0b, 0x590f, 0x4ee4, 0x65f6,
0x95f4, 0x6fb3, 0x6d32, 0x4e2d, 0x90e8, 0x65f6, 0x95f4, 0x6fb3, 0x6d32, 0x4e2d, 0x90e8, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x6fb3, 0x6d32, 0x4e2d, 0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x6fb3,
0x6d32, 0x4e2d, 0x897f, 0x90e8, 0x65f6, 0x95f4, 0x6fb3, 0x6d32, 0x4e2d, 0x897f, 0x90e8, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x6fb3, 0x6d32, 0x4e2d, 0x897f, 0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4,
0x6fb3, 0x6d32, 0x4e1c, 0x90e8, 0x65f6, 0x95f4, 0x6fb3, 0x6d32, 0x4e1c, 0x90e8, 0x6807, 0x51c6,
0x65f6, 0x95f4, 0x6fb3, 0x6d32, 0x4e1c, 0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x6fb3, 0x6d32,
0x897f, 0x90e8, 0x65f6, 0x95f4, 0x6fb3, 0x6d32, 0x897f, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x6fb3, 0x6d32, 0x897f, 0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4e9a, 0x585e, 0x62dc, 0x7136,
0x65f6, 0x95f4, 0x4e9a, 0x585e, 0x62dc, 0x7136, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e9a, 0x585e,
0x62dc, 0x7136, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4e9a, 0x901f, 0x5c14, 0x7fa4, 0x5c9b, 0x65f6,
0x95f4, 0x4e9a, 0x901f, 0x5c14, 0x7fa4, 0x5c9b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e9a, 0x901f,
0x5c14, 0x7fa4, 0x5c9b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5b5f, 0x52a0, 0x62c9, 0x65f6, 0x95f4,
0x5b5f, 0x52a0, 0x62c9, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5b5f, 0x52a0, 0x62c9, 0x590f, 0x4ee4,
0x65f6, 0x95f4, 0x4e0d, 0x4e39, 0x65f6, 0x95f4, 0x73bb, 0x5229, 0x7ef4, 0x4e9a, 0x65f6, 0x95f4,
0x5df4, 0x897f, 0x5229, 0x4e9a, 0x65f6, 0x95f4, 0x5df4, 0x897f, 0x5229, 0x4e9a, 0x6807, 0x51c6,
0x65f6, 0x95f4, 0x5df4, 0x897f, 0x5229, 0x4e9a, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x6c76, 0x83b1,
0x65f6, 0x95f4, 0x7ef4, 0x5fb7, 0x89d2, 0x65f6, 0x95f4, 0x7ef4, 0x5fb7, 0x89d2, 0x6807, 0x51c6,
0x65f6, 0x95f4, 0x7ef4, 0x5fb7, 0x89d2, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x51ef, 0x897f, 0x7ad9,
0x65f6, 0x95f4, 0x67e5, 0x83ab, 0x6d1b, 0x65f6, 0x95f4, 0x67e5, 0x5766, 0x7fa4, 0x5c9b, 0x65f6,
0x95f4, 0x67e5, 0x5766, 0x7fa4, 0x5c9b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x67e5, 0x5766, 0x7fa4,
0x5c9b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x667a, 0x5229, 0x65f6, 0x95f4, 0x667a, 0x5229, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x667a, 0x5229, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4e2d, 0x56fd, 0x65f6,
0x95f4, 0x4e2d, 0x56fd, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e2d, 0x56fd, 0x590f, 0x4ee4, 0x65f6,
0x95f4, 0x5723, 0x8bde, 0x5c9b, 0x65f6, 0x95f4, 0x79d1, 0x79d1, 0x65af, 0x7fa4, 0x5c9b, 0x65f6,
0x95f4, 0x54e5, 0x4f26, 0x6bd4, 0x4e9a, 0x65f6, 0x95f4, 0x54e5, 0x4f26, 0x6bd4, 0x4e9a, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x54e5, 0x4f26, 0x6bd4, 0x4e9a, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5e93,
0x514b, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x5e93, 0x514b, 0x7fa4, 0x5c9b, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x5e93, 0x514b, 0x7fa4, 0x5c9b, 0x534a, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x53e4, 0x5df4,
0x65f6, 0x95f4, 0x53e4, 0x5df4, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x53e4, 0x5df4, 0x590f, 0x4ee4,
0x65f6, 0x95f4, 0x6234, 0x7ef4, 0x65af, 0x65f6, 0x95f4, 0x675c, 0x8499, 0x675c, 0x6bd4, 0x5c14,
0x65f6, 0x95f4, 0x4e1c, 0x5e1d, 0x6c76, 0x65f6, 0x95f4, 0x590d, 0x6d3b, 0x8282, 0x5c9b, 0x65f6,
0x95f4, 0x590d, 0x6d3b, 0x8282, 0x5c9b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x590d, 0x6d3b, 0x8282,
0x5c9b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5384, 0x74dc, 0x591a, 0x65f6, 0x95f4, 0x4e2d, 0x6b27,
0x65f6, 0x95f4, 0x4e2d, 0x6b27, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e2d, 0x6b27, 0x590f, 0x4ee4,
0x65f6, 0x95f4, 0x4e1c, 0x6b27, 0x65f6, 0x95f4, 0x4e1c, 0x6b27, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x4e1c, 0x6b27, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x6b27, 0x6d32, 0x8fdc, 0x4e1c, 0x65f6, 0x95f4,
0x897f, 0x6b27, 0x65f6, 0x95f4, 0x897f, 0x6b27, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x897f, 0x6b27,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x798f, 0x514b, 0x5170, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x798f,
0x514b, 0x5170, 0x7fa4, 0x5c9b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x798f, 0x514b, 0x5170, 0x7fa4,
0x5c9b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x6590, 0x6d4e, 0x65f6, 0x95f4, 0x6590, 0x6d4e, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x6590, 0x6d4e, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x6cd5, 0x5c5e, 0x572d,
0x4e9a, 0x90a3, 0x65f6, 0x95f4, 0x6cd5, 0x56fd, 0x5357, 0x65b9, 0x53ca, 0x5357, 0x6781, 0x65f6,
0x95f4, 0x52a0, 0x62c9, 0x5df4, 0x54e5, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x7518, 0x6bd4, 0x5c14,
0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x4e54, 0x6cbb, 0x4e9a, 0x65f6, 0x95f4, 0x4e54, 0x6cbb, 0x4e9a,
0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e54, 0x6cbb, 0x4e9a, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5409,
0x5c14, 0x4f2f, 0x7279, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x683c, 0x6797, 0x5a01, 0x6cbb, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x683c, 0x9675, 0x5170, 0x4e1c, 0x90e8, 0x65f6, 0x95f4, 0x683c, 0x9675,
0x5170, 0x4e1c, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x683c, 0x9675, 0x5170, 0x4e1c, 0x90e8,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x683c, 0x9675, 0x5170, 0x897f, 0x90e8, 0x65f6, 0x95f4, 0x683c,
0x9675, 0x5170, 0x897f, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x683c, 0x9675, 0x5170, 0x897f,
0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5173, 0x5c9b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x6ce2,
0x65af, 0x6e7e, 0x6d77, 0x57df, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x76d6, 0x4e9a, 0x90a3, 0x65f6,
0x95f4, 0x590f, 0x5a01, 0x5937, 0x2d, 0x963f, 0x7559, 0x7533, 0x65f6, 0x95f4, 0x590f, 0x5a01,
0x5937, 0x2d, 0x963f, 0x7559, 0x7533, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x590f, 0x5a01, 0x5937,
0x2d, 0x963f, 0x7559, 0x7533, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x9999, 0x6e2f, 0x65f6, 0x95f4,
0x9999, 0x6e2f, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x9999, 0x6e2f, 0x590f, 0x4ee4, 0x65f6, 0x95f4,
0x79d1, 0x5e03, 0x591a, 0x65f6, 0x95f4, 0x79d1, 0x5e03, 0x591a, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x79d1, 0x5e03, 0x591a, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5370, 0x5ea6, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x5370, 0x5ea6, 0x6d0b, 0x65f6, 0x95f4, 0x5370, 0x5ea6, 0x652f, 0x90a3, 0x65f6, 0x95f4,
0x5370, 0x5c3c, 0x4e2d, 0x90e8, 0x65f6, 0x95f4, 0x5370, 0x5c3c, 0x4e1c, 0x90e8, 0x65f6, 0x95f4,
0x5370, 0x5c3c, 0x897f, 0x90e8, 0x65f6, 0x95f4, 0x4f0a, 0x6717, 0x65f6, 0x95f4, 0x4f0a, 0x6717,
0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4f0a, 0x6717, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4f0a, 0x5c14,
0x5e93, 0x6b21, 0x514b, 0x65f6, 0x95f4, 0x4f0a, 0x5c14, 0x5e93, 0x6b21, 0x514b, 0x6807, 0x51c6,
0x65f6, 0x95f4, 0x4f0a, 0x5c14, 0x5e93, 0x6b21, 0x514b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4ee5,
0x8272, 0x5217, 0x65f6, 0x95f4, 0x4ee5, 0x8272, 0x5217, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4ee5,
0x8272, 0x5217, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x65e5, 0x672c, 0x65f6, 0x95f4, 0x65e5, 0x672c,
0x6807, 0x51c6, 0x65f6, 0x95f4, 0x65e5, 0x672c, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5f7c, 0x5f97,
0x7f57, 0x5df4, 0x752b, 0x6d1b, 0x592b, 0x65af, 0x514b, 0x65f6, 0x95f4, 0x5f7c, 0x5f97, 0x7f57,
0x5df4, 0x752b, 0x6d1b, 0x592b, 0x65af, 0x514b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5f7c, 0x5f97,
0x7f57, 0x5df4, 0x752b, 0x6d1b, 0x592b, 0x65af, 0x514b, 0x65e5, 0x5149, 0x8282, 0x7ea6, 0x65f6,
0x95f4, 0x54c8, 0x8428, 0x514b, 0x65f6, 0x95f4, 0x4e1c, 0x54c8, 0x8428, 0x514b, 0x65f6, 0x95f4,
0x897f, 0x54c8, 0x8428, 0x514b, 0x65f6, 0x95f4, 0x97e9, 0x56fd, 0x65f6, 0x95f4, 0x97e9, 0x56fd,
0x6807, 0x51c6, 0x65f6, 0x95f4, 0x97e9, 0x56fd, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x79d1, 0x65af,
0x745e, 0x65f6, 0x95f4, 0x514b, 0x62c9, 0x65af, 0x8bfa, 0x4e9a, 0x5c14, 0x65af, 0x514b, 0x65f6,
0x95f4, 0x514b, 0x62c9, 0x65af, 0x8bfa, 0x4e9a, 0x5c14, 0x65af, 0x514b, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x514b, 0x62c9, 0x65af, 0x8bfa, 0x4e9a, 0x5c14, 0x65af, 0x514b, 0x590f, 0x4ee4, 0x65f6,
0x95f4, 0x5409, 0x5c14, 0x5409, 0x65af, 0x65f6, 0x95f4, 0x5170, 0x5361, 0x65f6, 0x95f4, 0x83b1,
0x6069, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x8c6a, 0x52cb, 0x7235, 0x5c9b, 0x65f6, 0x95f4, 0x8c6a,
0x52cb, 0x7235, 0x5c9b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x8c6a, 0x52cb, 0x7235, 0x5c9b, 0x590f,
0x4ee4, 0x65f6, 0x95f4, 0x6fb3, 0x95e8, 0x65f6, 0x95f4, 0x6fb3, 0x95e8, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x6fb3, 0x95e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x9a6c, 0x52a0, 0x4e39, 0x65f6, 0x95f4,
0x9a6c, 0x52a0, 0x4e39, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x9a6c, 0x52a0, 0x4e39, 0x590f, 0x4ee4,
0x65f6, 0x95f4, 0x9a6c, 0x6765, 0x897f, 0x4e9a, 0x65f6, 0x95f4, 0x9a6c, 0x5c14, 0x5730, 0x592b,
0x65f6, 0x95f4, 0x9a6c, 0x53ef, 0x8428, 0x65af, 0x65f6, 0x95f4, 0x9a6c, 0x7ecd, 0x5c14, 0x7fa4,
0x5c9b, 0x65f6, 0x95f4, 0x6a21, 0x91cc, 0x897f, 0x65af, 0x65f6, 0x95f4, 0x6a21, 0x91cc, 0x897f,
0x65af, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x6a21, 0x91cc, 0x897f, 0x65af, 0x590f, 0x4ee4, 0x65f6,
0x95f4, 0x83ab, 0x68ee, 0x65f6, 0x95f4, 0x58a8, 0x897f, 0x54e5, 0x592a, 0x5e73, 0x6d0b, 0x65f6,
0x95f4, 0x58a8, 0x897f, 0x54e5, 0x592a, 0x5e73, 0x6d0b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x58a8,
0x897f, 0x54e5, 0x592a, 0x5e73, 0x6d0b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4e4c, 0x5170, 0x5df4,
0x6258, 0x65f6, 0x95f4, 0x4e4c, 0x5170, 0x5df4, 0x6258, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e4c,
0x5170, 0x5df4, 0x6258, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x83ab, 0x65af, 0x79d1, 0x65f6, 0x95f4,
0x83ab, 0x65af, 0x79d1, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x83ab, 0x65af, 0x79d1, 0x590f, 0x4ee4,
0x65f6, 0x95f4, 0x7f05, 0x7538, 0x65f6, 0x95f4, 0x8bfa, 0x9c81, 0x65f6, 0x95f4, 0x5c3c, 0x6cca,
0x5c14, 0x65f6, 0x95f4, 0x65b0, 0x5580, 0x91cc, 0x591a, 0x5c3c, 0x4e9a, 0x65f6, 0x95f4, 0x65b0,
0x5580, 0x91cc, 0x591a, 0x5c3c, 0x4e9a, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x65b0, 0x5580, 0x91cc,
0x591a, 0x5c3c, 0x4e9a, 0x7fa4, 0x5c9b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x7ebd, 0x897f, 0x5170,
0x65f6, 0x95f4, 0x7ebd, 0x897f, 0x5170, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x7ebd, 0x897f, 0x5170,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x7ebd, 0x82ac, 0x5170, 0x65f6, 0x95f4, 0x7ebd, 0x82ac, 0x5170,
0x6807, 0x51c6, 0x65f6, 0x95f4, 0x7ebd, 0x82ac, 0x5170, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x7ebd,
0x57c3, 0x5c9b, 0x65f6, 0x95f4, 0x8bfa, 0x798f, 0x514b, 0x5c9b, 0x65f6, 0x95f4, 0x8bfa, 0x798f,
0x514b, 0x5c9b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x8bfa, 0x798f, 0x514b, 0x5c9b, 0x590f, 0x4ee4,
0x65f6, 0x95f4, 0x8d39, 0x5c14, 0x5357, 0x591a, 0x20, 0x2d, 0x20, 0x8fea, 0x8bfa, 0x7f57,
0x5c3c, 0x4e9a, 0x65f6, 0x95f4, 0x8d39, 0x5c14, 0x5357, 0x591a, 0x20, 0x2d, 0x20, 0x8fea,
0x8bfa, 0x7f57, 0x5c3c, 0x4e9a, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x8d39, 0x5c14, 0x5357, 0x591a,
0x20, 0x2d, 0x20, 0x8fea, 0x8bfa, 0x7f57, 0x5c3c, 0x4e9a, 0x590f, 0x4ee4, 0x65f6, 0x95f4,
0x5317, 0x9a6c, 0x91cc, 0x4e9a, 0x7eb3, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x65b0, 0x897f, 0x4f2f,
0x5229, 0x4e9a, 0x65f6, 0x95f4, 0x65b0, 0x897f, 0x4f2f, 0x5229, 0x4e9a, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x65b0, 0x897f, 0x4f2f, 0x5229, 0x4e9a, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x9102, 0x6728,
0x65af, 0x514b, 0x65f6, 0x95f4, 0x9102, 0x6728, 0x65af, 0x514b, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x9102, 0x6728, 0x65af, 0x514b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5df4, 0x57fa, 0x65af, 0x5766,
0x65f6, 0x95f4, 0x5df4, 0x57fa, 0x65af, 0x5766, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5df4, 0x57fa,
0x65af, 0x5766, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5e1b, 0x7409, 0x65f6, 0x95f4, 0x5df4, 0x5e03,
0x4e9a, 0x7ebd, 0x51e0, 0x5185, 0x4e9a, 0x65f6, 0x95f4, 0x5df4, 0x62c9, 0x572d, 0x65f6, 0x95f4,
0x5df4, 0x62c9, 0x572d, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5df4, 0x62c9, 0x572d, 0x590f, 0x4ee4,
0x65f6, 0x95f4, 0x79d8, 0x9c81, 0x65f6, 0x95f4, 0x79d8, 0x9c81, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x79d8, 0x9c81, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x83f2, 0x5f8b, 0x5bbe, 0x65f6, 0x95f4, 0x83f2,
0x5f8b, 0x5bbe, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x83f2, 0x5f8b, 0x5bbe, 0x590f, 0x4ee4, 0x65f6,
0x95f4, 0x51e4, 0x51f0, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x5723, 0x76ae, 0x57c3, 0x5c14, 0x548c,
0x5bc6, 0x514b, 0x9686, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x5723, 0x76ae, 0x57c3, 0x5c14, 0x548c,
0x5bc6, 0x514b, 0x9686, 0x7fa4, 0x5c9b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5723, 0x76ae, 0x57c3,
0x5c14, 0x548c, 0x5bc6, 0x514b, 0x9686, 0x7fa4, 0x5c9b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x76ae,
0x7279, 0x80af, 0x65f6, 0x95f4, 0x6ce2, 0x7eb3, 0x4f69, 0x65f6, 0x95f4, 0x5e73, 0x58e4, 0x65f6,
0x95f4, 0x514b, 0x5b5c, 0x52d2, 0x5965, 0x5c14, 0x8fbe, 0x65f6, 0x95f4, 0x514b, 0x5b5c, 0x52d2,
0x5965, 0x5c14, 0x8fbe, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x514b, 0x5b5c, 0x52d2, 0x5965, 0x5c14,
0x8fbe, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x7559, 0x5c3c, 0x65fa, 0x65f6, 0x95f4, 0x7f57, 0x745f,
0x62c9, 0x65f6, 0x95f4, 0x5e93, 0x9875, 0x5c9b, 0x65f6, 0x95f4, 0x5e93, 0x9875, 0x5c9b, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x5e93, 0x9875, 0x5c9b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x8428, 0x9a6c,
0x62c9, 0x65f6, 0x95f4, 0x8428, 0x9a6c, 0x62c9, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x8428, 0x9a6c,
0x62c9, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x8428, 0x6469, 0x4e9a, 0x65f6, 0x95f4, 0x8428, 0x6469,
0x4e9a, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x8428, 0x6469, 0x4e9a, 0x590f, 0x4ee4, 0x65f6, 0x95f4,
0x585e, 0x5e2d, 0x5c14, 0x65f6, 0x95f4, 0x65b0, 0x52a0, 0x5761, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x7d22, 0x7f57, 0x95e8, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x5357, 0x4e54, 0x6cbb, 0x4e9a, 0x65f6,
0x95f4, 0x82cf, 0x5229, 0x5357, 0x65f6, 0x95f4, 0x662d, 0x548c, 0x57fa, 0x5730, 0x65f6, 0x95f4,
0x5927, 0x6eaa, 0x5730, 0x65f6, 0x95f4, 0x53f0, 0x5317, 0x65f6, 0x95f4, 0x53f0, 0x5317, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x53f0, 0x5317, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5854, 0x5409, 0x514b,
0x65f6, 0x95f4, 0x6258, 0x514b, 0x52b3, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x4e1c, 0x52a0, 0x65f6,
0x95f4, 0x4e1c, 0x52a0, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e1c, 0x52a0, 0x590f, 0x4ee4, 0x65f6,
0x95f4, 0x695a, 0x514b, 0x5c9b, 0x65f6, 0x95f4, 0x571f, 0x5e93, 0x66fc, 0x65f6, 0x95f4, 0x571f,
0x5e93, 0x66fc, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x571f, 0x5e93, 0x66fc, 0x590f, 0x4ee4, 0x65f6,
0x95f4, 0x5410, 0x74e6, 0x9c81, 0x65f6, 0x95f4, 0x4e4c, 0x62c9, 0x572d, 0x65f6, 0x95f4, 0x4e4c,
0x62c9, 0x572d, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e4c, 0x62c9, 0x572d, 0x590f, 0x4ee4, 0x65f6,
0x95f4, 0x4e4c, 0x5179, 0x522b, 0x514b, 0x65f6, 0x95f4, 0x4e4c, 0x5179, 0x522b, 0x514b, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x4e4c, 0x5179, 0x522b, 0x514b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4e07,
0x90a3, 0x675c, 0x65f6, 0x95f4, 0x4e07, 0x90a3, 0x675c, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e07,
0x90a3, 0x675c, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x59d4, 0x5185, 0x745e, 0x62c9, 0x65f6, 0x95f4,
0x6d77, 0x53c2, 0x5d34, 0x65f6, 0x95f4, 0x6d77, 0x53c2, 0x5d34, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x6d77, 0x53c2, 0x5d34, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4f0f, 0x5c14, 0x52a0, 0x683c, 0x52d2,
0x65f6, 0x95f4, 0x4f0f, 0x5c14, 0x52a0, 0x683c, 0x52d2, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4f0f,
0x5c14, 0x52a0, 0x683c, 0x52d2, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x6c83, 0x65af, 0x6258, 0x514b,
0x65f6, 0x95f4, 0x5a01, 0x514b, 0x5c9b, 0x65f6, 0x95f4, 0x74e6, 0x5229, 0x65af, 0x548c, 0x5bcc,
0x56fe, 0x7eb3, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x96c5, 0x5e93, 0x6b21, 0x514b, 0x65f6, 0x95f4,
0x96c5, 0x5e93, 0x6b21, 0x514b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x96c5, 0x5e93, 0x6b21, 0x514b,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x53f6, 0x5361, 0x6377, 0x7433, 0x5821, 0x65f6, 0x95f4, 0x53f6,
0x5361, 0x6377, 0x7433, 0x5821, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x53f6, 0x5361, 0x6377, 0x7433,
0x5821, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x80b2, 0x7a7a, 0x65f6, 0x95f4, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x66, 0x67, 0x61, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0xc0, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0xc0, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0xc0, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6d, 0x65, 0x72,
0x69, 0x64, 0x69, 0x6f, 0x6e, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0xc0, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20,
0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xc0, 0x66, 0x72, 0x69, 0x63, 0x61,
0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0xc0, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20,
0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x65, 0x73,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0xe8,
0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72,
0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20,
0x64, 0x2019, 0x41, 0x6d, 0xe8, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0xe8, 0x72, 0x69, 0x63, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x64,
0x2019, 0x41, 0x6d, 0xe8, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x4e, 0x6f, 0x72, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0xe8, 0x72, 0x69,
0x63, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x64, 0x2019, 0x41,
0x6d, 0xe8, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e,
0x6f, 0x72, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6d,
0x75, 0x6e, 0x74, 0x61, 0x6e, 0x79, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x6d,
0xe8, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f,
0x72, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6d, 0x75, 0x6e, 0x74,
0x61, 0x6e, 0x79, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0xe8, 0x72, 0x69,
0x63, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x65, 0x20, 0x6d, 0x75, 0x6e, 0x74, 0x61, 0x6e, 0x79, 0x61, 0x20,
0x64, 0x2019, 0x41, 0x6d, 0xe8, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x20, 0x64,
0x2019, 0x41, 0x6d, 0xe8, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x4e, 0x6f, 0x72, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x20, 0x64, 0x2019, 0x41, 0x6d,
0xe8, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f,
0x72, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66,
0x69, 0x63, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0xe8, 0x72, 0x69, 0x63, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x41, 0x6e, 0xe0, 0x64, 0x69, 0x72, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x2019, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x48, 0x6f, 0x72,
0x61, 0x72, 0x69, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x2019, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x2019, 0x41, 0x70, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41, 0x70, 0x69, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0xe0, 0x72, 0x61, 0x62, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0xe0, 0x72, 0x61, 0x62, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0xe0, 0x72, 0x61, 0x62, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x6f, 0x65,
0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x6f, 0x65, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x6f, 0x65, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x6d, 0xe8, 0x6e, 0x69, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x6d, 0xe8, 0x6e, 0x69, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x2019, 0x41, 0x72, 0x6d, 0xe8, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c, 0xe0, 0x6e,
0x74, 0x69, 0x63, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41,
0x74, 0x6c, 0xe0, 0x6e, 0x74, 0x69, 0x63, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x41, 0x74, 0x6c, 0xe0, 0x6e, 0x74, 0x69, 0x63, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69,
0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41, 0x75,
0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x75,
0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x65, 0x2d, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0,
0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x2d, 0x6f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019,
0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x65, 0x2d, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x75,
0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73,
0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0,
0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72,
0xe0, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74,
0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72,
0xe0, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x6a, 0x61, 0x6e,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x61, 0x69, 0x64, 0x6a, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x6a, 0x61, 0x6e,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20,
0x41, 0xe7, 0x6f, 0x72, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x65, 0x73, 0x20, 0x41, 0xe7, 0x6f, 0x72, 0x65, 0x73, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x41, 0xe7, 0x6f, 0x72, 0x65, 0x73,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64,
0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x68, 0x75, 0x74,
0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x6f,
0x6c, 0xed, 0x76, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x75, 0x6e, 0x65,
0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x70, 0x20,
0x56, 0x65, 0x72, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61,
0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61,
0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65,
0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x58, 0x69, 0x6c, 0x65, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x58, 0x69, 0x6c, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x58, 0x69,
0x6c, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61,
0x20, 0x58, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x20, 0x58, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61,
0x20, 0x58, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4b, 0x69, 0x72, 0x69, 0x74, 0x69, 0x6d, 0x61, 0x74, 0x69, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x69,
0x6c, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0xf2, 0x6d, 0x62,
0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0xf2,
0x6d, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0xf2,
0x6d, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x65, 0x73, 0x20, 0x69, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f,
0x6f, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20,
0x69, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6d, 0x69, 0x67, 0x20, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x69,
0x6c, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x43,
0x75, 0x62, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x44,
0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x20, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69,
0x6c, 0x6c, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x69, 0x6c, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73,
0x71, 0x75, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x69,
0x6c, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x71, 0x75,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x69, 0x6c, 0x6c, 0x61, 0x20,
0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x71, 0x75, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x71, 0x75, 0x61, 0x64,
0x6f, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x65, 0x78, 0x74,
0x72, 0x65, 0x6d, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x73, 0x20, 0x69, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x6c,
0x76, 0x69, 0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x73, 0x20, 0x69, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x6c,
0x76, 0x69, 0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73,
0x20, 0x69, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69,
0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x46,
0x69, 0x6a, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6a,
0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x47, 0x75, 0x61, 0x69,
0x61, 0x6e, 0x61, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x6e, 0x74, 0xe0, 0x72,
0x74, 0x69, 0x64, 0x61, 0x20, 0x69, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65,
0x73, 0x20, 0x54, 0x65, 0x72, 0x72, 0x65, 0x73, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x73, 0x20, 0x41, 0x6e, 0x74, 0xe0, 0x72, 0x74,
0x69, 0x71, 0x75, 0x65, 0x73, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65,
0x73, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47,
0x65, 0xf2, 0x72, 0x67, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x47, 0x65, 0xf2, 0x72, 0x67, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x47,
0x65, 0xf2, 0x72, 0x67, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x69, 0x6c, 0x6c, 0x65, 0x73, 0x20,
0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x6d, 0x65, 0x72, 0x69, 0x64, 0x69, 0xe0, 0x20,
0x64, 0x65, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65,
0x6e, 0x6c, 0xe0, 0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe0, 0x6e, 0x64, 0x69, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75,
0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe0, 0x6e,
0x64, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x45, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65,
0x6e, 0x6c, 0xe0, 0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72,
0x6f, 0x65, 0x6e, 0x6c, 0xe0, 0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72,
0x6f, 0x65, 0x6e, 0x6c, 0xe0, 0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x65, 0x73, 0x74, 0x20,
0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe0, 0x6e, 0x64,
0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x65,
0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c,
0xe0, 0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f,
0x65, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e,
0x6c, 0xe0, 0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x47, 0x6f, 0x6c, 0x66, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x69, 0x61, 0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61,
0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61,
0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x48,
0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x68, 0x6f, 0x76,
0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x68, 0x6f, 0x76, 0x64,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75,
0x20, 0x64, 0x65, 0x20, 0x4b, 0x68, 0x6f, 0x76, 0x64, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xcd, 0x6e, 0x64, 0x69, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x6f, 0x63,
0x65, 0xe0, 0x20, 0xcd, 0x6e, 0x64, 0x69, 0x63, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x78, 0x69,
0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x20, 0x64, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0xe8, 0x73,
0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x65, 0x73, 0x74, 0x20, 0x64, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0xe8,
0x73, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x6f, 0x65, 0x73, 0x74, 0x20, 0x64, 0x2019, 0x49, 0x6e, 0x64, 0x6f,
0x6e, 0xe8, 0x73, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x49, 0x72, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x49, 0x72, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x49, 0x72, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x2019, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4a, 0x61, 0x70, 0xf3,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4a, 0x61, 0x70, 0xf3, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x4a, 0x61, 0x70, 0xf3, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x6d, 0x74, 0x78, 0x61, 0x74, 0x6b,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x20, 0x64, 0x65, 0x20,
0x4b, 0x61, 0x6d, 0x74, 0x78, 0x61, 0x74, 0x6b, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x72, 0x69, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c,
0x6f, 0x76, 0x73, 0x6b, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x6d, 0x74,
0x78, 0x61, 0x74, 0x6b, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x65, 0x73,
0x74, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68,
0x73, 0x74, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x6f, 0x65, 0x73, 0x74, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4b,
0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20,
0x43, 0x6f, 0x72, 0x65, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x69, 0x61, 0x72, 0x73, 0x6b,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x69, 0x61, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x69, 0x61, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x4b, 0x69, 0x72, 0x67, 0x75, 0x69, 0x7a,
0x73, 0x74, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x65, 0x73, 0x20, 0x69, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x4c, 0x69,
0x6e, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x72, 0x69, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x61, 0x63, 0x61, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x6c, 0xe0, 0x69, 0x73, 0x69, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20,
0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x69, 0x6c, 0x6c, 0x65,
0x73, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66,
0x69, 0x63, 0x20, 0x64, 0x65, 0x20, 0x4d, 0xe8, 0x78, 0x69, 0x63, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69,
0x63, 0x20, 0x64, 0x65, 0x20, 0x4d, 0xe8, 0x78, 0x69, 0x63, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x20, 0x64,
0x65, 0x20, 0x4d, 0xe8, 0x78, 0x69, 0x63, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61,
0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62,
0x61, 0x61, 0x74, 0x61, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x55, 0x6c, 0x61, 0x61,
0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x75, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x75, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65,
0x64, 0xf2, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0xf2, 0x6e, 0x69,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x43, 0x61,
0x6c, 0x65, 0x64, 0xf2, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76,
0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72,
0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x54, 0x65, 0x72, 0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65,
0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x69, 0x6c, 0x6c, 0x61,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x69, 0x6c, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x69, 0x6c,
0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x4f, 0x6d, 0x73,
0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x2019, 0x4f, 0x6d, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20,
0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x69, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x65,
0x72, 0xfa, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x65, 0x72, 0xfa, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x46,
0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69,
0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x46,
0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x69, 0x6c, 0x6c, 0x65,
0x73, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d,
0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50,
0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0xed, 0x6e,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c,
0xed, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c,
0xed, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x53, 0x65, 0x79, 0x63,
0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x69, 0x6c,
0x6c, 0x65, 0x73, 0x20, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0xf3, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x65, 0xf2, 0x72, 0x67, 0x69,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x53, 0x75, 0x64, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x68, 0x69,
0x74, 0xed, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61,
0x69, 0x70, 0x65, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61,
0x69, 0x70, 0x65, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x54,
0x61, 0x64, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65,
0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x54, 0x75,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x69, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x69, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x69,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x7a,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x7a, 0x62, 0x65,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x56, 0x61, 0x6e, 0x61, 0x74, 0x75, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x56, 0x61, 0x6e, 0x61, 0x74, 0x75, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20,
0x56, 0x61, 0x6e, 0x61, 0x74, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x56, 0x65, 0x6e, 0x65, 0xe7, 0x75, 0x65, 0x6c, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x65, 0x73, 0x20, 0x69, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x57, 0x61,
0x6b, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x57, 0x61,
0x6c, 0x6c, 0x69, 0x73, 0x20, 0x69, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x49, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x49, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x49, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x49, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20,
0x49, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x59, 0x75, 0x6b,
0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x66,
0x67, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x20, 0x41, 0x66, 0x72, 0x69,
0x63, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53,
0x6f, 0x75, 0x74, 0x68, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x57, 0x65, 0x73, 0x74, 0x20,
0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x57, 0x65, 0x73, 0x74, 0x20, 0x41, 0x66, 0x72, 0x69,
0x63, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69,
0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x57,
0x65, 0x73, 0x74, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20,
0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d,
0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x61,
0x73, 0x69, 0x70, 0x69, 0x6b, 0x6f, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x50, 0x61, 0x73, 0x69, 0x70, 0x69, 0x6b, 0x6f, 0x44,
0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x67, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x61, 0x73, 0x69,
0x70, 0x69, 0x6b, 0x6f, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x41, 0x70, 0x69, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x41, 0x70, 0x69, 0x61, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68,
0x74, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x41, 0x70, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x44,
0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x67, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x72, 0x61, 0x62,
0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e,
0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e,
0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65,
0x72, 0x6e, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x73, 0x61, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x63, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x44, 0x61, 0x79, 0x6c,
0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x63, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74,
0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x57, 0x65,
0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x57, 0x65, 0x73, 0x74,
0x65, 0x72, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x67,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72,
0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65,
0x72, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x45, 0x61, 0x73,
0x74, 0x65, 0x72, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e,
0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x45,
0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x44, 0x61, 0x79, 0x6c, 0x69,
0x67, 0x68, 0x74, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69,
0x6a, 0x61, 0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62,
0x61, 0x69, 0x6a, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65,
0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x73, 0x68, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69,
0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x42, 0x6f, 0x6c, 0x69, 0x76,
0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x42, 0x72,
0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d,
0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x42, 0x72, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73,
0x73, 0x61, 0x6c, 0x61, 0x6d, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x61, 0x70, 0x65,
0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20,
0x73, 0x61, 0x20, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64,
0x65, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x68,
0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x67,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69,
0x74, 0x20, 0x73, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6e,
0x61, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x67,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x68,
0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43,
0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43,
0x6f, 0x63, 0x6f, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0x6f,
0x6d, 0x62, 0x69, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e,
0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x6f,
0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4b, 0x61, 0x74, 0x75, 0x6e,
0x67, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73,
0x61, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x75,
0x62, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43,
0x75, 0x62, 0x61, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20,
0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x43, 0x75, 0x62, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x44, 0x61, 0x76, 0x69, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72,
0x76, 0x69, 0x6c, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x45, 0x61, 0x73, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65,
0x72, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20,
0x73, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61,
0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x45,
0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x45, 0x61,
0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67,
0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x45, 0x61, 0x73,
0x74, 0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x46, 0x75, 0x72, 0x74, 0x68,
0x65, 0x72, 0x2d, 0x65, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69,
0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x57,
0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x46, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x46, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69,
0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x46,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x46,
0x69, 0x6a, 0x69, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x46, 0x69, 0x6a, 0x69, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61,
0x20, 0x46, 0x69, 0x6a, 0x69, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x46, 0x72, 0x65, 0x6e, 0x63, 0x68, 0x20, 0x47, 0x75, 0x69, 0x61,
0x6e, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x46, 0x72,
0x65, 0x6e, 0x63, 0x68, 0x20, 0x53, 0x6f, 0x75, 0x74, 0x68, 0x65, 0x72,
0x6e, 0x20, 0x75, 0x67, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x63, 0x74,
0x69, 0x63, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x47, 0x61,
0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x47, 0x65,
0x6f, 0x72, 0x67, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73,
0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63,
0x68, 0x20, 0x4d, 0x65, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x45, 0x61, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20,
0x45, 0x61, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x57, 0x65,
0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x57, 0x65, 0x73,
0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d,
0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x57, 0x65, 0x73, 0x74,
0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x47, 0x75, 0x6c, 0x66, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e,
0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e,
0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x67, 0x61,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x48, 0x6f, 0x6e, 0x67,
0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d,
0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x48, 0x6f, 0x6e, 0x67,
0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x48, 0x6f, 0x76, 0x64, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20,
0x73, 0x61, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x20,
0x4f, 0x63, 0x65, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65,
0x72, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x57, 0x65, 0x73, 0x74,
0x65, 0x72, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69,
0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x49, 0x72, 0x61,
0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x49, 0x72,
0x61, 0x6e, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e,
0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x49,
0x72, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x49,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x49, 0x73,
0x72, 0x61, 0x65, 0x6c, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x44, 0x61, 0x79, 0x6c, 0x69,
0x67, 0x68, 0x74, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4a, 0x61, 0x70, 0x61,
0x6e, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x67,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4a, 0x61,
0x70, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b,
0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x20, 0x4b, 0x61,
0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x57, 0x65, 0x73, 0x74, 0x20, 0x4b, 0x61, 0x7a,
0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x44, 0x61,
0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x6f, 0x73, 0x72,
0x61, 0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x79, 0x61, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73,
0x61, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73,
0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x79, 0x72,
0x67, 0x79, 0x7a, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x4c, 0x69, 0x6e, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e,
0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x4d, 0x61, 0x6c, 0x61, 0x79, 0x73, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69,
0x76, 0x65, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4d,
0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x73, 0x61, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4d, 0x61, 0x77,
0x73, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4d,
0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66,
0x69, 0x63, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4d,
0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66,
0x69, 0x63, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e,
0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4d,
0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66,
0x69, 0x63, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x55, 0x6c,
0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62,
0x61, 0x61, 0x74, 0x61, 0x72, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73,
0x61, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61,
0x72, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4d, 0x6f, 0x73,
0x63, 0x6f, 0x77, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20,
0x73, 0x61, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x61, 0x75, 0x72,
0x75, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x65, 0x70,
0x61, 0x6c, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x65,
0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20,
0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e,
0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61,
0x6e, 0x64, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e,
0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x44, 0x61,
0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x5a,
0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e,
0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x44,
0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x67, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x66,
0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68,
0x74, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d,
0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20,
0x73, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4f, 0x6d,
0x73, 0x6b, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4f,
0x6d, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20,
0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e,
0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x65, 0x77,
0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x79, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61,
0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x65, 0x72, 0x75, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x65, 0x72, 0x75, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x50, 0x65, 0x72, 0x75, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x69, 0x6c, 0x69, 0x70,
0x69, 0x6e, 0x61, 0x73, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x50, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x50, 0x69, 0x6c, 0x69, 0x70,
0x69, 0x6e, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53,
0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e,
0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53,
0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x6f, 0x6e, 0x61,
0x70, 0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x79,
0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72,
0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x61, 0x6b,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67,
0x68, 0x74, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c,
0x65, 0x73, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53,
0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x53, 0x6f, 0x75, 0x74, 0x68, 0x20, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61,
0x69, 0x70, 0x65, 0x69, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74,
0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x54, 0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x6f, 0x6b,
0x65, 0x6c, 0x61, 0x75, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x73, 0x61, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x75, 0x76, 0x61,
0x6c, 0x75, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x79, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e,
0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61,
0x79, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x55, 0x7a, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69,
0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x55,
0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61,
0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c,
0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61,
0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d,
0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x57, 0x61,
0x6c, 0x6c, 0x69, 0x73, 0x20, 0x26, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e,
0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x59, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69,
0x74, 0x20, 0x73, 0x61, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x59, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x73, 0x61, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd09,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804,
0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804,
0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd1b, 0xd804,
0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd16,
0xd804, 0xdd34, 0xd804, 0xdd19, 0xd804, 0xdd33, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd1c, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd07, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b,
0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd1c, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd07, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd18, 0xd804, 0xdd27, 0xd804, 0xdd09, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd1c, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804,
0xdd07, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804,
0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd1c, 0xd804, 0xdd33,
0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd07, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804,
0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd1c, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd07,
0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2c,
0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd1c, 0xd804, 0xdd33, 0xd804,
0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd07, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd0c, 0xd804,
0xdd34, 0xd804, 0xdd07, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd0c, 0xd804, 0xdd34,
0xd804, 0xdd07, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd07,
0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804,
0xdd23, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1f,
0xd804, 0xdd11, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804,
0xdd1f, 0xd804, 0xdd11, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804,
0xdd1f, 0xd804, 0xdd11, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1f, 0xd804, 0xdd0e, 0xd804,
0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd33, 0xd804, 0xdd20,
0xd804, 0xdd1f, 0xd804, 0xdd0e, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03,
0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1f, 0xd804, 0xdd0e, 0xd804, 0xdd27, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804,
0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03,
0xd804, 0xdd0f, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804,
0xdd22, 0xd804, 0xdd33, 0xd804, 0xdd26, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd0f, 0xd804, 0xdd27,
0xd804, 0xdd23, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd33, 0xd804, 0xdd26, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd0f, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd33, 0xd804, 0xdd26,
0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804,
0xdd23, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2e,
0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0x20, 0xd804,
0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0x20,
0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804,
0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd28,
0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd27, 0xd804, 0xdd31, 0x20, 0xd804, 0xdd1e, 0xd804,
0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd26,
0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd27, 0xd804, 0xdd31, 0x20, 0xd804,
0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804,
0xdd27, 0xd804, 0xdd31, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804,
0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd22,
0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0x20, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd25, 0xd804, 0xdd09, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1e,
0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd26,
0xd804, 0xdd25, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804,
0xdd25, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd1f,
0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd25, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804,
0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28,
0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd18,
0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804,
0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd1b, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a,
0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd1f, 0xd804,
0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd09, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804,
0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd03, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804,
0xdd1d, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27,
0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1d, 0xd804,
0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1d, 0xd804, 0xdd28,
0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd03, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1d, 0xd804, 0xdd28, 0x20, 0xd804,
0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804,
0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd22, 0xd804, 0xdd34,
0xd804, 0xdd0e, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd28,
0xd804, 0xdd1a, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804,
0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0x20,
0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd03, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804,
0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd2c, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804,
0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804,
0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd2c,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0x20, 0xd804,
0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f,
0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd22, 0xd804,
0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804,
0xdd28, 0xd804, 0xdd1a, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27,
0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804,
0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd20,
0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd03, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd1a,
0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd23,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd28, 0xd804, 0xdd09, 0xd804, 0xdd2e,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804,
0xdd23, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd28, 0xd804, 0xdd09, 0xd804,
0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804,
0xdd23, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd28, 0xd804, 0xdd09, 0xd804,
0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a,
0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd0f, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd33, 0xd804, 0xdd26, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804,
0xdd11, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804,
0xdd20, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd34,
0xd804, 0xdd11, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd28,
0xd804, 0xdd20, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd0f, 0xd804, 0xdd27, 0xd804,
0xdd23, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd33,
0xd804, 0xdd26, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd0c,
0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd23,
0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd0f, 0xd804,
0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd33, 0xd804, 0xdd26, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd18, 0xd804,
0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2c,
0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804,
0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804,
0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd0c,
0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd23,
0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804,
0xdd09, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804,
0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd1b, 0xd804, 0xdd2a,
0xd804, 0xdd09, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd18, 0xd804,
0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd28,
0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804,
0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd33, 0xd804, 0xdd22,
0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd27, 0x20, 0xd804,
0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804,
0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804,
0xdd11, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804,
0xdd20, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd28,
0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd18, 0xd804,
0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd0e, 0xd804, 0xdd22, 0xd804, 0xdd34,
0xd804, 0xdd1d, 0xd804, 0xdd2d, 0xd804, 0xdd0e, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd03, 0xd804, 0xdd0e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2d, 0xd804,
0xdd0e, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd0e, 0xd804, 0xdd22, 0xd804,
0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2d, 0xd804, 0xdd0e, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd0e, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804,
0xdd0c, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd0e, 0xd804, 0xdd2e,
0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804,
0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2c,
0xd804, 0xdd0e, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd0c, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804,
0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1d, 0xd804, 0xdd01, 0xd804, 0xdd23, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd0c,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1d, 0xd804, 0xdd01, 0xd804, 0xdd23, 0xd804, 0xdd18, 0xd804,
0xdd2c, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1d, 0xd804, 0xdd01, 0xd804, 0xdd23, 0xd804,
0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1e, 0xd804, 0xdd2a, 0xd804,
0xdd11, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd23,
0xd804, 0xdd28, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1d, 0xd804,
0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804,
0xdd20, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd25, 0xd804, 0xdd28,
0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd22,
0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804,
0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804,
0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1d,
0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd2c, 0xd804, 0xdd2d,
0x20, 0xd804, 0xdd18, 0xd804, 0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804,
0xdd25, 0xd804, 0xdd23, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd2c,
0xd804, 0xdd1b, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804,
0xdd18, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd2c, 0xd804, 0xdd1b, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd1e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd2c, 0xd804, 0xdd1b, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd1f, 0xd804, 0xdd2c,
0xd804, 0xdd22, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd17,
0xd804, 0xdd1f, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804,
0xdd17, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804,
0xdd17, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a,
0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0c,
0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd28, 0xd804, 0xdd23,
0xd804, 0xdd28, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804,
0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd0c, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd28, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e,
0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804,
0xdd1f, 0xd804, 0xdd25, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804,
0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd0c, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20,
0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a,
0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804,
0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd27,
0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804,
0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1d,
0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd07, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804,
0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804,
0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd2a,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09,
0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804,
0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18,
0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804,
0xdd19, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804,
0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd1d, 0x20, 0xd804, 0xdd1f, 0xd804,
0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd28,
0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd1d, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804,
0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd13, 0xd804, 0xdd2c, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd25,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd13, 0xd804, 0xdd2a, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x2d, 0xd804, 0xdd18, 0xd804, 0xdd33,
0xd804, 0xdd20, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd05, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804,
0xdd34, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b,
0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804,
0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d,
0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03,
0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804,
0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804,
0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09,
0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28,
0xd804, 0xdd18, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd2a,
0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd13, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd19, 0xd804,
0xdd33, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a,
0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd19, 0xd804,
0xdd33, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a,
0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd19, 0xd804, 0xdd33,
0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804,
0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2e,
0xd804, 0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804,
0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1d,
0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09,
0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804,
0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd2e, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27,
0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd19, 0xd804, 0xdd33, 0xd804, 0xdd06, 0xd804, 0xdd2c, 0xd804,
0xdd23, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2c,
0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804,
0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f,
0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804,
0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f,
0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804,
0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd2e, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd2c, 0xd804,
0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28,
0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd2e,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d,
0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804,
0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1c,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804,
0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804,
0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20,
0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a,
0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804,
0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd28,
0xd804, 0xdd0e, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd28, 0xd804, 0xdd0e, 0xd804,
0xdd28, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd28, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd09,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd25, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd20,
0xd804, 0xdd1a, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd25, 0xd804,
0xdd28, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd27, 0xd804, 0xdd09, 0xd804, 0xdd28, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd33, 0xd804, 0xdd03, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd27, 0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd07, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd09, 0xd804,
0xdd23, 0xd804, 0xdd1b, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1d,
0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0e, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd0e, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd28,
0xd804, 0xdd20, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd0e, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd0e,
0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd09, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd34,
0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804,
0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd09,
0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd0c,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd11, 0xd804, 0xdd2c, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1b, 0xd804, 0xdd2a,
0xd804, 0xdd09, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd09, 0xd804,
0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804,
0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2c, 0xd804,
0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd28, 0xd804, 0xdd0f,
0xd804, 0xdd2c, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804,
0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f,
0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804,
0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804,
0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f,
0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804,
0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804,
0xdd28, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd1f,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03,
0xd804, 0xdd2a, 0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd09, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd1f, 0xd804,
0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd09, 0xd804, 0xdd2a,
0xd804, 0xdd20, 0xd804, 0xdd1a, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804,
0xdd20, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd23,
0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd20,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804,
0xdd2e, 0xd804, 0xdd20, 0xd804, 0xdd2d, 0x2d, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2c,
0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd16, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804,
0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd27,
0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd20, 0xd804, 0xdd2d, 0x2d, 0xd804, 0xdd03, 0xd804,
0xdd23, 0xd804, 0xdd2c, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd16, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23,
0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd27, 0xd804,
0xdd01, 0x20, 0xd804, 0xdd07, 0xd804, 0xdd27, 0xd804, 0xdd01, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804,
0xdd27, 0xd804, 0xdd01, 0x20, 0xd804, 0xdd07, 0xd804, 0xdd27, 0xd804, 0xdd01, 0x20, 0xd804,
0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd26,
0xd804, 0xdd27, 0xd804, 0xdd01, 0x20, 0xd804, 0xdd07, 0xd804, 0xdd27, 0xd804, 0xdd01, 0x20,
0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd26, 0xd804, 0xdd2e, 0xd804, 0xdd1e, 0xd804, 0xdd27, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd26, 0xd804, 0xdd2e, 0xd804, 0xdd1e, 0xd804, 0xdd27, 0xd804, 0xdd13, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd26, 0xd804, 0xdd2e, 0xd804, 0xdd1e, 0xd804, 0xdd27, 0xd804, 0xdd13, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804,
0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd28,
0xd804, 0xdd20, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd1f, 0xd804,
0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd25, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd2e, 0xd804, 0xdd0c, 0xd804, 0xdd29, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd0f, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd22, 0xd804, 0xdd33, 0xd804, 0xdd26, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd2e, 0xd804,
0xdd1a, 0xd804, 0xdd2c, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b,
0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd2e, 0xd804,
0xdd1a, 0xd804, 0xdd2c, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b,
0xd804, 0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd18,
0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd18, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd2c, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804,
0xdd20, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd18,
0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2c, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd2a, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804,
0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd2a, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd25,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd2a, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd25,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd0e,
0xd804, 0xdd22, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd0e, 0xd804, 0xdd34, 0xd804, 0xdd22, 0xd804, 0xdd20, 0xd804,
0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd0e, 0xd804,
0xdd34, 0xd804, 0xdd22, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23,
0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0e, 0xd804, 0xdd1b, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0e, 0xd804, 0xdd1b, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd0e, 0xd804, 0xdd1b, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd18, 0xd804,
0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd28, 0xd804, 0xdd16, 0xd804, 0xdd33,
0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1e,
0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x2d, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1f, 0xd804,
0xdd34, 0xd804, 0xdd0c, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804,
0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd28, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1b, 0xd804, 0xdd28, 0xd804, 0xdd16, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c,
0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1e, 0xd804, 0xdd27, 0xd804, 0xdd23,
0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x2d, 0xd804,
0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd0c, 0xd804,
0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804,
0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd28, 0xd804, 0xdd16, 0xd804, 0xdd33, 0xd804,
0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1e, 0xd804,
0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x2d, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1f, 0xd804, 0xdd34,
0xd804, 0xdd0c, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd03,
0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd28, 0x20, 0xd804,
0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804,
0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b,
0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804,
0xdd07, 0xd804, 0xdd0e, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd33, 0xd804,
0xdd16, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd0f, 0xd804,
0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd07,
0xd804, 0xdd0e, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd33, 0xd804, 0xdd16,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd28,
0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804,
0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f,
0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23,
0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804,
0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd22, 0xd804, 0xdd33, 0xd804, 0xdd06, 0xd804, 0xdd2c, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd1a,
0xd804, 0xdd2e, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804,
0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804,
0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f,
0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804,
0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804,
0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd01, 0xd804, 0xdd07, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804,
0xdd2d, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804,
0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c,
0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd26, 0xd804, 0xdd24, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd26,
0xd804, 0xdd24, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34,
0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd26, 0xd804, 0xdd24, 0xd804, 0xdd2c, 0x20,
0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23,
0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd07, 0xd804,
0xdd03, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd07, 0xd804, 0xdd03, 0xd804, 0xdd2e,
0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd07,
0xd804, 0xdd03, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd09,
0xd804, 0xdd13, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd33, 0xd804,
0xdd20, 0xd804, 0xdd09, 0xd804, 0xdd13, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f,
0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804,
0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd09, 0xd804, 0xdd13, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd1f, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd25, 0xd804,
0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd18,
0xd804, 0xdd28, 0xd804, 0xdd1b, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd22, 0xd804,
0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2c, 0xd804, 0xdd25, 0xd804, 0xdd25, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1f, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd23, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804,
0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804,
0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd25, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd25, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804,
0xdd25, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27,
0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd33, 0xd804,
0xdd26, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd2c,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804,
0xdd25, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd1f,
0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd25, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd25,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd1f, 0xd804,
0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd25, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804,
0xdd26, 0xd804, 0xdd25, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e,
0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd23, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1d,
0xd804, 0xdd16, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804,
0xdd2a, 0xd804, 0xdd23, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1d, 0xd804, 0xdd16,
0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd23,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1d, 0xd804, 0xdd16, 0xd804, 0xdd2e, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27,
0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804,
0xdd07, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd1f, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd03, 0xd804,
0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2a, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd2c, 0xd804, 0xdd1b,
0xd804, 0xdd23, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804,
0xdd2a, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd23, 0xd804, 0xdd2c,
0xd804, 0xdd13, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0x20, 0xd804, 0xdd07, 0xd804, 0xdd33,
0xd804, 0xdd20, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804, 0xdd13, 0xd804, 0xdd2e, 0xd804, 0xdd1a,
0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a,
0x20, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804,
0xdd13, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd09,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804,
0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804,
0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd0e, 0xd804, 0xdd28,
0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd0e,
0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804,
0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd1c, 0xd804, 0xdd03,
0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804,
0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd1c, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804,
0xdd2a, 0xd804, 0xdd1c, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804,
0xdd28, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03,
0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804,
0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd03, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1c, 0xd804, 0xdd2e, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d,
0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804,
0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1c, 0xd804, 0xdd2e, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d,
0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804,
0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f,
0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1c, 0xd804, 0xdd2e, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20,
0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a,
0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd18, 0xd804,
0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1a,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd2e, 0xd804, 0xdd13, 0xd804, 0xdd28,
0x20, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd26, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1a,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd2e, 0xd804, 0xdd13, 0xd804, 0xdd28,
0x20, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd26, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd2e, 0xd804, 0xdd13, 0xd804, 0xdd28, 0x20,
0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd26, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804,
0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd05, 0xd804, 0xdd2a, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804,
0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18,
0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd1e, 0xd804, 0xdd2e, 0xd804,
0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd1d, 0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804,
0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd2e,
0xd804, 0xdd1e, 0xd804, 0xdd2e, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd1d, 0xd804, 0xdd28,
0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd1e, 0xd804, 0xdd2e, 0xd804, 0xdd25, 0xd804, 0xdd28,
0xd804, 0xdd1d, 0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd27,
0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804,
0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804,
0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804,
0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b,
0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd23, 0xd804, 0xdd05,
0xd804, 0xdd2a, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0x20,
0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0x20, 0xd804, 0xdd09, 0xd804,
0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd20,
0xd804, 0xdd22, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804,
0xdd20, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804,
0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd2c, 0xd804,
0xdd22, 0xd804, 0xdd2a, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd2a,
0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1b, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd2a, 0x20, 0xd804, 0xdd09, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804,
0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd28,
0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd1b, 0xd804, 0xdd2d, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd1b, 0xd804,
0xdd2d, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804,
0xdd28, 0xd804, 0xdd1b, 0xd804, 0xdd2d, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804,
0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20,
0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a,
0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804,
0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1b,
0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd2c,
0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804,
0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1b,
0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd2c,
0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804,
0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd2c,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1b, 0xd804,
0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd2e, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd2c, 0xd804,
0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28,
0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd28, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804,
0xdd07, 0xd804, 0xdd2c, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd1b, 0xd804, 0xdd2c,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd27, 0xd804, 0xdd01, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd01, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd28, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804,
0xdd34, 0xd804, 0xdd13, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd0e, 0xd804, 0xdd28,
0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd13, 0x20, 0xd804,
0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd0e,
0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd13,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804,
0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a,
0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd17, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25,
0xd804, 0xdd08, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd25, 0xd804, 0xdd08, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd25, 0xd804, 0xdd08, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd25, 0xd804, 0xdd1f, 0xd804, 0xdd22, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd1f, 0xd804, 0xdd22,
0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd1f,
0xd804, 0xdd22, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804,
0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd1f, 0xd804, 0xdd2e, 0xd804, 0xdd20, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd25, 0xd804, 0xdd1f, 0xd804, 0xdd2e, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd1f, 0xd804,
0xdd2e, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27,
0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804,
0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd28,
0xd804, 0xdd01, 0xd804, 0xdd09, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0x20, 0xd804,
0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25,
0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d,
0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804,
0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd18,
0xd804, 0xdd27, 0xd804, 0xdd09, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd0e, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804,
0xdd20, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a,
0xd804, 0xdd1f, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd20, 0xd804, 0xdd2e, 0xd804,
0xdd24, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd26, 0xd804, 0xdd28, 0xd804, 0xdd16, 0xd804, 0xdd28,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd2d, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd06, 0xd804,
0xdd2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd2d, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd06,
0xd804, 0xdd2c, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd2d, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd06,
0xd804, 0xdd2c, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804,
0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd16,
0xd804, 0xdd0e, 0xd804, 0xdd08, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd07, 0xd804, 0xdd2c, 0xd804,
0xdd23, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd0b,
0xd804, 0xdd34, 0xd804, 0xdd09, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd0b, 0xd804,
0xdd34, 0xd804, 0xdd09, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd0b, 0xd804, 0xdd34, 0xd804,
0xdd09, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd2a, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd16,
0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd1f,
0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804,
0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804,
0xdd28, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd16, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd11, 0xd804, 0xdd2a, 0xd804, 0xdd1e, 0xd804,
0xdd23, 0xd804, 0xdd2a, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2a,
0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804,
0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804,
0xdd2c, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804,
0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd0e, 0xd804,
0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd16, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd0e, 0xd804, 0xdd34,
0xd804, 0xdd1d, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd16, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd0e, 0xd804, 0xdd34, 0xd804, 0xdd1d,
0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd16, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804,
0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1e, 0xd804, 0xdd1a, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd16, 0xd804, 0xdd2a,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1e, 0xd804, 0xdd1a, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd16, 0xd804,
0xdd2a, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1e, 0xd804, 0xdd1a, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd16, 0xd804,
0xdd2a, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1e, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd2c, 0xd804, 0xdd0e, 0xd804,
0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1d, 0xd804, 0xdd33,
0xd804, 0xdd23, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd25,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd2e, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd23, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1d, 0xd804,
0xdd2e, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd2e, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd23, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804,
0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd2e, 0xd804,
0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27,
0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1e, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804,
0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd13, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd1e, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd09,
0xd804, 0xdd2e, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd13, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd1e, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd09,
0xd804, 0xdd2e, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd13, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804,
0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1e, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804,
0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804,
0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd24, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd33, 0xd804, 0xdd03, 0x20, 0xd804, 0xdd1c, 0xd804, 0xdd2a, 0xd804, 0xdd11, 0xd804,
0xdd2a, 0xd804, 0xdd1a, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd16,
0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804,
0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804,
0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd16, 0xd804,
0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804,
0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03,
0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd16, 0xd804, 0xdd2c,
0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2a,
0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804,
0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28,
0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd16, 0xd804, 0xdd2c, 0xd804, 0xdd22,
0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2a, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0x41e, 0x432, 0x445, 0x4c0, 0x430, 0x43d,
0x42e, 0x43a, 0x43a, 0x44a, 0x435, 0x440, 0x430, 0x20, 0x410, 0x444, 0x440, 0x438,
0x43a, 0x430, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x430, 0x43b, 0x435, 0x43d, 0x20,
0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x41a, 0x44a, 0x438, 0x43b, 0x431, 0x430,
0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x41c, 0x430, 0x43b, 0x445, 0x431,
0x443, 0x437, 0x435, 0x43d, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x41c,
0x430, 0x43b, 0x445, 0x431, 0x443, 0x437, 0x435, 0x43d, 0x20, 0x410, 0x444, 0x440,
0x438, 0x43a, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445, 0x431,
0x443, 0x437, 0x435, 0x43d, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x2c,
0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x410,
0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435,
0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x410, 0x43c, 0x430, 0x437, 0x43e,
0x43d, 0x43a, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x410, 0x43c, 0x430, 0x437, 0x43e,
0x43d, 0x43a, 0x430, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x42e, 0x43a, 0x43a, 0x44a, 0x435, 0x440, 0x430,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x42e, 0x43a, 0x43a, 0x44a,
0x435, 0x440, 0x430, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x42e, 0x43a, 0x43a, 0x44a, 0x435, 0x440, 0x430, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a,
0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445,
0x431, 0x430, 0x43b, 0x435, 0x43d, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a,
0x430, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x430, 0x43b, 0x435, 0x43d, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430,
0x43b, 0x445, 0x431, 0x430, 0x43b, 0x435, 0x43d, 0x20, 0x410, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41b, 0x430, 0x44c, 0x43c, 0x43d, 0x438, 0x439,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x20, 0x28, 0x410, 0x426, 0x428, 0x29, 0x41b,
0x430, 0x44c, 0x43c, 0x43d, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x20, 0x28,
0x410, 0x426, 0x428, 0x29, 0x41b, 0x430, 0x44c, 0x43c, 0x43d, 0x438, 0x439, 0x43d,
0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x20, 0x28, 0x410, 0x426, 0x428, 0x29, 0x422, 0x438, 0x439, 0x43d, 0x430,
0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x422, 0x438, 0x439, 0x43d, 0x430, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x430,
0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x422, 0x438, 0x439, 0x43d, 0x430, 0x20, 0x43e, 0x43a,
0x435, 0x430, 0x43d, 0x430, 0x43d, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d,
0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x445, 0x430, 0x43d, 0x20, 0x410, 0x43f,
0x438, 0x430, 0x2c, 0x20, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x20,
0x410, 0x43f, 0x438, 0x430, 0x2c, 0x20, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x430,
0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x20,
0x410, 0x43f, 0x438, 0x430, 0x2c, 0x20, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x421,
0x430, 0x4c0, 0x443, 0x434, 0x438, 0x439, 0x43d, 0x20, 0x4c0, 0x430, 0x44c, 0x440,
0x431, 0x438, 0x439, 0x447, 0x43e, 0x44c, 0x421, 0x430, 0x4c0, 0x443, 0x434, 0x438,
0x439, 0x43d, 0x20, 0x4c0, 0x430, 0x44c, 0x440, 0x431, 0x438, 0x439, 0x447, 0x43e,
0x44c, 0x43d, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x421, 0x430, 0x4c0, 0x443, 0x434,
0x438, 0x439, 0x43d, 0x20, 0x4c0, 0x430, 0x44c, 0x440, 0x431, 0x438, 0x439, 0x447,
0x43e, 0x44c, 0x43d, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435,
0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x410, 0x440, 0x433, 0x435, 0x43d,
0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x410, 0x440, 0x433, 0x435,
0x43d, 0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435,
0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445, 0x431,
0x443, 0x437, 0x435, 0x43d, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438,
0x43d, 0x430, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x443, 0x437, 0x435, 0x43d, 0x20,
0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x443, 0x437, 0x435, 0x43d, 0x20, 0x410,
0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x430, 0x44c,
0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x42d, 0x440,
0x43c, 0x430, 0x43b, 0x43e, 0x439, 0x447, 0x43e, 0x44c, 0x42d, 0x440, 0x43c, 0x430,
0x43b, 0x43e, 0x439, 0x447, 0x43e, 0x44c, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x42d, 0x440,
0x43c, 0x430, 0x43b, 0x43e, 0x439, 0x447, 0x43e, 0x44c, 0x2c, 0x20, 0x430, 0x44c,
0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x410, 0x442,
0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x430, 0x43d, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x430, 0x43d, 0x20,
0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x42e, 0x43a, 0x43a, 0x44a, 0x435, 0x440, 0x430, 0x20, 0x410, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x42e, 0x43a, 0x43a, 0x44a, 0x435, 0x440, 0x430, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x42e, 0x43a, 0x43a, 0x44a, 0x435, 0x440, 0x430, 0x20, 0x410, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d,
0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x42e, 0x43a, 0x43a, 0x44a, 0x435, 0x440,
0x430, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x2c, 0x20,
0x43c, 0x430, 0x43b, 0x445, 0x431, 0x443, 0x437, 0x435, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x42e, 0x43a, 0x43a, 0x44a, 0x435, 0x440, 0x430, 0x20, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x2c, 0x20, 0x43c, 0x430, 0x43b, 0x445, 0x431,
0x443, 0x437, 0x435, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x42e, 0x43a, 0x43a, 0x44a, 0x435,
0x440, 0x430, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x2c,
0x20, 0x43c, 0x430, 0x43b, 0x445, 0x431, 0x443, 0x437, 0x435, 0x43d, 0x20, 0x430,
0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c,
0x430, 0x43b, 0x445, 0x431, 0x430, 0x43b, 0x435, 0x43d, 0x20, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x430, 0x43b,
0x435, 0x43d, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x430, 0x43b, 0x435, 0x43d,
0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x2c, 0x20, 0x430,
0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c,
0x430, 0x43b, 0x445, 0x431, 0x443, 0x437, 0x435, 0x43d, 0x20, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x443, 0x437,
0x435, 0x43d, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x443, 0x437, 0x435, 0x43d,
0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x2c, 0x20, 0x430,
0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x410,
0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x434, 0x436, 0x430, 0x43d, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445,
0x430, 0x43d, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x434, 0x436, 0x430,
0x43d, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x410, 0x437, 0x43e, 0x440, 0x430, 0x43d, 0x20, 0x433, 0x4c0,
0x430, 0x439, 0x440, 0x435, 0x43d, 0x430, 0x448, 0x410, 0x437, 0x43e, 0x440, 0x430,
0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439, 0x440, 0x435, 0x43d, 0x430, 0x448, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x410, 0x437, 0x43e, 0x440, 0x430, 0x43d, 0x20, 0x433, 0x4c0,
0x430, 0x439, 0x440, 0x435, 0x43d, 0x430, 0x448, 0x2c, 0x20, 0x430, 0x44c, 0x445,
0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x411, 0x430, 0x43d,
0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x411, 0x430,
0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x2c, 0x20, 0x430, 0x44c, 0x445,
0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x411, 0x440, 0x430,
0x437, 0x438, 0x43b, 0x438, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x411, 0x440, 0x430, 0x437,
0x438, 0x43b, 0x438, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x411, 0x440, 0x443, 0x43d, 0x435, 0x439, 0x2d,
0x414, 0x430, 0x440, 0x443, 0x441, 0x441, 0x430, 0x43b, 0x430, 0x43c, 0x41a, 0x430,
0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x2c, 0x20,
0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x427, 0x430, 0x442, 0x435, 0x43c, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x427, 0x430, 0x442,
0x435, 0x43c, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x427, 0x438, 0x43b, 0x438, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x427, 0x438, 0x43b, 0x438, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d,
0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x426, 0x438, 0x439, 0x447, 0x43e, 0x44c,
0x426, 0x438, 0x439, 0x447, 0x43e, 0x44c, 0x43d, 0x430, 0x43d, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x426, 0x438, 0x439, 0x447, 0x43e, 0x44c, 0x43d, 0x430, 0x43d, 0x2c, 0x20,
0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x4c0, 0x438, 0x439, 0x441, 0x430, 0x20, 0x43f, 0x430, 0x439, 0x445, 0x430, 0x43c,
0x430, 0x440, 0x20, 0x432, 0x438, 0x43d, 0x447, 0x443, 0x20, 0x434, 0x435, 0x43d,
0x20, 0x433, 0x4c0, 0x430, 0x439, 0x440, 0x435, 0x41a, 0x43e, 0x43a, 0x43e, 0x441,
0x438, 0x439, 0x43d, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x41a,
0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41a, 0x43e,
0x43b, 0x443, 0x43c, 0x431, 0x438, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435,
0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41a, 0x443, 0x43a, 0x430, 0x43d,
0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x41a, 0x443, 0x43a, 0x430,
0x43d, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x41a, 0x443, 0x43a, 0x430, 0x43d, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d,
0x430, 0x448, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x41a, 0x443, 0x431, 0x430, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x41a, 0x443, 0x431, 0x430, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d,
0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x414, 0x44e, 0x43c, 0x43e, 0x43d, 0x2d,
0x434, 0x2019, 0x42e, 0x440, 0x432, 0x438, 0x43b, 0x44c, 0x41c, 0x430, 0x43b, 0x445,
0x431, 0x430, 0x43b, 0x435, 0x43d, 0x20, 0x422, 0x438, 0x43c, 0x43e, 0x440, 0x41c,
0x430, 0x440, 0x445, 0x438, 0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439, 0x440, 0x435,
0x41c, 0x430, 0x440, 0x445, 0x438, 0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439, 0x440,
0x435, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x440, 0x445, 0x438, 0x43d, 0x20,
0x433, 0x4c0, 0x430, 0x439, 0x440, 0x435, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a,
0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x42e, 0x43a, 0x43a, 0x44a,
0x435, 0x440, 0x430, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x42e, 0x43a,
0x43a, 0x44a, 0x435, 0x440, 0x430, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x42e, 0x43a, 0x43a, 0x44a, 0x435, 0x440, 0x430, 0x20,
0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a,
0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445,
0x431, 0x430, 0x43b, 0x435, 0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430,
0x41c, 0x430, 0x43b, 0x445, 0x431, 0x430, 0x43b, 0x435, 0x43d, 0x20, 0x415, 0x432,
0x440, 0x43e, 0x43f, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445,
0x431, 0x430, 0x43b, 0x435, 0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430,
0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445,
0x430, 0x43d, 0x41a, 0x430, 0x43b, 0x438, 0x43d, 0x438, 0x43d, 0x433, 0x440, 0x430,
0x434, 0x435, 0x440, 0x430, 0x20, 0x430, 0x2c, 0x20, 0x41c, 0x438, 0x43d, 0x441,
0x43a, 0x435, 0x440, 0x430, 0x20, 0x430, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430,
0x43b, 0x445, 0x431, 0x443, 0x437, 0x435, 0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e,
0x43f, 0x430, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x443, 0x437, 0x435, 0x43d, 0x20,
0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430,
0x43b, 0x445, 0x431, 0x443, 0x437, 0x435, 0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e,
0x43f, 0x430, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434,
0x430, 0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439, 0x440, 0x435, 0x43d, 0x430, 0x448,
0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x430, 0x43d, 0x20, 0x433,
0x4c0, 0x430, 0x439, 0x440, 0x435, 0x43d, 0x430, 0x448, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x430, 0x43d, 0x20, 0x433,
0x4c0, 0x430, 0x439, 0x440, 0x435, 0x43d, 0x430, 0x448, 0x2c, 0x20, 0x430, 0x44c,
0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x424, 0x438,
0x434, 0x436, 0x438, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x424, 0x438, 0x434, 0x436, 0x438,
0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445,
0x430, 0x43d, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x443, 0x437, 0x438, 0x439, 0x43d,
0x20, 0x413, 0x432, 0x438, 0x430, 0x43d, 0x430, 0x424, 0x440, 0x430, 0x43d, 0x446,
0x443, 0x437, 0x438, 0x439, 0x43d, 0x20, 0x43a, 0x44a, 0x438, 0x43b, 0x431, 0x430,
0x20, 0x430, 0x2c, 0x20, 0x410, 0x43d, 0x442, 0x430, 0x440, 0x43a, 0x442, 0x438,
0x434, 0x430, 0x43d, 0x20, 0x430, 0x20, 0x445, 0x430, 0x43d, 0x413, 0x430, 0x43b,
0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441, 0x430, 0x43d, 0x20, 0x433, 0x4c0, 0x430,
0x439, 0x440, 0x435, 0x43d, 0x430, 0x448, 0x413, 0x430, 0x43c, 0x431, 0x44c, 0x435,
0x413, 0x443, 0x44c, 0x440, 0x436, 0x438, 0x439, 0x447, 0x43e, 0x44c, 0x413, 0x443,
0x44c, 0x440, 0x436, 0x438, 0x439, 0x447, 0x43e, 0x44c, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x413, 0x443, 0x44c, 0x440, 0x436, 0x438, 0x439, 0x447, 0x43e, 0x44c, 0x2c, 0x20,
0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x413, 0x438, 0x43b, 0x431, 0x435, 0x440, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x433,
0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x413, 0x440, 0x438, 0x43d, 0x432, 0x438, 0x447,
0x438, 0x446, 0x430, 0x20, 0x44e, 0x43a, 0x44a, 0x430, 0x440, 0x430, 0x20, 0x445,
0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x430, 0x43b, 0x435, 0x43d, 0x20,
0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x41c, 0x430, 0x43b,
0x445, 0x431, 0x430, 0x43b, 0x435, 0x43d, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b,
0x430, 0x43d, 0x434, 0x438, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445,
0x431, 0x430, 0x43b, 0x435, 0x43d, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430,
0x43d, 0x434, 0x438, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x443, 0x437,
0x435, 0x43d, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438,
0x41c, 0x430, 0x43b, 0x445, 0x431, 0x443, 0x437, 0x435, 0x43d, 0x20, 0x413, 0x440,
0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c,
0x430, 0x43b, 0x445, 0x431, 0x443, 0x437, 0x435, 0x43d, 0x20, 0x413, 0x440, 0x435,
0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a,
0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x413, 0x4c0, 0x430, 0x436,
0x430, 0x440, 0x438, 0x439, 0x43d, 0x20, 0x430, 0x439, 0x43c, 0x430, 0x413, 0x430,
0x439, 0x430, 0x43d, 0x430, 0x413, 0x430, 0x432, 0x430, 0x439, 0x43d, 0x2d, 0x430,
0x43b, 0x435, 0x443, 0x442, 0x438, 0x439, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x413,
0x430, 0x432, 0x430, 0x439, 0x43d, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x438,
0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x413, 0x430, 0x432, 0x430, 0x439, 0x43d, 0x2d,
0x430, 0x43b, 0x435, 0x443, 0x442, 0x438, 0x439, 0x43d, 0x20, 0x430, 0x44c, 0x445,
0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x413, 0x43e, 0x43d,
0x43a, 0x43e, 0x43d, 0x433, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x2c, 0x20,
0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x425, 0x43e, 0x432, 0x434, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x425, 0x43e, 0x432, 0x434,
0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445,
0x430, 0x43d, 0x425, 0x4c0, 0x438, 0x43d, 0x434, 0x438, 0x418, 0x43d, 0x434, 0x438,
0x43d, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x42e, 0x43a, 0x43a, 0x44a, 0x435,
0x440, 0x430, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x41c,
0x430, 0x43b, 0x445, 0x431, 0x430, 0x43b, 0x435, 0x43d, 0x20, 0x418, 0x43d, 0x434,
0x43e, 0x43d, 0x435, 0x437, 0x438, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x443, 0x437,
0x435, 0x43d, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x413,
0x4c0, 0x430, 0x436, 0x430, 0x440, 0x438, 0x439, 0x447, 0x43e, 0x44c, 0x413, 0x4c0,
0x430, 0x436, 0x430, 0x440, 0x438, 0x439, 0x447, 0x43e, 0x44c, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x413, 0x4c0, 0x430, 0x436, 0x430, 0x440, 0x438, 0x439, 0x447, 0x43e, 0x44c,
0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445,
0x430, 0x43d, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x2c, 0x20, 0x430, 0x44c,
0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x418, 0x437,
0x440, 0x430, 0x438, 0x43b, 0x44c, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x418, 0x437, 0x440,
0x430, 0x438, 0x43b, 0x44c, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d,
0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x42f, 0x43f, 0x43e, 0x43d, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445,
0x430, 0x43d, 0x42f, 0x43f, 0x43e, 0x43d, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a,
0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445,
0x431, 0x430, 0x43b, 0x435, 0x43d, 0x20, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441,
0x442, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x443, 0x437, 0x435, 0x43d,
0x20, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x41a, 0x43e,
0x440, 0x435, 0x439, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41a, 0x43e, 0x440, 0x435, 0x439,
0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445,
0x430, 0x43d, 0x41a, 0x43e, 0x441, 0x440, 0x430, 0x44d, 0x41a, 0x440, 0x430, 0x441,
0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41a, 0x440,
0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x2c, 0x20, 0x430, 0x44c,
0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41b, 0x430,
0x439, 0x43d, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x41b, 0x43e,
0x440, 0x434, 0x2d, 0x425, 0x430, 0x443, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41b, 0x43e,
0x440, 0x434, 0x2d, 0x425, 0x430, 0x443, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a,
0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x433, 0x430,
0x434, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x433, 0x430, 0x434,
0x430, 0x43d, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x44c, 0x434, 0x438, 0x432, 0x430,
0x448, 0x41c, 0x430, 0x440, 0x43a, 0x438, 0x437, 0x430, 0x43d, 0x2c, 0x20, 0x433,
0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x41c, 0x430, 0x440, 0x448, 0x430, 0x43b, 0x430,
0x43d, 0x20, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x41c, 0x430,
0x432, 0x440, 0x438, 0x43a, 0x438, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x432,
0x440, 0x438, 0x43a, 0x438, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d,
0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x422, 0x438, 0x439, 0x43d, 0x430, 0x20,
0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x430, 0x43d, 0x20, 0x41c, 0x435, 0x43a, 0x441,
0x438, 0x43a, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x422, 0x438, 0x439, 0x43d,
0x430, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x430, 0x43d, 0x20, 0x41c, 0x435,
0x43a, 0x441, 0x438, 0x43a, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x422, 0x438, 0x439,
0x43d, 0x430, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x430, 0x43d, 0x20, 0x41c,
0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430, 0x43d, 0x20, 0x430, 0x44c, 0x445, 0x43a,
0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x423, 0x43b, 0x430, 0x43d,
0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x423, 0x43b,
0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x2c, 0x20, 0x430, 0x44c,
0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x43e,
0x441, 0x43a, 0x432, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x43e, 0x441, 0x43a,
0x432, 0x430, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x41c, 0x44c, 0x44f, 0x43d, 0x43c, 0x430, 0x41a, 0x435,
0x440, 0x43b, 0x430, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438,
0x41a, 0x435, 0x440, 0x43b, 0x430, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e,
0x43d, 0x438, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41a, 0x435, 0x440, 0x43b, 0x430, 0x20,
0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x2c, 0x20, 0x430, 0x44c,
0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41a, 0x435,
0x440, 0x43b, 0x430, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x41a,
0x435, 0x440, 0x43b, 0x430, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x41a, 0x435, 0x440, 0x43b, 0x430, 0x20, 0x417, 0x435,
0x43b, 0x430, 0x43d, 0x434, 0x438, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435,
0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41d, 0x44c, 0x44e, 0x444, 0x430,
0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x41d, 0x44c, 0x44e, 0x444, 0x430,
0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41d,
0x44c, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x2c,
0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x41d, 0x438, 0x443, 0x44d, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x2c,
0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x438,
0x2d, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x424, 0x435, 0x440, 0x43d,
0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x438, 0x2d, 0x41d, 0x43e, 0x440, 0x43e,
0x43d, 0x44c, 0x44f, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x424, 0x435, 0x440, 0x43d, 0x430,
0x43d, 0x434, 0x443, 0x2d, 0x434, 0x438, 0x2d, 0x41d, 0x43e, 0x440, 0x43e, 0x43d,
0x44c, 0x44f, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438,
0x440, 0x441, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41d, 0x43e, 0x432, 0x43e, 0x441,
0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a,
0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41e, 0x43c, 0x441, 0x43a,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x41e, 0x43c, 0x441, 0x43a, 0x2c, 0x20, 0x430, 0x44c,
0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41f, 0x430,
0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41f, 0x430,
0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a,
0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41f, 0x430, 0x43f, 0x443,
0x430, 0x20, 0x2013, 0x20, 0x41a, 0x435, 0x440, 0x43b, 0x430, 0x20, 0x413, 0x432,
0x438, 0x43d, 0x435, 0x439, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439,
0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445,
0x430, 0x43d, 0x41f, 0x435, 0x440, 0x443, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41f, 0x435,
0x440, 0x443, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d,
0x430, 0x448, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x430, 0x448,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d,
0x430, 0x448, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x424, 0x435, 0x43d, 0x438, 0x43a, 0x441, 0x2c, 0x20,
0x433, 0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x44c,
0x435, 0x440, 0x20, 0x430, 0x2c, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e,
0x43d, 0x20, 0x430, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20,
0x430, 0x2c, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x430,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x44c, 0x435, 0x440,
0x20, 0x430, 0x2c, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20,
0x430, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x41f, 0x438, 0x442, 0x43a, 0x44d, 0x440, 0x43d, 0x41f, 0x43e,
0x43d, 0x430, 0x43f, 0x435, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d, 0x430, 0x448,
0x420, 0x435, 0x44e, 0x43d, 0x44c, 0x43e, 0x43d, 0x421, 0x430, 0x445, 0x430, 0x43b,
0x438, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438,
0x43d, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x421, 0x430, 0x43c, 0x43e, 0x430, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435,
0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x421, 0x435, 0x439, 0x448, 0x435,
0x43b, 0x430, 0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439, 0x440, 0x435, 0x43d, 0x430,
0x448, 0x421, 0x43e, 0x43b, 0x43e, 0x43c, 0x43e, 0x43d, 0x430, 0x43d, 0x2c, 0x20,
0x433, 0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x41a, 0x44a, 0x438, 0x43b, 0x431, 0x430,
0x20, 0x413, 0x435, 0x43e, 0x440, 0x433, 0x438, 0x422, 0x430, 0x438, 0x442, 0x438,
0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x422, 0x430, 0x439, 0x432,
0x430, 0x43d, 0x44c, 0x422, 0x430, 0x439, 0x432, 0x430, 0x43d, 0x44c, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445,
0x430, 0x43d, 0x422, 0x430, 0x439, 0x432, 0x430, 0x43d, 0x44c, 0x2c, 0x20, 0x430,
0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x422,
0x43e, 0x43d, 0x433, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x422, 0x43e, 0x43d, 0x433,
0x430, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x427, 0x443, 0x443, 0x43a, 0x422, 0x443, 0x440, 0x43a, 0x43c,
0x435, 0x43d, 0x438, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x422, 0x443, 0x440, 0x43a, 0x43c,
0x435, 0x43d, 0x438, 0x43d, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x2c,
0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x430,
0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442,
0x430, 0x43d, 0x430, 0x43d, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x2c,
0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x412, 0x435, 0x43d, 0x435, 0x441, 0x443, 0x44d, 0x43b, 0x430, 0x412, 0x43b,
0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a,
0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445,
0x430, 0x43d, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434,
0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445,
0x430, 0x43d, 0x423, 0x44d, 0x439, 0x43a, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x435,
0x423, 0x43e, 0x43b, 0x43b, 0x438, 0x441, 0x20, 0x430, 0x2c, 0x20, 0x424, 0x443,
0x442, 0x443, 0x43d, 0x430, 0x20, 0x430, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x2c, 0x20,
0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d,
0x431, 0x443, 0x440, 0x433, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d,
0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x13a0, 0x13eb, 0x13a8, 0x13c2, 0x13cd, 0x13d6,
0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13f0, 0x13df, 0x20, 0x13ac,
0x13bf, 0x13a8, 0x13cd, 0x13db, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7,
0x13b8, 0x13ac, 0x20, 0x13ac, 0x13bf, 0x13a8, 0x13cd, 0x13db, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13e7, 0x13a6, 0x13be, 0x13ee, 0x20, 0x13ac, 0x13bf, 0x13a8, 0x13cd, 0x13db,
0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13ac, 0x13bf, 0x13a8, 0x13cd, 0x13db, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13ac, 0x13bf, 0x13a8,
0x13cd, 0x13db, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13ac, 0x13bf, 0x13a8, 0x13cd, 0x13db,
0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13b3, 0x13cd,
0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13b3, 0x13cd, 0x13a6, 0x20,
0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0,
0x13b3, 0x13cd, 0x13a6, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13ba, 0x13cc, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13a0, 0x13ba, 0x13cc, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13ba, 0x13cc, 0x13c2, 0x20, 0x13aa, 0x13a9,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13f0, 0x13df, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13f0, 0x13df, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13f0, 0x13df, 0x20, 0x13aa, 0x13af,
0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7, 0x13b8,
0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7,
0x13b8, 0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20, 0x13d7, 0x13dc,
0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13a3, 0x13d3, 0x13b8, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a3, 0x13d3, 0x13b8,
0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13a3, 0x13d3, 0x13b8, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13aa, 0x13af,
0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13c8, 0x13a0,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13c8, 0x13a0, 0x20, 0x13a0, 0x13df,
0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13c8, 0x13a0,
0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13a0, 0x13b4, 0x13c8, 0x13ef, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13b4,
0x13c8, 0x13ef, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13a0, 0x13b4, 0x13c8, 0x13ef, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13e5, 0x13c2, 0x13d8, 0x13c2, 0x13a0,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13e5, 0x13c2, 0x13d8, 0x13c2, 0x13a0,
0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13a0, 0x13e5, 0x13c2, 0x13d8, 0x13c2, 0x13a0, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13a0,
0x13e5, 0x13c2, 0x13d8, 0x13c2, 0x13a0, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ed,
0x13d5, 0x13b5, 0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13a0, 0x13e5, 0x13c2, 0x13d8, 0x13c2,
0x13a0, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13a0, 0x13e5, 0x13c2,
0x13d8, 0x13c2, 0x13a0, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13b5, 0x13bb, 0x13c2, 0x13a0, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13b5, 0x13bb, 0x13c2, 0x13a0, 0x20, 0x13a0,
0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13b5,
0x13bb, 0x13c2, 0x13a0, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7,
0x13b8, 0x13ac, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13f0, 0x13df, 0x20, 0x13a1, 0x13b3,
0x13d7, 0x13dc, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13f0, 0x13df, 0x20,
0x13a1, 0x13b3, 0x13d7, 0x13dc, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13f0, 0x13df, 0x20, 0x13a1, 0x13b3, 0x13d7, 0x13dc,
0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13a0, 0x13f0, 0x13df, 0x20, 0x13a1, 0x13b3, 0x13d7, 0x13dc, 0x20, 0x13ed, 0x13d5, 0x13b5,
0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13f0,
0x13df, 0x20, 0x13a1, 0x13b3, 0x13d7, 0x13dc, 0x20, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20,
0x13d7, 0x13dc, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13a0, 0x13f0, 0x13df, 0x20, 0x13a1, 0x13b3, 0x13d7, 0x13dc, 0x20, 0x13ed,
0x13d5, 0x13b5, 0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a1, 0x13b3, 0x13d7, 0x13dc, 0x20, 0x13d7,
0x13a7, 0x13b8, 0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13a1, 0x13b3, 0x13d7, 0x13dc, 0x20, 0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20, 0x13d7, 0x13dc,
0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13a1, 0x13b3, 0x13d7, 0x13dc, 0x20, 0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20, 0x13d7, 0x13dc,
0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13a0, 0x13cf, 0x13b5, 0x13c6, 0x13cc, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13a0, 0x13cf, 0x13b5, 0x13c6, 0x13cc, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13cf, 0x13b5, 0x13c6, 0x13cc, 0x13c2,
0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13d0, 0x13b4,
0x13cf, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13d0, 0x13b4, 0x13cf, 0x20,
0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0,
0x13d0, 0x13b4, 0x13cf, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13c6, 0x13c2, 0x13a6, 0x13b5, 0x13d5, 0x13cd, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13c6, 0x13c2, 0x13a6, 0x13b5, 0x13d5, 0x13cd, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c6, 0x13c2, 0x13a6, 0x13b5, 0x13d5, 0x13cd,
0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ca, 0x13d4, 0x13c2,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c9, 0x13b5, 0x13eb, 0x13a0, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c6, 0x13cf, 0x13b5, 0x13ef, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13c6, 0x13cf, 0x13b5, 0x13ef, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c6, 0x13cf, 0x13b5, 0x13ef, 0x20, 0x13aa,
0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ca, 0x13be, 0x13a2, 0x20, 0x13d3,
0x13b7, 0x13cc, 0x13b3, 0x13bb, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13ac,
0x13be, 0x13d5, 0x13be, 0x20, 0x13a2, 0x13e4, 0x13f3, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13ac, 0x13be, 0x13d5, 0x13be, 0x20, 0x13a2, 0x13e4, 0x13f3,
0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13a2, 0x13ac, 0x13be, 0x13d5, 0x13be, 0x20, 0x13a2, 0x13e4, 0x13f3, 0x13cd,
0x13d7, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e3, 0x13bc,
0x13b6, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13e3, 0x13dd, 0x13bb, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e3, 0x13dd,
0x13bb, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13e3, 0x13dd, 0x13bb, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e5, 0x13b5, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13e5, 0x13b5, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13e5, 0x13b5, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13d3, 0x13b6, 0x13c2, 0x13a8, 0x13cd, 0x13db, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13d3, 0x13b6, 0x13c2, 0x13a8, 0x13cd, 0x13db, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd,
0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d3, 0x13b6, 0x13c2, 0x13a8, 0x13cd,
0x13db, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13d3, 0x13c2, 0x13cd, 0x13d3, 0x13f2, 0x13af, 0x13b2, 0x20, 0x13a4, 0x13a6, 0x13da,
0x13db, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13aa, 0x13aa, 0x13cd, 0x20,
0x13da, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13aa,
0x13b8, 0x13bb, 0x13c8, 0x13a2, 0x13a0, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13aa,
0x13b8, 0x13bb, 0x13c8, 0x13a2, 0x13a0, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13aa, 0x13b8, 0x13bb, 0x13c8, 0x13a2, 0x13a0, 0x20,
0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13d3, 0x13cd, 0x13d3,
0x13f4, 0x13b2, 0x13cd, 0x13a9, 0x20, 0x13da, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13d3, 0x13cd, 0x13d3, 0x13f4, 0x13b2, 0x13cd, 0x13a9,
0x20, 0x13da, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13d3, 0x13cd, 0x13d3, 0x13f4, 0x13b2,
0x13cd, 0x13a9, 0x20, 0x13da, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13a0, 0x13f0, 0x13df,
0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ab, 0x13c6, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ab, 0x13c6, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd,
0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ab, 0x13c6, 0x20, 0x13aa, 0x13af,
0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d5, 0x13eb, 0x13cf,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13da, 0x13bc, 0x13be, 0x13d8, 0x2d, 0x13d7,
0x2019, 0x13a4, 0x13b5, 0x13eb, 0x13b5, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7,
0x13a7, 0x13b8, 0x13ac, 0x20, 0x13d8, 0x13bc, 0x13b5, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13e5, 0x13cc, 0x13d5, 0x13b4, 0x13af, 0x13cc, 0x13c5, 0x20, 0x13a4, 0x13a6, 0x13da,
0x13db, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e5, 0x13cc, 0x13d5, 0x13b4,
0x13af, 0x13cc, 0x13c5, 0x20, 0x13a4, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13a0, 0x13df,
0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e5, 0x13cc, 0x13d5,
0x13b4, 0x13af, 0x13cc, 0x13c5, 0x20, 0x13a4, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13aa,
0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a1, 0x13c6, 0x13d9, 0x13b5, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13f0, 0x13df, 0x20, 0x13f3, 0x13b3, 0x13c8,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13f0, 0x13df, 0x20, 0x13f3, 0x13b3,
0x13c8, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13a0, 0x13f0, 0x13df, 0x20, 0x13f3, 0x13b3, 0x13c8, 0x20, 0x13aa, 0x13a9, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20, 0x13d7, 0x13dc,
0x20, 0x13f3, 0x13b3, 0x13c8, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7,
0x13b8, 0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13f3, 0x13b3, 0x13c8, 0x20, 0x13a0, 0x13df,
0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7, 0x13b8,
0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13f3, 0x13b3, 0x13c8, 0x20, 0x13aa, 0x13a9, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20, 0x13f3, 0x13b3,
0x13c8, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20,
0x13d7, 0x13dc, 0x20, 0x13f3, 0x13b3, 0x13c8, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13f3, 0x13b3, 0x13c8, 0x20,
0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ed,
0x13d5, 0x13b5, 0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13f3, 0x13b3, 0x13c8, 0x20, 0x13aa,
0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e9, 0x13a9, 0x20, 0x13da, 0x13a6,
0x13da, 0x13db, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e9, 0x13a9, 0x20,
0x13da, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e9, 0x13a9, 0x20, 0x13da, 0x13a6, 0x13da, 0x13db,
0x13a2, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13eb, 0x13e5,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13eb, 0x13e5, 0x20, 0x13a0, 0x13df, 0x13b6,
0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13eb, 0x13e5, 0x20, 0x13aa,
0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13c2, 0x13a6, 0x13b8, 0x20,
0x13c8, 0x13ef, 0x13be, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13c2, 0x13a6,
0x13b8, 0x13e5, 0x20, 0x13a4, 0x13a6, 0x13c3, 0x13ee, 0x20, 0x26, 0x20, 0x13a4, 0x13c1,
0x13cd, 0x13d3, 0x13b6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a1, 0x13c6, 0x20,
0x13d3, 0x13a6, 0x13cf, 0x20, 0x13a4, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13a6, 0x13bb, 0x13c7, 0x13b5, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13e3, 0x13a0, 0x13e5, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e3,
0x13a0, 0x13e5, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13e3, 0x13a0, 0x13e5, 0x13a2, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a9, 0x13b5, 0x13c7, 0x13d8, 0x20, 0x13da, 0x13a6, 0x13da,
0x13db, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13e4, 0x20, 0x13a2,
0x13f3, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7, 0x13b8,
0x13ac, 0x20, 0x13a2, 0x13e4, 0x13cd, 0x13db, 0x13f1, 0x20, 0x13a0, 0x13b5, 0x13a2, 0x13b5,
0x13d2, 0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20, 0x13a2, 0x13e4, 0x13cd, 0x13db, 0x13f1, 0x20,
0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13b5, 0x13a2, 0x13b5, 0x13d2, 0x13d7,
0x13a7, 0x13b8, 0x13ac, 0x20, 0x13a2, 0x13e4, 0x13cd, 0x13db, 0x13f1, 0x20, 0x13aa, 0x13a9,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13a2,
0x13e4, 0x13cd, 0x13db, 0x13f1, 0x20, 0x13a0, 0x13b5, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5,
0x13b5, 0x13ac, 0x20, 0x13a2, 0x13e4, 0x13cd, 0x13db, 0x13f1, 0x20, 0x13a0, 0x13df, 0x13b6,
0x13cd, 0x13d7, 0x20, 0x13a0, 0x13b5, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5, 0x13b5, 0x13ac,
0x20, 0x13a2, 0x13e4, 0x13cd, 0x13db, 0x13f1, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13a1, 0x13c9, 0x13c4, 0x13b8, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13b6,
0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a6, 0x13ef, 0x13be, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ad, 0x13e9, 0x13f1, 0x2d, 0x13a0, 0x13b5, 0x13f3,
0x13cf, 0x13a0, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ad, 0x13e9, 0x13f1,
0x2d, 0x13a0, 0x13b5, 0x13f3, 0x13cf, 0x13a0, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd,
0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ad, 0x13e9, 0x13f1, 0x2d, 0x13a0,
0x13b5, 0x13f3, 0x13cf, 0x13a0, 0x13c2, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b0, 0x13c2, 0x13a9, 0x20, 0x13aa, 0x13c2, 0x13a9,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b0, 0x13c2, 0x13a9, 0x20, 0x13aa, 0x13c2,
0x13a9, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13b0, 0x13c2, 0x13a9, 0x20, 0x13aa, 0x13c2, 0x13a9, 0x20, 0x13aa, 0x13a9, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b0, 0x13e9, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13b0, 0x13e9, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b0, 0x13e9, 0x13d7, 0x20, 0x13aa, 0x13a9, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13c2, 0x13d7, 0x13a2, 0x13a0, 0x20, 0x13a0,
0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13c2,
0x13f4, 0x13eb, 0x13ef, 0x20, 0x13a0, 0x13ba, 0x13c9, 0x13af, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13a2, 0x13c2, 0x13d9, 0x13d3, 0x13b6, 0x13c2, 0x13a8, 0x13cd, 0x13db, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13f0, 0x13df, 0x20, 0x13a2, 0x13c2, 0x13d9,
0x13c2, 0x13cd, 0x13ef, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7, 0x13b8,
0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13a2, 0x13c2, 0x13d9, 0x13c2, 0x13cd, 0x13ef, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13d7, 0x13dc,
0x20, 0x13a2, 0x13c2, 0x13d9, 0x13c2, 0x13cd, 0x13ef, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13a2, 0x13b3, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13b3,
0x13c2, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13a2, 0x13b3, 0x13c2, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13ab, 0x13e5, 0x13a7, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13a2, 0x13ab, 0x13e5, 0x13a7, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13ab, 0x13e5, 0x13a7, 0x20, 0x13aa,
0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13cf, 0x13b5, 0x13f1, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13cf, 0x13b5, 0x13f1, 0x20, 0x13a0, 0x13df,
0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13cf, 0x13b5,
0x13f1, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13a9, 0x13e3, 0x13e9, 0x13c2, 0x13cf, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13e3, 0x13e9, 0x13c2, 0x13cf, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e3, 0x13e9, 0x13c2, 0x13cf, 0x20, 0x13aa, 0x13af, 0x20,
0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a7, 0x13ce, 0x13a7, 0x13cd,
0x13d5, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7, 0x13b8, 0x13ac,
0x20, 0x13a7, 0x13ce, 0x13a7, 0x13cd, 0x13d5, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13a7, 0x13ce, 0x13a7, 0x13cd, 0x13d5, 0x13c2,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13aa, 0x13b5, 0x13a0, 0x13c2, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13aa, 0x13b5, 0x13a0, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13b6,
0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13aa, 0x13b5, 0x13a0, 0x13c2,
0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13aa, 0x13cd, 0x13b4, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13dd, 0x13cd, 0x13c3,
0x13ef, 0x13cd, 0x13a7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13dd, 0x13cd, 0x13c3,
0x13ef, 0x13cd, 0x13a7, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13dd, 0x13cd, 0x13c3, 0x13ef, 0x13cd, 0x13a7, 0x20, 0x13aa, 0x13a9,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a9, 0x13b5, 0x13e3, 0x13a2, 0x13cd, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13cd, 0x13d3, 0x13c5, 0x13c5, 0x20, 0x13da,
0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a4, 0x13ac,
0x13eb, 0x13f3, 0x13af, 0x20, 0x13ad, 0x13eb, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13a4, 0x13ac, 0x13eb, 0x13f3, 0x13af, 0x20, 0x13ad, 0x13eb, 0x20, 0x13a0, 0x13df, 0x13b6,
0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a4, 0x13ac, 0x13eb, 0x13f3,
0x13af, 0x20, 0x13ad, 0x13eb, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b9, 0x13a6, 0x13d3, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13b9, 0x13a6, 0x13d3, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b9, 0x13a6, 0x13d3, 0x13c2, 0x20, 0x13aa,
0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b9, 0x13b4, 0x13cf, 0x13a2, 0x13a0,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b9, 0x13b5, 0x13d7, 0x13eb, 0x13cd, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b9, 0x13b5, 0x13a8, 0x13cc, 0x13cf, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b9, 0x13cc, 0x13b5, 0x20, 0x13da, 0x13a6, 0x13da, 0x13db,
0x13a2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13bc, 0x13b5, 0x13cf, 0x13a5, 0x13cd,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13bc, 0x13b5, 0x13cf, 0x13a5, 0x13cd, 0x20,
0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13bc,
0x13b5, 0x13cf, 0x13a5, 0x13cd, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13b9, 0x13cc, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13c2,
0x13cd, 0x13c6, 0x13c2, 0x20, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13a0, 0x13c2, 0x13cd, 0x13c6, 0x13c2, 0x20, 0x13ed, 0x13d5, 0x13b5, 0x13ac,
0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13a0, 0x13c2, 0x13cd, 0x13c6, 0x13c2, 0x20, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13aa,
0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a4, 0x13b3,
0x13c2, 0x20, 0x13c6, 0x13d9, 0x13b8, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a4,
0x13b3, 0x13c2, 0x20, 0x13c6, 0x13d9, 0x13b8, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a4, 0x13b3, 0x13c2, 0x20, 0x13c6, 0x13d9,
0x13b8, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b9, 0x13cd,
0x13a6, 0x13eb, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b9, 0x13cd, 0x13a6, 0x13eb,
0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13b9, 0x13cd, 0x13a6, 0x13eb, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13b9, 0x13ef, 0x13b9, 0x13b5, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13be,
0x13b7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c1, 0x13c6, 0x13b5, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13e4, 0x20, 0x13a7, 0x13b5, 0x13d9, 0x13c2, 0x13a0,
0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13e4, 0x20, 0x13a7, 0x13b5,
0x13d9, 0x13c2, 0x13a0, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13e4, 0x20, 0x13a7, 0x13b5, 0x13d9, 0x13c2, 0x13a0,
0x13c2, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13e4,
0x20, 0x13cf, 0x13a2, 0x13b4, 0x13c2, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13a2, 0x13e4, 0x20, 0x13cf, 0x13a2, 0x13b4, 0x13c2, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13b6,
0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13e4, 0x20, 0x13cf,
0x13a2, 0x13b4, 0x13c2, 0x13d7, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13e4, 0x13a4, 0x13c2, 0x13e9, 0x13db, 0x13d3, 0x13a6,
0x13d9, 0x13af, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13e4, 0x13a4, 0x13c2,
0x13e9, 0x13db, 0x13d3, 0x13a6, 0x13d9, 0x13af, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13e4, 0x13a4, 0x13c2, 0x13e9, 0x13db,
0x13d3, 0x13a6, 0x13d9, 0x13af, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c2, 0x13f3, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13c3, 0x13b5, 0x13ec, 0x13b5, 0x13a9, 0x20, 0x13a4, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c3, 0x13b5, 0x13ec, 0x13b5, 0x13a9, 0x20, 0x13a4,
0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c3, 0x13b5, 0x13ec, 0x13b5, 0x13a9, 0x20, 0x13a4, 0x13a6,
0x13da, 0x13db, 0x13a2, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13ea, 0x13be, 0x13c5, 0x13d9, 0x20, 0x13d5, 0x20, 0x13c3, 0x13b6, 0x13be, 0x13ad, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ea, 0x13be, 0x13c5, 0x13d9, 0x20, 0x13d5, 0x20,
0x13c3, 0x13b6, 0x13be, 0x13ad, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ea, 0x13be, 0x13c5, 0x13d9, 0x20, 0x13d5, 0x20, 0x13c3,
0x13b6, 0x13be, 0x13ad, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13c3, 0x13ec, 0x13cf, 0x13c8, 0x13cd, 0x13a7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13c3, 0x13ec, 0x13cf, 0x13c8, 0x13cd, 0x13a7, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c3, 0x13ec, 0x13cf, 0x13c8, 0x13cd, 0x13a7,
0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a3, 0x13bb, 0x13cd,
0x13a7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a3, 0x13bb, 0x13cd, 0x13a7, 0x20,
0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a3,
0x13bb, 0x13cd, 0x13a7, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13c6, 0x13a9, 0x13cd, 0x13d6, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c6,
0x13a9, 0x13cd, 0x13d6, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c6, 0x13a9, 0x13cd, 0x13d6, 0x13c2, 0x20, 0x13aa, 0x13a9,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c6, 0x13b7, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13c6, 0x13c7, 0x20, 0x13a2, 0x13e4, 0x20, 0x13a9, 0x13a2, 0x13c2, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c6, 0x13b5, 0x13c7, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13c6, 0x13b5, 0x13c7, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c6, 0x13b5, 0x13c7, 0x20, 0x13aa, 0x13a9, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c7, 0x13b7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13c7, 0x13b7, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13c7, 0x13b7, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13a0, 0x13c2, 0x13c8, 0x13b5, 0x13a9, 0x13c3, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13a0, 0x13c2, 0x13c8, 0x13b5, 0x13a9, 0x13c3, 0x20, 0x13a0, 0x13df, 0x13b6,
0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13c2, 0x13c8, 0x13b5,
0x13a9, 0x13c3, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e7,
0x13b4, 0x13af, 0x13cc, 0x13c5, 0x13af, 0x20, 0x13da, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a4, 0x13d3, 0x13c5, 0x13d8, 0x20, 0x13c8, 0x13f0,
0x20, 0x26, 0x20, 0x13bb, 0x13c7, 0x13b6, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13a4, 0x13d3, 0x13c5, 0x13d8, 0x20, 0x13c8, 0x13f0, 0x20, 0x26, 0x20, 0x13bb,
0x13c7, 0x13b6, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13a4, 0x13d3, 0x13c5, 0x13d8, 0x20, 0x13c8, 0x13f0, 0x20, 0x26,
0x20, 0x13bb, 0x13c7, 0x13b6, 0x13c2, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c8, 0x13a7, 0x13b5, 0x13c2, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13c9, 0x13be, 0x13c7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13c8, 0x13ef, 0x13c2, 0x13a9, 0x13f0, 0x13c2, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13b4, 0x13f3, 0x13c2, 0x13a0, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13b3, 0x13de, 0x13b3, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13cc, 0x13a7, 0x13b5,
0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13cc, 0x13a7, 0x13b5, 0x13c2, 0x20,
0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13cc,
0x13a7, 0x13b5, 0x13c2, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13cc, 0x13bc, 0x13a0, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13cc, 0x13bc, 0x13a0,
0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13cc, 0x13bc, 0x13a0, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13ce, 0x13e4, 0x13b4, 0x13cf, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13cf, 0x13c2, 0x13a6, 0x13c9, 0x13b5, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d0, 0x13b6, 0x13b9, 0x13c2, 0x20, 0x13da,
0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e7, 0x13a6,
0x13be, 0x13ee, 0x20, 0x13e3, 0x13a0, 0x13e5, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13d2, 0x13b5, 0x13be, 0x13bb, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13cf,
0x13f2, 0x13e9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d4, 0x13af, 0x13d8, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d4, 0x13f1, 0x13c7, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13d4, 0x13f1, 0x13c7, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d4, 0x13f1, 0x13c7, 0x20, 0x13aa, 0x13af, 0x20,
0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d4, 0x13e5, 0x13a9, 0x13cd,
0x13d5, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d9, 0x13a8, 0x13b3, 0x13a4,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d9, 0x13be, 0x13a6, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13d9, 0x13be, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d9, 0x13be, 0x13a6, 0x20, 0x13aa, 0x13a9,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e7, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13db, 0x13b5, 0x13a9, 0x13ba, 0x13c2, 0x13cd, 0x13d4, 0x13c2, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13db, 0x13b5, 0x13a9, 0x13ba, 0x13c2, 0x13cd, 0x13d4, 0x13c2,
0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13db, 0x13b5, 0x13a9, 0x13ba, 0x13c2, 0x13cd, 0x13d4, 0x13c2, 0x20, 0x13aa, 0x13a9, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13da, 0x13e9, 0x13b7, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13f3, 0x13b7, 0x13c7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13f3,
0x13b7, 0x13c7, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13f3, 0x13b7, 0x13c7, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13a4, 0x13cd, 0x13c7, 0x13a9, 0x13cd, 0x13d6, 0x13c2, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13a4, 0x13cd, 0x13c7, 0x13a9, 0x13cd, 0x13d6, 0x13c2, 0x20, 0x13a0,
0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a4, 0x13cd,
0x13c7, 0x13a9, 0x13cd, 0x13d6, 0x13c2, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13e9, 0x13c4, 0x13e9, 0x13da, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13e9, 0x13c4, 0x13e9, 0x13da, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e9, 0x13c4, 0x13e9, 0x13da, 0x20, 0x13aa, 0x13a9, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ea, 0x13c1, 0x13d1, 0x13ea, 0x13b3, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13b3, 0x13d7, 0x13ec, 0x13cd, 0x13d9, 0x13a9, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13b3, 0x13d7, 0x13ec, 0x13cd, 0x13d9, 0x13a9,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13ed, 0x13b3, 0x13d7, 0x13ec, 0x13cd, 0x13d9, 0x13a9, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ec, 0x13b6, 0x13aa, 0x13dd, 0x13d7, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13ec, 0x13b6, 0x13aa, 0x13dd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13b6,
0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ec, 0x13b6, 0x13aa, 0x13dd,
0x13d7, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ec, 0x13cd,
0x13d9, 0x13a7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a4, 0x13f0, 0x13e8, 0x20,
0x13a4, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e9,
0x13b5, 0x13cd, 0x20, 0x13a0, 0x13b4, 0x20, 0x13ca, 0x13da, 0x13be, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13ef, 0x13ab, 0x13e5, 0x13a7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13ef, 0x13ab, 0x13e5, 0x13a7, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ef, 0x13ab, 0x13e5, 0x13a7, 0x20, 0x13aa, 0x13a9,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13f0, 0x13a7, 0x13d6, 0x13b5, 0x13c2, 0x13ca,
0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13f0, 0x13a7, 0x13d6, 0x13b5, 0x13c2,
0x13ca, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13f0, 0x13a7, 0x13d6, 0x13b5, 0x13c2, 0x13ca, 0x13a6, 0x20, 0x13aa, 0x13a9,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13f3, 0x13a7, 0x13c2, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x963f, 0x514b, 0x91cc, 0x65f6, 0x95f4, 0x963f, 0x514b, 0x91cc, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x963f, 0x514b, 0x91cc, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4e2d, 0x90e8,
0x975e, 0x6d32, 0x65f6, 0x95f4, 0x4e1c, 0x90e8, 0x975e, 0x6d32, 0x65f6, 0x95f4, 0x897f, 0x90e8,
0x975e, 0x6d32, 0x65f6, 0x95f4, 0x897f, 0x90e8, 0x975e, 0x6d32, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x897f, 0x90e8, 0x975e, 0x6d32, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5317, 0x7f8e, 0x4e2d, 0x90e8,
0x65f6, 0x95f4, 0x5317, 0x7f8e, 0x4e2d, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5317, 0x7f8e,
0x4e2d, 0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5317, 0x7f8e, 0x4e1c, 0x90e8, 0x65f6, 0x95f4,
0x5317, 0x7f8e, 0x4e1c, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5317, 0x7f8e, 0x4e1c, 0x90e8,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5317, 0x7f8e, 0x5c71, 0x533a, 0x65f6, 0x95f4, 0x5317, 0x7f8e,
0x5c71, 0x533a, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5317, 0x7f8e, 0x5c71, 0x533a, 0x590f, 0x4ee4,
0x65f6, 0x95f4, 0x5317, 0x7f8e, 0x592a, 0x5e73, 0x6d0b, 0x65f6, 0x95f4, 0x5317, 0x7f8e, 0x592a,
0x5e73, 0x6d0b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5317, 0x7f8e, 0x592a, 0x5e73, 0x6d0b, 0x590f,
0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x7eb3, 0x5fb7, 0x5c14, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x963f,
0x7eb3, 0x5fb7, 0x5c14, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x514b, 0x5957, 0x65f6, 0x95f4,
0x963f, 0x514b, 0x5957, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x963f, 0x514b, 0x5957, 0x590f, 0x4ee4,
0x65f6, 0x95f4, 0x963f, 0x514b, 0x6258, 0x522b, 0x65f6, 0x95f4, 0x963f, 0x514b, 0x6258, 0x522b,
0x6807, 0x51c6, 0x65f6, 0x95f4, 0x963f, 0x514b, 0x6258, 0x522b, 0x590f, 0x4ee4, 0x65f6, 0x95f4,
0x6fb3, 0x5927, 0x5229, 0x4e9a, 0x4e2d, 0x90e8, 0x65f6, 0x95f4, 0x6fb3, 0x5927, 0x5229, 0x4e9a,
0x4e2d, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x6fb3, 0x5927, 0x5229, 0x4e9a, 0x4e2d, 0x90e8,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x6fb3, 0x5927, 0x5229, 0x4e9a, 0x4e2d, 0x897f, 0x90e8, 0x65f6,
0x95f4, 0x6fb3, 0x5927, 0x5229, 0x4e9a, 0x4e2d, 0x897f, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x6fb3, 0x5927, 0x5229, 0x4e9a, 0x4e2d, 0x897f, 0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x6fb3,
0x5927, 0x5229, 0x4e9a, 0x4e1c, 0x90e8, 0x65f6, 0x95f4, 0x6fb3, 0x5927, 0x5229, 0x4e9a, 0x4e1c,
0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x6fb3, 0x5927, 0x5229, 0x4e9a, 0x4e1c, 0x90e8, 0x590f,
0x4ee4, 0x65f6, 0x95f4, 0x6fb3, 0x5927, 0x5229, 0x4e9a, 0x897f, 0x90e8, 0x65f6, 0x95f4, 0x6fb3,
0x5927, 0x5229, 0x4e9a, 0x897f, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x6fb3, 0x5927, 0x5229,
0x4e9a, 0x897f, 0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x585e, 0x62dc, 0x7586, 0x65f6,
0x95f4, 0x963f, 0x585e, 0x62dc, 0x7586, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x963f, 0x585e, 0x62dc,
0x7586, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x73bb, 0x5229, 0x7ef4, 0x4e9a, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x6587, 0x83b1, 0x8fbe, 0x9c81, 0x8428, 0x5170, 0x65f6, 0x95f4, 0x4f5b, 0x5f97, 0x89d2,
0x65f6, 0x95f4, 0x4f5b, 0x5f97, 0x89d2, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4f5b, 0x5f97, 0x89d2,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x51ef, 0x897f, 0x65f6, 0x95f4, 0x67e5, 0x83ab, 0x7f57, 0x65f6,
0x95f4, 0x67e5, 0x5854, 0x59c6, 0x65f6, 0x95f4, 0x67e5, 0x5854, 0x59c6, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x67e5, 0x5854, 0x59c6, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5e93, 0x514b, 0x7fa4, 0x5c9b,
0x4ef2, 0x590f, 0x65f6, 0x95f4, 0x8fea, 0x8499, 0xb7, 0x8fea, 0x7ef4, 0x5c14, 0x65f6, 0x95f4,
0x5384, 0x74dc, 0x591a, 0x5c14, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x6b27, 0x6d32, 0x6781, 0x4e1c,
0x65f6, 0x95f4, 0x6cd5, 0x5c5e, 0x572d, 0x4e9a, 0x90a3, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x6cd5,
0x5c5e, 0x5357, 0x65b9, 0x548c, 0x5357, 0x6781, 0x9886, 0x5730, 0x65f6, 0x95f4, 0x79d1, 0x9686,
0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x7518, 0x6bd4, 0x5c14, 0x65f6, 0x95f4, 0x683c, 0x9c81, 0x5409,
0x4e9a, 0x65f6, 0x95f4, 0x683c, 0x9c81, 0x5409, 0x4e9a, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x683c,
0x9c81, 0x5409, 0x4e9a, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x683c, 0x6797, 0x5c3c, 0x6cbb, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x683c, 0x9675, 0x5170, 0x5c9b, 0x4e1c, 0x90e8, 0x65f6, 0x95f4, 0x683c,
0x9675, 0x5170, 0x5c9b, 0x4e1c, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x683c, 0x9675, 0x5170,
0x5c9b, 0x4e1c, 0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x683c, 0x9675, 0x5170, 0x5c9b, 0x897f,
0x90e8, 0x65f6, 0x95f4, 0x683c, 0x9675, 0x5170, 0x5c9b, 0x897f, 0x90e8, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x683c, 0x9675, 0x5170, 0x5c9b, 0x897f, 0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5173,
0x5c9b, 0x65f6, 0x95f4, 0x6d77, 0x6e7e, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5370, 0x5ea6, 0x65f6,
0x95f4, 0x4e2d, 0x5357, 0x534a, 0x5c9b, 0x65f6, 0x95f4, 0x5370, 0x5ea6, 0x5c3c, 0x897f, 0x4e9a,
0x4e2d, 0x90e8, 0x65f6, 0x95f4, 0x5370, 0x5ea6, 0x5c3c, 0x897f, 0x4e9a, 0x4e1c, 0x90e8, 0x65f6,
0x95f4, 0x5370, 0x5ea6, 0x5c3c, 0x897f, 0x4e9a, 0x897f, 0x90e8, 0x65f6, 0x95f4, 0x4f0a, 0x5c14,
0x5e93, 0x8328, 0x514b, 0x65f6, 0x95f4, 0x4f0a, 0x5c14, 0x5e93, 0x8328, 0x514b, 0x6807, 0x51c6,
0x65f6, 0x95f4, 0x4f0a, 0x5c14, 0x5e93, 0x8328, 0x514b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5f7c,
0x5f97, 0x7f57, 0x5df4, 0x752b, 0x6d1b, 0x592b, 0x65af, 0x514b, 0x2d, 0x582a, 0x5bdf, 0x52a0,
0x65f6, 0x95f4, 0x5f7c, 0x5f97, 0x7f57, 0x5df4, 0x752b, 0x6d1b, 0x592b, 0x65af, 0x514b, 0x2d,
0x582a, 0x5bdf, 0x52a0, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5f7c, 0x5f97, 0x7f57, 0x5df4, 0x752b,
0x6d1b, 0x592b, 0x65af, 0x514b, 0x2d, 0x582a, 0x5bdf, 0x52a0, 0x590f, 0x4ee4, 0x65f6, 0x95f4,
0x54c8, 0x8428, 0x514b, 0x65af, 0x5766, 0x65f6, 0x95f4, 0x54c8, 0x8428, 0x514b, 0x65af, 0x5766,
0x4e1c, 0x90e8, 0x65f6, 0x95f4, 0x54c8, 0x8428, 0x514b, 0x65af, 0x5766, 0x897f, 0x90e8, 0x65f6,
0x95f4, 0x79d1, 0x65af, 0x96f7, 0x65f6, 0x95f4, 0x5409, 0x5c14, 0x5409, 0x65af, 0x65af, 0x5766,
0x65f6, 0x95f4, 0x9a6c, 0x5c14, 0x4ee3, 0x592b, 0x65f6, 0x95f4, 0x9a6c, 0x514b, 0x8428, 0x65af,
0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x6bdb, 0x91cc, 0x6c42, 0x65af, 0x65f6, 0x95f4, 0x6bdb, 0x91cc,
0x6c42, 0x65af, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x6bdb, 0x91cc, 0x6c42, 0x65af, 0x590f, 0x4ee4,
0x65f6, 0x95f4, 0x7459, 0x9c81, 0x65f6, 0x95f4, 0x65b0, 0x5580, 0x91cc, 0x591a, 0x5c3c, 0x4e9a,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x65b0, 0x897f, 0x5170, 0x65f6, 0x95f4, 0x65b0, 0x897f, 0x5170,
0x6807, 0x51c6, 0x65f6, 0x95f4, 0x65b0, 0x897f, 0x5170, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x7ebd,
0x57c3, 0x65f6, 0x95f4, 0x8d39, 0x5c14, 0x5357, 0x591a, 0x2d, 0x8fea, 0x8bfa, 0x7f57, 0x5c3c,
0x4e9a, 0x5c9b, 0x65f6, 0x95f4, 0x8d39, 0x5c14, 0x5357, 0x591a, 0x2d, 0x8fea, 0x8bfa, 0x7f57,
0x5c3c, 0x4e9a, 0x5c9b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x8d39, 0x5c14, 0x5357, 0x591a, 0x2d,
0x8fea, 0x8bfa, 0x7f57, 0x5c3c, 0x4e9a, 0x5c9b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5e15, 0x52b3,
0x65f6, 0x95f4, 0x5df4, 0x5e03, 0x4e9a, 0x65b0, 0x51e0, 0x5185, 0x4e9a, 0x65f6, 0x95f4, 0x83f2,
0x5c3c, 0x514b, 0x65af, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x76ae, 0x7279, 0x51ef, 0x6069, 0x65f6,
0x95f4, 0x514b, 0x5b5c, 0x6d1b, 0x5c14, 0x8fbe, 0x65f6, 0x95f4, 0x514b, 0x5b5c, 0x6d1b, 0x5c14,
0x8fbe, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x514b, 0x5b5c, 0x6d1b, 0x5c14, 0x8fbe, 0x590f, 0x4ee4,
0x65f6, 0x95f4, 0x7559, 0x5c3c, 0x6c6a, 0x65f6, 0x95f4, 0x585e, 0x820c, 0x5c14, 0x65f6, 0x95f4,
0x6240, 0x7f57, 0x95e8, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x5357, 0x4e54, 0x6cbb, 0x4e9a, 0x5c9b,
0x65f6, 0x95f4, 0x82cf, 0x91cc, 0x5357, 0x65f6, 0x95f4, 0x662d, 0x548c, 0x65f6, 0x95f4, 0x5854,
0x5e0c, 0x63d0, 0x5c9b, 0x65f6, 0x95f4, 0x5854, 0x5409, 0x514b, 0x65af, 0x5766, 0x65f6, 0x95f4,
0x6258, 0x514b, 0x52b3, 0x65f6, 0x95f4, 0x6c64, 0x52a0, 0x65f6, 0x95f4, 0x6c64, 0x52a0, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x6c64, 0x52a0, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x695a, 0x514b, 0x65f6,
0x95f4, 0x571f, 0x5e93, 0x66fc, 0x65af, 0x5766, 0x65f6, 0x95f4, 0x571f, 0x5e93, 0x66fc, 0x65af,
0x5766, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x571f, 0x5e93, 0x66fc, 0x65af, 0x5766, 0x590f, 0x4ee4,
0x65f6, 0x95f4, 0x56fe, 0x74e6, 0x5362, 0x65f6, 0x95f4, 0x4e4c, 0x5179, 0x522b, 0x514b, 0x65af,
0x5766, 0x65f6, 0x95f4, 0x4e4c, 0x5179, 0x522b, 0x514b, 0x65af, 0x5766, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x4e4c, 0x5179, 0x522b, 0x514b, 0x65af, 0x5766, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x74e6,
0x52aa, 0x963f, 0x56fe, 0x65f6, 0x95f4, 0x74e6, 0x52aa, 0x963f, 0x56fe, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x74e6, 0x52aa, 0x963f, 0x56fe, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x74e6, 0x5229, 0x65af,
0x548c, 0x5bcc, 0x56fe, 0x7eb3, 0x65f6, 0x95f4, 0x96c5, 0x5e93, 0x8328, 0x514b, 0x65f6, 0x95f4,
0x96c5, 0x5e93, 0x8328, 0x514b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x96c5, 0x5e93, 0x8328, 0x514b,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5357, 0x975e, 0x6642, 0x9593, 0x5317, 0x7f8e, 0x4e2d, 0x90e8,
0x6642, 0x9593, 0x5317, 0x7f8e, 0x4e2d, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x5317, 0x7f8e,
0x4e2d, 0x90e8, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x5317, 0x7f8e, 0x6771, 0x90e8, 0x6642, 0x9593,
0x5317, 0x7f8e, 0x6771, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x5317, 0x7f8e, 0x6771, 0x90e8,
0x590f, 0x4ee4, 0x6642, 0x9593, 0x5317, 0x7f8e, 0x5c71, 0x5340, 0x6642, 0x9593, 0x5317, 0x7f8e,
0x5c71, 0x5340, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x5317, 0x7f8e, 0x5c71, 0x5340, 0x590f, 0x4ee4,
0x6642, 0x9593, 0x5317, 0x7f8e, 0x592a, 0x5e73, 0x6d0b, 0x6642, 0x9593, 0x5317, 0x7f8e, 0x592a,
0x5e73, 0x6d0b, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x5317, 0x7f8e, 0x592a, 0x5e73, 0x6d0b, 0x590f,
0x4ee4, 0x6642, 0x9593, 0x4e9e, 0x585e, 0x62dc, 0x7586, 0x6642, 0x9593, 0x4e9e, 0x585e, 0x62dc,
0x7586, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x4e9e, 0x585e, 0x62dc, 0x7586, 0x590f, 0x4ee4, 0x6642,
0x9593, 0x4f5b, 0x5f97, 0x89d2, 0x6642, 0x9593, 0x4f5b, 0x5f97, 0x89d2, 0x6a19, 0x6e96, 0x6642,
0x9593, 0x4f5b, 0x5f97, 0x89d2, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x53ef, 0x53ef, 0x65af, 0x7fa4,
0x5cf6, 0x6642, 0x9593, 0x8fea, 0x8499, 0x8fea, 0x7dad, 0x723e, 0x6642, 0x9593, 0x5384, 0x74dc,
0x591a, 0x723e, 0x6642, 0x9593, 0x52a0, 0x62c9, 0x5e15, 0x6208, 0x7fa4, 0x5cf6, 0x6642, 0x9593,
0x683c, 0x9b6f, 0x5409, 0x4e9e, 0x6642, 0x9593, 0x683c, 0x9b6f, 0x5409, 0x4e9e, 0x6a19, 0x6e96,
0x6642, 0x9593, 0x683c, 0x9b6f, 0x5409, 0x4e9e, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x6ce2, 0x65af,
0x7063, 0x6d77, 0x57df, 0x6642, 0x9593, 0x5370, 0x5ea6, 0x6642, 0x9593, 0x4f0a, 0x723e, 0x5eab,
0x8328, 0x514b, 0x6642, 0x9593, 0x4f0a, 0x723e, 0x5eab, 0x8328, 0x514b, 0x6a19, 0x6e96, 0x6642,
0x9593, 0x4f0a, 0x723e, 0x5eab, 0x8328, 0x514b, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x79d1, 0x65af,
0x96f7, 0x6642, 0x9593, 0x99ac, 0x723e, 0x4ee3, 0x592b, 0x6642, 0x9593, 0x99ac, 0x514b, 0x85a9,
0x65af, 0x6642, 0x9593, 0x6bdb, 0x91cc, 0x88d8, 0x65af, 0x6642, 0x9593, 0x6bdb, 0x91cc, 0x88d8,
0x65af, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x6bdb, 0x91cc, 0x88d8, 0x65af, 0x590f, 0x4ee4, 0x6642,
0x9593, 0x7459, 0x9b6f, 0x6642, 0x9593, 0x65b0, 0x5580, 0x91cc, 0x591a, 0x5c3c, 0x4e9e, 0x590f,
0x4ee4, 0x6642, 0x9593, 0x8cbb, 0x723e, 0x5357, 0x591a, 0x8fea, 0x8afe, 0x7f85, 0x5c3c, 0x4e9e,
0x6642, 0x9593, 0x8cbb, 0x723e, 0x5357, 0x591a, 0x8fea, 0x8afe, 0x7f85, 0x5c3c, 0x4e9e, 0x6a19,
0x6e96, 0x6642, 0x9593, 0x8cbb, 0x723e, 0x5357, 0x591a, 0x8fea, 0x8afe, 0x7f85, 0x5c3c, 0x4e9e,
0x590f, 0x4ee4, 0x6642, 0x9593, 0x5df4, 0x5e03, 0x4e9e, 0x65b0, 0x757f, 0x5167, 0x4e9e, 0x6642,
0x9593, 0x76ae, 0x7279, 0x5eb7, 0x6642, 0x9593, 0x585e, 0x820c, 0x723e, 0x6642, 0x9593, 0x65b0,
0x52a0, 0x5761, 0x6642, 0x9593, 0x6240, 0x7f85, 0x9580, 0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x8607,
0x91cc, 0x5357, 0x6642, 0x9593, 0x6e6f, 0x52a0, 0x6642, 0x9593, 0x6e6f, 0x52a0, 0x6a19, 0x6e96,
0x6642, 0x9593, 0x6e6f, 0x52a0, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x5716, 0x74e6, 0x76e7, 0x6642,
0x9593, 0x74e6, 0x52aa, 0x963f, 0x5716, 0x6642, 0x9593, 0x74e6, 0x52aa, 0x963f, 0x5716, 0x6a19,
0x6e96, 0x6642, 0x9593, 0x74e6, 0x52aa, 0x963f, 0x5716, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x96c5,
0x5eab, 0x8328, 0x514b, 0x6642, 0x9593, 0x96c5, 0x5eab, 0x8328, 0x514b, 0x6a19, 0x6e96, 0x6642,
0x9593, 0x96c5, 0x5eab, 0x8328, 0x514b, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x4e2d, 0x5357, 0x534a,
0x5cf6, 0x6642, 0x9593, 0x8056, 0x76ae, 0x57c3, 0x8207, 0x5bc6, 0x514b, 0x9686, 0x7fa4, 0x5cf6,
0x6642, 0x9593, 0x8056, 0x76ae, 0x57c3, 0x8207, 0x5bc6, 0x514b, 0x9686, 0x7fa4, 0x5cf6, 0x6a19,
0x6e96, 0x6642, 0x9593, 0x8056, 0x76ae, 0x57c3, 0x8207, 0x5bc6, 0x514b, 0x9686, 0x7fa4, 0x5cf6,
0x590f, 0x4ee4, 0x6642, 0x9593, 0x80b2, 0x7a7a, 0x5730, 0x5340, 0x6642, 0x9593, 0x441, 0x440,
0x435, 0x434, 0x43d, 0x435, 0x430, 0x43c, 0x435, 0x440, 0x457, 0x43a, 0x430, 0x301,
0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467,
0x441, 0x440, 0x435, 0x434, 0x43d, 0x435, 0x430, 0x43c, 0x435, 0x440, 0x457, 0x43a,
0x430, 0x301, 0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x437, 0x438, 0x301, 0x43c,
0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x441, 0x440,
0x435, 0x434, 0x43d, 0x435, 0x430, 0x43c, 0x435, 0x440, 0x457, 0x43a, 0x430, 0x301,
0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x432, 0x43e, 0x441, 0x442,
0x43e, 0x447, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x457, 0x43a, 0x430, 0x301,
0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467,
0x432, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440,
0x457, 0x43a, 0x430, 0x301, 0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x437, 0x438,
0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467,
0x432, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440,
0x457, 0x43a, 0x430, 0x301, 0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463,
0x301, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467,
0x430, 0x486, 0x43c, 0x435, 0x440, 0x457, 0x43a, 0x430, 0x301, 0x43d, 0x441, 0x43a,
0x43e, 0x435, 0x20, 0x43d, 0x430, 0x433, 0x43e, 0x301, 0x440, 0x43d, 0x435, 0x435,
0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x430, 0x486, 0x43c, 0x435, 0x440,
0x457, 0x43a, 0x430, 0x301, 0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43d, 0x430,
0x433, 0x43e, 0x301, 0x440, 0x43d, 0x435, 0x435, 0x20, 0x437, 0x438, 0x301, 0x43c,
0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x430, 0x486,
0x43c, 0x435, 0x440, 0x457, 0x43a, 0x430, 0x301, 0x43d, 0x441, 0x43a, 0x43e, 0x435,
0x20, 0x43d, 0x430, 0x433, 0x43e, 0x301, 0x440, 0x43d, 0x435, 0x435, 0x20, 0x43b,
0x463, 0x301, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c,
0x467, 0x442, 0x438, 0x445, 0x43e, 0x47b, 0x43a, 0x435, 0x430, 0x301, 0x43d, 0x441,
0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x442, 0x438,
0x445, 0x43e, 0x47b, 0x43a, 0x435, 0x430, 0x301, 0x43d, 0x441, 0x43a, 0x43e, 0x435,
0x20, 0x437, 0x438, 0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435,
0x301, 0x43c, 0x467, 0x442, 0x438, 0x445, 0x43e, 0x47b, 0x43a, 0x435, 0x430, 0x301,
0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x430, 0x486, 0x442, 0x43b,
0x430, 0x43d, 0x442, 0x456, 0x301, 0x447, 0x435, 0x441, 0x43a, 0x43e, 0x435, 0x20,
0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x430, 0x486, 0x442, 0x43b, 0x430, 0x43d,
0x442, 0x456, 0x301, 0x447, 0x435, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x437, 0x438,
0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467,
0x430, 0x486, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x456, 0x301, 0x447, 0x435, 0x441,
0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301, 0x442, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x441, 0x440, 0x435, 0x434, 0x43d, 0x435,
0x454, 0x475, 0x440, 0x461, 0x43f, 0x435, 0x301, 0x439, 0x441, 0x43a, 0x43e, 0x435,
0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x441, 0x440, 0x435, 0x434, 0x43d,
0x435, 0x454, 0x475, 0x440, 0x461, 0x43f, 0x435, 0x301, 0x439, 0x441, 0x43a, 0x43e,
0x435, 0x20, 0x437, 0x438, 0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440,
0x435, 0x301, 0x43c, 0x467, 0x441, 0x440, 0x435, 0x434, 0x43d, 0x435, 0x454, 0x475,
0x440, 0x461, 0x43f, 0x435, 0x301, 0x439, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b,
0x463, 0x301, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c,
0x467, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x454, 0x475, 0x440,
0x461, 0x43f, 0x435, 0x301, 0x439, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440,
0x435, 0x301, 0x43c, 0x467, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e,
0x454, 0x475, 0x440, 0x461, 0x43f, 0x435, 0x301, 0x439, 0x441, 0x43a, 0x43e, 0x435,
0x20, 0x437, 0x438, 0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435,
0x301, 0x43c, 0x467, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x454,
0x475, 0x440, 0x461, 0x43f, 0x435, 0x301, 0x439, 0x441, 0x43a, 0x43e, 0x435, 0x20,
0x43b, 0x463, 0x301, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301,
0x43c, 0x467, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x20, 0x432, 0x44a, 0x20,
0x43a, 0x430, 0x43b, 0x438, 0x43d, 0x438, 0x43d, 0x433, 0x440, 0x430, 0x301, 0x434,
0x463, 0x20, 0x438, 0x486, 0x20, 0x43c, 0x438, 0x301, 0x43d, 0x441, 0x43a, 0x463,
0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x454, 0x475, 0x440, 0x461, 0x43f,
0x435, 0x301, 0x439, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301,
0x43c, 0x467, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x454, 0x475, 0x440,
0x461, 0x43f, 0x435, 0x301, 0x439, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x437, 0x438,
0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467,
0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x454, 0x475, 0x440, 0x461, 0x43f,
0x435, 0x301, 0x439, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301, 0x442,
0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x441, 0x440,
0x435, 0x301, 0x434, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c,
0x467, 0x20, 0x43f, 0x43e, 0x20, 0x433, 0x440, 0x456, 0x301, 0x43d, 0xa64b, 0x438,
0x447, 0xa64b, 0x438, 0x486, 0x440, 0x43a, 0xa64b, 0x301, 0x442, 0x441, 0x43a, 0x43e,
0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x438, 0x486, 0x440, 0x43a,
0xa64b, 0x301, 0x442, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x437, 0x438, 0x301, 0x43c,
0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x438, 0x486,
0x440, 0x43a, 0xa64b, 0x301, 0x442, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463,
0x301, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467,
0x432, 0x43e, 0x441, 0x442, 0x43e, 0x301, 0x447, 0x43d, 0x44b, 0x439, 0x20, 0x43a,
0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x301, 0x43d, 0x44a, 0x437, 0x430,
0x301, 0x43f, 0x430, 0x434, 0x43d, 0x44b, 0x439, 0x20, 0x43a, 0x430, 0x437, 0x430,
0x445, 0x441, 0x442, 0x430, 0x301, 0x43d, 0x44a, 0x43a, 0x440, 0x430, 0x441, 0x43d,
0x43e, 0x467, 0x301, 0x440, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435,
0x301, 0x43c, 0x467, 0x43a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x467, 0x301, 0x440,
0x441, 0x43a, 0x43e, 0x435, 0x20, 0x437, 0x438, 0x301, 0x43c, 0x43d, 0x435, 0x435,
0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x43a, 0x440, 0x430, 0x441, 0x43d,
0x43e, 0x467, 0x301, 0x440, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301,
0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x43a,
0x438, 0x440, 0x433, 0x438, 0x301, 0x437, 0x457, 0x430, 0x43c, 0x430, 0x433, 0x430,
0x434, 0x430, 0x301, 0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435,
0x301, 0x43c, 0x467, 0x43c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x301, 0x43d, 0x441,
0x43a, 0x43e, 0x435, 0x20, 0x437, 0x438, 0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x43c, 0x430, 0x433, 0x430, 0x434, 0x430,
0x301, 0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301, 0x442, 0x43d,
0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x43c, 0x43e, 0x441,
0x43a, 0x43e, 0x301, 0x432, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435,
0x301, 0x43c, 0x467, 0x43c, 0x43e, 0x441, 0x43a, 0x43e, 0x301, 0x432, 0x441, 0x43a,
0x43e, 0x435, 0x20, 0x437, 0x438, 0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432,
0x440, 0x435, 0x301, 0x43c, 0x467, 0x43c, 0x43e, 0x441, 0x43a, 0x43e, 0x301, 0x432,
0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301, 0x442, 0x43d, 0x435, 0x435,
0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x43d, 0x43e, 0x432, 0x43e, 0x441,
0x438, 0x431, 0x438, 0x301, 0x440, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440,
0x435, 0x301, 0x43c, 0x467, 0x43d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438,
0x301, 0x440, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x437, 0x438, 0x301, 0x43c, 0x43d,
0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x43d, 0x43e, 0x432,
0x43e, 0x441, 0x438, 0x431, 0x438, 0x301, 0x440, 0x441, 0x43a, 0x43e, 0x435, 0x20,
0x43b, 0x463, 0x301, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301,
0x43c, 0x467, 0x47b, 0x486, 0x301, 0x43c, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432,
0x440, 0x435, 0x301, 0x43c, 0x467, 0x47b, 0x486, 0x301, 0x43c, 0x441, 0x43a, 0x43e,
0x435, 0x20, 0x437, 0x438, 0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440,
0x435, 0x301, 0x43c, 0x467, 0x47b, 0x486, 0x301, 0x43c, 0x441, 0x43a, 0x43e, 0x435,
0x20, 0x43b, 0x463, 0x301, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435,
0x301, 0x43c, 0x467, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x20, 0x43d, 0x430,
0x20, 0x441, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x301, 0x43d, 0x463, 0x437, 0x438,
0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467,
0x20, 0x43d, 0x430, 0x20, 0x441, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x301, 0x43d,
0x463, 0x43b, 0x463, 0x301, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435,
0x301, 0x43c, 0x467, 0x20, 0x43d, 0x430, 0x20, 0x441, 0x430, 0x445, 0x430, 0x43b,
0x438, 0x301, 0x43d, 0x463, 0x432, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441,
0x442, 0x43e, 0x301, 0x446, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301,
0x43c, 0x467, 0x432, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e,
0x301, 0x446, 0x43a, 0x43e, 0x435, 0x20, 0x437, 0x438, 0x301, 0x43c, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x432, 0x43b, 0x430, 0x434,
0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x301, 0x446, 0x43a, 0x43e, 0x435, 0x20,
0x43b, 0x463, 0x301, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301,
0x43c, 0x467, 0x432, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x301, 0x434,
0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x432,
0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x301, 0x434, 0x441, 0x43a, 0x43e,
0x435, 0x20, 0x437, 0x438, 0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440,
0x435, 0x301, 0x43c, 0x467, 0x432, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430,
0x301, 0x434, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301, 0x442, 0x43d,
0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0xa657, 0x486, 0x43a,
0xa64b, 0x301, 0x442, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301,
0x43c, 0x467, 0xa657, 0x486, 0x43a, 0xa64b, 0x301, 0x442, 0x441, 0x43a, 0x43e, 0x435,
0x20, 0x437, 0x438, 0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435,
0x301, 0x43c, 0x467, 0xa657, 0x486, 0x43a, 0xa64b, 0x301, 0x442, 0x441, 0x43a, 0x43e,
0x435, 0x20, 0x43b, 0x463, 0x301, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440,
0x435, 0x301, 0x43c, 0x467, 0x454, 0x486, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x457,
0x43d, 0x431, 0xa64b, 0x301, 0x440, 0x436, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432,
0x440, 0x435, 0x301, 0x43c, 0x467, 0x454, 0x486, 0x43a, 0x430, 0x442, 0x435, 0x440,
0x457, 0x43d, 0x431, 0xa64b, 0x301, 0x440, 0x436, 0x441, 0x43a, 0x43e, 0x435, 0x20,
0x437, 0x438, 0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301,
0x43c, 0x467, 0x454, 0x486, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x457, 0x43d, 0x431,
0xa64b, 0x301, 0x440, 0x436, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301,
0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x410,
0x43a, 0x440, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43a,
0x440, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b,
0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43a, 0x440, 0x438,
0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x410, 0x444, 0x433, 0x430, 0x43d, 0x438, 0x441, 0x442, 0x430,
0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x4d7, 0x43f, 0x20,
0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x425, 0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab, 0x20,
0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x41a, 0x4d1, 0x43d, 0x442, 0x4d1, 0x440, 0x20, 0x410, 0x444, 0x440, 0x438,
0x43a, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43d, 0x4d1,
0x4ab, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab, 0x20, 0x410, 0x444, 0x440, 0x438,
0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b,
0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab,
0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x4ab, 0x443, 0x43b, 0x43b,
0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43b,
0x44f, 0x441, 0x43a, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410,
0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410,
0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445,
0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43b, 0x43c, 0x430,
0x442, 0x44b, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43b, 0x43c,
0x430, 0x442, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43b, 0x43c,
0x430, 0x442, 0x44b, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d,
0x43a, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43c, 0x430,
0x437, 0x43e, 0x43d, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410,
0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a, 0x430, 0x20, 0x4ab, 0x443, 0x43b, 0x43b,
0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x4d7,
0x43f, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x4d7, 0x43f, 0x20, 0x410, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x4d7, 0x43f,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x4ab, 0x443, 0x43b,
0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x425,
0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab, 0x20, 0x410, 0x43c,
0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x425, 0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x425, 0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430,
0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x443, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x20, 0x28, 0x4aa, 0x443, 0x440, 0x4ab,
0x4d7, 0x440, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x29, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x442, 0x443,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x20, 0x28, 0x4aa, 0x443, 0x440,
0x4ab, 0x4d7, 0x440, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x29,
0x4aa, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x442, 0x443, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x20, 0x28, 0x4aa, 0x443, 0x440, 0x4ab, 0x4d7,
0x440, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x29, 0x41b, 0x4d1,
0x43f, 0x43a, 0x4d1, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x41b, 0x4d1, 0x43f, 0x43a, 0x4d1, 0x20, 0x43e, 0x43a,
0x435, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41b, 0x4d1, 0x43f,
0x43a, 0x4d1, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x20, 0x4ab, 0x443, 0x43b,
0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410,
0x43d, 0x430, 0x434, 0x44b, 0x440, 0x44c, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x410, 0x43d, 0x430, 0x434, 0x44b, 0x440, 0x44c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x410, 0x43d, 0x430, 0x434, 0x44b, 0x440, 0x44c, 0x20, 0x4ab, 0x443,
0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x410, 0x43f, 0x438, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410,
0x43f, 0x438, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43f, 0x438,
0x430, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43a, 0x442, 0x430, 0x443, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43a, 0x442, 0x430, 0x443, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x410, 0x43a, 0x442, 0x430, 0x443, 0x20, 0x4ab, 0x443, 0x43b,
0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410,
0x43a, 0x442, 0x43e, 0x431, 0x435, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x410, 0x43a, 0x442, 0x43e, 0x431, 0x435, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x410, 0x43a, 0x442, 0x43e, 0x431, 0x435, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430,
0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x440, 0x430,
0x43f, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x440, 0x430, 0x43f,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x440, 0x430, 0x43f, 0x20, 0x4ab,
0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438,
0x43d, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b,
0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x440, 0x433, 0x435,
0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445,
0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab,
0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab, 0x20, 0x410, 0x440,
0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442,
0x438, 0x43d, 0x430, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x438,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x440, 0x43c, 0x435, 0x43d,
0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x440, 0x43c, 0x435, 0x43d,
0x438, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a,
0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x442, 0x43b, 0x430,
0x43d, 0x442, 0x438, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410,
0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x430, 0x20, 0x4ab, 0x443, 0x43b,
0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422,
0x4d7, 0x43f, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x4d7, 0x43f, 0x20, 0x410, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x422, 0x4d7, 0x43f, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x422, 0x4d7, 0x43f, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x20, 0x430, 0x43d, 0x4d1, 0x4ab, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x422, 0x4d7, 0x43f, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x20, 0x430, 0x43d, 0x4d1, 0x4ab, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x422, 0x4d7, 0x43f, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x20, 0x430, 0x43d, 0x4d1, 0x4ab, 0x20, 0x4ab, 0x443, 0x43b, 0x43b,
0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x425, 0x4d7,
0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab, 0x20, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x425, 0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab, 0x20, 0x410,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x425, 0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x20, 0x4ab, 0x443, 0x43b,
0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410,
0x43d, 0x4d1, 0x4ab, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430,
0x439, 0x434, 0x436, 0x430, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x434, 0x436, 0x430, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439,
0x434, 0x436, 0x430, 0x43d, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x437, 0x43e, 0x440, 0x20,
0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x437, 0x43e, 0x440, 0x20, 0x443, 0x442, 0x440,
0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x410, 0x437, 0x43e, 0x440, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441,
0x435, 0x43d, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434,
0x435, 0x448, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x411, 0x430, 0x43d,
0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x20, 0x4ab, 0x443,
0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x411, 0x443, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x411, 0x43e, 0x43b, 0x438, 0x432, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x20,
0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x411, 0x440, 0x443, 0x43d, 0x435, 0x439, 0x2d, 0x414, 0x430, 0x440,
0x443, 0x441, 0x441, 0x430, 0x43b, 0x430, 0x43c, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x430, 0x431, 0x43e, 0x2d,
0x412, 0x435, 0x440, 0x434, 0x435, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a,
0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x20, 0x4ab, 0x443,
0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x41a, 0x435, 0x439, 0x441, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x427, 0x430, 0x43c, 0x43e, 0x440, 0x440, 0x43e, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x427, 0x430, 0x442, 0x435, 0x43c, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x427, 0x430, 0x442, 0x435, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x427, 0x430, 0x442, 0x435, 0x43c, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430,
0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x427, 0x438, 0x43b,
0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x427, 0x438, 0x43b, 0x438,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x427, 0x438, 0x43b, 0x438, 0x20, 0x4ab,
0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x41a, 0x438, 0x442, 0x430, 0x439, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x41a, 0x438, 0x442, 0x430, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x41a, 0x438, 0x442, 0x430, 0x439, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445,
0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x420, 0x430, 0x448, 0x442,
0x430, 0x432, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x43d, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x20, 0x443,
0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431,
0x438, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x443, 0x43a, 0x4d1, 0x43d, 0x20, 0x443, 0x442,
0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x41a, 0x443, 0x43a, 0x4d1, 0x43d, 0x20, 0x443, 0x442, 0x440, 0x430,
0x432, 0x4d7, 0x441, 0x435, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a,
0x443, 0x43a, 0x4d1, 0x43d, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441,
0x435, 0x43d, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x443, 0x431, 0x430, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x443, 0x431, 0x430, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x41a, 0x443, 0x431, 0x430, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430,
0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x414, 0x435, 0x439,
0x432, 0x438, 0x441, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x414, 0x44e,
0x43c, 0x43e, 0x43d, 0x2d, 0x434, 0x2019, 0x42e, 0x440, 0x432, 0x438, 0x43b, 0x44c,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x425, 0x4d7, 0x432, 0x435, 0x43b,
0x442, 0x443, 0x445, 0x4d1, 0x4ab, 0x20, 0x422, 0x438, 0x43c, 0x43e, 0x440, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x4d1, 0x43d, 0x43a, 0x443, 0x43d,
0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x43d, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x41c, 0x4d1, 0x43d, 0x43a, 0x443, 0x43d, 0x20, 0x443, 0x442,
0x440, 0x430, 0x432, 0x4d7, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c,
0x4d1, 0x43d, 0x43a, 0x443, 0x43d, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7,
0x43d, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x42d, 0x43a, 0x432, 0x430, 0x434, 0x43e, 0x440, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x4d7, 0x43f, 0x20, 0x415, 0x432,
0x440, 0x43e, 0x43f, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422,
0x4d7, 0x43f, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x422, 0x4d7, 0x43f, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f,
0x430, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x425, 0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443, 0x445,
0x4d1, 0x4ab, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x425, 0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443, 0x445,
0x4d1, 0x4ab, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x425, 0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1,
0x4ab, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x4ab, 0x443, 0x43b,
0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x418,
0x43d, 0x4ab, 0x435, 0x442, 0x2d, 0x445, 0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443,
0x445, 0x4d1, 0x4ab, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab, 0x20, 0x415, 0x432,
0x440, 0x43e, 0x43f, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410,
0x43d, 0x4d1, 0x4ab, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab, 0x20, 0x415, 0x432, 0x440,
0x43e, 0x43f, 0x430, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435,
0x43d, 0x434, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x424, 0x43e, 0x43b, 0x43a, 0x43b,
0x435, 0x43d, 0x434, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435,
0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x424, 0x43e, 0x43b, 0x43a, 0x43b,
0x435, 0x43d, 0x434, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435,
0x43d, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x424, 0x438, 0x434, 0x436, 0x438, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x424, 0x438, 0x434, 0x436, 0x438, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x424, 0x438, 0x434, 0x436, 0x438, 0x20, 0x4ab, 0x443, 0x43b,
0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x424,
0x440, 0x430, 0x43d, 0x446, 0x438, 0x20, 0x413, 0x432, 0x438, 0x430, 0x43d, 0x430,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x424, 0x440, 0x430, 0x43d, 0x446,
0x438, 0x20, 0x41a, 0x4d1, 0x43d, 0x442, 0x4d1, 0x440, 0x43f, 0x430, 0x20, 0x410,
0x43d, 0x442, 0x430, 0x440, 0x43a, 0x442, 0x438, 0x43a, 0x430, 0x20, 0x442, 0x435,
0x440, 0x440, 0x438, 0x442, 0x43e, 0x440, 0x438, 0x439, 0x4d7, 0x441, 0x435, 0x43d,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x430, 0x43b, 0x430, 0x43f,
0x430, 0x433, 0x43e, 0x441, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441,
0x435, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x430, 0x43c,
0x431, 0x44c, 0x435, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x440,
0x443, 0x437, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x440,
0x443, 0x437, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x440, 0x443,
0x437, 0x438, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x438, 0x43b, 0x431, 0x435, 0x440, 0x442,
0x4d1, 0x43d, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x440, 0x438, 0x43d, 0x432,
0x438, 0x447, 0x43f, 0x430, 0x20, 0x432, 0x4d1, 0x442, 0x430, 0x43c, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d,
0x434, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x440, 0x435,
0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x20, 0x4ab, 0x443,
0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x425, 0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab, 0x20, 0x413,
0x440, 0x438, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x425, 0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1,
0x4ab, 0x20, 0x413, 0x440, 0x438, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x425, 0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443,
0x445, 0x4d1, 0x4ab, 0x20, 0x413, 0x440, 0x438, 0x43d, 0x43b, 0x430, 0x43d, 0x434,
0x438, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab, 0x20, 0x413, 0x440, 0x438,
0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab, 0x20, 0x413, 0x440, 0x438, 0x43d, 0x43b, 0x430,
0x43d, 0x434, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43d, 0x4d1,
0x4ab, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x443, 0x430, 0x43c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x41f, 0x435, 0x440, 0x441, 0x438, 0x20, 0x437, 0x430, 0x43b, 0x438,
0x432, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x430, 0x439, 0x430,
0x43d, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x430, 0x432,
0x430, 0x439, 0x438, 0x20, 0x410, 0x43b, 0x435, 0x443, 0x442, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x430, 0x432, 0x430, 0x439, 0x438, 0x20, 0x410,
0x43b, 0x435, 0x443, 0x442, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x430,
0x432, 0x430, 0x439, 0x438, 0x20, 0x410, 0x43b, 0x435, 0x443, 0x442, 0x20, 0x4ab,
0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x20,
0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x425, 0x43e, 0x432, 0x434, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x425, 0x43e, 0x432, 0x434, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x425,
0x43e, 0x432, 0x434, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x418, 0x43d, 0x434, 0x438, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x418, 0x43d, 0x434, 0x438, 0x20, 0x43e, 0x43a,
0x435, 0x430, 0x43d, 0x4d7, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x418,
0x43d, 0x434, 0x43e, 0x43a, 0x438, 0x442, 0x430, 0x439, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x422, 0x4d7, 0x43f, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d,
0x435, 0x437, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x425, 0x4d7,
0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab, 0x20, 0x418, 0x43d, 0x434,
0x43e, 0x43d, 0x435, 0x437, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x410, 0x43d, 0x4d1, 0x4ab, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437,
0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x418, 0x440, 0x430, 0x43d,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x418, 0x440, 0x430, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x418, 0x440, 0x430, 0x43d, 0x20, 0x4ab, 0x443,
0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x4ab,
0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x418, 0x437, 0x440, 0x430, 0x438, 0x43b, 0x44c, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x418, 0x437, 0x440, 0x430, 0x438, 0x43b, 0x44c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x418, 0x437, 0x440, 0x430, 0x438, 0x43b, 0x44c, 0x20,
0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x42f, 0x43f, 0x43e, 0x43d, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x42f, 0x43f, 0x43e, 0x43d, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x42f, 0x43f, 0x43e, 0x43d, 0x438, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430,
0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41f, 0x435, 0x442,
0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x2d, 0x41a,
0x430, 0x43c, 0x447, 0x430, 0x442, 0x43a, 0x103, 0x440, 0x438, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x41f, 0x435, 0x442, 0x440, 0x43e, 0x43f, 0x430, 0x432,
0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x2d, 0x41a, 0x430, 0x43c, 0x447, 0x430, 0x442,
0x43a, 0x103, 0x440, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41f, 0x435,
0x442, 0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x2d,
0x41a, 0x430, 0x43c, 0x447, 0x430, 0x442, 0x43a, 0x103, 0x440, 0x438, 0x20, 0x4ab,
0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x425, 0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443,
0x445, 0x4d1, 0x4ab, 0x20, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430,
0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab,
0x20, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x43e, 0x440, 0x435, 0x439, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x43e, 0x440, 0x435, 0x439, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x43e, 0x440, 0x435, 0x439, 0x20, 0x4ab, 0x443,
0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x41a, 0x43e, 0x441, 0x440, 0x430, 0x435, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e,
0x44f, 0x440, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x440,
0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x20, 0x4ab, 0x443, 0x43b,
0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a,
0x438, 0x440, 0x433, 0x438, 0x437, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x41b, 0x430, 0x43d, 0x43a, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x41b, 0x430, 0x439, 0x43d, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7,
0x441, 0x435, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41b, 0x43e,
0x440, 0x434, 0x2d, 0x425, 0x430, 0x443, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x425, 0x430, 0x443, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x425, 0x430, 0x443, 0x20,
0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x41c, 0x430, 0x43a, 0x430, 0x43e, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x41c, 0x430, 0x43a, 0x430, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x41c, 0x430, 0x43a, 0x430, 0x43e, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430,
0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x430, 0x433,
0x430, 0x434, 0x430, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c,
0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x20, 0x4ab, 0x443, 0x43b, 0x43b,
0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x430,
0x43b, 0x430, 0x439, 0x437, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x41c, 0x430, 0x43b, 0x44c, 0x434, 0x438, 0x432, 0x441, 0x435, 0x43c, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x430, 0x440, 0x43a, 0x438, 0x437, 0x430,
0x441, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x430, 0x440, 0x448, 0x430, 0x43b,
0x43b, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a,
0x438, 0x439, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x430, 0x432,
0x440, 0x438, 0x43a, 0x438, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c,
0x430, 0x432, 0x440, 0x438, 0x43a, 0x438, 0x439, 0x20, 0x4ab, 0x443, 0x43b, 0x43b,
0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x43e,
0x443, 0x441, 0x43e, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c,
0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430, 0x20, 0x41b, 0x4d1, 0x43f, 0x43a, 0x4d1,
0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430, 0x20, 0x41b, 0x4d1, 0x43f,
0x43a, 0x4d1, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430, 0x20, 0x41b, 0x4d1,
0x43f, 0x43a, 0x4d1, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x20, 0x4ab, 0x443,
0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430,
0x442, 0x43e, 0x440, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x423, 0x43b, 0x430,
0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x20, 0x4ab, 0x443, 0x43b, 0x43b,
0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x443,
0x441, 0x43a, 0x430, 0x432, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c,
0x443, 0x441, 0x43a, 0x430, 0x432, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c,
0x443, 0x441, 0x43a, 0x430, 0x432, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445,
0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x44c, 0x44f, 0x43d,
0x43c, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41d, 0x430, 0x443,
0x440, 0x443, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41d, 0x435, 0x43f,
0x430, 0x43b, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x4aa, 0x4d7, 0x43d,
0x4d7, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x4aa, 0x4d7, 0x43d, 0x4d7, 0x20, 0x41a, 0x430,
0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x4aa, 0x4d7, 0x43d, 0x4d7, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d,
0x438, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x4aa, 0x4d7, 0x43d, 0x4d7, 0x20, 0x417, 0x435, 0x43b,
0x430, 0x43d, 0x434, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x4aa,
0x4d7, 0x43d, 0x4d7, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x4aa, 0x4d7, 0x43d, 0x4d7, 0x20, 0x417, 0x435,
0x43b, 0x430, 0x43d, 0x434, 0x438, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445,
0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41d, 0x44c, 0x44e, 0x444,
0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b,
0x435, 0x43d, 0x434, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41d, 0x44c, 0x44e,
0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x4ab, 0x443,
0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x41d, 0x438, 0x443, 0x44d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41d,
0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x4ab, 0x443,
0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x438, 0x2d,
0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434,
0x438, 0x2d, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d,
0x434, 0x438, 0x2d, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x20, 0x4ab,
0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x41c, 0x430, 0x440, 0x438, 0x430, 0x43d, 0x441, 0x435, 0x43d, 0x20, 0x4ab,
0x443, 0x440, 0x4ab, 0x4d7, 0x440, 0x442, 0x438, 0x20, 0x443, 0x442, 0x440, 0x430,
0x432, 0x4d7, 0x441, 0x435, 0x43c, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438,
0x431, 0x438, 0x440, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41d,
0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x20, 0x4ab,
0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x41e, 0x43c, 0x441, 0x43a, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x41e, 0x43c, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41e, 0x43c,
0x441, 0x43a, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430,
0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41f, 0x430, 0x43a, 0x438,
0x441, 0x442, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41f, 0x430,
0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430,
0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41f, 0x430, 0x43b,
0x430, 0x443, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41f, 0x430, 0x43f,
0x443, 0x430, 0x20, 0x2014, 0x20, 0x4aa, 0x4d7, 0x43d, 0x4d7, 0x20, 0x413, 0x432,
0x438, 0x43d, 0x435, 0x439, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41f,
0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439,
0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x41f, 0x435, 0x440, 0x443, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x41f, 0x435, 0x440, 0x443, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x41f, 0x435, 0x440, 0x443, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x424, 0x438, 0x43b, 0x438, 0x43f,
0x43f, 0x438, 0x43d, 0x441, 0x435, 0x43c, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x441, 0x435, 0x43c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f,
0x438, 0x43d, 0x441, 0x435, 0x43c, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445,
0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x424, 0x435, 0x43d, 0x438,
0x43a, 0x441, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x435, 0x43d,
0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20, 0x442, 0x430, 0x442, 0x430, 0x20, 0x41c,
0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20, 0x442, 0x430,
0x442, 0x430, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x44c, 0x435, 0x440,
0x20, 0x442, 0x430, 0x442, 0x430, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e,
0x43d, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x41f, 0x438, 0x442, 0x43a, 0x44d, 0x440, 0x43d, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41f, 0x43e, 0x43d, 0x43f, 0x435, 0x438,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x44b, 0x437, 0x44b, 0x43b,
0x43e, 0x440, 0x434, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a,
0x44b, 0x437, 0x44b, 0x43b, 0x43e, 0x440, 0x434, 0x430, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x41a, 0x44b, 0x437, 0x44b, 0x43b, 0x43e, 0x440, 0x434, 0x430, 0x20,
0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x420, 0x435, 0x44e, 0x43d, 0x44c, 0x43e, 0x43d, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x420, 0x43e, 0x442, 0x435, 0x440, 0x430, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x430, 0x445, 0x430, 0x43b,
0x438, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b,
0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x430, 0x445, 0x430,
0x43b, 0x438, 0x43d, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x20, 0x4ab,
0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x421, 0x430, 0x43c, 0x43e, 0x430, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445,
0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x435, 0x439, 0x448,
0x435, 0x43b, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x438, 0x43d, 0x433, 0x430,
0x43f, 0x443, 0x440, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x43e,
0x43b, 0x43e, 0x43c, 0x43e, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x41a, 0x4d1, 0x43d, 0x442, 0x4d1, 0x440, 0x20, 0x413, 0x435, 0x43e, 0x440, 0x433,
0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x443, 0x440, 0x438,
0x43d, 0x430, 0x43c, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x451,
0x432, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x430, 0x438,
0x442, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x430, 0x439,
0x43f, 0x435, 0x439, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x430,
0x439, 0x43f, 0x435, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x430,
0x439, 0x43f, 0x435, 0x439, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x430, 0x434, 0x436, 0x438,
0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x422, 0x43e, 0x43a, 0x435, 0x43b, 0x430, 0x443, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x4ab, 0x443, 0x43b, 0x43b,
0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x440,
0x443, 0x43a, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x443, 0x440,
0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438,
0x441, 0x442, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x443,
0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x4ab,
0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x422, 0x443, 0x432, 0x430, 0x43b, 0x443, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439,
0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430,
0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x423, 0x437, 0x431, 0x435,
0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x4ab,
0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20,
0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x412, 0x435, 0x43d, 0x435, 0x441, 0x443, 0x44d, 0x43b, 0x430, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432,
0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e,
0x441, 0x442, 0x43e, 0x43a, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x412, 0x43e, 0x43b, 0x433, 0x43e,
0x433, 0x440, 0x430, 0x434, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x412,
0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x20,
0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x423, 0x44d, 0x439, 0x43a, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x423, 0x43e, 0x43b, 0x43b, 0x438, 0x441, 0x20, 0x442, 0x430, 0x442,
0x430, 0x20, 0x424, 0x443, 0x442, 0x443, 0x43d, 0x430, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x4ab,
0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440,
0x433, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x415, 0x43a, 0x430, 0x442,
0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443,
0x440, 0x433, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x42e, 0x43a, 0x43e, 0x43d, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c,
0x2d, 0x41, 0x66, 0x66, 0x72, 0x65, 0x6b, 0x61, 0x61, 0x6e, 0x65, 0x73,
0x63, 0x68, 0x65, 0x20, 0x5a, 0x69, 0x67, 0x67, 0x4f, 0xdf, 0x2d, 0x41,
0x66, 0x66, 0x72, 0x65, 0x6b, 0x61, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68,
0x65, 0x20, 0x5a, 0x69, 0x67, 0x67, 0x53, 0xf6, 0x64, 0x2d, 0x41, 0x66,
0x66, 0x72, 0x65, 0x6b, 0x61, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x65,
0x20, 0x5a, 0x69, 0x67, 0x67, 0x57, 0xe4, 0xdf, 0x2d, 0x41, 0x66, 0x66,
0x72, 0x65, 0x6b, 0x61, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x65, 0x20,
0x5a, 0x69, 0x67, 0x67, 0x4a, 0x65, 0x77, 0xf6, 0x68, 0x6e, 0x6c, 0x69,
0x6a, 0x6a, 0x65, 0x20, 0x57, 0xe4, 0xdf, 0x2d, 0x41, 0x66, 0x66, 0x72,
0x65, 0x6b, 0x61, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a,
0x69, 0x67, 0x67, 0x57, 0xe4, 0xdf, 0x2d, 0x41, 0x66, 0x66, 0x72, 0x65,
0x6b, 0x61, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x67, 0x67, 0x64, 0x65, 0x20, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x20, 0x69, 0x65, 0x72, 0x20, 0x5a, 0x69, 0x63,
0x6b, 0x64, 0x65, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x20, 0x69, 0x65,
0x72, 0x20, 0x6a, 0x65, 0x77, 0xf6, 0x68, 0x6e, 0x6c, 0x69, 0x6a, 0x6a,
0x65, 0x20, 0x5a, 0x69, 0x63, 0x6b, 0x64, 0x65, 0x20, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x20, 0x69, 0x65, 0x72, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0x69, 0x63, 0x6b, 0x4b, 0x61, 0x70, 0x76, 0xe4, 0x72, 0x64,
0x65, 0x6a, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x69,
0x67, 0x67, 0x4a, 0x65, 0x77, 0xf6, 0x68, 0x6e, 0x6c, 0x69, 0x6a, 0x6a,
0x65, 0x20, 0x4b, 0x61, 0x70, 0x76, 0xe4, 0x72, 0x64, 0x65, 0x6a, 0x61,
0x61, 0x6e, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x69, 0x67, 0x67, 0x4b,
0x61, 0x70, 0x76, 0xe4, 0x72, 0x64, 0x65, 0x6a, 0x61, 0x61, 0x6e, 0x73,
0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69,
0x67, 0x67, 0x4d, 0x65, 0x64, 0x64, 0x65, 0x6c, 0x2d, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x70, 0x61, 0x20, 0x73, 0x69, 0x6e, 0x67, 0x20, 0x5a, 0x69,
0x63, 0x6b, 0x4d, 0x65, 0x64, 0x64, 0x65, 0x6c, 0x2d, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x70, 0x61, 0x20, 0x73, 0x69, 0x6e, 0x67, 0x20, 0x6a, 0x65,
0x77, 0xf6, 0x68, 0x6e, 0x6c, 0x69, 0x6a, 0x6a, 0x65, 0x20, 0x5a, 0x69,
0x63, 0x6b, 0x4d, 0x65, 0x64, 0x64, 0x65, 0x6c, 0x2d, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x70, 0x61, 0x20, 0x73, 0x69, 0x6e, 0x67, 0x20, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x63, 0x6b, 0x4f, 0xdf, 0x2d, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x70, 0x61, 0x20, 0x73, 0x69, 0x6e, 0x67, 0x20,
0x5a, 0x69, 0x63, 0x6b, 0x4f, 0xdf, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x70, 0x61, 0x20, 0x73, 0x69, 0x6e, 0x67, 0x20, 0x6a, 0x65, 0x77, 0xf6,
0x68, 0x6e, 0x6c, 0x69, 0x6a, 0x6a, 0x65, 0x20, 0x5a, 0x69, 0x63, 0x6b,
0x4f, 0xdf, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x70, 0x61, 0x20, 0x73,
0x69, 0x6e, 0x67, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69,
0x63, 0x6b, 0x57, 0x65, 0xdf, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x70,
0x61, 0x20, 0x73, 0x69, 0x6e, 0x67, 0x20, 0x5a, 0x69, 0x63, 0x6b, 0x57,
0x65, 0xdf, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x70, 0x61, 0x20, 0x73,
0x69, 0x6e, 0x67, 0x20, 0x6a, 0x65, 0x77, 0xf6, 0x68, 0x6e, 0x6c, 0x69,
0x6a, 0x6a, 0x65, 0x20, 0x5a, 0x69, 0x63, 0x6b, 0x57, 0x65, 0xdf, 0x2d,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x70, 0x61, 0x20, 0x73, 0x69, 0x6e, 0x67,
0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x63, 0x6b, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x20, 0x73, 0x69, 0x6e,
0x67, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x74, 0x2d, 0x5a, 0x69,
0x63, 0x6b, 0x64, 0x65, 0x6d, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x4f, 0x7a, 0x65, 0x6a, 0x61, 0x6e, 0x20, 0x73, 0x69,
0x6e, 0x67, 0x20, 0x5a, 0x69, 0x63, 0x6b, 0x5a, 0x69, 0x67, 0x67, 0x20,
0x76, 0x75, 0x6e, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0x4a, 0x65, 0x77, 0xf6, 0x68, 0x6e, 0x6c, 0x69, 0x6a, 0x6a, 0x65,
0x20, 0x5a, 0x69, 0x67, 0x67, 0x20, 0x76, 0x75, 0x6e, 0x20, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0x69, 0x67, 0x67, 0x20, 0x76, 0x75, 0x6e, 0x20, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x5a, 0x69, 0x67, 0x67, 0x20,
0x76, 0x75, 0x6e, 0x20, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x5a,
0x69, 0x67, 0x67, 0x20, 0x76, 0x75, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x53,
0x65, 0x69, 0x73, 0x63, 0x68, 0xe4, 0x6c, 0x6c, 0x65, 0x6f, 0x72, 0x61,
0x20, 0x6d, 0x65, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x20, 0x64, 0x69, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x41, 0x63, 0x72,
0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x63, 0x72,
0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x63, 0x72, 0x65,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x61, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x73, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x10d,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69, 0x73,
0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6a, 0x75, 0x17e, 0x6e,
0x6f, 0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x61,
0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x61, 0x66, 0x72,
0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a,
0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x61, 0x6c, 0x6a, 0x61, 0x161, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x6c, 0x6a, 0x61, 0x161,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x6c, 0x6a,
0x61, 0x161, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x6c, 0x74, 0x6d, 0x61,
0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x61, 0x6c, 0x74, 0x6d, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x61, 0x6c, 0x74, 0x6d, 0x61, 0x6a, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x61, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x61, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x73, 0x72, 0x65, 0x64, 0x69, 0x161, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x72, 0x65, 0x64, 0x69, 0x161,
0x6e, 0x6a, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x72,
0x65, 0x64, 0x69, 0x161, 0x6e, 0x6a, 0x65, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x6e,
0x61, 0x64, 0x69, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x61, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x6e, 0x61, 0x64, 0x69,
0x72, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x20, 0x41, 0x70, 0x69, 0x6a, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x20, 0x41, 0x70, 0x69, 0x6a, 0x65, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x41,
0x70, 0x69, 0x6a, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20,
0x67, 0x72, 0x61, 0x64, 0x61, 0x20, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x67, 0x72, 0x61, 0x64, 0x61, 0x20,
0x41, 0x6b, 0x74, 0x61, 0x75, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x67, 0x72, 0x61, 0x64,
0x61, 0x20, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x20, 0x67, 0x72, 0x61, 0x64, 0x61, 0x20, 0x41, 0x6b, 0x74,
0x6f, 0x62, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x67, 0x72,
0x61, 0x64, 0x61, 0x20, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x65, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x67, 0x72, 0x61, 0x64, 0x61, 0x20, 0x41, 0x6b, 0x74, 0x6f, 0x62,
0x65, 0x61, 0x72, 0x61, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x61, 0x72, 0x61, 0x70, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x72, 0x61, 0x70, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x61,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e,
0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61, 0x70, 0x61, 0x64,
0x6e, 0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x61, 0x72, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x72, 0x6d, 0x65,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61,
0x72, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x72,
0x65, 0x64, 0x6e, 0x6a, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73,
0x72, 0x65, 0x64, 0x6e, 0x6a, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73,
0x72, 0x65, 0x64, 0x6e, 0x6a, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x72, 0x65, 0x64, 0x69, 0x161,
0x6e, 0x6a, 0x65, 0x20, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x72, 0x65, 0x64, 0x69, 0x161,
0x6e, 0x6a, 0x65, 0x20, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x72, 0x65, 0x64, 0x69, 0x161, 0x6e,
0x6a, 0x65, 0x20, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61,
0x70, 0x61, 0x64, 0x6e, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a,
0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a,
0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x7a, 0x65, 0x72, 0x62,
0x61, 0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a,
0x64, 0x17e, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x61, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x17e, 0x61,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x7a, 0x6f, 0x72, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x7a,
0x6f, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x61, 0x7a, 0x6f, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x62, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x161, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x62,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x62, 0x75, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x62, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x62,
0x72, 0x61, 0x7a, 0x69, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x62, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x73, 0x6b,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x62, 0x72, 0x61, 0x7a,
0x69, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x20, 0x7a, 0x61, 0x20, 0x42, 0x72, 0x75, 0x6e, 0x65,
0x6a, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x5a, 0x65, 0x6c, 0x65,
0x6e, 0x6f, 0x72, 0x74, 0x73, 0x6b, 0x6f, 0x67, 0x20, 0x6f, 0x74, 0x6f,
0x10d, 0x6a, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x5a, 0x65,
0x6c, 0x65, 0x6e, 0x6f, 0x72, 0x74, 0x73, 0x6b, 0x6f, 0x67, 0x20, 0x6f,
0x74, 0x6f, 0x10d, 0x6a, 0x61, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x5a, 0x65, 0x6c, 0x65,
0x6e, 0x6f, 0x72, 0x74, 0x73, 0x6b, 0x6f, 0x67, 0x20, 0x6f, 0x74, 0x6f,
0x10d, 0x6a, 0x61, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x43,
0x61, 0x73, 0x65, 0x79, 0x6a, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x61, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x61, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x61, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x61, 0x10d, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10d, 0x69, 0x6c, 0x65,
0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x10d, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x6b, 0x69, 0x6e, 0x65, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x6b, 0x69, 0x6e, 0x65, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x42, 0x6f, 0x17e, 0x69,
0x107, 0x6e, 0x6f, 0x67, 0x20, 0x4f, 0x74, 0x6f, 0x6b, 0x61, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x6f,
0x76, 0x69, 0x68, 0x20, 0x4f, 0x74, 0x6f, 0x6b, 0x61, 0x6b, 0x6f, 0x6c,
0x75, 0x6d, 0x62, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x6b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6b, 0x6f,
0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x43, 0x6f, 0x6f, 0x6b,
0x6f, 0x76, 0x69, 0x68, 0x20, 0x4f, 0x74, 0x6f, 0x6b, 0x61, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x76, 0x69,
0x68, 0x20, 0x4f, 0x74, 0x6f, 0x6b, 0x61, 0x43, 0x6f, 0x6f, 0x6b, 0x6f,
0x76, 0x69, 0x20, 0x4f, 0x74, 0x6f, 0x63, 0x69, 0x2c, 0x20, 0x70, 0x6f,
0x6c, 0x75, 0x73, 0x61, 0x74, 0x6e, 0x69, 0x20, 0x70, 0x6f, 0x6d, 0x61,
0x6b, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6b, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6b, 0x75, 0x62,
0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x6b, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73,
0x61, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x44, 0x75, 0x6d,
0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c,
0x65, 0x61, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x74, 0x69, 0x6d,
0x6f, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x55, 0x73, 0x6b,
0x72, 0x161, 0x6e, 0x6a, 0x65, 0x67, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x61,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x55, 0x73, 0x6b, 0x72, 0x161,
0x6e, 0x6a, 0x65, 0x67, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x61, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x55, 0x73, 0x6b, 0x72, 0x161, 0x6e, 0x6a, 0x65, 0x67, 0x20, 0x6f,
0x74, 0x6f, 0x6b, 0x61, 0x65, 0x6b, 0x76, 0x61, 0x64, 0x6f, 0x72, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x72,
0x65, 0x64, 0x6e, 0x6a, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x72, 0x65,
0x64, 0x6e, 0x6a, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x72, 0x65, 0x64, 0x6e,
0x6a, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x65, 0x75, 0x72, 0x6f,
0x70, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69, 0x73,
0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x64, 0x61, 0x6c, 0x65, 0x6b, 0x6f, 0x69, 0x73,
0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x20, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61,
0x70, 0x61, 0x64, 0x6e, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61, 0x70,
0x61, 0x64, 0x6e, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61, 0x70, 0x61, 0x64,
0x6e, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x66, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x66, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x66, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20,
0x46, 0x69, 0x64, 0x17e, 0x69, 0x6a, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x20, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x6a, 0x61, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20,
0x46, 0x69, 0x64, 0x17e, 0x69, 0x6a, 0x61, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x75, 0x73, 0x6b, 0x65,
0x20, 0x47, 0x69, 0x6a, 0x61, 0x6e, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x75, 0x73, 0x6b, 0x69,
0x68, 0x20, 0x4a, 0x75, 0x17e, 0x6e, 0x69, 0x68, 0x20, 0x54, 0x65, 0x72,
0x69, 0x74, 0x6f, 0x72, 0x69, 0x6a, 0x61, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73,
0x61, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x47, 0x61, 0x6d,
0x62, 0x69, 0x65, 0x72, 0x61, 0x67, 0x72, 0x75, 0x7a, 0x69, 0x6a, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x67, 0x72,
0x75, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x67, 0x72, 0x75, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x47, 0x69,
0x6c, 0x62, 0x65, 0x72, 0x74, 0x6f, 0x76, 0x69, 0x68, 0x20, 0x6f, 0x74,
0x6f, 0x6b, 0x61, 0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x7a, 0x61, 0x6c,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69, 0x73,
0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61,
0x70, 0x61, 0x64, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x67, 0x75,
0x61, 0x6d, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x7a, 0x61, 0x6c, 0x6a, 0x65, 0x76, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x68, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f,
0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x68, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b,
0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x68, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b,
0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x68, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x161, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x68, 0x6f, 0x6e, 0x67, 0x6b,
0x6f, 0x6e, 0x161, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x68, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x161, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x68, 0x6f, 0x76, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x68, 0x6f, 0x76, 0x64, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x68, 0x6f, 0x76, 0x64, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x69, 0x6e, 0x64, 0x69, 0x6a, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x6a, 0x73, 0x6b, 0x6f,
0x67, 0x20, 0x6f, 0x63, 0x65, 0x61, 0x6e, 0x61, 0x69, 0x6e, 0x64, 0x6f,
0x6b, 0x69, 0x6e, 0x65, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x73, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x6f, 0x69, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e,
0x6f, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61, 0x70,
0x61, 0x64, 0x6e, 0x6f, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69,
0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x69, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x69, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x69, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69, 0x7a, 0x72,
0x61, 0x65, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x69, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69, 0x7a, 0x72, 0x61, 0x65, 0x6c,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6a, 0x61, 0x70, 0x61, 0x6e, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6a, 0x61,
0x70, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x6a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73,
0x6b, 0x2d, 0x6b, 0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x6b, 0x61, 0x6d,
0x10d, 0x61, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76,
0x73, 0x6b, 0x2d, 0x6b, 0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0x6f,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x6b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6b, 0x6f, 0x72, 0x65, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6b, 0x6f,
0x72, 0x65, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x6b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x6b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6c, 0x61, 0x6e, 0x6b, 0x61, 0x6e, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x45, 0x6b, 0x76, 0x61, 0x74, 0x6f,
0x72, 0x73, 0x6b, 0x69, 0x68, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x61, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x61,
0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x61, 0x20, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6f,
0x74, 0x6f, 0x6b, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x6d, 0x61, 0x6b, 0x61, 0x6f, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x6d, 0x61, 0x6b, 0x61, 0x6f, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x6d, 0x61, 0x6b, 0x61, 0x6f, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x6d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6d, 0x61, 0x6c, 0x65,
0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x6d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6d, 0x61, 0x72, 0x6b, 0x69,
0x17e, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x4d, 0x61,
0x72, 0x161, 0x61, 0x6c, 0x6f, 0x76, 0x69, 0x68, 0x20, 0x4f, 0x74, 0x6f,
0x6b, 0x61, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x63, 0x69, 0x6a, 0x75, 0x73, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6a,
0x75, 0x73, 0x61, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63,
0x69, 0x6a, 0x75, 0x73, 0x61, 0x6d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6d, 0x65,
0x6b, 0x73, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66,
0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x6d, 0x65, 0x6b, 0x73, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x61, 0x63,
0x69, 0x66, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x6d, 0x65, 0x6b, 0x73, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x61,
0x63, 0x69, 0x66, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x75, 0x6c,
0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x75, 0x6c, 0x61, 0x6e, 0x62, 0x61,
0x74, 0x6f, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x75, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x6d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6d, 0x6f, 0x73,
0x6b, 0x6f, 0x76, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x6d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x6d, 0x6a, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x61, 0x6e, 0x65, 0x70,
0x61, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x4e, 0x6f, 0x76,
0x65, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x65, 0x20,
0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x65, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x4e, 0x6f, 0x76, 0x65, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x69, 0x6a, 0x65, 0x6e, 0x6f, 0x76, 0x6f, 0x7a, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x6e, 0x6f, 0x76, 0x6f, 0x7a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6e, 0x6f, 0x76,
0x6f, 0x7a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x6e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x6e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6e, 0x65,
0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20,
0x4e, 0x69, 0x75, 0x65, 0x61, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x4f, 0x74, 0x6f, 0x6b, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x4f, 0x74, 0x6f,
0x6b, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x4f, 0x74, 0x6f, 0x6b, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x67, 0x72,
0x61, 0x64, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x67, 0x72, 0x61, 0x64, 0x61, 0x20,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x67, 0x72,
0x61, 0x64, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x53, 0x6a, 0x65, 0x76, 0x65,
0x72, 0x6e, 0x6f, 0x6d, 0x61, 0x72, 0x69, 0x6a, 0x61, 0x6e, 0x73, 0x6b,
0x69, 0x68, 0x20, 0x4f, 0x74, 0x6f, 0x6b, 0x61, 0x6e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x6e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x6f, 0x6d, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x6f, 0x6d, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x6f, 0x6d, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x70,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x70, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x70, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x50, 0x61,
0x6c, 0x61, 0x75, 0x61, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20,
0x50, 0x61, 0x70, 0x75, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x65, 0x20, 0x47,
0x76, 0x69, 0x6e, 0x65, 0x6a, 0x65, 0x70, 0x61, 0x72, 0x61, 0x67, 0x76,
0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x70, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x70, 0x61, 0x72, 0x61, 0x67,
0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x70, 0x65, 0x72,
0x75, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x70, 0x65, 0x72, 0x75, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x70, 0x65, 0x72, 0x75, 0x61, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x66, 0x69, 0x6c, 0x69, 0x70, 0x69,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x66, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x66, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x20, 0x4f, 0x74, 0x6f, 0x6b, 0x61, 0x20, 0x50, 0x68, 0x6f, 0x65,
0x6e, 0x69, 0x78, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x7a,
0x61, 0x20, 0x53, 0x76, 0x65, 0x74, 0x69, 0x20, 0x50, 0x65, 0x74, 0x61,
0x72, 0x20, 0x69, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x76, 0x65,
0x74, 0x69, 0x20, 0x50, 0x65, 0x74, 0x61, 0x72, 0x20, 0x69, 0x20, 0x4d,
0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x7a, 0x61, 0x20,
0x53, 0x76, 0x65, 0x74, 0x69, 0x20, 0x50, 0x65, 0x74, 0x61, 0x72, 0x20,
0x69, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x70, 0x69, 0x74,
0x63, 0x61, 0x69, 0x72, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x70, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x6a, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x70, 0x6a,
0x6f, 0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x161, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x67, 0x72, 0x61, 0x64, 0x61, 0x20, 0x4b, 0x69, 0x7a, 0x69, 0x6c,
0x6f, 0x72, 0x64, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x67,
0x72, 0x61, 0x64, 0x61, 0x20, 0x4b, 0x69, 0x7a, 0x69, 0x6c, 0x6f, 0x72,
0x64, 0x65, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x20, 0x67, 0x72, 0x61, 0x64, 0x61, 0x20, 0x4b,
0x69, 0x7a, 0x69, 0x6c, 0x6f, 0x72, 0x64, 0x65, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x20, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x61,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x52, 0x6f, 0x74, 0x68,
0x65, 0x72, 0x65, 0x73, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x61, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x73, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x73, 0x61, 0x6d, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x61, 0x6d, 0x61, 0x72,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x61,
0x6d, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x61, 0x6d,
0x6f, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x73, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x61, 0x6d, 0x6f, 0x61, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x65, 0x6a, 0x161, 0x65, 0x6c,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73,
0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x20, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x6f, 0x76, 0x69,
0x68, 0x20, 0x4f, 0x74, 0x6f, 0x6b, 0x61, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x20, 0x4a, 0x75, 0x17e, 0x6e, 0x65, 0x20, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x6a, 0x65, 0x73, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x65,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x54, 0x61, 0x68, 0x69,
0x74, 0x69, 0x6a, 0x61, 0x74, 0x61, 0x6a, 0x70, 0x65, 0x161, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x74, 0x61, 0x6a, 0x70,
0x65, 0x161, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x74,
0x61, 0x6a, 0x70, 0x65, 0x161, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61,
0x75, 0x61, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x54, 0x6f,
0x6e, 0x67, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x54, 0x6f,
0x6e, 0x67, 0x65, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x65, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b,
0x61, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x54, 0x75, 0x76,
0x61, 0x6c, 0x75, 0x61, 0x75, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x75, 0x72,
0x75, 0x67, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x75, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x61, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x61, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x61, 0x76, 0x65,
0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x76, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x10d, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x10d, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x76, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x20, 0x4f, 0x74, 0x6f, 0x6b, 0x61, 0x20, 0x57, 0x61,
0x6b, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x4f, 0x74,
0x6f, 0x6b, 0x61, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x69,
0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x6a, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6a,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x6a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x6a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x161, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x6a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x161, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6a, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x161, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x6a, 0x75, 0x6b, 0x6f, 0x6e, 0x161, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x63, 0x72, 0x65,
0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x63, 0x72, 0x65,
0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x63, 0x72, 0x65, 0x6a,
0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x61, 0x66, 0x67, 0x68, 0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x73, 0x74, 0x159, 0x65, 0x64, 0x6f, 0x61, 0x66, 0x72, 0x69,
0x63, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f,
0x64, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x6a, 0x69, 0x68, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x63, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x66,
0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70,
0x61, 0x64, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x63,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x61, 0x6c, 0x6a, 0x61, 0x161, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x61, 0x6c, 0x6a, 0x61, 0x161, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61,
0x6c, 0x6a, 0x61, 0x161, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x73, 0x6b,
0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x63, 0x65, 0x6e,
0x74, 0x72, 0xe1, 0x6c, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65,
0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b, 0xfd,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0xe1, 0x6c, 0x6e, 0xed, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x6b, 0xfd, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0xe1, 0x6c, 0x6e,
0xed, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x73,
0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b,
0xfd, 0x20, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x63, 0x6b, 0xfd, 0x20, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6e,
0xed, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x76, 0xfd, 0x63, 0x68, 0x6f,
0x64, 0x6e, 0xed, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x6b, 0xfd, 0x20, 0x68, 0x6f, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x63, 0x6b, 0xfd, 0x20, 0x68, 0x6f, 0x72, 0x73, 0x6b, 0xfd, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x63, 0x6b, 0xfd, 0x20, 0x68, 0x6f, 0x72, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x76,
0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20,
0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x6b, 0xfd, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x6b,
0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66,
0x69, 0x63, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x61, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x61, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x73, 0x6b, 0xfd,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x61, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x70,
0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x70, 0x69,
0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x70, 0x69, 0x6a, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x41, 0x6b, 0x74, 0x61, 0x75, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x41, 0x6b, 0x74, 0x61, 0x75, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x41,
0x6b, 0x74, 0x61, 0x75, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x73, 0x6b,
0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72,
0x61, 0x62, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x61,
0x62, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x61, 0x62, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x7a,
0xe1, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64,
0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72,
0x6d, 0xe9, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72,
0x6d, 0xe9, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x6d,
0xe9, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x63, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x63, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x73, 0x74, 0x159, 0x65, 0x64, 0x6f, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x74,
0x159, 0x65, 0x64, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x74, 0x159, 0x65, 0x64, 0x6f, 0x61,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x74, 0x159, 0x65, 0x64,
0x6f, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6e, 0xed, 0x20, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73,
0x74, 0x159, 0x65, 0x64, 0x6f, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6e, 0xed,
0x20, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0xfd, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x73, 0x74, 0x159, 0x65, 0x64, 0x6f, 0x7a, 0xe1, 0x70, 0x61,
0x64, 0x6e, 0xed, 0x20, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63,
0x68, 0x6f, 0x64, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f,
0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61,
0x64, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x7a,
0xe1, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0xe1, 0x7a, 0x65, 0x72, 0x62, 0xe1, 0x6a, 0x64, 0x17e, 0xe1, 0x6e,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0xe1, 0x7a, 0x65, 0x72, 0x62,
0xe1, 0x6a, 0x64, 0x17e, 0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0xe1, 0x7a, 0x65, 0x72, 0x62, 0xe1, 0x6a, 0x64, 0x17e, 0xe1, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x61, 0x7a, 0x6f, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61,
0x7a, 0x6f, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x7a, 0x6f,
0x72, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0xe9, 0x161, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0xe9, 0x161, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0xe9, 0x161, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x68, 0xfa, 0x74, 0xe1,
0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x6f, 0x6c, 0x69,
0x76, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x72,
0x61, 0x73, 0x69, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x62, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0xfd,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x62, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x6a, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x62, 0x72, 0x75, 0x6e, 0x65, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x6b, 0x61, 0x70, 0x76, 0x65, 0x72, 0x64, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x6b, 0x61, 0x70, 0x76, 0x65, 0x72, 0x64, 0x73, 0x6b,
0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x6b, 0x61, 0x70, 0x76, 0x65, 0x72, 0x64, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x10d, 0x61, 0x73, 0x20, 0x43, 0x61, 0x73, 0x65, 0x79, 0x68, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x63, 0x65, 0x63, 0x68, 0x61, 0x6d, 0x6f,
0x72, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x63,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x63, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x69, 0x6c,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x69, 0x6c, 0x73,
0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x69, 0x6c, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0xed,
0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0xed, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0xed, 0x6e, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73,
0x20, 0x56, 0xe1, 0x6e, 0x6f, 0x10d, 0x6e, 0xed, 0x68, 0x6f, 0x20, 0x6f,
0x73, 0x74, 0x72, 0x6f, 0x76, 0x61, 0x10d, 0x61, 0x73, 0x20, 0x4b, 0x6f,
0x6b, 0x6f, 0x73, 0x6f, 0x76, 0xfd, 0x63, 0x68, 0x20, 0x6f, 0x73, 0x74,
0x72, 0x6f, 0x76, 0x16f, 0x6b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x6f, 0x6c, 0x75, 0x6d,
0x62, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x6f, 0x6c,
0x75, 0x6d, 0x62, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x43, 0x6f,
0x6f, 0x6b, 0x6f, 0x76, 0xfd, 0x63, 0x68, 0x20, 0x6f, 0x73, 0x74, 0x72,
0x6f, 0x76, 0x16f, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x76,
0xfd, 0x63, 0x68, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x16f, 0x6c,
0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f,
0x6b, 0x6f, 0x76, 0xfd, 0x63, 0x68, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f,
0x76, 0x16f, 0x6b, 0x75, 0x62, 0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x6b, 0x75, 0x62, 0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x6b, 0x75, 0x62, 0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x44,
0x61, 0x76, 0x69, 0x73, 0x6f, 0x76, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x63, 0x65, 0x10d, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x63, 0x65, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x20, 0x64,
0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x61, 0x76, 0xfd, 0x63, 0x68,
0x6f, 0x64, 0x6f, 0x74, 0x69, 0x6d, 0x6f, 0x72, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x56, 0x65, 0x6c, 0x69, 0x6b,
0x6f, 0x6e, 0x6f, 0x10d, 0x6e, 0xed, 0x68, 0x6f, 0x20, 0x6f, 0x73, 0x74,
0x72, 0x6f, 0x76, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x56, 0x65, 0x6c, 0x69, 0x6b,
0x6f, 0x6e, 0x6f, 0x10d, 0x6e, 0xed, 0x68, 0x6f, 0x20, 0x6f, 0x73, 0x74,
0x72, 0x6f, 0x76, 0x61, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x20, 0x56, 0x65, 0x6c, 0x69, 0x6b, 0x6f, 0x6e, 0x6f, 0x10d, 0x6e,
0xed, 0x68, 0x6f, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x61, 0x65,
0x6b, 0x76, 0xe1, 0x64, 0x6f, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x73, 0x74, 0x159, 0x65, 0x64, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x74, 0x159, 0x65, 0x64,
0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x73, 0x74, 0x159, 0x65, 0x64, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f,
0x64, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0xfd, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x65, 0x76, 0x72, 0x6f,
0x70, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x64, 0xe1, 0x6c, 0x6e, 0x11b, 0x76, 0xfd, 0x63, 0x68, 0x6f,
0x64, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x65, 0x76, 0x72, 0x6f,
0x70, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61,
0x64, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0xfd, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70,
0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x66, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x66, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x66, 0x69, 0x64, 0x17e, 0x69, 0x6a, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x66, 0x69, 0x64, 0x17e, 0x69, 0x6a, 0x73, 0x6b,
0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x66, 0x69, 0x64, 0x17e, 0x69, 0x6a, 0x73, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x66,
0x72, 0x61, 0x6e, 0x63, 0x6f, 0x75, 0x7a, 0x73, 0x6b, 0x6f, 0x67, 0x75,
0x79, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61,
0x73, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x6f, 0x75, 0x7a, 0x73, 0x6b,
0xfd, 0x63, 0x68, 0x20, 0x6a, 0x69, 0x17e, 0x6e, 0xed, 0x63, 0x68, 0x20,
0x61, 0x20, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x63, 0x6b,
0xfd, 0x63, 0x68, 0x20, 0xfa, 0x7a, 0x65, 0x6d, 0xed, 0x67, 0x61, 0x6c,
0x61, 0x70, 0xe1, 0x17e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x67,
0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x67, 0x72, 0x75, 0x7a, 0xed, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x67, 0x72, 0x75, 0x7a, 0xed, 0x6e, 0x73, 0x6b, 0xfd, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x67, 0x72, 0x75, 0x7a, 0xed, 0x6e, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73,
0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x6f, 0x76, 0xfd, 0x63,
0x68, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x16f, 0x67, 0x72, 0x65,
0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74,
0x159, 0x65, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63,
0x68, 0x6f, 0x64, 0x6f, 0x67, 0x72, 0xf3, 0x6e, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x67, 0x72,
0xf3, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68,
0x6f, 0x64, 0x6f, 0x67, 0x72, 0xf3, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61,
0x64, 0x6f, 0x67, 0x72, 0xf3, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x67, 0x72, 0xf3, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x67,
0x72, 0xf3, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x47, 0x75, 0x61, 0x6d, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x50, 0x65, 0x72, 0x73, 0x6b, 0xe9,
0x68, 0x6f, 0x20, 0x7a, 0xe1, 0x6c, 0x69, 0x76, 0x75, 0x68, 0x61, 0x76,
0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x68, 0x61, 0x76, 0x61, 0x6a, 0x73,
0x6b, 0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0xfd, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x68, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x2d, 0x61,
0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x68, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e,
0x67, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x68, 0x6f, 0x6e, 0x67,
0x6b, 0x6f, 0x6e, 0x67, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x68, 0x6f,
0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x68, 0x6f, 0x76, 0x64, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x68, 0x6f, 0x76, 0x64, 0x73, 0x6b,
0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x68, 0x6f, 0x76, 0x64, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x6e, 0x64,
0x69, 0x63, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x6e, 0x64, 0x69,
0x63, 0x6b, 0x6f, 0x6f, 0x63, 0x65, 0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x69, 0x6e, 0x64, 0x6f, 0x10d, 0xed, 0x6e, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x74, 0x159, 0x65, 0x64, 0x6f, 0x69,
0x6e, 0x64, 0x6f, 0x6e, 0xe9, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x69, 0x6e, 0x64, 0x6f, 0x6e,
0xe9, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61,
0x64, 0x6f, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0xe9, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0xed, 0x72, 0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0xed, 0x72, 0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0xed, 0x72, 0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x69, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x7a,
0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x69,
0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x69,
0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6a, 0x61, 0x70, 0x6f, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6a, 0x61, 0x70, 0x6f, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x6a, 0x61, 0x70, 0x6f, 0x6e, 0x73, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x70,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x6f, 0x2d, 0x6b, 0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x70, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c,
0x6f, 0x76, 0x73, 0x6b, 0x6f, 0x2d, 0x6b, 0x61, 0x6d, 0x10d, 0x61, 0x74,
0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x65, 0x74, 0x72, 0x6f, 0x70,
0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x6f, 0x2d, 0x6b, 0x61, 0x6d,
0x10d, 0x61, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x6b, 0x61, 0x7a, 0x61, 0x63, 0x68, 0x73, 0x74,
0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63,
0x68, 0x6f, 0x64, 0x6f, 0x6b, 0x61, 0x7a, 0x61, 0x63, 0x68, 0x73, 0x74,
0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70,
0x61, 0x64, 0x6f, 0x6b, 0x61, 0x7a, 0x61, 0x63, 0x68, 0x73, 0x74, 0xe1,
0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x6f, 0x72, 0x65,
0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x6f, 0x72, 0x65,
0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x6f, 0x72, 0x65, 0x6a,
0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x6b, 0x6f, 0x73, 0x72, 0x61, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x6b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x6a, 0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x79, 0x72, 0x67, 0x79,
0x7a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x53, 0x72, 0xed, 0x6c,
0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73,
0x20, 0x52, 0x6f, 0x76, 0x6e, 0xed, 0x6b, 0x6f, 0x76, 0xfd, 0x63, 0x68,
0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x16f, 0x10d, 0x61, 0x73, 0x20,
0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x61, 0x20, 0x6c, 0x6f, 0x72, 0x64,
0x61, 0x20, 0x48, 0x6f, 0x77, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x6f, 0x73, 0x74,
0x72, 0x6f, 0x76, 0x61, 0x20, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x20, 0x48,
0x6f, 0x77, 0x61, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x61, 0x20, 0x6c, 0x6f, 0x72,
0x64, 0x61, 0x20, 0x48, 0x6f, 0x77, 0x61, 0x4d, 0x61, 0x63, 0x61, 0x6a,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x4d, 0x61, 0x63, 0x61, 0x6a,
0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x4d, 0x61, 0x63, 0x61, 0x6a, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x6d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x6d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x73, 0x6b, 0xfd,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x6d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x73, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6d,
0x61, 0x6c, 0x61, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d,
0x61, 0x6c, 0x65, 0x64, 0x69, 0x76, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x6d, 0x61, 0x72, 0x6b, 0xe9, 0x7a, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61,
0x6c, 0x6c, 0x6f, 0x76, 0xfd, 0x63, 0x68, 0x20, 0x6f, 0x73, 0x74, 0x72,
0x6f, 0x76, 0x16f, 0x6d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6a, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61, 0x75, 0x72, 0x69, 0x63,
0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61, 0x75, 0x72,
0x69, 0x63, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x77,
0x73, 0x6f, 0x6e, 0x6f, 0x76, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x63, 0x65, 0x6d, 0x65, 0x78, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x70, 0x61,
0x63, 0x69, 0x66, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d,
0x65, 0x78, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66,
0x69, 0x63, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x65, 0x78, 0x69, 0x63,
0x6b, 0xfd, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x75, 0x6c,
0xe1, 0x6e, 0x62, 0xe1, 0x74, 0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x75, 0x6c, 0xe1, 0x6e, 0x62, 0xe1, 0x74, 0x61, 0x72, 0x73,
0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x75, 0x6c, 0xe1, 0x6e, 0x62, 0xe1, 0x74,
0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x6d, 0x6f, 0x73, 0x6b, 0x65, 0x76, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x6d, 0x6f, 0x73, 0x6b, 0x65, 0x76, 0x73, 0x6b,
0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x6d, 0x6f, 0x73, 0x6b, 0x65, 0x76, 0x73, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6d,
0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x6e, 0x61, 0x75, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x6e, 0x65, 0x70, 0xe1, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x6e, 0x6f, 0x76, 0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x76, 0x6f, 0x6b, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6e,
0x6f, 0x76, 0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x73, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6e,
0x6f, 0x76, 0x6f, 0x7a, 0xe9, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x76, 0x6f, 0x7a, 0xe9, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x76, 0x6f,
0x7a, 0xe9, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x65, 0x77, 0x66, 0x6f,
0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x6e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x65, 0x77, 0x66,
0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x69, 0x75,
0x65, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x6e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20,
0x73, 0x6f, 0x75, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0xed, 0x20, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x73, 0x6f, 0x75,
0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0xed, 0x20, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x20, 0x73, 0x6f, 0x75, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0xed, 0x20,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x53, 0x65, 0x76, 0x65, 0x72,
0x6f, 0x6d, 0x61, 0x72, 0x69, 0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x6e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x76,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6f, 0x6d, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x6f, 0x6d, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x6f, 0x6d, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x70, 0xe1, 0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x70, 0xe1, 0x6b, 0x69, 0x73, 0x74,
0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x70, 0xe1, 0x6b, 0x69,
0x73, 0x74, 0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x50, 0x61, 0x70,
0x75, 0x79, 0x2d, 0x4e, 0x6f, 0x76, 0xe9, 0x20, 0x47, 0x75, 0x69, 0x6e,
0x65, 0x79, 0x70, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x79, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x70, 0x65, 0x72, 0x75, 0xe1, 0x6e, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x65, 0x72, 0x75, 0xe1, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x65, 0x72, 0x75, 0xe1, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x66, 0x69, 0x6c, 0x69, 0x70, 0xed, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x66, 0x69, 0x6c, 0x69, 0x70, 0xed, 0x6e, 0x73, 0x6b, 0xfd,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x66, 0x69, 0x6c, 0x69, 0x70, 0xed, 0x6e, 0x73, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x10d,
0x61, 0x73, 0x20, 0x46, 0xe9, 0x6e, 0x69, 0x78, 0x6f, 0x76, 0xfd, 0x63,
0x68, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x16f, 0x70, 0x69, 0x65,
0x72, 0x72, 0x65, 0x2d, 0x6d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x69, 0x65, 0x72, 0x72,
0x65, 0x2d, 0x6d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x73, 0x6b,
0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x70, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x6d,
0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20,
0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x6f, 0x76, 0xfd, 0x63,
0x68, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x16f, 0x70, 0x6f, 0x6e,
0x61, 0x70, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x63, 0x68,
0x6a, 0x6f, 0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x67, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x4b, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x79, 0x7a, 0x79, 0x6c, 0x6f,
0x72, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x79, 0x7a, 0x79,
0x6c, 0x6f, 0x72, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x72, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x52,
0x6f, 0x74, 0x68, 0x65, 0x72, 0x6f, 0x76, 0x79, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x63, 0x65, 0x73, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x63, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x63,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x6d, 0x61, 0x72, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x6d, 0x61, 0x72, 0x73,
0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x6d, 0x61, 0x72, 0x73, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x73,
0x61, 0x6d, 0x6f, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73,
0x61, 0x6d, 0x6f, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x61,
0x6d, 0x6f, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x69, 0x6e, 0x67, 0x61, 0x70,
0x75, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73,
0x20, 0x160, 0x61, 0x6c, 0x61, 0x6d, 0x6f, 0x75, 0x6e, 0x6f, 0x76, 0xfd,
0x63, 0x68, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x16f, 0x10d, 0x61,
0x73, 0x20, 0x4a, 0x69, 0x17e, 0x6e, 0xed, 0x20, 0x47, 0x65, 0x6f, 0x72,
0x67, 0x69, 0x65, 0x73, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x63, 0x65, 0x20, 0x160, 0xf3, 0x77, 0x61, 0x74, 0x61, 0x68,
0x69, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x63, 0x68,
0x61, 0x6a, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x74, 0x63, 0x68, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0xfd, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x74, 0x63, 0x68, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x73, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x74,
0xe1, 0x64, 0x17e, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x74,
0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x74, 0x6f, 0x6e, 0x17e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x74, 0x6f, 0x6e, 0x17e, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x6f,
0x6e, 0x17e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x63, 0x68, 0x75, 0x75, 0x6b, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x74, 0x75, 0x76, 0x61, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x75, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x75, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x73,
0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x75, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79,
0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x75, 0x7a, 0x62, 0x65, 0x63, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x75, 0x7a, 0x62, 0x65, 0x63, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x75, 0x7a,
0x62, 0x65, 0x63, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x73, 0x6b,
0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x73, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x76,
0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x76, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x63, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x76, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x63, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x76,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x63, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x76, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x76, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x73,
0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x76, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x63, 0x65, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x10d, 0x61, 0x73,
0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x61, 0x20, 0x57, 0x61, 0x6b,
0x65, 0x10d, 0x61, 0x73, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x16f,
0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x61, 0x20, 0x46, 0x75,
0x74, 0x75, 0x6e, 0x61, 0x6a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x6a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xfd,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x6a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6a, 0x65, 0x6b,
0x61, 0x74, 0x11b, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x6a, 0x65, 0x6b, 0x61, 0x74, 0x11b, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6a, 0x65,
0x6b, 0x61, 0x74, 0x11b, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x73, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x79,
0x75, 0x6b, 0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x41,
0x63, 0x72, 0x65, 0x2d, 0x74, 0x69, 0x64, 0x41, 0x63, 0x72, 0x65, 0x2d,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x63, 0x72,
0x65, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x41,
0x66, 0x67, 0x68, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0xd8, 0x73, 0x74, 0x61, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x53,
0x79, 0x64, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74,
0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x61,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x2d, 0x74, 0x69, 0x64, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x2d, 0x74, 0x69, 0x64, 0x41, 0x6c,
0x6d, 0x61, 0x74, 0x79, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x2d, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x61, 0x73, 0x2d, 0x74, 0x69, 0x64, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x61, 0x73, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x2d, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x2d, 0x74, 0x69, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x2d, 0x74,
0x69, 0x64, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x2d, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x45, 0x61, 0x73, 0x74, 0x65,
0x72, 0x6e, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x2d, 0x74, 0x69, 0x64,
0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x2d, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61,
0x69, 0x6e, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x2d, 0x74, 0x69, 0x64, 0x50,
0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x2d,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x41, 0x6e, 0x61,
0x64, 0x79, 0x72, 0x2d, 0x74, 0x69, 0x64, 0x41, 0x6e, 0x61, 0x64, 0x79,
0x72, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x41, 0x70, 0x69, 0x61, 0x2d, 0x74, 0x69, 0x64, 0x41,
0x70, 0x69, 0x61, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69,
0x64, 0x41, 0x70, 0x69, 0x61, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x41, 0x71, 0x74, 0x61, 0x75, 0x2d, 0x74, 0x69, 0x64,
0x41, 0x71, 0x74, 0x61, 0x75, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x41, 0x71, 0x74, 0x61, 0x75, 0x2d, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65,
0x2d, 0x74, 0x69, 0x64, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x2d, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x71, 0x74, 0x6f,
0x62, 0x65, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x41, 0x72, 0x61, 0x62, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x41,
0x72, 0x61, 0x62, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x41, 0x72, 0x61, 0x62, 0x69, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74,
0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74,
0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x41, 0x72, 0x6d, 0x65, 0x6e,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x41, 0x72, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x63, 0x2d, 0x74, 0x69, 0x64, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x63, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x2d, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x6c, 0x69, 0x67,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73,
0x74, 0x6c, 0x69, 0x67, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x6c,
0x69, 0x67, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0xd8, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0xd8, 0x73, 0x74,
0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0xd8, 0x73, 0x74, 0x61, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x56, 0x65,
0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73,
0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x41, 0x73, 0x65, 0x72,
0x62, 0x61, 0x6a, 0x64, 0x73, 0x6a, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x73, 0x6a,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x73, 0x6a,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x72, 0x6e, 0x65, 0x2d, 0x74,
0x69, 0x64, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x72, 0x6e, 0x65, 0x2d, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x7a, 0x6f, 0x72,
0x65, 0x72, 0x6e, 0x65, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x2d, 0x74, 0x69, 0x64, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x73, 0x68, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x2d, 0x73,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x42, 0x68, 0x75, 0x74,
0x61, 0x6e, 0x2d, 0x74, 0x69, 0x64, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x42, 0x72, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x42,
0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x42, 0x72, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20,
0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x2d, 0x74,
0x69, 0x64, 0x4b, 0x61, 0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x2d,
0x74, 0x69, 0x64, 0x4b, 0x61, 0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65,
0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4b, 0x61,
0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x2d, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72,
0x6f, 0x2d, 0x74, 0x69, 0x64, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x2d, 0x74, 0x69, 0x64, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x2d,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x43, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x6e, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x6e, 0x73, 0x6b, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x43, 0x68, 0x69, 0x6c,
0x65, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x4b, 0x69, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x4b, 0x69, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4b, 0x69, 0x6e, 0x65,
0x73, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x4a, 0x75, 0x6c, 0x65, 0xf8, 0x65, 0x6e, 0x2d, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x43, 0x6f, 0x63, 0x6f, 0x73,
0xf8, 0x65, 0x72, 0x6e, 0x65, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x43, 0x6f, 0x6f, 0x6b, 0xf8, 0x65, 0x72, 0x6e, 0x65, 0x2d, 0x74, 0x69,
0x64, 0x43, 0x6f, 0x6f, 0x6b, 0xf8, 0x65, 0x72, 0x6e, 0x65, 0x2d, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x43, 0x6f, 0x6f, 0x6b,
0xf8, 0x65, 0x72, 0x6e, 0x65, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x43, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x43, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x43, 0x75, 0x62, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x44, 0x61, 0x76, 0x69, 0x73, 0x2d, 0x74, 0x69, 0x64, 0x44, 0x75, 0x6d,
0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c,
0x65, 0x2d, 0x74, 0x69, 0x64, 0xd8, 0x73, 0x74, 0x74, 0x69, 0x6d, 0x6f,
0x72, 0x2d, 0x74, 0x69, 0x64, 0x50, 0xe5, 0x73, 0x6b, 0x65, 0xf8, 0x65,
0x6e, 0x2d, 0x74, 0x69, 0x64, 0x50, 0xe5, 0x73, 0x6b, 0x65, 0xf8, 0x65,
0x6e, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x50,
0xe5, 0x73, 0x6b, 0x65, 0xf8, 0x65, 0x6e, 0x2d, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72,
0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe6, 0x69, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x65, 0x75, 0x72, 0x6f, 0x70, 0xe6, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe6, 0x69, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0xd8, 0x73, 0x74,
0x65, 0x75, 0x72, 0x6f, 0x70, 0xe6, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0xd8, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe6, 0x69, 0x73,
0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0xd8,
0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe6, 0x69, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x46, 0x6a, 0x65,
0x72, 0x6e, 0xf8, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe6, 0x69,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x65, 0x75,
0x72, 0x6f, 0x70, 0xe6, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x56,
0x65, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe6, 0x69, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x56, 0x65,
0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe6, 0x69, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x46, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0xf8, 0x65, 0x72, 0x6e, 0x65, 0x2d,
0x74, 0x69, 0x64, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0xf8, 0x65, 0x72, 0x6e, 0x65, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0xf8, 0x65, 0x72, 0x6e, 0x65, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x46, 0x69, 0x6a, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x46, 0x69, 0x6a, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x46, 0x69, 0x6a,
0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x46, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x47, 0x75,
0x79, 0x61, 0x6e, 0x61, 0x2d, 0x74, 0x69, 0x64, 0x46, 0x72, 0x61, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x53, 0x79, 0x64, 0x6c, 0x69, 0x67, 0x65, 0x20,
0x6f, 0x67, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x73,
0x6b, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x69, 0x74, 0x6f, 0x72, 0x69,
0x65, 0x72, 0x2d, 0x74, 0x69, 0x64, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61,
0x67, 0x6f, 0x73, 0x2d, 0x74, 0x69, 0x64, 0x47, 0x61, 0x6d, 0x62, 0x69,
0x65, 0x72, 0x2d, 0x74, 0x69, 0x64, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72,
0x74, 0xf8, 0x65, 0x72, 0x6e, 0x65, 0x2d, 0x74, 0x69, 0x64, 0x47, 0x4d,
0x54, 0xd8, 0x73, 0x74, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0xd8, 0x73, 0x74, 0x67, 0x72, 0xf8,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0xd8, 0x73, 0x74, 0x67, 0x72, 0xf8, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x67, 0x72, 0xf8, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x56, 0x65,
0x73, 0x74, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x56, 0x65,
0x73, 0x74, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x47, 0x75,
0x61, 0x6d, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x47, 0x6f, 0x6c, 0x66, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x6e, 0x65, 0x2d,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e,
0x2d, 0x74, 0x69, 0x64, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x2d, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x2d, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f,
0x6e, 0x67, 0x2d, 0x74, 0x69, 0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f,
0x6e, 0x67, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x2d, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x48, 0x6f, 0x76, 0x64, 0x2d, 0x74,
0x69, 0x64, 0x48, 0x6f, 0x76, 0x64, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x48, 0x6f, 0x76, 0x64, 0x2d, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x49, 0x6e, 0x64, 0x69, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x49, 0x6e,
0x64, 0x69, 0x73, 0x6b, 0x65, 0x20, 0x4f, 0x63, 0x65, 0x61, 0x6e, 0x2d,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x49, 0x6e, 0x64,
0x6f, 0x6b, 0x69, 0x6e, 0x61, 0x2d, 0x74, 0x69, 0x64, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0xd8, 0x73, 0x74, 0x69, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x56,
0x65, 0x73, 0x74, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x49, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x49, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x49, 0x72, 0x61, 0x6e, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x49,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2d, 0x74, 0x69, 0x64, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2d, 0x73,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x4a, 0x61, 0x70, 0x61,
0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x4a, 0x61, 0x70, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x4a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61,
0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68,
0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x74, 0x69, 0x64, 0x50, 0x65, 0x74,
0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b,
0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x4b, 0x61, 0x73, 0x61, 0x6b, 0x68, 0x73,
0x74, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0xd8, 0x73, 0x74,
0x6b, 0x61, 0x73, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x6b, 0x61, 0x73, 0x61,
0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x2d, 0x74, 0x69, 0x64, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2d, 0x74, 0x69,
0x64, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b,
0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2d, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x4b, 0x69, 0x72, 0x67, 0x69,
0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x4c, 0x61, 0x6e, 0x67,
0x61, 0x20, 0x74, 0x69, 0x64, 0x4c, 0x69, 0x6e, 0x6a, 0x65, 0xf8, 0x65,
0x72, 0x6e, 0x65, 0x2d, 0x74, 0x69, 0x64, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x2d, 0x74, 0x69, 0x64, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x4d, 0x61,
0x63, 0x61, 0x6f, 0x2d, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x63, 0x61, 0x6f,
0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4d, 0x61,
0x63, 0x61, 0x6f, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x2d, 0x74, 0x69, 0x64,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x2d, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x4d, 0x61,
0x6c, 0x61, 0x79, 0x73, 0x69, 0x61, 0x2d, 0x74, 0x69, 0x64, 0x4d, 0x61,
0x6c, 0x64, 0x69, 0x76, 0x65, 0x72, 0x6e, 0x65, 0x2d, 0x74, 0x69, 0x64,
0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x2d, 0x74, 0x69,
0x64, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0xf8, 0x65, 0x72,
0x6e, 0x65, 0x2d, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74,
0x69, 0x75, 0x73, 0x2d, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x74, 0x69, 0x75, 0x73, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x2d,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x77,
0x73, 0x6f, 0x6e, 0x2d, 0x74, 0x69, 0x64, 0x4d, 0x65, 0x78, 0x69, 0x63,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63,
0x2d, 0x74, 0x69, 0x64, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x73,
0x6b, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x2d, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4d, 0x65, 0x78, 0x69, 0x63,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63,
0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x55, 0x6c,
0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x2d, 0x74, 0x69, 0x64,
0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x2d, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x55, 0x6c, 0x61, 0x6e,
0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x2d, 0x74,
0x69, 0x64, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x2d, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61,
0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x4d, 0x79,
0x61, 0x6e, 0x6d, 0x61, 0x72, 0x2d, 0x74, 0x69, 0x64, 0x4e, 0x61, 0x75,
0x72, 0x75, 0x2d, 0x74, 0x69, 0x64, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x65,
0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x4e, 0x79, 0x20, 0x4b,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x65, 0x6e, 0x2d, 0x74, 0x69,
0x64, 0x4e, 0x79, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69,
0x65, 0x6e, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x4e, 0x79, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x65,
0x6e, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x4e,
0x65, 0x77, 0x7a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x4e, 0x65, 0x77, 0x7a, 0x65, 0x61, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69,
0x64, 0x4e, 0x65, 0x77, 0x7a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x4e,
0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e,
0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e,
0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x4e, 0x69, 0x75, 0x65, 0x2d, 0x74, 0x69,
0x64, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x2d, 0x74, 0x69, 0x64, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x2d, 0x74, 0x69, 0x64, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x4e, 0x6f, 0x72, 0x64, 0x6d, 0x61, 0x72, 0x69,
0x61, 0x6e, 0x65, 0x72, 0x6e, 0x65, 0x2d, 0x74, 0x69, 0x64, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2d, 0x74, 0x69,
0x64, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2d, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x4f, 0x6d, 0x73, 0x6b, 0x2d,
0x74, 0x69, 0x64, 0x4f, 0x6d, 0x73, 0x6b, 0x2d, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x4f, 0x6d, 0x73, 0x6b, 0x2d, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x2d, 0x74, 0x69, 0x64, 0x50, 0x61,
0x70, 0x75, 0x61, 0x20, 0x4e, 0x79, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65,
0x61, 0x2d, 0x74, 0x69, 0x64, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x50, 0x61, 0x72,
0x61, 0x67, 0x75, 0x61, 0x79, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x79, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x50, 0x65, 0x72, 0x75, 0x76, 0x69, 0x61,
0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x50, 0x65, 0x72, 0x75, 0x76,
0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x50, 0x65, 0x72, 0x75, 0x76, 0x69, 0x61, 0x6e, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x46,
0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x73, 0x6b, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x46, 0x69, 0x6c,
0x69, 0x70, 0x70, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78,
0xf8, 0x65, 0x6e, 0x2d, 0x74, 0x69, 0x64, 0x53, 0x61, 0x69, 0x6e, 0x74,
0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x20, 0x6f, 0x67, 0x20,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x74, 0x69, 0x64,
0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x2d, 0x20, 0x6f, 0x67, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f,
0x6e, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d,
0x20, 0x6f, 0x67, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x50, 0x69,
0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x2d, 0x74, 0x69, 0x64, 0x50, 0x6f,
0x6e, 0x61, 0x70, 0x65, 0x2d, 0x74, 0x69, 0x64, 0x50, 0x79, 0x6f, 0x6e,
0x67, 0x79, 0x61, 0x6e, 0x67, 0x2d, 0x74, 0x69, 0x64, 0x51, 0x79, 0x7a,
0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x2d, 0x74, 0x69, 0x64, 0x51, 0x79,
0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x2d, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72,
0x64, 0x61, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x2d, 0x74, 0x69, 0x64, 0x52,
0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x2d, 0x74, 0x69, 0x64, 0x53, 0x61,
0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x74, 0x69, 0x64, 0x53, 0x61,
0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x53, 0x61,
0x6d, 0x61, 0x72, 0x61, 0x2d, 0x74, 0x69, 0x64, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x61, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c,
0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x53, 0x69, 0x6e, 0x67, 0x61,
0x70, 0x6f, 0x72, 0x65, 0x2d, 0x74, 0x69, 0x64, 0x53, 0x61, 0x6c, 0x6f,
0x6d, 0x6f, 0x6e, 0xf8, 0x65, 0x72, 0x6e, 0x65, 0x2d, 0x74, 0x69, 0x64,
0x53, 0x6f, 0x75, 0x74, 0x68, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x2d, 0x74, 0x69, 0x64, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d,
0x2d, 0x74, 0x69, 0x64, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x2d, 0x74, 0x69,
0x64, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x2d, 0x74, 0x69, 0x64, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0x2d, 0x74, 0x69, 0x64, 0x54, 0x61, 0x69,
0x70, 0x65, 0x69, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69,
0x64, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x2d, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x54, 0x61, 0x64, 0x73, 0x6a, 0x69, 0x6b,
0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x54, 0x6f, 0x6b, 0x65, 0x6c,
0x61, 0x75, 0x2d, 0x74, 0x69, 0x64, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x2d,
0x74, 0x69, 0x64, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x73,
0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x54,
0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75,
0x2d, 0x74, 0x69, 0x64, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x61,
0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x79, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x61,
0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x55, 0x73, 0x62, 0x65, 0x6b,
0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x2d, 0x74, 0x69, 0x64,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x2d, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75,
0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x56, 0x65,
0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x2d, 0x74, 0x69, 0x64, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2d, 0x74, 0x69, 0x64,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2d, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x74, 0x69, 0x64,
0x57, 0x61, 0x6b, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x2d,
0x74, 0x69, 0x64, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x6f, 0x67,
0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x2d, 0x74, 0x69, 0x64, 0x4a,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2d, 0x74, 0x69, 0x64, 0x4a, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2d, 0x73,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x4a, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x2d, 0x74, 0x69,
0x64, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x59,
0x75, 0x6b, 0x6f, 0x6e, 0x2d, 0x74, 0x69, 0x64, 0x909, 0x924, 0x94d, 0x924,
0x930, 0x940, 0x20, 0x905, 0x92e, 0x930, 0x940, 0x915, 0x940, 0x20, 0x915, 0x947,
0x902, 0x926, 0x930, 0x940, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x909, 0x924, 0x94d,
0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x930, 0x940, 0x915, 0x940, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x915, 0x947, 0x902, 0x926, 0x930, 0x940, 0x20, 0x938,
0x92e, 0x93e, 0x902, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e,
0x930, 0x940, 0x915, 0x940, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20,
0x915, 0x947, 0x902, 0x926, 0x930, 0x940, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x909,
0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x930, 0x940, 0x915, 0x940,
0x20, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x938, 0x92e, 0x93e, 0x902,
0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x930, 0x940, 0x915,
0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x92a, 0x942, 0x930, 0x94d, 0x935,
0x940, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940,
0x20, 0x905, 0x92e, 0x930, 0x940, 0x915, 0x940, 0x20, 0x921, 0x947, 0x932, 0x93e,
0x907, 0x91f, 0x20, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x938, 0x92e,
0x93e, 0x902, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x930,
0x940, 0x915, 0x940, 0x20, 0x92e, 0x93e, 0x909, 0x902, 0x91f, 0x947, 0x928, 0x20,
0x938, 0x92e, 0x93e, 0x902, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905,
0x92e, 0x930, 0x940, 0x915, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x92e,
0x93e, 0x909, 0x902, 0x91f, 0x947, 0x928, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x909,
0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x930, 0x940, 0x915, 0x940,
0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92e, 0x93e, 0x909, 0x902,
0x91f, 0x947, 0x928, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x909, 0x924, 0x94d, 0x924,
0x930, 0x940, 0x20, 0x905, 0x92e, 0x930, 0x940, 0x915, 0x940, 0x20, 0x92a, 0x94d,
0x930, 0x936, 0x93e, 0x902, 0x924, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x909, 0x924,
0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x930, 0x940, 0x915, 0x940, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924,
0x20, 0x938, 0x92e, 0x93e, 0x902, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20,
0x905, 0x92e, 0x930, 0x940, 0x915, 0x940, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907,
0x91f, 0x20, 0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924, 0x20, 0x938, 0x92e,
0x93e, 0x902, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20, 0x938,
0x92e, 0x93e, 0x902, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x905, 0x91f, 0x932,
0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f,
0x20, 0x938, 0x92e, 0x93e, 0x902, 0x915, 0x947, 0x902, 0x926, 0x930, 0x940, 0x20,
0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x915,
0x947, 0x902, 0x926, 0x930, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x915, 0x947,
0x902, 0x926, 0x930, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x20,
0x917, 0x930, 0x94d, 0x92e, 0x93f, 0x92f, 0x947, 0x902, 0x20, 0x926, 0x93e, 0x20,
0x938, 0x92e, 0x93e, 0x902, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x92f,
0x942, 0x930, 0x94b, 0x92a, 0x940, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x909, 0x924,
0x94d, 0x924, 0x930, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x909, 0x924, 0x94d,
0x924, 0x930, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x20, 0x917,
0x930, 0x94d, 0x92e, 0x93f, 0x92f, 0x947, 0x902, 0x20, 0x926, 0x93e, 0x20, 0x938,
0x92e, 0x93e, 0x902, 0x92a, 0x91a, 0x94d, 0x91b, 0x92e, 0x940, 0x20, 0x92f, 0x942,
0x930, 0x94b, 0x92a, 0x940, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x92a, 0x91a, 0x94d,
0x91b, 0x92e, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x92a, 0x91a, 0x94d, 0x91b,
0x92e, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x20, 0x917, 0x930,
0x94d, 0x92e, 0x93f, 0x92f, 0x947, 0x902, 0x20, 0x926, 0x93e, 0x20, 0x938, 0x92e,
0x93e, 0x902, 0x41, 0x63, 0x72, 0x65, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x41,
0x63, 0x72, 0x65, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x63, 0x72, 0x65, 0x2d, 0x7a, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x66, 0x67, 0x68, 0x61,
0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x61, 0x6c, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4f, 0x6f, 0x73,
0x74, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x65,
0x20, 0x74, 0x69, 0x6a, 0x64, 0x5a, 0x75, 0x69, 0x64, 0x2d, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a,
0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x65, 0x73,
0x74, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69,
0x6a, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x6a, 0x64, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x74, 0x69,
0x6a, 0x64, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x6a, 0x64, 0x41, 0x6c, 0x6d, 0x61, 0x2d, 0x41, 0x74, 0x61, 0x2d, 0x74,
0x69, 0x6a, 0x64, 0x41, 0x6c, 0x6d, 0x61, 0x2d, 0x41, 0x74, 0x61, 0x2d,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a,
0x64, 0x41, 0x6c, 0x6d, 0x61, 0x2d, 0x41, 0x74, 0x61, 0x2d, 0x7a, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x65, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x65, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x74, 0x69, 0x6a, 0x64, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x65, 0x2d,
0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x45,
0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x45,
0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x45, 0x61, 0x73, 0x74,
0x65, 0x72, 0x6e, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a,
0x64, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x2d, 0x74, 0x69,
0x6a, 0x64, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64,
0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x2d, 0x7a, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69,
0x63, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69,
0x63, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x69, 0x6a, 0x64, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x2d, 0x7a,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x6e, 0x61, 0x64,
0x79, 0x72, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x6e, 0x61, 0x64, 0x79,
0x72, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x69, 0x6a, 0x64, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x2d, 0x7a, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x70, 0x69, 0x61, 0x2d,
0x74, 0x69, 0x6a, 0x64, 0x41, 0x70, 0x69, 0x61, 0x2d, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x70,
0x69, 0x61, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64,
0x41, 0x71, 0x74, 0x61, 0x75, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x71,
0x74, 0x61, 0x75, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x71, 0x74, 0x61, 0x75, 0x2d, 0x7a,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x71, 0x74, 0xf6,
0x62, 0x65, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x71, 0x74, 0xf6, 0x62,
0x65, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x69, 0x6a, 0x64, 0x41, 0x71, 0x74, 0xf6, 0x62, 0x65, 0x2d, 0x7a, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x72, 0x61, 0x62, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x72, 0x61,
0x62, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x72, 0x61, 0x62,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x6a, 0x64, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6a, 0x6e,
0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6a, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6a, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6a, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69,
0x6a, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6a, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x65, 0x73, 0x74,
0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6a, 0x6e, 0x73, 0x65,
0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x72,
0x6d, 0x65, 0x65, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x41,
0x72, 0x6d, 0x65, 0x65, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x72, 0x6d,
0x65, 0x65, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x6a, 0x64, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x2d,
0x74, 0x69, 0x6a, 0x64, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69,
0x6a, 0x64, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x2d, 0x7a,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x69, 0x64, 0x64,
0x65, 0x6e, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x69, 0x64, 0x64,
0x65, 0x6e, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x69, 0x64, 0x64, 0x65, 0x6e, 0x2d,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x69,
0x64, 0x64, 0x65, 0x6e, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x77, 0x65, 0x73, 0x74, 0x65, 0x6c,
0x69, 0x6a, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x69, 0x64,
0x64, 0x65, 0x6e, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x77, 0x65, 0x73, 0x74, 0x65, 0x6c, 0x69,
0x6a, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x69, 0x64, 0x64, 0x65, 0x6e, 0x2d,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x77, 0x65, 0x73, 0x74, 0x65, 0x6c, 0x69, 0x6a, 0x6b, 0x65, 0x20,
0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x4f, 0x6f, 0x73,
0x74, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4f, 0x6f, 0x73, 0x74, 0x2d,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69,
0x6a, 0x64, 0x4f, 0x6f, 0x73, 0x74, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x74,
0x69, 0x6a, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x65,
0x73, 0x74, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a,
0x64, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65, 0x69, 0x64, 0x7a, 0x6a, 0x61,
0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x65, 0x69, 0x64, 0x7a, 0x6a, 0x61, 0x61, 0x6e, 0x73, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69,
0x6a, 0x64, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65, 0x69, 0x64, 0x7a, 0x6a,
0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x6a, 0x64, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x6e, 0x2d, 0x74, 0x69,
0x6a, 0x64, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x6e, 0x2d, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x7a,
0x6f, 0x72, 0x65, 0x6e, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x6a, 0x64, 0x42, 0x65, 0x6e, 0x67, 0x61, 0x6c, 0x65, 0x73, 0x65, 0x20,
0x74, 0x69, 0x6a, 0x64, 0x42, 0x65, 0x6e, 0x67, 0x61, 0x6c, 0x65, 0x73,
0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x69, 0x6a, 0x64, 0x42, 0x65, 0x6e, 0x67, 0x61, 0x6c, 0x65, 0x73, 0x65,
0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x42, 0x68,
0x75, 0x74, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20,
0x74, 0x69, 0x6a, 0x64, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x61,
0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x42, 0x72, 0x61,
0x7a, 0x69, 0x6c, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x42,
0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20,
0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x42, 0x72, 0x75,
0x6e, 0x65, 0x69, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4b, 0x61,
0x61, 0x70, 0x76, 0x65, 0x72, 0x64, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x74, 0x69, 0x6a, 0x64, 0x4b, 0x61, 0x61, 0x70, 0x76, 0x65, 0x72, 0x64,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4b, 0x61, 0x61, 0x70, 0x76,
0x65, 0x72, 0x64, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x7a, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x61, 0x73, 0x65, 0x79, 0x20,
0x74, 0x69, 0x6a, 0x64, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f,
0x2d, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x2d, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69,
0x6a, 0x64, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x2d, 0x7a, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x68, 0x69, 0x6c, 0x65,
0x65, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x68, 0x69,
0x6c, 0x65, 0x65, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x68, 0x69, 0x6c,
0x65, 0x65, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x6a, 0x64, 0x43, 0x68, 0x69, 0x6e, 0x65, 0x73, 0x65, 0x20, 0x74,
0x69, 0x6a, 0x64, 0x43, 0x68, 0x69, 0x6e, 0x65, 0x73, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64,
0x43, 0x68, 0x69, 0x6e, 0x65, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d,
0x61, 0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x74,
0x69, 0x6a, 0x64, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x65, 0x69, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x6f, 0x6c,
0x6f, 0x6d, 0x62, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69,
0x6a, 0x64, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x61, 0x6e,
0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x74, 0x69, 0x6a, 0x64, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61,
0x61, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x6a, 0x64, 0x43, 0x6f, 0x6f, 0x6b, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x6f, 0x6f, 0x6b, 0x65,
0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x6f, 0x6f,
0x6b, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x68, 0x61,
0x6c, 0x76, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a,
0x64, 0x43, 0x75, 0x62, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69,
0x6a, 0x64, 0x43, 0x75, 0x62, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64,
0x43, 0x75, 0x62, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x44, 0x61, 0x76, 0x69, 0x73, 0x2d,
0x74, 0x69, 0x6a, 0x64, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64,
0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x2d, 0x74, 0x69, 0x6a,
0x64, 0x4f, 0x6f, 0x73, 0x74, 0x2d, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x65,
0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x61, 0x61, 0x73, 0x65,
0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x50, 0x61, 0x61, 0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69,
0x6a, 0x64, 0x50, 0x61, 0x61, 0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64,
0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x61, 0x61, 0x6e, 0x73, 0x65,
0x20, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x69, 0x64, 0x64, 0x65, 0x6e, 0x2d,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a,
0x64, 0x4d, 0x69, 0x64, 0x64, 0x65, 0x6e, 0x2d, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x69, 0x64, 0x64, 0x65, 0x6e,
0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x73, 0x65, 0x20, 0x7a, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x4f, 0x6f, 0x73, 0x74, 0x2d,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a,
0x64, 0x4f, 0x6f, 0x73, 0x74, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x74, 0x69, 0x6a, 0x64, 0x4f, 0x6f, 0x73, 0x74, 0x2d, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x6a, 0x64, 0x56, 0x65, 0x72, 0x64, 0x65, 0x72, 0x2d, 0x6f, 0x6f,
0x73, 0x74, 0x65, 0x6c, 0x69, 0x6a, 0x6b, 0x2d, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x65, 0x73,
0x74, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x73, 0x65, 0x20, 0x74,
0x69, 0x6a, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x6a, 0x64, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x74, 0x69,
0x6a, 0x64, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x69,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65,
0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x46, 0x69,
0x6a, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x46,
0x69, 0x6a, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x46, 0x69, 0x6a,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x6a, 0x64, 0x46, 0x72, 0x61, 0x6e, 0x73, 0x2d, 0x47, 0x75, 0x79,
0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x46, 0x72,
0x61, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x75, 0x69, 0x64, 0x65, 0x6c, 0x69,
0x6a, 0x6b, 0x65, 0x20, 0x65, 0x6e, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72,
0x63, 0x74, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x65, 0x69, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x47, 0x61,
0x6d, 0x62, 0x69, 0x65, 0x72, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x6a, 0x64, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74,
0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a,
0x64, 0x4f, 0x6f, 0x73, 0x74, 0x2d, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4f, 0x6f,
0x73, 0x74, 0x2d, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x74, 0x69, 0x6a, 0x64, 0x4f, 0x6f, 0x73, 0x74, 0x2d, 0x47, 0x72, 0x6f,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x47,
0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x74,
0x69, 0x6a, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x47, 0x72, 0x6f, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x65, 0x73,
0x74, 0x2d, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x47,
0x75, 0x61, 0x6d, 0x65, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x47, 0x6f, 0x6c, 0x66,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69,
0x6a, 0x64, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65,
0x6f, 0x65, 0x74, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x74, 0x69, 0x6a,
0x64, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x6f,
0x65, 0x74, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x6f, 0x65, 0x74, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a,
0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x73, 0x65, 0x20,
0x74, 0x69, 0x6a, 0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67,
0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x74, 0x69, 0x6a, 0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67,
0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64,
0x48, 0x6f, 0x76, 0x64, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x48, 0x6f, 0x76,
0x64, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x69, 0x6a, 0x64, 0x48, 0x6f, 0x76, 0x64, 0x2d, 0x7a, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x6a, 0x64, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x73, 0x65,
0x20, 0x74, 0x69, 0x6a, 0x64, 0x49, 0x6e, 0x64, 0x69, 0x73, 0x63, 0x68,
0x65, 0x20, 0x4f, 0x63, 0x65, 0x61, 0x61, 0x6e, 0x2d, 0x74, 0x69, 0x6a,
0x64, 0x49, 0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x65, 0x73, 0x65,
0x20, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61,
0x6c, 0x2d, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4f, 0x6f, 0x73, 0x74, 0x2d,
0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x74,
0x69, 0x6a, 0x64, 0x49, 0x72, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74,
0x69, 0x6a, 0x64, 0x49, 0x72, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64,
0x49, 0x72, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x6a, 0x64, 0x49, 0x72, 0x6b, 0x6f, 0x65, 0x74, 0x73,
0x6b, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x49, 0x72, 0x6b, 0x6f, 0x65, 0x74,
0x73, 0x6b, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x74, 0x69, 0x6a, 0x64, 0x49, 0x72, 0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b,
0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x49, 0x73,
0x72, 0x61, 0xeb, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x74, 0x69,
0x6a, 0x64, 0x49, 0x73, 0x72, 0x61, 0xeb, 0x6c, 0x69, 0x73, 0x63, 0x68,
0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x69, 0x6a, 0x64, 0x49, 0x73, 0x72, 0x61, 0xeb, 0x6c, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64,
0x4a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x4a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4a, 0x61, 0x70,
0x61, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x6a, 0x64, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x74, 0x73, 0x6a, 0x61, 0x74,
0x73, 0x6b, 0x69, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x65, 0x74, 0x72,
0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61,
0x6d, 0x74, 0x73, 0x6a, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x50,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x2d, 0x4b, 0x61, 0x6d, 0x74, 0x73, 0x6a, 0x61, 0x74, 0x73, 0x6b, 0x69,
0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x4b, 0x61,
0x7a, 0x61, 0x63, 0x68, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4f,
0x6f, 0x73, 0x74, 0x2d, 0x4b, 0x61, 0x7a, 0x61, 0x63, 0x68, 0x73, 0x65,
0x20, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x4b, 0x61,
0x7a, 0x61, 0x63, 0x68, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4b,
0x6f, 0x72, 0x65, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a,
0x64, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64,
0x4b, 0x6f, 0x72, 0x65, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x4b, 0x6f, 0x73, 0x72, 0x61,
0x65, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2d, 0x74, 0x69, 0x6a, 0x64,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2d,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a,
0x64, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b,
0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x4b, 0x69,
0x72, 0x67, 0x69, 0x7a, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x74, 0x69,
0x6a, 0x64, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x2d, 0x74, 0x69, 0x6a, 0x64,
0x4c, 0x69, 0x6e, 0x65, 0x2d, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x2d, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65,
0x20, 0x74, 0x69, 0x6a, 0x64, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x2d, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a,
0x64, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0x65,
0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x73, 0x65,
0x20, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x73, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69,
0x6a, 0x64, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x73, 0x65, 0x20, 0x7a, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x74, 0x69, 0x6a, 0x64, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x2d,
0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x61, 0x6c,
0x65, 0x69, 0x73, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x74, 0x69, 0x6a,
0x64, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73,
0x61, 0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x74,
0x69, 0x6a, 0x64, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x65,
0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x65,
0x20, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69,
0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x74, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x2d,
0x74, 0x69, 0x6a, 0x64, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x61, 0x6e,
0x73, 0x65, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x2d, 0x74,
0x69, 0x6a, 0x64, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x61, 0x6e, 0x73,
0x65, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4d,
0x65, 0x78, 0x69, 0x63, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x50, 0x61,
0x63, 0x69, 0x66, 0x69, 0x63, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x6a, 0x64, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74,
0x61, 0x72, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x55, 0x6c, 0x61, 0x61, 0x6e,
0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x55, 0x6c, 0x61, 0x61,
0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x2d, 0x7a, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x75, 0x2d,
0x74, 0x69, 0x6a, 0x64, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x75, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64,
0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x75, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x6a, 0x64, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x65,
0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4e, 0x61, 0x75, 0x72, 0x75,
0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4e, 0x65,
0x70, 0x61, 0x6c, 0x65, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4e,
0x69, 0x65, 0x75, 0x77, 0x2d, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4e, 0x69,
0x65, 0x75, 0x77, 0x2d, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4e, 0x69, 0x65, 0x75, 0x77, 0x2d,
0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x4e, 0x69,
0x65, 0x75, 0x77, 0x2d, 0x5a, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4e, 0x69, 0x65, 0x75, 0x77, 0x2d,
0x5a, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4e,
0x69, 0x65, 0x75, 0x77, 0x2d, 0x5a, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64,
0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64,
0x2d, 0x74, 0x69, 0x6a, 0x64, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e,
0x64, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4e, 0x65, 0x77, 0x66, 0x6f,
0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x7a, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x6a, 0x64, 0x4e, 0x69, 0x75, 0x65, 0x73, 0x65, 0x20,
0x74, 0x69, 0x6a, 0x64, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x65,
0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x65, 0x69, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x69, 0x6a, 0x64, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x69, 0x6a, 0x64, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x7a,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x4e, 0x6f, 0x6f, 0x72,
0x64, 0x65, 0x6c, 0x69, 0x6a, 0x6b, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x61,
0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2d, 0x74, 0x69, 0x6a,
0x64, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69,
0x6a, 0x64, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x4f,
0x6d, 0x73, 0x6b, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x4f, 0x6d, 0x73, 0x6b,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69,
0x6a, 0x64, 0x4f, 0x6d, 0x73, 0x6b, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x6a, 0x64, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61,
0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x42, 0x65, 0x6c, 0x61, 0x75, 0x73,
0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x61, 0x70, 0x6f, 0x65, 0x61,
0x2d, 0x4e, 0x69, 0x65, 0x75, 0x77, 0x2d, 0x47, 0x75, 0x69, 0x6e, 0x65,
0x65, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x79, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69,
0x6a, 0x64, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x61, 0x61,
0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x6a, 0x64, 0x50, 0x65, 0x72, 0x75, 0x61, 0x61, 0x6e, 0x73,
0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x65, 0x72, 0x75, 0x61, 0x61,
0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x65, 0x72, 0x75, 0x61, 0x61, 0x6e,
0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64,
0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6a, 0x6e, 0x73, 0x65, 0x20, 0x74,
0x69, 0x6a, 0x64, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6a, 0x6e, 0x73,
0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x69, 0x6a, 0x64, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6a, 0x6e, 0x73,
0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x50,
0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x53, 0x61, 0x69, 0x6e, 0x74,
0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x6e, 0x20, 0x4d,
0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x74, 0x69, 0x6a, 0x64,
0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x20, 0x65, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69,
0x6a, 0x64, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x20, 0x65, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64,
0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x65, 0x69, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x6f, 0x68,
0x6e, 0x70, 0x65, 0x69, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x79, 0x6f,
0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x51,
0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x2d, 0x74, 0x69, 0x6a,
0x64, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64,
0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x2d, 0x7a, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x52, 0xe9, 0x75, 0x6e, 0x69,
0x6f, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x52, 0x6f, 0x74,
0x68, 0x65, 0x72, 0x61, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x53, 0x61, 0x63,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x53, 0x61,
0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x53, 0x61, 0x63, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x6a, 0x64, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d, 0x74, 0x69, 0x6a,
0x64, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x53, 0x61, 0x6d,
0x61, 0x72, 0x61, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a,
0x64, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74,
0x69, 0x6a, 0x64, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x61, 0x6e, 0x73, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69,
0x6a, 0x64, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20,
0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x53, 0x65, 0x79,
0x63, 0x68, 0x65, 0x6c, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x53,
0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65, 0x73, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64,
0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x73, 0x65, 0x69, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x5a, 0x75, 0x69,
0x64, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x74, 0x69, 0x6a, 0x64, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x61,
0x6d, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x53, 0x79, 0x6f, 0x77,
0x61, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69,
0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x54, 0x61,
0x69, 0x70, 0x65, 0x69, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x54, 0x61, 0x69,
0x70, 0x65, 0x69, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x69, 0x6a, 0x64, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x2d,
0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x54, 0x61, 0x64,
0x7a, 0x6a, 0x69, 0x65, 0x6b, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x2d, 0x65, 0x69, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x73, 0x65,
0x20, 0x74, 0x69, 0x6a, 0x64, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x65,
0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x65, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x65, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x6a, 0x64, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x61, 0x61,
0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x55, 0x72, 0x75, 0x67,
0x75, 0x61, 0x79, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a,
0x64, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x61, 0x61, 0x6e, 0x73,
0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x69, 0x6a, 0x64, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x61, 0x61,
0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a,
0x64, 0x4f, 0x65, 0x7a, 0x62, 0x65, 0x65, 0x6b, 0x73, 0x65, 0x20, 0x74,
0x69, 0x6a, 0x64, 0x4f, 0x65, 0x7a, 0x62, 0x65, 0x65, 0x6b, 0x73, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69,
0x6a, 0x64, 0x4f, 0x65, 0x7a, 0x62, 0x65, 0x65, 0x6b, 0x73, 0x65, 0x20,
0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69,
0x6a, 0x64, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x61, 0x61, 0x6e,
0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x74, 0x69, 0x6a, 0x64, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x61,
0x61, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x6a, 0x64, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x6f, 0x6c, 0x61, 0x61, 0x6e,
0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64,
0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d,
0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x57,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2d, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2d, 0x7a, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x6a, 0x64, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d,
0x74, 0x69, 0x6a, 0x64, 0x57, 0x61, 0x6b, 0x65, 0x2d, 0x65, 0x69, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x61,
0x6c, 0x6c, 0x69, 0x73, 0x20, 0x65, 0x6e, 0x20, 0x46, 0x75, 0x74, 0x75,
0x6e, 0x61, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4a, 0x61, 0x6b,
0x6f, 0x65, 0x74, 0x73, 0x6b, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x4a, 0x61,
0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4a, 0x61, 0x6b, 0x6f,
0x65, 0x74, 0x73, 0x6b, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x6a, 0x64, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x65,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x4a, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x65, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x69, 0x6a, 0x64, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x65, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x6a, 0x64, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x2d, 0x74, 0x69,
0x6a, 0x64, 0xf68, 0xf55, 0xf0b, 0xf42, 0xf0b, 0xf53, 0xf72, 0xf0b, 0xf66, 0xf84,
0xf4f, 0xf71, 0xf53, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf51, 0xf56, 0xf74,
0xf66, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf55, 0xf0b, 0xf62,
0xf72, 0xf0b, 0xf40, 0xf71, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf64,
0xf62, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf55, 0xf0b, 0xf62,
0xf72, 0xf0b, 0xf40, 0xf71, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf63,
0xfb7, 0xf7c, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf55, 0xf0b,
0xf62, 0xf72, 0xf0b, 0xf40, 0xf71, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf53, 0xf74, 0xf56, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf55,
0xf0b, 0xf62, 0xf72, 0xf0b, 0xf40, 0xf71, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf53, 0xf74, 0xf56, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68,
0xf55, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf40, 0xf71, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63,
0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf74, 0xf56,
0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf55, 0xf0b, 0xf62, 0xf72,
0xf0b, 0xf40, 0xf71, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf0b, 0xf63, 0xf66, 0xf72,
0xf0b, 0xf40, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf0b, 0xf63,
0xf66, 0xf72, 0xf0b, 0xf40, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf0b, 0xf63, 0xf66, 0xf72, 0xf0b,
0xf40, 0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf7a, 0xf0b, 0xf58, 0xf0b, 0xf5b, 0xf71,
0xf7c, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf7a, 0xf0b,
0xf58, 0xf0b, 0xf5b, 0xf71, 0xf7c, 0xf53, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1,
0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf7a, 0xf0b, 0xf58,
0xf0b, 0xf5b, 0xf71, 0xf7c, 0xf53, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51,
0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb1, 0xf44,
0xf0b, 0xf68, 0xf0b, 0xf58, 0xf72, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf40, 0xf0b, 0xf51,
0xf56, 0xf74, 0xf66, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb1, 0xf44, 0xf0b, 0xf68, 0xf0b, 0xf58, 0xf72,
0xf0b, 0xf62, 0xf72, 0xf0b, 0xf40, 0xf0b, 0xf51, 0xf56, 0xf74, 0xf66, 0xf0b, 0xf55,
0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb1, 0xf44, 0xf0b, 0xf68, 0xf0b,
0xf58, 0xf72, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf40, 0xf0b, 0xf51, 0xf56, 0xf74, 0xf66,
0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66,
0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb1,
0xf44, 0xf0b, 0xf68, 0xf0b, 0xf58, 0xf72, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf40, 0xf0b,
0xf64, 0xf62, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb1, 0xf44, 0xf0b, 0xf68, 0xf0b, 0xf58, 0xf72, 0xf0b,
0xf62, 0xf72, 0xf0b, 0xf40, 0xf0b, 0xf64, 0xf62, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42,
0xf66, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb1, 0xf44, 0xf0b, 0xf68, 0xf0b, 0xf58, 0xf72, 0xf0b,
0xf62, 0xf72, 0xf0b, 0xf40, 0xf0b, 0xf64, 0xf62, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42,
0xf66, 0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb1, 0xf44, 0xf0b, 0xf68, 0xf0b, 0xf58,
0xf72, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf40, 0xf0b, 0xf58, 0xf60, 0xf74, 0xf0b, 0xf4a,
0xf7a, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb1, 0xf44,
0xf0b, 0xf68, 0xf0b, 0xf58, 0xf72, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf40, 0xf0b, 0xf58,
0xf60, 0xf74, 0xf0b, 0xf4a, 0xf7a, 0xf53, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1,
0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb1, 0xf44, 0xf0b,
0xf68, 0xf0b, 0xf58, 0xf72, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf40, 0xf0b, 0xf58, 0xf60,
0xf74, 0xf0b, 0xf4a, 0xf7a, 0xf53, 0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2,
0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb1, 0xf44,
0xf0b, 0xf68, 0xf0b, 0xf58, 0xf72, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf40, 0xf0b, 0xf54,
0xf7a, 0xf0b, 0xf66, 0xf72, 0xf0b, 0xf55, 0xf72, 0xf42, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb1, 0xf44, 0xf0b, 0xf68, 0xf0b, 0xf58, 0xf72, 0xf0b,
0xf62, 0xf72, 0xf0b, 0xf40, 0xf0b, 0xf54, 0xf7a, 0xf0b, 0xf66, 0xf72, 0xf0b, 0xf55,
0xf72, 0xf42, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb1, 0xf44, 0xf0b, 0xf68, 0xf0b, 0xf58, 0xf72,
0xf0b, 0xf62, 0xf72, 0xf0b, 0xf40, 0xf0b, 0xf54, 0xf7a, 0xf0b, 0xf66, 0xf72, 0xf0b,
0xf55, 0xf72, 0xf42, 0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf0b, 0xf62, 0xf7a, 0xf0b,
0xf56, 0xfb7, 0xf72, 0xf0b, 0xf61, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf68, 0xf0b, 0xf62, 0xf7a, 0xf0b, 0xf56, 0xfb7, 0xf72, 0xf0b, 0xf61, 0xf53,
0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf68, 0xf0b, 0xf62, 0xf7a, 0xf0b, 0xf56, 0xfb7, 0xf72, 0xf0b, 0xf61,
0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf68, 0xf62, 0xf0b, 0xf47, 0xf7a, 0xf53, 0xf0b, 0xf4a, 0xf72, 0xf0b, 0xf53,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf62, 0xf0b, 0xf47, 0xf7a,
0xf53, 0xf0b, 0xf4a, 0xf72, 0xf0b, 0xf53, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1,
0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf62, 0xf0b, 0xf47,
0xf7a, 0xf53, 0xf0b, 0xf4a, 0xf72, 0xf0b, 0xf53, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62,
0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53,
0xf74, 0xf56, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf62, 0xf0b,
0xf47, 0xf7a, 0xf53, 0xf0b, 0xf4a, 0xf72, 0xf0b, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf53, 0xf74, 0xf56, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66,
0xf0b, 0xf68, 0xf62, 0xf0b, 0xf47, 0xf7a, 0xf53, 0xf0b, 0xf4a, 0xf72, 0xf0b, 0xf53,
0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf53, 0xf74, 0xf56, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b,
0xf68, 0xf62, 0xf0b, 0xf47, 0xf7a, 0xf53, 0xf0b, 0xf4a, 0xf72, 0xf0b, 0xf53, 0xf0b,
0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf68, 0xf62, 0xf0b, 0xf58, 0xf72, 0xf0b, 0xf53, 0xf72, 0xf0b,
0xf61, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf62, 0xf0b, 0xf58,
0xf72, 0xf0b, 0xf53, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1,
0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf62, 0xf0b, 0xf58,
0xf72, 0xf0b, 0xf53, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b,
0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf7a,
0xf4a, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf0b, 0xf4a, 0xf72, 0xf40, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf7a, 0xf4a, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf0b,
0xf4a, 0xf72, 0xf40, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf7a, 0xf4a, 0xf0b, 0xf63, 0xf7a, 0xf53,
0xf0b, 0xf4a, 0xf72, 0xf40, 0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74,
0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf51, 0xf56, 0xf74, 0xf66,
0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf71, 0xf7c, 0xf66, 0xf0b,
0xf4a, 0xfb2, 0xf7a, 0xf63, 0xf0b, 0xf63, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf51, 0xf56, 0xf74, 0xf66, 0xf0b, 0xf55, 0xfb1, 0xf7c,
0xf42, 0xf66, 0xf0b, 0xf68, 0xf71, 0xf7c, 0xf66, 0xf0b, 0xf4a, 0xfb2, 0xf7a, 0xf63,
0xf0b, 0xf63, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf51, 0xf56, 0xf74, 0xf66, 0xf0b,
0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf71, 0xf7c, 0xf66, 0xf0b, 0xf4a,
0xfb2, 0xf7a, 0xf63, 0xf0b, 0xf63, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf49, 0xf72, 0xf53,
0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf56, 0xf74, 0xf66, 0xf0b, 0xf53, 0xf74, 0xf56, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42,
0xf66, 0xf0b, 0xf68, 0xf71, 0xf7c, 0xf66, 0xf0b, 0xf4a, 0xfb2, 0xf7a, 0xf63, 0xf0b,
0xf63, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf51,
0xf56, 0xf74, 0xf66, 0xf0b, 0xf53, 0xf74, 0xf56, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42,
0xf66, 0xf0b, 0xf68, 0xf71, 0xf7c, 0xf66, 0xf0b, 0xf4a, 0xfb2, 0xf7a, 0xf63, 0xf0b,
0xf63, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf51, 0xf56, 0xf74, 0xf66, 0xf0b, 0xf53,
0xf74, 0xf56, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf71, 0xf7c,
0xf66, 0xf0b, 0xf4a, 0xfb2, 0xf7a, 0xf63, 0xf0b, 0xf63, 0xf72, 0xf0b, 0xf61, 0xf0b,
0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf64, 0xf62, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b,
0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf71, 0xf7c, 0xf66, 0xf0b, 0xf4a,
0xfb2, 0xf7a, 0xf63, 0xf0b, 0xf63, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf64, 0xf62, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b,
0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf71, 0xf7c, 0xf66, 0xf0b, 0xf4a,
0xfb2, 0xf7a, 0xf63, 0xf0b, 0xf63, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf5a, 0xf51, 0xf0b,
0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf64, 0xf62,
0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66,
0xf0b, 0xf68, 0xf71, 0xf7c, 0xf66, 0xf0b, 0xf4a, 0xfb2, 0xf7a, 0xf63, 0xf0b, 0xf63,
0xf72, 0xf0b, 0xf61, 0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf0b, 0xf5b, 0xf62, 0xf0b,
0xf56, 0xfb7, 0xf60, 0xf72, 0xf0b, 0xf47, 0xf71, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf68, 0xf0b, 0xf5b, 0xf62, 0xf0b, 0xf56, 0xfb7, 0xf60, 0xf72,
0xf0b, 0xf47, 0xf71, 0xf53, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf0b, 0xf5b, 0xf62, 0xf0b, 0xf56,
0xfb7, 0xf60, 0xf72, 0xf0b, 0xf47, 0xf71, 0xf53, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62,
0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68,
0xf7a, 0xf0b, 0xf5b, 0xf7c, 0xf62, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf68, 0xf7a, 0xf0b, 0xf5b, 0xf7c, 0xf62, 0xf66, 0xf0b, 0xf5a, 0xf51, 0xf0b,
0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf7a,
0xf0b, 0xf5b, 0xf7c, 0xf62, 0xf66, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51,
0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xf44, 0xf92,
0xf0b, 0xf63, 0xf0b, 0xf51, 0xf7a, 0xf64, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf56, 0xf44, 0xf92, 0xf0b, 0xf63, 0xf0b, 0xf51, 0xf7a, 0xf64, 0xf0b, 0xf5a,
0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf56, 0xf44, 0xf92, 0xf0b, 0xf63, 0xf0b, 0xf51, 0xf7a, 0xf64, 0xf0b, 0xf56, 0xfb1,
0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf60, 0xf56, 0xfb2, 0xf74, 0xf42, 0xf0b, 0xf61, 0xf74, 0xf63, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb7, 0xf7c, 0xf0b, 0xf63, 0xf72, 0xf0b,
0xf56, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56,
0xfb2, 0xf0b, 0xf5b, 0xf72, 0xf0b, 0xf63, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb2, 0xf0b, 0xf5b, 0xf72, 0xf0b, 0xf63, 0xf72,
0xf0b, 0xf61, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb2, 0xf0b, 0xf5b, 0xf72, 0xf0b, 0xf63, 0xf72,
0xf0b, 0xf61, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf40, 0xf7a, 0xf54, 0xf0b, 0xf56, 0xf71,
0xf4c, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf40, 0xf7a, 0xf54, 0xf0b,
0xf56, 0xf71, 0xf4c, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf40, 0xf7a, 0xf54, 0xf0b, 0xf56, 0xf71, 0xf4c,
0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf45, 0xf72, 0xf0b, 0xf63, 0xf72, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf45, 0xf72, 0xf0b, 0xf63, 0xf72, 0xf0b, 0xf5a, 0xf51,
0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf45,
0xf72, 0xf0b, 0xf63, 0xf72, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74,
0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf62, 0xf92, 0xfb1, 0xf0b,
0xf53, 0xf42, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf62, 0xf92, 0xfb1,
0xf0b, 0xf53, 0xf42, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf62, 0xf92, 0xfb1, 0xf0b, 0xf53, 0xf42, 0xf0b,
0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf41, 0xf72, 0xf0b, 0xf62, 0xf72, 0xf66, 0xf9f, 0xf0b, 0xf58,
0xf7a, 0xf66, 0xf0b, 0xf58, 0xf5a, 0xf7c, 0xf0b, 0xf42, 0xfb3, 0xf72, 0xf44, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf40, 0xf7c, 0xf0b, 0xf63, 0xf7c, 0xf58,
0xf0b, 0xf56, 0xfb7, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf40, 0xf7c, 0xf0b, 0xf63, 0xf7c, 0xf58, 0xf0b, 0xf56, 0xfb7, 0xf72, 0xf0b,
0xf61, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf40, 0xf7c, 0xf0b, 0xf63, 0xf7c, 0xf58, 0xf0b, 0xf56, 0xfb7,
0xf72, 0xf0b, 0xf61, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf40, 0xf72, 0xf60, 0xf74, 0xf0b,
0xf56, 0xfb7, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf40, 0xf72, 0xf60,
0xf74, 0xf0b, 0xf56, 0xfb7, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf40, 0xf72, 0xf60, 0xf74, 0xf0b, 0xf56,
0xfb7, 0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf72, 0xf66, 0xf72, 0xf0b, 0xf4a, 0xf62,
0xf0b, 0xf68, 0xf71, 0xf60, 0xf72, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf72, 0xf66, 0xf72, 0xf0b, 0xf4a, 0xf62,
0xf0b, 0xf68, 0xf71, 0xf60, 0xf72, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf5a,
0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf68, 0xf72, 0xf66, 0xf72, 0xf0b, 0xf4a, 0xf62, 0xf0b, 0xf68, 0xf71, 0xf60, 0xf72,
0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51,
0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf7a, 0xf0b,
0xf40, 0xf74, 0xf0b, 0xf4c, 0xf7d, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf51, 0xf56, 0xf74, 0xf66, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf61,
0xf74, 0xf0b, 0xf62, 0xf7c, 0xf0b, 0xf54, 0xf7a, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf51, 0xf56, 0xf74, 0xf66, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42,
0xf66, 0xf0b, 0xf61, 0xf74, 0xf0b, 0xf62, 0xf7c, 0xf0b, 0xf54, 0xf7a, 0xf53, 0xf0b,
0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf51, 0xf56, 0xf74, 0xf66, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b,
0xf61, 0xf74, 0xf0b, 0xf62, 0xf7c, 0xf0b, 0xf54, 0xf7a, 0xf53, 0xf0b, 0xf56, 0xfb1,
0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf64, 0xf62, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf61, 0xf74,
0xf0b, 0xf62, 0xf7c, 0xf0b, 0xf54, 0xf7a, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf64, 0xf62, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf61,
0xf74, 0xf0b, 0xf62, 0xf7c, 0xf0b, 0xf54, 0xf7a, 0xf53, 0xf0b, 0xf5a, 0xf51, 0xf0b,
0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf64, 0xf62,
0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf61, 0xf74, 0xf0b, 0xf62, 0xf7c,
0xf0b, 0xf54, 0xf7a, 0xf53, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74,
0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf74, 0xf56, 0xf0b,
0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf61, 0xf74, 0xf0b, 0xf62, 0xf7c, 0xf0b,
0xf54, 0xf7a, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf74,
0xf56, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf61, 0xf74, 0xf0b, 0xf62,
0xf7c, 0xf0b, 0xf54, 0xf7a, 0xf53, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf74, 0xf56, 0xf0b, 0xf55,
0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf61, 0xf74, 0xf0b, 0xf62, 0xf7c, 0xf0b, 0xf54,
0xf7a, 0xf53, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf55, 0xf63, 0xf40, 0xf0b, 0xf63, 0xf7a,
0xf53, 0xf4c, 0xf0b, 0xf68, 0xf71, 0xf60, 0xf72, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c,
0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf55, 0xf63, 0xf40, 0xf0b,
0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf68, 0xf71, 0xf60, 0xf72, 0xf0b, 0xf63, 0xf7a,
0xf53, 0xf4c, 0xf66, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf55, 0xf63, 0xf40, 0xf0b, 0xf63, 0xf7a, 0xf53,
0xf4c, 0xf0b, 0xf68, 0xf71, 0xf60, 0xf72, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf66,
0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf55, 0xf72, 0xf62, 0xf7a, 0xf53, 0xf45, 0xf0b, 0xf42,
0xf72, 0xf0b, 0xf61, 0xf0b, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf42, 0xf0b, 0xf63, 0xf0b, 0xf54, 0xf0b, 0xf42, 0xf7c, 0xf66, 0xf72, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf47, 0xf7d, 0xf0b, 0xf47, 0xf71, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf47, 0xf7d, 0xf0b, 0xf47, 0xf71, 0xf0b, 0xf5a,
0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf47, 0xf7d, 0xf0b, 0xf47, 0xf71, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51,
0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf42, 0xf72, 0xf62,
0xf72, 0xf53, 0xf0b, 0xf5d, 0xf72, 0xf46, 0xf0b, 0xf63, 0xf74, 0xf0b, 0xf61, 0xf7c,
0xf51, 0xf0b, 0xf54, 0xf60, 0xf72, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf64, 0xf62, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf42, 0xf72, 0xf62,
0xf72, 0xf53, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf64, 0xf62, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf42,
0xf72, 0xf62, 0xf72, 0xf53, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf5a, 0xf51,
0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf64,
0xf62, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf42, 0xf72, 0xf62, 0xf72,
0xf53, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b,
0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf74,
0xf56, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf42, 0xf72, 0xf62, 0xf72,
0xf53, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf53, 0xf74, 0xf56, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf42,
0xf72, 0xf62, 0xf72, 0xf53, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf5a, 0xf51,
0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53,
0xf74, 0xf56, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf42, 0xf72, 0xf62,
0xf72, 0xf53, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62,
0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf42,
0xf71, 0xf63, 0xf55, 0xf72, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf42,
0xf74, 0xf0b, 0xf61, 0xf0b, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf67, 0xf0b, 0xf5d, 0xf60, 0xf72, 0xf0b, 0x2d, 0xf68, 0xf7a, 0xf0b, 0xf63, 0xf72,
0xf60, 0xf74, 0xf0b, 0xf64, 0xf7a, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf67, 0xf0b, 0xf5d, 0xf60, 0xf72, 0xf0b, 0x2d, 0xf68, 0xf7a, 0xf0b, 0xf63,
0xf72, 0xf60, 0xf74, 0xf0b, 0xf64, 0xf7a, 0xf53, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63,
0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf67, 0xf0b, 0xf5d,
0xf60, 0xf72, 0xf0b, 0x2d, 0xf68, 0xf7a, 0xf0b, 0xf63, 0xf72, 0xf60, 0xf74, 0xf0b,
0xf64, 0xf7a, 0xf53, 0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf62, 0xf92, 0xfb1, 0xf0b, 0xf42,
0xf62, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf62, 0xf92, 0xfb1, 0xf0b,
0xf42, 0xf62, 0xf0b, 0xf42, 0xfb1, 0xf72, 0xf0b, 0xf62, 0xf92, 0xfb1, 0xf0b, 0xf58,
0xf5a, 0xf7c, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf72, 0xf53,
0xf0b, 0xf4c, 0xf7c, 0xf0b, 0xf45, 0xf71, 0xf60, 0xf72, 0xf0b, 0xf53, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf51, 0xf56, 0xf74, 0xf66, 0xf0b, 0xf55, 0xfb1,
0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf72, 0xf53, 0xf0b, 0xf4c, 0xf7c, 0xf0b, 0xf53,
0xf7a, 0xf0b, 0xf64, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf64, 0xf62, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf72,
0xf53, 0xf0b, 0xf4c, 0xf7c, 0xf0b, 0xf53, 0xf7a, 0xf0b, 0xf64, 0xf72, 0xf0b, 0xf61,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf74, 0xf56, 0xf0b, 0xf55,
0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf72, 0xf53, 0xf0b, 0xf4c, 0xf7c, 0xf0b,
0xf53, 0xf7a, 0xf0b, 0xf64, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf68, 0xf72, 0xf0b, 0xf62, 0xf71, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf68, 0xf72, 0xf0b, 0xf62, 0xf71, 0xf53, 0xf0b, 0xf5a, 0xf51,
0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68,
0xf72, 0xf0b, 0xf62, 0xf71, 0xf53, 0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2,
0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf62, 0xf0b,
0xf40, 0xf74, 0xf59, 0xf72, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68,
0xf62, 0xf0b, 0xf40, 0xf74, 0xf59, 0xf72, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1,
0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf62, 0xf0b, 0xf40,
0xf74, 0xf59, 0xf72, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf72, 0xf66, 0xf0b, 0xf62,
0xf7a, 0xf63, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf72, 0xf66,
0xf0b, 0xf62, 0xf7a, 0xf63, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf72, 0xf66, 0xf0b, 0xf62, 0xf7a,
0xf63, 0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf47, 0xf0b, 0xf54, 0xf71, 0xf53, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf47, 0xf0b, 0xf54, 0xf71, 0xf53, 0xf0b, 0xf5a,
0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf47, 0xf0b, 0xf54, 0xf71, 0xf53, 0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2,
0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf40, 0xf7c, 0xf0b,
0xf62, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf40,
0xf7c, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1,
0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf40, 0xf7c, 0xf0b, 0xf62,
0xf72, 0xf0b, 0xf61, 0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf40, 0xf62, 0xf66, 0xf72, 0xf0b,
0xf53, 0xf7c, 0xf0b, 0xf61, 0xf62, 0xf66, 0xf40, 0xf72, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf40, 0xf62, 0xf66, 0xf72, 0xf0b, 0xf53, 0xf7c, 0xf0b, 0xf61,
0xf62, 0xf66, 0xf40, 0xf72, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf40, 0xf62, 0xf66, 0xf72, 0xf0b, 0xf53,
0xf7c, 0xf0b, 0xf61, 0xf62, 0xf66, 0xf40, 0xf72, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62,
0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf58,
0xf0b, 0xf42, 0xfb7, 0xf0b, 0xf51, 0xf71, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf58, 0xf0b, 0xf42, 0xfb7, 0xf0b, 0xf51, 0xf71, 0xf53, 0xf0b, 0xf5a,
0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf58, 0xf0b, 0xf42, 0xfb7, 0xf0b, 0xf51, 0xf71, 0xf53, 0xf0b, 0xf56, 0xfb1, 0xf71,
0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf58, 0xf71, 0xf63, 0xf0b, 0xf51, 0xf72, 0xf56, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf58, 0xf7c, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf64, 0xf71, 0xf66,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf58, 0xf7c, 0xf0b, 0xf62, 0xf72,
0xf0b, 0xf64, 0xf71, 0xf66, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf58, 0xf7c, 0xf0b, 0xf62, 0xf72, 0xf0b,
0xf64, 0xf71, 0xf66, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf58, 0xf7d, 0xf66, 0xf0b, 0xf40,
0xf7c, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf58, 0xf7d, 0xf66, 0xf0b,
0xf40, 0xf7c, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf58, 0xf7d, 0xf66, 0xf0b, 0xf40, 0xf7c, 0xf0b, 0xf56,
0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf53, 0xf7a, 0xf0b, 0xf54, 0xf71, 0xf63, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf53, 0xf72, 0xf60, 0xf74, 0xf0b, 0xf5b, 0xf72, 0xf0b, 0xf63,
0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf72,
0xf60, 0xf74, 0xf0b, 0xf5b, 0xf72, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf5a,
0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf53, 0xf72, 0xf60, 0xf74, 0xf0b, 0xf5b, 0xf72, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c,
0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf72, 0xf60, 0xf74, 0xf0b, 0xf55, 0xf60, 0xf74,
0xf53, 0xf4c, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf53, 0xf72, 0xf60, 0xf74, 0xf0b, 0xf55, 0xf60, 0xf74, 0xf53, 0xf4c,
0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf72, 0xf60, 0xf74, 0xf0b,
0xf55, 0xf60, 0xf74, 0xf53, 0xf4c, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf49,
0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf55, 0xf62, 0xf0b, 0xf53, 0xf7a, 0xf53, 0xf0b, 0xf4c, 0xf7c, 0xf0b,
0x20, 0xf4c, 0xf72, 0xf0b, 0x20, 0xf53, 0xf7c, 0xf0b, 0xf62, 0xf71, 0xf7c, 0xf53,
0xf0b, 0xf67, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf55, 0xf62, 0xf0b,
0xf53, 0xf7a, 0xf53, 0xf0b, 0xf4c, 0xf7c, 0xf0b, 0x20, 0xf4c, 0xf72, 0xf0b, 0x20,
0xf53, 0xf7c, 0xf0b, 0xf62, 0xf71, 0xf7c, 0xf53, 0xf0b, 0xf67, 0xf0b, 0xf5a, 0xf51,
0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf55,
0xf62, 0xf0b, 0xf53, 0xf7a, 0xf53, 0xf0b, 0xf4c, 0xf7c, 0xf0b, 0x20, 0xf4c, 0xf72,
0xf0b, 0x20, 0xf53, 0xf7c, 0xf0b, 0xf62, 0xf71, 0xf7c, 0xf53, 0xf0b, 0xf67, 0xf0b,
0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf53, 0xf7c, 0xf0b, 0xf56, 0xf7c, 0xf0b, 0xf66, 0xf72, 0xf0b,
0xf56, 0xf72, 0xf62, 0xf66, 0xf40, 0xf72, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf53, 0xf7c, 0xf0b, 0xf56, 0xf7c, 0xf0b, 0xf66, 0xf72, 0xf0b, 0xf56, 0xf72,
0xf62, 0xf66, 0xf40, 0xf72, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf7c, 0xf0b, 0xf56, 0xf7c, 0xf0b,
0xf66, 0xf72, 0xf0b, 0xf56, 0xf72, 0xf62, 0xf66, 0xf40, 0xf72, 0xf0b, 0xf56, 0xfb1,
0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf68, 0xf7c, 0xf58, 0xf66, 0xf40, 0xf72, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf68, 0xf7c, 0xf58, 0xf66, 0xf40, 0xf72, 0xf0b, 0xf5a, 0xf51, 0xf0b,
0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf7c,
0xf58, 0xf66, 0xf40, 0xf72, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74,
0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf54, 0xf0b, 0xf40, 0xf72,
0xf0b, 0xf66, 0xf84, 0xf4f, 0xf71, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf54, 0xf0b, 0xf40, 0xf72, 0xf0b, 0xf66, 0xf84, 0xf4f, 0xf71, 0xf53, 0xf0b,
0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf54, 0xf0b, 0xf40, 0xf72, 0xf0b, 0xf66, 0xf84, 0xf4f, 0xf71, 0xf53, 0xf0b,
0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf54, 0xf0b, 0xf62, 0xf0b, 0xf42, 0xf74, 0xf0b, 0xf5d, 0xf60,
0xf72, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf54, 0xf0b, 0xf62, 0xf0b,
0xf42, 0xf74, 0xf0b, 0xf5d, 0xf60, 0xf72, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1,
0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf54, 0xf0b, 0xf62, 0xf0b,
0xf42, 0xf74, 0xf0b, 0xf5d, 0xf60, 0xf72, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b,
0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf54, 0xf0b,
0xf62, 0xf74, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf54, 0xf0b, 0xf62,
0xf74, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf54, 0xf0b, 0xf62, 0xf74, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62,
0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf54,
0xf60, 0xf72, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf51, 0xf44, 0xf0b, 0xf58, 0xf72, 0xf0b,
0xf40, 0xf72, 0xf0b, 0xf63, 0xf71, 0xf7c, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf54, 0xf60, 0xf72, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf51, 0xf44, 0xf0b,
0xf58, 0xf72, 0xf0b, 0xf40, 0xf72, 0xf0b, 0xf63, 0xf71, 0xf7c, 0xf53, 0xf0b, 0xf5a,
0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf54, 0xf60, 0xf72, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf51, 0xf44, 0xf0b, 0xf58, 0xf72,
0xf0b, 0xf40, 0xf72, 0xf0b, 0xf63, 0xf71, 0xf7c, 0xf53, 0xf0b, 0xf49, 0xf72, 0xf53,
0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf62, 0xf72, 0xf0b, 0xf61, 0xf74, 0xf0b, 0xf53, 0xf72, 0xf0b, 0xf61, 0xf71, 0xf53,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf66, 0xf0b, 0xf41, 0xf0b, 0xf63,
0xf72, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf66, 0xf0b, 0xf41,
0xf0b, 0xf63, 0xf72, 0xf53, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf66, 0xf0b, 0xf41, 0xf0b, 0xf63, 0xf72,
0xf53, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf66, 0xf7a, 0xf0b, 0xf64, 0xf71, 0xf63, 0xf66,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf66, 0xf74, 0xf0b, 0xf62, 0xf72,
0xf0b, 0xf53, 0xf71, 0xf58, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf61,
0xf74, 0xf0b, 0xf62, 0xf74, 0xf0b, 0xf42, 0xf74, 0xf0b, 0xf5d, 0xf71, 0xf60, 0xf72,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf61, 0xf74, 0xf0b, 0xf62, 0xf74,
0xf0b, 0xf42, 0xf74, 0xf0b, 0xf5d, 0xf71, 0xf60, 0xf72, 0xf0b, 0xf5a, 0xf51, 0xf0b,
0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf61, 0xf74,
0xf0b, 0xf62, 0xf74, 0xf0b, 0xf42, 0xf74, 0xf0b, 0xf5d, 0xf71, 0xf60, 0xf72, 0xf0b,
0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf56, 0xf7a, 0xf0b, 0xf53, 0xf72, 0xf0b, 0xf5b, 0xf74, 0xf0b,
0xf5d, 0xf7a, 0xf0b, 0xf63, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56,
0xf0b, 0xf63, 0xf0b, 0xf4c, 0xf72, 0xf0b, 0xf56, 0xf7c, 0xf66, 0xf72, 0xf0b, 0xf4f,
0xf7c, 0xf40, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xf0b, 0xf63,
0xf0b, 0xf4c, 0xf72, 0xf0b, 0xf56, 0xf7c, 0xf66, 0xf72, 0xf0b, 0xf4f, 0xf7c, 0xf40,
0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf56, 0xf0b, 0xf63, 0xf0b, 0xf4c, 0xf72, 0xf0b, 0xf56, 0xf7c, 0xf66,
0xf72, 0xf0b, 0xf4f, 0xf7c, 0xf40, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51,
0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xf71, 0xf7c,
0xf63, 0xf0b, 0xf42, 0xf7c, 0xf0b, 0xf42, 0xf72, 0xf62, 0xf7a, 0xf4c, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xf71, 0xf7c, 0xf63, 0xf0b, 0xf42, 0xf7c,
0xf0b, 0xf42, 0xf72, 0xf62, 0xf7a, 0xf4c, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1,
0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xf71, 0xf7c, 0xf63,
0xf0b, 0xf42, 0xf7c, 0xf0b, 0xf42, 0xf72, 0xf62, 0xf7a, 0xf4c, 0xf0b, 0xf56, 0xfb1,
0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf61, 0xf0b, 0xf40, 0xf74, 0xf59, 0xf72, 0xf40, 0xf72, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf61, 0xf0b, 0xf40, 0xf74, 0xf59, 0xf72, 0xf40, 0xf72,
0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf61, 0xf0b, 0xf40, 0xf74, 0xf59, 0xf72, 0xf40, 0xf72, 0xf0b, 0xf56,
0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf61, 0xf7a, 0xf0b, 0xf40, 0xf0b, 0xf4f, 0xf7a, 0xf0b, 0xf62, 0xf72,
0xf53, 0xf0b, 0xf56, 0xf62, 0xf42, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf61, 0xf7a, 0xf0b, 0xf40, 0xf0b, 0xf4f, 0xf7a, 0xf0b, 0xf62, 0xf72, 0xf53, 0xf0b,
0xf56, 0xf62, 0xf42, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf61, 0xf7a, 0xf0b, 0xf40, 0xf0b, 0xf4f, 0xf7a,
0xf0b, 0xf62, 0xf72, 0xf53, 0xf0b, 0xf56, 0xf62, 0xf42, 0xf0b, 0xf56, 0xfb1, 0xf71,
0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xd801, 0xdc0a, 0xd801, 0xdc4a, 0xd801, 0xdc30, 0xd801, 0xdc45, 0xd801, 0xdc3f, 0xd801, 0xdc32,
0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc0a, 0xd801, 0xdc4a, 0xd801,
0xdc30, 0xd801, 0xdc45, 0xd801, 0xdc3f, 0xd801, 0xdc32, 0x20, 0xd801, 0xdc1d, 0xd801, 0xdc3b,
0xd801, 0xdc30, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc3c,
0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc0a, 0xd801, 0xdc4a, 0xd801,
0xdc30, 0xd801, 0xdc45, 0xd801, 0xdc3f, 0xd801, 0xdc32, 0x20, 0xd801, 0xdc14, 0xd801, 0xdc29,
0xd801, 0xdc4a, 0xd801, 0xdc34, 0xd801, 0xdc3b, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801,
0xdc4b, 0xd801, 0xdc1d, 0xd801, 0xdc2f, 0xd801, 0xdc4c, 0xd801, 0xdc3b, 0xd801, 0xdc49, 0xd801,
0xdc32, 0xd801, 0xdc4a, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc1d,
0xd801, 0xdc2f, 0xd801, 0xdc4c, 0xd801, 0xdc3b, 0xd801, 0xdc49, 0xd801, 0xdc32, 0xd801, 0xdc4a,
0x20, 0xd801, 0xdc1d, 0xd801, 0xdc3b, 0xd801, 0xdc30, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801,
0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc3c, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b,
0xd801, 0xdc1d, 0xd801, 0xdc2f, 0xd801, 0xdc4c, 0xd801, 0xdc3b, 0xd801, 0xdc49, 0xd801, 0xdc32,
0xd801, 0xdc4a, 0x20, 0xd801, 0xdc14, 0xd801, 0xdc29, 0xd801, 0xdc4a, 0xd801, 0xdc34, 0xd801,
0xdc3b, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc00, 0xd801, 0xdc45,
0xd801, 0xdc3b, 0xd801, 0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc4c, 0x20, 0xd801, 0xdc13, 0xd801,
0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc00, 0xd801, 0xdc45, 0xd801, 0xdc3b, 0xd801, 0xdc32, 0xd801,
0xdc49, 0xd801, 0xdc4c, 0x20, 0xd801, 0xdc1d, 0xd801, 0xdc3b, 0xd801, 0xdc30, 0xd801, 0xdc4c,
0xd801, 0xdc3c, 0xd801, 0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc3c, 0x20, 0xd801, 0xdc13, 0xd801,
0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc00, 0xd801, 0xdc45, 0xd801, 0xdc3b, 0xd801, 0xdc32, 0xd801,
0xdc49, 0xd801, 0xdc4c, 0x20, 0xd801, 0xdc14, 0xd801, 0xdc29, 0xd801, 0xdc4a, 0xd801, 0xdc34,
0xd801, 0xdc3b, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc23, 0xd801,
0xdc35, 0xd801, 0xdc4c, 0xd801, 0xdc3b, 0xd801, 0xdc32, 0xd801, 0xdc4c, 0x20, 0xd801, 0xdc13,
0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc23, 0xd801, 0xdc35, 0xd801, 0xdc4c, 0xd801, 0xdc3b,
0xd801, 0xdc32, 0xd801, 0xdc4c, 0x20, 0xd801, 0xdc1d, 0xd801, 0xdc3b, 0xd801, 0xdc30, 0xd801,
0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc3c, 0x20, 0xd801, 0xdc13,
0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc23, 0xd801, 0xdc35, 0xd801, 0xdc4c, 0xd801, 0xdc3b,
0xd801, 0xdc32, 0xd801, 0xdc4c, 0x20, 0xd801, 0xdc14, 0xd801, 0xdc29, 0xd801, 0xdc4a, 0xd801,
0xdc34, 0xd801, 0xdc3b, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc11,
0xd801, 0xdc32, 0xd801, 0xdc45, 0xd801, 0xdc2e, 0xd801, 0xdc41, 0xd801, 0xdc2e, 0xd801, 0xdc3f,
0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc11, 0xd801, 0xdc32, 0xd801,
0xdc45, 0xd801, 0xdc2e, 0xd801, 0xdc41, 0xd801, 0xdc2e, 0xd801, 0xdc3f, 0x20, 0xd801, 0xdc1d,
0xd801, 0xdc3b, 0xd801, 0xdc30, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc32, 0xd801, 0xdc49,
0xd801, 0xdc3c, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc11, 0xd801,
0xdc32, 0xd801, 0xdc45, 0xd801, 0xdc2e, 0xd801, 0xdc41, 0xd801, 0xdc2e, 0xd801, 0xdc3f, 0x20,
0xd801, 0xdc14, 0xd801, 0xdc29, 0xd801, 0xdc4a, 0xd801, 0xdc34, 0xd801, 0xdc3b, 0x20, 0xd801,
0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc08, 0xd801, 0xdc3b, 0xd801, 0xdc4a, 0xd801,
0xdc30, 0xd801, 0xdc4c, 0xd801, 0xdc3b, 0xd801, 0xdc2e, 0xd801, 0xdc3f, 0x20, 0xd801, 0xdc13,
0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc08, 0xd801, 0xdc3b, 0xd801, 0xdc4a, 0xd801, 0xdc30,
0xd801, 0xdc4c, 0xd801, 0xdc3b, 0xd801, 0xdc2e, 0xd801, 0xdc3f, 0x20, 0xd801, 0xdc1d, 0xd801,
0xdc3b, 0xd801, 0xdc30, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc32, 0xd801, 0xdc49, 0xd801,
0xdc3c, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc08, 0xd801, 0xdc3b,
0xd801, 0xdc4a, 0xd801, 0xdc30, 0xd801, 0xdc4c, 0xd801, 0xdc3b, 0xd801, 0xdc2e, 0xd801, 0xdc3f,
0x20, 0xd801, 0xdc14, 0xd801, 0xdc29, 0xd801, 0xdc4a, 0xd801, 0xdc34, 0xd801, 0xdc3b, 0x20,
0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc10, 0xd801, 0xdc31, 0xd801, 0xdc4d,
0x20, 0xd801, 0xdc17, 0xd801, 0xdc31, 0xd801, 0xdc4d, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34,
0xd801, 0xdc4b, 0xd801, 0xdc10, 0xd801, 0xdc31, 0xd801, 0xdc4d, 0x20, 0xd801, 0xdc17, 0xd801,
0xdc31, 0xd801, 0xdc4d, 0x20, 0xd801, 0xdc1d, 0xd801, 0xdc3b, 0xd801, 0xdc30, 0xd801, 0xdc4c,
0xd801, 0xdc3c, 0xd801, 0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc3c, 0x20, 0xd801, 0xdc13, 0xd801,
0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc10, 0xd801, 0xdc31, 0xd801, 0xdc4d, 0x20, 0xd801, 0xdc17,
0xd801, 0xdc31, 0xd801, 0xdc4d, 0x20, 0xd801, 0xdc14, 0xd801, 0xdc29, 0xd801, 0xdc4a, 0xd801,
0xdc34, 0xd801, 0xdc3b, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc24,
0xd801, 0xdc2d, 0xd801, 0xdc41, 0xd801, 0xdc32, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc4a,
0xd801, 0xdc32, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801,
0xdc4b, 0xd801, 0xdc24, 0xd801, 0xdc2d, 0xd801, 0xdc41, 0xd801, 0xdc32, 0xd801, 0xdc4c, 0xd801,
0xdc3c, 0xd801, 0xdc4a, 0xd801, 0xdc32, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0x20, 0xd801, 0xdc1d,
0xd801, 0xdc3b, 0xd801, 0xdc30, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc32, 0xd801, 0xdc49,
0xd801, 0xdc3c, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc24, 0xd801,
0xdc2d, 0xd801, 0xdc41, 0xd801, 0xdc32, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc4a, 0xd801,
0xdc32, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0x20, 0xd801, 0xdc14, 0xd801, 0xdc29, 0xd801, 0xdc4a,
0xd801, 0xdc34, 0xd801, 0xdc3b, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0x53,
0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d,
0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x53, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53, 0x74,
0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69,
0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x68, 0x69, 0x6e,
0x61, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6e,
0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41,
0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67,
0x68, 0x74, 0x20, 0x53, 0x61, 0x76, 0x69, 0x6e, 0x67, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x44, 0x61,
0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x53, 0x61, 0x76, 0x69, 0x6e,
0x67, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72,
0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x53,
0x61, 0x76, 0x69, 0x6e, 0x67, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x46, 0x72,
0x65, 0x6e, 0x63, 0x68, 0x20, 0x53, 0x6f, 0x75, 0x74, 0x68, 0x65, 0x72,
0x6e, 0x20, 0x61, 0x6e, 0x64, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x63,
0x74, 0x69, 0x63, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x49, 0x72, 0x61, 0x6e,
0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x53, 0x61,
0x76, 0x69, 0x6e, 0x67, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53, 0x61, 0x69,
0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65, 0x74,
0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68,
0x74, 0x20, 0x53, 0x61, 0x76, 0x69, 0x6e, 0x67, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x61, 0x6e, 0x64, 0x20, 0x46,
0x75, 0x74, 0x75, 0x6e, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4b, 0x68,
0x6f, 0x76, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4b, 0x68, 0x6f, 0x76,
0x64, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x4b, 0x68, 0x6f, 0x76, 0x64, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x410, 0x444, 0x433, 0x430,
0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a,
0x430, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x43d, 0x44c, 0x20, 0x448, 0x43a,
0x430, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x43d, 0x44c, 0x20, 0x441, 0x432,
0x430, 0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x410, 0x43b, 0x44f,
0x441, 0x43a, 0x430, 0x43d, 0x44c, 0x20, 0x43a, 0x438, 0x437, 0x44d, 0x43d, 0x44c,
0x20, 0x448, 0x43a, 0x430, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43e, 0x43d,
0x44c, 0x20, 0x448, 0x43a, 0x430, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43e,
0x43d, 0x44c, 0x20, 0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448,
0x43a, 0x430, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43e, 0x43d, 0x44c, 0x20,
0x43a, 0x438, 0x437, 0x44d, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x411, 0x430,
0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x435, 0x43d, 0x44c, 0x20, 0x448,
0x43a, 0x430, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x435,
0x43d, 0x44c, 0x20, 0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448,
0x43a, 0x430, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x435,
0x43d, 0x44c, 0x20, 0x43a, 0x438, 0x437, 0x44d, 0x43d, 0x44c, 0x20, 0x448, 0x43a,
0x430, 0x411, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x44f, 0x43d, 0x44c, 0x20, 0x448,
0x43a, 0x430, 0x411, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x44f, 0x43d, 0x44c, 0x20,
0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x411,
0x430, 0x437, 0x438, 0x43b, 0x438, 0x44f, 0x43d, 0x44c, 0x20, 0x43a, 0x438, 0x437,
0x44d, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x427, 0x438, 0x43b, 0x438, 0x43d,
0x44c, 0x20, 0x448, 0x43a, 0x430, 0x427, 0x438, 0x43b, 0x438, 0x43d, 0x44c, 0x20,
0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x427,
0x438, 0x43b, 0x438, 0x43d, 0x44c, 0x20, 0x43a, 0x438, 0x437, 0x44d, 0x43d, 0x44c,
0x20, 0x448, 0x43a, 0x430, 0x41a, 0x438, 0x442, 0x430, 0x435, 0x43d, 0x44c, 0x20,
0x448, 0x43a, 0x430, 0x41a, 0x438, 0x442, 0x430, 0x435, 0x43d, 0x44c, 0x20, 0x441,
0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41a, 0x438,
0x442, 0x430, 0x435, 0x43d, 0x44c, 0x20, 0x43a, 0x438, 0x437, 0x44d, 0x43d, 0x44c,
0x20, 0x448, 0x43a, 0x430, 0x41a, 0x443, 0x431, 0x430, 0x43d, 0x44c, 0x20, 0x448,
0x43a, 0x430, 0x41a, 0x443, 0x431, 0x430, 0x43d, 0x44c, 0x20, 0x441, 0x432, 0x430,
0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41a, 0x443, 0x431, 0x430,
0x43d, 0x44c, 0x20, 0x43a, 0x438, 0x437, 0x44d, 0x43d, 0x44c, 0x20, 0x448, 0x43a,
0x430, 0x418, 0x43d, 0x434, 0x438, 0x44f, 0x43d, 0x44c, 0x20, 0x441, 0x432, 0x430,
0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x418, 0x440, 0x430, 0x43d,
0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x418, 0x440, 0x430, 0x43d, 0x43e,
0x43d, 0x44c, 0x20, 0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448,
0x43a, 0x430, 0x418, 0x440, 0x430, 0x43d, 0x43e, 0x43d, 0x44c, 0x20, 0x43a, 0x438,
0x437, 0x44d, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x42f, 0x43f, 0x43e, 0x43d,
0x438, 0x44f, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x42f, 0x43f, 0x43e, 0x43d,
0x438, 0x44f, 0x43d, 0x44c, 0x20, 0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c,
0x20, 0x448, 0x43a, 0x430, 0x42f, 0x43f, 0x43e, 0x43d, 0x438, 0x44f, 0x43d, 0x44c,
0x20, 0x43a, 0x438, 0x437, 0x44d, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41a,
0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x43e, 0x43d, 0x44c, 0x20,
0x447, 0x438, 0x43b, 0x438, 0x441, 0x435, 0x43c, 0x430, 0x43d, 0x44c, 0x20, 0x448,
0x43a, 0x430, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x43e,
0x43d, 0x44c, 0x20, 0x447, 0x438, 0x432, 0x430, 0x43b, 0x433, 0x43e, 0x43c, 0x430,
0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41a, 0x43e, 0x440, 0x435, 0x430, 0x43d,
0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41a, 0x43e, 0x440, 0x435, 0x430, 0x43d, 0x44c,
0x20, 0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430,
0x41a, 0x43e, 0x440, 0x435, 0x430, 0x43d, 0x44c, 0x20, 0x43a, 0x438, 0x437, 0x44d,
0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41c, 0x43e, 0x441, 0x43a, 0x43e, 0x432,
0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41c, 0x43e, 0x441, 0x43a, 0x43e,
0x432, 0x43e, 0x43d, 0x44c, 0x20, 0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c,
0x20, 0x448, 0x43a, 0x430, 0x41c, 0x43e, 0x441, 0x43a, 0x43e, 0x432, 0x43e, 0x43d,
0x44c, 0x20, 0x43a, 0x438, 0x437, 0x44d, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430,
0x41e, 0x434, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x43d,
0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41e, 0x434, 0x20, 0x417, 0x435, 0x43b, 0x430,
0x43d, 0x434, 0x438, 0x44f, 0x43d, 0x44c, 0x20, 0x441, 0x432, 0x430, 0x43b, 0x43e,
0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41e, 0x434, 0x20, 0x417, 0x435, 0x43b,
0x430, 0x43d, 0x434, 0x438, 0x44f, 0x43d, 0x44c, 0x20, 0x43a, 0x438, 0x437, 0x44d,
0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41e, 0x43c, 0x441, 0x43a, 0x43e, 0x435,
0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41e, 0x43c, 0x441, 0x43a, 0x43e, 0x435,
0x43d, 0x44c, 0x20, 0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448,
0x43a, 0x430, 0x41e, 0x43c, 0x441, 0x43a, 0x43e, 0x435, 0x43d, 0x44c, 0x20, 0x43a,
0x438, 0x437, 0x44d, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41f, 0x430, 0x440,
0x430, 0x433, 0x432, 0x430, 0x435, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41f,
0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x435, 0x43d, 0x44c, 0x20, 0x441, 0x432,
0x430, 0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41f, 0x430, 0x440,
0x430, 0x433, 0x432, 0x430, 0x435, 0x43d, 0x44c, 0x20, 0x43a, 0x438, 0x437, 0x44d,
0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438,
0x43d, 0x44d, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x421, 0x430, 0x445, 0x430,
0x43b, 0x438, 0x43d, 0x44d, 0x43d, 0x44c, 0x20, 0x441, 0x432, 0x430, 0x43b, 0x43e,
0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438,
0x43d, 0x44d, 0x43d, 0x44c, 0x20, 0x43a, 0x438, 0x437, 0x44d, 0x43d, 0x44c, 0x20,
0x448, 0x43a, 0x430, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x435, 0x43d, 0x44c,
0x20, 0x448, 0x43a, 0x430, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x435, 0x43d,
0x44c, 0x20, 0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a,
0x430, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x435, 0x43d, 0x44c, 0x20, 0x43a,
0x438, 0x437, 0x44d, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x423, 0x437, 0x431,
0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x43e, 0x43d, 0x44c, 0x20, 0x448,
0x43a, 0x430, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d,
0x43e, 0x43d, 0x44c, 0x20, 0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c, 0x20,
0x448, 0x43a, 0x430, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430,
0x43d, 0x43e, 0x43d, 0x44c, 0x20, 0x43a, 0x438, 0x437, 0x44d, 0x43d, 0x44c, 0x20,
0x448, 0x43a, 0x430, 0x61, 0x66, 0x67, 0x61, 0x6e, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x73, 0x75, 0x64, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x6b, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x6f, 0x6b, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x6f, 0x6b, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x61, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x69, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x69,
0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6d, 0x6f, 0x6e, 0x74, 0x61,
0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6d, 0x6f, 0x6e, 0x74,
0x61, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x73,
0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x70,
0x61, 0x63, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x64,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x70, 0x61, 0x63,
0x69, 0x66, 0x69, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x3a,
0x20, 0x41, 0x70, 0x69, 0x6f, 0x41, 0x70, 0x69, 0x6f, 0x20, 0x28, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x29, 0x41,
0x70, 0x69, 0x6f, 0x20, 0x28, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x29, 0x61, 0x72, 0x61, 0x62, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61, 0x72, 0x61, 0x62, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61, 0x72,
0x61, 0x62, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x6f, 0x6b, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x2d, 0x61,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x6f, 0x6b, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x2d, 0x61,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x6b, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x2d, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x61, 0x72, 0x6d, 0x65, 0x6e, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x61, 0x72, 0x6d, 0x65, 0x6e, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61, 0x72, 0x6d,
0x65, 0x6e, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x20, 0x61, 0x16d, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x20, 0x61, 0x16d, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x20, 0x61,
0x16d, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x6f, 0x6b, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x20,
0x61, 0x16d, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x6b, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x20, 0x61, 0x16d, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x6b, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x20, 0x61, 0x16d, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x20,
0x61, 0x16d, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x20, 0x61,
0x16d, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x20, 0x61, 0x16d, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x61, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x11d, 0x61, 0x6e,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61, 0x7a, 0x65, 0x72, 0x62,
0x61, 0x6a, 0x11d, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61, 0x7a, 0x65, 0x72, 0x62, 0x61,
0x6a, 0x11d, 0x61, 0x6e, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x3a,
0x20, 0x41, 0x63, 0x6f, 0x72, 0x6f, 0x6a, 0x41, 0x63, 0x6f, 0x72, 0x6f,
0x6a, 0x20, 0x28, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x29, 0x41, 0x63, 0x6f, 0x72, 0x6f, 0x6a, 0x20, 0x28, 0x73,
0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x29,
0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x15d, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x15d, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x15d, 0x61,
0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x62, 0x75, 0x74, 0x61, 0x6e, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x62, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x62, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x6a, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x62, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x6a, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x62, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x6a, 0x61, 0x20, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x62, 0x72, 0x75,
0x6e, 0x65, 0x6a, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x61,
0x62, 0x6f, 0x76, 0x65, 0x72, 0x64, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x6b, 0x61, 0x62, 0x6f, 0x76, 0x65, 0x72, 0x64, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x61,
0x62, 0x6f, 0x76, 0x65, 0x72, 0x64, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65,
0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x109, 0x61, 0x6d, 0x6f,
0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x109, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x109, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x109, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x61, 0x20,
0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x109, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x109,
0x69, 0x6c, 0x69, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x109, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x6f,
0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x109, 0x69,
0x6e, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x109, 0x69, 0x6e, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x109, 0x69, 0x6e, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x72, 0x69, 0x73, 0x74, 0x6e, 0x61,
0x73, 0x6b, 0x69, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x6b, 0x6f, 0x6b, 0x6f, 0x73, 0x69, 0x6e, 0x73, 0x75, 0x6c,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x6f, 0x6c, 0x6f, 0x6d,
0x62, 0x69, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x6f, 0x6c,
0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69,
0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x6b, 0x75, 0x6b, 0x69, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x75, 0x6b, 0x69, 0x6e, 0x73, 0x75,
0x6c, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x6b, 0x75, 0x6b, 0x69, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20,
0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x3a, 0x20, 0x4b, 0x75, 0x62, 0x6f, 0x4b,
0x75, 0x62, 0x6f, 0x20, 0x28, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x29, 0x4b, 0x75, 0x62, 0x6f, 0x20, 0x28, 0x73,
0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x29,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x3a, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x3a, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e,
0x74, 0x20, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x6f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x2d, 0x74, 0x69, 0x6d, 0x6f, 0x72, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x70, 0x61, 0x73, 0x6b, 0x69, 0x6e,
0x73, 0x75, 0x6c, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x70, 0x61,
0x73, 0x6b, 0x69, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x70, 0x61, 0x73, 0x6b,
0x69, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x65, 0x6b, 0x76, 0x61, 0x64,
0x6f, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x65, 0x16d, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x16d, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x16d, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x65, 0x16d, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x65, 0x16d, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x65, 0x16d, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65,
0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x65, 0x6b, 0x73, 0x74,
0x72, 0x65, 0x6d, 0x2d, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x65, 0x16d,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x6b,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x65, 0x16d, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x6b, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x65, 0x16d, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x6b, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x65, 0x16d, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x73,
0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x66,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x66, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x66,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x66, 0x69, 0x11d,
0x69, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x66, 0x69, 0x11d, 0x69,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x66, 0x69, 0x11d, 0x69, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x3a, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x61, 0x20, 0x47, 0x75, 0x6a,
0x61, 0x6e, 0x6f, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x3a, 0x20, 0x46, 0x72,
0x61, 0x6e, 0x63, 0x61, 0x6a, 0x20, 0x53, 0x75, 0x64, 0x61, 0x6a, 0x20,
0x54, 0x65, 0x72, 0x69, 0x74, 0x6f, 0x72, 0x69, 0x6f, 0x6a, 0x67, 0x61,
0x6c, 0x61, 0x70, 0x61, 0x67, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x3a, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69,
0x65, 0x72, 0x6b, 0x61, 0x72, 0x74, 0x76, 0x65, 0x6c, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x61, 0x72, 0x74, 0x76, 0x65, 0x6c, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x6b, 0x61, 0x72, 0x74, 0x76, 0x65, 0x6c, 0x61, 0x20, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x67, 0x69, 0x6c,
0x62, 0x65, 0x72, 0x74, 0x69, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61,
0x6c, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x20, 0x6b, 0x75, 0x6e,
0x6f, 0x72, 0x64, 0x69, 0x67, 0x69, 0x74, 0x61, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x20, 0x67, 0x72, 0x6f, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x20, 0x67, 0x72, 0x6f, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x20, 0x67, 0x72, 0x6f, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x6b, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x20, 0x67, 0x72, 0x6f, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x6b, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x20, 0x67, 0x72, 0x6f, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x6f, 0x6b, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x20,
0x67, 0x72, 0x6f, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x73, 0x6f,
0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61, 0x72,
0x61, 0x62, 0x67, 0x6f, 0x6c, 0x66, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x67, 0x75, 0x6a, 0x61, 0x6e,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x3a, 0x20, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x6f, 0x2d, 0x41, 0x6c, 0x65,
0x75, 0x74, 0x6f, 0x6a, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x6f, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x6f, 0x6a, 0x20, 0x28, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x29, 0x48, 0x61, 0x76, 0x61,
0x6a, 0x6f, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x6f, 0x6a, 0x20, 0x28,
0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x29, 0x68, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x68, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e, 0x67, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x68,
0x6f, 0x6e, 0x6b, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65,
0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x125, 0x6f, 0x76, 0x64,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x125, 0x6f, 0x76, 0x64, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x125, 0x6f, 0x76, 0x64, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x68, 0x69, 0x6e, 0x64, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x68,
0x69, 0x6e, 0x64, 0x6f, 0x63, 0x65, 0x61, 0x6e, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x68, 0x69, 0x6e, 0x64, 0x6f, 0x109, 0x69, 0x6e, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x2d,
0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x2d, 0x69, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x6f, 0x6b, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x2d, 0x69, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x69, 0x72, 0x61, 0x6e, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x69, 0x72, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x69, 0x72, 0x61, 0x6e, 0x61, 0x20, 0x73,
0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x69,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x69, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x69, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x69, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x69, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x69, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x61, 0x20, 0x73, 0x6f,
0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6a, 0x61,
0x70, 0x61, 0x6e, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6a, 0x61,
0x70, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x6a, 0x61, 0x70, 0x61, 0x6e, 0x61, 0x20, 0x73,
0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b,
0x61, 0x7a, 0x61, 0x125, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x2d, 0x6b, 0x61, 0x7a, 0x61, 0x125, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x6b, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x2d, 0x6b, 0x61, 0x7a, 0x61, 0x125, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x6b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x6b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x6f, 0x72, 0x65, 0x61,
0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x3a, 0x20, 0x4b, 0x6f, 0x73, 0x72,
0x61, 0x65, 0x6b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73,
0x6b, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x61, 0x20, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x69, 0x72,
0x67, 0x69, 0x7a, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x3a, 0x20, 0x4c, 0x69, 0x6e, 0x69, 0x61, 0x6a, 0x20,
0x49, 0x6e, 0x73, 0x75, 0x6c, 0x6f, 0x6a, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x3a, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x28, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x29, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x28, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x29, 0x6d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x6d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6d, 0x61, 0x6c, 0x61, 0x6a, 0x7a, 0x69,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6d, 0x61, 0x6c, 0x64, 0x69,
0x76, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6d, 0x61, 0x72, 0x6b,
0x69, 0x7a, 0x69, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x6d, 0x61, 0x72, 0x15d, 0x61, 0x6c, 0x69, 0x6e, 0x73, 0x75,
0x6c, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6d, 0x61, 0x16d, 0x72,
0x69, 0x63, 0x69, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6d, 0x61,
0x16d, 0x72, 0x69, 0x63, 0x69, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6d, 0x61, 0x16d, 0x72, 0x69, 0x63,
0x69, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x3a, 0x20, 0x4d, 0x61,
0x77, 0x73, 0x6f, 0x6e, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x6b, 0x61,
0x20, 0x6d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x20, 0x6d,
0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69,
0x6b, 0x61, 0x20, 0x6d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x20, 0x73,
0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x75,
0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x75, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x75, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x61, 0x20,
0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x6d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x6d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6d, 0x6f, 0x73, 0x6b, 0x76, 0x61,
0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x62, 0x69, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x6e, 0x61, 0x75, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6e,
0x65, 0x70, 0x61, 0x6c, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6e,
0x6f, 0x76, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6e, 0x6f, 0x76, 0x6b, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6e, 0x6f, 0x76, 0x6b, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6e, 0x6f, 0x76, 0x7a, 0x65, 0x6c,
0x61, 0x6e, 0x64, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6e, 0x6f,
0x76, 0x7a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6e, 0x6f, 0x76, 0x7a,
0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x3a, 0x20, 0x4e, 0x6f, 0x76, 0x6c, 0x61, 0x6e, 0x64, 0x6f, 0x4e, 0x6f,
0x76, 0x6c, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x28, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x29, 0x4e, 0x6f, 0x76, 0x6c,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x28, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x29, 0x6e, 0x69, 0x75, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x69, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x6e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x69, 0x6e, 0x73, 0x75, 0x6c,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x6e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x69, 0x6e, 0x73, 0x75,
0x6c, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x3a, 0x20, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x68, 0x61, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x20, 0x28, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x29, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x28, 0x73,
0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x29,
0x6e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x6d, 0x73, 0x6b, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x6d, 0x73, 0x6b, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f,
0x6d, 0x73, 0x6b, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x70, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x70, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x70, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x70, 0x61, 0x6c, 0x61, 0x16d, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x3a, 0x20, 0x50, 0x61,
0x70, 0x75, 0x6f, 0x2d, 0x4e, 0x6f, 0x76, 0x2d, 0x47, 0x76, 0x69, 0x6e,
0x65, 0x6f, 0x70, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x70, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61,
0x6a, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x70, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x61, 0x20,
0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x70, 0x65, 0x72, 0x75, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x70,
0x65, 0x72, 0x75, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x70, 0x65, 0x72, 0x75, 0x61, 0x20, 0x73, 0x6f,
0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x66, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x66, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x66, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x66, 0x65, 0x6e, 0x69, 0x6b, 0x73, 0x69,
0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x3a, 0x20, 0x53, 0x61, 0x6e, 0x6b, 0x74, 0x61,
0x20, 0x50, 0x69, 0x65, 0x72, 0x6f, 0x20, 0x6b, 0x61, 0x6a, 0x20, 0x4d,
0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x6f, 0x53, 0x61, 0x6e, 0x6b, 0x74,
0x61, 0x20, 0x50, 0x69, 0x65, 0x72, 0x6f, 0x20, 0x6b, 0x61, 0x6a, 0x20,
0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x6f, 0x20, 0x28, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x29, 0x53, 0x61,
0x6e, 0x6b, 0x74, 0x61, 0x20, 0x50, 0x69, 0x65, 0x72, 0x6f, 0x20, 0x6b,
0x61, 0x6a, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x6f, 0x20,
0x28, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x29, 0x70, 0x69, 0x74, 0x6b, 0x61, 0x72, 0x6e, 0x69, 0x6e, 0x73,
0x75, 0x6c, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x3a, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x70, 0x6a,
0x6f, 0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x67, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x3a, 0x20, 0x52, 0x65, 0x75,
0x6e, 0x69, 0x6f, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x3a, 0x20, 0x52, 0x6f,
0x74, 0x68, 0x65, 0x72, 0x61, 0x73, 0x61, 0x125, 0x61, 0x6c, 0x65, 0x6e,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x73, 0x61, 0x125, 0x61, 0x6c,
0x65, 0x6e, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x73, 0x61, 0x125, 0x61, 0x6c, 0x65, 0x6e, 0x61, 0x20,
0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x73, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x73,
0x61, 0x6d, 0x6f, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x73, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x73, 0x6f,
0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x73, 0x65,
0x6a, 0x15d, 0x65, 0x6c, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x73,
0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x3a, 0x20, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x6f, 0x6a,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x3a, 0x20, 0x53, 0x75, 0x64, 0x2d, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x6f, 0x73, 0x75, 0x72, 0x69, 0x6e, 0x61,
0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x3a, 0x20, 0x53, 0x68, 0x6f, 0x77, 0x61, 0x74, 0x61, 0x68, 0x69,
0x74, 0x69, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x61, 0x6a,
0x70, 0x65, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x61, 0x6a,
0x70, 0x65, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x74, 0x61, 0x6a, 0x70, 0x65, 0x61, 0x20, 0x73, 0x6f,
0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x61,
0x11d, 0x69, 0x6b, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x6f,
0x6b, 0x65, 0x6c, 0x61, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74,
0x6f, 0x6e, 0x67, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x6f,
0x6e, 0x67, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x74, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x3a, 0x20, 0x108, 0x75, 0x75, 0x6b, 0x74, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x75,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x74, 0x75, 0x76, 0x61, 0x6c, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x75, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x75, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x75, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x61, 0x20, 0x73, 0x6f,
0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x75, 0x7a,
0x62, 0x65, 0x6b, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x75, 0x7a,
0x62, 0x65, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x75, 0x7a, 0x62, 0x65, 0x6b, 0x61, 0x20, 0x73,
0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x76,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x76, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x76, 0x65, 0x6e, 0x65, 0x7a, 0x75,
0x65, 0x6c, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x76, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x76, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x76, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x76, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x76, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x76, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x3a, 0x20,
0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x76, 0x65, 0x6a, 0x6b, 0x69, 0x6e,
0x73, 0x75, 0x6c, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x3a, 0x20, 0x56, 0x61, 0x6c, 0x69, 0x73, 0x6f, 0x20,
0x6b, 0x61, 0x6a, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x6f, 0x6a, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x6a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6a, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x6a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x6a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6a, 0x75, 0x6b, 0x6f, 0x6e, 0x69,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x41, 0x63, 0x72, 0x65, 0x20,
0x61, 0x65, 0x67, 0x41, 0x63, 0x72, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x41, 0x63, 0x72, 0x65, 0x20,
0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x41, 0x66, 0x67, 0x61, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4b, 0x65,
0x73, 0x6b, 0x2d, 0x41, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x61,
0x65, 0x67, 0x49, 0x64, 0x61, 0x2d, 0x41, 0x61, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x61, 0x65, 0x67, 0x4c, 0xf5, 0x75, 0x6e, 0x61, 0x2d, 0x41,
0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x2d,
0x41, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x61, 0x65, 0x67, 0x4c,
0xe4, 0xe4, 0x6e, 0x65, 0x2d, 0x41, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67,
0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x2d, 0x41, 0x61, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x20, 0x61, 0x65, 0x67, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65,
0x67, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65,
0x61, 0x65, 0x67, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0xf5, 0x20, 0x61, 0x65,
0x67, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0xf5, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x41, 0x6c, 0x6d, 0x61, 0x74,
0xf5, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x65, 0x20, 0x61, 0x65, 0x67, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x61, 0x73, 0x65, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67,
0x4b, 0x65, 0x73, 0x6b, 0x2d, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x61, 0x65, 0x67, 0x4b, 0x65, 0x73, 0x6b, 0x2d, 0x41, 0x6d,
0x65, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4b, 0x65, 0x73, 0x6b, 0x2d, 0x41,
0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65,
0x61, 0x65, 0x67, 0x49, 0x64, 0x61, 0x72, 0x61, 0x6e, 0x6e, 0x69, 0x6b,
0x75, 0x20, 0x61, 0x65, 0x67, 0x49, 0x64, 0x61, 0x72, 0x61, 0x6e, 0x6e,
0x69, 0x6b, 0x75, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x65, 0x67, 0x49, 0x64, 0x61, 0x72, 0x61, 0x6e, 0x6e, 0x69, 0x6b,
0x75, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x4d, 0xe4, 0x65,
0x73, 0x74, 0x69, 0x6b, 0x75, 0x76, 0xf6, 0xf6, 0x6e, 0x64, 0x69, 0x20,
0x61, 0x65, 0x67, 0x4d, 0xe4, 0x65, 0x73, 0x74, 0x69, 0x6b, 0x75, 0x76,
0xf6, 0xf6, 0x6e, 0x64, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x65, 0x67, 0x4d, 0xe4, 0x65, 0x73, 0x74, 0x69, 0x6b,
0x75, 0x76, 0xf6, 0xf6, 0x6e, 0x64, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65,
0x61, 0x65, 0x67, 0x56, 0x61, 0x69, 0x6b, 0x73, 0x65, 0x20, 0x6f, 0x6f,
0x6b, 0x65, 0x61, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x56, 0x61, 0x69,
0x6b, 0x73, 0x65, 0x20, 0x6f, 0x6f, 0x6b, 0x65, 0x61, 0x6e, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x56,
0x61, 0x69, 0x6b, 0x73, 0x65, 0x20, 0x6f, 0x6f, 0x6b, 0x65, 0x61, 0x6e,
0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x41, 0x6e, 0x61,
0x64, 0xf5, 0x72, 0x69, 0x20, 0x61, 0x65, 0x67, 0x41, 0x6e, 0x61, 0x64,
0xf5, 0x72, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x65, 0x67, 0x41, 0x6e, 0x61, 0x64, 0xf5, 0x72, 0x69, 0x20, 0x73,
0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x41, 0x70, 0x69, 0x61, 0x20, 0x61,
0x65, 0x67, 0x41, 0x70, 0x69, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x41, 0x70, 0x69, 0x61, 0x20, 0x73,
0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x20,
0x61, 0x65, 0x67, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x41, 0x6b, 0x74, 0x61,
0x75, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x41, 0x6b, 0x74,
0xf6, 0x62, 0x65, 0x20, 0x61, 0x65, 0x67, 0x41, 0x6b, 0x74, 0xf6, 0x62,
0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65,
0x67, 0x41, 0x6b, 0x74, 0xf6, 0x62, 0x65, 0x20, 0x73, 0x75, 0x76, 0x65,
0x61, 0x65, 0x67, 0x41, 0x72, 0x61, 0x61, 0x62, 0x69, 0x61, 0x20, 0x61,
0x65, 0x67, 0x41, 0x72, 0x61, 0x61, 0x62, 0x69, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x41, 0x72, 0x61,
0x61, 0x62, 0x69, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x61, 0x65,
0x67, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65,
0x61, 0x65, 0x67, 0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x2d, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x61, 0x65, 0x67, 0x4c, 0xe4,
0xe4, 0x6e, 0x65, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65,
0x67, 0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67,
0x41, 0x72, 0x6d, 0x65, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x61, 0x65, 0x67,
0x41, 0x72, 0x6d, 0x65, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x41, 0x72, 0x6d, 0x65,
0x65, 0x6e, 0x69, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x61, 0x65, 0x67, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x4b, 0x65, 0x73,
0x6b, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x61, 0x65, 0x67, 0x4b, 0x65, 0x73, 0x6b, 0x2d, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4b, 0x65, 0x73, 0x6b, 0x2d,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73,
0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x4b, 0x65, 0x73, 0x6b, 0x2d, 0x4c, 0xe4,
0xe4, 0x6e, 0x65, 0x20, 0x61, 0x65, 0x67, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x4b, 0x65, 0x73, 0x6b, 0x2d, 0x4c,
0xe4, 0xe4, 0x6e, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x65, 0x67, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c,
0x69, 0x61, 0x20, 0x4b, 0x65, 0x73, 0x6b, 0x2d, 0x4c, 0xe4, 0xe4, 0x6e,
0x65, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x49, 0x64, 0x61,
0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x20,
0x61, 0x65, 0x67, 0x49, 0x64, 0x61, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x65, 0x67, 0x49, 0x64, 0x61, 0x2d, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65,
0x61, 0x65, 0x67, 0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x2d, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x61, 0x65, 0x67, 0x4c,
0xe4, 0xe4, 0x6e, 0x65, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x65, 0x67, 0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x2d, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65,
0x61, 0x65, 0x67, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x17e,
0x61, 0x61, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x41, 0x73, 0x65, 0x72,
0x62, 0x61, 0x69, 0x64, 0x17e, 0x61, 0x61, 0x6e, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x41, 0x73, 0x65,
0x72, 0x62, 0x61, 0x69, 0x64, 0x17e, 0x61, 0x61, 0x6e, 0x69, 0x20, 0x73,
0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x41, 0x73, 0x73, 0x6f, 0x6f, 0x72,
0x69, 0x64, 0x65, 0x20, 0x61, 0x65, 0x67, 0x41, 0x73, 0x73, 0x6f, 0x6f,
0x72, 0x69, 0x64, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x65, 0x67, 0x41, 0x73, 0x73, 0x6f, 0x6f, 0x72, 0x69, 0x64,
0x65, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x69, 0x20, 0x61, 0x65, 0x67,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x69, 0x20, 0x73,
0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x61, 0x65, 0x67, 0x42, 0x6f, 0x6c, 0x69, 0x69, 0x76, 0x69,
0x61, 0x20, 0x61, 0x65, 0x67, 0x42, 0x72, 0x61, 0x73, 0x69, 0x69, 0x6c,
0x69, 0x61, 0x20, 0x61, 0x65, 0x67, 0x42, 0x72, 0x61, 0x73, 0x69, 0x69,
0x6c, 0x69, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x65, 0x67, 0x42, 0x72, 0x61, 0x73, 0x69, 0x69, 0x6c, 0x69, 0x61,
0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x42, 0x72, 0x75, 0x6e,
0x65, 0x69, 0x20, 0x61, 0x65, 0x67, 0x52, 0x6f, 0x68, 0x65, 0x6e, 0x65,
0x65, 0x6d, 0x65, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x20, 0x61, 0x65,
0x67, 0x52, 0x6f, 0x68, 0x65, 0x6e, 0x65, 0x65, 0x6d, 0x65, 0x73, 0x61,
0x61, 0x72, 0x74, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x65, 0x67, 0x52, 0x6f, 0x68, 0x65, 0x6e, 0x65, 0x65, 0x6d,
0x65, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x20, 0x73, 0x75, 0x76, 0x65,
0x61, 0x65, 0x67, 0x43, 0x61, 0x73, 0x65, 0x79, 0x20, 0x61, 0x65, 0x67,
0x54, 0x161, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x69, 0x20, 0x61, 0x65, 0x67, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x65, 0x67, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x69,
0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x54, 0x161, 0x69, 0x69,
0x6c, 0x69, 0x20, 0x61, 0x65, 0x67, 0x54, 0x161, 0x69, 0x69, 0x6c, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67,
0x54, 0x161, 0x69, 0x69, 0x6c, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61,
0x65, 0x67, 0x48, 0x69, 0x69, 0x6e, 0x61, 0x20, 0x61, 0x65, 0x67, 0x48,
0x69, 0x69, 0x6e, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x65, 0x67, 0x48, 0x69, 0x69, 0x6e, 0x61, 0x20, 0x73, 0x75,
0x76, 0x65, 0x61, 0x65, 0x67, 0x4a, 0xf5, 0x75, 0x6c, 0x75, 0x73, 0x61,
0x61, 0x72, 0x65, 0x20, 0x61, 0x65, 0x67, 0x4b, 0x6f, 0x6f, 0x6b, 0x6f,
0x73, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x20, 0x61, 0x65, 0x67, 0x43,
0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x61, 0x65, 0x67, 0x43,
0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x43, 0x6f, 0x6c, 0x6f, 0x6d,
0x62, 0x69, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x43,
0x6f, 0x6f, 0x6b, 0x69, 0x20, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x20,
0x61, 0x65, 0x67, 0x43, 0x6f, 0x6f, 0x6b, 0x69, 0x20, 0x73, 0x61, 0x61,
0x72, 0x74, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x65, 0x67, 0x43, 0x6f, 0x6f, 0x6b, 0x69, 0x20, 0x73, 0x61, 0x61,
0x72, 0x74, 0x65, 0x20, 0x6f, 0x73, 0x61, 0x6c, 0x69, 0x6e, 0x65, 0x20,
0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x4b, 0x75, 0x75, 0x62, 0x61,
0x20, 0x61, 0x65, 0x67, 0x4b, 0x75, 0x75, 0x62, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4b, 0x75, 0x75,
0x62, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x44, 0x61,
0x76, 0x69, 0x73, 0x65, 0x20, 0x61, 0x65, 0x67, 0x44, 0x75, 0x6d, 0x6f,
0x6e, 0x74, 0x20, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65,
0x2019, 0x69, 0x20, 0x61, 0x65, 0x67, 0x49, 0x64, 0x61, 0x2d, 0x54, 0x69,
0x6d, 0x6f, 0x72, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4c, 0x69, 0x68, 0x61,
0x76, 0xf5, 0x74, 0x74, 0x65, 0x73, 0x61, 0x61, 0x72, 0x65, 0x20, 0x61,
0x65, 0x67, 0x4c, 0x69, 0x68, 0x61, 0x76, 0xf5, 0x74, 0x74, 0x65, 0x73,
0x61, 0x61, 0x72, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x65, 0x67, 0x4c, 0x69, 0x68, 0x61, 0x76, 0xf5, 0x74, 0x74,
0x65, 0x73, 0x61, 0x61, 0x72, 0x65, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61,
0x65, 0x67, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x69, 0x20, 0x61,
0x65, 0x67, 0x4b, 0x65, 0x73, 0x6b, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x6f,
0x70, 0x61, 0x20, 0x61, 0x65, 0x67, 0x4b, 0x65, 0x73, 0x6b, 0x2d, 0x45,
0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4b, 0x65, 0x73, 0x6b, 0x2d, 0x45,
0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61,
0x65, 0x67, 0x49, 0x64, 0x61, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x6f, 0x70,
0x61, 0x20, 0x61, 0x65, 0x67, 0x49, 0x64, 0x61, 0x2d, 0x45, 0x75, 0x72,
0x6f, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x65, 0x67, 0x49, 0x64, 0x61, 0x2d, 0x45, 0x75, 0x72, 0x6f,
0x6f, 0x70, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x4b,
0x61, 0x6c, 0x69, 0x6e, 0x69, 0x6e, 0x67, 0x72, 0x61, 0x64, 0x69, 0x20,
0x6a, 0x61, 0x20, 0x56, 0x61, 0x6c, 0x67, 0x65, 0x76, 0x65, 0x6e, 0x65,
0x20, 0x61, 0x65, 0x67, 0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x2d, 0x45, 0x75,
0x72, 0x6f, 0x6f, 0x70, 0x61, 0x20, 0x61, 0x65, 0x67, 0x4c, 0xe4, 0xe4,
0x6e, 0x65, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4c, 0xe4,
0xe4, 0x6e, 0x65, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61, 0x20,
0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x20, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x20,
0x61, 0x65, 0x67, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x20, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x20, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x20,
0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x46, 0x69, 0x64, 0x17e, 0x69,
0x20, 0x61, 0x65, 0x67, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x46, 0x69, 0x64,
0x17e, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x50, 0x72,
0x61, 0x6e, 0x74, 0x73, 0x75, 0x73, 0x65, 0x20, 0x47, 0x75, 0x61, 0x6a,
0x61, 0x61, 0x6e, 0x61, 0x20, 0x61, 0x65, 0x67, 0x50, 0x72, 0x61, 0x6e,
0x74, 0x73, 0x75, 0x73, 0x65, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b,
0x74, 0x69, 0x6c, 0x69, 0x73, 0x74, 0x65, 0x20, 0x6a, 0x61, 0x20, 0x4c,
0xf5, 0x75, 0x6e, 0x61, 0x61, 0x6c, 0x61, 0x64, 0x65, 0x20, 0x61, 0x65,
0x67, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x65, 0x20,
0x61, 0x65, 0x67, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x2019, 0x20,
0x61, 0x65, 0x67, 0x47, 0x72, 0x75, 0x75, 0x73, 0x69, 0x61, 0x20, 0x61,
0x65, 0x67, 0x47, 0x72, 0x75, 0x75, 0x73, 0x69, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x47, 0x72, 0x75,
0x75, 0x73, 0x69, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67,
0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x69, 0x20, 0x73, 0x61, 0x61,
0x72, 0x74, 0x65, 0x20, 0x61, 0x65, 0x67, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x77, 0x69, 0x63, 0x68, 0x69, 0x20, 0x61, 0x65, 0x67, 0x47, 0x72, 0xf6,
0xf6, 0x6e, 0x69, 0x6d, 0x61, 0x61, 0x20, 0x61, 0x65, 0x67, 0x47, 0x72,
0xf6, 0xf6, 0x6e, 0x69, 0x6d, 0x61, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x47, 0x72, 0xf6, 0xf6, 0x6e,
0x69, 0x6d, 0x61, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67,
0x49, 0x64, 0x61, 0x2d, 0x47, 0x72, 0xf6, 0xf6, 0x6e, 0x69, 0x6d, 0x61,
0x61, 0x20, 0x61, 0x65, 0x67, 0x49, 0x64, 0x61, 0x2d, 0x47, 0x72, 0xf6,
0xf6, 0x6e, 0x69, 0x6d, 0x61, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x49, 0x64, 0x61, 0x2d, 0x47, 0x72,
0xf6, 0xf6, 0x6e, 0x69, 0x6d, 0x61, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65,
0x61, 0x65, 0x67, 0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x2d, 0x47, 0x72, 0xf6,
0xf6, 0x6e, 0x69, 0x6d, 0x61, 0x61, 0x20, 0x61, 0x65, 0x67, 0x4c, 0xe4,
0xe4, 0x6e, 0x65, 0x2d, 0x47, 0x72, 0xf6, 0xf6, 0x6e, 0x69, 0x6d, 0x61,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65,
0x67, 0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x2d, 0x47, 0x72, 0xf6, 0xf6, 0x6e,
0x69, 0x6d, 0x61, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67,
0x47, 0x75, 0x61, 0x6d, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x65, 0x67, 0x50, 0xe4, 0x72, 0x73, 0x69, 0x61, 0x20,
0x6c, 0x61, 0x68, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x65, 0x67, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x20, 0x61,
0x65, 0x67, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65,
0x75, 0x75, 0x64, 0x69, 0x20, 0x61, 0x65, 0x67, 0x48, 0x61, 0x77, 0x61,
0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x75, 0x64, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x48, 0x61,
0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x75, 0x64, 0x69,
0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x48, 0x6f, 0x6e, 0x67,
0x6b, 0x6f, 0x6e, 0x67, 0x69, 0x20, 0x61, 0x65, 0x67, 0x48, 0x6f, 0x6e,
0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f,
0x6e, 0x67, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x48,
0x6f, 0x76, 0x64, 0x69, 0x20, 0x61, 0x65, 0x67, 0x48, 0x6f, 0x76, 0x64,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65,
0x67, 0x48, 0x6f, 0x76, 0x64, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61,
0x65, 0x67, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x61, 0x65, 0x67, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x6f, 0x6b, 0x65, 0x61, 0x6e, 0x69,
0x20, 0x61, 0x65, 0x67, 0x49, 0x6e, 0x64, 0x6f, 0x68, 0x69, 0x69, 0x6e,
0x61, 0x20, 0x61, 0x65, 0x67, 0x4b, 0x65, 0x73, 0x6b, 0x2d, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x65, 0x73, 0x69, 0x61, 0x20, 0x61, 0x65, 0x67,
0x49, 0x64, 0x61, 0x2d, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x65, 0x73,
0x69, 0x61, 0x20, 0x61, 0x65, 0x67, 0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x2d,
0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x65, 0x73, 0x69, 0x61, 0x20, 0x61,
0x65, 0x67, 0x49, 0x72, 0x61, 0x61, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67,
0x49, 0x72, 0x61, 0x61, 0x6e, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x49, 0x72, 0x61, 0x61, 0x6e, 0x69,
0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x69, 0x20, 0x61, 0x65, 0x67, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x65, 0x67, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x49, 0x69, 0x73, 0x72,
0x61, 0x65, 0x6c, 0x69, 0x20, 0x61, 0x65, 0x67, 0x49, 0x69, 0x73, 0x72,
0x61, 0x65, 0x6c, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x65, 0x67, 0x49, 0x69, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x69,
0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x4a, 0x61, 0x61, 0x70,
0x61, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4a, 0x61, 0x61, 0x70, 0x61,
0x6e, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61,
0x65, 0x67, 0x4a, 0x61, 0x61, 0x70, 0x61, 0x6e, 0x69, 0x20, 0x73, 0x75,
0x76, 0x65, 0x61, 0x65, 0x67, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61,
0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x74, 0x161,
0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4b, 0x61, 0x6d,
0x74, 0x161, 0x61, 0x74, 0x6b, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4b, 0x61, 0x6d, 0x74, 0x161, 0x61,
0x74, 0x6b, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x4b,
0x61, 0x73, 0x61, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x61, 0x65,
0x67, 0x49, 0x64, 0x61, 0x2d, 0x4b, 0x61, 0x73, 0x61, 0x68, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4c, 0xe4, 0xe4, 0x6e, 0x65,
0x2d, 0x4b, 0x61, 0x73, 0x61, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20,
0x61, 0x65, 0x67, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x61, 0x65, 0x67,
0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x65, 0x67, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x73,
0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65,
0x20, 0x61, 0x65, 0x67, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61,
0x72, 0x73, 0x6b, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x75, 0x76,
0x65, 0x61, 0x65, 0x67, 0x4b, 0xf5, 0x72, 0x67, 0xf5, 0x7a, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x53, 0x72, 0x69, 0x20, 0x4c,
0x61, 0x6e, 0x6b, 0x61, 0x20, 0x61, 0x65, 0x67, 0x4c, 0x69, 0x6e, 0x65,
0x2019, 0x69, 0x20, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x20, 0x61, 0x65,
0x67, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x2019, 0x69,
0x20, 0x61, 0x65, 0x67, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x2019, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x65, 0x67, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0x2019, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x4d, 0x61,
0x63, 0x61, 0x75, 0x20, 0x61, 0x65, 0x67, 0x4d, 0x61, 0x63, 0x61, 0x75,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67,
0x4d, 0x61, 0x63, 0x61, 0x75, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65,
0x67, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x69, 0x20, 0x61, 0x65,
0x67, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65,
0x67, 0x4d, 0x61, 0x6c, 0x61, 0x69, 0x73, 0x69, 0x61, 0x20, 0x61, 0x65,
0x67, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x69, 0x76, 0x69, 0x20, 0x61, 0x65,
0x67, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x69, 0x73, 0x61, 0x61, 0x72, 0x74,
0x65, 0x20, 0x61, 0x65, 0x67, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c,
0x6c, 0x69, 0x20, 0x53, 0x61, 0x61, 0x72, 0x74, 0x65, 0x20, 0x61, 0x65,
0x67, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x65, 0x20,
0x61, 0x65, 0x67, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73,
0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65,
0x67, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x65, 0x20,
0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x4d, 0x61, 0x77, 0x73, 0x6f,
0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4d, 0x65, 0x68, 0x68, 0x69, 0x6b,
0x6f, 0x20, 0x56, 0x61, 0x69, 0x6b, 0x73, 0x65, 0x20, 0x6f, 0x6f, 0x6b,
0x65, 0x61, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4d, 0x65, 0x68, 0x68,
0x69, 0x6b, 0x6f, 0x20, 0x56, 0x61, 0x69, 0x6b, 0x73, 0x65, 0x20, 0x6f,
0x6f, 0x6b, 0x65, 0x61, 0x6e, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4d, 0x65, 0x68, 0x68, 0x69, 0x6b,
0x6f, 0x20, 0x56, 0x61, 0x69, 0x6b, 0x73, 0x65, 0x20, 0x6f, 0x6f, 0x6b,
0x65, 0x61, 0x6e, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67,
0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x69,
0x20, 0x61, 0x65, 0x67, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61,
0x74, 0x61, 0x72, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x65, 0x67, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61,
0x74, 0x61, 0x72, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67,
0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20, 0x61, 0x65, 0x67, 0x4d, 0x6f,
0x73, 0x6b, 0x76, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x65, 0x67, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20, 0x73,
0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x42, 0x69, 0x72, 0x6d, 0x61, 0x20,
0x61, 0x65, 0x67, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x20, 0x61, 0x65, 0x67,
0x4e, 0x65, 0x70, 0x61, 0x6c, 0x69, 0x20, 0x61, 0x65, 0x67, 0x55, 0x75,
0x73, 0x2d, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6f, 0x6e, 0x69, 0x61,
0x20, 0x61, 0x65, 0x67, 0x55, 0x75, 0x73, 0x2d, 0x4b, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x55, 0x75, 0x73, 0x2d, 0x4b, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x73, 0x75, 0x76,
0x65, 0x61, 0x65, 0x67, 0x55, 0x75, 0x73, 0x2d, 0x4d, 0x65, 0x72, 0x65,
0x6d, 0x61, 0x61, 0x20, 0x61, 0x65, 0x67, 0x55, 0x75, 0x73, 0x2d, 0x4d,
0x65, 0x72, 0x65, 0x6d, 0x61, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x55, 0x75, 0x73, 0x2d, 0x4d, 0x65,
0x72, 0x65, 0x6d, 0x61, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65,
0x67, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75,
0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4e, 0x65, 0x77, 0x66, 0x6f,
0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x73, 0x75, 0x76,
0x65, 0x61, 0x65, 0x67, 0x4e, 0x69, 0x75, 0x65, 0x20, 0x61, 0x65, 0x67,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x69, 0x20, 0x73, 0x61, 0x61,
0x72, 0x65, 0x20, 0x61, 0x65, 0x67, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x69, 0x20, 0x73, 0x61, 0x61, 0x72, 0x65, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x69, 0x20, 0x73, 0x61, 0x61, 0x72, 0x65, 0x20, 0x73,
0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x20, 0x61, 0x65, 0x67, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65,
0x61, 0x65, 0x67, 0x50, 0xf5, 0x68, 0x6a, 0x61, 0x2d, 0x4d, 0x61, 0x72,
0x69, 0x61, 0x61, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x61, 0x65,
0x67, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65,
0x67, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x4f, 0x6d, 0x73,
0x6b, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4f, 0x6d, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4f,
0x6d, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x61, 0x65,
0x67, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65,
0x61, 0x65, 0x67, 0x42, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x61, 0x65, 0x67,
0x50, 0x61, 0x61, 0x70, 0x75, 0x61, 0x20, 0x55, 0x75, 0x73, 0x2d, 0x47,
0x75, 0x69, 0x6e, 0x65, 0x61, 0x20, 0x61, 0x65, 0x67, 0x50, 0x61, 0x72,
0x61, 0x67, 0x75, 0x61, 0x79, 0x20, 0x61, 0x65, 0x67, 0x50, 0x61, 0x72,
0x61, 0x67, 0x75, 0x61, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x65, 0x67, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x50, 0x65, 0x72,
0x75, 0x75, 0x20, 0x61, 0x65, 0x67, 0x50, 0x65, 0x72, 0x75, 0x75, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x50,
0x65, 0x72, 0x75, 0x75, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67,
0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x69, 0x6e, 0x69, 0x20, 0x61, 0x65,
0x67, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x69, 0x6e, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x69, 0x6e, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65,
0x61, 0x65, 0x67, 0x46, 0xf6, 0xf6, 0x6e, 0x69, 0x6b, 0x73, 0x69, 0x73,
0x61, 0x61, 0x72, 0x74, 0x65, 0x20, 0x61, 0x65, 0x67, 0x53, 0x61, 0x69,
0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2019, 0x69, 0x20,
0x6a, 0x61, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x69,
0x20, 0x61, 0x65, 0x67, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x2019, 0x69, 0x20, 0x6a, 0x61, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x53, 0x61, 0x69, 0x6e, 0x74,
0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2019, 0x69, 0x20, 0x6a, 0x61,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x69, 0x20, 0x73,
0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69,
0x72, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x50, 0x6f, 0x68, 0x6e, 0x70,
0x65, 0x69, 0x20, 0x61, 0x65, 0x67, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79,
0x61, 0x6e, 0x67, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4b, 0xf5, 0x7a, 0xf5,
0x6c, 0x6f, 0x72, 0x64, 0x61, 0x20, 0x61, 0x65, 0x67, 0x4b, 0xf5, 0x7a,
0xf5, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4b, 0xf5, 0x7a, 0xf5, 0x6c, 0x6f,
0x72, 0x64, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x52,
0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x52,
0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x20, 0x61, 0x65, 0x67, 0x53, 0x61,
0x68, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x53,
0x61, 0x68, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x53, 0x61, 0x68, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65,
0x67, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20, 0x61, 0x65, 0x67, 0x53,
0x61, 0x6d, 0x61, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x65, 0x67, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20,
0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x20, 0x61, 0x65, 0x67, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x53, 0x65,
0x69, 0x161, 0x65, 0x6c, 0x6c, 0x69, 0x20, 0x61, 0x65, 0x67, 0x53, 0x69,
0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x53, 0x61, 0x61, 0x6c, 0x6f,
0x6d, 0x6f, 0x6e, 0x69, 0x20, 0x53, 0x61, 0x61, 0x72, 0x74, 0x65, 0x20,
0x61, 0x65, 0x67, 0x4c, 0xf5, 0x75, 0x6e, 0x61, 0x2d, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x61, 0x20, 0x61, 0x65, 0x67, 0x53, 0x75, 0x72, 0x69,
0x6e, 0x61, 0x6d, 0x65, 0x20, 0x61, 0x65, 0x67, 0x53, 0x79, 0x6f, 0x77,
0x61, 0x20, 0x61, 0x65, 0x67, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x20,
0x61, 0x65, 0x67, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x61, 0x65,
0x67, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x54, 0x61, 0x64,
0x17e, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x61, 0x65,
0x67, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x61, 0x65, 0x67,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x61, 0x65, 0x67, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61,
0x65, 0x67, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65,
0x61, 0x65, 0x67, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x69, 0x20, 0x61, 0x65,
0x67, 0x54, 0xfc, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x54, 0xfc, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x54, 0xfc, 0x72, 0x6b, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x73, 0x75, 0x76,
0x65, 0x61, 0x65, 0x67, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x20, 0x61,
0x65, 0x67, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x20, 0x61, 0x65,
0x67, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x55, 0x72, 0x75, 0x67,
0x75, 0x61, 0x79, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x55,
0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x61,
0x65, 0x67, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65,
0x67, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x20, 0x61, 0x65, 0x67, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61,
0x65, 0x67, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x73, 0x75,
0x76, 0x65, 0x61, 0x65, 0x67, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65,
0x6c, 0x61, 0x20, 0x61, 0x65, 0x67, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x69, 0x20, 0x61, 0x65, 0x67, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x69, 0x20, 0x73,
0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x69, 0x20, 0x61, 0x65, 0x67, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67,
0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x69, 0x20, 0x61, 0x65, 0x67, 0x57,
0x61, 0x6b, 0x65, 0x2019, 0x69, 0x20, 0x61, 0x65, 0x67, 0x57, 0x61, 0x6c,
0x6c, 0x69, 0x73, 0x65, 0x20, 0x6a, 0x61, 0x20, 0x46, 0x75, 0x74, 0x75,
0x6e, 0x61, 0x20, 0x61, 0x65, 0x67, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61,
0x65, 0x67, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x69, 0x20, 0x61, 0x65, 0x67,
0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61,
0x65, 0x67, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67,
0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x45, 0x6b,
0x65, 0x72, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x45,
0x6b, 0x65, 0x72, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x256, 0x6f,
0x61, 0x6e, 0x79, 0x69, 0x6d, 0x65, 0x45, 0x6b, 0x65, 0x72, 0x20, 0x64,
0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x41, 0x66, 0x67, 0x68, 0x61, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x41, 0x66,
0x72, 0x69, 0x63, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63,
0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x53,
0x6f, 0x75, 0x74, 0x68, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20,
0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x20, 0x41, 0x66, 0x72,
0x69, 0x63, 0x61, 0x20, 0x67, 0x61, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74,
0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6e, 0x75, 0x74, 0x6f,
0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x57, 0x65, 0x73, 0x74, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20,
0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x41, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x69,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x41, 0x6c, 0x6d,
0x61, 0x74, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x256, 0x6f,
0x61, 0x6e, 0x79, 0x69, 0x6d, 0x65, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x69,
0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c,
0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x54,
0x69, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x54, 0x69,
0x74, 0x69, 0x6e, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61,
0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x54, 0x69, 0x74, 0x69, 0x6e, 0x61, 0x20,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6b, 0x65, 0x6c, 0x65,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x45, 0x61,
0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x45,
0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x65,
0x72, 0x6e, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6b,
0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x6f, 0x75, 0x6e, 0x74,
0x61, 0x69, 0x6e, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x6f, 0x75, 0x6e,
0x74, 0x61, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50, 0x61, 0x63, 0x69, 0x66,
0x69, 0x63, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x50,
0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d,
0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50,
0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x6e, 0x61,
0x64, 0x69, 0x72, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65,
0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x256, 0x6f, 0x61, 0x6e, 0x79, 0x69, 0x6d, 0x65, 0x41, 0x6e, 0x61,
0x64, 0x69, 0x72, 0x20, 0x14b, 0x6b, 0x65, 0x6b, 0x65, 0x6d, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x41, 0x70, 0x69, 0x61,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x70,
0x69, 0x61, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x70, 0x69, 0x61, 0x20,
0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x6d, 0x65, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x256, 0x6f, 0x61, 0x6e, 0x79, 0x69, 0x6d, 0x65,
0x41, 0x6b, 0x74, 0x61, 0x75, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b,
0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65,
0x41, 0x6b, 0x74, 0x6f, 0x62, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x6d, 0x65, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x256, 0x6f, 0x61, 0x6e, 0x79, 0x69, 0x6d, 0x65,
0x41, 0x6b, 0x74, 0x74, 0x6f, 0x62, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x6d, 0x65, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x72, 0x61, 0x62,
0x69, 0x61, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x72, 0x61, 0x62, 0x69,
0x61, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c,
0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x194,
0x65, 0x74, 0x6f, 0x256, 0x6f, 0x192, 0x65, 0x20, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x194, 0x65, 0x74, 0x6f, 0x256, 0x6f, 0x192, 0x65, 0x20,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6e, 0x75,
0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x194, 0x65, 0x74, 0x6f, 0x256, 0x6f, 0x192, 0x65, 0x20, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x64, 0x7a, 0x6f,
0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x72, 0x6d,
0x65, 0x6e, 0x69, 0x61, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x72, 0x6d,
0x65, 0x6e, 0x69, 0x61, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254,
0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x63, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x63, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x6e, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x6e, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20,
0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x6e, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x263, 0x65,
0x74, 0x6f, 0x256, 0x6f, 0x66, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x6e, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x57,
0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d,
0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72,
0x6e, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72,
0x6e, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e,
0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x6e, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e,
0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x6e, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20,
0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x20, 0x6e, 0x75, 0x74,
0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x20,
0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x7a, 0x6f, 0x72, 0x65,
0x73, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x73, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x7a,
0x6f, 0x72, 0x65, 0x73, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254,
0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d,
0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x20, 0x64, 0x7a,
0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x42, 0x6f, 0x6c,
0x69, 0x76, 0x69, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x42, 0x72, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x61, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b,
0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75,
0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72,
0x64, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x6e,
0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64,
0x65, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x68, 0x61,
0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x68, 0x69,
0x6c, 0x65, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x68, 0x69, 0x6c, 0x65,
0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x68, 0x69, 0x6e,
0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43,
0x68, 0x69, 0x6e, 0x61, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x68, 0x69,
0x6e, 0x61, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d,
0x61, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x6f, 0x63, 0x6f, 0x73,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x6e, 0x75, 0x74,
0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x64, 0x7a,
0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x6f, 0x6f, 0x6b,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x64, 0x7a, 0x6f,
0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x43, 0x75, 0x62, 0x61, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x43, 0x75, 0x62, 0x61, 0x20, 0x6e, 0x75,
0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d,
0x65, 0x43, 0x75, 0x62, 0x61, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x44, 0x61, 0x76, 0x69, 0x73,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x44, 0x75,
0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c,
0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x45, 0x61, 0x73, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74,
0x65, 0x72, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x65,
0x72, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6e, 0x75, 0x74,
0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x45, 0x63,
0x75, 0x61, 0x64, 0x6f, 0x72, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x194, 0x65, 0x64, 0x7a,
0x65, 0x192, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x194, 0x65, 0x64, 0x7a, 0x65,
0x192, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x256, 0x6f, 0x61, 0x6e, 0x79, 0x69, 0x6d, 0x65,
0x194, 0x65, 0x64, 0x7a, 0x65, 0x192, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x20, 0x14b, 0x6b, 0x65, 0x6b, 0x65, 0x6d, 0x65, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x65,
0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x65,
0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x6e, 0x75,
0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254,
0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d,
0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x46,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c,
0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x46,
0x69, 0x6a, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x46, 0x69,
0x6a, 0x69, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x46, 0x72,
0x65, 0x6e, 0x63, 0x68, 0x20, 0x47, 0x75, 0x69, 0x61, 0x6e, 0x61, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x46, 0x72, 0x65,
0x6e, 0x63, 0x68, 0x20, 0x53, 0x6f, 0x75, 0x74, 0x68, 0x65, 0x72, 0x6e,
0x20, 0x26, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x63, 0x74, 0x69, 0x63,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x47, 0x61,
0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x47, 0x65,
0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x6e,
0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64,
0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72,
0x74, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x77, 0x69, 0x63, 0x68, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d,
0x65, 0x45, 0x61, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74,
0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x64,
0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x20, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x20, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6e, 0x75, 0x74,
0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x57, 0x65, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x47, 0x75, 0x6c, 0x66, 0x20, 0x6e,
0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x48, 0x61, 0x77, 0x61, 0x69,
0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x48, 0x61, 0x77, 0x61, 0x69,
0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x20, 0x6e, 0x75,
0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65,
0x75, 0x74, 0x69, 0x61, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6e, 0x67, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x20,
0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e,
0x67, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x48, 0x6f, 0x76,
0x64, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x48,
0x6f, 0x76, 0x64, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x48, 0x6f, 0x76, 0x64,
0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x49, 0x6e, 0x64, 0x69,
0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x6e, 0x20, 0x4f, 0x63, 0x65, 0x61, 0x6e, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x49, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20,
0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x65,
0x72, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x57, 0x65,
0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x73, 0x69, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x6e, 0x75, 0x74, 0x6f,
0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x49, 0x72, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x6e, 0x75, 0x74, 0x6f,
0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x64, 0x7a, 0x6f, 0x6d,
0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x49, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x20,
0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x20, 0x64, 0x7a,
0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61,
0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x74, 0x73,
0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x6d, 0x65, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x74, 0x73, 0x61, 0x74, 0x73,
0x6b, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x256, 0x6f, 0x61,
0x6e, 0x79, 0x69, 0x6d, 0x65, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61,
0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x74, 0x73,
0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x14b, 0x6b, 0x65, 0x6b, 0x65, 0x6d,
0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x45, 0x61,
0x73, 0x74, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x57,
0x65, 0x73, 0x74, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x6e, 0x75, 0x74,
0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65,
0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x79, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65,
0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x4b, 0x79, 0x72, 0x67, 0x79, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4c, 0x69, 0x6e,
0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20,
0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x61, 0x6b, 0x61, 0x75, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x4d, 0x61, 0x6b, 0x61, 0x75, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x256, 0x6f, 0x61, 0x6e, 0x79, 0x69,
0x6d, 0x65, 0x4d, 0x61, 0x6b, 0x61, 0x75, 0x20, 0x14b, 0x6b, 0x65, 0x6b,
0x65, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x61, 0x6c, 0x61,
0x79, 0x73, 0x69, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x73, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x61, 0x72, 0x71,
0x75, 0x65, 0x73, 0x61, 0x73, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69,
0x75, 0x73, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x20, 0x6e, 0x75,
0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x20,
0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x61, 0x77, 0x73, 0x6f,
0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d,
0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66,
0x69, 0x63, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69,
0x66, 0x69, 0x63, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x65, 0x78, 0x69,
0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20,
0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d,
0x65, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x55, 0x6c, 0x61,
0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x6e, 0x75, 0x74, 0x6f,
0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x64,
0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x6f,
0x73, 0x63, 0x6f, 0x77, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x6f, 0x73,
0x63, 0x6f, 0x77, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c,
0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d,
0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4e, 0x65, 0x70, 0x61, 0x6c,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4e, 0x65,
0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4e, 0x65, 0x77,
0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x6e,
0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x69, 0x61, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254,
0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4e, 0x65, 0x77,
0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6e, 0x75, 0x74,
0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65,
0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x6d, 0x65, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x4e, 0x69, 0x75, 0x65, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6e,
0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b,
0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x64, 0x7a,
0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b, 0x20,
0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x64, 0x7a, 0x6f,
0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d,
0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x64, 0x7a, 0x6f, 0x6d,
0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20,
0x4e, 0x65, 0x77, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x79, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x20, 0x6e,
0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x20,
0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50, 0x65, 0x72, 0x75, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50, 0x65, 0x72,
0x75, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50, 0x65, 0x72, 0x75, 0x20, 0x64,
0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70,
0x69, 0x6e, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x65, 0x20,
0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69,
0x6e, 0x65, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50, 0x68,
0x6f, 0x65, 0x6e, 0x69, 0x78, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x53,
0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72,
0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50,
0x6f, 0x6e, 0x61, 0x70, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4b, 0x69,
0x7a, 0x69, 0x6c, 0x254, 0x72, 0x64, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x6d, 0x65, 0x4b, 0x69, 0x7a, 0x69, 0x6c, 0x254, 0x72, 0x64,
0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x256, 0x6f, 0x61, 0x6e,
0x79, 0x69, 0x6d, 0x65, 0x4b, 0x69, 0x7a, 0x69, 0x6c, 0x254, 0x72, 0x64,
0x61, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x52, 0x65, 0x75, 0x6e,
0x69, 0x6f, 0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x6e, 0x75, 0x74,
0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x64, 0x7a,
0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x256, 0x6f, 0x61,
0x6e, 0x79, 0x69, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20,
0x14b, 0x6b, 0x65, 0x6b, 0x65, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x6b,
0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x53, 0x69, 0x6e,
0x67, 0x61, 0x70, 0x6f, 0x72, 0x65, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d,
0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x53,
0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x53, 0x6f, 0x75, 0x74, 0x68, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x53,
0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x6d, 0x65, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x54, 0x61, 0x68, 0x69, 0x74,
0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x6e, 0x75,
0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x6b, 0x65, 0x6c,
0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x54,
0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x54, 0x6f, 0x6b, 0x65, 0x6c,
0x61, 0x75, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x6e, 0x75, 0x74,
0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65,
0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x64, 0x7a, 0x6f,
0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x55, 0x72, 0x75, 0x67,
0x75, 0x61, 0x79, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x20, 0x6e, 0x75, 0x74,
0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x20, 0x64, 0x7a, 0x6f,
0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6e,
0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x6e, 0x75,
0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x64, 0x7a,
0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65,
0x6c, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x6e, 0x75, 0x74,
0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x56, 0x6f, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x56, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x57, 0x61, 0x6b, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x57, 0x61,
0x6c, 0x6c, 0x69, 0x73, 0x20, 0x26, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e,
0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x59,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20,
0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20,
0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x59, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x6e, 0x75, 0x74, 0x6f,
0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x66, 0x67,
0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x74, 0xed, 0xf0, 0x4d,
0x69, 0xf0, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x74, 0xed, 0xf0,
0x45, 0x79, 0x73, 0x74, 0x75, 0x72, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x74, 0xed, 0xf0, 0x53, 0x75, 0xf0, 0x75, 0x72, 0x61, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74,
0xed, 0xf0, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x61, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72,
0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69,
0x67, 0x20, 0x74, 0xed, 0xf0, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x61,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0xed, 0xf0, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x74, 0xed,
0xf0, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c,
0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x41, 0x6d,
0x61, 0x73, 0x6f, 0x6e, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x41, 0x6d, 0x61,
0x73, 0x6f, 0x6e, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20,
0x74, 0xed, 0xf0, 0x41, 0x6d, 0x61, 0x73, 0x6f, 0x6e, 0x61, 0x20, 0x73,
0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x20, 0x74, 0xed, 0xf0, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed,
0xf0, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x73, 0x75, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72,
0x6e, 0x20, 0x74, 0xed, 0xf0, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e,
0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x45,
0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0xed, 0xf0, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e,
0x20, 0x74, 0xed, 0xf0, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e,
0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x4d,
0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x20, 0x73, 0x75, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0xed, 0xf0, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63,
0x20, 0x74, 0xed, 0xf0, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20,
0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x50, 0x61,
0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0xed, 0xf0, 0x41, 0x70, 0x69, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x41,
0x70, 0x69, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74,
0xed, 0xf0, 0x41, 0x70, 0x69, 0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0xed, 0xf0, 0x41, 0x72, 0x61, 0x62, 0x69, 0x73, 0x6b, 0x20,
0x74, 0xed, 0xf0, 0x41, 0x72, 0x61, 0x62, 0x69, 0x73, 0x6b, 0x20, 0x76,
0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x41, 0x72, 0x61,
0x62, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0xed, 0xf0, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20,
0x74, 0xed, 0xf0, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x73, 0x75, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72,
0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x74,
0xed, 0xf0, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x20, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69,
0x67, 0x20, 0x74, 0xed, 0xf0, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x20,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x73, 0x75,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x41, 0x72, 0x6d, 0x65, 0x6e,
0x69, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0,
0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0xed, 0xf0, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x63, 0x20, 0x74, 0xed, 0xf0, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x63, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x20, 0x73, 0x75, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x6d, 0x69, 0xf0, 0x20, 0x41, 0x76,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x6d,
0x69, 0xf0, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x6d,
0x69, 0xf0, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x6d, 0x69,
0xf0, 0x76, 0x65, 0x73, 0x74, 0x75, 0x72, 0x20, 0x41, 0x76, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x6d, 0x69, 0xf0,
0x76, 0x65, 0x73, 0x74, 0x75, 0x72, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20,
0x74, 0xed, 0xf0, 0x6d, 0x69, 0xf0, 0x76, 0x65, 0x73, 0x74, 0x75, 0x72,
0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73,
0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x65, 0x79, 0x73, 0x74,
0x75, 0x72, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x74, 0xed, 0xf0, 0x65, 0x79, 0x73, 0x74, 0x75, 0x72, 0x20, 0x41,
0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x76, 0x61, 0x6e,
0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x65, 0x79, 0x73, 0x74, 0x75,
0x72, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20,
0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x41, 0x73, 0x65,
0x72, 0x62, 0x61, 0x64, 0x6a, 0x61, 0x6e, 0x20, 0x74, 0xed, 0xf0, 0x41,
0x73, 0x65, 0x72, 0x62, 0x61, 0x64, 0x6a, 0x61, 0x6e, 0x20, 0x76, 0x61,
0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x41, 0x73, 0x65, 0x72,
0x62, 0x61, 0x64, 0x6a, 0x61, 0x6e, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0xed, 0xf0, 0x41, 0x7a, 0x6f, 0x72, 0x75, 0x72, 0x6e, 0x61,
0x72, 0x20, 0x74, 0xed, 0xf0, 0x41, 0x7a, 0x6f, 0x72, 0x75, 0x72, 0x6e,
0x61, 0x72, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed,
0xf0, 0x41, 0x7a, 0x6f, 0x72, 0x75, 0x72, 0x6e, 0x61, 0x72, 0x20, 0x73,
0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x6a, 0x20, 0x74, 0xed, 0xf0, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x6a, 0x20, 0x76, 0x61, 0x6e,
0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x73, 0x6a, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0xed, 0xf0, 0x42, 0x75, 0x74, 0x61, 0x6e, 0x20, 0x74, 0xed, 0xf0,
0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x42,
0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x42,
0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c,
0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c,
0x69, 0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0,
0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73,
0x73, 0x61, 0x6c, 0x61, 0x6d, 0x20, 0x74, 0xed, 0xf0, 0x47, 0x72, 0xf8,
0x6e, 0x68, 0xf8, 0x76, 0x64, 0x61, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61,
0x72, 0x20, 0x74, 0xed, 0xf0, 0x47, 0x72, 0xf8, 0x6e, 0x68, 0xf8, 0x76,
0x64, 0x61, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20, 0x76, 0x61,
0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x47, 0x72, 0xf8, 0x6e,
0x68, 0xf8, 0x76, 0x64, 0x61, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72,
0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x43, 0x68,
0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69,
0x67, 0x20, 0x74, 0xed, 0xf0, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x20, 0x74, 0xed, 0xf0, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x20,
0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0xed, 0xf0, 0x4b, 0x69, 0x6c, 0x69, 0x20, 0x74, 0xed, 0xf0, 0x4b,
0x69, 0x6c, 0x69, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74,
0xed, 0xf0, 0x4b, 0x69, 0x6c, 0x69, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0xed, 0xf0, 0x4b, 0x69, 0x6e, 0x61, 0x20, 0x74, 0xed, 0xf0,
0x4b, 0x69, 0x6e, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20,
0x74, 0xed, 0xf0, 0x4b, 0x69, 0x6e, 0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0xed, 0xf0, 0x4a, 0xf3, 0x6c, 0x61, 0x6f, 0x79, 0x67,
0x67, 0x6a, 0x20, 0x74, 0xed, 0xf0, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x6f,
0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20, 0x74, 0xed, 0xf0, 0x4b, 0x6f,
0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x4b, 0x6f,
0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69,
0x67, 0x20, 0x74, 0xed, 0xf0, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69,
0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x43,
0x6f, 0x6f, 0x6b, 0x73, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20,
0x74, 0xed, 0xf0, 0x43, 0x6f, 0x6f, 0x6b, 0x73, 0x6f, 0x79, 0x67, 0x67,
0x6a, 0x61, 0x72, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74,
0xed, 0xf0, 0x43, 0x6f, 0x6f, 0x6b, 0x73, 0x6f, 0x79, 0x67, 0x67, 0x6a,
0x61, 0x72, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0,
0x43, 0x75, 0x62, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x43, 0x75, 0x62, 0x61,
0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x43,
0x75, 0x62, 0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed,
0xf0, 0x44, 0x61, 0x76, 0x69, 0x73, 0x20, 0x74, 0xed, 0xf0, 0x44, 0x75,
0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c,
0x6c, 0x65, 0x20, 0x74, 0xed, 0xf0, 0x45, 0x79, 0x73, 0x74, 0x75, 0x72,
0x74, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x74, 0xed, 0xf0, 0x50, 0xe1, 0x73,
0x6b, 0x61, 0x6f, 0x79, 0x67, 0x67, 0x69, 0x6e, 0x20, 0x74, 0xed, 0xf0,
0x50, 0xe1, 0x73, 0x6b, 0x61, 0x6f, 0x79, 0x67, 0x67, 0x69, 0x6e, 0x20,
0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x50, 0xe1,
0x73, 0x6b, 0x61, 0x6f, 0x79, 0x67, 0x67, 0x69, 0x6e, 0x20, 0x73, 0x75,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x45, 0x6b, 0x76, 0x61, 0x64,
0x6f, 0x72, 0x20, 0x74, 0xed, 0xf0, 0x4d, 0x69, 0xf0, 0x65, 0x76, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x4d, 0x69, 0xf0, 0x65, 0x76,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20,
0x74, 0xed, 0xf0, 0x4d, 0x69, 0xf0, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x45, 0x79,
0x73, 0x74, 0x75, 0x72, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x74,
0xed, 0xf0, 0x45, 0x79, 0x73, 0x74, 0x75, 0x72, 0x65, 0x76, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed,
0xf0, 0x45, 0x79, 0x73, 0x74, 0x75, 0x72, 0x65, 0x76, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x6c,
0x6f, 0x6e, 0x67, 0x72, 0x69, 0x20, 0x45, 0x79, 0x73, 0x74, 0x75, 0x72,
0x65, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x56, 0x65,
0x73, 0x74, 0x75, 0x72, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x74,
0xed, 0xf0, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x65, 0x76, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed,
0xf0, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x65, 0x76, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x46,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6f, 0x79, 0x67, 0x67,
0x6a, 0x61, 0x72, 0x20, 0x74, 0xed, 0xf0, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20,
0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x46, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6f, 0x79, 0x67, 0x67, 0x6a,
0x61, 0x72, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0,
0x46, 0x69, 0x6a, 0x69, 0x20, 0x74, 0xed, 0xf0, 0x46, 0x69, 0x6a, 0x69,
0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x46,
0x69, 0x6a, 0x69, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed,
0xf0, 0x46, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x61, 0x20, 0x47, 0x75, 0x6a,
0x61, 0x6e, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x46, 0x72, 0x6f, 0x6e, 0x73,
0x6b, 0x75, 0x20, 0x73, 0x75, 0x6e, 0x6e, 0x61, 0x72, 0x75, 0x20, 0x6c,
0x61, 0x6e, 0x64, 0x61, 0xf8, 0x6b, 0x69, 0x20, 0x6f, 0x67, 0x20, 0x41,
0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x73, 0x20, 0x74, 0xed, 0xf0,
0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x20, 0x74, 0xed,
0xf0, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x20, 0x74, 0xed, 0xf0,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69,
0x67, 0x20, 0x74, 0xed, 0xf0, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61,
0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x47, 0x69,
0x6c, 0x62, 0x65, 0x72, 0x74, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72,
0x20, 0x74, 0xed, 0xf0, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63,
0x68, 0x20, 0x4d, 0x65, 0x61, 0x6e, 0x20, 0x74, 0xed, 0xf0, 0x45, 0x79,
0x73, 0x74, 0x75, 0x72, 0x20, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x65, 0x6e,
0x64, 0x73, 0x6b, 0x20, 0x74, 0xed, 0xf0, 0x45, 0x79, 0x73, 0x74, 0x75,
0x72, 0x20, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x65, 0x6e, 0x64, 0x73, 0x6b,
0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x45,
0x79, 0x73, 0x74, 0x75, 0x72, 0x20, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x65,
0x6e, 0x64, 0x73, 0x6b, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0xed, 0xf0, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x20, 0x67, 0x72, 0xf8,
0x6e, 0x6c, 0x65, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x74, 0xed, 0xf0, 0x56,
0x65, 0x73, 0x74, 0x75, 0x72, 0x20, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x65,
0x6e, 0x64, 0x73, 0x6b, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20,
0x74, 0xed, 0xf0, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x20, 0x67, 0x72,
0xf8, 0x6e, 0x6c, 0x65, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x73, 0x75, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x47, 0x75, 0x6c, 0x66, 0x20, 0x76,
0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e,
0x20, 0x74, 0xed, 0xf0, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x20, 0x76, 0x61, 0x6e, 0x6c,
0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x20, 0x73, 0x75,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x48, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6e, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x48, 0x6f, 0x6e, 0x67,
0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67,
0x20, 0x74, 0xed, 0xf0, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e,
0x67, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x48,
0x6f, 0x76, 0x64, 0x20, 0x74, 0xed, 0xf0, 0x48, 0x6f, 0x76, 0x64, 0x20,
0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x48, 0x6f,
0x76, 0x64, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x49, 0x6e, 0x64,
0x69, 0x61, 0x68, 0x61, 0x76, 0x20, 0x74, 0xed, 0xf0, 0x49, 0x6e, 0x64,
0x6f, 0x6b, 0x69, 0x6e, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x4d, 0x69, 0xf0,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x74,
0xed, 0xf0, 0x45, 0x79, 0x73, 0x74, 0x75, 0x72, 0x20, 0x49, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x56, 0x65,
0x73, 0x74, 0x75, 0x72, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73,
0x69, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x74,
0xed, 0xf0, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69,
0x67, 0x20, 0x74, 0xed, 0xf0, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x73, 0x75,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x20, 0x74, 0xed, 0xf0, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x73, 0x75, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0xed, 0xf0, 0xcd, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x20,
0x74, 0xed, 0xf0, 0xcd, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x20, 0x76, 0x61,
0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0xcd, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0,
0x4a, 0x61, 0x70, 0x61, 0x6e, 0x20, 0x74, 0xed, 0xf0, 0x4a, 0x61, 0x70,
0x61, 0x6e, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed,
0xf0, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0xed, 0xf0, 0x4b, 0x61, 0x73, 0x61, 0x6b, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x74, 0xed, 0xf0, 0x45, 0x79, 0x73, 0x74, 0x75, 0x72, 0x20,
0x4b, 0x61, 0x73, 0x61, 0x6b, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x74, 0xed,
0xf0, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x20, 0x4b, 0x61, 0x73, 0x61,
0x6b, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x74, 0xed, 0xf0, 0x4b, 0x6f, 0x72,
0x65, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20,
0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x4b, 0x6f,
0x72, 0x65, 0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed,
0xf0, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x20, 0x74, 0xed, 0xf0, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x74,
0xed, 0xf0, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73,
0x6b, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20,
0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x4b, 0x69, 0x72,
0x67, 0x69, 0x73, 0x69, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x4c, 0x69, 0x6e,
0x65, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20, 0x74, 0xed, 0xf0,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x74, 0xed,
0xf0, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x76,
0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0xed, 0xf0, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20,
0x74, 0xed, 0xf0, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x76,
0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0xed, 0xf0, 0x4d, 0x61, 0x6c, 0x61, 0x69, 0x73, 0x69, 0x61, 0x20, 0x74,
0xed, 0xf0, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x6f, 0x79, 0x67, 0x67,
0x6a, 0x61, 0x72, 0x20, 0x74, 0xed, 0xf0, 0x4d, 0x61, 0x72, 0x71, 0x75,
0x65, 0x73, 0x61, 0x73, 0x20, 0x74, 0xed, 0xf0, 0x4d, 0x61, 0x72, 0x73,
0x68, 0x61, 0x6c, 0x6c, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20,
0x74, 0xed, 0xf0, 0x4d, 0xf3, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x20,
0x74, 0xed, 0xf0, 0x4d, 0xf3, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x20,
0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x4d, 0xf3,
0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0xed, 0xf0, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x20, 0x74,
0xed, 0xf0, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61,
0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x74, 0xed, 0xf0, 0x4d, 0x65, 0x78,
0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63,
0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x4d,
0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66,
0x69, 0x63, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0,
0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x74,
0xed, 0xf0, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72,
0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x55,
0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x73, 0x75,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x4d, 0x6f, 0x73, 0x6b, 0x76,
0x61, 0x20, 0x74, 0xed, 0xf0, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20,
0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x4d, 0x6f,
0x73, 0x6b, 0x76, 0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0xed, 0xf0, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x20, 0x28, 0x42,
0x75, 0x72, 0x6d, 0x61, 0x29, 0x20, 0x74, 0xed, 0xf0, 0x4e, 0x61, 0x75,
0x72, 0x75, 0x20, 0x74, 0xed, 0xf0, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x20,
0x74, 0xed, 0xf0, 0x4e, 0xfd, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0xf3, 0x6e,
0x69, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x4e, 0xfd, 0x6b, 0x61, 0x6c, 0x65,
0x64, 0xf3, 0x6e, 0x69, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67,
0x20, 0x74, 0xed, 0xf0, 0x4e, 0xfd, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0xf3,
0x6e, 0x69, 0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed,
0xf0, 0x4e, 0xfd, 0x73, 0xe6, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x74, 0xed,
0xf0, 0x4e, 0xfd, 0x73, 0xe6, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x76, 0x61,
0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x4e, 0xfd, 0x73, 0xe6,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0xed, 0xf0, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x74, 0xed, 0xf0, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75,
0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69,
0x67, 0x20, 0x74, 0xed, 0xf0, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e,
0x64, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0xed, 0xf0, 0x4e, 0x69, 0x75, 0x65, 0x20, 0x74, 0xed, 0xf0, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x20,
0x74, 0xed, 0xf0, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x6f, 0x79,
0x67, 0x67, 0x6a, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74,
0xed, 0xf0, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x6f, 0x79, 0x67,
0x67, 0x6a, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69,
0x67, 0x20, 0x74, 0xed, 0xf0, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x74, 0xed,
0xf0, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x4e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x73,
0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x4f, 0x6d, 0x73, 0x6b,
0x20, 0x74, 0xed, 0xf0, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x76, 0x61, 0x6e,
0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x4f, 0x6d, 0x73, 0x6b, 0x20,
0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x74, 0xed, 0xf0, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67,
0x20, 0x74, 0xed, 0xf0, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x50, 0x61,
0x6c, 0x61, 0x75, 0x20, 0x74, 0xed, 0xf0, 0x50, 0x61, 0x70, 0x75, 0x61,
0x20, 0x4e, 0xfd, 0x67, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x20, 0x74, 0xed,
0xf0, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x69, 0x20, 0x74, 0xed,
0xf0, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x69, 0x20, 0x76, 0x61,
0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x69, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0xed, 0xf0, 0x50, 0x65, 0x72, 0x75, 0x20, 0x74, 0xed, 0xf0, 0x50, 0x65,
0x72, 0x75, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed,
0xf0, 0x50, 0x65, 0x72, 0x75, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0xed, 0xf0, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x73, 0x6f, 0x79, 0x67,
0x67, 0x6a, 0x61, 0x72, 0x20, 0x74, 0xed, 0xf0, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x73, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20, 0x76, 0x61,
0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x73, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20, 0x73, 0x75,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x50, 0x68, 0x6f, 0x65, 0x6e,
0x69, 0x78, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20, 0x74, 0xed,
0xf0, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20,
0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x74,
0xed, 0xf0, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20,
0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x53, 0x74,
0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d,
0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x73, 0x75, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0xed, 0xf0, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72,
0x6e, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20, 0x74, 0xed, 0xf0,
0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x20, 0x74, 0xed, 0xf0, 0x50, 0x79,
0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x52,
0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x20, 0x74, 0xed, 0xf0, 0x52, 0x6f,
0x74, 0x68, 0x65, 0x72, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x53, 0x61, 0x6b,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x74, 0xed, 0xf0, 0x53, 0x61, 0x6b,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67,
0x20, 0x74, 0xed, 0xf0, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0xed, 0xf0, 0x53, 0x65, 0x79, 0x73, 0x6b, 0x65, 0x6c, 0x6c, 0x6f, 0x79,
0x67, 0x67, 0x6a, 0x61, 0x72, 0x20, 0x74, 0xed, 0xf0, 0x53, 0x69, 0x6e,
0x67, 0x61, 0x70, 0x6f, 0x72, 0x20, 0x74, 0xed, 0xf0, 0x53, 0x61, 0x6c,
0x6f, 0x6d, 0x6f, 0x6e, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20,
0x74, 0xed, 0xf0, 0x53, 0x75, 0xf0, 0x75, 0x72, 0x67, 0x65, 0x6f, 0x72,
0x67, 0x69, 0x61, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20, 0x74,
0xed, 0xf0, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x20, 0x74, 0xed,
0xf0, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x54, 0x61,
0x68, 0x69, 0x74, 0x69, 0x20, 0x74, 0xed, 0xf0, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x20, 0x74, 0xed, 0xf0, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69,
0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0xed, 0xf0, 0x54, 0x61, 0x64, 0x73, 0x6a, 0x69, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x74, 0xed, 0xf0, 0x54, 0x6f, 0x6b, 0x65, 0x6c,
0x61, 0x75, 0x20, 0x74, 0xed, 0xf0, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20,
0x74, 0xed, 0xf0, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x76, 0x61, 0x6e,
0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x43, 0x68,
0x75, 0x75, 0x6b, 0x20, 0x74, 0xed, 0xf0, 0x54, 0x75, 0x72, 0x6b, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x74, 0xed, 0xf0, 0x54,
0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x54, 0x75,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x73,
0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x54, 0x75, 0x76, 0x61,
0x6c, 0x75, 0x20, 0x74, 0xed, 0xf0, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61,
0x69, 0x20, 0x74, 0xed, 0xf0, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x69,
0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x69, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0xed, 0xf0, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x74, 0xed, 0xf0, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20,
0x74, 0xed, 0xf0, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x74, 0xed, 0xf0, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67,
0x20, 0x74, 0xed, 0xf0, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20,
0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x56, 0x65, 0x6e,
0x65, 0x73, 0x75, 0x65, 0x6c, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x74, 0xed,
0xf0, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x73,
0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x74, 0xed, 0xf0, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69,
0x67, 0x20, 0x74, 0xed, 0xf0, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0,
0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x74, 0xed, 0xf0, 0x57, 0x61,
0x6b, 0x65, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x20, 0x74, 0xed, 0xf0, 0x57,
0x61, 0x6c, 0x6c, 0x69, 0x73, 0x2d, 0x20, 0x6f, 0x67, 0x20, 0x46, 0x75,
0x74, 0x75, 0x6e, 0x61, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20,
0x74, 0xed, 0xf0, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x74,
0xed, 0xf0, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x76, 0x61,
0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x59, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed,
0xf0, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x76, 0x61, 0x6e, 0x6c,
0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x73, 0x75, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0xed, 0xf0, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x20, 0x74,
0xed, 0xf0, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x47, 0x69,
0x74, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x69, 0x6c, 0x61,
0x6e, 0x67, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x6f,
0x67, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e,
0x67, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e,
0x67, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x6e, 0x67, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e,
0x67, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x44, 0x61,
0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20,
0x73, 0x61, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x6e, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x74,
0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x53, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x20, 0x6e, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x20, 0x6e, 0x61, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74,
0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x69, 0x6c,
0x61, 0x6e, 0x67, 0x61, 0x6e, 0x20, 0x6e, 0x67, 0x20, 0x48, 0x69, 0x6c,
0x61, 0x67, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x69, 0x6c,
0x61, 0x6e, 0x67, 0x61, 0x6e, 0x20, 0x6e, 0x67, 0x20, 0x48, 0x69, 0x6c,
0x61, 0x6e, 0x67, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x69,
0x6c, 0x61, 0x6e, 0x67, 0x61, 0x6e, 0x20, 0x6e, 0x67, 0x20, 0x48, 0x69,
0x6c, 0x61, 0x67, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x42, 0x75,
0x6e, 0x64, 0x6f, 0x6b, 0x20, 0x73, 0x61, 0x20, 0x48, 0x69, 0x6c, 0x61,
0x67, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x42, 0x75, 0x6e, 0x64,
0x6f, 0x6b, 0x20, 0x73, 0x61, 0x20, 0x48, 0x69, 0x6c, 0x61, 0x67, 0x61,
0x6e, 0x67, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x44, 0x61,
0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x42, 0x75, 0x6e, 0x64, 0x6f, 0x6b,
0x20, 0x73, 0x61, 0x20, 0x48, 0x69, 0x6c, 0x61, 0x67, 0x61, 0x6e, 0x67,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x50, 0x61, 0x73, 0x69, 0x70, 0x69, 0x6b, 0x6f,
0x20, 0x73, 0x61, 0x20, 0x48, 0x49, 0x6c, 0x61, 0x67, 0x61, 0x6e, 0x67,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x50, 0x61, 0x73, 0x69, 0x70, 0x69, 0x6b, 0x6f,
0x20, 0x73, 0x61, 0x20, 0x48, 0x69, 0x6c, 0x61, 0x67, 0x61, 0x6e, 0x67,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x44, 0x61, 0x79, 0x6c,
0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x50, 0x61, 0x73, 0x69, 0x70, 0x69, 0x6b, 0x6f,
0x20, 0x73, 0x61, 0x20, 0x48, 0x69, 0x6c, 0x61, 0x67, 0x61, 0x6e, 0x67,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20,
0x73, 0x61, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20,
0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x41, 0x70, 0x69, 0x61, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67,
0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20, 0x41,
0x70, 0x69, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41,
0x72, 0x61, 0x62, 0x69, 0x61, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68,
0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20, 0x41, 0x72,
0x61, 0x62, 0x69, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69,
0x74, 0x20, 0x6e, 0x67, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x61,
0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e, 0x67, 0x20, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x6e, 0x67, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e,
0x67, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e,
0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61,
0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x41, 0x72,
0x6d, 0x65, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x6f, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x6b, 0x6f, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20,
0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x6f, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x47, 0x69, 0x74, 0x6e, 0x61, 0x6e, 0x67, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x47, 0x69, 0x74, 0x6e, 0x61, 0x6e, 0x67,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x44, 0x61,
0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20,
0x73, 0x61, 0x20, 0x47, 0x69, 0x74, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x6e, 0x67, 0x20, 0x47, 0x69, 0x74, 0x6e, 0x61, 0x6e, 0x67, 0x20,
0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e, 0x20, 0x6e, 0x67, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x6e,
0x67, 0x20, 0x47, 0x69, 0x74, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4b, 0x61,
0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e, 0x20, 0x6e, 0x67, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x44, 0x61, 0x79, 0x6c, 0x69,
0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20,
0x47, 0x69, 0x74, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4b, 0x61, 0x6e, 0x6c,
0x75, 0x72, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53,
0x69, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x53, 0x69, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x6e, 0x67, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x44, 0x61, 0x79,
0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73,
0x61, 0x20, 0x53, 0x69, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x6e, 0x67, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61,
0x6e, 0x67, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x61, 0x6e, 0x6c,
0x75, 0x72, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x61, 0x6e, 0x6c,
0x75, 0x72, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41,
0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69,
0x74, 0x20, 0x6e, 0x67, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69,
0x6a, 0x61, 0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67,
0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73,
0x68, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67,
0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43,
0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69,
0x74, 0x20, 0x6e, 0x67, 0x20, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65,
0x72, 0x64, 0x65, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43,
0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x44, 0x61,
0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20,
0x73, 0x61, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6e,
0x61, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x20, 0x73, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d,
0x62, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x43,
0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b,
0x61, 0x6c, 0x61, 0x68, 0x61, 0x74, 0x69, 0x20, 0x6e, 0x67, 0x20, 0x54,
0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x43,
0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x75, 0x62, 0x61, 0x44,
0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x75, 0x62, 0x61, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65,
0x72, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x6e, 0x67, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x47, 0x69, 0x74, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x47,
0x69, 0x74, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67,
0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x47, 0x69, 0x74,
0x6e, 0x61, 0x6e, 0x67, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x69, 0x6c, 0x61, 0x6e,
0x67, 0x61, 0x6e, 0x67, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x69, 0x6c, 0x61, 0x6e,
0x67, 0x61, 0x6e, 0x67, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x53, 0x69, 0x6c, 0x61, 0x6e,
0x67, 0x61, 0x6e, 0x67, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x69, 0x6e, 0x61, 0x6b,
0x61, 0x2d, 0x73, 0x69, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x6e, 0x67, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e, 0x67, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e, 0x67, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e,
0x67, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e, 0x67, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e,
0x67, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e,
0x67, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x4b, 0x61, 0x74, 0x69, 0x6d, 0x75, 0x67, 0x61, 0x6e, 0x67,
0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x20, 0x61, 0x74, 0x20, 0x41,
0x6e, 0x74, 0x61, 0x72, 0x74, 0x69, 0x6b, 0x6f, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x69,
0x6c, 0x61, 0x6e, 0x67, 0x61, 0x6e, 0x67, 0x20, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x53, 0x69, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x6e, 0x67, 0x20, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x6e, 0x67, 0x20, 0x53, 0x69, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x6e,
0x67, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75,
0x72, 0x61, 0x6e, 0x67, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x61,
0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e, 0x67, 0x20, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67,
0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e, 0x67, 0x20, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20,
0x6e, 0x67, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6e, 0x67, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x48, 0x6f, 0x76, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20,
0x48, 0x6f, 0x76, 0x64, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x47, 0x69, 0x74, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x49, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x53, 0x69, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x6e, 0x67, 0x20,
0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61,
0x6e, 0x67, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x49, 0x72, 0x61, 0x6e,
0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x20, 0x73, 0x61, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d,
0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x49,
0x73, 0x72, 0x61, 0x65, 0x6c, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68,
0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20, 0x49, 0x73,
0x72, 0x61, 0x65, 0x6c, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x4a, 0x61, 0x70, 0x61, 0x6e, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68,
0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20, 0x4a, 0x61,
0x70, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x20, 0x73, 0x61, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76,
0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61,
0x74, 0x73, 0x6b, 0x69, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x6e, 0x67, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61,
0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x69, 0x6c,
0x61, 0x6e, 0x67, 0x61, 0x6e, 0x67, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b,
0x68, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e, 0x67, 0x20, 0x4b,
0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x44, 0x61,
0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20,
0x73, 0x61, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x79, 0x72, 0x67, 0x79, 0x73,
0x74, 0x61, 0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x44, 0x61, 0x79, 0x6c,
0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61,
0x20, 0x4c, 0x6f, 0x72, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61,
0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e,
0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74,
0x69, 0x75, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50,
0x61, 0x73, 0x69, 0x70, 0x69, 0x6b, 0x6f, 0x20, 0x6e, 0x67, 0x20, 0x4d,
0x65, 0x78, 0x69, 0x63, 0x6f, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x50, 0x61, 0x73, 0x69, 0x70, 0x69, 0x6b, 0x6f, 0x20, 0x6e, 0x67,
0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f, 0x44, 0x61, 0x79, 0x6c, 0x69,
0x67, 0x68, 0x74, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x50, 0x61, 0x73, 0x69, 0x70, 0x69, 0x6b, 0x6f, 0x20,
0x6e, 0x67, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x55,
0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69,
0x74, 0x20, 0x6e, 0x67, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4d,
0x6f, 0x73, 0x63, 0x6f, 0x77, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67,
0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20,
0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69,
0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x65, 0x77,
0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x44, 0x61, 0x79, 0x6c,
0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61,
0x20, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x66,
0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x44, 0x61, 0x79, 0x6c,
0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d,
0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e,
0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67,
0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69,
0x74, 0x20, 0x6e, 0x67, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x61, 0x72,
0x61, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67,
0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x65, 0x72, 0x75, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69,
0x74, 0x20, 0x6e, 0x67, 0x20, 0x50, 0x65, 0x72, 0x75, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e,
0x61, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61,
0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x50, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x61,
0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x44, 0x61, 0x79,
0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x6e, 0x67, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20, 0x44, 0x61, 0x79, 0x6c,
0x69, 0x67, 0x68, 0x74, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68,
0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53,
0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x67, 0x20, 0x47, 0x65,
0x6f, 0x72, 0x67, 0x69, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x44, 0x61, 0x79, 0x6c, 0x69,
0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20,
0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20,
0x6e, 0x67, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x55, 0x72, 0x75, 0x67,
0x75, 0x61, 0x79, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d,
0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x55, 0x7a, 0x62, 0x65,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69,
0x74, 0x20, 0x6e, 0x67, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20,
0x6e, 0x67, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20,
0x6e, 0x67, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x59, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x59,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x59,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x41, 0x63, 0x72, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x63,
0x72, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69,
0x61, 0x69, 0x6b, 0x61, 0x41, 0x63, 0x72, 0x65, 0x6e, 0x20, 0x6b, 0x65,
0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4b,
0x65, 0x73, 0x6b, 0x69, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x74, 0xe4, 0x2d, 0x41, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x45, 0x74, 0x65,
0x6c, 0xe4, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x73, 0x69, 0x2d, 0x41, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e,
0x73, 0x69, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4c,
0xe4, 0x6e, 0x73, 0x69, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c,
0x69, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x6e,
0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x6c, 0x6d,
0x61, 0x74, 0x79, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x6c, 0x6d,
0x61, 0x74, 0x79, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c,
0x69, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x6e,
0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61,
0x59, 0x68, 0x64, 0x79, 0x73, 0x76, 0x61, 0x6c, 0x74, 0x61, 0x69, 0x6e,
0x20, 0x6b, 0x65, 0x73, 0x6b, 0x69, 0x6e, 0x65, 0x6e, 0x20, 0x61, 0x69,
0x6b, 0x61, 0x59, 0x68, 0x64, 0x79, 0x73, 0x76, 0x61, 0x6c, 0x74, 0x61,
0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0x6b, 0x69, 0x6e, 0x65, 0x6e, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61,
0x59, 0x68, 0x64, 0x79, 0x73, 0x76, 0x61, 0x6c, 0x74, 0x61, 0x69, 0x6e,
0x20, 0x6b, 0x65, 0x73, 0x6b, 0x69, 0x6e, 0x65, 0x6e, 0x20, 0x6b, 0x65,
0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x59, 0x68, 0x64, 0x79, 0x73, 0x76,
0x61, 0x6c, 0x74, 0x61, 0x69, 0x6e, 0x20, 0x69, 0x74, 0xe4, 0x69, 0x6e,
0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x59, 0x68, 0x64, 0x79, 0x73,
0x76, 0x61, 0x6c, 0x74, 0x61, 0x69, 0x6e, 0x20, 0x69, 0x74, 0xe4, 0x69,
0x6e, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69,
0x61, 0x69, 0x6b, 0x61, 0x59, 0x68, 0x64, 0x79, 0x73, 0x76, 0x61, 0x6c,
0x74, 0x61, 0x69, 0x6e, 0x20, 0x69, 0x74, 0xe4, 0x69, 0x6e, 0x65, 0x6e,
0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x61, 0x6c,
0x6c, 0x69, 0x6f, 0x76, 0x75, 0x6f, 0x72, 0x74, 0x65, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x4b, 0x61, 0x6c, 0x6c, 0x69, 0x6f, 0x76, 0x75, 0x6f,
0x72, 0x74, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c,
0x69, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x61, 0x6c, 0x6c, 0x69, 0x6f, 0x76,
0x75, 0x6f, 0x72, 0x74, 0x65, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61,
0x69, 0x6b, 0x61, 0x59, 0x68, 0x64, 0x79, 0x73, 0x76, 0x61, 0x6c, 0x74,
0x61, 0x69, 0x6e, 0x20, 0x54, 0x79, 0x79, 0x6e, 0x65, 0x6e, 0x6d, 0x65,
0x72, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x59, 0x68, 0x64, 0x79,
0x73, 0x76, 0x61, 0x6c, 0x74, 0x61, 0x69, 0x6e, 0x20, 0x54, 0x79, 0x79,
0x6e, 0x65, 0x6e, 0x6d, 0x65, 0x72, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x59, 0x68, 0x64,
0x79, 0x73, 0x76, 0x61, 0x6c, 0x74, 0x61, 0x69, 0x6e, 0x20, 0x54, 0x79,
0x79, 0x6e, 0x65, 0x6e, 0x6d, 0x65, 0x72, 0x65, 0x6e, 0x20, 0x6b, 0x65,
0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x6e, 0x61, 0x64, 0x79,
0x72, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69,
0x61, 0x69, 0x6b, 0x61, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x69, 0x6e,
0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x70, 0x69,
0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x70, 0x69, 0x61, 0x6e,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b,
0x61, 0x41, 0x70, 0x69, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61,
0x69, 0x6b, 0x61, 0x41, 0x71, 0x74, 0x61, 0x77, 0x2019, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x41, 0x71, 0x74, 0x61, 0x77, 0x2019, 0x6e, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x41,
0x71, 0x74, 0x61, 0x77, 0x2019, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61,
0x69, 0x6b, 0x61, 0x41, 0x71, 0x74, 0xf6, 0x62, 0x65, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x41, 0x71, 0x74, 0xf6, 0x62, 0x65, 0x6e, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x41,
0x71, 0x74, 0xf6, 0x62, 0x65, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61,
0x69, 0x6b, 0x61, 0x53, 0x61, 0x75, 0x64, 0x69, 0x2d, 0x41, 0x72, 0x61,
0x62, 0x69, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x53, 0x61, 0x75,
0x64, 0x69, 0x2d, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x6e, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x53,
0x61, 0x75, 0x64, 0x69, 0x2d, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x6e,
0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x61, 0x6e,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b,
0x61, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x61, 0x6e,
0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e,
0x73, 0x69, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e,
0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x73, 0x69,
0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x61, 0x6e,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b,
0x61, 0x4c, 0xe4, 0x6e, 0x73, 0x69, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x69, 0x6e, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61,
0x69, 0x6b, 0x61, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x6e,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b,
0x61, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x6e, 0x20, 0x6b, 0x65,
0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x61, 0x6e, 0x61, 0x64, 0x61,
0x6e, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x4b, 0x61, 0x6e, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x61, 0x6e, 0x61,
0x64, 0x61, 0x6e, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6e,
0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x65, 0x73,
0x6b, 0x69, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x65, 0x73, 0x6b, 0x69, 0x2d,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4b,
0x65, 0x73, 0x6b, 0x69, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61,
0x4c, 0xe4, 0x6e, 0x74, 0x69, 0x73, 0x65, 0x6e, 0x20, 0x4b, 0x65, 0x73,
0x6b, 0x69, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x74, 0x69, 0x73,
0x65, 0x6e, 0x20, 0x4b, 0x65, 0x73, 0x6b, 0x69, 0x2d, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x74,
0x69, 0x73, 0x65, 0x6e, 0x20, 0x4b, 0x65, 0x73, 0x6b, 0x69, 0x2d, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x6b, 0x65,
0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x74, 0xe4, 0x2d, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x49, 0x74, 0xe4, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69,
0x61, 0x69, 0x6b, 0x61, 0x49, 0x74, 0xe4, 0x2d, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61,
0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x73, 0x69, 0x2d, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x4c, 0xe4, 0x6e, 0x73, 0x69, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c,
0x69, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x73, 0x69, 0x2d, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x6b, 0x65,
0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61,
0x69, 0x64, 0x17e, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x17e, 0x61, 0x6e, 0x69,
0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69,
0x6b, 0x61, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x17e, 0x61,
0x6e, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61,
0x41, 0x7a, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x41, 0x7a, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x7a, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x69, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x73, 0x68, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61,
0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x73, 0x68, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61,
0x69, 0x6b, 0x61, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x69, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69,
0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x42, 0x72, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c,
0x69, 0x61, 0x69, 0x6b, 0x61, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69,
0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x42,
0x72, 0x75, 0x6e, 0x65, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4b,
0x61, 0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x6e, 0x20, 0x61, 0x69,
0x6b, 0x61, 0x4b, 0x61, 0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x6e,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b,
0x61, 0x4b, 0x61, 0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x6e, 0x20,
0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x43, 0x61, 0x73, 0x65,
0x79, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x54, 0x161, 0x61, 0x6d, 0x6f,
0x72, 0x72, 0x6f, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x43, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x43,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x43, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61,
0x69, 0x6b, 0x61, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x6e, 0x20, 0x61, 0x69,
0x6b, 0x61, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x43, 0x68, 0x69,
0x6c, 0x65, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61,
0x4b, 0x69, 0x69, 0x6e, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4b,
0x69, 0x69, 0x6e, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61,
0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x69, 0x69, 0x6e, 0x61, 0x6e,
0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4a, 0x6f, 0x75,
0x6c, 0x75, 0x73, 0x61, 0x61, 0x72, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x4b, 0x6f, 0x6f, 0x6b, 0x6f, 0x73, 0x73, 0x61, 0x61, 0x72, 0x74,
0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x6f, 0x6c, 0x75, 0x6d,
0x62, 0x69, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x6f, 0x6c,
0x75, 0x6d, 0x62, 0x69, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x6f, 0x6c, 0x75, 0x6d,
0x62, 0x69, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b,
0x61, 0x43, 0x6f, 0x6f, 0x6b, 0x69, 0x6e, 0x73, 0x61, 0x61, 0x72, 0x74,
0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x43, 0x6f, 0x6f, 0x6b, 0x69,
0x6e, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x43, 0x6f, 0x6f,
0x6b, 0x69, 0x6e, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x6e, 0x20, 0x6b,
0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x75, 0x75, 0x62, 0x61,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x75, 0x75, 0x62, 0x61, 0x6e,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b,
0x61, 0x4b, 0x75, 0x75, 0x62, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4,
0x61, 0x69, 0x6b, 0x61, 0x44, 0x61, 0x76, 0x69, 0x73, 0x69, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x20, 0x64,
0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x6e, 0x20, 0x61, 0x69,
0x6b, 0x61, 0x49, 0x74, 0xe4, 0x2d, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x69,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x50, 0xe4, 0xe4, 0x73, 0x69, 0xe4,
0x69, 0x73, 0x73, 0x61, 0x61, 0x72, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x50, 0xe4, 0xe4, 0x73, 0x69, 0xe4, 0x69, 0x73, 0x73, 0x61, 0x61,
0x72, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69,
0x61, 0x69, 0x6b, 0x61, 0x50, 0xe4, 0xe4, 0x73, 0x69, 0xe4, 0x69, 0x73,
0x73, 0x61, 0x61, 0x72, 0x65, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61,
0x69, 0x6b, 0x61, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x69, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x65, 0x73, 0x6b, 0x69, 0x2d, 0x45,
0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x4b, 0x65, 0x73, 0x6b, 0x69, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x6f, 0x70,
0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61,
0x69, 0x6b, 0x61, 0x4b, 0x65, 0x73, 0x6b, 0x69, 0x2d, 0x45, 0x75, 0x72,
0x6f, 0x6f, 0x70, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69,
0x6b, 0x61, 0x49, 0x74, 0xe4, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x6f, 0x70,
0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x74, 0xe4, 0x2d, 0x45,
0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x74, 0xe4, 0x2d,
0x45, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0xe4, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x74, 0xe4, 0x69, 0x73, 0x65, 0x6d,
0x6d, 0xe4, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x73, 0x69, 0x2d, 0x45,
0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x4c, 0xe4, 0x6e, 0x73, 0x69, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x6f, 0x70,
0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61,
0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x73, 0x69, 0x2d, 0x45, 0x75, 0x72,
0x6f, 0x6f, 0x70, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69,
0x6b, 0x61, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x6e,
0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x6e, 0x73, 0x61,
0x61, 0x72, 0x74, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61,
0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x6e, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x6e, 0x20,
0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x46, 0x69, 0x64, 0x17e,
0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x46, 0x69, 0x64, 0x17e, 0x69,
0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69,
0x6b, 0x61, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0xe4, 0x61, 0x69, 0x6b, 0x61, 0x52, 0x61, 0x6e, 0x73, 0x6b, 0x61, 0x6e,
0x20, 0x47, 0x75, 0x61, 0x79, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x61, 0x69,
0x6b, 0x61, 0x52, 0x61, 0x6e, 0x73, 0x6b, 0x61, 0x6e, 0x20, 0x65, 0x74,
0x65, 0x6c, 0xe4, 0x69, 0x73, 0x74, 0x65, 0x6e, 0x20, 0x6a, 0x61, 0x20,
0x61, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x73, 0x74, 0x65, 0x6e,
0x20, 0x61, 0x6c, 0x75, 0x65, 0x69, 0x64, 0x65, 0x6e, 0x20, 0x61, 0x69,
0x6b, 0x61, 0x47, 0x61, 0x6c, 0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x73,
0x61, 0x61, 0x72, 0x74, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x47,
0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x47, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69,
0x61, 0x69, 0x6b, 0x61, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x6e,
0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x47, 0x69, 0x6c,
0x62, 0x65, 0x72, 0x74, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63,
0x68, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69,
0x61, 0x69, 0x6b, 0x61, 0x49, 0x74, 0xe4, 0x2d, 0x47, 0x72, 0xf6, 0x6e,
0x6c, 0x61, 0x6e, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x49,
0x74, 0xe4, 0x2d, 0x47, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x6e, 0x69,
0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69,
0x6b, 0x61, 0x49, 0x74, 0xe4, 0x2d, 0x47, 0x72, 0xf6, 0x6e, 0x6c, 0x61,
0x6e, 0x6e, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b,
0x61, 0x4c, 0xe4, 0x6e, 0x73, 0x69, 0x2d, 0x47, 0x72, 0xf6, 0x6e, 0x6c,
0x61, 0x6e, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4,
0x6e, 0x73, 0x69, 0x2d, 0x47, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x6e,
0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61,
0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x73, 0x69, 0x2d, 0x47, 0x72, 0xf6,
0x6e, 0x6c, 0x61, 0x6e, 0x6e, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4,
0x61, 0x69, 0x6b, 0x61, 0x47, 0x75, 0x61, 0x6d, 0x69, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x41, 0x72, 0x61, 0x62, 0x69, 0x65, 0x6d, 0x69, 0x69,
0x72, 0x69, 0x6b, 0x75, 0x6e, 0x74, 0x69, 0x65, 0x6e, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x47, 0x75,
0x79, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x48, 0x61,
0x76, 0x61, 0x69, 0x6a, 0x69, 0x6e, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x74, 0x69, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x48, 0x61, 0x76,
0x61, 0x69, 0x6a, 0x69, 0x6e, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x74,
0x69, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69,
0x61, 0x69, 0x6b, 0x61, 0x48, 0x61, 0x76, 0x61, 0x69, 0x6a, 0x69, 0x6e,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x74, 0x69, 0x65, 0x6e, 0x20, 0x6b,
0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x48, 0x6f, 0x6e, 0x67, 0x6b,
0x6f, 0x6e, 0x67, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x48, 0x6f,
0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x48, 0x6f, 0x6e,
0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4,
0x61, 0x69, 0x6b, 0x61, 0x48, 0x6f, 0x76, 0x64, 0x69, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x48, 0x6f, 0x76, 0x64, 0x69, 0x6e, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x48, 0x6f,
0x76, 0x64, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b,
0x61, 0x49, 0x6e, 0x74, 0x69, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x49, 0x6e, 0x74, 0x69, 0x61, 0x6e, 0x20, 0x76, 0x61, 0x6c, 0x74, 0x61,
0x6d, 0x65, 0x72, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x6e,
0x64, 0x6f, 0x6b, 0x69, 0x69, 0x6e, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x4b, 0x65, 0x73, 0x6b, 0x69, 0x2d, 0x49, 0x6e, 0x64, 0x6f, 0x6e,
0x65, 0x73, 0x69, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x74,
0xe4, 0x2d, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x73, 0x69, 0x2d, 0x49,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x6e, 0x20, 0x61, 0x69,
0x6b, 0x61, 0x49, 0x72, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x49, 0x72, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x72, 0x61, 0x6e,
0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x49,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x6e, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x49,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0xe4, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x69,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61,
0x69, 0x6b, 0x61, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x6e, 0x20,
0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4a, 0x61, 0x70, 0x61,
0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4a, 0x61, 0x70, 0x61,
0x6e, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69,
0x61, 0x69, 0x6b, 0x61, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x69, 0x6e, 0x20,
0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x61, 0x6d, 0x74,
0x161, 0x61, 0x74, 0x6b, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4b,
0x61, 0x6d, 0x74, 0x161, 0x61, 0x74, 0x6b, 0x61, 0x6e, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x61,
0x6d, 0x74, 0x161, 0x61, 0x74, 0x6b, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x74, 0xe4,
0x2d, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x73, 0x69, 0x2d, 0x4b,
0x61, 0x7a, 0x61, 0x6b, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x20, 0x61, 0x69,
0x6b, 0x61, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x6f, 0x72,
0x65, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61,
0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x69,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x6a, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x6e, 0x20, 0x6b, 0x65,
0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x69, 0x72, 0x67, 0x69, 0x73,
0x69, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x53, 0x72, 0x69, 0x20,
0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4c,
0x69, 0x6e, 0x65, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c,
0x69, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61,
0x4d, 0x61, 0x63, 0x61, 0x6f, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4d,
0x61, 0x63, 0x61, 0x6f, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61,
0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x6e,
0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61,
0x69, 0x6b, 0x61, 0x4d, 0x61, 0x6c, 0x65, 0x73, 0x69, 0x61, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x4d, 0x61, 0x6c, 0x65, 0x64, 0x69, 0x69, 0x76,
0x69, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x61, 0x72, 0x71,
0x75, 0x65, 0x73, 0x61, 0x73, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c,
0x6c, 0x69, 0x6e, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x6b,
0x73, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x74, 0x69, 0x75, 0x6b, 0x73, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x74, 0x69, 0x75, 0x6b, 0x73, 0x65, 0x6e, 0x20, 0x6b, 0x65,
0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e,
0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x65, 0x6b, 0x73, 0x69,
0x6b, 0x6f, 0x6e, 0x20, 0x54, 0x79, 0x79, 0x6e, 0x65, 0x6e, 0x6d, 0x65,
0x72, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x65, 0x6b, 0x73,
0x69, 0x6b, 0x6f, 0x6e, 0x20, 0x54, 0x79, 0x79, 0x6e, 0x65, 0x6e, 0x6d,
0x65, 0x72, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c,
0x69, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f,
0x6e, 0x20, 0x54, 0x79, 0x79, 0x6e, 0x65, 0x6e, 0x6d, 0x65, 0x72, 0x65,
0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x55, 0x6c,
0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x69, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f,
0x72, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69,
0x61, 0x69, 0x6b, 0x61, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74,
0x6f, 0x72, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b,
0x61, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x61, 0x6e, 0x20, 0x61, 0x69,
0x6b, 0x61, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x61, 0x6e, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4d,
0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4,
0x61, 0x69, 0x6b, 0x61, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x69,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x69, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x55, 0x75, 0x64, 0x65, 0x6e, 0x2d, 0x4b,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x6e, 0x20, 0x61, 0x69,
0x6b, 0x61, 0x55, 0x75, 0x64, 0x65, 0x6e, 0x2d, 0x4b, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x69, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x55, 0x75, 0x64, 0x65, 0x6e,
0x2d, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x6e, 0x20,
0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x55, 0x75, 0x64, 0x65,
0x6e, 0x2d, 0x53, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x6e, 0x69, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x55, 0x75, 0x64, 0x65, 0x6e, 0x2d, 0x53, 0x65,
0x65, 0x6c, 0x61, 0x6e, 0x6e, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x55, 0x75, 0x64, 0x65,
0x6e, 0x2d, 0x53, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x6e, 0x69, 0x6e, 0x20,
0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4e, 0x65, 0x77, 0x66,
0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61,
0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75,
0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4e, 0x69, 0x75, 0x65, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x69, 0x6e,
0x73, 0x61, 0x61, 0x72, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x69, 0x6e, 0x73, 0x61, 0x61, 0x72,
0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61,
0x69, 0x6b, 0x61, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x69, 0x6e,
0x73, 0x61, 0x61, 0x72, 0x65, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61,
0x69, 0x6b, 0x61, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x6e,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b,
0x61, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x6e, 0x20, 0x6b, 0x65,
0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x50, 0x6f, 0x68, 0x6a, 0x6f, 0x69,
0x73, 0x2d, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x61, 0x6e, 0x69, 0x65, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x69, 0x6e,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b,
0x61, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4f,
0x6d, 0x73, 0x6b, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4f, 0x6d,
0x73, 0x6b, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c,
0x69, 0x61, 0x69, 0x6b, 0x61, 0x4f, 0x6d, 0x73, 0x6b, 0x69, 0x6e, 0x20,
0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x50, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0xe4, 0x61, 0x69, 0x6b, 0x61, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x50, 0x61, 0x70, 0x75, 0x61, 0x2d, 0x55, 0x75,
0x64, 0x65, 0x6e, 0x2d, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69,
0x61, 0x69, 0x6b, 0x61, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79,
0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x50, 0x65,
0x72, 0x75, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x50, 0x65, 0x72, 0x75,
0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69,
0x6b, 0x61, 0x50, 0x65, 0x72, 0x75, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4,
0x61, 0x69, 0x6b, 0x61, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x69,
0x6e, 0x69, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x46, 0x69, 0x6c,
0x69, 0x70, 0x70, 0x69, 0x69, 0x6e, 0x69, 0x65, 0x6e, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x70, 0x69, 0x69, 0x6e, 0x69, 0x65, 0x6e, 0x20, 0x6b,
0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x50, 0x68, 0x6f, 0x65, 0x6e,
0x69, 0x78, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x6e, 0x20, 0x61, 0x69,
0x6b, 0x61, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x6e, 0x20, 0x6a, 0x61, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x53, 0x61,
0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x6e, 0x20,
0x6a, 0x61, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x69,
0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69,
0x6b, 0x61, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x6e, 0x20, 0x6a, 0x61, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69,
0x6b, 0x61, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x69, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x50, 0x6f, 0x68, 0x6e, 0x70, 0x65, 0x69,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x50, 0x6a, 0x6f, 0x6e, 0x67, 0x6a,
0x61, 0x6e, 0x67, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x51, 0x131,
0x7a, 0x131, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x51, 0x131, 0x7a, 0x131, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x6e, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61,
0x51, 0x131, 0x7a, 0x131, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x6e, 0x20, 0x6b,
0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x52, 0xe9, 0x75, 0x6e, 0x69,
0x6f, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x52, 0x6f, 0x74,
0x68, 0x65, 0x72, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x53, 0x61,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x6e, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x53, 0x61,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4,
0x61, 0x69, 0x6b, 0x61, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x6e, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61,
0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4,
0x61, 0x69, 0x6b, 0x61, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b,
0x61, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x69, 0x65, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f,
0x72, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x53, 0x61, 0x6c, 0x6f,
0x6d, 0x6f, 0x6e, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x45, 0x74, 0x65, 0x6c, 0xe4, 0x2d, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x53, 0x75,
0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x53, 0x79, 0x6f, 0x77, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x54,
0x61, 0x68, 0x69, 0x74, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x54,
0x61, 0x64, 0x17e, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x6e, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61,
0x69, 0x6b, 0x61, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x69, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x54, 0x75,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x6e,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b,
0x61, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61,
0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x6e, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x79, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61,
0x69, 0x6b, 0x61, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x55, 0x7a, 0x62, 0x65,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x55, 0x7a, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x6b, 0x65,
0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69,
0x61, 0x69, 0x6b, 0x61, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x6e,
0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x56, 0x65, 0x6e,
0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x69,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x69, 0x6e, 0x20, 0x6b, 0x65,
0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x69, 0x6e, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x69, 0x6e, 0x20, 0x6b, 0x65,
0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x6b, 0x65, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x69,
0x6e, 0x20, 0x6a, 0x61, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4a, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c,
0x69, 0x61, 0x69, 0x6b, 0x61, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4a,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4a, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x69, 0x6e, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4a,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x59,
0x75, 0x6b, 0x6f, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x63,
0x72, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x63, 0x72,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x63, 0x72, 0x65, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x66, 0x67,
0x68, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019,
0x41, 0x66, 0x72, 0x69, 0x71, 0x75, 0x65, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x66, 0x72, 0x69,
0x71, 0x75, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x71, 0x75, 0x65, 0x20,
0x6d, 0xe9, 0x72, 0x69, 0x64, 0x69, 0x6f, 0x6e, 0x61, 0x6c, 0x65, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x71,
0x75, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73,
0x74, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x71, 0x75, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64,
0x2019, 0x41, 0x66, 0x72, 0x69, 0x71, 0x75, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x6c, 0x6d, 0x61,
0x20, 0x41, 0x74, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x6c, 0x6d, 0x61,
0x20, 0x41, 0x74, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019,
0xe9, 0x74, 0xe9, 0x20, 0x64, 0x2019, 0x41, 0x6c, 0x6d, 0x61, 0x20, 0x41,
0x74, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x69, 0x65, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x69, 0x65,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x69,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d,
0xe9, 0x72, 0x69, 0x63, 0x61, 0x69, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x75, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d,
0x61, 0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61, 0x69, 0x6e, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x75, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d,
0x61, 0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61, 0x69, 0x6e, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74, 0x20,
0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61,
0x69, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74,
0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0xe9, 0x72, 0x69, 0x63,
0x61, 0x69, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9,
0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74, 0x20,
0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61,
0x69, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20,
0x52, 0x6f, 0x63, 0x68, 0x65, 0x75, 0x73, 0x65, 0x73, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x73, 0x20, 0x52, 0x6f, 0x63, 0x68, 0x65, 0x75, 0x73, 0x65, 0x73,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20,
0x64, 0x65, 0x73, 0x20, 0x52, 0x6f, 0x63, 0x68, 0x65, 0x75, 0x73, 0x65,
0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x50, 0x61,
0x63, 0x69, 0x66, 0x69, 0x71, 0x75, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x64,
0x2d, 0x61, 0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61, 0x69, 0x6e, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x75, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x71, 0x75, 0x65,
0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0xe9, 0x72, 0x69, 0x63,
0x61, 0x69, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9,
0x74, 0xe9, 0x20, 0x64, 0x75, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69,
0x71, 0x75, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0xe9,
0x72, 0x69, 0x63, 0x61, 0x69, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x2019, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019,
0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x2019, 0x41, 0x6e, 0x61, 0x64,
0x79, 0x72, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x70,
0x69, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x70, 0x69, 0x61, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x2019,
0x41, 0x70, 0x69, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019,
0x41, 0x6b, 0x74, 0x61, 0x6f, 0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x6b,
0x74, 0x61, 0x6f, 0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019,
0xe9, 0x74, 0xe9, 0x20, 0x64, 0x2019, 0x41, 0x6b, 0x74, 0x61, 0x6f, 0x75,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x71, 0x74, 0xf6,
0x62, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x71, 0x74, 0xf6, 0x62, 0x65,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20,
0x64, 0x2019, 0x41, 0x71, 0x74, 0xf6, 0x62, 0x65, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x61, 0x62, 0x69,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x61, 0x62,
0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74,
0xe9, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x61, 0x62, 0x69,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x65, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x65, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x65,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f,
0x75, 0x65, 0x73, 0x74, 0x20, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73,
0x74, 0x20, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x20, 0x61, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x6d, 0xe9, 0x6e, 0x69, 0x65, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x6d, 0xe9, 0x6e, 0x69,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9,
0x20, 0x64, 0x2019, 0x41, 0x72, 0x6d, 0xe9, 0x6e, 0x69, 0x65, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x71, 0x75, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x71, 0x75, 0x65,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x71, 0x75, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x75, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64,
0x75, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x65, 0x2d, 0x6f, 0x75, 0x65, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x75, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x2d, 0x6f,
0x75, 0x65, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x75, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x65, 0x2d, 0x6f, 0x75, 0x65, 0x73, 0x74, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x45, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x45, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x45, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73,
0x74, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019,
0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65,
0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0xef, 0x64,
0x6a, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x61, 0xef, 0x64, 0x6a, 0x61, 0x6e, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x2019, 0x41,
0x7a, 0x65, 0x72, 0x62, 0x61, 0xef, 0x64, 0x6a, 0x61, 0x6e, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0x41, 0xe7, 0x6f, 0x72,
0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0x41, 0xe7, 0x6f, 0x72,
0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74,
0xe9, 0x20, 0x64, 0x65, 0x73, 0x20, 0x41, 0xe7, 0x6f, 0x72, 0x65, 0x73,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x75, 0x20,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x75,
0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x42, 0x68, 0x6f, 0x75,
0x74, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20,
0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69,
0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9,
0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x42, 0x72, 0x75,
0x6e, 0x65, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20,
0x43, 0x61, 0x70, 0x2d, 0x56, 0x65, 0x72, 0x74, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x75,
0x20, 0x43, 0x61, 0x70, 0x2d, 0x56, 0x65, 0x72, 0x74, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x75, 0x20,
0x43, 0x61, 0x70, 0x2d, 0x56, 0x65, 0x72, 0x74, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x73, 0x65, 0x79, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0x43, 0x68, 0x61, 0x6d,
0x6f, 0x72, 0x72, 0x6f, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65,
0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0xee, 0x6c, 0x65, 0x73,
0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x73, 0x20,
0xee, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x43, 0x68, 0x69,
0x6c, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x75, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x69,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20,
0x64, 0x75, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x69, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6e,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x43, 0x68, 0x69,
0x6e, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74,
0xe9, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x65, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xee, 0x6c, 0x65,
0x20, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0xee, 0x6c, 0x65, 0x73,
0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x65, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x65,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20,
0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x65, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0xee, 0x6c, 0x65,
0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20,
0xee, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x73,
0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x43,
0x75, 0x62, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20,
0x44, 0x61, 0x76, 0x69, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x65, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55,
0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x75, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0xee, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x50,
0xe2, 0x71, 0x75, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0xee, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x50, 0xe2, 0x71, 0x75, 0x65,
0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xee, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x50, 0xe2, 0x71, 0x75, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xc9, 0x71, 0x75, 0x61, 0x74, 0x65,
0x75, 0x72, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x2019, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x6c,
0x69, 0x6e, 0x69, 0x6e, 0x67, 0x72, 0x61, 0x64, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73,
0x74, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0xee,
0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x6f, 0x75, 0x69, 0x6e, 0x65,
0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20,
0x4d, 0x61, 0x6c, 0x6f, 0x75, 0x69, 0x6e, 0x65, 0x73, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x73,
0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x6f, 0x75, 0x69,
0x6e, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73,
0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x46, 0x69, 0x64, 0x6a, 0x69, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x46, 0x69,
0x64, 0x6a, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9,
0x74, 0xe9, 0x20, 0x64, 0x65, 0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20,
0x46, 0x69, 0x64, 0x6a, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x65, 0x20,
0x66, 0x72, 0x61, 0x6e, 0xe7, 0x61, 0x69, 0x73, 0x65, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0x54, 0x65, 0x72, 0x72, 0x65,
0x73, 0x20, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x73, 0x20,
0x65, 0x74, 0x20, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x63, 0x74, 0x69, 0x71,
0x75, 0x65, 0x73, 0x20, 0x66, 0x72, 0x61, 0x6e, 0xe7, 0x61, 0x69, 0x73,
0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20,
0xee, 0x6c, 0x65, 0x73, 0x20, 0x47, 0x61, 0x6c, 0xe1, 0x70, 0x61, 0x67,
0x6f, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20,
0xee, 0x6c, 0x65, 0x73, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20,
0x47, 0xe9, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x20, 0x47, 0xe9, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65,
0x20, 0x47, 0xe9, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x47,
0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x6d, 0x6f, 0x79, 0x65, 0x6e, 0x6e, 0x65, 0x20, 0x64, 0x65, 0x20, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74, 0x20, 0x64,
0x75, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74, 0x20, 0x64, 0x75,
0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74, 0x20, 0x64, 0x75, 0x20, 0x47, 0x72,
0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x20,
0x64, 0x75, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74,
0x20, 0x64, 0x75, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x20,
0x64, 0x75, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x47, 0x75, 0x61,
0x6d, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x47, 0x6f,
0x6c, 0x66, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20,
0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x2019, 0x48, 0x61, 0x77, 0x61, 0xef, 0x20, 0x2d, 0x20, 0x41, 0x6c,
0xe9, 0x6f, 0x75, 0x74, 0x69, 0x65, 0x6e, 0x6e, 0x65, 0x73, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x2019, 0x48, 0x61, 0x77, 0x61, 0xef, 0x20, 0x2d, 0x20, 0x41, 0x6c,
0xe9, 0x6f, 0x75, 0x74, 0x69, 0x65, 0x6e, 0x6e, 0x65, 0x73, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x2019,
0x48, 0x61, 0x77, 0x61, 0xef, 0x20, 0x2d, 0x20, 0x41, 0x6c, 0xe9, 0x6f,
0x75, 0x74, 0x69, 0x65, 0x6e, 0x6e, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f,
0x6e, 0x67, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6e, 0x67, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019,
0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6e, 0x67, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65,
0x20, 0x48, 0x6f, 0x76, 0x64, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f,
0x76, 0x64, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74,
0xe9, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x6e, 0x64, 0x65,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f,
0x63, 0xe9, 0x61, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x65, 0x6e, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x63,
0x68, 0x69, 0x6e, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75,
0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x20, 0x69, 0x6e, 0x64, 0x6f,
0x6e, 0xe9, 0x73, 0x69, 0x65, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74, 0x20, 0x69, 0x6e, 0x64,
0x6f, 0x6e, 0xe9, 0x73, 0x69, 0x65, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x20,
0x69, 0x6e, 0x64, 0x6f, 0x6e, 0xe9, 0x73, 0x69, 0x65, 0x6e, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x72, 0x61,
0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x2019, 0x49, 0x72, 0x61, 0x6e, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x2019, 0x49,
0x72, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x49,
0x72, 0x6b, 0x6f, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019, 0x49,
0x72, 0x6b, 0x6f, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x2019, 0x49, 0x72, 0x6b,
0x6f, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x2019, 0x49, 0x73, 0x72, 0x61, 0xeb, 0x6c, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019, 0x49,
0x73, 0x72, 0x61, 0xeb, 0x6c, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x2019, 0x49, 0x73, 0x72, 0x61, 0xeb,
0x6c, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x4a, 0x61,
0x70, 0x6f, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x75, 0x20, 0x4a, 0x61, 0x70, 0x6f,
0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9,
0x20, 0x64, 0x75, 0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70,
0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63,
0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x50,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64,
0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73,
0x6b, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x4b,
0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74, 0x20,
0x64, 0x75, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61,
0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x4f, 0x75, 0x65, 0x73, 0x74, 0x20, 0x64, 0x75, 0x20, 0x4b, 0x61, 0x7a,
0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x43, 0x6f, 0x72, 0xe9, 0x65,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x43, 0x6f, 0x72, 0xe9,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9,
0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x72, 0xe9, 0x65, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0xef, 0x61, 0x72, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0xef, 0x61, 0x72, 0x73, 0x6b,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20,
0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0xef, 0x61, 0x72,
0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x4b,
0x69, 0x72, 0x67, 0x68, 0x69, 0x7a, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x61, 0x6e, 0x6b,
0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0xee,
0x6c, 0x65, 0x73, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x4c, 0x69,
0x67, 0x6e, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64,
0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x63, 0x61,
0x6f, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x4d, 0x61, 0x6c,
0x61, 0x69, 0x73, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x65, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x73, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0xee, 0x6c, 0x65,
0x73, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x69, 0x73, 0x65, 0x73, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0xee, 0x6c, 0x65,
0x73, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x63, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x63, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74,
0xe9, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x65,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x77,
0x73, 0x6f, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20,
0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x71, 0x75, 0x65, 0x20, 0x6d, 0x65,
0x78, 0x69, 0x63, 0x61, 0x69, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x75, 0x20, 0x50,
0x61, 0x63, 0x69, 0x66, 0x69, 0x71, 0x75, 0x65, 0x20, 0x6d, 0x65, 0x78,
0x69, 0x63, 0x61, 0x69, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x75, 0x20, 0x50, 0x61, 0x63, 0x69,
0x66, 0x69, 0x71, 0x75, 0x65, 0x20, 0x6d, 0x65, 0x78, 0x69, 0x63, 0x61,
0x69, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x4f, 0x75,
0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x2019, 0x4f, 0x75, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20,
0x64, 0x2019, 0x4f, 0x75, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f,
0x72, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f,
0x73, 0x63, 0x6f, 0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73,
0x63, 0x6f, 0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9,
0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x75,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x4d, 0x79, 0x61,
0x6e, 0x6d, 0x61, 0x72, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x75, 0x20, 0x4e, 0xe9, 0x70, 0x61, 0x6c, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x75, 0x76,
0x65, 0x6c, 0x6c, 0x65, 0x2d, 0x43, 0x61, 0x6c, 0xe9, 0x64, 0x6f, 0x6e,
0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x4e, 0x6f,
0x75, 0x76, 0x65, 0x6c, 0x6c, 0x65, 0x2d, 0x43, 0x61, 0x6c, 0xe9, 0x64,
0x6f, 0x6e, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019,
0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x75, 0x76, 0x65,
0x6c, 0x6c, 0x65, 0x2d, 0x43, 0x61, 0x6c, 0xe9, 0x64, 0x6f, 0x6e, 0x69,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61,
0x20, 0x4e, 0x6f, 0x75, 0x76, 0x65, 0x6c, 0x6c, 0x65, 0x2d, 0x5a, 0xe9,
0x6c, 0x61, 0x6e, 0x64, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61,
0x20, 0x4e, 0x6f, 0x75, 0x76, 0x65, 0x6c, 0x6c, 0x65, 0x2d, 0x5a, 0xe9,
0x6c, 0x61, 0x6e, 0x64, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x4e,
0x6f, 0x75, 0x76, 0x65, 0x6c, 0x6c, 0x65, 0x2d, 0x5a, 0xe9, 0x6c, 0x61,
0x6e, 0x64, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20,
0x54, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x4e, 0x65, 0x75, 0x76, 0x65, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x4e, 0x65,
0x75, 0x76, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9,
0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x65, 0x2d,
0x4e, 0x65, 0x75, 0x76, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0xee, 0x6c, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xee,
0x6c, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0xee, 0x6c, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65,
0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x61,
0x6e, 0x6e, 0x65, 0x73, 0x20, 0x64, 0x75, 0x20, 0x4e, 0x6f, 0x72, 0x64,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74,
0xe9, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4f,
0x6d, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9,
0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x75, 0x20, 0x50, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019,
0xe9, 0x74, 0xe9, 0x20, 0x64, 0x75, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73,
0x20, 0x50, 0x61, 0x6c, 0x61, 0x6f, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x50, 0x61, 0x70, 0x6f, 0x75,
0x61, 0x73, 0x69, 0x65, 0x2d, 0x4e, 0x6f, 0x75, 0x76, 0x65, 0x6c, 0x6c,
0x65, 0x2d, 0x47, 0x75, 0x69, 0x6e, 0xe9, 0x65, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x75, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x75, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74,
0xe9, 0x20, 0x64, 0x75, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x50, 0xe9,
0x72, 0x6f, 0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x75, 0x20, 0x50, 0xe9, 0x72, 0x6f,
0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9,
0x20, 0x64, 0x75, 0x20, 0x50, 0xe9, 0x72, 0x6f, 0x75, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0x50, 0x68, 0x69, 0x6c, 0x69,
0x70, 0x70, 0x69, 0x6e, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20,
0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x65, 0x73, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64,
0x65, 0x73, 0x20, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e,
0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20,
0xee, 0x6c, 0x65, 0x73, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x69,
0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65, 0x74,
0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74,
0xe9, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x65, 0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x50, 0x69, 0x74, 0x63,
0x61, 0x69, 0x72, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0xee, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x50, 0x6f,
0x68, 0x6e, 0x70, 0x65, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x65, 0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x79, 0x7a, 0x79,
0x6c, 0x6f, 0x72, 0x64, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x79,
0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x79,
0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x20, 0x4c, 0x61, 0x20, 0x52, 0xe9, 0x75, 0x6e, 0x69,
0x6f, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x52,
0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x65,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74,
0xe9, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x53,
0x61, 0x6d, 0x61, 0x72, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61,
0x6d, 0x61, 0x72, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019,
0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9,
0x74, 0xe9, 0x20, 0x64, 0x65, 0x73, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0x53, 0x65,
0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x65, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f,
0x75, 0x72, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20,
0xee, 0x6c, 0x65, 0x73, 0x20, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x47, 0xe9, 0x6f,
0x72, 0x67, 0x69, 0x65, 0x20, 0x64, 0x75, 0x20, 0x53, 0x75, 0x64, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x53, 0x75, 0x72, 0x69,
0x6e, 0x61, 0x6d, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65,
0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x65, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64,
0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x75, 0x20, 0x54, 0x61, 0x64, 0x6a, 0x69, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65,
0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x73, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65,
0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x65, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x75, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0xe9, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x75, 0x20, 0x54, 0x75,
0x72, 0x6b, 0x6d, 0xe9, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x75,
0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0xe9, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0x54,
0x75, 0x76, 0x61, 0x6c, 0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61,
0x79, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61,
0x79, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x4f, 0x75, 0x7a, 0x62, 0xe9, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x7a, 0x62, 0xe9, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75,
0x7a, 0x62, 0xe9, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x75, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9,
0x20, 0x64, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x56, 0x65, 0x6e, 0x65,
0x7a, 0x75, 0x65, 0x6c, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9,
0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20,
0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0xee, 0x6c, 0x65, 0x20, 0x57, 0x61, 0x6b,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x57, 0x61,
0x6c, 0x6c, 0x69, 0x73, 0x2d, 0x65, 0x74, 0x2d, 0x46, 0x75, 0x74, 0x75,
0x6e, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x49,
0x61, 0x6b, 0x6f, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x49, 0x61, 0x6b, 0x6f, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x49,
0x61, 0x6b, 0x6f, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x2019, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x6f, 0x75, 0x72, 0x67, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019, 0x45, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x6f, 0x75, 0x72, 0x67, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x2019, 0x45,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x6f, 0x75, 0x72, 0x67,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x75, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x63, 0x72, 0x65, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x71, 0x75, 0x65,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x71, 0x75, 0x65,
0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x71, 0x75, 0x65, 0x20, 0x64, 0x75,
0x20, 0x53, 0x75, 0x64, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76,
0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x66, 0x72, 0x69,
0x71, 0x75, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65,
0x73, 0x74, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e,
0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61,
0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x6c, 0x6d, 0x61, 0x20,
0x41, 0x74, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61,
0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x75, 0x20, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61,
0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x75, 0x20, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76,
0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45,
0x73, 0x74, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e,
0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0x52, 0x6f, 0x63, 0x68,
0x65, 0x75, 0x73, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61,
0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x75, 0x20, 0x50, 0x61,
0x63, 0x69, 0x66, 0x69, 0x71, 0x75, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x2019, 0x41,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61,
0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x70, 0x69,
0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63,
0xe9, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x6b, 0x74, 0x61, 0x6f, 0x75, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65,
0x20, 0x64, 0x2019, 0x41, 0x71, 0x74, 0xf6, 0x62, 0x65, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x61, 0x62, 0x69, 0x65, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e,
0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65,
0x73, 0x74, 0x20, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65,
0x20, 0x64, 0x2019, 0x41, 0x72, 0x6d, 0xe9, 0x6e, 0x69, 0x65, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x71, 0x75, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61,
0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x75, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61,
0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x75, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x65, 0x2d, 0x6f, 0x75, 0x65, 0x73, 0x74, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63,
0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e,
0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65,
0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61,
0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0xef, 0x64, 0x6a, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65,
0x73, 0x20, 0x41, 0xe7, 0x6f, 0x72, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x75,
0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65,
0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x42, 0x72, 0x75,
0x6e, 0xe9, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61,
0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x75, 0x20, 0x43, 0x61, 0x70, 0x2d,
0x56, 0x65, 0x72, 0x74, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76,
0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x75, 0x20, 0x43, 0x68, 0x69,
0x6c, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x43,
0x68, 0x69, 0x6e, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69,
0x6e, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e,
0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x65,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9,
0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63,
0xe9, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20,
0x43, 0x6f, 0x6f, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76,
0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62,
0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63,
0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xee, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x50, 0xe2, 0x71, 0x75, 0x65, 0x73, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x45, 0x73, 0x74, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64,
0x65, 0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x6f,
0x75, 0x69, 0x6e, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61,
0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0xee,
0x6c, 0x65, 0x73, 0x20, 0x46, 0x69, 0x64, 0x6a, 0x69, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x65,
0x20, 0x66, 0x72, 0x61, 0x6e, 0xe7, 0x61, 0x69, 0x73, 0x65, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20,
0x64, 0x65, 0x20, 0x47, 0xe9, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74, 0x20, 0x64, 0x75, 0x20,
0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x20, 0x64, 0x75,
0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x48, 0x61, 0x77, 0x61, 0xef, 0x2d,
0x41, 0x6c, 0xe9, 0x6f, 0x75, 0x74, 0x69, 0x65, 0x6e, 0x6e, 0x65, 0x73,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x2019, 0x48, 0x61, 0x77, 0x61, 0xef, 0x2d, 0x41, 0x6c,
0xe9, 0x6f, 0x75, 0x74, 0x69, 0x65, 0x6e, 0x6e, 0x65, 0x73, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20,
0x64, 0x2019, 0x48, 0x61, 0x77, 0x61, 0xef, 0x2d, 0x41, 0x6c, 0xe9, 0x6f,
0x75, 0x74, 0x69, 0x65, 0x6e, 0x6e, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65,
0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20,
0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x6f, 0x63, 0xe9, 0x61, 0x6e, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x65, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x2019, 0x49, 0x72,
0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e,
0x63, 0xe9, 0x65, 0x20, 0x64, 0x2019, 0x49, 0x72, 0x6b, 0x6f, 0x75, 0x74,
0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e,
0x63, 0xe9, 0x65, 0x20, 0x64, 0x2019, 0x49, 0x73, 0x72, 0x61, 0xeb, 0x6c,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9,
0x65, 0x20, 0x64, 0x75, 0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20,
0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c,
0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74,
0x73, 0x6b, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61,
0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x72, 0xe9,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63,
0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0xef, 0x61, 0x72, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61,
0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x61, 0x63, 0x61, 0x6f, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61,
0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61,
0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x63, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61,
0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x75, 0x20, 0x50, 0x61,
0x63, 0x69, 0x66, 0x69, 0x71, 0x75, 0x65, 0x20, 0x6d, 0x65, 0x78, 0x69,
0x63, 0x61, 0x69, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76,
0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x2019, 0x4f, 0x75, 0x6c, 0x61,
0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x75, 0x76, 0x65, 0x6c, 0x6c, 0x65, 0x2d, 0x43, 0x61, 0x6c, 0xe9,
0x64, 0x6f, 0x6e, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61,
0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61,
0x20, 0x4e, 0x6f, 0x75, 0x76, 0x65, 0x6c, 0x6c, 0x65, 0x2d, 0x5a, 0xe9,
0x6c, 0x61, 0x6e, 0x64, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61,
0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65,
0x72, 0x72, 0x65, 0x2d, 0x4e, 0x65, 0x75, 0x76, 0x65, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x69, 0x6f, 0x75, 0xe9, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xee, 0x6c, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61,
0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x68, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61,
0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x4f, 0x6d, 0x73, 0x6b, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x2019, 0x4f, 0x6d, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x2019, 0x4f, 0x6d,
0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e,
0x63, 0xe9, 0x65, 0x20, 0x64, 0x75, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61,
0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x75, 0x20, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x79, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76,
0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x75, 0x20, 0x50, 0xe9, 0x72,
0x6f, 0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e,
0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0x50, 0x68, 0x69, 0x6c,
0x69, 0x70, 0x70, 0x69, 0x6e, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20,
0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20,
0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20,
0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x65, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64,
0x65, 0x73, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65,
0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61,
0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x75, 0x20, 0x54, 0x75,
0x72, 0x6b, 0x6d, 0xe9, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9,
0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x7a, 0x62, 0xe9,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20,
0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9,
0x65, 0x20, 0x64, 0x65, 0x20, 0x49, 0x61, 0x6b, 0x6f, 0x75, 0x74, 0x73,
0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63,
0xe9, 0x65, 0x20, 0x64, 0x2019, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x6f, 0x75, 0x72, 0x67, 0x4f, 0x72, 0x65, 0x20, 0x64, 0x65,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0xe2, 0x6c, 0x4f, 0x72, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0xe2, 0x6c, 0x4f, 0x72, 0x65,
0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0xe2,
0x6c, 0x4f, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0xe2, 0x6c, 0x4f,
0x72, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0xe2, 0x6c, 0x4f, 0x72, 0x65, 0x20, 0x65, 0x73,
0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0xe2, 0x6c, 0x4f,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x20, 0x6f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0xe2, 0x6c, 0x4f, 0x72,
0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x6f, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0xe2, 0x6c, 0x4f, 0x72, 0x65, 0x20, 0x65, 0x73,
0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x20, 0x6f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0xe2, 0x6c,
0x4f, 0x72, 0x65, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x68,
0x65, 0x4f, 0x72, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x68, 0x65, 0x4f,
0x72, 0x65, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x69,
0x20, 0x4d, 0x6f, 0x73, 0x63, 0x68, 0x65, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00,
0xd83a, 0xdd44, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a,
0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd44, 0xd83a, 0xdd33,
0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a,
0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd44, 0xd83a, 0xdd33,
0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2c, 0xd83a,
0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd35,
0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d,
0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2c, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd2f,
0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2c, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31,
0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2c,
0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00,
0xd83a, 0xdd2c, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a,
0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16,
0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2c, 0xd83a,
0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a,
0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2c,
0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31,
0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0x20, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd32,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a,
0xdd35, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd33,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0x20, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a,
0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd24, 0xd83a, 0xdd25, 0xd83a, 0xdd22,
0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a,
0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd24, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd3c,
0xd83a, 0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd24, 0xd83a, 0xdd25, 0xd83a, 0xdd22,
0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd25, 0xd83a,
0xdd22, 0xd83a, 0xdd41, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32,
0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd25, 0xd83a,
0xdd22, 0xd83a, 0xdd41, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a,
0xdd00, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd41, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26,
0xd83a, 0xdd2e, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00,
0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a,
0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0x20, 0xd83a,
0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd25, 0xd83a, 0xdd2b,
0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20,
0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0x20, 0xd83a,
0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd25, 0xd83a, 0xdd2b,
0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2d, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a,
0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a,
0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20,
0xd83a, 0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd25, 0xd83a,
0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35,
0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2d, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a,
0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd46,
0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00,
0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a,
0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd06, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd46, 0xd83a, 0xdd2d, 0xd83a, 0xdd32,
0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd25, 0xd83a, 0xdd2b,
0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20,
0xd83a, 0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd25, 0xd83a,
0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd01, 0xd83a, 0xdd2b, 0xd83a, 0xdd30, 0xd83a, 0xdd46, 0xd83a, 0xdd2e,
0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd25,
0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd01, 0xd83a, 0xdd2b, 0xd83a, 0xdd30, 0xd83a, 0xdd46, 0xd83a,
0xdd2e, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35,
0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a,
0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a,
0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a,
0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd01, 0xd83a, 0xdd2b, 0xd83a, 0xdd30, 0xd83a, 0xdd46, 0xd83a, 0xdd2e, 0x20, 0xd83a, 0xdd15,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd10, 0xd83a,
0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a,
0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a,
0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd00, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a,
0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd00, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f,
0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a,
0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd44,
0xd83a, 0xdd28, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a,
0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd44, 0xd83a, 0xdd28,
0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd44,
0xd83a, 0xdd28, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd00, 0xd83a, 0xdd33, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32,
0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd33, 0xd83a,
0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b,
0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd33, 0xd83a,
0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd33,
0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a,
0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd33,
0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20,
0xd83a, 0xdd00, 0xd83a, 0xdd33, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd26, 0xd83a, 0xdd2d,
0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd44, 0xd83a, 0xdd2a, 0xd83a,
0xdd22, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32,
0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd44, 0xd83a,
0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35,
0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd44, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd26, 0xd83a,
0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a,
0xd83a, 0xdd36, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd32,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a,
0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd36, 0xd83a, 0xdd22, 0xd83a, 0xdd32,
0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20,
0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd36, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3c,
0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00,
0xd83a, 0xdd2a, 0xd83a, 0xdd36, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d,
0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a,
0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd3a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd36, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35,
0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd36, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3c,
0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a,
0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22,
0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a,
0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a,
0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a,
0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd2b,
0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd2b, 0xd83a,
0xdd33, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a,
0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20,
0xd83a, 0xdd1a, 0xd83a, 0xdd2b, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd0c, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22,
0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a,
0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a,
0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd0c, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a,
0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35,
0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd0c, 0xd83a, 0xdd27, 0xd83a, 0xdd3c,
0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a,
0xdd26, 0xd83a, 0xdd2e, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd0c, 0xd83a, 0xdd27, 0xd83a,
0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31,
0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0x20,
0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd0c, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a,
0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35,
0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0x20,
0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd0c, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a,
0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a, 0xdd32, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd0c, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a,
0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35,
0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a,
0xdd35, 0xd83a, 0xdd2f, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a,
0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd0c,
0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd2d,
0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35,
0xd83a, 0xdd2f, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd0c, 0xd83a,
0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a,
0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32,
0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a,
0xdd0c, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a,
0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16,
0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b,
0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd0c, 0xd83a, 0xdd27,
0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd34,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a,
0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd0c, 0xd83a,
0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a,
0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd36,
0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd36,
0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a,
0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd36, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd26,
0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd36, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20,
0xd83a, 0xdd00, 0xd83a, 0xdd36, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd26, 0xd83a, 0xdd22,
0xd83a, 0xdd34, 0xd83a, 0xdd36, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd00, 0xd83a, 0xdd41, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd27, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35,
0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a,
0xdd41, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd27, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a,
0xdd00, 0xd83a, 0xdd41, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd27, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2d,
0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd27,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a,
0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd04,
0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd22,
0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd04,
0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd22,
0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd04, 0xd83a, 0xdd35, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd04, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd3e,
0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd04, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a,
0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16,
0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b,
0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd04, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a,
0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f,
0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd04, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a,
0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd04, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22,
0xd83a, 0xdd34, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd26, 0xd83a, 0xdd2e, 0x2d, 0xd83a, 0xdd1c, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd23,
0xd83a, 0xdd2b, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a,
0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0x20, 0xd83a,
0xdd1c, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a,
0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0x2d, 0xd83a, 0xdd1c,
0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd27, 0xd83a, 0xdd2b, 0xd83a, 0xdd34, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35,
0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a,
0xdd2e, 0xd83a, 0xdd25, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd25, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a,
0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd15, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd25,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a,
0xdd35, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd3c, 0xd83a, 0xdd22,
0xd83a, 0xdd25, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31,
0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35,
0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd32, 0xd83a, 0xdd22,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a,
0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15,
0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd15,
0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd05, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd27, 0xd83a, 0xdd2d,
0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd27, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd25,
0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a,
0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2e,
0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd34,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a,
0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd24,
0xd83a, 0xdd2e, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a,
0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd36, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd35,
0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a,
0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45,
0xd83a, 0xdd36, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a,
0xdd33, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd2b, 0xd83a, 0xdd37, 0xd83a, 0xdd46,
0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a,
0xdd35, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45,
0xd83a, 0xdd36, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a,
0xdd33, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd26,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a,
0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd26, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd26,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd01, 0xd83a, 0xdd2b, 0xd83a,
0xdd45, 0xd83a, 0xdd3e, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd01,
0xd83a, 0xdd2d, 0xd83a, 0xdd25, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd35,
0x2d, 0xd83a, 0xdd01, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd3e, 0xd83a, 0xdd2d, 0xd83a,
0xdd24, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a, 0xdd32,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd25, 0xd83a, 0xdd2e, 0xd83a,
0xdd2a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0x2d, 0xd83a, 0xdd00, 0xd83a, 0xdd34, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31,
0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0x2d,
0xd83a, 0xdd00, 0xd83a, 0xdd34, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a,
0xdd35, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0x2d, 0xd83a, 0xdd00, 0xd83a, 0xdd34, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd31,
0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a,
0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd00, 0xd83a,
0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a,
0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a,
0xdd2f, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a,
0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a,
0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a, 0xdd32, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35,
0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00,
0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a,
0xdd4b, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a,
0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd3a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45,
0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a,
0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd2e, 0xd83a,
0xdd24, 0xd83a, 0xdd33, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0x2d,
0xd83a, 0xdd00, 0xd83a, 0xdd34, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a,
0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a,
0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd33, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32,
0xd83a, 0xdd23, 0x2d, 0xd83a, 0xdd00, 0xd83a, 0xdd34, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45,
0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a,
0xdd33, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0x2d, 0xd83a, 0xdd00,
0xd83a, 0xdd34, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35,
0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd0a, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd18,
0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0x2d, 0xd83a, 0xdd0a, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd27, 0xd83a,
0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd02, 0xd83a, 0xdd2b, 0xd83a, 0xdd34, 0xd83a, 0xdd2a,
0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd27, 0xd83a, 0xdd2d, 0x20, 0x26, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd3c,
0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd18, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a,
0xdd28, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd18, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2d,
0xd83a, 0xdd34, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd14, 0xd83a,
0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd36, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35,
0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd14, 0xd83a,
0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd36, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35,
0x20, 0xd83a, 0xdd14, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd36, 0xd83a, 0xdd2d, 0xd83a,
0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd35, 0xd83a, 0xdd2a,
0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd36, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd18, 0xd83a,
0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd26, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd3c, 0xd83a,
0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd2d,
0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd18, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a,
0xdd31, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a,
0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32,
0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd18, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a,
0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16,
0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b,
0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd2c, 0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a,
0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd18, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32,
0xd83a, 0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a, 0xdd32,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd18, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd18, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31,
0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd18,
0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a,
0xdd35, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd18, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a,
0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd18, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd25, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd02, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd3b, 0xd83a, 0xdd35,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd34, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0x2d, 0xd83a, 0xdd00, 0xd83a, 0xdd24,
0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a,
0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16,
0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd34, 0xd83a, 0xdd2d,
0xd83a, 0xdd45, 0x2d, 0xd83a, 0xdd00, 0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a,
0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd34, 0xd83a, 0xdd2d, 0xd83a,
0xdd45, 0x2d, 0xd83a, 0xdd00, 0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd2d,
0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd32,
0xd83a, 0xdd3a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a,
0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd16, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0x20, 0xd83a, 0xdd11, 0xd83a,
0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b,
0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2e, 0xd83a,
0xdd32, 0xd83a, 0xdd3a, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3a,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2e, 0xd83a, 0xdd2c, 0xd83a, 0xdd23, 0xd83a,
0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a,
0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd16, 0xd83a, 0xdd2e, 0xd83a, 0xdd2c, 0xd83a, 0xdd23, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a,
0xdd16, 0xd83a, 0xdd2e, 0xd83a, 0xdd2c, 0xd83a, 0xdd23, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32,
0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd32, 0xd83a,
0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd03,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd34, 0xd83a, 0xdd2e, 0x20, 0xd83a, 0xdd0b, 0xd83a,
0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a,
0xdd2e, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2e,
0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd1a, 0xd83a, 0xdd2e, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2e,
0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd34,
0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a,
0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd3a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd32,
0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd3a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd32,
0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a,
0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd0b, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a,
0xdd0b, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd0b, 0xd83a, 0xdd2a, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d,
0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a,
0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd2a, 0xd83a, 0xdd33, 0xd83a, 0xdd35,
0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20,
0xd83a, 0xdd0b, 0xd83a, 0xdd2a, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d,
0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0b, 0xd83a,
0xdd27, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a,
0xdd24, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31,
0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd27, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd24, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd0b, 0xd83a,
0xdd27, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a,
0xdd24, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2d, 0xd83a, 0xdd28,
0xd83a, 0xdd46, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a,
0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2d, 0xd83a, 0xdd28, 0xd83a, 0xdd46,
0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a,
0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2d, 0xd83a, 0xdd28,
0xd83a, 0xdd46, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a,
0xdd22, 0xd83a, 0xdd41, 0xd83a, 0xdd22, 0xd83a, 0xdd3f, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a,
0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16,
0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32,
0xd83a, 0xdd4b, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a,
0xdd41, 0xd83a, 0xdd22, 0xd83a, 0xdd3f, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a,
0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd34, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd27,
0xd83a, 0xdd32, 0xd83a, 0xdd2e, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd27,
0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a,
0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd32,
0xd83a, 0xdd2e, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd27, 0xd83a, 0xdd2d,
0xd83a, 0xdd33, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22,
0xd83a, 0xdd27, 0xd83a, 0xdd32, 0xd83a, 0xdd2e, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd2a,
0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd3a, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd35,
0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd36, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd02, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd02, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd35, 0x2d, 0xd83a,
0xdd16, 0xd83a, 0xdd2e, 0xd83a, 0xdd31, 0xd83a, 0xdd2b, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16,
0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b,
0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd02, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a,
0xdd23, 0xd83a, 0xdd35, 0x2d, 0xd83a, 0xdd16, 0xd83a, 0xdd2e, 0xd83a, 0xdd31, 0xd83a, 0xdd2b,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a,
0xdd35, 0x20, 0xd83a, 0xdd02, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd35,
0x2d, 0xd83a, 0xdd16, 0xd83a, 0xdd2e, 0xd83a, 0xdd31, 0xd83a, 0xdd2b, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a,
0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22,
0xd83a, 0xdd31, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd03, 0xd83a,
0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35,
0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd03, 0xd83a,
0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35,
0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a,
0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd24,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd23, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd34, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd33, 0xd83a, 0xdd2b, 0xd83a, 0xdd27,
0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd35, 0xd83a,
0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd36, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd03,
0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd27, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a,
0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31,
0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a,
0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45,
0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a,
0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd03,
0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd27, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd01, 0xd83a, 0xdd2b, 0xd83a, 0xdd30, 0xd83a, 0xdd46, 0xd83a, 0xdd2e, 0x20,
0xd83a, 0xdd03, 0xd83a, 0xdd2b, 0xd83a, 0xdd33, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd33,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a,
0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd01, 0xd83a, 0xdd2b, 0xd83a, 0xdd30, 0xd83a, 0xdd46, 0xd83a, 0xdd2e,
0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd2b, 0xd83a, 0xdd33, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a,
0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b,
0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd01, 0xd83a, 0xdd2b, 0xd83a,
0xdd30, 0xd83a, 0xdd46, 0xd83a, 0xdd2e, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd2b, 0xd83a, 0xdd33,
0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd13, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd26, 0xd83a,
0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16,
0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b,
0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd13, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35,
0x20, 0xd83a, 0xdd13, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd26, 0xd83a,
0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd03,
0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a,
0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd2e, 0xd83a, 0xdd27,
0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a,
0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd2e, 0xd83a, 0xdd27,
0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd2d, 0xd83a,
0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd2a, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd35,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2b, 0xd83a, 0xdd28, 0xd83a, 0xdd22, 0xd83a,
0xdd24, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd2d,
0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2b, 0xd83a, 0xdd27, 0xd83a, 0xdd2e, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35,
0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a,
0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a,
0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2b,
0xd83a, 0xdd27, 0xd83a, 0xdd2e, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a,
0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd24,
0xd83a, 0xdd2d, 0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd34,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2b, 0xd83a, 0xdd27, 0xd83a,
0xdd2e, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2b, 0xd83a, 0xdd31, 0x2d, 0xd83a,
0xdd1f, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a,
0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a,
0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd10, 0xd83a, 0xdd2b, 0xd83a, 0xdd31, 0x2d, 0xd83a, 0xdd1f, 0xd83a, 0xdd2b, 0xd83a, 0xdd24,
0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd10,
0xd83a, 0xdd2b, 0xd83a, 0xdd31, 0x2d, 0xd83a, 0xdd1f, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a,
0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10,
0xd83a, 0xdd2b, 0xd83a, 0xdd31, 0x2d, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a,
0xdd23, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35,
0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a,
0xdd2b, 0xd83a, 0xdd31, 0x2d, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd23,
0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20,
0xd83a, 0xdd10, 0xd83a, 0xdd2b, 0xd83a, 0xdd31, 0x2d, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a,
0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd31,
0xd83a, 0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a,
0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2e,
0xd83a, 0xdd2a, 0xd83a, 0xdd2c, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd33, 0xd83a, 0xdd35,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a,
0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05,
0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2c, 0xd83a, 0xdd2e, 0xd83a,
0xdd24, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b,
0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd35, 0xd83a,
0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10,
0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2c, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd33,
0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a,
0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0x20,
0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a,
0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3d, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32,
0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd2b, 0xd83a,
0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0x20, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a,
0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3d, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35,
0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0x20, 0xd83a, 0xdd23, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a,
0xdd3d, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2e,
0xd83a, 0xdd3e, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a,
0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a,
0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd3e, 0xd83a, 0xdd2e,
0xd83a, 0xdd27, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd27, 0xd83a, 0xdd2d,
0xd83a, 0xdd33, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd3e, 0xd83a, 0xdd2e,
0xd83a, 0xdd27, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd27, 0xd83a, 0xdd2d,
0xd83a, 0xdd33, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0c, 0xd83a, 0xdd25, 0xd83a, 0xdd27, 0xd83a,
0xdd35, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22,
0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0c, 0xd83a, 0xdd25, 0xd83a, 0xdd27, 0xd83a, 0xdd35, 0xd83a,
0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45,
0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd0c, 0xd83a, 0xdd25, 0xd83a, 0xdd27, 0xd83a,
0xdd35, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd22,
0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a,
0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd06, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd3c,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd22,
0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd31, 0xd83a, 0xdd2e, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd06,
0xd83a, 0xdd22, 0xd83a, 0xdd28, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0x20, 0xd83a,
0xdd18, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2b,
0xd83a, 0xdd27, 0xd83a, 0xdd2e, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd34, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35,
0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd06, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a,
0xdd34, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f,
0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd2a, 0xd83a,
0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd34, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd06,
0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a,
0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a,
0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a,
0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a,
0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45,
0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a,
0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd2d, 0xd83a, 0xdd24,
0xd83a, 0xdd2d, 0xd83a, 0xdd28, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a,
0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd2d,
0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd28, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd32,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a,
0xdd35, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd28,
0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a,
0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd36, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd0a, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd27,
0xd83a, 0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd3c, 0x2e, 0x20, 0xd83a,
0xdd06, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd34, 0x20,
0x26, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd2b, 0xd83a, 0xdd24,
0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a,
0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd3c, 0x2e, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd2d,
0xd83a, 0xdd34, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0x20, 0x26, 0x20, 0xd83a,
0xdd03, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a,
0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f,
0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd3c, 0x2e, 0x20, 0xd83a, 0xdd06, 0xd83a,
0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0x20, 0x26, 0x20,
0xd83a, 0xdd03, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd2e,
0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a,
0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd28, 0xd83a, 0xdd2b, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd2d, 0xd83a,
0xdd34, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd24, 0xd83a, 0xdd2e,
0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a,
0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a,
0xdd35, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd24, 0xd83a, 0xdd2e,
0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd08, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd2e, 0xd83a,
0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd08, 0xd83a, 0xdd2e, 0xd83a, 0xdd3c, 0xd83a, 0xdd2b,
0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a,
0xdd3f, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35,
0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a,
0xdd22, 0xd83a, 0xdd3f, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a,
0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f,
0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd3f, 0xd83a, 0xdd22, 0xd83a,
0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05,
0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a,
0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd22,
0xd83a, 0xdd25, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd05,
0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd2b, 0xd83a, 0xdd34, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a,
0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31,
0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a,
0xdd28, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15,
0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd36, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a,
0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05,
0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2c, 0x2d, 0xd83a, 0xdd14, 0xd83a, 0xdd2e, 0xd83a,
0xdd2a, 0xd83a, 0xdd36, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd32,
0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd34, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd1a, 0xd83a, 0xdd22, 0xd83a, 0xdd38, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d,
0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a,
0xdd28, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22,
0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd28, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45,
0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a,
0xdd28, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd22,
0xd83a, 0xdd36, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd3c,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd31, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd2e,
0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a,
0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22,
0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23,
0xd83a, 0xdd2b, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3a,
0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a,
0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd2a,
0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd2d,
0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a,
0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd35,
0xd83a, 0xdd2a, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd32,
0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a,
0xdd35, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd33, 0xd83a, 0xdd35,
0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd3c,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a,
0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd3e, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd35, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd12, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd3a,
0xd83a, 0xdd2e, 0xd83a, 0xdd34, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a,
0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd12, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd3a,
0xd83a, 0xdd2e, 0xd83a, 0xdd34, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a,
0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd12, 0xd83a, 0xdd35, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd34, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd13, 0xd83a, 0xdd41, 0xd83a, 0xdd26, 0xd83a, 0xdd2b, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a,
0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32,
0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd13, 0xd83a, 0xdd41, 0xd83a,
0xdd26, 0xd83a, 0xdd2b, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b,
0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd13, 0xd83a, 0xdd41, 0xd83a,
0xdd26, 0xd83a, 0xdd2b, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1c, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd3c,
0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a,
0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd1c, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd1c, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd3c,
0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1c, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a,
0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd41, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd24, 0xd83a,
0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1c, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd22,
0xd83a, 0xdd3e, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd33,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a,
0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1c,
0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd3e, 0xd83a, 0xdd2e, 0xd83a, 0xdd27,
0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd33, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd1c, 0xd83a, 0xdd2d,
0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd3e, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd3c,
0xd83a, 0xdd2e, 0xd83a, 0xdd33, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1c, 0xd83a, 0xdd2e, 0xd83a,
0xdd24, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22,
0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1c, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd3a, 0xd83a,
0xdd2e, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f,
0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd1c, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd3a, 0xd83a,
0xdd2e, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1c, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd3c,
0xd83a, 0xdd2e, 0xd83a, 0xdd33, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd35, 0xd83a,
0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0f,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd0f, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd46, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a,
0xdd27, 0x20, 0x26, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd3c, 0xd83a, 0xdd35,
0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd12, 0xd83a, 0xdd22, 0xd83a,
0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a,
0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a,
0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd12, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a,
0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b,
0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd12, 0xd83a, 0xdd22, 0xd83a,
0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a,
0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd12, 0xd83a, 0xdd2b, 0xd83a, 0xdd33, 0xd83a, 0xdd22,
0xd83a, 0xdd3c, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd26,
0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd3a, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a,
0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd12, 0xd83a, 0xdd2b, 0xd83a, 0xdd33,
0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd32,
0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd3a, 0xd83a, 0xdd35, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20,
0xd83a, 0xdd12, 0xd83a, 0xdd2b, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2b,
0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd2a,
0xd83a, 0xdd3a, 0xd83a, 0xdd35, 0xc0, 0x6d, 0x20, 0x41, 0x63, 0x72, 0x65, 0x42,
0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x41, 0x63, 0x72, 0x65, 0x54, 0xec,
0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x41, 0x63, 0x72, 0x65, 0xc0, 0x6d, 0x20, 0x41, 0x66, 0x67, 0x68,
0x61, 0x6e, 0x61, 0x73, 0x74, 0xe0, 0x69, 0x6e, 0xc0, 0x6d, 0x20, 0x4d,
0x65, 0x61, 0x64, 0x68, 0x61, 0x6e, 0x20, 0x41, 0x66, 0x72, 0x61, 0x67,
0x61, 0xc0, 0x6d, 0x20, 0x41, 0x66, 0x72, 0x61, 0x67, 0x61, 0x20, 0x61,
0x6e, 0x20, 0x45, 0x61, 0x72, 0xc0, 0x6d, 0x20, 0x41, 0x66, 0x72, 0x61,
0x67, 0x61, 0x20, 0x61, 0x20, 0x44, 0x65, 0x61, 0x73, 0xc0, 0x6d, 0x20,
0x41, 0x66, 0x72, 0x61, 0x67, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x49, 0x61,
0x72, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x41, 0x66, 0x72, 0x61,
0x67, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x49, 0x61, 0x72, 0x54, 0xec, 0x64,
0x65, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x41, 0x66, 0x72, 0x61, 0x67, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x49, 0x61,
0x72, 0xc0, 0x6d, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x42, 0x75,
0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x54,
0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0xc0, 0x6d, 0x20, 0x41,
0x6c, 0x6d, 0x61, 0x74, 0x79, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20,
0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41, 0x6c, 0x6d,
0x61, 0x74, 0x79, 0xc0, 0x6d, 0x20, 0x41, 0x6d, 0x61, 0x73, 0x6f, 0x69,
0x6e, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x41, 0x6d, 0x61, 0x73,
0x6f, 0x69, 0x6e, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41, 0x6d, 0x61, 0x73, 0x6f, 0x69,
0x6e, 0xc0, 0x6d, 0x20, 0x4d, 0x65, 0x61, 0x64, 0x68, 0x61, 0x6e, 0x20,
0x41, 0x69, 0x6d, 0x65, 0x69, 0x72, 0x65, 0x61, 0x67, 0x61, 0x42, 0x75,
0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x4d, 0x65, 0x61, 0x64, 0x68, 0x61, 0x6e,
0x20, 0x41, 0x69, 0x6d, 0x65, 0x69, 0x72, 0x65, 0x61, 0x67, 0x61, 0x54,
0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x4d, 0x65, 0x61, 0x64, 0x68, 0x61, 0x6e, 0x20, 0x41, 0x69,
0x6d, 0x65, 0x69, 0x72, 0x65, 0x61, 0x67, 0x61, 0xc0, 0x6d, 0x20, 0x41,
0x69, 0x6d, 0x65, 0x69, 0x72, 0x65, 0x61, 0x67, 0x61, 0x20, 0x61, 0x6e,
0x20, 0x45, 0x61, 0x72, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x41,
0x69, 0x6d, 0x65, 0x69, 0x72, 0x65, 0x61, 0x67, 0x61, 0x20, 0x61, 0x6e,
0x20, 0x45, 0x61, 0x72, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41, 0x69, 0x6d, 0x65, 0x69,
0x72, 0x65, 0x61, 0x67, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x72,
0xc0, 0x6d, 0x20, 0x4d, 0x6f, 0x6e, 0x61, 0x64, 0x68, 0x20, 0x41, 0x69,
0x6d, 0x65, 0x69, 0x72, 0x65, 0x61, 0x67, 0x61, 0x42, 0x75, 0x6e, 0x2d,
0xe0, 0x6d, 0x20, 0x4d, 0x6f, 0x6e, 0x61, 0x64, 0x68, 0x20, 0x41, 0x69,
0x6d, 0x65, 0x69, 0x72, 0x65, 0x61, 0x67, 0x61, 0x54, 0xec, 0x64, 0x65,
0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4d,
0x6f, 0x6e, 0x61, 0x64, 0x68, 0x20, 0x41, 0x69, 0x6d, 0x65, 0x69, 0x72,
0x65, 0x61, 0x67, 0x61, 0xc0, 0x6d, 0x20, 0x61, 0x2019, 0x20, 0x43, 0x68,
0x75, 0x61, 0x69, 0x6e, 0x20, 0x53, 0x68, 0xe8, 0x69, 0x6d, 0x68, 0x42,
0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x61, 0x2019, 0x20, 0x43, 0x68, 0x75,
0x61, 0x69, 0x6e, 0x20, 0x53, 0x68, 0xe8, 0x69, 0x6d, 0x68, 0x54, 0xec,
0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x61, 0x2019, 0x20, 0x43, 0x68, 0x75, 0x61, 0x69, 0x6e, 0x20, 0x53,
0x68, 0xe8, 0x69, 0x6d, 0x68, 0xc0, 0x6d, 0x20, 0x41, 0x6e, 0x61, 0x64,
0x79, 0x72, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x41, 0x6e, 0x61,
0x64, 0x79, 0x72, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0xc0, 0x6d, 0x20, 0x41, 0x70, 0x69, 0x61, 0x42, 0x75, 0x6e, 0x2d, 0xe0,
0x6d, 0x20, 0x41, 0x70, 0x69, 0x61, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41, 0x70, 0x69,
0x61, 0xc0, 0x6d, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x42, 0x75, 0x6e,
0x2d, 0xe0, 0x6d, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x54, 0xec, 0x64,
0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x41, 0x71, 0x74, 0x61, 0x75, 0xc0, 0x6d, 0x20, 0x41, 0x71, 0x74, 0x6f,
0x62, 0x65, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x41, 0x71, 0x74,
0x6f, 0x62, 0x65, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65,
0xc0, 0x6d, 0x20, 0x41, 0x72, 0x61, 0x62, 0x61, 0x63, 0x68, 0x42, 0x75,
0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x41, 0x72, 0x61, 0x62, 0x61, 0x63, 0x68,
0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x41, 0x72, 0x61, 0x62, 0x61, 0x63, 0x68, 0xc0, 0x6d,
0x20, 0x6e, 0x61, 0x20, 0x68, 0x2d, 0x41, 0x72, 0x67, 0x61, 0x6e, 0x74,
0x61, 0x69, 0x6e, 0x65, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x6e,
0x61, 0x20, 0x68, 0x2d, 0x41, 0x72, 0x67, 0x61, 0x6e, 0x74, 0x61, 0x69,
0x6e, 0x65, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72,
0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x2d, 0x41, 0x72,
0x67, 0x61, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x65, 0xc0, 0x6d, 0x20, 0x6e,
0x61, 0x20, 0x68, 0x2d, 0x41, 0x72, 0x67, 0x61, 0x6e, 0x74, 0x61, 0x69,
0x6e, 0x65, 0x20, 0x53, 0x69, 0x61, 0x72, 0x61, 0x69, 0x63, 0x68, 0x42,
0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x2d, 0x41,
0x72, 0x67, 0x61, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x65, 0x20, 0x53, 0x69,
0x61, 0x72, 0x61, 0x69, 0x63, 0x68, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20,
0x68, 0x2d, 0x41, 0x72, 0x67, 0x61, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x65,
0x20, 0x53, 0x69, 0x61, 0x72, 0x61, 0x69, 0x63, 0x68, 0xc0, 0x6d, 0x20,
0x41, 0x69, 0x72, 0x6d, 0x65, 0x69, 0x6e, 0x69, 0x61, 0x42, 0x75, 0x6e,
0x2d, 0xe0, 0x6d, 0x20, 0x41, 0x69, 0x72, 0x6d, 0x65, 0x69, 0x6e, 0x69,
0x61, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x41, 0x69, 0x72, 0x6d, 0x65, 0x69, 0x6e, 0x69,
0x61, 0xc0, 0x6d, 0x20, 0x61, 0x2019, 0x20, 0x43, 0x68, 0x75, 0x61, 0x69,
0x6e, 0x20, 0x53, 0x68, 0x69, 0x61, 0x72, 0x42, 0x75, 0x6e, 0x2d, 0xe0,
0x6d, 0x20, 0x61, 0x2019, 0x20, 0x43, 0x68, 0x75, 0x61, 0x69, 0x6e, 0x20,
0x53, 0x68, 0x69, 0x61, 0x72, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x61, 0x2019, 0x20, 0x43,
0x68, 0x75, 0x61, 0x69, 0x6e, 0x20, 0x53, 0x68, 0x69, 0x61, 0x72, 0xc0,
0x6d, 0x20, 0x4d, 0x65, 0x61, 0x64, 0x68, 0x61, 0x6e, 0x20, 0x41, 0x73,
0x74, 0x72, 0xe0, 0x69, 0x6c, 0x69, 0x61, 0x42, 0x75, 0x6e, 0x2d, 0xe0,
0x6d, 0x20, 0x4d, 0x65, 0x61, 0x64, 0x68, 0x61, 0x6e, 0x20, 0x41, 0x73,
0x74, 0x72, 0xe0, 0x69, 0x6c, 0x69, 0x61, 0x54, 0xec, 0x64, 0x65, 0x20,
0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4d, 0x65,
0x61, 0x64, 0x68, 0x61, 0x6e, 0x20, 0x41, 0x73, 0x74, 0x72, 0xe0, 0x69,
0x6c, 0x69, 0x61, 0xc0, 0x6d, 0x20, 0x4d, 0x65, 0x61, 0x64, 0x68, 0x61,
0x6e, 0x20, 0x41, 0x73, 0x74, 0x72, 0xe0, 0x69, 0x6c, 0x69, 0x61, 0x20,
0x61, 0x6e, 0x20, 0x49, 0x61, 0x72, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d,
0x20, 0x4d, 0x65, 0x61, 0x64, 0x68, 0x61, 0x6e, 0x20, 0x41, 0x73, 0x74,
0x72, 0xe0, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x49, 0x61,
0x72, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x4d, 0x65, 0x61, 0x64, 0x68, 0x61, 0x6e, 0x20,
0x41, 0x73, 0x74, 0x72, 0xe0, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x61, 0x6e,
0x20, 0x49, 0x61, 0x72, 0xc0, 0x6d, 0x20, 0x41, 0x73, 0x74, 0x72, 0xe0,
0x69, 0x6c, 0x69, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x72, 0x42,
0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x41, 0x73, 0x74, 0x72, 0xe0, 0x69,
0x6c, 0x69, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x72, 0x54, 0xec,
0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x41, 0x73, 0x74, 0x72, 0xe0, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x61,
0x6e, 0x20, 0x45, 0x61, 0x72, 0xc0, 0x6d, 0x20, 0x41, 0x73, 0x74, 0x72,
0xe0, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x49, 0x61, 0x72,
0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x41, 0x73, 0x74, 0x72, 0xe0,
0x69, 0x6c, 0x69, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x49, 0x61, 0x72, 0x54,
0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x41, 0x73, 0x74, 0x72, 0xe0, 0x69, 0x6c, 0x69, 0x61, 0x20,
0x61, 0x6e, 0x20, 0x49, 0x61, 0x72, 0xc0, 0x6d, 0x20, 0x41, 0x73, 0x61,
0x72, 0x62, 0x61, 0x69, 0x64, 0x65, 0xe0, 0x69, 0x6e, 0x42, 0x75, 0x6e,
0x2d, 0xe0, 0x6d, 0x20, 0x41, 0x73, 0x61, 0x72, 0x62, 0x61, 0x69, 0x64,
0x65, 0xe0, 0x69, 0x6e, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41, 0x73, 0x61, 0x72, 0x62,
0x61, 0x69, 0x64, 0x65, 0xe0, 0x69, 0x6e, 0xc0, 0x6d, 0x20, 0x6e, 0x61,
0x6e, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x41,
0x73, 0x6f, 0x72, 0x61, 0x63, 0x68, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d,
0x20, 0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61,
0x6e, 0x20, 0x41, 0x73, 0x6f, 0x72, 0x61, 0x63, 0x68, 0x54, 0xec, 0x64,
0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e,
0x20, 0x41, 0x73, 0x6f, 0x72, 0x61, 0x63, 0x68, 0xc0, 0x6d, 0x20, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x61, 0x69, 0x73, 0x42, 0x75, 0x6e,
0x2d, 0xe0, 0x6d, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x61,
0x69, 0x73, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72,
0x61, 0x69, 0x64, 0x68, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x61, 0x69, 0x73, 0xc0, 0x6d, 0x20, 0x42, 0x75, 0x74, 0xe0, 0x69, 0x6e,
0xc0, 0x6d, 0x20, 0x42, 0x6f, 0x69, 0x6c, 0x69, 0x62, 0x68, 0x69, 0x61,
0xc0, 0x6d, 0x20, 0x42, 0x68, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61,
0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x42, 0x68, 0x72, 0x61, 0x73,
0xed, 0x6c, 0x69, 0x61, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x42, 0x68, 0x72, 0x61, 0x73,
0xed, 0x6c, 0x69, 0x61, 0xc0, 0x6d, 0x20, 0x42, 0x68, 0x72, 0xf9, 0x6e,
0x61, 0x69, 0x67, 0x68, 0x20, 0x44, 0xe0, 0x72, 0x20, 0x61, 0x73, 0x2d,
0x53, 0x61, 0x6c, 0xe0, 0x6d, 0xc0, 0x6d, 0x20, 0x61, 0x2019, 0x20, 0x43,
0x68, 0x69, 0x70, 0x20, 0x55, 0x61, 0x69, 0x6e, 0x65, 0x42, 0x75, 0x6e,
0x2d, 0xe0, 0x6d, 0x20, 0x61, 0x2019, 0x20, 0x43, 0x68, 0x69, 0x70, 0x20,
0x55, 0x61, 0x69, 0x6e, 0x65, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x61, 0x2019, 0x20, 0x43,
0x68, 0x69, 0x70, 0x20, 0x55, 0x61, 0x69, 0x6e, 0x65, 0xc0, 0x6d, 0x20,
0x43, 0x68, 0x61, 0x73, 0x65, 0x79, 0xc0, 0x6d, 0x20, 0x43, 0x68, 0x61,
0x6d, 0x6f, 0x72, 0x72, 0x6f, 0xc0, 0x6d, 0x20, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x53, 0x69, 0x6c,
0x65, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x53,
0x69, 0x6c, 0x65, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x53, 0x69, 0x6c,
0x65, 0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x53, 0xec, 0x6e, 0x65, 0x42,
0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x53, 0xec, 0x6e,
0x65, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x53, 0xec, 0x6e, 0x65, 0xc0,
0x6d, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x20, 0x6e, 0x61, 0x20,
0x4e, 0x6f, 0x6c, 0x6c, 0x61, 0x69, 0x67, 0xc0, 0x6d, 0x20, 0x45, 0x69,
0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x43, 0x68, 0x6f, 0x63, 0x6f,
0x73, 0xc0, 0x6d, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x69, 0x6d, 0x62, 0x69,
0x61, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x43, 0x6f, 0x6c, 0x6f,
0x69, 0x6d, 0x62, 0x69, 0x61, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x43, 0x6f, 0x6c, 0x6f,
0x69, 0x6d, 0x62, 0x69, 0x61, 0xc0, 0x6d, 0x20, 0x45, 0x69, 0x6c, 0x65,
0x61, 0x6e, 0x61, 0x6e, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x42, 0x75, 0x6e,
0x2d, 0xe0, 0x6d, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e,
0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x4c, 0x65, 0x74, 0x68, 0x2d, 0x74, 0x68,
0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x43,
0x6f, 0x6f, 0x6b, 0xc0, 0x6d, 0x20, 0x43, 0xf9, 0x62, 0x61, 0x42, 0x75,
0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x43, 0xf9, 0x62, 0x61, 0x54, 0xec, 0x64,
0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x43, 0xf9, 0x62, 0x61, 0xc0, 0x6d, 0x20, 0x44, 0x68, 0x61, 0x76, 0x69,
0x73, 0xc0, 0x6d, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64,
0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0xc0, 0x6d, 0x20, 0x54,
0x68, 0xec, 0x6f, 0x6d, 0x6f, 0x72, 0x20, 0x61, 0x6e, 0x20, 0x45, 0x61,
0x72, 0xc0, 0x6d, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x20, 0x6e,
0x61, 0x20, 0x43, 0xe0, 0x69, 0x73, 0x67, 0x65, 0x42, 0x75, 0x6e, 0x2d,
0xe0, 0x6d, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x20, 0x6e, 0x61,
0x20, 0x43, 0xe0, 0x69, 0x73, 0x67, 0x65, 0x54, 0xec, 0x64, 0x65, 0x20,
0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x45, 0x69,
0x6c, 0x65, 0x61, 0x6e, 0x20, 0x6e, 0x61, 0x20, 0x43, 0xe0, 0x69, 0x73,
0x67, 0x65, 0xc0, 0x6d, 0x20, 0x45, 0x61, 0x63, 0x75, 0x61, 0x64, 0x6f,
0x69, 0x72, 0xc0, 0x6d, 0x20, 0x4d, 0x65, 0x61, 0x64, 0x68, 0x61, 0x6e,
0x20, 0x6e, 0x61, 0x20, 0x52, 0x6f, 0x69, 0x6e, 0x6e, 0x2d, 0x45, 0xf2,
0x72, 0x70, 0x61, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x4d, 0x65,
0x61, 0x64, 0x68, 0x61, 0x6e, 0x20, 0x6e, 0x61, 0x20, 0x52, 0x6f, 0x69,
0x6e, 0x6e, 0x2d, 0x45, 0xf2, 0x72, 0x70, 0x61, 0x54, 0xec, 0x64, 0x65,
0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4d,
0x65, 0x61, 0x64, 0x68, 0x61, 0x6e, 0x20, 0x6e, 0x61, 0x20, 0x52, 0x6f,
0x69, 0x6e, 0x6e, 0x2d, 0x45, 0xf2, 0x72, 0x70, 0x61, 0xc0, 0x6d, 0x20,
0x6e, 0x61, 0x20, 0x52, 0x6f, 0x69, 0x6e, 0x6e, 0x2d, 0x45, 0xf2, 0x72,
0x70, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x72, 0x42, 0x75, 0x6e,
0x2d, 0xe0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x52, 0x6f, 0x69, 0x6e, 0x6e,
0x2d, 0x45, 0xf2, 0x72, 0x70, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x45, 0x61,
0x72, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x52, 0x6f, 0x69, 0x6e, 0x6e,
0x2d, 0x45, 0xf2, 0x72, 0x70, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x45, 0x61,
0x72, 0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x52, 0x6f, 0x69, 0x6e, 0x6e,
0x2d, 0x45, 0xf2, 0x72, 0x70, 0x61, 0x20, 0x6e, 0x61, 0x73, 0x20, 0x66,
0x68, 0x61, 0x69, 0x64, 0x65, 0x20, 0x65, 0x61, 0x72, 0xc0, 0x6d, 0x20,
0x6e, 0x61, 0x20, 0x52, 0x6f, 0x69, 0x6e, 0x6e, 0x2d, 0x45, 0xf2, 0x72,
0x70, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x49, 0x61, 0x72, 0x42, 0x75, 0x6e,
0x2d, 0xe0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x52, 0x6f, 0x69, 0x6e, 0x6e,
0x2d, 0x45, 0xf2, 0x72, 0x70, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x49, 0x61,
0x72, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x52, 0x6f, 0x69, 0x6e, 0x6e,
0x2d, 0x45, 0xf2, 0x72, 0x70, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x49, 0x61,
0x72, 0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x65,
0x61, 0x6e, 0x61, 0x6e, 0x20, 0x46, 0xe0, 0x63, 0x6c, 0x61, 0x6e, 0x6e,
0x61, 0x63, 0x68, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x6e, 0x61,
0x6e, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x46,
0xe0, 0x63, 0x6c, 0x61, 0x6e, 0x6e, 0x61, 0x63, 0x68, 0x54, 0xec, 0x64,
0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e,
0x20, 0x46, 0xe0, 0x63, 0x6c, 0x61, 0x6e, 0x6e, 0x61, 0x63, 0x68, 0xc0,
0x6d, 0x20, 0x46, 0xec, 0x64, 0x69, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d,
0x20, 0x46, 0xec, 0x64, 0x69, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x46, 0xec, 0x64, 0x69,
0xc0, 0x6d, 0x20, 0x47, 0x75, 0x69, 0x64, 0x68, 0x65, 0xe0, 0x6e, 0x61,
0x20, 0x6e, 0x61, 0x20, 0x46, 0x72, 0x61, 0x69, 0x6e, 0x67, 0x65, 0xc0,
0x6d, 0x20, 0x44, 0x65, 0x61, 0x73, 0x61, 0x63, 0x68, 0x20, 0x61, 0x67,
0x75, 0x73, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x74, 0x61, 0x69, 0x67,
0x65, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x46, 0x72, 0x61, 0x69,
0x6e, 0x67, 0x65, 0xc0, 0x6d, 0x20, 0x47, 0x68, 0x61, 0x6c, 0x61, 0x70,
0x61, 0x67, 0x6f, 0x73, 0xc0, 0x6d, 0x20, 0x47, 0x68, 0x61, 0x6d, 0x62,
0x69, 0x65, 0x72, 0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x43, 0x61, 0x69,
0x72, 0x74, 0x62, 0x68, 0x65, 0x69, 0x6c, 0x65, 0x42, 0x75, 0x6e, 0x2d,
0xe0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x43, 0x61, 0x69, 0x72, 0x74, 0x62,
0x68, 0x65, 0x69, 0x6c, 0x65, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x43,
0x61, 0x69, 0x72, 0x74, 0x62, 0x68, 0x65, 0x69, 0x6c, 0x65, 0xc0, 0x6d,
0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x47, 0x68,
0x69, 0x6c, 0x65, 0x61, 0x62, 0x61, 0x69, 0x72, 0x74, 0xc0, 0x6d, 0x20,
0x6e, 0x61, 0x20, 0x47, 0x72, 0x61, 0x6f, 0x6e, 0x6c, 0x61, 0x69, 0x6e,
0x6e, 0x65, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x6e, 0x61, 0x20,
0x47, 0x72, 0x61, 0x6f, 0x6e, 0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x65, 0x54,
0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x72, 0x61, 0x6f, 0x6e, 0x6c, 0x61,
0x69, 0x6e, 0x6e, 0x65, 0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x72,
0x61, 0x6f, 0x6e, 0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x20, 0x61, 0x6e, 0x20,
0x45, 0x61, 0x72, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x6e, 0x61,
0x20, 0x47, 0x72, 0x61, 0x6f, 0x6e, 0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x20,
0x61, 0x6e, 0x20, 0x45, 0x61, 0x72, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20,
0x47, 0x72, 0x61, 0x6f, 0x6e, 0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x20, 0x61,
0x6e, 0x20, 0x45, 0x61, 0x72, 0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x47,
0x72, 0x61, 0x6f, 0x6e, 0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x20, 0x61, 0x6e,
0x20, 0x49, 0x61, 0x72, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x6e,
0x61, 0x20, 0x47, 0x72, 0x61, 0x6f, 0x6e, 0x6c, 0x61, 0x69, 0x6e, 0x6e,
0x20, 0x61, 0x6e, 0x20, 0x49, 0x61, 0x72, 0x54, 0xec, 0x64, 0x65, 0x20,
0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61,
0x20, 0x47, 0x72, 0x61, 0x6f, 0x6e, 0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x20,
0x61, 0x6e, 0x20, 0x49, 0x61, 0x72, 0xc0, 0x6d, 0x20, 0x47, 0x75, 0x61,
0x6d, 0xc0, 0x6d, 0x20, 0x61, 0x2019, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x61,
0x69, 0x73, 0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69, 0x6c,
0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x2019,
0x69, 0x20, 0x2019, 0x73, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x61, 0x63,
0x68, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x6e, 0x61, 0x6e, 0x20,
0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x48, 0x61, 0x77,
0x61, 0x69, 0x2019, 0x69, 0x20, 0x2019, 0x73, 0x20, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x61, 0x63, 0x68, 0x54, 0xec, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x6e, 0x20, 0x45,
0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x48, 0x61, 0x77, 0x61,
0x69, 0x2019, 0x69, 0x20, 0x2019, 0x73, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x61, 0x63, 0x68, 0xc0, 0x6d, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b,
0x6f, 0x6e, 0x67, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x48, 0x6f,
0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x54, 0xec, 0x64, 0x65, 0x20,
0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x48, 0x6f,
0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0xc0, 0x6d, 0x20, 0x48, 0x6f,
0x76, 0x64, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x48, 0x6f, 0x76,
0x64, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x48, 0x6f, 0x76, 0x64, 0xc0, 0x6d, 0x20, 0x6e,
0x61, 0x6e, 0x20, 0x49, 0x6e, 0x6e, 0x73, 0x65, 0x61, 0x63, 0x68, 0x61,
0x6e, 0xc0, 0x6d, 0x20, 0x43, 0x75, 0x61, 0x6e, 0x20, 0x6e, 0x61, 0x6e,
0x20, 0x49, 0x6e, 0x6e, 0x73, 0x65, 0x61, 0x63, 0x68, 0x61, 0x6e, 0xc0,
0x6d, 0x20, 0x53, 0xec, 0x6e, 0x2d, 0x49, 0x6e, 0x6e, 0x73, 0x65, 0x61,
0x6e, 0x61, 0x63, 0x68, 0xc0, 0x6d, 0x20, 0x4d, 0x65, 0x61, 0x64, 0x68,
0x61, 0x6e, 0x20, 0x6e, 0x61, 0x6e, 0x20, 0x49, 0x6e, 0x6e, 0x64, 0x2d,
0x49, 0x6e, 0x6e, 0x73, 0x65, 0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x6e, 0x20,
0x49, 0x6e, 0x6e, 0x64, 0x2d, 0x49, 0x6e, 0x6e, 0x73, 0x65, 0x20, 0x61,
0x6e, 0x20, 0x45, 0x61, 0x72, 0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x6e, 0x20,
0x49, 0x6e, 0x6e, 0x64, 0x2d, 0x49, 0x6e, 0x6e, 0x73, 0x65, 0x20, 0x61,
0x6e, 0x20, 0x49, 0x61, 0x72, 0xc0, 0x6d, 0x20, 0x49, 0x6f, 0x72, 0xe0,
0x69, 0x6e, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x49, 0x6f, 0x72,
0xe0, 0x69, 0x6e, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x49, 0x6f, 0x72, 0xe0, 0x69, 0x6e,
0xc0, 0x6d, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x42, 0x75,
0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x54, 0xec, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xc0, 0x6d,
0x20, 0x49, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x42, 0x75, 0x6e, 0x2d,
0xe0, 0x6d, 0x20, 0x49, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x54, 0xec,
0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x49, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x6c, 0xc0, 0x6d, 0x20, 0x6e,
0x61, 0x20, 0x53, 0x65, 0x61, 0x70, 0x61, 0x69, 0x6e, 0x65, 0x42, 0x75,
0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x53, 0x65, 0x61, 0x70,
0x61, 0x69, 0x6e, 0x65, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x53, 0x65,
0x61, 0x70, 0x61, 0x69, 0x6e, 0x65, 0xc0, 0x6d, 0x20, 0x50, 0x65, 0x74,
0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b,
0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x79, 0x42, 0x75, 0x6e,
0x2d, 0xe0, 0x6d, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76,
0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61,
0x74, 0x73, 0x6b, 0x79, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x79, 0xc0, 0x6d, 0x20, 0x43, 0x61,
0x73, 0x61, 0x63, 0x68, 0x73, 0x74, 0xe0, 0x69, 0x6e, 0xc0, 0x6d, 0x20,
0x43, 0x61, 0x73, 0x61, 0x63, 0x68, 0x73, 0x74, 0xe0, 0x69, 0x6e, 0x20,
0x61, 0x6e, 0x20, 0x45, 0x61, 0x72, 0xc0, 0x6d, 0x20, 0x43, 0x61, 0x73,
0x61, 0x63, 0x68, 0x73, 0x74, 0xe0, 0x69, 0x6e, 0x20, 0x61, 0x6e, 0x20,
0x49, 0x61, 0x72, 0xc0, 0x6d, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x72, 0xe8,
0x61, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x43, 0x68, 0x6f, 0x69,
0x72, 0xe8, 0x61, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x72, 0xe8,
0x61, 0xc0, 0x6d, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0xc0, 0x6d,
0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b,
0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0xc0, 0x6d, 0x20, 0x43,
0xec, 0x6f, 0x72, 0x67, 0x61, 0x73, 0x74, 0x61, 0x69, 0x6e, 0xc0, 0x6d,
0x20, 0x4c, 0x61, 0x6e, 0x63, 0x61, 0xc0, 0x6d, 0x20, 0x45, 0x69, 0x6c,
0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x54, 0x65, 0x72, 0x61, 0x69, 0x6e,
0x61, 0xc0, 0x6d, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0xc0, 0x6d, 0x20, 0x4d, 0x61, 0x63, 0xe0,
0x74, 0x68, 0x75, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x4d, 0x61,
0x63, 0xe0, 0x74, 0x68, 0x75, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4d, 0x61, 0x63, 0xe0,
0x74, 0x68, 0x75, 0xc0, 0x6d, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x54, 0xec, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0xc0, 0x6d, 0x20, 0x4d, 0x68, 0x61, 0x6c, 0x61, 0x69, 0x64, 0x68,
0x73, 0x65, 0x61, 0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69,
0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x4d, 0x68, 0x61, 0x6c, 0x61,
0x64, 0x61, 0x69, 0x62, 0x68, 0xc0, 0x6d, 0x20, 0x45, 0x69, 0x6c, 0x65,
0x61, 0x6e, 0x61, 0x6e, 0x20, 0x61, 0x2019, 0x20, 0x4d, 0x68, 0x61, 0x72,
0x63, 0x61, 0x69, 0x73, 0xc0, 0x6d, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61,
0x6e, 0x61, 0x6e, 0x20, 0x4d, 0x68, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c,
0x6c, 0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x65,
0x61, 0x6e, 0x61, 0x6e, 0x20, 0x4d, 0x68, 0x6f, 0x69, 0x72, 0x69, 0x73,
0x65, 0x61, 0x73, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x6e, 0x61,
0x6e, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x4d,
0x68, 0x6f, 0x69, 0x72, 0x69, 0x73, 0x65, 0x61, 0x73, 0x54, 0xec, 0x64,
0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e,
0x20, 0x4d, 0x68, 0x6f, 0x69, 0x72, 0x69, 0x73, 0x65, 0x61, 0x73, 0xc0,
0x6d, 0x20, 0x4d, 0x68, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0xc0, 0x6d, 0x20,
0x61, 0x2019, 0x20, 0x43, 0x68, 0x75, 0x61, 0x69, 0x6e, 0x20, 0x53, 0x68,
0xe8, 0x69, 0x6d, 0x68, 0x20, 0x4d, 0x68, 0x65, 0x61, 0x67, 0x73, 0x61,
0x67, 0x6f, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x61, 0x2019, 0x20,
0x43, 0x68, 0x75, 0x61, 0x69, 0x6e, 0x20, 0x53, 0x68, 0xe8, 0x69, 0x6d,
0x68, 0x20, 0x4d, 0x68, 0x65, 0x61, 0x67, 0x73, 0x61, 0x67, 0x6f, 0x54,
0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x61, 0x2019, 0x20, 0x43, 0x68, 0x75, 0x61, 0x69, 0x6e, 0x20,
0x53, 0x68, 0xe8, 0x69, 0x6d, 0x68, 0x20, 0x4d, 0x68, 0x65, 0x61, 0x67,
0x73, 0x61, 0x67, 0x6f, 0xc0, 0x6d, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20,
0x42, 0x61, 0x74, 0x6f, 0x72, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20,
0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x54, 0xec,
0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0xc0,
0x6d, 0x20, 0x4d, 0x68, 0x6f, 0x73, 0x67, 0x6f, 0x42, 0x75, 0x6e, 0x2d,
0xe0, 0x6d, 0x20, 0x4d, 0x68, 0x6f, 0x73, 0x67, 0x6f, 0x54, 0xec, 0x64,
0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x4d, 0x68, 0x6f, 0x73, 0x67, 0x6f, 0xc0, 0x6d, 0x20, 0x4d, 0x69, 0xe0,
0x6e, 0x6d, 0x61, 0x72, 0xc0, 0x6d, 0x20, 0x4e, 0x61, 0x62, 0x68, 0x72,
0x75, 0xc0, 0x6d, 0x20, 0x4e, 0x65, 0x61, 0x70, 0xe0, 0x6c, 0xc0, 0x6d,
0x20, 0x43, 0x68, 0x61, 0x69, 0x6c, 0x6c, 0x65, 0x61, 0x6e, 0x6e, 0x20,
0x4e, 0x75, 0x61, 0x69, 0x64, 0x68, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d,
0x20, 0x43, 0x68, 0x61, 0x69, 0x6c, 0x6c, 0x65, 0x61, 0x6e, 0x6e, 0x20,
0x4e, 0x75, 0x61, 0x69, 0x64, 0x68, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x43, 0x68, 0x61,
0x69, 0x6c, 0x6c, 0x65, 0x61, 0x6e, 0x6e, 0x20, 0x4e, 0x75, 0x61, 0x69,
0x64, 0x68, 0xc0, 0x6d, 0x20, 0x53, 0x68, 0x65, 0x61, 0x6c, 0x61, 0x69,
0x6e, 0x6e, 0x20, 0x4e, 0x75, 0x61, 0x69, 0x64, 0x68, 0x42, 0x75, 0x6e,
0x2d, 0xe0, 0x6d, 0x20, 0x53, 0x68, 0x65, 0x61, 0x6c, 0x61, 0x69, 0x6e,
0x6e, 0x20, 0x4e, 0x75, 0x61, 0x69, 0x64, 0x68, 0x54, 0xec, 0x64, 0x65,
0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x53,
0x68, 0x65, 0x61, 0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x20, 0x4e, 0x75, 0x61,
0x69, 0x64, 0x68, 0xc0, 0x6d, 0x20, 0x54, 0x61, 0x6c, 0x61, 0x6d, 0x68,
0x20, 0x61, 0x6e, 0x20, 0xc8, 0x69, 0x73, 0x67, 0x42, 0x75, 0x6e, 0x2d,
0xe0, 0x6d, 0x20, 0x54, 0x61, 0x6c, 0x61, 0x6d, 0x68, 0x20, 0x61, 0x6e,
0x20, 0xc8, 0x69, 0x73, 0x67, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x54, 0x61, 0x6c, 0x61,
0x6d, 0x68, 0x20, 0x61, 0x6e, 0x20, 0xc8, 0x69, 0x73, 0x67, 0xc0, 0x6d,
0x20, 0x4e, 0x69, 0x75, 0x65, 0xc0, 0x6d, 0x20, 0x45, 0x69, 0x6c, 0x65,
0x69, 0x6e, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x42, 0x75,
0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x69, 0x6e, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x54, 0xec, 0x64, 0x65, 0x20,
0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x45, 0x69,
0x6c, 0x65, 0x69, 0x6e, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0xc0, 0x6d, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x42, 0x75,
0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x54, 0xec, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0xc0, 0x6d,
0x20, 0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61,
0x6e, 0x20, 0x4d, 0x61, 0x69, 0x72, 0x69, 0x61, 0x6e, 0x61, 0x63, 0x68,
0x20, 0x61, 0x20, 0x54, 0x75, 0x61, 0x74, 0x68, 0xc0, 0x6d, 0x20, 0x4e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x42, 0x75,
0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0xc0, 0x6d, 0x20, 0x4f, 0x6d, 0x73,
0x6b, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x4f, 0x6d, 0x73, 0x6b,
0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0xc0, 0x6d, 0x20, 0x50, 0x61,
0x67, 0x61, 0x73, 0x74, 0xe0, 0x69, 0x6e, 0x42, 0x75, 0x6e, 0x2d, 0xe0,
0x6d, 0x20, 0x50, 0x61, 0x67, 0x61, 0x73, 0x74, 0xe0, 0x69, 0x6e, 0x54,
0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x50, 0x61, 0x67, 0x61, 0x73, 0x74, 0xe0, 0x69, 0x6e, 0xc0,
0x6d, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x62, 0x68, 0xc0, 0x6d, 0x20, 0x47,
0x69, 0x6e, 0x69, 0x20, 0x4e, 0x75, 0x61, 0x69, 0x64, 0x68, 0x20, 0x50,
0x61, 0x70, 0x75, 0x74, 0x68, 0x61, 0x69, 0x63, 0x68, 0x65, 0xc0, 0x6d,
0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x69, 0x64, 0x68, 0x42,
0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x69, 0x64, 0x68, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x69, 0x64, 0x68, 0xc0, 0x6d, 0x20, 0x50, 0x65, 0x61, 0x72,
0xf9, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x50, 0x65, 0x61, 0x72,
0xf9, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x50, 0x65, 0x61, 0x72, 0xf9, 0xc0, 0x6d, 0x20,
0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x20, 0x46,
0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x61, 0x63, 0x68, 0x42, 0x75,
0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69, 0x6c,
0x65, 0x61, 0x6e, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65,
0x61, 0x63, 0x68, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69,
0x6c, 0x65, 0x61, 0x6e, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e,
0x65, 0x61, 0x63, 0x68, 0xc0, 0x6d, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61,
0x6e, 0x61, 0x6e, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0xc0,
0x6d, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x20, 0x61, 0x67, 0x75, 0x73, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20,
0x61, 0x67, 0x75, 0x73, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f,
0x6e, 0x54, 0xec, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x20, 0x61, 0x67, 0x75, 0x73, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0xc0, 0x6d, 0x20, 0x50, 0x65, 0x69,
0x74, 0x20, 0x61, 0x2019, 0x20, 0x43, 0x68, 0xe0, 0x69, 0x72, 0x6e, 0xc0,
0x6d, 0x20, 0x50, 0x6f, 0x68, 0x6e, 0x70, 0x65, 0x69, 0xc0, 0x6d, 0x20,
0x50, 0x79, 0x65, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0xc0, 0x6d,
0x20, 0x51, 0x131, 0x7a, 0x131, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x42, 0x75,
0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x51, 0x131, 0x7a, 0x131, 0x6c, 0x6f, 0x72,
0x64, 0x61, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72,
0x61, 0x69, 0x64, 0x68, 0x20, 0x51, 0x131, 0x7a, 0x131, 0x6c, 0x6f, 0x72,
0x64, 0x61, 0xc0, 0x6d, 0x20, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e,
0xc0, 0x6d, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0xc0, 0x6d,
0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x42, 0x75, 0x6e,
0x2d, 0xe0, 0x6d, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0xc0,
0x6d, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x42, 0x75, 0x6e, 0x2d,
0xe0, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x54, 0xec, 0x64,
0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0xc0, 0x6d, 0x20, 0x53, 0x61, 0x6d,
0x6f, 0x74, 0x68, 0x61, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x53,
0x61, 0x6d, 0x6f, 0x74, 0x68, 0x61, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x53, 0x61, 0x6d,
0x6f, 0x74, 0x68, 0x61, 0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x6e, 0x20, 0x45,
0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x53, 0x68, 0x65, 0x69,
0x73, 0x65, 0x61, 0x6c, 0x6c, 0xc0, 0x6d, 0x20, 0x53, 0x69, 0x6e, 0x67,
0x65, 0x61, 0x70, 0xf2, 0x72, 0xc0, 0x6d, 0x20, 0x45, 0x69, 0x6c, 0x65,
0x61, 0x6e, 0x61, 0x6e, 0x20, 0x53, 0x68, 0x6f, 0x6c, 0x61, 0x69, 0x6d,
0x68, 0xc0, 0x6d, 0x20, 0x53, 0x65, 0xf2, 0x72, 0x73, 0x65, 0x61, 0x20,
0x61, 0x20, 0x44, 0x65, 0x61, 0x73, 0xc0, 0x6d, 0x20, 0x53, 0x75, 0x72,
0x61, 0x6e, 0x61, 0x69, 0x6d, 0xc0, 0x6d, 0x20, 0x53, 0x68, 0x79, 0x6f,
0x77, 0x61, 0xc0, 0x6d, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0xc0,
0x6d, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x42, 0x75, 0x6e, 0x2d,
0xe0, 0x6d, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x54, 0xec, 0x64,
0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0xc0, 0x6d, 0x20, 0x54, 0x61, 0x69,
0x64, 0x69, 0x67, 0x65, 0x61, 0x73, 0x74, 0xe0, 0x69, 0x6e, 0xc0, 0x6d,
0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0xc0, 0x6d, 0x20, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0xc0, 0x6d, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0xc0, 0x6d, 0x20, 0x54,
0x75, 0x72, 0x63, 0x6d, 0x61, 0x6e, 0x61, 0x73, 0x74, 0xe0, 0x69, 0x6e,
0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x54, 0x75, 0x72, 0x63, 0x6d,
0x61, 0x6e, 0x61, 0x73, 0x74, 0xe0, 0x69, 0x6e, 0x54, 0xec, 0x64, 0x65,
0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x54,
0x75, 0x72, 0x63, 0x6d, 0x61, 0x6e, 0x61, 0x73, 0x74, 0xe0, 0x69, 0x6e,
0xc0, 0x6d, 0x20, 0x54, 0x75, 0x62, 0x68, 0x61, 0x6c, 0x75, 0xc0, 0x6d,
0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x69, 0x64, 0x68, 0x42, 0x75,
0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x69,
0x64, 0x68, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72,
0x61, 0x69, 0x64, 0x68, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x69,
0x64, 0x68, 0xc0, 0x6d, 0x20, 0x55, 0x73, 0x62, 0x61, 0x67, 0x61, 0x73,
0x74, 0xe0, 0x6e, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x55, 0x73,
0x62, 0x61, 0x67, 0x61, 0x73, 0x74, 0xe0, 0x6e, 0x54, 0xec, 0x64, 0x65,
0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x55,
0x73, 0x62, 0x61, 0x67, 0x61, 0x73, 0x74, 0xe0, 0x6e, 0xc0, 0x6d, 0x20,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x42, 0x75, 0x6e, 0x2d, 0xe0,
0x6d, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x54, 0xec, 0x64,
0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0xc0, 0x6d, 0x20, 0x6e, 0x61,
0x20, 0x42, 0x68, 0x65, 0x69, 0x6e, 0x69, 0x73, 0x65, 0x61, 0x6c, 0x61,
0xc0, 0x6d, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x54, 0xec, 0x64, 0x65,
0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0xc0, 0x6d,
0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x42, 0x75,
0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72,
0x61, 0x69, 0x64, 0x68, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0xc0, 0x6d, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0xc0,
0x6d, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x20, 0x57, 0x61, 0x6b,
0x65, 0xc0, 0x6d, 0x20, 0x55, 0x61, 0x6c, 0x6c, 0x61, 0x73, 0x20, 0x61,
0x67, 0x75, 0x73, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0xc0, 0x6d,
0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x42, 0x75, 0x6e, 0x2d,
0xe0, 0x6d, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x54, 0xec,
0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xc0, 0x6d, 0x20, 0x59,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x54, 0xec, 0x64, 0x65,
0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x59,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0xc0, 0x6d, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x41, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x54, 0x65, 0x14b, 0x67, 0x62, 0x25b, 0x20, 0x42, 0x65,
0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x42, 0x6f, 0x6b, 0x61, 0x67,
0x62, 0x25b, 0x20, 0x42, 0x65, 0x53, 0x6f, 0x75, 0x74, 0x68, 0x20, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x42, 0x65, 0x41, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x41, 0x6e, 0x61, 0x69, 0x67, 0x62, 0x25b, 0x20, 0x42,
0x65, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x41, 0x6e, 0x61, 0x69,
0x67, 0x62, 0x25b, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x46, 0x25b,
0x69, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x41, 0x6e, 0x61, 0x69, 0x67, 0x62, 0x25b, 0x20, 0x42, 0x65,
0x20, 0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73, 0x61, 0x20, 0x42, 0x65,
0x69, 0x61, 0x14b, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x42, 0x65,
0x20, 0x59, 0x25b, 0x20, 0x46, 0x25b, 0x69, 0x20, 0x42, 0x65, 0x69, 0x61,
0x14b, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x42, 0x65, 0x20, 0x59,
0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73, 0x61, 0x20, 0x42, 0x65, 0x69, 0x61,
0x14b, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x54, 0x65, 0x14b,
0x67, 0x62, 0x25b, 0x62, 0x69, 0x69, 0x20, 0x41, 0x62, 0x65, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x54, 0x65, 0x14b, 0x67, 0x62, 0x25b,
0x62, 0x69, 0x69, 0x20, 0x41, 0x62, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x46,
0x25b, 0x69, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x54, 0x65, 0x14b, 0x67, 0x62, 0x25b, 0x62, 0x69,
0x69, 0x20, 0x41, 0x62, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74,
0x73, 0x61, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x42, 0x6f, 0x6b, 0xe3, 0x67, 0x62, 0x25b, 0x62,
0x69, 0x69, 0x20, 0x41, 0x62, 0x65, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x42, 0x6f, 0x6b, 0xe3, 0x67, 0x62, 0x25b, 0x62, 0x69, 0x69,
0x20, 0x41, 0x62, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x46, 0x25b, 0x69, 0x20,
0x42, 0x65, 0x69, 0x61, 0x14b, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x42, 0x6f, 0x6b, 0xe3, 0x67, 0x62, 0x25b, 0x62, 0x69, 0x69, 0x20,
0x41, 0x62, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73, 0x61,
0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x47, 0x254, 0x6a, 0x69, 0x61, 0x6e, 0x254, 0x62, 0x69, 0x69,
0x20, 0x41, 0x62, 0x65, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x47, 0x254, 0x6a, 0x69, 0x61, 0x6e, 0x254, 0x62, 0x69, 0x69, 0x20, 0x41,
0x62, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x46, 0x25b, 0x69, 0x20, 0x42, 0x65,
0x69, 0x61, 0x14b, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x47,
0x254, 0x6a, 0x69, 0x61, 0x6e, 0x254, 0x62, 0x69, 0x69, 0x20, 0x41, 0x62,
0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73, 0x61, 0x20, 0x42,
0x65, 0x69, 0x61, 0x14b, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20,
0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x46, 0x25b, 0x69, 0x20, 0x42, 0x65,
0x69, 0x61, 0x14b, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x42,
0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73, 0x61, 0x20, 0x42,
0x65, 0x69, 0x61, 0x14b, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b,
0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x46, 0x25b, 0x69, 0x20, 0x42,
0x65, 0x69, 0x61, 0x14b, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b,
0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73, 0x61,
0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73,
0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x46, 0x25b, 0x69, 0x20, 0x42,
0x65, 0x69, 0x61, 0x14b, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x20, 0x42,
0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73, 0x61, 0x20, 0x42,
0x65, 0x69, 0x61, 0x14b, 0x4b, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72,
0x64, 0x65, 0x20, 0x42, 0x65, 0x4b, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65,
0x72, 0x64, 0x65, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x46, 0x25b,
0x69, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x4b, 0x61, 0x70, 0x65, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20,
0x4c, 0x61, 0x74, 0x73, 0x61, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x4b,
0x75, 0x62, 0x61, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x46, 0x25b,
0x69, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x4b, 0x75, 0x62, 0x61, 0x20,
0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73, 0x61, 0x20,
0x42, 0x65, 0x69, 0x61, 0x14b, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74,
0x69, 0x6b, 0x20, 0x4b, 0x25b, 0x20, 0x57, 0x75, 0x6f, 0x79, 0x69, 0x67,
0x62, 0x25b, 0x62, 0x69, 0x69, 0x20, 0x4e, 0x69, 0x20, 0x57, 0x69, 0x65,
0x254, 0x20, 0x46, 0x72, 0x25b, 0x6e, 0x74, 0x73, 0x69, 0x20, 0x42, 0x65,
0x42, 0x65, 0x74, 0x73, 0x254, 0x254, 0x6d, 0x254, 0x20, 0x6e, 0x69, 0x20,
0x6b, 0x61, 0x20, 0x14b, 0x25b, 0x6c, 0x25b, 0x20, 0x6b, 0x6f, 0x6d, 0x65,
0x20, 0x6e, 0x254, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x42, 0x6f, 0x6b, 0xe3, 0x67, 0x62, 0x25b, 0x20, 0x42, 0x65, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x42, 0x6f, 0x6b,
0xe3, 0x67, 0x62, 0x25b, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x46,
0x25b, 0x69, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x42, 0x6f, 0x6b, 0xe3, 0x67, 0x62,
0x25b, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73,
0x61, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x41, 0x6e, 0x61, 0x69, 0x67, 0x62, 0x25b,
0x20, 0x42, 0x65, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x41, 0x6e, 0x61, 0x69, 0x67, 0x62, 0x25b, 0x20, 0x42, 0x65, 0x20,
0x59, 0x25b, 0x20, 0x46, 0x25b, 0x69, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x41, 0x6e,
0x61, 0x69, 0x67, 0x62, 0x25b, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20,
0x4c, 0x61, 0x74, 0x73, 0x61, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x48,
0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69,
0x61, 0x20, 0x42, 0x65, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b,
0x20, 0x46, 0x25b, 0x69, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x48, 0x61,
0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61,
0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73, 0x61,
0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e,
0x20, 0x14a, 0x73, 0x68, 0x254, 0x20, 0x4c, 0x25b, 0x20, 0x42, 0x65, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x20, 0x42, 0x65, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x20, 0x42, 0x65, 0x20,
0x59, 0x25b, 0x20, 0x46, 0x25b, 0x69, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x20, 0x42, 0x65,
0x20, 0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73, 0x61, 0x20, 0x42, 0x65,
0x69, 0x61, 0x14b, 0x4d, 0x65, 0x7a, 0x69, 0x6b, 0x6f, 0x20, 0x50, 0x61,
0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x42, 0x65, 0x4d, 0x65, 0x7a, 0x69,
0x6b, 0x6f, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x42,
0x65, 0x20, 0x59, 0x25b, 0x20, 0x46, 0x25b, 0x69, 0x20, 0x42, 0x65, 0x69,
0x61, 0x14b, 0x4d, 0x65, 0x7a, 0x69, 0x6b, 0x6f, 0x20, 0x50, 0x61, 0x73,
0x69, 0x66, 0x69, 0x6b, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c,
0x61, 0x74, 0x73, 0x61, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x4e, 0x65,
0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x42,
0x65, 0x20, 0x59, 0x25b, 0x20, 0x46, 0x25b, 0x69, 0x20, 0x42, 0x65, 0x69,
0x61, 0x14b, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74,
0x73, 0x61, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x53, 0x74, 0x2e, 0x20,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x4b, 0x25b, 0x20, 0x4d, 0x69,
0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x42, 0x65, 0x53, 0x74, 0x2e, 0x20,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x4b, 0x25b, 0x20, 0x4d, 0x69,
0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20,
0x46, 0x25b, 0x69, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x53, 0x74, 0x2e,
0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x4b, 0x25b, 0x20, 0x4d,
0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b,
0x20, 0x4c, 0x61, 0x74, 0x73, 0x61, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b,
0x53, 0x65, 0x79, 0x73, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x42,
0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x63, 0x72,
0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x66, 0x67,
0x61, 0x6e, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4d, 0x65, 0x72, 0x69, 0x64,
0x69, 0x6f, 0x6e, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0xc1,
0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0xc1, 0x66, 0x72,
0x69, 0x63, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6c,
0x61, 0x73, 0x63, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6c, 0x61,
0x73, 0x63, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6c, 0x61, 0x73,
0x63, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x6f, 0x20, 0x41,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x6f, 0x20,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x2c, 0x20, 0x4e, 0x6f,
0x72, 0x74, 0x65, 0x61, 0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x2c, 0x20, 0x4e, 0x6f, 0x72,
0x74, 0x65, 0x61, 0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x2c, 0x20, 0x4e, 0x6f, 0x72, 0x74,
0x65, 0x61, 0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x6f, 0x20, 0x6c, 0x65, 0x73, 0x74, 0x65, 0x2c, 0x20, 0x41,
0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4e, 0x6f,
0x72, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x6f, 0x20, 0x6c, 0x65, 0x73, 0x74,
0x65, 0x2c, 0x20, 0x41, 0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64,
0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x6f, 0x20,
0x6c, 0x65, 0x73, 0x74, 0x65, 0x2c, 0x20, 0x41, 0x6d, 0xe9, 0x72, 0x69,
0x63, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61,
0xf1, 0x61, 0x2c, 0x20, 0x41, 0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61, 0x20,
0x64, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x61,
0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0xf1, 0x61, 0x2c, 0x20, 0x41, 0x6d,
0xe9, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4e, 0x6f, 0x72,
0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe1, 0x6e, 0x20, 0x64, 0x61, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61,
0xf1, 0x61, 0x2c, 0x20, 0x41, 0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61, 0x20,
0x64, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f,
0x2c, 0x20, 0x41, 0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x6f,
0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x6f, 0x20, 0x50,
0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x2c, 0x20, 0x41, 0x6d, 0xe9,
0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74,
0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe1, 0x6e, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69,
0x63, 0x6f, 0x2c, 0x20, 0x41, 0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61, 0x20,
0x64, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x48, 0x6f,
0x72, 0x61, 0x72, 0x69, 0x6f, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72,
0x48, 0x6f, 0x72, 0x61, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6e, 0x61, 0x64,
0x69, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70,
0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e,
0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0xe1, 0x72, 0x61, 0x62, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0xe1, 0x72, 0x61, 0x62,
0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe1, 0x6e, 0x20, 0xe1, 0x72, 0x61, 0x62, 0x65, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0x78, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0x78, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0x78, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61,
0x20, 0x41, 0x72, 0x78, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x4f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x61, 0x20, 0x41, 0x72, 0x78, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20,
0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20,
0x64, 0x61, 0x20, 0x41, 0x72, 0x78, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e,
0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e,
0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e,
0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x74,
0x6c, 0xe1, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x6f, 0x20,
0x41, 0x74, 0x6c, 0xe1, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64,
0x6f, 0x20, 0x41, 0x74, 0x6c, 0xe1, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20,
0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x4f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x63,
0x65, 0x72, 0x62, 0x61, 0x69, 0x78, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x41, 0x63, 0x65, 0x72, 0x62, 0x61, 0x69, 0x78, 0xe1, 0x6e, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e,
0x20, 0x64, 0x65, 0x20, 0x41, 0x63, 0x65, 0x72, 0x62, 0x61, 0x69, 0x78,
0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x73, 0x20, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x6f, 0x73, 0x20, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x6f, 0x73, 0x20, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x73, 0x68, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x73, 0x68, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x42, 0x75, 0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c,
0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42,
0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73,
0x61, 0x6c, 0x61, 0x6d, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x43, 0x61, 0x62, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x43, 0x61, 0x62, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64,
0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x62, 0x6f, 0x20, 0x56,
0x65, 0x72, 0x64, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x63, 0x68, 0x61, 0x6d, 0x6f, 0x72,
0x72, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43,
0x68, 0x69, 0x6c, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69,
0x6c, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x43, 0x68,
0x69, 0x6e, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6e,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe1, 0x6e, 0x20, 0x64, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6c, 0x6c, 0x61, 0x20,
0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x6c, 0x61, 0x73, 0x20,
0x43, 0x6f, 0x63, 0x6f, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20,
0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x6c, 0x61,
0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x61, 0x73, 0x20,
0x49, 0x6c, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20,
0x6d, 0x65, 0x64, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c,
0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x43, 0x75, 0x62, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75,
0x62, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x44, 0x61,
0x76, 0x69, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x44,
0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69,
0x6c, 0x6c, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54,
0x69, 0x6d, 0x6f, 0x72, 0x20, 0x4c, 0x65, 0x73, 0x74, 0x65, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6c, 0x6c, 0x61, 0x20, 0x64,
0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x61,
0x20, 0x49, 0x6c, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73,
0x63, 0x75, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6c, 0x6c, 0x61,
0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f,
0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x65, 0x78, 0x74, 0x72, 0x65, 0x6d, 0x6f, 0x20, 0x6c, 0x65, 0x73, 0x74,
0x65, 0x20, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6f, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x61, 0x73, 0x20, 0x49, 0x6c, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c,
0x76, 0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49,
0x6c, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61,
0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe1, 0x6e, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x6c, 0x61, 0x73,
0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x78, 0x69, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x46, 0x69, 0x78, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x46,
0x69, 0x78, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x47,
0xfc, 0x69, 0x61, 0x6e, 0x61, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65,
0x73, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x54,
0x65, 0x72, 0x72, 0x61, 0x73, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x69, 0x73, 0x20, 0x65, 0x20, 0x41, 0x6e, 0x74, 0xe1, 0x72, 0x74, 0x69,
0x63, 0x61, 0x73, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x61,
0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x47, 0x61,
0x6c, 0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x58, 0x65, 0x6f, 0x72, 0x78, 0x69,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x58, 0x65, 0x6f, 0x72, 0x78, 0x69,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x58, 0x65, 0x6f, 0x72, 0x78, 0x69,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c,
0x6c, 0x61, 0x73, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x6d, 0x65, 0x72, 0x69, 0x64,
0x69, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x77, 0x69, 0x63, 0x68, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61,
0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20,
0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72,
0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65,
0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61,
0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x47, 0x6f, 0x6c, 0x66, 0x6f,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61,
0x69, 0x2d, 0x69, 0x6c, 0x6c, 0x61, 0x73, 0x20, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x69, 0x61, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x48,
0x61, 0x77, 0x61, 0x69, 0x2d, 0x69, 0x6c, 0x6c, 0x61, 0x73, 0x20, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64,
0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x2d, 0x69, 0x6c, 0x6c, 0x61,
0x73, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x61, 0x73,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67,
0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f,
0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20,
0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x48,
0x6f, 0x76, 0x64, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x4f, 0x63, 0xe9, 0x61, 0x6e, 0x6f, 0x20, 0xcd, 0x6e, 0x64, 0x69, 0x63,
0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x6e, 0x64,
0x6f, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61,
0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73,
0x69, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0xe1,
0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0xe1, 0x6e, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20,
0x64, 0x65, 0x20, 0x49, 0x72, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20,
0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e,
0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x58, 0x61, 0x70, 0xf3, 0x6e, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x6f, 0x20, 0x58, 0x61, 0x70, 0xf3, 0x6e, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64,
0x6f, 0x20, 0x58, 0x61, 0x70, 0xf3, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73,
0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69,
0x48, 0x6f, 0x72, 0x61, 0x72, 0x69, 0x6f, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72,
0x6f, 0x70, 0xe1, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61,
0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x48, 0x6f, 0x72, 0x61,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e,
0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0xe1, 0x76,
0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61,
0x74, 0x73, 0x6b, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x69,
0x73, 0x74, 0xe1, 0x6e, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x7a,
0x61, 0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0x4f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x43, 0x6f, 0x72, 0x65, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f,
0x72, 0x65, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b,
0x6f, 0x73, 0x72, 0x61, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x69, 0x61, 0x72, 0x73, 0x6b,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x69,
0x61, 0x72, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x69, 0x61, 0x72, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4b, 0x69, 0x72, 0x67, 0x75, 0x69, 0x7a, 0x69,
0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73,
0x20, 0x49, 0x6c, 0x6c, 0x61, 0x73, 0x20, 0x64, 0x61, 0x20, 0x4c, 0x69,
0xf1, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64,
0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61,
0x6c, 0x61, 0x69, 0x73, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x61, 0x73, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x71,
0x75, 0x65, 0x73, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61,
0x73, 0x20, 0x49, 0x6c, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x73,
0x68, 0x61, 0x6c, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6f, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6f, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6f, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x63, 0xed,
0x66, 0x69, 0x63, 0x6f, 0x20, 0x6d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e,
0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69,
0x63, 0x6f, 0x20, 0x6d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x6f, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e,
0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f,
0x20, 0x6d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x6f, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61,
0x61, 0x74, 0x61, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0x61,
0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65,
0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63,
0x6f, 0x76, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63,
0x6f, 0x76, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63,
0x6f, 0x76, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x43,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x20, 0x4e, 0x6f,
0x76, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61,
0x20, 0x4e, 0x6f, 0x76, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x69, 0x75, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61,
0x20, 0x49, 0x6c, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6c, 0x6c, 0x61, 0x20, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x61, 0x20, 0x49,
0x6c, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d,
0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x71, 0x75, 0x69,
0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x71,
0x75, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x50,
0x61, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x70, 0xfa, 0x61, 0x2d, 0x4e,
0x6f, 0x76, 0x61, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x50, 0x65, 0x72, 0xfa, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x65, 0x72,
0xfa, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe1, 0x6e, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x65, 0x72, 0xfa, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69,
0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x6c, 0x61, 0x73, 0x20, 0x46, 0xe9,
0x6e, 0x69, 0x78, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20,
0x65, 0x74, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x74, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x69,
0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x74,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72,
0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x6f, 0x68,
0x6e, 0x70, 0x65, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x52, 0x65, 0x75, 0x6e, 0x69, 0xf3, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c,
0xed, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61,
0x6c, 0xed, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0xed, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x72, 0x69,
0x6f, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64,
0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x61, 0x73, 0x20, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c,
0x65, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x69,
0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x61, 0x73, 0x20, 0x49, 0x6c, 0x6c, 0x61, 0x73, 0x20, 0x53, 0x61, 0x6c,
0x6f, 0x6d, 0xf3, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x58, 0x65, 0x6f, 0x72, 0x78, 0x69, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x53,
0x75, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x75,
0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0xed, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20,
0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x78, 0x69, 0x71, 0x75, 0x69,
0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x43, 0x68, 0x75, 0x75, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0xe1,
0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x54,
0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x75, 0x76, 0x61, 0x6c,
0x75, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x6f, 0x20, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x6f, 0x20, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0xe1,
0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e,
0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1,
0x6e, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61,
0x20, 0x49, 0x6c, 0x6c, 0x61, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73,
0x20, 0x65, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x49, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1,
0x6e, 0x20, 0x64, 0x65, 0x20, 0x49, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62, 0x75,
0x72, 0x67, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x45, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x10d0, 0x10d5,
0x10e6, 0x10d0, 0x10dc, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10ea,
0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10e3, 0x10e0, 0x10d8, 0x20, 0x10d0, 0x10e4,
0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10db,
0x10dd, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10e4, 0x10e0, 0x10d8,
0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d0, 0x10db, 0x10ee, 0x10e0,
0x10d4, 0x10d7, 0x20, 0x10d0, 0x10e4, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10d3, 0x10d0, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d0,
0x10e4, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d0,
0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10e4, 0x10e0, 0x10d8, 0x10d9,
0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3,
0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d0, 0x10e1, 0x10d0, 0x10d5, 0x10da,
0x10d4, 0x10d7, 0x20, 0x10d0, 0x10e4, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d6,
0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0,
0x10da, 0x10d0, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10da,
0x10d0, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0,
0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10da, 0x10d0,
0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10db, 0x10d0, 0x10d6, 0x10dd, 0x10dc, 0x10d8,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10db, 0x10d0, 0x10d6, 0x10dd, 0x10dc,
0x10d8, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2,
0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10db, 0x10d0, 0x10d6, 0x10dd,
0x10dc, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10e0, 0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4,
0x10d7, 0x20, 0x10d0, 0x10db, 0x10d4, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10ea,
0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10e3, 0x10e0, 0x10d8, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10e9, 0x10e0, 0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10db,
0x10d4, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0,
0x10d0, 0x10da, 0x10e3, 0x10e0, 0x10d8, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0,
0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10e0, 0x10d3,
0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10db, 0x10d4, 0x10e0, 0x10d8, 0x10d9,
0x10d8, 0x10e1, 0x20, 0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10e3, 0x10e0,
0x10d8, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10e9, 0x10e0, 0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7, 0x20, 0x10d0,
0x10db, 0x10d4, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d0, 0x10e6, 0x10db, 0x10dd,
0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10e9, 0x10e0, 0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10db, 0x10d4,
0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1, 0x10d0,
0x10d5, 0x10da, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3,
0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10e0,
0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10db, 0x10d4, 0x10e0, 0x10d8,
0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5, 0x10da,
0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10e0, 0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4,
0x10d7, 0x20, 0x10d0, 0x10db, 0x10d4, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10db,
0x10d0, 0x10e3, 0x10dc, 0x10d7, 0x10d8, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10e9, 0x10e0, 0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10db, 0x10d4,
0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10db, 0x10d0, 0x10e3, 0x10dc, 0x10d7, 0x10d8,
0x10dc, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2,
0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10e0, 0x10d3, 0x10d8, 0x10da,
0x10dd, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10db, 0x10d4, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1,
0x20, 0x10db, 0x10d0, 0x10e3, 0x10dc, 0x10d7, 0x10d8, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d6,
0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9,
0x10e0, 0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10db, 0x10d4, 0x10e0,
0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10ec, 0x10e7, 0x10dc, 0x10d0, 0x10e0, 0x10d8, 0x20,
0x10dd, 0x10d9, 0x10d4, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9,
0x10e0, 0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10db, 0x10d4, 0x10e0,
0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10ec, 0x10e7, 0x10dc, 0x10d0, 0x10e0, 0x10d8, 0x20,
0x10dd, 0x10d9, 0x10d4, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc,
0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9,
0x10e0, 0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10db, 0x10d4, 0x10e0,
0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10ec, 0x10e7, 0x10dc, 0x10d0, 0x10e0, 0x10d8, 0x20,
0x10dd, 0x10d9, 0x10d4, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee,
0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10de, 0x10d8, 0x10d0,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10de, 0x10d8, 0x10d0, 0x10e1, 0x20, 0x10e1,
0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10d0, 0x10de, 0x10d8, 0x10d0, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee,
0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e0, 0x10d0, 0x10d1,
0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e0, 0x10d0, 0x10d1,
0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0,
0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e0, 0x10d0, 0x10d1,
0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e0, 0x10d2, 0x10d4, 0x10dc, 0x10e2, 0x10d8,
0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e0, 0x10d2, 0x10d4, 0x10dc,
0x10e2, 0x10d8, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0,
0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e0, 0x10d2,
0x10d4, 0x10dc, 0x10e2, 0x10d8, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee,
0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d0, 0x10e1, 0x10d0,
0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10e0, 0x10d2, 0x10d4, 0x10dc, 0x10e2, 0x10d8,
0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d0, 0x10e1, 0x10d0, 0x10d5,
0x10da, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10e0, 0x10d2, 0x10d4, 0x10dc, 0x10e2, 0x10d8, 0x10dc,
0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3,
0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d0, 0x10e1, 0x10d0, 0x10d5, 0x10da,
0x10d4, 0x10d7, 0x20, 0x10d0, 0x10e0, 0x10d2, 0x10d4, 0x10dc, 0x10e2, 0x10d8, 0x10dc, 0x10d8,
0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10e1, 0x10dd, 0x10db, 0x10ee, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10e1, 0x10dd, 0x10db, 0x10ee, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10e1,
0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10e1, 0x10dd, 0x10db, 0x10ee, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d6,
0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0,
0x10e2, 0x10da, 0x10d0, 0x10dc, 0x10e2, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10dd, 0x10d9,
0x10d4, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e2, 0x10da,
0x10d0, 0x10dc, 0x10e2, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10dd, 0x10d9, 0x10d4, 0x10d0,
0x10dc, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2,
0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e2, 0x10da, 0x10d0, 0x10dc,
0x10e2, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10dd, 0x10d9, 0x10d4, 0x10d0, 0x10dc, 0x10d8,
0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10e3, 0x10e0, 0x10d8,
0x20, 0x10d0, 0x10d5, 0x10e1, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10d8, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10d5, 0x10e1, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10d8, 0x10d8,
0x10e1, 0x20, 0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10e3, 0x10e0, 0x10d8,
0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10d5, 0x10e1, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10d8,
0x10d8, 0x10e1, 0x20, 0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10e3, 0x10e0,
0x10d8, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10e3, 0x10e0, 0x10d8,
0x20, 0x10d3, 0x10d0, 0x20, 0x10d3, 0x10d0, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7,
0x20, 0x10d0, 0x10d5, 0x10e1, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10d8, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10e3, 0x10e0,
0x10d8, 0x20, 0x10d3, 0x10d0, 0x20, 0x10d3, 0x10d0, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4,
0x10d7, 0x20, 0x10d0, 0x10d5, 0x10e1, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10d8, 0x10d8, 0x10e1,
0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10e3,
0x10e0, 0x10d8, 0x20, 0x10d3, 0x10d0, 0x20, 0x10d3, 0x10d0, 0x10e1, 0x10d0, 0x10d5, 0x10da,
0x10d4, 0x10d7, 0x20, 0x10d0, 0x10d5, 0x10e1, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10d8, 0x10d8,
0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7,
0x20, 0x10d0, 0x10d5, 0x10e1, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10d8, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4,
0x10d7, 0x20, 0x10d0, 0x10d5, 0x10e1, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10d8, 0x10d8, 0x10e1,
0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5, 0x10da,
0x10d4, 0x10d7, 0x20, 0x10d0, 0x10d5, 0x10e1, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10d8, 0x10d8,
0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10d0, 0x10d6, 0x10d4, 0x10e0, 0x10d1, 0x10d0, 0x10d8, 0x10ef, 0x10d0, 0x10dc,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10d6, 0x10d4, 0x10e0, 0x10d1, 0x10d0,
0x10d8, 0x10ef, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3,
0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10d6,
0x10d4, 0x10e0, 0x10d1, 0x10d0, 0x10d8, 0x10ef, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d6,
0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0,
0x10d6, 0x10dd, 0x10e0, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da,
0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10d6, 0x10dd, 0x10e0,
0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8,
0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da,
0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10d6, 0x10dd, 0x10e0, 0x10d8, 0x10e1, 0x20,
0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d6,
0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d1,
0x10d0, 0x10dc, 0x10d2, 0x10da, 0x10d0, 0x10d3, 0x10d4, 0x10e8, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10d1, 0x10d0, 0x10dc, 0x10d2, 0x10da, 0x10d0, 0x10d3, 0x10d4, 0x10e8, 0x10d8,
0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da,
0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d1, 0x10d0, 0x10dc, 0x10d2, 0x10da, 0x10d0, 0x10d3,
0x10d4, 0x10e8, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d1, 0x10e3, 0x10e2, 0x10d0, 0x10dc, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d1, 0x10dd, 0x10da, 0x10d8, 0x10d5, 0x10d8, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d1, 0x10e0, 0x10d0, 0x10d6, 0x10d8, 0x10da, 0x10d8, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d1, 0x10e0, 0x10d0, 0x10d6, 0x10d8, 0x10da, 0x10d8,
0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3,
0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d1, 0x10e0, 0x10d0, 0x10d6, 0x10d8, 0x10da,
0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d1, 0x10e0, 0x10e3, 0x10dc, 0x10d4, 0x10d8, 0x2d, 0x10d3,
0x10d0, 0x10e0, 0x10e3, 0x10e1, 0x10d0, 0x10da, 0x10d0, 0x10db, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10d9, 0x10d0, 0x10d1, 0x10dd, 0x2d, 0x10d5, 0x10d4, 0x10e0, 0x10d3, 0x10d4,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10d0, 0x10d1, 0x10dd, 0x2d, 0x10d5, 0x10d4,
0x10e0, 0x10d3, 0x10d4, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0,
0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10d0, 0x10d1, 0x10dd,
0x2d, 0x10d5, 0x10d4, 0x10e0, 0x10d3, 0x10d4, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee,
0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10d0, 0x10db, 0x10dd,
0x10e0, 0x10dd, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10d0, 0x10e2, 0x10d4, 0x10db,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10d0, 0x10e2, 0x10d4, 0x10db, 0x10d8,
0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da,
0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10d0, 0x10e2, 0x10d4, 0x10db, 0x10d8, 0x10e1,
0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10e9, 0x10d8, 0x10da, 0x10d4, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10d8,
0x10da, 0x10d4, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2,
0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10d8, 0x10da, 0x10d4, 0x10e1,
0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10e9, 0x10d8, 0x10dc, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10e9, 0x10d8, 0x10dc, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc,
0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9,
0x10d8, 0x10dc, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10e1,
0x20, 0x10e1, 0x10d0, 0x10e0, 0x10e2, 0x10e7, 0x10d4, 0x10da, 0x10d8, 0x10e8, 0x10dd, 0x10d1,
0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10e5, 0x10dd, 0x10e5, 0x10dd, 0x10e1, 0x10d8, 0x10e1, 0x20, 0x10d9,
0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10d9, 0x10dd, 0x10da, 0x10e3, 0x10db, 0x10d1, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10d9, 0x10dd, 0x10da, 0x10e3, 0x10db, 0x10d1, 0x10d8, 0x10d8, 0x10e1, 0x20,
0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10dd, 0x10da, 0x10e3, 0x10db, 0x10d1, 0x10d8, 0x10d8, 0x10e1,
0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10d9, 0x10e3, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3,
0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10e3, 0x10d9,
0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8,
0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da,
0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10e3, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d9,
0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10dc, 0x10d0,
0x10ee, 0x10d4, 0x10d5, 0x10e0, 0x10d0, 0x10d3, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3,
0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10e3, 0x10d1, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10e3, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2,
0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10d9, 0x10e3, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3,
0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d4, 0x10d5, 0x10d8, 0x10e1,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10e3, 0x10db, 0x10dd, 0x10dc, 0x2d,
0x10d3, 0x10e3, 0x10e0, 0x10d5, 0x10d8, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10e2,
0x10d8, 0x10db, 0x10dd, 0x10e0, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e6,
0x10d3, 0x10d2, 0x10dd, 0x10db, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3,
0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10d3, 0x10d2, 0x10dd,
0x10db, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d8, 0x10e1,
0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10d3, 0x10d2, 0x10dd, 0x10db, 0x10d8, 0x10e1,
0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0,
0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d4, 0x10d9,
0x10d5, 0x10d0, 0x10d3, 0x10dd, 0x10e0, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10ea,
0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10e3, 0x10e0, 0x10d8, 0x20, 0x10d4, 0x10d5,
0x10e0, 0x10dd, 0x10de, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10ea, 0x10d4, 0x10dc,
0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10e3, 0x10e0, 0x10d8, 0x20, 0x10d4, 0x10d5, 0x10e0, 0x10dd,
0x10de, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2,
0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0,
0x10d0, 0x10da, 0x10e3, 0x10e0, 0x10d8, 0x20, 0x10d4, 0x10d5, 0x10e0, 0x10dd, 0x10de, 0x10d8,
0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7,
0x20, 0x10d4, 0x10d5, 0x10e0, 0x10dd, 0x10de, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d4,
0x10d5, 0x10e0, 0x10dd, 0x10de, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3,
0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e6,
0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d4, 0x10d5, 0x10e0,
0x10dd, 0x10de, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e8, 0x10dd, 0x10e0, 0x10d4, 0x10e3, 0x10da, 0x10d8,
0x20, 0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20,
0x10d4, 0x10d5, 0x10e0, 0x10dd, 0x10de, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3,
0x10d0, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d4, 0x10d5, 0x10e0, 0x10dd,
0x10de, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d0, 0x10e1, 0x10d0, 0x10d5,
0x10da, 0x10d4, 0x10d7, 0x20, 0x10d4, 0x10d5, 0x10e0, 0x10dd, 0x10de, 0x10d8, 0x10e1, 0x20,
0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d0, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20,
0x10d4, 0x10d5, 0x10e0, 0x10dd, 0x10de, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee,
0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e4, 0x10dd, 0x10da, 0x10d9,
0x10da, 0x10d4, 0x10dc, 0x10d3, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3,
0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e4, 0x10dd, 0x10da,
0x10d9, 0x10da, 0x10d4, 0x10dc, 0x10d3, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb,
0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3,
0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e4, 0x10dd,
0x10da, 0x10d9, 0x10da, 0x10d4, 0x10dc, 0x10d3, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc,
0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee,
0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e4, 0x10d8, 0x10ef, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e4, 0x10d8, 0x10ef, 0x10d8, 0x10e1, 0x20, 0x10e1,
0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10e4, 0x10d8, 0x10ef, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee,
0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d0, 0x10e4, 0x10e0,
0x10d0, 0x10dc, 0x10d2, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d2, 0x10d5, 0x10d8, 0x10d0,
0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e4, 0x10e0, 0x10d0, 0x10dc, 0x10d2,
0x10e3, 0x10da, 0x10d8, 0x20, 0x10e1, 0x10d0, 0x10db, 0x10ee, 0x10e0, 0x10d4, 0x10d7, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10d0, 0x20, 0x10d0, 0x10dc, 0x10e2, 0x10d0, 0x10e0, 0x10e5, 0x10e2,
0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d2, 0x10d0, 0x10da, 0x10d0,
0x10de, 0x10d0, 0x10d2, 0x10dd, 0x10e1, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d2,
0x10d0, 0x10db, 0x10d1, 0x10d8, 0x10d4, 0x10e0, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10e1, 0x10d0, 0x10e5, 0x10d0, 0x10e0, 0x10d7, 0x10d5, 0x10d4, 0x10da, 0x10dd, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d0, 0x10e5, 0x10d0, 0x10e0, 0x10d7, 0x10d5, 0x10d4, 0x10da,
0x10dd, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3,
0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d0, 0x10e5, 0x10d0, 0x10e0, 0x10d7,
0x10d5, 0x10d4, 0x10da, 0x10dd, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d2, 0x10d8, 0x10da, 0x10d1, 0x10d4, 0x10e0,
0x10e2, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d2, 0x10e0, 0x10d8, 0x10dc, 0x10d5, 0x10d8,
0x10e9, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10d0, 0x10e8, 0x10e3, 0x10d0, 0x10da, 0x10dd, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4,
0x10d7, 0x20, 0x10d2, 0x10e0, 0x10d4, 0x10dc, 0x10da, 0x10d0, 0x10dc, 0x10d3, 0x10d8, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5,
0x10da, 0x10d4, 0x10d7, 0x20, 0x10d2, 0x10e0, 0x10d4, 0x10dc, 0x10da, 0x10d0, 0x10dc, 0x10d3,
0x10d8, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2,
0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1,
0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d2, 0x10e0, 0x10d4, 0x10dc, 0x10da, 0x10d0,
0x10dc, 0x10d3, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d0, 0x10e1, 0x10d0, 0x10d5, 0x10da,
0x10d4, 0x10d7, 0x20, 0x10d2, 0x10e0, 0x10d4, 0x10dc, 0x10da, 0x10d0, 0x10dc, 0x10d3, 0x10d8,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d0, 0x10e1, 0x10d0, 0x10d5, 0x10da,
0x10d4, 0x10d7, 0x20, 0x10d2, 0x10e0, 0x10d4, 0x10dc, 0x10da, 0x10d0, 0x10dc, 0x10d3, 0x10d8,
0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3,
0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d0, 0x10e1, 0x10d0, 0x10d5, 0x10da,
0x10d4, 0x10d7, 0x20, 0x10d2, 0x10e0, 0x10d4, 0x10dc, 0x10da, 0x10d0, 0x10dc, 0x10d3, 0x10d8,
0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10de, 0x10d0, 0x10e0, 0x10e1, 0x10d4, 0x10d7, 0x10d8, 0x10e1,
0x20, 0x10e7, 0x10e3, 0x10e0, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3,
0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d2, 0x10d0,
0x10d8, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10f0, 0x10d0, 0x10d5,
0x10d0, 0x10d8, 0x10e1, 0x10d0, 0x20, 0x10d3, 0x10d0, 0x20, 0x10d0, 0x10da, 0x10d4, 0x10e3,
0x10e2, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10f0, 0x10d0, 0x10d5, 0x10d0, 0x10d8,
0x10e1, 0x10d0, 0x20, 0x10d3, 0x10d0, 0x20, 0x10d0, 0x10da, 0x10d4, 0x10e3, 0x10e2, 0x10d8,
0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da,
0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10f0, 0x10d0, 0x10d5, 0x10d0, 0x10d8, 0x10e1, 0x10d0,
0x20, 0x10d3, 0x10d0, 0x20, 0x10d0, 0x10da, 0x10d4, 0x10e3, 0x10e2, 0x10d8, 0x10e1, 0x20,
0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10f0, 0x10dd, 0x10dc, 0x10d9, 0x10dd, 0x10dc, 0x10d2, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10f0, 0x10dd, 0x10dc, 0x10d9, 0x10dd, 0x10dc, 0x10d2, 0x10d8, 0x10e1, 0x20, 0x10e1,
0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10f0, 0x10dd, 0x10dc, 0x10d9, 0x10dd, 0x10dc, 0x10d2, 0x10d8, 0x10e1, 0x20,
0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10f0, 0x10dd, 0x10d5, 0x10d3, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10f0, 0x10dd,
0x10d5, 0x10d3, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0,
0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10f0, 0x10dd, 0x10d5, 0x10d3,
0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10dc, 0x10d3, 0x10dd, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10dc, 0x10d3, 0x10dd, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20,
0x10dd, 0x10d9, 0x10d4, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb,
0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10dc,
0x10d3, 0x10dd, 0x10e9, 0x10d8, 0x10dc, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10e3, 0x10e0, 0x10d8, 0x20,
0x10d8, 0x10dc, 0x10d3, 0x10dd, 0x10dc, 0x10d4, 0x10d6, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7,
0x20, 0x10d8, 0x10dc, 0x10d3, 0x10dd, 0x10dc, 0x10d4, 0x10d6, 0x10d8, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d0, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20,
0x10d8, 0x10dc, 0x10d3, 0x10dd, 0x10dc, 0x10d4, 0x10d6, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10d8, 0x10e0, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10d8, 0x10e0, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3,
0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10e0,
0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10e1, 0x20, 0x10e1,
0x10d0, 0x10e0, 0x10e2, 0x10e7, 0x10d4, 0x10da, 0x10d8, 0x10d8, 0x10e0, 0x10d9, 0x10e3, 0x10e2,
0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10e0, 0x10d9, 0x10e3,
0x10e2, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0,
0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10e0, 0x10d9,
0x10e3, 0x10e2, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3,
0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10e1, 0x10e0, 0x10d0, 0x10d4,
0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10e1, 0x10e0, 0x10d0, 0x10d4,
0x10da, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2,
0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10e1, 0x10e0, 0x10d0, 0x10d4,
0x10da, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10d0, 0x10de, 0x10dd, 0x10dc, 0x10d8, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10d0, 0x10de, 0x10dd, 0x10dc, 0x10d8, 0x10d8, 0x10e1,
0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10d0, 0x10de, 0x10dd, 0x10dc, 0x10d8, 0x10d8, 0x10e1,
0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10e7, 0x10d0, 0x10d6, 0x10d0, 0x10ee, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7,
0x20, 0x10e7, 0x10d0, 0x10d6, 0x10d0, 0x10ee, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10d3, 0x10d0, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10e7,
0x10d0, 0x10d6, 0x10d0, 0x10ee, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10d9, 0x10dd, 0x10e0, 0x10d4, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10dd,
0x10e0, 0x10d4, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0,
0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10dd, 0x10e0, 0x10d4,
0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10dd, 0x10e1, 0x10e0, 0x10d4, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10d9, 0x10e0, 0x10d0, 0x10e1, 0x10dc, 0x10dd, 0x10d8, 0x10d0, 0x10e0, 0x10e1, 0x10d9,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10e0, 0x10d0, 0x10e1, 0x10dc, 0x10dd,
0x10d8, 0x10d0, 0x10e0, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc,
0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9,
0x10e0, 0x10d0, 0x10e1, 0x10dc, 0x10dd, 0x10d8, 0x10d0, 0x10e0, 0x10e1, 0x10d9, 0x10d8, 0x10e1,
0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10e7, 0x10d8, 0x10e0, 0x10d2, 0x10d8, 0x10d6, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10da, 0x10d0, 0x10d8, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3,
0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10da, 0x10dd, 0x10e0, 0x10d3, 0x2d, 0x10f0, 0x10d0, 0x10e3, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10da, 0x10dd, 0x10e0, 0x10d3, 0x2d, 0x10f0, 0x10d0, 0x10e3, 0x10e1, 0x20, 0x10e1,
0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10da, 0x10dd, 0x10e0, 0x10d3, 0x2d, 0x10f0, 0x10d0, 0x10e3, 0x10e1, 0x20,
0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10db, 0x10d0, 0x10d2, 0x10d0, 0x10d3, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10db, 0x10d0, 0x10d2, 0x10d0, 0x10d3, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10e1,
0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10db, 0x10d0, 0x10d2, 0x10d0, 0x10d3, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20,
0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10db, 0x10d0, 0x10da, 0x10d0, 0x10d8, 0x10d6, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10db, 0x10d0, 0x10da, 0x10d3, 0x10d8, 0x10d5, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10d0, 0x10e0, 0x10d9, 0x10d8, 0x10d6, 0x10d8, 0x10e1, 0x20,
0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10db, 0x10d0, 0x10e0, 0x10e8, 0x10d0, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d9,
0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10db, 0x10d0, 0x10d5, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10db, 0x10d0, 0x10d5, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2,
0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10db, 0x10d0, 0x10d5, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0,
0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10dd,
0x10e3, 0x10e1, 0x10dd, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10d4,
0x10e5, 0x10e1, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10ec, 0x10e7, 0x10dc, 0x10d0, 0x10e0,
0x10d8, 0x20, 0x10dd, 0x10d9, 0x10d4, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10db, 0x10d4, 0x10e5, 0x10e1, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10ec, 0x10e7,
0x10dc, 0x10d0, 0x10e0, 0x10d8, 0x20, 0x10dd, 0x10d9, 0x10d4, 0x10d0, 0x10dc, 0x10d8, 0x10e1,
0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10d4, 0x10e5, 0x10e1, 0x10d8, 0x10d9, 0x10d8, 0x10e1,
0x20, 0x10ec, 0x10e7, 0x10dc, 0x10d0, 0x10e0, 0x10d8, 0x20, 0x10dd, 0x10d9, 0x10d4, 0x10d0,
0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e3, 0x10da, 0x10d0, 0x10dc, 0x2d, 0x10d1, 0x10d0, 0x10e2,
0x10dd, 0x10e0, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e3, 0x10da, 0x10d0, 0x10dc,
0x2d, 0x10d1, 0x10d0, 0x10e2, 0x10dd, 0x10e0, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0,
0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10e3, 0x10da, 0x10d0, 0x10dc, 0x2d, 0x10d1, 0x10d0, 0x10e2, 0x10dd, 0x10e0, 0x10d8, 0x10e1,
0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10db, 0x10dd, 0x10e1, 0x10d9, 0x10dd, 0x10d5, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10db, 0x10dd, 0x10e1, 0x10d9, 0x10dd, 0x10d5, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2,
0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10db, 0x10dd, 0x10e1, 0x10d9, 0x10dd, 0x10d5, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0,
0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10d8,
0x10d0, 0x10dc, 0x10db, 0x10d0, 0x10e0, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10dc,
0x10d0, 0x10e3, 0x10e0, 0x10e3, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10dc, 0x10d4, 0x10de,
0x10d0, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10ee, 0x10d0, 0x10da,
0x10d8, 0x20, 0x10d9, 0x10d0, 0x10da, 0x10d4, 0x10d3, 0x10dd, 0x10dc, 0x10d8, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10ee, 0x10d0, 0x10da, 0x10d8, 0x20, 0x10d9, 0x10d0,
0x10da, 0x10d4, 0x10d3, 0x10dd, 0x10dc, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0,
0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10d0, 0x10ee, 0x10d0, 0x10da, 0x10d8, 0x20, 0x10d9, 0x10d0, 0x10da, 0x10d4, 0x10d3, 0x10dd,
0x10dc, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10ee, 0x10d0, 0x10da, 0x10d8, 0x20, 0x10d6,
0x10d4, 0x10da, 0x10d0, 0x10dc, 0x10d3, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10d0, 0x10ee, 0x10d0, 0x10da, 0x10d8, 0x20, 0x10d6, 0x10d4, 0x10da, 0x10d0, 0x10dc, 0x10d3,
0x10d8, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2,
0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10ee, 0x10d0, 0x10da, 0x10d8,
0x20, 0x10d6, 0x10d4, 0x10da, 0x10d0, 0x10dc, 0x10d3, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d6,
0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10dc,
0x10d8, 0x10e3, 0x10e4, 0x10d0, 0x10e3, 0x10dc, 0x10d3, 0x10da, 0x10d4, 0x10dc, 0x10d3, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10dc, 0x10d8, 0x10e3, 0x10e4, 0x10d0, 0x10e3, 0x10dc,
0x10d3, 0x10da, 0x10d4, 0x10dc, 0x10d3, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc,
0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10dc,
0x10d8, 0x10e3, 0x10e4, 0x10d0, 0x10e3, 0x10dc, 0x10d3, 0x10da, 0x10d4, 0x10dc, 0x10d3, 0x10d8,
0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10dc, 0x10d8, 0x10e3, 0x10d4, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10dc,
0x10dd, 0x10e0, 0x10e4, 0x10dd, 0x10da, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc,
0x10eb, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10dc, 0x10dd, 0x10e0,
0x10e4, 0x10dd, 0x10da, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3,
0x10da, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2,
0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10dc, 0x10dd, 0x10e0, 0x10e4, 0x10dd,
0x10da, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d8,
0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10e4, 0x10d4, 0x10e0, 0x10dc, 0x10d0, 0x10dc, 0x10d3, 0x10dd, 0x2d, 0x10d3,
0x10d4, 0x2d, 0x10dc, 0x10dd, 0x10e0, 0x10dd, 0x10dc, 0x10d8, 0x10d0, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10e4, 0x10d4, 0x10e0, 0x10dc, 0x10d0, 0x10dc, 0x10d3, 0x10dd, 0x2d, 0x10d3,
0x10d4, 0x2d, 0x10dc, 0x10dd, 0x10e0, 0x10dd, 0x10dc, 0x10d8, 0x10d0, 0x10e1, 0x20, 0x10e1,
0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10e4, 0x10d4, 0x10e0, 0x10dc, 0x10d0, 0x10dc, 0x10d3, 0x10dd, 0x2d, 0x10d3,
0x10d4, 0x2d, 0x10dc, 0x10dd, 0x10e0, 0x10dd, 0x10dc, 0x10d8, 0x10d0, 0x10e1, 0x20, 0x10d6,
0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10dc,
0x10dd, 0x10d5, 0x10dd, 0x10e1, 0x10d8, 0x10d1, 0x10d8, 0x10e0, 0x10e1, 0x10d9, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10dc, 0x10dd, 0x10d5, 0x10dd, 0x10e1, 0x10d8, 0x10d1, 0x10d8,
0x10e0, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0,
0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10dc, 0x10dd, 0x10d5,
0x10dd, 0x10e1, 0x10d8, 0x10d1, 0x10d8, 0x10e0, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d6,
0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10dd,
0x10db, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10dd, 0x10db, 0x10e1,
0x10d9, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2,
0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10dd, 0x10db, 0x10e1, 0x10d9, 0x10d8,
0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10de, 0x10d0, 0x10d9, 0x10d8, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10de, 0x10d0, 0x10d9, 0x10d8, 0x10e1, 0x10e2, 0x10d0, 0x10dc,
0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3,
0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10de, 0x10d0, 0x10d9, 0x10d8, 0x10e1, 0x10e2,
0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10de, 0x10d0, 0x10da, 0x10d0, 0x10e3, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10de, 0x10d0, 0x10de, 0x10e3, 0x10d0, 0x2d, 0x10d0, 0x10ee, 0x10d0,
0x10da, 0x10d8, 0x20, 0x10d2, 0x10d5, 0x10d8, 0x10dc, 0x10d4, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10de, 0x10d0, 0x10e0, 0x10d0, 0x10d2, 0x10d5, 0x10d0, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10de, 0x10d0, 0x10e0, 0x10d0, 0x10d2, 0x10d5, 0x10d0, 0x10d8, 0x10e1,
0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10de, 0x10d0, 0x10e0, 0x10d0, 0x10d2, 0x10d5, 0x10d0, 0x10d8,
0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10de, 0x10d4, 0x10e0, 0x10e3, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10de,
0x10d4, 0x10e0, 0x10e3, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0,
0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10de, 0x10d4, 0x10e0, 0x10e3,
0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10e4, 0x10d8, 0x10da, 0x10d8, 0x10de, 0x10d8, 0x10dc, 0x10d4, 0x10d1, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e4, 0x10d8, 0x10da, 0x10d8, 0x10de, 0x10d8, 0x10dc,
0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0,
0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e4, 0x10d8, 0x10da, 0x10d8,
0x10de, 0x10d8, 0x10dc, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee,
0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e4, 0x10d4, 0x10dc, 0x10d8,
0x10e5, 0x10e1, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4,
0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d4, 0x10dc, 0x2d, 0x10de,
0x10d8, 0x10d4, 0x10e0, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10d0, 0x20, 0x10db, 0x10d8, 0x10d9,
0x10d4, 0x10da, 0x10dd, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d4,
0x10dc, 0x2d, 0x10de, 0x10d8, 0x10d4, 0x10e0, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10d0, 0x20,
0x10db, 0x10d8, 0x10d9, 0x10d4, 0x10da, 0x10dd, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2,
0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10e1, 0x10d4, 0x10dc, 0x2d, 0x10de, 0x10d8, 0x10d4, 0x10e0, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10d0, 0x20, 0x10db, 0x10d8, 0x10d9, 0x10d4, 0x10da, 0x10dd, 0x10dc, 0x10d8, 0x10e1,
0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10de, 0x10d8, 0x10e2, 0x10d9, 0x10d4, 0x10e0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10de, 0x10dd, 0x10dc, 0x10d0, 0x10de, 0x10d4, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10e4, 0x10ee, 0x10d4, 0x10dc, 0x10d8, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10e0, 0x10d4, 0x10d8, 0x10e3, 0x10dc, 0x10d8, 0x10dd, 0x10dc, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e0, 0x10dd, 0x10d7, 0x10d4, 0x10e0, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d0, 0x10ee, 0x10d0, 0x10da, 0x10d8, 0x10dc, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d0, 0x10ee, 0x10d0, 0x10da, 0x10d8, 0x10dc, 0x10d8,
0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da,
0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d0, 0x10ee, 0x10d0, 0x10da, 0x10d8, 0x10dc,
0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d0, 0x10db, 0x10dd, 0x10d0, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10e1, 0x10d0, 0x10db, 0x10dd, 0x10d0, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc,
0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1,
0x10d0, 0x10db, 0x10dd, 0x10d0, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d4, 0x10d8, 0x10e8, 0x10d4, 0x10da,
0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d8, 0x10dc, 0x10d2, 0x10d0, 0x10de, 0x10e3,
0x10e0, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10dd, 0x10da, 0x10dd, 0x10db,
0x10dd, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4,
0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d0, 0x10db, 0x10ee, 0x10e0,
0x10d4, 0x10d7, 0x20, 0x10d2, 0x10d4, 0x10dd, 0x10e0, 0x10d2, 0x10d8, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10e3, 0x10e0, 0x10d8, 0x10dc, 0x10d0, 0x10db, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d8, 0x10dd, 0x10d5, 0x10d0, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10e2, 0x10d0, 0x10d8, 0x10e2, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10e2, 0x10d0, 0x10d8, 0x10d1, 0x10d4, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e2,
0x10d0, 0x10d8, 0x10d1, 0x10d4, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3,
0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e2, 0x10d0,
0x10d8, 0x10d1, 0x10d4, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e2, 0x10d0, 0x10ef, 0x10d8, 0x10d9, 0x10d4,
0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e2, 0x10dd, 0x10d9, 0x10d4, 0x10da,
0x10d0, 0x10e3, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e2, 0x10dd, 0x10dc, 0x10d2, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e2, 0x10dd, 0x10dc, 0x10d2, 0x10d8, 0x10e1, 0x20,
0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10e2, 0x10dd, 0x10dc, 0x10d2, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0,
0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10e3,
0x10e3, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d7, 0x10e3, 0x10e0, 0x10e5,
0x10db, 0x10d4, 0x10dc, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d7,
0x10e3, 0x10e0, 0x10e5, 0x10db, 0x10d4, 0x10dc, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10e1,
0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10d7, 0x10e3, 0x10e0, 0x10e5, 0x10db, 0x10d4, 0x10dc, 0x10d4, 0x10d7, 0x10d8,
0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10e2, 0x10e3, 0x10d5, 0x10d0, 0x10da, 0x10e3, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10e3, 0x10e0, 0x10e3, 0x10d2, 0x10d5, 0x10d0, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10e3, 0x10e0, 0x10e3, 0x10d2, 0x10d5, 0x10d0, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2,
0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10e3, 0x10e0, 0x10e3, 0x10d2, 0x10d5, 0x10d0, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0,
0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e3, 0x10d6,
0x10d1, 0x10d4, 0x10d9, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e3,
0x10d6, 0x10d1, 0x10d4, 0x10d9, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0,
0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10e3, 0x10d6, 0x10d1, 0x10d4, 0x10d9, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0,
0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d5, 0x10d0,
0x10dc, 0x10e3, 0x10d0, 0x10e2, 0x10e3, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d5, 0x10d0,
0x10dc, 0x10e3, 0x10d0, 0x10e2, 0x10e3, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3,
0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d5, 0x10d0,
0x10dc, 0x10e3, 0x10d0, 0x10e2, 0x10e3, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3,
0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d5, 0x10d4, 0x10dc, 0x10d4, 0x10e1,
0x10e3, 0x10d4, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d5, 0x10da, 0x10d0,
0x10d3, 0x10d8, 0x10d5, 0x10dd, 0x10e1, 0x10e2, 0x10dd, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10d5, 0x10da, 0x10d0, 0x10d3, 0x10d8, 0x10d5, 0x10dd, 0x10e1, 0x10e2, 0x10dd,
0x10d9, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2,
0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d5, 0x10da, 0x10d0, 0x10d3, 0x10d8,
0x10d5, 0x10dd, 0x10e1, 0x10e2, 0x10dd, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4,
0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d5, 0x10dd, 0x10da,
0x10d2, 0x10dd, 0x10d2, 0x10e0, 0x10d0, 0x10d3, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10d5, 0x10dd, 0x10da, 0x10d2, 0x10dd, 0x10d2, 0x10e0, 0x10d0, 0x10d3, 0x10d8, 0x10e1, 0x20,
0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10d5, 0x10dd, 0x10da, 0x10d2, 0x10dd, 0x10d2, 0x10e0, 0x10d0, 0x10d3,
0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10d5, 0x10d4, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3,
0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d5, 0x10dd,
0x10da, 0x10d8, 0x10e1, 0x10d8, 0x20, 0x10d3, 0x10d0, 0x20, 0x10e4, 0x10e3, 0x10e2, 0x10e3,
0x10dc, 0x10d0, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10d0, 0x10d9, 0x10e3, 0x10e2,
0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10d0, 0x10d9, 0x10e3,
0x10e2, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0,
0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10d0, 0x10d9,
0x10e3, 0x10e2, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3,
0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d4, 0x10d9, 0x10d0, 0x10e2, 0x10d4,
0x10e0, 0x10d8, 0x10dc, 0x10d1, 0x10e3, 0x10e0, 0x10d2, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10d4, 0x10d9, 0x10d0, 0x10e2, 0x10d4, 0x10e0, 0x10d8, 0x10dc, 0x10d1, 0x10e3, 0x10e0,
0x10d2, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2,
0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d4, 0x10d9, 0x10d0, 0x10e2, 0x10d4,
0x10e0, 0x10d8, 0x10dc, 0x10d1, 0x10e3, 0x10e0, 0x10d2, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0,
0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10e3,
0x10d9, 0x10dd, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x41, 0x63, 0x72,
0x65, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x41, 0x63, 0x72, 0x65, 0x2d, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x63, 0x72,
0x65, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74,
0x41, 0x66, 0x67, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d,
0x5a, 0x65, 0x69, 0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x5a, 0x65, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74,
0x53, 0xfc, 0x64, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74,
0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x5a, 0x65, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x61, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74,
0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x41,
0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x41, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a,
0x65, 0x69, 0x74, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x53, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x6c, 0x6d, 0x61,
0x74, 0x79, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x41, 0x6c, 0x6d, 0x61, 0x74,
0x79, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74,
0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61,
0x73, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x61, 0x73, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69,
0x74, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x2d, 0x53, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4e, 0x6f, 0x72, 0x64,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68,
0x65, 0x20, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x7a, 0x65, 0x69,
0x74, 0x4e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69,
0x74, 0x4e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69,
0x74, 0x4e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4f, 0x73, 0x74, 0x6b, 0xfc,
0x73, 0x74, 0x65, 0x6e, 0x7a, 0x65, 0x69, 0x74, 0x4e, 0x6f, 0x72, 0x64,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68,
0x65, 0x20, 0x4f, 0x73, 0x74, 0x6b, 0xfc, 0x73, 0x74, 0x65, 0x6e, 0x2d,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4e, 0x6f,
0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x4f, 0x73, 0x74, 0x6b, 0xfc, 0x73, 0x74, 0x65,
0x6e, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74,
0x52, 0x6f, 0x63, 0x6b, 0x79, 0x2d, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61,
0x69, 0x6e, 0x73, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x52, 0x6f, 0x63, 0x6b,
0x79, 0x2d, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x73, 0x2d,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x52, 0x6f,
0x63, 0x6b, 0x79, 0x2d, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e,
0x73, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74,
0x4e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x57, 0x65, 0x73, 0x74, 0x6b, 0xfc,
0x73, 0x74, 0x65, 0x6e, 0x7a, 0x65, 0x69, 0x74, 0x4e, 0x6f, 0x72, 0x64,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68,
0x65, 0x20, 0x57, 0x65, 0x73, 0x74, 0x6b, 0xfc, 0x73, 0x74, 0x65, 0x6e,
0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4e,
0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x57, 0x65, 0x73, 0x74, 0x6b, 0xfc, 0x73,
0x74, 0x65, 0x6e, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65,
0x69, 0x74, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x20, 0x5a, 0x65, 0x69,
0x74, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x20, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x41,
0x70, 0x69, 0x61, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x41, 0x70, 0x69, 0x61,
0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x41,
0x70, 0x69, 0x61, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65,
0x69, 0x74, 0x41, 0x71, 0x74, 0x61, 0x75, 0x2d, 0x5a, 0x65, 0x69, 0x74,
0x41, 0x71, 0x74, 0x61, 0x75, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0x65, 0x69, 0x74, 0x41, 0x71, 0x74, 0x61, 0x75, 0x2d, 0x53, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x71, 0x74, 0xf6,
0x62, 0x65, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x41, 0x71, 0x74, 0xf6, 0x62,
0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74,
0x41, 0x71, 0x74, 0xf6, 0x62, 0x65, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x72, 0x61, 0x62, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x41, 0x72, 0x61, 0x62, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a,
0x65, 0x69, 0x74, 0x41, 0x72, 0x61, 0x62, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x5a, 0x65, 0x69, 0x74, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0x65, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x61, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a,
0x65, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x61, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x61,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x41,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65,
0x69, 0x74, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x41,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x74, 0x6c, 0x61,
0x6e, 0x74, 0x69, 0x6b, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x6b, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0x65, 0x69, 0x74, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b,
0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x5a,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x5a,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0x65, 0x69, 0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x5a,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x2d, 0x2f, 0x57, 0x65, 0x73, 0x74,
0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x5a, 0x65, 0x69, 0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x2d, 0x2f, 0x57, 0x65, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0x65, 0x69, 0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x2d, 0x2f, 0x57, 0x65, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0x65, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69,
0x74, 0x4f, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a,
0x65, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x69,
0x64, 0x73, 0x63, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x5a, 0x65, 0x69, 0x74, 0x41, 0x73, 0x65, 0x72, 0x62, 0x65, 0x69, 0x64,
0x73, 0x63, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x73, 0x65,
0x72, 0x62, 0x61, 0x69, 0x64, 0x73, 0x63, 0x68, 0x61, 0x6e, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65,
0x69, 0x74, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x6e, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x6e,
0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x63, 0x68, 0x2d, 0x5a,
0x65, 0x69, 0x74, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73,
0x63, 0x68, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69,
0x74, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x63, 0x68,
0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x42,
0x68, 0x75, 0x74, 0x61, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x42, 0x6f,
0x6c, 0x69, 0x76, 0x69, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x5a, 0x65, 0x69, 0x74, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61,
0x2d, 0x5a, 0x65, 0x69, 0x74, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69,
0x61, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74,
0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x2d, 0x53, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x42, 0x72, 0x75, 0x6e, 0x65,
0x69, 0x2d, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d,
0x2d, 0x5a, 0x65, 0x69, 0x74, 0x43, 0x61, 0x62, 0x6f, 0x2d, 0x56, 0x65,
0x72, 0x64, 0x65, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x43, 0x61, 0x62, 0x6f,
0x2d, 0x56, 0x65, 0x72, 0x64, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0x65, 0x69, 0x74, 0x43, 0x61, 0x62, 0x6f, 0x2d, 0x56, 0x65,
0x72, 0x64, 0x65, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65,
0x69, 0x74, 0x43, 0x61, 0x73, 0x65, 0x79, 0x2d, 0x5a, 0x65, 0x69, 0x74,
0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x2d, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x43, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69,
0x74, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x5a, 0x65, 0x69, 0x74, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x6e, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a,
0x65, 0x69, 0x74, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x6e, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69,
0x74, 0x43, 0x68, 0x69, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x5a, 0x65, 0x69, 0x74, 0x43, 0x68, 0x69, 0x6e, 0x65, 0x73, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a,
0x65, 0x69, 0x74, 0x43, 0x68, 0x69, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69,
0x74, 0x57, 0x65, 0x69, 0x68, 0x6e, 0x61, 0x63, 0x68, 0x74, 0x73, 0x69,
0x6e, 0x73, 0x65, 0x6c, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4b, 0x6f, 0x6b,
0x6f, 0x73, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x6e, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61, 0x6e, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x4b, 0x6f, 0x6c, 0x75,
0x6d, 0x62, 0x69, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4b, 0x6f, 0x6c,
0x75, 0x6d, 0x62, 0x69, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x43, 0x6f,
0x6f, 0x6b, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x6e, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x43, 0x6f, 0x6f, 0x6b, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x6e, 0x2d,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x43, 0x6f,
0x6f, 0x6b, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x6e, 0x2d, 0x53, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4b, 0x75, 0x62, 0x61, 0x6e,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x4b, 0x75,
0x62, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4b, 0x75, 0x62, 0x61, 0x6e,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0x65, 0x69, 0x74, 0x44, 0x61, 0x76, 0x69, 0x73, 0x2d, 0x5a, 0x65,
0x69, 0x74, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55,
0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4f,
0x73, 0x74, 0x74, 0x69, 0x6d, 0x6f, 0x72, 0x2d, 0x5a, 0x65, 0x69, 0x74,
0x4f, 0x73, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x2d, 0x5a,
0x65, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x73, 0x65,
0x6c, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74,
0x4f, 0x73, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x2d, 0x53,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x45, 0x63, 0x75,
0x61, 0x64, 0x6f, 0x72, 0x69, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x5a, 0x65, 0x69, 0x74, 0x4d, 0x69, 0x74, 0x74, 0x65, 0x6c, 0x65,
0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a,
0x65, 0x69, 0x74, 0x4d, 0x69, 0x74, 0x74, 0x65, 0x6c, 0x65, 0x75, 0x72,
0x6f, 0x70, 0xe4, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4d, 0x69, 0x74, 0x74, 0x65,
0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4f,
0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73, 0x63, 0x68,
0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x65, 0x75, 0x72,
0x6f, 0x70, 0xe4, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x65, 0x75,
0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4b, 0x61, 0x6c, 0x69,
0x6e, 0x69, 0x6e, 0x67, 0x72, 0x61, 0x64, 0x65, 0x72, 0x20, 0x5a, 0x65,
0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x57, 0x65,
0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73, 0x63, 0x68,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74,
0x57, 0x65, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65,
0x69, 0x74, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x6e,
0x73, 0x65, 0x6c, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x46, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x6e, 0x2d,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x46, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x6e,
0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x46,
0x69, 0x64, 0x73, 0x63, 0x68, 0x69, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x46,
0x69, 0x64, 0x73, 0x63, 0x68, 0x69, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0x65, 0x69, 0x74, 0x46, 0x69, 0x64, 0x73, 0x63, 0x68, 0x69,
0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x46,
0x72, 0x61, 0x6e, 0x7a, 0xf6, 0x73, 0x69, 0x73, 0x63, 0x68, 0x2d, 0x47,
0x75, 0x61, 0x79, 0x61, 0x6e, 0x61, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x46,
0x72, 0x61, 0x6e, 0x7a, 0xf6, 0x73, 0x69, 0x73, 0x63, 0x68, 0x65, 0x2d,
0x53, 0xfc, 0x64, 0x2d, 0x75, 0x6e, 0x64, 0x2d, 0x41, 0x6e, 0x74, 0x61,
0x72, 0x6b, 0x74, 0x69, 0x73, 0x67, 0x65, 0x62, 0x69, 0x65, 0x74, 0x65,
0x2d, 0x5a, 0x65, 0x69, 0x74, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67,
0x6f, 0x73, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x47, 0x61, 0x6d, 0x62, 0x69,
0x65, 0x72, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x47, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x47, 0x65,
0x6f, 0x72, 0x67, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x47, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0x65, 0x69, 0x74, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x2d,
0x49, 0x6e, 0x73, 0x65, 0x6c, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4d,
0x69, 0x74, 0x74, 0x6c, 0x65, 0x72, 0x65, 0x20, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x77, 0x69, 0x63, 0x68, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4f, 0x73,
0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x5a, 0x65,
0x69, 0x74, 0x4f, 0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74,
0x4f, 0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2d,
0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x57, 0x65,
0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x5a,
0x65, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65,
0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69,
0x74, 0x47, 0x75, 0x61, 0x6d, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x47, 0x6f,
0x6c, 0x66, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x47, 0x75, 0x79, 0x61, 0x6e,
0x61, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x65, 0x6e, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x65, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65,
0x69, 0x74, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65,
0x75, 0x74, 0x65, 0x6e, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0x65, 0x69, 0x74, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x2d,
0x5a, 0x65, 0x69, 0x74, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67,
0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x48,
0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x2d, 0x53, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x43, 0x68, 0x6f, 0x77, 0x64, 0x2d,
0x5a, 0x65, 0x69, 0x74, 0x43, 0x68, 0x6f, 0x77, 0x64, 0x2d, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x43, 0x68, 0x6f, 0x77,
0x64, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74,
0x49, 0x6e, 0x64, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x49, 0x6e, 0x64, 0x69, 0x73,
0x63, 0x68, 0x65, 0x72, 0x2d, 0x4f, 0x7a, 0x65, 0x61, 0x6e, 0x2d, 0x5a,
0x65, 0x69, 0x74, 0x49, 0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x61,
0x2d, 0x5a, 0x65, 0x69, 0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x5a, 0x65, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x69, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x73, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69,
0x74, 0x57, 0x65, 0x73, 0x74, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x49, 0x72,
0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74,
0x49, 0x72, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x49, 0x72, 0x61, 0x6e,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0x65, 0x69, 0x74, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x65,
0x72, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x65, 0x72, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65,
0x69, 0x74, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x65, 0x72, 0x20,
0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x49, 0x73,
0x72, 0x61, 0x65, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65,
0x69, 0x74, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x73, 0x63, 0x68,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74,
0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4a, 0x61,
0x70, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69,
0x74, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4a, 0x61,
0x70, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4b, 0x61, 0x6d, 0x74, 0x73,
0x63, 0x68, 0x61, 0x74, 0x6b, 0x61, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4b,
0x61, 0x6d, 0x74, 0x73, 0x63, 0x68, 0x61, 0x74, 0x6b, 0x61, 0x2d, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4b, 0x61, 0x6d,
0x74, 0x73, 0x63, 0x68, 0x61, 0x74, 0x6b, 0x61, 0x2d, 0x53, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4b, 0x61, 0x73, 0x61, 0x63,
0x68, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x4f,
0x73, 0x74, 0x6b, 0x61, 0x73, 0x61, 0x63, 0x68, 0x69, 0x73, 0x63, 0x68,
0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x6b, 0x61,
0x73, 0x61, 0x63, 0x68, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65,
0x69, 0x74, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68,
0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6e,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0x65, 0x69, 0x74, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65,
0x69, 0x74, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b,
0x65, 0x72, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x65, 0x72, 0x20, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x65, 0x72, 0x20, 0x53, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4b, 0x69, 0x72, 0x67, 0x69,
0x73, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x53,
0x72, 0x69, 0x2d, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x4c, 0x69, 0x6e, 0x69, 0x65, 0x6e, 0x69, 0x6e, 0x73, 0x65, 0x6c,
0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48,
0x6f, 0x77, 0x65, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4c, 0x6f, 0x72, 0x64,
0x2d, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0x65, 0x69, 0x74, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48, 0x6f, 0x77,
0x65, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74,
0x4d, 0x61, 0x63, 0x61, 0x75, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4d, 0x61,
0x63, 0x61, 0x75, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65,
0x69, 0x74, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x2d, 0x53, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x2d, 0x53, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4d, 0x61, 0x6c, 0x61, 0x79, 0x73,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x4d, 0x61,
0x6c, 0x65, 0x64, 0x69, 0x76, 0x65, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74,
0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x2d, 0x5a, 0x65,
0x69, 0x74, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x69, 0x6e,
0x73, 0x65, 0x6c, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x2d, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x74, 0x69, 0x75, 0x73, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0x65, 0x69, 0x74, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x2d, 0x5a, 0x65,
0x69, 0x74, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x50, 0x61, 0x7a, 0x69, 0x66, 0x69, 0x6b, 0x7a, 0x65,
0x69, 0x74, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x50, 0x61, 0x7a, 0x69, 0x66, 0x69, 0x6b, 0x2d, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4d, 0x65, 0x78,
0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x50, 0x61,
0x7a, 0x69, 0x66, 0x69, 0x6b, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0x65, 0x69, 0x74, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61,
0x74, 0x61, 0x72, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x55, 0x6c, 0x61, 0x61,
0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x2d, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62,
0x61, 0x61, 0x74, 0x61, 0x72, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0x65, 0x69, 0x74, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75, 0x65, 0x72,
0x20, 0x5a, 0x65, 0x69, 0x74, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75, 0x65,
0x72, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74,
0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75, 0x65, 0x72, 0x20, 0x53, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4d, 0x79, 0x61, 0x6e, 0x6d,
0x61, 0x72, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4e, 0x61, 0x75, 0x72, 0x75,
0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x65, 0x73,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x4e, 0x65,
0x75, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x73, 0x63, 0x68,
0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x4e, 0x65, 0x75, 0x6b, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4e, 0x65, 0x75, 0x6b,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4e, 0x65,
0x75, 0x73, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x4e, 0x65, 0x75, 0x73, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x2d,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4e, 0x65,
0x75, 0x73, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x53, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4e, 0x65, 0x75, 0x66, 0x75,
0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4e,
0x65, 0x75, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4e, 0x65, 0x75,
0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x53, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4e, 0x69, 0x75, 0x65, 0x2d,
0x5a, 0x65, 0x69, 0x74, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x69,
0x6e, 0x73, 0x65, 0x6c, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x2d, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x2d, 0x53, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x2d, 0x64, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x2d, 0x64, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69,
0x74, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x2d, 0x64, 0x65,
0x2d, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x53, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4e, 0xf6, 0x72, 0x64, 0x6c,
0x69, 0x63, 0x68, 0x65, 0x2d, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x6e, 0x65,
0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x65, 0x72, 0x20, 0x5a, 0x65, 0x69, 0x74,
0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x65,
0x72, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74,
0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x65,
0x72, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74,
0x4f, 0x6d, 0x73, 0x6b, 0x65, 0x72, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x4f,
0x6d, 0x73, 0x6b, 0x65, 0x72, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0x65, 0x69, 0x74, 0x4f, 0x6d, 0x73, 0x6b, 0x65, 0x72, 0x20, 0x53,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a,
0x65, 0x69, 0x74, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a,
0x65, 0x69, 0x74, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0x65, 0x69, 0x74, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x50, 0x61, 0x70, 0x75, 0x61, 0x2d, 0x4e, 0x65, 0x75, 0x67, 0x75,
0x69, 0x6e, 0x65, 0x61, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x50, 0x61, 0x72,
0x61, 0x67, 0x75, 0x61, 0x79, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a,
0x65, 0x69, 0x74, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a,
0x65, 0x69, 0x74, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0x65, 0x69, 0x74, 0x50, 0x65, 0x72, 0x75, 0x61, 0x6e, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x50, 0x65, 0x72, 0x75, 0x61,
0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0x65, 0x69, 0x74, 0x50, 0x65, 0x72, 0x75, 0x61, 0x6e, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0x65, 0x69, 0x74, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x50, 0x68,
0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x50,
0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x69, 0x73, 0x63, 0x68,
0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74,
0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x69, 0x6e, 0x73, 0x65, 0x6c,
0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x53, 0x74, 0x2e, 0x2d, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x2d, 0x75, 0x6e, 0x64, 0x2d, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x53, 0x74,
0x2e, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x75, 0x6e, 0x64,
0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x53, 0x74, 0x2e, 0x2d,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x75, 0x6e, 0x64, 0x2d, 0x4d,
0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x53, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69,
0x72, 0x6e, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x6e, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x2d, 0x5a, 0x65, 0x69, 0x74,
0x50, 0x6a, 0xf6, 0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x67, 0x2d, 0x5a, 0x65,
0x69, 0x74, 0x51, 0x75, 0x79, 0x73, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61,
0x2d, 0x5a, 0x65, 0x69, 0x74, 0x51, 0x75, 0x79, 0x73, 0x79, 0x6c, 0x6f,
0x72, 0x64, 0x61, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65,
0x69, 0x74, 0x51, 0x79, 0x73, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x2d,
0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x52, 0xe9,
0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x52, 0x6f,
0x74, 0x68, 0x65, 0x72, 0x61, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x53, 0x61,
0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x53,
0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69,
0x74, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d, 0x5a, 0x65, 0x69, 0x74,
0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0x65, 0x69, 0x74, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d,
0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x53, 0x61, 0x6d, 0x6f,
0x61, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0x65, 0x69, 0x74, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c,
0x65, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x53, 0x69, 0x6e, 0x67, 0x61,
0x70, 0x75, 0x72, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x53, 0x61, 0x6c, 0x6f, 0x6d,
0x6f, 0x6e, 0x65, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x53, 0xfc, 0x64,
0x67, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a,
0x65, 0x69, 0x74, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x2d,
0x5a, 0x65, 0x69, 0x74, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x2d, 0x5a, 0x65,
0x69, 0x74, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x54, 0x61, 0x69, 0x70, 0x65, 0x68, 0x2d, 0x5a, 0x65, 0x69, 0x74,
0x54, 0x61, 0x69, 0x70, 0x65, 0x68, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0x65, 0x69, 0x74, 0x54, 0x61, 0x69, 0x70, 0x65, 0x68, 0x2d,
0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x54, 0x61,
0x64, 0x73, 0x63, 0x68, 0x69, 0x6b, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x5a, 0x65, 0x69, 0x74, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x2d,
0x5a, 0x65, 0x69, 0x74, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0x65, 0x69, 0x74, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69,
0x74, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x54,
0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d,
0x5a, 0x65, 0x69, 0x74, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a,
0x65, 0x69, 0x74, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65,
0x69, 0x74, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x69, 0x73, 0x63, 0x68,
0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61,
0x79, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0x65, 0x69, 0x74, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0x65, 0x69, 0x74, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x55, 0x73, 0x62, 0x65, 0x6b,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0x65, 0x69, 0x74, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69,
0x74, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x2d, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69,
0x74, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x2d, 0x5a,
0x65, 0x69, 0x74, 0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x65, 0x72, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x57, 0x6c, 0x61,
0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x65, 0x72, 0x20, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x57, 0x6c, 0x61,
0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x65, 0x72, 0x20, 0x53,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x57, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x65, 0x72, 0x20, 0x5a, 0x65, 0x69,
0x74, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x65, 0x72,
0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x57,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x65, 0x72, 0x20, 0x53,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x57, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x57, 0x61, 0x6b, 0x65,
0x2d, 0x49, 0x6e, 0x73, 0x65, 0x6c, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x57,
0x61, 0x6c, 0x6c, 0x69, 0x73, 0x2d, 0x75, 0x6e, 0x64, 0x2d, 0x46, 0x75,
0x74, 0x75, 0x6e, 0x61, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4a, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x65, 0x72, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x4a,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x65, 0x72, 0x20, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4a, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x65, 0x72, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0x65, 0x69, 0x74, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x65, 0x72, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x4a,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x65, 0x72, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69,
0x74, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x65, 0x72, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0x65, 0x69, 0x74, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x2d, 0x5a, 0x65, 0x69, 0x74,
0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x69, 0x6e, 0x73, 0x65, 0x6c,
0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x38f, 0x3c1, 0x3b1, 0x20, 0x391, 0x3c6,
0x3b3, 0x3b1, 0x3bd, 0x3b9, 0x3c3, 0x3c4, 0x3ac, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20,
0x39a, 0x3b5, 0x3bd, 0x3c4, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x391, 0x3c6,
0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x391, 0x3bd, 0x3b1,
0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x391, 0x3c6, 0x3c1, 0x3b9,
0x3ba, 0x3ae, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae,
0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3cc, 0x3c4, 0x3b9, 0x3b1, 0x3c2, 0x20,
0x391, 0x3c6, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x394,
0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x391, 0x3c6, 0x3c1, 0x3b9, 0x3ba,
0x3ae, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20,
0x391, 0x3c6, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae,
0x3c2, 0x20, 0x391, 0x3c6, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x38f, 0x3c1, 0x3b1,
0x20, 0x391, 0x3bb, 0x3ac, 0x3c3, 0x3ba, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3bb,
0x3ac, 0x3c3, 0x3ba, 0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3bb, 0x3ac, 0x3c3, 0x3ba, 0x3b1, 0x3c2, 0x38f,
0x3c1, 0x3b1, 0x20, 0x391, 0x3bc, 0x3b1, 0x3b6, 0x3bf, 0x3bd, 0x3af, 0x3bf, 0x3c5,
0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x391, 0x3bc, 0x3b1, 0x3b6, 0x3bf, 0x3bd, 0x3af, 0x3bf, 0x3c5, 0x398,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3bc,
0x3b1, 0x3b6, 0x3bf, 0x3bd, 0x3af, 0x3bf, 0x3c5, 0x39a, 0x3b5, 0x3bd, 0x3c4, 0x3c1,
0x3b9, 0x3ba, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x392, 0x3cc, 0x3c1, 0x3b5,
0x3b9, 0x3b1, 0x3c2, 0x20, 0x391, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2,
0x39a, 0x3b5, 0x3bd, 0x3c4, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x20, 0x3c7, 0x3b5, 0x3b9,
0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x392,
0x3cc, 0x3c1, 0x3b5, 0x3b9, 0x3b1, 0x3c2, 0x20, 0x391, 0x3bc, 0x3b5, 0x3c1, 0x3b9,
0x3ba, 0x3ae, 0x3c2, 0x39a, 0x3b5, 0x3bd, 0x3c4, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x20,
0x3b8, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x392,
0x3cc, 0x3c1, 0x3b5, 0x3b9, 0x3b1, 0x3c2, 0x20, 0x391, 0x3bc, 0x3b5, 0x3c1, 0x3b9,
0x3ba, 0x3ae, 0x3c2, 0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae,
0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x392, 0x3cc, 0x3c1, 0x3b5, 0x3b9, 0x3b1, 0x3c2,
0x20, 0x391, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x391, 0x3bd, 0x3b1,
0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae, 0x20, 0x3c7, 0x3b5, 0x3b9, 0x3bc, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x392, 0x3cc, 0x3c1,
0x3b5, 0x3b9, 0x3b1, 0x3c2, 0x20, 0x391, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3ba, 0x3ae,
0x3c2, 0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae, 0x20, 0x3b8,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x392, 0x3cc,
0x3c1, 0x3b5, 0x3b9, 0x3b1, 0x3c2, 0x20, 0x391, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3ba,
0x3ae, 0x3c2, 0x39f, 0x3c1, 0x3b5, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x392, 0x3cc, 0x3c1, 0x3b5, 0x3b9, 0x3b1, 0x3c2, 0x20, 0x391, 0x3bc, 0x3b5,
0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x39f, 0x3c1, 0x3b5, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3c7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x392, 0x3cc, 0x3c1, 0x3b5, 0x3b9, 0x3b1, 0x3c2, 0x20, 0x391, 0x3bc,
0x3b5, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x39f, 0x3c1, 0x3b5, 0x3b9, 0x3bd, 0x3ae,
0x20, 0x3b8, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x392, 0x3cc, 0x3c1, 0x3b5, 0x3b9, 0x3b1, 0x3c2, 0x20, 0x391, 0x3bc, 0x3b5, 0x3c1,
0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x395, 0x3b9, 0x3c1, 0x3b7,
0x3bd, 0x3b9, 0x3ba, 0x3bf, 0x3cd, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x395, 0x3b9, 0x3c1, 0x3b7, 0x3bd,
0x3b9, 0x3ba, 0x3bf, 0x3cd, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x395, 0x3b9, 0x3c1, 0x3b7, 0x3bd, 0x3b9, 0x3ba, 0x3bf, 0x3cd,
0x38f, 0x3c1, 0x3b1, 0x20, 0x391, 0x3bd, 0x3b1, 0x3bd, 0x3c4, 0x3af, 0x3c1, 0x3a7,
0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x391, 0x3bd, 0x3b1, 0x3bd, 0x3c4, 0x3af, 0x3c1, 0x398, 0x3b5, 0x3c1, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3bd, 0x3b1, 0x3bd, 0x3c4,
0x3af, 0x3c1, 0x38f, 0x3c1, 0x3b1, 0x20, 0x391, 0x3c0, 0x3af, 0x3b1, 0x3a7, 0x3b5,
0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x391, 0x3c0, 0x3af, 0x3b1, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x391, 0x3c0, 0x3af, 0x3b1, 0x391, 0x3c1, 0x3b1, 0x3b2, 0x3b9,
0x3ba, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x391, 0x3c1, 0x3b1, 0x3b2, 0x3b9, 0x3ba,
0x3ae, 0x20, 0x3c7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x391, 0x3c1, 0x3b1, 0x3b2, 0x3b9, 0x3ba, 0x3ae, 0x20, 0x3b8,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x38f, 0x3c1, 0x3b1,
0x20, 0x391, 0x3c1, 0x3b3, 0x3b5, 0x3bd, 0x3c4, 0x3b9, 0x3bd, 0x3ae, 0x3c2, 0x3a7,
0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x391, 0x3c1, 0x3b3, 0x3b5, 0x3bd, 0x3c4, 0x3b9, 0x3bd, 0x3ae, 0x3c2, 0x398,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3c1,
0x3b3, 0x3b5, 0x3bd, 0x3c4, 0x3b9, 0x3bd, 0x3ae, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20,
0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x391, 0x3c1, 0x3b3, 0x3b5,
0x3bd, 0x3c4, 0x3b9, 0x3bd, 0x3ae, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x394, 0x3c5, 0x3c4, 0x3b9,
0x3ba, 0x3ae, 0x3c2, 0x20, 0x391, 0x3c1, 0x3b3, 0x3b5, 0x3bd, 0x3c4, 0x3b9, 0x3bd,
0x3ae, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x391, 0x3c1, 0x3b3,
0x3b5, 0x3bd, 0x3c4, 0x3b9, 0x3bd, 0x3ae, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x391,
0x3c1, 0x3bc, 0x3b5, 0x3bd, 0x3af, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3c1, 0x3bc,
0x3b5, 0x3bd, 0x3af, 0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3c1, 0x3bc, 0x3b5, 0x3bd, 0x3af, 0x3b1, 0x3c2,
0x38f, 0x3c1, 0x3b1, 0x20, 0x391, 0x3c4, 0x3bb, 0x3b1, 0x3bd, 0x3c4, 0x3b9, 0x3ba,
0x3bf, 0x3cd, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3c4, 0x3bb, 0x3b1, 0x3bd, 0x3c4, 0x3b9, 0x3ba,
0x3bf, 0x3cd, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x391, 0x3c4, 0x3bb, 0x3b1, 0x3bd, 0x3c4, 0x3b9, 0x3ba, 0x3bf, 0x3cd, 0x38f,
0x3c1, 0x3b1, 0x20, 0x39a, 0x3b5, 0x3bd, 0x3c4, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2,
0x20, 0x391, 0x3c5, 0x3c3, 0x3c4, 0x3c1, 0x3b1, 0x3bb, 0x3af, 0x3b1, 0x3c2, 0x3a7,
0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x39a, 0x3b5, 0x3bd, 0x3c4, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x391,
0x3c5, 0x3c3, 0x3c4, 0x3c1, 0x3b1, 0x3bb, 0x3af, 0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3b5, 0x3bd, 0x3c4,
0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x391, 0x3c5, 0x3c3, 0x3c4, 0x3c1, 0x3b1,
0x3bb, 0x3af, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3b5, 0x3bd, 0x3c4,
0x3c1, 0x3bf, 0x3b4, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x391, 0x3c5,
0x3c3, 0x3c4, 0x3c1, 0x3b1, 0x3bb, 0x3af, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3b5,
0x3bd, 0x3c4, 0x3c1, 0x3bf, 0x3b4, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20,
0x391, 0x3c5, 0x3c3, 0x3c4, 0x3c1, 0x3b1, 0x3bb, 0x3af, 0x3b1, 0x3c2, 0x398, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3b5, 0x3bd,
0x3c4, 0x3c1, 0x3bf, 0x3b4, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x391,
0x3c5, 0x3c3, 0x3c4, 0x3c1, 0x3b1, 0x3bb, 0x3af, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1,
0x20, 0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20,
0x391, 0x3c5, 0x3c3, 0x3c4, 0x3c1, 0x3b1, 0x3bb, 0x3af, 0x3b1, 0x3c2, 0x3a7, 0x3b5,
0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x391,
0x3c5, 0x3c3, 0x3c4, 0x3c1, 0x3b1, 0x3bb, 0x3af, 0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3bd, 0x3b1, 0x3c4,
0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x391, 0x3c5, 0x3c3, 0x3c4, 0x3c1,
0x3b1, 0x3bb, 0x3af, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x394, 0x3c5, 0x3c4,
0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x391, 0x3c5, 0x3c3, 0x3c4, 0x3c1, 0x3b1, 0x3bb,
0x3af, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae,
0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2,
0x20, 0x391, 0x3c5, 0x3c3, 0x3c4, 0x3c1, 0x3b1, 0x3bb, 0x3af, 0x3b1, 0x3c2, 0x398,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x394, 0x3c5,
0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x391, 0x3c5, 0x3c3, 0x3c4, 0x3c1, 0x3b1,
0x3bb, 0x3af, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x391, 0x3b6, 0x3b5, 0x3c1,
0x3bc, 0x3c0, 0x3b1, 0x3ca, 0x3c4, 0x3b6, 0x3ac, 0x3bd, 0x3a7, 0x3b5, 0x3b9, 0x3bc,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3b6,
0x3b5, 0x3c1, 0x3bc, 0x3c0, 0x3b1, 0x3ca, 0x3c4, 0x3b6, 0x3ac, 0x3bd, 0x398, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3b6, 0x3b5,
0x3c1, 0x3bc, 0x3c0, 0x3b1, 0x3ca, 0x3c4, 0x3b6, 0x3ac, 0x3bd, 0x38f, 0x3c1, 0x3b1,
0x20, 0x391, 0x3b6, 0x3bf, 0x3c1, 0x3ce, 0x3bd, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3b6, 0x3bf,
0x3c1, 0x3ce, 0x3bd, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x391, 0x3b6, 0x3bf, 0x3c1, 0x3ce, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20,
0x39c, 0x3c0, 0x3b1, 0x3bd, 0x3b3, 0x3ba, 0x3bb, 0x3b1, 0x3bd, 0x3c4, 0x3ad, 0x3c2,
0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x39c, 0x3c0, 0x3b1, 0x3bd, 0x3b3, 0x3ba, 0x3bb, 0x3b1, 0x3bd, 0x3c4,
0x3ad, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x39c, 0x3c0, 0x3b1, 0x3bd, 0x3b3, 0x3ba, 0x3bb, 0x3b1, 0x3bd, 0x3c4, 0x3ad,
0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3c0, 0x3bf, 0x3c5, 0x3c4, 0x3ac, 0x3bd,
0x38f, 0x3c1, 0x3b1, 0x20, 0x392, 0x3bf, 0x3bb, 0x3b9, 0x3b2, 0x3af, 0x3b1, 0x3c2,
0x38f, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3c0, 0x3c1, 0x3b1, 0x3b6, 0x3af, 0x3bb, 0x3b9,
0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3c0, 0x3c1, 0x3b1, 0x3b6, 0x3af, 0x3bb, 0x3b9,
0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x39c, 0x3c0, 0x3c1, 0x3b1, 0x3b6, 0x3af, 0x3bb, 0x3b9, 0x3b1, 0x3c2, 0x38f,
0x3c1, 0x3b1, 0x20, 0x39c, 0x3c0, 0x3c1, 0x3bf, 0x3c5, 0x3bd, 0x3ad, 0x3b9, 0x20,
0x39d, 0x3c4, 0x3b1, 0x3c1, 0x3bf, 0x3c5, 0x3c3, 0x3b1, 0x3bb, 0x3ac, 0x3bc, 0x38f,
0x3c1, 0x3b1, 0x20, 0x3a0, 0x3c1, 0x3ac, 0x3c3, 0x3b9, 0x3bd, 0x3bf, 0x3c5, 0x20,
0x391, 0x3ba, 0x3c1, 0x3c9, 0x3c4, 0x3b7, 0x3c1, 0x3af, 0x3bf, 0x3c5, 0x3a7, 0x3b5,
0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x3a0, 0x3c1, 0x3ac, 0x3c3, 0x3b9, 0x3bd, 0x3bf, 0x3c5, 0x20, 0x391, 0x3ba, 0x3c1,
0x3c9, 0x3c4, 0x3b7, 0x3c1, 0x3af, 0x3bf, 0x3c5, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a0, 0x3c1, 0x3ac, 0x3c3, 0x3b9, 0x3bd,
0x3bf, 0x3c5, 0x20, 0x391, 0x3ba, 0x3c1, 0x3c9, 0x3c4, 0x3b7, 0x3c1, 0x3af, 0x3bf,
0x3c5, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a4, 0x3c3, 0x3b1, 0x3bc, 0x3cc, 0x3c1, 0x3bf,
0x38f, 0x3c1, 0x3b1, 0x20, 0x3a4, 0x3c3, 0x3ac, 0x3c4, 0x3b1, 0x3bc, 0x3a7, 0x3b5,
0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x3a4, 0x3c3, 0x3ac, 0x3c4, 0x3b1, 0x3bc, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae,
0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a4, 0x3c3, 0x3ac, 0x3c4, 0x3b1, 0x3bc, 0x38f,
0x3c1, 0x3b1, 0x20, 0x3a7, 0x3b9, 0x3bb, 0x3ae, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a7, 0x3b9,
0x3bb, 0x3ae, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x3a7, 0x3b9, 0x3bb, 0x3ae, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39a,
0x3af, 0x3bd, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3af, 0x3bd, 0x3b1, 0x3c2, 0x398,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3af,
0x3bd, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3bf, 0x3c5,
0x20, 0x3a7, 0x3c1, 0x3b9, 0x3c3, 0x3c4, 0x3bf, 0x3c5, 0x3b3, 0x3ad, 0x3bd, 0x3bd,
0x3c9, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3c9, 0x3bd, 0x20,
0x39a, 0x3cc, 0x3ba, 0x3bf, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3bf, 0x3bb,
0x3bf, 0x3bc, 0x3b2, 0x3af, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3bf, 0x3bb, 0x3bf,
0x3bc, 0x3b2, 0x3af, 0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3bf, 0x3bb, 0x3bf, 0x3bc, 0x3b2, 0x3af, 0x3b1,
0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3c9, 0x3bd, 0x20, 0x39a,
0x3bf, 0x3c5, 0x3ba, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae,
0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3c9, 0x3bd, 0x20, 0x39a,
0x3bf, 0x3c5, 0x3ba, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3c9, 0x3bd, 0x20, 0x39a, 0x3bf, 0x3c5, 0x3ba,
0x38f, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3bf, 0x3cd, 0x3b2, 0x3b1, 0x3c2, 0x3a7, 0x3b5,
0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x39a, 0x3bf, 0x3cd, 0x3b2, 0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae,
0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3bf, 0x3cd, 0x3b2, 0x3b1, 0x3c2, 0x38f,
0x3c1, 0x3b1, 0x20, 0x39d, 0x3c4, 0x3ad, 0x3b9, 0x3b2, 0x3b9, 0x3c2, 0x38f, 0x3c1,
0x3b1, 0x20, 0x39d, 0x3c4, 0x3b9, 0x3bc, 0x3cc, 0x3bd, 0x20, 0x3bd, 0x3c4, 0x2019,
0x20, 0x39f, 0x3c5, 0x3c1, 0x3b2, 0x3af, 0x3bb, 0x38f, 0x3c1, 0x3b1, 0x20, 0x391,
0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3bf, 0x3cd, 0x20, 0x3a4, 0x3b9,
0x3bc, 0x3cc, 0x3c1, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3bf, 0x3c5,
0x20, 0x3a0, 0x3ac, 0x3c3, 0x3c7, 0x3b1, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3bf,
0x3c5, 0x20, 0x3a0, 0x3ac, 0x3c3, 0x3c7, 0x3b1, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3bf, 0x3c5, 0x20,
0x3a0, 0x3ac, 0x3c3, 0x3c7, 0x3b1, 0x38f, 0x3c1, 0x3b1, 0x20, 0x399, 0x3c3, 0x3b7,
0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3bf, 0x3cd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39a,
0x3b5, 0x3bd, 0x3c4, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x395, 0x3c5, 0x3c1,
0x3ce, 0x3c0, 0x3b7, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3b5, 0x3bd, 0x3c4, 0x3c1, 0x3b9,
0x3ba, 0x3ae, 0x3c2, 0x20, 0x395, 0x3c5, 0x3c1, 0x3ce, 0x3c0, 0x3b7, 0x3c2, 0x398,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3b5,
0x3bd, 0x3c4, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x395, 0x3c5, 0x3c1, 0x3ce,
0x3c0, 0x3b7, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf,
0x3bb, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x395, 0x3c5, 0x3c1, 0x3ce, 0x3c0, 0x3b7,
0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae,
0x3c2, 0x20, 0x395, 0x3c5, 0x3c1, 0x3ce, 0x3c0, 0x3b7, 0x3c2, 0x398, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3bd, 0x3b1, 0x3c4,
0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x395, 0x3c5, 0x3c1, 0x3ce, 0x3c0,
0x3b7, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3c0, 0x3b5, 0x3c1, 0x3b1, 0x3b9, 0x3c4,
0x3ad, 0x3c1, 0x3c9, 0x20, 0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba,
0x3ae, 0x3c2, 0x20, 0x395, 0x3c5, 0x3c1, 0x3ce, 0x3c0, 0x3b7, 0x3c2, 0x38f, 0x3c1,
0x3b1, 0x20, 0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x395, 0x3c5,
0x3c1, 0x3ce, 0x3c0, 0x3b7, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba,
0x3ae, 0x3c2, 0x20, 0x395, 0x3c5, 0x3c1, 0x3ce, 0x3c0, 0x3b7, 0x3c2, 0x398, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x394, 0x3c5, 0x3c4,
0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x395, 0x3c5, 0x3c1, 0x3ce, 0x3c0, 0x3b7, 0x3c2,
0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3c9, 0x3bd, 0x20, 0x3a6, 0x3cc,
0x3ba, 0x3bb, 0x3b1, 0x3bd, 0x3c4, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3c9, 0x3bd,
0x20, 0x3a6, 0x3cc, 0x3ba, 0x3bb, 0x3b1, 0x3bd, 0x3c4, 0x398, 0x3b5, 0x3c1, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3c9, 0x3bd,
0x20, 0x3a6, 0x3cc, 0x3ba, 0x3bb, 0x3b1, 0x3bd, 0x3c4, 0x38f, 0x3c1, 0x3b1, 0x20,
0x3a6, 0x3af, 0x3c4, 0x3b6, 0x3b9, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a6, 0x3af, 0x3c4, 0x3b6, 0x3b9,
0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a6,
0x3af, 0x3c4, 0x3b6, 0x3b9, 0x38f, 0x3c1, 0x3b1, 0x20, 0x393, 0x3b1, 0x3bb, 0x3bb,
0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x393, 0x3bf, 0x3c5, 0x3b9, 0x3ac, 0x3bd, 0x3b1,
0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x393, 0x3b1, 0x3bb, 0x3bb, 0x3b9, 0x3ba, 0x3bf,
0x3cd, 0x20, 0x39d, 0x3cc, 0x3c4, 0x3bf, 0x3c5, 0x20, 0x3ba, 0x3b1, 0x3b9, 0x20,
0x391, 0x3bd, 0x3c4, 0x3b1, 0x3c1, 0x3ba, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x38f,
0x3c1, 0x3b1, 0x20, 0x393, 0x3ba, 0x3b1, 0x3bb, 0x3ac, 0x3c0, 0x3b1, 0x3b3, 0x3ba,
0x3bf, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x393, 0x3ba, 0x3ac, 0x3bc, 0x3c0, 0x3b9,
0x3b5, 0x3c1, 0x38f, 0x3c1, 0x3b1, 0x20, 0x393, 0x3b5, 0x3c9, 0x3c1, 0x3b3, 0x3af,
0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x393, 0x3b5, 0x3c9, 0x3c1, 0x3b3, 0x3af, 0x3b1, 0x3c2,
0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x393,
0x3b5, 0x3c9, 0x3c1, 0x3b3, 0x3af, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d,
0x3ae, 0x3c3, 0x3c9, 0x3bd, 0x20, 0x393, 0x3ba, 0x3af, 0x3bb, 0x3bc, 0x3c0, 0x3b5,
0x3c1, 0x3c4, 0x39c, 0x3ad, 0x3c3, 0x3b7, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x393,
0x3ba, 0x3c1, 0x3af, 0x3bd, 0x3bf, 0x3c5, 0x3b9, 0x3c4, 0x3c2, 0x38f, 0x3c1, 0x3b1,
0x20, 0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20,
0x393, 0x3c1, 0x3bf, 0x3b9, 0x3bb, 0x3b1, 0x3bd, 0x3b4, 0x3af, 0x3b1, 0x3c2, 0x3a7,
0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20,
0x393, 0x3c1, 0x3bf, 0x3b9, 0x3bb, 0x3b1, 0x3bd, 0x3b4, 0x3af, 0x3b1, 0x3c2, 0x398,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3bd,
0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x393, 0x3c1, 0x3bf,
0x3b9, 0x3bb, 0x3b1, 0x3bd, 0x3b4, 0x3af, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20,
0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x393, 0x3c1, 0x3bf, 0x3b9,
0x3bb, 0x3b1, 0x3bd, 0x3b4, 0x3af, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x394, 0x3c5, 0x3c4,
0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x393, 0x3c1, 0x3bf, 0x3b9, 0x3bb, 0x3b1, 0x3bd,
0x3b4, 0x3af, 0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x393,
0x3c1, 0x3bf, 0x3b9, 0x3bb, 0x3b1, 0x3bd, 0x3b4, 0x3af, 0x3b1, 0x3c2, 0x38f, 0x3c1,
0x3b1, 0x20, 0x393, 0x3ba, 0x3bf, 0x3c5, 0x3ac, 0x3bc, 0x38f, 0x3c1, 0x3b1, 0x20,
0x39a, 0x3cc, 0x3bb, 0x3c0, 0x3bf, 0x3c5, 0x38f, 0x3c1, 0x3b1, 0x20, 0x393, 0x3bf,
0x3c5, 0x3b9, 0x3ac, 0x3bd, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a7, 0x3b1,
0x3b2, 0x3ac, 0x3b7, 0x3c2, 0x2d, 0x391, 0x3bb, 0x3b5, 0x3bf, 0x3cd, 0x3c4, 0x3b9,
0x3c9, 0x3bd, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3c9, 0x3bd, 0x3a7, 0x3b5, 0x3b9, 0x3bc,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a7, 0x3b1,
0x3b2, 0x3ac, 0x3b7, 0x3c2, 0x2d, 0x391, 0x3bb, 0x3b5, 0x3bf, 0x3cd, 0x3c4, 0x3b9,
0x3c9, 0x3bd, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3c9, 0x3bd, 0x398, 0x3b5, 0x3c1, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a7, 0x3b1, 0x3b2, 0x3ac, 0x3b7,
0x3c2, 0x2d, 0x391, 0x3bb, 0x3b5, 0x3bf, 0x3cd, 0x3c4, 0x3b9, 0x3c9, 0x3bd, 0x20,
0x39d, 0x3ae, 0x3c3, 0x3c9, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a7, 0x3bf, 0x3bd,
0x3b3, 0x3ba, 0x20, 0x39a, 0x3bf, 0x3bd, 0x3b3, 0x3ba, 0x3a7, 0x3b5, 0x3b9, 0x3bc,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a7, 0x3bf,
0x3bd, 0x3b3, 0x3ba, 0x20, 0x39a, 0x3bf, 0x3bd, 0x3b3, 0x3ba, 0x398, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a7, 0x3bf, 0x3bd, 0x3b3,
0x3ba, 0x20, 0x39a, 0x3bf, 0x3bd, 0x3b3, 0x3ba, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a7,
0x3bf, 0x3b2, 0x3bd, 0x3c4, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a7, 0x3bf, 0x3b2, 0x3bd, 0x3c4, 0x398,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a7, 0x3bf,
0x3b2, 0x3bd, 0x3c4, 0x38f, 0x3c1, 0x3b1, 0x20, 0x399, 0x3bd, 0x3b4, 0x3af, 0x3b1,
0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x399, 0x3bd, 0x3b4, 0x3b9, 0x3ba, 0x3bf, 0x3cd,
0x20, 0x3a9, 0x3ba, 0x3b5, 0x3b1, 0x3bd, 0x3bf, 0x3cd, 0x38f, 0x3c1, 0x3b1, 0x20,
0x399, 0x3bd, 0x3b4, 0x3bf, 0x3ba, 0x3af, 0x3bd, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1,
0x20, 0x39a, 0x3b5, 0x3bd, 0x3c4, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x399,
0x3bd, 0x3b4, 0x3bf, 0x3bd, 0x3b7, 0x3c3, 0x3af, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1,
0x20, 0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20,
0x399, 0x3bd, 0x3b4, 0x3bf, 0x3bd, 0x3b7, 0x3c3, 0x3af, 0x3b1, 0x3c2, 0x38f, 0x3c1,
0x3b1, 0x20, 0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x399, 0x3bd,
0x3b4, 0x3bf, 0x3bd, 0x3b7, 0x3c3, 0x3af, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20,
0x399, 0x3c1, 0x3ac, 0x3bd, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x399, 0x3c1, 0x3ac, 0x3bd, 0x398, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x399, 0x3c1, 0x3ac,
0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x399, 0x3c1, 0x3ba, 0x3bf, 0x3cd, 0x3c4, 0x3c3,
0x3ba, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x399, 0x3c1, 0x3ba, 0x3bf, 0x3cd, 0x3c4, 0x3c3, 0x3ba, 0x398,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x399, 0x3c1,
0x3ba, 0x3bf, 0x3cd, 0x3c4, 0x3c3, 0x3ba, 0x38f, 0x3c1, 0x3b1, 0x20, 0x399, 0x3c3,
0x3c1, 0x3b1, 0x3ae, 0x3bb, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x399, 0x3c3, 0x3c1, 0x3b1, 0x3ae, 0x3bb,
0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x399,
0x3c3, 0x3c1, 0x3b1, 0x3ae, 0x3bb, 0x38f, 0x3c1, 0x3b1, 0x20, 0x399, 0x3b1, 0x3c0,
0x3c9, 0x3bd, 0x3af, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x399, 0x3b1, 0x3c0, 0x3c9, 0x3bd,
0x3af, 0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x399, 0x3b1, 0x3c0, 0x3c9, 0x3bd, 0x3af, 0x3b1, 0x3c2, 0x38f, 0x3c1,
0x3b1, 0x20, 0x39a, 0x3b1, 0x3bc, 0x3c4, 0x3c3, 0x3ac, 0x3c4, 0x3ba, 0x3b1, 0x3a7,
0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x3a0, 0x3b5, 0x3c4, 0x3c1, 0x3bf, 0x3c0, 0x3b1, 0x3b2, 0x3bb, 0x3cc, 0x3b2,
0x3c3, 0x3ba, 0x2d, 0x39a, 0x3b1, 0x3bc, 0x3c4, 0x3c3, 0x3ac, 0x3c4, 0x3c3, 0x3ba,
0x3b9, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x3a0, 0x3b5, 0x3c4, 0x3c1, 0x3bf, 0x3c0, 0x3b1, 0x3b2, 0x3bb, 0x3cc, 0x3b2, 0x3c3,
0x3ba, 0x2d, 0x39a, 0x3b1, 0x3bc, 0x3c4, 0x3c3, 0x3ac, 0x3c4, 0x3c3, 0x3ba, 0x3b9,
0x38f, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3b1, 0x3b6, 0x3b1, 0x3ba, 0x3c3, 0x3c4, 0x3ac,
0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9,
0x3ba, 0x3bf, 0x3cd, 0x20, 0x39a, 0x3b1, 0x3b6, 0x3b1, 0x3ba, 0x3c3, 0x3c4, 0x3ac,
0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3bf, 0x3cd,
0x20, 0x39a, 0x3b1, 0x3b6, 0x3b1, 0x3ba, 0x3c3, 0x3c4, 0x3ac, 0x3bd, 0x38f, 0x3c1,
0x3b1, 0x20, 0x39a, 0x3bf, 0x3c1, 0x3ad, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3bf,
0x3c1, 0x3ad, 0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x39a, 0x3bf, 0x3c1, 0x3ad, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1,
0x20, 0x39a, 0x3cc, 0x3c3, 0x3c1, 0x3b1, 0x3ca, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39a,
0x3c1, 0x3b1, 0x3c3, 0x3bd, 0x3bf, 0x3b3, 0x3b9, 0x3ac, 0x3c1, 0x3c3, 0x3ba, 0x3a7,
0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x39a, 0x3c1, 0x3b1, 0x3c3, 0x3bd, 0x3bf, 0x3b3, 0x3b9, 0x3ac, 0x3c1, 0x3c3,
0x3ba, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x39a, 0x3c1, 0x3b1, 0x3c3, 0x3bd, 0x3bf, 0x3b3, 0x3b9, 0x3ac, 0x3c1, 0x3c3, 0x3ba,
0x38f, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3b9, 0x3c1, 0x3b3, 0x3b9, 0x3c3, 0x3c4, 0x3ac,
0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3c9, 0x3bd, 0x20, 0x39b,
0x3ac, 0x3b9, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39b, 0x3bf, 0x3c1, 0x3bd, 0x3c4,
0x20, 0x3a7, 0x3ac, 0x3bf, 0x3c5, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39b, 0x3bf, 0x3c1, 0x3bd, 0x3c4,
0x20, 0x3a7, 0x3ac, 0x3bf, 0x3c5, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x39b, 0x3bf, 0x3c1, 0x3bd, 0x3c4, 0x20, 0x3a7, 0x3ac,
0x3bf, 0x3c5, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3b1, 0x3ba, 0x3ac, 0x3bf, 0x3a7,
0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x39c, 0x3b1, 0x3ba, 0x3ac, 0x3bf, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae,
0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3b1, 0x3ba, 0x3ac, 0x3bf, 0x38f, 0x3c1,
0x3b1, 0x20, 0x39c, 0x3b1, 0x3b3, 0x3ba, 0x3b1, 0x3bd, 0x3c4, 0x3ac, 0x3bd, 0x3a7,
0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x39c, 0x3b1, 0x3b3, 0x3ba, 0x3b1, 0x3bd, 0x3c4, 0x3ac, 0x3bd, 0x398, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3b1, 0x3b3,
0x3ba, 0x3b1, 0x3bd, 0x3c4, 0x3ac, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3b1,
0x3bb, 0x3b1, 0x3b9, 0x3c3, 0x3af, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39c,
0x3b1, 0x3bb, 0x3b4, 0x3af, 0x3b2, 0x3c9, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39c,
0x3b1, 0x3c1, 0x3ba, 0x3ad, 0x3b6, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d,
0x3ae, 0x3c3, 0x3c9, 0x3bd, 0x20, 0x39c, 0x3ac, 0x3c1, 0x3c3, 0x3b1, 0x3bb, 0x38f,
0x3c1, 0x3b1, 0x20, 0x39c, 0x3b1, 0x3c5, 0x3c1, 0x3af, 0x3ba, 0x3b9, 0x3bf, 0x3c5,
0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x39c, 0x3b1, 0x3c5, 0x3c1, 0x3af, 0x3ba, 0x3b9, 0x3bf, 0x3c5, 0x398,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3b1,
0x3c5, 0x3c1, 0x3af, 0x3ba, 0x3b9, 0x3bf, 0x3c5, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39c,
0x3cc, 0x3c3, 0x3bf, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x395, 0x3b9, 0x3c1, 0x3b7,
0x3bd, 0x3b9, 0x3ba, 0x3bf, 0x3cd, 0x20, 0x39c, 0x3b5, 0x3be, 0x3b9, 0x3ba, 0x3bf,
0x3cd, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x395, 0x3b9, 0x3c1, 0x3b7, 0x3bd, 0x3b9, 0x3ba, 0x3bf, 0x3cd,
0x20, 0x39c, 0x3b5, 0x3be, 0x3b9, 0x3ba, 0x3bf, 0x3cd, 0x398, 0x3b5, 0x3c1, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x395, 0x3b9, 0x3c1, 0x3b7, 0x3bd,
0x3b9, 0x3ba, 0x3bf, 0x3cd, 0x20, 0x39c, 0x3b5, 0x3be, 0x3b9, 0x3ba, 0x3bf, 0x3cd,
0x38f, 0x3c1, 0x3b1, 0x20, 0x39f, 0x3c5, 0x3bb, 0x3ac, 0x3bd, 0x20, 0x39c, 0x3c0,
0x3b1, 0x3c4, 0x3cc, 0x3c1, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39f, 0x3c5, 0x3bb, 0x3ac, 0x3bd, 0x20,
0x39c, 0x3c0, 0x3b1, 0x3c4, 0x3cc, 0x3c1, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae,
0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39f, 0x3c5, 0x3bb, 0x3ac, 0x3bd, 0x20, 0x39c,
0x3c0, 0x3b1, 0x3c4, 0x3cc, 0x3c1, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3cc, 0x3c3,
0x3c7, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae,
0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3cc, 0x3c3, 0x3c7, 0x3b1, 0x3c2, 0x398,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3cc,
0x3c3, 0x3c7, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3b9, 0x3b1, 0x3bd,
0x3bc, 0x3ac, 0x3c1, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3b1, 0x3bf, 0x3cd, 0x3c1,
0x3bf, 0x3c5, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3b5, 0x3c0, 0x3ac, 0x3bb, 0x38f,
0x3c1, 0x3b1, 0x20, 0x39d, 0x3ad, 0x3b1, 0x3c2, 0x20, 0x39a, 0x3b1, 0x3bb, 0x3b7,
0x3b4, 0x3bf, 0x3bd, 0x3af, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ad, 0x3b1, 0x3c2,
0x20, 0x39a, 0x3b1, 0x3bb, 0x3b7, 0x3b4, 0x3bf, 0x3bd, 0x3af, 0x3b1, 0x3c2, 0x398,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ad,
0x3b1, 0x3c2, 0x20, 0x39a, 0x3b1, 0x3bb, 0x3b7, 0x3b4, 0x3bf, 0x3bd, 0x3af, 0x3b1,
0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ad, 0x3b1, 0x3c2, 0x20, 0x396, 0x3b7,
0x3bb, 0x3b1, 0x3bd, 0x3b4, 0x3af, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ad, 0x3b1,
0x3c2, 0x20, 0x396, 0x3b7, 0x3bb, 0x3b1, 0x3bd, 0x3b4, 0x3af, 0x3b1, 0x3c2, 0x398,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ad,
0x3b1, 0x3c2, 0x20, 0x396, 0x3b7, 0x3bb, 0x3b1, 0x3bd, 0x3b4, 0x3af, 0x3b1, 0x3c2,
0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ad, 0x3b1, 0x3c2, 0x20, 0x393, 0x3b7, 0x3c2,
0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x39d, 0x3ad, 0x3b1, 0x3c2, 0x20, 0x393, 0x3b7, 0x3c2, 0x398, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ad, 0x3b1,
0x3c2, 0x20, 0x393, 0x3b7, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3b9, 0x3bf,
0x3cd, 0x3b5, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3bf, 0x3c5, 0x20,
0x39d, 0x3cc, 0x3c1, 0x3c6, 0x3bf, 0x3bb, 0x3ba, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3,
0x3bf, 0x3c5, 0x20, 0x39d, 0x3cc, 0x3c1, 0x3c6, 0x3bf, 0x3bb, 0x3ba, 0x398, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3,
0x3bf, 0x3c5, 0x20, 0x39d, 0x3cc, 0x3c1, 0x3c6, 0x3bf, 0x3bb, 0x3ba, 0x38f, 0x3c1,
0x3b1, 0x20, 0x3a6, 0x3b5, 0x3c1, 0x3bd, 0x3ac, 0x3c1, 0x3bd, 0x3c4, 0x3bf, 0x20,
0x3bd, 0x3c4, 0x3b5, 0x20, 0x39d, 0x3bf, 0x3c1, 0x3cc, 0x3bd, 0x3b9, 0x3b1, 0x3a7,
0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x3a6, 0x3b5, 0x3c1, 0x3bd, 0x3ac, 0x3c1, 0x3bd, 0x3c4, 0x3bf, 0x20, 0x3bd,
0x3c4, 0x3b5, 0x20, 0x39d, 0x3bf, 0x3c1, 0x3cc, 0x3bd, 0x3b9, 0x3b1, 0x398, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a6, 0x3b5, 0x3c1,
0x3bd, 0x3ac, 0x3c1, 0x3bd, 0x3c4, 0x3bf, 0x20, 0x3bd, 0x3c4, 0x3b5, 0x20, 0x39d,
0x3bf, 0x3c1, 0x3cc, 0x3bd, 0x3b9, 0x3b1, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3b7,
0x3c3, 0x3b9, 0x3ce, 0x3bd, 0x20, 0x392, 0x3cc, 0x3c1, 0x3b5, 0x3b9, 0x3b5, 0x3c2,
0x20, 0x39c, 0x3b1, 0x3c1, 0x3b9, 0x3ac, 0x3bd, 0x3b5, 0x3c2, 0x38f, 0x3c1, 0x3b1,
0x20, 0x39d, 0x3bf, 0x3b2, 0x3bf, 0x3c3, 0x3b9, 0x3bc, 0x3c0, 0x3af, 0x3c1, 0x3c3,
0x3ba, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x39d, 0x3bf, 0x3b2, 0x3bf, 0x3c3, 0x3b9, 0x3bc, 0x3c0, 0x3af,
0x3c1, 0x3c3, 0x3ba, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x39d, 0x3bf, 0x3b2, 0x3bf, 0x3c3, 0x3b9, 0x3bc, 0x3c0, 0x3af, 0x3c1,
0x3c3, 0x3ba, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39f, 0x3bc, 0x3c3, 0x3ba, 0x3a7, 0x3b5,
0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x39f, 0x3bc, 0x3c3, 0x3ba, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x39f, 0x3bc, 0x3c3, 0x3ba, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a0,
0x3b1, 0x3ba, 0x3b9, 0x3c3, 0x3c4, 0x3ac, 0x3bd, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a0, 0x3b1, 0x3ba,
0x3b9, 0x3c3, 0x3c4, 0x3ac, 0x3bd, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a0, 0x3b1, 0x3ba, 0x3b9, 0x3c3, 0x3c4, 0x3ac, 0x3bd,
0x38f, 0x3c1, 0x3b1, 0x20, 0x3a0, 0x3b1, 0x3bb, 0x3ac, 0x3bf, 0x3c5, 0x38f, 0x3c1,
0x3b1, 0x20, 0x3a0, 0x3b1, 0x3c0, 0x3bf, 0x3cd, 0x3b1, 0x3c2, 0x20, 0x39d, 0x3ad,
0x3b1, 0x3c2, 0x20, 0x393, 0x3bf, 0x3c5, 0x3b9, 0x3bd, 0x3ad, 0x3b1, 0x3c2, 0x38f,
0x3c1, 0x3b1, 0x20, 0x3a0, 0x3b1, 0x3c1, 0x3b1, 0x3b3, 0x3bf, 0x3c5, 0x3ac, 0x3b7,
0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x3a0, 0x3b1, 0x3c1, 0x3b1, 0x3b3, 0x3bf, 0x3c5, 0x3ac, 0x3b7,
0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x3a0, 0x3b1, 0x3c1, 0x3b1, 0x3b3, 0x3bf, 0x3c5, 0x3ac, 0x3b7, 0x3c2, 0x38f, 0x3c1,
0x3b1, 0x20, 0x3a0, 0x3b5, 0x3c1, 0x3bf, 0x3cd, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a0, 0x3b5, 0x3c1,
0x3bf, 0x3cd, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x3a0, 0x3b5, 0x3c1, 0x3bf, 0x3cd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a6, 0x3b9,
0x3bb, 0x3b9, 0x3c0, 0x3c0, 0x3b9, 0x3bd, 0x3ce, 0x3bd, 0x3a7, 0x3b5, 0x3b9, 0x3bc,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a6, 0x3b9,
0x3bb, 0x3b9, 0x3c0, 0x3c0, 0x3b9, 0x3bd, 0x3ce, 0x3bd, 0x398, 0x3b5, 0x3c1, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a6, 0x3b9, 0x3bb, 0x3b9, 0x3c0,
0x3c0, 0x3b9, 0x3bd, 0x3ce, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3,
0x3c9, 0x3bd, 0x20, 0x3a6, 0x3bf, 0x3af, 0x3bd, 0x3b9, 0x3be, 0x38f, 0x3c1, 0x3b1,
0x20, 0x3a3, 0x3b5, 0x3bd, 0x20, 0x3a0, 0x3b9, 0x3b5, 0x3c1, 0x20, 0x3ba, 0x3b1,
0x3b9, 0x20, 0x39c, 0x3b9, 0x3ba, 0x3b5, 0x3bb, 0x3cc, 0x3bd, 0x3a7, 0x3b5, 0x3b9,
0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a3,
0x3b5, 0x3bd, 0x20, 0x3a0, 0x3b9, 0x3b5, 0x3c1, 0x20, 0x3ba, 0x3b1, 0x3b9, 0x20,
0x39c, 0x3b9, 0x3ba, 0x3b5, 0x3bb, 0x3cc, 0x3bd, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a3, 0x3b5, 0x3bd, 0x20, 0x3a0, 0x3b9,
0x3b5, 0x3c1, 0x20, 0x3ba, 0x3b1, 0x3b9, 0x20, 0x39c, 0x3b9, 0x3ba, 0x3b5, 0x3bb,
0x3cc, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a0, 0x3af, 0x3c4, 0x3ba, 0x3b5, 0x3c1,
0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a0, 0x3bf, 0x3bd, 0x3ac, 0x3c0, 0x3b5, 0x38f,
0x3c1, 0x3b1, 0x20, 0x3a0, 0x3b9, 0x3bf, 0x3bd, 0x3b3, 0x3b9, 0x3ac, 0x3bd, 0x3b3,
0x3ba, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a1, 0x3b5, 0x3ca, 0x3bd, 0x3b9, 0x3cc, 0x3bd,
0x38f, 0x3c1, 0x3b1, 0x20, 0x3a1, 0x3cc, 0x3b8, 0x3b5, 0x3c1, 0x3b1, 0x38f, 0x3c1,
0x3b1, 0x20, 0x3a3, 0x3b1, 0x3c7, 0x3b1, 0x3bb, 0x3af, 0x3bd, 0x3b7, 0x3c2, 0x3a7,
0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x3a3, 0x3b1, 0x3c7, 0x3b1, 0x3bb, 0x3af, 0x3bd, 0x3b7, 0x3c2, 0x398, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a3, 0x3b1, 0x3c7,
0x3b1, 0x3bb, 0x3af, 0x3bd, 0x3b7, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a3, 0x3ac,
0x3bc, 0x3b1, 0x3c1, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a3, 0x3ac, 0x3bc, 0x3b1, 0x3c1,
0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x3a3, 0x3b1, 0x3bc, 0x3ac, 0x3c1, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20,
0x3a3, 0x3b1, 0x3bc, 0x3cc, 0x3b1, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a3, 0x3b1, 0x3bc, 0x3cc, 0x3b1,
0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a3,
0x3b1, 0x3bc, 0x3cc, 0x3b1, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a3, 0x3b5, 0x3cb, 0x3c7,
0x3b5, 0x3bb, 0x3bb, 0x3ce, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a3, 0x3b9, 0x3b3,
0x3ba, 0x3b1, 0x3c0, 0x3bf, 0x3cd, 0x3c1, 0x3b7, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20,
0x39d, 0x3ae, 0x3c3, 0x3c9, 0x3bd, 0x20, 0x3a3, 0x3bf, 0x3bb, 0x3bf, 0x3bc, 0x3ce,
0x3bd, 0x3c4, 0x3bf, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3cc, 0x3c4, 0x3b9,
0x3b1, 0x3c2, 0x20, 0x393, 0x3b5, 0x3c9, 0x3c1, 0x3b3, 0x3af, 0x3b1, 0x3c2, 0x38f,
0x3c1, 0x3b1, 0x20, 0x3a3, 0x3bf, 0x3c5, 0x3c1, 0x3b9, 0x3bd, 0x3ac, 0x3bc, 0x38f,
0x3c1, 0x3b1, 0x20, 0x3a3, 0x3af, 0x3bf, 0x3b2, 0x3b1, 0x38f, 0x3c1, 0x3b1, 0x20,
0x3a4, 0x3b1, 0x3ca, 0x3c4, 0x3ae, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a4, 0x3b1,
0x3ca, 0x3c0, 0x3ad, 0x3b9, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a4, 0x3b1, 0x3ca, 0x3c0, 0x3ad, 0x3b9,
0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a4,
0x3b1, 0x3ca, 0x3c0, 0x3ad, 0x3b9, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a4, 0x3b1, 0x3c4,
0x3b6, 0x3b9, 0x3ba, 0x3b9, 0x3c3, 0x3c4, 0x3ac, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20,
0x3a4, 0x3bf, 0x3ba, 0x3b5, 0x3bb, 0x3ac, 0x3bf, 0x3c5, 0x38f, 0x3c1, 0x3b1, 0x20,
0x3a4, 0x3cc, 0x3bd, 0x3b3, 0x3ba, 0x3b1, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a4, 0x3cc, 0x3bd, 0x3b3,
0x3ba, 0x3b1, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x3a4, 0x3cc, 0x3bd, 0x3b3, 0x3ba, 0x3b1, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a4,
0x3c3, 0x3bf, 0x3c5, 0x3ba, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a4, 0x3bf, 0x3c5, 0x3c1,
0x3ba, 0x3bc, 0x3b5, 0x3bd, 0x3b9, 0x3c3, 0x3c4, 0x3ac, 0x3bd, 0x3a7, 0x3b5, 0x3b9,
0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a4,
0x3bf, 0x3c5, 0x3c1, 0x3ba, 0x3bc, 0x3b5, 0x3bd, 0x3b9, 0x3c3, 0x3c4, 0x3ac, 0x3bd,
0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a4,
0x3bf, 0x3c5, 0x3c1, 0x3ba, 0x3bc, 0x3b5, 0x3bd, 0x3b9, 0x3c3, 0x3c4, 0x3ac, 0x3bd,
0x38f, 0x3c1, 0x3b1, 0x20, 0x3a4, 0x3bf, 0x3c5, 0x3b2, 0x3b1, 0x3bb, 0x3bf, 0x3cd,
0x38f, 0x3c1, 0x3b1, 0x20, 0x39f, 0x3c5, 0x3c1, 0x3bf, 0x3c5, 0x3b3, 0x3bf, 0x3c5,
0x3ac, 0x3b7, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae,
0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39f, 0x3c5, 0x3c1, 0x3bf, 0x3c5, 0x3b3, 0x3bf,
0x3c5, 0x3ac, 0x3b7, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x39f, 0x3c5, 0x3c1, 0x3bf, 0x3c5, 0x3b3, 0x3bf, 0x3c5, 0x3ac,
0x3b7, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39f, 0x3c5, 0x3b6, 0x3bc, 0x3c0, 0x3b5,
0x3ba, 0x3b9, 0x3c3, 0x3c4, 0x3ac, 0x3bd, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39f, 0x3c5, 0x3b6, 0x3bc,
0x3c0, 0x3b5, 0x3ba, 0x3b9, 0x3c3, 0x3c4, 0x3ac, 0x3bd, 0x398, 0x3b5, 0x3c1, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39f, 0x3c5, 0x3b6, 0x3bc, 0x3c0,
0x3b5, 0x3ba, 0x3b9, 0x3c3, 0x3c4, 0x3ac, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x392,
0x3b1, 0x3bd, 0x3bf, 0x3c5, 0x3ac, 0x3c4, 0x3bf, 0x3c5, 0x3a7, 0x3b5, 0x3b9, 0x3bc,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x392, 0x3b1,
0x3bd, 0x3bf, 0x3c5, 0x3ac, 0x3c4, 0x3bf, 0x3c5, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x392, 0x3b1, 0x3bd, 0x3bf, 0x3c5, 0x3ac,
0x3c4, 0x3bf, 0x3c5, 0x38f, 0x3c1, 0x3b1, 0x20, 0x392, 0x3b5, 0x3bd, 0x3b5, 0x3b6,
0x3bf, 0x3c5, 0x3ad, 0x3bb, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x392, 0x3bb,
0x3b1, 0x3b4, 0x3b9, 0x3b2, 0x3bf, 0x3c3, 0x3c4, 0x3cc, 0x3ba, 0x3a7, 0x3b5, 0x3b9,
0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x392,
0x3bb, 0x3b1, 0x3b4, 0x3b9, 0x3b2, 0x3bf, 0x3c3, 0x3c4, 0x3cc, 0x3ba, 0x398, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x392, 0x3bb, 0x3b1,
0x3b4, 0x3b9, 0x3b2, 0x3bf, 0x3c3, 0x3c4, 0x3cc, 0x3ba, 0x38f, 0x3c1, 0x3b1, 0x20,
0x392, 0x3cc, 0x3bb, 0x3b3, 0x3ba, 0x3bf, 0x3b3, 0x3ba, 0x3c1, 0x3b1, 0x3bd, 0x3c4,
0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x392, 0x3cc, 0x3bb, 0x3b3, 0x3ba, 0x3bf, 0x3b3, 0x3ba, 0x3c1, 0x3b1,
0x3bd, 0x3c4, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x392, 0x3cc, 0x3bb, 0x3b3, 0x3ba, 0x3bf, 0x3b3, 0x3ba, 0x3c1, 0x3b1, 0x3bd,
0x3c4, 0x38f, 0x3c1, 0x3b1, 0x20, 0x392, 0x3cc, 0x3c3, 0x3c4, 0x3bf, 0x3ba, 0x38f,
0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3bf, 0x3c5, 0x20, 0x393, 0x3bf, 0x3c5,
0x3ad, 0x3b9, 0x3ba, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39f, 0x3c5, 0x3ac, 0x3bb, 0x3b9,
0x3c2, 0x20, 0x3ba, 0x3b1, 0x3b9, 0x20, 0x3a6, 0x3bf, 0x3c5, 0x3c4, 0x3bf, 0x3cd,
0x3bd, 0x3b1, 0x38f, 0x3c1, 0x3b1, 0x20, 0x393, 0x3b9, 0x3b1, 0x3ba, 0x3bf, 0x3cd,
0x3c4, 0x3c3, 0x3ba, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae,
0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x393, 0x3b9, 0x3b1, 0x3ba, 0x3bf, 0x3cd, 0x3c4,
0x3c3, 0x3ba, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x393, 0x3b9, 0x3b1, 0x3ba, 0x3bf, 0x3cd, 0x3c4, 0x3c3, 0x3ba, 0x38f, 0x3c1,
0x3b1, 0x20, 0x391, 0x3b9, 0x3ba, 0x3b1, 0x3c4, 0x3b5, 0x3c1, 0x3af, 0x3bd, 0x3bc,
0x3c0, 0x3bf, 0x3c5, 0x3c1, 0x3b3, 0x3ba, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3b9, 0x3ba, 0x3b1,
0x3c4, 0x3b5, 0x3c1, 0x3af, 0x3bd, 0x3bc, 0x3c0, 0x3bf, 0x3c5, 0x3c1, 0x3b3, 0x3ba,
0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391,
0x3b9, 0x3ba, 0x3b1, 0x3c4, 0x3b5, 0x3c1, 0x3af, 0x3bd, 0x3bc, 0x3c0, 0x3bf, 0x3c5,
0x3c1, 0x3b3, 0x3ba, 0x38f, 0x3c1, 0x3b1, 0x20, 0x393, 0x3b9, 0x3bf, 0x3cd, 0x3ba,
0x3bf, 0x3bd, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x20, 0xf3, 0x72,
0x61, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x20, 0xf3, 0x72, 0x61,
0x47, 0x61, 0x6c, 0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x20, 0xf3, 0x72,
0x61, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x20, 0xf3,
0x72, 0x61, 0xa8f, 0xa95, 0xab0, 0x20, 0xab8, 0xaae, 0xaaf, 0xa8f, 0xa95, 0xab0,
0x20, 0xaaa, 0xacd, 0xab0, 0xaae, 0xabe, 0xaa3, 0xaad, 0xac2, 0xaa4, 0x20, 0xab8,
0xaae, 0xaaf, 0xa8f, 0xa95, 0xab0, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd,
0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa85, 0xaab, 0xa98, 0xabe, 0xaa8, 0xabf, 0xab8,
0xacd, 0xaa4, 0xabe, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xaa7, 0xacd, 0xaaf,
0x20, 0xa86, 0xaab, 0xacd, 0xab0, 0xabf, 0xa95, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf,
0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0x20, 0xa86, 0xaab, 0xacd, 0xab0, 0xabf, 0xa95,
0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa6, 0xa95, 0xacd, 0xab7, 0xabf, 0xaa3, 0x20,
0xa86, 0xaab, 0xacd, 0xab0, 0xabf, 0xa95, 0xabe, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95,
0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf, 0xaae, 0x20, 0xa86,
0xaab, 0xacd, 0xab0, 0xabf, 0xa95, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xab6,
0xacd, 0xa9a, 0xabf, 0xaae, 0x20, 0xa86, 0xaab, 0xacd, 0xab0, 0xabf, 0xa95, 0xabe,
0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xab6, 0xacd,
0xa9a, 0xabf, 0xaae, 0x20, 0xa86, 0xaab, 0xacd, 0xab0, 0xabf, 0xa95, 0xabe, 0x20,
0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa85,
0xab2, 0xabe, 0xab8, 0xacd, 0xa95, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xa85, 0xab2,
0xabe, 0xab8, 0xacd, 0xa95, 0xabe, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8,
0xaae, 0xaaf, 0xa85, 0xab2, 0xabe, 0xab8, 0xacd, 0xa95, 0xabe, 0x20, 0xaa6, 0xabf,
0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa85, 0xab2, 0xacd, 0xaae, 0xabe, 0xa9f,
0xac0, 0x20, 0xab8, 0xaae, 0xaaf, 0xa85, 0xab2, 0xacd, 0xaae, 0xabe, 0xa9f, 0xac0,
0x20, 0xaaa, 0xacd, 0xab0, 0xaae, 0xabe, 0xaa3, 0xaad, 0xac2, 0xaa4, 0x20, 0xab8,
0xaae, 0xaaf, 0xa85, 0xab2, 0xacd, 0xaae, 0xabe, 0xa9f, 0xac0, 0x20, 0xa97, 0xacd,
0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa8f, 0xaae, 0xac7,
0xa9d, 0xacb, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa8f, 0xaae, 0xac7, 0xa9d, 0xacb,
0xaa8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa8f, 0xaae,
0xac7, 0xa9d, 0xacb, 0xaa8, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae,
0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0, 0x20, 0xa85, 0xaae,
0xac7, 0xab0, 0xabf, 0xa95, 0xaa8, 0x20, 0xa95, 0xac7, 0xaa8, 0xacd, 0xaa6, 0xacd,
0xab0, 0xac0, 0xaaf, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0,
0x20, 0xa85, 0xaae, 0xac7, 0xab0, 0xabf, 0xa95, 0xaa8, 0x20, 0xa95, 0xac7, 0xaa8,
0xacd, 0xaa6, 0xacd, 0xab0, 0xac0, 0xaaf, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20,
0xab8, 0xaae, 0xaaf, 0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0, 0x20, 0xa85, 0xaae, 0xac7,
0xab0, 0xabf, 0xa95, 0xaa8, 0x20, 0xa95, 0xac7, 0xaa8, 0xacd, 0xaa6, 0xacd, 0xab0,
0xac0, 0xaaf, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89,
0xaa4, 0xacd, 0xaa4, 0xab0, 0x20, 0xa85, 0xaae, 0xac7, 0xab0, 0xabf, 0xa95, 0xaa8,
0x20, 0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0xac0, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89,
0xaa4, 0xacd, 0xaa4, 0xab0, 0x20, 0xa85, 0xaae, 0xac7, 0xab0, 0xabf, 0xa95, 0xaa8,
0x20, 0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0xac0, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95,
0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0, 0x20, 0xa85, 0xaae,
0xac7, 0xab0, 0xabf, 0xa95, 0xaa8, 0x20, 0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0xac0,
0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xaa4, 0xacd,
0xaa4, 0xab0, 0x20, 0xa85, 0xaae, 0xac7, 0xab0, 0xabf, 0xa95, 0xaa8, 0x20, 0xaae,
0xabe, 0xa89, 0xaa8, 0xacd, 0xa9f, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xaa4,
0xacd, 0xaa4, 0xab0, 0x20, 0xa85, 0xaae, 0xac7, 0xab0, 0xabf, 0xa95, 0xaa8, 0x20,
0xaae, 0xabe, 0xa89, 0xaa8, 0xacd, 0xa9f, 0xaa8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95,
0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0, 0x20, 0xa85, 0xaae,
0xac7, 0xab0, 0xabf, 0xa95, 0xaa8, 0x20, 0xaae, 0xabe, 0xa89, 0xaa8, 0xacd, 0xa9f,
0xaa8, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xaa4,
0xacd, 0xaa4, 0xab0, 0x20, 0xa85, 0xaae, 0xac7, 0xab0, 0xabf, 0xa95, 0xaa8, 0x20,
0xaaa, 0xac7, 0xab8, 0xabf, 0xaab, 0xabf, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89,
0xaa4, 0xacd, 0xaa4, 0xab0, 0x20, 0xa85, 0xaae, 0xac7, 0xab0, 0xabf, 0xa95, 0xaa8,
0x20, 0xaaa, 0xac7, 0xab8, 0xabf, 0xaab, 0xabf, 0xa95, 0x20, 0xaae, 0xabe, 0xaa8,
0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0, 0x20, 0xa85,
0xaae, 0xac7, 0xab0, 0xabf, 0xa95, 0xaa8, 0x20, 0xaaa, 0xac7, 0xab8, 0xabf, 0xaab,
0xabf, 0xa95, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa85,
0xaa8, 0xabe, 0xaa6, 0xabf, 0xab0, 0x20, 0xab8, 0xaae, 0xaaf, 0xa85, 0xaa8, 0xabe,
0xaa1, 0xabf, 0xab0, 0x20, 0xaaa, 0xacd, 0xab0, 0xaae, 0xabe, 0xaa3, 0xaad, 0xac2,
0xaa4, 0x20, 0xab8, 0xaae, 0xaaf, 0xa85, 0xaa8, 0xabe, 0xaa1, 0xabf, 0xab0, 0x20,
0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa8f,
0xaaa, 0xabf, 0xaaf, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xa8f, 0xaaa, 0xabf, 0xaaf,
0xabe, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa8f, 0xaaa,
0xabf, 0xaaf, 0xabe, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf,
0xa85, 0xa95, 0xacd, 0xaa4, 0xabe, 0xa89, 0x20, 0xab8, 0xaae, 0xaaf, 0xa85, 0xa95,
0xacd, 0xaa4, 0xabe, 0xa89, 0x20, 0xaaa, 0xacd, 0xab0, 0xaae, 0xabe, 0xaa3, 0xaad,
0xac2, 0xaa4, 0x20, 0xab8, 0xaae, 0xaaf, 0xa85, 0xa95, 0xacd, 0xaa4, 0xabe, 0xa89,
0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf,
0xa8d, 0xa95, 0xacd, 0xa9f, 0xacb, 0xaac, 0x20, 0xab8, 0xaae, 0xaaf, 0xa8d, 0xa95,
0xacd, 0xa9f, 0xacb, 0xaac, 0x20, 0xaaa, 0xacd, 0xab0, 0xaae, 0xabe, 0xaa3, 0xaad,
0xac2, 0xaa4, 0x20, 0xab8, 0xaae, 0xaaf, 0xa8d, 0xa95, 0xacd, 0xa9f, 0xacb, 0xaac,
0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf,
0xa85, 0xab0, 0xac7, 0xaac, 0xabf, 0xaaf, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa85,
0xab0, 0xac7, 0xaac, 0xabf, 0xaaf, 0xaa8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20,
0xab8, 0xaae, 0xaaf, 0xa85, 0xab0, 0xac7, 0xaac, 0xabf, 0xaaf, 0xaa8, 0x20, 0xaa6,
0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa86, 0xab0, 0xacd, 0xa9c, 0xac7,
0xaa8, 0xacd, 0xa9f, 0xac0, 0xaa8, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xa86, 0xab0,
0xacd, 0xa9c, 0xac7, 0xaa8, 0xacd, 0xa9f, 0xac0, 0xaa8, 0xabe, 0x20, 0xaae, 0xabe,
0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa86, 0xab0, 0xacd, 0xa9c, 0xac7, 0xaa8,
0xacd, 0xa9f, 0xac0, 0xaa8, 0xabe, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd,
0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf, 0xaae, 0xac0,
0x20, 0xa86, 0xab0, 0xacd, 0xa9c, 0xac7, 0xaa8, 0xacd, 0xa9f, 0xac0, 0xaa8, 0xabe,
0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf, 0xaae, 0xac0, 0x20,
0xa86, 0xab0, 0xacd, 0xa9c, 0xac7, 0xaa8, 0xacd, 0xa9f, 0xac0, 0xaa8, 0xabe, 0x20,
0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xab6, 0xacd, 0xa9a,
0xabf, 0xaae, 0xac0, 0x20, 0xa86, 0xab0, 0xacd, 0xa9c, 0xac7, 0xaa8, 0xacd, 0xa9f,
0xac0, 0xaa8, 0xabe, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20,
0xab8, 0xaae, 0xaaf, 0xa86, 0xab0, 0xacd, 0xaae, 0xac7, 0xaa8, 0xabf, 0xaaf, 0xabe,
0x20, 0xab8, 0xaae, 0xaaf, 0xa86, 0xab0, 0xacd, 0xaae, 0xac7, 0xaa8, 0xabf, 0xaaf,
0xabe, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa86, 0xab0,
0xacd, 0xaae, 0xac7, 0xaa8, 0xabf, 0xaaf, 0xabe, 0x20, 0xa97, 0xacd, 0xab0, 0xac0,
0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa8f, 0xa9f, 0xab2, 0xabe, 0xaa8,
0xacd, 0xa9f, 0xabf, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa8f, 0xa9f, 0xab2, 0xabe,
0xaa8, 0xacd, 0xa9f, 0xabf, 0xa95, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8,
0xaae, 0xaaf, 0xa8f, 0xa9f, 0xab2, 0xabe, 0xaa8, 0xacd, 0xa9f, 0xabf, 0xa95, 0x20,
0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xac7, 0xaa8, 0xacd,
0xaa6, 0xacd, 0xab0, 0xac0, 0xaaf, 0x20, 0xa91, 0xab8, 0xacd, 0xa9f, 0xacd, 0xab0,
0xac7, 0xab2, 0xabf, 0xaaf, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa91, 0xab8, 0xacd,
0xa9f, 0xacd, 0xab0, 0xac7, 0xab2, 0xabf, 0xaaf, 0xaa8, 0x20, 0xa95, 0xac7, 0xaa8,
0xacd, 0xaa6, 0xacd, 0xab0, 0xac0, 0xaaf, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20,
0xab8, 0xaae, 0xaaf, 0xa93, 0xab8, 0xacd, 0xa9f, 0xacd, 0xab0, 0xac7, 0xab2, 0xabf,
0xaaf, 0xaa8, 0x20, 0xa95, 0xac7, 0xaa8, 0xacd, 0xaa6, 0xacd, 0xab0, 0xac0, 0xaaf,
0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa91, 0xab8, 0xacd,
0xa9f, 0xacd, 0xab0, 0xac7, 0xab2, 0xabf, 0xaaf, 0xaa8, 0x20, 0xa95, 0xac7, 0xaa8,
0xacd, 0xaa6, 0xacd, 0xab0, 0xac0, 0xaaf, 0x20, 0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf,
0xaae, 0xac0, 0x20, 0xab8, 0xaae, 0xaaf, 0xa91, 0xab8, 0xacd, 0xa9f, 0xacd, 0xab0,
0xac7, 0xab2, 0xabf, 0xaaf, 0xaa8, 0x20, 0xa95, 0xac7, 0xaa8, 0xacd, 0xaa6, 0xacd,
0xab0, 0xac0, 0xaaf, 0x20, 0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf, 0xaae, 0xac0, 0x20,
0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa93, 0xab8, 0xacd, 0xa9f,
0xacd, 0xab0, 0xac7, 0xab2, 0xabf, 0xaaf, 0xaa8, 0x20, 0xa95, 0xac7, 0xaa8, 0xacd,
0xaa6, 0xacd, 0xab0, 0xac0, 0xaaf, 0x20, 0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf, 0xaae,
0xac0, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xac2,
0xab0, 0xacd, 0xab5, 0xac0, 0xaaf, 0x20, 0xa91, 0xab8, 0xacd, 0xa9f, 0xacd, 0xab0,
0xac7, 0xab2, 0xabf, 0xaaf, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xa91, 0xab8, 0xacd,
0xa9f, 0xacd, 0xab0, 0xac7, 0xab2, 0xabf, 0xaaf, 0xaa8, 0x20, 0xaaa, 0xac2, 0xab0,
0xacd, 0xab5, 0xac0, 0xaaf, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae,
0xaaf, 0xa93, 0xab8, 0xacd, 0xa9f, 0xacd, 0xab0, 0xac7, 0xab2, 0xabf, 0xaaf, 0xaa8,
0x20, 0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0xac0, 0xaaf, 0x20, 0xaa6, 0xabf, 0xab5,
0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf, 0xaae, 0xac0,
0x20, 0xa91, 0xab8, 0xacd, 0xa9f, 0xacd, 0xab0, 0xac7, 0xab2, 0xabf, 0xaaf, 0xabe,
0x20, 0xab8, 0xaae, 0xaaf, 0xa91, 0xab8, 0xacd, 0xa9f, 0xacd, 0xab0, 0xac7, 0xab2,
0xabf, 0xaaf, 0xaa8, 0x20, 0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf, 0xaae, 0xac0, 0x20,
0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa93, 0xab8, 0xacd, 0xa9f,
0xacd, 0xab0, 0xac7, 0xab2, 0xabf, 0xaaf, 0xaa8, 0x20, 0xaaa, 0xab6, 0xacd, 0xa9a,
0xabf, 0xaae, 0xac0, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf,
0xa85, 0xa9d, 0xab0, 0xaac, 0xac8, 0xa9c, 0xabe, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf,
0xa85, 0xa9d, 0xab0, 0xaac, 0xac8, 0xa9c, 0xabe, 0xaa8, 0x20, 0xaae, 0xabe, 0xaa8,
0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa85, 0xa9d, 0xab0, 0xaac, 0xac8, 0xa9c, 0xabe,
0xaa8, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae,
0xaaf, 0xa8f, 0xa9d, 0xacb, 0xab0, 0xacd, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa8f,
0xa9d, 0xacb, 0xab0, 0xacd, 0xab8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8,
0xaae, 0xaaf, 0xa8f, 0xa9d, 0xacb, 0xab0, 0xacd, 0xab8, 0x20, 0xa97, 0xacd, 0xab0,
0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaac, 0xabe, 0xa82, 0xa97,
0xacd, 0xab2, 0xabe, 0xaa6, 0xac7, 0xab6, 0x20, 0xab8, 0xaae, 0xaaf, 0xaac, 0xabe,
0xa82, 0xa97, 0xacd, 0xab2, 0xabe, 0xaa6, 0xac7, 0xab6, 0x20, 0xaae, 0xabe, 0xaa8,
0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaac, 0xabe, 0xa82, 0xa97, 0xacd, 0xab2, 0xabe,
0xaa6, 0xac7, 0xab6, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20,
0xab8, 0xaae, 0xaaf, 0xaad, 0xac2, 0xa9f, 0xabe, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf,
0xaac, 0xacb, 0xab2, 0xabf, 0xab5, 0xabf, 0xaaf, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf,
0xaac, 0xacd, 0xab0, 0xabe, 0xa9d, 0xabf, 0xab2, 0xabf, 0xaaf, 0xabe, 0x20, 0xab8,
0xaae, 0xaaf, 0xaac, 0xacd, 0xab0, 0xabe, 0xa9d, 0xabf, 0xab2, 0xabf, 0xaaf, 0xabe,
0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaac, 0xacd, 0xab0,
0xabe, 0xa9d, 0xabf, 0xab2, 0xabf, 0xaaf, 0xabe, 0x20, 0xa97, 0xacd, 0xab0, 0xac0,
0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaac, 0xacd, 0xab0, 0xac1, 0xaa8,
0xac7, 0xa87, 0x20, 0xaa6, 0xab0, 0xac1, 0xab8, 0xab2, 0xabe, 0xaae, 0x20, 0xab8,
0xaae, 0xaaf, 0xa95, 0xac5, 0xaaa, 0x20, 0xab5, 0xab0, 0xacd, 0xaa1, 0xac7, 0x20,
0xab8, 0xaae, 0xaaf, 0xa95, 0xac5, 0xaaa, 0x20, 0xab5, 0xab0, 0xacd, 0xaa1, 0xac7,
0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xac5, 0xaaa,
0x20, 0xab5, 0xab0, 0xacd, 0xaa1, 0xac7, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7,
0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xac7, 0xaae, 0xacb, 0xab0, 0xacb,
0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9a, 0xac7, 0xaa4,
0xab9, 0xabe, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9a, 0xac7, 0xaa4, 0xab9, 0xabe,
0xaae, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9a, 0xac7,
0xaa4, 0xab9, 0xabe, 0xaae, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae,
0xaaf, 0xa9a, 0xabf, 0xab2, 0xac0, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9a, 0xabf, 0xab2,
0xac0, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9a, 0xabf,
0xab2, 0xac0, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8,
0xaae, 0xaaf, 0xa9a, 0xac0, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9a, 0xac0, 0xaa8,
0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9a, 0xac0, 0xaa8,
0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xacd, 0xab0,
0xabf, 0xab8, 0xaae, 0xab8, 0x20, 0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1,
0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xacb, 0xa95, 0xacb, 0xa9d, 0x20, 0xa86, 0xa87,
0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0xacd, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95,
0xacb, 0xab2, 0xaae, 0xacd, 0xaac, 0xabf, 0xaaf, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf,
0xa95, 0xacb, 0xab2, 0xaae, 0xacd, 0xaac, 0xabf, 0xaaf, 0xabe, 0x20, 0xaae, 0xabe,
0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xacb, 0xab2, 0xaae, 0xacd, 0xaac,
0xabf, 0xaaf, 0xabe, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20,
0xab8, 0xaae, 0xaaf, 0xa95, 0xac1, 0xa95, 0x20, 0xa86, 0xa87, 0xab2, 0xac7, 0xaa8,
0xacd, 0xaa1, 0xacd, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xac1, 0xa95, 0x20,
0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0xacd, 0xab8, 0x20, 0xaae, 0xabe,
0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xac1, 0xa95, 0x20, 0xa86, 0xa87,
0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0xacd, 0xab8, 0x20, 0xa85, 0xab0, 0xacd, 0xaa7,
0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf,
0xa95, 0xacd, 0xaaf, 0xac1, 0xaac, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xacd,
0xaaf, 0xac1, 0xaac, 0xabe, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae,
0xaaf, 0xa95, 0xacd, 0xaaf, 0xac1, 0xaac, 0xabe, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8,
0x20, 0xab8, 0xaae, 0xaaf, 0xaa1, 0xac7, 0xab5, 0xabf, 0xab8, 0x20, 0xab8, 0xaae,
0xaaf, 0xaa1, 0xacd, 0xaaf, 0xac1, 0xaae, 0xacb, 0xaa8, 0xacd, 0xa9f, 0x2d, 0xaa1,
0xac0, 0x2018, 0xa89, 0xab0, 0xacd, 0xab5, 0xabf, 0xab2, 0x20, 0xab8, 0xaae, 0xaaf,
0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0x20, 0xaa4, 0xabf, 0xaae, 0xacb, 0xab0, 0x20,
0xab8, 0xaae, 0xaaf, 0xa87, 0xab8, 0xacd, 0xa9f, 0xab0, 0x20, 0xa86, 0xa87, 0xab2,
0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xab8, 0xaae, 0xaaf, 0xa87, 0xab8, 0xacd, 0xa9f,
0xab0, 0x20, 0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xaae, 0xabe,
0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa87, 0xab8, 0xacd, 0xa9f, 0xab0, 0x20,
0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xa97, 0xacd, 0xab0, 0xac0,
0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa8f, 0xa95, 0xacd, 0xab5, 0xabe,
0xaa1, 0xacb, 0xab0, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xaa7, 0xacd, 0xaaf, 0x20,
0xaaf, 0xac1, 0xab0, 0xacb, 0xaaa, 0xabf, 0xaaf, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf,
0xaae, 0xaa7, 0xacd, 0xaaf, 0x20, 0xaaf, 0xac1, 0xab0, 0xacb, 0xaaa, 0xabf, 0xaaf,
0xaa8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xaa7,
0xacd, 0xaaf, 0x20, 0xaaf, 0xac1, 0xab0, 0xacb, 0xaaa, 0xabf, 0xaaf, 0xaa8, 0x20,
0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa,
0xac2, 0xab0, 0xacd, 0xab5, 0xac0, 0x20, 0xaaf, 0xac1, 0xab0, 0xacb, 0xaaa, 0xabf,
0xaaf, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0xac0,
0x20, 0xaaf, 0xac1, 0xab0, 0xacb, 0xaaa, 0xabf, 0xaaf, 0xaa8, 0x20, 0xaae, 0xabe,
0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0xac0,
0x20, 0xaaf, 0xac1, 0xab0, 0xacb, 0xaaa, 0xabf, 0xaaf, 0xaa8, 0x20, 0xa97, 0xacd,
0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xab8, 0xac1, 0xaa6,
0xac2, 0xab0, 0x2d, 0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0xac0, 0x20, 0xaaf, 0xac1,
0xab0, 0xacb, 0xaaa, 0xabf, 0xaaf, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xab6,
0xacd, 0xa9a, 0xabf, 0xaae, 0xac0, 0x20, 0xaaf, 0xac1, 0xab0, 0xacb, 0xaaa, 0xabf,
0xaaf, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf, 0xaae,
0xac0, 0x20, 0xaaf, 0xac1, 0xab0, 0xacb, 0xaaa, 0xabf, 0xaaf, 0xaa8, 0x20, 0xaae,
0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf,
0xaae, 0xac0, 0x20, 0xaaf, 0xac1, 0xab0, 0xacb, 0xaaa, 0xabf, 0xaaf, 0xaa8, 0x20,
0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaab,
0xac9, 0xa95, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xa86, 0xa87, 0xab2, 0xac7,
0xaa8, 0xacd, 0xaa1, 0xacd, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaab, 0xac9, 0xa95,
0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd,
0xaa1, 0xacd, 0xab8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf,
0xaab, 0xac9, 0xa95, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xa86, 0xa87, 0xab2,
0xac7, 0xaa8, 0xacd, 0xaa1, 0xacd, 0xab8, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7,
0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaab, 0xac0, 0xa9c, 0xac0, 0x20, 0xab8,
0xaae, 0xaaf, 0xaab, 0xac0, 0xa9c, 0xac0, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20,
0xab8, 0xaae, 0xaaf, 0xaab, 0xac0, 0xa9c, 0xac0, 0x20, 0xa97, 0xacd, 0xab0, 0xac0,
0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaab, 0xacd, 0xab0, 0xac7, 0xa82,
0xa9a, 0x20, 0xa97, 0xaaf, 0xabe, 0xaa8, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xaab,
0xacd, 0xab0, 0xac7, 0xa82, 0xa9a, 0x20, 0xab8, 0xaa7, 0xab0, 0xacd, 0xaa8, 0x20,
0xa85, 0xaa8, 0xac7, 0x20, 0xa8f, 0xaa8, 0xacd, 0xa9f, 0xabe, 0xab0, 0xacd, 0xa95,
0xa9f, 0xabf, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa97, 0xabe, 0xab2, 0xabe, 0xaaa,
0xabe, 0xa97, 0xacb, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa97, 0xac7, 0xaae, 0xacd,
0xaac, 0xabf, 0xaaf, 0xab0, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9c, 0xacd, 0xaaf, 0xacb,
0xab0, 0xacd, 0xa9c, 0xabf, 0xaaf, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9c, 0xacd,
0xaaf, 0xacb, 0xab0, 0xacd, 0xa9c, 0xabf, 0xaaf, 0xabe, 0x20, 0xaae, 0xabe, 0xaa8,
0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9c, 0xacd, 0xaaf, 0xacb, 0xab0, 0xacd, 0xa9c,
0xabf, 0xaaf, 0xabe, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20,
0xab8, 0xaae, 0xaaf, 0xa97, 0xabf, 0xab2, 0xaac, 0xab0, 0xacd, 0xa9f, 0x20, 0xa86,
0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0xacd, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf,
0xa97, 0xacd, 0xab0, 0xac0, 0xaa8, 0xab5, 0xabf, 0xa9a, 0x20, 0xaae, 0xaa7, 0xacd,
0xaaf, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0x20,
0xa97, 0xacd, 0xab0, 0xac0, 0xaa8, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xab8,
0xaae, 0xaaf, 0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0x20, 0xa97, 0xacd, 0xab0, 0xac0,
0xaa8, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20,
0xab8, 0xaae, 0xaaf, 0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0x20, 0xa97, 0xacd, 0xab0,
0xac0, 0xaa8, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xa97, 0xacd, 0xab0, 0xac0,
0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf,
0xaae, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xaa8, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1,
0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf, 0xaae, 0x20, 0xa97,
0xacd, 0xab0, 0xac0, 0xaa8, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xaae, 0xabe,
0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf, 0xaae,
0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xaa8, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20,
0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa97,
0xacd, 0xab5, 0xabe, 0xaae, 0x20, 0xaaa, 0xacd, 0xab0, 0xaae, 0xabe, 0xaa3, 0xaad,
0xac2, 0xaa4, 0x20, 0xab8, 0xaae, 0xaaf, 0xa97, 0xab2, 0xacd, 0xaab, 0x20, 0xaae,
0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xab9, 0xab5, 0xabe, 0xa88, 0x2d,
0xa8f, 0xab2, 0xacd, 0xaaf, 0xac1, 0xab6, 0xabf, 0xa85, 0xaa8, 0x20, 0xab8, 0xaae,
0xaaf, 0xab9, 0xab5, 0xabe, 0xa87, 0x2d, 0xa8f, 0xab2, 0xacd, 0xaaf, 0xac1, 0xab6,
0xabf, 0xa85, 0xaa8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf,
0xab9, 0xab5, 0xabe, 0xa87, 0x2d, 0xa8f, 0xab2, 0xacd, 0xaaf, 0xac1, 0xab6, 0xabf,
0xa85, 0xaa8, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xab9,
0xacb, 0xa82, 0xa97, 0x20, 0xa95, 0xacb, 0xa82, 0xa97, 0x20, 0xab8, 0xaae, 0xaaf,
0xab9, 0xacb, 0xa82, 0xa97, 0x20, 0xa95, 0xacb, 0xa82, 0xa97, 0x20, 0xaae, 0xabe,
0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xab9, 0xacb, 0xa82, 0xa97, 0x20, 0xa95,
0xacb, 0xa82, 0xa97, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20,
0xab8, 0xaae, 0xaaf, 0xab9, 0xacb, 0xab5, 0xacd, 0xaa1, 0x20, 0xab8, 0xaae, 0xaaf,
0xab9, 0xacb, 0xab5, 0xacd, 0xaa1, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8,
0xaae, 0xaaf, 0xab9, 0xacb, 0xab5, 0xacd, 0xaa1, 0x20, 0xa97, 0xacd, 0xab0, 0xac0,
0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaad, 0xabe, 0xab0, 0xaa4, 0xac0,
0xaaf, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaad, 0xabe,
0xab0, 0xaa4, 0xac0, 0xaaf, 0x20, 0xaae, 0xab9, 0xabe, 0xab8, 0xabe, 0xa97, 0xab0,
0x20, 0xab8, 0xaae, 0xaaf, 0xa87, 0xaa8, 0xacd, 0xaa1, 0xacb, 0xa9a, 0xabe, 0xa87,
0xaa8, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xaa7, 0xacd, 0xaaf, 0x20, 0xa87,
0xaa8, 0xacd, 0xaa1, 0xacb, 0xaa8, 0xac7, 0xab6, 0xabf, 0xaaf, 0xabe, 0x20, 0xab8,
0xaae, 0xaaf, 0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0xac0, 0xaaf, 0x20, 0xa87, 0xaa8,
0xacd, 0xaa1, 0xacb, 0xaa8, 0xac7, 0xab6, 0xabf, 0xaaf, 0xabe, 0x20, 0xab8, 0xaae,
0xaaf, 0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf, 0xaae, 0xac0, 0x20, 0xa87, 0xaa8, 0xacd,
0xaa1, 0xacb, 0xaa8, 0xac7, 0xab6, 0xabf, 0xaaf, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf,
0xa88, 0xab0, 0xabe, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa88, 0xab0, 0xabe, 0xaa8,
0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa88, 0xab0, 0xabe,
0xaa8, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa87, 0xab0,
0xacd, 0xa95, 0xac1, 0xaa4, 0xacd, 0xab8, 0xacd, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf,
0xa87, 0xab0, 0xacd, 0xa95, 0xac1, 0xaa4, 0xacd, 0xab8, 0xacd, 0xa95, 0x20, 0xaae,
0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa87, 0xab0, 0xacd, 0xa95, 0xac1,
0xaa4, 0xacd, 0xab8, 0xacd, 0xa95, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd,
0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa87, 0xa9d, 0xab0, 0xabe, 0xa87, 0xab2, 0x20,
0xab8, 0xaae, 0xaaf, 0xa87, 0xa9d, 0xab0, 0xabe, 0xa87, 0xab2, 0x20, 0xaae, 0xabe,
0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa87, 0xa9d, 0xab0, 0xabe, 0xa87, 0xab2,
0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9c, 0xabe, 0xaaa,
0xabe, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9c, 0xabe, 0xaaa, 0xabe, 0xaa8, 0x20,
0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9c, 0xabe, 0xaaa, 0xabe,
0xaa8, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xac7,
0xa9f, 0xacd, 0xab0, 0xacb, 0xaaa, 0xac7, 0xab5, 0xab2, 0xacb, 0xab5, 0xacd, 0xab8,
0xacd, 0xa95, 0x2d, 0xa95, 0xabe, 0xaae, 0xa9a, 0xaa4, 0xab8, 0xacd, 0xa95, 0xac0,
0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xac7, 0xa9f, 0xacd, 0xab0, 0xacb, 0xaaa, 0xac7,
0xab5, 0xab2, 0xacb, 0xab5, 0xacd, 0xab8, 0xacd, 0xa95, 0x2d, 0xa95, 0xabe, 0xaae,
0xa9a, 0xaa4, 0xab8, 0xacd, 0xa95, 0xac0, 0x20, 0xaaa, 0xacd, 0xab0, 0xaae, 0xabe,
0xaa3, 0xaad, 0xac2, 0xaa4, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xac7, 0xa9f, 0xacd,
0xab0, 0xacb, 0xaaa, 0xac7, 0xab5, 0xab2, 0xacb, 0xab5, 0xacd, 0xab8, 0xacd, 0xa95,
0x2d, 0xa95, 0xabe, 0xaae, 0xa9a, 0xaa4, 0xab8, 0xacd, 0xa95, 0xac0, 0x20, 0xa97,
0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xa9d,
0xabe, 0xa95, 0xabf, 0xab8, 0xacd, 0xaa4, 0xabe, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf,
0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0x20, 0xa95, 0xa9d, 0xabe, 0xa95, 0xabf, 0xab8,
0xacd, 0xaa4, 0xabe, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xab6, 0xacd, 0xa9a,
0xabf, 0xaae, 0x20, 0xa95, 0xa9d, 0xabe, 0xa95, 0xabf, 0xab8, 0xacd, 0xaa4, 0xabe,
0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xacb, 0xab0, 0xabf, 0xaaf, 0xaa8, 0x20,
0xab8, 0xaae, 0xaaf, 0xa95, 0xacb, 0xab0, 0xabf, 0xaaf, 0xaa8, 0x20, 0xaae, 0xabe,
0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xacb, 0xab0, 0xabf, 0xaaf, 0xaa8,
0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xacb, 0xab8,
0xab0, 0xac7, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xacd, 0xab0, 0xac7, 0xab8, 0xacd,
0xaa8, 0xacb, 0xaaf, 0xabe, 0xab0, 0xacd, 0xab8, 0xacd, 0xa95, 0x20, 0xab8, 0xaae,
0xaaf, 0xa95, 0xacd, 0xab0, 0xac7, 0xab8, 0xacd, 0xaa8, 0xacb, 0xaaf, 0xabe, 0xab0,
0xacd, 0xab8, 0xacd, 0xa95, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae,
0xaaf, 0xa95, 0xacd, 0xab0, 0xac7, 0xab8, 0xacd, 0xaa8, 0xacb, 0xaaf, 0xabe, 0xab0,
0xacd, 0xab8, 0xacd, 0xa95, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae,
0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xabf, 0xab0, 0xacd, 0xa97, 0xabf, 0xa9d, 0xacd,
0xab8, 0xacd, 0xaa4, 0xabe, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xab2, 0xa82, 0xa95,
0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xab2, 0xabe, 0xa87, 0xaa8, 0x20, 0xa86, 0xa87,
0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0xacd, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xab2,
0xacb, 0xab0, 0xacd, 0xaa1, 0x20, 0xab9, 0xacb, 0xab5, 0x20, 0xab8, 0xaae, 0xaaf,
0xab2, 0xacb, 0xab0, 0xacd, 0xaa1, 0x20, 0xab9, 0xacb, 0xab5, 0x20, 0xaae, 0xabe,
0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xab2, 0xacb, 0xab0, 0xacd, 0xaa1, 0x20,
0xab9, 0xacb, 0xab5, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf,
0xaae, 0xa95, 0xabe, 0xa89, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xa95, 0xabe, 0xa89,
0x20, 0xaaa, 0xacd, 0xab0, 0xaae, 0xabe, 0xaa3, 0xaad, 0xac2, 0xaa4, 0x20, 0xab8,
0xaae, 0xaaf, 0xaae, 0xa95, 0xabe, 0xa89, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7,
0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xa97, 0xabe, 0xaa1, 0xaa8, 0x20,
0xab8, 0xaae, 0xaaf, 0xaae, 0xa97, 0xabe, 0xaa1, 0xaa8, 0x20, 0xaae, 0xabe, 0xaa8,
0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xa97, 0xabe, 0xaa1, 0xaa8, 0x20, 0xa97,
0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xab2,
0xac7, 0xab6, 0xabf, 0xaaf, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xabe, 0xab2,
0xaa6, 0xac0, 0xab5, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xabe, 0xab0, 0xacd, 0xa95,
0xacd, 0xaaf, 0xac2, 0xab8, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xabe, 0xab0,
0xacd, 0xab6, 0xab2, 0x20, 0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0xacd,
0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xacb, 0xab0, 0xabf, 0xab6, 0xabf, 0xaaf,
0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xacb, 0xab0, 0xabf, 0xab6, 0xabf, 0xaaf,
0xab8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xacb,
0xab0, 0xabf, 0xab6, 0xabf, 0xaaf, 0xab8, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7,
0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xacb, 0xab8, 0xaa8, 0x20, 0xab8,
0xaae, 0xaaf, 0xaae, 0xac7, 0xa95, 0xacd, 0xab8, 0xabf, 0xa95, 0xaa8, 0x20, 0xaaa,
0xac7, 0xab8, 0xabf, 0xaab, 0xabf, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xac7,
0xa95, 0xacd, 0xab8, 0xabf, 0xa95, 0xaa8, 0x20, 0xaaa, 0xac7, 0xab8, 0xabf, 0xaab,
0xabf, 0xa95, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae,
0xac7, 0xa95, 0xacd, 0xab8, 0xabf, 0xa95, 0xaa8, 0x20, 0xaaa, 0xac7, 0xab8, 0xabf,
0xaab, 0xabf, 0xa95, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf,
0xa89, 0xab2, 0xabe, 0xaa8, 0x20, 0xaac, 0xabe, 0xa9f, 0xacb, 0xab0, 0x20, 0xab8,
0xaae, 0xaaf, 0xa89, 0xab2, 0xabe, 0xaa8, 0x20, 0xaac, 0xabe, 0xa9f, 0xacb, 0xab0,
0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xab2, 0xabe,
0xaa8, 0x20, 0xaac, 0xa9f, 0xacb, 0xab0, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7,
0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xacb, 0xab8, 0xacd, 0xa95, 0xacb,
0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xacb, 0xab8, 0xacd, 0xa95, 0xacb, 0x20, 0xaae,
0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xacb, 0xab8, 0xacd, 0xa95,
0xacb, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae,
0xaaf, 0xaae, 0xacd, 0xaaf, 0xabe, 0xaa8, 0xaae, 0xabe, 0xab0, 0x20, 0xab8, 0xaae,
0xaaf, 0xaa8, 0xacc, 0xab0, 0xac1, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa8, 0xac7, 0xaaa,
0xabe, 0xab3, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa8, 0xacd, 0xaaf, 0xac1, 0x20, 0xab8,
0xac7, 0xab2, 0xac7, 0xaa1, 0xacb, 0xaa8, 0xabf, 0xaaf, 0xabe, 0x20, 0xab8, 0xaae,
0xaaf, 0xaa8, 0xacd, 0xaaf, 0xac1, 0x20, 0xab8, 0xac7, 0xab2, 0xac7, 0xaa1, 0xacb,
0xaa8, 0xabf, 0xaaf, 0xabe, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae,
0xaaf, 0xaa8, 0xacd, 0xaaf, 0xac1, 0x20, 0xab8, 0xac7, 0xab2, 0xac7, 0xaa1, 0xacb,
0xaa8, 0xabf, 0xaaf, 0xabe, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae,
0x20, 0xab8, 0xaae, 0xaaf, 0xaa8, 0xacd, 0xaaf, 0xac1, 0xa9d, 0xac0, 0xab2, 0xac7,
0xaa8, 0xacd, 0xaa1, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa8, 0xacd, 0xaaf, 0xac1, 0xa9d,
0xac0, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20,
0xab8, 0xaae, 0xaaf, 0xaa8, 0xacd, 0xaaf, 0xac1, 0xa9d, 0xac0, 0xab2, 0xac7, 0xaa8,
0xacd, 0xaa1, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa8,
0xacd, 0xaaf, 0xac2, 0xaab, 0xabe, 0xa89, 0xaa8, 0xacd, 0xaa1, 0xab2, 0xac7, 0xaa8,
0xacd, 0xaa1, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa8, 0xacd, 0xaaf, 0xac2, 0xaab, 0xabe,
0xa89, 0xaa8, 0xacd, 0xaa1, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xaae, 0xabe,
0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa8, 0xacd, 0xaaf, 0xac2, 0xaab, 0xabe,
0xa89, 0xaa8, 0xacd, 0xaa1, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xaa6, 0xabf,
0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa8, 0xac0, 0xaaf, 0xac1, 0x20, 0xab8,
0xaae, 0xaaf, 0xaa8, 0xacb, 0xab0, 0xaab, 0xac9, 0xa95, 0x20, 0xa86, 0xa87, 0xab2,
0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa8, 0xacb, 0xab0, 0xaab,
0xac9, 0xa95, 0x20, 0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xaae,
0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa8, 0xacb, 0xab0, 0xaab, 0xac9,
0xa95, 0x20, 0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xaa6, 0xabf,
0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaab, 0xab0, 0xacd, 0xaa8, 0xabe, 0xaa8,
0xacd, 0xaa1, 0xacb, 0x20, 0xaa1, 0xac0, 0x20, 0xaa8, 0xacb, 0xab0, 0xacb, 0xaa8,
0xacd, 0xab9, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xaab, 0xab0, 0xacd, 0xaa8, 0xabe,
0xaa8, 0xacd, 0xaa1, 0xacb, 0x20, 0xaa1, 0xac0, 0x20, 0xaa8, 0xacb, 0xab0, 0xacb,
0xaa8, 0xacd, 0xab9, 0xabe, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae,
0xaaf, 0xaab, 0xab0, 0xacd, 0xaa8, 0xabe, 0xaa8, 0xacd, 0xaa1, 0xacb, 0x20, 0xaa6,
0xac7, 0x20, 0xaa8, 0xacb, 0xab0, 0xacb, 0xab9, 0xabe, 0x20, 0xa97, 0xacd, 0xab0,
0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xaa4, 0xacd, 0xaa4,
0xab0, 0x20, 0xaae, 0xabe, 0xab0, 0xabf, 0xaaf, 0xabe, 0xaa8, 0xabe, 0x20, 0xa86,
0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0xacd, 0xab8, 0xaa8, 0xacb, 0xab5, 0xab8,
0xabf, 0xaac, 0xabf, 0xab0, 0xacd, 0xab8, 0xacd, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf,
0xaa8, 0xacb, 0xab5, 0xab8, 0xabf, 0xaac, 0xabf, 0xab0, 0xacd, 0xab8, 0xacd, 0xa95,
0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa8, 0xacb, 0xab5,
0xab8, 0xabf, 0xaac, 0xabf, 0xab0, 0xacd, 0xab8, 0xacd, 0xa95, 0x20, 0xa97, 0xacd,
0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa93, 0xaae, 0xacd,
0xab8, 0xacd, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa93, 0xaae, 0xacd, 0xab8, 0xacd,
0xa95, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa93, 0xaae,
0xacd, 0xab8, 0xacd, 0xa95, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae,
0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xabe, 0xa95, 0xabf, 0xab8, 0xacd, 0xaa4, 0xabe,
0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xabe, 0xa95, 0xabf, 0xab8, 0xacd, 0xaa4,
0xabe, 0xaa8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa,
0xabe, 0xa95, 0xabf, 0xab8, 0xacd, 0xaa4, 0xabe, 0xaa8, 0x20, 0xa97, 0xacd, 0xab0,
0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xab2, 0xabe, 0xa89,
0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xabe, 0xaaa, 0xac1, 0xa86, 0x20, 0xaa8, 0xacd,
0xaaf, 0xac1, 0x20, 0xa97, 0xabf, 0xaa8, 0xac0, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa,
0xac7, 0xab0, 0xabe, 0xa97, 0xacd, 0xab5, 0xac7, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa,
0xac7, 0xab0, 0xabe, 0xa97, 0xacd, 0xab5, 0xac7, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95,
0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xac7, 0xab0, 0xabe, 0xa97, 0xacd, 0xab5, 0xac7,
0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf,
0xaaa, 0xac7, 0xab0, 0xac1, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xac7, 0xab0, 0xac1,
0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xac7, 0xab0,
0xac1, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae,
0xaaf, 0xaab, 0xabf, 0xab2, 0xabf, 0xaaa, 0xabe, 0xa87, 0xaa8, 0x20, 0xab8, 0xaae,
0xaaf, 0xaab, 0xabf, 0xab2, 0xabf, 0xaaa, 0xabe, 0xa87, 0xaa8, 0x20, 0xaae, 0xabe,
0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaab, 0xabf, 0xab2, 0xabf, 0xaaa, 0xabe,
0xa87, 0xaa8, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8,
0xaae, 0xaaf, 0xaab, 0xacb, 0xaa8, 0xabf, 0xa95, 0xacd, 0xab8, 0x20, 0xa86, 0xa87,
0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0xacd, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xab8,
0xac7, 0xa82, 0xa9f, 0x20, 0xaaa, 0xac0, 0xa8f, 0xab0, 0xac0, 0x20, 0xa85, 0xaa8,
0xac7, 0x20, 0xaae, 0xabf, 0xa95, 0xacd, 0xaaf, 0xac1, 0xab2, 0xacb, 0xaa8, 0x20,
0xab8, 0xaae, 0xaaf, 0xab8, 0xac7, 0xa82, 0xa9f, 0x20, 0xaaa, 0xac0, 0xa8f, 0xab0,
0xac0, 0x20, 0xa85, 0xaa8, 0xac7, 0x20, 0xaae, 0xabf, 0xa95, 0xacd, 0xaaf, 0xac1,
0xab2, 0xacb, 0xaa8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf,
0xab8, 0xac7, 0xa82, 0xa9f, 0x20, 0xaaa, 0xac0, 0xa8f, 0xab0, 0xac0, 0x20, 0xa85,
0xaa8, 0xac7, 0x20, 0xaae, 0xabf, 0xa95, 0xacd, 0xaaf, 0xac1, 0xab2, 0xacb, 0xaa8,
0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xabf, 0xa9f,
0xa95, 0xac7, 0xaaf, 0xab0, 0xacd, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xacb,
0xaa8, 0xaaa, 0xac7, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xacd, 0xaaf, 0xacb, 0xa82,
0xa97, 0xaaf, 0xabe, 0xa82, 0xa97, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xabf, 0xa9d,
0xabf, 0xab2, 0xacb, 0xab0, 0xacd, 0xaa1, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95,
0xabf, 0xa9d, 0xabf, 0xab2, 0xacb, 0xab0, 0xacd, 0xaa1, 0xabe, 0x20, 0xaaa, 0xacd,
0xab0, 0xaae, 0xabe, 0xaa3, 0xaad, 0xac2, 0xaa4, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95,
0xabf, 0xa9d, 0xabf, 0xab2, 0xacb, 0xab0, 0xacd, 0xaa1, 0xabe, 0x20, 0xa97, 0xacd,
0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xab0, 0xac0, 0xaaf,
0xac1, 0xaa8, 0xabf, 0xaaf, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xab0, 0xacb, 0xaa5,
0xac7, 0xab0, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xab8, 0xa96, 0xabe, 0xab2, 0xabf,
0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xab8, 0xa96, 0xabe, 0xab2, 0xabf, 0xaa8, 0x20,
0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xab8, 0xa96, 0xabe, 0xab2,
0xabf, 0xaa8, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8,
0xaae, 0xaaf, 0xab8, 0xaae, 0xabe, 0xab0, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xab8,
0xaae, 0xabe, 0xab0, 0xabe, 0x20, 0xaaa, 0xacd, 0xab0, 0xaae, 0xabe, 0xaa3, 0xaad,
0xac2, 0xaa4, 0x20, 0xab8, 0xaae, 0xaaf, 0xab8, 0xaae, 0xabe, 0xab0, 0xabe, 0x20,
0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xab8,
0xaae, 0xacb, 0xa86, 0x20, 0xab8, 0xaae, 0xaaf, 0xab8, 0xaae, 0xacb, 0xa86, 0x20,
0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xab8, 0xaae, 0xacb, 0xa86,
0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf,
0xab8, 0xac7, 0xab6, 0xab2, 0xacd, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xab8, 0xabf,
0xa82, 0xa97, 0xabe, 0xaaa, 0xac1, 0xab0, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20,
0xab8, 0xaae, 0xaaf, 0xab8, 0xacb, 0xab2, 0xacb, 0xaae, 0xaa8, 0x20, 0xa86, 0xa87,
0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0xacd, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa6,
0xa95, 0xacd, 0xab7, 0xabf, 0xaa3, 0x20, 0xa9c, 0xacd, 0xaaf, 0xacb, 0xab0, 0xacd,
0xa9c, 0xabf, 0xaaf, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xab8, 0xac1, 0xab0, 0xac0,
0xaa8, 0xabe, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xab8, 0xacd, 0xaaf, 0xacb, 0xab5,
0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa4, 0xabe, 0xab9, 0xabf, 0xaa4, 0xac0, 0x20,
0xab8, 0xaae, 0xaaf, 0xaa4, 0xabe, 0xa87, 0xaaa, 0xac7, 0xa87, 0x20, 0xab8, 0xaae,
0xaaf, 0xaa4, 0xabe, 0xa87, 0xaaa, 0xac7, 0xa87, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95,
0x20, 0xab8, 0xaae, 0xaaf, 0xaa4, 0xabe, 0xa87, 0xaaa, 0xac7, 0xa87, 0x20, 0xaa6,
0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa4, 0xabe, 0xa9c, 0xac0, 0xa95,
0xabf, 0xab8, 0xacd, 0xaa4, 0xabe, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9f, 0xacb,
0xa95, 0xab2, 0xabe, 0xa89, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9f, 0xacb, 0xa82, 0xa97,
0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9f, 0xacb, 0xa82, 0xa97, 0xabe, 0x20, 0xaae,
0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9f, 0xacb, 0xa82, 0xa97, 0xabe,
0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf,
0xa9a, 0xac1, 0xa89, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa4, 0xac1, 0xab0, 0xacd,
0xa95, 0xaae, 0xac7, 0xaa8, 0xabf, 0xab8, 0xacd, 0xaa4, 0xabe, 0xaa8, 0x20, 0xab8,
0xaae, 0xaaf, 0xaa4, 0xac1, 0xab0, 0xacd, 0xa95, 0xaae, 0xac7, 0xaa8, 0xabf, 0xab8,
0xacd, 0xaa4, 0xabe, 0xaa8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae,
0xaaf, 0xaa4, 0xac1, 0xab0, 0xacd, 0xa95, 0xaae, 0xac7, 0xaa8, 0xabf, 0xab8, 0xacd,
0xaa4, 0xabe, 0xaa8, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20,
0xab8, 0xaae, 0xaaf, 0xa9f, 0xab5, 0xabe, 0xab2, 0xac2, 0x20, 0xab8, 0xaae, 0xaaf,
0xa89, 0xab0, 0xac1, 0xa97, 0xacd, 0xab5, 0xac7, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89,
0xab0, 0xac1, 0xa97, 0xacd, 0xab5, 0xac7, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20,
0xab8, 0xaae, 0xaaf, 0xa89, 0xab0, 0xac1, 0xa97, 0xacd, 0xab5, 0xac7, 0x20, 0xa97,
0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xa9d,
0xacd, 0xaac, 0xac7, 0xa95, 0xabf, 0xab8, 0xacd, 0xaa4, 0xabe, 0xaa8, 0x20, 0xab8,
0xaae, 0xaaf, 0xa89, 0xa9d, 0xacd, 0xaac, 0xac7, 0xa95, 0xabf, 0xab8, 0xacd, 0xaa4,
0xabe, 0xaa8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89,
0xa9d, 0xacd, 0xaac, 0xac7, 0xa95, 0xabf, 0xab8, 0xacd, 0xaa4, 0xabe, 0xaa8, 0x20,
0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xab5,
0xaa8, 0xabe, 0xaa4, 0xac2, 0x20, 0xab8, 0xaae, 0xaaf, 0xab5, 0xaa8, 0xabe, 0xaa4,
0xac2, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xab5, 0xaa8,
0xabe, 0xaa4, 0xac2, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20,
0xab8, 0xaae, 0xaaf, 0xab5, 0xac7, 0xaa8, 0xac7, 0xa9d, 0xac1, 0xa8f, 0xab2, 0xabe,
0x20, 0xab8, 0xaae, 0xaaf, 0xab5, 0xacd, 0xab2, 0xabe, 0xaa1, 0xabf, 0xab5, 0xacb,
0xab8, 0xacd, 0xa9f, 0xacb, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xab5, 0xacd, 0xab2,
0xabe, 0xaa1, 0xabf, 0xab5, 0xacb, 0xab8, 0xacd, 0xa9f, 0xacb, 0xa95, 0x20, 0xaae,
0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xab5, 0xacd, 0xab2, 0xabe, 0xaa1,
0xabf, 0xab5, 0xacb, 0xab8, 0xacd, 0xa9f, 0xacb, 0xa95, 0x20, 0xa97, 0xacd, 0xab0,
0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xab5, 0xacb, 0xab2, 0xacd,
0xa97, 0xacb, 0xa97, 0xacd, 0xab0, 0xac7, 0xaa1, 0x20, 0xab8, 0xaae, 0xaaf, 0xab5,
0xacb, 0xab2, 0xacd, 0xa97, 0xacb, 0xa97, 0xacd, 0xab0, 0xac7, 0xaa1, 0x20, 0xaae,
0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xab5, 0xacb, 0xab2, 0xacd, 0xa97,
0xacb, 0xa97, 0xacd, 0xab0, 0xac7, 0xaa1, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7,
0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xab5, 0xac5, 0xa95, 0x20, 0xa86, 0xa87,
0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xab8, 0xaae, 0xaaf, 0xab5, 0xac9, 0xab2,
0xabf, 0xab8, 0x20, 0xa85, 0xaa8, 0xac7, 0x20, 0xaab, 0xacd, 0xaaf, 0xac1, 0xa9a,
0xac1, 0xaa8, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaf, 0xabe, 0xa95, 0xac1, 0xaa4,
0xacd, 0xab8, 0xacd, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaf, 0xabe, 0xa95, 0xac1,
0xaa4, 0xacd, 0xab8, 0xacd, 0xa95, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8,
0xaae, 0xaaf, 0xaaf, 0xabe, 0xa95, 0xac1, 0xaa4, 0xacd, 0xab8, 0xacd, 0xa95, 0x20,
0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaf,
0xac7, 0xa95, 0xabe, 0xa9f, 0xac7, 0xab0, 0xabf, 0xaa8, 0xaac, 0xab0, 0xacd, 0xa97,
0x20, 0xab8, 0xaae, 0xaaf, 0xaaf, 0xac7, 0xa95, 0xabe, 0xa9f, 0xac7, 0xab0, 0xabf,
0xaa8, 0xaac, 0xab0, 0xacd, 0xa97, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8,
0xaae, 0xaaf, 0xaaf, 0xac7, 0xa95, 0xabe, 0xa9f, 0xac7, 0xab0, 0xabf, 0xaa8, 0xaac,
0xab0, 0xacd, 0xa97, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20,
0xab8, 0xaae, 0xaaf, 0xaaf, 0xac1, 0xa95, 0xacb, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x66, 0x67, 0x68,
0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x41, 0x66, 0x69, 0x72, 0x6b, 0x61, 0x20, 0x74, 0x61,
0x20, 0x54, 0x73, 0x61, 0x6b, 0x69, 0x79, 0x61, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x47, 0x61, 0x62, 0x61, 0x73, 0x68, 0x69, 0x6e,
0x20, 0x41, 0x66, 0x69, 0x72, 0x6b, 0x61, 0x54, 0x73, 0x61, 0x79, 0x61,
0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x41, 0x66, 0x69, 0x72, 0x6b, 0x61, 0x20, 0x74, 0x61, 0x20, 0x4b,
0x75, 0x64, 0x75, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41,
0x66, 0x69, 0x72, 0x6b, 0x61, 0x20, 0x74, 0x61, 0x20, 0x59, 0x61, 0x6d,
0x6d, 0x61, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x66, 0x69, 0x72,
0x6b, 0x61, 0x20, 0x74, 0x61, 0x20, 0x59, 0x61, 0x6d, 0x6d, 0x61, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72,
0x61, 0x20, 0x6e, 0x61, 0x20, 0x41, 0x66, 0x69, 0x72, 0x6b, 0x61, 0x20,
0x74, 0x61, 0x20, 0x59, 0x61, 0x6d, 0x6d, 0x61, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x54, 0x73,
0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e,
0x61, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x54, 0x73,
0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61,
0x20, 0x6e, 0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x20, 0x64, 0x61, 0x6b, 0x65, 0x20, 0x41, 0x6d,
0x75, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x61, 0x72, 0x65, 0x77, 0x61, 0x20,
0x74, 0x61, 0x20, 0x74, 0x73, 0x61, 0x6b, 0x69, 0x79, 0x61, 0x72, 0x54,
0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x20, 0x64, 0x61, 0x6b, 0x65, 0x20, 0x41, 0x72, 0x65,
0x77, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x6d, 0x75, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x74, 0x61, 0x20, 0x54, 0x73, 0x61, 0x6b, 0x69, 0x79, 0x61,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61,
0x20, 0x64, 0x61, 0x6b, 0x65, 0x20, 0x41, 0x72, 0x65, 0x77, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x41, 0x6d, 0x75, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x74,
0x61, 0x20, 0x54, 0x73, 0x61, 0x6b, 0x69, 0x79, 0x61, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x61, 0x62, 0x61, 0x73, 0x20, 0x64,
0x61, 0x6b, 0x65, 0x20, 0x41, 0x72, 0x65, 0x77, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x41, 0x6d, 0x75, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x54, 0x73, 0x61,
0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x47, 0x61, 0x62, 0x61, 0x73, 0x20, 0x64, 0x61, 0x6b,
0x65, 0x20, 0x41, 0x72, 0x65, 0x77, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41,
0x6d, 0x75, 0x72, 0x69, 0x6b, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x74, 0x61, 0x20, 0x47, 0x61,
0x62, 0x61, 0x73, 0x20, 0x64, 0x61, 0x6b, 0x65, 0x20, 0x41, 0x72, 0x65,
0x77, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x6d, 0x75, 0x72, 0x69, 0x6b,
0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x54, 0x73, 0x61,
0x75, 0x6e, 0x69, 0x20, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x65, 0x77, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x41, 0x6d, 0x75, 0x72, 0x6b, 0x61, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x20, 0x74, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79,
0x65, 0x20, 0x6e, 0x61, 0x20, 0x74, 0x73, 0x61, 0x75, 0x6e, 0x69, 0x20,
0x61, 0x20, 0x41, 0x72, 0x65, 0x77, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41,
0x6d, 0x75, 0x72, 0x69, 0x63, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x54, 0x73,
0x61, 0x75, 0x6e, 0x69, 0x6e, 0x20, 0x41, 0x72, 0x65, 0x77, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x41, 0x6d, 0x75, 0x72, 0x6b, 0x61, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x72, 0x65, 0x77, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x41, 0x6d, 0x75, 0x72, 0x69, 0x6b, 0x61, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x20, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65,
0x20, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x65, 0x77, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x41, 0x6d, 0x75, 0x72, 0x69, 0x6b, 0x61, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20,
0x41, 0x72, 0x65, 0x77, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x6d, 0x75,
0x72, 0x6b, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41,
0x70, 0x69, 0x61, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e,
0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x70, 0x69,
0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e,
0x61, 0x20, 0x6e, 0x61, 0x20, 0x41, 0x70, 0x69, 0x61, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x72, 0x65, 0x62, 0x69, 0x79, 0x61,
0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x6c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x72, 0x65, 0x62, 0x69, 0x79,
0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e,
0x61, 0x20, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x65, 0x62, 0x69, 0x79, 0x61,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79,
0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e,
0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x59, 0x61, 0x6d, 0x6d, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x59, 0x61, 0x6d, 0x6d, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a,
0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x59, 0x61, 0x6d, 0x6d, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x72, 0x6d,
0x65, 0x6e, 0x69, 0x61, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65,
0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x72,
0x6d, 0x65, 0x6e, 0x69, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x41,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x4b, 0x61, 0x6e, 0x61, 0x64, 0x61, 0x2c, 0x20, 0x50, 0x75,
0x65, 0x72, 0x74, 0x6f, 0x20, 0x52, 0x69, 0x63, 0x6f, 0x20, 0x64, 0x61,
0x20, 0x56, 0x69, 0x72, 0x67, 0x69, 0x6e, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e,
0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4b, 0x61, 0x6e,
0x61, 0x64, 0x61, 0x2c, 0x20, 0x50, 0x75, 0x65, 0x72, 0x74, 0x6f, 0x20,
0x52, 0x69, 0x63, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x56, 0x69, 0x72, 0x67,
0x69, 0x6e, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e,
0x61, 0x20, 0x4b, 0x61, 0x6e, 0x61, 0x64, 0x61, 0x2c, 0x20, 0x50, 0x75,
0x65, 0x72, 0x74, 0x6f, 0x20, 0x52, 0x69, 0x63, 0x6f, 0x20, 0x64, 0x61,
0x20, 0x56, 0x69, 0x72, 0x67, 0x69, 0x6e, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x54,
0x73, 0x61, 0x6b, 0x69, 0x79, 0x61, 0x72, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79,
0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x54, 0x73, 0x61, 0x6b, 0x69, 0x79, 0x61, 0x72, 0x20, 0x41, 0x75, 0x73,
0x74, 0x69, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20,
0x54, 0x73, 0x61, 0x6b, 0x69, 0x79, 0x61, 0x72, 0x20, 0x41, 0x75, 0x73,
0x74, 0x69, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x59, 0x61, 0x6d, 0x6d, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x54, 0x73, 0x61, 0x6b, 0x69, 0x79, 0x61, 0x72, 0x20, 0x41, 0x75,
0x73, 0x74, 0x69, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x54, 0x73, 0x61,
0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x59, 0x61, 0x6d, 0x6d, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x54, 0x73, 0x61, 0x6b, 0x69, 0x79, 0x61, 0x72, 0x20, 0x41, 0x75, 0x73,
0x74, 0x69, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20,
0x59, 0x61, 0x6d, 0x6d, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x54, 0x73, 0x61,
0x6b, 0x69, 0x79, 0x61, 0x72, 0x20, 0x41, 0x75, 0x73, 0x74, 0x69, 0x72,
0x61, 0x6c, 0x69, 0x79, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x47, 0x61, 0x62, 0x61, 0x73, 0x68, 0x69, 0x6e, 0x20, 0x41, 0x75,
0x73, 0x74, 0x69, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x54, 0x73, 0x61,
0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x6c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x47, 0x61, 0x62, 0x61, 0x73, 0x68, 0x69, 0x6e, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61,
0x20, 0x47, 0x61, 0x62, 0x61, 0x73, 0x68, 0x69, 0x6e, 0x20, 0x41, 0x75,
0x73, 0x74, 0x69, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x59, 0x61, 0x6d, 0x6d, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x59, 0x61, 0x6d, 0x6d, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e,
0x61, 0x20, 0x6e, 0x61, 0x20, 0x59, 0x61, 0x6d, 0x6d, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79,
0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61,
0x20, 0x6e, 0x61, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a,
0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x7a,
0x6f, 0x72, 0x65, 0x73, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x20, 0x44, 0x61, 0x69, 0x64, 0x61,
0x69, 0x74, 0x61, 0x63, 0x63, 0x65, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x41, 0x7a, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x42,
0x61, 0x7a, 0x61, 0x72, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x54,
0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x73, 0x68, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42,
0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x72, 0x61, 0x73,
0x69, 0x6c, 0x6c, 0x69, 0x61, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79,
0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42,
0x72, 0x61, 0x73, 0x69, 0x6c, 0x6c, 0x69, 0x61, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e,
0x61, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x6c, 0x69, 0x61, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x72, 0x75, 0x6e, 0x65,
0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x43, 0x61, 0x70, 0x65,
0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79,
0x79, 0x65, 0x6e, 0x20, 0x6c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61,
0x20, 0x6e, 0x61, 0x20, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72,
0x64, 0x65, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x61, 0x6d,
0x6f, 0x72, 0x72, 0x6f, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x54, 0x73, 0x61, 0x79, 0x61,
0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x54, 0x73, 0x61, 0x79,
0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61,
0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x53, 0x69, 0x6e, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79,
0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x53,
0x69, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61,
0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x53, 0x69, 0x6e, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d,
0x61, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x54, 0x73,
0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61,
0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x43, 0x6f,
0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x54, 0x73,
0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x52, 0x61, 0x62, 0x69, 0x6e, 0x20, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61,
0x20, 0x6e, 0x61, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x20, 0x6e,
0x61, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79,
0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x4b, 0x75, 0x62, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x4b, 0x75, 0x62, 0x61,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x44, 0x61, 0x76, 0x69,
0x73, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x44, 0x75, 0x6d,
0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c,
0x65, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x45, 0x61, 0x73,
0x74, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65,
0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x45, 0x61,
0x73, 0x74, 0x65, 0x72, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72,
0x61, 0x20, 0x6e, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x54, 0x73, 0x61,
0x6b, 0x69, 0x79, 0x61, 0x72, 0x20, 0x61, 0x20, 0x6c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x20, 0x74, 0x75, 0x72, 0x61, 0x69, 0x49, 0x64, 0x61, 0x20,
0x54, 0x73, 0x61, 0x6b, 0x69, 0x79, 0x61, 0x72, 0x20, 0x61, 0x20, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x20, 0x54, 0x75, 0x72, 0x61, 0x69, 0x54,
0x73, 0x61, 0x6b, 0x69, 0x79, 0x61, 0x72, 0x20, 0x62, 0x61, 0x7a, 0x61,
0x72, 0x61, 0x20, 0x61, 0x20, 0x6c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x20,
0x74, 0x75, 0x72, 0x61, 0x69, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x20,
0x61, 0x20, 0x74, 0x75, 0x72, 0x61, 0x69, 0x20, 0x67, 0x61, 0x62, 0x61,
0x73, 0x49, 0x64, 0x61, 0x20, 0x6c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x20,
0x61, 0x20, 0x74, 0x75, 0x72, 0x61, 0x69, 0x20, 0x67, 0x61, 0x62, 0x61,
0x73, 0x47, 0x61, 0x62, 0x61, 0x73, 0x20, 0x61, 0x20, 0x6c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x20, 0x74, 0x75, 0x72, 0x61, 0x69, 0x20, 0x64, 0x61,
0x20, 0x62, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x47, 0x61, 0x62, 0x61, 0x73, 0x68, 0x69, 0x6e, 0x20,
0x54, 0x75, 0x72, 0x61, 0x69, 0x20, 0x6d, 0x61, 0x69, 0x20, 0x4e, 0x69,
0x73, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x20, 0x74, 0x61, 0x20,
0x79, 0x61, 0x6d, 0x6d, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x74, 0x75, 0x72,
0x61, 0x69, 0x49, 0x64, 0x61, 0x20, 0x6c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x20, 0x74, 0x61, 0x20, 0x79, 0x61, 0x6d, 0x6d, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x74, 0x75, 0x72, 0x61, 0x69, 0x49, 0x64, 0x61, 0x20, 0x6c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x20, 0x74, 0x61, 0x20, 0x79, 0x61, 0x6d, 0x6d,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x74, 0x75, 0x72, 0x61, 0x69, 0x20, 0x64,
0x61, 0x20, 0x62, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x54, 0x73, 0x61, 0x79,
0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x46, 0x69, 0x6a, 0x69, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65,
0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x46, 0x69,
0x6a, 0x69, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61,
0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x46, 0x69, 0x6a, 0x69,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x46, 0x72, 0x65, 0x6e,
0x63, 0x68, 0x20, 0x47, 0x75, 0x69, 0x61, 0x6e, 0x61, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x4b, 0x75, 0x64, 0x61, 0x6e, 0x63, 0x69,
0x6e, 0x20, 0x46, 0x61, 0x72, 0x61, 0x6e, 0x73, 0x61, 0x20, 0x64, 0x61,
0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x63, 0x74, 0x69, 0x63, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61,
0x67, 0x6f, 0x73, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x47,
0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x54, 0x73, 0x61,
0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61,
0x20, 0x6e, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65,
0x72, 0x74, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77,
0x69, 0x63, 0x68, 0x20, 0x61, 0x20, 0x49, 0x6e, 0x67, 0x69, 0x6c, 0x61,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x61, 0x62, 0x61,
0x73, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x61, 0x62, 0x61,
0x73, 0x68, 0x69, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61,
0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x61, 0x62, 0x61, 0x73, 0x68,
0x69, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x59, 0x61, 0x6d, 0x6d,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x59, 0x61, 0x6d, 0x6d,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61,
0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x59, 0x61, 0x6d, 0x6d, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x6c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x75, 0x6c, 0x66, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x20, 0x6e, 0x61, 0x20, 0x48, 0x61,
0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61,
0x6e, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69,
0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e,
0x61, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65,
0x75, 0x74, 0x69, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x54, 0x73,
0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e,
0x67, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a,
0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6e, 0x67, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x48, 0x6f, 0x76, 0x64, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65,
0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x48, 0x6f,
0x76, 0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61,
0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x48, 0x6f, 0x76, 0x64,
0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x6c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x79, 0x61,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x54, 0x65, 0x6b, 0x75,
0x6e, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e,
0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x49, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x74, 0x61, 0x20, 0x54, 0x73,
0x61, 0x6b, 0x69, 0x79, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x47, 0x61, 0x62, 0x61, 0x73, 0x68, 0x69, 0x6e, 0x20, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x59, 0x61, 0x6d, 0x6d, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x54, 0x73, 0x61,
0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x49,
0x72, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x49,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79,
0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61,
0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x49, 0x73, 0x72, 0x61, 0x2bc, 0x69, 0x6c, 0x61,
0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x6c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x49, 0x73, 0x72, 0x61, 0x2bc, 0x69,
0x6c, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x48, 0x61,
0x73, 0x6b, 0x65, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61,
0x20, 0x49, 0x73, 0x72, 0x61, 0x2bc, 0x69, 0x6c, 0x61, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x54, 0x73,
0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x6c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x48, 0x61, 0x73, 0x6b, 0x65, 0x6e, 0x20,
0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x4a, 0x61, 0x70, 0x61,
0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4b, 0x61, 0x7a,
0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x47, 0x61, 0x62, 0x61, 0x73, 0x68, 0x69, 0x6e, 0x20,
0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x59, 0x61, 0x6d, 0x6d, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61,
0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4b, 0x6f, 0x72,
0x69, 0x79, 0x61, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e,
0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4b, 0x6f, 0x72,
0x69, 0x79, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52,
0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x4b, 0x6f, 0x72, 0x69, 0x79,
0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4b, 0x6f, 0x73,
0x72, 0x61, 0x65, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x54, 0x73,
0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x73, 0x6b, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42,
0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x4b, 0x79, 0x72, 0x67, 0x79, 0x7a, 0x73, 0x74,
0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4c, 0x69,
0x6e, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e,
0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x56, 0x6f,
0x74, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e,
0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x4d, 0x61, 0x6c, 0x61, 0x79, 0x73, 0x69, 0x61, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65,
0x73, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4d, 0x61, 0x72,
0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x54,
0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69,
0x75, 0x73, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61,
0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x74, 0x69, 0x75, 0x73, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x4d, 0x65, 0x6b, 0x7a, 0x69, 0x6b, 0x6f, 0x20, 0x50,
0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79,
0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x4d, 0x65, 0x6b, 0x7a, 0x69, 0x6b, 0x6f, 0x20, 0x50, 0x61, 0x63, 0x69,
0x66, 0x69, 0x63, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52,
0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x4d, 0x65, 0x6b, 0x7a, 0x69,
0x6b, 0x6f, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62,
0x61, 0x61, 0x74, 0x61, 0x72, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79,
0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x55,
0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61,
0x20, 0x6e, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61,
0x74, 0x61, 0x72, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4d,
0x6f, 0x73, 0x63, 0x6f, 0x77, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79,
0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4d,
0x6f, 0x73, 0x63, 0x6f, 0x77, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x4d,
0x6f, 0x73, 0x63, 0x6f, 0x77, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x54, 0x73, 0x61, 0x79, 0x61,
0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x69, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61,
0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20,
0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61,
0x6c, 0x61, 0x6e, 0x64, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65,
0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4e, 0x65,
0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61,
0x20, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4e, 0x65, 0x77, 0x66,
0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x20, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x20,
0x74, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c,
0x61, 0x6e, 0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x20, 0x72, 0x61,
0x6e, 0x61, 0x20, 0x74, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75,
0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65,
0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61,
0x20, 0x6e, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x54, 0x73, 0x61,
0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61,
0x20, 0x6e, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x54,
0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20,
0x6e, 0x61, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x54,
0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a,
0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x50,
0x61, 0x6c, 0x61, 0x75, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x47, 0x75,
0x69, 0x6e, 0x65, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x54, 0x73, 0x61, 0x79,
0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61,
0x20, 0x6e, 0x61, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x50, 0x65, 0x72, 0x75,
0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x50, 0x65, 0x72, 0x75, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61,
0x20, 0x6e, 0x61, 0x20, 0x50, 0x65, 0x72, 0x75, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69,
0x6e, 0x65, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x50, 0x68, 0x69, 0x6c,
0x69, 0x70, 0x70, 0x69, 0x6e, 0x65, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20,
0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x65, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69,
0x78, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x64, 0x61, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e,
0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x53, 0x74, 0x2e,
0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x64, 0x61, 0x20, 0x4d,
0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x53,
0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x64, 0x61,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72,
0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x50, 0x6f, 0x6e,
0x61, 0x70, 0x65, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x50,
0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65,
0x72, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x53, 0x61,
0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79,
0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e,
0x61, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x54,
0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e,
0x61, 0x20, 0x56, 0x6f, 0x74, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x53, 0x65, 0x79, 0x63,
0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79,
0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61,
0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x4b, 0x75, 0x64, 0x61, 0x6e, 0x63, 0x69, 0x6e,
0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x53, 0x79, 0x6f, 0x77,
0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x68,
0x69, 0x74, 0x69, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79,
0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x54, 0x61, 0x69,
0x70, 0x65, 0x69, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x54,
0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x54, 0x6f, 0x6e, 0x67,
0x61, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61,
0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x54, 0x73, 0x61, 0x79,
0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61,
0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61,
0x79, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x79, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61,
0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x55, 0x72, 0x75, 0x67,
0x75, 0x61, 0x79, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x55,
0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x54, 0x73, 0x61,
0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a,
0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x54, 0x73, 0x61, 0x79,
0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20,
0x6e, 0x61, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75,
0x65, 0x6c, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x54, 0x73,
0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42,
0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x64, 0x61, 0x20, 0x46, 0x75,
0x74, 0x75, 0x6e, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x54, 0x73, 0x61, 0x79, 0x61,
0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e,
0x61, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79,
0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a,
0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d0, 0x5e4, 0x5d2, 0x5e0, 0x5d9, 0x5e1, 0x5d8, 0x5df, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5de, 0x5e8, 0x5db, 0x5d6, 0x20, 0x5d0, 0x5e4, 0x5e8, 0x5d9,
0x5e7, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d6, 0x5e8, 0x5d7, 0x20,
0x5d0, 0x5e4, 0x5e8, 0x5d9, 0x5e7, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d3,
0x5e8, 0x5d5, 0x5dd, 0x20, 0x5d0, 0x5e4, 0x5e8, 0x5d9, 0x5e7, 0x5d4, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d0, 0x5e4, 0x5e8, 0x5d9,
0x5e7, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20,
0x5d0, 0x5e4, 0x5e8, 0x5d9, 0x5e7, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d0,
0x5e4, 0x5e8, 0x5d9, 0x5e7, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5dc, 0x5e1, 0x5e7, 0x5d4, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d0, 0x5dc, 0x5e1, 0x5e7, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8,
0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5dc, 0x5e1, 0x5e7, 0x5d4,
0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0,
0x5dc, 0x5de, 0x5d8, 0x5d9, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5dc, 0x5de,
0x5d8, 0x5d9, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d0, 0x5dc, 0x5de, 0x5d8, 0x5d9, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5de, 0x5d6, 0x5d5, 0x5e0, 0x5e1,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5de, 0x5d6, 0x5d5, 0x5e0, 0x5e1, 0x20,
0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0,
0x5de, 0x5d6, 0x5d5, 0x5e0, 0x5e1, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e8, 0x5db, 0x5d6, 0x20, 0x5d0, 0x5e8, 0x5d4,
0x5f4, 0x5d1, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e8, 0x5db, 0x5d6, 0x20,
0x5d0, 0x5e8, 0x5d4, 0x5f4, 0x5d1, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e8, 0x5db, 0x5d6, 0x20, 0x5d0, 0x5e8,
0x5d4, 0x5f4, 0x5d1, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d4, 0x5d7, 0x5d5, 0x5e3, 0x20, 0x5d4, 0x5de, 0x5d6, 0x5e8, 0x5d7,
0x5d9, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d7, 0x5d5, 0x5e3, 0x20, 0x5d4,
0x5de, 0x5d6, 0x5e8, 0x5d7, 0x5d9, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d7, 0x5d5, 0x5e3, 0x20, 0x5d4, 0x5de,
0x5d6, 0x5e8, 0x5d7, 0x5d9, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d0, 0x5d6, 0x5d5, 0x5e8, 0x20, 0x5d4, 0x5d4, 0x5e8, 0x5d9,
0x5dd, 0x20, 0x5d1, 0x5d0, 0x5e8, 0x5d4, 0x5f4, 0x5d1, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d0, 0x5d6, 0x5d5, 0x5e8, 0x20, 0x5d4, 0x5d4, 0x5e8, 0x5d9, 0x5dd, 0x20,
0x5d1, 0x5d0, 0x5e8, 0x5d4, 0x5f4, 0x5d1, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d6, 0x5d5, 0x5e8, 0x20, 0x5d4,
0x5d4, 0x5e8, 0x5d9, 0x5dd, 0x20, 0x5d1, 0x5d0, 0x5e8, 0x5d4, 0x5f4, 0x5d1, 0x20,
0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2,
0x5e8, 0x5d1, 0x20, 0x5d0, 0x5e8, 0x5d4, 0x5f4, 0x5d1, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d0, 0x5e8, 0x5d4, 0x5f4, 0x5d1, 0x20,
0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de,
0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d0, 0x5e8, 0x5d4, 0x5f4, 0x5d1, 0x20, 0x28, 0x5e7,
0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5e0, 0x5d3, 0x5d9,
0x5e8, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e8, 0x5d2, 0x5d9, 0x5dc, 0x20, 0x5d0,
0x5e0, 0x5d3, 0x5d9, 0x5e8, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d9, 0x5e5,
0x20, 0x5d0, 0x5e0, 0x5d3, 0x5d9, 0x5e8, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0,
0x5e4, 0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5e4, 0x5d9, 0x5d4,
0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d0, 0x5e4, 0x5d9, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d0, 0x5e7, 0x5d8, 0x5d0, 0x5d5, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d0, 0x5e7, 0x5d8, 0x5d0, 0x5d5, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5e7, 0x5d8, 0x5d0, 0x5d5, 0x20,
0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5e7,
0x5d8, 0x5d5, 0x5d1, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d5, 0x5e7,
0x5d8, 0x5d5, 0x5d1, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d5, 0x5e7, 0x5d8, 0x5d5, 0x5d1, 0x5d4, 0x20,
0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d7, 0x5e6,
0x5d9, 0x20, 0x5d4, 0x5d0, 0x5d9, 0x20, 0x5e2, 0x5e8, 0x5d1, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d7, 0x5e6, 0x5d9, 0x20, 0x5d4, 0x5d0, 0x5d9, 0x20, 0x5e2, 0x5e8,
0x5d1, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d7, 0x5e6, 0x5d9, 0x20, 0x5d4, 0x5d0, 0x5d9, 0x20, 0x5e2, 0x5e8, 0x5d1,
0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0,
0x5e8, 0x5d2, 0x5e0, 0x5d8, 0x5d9, 0x5e0, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d0, 0x5e8, 0x5d2, 0x5e0, 0x5d8, 0x5d9, 0x5e0, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5,
0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5e8, 0x5d2, 0x5e0,
0x5d8, 0x5d9, 0x5e0, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d0, 0x5e8, 0x5d2, 0x5e0,
0x5d8, 0x5d9, 0x5e0, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8,
0x5d1, 0x20, 0x5d0, 0x5e8, 0x5d2, 0x5e0, 0x5d8, 0x5d9, 0x5e0, 0x5d4, 0x20, 0x28,
0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2,
0x5e8, 0x5d1, 0x20, 0x5d0, 0x5e8, 0x5d2, 0x5e0, 0x5d8, 0x5d9, 0x5e0, 0x5d4, 0x20,
0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5e8,
0x5de, 0x5e0, 0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5e8, 0x5de,
0x5e0, 0x5d9, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d0, 0x5e8, 0x5de, 0x5e0, 0x5d9, 0x5d4, 0x20, 0x28, 0x5e7,
0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0, 0x5d5, 0x5e7,
0x5d9, 0x5d9, 0x5e0, 0x5d5, 0x5e1, 0x20, 0x5d4, 0x5d0, 0x5d8, 0x5dc, 0x5e0, 0x5d8,
0x5d9, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0, 0x5d5, 0x5e7, 0x5d9, 0x5d9,
0x5e0, 0x5d5, 0x5e1, 0x20, 0x5d4, 0x5d0, 0x5d8, 0x5dc, 0x5e0, 0x5d8, 0x5d9, 0x20,
0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4,
0x5d0, 0x5d5, 0x5e7, 0x5d9, 0x5d9, 0x5e0, 0x5d5, 0x5e1, 0x20, 0x5d4, 0x5d0, 0x5d8,
0x5dc, 0x5e0, 0x5d8, 0x5d9, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5de, 0x5e8, 0x5db, 0x5d6, 0x20, 0x5d0, 0x5d5, 0x5e1, 0x5d8,
0x5e8, 0x5dc, 0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e8, 0x5db,
0x5d6, 0x20, 0x5d0, 0x5d5, 0x5e1, 0x5d8, 0x5e8, 0x5dc, 0x5d9, 0x5d4, 0x20, 0x28,
0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e8,
0x5db, 0x5d6, 0x20, 0x5d0, 0x5d5, 0x5e1, 0x5d8, 0x5e8, 0x5dc, 0x5d9, 0x5d4, 0x20,
0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e8,
0x5db, 0x5d6, 0x2d, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d0, 0x5d5, 0x5e1, 0x5d8,
0x5e8, 0x5dc, 0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e8, 0x5db,
0x5d6, 0x2d, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d0, 0x5d5, 0x5e1, 0x5d8, 0x5e8,
0x5dc, 0x5d9, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5de, 0x5e8, 0x5db, 0x5d6, 0x2d, 0x5de, 0x5e2, 0x5e8, 0x5d1,
0x20, 0x5d0, 0x5d5, 0x5e1, 0x5d8, 0x5e8, 0x5dc, 0x5d9, 0x5d4, 0x20, 0x28, 0x5e7,
0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d6, 0x5e8, 0x5d7,
0x20, 0x5d0, 0x5d5, 0x5e1, 0x5d8, 0x5e8, 0x5dc, 0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5de, 0x5d6, 0x5e8, 0x5d7, 0x20, 0x5d0, 0x5d5, 0x5e1, 0x5d8, 0x5e8,
0x5dc, 0x5d9, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5de, 0x5d6, 0x5e8, 0x5d7, 0x20, 0x5d0, 0x5d5, 0x5e1, 0x5d8,
0x5e8, 0x5dc, 0x5d9, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d0, 0x5d5, 0x5e1, 0x5d8,
0x5e8, 0x5dc, 0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8,
0x5d1, 0x20, 0x5d0, 0x5d5, 0x5e1, 0x5d8, 0x5e8, 0x5dc, 0x5d9, 0x5d4, 0x20, 0x28,
0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2,
0x5e8, 0x5d1, 0x20, 0x5d0, 0x5d5, 0x5e1, 0x5d8, 0x5e8, 0x5dc, 0x5d9, 0x5d4, 0x20,
0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d6,
0x5e8, 0x5d1, 0x5d9, 0x5d9, 0x5d2, 0x5f3, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d0, 0x5d6, 0x5e8, 0x5d1, 0x5d9, 0x5d9, 0x5d2, 0x5f3, 0x5df, 0x20, 0x28, 0x5d7,
0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d6, 0x5e8,
0x5d1, 0x5d9, 0x5d9, 0x5d2, 0x5f3, 0x5df, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0, 0x5d9, 0x5d9, 0x5dd, 0x20, 0x5d4,
0x5d0, 0x5d6, 0x5d5, 0x5e8, 0x5d9, 0x5d9, 0x5dd, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d4, 0x5d0, 0x5d9, 0x5d9, 0x5dd, 0x20, 0x5d4, 0x5d0, 0x5d6, 0x5d5, 0x5e8, 0x5d9,
0x5d9, 0x5dd, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d4, 0x5d0, 0x5d9, 0x5d9, 0x5dd, 0x20, 0x5d4, 0x5d0, 0x5d6, 0x5d5,
0x5e8, 0x5d9, 0x5d9, 0x5dd, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d1, 0x5e0, 0x5d2, 0x5dc, 0x5d3, 0x5e9, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d1, 0x5e0, 0x5d2, 0x5dc, 0x5d3, 0x5e9, 0x20, 0x28, 0x5d7, 0x5d5,
0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d1, 0x5e0, 0x5d2, 0x5dc,
0x5d3, 0x5e9, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d1, 0x5d4, 0x5d5, 0x5d8, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d1,
0x5d5, 0x5dc, 0x5d9, 0x5d1, 0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d1,
0x5e8, 0x5d6, 0x5d9, 0x5dc, 0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d1,
0x5e8, 0x5d6, 0x5d9, 0x5dc, 0x5d9, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d1, 0x5e8, 0x5d6, 0x5d9, 0x5dc, 0x5d9,
0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d1, 0x5e8, 0x5d5, 0x5e0, 0x5d9, 0x5d9, 0x20, 0x5d3, 0x5d0, 0x5e8, 0x5d5, 0x5e1,
0x5dc, 0x5d0, 0x5dd, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5db, 0x5e3, 0x20, 0x5d5,
0x5e8, 0x5d3, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5db, 0x5e3, 0x20, 0x5d5,
0x5e8, 0x5d3, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5db, 0x5e3, 0x20, 0x5d5, 0x5e8, 0x5d3, 0x5d4, 0x20, 0x28,
0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e6, 0x5f3, 0x5d0,
0x5de, 0x5d5, 0x5e8, 0x5d5, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e6, 0x5f3, 0x5d8,
0x5d4, 0x5d0, 0x5dd, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e6, 0x5f3, 0x5d8, 0x5d4,
0x5d0, 0x5dd, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5e6, 0x5f3, 0x5d8, 0x5d4, 0x5d0, 0x5dd, 0x20, 0x28, 0x5e7, 0x5d9,
0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e6, 0x5f3, 0x5d9, 0x5dc, 0x5d4,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e6, 0x5f3, 0x5d9, 0x5dc, 0x5d4, 0x20, 0x28,
0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e6, 0x5f3,
0x5d9, 0x5dc, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5e1, 0x5d9, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e1, 0x5d9,
0x5df, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5e1, 0x5d9, 0x5df, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0, 0x5d9, 0x20, 0x5db, 0x5e8, 0x5d9, 0x5e1, 0x5d8,
0x5de, 0x5e1, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x20, 0x5e7,
0x5d5, 0x5e7, 0x5d5, 0x5e1, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d5, 0x5dc,
0x5d5, 0x5de, 0x5d1, 0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d5,
0x5dc, 0x5d5, 0x5de, 0x5d1, 0x5d9, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d5, 0x5dc, 0x5d5, 0x5de, 0x5d1,
0x5d9, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d0, 0x5d9, 0x5d9, 0x20, 0x5e7, 0x5d5, 0x5e7, 0x20, 0x28, 0x5d7, 0x5d5,
0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x20,
0x5e7, 0x5d5, 0x5e7, 0x20, 0x28, 0x5de, 0x5d7, 0x5e6, 0x5d9, 0x5ea, 0x20, 0x5d4,
0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d5, 0x5d1,
0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d5, 0x5d1, 0x5d4, 0x20, 0x28,
0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d5,
0x5d1, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d3, 0x5d9, 0x5d5, 0x5d5, 0x5d9, 0x5e1, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d3, 0x5d5, 0x5de, 0x5d5, 0x5df, 0x20, 0x5d3, 0x5f3, 0x5d0, 0x5d5, 0x5e8, 0x5d5,
0x5d5, 0x5d9, 0x5dc, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d6, 0x5e8, 0x5d7,
0x20, 0x5d8, 0x5d9, 0x5de, 0x5d5, 0x5e8, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0,
0x5d9, 0x20, 0x5d4, 0x5e4, 0x5e1, 0x5d7, 0x5d0, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d0, 0x5d9, 0x20, 0x5d4, 0x5e4, 0x5e1, 0x5d7, 0x5d0, 0x20, 0x28, 0x5d7, 0x5d5,
0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x20, 0x5d4,
0x5e4, 0x5e1, 0x5d7, 0x5d0, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d0, 0x5e7, 0x5d5, 0x5d5, 0x5d3, 0x5d5, 0x5e8, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5de, 0x5e8, 0x5db, 0x5d6, 0x20, 0x5d0, 0x5d9, 0x5e8, 0x5d5,
0x5e4, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e8, 0x5db, 0x5d6, 0x20,
0x5d0, 0x5d9, 0x5e8, 0x5d5, 0x5e4, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e8, 0x5db, 0x5d6, 0x20, 0x5d0,
0x5d9, 0x5e8, 0x5d5, 0x5e4, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d6, 0x5e8, 0x5d7, 0x20, 0x5d0, 0x5d9, 0x5e8,
0x5d5, 0x5e4, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d6, 0x5e8, 0x5d7,
0x20, 0x5d0, 0x5d9, 0x5e8, 0x5d5, 0x5e4, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8,
0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d6, 0x5e8, 0x5d7, 0x20,
0x5d0, 0x5d9, 0x5e8, 0x5d5, 0x5e4, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d9, 0x5e0, 0x5e1, 0x5e7, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d0, 0x5d9, 0x5e8, 0x5d5,
0x5e4, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20,
0x5d0, 0x5d9, 0x5e8, 0x5d5, 0x5e4, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d0,
0x5d9, 0x5e8, 0x5d5, 0x5e4, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x20, 0x5e4, 0x5d5, 0x5e7, 0x5dc,
0x5e0, 0x5d3, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x20, 0x5e4,
0x5d5, 0x5e7, 0x5dc, 0x5e0, 0x5d3, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x20, 0x5e4, 0x5d5, 0x5e7,
0x5dc, 0x5e0, 0x5d3, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5e4, 0x5d9, 0x5d2, 0x5f3, 0x5d9, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5e4, 0x5d9, 0x5d2, 0x5f3, 0x5d9, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5d9, 0x5d2, 0x5f3, 0x5d9, 0x20, 0x28,
0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d2, 0x5d9, 0x5d0,
0x5e0, 0x5d4, 0x20, 0x5d4, 0x5e6, 0x5e8, 0x5e4, 0x5ea, 0x5d9, 0x5ea, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0, 0x5e8, 0x5e6, 0x5d5, 0x5ea, 0x20, 0x5d4, 0x5d3,
0x5e8, 0x5d5, 0x5de, 0x5d9, 0x5d5, 0x5ea, 0x20, 0x5d5, 0x5d4, 0x5d0, 0x5e0, 0x5d8,
0x5d0, 0x5e8, 0x5e7, 0x5d8, 0x5d9, 0x5d5, 0x5ea, 0x20, 0x5e9, 0x5dc, 0x20, 0x5e6,
0x5e8, 0x5e4, 0x5ea, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x20,
0x5d2, 0x5dc, 0x5d0, 0x5e4, 0x5d2, 0x5d5, 0x5e1, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d0, 0x5d9, 0x5d9, 0x20, 0x5d2, 0x5de, 0x5d1, 0x5d9, 0x5d9, 0x5d4, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d2, 0x5d0, 0x5d5, 0x5e8, 0x5d2, 0x5d9, 0x5d4, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d2, 0x5d0, 0x5d5, 0x5e8, 0x5d2, 0x5d9, 0x5d4, 0x20, 0x28,
0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d2, 0x5d0,
0x5d5, 0x5e8, 0x5d2, 0x5d9, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x20, 0x5d2, 0x5d9, 0x5dc, 0x5d1,
0x5e8, 0x5d8, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d2, 0x5e8, 0x5d9, 0x5e0, 0x5d9,
0x5e5, 0x5f3, 0x200f, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d6, 0x5e8, 0x5d7,
0x20, 0x5d2, 0x5e8, 0x5d9, 0x5e0, 0x5dc, 0x5e0, 0x5d3, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5de, 0x5d6, 0x5e8, 0x5d7, 0x20, 0x5d2, 0x5e8, 0x5d9, 0x5e0, 0x5dc, 0x5e0,
0x5d3, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5de, 0x5d6, 0x5e8, 0x5d7, 0x20, 0x5d2, 0x5e8, 0x5d9, 0x5e0, 0x5dc, 0x5e0,
0x5d3, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d2, 0x5e8, 0x5d9, 0x5e0, 0x5dc, 0x5e0, 0x5d3,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d2, 0x5e8,
0x5d9, 0x5e0, 0x5dc, 0x5e0, 0x5d3, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d2, 0x5e8,
0x5d9, 0x5e0, 0x5dc, 0x5e0, 0x5d3, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d2, 0x5d5, 0x5d0, 0x5dd, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5de, 0x5d3, 0x5d9, 0x5e0, 0x5d5, 0x5ea, 0x20, 0x5d4, 0x5de, 0x5e4, 0x5e8,
0x5e5, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0, 0x5d9, 0x5d9, 0x5dd, 0x20,
0x5d4, 0x5d0, 0x5dc, 0x5d0, 0x5d5, 0x5d8, 0x5d9, 0x5d9, 0x5dd, 0x20, 0x5d4, 0x5d5,
0x5d5, 0x5d0, 0x5d9, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0, 0x5d9, 0x5d9,
0x5dd, 0x20, 0x5d4, 0x5d0, 0x5dc, 0x5d0, 0x5d5, 0x5d8, 0x5d9, 0x5d9, 0x5dd, 0x20,
0x5d4, 0x5d5, 0x5d5, 0x5d0, 0x5d9, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0, 0x5d9, 0x5d9, 0x5dd, 0x20, 0x5d4,
0x5d0, 0x5dc, 0x5d0, 0x5d5, 0x5d8, 0x5d9, 0x5d9, 0x5dd, 0x20, 0x5d4, 0x5d5, 0x5d5,
0x5d0, 0x5d9, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d4, 0x5d5, 0x5e0, 0x5d2, 0x20, 0x5e7, 0x5d5, 0x5e0, 0x5d2, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d4, 0x5d5, 0x5e0, 0x5d2, 0x20, 0x5e7, 0x5d5, 0x5e0, 0x5d2,
0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d4, 0x5d5, 0x5e0, 0x5d2, 0x20, 0x5e7, 0x5d5, 0x5e0, 0x5d2, 0x20, 0x28, 0x5e7,
0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d7, 0x5d5, 0x5d1, 0x5d3,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d7, 0x5d5, 0x5d1, 0x5d3, 0x20, 0x28, 0x5d7,
0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d7, 0x5d5, 0x5d1,
0x5d3, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d4, 0x5d5, 0x5d3, 0x5d5, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0, 0x5d5,
0x5e7, 0x5d9, 0x5d9, 0x5e0, 0x5d5, 0x5e1, 0x20, 0x5d4, 0x5d4, 0x5d5, 0x5d3, 0x5d9,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d5, 0x5d3, 0x5d5, 0x2d, 0x5e1, 0x5d9,
0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e8, 0x5db, 0x5d6, 0x20, 0x5d0,
0x5d9, 0x5e0, 0x5d3, 0x5d5, 0x5e0, 0x5d6, 0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5de, 0x5d6, 0x5e8, 0x5d7, 0x20, 0x5d0, 0x5d9, 0x5e0, 0x5d3, 0x5d5, 0x5e0,
0x5d6, 0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1,
0x20, 0x5d0, 0x5d9, 0x5e0, 0x5d3, 0x5d5, 0x5e0, 0x5d6, 0x5d9, 0x5d4, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5e8, 0x5d0, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d0, 0x5d9, 0x5e8, 0x5d0, 0x5df, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5e8, 0x5d0, 0x5df, 0x20,
0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9,
0x5e8, 0x5e7, 0x5d5, 0x5d8, 0x5e1, 0x5e7, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0,
0x5d9, 0x5e8, 0x5e7, 0x5d5, 0x5d8, 0x5e1, 0x5e7, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8,
0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5e8, 0x5e7, 0x5d5,
0x5e1, 0x5d8, 0x5e7, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d9, 0x5e9, 0x5e8, 0x5d0, 0x5dc, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d9, 0x5e9, 0x5e8, 0x5d0, 0x5dc, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d9, 0x5e9, 0x5e8, 0x5d0, 0x5dc, 0x20, 0x28,
0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d9, 0x5e4, 0x5df,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d9, 0x5e4, 0x5df, 0x20, 0x28, 0x5d7, 0x5d5,
0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d9, 0x5e4, 0x5df, 0x20,
0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5d8,
0x5e8, 0x5d5, 0x5e4, 0x5d1, 0x5dc, 0x5d5, 0x5d1, 0x5e1, 0x5e7, 0x2d, 0x5e7, 0x5de,
0x5e6, 0x5f3, 0x5d8, 0x5e1, 0x5e7, 0x5d9, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e8,
0x5d2, 0x5d9, 0x5dc, 0x20, 0x5e4, 0x5d8, 0x5e8, 0x5d5, 0x5e4, 0x5d1, 0x5dc, 0x5d5,
0x5d1, 0x5e1, 0x5e7, 0x2d, 0x5e7, 0x5de, 0x5e6, 0x5f3, 0x5d8, 0x5e1, 0x5e7, 0x5d9,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d9, 0x5e5, 0x20, 0x5e4, 0x5d8, 0x5e8,
0x5d5, 0x5e4, 0x5d1, 0x5dc, 0x5d5, 0x5d1, 0x5e1, 0x5e7, 0x2d, 0x5e7, 0x5de, 0x5e6,
0x5f3, 0x5d8, 0x5e1, 0x5e7, 0x5d9, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d6,
0x5d7, 0x5e1, 0x5d8, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d6, 0x5e8,
0x5d7, 0x20, 0x5e7, 0x5d6, 0x5d7, 0x5e1, 0x5d8, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5e7, 0x5d6, 0x5d7, 0x5e1, 0x5d8, 0x5df,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d5, 0x5e8, 0x5d9, 0x5d0, 0x5d4, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d5, 0x5e8, 0x5d9, 0x5d0, 0x5d4, 0x20, 0x28,
0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d5,
0x5e8, 0x5d9, 0x5d0, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5e7, 0x5d5, 0x5e1, 0x5e8, 0x5d0, 0x5d4, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5e7, 0x5e8, 0x5e1, 0x5e0, 0x5d5, 0x5d9, 0x5d0, 0x5e8, 0x5e1, 0x5e7,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5e8, 0x5e1, 0x5e0, 0x5d5, 0x5d9, 0x5d0,
0x5e8, 0x5e1, 0x5e7, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5e7, 0x5e8, 0x5e1, 0x5e0, 0x5d5, 0x5d9, 0x5d0, 0x5e8, 0x5e1,
0x5e7, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5e7, 0x5d9, 0x5e8, 0x5d2, 0x5d9, 0x5d6, 0x5e1, 0x5d8, 0x5df, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x20, 0x5dc, 0x5d9, 0x5d9, 0x5df, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x20, 0x5d4, 0x5dc, 0x5d5, 0x5e8, 0x5d3, 0x20,
0x5d4, 0x5d0, 0x5d5, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x20, 0x5d4,
0x5dc, 0x5d5, 0x5e8, 0x5d3, 0x20, 0x5d4, 0x5d0, 0x5d5, 0x20, 0x28, 0x5d7, 0x5d5,
0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x20, 0x5d4,
0x5dc, 0x5d5, 0x5e8, 0x5d3, 0x20, 0x5d4, 0x5d0, 0x5d5, 0x20, 0x28, 0x5e7, 0x5d9,
0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e7, 0x5d0, 0x5d5, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x20, 0x5de, 0x5e7, 0x5d0,
0x5d5, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d9, 0x5e5, 0x20, 0x5de, 0x5e7,
0x5d0, 0x5d5, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d2, 0x5d3, 0x5df, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d2, 0x5d3, 0x5df, 0x20, 0x28, 0x5d7, 0x5d5,
0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d2, 0x5d3, 0x5df,
0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de,
0x5dc, 0x5d6, 0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0, 0x5d9,
0x5d9, 0x5dd, 0x20, 0x5d4, 0x5de, 0x5dc, 0x5d3, 0x5d9, 0x5d1, 0x5d9, 0x5d9, 0x5dd,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x20, 0x5de, 0x5e8, 0x5e7,
0x5d9, 0x5d6, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x20, 0x5de,
0x5e8, 0x5e9, 0x5dc, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d0, 0x5d5, 0x5e8,
0x5d9, 0x5e6, 0x5d9, 0x5d5, 0x5e1, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d0,
0x5d5, 0x5e8, 0x5d9, 0x5e6, 0x5d9, 0x5d5, 0x5e1, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8,
0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d0, 0x5d5, 0x5e8, 0x5d9,
0x5e6, 0x5d9, 0x5d5, 0x5e1, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5de, 0x5d0, 0x5d5, 0x5e1, 0x5d5, 0x5df, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5de, 0x5e7, 0x5e1, 0x5d9, 0x5e7,
0x5d5, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5de,
0x5e7, 0x5e1, 0x5d9, 0x5e7, 0x5d5, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5de, 0x5e7,
0x5e1, 0x5d9, 0x5e7, 0x5d5, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d0, 0x5d5, 0x5dc, 0x5d0, 0x5df, 0x20, 0x5d1, 0x5d0, 0x5d8,
0x5d5, 0x5e8, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d5, 0x5dc, 0x5d0, 0x5df,
0x20, 0x5d1, 0x5d0, 0x5d8, 0x5d5, 0x5e8, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d5, 0x5dc, 0x5d0, 0x5df, 0x20,
0x5d1, 0x5d0, 0x5d8, 0x5d5, 0x5e8, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d5, 0x5e1, 0x5e7, 0x5d1, 0x5d4, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5de, 0x5d5, 0x5e1, 0x5e7, 0x5d1, 0x5d4, 0x20, 0x28, 0x5d7,
0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d5, 0x5e1,
0x5e7, 0x5d1, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5de, 0x5d9, 0x5d0, 0x5e0, 0x5de, 0x5e8, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5e0, 0x5d0, 0x5d5, 0x5e8, 0x5d5, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e0,
0x5e4, 0x5d0, 0x5dc, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5dc, 0x5d3, 0x5d5,
0x5e0, 0x5d9, 0x5d4, 0x20, 0x5d4, 0x5d7, 0x5d3, 0x5e9, 0x5d4, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5e7, 0x5dc, 0x5d3, 0x5d5, 0x5e0, 0x5d9, 0x5d4, 0x20, 0x5d4, 0x5d7,
0x5d3, 0x5e9, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5e7, 0x5dc, 0x5d3, 0x5d5, 0x5e0, 0x5d9, 0x5d4, 0x20, 0x5d4,
0x5d7, 0x5d3, 0x5e9, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5e0, 0x5d9, 0x5d5, 0x20, 0x5d6, 0x5d9, 0x5dc, 0x5e0, 0x5d3,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e0, 0x5d9, 0x5d5, 0x20, 0x5d6, 0x5d9, 0x5dc,
0x5e0, 0x5d3, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5e0, 0x5d9, 0x5d5, 0x20, 0x5d6, 0x5d9, 0x5dc, 0x5e0, 0x5d3, 0x20,
0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e0, 0x5d9,
0x5d5, 0x5e4, 0x5d0, 0x5d5, 0x5e0, 0x5d3, 0x5dc, 0x5e0, 0x5d3, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5e0, 0x5d9, 0x5d5, 0x5e4, 0x5d0, 0x5d5, 0x5e0, 0x5d3, 0x5dc, 0x5e0,
0x5d3, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5e0, 0x5d9, 0x5d5, 0x5e4, 0x5d0, 0x5d5, 0x5e0, 0x5d3, 0x5dc, 0x5e0, 0x5d3,
0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e0,
0x5d9, 0x5d5, 0x5d0, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0, 0x5d9,
0x20, 0x5e0, 0x5d5, 0x5e8, 0x5e4, 0x5d5, 0x5e7, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d4, 0x5d0, 0x5d9, 0x20, 0x5e0, 0x5d5, 0x5e8, 0x5e4, 0x5d5, 0x5e7, 0x20, 0x28,
0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0,
0x5d9, 0x20, 0x5e0, 0x5d5, 0x5e8, 0x5e4, 0x5d5, 0x5e7, 0x20, 0x28, 0x5e7, 0x5d9,
0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5e8, 0x5e0, 0x5e0, 0x5d3,
0x5d5, 0x20, 0x5d3, 0x5d9, 0x20, 0x5e0, 0x5d5, 0x5e8, 0x5d5, 0x5e0, 0x5d9, 0x5d4,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5e8, 0x5e0, 0x5e0, 0x5d3, 0x5d5, 0x20,
0x5d3, 0x5d9, 0x20, 0x5e0, 0x5d5, 0x5e8, 0x5d5, 0x5e0, 0x5d9, 0x5d4, 0x20, 0x28,
0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5e8,
0x5e0, 0x5e0, 0x5d3, 0x5d5, 0x20, 0x5d3, 0x5d9, 0x20, 0x5e0, 0x5d5, 0x5e8, 0x5d5,
0x5e0, 0x5d9, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x20, 0x5de, 0x5e8, 0x5d9, 0x5d0, 0x5e0, 0x5d4,
0x20, 0x5d4, 0x5e6, 0x5e4, 0x5d5, 0x5e0, 0x5d9, 0x5d9, 0x5dd, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5e0, 0x5d5, 0x5d1, 0x5d5, 0x5e1, 0x5d9, 0x5d1, 0x5d9, 0x5e8, 0x5e1,
0x5e7, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e0, 0x5d5, 0x5d1, 0x5d5, 0x5e1, 0x5d9,
0x5d1, 0x5d9, 0x5e8, 0x5e1, 0x5e7, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e0, 0x5d5, 0x5d1, 0x5d5, 0x5e1, 0x5d9, 0x5d1,
0x5d9, 0x5e8, 0x5e1, 0x5e7, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d0, 0x5d5, 0x5de, 0x5e1, 0x5e7, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d0, 0x5d5, 0x5de, 0x5e1, 0x5e7, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d5, 0x5de, 0x5e1, 0x5e7, 0x20,
0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5e7,
0x5d9, 0x5e1, 0x5d8, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5e7, 0x5d9,
0x5e1, 0x5d8, 0x5df, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5e4, 0x5e7, 0x5d9, 0x5e1, 0x5d8, 0x5df, 0x20, 0x28, 0x5e7,
0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5dc, 0x5d0, 0x5d5,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5e4, 0x5d5, 0x5d0, 0x5d4, 0x20, 0x5d2,
0x5d9, 0x5e0, 0x5d9, 0x5d0, 0x5d4, 0x20, 0x5d4, 0x5d7, 0x5d3, 0x5e9, 0x5d4, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5e8, 0x5d2, 0x5d5, 0x5d5, 0x5d0, 0x5d9, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5e8, 0x5d2, 0x5d5, 0x5d5, 0x5d0, 0x5d9, 0x20,
0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4,
0x5e8, 0x5d2, 0x5d5, 0x5d5, 0x5d0, 0x5d9, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5e8, 0x5d5, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5e4, 0x5e8, 0x5d5, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5e8, 0x5d5, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5e4, 0x5d9, 0x5dc, 0x5d9, 0x5e4,
0x5d9, 0x5e0, 0x5d9, 0x5dd, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5e4, 0x5d9,
0x5dc, 0x5d9, 0x5e4, 0x5d9, 0x5e0, 0x5d9, 0x5dd, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8,
0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5e4, 0x5d9, 0x5dc, 0x5d9,
0x5e4, 0x5d9, 0x5e0, 0x5d9, 0x5dd, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x20, 0x5e4, 0x5d9, 0x5e0, 0x5d9,
0x5e7, 0x5e1, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e1, 0x5e0, 0x5d8, 0x20, 0x5e4,
0x5d9, 0x5d9, 0x5e8, 0x20, 0x5d5, 0x5de, 0x5d9, 0x5e7, 0x5dc, 0x5d5, 0x5df, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5e1, 0x5e0, 0x5d8, 0x20, 0x5e4, 0x5d9, 0x5d9, 0x5e8,
0x20, 0x5d5, 0x5de, 0x5d9, 0x5e7, 0x5dc, 0x5d5, 0x5df, 0x20, 0x28, 0x5d7, 0x5d5,
0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e1, 0x5e0, 0x5d8, 0x20,
0x5e4, 0x5d9, 0x5d9, 0x5e8, 0x20, 0x5d5, 0x5de, 0x5d9, 0x5e7, 0x5dc, 0x5d5, 0x5df,
0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4,
0x5d9, 0x5d8, 0x5e7, 0x5e8, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5d5,
0x5e0, 0x5d0, 0x5e4, 0x5d9, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5d9, 0x5d5,
0x5e0, 0x5d2, 0x5d9, 0x5d0, 0x5e0, 0x5d2, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7,
0x5d9, 0x5d6, 0x5d9, 0x5dc, 0x5d5, 0x5e8, 0x5d3, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5e7, 0x5d9, 0x5d6, 0x5d9, 0x5dc, 0x5d5, 0x5e8, 0x5d3, 0x5d4, 0x20, 0x28,
0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d9,
0x5d6, 0x5d9, 0x5dc, 0x5d5, 0x5e8, 0x5d3, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e8, 0x5d0, 0x5d5, 0x5e0, 0x5d9, 0x5d5,
0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e8, 0x5d5, 0x5ea, 0x5f3, 0x5e8, 0x5d4,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e1, 0x5d7, 0x5dc, 0x5d9, 0x5df, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5e1, 0x5d7, 0x5dc, 0x5d9, 0x5df, 0x20, 0x28, 0x5d7, 0x5d5,
0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e1, 0x5d7, 0x5dc, 0x5d9,
0x5df, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5e1, 0x5de, 0x5e8, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e8, 0x5d2, 0x5d9,
0x5dc, 0x20, 0x5e1, 0x5de, 0x5e8, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7,
0x5d9, 0x5e5, 0x20, 0x5e1, 0x5de, 0x5e8, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5e1, 0x5de, 0x5d5, 0x5d0, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e1, 0x5de,
0x5d5, 0x5d0, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5e1, 0x5de, 0x5d5, 0x5d0, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9,
0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x20, 0x5e1,
0x5d9, 0x5d9, 0x5e9, 0x5dc, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e1, 0x5d9, 0x5e0,
0x5d2, 0x5e4, 0x5d5, 0x5e8, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9,
0x20, 0x5e9, 0x5dc, 0x5de, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d3, 0x5e8,
0x5d5, 0x5dd, 0x20, 0x5d2, 0x5f3, 0x5d5, 0x5e8, 0x5d2, 0x5f3, 0x5d9, 0x5d4, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5e1, 0x5d5, 0x5e8, 0x5d9, 0x5e0, 0x5d0, 0x5dd, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5e1, 0x5d9, 0x5d9, 0x5d5, 0x5d5, 0x5d4, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d8, 0x5d4, 0x5d9, 0x5d8, 0x5d9, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d8, 0x5d0, 0x5d9, 0x5e4, 0x5d9, 0x5d9, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d8, 0x5d0, 0x5d9, 0x5e4, 0x5d9, 0x5d9, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d8, 0x5d0, 0x5d9, 0x5e4, 0x5d9, 0x5d9,
0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d8,
0x5d2, 0x5f3, 0x5d9, 0x5e7, 0x5d9, 0x5e1, 0x5d8, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d8, 0x5d5, 0x5e7, 0x5dc, 0x5d0, 0x5d5, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d8, 0x5d5, 0x5e0, 0x5d2, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d8, 0x5d5,
0x5e0, 0x5d2, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d8, 0x5d5, 0x5e0, 0x5d2, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9,
0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e6, 0x5f3, 0x5d5, 0x5e7, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d8, 0x5d5, 0x5e8, 0x5e7, 0x5de, 0x5e0, 0x5d9, 0x5e1,
0x5d8, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d8, 0x5d5, 0x5e8, 0x5e7, 0x5de,
0x5e0, 0x5d9, 0x5e1, 0x5d8, 0x5df, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d8, 0x5d5, 0x5e8, 0x5e7, 0x5de, 0x5e0, 0x5d9,
0x5e1, 0x5d8, 0x5df, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d8, 0x5d5, 0x5d1, 0x5d0, 0x5dc, 0x5d5, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d0, 0x5d5, 0x5e8, 0x5d5, 0x5d2, 0x5d5, 0x5d5, 0x5d0, 0x5d9, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d0, 0x5d5, 0x5e8, 0x5d5, 0x5d2, 0x5d5, 0x5d5, 0x5d0, 0x5d9,
0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d0, 0x5d5, 0x5e8, 0x5d5, 0x5d2, 0x5d5, 0x5d5, 0x5d0, 0x5d9, 0x20, 0x28, 0x5e7,
0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d5, 0x5d6, 0x5d1,
0x5e7, 0x5d9, 0x5e1, 0x5d8, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d5,
0x5d6, 0x5d1, 0x5e7, 0x5d9, 0x5e1, 0x5d8, 0x5df, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8,
0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d5, 0x5d6, 0x5d1, 0x5e7,
0x5d9, 0x5e1, 0x5d8, 0x5df, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d5, 0x5e0, 0x5d5, 0x5d0, 0x5d8, 0x5d5, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d5, 0x5e0, 0x5d5, 0x5d0, 0x5d8, 0x5d5, 0x20, 0x28, 0x5d7, 0x5d5,
0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d5, 0x5e0, 0x5d5, 0x5d0,
0x5d8, 0x5d5, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d5, 0x5e0, 0x5e6, 0x5d5, 0x5d0, 0x5dc, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d5, 0x5dc, 0x5d3, 0x5d9, 0x5d5, 0x5d5, 0x5e1, 0x5d8, 0x5d5, 0x5e7, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d5, 0x5dc, 0x5d3, 0x5d9, 0x5d5, 0x5d5, 0x5e1, 0x5d8,
0x5d5, 0x5e7, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d5, 0x5dc, 0x5d3, 0x5d9, 0x5d5, 0x5d5, 0x5e1, 0x5d8, 0x5d5, 0x5e7,
0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d5,
0x5d5, 0x5dc, 0x5d2, 0x5d5, 0x5d2, 0x5e8, 0x5d3, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d5, 0x5d5, 0x5dc, 0x5d2, 0x5d5, 0x5d2, 0x5e8, 0x5d3, 0x20, 0x28, 0x5d7, 0x5d5,
0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d5, 0x5d5, 0x5dc, 0x5d2,
0x5d5, 0x5d2, 0x5e8, 0x5d3, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d5, 0x5d5, 0x5e1, 0x5d8, 0x5d5, 0x5e7, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d4, 0x5d0, 0x5d9, 0x20, 0x5d5, 0x5d9, 0x5d9, 0x5e7, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d5, 0x5d5, 0x5d0, 0x5dc, 0x5d9, 0x5e1, 0x20, 0x5d5, 0x5e4,
0x5d5, 0x5d8, 0x5d5, 0x5e0, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d9, 0x5e7,
0x5d5, 0x5d8, 0x5e1, 0x5e7, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d9, 0x5e7, 0x5d5,
0x5d8, 0x5e1, 0x5e7, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d9, 0x5e7, 0x5d5, 0x5d8, 0x5e1, 0x5e7, 0x20, 0x28, 0x5e7,
0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d9, 0x5e7, 0x5d8, 0x5e8,
0x5d9, 0x5e0, 0x5d1, 0x5d5, 0x5e8, 0x5d2, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d9,
0x5e7, 0x5d8, 0x5e8, 0x5d9, 0x5e0, 0x5d1, 0x5d5, 0x5e8, 0x5d2, 0x20, 0x28, 0x5d7,
0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d9, 0x5e7, 0x5d8,
0x5e8, 0x5d9, 0x5e0, 0x5d1, 0x5d5, 0x5e8, 0x5d2, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d9, 0x5d5, 0x5e7, 0x5d5, 0x5df, 0x905,
0x92b, 0x93c, 0x917, 0x93e, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x905, 0x92b, 0x93c, 0x94d,
0x930, 0x940, 0x915, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d,
0x935, 0x940, 0x20, 0x905, 0x92b, 0x93c, 0x94d, 0x930, 0x940, 0x915, 0x93e, 0x20,
0x938, 0x92e, 0x92f, 0x926, 0x915, 0x94d, 0x937, 0x93f, 0x923, 0x20, 0x905, 0x92b,
0x93c, 0x94d, 0x930, 0x940, 0x915, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x905, 0x92b,
0x93c, 0x94d, 0x930, 0x940, 0x915, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936,
0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x905, 0x92b, 0x93c, 0x94d, 0x930, 0x940, 0x915,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936,
0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x905, 0x92b, 0x93c, 0x94d, 0x930, 0x940, 0x915,
0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x932, 0x93e, 0x938, 0x94d, 0x915,
0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x932, 0x93e, 0x938, 0x94d, 0x200d, 0x915,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x932,
0x93e, 0x938, 0x94d, 0x200d, 0x915, 0x93e, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907,
0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x92e, 0x947, 0x91c, 0x93c, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x905, 0x92e, 0x947, 0x91c, 0x93c, 0x928, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x92e, 0x947, 0x91c, 0x93c, 0x928,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940,
0x928, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20,
0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x915, 0x947, 0x902, 0x926,
0x94d, 0x930, 0x940, 0x92f, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x924, 0x94d, 0x924,
0x930, 0x940, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x915,
0x947, 0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905,
0x92e, 0x947, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x915, 0x947, 0x902, 0x926, 0x94d,
0x930, 0x940, 0x92f, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938,
0x92e, 0x92f, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x947,
0x930, 0x93f, 0x915, 0x940, 0x20, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20,
0x938, 0x92e, 0x92f, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e,
0x947, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x924, 0x94d,
0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x940, 0x20,
0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907,
0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20,
0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x92e, 0x93e, 0x909, 0x902,
0x91f, 0x947, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x924, 0x94d, 0x924, 0x930,
0x940, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x92e, 0x93e,
0x909, 0x902, 0x91f, 0x947, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x947,
0x930, 0x93f, 0x915, 0x940, 0x20, 0x92e, 0x93e, 0x909, 0x902, 0x91f, 0x947, 0x928,
0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x909,
0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915,
0x940, 0x20, 0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924, 0x20, 0x938, 0x92e,
0x92f, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x947, 0x930,
0x93f, 0x915, 0x940, 0x20, 0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x924, 0x94d, 0x924,
0x930, 0x940, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x92a,
0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907,
0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x90f, 0x928, 0x93e, 0x921, 0x940, 0x92f, 0x930,
0x20, 0x938, 0x92e, 0x92f, 0x90f, 0x928, 0x93e, 0x921, 0x940, 0x92f, 0x930, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x90f, 0x928, 0x93e, 0x921,
0x940, 0x92f, 0x930, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915,
0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x90f, 0x92a, 0x93f, 0x906,
0x20, 0x938, 0x92e, 0x92f, 0x90f, 0x92a, 0x93f, 0x906, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x90f, 0x92a, 0x93f, 0x906, 0x20, 0x921, 0x947,
0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x930, 0x92c, 0x20,
0x938, 0x92e, 0x92f, 0x905, 0x930, 0x92c, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x905, 0x930, 0x92c, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907,
0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f,
0x940, 0x928, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x930, 0x94d, 0x91c, 0x947,
0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940,
0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940,
0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20,
0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905,
0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e,
0x940, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940,
0x928, 0x20, 0x938, 0x92e, 0x92f, 0x906, 0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f,
0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x906, 0x930, 0x94d, 0x92e, 0x947, 0x928,
0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x906, 0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938,
0x92e, 0x92f, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x91f, 0x932, 0x93e, 0x902,
0x91f, 0x93f, 0x915, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938,
0x92e, 0x92f, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x911, 0x938, 0x94d, 0x91f, 0x94d,
0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x938, 0x92e, 0x92f, 0x911,
0x938, 0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908,
0x20, 0x915, 0x947, 0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x911, 0x938, 0x94d, 0x200d, 0x91f, 0x94d,
0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x915, 0x947, 0x902, 0x926,
0x94d, 0x930, 0x940, 0x92f, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20,
0x938, 0x92e, 0x92f, 0x911, 0x938, 0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947, 0x932,
0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x915, 0x947, 0x902, 0x926, 0x94d, 0x930, 0x940,
0x92f, 0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x938, 0x92e,
0x92f, 0x911, 0x938, 0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f,
0x93e, 0x908, 0x20, 0x915, 0x947, 0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20,
0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x911, 0x938, 0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947,
0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x915, 0x947, 0x902, 0x926, 0x94d, 0x930,
0x940, 0x92f, 0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x921,
0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930,
0x94d, 0x935, 0x940, 0x20, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932,
0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x911, 0x938, 0x94d, 0x200d, 0x91f,
0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x92a, 0x942, 0x930,
0x94d, 0x935, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x911, 0x938, 0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e,
0x908, 0x20, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x921, 0x947, 0x932,
0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f,
0x92e, 0x940, 0x20, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f,
0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930,
0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f,
0x92e, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x911,
0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20,
0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x921, 0x947, 0x932, 0x93e,
0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x91c, 0x93c, 0x930, 0x92c, 0x948,
0x91c, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x91c, 0x93c, 0x930, 0x92c,
0x948, 0x91c, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x905, 0x91c, 0x93c, 0x930, 0x92c, 0x948, 0x91c, 0x93e, 0x928, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x905, 0x91c, 0x93c, 0x94b, 0x930, 0x947, 0x938, 0x20, 0x938,
0x92e, 0x92f, 0x905, 0x91c, 0x93c, 0x94b, 0x930, 0x947, 0x938, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x91c, 0x93c, 0x94b, 0x930, 0x947,
0x938, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92c, 0x93e, 0x902, 0x917, 0x94d, 0x932,
0x93e, 0x926, 0x947, 0x936, 0x20, 0x938, 0x92e, 0x92f, 0x92c, 0x93e, 0x902, 0x917,
0x94d, 0x932, 0x93e, 0x926, 0x947, 0x936, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x92c, 0x93e, 0x902, 0x917, 0x94d, 0x932, 0x93e, 0x926, 0x947,
0x936, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92d, 0x942, 0x91f, 0x93e, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x92c, 0x94b, 0x932, 0x940, 0x935, 0x93f, 0x92f, 0x93e, 0x20,
0x938, 0x92e, 0x92f, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x93c, 0x940, 0x932, 0x93f,
0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x93c,
0x940, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x93c, 0x940, 0x932, 0x93f, 0x92f,
0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92c, 0x94d, 0x930, 0x942, 0x928, 0x947,
0x908, 0x20, 0x926, 0x93e, 0x930, 0x942, 0x938, 0x94d, 0x938, 0x932, 0x92e, 0x20,
0x938, 0x92e, 0x92f, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x930, 0x94d, 0x921, 0x20,
0x938, 0x92e, 0x92f, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x930, 0x94d, 0x921, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x947, 0x92a, 0x20,
0x935, 0x930, 0x94d, 0x921, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x91a, 0x93e, 0x92e,
0x94b, 0x930, 0x94b, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x91a, 0x948, 0x925, 0x92e, 0x20, 0x938, 0x92e, 0x92f, 0x91a, 0x948, 0x925, 0x92e,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x91a, 0x948, 0x925,
0x92e, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f,
0x91a, 0x93f, 0x932, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x91a, 0x93f, 0x932, 0x940,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x91a, 0x93f, 0x932,
0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x91a, 0x940, 0x928, 0x20, 0x938, 0x92e,
0x92f, 0x91a, 0x940, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x91a, 0x940, 0x928, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20,
0x938, 0x92e, 0x92f, 0x915, 0x94d, 0x930, 0x93f, 0x938, 0x92e, 0x938, 0x20, 0x926,
0x94d, 0x935, 0x940, 0x92a, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x915, 0x94b,
0x938, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x938, 0x92e, 0x942, 0x939, 0x20,
0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x932, 0x902, 0x92c, 0x93f, 0x92f, 0x93e, 0x20,
0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x932, 0x902, 0x92c, 0x93f, 0x92f, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x932, 0x902,
0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x941, 0x915,
0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x938, 0x92e, 0x942, 0x939, 0x20, 0x938,
0x92e, 0x92f, 0x915, 0x941, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x938,
0x92e, 0x942, 0x939, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x915, 0x941, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x938, 0x92e, 0x942,
0x939, 0x20, 0x905, 0x930, 0x94d, 0x926, 0x94d, 0x927, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e,
0x92f, 0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x915,
0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x20, 0x921, 0x947, 0x932,
0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x921, 0x947, 0x935, 0x93f, 0x938,
0x20, 0x938, 0x92e, 0x92f, 0x921, 0x94d, 0x92f, 0x942, 0x92e, 0x94b, 0x902, 0x91f,
0x20, 0x921, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x935, 0x93f, 0x932, 0x947, 0x20,
0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x924, 0x93f,
0x92e, 0x94b, 0x930, 0x20, 0x938, 0x92e, 0x92f, 0x908, 0x938, 0x94d, 0x91f, 0x930,
0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x938, 0x92e, 0x92f, 0x908, 0x938,
0x94d, 0x91f, 0x930, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x908, 0x938, 0x94d, 0x91f, 0x930, 0x20,
0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x907, 0x915,
0x94d, 0x935, 0x93e, 0x921, 0x94b, 0x930, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x927,
0x94d, 0x92f, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x92f, 0x20, 0x938,
0x92e, 0x92f, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a,
0x940, 0x92f, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92e,
0x927, 0x94d, 0x200d, 0x92f, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x92f,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x200d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940,
0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x92f, 0x20, 0x938, 0x92e, 0x92f,
0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a,
0x940, 0x92f, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a,
0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940,
0x92f, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x917, 0x94d, 0x930, 0x20, 0x92a,
0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940,
0x92f, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940,
0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x92f, 0x20, 0x938, 0x92e, 0x92f,
0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b,
0x92a, 0x940, 0x92f, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b,
0x92a, 0x940, 0x92f, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x200d, 0x92e,
0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93c, 0x949,
0x915, 0x932, 0x948, 0x902, 0x921, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x938,
0x92e, 0x942, 0x939, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93c, 0x949, 0x915, 0x932,
0x948, 0x902, 0x921, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x938, 0x92e, 0x942,
0x939, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93c,
0x949, 0x915, 0x932, 0x948, 0x902, 0x921, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a,
0x938, 0x92e, 0x942, 0x939, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93c, 0x93f,
0x91c, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93c, 0x93f, 0x91c, 0x940, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93c, 0x93f, 0x91c,
0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93c, 0x94d, 0x930, 0x947, 0x902,
0x91a, 0x20, 0x917, 0x941, 0x92f, 0x93e, 0x928, 0x93e, 0x20, 0x938, 0x92e, 0x92f,
0x926, 0x915, 0x94d, 0x937, 0x93f, 0x923, 0x940, 0x20, 0x92b, 0x93c, 0x94d, 0x930,
0x93e, 0x902, 0x938, 0x20, 0x914, 0x930, 0x20, 0x905, 0x902, 0x91f, 0x93e, 0x930,
0x94d, 0x915, 0x91f, 0x93f, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x917, 0x948, 0x932,
0x93e, 0x92a, 0x947, 0x917, 0x94b, 0x938, 0x20, 0x915, 0x93e, 0x20, 0x938, 0x92e,
0x92f, 0x917, 0x948, 0x902, 0x92c, 0x93f, 0x92f, 0x930, 0x20, 0x938, 0x92e, 0x92f,
0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f,
0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f,
0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x917, 0x93f, 0x932, 0x94d, 0x92c, 0x930,
0x94d, 0x91f, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x938, 0x92e, 0x942, 0x939,
0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x928, 0x932, 0x948, 0x902, 0x921, 0x20, 0x938, 0x92e, 0x92f,
0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928,
0x932, 0x948, 0x902, 0x921, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x928, 0x932, 0x948, 0x902, 0x921, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936,
0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932,
0x948, 0x902, 0x921, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f,
0x92e, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x948, 0x902, 0x921,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d,
0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x948,
0x902, 0x921, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e,
0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x916, 0x93e, 0x921, 0x93c, 0x940,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x935, 0x93e,
0x908, 0x2013, 0x906, 0x932, 0x94d, 0x92f, 0x942, 0x936, 0x928, 0x20, 0x938, 0x92e,
0x92f, 0x939, 0x935, 0x93e, 0x908, 0x2013, 0x906, 0x932, 0x94d, 0x92f, 0x942, 0x936,
0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x935,
0x93e, 0x908, 0x2013, 0x906, 0x932, 0x94d, 0x92f, 0x942, 0x936, 0x928, 0x20, 0x921,
0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x93e, 0x901,
0x917, 0x20, 0x915, 0x93e, 0x901, 0x917, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x93e,
0x901, 0x917, 0x20, 0x915, 0x93e, 0x901, 0x917, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x939, 0x93e, 0x901, 0x917, 0x20, 0x915, 0x93e, 0x901,
0x917, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x94b, 0x935, 0x94d, 0x921, 0x20,
0x938, 0x92e, 0x92f, 0x939, 0x94b, 0x935, 0x94d, 0x921, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x94b, 0x935, 0x94d, 0x921, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x92d, 0x93e, 0x930, 0x924, 0x940, 0x92f, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x93f, 0x902, 0x926, 0x20, 0x92e,
0x939, 0x93e, 0x938, 0x93e, 0x917, 0x930, 0x20, 0x938, 0x92e, 0x92f, 0x907, 0x902,
0x921, 0x94b, 0x91a, 0x93e, 0x907, 0x928, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92e,
0x927, 0x94d, 0x92f, 0x20, 0x907, 0x902, 0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f,
0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940,
0x20, 0x907, 0x902, 0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20,
0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x907,
0x902, 0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e,
0x92f, 0x908, 0x930, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x908, 0x930, 0x93e,
0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x908, 0x930,
0x93e, 0x928, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e,
0x92f, 0x907, 0x930, 0x94d, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x907, 0x930, 0x94d, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d,
0x915, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x907, 0x930,
0x94d, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e,
0x92f, 0x907, 0x91c, 0x93c, 0x930, 0x93e, 0x907, 0x932, 0x20, 0x938, 0x92e, 0x92f,
0x907, 0x91c, 0x93c, 0x930, 0x93e, 0x907, 0x932, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x907, 0x91c, 0x93c, 0x930, 0x93e, 0x907, 0x932, 0x20,
0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x91c, 0x93e,
0x92a, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x91c, 0x93e, 0x92a, 0x93e, 0x928,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x91c, 0x93e, 0x92a,
0x93e, 0x928, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e,
0x92f, 0x92a, 0x947, 0x91f, 0x94d, 0x930, 0x94b, 0x92a, 0x947, 0x935, 0x932, 0x93e,
0x938, 0x94d, 0x915, 0x2d, 0x915, 0x948, 0x92e, 0x91a, 0x93e, 0x924, 0x94d, 0x938,
0x915, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x947, 0x91f, 0x94d, 0x930, 0x94b,
0x92a, 0x947, 0x935, 0x932, 0x93e, 0x938, 0x94d, 0x915, 0x2d, 0x915, 0x948, 0x92e,
0x91a, 0x93e, 0x924, 0x94d, 0x938, 0x915, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x947, 0x91f, 0x94d, 0x930, 0x94b, 0x92a, 0x947,
0x935, 0x932, 0x93e, 0x938, 0x94d, 0x915, 0x2d, 0x915, 0x948, 0x92e, 0x91a, 0x93e,
0x924, 0x94d, 0x938, 0x915, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x91c,
0x93c, 0x93e, 0x916, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f,
0x92a, 0x942, 0x930, 0x94d, 0x935, 0x20, 0x915, 0x91c, 0x93c, 0x93e, 0x916, 0x938,
0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a,
0x93f, 0x92e, 0x20, 0x915, 0x91c, 0x93c, 0x93e, 0x916, 0x938, 0x94d, 0x924, 0x93e,
0x928, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x930, 0x93f, 0x92f, 0x93e, 0x908,
0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x930, 0x93f, 0x92f, 0x93e, 0x908, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x930, 0x93f,
0x92f, 0x93e, 0x908, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938,
0x92e, 0x92f, 0x915, 0x94b, 0x938, 0x930, 0x93e, 0x90f, 0x20, 0x938, 0x92e, 0x92f,
0x915, 0x94d, 0x930, 0x93e, 0x938, 0x94d, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d,
0x938, 0x94d, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94d, 0x930, 0x93e, 0x938,
0x94d, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94d, 0x930, 0x93e, 0x938,
0x94d, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x915, 0x93f, 0x930, 0x94d, 0x917, 0x93f, 0x938, 0x94d, 0x200d,
0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x932, 0x93e, 0x907, 0x928, 0x20,
0x926, 0x94d, 0x935, 0x940, 0x92a, 0x938, 0x92e, 0x942, 0x939, 0x20, 0x938, 0x92e,
0x92f, 0x932, 0x949, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x94b, 0x935, 0x947, 0x20,
0x938, 0x92e, 0x92f, 0x932, 0x949, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x94b, 0x935,
0x947, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x932, 0x949,
0x930, 0x94d, 0x921, 0x20, 0x939, 0x94b, 0x935, 0x947, 0x20, 0x921, 0x947, 0x932,
0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x93e, 0x917, 0x93e, 0x926,
0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x93e, 0x917, 0x93e, 0x926, 0x93e,
0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x93e,
0x917, 0x93e, 0x926, 0x93e, 0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x932,
0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x93e, 0x932,
0x926, 0x940, 0x935, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x93e, 0x930, 0x94d, 0x915,
0x947, 0x938, 0x938, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x93e, 0x930, 0x94d, 0x936,
0x932, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x938, 0x92e, 0x942, 0x939, 0x20,
0x938, 0x92e, 0x92f, 0x92e, 0x949, 0x930, 0x940, 0x936, 0x938, 0x20, 0x938, 0x92e,
0x92f, 0x92e, 0x949, 0x930, 0x940, 0x936, 0x938, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x949, 0x930, 0x940, 0x936, 0x938, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x92e, 0x93e, 0x935, 0x94d, 0x938, 0x928, 0x20, 0x938, 0x92e,
0x92f, 0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x928, 0x20, 0x92a, 0x94d,
0x930, 0x936, 0x93e, 0x902, 0x924, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x947, 0x915,
0x94d, 0x938, 0x93f, 0x915, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902,
0x924, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x947,
0x915, 0x94d, 0x938, 0x93f, 0x915, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x936, 0x93e,
0x902, 0x924, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e,
0x92f, 0x909, 0x932, 0x93e, 0x928, 0x20, 0x92c, 0x91f, 0x94b, 0x930, 0x20, 0x938,
0x92e, 0x92f, 0x909, 0x932, 0x93e, 0x928, 0x20, 0x92c, 0x91f, 0x94b, 0x930, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x932, 0x93e, 0x928,
0x20, 0x92c, 0x91f, 0x94b, 0x930, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x949,
0x938, 0x94d, 0x915, 0x94b, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x949, 0x938, 0x94d,
0x915, 0x94b, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92e,
0x949, 0x938, 0x94d, 0x915, 0x94b, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x94d,
0x92f, 0x93e, 0x902, 0x92e, 0x93e, 0x930, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94c,
0x930, 0x942, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x947, 0x92a, 0x93e, 0x932, 0x20,
0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x915, 0x948, 0x932, 0x947,
0x921, 0x94b, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d,
0x92f, 0x942, 0x20, 0x915, 0x948, 0x932, 0x947, 0x921, 0x94b, 0x928, 0x93f, 0x92f,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d,
0x92f, 0x942, 0x20, 0x915, 0x948, 0x932, 0x947, 0x921, 0x94b, 0x928, 0x93f, 0x92f,
0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x91c, 0x93c,
0x940, 0x932, 0x948, 0x902, 0x921, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f,
0x942, 0x91c, 0x93c, 0x940, 0x932, 0x948, 0x902, 0x921, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x91c, 0x93c, 0x940,
0x932, 0x948, 0x902, 0x921, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20,
0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x92b, 0x93c, 0x93e, 0x909, 0x902,
0x921, 0x932, 0x948, 0x902, 0x921, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f,
0x942, 0x92b, 0x93c, 0x93e, 0x909, 0x902, 0x921, 0x932, 0x948, 0x902, 0x921, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942,
0x92b, 0x93c, 0x93e, 0x909, 0x902, 0x921, 0x932, 0x948, 0x902, 0x921, 0x20, 0x921,
0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x940, 0x92f,
0x942, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x949, 0x930, 0x92b, 0x93c, 0x949, 0x915,
0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x949,
0x930, 0x92b, 0x93c, 0x949, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x949, 0x930, 0x92b,
0x93c, 0x949, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x921, 0x947,
0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93c, 0x930, 0x94d,
0x928, 0x93e, 0x902, 0x930, 0x94d, 0x921, 0x94b, 0x20, 0x921, 0x947, 0x20, 0x928,
0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92b,
0x93c, 0x930, 0x94d, 0x928, 0x93e, 0x902, 0x930, 0x94d, 0x921, 0x94b, 0x20, 0x921,
0x947, 0x20, 0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93c, 0x930, 0x94d, 0x928,
0x93e, 0x902, 0x930, 0x94d, 0x921, 0x94b, 0x20, 0x921, 0x947, 0x20, 0x928, 0x94b,
0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x928,
0x94b, 0x935, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94b, 0x935, 0x94b, 0x938, 0x93f, 0x92c, 0x93f,
0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x928, 0x94b, 0x935, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d,
0x938, 0x94d, 0x915, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915,
0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x913, 0x92e, 0x94d, 0x938,
0x94d, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x913, 0x92e, 0x94d, 0x938, 0x94d, 0x915,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x913, 0x92e, 0x94d,
0x938, 0x94d, 0x915, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915,
0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x93e, 0x915, 0x93f,
0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x93e, 0x915,
0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940,
0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x932, 0x93e, 0x909, 0x20, 0x938, 0x92e,
0x92f, 0x92a, 0x93e, 0x92a, 0x941, 0x906, 0x20, 0x928, 0x94d, 0x92f, 0x942, 0x20,
0x917, 0x93f, 0x928, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x948, 0x930, 0x93e,
0x917, 0x94d, 0x935, 0x947, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x948, 0x930, 0x93e,
0x917, 0x94d, 0x935, 0x947, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x92a, 0x948, 0x930, 0x93e, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938,
0x92e, 0x92f, 0x92a, 0x947, 0x930, 0x942, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x947,
0x930, 0x942, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a,
0x947, 0x930, 0x942, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915,
0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93c, 0x93f, 0x932,
0x93f, 0x92a, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93c, 0x93f, 0x932,
0x93f, 0x92a, 0x940, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x92b, 0x93c, 0x93f, 0x932, 0x93f, 0x92a, 0x940, 0x928, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938,
0x92e, 0x92f, 0x92b, 0x93c, 0x940, 0x928, 0x93f, 0x915, 0x94d, 0x938, 0x20, 0x926,
0x94d, 0x935, 0x940, 0x92a, 0x938, 0x92e, 0x942, 0x939, 0x20, 0x938, 0x92e, 0x92f,
0x938, 0x947, 0x902, 0x91f, 0x20, 0x92a, 0x93f, 0x90f, 0x930, 0x947, 0x20, 0x914,
0x930, 0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932, 0x93e, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x92a, 0x93f, 0x90f, 0x930,
0x947, 0x20, 0x914, 0x930, 0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932,
0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x938,
0x947, 0x902, 0x91f, 0x20, 0x92a, 0x93f, 0x90f, 0x930, 0x947, 0x20, 0x914, 0x930,
0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932, 0x93e, 0x928, 0x20, 0x921,
0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x93f, 0x91f,
0x915, 0x948, 0x930, 0x94d, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x94b, 0x928,
0x93e, 0x92a, 0x947, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x94d, 0x92f, 0x94b, 0x902,
0x917, 0x92f, 0x93e, 0x902, 0x917, 0x20, 0x938, 0x92e, 0x92f, 0x930, 0x940, 0x92f,
0x942, 0x928, 0x93f, 0x92f, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x930, 0x94b, 0x925,
0x947, 0x930, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x916, 0x93e, 0x932, 0x93f,
0x928, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x916, 0x93e, 0x932, 0x93f, 0x928, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x916, 0x93e, 0x932,
0x93f, 0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e,
0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x92e, 0x93e, 0x930, 0x93e,
0x20, 0x938, 0x92e, 0x92f, 0x938, 0x92e, 0x93e, 0x930, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x92e, 0x93e, 0x930, 0x93e, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928,
0x20, 0x938, 0x92e, 0x92f, 0x938, 0x92e, 0x94b, 0x906, 0x20, 0x938, 0x92e, 0x92f,
0x938, 0x92e, 0x94b, 0x906, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x938, 0x92e, 0x94b, 0x906, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f,
0x20, 0x938, 0x92e, 0x92f, 0x938, 0x947, 0x936, 0x947, 0x932, 0x94d, 0x938, 0x20,
0x938, 0x92e, 0x92f, 0x938, 0x93f, 0x902, 0x917, 0x93e, 0x92a, 0x941, 0x930, 0x20,
0x938, 0x92e, 0x92f, 0x938, 0x94b, 0x932, 0x94b, 0x92e, 0x928, 0x20, 0x926, 0x94d,
0x935, 0x940, 0x92a, 0x938, 0x92e, 0x942, 0x939, 0x20, 0x938, 0x92e, 0x92f, 0x926,
0x915, 0x94d, 0x937, 0x93f, 0x923, 0x940, 0x20, 0x91c, 0x949, 0x930, 0x94d, 0x91c,
0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x942, 0x930, 0x940, 0x928,
0x93e, 0x92e, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x94d, 0x92f, 0x94b, 0x935, 0x93e,
0x20, 0x938, 0x92e, 0x92f, 0x924, 0x93e, 0x939, 0x93f, 0x924, 0x940, 0x20, 0x938,
0x92e, 0x92f, 0x924, 0x93e, 0x907, 0x92a, 0x947, 0x20, 0x938, 0x92e, 0x92f, 0x924,
0x93e, 0x907, 0x92a, 0x947, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x924, 0x93e, 0x907, 0x92a, 0x947, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907,
0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x924, 0x93e, 0x91c, 0x93f, 0x915, 0x93f, 0x938,
0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x91f, 0x94b, 0x915, 0x947,
0x932, 0x93e, 0x90a, 0x20, 0x938, 0x92e, 0x92f, 0x91f, 0x94b, 0x902, 0x917, 0x93e,
0x20, 0x938, 0x92e, 0x92f, 0x91f, 0x94b, 0x902, 0x917, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x91f, 0x94b, 0x902, 0x917, 0x93e, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928,
0x20, 0x938, 0x92e, 0x92f, 0x91a, 0x941, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x924,
0x941, 0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e,
0x928, 0x20, 0x938, 0x92e, 0x92f, 0x924, 0x941, 0x930, 0x94d, 0x915, 0x92e, 0x947,
0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x924, 0x941, 0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928,
0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x924,
0x941, 0x935, 0x93e, 0x932, 0x942, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x930, 0x941,
0x917, 0x94d, 0x935, 0x947, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x930, 0x941, 0x917,
0x94d, 0x935, 0x947, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x909, 0x930, 0x941, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f,
0x909, 0x91c, 0x93c, 0x94d, 0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e,
0x928, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x91c, 0x93c, 0x94d, 0x92c, 0x947, 0x915,
0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x909, 0x91c, 0x93c, 0x94d, 0x92c, 0x947, 0x915, 0x93f, 0x938,
0x94d, 0x924, 0x93e, 0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x935, 0x928, 0x941,
0x906, 0x924, 0x942, 0x20, 0x938, 0x92e, 0x92f, 0x935, 0x928, 0x941, 0x906, 0x924,
0x942, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x935, 0x928,
0x941, 0x906, 0x924, 0x942, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x935, 0x947, 0x928,
0x947, 0x91c, 0x93c, 0x941, 0x90f, 0x932, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x935,
0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x935, 0x94b, 0x938, 0x94d, 0x924, 0x94b, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x935, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x935, 0x94b,
0x938, 0x94d, 0x924, 0x94b, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x935, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x935, 0x94b, 0x938, 0x94d,
0x924, 0x94b, 0x915, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915,
0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x935, 0x94b, 0x932, 0x94d,
0x917, 0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x921, 0x20, 0x938, 0x92e, 0x92f, 0x935,
0x94b, 0x932, 0x94d, 0x917, 0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x921, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x935, 0x94b, 0x932, 0x94d, 0x917,
0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x921, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x935,
0x947, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x938, 0x92e, 0x92f,
0x935, 0x93e, 0x932, 0x93f, 0x938, 0x20, 0x914, 0x930, 0x20, 0x92b, 0x93c, 0x94d,
0x92f, 0x942, 0x91a, 0x942, 0x928, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92f, 0x93e,
0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x92f, 0x93e, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d,
0x915, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92f, 0x947, 0x915, 0x93e, 0x924, 0x947,
0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20, 0x938, 0x92e, 0x92f, 0x92f,
0x947, 0x915, 0x93e, 0x924, 0x947, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92f, 0x947, 0x915,
0x93e, 0x924, 0x947, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x92f, 0x941, 0x915, 0x949, 0x928, 0x20, 0x938, 0x92e, 0x92f,
0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x63, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x44,
0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x63, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x44,
0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0x20, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x61, 0x20, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69,
0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x61, 0x20, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69,
0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x61, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66,
0x69, 0x63, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69,
0x63, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x41, 0x63, 0x72, 0x65, 0x20, 0x69, 0x64, 0x151, 0x41,
0x63, 0x72, 0x65, 0x20, 0x7a, 0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151, 0x41,
0x63, 0x72, 0x65, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64,
0x151, 0x61, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x7a, 0x74, 0xe1, 0x6e,
0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0xf6, 0x7a, 0xe9, 0x70, 0x2d, 0x61,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20,
0x69, 0x64, 0x151, 0x6b, 0x65, 0x6c, 0x65, 0x74, 0x2d, 0x61, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64,
0x151, 0x64, 0xe9, 0x6c, 0x2d, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x69,
0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6e, 0x79, 0x75,
0x67, 0x61, 0x74, 0x2d, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x69, 0x20,
0x69, 0x64, 0x151, 0x7a, 0xf3, 0x6e, 0x61, 0x6e, 0x79, 0x75, 0x67, 0x61,
0x74, 0x2d, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x69, 0x20, 0x74, 0xe9,
0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6e, 0x79, 0x75, 0x67, 0x61, 0x74,
0x2d, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1,
0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61, 0x6c, 0x61, 0x73, 0x7a, 0x6b,
0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61, 0x6c, 0x61, 0x73, 0x7a, 0x6b,
0x61, 0x69, 0x20, 0x7a, 0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151, 0x61, 0x6c,
0x61, 0x73, 0x7a, 0x6b, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69,
0x20, 0x69, 0x64, 0x151, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x69, 0x20, 0x69,
0x64, 0x151, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x69, 0x20, 0x7a, 0xf3, 0x6e,
0x61, 0x69, 0x64, 0x151, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x69, 0x20, 0x6e,
0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61, 0x6d, 0x61, 0x7a,
0xf3, 0x6e, 0x69, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61, 0x6d, 0x61,
0x7a, 0xf3, 0x6e, 0x69, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20,
0x69, 0x64, 0x151, 0x61, 0x6d, 0x61, 0x7a, 0xf3, 0x6e, 0x69, 0x61, 0x69,
0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0xf6,
0x7a, 0xe9, 0x70, 0x73, 0x151, 0x20, 0xe1, 0x6c, 0x6c, 0x61, 0x6d, 0x6f,
0x6b, 0x62, 0x65, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0xf6, 0x7a,
0xe9, 0x70, 0x73, 0x151, 0x20, 0xe1, 0x6c, 0x6c, 0x61, 0x6d, 0x6f, 0x6b,
0x62, 0x65, 0x6c, 0x69, 0x20, 0x7a, 0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151,
0x6b, 0xf6, 0x7a, 0xe9, 0x70, 0x73, 0x151, 0x20, 0xe1, 0x6c, 0x6c, 0x61,
0x6d, 0x6f, 0x6b, 0x62, 0x65, 0x6c, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72,
0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x65, 0x6c, 0x65, 0x74, 0x69, 0x20,
0xe1, 0x6c, 0x6c, 0x61, 0x6d, 0x6f, 0x6b, 0x62, 0x65, 0x6c, 0x69, 0x20,
0x69, 0x64, 0x151, 0x6b, 0x65, 0x6c, 0x65, 0x74, 0x69, 0x20, 0xe1, 0x6c,
0x6c, 0x61, 0x6d, 0x6f, 0x6b, 0x62, 0x65, 0x6c, 0x69, 0x20, 0x7a, 0xf3,
0x6e, 0x61, 0x69, 0x64, 0x151, 0x6b, 0x65, 0x6c, 0x65, 0x74, 0x69, 0x20,
0xe1, 0x6c, 0x6c, 0x61, 0x6d, 0x6f, 0x6b, 0x62, 0x65, 0x6c, 0x69, 0x20,
0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x68, 0x65, 0x67,
0x79, 0x76, 0x69, 0x64, 0xe9, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x68,
0x65, 0x67, 0x79, 0x76, 0x69, 0x64, 0xe9, 0x6b, 0x69, 0x20, 0x7a, 0xf3,
0x6e, 0x61, 0x69, 0x64, 0x151, 0x68, 0x65, 0x67, 0x79, 0x76, 0x69, 0x64,
0xe9, 0x6b, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64,
0x151, 0x63, 0x73, 0x65, 0x6e, 0x64, 0x65, 0x73, 0x2d, 0xf3, 0x63, 0x65,
0xe1, 0x6e, 0x69, 0x20, 0x69, 0x64, 0x151, 0x63, 0x73, 0x65, 0x6e, 0x64,
0x65, 0x73, 0x2d, 0xf3, 0x63, 0x65, 0xe1, 0x6e, 0x69, 0x20, 0x7a, 0xf3,
0x6e, 0x61, 0x69, 0x64, 0x151, 0x63, 0x73, 0x65, 0x6e, 0x64, 0x65, 0x73,
0x2d, 0xf3, 0x63, 0x65, 0xe1, 0x6e, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72,
0x69, 0x20, 0x69, 0x64, 0x151, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x69,
0x20, 0x69, 0x64, 0x151, 0x41, 0x6e, 0x61, 0x64, 0xed, 0x72, 0x69, 0x20,
0x7a, 0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151, 0x41, 0x6e, 0x61, 0x64, 0xed,
0x72, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151,
0x61, 0x70, 0x69, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61, 0x70, 0x69,
0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61,
0x70, 0x69, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69,
0x64, 0x151, 0x41, 0x71, 0x74, 0x61, 0x75, 0x69, 0x20, 0x69, 0x64, 0x151,
0x41, 0x71, 0x74, 0x61, 0x75, 0x69, 0x20, 0x7a, 0xf3, 0x6e, 0x61, 0x69,
0x64, 0x151, 0x41, 0x71, 0x74, 0x61, 0x75, 0x69, 0x20, 0x6e, 0x79, 0xe1,
0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65,
0x69, 0x20, 0x69, 0x64, 0x151, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x69,
0x20, 0x7a, 0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151, 0x41, 0x71, 0x74, 0x6f,
0x62, 0x65, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64,
0x151, 0x61, 0x72, 0x61, 0x62, 0x20, 0x69, 0x64, 0x151, 0x61, 0x72, 0x61,
0x62, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61, 0x72,
0x61, 0x62, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151,
0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e, 0x61, 0x69, 0x20, 0x69,
0x64, 0x151, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e, 0x61, 0x69,
0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61, 0x72, 0x67,
0x65, 0x6e, 0x74, 0xed, 0x6e, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72,
0x69, 0x20, 0x69, 0x64, 0x151, 0x6e, 0x79, 0x75, 0x67, 0x61, 0x74, 0x2d,
0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e, 0x61, 0x69, 0x20, 0x69,
0x64, 0x151, 0x7a, 0xf3, 0x6e, 0x61, 0x6e, 0x79, 0x75, 0x67, 0x61, 0x74,
0x2d, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e, 0x61, 0x69, 0x20,
0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6e, 0x79, 0x75, 0x67,
0x61, 0x74, 0x2d, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e, 0x61,
0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0xf6,
0x72, 0x6d, 0xe9, 0x6e, 0x79, 0x6f, 0x72, 0x73, 0x7a, 0xe1, 0x67, 0x69,
0x20, 0x69, 0x64, 0x151, 0xf6, 0x72, 0x6d, 0xe9, 0x6e, 0x79, 0x6f, 0x72,
0x73, 0x7a, 0xe1, 0x67, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69,
0x64, 0x151, 0xf6, 0x72, 0x6d, 0xe9, 0x6e, 0x79, 0x6f, 0x72, 0x73, 0x7a,
0xe1, 0x67, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64,
0x151, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x2d, 0xf3, 0x63, 0x65,
0xe1, 0x6e, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x2d, 0xf3, 0x63, 0x65, 0xe1, 0x6e, 0x69, 0x20, 0x7a, 0xf3,
0x6e, 0x61, 0x69, 0x64, 0x151, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x2d, 0xf3, 0x63, 0x65, 0xe1, 0x6e, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72,
0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0xf6, 0x7a, 0xe9, 0x70, 0x2d, 0x61,
0x75, 0x73, 0x7a, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x69, 0x20, 0x69,
0x64, 0x151, 0x6b, 0xf6, 0x7a, 0xe9, 0x70, 0x2d, 0x61, 0x75, 0x73, 0x7a,
0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69,
0x20, 0x69, 0x64, 0x151, 0x6b, 0xf6, 0x7a, 0xe9, 0x70, 0x2d, 0x61, 0x75,
0x73, 0x7a, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x69, 0x20, 0x6e, 0x79,
0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0xf6, 0x7a, 0xe9, 0x70,
0x2d, 0x6e, 0x79, 0x75, 0x67, 0x61, 0x74, 0x2d, 0x61, 0x75, 0x73, 0x7a,
0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b,
0xf6, 0x7a, 0xe9, 0x70, 0x2d, 0x6e, 0x79, 0x75, 0x67, 0x61, 0x74, 0x2d,
0x61, 0x75, 0x73, 0x7a, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x69, 0x20,
0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0xf6, 0x7a, 0xe9,
0x70, 0x2d, 0x6e, 0x79, 0x75, 0x67, 0x61, 0x74, 0x2d, 0x61, 0x75, 0x73,
0x7a, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1,
0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x65, 0x6c, 0x65, 0x74, 0x2d,
0x61, 0x75, 0x73, 0x7a, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x69, 0x20,
0x69, 0x64, 0x151, 0x6b, 0x65, 0x6c, 0x65, 0x74, 0x2d, 0x61, 0x75, 0x73,
0x7a, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c,
0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x65, 0x6c, 0x65, 0x74, 0x2d, 0x61,
0x75, 0x73, 0x7a, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x69, 0x20, 0x6e,
0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61, 0x7a, 0x65, 0x72,
0x62, 0x61, 0x6a, 0x64, 0x7a, 0x73, 0xe1, 0x6e, 0x69, 0x20, 0x69, 0x64,
0x151, 0x61, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x7a, 0x73, 0xe1,
0x6e, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61,
0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x7a, 0x73, 0xe1, 0x6e, 0x69,
0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61, 0x7a,
0x6f, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x7a, 0xf3, 0x6e, 0x61, 0x61,
0x7a, 0x6f, 0x72, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64,
0x151, 0x61, 0x7a, 0x6f, 0x72, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69,
0x20, 0x69, 0x64, 0x151, 0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x73, 0x69, 0x20, 0x69, 0x64, 0x151, 0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x73, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64,
0x151, 0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x69, 0x20,
0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x62, 0x75, 0x74,
0xe1, 0x6e, 0x69, 0x20, 0x69, 0x64, 0x151, 0x62, 0x6f, 0x6c, 0xed, 0x76,
0x69, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151,
0x62, 0x72, 0x61, 0x7a, 0xed, 0x6c, 0x69, 0x61, 0x69, 0x20, 0x69, 0x64,
0x151, 0x62, 0x72, 0x61, 0x7a, 0xed, 0x6c, 0x69, 0x61, 0x69, 0x20, 0x74,
0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x62, 0x72, 0x61, 0x7a, 0xed,
0x6c, 0x69, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69,
0x64, 0x151, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72,
0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x2d, 0x69, 0x20, 0x69, 0x64,
0x151, 0x7a, 0xf6, 0x6c, 0x64, 0x2d, 0x66, 0x6f, 0x6b, 0x69, 0x2d, 0x73,
0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151,
0x7a, 0xf3, 0x6e, 0x61, 0x7a, 0xf6, 0x6c, 0x64, 0x2d, 0x66, 0x6f, 0x6b,
0x69, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20,
0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x7a, 0xf6, 0x6c, 0x64,
0x2d, 0x66, 0x6f, 0x6b, 0x69, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74,
0x65, 0x6b, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64,
0x151, 0x63, 0x61, 0x73, 0x65, 0x79, 0x2d, 0x69, 0x20, 0x69, 0x64, 0x151,
0x63, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0xf3, 0x69, 0x20, 0x74, 0xe9,
0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x63, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x69, 0x20, 0x69, 0x64, 0x151, 0x63, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x63,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72,
0x69, 0x20, 0x69, 0x64, 0x151, 0x63, 0x68, 0x69, 0x6c, 0x65, 0x69, 0x20,
0x69, 0x64, 0x151, 0x7a, 0xf3, 0x6e, 0x61, 0x63, 0x68, 0x69, 0x6c, 0x65,
0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x63, 0x68,
0x69, 0x6c, 0x65, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69,
0x64, 0x151, 0x6b, 0xed, 0x6e, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b,
0xed, 0x6e, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64,
0x151, 0x6b, 0xed, 0x6e, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69,
0x20, 0x69, 0x64, 0x151, 0x6b, 0x61, 0x72, 0xe1, 0x63, 0x73, 0x6f, 0x6e,
0x79, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x69, 0x20, 0x69, 0x64,
0x151, 0x6b, 0xf3, 0x6b, 0x75, 0x73, 0x7a, 0x2d, 0x73, 0x7a, 0x69, 0x67,
0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69,
0x64, 0x151, 0x6b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61, 0x69, 0x20,
0x69, 0x64, 0x151, 0x6b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61, 0x69,
0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x6f, 0x6c,
0x75, 0x6d, 0x62, 0x69, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69,
0x20, 0x69, 0x64, 0x151, 0x63, 0x6f, 0x6f, 0x6b, 0x2d, 0x73, 0x7a, 0x69,
0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x63, 0x6f,
0x6f, 0x6b, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69,
0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x63, 0x6f, 0x6f,
0x6b, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20,
0x66, 0xe9, 0x6c, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64,
0x151, 0x6b, 0x75, 0x62, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x7a, 0xf3,
0x6e, 0x61, 0x6b, 0x75, 0x62, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69,
0x20, 0x69, 0x64, 0x151, 0x6b, 0x75, 0x62, 0x61, 0x69, 0x20, 0x6e, 0x79,
0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x64, 0x61, 0x76, 0x69, 0x73,
0x69, 0x20, 0x69, 0x64, 0x151, 0x64, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d,
0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x2d, 0x69, 0x20,
0x69, 0x64, 0x151, 0x6b, 0x65, 0x6c, 0x65, 0x74, 0x2d, 0x74, 0x69, 0x6d,
0x6f, 0x72, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151,
0x68, 0xfa, 0x73, 0x76, 0xe9, 0x74, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65,
0x74, 0x69, 0x20, 0x69, 0x64, 0x151, 0x7a, 0xf3, 0x6e, 0x61, 0x68, 0xfa,
0x73, 0x76, 0xe9, 0x74, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x69,
0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x68, 0xfa, 0x73,
0x76, 0xe9, 0x74, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x69, 0x20,
0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x65, 0x63, 0x75,
0x61, 0x64, 0x6f, 0x72, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69,
0x64, 0x151, 0x6b, 0xf6, 0x7a, 0xe9, 0x70, 0x2d, 0x65, 0x75, 0x72, 0xf3,
0x70, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x7a, 0xf3, 0x6e, 0x61, 0x6b,
0xf6, 0x7a, 0xe9, 0x70, 0x2d, 0x65, 0x75, 0x72, 0xf3, 0x70, 0x61, 0x69,
0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0xf6, 0x7a,
0xe9, 0x70, 0x2d, 0x65, 0x75, 0x72, 0xf3, 0x70, 0x61, 0x69, 0x20, 0x6e,
0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x65, 0x6c, 0x65,
0x74, 0x2d, 0x65, 0x75, 0x72, 0xf3, 0x70, 0x61, 0x69, 0x20, 0x69, 0x64,
0x151, 0x7a, 0xf3, 0x6e, 0x61, 0x6b, 0x65, 0x6c, 0x65, 0x74, 0x2d, 0x65,
0x75, 0x72, 0xf3, 0x70, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20,
0x69, 0x64, 0x151, 0x6b, 0x65, 0x6c, 0x65, 0x74, 0x2d, 0x65, 0x75, 0x72,
0xf3, 0x70, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69,
0x64, 0x151, 0x6d, 0x69, 0x6e, 0x73, 0x7a, 0x6b, 0x69, 0x20, 0x69, 0x64,
0x151, 0x6e, 0x79, 0x75, 0x67, 0x61, 0x74, 0x2d, 0x65, 0x75, 0x72, 0xf3,
0x70, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x7a, 0xf3, 0x6e, 0x61, 0x6e,
0x79, 0x75, 0x67, 0x61, 0x74, 0x2d, 0x65, 0x75, 0x72, 0xf3, 0x70, 0x61,
0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6e, 0x79,
0x75, 0x67, 0x61, 0x74, 0x2d, 0x65, 0x75, 0x72, 0xf3, 0x70, 0x61, 0x69,
0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x66, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65,
0x74, 0x65, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x66, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65,
0x6b, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x66,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x73, 0x7a, 0x69, 0x67,
0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20,
0x69, 0x64, 0x151, 0x66, 0x69, 0x64, 0x7a, 0x73, 0x69, 0x20, 0x69, 0x64,
0x151, 0x66, 0x69, 0x64, 0x7a, 0x73, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69,
0x20, 0x69, 0x64, 0x151, 0x66, 0x69, 0x64, 0x7a, 0x73, 0x69, 0x20, 0x6e,
0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x66, 0x72, 0x61, 0x6e,
0x63, 0x69, 0x61, 0x2d, 0x67, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x69, 0x20,
0x69, 0x64, 0x151, 0x66, 0x72, 0x61, 0x6e, 0x63, 0x69, 0x61, 0x20, 0x64,
0xe9, 0x6c, 0x69, 0x20, 0xe9, 0x73, 0x20, 0x61, 0x6e, 0x74, 0x61, 0x72,
0x6b, 0x74, 0x69, 0x73, 0x7a, 0x69, 0x20, 0x69, 0x64, 0x151, 0x67, 0x61,
0x6c, 0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x69, 0x20, 0x74, 0xe9, 0x6c,
0x69, 0x20, 0x69, 0x64, 0x151, 0x67, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72,
0x69, 0x20, 0x69, 0x64, 0x151, 0x67, 0x72, 0xfa, 0x7a, 0x69, 0x61, 0x69,
0x20, 0x69, 0x64, 0x151, 0x67, 0x72, 0xfa, 0x7a, 0x69, 0x61, 0x69, 0x20,
0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x67, 0x72, 0xfa, 0x7a,
0x69, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64,
0x151, 0x67, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x2d, 0x73, 0x7a, 0x69,
0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x67, 0x72,
0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x69, 0x20, 0x6b, 0xf6, 0x7a,
0xe9, 0x70, 0x69, 0x64, 0x151, 0x2c, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20,
0x69, 0x64, 0x151, 0x6b, 0x65, 0x6c, 0x65, 0x74, 0x2d, 0x67, 0x72, 0xf6,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x69, 0x64, 0x151, 0x7a, 0xf3,
0x6e, 0x61, 0x6b, 0x65, 0x6c, 0x65, 0x74, 0x2d, 0x67, 0x72, 0xf6, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69,
0x64, 0x151, 0x6b, 0x65, 0x6c, 0x65, 0x74, 0x2d, 0x67, 0x72, 0xf6, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20,
0x69, 0x64, 0x151, 0x6e, 0x79, 0x75, 0x67, 0x61, 0x74, 0x2d, 0x67, 0x72,
0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x69, 0x64, 0x151, 0x7a,
0xf3, 0x6e, 0x61, 0x6e, 0x79, 0x75, 0x67, 0x61, 0x74, 0x2d, 0x67, 0x72,
0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69,
0x20, 0x69, 0x64, 0x151, 0x6e, 0x79, 0x75, 0x67, 0x61, 0x74, 0x2d, 0x67,
0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x6e, 0x79, 0xe1,
0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x47, 0x75, 0x61, 0x6d, 0x69, 0x20,
0x7a, 0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151, 0xf6, 0x62, 0xf6, 0x6c, 0x62,
0x65, 0x6c, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151,
0x67, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69,
0x20, 0x69, 0x64, 0x151, 0x68, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x61,
0x6c, 0x65, 0x75, 0x74, 0x69, 0x20, 0x69, 0x64, 0x151, 0x7a, 0xf3, 0x6e,
0x61, 0x68, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x61, 0x6c, 0x65, 0x75,
0x74, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x68,
0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x69,
0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x68, 0x6f,
0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x69, 0x20, 0x69, 0x64, 0x151, 0x7a,
0xf3, 0x6e, 0x61, 0x68, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x69,
0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x68, 0x6f, 0x6e,
0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69,
0x20, 0x69, 0x64, 0x151, 0x68, 0x6f, 0x76, 0x64, 0x69, 0x20, 0x69, 0x64,
0x151, 0x68, 0x6f, 0x76, 0x64, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20,
0x69, 0x64, 0x151, 0x68, 0x6f, 0x76, 0x64, 0x69, 0x20, 0x6e, 0x79, 0xe1,
0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x69, 0x6e, 0x64, 0x69, 0x61, 0x69,
0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x69, 0x6e, 0x64,
0x69, 0x61, 0x69, 0x2d, 0xf3, 0x63, 0x65, 0xe1, 0x6e, 0x69, 0x20, 0x69,
0x64, 0x151, 0x69, 0x6e, 0x64, 0x6f, 0x6b, 0xed, 0x6e, 0x61, 0x69, 0x20,
0x69, 0x64, 0x151, 0x6b, 0xf6, 0x7a, 0xe9, 0x70, 0x2d, 0x69, 0x6e, 0x64,
0x6f, 0x6e, 0xe9, 0x7a, 0x69, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b,
0x65, 0x6c, 0x65, 0x74, 0x2d, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0xe9, 0x7a,
0x69, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6e, 0x79, 0x75, 0x67, 0x61,
0x74, 0x2d, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0xe9, 0x7a, 0x69, 0x61, 0x69,
0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x69, 0x72, 0xe1,
0x6e, 0x69, 0x20, 0x69, 0x64, 0x151, 0x69, 0x72, 0xe1, 0x6e, 0x69, 0x20,
0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x69, 0x72, 0xe1, 0x6e,
0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x69,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x7a, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151,
0x69, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x7a, 0x6b, 0x69, 0x20, 0x74, 0xe9,
0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x69, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x7a, 0x6b, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64,
0x151, 0x69, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151,
0x69, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69,
0x20, 0x69, 0x64, 0x151, 0x69, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x20,
0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6a, 0x61, 0x70,
0xe1, 0x6e, 0x20, 0x69, 0x64, 0x151, 0x6a, 0x61, 0x70, 0xe1, 0x6e, 0x20,
0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6a, 0x61, 0x70, 0xe1,
0x6e, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x50,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x7a,
0x6b, 0x2d, 0x6b, 0x61, 0x6d, 0x63, 0x73, 0x61, 0x74, 0x6b, 0x61, 0x69,
0x20, 0x69, 0x64, 0x151, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76,
0x6c, 0x6f, 0x76, 0x73, 0x7a, 0x6b, 0x2d, 0x6b, 0x61, 0x6d, 0x63, 0x73,
0x61, 0x74, 0x6b, 0x61, 0x69, 0x20, 0x7a, 0xf3, 0x6e, 0x61, 0x69, 0x64,
0x151, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76,
0x73, 0x7a, 0x6b, 0x2d, 0x6b, 0x61, 0x6d, 0x63, 0x73, 0x61, 0x74, 0x6b,
0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151,
0x6b, 0x61, 0x7a, 0x61, 0x68, 0x73, 0x7a, 0x74, 0xe1, 0x6e, 0x69, 0x20,
0x69, 0x64, 0x151, 0x6b, 0x65, 0x6c, 0x65, 0x74, 0x2d, 0x6b, 0x61, 0x7a,
0x61, 0x68, 0x73, 0x7a, 0x74, 0xe1, 0x6e, 0x69, 0x20, 0x69, 0x64, 0x151,
0x6e, 0x79, 0x75, 0x67, 0x61, 0x74, 0x2d, 0x6b, 0x61, 0x7a, 0x61, 0x68,
0x73, 0x7a, 0x74, 0xe1, 0x6e, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x6f,
0x72, 0x65, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x6f, 0x72, 0x65,
0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b,
0x6f, 0x72, 0x65, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20,
0x69, 0x64, 0x151, 0x6b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x69, 0x20, 0x69,
0x64, 0x151, 0x6b, 0x72, 0x61, 0x73, 0x7a, 0x6e, 0x6f, 0x6a, 0x61, 0x72,
0x73, 0x7a, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x72, 0x61, 0x73,
0x7a, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x7a, 0x6b, 0x69, 0x20, 0x74,
0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x72, 0x61, 0x73, 0x7a,
0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x7a, 0x6b, 0x69, 0x20, 0x6e, 0x79,
0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x69, 0x72, 0x67, 0x69,
0x7a, 0x69, 0x73, 0x7a, 0x74, 0xe1, 0x6e, 0x69, 0x20, 0x69, 0x64, 0x151,
0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x73, 0x6f,
0x72, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20,
0x69, 0x64, 0x151, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x69, 0x20, 0x69, 0x64, 0x151,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0x73, 0x7a,
0x69, 0x67, 0x65, 0x74, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69,
0x64, 0x151, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d,
0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72,
0x69, 0x20, 0x69, 0x64, 0x151, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x69, 0x20,
0x69, 0x64, 0x151, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x69, 0x20, 0x7a, 0xf3,
0x6e, 0x61, 0x69, 0x64, 0x151, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x69, 0x20,
0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6d, 0x61, 0x67,
0x61, 0x64, 0xe1, 0x6e, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69,
0x64, 0x151, 0x6d, 0x61, 0x67, 0x61, 0x64, 0xe1, 0x6e, 0x69, 0x20, 0x6e,
0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6d, 0x61, 0x6c, 0x61,
0x6a, 0x7a, 0x69, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6d, 0x61, 0x6c,
0x64, 0xed, 0x76, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b,
0x69, 0x20, 0x69, 0x64, 0x151, 0x6d, 0x61, 0x72, 0x71, 0x75, 0x69, 0x73,
0x65, 0x73, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69,
0x20, 0x69, 0x64, 0x151, 0x6d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c,
0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x69,
0x64, 0x151, 0x6d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x69,
0x20, 0x69, 0x64, 0x151, 0x7a, 0xf3, 0x6e, 0x61, 0x6d, 0x61, 0x75, 0x72,
0x69, 0x74, 0x69, 0x75, 0x73, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20,
0x69, 0x64, 0x151, 0x6d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73,
0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6d,
0x61, 0x77, 0x73, 0x6f, 0x6e, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6d, 0x65,
0x78, 0x69, 0x6b, 0xf3, 0x69, 0x20, 0x63, 0x73, 0x65, 0x6e, 0x64, 0x65,
0x73, 0x2d, 0xf3, 0x63, 0x65, 0xe1, 0x6e, 0x69, 0x20, 0x69, 0x64, 0x151,
0x6d, 0x65, 0x78, 0x69, 0x6b, 0xf3, 0x69, 0x20, 0x63, 0x73, 0x65, 0x6e,
0x64, 0x65, 0x73, 0x2d, 0xf3, 0x63, 0x65, 0xe1, 0x6e, 0x69, 0x20, 0x7a,
0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151, 0x6d, 0x65, 0x78, 0x69, 0x6b, 0xf3,
0x69, 0x20, 0x63, 0x73, 0x65, 0x6e, 0x64, 0x65, 0x73, 0x2d, 0xf3, 0x63,
0x65, 0xe1, 0x6e, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69,
0x64, 0x151, 0x75, 0x6c, 0xe1, 0x6e, 0x62, 0xe1, 0x74, 0x6f, 0x72, 0x69,
0x20, 0x69, 0x64, 0x151, 0x75, 0x6c, 0xe1, 0x6e, 0x62, 0xe1, 0x74, 0x6f,
0x72, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x75,
0x6c, 0xe1, 0x6e, 0x62, 0xe1, 0x74, 0x6f, 0x72, 0x69, 0x20, 0x6e, 0x79,
0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6d, 0x6f, 0x73, 0x7a, 0x6b,
0x76, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6d, 0x6f, 0x73, 0x7a, 0x6b,
0x76, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151,
0x6d, 0x6f, 0x73, 0x7a, 0x6b, 0x76, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1,
0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6d, 0x69, 0x61, 0x6e, 0x6d, 0x61,
0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6e, 0x61, 0x75, 0x72, 0x75, 0x69,
0x20, 0x69, 0x64, 0x151, 0x6e, 0x65, 0x70, 0xe1, 0x6c, 0x69, 0x20, 0x69,
0x64, 0x151, 0xfa, 0x6a, 0x2d, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0xf3, 0x6e,
0x69, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0xfa, 0x6a, 0x2d, 0x6b, 0x61,
0x6c, 0x65, 0x64, 0xf3, 0x6e, 0x69, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c,
0x69, 0x20, 0x69, 0x64, 0x151, 0xfa, 0x6a, 0x2d, 0x6b, 0x61, 0x6c, 0x65,
0x64, 0xf3, 0x6e, 0x69, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69,
0x20, 0x69, 0x64, 0x151, 0xfa, 0x6a, 0x2d, 0x7a, 0xe9, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x20, 0x69, 0x64, 0x151, 0xfa, 0x6a, 0x2d, 0x7a, 0xe9, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64,
0x151, 0xfa, 0x6a, 0x2d, 0x7a, 0xe9, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20,
0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0xfa, 0x6a, 0x2d,
0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x69, 0x64,
0x151, 0xfa, 0x6a, 0x2d, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x20, 0x7a, 0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151, 0xfa, 0x6a, 0x2d,
0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x6e, 0x79,
0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6e, 0x69, 0x75, 0x65, 0x69,
0x20, 0x69, 0x64, 0x151, 0x6e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x2d,
0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x69, 0x64,
0x151, 0x6e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x2d, 0x73, 0x7a, 0x69,
0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20,
0x69, 0x64, 0x151, 0x6e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x2d, 0x73,
0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x6e, 0x79, 0xe1,
0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x2d, 0x69, 0x20, 0x69, 0x64, 0x151, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x2d, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64,
0x151, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x69, 0x20, 0x6e,
0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0xc9, 0x73, 0x7a, 0x61,
0x6b, 0x2d, 0x6d, 0x61, 0x72, 0x69, 0x61, 0x6e, 0x61, 0x2d, 0x73, 0x7a,
0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6e,
0x6f, 0x76, 0x6f, 0x73, 0x7a, 0x69, 0x62, 0x69, 0x72, 0x73, 0x7a, 0x6b,
0x69, 0x20, 0x69, 0x64, 0x151, 0x6e, 0x6f, 0x76, 0x6f, 0x73, 0x7a, 0x69,
0x62, 0x69, 0x72, 0x73, 0x7a, 0x6b, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69,
0x20, 0x69, 0x64, 0x151, 0x6e, 0x6f, 0x76, 0x6f, 0x73, 0x7a, 0x69, 0x62,
0x69, 0x72, 0x73, 0x7a, 0x6b, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69,
0x20, 0x69, 0x64, 0x151, 0x6f, 0x6d, 0x73, 0x7a, 0x6b, 0x69, 0x20, 0x69,
0x64, 0x151, 0x6f, 0x6d, 0x73, 0x7a, 0x6b, 0x69, 0x20, 0x74, 0xe9, 0x6c,
0x69, 0x20, 0x69, 0x64, 0x151, 0x6f, 0x6d, 0x73, 0x7a, 0x6b, 0x69, 0x20,
0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x70, 0x61, 0x6b,
0x69, 0x73, 0x7a, 0x74, 0xe1, 0x6e, 0x69, 0x20, 0x69, 0x64, 0x151, 0x70,
0x61, 0x6b, 0x69, 0x73, 0x7a, 0x74, 0xe1, 0x6e, 0x69, 0x20, 0x74, 0xe9,
0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x70, 0x61, 0x6b, 0x69, 0x73, 0x7a,
0x74, 0xe1, 0x6e, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69,
0x64, 0x151, 0x70, 0x61, 0x6c, 0x61, 0x75, 0x69, 0x20, 0x69, 0x64, 0x151,
0x70, 0xe1, 0x70, 0x75, 0x61, 0x20, 0xfa, 0x6a, 0x2d, 0x67, 0x75, 0x69,
0x6e, 0x65, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x70, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x79, 0x69, 0x20, 0x69, 0x64, 0x151, 0x70, 0x61, 0x72,
0x61, 0x67, 0x75, 0x61, 0x79, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20,
0x69, 0x64, 0x151, 0x70, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x69,
0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x70, 0x65,
0x72, 0x75, 0x69, 0x20, 0x69, 0x64, 0x151, 0x70, 0x65, 0x72, 0x75, 0x69,
0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x70, 0x65, 0x72,
0x75, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151,
0x66, 0xfc, 0x6c, 0xf6, 0x70, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74,
0x65, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x66, 0xfc, 0x6c, 0xf6, 0x70,
0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x74,
0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x66, 0xfc, 0x6c, 0xf6, 0x70,
0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x6e,
0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x70, 0x68, 0x6f, 0x65,
0x6e, 0x69, 0x78, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b,
0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x53, 0x61,
0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0xe9,
0x73, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x69,
0x20, 0x69, 0x64, 0x151, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x20, 0xe9, 0x73, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x69, 0x20, 0x7a, 0xf3, 0x6e, 0x61, 0x69,
0x64, 0x151, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x20, 0xe9, 0x73, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x2d, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69,
0x64, 0x151, 0x70, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x2d, 0x73,
0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151,
0x70, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65,
0x74, 0x69, 0x20, 0x69, 0x64, 0x151, 0x70, 0x68, 0x65, 0x6e, 0x6a, 0x61,
0x6e, 0x69, 0x20, 0x69, 0x64, 0x151, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f,
0x72, 0x64, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x51, 0x79, 0x7a, 0x79,
0x6c, 0x6f, 0x72, 0x64, 0x61, 0x69, 0x20, 0x7a, 0xf3, 0x6e, 0x61, 0x69,
0x64, 0x151, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x69,
0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x72, 0xe9,
0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x69, 0x20, 0x69, 0x64, 0x151, 0x72, 0x6f,
0x74, 0x68, 0x65, 0x72, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x73, 0x7a,
0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x20, 0x69, 0x64, 0x151, 0x73,
0x7a, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x20, 0x74, 0xe9, 0x6c,
0x69, 0x20, 0x69, 0x64, 0x151, 0x73, 0x7a, 0x61, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151,
0x53, 0x7a, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151,
0x53, 0x7a, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x69, 0x20, 0x7a, 0xf3, 0x6e,
0x61, 0x69, 0x64, 0x151, 0x53, 0x7a, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x69,
0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x73, 0x7a,
0x61, 0x6d, 0x6f, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x73, 0x7a, 0x61,
0x6d, 0x6f, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64,
0x151, 0x73, 0x7a, 0x61, 0x6d, 0x6f, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1,
0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x73, 0x65, 0x79, 0x63, 0x68, 0x65,
0x6c, 0x6c, 0x65, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b,
0x69, 0x20, 0x69, 0x64, 0x151, 0x73, 0x7a, 0x69, 0x6e, 0x67, 0x61, 0x70,
0xfa, 0x72, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151,
0x73, 0x61, 0x6c, 0x61, 0x6d, 0x6f, 0x6e, 0x2d, 0x73, 0x7a, 0x69, 0x67,
0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x64, 0xe9, 0x6c,
0x69, 0x2d, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x69, 0x20, 0x69,
0x64, 0x151, 0x73, 0x7a, 0x75, 0x72, 0x69, 0x6e, 0xe1, 0x6d, 0x69, 0x20,
0x69, 0x64, 0x151, 0x73, 0x79, 0x6f, 0x77, 0x61, 0x69, 0x20, 0x69, 0x64,
0x151, 0x74, 0x61, 0x68, 0x69, 0x74, 0x69, 0x20, 0x69, 0x64, 0x151, 0x74,
0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x69, 0x64, 0x151, 0x74, 0x61, 0x69,
0x70, 0x65, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151,
0x74, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69,
0x20, 0x69, 0x64, 0x151, 0x74, 0xe1, 0x64, 0x7a, 0x73, 0x69, 0x6b, 0x69,
0x73, 0x7a, 0x74, 0xe1, 0x6e, 0x69, 0x20, 0x69, 0x64, 0x151, 0x74, 0x6f,
0x6b, 0x65, 0x6c, 0x61, 0x75, 0x69, 0x20, 0x69, 0x64, 0x151, 0x74, 0x6f,
0x6e, 0x67, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x74, 0x6f, 0x6e, 0x67,
0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x74,
0x6f, 0x6e, 0x67, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20,
0x69, 0x64, 0x151, 0x74, 0x72, 0x75, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151,
0x74, 0xfc, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x7a, 0x74, 0xe1,
0x6e, 0x69, 0x20, 0x69, 0x64, 0x151, 0x74, 0xfc, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x7a, 0x74, 0xe1, 0x6e, 0x69, 0x20, 0x74, 0xe9, 0x6c,
0x69, 0x20, 0x69, 0x64, 0x151, 0x74, 0xfc, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x7a, 0x74, 0xe1, 0x6e, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72,
0x69, 0x20, 0x69, 0x64, 0x151, 0x74, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x69,
0x20, 0x69, 0x64, 0x151, 0x75, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x69,
0x20, 0x69, 0x64, 0x151, 0x75, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x69,
0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x75, 0x72, 0x75,
0x67, 0x75, 0x61, 0x79, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20,
0x69, 0x64, 0x151, 0xfc, 0x7a, 0x62, 0x65, 0x67, 0x69, 0x73, 0x7a, 0x74,
0xe1, 0x6e, 0x69, 0x20, 0x69, 0x64, 0x151, 0xfc, 0x7a, 0x62, 0x65, 0x67,
0x69, 0x73, 0x7a, 0x74, 0xe1, 0x6e, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69,
0x20, 0x69, 0x64, 0x151, 0xfc, 0x7a, 0x62, 0x65, 0x67, 0x69, 0x73, 0x7a,
0x74, 0xe1, 0x6e, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69,
0x64, 0x151, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x69, 0x20, 0x69,
0x64, 0x151, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x69, 0x20, 0x74,
0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x76, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64,
0x151, 0x76, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x69, 0x20,
0x69, 0x64, 0x151, 0x76, 0x6c, 0x61, 0x67, 0x79, 0x69, 0x76, 0x6f, 0x73,
0x7a, 0x74, 0x6f, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x76, 0x6c, 0x61,
0x67, 0x79, 0x69, 0x76, 0x6f, 0x73, 0x7a, 0x74, 0x6f, 0x6b, 0x69, 0x20,
0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x76, 0x6c, 0x61, 0x67,
0x79, 0x69, 0x76, 0x6f, 0x73, 0x7a, 0x74, 0x6f, 0x6b, 0x69, 0x20, 0x6e,
0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x76, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0xe1, 0x64, 0x69, 0x20, 0x69, 0x64, 0x151, 0x76, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0xe1, 0x64, 0x69, 0x20, 0x74, 0xe9, 0x6c,
0x69, 0x20, 0x69, 0x64, 0x151, 0x76, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0xe1, 0x64, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64,
0x151, 0x77, 0x61, 0x6b, 0x65, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74,
0x69, 0x20, 0x69, 0x64, 0x151, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20,
0xe9, 0x73, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x2d, 0x69, 0x20,
0x69, 0x64, 0x151, 0x6a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x7a, 0x6b, 0x69,
0x20, 0x69, 0x64, 0x151, 0x6a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x7a, 0x6b,
0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6a, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x7a, 0x6b, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72,
0x69, 0x20, 0x69, 0x64, 0x151, 0x6a, 0x65, 0x6b, 0x61, 0x74, 0x79, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x69, 0x20, 0x69, 0x64, 0x151,
0x6a, 0x65, 0x6b, 0x61, 0x74, 0x79, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151,
0x6a, 0x65, 0x6b, 0x61, 0x74, 0x79, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64,
0x151, 0x79, 0x75, 0x6b, 0x6f, 0x6e, 0x69, 0x20, 0x69, 0x64, 0x151, 0x41,
0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x74, 0xed, 0x6d,
0x69, 0x4d, 0x69, 0xf0, 0x2d, 0x41, 0x66, 0x72, 0xed, 0x6b, 0x75, 0x74,
0xed, 0x6d, 0x69, 0x41, 0x75, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x41, 0x66,
0x72, 0xed, 0x6b, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x75, 0xf0, 0x75,
0x72, 0x2d, 0x41, 0x66, 0x72, 0xed, 0x6b, 0x75, 0x74, 0xed, 0x6d, 0x69,
0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x41, 0x66, 0x72, 0xed, 0x6b,
0x75, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72,
0x2d, 0x41, 0x66, 0x72, 0xed, 0x6b, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x56, 0x65, 0x73, 0x74, 0x75,
0x72, 0x2d, 0x41, 0x66, 0x72, 0xed, 0x6b, 0x75, 0x54, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x53, 0x74, 0x61,
0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xed, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x41, 0x6d,
0x61, 0x73, 0xf3, 0x6e, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0,
0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x41, 0x6d, 0x61,
0x73, 0xf3, 0x6e, 0x73, 0x76, 0xe6, 0xf0, 0x69, 0x6e, 0x75, 0x53, 0x75,
0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x41, 0x6d,
0x61, 0x73, 0xf3, 0x6e, 0x73, 0x76, 0xe6, 0xf0, 0x69, 0x6e, 0x75, 0x54,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x6d, 0x69, 0xf0, 0x68, 0x6c, 0x75,
0x74, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x61, 0x72, 0xed, 0x6b, 0x6a,
0x61, 0x6e, 0x6e, 0x61, 0x20, 0x6f, 0x67, 0x20, 0x4b, 0x61, 0x6e, 0x61,
0x64, 0x61, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x6d, 0x69, 0xf0, 0x68, 0x6c, 0x75, 0x74, 0x61, 0x20,
0x42, 0x61, 0x6e, 0x64, 0x61, 0x72, 0xed, 0x6b, 0x6a, 0x61, 0x6e, 0x6e,
0x61, 0x20, 0x6f, 0x67, 0x20, 0x4b, 0x61, 0x6e, 0x61, 0x64, 0x61, 0x53,
0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x6d,
0x69, 0xf0, 0x68, 0x6c, 0x75, 0x74, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x64,
0x61, 0x72, 0xed, 0x6b, 0x6a, 0x61, 0x6e, 0x6e, 0x61, 0x20, 0x6f, 0x67,
0x20, 0x4b, 0x61, 0x6e, 0x61, 0x64, 0x61, 0x54, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0x61, 0x75, 0x73, 0x74, 0x75, 0x72, 0x68, 0x6c, 0x75, 0x74,
0x61, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x61, 0x72, 0xed, 0x6b, 0x6a, 0x61,
0x6e, 0x6e, 0x61, 0x20, 0x6f, 0x67, 0x20, 0x4b, 0x61, 0x6e, 0x61, 0x64,
0x61, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0x61, 0x75, 0x73, 0x74, 0x75, 0x72, 0x68, 0x6c, 0x75, 0x74,
0x61, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x61, 0x72, 0xed, 0x6b, 0x6a, 0x61,
0x6e, 0x6e, 0x61, 0x20, 0x6f, 0x67, 0x20, 0x4b, 0x61, 0x6e, 0x61, 0x64,
0x61, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0x61, 0x75, 0x73, 0x74, 0x75, 0x72, 0x68, 0x6c, 0x75, 0x74, 0x61,
0x20, 0x42, 0x61, 0x6e, 0x64, 0x61, 0x72, 0xed, 0x6b, 0x6a, 0x61, 0x6e,
0x6e, 0x61, 0x20, 0x6f, 0x67, 0x20, 0x4b, 0x61, 0x6e, 0x61, 0x64, 0x61,
0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4b, 0x6c, 0x65, 0x74, 0x74,
0x61, 0x66, 0x6a, 0xf6, 0x6c, 0x6c, 0x75, 0x6d, 0x53, 0x74, 0x61, 0xf0,
0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4b, 0x6c, 0x65,
0x74, 0x74, 0x61, 0x66, 0x6a, 0xf6, 0x6c, 0x6c, 0x75, 0x6d, 0x53, 0x75,
0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4b, 0x6c,
0x65, 0x74, 0x74, 0x61, 0x66, 0x6a, 0xf6, 0x6c, 0x6c, 0x75, 0x6d, 0x54,
0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x4b, 0x79, 0x72, 0x72, 0x61, 0x68,
0x61, 0x66, 0x73, 0x73, 0x76, 0xe6, 0xf0, 0x69, 0x6e, 0x75, 0x53, 0x74,
0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x4b,
0x79, 0x72, 0x72, 0x61, 0x68, 0x61, 0x66, 0x73, 0x73, 0x76, 0xe6, 0xf0,
0x69, 0x6e, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xe1, 0x20, 0x4b, 0x79, 0x72, 0x72, 0x61, 0x68, 0x61, 0x66, 0x73,
0x73, 0x76, 0xe6, 0xf0, 0x69, 0x6e, 0x75, 0x54, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x53, 0x74, 0x61, 0xf0,
0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41, 0x6e, 0x61,
0x64, 0x79, 0x72, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x54, 0xed, 0x6d,
0x69, 0x20, 0xed, 0x20, 0x41, 0x70, 0xed, 0x61, 0x53, 0x74, 0x61, 0xf0,
0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41, 0x70, 0xed,
0x61, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0x41, 0x70, 0xed, 0x61, 0x41, 0x72, 0x61, 0x62, 0xed, 0x75, 0x74,
0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xed, 0x20, 0x41, 0x72, 0x61, 0x62, 0xed, 0x75, 0x53, 0x75,
0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41, 0x72,
0x61, 0x62, 0xed, 0x75, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e,
0x75, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0xed, 0x6e, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e, 0x75,
0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0xed, 0x6e, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0,
0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x56, 0x65, 0x73,
0x74, 0x75, 0x72, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e,
0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0xed, 0x6e, 0x75, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0xed, 0x75,
0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0xed, 0x75,
0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0x41, 0x72, 0x6d, 0x65, 0x6e, 0xed, 0x75, 0x54, 0xed, 0x6d, 0x69, 0x20,
0xe1, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x73, 0x68, 0x61, 0x66,
0x73, 0x73, 0x76, 0xe6, 0xf0, 0x69, 0x6e, 0x75, 0x53, 0x74, 0x61, 0xf0,
0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x73, 0x68, 0x61, 0x66, 0x73, 0x73, 0x76, 0xe6, 0xf0,
0x69, 0x6e, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xe1, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x73, 0x68, 0x61,
0x66, 0x73, 0x73, 0x76, 0xe6, 0xf0, 0x69, 0x6e, 0x75, 0x54, 0xed, 0x6d,
0x69, 0x20, 0xed, 0x20, 0x4d, 0x69, 0xf0, 0x2d, 0xc1, 0x73, 0x74, 0x72,
0x61, 0x6c, 0xed, 0x75, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x4d, 0x69, 0xf0, 0x2d, 0xc1, 0x73, 0x74,
0x72, 0x61, 0x6c, 0xed, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x4d, 0x69, 0xf0, 0x2d, 0xc1, 0x73, 0x74,
0x72, 0x61, 0x6c, 0xed, 0x75, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0x6d, 0x69, 0xf0, 0x76, 0x65, 0x73, 0x74, 0x75, 0x72, 0x68, 0x6c, 0x75,
0x74, 0x61, 0x20, 0xc1, 0x73, 0x74, 0x72, 0x61, 0x6c, 0xed, 0x75, 0x53,
0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0x6d, 0x69, 0xf0, 0x76, 0x65, 0x73, 0x74, 0x75, 0x72, 0x68, 0x6c, 0x75,
0x74, 0x61, 0x20, 0xc1, 0x73, 0x74, 0x72, 0x61, 0x6c, 0xed, 0x75, 0x53,
0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x6d,
0x69, 0xf0, 0x76, 0x65, 0x73, 0x74, 0x75, 0x72, 0x68, 0x6c, 0x75, 0x74,
0x61, 0x20, 0xc1, 0x73, 0x74, 0x72, 0x61, 0x6c, 0xed, 0x75, 0x54, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x41, 0x75, 0x73, 0x74, 0x75, 0x72, 0x2d,
0xc1, 0x73, 0x74, 0x72, 0x61, 0x6c, 0xed, 0x75, 0x53, 0x74, 0x61, 0xf0,
0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41, 0x75, 0x73,
0x74, 0x75, 0x72, 0x2d, 0xc1, 0x73, 0x74, 0x72, 0x61, 0x6c, 0xed, 0x75,
0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0x41, 0x75, 0x73, 0x74, 0x75, 0x72, 0x2d, 0xc1, 0x73, 0x74, 0x72, 0x61,
0x6c, 0xed, 0x75, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x56, 0x65,
0x73, 0x74, 0x75, 0x72, 0x2d, 0xc1, 0x73, 0x74, 0x72, 0x61, 0x6c, 0xed,
0x75, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x2d, 0xc1, 0x73, 0x74,
0x72, 0x61, 0x6c, 0xed, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x2d,
0xc1, 0x73, 0x74, 0x72, 0x61, 0x6c, 0xed, 0x75, 0x41, 0x73, 0x65, 0x72,
0x62, 0x61, 0xed, 0x64, 0x73, 0x6a, 0x61, 0x6e, 0x74, 0xed, 0x6d, 0x69,
0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0xed, 0x64, 0x73, 0x6a, 0x61,
0x6e, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0xed, 0x64, 0x73, 0x6a, 0x61,
0x6e, 0x41, 0x73, 0xf3, 0x72, 0x65, 0x79, 0x6a, 0x61, 0x74, 0xed, 0x6d,
0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xe1, 0x20, 0x41, 0x73, 0xf3, 0x72, 0x65, 0x79, 0x6a, 0x75, 0x6d, 0x53,
0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x41,
0x73, 0xf3, 0x72, 0x65, 0x79, 0x6a, 0x75, 0x6d, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x73, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x53,
0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x73, 0x53, 0x75,
0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x73, 0x42, 0xfa, 0x74, 0x61,
0x6e, 0x74, 0xed, 0x6d, 0x69, 0x42, 0xf3, 0x6c, 0x69, 0x76, 0xed, 0x75,
0x74, 0xed, 0x6d, 0x69, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0xed, 0x75,
0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0xed,
0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0xed, 0x75, 0x42, 0x72, 0xfa,
0x6e, 0x65, 0x69, 0x74, 0xed, 0x6d, 0x69, 0x47, 0x72, 0xe6, 0x6e, 0x68,
0xf6, 0x66, 0xf0, 0x61, 0x65, 0x79, 0x6a, 0x61, 0x74, 0xed, 0x6d, 0x69,
0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1,
0x20, 0x47, 0x72, 0xe6, 0x6e, 0x68, 0xf6, 0x66, 0xf0, 0x61, 0x65, 0x79,
0x6a, 0x75, 0x6d, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xe1, 0x20, 0x47, 0x72, 0xe6, 0x6e, 0x68, 0xf6, 0x66, 0xf0, 0x61,
0x65, 0x79, 0x6a, 0x75, 0x6d, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72,
0x6f, 0x2d, 0x73, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x2d, 0x74, 0xed, 0x6d, 0x69,
0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x53, 0x75, 0x6d, 0x61,
0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x53, 0xed, 0x6c, 0x65, 0x74, 0xed, 0x6d, 0x69, 0x53,
0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0x53, 0xed, 0x6c, 0x65, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xed, 0x20, 0x53, 0xed, 0x6c, 0x65, 0x4b, 0xed, 0x6e, 0x61,
0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x4b, 0xed, 0x6e, 0x61, 0x53, 0x75, 0x6d,
0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4b, 0xed, 0x6e,
0x61, 0x4a, 0xf3, 0x6c, 0x61, 0x65, 0x79, 0x6a, 0x61, 0x72, 0x74, 0xed,
0x6d, 0x69, 0x4b, 0xf3, 0x6b, 0x6f, 0x73, 0x65, 0x79, 0x6a, 0x61, 0x74,
0xed, 0x6d, 0x69, 0x4b, 0xf3, 0x6c, 0x75, 0x6d, 0x62, 0xed, 0x75, 0x74,
0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xed, 0x20, 0x4b, 0xf3, 0x6c, 0x75, 0x6d, 0x62, 0xed, 0x75,
0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0x4b, 0xf3, 0x6c, 0x75, 0x6d, 0x62, 0xed, 0x75, 0x43, 0x6f, 0x6f, 0x6b,
0x73, 0x2d, 0x65, 0x79, 0x6a, 0x61, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74,
0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x43,
0x6f, 0x6f, 0x6b, 0x73, 0x2d, 0x65, 0x79, 0x6a, 0x75, 0x6d, 0x48, 0xe1,
0x6c, 0x66, 0x73, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xe1, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x73, 0x2d, 0x65, 0x79, 0x6a, 0x75,
0x6d, 0x4b, 0xfa, 0x62, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61,
0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x4b, 0xfa,
0x62, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xe1, 0x20, 0x4b, 0xfa, 0x62, 0x75, 0x44, 0x61, 0x76, 0x69, 0x73, 0x2d,
0x74, 0xed, 0x6d, 0x69, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x44,
0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69,
0x6c, 0x6c, 0x65, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x54, 0xed,
0x6d, 0x6f, 0x72, 0x2d, 0x4c, 0x65, 0x73, 0x74, 0x65, 0x50, 0xe1, 0x73,
0x6b, 0x61, 0x65, 0x79, 0x6a, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74,
0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x50,
0xe1, 0x73, 0x6b, 0x61, 0x65, 0x79, 0x6a, 0x75, 0x53, 0x75, 0x6d, 0x61,
0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x50, 0xe1, 0x73, 0x6b,
0x61, 0x65, 0x79, 0x6a, 0x75, 0x45, 0x6b, 0x76, 0x61, 0x64, 0x6f, 0x72,
0x74, 0xed, 0x6d, 0x69, 0x4d, 0x69, 0xf0, 0x2d, 0x45, 0x76, 0x72, 0xf3,
0x70, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4d, 0x69, 0xf0, 0x2d, 0x45,
0x76, 0x72, 0xf3, 0x70, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x4d, 0x69, 0xf0, 0x2d, 0x45, 0x76, 0x72,
0xf3, 0x70, 0x75, 0x41, 0x75, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x45, 0x76,
0x72, 0xf3, 0x70, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0,
0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41, 0x75, 0x73,
0x74, 0x75, 0x72, 0x2d, 0x45, 0x76, 0x72, 0xf3, 0x70, 0x75, 0x53, 0x75,
0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41, 0x75,
0x73, 0x74, 0x75, 0x72, 0x2d, 0x45, 0x76, 0x72, 0xf3, 0x70, 0x75, 0x53,
0x74, 0x61, 0xf0, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0x4b, 0x61,
0x6c, 0xed, 0x6e, 0xed, 0x6e, 0x67, 0x72, 0x61, 0x64, 0x56, 0x65, 0x73,
0x74, 0x75, 0x72, 0x2d, 0x45, 0x76, 0x72, 0xf3, 0x70, 0x75, 0x74, 0xed,
0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x45, 0x76,
0x72, 0xf3, 0x70, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xed, 0x20, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x45,
0x76, 0x72, 0xf3, 0x70, 0x75, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x65, 0x79, 0x6a, 0x61, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74,
0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x46,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x79, 0x6a, 0x75,
0x6d, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1,
0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x79,
0x6a, 0x75, 0x6d, 0x46, 0xed, 0x64, 0x6a, 0xed, 0x65, 0x79, 0x6a, 0x61,
0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xe1, 0x20, 0x46, 0xed, 0x64, 0x6a, 0xed, 0x65, 0x79,
0x6a, 0x75, 0x6d, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xe1, 0x20, 0x46, 0xed, 0x64, 0x6a, 0xed, 0x65, 0x79, 0x6a, 0x75,
0x6d, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x46, 0x72, 0xf6, 0x6e,
0x73, 0x6b, 0x75, 0x20, 0x47, 0x76, 0xe6, 0x6a, 0x61, 0x6e, 0x61, 0x54,
0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x66, 0x72, 0xf6, 0x6e, 0x73, 0x6b,
0x75, 0x20, 0x73, 0x75, 0xf0, 0x75, 0x72, 0x68, 0x61, 0x66, 0x73, 0x73,
0x76, 0xe6, 0xf0, 0x75, 0x6d, 0x20, 0x6f, 0x67, 0x20, 0x53, 0x75, 0xf0,
0x75, 0x72, 0x73, 0x6b, 0x61, 0x75, 0x74, 0x73, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x73, 0x76, 0xe6, 0xf0, 0x69, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61,
0x67, 0x6f, 0x73, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x47, 0x61, 0x6d, 0x62,
0x69, 0x65, 0x72, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x47, 0x65, 0x6f, 0x72,
0x67, 0xed, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61,
0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x47, 0x65, 0x6f, 0x72,
0x67, 0xed, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0xed, 0x75, 0x54, 0xed,
0x6d, 0x69, 0x20, 0xe1, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74,
0x2d, 0x65, 0x79, 0x6a, 0x75, 0x6d, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77,
0x69, 0x63, 0x68, 0x2d, 0x73, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed,
0x6d, 0x69, 0x41, 0x75, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x47, 0x72, 0xe6,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74,
0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x41,
0x75, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x47, 0x72, 0xe6, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xe1, 0x20, 0x41, 0x75, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x47, 0x72,
0xe6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x56, 0x65, 0x73, 0x74, 0x75,
0x72, 0x2d, 0x47, 0x72, 0xe6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x74,
0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xe1, 0x20, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x47,
0x72, 0xe6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x53, 0x75, 0x6d, 0x61,
0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x56, 0x65, 0x73, 0x74,
0x75, 0x72, 0x2d, 0x47, 0x72, 0xe6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0x76,
0x69, 0xf0, 0x20, 0x50, 0x65, 0x72, 0x73, 0x61, 0x66, 0x6c, 0xf3, 0x61,
0x47, 0x76, 0xe6, 0x6a, 0x61, 0x6e, 0x61, 0x74, 0xed, 0x6d, 0x69, 0x54,
0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x48, 0x61, 0x76, 0x61, 0xed, 0x20,
0x6f, 0x67, 0x20, 0x41, 0x6c, 0x65, 0xfa, 0x74, 0x61, 0x53, 0x74, 0x61,
0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x48, 0x61,
0x76, 0x61, 0xed, 0x20, 0x6f, 0x67, 0x20, 0x41, 0x6c, 0x65, 0xfa, 0x74,
0x61, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1,
0x20, 0x48, 0x61, 0x76, 0x61, 0xed, 0x20, 0x6f, 0x67, 0x20, 0x41, 0x6c,
0x65, 0xfa, 0x74, 0x61, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e,
0x67, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6e, 0x67, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xed, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e,
0x67, 0x48, 0x6f, 0x76, 0x64, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74,
0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x48,
0x6f, 0x76, 0x64, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x49, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x74, 0xed, 0x6d, 0x69, 0x49, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x68, 0x61, 0x66, 0x73, 0x74, 0xed, 0x6d, 0x69, 0x49,
0x6e, 0x64, 0xf3, 0x6b, 0xed, 0x6e, 0x61, 0x74, 0xed, 0x6d, 0x69, 0x4d,
0x69, 0xf0, 0x2d, 0x49, 0x6e, 0x64, 0xf3, 0x6e, 0x65, 0x73, 0xed, 0x75,
0x74, 0xed, 0x6d, 0x69, 0x41, 0x75, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x49,
0x6e, 0x64, 0xf3, 0x6e, 0x65, 0x73, 0xed, 0x75, 0x74, 0xed, 0x6d, 0x69,
0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x49, 0x6e, 0x64, 0xf3, 0x6e,
0x65, 0x73, 0xed, 0x75, 0x74, 0xed, 0x6d, 0x69, 0xcd, 0x72, 0x61, 0x6e,
0x73, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0xcd, 0x72, 0x61, 0x6e, 0x53, 0x75,
0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0xcd, 0x72,
0x61, 0x6e, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xcd, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x73, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0xcd, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0xcd, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x4a, 0x61, 0x70, 0x61, 0x6e,
0x73, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x53,
0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4a,
0x61, 0x70, 0x61, 0x6e, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x50,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x53,
0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73,
0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69,
0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73,
0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69,
0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4b, 0x61, 0x73, 0x61, 0x6b,
0x73, 0x74, 0x61, 0x6e, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41,
0x75, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x4b, 0x61, 0x73, 0x61, 0x6b, 0x73,
0x74, 0x61, 0x6e, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x56, 0x65,
0x73, 0x74, 0x75, 0x72, 0x2d, 0x4b, 0x61, 0x73, 0x61, 0x6b, 0x73, 0x74,
0x61, 0x6e, 0x4b, 0xf3, 0x72, 0x65, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x53,
0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0x4b, 0xf3, 0x72, 0x65, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x4b, 0xf3, 0x72, 0x65, 0x75, 0x4b, 0x6f,
0x73, 0x72, 0x61, 0x65, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x54, 0xed, 0x6d,
0x69, 0x20, 0xed, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x73, 0x6b, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xed, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x73, 0x6b, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x4b, 0x69, 0x72, 0x67, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d,
0x74, 0xed, 0x6d, 0x69, 0x4c, 0xed, 0x6e, 0x75, 0x65, 0x79, 0x6a, 0x61,
0x2d, 0x74, 0xed, 0x6d, 0x69, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0x65, 0x79,
0x6a, 0x75, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xe1, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0x2d, 0x65, 0x79, 0x6a, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xe1, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x2d, 0x65, 0x79, 0x6a, 0x75, 0x54, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x53, 0x74, 0x61,
0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x4d, 0x61, 0x6c, 0x61, 0x73, 0xed, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x4d,
0x61, 0x6c, 0x64, 0xed, 0x76, 0x65, 0x79, 0x6a, 0x61, 0x2d, 0x74, 0xed,
0x6d, 0x69, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x4d, 0x61, 0x72,
0x6b, 0x67, 0x72, 0x65, 0x69, 0x66, 0x61, 0x66, 0x72, 0xfa, 0x72, 0x65,
0x79, 0x6a, 0x75, 0x6d, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x4d,
0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x2d, 0x65, 0x79, 0x6a, 0x75,
0x6d, 0x4d, 0xe1, 0x72, 0x69, 0x74, 0xed, 0x75, 0x73, 0x74, 0xed, 0x6d,
0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xe1, 0x20, 0x4d, 0xe1, 0x72, 0x69, 0x74, 0xed, 0x75, 0x73, 0x53, 0x75,
0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x4d, 0xe1,
0x72, 0x69, 0x74, 0xed, 0x75, 0x73, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e,
0x2d, 0x74, 0xed, 0x6d, 0x69, 0x4b, 0x79, 0x72, 0x72, 0x61, 0x68, 0x61,
0x66, 0x73, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4d, 0x65, 0x78,
0xed, 0x6b, 0xf3, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xed, 0x20, 0x4d, 0x65, 0x78, 0xed, 0x6b, 0xf3, 0x20, 0xe1,
0x20, 0x4b, 0x79, 0x72, 0x72, 0x61, 0x68, 0x61, 0x66, 0x73, 0x73, 0x76,
0xe6, 0xf0, 0x69, 0x6e, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x4d, 0x65, 0x78, 0xed, 0x6b, 0xf3, 0x20,
0xe1, 0x20, 0x4b, 0x79, 0x72, 0x72, 0x61, 0x68, 0x61, 0x66, 0x73, 0x73,
0x76, 0xe6, 0xf0, 0x69, 0x6e, 0x75, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0xda, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x53,
0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0xda, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x53, 0x75,
0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0xda, 0x6c,
0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x4d, 0x6f, 0x73, 0x6b,
0x76, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4d, 0x6f, 0x73, 0x6b, 0x76,
0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x75, 0x4d, 0x6a, 0x61, 0x6e, 0x6d,
0x61, 0x72, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x4e, 0xe1, 0x72, 0xfa, 0x2d,
0x74, 0xed, 0x6d, 0x69, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x74, 0xed, 0x6d,
0x69, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4e, 0xfd, 0x6a, 0x75,
0x2d, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0xf3, 0x6e, 0xed, 0x75, 0x53, 0x74,
0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4e,
0xfd, 0x6a, 0x75, 0x2d, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0xf3, 0x6e, 0xed,
0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0x4e, 0xfd, 0x6a, 0x75, 0x2d, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0xf3,
0x6e, 0xed, 0x75, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x4e, 0xfd,
0x6a, 0x61, 0x2d, 0x53, 0x6a, 0xe1, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x53,
0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20,
0x4e, 0xfd, 0x6a, 0x61, 0x2d, 0x53, 0x6a, 0xe1, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1,
0x20, 0x4e, 0xfd, 0x6a, 0x61, 0x2d, 0x53, 0x6a, 0xe1, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x4e, 0xfd, 0x66,
0x75, 0x6e, 0x64, 0x6e, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x53, 0x74,
0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x4e,
0xfd, 0x66, 0x75, 0x6e, 0x64, 0x6e, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20,
0x4e, 0xfd, 0x66, 0x75, 0x6e, 0x64, 0x6e, 0x61, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x4e, 0x69, 0x75, 0x65, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x54, 0xed,
0x6d, 0x69, 0x20, 0xe1, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x2d, 0x65, 0x79, 0x6a, 0x75, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x2d, 0x65, 0x79, 0x6a, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x2d, 0x65, 0x79, 0x6a, 0x75, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x53, 0x74, 0x61, 0xf0,
0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xed, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x54,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4f,
0x6d, 0x73, 0x6b, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xed, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x53, 0x75, 0x6d, 0x61,
0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4f, 0x6d, 0x73, 0x6b,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x74, 0xed, 0x6d, 0x69,
0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x53, 0x75, 0x6d,
0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x50, 0x61, 0x6c, 0xe1, 0x74, 0xed, 0x6d,
0x69, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x50, 0x61, 0x70, 0xfa,
0x61, 0x20, 0x4e, 0xfd, 0x6a, 0x75, 0x2d, 0x47, 0xed, 0x6e, 0x65, 0x75,
0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0xe6, 0x74, 0xed, 0x6d, 0x69, 0x53,
0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0xe6, 0x53, 0x75, 0x6d, 0x61, 0x72,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67,
0x76, 0xe6, 0x50, 0x65, 0x72, 0xfa, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74,
0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x50,
0x65, 0x72, 0xfa, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x50, 0x65, 0x72, 0xfa, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x70, 0x73, 0x65, 0x79, 0x6a, 0x61, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74,
0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x46,
0x69, 0x6c, 0x69, 0x70, 0x70, 0x73, 0x65, 0x79, 0x6a, 0x75, 0x6d, 0x53,
0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x46,
0x69, 0x6c, 0x69, 0x70, 0x70, 0x73, 0x65, 0x79, 0x6a, 0x75, 0x6d, 0x46,
0xf6, 0x6e, 0x69, 0x78, 0x65, 0x79, 0x6a, 0x61, 0x74, 0xed, 0x6d, 0x69,
0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x53, 0x61, 0x6e, 0x6b, 0x74,
0x69, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x6f, 0x67, 0x20,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x53, 0x74, 0x61, 0xf0,
0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x53, 0x61, 0x6e,
0x6b, 0x74, 0x69, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x6f,
0x67, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x53, 0x75,
0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x53, 0x61,
0x6e, 0x6b, 0x74, 0x69, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20,
0x6f, 0x67, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x50,
0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x2d, 0x74, 0xed, 0x6d, 0x69,
0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x54,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x50, 0x6a, 0x6f, 0x6e, 0x67, 0x6a,
0x61, 0x6e, 0x67, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x2d, 0x74,
0xed, 0x6d, 0x69, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x2d, 0x74,
0xed, 0x6d, 0x69, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x53, 0x61,
0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xe1, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x54,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x53, 0x75, 0x6d, 0x61, 0x72,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x53, 0x61, 0x6d, 0xf3, 0x61, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x53,
0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20,
0x53, 0x61, 0x6d, 0xf3, 0x61, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xe1, 0x20, 0x53, 0x61, 0x6d, 0xf3, 0x61, 0x53, 0x65,
0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x2d, 0x65, 0x79, 0x6a,
0x61, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0xfa,
0x72, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x61, 0x6c, 0xf3, 0x6d, 0x6f, 0x6e,
0x73, 0x65, 0x79, 0x6a, 0x61, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x75, 0xf0,
0x75, 0x72, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0xed, 0x75, 0x74, 0xed,
0x6d, 0x69, 0x53, 0xfa, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x74, 0xed, 0x6d,
0x69, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x54,
0x61, 0x68, 0xed, 0x74, 0xed, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x54, 0x61,
0x69, 0x70, 0x65, 0x69, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61,
0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x54, 0x61,
0x69, 0x70, 0x65, 0x69, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xed, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x54, 0x61,
0x64, 0x73, 0x6a, 0xed, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x74,
0xed, 0x6d, 0x69, 0x54, 0xf3, 0x6b, 0x65, 0x6c, 0xe1, 0x2d, 0x74, 0xed,
0x6d, 0x69, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x74, 0xed, 0x6d, 0x69, 0x53,
0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xe1, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x43, 0x68,
0x75, 0x75, 0x6b, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x54, 0xfa, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x74, 0xed, 0x6d,
0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0x54, 0xfa, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0x54, 0xfa, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x54, 0xfa, 0x76, 0x61, 0x6c, 0xfa, 0x74, 0xed, 0x6d, 0x69,
0xda, 0x72, 0xfa, 0x67, 0x76, 0xe6, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74,
0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0xda,
0x72, 0xfa, 0x67, 0x76, 0xe6, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0xda, 0x72, 0xfa, 0x67, 0x76, 0xe6, 0xda,
0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x74, 0xed,
0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0xda, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0xda, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x56,
0x61, 0x6e, 0xfa, 0x61, 0x74, 0xfa, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x53,
0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20,
0x56, 0x61, 0x6e, 0xfa, 0x61, 0x74, 0xfa, 0x53, 0x75, 0x6d, 0x61, 0x72,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x56, 0x61, 0x6e, 0xfa, 0x61,
0x74, 0xfa, 0x56, 0x65, 0x6e, 0x65, 0x73, 0xfa, 0x65, 0x6c, 0x61, 0x74,
0xed, 0x6d, 0x69, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x53, 0x74, 0x61,
0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x53, 0x75, 0x6d,
0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x54, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x53, 0x75,
0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x56, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1,
0x20, 0x57, 0x61, 0x6b, 0x65, 0x2d, 0x65, 0x79, 0x6a, 0x75, 0x54, 0xed,
0x6d, 0x69, 0x20, 0xe1, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x2d,
0x20, 0x6f, 0x67, 0x20, 0x46, 0xfa, 0x74, 0xfa, 0x6e, 0x61, 0x65, 0x79,
0x6a, 0x75, 0x6d, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x54, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x6f, 0x72, 0x67, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x54, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0x4a, 0xfa, 0x6b, 0x6f, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x41,
0x66, 0x67, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x67,
0x65, 0x20, 0x45, 0x74, 0x69, 0x74, 0x69, 0x20, 0x41, 0x66, 0x72, 0x1ecb,
0x6b, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61,
0x72, 0x61, 0x20, 0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20, 0x41, 0x6e, 0x79,
0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x41, 0x66, 0x72, 0x1ecb, 0x6b, 0x61, 0x4f,
0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4d, 0x70,
0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x4d, 0x67, 0x62, 0x61, 0x64,
0x61, 0x20, 0x55, 0x67, 0x77, 0x75, 0x20, 0x41, 0x66, 0x72, 0x1ecb, 0x6b,
0x61, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72,
0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61,
0x6e, 0x77, 0x1ee5, 0x20, 0x41, 0x66, 0x72, 0x1ecb, 0x6b, 0x61, 0x4f, 0x67,
0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4d, 0x70, 0x61,
0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64, 0x61, 0x20,
0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x41, 0x66, 0x72, 0x1ecb,
0x6b, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd,
0x6b, 0x1ee5, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20,
0x1ecc, 0x64, 0x1ecb, 0x64, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77,
0x1ee5, 0x20, 0x41, 0x66, 0x72, 0x1ecb, 0x6b, 0x61, 0x4f, 0x67, 0x65, 0x20,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a,
0x75, 0x67, 0x62, 0x65, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4f,
0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x4f, 0x67, 0x65, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x4f,
0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f,
0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x4f,
0x67, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20,
0x45, 0x74, 0x69, 0x74, 0x69, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75,
0x67, 0x62, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61,
0x20, 0x45, 0x74, 0x69, 0x74, 0x69, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68,
0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x45,
0x74, 0x69, 0x74, 0x69, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67,
0x62, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20,
0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77,
0x1ee5, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x4d, 0x70, 0x61,
0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20,
0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x4f, 0x67, 0x65, 0x20, 0x4d,
0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x55, 0x67, 0x77, 0x75,
0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4d,
0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x55, 0x67, 0x77, 0x75,
0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67,
0x68, 0x61, 0x72, 0x61, 0x20, 0x55, 0x67, 0x77, 0x75, 0x4f, 0x67, 0x65,
0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x50, 0x61,
0x63, 0x69, 0x66, 0x69, 0x63, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75,
0x67, 0x62, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61,
0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x4f, 0x67, 0x65, 0x20,
0x49, 0x68, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61,
0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x4f, 0x67, 0x65, 0x20,
0x41, 0x70, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67,
0x62, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49,
0x68, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x41,
0x72, 0x61, 0x62, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62,
0x65, 0x20, 0x41, 0x72, 0x61, 0x62, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68,
0x65, 0x20, 0x41, 0x72, 0x61, 0x62, 0x4f, 0x67, 0x65, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49,
0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d,
0x1ecd, 0x6b, 0x1ee5, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72,
0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61,
0x6e, 0x77, 0x1ee5, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20,
0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb,
0x64, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x67, 0x65, 0x20,
0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4d, 0x70, 0x61,
0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64, 0x61, 0x20,
0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x41, 0x72, 0x6d,
0x65, 0x6e, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67,
0x62, 0x65, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x4f, 0x67,
0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x41,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70,
0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x63, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62,
0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x4f, 0x67, 0x65, 0x20, 0x49,
0x68, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x4f, 0x67, 0x65, 0x20,
0x45, 0x74, 0x69, 0x74, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62,
0x65, 0x20, 0x45, 0x74, 0x69, 0x74, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65,
0x20, 0x45, 0x74, 0x69, 0x74, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67,
0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64, 0x61, 0x20, 0x41,
0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x45, 0x74, 0x69, 0x74, 0x69,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f, 0x67,
0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4d, 0x70, 0x61,
0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64, 0x61, 0x20,
0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x45, 0x74, 0x69, 0x74,
0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f,
0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68,
0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64, 0x61, 0x20, 0x41, 0x6e,
0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x45, 0x74, 0x69, 0x74, 0x69, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f, 0x67, 0x65,
0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x77,
0x1ee5, 0x77, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f, 0x67, 0x65,
0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67,
0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20, 0x41,
0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20,
0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x77, 0x1ee5,
0x77, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20,
0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb,
0x64, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20,
0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68,
0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64, 0x61, 0x20, 0x41, 0x6e,
0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x4d,
0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64,
0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x41,
0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x4f, 0x67, 0x65,
0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b,
0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62,
0x61, 0x69, 0x6a, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62,
0x65, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x4f, 0x67, 0x65, 0x20,
0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x73, 0x68, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75,
0x67, 0x62, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x73, 0x68, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd,
0x6b, 0x1ee5, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73,
0x68, 0x4f, 0x67, 0x65, 0x20, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x4f,
0x67, 0x65, 0x20, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x4f, 0x67,
0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x4f, 0x67,
0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70,
0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c,
0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69,
0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x4f,
0x67, 0x65, 0x20, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64,
0x65, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20,
0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x4f, 0x67,
0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x43,
0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x4f, 0x67, 0x65,
0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x43, 0x68, 0x61, 0x6d,
0x6f, 0x72, 0x72, 0x6f, 0x4f, 0x67, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62,
0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x4f, 0x67, 0x65,
0x20, 0x49, 0x68, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x4f, 0x67, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x4f, 0x67, 0x65,
0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6c,
0x65, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b,
0x1ee5, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x43,
0x68, 0x69, 0x6e, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67,
0x62, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x4f, 0x67, 0x65, 0x20,
0x49, 0x68, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x4f, 0x67, 0x65,
0x20, 0x45, 0x6b, 0x65, 0x72, 0x65, 0x73, 0x69, 0x6d, 0x65, 0x73, 0x69,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x43,
0x6f, 0x63, 0x6f, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x4f, 0x67, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61,
0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x43,
0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4f,
0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x43, 0x6f, 0x6c, 0x75,
0x6d, 0x62, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x43, 0x6f, 0x6f, 0x6b,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x67, 0x65, 0x20,
0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x1ecc,
0x6b, 0x61, 0x72, 0x61, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b,
0x1ee5, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x4f, 0x67,
0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x43, 0x75, 0x62,
0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x4d, 0x70, 0x61,
0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x43, 0x75, 0x62, 0x61, 0x4f, 0x67,
0x65, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x44,
0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69,
0x6c, 0x6c, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68,
0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20, 0x41, 0x6e,
0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x4f,
0x67, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20,
0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77,
0x1ee5, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20,
0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68,
0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20, 0x41, 0x6e,
0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64,
0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5,
0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x77,
0x1ee5, 0x77, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x45, 0x63,
0x75, 0x61, 0x64, 0x6f, 0x72, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70, 0x61,
0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x45, 0x74, 0x69, 0x74, 0x69, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a,
0x75, 0x67, 0x62, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72,
0x61, 0x20, 0x45, 0x74, 0x69, 0x74, 0x69, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd,
0x6b, 0x1ee5, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20,
0x45, 0x74, 0x69, 0x74, 0x69, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61,
0x20, 0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e,
0x77, 0x1ee5, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f, 0x67, 0x65,
0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67,
0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20, 0x41,
0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b,
0x1ee5, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc,
0x77, 0x1ee5, 0x77, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x4d,
0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64,
0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67,
0x62, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20,
0x1ecc, 0x64, 0x1ecb, 0x64, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77,
0x1ee5, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f, 0x67, 0x65, 0x20,
0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4d, 0x70, 0x61,
0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64, 0x61, 0x20,
0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x67,
0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x46, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b,
0x1ee5, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x46, 0x69,
0x6a, 0x69, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65,
0x20, 0x46, 0x69, 0x6a, 0x69, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70,
0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x4f, 0x67,
0x65, 0x20, 0x46, 0x72, 0x65, 0x6e, 0x63, 0x68, 0x20, 0x47, 0x75, 0x69,
0x61, 0x6e, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x46, 0x72, 0x65, 0x6e, 0x63,
0x68, 0x20, 0x53, 0x6f, 0x75, 0x74, 0x68, 0x65, 0x72, 0x6e, 0x20, 0x26,
0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x63, 0x74, 0x69, 0x63, 0x4f, 0x67,
0x65, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x4f,
0x67, 0x65, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x4f, 0x67,
0x65, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x4f, 0x67, 0x65,
0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x47, 0x65, 0x6f, 0x72,
0x67, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d,
0x1ecd, 0x6b, 0x1ee5, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x4f,
0x67, 0x65, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70,
0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6d,
0x77, 0x69, 0x63, 0x68, 0x20, 0x4d, 0x65, 0x61, 0x6e, 0x4f, 0x67, 0x65,
0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x77,
0x1ee5, 0x77, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65,
0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67,
0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20, 0x41,
0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f,
0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72,
0x61, 0x20, 0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61,
0x6e, 0x77, 0x1ee5, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72,
0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61,
0x6e, 0x77, 0x1ee5, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20,
0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb,
0x64, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20,
0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4d, 0x70, 0x61,
0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64, 0x61, 0x20,
0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75,
0x67, 0x62, 0x65, 0x20, 0x47, 0x75, 0x6c, 0x66, 0x4f, 0x67, 0x65, 0x20,
0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x48, 0x61,
0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61,
0x6e, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20,
0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x69, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x48,
0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69,
0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b,
0x6f, 0x6e, 0x67, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62,
0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x4f,
0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20,
0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x4f, 0x67, 0x65,
0x20, 0x48, 0x6f, 0x76, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75,
0x67, 0x62, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x4f, 0x67, 0x65, 0x20,
0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x48, 0x6f, 0x76,
0x64, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x73, 0x69,
0x6d, 0x69, 0x72, 0x69, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x4f, 0x67,
0x65, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x4f,
0x67, 0x65, 0x20, 0x45, 0x74, 0x69, 0x74, 0x69, 0x20, 0x49, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70,
0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x77, 0x1ee5, 0x77, 0x61,
0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x49, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70,
0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64, 0x61,
0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x49, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x72,
0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65,
0x20, 0x49, 0x72, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65,
0x20, 0x49, 0x72, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67,
0x62, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x67,
0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x49,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x73,
0x72, 0x61, 0x65, 0x6c, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67,
0x62, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x4f, 0x67, 0x65,
0x20, 0x49, 0x68, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x4f,
0x67, 0x65, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20,
0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e,
0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x4a, 0x61, 0x70, 0x61,
0x6e, 0x4f, 0x67, 0x65, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73,
0x74, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68,
0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20, 0x41, 0x6e,
0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68,
0x73, 0x74, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67,
0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64, 0x61, 0x20, 0x41,
0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b,
0x68, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x4b, 0x6f, 0x72,
0x65, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65,
0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68,
0x65, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4b,
0x6f, 0x73, 0x72, 0x61, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x4f, 0x67, 0x65, 0x20,
0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b,
0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x4f, 0x67, 0x65, 0x20, 0x4b, 0x79,
0x72, 0x67, 0x79, 0x7a, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20,
0x4c, 0x69, 0x6e, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x4f, 0x67, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x4f, 0x67, 0x65,
0x20, 0x49, 0x68, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x4f,
0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x61, 0x6c, 0x61, 0x79,
0x73, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69,
0x76, 0x65, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75,
0x65, 0x73, 0x61, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x61, 0x72, 0x73,
0x68, 0x61, 0x6c, 0x6c, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x4f, 0x67, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x4f, 0x67, 0x65,
0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x4d,
0x61, 0x77, 0x73, 0x6f, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x65, 0x78,
0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63,
0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4d,
0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66,
0x69, 0x63, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x4d, 0x65,
0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69,
0x63, 0x4f, 0x67, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61,
0x61, 0x74, 0x61, 0x72, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67,
0x62, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74,
0x61, 0x72, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd,
0x6b, 0x1ee5, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74,
0x61, 0x72, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77,
0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4d,
0x6f, 0x73, 0x63, 0x6f, 0x77, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70,
0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77,
0x4f, 0x67, 0x65, 0x20, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x4f,
0x67, 0x65, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x4f, 0x67, 0x65, 0x20,
0x4e, 0x65, 0x70, 0x61, 0x6c, 0x4f, 0x67, 0x65, 0x20, 0x4e, 0x65, 0x77,
0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x67,
0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4e, 0x65, 0x77,
0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x67,
0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4e,
0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61,
0x4f, 0x67, 0x65, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c,
0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62,
0x65, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e,
0x64, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x4e, 0x65, 0x77,
0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20,
0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64,
0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4e,
0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x4f,
0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f,
0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x4e,
0x69, 0x75, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65,
0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67,
0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x4f, 0x67, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b,
0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75,
0x67, 0x62, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d,
0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4f,
0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4f, 0x6d,
0x73, 0x6b, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd,
0x6b, 0x1ee5, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x67, 0x65, 0x20, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x49,
0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd,
0x6b, 0x1ee5, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f,
0x67, 0x65, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x4f, 0x67, 0x65, 0x20,
0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x47, 0x75,
0x69, 0x6e, 0x65, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x79, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67,
0x62, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x4f,
0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x67, 0x65, 0x20,
0x50, 0x65, 0x72, 0x75, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67,
0x62, 0x65, 0x20, 0x50, 0x65, 0x72, 0x75, 0x4f, 0x67, 0x65, 0x20, 0x4f,
0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x50, 0x65, 0x72, 0x75,
0x4f, 0x67, 0x65, 0x20, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69,
0x6e, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65,
0x20, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x65, 0x4f,
0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20,
0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x65, 0x4f, 0x67,
0x65, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x53, 0x74, 0x2e,
0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a,
0x75, 0x67, 0x62, 0x65, 0x20, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x53, 0x74,
0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d,
0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x50,
0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x50,
0x6f, 0x6e, 0x61, 0x70, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x50, 0x79, 0x6f,
0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x4f, 0x67, 0x65, 0x20, 0x52, 0xe9,
0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x52, 0x6f, 0x74,
0x68, 0x65, 0x72, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67,
0x62, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x4f,
0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20,
0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x4f, 0x67, 0x65, 0x20,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75,
0x67, 0x62, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x4f, 0x67, 0x65,
0x20, 0x49, 0x68, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x4f, 0x67,
0x65, 0x20, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73,
0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x53,
0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65, 0x4f, 0x67, 0x65, 0x20,
0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x53, 0x6f, 0x75, 0x74, 0x68,
0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20,
0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x4f, 0x67, 0x65, 0x20,
0x53, 0x79, 0x6f, 0x77, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x54, 0x61, 0x68,
0x69, 0x74, 0x69, 0x4f, 0x67, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20,
0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68,
0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x4f, 0x67, 0x65, 0x20,
0x54, 0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x67,
0x65, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x4f, 0x67, 0x65,
0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a,
0x75, 0x67, 0x62, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x4f, 0x67,
0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x43, 0x68, 0x75, 0x75,
0x6b, 0x4f, 0x67, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62,
0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5,
0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x4f,
0x67, 0x65, 0x20, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x4f, 0x67, 0x65,
0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x67, 0x65, 0x20,
0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x79, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd,
0x6b, 0x1ee5, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x67,
0x65, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x4f, 0x67,
0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x55, 0x7a, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70,
0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x74, 0x4f, 0x67, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f, 0x67, 0x65, 0x20, 0x4f,
0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x4f, 0x67, 0x65, 0x20, 0x56, 0x65, 0x6e, 0x65, 0x7a,
0x75, 0x65, 0x6c, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x67, 0x65, 0x20, 0x49,
0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70,
0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x67, 0x65, 0x20, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a,
0x75, 0x67, 0x62, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd,
0x6b, 0x1ee5, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x4f, 0x67, 0x65, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x67,
0x65, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x4f, 0x67, 0x65, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20,
0x26, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x4f, 0x67, 0x65, 0x20,
0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x67, 0x65, 0x20, 0x49,
0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b,
0x1ee5, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x67, 0x65,
0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65,
0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd,
0x6b, 0x1ee5, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x4f, 0x67, 0x65, 0x20, 0x59, 0x75, 0x6b, 0x6f,
0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x63, 0x72, 0x65, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x41, 0x63, 0x72, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d,
0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x41,
0x63, 0x72, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x66, 0x67,
0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x54, 0x65, 0x6e, 0x67,
0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x53, 0x65, 0x6c, 0x61, 0x74, 0x61, 0x6e,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61,
0x73, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x42, 0x61, 0x72,
0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61,
0x6e, 0x61, 0x73, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20,
0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69,
0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x41, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x65, 0x6e, 0x67,
0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61,
0x6e, 0x61, 0x73, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x54,
0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75,
0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x54, 0x69,
0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x65, 0x67,
0x75, 0x6e, 0x75, 0x6e, 0x67, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x50, 0x65, 0x67,
0x75, 0x6e, 0x75, 0x6e, 0x67, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x50, 0x65, 0x67, 0x75, 0x6e, 0x75, 0x6e, 0x67, 0x61, 0x6e, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e,
0x61, 0x73, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20,
0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x41, 0x70, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x70, 0x69, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61,
0x6e, 0x61, 0x73, 0x20, 0x41, 0x70, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x71, 0x74,
0x61, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69,
0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x41, 0x71, 0x74, 0x61,
0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62,
0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e,
0x61, 0x73, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x41, 0x72, 0x61, 0x62, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x72, 0x61,
0x62, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d,
0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x41, 0x72, 0x61, 0x62, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d,
0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x42, 0x61, 0x67, 0x69,
0x61, 0x6e, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x42, 0x61, 0x67, 0x69,
0x61, 0x6e, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61,
0x73, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20,
0x42, 0x61, 0x67, 0x69, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61,
0x6e, 0x61, 0x73, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50,
0x61, 0x6e, 0x61, 0x73, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61,
0x68, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d,
0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x54,
0x65, 0x6e, 0x67, 0x61, 0x68, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x42, 0x61, 0x72,
0x61, 0x74, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x42, 0x61, 0x72,
0x61, 0x74, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61,
0x68, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x54, 0x69,
0x6d, 0x75, 0x72, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d,
0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x42, 0x61, 0x72,
0x61, 0x74, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20,
0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61,
0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61,
0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d,
0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62,
0x61, 0x69, 0x6a, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65,
0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d,
0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65,
0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x73, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x73, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d,
0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20,
0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73,
0x73, 0x61, 0x6c, 0x61, 0x6d, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x54,
0x61, 0x6e, 0x6a, 0x75, 0x6e, 0x67, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x54, 0x61, 0x6e, 0x6a, 0x75, 0x6e, 0x67, 0x20, 0x56, 0x65,
0x72, 0x64, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73,
0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x54, 0x61, 0x6e,
0x6a, 0x75, 0x6e, 0x67, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x43, 0x61, 0x73, 0x65, 0x79, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x43,
0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d,
0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x43,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x43, 0x69, 0x6c, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x43, 0x69, 0x6c, 0x65, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61,
0x6e, 0x61, 0x73, 0x20, 0x43, 0x69, 0x6c, 0x65, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x54, 0x69, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6b, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x54, 0x69, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6b, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61,
0x73, 0x20, 0x54, 0x69, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6b, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x50, 0x75, 0x6c, 0x61, 0x75, 0x20, 0x4e, 0x61,
0x74, 0x61, 0x6c, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x70,
0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20,
0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x70, 0x2e,
0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4b, 0x65, 0x70, 0x2e, 0x20,
0x43, 0x6f, 0x6f, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x65,
0x6e, 0x67, 0x61, 0x68, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50,
0x61, 0x6e, 0x61, 0x73, 0x20, 0x4b, 0x65, 0x70, 0x2e, 0x20, 0x43, 0x6f,
0x6f, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x75, 0x62, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20,
0x4b, 0x75, 0x62, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x44, 0x61,
0x76, 0x69, 0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x44, 0x75, 0x6d,
0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c,
0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72,
0x20, 0x4c, 0x65, 0x73, 0x74, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x75, 0x6c, 0x61, 0x75, 0x20, 0x50, 0x61, 0x73, 0x6b, 0x61, 0x68,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x50, 0x75, 0x6c, 0x61, 0x75, 0x20, 0x50, 0x61, 0x73, 0x6b,
0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69,
0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x50, 0x75, 0x6c, 0x61,
0x75, 0x20, 0x50, 0x61, 0x73, 0x6b, 0x61, 0x68, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x45, 0x6b, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x54, 0x65, 0x6e,
0x67, 0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x54,
0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d,
0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x45,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x54,
0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d,
0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x45,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x54, 0x69,
0x6d, 0x75, 0x72, 0x20, 0x4a, 0x61, 0x75, 0x68, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x42, 0x61, 0x72, 0x61,
0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x42, 0x61, 0x72,
0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69,
0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x45, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20, 0x46,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4b, 0x65, 0x70,
0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73,
0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x4b, 0x65, 0x70,
0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x46, 0x69, 0x6a,
0x69, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x46, 0x69, 0x6a, 0x69, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x47,
0x75, 0x79, 0x61, 0x6e, 0x61, 0x20, 0x50, 0x72, 0x61, 0x6e, 0x63, 0x69,
0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x57, 0x69, 0x6c, 0x61, 0x79,
0x61, 0x68, 0x20, 0x53, 0x65, 0x6c, 0x61, 0x74, 0x61, 0x6e, 0x20, 0x64,
0x61, 0x6e, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x6b,
0x61, 0x20, 0x50, 0x72, 0x61, 0x6e, 0x63, 0x69, 0x73, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65,
0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50,
0x61, 0x6e, 0x61, 0x73, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x70, 0x2e, 0x20, 0x47,
0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61,
0x73, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x42, 0x61, 0x72,
0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x42,
0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x47, 0x75,
0x61, 0x6d, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x54, 0x65, 0x6c, 0x75, 0x6b, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x48, 0x61, 0x77, 0x61,
0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50,
0x61, 0x6e, 0x61, 0x73, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61,
0x6e, 0x61, 0x73, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e,
0x67, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x48, 0x6f, 0x76, 0x64, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d,
0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x48,
0x6f, 0x76, 0x64, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x49, 0x6e, 0x64,
0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x61, 0x6d, 0x75,
0x64, 0x65, 0x72, 0x61, 0x20, 0x48, 0x69, 0x6e, 0x64, 0x69, 0x61, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69,
0x6e, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x49, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x73, 0x69, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61,
0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x49, 0x72, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61,
0x6e, 0x61, 0x73, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x49,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x49, 0x73, 0x72,
0x61, 0x65, 0x6c, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73,
0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x49, 0x73, 0x72,
0x61, 0x65, 0x6c, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4a, 0x65, 0x70,
0x61, 0x6e, 0x67, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x4a, 0x65, 0x70, 0x61, 0x6e, 0x67, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50,
0x61, 0x6e, 0x61, 0x73, 0x20, 0x4a, 0x65, 0x70, 0x61, 0x6e, 0x67, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61,
0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68,
0x61, 0x74, 0x73, 0x6b, 0x79, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x79, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76,
0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b,
0x69, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b,
0x68, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x69,
0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x61, 0x7a,
0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x72, 0x61,
0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20,
0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x79, 0x61, 0x72, 0x73, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x69, 0x72, 0x67, 0x69, 0x7a, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x65, 0x70, 0x2e, 0x20, 0x4c, 0x69, 0x6e, 0x65, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d,
0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x61,
0x6b, 0x61, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x4d, 0x61, 0x6b, 0x61, 0x75, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61,
0x6e, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6b, 0x61, 0x75, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4d, 0x61, 0x6c, 0x61, 0x79, 0x73, 0x69, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x6c, 0x61, 0x64, 0x65, 0x77, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65,
0x73, 0x61, 0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x70,
0x2e, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20,
0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74,
0x69, 0x75, 0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x77,
0x73, 0x6f, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x61, 0x73,
0x69, 0x66, 0x69, 0x6b, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x4d, 0x65,
0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d,
0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x50,
0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69,
0x6b, 0x6f, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x55, 0x6c, 0x61, 0x6e,
0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x55, 0x6c, 0x61, 0x6e,
0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20,
0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x77, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x77, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20,
0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x77, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4e, 0x65, 0x70, 0x61, 0x6c, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x42, 0x61, 0x72,
0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61,
0x20, 0x42, 0x61, 0x72, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d,
0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x4b,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x42, 0x61, 0x72,
0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x65, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x61, 0x20, 0x42, 0x61, 0x72, 0x75, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x53, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x42, 0x61, 0x72, 0x75, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50,
0x61, 0x6e, 0x61, 0x73, 0x20, 0x53, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x61, 0x20, 0x42, 0x61, 0x72, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69,
0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x4e, 0x65, 0x77, 0x66,
0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x75, 0x6c, 0x61, 0x75, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x50, 0x75, 0x6c, 0x61, 0x75, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75,
0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x50, 0x75,
0x6c, 0x61, 0x75, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e,
0x61, 0x73, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x70, 0x2e, 0x20, 0x4d, 0x61, 0x72,
0x69, 0x61, 0x6e, 0x61, 0x20, 0x55, 0x74, 0x61, 0x72, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75,
0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4f, 0x6d, 0x73,
0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d,
0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50,
0x61, 0x6e, 0x61, 0x73, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20,
0x4e, 0x75, 0x67, 0x69, 0x6e, 0x69, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x50, 0x65, 0x72, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x50, 0x65, 0x72, 0x75,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20,
0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x50, 0x65, 0x72, 0x75, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61,
0x6e, 0x61, 0x73, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x61,
0x75, 0x61, 0x6e, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x64, 0x61, 0x6e, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x53, 0x61, 0x69, 0x6e,
0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x64, 0x61, 0x6e,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e,
0x61, 0x73, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x64, 0x61, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69,
0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50,
0x61, 0x6e, 0x61, 0x73, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72,
0x64, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x52, 0x65, 0x75, 0x6e,
0x69, 0x6f, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x52, 0x6f, 0x74,
0x68, 0x65, 0x72, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x61,
0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75,
0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x53, 0x61,
0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69,
0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x61, 0x6d, 0x6f,
0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61,
0x73, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x61, 0x75, 0x61,
0x6e, 0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x53,
0x65, 0x6c, 0x61, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x54, 0x61, 0x69,
0x70, 0x65, 0x69, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73,
0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x54, 0x61, 0x69,
0x70, 0x65, 0x69, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x61, 0x6a,
0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69,
0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x54, 0x6f, 0x6e, 0x67,
0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61,
0x73, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x75, 0x76, 0x61,
0x6c, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x55, 0x72, 0x75, 0x67,
0x75, 0x61, 0x79, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20,
0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61,
0x79, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d,
0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x55,
0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d,
0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20,
0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x56, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x70,
0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20,
0x64, 0x61, 0x6e, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61,
0x73, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x59, 0x75, 0x6b, 0x6f,
0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x66, 0x67,
0x68, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x45, 0x73, 0x74, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x66, 0x72,
0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x53, 0x75, 0x64, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x57, 0x65, 0x73, 0x74, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x57, 0x65, 0x73, 0x74, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41, 0x66, 0x72,
0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x57, 0x65, 0x73, 0x74,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64,
0x65, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x69, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x69,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65,
0x20, 0x64, 0x65, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x69, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x65,
0x73, 0x74, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x65, 0x73, 0x74, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x65, 0x73, 0x74, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x69, 0x61, 0x73, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x69, 0x61, 0x73, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x69, 0x61, 0x73,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20,
0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66,
0x69, 0x63, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64,
0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x61,
0x72, 0x61, 0x62, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x72, 0x61, 0x62, 0x65, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x61,
0x72, 0x61, 0x62, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64,
0x65, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69,
0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x63, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x61, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x6f, 0x2d, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x2d, 0x6f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x6f, 0x2d, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69,
0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x41, 0x7a, 0x65, 0x72, 0x62, 0x65, 0x69, 0x64, 0x7a, 0x68, 0x61, 0x6e,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65, 0x69,
0x64, 0x7a, 0x68, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x65, 0x69, 0x64, 0x7a, 0x68, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65,
0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x68, 0x75, 0x74, 0x61,
0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c,
0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76,
0x65, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x70,
0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x43, 0x61, 0x70, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65,
0x20, 0x43, 0x61, 0x70, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20,
0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x43,
0x68, 0x69, 0x6c, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x43, 0x68, 0x69, 0x6e, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68,
0x69, 0x6e, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69,
0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75,
0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x61, 0x74, 0x61, 0x6c, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75,
0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65,
0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61,
0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20,
0x43, 0x6f, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64,
0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x45, 0x73, 0x74, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61,
0x73, 0x63, 0x68, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e,
0x73, 0x75, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63,
0x68, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76,
0x65, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61,
0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x68, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65,
0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64,
0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x75, 0x6c, 0x74, 0x72,
0x61, 0x2d, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64,
0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e,
0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76,
0x65, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61,
0x73, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69,
0x6a, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x75,
0x69, 0x61, 0x6e, 0x61, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73,
0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x66, 0x72, 0x61, 0x6e, 0x63, 0x65,
0x73, 0x65, 0x20, 0x6d, 0x65, 0x72, 0x69, 0x64, 0x69, 0x6f, 0x6e, 0x61,
0x6c, 0x20, 0x65, 0x20, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x63, 0x74, 0x69,
0x63, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x47, 0x61,
0x6c, 0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65,
0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x73,
0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x6d, 0x65, 0x64, 0x69, 0x65, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64,
0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65,
0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61,
0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x47,
0x6f, 0x6c, 0x66, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65,
0x75, 0x74, 0x69, 0x61, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61,
0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61,
0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69,
0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x61, 0x73, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x6b,
0x6f, 0x6e, 0x67, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67,
0x6b, 0x6f, 0x6e, 0x67, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x6b,
0x6f, 0x6e, 0x67, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20,
0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x4f, 0x63, 0x65, 0x61, 0x6e, 0x6f, 0x20, 0x49, 0x6e, 0x64,
0x69, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x45, 0x73, 0x74, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x57, 0x65, 0x73, 0x74, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x49,
0x72, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69,
0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65,
0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x49,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x49,
0x73, 0x72, 0x61, 0x65, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4a, 0x61,
0x70, 0x6f, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4a, 0x61, 0x70, 0x6f,
0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65,
0x20, 0x64, 0x65, 0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73,
0x74, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b,
0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x45, 0x73, 0x74, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x57, 0x65, 0x73, 0x74, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x43, 0x6f, 0x72, 0x65, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x72,
0x65, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64,
0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73,
0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x69, 0x72,
0x67, 0x68, 0x69, 0x7a, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61,
0x73, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4c, 0x69, 0x6e, 0x65, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65,
0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x6c, 0x61, 0x79, 0x73, 0x69,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4d, 0x61,
0x6c, 0x64, 0x69, 0x76, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75,
0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x74, 0x69, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x74, 0x69, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x74, 0x69, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x6d, 0x65,
0x78, 0x69, 0x63, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63,
0x69, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x6d, 0x65, 0x78, 0x69, 0x63, 0x61,
0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63,
0x6f, 0x20, 0x6d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x6e,
0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0x61,
0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f,
0x73, 0x63, 0x6f, 0x76, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63,
0x6f, 0x76, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x76, 0x65, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x65, 0x20, 0x43,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x76, 0x65, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76,
0x65, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x76, 0x65, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e,
0x64, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76,
0x65, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x65, 0x20, 0x5a, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65,
0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65,
0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c,
0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73,
0x75, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64,
0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4f,
0x6d, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x6f, 0x76,
0x65, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65,
0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x72, 0x75,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x72, 0x75, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20,
0x50, 0x65, 0x72, 0x75, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x61, 0x73,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70,
0x70, 0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x68, 0x69,
0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x73,
0x20, 0x50, 0x68, 0x65, 0x6e, 0x69, 0x63, 0x65, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x69, 0x6e,
0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4d,
0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x63, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x63, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69,
0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c,
0x6c, 0x65, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x53, 0x69, 0x6e, 0x67,
0x61, 0x70, 0x6f, 0x72, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x53, 0x6f,
0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x53, 0x75, 0x64, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61,
0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x75,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69,
0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20,
0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x57,
0x61, 0x6b, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x59,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65,
0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65,
0x20, 0x64, 0x65, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x61, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x6e, 0x20,
0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x61, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x74,
0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x61, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x6e,
0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61, 0x6c, 0x20, 0x74, 0xe9, 0x6d,
0x70, 0x6f, 0x72, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x74, 0xe9, 0x6d, 0x70,
0x6f, 0x72, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61, 0x6c, 0x20, 0x74,
0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61,
0x6e, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x74,
0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61,
0x6e, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x74, 0xe9, 0x6d, 0x70,
0x6f, 0x72, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x6f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x65, 0x73, 0x74, 0x69,
0x76, 0x61, 0x6c, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61,
0x6e, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x63, 0x61, 0x6e, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x74, 0xe9, 0x6d,
0x70, 0x6f, 0x72, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20,
0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76,
0x61, 0x6c, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69,
0x63, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x63, 0x61, 0x6e, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x74, 0xe9,
0x6d, 0x70, 0x6f, 0x72, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e,
0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x65, 0x73, 0x74,
0x69, 0x76, 0x61, 0x6c, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x61, 0x74, 0x6c, 0x61,
0x6e, 0x74, 0x69, 0x63, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x61, 0x74, 0x6c, 0x61,
0x6e, 0x74, 0x69, 0x63, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x63, 0x61, 0x6e, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x63, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61, 0x6c, 0x20, 0x74, 0xe9,
0x6d, 0x70, 0x6f, 0x72, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x74, 0xe9, 0x6d,
0x70, 0x6f, 0x72, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x74,
0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61,
0x6c, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73,
0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49,
0x6e, 0x73, 0x75, 0x6c, 0x20, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d,
0x61, 0x73, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x63,
0x6f, 0x73, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f,
0x6b, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73,
0x75, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x74, 0xe9, 0x6d,
0x70, 0x6f, 0x72, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61, 0x6c, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x73, 0x20,
0x43, 0x6f, 0x6f, 0x6b, 0x63, 0x75, 0x62, 0x61, 0x6e, 0x20, 0x74, 0xe9,
0x6d, 0x70, 0x6f, 0x72, 0x63, 0x75, 0x62, 0x61, 0x6e, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f,
0x72, 0x63, 0x75, 0x62, 0x61, 0x6e, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76,
0x61, 0x6c, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x20, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x6e,
0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x20, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x6e, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x74, 0xe9, 0x6d, 0x70,
0x6f, 0x72, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x65, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x6e, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61,
0x6c, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x6f, 0x73, 0x74, 0x2d,
0x65, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x6e, 0x20, 0x74, 0xe9, 0x6d, 0x70,
0x6f, 0x72, 0x6f, 0x73, 0x74, 0x2d, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x74,
0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x6f, 0x73, 0x74, 0x2d, 0x65, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x6e, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61, 0x6c,
0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x77, 0x65, 0x73, 0x74, 0x2d,
0x65, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x6e, 0x20, 0x74, 0xe9, 0x6d, 0x70,
0x6f, 0x72, 0x77, 0x65, 0x73, 0x74, 0x2d, 0x65, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x77, 0x65, 0x73, 0x74, 0x2d, 0x65,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x6e, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76,
0x61, 0x6c, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x74, 0xe9, 0x6d,
0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75,
0x6c, 0x65, 0x73, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75,
0x6c, 0x65, 0x73, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76,
0x61, 0x6c, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c,
0x65, 0x73, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x74,
0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e,
0x73, 0x75, 0x6c, 0x65, 0x73, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72,
0x74, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x65, 0x20, 0x6d, 0x65, 0x64,
0x69, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x77, 0x69, 0x63, 0x68, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x74, 0xe9, 0x6d,
0x70, 0x6f, 0x72, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x74,
0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61,
0x6c, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x74, 0xe9, 0x6d, 0x70, 0x6f,
0x72, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x74,
0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61,
0x6c, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x20,
0x48, 0x61, 0x77, 0x61, 0x69, 0x20, 0x65, 0x20, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x65, 0x73, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61,
0x77, 0x61, 0x69, 0x20, 0x65, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x65,
0x73, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x65, 0x73, 0x74, 0x69,
0x76, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69,
0x20, 0x65, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x65, 0x73, 0x74, 0xe9,
0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73,
0x75, 0x6c, 0x65, 0x73, 0x20, 0x4c, 0x69, 0x6e, 0x65, 0x74, 0xe9, 0x6d,
0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75,
0x6c, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48, 0x6f, 0x77, 0x65, 0x74,
0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c,
0x20, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48, 0x6f, 0x77, 0x65, 0x74, 0xe9,
0x6d, 0x70, 0x6f, 0x72, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61, 0x6c,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x20, 0x4c,
0x6f, 0x72, 0x64, 0x2d, 0x48, 0x6f, 0x77, 0x65, 0x74, 0xe9, 0x6d, 0x70,
0x6f, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75,
0x65, 0x73, 0x61, 0x73, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x73, 0x20, 0x4d,
0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x6d, 0x65, 0x78, 0x69, 0x63,
0x61, 0x6e, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x74,
0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x6d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e,
0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72,
0x6d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x70, 0x61, 0x63, 0x69,
0x66, 0x69, 0x63, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61, 0x6c, 0x20,
0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x74,
0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61,
0x6c, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e,
0x64, 0x6c, 0x61, 0x6e, 0x64, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x65, 0x73, 0x74,
0x69, 0x76, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73,
0x75, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x74, 0xe9,
0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73,
0x75, 0x6c, 0x65, 0x73, 0x20, 0x46, 0xe9, 0x6e, 0x69, 0x78, 0x74, 0xe9,
0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73,
0x75, 0x6c, 0x65, 0x73, 0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e,
0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49,
0x6e, 0x73, 0x75, 0x6c, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x74, 0xe9, 0x6d,
0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e,
0x41, 0x6d, 0x20, 0x41, 0x63, 0x72, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61,
0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x41,
0x63, 0x72, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x41, 0x63, 0x72, 0x65, 0x41, 0x6d, 0x20, 0x6e,
0x61, 0x20, 0x68, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x61, 0x73, 0x74, 0xe1,
0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x66,
0x72, 0x61, 0x69, 0x63, 0x65, 0x20, 0x4c, 0xe1, 0x69, 0x72, 0x41, 0x6d,
0x20, 0x4f, 0x69, 0x72, 0x74, 0x68, 0x65, 0x61, 0x72, 0x20, 0x6e, 0x61,
0x20, 0x68, 0x41, 0x66, 0x72, 0x61, 0x69, 0x63, 0x65, 0x41, 0x6d, 0x20,
0x6e, 0x61, 0x20, 0x68, 0x41, 0x66, 0x72, 0x61, 0x69, 0x63, 0x65, 0x20,
0x54, 0x68, 0x65, 0x61, 0x73, 0x41, 0x6d, 0x20, 0x49, 0x61, 0x72, 0x74,
0x68, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x66, 0x72, 0x61,
0x69, 0x63, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64,
0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x49, 0x61, 0x72, 0x74, 0x68,
0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x66, 0x72, 0x61, 0x69,
0x63, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x49, 0x61, 0x72, 0x74, 0x68, 0x61, 0x72, 0x20, 0x6e,
0x61, 0x20, 0x68, 0x41, 0x66, 0x72, 0x61, 0x69, 0x63, 0x65, 0x41, 0x6d,
0x20, 0x41, 0x6c, 0x61, 0x73, 0x63, 0x61, 0x41, 0x6d, 0x20, 0x43, 0x61,
0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x41,
0x6c, 0x61, 0x73, 0x63, 0x61, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x63, 0x61,
0x41, 0x6d, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x41, 0x6d, 0x20,
0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68,
0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x41, 0x6d, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41, 0x6c, 0x6d, 0x61,
0x74, 0x79, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x6d, 0x61,
0x73, 0xf3, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67,
0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20,
0x68, 0x41, 0x6d, 0x61, 0x73, 0xf3, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20,
0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61,
0x20, 0x68, 0x41, 0x6d, 0x61, 0x73, 0xf3, 0x69, 0x6e, 0x65, 0x41, 0x6d,
0x20, 0x4c, 0xe1, 0x72, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4d, 0x68, 0x65,
0x69, 0x72, 0x69, 0x63, 0x65, 0xe1, 0x20, 0x54, 0x68, 0x75, 0x61, 0x69,
0x64, 0x68, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65,
0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4c, 0xe1, 0x72, 0x6e, 0x61, 0x63,
0x68, 0x20, 0x4d, 0x68, 0x65, 0x69, 0x72, 0x69, 0x63, 0x65, 0xe1, 0x20,
0x54, 0x68, 0x75, 0x61, 0x69, 0x64, 0x68, 0x41, 0x6d, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4c, 0xe1, 0x72, 0x6e,
0x61, 0x63, 0x68, 0x20, 0x4d, 0x68, 0x65, 0x69, 0x72, 0x69, 0x63, 0x65,
0xe1, 0x20, 0x54, 0x68, 0x75, 0x61, 0x69, 0x64, 0x68, 0x41, 0x6d, 0x20,
0x4f, 0x69, 0x72, 0x74, 0x68, 0x65, 0x61, 0x72, 0x61, 0x63, 0x68, 0x20,
0x4d, 0x68, 0x65, 0x69, 0x72, 0x69, 0x63, 0x65, 0xe1, 0x20, 0x54, 0x68,
0x75, 0x61, 0x69, 0x64, 0x68, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67,
0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4f, 0x69, 0x72,
0x74, 0x68, 0x65, 0x61, 0x72, 0x61, 0x63, 0x68, 0x20, 0x4d, 0x68, 0x65,
0x69, 0x72, 0x69, 0x63, 0x65, 0xe1, 0x20, 0x54, 0x68, 0x75, 0x61, 0x69,
0x64, 0x68, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x4f, 0x69, 0x72, 0x74, 0x68, 0x65, 0x61, 0x72, 0x61,
0x63, 0x68, 0x20, 0x4d, 0x68, 0x65, 0x69, 0x72, 0x69, 0x63, 0x65, 0xe1,
0x20, 0x54, 0x68, 0x75, 0x61, 0x69, 0x64, 0x68, 0x41, 0x6d, 0x20, 0x53,
0x6c, 0xe9, 0x69, 0x62, 0x68, 0x74, 0x65, 0x20, 0x4d, 0x68, 0x65, 0x69,
0x72, 0x69, 0x63, 0x65, 0xe1, 0x20, 0x54, 0x68, 0x75, 0x61, 0x69, 0x64,
0x68, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1,
0x6e, 0x61, 0x63, 0x68, 0x20, 0x53, 0x6c, 0xe9, 0x69, 0x62, 0x68, 0x74,
0x65, 0x20, 0x4d, 0x68, 0x65, 0x69, 0x72, 0x69, 0x63, 0x65, 0xe1, 0x20,
0x54, 0x68, 0x75, 0x61, 0x69, 0x64, 0x68, 0x41, 0x6d, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x53, 0x6c, 0xe9, 0x69,
0x62, 0x68, 0x74, 0x65, 0x20, 0x4d, 0x68, 0x65, 0x69, 0x72, 0x69, 0x63,
0x65, 0xe1, 0x20, 0x54, 0x68, 0x75, 0x61, 0x69, 0x64, 0x68, 0x41, 0x6d,
0x20, 0x61, 0x6e, 0x20, 0x41, 0x69, 0x67, 0xe9, 0x69, 0x6e, 0x20, 0x43,
0x68, 0x69, 0xfa, 0x69, 0x6e, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67,
0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x61, 0x6e, 0x20,
0x41, 0x69, 0x67, 0xe9, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x69, 0xfa, 0x69,
0x6e, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x69, 0x67, 0xe9, 0x69, 0x6e, 0x20,
0x43, 0x68, 0x69, 0xfa, 0x69, 0x6e, 0x41, 0x6d, 0x20, 0x41, 0x6e, 0x61,
0x64, 0x79, 0x72, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64,
0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79,
0x72, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x41, 0x6d, 0x20, 0x41,
0x70, 0x69, 0x61, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64,
0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x41, 0x70, 0x69, 0x61, 0x41,
0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x41, 0x70, 0x69, 0x61, 0x41, 0x6d, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75,
0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e,
0x61, 0x63, 0x68, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x41, 0x6d, 0x20,
0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41, 0x71,
0x74, 0x61, 0x75, 0x41, 0x6d, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65,
0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e,
0x61, 0x63, 0x68, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x41, 0x6d,
0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41,
0x71, 0x74, 0x6f, 0x62, 0x65, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x68,
0x41, 0x72, 0x61, 0x69, 0x62, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69,
0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61,
0x20, 0x68, 0x41, 0x72, 0x61, 0x69, 0x62, 0x65, 0x41, 0x6d, 0x20, 0x53,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20,
0x68, 0x41, 0x72, 0x61, 0x69, 0x62, 0x65, 0x41, 0x6d, 0x20, 0x6e, 0x61,
0x20, 0x68, 0x41, 0x69, 0x72, 0x67, 0x69, 0x6e, 0x74, 0xed, 0x6e, 0x65,
0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e,
0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x69, 0x72, 0x67,
0x69, 0x6e, 0x74, 0xed, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41,
0x69, 0x72, 0x67, 0x69, 0x6e, 0x74, 0xed, 0x6e, 0x65, 0x41, 0x6d, 0x20,
0x49, 0x61, 0x72, 0x74, 0x68, 0x61, 0x72, 0x61, 0x63, 0x68, 0x20, 0x6e,
0x61, 0x20, 0x68, 0x41, 0x69, 0x72, 0x67, 0x69, 0x6e, 0x74, 0xed, 0x6e,
0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1,
0x6e, 0x61, 0x63, 0x68, 0x20, 0x49, 0x61, 0x72, 0x74, 0x68, 0x61, 0x72,
0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x69, 0x72, 0x67,
0x69, 0x6e, 0x74, 0xed, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x49, 0x61, 0x72, 0x74, 0x68,
0x61, 0x72, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x69,
0x72, 0x67, 0x69, 0x6e, 0x74, 0xed, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x6e,
0x61, 0x20, 0x68, 0x41, 0x69, 0x72, 0x6d, 0xe9, 0x69, 0x6e, 0x65, 0x41,
0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61,
0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x69, 0x72, 0x6d, 0xe9,
0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x69, 0x72, 0x6d,
0xe9, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x61, 0x69, 0x67, 0x68, 0x41, 0x6d, 0x20, 0x43,
0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20,
0x61, 0x6e, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x61, 0x69, 0x67,
0x68, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x61,
0x69, 0x67, 0x68, 0x41, 0x6d, 0x20, 0x4c, 0xe1, 0x72, 0x20, 0x6e, 0x61,
0x20, 0x68, 0x41, 0x73, 0x74, 0x72, 0xe1, 0x69, 0x6c, 0x65, 0x41, 0x6d,
0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63,
0x68, 0x20, 0x4c, 0xe1, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x73,
0x74, 0x72, 0xe1, 0x69, 0x6c, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4c, 0xe1, 0x72, 0x20, 0x6e,
0x61, 0x20, 0x68, 0x41, 0x73, 0x74, 0x72, 0xe1, 0x69, 0x6c, 0x65, 0x41,
0x6d, 0x20, 0x4d, 0x68, 0x65, 0xe1, 0x6e, 0x69, 0x61, 0x72, 0x74, 0x68,
0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x73, 0x74, 0x72, 0xe1,
0x69, 0x6c, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64,
0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4d, 0x68, 0x65, 0xe1, 0x6e,
0x69, 0x61, 0x72, 0x74, 0x68, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68,
0x41, 0x73, 0x74, 0x72, 0xe1, 0x69, 0x6c, 0x65, 0x41, 0x6d, 0x20, 0x53,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4d, 0x68, 0x65,
0xe1, 0x6e, 0x69, 0x61, 0x72, 0x74, 0x68, 0x61, 0x72, 0x20, 0x6e, 0x61,
0x20, 0x68, 0x41, 0x73, 0x74, 0x72, 0xe1, 0x69, 0x6c, 0x65, 0x41, 0x6d,
0x20, 0x4f, 0x69, 0x72, 0x74, 0x68, 0x65, 0x61, 0x72, 0x20, 0x6e, 0x61,
0x20, 0x68, 0x41, 0x73, 0x74, 0x72, 0xe1, 0x69, 0x6c, 0x65, 0x41, 0x6d,
0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63,
0x68, 0x20, 0x4f, 0x69, 0x72, 0x74, 0x68, 0x65, 0x61, 0x72, 0x20, 0x6e,
0x61, 0x20, 0x68, 0x41, 0x73, 0x74, 0x72, 0xe1, 0x69, 0x6c, 0x65, 0x41,
0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x4f, 0x69, 0x72, 0x74, 0x68, 0x65, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20,
0x68, 0x41, 0x73, 0x74, 0x72, 0xe1, 0x69, 0x6c, 0x65, 0x41, 0x6d, 0x20,
0x49, 0x61, 0x72, 0x74, 0x68, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68,
0x41, 0x73, 0x74, 0x72, 0xe1, 0x69, 0x6c, 0x65, 0x41, 0x6d, 0x20, 0x43,
0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20,
0x49, 0x61, 0x72, 0x74, 0x68, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68,
0x41, 0x73, 0x74, 0x72, 0xe1, 0x69, 0x6c, 0x65, 0x41, 0x6d, 0x20, 0x53,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x49, 0x61, 0x72,
0x74, 0x68, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x73, 0x74,
0x72, 0xe1, 0x69, 0x6c, 0x65, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x68,
0x41, 0x73, 0x61, 0x72, 0x62, 0x61, 0x69, 0x73, 0x65, 0xe1, 0x69, 0x6e,
0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1,
0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x73, 0x61,
0x72, 0x62, 0x61, 0x69, 0x73, 0x65, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d,
0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e,
0x61, 0x20, 0x68, 0x41, 0x73, 0x61, 0x72, 0x62, 0x61, 0x69, 0x73, 0x65,
0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x6e, 0x41,
0x73, 0xf3, 0x72, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64,
0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x6e, 0x41,
0x73, 0xf3, 0x72, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x6e, 0x41, 0x73, 0xf3, 0x72,
0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x69, 0x64, 0xe9, 0x69, 0x73, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69,
0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61,
0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x69, 0x64, 0xe9, 0x69, 0x73,
0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x6e, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x69,
0x64, 0xe9, 0x69, 0x73, 0x65, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x42,
0xfa, 0x74, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20,
0x42, 0x6f, 0x6c, 0x61, 0x69, 0x76, 0x65, 0x41, 0x6d, 0x20, 0x42, 0x68,
0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x41, 0x6d, 0x20, 0x43, 0x61,
0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x42,
0x68, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x41, 0x6d, 0x20, 0x53,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x42, 0x68, 0x72,
0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x41, 0x6d, 0x20, 0x42, 0x68, 0x72,
0xfa, 0x69, 0x6e, 0xe9, 0x20, 0x44, 0x61, 0x72, 0x61, 0x73, 0x61, 0x6c,
0xe1, 0x6d, 0x41, 0x6d, 0x20, 0x52, 0x69, 0x6e, 0x6e, 0x20, 0x56, 0x65,
0x72, 0x64, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64,
0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x52, 0x69, 0x6e, 0x6e, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x52, 0x69, 0x6e, 0x6e, 0x20, 0x56,
0x65, 0x72, 0x64, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x74, 0xe1, 0x69, 0x73,
0x69, 0xfa, 0x6e, 0x20, 0x43, 0x61, 0x73, 0x65, 0x79, 0x41, 0x6d, 0x20,
0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68,
0x20, 0x53, 0x65, 0x61, 0x6d, 0xf3, 0x72, 0x61, 0x63, 0x68, 0x41, 0x6d,
0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x41, 0x6d, 0x20, 0x43,
0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x41, 0x6d, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x53, 0x69, 0x6c,
0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1,
0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x53, 0x69, 0x6c, 0x65,
0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x6e, 0x61, 0x20, 0x53, 0x69, 0x6c, 0x65, 0x41, 0x6d, 0x20, 0x6e,
0x61, 0x20, 0x53, 0xed, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69,
0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61,
0x20, 0x53, 0xed, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x53, 0xed, 0x6e,
0x65, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x6e,
0x61, 0x20, 0x4e, 0x6f, 0x6c, 0x6c, 0x61, 0x67, 0x41, 0x6d, 0x20, 0x4f,
0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73,
0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0xf3, 0x69, 0x6d,
0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1,
0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0xf3,
0x69, 0x6d, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0xf3, 0x69,
0x6d, 0x65, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e,
0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67,
0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4f, 0x69, 0x6c,
0x65, 0xe1, 0x69, 0x6e, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x41, 0x6d, 0x20,
0x4c, 0x65, 0x61, 0x74, 0x68, 0x73, 0x68, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e, 0x20,
0x43, 0x6f, 0x6f, 0x6b, 0x41, 0x6d, 0x20, 0x43, 0x68, 0xfa, 0x62, 0x61,
0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e,
0x61, 0x63, 0x68, 0x20, 0x43, 0x68, 0xfa, 0x62, 0x61, 0x41, 0x6d, 0x20,
0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x43, 0x68,
0xfa, 0x62, 0x61, 0x41, 0x6d, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x41,
0x6d, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55,
0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x41, 0x6d, 0x20, 0x54, 0x68, 0xed,
0x6f, 0x6d, 0xf3, 0x72, 0x20, 0x54, 0x68, 0x6f, 0x69, 0x72, 0x41, 0x6d,
0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x6e, 0x61, 0x20, 0x43,
0xe1, 0x73, 0x63, 0x61, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68,
0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4f, 0x69, 0x6c, 0x65,
0xe1, 0x6e, 0x20, 0x6e, 0x61, 0x20, 0x43, 0xe1, 0x73, 0x63, 0x61, 0x41,
0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x6e, 0x61, 0x20, 0x43, 0xe1,
0x73, 0x63, 0x61, 0x41, 0x6d, 0x20, 0x45, 0x61, 0x63, 0x75, 0x61, 0x64,
0xf3, 0x72, 0x41, 0x6d, 0x20, 0x4c, 0xe1, 0x72, 0x20, 0x6e, 0x61, 0x20,
0x68, 0x45, 0x6f, 0x72, 0x70, 0x61, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69,
0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4c, 0xe1,
0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x45, 0x6f, 0x72, 0x70, 0x61, 0x41,
0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x4c, 0xe1, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x45, 0x6f, 0x72, 0x70,
0x61, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x72, 0x74, 0x68, 0x65, 0x61, 0x72,
0x20, 0x6e, 0x61, 0x20, 0x68, 0x45, 0x6f, 0x72, 0x70, 0x61, 0x41, 0x6d,
0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63,
0x68, 0x20, 0x4f, 0x69, 0x72, 0x74, 0x68, 0x65, 0x61, 0x72, 0x20, 0x6e,
0x61, 0x20, 0x68, 0x45, 0x6f, 0x72, 0x70, 0x61, 0x41, 0x6d, 0x20, 0x53,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4f, 0x69, 0x72,
0x74, 0x68, 0x65, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x45, 0x6f,
0x72, 0x70, 0x61, 0x41, 0x6d, 0x20, 0x43, 0x68, 0x69, 0x61, 0x6e, 0x6f,
0x69, 0x72, 0x74, 0x68, 0x65, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68,
0x45, 0x6f, 0x72, 0x70, 0x61, 0x41, 0x6d, 0x20, 0x49, 0x61, 0x72, 0x74,
0x68, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x45, 0x6f, 0x72, 0x70,
0x61, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1,
0x6e, 0x61, 0x63, 0x68, 0x20, 0x49, 0x61, 0x72, 0x74, 0x68, 0x61, 0x72,
0x20, 0x6e, 0x61, 0x20, 0x68, 0x45, 0x6f, 0x72, 0x70, 0x61, 0x41, 0x6d,
0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x49,
0x61, 0x72, 0x74, 0x68, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x45,
0x6f, 0x72, 0x70, 0x61, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1,
0x69, 0x6e, 0x20, 0x46, 0x68, 0xe1, 0x63, 0x6c, 0x61, 0x69, 0x6e, 0x6e,
0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1,
0x6e, 0x61, 0x63, 0x68, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e,
0x20, 0x46, 0x68, 0xe1, 0x63, 0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x65, 0x41,
0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e, 0x20, 0x46, 0x68, 0xe1, 0x63,
0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x46, 0x68, 0x69,
0x64, 0x73, 0xed, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64,
0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x46, 0x68, 0x69, 0x64, 0x73,
0xed, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x46, 0x68, 0x69, 0x64, 0x73, 0xed, 0x41, 0x6d, 0x20, 0x47,
0x68, 0x75, 0xe1, 0x69, 0x6e, 0x20, 0x6e, 0x61, 0x20, 0x46, 0x72, 0x61,
0x69, 0x6e, 0x63, 0x65, 0x41, 0x6d, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63,
0x61, 0x63, 0x68, 0x20, 0x44, 0x68, 0x65, 0x69, 0x73, 0x63, 0x65, 0x61,
0x72, 0x74, 0x20, 0x61, 0x6e, 0x20, 0x44, 0x6f, 0x6d, 0x68, 0x61, 0x69,
0x6e, 0x20, 0x61, 0x67, 0x75, 0x73, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x6e,
0x74, 0x61, 0x72, 0x74, 0x61, 0x69, 0x67, 0x68, 0x41, 0x6d, 0x20, 0x4f,
0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e, 0x20, 0x47, 0x61, 0x6c, 0xe1, 0x70,
0x61, 0x67, 0x6f, 0x73, 0x41, 0x6d, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69,
0x65, 0x72, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x53, 0x65, 0x6f, 0x69,
0x72, 0x73, 0x69, 0x61, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68,
0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x53,
0x65, 0x6f, 0x69, 0x72, 0x73, 0x69, 0x61, 0x41, 0x6d, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x53,
0x65, 0x6f, 0x69, 0x72, 0x73, 0x69, 0x61, 0x41, 0x6d, 0x20, 0x43, 0x68,
0x69, 0x72, 0x65, 0x61, 0x62, 0x61, 0x69, 0x74, 0xed, 0x4d, 0x65, 0xe1,
0x6e, 0x2d, 0x41, 0x6d, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69,
0x63, 0x68, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x72, 0x74, 0x68, 0x65, 0x61,
0x72, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x72, 0x61, 0x6f, 0x6e, 0x6c, 0x61,
0x69, 0x6e, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68,
0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4f, 0x69, 0x72, 0x74,
0x68, 0x65, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x72, 0x61, 0x6f,
0x6e, 0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4f, 0x69, 0x72, 0x74,
0x68, 0x65, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x72, 0x61, 0x6f,
0x6e, 0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x49, 0x61,
0x72, 0x74, 0x68, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x72, 0x61,
0x6f, 0x6e, 0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x43,
0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20,
0x49, 0x61, 0x72, 0x74, 0x68, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x47,
0x72, 0x61, 0x6f, 0x6e, 0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x65, 0x41, 0x6d,
0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x49,
0x61, 0x72, 0x74, 0x68, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x72,
0x61, 0x6f, 0x6e, 0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x65, 0x41, 0x6d, 0x20,
0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68,
0x20, 0x47, 0x68, 0x75, 0x61, 0x6d, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69,
0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61,
0x20, 0x4d, 0x75, 0x72, 0x61, 0x73, 0x63, 0x61, 0x69, 0x6c, 0x6c, 0x65,
0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x75, 0xe1, 0x69, 0x6e, 0x65,
0x41, 0x6d, 0x20, 0x48, 0x61, 0x76, 0xe1, 0xed, 0x2d, 0x41, 0x69, 0x6c,
0x69, 0xfa, 0x69, 0x74, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68,
0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x48, 0x61, 0x76, 0xe1,
0xed, 0x2d, 0x41, 0x69, 0x6c, 0x69, 0xfa, 0x69, 0x74, 0x41, 0x6d, 0x20,
0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x48, 0x61,
0x76, 0xe1, 0xed, 0x2d, 0x41, 0x69, 0x6c, 0x69, 0xfa, 0x69, 0x74, 0x41,
0x6d, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x43, 0x6f, 0x6e, 0x67, 0x41,
0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61,
0x63, 0x68, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x43, 0x6f, 0x6e, 0x67,
0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x43, 0x6f, 0x6e, 0x67, 0x41, 0x6d,
0x20, 0x48, 0x6f, 0x76, 0x64, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67,
0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x48, 0x6f, 0x76,
0x64, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69,
0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61,
0x20, 0x68, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x41, 0x6d, 0x20, 0x61, 0x6e,
0x20, 0x41, 0x69, 0x67, 0xe9, 0x69, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x69,
0x61, 0x69, 0x67, 0x68, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x49,
0x6e, 0x64, 0x2d, 0x53, 0xed, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x4c, 0xe1,
0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x49, 0x6e, 0x64, 0x69, 0x6e, 0xe9,
0x69, 0x73, 0x65, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x72, 0x74, 0x68, 0x65,
0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x49, 0x6e, 0x64, 0x69, 0x6e,
0xe9, 0x69, 0x73, 0x65, 0x41, 0x6d, 0x20, 0x49, 0x61, 0x72, 0x74, 0x68,
0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x49, 0x6e, 0x64, 0x69, 0x6e,
0xe9, 0x69, 0x73, 0x65, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x49,
0x61, 0x72, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69,
0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61,
0x20, 0x68, 0x49, 0x61, 0x72, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20,
0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61,
0x20, 0x68, 0x49, 0x61, 0x72, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x41, 0x6d, 0x20, 0x43, 0x61,
0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x49,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x41, 0x6d, 0x20, 0x49, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e,
0x61, 0x63, 0x68, 0x20, 0x49, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x41,
0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x49, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x41, 0x6d, 0x20, 0x6e, 0x61,
0x20, 0x53, 0x65, 0x61, 0x70, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20,
0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68,
0x20, 0x6e, 0x61, 0x20, 0x53, 0x65, 0x61, 0x70, 0xe1, 0x69, 0x6e, 0x65,
0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x6e, 0x61, 0x20, 0x53, 0x65, 0x61, 0x70, 0xe1, 0x69, 0x6e, 0x65,
0x41, 0x6d, 0x20, 0x50, 0x68, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76,
0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61,
0x74, 0x73, 0x6b, 0x69, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68,
0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x50, 0x68, 0x65, 0x74,
0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b,
0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x41, 0x6d, 0x20,
0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x50, 0x68,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x41,
0x6d, 0x20, 0x6e, 0x61, 0x20, 0x43, 0x61, 0x73, 0x61, 0x63, 0x73, 0x74,
0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x72, 0x74, 0x68,
0x65, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x43, 0x61, 0x73, 0x61, 0x63,
0x73, 0x74, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x49, 0x61, 0x72,
0x74, 0x68, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x43, 0x61, 0x73, 0x61,
0x63, 0x73, 0x74, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x6e, 0x61,
0x20, 0x43, 0xf3, 0x69, 0x72, 0xe9, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69,
0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61,
0x20, 0x43, 0xf3, 0x69, 0x72, 0xe9, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x43, 0xf3,
0x69, 0x72, 0xe9, 0x41, 0x6d, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65,
0x41, 0x6d, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65,
0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x79, 0x61, 0x72, 0x73, 0x6b, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x79, 0x61, 0x72, 0x73, 0x6b, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x43,
0x69, 0x72, 0x67, 0x65, 0x61, 0x73, 0x74, 0xe1, 0x69, 0x6e, 0x65, 0x41,
0x6d, 0x20, 0x53, 0x68, 0x72, 0xed, 0x20, 0x4c, 0x61, 0x6e, 0x63, 0x61,
0x41, 0x6d, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e, 0x20, 0x6e,
0x61, 0x20, 0x4c, 0xed, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69,
0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x41, 0x6d, 0x20, 0x4d, 0x68, 0x61, 0x63,
0x61, 0x6f, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65,
0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4d, 0x68, 0x61, 0x63, 0x61, 0x6f,
0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x4d, 0x68, 0x61, 0x63, 0x61, 0x6f, 0x41, 0x6d, 0x20, 0x4d, 0x68,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69,
0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4d, 0x68,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4d, 0x68, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x4d, 0x61, 0x6c,
0x61, 0x65, 0x69, 0x73, 0x69, 0x61, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x6c,
0x65, 0xe1, 0x69, 0x6e, 0x20, 0x4d, 0x68, 0x61, 0x69, 0x6c, 0x64, 0xed,
0x76, 0x65, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x6e, 0x4f, 0x69, 0x6c,
0x65, 0xe1, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x63, 0x61, 0x73, 0x61, 0x63,
0x68, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e, 0x20,
0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x41, 0x6d, 0x20, 0x4f,
0x69, 0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x4d, 0x68, 0x75, 0x69, 0x72, 0xed,
0x73, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1,
0x6e, 0x61, 0x63, 0x68, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x6e, 0x20,
0x4d, 0x68, 0x75, 0x69, 0x72, 0xed, 0x73, 0x41, 0x6d, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4f, 0x69, 0x6c, 0x65,
0xe1, 0x6e, 0x20, 0x4d, 0x68, 0x75, 0x69, 0x72, 0xed, 0x73, 0x41, 0x6d,
0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x41, 0x6d, 0x20, 0x4d, 0x65,
0x69, 0x63, 0x73, 0x69, 0x63, 0x65, 0x61, 0x63, 0x68, 0x20, 0x61, 0x6e,
0x20, 0x41, 0x69, 0x67, 0xe9, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x69, 0xfa,
0x69, 0x6e, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65,
0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4d, 0x65, 0x69, 0x63, 0x73, 0x69,
0x63, 0x65, 0x61, 0x63, 0x68, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x69, 0x67,
0xe9, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x69, 0xfa, 0x69, 0x6e, 0x41, 0x6d,
0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4d,
0x65, 0x69, 0x63, 0x73, 0x69, 0x63, 0x65, 0x61, 0x63, 0x68, 0x20, 0x61,
0x6e, 0x20, 0x41, 0x69, 0x67, 0xe9, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x69,
0xfa, 0x69, 0x6e, 0x41, 0x6d, 0x20, 0x55, 0x6c, 0xe1, 0x6e, 0x62, 0xe1,
0x74, 0x61, 0x72, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64,
0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x55, 0x6c, 0xe1, 0x6e, 0x62,
0xe1, 0x74, 0x61, 0x72, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72,
0x61, 0x69, 0x64, 0x68, 0x20, 0x55, 0x6c, 0xe1, 0x6e, 0x62, 0xe1, 0x74,
0x61, 0x72, 0x41, 0x6d, 0x20, 0x4d, 0x68, 0x6f, 0x73, 0x63, 0xf3, 0x41,
0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61,
0x63, 0x68, 0x20, 0x4d, 0x68, 0x6f, 0x73, 0x63, 0xf3, 0x41, 0x6d, 0x20,
0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4d, 0x68,
0x6f, 0x73, 0x63, 0xf3, 0x41, 0x6d, 0x20, 0x4d, 0x68, 0x61, 0x65, 0x6e,
0x6d, 0x61, 0x72, 0x41, 0x6d, 0x20, 0x4e, 0xe1, 0x72, 0xfa, 0x41, 0x6d,
0x20, 0x4e, 0x65, 0x69, 0x70, 0x65, 0x61, 0x6c, 0x41, 0x6d, 0x20, 0x6e,
0x61, 0x20, 0x4e, 0x75, 0x61, 0x2d, 0x43, 0x68, 0x61, 0x6c, 0x61, 0x64,
0xf3, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68,
0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x4e,
0x75, 0x61, 0x2d, 0x43, 0x68, 0x61, 0x6c, 0x61, 0x64, 0xf3, 0x69, 0x6e,
0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x6e, 0x61, 0x20, 0x4e, 0x75, 0x61, 0x2d, 0x43, 0x68, 0x61,
0x6c, 0x61, 0x64, 0xf3, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x6e, 0x61,
0x20, 0x4e, 0x75, 0x61, 0x2d, 0x53, 0x68, 0xe9, 0x61, 0x6c, 0x61, 0x69,
0x6e, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64,
0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x4e, 0x75,
0x61, 0x2d, 0x53, 0x68, 0xe9, 0x61, 0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x65,
0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x6e, 0x61, 0x20, 0x4e, 0x75, 0x61, 0x2d, 0x53, 0x68, 0xe9, 0x61,
0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x54, 0x68, 0x61,
0x6c, 0x61, 0x6d, 0x68, 0x20, 0x61, 0x6e, 0x20, 0xc9, 0x69, 0x73, 0x63,
0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e,
0x61, 0x63, 0x68, 0x20, 0x54, 0x68, 0x61, 0x6c, 0x61, 0x6d, 0x68, 0x20,
0x61, 0x6e, 0x20, 0xc9, 0x69, 0x73, 0x63, 0x41, 0x6d, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x54, 0x68, 0x61, 0x6c,
0x61, 0x6d, 0x68, 0x20, 0x61, 0x6e, 0x20, 0xc9, 0x69, 0x73, 0x63, 0x41,
0x6d, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x6c,
0x65, 0xe1, 0x6e, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x41,
0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61,
0x63, 0x68, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x6e,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x41, 0x6d, 0x20, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69,
0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x68, 0x61, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x6e, 0x4f, 0x69, 0x6c, 0x65,
0xe1, 0x6e, 0x20, 0x4d, 0xe1, 0x69, 0x72, 0x69, 0x61, 0x6e, 0x61, 0x63,
0x68, 0x20, 0x54, 0x68, 0x75, 0x61, 0x69, 0x64, 0x68, 0x41, 0x6d, 0x20,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x41,
0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61,
0x63, 0x68, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x41, 0x6d, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x41, 0x6d, 0x20,
0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68,
0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x41, 0x6d,
0x20, 0x6e, 0x61, 0x20, 0x50, 0x61, 0x63, 0x61, 0x73, 0x74, 0xe1, 0x69,
0x6e, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65,
0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x50, 0x61, 0x63,
0x61, 0x73, 0x74, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x50,
0x61, 0x63, 0x61, 0x73, 0x74, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20,
0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e, 0x20, 0x50, 0x61, 0x6c, 0x61,
0x75, 0x41, 0x6d, 0x20, 0x4e, 0x75, 0x61, 0x2d, 0x47, 0x68, 0x75, 0x69,
0x6e, 0x65, 0x20, 0x50, 0x68, 0x61, 0x70, 0x75, 0x61, 0x41, 0x6d, 0x20,
0x50, 0x68, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x41, 0x6d, 0x20, 0x43,
0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20,
0x50, 0x68, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x41, 0x6d, 0x20, 0x53,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x50, 0x68, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x41, 0x6d, 0x20, 0x50, 0x68, 0x65, 0x69,
0x72, 0x69, 0xfa, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64,
0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x50, 0x68, 0x65, 0x69, 0x72,
0x69, 0xfa, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x50, 0x68, 0x65, 0x69, 0x72, 0x69, 0xfa, 0x41, 0x6d,
0x20, 0x6e, 0x61, 0x20, 0x6e, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x6e, 0x20,
0x46, 0x69, 0x6c, 0x69, 0x70, 0xed, 0x6e, 0x65, 0x61, 0x63, 0x68, 0x41,
0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61,
0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x6e, 0x4f, 0x69, 0x6c, 0x65, 0xe1,
0x6e, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0xed, 0x6e, 0x65, 0x61, 0x63,
0x68, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x6e, 0x61, 0x20, 0x6e, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x6e,
0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0xed, 0x6e, 0x65, 0x61, 0x63, 0x68,
0x41, 0x6d, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e, 0x20, 0x61,
0x6e, 0x20, 0x46, 0x68, 0xe9, 0x69, 0x6e, 0x69, 0x63, 0x73, 0x41, 0x6d,
0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f,
0x6e, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1,
0x6e, 0x61, 0x63, 0x68, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x41, 0x6d, 0x20, 0x50, 0x69, 0x74,
0x63, 0x61, 0x69, 0x72, 0x6e, 0x41, 0x6d, 0x20, 0x50, 0x68, 0x6f, 0x68,
0x6e, 0x70, 0x65, 0x69, 0x41, 0x6d, 0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67,
0x79, 0x61, 0x6e, 0x67, 0x41, 0x6d, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c,
0x6f, 0x72, 0x64, 0x61, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68,
0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x51, 0x79, 0x7a, 0x79,
0x6c, 0x6f, 0x72, 0x64, 0x61, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f,
0x72, 0x64, 0x61, 0x41, 0x6d, 0x20, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f,
0x6e, 0x41, 0x6d, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x41,
0x6d, 0x20, 0x53, 0x68, 0x61, 0x63, 0x61, 0x69, 0x6c, 0xed, 0x6e, 0x41,
0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61,
0x63, 0x68, 0x20, 0x53, 0x68, 0x61, 0x63, 0x61, 0x69, 0x6c, 0xed, 0x6e,
0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x53, 0x68, 0x61, 0x63, 0x61, 0x69, 0x6c, 0xed, 0x6e, 0x41, 0x6d,
0x20, 0x53, 0x68, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x41, 0x6d, 0x20, 0x43,
0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20,
0x53, 0x68, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x41, 0x6d, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x53, 0x68, 0x61, 0x6d,
0x61, 0x72, 0x61, 0x41, 0x6d, 0x20, 0x53, 0x68, 0x61, 0x6d, 0xf3, 0x41,
0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61,
0x63, 0x68, 0x20, 0x53, 0x68, 0x61, 0x6d, 0xf3, 0x41, 0x6d, 0x20, 0x53,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x53, 0x68, 0x61,
0x6d, 0xf3, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x53, 0xe9, 0x69, 0x73,
0xe9, 0x61, 0x6c, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64,
0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x53, 0x68, 0x69, 0x6e, 0x67,
0x65, 0x61, 0x70, 0xf3, 0x72, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x6c, 0x65,
0xe1, 0x69, 0x6e, 0x20, 0x53, 0x68, 0x6f, 0x6c, 0x61, 0x69, 0x6d, 0x68,
0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x53, 0x65, 0x6f, 0x69, 0x72, 0x73,
0x69, 0x61, 0x20, 0x54, 0x68, 0x65, 0x61, 0x73, 0x41, 0x6d, 0x20, 0x53,
0x68, 0x75, 0x72, 0x61, 0x6e, 0x61, 0x6d, 0x41, 0x6d, 0x20, 0x53, 0x79,
0x6f, 0x77, 0x61, 0x41, 0x6d, 0x20, 0x54, 0x68, 0x61, 0x69, 0x68, 0xed,
0x74, 0xed, 0x41, 0x6d, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x41,
0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61,
0x63, 0x68, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x41, 0x6d, 0x20,
0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x54, 0x61,
0x69, 0x70, 0x65, 0x69, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x54, 0xe1,
0x69, 0x64, 0x73, 0xed, 0x63, 0x65, 0x61, 0x73, 0x74, 0xe1, 0x69, 0x6e,
0x65, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e, 0x20,
0x54, 0xf3, 0x63, 0x61, 0x6c, 0xe1, 0x41, 0x6d, 0x20, 0x54, 0x68, 0x6f,
0x6e, 0x67, 0x61, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64,
0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x54, 0x68, 0x6f, 0x6e, 0x67,
0x61, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x54, 0x68, 0x6f, 0x6e, 0x67, 0x61, 0x41, 0x6d, 0x20, 0x43,
0x68, 0x75, 0x75, 0x6b, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x54, 0x75,
0x69, 0x72, 0x63, 0x6d, 0xe9, 0x61, 0x6e, 0x61, 0x73, 0x74, 0xe1, 0x69,
0x6e, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65,
0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x54, 0x75, 0x69,
0x72, 0x63, 0x6d, 0xe9, 0x61, 0x6e, 0x61, 0x73, 0x74, 0xe1, 0x69, 0x6e,
0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x6e, 0x61, 0x20, 0x54, 0x75, 0x69, 0x72, 0x63, 0x6d, 0xe9,
0x61, 0x6e, 0x61, 0x73, 0x74, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20,
0x54, 0x68, 0xfa, 0x76, 0x61, 0x6c, 0xfa, 0x41, 0x6d, 0x20, 0x55, 0x72,
0x61, 0x67, 0x75, 0x61, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68,
0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x55, 0x72, 0x61, 0x67,
0x75, 0x61, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x55, 0x72, 0x61, 0x67, 0x75, 0x61, 0x41, 0x6d, 0x20,
0x6e, 0x61, 0x20, 0x68, 0xda, 0x69, 0x73, 0x62, 0xe9, 0x69, 0x63, 0x65,
0x61, 0x73, 0x74, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61,
0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e,
0x61, 0x20, 0x68, 0xda, 0x69, 0x73, 0x62, 0xe9, 0x69, 0x63, 0x65, 0x61,
0x73, 0x74, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x68, 0xda,
0x69, 0x73, 0x62, 0xe9, 0x69, 0x63, 0x65, 0x61, 0x73, 0x74, 0xe1, 0x69,
0x6e, 0x65, 0x41, 0x6d, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0xfa,
0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e,
0x61, 0x63, 0x68, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0xfa, 0x41,
0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0xfa, 0x41, 0x6d, 0x20, 0x56, 0x65,
0x69, 0x6e, 0x69, 0x73, 0xe9, 0x61, 0x6c, 0x61, 0x41, 0x6d, 0x20, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x41, 0x6d,
0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63,
0x68, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x41, 0x6d, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1,
0x6e, 0x61, 0x63, 0x68, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x41, 0x6d, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x41, 0x6d, 0x20,
0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x41,
0x6d, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x61, 0x67, 0x75,
0x73, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x41, 0x6d, 0x20, 0x49,
0x61, 0x63, 0xfa, 0x74, 0x73, 0x63, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69,
0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x49, 0x61,
0x63, 0xfa, 0x74, 0x73, 0x63, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x49, 0x61, 0x63, 0xfa, 0x74, 0x73,
0x63, 0x41, 0x6d, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67,
0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x59, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x41, 0x6d,
0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x59,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x41, 0x6d, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x66, 0x67, 0x68, 0x61, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c,
0x2019, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c,
0x2019, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6d, 0x65, 0x72,
0x69, 0x64, 0x69, 0x6f, 0x6e, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20,
0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20,
0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x6c, 0x6c, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x7a, 0x7a,
0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x6d,
0x61, 0x7a, 0x7a, 0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c,
0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41,
0x6d, 0x61, 0x7a, 0x7a, 0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x20, 0x55, 0x53, 0x41,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x20, 0x55, 0x53,
0x41, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x20, 0x55, 0x53, 0x41,
0x4f, 0x72, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x20, 0x55, 0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x65, 0x20, 0x55, 0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x6c,
0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x65, 0x20, 0x55, 0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x4d,
0x6f, 0x6e, 0x74, 0x61, 0x67, 0x6e, 0x65, 0x20, 0x52, 0x6f, 0x63, 0x63,
0x69, 0x6f, 0x73, 0x65, 0x20, 0x55, 0x53, 0x41, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x4d, 0x6f, 0x6e,
0x74, 0x61, 0x67, 0x6e, 0x65, 0x20, 0x52, 0x6f, 0x63, 0x63, 0x69, 0x6f,
0x73, 0x65, 0x20, 0x55, 0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61, 0x67, 0x6e,
0x65, 0x20, 0x52, 0x6f, 0x63, 0x63, 0x69, 0x6f, 0x73, 0x65, 0x20, 0x55,
0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61,
0x63, 0x69, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x55, 0x53, 0x41, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x6f, 0x20,
0x55, 0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69,
0x63, 0x6f, 0x20, 0x55, 0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x41,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79,
0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x41, 0x70, 0x69, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x69, 0x20, 0x41, 0x70, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x41, 0x70,
0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x61, 0x72, 0x61, 0x62, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x72, 0x61, 0x62, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x61, 0x72, 0x61, 0x62, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x2019, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x6c, 0x6c, 0x2019, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x2019, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x63, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c,
0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c,
0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20,
0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x61, 0x69, 0x67, 0x69, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c,
0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x67, 0x69, 0x61, 0x6e,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x67,
0x69, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65,
0x20, 0x41, 0x7a, 0x7a, 0x6f, 0x72, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x65, 0x20, 0x41, 0x7a, 0x7a, 0x6f, 0x72, 0x72, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x65, 0x20, 0x41, 0x7a, 0x7a, 0x6f, 0x72, 0x72, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x73, 0x68, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x4f, 0x72, 0x61, 0x20, 0x6c,
0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x42, 0x6f, 0x6c, 0x69,
0x76, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x42, 0x72,
0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x42, 0x72,
0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x42, 0x72, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73,
0x73, 0x61, 0x6c, 0x61, 0x6d, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x43, 0x61, 0x70, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x69, 0x20, 0x43, 0x61, 0x70, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x69, 0x20, 0x43, 0x61, 0x70, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f,
0x72, 0x72, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65,
0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x43, 0x69, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x43, 0x69, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c,
0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x43, 0x69,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20,
0x43, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x43,
0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x43, 0x69, 0x6e, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x49, 0x73, 0x6f,
0x6c, 0x61, 0x20, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x49, 0x73,
0x6f, 0x6c, 0x65, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d,
0x62, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x43, 0x6f,
0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x43,
0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x65, 0x20, 0x69, 0x73, 0x6f, 0x6c, 0x65, 0x20, 0x43,
0x6f, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x69, 0x73,
0x6f, 0x6c, 0x65, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x6d, 0x65, 0x64, 0x69, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x69, 0x73, 0x6f, 0x6c, 0x65,
0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x43, 0x75, 0x62, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x43, 0x75, 0x62, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x69, 0x20, 0x43, 0x75, 0x62, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72,
0x76, 0x69, 0x6c, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x45, 0x73, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x49, 0x73, 0x6f, 0x6c, 0x61, 0x20,
0x64, 0x69, 0x20, 0x50, 0x61, 0x73, 0x71, 0x75, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x6c, 0x6c, 0x2019, 0x49, 0x73, 0x6f, 0x6c, 0x61, 0x20, 0x64, 0x69, 0x20,
0x50, 0x61, 0x73, 0x71, 0x75, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x49, 0x73,
0x6f, 0x6c, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x61, 0x73, 0x71, 0x75,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x45, 0x63,
0x75, 0x61, 0x64, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c,
0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x65, 0x20, 0x28, 0x4b, 0x61, 0x6c, 0x69, 0x6e,
0x69, 0x6e, 0x67, 0x72, 0x61, 0x64, 0x29, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x49, 0x73, 0x6f, 0x6c, 0x65, 0x20,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x65, 0x20, 0x49, 0x73, 0x6f, 0x6c, 0x65, 0x20, 0x46, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x49, 0x73,
0x6f, 0x6c, 0x65, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x46, 0x69,
0x67, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x46, 0x69, 0x67,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x46, 0x69, 0x67, 0x69, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x47, 0x75, 0x69, 0x61,
0x6e, 0x61, 0x20, 0x66, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x54, 0x65, 0x72,
0x72, 0x65, 0x20, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x20,
0x65, 0x20, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x74, 0x69, 0x63, 0x68, 0x65,
0x20, 0x66, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x69, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x70,
0x61, 0x67, 0x6f, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x47,
0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x6c, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x69,
0x73, 0x6f, 0x6c, 0x65, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6d, 0x65, 0x72, 0x69,
0x64, 0x69, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x69, 0x20, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x6c, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x47, 0x72,
0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x47,
0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x47, 0x6f,
0x6c, 0x66, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61,
0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x65, 0x20, 0x69, 0x73, 0x6f, 0x6c, 0x65, 0x20, 0x48,
0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69,
0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x49, 0x73, 0x6f,
0x6c, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x49,
0x73, 0x6f, 0x6c, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x69, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e,
0x67, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x69, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x69, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x6c,
0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x48, 0x6f, 0x76,
0x64, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x49, 0x6e, 0x64, 0x69, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x4f, 0x63, 0x65,
0x61, 0x6e, 0x6f, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x6f, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x49, 0x6e, 0x64, 0x6f,
0x63, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c,
0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69,
0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x49, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x2019, 0x49, 0x72, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019,
0x49, 0x72, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x49, 0x72, 0x61, 0x6e,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x69, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x69, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69,
0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x47, 0x69, 0x61, 0x70, 0x70, 0x6f, 0x6e, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x47, 0x69, 0x61, 0x70, 0x70, 0x6f, 0x6e,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x47, 0x69, 0x61, 0x70, 0x70, 0x6f, 0x6e, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x50,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69,
0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76,
0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4b, 0x61, 0x7a,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x63, 0x6f, 0x72,
0x65, 0x61, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x63, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x63,
0x6f, 0x72, 0x65, 0x61, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x79, 0x61, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x4b, 0x69, 0x72, 0x67, 0x68, 0x69, 0x7a, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20,
0x53, 0x70, 0x6f, 0x72, 0x61, 0x64, 0x69, 0x20, 0x65, 0x71, 0x75, 0x61,
0x74, 0x6f, 0x72, 0x69, 0x61, 0x6c, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x69, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x69, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x69, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x61,
0x63, 0x61, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x4d,
0x61, 0x6c, 0x65, 0x73, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x6c, 0x65, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x4d, 0x61, 0x72,
0x63, 0x68, 0x65, 0x73, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x65, 0x20, 0x49, 0x73, 0x6f, 0x6c, 0x65, 0x20, 0x4d, 0x61, 0x72,
0x73, 0x68, 0x61, 0x6c, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x74, 0x69, 0x75, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x6f, 0x20,
0x28, 0x4d, 0x65, 0x73, 0x73, 0x69, 0x63, 0x6f, 0x29, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x28,
0x4d, 0x65, 0x73, 0x73, 0x69, 0x63, 0x6f, 0x29, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50,
0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x28, 0x4d, 0x65, 0x73,
0x73, 0x69, 0x63, 0x6f, 0x29, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x69, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x69, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x69, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x4d,
0x6f, 0x73, 0x63, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c,
0x61, 0x20, 0x42, 0x69, 0x72, 0x6d, 0x61, 0x6e, 0x69, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x4e, 0x75, 0x6f,
0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x4e, 0x75, 0x6f, 0x76, 0x61,
0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x61, 0x20, 0x4e, 0x75, 0x6f, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x6c, 0x61, 0x20, 0x4e, 0x75, 0x6f, 0x76, 0x61, 0x20, 0x5a, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20,
0x4e, 0x75, 0x6f, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x4e, 0x75, 0x6f, 0x76, 0x61, 0x20,
0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x69, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x69, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x49, 0x73, 0x6f, 0x6c,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x6c, 0x6c, 0x65, 0x20, 0x49, 0x73, 0x6f, 0x6c, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x49, 0x73,
0x6f, 0x6c, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x69, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x69, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x69, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x4f, 0x6d,
0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x69, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x50, 0x61,
0x70, 0x75, 0x61, 0x20, 0x4e, 0x75, 0x6f, 0x76, 0x61, 0x20, 0x47, 0x75,
0x69, 0x6e, 0x65, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x50, 0x65, 0x72, 0xf9, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x50, 0x65, 0x72, 0xf9, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x65, 0x72, 0xf9,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c,
0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x6c, 0x6c, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x49,
0x73, 0x6f, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x46,
0x65, 0x6e, 0x69, 0x63, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x69, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x69, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65,
0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x69, 0x20, 0x50, 0x6f, 0x68, 0x6e, 0x70, 0x65, 0x69, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79,
0x61, 0x6e, 0x67, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x52, 0x69,
0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x69, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69,
0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x69, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x69, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x53, 0x61,
0x6d, 0x61, 0x72, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x53, 0x61, 0x6d, 0x6f,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x69, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65,
0x6c, 0x6c, 0x65, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x53,
0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x49, 0x73, 0x6f, 0x6c, 0x65, 0x20,
0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x53, 0x75, 0x64, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x53, 0x79, 0x6f, 0x77,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x54, 0x61, 0x68, 0x69,
0x74, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x54, 0x61, 0x69,
0x70, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x69, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x54, 0x61, 0x67, 0x69, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x54, 0x6f,
0x6b, 0x65, 0x6c, 0x61, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x69, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x54,
0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x54, 0x75, 0x76,
0x61, 0x6c, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019,
0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c,
0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019,
0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x2019, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x55, 0x7a, 0x62, 0x65,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x55, 0x7a,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65,
0x6c, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20,
0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69,
0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x2019, 0x49, 0x73, 0x6f, 0x6c, 0x61, 0x20, 0x64, 0x69, 0x20, 0x57,
0x61, 0x6b, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x57, 0x61,
0x6c, 0x6c, 0x69, 0x73, 0x20, 0x65, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x59, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x69, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x45, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20,
0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x6f, 0x20, 0x59, 0x75,
0x6b, 0x6f, 0x6e, 0x30a2, 0x30af, 0x30ec, 0x6642, 0x9593, 0x30a2, 0x30af, 0x30ec, 0x6a19,
0x6e96, 0x6642, 0x30a2, 0x30af, 0x30ec, 0x590f, 0x6642, 0x9593, 0x30a2, 0x30d5, 0x30ac, 0x30cb,
0x30b9, 0x30bf, 0x30f3, 0x6642, 0x9593, 0x4e2d, 0x592e, 0x30a2, 0x30d5, 0x30ea, 0x30ab, 0x6642,
0x9593, 0x6771, 0x30a2, 0x30d5, 0x30ea, 0x30ab, 0x6642, 0x9593, 0x5357, 0x30a2, 0x30d5, 0x30ea,
0x30ab, 0x6a19, 0x6e96, 0x6642, 0x897f, 0x30a2, 0x30d5, 0x30ea, 0x30ab, 0x6642, 0x9593, 0x897f,
0x30a2, 0x30d5, 0x30ea, 0x30ab, 0x6a19, 0x6e96, 0x6642, 0x897f, 0x30a2, 0x30d5, 0x30ea, 0x30ab,
0x590f, 0x6642, 0x9593, 0x30a2, 0x30e9, 0x30b9, 0x30ab, 0x6642, 0x9593, 0x30a2, 0x30e9, 0x30b9,
0x30ab, 0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30e9, 0x30b9, 0x30ab, 0x590f, 0x6642, 0x9593, 0x30a2,
0x30eb, 0x30c8, 0x30de, 0x30a4, 0x6642, 0x9593, 0x30a2, 0x30eb, 0x30c8, 0x30de, 0x30a4, 0x6a19,
0x6e96, 0x6642, 0x30a2, 0x30eb, 0x30de, 0x30c8, 0x30a4, 0x590f, 0x6642, 0x9593, 0x30a2, 0x30de,
0x30be, 0x30f3, 0x6642, 0x9593, 0x30a2, 0x30de, 0x30be, 0x30f3, 0x6a19, 0x6e96, 0x6642, 0x30a2,
0x30de, 0x30be, 0x30f3, 0x590f, 0x6642, 0x9593, 0x30a2, 0x30e1, 0x30ea, 0x30ab, 0x4e2d, 0x90e8,
0x6642, 0x9593, 0x30a2, 0x30e1, 0x30ea, 0x30ab, 0x4e2d, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x30a2,
0x30e1, 0x30ea, 0x30ab, 0x4e2d, 0x90e8, 0x590f, 0x6642, 0x9593, 0x30a2, 0x30e1, 0x30ea, 0x30ab,
0x6771, 0x90e8, 0x6642, 0x9593, 0x30a2, 0x30e1, 0x30ea, 0x30ab, 0x6771, 0x90e8, 0x6a19, 0x6e96,
0x6642, 0x30a2, 0x30e1, 0x30ea, 0x30ab, 0x6771, 0x90e8, 0x590f, 0x6642, 0x9593, 0x30a2, 0x30e1,
0x30ea, 0x30ab, 0x5c71, 0x5730, 0x6642, 0x9593, 0x30a2, 0x30e1, 0x30ea, 0x30ab, 0x5c71, 0x5730,
0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30e1, 0x30ea, 0x30ab, 0x5c71, 0x5730, 0x590f, 0x6642, 0x9593,
0x30a2, 0x30e1, 0x30ea, 0x30ab, 0x592a, 0x5e73, 0x6d0b, 0x6642, 0x9593, 0x30a2, 0x30e1, 0x30ea,
0x30ab, 0x592a, 0x5e73, 0x6d0b, 0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30e1, 0x30ea, 0x30ab, 0x592a,
0x5e73, 0x6d0b, 0x590f, 0x6642, 0x9593, 0x30a2, 0x30ca, 0x30c7, 0x30a3, 0x30ea, 0x6642, 0x9593,
0x30a2, 0x30ca, 0x30c7, 0x30a3, 0x30ea, 0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30ca, 0x30c7, 0x30a3,
0x30ea, 0x590f, 0x6642, 0x9593, 0x30a2, 0x30d4, 0x30a2, 0x6642, 0x9593, 0x30a2, 0x30d4, 0x30a2,
0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30d4, 0x30a2, 0x590f, 0x6642, 0x9593, 0x30a2, 0x30af, 0x30bf,
0x30a6, 0x6642, 0x9593, 0x30a2, 0x30af, 0x30bf, 0x30a6, 0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30af,
0x30bf, 0x30a6, 0x590f, 0x6642, 0x9593, 0x30a2, 0x30af, 0x30c8, 0x30d9, 0x6642, 0x9593, 0x30a2,
0x30af, 0x30c8, 0x30d9, 0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30af, 0x30c8, 0x30d9, 0x590f, 0x6642,
0x9593, 0x30a2, 0x30e9, 0x30d3, 0x30a2, 0x6642, 0x9593, 0x30a2, 0x30e9, 0x30d3, 0x30a2, 0x6a19,
0x6e96, 0x6642, 0x30a2, 0x30e9, 0x30d3, 0x30a2, 0x590f, 0x6642, 0x9593, 0x30a2, 0x30eb, 0x30bc,
0x30f3, 0x30c1, 0x30f3, 0x6642, 0x9593, 0x30a2, 0x30eb, 0x30bc, 0x30f3, 0x30c1, 0x30f3, 0x6a19,
0x6e96, 0x6642, 0x30a2, 0x30eb, 0x30bc, 0x30f3, 0x30c1, 0x30f3, 0x590f, 0x6642, 0x9593, 0x897f,
0x90e8, 0x30a2, 0x30eb, 0x30bc, 0x30f3, 0x30c1, 0x30f3, 0x6642, 0x9593, 0x897f, 0x90e8, 0x30a2,
0x30eb, 0x30bc, 0x30f3, 0x30c1, 0x30f3, 0x6a19, 0x6e96, 0x6642, 0x897f, 0x90e8, 0x30a2, 0x30eb,
0x30bc, 0x30f3, 0x30c1, 0x30f3, 0x590f, 0x6642, 0x9593, 0x30a2, 0x30eb, 0x30e1, 0x30cb, 0x30a2,
0x6642, 0x9593, 0x30a2, 0x30eb, 0x30e1, 0x30cb, 0x30a2, 0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30eb,
0x30e1, 0x30cb, 0x30a2, 0x590f, 0x6642, 0x9593, 0x5927, 0x897f, 0x6d0b, 0x590f, 0x6642, 0x9593,
0x30aa, 0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea, 0x30a2, 0x4e2d, 0x90e8, 0x6642, 0x9593, 0x30aa,
0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea, 0x30a2, 0x4e2d, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x30aa,
0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea, 0x30a2, 0x4e2d, 0x90e8, 0x590f, 0x6642, 0x9593, 0x30aa,
0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea, 0x30a2, 0x4e2d, 0x897f, 0x90e8, 0x6642, 0x9593, 0x30aa,
0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea, 0x30a2, 0x4e2d, 0x897f, 0x90e8, 0x6a19, 0x6e96, 0x6642,
0x30aa, 0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea, 0x30a2, 0x4e2d, 0x897f, 0x90e8, 0x590f, 0x6642,
0x9593, 0x30aa, 0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea, 0x30a2, 0x6771, 0x90e8, 0x6642, 0x9593,
0x30aa, 0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea, 0x30a2, 0x6771, 0x90e8, 0x6a19, 0x6e96, 0x6642,
0x30aa, 0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea, 0x30a2, 0x6771, 0x90e8, 0x590f, 0x6642, 0x9593,
0x30aa, 0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea, 0x30a2, 0x897f, 0x90e8, 0x6642, 0x9593, 0x30aa,
0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea, 0x30a2, 0x897f, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x30aa,
0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea, 0x30a2, 0x897f, 0x90e8, 0x590f, 0x6642, 0x9593, 0x30a2,
0x30bc, 0x30eb, 0x30d0, 0x30a4, 0x30b8, 0x30e3, 0x30f3, 0x6642, 0x9593, 0x30a2, 0x30bc, 0x30eb,
0x30d0, 0x30a4, 0x30b8, 0x30e3, 0x30f3, 0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30bc, 0x30eb, 0x30d0,
0x30a4, 0x30b8, 0x30e3, 0x30f3, 0x590f, 0x6642, 0x9593, 0x30a2, 0x30be, 0x30ec, 0x30b9, 0x6642,
0x9593, 0x30a2, 0x30be, 0x30ec, 0x30b9, 0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30be, 0x30ec, 0x30b9,
0x590f, 0x6642, 0x9593, 0x30d0, 0x30f3, 0x30b0, 0x30e9, 0x30c7, 0x30b7, 0x30e5, 0x6642, 0x9593,
0x30d0, 0x30f3, 0x30b0, 0x30e9, 0x30c7, 0x30b7, 0x30e5, 0x6a19, 0x6e96, 0x6642, 0x30d0, 0x30f3,
0x30b0, 0x30e9, 0x30c7, 0x30b7, 0x30e5, 0x590f, 0x6642, 0x9593, 0x30d6, 0x30fc, 0x30bf, 0x30f3,
0x6642, 0x9593, 0x30dc, 0x30ea, 0x30d3, 0x30a2, 0x6642, 0x9593, 0x30d6, 0x30e9, 0x30b8, 0x30ea,
0x30a2, 0x6642, 0x9593, 0x30d6, 0x30e9, 0x30b8, 0x30ea, 0x30a2, 0x6a19, 0x6e96, 0x6642, 0x30d6,
0x30e9, 0x30b8, 0x30ea, 0x30a2, 0x590f, 0x6642, 0x9593, 0x30d6, 0x30eb, 0x30cd, 0x30a4, 0x30fb,
0x30c0, 0x30eb, 0x30b5, 0x30e9, 0x30fc, 0x30e0, 0x6642, 0x9593, 0x30ab, 0x30fc, 0x30dc, 0x30d9,
0x30eb, 0x30c7, 0x6642, 0x9593, 0x30ab, 0x30fc, 0x30dc, 0x30d9, 0x30eb, 0x30c7, 0x6a19, 0x6e96,
0x6642, 0x30ab, 0x30fc, 0x30dc, 0x30d9, 0x30eb, 0x30c7, 0x590f, 0x6642, 0x9593, 0x30b1, 0x30a4,
0x30b7, 0x30fc, 0x57fa, 0x5730, 0x6642, 0x9593, 0x30c1, 0x30e3, 0x30e2, 0x30ed, 0x6642, 0x9593,
0x30c1, 0x30e3, 0x30bf, 0x30e0, 0x6642, 0x9593, 0x30c1, 0x30e3, 0x30bf, 0x30e0, 0x6a19, 0x6e96,
0x6642, 0x30c1, 0x30e3, 0x30bf, 0x30e0, 0x590f, 0x6642, 0x9593, 0x30c1, 0x30ea, 0x6642, 0x9593,
0x30c1, 0x30ea, 0x6a19, 0x6e96, 0x6642, 0x30c1, 0x30ea, 0x590f, 0x6642, 0x9593, 0x4e2d, 0x56fd,
0x6642, 0x9593, 0x4e2d, 0x56fd, 0x6a19, 0x6e96, 0x6642, 0x4e2d, 0x56fd, 0x590f, 0x6642, 0x9593,
0x30af, 0x30ea, 0x30b9, 0x30de, 0x30b9, 0x5cf6, 0x6642, 0x9593, 0x30b3, 0x30b3, 0x30b9, 0x8af8,
0x5cf6, 0x6642, 0x9593, 0x30b3, 0x30ed, 0x30f3, 0x30d3, 0x30a2, 0x6642, 0x9593, 0x30b3, 0x30ed,
0x30f3, 0x30d3, 0x30a2, 0x6a19, 0x6e96, 0x6642, 0x30b3, 0x30ed, 0x30f3, 0x30d3, 0x30a2, 0x590f,
0x6642, 0x9593, 0x30af, 0x30c3, 0x30af, 0x8af8, 0x5cf6, 0x6642, 0x9593, 0x30af, 0x30c3, 0x30af,
0x8af8, 0x5cf6, 0x6a19, 0x6e96, 0x6642, 0x30af, 0x30c3, 0x30af, 0x8af8, 0x5cf6, 0x590f, 0x6642,
0x9593, 0x30ad, 0x30e5, 0x30fc, 0x30d0, 0x6642, 0x9593, 0x30ad, 0x30e5, 0x30fc, 0x30d0, 0x6a19,
0x6e96, 0x6642, 0x30ad, 0x30e5, 0x30fc, 0x30d0, 0x590f, 0x6642, 0x9593, 0x30c7, 0x30fc, 0x30d3,
0x30b9, 0x57fa, 0x5730, 0x6642, 0x9593, 0x30c7, 0x30e5, 0x30e2, 0x30f3, 0x30fb, 0x30c7, 0x30e5,
0x30eb, 0x30f4, 0x30a3, 0x30eb, 0x57fa, 0x5730, 0x6642, 0x9593, 0x6771, 0x30c6, 0x30a3, 0x30e2,
0x30fc, 0x30eb, 0x6642, 0x9593, 0x30a4, 0x30fc, 0x30b9, 0x30bf, 0x30fc, 0x5cf6, 0x6642, 0x9593,
0x30a4, 0x30fc, 0x30b9, 0x30bf, 0x30fc, 0x5cf6, 0x6a19, 0x6e96, 0x6642, 0x30a4, 0x30fc, 0x30b9,
0x30bf, 0x30fc, 0x5cf6, 0x590f, 0x6642, 0x9593, 0x30a8, 0x30af, 0x30a2, 0x30c9, 0x30eb, 0x6642,
0x9593, 0x4e2d, 0x592e, 0x30e8, 0x30fc, 0x30ed, 0x30c3, 0x30d1, 0x6642, 0x9593, 0x4e2d, 0x592e,
0x30e8, 0x30fc, 0x30ed, 0x30c3, 0x30d1, 0x6a19, 0x6e96, 0x6642, 0x4e2d, 0x592e, 0x30e8, 0x30fc,
0x30ed, 0x30c3, 0x30d1, 0x590f, 0x6642, 0x9593, 0x6771, 0x30e8, 0x30fc, 0x30ed, 0x30c3, 0x30d1,
0x6642, 0x9593, 0x6771, 0x30e8, 0x30fc, 0x30ed, 0x30c3, 0x30d1, 0x6a19, 0x6e96, 0x6642, 0x6771,
0x30e8, 0x30fc, 0x30ed, 0x30c3, 0x30d1, 0x590f, 0x6642, 0x9593, 0x6975, 0x6771, 0x30e8, 0x30fc,
0x30ed, 0x30c3, 0x30d1, 0x6642, 0x9593, 0x897f, 0x30e8, 0x30fc, 0x30ed, 0x30c3, 0x30d1, 0x6642,
0x9593, 0x897f, 0x30e8, 0x30fc, 0x30ed, 0x30c3, 0x30d1, 0x6a19, 0x6e96, 0x6642, 0x897f, 0x30e8,
0x30fc, 0x30ed, 0x30c3, 0x30d1, 0x590f, 0x6642, 0x9593, 0x30d5, 0x30a9, 0x30fc, 0x30af, 0x30e9,
0x30f3, 0x30c9, 0x8af8, 0x5cf6, 0x6642, 0x9593, 0x30d5, 0x30a9, 0x30fc, 0x30af, 0x30e9, 0x30f3,
0x30c9, 0x8af8, 0x5cf6, 0x6a19, 0x6e96, 0x6642, 0x30d5, 0x30a9, 0x30fc, 0x30af, 0x30e9, 0x30f3,
0x30c9, 0x8af8, 0x5cf6, 0x590f, 0x6642, 0x9593, 0x30d5, 0x30a3, 0x30b8, 0x30fc, 0x6642, 0x9593,
0x30d5, 0x30a3, 0x30b8, 0x30fc, 0x6a19, 0x6e96, 0x6642, 0x30d5, 0x30a3, 0x30b8, 0x30fc, 0x590f,
0x6642, 0x9593, 0x4ecf, 0x9818, 0x30ae, 0x30a2, 0x30ca, 0x6642, 0x9593, 0x4ecf, 0x9818, 0x5357,
0x65b9, 0x5357, 0x6975, 0x6642, 0x9593, 0x30ac, 0x30e9, 0x30d1, 0x30b4, 0x30b9, 0x6642, 0x9593,
0x30ac, 0x30f3, 0x30d3, 0x30a8, 0x8af8, 0x5cf6, 0x6642, 0x9593, 0x30b8, 0x30e7, 0x30fc, 0x30b8,
0x30a2, 0x6642, 0x9593, 0x30b8, 0x30e7, 0x30fc, 0x30b8, 0x30a2, 0x6a19, 0x6e96, 0x6642, 0x30b8,
0x30e7, 0x30fc, 0x30b8, 0x30a2, 0x590f, 0x6642, 0x9593, 0x30ae, 0x30eb, 0x30d0, 0x30fc, 0x30c8,
0x8af8, 0x5cf6, 0x6642, 0x9593, 0x30b0, 0x30ea, 0x30cb, 0x30c3, 0x30b8, 0x6a19, 0x6e96, 0x6642,
0x30b0, 0x30ea, 0x30fc, 0x30f3, 0x30e9, 0x30f3, 0x30c9, 0x6771, 0x90e8, 0x6642, 0x9593, 0x30b0,
0x30ea, 0x30fc, 0x30f3, 0x30e9, 0x30f3, 0x30c9, 0x6771, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x30b0,
0x30ea, 0x30fc, 0x30f3, 0x30e9, 0x30f3, 0x30c9, 0x6771, 0x90e8, 0x590f, 0x6642, 0x9593, 0x30b0,
0x30ea, 0x30fc, 0x30f3, 0x30e9, 0x30f3, 0x30c9, 0x897f, 0x90e8, 0x6642, 0x9593, 0x30b0, 0x30ea,
0x30fc, 0x30f3, 0x30e9, 0x30f3, 0x30c9, 0x897f, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x30b0, 0x30ea,
0x30fc, 0x30f3, 0x30e9, 0x30f3, 0x30c9, 0x897f, 0x90e8, 0x590f, 0x6642, 0x9593, 0x30b0, 0x30a2,
0x30e0, 0x6642, 0x9593, 0x6e7e, 0x5cb8, 0x6a19, 0x6e96, 0x6642, 0x30ac, 0x30a4, 0x30a2, 0x30ca,
0x6642, 0x9593, 0x30cf, 0x30ef, 0x30a4, 0x30fb, 0x30a2, 0x30ea, 0x30e5, 0x30fc, 0x30b7, 0x30e3,
0x30f3, 0x6642, 0x9593, 0x30cf, 0x30ef, 0x30a4, 0x30fb, 0x30a2, 0x30ea, 0x30e5, 0x30fc, 0x30b7,
0x30e3, 0x30f3, 0x6a19, 0x6e96, 0x6642, 0x30cf, 0x30ef, 0x30a4, 0x30fb, 0x30a2, 0x30ea, 0x30e5,
0x30fc, 0x30b7, 0x30e3, 0x30f3, 0x590f, 0x6642, 0x9593, 0x9999, 0x6e2f, 0x590f, 0x6642, 0x9593,
0x30db, 0x30d6, 0x30c9, 0x6642, 0x9593, 0x30db, 0x30d6, 0x30c9, 0x6a19, 0x6e96, 0x6642, 0x30db,
0x30d6, 0x30c9, 0x590f, 0x6642, 0x9593, 0x30a4, 0x30f3, 0x30c9, 0x6a19, 0x6e96, 0x6642, 0x30a4,
0x30f3, 0x30c9, 0x6d0b, 0x6642, 0x9593, 0x30a4, 0x30f3, 0x30c9, 0x30b7, 0x30ca, 0x6642, 0x9593,
0x30a4, 0x30f3, 0x30c9, 0x30cd, 0x30b7, 0x30a2, 0x4e2d, 0x90e8, 0x6642, 0x9593, 0x30a4, 0x30f3,
0x30c9, 0x30cd, 0x30b7, 0x30a2, 0x6771, 0x90e8, 0x6642, 0x9593, 0x30a4, 0x30f3, 0x30c9, 0x30cd,
0x30b7, 0x30a2, 0x897f, 0x90e8, 0x6642, 0x9593, 0x30a4, 0x30e9, 0x30f3, 0x6642, 0x9593, 0x30a4,
0x30e9, 0x30f3, 0x6a19, 0x6e96, 0x6642, 0x30a4, 0x30e9, 0x30f3, 0x590f, 0x6642, 0x9593, 0x30a4,
0x30eb, 0x30af, 0x30fc, 0x30c4, 0x30af, 0x6642, 0x9593, 0x30a4, 0x30eb, 0x30af, 0x30fc, 0x30c4,
0x30af, 0x6a19, 0x6e96, 0x6642, 0x30a4, 0x30eb, 0x30af, 0x30fc, 0x30c4, 0x30af, 0x590f, 0x6642,
0x9593, 0x30a4, 0x30b9, 0x30e9, 0x30a8, 0x30eb, 0x6642, 0x9593, 0x30a4, 0x30b9, 0x30e9, 0x30a8,
0x30eb, 0x6a19, 0x6e96, 0x6642, 0x30a4, 0x30b9, 0x30e9, 0x30a8, 0x30eb, 0x590f, 0x6642, 0x9593,
0x65e5, 0x672c, 0x590f, 0x6642, 0x9593, 0x30da, 0x30c8, 0x30ed, 0x30d1, 0x30d6, 0x30ed, 0x30d5,
0x30b9, 0x30af, 0x30fb, 0x30ab, 0x30e0, 0x30c1, 0x30e3, 0x30c4, 0x30ad, 0x30fc, 0x6642, 0x9593,
0x30da, 0x30c8, 0x30ed, 0x30d1, 0x30d6, 0x30ed, 0x30d5, 0x30b9, 0x30af, 0x30fb, 0x30ab, 0x30e0,
0x30c1, 0x30e3, 0x30c4, 0x30ad, 0x30fc, 0x6a19, 0x6e96, 0x6642, 0x30da, 0x30c8, 0x30ed, 0x30d1,
0x30d6, 0x30ed, 0x30d5, 0x30b9, 0x30af, 0x30fb, 0x30ab, 0x30e0, 0x30c1, 0x30e3, 0x30c4, 0x30ad,
0x30fc, 0x590f, 0x6642, 0x9593, 0x30ab, 0x30b6, 0x30d5, 0x30b9, 0x30bf, 0x30f3, 0x6642, 0x9593,
0x6771, 0x30ab, 0x30b6, 0x30d5, 0x30b9, 0x30bf, 0x30f3, 0x6642, 0x9593, 0x897f, 0x30ab, 0x30b6,
0x30d5, 0x30b9, 0x30bf, 0x30f3, 0x6642, 0x9593, 0x97d3, 0x56fd, 0x6642, 0x9593, 0x97d3, 0x56fd,
0x6a19, 0x6e96, 0x6642, 0x97d3, 0x56fd, 0x590f, 0x6642, 0x9593, 0x30b3, 0x30b9, 0x30e9, 0x30a8,
0x6642, 0x9593, 0x30af, 0x30e9, 0x30b9, 0x30ce, 0x30e4, 0x30eb, 0x30b9, 0x30af, 0x6642, 0x9593,
0x30af, 0x30e9, 0x30b9, 0x30ce, 0x30e4, 0x30eb, 0x30b9, 0x30af, 0x6a19, 0x6e96, 0x6642, 0x30af,
0x30e9, 0x30b9, 0x30ce, 0x30e4, 0x30eb, 0x30b9, 0x30af, 0x590f, 0x6642, 0x9593, 0x30ad, 0x30eb,
0x30ae, 0x30b9, 0x6642, 0x9593, 0x30e9, 0x30f3, 0x30ab, 0x6642, 0x9593, 0x30e9, 0x30a4, 0x30f3,
0x8af8, 0x5cf6, 0x6642, 0x9593, 0x30ed, 0x30fc, 0x30c9, 0x30cf, 0x30a6, 0x6642, 0x9593, 0x30ed,
0x30fc, 0x30c9, 0x30cf, 0x30a6, 0x6a19, 0x6e96, 0x6642, 0x30ed, 0x30fc, 0x30c9, 0x30cf, 0x30a6,
0x590f, 0x6642, 0x9593, 0x30de, 0x30ab, 0x30aa, 0x6642, 0x9593, 0x30de, 0x30ab, 0x30aa, 0x6a19,
0x6e96, 0x6642, 0x30de, 0x30ab, 0x30aa, 0x590f, 0x6642, 0x9593, 0x30de, 0x30ac, 0x30c0, 0x30f3,
0x6642, 0x9593, 0x30de, 0x30ac, 0x30c0, 0x30f3, 0x6a19, 0x6e96, 0x6642, 0x30de, 0x30ac, 0x30c0,
0x30f3, 0x590f, 0x6642, 0x9593, 0x30de, 0x30ec, 0x30fc, 0x30b7, 0x30a2, 0x6642, 0x9593, 0x30e2,
0x30eb, 0x30c7, 0x30a3, 0x30d6, 0x6642, 0x9593, 0x30de, 0x30eb, 0x30ad, 0x30fc, 0x30ba, 0x6642,
0x9593, 0x30de, 0x30fc, 0x30b7, 0x30e3, 0x30eb, 0x8af8, 0x5cf6, 0x6642, 0x9593, 0x30e2, 0x30fc,
0x30ea, 0x30b7, 0x30e3, 0x30b9, 0x6642, 0x9593, 0x30e2, 0x30fc, 0x30ea, 0x30b7, 0x30e3, 0x30b9,
0x6a19, 0x6e96, 0x6642, 0x30e2, 0x30fc, 0x30ea, 0x30b7, 0x30e3, 0x30b9, 0x590f, 0x6642, 0x9593,
0x30e2, 0x30fc, 0x30bd, 0x30f3, 0x57fa, 0x5730, 0x6642, 0x9593, 0x30e1, 0x30ad, 0x30b7, 0x30b3,
0x592a, 0x5e73, 0x6d0b, 0x6642, 0x9593, 0x30e1, 0x30ad, 0x30b7, 0x30b3, 0x592a, 0x5e73, 0x6d0b,
0x6a19, 0x6e96, 0x6642, 0x30e1, 0x30ad, 0x30b7, 0x30b3, 0x592a, 0x5e73, 0x6d0b, 0x590f, 0x6642,
0x9593, 0x30a6, 0x30e9, 0x30f3, 0x30d0, 0x30fc, 0x30c8, 0x30eb, 0x6642, 0x9593, 0x30a6, 0x30e9,
0x30f3, 0x30d0, 0x30fc, 0x30c8, 0x30eb, 0x6a19, 0x6e96, 0x6642, 0x30a6, 0x30e9, 0x30f3, 0x30d0,
0x30fc, 0x30c8, 0x30eb, 0x590f, 0x6642, 0x9593, 0x30e2, 0x30b9, 0x30af, 0x30ef, 0x6642, 0x9593,
0x30e2, 0x30b9, 0x30af, 0x30ef, 0x6a19, 0x6e96, 0x6642, 0x30e2, 0x30b9, 0x30af, 0x30ef, 0x590f,
0x6642, 0x9593, 0x30df, 0x30e3, 0x30f3, 0x30de, 0x30fc, 0x6642, 0x9593, 0x30ca, 0x30a6, 0x30eb,
0x6642, 0x9593, 0x30cd, 0x30d1, 0x30fc, 0x30eb, 0x6642, 0x9593, 0x30cb, 0x30e5, 0x30fc, 0x30ab,
0x30ec, 0x30c9, 0x30cb, 0x30a2, 0x6642, 0x9593, 0x30cb, 0x30e5, 0x30fc, 0x30ab, 0x30ec, 0x30c9,
0x30cb, 0x30a2, 0x6a19, 0x6e96, 0x6642, 0x30cb, 0x30e5, 0x30fc, 0x30ab, 0x30ec, 0x30c9, 0x30cb,
0x30a2, 0x590f, 0x6642, 0x9593, 0x30cb, 0x30e5, 0x30fc, 0x30b8, 0x30fc, 0x30e9, 0x30f3, 0x30c9,
0x6642, 0x9593, 0x30cb, 0x30e5, 0x30fc, 0x30b8, 0x30fc, 0x30e9, 0x30f3, 0x30c9, 0x6a19, 0x6e96,
0x6642, 0x30cb, 0x30e5, 0x30fc, 0x30b8, 0x30fc, 0x30e9, 0x30f3, 0x30c9, 0x590f, 0x6642, 0x9593,
0x30cb, 0x30e5, 0x30fc, 0x30d5, 0x30a1, 0x30f3, 0x30c9, 0x30e9, 0x30f3, 0x30c9, 0x6642, 0x9593,
0x30cb, 0x30e5, 0x30fc, 0x30d5, 0x30a1, 0x30f3, 0x30c9, 0x30e9, 0x30f3, 0x30c9, 0x6a19, 0x6e96,
0x6642, 0x30cb, 0x30e5, 0x30fc, 0x30d5, 0x30a1, 0x30f3, 0x30c9, 0x30e9, 0x30f3, 0x30c9, 0x590f,
0x6642, 0x9593, 0x30cb, 0x30a6, 0x30a8, 0x6642, 0x9593, 0x30ce, 0x30fc, 0x30d5, 0x30a9, 0x30fc,
0x30af, 0x5cf6, 0x6642, 0x9593, 0x30ce, 0x30fc, 0x30d5, 0x30a9, 0x30fc, 0x30af, 0x5cf6, 0x6a19,
0x6e96, 0x6642, 0x30ce, 0x30fc, 0x30d5, 0x30a9, 0x30fc, 0x30af, 0x5cf6, 0x590f, 0x6642, 0x9593,
0x30d5, 0x30a7, 0x30eb, 0x30ca, 0x30f3, 0x30c9, 0x30fb, 0x30c7, 0x30fb, 0x30ce, 0x30ed, 0x30fc,
0x30cb, 0x30e3, 0x6642, 0x9593, 0x30d5, 0x30a7, 0x30eb, 0x30ca, 0x30f3, 0x30c9, 0x30fb, 0x30c7,
0x30fb, 0x30ce, 0x30ed, 0x30fc, 0x30cb, 0x30e3, 0x6a19, 0x6e96, 0x6642, 0x30d5, 0x30a7, 0x30eb,
0x30ca, 0x30f3, 0x30c9, 0x30fb, 0x30c7, 0x30fb, 0x30ce, 0x30ed, 0x30fc, 0x30cb, 0x30e3, 0x590f,
0x6642, 0x9593, 0x5317, 0x30de, 0x30ea, 0x30a2, 0x30ca, 0x8af8, 0x5cf6, 0x6642, 0x9593, 0x30ce,
0x30f4, 0x30a9, 0x30b7, 0x30d3, 0x30eb, 0x30b9, 0x30af, 0x6642, 0x9593, 0x30ce, 0x30f4, 0x30a9,
0x30b7, 0x30d3, 0x30eb, 0x30b9, 0x30af, 0x6a19, 0x6e96, 0x6642, 0x30ce, 0x30f4, 0x30a9, 0x30b7,
0x30d3, 0x30eb, 0x30b9, 0x30af, 0x590f, 0x6642, 0x9593, 0x30aa, 0x30e0, 0x30b9, 0x30af, 0x6642,
0x9593, 0x30aa, 0x30e0, 0x30b9, 0x30af, 0x6a19, 0x6e96, 0x6642, 0x30aa, 0x30e0, 0x30b9, 0x30af,
0x590f, 0x6642, 0x9593, 0x30d1, 0x30ad, 0x30b9, 0x30bf, 0x30f3, 0x6642, 0x9593, 0x30d1, 0x30ad,
0x30b9, 0x30bf, 0x30f3, 0x6a19, 0x6e96, 0x6642, 0x30d1, 0x30ad, 0x30b9, 0x30bf, 0x30f3, 0x590f,
0x6642, 0x9593, 0x30d1, 0x30e9, 0x30aa, 0x6642, 0x9593, 0x30d1, 0x30d7, 0x30a2, 0x30cb, 0x30e5,
0x30fc, 0x30ae, 0x30cb, 0x30a2, 0x6642, 0x9593, 0x30d1, 0x30e9, 0x30b0, 0x30a2, 0x30a4, 0x6642,
0x9593, 0x30d1, 0x30e9, 0x30b0, 0x30a2, 0x30a4, 0x6a19, 0x6e96, 0x6642, 0x30d1, 0x30e9, 0x30b0,
0x30a2, 0x30a4, 0x590f, 0x6642, 0x9593, 0x30da, 0x30eb, 0x30fc, 0x6642, 0x9593, 0x30da, 0x30eb,
0x30fc, 0x6a19, 0x6e96, 0x6642, 0x30da, 0x30eb, 0x30fc, 0x590f, 0x6642, 0x9593, 0x30d5, 0x30a3,
0x30ea, 0x30d4, 0x30f3, 0x6642, 0x9593, 0x30d5, 0x30a3, 0x30ea, 0x30d4, 0x30f3, 0x6a19, 0x6e96,
0x6642, 0x30d5, 0x30a3, 0x30ea, 0x30d4, 0x30f3, 0x590f, 0x6642, 0x9593, 0x30d5, 0x30a7, 0x30cb,
0x30c3, 0x30af, 0x30b9, 0x8af8, 0x5cf6, 0x6642, 0x9593, 0x30b5, 0x30f3, 0x30d4, 0x30a8, 0x30fc,
0x30eb, 0x5cf6, 0x30fb, 0x30df, 0x30af, 0x30ed, 0x30f3, 0x5cf6, 0x6642, 0x9593, 0x30b5, 0x30f3,
0x30d4, 0x30a8, 0x30fc, 0x30eb, 0x5cf6, 0x30fb, 0x30df, 0x30af, 0x30ed, 0x30f3, 0x5cf6, 0x6a19,
0x6e96, 0x6642, 0x30b5, 0x30f3, 0x30d4, 0x30a8, 0x30fc, 0x30eb, 0x5cf6, 0x30fb, 0x30df, 0x30af,
0x30ed, 0x30f3, 0x5cf6, 0x590f, 0x6642, 0x9593, 0x30d4, 0x30c8, 0x30b1, 0x30a2, 0x30f3, 0x6642,
0x9593, 0x30dd, 0x30ca, 0x30da, 0x6642, 0x9593, 0x5e73, 0x58cc, 0x6642, 0x9593, 0x30af, 0x30ba,
0x30ed, 0x30eb, 0x30c0, 0x6642, 0x9593, 0x30af, 0x30ba, 0x30ed, 0x30eb, 0x30c0, 0x6a19, 0x6e96,
0x6642, 0x30af, 0x30ba, 0x30ed, 0x30eb, 0x30c0, 0x590f, 0x6642, 0x9593, 0x30ec, 0x30e6, 0x30cb,
0x30aa, 0x30f3, 0x6642, 0x9593, 0x30ed, 0x30bc, 0x30e9, 0x57fa, 0x5730, 0x6642, 0x9593, 0x30b5,
0x30cf, 0x30ea, 0x30f3, 0x6642, 0x9593, 0x30b5, 0x30cf, 0x30ea, 0x30f3, 0x6a19, 0x6e96, 0x6642,
0x30b5, 0x30cf, 0x30ea, 0x30f3, 0x590f, 0x6642, 0x9593, 0x30b5, 0x30de, 0x30e9, 0x6642, 0x9593,
0x30b5, 0x30de, 0x30e9, 0x6a19, 0x6e96, 0x6642, 0x30b5, 0x30de, 0x30e9, 0x590f, 0x6642, 0x9593,
0x30b5, 0x30e2, 0x30a2, 0x6642, 0x9593, 0x30b5, 0x30e2, 0x30a2, 0x6a19, 0x6e96, 0x6642, 0x30b5,
0x30e2, 0x30a2, 0x590f, 0x6642, 0x9593, 0x30bb, 0x30fc, 0x30b7, 0x30a7, 0x30eb, 0x6642, 0x9593,
0x30b7, 0x30f3, 0x30ac, 0x30dd, 0x30fc, 0x30eb, 0x6a19, 0x6e96, 0x6642, 0x30bd, 0x30ed, 0x30e2,
0x30f3, 0x8af8, 0x5cf6, 0x6642, 0x9593, 0x30b5, 0x30a6, 0x30b9, 0x30b8, 0x30e7, 0x30fc, 0x30b8,
0x30a2, 0x6642, 0x9593, 0x30b9, 0x30ea, 0x30ca, 0x30e0, 0x6642, 0x9593, 0x30bf, 0x30d2, 0x30c1,
0x6642, 0x9593, 0x53f0, 0x5317, 0x590f, 0x6642, 0x9593, 0x30bf, 0x30b8, 0x30ad, 0x30b9, 0x30bf,
0x30f3, 0x6642, 0x9593, 0x30c8, 0x30b1, 0x30e9, 0x30a6, 0x6642, 0x9593, 0x30c8, 0x30f3, 0x30ac,
0x6642, 0x9593, 0x30c8, 0x30f3, 0x30ac, 0x6a19, 0x6e96, 0x6642, 0x30c8, 0x30f3, 0x30ac, 0x590f,
0x6642, 0x9593, 0x30c1, 0x30e5, 0x30fc, 0x30af, 0x6642, 0x9593, 0x30c8, 0x30eb, 0x30af, 0x30e1,
0x30cb, 0x30b9, 0x30bf, 0x30f3, 0x6642, 0x9593, 0x30c8, 0x30eb, 0x30af, 0x30e1, 0x30cb, 0x30b9,
0x30bf, 0x30f3, 0x6a19, 0x6e96, 0x6642, 0x30c8, 0x30eb, 0x30af, 0x30e1, 0x30cb, 0x30b9, 0x30bf,
0x30f3, 0x590f, 0x6642, 0x9593, 0x30c4, 0x30d0, 0x30eb, 0x6642, 0x9593, 0x30a6, 0x30eb, 0x30b0,
0x30a2, 0x30a4, 0x6642, 0x9593, 0x30a6, 0x30eb, 0x30b0, 0x30a2, 0x30a4, 0x6a19, 0x6e96, 0x6642,
0x30a6, 0x30eb, 0x30b0, 0x30a2, 0x30a4, 0x590f, 0x6642, 0x9593, 0x30a6, 0x30ba, 0x30d9, 0x30ad,
0x30b9, 0x30bf, 0x30f3, 0x6642, 0x9593, 0x30a6, 0x30ba, 0x30d9, 0x30ad, 0x30b9, 0x30bf, 0x30f3,
0x6a19, 0x6e96, 0x6642, 0x30a6, 0x30ba, 0x30d9, 0x30ad, 0x30b9, 0x30bf, 0x30f3, 0x590f, 0x6642,
0x9593, 0x30d0, 0x30cc, 0x30a2, 0x30c4, 0x6642, 0x9593, 0x30d0, 0x30cc, 0x30a2, 0x30c4, 0x6a19,
0x6e96, 0x6642, 0x30d0, 0x30cc, 0x30a2, 0x30c4, 0x590f, 0x6642, 0x9593, 0x30d9, 0x30cd, 0x30ba,
0x30a8, 0x30e9, 0x6642, 0x9593, 0x30a6, 0x30e9, 0x30b8, 0x30aa, 0x30b9, 0x30c8, 0x30af, 0x6642,
0x9593, 0x30a6, 0x30e9, 0x30b8, 0x30aa, 0x30b9, 0x30c8, 0x30af, 0x6a19, 0x6e96, 0x6642, 0x30a6,
0x30e9, 0x30b8, 0x30aa, 0x30b9, 0x30c8, 0x30af, 0x590f, 0x6642, 0x9593, 0x30dc, 0x30eb, 0x30b4,
0x30b0, 0x30e9, 0x30fc, 0x30c9, 0x6642, 0x9593, 0x30dc, 0x30eb, 0x30b4, 0x30b0, 0x30e9, 0x30fc,
0x30c9, 0x6a19, 0x6e96, 0x6642, 0x30dc, 0x30eb, 0x30b4, 0x30b0, 0x30e9, 0x30fc, 0x30c9, 0x590f,
0x6642, 0x9593, 0x30dc, 0x30b9, 0x30c8, 0x30fc, 0x30af, 0x57fa, 0x5730, 0x6642, 0x9593, 0x30a6,
0x30a7, 0x30fc, 0x30af, 0x5cf6, 0x6642, 0x9593, 0x30a6, 0x30a9, 0x30ea, 0x30b9, 0x30fb, 0x30d5,
0x30c4, 0x30ca, 0x6642, 0x9593, 0x30e4, 0x30af, 0x30fc, 0x30c4, 0x30af, 0x6642, 0x9593, 0x30e4,
0x30af, 0x30fc, 0x30c4, 0x30af, 0x6a19, 0x6e96, 0x6642, 0x30e4, 0x30af, 0x30fc, 0x30c4, 0x30af,
0x590f, 0x6642, 0x9593, 0x30a8, 0x30ab, 0x30c6, 0x30ea, 0x30f3, 0x30d6, 0x30eb, 0x30b0, 0x6642,
0x9593, 0x30a8, 0x30ab, 0x30c6, 0x30ea, 0x30f3, 0x30d6, 0x30eb, 0x30b0, 0x6a19, 0x6e96, 0x6642,
0x30a8, 0x30ab, 0x30c6, 0x30ea, 0x30f3, 0x30d6, 0x30eb, 0x30b0, 0x590f, 0x6642, 0x9593, 0x30e6,
0x30fc, 0x30b3, 0x30f3, 0x6642, 0x9593, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x41,
0x66, 0x67, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x54,
0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x57, 0x65, 0x74, 0x61, 0x6e, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x4b, 0x69, 0x64, 0x75,
0x6c, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x65,
0x6e, 0x67, 0x61, 0x68, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f,
0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x57, 0x65, 0x74, 0x61, 0x6e,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x57, 0x65, 0x74, 0x61,
0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67,
0x6f, 0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x57, 0x65, 0x74, 0x61,
0x68, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x47, 0x69, 0x72, 0x69, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x47, 0x69, 0x72, 0x69, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x47, 0x69, 0x72, 0x69, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x50, 0x61, 0x73,
0x69, 0x66, 0x69, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x70,
0x69, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x41, 0x70, 0x69, 0x61, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x41, 0x70, 0x69,
0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x72, 0x61, 0x62, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x41, 0x72, 0x61, 0x62, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x41, 0x72, 0x61, 0x62, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f,
0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x4b, 0x75, 0x6c, 0x6f,
0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f,
0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x73,
0x69, 0x73, 0x69, 0x68, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x41, 0x72, 0x6d,
0x65, 0x6e, 0x69, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x74, 0x6c, 0x61,
0x6e, 0x74, 0x69, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65,
0x74, 0x69, 0x67, 0x6f, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x54,
0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x20, 0x73, 0x69, 0x73,
0x69, 0x68, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x54, 0x65, 0x6e, 0x67,
0x61, 0x68, 0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x4b, 0x75, 0x6c,
0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69,
0x67, 0x6f, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x20, 0x73, 0x69, 0x73, 0x69,
0x68, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73,
0x69, 0x73, 0x69, 0x68, 0x20, 0x57, 0x65, 0x74, 0x61, 0x6e, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x69,
0x73, 0x69, 0x68, 0x20, 0x57, 0x65, 0x74, 0x61, 0x6e, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x69, 0x73, 0x69,
0x68, 0x20, 0x57, 0x65, 0x74, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73,
0x69, 0x73, 0x69, 0x68, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x69,
0x73, 0x69, 0x68, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x69, 0x73, 0x69,
0x68, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20,
0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65,
0x73, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x73, 0x68, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x73, 0x68, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x73, 0x68, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x42, 0x68,
0x75, 0x74, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x42, 0x6f,
0x6c, 0x69, 0x76, 0x69, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x42,
0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c,
0x69, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x42, 0x72, 0x75, 0x6e,
0x61, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61,
0x6d, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x61, 0x6e, 0x6a, 0x75,
0x6e, 0x67, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x54, 0x61,
0x6e, 0x6a, 0x75, 0x6e, 0x67, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20,
0x54, 0x61, 0x6e, 0x6a, 0x75, 0x6e, 0x67, 0x20, 0x56, 0x65, 0x72, 0x64,
0x65, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f,
0x72, 0x72, 0x6f, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x43, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67,
0x6f, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x69, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x43, 0x68,
0x69, 0x6c, 0x69, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74,
0x69, 0x67, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x69, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x43, 0x69, 0x6e, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x43, 0x69, 0x6e,
0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67,
0x6f, 0x20, 0x43, 0x69, 0x6e, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x75, 0x6c, 0x6f, 0x20, 0x4e, 0x61, 0x74, 0x61, 0x6c, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x6f, 0x61, 0x6e,
0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4b, 0x6f,
0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d,
0x62, 0x69, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x70,
0x75, 0x6c, 0x6f, 0x61, 0x6e, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x4b, 0x65, 0x70, 0x75, 0x6c, 0x6f, 0x61, 0x6e, 0x20, 0x43, 0x6f, 0x6f,
0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67,
0x6f, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x6f, 0x61, 0x6e, 0x20, 0x43,
0x6f, 0x6f, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x75, 0x62,
0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x4b, 0x75, 0x62, 0x61,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d,
0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x4c, 0x65, 0x73,
0x74, 0x65, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x75, 0x6c, 0x6f,
0x20, 0x50, 0x61, 0x73, 0x6b, 0x61, 0x68, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x50, 0x75, 0x6c,
0x6f, 0x20, 0x50, 0x61, 0x73, 0x6b, 0x61, 0x68, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x50, 0x75, 0x6c,
0x6f, 0x20, 0x50, 0x61, 0x73, 0x6b, 0x61, 0x68, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x45, 0x6b, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x54, 0x65, 0x6e,
0x67, 0x61, 0x68, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x54,
0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x45, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20,
0x57, 0x65, 0x74, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x57, 0x65, 0x74, 0x61, 0x6e,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f,
0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x69, 0x73, 0x69, 0x68,
0x20, 0x57, 0x65, 0x74, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x45, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20,
0x57, 0x65, 0x74, 0x61, 0x6e, 0x20, 0x73, 0x65, 0x6e, 0x67, 0x20, 0x4c,
0x75, 0x77, 0x69, 0x68, 0x20, 0x41, 0x64, 0x6f, 0x68, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x69, 0x73,
0x69, 0x68, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x45, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x4b, 0x75,
0x6c, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74,
0x69, 0x67, 0x6f, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x69,
0x73, 0x69, 0x68, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x6f, 0x61, 0x6e, 0x20,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4b, 0x65,
0x70, 0x75, 0x6c, 0x6f, 0x61, 0x6e, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74,
0x69, 0x67, 0x6f, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x6f, 0x61, 0x6e,
0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x46, 0x69, 0x6a,
0x69, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67,
0x6f, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x47, 0x75, 0x69, 0x61, 0x6e, 0x61, 0x20, 0x50, 0x72, 0x61, 0x6e, 0x63,
0x69, 0x73, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x6e, 0x74, 0x61,
0x72, 0x74, 0x69, 0x6b, 0x61, 0x20, 0x6c, 0x61, 0x6e, 0x20, 0x50, 0x72,
0x61, 0x6e, 0x63, 0x69, 0x73, 0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20,
0x4b, 0x69, 0x64, 0x75, 0x6c, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x47,
0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x47, 0x65, 0x6f, 0x72,
0x67, 0x69, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x70,
0x75, 0x6c, 0x6f, 0x61, 0x6e, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72,
0x74, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x52, 0x65, 0x72, 0x61, 0x74,
0x61, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x57, 0x65, 0x74, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x47, 0x72, 0x69,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x57, 0x65, 0x74, 0x61, 0x6e, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20,
0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x57, 0x65, 0x74,
0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x47, 0x72, 0x69, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x4b, 0x75, 0x6c,
0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69,
0x67, 0x6f, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x54, 0x65, 0x6c, 0x75, 0x6b,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x48,
0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69,
0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69,
0x67, 0x6f, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x48,
0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x48, 0x6f, 0x6e,
0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x48, 0x6f, 0x76, 0x64, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x48,
0x6f, 0x76, 0x64, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x65, 0x67, 0x6f, 0x72, 0x6f, 0x20, 0x48,
0x69, 0x6e, 0x64, 0x69, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x49,
0x6e, 0x64, 0x6f, 0x63, 0x69, 0x6e, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x54,
0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x49,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x73, 0x69, 0x73,
0x69, 0x68, 0x20, 0x57, 0x65, 0x74, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20,
0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x49,
0x72, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74,
0x69, 0x67, 0x6f, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x49,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x49,
0x73, 0x72, 0x61, 0x65, 0x6c, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4a,
0x65, 0x70, 0x61, 0x6e, 0x67, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4a, 0x65, 0x70, 0x61, 0x6e,
0x67, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67,
0x6f, 0x20, 0x4a, 0x65, 0x70, 0x61, 0x6e, 0x67, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x57, 0x65, 0x74, 0x61, 0x6e, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4b, 0x6f, 0x72,
0x65, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69,
0x67, 0x6f, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73,
0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69,
0x67, 0x6f, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x69, 0x72, 0x67,
0x69, 0x7a, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x4b, 0x65, 0x70, 0x75, 0x6c, 0x6f, 0x61, 0x6e, 0x20, 0x4c, 0x69, 0x6e,
0x65, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74,
0x69, 0x67, 0x6f, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x61, 0x6c, 0x61, 0x79, 0x73, 0x69, 0x61, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x4d, 0x61, 0x6c, 0x61, 0x64, 0x65, 0x77, 0x61, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73,
0x61, 0x73, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x70, 0x75,
0x6c, 0x6f, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c,
0x6c, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x74, 0x69, 0x75, 0x73, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74,
0x69, 0x75, 0x73, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74,
0x69, 0x67, 0x6f, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f,
0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66,
0x69, 0x6b, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x4d, 0x65, 0x6b, 0x73,
0x69, 0x6b, 0x6f, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74,
0x69, 0x67, 0x6f, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20,
0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61,
0x72, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67,
0x6f, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61,
0x72, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f,
0x77, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x4d, 0x6f,
0x73, 0x63, 0x6f, 0x77, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x79,
0x61, 0x6e, 0x6d, 0x61, 0x72, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4e,
0x61, 0x75, 0x72, 0x75, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4e, 0x65,
0x70, 0x61, 0x6c, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x72,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20,
0x41, 0x6e, 0x79, 0x61, 0x72, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x69, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x72, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20,
0x41, 0x6e, 0x79, 0x61, 0x72, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x53, 0x65, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x72, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x53, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x72,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75,
0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4e, 0x65, 0x77, 0x66,
0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x4e, 0x65, 0x77,
0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x75, 0x6c, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x50, 0x75, 0x6c, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74,
0x69, 0x67, 0x6f, 0x20, 0x50, 0x75, 0x6c, 0x6f, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x68, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69,
0x67, 0x6f, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65,
0x74, 0x69, 0x67, 0x6f, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4f, 0x6d,
0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x4f, 0x6d, 0x73,
0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67,
0x6f, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x75, 0x67,
0x69, 0x6e, 0x69, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x61, 0x72,
0x61, 0x67, 0x75, 0x61, 0x79, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x79, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74,
0x69, 0x67, 0x6f, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x65, 0x72, 0x75, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x50, 0x65, 0x72, 0x75, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65,
0x74, 0x69, 0x67, 0x6f, 0x20, 0x50, 0x65, 0x72, 0x75, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x4b, 0x65, 0x70, 0x75, 0x6c, 0x6f, 0x61, 0x6e, 0x20, 0x50, 0x68, 0x6f,
0x65, 0x6e, 0x69, 0x78, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x61,
0x6e, 0x74, 0x61, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x6c,
0x61, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x20, 0x6c, 0x61, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69,
0x67, 0x6f, 0x20, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x20, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x6c, 0x61, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69,
0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20,
0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69,
0x67, 0x6f, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x65, 0x69, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70,
0x75, 0x72, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x70,
0x75, 0x6c, 0x6f, 0x61, 0x6e, 0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f,
0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x61, 0x20, 0x4b, 0x69, 0x64, 0x75, 0x6c, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x61, 0x6a, 0x69, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x6f,
0x6b, 0x65, 0x6c, 0x61, 0x75, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x43,
0x68, 0x75, 0x75, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x75,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f,
0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x75, 0x76, 0x61, 0x6c,
0x75, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x79, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20,
0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20,
0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x56, 0x65,
0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67,
0x6f, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74,
0x69, 0x67, 0x6f, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x75, 0x6c, 0x6f, 0x20,
0x57, 0x61, 0x6b, 0x65, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x57, 0x61,
0x6c, 0x6c, 0x69, 0x73, 0x20, 0x6c, 0x61, 0x6e, 0x20, 0x46, 0x75, 0x74,
0x75, 0x6e, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x59, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67,
0x6f, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x59, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x53, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0xc1, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x53, 0x75,
0x6c, 0x20, 0x64, 0x69, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x4f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72,
0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20,
0xc1, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56,
0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0xc1, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x4f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20,
0x64, 0x69, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x69, 0x20, 0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69,
0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x53,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61,
0x64, 0x72, 0x6f, 0x6e, 0x20, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x65, 0x72, 0x6f, 0x6e,
0x20, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20,
0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20,
0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x65,
0x72, 0x6f, 0x6e, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61,
0x6e, 0x68, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f,
0x6e, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x68,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x65, 0x72, 0x6f,
0x6e, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x68,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x61, 0x73, 0xed,
0x66, 0x69, 0x6b, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64, 0x72,
0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x50, 0x61, 0x73, 0xed, 0x66, 0x69,
0x6b, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x65, 0x72,
0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x50, 0x61, 0x73, 0xed, 0x66, 0x69,
0x6b, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x41, 0x74, 0x6c,
0xe1, 0x6e, 0x74, 0x69, 0x6b, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61,
0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x41, 0x74, 0x6c, 0xe1,
0x6e, 0x74, 0x69, 0x6b, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x41, 0x74, 0x6c,
0xe1, 0x6e, 0x74, 0x69, 0x6b, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x53,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61,
0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x65, 0x72, 0x6f,
0x6e, 0x20, 0x64, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c,
0x69, 0x61, 0x20, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c,
0x69, 0x61, 0x20, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x75, 0x2d, 0x4f, 0x73,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x50,
0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x53, 0x65, 0x6e, 0x74, 0x72,
0x75, 0x2d, 0x4f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20,
0x64, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61,
0x20, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x75, 0x2d, 0x4f, 0x73, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61,
0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x65, 0x72,
0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1,
0x6c, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0xe1, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e,
0x20, 0x64, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69,
0x61, 0x20, 0x4f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20,
0x64, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61,
0x20, 0x4f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x20, 0x4b, 0x61, 0x62, 0x75, 0x20, 0x56, 0x65,
0x72, 0x64, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f,
0x6e, 0x20, 0x64, 0x69, 0x20, 0x4b, 0x61, 0x62, 0x75, 0x20, 0x56, 0x65,
0x72, 0x64, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x65,
0x72, 0x6f, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x62, 0x75, 0x20,
0x56, 0x65, 0x72, 0x64, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x4b, 0x75, 0x62, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64, 0x72,
0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64,
0x69, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x53, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f,
0x6e, 0x20, 0x64, 0x69, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f,
0x72, 0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56,
0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x4f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72,
0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x73, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56,
0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x4f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x4f, 0x72, 0x61, 0x20, 0x4d, 0xe9, 0x64, 0x69, 0x75, 0x20, 0x64, 0x69,
0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x20, 0x47, 0x72, 0x6f, 0x6e, 0x65, 0x6c, 0xe1,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20,
0x64, 0x69, 0x20, 0x47, 0x72, 0x6f, 0x6e, 0x65, 0x6c, 0xe1, 0x6e, 0x64,
0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20,
0x64, 0x69, 0x20, 0x47, 0x72, 0x6f, 0x6e, 0x65, 0x6c, 0xe1, 0x6e, 0x64,
0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x47, 0x72, 0x6f, 0x6e, 0x65, 0x6c,
0xe1, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x73, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f,
0x6e, 0x20, 0x64, 0x69, 0x20, 0x47, 0x72, 0x6f, 0x6e, 0x65, 0x6c, 0xe1,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x65, 0x72,
0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x47, 0x72, 0x6f, 0x6e, 0x65, 0x6c,
0xe1, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x73, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x41, 0x76,
0x61, 0xed, 0x20, 0x69, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x61, 0x73,
0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64,
0x69, 0x20, 0x41, 0x76, 0x61, 0xed, 0x20, 0x69, 0x20, 0x41, 0x6c, 0x65,
0x75, 0x74, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56,
0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x41, 0x76, 0x61, 0xed,
0x20, 0x69, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x61, 0x73, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x61, 0x73, 0xed, 0x66, 0x69, 0x6b,
0x75, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x61, 0x6e, 0x75, 0x4f, 0x72,
0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20,
0x50, 0x61, 0x73, 0xed, 0x66, 0x69, 0x6b, 0x75, 0x20, 0x4d, 0x65, 0x78,
0x69, 0x6b, 0x61, 0x6e, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x50, 0x61, 0x73,
0xed, 0x66, 0x69, 0x6b, 0x75, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x61,
0x6e, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x54, 0x65, 0x72,
0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61,
0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x54, 0x65, 0x72, 0x61,
0x20, 0x4e, 0x6f, 0x76, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x54, 0x65, 0x72,
0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x53, 0x61, 0x6e, 0x20, 0x50, 0x65, 0x64, 0x72, 0x75, 0x20, 0x69,
0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x53, 0x61,
0x6e, 0x20, 0x50, 0x65, 0x64, 0x72, 0x75, 0x20, 0x69, 0x20, 0x4d, 0x69,
0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x53, 0x61, 0x6e,
0x20, 0x50, 0x65, 0x64, 0x72, 0x75, 0x20, 0x69, 0x20, 0x4d, 0x69, 0x6b,
0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x59,
0x75, 0x6b, 0x6f, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x41,
0x66, 0x263, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x66, 0x72, 0x69, 0x71, 0x74, 0x20,
0x74, 0x61, 0x6c, 0x65, 0x6d, 0x6d, 0x61, 0x73, 0x74, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x66, 0x72, 0x69, 0x71, 0x74, 0x20,
0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20,
0x54, 0x65, 0x66, 0x72, 0x69, 0x71, 0x74, 0x20, 0x6e, 0x20, 0x75, 0x6e,
0x1e93, 0x75, 0x6c, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x65,
0x66, 0x72, 0x69, 0x71, 0x74, 0x20, 0x6e, 0x20, 0x75, 0x6d, 0x61, 0x6c,
0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61,
0x79, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x66, 0x72, 0x69, 0x71, 0x74, 0x20,
0x6e, 0x20, 0x75, 0x6d, 0x61, 0x6c, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x54,
0x65, 0x66, 0x72, 0x69, 0x71, 0x74, 0x20, 0x6e, 0x20, 0x75, 0x6d, 0x61,
0x6c, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a,
0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64,
0x75, 0x20, 0x6e, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x41, 0x6d, 0x61, 0x1e93, 0x75, 0x6e, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20,
0x6e, 0x20, 0x41, 0x6d, 0x61, 0x1e93, 0x75, 0x6e, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20,
0x41, 0x6d, 0x61, 0x1e93, 0x75, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x74, 0x6c, 0x65, 0x6d, 0x6d, 0x61, 0x73, 0x74, 0x20, 0x6e, 0x20,
0x4d, 0x61, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x74, 0x6c,
0x65, 0x6d, 0x6d, 0x61, 0x73, 0x74, 0x20, 0x6e, 0x20, 0x4d, 0x61, 0x72,
0x69, 0x6b, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75,
0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x74, 0x6c, 0x65, 0x6d,
0x6d, 0x61, 0x73, 0x74, 0x20, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x73, 0x61,
0x6d, 0x6d, 0x61, 0x72, 0x20, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64,
0x61, 0x79, 0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72,
0x20, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20,
0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x20, 0x6e, 0x20,
0x4d, 0x61, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x79, 0x69, 0x64, 0x75, 0x72, 0x61, 0x72, 0x20, 0x6e, 0x20,
0x4d, 0x61, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x79, 0x69,
0x64, 0x75, 0x72, 0x61, 0x72, 0x20, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69,
0x6b, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e,
0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x79, 0x69, 0x64, 0x75, 0x72,
0x61, 0x72, 0x20, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x6c, 0x77, 0x69, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20,
0x61, 0x6d, 0x65, 0x6c, 0x77, 0x69, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x75,
0x6d, 0x65, 0x6c, 0x77, 0x69, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62,
0x64, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x41, 0x70, 0x79,
0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61,
0x79, 0x20, 0x6e, 0x20, 0x41, 0x70, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20,
0x41, 0x70, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x25b, 0x72,
0x61, 0x62, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x25b, 0x72, 0x61, 0x62,
0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x61, 0x25b, 0x72, 0x61, 0x62, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65,
0x62, 0x64, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x41, 0x72,
0x6a, 0x75, 0x6e, 0x74, 0x69, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61,
0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x41, 0x72, 0x6a,
0x75, 0x6e, 0x74, 0x69, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x41, 0x72, 0x6a,
0x75, 0x6e, 0x74, 0x69, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x41, 0x72, 0x6a, 0x75, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0x6e, 0x20, 0x75,
0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61,
0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x41, 0x72, 0x6a,
0x75, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d,
0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e,
0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x41, 0x72, 0x6a, 0x75, 0x6e,
0x74, 0x69, 0x6e, 0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61,
0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x41, 0x72, 0x6d, 0x69,
0x6e, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a,
0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x41, 0x72, 0x6d, 0x69, 0x6e, 0x79,
0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62,
0x64, 0x75, 0x20, 0x6e, 0x20, 0x41, 0x72, 0x6d, 0x69, 0x6e, 0x79, 0x61,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x1e6d, 0x6c, 0x61, 0x73, 0x61, 0x6e,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79,
0x20, 0x61, 0x1e6d, 0x6c, 0x61, 0x73, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x61, 0x1e6d, 0x6c, 0x61, 0x73, 0x61, 0x6e, 0x20, 0x6e, 0x20, 0x75,
0x6e, 0x65, 0x62, 0x64, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x55, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x74, 0x61, 0x6c,
0x65, 0x6d, 0x6d, 0x61, 0x73, 0x74, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61,
0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x55, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x74, 0x61, 0x6c, 0x65, 0x6d, 0x6d,
0x61, 0x73, 0x74, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e,
0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x55, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x79, 0x61, 0x20, 0x74, 0x61, 0x6c, 0x65, 0x6d, 0x6d, 0x61, 0x73,
0x74, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x74, 0x6c, 0x65, 0x6d,
0x6d, 0x61, 0x73, 0x74, 0x20, 0x6e, 0x20, 0x75, 0x6d, 0x61, 0x6c, 0x75,
0x20, 0x6e, 0x20, 0x55, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20,
0x6e, 0x20, 0x74, 0x6c, 0x65, 0x6d, 0x6d, 0x61, 0x73, 0x74, 0x20, 0x6e,
0x20, 0x75, 0x6d, 0x61, 0x6c, 0x75, 0x20, 0x6e, 0x20, 0x55, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x74, 0x6c, 0x65,
0x6d, 0x6d, 0x61, 0x73, 0x74, 0x20, 0x6e, 0x20, 0x75, 0x6d, 0x61, 0x6c,
0x75, 0x20, 0x6e, 0x20, 0x55, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x55, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x79, 0x61, 0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61,
0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61,
0x79, 0x20, 0x6e, 0x20, 0x55, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61,
0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20,
0x6e, 0x20, 0x55, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x6e,
0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x55, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x20,
0x6e, 0x20, 0x75, 0x6d, 0x61, 0x6c, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x55, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x6e, 0x20, 0x75, 0x6d, 0x61,
0x6c, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65,
0x62, 0x64, 0x75, 0x20, 0x55, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61,
0x20, 0x6e, 0x20, 0x75, 0x6d, 0x61, 0x6c, 0x75, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x41, 0x1e93, 0x65, 0x72, 0x62, 0x69, 0x1e7, 0x61, 0x6e,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79,
0x20, 0x6e, 0x20, 0x41, 0x1e93, 0x65, 0x72, 0x62, 0x69, 0x1e7, 0x61, 0x6e,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64,
0x75, 0x20, 0x6e, 0x20, 0x41, 0x1e93, 0x65, 0x72, 0x62, 0x69, 0x1e7, 0x61,
0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x41, 0x1e93, 0x75, 0x72,
0x69, 0x73, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64,
0x61, 0x79, 0x20, 0x6e, 0x20, 0x41, 0x1e93, 0x75, 0x72, 0x69, 0x73, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75,
0x20, 0x6e, 0x20, 0x41, 0x1e93, 0x75, 0x72, 0x69, 0x73, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x61,
0x63, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61,
0x79, 0x20, 0x6e, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x61,
0x63, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62,
0x64, 0x75, 0x20, 0x6e, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x61, 0x63, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x42, 0x75, 0x74,
0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x42, 0x75, 0x6c,
0x69, 0x76, 0x69, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x42, 0x72,
0x61, 0x1e93, 0x69, 0x6c, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61,
0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x42, 0x72, 0x61,
0x1e93, 0x69, 0x6c, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x42, 0x72, 0x61,
0x1e93, 0x69, 0x6c, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x42, 0x72, 0x75, 0x6e, 0x61, 0x79, 0x20, 0x44, 0x61, 0x72, 0x20, 0x53,
0x61, 0x6c, 0x61, 0x6d, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x4b,
0x61, 0x70, 0x20, 0x56, 0x69, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61,
0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x4b, 0x61, 0x70,
0x20, 0x56, 0x69, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75,
0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x4b, 0x61, 0x70, 0x20,
0x56, 0x69, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a,
0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x43, 0x61, 0x6d, 0x75, 0x72, 0x72,
0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x43, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64,
0x61, 0x79, 0x20, 0x6e, 0x20, 0x43, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75,
0x20, 0x43, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x43, 0x69, 0x6c, 0x69, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61,
0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x43, 0x69, 0x6c,
0x69, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62,
0x64, 0x75, 0x20, 0x6e, 0x20, 0x43, 0x69, 0x6c, 0x69, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x43, 0x69, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x43, 0x69,
0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62,
0x64, 0x75, 0x20, 0x6e, 0x20, 0x43, 0x69, 0x6e, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x54, 0x65, 0x67, 0x7a, 0x69, 0x72, 0x69, 0x6e, 0x20,
0x6e, 0x20, 0x4b, 0x72, 0x69, 0x73, 0x6d, 0x61, 0x73, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x67, 0x7a, 0x69, 0x72, 0x69, 0x6e,
0x20, 0x6e, 0x20, 0x4b, 0x75, 0x6b, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x4b, 0x75, 0x6c, 0x75, 0x6d, 0x62, 0x79, 0x61, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e,
0x20, 0x4b, 0x75, 0x6c, 0x75, 0x6d, 0x62, 0x79, 0x61, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e,
0x20, 0x4b, 0x75, 0x6c, 0x75, 0x6d, 0x62, 0x79, 0x61, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20,
0x54, 0x65, 0x67, 0x7a, 0x69, 0x72, 0x69, 0x6e, 0x20, 0x6e, 0x20, 0x4b,
0x75, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65,
0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x67, 0x7a, 0x69, 0x72,
0x69, 0x6e, 0x20, 0x6e, 0x20, 0x4b, 0x75, 0x6b, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x4b, 0x75,
0x62, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65,
0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d,
0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x20, 0x6e, 0x20,
0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6e, 0x20, 0x75,
0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61,
0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61,
0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62,
0x64, 0x75, 0x20, 0x6e, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x49, 0x6b, 0x77, 0x61, 0x1e6d, 0x75, 0x72, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x75, 0x72, 0x75, 0x66, 0x74,
0x20, 0x74, 0x61, 0x6c, 0x65, 0x6d, 0x6d, 0x61, 0x73, 0x74, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e,
0x20, 0x54, 0x75, 0x72, 0x75, 0x66, 0x74, 0x20, 0x74, 0x61, 0x6c, 0x65,
0x6d, 0x6d, 0x61, 0x73, 0x74, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x54, 0x75, 0x72,
0x75, 0x66, 0x74, 0x20, 0x74, 0x61, 0x6c, 0x65, 0x6d, 0x6d, 0x61, 0x73,
0x74, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x75, 0x72, 0x75,
0x66, 0x74, 0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79,
0x20, 0x6e, 0x20, 0x54, 0x75, 0x72, 0x75, 0x66, 0x74, 0x20, 0x6e, 0x20,
0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x54,
0x75, 0x72, 0x75, 0x66, 0x74, 0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d,
0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x6e, 0x69, 0x1e0d,
0x65, 0x6e, 0x20, 0x6e, 0x20, 0x54, 0x75, 0x72, 0x75, 0x66, 0x74, 0x20,
0x6e, 0x20, 0x55, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x54, 0x75, 0x72, 0x75, 0x66, 0x74, 0x20, 0x6e,
0x20, 0x75, 0x6d, 0x61, 0x6c, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61,
0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x54, 0x75, 0x72,
0x75, 0x66, 0x74, 0x20, 0x6e, 0x20, 0x75, 0x6d, 0x61, 0x6c, 0x75, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75,
0x20, 0x54, 0x75, 0x72, 0x75, 0x66, 0x74, 0x20, 0x6e, 0x20, 0x75, 0x6d,
0x61, 0x6c, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x65,
0x67, 0x7a, 0x69, 0x72, 0x69, 0x6e, 0x20, 0x6e, 0x20, 0x46, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d,
0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x67, 0x7a,
0x69, 0x72, 0x69, 0x6e, 0x20, 0x6e, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e,
0x65, 0x62, 0x64, 0x75, 0x20, 0x54, 0x65, 0x67, 0x7a, 0x69, 0x72, 0x69,
0x6e, 0x20, 0x6e, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20,
0x6e, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x46, 0x69,
0x6a, 0x69, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x47, 0x77, 0x69,
0x79, 0x61, 0x6e, 0x20, 0x74, 0x61, 0x66, 0x72, 0x61, 0x6e, 0x73, 0x69,
0x73, 0x74, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x77, 0x61, 0x6b,
0x61, 0x6c, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x1e93, 0x75, 0x6c, 0x20, 0x64,
0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x6b, 0x20, 0x6e,
0x20, 0x46, 0x72, 0x61, 0x6e, 0x73, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x75, 0x73, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x67, 0x7a, 0x69, 0x72,
0x69, 0x6e, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x79, 0x65, 0x72, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x4a, 0x79, 0x75, 0x72, 0x6a, 0x79, 0x61,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79,
0x20, 0x6e, 0x20, 0x4a, 0x79, 0x75, 0x72, 0x6a, 0x79, 0x61, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20,
0x6e, 0x20, 0x4a, 0x79, 0x75, 0x72, 0x6a, 0x79, 0x61, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x67, 0x7a, 0x69, 0x72, 0x69, 0x6e,
0x20, 0x4a, 0x69, 0x6c, 0x62, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x61, 0x6c, 0x65, 0x6d, 0x6d, 0x61, 0x73, 0x20, 0x6e, 0x20, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6e,
0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x47,
0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x73,
0x61, 0x6d, 0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x47, 0x72, 0x69,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d,
0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x47, 0x72,
0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6d, 0x61,
0x6c, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64,
0x61, 0x79, 0x20, 0x6e, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x6e, 0x20, 0x75, 0x6d, 0x61, 0x6c, 0x75, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e,
0x20, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6e, 0x20,
0x75, 0x6d, 0x61, 0x6c, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d,
0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x47, 0x75, 0x6c, 0x66,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x48, 0x61, 0x77, 0x61, 0x79,
0x2d, 0x41, 0x6c, 0x69, 0x77, 0x73, 0x79, 0x61, 0x6e, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20,
0x48, 0x61, 0x77, 0x61, 0x79, 0x2d, 0x41, 0x6c, 0x69, 0x77, 0x73, 0x79,
0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65,
0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x48, 0x61, 0x77, 0x61, 0x79, 0x2d,
0x41, 0x6c, 0x69, 0x77, 0x73, 0x79, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x48, 0x75, 0x6e, 0x67, 0x20, 0x4b, 0x75, 0x6e, 0x67,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79,
0x20, 0x6e, 0x20, 0x48, 0x75, 0x6e, 0x67, 0x20, 0x4b, 0x75, 0x6e, 0x67,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64,
0x75, 0x20, 0x6e, 0x20, 0x48, 0x75, 0x6e, 0x67, 0x20, 0x4b, 0x75, 0x6e,
0x67, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x48, 0x75, 0x76, 0x64,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79,
0x20, 0x6e, 0x20, 0x48, 0x75, 0x76, 0x64, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x48,
0x75, 0x76, 0x64, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a,
0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x4c, 0x68, 0x65, 0x6e, 0x64, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x55, 0x67, 0x61, 0x72, 0x61, 0x77,
0x20, 0x41, 0x68, 0x65, 0x6e, 0x64, 0x69, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x49, 0x6e, 0x64, 0x75, 0x63, 0x69, 0x6e, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x74, 0x6c, 0x65, 0x6d, 0x6d, 0x61, 0x73, 0x74,
0x20, 0x6e, 0x20, 0x41, 0x6e, 0x64, 0x75, 0x6e, 0x69, 0x73, 0x79, 0x61,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d,
0x61, 0x72, 0x20, 0x6e, 0x20, 0x41, 0x6e, 0x64, 0x75, 0x6e, 0x69, 0x73,
0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6d, 0x61,
0x6c, 0x75, 0x20, 0x6e, 0x20, 0x41, 0x6e, 0x64, 0x75, 0x6e, 0x69, 0x73,
0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x49, 0x72, 0x61,
0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61,
0x79, 0x20, 0x6e, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x49, 0x72,
0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65,
0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65,
0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x49, 0x7a, 0x72, 0x61,
0x79, 0x69, 0x6c, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a,
0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x49, 0x7a, 0x72, 0x61, 0x79, 0x69,
0x6c, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62,
0x64, 0x75, 0x20, 0x6e, 0x20, 0x49, 0x7a, 0x72, 0x61, 0x79, 0x69, 0x6c,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x4a, 0x61, 0x70, 0x75, 0x6e,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79,
0x20, 0x6e, 0x20, 0x4a, 0x61, 0x70, 0x75, 0x6e, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20,
0x4a, 0x61, 0x70, 0x75, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x20, 0x6e, 0x20, 0x4b, 0x61,
0x7a, 0x61, 0x78, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x6e, 0x20, 0x75, 0x6d, 0x61, 0x6c, 0x75, 0x20, 0x6e,
0x20, 0x4b, 0x61, 0x7a, 0x61, 0x78, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x4b, 0x75, 0x72, 0x79, 0x61, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20,
0x6e, 0x20, 0x4b, 0x75, 0x72, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x4b,
0x75, 0x72, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x4b,
0x6f, 0x73, 0x72, 0x61, 0x65, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x61, 0x67, 0x6e, 0x75, 0x20, 0x6e,
0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64,
0x75, 0x20, 0x6e, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x73, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x4b, 0x69,
0x72, 0x69, 0x67, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x54, 0x65, 0x67, 0x7a, 0x69, 0x72, 0x69, 0x6e, 0x20,
0x4c, 0x69, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x41, 0x6d, 0x61, 0x67, 0x6e, 0x75, 0x20, 0x6e, 0x20, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x55, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x4d, 0x61, 0x6c, 0x69, 0x7a, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x67, 0x7a, 0x69, 0x72, 0x69, 0x6e,
0x20, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x7a, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x54, 0x65, 0x67, 0x7a, 0x69, 0x72, 0x69, 0x6e, 0x20, 0x4d,
0x61, 0x72, 0x63, 0x61, 0x6c, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x4d, 0x75, 0x72, 0x69, 0x73, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d,
0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x4d, 0x75, 0x72, 0x69,
0x73, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62,
0x64, 0x75, 0x20, 0x6e, 0x20, 0x4d, 0x75, 0x72, 0x69, 0x73, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x75, 0x6e, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x6c, 0x77, 0x69, 0x20, 0x6e,
0x20, 0x4d, 0x69, 0x6b, 0x73, 0x69, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x61, 0x6d, 0x65, 0x6c,
0x77, 0x69, 0x20, 0x6e, 0x20, 0x4d, 0x69, 0x6b, 0x73, 0x69, 0x6b, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x6c, 0x77, 0x69, 0x20, 0x6e,
0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x4d, 0x69,
0x6b, 0x73, 0x69, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x55,
0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x55,
0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20,
0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x4d, 0x75, 0x1e63, 0x6b, 0x75, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20,
0x4d, 0x75, 0x1e63, 0x6b, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x4d, 0x75, 0x1e63,
0x6b, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x4d, 0x79, 0x61,
0x6e, 0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x4e,
0x75, 0x72, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x4e, 0x69,
0x70, 0x61, 0x6c, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x4b, 0x61,
0x6c, 0x69, 0x64, 0x75, 0x6e, 0x79, 0x61, 0x20, 0x54, 0x61, 0x6d, 0x61,
0x79, 0x6e, 0x75, 0x74, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65,
0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x4b, 0x61, 0x6c, 0x69, 0x64,
0x75, 0x6e, 0x79, 0x61, 0x20, 0x54, 0x61, 0x6d, 0x61, 0x79, 0x6e, 0x75,
0x74, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62,
0x64, 0x75, 0x20, 0x6e, 0x20, 0x4b, 0x61, 0x6c, 0x69, 0x64, 0x75, 0x6e,
0x79, 0x61, 0x20, 0x54, 0x61, 0x6d, 0x61, 0x79, 0x6e, 0x75, 0x74, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x5a, 0x69, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x54, 0x61, 0x6d, 0x61, 0x79, 0x6e, 0x75, 0x74, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20,
0x5a, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x61, 0x6d, 0x61, 0x79,
0x6e, 0x75, 0x74, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e,
0x65, 0x62, 0x64, 0x75, 0x20, 0x5a, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x54, 0x61, 0x6d, 0x61, 0x79, 0x6e, 0x75, 0x74, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x57, 0x61, 0x6b, 0x61, 0x6c, 0x20, 0x61, 0x6d, 0x61,
0x79, 0x6e, 0x75, 0x74, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65,
0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x57, 0x61, 0x6b, 0x61, 0x6c,
0x20, 0x61, 0x6d, 0x61, 0x79, 0x6e, 0x75, 0x74, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20,
0x57, 0x61, 0x6b, 0x61, 0x6c, 0x20, 0x61, 0x6d, 0x61, 0x79, 0x6e, 0x75,
0x74, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x4e, 0x69, 0x79, 0x75,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x67, 0x7a, 0x69,
0x72, 0x74, 0x20, 0x6e, 0x20, 0x4e, 0x75, 0x72, 0x66, 0x75, 0x6c, 0x6b,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79,
0x20, 0x6e, 0x20, 0x54, 0x65, 0x67, 0x7a, 0x69, 0x72, 0x74, 0x20, 0x6e,
0x20, 0x4e, 0x75, 0x72, 0x66, 0x75, 0x6c, 0x6b, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20,
0x54, 0x65, 0x67, 0x7a, 0x69, 0x72, 0x74, 0x20, 0x6e, 0x20, 0x4e, 0x75,
0x72, 0x66, 0x75, 0x6c, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x46, 0x69, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x4e,
0x75, 0x72, 0x75, 0x6e, 0x68, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61,
0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x46, 0x69, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x4e, 0x75, 0x72, 0x75,
0x6e, 0x68, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e,
0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x46, 0x69, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x4e, 0x75, 0x72, 0x75, 0x6e, 0x68,
0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x41, 0x6d, 0x61, 0x67, 0x6e, 0x75, 0x20, 0x6e, 0x20, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x55, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x41, 0x6d, 0x61, 0x67, 0x6e, 0x75, 0x20, 0x6e, 0x20,
0x4f, 0x6d, 0x73, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x55,
0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x4f, 0x6d, 0x73, 0x6b,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a,
0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65,
0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x50, 0x61, 0x6c,
0x61, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x50, 0x61, 0x70,
0x77, 0x61, 0x20, 0x6e, 0x20, 0x194, 0x69, 0x6e, 0x79, 0x61, 0x20, 0x54,
0x61, 0x6d, 0x61, 0x79, 0x6e, 0x75, 0x74, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x77, 0x61, 0x79, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e,
0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x77, 0x61, 0x79, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e,
0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x77, 0x61, 0x79, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x50, 0x69, 0x72, 0x75, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x50,
0x69, 0x72, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e,
0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x50, 0x69, 0x72, 0x75, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69,
0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61,
0x79, 0x20, 0x6e, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75,
0x20, 0x6e, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x67, 0x7a, 0x69, 0x72, 0x69,
0x6e, 0x20, 0x6e, 0x20, 0x46, 0x69, 0x6e, 0x69, 0x6b, 0x73, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x53, 0x61, 0x6e, 0x20, 0x50, 0x79, 0x69,
0x72, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x6b, 0x6c, 0x75, 0x6e, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e,
0x20, 0x53, 0x61, 0x6e, 0x20, 0x50, 0x79, 0x69, 0x72, 0x20, 0x26, 0x20,
0x4d, 0x69, 0x6b, 0x6c, 0x75, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x53, 0x61,
0x6e, 0x20, 0x50, 0x79, 0x69, 0x72, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x6b,
0x6c, 0x75, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x50, 0x69,
0x74, 0x6b, 0x61, 0x72, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x50, 0x79, 0x75, 0x6e, 0x67, 0x79, 0x75, 0x6e, 0x67, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x52, 0x69, 0x79, 0x75, 0x6e, 0x79, 0x75,
0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x52, 0x6f, 0x74, 0x68,
0x65, 0x72, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x53, 0x61,
0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x41,
0x6d, 0x61, 0x67, 0x6e, 0x75, 0x20, 0x6e, 0x20, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x55,
0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x53,
0x61, 0x6d, 0x77, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65,
0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x53, 0x61, 0x6d, 0x77, 0x61,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64,
0x75, 0x20, 0x6e, 0x20, 0x53, 0x61, 0x6d, 0x77, 0x61, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x53, 0x61, 0x79, 0x63, 0x61, 0x6c, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e,
0x20, 0x53, 0x61, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x67, 0x7a, 0x69, 0x72, 0x69, 0x6e,
0x20, 0x53, 0x61, 0x6c, 0x75, 0x6d, 0x75, 0x6e, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x4a, 0x79, 0x75, 0x72, 0x6a, 0x79, 0x61, 0x20, 0x6e,
0x20, 0x75, 0x6e, 0x1e93, 0x75, 0x6c, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x53, 0x79, 0x75, 0x77, 0x61, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x54, 0x61, 0x79, 0x70, 0x61, 0x79, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e,
0x20, 0x54, 0x61, 0x79, 0x70, 0x61, 0x79, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x54,
0x61, 0x79, 0x70, 0x61, 0x79, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x1e6c, 0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x1e6c, 0x75, 0x6b, 0x69, 0x6c, 0x61, 0x77,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x1e6c, 0x75, 0x6e, 0x67, 0x61,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79,
0x20, 0x6e, 0x20, 0x1e6c, 0x75, 0x6e, 0x67, 0x61, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20,
0x1e6c, 0x75, 0x6e, 0x67, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x43, 0x68, 0x75, 0x75, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79,
0x20, 0x6e, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x61, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e,
0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d,
0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x55, 0x72, 0x75, 0x67, 0x77, 0x61, 0x79, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e,
0x20, 0x55, 0x72, 0x75, 0x67, 0x77, 0x61, 0x79, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20,
0x55, 0x72, 0x75, 0x67, 0x77, 0x61, 0x79, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x55, 0x7a, 0x62, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79,
0x20, 0x6e, 0x20, 0x55, 0x7a, 0x62, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62,
0x64, 0x75, 0x20, 0x6e, 0x20, 0x55, 0x7a, 0x62, 0x69, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x56, 0x61,
0x6e, 0x75, 0x79, 0x61, 0x74, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x41,
0x6d, 0x61, 0x67, 0x6e, 0x75, 0x20, 0x6e, 0x20, 0x56, 0x61, 0x6e, 0x75,
0x79, 0x61, 0x74, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x55,
0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x56, 0x61, 0x6e, 0x75,
0x79, 0x61, 0x74, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x56,
0x69, 0x6e, 0x69, 0x7a, 0x77, 0x69, 0x6c, 0x61, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x41, 0x6d, 0x61, 0x67, 0x6e,
0x75, 0x20, 0x6e, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x55, 0x6e,
0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x41, 0x6d, 0x61, 0x67, 0x6e, 0x75, 0x20, 0x6e, 0x20,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x55, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e,
0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x67, 0x7a, 0x69, 0x72,
0x69, 0x6e, 0x20, 0x6e, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x61,
0x6b, 0x6b, 0x65, 0x64, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61,
0x79, 0x20, 0x6e, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75,
0x20, 0x6e, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x41,
0x6d, 0x61, 0x67, 0x6e, 0x75, 0x20, 0x6e, 0x20, 0x59, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x55, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e,
0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0xc8f, 0xc95, 0xcb0, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc8f, 0xc95,
0xcb0, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20,
0xcb8, 0xcae, 0xcaf, 0xc8f, 0xc95, 0xcb0, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf,
0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xcab, 0xc98, 0xcbe, 0xca8, 0xcbf,
0xcb8, 0xccd, 0xca4, 0xcbe, 0xca8, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xca7, 0xccd,
0xcaf, 0x20, 0xc86, 0xcab, 0xccd, 0xcb0, 0xcbf, 0xc95, 0xcbe, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcaa, 0xcc2, 0xcb0, 0xccd, 0xcb5, 0x20, 0xc86, 0xcab, 0xccd, 0xcb0, 0xcbf,
0xc95, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca6, 0xc95, 0xccd, 0xcb7, 0xcbf, 0xca3,
0x20, 0xc86, 0xcab, 0xccd, 0xcb0, 0xcbf, 0xc95, 0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0,
0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd,
0xc9a, 0xcbf, 0xcae, 0x20, 0xc86, 0xcab, 0xccd, 0xcb0, 0xcbf, 0xc95, 0xcbe, 0x20,
0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae, 0x20, 0xc86, 0xcab,
0xccd, 0xcb0, 0xcbf, 0xc95, 0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3,
0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae,
0x20, 0xc86, 0xcab, 0xccd, 0xcb0, 0xcbf, 0xc95, 0xcbe, 0x20, 0xcac, 0xcc7, 0xcb8,
0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xcb2, 0xcbe, 0xcb8, 0xccd,
0xc95, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xcb2, 0xcbe, 0xcb8, 0xccd, 0xc95,
0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8,
0xcae, 0xcaf, 0x200c, 0xc85, 0xcb2, 0xcbe, 0xcb8, 0xccd, 0xc95, 0xcbe, 0x20, 0xcb9,
0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc86, 0xcb2, 0xccd, 0xcae, 0xcc7,
0xc9f, 0xcbf, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc86, 0xcb2, 0xccd, 0xcae, 0xcc7, 0xc9f,
0xcbf, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc86, 0xcb2, 0xccd, 0xcae, 0xcc7, 0xc9f, 0xcbf, 0x20, 0xcac, 0xcc7,
0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xcae, 0xcc6, 0xc9c,
0xcbe, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xcae, 0xcc6, 0xc9c, 0xcbe,
0xca8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20,
0xcb8, 0xcae, 0xcaf, 0xc85, 0xcae, 0xcc6, 0xc9c, 0xcbe, 0xca8, 0xccd, 0x20, 0xcac,
0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xca4, 0xccd,
0xca4, 0xcb0, 0x20, 0xc85, 0xcae, 0xcc6, 0xcb0, 0xcbf, 0xc95, 0xca6, 0x20, 0xc95,
0xcc7, 0xc82, 0xca6, 0xccd, 0xcb0, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xca4, 0xccd,
0xca4, 0xcb0, 0x20, 0xc85, 0xcae, 0xcc6, 0xcb0, 0xcbf, 0xc95, 0xca6, 0x20, 0xc95,
0xcc7, 0xc82, 0xca6, 0xccd, 0xcb0, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3,
0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xca4, 0xccd, 0xca4, 0xcb0, 0x20,
0xc85, 0xcae, 0xcc6, 0xcb0, 0xcbf, 0xc95, 0xca6, 0x20, 0xc95, 0xcc7, 0xc82, 0xca6,
0xccd, 0xcb0, 0xcc0, 0xcaf, 0x20, 0xca6, 0xcbf, 0xca8, 0xca6, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc89, 0xca4, 0xccd, 0xca4, 0xcb0, 0x20, 0xc85, 0xcae, 0xcc6, 0xcb0, 0xcbf,
0xc95, 0xca6, 0x20, 0xcaa, 0xcc2, 0xcb0, 0xccd, 0xcb5, 0xca6, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc89, 0xca4, 0xccd, 0xca4, 0xcb0, 0x20, 0xc85, 0xcae, 0xcc6, 0xcb0, 0xcbf,
0xc95, 0xca6, 0x20, 0xcaa, 0xcc2, 0xcb0, 0xccd, 0xcb5, 0xca6, 0x20, 0xcaa, 0xccd,
0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xca4,
0xccd, 0xca4, 0xcb0, 0x20, 0xc85, 0xcae, 0xcc6, 0xcb0, 0xcbf, 0xc95, 0xca6, 0x20,
0xcaa, 0xcc2, 0xcb0, 0xccd, 0xcb5, 0xca6, 0x20, 0xca6, 0xcbf, 0xca8, 0xca6, 0x20,
0xcb8, 0xcae, 0xcaf, 0xc89, 0xca4, 0xccd, 0xca4, 0xcb0, 0x20, 0xc85, 0xcae, 0xcc6,
0xcb0, 0xcbf, 0xc95, 0xca6, 0x20, 0xcaa, 0xcb0, 0xccd, 0xcb5, 0xca4, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc89, 0xca4, 0xccd, 0xca4, 0xcb0, 0x20, 0xc85, 0xcae, 0xcc6, 0xcb0,
0xcbf, 0xc95, 0xca6, 0x20, 0xcaa, 0xcb0, 0xccd, 0xcb5, 0xca4, 0x20, 0xcaa, 0xccd,
0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xca4,
0xccd, 0xca4, 0xcb0, 0x20, 0xc85, 0xcae, 0xcc6, 0xcb0, 0xcbf, 0xc95, 0xca6, 0x20,
0xcaa, 0xcb0, 0xccd, 0xcb5, 0xca4, 0x20, 0xca6, 0xcbf, 0xca8, 0xca6, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc89, 0xca4, 0xccd, 0xca4, 0xcb0, 0x20, 0xc85, 0xcae, 0xcc6, 0xcb0,
0xcbf, 0xc95, 0xca6, 0x20, 0xcaa, 0xcc6, 0xcb8, 0xcbf, 0xcab, 0xcbf, 0xc95, 0xccd,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xca4, 0xccd, 0xca4, 0xcb0, 0x20, 0xc85, 0xcae,
0xcc6, 0xcb0, 0xcbf, 0xc95, 0xca6, 0x20, 0xcaa, 0xcc6, 0xcb8, 0xcbf, 0xcab, 0xcbf,
0xc95, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20,
0xcb8, 0xcae, 0xcaf, 0xc89, 0xca4, 0xccd, 0xca4, 0xcb0, 0x20, 0xc85, 0xcae, 0xcc6,
0xcb0, 0xcbf, 0xc95, 0xca6, 0x20, 0xcaa, 0xcc6, 0xcb8, 0xcbf, 0xcab, 0xcbf, 0xc95,
0xccd, 0x20, 0xca6, 0xcbf, 0xca8, 0xca6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xca8,
0xca1, 0xcc0, 0xcb0, 0xccd, 0x200c, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xca8, 0xca1,
0xcc0, 0xcb0, 0xccd, 0x200c, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf,
0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xca8, 0xca1, 0xcc0, 0xcb0, 0xccd, 0x200c,
0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xcaa, 0xcbf,
0xcaf, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xcaa, 0xcbf, 0xcaf, 0xcbe, 0x20,
0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc85, 0xcaa, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc85, 0xc95, 0xccd, 0xc9f, 0xccc, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85,
0xc95, 0xccd, 0xc9f, 0xccc, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf,
0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xc95, 0xccd, 0xc9f, 0xccc, 0x20, 0xcac,
0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xc95, 0xccd,
0xc9f, 0xccb, 0xcac, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xc95, 0xccd, 0xc9f,
0xccb, 0xcac, 0xcc6, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xc95, 0xccd, 0xc9f, 0xccb, 0xcac, 0xcc6, 0x20,
0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xcb0,
0xcc7, 0xcac, 0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xcb0,
0xcc7, 0xcac, 0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe,
0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xcb0, 0xcc7, 0xcac, 0xcbf,
0xcaf, 0xca8, 0xccd, 0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc85, 0xcb0, 0xccd, 0xc9c, 0xcc6, 0xc82, 0xc9f, 0xcc0, 0xca8, 0xcbe, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc85, 0xcb0, 0xccd, 0xc9c, 0xcc6, 0xc82, 0xc9f, 0xcc0, 0xca8, 0xcbe,
0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc85, 0xcb0, 0xccd, 0xc9c, 0xcc6, 0xc82, 0xc9f, 0xcbf, 0xca8, 0xcbe, 0x20,
0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6,
0xccd, 0xc9a, 0xcbf, 0xcae, 0x20, 0xc85, 0xcb0, 0xccd, 0xc9c, 0xcc6, 0xc82, 0xc9f,
0xcc0, 0xca8, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf,
0xcae, 0x20, 0xc85, 0xcb0, 0xccd, 0xc9c, 0xcc6, 0xc82, 0xc9f, 0xcc0, 0xca8, 0xcbe,
0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae, 0x20, 0xc85, 0xcb0, 0xccd, 0xc9c,
0xcc6, 0xc82, 0xc9f, 0xcc0, 0xca8, 0xcbe, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97,
0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xcb0, 0xccd, 0xcae, 0xcc7, 0xca8, 0xcbf,
0xcaf, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xcb0, 0xccd, 0xcae, 0xcc7, 0xca8, 0xcbf,
0xcaf, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc85, 0xcb0, 0xccd, 0xcae, 0xcc7, 0xca8, 0xcbf, 0xcaf, 0x20, 0xcac,
0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xc9f, 0xccd,
0xcb2, 0xcbe, 0xc82, 0xc9f, 0xcbf, 0xc95, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85,
0xc9f, 0xccd, 0xcb2, 0xcbe, 0xc82, 0xc9f, 0xcbf, 0xc95, 0xccd, 0x20, 0xcaa, 0xccd,
0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xc9f,
0xccd, 0xcb2, 0xcbe, 0xc82, 0xc9f, 0xcbf, 0xc95, 0xccd, 0x20, 0xca6, 0xcbf, 0xca8,
0xca6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xcc7, 0xc82, 0xca6, 0xccd, 0xcb0, 0x20,
0xc86, 0xcb8, 0xccd, 0xc9f, 0xccd, 0xcb0, 0xcc7, 0xcb2, 0xcbf, 0xcaf, 0xcbe, 0x20,
0xcb8, 0xcae, 0xcaf, 0xc86, 0xcb8, 0xccd, 0xc9f, 0xccd, 0xcb0, 0xcc7, 0xcb2, 0xcbf,
0xcaf, 0xcbe, 0xca6, 0x20, 0xc95, 0xcc7, 0xc82, 0xca6, 0xccd, 0xcb0, 0x20, 0xcaa,
0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc86,
0xcb8, 0xccd, 0xc9f, 0xccd, 0xcb0, 0xcc7, 0xcb2, 0xcbf, 0xcaf, 0xcbe, 0xca6, 0x20,
0xc95, 0xcc7, 0xc82, 0xca6, 0xccd, 0xcb0, 0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1, 0x20,
0xcb8, 0xcae, 0xcaf, 0xc86, 0xcb8, 0xccd, 0xc9f, 0xccd, 0xcb0, 0xcc7, 0xcb2, 0xcbf,
0xcaf, 0xcbe, 0xca6, 0x20, 0xc95, 0xcc7, 0xc82, 0xca6, 0xccd, 0xcb0, 0x20, 0xcaa,
0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc86, 0xcb8, 0xccd,
0xc9f, 0xccd, 0xcb0, 0xcc7, 0xcb2, 0xcbf, 0xcaf, 0xcbe, 0xca6, 0x20, 0xc95, 0xcc7,
0xc82, 0xca6, 0xccd, 0xcb0, 0x20, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae, 0x20,
0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc86, 0xcb8, 0xccd, 0xc9f, 0xccd, 0xcb0, 0xcc7, 0xcb2, 0xcbf, 0xcaf, 0xcbe, 0xca6,
0x20, 0xc95, 0xcc7, 0xc82, 0xca6, 0xccd, 0xcb0, 0x20, 0xcaa, 0xcb6, 0xccd, 0xc9a,
0xcbf, 0xcae, 0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa,
0xcc2, 0xcb0, 0xccd, 0xcb5, 0x20, 0xc86, 0xcb8, 0xccd, 0xc9f, 0xccd, 0xcb0, 0xcc7,
0xcb2, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc86, 0xcb8, 0xccd, 0xc9f,
0xccd, 0xcb0, 0xcc7, 0xcb2, 0xcbf, 0xcaf, 0xcbe, 0xca6, 0x20, 0xcaa, 0xcc2, 0xcb0,
0xccd, 0xcb5, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20,
0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcc2, 0xcb0, 0xccd, 0xcb5, 0x20, 0xc86, 0xcb8, 0xccd,
0xc9f, 0xccd, 0xcb0, 0xcc7, 0xcb2, 0xcbf, 0xcaf, 0xcbe, 0xca6, 0x20, 0xcb9, 0xc97,
0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae,
0x20, 0xc86, 0xcb8, 0xccd, 0xc9f, 0xccd, 0xcb0, 0xcc7, 0xcb2, 0xcbf, 0xcaf, 0xcbe,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc86, 0xcb8, 0xccd, 0xc9f, 0xccd, 0xcb0, 0xcc7, 0xcb2,
0xcbf, 0xcaf, 0xcbe, 0xca6, 0x20, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae, 0x20,
0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc86, 0xcb8, 0xccd, 0xc9f, 0xccd, 0xcb0, 0xcc7, 0xcb2, 0xcbf, 0xcaf, 0xcbe, 0xca6,
0x20, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae, 0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xc9c, 0xcb0, 0xccd, 0xcac, 0xcc8, 0xc9c, 0xcbe,
0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xc9c, 0xcb0, 0xccd, 0xcac, 0xcc8,
0xc9c, 0xcbe, 0xca8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf,
0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xc9c, 0xcb0, 0xccd, 0xcac, 0xcc8, 0xc9c,
0xcbe, 0xca8, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc85, 0xc9c, 0xccb, 0xcb0, 0xcb8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc85, 0xc9c, 0xccb, 0xcb0, 0xcb8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe,
0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xc9c, 0xccb, 0xcb0, 0xcb8,
0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcac, 0xcbe, 0xc82, 0xc97, 0xccd, 0xcb2, 0xcbe, 0xca6, 0xcc7, 0xcb6, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcac, 0xcbe, 0xc82, 0xc97, 0xccd, 0xcb2, 0xcbe, 0xca6, 0xcc7, 0xcb6,
0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcac, 0xcbe, 0xc82, 0xc97, 0xccd, 0xcb2, 0xcbe, 0xca6, 0xcc7, 0xcb6, 0x20,
0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcad, 0xcc2,
0xca4, 0xcbe, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcac, 0xcca, 0xcb2, 0xcbf,
0xcb5, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcac, 0xccd, 0xcb0, 0xcc6,
0xcb8, 0xcbf, 0xcb2, 0xcbf, 0xcaf, 0xcbe, 0xca6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcac,
0xccd, 0xcb0, 0xcc6, 0xcb8, 0xcbf, 0xcb2, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcaa, 0xccd,
0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcac, 0xccd,
0xcb0, 0xcc6, 0xcb8, 0xcbf, 0xcb2, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcac, 0xcc7, 0xcb8,
0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcac, 0xccd, 0xcb0, 0xcc2, 0xca8,
0xcbf, 0x20, 0xca6, 0xcbe, 0xcb0, 0xcc1, 0xcb8, 0xcb2, 0xcc6, 0xcae, 0xccd, 0x20,
0xcb8, 0xcae, 0xcaf, 0xc95, 0xcc7, 0xcaa, 0xccd, 0x20, 0xcb5, 0xcb0, 0xccd, 0xca1,
0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xcc7, 0xcaa, 0xccd, 0x20, 0xcb5, 0xcb0,
0xccd, 0xca1, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xcc7, 0xcaa, 0xccd, 0x20, 0xcb5, 0xcb0, 0xccd,
0xca1, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc9a, 0xcae, 0xccb, 0xcb0, 0xccb, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe,
0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9a, 0xca5, 0xcbe, 0xcae, 0xccd,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc9a, 0xca5, 0xcbe, 0xcae, 0xccd, 0x20, 0xcaa, 0xccd,
0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9a, 0xca5,
0xcbe, 0xcae, 0xccd, 0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc9a, 0xcbf, 0xcb2, 0xcbf, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9a, 0xcbf, 0xcb2, 0xcbf,
0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc9a, 0xcbf, 0xcb2, 0xcbf, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc9a, 0xcc0, 0xca8, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc9a, 0xcc0, 0xca8, 0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf,
0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9a, 0xcc0, 0xca8, 0xcbe, 0x20, 0xcb9, 0xc97,
0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xccd, 0xcb0, 0xcbf, 0xcb8, 0xccd,
0xcae, 0xcb8, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc95, 0xccb, 0xc95, 0xccb, 0xcb8, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0,
0xcaa, 0xc97, 0xcb3, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xcca, 0xcb2, 0xc82, 0xcac,
0xcbf, 0xcaf, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xcca, 0xcb2, 0xc82, 0xcac,
0xcbf, 0xcaf, 0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xcca, 0xcb2, 0xc82, 0xcac, 0xcbf, 0xcaf, 0xcbe,
0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95,
0xcc1, 0xc95, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa, 0xc97, 0xcb3, 0x20,
0xcb8, 0xcae, 0xcaf, 0xc95, 0xcc1, 0xc95, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0,
0xcaa, 0xc97, 0xcb3, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xcc1, 0xc95, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5,
0xcc0, 0xcaa, 0xc97, 0xcb3, 0x20, 0xcae, 0xca7, 0xccd, 0xcaf, 0xc95, 0xcbe, 0xcb2,
0xcc0, 0xca8, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc95, 0xccd, 0xcaf, 0xcc2, 0xcac, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95,
0xccd, 0xcaf, 0xcc2, 0xcac, 0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3,
0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xccd, 0xcaf, 0xcc2, 0xcac, 0xcbe,
0x20, 0xca6, 0xcbf, 0xca8, 0xca6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca1, 0xcc7, 0xcb5,
0xcbf, 0xcb8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca1, 0xcc1, 0xcae, 0xc82, 0xc9f,
0xccd, 0x2d, 0xca1, 0xcbf, 0x20, 0xc89, 0xcb0, 0xccd, 0xcb5, 0xcbf, 0xcb2, 0xcc6,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcc2, 0xcb0, 0xccd, 0xcb5, 0x20, 0xc9f, 0xcbf,
0xcae, 0xccb, 0xcb0, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc88, 0xcb8, 0xccd, 0xc9f,
0xcb0, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc88, 0xcb8, 0xccd, 0xc9f, 0xcb0, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa,
0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc88, 0xcb8, 0xccd, 0xc9f, 0xcb0, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0,
0xcaa, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc88, 0xc95, 0xccd, 0xcb5, 0xcc6, 0xca1, 0xcbe, 0xcb0, 0xccd, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcae, 0xca7, 0xccd, 0xcaf, 0x20, 0xcaf, 0xcc1, 0xcb0, 0xccb, 0xcaa, 0xcbf,
0xcaf, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xca7, 0xccd, 0xcaf, 0x20,
0xcaf, 0xcc1, 0xcb0, 0xccb, 0xcaa, 0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcaa, 0xccd,
0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xca7,
0xccd, 0xcaf, 0x20, 0xcaf, 0xcc1, 0xcb0, 0xccb, 0xcaa, 0xcbf, 0xcaf, 0xca8, 0xccd,
0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa,
0xcc2, 0xcb0, 0xccd, 0xcb5, 0x20, 0xcaf, 0xcc1, 0xcb0, 0xccb, 0xcaa, 0xcbf, 0xcaf,
0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcc2, 0xcb0, 0xccd, 0xcb5, 0x20,
0xcaf, 0xcc1, 0xcb0, 0xccb, 0xcaa, 0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcaa, 0xccd,
0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcc2,
0xcb0, 0xccd, 0xcb5, 0x20, 0xcaf, 0xcc1, 0xcb0, 0xccb, 0xcaa, 0xcbf, 0xcaf, 0xca8,
0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcae, 0xca4, 0xccd, 0xca4, 0xcb7, 0xccd, 0xc9f, 0xcc1, 0x2d, 0xcaa, 0xcc2, 0xcb0,
0xccd, 0xcb5, 0x20, 0xcaf, 0xcc1, 0xcb0, 0xccb, 0xcaa, 0xcbf, 0xcaf, 0xca8, 0xccd,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae, 0x20, 0xcaf,
0xcc1, 0xcb0, 0xccb, 0xcaa, 0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae, 0x20, 0xcaf, 0xcc1, 0xcb0, 0xccb, 0xcaa,
0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf,
0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae, 0x20,
0xcaf, 0xcc1, 0xcb0, 0xccb, 0xcaa, 0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcac, 0xcc7,
0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcab, 0xcbe, 0xc95, 0xccd,
0x200c, 0xcb2, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5,
0xcc0, 0xcaa, 0xc97, 0xcb3, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcab, 0xcbe, 0xc95, 0xccd,
0x200c, 0xcb2, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5,
0xcc0, 0xcaa, 0xc97, 0xcb3, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf,
0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcab, 0xcbe, 0xc95, 0xccd, 0x200c, 0xcb2, 0xccd,
0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa, 0xc97,
0xcb3, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcab, 0xcbf, 0xc9c, 0xcbf, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcab, 0xcbf, 0xc9c, 0xcbf,
0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcab, 0xcbf, 0xc9c, 0xcbf, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcab, 0xccd, 0xcb0, 0xcc6, 0xc82, 0xc9a, 0xccd, 0x20,
0xc97, 0xcaf, 0xcbe, 0xca8, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca6, 0xc95, 0xccd,
0xcb7, 0xcbf, 0xca3, 0x20, 0xcab, 0xccd, 0xcb0, 0xcc6, 0xc82, 0xc9a, 0xccd, 0x20,
0xcae, 0xca4, 0xccd, 0xca4, 0xcc1, 0x20, 0xc85, 0xc82, 0xc9f, 0xcbe, 0xcb0, 0xccd,
0xc9f, 0xcbf, 0xc95, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc97, 0xcbe, 0xcb2, 0xcbe,
0xcaa, 0xcbe, 0xc97, 0xccb, 0xcb8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc97, 0xccd,
0xcaf, 0xcbe, 0xc82, 0xcac, 0xcbf, 0xcaf, 0xcb0, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc9c, 0xcbe, 0xcb0, 0xccd, 0xc9c, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc9c, 0xcbe, 0xcb0, 0xccd, 0xc9c, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0,
0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9c, 0xcbe, 0xcb0,
0xccd, 0xc9c, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc97, 0xcbf, 0xcb2, 0xccd, 0xcac, 0xcb0, 0xccd, 0xc9f,
0xccd, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa, 0xc97, 0xcb3, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc97, 0xccd, 0xcb0, 0xcc0, 0xca8, 0xccd, 0x200c, 0xcb5, 0xcbf, 0xc9a, 0xccd,
0x20, 0xcb8, 0xcb0, 0xcbe, 0xcb8, 0xcb0, 0xcbf, 0x20, 0xc95, 0xcbe, 0xcb2, 0xcae,
0xcbe, 0xca8, 0xcaa, 0xcc2, 0xcb0, 0xccd, 0xcb5, 0x20, 0xc97, 0xccd, 0xcb0, 0xcc0,
0xca8, 0xccd, 0x200c, 0xcb2, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcaa, 0xcc2, 0xcb0, 0xccd, 0xcb5, 0x20, 0xc97, 0xccd, 0xcb0, 0xcc0,
0xca8, 0xccd, 0x200c, 0xcb2, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd, 0x20, 0xcaa,
0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa,
0xcc2, 0xcb0, 0xccd, 0xcb5, 0x20, 0xc97, 0xccd, 0xcb0, 0xcc0, 0xca8, 0xccd, 0x200c,
0xcb2, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf,
0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae,
0x20, 0xc97, 0xccd, 0xcb0, 0xcc0, 0xca8, 0xccd, 0x200c, 0xcb2, 0xccd, 0xcaf, 0xcbe,
0xc82, 0xca1, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf,
0xcae, 0x20, 0xc97, 0xccd, 0xcb0, 0xcc0, 0xca8, 0xccd, 0x200c, 0xcb2, 0xccd, 0xcaf,
0xcbe, 0xc82, 0xca1, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf,
0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae, 0x20,
0xc97, 0xccd, 0xcb0, 0xcc0, 0xca8, 0xccd, 0x200c, 0xcb2, 0xccd, 0xcaf, 0xcbe, 0xc82,
0xca1, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc97, 0xcc1, 0xcb5, 0xcbe, 0xcae, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae,
0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc97, 0xcb2, 0xccd, 0xcab,
0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcb9, 0xcb5, 0xcbe, 0xcaf, 0xcbf, 0x2d, 0xc85, 0xcb2, 0xccd, 0xcaf,
0xcc1, 0xc9f, 0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb9, 0xcb5,
0xcbe, 0xcaf, 0xcbf, 0x2d, 0xc85, 0xcb2, 0xccd, 0xcaf, 0xcc1, 0xc9f, 0xcbf, 0xcaf,
0xca8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20,
0xcb8, 0xcae, 0xcaf, 0xcb9, 0xcb5, 0xcbe, 0xcaf, 0xcbf, 0x2d, 0xc85, 0xcb2, 0xccd,
0xcaf, 0xcc1, 0xc9f, 0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcb9, 0xcbe, 0xc82, 0xc97, 0xccd, 0x20, 0xc95, 0xcbe,
0xc82, 0xc97, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb9, 0xcbe, 0xc82, 0xc97, 0xccd,
0x20, 0xc95, 0xcbe, 0xc82, 0xc97, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe,
0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb9, 0xcbe, 0xc82, 0xc97, 0xccd,
0x20, 0xc95, 0xcbe, 0xc82, 0xc97, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97,
0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb9, 0xcb5, 0xccd, 0xca1, 0xccd, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcb9, 0xcb5, 0xccd, 0xca1, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae,
0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb9, 0xcb5, 0xccd, 0xca1,
0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcad, 0xcbe, 0xcb0, 0xca4, 0xcc0, 0xcaf, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe,
0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb9, 0xcbf, 0xc82, 0xca6, 0xcc2,
0x20, 0xcae, 0xcb9, 0xcbe, 0xcb8, 0xcbe, 0xc97, 0xcb0, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc87, 0xc82, 0xca1, 0xcca, 0xc9a, 0xcc8, 0xca8, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcae, 0xca7, 0xccd, 0xcaf, 0x20, 0xc87, 0xc82, 0xca1, 0xccb, 0xca8, 0xcc7, 0xcb7,
0xcbf, 0xcaf, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcc2, 0xcb0, 0xccd, 0xcb5,
0x20, 0xc87, 0xc82, 0xca1, 0xccb, 0xca8, 0xcc7, 0xcb7, 0xcbf, 0xcaf, 0xcbe, 0x20,
0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae, 0x20, 0xc87, 0xc82,
0xca1, 0xccb, 0xca8, 0xcc7, 0xcb7, 0xcbf, 0xcaf, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc87,
0xcb0, 0xcbe, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc87, 0xcb0, 0xcbe, 0xca8,
0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc87, 0xcb0, 0xcbe, 0xca8, 0xccd, 0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc87, 0xcb0, 0xccd, 0x200c, 0xc95, 0xcc1, 0xc9f, 0xcb8,
0xccd, 0xc95, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc87, 0xcb0, 0xccd, 0x200c, 0xc95,
0xcc1, 0xc9f, 0xcb8, 0xccd, 0xc95, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe,
0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc87, 0xcb0, 0xccd, 0x200c, 0xc95,
0xcc1, 0xc9f, 0xcb8, 0xccd, 0xc95, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97,
0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc87, 0xcb8, 0xccd, 0xcb0, 0xcc7, 0xcb2, 0xccd,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc87, 0xcb8, 0xccd, 0xcb0, 0xcc7, 0xcb2, 0xccd, 0x20,
0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc87, 0xcb8, 0xccd, 0xcb0, 0xcc7, 0xcb2, 0xccd, 0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc9c, 0xcaa, 0xcbe, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc9c, 0xcaa, 0xcbe, 0xca8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe,
0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9c, 0xcaa, 0xcbe, 0xca8, 0xccd,
0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcc6, 0xca4,
0xccd, 0xcb0, 0xccb, 0xcaa, 0xcbe, 0xcb5, 0xccd, 0xcb2, 0xccb, 0xcb8, 0xccd, 0xc95,
0xccd, 0x200c, 0x200c, 0x2d, 0xc95, 0xcae, 0xccd, 0xc9a, 0xca4, 0xccd, 0xcb8, 0xccd,
0x200c, 0xc95, 0xcc0, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcc6, 0xca4, 0xccd, 0xcb0,
0xccb, 0xcaa, 0xcbe, 0xcb5, 0xccd, 0xcb2, 0xccb, 0xcb8, 0xccd, 0xc95, 0xccd, 0x200c,
0x200c, 0x2d, 0xc95, 0xcae, 0xccd, 0xc9a, 0xca4, 0xccd, 0xcb8, 0xccd, 0x200c, 0xc95,
0xcc0, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcaa, 0xcc6, 0xca4, 0xccd, 0xcb0, 0xccb, 0xcaa, 0xcbe, 0xcb5, 0xccd,
0xcb2, 0xccb, 0xcb8, 0xccd, 0xc95, 0xccd, 0x200c, 0x200c, 0x2d, 0xc95, 0xcae, 0xccd,
0xc9a, 0xca4, 0xccd, 0xcb8, 0xccd, 0x200c, 0xc95, 0xcc0, 0x20, 0xcac, 0xcc7, 0xcb8,
0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xc9d, 0xcbe, 0xc96, 0xccd,
0x200c, 0xcb8, 0xccd, 0xca4, 0xcbe, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa,
0xcc2, 0xcb0, 0xccd, 0xcb5, 0x20, 0xc95, 0xc9d, 0xcbe, 0xc96, 0xccd, 0x200c, 0xcb8,
0xccd, 0xca4, 0xcbe, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd,
0xc9a, 0xcbf, 0xcae, 0x20, 0xc95, 0xc9d, 0xcbe, 0xc96, 0xccd, 0x200c, 0xcb8, 0xccd,
0xca4, 0xcbe, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xcca, 0xcb0, 0xcbf,
0xcaf, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xcca, 0xcb0, 0xcbf, 0xcaf,
0xca8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20,
0xcb8, 0xcae, 0xcaf, 0xc95, 0xcca, 0xcb0, 0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcb9,
0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xcca, 0xcb8, 0xcb0, 0xcc6,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xccd, 0xcb0, 0xcbe, 0xcb8, 0xccd, 0x200c, 0xca8,
0xcca, 0xcaf, 0xcbe, 0xcb0, 0xccd, 0xcb8, 0xccd, 0xc95, 0xccd, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc95, 0xccd, 0xcb0, 0xcbe, 0xcb8, 0xccd, 0x200c, 0xca8, 0xcca, 0xcaf, 0xcbe,
0xcb0, 0xccd, 0xcb8, 0xccd, 0xc95, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe,
0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xccd, 0xcb0, 0xcbe, 0xcb8,
0xccd, 0x200c, 0xca8, 0xcca, 0xcaf, 0xcbe, 0xcb0, 0xccd, 0xcb8, 0xccd, 0xc95, 0xccd,
0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95,
0xcbf, 0xcb0, 0xccd, 0xc97, 0xcbf, 0xcb8, 0xccd, 0xca4, 0xcbe, 0xca8, 0xccd, 0x20,
0xcb8, 0xcae, 0xcaf, 0xcb2, 0xc82, 0xc95, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb2,
0xcc8, 0xca8, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa, 0xc97, 0xcb3, 0x20,
0xcb8, 0xcae, 0xcaf, 0xcb2, 0xcbe, 0xcb0, 0xccd, 0xca1, 0xccd, 0x20, 0xcb9, 0xccb,
0xcb5, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb2, 0xcbe, 0xcb0, 0xccd, 0xca1, 0xccd,
0x20, 0xcb9, 0xccb, 0xcb5, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3,
0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb2, 0xcbe, 0xcb0, 0xccd, 0xca1, 0xccd,
0x20, 0xcb9, 0xccb, 0xcb5, 0xccd, 0x20, 0xcac, 0xcc6, 0xcb3, 0xc97, 0xcbf, 0xca8,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xc95, 0xcbe, 0xcb5, 0xccd, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcae, 0xc95, 0xcbe, 0xcb5, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe,
0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xc95, 0xcbe, 0xcb5, 0xccd,
0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae,
0xc97, 0xca1, 0xcbe, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xc97, 0xca1,
0xcbe, 0xca8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xc97, 0xca1, 0xcbe, 0xca8, 0xccd, 0x20, 0xcac,
0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xcb2, 0xcc7,
0xcb7, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xcbe, 0xcb2, 0xccd,
0xca1, 0xcc0, 0xcb5, 0xccd, 0xcb8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xcbe,
0xcb0, 0xccd, 0xc95, 0xccd, 0xcaf, 0xcc1, 0xcb8, 0xcb8, 0xccd, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcae, 0xcbe, 0xcb0, 0xccd, 0xcb7, 0xcb2, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5,
0xcc0, 0xcaa, 0xc97, 0xcb3, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xcbe, 0xcb0, 0xcbf,
0xcb7, 0xcb8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xcbe, 0xcb0, 0xcbf, 0xcb7,
0xcb8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20,
0xcb8, 0xcae, 0xcaf, 0xcae, 0xcbe, 0xcb0, 0xcbf, 0xcb7, 0xcb8, 0xccd, 0x20, 0xcac,
0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xcbe, 0xcb5,
0xccd, 0x200c, 0xcb8, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xcc6, 0xc95,
0xccd, 0xcb8, 0xcbf, 0xc95, 0xca8, 0xccd, 0x20, 0xcaa, 0xcc6, 0xcb8, 0xcbf, 0xcab,
0xcbf, 0xc95, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xcc6, 0xc95, 0xccd, 0xcb8,
0xcbf, 0xc95, 0xca8, 0xccd, 0x20, 0xcaa, 0xcc6, 0xcb8, 0xcbf, 0xcab, 0xcbf, 0xc95,
0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcae, 0xcc6, 0xc95, 0xccd, 0xcb8, 0xcbf, 0xc95, 0xca8, 0xccd, 0x20,
0xcaa, 0xcc6, 0xcb8, 0xcbf, 0xcab, 0xcbf, 0xc95, 0xccd, 0x20, 0xcb9, 0xc97, 0xcb2,
0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xcb2, 0xcbe, 0xca8, 0xccd, 0x20, 0xcac,
0xca4, 0xcca, 0xcb0, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xcb2, 0xcbe, 0xca8,
0xccd, 0x20, 0xcac, 0xca4, 0xcca, 0xcb0, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae,
0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xcb2, 0xcbe, 0xca8,
0xccd, 0x20, 0xcac, 0xca4, 0xcca, 0xcb0, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf,
0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xcbe, 0xcb8, 0xccd, 0xc95, 0xccb,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xcbe, 0xcb8, 0xccd, 0xc95, 0xccb, 0x20, 0xcaa,
0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae,
0xcbe, 0xcb8, 0xccd, 0xc95, 0xccb, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xccd, 0xcaf, 0xcbe, 0xca8, 0xccd, 0xcae, 0xcbe,
0xcb0, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca8, 0xccc, 0xcb0, 0xcc1, 0x20, 0xcb8,
0xcae, 0xcaf, 0xca8, 0xcc7, 0xcaa, 0xcbe, 0xcb3, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb9,
0xcca, 0xcb8, 0x20, 0xc95, 0xccd, 0xcaf, 0xcbe, 0xcb2, 0xcc6, 0xca1, 0xccb, 0xca8,
0xcbf, 0xcaf, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb9, 0xcca, 0xcb8, 0x20, 0xc95,
0xccd, 0xcaf, 0xcbe, 0xcb2, 0xcc6, 0xca1, 0xccb, 0xca8, 0xcbf, 0xcaf, 0xcbe, 0x20,
0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcb9, 0xcca, 0xcb8, 0x20, 0xc95, 0xccd, 0xcaf, 0xcbe, 0xcb2, 0xcc6, 0xca1, 0xccb,
0xca8, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0xcaf,
0x20, 0xcb8, 0xcae, 0xcaf, 0xca8, 0xccd, 0xcaf, 0xcc2, 0xc9c, 0xcbf, 0xcb2, 0xccd,
0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca8, 0xccd, 0xcaf,
0xcc2, 0xc9c, 0xcbf, 0xcb2, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd, 0x20, 0xcaa,
0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca8,
0xccd, 0xcaf, 0xcc2, 0xc9c, 0xcbf, 0xcb2, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd,
0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca8, 0xccd, 0xcaf,
0xcc2, 0xcab, 0xccc, 0xc82, 0xca1, 0xccd, 0x200c, 0xcb2, 0xccd, 0xcaf, 0xcbe, 0xc82,
0xca1, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca8, 0xccd, 0xcaf, 0xcc2, 0xcab, 0xccc,
0xc82, 0xca1, 0xccd, 0x200c, 0xcb2, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd, 0x20,
0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf,
0xca8, 0xccd, 0xcaf, 0xcc2, 0xcab, 0xccc, 0xc82, 0xca1, 0xccd, 0x200c, 0xcb2, 0xccd,
0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd, 0x20, 0xca6, 0xcbf, 0xca8, 0xca6, 0x20, 0xcb8,
0xcae, 0xcaf, 0xca8, 0xcbf, 0xcaf, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca8, 0xcbe,
0xcb0, 0xccd, 0xcab, 0xccb, 0xc95, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa,
0x20, 0xcb8, 0xcae, 0xcaf, 0xca8, 0xcbe, 0xcb0, 0xccd, 0xcab, 0xccb, 0xc95, 0xccd,
0x20, 0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe,
0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca8, 0xcbe, 0xcb0, 0xccd, 0xcab,
0xccb, 0xc95, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa, 0x20, 0xcac, 0xcc7,
0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcab, 0xcc6, 0xcb0, 0xccd,
0xca8, 0xcbe, 0xc82, 0xca1, 0xccb, 0x20, 0xca1, 0xcc6, 0x20, 0xca8, 0xcca, 0xcb0,
0xcca, 0xca8, 0xccd, 0xcb9, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcab, 0xcc6, 0xcb0,
0xccd, 0xca8, 0xcbe, 0xc82, 0xca1, 0xccb, 0x20, 0xca1, 0xcc6, 0x20, 0xca8, 0xcca,
0xcb0, 0xcca, 0xca8, 0xccd, 0xcb9, 0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe,
0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcab, 0xcb0, 0xccd, 0xca8, 0xcbe,
0xc82, 0xca1, 0xccb, 0x20, 0xca6, 0xcc6, 0x20, 0xca8, 0xcca, 0xcb0, 0xcca, 0xca8,
0xccd, 0xcb9, 0xcbe, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc89, 0xca4, 0xccd, 0xca4, 0xcb0, 0x20, 0xcae, 0xcb0, 0xcbf, 0xcaf,
0xcbe, 0xca8, 0xcbe, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa, 0xc97, 0xcb3, 0x20,
0xcb8, 0xcae, 0xcaf, 0xca8, 0xcca, 0xcb5, 0xcca, 0xcb8, 0xcbf, 0xcac, 0xcbf, 0xcb0,
0xccd, 0x200c, 0xcb8, 0xccd, 0xc95, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca8, 0xcca,
0xcb5, 0xcca, 0xcb8, 0xcbf, 0xcac, 0xcbf, 0xcb0, 0xccd, 0x200c, 0xcb8, 0xccd, 0xc95,
0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8,
0xcae, 0xcaf, 0xca8, 0xcca, 0xcb5, 0xcca, 0xcb8, 0xcbf, 0xcac, 0xcbf, 0xcb0, 0xccd,
0x200c, 0xcb8, 0xccd, 0xc95, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc92, 0xcae, 0xcbe, 0xcb8, 0xccd, 0xc95, 0xccd, 0x20,
0xcb8, 0xcae, 0xcaf, 0xc92, 0xcae, 0xcbe, 0xcb8, 0xccd, 0xc95, 0xccd, 0x20, 0xcaa,
0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc92,
0xcae, 0xcbe, 0xcb8, 0xccd, 0xc95, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97,
0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcbe, 0xc95, 0xcbf, 0xcb8, 0xccd, 0xca4,
0xcbe, 0xca8, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcbe, 0xc95, 0xcbf, 0xcb8, 0xccd,
0xca4, 0xcbe, 0xca8, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcbe, 0xc95, 0xcbf, 0xcb8, 0xccd, 0xca4, 0xcbe,
0xca8, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcaa, 0xcbe, 0xcb2, 0xcbe, 0xcb5, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcaa,
0xcc1, 0xcb5, 0xcbe, 0x20, 0xca8, 0xccd, 0xcaf, 0xcc2, 0x20, 0xc97, 0xcbf, 0xca8,
0xcbf, 0xcaf, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb0, 0xcbe, 0xc97, 0xccd,
0xcb5, 0xcc7, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb0, 0xcbe, 0xc97, 0xccd, 0xcb5,
0xcc7, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcaa, 0xcb0, 0xcbe, 0xc97, 0xccd, 0xcb5, 0xcc7, 0x20, 0xcac, 0xcc7,
0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcc6, 0xcb0, 0xcc1,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcc6, 0xcb0, 0xcc1, 0x20, 0xcaa, 0xccd, 0xcb0,
0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcc6, 0xcb0,
0xcc1, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcab, 0xcbf, 0xcb2, 0xcbf, 0xcab, 0xcc8, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcab, 0xcbf, 0xcb2, 0xcbf, 0xcab, 0xcc8, 0xca8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0,
0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcab, 0xcbf, 0xcb2,
0xcbf, 0xcab, 0xcc8, 0xca8, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcab, 0xcbf, 0xca8, 0xcbf, 0xc95, 0xccd, 0xcb8, 0xccd,
0x20, 0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa, 0xc97, 0xcb3, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcb8, 0xcc7, 0xc82, 0xc9f, 0xccd, 0x20, 0xcaa, 0xcbf, 0xcaf, 0xcb0, 0xccd, 0x20,
0xcae, 0xca4, 0xccd, 0xca4, 0xcc1, 0x20, 0xcae, 0xcbf, 0xc95, 0xccd, 0xcb5, 0xcc6,
0xcb2, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb8, 0xcc7, 0xc82, 0xc9f, 0xccd,
0x20, 0xcaa, 0xcbf, 0xcaf, 0xcb0, 0xccd, 0x20, 0xcae, 0xca4, 0xccd, 0xca4, 0xcc1,
0x20, 0xcae, 0xcbf, 0xc95, 0xccd, 0xcb5, 0xcc6, 0xcb2, 0xca8, 0xccd, 0x20, 0xcaa,
0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb8,
0xcc7, 0xc82, 0xc9f, 0xccd, 0x20, 0xcaa, 0xcbf, 0xcaf, 0xcb0, 0xccd, 0x20, 0xcae,
0xca4, 0xccd, 0xca4, 0xcc1, 0x20, 0xcae, 0xcbf, 0xc95, 0xccd, 0xcb5, 0xcc6, 0xcb2,
0xca8, 0xccd, 0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa,
0xcbf, 0xc9f, 0xccd, 0x200c, 0xc95, 0xcc8, 0xcb0, 0xccd, 0xca8, 0xccd, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcaa, 0xcca, 0xca8, 0xcbe, 0xcaa, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcaa, 0xccd, 0xcaf, 0xcca, 0xc82, 0xc97, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xc97, 0xccd,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xccb, 0xc9c, 0xcbf, 0xcb2, 0xccb, 0xcb0, 0xccd,
0xca1, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xccb, 0xc9c, 0xcbf, 0xcb2, 0xccb,
0xcb0, 0xccd, 0xca1, 0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf,
0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xccb, 0xc9c, 0xcbf, 0xcb2, 0xccb, 0xcb0,
0xccd, 0xca1, 0xcbe, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcb0, 0xcbf, 0xcaf, 0xcc2, 0xca8, 0xcbf, 0xcaf, 0xca8, 0xccd, 0x20,
0xcb8, 0xcae, 0xcaf, 0xcb0, 0xcca, 0xca4, 0xcc7, 0xcb0, 0xcbe, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcb8, 0xccd, 0xcaf, 0xcbe, 0xc95, 0xccd, 0x200c, 0xcb9, 0xcb2, 0xcbf, 0xca8,
0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb8, 0xccd, 0xcaf, 0xcbe, 0xc95, 0xccd, 0x200c,
0xcb9, 0xcb2, 0xcbf, 0xca8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3,
0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb8, 0xccd, 0xcaf, 0xcbe, 0xc95, 0xccd,
0x200c, 0xcb9, 0xcb2, 0xcbf, 0xca8, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97,
0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb8, 0xcae, 0xcb0, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcb8, 0xcae, 0xcb0, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcb8, 0xcae, 0xcb0, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf,
0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb8, 0xcae, 0xccb, 0xcb5, 0xcbe, 0x20,
0xcb8, 0xcae, 0xcaf, 0xcb8, 0xcae, 0xccb, 0xcb5, 0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0,
0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb8, 0xcae, 0xccb,
0xcb5, 0xcbe, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcb8, 0xcc0, 0xcb7, 0xcc6, 0xcb2, 0xccd, 0xcb8, 0xccd, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcb8, 0xcbf, 0xc82, 0xc97, 0xcaa, 0xcc1, 0xcb0, 0xccd, 0x20, 0xcaa, 0xccd,
0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb8, 0xcbe,
0xcb2, 0xcae, 0xca8, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa, 0xc97, 0xcb3,
0x20, 0xcb8, 0xcae, 0xcaf, 0xca6, 0xc95, 0xccd, 0xcb7, 0xcbf, 0xca3, 0x20, 0xc9c,
0xcbe, 0xcb0, 0xccd, 0xc9c, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb8,
0xcc1, 0xcb0, 0xcbf, 0xca8, 0xcc7, 0xcae, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb8,
0xccd, 0xcaf, 0xcca, 0xcb5, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca4, 0xcbe, 0xcb9,
0xcbf, 0xca4, 0xcbf, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca4, 0xcc8, 0xcaa, 0xcc6, 0x20,
0xcb8, 0xcae, 0xcaf, 0xca4, 0xcc8, 0xcaa, 0xcc6, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae,
0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca4, 0xcc8, 0xcaa, 0xcc6,
0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca4, 0xc9d, 0xc95,
0xcbf, 0xcb8, 0xccd, 0xca4, 0xcbe, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9f,
0xcca, 0xc95, 0xcc6, 0xcb2, 0xcbe, 0xcb5, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9f,
0xcca, 0xc82, 0xc97, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9f, 0xcca, 0xc82, 0xc97,
0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc9f, 0xcca, 0xc82, 0xc97, 0xcbe, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf,
0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9a, 0xcc1, 0xc95, 0xccd, 0x20, 0xcb8,
0xcae, 0xcaf, 0xca4, 0xcc1, 0xcb0, 0xccd, 0xc95, 0xccd, 0x200c, 0xcae, 0xcc7, 0xca8,
0xcbf, 0xcb8, 0xccd, 0xca4, 0xcbe, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca4,
0xcc1, 0xcb0, 0xccd, 0xc95, 0xccd, 0x200c, 0xcae, 0xcc7, 0xca8, 0xcbf, 0xcb8, 0xccd,
0xca4, 0xcbe, 0xca8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf,
0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca4, 0xcc1, 0xcb0, 0xccd, 0xc95, 0xccd, 0x200c,
0xcae, 0xcc7, 0xca8, 0xcbf, 0xcb8, 0xccd, 0xca4, 0xcbe, 0xca8, 0xccd, 0x20, 0xcac,
0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca4, 0xcc1, 0xcb5,
0xcbe, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xcb0, 0xcc1, 0xc97, 0xccd,
0xcb5, 0xcc7, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xcb0, 0xcc1, 0xc97, 0xccd, 0xcb5,
0xcc7, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc89, 0xcb0, 0xcc1, 0xc97, 0xccd, 0xcb5, 0xcc7, 0x20, 0xcac, 0xcc7,
0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xc9c, 0xccd, 0xcac,
0xcc7, 0xc95, 0xcbf, 0xcb8, 0xccd, 0xca4, 0xcbe, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc89, 0xc9c, 0xccd, 0xcac, 0xcc7, 0xc95, 0xcbf, 0xcb8, 0xccd, 0xca4, 0xcbe,
0xca8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20,
0xcb8, 0xcae, 0xcaf, 0xc89, 0xc9c, 0xccd, 0xcac, 0xcc7, 0xc95, 0xcbf, 0xcb8, 0xccd,
0xca4, 0xcbe, 0xca8, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20,
0xcb8, 0xcae, 0xcaf, 0xcb5, 0xca8, 0xccc, 0xca4, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcb5, 0xca8, 0xccc, 0xca4, 0xcc1, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3,
0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb5, 0xca8, 0xccc, 0xca4, 0xcc1, 0x20,
0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb5, 0xcc6,
0xca8, 0xcbf, 0xc9c, 0xcc1, 0xcb5, 0xcc6, 0xcb2, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcb5, 0xccd, 0xcb2, 0xcbe, 0xca1, 0xcbf, 0xcb5, 0xccb, 0xcb8, 0xccd, 0xc9f, 0xccb,
0xc95, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb5, 0xccd, 0xcb2, 0xcbe, 0xca1, 0xcbf,
0xcb5, 0xccb, 0xcb8, 0xccd, 0xc9f, 0xccb, 0xc95, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0,
0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb5, 0xccd, 0xcb2,
0xcbe, 0xca1, 0xcbf, 0xcb5, 0xccb, 0xcb8, 0xccd, 0xc9f, 0xccb, 0xc95, 0xccd, 0x20,
0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb5, 0xccb,
0xcb2, 0xccd, 0xc97, 0xccb, 0xc97, 0xcbe, 0xcb0, 0xccd, 0xca1, 0xccd, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcb5, 0xccb, 0xcb2, 0xccd, 0xc97, 0xccb, 0xc97, 0xcbe, 0xcb0, 0xccd,
0xca1, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20,
0xcb8, 0xcae, 0xcaf, 0xcb5, 0xccb, 0xcb2, 0xccd, 0xc97, 0xccb, 0xc97, 0xcbe, 0xcb0,
0xccd, 0xca1, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcb5, 0xccb, 0xcb8, 0xccd, 0xc9f, 0xcca, 0xc95, 0xccd, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcb5, 0xcc7, 0xc95, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcb5, 0xccd, 0xcaf, 0xcbe, 0xcb2, 0xcc0, 0xcb8, 0xccd,
0x20, 0xcae, 0xca4, 0xccd, 0xca4, 0xcc1, 0x20, 0xcab, 0xccd, 0xcaf, 0xcc1, 0xc9f,
0xcc1, 0xca8, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaf, 0xcbe, 0xc95, 0xcc1, 0xc9f,
0xccd, 0xcb8, 0xc95, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaf, 0xcbe, 0xc95, 0xcc1,
0xc9f, 0xccd, 0xcb8, 0xc95, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3,
0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaf, 0xcbe, 0xc95, 0xcc1, 0xc9f, 0xccd,
0xcb8, 0xc95, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcaf, 0xcc7, 0xc95, 0xc9f, 0xcb0, 0xcbf, 0xca8, 0xccd, 0x200c, 0xcac,
0xcb0, 0xccd, 0xc97, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaf, 0xcc7, 0xc95, 0xc9f,
0xcb0, 0xcbf, 0xca8, 0xccd, 0x200c, 0xcac, 0xcb0, 0xccd, 0xc97, 0xccd, 0x20, 0xcaa,
0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaf,
0xcc7, 0xc95, 0xcc7, 0xc9f, 0xcb0, 0xcbf, 0xca8, 0xccd, 0x200d, 0xcac, 0xcb0, 0xccd,
0xc97, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcaf, 0xcc1, 0xc95, 0xcbe, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0x627,
0x620, 0x6a9, 0x631, 0x6d2, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620,
0x6a9, 0x631, 0x6d2, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620, 0x6a9, 0x631, 0x6d2, 0x20, 0x633,
0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x641,
0x63a, 0x627, 0x646, 0x650, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x645, 0x631, 0x6a9, 0x632, 0x6cc, 0x20, 0x627, 0x641, 0x631, 0x6cc,
0x656, 0x642, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x634, 0x631,
0x642, 0x6cc, 0x20, 0x627, 0x641, 0x631, 0x6cc, 0x656, 0x642, 0x627, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x62c, 0x646, 0x648, 0x657, 0x628, 0x6cc, 0x20, 0x627,
0x641, 0x631, 0x6cc, 0x642, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645,
0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627, 0x641, 0x631, 0x6cc, 0x656, 0x642, 0x627,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20,
0x627, 0x641, 0x631, 0x6cc, 0x656, 0x642, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646,
0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x63a, 0x631,
0x628, 0x6cc, 0x20, 0x627, 0x641, 0x631, 0x6cc, 0x656, 0x642, 0x627, 0x20, 0x633,
0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620,
0x644, 0x627, 0x633, 0x6a9, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627,
0x620, 0x644, 0x627, 0x633, 0x6a9, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691,
0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620, 0x644, 0x627,
0x633, 0x6a9, 0x627, 0x20, 0x688, 0x6d2, 0x644, 0x627, 0x6cc, 0x650, 0x654, 0x679,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620, 0x644, 0x645, 0x620, 0x679,
0x6cc, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620, 0x644, 0x645, 0x620,
0x679, 0x6cc, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620, 0x644, 0x645, 0x620, 0x679, 0x6cc, 0x20,
0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627,
0x620, 0x645, 0x64e, 0x632, 0x64e, 0x646, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x627, 0x620, 0x645, 0x64e, 0x632, 0x64e, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646,
0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620, 0x645,
0x64e, 0x632, 0x64e, 0x646, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x645, 0x631, 0x6a9, 0x632, 0x6cc, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x645, 0x631, 0x6a9, 0x632, 0x6cc, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x631,
0x6a9, 0x632, 0x6cc, 0x20, 0x688, 0x6d2, 0x644, 0x627, 0x6cc, 0x650, 0x654, 0x679,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x633,
0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x688, 0x6d2, 0x644, 0x627, 0x6cc, 0x650,
0x654, 0x679, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x627, 0x648, 0x646,
0x679, 0x6cc, 0x646, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x627, 0x648,
0x646, 0x679, 0x6cc, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x627, 0x648, 0x646, 0x679, 0x6cc,
0x646, 0x20, 0x688, 0x6d2, 0x644, 0x627, 0x6cc, 0x650, 0x654, 0x679, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x6cc, 0x633, 0x650, 0x641, 0x650, 0x6a9, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x6cc, 0x633, 0x650, 0x641, 0x650, 0x6a9,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x67e, 0x6cc, 0x633, 0x650, 0x641, 0x650, 0x6a9, 0x20, 0x688, 0x6d2,
0x644, 0x627, 0x6cc, 0x650, 0x654, 0x679, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x627, 0x620, 0x646, 0x64e, 0x691, 0x6cc, 0x656, 0x631, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x627, 0x620, 0x646, 0x64e, 0x691, 0x6cc, 0x656, 0x631, 0x20, 0x633,
0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x627, 0x620, 0x646, 0x691, 0x6cc, 0x656, 0x631, 0x20, 0x633, 0x64e, 0x645, 0x64e,
0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620, 0x6a9, 0x679, 0x627,
0x624, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620, 0x6a9, 0x679, 0x627,
0x624, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x627, 0x620, 0x6a9, 0x679, 0x627, 0x624, 0x20, 0x633, 0x64e,
0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620, 0x6a9,
0x679, 0x648, 0x628, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620, 0x6a9,
0x679, 0x648, 0x628, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620, 0x6a9, 0x679, 0x648, 0x628, 0x20,
0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627,
0x631, 0x620, 0x628, 0x650, 0x6cc, 0x64e, 0x646, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x627, 0x631, 0x620, 0x628, 0x650, 0x6cc, 0x64e, 0x646, 0x20, 0x633, 0x679,
0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627,
0x631, 0x620, 0x628, 0x650, 0x6cc, 0x64e, 0x646, 0x20, 0x688, 0x6d2, 0x644, 0x627,
0x6cc, 0x650, 0x654, 0x679, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x631,
0x62c, 0x620, 0x646, 0x679, 0x6cc, 0x656, 0x646, 0x627, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x627, 0x631, 0x62c, 0x620, 0x646, 0x679, 0x6cc, 0x656, 0x646, 0x627,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x627, 0x631, 0x62c, 0x620, 0x646, 0x679, 0x6cc, 0x656, 0x646, 0x627,
0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627, 0x631, 0x62c, 0x620, 0x646, 0x679,
0x6cc, 0x656, 0x646, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x63a,
0x631, 0x628, 0x6cc, 0x20, 0x627, 0x631, 0x62c, 0x620, 0x646, 0x679, 0x6cc, 0x656,
0x646, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627, 0x631,
0x62c, 0x620, 0x646, 0x679, 0x6cc, 0x656, 0x646, 0x627, 0x20, 0x633, 0x64e, 0x645,
0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x631, 0x645, 0x6cc,
0x656, 0x646, 0x650, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627,
0x631, 0x645, 0x6cc, 0x656, 0x646, 0x650, 0x6cc, 0x627, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x631,
0x645, 0x6cc, 0x656, 0x646, 0x650, 0x6cc, 0x627, 0x20, 0x633, 0x64e, 0x645, 0x64e,
0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620, 0x679, 0x644, 0x627,
0x646, 0x679, 0x650, 0x6a9, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620,
0x679, 0x644, 0x627, 0x646, 0x679, 0x650, 0x6a9, 0x20, 0x633, 0x679, 0x6cc, 0x646,
0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620, 0x679,
0x644, 0x627, 0x646, 0x679, 0x650, 0x6a9, 0x20, 0x688, 0x6d2, 0x644, 0x627, 0x6cc,
0x650, 0x654, 0x679, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x631, 0x6a9,
0x632, 0x6cc, 0x20, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x650, 0x6cc, 0x64e,
0x646, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x622, 0x633, 0x679, 0x631, 0x6cc,
0x644, 0x650, 0x6cc, 0x64e, 0x646, 0x20, 0x645, 0x631, 0x6a9, 0x632, 0x6cc, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x650, 0x6cc, 0x64e, 0x646, 0x20,
0x645, 0x631, 0x6a9, 0x632, 0x6cc, 0x20, 0x688, 0x6d2, 0x644, 0x627, 0x6cc, 0x650,
0x654, 0x679, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x622, 0x633, 0x679, 0x631,
0x6cc, 0x644, 0x650, 0x6cc, 0x64e, 0x646, 0x20, 0x645, 0x631, 0x6a9, 0x632, 0x6cc,
0x20, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x650, 0x6cc, 0x64e, 0x646, 0x20, 0x645,
0x631, 0x6a9, 0x632, 0x6cc, 0x20, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x633,
0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x650, 0x6cc, 0x64e, 0x646, 0x20, 0x645,
0x631, 0x6a9, 0x632, 0x6cc, 0x20, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x688,
0x6d2, 0x644, 0x627, 0x6cc, 0x650, 0x654, 0x679, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x645, 0x634, 0x631, 0x650, 0x642, 0x6cc, 0x20, 0x622, 0x633, 0x679, 0x631,
0x6cc, 0x644, 0x650, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x622,
0x633, 0x679, 0x631, 0x6cc, 0x644, 0x650, 0x6cc, 0x64e, 0x646, 0x20, 0x645, 0x634,
0x631, 0x642, 0x6cc, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x650,
0x6cc, 0x64e, 0x646, 0x20, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x688, 0x6d2,
0x644, 0x627, 0x6cc, 0x650, 0x654, 0x679, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x645, 0x63a, 0x631, 0x650, 0x628, 0x6cc, 0x20, 0x622, 0x633, 0x679, 0x631, 0x6cc,
0x644, 0x650, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x622, 0x633,
0x679, 0x631, 0x6cc, 0x644, 0x650, 0x6cc, 0x64e, 0x646, 0x20, 0x645, 0x63a, 0x631,
0x650, 0x628, 0x6cc, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x650,
0x6cc, 0x64e, 0x646, 0x20, 0x645, 0x63a, 0x631, 0x650, 0x628, 0x6cc, 0x656, 0x20,
0x688, 0x6d2, 0x644, 0x627, 0x6cc, 0x654, 0x679, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x627, 0x632, 0x631, 0x628, 0x627, 0x626, 0x6cc, 0x62c, 0x627, 0x646, 0x20,
0x679, 0x627, 0x626, 0x645, 0x627, 0x632, 0x631, 0x628, 0x627, 0x626, 0x6cc, 0x62c,
0x627, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x626, 0x645, 0x627, 0x632, 0x631, 0x628, 0x627, 0x626, 0x6cc, 0x62c, 0x627,
0x646, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645,
0x627, 0x620, 0x632, 0x648, 0x631, 0x655, 0x633, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x627, 0x620, 0x632, 0x648, 0x631, 0x655, 0x633, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620,
0x632, 0x648, 0x631, 0x655, 0x633, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20,
0x679, 0x627, 0x626, 0x645, 0x628, 0x64e, 0x646, 0x6af, 0x644, 0x627, 0x62f, 0x6cc,
0x634, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x628, 0x64e, 0x646, 0x6af, 0x644,
0x627, 0x62f, 0x6cc, 0x634, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x628, 0x64e, 0x646, 0x6af, 0x644, 0x627,
0x62f, 0x6cc, 0x634, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x628, 0x648, 0x657, 0x679, 0x627, 0x646, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x628, 0x648, 0x644, 0x650, 0x648, 0x650, 0x6cc, 0x627, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x628, 0x631, 0x620, 0x633, 0x650, 0x644, 0x650,
0x6cc, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x628, 0x631, 0x620, 0x633,
0x650, 0x644, 0x650, 0x6cc, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627,
0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x628, 0x631, 0x620, 0x633, 0x650,
0x644, 0x650, 0x6cc, 0x627, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x628, 0x631, 0x648, 0x657, 0x646, 0x64e, 0x6d2, 0x20,
0x62f, 0x64e, 0x631, 0x648, 0x657, 0x633, 0x64e, 0x644, 0x64e, 0x645, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x6cc, 0x67e, 0x20, 0x624, 0x631, 0x691, 0x648,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x6cc, 0x67e, 0x20, 0x624, 0x631,
0x691, 0x648, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x6cc, 0x67e, 0x20, 0x633, 0x64e, 0x645, 0x64e,
0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x620, 0x645, 0x648, 0x631,
0x648, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x6a9, 0x620, 0x62a, 0x6be, 0x64e, 0x645, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x6a9, 0x620, 0x62a, 0x6be, 0x64e, 0x645, 0x20, 0x633, 0x679,
0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x686,
0x620, 0x62a, 0x6be, 0x64e, 0x645, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x686, 0x650, 0x644, 0x6cc, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x686, 0x650, 0x644, 0x6cc, 0x20, 0x633, 0x679, 0x6cc, 0x646,
0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x686, 0x650, 0x644,
0x6cc, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x686, 0x64e, 0x6cc, 0x646, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x686, 0x64e, 0x6cc, 0x646, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631,
0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x686, 0x64e, 0x6cc, 0x646, 0x627,
0x20, 0x688, 0x6d2, 0x644, 0x627, 0x6cc, 0x650, 0x654, 0x679, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x6a9, 0x631, 0x633, 0x645, 0x64e, 0x633, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x6a9, 0x648, 0x6a9, 0x648, 0x632, 0x20, 0x627, 0x64e, 0x6cc,
0x644, 0x6cc, 0x646, 0x691, 0x632, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9,
0x648, 0x644, 0x648, 0x645, 0x628, 0x650, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x6a9, 0x648, 0x644, 0x648, 0x645, 0x628, 0x650, 0x6cc, 0x627, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x6a9, 0x648, 0x644, 0x648, 0x645, 0x628, 0x650, 0x6cc, 0x627, 0x20, 0x633,
0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x64f,
0x6a9, 0x20, 0x627, 0x64e, 0x6cc, 0x644, 0x6cc, 0x646, 0x691, 0x632, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x64f, 0x6a9, 0x20, 0x627, 0x64e, 0x6cc, 0x644,
0x6cc, 0x646, 0x691, 0x632, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x64f, 0x6a9, 0x20, 0x627, 0x64e,
0x6cc, 0x644, 0x6cc, 0x646, 0x691, 0x632, 0x20, 0x62d, 0x627, 0x641, 0x20, 0x633,
0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x6cc,
0x648, 0x657, 0x628, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x6cc,
0x648, 0x657, 0x628, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x6cc, 0x648, 0x657, 0x628, 0x627,
0x20, 0x688, 0x6d2, 0x644, 0x627, 0x6cc, 0x650, 0x654, 0x679, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x691, 0x6cc, 0x648, 0x650, 0x633, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x691, 0x645, 0x627, 0x646, 0x679, 0x20, 0x688, 0x6cc, 0x20, 0x627,
0x64f, 0x631, 0x648, 0x6cc, 0x656, 0x644, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x627, 0x6cc, 0x656, 0x633, 0x679, 0x20, 0x679, 0x6cc, 0x656, 0x645, 0x64e, 0x631,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x6cc, 0x656, 0x633, 0x679, 0x64e,
0x631, 0x20, 0x62c, 0x632, 0x6cc, 0x631, 0x655, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x627, 0x6cc, 0x656, 0x633, 0x679, 0x64e, 0x631, 0x20, 0x62c, 0x632, 0x6cc,
0x631, 0x655, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x627, 0x6cc, 0x656, 0x633, 0x679, 0x64e, 0x631, 0x20,
0x62c, 0x632, 0x6cc, 0x631, 0x655, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x650, 0x6a9, 0x648, 0x6cc, 0x691, 0x64e,
0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x631, 0x6a9, 0x632, 0x6cc,
0x20, 0x6cc, 0x648, 0x657, 0x631, 0x67e, 0x6cc, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x645, 0x631, 0x6a9, 0x632, 0x6cc, 0x20, 0x6cc, 0x648, 0x657, 0x631, 0x67e,
0x6cc, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x645, 0x631, 0x6a9, 0x632, 0x6cc, 0x20, 0x6cc, 0x648, 0x657,
0x631, 0x67e, 0x6cc, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x6cc, 0x648, 0x657,
0x631, 0x67e, 0x6cc, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x634, 0x631,
0x642, 0x6cc, 0x20, 0x6cc, 0x648, 0x657, 0x631, 0x67e, 0x6cc, 0x20, 0x633, 0x679,
0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645,
0x634, 0x631, 0x642, 0x6cc, 0x20, 0x6cc, 0x648, 0x657, 0x631, 0x67e, 0x6cc, 0x20,
0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645,
0x632, 0x6cc, 0x62f, 0x20, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x6cc, 0x648,
0x631, 0x67e, 0x6cc, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x63a, 0x631, 0x650,
0x628, 0x6cc, 0x20, 0x6cc, 0x648, 0x657, 0x631, 0x67e, 0x6cc, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x645, 0x63a, 0x631, 0x650, 0x628, 0x6cc, 0x20, 0x6cc, 0x648,
0x657, 0x631, 0x67e, 0x6cc, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x63a, 0x631, 0x650, 0x628, 0x6cc,
0x20, 0x6cc, 0x648, 0x657, 0x631, 0x650, 0x67e, 0x6cc, 0x20, 0x633, 0x64e, 0x645,
0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x641, 0x627, 0x644, 0x6a9,
0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x62c, 0x632, 0x6cc, 0x631, 0x655, 0x20,
0x679, 0x627, 0x626, 0x645, 0x641, 0x627, 0x644, 0x6a9, 0x20, 0x644, 0x6cc, 0x646,
0x688, 0x20, 0x62c, 0x632, 0x6cc, 0x631, 0x655, 0x20, 0x633, 0x679, 0x6cc, 0x646,
0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x641, 0x627, 0x644, 0x6a9,
0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x62c, 0x632, 0x6cc, 0x631, 0x655, 0x20,
0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x641, 0x6cc,
0x656, 0x62c, 0x6cc, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x641, 0x6cc, 0x656,
0x62c, 0x6cc, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x641, 0x6cc, 0x656, 0x62c, 0x6cc, 0x20, 0x633, 0x64e,
0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x641, 0x631, 0x620,
0x646, 0x686, 0x20, 0x6af, 0x6cc, 0x648, 0x657, 0x620, 0x646, 0x627, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x641, 0x631, 0x6cc, 0x646, 0x686, 0x20, 0x62c, 0x646,
0x648, 0x628, 0x6cc, 0x20, 0x62a, 0x655, 0x6c1, 0x20, 0x627, 0x646, 0x679, 0x627,
0x631, 0x6a9, 0x679, 0x650, 0x6a9, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6af, 0x620,
0x644, 0x620, 0x67e, 0x6cc, 0x6af, 0x648, 0x632, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x6af, 0x620, 0x645, 0x628, 0x650, 0x6cc, 0x64e, 0x631, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x62c, 0x648, 0x631, 0x62c, 0x650, 0x6cc, 0x627, 0x6c1, 0x64f,
0x6a9, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x62c, 0x648, 0x631, 0x62c, 0x650,
0x6cc, 0x627, 0x6c1, 0x64f, 0x6a9, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631,
0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x62c, 0x648, 0x631, 0x62c, 0x650,
0x6cc, 0x627, 0x6c1, 0x64f, 0x6a9, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6af, 0x650, 0x644, 0x628, 0x672, 0x679, 0x20,
0x62c, 0x62c, 0x6cc, 0x656, 0x631, 0x64f, 0x6a9, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x6af, 0x631, 0x6cc, 0x656, 0x646, 0x20, 0x648, 0x650, 0x686, 0x20, 0x645,
0x6cc, 0x656, 0x646, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x634, 0x631,
0x650, 0x642, 0x6cc, 0x20, 0x6af, 0x631, 0x6cc, 0x656, 0x646, 0x20, 0x644, 0x6cc,
0x646, 0x688, 0x64f, 0x6a9, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x634,
0x631, 0x650, 0x642, 0x6cc, 0x20, 0x6af, 0x631, 0x6cc, 0x656, 0x646, 0x20, 0x644,
0x6cc, 0x646, 0x688, 0x64f, 0x6a9, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627,
0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x634, 0x631, 0x650, 0x642,
0x6cc, 0x20, 0x6af, 0x631, 0x6cc, 0x656, 0x646, 0x20, 0x644, 0x6cc, 0x646, 0x688,
0x64f, 0x6a9, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x645, 0x63a, 0x631, 0x650, 0x628, 0x6cc, 0x20, 0x6af, 0x631, 0x6cc,
0x656, 0x646, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x64f, 0x6a9, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x645, 0x63a, 0x631, 0x650, 0x628, 0x6cc, 0x20, 0x6af, 0x631,
0x6cc, 0x656, 0x646, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x64f, 0x6a9, 0x20, 0x633,
0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x645, 0x63a, 0x631, 0x650, 0x628, 0x6cc, 0x20, 0x6af, 0x631, 0x6cc, 0x656, 0x646,
0x20, 0x644, 0x6cc, 0x646, 0x688, 0x64f, 0x6a9, 0x20, 0x633, 0x64e, 0x645, 0x64e,
0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6af, 0x64f, 0x648, 0x620, 0x645,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6af, 0x64e, 0x644, 0x641, 0x20, 0x633,
0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x6af, 0x64f, 0x6cc, 0x64e, 0x646, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x62d, 0x64e, 0x648, 0x627, 0x6d2, 0x20, 0x627, 0x620, 0x644, 0x6cc, 0x648, 0x657,
0x679, 0x650, 0x6cc, 0x64e, 0x646, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x62d,
0x64e, 0x648, 0x627, 0x6d2, 0x20, 0x627, 0x620, 0x644, 0x6cc, 0x648, 0x657, 0x679,
0x650, 0x6cc, 0x64e, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x62d, 0x64e, 0x648, 0x627, 0x6d2, 0x20,
0x627, 0x620, 0x644, 0x6cc, 0x648, 0x657, 0x679, 0x650, 0x6cc, 0x64e, 0x646, 0x20,
0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6c1,
0x627, 0x646, 0x6af, 0x20, 0x6a9, 0x627, 0x646, 0x6af, 0x20, 0x679, 0x627, 0x626,
0x645, 0x6c1, 0x627, 0x646, 0x6af, 0x20, 0x6a9, 0x627, 0x646, 0x6af, 0x20, 0x633,
0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6c1,
0x627, 0x646, 0x6af, 0x20, 0x6a9, 0x627, 0x646, 0x6af, 0x20, 0x633, 0x64e, 0x645,
0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x62d, 0x648, 0x648, 0x691, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x62d, 0x648, 0x648, 0x691, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x62d, 0x648,
0x648, 0x691, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x6c1, 0x650, 0x646, 0x62f, 0x648, 0x633, 0x62a, 0x627, 0x646, 0x6c1,
0x650, 0x646, 0x62f, 0x648, 0x633, 0x62a, 0x672, 0x646, 0x620, 0x20, 0x627, 0x648,
0x634, 0x64e, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x650, 0x646, 0x691,
0x648, 0x686, 0x64e, 0x6cc, 0x646, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x645, 0x631, 0x6a9, 0x632, 0x6cc, 0x20, 0x627, 0x650, 0x646, 0x691, 0x648, 0x646,
0x6cc, 0x634, 0x650, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645,
0x634, 0x631, 0x650, 0x642, 0x6cc, 0x20, 0x627, 0x650, 0x646, 0x691, 0x648, 0x646,
0x6cc, 0x634, 0x650, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645,
0x63a, 0x631, 0x650, 0x628, 0x6cc, 0x20, 0x627, 0x650, 0x646, 0x691, 0x648, 0x646,
0x6cc, 0x634, 0x650, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627,
0x650, 0x6cc, 0x656, 0x631, 0x672, 0x646, 0x620, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x627, 0x650, 0x6cc, 0x656, 0x631, 0x672, 0x646, 0x620, 0x20, 0x633, 0x679,
0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627,
0x650, 0x6cc, 0x656, 0x631, 0x672, 0x646, 0x6cc, 0x20, 0x633, 0x64e, 0x645, 0x64e,
0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x650, 0x631, 0x6a9, 0x64f,
0x679, 0x633, 0x6a9, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x650, 0x631,
0x6a9, 0x64f, 0x679, 0x633, 0x6a9, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631,
0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x650, 0x631, 0x6a9, 0x64f,
0x679, 0x633, 0x6a9, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x627, 0x650, 0x633, 0x631, 0x672, 0x6cc, 0x650, 0x644, 0x6cc,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x650, 0x633, 0x631, 0x672, 0x6cc,
0x650, 0x644, 0x6cc, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x650, 0x633, 0x631, 0x672, 0x6cc, 0x650,
0x644, 0x6cc, 0x20, 0x691, 0x6d2, 0x644, 0x627, 0x6cc, 0x650, 0x654, 0x679, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x62c, 0x627, 0x67e, 0x672, 0x646, 0x620, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x62c, 0x627, 0x67e, 0x672, 0x646, 0x620, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x62c, 0x627, 0x67e, 0x672, 0x646, 0x620, 0x20, 0x691, 0x6d2, 0x644, 0x627,
0x6cc, 0x650, 0x654, 0x679, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x64e,
0x645, 0x686, 0x64e, 0x679, 0x6a9, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x6a9, 0x64e, 0x645, 0x686, 0x64e, 0x679, 0x6a9, 0x627, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x64e,
0x645, 0x686, 0x64e, 0x679, 0x6a9, 0x627, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20,
0x642, 0x627, 0x632, 0x642, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x679, 0x627, 0x626,
0x645, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x642, 0x627, 0x632, 0x642, 0x633,
0x62a, 0x627, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x648, 0x631, 0x650,
0x6cc, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x648, 0x631, 0x650,
0x6cc, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x648, 0x631, 0x650, 0x6cc, 0x627, 0x20, 0x691,
0x6d2, 0x644, 0x627, 0x6cc, 0x650, 0x654, 0x679, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x6a9, 0x648, 0x631, 0x633, 0x64e, 0x6d2, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x6a9, 0x631, 0x620, 0x633, 0x646, 0x648, 0x6d2, 0x6cc, 0x627, 0x631, 0x633,
0x6a9, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x631, 0x620, 0x633, 0x646,
0x648, 0x6d2, 0x6cc, 0x627, 0x631, 0x633, 0x6a9, 0x20, 0x633, 0x679, 0x6cc, 0x646,
0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x631, 0x620,
0x633, 0x646, 0x648, 0x6d2, 0x6cc, 0x627, 0x631, 0x633, 0x6a9, 0x20, 0x633, 0x64e,
0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x631, 0x63a,
0x632, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x644, 0x64e,
0x646, 0x6a9, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x644, 0x627, 0x6cc,
0x650, 0x654, 0x646, 0x20, 0x62c, 0x62c, 0x6cc, 0x656, 0x631, 0x64f, 0x6a9, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x644, 0x639, 0x627, 0x691, 0x20, 0x62d, 0x648,
0x648, 0x6d2, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x644, 0x639, 0x627, 0x691,
0x20, 0x62d, 0x648, 0x648, 0x6d2, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627,
0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x644, 0x639, 0x627, 0x691, 0x20,
0x691, 0x6d2, 0x644, 0x627, 0x6cc, 0x654, 0x679, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x645, 0x64e, 0x6a9, 0x64e, 0x639, 0x648, 0x657, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x645, 0x64e, 0x6a9, 0x64e, 0x639, 0x648, 0x657, 0x20, 0x633, 0x679,
0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645,
0x64e, 0x6a9, 0x64e, 0x639, 0x648, 0x657, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x64e, 0x6af, 0x64e, 0x62f, 0x64e,
0x646, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x64e, 0x6af, 0x64e, 0x62f,
0x64e, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x645, 0x64e, 0x6af, 0x64e, 0x62f, 0x64e, 0x646, 0x20,
0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645,
0x64e, 0x644, 0x6cc, 0x634, 0x650, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x645, 0x627, 0x644, 0x62f, 0x6cc, 0x656, 0x648, 0x655, 0x632, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x645, 0x627, 0x631, 0x642, 0x6cc, 0x648, 0x657, 0x633,
0x64e, 0x633, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x627, 0x631, 0x634,
0x64e, 0x644, 0x20, 0x62c, 0x62c, 0x6cc, 0x656, 0x631, 0x64f, 0x6a9, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x645, 0x648, 0x631, 0x650, 0x634, 0x64e, 0x633, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x648, 0x631, 0x650, 0x634, 0x64e, 0x633,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x645, 0x648, 0x631, 0x650, 0x634, 0x64e, 0x633, 0x20, 0x633, 0x64e,
0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x627, 0x633,
0x64e, 0x646, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x6cc, 0x6a9, 0x633,
0x6cc, 0x6a9, 0x646, 0x20, 0x67e, 0x6cc, 0x633, 0x6cc, 0x641, 0x6a9, 0x20, 0x679,
0x627, 0x626, 0x645, 0x645, 0x6cc, 0x6a9, 0x633, 0x6cc, 0x6a9, 0x646, 0x20, 0x67e,
0x6cc, 0x633, 0x6cc, 0x641, 0x6a9, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631,
0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x6cc, 0x6a9, 0x633, 0x6cc, 0x6a9,
0x646, 0x20, 0x67e, 0x6cc, 0x633, 0x6cc, 0x641, 0x6a9, 0x20, 0x688, 0x6d2, 0x20,
0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x648, 0x644,
0x646, 0x20, 0x628, 0x627, 0x679, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627,
0x648, 0x644, 0x646, 0x20, 0x628, 0x627, 0x679, 0x631, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x648, 0x644,
0x646, 0x20, 0x628, 0x627, 0x679, 0x631, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x627, 0x633, 0x6a9, 0x64e, 0x648,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x627, 0x633, 0x6a9, 0x648, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x645, 0x627, 0x633, 0x6a9, 0x648, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x650, 0x6cc, 0x627, 0x646, 0x645,
0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x646, 0x64e, 0x639, 0x648,
0x657, 0x631, 0x648, 0x657, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x646, 0x620,
0x67e, 0x672, 0x644, 0x620, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x646, 0x650,
0x648, 0x20, 0x6a9, 0x6cc, 0x644, 0x620, 0x691, 0x648, 0x646, 0x650, 0x6cc, 0x627,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x646, 0x650, 0x648, 0x20, 0x6a9, 0x6cc,
0x644, 0x620, 0x691, 0x648, 0x646, 0x650, 0x6cc, 0x627, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x646, 0x650,
0x648, 0x20, 0x6a9, 0x6cc, 0x644, 0x620, 0x691, 0x648, 0x646, 0x650, 0x6cc, 0x633,
0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x646, 0x650, 0x648, 0x632, 0x650, 0x644, 0x6cc, 0x646, 0x691, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x646, 0x650, 0x648, 0x632, 0x650, 0x644, 0x6cc, 0x646, 0x691,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x646, 0x650, 0x648, 0x632, 0x650, 0x644, 0x6cc, 0x646, 0x691, 0x20,
0x691, 0x6d2, 0x644, 0x627, 0x6cc, 0x654, 0x679, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x646, 0x6cc, 0x648, 0x20, 0x641, 0x627, 0x624, 0x646, 0x688, 0x20, 0x644,
0x6cc, 0x646, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x646, 0x6cc, 0x648, 0x20,
0x641, 0x627, 0x624, 0x646, 0x688, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x633,
0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x646,
0x6cc, 0x648, 0x20, 0x641, 0x627, 0x624, 0x646, 0x688, 0x20, 0x644, 0x6cc, 0x646,
0x688, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627,
0x626, 0x645, 0x646, 0x650, 0x6cc, 0x648, 0x657, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x646, 0x648, 0x631, 0x641, 0x639, 0x627, 0x6a9, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x646, 0x648, 0x631, 0x641, 0x639, 0x627, 0x6a9, 0x20, 0x633, 0x679,
0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x646,
0x648, 0x631, 0x641, 0x639, 0x627, 0x6a9, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x641, 0x631, 0x646, 0x6cc, 0x646, 0x688,
0x648, 0x20, 0x688, 0x6cc, 0x20, 0x646, 0x648, 0x631, 0x648, 0x646, 0x6c1, 0x627,
0x20, 0x679, 0x627, 0x626, 0x645, 0x641, 0x631, 0x646, 0x6cc, 0x646, 0x688, 0x648,
0x20, 0x688, 0x6cc, 0x20, 0x646, 0x648, 0x631, 0x648, 0x646, 0x6c1, 0x627, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645,
0x641, 0x631, 0x646, 0x6cc, 0x646, 0x688, 0x648, 0x20, 0x688, 0x6cc, 0x20, 0x646,
0x648, 0x631, 0x648, 0x646, 0x6c1, 0x627, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631,
0x20, 0x679, 0x627, 0x626, 0x645, 0x634, 0x64f, 0x645, 0x672, 0x644, 0x6cc, 0x20,
0x645, 0x64e, 0x631, 0x650, 0x6cc, 0x627, 0x646, 0x627, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x646, 0x6c4, 0x648, 0x6c4, 0x633, 0x650, 0x628, 0x654, 0x631, 0x633,
0x6a9, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x646, 0x6c4, 0x648, 0x6c4, 0x633,
0x650, 0x628, 0x654, 0x631, 0x633, 0x6a9, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688,
0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x646, 0x6c4, 0x648, 0x6c4,
0x633, 0x650, 0x628, 0x654, 0x631, 0x633, 0x6a9, 0x20, 0x633, 0x64e, 0x645, 0x64e,
0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x6c4, 0x645, 0x633, 0x6a9,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x6c4, 0x645, 0x633, 0x6a9, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x627, 0x6c4, 0x645, 0x633, 0x6a9, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x627, 0x6a9, 0x650, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x627, 0x6a9, 0x650,
0x633, 0x62a, 0x627, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x627, 0x6a9, 0x650, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x67e, 0x64e, 0x644, 0x627, 0x648, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x67e, 0x627, 0x67e, 0x64f, 0x639, 0x627, 0x20, 0x646, 0x6cc, 0x648, 0x657,
0x20, 0x6af, 0x620, 0x646, 0x6cc, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e,
0x6cc, 0x631, 0x620, 0x6af, 0x648, 0x6d2, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x67e, 0x6cc, 0x631, 0x620, 0x6af, 0x648, 0x6d2, 0x20, 0x633, 0x679, 0x6cc, 0x646,
0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x6cc, 0x631,
0x620, 0x6af, 0x648, 0x6d2, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x654, 0x631, 0x648, 0x657, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x67e, 0x654, 0x631, 0x648, 0x657, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x654,
0x631, 0x648, 0x657, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x67e, 0x6be, 0x650, 0x644, 0x650, 0x67e, 0x627, 0x6cc, 0x650,
0x646, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x6be, 0x650, 0x644, 0x650,
0x67e, 0x627, 0x6cc, 0x650, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627,
0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x6be, 0x650, 0x644, 0x650,
0x67e, 0x627, 0x6cc, 0x650, 0x646, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x6be, 0x648, 0x646, 0x650, 0x6a9, 0x633,
0x20, 0x62c, 0x62c, 0x6cc, 0x656, 0x631, 0x64f, 0x6a9, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x633, 0x6cc, 0x646, 0x679, 0x20, 0x67e, 0x64e, 0x6cc, 0x631, 0x6cc,
0x20, 0x645, 0x650, 0x642, 0x6cc, 0x648, 0x657, 0x644, 0x64e, 0x646, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x633, 0x6cc, 0x646, 0x679, 0x20, 0x67e, 0x64e, 0x6cc,
0x631, 0x6cc, 0x20, 0x645, 0x650, 0x642, 0x6cc, 0x648, 0x657, 0x644, 0x64e, 0x646,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x633, 0x6cc, 0x646, 0x679, 0x20, 0x67e, 0x64e, 0x6cc, 0x631, 0x6cc,
0x20, 0x645, 0x650, 0x642, 0x6cc, 0x648, 0x657, 0x644, 0x64e, 0x646, 0x20, 0x691,
0x6d2, 0x644, 0x627, 0x6cc, 0x650, 0x654, 0x679, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x67e, 0x650, 0x679, 0x6a9, 0x6cc, 0x631, 0x655, 0x646, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x67e, 0x648, 0x646, 0x6cc, 0x67e, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x6cc, 0x648, 0x646, 0x6af, 0x20, 0x6cc, 0x627, 0x646, 0x6af, 0x20,
0x679, 0x627, 0x626, 0x645, 0x642, 0x650, 0x632, 0x644, 0x648, 0x691, 0x627, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x642, 0x650, 0x632, 0x644, 0x648, 0x691, 0x627,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x642, 0x650, 0x632, 0x644, 0x648, 0x691, 0x627, 0x20, 0x633, 0x64e,
0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x631, 0x650, 0x6cc,
0x648, 0x657, 0x646, 0x650, 0x6cc, 0x64e, 0x646, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x631, 0x648, 0x62a, 0x6be, 0x620, 0x631, 0x627, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x633, 0x64e, 0x6a9, 0x6be, 0x620, 0x644, 0x650, 0x646, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x633, 0x64e, 0x6a9, 0x6be, 0x620, 0x644, 0x650, 0x646,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x633, 0x64e, 0x6a9, 0x6be, 0x620, 0x644, 0x650, 0x646, 0x20, 0x633,
0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x633, 0x645,
0x620, 0x631, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x633, 0x645, 0x620,
0x631, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x633, 0x645, 0x620, 0x631, 0x627, 0x20, 0x633, 0x64e,
0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x633, 0x620, 0x645,
0x648, 0x622, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x633, 0x620, 0x645, 0x648,
0x622, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x633, 0x620, 0x645, 0x648, 0x622, 0x20, 0x633, 0x64e, 0x645,
0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x633, 0x6cc, 0x634, 0x620,
0x644, 0x655, 0x632, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x633, 0x650, 0x646,
0x6af, 0x627, 0x67e, 0x648, 0x657, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x633, 0x648, 0x644, 0x648, 0x645, 0x64e, 0x646, 0x20, 0x62c, 0x62c, 0x6cc, 0x656,
0x631, 0x64e, 0x646, 0x20, 0x6c1, 0x64f, 0x646, 0x62f, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x634, 0x64f, 0x645, 0x672, 0x644, 0x6cc, 0x20, 0x62c, 0x648, 0x631,
0x62c, 0x650, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x633, 0x64f,
0x631, 0x650, 0x646, 0x627, 0x645, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x633,
0x6cc, 0x648, 0x648, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x679, 0x627,
0x6c1, 0x650, 0x679, 0x6cc, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x679, 0x6d2,
0x20, 0x67e, 0x6d2, 0x20, 0x679, 0x627, 0x626, 0x645, 0x679, 0x6d2, 0x20, 0x67e,
0x6d2, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x626, 0x645, 0x679, 0x6d2, 0x20, 0x67e, 0x6d2, 0x20, 0x688, 0x6d2, 0x20, 0x644,
0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x62a, 0x627, 0x62c, 0x6a9,
0x633, 0x62a, 0x627, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x679, 0x648, 0x6a9,
0x620, 0x644, 0x648, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x679, 0x639, 0x627,
0x646, 0x6af, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x679, 0x639, 0x627,
0x646, 0x6af, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x679, 0x639, 0x627, 0x646, 0x6af, 0x627, 0x20,
0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x679,
0x654, 0x631, 0x6a9, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x62a, 0x631, 0x6a9,
0x645, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645,
0x62a, 0x631, 0x6a9, 0x645, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633,
0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x62a,
0x64f, 0x631, 0x6a9, 0x645, 0x620, 0x646, 0x650, 0x633, 0x62a, 0x627, 0x646, 0x20,
0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x679,
0x648, 0x657, 0x648, 0x64e, 0x644, 0x648, 0x657, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x6cc, 0x648, 0x657, 0x631, 0x620, 0x6af, 0x648, 0x64e, 0x6d2, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x6cc, 0x648, 0x657, 0x631, 0x620, 0x6af, 0x648, 0x64e,
0x6d2, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x6cc, 0x648, 0x657, 0x631, 0x620, 0x6af, 0x648, 0x64e, 0x6d2,
0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x627, 0x64f, 0x632, 0x628, 0x6cc, 0x6a9, 0x650, 0x633, 0x62a, 0x627, 0x646, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x64f, 0x632, 0x628, 0x6cc, 0x6a9, 0x650,
0x633, 0x62a, 0x627, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x64f, 0x632, 0x628, 0x6cc, 0x6a9,
0x650, 0x633, 0x62a, 0x627, 0x646, 0x64f, 0x6a9, 0x20, 0x633, 0x64e, 0x645, 0x64e,
0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x648, 0x64e, 0x646, 0x648, 0x657,
0x627, 0x64e, 0x679, 0x648, 0x657, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x648,
0x64e, 0x646, 0x648, 0x657, 0x627, 0x64e, 0x679, 0x648, 0x657, 0x20, 0x633, 0x679,
0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x648,
0x64e, 0x646, 0x648, 0x657, 0x627, 0x64e, 0x679, 0x648, 0x657, 0x20, 0x633, 0x64e,
0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x648, 0x620, 0x646,
0x620, 0x632, 0x6cc, 0x648, 0x657, 0x644, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x648, 0x644, 0x627, 0x691, 0x650, 0x648, 0x648, 0x633, 0x679, 0x648, 0x6a9,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x648, 0x644, 0x627, 0x691, 0x650, 0x648,
0x648, 0x633, 0x679, 0x648, 0x6a9, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631,
0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x648, 0x644, 0x627, 0x691, 0x650,
0x648, 0x648, 0x633, 0x679, 0x648, 0x6a9, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x648, 0x648, 0x644, 0x6af, 0x648, 0x6af,
0x631, 0x6cc, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x648, 0x648, 0x644,
0x6af, 0x648, 0x6af, 0x631, 0x6cc, 0x691, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688,
0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x648, 0x648, 0x644, 0x6af,
0x648, 0x6af, 0x631, 0x6cc, 0x691, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x648, 0x6cc, 0x6a9, 0x20, 0x62c, 0x62c, 0x6cc,
0x656, 0x631, 0x64f, 0x6a9, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x648, 0x627,
0x644, 0x650, 0x633, 0x20, 0x62a, 0x6c1, 0x655, 0x20, 0x641, 0x6cc, 0x648, 0x657,
0x679, 0x6cc, 0x648, 0x657, 0x646, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x6cc, 0x64e, 0x6a9, 0x64f, 0x679, 0x633, 0x6a9, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x6cc, 0x64e, 0x6a9, 0x64f, 0x679, 0x633, 0x6a9, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6cc, 0x64e,
0x6a9, 0x64f, 0x679, 0x64f, 0x633, 0x6a9, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6cc, 0x620, 0x6a9, 0x64e, 0x679, 0x654,
0x631, 0x650, 0x646, 0x628, 0x654, 0x631, 0x6af, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x6cc, 0x620, 0x6a9, 0x64e, 0x679, 0x654, 0x631, 0x650, 0x646, 0x628, 0x654,
0x631, 0x6af, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x6cc, 0x620, 0x6a9, 0x64e, 0x679, 0x631, 0x650, 0x646,
0x628, 0x654, 0x631, 0x6af, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x6cc, 0x648, 0x6a9, 0x648, 0x646, 0x20, 0x679, 0x627,
0x626, 0x645, 0x938, 0x947, 0x902, 0x91f, 0x94d, 0x930, 0x932, 0x20, 0x935, 0x916,
0x938, 0x947, 0x902, 0x91f, 0x94d, 0x930, 0x932, 0x20, 0x938, 0x94d, 0x91f, 0x948,
0x928, 0x94d, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x935, 0x916, 0x938, 0x947, 0x902,
0x91f, 0x94d, 0x930, 0x932, 0x20, 0x921, 0x947, 0x20, 0x932, 0x93e, 0x907, 0x91f,
0x20, 0x935, 0x916, 0x92e, 0x936, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x935, 0x916,
0x92e, 0x936, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x928,
0x94d, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x935, 0x916, 0x92e, 0x936, 0x930, 0x93f,
0x915, 0x940, 0x20, 0x921, 0x947, 0x20, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x935,
0x916, 0x92e, 0x93e, 0x909, 0x902, 0x91f, 0x947, 0x928, 0x20, 0x935, 0x916, 0x92e,
0x93e, 0x909, 0x902, 0x91f, 0x947, 0x928, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x928,
0x94d, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x935, 0x916, 0x92e, 0x93e, 0x909, 0x902,
0x91f, 0x947, 0x928, 0x20, 0x921, 0x947, 0x20, 0x932, 0x93e, 0x907, 0x91f, 0x20,
0x935, 0x916, 0x92a, 0x947, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x935, 0x916,
0x92a, 0x947, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x938, 0x94d, 0x91f, 0x948,
0x928, 0x94d, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x935, 0x916, 0x92a, 0x947, 0x938,
0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x921, 0x947, 0x20, 0x932, 0x93e, 0x907, 0x91f,
0x20, 0x935, 0x916, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20,
0x935, 0x916, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20, 0x938,
0x94d, 0x91f, 0x948, 0x928, 0x94d, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x935, 0x916,
0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20, 0x921, 0x947, 0x20,
0x932, 0x93e, 0x907, 0x91f, 0x20, 0x935, 0x916, 0x92e, 0x930, 0x915, 0x91c, 0x93c,
0x940, 0x20, 0x92f, 0x942, 0x930, 0x92a, 0x940, 0x20, 0x935, 0x916, 0x92e, 0x930,
0x915, 0x91c, 0x93c, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x92a, 0x940, 0x20, 0x938,
0x94d, 0x91f, 0x948, 0x928, 0x94d, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x935, 0x916,
0x92e, 0x930, 0x915, 0x91c, 0x93c, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x92a, 0x940,
0x20, 0x938, 0x92e, 0x930, 0x20, 0x935, 0x916, 0x92e, 0x936, 0x930, 0x93f, 0x915,
0x940, 0x20, 0x92f, 0x942, 0x930, 0x92a, 0x940, 0x20, 0x935, 0x916, 0x92e, 0x936,
0x930, 0x93f, 0x915, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x92a, 0x940, 0x20, 0x938,
0x94d, 0x91f, 0x948, 0x928, 0x94d, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x935, 0x916,
0x92e, 0x936, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x92a, 0x940,
0x20, 0x938, 0x92e, 0x930, 0x20, 0x935, 0x916, 0x92e, 0x917, 0x930, 0x940, 0x92c,
0x940, 0x20, 0x92f, 0x942, 0x930, 0x92a, 0x940, 0x20, 0x935, 0x916, 0x92e, 0x917,
0x930, 0x940, 0x92c, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x92a, 0x940, 0x20, 0x938,
0x94d, 0x91f, 0x948, 0x928, 0x94d, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x935, 0x916,
0x92e, 0x917, 0x930, 0x940, 0x92c, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x92a, 0x940,
0x20, 0x938, 0x92e, 0x930, 0x20, 0x935, 0x916, 0x917, 0x94d, 0x930, 0x940, 0x928,
0x935, 0x93f, 0x91a, 0x20, 0x913, 0x938, 0x924, 0x20, 0x935, 0x916, 0x410, 0x443,
0x493, 0x430, 0x43d, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x41e, 0x440, 0x442, 0x430, 0x43b, 0x44b, 0x49b, 0x20, 0x410, 0x444,
0x440, 0x438, 0x43a, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x428,
0x44b, 0x493, 0x44b, 0x441, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41e, 0x4a3, 0x442, 0x4af, 0x441, 0x442,
0x456, 0x43a, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x441, 0x20, 0x410, 0x444, 0x440,
0x438, 0x43a, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430,
0x442, 0x44b, 0x441, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x441, 0x20, 0x410, 0x444,
0x440, 0x438, 0x43a, 0x430, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430,
0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a, 0x430, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d,
0x43a, 0x430, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x421, 0x43e, 0x43b, 0x442, 0x4af, 0x441, 0x442, 0x456, 0x43a,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x43e, 0x440, 0x442,
0x430, 0x43b, 0x44b, 0x49b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421,
0x43e, 0x43b, 0x442, 0x4af, 0x441, 0x442, 0x456, 0x43a, 0x20, 0x410, 0x43c, 0x435,
0x440, 0x438, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x442, 0x44b, 0x20, 0x43e, 0x440, 0x442, 0x430, 0x43b, 0x44b, 0x49b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x43e, 0x43b, 0x442, 0x4af, 0x441,
0x442, 0x456, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20,
0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x43e, 0x440, 0x442, 0x430, 0x43b, 0x44b,
0x49b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x43e, 0x43b, 0x442,
0x4af, 0x441, 0x442, 0x456, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a,
0x430, 0x20, 0x448, 0x44b, 0x493, 0x44b, 0x441, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x421, 0x43e, 0x43b, 0x442, 0x4af, 0x441, 0x442, 0x456, 0x43a, 0x20,
0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x448, 0x44b, 0x493, 0x44b, 0x441,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x43e, 0x43b, 0x442, 0x4af,
0x441, 0x442, 0x456, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430,
0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x448, 0x44b, 0x493, 0x44b, 0x441,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x43e, 0x43b, 0x442, 0x4af,
0x441, 0x442, 0x456, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430,
0x20, 0x442, 0x430, 0x443, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421,
0x43e, 0x43b, 0x442, 0x4af, 0x441, 0x442, 0x456, 0x43a, 0x20, 0x410, 0x43c, 0x435,
0x440, 0x438, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x442, 0x44b, 0x20, 0x442, 0x430, 0x443, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x421, 0x43e, 0x43b, 0x442, 0x4af, 0x441, 0x442, 0x456, 0x43a, 0x20,
0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x436, 0x430, 0x437, 0x493,
0x44b, 0x20, 0x442, 0x430, 0x443, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x421, 0x43e, 0x43b, 0x442, 0x4af, 0x441, 0x442, 0x456, 0x43a, 0x20, 0x410, 0x43c,
0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x422, 0x44b, 0x43d, 0x44b, 0x49b, 0x20,
0x43c, 0x4b1, 0x445, 0x438, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x421, 0x43e, 0x43b, 0x442, 0x4af, 0x441, 0x442, 0x456, 0x43a, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x422, 0x44b, 0x43d, 0x44b, 0x49b, 0x20,
0x43c, 0x4b1, 0x445, 0x438, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x421, 0x43e, 0x43b, 0x442, 0x4af, 0x441, 0x442, 0x456, 0x43a, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b,
0x20, 0x422, 0x44b, 0x43d, 0x44b, 0x49b, 0x20, 0x43c, 0x4b1, 0x445, 0x438, 0x442,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x43f, 0x438, 0x430,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x43f, 0x438, 0x430, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x43f, 0x438, 0x430, 0x20, 0x436, 0x430,
0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x430,
0x443, 0x434, 0x20, 0x410, 0x440, 0x430, 0x431, 0x438, 0x44f, 0x441, 0x44b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x430, 0x443, 0x434, 0x20, 0x410,
0x440, 0x430, 0x431, 0x438, 0x44f, 0x441, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x421, 0x430, 0x443, 0x434, 0x20, 0x410, 0x440, 0x430, 0x431, 0x438, 0x44f,
0x441, 0x44b, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x440, 0x433, 0x435, 0x43d,
0x442, 0x438, 0x43d, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x440,
0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x436, 0x430, 0x437, 0x493,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x442, 0x44b,
0x441, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x441, 0x20,
0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x441, 0x20, 0x410, 0x440, 0x433,
0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x440, 0x43c, 0x435, 0x43d,
0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x440, 0x43c,
0x435, 0x43d, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x440,
0x43c, 0x435, 0x43d, 0x438, 0x44f, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442,
0x438, 0x43a, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x442,
0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x430, 0x20, 0x436,
0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41e,
0x440, 0x442, 0x430, 0x43b, 0x44b, 0x49b, 0x20, 0x410, 0x443, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410,
0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x43e, 0x440, 0x442, 0x430,
0x43b, 0x44b, 0x49b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x443,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x436, 0x430, 0x437, 0x493,
0x44b, 0x20, 0x43e, 0x440, 0x442, 0x430, 0x43b, 0x44b, 0x49b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x44f, 0x20, 0x43e, 0x440, 0x442, 0x430, 0x43b, 0x44b, 0x49b, 0x2d, 0x431, 0x430,
0x442, 0x44b, 0x441, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x443,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x43e, 0x440, 0x442, 0x430, 0x43b,
0x44b, 0x49b, 0x2d, 0x431, 0x430, 0x442, 0x44b, 0x441, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f,
0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x43e, 0x440, 0x442, 0x430, 0x43b,
0x44b, 0x49b, 0x2d, 0x431, 0x430, 0x442, 0x44b, 0x441, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x428, 0x44b, 0x493, 0x44b, 0x441, 0x20, 0x410, 0x443, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x448, 0x44b,
0x493, 0x44b, 0x441, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x443,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x436, 0x430, 0x437, 0x493,
0x44b, 0x20, 0x448, 0x44b, 0x493, 0x44b, 0x441, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x441, 0x20, 0x410, 0x443, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x431, 0x430, 0x442,
0x44b, 0x441, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x443, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b,
0x20, 0x431, 0x430, 0x442, 0x44b, 0x441, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x4d8, 0x437, 0x456, 0x440, 0x431, 0x430, 0x439, 0x436, 0x430, 0x43d, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x4d8, 0x437, 0x456, 0x440, 0x431, 0x430,
0x439, 0x436, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x4d8, 0x437,
0x456, 0x440, 0x431, 0x430, 0x439, 0x436, 0x430, 0x43d, 0x20, 0x436, 0x430, 0x437,
0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x437, 0x43e,
0x440, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x437, 0x43e, 0x440, 0x20, 0x430, 0x440,
0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x410, 0x437, 0x43e, 0x440, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440,
0x44b, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430,
0x434, 0x435, 0x448, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x43d,
0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x443, 0x442, 0x430, 0x43d,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x43e, 0x43b, 0x438, 0x432,
0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x440, 0x430,
0x437, 0x438, 0x43b, 0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x44f, 0x20, 0x436,
0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411,
0x440, 0x443, 0x43d, 0x435, 0x439, 0x2d, 0x414, 0x430, 0x440, 0x443, 0x441, 0x441,
0x430, 0x43b, 0x430, 0x43c, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a,
0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440,
0x434, 0x435, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x430, 0x431, 0x43e,
0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x427, 0x430, 0x43c, 0x43e, 0x440,
0x440, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x427, 0x430, 0x442, 0x435,
0x43c, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x427, 0x430, 0x442, 0x435,
0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x427, 0x430, 0x442, 0x435, 0x43c,
0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x427, 0x438, 0x43b, 0x438, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x427, 0x438, 0x43b, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x427, 0x438,
0x43b, 0x438, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x49a, 0x44b, 0x442, 0x430, 0x439, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x49a, 0x44b, 0x442, 0x430, 0x439, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x49a, 0x44b, 0x442, 0x430, 0x439, 0x20, 0x436, 0x430, 0x437, 0x493,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x420, 0x43e, 0x436, 0x434,
0x435, 0x441, 0x442, 0x432, 0x43e, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x44b, 0x43d,
0x44b, 0x4a3, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x43e, 0x43a,
0x43e, 0x441, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d,
0x44b, 0x4a3, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x43e, 0x43b,
0x443, 0x43c, 0x431, 0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x44f, 0x20, 0x436,
0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a,
0x443, 0x43a, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d,
0x44b, 0x4a3, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x443, 0x43a,
0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x4a3,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x443, 0x43a, 0x20, 0x430, 0x440,
0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x4a3, 0x20, 0x436, 0x430,
0x440, 0x442, 0x44b, 0x43b, 0x430, 0x439, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x443, 0x431, 0x430, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x443, 0x431, 0x430, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x443, 0x431, 0x430, 0x20, 0x436, 0x430, 0x437,
0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x414, 0x435, 0x439,
0x432, 0x438, 0x441, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x414, 0x44e,
0x43c, 0x43e, 0x43d, 0x2d, 0x434, 0x2019, 0x42e, 0x440, 0x432, 0x438, 0x43b, 0x44c,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x428, 0x44b, 0x493, 0x44b, 0x441,
0x20, 0x422, 0x438, 0x43c, 0x43e, 0x440, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x41f, 0x430, 0x441, 0x445, 0x430, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x441, 0x445, 0x430,
0x20, 0x430, 0x440, 0x430, 0x43b, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x41f, 0x430, 0x441, 0x445, 0x430, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x44b, 0x20,
0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x42d, 0x43a, 0x432, 0x430, 0x434, 0x43e, 0x440, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x41e, 0x440, 0x442, 0x430, 0x43b, 0x44b, 0x49b, 0x20, 0x415, 0x443,
0x440, 0x43e, 0x43f, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41e,
0x440, 0x442, 0x430, 0x43b, 0x44b, 0x49b, 0x20, 0x415, 0x443, 0x440, 0x43e, 0x43f,
0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41e, 0x440, 0x442, 0x430, 0x43b,
0x44b, 0x49b, 0x20, 0x415, 0x443, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x436, 0x430,
0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x428, 0x44b,
0x493, 0x44b, 0x441, 0x20, 0x415, 0x443, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x428, 0x44b, 0x493, 0x44b, 0x441, 0x20, 0x415,
0x443, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x428,
0x44b, 0x493, 0x44b, 0x441, 0x20, 0x415, 0x443, 0x440, 0x43e, 0x43f, 0x430, 0x20,
0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x49a, 0x438, 0x44b, 0x440, 0x20, 0x428, 0x44b, 0x493, 0x44b, 0x441, 0x20, 0x415,
0x443, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x411, 0x430, 0x442, 0x44b, 0x441, 0x20, 0x415, 0x443, 0x440, 0x43e, 0x43f, 0x430,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x441,
0x20, 0x415, 0x443, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x411, 0x430, 0x442, 0x44b, 0x441, 0x20, 0x415, 0x443, 0x440, 0x43e, 0x43f,
0x430, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x430,
0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x430,
0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x430, 0x440,
0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x424, 0x438, 0x434, 0x436, 0x438,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x424, 0x438, 0x434, 0x436, 0x438,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x424, 0x438, 0x434, 0x436, 0x438, 0x20,
0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x424, 0x440, 0x430, 0x43d, 0x446, 0x443, 0x437, 0x20, 0x413, 0x432, 0x438, 0x430,
0x43d, 0x430, 0x441, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x424,
0x440, 0x430, 0x43d, 0x446, 0x438, 0x44f, 0x43d, 0x44b, 0x4a3, 0x20, 0x43e, 0x4a3,
0x442, 0x4af, 0x441, 0x442, 0x456, 0x43a, 0x20, 0x430, 0x439, 0x43c, 0x430, 0x493,
0x44b, 0x20, 0x436, 0x4d9, 0x43d, 0x435, 0x20, 0x410, 0x43d, 0x442, 0x430, 0x440,
0x43a, 0x442, 0x438, 0x43a, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x413, 0x430, 0x43c, 0x431, 0x44c, 0x435, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x413, 0x440, 0x443, 0x437, 0x438, 0x44f, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x413, 0x440, 0x443, 0x437, 0x438, 0x44f,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x413, 0x440, 0x443, 0x437, 0x438, 0x44f,
0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x413, 0x438, 0x43b, 0x431, 0x435, 0x440, 0x442, 0x20, 0x430, 0x440, 0x430,
0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x4a3, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x413, 0x440, 0x438, 0x43d, 0x432, 0x438, 0x447, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x428, 0x44b, 0x493, 0x44b, 0x441, 0x20, 0x413,
0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x428, 0x44b, 0x493, 0x44b, 0x441, 0x20, 0x413, 0x440,
0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x428, 0x44b, 0x493, 0x44b, 0x441, 0x20, 0x413, 0x440, 0x435, 0x43d,
0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x441,
0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x441, 0x20,
0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x441, 0x20, 0x413, 0x440,
0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x436, 0x430, 0x437,
0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x440,
0x441, 0x44b, 0x20, 0x448, 0x44b, 0x493, 0x430, 0x43d, 0x430, 0x493, 0x44b, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x413, 0x430, 0x439, 0x430, 0x43d, 0x430, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x413, 0x430, 0x432, 0x430, 0x439, 0x20,
0x436, 0x4d9, 0x43d, 0x435, 0x20, 0x410, 0x43b, 0x435, 0x443, 0x442, 0x20, 0x430,
0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x413, 0x430, 0x432, 0x430, 0x439, 0x20, 0x436, 0x4d9, 0x43d, 0x435,
0x20, 0x410, 0x43b, 0x435, 0x443, 0x442, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434,
0x430, 0x440, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x413, 0x430, 0x432,
0x430, 0x439, 0x20, 0x436, 0x4d9, 0x43d, 0x435, 0x20, 0x410, 0x43b, 0x435, 0x443,
0x442, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20, 0x436,
0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x413,
0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x20, 0x436, 0x430,
0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x425, 0x43e,
0x432, 0x434, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x425, 0x43e, 0x432,
0x434, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x425, 0x43e, 0x432, 0x434, 0x20,
0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x4ae, 0x43d, 0x434, 0x456, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x4ae, 0x43d, 0x434, 0x456, 0x20, 0x43c, 0x4b1, 0x445, 0x438, 0x442,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x4ae, 0x43d, 0x434, 0x456,
0x49b, 0x44b, 0x442, 0x430, 0x439, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x41e, 0x440, 0x442, 0x430, 0x43b, 0x44b, 0x49b, 0x20, 0x418, 0x43d, 0x434, 0x43e,
0x43d, 0x435, 0x437, 0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x428, 0x44b, 0x493, 0x44b, 0x441, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435,
0x437, 0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430,
0x442, 0x44b, 0x441, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438,
0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x418, 0x440, 0x430, 0x43d,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x418, 0x440, 0x430, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x418, 0x440, 0x430, 0x43d, 0x20, 0x436, 0x430,
0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x418, 0x440,
0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x436, 0x430, 0x437,
0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x418, 0x437, 0x440,
0x430, 0x438, 0x43b, 0x44c, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x418,
0x437, 0x440, 0x430, 0x438, 0x43b, 0x44c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x418, 0x437, 0x440, 0x430, 0x438, 0x43b, 0x44c, 0x20, 0x436, 0x430, 0x437, 0x493,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x416, 0x430, 0x43f, 0x43e,
0x43d, 0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x416, 0x430,
0x43f, 0x43e, 0x43d, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x416,
0x430, 0x43f, 0x43e, 0x43d, 0x438, 0x44f, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x49a, 0x430, 0x437, 0x430, 0x49b,
0x441, 0x442, 0x430, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x428,
0x44b, 0x493, 0x44b, 0x441, 0x20, 0x49a, 0x430, 0x437, 0x430, 0x49b, 0x441, 0x442,
0x430, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x442,
0x44b, 0x441, 0x20, 0x49a, 0x430, 0x437, 0x430, 0x49b, 0x441, 0x442, 0x430, 0x43d,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x43e, 0x440, 0x435, 0x44f,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x43e, 0x440, 0x435, 0x44f,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x43e, 0x440, 0x435, 0x44f, 0x20,
0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x41a, 0x443, 0x441, 0x430, 0x438, 0x435, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e,
0x44f, 0x440, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x440,
0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x20, 0x436, 0x430, 0x437,
0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x49a, 0x44b, 0x440,
0x493, 0x44b, 0x437, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x41b, 0x430, 0x439, 0x43d, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434,
0x430, 0x440, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41b, 0x43e,
0x440, 0x434, 0x2d, 0x425, 0x430, 0x443, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x425, 0x430, 0x443, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x425, 0x430, 0x443, 0x20,
0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x20, 0x436,
0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41c,
0x430, 0x43b, 0x430, 0x439, 0x437, 0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x41c, 0x430, 0x43b, 0x44c, 0x434, 0x438, 0x432, 0x20, 0x430, 0x440,
0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x41c, 0x430, 0x440, 0x43a, 0x438, 0x437, 0x20, 0x430, 0x440, 0x430, 0x43b,
0x434, 0x430, 0x440, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41c,
0x430, 0x440, 0x448, 0x430, 0x43b, 0x43b, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434,
0x430, 0x440, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41c, 0x430,
0x432, 0x440, 0x438, 0x43a, 0x438, 0x439, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438, 0x439, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438, 0x439, 0x20,
0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x41c, 0x43e, 0x443, 0x441, 0x43e, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430, 0x20, 0x422, 0x44b, 0x43d,
0x44b, 0x49b, 0x20, 0x43c, 0x4b1, 0x445, 0x438, 0x442, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x422, 0x44b,
0x43d, 0x44b, 0x49b, 0x20, 0x43c, 0x4b1, 0x445, 0x438, 0x442, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430, 0x20,
0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x422, 0x44b, 0x43d, 0x44b, 0x49b, 0x20,
0x43c, 0x4b1, 0x445, 0x438, 0x442, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x4b0, 0x43b, 0x430, 0x43d, 0x431, 0x430, 0x442, 0x44b, 0x440, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x4b0, 0x43b, 0x430, 0x43d, 0x431, 0x430, 0x442, 0x44b,
0x440, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x4b0, 0x43b, 0x430, 0x43d, 0x431,
0x430, 0x442, 0x44b, 0x440, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41c, 0x4d9, 0x441, 0x43a, 0x435, 0x443, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41c, 0x4d9, 0x441, 0x43a, 0x435, 0x443,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41c, 0x4d9, 0x441, 0x43a, 0x435, 0x443,
0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x41c, 0x44c, 0x44f, 0x43d, 0x43c, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x41d, 0x430, 0x443, 0x440, 0x443, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x41d, 0x435, 0x43f, 0x430, 0x43b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x416, 0x430, 0x4a3, 0x430, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434,
0x43e, 0x43d, 0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x416,
0x430, 0x4a3, 0x430, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438,
0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x416, 0x430, 0x4a3, 0x430, 0x20,
0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x44f, 0x20, 0x436, 0x430,
0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x416, 0x430,
0x4a3, 0x430, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x416, 0x430, 0x4a3, 0x430, 0x20, 0x417,
0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x416, 0x430, 0x4a3, 0x430, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434,
0x438, 0x44f, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b,
0x435, 0x43d, 0x434, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41d, 0x44c,
0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434,
0x43b, 0x435, 0x43d, 0x434, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41d, 0x438, 0x443, 0x44d, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20,
0x430, 0x440, 0x430, 0x43b, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x430, 0x440, 0x430, 0x43b,
0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41d, 0x43e, 0x440, 0x444, 0x43e,
0x43b, 0x43a, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x44b, 0x20, 0x436, 0x430, 0x437,
0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x424, 0x435, 0x440,
0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x438, 0x2d, 0x41d, 0x43e, 0x440,
0x43e, 0x43d, 0x44c, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x424,
0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x438, 0x2d, 0x41d,
0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x438, 0x2d,
0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x20, 0x436, 0x430, 0x437, 0x493,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41d, 0x43e, 0x432, 0x43e,
0x441, 0x456, 0x431, 0x456, 0x440, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x456, 0x431, 0x456, 0x440, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x456, 0x431, 0x456, 0x440,
0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x41e, 0x43c, 0x431, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x41e, 0x43c, 0x431, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41e, 0x43c,
0x431, 0x44b, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x41f, 0x4d9, 0x43a, 0x456, 0x441, 0x442, 0x430, 0x43d, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f, 0x4d9, 0x43a, 0x456, 0x441, 0x442,
0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f, 0x4d9, 0x43a, 0x456,
0x441, 0x442, 0x430, 0x43d, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x43b, 0x430, 0x443, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x43f, 0x443, 0x430, 0x20, 0x2013,
0x20, 0x416, 0x430, 0x4a3, 0x430, 0x20, 0x413, 0x432, 0x438, 0x43d, 0x435, 0x44f,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x440, 0x430, 0x433,
0x432, 0x430, 0x439, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f, 0x430,
0x440, 0x430, 0x433, 0x432, 0x430, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439, 0x20, 0x436, 0x430, 0x437,
0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f, 0x435, 0x440,
0x443, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f, 0x435, 0x440, 0x443,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f, 0x435, 0x440, 0x443, 0x20, 0x436,
0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x424,
0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x20, 0x430, 0x440, 0x430, 0x43b,
0x434, 0x430, 0x440, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x424,
0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x20, 0x430, 0x440, 0x430, 0x43b,
0x434, 0x430, 0x440, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x424, 0x438,
0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434,
0x430, 0x440, 0x44b, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x424, 0x435, 0x43d, 0x438, 0x43a, 0x441, 0x20, 0x430,
0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20, 0x436,
0x4d9, 0x43d, 0x435, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x44c,
0x435, 0x440, 0x20, 0x436, 0x4d9, 0x43d, 0x435, 0x20, 0x41c, 0x438, 0x43a, 0x435,
0x43b, 0x43e, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x435, 0x43d,
0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20, 0x436, 0x4d9, 0x43d, 0x435, 0x20, 0x41c,
0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f, 0x438, 0x442, 0x43a, 0x44d,
0x440, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f, 0x43e, 0x43d,
0x43f, 0x435, 0x438, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f, 0x445,
0x435, 0x43d, 0x44c, 0x44f, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x420, 0x435, 0x44e, 0x43d, 0x44c, 0x43e, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x420, 0x43e, 0x442, 0x435, 0x440, 0x430, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438,
0x43d, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x435, 0x439, 0x448, 0x435,
0x43b, 0x44c, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x438, 0x43d, 0x433, 0x430, 0x43f,
0x443, 0x440, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x43e, 0x43b, 0x43e,
0x43c, 0x43e, 0x43d, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41e, 0x4a3, 0x442, 0x4af, 0x441,
0x442, 0x456, 0x43a, 0x20, 0x413, 0x435, 0x43e, 0x440, 0x433, 0x438, 0x44f, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x443, 0x440, 0x438, 0x43d, 0x430,
0x43c, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x451, 0x432, 0x430,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x422, 0x430, 0x438, 0x442, 0x438,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x422, 0x430, 0x439, 0x431, 0x44d,
0x439, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x422, 0x430, 0x439, 0x431,
0x44d, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x422, 0x430, 0x439, 0x431,
0x44d, 0x439, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x422, 0x4d9, 0x436, 0x456, 0x43a, 0x441, 0x442, 0x430, 0x43d,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x422, 0x43e, 0x43a, 0x435, 0x43b,
0x430, 0x443, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x422, 0x43e, 0x43d,
0x433, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x422, 0x43e, 0x43d,
0x433, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x422, 0x43e, 0x43d, 0x433,
0x430, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x422, 0x440, 0x443, 0x43a, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x422, 0x4af, 0x440, 0x456, 0x43a, 0x43c, 0x435, 0x43d, 0x441, 0x442, 0x430,
0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x422, 0x4af, 0x440, 0x456,
0x43a, 0x43c, 0x435, 0x43d, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x422, 0x4af, 0x440, 0x456, 0x43a, 0x43c, 0x435, 0x43d, 0x441, 0x442,
0x430, 0x43d, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x422, 0x443, 0x432, 0x430, 0x43b, 0x443, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430,
0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x423, 0x440, 0x443, 0x433, 0x432,
0x430, 0x439, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x4e8, 0x437, 0x431, 0x435, 0x43a, 0x441, 0x442, 0x430, 0x43d,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x4e8, 0x437, 0x431, 0x435, 0x43a,
0x441, 0x442, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x4e8, 0x437,
0x431, 0x435, 0x43a, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x436, 0x430, 0x437, 0x493,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x412, 0x430, 0x43d, 0x443,
0x430, 0x442, 0x443, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x412, 0x430,
0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x412,
0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x412, 0x435, 0x43d, 0x435, 0x441,
0x443, 0x44d, 0x43b, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x412,
0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e,
0x441, 0x442, 0x43e, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x412, 0x43b,
0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x436, 0x430,
0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x412, 0x43e,
0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440,
0x430, 0x434, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x423, 0x44d, 0x439, 0x43a, 0x20, 0x430, 0x440, 0x430,
0x43b, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x423, 0x43e, 0x43b,
0x43b, 0x438, 0x441, 0x20, 0x436, 0x4d9, 0x43d, 0x435, 0x20, 0x424, 0x443, 0x442,
0x443, 0x43d, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x42f, 0x43a,
0x443, 0x442, 0x441, 0x43a, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x42f,
0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x42f,
0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440,
0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440,
0x433, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x415, 0x43a, 0x430, 0x442, 0x435,
0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x20, 0x436, 0x430, 0x437, 0x493,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x42e, 0x43a, 0x43e, 0x43d,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x627, 0x6cb, 0x639, 0x627, 0x646,
0x633, 0x62a, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x648,
0x631, 0x62a, 0x627, 0x644, 0x649, 0x642, 0x20, 0x627, 0x641, 0x631, 0x64a, 0x643,
0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x634, 0x649, 0x639, 0x649,
0x633, 0x20, 0x627, 0x641, 0x631, 0x64a, 0x643, 0x627, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x648, 0x6ad, 0x62a, 0x6c7, 0x633, 0x62a, 0x649, 0x643, 0x20,
0x627, 0x641, 0x631, 0x64a, 0x643, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f,
0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x628, 0x627, 0x62a, 0x649, 0x633, 0x20, 0x627, 0x641, 0x631, 0x64a, 0x643, 0x627,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x628, 0x627, 0x62a, 0x649, 0x633,
0x20, 0x627, 0x641, 0x631, 0x64a, 0x643, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646,
0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x628, 0x627, 0x62a, 0x649, 0x633, 0x20, 0x627, 0x641, 0x631, 0x64a, 0x643,
0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x627, 0x644, 0x627, 0x633, 0x643, 0x627, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x627, 0x644, 0x627, 0x633, 0x643, 0x627, 0x20, 0x633, 0x62a,
0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x627, 0x644, 0x627, 0x633, 0x643, 0x627, 0x20, 0x62c, 0x627,
0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x644,
0x645, 0x627, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627,
0x644, 0x645, 0x627, 0x62a, 0x649, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627,
0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627,
0x644, 0x645, 0x627, 0x62a, 0x649, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x645, 0x627, 0x632, 0x648, 0x646,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x645, 0x627, 0x632, 0x648,
0x646, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x645, 0x627, 0x632, 0x648,
0x646, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x633, 0x648, 0x644, 0x62a, 0x6c7, 0x633, 0x62a, 0x649, 0x643, 0x20,
0x627, 0x645, 0x6d5, 0x631, 0x64a, 0x643, 0x627, 0x20, 0x648, 0x631, 0x62a, 0x627,
0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x633, 0x648,
0x644, 0x62a, 0x6c7, 0x633, 0x62a, 0x649, 0x643, 0x20, 0x627, 0x645, 0x6d5, 0x631,
0x64a, 0x643, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a,
0x62a, 0x649, 0x20, 0x648, 0x631, 0x62a, 0x627, 0x644, 0x649, 0x642, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x648, 0x644, 0x62a, 0x6c7, 0x633, 0x62a, 0x649,
0x643, 0x20, 0x627, 0x645, 0x6d5, 0x631, 0x64a, 0x643, 0x627, 0x20, 0x62c, 0x627,
0x632, 0x639, 0x649, 0x20, 0x648, 0x631, 0x62a, 0x627, 0x644, 0x649, 0x642, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x633, 0x648, 0x644, 0x62a, 0x6c7, 0x633,
0x62a, 0x649, 0x643, 0x20, 0x627, 0x645, 0x6d5, 0x631, 0x64a, 0x643, 0x627, 0x20,
0x634, 0x649, 0x639, 0x649, 0x633, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x633, 0x648, 0x644, 0x62a, 0x6c7, 0x633, 0x62a, 0x649, 0x643, 0x20, 0x627, 0x645,
0x6d5, 0x631, 0x64a, 0x643, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627,
0x631, 0x62a, 0x62a, 0x649, 0x20, 0x634, 0x649, 0x639, 0x649, 0x633, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x633, 0x648, 0x644, 0x62a, 0x6c7, 0x633, 0x62a,
0x649, 0x643, 0x20, 0x627, 0x645, 0x6d5, 0x631, 0x64a, 0x643, 0x627, 0x20, 0x62c,
0x627, 0x632, 0x639, 0x649, 0x20, 0x634, 0x649, 0x639, 0x649, 0x633, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x633, 0x648, 0x644, 0x62a, 0x6c7, 0x633, 0x62a,
0x649, 0x643, 0x20, 0x627, 0x645, 0x6d5, 0x631, 0x64a, 0x643, 0x627, 0x20, 0x62a,
0x627, 0x6cb, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x633, 0x648, 0x644,
0x62a, 0x6c7, 0x633, 0x62a, 0x649, 0x643, 0x20, 0x627, 0x645, 0x6d5, 0x631, 0x64a,
0x643, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a,
0x649, 0x20, 0x62a, 0x627, 0x6cb, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x633, 0x648, 0x644, 0x62a, 0x6c7, 0x633, 0x62a, 0x649, 0x643, 0x20, 0x627, 0x645,
0x6d5, 0x631, 0x64a, 0x643, 0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20,
0x62a, 0x627, 0x6cb, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x633, 0x648,
0x644, 0x62a, 0x6c7, 0x633, 0x62a, 0x649, 0x643, 0x20, 0x627, 0x645, 0x6d5, 0x631,
0x64a, 0x643, 0x627, 0x20, 0x62a, 0x649, 0x646, 0x649, 0x642, 0x20, 0x645, 0x6c7,
0x62d, 0x64a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x633,
0x648, 0x644, 0x62a, 0x6c7, 0x633, 0x62a, 0x649, 0x643, 0x20, 0x627, 0x645, 0x6d5,
0x631, 0x64a, 0x643, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631,
0x62a, 0x62a, 0x649, 0x20, 0x62a, 0x649, 0x646, 0x649, 0x642, 0x20, 0x645, 0x6c7,
0x62d, 0x64a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x633,
0x648, 0x644, 0x62a, 0x6c7, 0x633, 0x62a, 0x649, 0x643, 0x20, 0x627, 0x645, 0x6d5,
0x631, 0x64a, 0x643, 0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x62a,
0x649, 0x646, 0x649, 0x642, 0x20, 0x645, 0x6c7, 0x62d, 0x64a, 0x62a, 0x649, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x67e, 0x64a, 0x627, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x67e, 0x64a, 0x627, 0x20, 0x633, 0x62a,
0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x627, 0x67e, 0x64a, 0x627, 0x20, 0x62c, 0x627, 0x632, 0x639,
0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x642, 0x62a, 0x627,
0x6cb, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x642, 0x62a, 0x627,
0x6cb, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x642, 0x62a, 0x627, 0x6cb,
0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x627, 0x642, 0x62a, 0x648, 0x628, 0x6d5, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x627, 0x642, 0x62a, 0x648, 0x628, 0x6d5, 0x20, 0x633, 0x62a, 0x627,
0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x627, 0x642, 0x62a, 0x648, 0x628, 0x6d5, 0x20, 0x62c, 0x627, 0x632,
0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x633, 0x627, 0x6cb,
0x62f, 0x20, 0x627, 0x631, 0x627, 0x628, 0x64a, 0x627, 0x633, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x633, 0x627, 0x6cb, 0x62f, 0x20, 0x627, 0x631,
0x627, 0x628, 0x64a, 0x627, 0x633, 0x649, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f,
0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x633, 0x627, 0x6cb, 0x62f, 0x20, 0x627, 0x631, 0x627, 0x628, 0x64a, 0x627, 0x633,
0x649, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x627, 0x631, 0x6af, 0x6d5, 0x646, 0x62a, 0x64a, 0x646, 0x627, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x631, 0x6af, 0x6d5, 0x646, 0x62a,
0x64a, 0x646, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a,
0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x631, 0x6af,
0x6d5, 0x646, 0x62a, 0x64a, 0x646, 0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x628, 0x627, 0x62a, 0x649, 0x633,
0x20, 0x627, 0x631, 0x6af, 0x6d5, 0x646, 0x62a, 0x64a, 0x646, 0x627, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x628, 0x627, 0x62a, 0x649, 0x633, 0x20, 0x627,
0x631, 0x6af, 0x6d5, 0x646, 0x62a, 0x64a, 0x646, 0x627, 0x20, 0x633, 0x62a, 0x627,
0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x628, 0x627, 0x62a, 0x649, 0x633, 0x20, 0x627, 0x631, 0x6af, 0x6d5,
0x646, 0x62a, 0x64a, 0x646, 0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x631, 0x645, 0x6d5, 0x646, 0x64a,
0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x631, 0x645, 0x6d5,
0x646, 0x64a, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a,
0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x631, 0x645,
0x6d5, 0x646, 0x64a, 0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x62a, 0x644, 0x627, 0x646, 0x62a, 0x64a,
0x643, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x62a, 0x644,
0x627, 0x646, 0x62a, 0x64a, 0x643, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f,
0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x627, 0x62a, 0x644, 0x627, 0x646, 0x62a, 0x64a, 0x643, 0x627, 0x20, 0x62c, 0x627,
0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x648, 0x631,
0x62a, 0x627, 0x644, 0x649, 0x642, 0x20, 0x627, 0x6cb, 0x633, 0x62a, 0x631, 0x627,
0x644, 0x64a, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x6cb,
0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646,
0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x648, 0x631, 0x62a, 0x627, 0x644,
0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x6cb, 0x633,
0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649,
0x20, 0x648, 0x631, 0x62a, 0x627, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x627, 0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627,
0x20, 0x648, 0x631, 0x62a, 0x627, 0x644, 0x649, 0x642, 0x2d, 0x628, 0x627, 0x62a,
0x649, 0x633, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x6cb, 0x633,
0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f,
0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x648, 0x631, 0x62a, 0x627, 0x644, 0x649,
0x642, 0x2d, 0x628, 0x627, 0x62a, 0x649, 0x633, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x627, 0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x20,
0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x648, 0x631, 0x62a, 0x627, 0x644, 0x649,
0x642, 0x2d, 0x628, 0x627, 0x62a, 0x649, 0x633, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x634, 0x649, 0x639, 0x649, 0x633, 0x20, 0x627, 0x6cb, 0x633, 0x62a,
0x631, 0x627, 0x644, 0x64a, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x627, 0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x20, 0x633, 0x62a,
0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x634, 0x649, 0x639,
0x649, 0x633, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x6cb, 0x633,
0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649,
0x20, 0x634, 0x649, 0x639, 0x649, 0x633, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x628, 0x627, 0x62a, 0x649, 0x633, 0x20, 0x627, 0x6cb, 0x633, 0x62a, 0x631,
0x627, 0x644, 0x64a, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627,
0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x20, 0x633, 0x62a, 0x627,
0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x628, 0x627, 0x62a, 0x649,
0x633, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x6cb, 0x633, 0x62a,
0x631, 0x627, 0x644, 0x64a, 0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20,
0x628, 0x627, 0x62a, 0x649, 0x633, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x621, 0x627, 0x632, 0x649, 0x631, 0x628, 0x627, 0x64a, 0x62c, 0x627, 0x646, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x621, 0x627, 0x632, 0x649, 0x631, 0x628,
0x627, 0x64a, 0x62c, 0x627, 0x646, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627,
0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x621,
0x627, 0x632, 0x649, 0x631, 0x628, 0x627, 0x64a, 0x62c, 0x627, 0x646, 0x20, 0x62c,
0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627,
0x632, 0x648, 0x631, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x632, 0x648, 0x631, 0x20,
0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x20, 0x633, 0x62a, 0x627,
0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x627, 0x632, 0x648, 0x631, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f,
0x627, 0x631, 0x649, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x628, 0x627, 0x646, 0x6af, 0x644, 0x627, 0x62f, 0x6d5,
0x634, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x628, 0x627, 0x646, 0x6af,
0x644, 0x627, 0x62f, 0x6d5, 0x634, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627,
0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x628,
0x627, 0x646, 0x6af, 0x644, 0x627, 0x62f, 0x6d5, 0x634, 0x20, 0x62c, 0x627, 0x632,
0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x628, 0x6cb, 0x62a,
0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x628, 0x648, 0x644,
0x64a, 0x6c6, 0x64a, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x628,
0x631, 0x627, 0x632, 0x64a, 0x644, 0x64a, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x628, 0x631, 0x627, 0x632, 0x64a, 0x644, 0x64a, 0x627, 0x20, 0x633,
0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x628, 0x631, 0x627, 0x632, 0x64a, 0x644, 0x64a, 0x627,
0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x628, 0x631, 0x6cb, 0x646, 0x6d5, 0x64a, 0x2d, 0x62f, 0x627, 0x631, 0x6cb,
0x633, 0x633, 0x627, 0x644, 0x627, 0x645, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x643, 0x627, 0x628, 0x648, 0x2d, 0x6c6, 0x6d5, 0x631, 0x62f, 0x6d5, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x643, 0x627, 0x628, 0x648, 0x2d, 0x6c6,
0x6d5, 0x631, 0x62f, 0x6d5, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631,
0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x643, 0x627,
0x628, 0x648, 0x2d, 0x6c6, 0x6d5, 0x631, 0x62f, 0x6d5, 0x20, 0x62c, 0x627, 0x632,
0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x686, 0x627, 0x645,
0x648, 0x631, 0x631, 0x648, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631,
0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x686, 0x627,
0x62a, 0x6d5, 0x645, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x686, 0x627,
0x62a, 0x6d5, 0x645, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a,
0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x686, 0x627, 0x62a,
0x6d5, 0x645, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x686, 0x64a, 0x644, 0x64a, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x686, 0x64a, 0x644, 0x64a, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f,
0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x686, 0x64a, 0x644, 0x64a, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x642, 0x649, 0x62a, 0x627, 0x64a, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x642, 0x649, 0x62a, 0x627, 0x64a, 0x20, 0x633,
0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x642, 0x649, 0x62a, 0x627, 0x64a, 0x20, 0x62c, 0x627,
0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x643, 0x631,
0x64a, 0x633, 0x62a, 0x645, 0x627, 0x633, 0x20, 0x627, 0x631, 0x627, 0x644, 0x649,
0x646, 0x649, 0x6ad, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x643, 0x648,
0x643, 0x648, 0x633, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649,
0x646, 0x649, 0x6ad, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x643, 0x648,
0x644, 0x6cb, 0x645, 0x628, 0x64a, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x643, 0x648, 0x644, 0x6cb, 0x645, 0x628, 0x64a, 0x627, 0x20, 0x633, 0x62a,
0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x643, 0x648, 0x644, 0x6cb, 0x645, 0x628, 0x64a, 0x627, 0x20,
0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x643, 0x6cb, 0x643, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649,
0x646, 0x649, 0x6ad, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x643, 0x6cb,
0x643, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x646, 0x649,
0x6ad, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x643, 0x6cb, 0x643, 0x20, 0x627,
0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x646, 0x649, 0x6ad, 0x20, 0x62c,
0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x643,
0x6cb, 0x628, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x643, 0x6cb,
0x628, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a,
0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x643, 0x6cb, 0x628, 0x627,
0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x62f, 0x6d5, 0x64a, 0x6c6, 0x64a, 0x633, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x62f, 0x64a, 0x6cb, 0x645, 0x648, 0x646, 0x2d, 0x62f, 0x64a, 0x6cb,
0x631, 0x6c6, 0x64a, 0x644, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x634,
0x649, 0x639, 0x649, 0x633, 0x20, 0x62a, 0x64a, 0x645, 0x648, 0x631, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x67e, 0x627, 0x633, 0x62d, 0x627, 0x20, 0x627,
0x631, 0x627, 0x644, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x67e,
0x627, 0x633, 0x62d, 0x627, 0x20, 0x627, 0x631, 0x627, 0x644, 0x649, 0x20, 0x633,
0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x67e, 0x627, 0x633, 0x62d, 0x627, 0x20, 0x627, 0x631,
0x627, 0x644, 0x649, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x6d5, 0x643, 0x6c6, 0x627, 0x62f, 0x648, 0x631, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x648, 0x631, 0x62a, 0x627, 0x644, 0x649,
0x642, 0x20, 0x6d5, 0x6cb, 0x631, 0x648, 0x67e, 0x627, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x648, 0x631, 0x62a, 0x627, 0x644, 0x649, 0x642, 0x20, 0x6d5,
0x6cb, 0x631, 0x648, 0x67e, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627,
0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x648,
0x631, 0x62a, 0x627, 0x644, 0x649, 0x642, 0x20, 0x6d5, 0x6cb, 0x631, 0x648, 0x67e,
0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x634, 0x649, 0x639, 0x649, 0x633, 0x20, 0x6d5, 0x6cb, 0x631, 0x648,
0x67e, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x634, 0x649, 0x639,
0x649, 0x633, 0x20, 0x6d5, 0x6cb, 0x631, 0x648, 0x67e, 0x627, 0x20, 0x633, 0x62a,
0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x634, 0x649, 0x639, 0x649, 0x633, 0x20, 0x6d5, 0x6cb, 0x631,
0x648, 0x67e, 0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x642, 0x64a, 0x649, 0x631, 0x20, 0x634, 0x649, 0x639,
0x649, 0x633, 0x20, 0x6d5, 0x6cb, 0x631, 0x648, 0x67e, 0x627, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x628, 0x627, 0x62a, 0x649, 0x633, 0x20, 0x6d5, 0x6cb,
0x631, 0x648, 0x67e, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x628,
0x627, 0x62a, 0x649, 0x633, 0x20, 0x6d5, 0x6cb, 0x631, 0x648, 0x67e, 0x627, 0x20,
0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x628, 0x627, 0x62a, 0x649, 0x633, 0x20, 0x6d5,
0x6cb, 0x631, 0x648, 0x67e, 0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x641, 0x648, 0x644, 0x643, 0x644, 0x6d5,
0x646, 0x62f, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x641, 0x648, 0x644, 0x643, 0x644, 0x6d5,
0x646, 0x62f, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x20,
0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x641, 0x648, 0x644, 0x643, 0x644, 0x6d5, 0x646,
0x62f, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x20, 0x62c,
0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x641,
0x64a, 0x62c, 0x64a, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x641, 0x64a,
0x62c, 0x64a, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a,
0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x641, 0x64a, 0x62c, 0x64a,
0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x641, 0x631, 0x627, 0x646, 0x62a, 0x633, 0x6cb, 0x632, 0x20, 0x6af, 0x6c6,
0x64a, 0x627, 0x646, 0x627, 0x633, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x641, 0x631, 0x627, 0x646, 0x62a, 0x633, 0x64a, 0x627, 0x646, 0x649, 0x6ad,
0x20, 0x648, 0x6ad, 0x62a, 0x6c7, 0x633, 0x62a, 0x649, 0x643, 0x20, 0x627, 0x64a,
0x645, 0x627, 0x639, 0x649, 0x20, 0x62c, 0x627, 0x646, 0x6d5, 0x20, 0x627, 0x646,
0x62a, 0x627, 0x631, 0x643, 0x62a, 0x64a, 0x643, 0x627, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x6af, 0x627, 0x644, 0x627, 0x67e, 0x627, 0x6af, 0x648, 0x633,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6af, 0x627, 0x645, 0x628, 0x6d5,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6af, 0x631, 0x6cb, 0x632, 0x64a,
0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6af, 0x631, 0x6cb, 0x632,
0x64a, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a,
0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6af, 0x631, 0x6cb, 0x632,
0x64a, 0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x6af, 0x64a, 0x644, 0x628, 0x6d5, 0x631, 0x62a, 0x20, 0x627,
0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x646, 0x649, 0x6ad, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x6af, 0x631, 0x64a, 0x646, 0x6c6, 0x64a, 0x686,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x634, 0x649, 0x639, 0x649, 0x633,
0x20, 0x6af, 0x631, 0x6d5, 0x646, 0x644, 0x627, 0x646, 0x62f, 0x64a, 0x627, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x634, 0x649, 0x639, 0x649, 0x633, 0x20,
0x6af, 0x631, 0x6d5, 0x646, 0x644, 0x627, 0x646, 0x62f, 0x64a, 0x627, 0x20, 0x633,
0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x634, 0x649, 0x639, 0x649, 0x633, 0x20, 0x6af, 0x631,
0x6d5, 0x646, 0x644, 0x627, 0x646, 0x62f, 0x64a, 0x627, 0x20, 0x62c, 0x627, 0x632,
0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x628, 0x627, 0x62a,
0x649, 0x633, 0x20, 0x6af, 0x631, 0x6d5, 0x646, 0x644, 0x627, 0x646, 0x62f, 0x64a,
0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x628, 0x627, 0x62a, 0x649,
0x633, 0x20, 0x6af, 0x631, 0x6d5, 0x646, 0x644, 0x627, 0x646, 0x62f, 0x64a, 0x627,
0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x628, 0x627, 0x62a, 0x649, 0x633, 0x20,
0x6af, 0x631, 0x6d5, 0x646, 0x644, 0x627, 0x646, 0x62f, 0x64a, 0x627, 0x20, 0x62c,
0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x67e,
0x627, 0x631, 0x633, 0x649, 0x20, 0x634, 0x649, 0x639, 0x627, 0x646, 0x627, 0x639,
0x649, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6af, 0x627, 0x64a, 0x627, 0x646,
0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6af, 0x627, 0x6c6, 0x627,
0x64a, 0x20, 0x62c, 0x627, 0x646, 0x6d5, 0x20, 0x627, 0x644, 0x6d5, 0x6cb, 0x62a,
0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x6af, 0x627, 0x6c6, 0x627, 0x64a, 0x20, 0x62c, 0x627,
0x646, 0x6d5, 0x20, 0x627, 0x644, 0x6d5, 0x6cb, 0x62a, 0x20, 0x627, 0x631, 0x627,
0x644, 0x62f, 0x627, 0x631, 0x649, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627,
0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6af,
0x627, 0x6c6, 0x627, 0x64a, 0x20, 0x62c, 0x627, 0x646, 0x6d5, 0x20, 0x627, 0x644,
0x6d5, 0x6cb, 0x62a, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649,
0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x62d, 0x648, 0x6ad, 0x643, 0x648, 0x6ad, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x62d, 0x648, 0x6ad, 0x643, 0x648, 0x6ad, 0x20, 0x633, 0x62a, 0x627,
0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x62d, 0x648, 0x6ad, 0x643, 0x648, 0x6ad, 0x20, 0x62c, 0x627, 0x632,
0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62d, 0x648, 0x6c6,
0x62f, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62d, 0x648, 0x6c6, 0x62f,
0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62d, 0x648, 0x6c6, 0x62f, 0x20, 0x62c,
0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x621,
0x6c7, 0x646, 0x62f, 0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633, 0x62a, 0x627,
0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x621, 0x6c7, 0x646, 0x62f, 0x649, 0x20, 0x645, 0x6c7, 0x62d, 0x64a,
0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x621, 0x6c7, 0x646,
0x62f, 0x649, 0x2d, 0x642, 0x649, 0x62a, 0x627, 0x64a, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x648, 0x631, 0x62a, 0x627, 0x644, 0x649, 0x642, 0x20, 0x64a,
0x646, 0x62f, 0x648, 0x646, 0x6d5, 0x632, 0x64a, 0x627, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x634, 0x649, 0x639, 0x649, 0x633, 0x20, 0x64a, 0x646, 0x62f,
0x648, 0x646, 0x6d5, 0x632, 0x64a, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x628, 0x627, 0x62a, 0x649, 0x633, 0x20, 0x64a, 0x646, 0x62f, 0x648, 0x646,
0x6d5, 0x632, 0x64a, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x64a,
0x631, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x64a, 0x631,
0x627, 0x646, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a,
0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x64a, 0x631, 0x627, 0x646,
0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x64a, 0x631, 0x643, 0x6cb, 0x62a, 0x633, 0x643, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x64a, 0x631, 0x643, 0x6cb, 0x62a, 0x633, 0x643, 0x20, 0x633,
0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x64a, 0x631, 0x643, 0x6cb, 0x62a, 0x633, 0x643, 0x20,
0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x64a, 0x632, 0x631, 0x627, 0x64a, 0x644, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x64a, 0x632, 0x631, 0x627, 0x64a, 0x644, 0x20, 0x633, 0x62a, 0x627, 0x646,
0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x64a, 0x632, 0x631, 0x627, 0x64a, 0x644, 0x20, 0x62c, 0x627, 0x632, 0x639,
0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62c, 0x627, 0x67e, 0x648,
0x646, 0x64a, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62c, 0x627,
0x67e, 0x648, 0x646, 0x64a, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627,
0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62c,
0x627, 0x67e, 0x648, 0x646, 0x64a, 0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x642, 0x627, 0x632, 0x627, 0x642,
0x20, 0x6d5, 0x644, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x634,
0x649, 0x639, 0x649, 0x633, 0x20, 0x642, 0x627, 0x632, 0x627, 0x642, 0x20, 0x6d5,
0x644, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x628, 0x627, 0x62a,
0x649, 0x633, 0x20, 0x642, 0x627, 0x632, 0x627, 0x642, 0x20, 0x6d5, 0x644, 0x649,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x643, 0x648, 0x631, 0x6d5, 0x64a,
0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x643, 0x648, 0x631, 0x6d5,
0x64a, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a,
0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x643, 0x648, 0x631, 0x6d5,
0x64a, 0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x643, 0x6cb, 0x633, 0x627, 0x64a, 0x6d5, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x643, 0x631, 0x627, 0x633, 0x646, 0x648, 0x64a, 0x627,
0x631, 0x633, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x643, 0x631,
0x627, 0x633, 0x646, 0x648, 0x64a, 0x627, 0x631, 0x633, 0x643, 0x20, 0x633, 0x62a,
0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x643, 0x631, 0x627, 0x633, 0x646, 0x648, 0x64a, 0x627, 0x631,
0x633, 0x643, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x642, 0x649, 0x631, 0x639, 0x649, 0x632, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x644, 0x627, 0x64a, 0x646,
0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x644, 0x648, 0x631, 0x62f, 0x2d, 0x62d, 0x627, 0x6cb,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x644, 0x648, 0x631, 0x62f, 0x2d,
0x62d, 0x627, 0x6cb, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a,
0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x644, 0x648, 0x631,
0x62f, 0x2d, 0x62d, 0x627, 0x6cb, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x645, 0x627, 0x6af, 0x627, 0x62f, 0x627,
0x646, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x645, 0x627, 0x6af, 0x627,
0x62f, 0x627, 0x646, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a,
0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x645, 0x627, 0x6af,
0x627, 0x62f, 0x627, 0x646, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x645, 0x627, 0x644, 0x627, 0x64a, 0x632, 0x64a,
0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x645, 0x627, 0x644, 0x62f,
0x64a, 0x6c6, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x645, 0x627, 0x631, 0x643, 0x64a, 0x632,
0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x645, 0x627, 0x631, 0x634, 0x627, 0x644, 0x20, 0x627,
0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x645, 0x627, 0x6c6, 0x631, 0x64a, 0x643, 0x64a, 0x64a, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x645, 0x627, 0x6c6, 0x631, 0x64a, 0x643, 0x64a,
0x64a, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x645, 0x627, 0x6c6, 0x631, 0x64a,
0x643, 0x64a, 0x64a, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x645, 0x648, 0x6cb, 0x633, 0x648, 0x646, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x645, 0x6d5, 0x643, 0x633, 0x64a, 0x643, 0x627,
0x20, 0x62a, 0x649, 0x646, 0x649, 0x642, 0x20, 0x645, 0x6c7, 0x62d, 0x64a, 0x62a,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x645, 0x6d5, 0x643, 0x633, 0x64a,
0x643, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a,
0x649, 0x20, 0x62a, 0x649, 0x646, 0x649, 0x642, 0x20, 0x645, 0x6c7, 0x62d, 0x64a,
0x62a, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x645, 0x6d5, 0x643, 0x633,
0x64a, 0x643, 0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x62a, 0x649,
0x646, 0x649, 0x642, 0x20, 0x645, 0x6c7, 0x62d, 0x64a, 0x62a, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x6c7, 0x644, 0x627, 0x646, 0x628, 0x627, 0x62a, 0x649,
0x631, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6c7, 0x644, 0x627, 0x646,
0x628, 0x627, 0x62a, 0x649, 0x631, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627,
0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6c7,
0x644, 0x627, 0x646, 0x628, 0x627, 0x62a, 0x649, 0x631, 0x20, 0x62c, 0x627, 0x632,
0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x645, 0x627, 0x633,
0x643, 0x6d5, 0x6cb, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x645, 0x627,
0x633, 0x643, 0x6d5, 0x6cb, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631,
0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x645, 0x627,
0x633, 0x643, 0x6d5, 0x6cb, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x645, 0x64a, 0x627, 0x646, 0x645, 0x627, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x646, 0x627, 0x6cb, 0x631, 0x6cb, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x646, 0x6d5, 0x67e, 0x627, 0x644, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62c, 0x627, 0x6ad, 0x627, 0x20, 0x643,
0x627, 0x644, 0x6d5, 0x62f, 0x648, 0x646, 0x64a, 0x627, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x62c, 0x627, 0x6ad, 0x627, 0x20, 0x643, 0x627, 0x644, 0x6d5,
0x62f, 0x648, 0x646, 0x64a, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627,
0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62c,
0x627, 0x6ad, 0x627, 0x20, 0x643, 0x627, 0x644, 0x6d5, 0x62f, 0x648, 0x646, 0x64a,
0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x62c, 0x627, 0x6ad, 0x627, 0x20, 0x632, 0x6d5, 0x644, 0x627, 0x646,
0x62f, 0x64a, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62c, 0x627,
0x6ad, 0x627, 0x20, 0x632, 0x6d5, 0x644, 0x627, 0x646, 0x62f, 0x64a, 0x627, 0x20,
0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x62c, 0x627, 0x6ad, 0x627, 0x20, 0x632, 0x6d5,
0x644, 0x627, 0x646, 0x62f, 0x64a, 0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x646, 0x64a, 0x6cb, 0x641, 0x627,
0x6cb, 0x646, 0x62f, 0x644, 0x6d5, 0x646, 0x62f, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x646, 0x64a, 0x6cb, 0x641, 0x627, 0x6cb, 0x646, 0x62f, 0x644, 0x6d5,
0x646, 0x62f, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a,
0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x646, 0x64a, 0x6cb, 0x641,
0x627, 0x6cb, 0x646, 0x62f, 0x644, 0x6d5, 0x646, 0x62f, 0x20, 0x62c, 0x627, 0x632,
0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x646, 0x64a, 0x6cb,
0x6d5, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x646, 0x648, 0x631, 0x641,
0x648, 0x644, 0x643, 0x20, 0x627, 0x631, 0x627, 0x644, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x646, 0x648, 0x631, 0x641, 0x648, 0x644, 0x643, 0x20,
0x627, 0x631, 0x627, 0x644, 0x649, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627,
0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x646,
0x648, 0x631, 0x641, 0x648, 0x644, 0x643, 0x20, 0x627, 0x631, 0x627, 0x644, 0x649,
0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x641, 0x6d5, 0x631, 0x646, 0x627, 0x646, 0x62f, 0x6cb, 0x2d, 0x62f, 0x64a,
0x2d, 0x646, 0x648, 0x631, 0x648, 0x646, 0x64a, 0x627, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x641, 0x6d5, 0x631, 0x646, 0x627, 0x646, 0x62f, 0x6cb, 0x2d,
0x62f, 0x64a, 0x2d, 0x646, 0x648, 0x631, 0x648, 0x646, 0x64a, 0x627, 0x20, 0x633,
0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x641, 0x6d5, 0x631, 0x646, 0x627, 0x646, 0x62f, 0x6cb,
0x2d, 0x62f, 0x64a, 0x2d, 0x646, 0x648, 0x631, 0x648, 0x646, 0x64a, 0x627, 0x20,
0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x62c, 0x627, 0x6ad, 0x627, 0x633, 0x649, 0x628, 0x649, 0x631, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x62c, 0x627, 0x6ad, 0x627, 0x633, 0x649, 0x628, 0x649,
0x631, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62c, 0x627, 0x6ad, 0x627, 0x633,
0x649, 0x628, 0x649, 0x631, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x648, 0x645, 0x628, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x648, 0x645, 0x628, 0x649, 0x20, 0x633, 0x62a, 0x627,
0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x648, 0x645, 0x628, 0x649, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x67e, 0x627, 0x643, 0x649, 0x633,
0x62a, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x67e, 0x627,
0x643, 0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f,
0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x67e, 0x627, 0x643, 0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x62c, 0x627, 0x632,
0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x67e, 0x627, 0x644,
0x627, 0x6cb, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x67e, 0x627, 0x67e,
0x6cb, 0x627, 0x20, 0x2013, 0x20, 0x62c, 0x627, 0x6ad, 0x627, 0x20, 0x6af, 0x6c6,
0x64a, 0x646, 0x6d5, 0x64a, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x67e, 0x627, 0x631, 0x627, 0x6af, 0x6c6, 0x627, 0x64a, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x67e, 0x627, 0x631, 0x627, 0x6af, 0x6c6, 0x627, 0x64a, 0x20,
0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x67e, 0x627, 0x631, 0x627, 0x6af, 0x6c6, 0x627,
0x64a, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x67e, 0x6d5, 0x631, 0x6cb, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x67e, 0x6d5, 0x631, 0x6cb, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627,
0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x67e,
0x6d5, 0x631, 0x6cb, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x641, 0x64a, 0x644, 0x64a, 0x67e, 0x64a, 0x646, 0x20,
0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x641, 0x64a, 0x644, 0x64a, 0x67e, 0x64a, 0x646, 0x20, 0x627,
0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x20, 0x633, 0x62a, 0x627, 0x646,
0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x641, 0x64a, 0x644, 0x64a, 0x67e, 0x64a, 0x646, 0x20, 0x627, 0x631, 0x627,
0x644, 0x62f, 0x627, 0x631, 0x649, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x641, 0x6d5, 0x646, 0x64a, 0x643, 0x633,
0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x633, 0x6d5, 0x646, 0x2d, 0x67e, 0x6d5, 0x631, 0x20,
0x62c, 0x627, 0x646, 0x6d5, 0x20, 0x645, 0x64a, 0x643, 0x6d5, 0x644, 0x648, 0x646,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x633, 0x6d5, 0x646, 0x2d, 0x67e,
0x6d5, 0x631, 0x20, 0x62c, 0x627, 0x646, 0x6d5, 0x20, 0x645, 0x64a, 0x643, 0x6d5,
0x644, 0x648, 0x646, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a,
0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x633, 0x6d5, 0x646,
0x2d, 0x67e, 0x6d5, 0x631, 0x20, 0x62c, 0x627, 0x646, 0x6d5, 0x20, 0x645, 0x64a,
0x643, 0x6d5, 0x644, 0x648, 0x646, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x67e, 0x64a, 0x62a, 0x643, 0x6d5, 0x631,
0x646, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x67e, 0x648, 0x646, 0x67e,
0x6d5, 0x64a, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x67e, 0x62d, 0x6d5,
0x646, 0x64a, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x642,
0x649, 0x632, 0x649, 0x644, 0x648, 0x631, 0x62f, 0x627, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x642, 0x649, 0x632, 0x649, 0x644, 0x648, 0x631, 0x62f, 0x627,
0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x642, 0x649, 0x632, 0x649, 0x644, 0x648,
0x631, 0x62f, 0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x631, 0x6d5, 0x64a, 0x6cb, 0x646, 0x648, 0x646, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x631, 0x648, 0x62a, 0x6d5, 0x631, 0x627,
0x20, 0x6cb, 0x642, 0x649, 0x62a, 0x649, 0x633, 0x627, 0x62d, 0x627, 0x644, 0x64a,
0x646, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x633, 0x627, 0x62d, 0x627,
0x644, 0x64a, 0x646, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a,
0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x633, 0x627, 0x62d,
0x627, 0x644, 0x64a, 0x646, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x633, 0x627, 0x645, 0x648, 0x627, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x633, 0x627, 0x645, 0x648, 0x627, 0x20, 0x633,
0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x633, 0x627, 0x645, 0x648, 0x627, 0x20, 0x62c, 0x627,
0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x633, 0x6d5,
0x64a, 0x634, 0x6d5, 0x644, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631,
0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x633, 0x64a, 0x646, 0x6af,
0x627, 0x67e, 0x6cb, 0x631, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631,
0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x633, 0x648,
0x644, 0x648, 0x645, 0x648, 0x646, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627,
0x631, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x648, 0x6ad, 0x62a,
0x6c7, 0x633, 0x62a, 0x649, 0x643, 0x20, 0x6af, 0x6d5, 0x648, 0x631, 0x6af, 0x64a,
0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x633, 0x6cb, 0x631, 0x64a,
0x646, 0x627, 0x645, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x633, 0x6d5,
0x6c6, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62a, 0x627, 0x64a,
0x62a, 0x64a, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62a, 0x627, 0x64a,
0x628, 0x6d5, 0x64a, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62a, 0x627,
0x64a, 0x628, 0x6d5, 0x64a, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631,
0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62a, 0x627,
0x64a, 0x628, 0x6d5, 0x64a, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x62a, 0x627, 0x62c, 0x649, 0x643, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62a, 0x648, 0x643,
0x6d5, 0x644, 0x627, 0x6cb, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62a,
0x648, 0x646, 0x6af, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62a,
0x648, 0x646, 0x6af, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631,
0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62a, 0x648,
0x646, 0x6af, 0x627, 0x20, 0x643, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x62a, 0x631, 0x6cb, 0x643, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x62a, 0x6c7, 0x631, 0x649, 0x643, 0x645, 0x6d5, 0x646, 0x633,
0x62a, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62a, 0x6c7,
0x631, 0x649, 0x643, 0x645, 0x6d5, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633,
0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x62a, 0x6c7, 0x631, 0x649, 0x643, 0x645, 0x6d5, 0x646,
0x633, 0x62a, 0x627, 0x646, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x62a, 0x6cb, 0x6c6, 0x627, 0x644, 0x6cb, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6cb, 0x631, 0x6cb, 0x6af, 0x6c6, 0x627,
0x64a, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6cb, 0x631, 0x6cb, 0x6af,
0x6c6, 0x627, 0x64a, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a,
0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6cb, 0x631, 0x6cb,
0x6af, 0x6c6, 0x627, 0x64a, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x648, 0x632, 0x628, 0x6d5, 0x643, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x648, 0x632, 0x628,
0x6d5, 0x643, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f,
0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x648, 0x632, 0x628, 0x6d5, 0x643, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x62c, 0x627,
0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6c6, 0x627,
0x646, 0x6cb, 0x627, 0x62a, 0x6cb, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x6c6, 0x627, 0x646, 0x6cb, 0x627, 0x62a, 0x6cb, 0x20, 0x633, 0x62a, 0x627, 0x646,
0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x6c6, 0x627, 0x646, 0x6cb, 0x627, 0x62a, 0x6cb, 0x20, 0x62c, 0x627, 0x632,
0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6c6, 0x6d5, 0x646,
0x6d5, 0x633, 0x6cb, 0x6d5, 0x644, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x6c6, 0x644, 0x627, 0x62f, 0x64a, 0x6c6, 0x648, 0x633, 0x62a, 0x648, 0x643,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6c6, 0x644, 0x627, 0x62f, 0x64a,
0x6c6, 0x648, 0x633, 0x62a, 0x648, 0x643, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f,
0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x6c6, 0x644, 0x627, 0x62f, 0x64a, 0x6c6, 0x648, 0x633, 0x62a, 0x648, 0x643, 0x20,
0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x6c6, 0x648, 0x644, 0x6af, 0x648, 0x6af, 0x631, 0x627, 0x62f, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x6c6, 0x648, 0x644, 0x6af, 0x648, 0x6af, 0x631, 0x627,
0x62f, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6c6, 0x648, 0x644, 0x6af, 0x648,
0x6af, 0x631, 0x627, 0x62f, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x6cb, 0x6d5, 0x64a, 0x643, 0x20, 0x627, 0x631,
0x627, 0x644, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6cb, 0x648,
0x644, 0x644, 0x64a, 0x633, 0x20, 0x62c, 0x627, 0x646, 0x6d5, 0x20, 0x641, 0x6cb,
0x62a, 0x6cb, 0x646, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x64a,
0x627, 0x643, 0x6cb, 0x62a, 0x633, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x64a, 0x627, 0x643, 0x6cb, 0x62a, 0x633, 0x643, 0x20, 0x633, 0x62a, 0x627,
0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x64a, 0x627, 0x643, 0x6cb, 0x62a, 0x633, 0x643, 0x20, 0x62c, 0x627,
0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6d5, 0x643,
0x627, 0x62a, 0x6d5, 0x631, 0x64a, 0x646, 0x628, 0x6cb, 0x631, 0x6af, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x6d5, 0x643, 0x627, 0x62a, 0x6d5, 0x631, 0x64a,
0x646, 0x628, 0x6cb, 0x631, 0x6af, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627,
0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6d5,
0x643, 0x627, 0x62a, 0x6d5, 0x631, 0x64a, 0x646, 0x628, 0x6cb, 0x631, 0x6af, 0x20,
0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x64a, 0x6cb, 0x643, 0x648, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x17a0, 0x17d2,
0x179c, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x1793, 0x17b8, 0x179f, 0x17d2, 0x1790, 0x17b6, 0x1793,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x17a0, 0x17d2,
0x179c, 0x17d2, 0x179a, 0x17b7, 0x1780, 0x200b, 0x1780, 0x178e, 0x17d2, 0x178a, 0x17b6, 0x179b,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x17a0, 0x17d2,
0x179c, 0x17d2, 0x179a, 0x17b7, 0x1780, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x1780, 0x17be,
0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x17a0,
0x17d2, 0x179c, 0x17d2, 0x179a, 0x17b7, 0x1780, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x178f,
0x17d2, 0x1794, 0x17bc, 0x1784, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b,
0x17a2, 0x17b6, 0x17a0, 0x17d2, 0x179c, 0x17d2, 0x179a, 0x17b7, 0x1780, 0x200b, 0x1781, 0x17b6,
0x1784, 0x200b, 0x179b, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2,
0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6,
0x17a0, 0x17d2, 0x179c, 0x17d2, 0x179a, 0x17b7, 0x1780, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b,
0x179b, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6,
0x17a0, 0x17d2, 0x179c, 0x17d2, 0x179a, 0x17b7, 0x1780, 0x200b, 0x200b, 0x200b, 0x1781, 0x17b6,
0x1784, 0x179b, 0x17b7, 0x1785, 0x200b, 0x200b, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c,
0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x200b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a2, 0x17b6, 0x17a1, 0x17b6, 0x179f, 0x17d2, 0x1780, 0x17b6, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b,
0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x17a1, 0x17b6, 0x179f, 0x17d2, 0x1780, 0x17b6, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3,
0x200b, 0x1793, 0x17c5, 0x200b, 0x200b, 0x17a2, 0x17b6, 0x17a1, 0x17b6, 0x179f, 0x17d2, 0x1780,
0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x1798,
0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x179f, 0x17bc, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x179f,
0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2,
0x17b6, 0x1798, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x179f, 0x17bc, 0x1793, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x1798, 0x17c9, 0x17b6, 0x17a0, 0x17d2,
0x179f, 0x17bc, 0x1793, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x1780, 0x17d2, 0x178f,
0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1791, 0x17d2,
0x179c, 0x17b8, 0x1794, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x179a, 0x17b7, 0x1780, 0x200b, 0x1781,
0x17b6, 0x1784, 0x200b, 0x1787, 0x17be, 0x1784, 0x1797, 0x17b6, 0x1782, 0x1780, 0x178e, 0x17d2,
0x178f, 0x17b6, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x179f, 0x17d2, 0x178f,
0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x1793, 0x17c5, 0x200b, 0x1791, 0x17d2, 0x179c, 0x17b8,
0x1794, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x179a, 0x17b7, 0x1780, 0x200b, 0x1781, 0x17b6, 0x1784,
0x200b, 0x1787, 0x17be, 0x1784, 0x1797, 0x17b6, 0x1782, 0x1780, 0x178e, 0x17d2, 0x178f, 0x17b6,
0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x1796, 0x17c1, 0x179b, 0x1790, 0x17d2,
0x1784, 0x17c3, 0x1793, 0x17c5, 0x200b, 0x1791, 0x17d2, 0x179c, 0x17b8, 0x1794, 0x17a2, 0x17b6,
0x1798, 0x17c1, 0x179a, 0x17b7, 0x1780, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x1787, 0x17be,
0x1784, 0x1797, 0x17b6, 0x1782, 0x1780, 0x178e, 0x17d2, 0x178f, 0x17b6, 0x179b, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x1793, 0x17c5, 0x1791, 0x17d2, 0x179c, 0x17b8, 0x1794, 0x17a2, 0x17b6, 0x1798,
0x179a, 0x17b7, 0x1780, 0x1781, 0x17b6, 0x1784, 0x1787, 0x17be, 0x1784, 0x1797, 0x17b6, 0x1782,
0x1781, 0x17b6, 0x1784, 0x1780, 0x17be, 0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x179f, 0x17d2,
0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x1793, 0x17c5, 0x1791, 0x17d2, 0x179c, 0x17b8,
0x1794, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x179a, 0x17b7, 0x1780, 0x1781, 0x17b6, 0x1784, 0x1787,
0x17be, 0x1784, 0x1797, 0x17b6, 0x1782, 0x1781, 0x17b6, 0x1784, 0x1780, 0x17be, 0x178f, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x1796, 0x17c1, 0x179b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x1793, 0x17c5,
0x1791, 0x17d2, 0x179c, 0x17b8, 0x1794, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x179a, 0x17b7, 0x1780,
0x1781, 0x17b6, 0x1784, 0x1787, 0x17be, 0x1784, 0x1797, 0x17b6, 0x1782, 0x1781, 0x17b6, 0x1784,
0x1780, 0x17be, 0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x178f, 0x17c6,
0x1794, 0x1793, 0x17cb, 0x1797, 0x17d2, 0x1793, 0x17c6, 0x1793, 0x17c3, 0x1791, 0x17d2, 0x179c,
0x17b8, 0x1794, 0x200b, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x179a, 0x17b7, 0x1780, 0x200b, 0x200b,
0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x1787, 0x17be, 0x1784, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x178f,
0x17c6, 0x1794, 0x1793, 0x17cb, 0x1797, 0x17d2, 0x1793, 0x17c6, 0x1793, 0x17c3, 0x1791, 0x17d2,
0x179c, 0x17b8, 0x1794, 0x200b, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x179a, 0x17b7, 0x1780, 0x200b,
0x200b, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x1787, 0x17be, 0x1784, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x1793, 0x17c5, 0x178f,
0x17c6, 0x1794, 0x1793, 0x17cb, 0x1797, 0x17d2, 0x1793, 0x17c6, 0x1793, 0x17c3, 0x1791, 0x17d2,
0x179c, 0x17b8, 0x1794, 0x200b, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x179a, 0x17b7, 0x1780, 0x200b,
0x200b, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x1787, 0x17be, 0x1784, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x1793, 0x17c5, 0x1794, 0x17c9, 0x17b6, 0x179f, 0x17ca, 0x17b8, 0x17a0, 0x17d2, 0x179c,
0x17b7, 0x1780, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x179a, 0x17b7, 0x1780, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x179f, 0x17d2, 0x178a, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x1793, 0x17c5, 0x1794,
0x17c9, 0x17b6, 0x179f, 0x17ca, 0x17b8, 0x17a0, 0x17d2, 0x179c, 0x17b7, 0x1780, 0x17a2, 0x17b6,
0x1798, 0x17c1, 0x179a, 0x17b7, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1796, 0x17c1, 0x179b,
0x1790, 0x17d2, 0x1784, 0x17c3, 0x1793, 0x17c5, 0x1794, 0x17c9, 0x17b6, 0x179f, 0x17ca, 0x17b8,
0x17a0, 0x17d2, 0x179c, 0x17b7, 0x1780, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x179a, 0x17b7, 0x1780,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x1794, 0x17d2,
0x1799, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb,
0x178a, 0x17b6, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x1794, 0x17d2, 0x1799, 0x17b6,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2, 0x1784,
0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x1794, 0x17d2, 0x1799, 0x17b6, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x179a, 0x17c9, 0x17b6,
0x1794, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb,
0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x179a, 0x17c9, 0x17b6,
0x1794, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790,
0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x179a, 0x17c9, 0x17b6,
0x1794, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6,
0x17a0, 0x17d2, 0x179f, 0x1784, 0x17cb, 0x1791, 0x17b8, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b,
0x17a2, 0x17b6, 0x17a0, 0x17d2, 0x179f, 0x1784, 0x17cb, 0x1791, 0x17b8, 0x1793, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x17a0, 0x17d2, 0x179f, 0x1784,
0x17cb, 0x1791, 0x17b8, 0x1793, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x1780, 0x17d2,
0x178f, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6,
0x17a0, 0x17d2, 0x179f, 0x1784, 0x17cb, 0x1791, 0x17b8, 0x1793, 0x1797, 0x17b6, 0x1782, 0x1781,
0x17b6, 0x1784, 0x179b, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x179f, 0x17d2, 0x178f,
0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x17a0,
0x17d2, 0x179f, 0x1784, 0x17cb, 0x1791, 0x17b8, 0x1793, 0x1797, 0x17b6, 0x1782, 0x1781, 0x17b6,
0x1784, 0x179b, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b,
0x17a2, 0x17b6, 0x17a0, 0x17d2, 0x179f, 0x1784, 0x17cb, 0x1791, 0x17b8, 0x1793, 0x1797, 0x17b6,
0x1782, 0x1781, 0x17b6, 0x1784, 0x179b, 0x17b7, 0x1785, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc,
0x179c, 0x1780, 0x17d2, 0x178f, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x200b, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x1793, 0x17b8, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x179f, 0x17d2, 0x178a, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b,
0x17a2, 0x17b6, 0x1798, 0x17c1, 0x1793, 0x17b8, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x1793, 0x17b8, 0x1793, 0x17b6, 0x179a, 0x178a,
0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x200b, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x178f, 0x17d2, 0x179b, 0x1784, 0x17cb, 0x1791,
0x17b7, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb,
0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x178f, 0x17d2, 0x179b,
0x1784, 0x17cb, 0x1791, 0x17b7, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1,
0x179b, 0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6,
0x178f, 0x17d2, 0x179b, 0x1784, 0x17cb, 0x1791, 0x17b7, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x17a2, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17d2, 0x179a, 0x17b6, 0x179b,
0x17b8, 0x200b, 0x1780, 0x178e, 0x17d2, 0x178a, 0x17b6, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5,
0x200b, 0x17a2, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17d2, 0x179a, 0x17b6, 0x179b, 0x17b8, 0x200b,
0x1780, 0x178e, 0x17d2, 0x178a, 0x17b6, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796,
0x17c1, 0x179b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x200b, 0x200b, 0x200b, 0x1793, 0x17c5,
0x200b, 0x17a2, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17d2, 0x179a, 0x17b6, 0x179b, 0x17b8, 0x200b,
0x1780, 0x178e, 0x17d2, 0x178a, 0x17b6, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x200b, 0x200b, 0x1797, 0x17b6, 0x1782, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b,
0x179b, 0x17b7, 0x1785, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17bc, 0x179f, 0x17d2, 0x178f,
0x17d2, 0x179a, 0x17b6, 0x179b, 0x17b8, 0x200b, 0x1780, 0x178e, 0x17d2, 0x178a, 0x17b6, 0x179b,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6,
0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1797, 0x17b6, 0x1782, 0x200b, 0x1781, 0x17b6, 0x1784,
0x200b, 0x179b, 0x17b7, 0x1785, 0x200b, 0x1793, 0x17c3, 0x200b, 0x17a2, 0x17bc, 0x179f, 0x17d2,
0x178f, 0x17d2, 0x179a, 0x17b6, 0x179b, 0x17b8, 0x200b, 0x1780, 0x178e, 0x17d2, 0x178a, 0x17b6,
0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2,
0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x200b, 0x1797, 0x17b6, 0x1782, 0x200b, 0x1781,
0x17b6, 0x1784, 0x200b, 0x179b, 0x17b7, 0x1785, 0x200b, 0x1793, 0x17c3, 0x200b, 0x17a2, 0x17bc,
0x179f, 0x17d2, 0x178f, 0x17d2, 0x179a, 0x17b6, 0x179b, 0x17b8, 0x200b, 0x1780, 0x178e, 0x17d2,
0x178a, 0x17b6, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2,
0x17bc, 0x179f, 0x17d2, 0x178f, 0x17d2, 0x179a, 0x17b6, 0x179b, 0x17b8, 0x200b, 0x1781, 0x17b6,
0x1784, 0x200b, 0x1780, 0x17be, 0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2,
0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17bc,
0x179f, 0x17d2, 0x178f, 0x17d2, 0x179a, 0x17b6, 0x179b, 0x17b8, 0x200b, 0x1781, 0x17b6, 0x1784,
0x200b, 0x1780, 0x17be, 0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b,
0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17bc, 0x179f,
0x17d2, 0x178f, 0x17d2, 0x179a, 0x17b6, 0x179b, 0x17b8, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b,
0x1780, 0x17be, 0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x200b, 0x1793, 0x17c5,
0x200b, 0x17a2, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17d2, 0x179a, 0x17b6, 0x179b, 0x17b8, 0x200b,
0x1781, 0x17b6, 0x1784, 0x200b, 0x179b, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5,
0x200b, 0x17a2, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17d2, 0x179a, 0x17b6, 0x179b, 0x17b8, 0x200b,
0x1781, 0x17b6, 0x1784, 0x200b, 0x179b, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b,
0x17a2, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17d2, 0x179a, 0x17b6, 0x179b, 0x17b8, 0x200b, 0x1781,
0x17b6, 0x1784, 0x200b, 0x179b, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a2, 0x17b6, 0x179f, 0x17ca, 0x17c2, 0x1794, 0x17c3, 0x17a0, 0x17d2, 0x179f,
0x1784, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb,
0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x179f, 0x17ca, 0x17c2,
0x1794, 0x17c3, 0x17a0, 0x17d2, 0x179f, 0x1784, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x179f, 0x17ca, 0x17c2, 0x1794, 0x17c3, 0x17a0,
0x17d2, 0x179f, 0x1784, 0x17cb, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780,
0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2,
0x17c1, 0x17a0, 0x17d2, 0x179f, 0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2,
0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17c1,
0x17a0, 0x17d2, 0x179f, 0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x1793, 0x17c5,
0x200b, 0x17a2, 0x17c1, 0x17a0, 0x17d2, 0x179f, 0x179f, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc,
0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x1794, 0x1784, 0x17cb, 0x1780, 0x17d2, 0x179b, 0x17b6, 0x178a, 0x17c2, 0x179f,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178a, 0x1784, 0x17cb, 0x178a, 0x17b6,
0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x1784, 0x17cb, 0x1780, 0x17d2, 0x179b, 0x17b6,
0x178a, 0x17c2, 0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x1793, 0x17c5, 0x200b,
0x1794, 0x1784, 0x17cb, 0x1780, 0x17d2, 0x179b, 0x17b6, 0x178a, 0x17c2, 0x179f, 0x1793, 0x17b6,
0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x1793, 0x17c5, 0x1794, 0x17ca, 0x17bc, 0x178f, 0x1784, 0x17cb, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17bc, 0x179b, 0x17b8, 0x179c, 0x17b8, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17d2, 0x179a, 0x17b6, 0x179f,
0x17ca, 0x17b8, 0x179b, 0x17b8, 0x1799, 0x17c9, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b,
0x1794, 0x17d2, 0x179a, 0x17b6, 0x179f, 0x17ca, 0x17b8, 0x179b, 0x17b8, 0x1799, 0x17c9, 0x17b6,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17d2, 0x179a, 0x17b6,
0x179f, 0x17ca, 0x17b8, 0x179b, 0x17b8, 0x1799, 0x17c9, 0x17b6, 0x1793, 0x17b6, 0x200b, 0x200b,
0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x1793, 0x17c5, 0x1796, 0x17d2, 0x179a, 0x17bb, 0x1799, 0x178e, 0x17c1, 0x178a, 0x17b6,
0x179a, 0x17bc, 0x179f, 0x17b6, 0x17a1, 0x17b9, 0x1798, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x1793, 0x17c5, 0x200b, 0x1780, 0x17b6, 0x1794, 0x17cb, 0x179c, 0x17c2, 0x179a, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b,
0x1793, 0x17c5, 0x200b, 0x1780, 0x17b6, 0x1794, 0x17cb, 0x179c, 0x17c2, 0x179a, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17b6, 0x1794, 0x17cb, 0x179c,
0x17c2, 0x179a, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a,
0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a,
0x17b6, 0x179a, 0x1793, 0x17c5, 0x200b, 0x1785, 0x17b6, 0x17c6, 0x1798, 0x17c9, 0x17bc, 0x179a,
0x17c9, 0x17bc, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1785, 0x17b6,
0x1790, 0x17b6, 0x17c6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784,
0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1785, 0x17b6, 0x1790, 0x17b6,
0x17c6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2,
0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1785, 0x17b6, 0x1790, 0x17b6, 0x17c6, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x179f, 0x17ca, 0x17b8, 0x179b, 0x17b8, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x1793, 0x17c5,
0x179f, 0x17ca, 0x17b8, 0x179b, 0x17b8, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x179f,
0x17ca, 0x17b8, 0x179b, 0x17b8, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x1780, 0x17d2,
0x178f, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1785, 0x17b7,
0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a,
0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1785, 0x17b7, 0x1793, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793,
0x17c5, 0x200b, 0x1785, 0x17b7, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x200b, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x1782, 0x17d2, 0x179a, 0x17b8, 0x179f, 0x17d2, 0x1798,
0x17b6, 0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17d2,
0x179a, 0x1787, 0x17bb, 0x17c6, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x1780, 0x17bc, 0x1780, 0x17bc,
0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17bc, 0x17a1,
0x17bb, 0x17c6, 0x1794, 0x17ca, 0x17b8, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2,
0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17bc,
0x17a1, 0x17bb, 0x17c6, 0x1794, 0x17ca, 0x17b8, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x1780, 0x17bc, 0x17a1, 0x17bb, 0x17c6, 0x1794, 0x17ca, 0x17b8, 0x1793, 0x17b6,
0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x1794, 0x17d2, 0x179a, 0x1787, 0x17bb, 0x17c6, 0x200b,
0x1780, 0x17c4, 0x17c7, 0x200b, 0x1781, 0x17bc, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b,
0x1794, 0x17d2, 0x179a, 0x1787, 0x17bb, 0x17c6, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x1781, 0x17bc,
0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x1794, 0x17d2, 0x179a, 0x1787,
0x17bb, 0x17c6, 0x200b, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x1781, 0x17bc, 0x1780, 0x1793, 0x17b6,
0x1796, 0x17b6, 0x1780, 0x17cb, 0x1780, 0x178e, 0x17d2, 0x178a, 0x17b6, 0x179b, 0x200b, 0x179a,
0x178a, 0x17bc, 0x179c, 0x200b, 0x200b, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1782, 0x17bb, 0x1799, 0x1794, 0x17b6, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a,
0x200b, 0x1793, 0x17c5, 0x200b, 0x1782, 0x17bb, 0x1799, 0x1794, 0x17b6, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793,
0x17c5, 0x200b, 0x1782, 0x17bb, 0x1799, 0x1794, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x1793, 0x17c5, 0x200b, 0x178a, 0x17b6, 0x179c, 0x17b8, 0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x178c, 0x17bb, 0x1799, 0x1798, 0x17c9, 0x17bb, 0x1784, 0x178a,
0x17ba, 0x17a2, 0x17ca, 0x17bb, 0x1799, 0x179c, 0x17b8, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x200b, 0x1791, 0x17b8, 0x1798, 0x17d0, 0x179a, 0x200b, 0x1781,
0x17b6, 0x1784, 0x200b, 0x1780, 0x17be, 0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5,
0x1780, 0x17c4, 0x17c7, 0x17a2, 0x17ca, 0x17b8, 0x179f, 0x17d2, 0x1791, 0x17d0, 0x179a, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x1793,
0x17c5, 0x1780, 0x17c4, 0x17c7, 0x17a2, 0x17ca, 0x17b8, 0x179f, 0x17d2, 0x1791, 0x17d0, 0x179a,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x1780, 0x17c4, 0x17c7, 0x17a2, 0x17ca, 0x17b8,
0x179f, 0x17d2, 0x1791, 0x17d0, 0x179a, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x1780,
0x17d2, 0x178f, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2,
0x17c1, 0x1780, 0x17d2, 0x179c, 0x17b6, 0x1791, 0x17d0, 0x179a, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ba, 0x179a, 0x17c9, 0x17bb, 0x1794, 0x200b, 0x1780,
0x178e, 0x17d2, 0x178a, 0x17b6, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2,
0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ba,
0x179a, 0x17c9, 0x17bb, 0x1794, 0x200b, 0x1780, 0x178e, 0x17d2, 0x178a, 0x17b6, 0x179b, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ba, 0x179a, 0x17c9, 0x17bb,
0x1794, 0x200b, 0x1780, 0x178e, 0x17d2, 0x178a, 0x17b6, 0x179b, 0x1793, 0x17b6, 0x200b, 0x179a,
0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ba, 0x179a, 0x17c9, 0x17bb, 0x1794, 0x200b, 0x200b,
0x1781, 0x17b6, 0x1784, 0x200b, 0x1780, 0x17be, 0x178f, 0x200b, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a2, 0x17ba, 0x179a, 0x17c9, 0x17bb, 0x1794, 0x200b, 0x200b, 0x1781, 0x17b6,
0x1784, 0x200b, 0x1780, 0x17be, 0x178f, 0x200b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a2, 0x17ba, 0x179a, 0x17c9, 0x17bb, 0x1794, 0x200b, 0x200b, 0x1781, 0x17b6,
0x1784, 0x200b, 0x1780, 0x17be, 0x178f, 0x200b, 0x1793, 0x17b6, 0x200b, 0x179a, 0x178a, 0x17bc,
0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x1785, 0x17bb, 0x1784, 0x1794, 0x17bc, 0x1796, 0x17cc, 0x17b6, 0x1793, 0x17c3,
0x1791, 0x17d2, 0x179c, 0x17b8, 0x1794, 0x200b, 0x17a2, 0x17ba, 0x179a, 0x17c9, 0x17bb, 0x1794,
0x200b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ba, 0x179a,
0x17c9, 0x17bb, 0x1794, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x179b, 0x17b7, 0x1785, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a,
0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ba, 0x179a, 0x17c9, 0x17bb, 0x1794, 0x200b, 0x1781,
0x17b6, 0x1784, 0x200b, 0x179b, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a2, 0x17ba, 0x179a, 0x17c9, 0x17bb, 0x1794, 0x200b, 0x1781, 0x17b6, 0x1784,
0x200b, 0x179b, 0x17b7, 0x1785, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780,
0x17d2, 0x178a, 0x17c5, 0x200b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b,
0x1794, 0x17d2, 0x179a, 0x1787, 0x17bb, 0x17c6, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x17a0, 0x17d2,
0x179c, 0x1780, 0x17cb, 0x17a1, 0x17c2, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f,
0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x1794, 0x17d2,
0x179a, 0x1787, 0x17bb, 0x17c6, 0x200b, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x17a0, 0x17d2, 0x179c,
0x1780, 0x17cb, 0x17a1, 0x17c2, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x1793,
0x17c5, 0x1794, 0x17d2, 0x179a, 0x1787, 0x17bb, 0x17c6, 0x200b, 0x1780, 0x17c4, 0x17c7, 0x200b,
0x17a0, 0x17d2, 0x179c, 0x1780, 0x17cb, 0x17a1, 0x17c2, 0x1793, 0x1793, 0x17b6, 0x179a, 0x178a,
0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x1793, 0x17c5, 0x200b, 0x17a0, 0x17d2, 0x179c, 0x17b8, 0x1787, 0x17b8, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a0, 0x17d2, 0x179c, 0x17b8, 0x1787, 0x17b8, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x200b, 0x17a0, 0x17d2, 0x179c, 0x17b8, 0x1787, 0x17b8, 0x1793,
0x17b6, 0x200b, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a0, 0x17d2, 0x1782, 0x17b8,
0x1799, 0x17c9, 0x17b6, 0x1793, 0x200b, 0x1794, 0x17b6, 0x179a, 0x17b6, 0x17c6, 0x1784, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x1794, 0x17b6, 0x179a, 0x17b6, 0x17c6, 0x1784, 0x1781,
0x17b6, 0x1784, 0x178f, 0x17d2, 0x1794, 0x17bc, 0x1784, 0x20, 0x1793, 0x17b7, 0x1784, 0x1793,
0x17c5, 0x17a2, 0x1784, 0x17cb, 0x178f, 0x17b6, 0x17c6, 0x1784, 0x1791, 0x17b7, 0x1780, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17b6, 0x17a1, 0x17b6, 0x1794,
0x17c9, 0x17b6, 0x1780, 0x17bc, 0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x200b, 0x1780, 0x17b6, 0x17c6, 0x1794, 0x17c0, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a0, 0x17d2, 0x179f, 0x1780, 0x17a0, 0x17d2, 0x179f, 0x17ca, 0x17b8, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a,
0x200b, 0x1793, 0x17c5, 0x200b, 0x17a0, 0x17d2, 0x179f, 0x1780, 0x17a0, 0x17d2, 0x179f, 0x17ca,
0x17b8, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a0, 0x17d2, 0x179f,
0x1780, 0x17a0, 0x17d2, 0x179f, 0x17ca, 0x17b8, 0x1793, 0x17b6, 0x200b, 0x200b, 0x179a, 0x178a,
0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x1793, 0x17c5, 0x200b, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x1780, 0x17b8, 0x1794, 0x17ca, 0x17ba,
0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x1782, 0x17d2, 0x179a, 0x17b8, 0x1793,
0x179c, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x1793, 0x17c5, 0x200b,
0x17a0, 0x17d2, 0x1782, 0x17d2, 0x179a, 0x17b8, 0x1793, 0x179b, 0x17c2, 0x1793, 0x200b, 0x1781,
0x17b6, 0x1784, 0x200b, 0x1780, 0x17be, 0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b,
0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5,
0x200b, 0x200b, 0x17a0, 0x17d2, 0x1782, 0x17d2, 0x179a, 0x17b8, 0x1793, 0x179b, 0x17c2, 0x1793,
0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x1780, 0x17be, 0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x17a0, 0x17d2, 0x1782, 0x17d2, 0x179a, 0x17b8, 0x1793, 0x179b,
0x17c2, 0x1793, 0x1781, 0x17b6, 0x1784, 0x200b, 0x1780, 0x17be, 0x178f, 0x1793, 0x17b6, 0x200b,
0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x1793, 0x17c5, 0x17a0, 0x17d2, 0x1782, 0x17d2, 0x179a, 0x17b8, 0x1793, 0x179b, 0x17c2,
0x1793, 0x1781, 0x17b6, 0x1784, 0x179b, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x179f,
0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x1793, 0x17c5, 0x17a0, 0x17d2, 0x1782,
0x17d2, 0x179a, 0x17b8, 0x1793, 0x179b, 0x17c2, 0x1793, 0x1781, 0x17b6, 0x1784, 0x179b, 0x17b7,
0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x17a0, 0x17d2, 0x1782, 0x17d2, 0x179a,
0x17b8, 0x1793, 0x179b, 0x17c2, 0x1793, 0x1781, 0x17b6, 0x1784, 0x179b, 0x17b7, 0x1785, 0x1793,
0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x1780, 0x17d2, 0x178f, 0x17c5, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x200b, 0x1793, 0x17c5,
0x200b, 0x1780, 0x17b6, 0x179b, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a0, 0x17b6, 0x179c, 0x17c3, 0x2d, 0x17a2, 0x17b6, 0x179b, 0x17cb, 0x178a,
0x17d2, 0x1799, 0x17bc, 0x179f, 0x17b7, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f,
0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x200b, 0x1793, 0x17c5, 0x200b,
0x17a0, 0x17b6, 0x179c, 0x17c3, 0x2d, 0x17a2, 0x17b6, 0x179b, 0x17cb, 0x178a, 0x17d2, 0x1799,
0x17bc, 0x179f, 0x17b7, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b,
0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a0, 0x17b6, 0x179c,
0x17c3, 0x2d, 0x17a2, 0x17b6, 0x179b, 0x17cb, 0x178a, 0x17d2, 0x1799, 0x17bc, 0x179f, 0x17b7,
0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a0, 0x17bb, 0x1784,
0x1780, 0x17bb, 0x1784, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784,
0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a0, 0x17bb, 0x1784, 0x1780,
0x17bb, 0x1784, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x200b, 0x17a0, 0x17bb, 0x1784,
0x1780, 0x17bb, 0x1784, 0x1793, 0x17b6, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780,
0x17d2, 0x178a, 0x17c5, 0x200b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b,
0x17a0, 0x17bc, 0x179c, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784,
0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x17a0, 0x17bc, 0x179c, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x1793, 0x17c5, 0x200b, 0x17a0, 0x17bc, 0x179c, 0x1793, 0x17b6, 0x200b, 0x179a,
0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x200b, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x1793, 0x17c5,
0x200b, 0x17a5, 0x178e, 0x17d2, 0x178c, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x1798, 0x17a0, 0x17b6, 0x179f, 0x1798, 0x17bb, 0x1791, 0x17d2, 0x179a, 0x200b,
0x17a5, 0x178e, 0x17d2, 0x178c, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x200b, 0x17a5, 0x178e, 0x17d2, 0x178c, 0x17bc, 0x1785, 0x17b7, 0x1793, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a5, 0x178e, 0x17d2, 0x178c, 0x17bc, 0x178e, 0x17c1,
0x179f, 0x17ca, 0x17b8, 0x200b, 0x200b, 0x200b, 0x1780, 0x178e, 0x17d2, 0x178a, 0x17b6, 0x179b,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a5, 0x178e, 0x17d2, 0x178c,
0x17bc, 0x178e, 0x17c1, 0x179f, 0x17ca, 0x17b8, 0x200b, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b,
0x1780, 0x17be, 0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a5,
0x178e, 0x17d2, 0x178c, 0x17bc, 0x178e, 0x17c1, 0x179f, 0x17ca, 0x17b8, 0x200b, 0x200b, 0x1781,
0x17b6, 0x1784, 0x200b, 0x179b, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17b8, 0x179a, 0x17c9, 0x1784, 0x17cb, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17b8, 0x179a, 0x17c9, 0x1784, 0x17cb, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17b8, 0x179a, 0x17c9, 0x1784, 0x17cb, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x1793, 0x17c5, 0x17a2, 0x17c0, 0x179a, 0x1782, 0x17bb, 0x178f, 0x179f, 0x17d2, 0x1780,
0x17b7, 0x17cd, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a,
0x17b6, 0x179a, 0x1793, 0x17c5, 0x17a2, 0x17c0, 0x179a, 0x1782, 0x17bb, 0x178f, 0x179f, 0x17d2,
0x1780, 0x17b7, 0x17cd, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x17a2, 0x17c0, 0x179a,
0x1782, 0x17bb, 0x178f, 0x179f, 0x17d2, 0x1780, 0x17b7, 0x17cd, 0x1793, 0x17b6, 0x179a, 0x178a,
0x17bc, 0x179c, 0x1780, 0x17d2, 0x178f, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17b8, 0x179f, 0x17d2, 0x179a, 0x17b6, 0x17a2, 0x17c2, 0x179b,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6,
0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17b8, 0x179f, 0x17d2, 0x179a, 0x17b6,
0x17a2, 0x17c2, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b,
0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17b8, 0x179f,
0x17d2, 0x179a, 0x17b6, 0x17a2, 0x17c2, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x1787, 0x1794, 0x17c9, 0x17bb, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b,
0x1787, 0x1794, 0x17c9, 0x17bb, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1,
0x179b, 0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x1787, 0x1794, 0x17c9,
0x17bb, 0x1793, 0x1796, 0x17c1, 0x179b, 0x179c, 0x17c1, 0x179b, 0x17b6, 0x1793, 0x17c5, 0x1780,
0x17b6, 0x17a0, 0x17d2, 0x179f, 0x17b6, 0x1780, 0x17cb, 0x179f, 0x17d2, 0x1790, 0x17b6, 0x1793,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1780, 0x17b6, 0x17a0, 0x17d2, 0x179f, 0x17b6, 0x1780,
0x17cb, 0x179f, 0x17d2, 0x1790, 0x17b6, 0x1793, 0x200b, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b,
0x1780, 0x17be, 0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780,
0x17b6, 0x17a0, 0x17d2, 0x179f, 0x17b6, 0x1780, 0x17cb, 0x179f, 0x17d2, 0x1790, 0x17b6, 0x1793,
0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x200b, 0x200b, 0x179b, 0x17b7, 0x1785, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17bc, 0x179a, 0x17c9, 0x17c1, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a,
0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17bc, 0x179a, 0x17c9, 0x17c1, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793,
0x17c5, 0x200b, 0x1780, 0x17bc, 0x179a, 0x17c9, 0x17c1, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x1793, 0x17c5, 0x200b, 0x1780, 0x17bc, 0x179f, 0x17d2, 0x179a, 0x17c3, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17d2, 0x179a, 0x17b6, 0x178e, 0x17bc, 0x1799,
0x17c9, 0x17b6, 0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784,
0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17d2, 0x179a, 0x17b6,
0x178e, 0x17bc, 0x1799, 0x17c9, 0x17b6, 0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x1780, 0x17d2, 0x179a, 0x17b6, 0x178e, 0x17bc, 0x1799, 0x17c9, 0x17b6, 0x179f,
0x1793, 0x17b6, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x1780, 0x17c0, 0x17a0, 0x17d2, 0x179f, 0x17ca,
0x17b8, 0x179f, 0x17ca, 0x17b8, 0x179f, 0x17d2, 0x1790, 0x17b6, 0x1793, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x17a1, 0x17b6, 0x1789,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a1, 0x178f, 0x17a0, 0x17c5,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6,
0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a1, 0x178f, 0x200b, 0x17a0, 0x17c5, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b,
0x1793, 0x17c5, 0x200b, 0x17a1, 0x178f, 0x17a0, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x1793, 0x17c5, 0x200b, 0x1798, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x1780, 0x17b6, 0x178a, 0x17b6,
0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a,
0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1798, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x1780,
0x17b6, 0x178a, 0x17b6, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b,
0x1798, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x1780, 0x17b6, 0x178a, 0x17b6, 0x1793, 0x200b, 0x1793,
0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1798, 0x17c9, 0x17b6, 0x17a1, 0x17c1, 0x179f,
0x17ca, 0x17b8, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1798, 0x17c9,
0x17b6, 0x179b, 0x17cb, 0x178c, 0x17b8, 0x179c, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x1798, 0x17c9, 0x17b6, 0x1782, 0x17ba, 0x179f,
0x17b6, 0x179f, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1798,
0x17c9, 0x17b6, 0x179f, 0x17b6, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x200b, 0x1798, 0x17c9, 0x17bc, 0x179a, 0x17b8, 0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b,
0x1798, 0x17c9, 0x17bc, 0x179a, 0x17b8, 0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b,
0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1793, 0x17c5, 0x200b,
0x1798, 0x17c9, 0x17bc, 0x179a, 0x17b8, 0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x1798, 0x17c9, 0x17c5, 0x200b, 0x179f, 0x17b6, 0x1793, 0x17cb, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17c9, 0x17b6, 0x179f, 0x17ca, 0x17b8,
0x17a0, 0x17d2, 0x179c, 0x17b7, 0x1780, 0x200b, 0x1798, 0x17c9, 0x17b7, 0x1780, 0x179f, 0x17ca,
0x17b7, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb,
0x178a, 0x17b6, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17c9, 0x17b6, 0x179f, 0x17ca, 0x17b8,
0x17a0, 0x17d2, 0x179c, 0x17b7, 0x1780, 0x200b, 0x1798, 0x17c9, 0x17b7, 0x1780, 0x179f, 0x17ca,
0x17b7, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790,
0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17c9, 0x17b6, 0x179f, 0x17ca,
0x17b8, 0x17a0, 0x17d2, 0x179c, 0x17b7, 0x1780, 0x200b, 0x1798, 0x17c9, 0x17b7, 0x1780, 0x179f,
0x17ca, 0x17b7, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2,
0x17ca, 0x17bc, 0x17a1, 0x17b6, 0x1793, 0x200b, 0x1794, 0x17b6, 0x1791, 0x17bc, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b,
0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17bc, 0x17a1, 0x17b6, 0x1793, 0x200b, 0x1794, 0x17b6,
0x1791, 0x17bc, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17bc,
0x17a1, 0x17b6, 0x1793, 0x200b, 0x1794, 0x17b6, 0x1791, 0x17bc, 0x1793, 0x17b6, 0x200b, 0x179a,
0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x200b, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1798, 0x17bc, 0x179f, 0x17d2, 0x1782, 0x17bc, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a,
0x200b, 0x1793, 0x17c5, 0x200b, 0x1798, 0x17bc, 0x179f, 0x17d2, 0x1782, 0x17bc, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1798, 0x17bc, 0x179f, 0x17d2, 0x1782, 0x17bc,
0x200b, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1798, 0x17b8, 0x1799, 0x17c9,
0x17b6, 0x1793, 0x17cb, 0x1798, 0x17c9, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x178e, 0x17bc, 0x179a, 0x17bc, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5,
0x1793, 0x17c1, 0x1794, 0x17c9, 0x17b6, 0x179b, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x1793, 0x17c5, 0x178e, 0x17bc, 0x179c, 0x17c9, 0x17c2, 0x179b, 0x1780, 0x17b6, 0x17a1, 0x17c1,
0x178a, 0x17bc, 0x1793, 0x17c0, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x179f, 0x17d2, 0x178f, 0x1784,
0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x178e, 0x17bc, 0x179c, 0x17c9, 0x17c2,
0x179b, 0x1780, 0x17b6, 0x17a1, 0x17c1, 0x178a, 0x17bc, 0x1793, 0x17c0, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1793, 0x17c5, 0x178e, 0x17bc, 0x179c, 0x17c9, 0x17c2, 0x179b, 0x1780, 0x17b6,
0x17a1, 0x17c1, 0x178a, 0x17bc, 0x1793, 0x17c0, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c,
0x1780, 0x17d2, 0x178f, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b,
0x1793, 0x17bc, 0x179c, 0x17c2, 0x179b, 0x179f, 0x17c1, 0x17a1, 0x1784, 0x17cb, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b,
0x1793, 0x17c5, 0x200b, 0x1793, 0x17bc, 0x179c, 0x17c2, 0x179b, 0x179f, 0x17c1, 0x17a1, 0x1784,
0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2,
0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1793, 0x17bc, 0x179c, 0x17c2, 0x179b, 0x179f,
0x17c1, 0x17a1, 0x1784, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x1793, 0x17c5,
0x1789, 0x17bc, 0x179c, 0x17a0, 0x17d2, 0x179c, 0x17c4, 0x1793, 0x17a1, 0x17c2, 0x1793, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6,
0x179a, 0x200b, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1789, 0x17bc, 0x179c, 0x17a0, 0x17d2, 0x179c,
0x17c4, 0x1793, 0x17a1, 0x17c2, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1,
0x179b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1789, 0x17bc, 0x179c,
0x17a0, 0x17d2, 0x179c, 0x17c4, 0x1793, 0x17a1, 0x17c2, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x1793, 0x17c5, 0x200b, 0x1793, 0x17b8, 0x179c, 0x17c9, 0x17c1, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x178e, 0x17d0, 0x179a, 0x17a0,
0x17d2, 0x179c, 0x1780, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178a,
0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x178e, 0x17d0, 0x179a,
0x17a0, 0x17d2, 0x179c, 0x1780, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x200b,
0x178e, 0x17d0, 0x179a, 0x17a0, 0x17d2, 0x179c, 0x1780, 0x17cb, 0x1793, 0x17b6, 0x200b, 0x179a,
0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x17a0, 0x17d2, 0x179c, 0x17ca, 0x17c2, 0x178e, 0x17b6, 0x1793, 0x17cb,
0x178a, 0x17bc, 0x200b, 0x178a, 0x17ba, 0x178e, 0x17bc, 0x179a, 0x17c9, 0x17bc, 0x1789, 0x17c9,
0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a,
0x17b6, 0x179a, 0x1793, 0x17c5, 0x17a0, 0x17d2, 0x179c, 0x17ca, 0x17c2, 0x178e, 0x17b6, 0x1793,
0x17cb, 0x178a, 0x17bc, 0x200b, 0x178a, 0x17ba, 0x178e, 0x17bc, 0x179a, 0x17c9, 0x17bc, 0x1789,
0x17c9, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x17a0, 0x17d2, 0x179c,
0x17ca, 0x17c2, 0x178e, 0x17b6, 0x1793, 0x17cb, 0x178a, 0x17bc, 0x200b, 0x178a, 0x17ba, 0x178e,
0x17bc, 0x179a, 0x17c9, 0x17bc, 0x1789, 0x17c9, 0x17b6, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc,
0x179c, 0x1780, 0x17d2, 0x178f, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x200b, 0x178e, 0x17bc, 0x179c, 0x17c9, 0x17bc, 0x179f, 0x17ca, 0x17b8, 0x1794, 0x17ca, 0x17b8,
0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a,
0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x178e, 0x17bc, 0x179c, 0x17c9, 0x17bc, 0x179f,
0x17ca, 0x17b8, 0x1794, 0x17ca, 0x17b8, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x178e, 0x17bc, 0x179c, 0x17c9, 0x17bc, 0x179f, 0x17ca, 0x17b8, 0x1794, 0x17ca,
0x17b8, 0x1780, 0x1793, 0x17b6, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2,
0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17bc,
0x1798, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a,
0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17bc, 0x1798, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17bc, 0x1798, 0x1793, 0x17b6, 0x200b, 0x179a,
0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17c9, 0x17b6, 0x1782, 0x17b8, 0x179f, 0x17d2, 0x1790,
0x17b6, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178a, 0x1784, 0x17cb,
0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17c9, 0x17b6, 0x1782, 0x17b8,
0x179f, 0x17d2, 0x1790, 0x17b6, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x200b,
0x1794, 0x17c9, 0x17b6, 0x1782, 0x17b8, 0x179f, 0x17d2, 0x1790, 0x17b6, 0x1793, 0x1793, 0x17b6,
0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x200b, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1795, 0x17b6, 0x17a1, 0x17c5, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x1794, 0x17c9, 0x17b6, 0x1796, 0x17bc, 0x17a2,
0x17b6, 0x179f, 0x17ca, 0x17b8, 0x20, 0x1793, 0x17bc, 0x179c, 0x17c2, 0x179b, 0x17a0, 0x17d2,
0x1782, 0x17b8, 0x178e, 0x17c1, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b,
0x1794, 0x17c9, 0x17b6, 0x179a, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x1799, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a,
0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17c9, 0x17b6, 0x179a, 0x17c9, 0x17b6, 0x17a0, 0x17d2,
0x1782, 0x17b6, 0x1799, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x1794, 0x17c9,
0x17b6, 0x179a, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x1799, 0x1793, 0x17b6, 0x200b,
0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17c9, 0x17c1, 0x179a, 0x17bc, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b,
0x1793, 0x17c5, 0x200b, 0x1794, 0x17c9, 0x17c1, 0x179a, 0x17bc, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17c9, 0x17c1, 0x179a, 0x17bc, 0x1793, 0x17b6,
0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a0, 0x17d2, 0x179c, 0x17b8, 0x179b, 0x17b8, 0x1796,
0x17b8, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb,
0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a0, 0x17d2, 0x179c, 0x17b8, 0x179b,
0x17b8, 0x1796, 0x17b8, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x1793, 0x17c5,
0x200b, 0x17a0, 0x17d2, 0x179c, 0x17b8, 0x179b, 0x17b8, 0x1796, 0x17b8, 0x1793, 0x1793, 0x17b6,
0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x1795, 0x17bc,
0x1793, 0x17b8, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x200b, 0x1793, 0x17c5,
0x179f, 0x1784, 0x17cb, 0x200b, 0x1796, 0x17d2, 0x1799, 0x17c2, 0x179a, 0x20, 0x1793, 0x17b7,
0x1784, 0x200b, 0x1798, 0x17b8, 0x1782, 0x17bb, 0x1799, 0x17a1, 0x17bb, 0x1784, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b,
0x1793, 0x17c5, 0x179f, 0x1784, 0x17cb, 0x200b, 0x1796, 0x17d2, 0x1799, 0x17c2, 0x179a, 0x20,
0x1793, 0x17b7, 0x1784, 0x200b, 0x1798, 0x17b8, 0x1782, 0x17bb, 0x1799, 0x17a1, 0x17bb, 0x1784,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2, 0x1784,
0x17c3, 0x200b, 0x1793, 0x17c5, 0x179f, 0x1784, 0x17cb, 0x200b, 0x1796, 0x17d2, 0x1799, 0x17c2,
0x179a, 0x20, 0x1793, 0x17b7, 0x1784, 0x200b, 0x1798, 0x17b8, 0x1782, 0x17bb, 0x1799, 0x17a1,
0x17bb, 0x1784, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1797, 0x17b8,
0x1783, 0x17ba, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794,
0x17c9, 0x17bc, 0x178e, 0x17b6, 0x1794, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5,
0x1796, 0x17d2, 0x1799, 0x17bb, 0x1784, 0x1799, 0x17c9, 0x17b6, 0x1784, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x1793, 0x17c5, 0x179a, 0x17c1, 0x17a2, 0x17ca, 0x17bb, 0x1799, 0x17c9, 0x17bb, 0x1784,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179a, 0x17c9, 0x17bc, 0x1792,
0x17ba, 0x179a, 0x17c9, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b,
0x179f, 0x17b6, 0x1780, 0x17cb, 0x1781, 0x17b6, 0x179b, 0x17b8, 0x1793, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793,
0x17c5, 0x200b, 0x179f, 0x17b6, 0x1780, 0x17cb, 0x1781, 0x17b6, 0x179b, 0x17b8, 0x1793, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179f, 0x17b6, 0x1780, 0x17cb, 0x1781,
0x17b6, 0x179b, 0x17b8, 0x1793, 0x1793, 0x17b6, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b,
0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b,
0x179f, 0x17b6, 0x1798, 0x17d0, 0x179a, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2,
0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x1793, 0x17c5, 0x200b, 0x179f, 0x17b6, 0x1798,
0x17d0, 0x179a, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179f, 0x17b6,
0x1798, 0x17d0, 0x179a, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x1780, 0x17d2, 0x178f,
0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179f, 0x17b8, 0x179f,
0x17d2, 0x17a0, 0x17c2, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b,
0x179f, 0x17b7, 0x1784, 0x17d2, 0x17a0, 0x1794, 0x17bc, 0x179a, 0x17b8, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x179f, 0x17bc, 0x17a1,
0x17bc, 0x1798, 0x17c9, 0x17bb, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x200b, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x17a0, 0x17d2, 0x179f, 0x1780, 0x17a0, 0x17d2, 0x179f,
0x17ca, 0x17b8, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179f, 0x17bc,
0x179a, 0x17b8, 0x178e, 0x17b6, 0x1798, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x200b, 0x179f, 0x17ca, 0x17b8, 0x17a2, 0x17bc, 0x179c, 0x17c9, 0x17b6, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x178f, 0x17b6, 0x17a0, 0x17b7, 0x1791, 0x17b8, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x178f, 0x17c3, 0x1794, 0x17c9, 0x17b7,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6,
0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x178f, 0x17c3, 0x1794, 0x17c9, 0x17b7, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b,
0x1793, 0x17c5, 0x200b, 0x178f, 0x17c3, 0x1794, 0x17c9, 0x17b7, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x1793, 0x17c5, 0x178f, 0x17b6, 0x1787, 0x17b8, 0x1782, 0x17b8, 0x179f, 0x17d2, 0x1790, 0x17b6,
0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x178f, 0x17bc, 0x1781,
0x17c1, 0x17a1, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x178f,
0x17bb, 0x1784, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f,
0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x200b, 0x1793, 0x17c5, 0x200b,
0x178f, 0x17bb, 0x1784, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x200b, 0x1793, 0x17c5, 0x200b, 0x178f, 0x17bb, 0x1784, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x1793,
0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1785, 0x17bc, 0x17a2, 0x17bb, 0x1780, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x178f, 0x17bd, 0x1780, 0x1798, 0x17c9,
0x17c1, 0x1793, 0x17b8, 0x179f, 0x17d2, 0x1790, 0x17b6, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5,
0x178f, 0x17bd, 0x1780, 0x1798, 0x17c9, 0x17c1, 0x1793, 0x17b8, 0x179f, 0x17d2, 0x1790, 0x17b6,
0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780,
0x17d2, 0x178a, 0x17c5, 0x200b, 0x1793, 0x17c5, 0x200b, 0x178f, 0x17bd, 0x1780, 0x1798, 0x17c9,
0x17c1, 0x1793, 0x17b8, 0x179f, 0x17d2, 0x1790, 0x17b6, 0x1793, 0x200b, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1791, 0x17bb, 0x1799, 0x179c, 0x17c9, 0x17b6, 0x179b,
0x17bc, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17bb,
0x1799, 0x179a, 0x17bc, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x1799, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5,
0x200b, 0x17a2, 0x17ca, 0x17bb, 0x1799, 0x179a, 0x17bc, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x1799,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17bb, 0x1799,
0x179a, 0x17bc, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x1799, 0x1793, 0x17b6, 0x200b, 0x200b, 0x179a,
0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17bc, 0x179f, 0x1794, 0x17c1, 0x1782, 0x17b8,
0x179f, 0x17d2, 0x1790, 0x17b6, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2,
0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ca,
0x17bc, 0x179f, 0x1794, 0x17c1, 0x1782, 0x17b8, 0x179f, 0x17d2, 0x1790, 0x17b6, 0x1793, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17bc, 0x179f, 0x1794,
0x17c1, 0x1782, 0x17b8, 0x179f, 0x17d2, 0x1790, 0x17b6, 0x1793, 0x1793, 0x17b6, 0x179a, 0x178a,
0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x200b, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x179c, 0x17c9, 0x17b6, 0x1793, 0x17bc, 0x1791, 0x17bc, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6,
0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179c, 0x17c9, 0x17b6, 0x1793, 0x17bc, 0x1791, 0x17bc,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179c, 0x17c9, 0x17b6, 0x1793,
0x17bc, 0x1791, 0x17bc, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2,
0x178a, 0x17c5, 0x200b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179c,
0x17c9, 0x17c1, 0x178e, 0x17c1, 0x179f, 0x17ca, 0x17bb, 0x1799, 0x17a2, 0x17c1, 0x17a1, 0x17b6,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179c, 0x17c9, 0x17d2, 0x179b,
0x17b6, 0x178c, 0x17b8, 0x179c, 0x17c9, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17bb, 0x1780, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a,
0x200b, 0x1793, 0x17c5, 0x200b, 0x179c, 0x17c9, 0x17d2, 0x179b, 0x17b6, 0x178c, 0x17b8, 0x179c,
0x17c9, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17bb, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x1793, 0x17c5, 0x200b, 0x179c, 0x17c9, 0x17d2, 0x179b, 0x17b6, 0x178c, 0x17b8, 0x179c, 0x17c9,
0x17bc, 0x179f, 0x17d2, 0x178f, 0x17bb, 0x1780, 0x1793, 0x17b6, 0x200b, 0x179a, 0x178a, 0x17bc,
0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x179c, 0x17c9, 0x17bc, 0x17a0, 0x17d2, 0x1780, 0x17c4, 0x1780, 0x17d2, 0x179a,
0x17b6, 0x178a, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb,
0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179c, 0x17c9, 0x17bc, 0x17a0, 0x17d2,
0x1780, 0x17c4, 0x1780, 0x17d2, 0x179a, 0x17b6, 0x178a, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x1793, 0x17c5, 0x200b, 0x179c, 0x17c9, 0x17bc, 0x17a0, 0x17d2, 0x1780, 0x17c4, 0x1780, 0x17d2,
0x179a, 0x17b6, 0x178a, 0x1793, 0x17b6, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780,
0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179c,
0x17d0, 0x179a, 0x179f, 0x17d2, 0x178f, 0x17bb, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x1793, 0x17c5, 0x200b, 0x1780, 0x17c4, 0x17c7, 0x179c, 0x17c1, 0x1780, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179c, 0x17c9, 0x17b6, 0x179b, 0x17b8, 0x179f, 0x20,
0x1793, 0x17b7, 0x1784, 0x17a0, 0x17d2, 0x179c, 0x17ca, 0x17bb, 0x1791, 0x17bc, 0x178e, 0x17b6,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1799, 0x17c9, 0x17b6, 0x1782,
0x17bb, 0x178f, 0x179f, 0x17d2, 0x1780, 0x17b7, 0x17cd, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b,
0x1799, 0x17c9, 0x17b6, 0x1782, 0x17bb, 0x178f, 0x179f, 0x17d2, 0x1780, 0x17b7, 0x17cd, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1799, 0x17c9, 0x17b6, 0x1782, 0x17bb,
0x178f, 0x179f, 0x17d2, 0x1780, 0x17b7, 0x17cd, 0x1793, 0x17b6, 0x200b, 0x179a, 0x178a, 0x17bc,
0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17b7, 0x1781, 0x17b6, 0x1792, 0x17ba, 0x179a, 0x17b8, 0x1793,
0x1794, 0x17ca, 0x17d0, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f,
0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17b7,
0x1781, 0x17b6, 0x1792, 0x17ba, 0x179a, 0x17b8, 0x1793, 0x1794, 0x17ca, 0x17d0, 0x1780, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17b7, 0x1781, 0x17b6,
0x1792, 0x17ba, 0x179a, 0x17b8, 0x1793, 0x1794, 0x17ca, 0x17d0, 0x1780, 0x1793, 0x17b6, 0x200b,
0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x1793, 0x17c5, 0x1799, 0x17bc, 0x1781, 0x1793, 0x17cb, 0x905, 0x92b, 0x917,
0x93e, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x935, 0x947, 0x933,
0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x93e,
0x20, 0x935, 0x947, 0x933, 0x909, 0x926, 0x947, 0x902, 0x924, 0x20, 0x906, 0x92b,
0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x926, 0x915, 0x94d,
0x937, 0x93f, 0x923, 0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933,
0x905, 0x938, 0x94d, 0x924, 0x902, 0x924, 0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f,
0x915, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x905, 0x938, 0x94d, 0x924, 0x902, 0x924,
0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x905, 0x938, 0x94d,
0x924, 0x902, 0x924, 0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x905,
0x932, 0x93e, 0x938, 0x94d, 0x915, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x905, 0x932,
0x93e, 0x938, 0x94d, 0x915, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x940, 0x924, 0x20, 0x935, 0x947, 0x933, 0x905, 0x932, 0x93e, 0x938, 0x94d, 0x915,
0x93e, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933,
0x905, 0x92e, 0x947, 0x91d, 0x94b, 0x928, 0x20, 0x935, 0x947, 0x933, 0x905, 0x92e,
0x947, 0x91d, 0x94b, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f,
0x924, 0x20, 0x935, 0x947, 0x933, 0x905, 0x92e, 0x947, 0x91d, 0x94b, 0x928, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x92e,
0x927, 0x94d, 0x92f, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x927, 0x94d, 0x92f, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933,
0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20,
0x935, 0x947, 0x933, 0x909, 0x926, 0x947, 0x902, 0x924, 0x20, 0x935, 0x947, 0x933,
0x909, 0x926, 0x947, 0x902, 0x924, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x909, 0x926, 0x947, 0x902, 0x924, 0x20,
0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x930,
0x94d, 0x935, 0x924, 0x940, 0x92f, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x930, 0x94d,
0x935, 0x924, 0x940, 0x92f, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f,
0x924, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x930, 0x94d, 0x935, 0x924, 0x940, 0x92f,
0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x92a,
0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x94d,
0x930, 0x936, 0x93e, 0x902, 0x924, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902,
0x924, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933,
0x905, 0x92a, 0x93f, 0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x905, 0x92a, 0x93f,
0x92f, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20,
0x935, 0x947, 0x933, 0x905, 0x92a, 0x93f, 0x92f, 0x93e, 0x20, 0x921, 0x947, 0x932,
0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x905, 0x930, 0x92c, 0x940, 0x20,
0x935, 0x947, 0x933, 0x905, 0x930, 0x92c, 0x940, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x905, 0x930, 0x92c, 0x940,
0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x905,
0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x93f, 0x928, 0x93e, 0x20, 0x935, 0x947,
0x933, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x93f, 0x928, 0x93e, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933,
0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x93f, 0x928, 0x93e, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x905, 0x938,
0x94d, 0x924, 0x902, 0x924, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f,
0x93f, 0x928, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x905, 0x938, 0x94d, 0x924, 0x902,
0x924, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x93f, 0x928, 0x93e,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947,
0x933, 0x905, 0x938, 0x94d, 0x924, 0x902, 0x924, 0x20, 0x905, 0x930, 0x94d, 0x91c,
0x947, 0x902, 0x91f, 0x93f, 0x928, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x906, 0x930, 0x94d, 0x92e, 0x947, 0x928,
0x93f, 0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x906, 0x930, 0x94d, 0x92e, 0x947,
0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f,
0x924, 0x20, 0x935, 0x947, 0x933, 0x906, 0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f,
0x92f, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935,
0x947, 0x933, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x940, 0x915, 0x20, 0x935,
0x947, 0x933, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x940, 0x915, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x905,
0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x940, 0x915, 0x20, 0x921, 0x947, 0x932, 0x93e,
0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x911,
0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x940, 0x92f, 0x93e, 0x20, 0x935,
0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x940, 0x92f,
0x928, 0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d,
0x930, 0x947, 0x932, 0x940, 0x92f, 0x928, 0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x20,
0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x911, 0x938,
0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x940, 0x92f, 0x928, 0x20, 0x92e, 0x927,
0x94d, 0x92f, 0x20, 0x905, 0x938, 0x94d, 0x924, 0x902, 0x924, 0x20, 0x935, 0x947,
0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x940, 0x92f, 0x928,
0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x905, 0x938, 0x94d, 0x924, 0x902, 0x924,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947,
0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x940, 0x92f, 0x928,
0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x905, 0x938, 0x94d, 0x924, 0x902, 0x924,
0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x909,
0x926, 0x947, 0x902, 0x924, 0x20, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947,
0x932, 0x940, 0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f,
0x94d, 0x930, 0x947, 0x932, 0x940, 0x92f, 0x928, 0x20, 0x909, 0x926, 0x947, 0x902,
0x924, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935,
0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x940, 0x92f,
0x928, 0x20, 0x909, 0x926, 0x947, 0x902, 0x924, 0x20, 0x921, 0x947, 0x932, 0x93e,
0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x905, 0x938, 0x94d, 0x924, 0x902, 0x924,
0x20, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x940, 0x92f, 0x93e,
0x20, 0x935, 0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932,
0x940, 0x92f, 0x928, 0x20, 0x905, 0x938, 0x94d, 0x924, 0x902, 0x924, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x911,
0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x940, 0x92f, 0x928, 0x20, 0x905,
0x938, 0x94d, 0x924, 0x902, 0x924, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f,
0x20, 0x935, 0x947, 0x933, 0x905, 0x91c, 0x930, 0x92c, 0x948, 0x91c, 0x93e, 0x928,
0x20, 0x935, 0x947, 0x933, 0x905, 0x91c, 0x930, 0x92c, 0x948, 0x91c, 0x93e, 0x928,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947,
0x933, 0x905, 0x91c, 0x930, 0x92c, 0x948, 0x91c, 0x93e, 0x928, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x905, 0x91d, 0x94b,
0x930, 0x947, 0x938, 0x905, 0x91d, 0x94b, 0x930, 0x947, 0x938, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x905, 0x91d,
0x94b, 0x930, 0x947, 0x938, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x20, 0x935, 0x947, 0x933, 0x92c, 0x93e, 0x902, 0x917, 0x932, 0x93e, 0x926, 0x947,
0x936, 0x20, 0x935, 0x947, 0x933, 0x92c, 0x93e, 0x902, 0x917, 0x932, 0x93e, 0x926,
0x947, 0x936, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20,
0x935, 0x947, 0x933, 0x92c, 0x93e, 0x902, 0x917, 0x932, 0x93e, 0x926, 0x947, 0x936,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933,
0x92d, 0x942, 0x924, 0x93e, 0x928, 0x92c, 0x94b, 0x932, 0x93f, 0x935, 0x94d, 0x939,
0x93f, 0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x92c, 0x94d, 0x930, 0x93e, 0x91d,
0x93f, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x92c, 0x94d, 0x930,
0x93e, 0x91d, 0x93f, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92c, 0x94d, 0x930, 0x93e,
0x91d, 0x93f, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x92c, 0x94d, 0x930, 0x941, 0x928, 0x947,
0x907, 0x20, 0x926, 0x93e, 0x930, 0x942, 0x938, 0x94d, 0x938, 0x932, 0x93e, 0x92e,
0x20, 0x935, 0x947, 0x933, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x930, 0x94d, 0x926,
0x947, 0x20, 0x935, 0x947, 0x933, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x930, 0x94d,
0x926, 0x947, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20,
0x935, 0x947, 0x933, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x930, 0x94d, 0x926, 0x947,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933,
0x915, 0x945, 0x92e, 0x94b, 0x930, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x945, 0x925, 0x92e, 0x20,
0x935, 0x947, 0x933, 0x91a, 0x945, 0x925, 0x92e, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x945, 0x925, 0x92e,
0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x91a,
0x93f, 0x932, 0x940, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x93f, 0x932, 0x940, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933,
0x91a, 0x93f, 0x932, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x20, 0x935, 0x947, 0x933, 0x91a, 0x940, 0x928, 0x20, 0x935, 0x947, 0x933, 0x91a,
0x940, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20,
0x935, 0x947, 0x933, 0x91a, 0x940, 0x928, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f,
0x91f, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94d, 0x930, 0x93f, 0x938, 0x92e, 0x938,
0x20, 0x906, 0x92f, 0x932, 0x901, 0x921, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94b,
0x915, 0x94b, 0x938, 0x20, 0x906, 0x92f, 0x932, 0x901, 0x921, 0x20, 0x935, 0x947,
0x933, 0x915, 0x94b, 0x932, 0x902, 0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x935, 0x947,
0x933, 0x915, 0x94b, 0x932, 0x902, 0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94b,
0x932, 0x902, 0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x915, 0x942, 0x915, 0x20, 0x906, 0x92f,
0x932, 0x901, 0x921, 0x20, 0x935, 0x947, 0x933, 0x915, 0x942, 0x915, 0x20, 0x906,
0x92f, 0x932, 0x901, 0x921, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f,
0x924, 0x20, 0x935, 0x947, 0x933, 0x915, 0x942, 0x915, 0x20, 0x906, 0x92f, 0x932,
0x901, 0x921, 0x20, 0x905, 0x930, 0x94d, 0x926, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94d, 0x92f, 0x941, 0x92c,
0x93e, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94d, 0x92f, 0x941, 0x92c, 0x93e, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x940, 0x924, 0x20, 0x935, 0x947, 0x933,
0x915, 0x94d, 0x92f, 0x941, 0x92c, 0x93e, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f,
0x91f, 0x20, 0x935, 0x947, 0x933, 0x921, 0x947, 0x935, 0x94d, 0x939, 0x940, 0x938,
0x20, 0x935, 0x947, 0x933, 0x926, 0x94d, 0x92f, 0x941, 0x92e, 0x93e, 0x901, 0x20,
0x926, 0x942, 0x930, 0x935, 0x940, 0x92f, 0x20, 0x935, 0x947, 0x933, 0x909, 0x926,
0x947, 0x902, 0x924, 0x20, 0x924, 0x93f, 0x92e, 0x94b, 0x930, 0x20, 0x935, 0x947,
0x933, 0x908, 0x938, 0x94d, 0x91f, 0x930, 0x20, 0x906, 0x92f, 0x932, 0x901, 0x921,
0x20, 0x935, 0x947, 0x933, 0x908, 0x938, 0x94d, 0x91f, 0x930, 0x20, 0x906, 0x92f,
0x932, 0x901, 0x921, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924,
0x20, 0x935, 0x947, 0x933, 0x908, 0x938, 0x94d, 0x91f, 0x930, 0x20, 0x906, 0x92f,
0x932, 0x901, 0x921, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20,
0x935, 0x947, 0x933, 0x907, 0x915, 0x94d, 0x935, 0x947, 0x921, 0x94b, 0x930, 0x20,
0x935, 0x947, 0x933, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x92f, 0x941, 0x930, 0x94b,
0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x927, 0x94d, 0x92f,
0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x927,
0x94d, 0x92f, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x909,
0x926, 0x947, 0x902, 0x924, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f,
0x928, 0x20, 0x935, 0x947, 0x933, 0x909, 0x926, 0x947, 0x902, 0x924, 0x20, 0x92f,
0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x909, 0x926, 0x947, 0x902,
0x924, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x906, 0x928,
0x940, 0x915, 0x2d, 0x909, 0x926, 0x947, 0x902, 0x924, 0x20, 0x92f, 0x941, 0x930,
0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x935, 0x947, 0x933, 0x905, 0x938, 0x94d,
0x924, 0x902, 0x924, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928,
0x20, 0x935, 0x947, 0x933, 0x905, 0x938, 0x94d, 0x924, 0x902, 0x924, 0x20, 0x92f,
0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x905, 0x938, 0x94d, 0x924,
0x902, 0x924, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x92b,
0x949, 0x915, 0x932, 0x901, 0x921, 0x20, 0x906, 0x92f, 0x932, 0x901, 0x921, 0x94d,
0x938, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x949, 0x915, 0x932, 0x901, 0x921, 0x20,
0x906, 0x92f, 0x932, 0x901, 0x921, 0x94d, 0x938, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x949, 0x915, 0x932,
0x901, 0x921, 0x20, 0x906, 0x92f, 0x932, 0x901, 0x921, 0x94d, 0x938, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x93f,
0x91c, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x93f, 0x91c, 0x940, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92b,
0x93f, 0x91c, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20,
0x935, 0x947, 0x933, 0x92b, 0x94d, 0x930, 0x947, 0x902, 0x91a, 0x20, 0x917, 0x92f,
0x93e, 0x928, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x94d, 0x930, 0x947, 0x928,
0x94d, 0x91a, 0x20, 0x926, 0x915, 0x94d, 0x937, 0x93f, 0x923, 0x20, 0x906, 0x928,
0x940, 0x20, 0x905, 0x902, 0x91f, 0x93e, 0x930, 0x94d, 0x915, 0x94d, 0x91f, 0x93f,
0x915, 0x20, 0x935, 0x947, 0x933, 0x917, 0x93e, 0x932, 0x93e, 0x92a, 0x93e, 0x917,
0x94b, 0x938, 0x20, 0x935, 0x947, 0x933, 0x917, 0x93e, 0x92e, 0x94d, 0x92c, 0x93f,
0x92f, 0x930, 0x20, 0x935, 0x947, 0x933, 0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f,
0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f,
0x92f, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20,
0x935, 0x947, 0x933, 0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x917,
0x93f, 0x932, 0x94d, 0x92c, 0x930, 0x94d, 0x91f, 0x20, 0x906, 0x92f, 0x932, 0x901,
0x921, 0x20, 0x935, 0x947, 0x933, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x935, 0x93f,
0x91a, 0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x935, 0x947, 0x933, 0x909, 0x926,
0x947, 0x902, 0x924, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x901, 0x921,
0x20, 0x935, 0x947, 0x933, 0x909, 0x926, 0x947, 0x902, 0x924, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x928, 0x932, 0x901, 0x921, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x940, 0x924, 0x20, 0x935, 0x947, 0x933, 0x909, 0x926, 0x947, 0x902, 0x924,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x901, 0x921, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x905, 0x938, 0x94d,
0x924, 0x902, 0x924, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x901, 0x921,
0x20, 0x935, 0x947, 0x933, 0x905, 0x938, 0x94d, 0x924, 0x902, 0x924, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x928, 0x932, 0x901, 0x921, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x940, 0x924, 0x20, 0x935, 0x947, 0x933, 0x905, 0x938, 0x94d, 0x924,
0x902, 0x924, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x901, 0x921, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x917,
0x932, 0x94d, 0x92b, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924,
0x20, 0x935, 0x947, 0x933, 0x917, 0x941, 0x92f, 0x93e, 0x928, 0x93e, 0x20, 0x935,
0x947, 0x933, 0x939, 0x935, 0x93e, 0x908, 0x2d, 0x905, 0x932, 0x947, 0x92f, 0x941,
0x936, 0x93f, 0x928, 0x20, 0x935, 0x947, 0x933, 0x939, 0x935, 0x93e, 0x908, 0x2d,
0x905, 0x932, 0x947, 0x92f, 0x941, 0x936, 0x93f, 0x928, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x940, 0x924, 0x20, 0x935, 0x947, 0x933, 0x939, 0x935, 0x93e,
0x908, 0x2d, 0x905, 0x932, 0x947, 0x92f, 0x941, 0x936, 0x93f, 0x928, 0x20, 0x921,
0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x939, 0x93e, 0x901,
0x917, 0x20, 0x915, 0x93e, 0x901, 0x917, 0x20, 0x935, 0x947, 0x933, 0x939, 0x93e,
0x901, 0x917, 0x20, 0x915, 0x93e, 0x901, 0x917, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x939, 0x93e, 0x901, 0x917,
0x20, 0x915, 0x93e, 0x901, 0x917, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x20, 0x935, 0x947, 0x933, 0x939, 0x94b, 0x935, 0x94d, 0x939, 0x921, 0x939,
0x94b, 0x935, 0x94d, 0x939, 0x921, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x939, 0x94b, 0x935, 0x94d, 0x939, 0x921,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933,
0x92d, 0x93e, 0x930, 0x924, 0x940, 0x92f, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x939, 0x93f, 0x902, 0x926, 0x20,
0x92e, 0x94d, 0x939, 0x93e, 0x938, 0x93e, 0x917, 0x930, 0x20, 0x935, 0x947, 0x933,
0x907, 0x902, 0x921, 0x94b, 0x91a, 0x940, 0x928, 0x20, 0x935, 0x947, 0x933, 0x92e,
0x927, 0x94d, 0x92f, 0x20, 0x907, 0x902, 0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f,
0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x909, 0x926, 0x947, 0x902, 0x924, 0x20,
0x907, 0x902, 0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x935,
0x947, 0x933, 0x905, 0x938, 0x94d, 0x924, 0x902, 0x924, 0x20, 0x907, 0x902, 0x921,
0x94b, 0x928, 0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x907,
0x930, 0x93e, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924,
0x20, 0x935, 0x947, 0x933, 0x907, 0x930, 0x93e, 0x928, 0x20, 0x921, 0x947, 0x932,
0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x907, 0x930, 0x94d, 0x915, 0x941,
0x91f, 0x938, 0x94d, 0x915, 0x20, 0x935, 0x947, 0x933, 0x908, 0x930, 0x94d, 0x915,
0x941, 0x91f, 0x938, 0x94d, 0x915, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x908, 0x930, 0x94d, 0x915, 0x941, 0x91f,
0x938, 0x94d, 0x915, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20,
0x935, 0x947, 0x933, 0x907, 0x91c, 0x93c, 0x930, 0x93e, 0x907, 0x932, 0x20, 0x935,
0x947, 0x933, 0x907, 0x91c, 0x93c, 0x930, 0x93e, 0x907, 0x932, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x907, 0x91c,
0x93c, 0x930, 0x93e, 0x907, 0x932, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f,
0x20, 0x935, 0x947, 0x933, 0x91c, 0x92a, 0x93e, 0x928, 0x20, 0x935, 0x947, 0x933,
0x91c, 0x92a, 0x93e, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f,
0x924, 0x20, 0x935, 0x947, 0x933, 0x91c, 0x92a, 0x93e, 0x928, 0x20, 0x921, 0x947,
0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x915, 0x91d, 0x93e, 0x916,
0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x935, 0x947, 0x933, 0x909, 0x926, 0x947,
0x902, 0x924, 0x20, 0x915, 0x91d, 0x93e, 0x915, 0x938, 0x94d, 0x924, 0x93e, 0x928,
0x20, 0x935, 0x947, 0x933, 0x905, 0x938, 0x94d, 0x924, 0x902, 0x924, 0x20, 0x915,
0x91d, 0x93e, 0x915, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x935, 0x947, 0x933,
0x915, 0x94b, 0x930, 0x93f, 0x92f, 0x928, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94b,
0x930, 0x93f, 0x92f, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f,
0x924, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94b, 0x930, 0x93f, 0x92f, 0x928, 0x20,
0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94b,
0x938, 0x930, 0x947, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94d, 0x930, 0x93e, 0x938,
0x94d, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x935,
0x947, 0x933, 0x915, 0x94d, 0x930, 0x93e, 0x938, 0x94d, 0x928, 0x94b, 0x92f, 0x93e,
0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94d, 0x930, 0x93e, 0x938, 0x94d,
0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x915, 0x93f, 0x930,
0x94d, 0x917, 0x93f, 0x91c, 0x93c, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x935,
0x947, 0x933, 0x932, 0x93e, 0x92f, 0x928, 0x20, 0x906, 0x92f, 0x932, 0x901, 0x921,
0x20, 0x935, 0x947, 0x933, 0x932, 0x949, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x94b,
0x935, 0x947, 0x20, 0x935, 0x947, 0x933, 0x932, 0x949, 0x930, 0x94d, 0x921, 0x20,
0x939, 0x94b, 0x935, 0x947, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f,
0x924, 0x20, 0x935, 0x947, 0x933, 0x932, 0x949, 0x930, 0x94d, 0x921, 0x20, 0x939,
0x94b, 0x935, 0x947, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935,
0x947, 0x933, 0x92e, 0x917, 0x93e, 0x926, 0x93e, 0x928, 0x20, 0x935, 0x947, 0x933,
0x92e, 0x917, 0x93e, 0x926, 0x93e, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x917, 0x93e, 0x926, 0x93e,
0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947,
0x933, 0x92e, 0x932, 0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933,
0x92e, 0x93e, 0x932, 0x926, 0x940, 0x935, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x93e,
0x930, 0x94d, 0x915, 0x93f, 0x938, 0x938, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x93e,
0x930, 0x94d, 0x936, 0x932, 0x20, 0x906, 0x92f, 0x932, 0x901, 0x921, 0x20, 0x935,
0x947, 0x933, 0x92e, 0x949, 0x930, 0x93f, 0x936, 0x938, 0x20, 0x935, 0x947, 0x933,
0x92e, 0x949, 0x930, 0x93f, 0x936, 0x938, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x949, 0x930, 0x93f, 0x936,
0x938, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947,
0x933, 0x92e, 0x949, 0x938, 0x928, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x947, 0x915,
0x94d, 0x938, 0x93f, 0x915, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902,
0x924, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915,
0x928, 0x20, 0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x940, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x947,
0x915, 0x94d, 0x938, 0x93f, 0x915, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x936, 0x93e,
0x902, 0x924, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947,
0x933, 0x909, 0x932, 0x93e, 0x928, 0x92c, 0x924, 0x93e, 0x930, 0x20, 0x935, 0x947,
0x933, 0x909, 0x932, 0x93e, 0x928, 0x92c, 0x924, 0x93e, 0x930, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x909, 0x932,
0x93e, 0x928, 0x92c, 0x924, 0x93e, 0x930, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x949, 0x938, 0x94d, 0x915, 0x94b,
0x20, 0x935, 0x947, 0x933, 0x92e, 0x949, 0x938, 0x94d, 0x915, 0x94b, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92e,
0x949, 0x938, 0x94d, 0x915, 0x94b, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x94d, 0x92f, 0x93e, 0x928, 0x92e, 0x93e,
0x930, 0x20, 0x935, 0x947, 0x933, 0x928, 0x930, 0x942, 0x20, 0x935, 0x947, 0x933,
0x928, 0x947, 0x92a, 0x93e, 0x933, 0x20, 0x935, 0x947, 0x933, 0x928, 0x94d, 0x92f,
0x941, 0x20, 0x915, 0x945, 0x932, 0x947, 0x921, 0x94b, 0x928, 0x93f, 0x92f, 0x93e,
0x20, 0x935, 0x947, 0x933, 0x928, 0x94d, 0x92f, 0x941, 0x20, 0x915, 0x945, 0x932,
0x947, 0x921, 0x94b, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x928, 0x94d, 0x92f, 0x941,
0x20, 0x915, 0x945, 0x932, 0x947, 0x921, 0x94b, 0x928, 0x93f, 0x92f, 0x93e, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x928,
0x94d, 0x92f, 0x941, 0x91d, 0x940, 0x932, 0x945, 0x928, 0x94d, 0x921, 0x20, 0x935,
0x947, 0x933, 0x928, 0x94d, 0x92f, 0x941, 0x91d, 0x940, 0x932, 0x945, 0x928, 0x94d,
0x921, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935,
0x947, 0x933, 0x928, 0x94d, 0x92f, 0x941, 0x91d, 0x940, 0x932, 0x945, 0x928, 0x94d,
0x921, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933,
0x928, 0x94d, 0x92f, 0x941, 0x92b, 0x93e, 0x935, 0x902, 0x921, 0x932, 0x901, 0x921,
0x20, 0x935, 0x947, 0x933, 0x928, 0x94d, 0x92f, 0x941, 0x92b, 0x93e, 0x935, 0x902,
0x921, 0x932, 0x901, 0x921, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x940,
0x924, 0x20, 0x935, 0x947, 0x933, 0x928, 0x94d, 0x92f, 0x941, 0x92b, 0x93e, 0x935,
0x902, 0x921, 0x932, 0x901, 0x921, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f,
0x20, 0x935, 0x947, 0x933, 0x928, 0x94d, 0x92f, 0x941, 0x90f, 0x20, 0x935, 0x947,
0x933, 0x928, 0x949, 0x930, 0x92b, 0x949, 0x915, 0x20, 0x906, 0x92f, 0x932, 0x901,
0x921, 0x20, 0x935, 0x947, 0x933, 0x928, 0x949, 0x930, 0x92b, 0x949, 0x915, 0x20,
0x906, 0x92f, 0x932, 0x901, 0x921, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x928, 0x949, 0x930, 0x92b, 0x949, 0x915,
0x20, 0x906, 0x92f, 0x932, 0x901, 0x921, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x930, 0x94d, 0x928, 0x93e, 0x902,
0x921, 0x94b, 0x20, 0x926, 0x940, 0x20, 0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d,
0x939, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x930, 0x94d, 0x928, 0x93e, 0x902,
0x921, 0x94b, 0x20, 0x926, 0x940, 0x20, 0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d,
0x939, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20,
0x935, 0x947, 0x933, 0x92b, 0x930, 0x94d, 0x928, 0x93e, 0x902, 0x921, 0x94b, 0x20,
0x926, 0x940, 0x20, 0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x928,
0x94b, 0x935, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947,
0x933, 0x928, 0x94b, 0x935, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938,
0x94d, 0x915, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935,
0x947, 0x933, 0x913, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x935, 0x947, 0x933,
0x913, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x913, 0x92e, 0x94d, 0x938, 0x94d,
0x915, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947,
0x933, 0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92a,
0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x932, 0x93e, 0x90a,
0x20, 0x935, 0x947, 0x933, 0x92a, 0x93e, 0x92a, 0x941, 0x906, 0x20, 0x928, 0x94d,
0x92f, 0x941, 0x20, 0x917, 0x93f, 0x928, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92a,
0x930, 0x93e, 0x917, 0x941, 0x906, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x930, 0x93e,
0x917, 0x941, 0x906, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924,
0x20, 0x935, 0x947, 0x933, 0x92a, 0x930, 0x93e, 0x917, 0x941, 0x906, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x947,
0x930, 0x942, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x947, 0x930, 0x942, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92a,
0x947, 0x930, 0x942, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20,
0x935, 0x947, 0x933, 0x92b, 0x93f, 0x932, 0x93f, 0x92a, 0x93e, 0x92f, 0x928, 0x20,
0x935, 0x947, 0x933, 0x92b, 0x93f, 0x932, 0x93f, 0x92a, 0x93e, 0x92f, 0x928, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933,
0x92b, 0x93f, 0x932, 0x93f, 0x92a, 0x93e, 0x92f, 0x928, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x93f, 0x928, 0x93f,
0x915, 0x94d, 0x938, 0x20, 0x906, 0x92f, 0x932, 0x901, 0x921, 0x20, 0x935, 0x947,
0x933, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x92a, 0x93f, 0x92f, 0x930, 0x20, 0x906,
0x928, 0x940, 0x20, 0x92e, 0x93f, 0x915, 0x932, 0x93e, 0x928, 0x20, 0x935, 0x947,
0x933, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x92a, 0x93f, 0x92f, 0x930, 0x20, 0x906,
0x928, 0x940, 0x20, 0x92e, 0x93f, 0x915, 0x932, 0x93e, 0x928, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x940, 0x924, 0x20, 0x935, 0x947, 0x933, 0x938, 0x947,
0x902, 0x91f, 0x20, 0x92a, 0x93f, 0x92f, 0x930, 0x20, 0x906, 0x928, 0x940, 0x20,
0x92e, 0x93f, 0x915, 0x932, 0x93e, 0x928, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f,
0x91f, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x93f, 0x91f, 0x915, 0x945, 0x930, 0x928,
0x20, 0x935, 0x947, 0x933, 0x92a, 0x94b, 0x928, 0x947, 0x92a, 0x20, 0x935, 0x947,
0x933, 0x92a, 0x94d, 0x92f, 0x94b, 0x902, 0x917, 0x92f, 0x93e, 0x902, 0x917, 0x20,
0x935, 0x947, 0x933, 0x930, 0x93f, 0x92f, 0x941, 0x928, 0x93f, 0x92f, 0x928, 0x20,
0x935, 0x947, 0x933, 0x930, 0x94b, 0x925, 0x947, 0x930, 0x93e, 0x20, 0x935, 0x947,
0x933, 0x938, 0x916, 0x932, 0x93f, 0x928, 0x20, 0x935, 0x947, 0x933, 0x938, 0x916,
0x932, 0x93f, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924,
0x20, 0x935, 0x947, 0x933, 0x938, 0x916, 0x932, 0x93f, 0x928, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x938, 0x93e, 0x92e,
0x94b, 0x906, 0x20, 0x935, 0x947, 0x933, 0x938, 0x93e, 0x92e, 0x94b, 0x906, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933,
0x938, 0x93e, 0x92e, 0x94b, 0x906, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f,
0x20, 0x935, 0x947, 0x933, 0x938, 0x947, 0x936, 0x947, 0x932, 0x94d, 0x938, 0x20,
0x935, 0x947, 0x933, 0x938, 0x93f, 0x902, 0x917, 0x93e, 0x92a, 0x942, 0x930, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933,
0x938, 0x94b, 0x932, 0x94b, 0x92e, 0x928, 0x20, 0x906, 0x907, 0x932, 0x901, 0x921,
0x94d, 0x938, 0x20, 0x935, 0x947, 0x933, 0x926, 0x915, 0x94d, 0x937, 0x93f, 0x923,
0x20, 0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x935, 0x947,
0x933, 0x938, 0x941, 0x930, 0x93f, 0x928, 0x93e, 0x92e, 0x20, 0x935, 0x947, 0x933,
0x938, 0x94d, 0x92f, 0x94b, 0x935, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x924, 0x93e,
0x939, 0x93f, 0x924, 0x940, 0x20, 0x935, 0x947, 0x933, 0x924, 0x948, 0x92a, 0x947,
0x908, 0x20, 0x935, 0x947, 0x933, 0x924, 0x948, 0x92a, 0x947, 0x908, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x924,
0x948, 0x92a, 0x947, 0x908, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20,
0x935, 0x947, 0x933, 0x924, 0x91c, 0x940, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e,
0x928, 0x20, 0x935, 0x947, 0x933, 0x91f, 0x94b, 0x915, 0x932, 0x93e, 0x90a, 0x20,
0x935, 0x947, 0x933, 0x91f, 0x94b, 0x902, 0x917, 0x93e, 0x20, 0x935, 0x947, 0x933,
0x91f, 0x94b, 0x902, 0x917, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x91f, 0x94b, 0x902, 0x917, 0x93e, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x91a,
0x941, 0x915, 0x20, 0x935, 0x947, 0x933, 0x924, 0x941, 0x930, 0x94d, 0x915, 0x92e,
0x947, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x924, 0x941, 0x930,
0x94d, 0x915, 0x92e, 0x947, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x91f,
0x941, 0x935, 0x93e, 0x932, 0x942, 0x20, 0x935, 0x947, 0x933, 0x909, 0x930, 0x941,
0x917, 0x94d, 0x935, 0x947, 0x20, 0x935, 0x947, 0x933, 0x909, 0x930, 0x941, 0x917,
0x94d, 0x935, 0x947, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924,
0x20, 0x935, 0x947, 0x933, 0x909, 0x930, 0x941, 0x917, 0x94d, 0x935, 0x947, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x909,
0x91c, 0x93c, 0x94d, 0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928,
0x20, 0x935, 0x947, 0x933, 0x909, 0x91c, 0x93c, 0x94d, 0x92c, 0x947, 0x915, 0x93f,
0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x909, 0x91c, 0x93c, 0x94d, 0x92c, 0x947,
0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x935, 0x928, 0x93e, 0x924, 0x942,
0x20, 0x935, 0x947, 0x933, 0x935, 0x928, 0x93e, 0x924, 0x942, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x935, 0x928,
0x93e, 0x924, 0x942, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20,
0x935, 0x947, 0x933, 0x935, 0x947, 0x928, 0x947, 0x91d, 0x941, 0x90f, 0x932, 0x93e,
0x20, 0x935, 0x947, 0x933, 0x935, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x935, 0x94b,
0x938, 0x94d, 0x924, 0x94b, 0x915, 0x20, 0x935, 0x947, 0x933, 0x935, 0x94d, 0x932,
0x93e, 0x926, 0x93f, 0x935, 0x94b, 0x938, 0x94d, 0x924, 0x94b, 0x915, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x935,
0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x935, 0x94b, 0x938, 0x94d, 0x924, 0x94b, 0x915,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933,
0x935, 0x94d, 0x939, 0x94b, 0x932, 0x94d, 0x917, 0x94b, 0x917, 0x94d, 0x930, 0x93e,
0x921, 0x20, 0x935, 0x947, 0x933, 0x935, 0x94d, 0x939, 0x94b, 0x932, 0x94d, 0x917,
0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x921, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x935, 0x94d, 0x939, 0x94b, 0x932,
0x94d, 0x917, 0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x921, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x935, 0x948, 0x915, 0x20,
0x906, 0x92f, 0x932, 0x901, 0x921, 0x20, 0x935, 0x947, 0x933, 0x935, 0x93e, 0x932,
0x93f, 0x938, 0x20, 0x906, 0x928, 0x940, 0x20, 0x92b, 0x94d, 0x92f, 0x942, 0x91a,
0x942, 0x928, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x92f, 0x915, 0x941, 0x924, 0x94d,
0x938, 0x94d, 0x915, 0x20, 0x935, 0x947, 0x933, 0x92f, 0x915, 0x941, 0x924, 0x94d,
0x938, 0x94d, 0x915, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924,
0x20, 0x935, 0x947, 0x933, 0x92f, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933,
0x92f, 0x947, 0x915, 0x93e, 0x924, 0x947, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d,
0x917, 0x20, 0x935, 0x947, 0x933, 0x92f, 0x947, 0x915, 0x93e, 0x924, 0x947, 0x930,
0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92f, 0x947, 0x915, 0x93e, 0x924,
0x947, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x92f, 0x941, 0x915, 0x949,
0x928, 0x20, 0x935, 0x947, 0x933, 0xc544, 0xd06c, 0xb808, 0x20, 0xc2dc, 0xac04, 0xc544,
0xd06c, 0xb808, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544, 0xd06c, 0xb808, 0x20, 0xd558, 0xacc4,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544, 0xd504, 0xac00, 0xb2c8, 0xc2a4, 0xd0c4, 0x20, 0xc2dc,
0xac04, 0xc911, 0xc559, 0xc544, 0xd504, 0xb9ac, 0xce74, 0x20, 0xc2dc, 0xac04, 0xb3d9, 0xc544,
0xd504, 0xb9ac, 0xce74, 0x20, 0xc2dc, 0xac04, 0xb0a8, 0xc544, 0xd504, 0xb9ac, 0xce74, 0x20,
0xc2dc, 0xac04, 0xc11c, 0xc544, 0xd504, 0xb9ac, 0xce74, 0x20, 0xc2dc, 0xac04, 0xc11c, 0xc544,
0xd504, 0xb9ac, 0xce74, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc11c, 0xc544, 0xd504, 0xb9ac, 0xce74,
0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc54c, 0xb798, 0xc2a4, 0xce74, 0x20,
0xc2dc, 0xac04, 0xc54c, 0xb798, 0xc2a4, 0xce74, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc54c, 0xb798,
0xc2a4, 0xce74, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc54c, 0xb9c8, 0xd2f0,
0x20, 0xd45c, 0xc900, 0x20, 0xc2dc, 0xac04, 0xc54c, 0xb9c8, 0xd2f0, 0x20, 0xd45c, 0xc900,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xc54c, 0xb9c8, 0xd2f0, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xc544, 0xb9c8, 0xc874, 0x20, 0xc2dc, 0xac04, 0xc544, 0xb9c8, 0xc874, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xc544, 0xb9c8, 0xc874, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xbbf8, 0x20, 0xc911, 0xbd80, 0x20, 0xc2dc, 0xac04, 0xbbf8, 0x20, 0xc911, 0xbd80,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xbbf8, 0x20, 0xc911, 0xbd80, 0x20, 0xd558, 0xacc4, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xbbf8, 0x20, 0xb3d9, 0xbd80, 0x20, 0xc2dc, 0xac04, 0xbbf8, 0x20,
0xb3d9, 0xbd80, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbbf8, 0x20, 0xb3d9, 0xbd80, 0x20, 0xd558,
0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbbf8, 0x20, 0xc0b0, 0xc9c0, 0x20, 0xc2dc, 0xac04,
0xbbf8, 0x20, 0xc0b0, 0xc545, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbbf8, 0x20, 0xc0b0, 0xc9c0,
0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbbf8, 0x20, 0xd0dc, 0xd3c9, 0xc591,
0x20, 0xc2dc, 0xac04, 0xbbf8, 0x20, 0xd0dc, 0xd3c9, 0xc591, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xbbf8, 0x20, 0xd0dc, 0xd3c9, 0xc591, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xc544, 0xb098, 0xb514, 0xb9ac, 0x20, 0xc2dc, 0xac04, 0xc544, 0xb098, 0xb514, 0xb9ac, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xc544, 0xb098, 0xb514, 0xb9ac, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xc544, 0xd53c, 0xc544, 0x20, 0xc2dc, 0xac04, 0xc544, 0xd53c, 0xc544, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xc544, 0xd53c, 0xc544, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xc545, 0xd0c0, 0xc6b0, 0x20, 0xd45c, 0xc900, 0x20, 0xc2dc, 0xac04, 0xc545, 0xd0c0,
0xc6b0, 0x20, 0xd45c, 0xc900, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc545, 0xd0c0, 0xc6b0, 0x20,
0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc545, 0xd1f4, 0xbca0, 0x20, 0xd45c, 0xc900,
0x20, 0xc2dc, 0xac04, 0xc545, 0xd1f4, 0xbca0, 0x20, 0xd45c, 0xc900, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xc545, 0xd1f4, 0xbca0, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544,
0xb77c, 0xbe44, 0xc544, 0x20, 0xc2dc, 0xac04, 0xc544, 0xb77c, 0xbe44, 0xc544, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xc544, 0xb77c, 0xbe44, 0xc544, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xc544, 0xb974, 0xd5e8, 0xd2f0, 0xb098, 0x20, 0xc2dc, 0xac04, 0xc544, 0xb974, 0xd5e8,
0xd2f0, 0xb098, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544, 0xb974, 0xd5e8, 0xd2f0, 0xb098, 0x20,
0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544, 0xb974, 0xd5e8, 0xd2f0, 0xb098, 0x20,
0xc11c, 0xbd80, 0x20, 0xc2dc, 0xac04, 0xc544, 0xb974, 0xd5e8, 0xd2f0, 0xb098, 0x20, 0xc11c,
0xbd80, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544, 0xb974, 0xd5e8, 0xd2f0, 0xb098, 0x20, 0xc11c,
0xbd80, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544, 0xb974, 0xba54, 0xb2c8,
0xc544, 0x20, 0xc2dc, 0xac04, 0xc544, 0xb974, 0xba54, 0xb2c8, 0xc544, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xc544, 0xb974, 0xba54, 0xb2c8, 0xc544, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xb300, 0xc11c, 0xc591, 0x20, 0xc2dc, 0xac04, 0xb300, 0xc11c, 0xc591, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xb300, 0xc11c, 0xc591, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xc624, 0xc2a4, 0xd2b8, 0xb808, 0xc77c, 0xb9ac, 0xc544, 0x20, 0xc911, 0xbd80, 0x20, 0xc2dc,
0xac04, 0xc624, 0xc2a4, 0xd2b8, 0xb808, 0xc77c, 0xb9ac, 0xc544, 0x20, 0xc911, 0xbd80, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xc624, 0xc2a4, 0xd2b8, 0xb808, 0xc77c, 0xb9ac, 0xc544, 0x20, 0xc911,
0xbd80, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc624, 0xc2a4, 0xd2b8, 0xb808,
0xc77c, 0xb9ac, 0xc544, 0x20, 0xc911, 0xc11c, 0xbd80, 0x20, 0xc2dc, 0xac04, 0xc624, 0xc2a4,
0xd2b8, 0xb808, 0xc77c, 0xb9ac, 0xc544, 0x20, 0xc911, 0xc11c, 0xbd80, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xc624, 0xc2a4, 0xd2b8, 0xb808, 0xc77c, 0xb9ac, 0xc544, 0x20, 0xc911, 0xc11c, 0xbd80,
0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc624, 0xc2a4, 0xd2b8, 0xb808, 0xc77c,
0xb9ac, 0xc544, 0x20, 0xb3d9, 0xbd80, 0x20, 0xc2dc, 0xac04, 0xc624, 0xc2a4, 0xd2b8, 0xb808,
0xc77c, 0xb9ac, 0xc544, 0x20, 0xb3d9, 0xbd80, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc624, 0xc2a4,
0xd2b8, 0xb808, 0xc77c, 0xb9ac, 0xc544, 0x20, 0xb3d9, 0xbd80, 0x20, 0xd558, 0xacc4, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xc624, 0xc2a4, 0xd2b8, 0xb808, 0xc77c, 0xb9ac, 0xc544, 0x20, 0xc11c,
0xbd80, 0x20, 0xc2dc, 0xac04, 0xc624, 0xc2a4, 0xd2b8, 0xb808, 0xc77c, 0xb9ac, 0xc544, 0x20,
0xc11c, 0xbd80, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc624, 0xc2a4, 0xd2b8, 0xb808, 0xc77c, 0xb9ac,
0xc544, 0x20, 0xc11c, 0xbd80, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544,
0xc81c, 0xb974, 0xbc14, 0xc774, 0xc794, 0x20, 0xc2dc, 0xac04, 0xc544, 0xc81c, 0xb974, 0xbc14,
0xc774, 0xc794, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544, 0xc81c, 0xb974, 0xbc14, 0xc774, 0xc794,
0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544, 0xc870, 0xb808, 0xc2a4, 0x20,
0xc2dc, 0xac04, 0xc544, 0xc870, 0xb808, 0xc2a4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544, 0xc870,
0xb808, 0xc2a4, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbc29, 0xae00, 0xb77c,
0xb370, 0xc2dc, 0x20, 0xc2dc, 0xac04, 0xbc29, 0xae00, 0xb77c, 0xb370, 0xc2dc, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xbc29, 0xae00, 0xb77c, 0xb370, 0xc2dc, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xbd80, 0xd0c4, 0x20, 0xc2dc, 0xac04, 0xbcfc, 0xb9ac, 0xbe44, 0xc544, 0x20,
0xc2dc, 0xac04, 0xbe0c, 0xb77c, 0xc9c8, 0xb9ac, 0xc544, 0x20, 0xc2dc, 0xac04, 0xbe0c, 0xb77c,
0xc9c8, 0xb9ac, 0xc544, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbe0c, 0xb77c, 0xc9c8, 0xb9ac, 0xc544,
0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbe0c, 0xb8e8, 0xb098, 0xc774, 0x20,
0xc2dc, 0xac04, 0xce74, 0xbcf4, 0x20, 0xbca0, 0xb974, 0xb370, 0x20, 0xc2dc, 0xac04, 0xce74,
0xbcf4, 0x20, 0xbca0, 0xb974, 0xb370, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xce74, 0xbcf4, 0x20,
0xbca0, 0xb974, 0xb370, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xcf00, 0xc774,
0xc2dc, 0x20, 0xc2dc, 0xac04, 0xcc28, 0xbaa8, 0xb85c, 0x20, 0xc2dc, 0xac04, 0xcc44, 0xd140,
0x20, 0xc2dc, 0xac04, 0xcc44, 0xd140, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xcc44, 0xd140, 0x20,
0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xce60, 0xb808, 0x20, 0xc2dc, 0xac04, 0xce60,
0xb808, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xce60, 0xb808, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xc911, 0xad6d, 0x20, 0xc2dc, 0xac04, 0xc911, 0xad6d, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xc911, 0xad6d, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd06c, 0xb9ac,
0xc2a4, 0xb9c8, 0xc2a4, 0xc12c, 0x20, 0xc2dc, 0xac04, 0xcf54, 0xcf54, 0xc2a4, 0x20, 0xc81c,
0xb3c4, 0x20, 0xc2dc, 0xac04, 0xcf5c, 0xb86c, 0xbe44, 0xc544, 0x20, 0xc2dc, 0xac04, 0xcf5c,
0xb86c, 0xbe44, 0xc544, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xcf5c, 0xb86c, 0xbe44, 0xc544, 0x20,
0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xcfe1, 0x20, 0xc81c, 0xb3c4, 0x20, 0xc2dc,
0xac04, 0xcfe1, 0x20, 0xc81c, 0xb3c4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xcfe1, 0x20, 0xc81c,
0xb3c4, 0x20, 0xc808, 0xbc18, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xcfe0,
0xbc14, 0x20, 0xc2dc, 0xac04, 0xcfe0, 0xbc14, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xcfe0, 0xbc14,
0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb370, 0xc774, 0xbe44, 0xc2a4, 0x20,
0xc2dc, 0xac04, 0xb4a4, 0xbabd, 0xb4a4, 0xb974, 0xbe4c, 0x20, 0xc2dc, 0xac04, 0xb3d9, 0xd2f0,
0xbaa8, 0xb974, 0x20, 0xc2dc, 0xac04, 0xc774, 0xc2a4, 0xd130, 0xc12c, 0x20, 0xc2dc, 0xac04,
0xc774, 0xc2a4, 0xd130, 0xc12c, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc774, 0xc2a4, 0xd130, 0xc12c,
0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc5d0, 0xcf70, 0xb3c4, 0xb974, 0x20,
0xc2dc, 0xac04, 0xc911, 0xbd80, 0xc720, 0xb7fd, 0x20, 0xc2dc, 0xac04, 0xc911, 0xbd80, 0xc720,
0xb7fd, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc911, 0xbd80, 0xc720, 0xb7fd, 0x20, 0xd558, 0xacc4,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xb3d9, 0xc720, 0xb7fd, 0x20, 0xc2dc, 0xac04, 0xb3d9, 0xc720,
0xb7fd, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb3d9, 0xc720, 0xb7fd, 0x20, 0xd558, 0xacc4, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xadf9, 0xb3d9, 0xc720, 0xb7fd, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc11c,
0xc720, 0xb7fd, 0x20, 0xc2dc, 0xac04, 0xc11c, 0xc720, 0xb7fd, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xc11c, 0xc720, 0xb7fd, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd3ec, 0xd074,
0xb79c, 0xb4dc, 0x20, 0xc81c, 0xb3c4, 0x20, 0xc2dc, 0xac04, 0xd3ec, 0xd074, 0xb79c, 0xb4dc,
0x20, 0xc81c, 0xb3c4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd3ec, 0xd074, 0xb79c, 0xb4dc, 0x20,
0xc81c, 0xb3c4, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd53c, 0xc9c0, 0x20,
0xc2dc, 0xac04, 0xd53c, 0xc9c0, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd53c, 0xc9c0, 0x20, 0xd558,
0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd504, 0xb791, 0xc2a4, 0xb839, 0x20, 0xac00, 0xc774,
0xc544, 0xb098, 0x20, 0xc2dc, 0xac04, 0xd504, 0xb791, 0xc2a4, 0xb839, 0x20, 0xb0a8, 0xbd80,
0x20, 0xc2dd, 0xbbfc, 0xc9c0, 0x20, 0xbc0f, 0x20, 0xb0a8, 0xadf9, 0x20, 0xc2dc, 0xac04,
0xac08, 0xb77c, 0xd30c, 0xace0, 0xc2a4, 0x20, 0xc2dc, 0xac04, 0xac10, 0xbe44, 0xc5d0, 0x20,
0xc2dc, 0xac04, 0xc870, 0xc9c0, 0xc544, 0x20, 0xc2dc, 0xac04, 0xc870, 0xc9c0, 0xc544, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xc870, 0xc9c0, 0xc544, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xae38, 0xbc84, 0xd2b8, 0x20, 0xc81c, 0xb3c4, 0x20, 0xc2dc, 0xac04, 0xadf8, 0xb9ac,
0xb2c8, 0xce58, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xadf8, 0xb9b0, 0xb780, 0xb4dc, 0x20, 0xb3d9,
0xbd80, 0x20, 0xc2dc, 0xac04, 0xadf8, 0xb9b0, 0xb780, 0xb4dc, 0x20, 0xb3d9, 0xbd80, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xadf8, 0xb9b0, 0xb780, 0xb4dc, 0x20, 0xb3d9, 0xbd80, 0x20, 0xd558,
0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xadf8, 0xb9b0, 0xb780, 0xb4dc, 0x20, 0xc11c, 0xbd80,
0x20, 0xc2dc, 0xac04, 0xadf8, 0xb9b0, 0xb780, 0xb4dc, 0x20, 0xc11c, 0xbd80, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xadf8, 0xb9b0, 0xb780, 0xb4dc, 0x20, 0xc11c, 0xbd80, 0x20, 0xd558, 0xacc4,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xad0c, 0x20, 0xd45c, 0xc900, 0x20, 0xc2dc, 0xac04, 0xac78,
0xd504, 0xb9cc, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd558, 0xc640, 0xc774, 0x20, 0xc54c, 0xb958,
0xc0e8, 0x20, 0xc2dc, 0xac04, 0xd558, 0xc640, 0xc774, 0x20, 0xc54c, 0xb958, 0xc0e8, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xd558, 0xc640, 0xc774, 0x20, 0xc54c, 0xb958, 0xc0e8, 0x20, 0xd558,
0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd64d, 0xcf69, 0x20, 0xc2dc, 0xac04, 0xd64d, 0xcf69,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xd64d, 0xcf69, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xd638, 0xbe0c, 0xb4dc, 0x20, 0xc2dc, 0xac04, 0xd638, 0xbe0c, 0xb4dc, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xd638, 0xbe0c, 0xb4dc, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xc778, 0xb3c4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc778, 0xb3c4, 0xc591, 0x20, 0xc2dc, 0xac04,
0xc778, 0xb3c4, 0xcc28, 0xc774, 0xb098, 0x20, 0xc2dc, 0xac04, 0xc911, 0xbd80, 0x20, 0xc778,
0xb3c4, 0xb124, 0xc2dc, 0xc544, 0x20, 0xc2dc, 0xac04, 0xb3d9, 0xbd80, 0x20, 0xc778, 0xb3c4,
0xb124, 0xc2dc, 0xc544, 0x20, 0xc2dc, 0xac04, 0xc11c, 0xbd80, 0x20, 0xc778, 0xb3c4, 0xb124,
0xc2dc, 0xc544, 0x20, 0xc2dc, 0xac04, 0xc774, 0xb780, 0x20, 0xc2dc, 0xac04, 0xc774, 0xb780,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xc774, 0xb780, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xc774, 0xb974, 0xcfe0, 0xce20, 0xd06c, 0x20, 0xc2dc, 0xac04, 0xc774, 0xb974, 0xcfe0,
0xce20, 0xd06c, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc774, 0xb974, 0xcfe0, 0xce20, 0xd06c, 0x20,
0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc774, 0xc2a4, 0xb77c, 0xc5d8, 0x20, 0xc2dc,
0xac04, 0xc774, 0xc2a4, 0xb77c, 0xc5d8, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc774, 0xc2a4, 0xb77c,
0xc5d8, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc77c, 0xbcf8, 0x20, 0xc2dc,
0xac04, 0xc77c, 0xbcf8, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc77c, 0xbcf8, 0x20, 0xd558, 0xacc4,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xd398, 0xd2b8, 0xb85c, 0xd30c, 0xbe14, 0xb86d, 0xc2a4, 0xd06c,
0x2d, 0xce84, 0xcc28, 0xce20, 0xd0a4, 0x20, 0xc2dc, 0xac04, 0xd398, 0xd2b8, 0xb85c, 0xd30c,
0xbe14, 0xb86d, 0xc2a4, 0xd06c, 0x2d, 0xce84, 0xcc28, 0xce20, 0xd0a4, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xd398, 0xd2b8, 0xb85c, 0xd30c, 0xbe14, 0xb86d, 0xc2a4, 0xd06c, 0x2d, 0xce84, 0xcc28,
0xce20, 0xd0a4, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xce74, 0xc790, 0xd750,
0xc2a4, 0xd0c4, 0x20, 0xc2dc, 0xac04, 0xb3d9, 0xbd80, 0x20, 0xce74, 0xc790, 0xd750, 0xc2a4,
0xd0c4, 0x20, 0xc2dc, 0xac04, 0xc11c, 0xbd80, 0x20, 0xce74, 0xc790, 0xd750, 0xc2a4, 0xd0c4,
0x20, 0xc2dc, 0xac04, 0xb300, 0xd55c, 0xbbfc, 0xad6d, 0x20, 0xc2dc, 0xac04, 0xb300, 0xd55c,
0xbbfc, 0xad6d, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb300, 0xd55c, 0xbbfc, 0xad6d, 0x20, 0xd558,
0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xcf54, 0xc2a4, 0xb77c, 0xc5d0, 0xc12c, 0x20, 0xc2dc,
0xac04, 0xd06c, 0xb77c, 0xc2a4, 0xb178, 0xc57c, 0xb974, 0xc2a4, 0xd06c, 0x20, 0xc2dc, 0xac04,
0xd06c, 0xb77c, 0xc2a4, 0xb178, 0xc57c, 0xb974, 0xc2a4, 0xd06c, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xd06c, 0xb77c, 0xc2a4, 0xb178, 0xc57c, 0xb974, 0xc2a4, 0xd06c, 0x20, 0xd558, 0xacc4, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xd0a4, 0xb974, 0xae30, 0xc2a4, 0xc2a4, 0xd0c4, 0x20, 0xc2dc, 0xac04,
0xb791, 0xce74, 0x20, 0xd45c, 0xc900, 0x20, 0xc2dc, 0xac04, 0xb77c, 0xc778, 0x20, 0xc81c,
0xb3c4, 0x20, 0xc2dc, 0xac04, 0xb85c, 0xb4dc, 0x20, 0xd558, 0xc6b0, 0x20, 0xc2dc, 0xac04,
0xb85c, 0xb4dc, 0x20, 0xd558, 0xc6b0, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb85c, 0xb4dc, 0x20,
0xd558, 0xc6b0, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb9c8, 0xce74, 0xc624,
0x20, 0xc2dc, 0xac04, 0xb9c8, 0xce74, 0xc624, 0x20, 0xd45c, 0xc900, 0x20, 0xc2dc, 0xac04,
0xb9c8, 0xce74, 0xc624, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb9c8, 0xac00,
0xb2e8, 0x20, 0xc2dc, 0xac04, 0xb9c8, 0xac00, 0xb2e8, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb9c8,
0xac00, 0xb2e8, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb9d0, 0xb808, 0xc774,
0xc2dc, 0xc544, 0x20, 0xc2dc, 0xac04, 0xbab0, 0xb514, 0xbe0c, 0x20, 0xc2dc, 0xac04, 0xb9c8,
0xb974, 0xd0a4, 0xc988, 0x20, 0xc81c, 0xb3c4, 0x20, 0xc2dc, 0xac04, 0xb9c8, 0xc15c, 0x20,
0xc81c, 0xb3c4, 0x20, 0xc2dc, 0xac04, 0xbaa8, 0xb9ac, 0xc154, 0xc2a4, 0x20, 0xc2dc, 0xac04,
0xbaa8, 0xb9ac, 0xc154, 0xc2a4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbaa8, 0xb9ac, 0xc154, 0xc2a4,
0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbaa8, 0xc2a8, 0x20, 0xc2dc, 0xac04,
0xba55, 0xc2dc, 0xcf54, 0x20, 0xd0dc, 0xd3c9, 0xc591, 0x20, 0xc2dc, 0xac04, 0xba55, 0xc2dc,
0xcf54, 0x20, 0xd0dc, 0xd3c9, 0xc591, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xba55, 0xc2dc, 0xcf54,
0x20, 0xd0dc, 0xd3c9, 0xc591, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc6b8,
0xb780, 0xbc14, 0xd1a0, 0xb974, 0x20, 0xc2dc, 0xac04, 0xc6b8, 0xb780, 0xbc14, 0xd1a0, 0xb974,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xc6b8, 0xb780, 0xbc14, 0xd1a0, 0xb974, 0x20, 0xd558, 0xacc4,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xbaa8, 0xc2a4, 0xd06c, 0xbc14, 0x20, 0xc2dc, 0xac04, 0xbaa8,
0xc2a4, 0xd06c, 0xbc14, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbaa8, 0xc2a4, 0xd06c, 0xbc14, 0x20,
0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbbf8, 0xc580, 0xb9c8, 0x20, 0xc2dc, 0xac04,
0xb098, 0xc6b0, 0xb8e8, 0x20, 0xc2dc, 0xac04, 0xb124, 0xd314, 0x20, 0xc2dc, 0xac04, 0xb274,
0xce7c, 0xb808, 0xb3c4, 0xb2c8, 0xc544, 0x20, 0xc2dc, 0xac04, 0xb274, 0xce7c, 0xb808, 0xb3c4,
0xb2c8, 0xc544, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb274, 0xce7c, 0xb808, 0xb3c4, 0xb2c8, 0xc544,
0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb274, 0xc9c8, 0xb79c, 0xb4dc, 0x20,
0xc2dc, 0xac04, 0xb274, 0xc9c8, 0xb79c, 0xb4dc, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb274, 0xc9c8,
0xb79c, 0xb4dc, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb274, 0xd380, 0xb4e4,
0xb79c, 0xb4dc, 0x20, 0xc2dc, 0xac04, 0xb274, 0xd380, 0xb4e4, 0xb79c, 0xb4dc, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xb274, 0xd380, 0xb4e4, 0xb79c, 0xb4dc, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xb2c8, 0xc6b0, 0xc5d0, 0x20, 0xc2dc, 0xac04, 0xb178, 0xd37d, 0xc12c, 0x20,
0xc2dc, 0xac04, 0xb178, 0xd37d, 0xc12c, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb178, 0xd37d, 0xc12c,
0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd398, 0xb974, 0xb09c, 0xb3c4, 0x20,
0xb370, 0x20, 0xb178, 0xb85c, 0xb0d0, 0x20, 0xc2dc, 0xac04, 0xd398, 0xb974, 0xb09c, 0xb3c4,
0x20, 0xb370, 0x20, 0xb178, 0xb85c, 0xb0d0, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd398, 0xb974,
0xb09c, 0xb3c4, 0x20, 0xb370, 0x20, 0xb178, 0xb85c, 0xb0d0, 0x20, 0xd558, 0xacc4, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xbd81, 0xb9c8, 0xb9ac, 0xc544, 0xb098, 0x20, 0xc81c, 0xb3c4, 0x20,
0xd45c, 0xc900, 0x20, 0xc2dc, 0xac04, 0xb178, 0xbcf4, 0xc2dc, 0xbe44, 0xb974, 0xc2a4, 0xd06c,
0x20, 0xc2dc, 0xac04, 0xb178, 0xbcf4, 0xc2dc, 0xbe44, 0xb974, 0xc2a4, 0xd06c, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xb178, 0xbcf4, 0xc2dc, 0xbe44, 0xb974, 0xc2a4, 0xd06c, 0x20, 0xd558, 0xacc4,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xc634, 0xc2a4, 0xd06c, 0x20, 0xc2dc, 0xac04, 0xc634, 0xc2a4,
0xd06c, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc634, 0xc2a4, 0xd06c, 0x20, 0xd558, 0xacc4, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xd30c, 0xd0a4, 0xc2a4, 0xd0c4, 0x20, 0xc2dc, 0xac04, 0xd30c, 0xd0a4,
0xc2a4, 0xd0c4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd30c, 0xd0a4, 0xc2a4, 0xd0c4, 0x20, 0xd558,
0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd314, 0xb77c, 0xc6b0, 0x20, 0xc2dc, 0xac04, 0xd30c,
0xd478, 0xc544, 0xb274, 0xae30, 0xb2c8, 0x20, 0xc2dc, 0xac04, 0xd30c, 0xb77c, 0xacfc, 0xc774,
0x20, 0xc2dc, 0xac04, 0xd30c, 0xb77c, 0xacfc, 0xc774, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd30c,
0xb77c, 0xacfc, 0xc774, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd398, 0xb8e8,
0x20, 0xc2dc, 0xac04, 0xd398, 0xb8e8, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd398, 0xb8e8, 0x20,
0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd544, 0xb9ac, 0xd540, 0x20, 0xc2dc, 0xac04,
0xd544, 0xb9ac, 0xd540, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd544, 0xb9ac, 0xd540, 0x20, 0xd558,
0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd53c, 0xb2c9, 0xc2a4, 0x20, 0xc81c, 0xb3c4, 0x20,
0xc2dc, 0xac04, 0xc138, 0xc778, 0xd2b8, 0xd53c, 0xc5d0, 0xb974, 0x20, 0xbbf8, 0xd074, 0xb871,
0x20, 0xc2dc, 0xac04, 0xc138, 0xc778, 0xd2b8, 0xd53c, 0xc5d0, 0xb974, 0x20, 0xbbf8, 0xd074,
0xb871, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc138, 0xc778, 0xd2b8, 0xd53c, 0xc5d0, 0xb974, 0x20,
0xbbf8, 0xd074, 0xb871, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd54f, 0xcf00,
0xc5b8, 0x20, 0xc2dc, 0xac04, 0xd3ec, 0xb098, 0xd398, 0x20, 0xc2dc, 0xac04, 0xd0a4, 0xc9c8,
0xb85c, 0xb974, 0xb2e4, 0x20, 0xc2dc, 0xac04, 0xd0a4, 0xc9c8, 0xb85c, 0xb974, 0xb2e4, 0x20,
0xd45c, 0xc900, 0x20, 0xc2dc, 0xac04, 0xd0a4, 0xc9c8, 0xb85c, 0xb974, 0xb2e4, 0x20, 0xd558,
0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb808, 0xc704, 0xb2c8, 0xc639, 0x20, 0xc2dc, 0xac04,
0xb85c, 0xb370, 0xb77c, 0x20, 0xc2dc, 0xac04, 0xc0ac, 0xd560, 0xb9b0, 0x20, 0xc2dc, 0xac04,
0xc0ac, 0xd560, 0xb9b0, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc0ac, 0xd560, 0xb9b0, 0x20, 0xd558,
0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc0ac, 0xb9c8, 0xb77c, 0x20, 0xc2dc, 0xac04, 0xc0ac,
0xb9c8, 0xb77c, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc0ac, 0xb9c8, 0xb77c, 0x20, 0xd558, 0xacc4,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xc0ac, 0xbaa8, 0xc544, 0x20, 0xc2dc, 0xac04, 0xc0ac, 0xbaa8,
0xc544, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc0ac, 0xbaa8, 0xc544, 0x20, 0xd558, 0xacc4, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xc138, 0xc774, 0xc178, 0x20, 0xc2dc, 0xac04, 0xc2f1, 0xac00, 0xd3ec,
0xb974, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc194, 0xb85c, 0xbaac, 0x20, 0xc81c, 0xb3c4, 0x20,
0xc2dc, 0xac04, 0xc0ac, 0xc6b0, 0xc2a4, 0x20, 0xc870, 0xc9c0, 0xc544, 0x20, 0xc2dc, 0xac04,
0xc218, 0xb9ac, 0xb0a8, 0x20, 0xc2dc, 0xac04, 0xc1fc, 0xc640, 0x20, 0xc2dc, 0xac04, 0xd0c0,
0xd788, 0xd2f0, 0x20, 0xc2dc, 0xac04, 0xb300, 0xb9cc, 0x20, 0xc2dc, 0xac04, 0xb300, 0xb9cc,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xb300, 0xb9cc, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xd0c0, 0xc9c0, 0xd0a4, 0xc2a4, 0xd0c4, 0x20, 0xc2dc, 0xac04, 0xd1a0, 0xcf08, 0xb77c,
0xc6b0, 0x20, 0xc2dc, 0xac04, 0xd1b5, 0xac00, 0x20, 0xc2dc, 0xac04, 0xd1b5, 0xac00, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xd1b5, 0xac00, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xcd94, 0xd06c, 0x20, 0xc2dc, 0xac04, 0xd22c, 0xb974, 0xd06c, 0xba54, 0xb2c8, 0xc2a4, 0xd0c4,
0x20, 0xc2dc, 0xac04, 0xd22c, 0xb974, 0xd06c, 0xba54, 0xb2c8, 0xc2a4, 0xd0c4, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xd22c, 0xb974, 0xd06c, 0xba54, 0xb2c8, 0xc2a4, 0xd0c4, 0x20, 0xd558, 0xacc4,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xd22c, 0xbc1c, 0xb8e8, 0x20, 0xc2dc, 0xac04, 0xc6b0, 0xb8e8,
0xacfc, 0xc774, 0x20, 0xc2dc, 0xac04, 0xc6b0, 0xb8e8, 0xacfc, 0xc774, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xc6b0, 0xb8e8, 0xacfc, 0xc774, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xc6b0, 0xc988, 0xbca0, 0xd0a4, 0xc2a4, 0xd0c4, 0x20, 0xc2dc, 0xac04, 0xc6b0, 0xc988, 0xbca0,
0xd0a4, 0xc2a4, 0xd0c4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc6b0, 0xc988, 0xbca0, 0xd0a4, 0xc2a4,
0xd0c4, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbc14, 0xb204, 0xc544, 0xd22c,
0x20, 0xc2dc, 0xac04, 0xbc14, 0xb204, 0xc544, 0xd22c, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbc14,
0xb204, 0xc544, 0xd22c, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbca0, 0xb124,
0xc218, 0xc5d8, 0xb77c, 0x20, 0xc2dc, 0xac04, 0xbe14, 0xb77c, 0xb514, 0xbcf4, 0xc2a4, 0xd1a0,
0xd06c, 0x20, 0xc2dc, 0xac04, 0xbe14, 0xb77c, 0xb514, 0xbcf4, 0xc2a4, 0xd1a0, 0xd06c, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xbe14, 0xb77c, 0xb514, 0xbcf4, 0xc2a4, 0xd1a0, 0xd06c, 0x20, 0xd558,
0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbcfc, 0xace0, 0xadf8, 0xb77c, 0xb4dc, 0x20, 0xc2dc,
0xac04, 0xbcfc, 0xace0, 0xadf8, 0xb77c, 0xb4dc, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbcfc, 0xace0,
0xadf8, 0xb77c, 0xb4dc, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbcf4, 0xc2a4,
0xd1a1, 0x20, 0xc2dc, 0xac04, 0xc6e8, 0xc774, 0xd06c, 0xc12c, 0x20, 0xc2dc, 0xac04, 0xc6d4,
0xb9ac, 0xc2a4, 0xd478, 0xd22c, 0xb098, 0x20, 0xc81c, 0xb3c4, 0x20, 0xc2dc, 0xac04, 0xc57c,
0xcfe0, 0xce20, 0xd06c, 0x20, 0xc2dc, 0xac04, 0xc57c, 0xcfe0, 0xce20, 0xd06c, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xc57c, 0xcfe0, 0xce20, 0xd06c, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xc608, 0xce74, 0xd14c, 0xb9b0, 0xbd80, 0xb974, 0xd06c, 0x20, 0xc2dc, 0xac04, 0xc608,
0xce74, 0xd14c, 0xb9b0, 0xbd80, 0xb974, 0xd06c, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc608, 0xce74,
0xd14c, 0xb9b0, 0xbd80, 0xb974, 0xd06c, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xc720, 0xcf58, 0x20, 0xc2dc, 0xac04, 0xc870, 0xc120, 0x20, 0xc2dc, 0xac04, 0xc870, 0xc120,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xc870, 0xc120, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900,
0xc2dc, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x45, 0x66, 0x78, 0x61, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x41, 0x66, 0x72, 0xee, 0x6b, 0x61, 0x79, 0x61, 0x20, 0x4e, 0x61, 0x76,
0xee, 0x6e, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x41, 0x66, 0x72, 0xee,
0x6b, 0x61, 0x79, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x68, 0x69, 0x6c, 0x61,
0x74, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x66, 0x72, 0xee, 0x6b, 0x61,
0x79, 0x61, 0x20, 0x42, 0x61, 0x15f, 0xfb, 0x72, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x41, 0x66, 0x72, 0xee, 0x6b, 0x61, 0x79, 0x61, 0x20, 0x52,
0x6f, 0x6a, 0x61, 0x76, 0x61, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x66,
0x72, 0xee, 0x6b, 0x61, 0x79, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76,
0x61, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e,
0xea, 0x20, 0x79, 0x61, 0x20, 0x41, 0x66, 0x72, 0xee, 0x6b, 0x61, 0x79,
0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x79, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x61, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x79, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20,
0x79, 0x61, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x79, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea,
0x20, 0x79, 0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x4e, 0x61, 0x76, 0x65, 0x6e, 0x64, 0xee,
0x20, 0x79, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x72, 0xee, 0x6b, 0x61, 0x79,
0x61, 0x20, 0x42, 0x61, 0x6b, 0x75, 0x72, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20,
0x4e, 0x61, 0x76, 0x65, 0x6e, 0x64, 0xee, 0x20, 0x79, 0x61, 0x20, 0x41,
0x6d, 0x65, 0x72, 0xee, 0x6b, 0x61, 0x79, 0x61, 0x20, 0x42, 0x61, 0x6b,
0x75, 0x72, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee,
0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x4e, 0x61, 0x76, 0x65, 0x6e, 0x64,
0xee, 0x20, 0x79, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x72, 0xee, 0x6b, 0x61,
0x79, 0x61, 0x20, 0x42, 0x61, 0x6b, 0x75, 0x72, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x52, 0x6f, 0x6a, 0x68, 0x69, 0x6c, 0x61, 0x74, 0x20, 0x61,
0x20, 0x41, 0x6d, 0x65, 0x72, 0xee, 0x6b, 0x61, 0x79, 0x61, 0x20, 0x42,
0x61, 0x6b, 0x75, 0x72, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x52, 0x6f, 0x6a,
0x68, 0x69, 0x6c, 0x61, 0x74, 0x20, 0x79, 0x61, 0x20, 0x41, 0x6d, 0x65,
0x72, 0xee, 0x6b, 0x61, 0x79, 0x61, 0x20, 0x42, 0x61, 0x6b, 0x75, 0x72,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea,
0x20, 0x79, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x68, 0x69, 0x6c, 0x61, 0x74,
0x20, 0x79, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x72, 0xee, 0x6b, 0x61, 0x79,
0x61, 0x20, 0x42, 0x61, 0x6b, 0x75, 0x72, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0xc7, 0xee, 0x79, 0x61, 0x79, 0xee, 0x20, 0x79, 0x61, 0x20, 0x41,
0x6d, 0x65, 0x72, 0xee, 0x6b, 0x61, 0x79, 0x61, 0x20, 0x42, 0x61, 0x6b,
0x75, 0x72, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0xc7, 0xee, 0x79, 0x61, 0x79,
0xee, 0x20, 0x79, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x72, 0xee, 0x6b, 0x61,
0x79, 0x61, 0x20, 0x42, 0x61, 0x6b, 0x75, 0x72, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20,
0xc7, 0xee, 0x79, 0x61, 0x79, 0xee, 0x20, 0x79, 0x61, 0x20, 0x41, 0x6d,
0x65, 0x72, 0xee, 0x6b, 0x61, 0x79, 0x61, 0x20, 0x42, 0x61, 0x6b, 0x75,
0x72, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x50, 0x61, 0x73, 0xee, 0x66,
0xee, 0x6b, 0xea, 0x20, 0x79, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x72, 0xee,
0x6b, 0x61, 0x79, 0x61, 0x20, 0x42, 0x61, 0x6b, 0x75, 0x72, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x61, 0x20, 0x50, 0x61, 0x73, 0xee, 0x66, 0xee, 0x6b, 0xea, 0x20,
0x79, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x72, 0xee, 0x6b, 0x61, 0x79, 0x61,
0x20, 0x42, 0x61, 0x6b, 0x75, 0x72, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x50, 0x61,
0x73, 0xee, 0x66, 0xee, 0x6b, 0xea, 0x20, 0x79, 0x61, 0x20, 0x41, 0x6d,
0x65, 0x72, 0xee, 0x6b, 0x61, 0x79, 0x61, 0x20, 0x42, 0x61, 0x6b, 0x75,
0x72, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x41, 0x70, 0x69, 0x61, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x70, 0x69, 0x61, 0x79, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea,
0x20, 0x79, 0x61, 0x20, 0x41, 0x70, 0x69, 0x61, 0x79, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x45, 0x72, 0x65, 0x62, 0x69, 0x73, 0x74, 0x61,
0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x45, 0x72, 0x65, 0x62, 0x69,
0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48,
0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x45, 0x72, 0x65,
0x62, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x41, 0x72, 0x6a, 0x61, 0x6e, 0x74, 0xee, 0x6e, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x61, 0x20, 0x41, 0x72, 0x6a, 0x61, 0x6e, 0x74, 0xee, 0x6e, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea,
0x20, 0x79, 0x61, 0x20, 0x41, 0x72, 0x6a, 0x61, 0x6e, 0x74, 0xee, 0x6e,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x41, 0x72, 0x6a, 0x61, 0x6e,
0x74, 0xee, 0x6e, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x61, 0x20, 0x41, 0x72, 0x6a, 0x61, 0x6e, 0x74, 0xee, 0x6e,
0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20,
0x41, 0x72, 0x6a, 0x61, 0x6e, 0x74, 0xee, 0x6e, 0x61, 0x20, 0x52, 0x6f,
0x6a, 0x61, 0x76, 0x61, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x45, 0x72,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x45, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e,
0xea, 0x20, 0x79, 0x61, 0x20, 0x45, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x41, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0xee, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0xee, 0x6b, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61,
0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0xee, 0x6b, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x41, 0x77, 0x69, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x79, 0x61, 0x79, 0x61, 0x20, 0x4e, 0x61, 0x76, 0xee, 0x6e, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x61, 0x20, 0x41, 0x77, 0x69, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79,
0x61, 0x79, 0x61, 0x20, 0x4e, 0x61, 0x76, 0xee, 0x6e, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61,
0x20, 0x41, 0x77, 0x69, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x79,
0x61, 0x20, 0x4e, 0x61, 0x76, 0xee, 0x6e, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x79, 0x61, 0x20, 0x4e, 0x61,
0x76, 0xee, 0x6e, 0x20, 0x61, 0x20, 0x41, 0x77, 0x69, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x79, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x52,
0x6f, 0x6a, 0x61, 0x76, 0x61, 0x79, 0x61, 0x20, 0x4e, 0x61, 0x76, 0xee,
0x6e, 0x20, 0x61, 0x20, 0x41, 0x77, 0x69, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x79, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61,
0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61,
0x76, 0x61, 0x79, 0x61, 0x20, 0x4e, 0x61, 0x76, 0xee, 0x6e, 0x20, 0x61,
0x20, 0x41, 0x77, 0x69, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x41, 0x77, 0x69, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x79, 0x61, 0x79, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x68,
0x69, 0x6c, 0x61, 0x74, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x77, 0x69,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x79, 0x61, 0x20, 0x52, 0x6f,
0x6a, 0x68, 0x69, 0x6c, 0x61, 0x74, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x41, 0x77,
0x69, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x79, 0x61, 0x20, 0x52,
0x6f, 0x6a, 0x68, 0x69, 0x6c, 0x61, 0x74, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x41, 0x77, 0x69, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x79,
0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61,
0x20, 0x41, 0x77, 0x69, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x79,
0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20,
0x41, 0x77, 0x69, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x79, 0x61,
0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65, 0x79, 0x63, 0x61, 0x6e, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65, 0x79,
0x63, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61,
0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x65, 0x79, 0x63, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x41, 0x7a, 0x6f, 0x72, 0x61, 0x6e, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20,
0x41, 0x7a, 0x6f, 0x72, 0x61, 0x6e, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x41, 0x7a,
0x6f, 0x72, 0x61, 0x6e, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x42, 0x65,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x15f, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61,
0x20, 0x42, 0x65, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x15f, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20,
0x79, 0x61, 0x20, 0x42, 0x65, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x15f,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x42, 0xfb, 0x74, 0x61, 0x6e,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x42, 0x6f, 0x6c, 0xee, 0x76,
0x79, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x42, 0x72,
0x61, 0x73, 0xee, 0x6c, 0x79, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61,
0x20, 0x42, 0x72, 0x61, 0x73, 0xee, 0x6c, 0x79, 0x61, 0x79, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20,
0x79, 0x61, 0x20, 0x42, 0x72, 0x61, 0x73, 0xee, 0x6c, 0x79, 0x61, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x42, 0x72, 0xfb, 0x6e, 0x65,
0x79, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x65, 0x6c, 0x61, 0x6d,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x43, 0x61, 0x70, 0x65, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20,
0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x79, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea,
0x20, 0x79, 0x61, 0x20, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72,
0x64, 0x65, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x43, 0x68, 0x61,
0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79,
0x61, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x15e, 0xee, 0x6c, 0xee, 0x79, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x61, 0x20, 0x15e, 0xee, 0x6c, 0xee, 0x79, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61,
0x20, 0x15e, 0xee, 0x6c, 0xee, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0xc7, 0xee, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0xc7, 0xee,
0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee,
0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0xc7, 0xee, 0x6e, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76, 0x61, 0x20, 0x43,
0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76, 0xea, 0x6e, 0x20, 0x43,
0x6f, 0x63, 0x6f, 0x73, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4b,
0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0xee, 0x79, 0x61, 0x79, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x61, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0xee, 0x79, 0x61,
0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee,
0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0xee, 0x79, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x47,
0x69, 0x72, 0x61, 0x76, 0xea, 0x6e, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76, 0xea, 0x6e,
0x20, 0x43, 0x6f, 0x6f, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x4e, 0xee, 0x76, 0x68, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61,
0x20, 0x47, 0x69, 0x72, 0x61, 0x76, 0xea, 0x6e, 0x20, 0x43, 0x6f, 0x6f,
0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4b, 0x75, 0x62, 0x61,
0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e,
0xea, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x79, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x44, 0x61, 0x76, 0xee, 0x73, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d,
0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x79, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x54, 0xee, 0x6d, 0xfb, 0x72, 0x61, 0x20,
0x52, 0x6f, 0x6a, 0x68, 0x69, 0x6c, 0x61, 0x74, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76, 0x61, 0x20, 0x50, 0x61, 0x73,
0x6b, 0x61, 0x6c, 0x79, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20,
0x47, 0x69, 0x72, 0x61, 0x76, 0x61, 0x20, 0x50, 0x61, 0x73, 0x6b, 0x61,
0x6c, 0x79, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48,
0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x47, 0x69, 0x72,
0x61, 0x76, 0x61, 0x20, 0x50, 0x61, 0x73, 0x6b, 0x61, 0x6c, 0x79, 0x61,
0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x45, 0x6b, 0x77, 0x61,
0x64, 0x6f, 0x72, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x45, 0x77,
0x72, 0x6f, 0x70, 0x61, 0x79, 0x61, 0x20, 0x4e, 0x61, 0x76, 0xee, 0x6e,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x61, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x61, 0x79,
0x61, 0x20, 0x4e, 0x61, 0x76, 0xee, 0x6e, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x45,
0x77, 0x72, 0x6f, 0x70, 0x61, 0x79, 0x61, 0x20, 0x4e, 0x61, 0x76, 0xee,
0x6e, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70,
0x61, 0x79, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x68, 0x69, 0x6c, 0x61, 0x74,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x61, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x61, 0x79,
0x61, 0x20, 0x52, 0x6f, 0x6a, 0x68, 0x69, 0x6c, 0x61, 0x74, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79,
0x61, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x61, 0x79, 0x61, 0x20, 0x52,
0x6f, 0x6a, 0x68, 0x69, 0x6c, 0x61, 0x74, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x61, 0x79, 0x61, 0x20, 0x52, 0x6f,
0x6a, 0x68, 0x69, 0x6c, 0x61, 0x74, 0x20, 0x61, 0x20, 0x50, 0xea, 0x15f,
0x74, 0x69, 0x72, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x45, 0x77, 0x72,
0x6f, 0x70, 0x61, 0x79, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x61, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x61, 0x79,
0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20,
0x45, 0x77, 0x72, 0x6f, 0x70, 0x61, 0x79, 0x61, 0x20, 0x52, 0x6f, 0x6a,
0x61, 0x76, 0x61, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x47, 0x69, 0x72,
0x61, 0x76, 0xea, 0x6e, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76,
0xea, 0x6e, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea,
0x20, 0x79, 0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76, 0xea, 0x6e, 0x20,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x46, 0xee, 0x6a, 0xee, 0x79, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x46, 0xee, 0x6a, 0xee, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20,
0x46, 0xee, 0x6a, 0xee, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x47, 0x75, 0x69, 0x79, 0x61, 0x6e, 0x61, 0x79, 0x61, 0x20, 0x46, 0x72,
0x61, 0x6e, 0x73, 0xee, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x41, 0x6e,
0x74, 0x61, 0x72, 0x6b, 0x74, 0xee, 0x6b, 0x61, 0x20, 0xfb, 0x20, 0x42,
0x61, 0x15f, 0xfb, 0x72, 0xea, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x73, 0x61,
0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x47, 0x61, 0x6c, 0x61,
0x70, 0x61, 0x67, 0x6f, 0x73, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x47, 0x75, 0x72, 0x63, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x61, 0x20, 0x47, 0x75, 0x72, 0x63, 0x69, 0x73, 0x74,
0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76,
0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x47, 0x75, 0x72, 0x63, 0x69,
0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x47,
0x69, 0x72, 0x61, 0x76, 0xea, 0x6e, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65,
0x72, 0x74, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4e, 0x61, 0x76,
0xee, 0x6e, 0xee, 0x20, 0x79, 0x61, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x77, 0x69, 0x63, 0x68, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x47,
0x72, 0xee, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x52, 0x6f, 0x6a,
0x68, 0x69, 0x6c, 0x61, 0x74, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x47, 0x72,
0xee, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x68,
0x69, 0x6c, 0x61, 0x74, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61,
0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x47, 0x72, 0xee, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x68, 0x69, 0x6c,
0x61, 0x74, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x47, 0x72, 0xee, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x61, 0x20, 0x47, 0x72, 0xee, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61,
0x20, 0x47, 0x72, 0xee, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x52,
0x6f, 0x6a, 0x61, 0x76, 0x61, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x4b, 0x65,
0x6e, 0x64, 0x61, 0x76, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x47,
0x75, 0x79, 0x61, 0x6e, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x69, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x48, 0x61,
0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61,
0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee,
0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e,
0x67, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6e, 0x67, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48,
0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x48, 0x6f, 0x6e,
0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x48, 0x6f, 0x76, 0x64, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x48,
0x6f, 0x76, 0x64, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61,
0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x48, 0x6f, 0x76, 0x64,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x48, 0x69, 0x6e, 0x64, 0x69, 0x73,
0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4f, 0x6b,
0x79, 0x61, 0x6e, 0xfb, 0x73, 0x61, 0x20, 0x48, 0x69, 0x6e, 0x64, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x69, 0x6e, 0x64, 0x69, 0xe7,
0xee, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x45, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x7a, 0x79, 0x61, 0x79, 0x61, 0x20, 0x4e, 0x61, 0x76,
0xee, 0x6e, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x45, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x7a, 0x79, 0x61, 0x79, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x68,
0x69, 0x6c, 0x61, 0x74, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x45, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x79, 0x61, 0x79, 0x61, 0x20, 0x52, 0x6f,
0x6a, 0x61, 0x76, 0x61, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0xce, 0x72,
0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0xce, 0x72, 0x61, 0x6e,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e,
0xea, 0x20, 0x79, 0x61, 0x20, 0xce, 0x72, 0x61, 0x6e, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x61, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e,
0xea, 0x20, 0x79, 0x61, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0xce, 0x73, 0x72, 0x61, 0xee,
0x6c, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0xce, 0x73, 0x72, 0x61, 0xee,
0x6c, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee,
0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0xce, 0x73, 0x72, 0x61, 0xee, 0x6c,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e,
0x79, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x4a, 0x61, 0x70,
0x6f, 0x6e, 0x79, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x48, 0x61, 0x76, 0x15f, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x61,
0x70, 0x6f, 0x6e, 0x79, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x51, 0x61, 0x7a, 0x61, 0x78, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x51, 0x61, 0x7a, 0x61, 0x78, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x68, 0x69, 0x6c,
0x61, 0x74, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x51, 0x61, 0x7a, 0x61,
0x78, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61,
0x76, 0x61, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4b, 0x6f, 0x72, 0x65,
0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e,
0xea, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x79, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea,
0x20, 0x79, 0x61, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x73, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x51, 0x69,
0x72, 0x78, 0x69, 0x7a, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76, 0xea, 0x6e, 0x20,
0x4c, 0xee, 0x6e, 0x65, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x79, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76,
0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61,
0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61,
0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x4d, 0x61, 0x6c, 0x65, 0x7a, 0x79, 0x61, 0x79, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0xee, 0x76,
0x61, 0x6e, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4d, 0x61, 0x72, 0x71,
0x75, 0x65, 0x73, 0x61, 0x73, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x47, 0x69, 0x72, 0x61, 0x76, 0xea, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x15f,
0x61, 0x6c, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79,
0x61, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x50, 0x61, 0x73, 0xee, 0x66,
0xee, 0x6b, 0x61, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0xee, 0x6b, 0x61, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x50, 0x61, 0x73, 0xee, 0x66, 0xee,
0x6b, 0x61, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0xee, 0x6b, 0x61, 0x79, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea,
0x20, 0x79, 0x61, 0x20, 0x50, 0x61, 0x73, 0xee, 0x66, 0xee, 0x6b, 0x61,
0x20, 0x4d, 0x65, 0x6b, 0x73, 0xee, 0x6b, 0x61, 0x79, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0xdb, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x61,
0x72, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0xdb, 0x6c, 0x61, 0x6e, 0x62,
0x61, 0x74, 0x61, 0x72, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48,
0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0xdb, 0x6c, 0x61,
0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x61, 0x79, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x61, 0x20, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x61, 0x79, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea,
0x20, 0x79, 0x61, 0x20, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x61, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4d, 0x79, 0x61, 0x6e, 0x6d,
0x61, 0x72, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4e, 0x61, 0xfb,
0x72, 0xfb, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4e, 0x65,
0x70, 0x61, 0x6c, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4b, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x79, 0x61, 0x79, 0x61, 0x20, 0x4e, 0xfb,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x61, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x79, 0x61, 0x79, 0x61, 0x20, 0x4e, 0xfb, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x4b,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x79, 0x61, 0x79, 0x61, 0x20, 0x4e,
0xfb, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e,
0x64, 0x61, 0x79, 0x61, 0x20, 0x4e, 0xfb, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20,
0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x79, 0x61, 0x20, 0x4e, 0xfb,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea,
0x20, 0x79, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x79,
0x61, 0x20, 0x4e, 0xfb, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4e, 0x65,
0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48,
0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x4e, 0x65, 0x77,
0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x79, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76, 0x61, 0x20, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20,
0x47, 0x69, 0x72, 0x61, 0x76, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76,
0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76,
0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76,
0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76,
0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x4f, 0x6d, 0x73, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x4f,
0x6d, 0x73, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61,
0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x4f, 0x6d, 0x73, 0x6b,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x47, 0xee, 0x6e, 0x65, 0x79, 0x61, 0x20, 0x4e, 0xfb, 0x20,
0x79, 0x61, 0x20, 0x50, 0x61, 0x70, 0xfb, 0x61, 0x79, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0xfb, 0x61, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0xfb,
0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76,
0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67,
0xfb, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x50, 0x65,
0x72, 0xfb, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x50, 0x65, 0x72,
0xfb, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76,
0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x50, 0x65, 0x72, 0xfb, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x46, 0xee, 0x6c, 0xee, 0x70,
0xee, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x46, 0xee, 0x6c, 0xee,
0x70, 0xee, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61,
0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x46, 0xee, 0x6c, 0xee,
0x70, 0xee, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x47, 0x69,
0x72, 0x61, 0x76, 0xea, 0x6e, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0xee,
0x6b, 0x73, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x61, 0x69, 0x6e,
0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0xfb, 0x20, 0x4d,
0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61,
0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x20, 0xfb, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e,
0xea, 0x20, 0x79, 0x61, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0xfb, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x50,
0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x79, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e,
0x67, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x52, 0xe9, 0x75, 0x6e,
0x69, 0x6f, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x52, 0x6f,
0x74, 0x68, 0x65, 0x72, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x53, 0x61, 0x78, 0x61, 0x6c, 0xee, 0x6e, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x53, 0x61, 0x78, 0x61, 0x6c, 0xee, 0x6e, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79,
0x61, 0x20, 0x53, 0x61, 0x78, 0x61, 0x6c, 0xee, 0x6e, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x79, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x61, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x79, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20,
0x79, 0x61, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x79, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x53, 0x65, 0x79, 0x15f, 0x65, 0x6c, 0x65, 0x72,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x53, 0xee, 0x6e, 0x67, 0x61, 0x70,
0xfb, 0x72, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x47, 0x69, 0x72,
0x61, 0x76, 0xea, 0x6e, 0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x61, 0x79, 0x61, 0x20, 0x42, 0x61, 0x15f, 0xfb, 0x72, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x53, 0x75, 0x72, 0xee, 0x6e, 0x61, 0x6d, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x54, 0x61, 0x68, 0xee, 0x74,
0xee, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x54, 0x61, 0xee,
0x70, 0x65, 0xee, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x54, 0x61,
0xee, 0x70, 0x65, 0xee, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x54, 0x61,
0xee, 0x70, 0x65, 0xee, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x54, 0x61, 0x63, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75,
0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x54, 0x6f, 0x6e, 0x67,
0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x54, 0x6f, 0x6e, 0x67,
0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76,
0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x43, 0x68, 0x75, 0x75,
0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x54, 0x69, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x54, 0x69, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76,
0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x54, 0x69, 0x72, 0x6b, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x54, 0xfb, 0x76, 0x61, 0x6c, 0xfb, 0x79, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0xdb, 0x72, 0xfb, 0x67, 0xfb, 0x61, 0x79, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x61, 0x20, 0xdb, 0x72, 0xfb, 0x67, 0xfb, 0x61, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e,
0xea, 0x20, 0x79, 0x61, 0x20, 0xdb, 0x72, 0xfb, 0x67, 0xfb, 0x61, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4f, 0x7a, 0x62, 0x65, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x4f,
0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79,
0x61, 0x20, 0x4f, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x56, 0x61, 0x6e, 0xfb, 0x61,
0x74, 0xfb, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x56, 0x61, 0x6e,
0xfb, 0x61, 0x74, 0xfb, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x56, 0x61,
0x6e, 0xfb, 0x61, 0x74, 0xfb, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0xfb, 0x65, 0x6c, 0x61, 0x79, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20,
0x79, 0x61, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79,
0x61, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76,
0x61, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0xfb, 0x20, 0x46,
0x75, 0x74, 0x75, 0x6e, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79,
0x61, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x59,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e,
0xea, 0x20, 0x79, 0x61, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0xea, 0x410, 0x444, 0x433, 0x430, 0x43d,
0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x411, 0x43e, 0x440, 0x431, 0x43e, 0x440, 0x434, 0x443, 0x43a, 0x20,
0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x427, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x410, 0x444, 0x440,
0x438, 0x43a, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x422, 0x4af, 0x448, 0x442, 0x4af, 0x43a, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a,
0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411, 0x430,
0x442, 0x44b, 0x448, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x448,
0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x43a, 0x44b, 0x448, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x442,
0x44b, 0x448, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x436, 0x430,
0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x20, 0x43a, 0x44b,
0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x43c, 0x430,
0x437, 0x43e, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x43c, 0x430,
0x437, 0x43e, 0x43d, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x431, 0x43e,
0x440, 0x431, 0x43e, 0x440, 0x434, 0x443, 0x43a, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x44b, 0x442, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a, 0x20, 0x410, 0x43c, 0x435,
0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x431, 0x43e, 0x440, 0x431, 0x43e, 0x440,
0x434, 0x443, 0x43a, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x431, 0x43e,
0x440, 0x431, 0x43e, 0x440, 0x434, 0x443, 0x43a, 0x20, 0x436, 0x430, 0x439, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x4af,
0x43d, 0x434, 0x4af, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430,
0x2c, 0x20, 0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x447, 0x44b, 0x433, 0x44b,
0x448, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x447, 0x44b, 0x433, 0x44b,
0x448, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x442, 0x43e, 0x43e, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x4af, 0x43d, 0x434,
0x4af, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20,
0x442, 0x43e, 0x43e, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x442, 0x43e,
0x43e, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x422, 0x44b, 0x43d, 0x447,
0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a, 0x20, 0x410, 0x43c,
0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x422, 0x44b, 0x43d, 0x447, 0x20,
0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x4af, 0x43d, 0x434,
0x4af, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20,
0x422, 0x44b, 0x43d, 0x447, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x20, 0x436,
0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x410, 0x43f, 0x438, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x410, 0x43f, 0x438, 0x430, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x43f, 0x438,
0x430, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x410, 0x440, 0x430, 0x431, 0x438, 0x44f, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x440, 0x430, 0x431, 0x438,
0x44f, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x410, 0x440, 0x430, 0x431, 0x438, 0x44f, 0x20, 0x436, 0x430,
0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410,
0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438,
0x43d, 0x430, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438,
0x43d, 0x430, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x410,
0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x410,
0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x43a, 0x44b, 0x448,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411,
0x430, 0x442, 0x44b, 0x448, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438,
0x43d, 0x430, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x44f,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x440, 0x43c,
0x435, 0x43d, 0x438, 0x44f, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x438,
0x44f, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a,
0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x442,
0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x430, 0x20, 0x43a, 0x44b, 0x448, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x442,
0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x430, 0x20, 0x436, 0x430, 0x439, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x431, 0x43e, 0x440, 0x431,
0x43e, 0x440, 0x434, 0x443, 0x43a, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20,
0x431, 0x43e, 0x440, 0x431, 0x43e, 0x440, 0x434, 0x443, 0x43a, 0x20, 0x43a, 0x44b,
0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x431, 0x43e, 0x440,
0x431, 0x43e, 0x440, 0x434, 0x443, 0x43a, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x431, 0x43e, 0x440, 0x431, 0x43e,
0x440, 0x434, 0x443, 0x43a, 0x20, 0x431, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x432, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x44f, 0x20, 0x431, 0x43e, 0x440, 0x431, 0x43e, 0x440, 0x434,
0x443, 0x43a, 0x20, 0x431, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x43a, 0x44b, 0x448,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x431, 0x43e, 0x440, 0x431,
0x43e, 0x440, 0x434, 0x443, 0x43a, 0x20, 0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20,
0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20,
0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20,
0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x432, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x44f, 0x20, 0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x436,
0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x431,
0x430, 0x442, 0x44b, 0x448, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x431,
0x430, 0x442, 0x44b, 0x448, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x44f, 0x20, 0x431, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x436, 0x430,
0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x436, 0x430, 0x43d, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x437, 0x435, 0x440, 0x431,
0x430, 0x439, 0x436, 0x430, 0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x437, 0x435, 0x440, 0x431,
0x430, 0x439, 0x436, 0x430, 0x43d, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x437, 0x43e, 0x440,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x437, 0x43e,
0x440, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x20, 0x436, 0x430, 0x439,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411,
0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434,
0x435, 0x448, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435,
0x448, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x411, 0x443, 0x442, 0x430, 0x43d, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411, 0x43e, 0x43b, 0x438, 0x432, 0x438,
0x44f, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411, 0x440,
0x430, 0x437, 0x438, 0x43b, 0x438, 0x44f, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x44f, 0x20,
0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x44f, 0x20, 0x436,
0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x411, 0x440, 0x443, 0x43d, 0x435, 0x439, 0x20, 0x414, 0x430, 0x440, 0x443,
0x441, 0x441, 0x430, 0x43b, 0x430, 0x43c, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x41a, 0x430, 0x43f, 0x435, 0x20, 0x412, 0x435, 0x440, 0x434,
0x435, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x430,
0x43f, 0x435, 0x20, 0x412, 0x435, 0x440, 0x434, 0x435, 0x20, 0x43a, 0x44b, 0x448,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x430,
0x43f, 0x435, 0x20, 0x412, 0x435, 0x440, 0x434, 0x435, 0x20, 0x436, 0x430, 0x439,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x427,
0x430, 0x43c, 0x43e, 0x440, 0x440, 0x43e, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x427, 0x430, 0x442, 0x430, 0x43c, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x427, 0x430, 0x442, 0x430, 0x43c, 0x20, 0x43a,
0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x427,
0x430, 0x442, 0x430, 0x43c, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x427, 0x438, 0x43b, 0x438, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x427, 0x438, 0x43b, 0x438,
0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x427, 0x438, 0x43b, 0x438, 0x20, 0x436, 0x430, 0x439, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x44b,
0x442, 0x430, 0x439, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x41a, 0x44b, 0x442, 0x430, 0x439, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x44b, 0x442, 0x430, 0x439,
0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x41a, 0x440, 0x438, 0x441, 0x43c, 0x430, 0x441, 0x20, 0x430, 0x440,
0x430, 0x43b, 0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x20, 0x430, 0x440, 0x430,
0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438,
0x44f, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x43e,
0x43b, 0x443, 0x43c, 0x431, 0x438, 0x44f, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x43e, 0x43b,
0x443, 0x43c, 0x431, 0x438, 0x44f, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x443, 0x43a, 0x20,
0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x43d, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x443, 0x43a, 0x20,
0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x43d, 0x20,
0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x41a, 0x443, 0x43a, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430,
0x440, 0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x436, 0x430, 0x440, 0x44b, 0x43c, 0x20,
0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x41a, 0x443, 0x431, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x41a, 0x443, 0x431, 0x430, 0x20, 0x43a, 0x44b, 0x448, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x443,
0x431, 0x430, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x414, 0x44d, 0x432, 0x438, 0x441, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x414, 0x44e, 0x43c, 0x43e, 0x43d,
0x2d, 0x434, 0x2d, 0x423, 0x440, 0x432, 0x438, 0x43b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x427, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x422,
0x438, 0x43c, 0x43e, 0x440, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x418, 0x441, 0x442, 0x435, 0x440, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x44b,
0x43d, 0x44b, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x418, 0x441, 0x442, 0x435, 0x440, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x44b, 0x43d,
0x44b, 0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x418, 0x441, 0x442, 0x435, 0x440, 0x20, 0x430, 0x440,
0x430, 0x43b, 0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x42d, 0x43a, 0x443, 0x430,
0x434, 0x43e, 0x440, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x411, 0x43e, 0x440, 0x431, 0x43e, 0x440, 0x434, 0x443, 0x43a, 0x20, 0x415, 0x432,
0x440, 0x43e, 0x43f, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x411, 0x43e, 0x440, 0x431, 0x43e, 0x440, 0x434, 0x443, 0x43a, 0x20, 0x415,
0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x411, 0x43e, 0x440, 0x431, 0x43e,
0x440, 0x434, 0x443, 0x43a, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20,
0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x427, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x415, 0x432, 0x440, 0x43e,
0x43f, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x427,
0x44b, 0x433, 0x44b, 0x448, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20,
0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x427, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f,
0x430, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x430, 0x43b, 0x438, 0x43d, 0x438, 0x43d, 0x433,
0x440, 0x430, 0x434, 0x20, 0x436, 0x430, 0x43d, 0x430, 0x20, 0x41c, 0x438, 0x43d,
0x441, 0x43a, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411,
0x430, 0x442, 0x44b, 0x448, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411, 0x430, 0x442, 0x44b,
0x448, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x43a, 0x44b, 0x448,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x411, 0x430,
0x442, 0x44b, 0x448, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x436,
0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x430, 0x440,
0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435,
0x43d, 0x434, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d,
0x44b, 0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434,
0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x43d,
0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x424, 0x438, 0x436, 0x438, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x424, 0x438, 0x436, 0x438, 0x20, 0x43a, 0x44b, 0x448,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x424, 0x438,
0x436, 0x438, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x443, 0x437,
0x20, 0x413, 0x432, 0x438, 0x430, 0x43d, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x443, 0x437, 0x20,
0x422, 0x4af, 0x448, 0x442, 0x4af, 0x433, 0x4af, 0x20, 0x436, 0x430, 0x43d, 0x430,
0x20, 0x410, 0x43d, 0x442, 0x430, 0x440, 0x43a, 0x442, 0x438, 0x43a, 0x430, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x413, 0x430, 0x43b, 0x430,
0x43f, 0x430, 0x433, 0x43e, 0x441, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x413, 0x430, 0x43c, 0x431, 0x438, 0x435, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x413, 0x440, 0x443, 0x437, 0x438, 0x44f, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x413, 0x440, 0x443, 0x437,
0x438, 0x44f, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x413, 0x440, 0x443, 0x437, 0x438, 0x44f, 0x20, 0x436,
0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x413, 0x438, 0x43b, 0x431, 0x435, 0x440, 0x442, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x413, 0x440, 0x438, 0x43d, 0x432, 0x438, 0x447,
0x20, 0x431, 0x43e, 0x44e, 0x43d, 0x447, 0x430, 0x20, 0x43e, 0x440, 0x442, 0x43e,
0x447, 0x43e, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x427, 0x44b, 0x433,
0x44b, 0x448, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438,
0x44f, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x427, 0x44b,
0x433, 0x44b, 0x448, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434,
0x438, 0x44f, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x427, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x413,
0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x436, 0x430,
0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x411, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430,
0x43d, 0x434, 0x438, 0x44f, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x411, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b,
0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411, 0x430, 0x442, 0x44b,
0x448, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f,
0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x411, 0x443, 0x43b, 0x443, 0x4a3, 0x434, 0x443, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x43a, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x413, 0x430, 0x432, 0x430, 0x439,
0x438, 0x2d, 0x410, 0x43b, 0x435, 0x443, 0x442, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x413, 0x430, 0x432, 0x430, 0x439, 0x438, 0x2d, 0x410,
0x43b, 0x435, 0x443, 0x442, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x413, 0x430, 0x432, 0x430, 0x439,
0x438, 0x2d, 0x410, 0x43b, 0x435, 0x443, 0x442, 0x20, 0x436, 0x430, 0x439, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x413, 0x43e,
0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x20, 0x43a, 0x44b,
0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x413,
0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x425, 0x43e, 0x432,
0x434, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x425, 0x43e,
0x432, 0x434, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x425, 0x43e, 0x432, 0x434, 0x20, 0x436, 0x430, 0x439,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x418,
0x43d, 0x434, 0x438, 0x44f, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x418, 0x43d, 0x434, 0x438, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x418, 0x43d, 0x434, 0x43e,
0x43a, 0x44b, 0x442, 0x430, 0x439, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x411, 0x43e, 0x440, 0x431, 0x43e, 0x440, 0x434, 0x443, 0x43a, 0x20,
0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x44f, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x427, 0x44b, 0x433, 0x44b, 0x448, 0x20,
0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x44f, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x448, 0x20,
0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x44f, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x418, 0x440, 0x430, 0x43d, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x418, 0x440, 0x430, 0x43d, 0x20,
0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x418, 0x440, 0x430, 0x43d, 0x20, 0x43a, 0x4af, 0x43d, 0x434, 0x4af, 0x437,
0x433, 0x4af, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x418,
0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x43a,
0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x436, 0x430, 0x439, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x418, 0x437, 0x440,
0x430, 0x438, 0x43b, 0x44c, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x418, 0x437, 0x440, 0x430, 0x438, 0x43b, 0x44c, 0x20, 0x43a, 0x44b, 0x448, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x418, 0x437, 0x440,
0x430, 0x438, 0x43b, 0x44c, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x416, 0x430, 0x43f, 0x43e, 0x43d, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x416, 0x430, 0x43f, 0x43e,
0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x416, 0x430, 0x43f, 0x43e, 0x43d, 0x20, 0x436, 0x430, 0x439,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a,
0x430, 0x437, 0x430, 0x43a, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x427, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x41a,
0x430, 0x437, 0x430, 0x43a, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x41a,
0x430, 0x437, 0x430, 0x43a, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x43e, 0x440, 0x435, 0x44f, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x43e, 0x440, 0x435, 0x44f,
0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x41a, 0x43e, 0x440, 0x435, 0x44f, 0x20, 0x436, 0x430, 0x439, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x43e,
0x441, 0x440, 0x430, 0x435, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x440, 0x430, 0x441,
0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x440, 0x430, 0x441,
0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x44b, 0x440,
0x433, 0x44b, 0x437, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x41b, 0x430, 0x439, 0x43d, 0x20, 0x430, 0x440, 0x430,
0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x430,
0x443, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41b, 0x43e,
0x440, 0x434, 0x20, 0x425, 0x430, 0x443, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41b, 0x43e, 0x440, 0x434,
0x20, 0x425, 0x430, 0x443, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41c, 0x430, 0x433, 0x430, 0x434,
0x430, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41c,
0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x433, 0x430,
0x434, 0x430, 0x43d, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41c, 0x430, 0x43b, 0x430, 0x439, 0x437,
0x438, 0x44f, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41c,
0x430, 0x43b, 0x44c, 0x434, 0x438, 0x432, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x41c, 0x430, 0x440, 0x43a, 0x435, 0x437, 0x430, 0x441, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41c, 0x430, 0x440, 0x448,
0x430, 0x43b, 0x43b, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b,
0x43d, 0x44b, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438, 0x439, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438,
0x439, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438, 0x439, 0x20,
0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x41c, 0x43e, 0x443, 0x441, 0x43e, 0x43d, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430,
0x2c, 0x20, 0x422, 0x44b, 0x43d, 0x447, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41c, 0x435, 0x43a,
0x441, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x422, 0x44b, 0x43d, 0x447, 0x20, 0x43e,
0x43a, 0x435, 0x430, 0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41c, 0x435, 0x43a, 0x441, 0x438,
0x43a, 0x430, 0x2c, 0x20, 0x422, 0x44b, 0x43d, 0x447, 0x20, 0x43e, 0x43a, 0x435,
0x430, 0x43d, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x423, 0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430,
0x442, 0x43e, 0x440, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x423, 0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x20, 0x43a,
0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x423, 0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x20, 0x436,
0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x20, 0x43a,
0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41c, 0x439, 0x430,
0x43d, 0x43c, 0x430, 0x440, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x41d, 0x430, 0x443, 0x440, 0x443, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x41d, 0x435, 0x43f, 0x430, 0x43b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x416, 0x430, 0x4a3, 0x44b, 0x20, 0x41a, 0x430,
0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x44f, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x416, 0x430, 0x4a3, 0x44b, 0x20, 0x41a, 0x430, 0x43b,
0x435, 0x434, 0x43e, 0x43d, 0x438, 0x44f, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x416, 0x430, 0x4a3,
0x44b, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x44f, 0x20,
0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x416, 0x430, 0x4a3, 0x44b, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d,
0x434, 0x438, 0x44f, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x416, 0x430, 0x4a3, 0x44b, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438,
0x44f, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x416, 0x430, 0x4a3, 0x44b, 0x20, 0x417, 0x435, 0x43b, 0x430,
0x43d, 0x434, 0x438, 0x44f, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41d, 0x44e, 0x444, 0x430, 0x443, 0x43d,
0x434, 0x43b, 0x44d, 0x43d, 0x434, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x41d, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x44d, 0x43d,
0x434, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x41d, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b,
0x44d, 0x43d, 0x434, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41d, 0x438, 0x443, 0x44d, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41d, 0x43e, 0x440, 0x444, 0x43e,
0x43b, 0x43a, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41d,
0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41d, 0x43e, 0x440,
0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x424, 0x435, 0x440, 0x43d, 0x430,
0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x43d,
0x44c, 0x44f, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x424,
0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d,
0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x424, 0x435, 0x440,
0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440,
0x43e, 0x43d, 0x44c, 0x44f, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41d, 0x43e, 0x432, 0x43e, 0x441,
0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440,
0x441, 0x43a, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438,
0x440, 0x441, 0x43a, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41e, 0x43c, 0x441, 0x43a, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41e, 0x43c, 0x441, 0x43a, 0x20,
0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x41e, 0x43c, 0x441, 0x43a, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41f, 0x430, 0x43a, 0x438,
0x441, 0x442, 0x430, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x43a, 0x44b,
0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41f,
0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x436, 0x430, 0x439, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41f, 0x430,
0x43b, 0x430, 0x443, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x41f, 0x430, 0x43f, 0x443, 0x430, 0x2d, 0x416, 0x430, 0x4a3, 0x44b, 0x20, 0x413,
0x432, 0x438, 0x43d, 0x435, 0x44f, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432,
0x430, 0x439, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430,
0x439, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x41f, 0x435, 0x440, 0x443, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41f, 0x435, 0x440, 0x443, 0x20, 0x43a, 0x44b,
0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x41f, 0x435, 0x440, 0x443, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x424, 0x438, 0x43b, 0x438, 0x43f,
0x43f, 0x438, 0x43d, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b,
0x43d, 0x44b, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x20, 0x430, 0x440, 0x430,
0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x43a, 0x44b, 0x448,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x424,
0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x20, 0x430, 0x440, 0x430, 0x43b,
0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x436, 0x430, 0x439, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x424, 0x435,
0x43d, 0x438, 0x43a, 0x441, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440,
0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x421, 0x435, 0x43d, 0x20, 0x41f, 0x44c, 0x435, 0x440, 0x20, 0x436, 0x430,
0x43d, 0x430, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x421, 0x435, 0x43d, 0x20, 0x41f,
0x44c, 0x435, 0x440, 0x20, 0x436, 0x430, 0x43d, 0x430, 0x20, 0x41c, 0x438, 0x43a,
0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x421, 0x435, 0x43d, 0x20, 0x41f,
0x44c, 0x435, 0x440, 0x20, 0x436, 0x430, 0x43d, 0x430, 0x20, 0x41c, 0x438, 0x43a,
0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41f, 0x438, 0x442, 0x43a, 0x44d,
0x440, 0x43d, 0x433, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x41f, 0x43e, 0x43d, 0x430, 0x43f, 0x435, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x41f, 0x445, 0x435, 0x43d, 0x44c, 0x44f, 0x43d, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x420, 0x435, 0x44e, 0x43d, 0x438, 0x43e,
0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x420, 0x43e,
0x442, 0x435, 0x440, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d,
0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20, 0x436, 0x430,
0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x421, 0x430, 0x43c, 0x43e, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x20, 0x43a, 0x44b, 0x448, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x421, 0x430,
0x43c, 0x43e, 0x430, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x421, 0x435, 0x439, 0x448, 0x435, 0x43b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x421, 0x438, 0x43d,
0x433, 0x430, 0x43f, 0x443, 0x440, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x421, 0x43e, 0x43b, 0x43e, 0x43c, 0x43e, 0x43d, 0x20, 0x430, 0x440,
0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x4af, 0x448, 0x442, 0x4af, 0x43a,
0x20, 0x416, 0x43e, 0x440, 0x436, 0x438, 0x44f, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x421, 0x443, 0x440, 0x438, 0x43d, 0x430, 0x43c, 0x435,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x421, 0x430, 0x43e,
0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x430,
0x438, 0x442, 0x438, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x422, 0x430, 0x439, 0x43f, 0x435, 0x439, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x422, 0x430, 0x439, 0x43f, 0x435, 0x439, 0x20, 0x43a, 0x44b,
0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422,
0x430, 0x439, 0x43f, 0x435, 0x439, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x430, 0x436, 0x438, 0x43a,
0x441, 0x442, 0x430, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x422, 0x43e, 0x43a, 0x435, 0x43b, 0x430, 0x443, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x43e, 0x43d, 0x433, 0x430,
0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x436, 0x430, 0x439,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x427,
0x443, 0x443, 0x43a, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x422, 0x4af, 0x440, 0x43a, 0x43c, 0x4e9, 0x43d, 0x441, 0x442, 0x430, 0x43d, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x4af, 0x440, 0x43a,
0x43c, 0x4e9, 0x43d, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x4af, 0x440,
0x43a, 0x43c, 0x4e9, 0x43d, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x436, 0x430, 0x439,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422,
0x443, 0x432, 0x430, 0x43b, 0x443, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430,
0x439, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x20,
0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x4e8, 0x437, 0x431, 0x435, 0x43a, 0x441, 0x442, 0x430, 0x43d, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x4e8, 0x437, 0x431, 0x435,
0x43a, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x4e8, 0x437, 0x431, 0x435, 0x43a,
0x441, 0x442, 0x430, 0x43d, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x412, 0x430, 0x43d, 0x443, 0x430,
0x442, 0x443, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x412,
0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x412, 0x430, 0x43d, 0x443,
0x430, 0x442, 0x443, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x412, 0x435, 0x43d, 0x435, 0x441, 0x443,
0x44d, 0x43b, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x412, 0x43b, 0x430, 0x434,
0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x43a, 0x44b, 0x448, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x412, 0x43b, 0x430,
0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x436, 0x430, 0x439,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x412,
0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440,
0x430, 0x434, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430,
0x434, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x423, 0x435, 0x439, 0x43a, 0x20,
0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x43d, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x423, 0x43e, 0x43b, 0x438,
0x441, 0x20, 0x436, 0x430, 0x43d, 0x430, 0x20, 0x424, 0x443, 0x442, 0x443, 0x43d,
0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x42f, 0x43a,
0x443, 0x442, 0x441, 0x43a, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x43a, 0x44b, 0x448, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x42f, 0x43a, 0x443,
0x442, 0x441, 0x43a, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440,
0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431,
0x443, 0x440, 0x433, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438,
0x43d, 0x431, 0x443, 0x440, 0x433, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x42e, 0x43a, 0x43e, 0x43d,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0xec0, 0xea7, 0xea5,
0xeb2, 0xe82, 0xead, 0xe87, 0xead, 0xeb2, 0xec0, 0xe81, 0xea3, 0xec0, 0xea7, 0xea5,
0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xe82, 0xead, 0xe87,
0xead, 0xeb2, 0xec0, 0xe81, 0xea3, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94,
0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xe82, 0xead, 0xe87, 0xead, 0xeb2, 0xec0, 0xe81,
0xea3, 0xec0, 0xea7, 0xea5, 0xeb2, 0x20, 0xead, 0xeb1, 0xe9f, 0xe81, 0xeb2, 0xe99,
0xeb4, 0xeaa, 0xe96, 0xeb2, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xead,
0xeb2, 0x200b, 0xe9f, 0xea3, 0xeb4, 0x200b, 0xe81, 0xeb2, 0x200b, 0xe81, 0xeb2, 0xe87,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xead, 0xeb2, 0x200b, 0xe9f, 0xea3, 0xeb4,
0x200b, 0xe81, 0xeb2, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b,
0xead, 0xead, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xead, 0xeb2, 0x200b,
0xe9f, 0xea3, 0xeb4, 0x200b, 0xe81, 0xeb2, 0x200b, 0xec3, 0xe95, 0xec9, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0xead, 0xeb2, 0x200b, 0xe9f, 0xea3, 0xeb4, 0x200b, 0xe81,
0xeb2, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xe95, 0xebb,
0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95,
0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xead, 0xeb2, 0x200b, 0xe9f, 0xea3, 0xeb4,
0x200b, 0xe81, 0xeb2, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b,
0xe95, 0xebb, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b,
0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0x200b, 0xead, 0xeb2, 0x200b, 0xe9f,
0xea3, 0xeb4, 0x200b, 0xe81, 0xeb2, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1,
0xe99, 0x200b, 0xe95, 0xebb, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xead, 0xeb0, 0xea5,
0xeb1, 0xeaa, 0xe81, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95,
0xeb0, 0xe96, 0xeb2, 0xe99, 0xead, 0xeb0, 0xea5, 0xeb1, 0xeaa, 0xe81, 0xeb2, 0xec0,
0xea7, 0xea5, 0xeb2, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0xeb1, 0xe99, 0xead, 0xeb0,
0xea5, 0xeb1, 0xeaa, 0xe81, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xead, 0xecd, 0xea1,
0xeb2, 0xe95, 0xeb5, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0,
0xe96, 0xeb2, 0xe99, 0xead, 0xecd, 0xea1, 0xeb2, 0xe95, 0xeb5, 0xec0, 0xea7, 0xea5,
0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xead, 0xecd, 0xea1,
0xeb2, 0xe95, 0xeb5, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe95, 0xeb2, 0xea1, 0xec0, 0xe82,
0xe94, 0xead, 0xeb2, 0xec0, 0xea1, 0xe8a, 0xead, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99,
0xead, 0xeb2, 0x200b, 0xec0, 0xea1, 0x200b, 0xe8a, 0xead, 0xe99, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead,
0xe99, 0xead, 0xeb2, 0x200b, 0xec0, 0xea1, 0x200b, 0xe8a, 0xead, 0xe99, 0xec0, 0xea7,
0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xe81, 0xeb2,
0xe87, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0xeb1, 0xe99,
0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe95, 0xeb2, 0xec0, 0xea7, 0xeb1,
0xe99, 0xead, 0xead, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95,
0xeb0, 0xe96, 0xeb2, 0xe99, 0xe95, 0xeb2, 0xec0, 0xea7, 0xeb1, 0xe99, 0xead, 0xead,
0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0xeb1, 0xe99,
0xe95, 0xeb2, 0xec0, 0xea7, 0xeb1, 0xe99, 0xead, 0xead, 0xe81, 0xec0, 0xea7, 0xea5,
0xeb2, 0xec1, 0xe96, 0xe9a, 0xe9e, 0xeb9, 0xec0, 0xe82, 0xebb, 0xeb2, 0xec0, 0xea7,
0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xec1, 0xe96,
0xe9a, 0xe9e, 0xeb9, 0xec0, 0xe82, 0xebb, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe81,
0xeb2, 0xe87, 0xec0, 0xea7, 0xeb1, 0xe99, 0xec1, 0xe96, 0xe9a, 0xe9e, 0xeb9, 0xec0,
0xe82, 0xebb, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xec1, 0xe9b, 0xe8a, 0xeb4, 0xe9f,
0xeb4, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96,
0xeb2, 0xe99, 0xec1, 0xe9b, 0xe8a, 0xeb4, 0xe9f, 0xeb4, 0xe81, 0xec0, 0xea7, 0xea5,
0xeb2, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0xeb1, 0xe99, 0xec1, 0xe9b, 0xe8a, 0xeb4,
0xe9f, 0xeb4, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xec0, 0xead, 0xec0, 0xe9e, 0xe8d,
0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99,
0xec0, 0xead, 0xec0, 0xe9e, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe81, 0xeb2, 0xe87,
0xec0, 0xea7, 0xeb1, 0xe99, 0xead, 0xeb2, 0xec0, 0xe9e, 0xe8d, 0xec0, 0xea7, 0xea5,
0xeb2, 0xead, 0xeb1, 0xe94, 0xe95, 0xeb2, 0xead, 0xeb9, 0xec0, 0xea7, 0xea5, 0xeb2,
0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xead, 0xeb1, 0xe94, 0xe95,
0xeb2, 0xead, 0xeb9, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae,
0xec9, 0xead, 0xe99, 0xead, 0xeb1, 0xe94, 0xe95, 0xeb2, 0xead, 0xeb9, 0xec0, 0xea7,
0xea5, 0xeb2, 0xead, 0xeb1, 0xe94, 0xec2, 0xe95, 0xec0, 0xe9a, 0xec0, 0xea7, 0xea5,
0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xead, 0xeb1, 0xe94,
0xec2, 0xe95, 0xec0, 0xe9a, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9,
0xeae, 0xec9, 0xead, 0xe99, 0xead, 0xeb1, 0xe94, 0xec2, 0xe95, 0xec0, 0xe9a, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xead, 0xeb2, 0x200b, 0xea3, 0xeb2, 0x200b, 0xe9a,
0xebd, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b,
0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xead, 0xeb2, 0x200b, 0xea3, 0xeb2, 0x200b,
0xe9a, 0xebd, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7,
0xeb1, 0xe99, 0xead, 0xeb2, 0xea3, 0xeb2, 0xe9a, 0xebd, 0xe99, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xead, 0xeb2, 0x200b, 0xec0, 0xe88, 0x200b, 0xe97, 0xeb4, 0x200b,
0xe99, 0xeb2, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94,
0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xead, 0xeb2, 0x200b, 0xec0, 0xe88,
0xe99, 0x200b, 0xe97, 0xeb4, 0x200b, 0xe99, 0xeb2, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99,
0x200b, 0xead, 0xeb2, 0x200b, 0xec0, 0xe88, 0xe99, 0x200b, 0xe97, 0xeb4, 0x200b, 0xe99,
0xeb2, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xec0, 0xea7, 0x200b, 0xeaa, 0x200b,
0xec0, 0xe97, 0xeb4, 0xe99, 0xead, 0xeb2, 0x200b, 0xec0, 0xe88, 0xe99, 0x200b, 0xe97,
0xeb4, 0x200b, 0xe99, 0xeb2, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2,
0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xec0, 0xea7, 0x200b, 0xeaa,
0x200b, 0xec0, 0xe97, 0xeb4, 0xe99, 0xead, 0xeb2, 0x200b, 0xec0, 0xe88, 0xe99, 0x200b,
0xe97, 0xeb4, 0x200b, 0xe99, 0xeb2, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5,
0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0xec0, 0xea7, 0x200b,
0xeaa, 0x200b, 0xec0, 0xe97, 0xeb4, 0xe99, 0xead, 0xeb2, 0x200b, 0xec0, 0xe88, 0xe99,
0x200b, 0xe97, 0xeb4, 0x200b, 0xe99, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xead, 0xeb2,
0xec0, 0xea1, 0xec0, 0xe99, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94,
0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xead, 0xeb2, 0xec0, 0xea1, 0xec0, 0xe99, 0xe8d,
0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99,
0xead, 0xeb2, 0xec0, 0xea1, 0xec0, 0xe99, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe82,
0xead, 0xe87, 0xead, 0xeb2, 0xec1, 0xea5, 0xe99, 0xe95, 0xeb4, 0xe81, 0xec0, 0xea7,
0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xe82, 0xead,
0xe87, 0xead, 0xeb2, 0xec1, 0xea5, 0xe99, 0xe95, 0xeb4, 0xe81, 0xec0, 0xea7, 0xea5,
0xeb2, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0xeb1, 0xe99, 0xe82, 0xead, 0xe87, 0xead,
0xeb2, 0xec1, 0xea5, 0xe99, 0xe95, 0xeb4, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0xead, 0xead, 0x200b, 0xeaa, 0x200b, 0xec0, 0xe95, 0xea3, 0x200b, 0xec0, 0xea5, 0xe8d,
0x200b, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94,
0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xead, 0xead, 0x200b, 0xeaa, 0x200b,
0xec0, 0xe95, 0xea3, 0x200b, 0xec0, 0xea5, 0xe8d, 0x200b, 0xe81, 0x200b, 0xeb2, 0xe87,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe95, 0xead, 0xe99, 0x200b, 0xe97, 0xec8,
0xebd, 0xe87, 0x200b, 0xead, 0xead, 0xeaa, 0x200b, 0xec0, 0xe95, 0xea3, 0x200b, 0xec0,
0xea5, 0xe8d, 0x200b, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0xead,
0xead, 0xeaa, 0x200b, 0xec0, 0xe95, 0xea3, 0x200b, 0xea5, 0xebd, 0xe99, 0x200b, 0xe81,
0xeb2, 0xe87, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xe95,
0xebb, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b,
0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xead, 0xead, 0xeaa, 0x200b, 0xec0, 0xe95,
0xea3, 0x200b, 0xea5, 0xebd, 0xe99, 0x200b, 0xe81, 0xeb2, 0xe87, 0x200b, 0xe95, 0xeb2,
0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xe95, 0xebb, 0xe81, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xe95, 0xead, 0xe99, 0x200b, 0xe97, 0xec8, 0xebd, 0xe87, 0x200b,
0xead, 0xead, 0xeaa, 0x200b, 0xec0, 0xe95, 0xea3, 0x200b, 0xea5, 0xebd, 0xe99, 0x200b,
0xe81, 0xeb2, 0xe87, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b,
0xe95, 0xebb, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0xead, 0xead, 0xeaa, 0x200b,
0xec0, 0xe95, 0xea3, 0x200b, 0xea5, 0xebd, 0xe99, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0,
0xea7, 0xeb1, 0xe99, 0x200b, 0xead, 0xead, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0x200b, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0x200b, 0x200b, 0x200b,
0xead, 0xead, 0xeaa, 0x200b, 0xec0, 0xe95, 0xea3, 0x200b, 0xea5, 0xebd, 0xe99, 0x200b,
0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xead, 0xead, 0xe81, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe95, 0xead, 0xe99, 0x200b, 0xe97, 0xec8, 0xebd,
0xe87, 0x200b, 0xead, 0xead, 0xeaa, 0x200b, 0xec0, 0xe95, 0xea3, 0x200b, 0xea5, 0xebd,
0xe99, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xead, 0xead,
0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xead, 0xead, 0xeaa, 0x200b, 0xec0,
0xe95, 0xea3, 0x200b, 0xec0, 0xea5, 0xe8d, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7,
0xeb1, 0xe99, 0x200b, 0xe95, 0xebb, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b,
0xea1, 0xeb2, 0x200b, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xead,
0xead, 0xeaa, 0x200b, 0xec0, 0xe95, 0xea3, 0x200b, 0xea5, 0xebd, 0xe99, 0x200b, 0xe95,
0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xe95, 0xebb, 0xe81, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0xe95, 0xead, 0xe99, 0x200b, 0xe97, 0xec8, 0xebd, 0xe87,
0x200b, 0xead, 0xead, 0xeaa, 0x200b, 0xec0, 0xe95, 0xea3, 0x200b, 0xea5, 0xebd, 0xe99,
0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xe95, 0xebb, 0xe81,
0xec0, 0xea7, 0xea5, 0xeb2, 0xead, 0xeb1, 0xeaa, 0xec0, 0xe8a, 0xeb5, 0xec4, 0xe9a,
0xe88, 0xeb1, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0,
0xe96, 0xeb2, 0xe99, 0xead, 0xeb1, 0xeaa, 0xec0, 0xe8a, 0xeb5, 0xec4, 0xe9a, 0xe88,
0xeb1, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9,
0xead, 0xe99, 0xead, 0xeb1, 0xeaa, 0xec0, 0xe8a, 0xeb5, 0xec4, 0xe9a, 0xe88, 0xeb1,
0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xead, 0xeb2, 0x200b, 0xec2, 0xe8a,
0x200b, 0xec0, 0xea3, 0x200b, 0xeaa, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1,
0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xead, 0xeb2, 0x200b,
0xec2, 0xe8a, 0x200b, 0xec0, 0xea3, 0x200b, 0xeaa, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0xead,
0xeb2, 0x200b, 0xec2, 0xe8a, 0x200b, 0xec0, 0xea3, 0x200b, 0xeaa, 0xec0, 0xea7, 0xea5,
0xeb2, 0x20, 0xe9a, 0xeb1, 0xe87, 0xe81, 0xeb0, 0xea5, 0xeb2, 0xec0, 0xe97, 0xe94,
0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99,
0x20, 0xe9a, 0xeb1, 0xe87, 0xe81, 0xeb0, 0xea5, 0xeb2, 0xec0, 0xe97, 0xe94, 0xec0,
0xea7, 0xea5, 0xeb2, 0x20, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99,
0x20, 0xe9a, 0xeb1, 0xe87, 0xe81, 0xeb0, 0xea5, 0xeb2, 0xec0, 0xe97, 0xe94, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe9e, 0xeb9, 0x200b, 0xe96, 0xeb2, 0xe99, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xec2, 0xe9a, 0x200b, 0xea5, 0xeb4, 0x200b, 0xec0,
0xea7, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe95, 0xeb2, 0xea1, 0xec0, 0xe82, 0xe94,
0xe9a, 0xea3, 0xeb2, 0xe8a, 0xeb4, 0xec0, 0xea5, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2,
0xea1, 0xeb2, 0xe94, 0xe95, 0xeb2, 0xe96, 0xeb2, 0xe99, 0xec0, 0xe9a, 0xea3, 0xe8a,
0xeb5, 0xec0, 0xea5, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe95, 0xeb2, 0xea1, 0xec0,
0xe82, 0xe94, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xe95, 0xeb2,
0xea1, 0xec0, 0xe82, 0xe94, 0xe9a, 0xea3, 0xeb2, 0xe8a, 0xeb5, 0xec0, 0xea5, 0xe8d,
0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0xe9a, 0xea3, 0xeb9, 0x200b, 0xec4, 0xe99,
0xe94, 0xeb2, 0x200b, 0xea3, 0xeb8, 0xeaa, 0x200b, 0xe8a, 0xeb2, 0x200b, 0xea5, 0xeb2,
0xea1, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xec0, 0xe84, 0xe9a, 0x200b, 0xec0,
0xea7, 0xeb5, 0xe94, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2,
0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xec0, 0xe84, 0xe9a,
0x200b, 0xec0, 0xea7, 0xeb5, 0xe94, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5,
0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0x200b, 0xec0, 0xe84,
0xe9a, 0x200b, 0xec0, 0xea7, 0xeb5, 0xe94, 0xec0, 0xea7, 0xea5, 0xeb2, 0xec0, 0xe84,
0xe8a, 0xeb5, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe88, 0xeb2, 0x200b, 0xec2,
0xea1, 0x200b, 0xec2, 0xea3, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe8a, 0xeb2,
0x200b, 0xe97, 0xeb2, 0xea1, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2,
0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xe8a, 0xeb2, 0x200b,
0xe97, 0xeb2, 0xea1, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe95, 0xead, 0xe99,
0x200b, 0xe97, 0xec8, 0xebd, 0xe87, 0x200b, 0xe8a, 0xeb2, 0x200b, 0xe97, 0xeb2, 0xea1,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe8a, 0xeb4, 0x200b, 0xea5, 0xeb5, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b,
0xe96, 0xeb2, 0xe99, 0xe8a, 0xeb4, 0x200b, 0xea5, 0xeb5, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99,
0xe8a, 0xeb4, 0x200b, 0xea5, 0xeb5, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe88,
0xeb5, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96,
0xeb2, 0xe99, 0xe88, 0xeb5, 0xe99, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b,
0xe95, 0xead, 0xe99, 0x200b, 0xe97, 0xec8, 0xebd, 0xe87, 0x200b, 0xe88, 0xeb5, 0xe99,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xec0, 0xe81, 0xeb2, 0xeb0, 0x200b, 0xe84,
0x200b, 0xea3, 0xeb4, 0xeaa, 0x200b, 0xea1, 0xeb2, 0xeaa, 0xec0, 0xea7, 0xea5, 0xeb2,
0xeab, 0xea1, 0xeb9, 0xec8, 0xec0, 0xe81, 0xeb2, 0xeb0, 0xec2, 0xe81, 0xec2, 0xe81,
0xeaa, 0xec0, 0xea7, 0xea5, 0xeb2, 0xec2, 0xe84, 0xea5, 0xecd, 0xeb2, 0xec0, 0xe9a,
0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2,
0xe99, 0xec2, 0xe84, 0xea5, 0xeb3, 0xec0, 0xe9a, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2,
0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xec2, 0xe84, 0xea5, 0xecd,
0xeb2, 0xec0, 0xe9a, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xeab, 0xea1, 0xeb9, 0xec8,
0xec0, 0xe81, 0xeb2, 0xeb0, 0xe84, 0xeb8, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1,
0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xeab, 0xea1, 0xeb9, 0xec8, 0xec0,
0xe81, 0xeb2, 0xeb0, 0xe84, 0xeb8, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b,
0xec0, 0xe84, 0xeb4, 0xec8, 0xe87, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b,
0xeae, 0xec9, 0xead, 0xe99, 0x200b, 0xedd, 0xeb9, 0xec8, 0x200b, 0xec0, 0xe81, 0xeb2,
0xeb0, 0x200b, 0xe84, 0xeb8, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe84, 0xeb4, 0xea7,
0xe9a, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96,
0xeb2, 0xe99, 0xe82, 0xead, 0xe87, 0xe84, 0xeb4, 0xea7, 0xe9a, 0xeb2, 0xec0, 0xea7,
0xea5, 0xeb2, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0xeb1, 0xe99, 0xe84, 0xeb4, 0xea7,
0xe9a, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xec0, 0xe94, 0xea7, 0xeb4, 0xe94, 0xec0,
0xea7, 0xea5, 0xeb2, 0xe94, 0xeb9, 0xea1, 0xead, 0xe87, 0xe94, 0xeb9, 0xea7, 0xeb4,
0xea5, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe95, 0xeb5, 0xea1, 0xecd, 0xe95, 0xeb2, 0xec0,
0xea7, 0xeb1, 0xe99, 0xead, 0xead, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b,
0xec0, 0xe81, 0xeb2, 0xeb0, 0x200b, 0xead, 0xeb5, 0x200b, 0xeaa, 0x200b, 0xec0, 0xe95,
0xeb5, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95,
0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xec0, 0xe81, 0xeb2, 0xeb0, 0x200b, 0xead, 0xeb5,
0x200b, 0xeaa, 0x200b, 0xec0, 0xe95, 0xeb5, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b,
0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0xec0, 0xe81,
0xeb2, 0xeb0, 0x200b, 0xead, 0xeb5, 0x200b, 0xeaa, 0x200b, 0xec0, 0xe95, 0xeb5, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xec0, 0xead, 0x200b, 0xe81, 0xebb, 0xea7, 0x200b,
0xe94, 0xecd, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea2, 0xeb9, 0x200b, 0xec2,
0xea3, 0xe9a, 0x200b, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b,
0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xea2,
0xeb9, 0x200b, 0xec2, 0xea3, 0xe9a, 0xe81, 0xeb2, 0xe87, 0x200b, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead,
0xe99, 0x200b, 0xea2, 0xeb9, 0x200b, 0xec2, 0xea3, 0xe9a, 0x200b, 0xe81, 0xeb2, 0xe87,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea2, 0xeb9, 0x200b, 0xec2, 0xea3, 0xe9a,
0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xead, 0xead, 0xe81,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0x200b, 0xe94, 0x200b, 0xe95,
0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xea2, 0xeb9, 0x200b, 0xec2, 0xea3, 0xe9a,
0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xead, 0xead, 0xe81,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b,
0xeae, 0xec9, 0xead, 0xe99, 0xea2, 0xeb9, 0x200b, 0xec2, 0xea3, 0xe9a, 0x200b, 0xe95,
0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xead, 0xead, 0xe81, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0x200b, 0xea2, 0xeb9, 0x200b, 0xec2, 0xea3, 0x200b, 0xe9b,
0xebd, 0xe99, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xead,
0xead, 0xe81, 0x200b, 0xec4, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea2,
0xeb9, 0x200b, 0xec2, 0xea3, 0xe9a, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1,
0xe99, 0x200b, 0xe95, 0xebb, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1,
0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xea2, 0xeb9, 0x200b,
0xec2, 0xea3, 0xe9a, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b,
0xe95, 0xebb, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b,
0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0xea2, 0xeb9, 0x200b, 0xec2, 0xea3,
0xe9a, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xe95, 0xebb,
0xe81, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xedd, 0xeb9, 0xec8, 0x200b,
0xec0, 0xe81, 0xeb2, 0xeb0, 0x200b, 0xe9f, 0xead, 0x200b, 0xea5, 0xecc, 0xe81, 0x200b,
0xec1, 0xea5, 0xe99, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2,
0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xedd, 0xeb9, 0xec8, 0x200b,
0xec0, 0xe81, 0xeb2, 0xeb0, 0x200b, 0xe9f, 0xead, 0x200b, 0xea5, 0xecc, 0xe81, 0x200b,
0xec1, 0xea5, 0xe99, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0,
0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0xedd, 0xeb9, 0xec8, 0x200b,
0xec0, 0xe81, 0xeb2, 0xeb0, 0x200b, 0xe9f, 0xead, 0x200b, 0xea5, 0xecc, 0xe81, 0x200b,
0xec1, 0xea5, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe9f, 0xeb4, 0xe88, 0xeb4, 0xec0,
0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xe9f,
0xeb4, 0xe88, 0xeb4, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae,
0xec9, 0xead, 0xe99, 0xe9f, 0xeb4, 0xe88, 0xeb4, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0x200b, 0xec0, 0xe9f, 0x200b, 0xea3, 0xe99, 0x200b, 0xe8a, 0xecc, 0x200b, 0xec0, 0xe81,
0xe8d, 0x200b, 0xe99, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe9d, 0xea3, 0xeb1, 0xec8,
0xe87, 0xe95, 0xead, 0xe99, 0xec3, 0xe95, 0xec9, 0x20, 0xec1, 0xea5, 0xeb0, 0x20,
0xec1, 0xead, 0xe99, 0xe95, 0xeb2, 0xe81, 0xe95, 0xeb4, 0xe81, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xe81, 0xeb2, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe9b, 0xeb2, 0x200b,
0xe81, 0xead, 0x200b, 0xeaa, 0xec0, 0xea7, 0xea5, 0xeb2, 0xec1, 0xe81, 0xea1, 0xec0,
0xe9a, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe88, 0xecd, 0xec0, 0xe88, 0xe8d, 0xec0,
0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xe88,
0xecd, 0xec0, 0xe88, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9,
0xeae, 0xec9, 0xead, 0xe99, 0xe88, 0xecd, 0xec0, 0xe88, 0xe8d, 0xec0, 0xea7, 0xea5,
0xeb2, 0xeab, 0xea1, 0xeb9, 0xec8, 0xec0, 0xe81, 0xeb2, 0xeb0, 0xe81, 0xeb4, 0xea5,
0xec0, 0xe9a, 0xeb5, 0xe94, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0xe81, 0xea3, 0xeb5,
0xe99, 0x200b, 0xea7, 0xeb4, 0x200b, 0xe8a, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe95, 0xeb2,
0xec0, 0xea7, 0xeb1, 0xe99, 0xead, 0xead, 0xe81, 0xe82, 0xead, 0xe87, 0xe81, 0xea3,
0xeb5, 0xe99, 0xec1, 0xea5, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94,
0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xe95, 0xeb2, 0xec0, 0xea7, 0xeb1, 0xe99, 0xead,
0xead, 0xe81, 0xe81, 0xea3, 0xeb5, 0xe99, 0xec1, 0xea5, 0xe99, 0xec0, 0xea7, 0xea5,
0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xe81, 0xea3, 0xeb5,
0xe99, 0xec1, 0xea5, 0xe99, 0xe95, 0xeb2, 0xec0, 0xea7, 0xeb1, 0xe99, 0xead, 0xead,
0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe81, 0xea3, 0xeb5, 0xe99, 0xec1, 0xea5, 0xe99,
0xe95, 0xeb2, 0xec0, 0xea7, 0xeb1, 0xe99, 0xe95, 0xebb, 0xe81, 0xec0, 0xea7, 0xea5,
0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xe81, 0xea3, 0xeb5,
0xe99, 0xec1, 0xea5, 0xe99, 0xe95, 0xeb2, 0xec0, 0xea7, 0xeb1, 0xe99, 0xe95, 0xebb,
0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe95, 0xead, 0xe99, 0xe97, 0xec8, 0xebd, 0xe87,
0xe81, 0xea3, 0xeb5, 0xe99, 0xec1, 0xea5, 0xe99, 0xe95, 0xeb2, 0xec0, 0xea7, 0xeb1,
0xe99, 0xe95, 0xebb, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe81, 0xea7, 0xea1, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe81, 0xeb9, 0x200b, 0xea5, 0x200b, 0xecc, 0xe9f,
0xec0, 0xea7, 0xea5, 0xeb2, 0xe81, 0xeb2, 0xe8d, 0xead, 0xeb2, 0xe99, 0xeb2, 0xec0,
0xea7, 0xea5, 0xeb2, 0xeae, 0xeb2, 0xea7, 0xeb2, 0xe8d, 0x2d, 0xec0, 0xead, 0xea5,
0xeb9, 0xe97, 0xebd, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95,
0xeb0, 0xe96, 0xeb2, 0xe99, 0xeae, 0xeb2, 0xea7, 0xeb2, 0xe8d, 0x2d, 0xec0, 0xead,
0xea5, 0xeb9, 0xe97, 0xebd, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe95, 0xead, 0xe99,
0xe97, 0xec8, 0xebd, 0xe87, 0xeae, 0xeb2, 0xea7, 0xeb2, 0xe8d, 0x2d, 0xec0, 0xead,
0xea5, 0xeb9, 0xe97, 0xebd, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xeae,
0xead, 0xe87, 0x200b, 0xe81, 0xebb, 0xe87, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b,
0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xeae,
0xead, 0xe87, 0x200b, 0xe81, 0xebb, 0xe87, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0x200b,
0xeae, 0xead, 0xe87, 0x200b, 0xe81, 0xebb, 0xe87, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0x200b, 0xeae, 0xead, 0xe9a, 0x200b, 0xe94, 0xecc, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99,
0x200b, 0xeae, 0xead, 0xe9a, 0x200b, 0xe94, 0xecc, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99,
0x200b, 0xeae, 0xead, 0xe9a, 0x200b, 0xe94, 0xecc, 0xec0, 0xea7, 0xea5, 0xeb2, 0x20,
0xead, 0xeb4, 0xe99, 0xec0, 0xe94, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb0,
0xeab, 0xeb2, 0xeaa, 0xeb0, 0xeab, 0xea1, 0xeb8, 0xe94, 0xead, 0xeb4, 0xe99, 0xec0,
0xe94, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xead, 0xeb4, 0xe99, 0xe94, 0xeb9, 0xe88,
0xeb5, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xead, 0xeb4, 0xe99, 0x200b,
0xec2, 0xe94, 0x200b, 0xec0, 0xe99, 0x200b, 0xec0, 0xe8a, 0xe8d, 0x200b, 0xe81, 0xeb2,
0xe87, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xead, 0xeb4, 0xe99, 0x200b, 0xec2,
0xe94, 0x200b, 0xec0, 0xe99, 0x200b, 0xec0, 0xe8a, 0xe8d, 0x200b, 0xe95, 0xeb2, 0x200b,
0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xead, 0xead, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xead, 0xeb4, 0xe99, 0x200b, 0xec2, 0xe94, 0x200b, 0xec0, 0xe99, 0x200b,
0xec0, 0xe8a, 0xe8d, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b,
0xe95, 0xebb, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xead, 0xeb5, 0x200b,
0xea3, 0xeb2, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94,
0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xead, 0xeb5, 0x200b, 0xea3, 0xeb2,
0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe95, 0xead, 0xe99, 0x200b, 0xe97,
0xec8, 0xebd, 0xe87, 0x200b, 0xead, 0xeb5, 0x200b, 0xea3, 0xeb2, 0x200b, 0xe99, 0xec0,
0xea7, 0x200b, 0xea5, 0x200b, 0xeb2, 0xead, 0xeb5, 0x200b, 0xe84, 0xeb8, 0xe94, 0x200b,
0xeaa, 0x200b, 0xe84, 0xecc, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94,
0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xead, 0xeb5, 0x200b, 0xe84, 0xeb8,
0xe94, 0x200b, 0xeaa, 0x200b, 0xe84, 0xecc, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0xea5,
0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0xead, 0xeb5, 0x200b,
0xe84, 0xeb8, 0xe94, 0x200b, 0xeaa, 0x200b, 0xe84, 0xecc, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xead, 0xeb4, 0x200b, 0xeaa, 0x200b, 0xea3, 0xeb2, 0x200b, 0xec0, 0xead,
0xea7, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95,
0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xead, 0xeb4, 0x200b, 0xeaa, 0x200b, 0xea3, 0xeb2,
0x200b, 0xec0, 0xead, 0xea7, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe81, 0xeb2, 0xe87, 0xec0,
0xea7, 0xeb1, 0xe99, 0xead, 0xeb4, 0xeaa, 0xea3, 0xeb2, 0xec0, 0xead, 0xea7, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe8d, 0xeb5, 0xec8, 0x200b, 0xe9b, 0xeb8, 0xec8,
0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95,
0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xe8d, 0xeb5, 0xec8, 0x200b, 0xe9b, 0xeb8,
0xec8, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe95, 0xead, 0xe99, 0x200b,
0xe97, 0xec8, 0xebd, 0xe87, 0x200b, 0xe8d, 0xeb5, 0xec8, 0x200b, 0xe9b, 0xeb8, 0xec8,
0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe84, 0xeb2, 0xe8a, 0xeb1, 0xe81, 0xeaa, 0xe96,
0xeb2, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe84, 0xeb2, 0x200b, 0xe8a,
0xeb1, 0xe81, 0x200b, 0xeaa, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xe95, 0xeb2, 0x200b,
0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xead, 0xead, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xe84, 0xeb2, 0x200b, 0xe8a, 0xeb1, 0xe81, 0x200b, 0xeaa, 0x200b, 0xe96,
0xeb2, 0xe99, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xe95,
0xebb, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xec0, 0xe81, 0xebb, 0xeb2, 0xeab, 0xebc,
0xeb5, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95,
0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xec0, 0xe81, 0xebb, 0xeb2, 0x200b, 0xeab,
0xea5, 0xeb5, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe95, 0xead, 0xe99, 0x200b,
0xe97, 0xec8, 0xebd, 0xe87, 0x200b, 0xec0, 0xe81, 0xebb, 0xeb2, 0x200b, 0xeab, 0xea5,
0xeb5, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe84, 0xead, 0xeaa, 0xec1, 0xea3, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0xe84, 0xea3, 0xeb1, 0xeaa, 0x200b, 0xec2, 0xe99, 0x200b,
0xea2, 0xeb2, 0x200b, 0xeaa, 0x200b, 0xe84, 0xecc, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xe84,
0xea3, 0xeb1, 0xeaa, 0x200b, 0xec2, 0xe99, 0x200b, 0xea2, 0xeb2, 0x200b, 0xeaa, 0x200b,
0xe84, 0xecc, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94,
0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0xe84, 0xea3, 0xeb1, 0xeaa, 0x200b, 0xec2,
0xe99, 0x200b, 0xea2, 0xeb2, 0x200b, 0xeaa, 0x200b, 0xe84, 0xecc, 0xec0, 0xea7, 0xea5,
0xeb2, 0xec0, 0xe84, 0xe8d, 0xe81, 0xeb4, 0xeaa, 0xe96, 0xeb2, 0xe99, 0xec0, 0xea7,
0xea5, 0xeb2, 0xea5, 0xeb1, 0xe87, 0xe81, 0xeb2, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0x200b, 0xedd, 0xeb9, 0xec8, 0x200b, 0xec0, 0xe81, 0xeb2, 0xeb0, 0x200b, 0xea5, 0xeb2,
0xe8d, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xead, 0xe94, 0x200b, 0xec0,
0xeae, 0xebb, 0xeb2, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94,
0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xea5, 0xead, 0xe94, 0x200b,
0xec0, 0xeae, 0xebb, 0xeb2, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0x200b, 0xeaa, 0xeb2,
0x200b, 0xe95, 0xead, 0xe99, 0x200b, 0x200b, 0xe97, 0xec8, 0xebd, 0xe87, 0x200b, 0xea5,
0xead, 0xe94, 0x200b, 0xec0, 0xeae, 0xebb, 0xeb2, 0x200b, 0xec0, 0xea7, 0xea5, 0xeb2,
0xea1, 0xeb2, 0xec0, 0xe81, 0xebb, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2,
0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xea1, 0xeb2, 0xec0, 0xe81, 0xebb, 0xeb2,
0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99,
0xea1, 0xeb2, 0xec0, 0xe81, 0xebb, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xec0, 0xea1,
0xeb1, 0xe81, 0xe81, 0xeb2, 0xec0, 0xe94, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1,
0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xec0, 0xea1, 0xeb1, 0xe81, 0xe81,
0xeb2, 0xec0, 0xe94, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9,
0xeae, 0xec9, 0xead, 0xe99, 0xec0, 0xea1, 0xeb1, 0xe81, 0xe81, 0xeb2, 0xec0, 0xe94,
0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0x200b, 0xec0, 0xea5,
0x200b, 0xec0, 0xe8a, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb1, 0xea5, 0xe94,
0xeb5, 0xe9f, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xec0, 0xe84, 0xeb5, 0xe8a,
0xeb1, 0xeaa, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xedd, 0xeb9, 0xec8, 0x200b,
0xec0, 0xe81, 0xeb2, 0xeb0, 0x200b, 0xea1, 0xeb2, 0x200b, 0xec1, 0xe8a, 0xea7, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xec0, 0xea1, 0xebb, 0xeb2, 0x200b, 0xea3, 0xeb4,
0x200b, 0xe97, 0xebd, 0x200b, 0xeaa, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1,
0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xec0, 0xea1, 0xebb,
0xeb2, 0x200b, 0xea3, 0xeb4, 0x200b, 0xe97, 0xebd, 0x200b, 0xeaa, 0x200b, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9,
0xead, 0xe99, 0x200b, 0xec0, 0xea1, 0xebb, 0xeb2, 0x200b, 0xea3, 0xeb4, 0x200b, 0xe97,
0xebd, 0x200b, 0xeaa, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xecd, 0xeaa, 0xeb1, 0xe99,
0xec0, 0xea7, 0xea5, 0xeb2, 0xec1, 0xe9b, 0xe8a, 0xeb4, 0xe9f, 0xeb4, 0xe81, 0xec0,
0xea1, 0xeb1, 0xe81, 0xe8a, 0xeb4, 0xe81, 0xeb1, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2,
0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xec1, 0xe9b, 0xe8a, 0xeb4,
0xe9f, 0xeb4, 0xe81, 0xec0, 0xea1, 0xeb1, 0xe81, 0xe8a, 0xeb4, 0xe81, 0xeb1, 0xe99,
0xec0, 0xea7, 0xea5, 0xeb2, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0xeb1, 0xe99, 0xec1,
0xe9b, 0xe8a, 0xeb4, 0xe9f, 0xeb4, 0xe81, 0xec0, 0xea1, 0xeb1, 0xe81, 0xe8a, 0xeb4,
0xe81, 0xeb1, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0x20, 0xead, 0xeb9, 0xea5, 0xeb2,
0xe99, 0xe9a, 0xeb2, 0xec0, 0xe95, 0xeb5, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2,
0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0x20, 0xead, 0xeb9, 0xea5, 0xeb2, 0xe99,
0xe9a, 0xeb2, 0xec0, 0xe95, 0xeb5, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94,
0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xead, 0xeb9, 0xea5, 0xeb2, 0xe99, 0xe9a, 0xeb2,
0xec0, 0xe95, 0xeb5, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xead, 0x200b,
0xeaa, 0x200b, 0xec2, 0xe84, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2,
0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xea1, 0xead, 0x200b, 0xeaa,
0x200b, 0xec2, 0xe84, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b,
0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0xea1, 0xead, 0x200b, 0xeaa, 0x200b,
0xec2, 0xe84, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xebd, 0xe99, 0xea1, 0xeb2, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe99, 0xeb2, 0x200b, 0xead, 0xeb9, 0x200b, 0xea3,
0xeb8, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xec0, 0xe99, 0x200b, 0xe9b,
0xeb2, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe99, 0xeb4, 0xea7, 0xec1, 0xe84, 0xea5,
0xeb4, 0xec2, 0xe94, 0xec0, 0xe99, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2,
0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xe99, 0xeb4, 0xea7, 0xec1, 0xe84, 0xea5,
0xeb4, 0xec2, 0xe94, 0xec0, 0xe99, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0,
0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xe99, 0xeb4, 0xea7, 0xec1, 0xe84, 0xea5,
0xeb4, 0xec2, 0xe94, 0xec0, 0xe99, 0xe8d, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b,
0xe99, 0xeb4, 0xea7, 0x200b, 0xe8a, 0xeb5, 0x200b, 0xec1, 0xea5, 0xe99, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96,
0xeb2, 0xe99, 0xe99, 0xeb4, 0xea7, 0x200b, 0xe8a, 0xeb5, 0x200b, 0xec1, 0xea5, 0xe99,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe95, 0xead, 0xe99, 0x200b, 0xe97, 0xec8,
0xebd, 0xe87, 0x200b, 0xe99, 0xeb4, 0xea7, 0x200b, 0xe8a, 0xeb5, 0x200b, 0xec1, 0xea5,
0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe99, 0xeb4, 0xea7, 0x200b, 0xe9f,
0xeb2, 0xea7, 0x200b, 0xec1, 0xea5, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b,
0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xe99,
0xeb4, 0xea7, 0x200b, 0xe9f, 0xeb2, 0xea7, 0x200b, 0xec1, 0xea5, 0xe99, 0xec0, 0xea7,
0xea5, 0xeb2, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0xeb1, 0xe99, 0xe99, 0xeb4, 0xea7,
0xe9f, 0xeb2, 0xea7, 0xec1, 0xea5, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe99, 0xeb4,
0xead, 0xeb9, 0xec0, 0xead, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xec0, 0xe81,
0xeb2, 0xeb0, 0x200b, 0xe99, 0xecd, 0x200b, 0xe9f, 0xead, 0x200b, 0xea5, 0xecc, 0xe81,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0,
0x200b, 0xe96, 0xeb2, 0xe99, 0xec0, 0xe81, 0xeb2, 0xeb0, 0x200b, 0xe99, 0xecd, 0x200b,
0xe9f, 0xead, 0x200b, 0xea5, 0xecc, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b,
0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0xec0, 0xe81,
0xeb2, 0xeb0, 0x200b, 0xe99, 0xecd, 0x200b, 0xe9f, 0xead, 0x200b, 0xea5, 0xecc, 0xe81,
0xec0, 0xea7, 0xea5, 0xeb2, 0xec0, 0xe9f, 0xe99, 0xeb1, 0xe99, 0xec2, 0xe94, 0xec0,
0xe94, 0xec2, 0xe99, 0xeae, 0xead, 0xe99, 0xeae, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2,
0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xec0, 0xe9f, 0xe99, 0xeb1,
0xe99, 0xec2, 0xe94, 0xec0, 0xe94, 0xec2, 0xe99, 0xeae, 0xead, 0xe99, 0xeae, 0xeb2,
0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99,
0xec0, 0xe9f, 0xe99, 0xeb1, 0xe99, 0xec2, 0xe94, 0xec0, 0xe94, 0xec2, 0xe99, 0xeae,
0xead, 0xe99, 0xeae, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xeab, 0xea1, 0xeb9, 0xec8,
0xec0, 0xe81, 0xeb2, 0xeb0, 0xea1, 0xeb2, 0xec0, 0xea3, 0xe8d, 0xe99, 0xeb2, 0xec0,
0xeab, 0xe99, 0xeb7, 0xead, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xec2, 0xe99,
0x200b, 0xec2, 0xe9a, 0x200b, 0xe8a, 0xeb4, 0x200b, 0xe9a, 0xeb4, 0x200b, 0xeaa, 0x200b,
0xe84, 0xecc, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b,
0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xec2, 0xe99, 0x200b, 0xec2, 0xe9a, 0x200b,
0xe8a, 0xeb4, 0x200b, 0xe9a, 0xeb4, 0x200b, 0xeaa, 0x200b, 0xe84, 0xecc, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9,
0xead, 0xe99, 0xec2, 0xe99, 0x200b, 0xec2, 0xe9a, 0x200b, 0xe8a, 0xeb4, 0x200b, 0xe9a,
0xeb4, 0x200b, 0xeaa, 0x200b, 0xe84, 0xecc, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0xead, 0xead, 0xea1, 0x200b, 0xeaa, 0x200b, 0xe84, 0xecc, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99,
0xead, 0xead, 0xea1, 0x200b, 0xeaa, 0x200b, 0xe84, 0xecc, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99,
0xead, 0xead, 0xea1, 0x200b, 0xeaa, 0x200b, 0xe84, 0xecc, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xe9b, 0xeb2, 0x200b, 0xe81, 0xeb5, 0xeaa, 0xe96, 0xeb2, 0xe99, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b,
0xe96, 0xeb2, 0xe99, 0x200b, 0xe9b, 0xeb2, 0x200b, 0xe81, 0xeb5, 0xeaa, 0xe96, 0xeb2,
0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9,
0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0x200b, 0xe9b, 0xeb2, 0x200b, 0xe81, 0xeb5, 0x200b,
0xeaa, 0x200b, 0xe96, 0xeb2, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe9b, 0xeb2, 0xec0,
0xea5, 0xebb, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe9b, 0xeb2, 0xe9b, 0xebb, 0xea7,
0xe81, 0xeb4, 0xe99, 0xeb5, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe9b, 0xeb2,
0x200b, 0xea3, 0xeb2, 0x200b, 0xe81, 0xea7, 0xe8d, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b,
0xe9b, 0xeb2, 0x200b, 0xea3, 0xeb2, 0x200b, 0xe81, 0xea7, 0xe8d, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead,
0xe99, 0x200b, 0xe9b, 0xeb2, 0x200b, 0xea3, 0xeb2, 0x200b, 0xe81, 0xea7, 0xe8d, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xec0, 0xe9b, 0x200b, 0xea3, 0xeb9, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0x200b, 0xea1, 0xeb2, 0x200b, 0xe94, 0x200b, 0xe95, 0xeb0,
0x200b, 0xe96, 0xeb2, 0xe99, 0xec0, 0xe9b, 0x200b, 0xea3, 0xeb9, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead,
0xe99, 0x200b, 0xec0, 0xe9b, 0x200b, 0xea3, 0xeb9, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xe9f, 0xeb4, 0x200b, 0xea5, 0xeb4, 0xe9a, 0x200b, 0xe9b, 0xeb4, 0xe99,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0,
0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xe9f, 0xeb4, 0x200b, 0xea5, 0xeb4, 0xe9a, 0x200b,
0xe9b, 0xeb4, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b,
0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0x200b, 0xe9f, 0xeb4, 0x200b, 0xea5,
0xeb4, 0xe9a, 0x200b, 0xe9b, 0xeb4, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xeab, 0xea1,
0xeb9, 0xec8, 0xec0, 0xe81, 0xeb2, 0xeb0, 0xe9f, 0xeb5, 0xe99, 0xeb4, 0xe81, 0x200b,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0xec0, 0xe8a, 0xe99, 0x200b, 0xe9b, 0xeb5, 0x200b,
0xec1, 0xead, 0x20, 0xec1, 0xea5, 0xeb0, 0x200b, 0xea1, 0xeb4, 0x200b, 0xe81, 0xebb,
0xea7, 0x200b, 0xea5, 0xead, 0xe99, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0xea1,
0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xec0, 0xe8a, 0xe99,
0x200b, 0xe9b, 0xeb5, 0x200b, 0xec1, 0xead, 0x20, 0xec1, 0xea5, 0xeb0, 0x200b, 0xea1,
0xeb4, 0x200b, 0xe81, 0xebb, 0xea7, 0x200b, 0xea5, 0xead, 0xe99, 0x200b, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0xe95, 0xead, 0xe99, 0x200b, 0xe97, 0xec8, 0xebd, 0xe87, 0xec0,
0xe8a, 0xe99, 0x200b, 0xe9b, 0xeb5, 0x200b, 0xec1, 0xead, 0x20, 0xec1, 0xea5, 0xeb0,
0x200b, 0xea1, 0xeb4, 0x200b, 0xe81, 0xebb, 0xea7, 0x200b, 0xea5, 0xead, 0xe99, 0xec0,
0xea7, 0xea5, 0xeb2, 0xe9e, 0xeb4, 0xe94, 0xec1, 0xe84, 0xea3, 0xecc, 0xe99, 0xec0,
0xea7, 0xea5, 0xeb2, 0xec2, 0xe9b, 0xec0, 0xe99, 0xe9a, 0xec0, 0xea7, 0xea5, 0xeb2,
0xe9b, 0xebd, 0xe87, 0xea2, 0xeb2, 0xe87, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe84, 0xeb5,
0xea7, 0xea5, 0xecd, 0xe94, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94,
0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xe84, 0xeb5, 0xea7, 0xea5, 0xecd, 0xe94, 0xeb2,
0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99,
0xe84, 0xeb5, 0xea7, 0xea5, 0xecd, 0xe94, 0xeb2, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0x200b, 0xec0, 0xea3, 0x200b, 0xead, 0xeb9, 0x200b, 0xe99, 0xeb4, 0x200b, 0xea2, 0xebb,
0xe87, 0xec0, 0xea7, 0xea5, 0xeb2, 0x20, 0xec2, 0xea3, 0xe97, 0xeb5, 0xe95, 0xeb2,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe8a, 0xeb2, 0x200b, 0xeae, 0xeb2, 0x200b,
0xea5, 0xeb4, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94,
0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xe8a, 0xeb2, 0x200b, 0xeae, 0xeb2,
0x200b, 0xea5, 0xeb4, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0,
0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0xe8a, 0xeb2, 0x200b, 0xeae,
0xeb2, 0x200b, 0xea5, 0xeb4, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe8a, 0xeb2, 0xea1,
0xebb, 0xea7, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96,
0xeb2, 0xe99, 0xe8a, 0xeb2, 0xea1, 0xebb, 0xea7, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5,
0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xe8a, 0xeb2, 0xea1, 0xebb, 0xea7,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xec0, 0xe8a, 0x200b, 0xec0, 0xe8a, 0x200b,
0xea5, 0x200b, 0xeaa, 0xecc, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xeaa, 0xeb4,
0xe87, 0x200b, 0xe81, 0xeb0, 0x200b, 0xec2, 0xe9b, 0xec0, 0xea7, 0xea5, 0xeb2, 0xeab,
0xea1, 0xeb9, 0xec8, 0xec0, 0xe81, 0xeb2, 0xeb0, 0xec2, 0xe8a, 0xec2, 0xea5, 0xea1,
0xead, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe88, 0xecd, 0xec0, 0xe88, 0xe8d, 0xec3,
0xe95, 0xec9, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe8a, 0xeb8, 0x200b, 0xea3,
0xeb4, 0x200b, 0xe99, 0xeb2, 0xea1, 0xec0, 0xea7, 0xea5, 0xeb2, 0x20, 0xec2, 0xe8a,
0xea7, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe97, 0xeb2, 0xeae, 0xeb4, 0xe95, 0xeb4,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xec4, 0xe97, 0x200b, 0xec0, 0xe9b, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b,
0xe96, 0xeb2, 0xe99, 0x200b, 0xec4, 0xe97, 0x200b, 0xec0, 0xe9b, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xe95, 0xead, 0xe99, 0x200b, 0xe97, 0xec8, 0x200b, 0xebd, 0xe87,
0x200b, 0xec4, 0xe97, 0x200b, 0xec0, 0xe9b, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe97, 0xeb2,
0xe88, 0xeb4, 0xe81, 0xeb4, 0xeaa, 0xe96, 0xeb2, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2,
0xec2, 0xe95, 0xec0, 0xe81, 0xec0, 0xea5, 0xebb, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2,
0xe95, 0xead, 0xe87, 0xe81, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94,
0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xe95, 0xead, 0xe87, 0xe81, 0xeb2, 0xec0, 0xea7,
0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xe95, 0xead,
0xe87, 0xe81, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe8a, 0xeb8, 0xe81, 0xec0, 0xea7,
0xea5, 0xeb2, 0xe95, 0xea7, 0xe81, 0xec0, 0xea1, 0xe99, 0xeb4, 0xeaa, 0xe96, 0xeb2,
0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2,
0xe99, 0xe95, 0xea7, 0xe81, 0xec0, 0xea1, 0xe99, 0xeb4, 0xeaa, 0xe96, 0xeb2, 0xe99,
0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99,
0xe95, 0xea7, 0xe81, 0xec0, 0xea1, 0xe99, 0xeb4, 0xeaa, 0xe96, 0xeb2, 0xe99, 0xec0,
0xea7, 0xea5, 0xeb2, 0xe95, 0xeb9, 0xea7, 0xeb2, 0xea5, 0xeb9, 0x200b, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0xead, 0xeb9, 0x200b, 0xea3, 0xeb9, 0x200b, 0xe81, 0xea7,
0xe8d, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95,
0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xead, 0xeb9, 0x200b, 0xea3, 0xeb9, 0x200b,
0xe81, 0xea7, 0xe8d, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b,
0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0x200b, 0xead, 0xeb9, 0x200b, 0xea3,
0xeb9, 0x200b, 0xe81, 0xea7, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xead, 0xeb8, 0xeaa,
0xec0, 0xe9a, 0xe81, 0xeb4, 0xe94, 0xeaa, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xec0, 0xea7,
0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xead, 0xeb8,
0xeaa, 0xec0, 0xe9a, 0xe81, 0xeb4, 0xe94, 0xeaa, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xec0,
0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xead,
0xeb8, 0xeaa, 0xec0, 0xe9a, 0xe81, 0xeb4, 0xe94, 0xeaa, 0xeb0, 0xe96, 0xeb2, 0xe99,
0xec0, 0xea7, 0xea5, 0xeb2, 0xea7, 0xeb2, 0xe99, 0xeb9, 0xead, 0xeb2, 0xe95, 0xeb9,
0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99,
0xea7, 0xeb2, 0xe99, 0xeb9, 0xead, 0xeb2, 0xe95, 0xeb9, 0xec0, 0xea7, 0xea5, 0xeb2,
0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xea7, 0xeb2, 0xe99, 0xeb9,
0xead, 0xeb2, 0xe95, 0xeb9, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xec0, 0xea7,
0x200b, 0xec0, 0xe99, 0x200b, 0xe8a, 0xeb9, 0x200b, 0xec0, 0xead, 0x200b, 0xea5, 0xeb2,
0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb2, 0xe94, 0xeb5, 0xec2, 0xea7, 0xeaa, 0xe95,
0xebb, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96,
0xeb2, 0xe99, 0xea5, 0xeb2, 0xe94, 0xeb5, 0xec2, 0xea7, 0xeaa, 0xe95, 0xebb, 0xe81,
0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99,
0xea5, 0xeb2, 0xe94, 0xeb5, 0xec2, 0xea7, 0xeaa, 0xe95, 0xebb, 0xe81, 0xec0, 0xea7,
0xea5, 0xeb2, 0xec2, 0xea7, 0xec2, 0xe81, 0xe81, 0xea3, 0xeb2, 0xe94, 0xec0, 0xea7,
0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xec2, 0xea7,
0xec2, 0xe81, 0xe81, 0xea3, 0xeb2, 0xe94, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0,
0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xec2, 0xea7, 0xec2, 0xe81, 0xe81, 0xea3,
0xeb2, 0xe94, 0xec0, 0xea7, 0xea5, 0xeb2, 0x20, 0xea7, 0xead, 0xeaa, 0xec2, 0xe95,
0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xec0, 0xe81, 0xeb2, 0xeb0, 0xec0, 0xea7, 0xe81,
0xec0, 0xea7, 0xea5, 0xeb2, 0xea7, 0xeb2, 0xea5, 0xea5, 0xeb4, 0xeaa, 0x20, 0xec1,
0xea5, 0xeb0, 0x20, 0xe9f, 0xeb9, 0xe95, 0xeb9, 0xe99, 0xeb2, 0xec0, 0xea7, 0xea5,
0xeb2, 0xea2, 0xeb2, 0xe81, 0xeb9, 0xe94, 0xeaa, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1,
0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xea2, 0xeb2, 0xe81, 0xeb9, 0xe94,
0xeaa, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead,
0xe99, 0xea2, 0xeb2, 0xe81, 0xeb9, 0xe94, 0xeaa, 0xec0, 0xea7, 0xea5, 0xeb2, 0xec0,
0xea2, 0xe81, 0xeb2, 0xec0, 0xe95, 0xea5, 0xeb4, 0xe99, 0xec0, 0xe9a, 0xeb5, 0xe81,
0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99,
0xec0, 0xea2, 0xe81, 0xeb2, 0xec0, 0xe95, 0xea5, 0xeb4, 0xe99, 0xec0, 0xe9a, 0xeb5,
0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead,
0xe99, 0xec0, 0xea2, 0xe81, 0xeb2, 0xec0, 0xe95, 0xea5, 0xeb4, 0xe99, 0xec0, 0xe9a,
0xeb5, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea2, 0xeb9, 0xe84, 0xead, 0xe99, 0x41,
0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x101, 0x6e, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c,
0x101, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x75, 0x6d, 0x101, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x44, 0x69, 0x65,
0x6e, 0x76, 0x69, 0x64, 0x101, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x73, 0x20,
0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x52, 0x69, 0x65, 0x74, 0x75, 0x6d, 0x101, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x52, 0x69, 0x65, 0x74, 0x75,
0x6d, 0x101, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65,
0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x52, 0x69, 0x65,
0x74, 0x75, 0x6d, 0x101, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x41, 0x13c, 0x61, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x41, 0x13c, 0x61, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65,
0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x13c, 0x61,
0x73, 0x6b, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x65, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0x65, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x65,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c, 0x61, 0x69,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x101, 0x6c, 0x61, 0x69, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x101,
0x6c, 0x61, 0x69, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x75,
0x6d, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74,
0x72, 0x75, 0x6d, 0x75, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x75, 0x6d,
0x75, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x4b, 0x61, 0x6c, 0x6e, 0x75, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x4b, 0x61, 0x6c, 0x6e, 0x75, 0x20, 0x7a, 0x69, 0x65, 0x6d,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x61, 0x6c, 0x6e,
0x75, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x4b, 0x6c, 0x75, 0x73, 0x101, 0x20, 0x6f, 0x6b, 0x65,
0x101, 0x6e, 0x61, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x6c, 0x75,
0x73, 0x101, 0x20, 0x6f, 0x6b, 0x65, 0x101, 0x6e, 0x61, 0x20, 0x7a, 0x69,
0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x6c,
0x75, 0x73, 0x101, 0x20, 0x6f, 0x6b, 0x65, 0x101, 0x6e, 0x61, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x61, 0x73, 0x20, 0x7a,
0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41,
0x6e, 0x61, 0x64, 0x69, 0x72, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x70, 0x69,
0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x70, 0x69,
0x6a, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x41, 0x70, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x41, 0x72, 0x101, 0x62, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x70, 0x75, 0x73,
0x73, 0x61, 0x6c, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41,
0x72, 0x101, 0x62, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x70, 0x75, 0x73, 0x73,
0x61, 0x6c, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x72, 0x101, 0x62, 0x69, 0x6a, 0x61,
0x73, 0x20, 0x70, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x73, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x12b, 0x6e, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x12b, 0x6e,
0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x12b, 0x6e, 0x61,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x52, 0x69, 0x65, 0x74, 0x75, 0x6d, 0x61, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x12b, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x52, 0x69, 0x65, 0x74, 0x75, 0x6d, 0x61, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x12b, 0x6e, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x52, 0x69, 0x65, 0x74, 0x75, 0x6d,
0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x12b, 0x6e, 0x61, 0x73, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x41, 0x72, 0x6d, 0x113, 0x6e, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x41, 0x72, 0x6d, 0x113, 0x6e, 0x69, 0x6a, 0x61, 0x73,
0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x41, 0x72, 0x6d, 0x113, 0x6e, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6a,
0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6a, 0x61,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x101, 0x6c, 0x69, 0x6a,
0x61, 0x73, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c, 0x61, 0x69,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74, 0x72,
0x101, 0x6c, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x101, 0x6c, 0x61, 0x69, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x101,
0x6c, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x101,
0x6c, 0x61, 0x69, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x101,
0x6c, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x101,
0x6c, 0x61, 0x69, 0x73, 0x20, 0x72, 0x69, 0x65, 0x74, 0x75, 0x6d, 0x75,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x101,
0x6c, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x101,
0x6c, 0x61, 0x69, 0x73, 0x20, 0x72, 0x69, 0x65, 0x74, 0x75, 0x6d, 0x75,
0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x101, 0x6c, 0x69, 0x6a, 0x61, 0x73,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c, 0x61, 0x69, 0x73, 0x20,
0x72, 0x69, 0x65, 0x74, 0x75, 0x6d, 0x75, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73,
0x74, 0x72, 0x101, 0x6c, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x61, 0x75, 0x73,
0x74, 0x72, 0x75, 0x6d, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41,
0x75, 0x73, 0x74, 0x72, 0x101, 0x6c, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x61,
0x75, 0x73, 0x74, 0x72, 0x75, 0x6d, 0x75, 0x20, 0x7a, 0x69, 0x65, 0x6d,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74,
0x72, 0x101, 0x6c, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x61, 0x75, 0x73, 0x74,
0x72, 0x75, 0x6d, 0x75, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x101,
0x6c, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x72, 0x69, 0x65, 0x74, 0x75, 0x6d,
0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74, 0x72,
0x101, 0x6c, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x72, 0x69, 0x65, 0x74, 0x75,
0x6d, 0x75, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x101, 0x6c, 0x69, 0x6a,
0x61, 0x73, 0x20, 0x72, 0x69, 0x65, 0x74, 0x75, 0x6d, 0x75, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x17e, 0x101, 0x6e, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x7a, 0x65, 0x72, 0x62,
0x61, 0x69, 0x64, 0x17e, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65,
0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x69, 0x64, 0x17e, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x41, 0x7a, 0x6f, 0x72, 0x75, 0x20, 0x73, 0x61, 0x6c, 0x75, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x41, 0x7a, 0x6f, 0x72, 0x75, 0x20, 0x73, 0x61,
0x6c, 0x75, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x41, 0x7a, 0x6f, 0x72, 0x75, 0x20, 0x73, 0x61, 0x6c,
0x75, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x161, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x161, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x42, 0x75, 0x74,
0x101, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x42, 0x6f,
0x6c, 0x12b, 0x76, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x42, 0x72, 0x61, 0x7a, 0x12b, 0x6c, 0x69, 0x6a, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x42, 0x72, 0x61, 0x7a, 0x12b, 0x6c, 0x69,
0x6a, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x42, 0x72, 0x61, 0x7a, 0x12b, 0x6c, 0x69, 0x6a,
0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x6a, 0x61, 0x73,
0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x61, 0x62, 0x6f, 0x76, 0x65,
0x72, 0x64, 0x65, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x61,
0x62, 0x6f, 0x76, 0x65, 0x72, 0x64, 0x65, 0x73, 0x20, 0x7a, 0x69, 0x65,
0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x61, 0x62,
0x6f, 0x76, 0x65, 0x72, 0x64, 0x65, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x10c, 0x61, 0x6d,
0x6f, 0x72, 0x72, 0x61, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x10c, 0x65, 0x74, 0x65, 0x6d, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x10c, 0x65, 0x74, 0x65, 0x6d, 0x61,
0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x10c, 0x65, 0x74, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x10c,
0x12b, 0x6c, 0x65, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x10c, 0x12b,
0x6c, 0x65, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x10c, 0x12b, 0x6c, 0x65, 0x73, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x136,
0x12b, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x136, 0x12b,
0x6e, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x136, 0x12b, 0x6e, 0x61, 0x73, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x5a,
0x69, 0x65, 0x6d, 0x73, 0x76, 0x113, 0x74, 0x6b, 0x75, 0x20, 0x73, 0x61,
0x6c, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x6f, 0x6b,
0x6f, 0x73, 0x75, 0x20, 0x28, 0x4b, 0x12b, 0x6c, 0x69, 0x6e, 0x67, 0x61,
0x29, 0x20, 0x73, 0x61, 0x6c, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a,
0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x61,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x4b, 0x75, 0x6b, 0x61, 0x20, 0x73, 0x61, 0x6c, 0x75,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x75, 0x6b, 0x61, 0x20, 0x73,
0x61, 0x6c, 0x75, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x4b, 0x75, 0x6b, 0x61, 0x20, 0x73, 0x61, 0x6c,
0x75, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x4b, 0x75, 0x62, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x4b, 0x75, 0x62, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x75, 0x62, 0x61,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x44, 0x65, 0x69, 0x76, 0x69, 0x73, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x44, 0x69, 0x6d, 0x6f, 0x6e, 0x64, 0x69,
0x72, 0x76, 0x69, 0x6c, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x41, 0x75, 0x73, 0x74, 0x72, 0x75, 0x6d, 0x74, 0x69, 0x6d, 0x6f, 0x72,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4c, 0x69, 0x65, 0x6c,
0x64, 0x69, 0x65, 0x6e, 0x75, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4c, 0x69, 0x65, 0x6c, 0x64, 0x69, 0x65,
0x6e, 0x75, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65,
0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4c, 0x69, 0x65,
0x6c, 0x64, 0x69, 0x65, 0x6e, 0x75, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x73,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x45, 0x6b, 0x76, 0x61, 0x64, 0x6f, 0x72, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c,
0x65, 0x69, 0x72, 0x6f, 0x70, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c, 0x65, 0x69, 0x72, 0x6f,
0x70, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c, 0x65,
0x69, 0x72, 0x6f, 0x70, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74,
0x72, 0x75, 0x6d, 0x65, 0x69, 0x72, 0x6f, 0x70, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x75, 0x6d, 0x65,
0x69, 0x72, 0x6f, 0x70, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74, 0x72,
0x75, 0x6d, 0x65, 0x69, 0x72, 0x6f, 0x70, 0x61, 0x73, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41,
0x75, 0x73, 0x74, 0x72, 0x75, 0x6d, 0x65, 0x69, 0x72, 0x6f, 0x70, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x20, 0x6a, 0x6f, 0x73, 0x6c,
0x61, 0x20, 0x28, 0x46, 0x45, 0x54, 0x29, 0x52, 0x69, 0x65, 0x74, 0x75,
0x6d, 0x65, 0x69, 0x72, 0x6f, 0x70, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x52, 0x69, 0x65, 0x74, 0x75, 0x6d, 0x65, 0x69, 0x72, 0x6f,
0x70, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x52, 0x69, 0x65, 0x74, 0x75, 0x6d, 0x65, 0x69,
0x72, 0x6f, 0x70, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x46, 0x6f, 0x6c, 0x6b, 0x6c,
0x65, 0x6e, 0x64, 0x61, 0x20, 0x28, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e,
0x75, 0x29, 0x20, 0x73, 0x61, 0x6c, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x65, 0x6e, 0x64, 0x61, 0x20, 0x28,
0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x75, 0x29, 0x20, 0x73, 0x61, 0x6c,
0x75, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x65, 0x6e, 0x64, 0x61, 0x20,
0x28, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x75, 0x29, 0x20, 0x73, 0x61,
0x6c, 0x75, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x20, 0x7a, 0x69, 0x65,
0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x46, 0x69, 0x64,
0x17e, 0x69, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x69, 0x6a, 0x61,
0x73, 0x20, 0x47, 0x76, 0x69, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x69, 0x6a, 0x61, 0x73,
0x20, 0x44, 0x69, 0x65, 0x6e, 0x76, 0x69, 0x64, 0x6a, 0x16b, 0x72, 0x75,
0x20, 0x75, 0x6e, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69,
0x6b, 0x61, 0x73, 0x20, 0x74, 0x65, 0x72, 0x69, 0x74, 0x6f, 0x72, 0x69,
0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x47, 0x61, 0x6c,
0x61, 0x70, 0x61, 0x67, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x47,
0x61, 0x6d, 0x62, 0x6a, 0x113, 0x20, 0x73, 0x61, 0x6c, 0x75, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x6a, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x6a,
0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x6a, 0x61, 0x73, 0x20,
0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x61, 0x20, 0x73, 0x61,
0x6c, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x47, 0x72, 0x69, 0x6e,
0x69, 0x10d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75,
0x73, 0x74, 0x72, 0x75, 0x6d, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x65, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73,
0x74, 0x72, 0x75, 0x6d, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x65, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x75, 0x6d, 0x67, 0x72,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x73, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x52, 0x69,
0x65, 0x74, 0x75, 0x6d, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x65, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x52, 0x69, 0x65, 0x74,
0x75, 0x6d, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x73,
0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x52, 0x69, 0x65, 0x74, 0x75, 0x6d, 0x67, 0x72, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x65, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x50, 0x65, 0x72, 0x73, 0x69,
0x6a, 0x61, 0x73, 0x20, 0x6c, 0x12b, 0x10d, 0x61, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x47, 0x61, 0x6a, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x75, 0x2013, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x48, 0x61,
0x76, 0x61, 0x6a, 0x75, 0x2013, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x75, 0x20,
0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x48, 0x61, 0x76, 0x61, 0x6a, 0x75, 0x2013, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x75, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x48, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e, 0x67, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x48, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e,
0x67, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x48, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e, 0x67, 0x61,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x48, 0x6f, 0x76, 0x64, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x48, 0x6f, 0x76, 0x64, 0x61, 0x73, 0x20, 0x7a, 0x69,
0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x48, 0x6f,
0x76, 0x64, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x49, 0x6e, 0x64, 0x69, 0x6a, 0x61,
0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x49, 0x6e, 0x64, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x6f, 0x6b,
0x65, 0x101, 0x6e, 0x61, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x49, 0x6e,
0x64, 0x6f, 0x137, 0x12b, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c, 0x69, 0x6e, 0x64, 0x6f,
0x6e, 0x113, 0x7a, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x75, 0x6d, 0x69, 0x6e, 0x64, 0x6f,
0x6e, 0x113, 0x7a, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x52, 0x69, 0x65, 0x74, 0x75, 0x6d, 0x69, 0x6e, 0x64, 0x6f, 0x6e,
0x113, 0x7a, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x49, 0x72, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x49, 0x72, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x49, 0x72, 0x101, 0x6e, 0x61,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x61,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x49, 0x7a, 0x72, 0x61, 0x113, 0x6c, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x49, 0x7a, 0x72, 0x61, 0x113, 0x6c, 0x61,
0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x49, 0x7a, 0x72, 0x61, 0x113, 0x6c, 0x61, 0x73, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x4a, 0x61, 0x70, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x4a, 0x61, 0x70, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65,
0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4a, 0x61, 0x70,
0x101, 0x6e, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70,
0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x61, 0x73, 0x2d, 0x4b, 0x61,
0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x61, 0x73, 0x2d, 0x4b, 0x61, 0x6d, 0x10d, 0x61, 0x74,
0x73, 0x6b, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61,
0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x61, 0x73, 0x2d, 0x4b, 0x61, 0x6d,
0x10d, 0x61, 0x74, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x61, 0x7a,
0x61, 0x68, 0x73, 0x74, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x75, 0x6d, 0x6b, 0x61, 0x7a,
0x61, 0x68, 0x73, 0x74, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x52, 0x69, 0x65, 0x74, 0x75, 0x6d, 0x6b, 0x61, 0x7a, 0x61,
0x68, 0x73, 0x74, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x4b, 0x6f, 0x72, 0x65, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x4b, 0x6f, 0x72, 0x65, 0x6a, 0x61, 0x73, 0x20, 0x7a, 0x69,
0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x6f,
0x72, 0x65, 0x6a, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x6f, 0x73, 0x72, 0x61,
0x65, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73,
0x6b, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61,
0x72, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x69, 0x72, 0x67, 0x69,
0x7a, 0x73, 0x74, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x4c, 0x61, 0x69, 0x6e, 0x61, 0x73, 0x20, 0x73, 0x61, 0x6c, 0x75,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4c, 0x6f, 0x72, 0x64, 0x61, 0x20,
0x48, 0x61, 0x76, 0x61, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x4c, 0x6f, 0x72, 0x64, 0x61, 0x20, 0x48, 0x61,
0x76, 0x61, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65,
0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4c, 0x6f, 0x72,
0x64, 0x61, 0x20, 0x48, 0x61, 0x76, 0x61, 0x20, 0x73, 0x61, 0x6c, 0x61,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x61,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x4d, 0x61, 0x6c, 0x61, 0x69, 0x7a, 0x69, 0x6a, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4d, 0x61, 0x6c, 0x64, 0x12b,
0x76, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4d,
0x61, 0x72, 0x137, 0x12b, 0x7a, 0x61, 0x20, 0x73, 0x61, 0x6c, 0x75, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4d, 0x101, 0x72, 0x161, 0x61, 0x6c, 0x61,
0x20, 0x73, 0x61, 0x6c, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4d,
0x61, 0x75, 0x72, 0x12b, 0x63, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x4d, 0x61, 0x75, 0x72, 0x12b, 0x63, 0x69, 0x6a, 0x61,
0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x4d, 0x61, 0x75, 0x72, 0x12b, 0x63, 0x69, 0x6a, 0x61, 0x73,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x4d, 0x6f, 0x73, 0x6f, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x73, 0x20,
0x4b, 0x6c, 0x75, 0x73, 0x101, 0x20, 0x6f, 0x6b, 0x65, 0x101, 0x6e, 0x61,
0x20, 0x70, 0x69, 0x65, 0x6b, 0x72, 0x61, 0x73, 0x74, 0x65, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61,
0x73, 0x20, 0x4b, 0x6c, 0x75, 0x73, 0x101, 0x20, 0x6f, 0x6b, 0x65, 0x101,
0x6e, 0x61, 0x20, 0x70, 0x69, 0x65, 0x6b, 0x72, 0x61, 0x73, 0x74, 0x65,
0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x4b,
0x6c, 0x75, 0x73, 0x101, 0x20, 0x6f, 0x6b, 0x65, 0x101, 0x6e, 0x61, 0x20,
0x70, 0x69, 0x65, 0x6b, 0x72, 0x61, 0x73, 0x74, 0x65, 0x73, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74,
0x6f, 0x72, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74,
0x6f, 0x72, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4d, 0x61, 0x73, 0x6b, 0x61, 0x76,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4d, 0x61, 0x73, 0x6b,
0x61, 0x76, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4d, 0x61, 0x73, 0x6b, 0x61, 0x76, 0x61,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x4d, 0x6a, 0x61, 0x6e, 0x6d, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x4e, 0x65, 0x70, 0x101, 0x6c, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x4a, 0x61, 0x75, 0x6e, 0x6b, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x4a, 0x61, 0x75, 0x6e, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x69, 0x6a, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4a, 0x61, 0x75, 0x6e, 0x6b, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4a, 0x61,
0x75, 0x6e, 0x7a, 0x113, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x4a, 0x61, 0x75, 0x6e, 0x7a, 0x113, 0x6c, 0x61,
0x6e, 0x64, 0x65, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4a, 0x61, 0x75, 0x6e, 0x7a, 0x113, 0x6c,
0x61, 0x6e, 0x64, 0x65, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x145, 0x16b, 0x66, 0x61, 0x75,
0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x145, 0x16b, 0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e,
0x64, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x145, 0x16b, 0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c,
0x65, 0x6e, 0x64, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4e, 0x69, 0x75, 0x65, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x61, 0x73, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x61, 0x73,
0x20, 0x73, 0x61, 0x6c, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x61, 0x73, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x73, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x75, 0x20, 0x64, 0x69, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x146, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x75, 0x20, 0x64, 0x69,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x146, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65,
0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x75, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x146, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4f, 0x6d, 0x73, 0x6b, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4f, 0x6d, 0x73, 0x6b, 0x61, 0x73,
0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x4f, 0x6d, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x101, 0x6e, 0x61, 0x73, 0x20,
0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x50, 0x61, 0x6c, 0x61, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x50,
0x61, 0x70, 0x75, 0x61, 0x2d, 0x4a, 0x61, 0x75, 0x6e, 0x67, 0x76, 0x69,
0x6e, 0x65, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x50,
0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x61,
0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x61, 0x73,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x50, 0x65, 0x72, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x50, 0x65, 0x72, 0x75, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x50, 0x65, 0x72, 0x75, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x46,
0x69, 0x6c, 0x69, 0x70, 0x12b, 0x6e, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x12b, 0x6e, 0x75, 0x20, 0x7a, 0x69,
0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x12b, 0x6e, 0x75, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x46, 0x113, 0x6e, 0x69,
0x6b, 0x73, 0x61, 0x20, 0x73, 0x61, 0x6c, 0x75, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x53, 0x65, 0x6e, 0x70, 0x6a, 0x113, 0x72, 0x61, 0x73, 0x20,
0x75, 0x6e, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x53, 0x65, 0x6e, 0x70, 0x6a, 0x113,
0x72, 0x61, 0x73, 0x20, 0x75, 0x6e, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c,
0x6f, 0x6e, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x53, 0x65, 0x6e, 0x70, 0x6a, 0x113, 0x72,
0x61, 0x73, 0x20, 0x75, 0x6e, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f,
0x6e, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x50, 0x69, 0x74, 0x6b, 0x113, 0x72, 0x6e,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x50, 0x6f, 0x6e, 0x61,
0x70, 0x113, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x50, 0x68, 0x65, 0x6e,
0x6a, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x52,
0x65, 0x69, 0x6e, 0x6a, 0x6f, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x52, 0x6f, 0x74, 0x65, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x12b, 0x6e, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x12b,
0x6e, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x12b, 0x6e, 0x61,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x73, 0x20,
0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x53, 0x61, 0x6d, 0x6f,
0x61, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x53, 0x65, 0x69,
0x161, 0x65, 0x13c, 0x75, 0x20, 0x73, 0x61, 0x6c, 0x75, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x16b, 0x72, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x5a, 0x101, 0x6c, 0x61, 0x6d,
0x61, 0x6e, 0x61, 0x20, 0x73, 0x61, 0x6c, 0x75, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x44, 0x69, 0x65, 0x6e, 0x76, 0x69, 0x64, 0x64, 0x17e, 0x6f,
0x72, 0x64, 0x17e, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x160, 0x6f, 0x76, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x54, 0x61, 0x69, 0x74, 0x69, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x54, 0x61, 0x69, 0x62, 0x65, 0x69, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x54, 0x61, 0x69, 0x62, 0x65, 0x69, 0x20, 0x7a, 0x69, 0x65,
0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x54, 0x61, 0x69,
0x62, 0x65, 0x69, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x54, 0x61, 0x64, 0x17e, 0x69, 0x6b, 0x69,
0x73, 0x74, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x54, 0x6f, 0x6e, 0x67,
0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x10c, 0x16b, 0x6b, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x54,
0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x101, 0x6e, 0x61,
0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x101, 0x6e, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x55, 0x72, 0x75, 0x67, 0x76, 0x61,
0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x55, 0x72, 0x75,
0x67, 0x76, 0x61, 0x6a, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x55, 0x72, 0x75, 0x67, 0x76,
0x61, 0x6a, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x74, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x101, 0x6e, 0x61, 0x73,
0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x101, 0x6e, 0x61,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20,
0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x56, 0x65, 0x6e,
0x65, 0x63, 0x75, 0x113, 0x6c, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x61, 0x73, 0x20, 0x7a, 0x69,
0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x61, 0x73, 0x20,
0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x56, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x56, 0x65, 0x69,
0x6b, 0x61, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x56, 0x6f, 0x6c, 0x69, 0x73, 0x61, 0x73, 0x20, 0x75, 0x6e,
0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x61, 0x73,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4a,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4a, 0x75, 0x6b, 0x6f,
0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4e, 0x74, 0xe1,
0x6e, 0x67, 0x6f, 0x20, 0x79, 0x61, 0x20, 0x4c, 0x75, 0x62, 0x75, 0x6d,
0x62, 0x61, 0x73, 0x68, 0x69, 0x4e, 0x74, 0xe1, 0x6e, 0x67, 0x6f, 0x20,
0x79, 0x61, 0x20, 0x41, 0x66, 0x72, 0xed, 0x6b, 0x61, 0x20, 0x79, 0x61,
0x20, 0x190, 0x301, 0x73, 0x69, 0x74, 0x61, 0x4e, 0x74, 0xe1, 0x6e, 0x67,
0x6f, 0x20, 0x79, 0x61, 0x20, 0x41, 0x66, 0x72, 0xed, 0x6b, 0x61, 0x20,
0x79, 0x61, 0x20, 0x53, 0x69, 0x64, 0x69, 0x4e, 0x74, 0xe1, 0x6e, 0x67,
0x6f, 0x20, 0x79, 0x61, 0x20, 0x4c, 0x6f, 0x6e, 0x64, 0x6f, 0x6e, 0x69,
0x4e, 0x74, 0xe1, 0x6e, 0x67, 0x6f, 0x20, 0x79, 0x61, 0x20, 0x53, 0x65,
0x79, 0x73, 0x68, 0x65, 0x6c, 0x41, 0x6b, 0x6f, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x41, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x69, 0x6e, 0x69, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x41, 0x6b, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x66, 0x67, 0x61,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x69, 0x6e, 0x117, 0x73, 0x20,
0x41, 0x66, 0x72, 0x69, 0x6b, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x52, 0x79, 0x74, 0x173, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x50, 0x69, 0x65,
0x74, 0x173, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x61, 0x6b, 0x61, 0x72, 0x173, 0x20,
0x41, 0x66, 0x72, 0x69, 0x6b, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x56, 0x61, 0x6b, 0x61, 0x72, 0x173, 0x20, 0x41, 0x66, 0x72,
0x69, 0x6b, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x61, 0x6b, 0x61, 0x72, 0x173,
0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41,
0x6c, 0x69, 0x61, 0x73, 0x6b, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x41, 0x6c, 0x69, 0x61, 0x73, 0x6b, 0x6f, 0x73, 0x20, 0x17e,
0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x41, 0x6c, 0x69, 0x61, 0x73, 0x6b, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41,
0x6c, 0x6d, 0x61, 0x74, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65,
0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x6c,
0x6d, 0x61, 0x74, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0x117, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x117, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x117, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x160, 0x69, 0x61, 0x75,
0x72, 0x117, 0x73, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x73,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x160, 0x69, 0x61, 0x75, 0x72, 0x117, 0x73, 0x20, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x6f, 0x73, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x160, 0x69, 0x61, 0x75, 0x72, 0x117, 0x73, 0x20, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x73, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x160, 0x69, 0x61, 0x75, 0x72, 0x117, 0x73,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x73, 0x20, 0x72, 0x79,
0x74, 0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x160, 0x69, 0x61,
0x75, 0x72, 0x117, 0x73, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x6f,
0x73, 0x20, 0x72, 0x79, 0x74, 0x173, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x160, 0x69, 0x61, 0x75,
0x72, 0x117, 0x73, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x73,
0x20, 0x72, 0x79, 0x74, 0x173, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x160, 0x69, 0x61, 0x75,
0x72, 0x117, 0x73, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x73,
0x20, 0x6b, 0x61, 0x6c, 0x6e, 0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x160, 0x69, 0x61, 0x75, 0x72, 0x117, 0x73, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x6f, 0x73, 0x20, 0x6b, 0x61, 0x6c, 0x6e, 0x173, 0x20,
0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x160, 0x69, 0x61, 0x75, 0x72, 0x117, 0x73, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x6f, 0x73, 0x20, 0x6b, 0x61, 0x6c, 0x6e, 0x173, 0x20,
0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x160, 0x69, 0x61, 0x75, 0x72, 0x117, 0x73, 0x20, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x6f, 0x73, 0x20, 0x52, 0x61, 0x6d, 0x69, 0x6f,
0x6a, 0x6f, 0x20, 0x76, 0x61, 0x6e, 0x64, 0x65, 0x6e, 0x79, 0x6e, 0x6f,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x160, 0x69, 0x61, 0x75, 0x72,
0x117, 0x73, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x73, 0x20,
0x52, 0x61, 0x6d, 0x69, 0x6f, 0x6a, 0x6f, 0x20, 0x76, 0x61, 0x6e, 0x64,
0x65, 0x6e, 0x79, 0x6e, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x160, 0x69, 0x61, 0x75, 0x72,
0x117, 0x73, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x73, 0x20,
0x52, 0x61, 0x6d, 0x69, 0x6f, 0x6a, 0x6f, 0x20, 0x76, 0x61, 0x6e, 0x64,
0x65, 0x6e, 0x79, 0x6e, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x6e, 0x61, 0x64,
0x79, 0x72, 0x117, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x117, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x6e, 0x61,
0x64, 0x79, 0x72, 0x117, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x70, 0x69, 0x6a,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x70, 0x69,
0x6a, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x70, 0x69, 0x6a, 0x6f, 0x73, 0x20,
0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x20, 0x17e, 0x69, 0x65, 0x6d,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x6b, 0x74,
0x61, 0x75, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x117, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x6b, 0x74, 0x6f, 0x62,
0x117, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x117, 0x73, 0x20,
0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x41, 0x72, 0x61, 0x62, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x72, 0x61, 0x62, 0x69, 0x6a, 0x6f,
0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x41, 0x72, 0x61, 0x62, 0x69, 0x6a, 0x6f, 0x73, 0x20,
0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x61, 0x6b, 0x61,
0x72, 0x173, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x61, 0x6b, 0x61,
0x72, 0x173, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x6f,
0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x56, 0x61, 0x6b, 0x61, 0x72, 0x173, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41,
0x72, 0x6d, 0x117, 0x6e, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x41, 0x72, 0x6d, 0x117, 0x6e, 0x69, 0x6a, 0x6f, 0x73,
0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x41, 0x72, 0x6d, 0x117, 0x6e, 0x69, 0x6a, 0x6f, 0x73, 0x20,
0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x6f, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x6f, 0x20,
0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x6f, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x69, 0x6e, 0x117, 0x73, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x69, 0x6e, 0x117, 0x73,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x6f, 0x73,
0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x69, 0x6e, 0x117, 0x73, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x6f, 0x73, 0x20,
0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x69, 0x6e, 0x117, 0x73, 0x20,
0x76, 0x61, 0x6b, 0x61, 0x72, 0x173, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x69, 0x6e, 0x117, 0x73, 0x20, 0x76,
0x61, 0x6b, 0x61, 0x72, 0x173, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x69, 0x6e, 0x117, 0x73, 0x20, 0x76, 0x61, 0x6b, 0x61, 0x72, 0x173, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x6f, 0x73, 0x20,
0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x52, 0x79, 0x74, 0x173, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x52, 0x79, 0x74, 0x173, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x52, 0x79, 0x74, 0x173, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x6f, 0x73, 0x20,
0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x56, 0x61, 0x6b, 0x61, 0x72, 0x173, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x56, 0x61, 0x6b, 0x61, 0x72, 0x173, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x17e, 0x69,
0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56,
0x61, 0x6b, 0x61, 0x72, 0x173, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x61, 0x69, 0x64, 0x17e, 0x61, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x17e,
0x61, 0x6e, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69,
0x64, 0x17e, 0x61, 0x6e, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x7a, 0x6f, 0x72,
0x173, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x41, 0x7a, 0x6f, 0x72, 0x173, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20,
0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x41, 0x7a, 0x6f, 0x72, 0x173, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20,
0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x6f,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x161, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x161, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x42, 0x75, 0x74, 0x61,
0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x42, 0x6f, 0x6c,
0x69, 0x76, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x6a, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c,
0x69, 0x6a, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c,
0x69, 0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x42, 0x72, 0x75, 0x6e, 0x117,
0x6a, 0x61, 0x75, 0x73, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x61, 0x6c,
0x61, 0x6d, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x17d, 0x61,
0x6c, 0x69, 0x6f, 0x6a, 0x6f, 0x20, 0x4b, 0x79, 0x161, 0x75, 0x6c, 0x69,
0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x17d, 0x61, 0x6c, 0x69,
0x6f, 0x6a, 0x6f, 0x20, 0x4b, 0x79, 0x161, 0x75, 0x6c, 0x69, 0x6f, 0x20,
0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x17d, 0x61, 0x6c, 0x69, 0x6f, 0x6a, 0x6f, 0x20, 0x4b, 0x79, 0x161,
0x75, 0x6c, 0x69, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x65, 0x69, 0x73, 0x69,
0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x10c, 0x61, 0x6d, 0x6f,
0x72, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x10c, 0x61, 0x74,
0x61, 0x6d, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x10c, 0x61,
0x74, 0x61, 0x6d, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x10c, 0x61, 0x74, 0x61, 0x6d, 0x6f,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x10c, 0x69, 0x6c, 0x117, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x10c, 0x69, 0x6c, 0x117, 0x73, 0x20, 0x17e, 0x69, 0x65,
0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x10c, 0x69,
0x6c, 0x117, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x69, 0x6e, 0x69, 0x6a, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x69, 0x6e, 0x69,
0x6a, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x69, 0x6e, 0x69, 0x6a, 0x6f, 0x73,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x4b, 0x61, 0x6c, 0x117, 0x64, 0x173, 0x20, 0x53, 0x61,
0x6c, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x6f,
0x6b, 0x6f, 0x73, 0x173, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x6f, 0x6c,
0x75, 0x6d, 0x62, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x6f, 0x6c,
0x75, 0x6d, 0x62, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x75,
0x6b, 0x6f, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x4b, 0x75, 0x6b, 0x6f, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20,
0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x4b, 0x75, 0x6b, 0x6f, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x70,
0x75, 0x73, 0x117, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x75, 0x62, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x75, 0x62, 0x6f, 0x73,
0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x4b, 0x75, 0x62, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x44, 0x65,
0x69, 0x76, 0x69, 0x73, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x44, 0x69, 0x75, 0x6d, 0x6f, 0x6e, 0x6f, 0x20, 0x64, 0x2019, 0x55, 0x72,
0x76, 0x69, 0x6c, 0x69, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x52, 0x79, 0x74, 0x173, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x6f, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x65, 0x6c, 0x79, 0x6b, 0x173,
0x20, 0x53, 0x61, 0x6c, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x56, 0x65, 0x6c, 0x79, 0x6b, 0x173, 0x20, 0x73, 0x61, 0x6c, 0x6f,
0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x56, 0x65, 0x6c, 0x79, 0x6b, 0x173, 0x20, 0x53, 0x61,
0x6c, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x45, 0x6b, 0x76, 0x61, 0x64, 0x6f,
0x72, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x69, 0x64,
0x75, 0x72, 0x69, 0x6f, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x69, 0x64, 0x75, 0x72,
0x69, 0x6f, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x6f, 0x73, 0x20, 0x17e,
0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x56, 0x69, 0x64, 0x75, 0x72, 0x69, 0x6f, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x52, 0x79, 0x74, 0x173, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x52, 0x79, 0x74, 0x173, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x6f,
0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x52, 0x79, 0x74, 0x173, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x54, 0x6f, 0x6c, 0x69, 0x6d, 0x173,
0x6a, 0x173, 0x20, 0x72, 0x79, 0x74, 0x173, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x61,
0x6b, 0x61, 0x72, 0x173, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x61, 0x6b, 0x61, 0x72,
0x173, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x6f, 0x73, 0x20, 0x17e, 0x69,
0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56,
0x61, 0x6b, 0x61, 0x72, 0x173, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x6f,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x173, 0x20, 0x53,
0x61, 0x6c, 0x173, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x46, 0x69,
0x64, 0x17e, 0x69, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x46,
0x69, 0x64, 0x17e, 0x69, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x46, 0x69, 0x64, 0x17e, 0x69,
0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x50, 0x72, 0x61, 0x6e, 0x63, 0x16b, 0x7a, 0x69,
0x6a, 0x6f, 0x73, 0x20, 0x47, 0x76, 0x69, 0x61, 0x6e, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x50, 0x69, 0x65, 0x74, 0x173, 0x20,
0x50, 0x72, 0x61, 0x6e, 0x63, 0x16b, 0x7a, 0x69, 0x6a, 0x6f, 0x73, 0x20,
0x69, 0x72, 0x20, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x6e,
0x69, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x47, 0x61, 0x6c,
0x61, 0x70, 0x61, 0x67, 0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x47, 0x61, 0x6d, 0x62, 0x79, 0x72, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x6a, 0x6f,
0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x6a, 0x6f, 0x73, 0x20,
0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x6f, 0x20, 0x53,
0x61, 0x6c, 0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x47, 0x72,
0x69, 0x6e, 0x76, 0x69, 0x10d, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x6a, 0x6f,
0x73, 0x20, 0x72, 0x79, 0x74, 0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x6a, 0x6f,
0x73, 0x20, 0x72, 0x79, 0x74, 0x173, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x47, 0x72, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x72, 0x79, 0x74,
0x173, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x6b, 0x61, 0x72, 0x173, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x6b, 0x61, 0x72,
0x173, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x6b, 0x61, 0x72, 0x173, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x47, 0x75, 0x61, 0x6d, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x50, 0x65, 0x72, 0x73, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x12f, 0x6c,
0x61, 0x6e, 0x6b, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x47, 0x61, 0x6a, 0x61, 0x6e, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x173, 0x2d, 0x41, 0x6c, 0x65,
0x75, 0x74, 0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x48, 0x61,
0x76, 0x61, 0x6a, 0x173, 0x2013, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x173, 0x20,
0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x173, 0x2013, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x173, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x48, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e, 0x67,
0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x48, 0x6f, 0x6e, 0x6b,
0x6f, 0x6e, 0x67, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x48, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e,
0x67, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x48, 0x6f, 0x76, 0x64, 0x6f, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x48, 0x6f, 0x76, 0x64, 0x6f, 0x20, 0x17e,
0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x48, 0x6f, 0x76, 0x64, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x49, 0x6e, 0x64, 0x69,
0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x49, 0x6e,
0x64, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x6e, 0x64, 0x65, 0x6e,
0x79, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x49, 0x6e,
0x64, 0x6f, 0x6b, 0x69, 0x6e, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x69, 0x6e, 0x117,
0x73, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x52, 0x79, 0x74, 0x173,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x61, 0x6b, 0x61, 0x72,
0x173, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x49, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x49, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x49, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x49,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x17e,
0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x49,
0x7a, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x6f, 0x20, 0x17e,
0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x6f, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4a,
0x61, 0x70, 0x6f, 0x6e, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x69, 0x6a, 0x6f, 0x73,
0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x69, 0x6a, 0x6f, 0x73, 0x20,
0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x4b, 0x61, 0x6d, 0x10d, 0x69, 0x61, 0x74, 0x6b, 0x6f, 0x73,
0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x61,
0x6d, 0x10d, 0x69, 0x61, 0x74, 0x6b, 0x6f, 0x73, 0x20, 0x50, 0x65, 0x74,
0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x6f, 0x20,
0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x4b, 0x61, 0x6d, 0x10d, 0x69, 0x61, 0x74, 0x6b, 0x6f, 0x73, 0x20,
0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x61, 0x7a, 0x61, 0x63, 0x68, 0x73,
0x74, 0x61, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x52,
0x79, 0x74, 0x173, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x63, 0x68, 0x73, 0x74,
0x61, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x61,
0x6b, 0x61, 0x72, 0x173, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x63, 0x68, 0x73,
0x74, 0x61, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b,
0x6f, 0x72, 0x117, 0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x4b, 0x6f, 0x72, 0x117, 0x6a, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65,
0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x6f,
0x72, 0x117, 0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x6f, 0x73, 0x72,
0x61, 0x6a, 0x117, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a,
0x61, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x69,
0x72, 0x67, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x4c, 0x61, 0x6e, 0x6b, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x4c, 0x61, 0x69, 0x6e, 0x6f, 0x20, 0x53, 0x61,
0x6c, 0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4c, 0x6f, 0x72,
0x64, 0x6f, 0x20, 0x48, 0x61, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x4c, 0x6f, 0x72, 0x64, 0x6f, 0x20, 0x48, 0x61, 0x75, 0x20, 0x17e,
0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x4c, 0x6f, 0x72, 0x64, 0x6f, 0x20, 0x48, 0x61, 0x75, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x4d, 0x61, 0x6b, 0x61, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x4d, 0x61, 0x6b, 0x61, 0x75, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d, 0x61, 0x6b, 0x61, 0x75,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x6f, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x6f,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x4d, 0x61, 0x6c, 0x61, 0x69, 0x7a, 0x69, 0x6a, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d, 0x61, 0x6c, 0x64,
0x79, 0x76, 0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d, 0x61,
0x72, 0x6b, 0x69, 0x7a, 0x6f, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d, 0x61, 0x72, 0x161, 0x61, 0x6c, 0x6f,
0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6a, 0x61, 0x75, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63,
0x69, 0x6a, 0x61, 0x75, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x63, 0x69, 0x6a, 0x61, 0x75, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d, 0x6f, 0x73,
0x6f, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d, 0x65,
0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x73, 0x20, 0x52, 0x61, 0x6d, 0x69, 0x6f,
0x6a, 0x6f, 0x20, 0x76, 0x61, 0x6e, 0x64, 0x65, 0x6e, 0x79, 0x6e, 0x6f,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d, 0x65, 0x6b, 0x73, 0x69,
0x6b, 0x6f, 0x73, 0x20, 0x52, 0x61, 0x6d, 0x69, 0x6f, 0x6a, 0x6f, 0x20,
0x76, 0x61, 0x6e, 0x64, 0x65, 0x6e, 0x79, 0x6e, 0x6f, 0x20, 0x17e, 0x69,
0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d,
0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x73, 0x20, 0x52, 0x61, 0x6d, 0x69,
0x6f, 0x6a, 0x6f, 0x20, 0x76, 0x61, 0x6e, 0x64, 0x65, 0x6e, 0x79, 0x6e,
0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74,
0x6f, 0x72, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x55, 0x6c,
0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x6f, 0x20, 0x17e, 0x69,
0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x55,
0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x6f, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x4d, 0x61, 0x73, 0x6b, 0x76, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x4d, 0x61, 0x73, 0x6b, 0x76, 0x6f, 0x73, 0x20, 0x17e,
0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x4d, 0x61, 0x73, 0x6b, 0x76, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d, 0x69,
0x61, 0x6e, 0x6d, 0x61, 0x72, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x4e, 0x61, 0x75, 0x6a, 0x6f, 0x73, 0x69, 0x6f, 0x73, 0x20,
0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4e, 0x61, 0x75, 0x6a, 0x6f, 0x73,
0x69, 0x6f, 0x73, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69,
0x6a, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x4e, 0x61, 0x75, 0x6a, 0x6f, 0x73, 0x69,
0x6f, 0x73, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a,
0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x4e, 0x61, 0x75, 0x6a, 0x6f, 0x73, 0x69,
0x6f, 0x73, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x6a, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4e, 0x61, 0x75, 0x6a,
0x6f, 0x73, 0x69, 0x6f, 0x73, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x6a, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4e, 0x61, 0x75, 0x6a, 0x6f, 0x73,
0x69, 0x6f, 0x73, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x6a,
0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x4e, 0x69, 0x75, 0x66, 0x61, 0x75, 0x6e,
0x64, 0x6c, 0x65, 0x6e, 0x64, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x4e, 0x69, 0x75, 0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e,
0x64, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x4e, 0x69, 0x75, 0x66, 0x61, 0x75, 0x6e, 0x64,
0x6c, 0x65, 0x6e, 0x64, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4e, 0x69, 0x75, 0x6a,
0x117, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x6f, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x6f,
0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x6f, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x6a, 0x6f, 0x73, 0x20, 0x17e,
0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x160,
0x69, 0x61, 0x75, 0x72, 0x117, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x61,
0x6e, 0x6f, 0x73, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x17e,
0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x4f, 0x6d, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x4f, 0x6d, 0x73, 0x6b, 0x6f, 0x20, 0x17e, 0x69, 0x65,
0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4f, 0x6d,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x50, 0x61, 0x6c,
0x61, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x50, 0x61, 0x70,
0x75, 0x61, 0x20, 0x4e, 0x61, 0x75, 0x6a, 0x6f, 0x73, 0x69, 0x6f, 0x73,
0x20, 0x47, 0x76, 0x69, 0x6e, 0x117, 0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a,
0x61, 0x75, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x50, 0x61,
0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x61, 0x75, 0x73, 0x20, 0x17e, 0x69,
0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x50,
0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x61, 0x75, 0x73, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x50, 0x65, 0x72, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x50, 0x65, 0x72, 0x75, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x50, 0x65, 0x72, 0x75, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x173, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x173,
0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x173, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x46, 0x65, 0x6e, 0x69, 0x6b, 0x73, 0x6f, 0x20, 0x53, 0x61, 0x6c,
0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x53, 0x65, 0x6e, 0x20,
0x50, 0x6a, 0x65, 0x72, 0x6f, 0x20, 0x69, 0x72, 0x20, 0x4d, 0x69, 0x6b,
0x65, 0x6c, 0x6f, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x53, 0x65, 0x6e, 0x20, 0x50, 0x6a, 0x65, 0x72, 0x6f, 0x20, 0x69, 0x72,
0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x6f, 0x20, 0x17e, 0x69,
0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x53,
0x65, 0x6e, 0x20, 0x50, 0x6a, 0x65, 0x72, 0x6f, 0x20, 0x69, 0x72, 0x20,
0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x6f, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x50,
0x69, 0x74, 0x6b, 0x65, 0x72, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x117, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x50, 0x63, 0x68, 0x65, 0x6e, 0x6a, 0x61, 0x6e,
0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x79, 0x7a, 0x79,
0x6c, 0x6f, 0x72, 0x64, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x4b, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x6f, 0x73, 0x20,
0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x4b, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x6f, 0x73, 0x20,
0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x52, 0x65, 0x75, 0x6e, 0x6a, 0x6f, 0x6e, 0x6f, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x52, 0x6f, 0x74, 0x65, 0x72, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x53, 0x61, 0x63, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x53,
0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x6f, 0x20, 0x17e, 0x69, 0x65,
0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x53, 0x61,
0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x53, 0x61,
0x6d, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x53, 0x61, 0x6d, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x53, 0x61, 0x6d,
0x61, 0x72, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x53, 0x65,
0x69, 0x161, 0x65, 0x6c, 0x69, 0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x16b, 0x72, 0x6f, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x53, 0x61, 0x6c, 0x69, 0x61, 0x6d, 0x6f,
0x6e, 0x6f, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x50, 0x69, 0x65, 0x74, 0x173, 0x20, 0x44, 0x17e, 0x6f, 0x72,
0x64, 0x17e, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x6f, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x53, 0x69, 0x6f, 0x76, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x54, 0x61, 0x68, 0x69, 0x10d, 0x69, 0x6f,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x54, 0x61, 0x69, 0x70, 0x117,
0x6a, 0x61, 0x75, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x54,
0x61, 0x69, 0x70, 0x117, 0x6a, 0x61, 0x75, 0x73, 0x20, 0x17e, 0x69, 0x65,
0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x54, 0x61,
0x69, 0x70, 0x117, 0x6a, 0x61, 0x75, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x54, 0x61,
0x64, 0x17e, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x54, 0x6f, 0x6e, 0x67, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x54, 0x6f, 0x6e, 0x67,
0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x54, 0x6f, 0x6e, 0x67, 0x6f, 0x73, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x10c, 0x75, 0x6b, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x117, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x117, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x20, 0x17e, 0x69,
0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x54,
0x75, 0x72, 0x6b, 0x6d, 0x117, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x61,
0x75, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x55, 0x72, 0x75,
0x67, 0x76, 0x61, 0x6a, 0x61, 0x75, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x55, 0x72, 0x75,
0x67, 0x76, 0x61, 0x6a, 0x61, 0x75, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x55, 0x7a,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x56, 0x65, 0x6e, 0x65, 0x73, 0x75, 0x65, 0x6c,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x6f, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f,
0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x6f, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x65, 0x69, 0x6b, 0x6f, 0x20, 0x53,
0x61, 0x6c, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56,
0x6f, 0x6c, 0x69, 0x73, 0x6f, 0x20, 0x69, 0x72, 0x20, 0x46, 0x75, 0x74,
0x16b, 0x6e, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4a,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x17e,
0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4a,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4a, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x20, 0x17e,
0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x4a, 0x75, 0x6b, 0x6f, 0x6e, 0x6f, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x74, 0x63, 0x69, 0x6b, 0x61, 0x20,
0x66, 0x6f, 0x20, 0x6c, 0x61, 0x20, 0x6b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x69, 0x61, 0x73, 0x6e, 0x61, 0x6c, 0x63, 0x69, 0x73, 0x74, 0x63,
0x69, 0x6b, 0x61, 0x20, 0x66, 0x6f, 0x20, 0x6c, 0x61, 0x20, 0x6b, 0x6f,
0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x69, 0x61, 0x73, 0x63, 0x69, 0x73, 0x74,
0x63, 0x69, 0x6b, 0x61, 0x20, 0x66, 0x6f, 0x20, 0x6c, 0x61, 0x20, 0x6b,
0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x69, 0x61, 0x73, 0x74, 0x63, 0x69,
0x6b, 0x61, 0x20, 0x66, 0x6f, 0x20, 0x6c, 0x61, 0x20, 0x67, 0x61, 0x6c,
0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x41, 0x66, 0x67, 0x68, 0x61, 0x6e,
0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x53, 0x72, 0x6a, 0x65,
0x6a, 0x17a, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x50, 0xf3, 0x64, 0x7a, 0x61, 0x6a, 0x74, 0x161, 0x6e, 0x6f,
0x61, 0x66, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50,
0xf3, 0x64, 0x70, 0x6f, 0x142, 0x64, 0x6e, 0x6a, 0x6f, 0x77, 0x6f, 0x61,
0x66, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3,
0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e, 0x6f, 0x61, 0x66, 0x72,
0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x77,
0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63,
0x6f, 0x72, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x6f, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x63, 0x61, 0x73, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x6f, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x41, 0x6d, 0x61, 0x63, 0x6f, 0x6e, 0x61, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x41, 0x6d, 0x61, 0x63, 0x6f, 0x6e, 0x61, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x63, 0x61, 0x73, 0x41, 0x6d, 0x61, 0x63, 0x6f, 0x6e, 0x61,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x70, 0x6f, 0x142, 0x6e, 0x6f,
0x63, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x79, 0x20, 0x63, 0x61,
0x73, 0x50, 0xf3, 0x64, 0x70, 0x6f, 0x142, 0x6e, 0x6f, 0x63, 0x6e, 0x6f,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x6e, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64,
0x70, 0x6f, 0x142, 0x6e, 0x6f, 0x63, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x6e, 0x79, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x70, 0x6f, 0x142, 0x6e, 0x6f, 0x63,
0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x70,
0xf3, 0x64, 0x7a, 0x61, 0x6a, 0x74, 0x161, 0x6e, 0x79, 0x20, 0x63, 0x61,
0x73, 0x50, 0xf3, 0x64, 0x70, 0x6f, 0x142, 0x6e, 0x6f, 0x63, 0x6e, 0x6f,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x70, 0xf3, 0x64,
0x7a, 0x61, 0x6a, 0x74, 0x161, 0x6e, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3,
0x64, 0x70, 0x6f, 0x142, 0x6e, 0x6f, 0x63, 0x6e, 0x6f, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x70, 0xf3, 0x64, 0x7a, 0x61, 0x6a,
0x74, 0x161, 0x6e, 0x79, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x70, 0x6f, 0x142, 0x6e,
0x6f, 0x63, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69,
0x20, 0x67, 0xf3, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50,
0xf3, 0x64, 0x70, 0x6f, 0x142, 0x6e, 0x6f, 0x63, 0x6e, 0x6f, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x67, 0xf3, 0x72, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79,
0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x70, 0x6f, 0x142, 0x6e, 0x6f,
0x63, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20,
0x67, 0xf3, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x70, 0x6f,
0x142, 0x6e, 0x6f, 0x63, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73,
0x6b, 0x69, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x70, 0x6f, 0x142, 0x6e, 0x6f,
0x63, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20,
0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73,
0x50, 0xf3, 0x64, 0x70, 0x6f, 0x142, 0x6e, 0x6f, 0x63, 0x6e, 0x6f, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x61, 0x63, 0x69,
0x66, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x41, 0x70, 0x69, 0x61, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x41, 0x70, 0x69, 0x61, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x63, 0x61, 0x73, 0x41, 0x70, 0x69, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x41,
0x72, 0x61, 0x62, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x41,
0x72, 0x61, 0x62, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x41, 0x72,
0x61, 0x62, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x50, 0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e, 0x6f, 0x61,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x50, 0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e,
0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72,
0x6e, 0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x41, 0x72, 0x6d, 0x65, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x41, 0x72, 0x6d, 0x65, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61,
0x73, 0x41, 0x72, 0x6d, 0x65, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x41, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61,
0x73, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x53, 0x72, 0x6a, 0x65, 0x6a, 0x17a, 0x6f, 0x61, 0x77, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x53, 0x72, 0x6a,
0x65, 0x6a, 0x17a, 0x6f, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x63, 0x61, 0x73, 0x53, 0x72, 0x6a, 0x65, 0x6a, 0x17a, 0x6f,
0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x53,
0x72, 0x6a, 0x65, 0x6a, 0x17a, 0x6f, 0x70, 0xf3, 0x64, 0x77, 0x6a, 0x61,
0x63, 0x6f, 0x72, 0x6e, 0x79, 0x20, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x53, 0x72, 0x6a, 0x65,
0x6a, 0x17a, 0x6f, 0x70, 0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72,
0x6e, 0x79, 0x20, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79,
0x20, 0x63, 0x61, 0x73, 0x53, 0x72, 0x6a, 0x65, 0x6a, 0x17a, 0x6f, 0x70,
0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e, 0x79, 0x20, 0x61,
0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3,
0x64, 0x7a, 0x61, 0x6a, 0x74, 0x161, 0x6e, 0x6f, 0x61, 0x77, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3,
0x64, 0x7a, 0x61, 0x6a, 0x74, 0x161, 0x6e, 0x6f, 0x61, 0x77, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64,
0x7a, 0x61, 0x6a, 0x74, 0x161, 0x6e, 0x6f, 0x61, 0x77, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x77, 0x6a, 0x61,
0x63, 0x6f, 0x72, 0x6e, 0x6f, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x77, 0x6a,
0x61, 0x63, 0x6f, 0x72, 0x6e, 0x6f, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x77, 0x6a,
0x61, 0x63, 0x6f, 0x72, 0x6e, 0x6f, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a,
0x64, 0x17e, 0x61, 0x6e, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x69,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61,
0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x41, 0x63,
0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x41, 0x63, 0x6f,
0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x41, 0x63, 0x6f, 0x72, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x161, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x42,
0x68, 0x75, 0x74, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x42, 0x6f, 0x6c, 0x69, 0x77, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x61,
0x73, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73,
0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x20, 0x6c, 0x11b, 0x15b,
0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x42, 0x72, 0x75,
0x6e, 0x65, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4b, 0x61,
0x70, 0x76, 0x65, 0x72, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x4b, 0x61, 0x70, 0x76, 0x65, 0x72, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61,
0x73, 0x4b, 0x61, 0x70, 0x76, 0x65, 0x72, 0x64, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x63, 0x61, 0x73, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x43, 0x68, 0x69, 0x6c, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x43, 0x68, 0x69, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61,
0x73, 0x43, 0x68, 0x69, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b,
0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x43, 0x68, 0x69,
0x6e, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x43, 0x68, 0x69, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x43, 0x68, 0x69, 0x6e, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x63, 0x61, 0x73, 0x20, 0x47, 0xf3, 0x64, 0x6f, 0x77, 0x6e,
0x79, 0x63, 0x68, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x63, 0x61, 0x73,
0x20, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x6f, 0x77, 0x79, 0x63, 0x68, 0x20,
0x6b, 0x75, 0x70, 0x6f, 0x77, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4b, 0x6f, 0x6c, 0x75, 0x6d,
0x62, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x4b, 0x6f, 0x6c, 0x75,
0x6d, 0x62, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x63, 0x61, 0x73, 0x20, 0x43,
0x6f, 0x6f, 0x6b, 0x6f, 0x77, 0x79, 0x63, 0x68, 0x20, 0x6b, 0x75, 0x70,
0x6f, 0x77, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79,
0x20, 0x63, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x77, 0x79,
0x63, 0x68, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x6c, 0x11b, 0x15b, 0x6f,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f,
0x6b, 0x6f, 0x77, 0x79, 0x63, 0x68, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77,
0x4b, 0x75, 0x62, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x4b, 0x75, 0x62, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x4b,
0x75, 0x62, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x44, 0x61, 0x76, 0x69,
0x73, 0x20, 0x63, 0x61, 0x73, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x44,
0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x20, 0x63, 0x61, 0x73, 0x50,
0xf3, 0x64, 0x7a, 0x61, 0x6a, 0x74, 0x161, 0x6e, 0x6f, 0x74, 0x69, 0x6d,
0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x63, 0x61, 0x73,
0x20, 0x4a, 0x61, 0x74, 0x161, 0x6f, 0x77, 0x6e, 0x65, 0x6a, 0x65, 0x20,
0x6b, 0x75, 0x70, 0x79, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x20, 0x4a, 0x61, 0x74, 0x161, 0x6f,
0x77, 0x6e, 0x65, 0x6a, 0x65, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x6c, 0x11b,
0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x20, 0x4a,
0x61, 0x74, 0x161, 0x6f, 0x77, 0x6e, 0x65, 0x6a, 0x65, 0x20, 0x6b, 0x75,
0x70, 0x79, 0x45, 0x6b, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x53, 0x72, 0x6a, 0x65, 0x6a, 0x17a, 0x6e, 0x6f,
0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x53, 0x72, 0x6a, 0x65, 0x6a, 0x17a, 0x6e, 0x6f, 0x65, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x53,
0x72, 0x6a, 0x65, 0x6a, 0x17a, 0x6e, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x7a, 0x61, 0x6a,
0x74, 0x161, 0x6e, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x7a, 0x61, 0x6a,
0x74, 0x161, 0x6e, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x7a, 0x61, 0x6a, 0x74,
0x161, 0x6e, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x4b, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x6e, 0x67, 0x72, 0x61,
0x64, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x77,
0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64,
0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e, 0x6f, 0x65, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64,
0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e, 0x6f, 0x65, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x46, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x46, 0x69, 0x64, 0x17a, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x46, 0x69, 0x64, 0x17a, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x46,
0x69, 0x64, 0x17a, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x46, 0x72, 0x61, 0x6e,
0x63, 0x6f, 0x6a, 0x73, 0x6b, 0x6f, 0x67, 0x75, 0x79, 0x61, 0x144, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x63, 0x61, 0x73, 0x20, 0x66, 0x72,
0x61, 0x6e, 0x63, 0x6f, 0x6a, 0x73, 0x6b, 0x65, 0x67, 0x6f, 0x20, 0x70,
0xf3, 0x64, 0x70, 0x6f, 0x142, 0x64, 0x6e, 0x6a, 0x6f, 0x77, 0x65, 0x67,
0x6f, 0x20, 0x61, 0x20, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69,
0x73, 0x6b, 0x65, 0x68, 0x6f, 0x20, 0x74, 0x65, 0x72, 0x69, 0x74, 0x6f,
0x72, 0x69, 0x75, 0x6d, 0x61, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67,
0x6f, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x47, 0x61, 0x6d, 0x62,
0x69, 0x65, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x47, 0x65,
0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b,
0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x63, 0x61, 0x73,
0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x6f, 0x77, 0x79, 0x63,
0x68, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x77, 0x69, 0x63, 0x68, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50,
0xf3, 0x64, 0x7a, 0x61, 0x6a, 0x74, 0x161, 0x6e, 0x6f, 0x67, 0x72, 0xf6,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x50, 0xf3, 0x64, 0x7a, 0x61, 0x6a, 0x74, 0x161, 0x6e, 0x6f, 0x67, 0x72,
0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73,
0x50, 0xf3, 0x64, 0x7a, 0x61, 0x6a, 0x74, 0x161, 0x6e, 0x6f, 0x67, 0x72,
0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3,
0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e, 0x6f, 0x67, 0x72, 0xf6,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x50, 0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e, 0x6f, 0x67,
0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61,
0x73, 0x50, 0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e, 0x6f,
0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x63, 0x61, 0x73, 0x20, 0x50, 0x65, 0x72, 0x73, 0x69, 0x73, 0x6b, 0x65,
0x67, 0x6f, 0x20, 0x67, 0x6f, 0x6c, 0x66, 0x61, 0x47, 0x75, 0x79, 0x61,
0x144, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x48, 0x61, 0x77, 0x61,
0x69, 0x69, 0x73, 0x6b, 0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x73, 0x6b, 0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x63, 0x61, 0x73, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x73, 0x6b, 0x6f,
0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x48, 0x6f,
0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x63, 0x61, 0x73, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x43, 0x68, 0x6f, 0x77, 0x64, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x43, 0x68, 0x6f, 0x77, 0x64, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63,
0x61, 0x73, 0x43, 0x68, 0x6f, 0x77, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x49,
0x6e, 0x64, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x49, 0x6e,
0x64, 0x69, 0x73, 0x6b, 0x6f, 0x6f, 0x63, 0x65, 0x61, 0x6e, 0x69, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x49, 0x6e, 0x64, 0x6f, 0x63, 0x68,
0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x53, 0x72, 0x6a,
0x65, 0x6a, 0x17a, 0x6f, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x7a, 0x61, 0x6a, 0x74,
0x161, 0x6e, 0x6f, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x6b, 0x69,
0x50, 0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e, 0x6f, 0x69,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x49, 0x72, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x49,
0x72, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x49, 0x72, 0x61,
0x144, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79,
0x20, 0x63, 0x61, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63,
0x61, 0x73, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x4a, 0x61, 0x70, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x4a, 0x61, 0x70, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x4a,
0x61, 0x70, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x6b, 0x61, 0x7a, 0x61,
0x63, 0x68, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3,
0x64, 0x7a, 0x61, 0x6a, 0x74, 0x161, 0x6e, 0x6f, 0x6b, 0x61, 0x7a, 0x61,
0x63, 0x68, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64,
0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e, 0x6f, 0x6b, 0x61, 0x7a, 0x61,
0x63, 0x68, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4b, 0x6f, 0x72,
0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4b, 0x6f, 0x72,
0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x4b, 0x6f, 0x72, 0x65,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x6a, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61,
0x73, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x4b, 0x69, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x63, 0x61, 0x73, 0x20, 0x4c, 0x69, 0x6e, 0x69, 0x6a, 0x6f,
0x77, 0x79, 0x63, 0x68, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x63, 0x61,
0x73, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x2d,
0x48, 0x6f, 0x77, 0x65, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48, 0x6f, 0x77, 0x65, 0x6c, 0x11b, 0x15b,
0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x20, 0x6b, 0x75,
0x70, 0x79, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48, 0x6f, 0x77, 0x65,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x144, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x63, 0x61, 0x73, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x144, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x4d, 0x61, 0x6c, 0x61, 0x6a, 0x7a, 0x69, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x4d, 0x61, 0x6c, 0x65, 0x64, 0x69, 0x77, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x63, 0x61, 0x73, 0x20, 0x4d,
0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x6f, 0x77, 0x79, 0x63, 0x68,
0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63,
0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x63, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x63, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4d, 0x61, 0x77, 0x73,
0x6f, 0x6e, 0x20, 0x63, 0x61, 0x73, 0x4d, 0x65, 0x78, 0x69, 0x73, 0x6b,
0x69, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x4d, 0x65, 0x78, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x70,
0x61, 0x63, 0x69, 0x66, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x4d,
0x65, 0x78, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66,
0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x55, 0x6c,
0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63,
0x61, 0x73, 0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x77, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x77, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x63, 0x61, 0x73, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x77, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x4e, 0x6f, 0x77, 0x6f, 0x6b, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x4e, 0x6f, 0x77, 0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x4e, 0x6f, 0x77, 0x6f, 0x6b, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4e, 0x6f,
0x77, 0x6f, 0x73, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x65, 0x65, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x4e, 0x6f, 0x77,
0x6f, 0x73, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x4e, 0x6f, 0x77, 0x6f, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4e, 0x6f, 0x77, 0x6f, 0x66,
0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61,
0x73, 0x4e, 0x6f, 0x77, 0x6f, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x4e, 0x69, 0x75, 0x65, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x63, 0x61, 0x73, 0x20, 0x6b, 0x75, 0x70, 0x79,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x20, 0x6b,
0x75, 0x70, 0x79, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x6c,
0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x20,
0x6b, 0x75, 0x70, 0x79, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x63, 0x61, 0x73, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61,
0x73, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x6c, 0x11b, 0x15b,
0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x20, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4e, 0x6f, 0x77,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73,
0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x4f, 0x6d, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4f, 0x6d,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x4f, 0x6d, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x144, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x63, 0x61, 0x73, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x144, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0x61, 0x70, 0x75, 0x61, 0x2d, 0x4e,
0x6f, 0x77, 0x6f, 0x67, 0x69, 0x6e, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x79, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0x65, 0x72, 0x75,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0x65, 0x72, 0x75, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x63, 0x61, 0x73, 0x50, 0x65, 0x72, 0x75, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79,
0x20, 0x63, 0x61, 0x73, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x63, 0x61, 0x73, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e,
0x69, 0x78, 0x6f, 0x77, 0x79, 0x63, 0x68, 0x20, 0x6b, 0x75, 0x70, 0x6f,
0x77, 0x53, 0x74, 0x2e, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d,
0x61, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x144, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x53, 0x74, 0x2e, 0x2d, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x2d, 0x61, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x53, 0x74, 0x2e, 0x2d,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x61, 0x2d, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b,
0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x63, 0x61, 0x73,
0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x6f, 0x77, 0x79,
0x63, 0x68, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x50, 0x6f, 0x6e, 0x61,
0x70, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0x6a, 0xf6, 0x6e,
0x67, 0x6a, 0x61, 0x6e, 0x67, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x63, 0x61, 0x73, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72,
0x61, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x63, 0x61, 0x73, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x53, 0x61, 0x6c, 0x6f, 0x6d,
0x6f, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64,
0x70, 0x6f, 0x142, 0x64, 0x6e, 0x6a, 0x6f, 0x77, 0x6f, 0x67, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x53, 0x75,
0x72, 0x69, 0x6e, 0x61, 0x6d, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x53, 0x79, 0x6f, 0x77, 0x61, 0x20, 0x63, 0x61, 0x73, 0x54, 0x61, 0x68,
0x69, 0x74, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x54, 0x63,
0x68, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x54, 0x63, 0x68, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x63, 0x61, 0x73, 0x54, 0x63, 0x68, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x54, 0x61, 0x64, 0x17a, 0x69, 0x6b, 0x69, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x54, 0x75, 0x76, 0x61,
0x6c, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x55, 0x72, 0x75, 0x67,
0x75, 0x61, 0x79, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x79, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x55, 0x7a,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x55,
0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x55,
0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b,
0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x56, 0x65, 0x6e, 0x65,
0x7a, 0x75, 0x65, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x57,
0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x57, 0x6c,
0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x57, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b,
0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x63, 0x61, 0x73,
0x20, 0x57, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x63, 0x61, 0x73, 0x20, 0x6b,
0x75, 0x70, 0x79, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x63, 0x61, 0x73, 0x20,
0x6b, 0x75, 0x70, 0x6f, 0x77, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73,
0x20, 0x61, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x4a, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4a, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x4a, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x4a, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x61,
0x73, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x61, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x54, 0x69, 0x65,
0x74, 0x4f, 0x6f, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x61,
0x6e, 0x73, 0x63, 0x68, 0x20, 0x54, 0x69, 0x65, 0x74, 0x53, 0xf6, 0xf6,
0x64, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68,
0x20, 0x54, 0x69, 0x65, 0x74, 0x57, 0x65, 0x73, 0x74, 0x61, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x54, 0x69, 0x65,
0x74, 0x57, 0x65, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x61,
0x6e, 0x73, 0x63, 0x68, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x74, 0x69, 0x65, 0x74, 0x57, 0x65, 0x73, 0x74, 0x61, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74, 0x4e, 0x6f, 0x6f, 0x72, 0x64,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68,
0x20, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x74, 0x69, 0x65,
0x74, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x7a, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x61, 0x6c, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x74, 0x69, 0x65, 0x74, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x7a, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x65, 0x74, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x6f,
0x6f, 0x73, 0x74, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x65, 0x74, 0x4e, 0x6f,
0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e,
0x73, 0x63, 0x68, 0x20, 0x6f, 0x6f, 0x73, 0x74, 0x65, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x65, 0x74, 0x4e,
0x6f, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61,
0x6e, 0x73, 0x63, 0x68, 0x20, 0x6f, 0x6f, 0x73, 0x74, 0x65, 0x6e, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74, 0x4e, 0x6f,
0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e,
0x73, 0x63, 0x68, 0x20, 0x42, 0x61, 0x72, 0x67, 0x74, 0x69, 0x65, 0x74,
0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x42, 0x61, 0x72, 0x67, 0x2d, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x65, 0x74, 0x4e,
0x6f, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61,
0x6e, 0x73, 0x63, 0x68, 0x20, 0x42, 0x61, 0x72, 0x67, 0x2d, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74, 0x4e, 0x6f, 0x6f, 0x72,
0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63,
0x68, 0x20, 0x50, 0x61, 0x7a, 0x69, 0x66, 0x69, 0x6b, 0x74, 0x69, 0x65,
0x74, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x50, 0x61, 0x7a, 0x69, 0x66,
0x69, 0x6b, 0x2d, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74,
0x69, 0x65, 0x74, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x50, 0x61, 0x7a,
0x69, 0x66, 0x69, 0x6b, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x65, 0x74, 0x41, 0x72, 0x61, 0x61, 0x62, 0x73, 0x63, 0x68, 0x20,
0x54, 0x69, 0x65, 0x74, 0x41, 0x72, 0x61, 0x61, 0x62, 0x73, 0x63, 0x68,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x65,
0x74, 0x41, 0x72, 0x61, 0x61, 0x62, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74, 0x4e, 0x6f, 0x6f, 0x72,
0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63,
0x68, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x74, 0x69,
0x65, 0x74, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x41, 0x74, 0x6c, 0x61,
0x6e, 0x74, 0x69, 0x6b, 0x2d, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x74, 0x69, 0x65, 0x74, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x2d, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x65, 0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x73, 0x63,
0x68, 0x20, 0x54, 0x69, 0x65, 0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x73, 0x63,
0x68, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69,
0x65, 0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x61, 0x75,
0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74, 0x57, 0x65, 0x73, 0x74,
0x7a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x61, 0x6c, 0x73, 0x63, 0x68, 0x20, 0x54, 0x69, 0x65, 0x74,
0x57, 0x65, 0x73, 0x74, 0x7a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c,
0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x73, 0x63, 0x68, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x65, 0x74,
0x57, 0x65, 0x73, 0x74, 0x7a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c,
0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x73, 0x63, 0x68, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74, 0x4f, 0x6f,
0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x73, 0x63,
0x68, 0x20, 0x54, 0x69, 0x65, 0x74, 0x4f, 0x6f, 0x73, 0x74, 0x61, 0x75,
0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x73, 0x63, 0x68, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x65, 0x74, 0x4f, 0x6f,
0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x73, 0x63,
0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74,
0x57, 0x65, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c,
0x73, 0x63, 0x68, 0x20, 0x54, 0x69, 0x65, 0x74, 0x57, 0x65, 0x73, 0x74,
0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x73, 0x63, 0x68, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x65, 0x74,
0x57, 0x65, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c,
0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x65, 0x74, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x2d, 0x54, 0x69, 0x65, 0x74,
0x43, 0x68, 0x69, 0x6e, 0x61, 0x2d, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x74, 0x69, 0x65, 0x74, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x2d,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74, 0x5a, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4,
0xe4, 0x73, 0x63, 0x68, 0x20, 0x54, 0x69, 0x65, 0x74, 0x5a, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x61, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0xe4,
0x73, 0x63, 0x68, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x74, 0x69, 0x65, 0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c,
0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0xe4, 0x73, 0x63, 0x68, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74, 0x4f, 0x6f, 0x73,
0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0xe4, 0x73, 0x63, 0x68, 0x20,
0x54, 0x69, 0x65, 0x74, 0x4f, 0x6f, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f,
0x70, 0xe4, 0xe4, 0x73, 0x63, 0x68, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x74, 0x69, 0x65, 0x74, 0x4f, 0x6f, 0x73, 0x74, 0x65,
0x75, 0x72, 0x6f, 0x70, 0xe4, 0xe4, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74, 0x57, 0x65, 0x73, 0x74,
0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0xe4, 0x73, 0x63, 0x68, 0x20, 0x54,
0x69, 0x65, 0x74, 0x57, 0x65, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70,
0xe4, 0xe4, 0x73, 0x63, 0x68, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x74, 0x69, 0x65, 0x74, 0x57, 0x65, 0x73, 0x74, 0x65, 0x75,
0x72, 0x6f, 0x70, 0xe4, 0xe4, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74, 0x47, 0x72, 0xf6, 0xf6, 0x6e,
0x77, 0x69, 0x73, 0x63, 0x68, 0x2d, 0x57, 0x65, 0x6c, 0x74, 0x74, 0x69,
0x65, 0x74, 0x49, 0x6e, 0x64, 0x69, 0x65, 0x6e, 0x2d, 0x54, 0x69, 0x65,
0x74, 0x53, 0xf6, 0xf6, 0x64, 0x6f, 0x6f, 0x73, 0x74, 0x61, 0x73, 0x69,
0x61, 0x61, 0x74, 0x73, 0x63, 0x68, 0x20, 0x54, 0x69, 0x65, 0x74, 0x49,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x74, 0x69, 0x65, 0x74, 0x4f, 0x6f,
0x73, 0x74, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x65, 0x73, 0x63, 0x68,
0x20, 0x54, 0x69, 0x65, 0x74, 0x57, 0x65, 0x73, 0x74, 0x69, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x65, 0x73, 0x63, 0x68, 0x20, 0x54, 0x69, 0x65, 0x74,
0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x2d, 0x54, 0x69, 0x65, 0x74, 0x49,
0x73, 0x72, 0x61, 0x65, 0x6c, 0x2d, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x74, 0x69, 0x65, 0x74, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74, 0x4a,
0x61, 0x70, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x54, 0x69, 0x65,
0x74, 0x4a, 0x61, 0x70, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x65, 0x74, 0x4a,
0x61, 0x70, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74, 0x4b, 0x6f, 0x72, 0x65, 0x61,
0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x54, 0x69, 0x65, 0x74, 0x4b, 0x6f,
0x72, 0x65, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x65, 0x74, 0x4b, 0x6f, 0x72,
0x65, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x65, 0x74, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75,
0x2d, 0x54, 0x69, 0x65, 0x74, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75, 0x2d,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x65, 0x74,
0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x65, 0x74, 0x41, 0x63, 0x72, 0x65, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x41, 0x63, 0x72, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x63, 0x72, 0x65, 0x2d, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x66, 0x67, 0x68,
0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74,
0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x4f,
0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x73, 0x63,
0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x53, 0xfc, 0x64, 0x61, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69,
0x74, 0x57, 0x65, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x57, 0x65, 0x73,
0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68,
0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x57,
0x65, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x73,
0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69,
0x74, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x5a, 0xe4, 0x69, 0x74,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x6c,
0x6d, 0x61, 0x74, 0x79, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x41, 0x6c, 0x6d,
0x61, 0x74, 0x79, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4,
0x69, 0x74, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x2d, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x61, 0x73, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x41, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0x61, 0x73, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a,
0xe4, 0x69, 0x74, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x2d,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4e, 0x6f,
0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x73,
0x63, 0x68, 0x20, 0x49, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x7a, 0xe4, 0x69,
0x74, 0x4e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x49, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4e,
0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x65,
0x73, 0x63, 0x68, 0x20, 0x49, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4e, 0x6f, 0x72,
0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x73, 0x63,
0x68, 0x20, 0x4f, 0x73, 0x74, 0x6b, 0xfc, 0x73, 0x74, 0x65, 0x6e, 0x7a,
0xe4, 0x69, 0x74, 0x4e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4f, 0x73, 0x74, 0x6b,
0xfc, 0x73, 0x74, 0x65, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0xe4, 0x69, 0x74, 0x4e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4f, 0x73, 0x74,
0x6b, 0xfc, 0x73, 0x74, 0x65, 0x6e, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0xe4, 0x69, 0x74, 0x52, 0x6f, 0x63, 0x6b, 0x79, 0x2d, 0x4d,
0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74,
0x52, 0x6f, 0x63, 0x6b, 0x79, 0x2d, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61,
0x69, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69,
0x74, 0x52, 0x6f, 0x63, 0x6b, 0x79, 0x2d, 0x4d, 0x6f, 0x75, 0x6e, 0x74,
0x61, 0x69, 0x6e, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4,
0x69, 0x74, 0x4e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x57, 0x65, 0x73, 0x74, 0x6b,
0xfc, 0x73, 0x74, 0x65, 0x6e, 0x7a, 0xe4, 0x69, 0x74, 0x4e, 0x6f, 0x72,
0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x73, 0x63,
0x68, 0x20, 0x57, 0x65, 0x73, 0x74, 0x6b, 0xfc, 0x73, 0x74, 0x65, 0x6e,
0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4e,
0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x65,
0x73, 0x63, 0x68, 0x20, 0x57, 0x65, 0x73, 0x74, 0x6b, 0xfc, 0x73, 0x74,
0x65, 0x6e, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69,
0x74, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x2d, 0x5a, 0xe4, 0x69, 0x74,
0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x2d,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x72,
0x61, 0x62, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x41,
0x72, 0x61, 0x62, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x72, 0x61, 0x62, 0x65, 0x73,
0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69,
0x74, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x65, 0x73, 0x63,
0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0xe4, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x61, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69,
0x74, 0x57, 0x65, 0x73, 0x74, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0xe4, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x61, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x72, 0x6d, 0x65, 0x6e,
0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x41, 0x72, 0x6d,
0x65, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x65, 0x73,
0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69,
0x74, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x2d, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x6b, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0xe4, 0x69, 0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a,
0xe4, 0x69, 0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x5a, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x73,
0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69,
0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x2d, 0x2f, 0x57, 0x65,
0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x73, 0x63,
0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x2d, 0x2f, 0x57, 0x65, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x2d, 0x2f, 0x57, 0x65, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0xe4, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x4f,
0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x73, 0x63,
0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74,
0x4f, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x73,
0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69,
0x74, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x73, 0x63, 0x68,
0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x41,
0x73, 0x65, 0x72, 0x62, 0x65, 0x69, 0x64, 0x73, 0x63, 0x68, 0x61, 0x6e,
0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a,
0xe4, 0x69, 0x74, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x73,
0x63, 0x68, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x7a, 0x6f, 0x72, 0x65,
0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x6e,
0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x6e, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0xe4, 0x69, 0x74, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x73, 0x63, 0x68, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x63, 0x68, 0x2d, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x73, 0x63, 0x68, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0xe4, 0x69, 0x74, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x2d, 0x5a,
0xe4, 0x69, 0x74, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x6e, 0x65,
0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x42, 0x72, 0x61, 0x73,
0xed, 0x6c, 0x69, 0x61, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x42, 0x72, 0x61,
0x73, 0xed, 0x6c, 0x69, 0x61, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0xe4, 0x69, 0x74, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61,
0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x42,
0x72, 0x75, 0x6e, 0x65, 0x69, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4b, 0x61,
0x70, 0x2d, 0x56, 0x65, 0x72, 0x64, 0x65, 0x2d, 0x5a, 0xe4, 0x69, 0x74,
0x4b, 0x61, 0x70, 0x2d, 0x56, 0x65, 0x72, 0x64, 0x65, 0x2d, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4b, 0x61, 0x70, 0x2d,
0x56, 0x65, 0x72, 0x64, 0x65, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0xe4, 0x69, 0x74, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f,
0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x43,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0xe4, 0x69, 0x74, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x6e, 0x65,
0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x43, 0x68, 0x69, 0x6c,
0x65, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x6e, 0x65,
0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4,
0x69, 0x74, 0x43, 0x68, 0x69, 0x6e, 0x65, 0x73, 0x65, 0x73, 0x63, 0x68,
0x20, 0x5a, 0xe4, 0x69, 0x74, 0x43, 0x68, 0x69, 0x6e, 0x65, 0x73, 0x65,
0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4,
0x69, 0x74, 0x43, 0x68, 0x69, 0x6e, 0x65, 0x73, 0x65, 0x73, 0x63, 0x68,
0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x43,
0x68, 0x72, 0xeb, 0x73, 0x63, 0x68, 0x74, 0x64, 0x61, 0x67, 0x73, 0x69,
0x6e, 0x73, 0x65, 0x6c, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4b, 0x6f, 0x6b,
0x6f, 0x73, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61, 0x6e, 0x65,
0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x4b, 0x6f, 0x6c, 0x75,
0x6d, 0x62, 0x69, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4b, 0x6f, 0x6c, 0x75,
0x6d, 0x62, 0x69, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x43, 0x6f, 0x6f, 0x6b,
0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74,
0x43, 0x6f, 0x6f, 0x6b, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x43, 0x6f,
0x6f, 0x6b, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4b, 0x75, 0x62, 0x61,
0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x4b, 0x75,
0x62, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4b, 0x75, 0x62, 0x61, 0x6e, 0x65,
0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4,
0x69, 0x74, 0x44, 0x61, 0x76, 0x69, 0x73, 0x2d, 0x5a, 0xe4, 0x69, 0x74,
0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76,
0x69, 0x6c, 0x6c, 0x65, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4f, 0x73, 0x74,
0x74, 0x69, 0x6d, 0x6f, 0x72, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4f, 0x75,
0x73, 0x63, 0x68, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x2d,
0x5a, 0xe4, 0x69, 0x74, 0x4f, 0x75, 0x73, 0x63, 0x68, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x73, 0x65, 0x6c, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0xe4, 0x69, 0x74, 0x4f, 0x75, 0x73, 0x63, 0x68, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x73, 0x65, 0x6c, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0xe4, 0x69, 0x74, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x69,
0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x4d,
0xeb, 0x74, 0x74, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x65,
0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x4d, 0xeb, 0x74, 0x74,
0x65, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x65, 0x73, 0x63, 0x68,
0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4d,
0xeb, 0x74, 0x74, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x65,
0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4,
0x69, 0x74, 0x4f, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x65,
0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x65,
0x75, 0x72, 0x6f, 0x70, 0xe4, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x65,
0x75, 0x72, 0x6f, 0x70, 0xe4, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74,
0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a,
0xe4, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70,
0xe4, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0xe4, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f,
0x70, 0xe4, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0xe4, 0x69, 0x74, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d, 0x5a, 0xe4, 0x69,
0x74, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x6e, 0x73,
0x65, 0x6c, 0x65, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a,
0xe4, 0x69, 0x74, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0xe4, 0x69, 0x74, 0x46, 0x69, 0x64, 0x73, 0x63, 0x68, 0x69,
0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x46, 0x69, 0x64, 0x73, 0x63, 0x68, 0x69,
0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x46,
0x69, 0x64, 0x73, 0x63, 0x68, 0x69, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0xe4, 0x69, 0x74, 0x46, 0x72, 0x61, 0x6e, 0x73, 0xe9, 0x69,
0x73, 0x63, 0x68, 0x2d, 0x47, 0x75, 0x61, 0x79, 0x61, 0x6e, 0x65, 0x2d,
0x5a, 0xe4, 0x69, 0x74, 0x46, 0x72, 0x61, 0x6e, 0x73, 0xe9, 0x69, 0x73,
0x63, 0x68, 0x20, 0x53, 0xfc, 0x64, 0x2d, 0x20, 0x61, 0x6e, 0x20, 0x41,
0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x73, 0x67, 0x65, 0x62, 0x69,
0x64, 0x64, 0x65, 0x72, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x47, 0x61, 0x6c,
0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x47,
0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69,
0x74, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0xe4, 0x69, 0x74, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74,
0x2d, 0x49, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d, 0x5a, 0xe4, 0x69,
0x74, 0x4d, 0xeb, 0x74, 0x74, 0x6c, 0x65, 0x72, 0x20, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4f,
0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x5a,
0xe4, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69,
0x74, 0x4f, 0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x57,
0x65, 0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2d,
0x5a, 0xe4, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a,
0xe4, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4,
0x69, 0x74, 0x47, 0x75, 0x61, 0x6d, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x47,
0x6f, 0x6c, 0x66, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x47, 0x75, 0x79, 0x61,
0x6e, 0x61, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x48, 0x61, 0x77, 0x61, 0x69,
0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x65, 0x6e, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65,
0x75, 0x74, 0x65, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a,
0xe4, 0x69, 0x74, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x65, 0x6e, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0xe4, 0x69, 0x74, 0x48, 0x6f, 0x6e, 0x67, 0x2d, 0x4b, 0x6f, 0x6e,
0x67, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x48, 0x6f, 0x6e, 0x67, 0x2d, 0x4b,
0x6f, 0x6e, 0x67, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4,
0x69, 0x74, 0x48, 0x6f, 0x6e, 0x67, 0x2d, 0x4b, 0x6f, 0x6e, 0x67, 0x2d,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x48, 0x6f,
0x76, 0x64, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x48, 0x6f, 0x76, 0x64, 0x2d,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x48, 0x6f,
0x76, 0x64, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69,
0x74, 0x49, 0x6e, 0x64, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69,
0x74, 0x49, 0x6e, 0x64, 0x65, 0x73, 0x63, 0x68, 0x65, 0x6e, 0x20, 0x4f,
0x7a, 0x65, 0x61, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x49, 0x6e, 0x64,
0x6f, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x5a,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x73, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x4f, 0x73,
0x74, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x65, 0x73, 0x63, 0x68,
0x20, 0x5a, 0xe4, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x69, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x73, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69,
0x74, 0x49, 0x72, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4,
0x69, 0x74, 0x49, 0x72, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x49, 0x72, 0x61,
0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0xe4, 0x69, 0x74, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2d,
0x5a, 0xe4, 0x69, 0x74, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2d,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0xe4, 0x69, 0x74, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x65, 0x73,
0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x49, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0xe4, 0x69, 0x74, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x65, 0x73,
0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69,
0x74, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a,
0xe4, 0x69, 0x74, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68,
0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4a,
0x61, 0x70, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4b, 0x61, 0x6d, 0x74, 0x73,
0x63, 0x68, 0x61, 0x74, 0x6b, 0x61, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4b,
0x61, 0x6d, 0x74, 0x73, 0x63, 0x68, 0x61, 0x74, 0x6b, 0x61, 0x2d, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4b, 0x61, 0x6d,
0x74, 0x73, 0x63, 0x68, 0x61, 0x74, 0x6b, 0x61, 0x2d, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x6b, 0x61,
0x73, 0x61, 0x63, 0x68, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69,
0x74, 0x57, 0x65, 0x73, 0x74, 0x6b, 0x61, 0x73, 0x61, 0x63, 0x68, 0x65,
0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x4b, 0x6f, 0x72, 0x65,
0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x4b,
0x6f, 0x72, 0x65, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4b, 0x6f, 0x72, 0x65,
0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x2d,
0x5a, 0xe4, 0x69, 0x74, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61,
0x72, 0x73, 0x6b, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2d, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0xe4, 0x69, 0x74, 0x4b, 0x69, 0x72, 0x67, 0x69, 0x73, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4c, 0x69, 0x6e, 0x6e,
0x65, 0x6e, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48, 0x6f, 0x77, 0x65, 0x2d,
0x5a, 0xe4, 0x69, 0x74, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48, 0x6f, 0x77,
0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74,
0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4,
0x69, 0x74, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x2d, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4d, 0x61, 0x6c, 0x61,
0x79, 0x73, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x4d,
0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74,
0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x69, 0x6e,
0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x2d, 0x5a, 0xe4, 0x69, 0x74,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x2d, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x74, 0x69, 0x75, 0x73, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0xe4, 0x69, 0x74, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x2d, 0x5a,
0xe4, 0x69, 0x74, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x73,
0x63, 0x68, 0x20, 0x50, 0x61, 0x7a, 0x69, 0x66, 0x69, 0x6b, 0x7a, 0xe4,
0x69, 0x74, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x73, 0x63,
0x68, 0x20, 0x50, 0x61, 0x7a, 0x69, 0x66, 0x69, 0x6b, 0x2d, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4d, 0x65, 0x78, 0x69,
0x6b, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x50, 0x61, 0x7a, 0x69,
0x66, 0x69, 0x6b, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4,
0x69, 0x74, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61,
0x72, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62,
0x61, 0x61, 0x74, 0x61, 0x72, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0xe4, 0x69, 0x74, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61,
0x74, 0x61, 0x72, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4,
0x69, 0x74, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75, 0x65, 0x72, 0x20, 0x5a,
0xe4, 0x69, 0x74, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75, 0x65, 0x72, 0x20,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4d, 0x6f,
0x73, 0x6b, 0x61, 0x75, 0x65, 0x72, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72,
0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x2d, 0x5a,
0xe4, 0x69, 0x74, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x65, 0x73, 0x65, 0x73,
0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x4e, 0x65, 0x69, 0x6b, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4,
0x69, 0x74, 0x4e, 0x65, 0x69, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a,
0xe4, 0x69, 0x74, 0x4e, 0x65, 0x69, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0xe4, 0x69, 0x74, 0x4e, 0x65, 0x69, 0x73, 0xe9, 0x69, 0x6c, 0x61,
0x6e, 0x64, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4e, 0x65, 0x69, 0x73, 0xe9,
0x69, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0xe4, 0x69, 0x74, 0x4e, 0x65, 0x69, 0x73, 0xe9, 0x69, 0x6c, 0x61,
0x6e, 0x64, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69,
0x74, 0x4e, 0x65, 0x69, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64,
0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4e, 0x65, 0x69, 0x66, 0x75, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a,
0xe4, 0x69, 0x74, 0x4e, 0x65, 0x69, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69,
0x74, 0x4e, 0x69, 0x75, 0x65, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e,
0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x2d, 0x64, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x2d, 0x64, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69,
0x74, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x2d, 0x64, 0x65,
0x2d, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4e, 0x6f, 0x77, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4e,
0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2d, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4e, 0x6f, 0x77,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2d, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4f, 0x6d, 0x73, 0x6b, 0x2d,
0x5a, 0xe4, 0x69, 0x74, 0x4f, 0x6d, 0x73, 0x6b, 0x2d, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4f, 0x6d, 0x73, 0x6b, 0x2d,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a,
0xe4, 0x69, 0x74, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x65,
0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4,
0x69, 0x74, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x65, 0x73,
0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69,
0x74, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x50,
0x61, 0x70, 0x75, 0x61, 0x2d, 0x4e, 0x65, 0x69, 0x67, 0x75, 0x69, 0x6e,
0x65, 0x61, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x79, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4,
0x69, 0x74, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x61, 0x6e,
0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a,
0xe4, 0x69, 0x74, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x61,
0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0xe4, 0x69, 0x74, 0x50, 0x65, 0x72, 0x75, 0x61, 0x6e, 0x65, 0x73,
0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x50, 0x65, 0x72, 0x75, 0x61,
0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0xe4, 0x69, 0x74, 0x50, 0x65, 0x72, 0x75, 0x61, 0x6e, 0x65, 0x73,
0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69,
0x74, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x6e, 0x65,
0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x50, 0x68, 0x69, 0x6c,
0x69, 0x70, 0x70, 0x69, 0x6e, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x50, 0x68, 0x69,
0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x50, 0x68,
0x6f, 0x65, 0x6e, 0x69, 0x78, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e,
0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x61, 0x2d, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x53, 0x61, 0x69,
0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x61, 0x2d,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x53, 0x61, 0x69, 0x6e, 0x74,
0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x61, 0x2d, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0xe4, 0x69, 0x74, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72,
0x6e, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d, 0x5a, 0xe4, 0x69,
0x74, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x2d, 0x5a, 0xe4, 0x69, 0x74,
0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74,
0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x2d, 0x5a, 0xe4, 0x69, 0x74,
0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x5a, 0xe4, 0x69,
0x74, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0xe4, 0x69, 0x74, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x53, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4,
0x69, 0x74, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x2d, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65,
0x6c, 0x6c, 0x65, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x53, 0x69, 0x6e,
0x67, 0x61, 0x70, 0x75, 0x72, 0x2d, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x7a, 0xe4, 0x69, 0x74, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f,
0x6e, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d, 0x5a, 0xe4, 0x69,
0x74, 0x53, 0xfc, 0x64, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x65, 0x73, 0x63,
0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61,
0x6d, 0x65, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x53, 0x79, 0x6f, 0x77, 0x61,
0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x2d,
0x5a, 0xe4, 0x69, 0x74, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x2d, 0x5a,
0xe4, 0x69, 0x74, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x2d, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69,
0x74, 0x54, 0x61, 0x64, 0x73, 0x63, 0x68, 0x69, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x54, 0x6f, 0x6b, 0x65, 0x6c,
0x61, 0x75, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0xe4, 0x69, 0x74, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x54, 0x75, 0x72, 0x6b, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0xe4, 0x69, 0x74, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x2d,
0x5a, 0xe4, 0x69, 0x74, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x61,
0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x55, 0x72,
0x75, 0x67, 0x75, 0x79, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x79, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x55, 0x73, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74,
0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x55, 0x73, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x2d, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75,
0x65, 0x6c, 0x61, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x57, 0x6c, 0x61, 0x64,
0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x5a, 0xe4, 0x69, 0x74,
0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x57, 0x6c,
0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x57, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x57, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2d, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4,
0x69, 0x74, 0x57, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x5a, 0xe4, 0x69,
0x74, 0x57, 0x61, 0x6b, 0x65, 0x2d, 0x49, 0x6e, 0x73, 0x65, 0x6c, 0x2d,
0x5a, 0xe4, 0x69, 0x74, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x2d, 0x61,
0x2d, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x2d, 0x5a, 0xe4, 0x69, 0x74,
0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2d, 0x5a, 0xe4, 0x69, 0x74,
0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2d, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74,
0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x65,
0x72, 0x67, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4a, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x65, 0x72, 0x67, 0x2d, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4a, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x65, 0x72, 0x67, 0x2d, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x410, 0x43a, 0x440,
0x435, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442,
0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x442, 0x43e, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x410,
0x432, 0x433, 0x430, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x421, 0x440, 0x435,
0x434, 0x43d, 0x43e, 0x430, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x441, 0x442, 0x43e, 0x447,
0x43d, 0x43e, 0x430, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x408, 0x443, 0x436, 0x43d, 0x43e, 0x430, 0x444, 0x440, 0x438,
0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x430, 0x20, 0x420, 0x435, 0x43f, 0x443, 0x431,
0x43b, 0x438, 0x43a, 0x430, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x430,
0x444, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x430, 0x444, 0x440,
0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x43d,
0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x412, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x410, 0x43b, 0x458, 0x430, 0x441, 0x43a, 0x430,
0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x410, 0x43b, 0x458, 0x430,
0x441, 0x43a, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435,
0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x410, 0x43b, 0x458, 0x430, 0x441,
0x43a, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x410,
0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x41b, 0x435, 0x442, 0x43d, 0x43e,
0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x410,
0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x410, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x435, 0x432,
0x435, 0x440, 0x43d, 0x430, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430,
0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20,
0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432,
0x43e, 0x20, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x410, 0x43c,
0x435, 0x440, 0x438, 0x43a, 0x430, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x435,
0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a,
0x430, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x418, 0x441, 0x442, 0x43e,
0x447, 0x43d, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c,
0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x435, 0x432, 0x435,
0x440, 0x43d, 0x430, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x41f,
0x43b, 0x430, 0x43d, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d,
0x430, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x41f, 0x43b, 0x430,
0x43d, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x41f, 0x43b, 0x430, 0x43d, 0x438, 0x43d,
0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c,
0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x435, 0x432, 0x435,
0x440, 0x43d, 0x430, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x41f,
0x430, 0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d,
0x430, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x41f, 0x430, 0x446,
0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x41f, 0x430, 0x446, 0x438, 0x444, 0x438,
0x447, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c,
0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x435, 0x432, 0x435,
0x440, 0x43d, 0x430, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x410,
0x43d, 0x430, 0x434, 0x438, 0x440, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x410, 0x43d, 0x430, 0x434, 0x438, 0x440, 0x441, 0x43a, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x410, 0x43d, 0x430, 0x434, 0x438, 0x440, 0x441, 0x43a,
0x43e, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x410, 0x43f,
0x438, 0x458, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x410,
0x43f, 0x438, 0x458, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x410, 0x43f, 0x438, 0x458, 0x430,
0x410, 0x440, 0x430, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x430, 0x440, 0x430, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x410, 0x440, 0x430, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d,
0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442,
0x438, 0x43d, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x410,
0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x41b, 0x435, 0x442, 0x43d,
0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20,
0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d,
0x430, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x437, 0x430, 0x43f, 0x430, 0x434,
0x43d, 0x430, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430,
0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a,
0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e,
0x20, 0x432, 0x43e, 0x20, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x20,
0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x415, 0x440, 0x43c, 0x435, 0x43d, 0x438,
0x458, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x415, 0x440,
0x43c, 0x435, 0x43d, 0x438, 0x458, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x415, 0x440, 0x43c,
0x435, 0x43d, 0x438, 0x458, 0x430, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x442, 0x43b, 0x430,
0x43d, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x442,
0x43b, 0x430, 0x43d, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x43d,
0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x412, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b,
0x43d, 0x430, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458,
0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x426, 0x435, 0x43d,
0x442, 0x440, 0x430, 0x43b, 0x43d, 0x430, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x458, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441,
0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x426, 0x435, 0x43d,
0x442, 0x440, 0x430, 0x43b, 0x43d, 0x430, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x458, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x430, 0x20,
0x438, 0x20, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x20, 0x410, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x430, 0x421, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d,
0x430, 0x20, 0x438, 0x20, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x430, 0x41b, 0x435,
0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20,
0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432,
0x43e, 0x20, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x430, 0x20,
0x438, 0x20, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x20, 0x410, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x430, 0x412, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x430,
0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x430, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x441, 0x442, 0x43e, 0x447,
0x43d, 0x430, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458,
0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430,
0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442,
0x43e, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x430,
0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x430, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x417, 0x430, 0x43f, 0x430,
0x434, 0x43d, 0x430, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x458, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x417, 0x430,
0x43f, 0x430, 0x434, 0x43d, 0x430, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x458, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c,
0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x417, 0x430, 0x43f, 0x430,
0x434, 0x43d, 0x430, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x458, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x410,
0x437, 0x435, 0x440, 0x431, 0x435, 0x458, 0x45f, 0x430, 0x43d, 0x421, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x410, 0x437, 0x435, 0x440, 0x431, 0x435, 0x458,
0x45f, 0x430, 0x43d, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x410, 0x437, 0x435, 0x440, 0x431, 0x435,
0x458, 0x45f, 0x430, 0x43d, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x43a, 0x438, 0x20, 0x41e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x438, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x410, 0x437, 0x43e, 0x440, 0x441, 0x43a, 0x438, 0x20, 0x41e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x438, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x43a,
0x438, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430,
0x434, 0x435, 0x448, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x411,
0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x41b, 0x435, 0x442, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x411,
0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x412, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x411, 0x443, 0x442, 0x430, 0x43d, 0x412, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x411, 0x43e, 0x43b, 0x438, 0x432,
0x438, 0x458, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x458, 0x430, 0x421, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438,
0x458, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442,
0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b,
0x438, 0x458, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x411, 0x440, 0x443, 0x43d, 0x435, 0x458, 0x20, 0x414, 0x430, 0x440, 0x443, 0x441,
0x430, 0x43b, 0x430, 0x43c, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x41a, 0x430, 0x431, 0x43e, 0x20, 0x412, 0x435, 0x440, 0x434, 0x435, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41a, 0x430, 0x431, 0x43e, 0x20,
0x412, 0x435, 0x440, 0x434, 0x435, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441,
0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x41a, 0x430, 0x431,
0x43e, 0x20, 0x412, 0x435, 0x440, 0x434, 0x435, 0x412, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x427, 0x430, 0x43c, 0x43e, 0x440, 0x43e, 0x412, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x427, 0x430, 0x442, 0x430, 0x43c,
0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x427, 0x430, 0x442, 0x430,
0x43c, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430,
0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442,
0x43e, 0x20, 0x432, 0x43e, 0x20, 0x427, 0x430, 0x442, 0x430, 0x43c, 0x412, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x427, 0x438, 0x43b, 0x435, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x427, 0x438, 0x43b, 0x435, 0x41b,
0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435,
0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20,
0x432, 0x43e, 0x20, 0x427, 0x438, 0x43b, 0x435, 0x412, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x41a, 0x438, 0x43d, 0x430, 0x421, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x41a, 0x438, 0x43d, 0x430, 0x41b, 0x435, 0x442, 0x43d,
0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20,
0x41a, 0x438, 0x43d, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x411, 0x43e, 0x436, 0x438, 0x45c, 0x435, 0x43d, 0x20, 0x41e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x43e, 0x432, 0x438, 0x20, 0x41e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x438, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x458, 0x430, 0x421, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431,
0x438, 0x458, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435,
0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x41a, 0x43e, 0x43b, 0x443, 0x43c,
0x431, 0x438, 0x458, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x41a, 0x443, 0x43a, 0x43e, 0x432, 0x438, 0x20, 0x41e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x438, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41a,
0x443, 0x43a, 0x43e, 0x432, 0x438, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432,
0x438, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x41a, 0x443, 0x43a, 0x43e, 0x432, 0x438, 0x20, 0x41e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x41a, 0x443, 0x431, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x41a, 0x443, 0x431, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e,
0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x41a,
0x443, 0x431, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x414, 0x435, 0x458, 0x432, 0x438, 0x441, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x414, 0x438, 0x43c, 0x43e, 0x43d, 0x20, 0x414, 0x438, 0x440,
0x432, 0x438, 0x43b, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x418, 0x441, 0x442, 0x43e, 0x447, 0x435, 0x43d, 0x20, 0x422, 0x438, 0x43c, 0x43e,
0x440, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x412, 0x435,
0x43b, 0x438, 0x433, 0x434, 0x435, 0x43d, 0x441, 0x43a, 0x438, 0x20, 0x41e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x412, 0x435, 0x43b, 0x438, 0x433, 0x434, 0x435, 0x43d, 0x441, 0x43a, 0x438, 0x20,
0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x412, 0x435, 0x43b,
0x438, 0x433, 0x434, 0x435, 0x43d, 0x441, 0x43a, 0x438, 0x20, 0x41e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x415, 0x43a, 0x432, 0x430, 0x434, 0x43e, 0x440, 0x421, 0x440, 0x435, 0x434, 0x43d,
0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x421, 0x440, 0x435, 0x434, 0x43d, 0x43e, 0x435, 0x432, 0x440,
0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x440,
0x435, 0x434, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e,
0x20, 0x43b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x441, 0x442,
0x43e, 0x447, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e,
0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x430, 0x43b, 0x438,
0x43d, 0x438, 0x43d, 0x433, 0x440, 0x430, 0x434, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x435,
0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e,
0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f,
0x430, 0x434, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e,
0x20, 0x43b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x424, 0x43e, 0x43b,
0x43a, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x438, 0x20, 0x41e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x438, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x438, 0x20,
0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x41b, 0x435, 0x442, 0x43d, 0x43e,
0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x424,
0x43e, 0x43b, 0x43a, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x438, 0x20, 0x41e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x424, 0x438, 0x45f, 0x438, 0x421, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x424, 0x438, 0x45f, 0x438, 0x41b, 0x435, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x424, 0x438,
0x45f, 0x438, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x424,
0x440, 0x430, 0x43d, 0x446, 0x443, 0x441, 0x43a, 0x430, 0x20, 0x413, 0x432, 0x430,
0x458, 0x430, 0x43d, 0x430, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x443, 0x441, 0x43a,
0x43e, 0x20, 0x458, 0x443, 0x436, 0x43d, 0x43e, 0x20, 0x438, 0x20, 0x430, 0x43d,
0x442, 0x430, 0x440, 0x43a, 0x442, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x413, 0x430, 0x43c, 0x431, 0x435, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x413, 0x440, 0x443, 0x437,
0x438, 0x458, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x413,
0x440, 0x443, 0x437, 0x438, 0x458, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x413, 0x440, 0x443,
0x437, 0x438, 0x458, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x413, 0x438, 0x43b, 0x431, 0x435, 0x440, 0x442, 0x43e, 0x432, 0x438, 0x20,
0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x421, 0x440, 0x435, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x43f, 0x43e, 0x20, 0x413,
0x440, 0x438, 0x43d, 0x438, 0x447, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x435, 0x43d, 0x20, 0x413, 0x440,
0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x435, 0x43d, 0x20, 0x413, 0x440,
0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20,
0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x441,
0x442, 0x43e, 0x447, 0x435, 0x43d, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430,
0x43d, 0x434, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x417,
0x430, 0x43f, 0x430, 0x434, 0x435, 0x43d, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b,
0x430, 0x43d, 0x434, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x417,
0x430, 0x43f, 0x430, 0x434, 0x435, 0x43d, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b,
0x430, 0x43d, 0x434, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435,
0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x417, 0x430, 0x43f, 0x430, 0x434,
0x435, 0x43d, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41f, 0x435, 0x440, 0x441,
0x438, 0x441, 0x43a, 0x438, 0x20, 0x417, 0x430, 0x43b, 0x438, 0x432, 0x412, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x413, 0x432, 0x430, 0x458, 0x430,
0x43d, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x425,
0x430, 0x432, 0x430, 0x438, 0x20, 0x2d, 0x20, 0x410, 0x43b, 0x435, 0x443, 0x442,
0x441, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x425, 0x430, 0x432, 0x430, 0x438,
0x20, 0x2d, 0x20, 0x410, 0x43b, 0x435, 0x443, 0x442, 0x441, 0x43a, 0x438, 0x20,
0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x41b, 0x435, 0x442, 0x43d, 0x43e,
0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x425,
0x430, 0x432, 0x430, 0x438, 0x20, 0x2d, 0x20, 0x410, 0x43b, 0x435, 0x443, 0x442,
0x441, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x425, 0x43e, 0x43d, 0x433,
0x20, 0x41a, 0x43e, 0x43d, 0x433, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x425, 0x43e, 0x43d, 0x433, 0x20, 0x41a, 0x43e, 0x43d, 0x433, 0x41b, 0x435,
0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x425, 0x43e, 0x43d, 0x433, 0x20, 0x41a, 0x43e, 0x43d, 0x433, 0x412, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x425, 0x43e, 0x432, 0x434, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x425, 0x43e, 0x432, 0x434, 0x41b,
0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x425, 0x43e, 0x432, 0x434, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x458, 0x430, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x441, 0x43a,
0x438, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x412, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43a, 0x438, 0x43d, 0x430,
0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x426, 0x435, 0x43d,
0x442, 0x440, 0x430, 0x43b, 0x43d, 0x430, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d,
0x435, 0x437, 0x438, 0x458, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x430, 0x20, 0x418, 0x43d,
0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x458, 0x430, 0x412, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430,
0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x458, 0x430, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x440, 0x430, 0x43d,
0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x440, 0x430, 0x43d,
0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a,
0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e,
0x20, 0x432, 0x43e, 0x20, 0x418, 0x440, 0x430, 0x43d, 0x412, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a,
0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x440, 0x43a, 0x443,
0x442, 0x441, 0x43a, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441,
0x43a, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x437,
0x440, 0x430, 0x435, 0x43b, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x418, 0x437, 0x440, 0x430, 0x435, 0x43b, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20,
0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x437,
0x440, 0x430, 0x435, 0x43b, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x408, 0x430, 0x43f, 0x43e, 0x43d, 0x438, 0x458, 0x430, 0x421, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x408, 0x430, 0x43f, 0x43e, 0x43d, 0x438, 0x458,
0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430,
0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442,
0x43e, 0x20, 0x432, 0x43e, 0x20, 0x408, 0x430, 0x43f, 0x43e, 0x43d, 0x438, 0x458,
0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41a, 0x430,
0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x412, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x435, 0x43d, 0x20,
0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x435,
0x43d, 0x20, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41a, 0x43e, 0x440, 0x435,
0x458, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41a, 0x43e,
0x440, 0x435, 0x458, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c,
0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x41a, 0x43e, 0x440, 0x435,
0x458, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41a,
0x43e, 0x441, 0x440, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x458, 0x430, 0x440, 0x441, 0x43a,
0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41a, 0x440, 0x430, 0x441,
0x43d, 0x43e, 0x458, 0x430, 0x440, 0x441, 0x43a, 0x41b, 0x435, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41a, 0x440,
0x430, 0x441, 0x43d, 0x43e, 0x458, 0x430, 0x440, 0x441, 0x43a, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41a, 0x438, 0x440, 0x433, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x41b, 0x438, 0x43d, 0x438, 0x441, 0x43a, 0x438, 0x20, 0x41e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x438, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x430, 0x443, 0x421, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x430, 0x443,
0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a,
0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e,
0x20, 0x432, 0x43e, 0x20, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x430, 0x443,
0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41c, 0x430, 0x433,
0x430, 0x434, 0x430, 0x43d, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x41b, 0x435, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41c, 0x430,
0x433, 0x430, 0x434, 0x430, 0x43d, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x41c, 0x430, 0x43b, 0x435, 0x437, 0x438, 0x458, 0x430, 0x412, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41c, 0x430, 0x43b, 0x434, 0x438,
0x432, 0x438, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41c,
0x430, 0x440, 0x43a, 0x435, 0x441, 0x430, 0x441, 0x412, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x41c, 0x430, 0x440, 0x448, 0x430, 0x43b, 0x441, 0x43a,
0x438, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x446,
0x438, 0x443, 0x441, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41c,
0x430, 0x432, 0x440, 0x438, 0x446, 0x438, 0x443, 0x441, 0x41b, 0x435, 0x442, 0x43d,
0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20,
0x41c, 0x430, 0x432, 0x440, 0x438, 0x446, 0x438, 0x443, 0x441, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41c, 0x43e, 0x441, 0x43e, 0x43d, 0x41f,
0x430, 0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a,
0x43e, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x43f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41c, 0x435, 0x43a, 0x441, 0x438,
0x43a, 0x43e, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x43f, 0x430, 0x446, 0x438,
0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x43e, 0x412, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x423, 0x43b, 0x430, 0x43d, 0x20,
0x411, 0x430, 0x442, 0x43e, 0x440, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x423, 0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x41b,
0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x423, 0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430, 0x442, 0x43e, 0x440,
0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41c, 0x43e, 0x441,
0x43a, 0x432, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41c,
0x43e, 0x441, 0x43a, 0x432, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441,
0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x41c, 0x43e, 0x441,
0x43a, 0x432, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x41c, 0x458, 0x430, 0x43d, 0x43c, 0x430, 0x440, 0x412, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x41d, 0x430, 0x443, 0x440, 0x443, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41d, 0x435, 0x43f, 0x430, 0x43b, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41d, 0x43e, 0x432, 0x430,
0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x458, 0x430, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41d, 0x43e, 0x432, 0x430, 0x20,
0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x458, 0x430, 0x41b, 0x435,
0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x41d, 0x43e, 0x432, 0x430, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e,
0x43d, 0x438, 0x458, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x41d, 0x43e, 0x432, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41d, 0x43e, 0x432, 0x20, 0x417,
0x435, 0x43b, 0x430, 0x43d, 0x434, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441,
0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x41d, 0x43e, 0x432,
0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x412, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x40a, 0x443, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b,
0x435, 0x43d, 0x434, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x40a,
0x443, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x41b, 0x435,
0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20,
0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432,
0x43e, 0x20, 0x40a, 0x443, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d,
0x434, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41d, 0x438,
0x443, 0x435, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41d,
0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x448, 0x43a, 0x438, 0x20, 0x41e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41d,
0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x448, 0x43a, 0x438, 0x20, 0x41e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435,
0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x41d, 0x43e, 0x440, 0x444, 0x43e,
0x43b, 0x448, 0x43a, 0x438, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x424, 0x435, 0x440, 0x43d,
0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e,
0x45a, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x424, 0x435,
0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d, 0x43e,
0x440, 0x43e, 0x45a, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c,
0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x424, 0x435, 0x440, 0x43d,
0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e,
0x45a, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41d,
0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x421, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438,
0x431, 0x438, 0x440, 0x441, 0x43a, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41d, 0x43e, 0x432, 0x43e,
0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x412, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x41e, 0x43c, 0x441, 0x43a, 0x421, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x41e, 0x43c, 0x441, 0x43a, 0x41b, 0x435, 0x442, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41e,
0x43c, 0x441, 0x43a, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x421, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d,
0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41f, 0x430, 0x43b, 0x430,
0x443, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41f, 0x430,
0x43f, 0x443, 0x430, 0x20, 0x41d, 0x43e, 0x432, 0x430, 0x20, 0x413, 0x432, 0x438,
0x43d, 0x435, 0x458, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x458, 0x421, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430,
0x458, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430,
0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442,
0x43e, 0x20, 0x432, 0x43e, 0x20, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430,
0x458, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41f, 0x435,
0x440, 0x443, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41f, 0x435,
0x440, 0x443, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442,
0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x41f, 0x435, 0x440, 0x443, 0x412, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x424, 0x438, 0x43b, 0x438, 0x43f,
0x438, 0x43d, 0x438, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x424,
0x438, 0x43b, 0x438, 0x43f, 0x438, 0x43d, 0x438, 0x41b, 0x435, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x424, 0x438,
0x43b, 0x438, 0x43f, 0x438, 0x43d, 0x438, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x442, 0x435,
0x20, 0x424, 0x435, 0x43d, 0x438, 0x43a, 0x441, 0x412, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x421, 0x435, 0x43d, 0x442, 0x20, 0x41f, 0x458, 0x435,
0x440, 0x20, 0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x430, 0x43d, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x435, 0x43d, 0x442, 0x20,
0x41f, 0x458, 0x435, 0x440, 0x20, 0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b,
0x430, 0x43d, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442,
0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x435, 0x43d, 0x442, 0x20, 0x41f,
0x458, 0x435, 0x440, 0x20, 0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x430,
0x43d, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41f, 0x438,
0x442, 0x43a, 0x435, 0x440, 0x43d, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x41f, 0x43e, 0x43d, 0x430, 0x43f, 0x435, 0x412, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x41f, 0x458, 0x43e, 0x43d, 0x433, 0x458, 0x430,
0x43d, 0x433, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x420,
0x435, 0x458, 0x443, 0x43d, 0x438, 0x43e, 0x43d, 0x412, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x420, 0x43e, 0x442, 0x435, 0x440, 0x430, 0x412, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x430, 0x43a, 0x430, 0x43b,
0x438, 0x43d, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x430,
0x43a, 0x430, 0x43b, 0x438, 0x43d, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x430, 0x43a, 0x430,
0x43b, 0x438, 0x43d, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x430, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20,
0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x412, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x430, 0x43c, 0x43e, 0x430,
0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x430, 0x43c, 0x43e,
0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x435, 0x458, 0x448, 0x435, 0x43b,
0x438, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x438,
0x43d, 0x433, 0x430, 0x43f, 0x443, 0x440, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x421, 0x43e, 0x43b, 0x43e, 0x43c, 0x43e, 0x43d, 0x441, 0x43a,
0x438, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x408, 0x443, 0x436, 0x43d, 0x430, 0x20,
0x413, 0x440, 0x443, 0x437, 0x438, 0x458, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x421, 0x443, 0x440, 0x438, 0x43d, 0x430, 0x43c, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x430, 0x458, 0x43e,
0x432, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x422,
0x430, 0x445, 0x438, 0x442, 0x438, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x422, 0x430, 0x458, 0x43f, 0x435, 0x458, 0x421, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x422, 0x430, 0x458, 0x43f, 0x435, 0x458, 0x41b, 0x435,
0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20,
0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432,
0x43e, 0x20, 0x422, 0x430, 0x458, 0x43f, 0x435, 0x458, 0x412, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x422, 0x430, 0x45f, 0x438, 0x43a, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x422, 0x43e, 0x43a, 0x435, 0x43b, 0x430, 0x443, 0x412, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x421, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x41b, 0x435,
0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x427, 0x443, 0x443, 0x43a, 0x412, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438,
0x441, 0x442, 0x430, 0x43d, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d,
0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x422, 0x443, 0x432, 0x430, 0x43b, 0x443, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x458, 0x421, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430,
0x458, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430,
0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442,
0x43e, 0x20, 0x432, 0x43e, 0x20, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x458,
0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x423, 0x437, 0x431,
0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x421, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430,
0x43d, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442,
0x430, 0x43d, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x412,
0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x41b, 0x435, 0x442,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x412, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x412, 0x435, 0x43d, 0x435, 0x446, 0x443, 0x435, 0x43b,
0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x412, 0x43b,
0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x421, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e,
0x441, 0x442, 0x43e, 0x43a, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x412, 0x43b, 0x430, 0x434, 0x438,
0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434,
0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x412, 0x43e, 0x43b, 0x433,
0x43e, 0x433, 0x440, 0x430, 0x434, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441,
0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x412, 0x43e, 0x43b,
0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432,
0x43e, 0x442, 0x20, 0x412, 0x435, 0x458, 0x43a, 0x412, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x412, 0x430, 0x43b, 0x438, 0x441, 0x20, 0x438, 0x20,
0x424, 0x443, 0x442, 0x443, 0x43d, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x408, 0x430, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x421, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x408, 0x430, 0x43a, 0x443, 0x442, 0x441,
0x43a, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x408, 0x430, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x415, 0x43a, 0x430, 0x442,
0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x421, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d,
0x431, 0x443, 0x440, 0x433, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x415, 0x43a, 0x430, 0x442, 0x435,
0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x412, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x408, 0x443, 0x43a, 0x43e, 0x43d, 0x905, 0x92b, 0x917,
0x93e, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x905, 0x92b, 0x94d, 0x930, 0x940, 0x915,
0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x942, 0x930, 0x92c, 0x20, 0x905,
0x92b, 0x94d, 0x930, 0x940, 0x915, 0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x926,
0x915, 0x94d, 0x937, 0x93f, 0x923, 0x20, 0x905, 0x92b, 0x94d, 0x930, 0x940, 0x915,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a,
0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x905, 0x92b, 0x94d, 0x930, 0x940, 0x915,
0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e,
0x20, 0x905, 0x92b, 0x94d, 0x930, 0x940, 0x915, 0x93e, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x905,
0x92b, 0x94d, 0x930, 0x940, 0x915, 0x93e, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x905, 0x932, 0x93e, 0x938, 0x94d, 0x915, 0x93e, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x932, 0x93e, 0x938, 0x94d,
0x915, 0x93e, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e,
0x92f, 0x905, 0x92e, 0x947, 0x91c, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x905,
0x92e, 0x947, 0x91c, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x905, 0x92e, 0x947, 0x91c, 0x928, 0x20, 0x938, 0x92e, 0x930, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x905, 0x930, 0x947, 0x92c, 0x93f, 0x92f, 0x928, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x905, 0x930, 0x947, 0x92c, 0x93f, 0x92f, 0x928, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x905, 0x930, 0x947,
0x92c, 0x93f, 0x92f, 0x928, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940,
0x928, 0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x905, 0x930, 0x94d, 0x91c, 0x947,
0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928,
0x93e, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x936,
0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902,
0x91f, 0x940, 0x928, 0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x936, 0x94d,
0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f,
0x940, 0x928, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905, 0x930, 0x94d,
0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20, 0x938, 0x92e, 0x930, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x906, 0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f, 0x92f,
0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x906, 0x930, 0x94d, 0x92e, 0x947, 0x928,
0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x906, 0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x938,
0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92e, 0x927, 0x94d, 0x92f, 0x20,
0x906, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x906, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932,
0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x906, 0x938, 0x94d, 0x91f, 0x94d,
0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x92e, 0x927, 0x94d, 0x92f,
0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x906, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908,
0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e,
0x940, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x906, 0x938, 0x94d, 0x91f, 0x94d, 0x930,
0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x20,
0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x906, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947,
0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x92a,
0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907,
0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x942, 0x930, 0x92c, 0x940, 0x20,
0x906, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x906, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932,
0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x92a, 0x942, 0x930, 0x92c, 0x940, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x906, 0x938, 0x94d, 0x91f,
0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x92a, 0x942, 0x930,
0x92c, 0x940, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x906, 0x938,
0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x906, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f,
0x93e, 0x908, 0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x906, 0x938, 0x94d, 0x91f,
0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x92a, 0x936, 0x94d,
0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x905, 0x91c, 0x930, 0x92c, 0x948, 0x91c, 0x93e, 0x928,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x905, 0x91c, 0x930, 0x92c, 0x948, 0x91c, 0x93e,
0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x905,
0x91c, 0x930, 0x92c, 0x948, 0x91c, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x930, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x90f, 0x91c, 0x94b, 0x930, 0x94d, 0x938, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x90f, 0x91c, 0x94b, 0x930, 0x94d, 0x938, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x90f, 0x91c, 0x94b, 0x930, 0x94d,
0x938, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92c, 0x902,
0x917, 0x932, 0x93e, 0x926, 0x947, 0x936, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92c,
0x902, 0x917, 0x932, 0x93e, 0x926, 0x947, 0x936, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92c, 0x902, 0x917, 0x932, 0x93e, 0x926, 0x947,
0x936, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92d, 0x942,
0x91f, 0x93e, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92c, 0x94b, 0x932, 0x940,
0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92c, 0x94d, 0x930,
0x93e, 0x91c, 0x940, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x940, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92c, 0x94d, 0x930, 0x93e,
0x91c, 0x940, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x92c, 0x94d, 0x930, 0x942, 0x928, 0x947, 0x908, 0x20, 0x926,
0x93e, 0x930, 0x947, 0x938, 0x932, 0x93e, 0x92e, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x915, 0x947, 0x92a, 0x20, 0x92c, 0x930, 0x94d, 0x921, 0x947, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x915, 0x947, 0x92a, 0x20, 0x92c, 0x930, 0x94d, 0x921, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x915, 0x947, 0x92a, 0x20,
0x92c, 0x930, 0x94d, 0x921, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x91a, 0x947, 0x925, 0x92e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x91a, 0x947,
0x925, 0x92e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x91a, 0x947, 0x925, 0x92e, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x91a, 0x93f, 0x932, 0x940, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x91a, 0x93f, 0x932, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x91a, 0x93f, 0x932, 0x940, 0x20, 0x938, 0x92e, 0x930, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x91a, 0x940, 0x928, 0x940, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x91a, 0x940, 0x928, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x924, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x91a, 0x940, 0x928, 0x940, 0x20, 0x921, 0x947, 0x932, 0x93e,
0x907, 0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x915, 0x94d, 0x930, 0x93f, 0x938,
0x92e, 0x938, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x915, 0x94b, 0x915, 0x94b, 0x938, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x915, 0x94b, 0x932, 0x902, 0x92c, 0x93f, 0x92f,
0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x915, 0x94b, 0x932, 0x902, 0x92c, 0x93f,
0x92f, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x915, 0x94b, 0x932, 0x902, 0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x930,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x915, 0x94d, 0x92f, 0x942,
0x92c, 0x93e, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x92a, 0x942, 0x930, 0x92c, 0x940, 0x20, 0x924, 0x93f, 0x92e, 0x94b,
0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x908, 0x938, 0x94d, 0x91f, 0x930, 0x20,
0x906, 0x907, 0x932, 0x948, 0x902, 0x921, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x908,
0x938, 0x94d, 0x91f, 0x930, 0x20, 0x906, 0x907, 0x932, 0x948, 0x902, 0x921, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x908, 0x938, 0x94d,
0x91f, 0x930, 0x20, 0x906, 0x907, 0x932, 0x948, 0x902, 0x921, 0x20, 0x938, 0x92e,
0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x907, 0x915, 0x94d, 0x935, 0x93e, 0x921,
0x94b, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92b, 0x930, 0x926, 0x930, 0x2d,
0x908, 0x938, 0x94d, 0x91f, 0x930, 0x94d, 0x928, 0x20, 0x92f, 0x942, 0x930, 0x94b,
0x92a, 0x940, 0x92f, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92b, 0x93e, 0x915,
0x932, 0x948, 0x902, 0x921, 0x20, 0x906, 0x907, 0x932, 0x948, 0x902, 0x921, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x92b, 0x93e, 0x915, 0x932, 0x948, 0x902, 0x921, 0x20,
0x906, 0x907, 0x932, 0x948, 0x902, 0x921, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x92b, 0x93e, 0x915, 0x932, 0x948, 0x902, 0x921, 0x20,
0x906, 0x907, 0x932, 0x948, 0x902, 0x921, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x92b, 0x94d, 0x930, 0x947, 0x902, 0x91a, 0x20, 0x917, 0x941,
0x92f, 0x93e, 0x928, 0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92b, 0x93c, 0x94d,
0x930, 0x93e, 0x902, 0x938, 0x940, 0x938, 0x940, 0x20, 0x926, 0x915, 0x94d, 0x937,
0x93f, 0x923, 0x940, 0x20, 0x915, 0x94d, 0x937, 0x947, 0x924, 0x94d, 0x930, 0x20,
0x906, 0x20, 0x905, 0x902, 0x91f, 0x93e, 0x930, 0x94d, 0x915, 0x91f, 0x93f, 0x915,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x917, 0x948, 0x932, 0x93e, 0x92a, 0x948, 0x917,
0x94b, 0x938, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x91c, 0x93e, 0x930, 0x94d, 0x91c,
0x93f, 0x92f, 0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x91c, 0x93e, 0x930, 0x94d,
0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x91c, 0x93e, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x938,
0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x942, 0x930, 0x92c, 0x940,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x948, 0x902, 0x921, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x92a, 0x942, 0x930, 0x92c, 0x940, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x928, 0x932, 0x948, 0x902, 0x921, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x942, 0x930, 0x92c, 0x940, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x928, 0x932, 0x948, 0x902, 0x921, 0x20, 0x938, 0x92e, 0x930, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x948, 0x902, 0x921, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x928, 0x932, 0x948, 0x902, 0x921, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x948, 0x902, 0x921, 0x20, 0x938,
0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x917, 0x932, 0x94d, 0x92b, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x935, 0x93e, 0x908,
0x2d, 0x90f, 0x932, 0x94d, 0x92f, 0x942, 0x91f, 0x93f, 0x92f, 0x928, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x939, 0x935, 0x93e, 0x908, 0x2d, 0x90f, 0x932, 0x94d, 0x92f,
0x942, 0x91f, 0x93f, 0x92f, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x939, 0x935, 0x93e, 0x908, 0x2d, 0x90f, 0x932, 0x94d, 0x92f,
0x942, 0x91f, 0x93f, 0x92f, 0x928, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x939, 0x93e, 0x902, 0x917, 0x20, 0x915, 0x93e,
0x902, 0x917, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x939, 0x93e, 0x902, 0x917, 0x20,
0x915, 0x93e, 0x902, 0x917, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x939, 0x93e, 0x902, 0x917, 0x20, 0x915, 0x93e, 0x902, 0x917, 0x20,
0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x939, 0x94b, 0x92c, 0x94d,
0x926, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x939, 0x94b, 0x92c, 0x94d, 0x926, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x939, 0x94b, 0x92c,
0x94d, 0x926, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x939,
0x93f, 0x902, 0x926, 0x20, 0x92e, 0x939, 0x93e, 0x938, 0x93e, 0x917, 0x930, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x907, 0x902, 0x921, 0x94b, 0x91a, 0x93e, 0x907, 0x928,
0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x907,
0x902, 0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x92a, 0x942, 0x930, 0x92c, 0x940, 0x20, 0x907, 0x902, 0x921, 0x94b,
0x928, 0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a,
0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x907, 0x902, 0x921, 0x94b, 0x928,
0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x908, 0x930,
0x93e, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x908, 0x930, 0x93e, 0x928, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x908, 0x930, 0x93e, 0x928,
0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x907, 0x930, 0x915, 0x942, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x907, 0x930, 0x915, 0x942, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x907, 0x930, 0x915,
0x942, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x907, 0x91c, 0x930, 0x93e, 0x92f, 0x932, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x907, 0x91c, 0x930, 0x93e, 0x92f, 0x932, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x907, 0x91c, 0x930, 0x93e, 0x92f, 0x932,
0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x91c, 0x93e, 0x92a, 0x93e, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x91c, 0x93e,
0x92a, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x91c, 0x93e, 0x92a, 0x93e, 0x928, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907,
0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x942, 0x930, 0x92c, 0x20, 0x915,
0x91c, 0x93e, 0x916, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x915, 0x91c, 0x93e, 0x916,
0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x915, 0x94b,
0x930, 0x93f, 0x92f, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x915, 0x94b, 0x930,
0x93f, 0x92f, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x915, 0x94b, 0x930, 0x93f, 0x92f, 0x928, 0x20, 0x921, 0x947, 0x932, 0x93e,
0x907, 0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x915, 0x94d, 0x930, 0x947, 0x938,
0x94d, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x915, 0x94d,
0x930, 0x947, 0x938, 0x94d, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d,
0x915, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x915,
0x94d, 0x930, 0x947, 0x938, 0x94d, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938,
0x94d, 0x915, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x915,
0x93f, 0x930, 0x94d, 0x917, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x932, 0x93e,
0x930, 0x94d, 0x921, 0x20, 0x939, 0x94b, 0x92c, 0x947, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x932, 0x93e, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x94b, 0x92c, 0x947, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x932, 0x93e, 0x930,
0x94d, 0x921, 0x20, 0x939, 0x94b, 0x92c, 0x947, 0x20, 0x921, 0x947, 0x932, 0x93e,
0x907, 0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92e, 0x917, 0x93e, 0x926, 0x93e,
0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92e, 0x917, 0x93e, 0x926, 0x93e, 0x928,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92e, 0x917,
0x93e, 0x926, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x92e, 0x932, 0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x92e, 0x93e, 0x932, 0x926, 0x940, 0x935, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x92e, 0x93e, 0x930, 0x940, 0x936, 0x938, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92e,
0x93e, 0x930, 0x940, 0x936, 0x938, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x92e, 0x93e, 0x930, 0x940, 0x936, 0x938, 0x20, 0x938, 0x92e,
0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92e, 0x948, 0x915, 0x94d, 0x938, 0x93f,
0x915, 0x928, 0x20, 0x92a, 0x947, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x92e, 0x948, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x928, 0x20,
0x92a, 0x947, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92e, 0x948, 0x915, 0x94d, 0x938, 0x93f, 0x915,
0x928, 0x20, 0x92a, 0x947, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x921, 0x947,
0x932, 0x93e, 0x907, 0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x909, 0x932, 0x928,
0x92c, 0x91f, 0x94b, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x909, 0x932, 0x928,
0x92c, 0x91f, 0x94b, 0x930, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x909, 0x932, 0x928, 0x92c, 0x91f, 0x94b, 0x930, 0x20, 0x938, 0x92e,
0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92e, 0x93e, 0x938, 0x94d, 0x915, 0x94b,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92e, 0x93e, 0x938, 0x94d, 0x915, 0x94b, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92e, 0x93e, 0x938,
0x94d, 0x915, 0x94b, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x92e, 0x92f, 0x928, 0x92e, 0x93e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x928,
0x947, 0x92a, 0x93e, 0x932, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x928, 0x94d, 0x92f,
0x942, 0x20, 0x915, 0x947, 0x932, 0x948, 0x921, 0x94b, 0x928, 0x93f, 0x92f, 0x93e,
0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x915, 0x947, 0x932,
0x948, 0x921, 0x94b, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x915, 0x947, 0x932,
0x948, 0x921, 0x94b, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x921, 0x947, 0x932, 0x93e,
0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x91c, 0x940,
0x932, 0x948, 0x902, 0x921, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x928, 0x94d, 0x92f,
0x942, 0x91c, 0x940, 0x932, 0x948, 0x902, 0x921, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x928, 0x94d, 0x92f, 0x942, 0x91c, 0x940, 0x932,
0x948, 0x902, 0x921, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x928, 0x94d, 0x92f, 0x942, 0x92b, 0x93e, 0x909, 0x902, 0x921,
0x932, 0x948, 0x902, 0x921, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x928, 0x94d, 0x92f,
0x942, 0x92b, 0x93e, 0x909, 0x902, 0x921, 0x932, 0x948, 0x902, 0x921, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x928, 0x94d, 0x92f, 0x942,
0x92b, 0x93e, 0x909, 0x902, 0x921, 0x932, 0x948, 0x902, 0x921, 0x20, 0x921, 0x947,
0x932, 0x93e, 0x907, 0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92b, 0x93e, 0x915,
0x932, 0x948, 0x902, 0x921, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x92b, 0x93e, 0x915, 0x932, 0x948, 0x902, 0x921, 0x20, 0x926,
0x94d, 0x935, 0x940, 0x92a, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x92b, 0x93e, 0x915, 0x932, 0x948, 0x902, 0x921, 0x20, 0x926, 0x94d,
0x935, 0x940, 0x92a, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x92b, 0x930, 0x94d, 0x928, 0x947, 0x902, 0x921, 0x94b, 0x20,
0x921, 0x940, 0x20, 0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x92b, 0x930, 0x94d, 0x928, 0x947, 0x902, 0x921, 0x94b,
0x20, 0x921, 0x940, 0x20, 0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92b, 0x930,
0x94d, 0x928, 0x947, 0x902, 0x921, 0x94b, 0x20, 0x921, 0x940, 0x20, 0x928, 0x94b,
0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x928, 0x94b, 0x92c, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930,
0x94d, 0x938, 0x94d, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92a, 0x928, 0x94b, 0x92c,
0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x928, 0x94b, 0x92c, 0x94b,
0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x938, 0x92e,
0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x913, 0x92e, 0x94d, 0x938, 0x94d, 0x915,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x913, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x913, 0x92e, 0x94d,
0x938, 0x94d, 0x915, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x93e, 0x915,
0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x92a, 0x930, 0x93e, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x92a, 0x930, 0x93e, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x930, 0x93e, 0x917,
0x94d, 0x935, 0x947, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x92a, 0x947, 0x930, 0x942, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x947, 0x930,
0x942, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a,
0x947, 0x930, 0x942, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x92b, 0x93f, 0x932, 0x93f, 0x92a, 0x940, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x92b, 0x93f, 0x932, 0x93f, 0x92a, 0x940, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92b, 0x93f, 0x932, 0x93f, 0x92a, 0x940, 0x928,
0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x938, 0x947, 0x902,
0x91f, 0x20, 0x92a, 0x93f, 0x90f, 0x930, 0x947, 0x20, 0x906, 0x20, 0x92e, 0x93f,
0x915, 0x94d, 0x935, 0x947, 0x932, 0x93e, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x938, 0x947, 0x902, 0x91f, 0x20, 0x92a, 0x93f, 0x90f, 0x930, 0x947, 0x20, 0x906,
0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932, 0x93e, 0x928, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x938, 0x947, 0x902, 0x91f,
0x20, 0x92a, 0x93f, 0x90f, 0x930, 0x947, 0x20, 0x906, 0x20, 0x92e, 0x93f, 0x915,
0x94d, 0x935, 0x947, 0x932, 0x93e, 0x928, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907,
0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x94d, 0x92f, 0x94b, 0x902, 0x917,
0x92f, 0x93e, 0x902, 0x917, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x930, 0x93f, 0x92f,
0x942, 0x928, 0x93f, 0x92f, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x938, 0x915,
0x93e, 0x932, 0x93f, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x938, 0x915, 0x93e,
0x932, 0x93f, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x938, 0x915, 0x93e, 0x932, 0x93f, 0x928, 0x20, 0x938, 0x92e, 0x930, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x938, 0x93e, 0x92e, 0x94b, 0x906, 0x20, 0x938, 0x92e,
0x92f, 0x938, 0x93e, 0x92e, 0x94b, 0x906, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x938, 0x93e, 0x92e, 0x94b, 0x906, 0x20, 0x938, 0x92e, 0x930,
0x20, 0x938, 0x92e, 0x92f, 0x938, 0x947, 0x936, 0x947, 0x932, 0x94d, 0x938, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x938, 0x93f, 0x902, 0x917, 0x93e, 0x92a, 0x941, 0x930,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x926, 0x915,
0x94d, 0x937, 0x93f, 0x923, 0x20, 0x91c, 0x93e, 0x930, 0x94d, 0x91c, 0x93f, 0x92f,
0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x938, 0x941, 0x930, 0x940, 0x928, 0x93e,
0x92e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x924, 0x93e, 0x907, 0x92a, 0x947, 0x908,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x924, 0x93e, 0x907, 0x92a, 0x947, 0x908, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x924, 0x93e, 0x907,
0x92a, 0x947, 0x908, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x924, 0x941, 0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928, 0x93f,
0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x924, 0x941,
0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x924, 0x941,
0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928,
0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x909, 0x930, 0x941,
0x917, 0x94d, 0x935, 0x947, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x909, 0x930, 0x941,
0x917, 0x94d, 0x935, 0x947, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x909, 0x930, 0x941, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x938, 0x92e,
0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x909, 0x91c, 0x92c, 0x947, 0x915, 0x93f,
0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x909, 0x91c,
0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x909, 0x91c, 0x92c, 0x947, 0x915,
0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x92c, 0x947, 0x928, 0x947, 0x91c, 0x941, 0x90f, 0x932, 0x93e,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92c, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x92c,
0x94b, 0x938, 0x94d, 0x924, 0x94b, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92c,
0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x92c, 0x94b, 0x938, 0x94d, 0x924, 0x94b, 0x915,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92c, 0x94d,
0x932, 0x93e, 0x926, 0x93f, 0x92c, 0x94b, 0x938, 0x94d, 0x924, 0x94b, 0x915, 0x20,
0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92c, 0x94b, 0x932, 0x94d,
0x917, 0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x926, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x92c, 0x94b, 0x932, 0x94d, 0x917, 0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x926, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92c, 0x94b, 0x932,
0x94d, 0x917, 0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x926, 0x20, 0x938, 0x92e, 0x930,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92f, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d,
0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92f, 0x915, 0x941, 0x924, 0x94d, 0x938,
0x94d, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x92f, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x938, 0x92e, 0x930,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92f, 0x947, 0x915, 0x93e, 0x91f, 0x948, 0x930,
0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92f,
0x947, 0x915, 0x93e, 0x91f, 0x948, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92f, 0x947,
0x915, 0x93e, 0x91f, 0x948, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20,
0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92f, 0x942, 0x915, 0x94b,
0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0xd0e, 0xd2f, 0xd4d, 0xd15, 0xd4d, 0xd15,
0xd7c, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd0e, 0xd2f, 0xd4d, 0xd15, 0xd4d, 0xd15,
0xd7c, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c,
0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd0e, 0xd2f, 0xd4d, 0xd15, 0xd4d,
0xd15, 0xd7c, 0x20, 0xd35, 0xd47, 0xd28, 0xd7d, 0xd15, 0xd4d, 0xd15, 0xd3e, 0xd32,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd2b, 0xd4d, 0x200c, 0xd17, 0xd3e, 0xd28,
0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2e,
0xd27, 0xd4d, 0xd2f, 0x20, 0xd06, 0xd2b, 0xd4d, 0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15, 0xd7b,
0x20, 0xd06, 0xd2b, 0xd4d, 0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd26, 0xd15, 0xd4d, 0xd37, 0xd3f, 0xd23, 0xd3e, 0xd2b, 0xd4d, 0xd30,
0xd3f, 0xd15, 0xd4d, 0xd15, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b,
0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f,
0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20, 0xd06, 0xd2b, 0xd4d, 0xd30,
0xd3f, 0xd15, 0xd4d, 0xd15, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f,
0xd1e, 0xd4d, 0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20, 0xd06, 0xd2b, 0xd4d, 0xd30, 0xd3f,
0xd15, 0xd4d, 0xd15, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21,
0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f,
0xd1e, 0xd4d, 0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20, 0xd06, 0xd2b, 0xd4d, 0xd30, 0xd3f,
0xd15, 0xd4d, 0xd15, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e,
0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd32, 0xd3e, 0xd38,
0xd4d, 0x200c, 0xd15, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd32, 0xd3e, 0xd38,
0xd4d, 0xd15, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47,
0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd32, 0xd3e, 0xd38,
0xd4d, 0x200c, 0xd15, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd7d, 0xd2e, 0xd24, 0xd3f, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd05, 0xd7d, 0xd2e, 0xd24, 0xd3f, 0x20, 0xd38, 0xd4d, 0xd31,
0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd05, 0xd7d, 0xd2e, 0xd24, 0xd3f, 0x20, 0xd35, 0xd47, 0xd28, 0xd7d,
0xd15, 0xd4d, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd06, 0xd2e,
0xd38, 0xd4b, 0xd7a, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd06, 0xd2e, 0xd38, 0xd4b,
0xd7a, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c,
0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd06, 0xd2e, 0xd38, 0xd4b, 0xd7a,
0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd1f, 0xd15, 0xd4d, 0xd15, 0xd46, 0x20,
0xd05, 0xd2e, 0xd47, 0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd38, 0xd46,
0xd7b, 0xd1f, 0xd4d, 0xd30, 0xd7d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd1f,
0xd15, 0xd4d, 0xd15, 0xd46, 0x20, 0xd05, 0xd2e, 0xd47, 0xd30, 0xd3f, 0xd15, 0xd4d,
0xd15, 0xd7b, 0x20, 0xd38, 0xd46, 0xd7b, 0xd1f, 0xd4d, 0xd30, 0xd7d, 0x20, 0xd38,
0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd1f, 0xd15, 0xd4d, 0xd15, 0xd46, 0x20, 0xd05,
0xd2e, 0xd47, 0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd38, 0xd46, 0xd7b,
0xd1f, 0xd4d, 0xd30, 0xd7d, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd1f, 0xd15, 0xd4d, 0xd15, 0xd46,
0x20, 0xd05, 0xd2e, 0xd47, 0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd15,
0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35,
0xd1f, 0xd15, 0xd4d, 0xd15, 0xd46, 0x20, 0xd05, 0xd2e, 0xd47, 0xd30, 0xd3f, 0xd15,
0xd4d, 0xd15, 0xd7b, 0x20, 0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd1f, 0xd15, 0xd4d, 0xd15, 0xd46, 0x20,
0xd05, 0xd2e, 0xd47, 0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd15, 0xd3f,
0xd34, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d,
0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd1f, 0xd15, 0xd4d, 0xd15,
0xd46, 0x20, 0xd05, 0xd2e, 0xd47, 0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20,
0xd2e, 0xd4c, 0xd23, 0xd4d, 0xd1f, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35,
0xd1f, 0xd15, 0xd4d, 0xd15, 0xd46, 0x20, 0xd05, 0xd2e, 0xd47, 0xd30, 0xd3f, 0xd15,
0xd4d, 0xd15, 0xd7b, 0x20, 0xd2e, 0xd57, 0xd23, 0xd4d, 0xd1f, 0xd7b, 0x20, 0xd38,
0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd1f, 0xd15, 0xd4d, 0xd15, 0xd46, 0x20, 0xd05,
0xd2e, 0xd47, 0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd2e, 0xd57, 0xd23,
0xd4d, 0xd1f, 0xd7b, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd1f, 0xd15, 0xd4d, 0xd15, 0xd46, 0x20,
0xd05, 0xd2e, 0xd47, 0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd2a, 0xd38,
0xd2b, 0xd3f, 0xd15, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd1f, 0xd15,
0xd4d, 0xd15, 0xd46, 0x20, 0xd05, 0xd2e, 0xd47, 0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15,
0xd7b, 0x20, 0xd2a, 0xd38, 0xd2b, 0xd3f, 0xd15, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31,
0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd35, 0xd1f, 0xd15, 0xd4d, 0xd15, 0xd46, 0x20, 0xd05, 0xd2e, 0xd47,
0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd2a, 0xd38, 0xd2b, 0xd3f, 0xd15,
0xd4d, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd05, 0xd28, 0xd3e, 0xd21, 0xd3f, 0xd7c, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd05, 0xd28, 0xd3e, 0xd21, 0xd3f, 0xd7c, 0x20, 0xd38, 0xd4d, 0xd31,
0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd05, 0xd28, 0xd3e, 0xd21, 0xd3f, 0xd7c, 0x20, 0xd35, 0xd47, 0xd28,
0xd7d, 0xd15, 0xd4d, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05,
0xd2a, 0xd3f, 0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd2a, 0xd3f, 0xd2f,
0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd2a, 0xd3f, 0xd2f, 0x20, 0xd21,
0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd05, 0xd16, 0xd4d, 0xd24, 0xd4c, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd16,
0xd4d, 0xd24, 0xd4c, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21,
0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd16, 0xd4d,
0xd24, 0xd4c, 0x20, 0xd35, 0xd47, 0xd28, 0xd7d, 0xd15, 0xd4d, 0xd15, 0xd3e, 0xd32,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd16, 0xd4d, 0xd24, 0xd4b, 0xd2c, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd16, 0xd4d, 0xd24, 0xd4b, 0xd2c, 0xd4d,
0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd16, 0xd4d, 0xd24, 0xd4b, 0xd2c,
0xd4d, 0x20, 0xd35, 0xd47, 0xd28, 0xd7d, 0xd15, 0xd4d, 0xd15, 0xd3e, 0xd32, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd31, 0xd47, 0xd2c, 0xd4d, 0xd2f, 0xd7b, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd31, 0xd47, 0xd2c, 0xd4d, 0xd2f, 0xd7b, 0x20,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd31, 0xd47, 0xd2c, 0xd4d, 0xd2f, 0xd7b,
0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd05, 0xd7c, 0xd1c, 0xd28, 0xd4d, 0xd31, 0xd40, 0xd28, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd05, 0xd7c, 0xd1c, 0xd28, 0xd4d, 0xd31, 0xd40, 0xd28, 0x20,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd7c, 0xd1c, 0xd28, 0xd4d, 0xd31, 0xd40,
0xd28, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e,
0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e,
0xd3e, 0xd31, 0xd7b, 0x20, 0xd05, 0xd7c, 0xd1c, 0xd28, 0xd4d, 0xd31, 0xd40, 0xd28,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e,
0xd31, 0xd7b, 0x20, 0xd05, 0xd7c, 0xd1c, 0xd28, 0xd4d, 0xd31, 0xd40, 0xd28, 0x20,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e,
0xd31, 0xd7b, 0x20, 0xd05, 0xd7c, 0xd1c, 0xd28, 0xd4d, 0xd31, 0xd40, 0xd28, 0x20,
0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd7c, 0xd2e, 0xd47, 0xd28, 0xd3f, 0xd2f, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd7c, 0xd2e, 0xd47, 0xd28, 0xd3f, 0xd2f, 0x20,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd7c, 0xd2e, 0xd47, 0xd28, 0xd3f, 0xd2f,
0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x200c, 0xd32,
0xd3e, 0xd28, 0xd4d, 0xd31, 0xd3f, 0xd15, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd05, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x200c, 0xd32, 0xd3e, 0xd28, 0xd4d, 0xd31, 0xd3f,
0xd15, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47,
0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd31, 0xd4d, 0xd31,
0xd4d, 0x200c, 0xd32, 0xd3e, 0xd28, 0xd4d, 0xd31, 0xd3f, 0xd15, 0xd4d, 0x20, 0xd21,
0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd38, 0xd46, 0xd7b, 0xd1f, 0xd4d, 0xd30, 0xd7d, 0x20, 0xd13, 0xd38, 0xd4d, 0xd1f,
0xd4d, 0xd30, 0xd47, 0xd32, 0xd3f, 0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd13,
0xd38, 0xd4d, 0xd1f, 0xd4d, 0xd30, 0xd47, 0xd32, 0xd3f, 0xd2f, 0xd7b, 0x20, 0xd38,
0xd46, 0xd7b, 0xd1f, 0xd4d, 0xd30, 0xd7d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd13, 0xd38, 0xd4d, 0xd1f, 0xd4d, 0xd30, 0xd47, 0xd32, 0xd3f, 0xd2f, 0xd7b, 0x20,
0xd38, 0xd46, 0xd7b, 0xd1f, 0xd4d, 0xd30, 0xd7d, 0x20, 0xd21, 0xd47, 0xd32, 0xd48,
0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd13, 0xd38, 0xd4d,
0xd1f, 0xd4d, 0xd30, 0xd47, 0xd32, 0xd3f, 0xd2f, 0xd7b, 0x20, 0xd38, 0xd46, 0xd7b,
0xd1f, 0xd4d, 0xd30, 0xd7d, 0x20, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e,
0xd31, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd13, 0xd38, 0xd4d, 0xd1f, 0xd4d,
0xd30, 0xd47, 0xd32, 0xd3f, 0xd2f, 0xd7b, 0x20, 0xd38, 0xd46, 0xd7b, 0xd1f, 0xd4d,
0xd30, 0xd7d, 0x20, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e, 0xd31, 0xd7b,
0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd13, 0xd38, 0xd4d, 0xd1f, 0xd4d, 0xd30,
0xd47, 0xd32, 0xd3f, 0xd2f, 0xd7b, 0x20, 0xd38, 0xd46, 0xd7b, 0xd1f, 0xd4d, 0xd30,
0xd7d, 0x20, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20,
0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd13, 0xd38, 0xd4d,
0x200c, 0xd1f, 0xd4d, 0xd30, 0xd47, 0xd32, 0xd3f, 0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd13, 0xd38, 0xd4d, 0x200c, 0xd1f, 0xd4d, 0xd30, 0xd47, 0xd32, 0xd3f, 0xd2f,
0xd7b, 0x20, 0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd38, 0xd4d,
0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd13, 0xd38, 0xd4d, 0x200c, 0xd1f, 0xd4d, 0xd30, 0xd47, 0xd32,
0xd3f, 0xd2f, 0xd7b, 0x20, 0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20,
0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20, 0xd13,
0xd38, 0xd4d, 0x200c, 0xd1f, 0xd4d, 0xd30, 0xd47, 0xd32, 0xd3f, 0xd2f, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd13, 0xd38, 0xd4d, 0x200c, 0xd1f, 0xd4d, 0xd30, 0xd47, 0xd32,
0xd3f, 0xd2f, 0xd7b, 0x20, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e, 0xd31,
0xd7b, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c,
0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd13, 0xd38, 0xd4d, 0x200c, 0xd1f,
0xd4d, 0xd30, 0xd47, 0xd32, 0xd3f, 0xd2f, 0xd7b, 0x20, 0xd2a, 0xd1f, 0xd3f, 0xd1e,
0xd4d, 0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d,
0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd38, 0xd7c, 0xd2c, 0xd48,
0xd1c, 0xd3e, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd38, 0xd7c, 0xd2c,
0xd48, 0xd1c, 0xd3e, 0xd7b, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b,
0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd38,
0xd7c, 0xd2c, 0xd48, 0xd1c, 0xd3e, 0xd7b, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37,
0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05,
0xd38, 0xd4b, 0xd7c, 0xd38, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd38,
0xd4b, 0xd7c, 0xd38, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b,
0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd38,
0xd4b, 0xd7c, 0xd38, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c,
0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2c, 0xd02, 0xd17,
0xd4d, 0xd32, 0xd3e, 0xd26, 0xd47, 0xd36, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd2c, 0xd02, 0xd17, 0xd4d, 0xd32, 0xd3e, 0xd26, 0xd47, 0xd36, 0xd4d, 0x20, 0xd38,
0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2c, 0xd02, 0xd17, 0xd4d, 0xd32, 0xd3e, 0xd26, 0xd47,
0xd36, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15,
0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2d, 0xd42, 0xd1f, 0xd4d, 0xd1f,
0xd3e, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2c, 0xd4a, 0xd33, 0xd40, 0xd35,
0xd3f, 0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2c, 0xd4d, 0xd30, 0xd38, 0xd40,
0xd32, 0xd3f, 0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2c, 0xd4d, 0xd30, 0xd38,
0xd40, 0xd32, 0xd3f, 0xd2f, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b,
0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2c, 0xd4d,
0xd30, 0xd38, 0xd40, 0xd32, 0xd3f, 0xd2f, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37,
0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2c,
0xd4d, 0xd30, 0xd42, 0xd23, 0xd48, 0x20, 0xd26, 0xd3e, 0xd31, 0xd41, 0xd38, 0xd4d,
0xd38, 0xd32, 0xd3e, 0xd02, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd47, 0xd2a,
0xd4d, 0x20, 0xd35, 0xd46, 0xd7c, 0xd26, 0xd46, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd15, 0xd47, 0xd2a, 0xd4d, 0x20, 0xd35, 0xd46, 0xd7c, 0xd26, 0xd46, 0x20, 0xd38,
0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd47, 0xd2a, 0xd4d, 0x20, 0xd35, 0xd46, 0xd7c,
0xd26, 0xd46, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15,
0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1a, 0xd2e, 0xd4b, 0xd31, 0xd4b,
0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1a, 0xd3e, 0xd24, 0xd4d, 0xd24, 0xd02,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1a, 0xd3e, 0xd24, 0xd4d, 0xd24, 0xd02, 0x20,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1a, 0xd3e, 0xd24, 0xd4d, 0xd24, 0xd02, 0x20,
0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1a, 0xd3f, 0xd32, 0xd3f, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd1a, 0xd3f, 0xd32, 0xd3f, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e,
0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1a,
0xd3f, 0xd32, 0xd3f, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e,
0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1a, 0xd48, 0xd28, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1a, 0xd48, 0xd28, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d,
0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd1a, 0xd48, 0xd28, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4d, 0xd30, 0xd3f, 0xd38, 0xd4d,
0x200c, 0xd2e, 0xd38, 0xd4d, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4a, 0xd15, 0xd4d, 0xd15, 0xd4b, 0xd38, 0xd4d,
0x20, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd41, 0xd15, 0xd7e, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd15, 0xd4a, 0xd33, 0xd02, 0xd2c, 0xd3f, 0xd2f, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd15, 0xd4a, 0xd33, 0xd02, 0xd2c, 0xd3f, 0xd2f, 0x20, 0xd38, 0xd4d,
0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4a, 0xd33, 0xd02, 0xd2c, 0xd3f, 0xd2f, 0x20, 0xd17,
0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd15, 0xd41, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd26, 0xd4d,
0xd35, 0xd40, 0xd2a, 0xd41, 0xd15, 0xd7e, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15,
0xd41, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd41,
0xd15, 0xd7e, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47,
0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd41, 0xd15, 0xd4d,
0xd15, 0xd4d, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd41, 0xd15, 0xd7e, 0x20,
0xd05, 0xd7c, 0xd26, 0xd4d, 0xd27, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d,
0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4d,
0xd2f, 0xd42, 0xd2c, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4d, 0xd2f, 0xd42,
0xd2c, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c,
0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4d, 0xd2f, 0xd42, 0xd2c,
0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd21, 0xd47, 0xd35, 0xd3f, 0xd38, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd21, 0xd41, 0xd2e, 0xd4b, 0xd23, 0xd4d, 0xd1f, 0xd4d, 0x20, 0xd21, 0xd3f,
0x20, 0xd09, 0xd7c, 0xd35, 0xd3f, 0xd32, 0xd4d, 0xd32, 0xd46, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd24, 0xd3f,
0xd2e, 0xd4b, 0xd7c, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd08, 0xd38, 0xd4d, 0xd31,
0xd4d, 0xd31, 0xd7c, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd4d, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd08, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd7c, 0x20, 0xd26,
0xd4d, 0xd35, 0xd40, 0xd2a, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e,
0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd08,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd7c, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a,
0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e,
0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd07, 0xd15, 0xd4d, 0xd35, 0xd21, 0xd4b,
0xd7c, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd46, 0xd7b, 0xd1f, 0xd4d, 0xd30,
0xd7d, 0x20, 0xd2f, 0xd42, 0xd31, 0xd4b, 0xd2a, 0xd4d, 0xd2f, 0xd7b, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd38, 0xd46, 0xd7b, 0xd1f, 0xd4d, 0xd30, 0xd7d, 0x20, 0xd2f,
0xd42, 0xd31, 0xd4b, 0xd2a, 0xd4d, 0xd2f, 0xd7b, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d,
0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd38, 0xd46, 0xd7b, 0xd1f, 0xd4d, 0xd30, 0xd7d, 0x20, 0xd2f, 0xd42, 0xd31,
0xd4b, 0xd2a, 0xd4d, 0xd2f, 0xd7b, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d,
0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd3f, 0xd34,
0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd2f, 0xd42, 0xd31, 0xd4b, 0xd2a, 0xd4d, 0xd2f,
0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15,
0xd7b, 0x20, 0xd2f, 0xd42, 0xd31, 0xd4b, 0xd2a, 0xd4d, 0xd2f, 0xd7b, 0x20, 0xd38,
0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20,
0xd2f, 0xd42, 0xd31, 0xd4b, 0xd2a, 0xd4d, 0xd2f, 0xd7b, 0x20, 0xd17, 0xd4d, 0xd30,
0xd40, 0xd37, 0xd4d, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15, 0xd47, 0xd2f, 0xd31, 0xd4d, 0xd31, 0xd24,
0xd4d, 0xd24, 0xd46, 0x20, 0xd2f, 0xd42, 0xd31, 0xd4b, 0xd2a, 0xd4d, 0xd2f, 0xd7b,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e,
0xd31, 0xd7b, 0x20, 0xd2f, 0xd42, 0xd31, 0xd4b, 0xd2a, 0xd4d, 0xd2f, 0xd7b, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e, 0xd31,
0xd7b, 0x20, 0xd2f, 0xd42, 0xd31, 0xd4b, 0xd2a, 0xd4d, 0xd2f, 0xd7b, 0x20, 0xd38,
0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e, 0xd31,
0xd7b, 0x20, 0xd2f, 0xd42, 0xd31, 0xd4b, 0xd2a, 0xd4d, 0xd2f, 0xd7b, 0x20, 0xd17,
0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd2b, 0xd3e, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x200c, 0xd32, 0xd3e,
0xd7b, 0xd21, 0xd4d, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd41, 0xd15, 0xd7e,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2b, 0xd3e, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x200c,
0xd32, 0xd3e, 0xd7b, 0xd21, 0xd4d, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd41,
0xd15, 0xd7e, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47,
0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2b, 0xd3e, 0xd15, 0xd4d,
0xd15, 0xd4d, 0x200c, 0xd32, 0xd3e, 0xd7b, 0xd21, 0xd4d, 0x20, 0xd26, 0xd4d, 0xd35,
0xd40, 0xd2a, 0xd41, 0xd15, 0xd7e, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d,
0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2b, 0xd3f,
0xd1c, 0xd3f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2b, 0xd3f, 0xd1c, 0xd3f, 0x20,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2b, 0xd3f, 0xd1c, 0xd3f, 0x20, 0xd17, 0xd4d,
0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd2b, 0xd4d, 0xd30, 0xd1e, 0xd4d, 0xd1a, 0xd4d, 0x20, 0xd17, 0xd2f,
0xd3e, 0xd28, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2b, 0xd4d, 0xd30, 0xd1e, 0xd4d,
0xd1a, 0xd4d, 0x20, 0xd38, 0xd24, 0xd47, 0xd7a, 0x2c, 0x20, 0xd05, 0xd28, 0xd4d,
0xd31, 0xd3e, 0xd7c, 0xd1f, 0xd4d, 0xd1f, 0xd3f, 0xd15, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd17, 0xd3e, 0xd32, 0xd2a, 0xd4d, 0xd2a, 0xd17, 0xd4b, 0xd38, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd17, 0xd3e, 0xd2e, 0xd4d, 0xd2a, 0xd3f, 0xd2f,
0xd7c, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1c, 0xd4b, 0xd7c, 0xd1c, 0xd4d, 0xd1c,
0xd3f, 0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1c, 0xd4b, 0xd7c, 0xd1c, 0xd4d,
0xd1c, 0xd3f, 0xd2f, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21,
0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1c, 0xd4b, 0xd7c,
0xd1c, 0xd4d, 0xd1c, 0xd3f, 0xd2f, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d,
0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd17, 0xd3f,
0xd7d, 0xd2c, 0xd47, 0xd7c, 0xd1f, 0xd4d, 0xd1f, 0xd4d, 0x20, 0xd26, 0xd4d, 0xd35,
0xd40, 0xd2a, 0xd41, 0xd15, 0xd7e, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd17, 0xd4d,
0xd30, 0xd40, 0xd7b, 0xd35, 0xd3f, 0xd1a, 0xd4d, 0xd1a, 0xd4d, 0x20, 0xd2e, 0xd40,
0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15,
0xd7b, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd7b, 0xd32, 0xd3e, 0xd7b, 0xd21, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15, 0xd7b,
0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd7b, 0xd32, 0xd3e, 0xd7b, 0xd21, 0xd4d, 0x20,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15, 0xd7b,
0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd7b, 0xd32, 0xd3e, 0xd7b, 0xd21, 0xd4d, 0x20,
0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e, 0xd31,
0xd7b, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd7b, 0xd32, 0xd3e, 0xd7b, 0xd21, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e,
0xd31, 0xd7b, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd7b, 0xd32, 0xd3e, 0xd7b, 0xd21,
0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c,
0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d,
0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd7b, 0xd32, 0xd3e,
0xd7b, 0xd21, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e,
0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd17, 0xd4d, 0xd35, 0xd3e,
0xd02, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c,
0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd17, 0xd7e, 0xd2b, 0xd4d, 0x20,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd39, 0xd35, 0xd3e, 0xd2f, 0xd4d, 0x2d, 0xd05,
0xd32, 0xd42, 0xd37, 0xd4d, 0xd2f, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd39,
0xd35, 0xd3e, 0xd2f, 0xd4d, 0x2d, 0xd05, 0xd32, 0xd42, 0xd37, 0xd4d, 0xd2f, 0xd7b,
0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd39, 0xd35, 0xd3e, 0xd2f, 0xd4d, 0x2d,
0xd05, 0xd32, 0xd42, 0xd37, 0xd4d, 0xd2f, 0xd7b, 0x20, 0xd21, 0xd47, 0xd32, 0xd48,
0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd39, 0xd4b, 0xd19,
0xd4d, 0xd15, 0xd4b, 0xd19, 0xd4d, 0xd19, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd39, 0xd4b, 0xd19, 0xd4d, 0xd15, 0xd4b, 0xd19, 0xd4d, 0xd19, 0xd4d, 0x20, 0xd38,
0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd39, 0xd4b, 0xd19, 0xd4d, 0xd15, 0xd4b, 0xd19, 0xd4d,
0xd19, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15,
0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd39, 0xd4b, 0xd21, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd39, 0xd4b, 0xd21, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31,
0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd39, 0xd4b, 0xd21, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37,
0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd07,
0xd28, 0xd4d, 0xd24, 0xd4d, 0xd2f, 0xd7b, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd07, 0xd28, 0xd4d, 0xd24, 0xd4d, 0xd2f, 0xd7b, 0x20, 0xd2e, 0xd39, 0xd3e, 0xd38,
0xd2e, 0xd41, 0xd26, 0xd4d, 0xd30, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd07, 0xd7b,
0xd21, 0xd4b, 0xd1a, 0xd48, 0xd28, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd27,
0xd4d, 0xd2f, 0x20, 0xd07, 0xd28, 0xd4d, 0xd24, 0xd4b, 0xd28, 0xd47, 0xd37, 0xd4d,
0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15,
0xd7b, 0x20, 0xd07, 0xd28, 0xd4d, 0xd24, 0xd4b, 0xd28, 0xd47, 0xd37, 0xd4d, 0xd2f,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e,
0xd31, 0xd7b, 0x20, 0xd07, 0xd28, 0xd4d, 0xd24, 0xd4b, 0xd28, 0xd47, 0xd37, 0xd4d,
0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd07, 0xd31, 0xd3e, 0xd7b, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd07, 0xd31, 0xd3e, 0xd7b, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d,
0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd07, 0xd31, 0xd3e, 0xd7b, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d,
0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd07, 0xd7c, 0xd15, 0xd38, 0xd4d,
0xd15, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd07, 0xd7c, 0xd15, 0xd38, 0xd4d,
0xd15, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47,
0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd07, 0xd7c, 0xd15, 0xd38,
0xd4d, 0x200c, 0xd15, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c,
0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd07, 0xd38, 0xd4d,
0xd30, 0xd3e, 0xd2f, 0xd47, 0xd7d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd07, 0xd38,
0xd4d, 0xd30, 0xd3e, 0xd2f, 0xd47, 0xd7d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd07, 0xd38, 0xd4d, 0xd30, 0xd3e, 0xd2f, 0xd47, 0xd7d, 0x20, 0xd21, 0xd47, 0xd32,
0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1c, 0xd2a,
0xd4d, 0xd2a, 0xd3e, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1c, 0xd2a, 0xd4d,
0xd2a, 0xd3e, 0xd7b, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21,
0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1c, 0xd2a, 0xd4d,
0xd2a, 0xd3e, 0xd7b, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd46, 0xd1f, 0xd4d, 0xd30, 0xd4b, 0xd2a,
0xd3e, 0xd35, 0xd4d, 0x200c, 0xd32, 0xd4b, 0xd38, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd15,
0xd02, 0xd1a, 0xd3e, 0xd38, 0xd4d, 0xd15, 0xd3f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd2a, 0xd46, 0xd1f, 0xd4d, 0xd30, 0xd4b, 0xd2a, 0xd3e, 0xd35, 0xd4d, 0x200c, 0xd32,
0xd4b, 0xd38, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd15, 0xd02, 0xd1a, 0xd3e, 0xd38, 0xd4d,
0xd15, 0xd3f, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47,
0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd46, 0xd1f, 0xd4d,
0xd30, 0xd4b, 0xd2a, 0xd3e, 0xd35, 0xd4d, 0x200c, 0xd32, 0xd4b, 0xd38, 0xd4d, 0xd15,
0xd4d, 0x20, 0xd15, 0xd02, 0xd1a, 0xd3e, 0xd38, 0xd4d, 0xd15, 0xd3f, 0x20, 0xd35,
0xd47, 0xd28, 0xd7d, 0xd15, 0xd4d, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd15, 0xd38, 0xd3e, 0xd16, 0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e, 0xd7b, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20,
0xd15, 0xd38, 0xd3e, 0xd16, 0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e, 0xd7b, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e, 0xd31, 0xd7b,
0x20, 0xd15, 0xd38, 0xd3e, 0xd16, 0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e, 0xd7b, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4a, 0xd31, 0xd3f, 0xd2f, 0xd7b, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4a, 0xd31, 0xd3f, 0xd2f, 0xd7b, 0x20, 0xd38, 0xd4d,
0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4a, 0xd31, 0xd3f, 0xd2f, 0xd7b, 0x20, 0xd21, 0xd47,
0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15,
0xd4a, 0xd38, 0xd4d, 0xd30, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4d, 0xd30,
0xd3e, 0xd38, 0xd4d, 0x200c, 0xd28, 0xd4b, 0xd2f, 0xd3e, 0xd7c, 0xd38, 0xd4d, 0x200c,
0xd15, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4d, 0xd30, 0xd3e, 0xd38,
0xd4d, 0x200c, 0xd28, 0xd4b, 0xd2f, 0xd3e, 0xd7c, 0xd38, 0xd4d, 0x200c, 0xd15, 0xd4d,
0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4d, 0xd30, 0xd3e, 0xd38, 0xd4d,
0x200c, 0xd28, 0xd4b, 0xd2f, 0xd3e, 0xd7c, 0xd38, 0xd4d, 0x200c, 0xd15, 0xd4d, 0x20,
0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd3f, 0xd7c, 0xd17, 0xd3f, 0xd38, 0xd4d, 0xd25,
0xd3e, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd32, 0xd19, 0xd4d, 0xd15, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd32, 0xd48, 0xd7b, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40,
0xd2a, 0xd41, 0xd15, 0xd7e, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd32, 0xd4b, 0xd7c,
0xd21, 0xd4d, 0x20, 0xd39, 0xd4b, 0xd35, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd32, 0xd4b, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd39, 0xd4b, 0xd35, 0xd4d, 0x20, 0xd38,
0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd32, 0xd4b, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd39, 0xd4b,
0xd35, 0xd4d, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd15, 0xd4c, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd2e, 0xd15, 0xd4c, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21,
0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd15, 0xd4c,
0x20, 0xd35, 0xd47, 0xd28, 0xd7d, 0xd15, 0xd4d, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd17, 0xd3e, 0xd26, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd2e, 0xd17, 0xd3e, 0xd26, 0xd7b, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd2e, 0xd17, 0xd3e, 0xd26, 0xd7b, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d,
0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd32,
0xd47, 0xd37, 0xd4d, 0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd3e, 0xd32,
0xd3f, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd2e, 0xd7c, 0xd15, 0xd4d, 0xd15, 0xd38, 0xd38, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd2e, 0xd3e, 0xd7c, 0xd37, 0xd7d, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a,
0xd41, 0xd15, 0xd7e, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd57, 0xd31, 0xd40,
0xd37, 0xd4d, 0xd2f, 0xd38, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd57,
0xd31, 0xd40, 0xd37, 0xd4d, 0xd2f, 0xd38, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d,
0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd2e, 0xd57, 0xd31, 0xd40, 0xd37, 0xd4d, 0xd2f, 0xd38, 0xd4d, 0x20, 0xd17,
0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd3e, 0xd38, 0xd7a, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd2e, 0xd46, 0xd15, 0xd4d, 0xd38, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd2a,
0xd38, 0xd2b, 0xd3f, 0xd15, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd46,
0xd15, 0xd4d, 0x200c, 0xd38, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd2a, 0xd38,
0xd2b, 0xd3f, 0xd15, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b,
0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd46,
0xd15, 0xd4d, 0xd38, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd2a, 0xd38, 0xd2b,
0xd3f, 0xd15, 0xd4d, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd09, 0xd32, 0xd3e, 0xd7b, 0xd2c, 0xd3e, 0xd24,
0xd4d, 0xd24, 0xd7c, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd09, 0xd32, 0xd3e, 0xd7b,
0xd2c, 0xd3e, 0xd24, 0xd4d, 0xd24, 0xd7c, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd09, 0xd32, 0xd3e, 0xd7b, 0xd2c, 0xd3e, 0xd24, 0xd4d, 0xd24, 0xd7c, 0x20, 0xd17,
0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd4b, 0xd38, 0xd4d, 0xd15, 0xd4b, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd2e, 0xd4b, 0xd38, 0xd4d, 0xd15, 0xd4b, 0x20, 0xd38, 0xd4d, 0xd31,
0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd2e, 0xd4b, 0xd38, 0xd4d, 0x200c, 0xd15, 0xd4b, 0x20, 0xd17, 0xd4d,
0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd2e, 0xd4d, 0xd2f, 0xd3e, 0xd7b, 0xd2e, 0xd3e, 0xd7c, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd28, 0xd57, 0xd31, 0xd41, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd28, 0xd47, 0xd2a, 0xd4d, 0xd2a, 0xd3e, 0xd7e, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd28, 0xd4d, 0xd2f, 0xd42, 0x20, 0xd15, 0xd3e, 0xd32, 0xd3f, 0xd21, 0xd4b, 0xd23,
0xd3f, 0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4d, 0xd2f, 0xd42, 0x20,
0xd15, 0xd3e, 0xd32, 0xd3f, 0xd21, 0xd4b, 0xd23, 0xd3f, 0xd2f, 0x20, 0xd38, 0xd4d,
0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4d, 0xd2f, 0xd42, 0x20, 0xd15, 0xd3e, 0xd32, 0xd3f,
0xd21, 0xd4b, 0xd23, 0xd3f, 0xd2f, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d,
0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4d,
0xd2f, 0xd42, 0xd38, 0xd3f, 0xd32, 0xd3e, 0xd7b, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd28, 0xd4d, 0xd2f, 0xd42, 0xd38, 0xd3f, 0xd32, 0xd3e, 0xd7b, 0xd21,
0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c,
0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4d, 0xd2f, 0xd42, 0xd38,
0xd3f, 0xd32, 0xd3e, 0xd7b, 0xd21, 0xd4d, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31,
0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4d, 0xd2f, 0xd42,
0xd2b, 0xd57, 0xd23, 0xd4d, 0xd1f, 0xd4d, 0x200c, 0xd32, 0xd3e, 0xd28, 0xd4d, 0xd31,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4d, 0xd2f, 0xd42, 0xd2b, 0xd57,
0xd23, 0xd4d, 0xd1f, 0xd4d, 0x200c, 0xd32, 0xd3e, 0xd28, 0xd4d, 0xd31, 0xd4d, 0x20,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4d, 0xd2f, 0xd42, 0xd2b, 0xd57, 0xd23,
0xd4d, 0xd1f, 0xd4d, 0x200c, 0xd32, 0xd3e, 0xd28, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd21,
0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd28, 0xd4d, 0xd2f, 0xd42, 0xd2f, 0xd3f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28,
0xd4b, 0xd7c, 0x200c, 0xd2b, 0xd4b, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd26, 0xd4d,
0xd35, 0xd40, 0xd2a, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4b, 0xd7c,
0x200c, 0xd2b, 0xd4b, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40,
0xd2a, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47,
0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4b, 0xd7c, 0x200c,
0xd2b, 0xd4b, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a,
0xd4d, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd2b, 0xd46, 0xd7c, 0xd23, 0xd3e, 0xd21, 0xd4b, 0x20, 0xd21,
0xd3f, 0x20, 0xd28, 0xd4a, 0xd31, 0xd4b, 0xd7b, 0xd39, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd2b, 0xd46, 0xd7c, 0xd23, 0xd3e, 0xd21, 0xd4b, 0x20, 0xd21, 0xd3f, 0x20,
0xd28, 0xd4a, 0xd31, 0xd4b, 0xd7b, 0xd39, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd2b, 0xd46, 0xd7c, 0xd23, 0xd3e, 0xd21, 0xd4b, 0x20, 0xd21, 0xd3f, 0x20, 0xd28,
0xd4a, 0xd31, 0xd4b, 0xd7b, 0xd39, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d,
0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4b,
0xd7c, 0xd24, 0xd4d, 0xd24, 0xd4d, 0x20, 0xd2e, 0xd31, 0xd3f, 0xd2f, 0xd3e, 0xd28,
0xd3e, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd41, 0xd15, 0xd7e, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4b, 0xd35, 0xd4b, 0xd38, 0xd3f, 0xd2c, 0xd3f, 0xd7c,
0xd38, 0xd4d, 0x200c, 0xd15, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4b,
0xd35, 0xd4b, 0xd38, 0xd3f, 0xd2c, 0xd3f, 0xd7c, 0xd38, 0xd4d, 0xd15, 0xd4d, 0x20,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4b, 0xd35, 0xd4b, 0xd38, 0xd3f, 0xd2c,
0xd3f, 0xd7c, 0xd38, 0xd4d, 0x200c, 0xd15, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40,
0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd13, 0xd02, 0xd38, 0xd4d, 0x200c, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd13, 0xd02, 0xd38, 0xd4d, 0x200c, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x20,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd13, 0xd02, 0xd38, 0xd4d, 0x200c, 0xd15, 0xd4d,
0xd15, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15,
0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd3e, 0xd15, 0xd4d, 0xd15,
0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a,
0xd3e, 0xd15, 0xd4d, 0xd15, 0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e, 0xd7b, 0x20, 0xd38,
0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd3e, 0xd15, 0xd4d, 0xd15, 0xd3f, 0xd38, 0xd4d,
0xd25, 0xd3e, 0xd7b, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e,
0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd32, 0xd3e, 0xd35,
0xd41, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd3e, 0xd2a, 0xd4d, 0xd2a, 0xd41,
0xd35, 0x20, 0xd28, 0xd4d, 0xd2f, 0xd42, 0x20, 0xd17, 0xd3f, 0xd28, 0xd3f, 0xd2f,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd30, 0xd3e, 0xd17, 0xd4d, 0xd35, 0xd47,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd30, 0xd3e, 0xd17, 0xd4d, 0xd35, 0xd47,
0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd30, 0xd3e, 0xd17, 0xd4d, 0xd35,
0xd47, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e,
0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd46, 0xd31, 0xd41, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd46, 0xd31, 0xd41, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d,
0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd2a, 0xd46, 0xd31, 0xd41, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d,
0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2b, 0xd3f,
0xd32, 0xd3f, 0xd2a, 0xd4d, 0xd2a, 0xd48, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd2b, 0xd3f, 0xd32, 0xd3f, 0xd2a, 0xd4d, 0xd2a, 0xd48, 0xd7b, 0x20, 0xd38, 0xd4d,
0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd2b, 0xd3f, 0xd32, 0xd3f, 0xd2a, 0xd4d, 0xd2a, 0xd48, 0xd7b,
0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2b, 0xd3f, 0xd28, 0xd3f, 0xd15, 0xd4d, 0x200c,
0xd38, 0xd4d, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd38, 0xd46, 0xd28, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd2a, 0xd3f, 0xd2f,
0xd31, 0xd3f, 0x20, 0xd06, 0xd7b, 0xd21, 0xd4d, 0x20, 0xd2e, 0xd3f, 0xd15, 0xd4d,
0xd35, 0xd32, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd46, 0xd28, 0xd4d,
0xd31, 0xd4d, 0x20, 0xd2a, 0xd3f, 0xd2f, 0xd31, 0xd3f, 0x20, 0xd06, 0xd7b, 0xd21,
0xd4d, 0x20, 0xd2e, 0xd3f, 0xd15, 0xd4d, 0xd35, 0xd32, 0xd7b, 0x20, 0xd38, 0xd4d,
0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd38, 0xd46, 0xd28, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd2a, 0xd3f,
0xd2f, 0xd31, 0xd3f, 0x20, 0xd06, 0xd7b, 0xd21, 0xd4d, 0x20, 0xd2e, 0xd3f, 0xd15,
0xd4d, 0xd35, 0xd32, 0xd7b, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd3f, 0xd31, 0xd4d, 0xd31, 0xd4d,
0xd15, 0xd47, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd4a, 0xd28, 0xd3e,
0xd2a, 0xd4d, 0xd2a, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd4d, 0xd2f,
0xd4b, 0xd02, 0xd17, 0xd4d, 0x200c, 0xd2f, 0xd3e, 0xd02, 0xd17, 0xd4d, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd16, 0xd3f, 0xd38, 0xd3f, 0xd32, 0xd4b, 0xd7c, 0xd21, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd16, 0xd3f, 0xd38, 0xd3f, 0xd32, 0xd4b, 0xd7c, 0xd21,
0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd16, 0xd3f, 0xd38, 0xd3f, 0xd32, 0xd4b,
0xd7c, 0xd21, 0x20, 0xd35, 0xd47, 0xd28, 0xd7d, 0xd15, 0xd4d, 0xd15, 0xd3e, 0xd32,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd31, 0xd40, 0xd2f, 0xd42, 0xd23, 0xd3f, 0xd2f,
0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd31, 0xd4b, 0xd25, 0xd46, 0xd31, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd16, 0xd3e, 0xd32, 0xd3f, 0xd7b, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd38, 0xd16, 0xd3e, 0xd32, 0xd3f, 0xd7b, 0x20, 0xd38, 0xd4d,
0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd38, 0xd16, 0xd3e, 0xd32, 0xd3f, 0xd7b, 0x20, 0xd17, 0xd4d,
0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd38, 0xd2e, 0xd3e, 0xd30, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38,
0xd2e, 0xd3e, 0xd31, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21,
0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd2e, 0xd3e,
0xd31, 0x20, 0xd35, 0xd47, 0xd28, 0xd7d, 0xd15, 0xd4d, 0xd15, 0xd3e, 0xd32, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd2e, 0xd4b, 0xd35, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd38, 0xd2e, 0xd4b, 0xd35, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e,
0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38,
0xd2e, 0xd4b, 0xd35, 0xd3e, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c,
0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd40, 0xd37,
0xd46, 0xd7d, 0xd38, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd3f, 0xd02,
0xd17, 0xd2a, 0xd4d, 0xd2a, 0xd42, 0xd7c, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd38, 0xd4b, 0xd33, 0xd2e, 0xd7b, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd41,
0xd15, 0xd7e, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd26, 0xd15, 0xd4d, 0xd37, 0xd3f,
0xd23, 0x20, 0xd1c, 0xd4b, 0xd7c, 0xd1c, 0xd4d, 0xd1c, 0xd3f, 0xd2f, 0xd7b, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd41, 0xd30, 0xd3f, 0xd28, 0xd46, 0xd2f, 0xd3f,
0xd02, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd2f, 0xd4b, 0xd35, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd24, 0xd3e, 0xd39, 0xd3f, 0xd24, 0xd3f, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd24, 0xd3e, 0xd2f, 0xd4d, 0x200c, 0xd2a, 0xd46, 0xd2f, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd24, 0xd3e, 0xd2f, 0xd4d, 0x200c, 0xd2a, 0xd46, 0xd2f,
0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c,
0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd24, 0xd3e, 0xd2f, 0xd4d, 0x200c,
0xd2a, 0xd46, 0xd2f, 0xd4d, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd24, 0xd3e, 0xd1c, 0xd3f, 0xd15, 0xd4d,
0xd15, 0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd1f, 0xd4b, 0xd15, 0xd4d, 0xd15, 0xd46, 0xd32, 0xd42, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd1f, 0xd4b, 0xd02, 0xd17, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1f, 0xd4b,
0xd02, 0xd17, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47,
0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1f, 0xd4b, 0xd02, 0xd17,
0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1a, 0xd42, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd24, 0xd41, 0xd7c, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x200c,
0xd2e, 0xd46, 0xd28, 0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e, 0xd7b, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd24, 0xd41, 0xd7c, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x200c, 0xd2e, 0xd46,
0xd28, 0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e, 0xd7b, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d,
0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd24, 0xd41, 0xd7c, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x200c, 0xd2e, 0xd46, 0xd28,
0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e, 0xd7b, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37,
0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1f,
0xd41, 0xd35, 0xd3e, 0xd32, 0xd41, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd09, 0xd31,
0xd41, 0xd17, 0xd4d, 0xd35, 0xd47, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd09, 0xd31,
0xd41, 0xd17, 0xd4d, 0xd35, 0xd47, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e,
0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd09,
0xd31, 0xd41, 0xd17, 0xd4d, 0xd35, 0xd47, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37,
0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd09,
0xd38, 0xd4d, 0x200c, 0xd2c, 0xd46, 0xd15, 0xd4d, 0xd15, 0xd3f, 0xd38, 0xd4d, 0xd25,
0xd3e, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd09, 0xd38, 0xd4d, 0x200c, 0xd2c,
0xd46, 0xd15, 0xd4d, 0xd15, 0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e, 0xd7b, 0x20, 0xd38,
0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd09, 0xd38, 0xd4d, 0x200c, 0xd2c, 0xd46, 0xd15, 0xd4d,
0xd15, 0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e, 0xd7b, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40,
0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd35, 0xd28, 0xd4d, 0xd35, 0xd3e, 0xd24, 0xd41, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd35, 0xd28, 0xd4d, 0xd35, 0xd3e, 0xd24, 0xd41, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d,
0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd35, 0xd28, 0xd4d, 0xd35, 0xd3e, 0xd24, 0xd41, 0x20, 0xd17, 0xd4d, 0xd30,
0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd35, 0xd46, 0xd28, 0xd3f, 0xd38, 0xd4d, 0xd35, 0xd47, 0xd32, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd35, 0xd4d, 0xd32, 0xd3e, 0xd21, 0xd3f, 0xd35, 0xd4b, 0xd38,
0xd4d, 0xd31, 0xd4d, 0xd31, 0xd4b, 0xd15, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd35, 0xd4d, 0xd32, 0xd3e, 0xd21, 0xd3f, 0xd35, 0xd4b, 0xd38, 0xd4d, 0xd31, 0xd4d,
0xd31, 0xd4b, 0xd15, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b,
0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd4d,
0xd32, 0xd3e, 0xd21, 0xd3f, 0xd35, 0xd4b, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd4b,
0xd15, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15,
0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd4b, 0xd7e, 0xd17, 0xd4b,
0xd17, 0xd4d, 0xd30, 0xd3e, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35,
0xd4b, 0xd7e, 0xd17, 0xd4b, 0xd17, 0xd4d, 0xd30, 0xd3e, 0xd21, 0xd4d, 0x20, 0xd38,
0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd4b, 0xd7e, 0xd17, 0xd4b, 0xd17, 0xd4d, 0xd30,
0xd3e, 0xd21, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e,
0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd47, 0xd15, 0xd4d,
0xd15, 0xd4d, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd35, 0xd3e, 0xd32, 0xd3f, 0xd38, 0xd4d, 0x20, 0xd06, 0xd28, 0xd4d,
0xd31, 0xd4d, 0x20, 0xd2b, 0xd4d, 0xd2f, 0xd42, 0xd1a, 0xd4d, 0xd2f, 0xd41, 0xd28,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2f, 0xd3e, 0xd15, 0xd38, 0xd4d, 0x200c, 0xd15,
0xd4d, 0xd15, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2f, 0xd3e, 0xd15, 0xd38,
0xd4d, 0x200c, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd2f, 0xd3e, 0xd15, 0xd38, 0xd4d, 0x200c, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd17,
0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd2f, 0xd46, 0xd15, 0xd4d, 0xd15, 0xd3e, 0xd31, 0xd4d, 0xd31,
0xd30, 0xd3f, 0xd7b, 0xd2c, 0xd7c, 0xd17, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd2f, 0xd46, 0xd15, 0xd4d, 0xd15, 0xd3e, 0xd31, 0xd4d, 0xd31, 0xd30, 0xd3f, 0xd7b,
0xd2c, 0xd7c, 0xd17, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b,
0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2f, 0xd46,
0xd15, 0xd4d, 0xd15, 0xd3e, 0xd31, 0xd4d, 0xd31, 0xd30, 0xd3f, 0xd7b, 0xd2c, 0xd7c,
0xd17, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15,
0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2f, 0xd42, 0xd15, 0xd4d, 0xd15,
0xd4b, 0xd7a, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x63, 0x72, 0x65,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x66, 0x67, 0x68, 0x61, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50,
0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x53, 0x65, 0x6c, 0x61, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69,
0x61, 0x6e, 0x67, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41,
0x6c, 0x6d, 0x61, 0x74, 0x79, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50,
0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x75, 0x73, 0x61, 0x74, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20,
0x50, 0x75, 0x73, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x69, 0x61, 0x6e, 0x67, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20,
0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x69, 0x61, 0x6e, 0x67, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x50, 0x65, 0x72, 0x67, 0x75, 0x6e, 0x75, 0x6e,
0x67, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61,
0x77, 0x61, 0x69, 0x20, 0x50, 0x65, 0x72, 0x67, 0x75, 0x6e, 0x75, 0x6e,
0x67, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61,
0x6e, 0x67, 0x20, 0x50, 0x65, 0x72, 0x67, 0x75, 0x6e, 0x75, 0x6e, 0x67,
0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77,
0x61, 0x69, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x50, 0x61,
0x73, 0x69, 0x66, 0x69, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50,
0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69,
0x20, 0x41, 0x70, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x69, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x70, 0x69, 0x61, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x41, 0x71, 0x74, 0x61, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50,
0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69,
0x20, 0x41, 0x72, 0x61, 0x62, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x69, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x72, 0x61, 0x62, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x42,
0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69,
0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61,
0x73, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20,
0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50,
0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61,
0x69, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x54, 0x65,
0x6e, 0x67, 0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69,
0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61,
0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61,
0x69, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x20, 0x54, 0x65, 0x6e, 0x67,
0x61, 0x68, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20,
0x42, 0x61, 0x72, 0x61, 0x74, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x54, 0x69, 0x6d, 0x75,
0x72, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x54, 0x69, 0x6d,
0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x42, 0x61,
0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61,
0x6e, 0x67, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62,
0x61, 0x69, 0x6a, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50,
0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69,
0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69,
0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61,
0x69, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61,
0x6e, 0x61, 0x73, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69,
0x20, 0x54, 0x61, 0x6e, 0x6a, 0x75, 0x6e, 0x67, 0x20, 0x56, 0x65, 0x72,
0x64, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77,
0x61, 0x69, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x43, 0x68, 0x69, 0x6c,
0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61,
0x69, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x43, 0x68, 0x69, 0x6e, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50,
0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x43,
0x68, 0x69, 0x6e, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x75,
0x6c, 0x61, 0x75, 0x20, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61,
0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d,
0x62, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61,
0x77, 0x61, 0x69, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20,
0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x70, 0x75,
0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4b,
0x65, 0x70, 0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20, 0x43, 0x6f, 0x6f,
0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d,
0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x53, 0x65, 0x70, 0x61, 0x72,
0x75, 0x68, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e,
0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x43,
0x75, 0x62, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61,
0x77, 0x61, 0x69, 0x20, 0x43, 0x75, 0x62, 0x61, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x43, 0x75, 0x62, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20,
0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50,
0x75, 0x6c, 0x61, 0x75, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20,
0x50, 0x75, 0x6c, 0x61, 0x75, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20,
0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x50, 0x75, 0x6c, 0x61, 0x75, 0x20,
0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x68, 0x20, 0x54, 0x65, 0x6e, 0x67,
0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77,
0x61, 0x69, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x68, 0x20, 0x54, 0x65,
0x6e, 0x67, 0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75,
0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x45, 0x72,
0x6f, 0x70, 0x61, 0x68, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x68, 0x20,
0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50,
0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x68,
0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20,
0x45, 0x72, 0x6f, 0x70, 0x61, 0x68, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x68,
0x20, 0x63, 0x65, 0x72, 0x75, 0x6b, 0x20, 0x74, 0x69, 0x6d, 0x75, 0x72,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x68,
0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61,
0x68, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x68, 0x20, 0x42, 0x61, 0x72, 0x61,
0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61,
0x69, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x46, 0x69, 0x6a,
0x69, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e,
0x61, 0x20, 0x50, 0x65, 0x72, 0x61, 0x6e, 0x63, 0x69, 0x73, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x50, 0x65, 0x72, 0x61, 0x6e, 0x63, 0x69, 0x73,
0x20, 0x53, 0x65, 0x6c, 0x61, 0x74, 0x61, 0x6e, 0x20, 0x64, 0x61, 0x6e,
0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x74, 0x69, 0x6b, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4b, 0x65, 0x70, 0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20, 0x47, 0x69,
0x6c, 0x62, 0x65, 0x72, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d,
0x69, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69,
0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54,
0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69,
0x61, 0x77, 0x61, 0x69, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x54, 0x65, 0x6c,
0x75, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77,
0x61, 0x69, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x48, 0x6f,
0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x48, 0x6f, 0x76, 0x64,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4c, 0x61, 0x75, 0x74, 0x61, 0x6e, 0x20, 0x48, 0x69, 0x6e, 0x64, 0x69,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69,
0x20, 0x49, 0x72, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x69, 0x61, 0x6e, 0x67, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50,
0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20,
0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4a, 0x65, 0x70, 0x75, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50,
0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4a, 0x65, 0x70, 0x75, 0x6e, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x4a,
0x65, 0x70, 0x75, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x65,
0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d,
0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x50,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20,
0x4b, 0x6f, 0x72, 0x65, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x69, 0x61, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4b, 0x79, 0x72, 0x67, 0x79, 0x73, 0x74, 0x61,
0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c,
0x61, 0x75, 0x61, 0x6e, 0x20, 0x4c, 0x69, 0x6e, 0x65, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x61,
0x63, 0x61, 0x6f, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61,
0x77, 0x61, 0x69, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e,
0x61, 0x73, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x61,
0x6c, 0x64, 0x69, 0x76, 0x65, 0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4b, 0x65, 0x70, 0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20, 0x4d, 0x61,
0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x74, 0x69, 0x75, 0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x61,
0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69,
0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x4d, 0x65, 0x78,
0x69, 0x63, 0x6f, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61,
0x6e, 0x67, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x4d,
0x65, 0x78, 0x69, 0x63, 0x6f, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50,
0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42,
0x61, 0x74, 0x6f, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x6f,
0x73, 0x63, 0x6f, 0x77, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69,
0x61, 0x77, 0x61, 0x69, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50,
0x61, 0x6e, 0x61, 0x73, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61,
0x73, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4e, 0x65, 0x77,
0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4e, 0x65, 0x77,
0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x4e, 0x65, 0x77, 0x20,
0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4e, 0x65, 0x77, 0x66,
0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x4e, 0x65, 0x77, 0x66,
0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4b, 0x65, 0x70, 0x75,
0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67,
0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x70, 0x75,
0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x6e,
0x61, 0x20, 0x55, 0x74, 0x61, 0x72, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4f, 0x6d, 0x73, 0x6b,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69,
0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x65, 0x77,
0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x79, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69,
0x61, 0x77, 0x61, 0x69, 0x20, 0x50, 0x65, 0x72, 0x75, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74,
0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x64, 0x61, 0x6e, 0x20,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x53, 0x61, 0x69, 0x6e,
0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x64, 0x61, 0x6e,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x51, 0x79,
0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69,
0x61, 0x77, 0x61, 0x69, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50,
0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70,
0x75, 0x72, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61,
0x77, 0x61, 0x69, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x54, 0x61,
0x69, 0x70, 0x65, 0x69, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69,
0x61, 0x77, 0x61, 0x69, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x54,
0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20,
0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x55, 0x7a, 0x62, 0x65,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77,
0x61, 0x69, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77,
0x61, 0x69, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x75, 0x6c, 0x61, 0x75, 0x20,
0x57, 0x61, 0x6b, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69,
0x61, 0x77, 0x61, 0x69, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69,
0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x4d, 0x61, 0x73, 0x61, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e,
0x648, 0x642, 0x62a, 0x648, 0x20, 0x628, 0x631, 0x648, 0x646, 0x64a, 0x20, 0x62f,
0x627, 0x631, 0x627, 0x644, 0x633, 0x644, 0x627, 0x645, 0x648, 0x642, 0x62a, 0x648,
0x20, 0x6a4, 0x64a, 0x627, 0x648, 0x627, 0x64a, 0x20, 0x627, 0x64a, 0x646, 0x62f,
0x64a, 0x627, 0x648, 0x642, 0x62a, 0x648, 0x20, 0x644, 0x627, 0x674, 0x648, 0x62a,
0x646, 0x20, 0x647, 0x64a, 0x646, 0x62f, 0x64a, 0x648, 0x642, 0x62a, 0x648, 0x20,
0x625, 0x646, 0x62f, 0x648, 0x646, 0x64a, 0x633, 0x64a, 0x627, 0x20, 0x62a, 0x6a0,
0x647, 0x648, 0x642, 0x62a, 0x648, 0x20, 0x625, 0x646, 0x62f, 0x648, 0x646, 0x64a,
0x633, 0x64a, 0x627, 0x20, 0x62a, 0x64a, 0x645, 0x648, 0x631, 0x648, 0x642, 0x62a,
0x648, 0x20, 0x625, 0x646, 0x62f, 0x648, 0x646, 0x64a, 0x633, 0x64a, 0x627, 0x20,
0x628, 0x627, 0x631, 0x627, 0x62a, 0x648, 0x642, 0x62a, 0x648, 0x20, 0x645, 0x644,
0x64a, 0x633, 0x64a, 0x627, 0x126, 0x69, 0x6e, 0x20, 0x10a, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x65, 0x77,
0x126, 0x69, 0x6e, 0x20, 0x10a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x65, 0x77, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x126, 0x69, 0x6e, 0x20, 0x10a, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x65,
0x77, 0x20, 0x74, 0x61, 0x73, 0x2d, 0x53, 0x61, 0x6a, 0x66, 0x9a8, 0x9cb,
0x9b0, 0x9cd, 0x9a5, 0x20, 0x985, 0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x995, 0x9be, 0x9a8,
0x20, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x9a4, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x20, 0x99f,
0x9be, 0x987, 0x9ae, 0x9a8, 0x9cb, 0x9b0, 0x9cd, 0x9a5, 0x20, 0x985, 0x9ae, 0x9c7,
0x9b0, 0x9bf, 0x995, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x9a4, 0x9cd,
0x9b0, 0x9c7, 0x9b2, 0x20, 0x9b7, 0x9cd, 0x99f, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9b0,
0x9cd, 0x9a6, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9a8, 0x9cb, 0x9b0, 0x9cd, 0x9a5,
0x20, 0x985, 0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x995, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9c7,
0x9a8, 0x9cd, 0x9a4, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x20, 0x9a6, 0x9c7, 0x9b2, 0x9be,
0x987, 0x99f, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9a8, 0x9cb, 0x9b0, 0x9cd, 0x9a5,
0x20, 0x985, 0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x995, 0x9be, 0x9a8, 0x20, 0x987, 0x9b7,
0x9cd, 0x99f, 0x9b0, 0x9cd, 0x9a8, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9a8, 0x9cb,
0x9b0, 0x9cd, 0x9a5, 0x20, 0x985, 0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x995, 0x9be, 0x9a8,
0x20, 0x987, 0x9b7, 0x9cd, 0x99f, 0x9b0, 0x9cd, 0x9a8, 0x20, 0x9b7, 0x9cd, 0x99f,
0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9b0, 0x9cd, 0x9a6, 0x20, 0x99f, 0x9be, 0x987, 0x9ae,
0x9a8, 0x9cb, 0x9b0, 0x9cd, 0x9a5, 0x20, 0x985, 0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x995,
0x9be, 0x9a8, 0x20, 0x987, 0x9b7, 0x9cd, 0x99f, 0x9b0, 0x9cd, 0x9a8, 0x20, 0x9a6,
0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9a8, 0x9cb,
0x9b0, 0x9cd, 0x9a5, 0x20, 0x985, 0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x995, 0x9be, 0x9a8,
0x20, 0x9ae, 0x9be, 0x989, 0x9a8, 0x9cd, 0x99f, 0x9c7, 0x9a8, 0x20, 0x99f, 0x9be,
0x987, 0x9ae, 0x9a8, 0x9cb, 0x9b0, 0x9cd, 0x9a5, 0x20, 0x985, 0x9ae, 0x9c7, 0x9b0,
0x9bf, 0x995, 0x9be, 0x9a8, 0x20, 0x9ae, 0x9be, 0x989, 0x9a8, 0x9cd, 0x99f, 0x9c7,
0x9a8, 0x20, 0x9b7, 0x9cd, 0x99f, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9b0, 0x9cd, 0x9a6,
0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9a8, 0x9cb, 0x9b0, 0x9cd, 0x9a5, 0x20, 0x985,
0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x995, 0x9be, 0x9a8, 0x20, 0x9ae, 0x9be, 0x989, 0x9a8,
0x9cd, 0x99f, 0x9c7, 0x9a8, 0x20, 0x9a6, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20,
0x99f, 0x9be, 0x987, 0x9ae, 0x9a8, 0x9cb, 0x9b0, 0x9cd, 0x9a5, 0x20, 0x985, 0x9ae,
0x9c7, 0x9b0, 0x9bf, 0x995, 0x9be, 0x9a8, 0x20, 0x9aa, 0x9c7, 0x9b8, 0x9bf, 0x9ab,
0x9bf, 0x995, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9a8, 0x9cb, 0x9b0, 0x9cd, 0x9a5,
0x20, 0x985, 0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x995, 0x9be, 0x9a8, 0x20, 0x9aa, 0x9c7,
0x9b8, 0x9bf, 0x9ab, 0x9bf, 0x995, 0x20, 0x9b7, 0x9cd, 0x99f, 0x9c7, 0x9a8, 0x9cd,
0x9a6, 0x9b0, 0x9cd, 0x9a6, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9a8, 0x9cb, 0x9b0,
0x9cd, 0x9a5, 0x20, 0x985, 0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x995, 0x9be, 0x9a8, 0x20,
0x9aa, 0x9c7, 0x9b8, 0x9bf, 0x9ab, 0x9bf, 0x995, 0x20, 0x9a6, 0x9c7, 0x9b2, 0x9be,
0x987, 0x99f, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x985, 0x99f, 0x9b2, 0x9be, 0x9a8,
0x9cd, 0x99f, 0x9bf, 0x995, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x985, 0x99f, 0x9b2,
0x9be, 0x9a8, 0x9cd, 0x99f, 0x9bf, 0x995, 0x20, 0x9b7, 0x9cd, 0x99f, 0x9c7, 0x9a8,
0x9cd, 0x9a6, 0x9b0, 0x9cd, 0x9a6, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x985, 0x99f,
0x9b2, 0x9be, 0x9a8, 0x9cd, 0x99f, 0x9bf, 0x995, 0x20, 0x9a6, 0x9c7, 0x9b2, 0x9be,
0x987, 0x99f, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x9a4,
0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x20, 0x9af, 0x9bc, 0x9c1, 0x9b0, 0x9cb, 0x9aa, 0x9bf,
0x9af, 0x9bc, 0x9be, 0x9a8, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9b8, 0x9c7, 0x9a8,
0x9cd, 0x9a4, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x20, 0x9af, 0x9bc, 0x9c1, 0x9b0, 0x9cb,
0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x20, 0x9b7, 0x9cd, 0x99f, 0x9c7, 0x9a8,
0x9cd, 0x9a6, 0x9b0, 0x9cd, 0x9a6, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9b8, 0x9c7,
0x9a8, 0x9cd, 0x9a4, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x20, 0x9af, 0x9bc, 0x9c1, 0x9b0,
0x9cb, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9b0, 0x20,
0x99f, 0x9be, 0x987, 0x9ae, 0x987, 0x9b7, 0x9cd, 0x99f, 0x9b0, 0x9cd, 0x9a8, 0x20,
0x9af, 0x9bc, 0x9c1, 0x9b0, 0x9cb, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x20,
0x99f, 0x9be, 0x987, 0x9ae, 0x987, 0x9b7, 0x9cd, 0x99f, 0x9b0, 0x9cd, 0x9a8, 0x20,
0x9af, 0x9bc, 0x9c1, 0x9b0, 0x9cb, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x20,
0x9b7, 0x9cd, 0x99f, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9b0, 0x9cd, 0x9a6, 0x20, 0x99f,
0x9be, 0x987, 0x9ae, 0x987, 0x9b7, 0x9cd, 0x99f, 0x9b0, 0x9cd, 0x9a8, 0x20, 0x9af,
0x9bc, 0x9c1, 0x9b0, 0x9cb, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9b0, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9f1, 0x9c7, 0x9b7, 0x9cd, 0x99f,
0x9b0, 0x9cd, 0x9a8, 0x20, 0x9af, 0x9bc, 0x9c1, 0x9b0, 0x9cb, 0x9aa, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x9a8, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9f1, 0x9c7, 0x9b7, 0x9cd,
0x99f, 0x9b0, 0x9cd, 0x9a8, 0x20, 0x9af, 0x9bc, 0x9c1, 0x9b0, 0x9cb, 0x9aa, 0x9bf,
0x9af, 0x9bc, 0x9be, 0x9a8, 0x20, 0x9b7, 0x9cd, 0x99f, 0x9c7, 0x9a8, 0x9cd, 0x9a6,
0x9b0, 0x9cd, 0x9a6, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9f1, 0x9c7, 0x9b7, 0x9cd,
0x99f, 0x9b0, 0x9cd, 0x9a8, 0x20, 0x9af, 0x9bc, 0x9c1, 0x9b0, 0x9cb, 0x9aa, 0x9bf,
0x9af, 0x9bc, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9b0, 0x20, 0x99f, 0x9be, 0x987,
0x9ae, 0x997, 0x9cd, 0x9b0, 0x9bf, 0x9a8, 0x9f1, 0x9bf, 0x99a, 0x20, 0x9ae, 0x9bf,
0x9a8, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x57, 0x101, 0x20, 0x41, 0x77, 0x68,
0x65, 0x6b, 0x65, 0x6e, 0x65, 0x74, 0x101, 0x6e, 0x61, 0x57, 0x101, 0x20,
0x6f, 0x20, 0x54, 0x65, 0x20, 0x50, 0x75, 0x6b, 0x75, 0x20, 0x6f, 0x20,
0x100, 0x77, 0x68, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x101, 0x20, 0x6f,
0x20, 0x100, 0x77, 0x68, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x6b, 0x69,
0x20, 0x74, 0x65, 0x20, 0x72, 0x101, 0x77, 0x68, 0x69, 0x74, 0x69, 0x57,
0x101, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x20,
0x6f, 0x20, 0x100, 0x77, 0x68, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x6b,
0x69, 0x20, 0x74, 0x65, 0x20, 0x74, 0x6f, 0x6e, 0x67, 0x61, 0x57, 0x101,
0x20, 0x6f, 0x20, 0x100, 0x77, 0x68, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x75, 0x72, 0x75, 0x57, 0x101, 0x20,
0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x20, 0x6f, 0x20,
0x100, 0x77, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x6b, 0x69, 0x20, 0x74,
0x65, 0x20, 0x75, 0x72, 0x75, 0x57, 0x101, 0x20, 0x52, 0x61, 0x75, 0x6d,
0x61, 0x74, 0x69, 0x20, 0x6f, 0x20, 0x100, 0x77, 0x68, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x75, 0x72, 0x75,
0x57, 0x101, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x57, 0x101, 0x20,
0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x20, 0x41, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x57, 0x101, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65,
0x61, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x57, 0x101, 0x20, 0x41,
0x6d, 0x61, 0x68, 0x6f, 0x6e, 0x61, 0x57, 0x101, 0x20, 0x41, 0x6d, 0x61,
0x68, 0x6f, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e,
0x75, 0x69, 0x57, 0x101, 0x20, 0x41, 0x6d, 0x61, 0x68, 0x6f, 0x6e, 0x61,
0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x57,
0x61, 0x65, 0x6e, 0x67, 0x61, 0x57, 0x101, 0x20, 0x41, 0x72, 0x6f, 0x77,
0x68, 0x101, 0x6e, 0x75, 0x69, 0x20, 0x57, 0x61, 0x65, 0x6e, 0x67, 0x61,
0x57, 0x101, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x20, 0x57, 0x61,
0x65, 0x6e, 0x67, 0x61, 0x57, 0x101, 0x20, 0x52, 0x101, 0x77, 0x68, 0x69,
0x74, 0x69, 0x57, 0x101, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e,
0x75, 0x69, 0x20, 0x52, 0x101, 0x77, 0x68, 0x69, 0x74, 0x69, 0x57, 0x101,
0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x20, 0x52, 0x101, 0x77, 0x68,
0x69, 0x74, 0x69, 0x57, 0x101, 0x20, 0x4d, 0x61, 0x75, 0x6e, 0x67, 0x61,
0x57, 0x101, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69,
0x20, 0x4d, 0x61, 0x75, 0x6e, 0x67, 0x61, 0x57, 0x101, 0x20, 0x41, 0x77,
0x61, 0x74, 0x65, 0x61, 0x20, 0x4d, 0x61, 0x75, 0x6e, 0x67, 0x61, 0x57,
0x101, 0x20, 0x4b, 0x69, 0x77, 0x61, 0x57, 0x101, 0x20, 0x41, 0x72, 0x6f,
0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x20, 0x4b, 0x69, 0x77, 0x61, 0x57,
0x101, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x20, 0x4b, 0x69, 0x77,
0x61, 0x57, 0x101, 0x20, 0x100, 0x70, 0x69, 0x61, 0x57, 0x101, 0x20, 0x100,
0x70, 0x69, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75,
0x69, 0x57, 0x101, 0x20, 0x100, 0x70, 0x69, 0x61, 0x20, 0x41, 0x77, 0x61,
0x74, 0x65, 0x61, 0x57, 0x101, 0x20, 0x41, 0x72, 0x101, 0x70, 0x69, 0x61,
0x57, 0x101, 0x20, 0x41, 0x72, 0x101, 0x70, 0x69, 0x61, 0x20, 0x41, 0x72,
0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x41, 0x72,
0x101, 0x70, 0x69, 0x61, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x57,
0x101, 0x20, 0x100, 0x6b, 0x65, 0x74, 0x69, 0x6e, 0x61, 0x57, 0x101, 0x20,
0x100, 0x6b, 0x65, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77,
0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x100, 0x6b, 0x65, 0x74,
0x69, 0x6e, 0x61, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57,
0x101, 0x20, 0x100, 0x6b, 0x65, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6b, 0x69,
0x20, 0x74, 0x65, 0x20, 0x75, 0x72, 0x75, 0x57, 0x101, 0x20, 0x100, 0x6b,
0x65, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20,
0x75, 0x72, 0x75, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75,
0x69, 0x57, 0x101, 0x20, 0x100, 0x6b, 0x65, 0x74, 0x69, 0x6e, 0x61, 0x20,
0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x75, 0x72, 0x75, 0x20, 0x52, 0x61,
0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x100, 0x6d, 0x65, 0x6e,
0x69, 0x61, 0x57, 0x101, 0x20, 0x100, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20,
0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20,
0x100, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61,
0x74, 0x69, 0x57, 0x101, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x74, 0x69, 0x6b,
0x69, 0x57, 0x101, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75,
0x69, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x74, 0x69, 0x6b, 0x69, 0x57, 0x101,
0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x20, 0x52, 0x61, 0x6e, 0x61,
0x74, 0x69, 0x6b, 0x69, 0x57, 0x101, 0x20, 0x41, 0x68, 0x69, 0x74, 0x65,
0x72, 0x65, 0x69, 0x72, 0x69, 0x61, 0x20, 0x57, 0x61, 0x65, 0x6e, 0x67,
0x61, 0x57, 0x101, 0x20, 0x41, 0x68, 0x69, 0x74, 0x65, 0x72, 0x65, 0x69,
0x72, 0x69, 0x61, 0x20, 0x57, 0x61, 0x65, 0x6e, 0x67, 0x61, 0x20, 0x41,
0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x41,
0x68, 0x69, 0x74, 0x65, 0x72, 0x65, 0x69, 0x72, 0x69, 0x61, 0x20, 0x57,
0x61, 0x65, 0x6e, 0x67, 0x61, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61,
0x57, 0x101, 0x20, 0x41, 0x68, 0x69, 0x74, 0x65, 0x72, 0x65, 0x69, 0x72,
0x69, 0x61, 0x20, 0x57, 0x61, 0x65, 0x6e, 0x67, 0x61, 0x2d, 0x55, 0x72,
0x75, 0x57, 0x101, 0x20, 0x41, 0x68, 0x69, 0x74, 0x65, 0x72, 0x65, 0x69,
0x72, 0x69, 0x61, 0x20, 0x57, 0x61, 0x65, 0x6e, 0x67, 0x61, 0x2d, 0x55,
0x72, 0x75, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69,
0x57, 0x101, 0x20, 0x41, 0x68, 0x69, 0x74, 0x65, 0x72, 0x65, 0x69, 0x72,
0x69, 0x61, 0x20, 0x57, 0x61, 0x65, 0x6e, 0x67, 0x61, 0x2d, 0x55, 0x72,
0x75, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x57, 0x101, 0x20, 0x41,
0x68, 0x69, 0x74, 0x65, 0x72, 0x65, 0x69, 0x72, 0x69, 0x61, 0x20, 0x6b,
0x69, 0x20, 0x74, 0x65, 0x20, 0x52, 0x101, 0x77, 0x68, 0x69, 0x74, 0x69,
0x57, 0x101, 0x20, 0x41, 0x68, 0x69, 0x74, 0x65, 0x72, 0x65, 0x69, 0x72,
0x69, 0x61, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x52, 0x101, 0x77,
0x68, 0x69, 0x74, 0x69, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e,
0x75, 0x69, 0x57, 0x101, 0x20, 0x41, 0x68, 0x69, 0x74, 0x65, 0x72, 0x65,
0x69, 0x72, 0x69, 0x61, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x52,
0x101, 0x77, 0x68, 0x69, 0x74, 0x69, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65,
0x61, 0x57, 0x101, 0x20, 0x41, 0x68, 0x69, 0x74, 0x65, 0x72, 0x65, 0x69,
0x72, 0x69, 0x61, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x55, 0x72,
0x75, 0x57, 0x101, 0x20, 0x41, 0x68, 0x69, 0x74, 0x65, 0x72, 0x65, 0x69,
0x72, 0x69, 0x61, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x55, 0x72,
0x75, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57,
0x101, 0x20, 0x41, 0x68, 0x69, 0x74, 0x65, 0x72, 0x65, 0x69, 0x72, 0x69,
0x61, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x55, 0x72, 0x75, 0x20,
0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x57, 0x101, 0x20, 0x41, 0x74, 0x65,
0x70, 0x61, 0x69, 0x68, 0x101, 0x6e, 0x69, 0x61, 0x57, 0x101, 0x20, 0x41,
0x74, 0x65, 0x70, 0x61, 0x69, 0x68, 0x101, 0x6e, 0x69, 0x61, 0x20, 0x41,
0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x41,
0x74, 0x65, 0x70, 0x61, 0x69, 0x68, 0x101, 0x6e, 0x69, 0x61, 0x20, 0x52,
0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x73, 0x57, 0x101, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73,
0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101,
0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x20, 0x52, 0x61, 0x75, 0x6d,
0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x50, 0x101, 0x6b, 0x61, 0x72, 0x61,
0x74, 0x113, 0x68, 0x69, 0x57, 0x101, 0x20, 0x50, 0x101, 0x6b, 0x61, 0x72,
0x61, 0x74, 0x113, 0x68, 0x69, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101,
0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x50, 0x101, 0x6b, 0x61, 0x72, 0x61,
0x74, 0x113, 0x68, 0x69, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69,
0x57, 0x101, 0x20, 0x50, 0x16b, 0x74, 0x61, 0x6e, 0x61, 0x57, 0x101, 0x20,
0x50, 0x6f, 0x72, 0x69, 0x77, 0x69, 0x61, 0x57, 0x101, 0x20, 0x50, 0x61,
0x72, 0x12b, 0x68, 0x69, 0x61, 0x57, 0x101, 0x20, 0x50, 0x61, 0x72, 0x12b,
0x68, 0x69, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75,
0x69, 0x57, 0x101, 0x20, 0x50, 0x61, 0x72, 0x12b, 0x68, 0x69, 0x61, 0x20,
0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x50, 0x6f,
0x72, 0x6f, 0x6e, 0x61, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73,
0x61, 0x6c, 0x61, 0x6d, 0x57, 0x101, 0x20, 0x6f, 0x20, 0x54, 0x65, 0x20,
0x4b, 0x16b, 0x72, 0x61, 0x65, 0x20, 0x4d, 0x61, 0x74, 0x6f, 0x6d, 0x61,
0x74, 0x6f, 0x57, 0x101, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e,
0x75, 0x69, 0x20, 0x6f, 0x20, 0x54, 0x65, 0x20, 0x4b, 0x16b, 0x72, 0x61,
0x65, 0x20, 0x4d, 0x61, 0x74, 0x6f, 0x6d, 0x61, 0x74, 0x6f, 0x57, 0x101,
0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x20, 0x6f, 0x20, 0x54,
0x65, 0x20, 0x4b, 0x16b, 0x72, 0x61, 0x65, 0x20, 0x4d, 0x61, 0x74, 0x6f,
0x6d, 0x61, 0x74, 0x6f, 0x57, 0x101, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f,
0x72, 0x72, 0x6f, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75,
0x69, 0x57, 0x101, 0x20, 0x52, 0x113, 0x6b, 0x6f, 0x68, 0x75, 0x57, 0x101,
0x20, 0x52, 0x113, 0x6b, 0x6f, 0x68, 0x75, 0x20, 0x41, 0x72, 0x6f, 0x77,
0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x52, 0x113, 0x6b, 0x6f,
0x68, 0x75, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x57, 0x101, 0x20,
0x48, 0x69, 0x72, 0x69, 0x57, 0x101, 0x20, 0x48, 0x69, 0x72, 0x69, 0x20,
0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20,
0x48, 0x69, 0x72, 0x69, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69,
0x57, 0x101, 0x20, 0x48, 0x61, 0x69, 0x6e, 0x61, 0x57, 0x101, 0x20, 0x48,
0x61, 0x69, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e,
0x75, 0x69, 0x57, 0x101, 0x20, 0x48, 0x61, 0x69, 0x6e, 0x61, 0x20, 0x41,
0x77, 0x61, 0x74, 0x65, 0x61, 0x57, 0x101, 0x20, 0x6f, 0x20, 0x54, 0x65,
0x20, 0x4d, 0x6f, 0x75, 0x74, 0x65, 0x72, 0x65, 0x20, 0x4b, 0x69, 0x72,
0x69, 0x68, 0x69, 0x6d, 0x65, 0x74, 0x65, 0x57, 0x101, 0x20, 0x6f, 0x20,
0x4e, 0x67, 0x101, 0x20, 0x4d, 0x6f, 0x75, 0x74, 0x65, 0x72, 0x65, 0x20,
0x4b, 0x6f, 0x6b, 0x6f, 0x68, 0x69, 0x57, 0x101, 0x20, 0x4b, 0x6f, 0x72,
0x6f, 0x6d, 0x14d, 0x70, 0x69, 0x61, 0x57, 0x101, 0x20, 0x4b, 0x6f, 0x72,
0x6f, 0x6d, 0x14d, 0x70, 0x69, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68,
0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x4b, 0x6f, 0x72, 0x6f, 0x6d,
0x14d, 0x70, 0x69, 0x61, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69,
0x57, 0x101, 0x20, 0x4b, 0x75, 0x6b, 0x69, 0x20, 0x41, 0x69, 0x72, 0x61,
0x6e, 0x69, 0x57, 0x101, 0x20, 0x4b, 0x75, 0x6b, 0x69, 0x20, 0x41, 0x69,
0x72, 0x61, 0x6e, 0x69, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e,
0x75, 0x69, 0x57, 0x101, 0x20, 0x4b, 0x75, 0x6b, 0x69, 0x20, 0x41, 0x69,
0x72, 0x61, 0x6e, 0x69, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69,
0x20, 0x48, 0x61, 0x75, 0x72, 0x75, 0x61, 0x57, 0x101, 0x20, 0x4b, 0x69,
0x75, 0x70, 0x61, 0x57, 0x101, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101,
0x6e, 0x75, 0x69, 0x20, 0x4b, 0x69, 0x75, 0x70, 0x61, 0x57, 0x101, 0x20,
0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x20, 0x4b, 0x69, 0x75, 0x70, 0x61,
0x57, 0x101, 0x20, 0x52, 0x113, 0x77, 0x65, 0x74, 0x69, 0x57, 0x101, 0x20,
0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76,
0x69, 0x6c, 0x6c, 0x65, 0x57, 0x101, 0x20, 0x6f, 0x20, 0x54, 0x69, 0x6d,
0x6f, 0x61, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x52, 0x101, 0x77,
0x68, 0x69, 0x74, 0x69, 0x57, 0x101, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65,
0x20, 0x4d, 0x6f, 0x75, 0x74, 0x65, 0x72, 0x65, 0x20, 0x6f, 0x20, 0x41,
0x72, 0x61, 0x6e, 0x67, 0x61, 0x57, 0x101, 0x20, 0x6b, 0x69, 0x20, 0x74,
0x65, 0x20, 0x4d, 0x6f, 0x75, 0x74, 0x65, 0x72, 0x65, 0x20, 0x41, 0x72,
0x61, 0x6e, 0x67, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e,
0x75, 0x69, 0x57, 0x101, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x4d,
0x6f, 0x75, 0x74, 0x65, 0x72, 0x65, 0x20, 0x41, 0x72, 0x61, 0x6e, 0x67,
0x61, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20,
0x45, 0x6b, 0x75, 0x61, 0x74, 0x6f, 0x61, 0x57, 0x101, 0x20, 0x55, 0x72,
0x6f, 0x70, 0x69, 0x20, 0x57, 0x61, 0x65, 0x6e, 0x67, 0x61, 0x57, 0x101,
0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x20, 0x55,
0x72, 0x6f, 0x70, 0x69, 0x20, 0x57, 0x61, 0x65, 0x6e, 0x67, 0x61, 0x57,
0x101, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x20, 0x55, 0x72,
0x6f, 0x70, 0x69, 0x20, 0x57, 0x61, 0x65, 0x6e, 0x67, 0x61, 0x57, 0x101,
0x20, 0x55, 0x72, 0x6f, 0x70, 0x69, 0x20, 0x52, 0x101, 0x77, 0x68, 0x69,
0x74, 0x69, 0x57, 0x101, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e,
0x75, 0x69, 0x20, 0x55, 0x72, 0x6f, 0x70, 0x69, 0x20, 0x52, 0x101, 0x77,
0x68, 0x69, 0x74, 0x69, 0x57, 0x101, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61,
0x74, 0x69, 0x20, 0x55, 0x72, 0x6f, 0x70, 0x69, 0x20, 0x52, 0x101, 0x77,
0x68, 0x69, 0x74, 0x69, 0x57, 0x101, 0x20, 0x16a, 0x72, 0x6f, 0x70, 0x69,
0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x72, 0x101, 0x77, 0x68, 0x69,
0x74, 0x69, 0x20, 0x72, 0x61, 0x77, 0x61, 0x57, 0x101, 0x20, 0x55, 0x72,
0x6f, 0x70, 0x69, 0x20, 0x55, 0x72, 0x75, 0x57, 0x101, 0x20, 0x41, 0x72,
0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x20, 0x55, 0x72, 0x6f, 0x70,
0x69, 0x20, 0x55, 0x72, 0x75, 0x57, 0x101, 0x20, 0x52, 0x61, 0x75, 0x6d,
0x61, 0x74, 0x69, 0x20, 0x55, 0x72, 0x6f, 0x70, 0x69, 0x20, 0x55, 0x72,
0x75, 0x57, 0x101, 0x20, 0x6b, 0x69, 0x20, 0x4e, 0x67, 0x101, 0x20, 0x4d,
0x6f, 0x74, 0x75, 0x20, 0x57, 0x68, 0x101, 0x6b, 0x61, 0x6e, 0x61, 0x57,
0x101, 0x20, 0x6b, 0x69, 0x20, 0x4e, 0x67, 0x101, 0x20, 0x4d, 0x6f, 0x74,
0x75, 0x20, 0x57, 0x68, 0x101, 0x6b, 0x61, 0x6e, 0x61, 0x20, 0x41, 0x72,
0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x6b, 0x69,
0x20, 0x4e, 0x67, 0x101, 0x20, 0x4d, 0x6f, 0x74, 0x75, 0x20, 0x57, 0x68,
0x101, 0x6b, 0x61, 0x6e, 0x61, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74,
0x69, 0x57, 0x101, 0x20, 0x57, 0x68, 0x12b, 0x74, 0x12b, 0x57, 0x101, 0x20,
0x57, 0x68, 0x12b, 0x74, 0x12b, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101,
0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x57, 0x68, 0x12b, 0x74, 0x12b, 0x20,
0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x4b, 0x69,
0x101, 0x6e, 0x61, 0x20, 0x57, 0x12b, 0x77, 0x12b, 0x57, 0x101, 0x20, 0x57,
0x12b, 0x77, 0x12b, 0x20, 0x6f, 0x20, 0x54, 0x65, 0x20, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x20, 0x6d, 0x65, 0x20, 0x74, 0x65, 0x20, 0x4b, 0x14d, 0x70,
0x61, 0x6b, 0x61, 0x74, 0x61, 0x6e, 0x67, 0x61, 0x20, 0x6b, 0x69, 0x20,
0x74, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x57, 0x101, 0x20, 0x47,
0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x57, 0x101, 0x20, 0x47,
0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x57, 0x101, 0x20, 0x48, 0x14d, 0x72,
0x69, 0x61, 0x57, 0x101, 0x20, 0x48, 0x14d, 0x72, 0x69, 0x61, 0x20, 0x41,
0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x48,
0x14d, 0x72, 0x69, 0x61, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69,
0x57, 0x101, 0x20, 0x4b, 0x69, 0x72, 0x69, 0x70, 0x61, 0x74, 0x69, 0x57,
0x101, 0x20, 0x54, 0x6f, 0x68, 0x61, 0x72, 0x69, 0x74, 0x65, 0x20, 0x4b,
0x69, 0x72, 0x69, 0x77, 0x12b, 0x74, 0x69, 0x57, 0x101, 0x20, 0x57, 0x68,
0x65, 0x6e, 0x75, 0x61, 0x6b, 0x101, 0x72, 0x69, 0x6b, 0x69, 0x20, 0x6b,
0x69, 0x20, 0x74, 0x65, 0x20, 0x72, 0x101, 0x77, 0x68, 0x69, 0x74, 0x69,
0x57, 0x101, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69,
0x20, 0x6f, 0x20, 0x57, 0x68, 0x65, 0x6e, 0x75, 0x61, 0x6b, 0x101, 0x72,
0x69, 0x6b, 0x69, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x72, 0x101,
0x77, 0x68, 0x69, 0x74, 0x69, 0x57, 0x101, 0x20, 0x52, 0x61, 0x75, 0x6d,
0x61, 0x74, 0x69, 0x20, 0x6f, 0x20, 0x57, 0x68, 0x65, 0x6e, 0x75, 0x61,
0x6b, 0x101, 0x72, 0x69, 0x6b, 0x69, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65,
0x20, 0x72, 0x101, 0x77, 0x68, 0x69, 0x74, 0x69, 0x57, 0x101, 0x20, 0x57,
0x68, 0x65, 0x6e, 0x75, 0x61, 0x6b, 0x101, 0x72, 0x69, 0x6b, 0x69, 0x20,
0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x75, 0x72, 0x75, 0x57, 0x101, 0x20,
0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x20, 0x6f, 0x20,
0x57, 0x68, 0x65, 0x6e, 0x75, 0x61, 0x6b, 0x101, 0x72, 0x69, 0x6b, 0x69,
0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x75, 0x72, 0x75, 0x57, 0x101,
0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x20, 0x6f, 0x20, 0x57,
0x68, 0x65, 0x6e, 0x75, 0x61, 0x6b, 0x101, 0x72, 0x69, 0x6b, 0x69, 0x20,
0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x75, 0x72, 0x75, 0x57, 0x101, 0x20,
0x57, 0x68, 0x61, 0x6e, 0x67, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68,
0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x4b, 0x61, 0x69, 0x61, 0x6e,
0x61, 0x57, 0x101, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x57, 0x101, 0x20, 0x41, 0x72,
0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x20, 0x48, 0x61, 0x77, 0x61,
0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x57,
0x101, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x20, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e,
0x57, 0x101, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x69, 0x70, 0x75, 0x61, 0x57,
0x101, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x69, 0x70, 0x75, 0x61, 0x20, 0x41,
0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x48,
0x6f, 0x6e, 0x67, 0x69, 0x70, 0x75, 0x61, 0x20, 0x52, 0x61, 0x75, 0x6d,
0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x57, 0x101,
0x20, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101,
0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x52,
0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x12a, 0x6e, 0x69,
0x61, 0x57, 0x101, 0x20, 0x6f, 0x20, 0x54, 0x65, 0x20, 0x4d, 0x6f, 0x61,
0x6e, 0x61, 0x20, 0x12a, 0x6e, 0x69, 0x61, 0x57, 0x101, 0x20, 0x12a, 0x6e,
0x69, 0x61, 0x68, 0x61, 0x69, 0x6e, 0x61, 0x57, 0x101, 0x20, 0x49, 0x6e,
0x69, 0x74, 0x6f, 0x6e, 0x12b, 0x68, 0x69, 0x61, 0x20, 0x57, 0x61, 0x65,
0x6e, 0x67, 0x61, 0x57, 0x101, 0x20, 0x49, 0x6e, 0x69, 0x74, 0x6f, 0x6e,
0x12b, 0x68, 0x69, 0x61, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x72,
0x101, 0x77, 0x68, 0x69, 0x74, 0x69, 0x57, 0x101, 0x20, 0x49, 0x6e, 0x69,
0x74, 0x6f, 0x6e, 0x12b, 0x68, 0x69, 0x61, 0x20, 0x6b, 0x69, 0x20, 0x74,
0x65, 0x20, 0x75, 0x72, 0x75, 0x57, 0x101, 0x20, 0x49, 0x72, 0x101, 0x6e,
0x61, 0x57, 0x101, 0x20, 0x49, 0x72, 0x101, 0x6e, 0x61, 0x20, 0x41, 0x72,
0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x49, 0x72,
0x101, 0x6e, 0x61, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x57, 0x101,
0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57, 0x101, 0x20, 0x49,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68,
0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101,
0x20, 0x49, 0x68, 0x61, 0x72, 0x61, 0x69, 0x72, 0x61, 0x57, 0x101, 0x20,
0x49, 0x68, 0x61, 0x72, 0x61, 0x69, 0x72, 0x61, 0x20, 0x41, 0x72, 0x6f,
0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x49, 0x68, 0x61,
0x72, 0x61, 0x69, 0x72, 0x61, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61,
0x57, 0x101, 0x20, 0x48, 0x61, 0x70, 0x61, 0x6e, 0x69, 0x57, 0x101, 0x20,
0x48, 0x61, 0x70, 0x61, 0x6e, 0x69, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68,
0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x48, 0x61, 0x70, 0x61, 0x6e,
0x69, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x57, 0x101, 0x20, 0x4b,
0x61, 0x74, 0x61, 0x74, 0x101, 0x6e, 0x67, 0x61, 0x57, 0x101, 0x20, 0x4b,
0x61, 0x74, 0x61, 0x74, 0x101, 0x6e, 0x67, 0x61, 0x20, 0x6b, 0x69, 0x20,
0x74, 0x65, 0x20, 0x52, 0x101, 0x77, 0x68, 0x69, 0x74, 0x69, 0x57, 0x101,
0x20, 0x4b, 0x61, 0x74, 0x61, 0x74, 0x101, 0x6e, 0x67, 0x61, 0x20, 0x6b,
0x69, 0x20, 0x74, 0x65, 0x20, 0x55, 0x72, 0x75, 0x57, 0x101, 0x20, 0x4b,
0x14d, 0x72, 0x65, 0x61, 0x57, 0x101, 0x20, 0x4b, 0x14d, 0x72, 0x65, 0x61,
0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101,
0x20, 0x4b, 0x14d, 0x72, 0x65, 0x61, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65,
0x61, 0x57, 0x101, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x57, 0x101,
0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b,
0x57, 0x101, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69,
0x57, 0x101, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101,
0x20, 0x4b, 0x69, 0x6b, 0x69, 0x74, 0x101, 0x6e, 0x67, 0x61, 0x57, 0x101,
0x20, 0x6f, 0x20, 0x4e, 0x67, 0x101, 0x20, 0x4d, 0x6f, 0x75, 0x74, 0x65,
0x72, 0x20, 0x6f, 0x20, 0x54, 0x65, 0x20, 0x52, 0x61, 0x69, 0x6e, 0x61,
0x57, 0x101, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0x57, 0x101, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101,
0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x41,
0x77, 0x61, 0x74, 0x65, 0x61, 0x57, 0x101, 0x20, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x57, 0x101, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57,
0x101, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x52, 0x61,
0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x4d, 0x61, 0x72, 0x65,
0x69, 0x61, 0x57, 0x101, 0x20, 0x4d, 0x101, 0x72, 0x61, 0x74, 0x69, 0x72,
0x69, 0x57, 0x101, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61,
0x73, 0x57, 0x101, 0x20, 0x6f, 0x20, 0x4e, 0x67, 0x101, 0x20, 0x4d, 0x6f,
0x74, 0x75, 0x20, 0x4d, 0x101, 0x68, 0x61, 0x72, 0x61, 0x57, 0x101, 0x20,
0x4d, 0x61, 0x72, 0x69, 0x68, 0x69, 0x57, 0x101, 0x20, 0x4d, 0x61, 0x72,
0x69, 0x68, 0x69, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75,
0x69, 0x57, 0x101, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x68, 0x69, 0x20, 0x52,
0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x4d, 0x61, 0x77,
0x73, 0x6f, 0x6e, 0x57, 0x101, 0x20, 0x4d, 0x113, 0x68, 0x69, 0x6b, 0x6f,
0x20, 0x4b, 0x69, 0x77, 0x61, 0x57, 0x101, 0x20, 0x41, 0x72, 0x6f, 0x77,
0x68, 0x101, 0x6e, 0x75, 0x69, 0x20, 0x4d, 0x113, 0x68, 0x69, 0x6b, 0x6f,
0x20, 0x4b, 0x69, 0x77, 0x61, 0x57, 0x101, 0x20, 0x41, 0x77, 0x61, 0x74,
0x65, 0x61, 0x20, 0x4d, 0x113, 0x68, 0x69, 0x6b, 0x6f, 0x20, 0x4b, 0x69,
0x77, 0x61, 0x57, 0x101, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61,
0x61, 0x74, 0x61, 0x72, 0x57, 0x101, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e,
0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68,
0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e,
0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61,
0x74, 0x69, 0x57, 0x101, 0x20, 0x4d, 0x6f, 0x68, 0x69, 0x6b, 0x61, 0x75,
0x57, 0x101, 0x20, 0x4d, 0x6f, 0x68, 0x69, 0x6b, 0x61, 0x75, 0x20, 0x41,
0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x4d,
0x6f, 0x68, 0x69, 0x6b, 0x61, 0x75, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61,
0x74, 0x69, 0x57, 0x101, 0x20, 0x50, 0x113, 0x6d, 0x61, 0x57, 0x101, 0x20,
0x4e, 0x61, 0x75, 0x72, 0x75, 0x57, 0x101, 0x20, 0x4e, 0x65, 0x70, 0x14d,
0x72, 0x61, 0x57, 0x101, 0x20, 0x57, 0x68, 0x65, 0x6e, 0x75, 0x61, 0x20,
0x4b, 0x61, 0x6e, 0x61, 0x6b, 0x69, 0x57, 0x101, 0x20, 0x57, 0x68, 0x65,
0x6e, 0x75, 0x61, 0x20, 0x4b, 0x61, 0x6e, 0x61, 0x6b, 0x69, 0x20, 0x41,
0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x57,
0x68, 0x65, 0x6e, 0x75, 0x61, 0x20, 0x4b, 0x61, 0x6e, 0x61, 0x6b, 0x69,
0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x41,
0x6f, 0x74, 0x65, 0x61, 0x72, 0x6f, 0x61, 0x57, 0x101, 0x20, 0x41, 0x6f,
0x74, 0x65, 0x61, 0x72, 0x6f, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68,
0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x41, 0x6f, 0x74, 0x65, 0x61,
0x72, 0x6f, 0x61, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x57, 0x101,
0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e,
0x64, 0x57, 0x101, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75,
0x69, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x57, 0x101, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x20,
0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64,
0x57, 0x101, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x57, 0x101, 0x20, 0x6f, 0x20,
0x54, 0x65, 0x20, 0x4d, 0x6f, 0x75, 0x74, 0x65, 0x72, 0x65, 0x20, 0x4e,
0x14d, 0x70, 0x6f, 0x6b, 0x65, 0x57, 0x101, 0x20, 0x6f, 0x20, 0x54, 0x65,
0x20, 0x4d, 0x6f, 0x75, 0x74, 0x65, 0x72, 0x65, 0x20, 0x4e, 0x14d, 0x70,
0x6f, 0x6b, 0x65, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75,
0x69, 0x57, 0x101, 0x20, 0x6f, 0x20, 0x54, 0x65, 0x20, 0x4d, 0x6f, 0x75,
0x74, 0x65, 0x72, 0x65, 0x20, 0x4e, 0x14d, 0x70, 0x6f, 0x6b, 0x65, 0x20,
0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x57, 0x101, 0x20, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x57, 0x101, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69,
0x57, 0x101, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x52,
0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x57, 0x101, 0x20, 0x4e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x41,
0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x4e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x52,
0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x4f, 0x6d, 0x73,
0x6b, 0x57, 0x101, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x41, 0x72, 0x6f,
0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x4f, 0x6d, 0x73,
0x6b, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20,
0x50, 0x61, 0x6b, 0x69, 0x74, 0x101, 0x6e, 0x65, 0x57, 0x101, 0x20, 0x50,
0x61, 0x6b, 0x69, 0x74, 0x101, 0x6e, 0x65, 0x20, 0x41, 0x72, 0x6f, 0x77,
0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x50, 0x61, 0x6b, 0x69,
0x74, 0x101, 0x6e, 0x65, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69,
0x57, 0x101, 0x20, 0x50, 0x101, 0x72, 0x61, 0x75, 0x57, 0x101, 0x20, 0x50,
0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x16b, 0x6b, 0x69, 0x6e, 0x69, 0x57,
0x101, 0x20, 0x50, 0x61, 0x72, 0x61, 0x6b, 0x61, 0x69, 0x57, 0x101, 0x20,
0x50, 0x61, 0x72, 0x61, 0x6b, 0x61, 0x69, 0x20, 0x41, 0x72, 0x6f, 0x77,
0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x50, 0x61, 0x72, 0x61,
0x6b, 0x61, 0x69, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57,
0x101, 0x20, 0x50, 0x65, 0x72, 0x75, 0x57, 0x101, 0x20, 0x50, 0x65, 0x72,
0x75, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57,
0x101, 0x20, 0x50, 0x65, 0x72, 0x75, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61,
0x74, 0x69, 0x57, 0x101, 0x20, 0x50, 0x69, 0x72, 0x69, 0x70, 0x12b, 0x6e,
0x69, 0x57, 0x101, 0x20, 0x50, 0x69, 0x72, 0x69, 0x70, 0x12b, 0x6e, 0x69,
0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101,
0x20, 0x50, 0x69, 0x72, 0x69, 0x70, 0x12b, 0x6e, 0x69, 0x20, 0x52, 0x61,
0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x6f, 0x20, 0x4e, 0x67,
0x101, 0x20, 0x4d, 0x6f, 0x75, 0x74, 0x65, 0x72, 0x65, 0x20, 0x50, 0x68,
0x6f, 0x65, 0x6e, 0x69, 0x78, 0x57, 0x101, 0x20, 0x53, 0x74, 0x2e, 0x20,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x6d, 0x65, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x57, 0x101, 0x20, 0x41, 0x72, 0x6f,
0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x20, 0x6f, 0x20, 0x53, 0x74, 0x2e,
0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x6d, 0x65, 0x20, 0x4d,
0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x57, 0x101, 0x20, 0x41, 0x77,
0x61, 0x74, 0x65, 0x61, 0x20, 0x6f, 0x20, 0x53, 0x74, 0x2e, 0x20, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x6d, 0x65, 0x20, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x57, 0x101, 0x20, 0x50, 0x69, 0x74, 0x63,
0x61, 0x69, 0x72, 0x6e, 0x57, 0x101, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70,
0x65, 0x57, 0x101, 0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e,
0x67, 0x57, 0x101, 0x20, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x57,
0x101, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x57, 0x101, 0x20,
0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x57, 0x101, 0x20, 0x53,
0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x41, 0x72, 0x6f, 0x77,
0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69,
0x57, 0x101, 0x20, 0x48, 0x101, 0x6d, 0x6f, 0x61, 0x57, 0x101, 0x20, 0x48,
0x101, 0x6d, 0x6f, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e,
0x75, 0x69, 0x57, 0x101, 0x20, 0x48, 0x101, 0x6d, 0x6f, 0x61, 0x20, 0x41,
0x77, 0x61, 0x74, 0x65, 0x61, 0x57, 0x101, 0x20, 0x48, 0x65, 0x69, 0x6b,
0x65, 0x72, 0x65, 0x57, 0x101, 0x20, 0x48, 0x69, 0x6e, 0x67, 0x61, 0x70,
0x6f, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69,
0x57, 0x101, 0x20, 0x6f, 0x20, 0x4e, 0x67, 0x101, 0x20, 0x4d, 0x6f, 0x74,
0x75, 0x20, 0x48, 0x6f, 0x72, 0x6f, 0x6d, 0x6f, 0x6e, 0x61, 0x57, 0x101,
0x20, 0x48, 0x14d, 0x72, 0x69, 0x61, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65,
0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x57, 0x101, 0x20, 0x48, 0x75, 0x72,
0x69, 0x6e, 0x61, 0x6d, 0x65, 0x57, 0x101, 0x20, 0x53, 0x79, 0x6f, 0x77,
0x61, 0x57, 0x101, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x57, 0x101,
0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x57, 0x101, 0x20, 0x54, 0x61,
0x69, 0x70, 0x65, 0x69, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e,
0x75, 0x69, 0x57, 0x101, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20,
0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x57, 0x101, 0x20, 0x54, 0x61, 0x6b,
0x69, 0x72, 0x69, 0x74, 0x101, 0x6e, 0x67, 0x61, 0x57, 0x101, 0x20, 0x54,
0x6f, 0x6b, 0x65, 0x72, 0x61, 0x75, 0x57, 0x101, 0x20, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x57, 0x101, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x41,
0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69,
0x57, 0x101, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x57, 0x101, 0x20, 0x54,
0x75, 0x6b, 0x75, 0x6d, 0x61, 0x6e, 0x61, 0x74, 0x101, 0x6e, 0x67, 0x61,
0x57, 0x101, 0x20, 0x54, 0x75, 0x6b, 0x75, 0x6d, 0x61, 0x6e, 0x61, 0x74,
0x101, 0x6e, 0x67, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e,
0x75, 0x69, 0x57, 0x101, 0x20, 0x54, 0x75, 0x6b, 0x75, 0x6d, 0x61, 0x6e,
0x61, 0x74, 0x101, 0x6e, 0x67, 0x61, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61,
0x74, 0x69, 0x57, 0x101, 0x20, 0x54, 0x16b, 0x77, 0x61, 0x72, 0x75, 0x57,
0x101, 0x20, 0x55, 0x72, 0x75, 0x6b, 0x6f, 0x69, 0x57, 0x101, 0x20, 0x55,
0x72, 0x75, 0x6b, 0x6f, 0x69, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101,
0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x55, 0x72, 0x75, 0x6b, 0x6f, 0x69,
0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x55,
0x68, 0x69, 0x70, 0x65, 0x6b, 0x65, 0x74, 0x101, 0x6e, 0x65, 0x57, 0x101,
0x20, 0x55, 0x68, 0x69, 0x70, 0x65, 0x6b, 0x65, 0x74, 0x101, 0x6e, 0x65,
0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101,
0x20, 0x55, 0x68, 0x69, 0x70, 0x65, 0x6b, 0x65, 0x74, 0x101, 0x6e, 0x65,
0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x57,
0x68, 0x65, 0x6e, 0x75, 0x61, 0x74, 0x16b, 0x57, 0x101, 0x20, 0x57, 0x68,
0x65, 0x6e, 0x75, 0x61, 0x74, 0x16b, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68,
0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x57, 0x68, 0x65, 0x6e, 0x75,
0x61, 0x74, 0x16b, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57,
0x101, 0x20, 0x50, 0x65, 0x6e, 0x65, 0x68, 0x16b, 0x65, 0x72, 0x61, 0x57,
0x101, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x57, 0x101, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75,
0x69, 0x57, 0x101, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57,
0x101, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x57,
0x101, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20,
0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x52, 0x61,
0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x56, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x57, 0x101, 0x20, 0x6f, 0x20, 0x54, 0x65, 0x20, 0x4d, 0x6f,
0x74, 0x75, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x57, 0x101, 0x20, 0x57, 0x101,
0x72, 0x69, 0x68, 0x69, 0x20, 0x6d, 0x65, 0x20, 0x57, 0x68, 0x75, 0x74,
0x75, 0x6e, 0x61, 0x57, 0x101, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x57, 0x101, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20,
0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20,
0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x52, 0x61, 0x75, 0x6d,
0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x57, 0x101, 0x20, 0x59, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20,
0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20,
0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20,
0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x90f, 0x915, 0x930, 0x20, 0x935, 0x947, 0x933,
0x90f, 0x915, 0x930, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x935, 0x947,
0x933, 0x910, 0x915, 0x930, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x20, 0x935, 0x947, 0x933, 0x905, 0x92b, 0x917, 0x93e, 0x923, 0x93f, 0x938, 0x94d,
0x924, 0x93e, 0x928, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x927, 0x94d, 0x200d, 0x92f,
0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x20, 0x935, 0x947, 0x933,
0x92a, 0x942, 0x930, 0x94d, 0x935, 0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915,
0x93e, 0x20, 0x935, 0x947, 0x933, 0x926, 0x915, 0x94d, 0x937, 0x93f, 0x923, 0x20,
0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e,
0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x20, 0x935, 0x947, 0x933,
0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f,
0x915, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947,
0x933, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x906, 0x92b, 0x94d, 0x930,
0x93f, 0x915, 0x93e, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20,
0x935, 0x947, 0x933, 0x905, 0x932, 0x93e, 0x938, 0x94d, 0x915, 0x93e, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x905, 0x932, 0x93e,
0x938, 0x94d, 0x915, 0x93e, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d,
0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x905, 0x932, 0x94d, 0x92e,
0x93e, 0x91f, 0x940, 0x20, 0x935, 0x947, 0x933, 0x905, 0x932, 0x94d, 0x92e, 0x93e,
0x91f, 0x940, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x935, 0x947, 0x933,
0x905, 0x932, 0x94d, 0x92e, 0x93e, 0x91f, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x935, 0x947, 0x933,
0x905, 0x200d, 0x945, 0x92e, 0x947, 0x91d, 0x949, 0x928, 0x20, 0x935, 0x947, 0x933,
0x905, 0x200d, 0x945, 0x92e, 0x947, 0x91d, 0x949, 0x928, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x905, 0x200d, 0x945, 0x92e, 0x947,
0x91d, 0x949, 0x928, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20,
0x935, 0x947, 0x933, 0x915, 0x947, 0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20,
0x935, 0x947, 0x933, 0x915, 0x947, 0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x915, 0x947,
0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f,
0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x94c,
0x930, 0x94d, 0x935, 0x93e, 0x924, 0x94d, 0x92f, 0x20, 0x935, 0x947, 0x933, 0x92a,
0x94c, 0x930, 0x94d, 0x935, 0x93e, 0x924, 0x94d, 0x92f, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x94c, 0x930, 0x94d, 0x935,
0x93e, 0x924, 0x94d, 0x92f, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d,
0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x930, 0x94d, 0x935,
0x924, 0x940, 0x92f, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935,
0x947, 0x933, 0x92a, 0x930, 0x94d, 0x935, 0x924, 0x940, 0x92f, 0x20, 0x938, 0x942,
0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947,
0x933, 0x92a, 0x945, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x935, 0x947, 0x933,
0x92a, 0x945, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x945, 0x938, 0x93f, 0x92b, 0x93f,
0x915, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e,
0x936, 0x20, 0x935, 0x947, 0x933, 0x90f, 0x928, 0x93e, 0x921, 0x940, 0x92f, 0x930,
0x20, 0x935, 0x947, 0x933, 0x905, 0x928, 0x93e, 0x926, 0x940, 0x930, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x935, 0x947, 0x933, 0x905, 0x928, 0x93e, 0x926,
0x940, 0x930, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e,
0x932, 0x940, 0x928, 0x20, 0x935, 0x947, 0x933, 0x90f, 0x92a, 0x93f, 0x92f, 0x93e,
0x20, 0x935, 0x947, 0x933, 0x90f, 0x92a, 0x93f, 0x92f, 0x93e, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x90f, 0x92a, 0x93f, 0x92f,
0x93e, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e,
0x936, 0x20, 0x935, 0x947, 0x933, 0x905, 0x200d, 0x945, 0x915, 0x94d, 0x924, 0x93e,
0x909, 0x20, 0x935, 0x947, 0x933, 0x905, 0x200d, 0x945, 0x915, 0x94d, 0x924, 0x93e,
0x909, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x935, 0x947, 0x933, 0x905,
0x200d, 0x945, 0x915, 0x94d, 0x924, 0x93e, 0x909, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x935, 0x947, 0x933,
0x905, 0x200d, 0x945, 0x915, 0x94d, 0x91f, 0x94b, 0x92c, 0x947, 0x20, 0x935, 0x947,
0x933, 0x905, 0x200d, 0x945, 0x915, 0x94d, 0x91f, 0x94b, 0x92c, 0x947, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x935, 0x947, 0x933, 0x905, 0x200d, 0x945, 0x915,
0x94d, 0x91f, 0x94b, 0x92c, 0x947, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x935, 0x947, 0x933, 0x905, 0x930,
0x947, 0x92c, 0x93f, 0x92f, 0x928, 0x20, 0x935, 0x947, 0x933, 0x905, 0x930, 0x947,
0x92c, 0x93f, 0x92f, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20,
0x935, 0x947, 0x933, 0x905, 0x930, 0x947, 0x92c, 0x93f, 0x92f, 0x928, 0x20, 0x938,
0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935,
0x947, 0x933, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x93f, 0x928, 0x93e,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x905,
0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x93f, 0x928, 0x93e, 0x20, 0x909, 0x928,
0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x936, 0x94d,
0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f,
0x93f, 0x928, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x936, 0x94d, 0x91a, 0x93f,
0x92e, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x93f, 0x928,
0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933,
0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x91c,
0x947, 0x902, 0x91f, 0x93f, 0x928, 0x93e, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e,
0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x906, 0x930, 0x94d, 0x92e, 0x947, 0x928,
0x93f, 0x92f, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935,
0x947, 0x933, 0x906, 0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20,
0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x905,
0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20, 0x935, 0x947, 0x933, 0x905,
0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f,
0x93f, 0x915, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915,
0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x911,
0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x935,
0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f,
0x928, 0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x20, 0x935, 0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947,
0x932, 0x93f, 0x92f, 0x928, 0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x938, 0x942,
0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947,
0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x928,
0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x2d, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e,
0x20, 0x935, 0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932,
0x93f, 0x92f, 0x928, 0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x2d, 0x92a, 0x936, 0x94d,
0x91a, 0x93f, 0x92e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935,
0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f,
0x928, 0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x2d, 0x92a, 0x936, 0x94d, 0x91a, 0x93f,
0x92e, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e,
0x936, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x20, 0x911,
0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x935,
0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f,
0x928, 0x20, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930,
0x947, 0x932, 0x93f, 0x92f, 0x928, 0x20, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x20,
0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20,
0x935, 0x947, 0x933, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x911, 0x938,
0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x935, 0x947,
0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x928,
0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930,
0x947, 0x932, 0x93f, 0x92f, 0x928, 0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e,
0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936,
0x20, 0x935, 0x947, 0x933, 0x905, 0x91d, 0x930, 0x92c, 0x948, 0x91c, 0x93e, 0x928,
0x20, 0x935, 0x947, 0x933, 0x905, 0x91d, 0x930, 0x92c, 0x948, 0x91c, 0x93e, 0x928,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x905,
0x91d, 0x930, 0x92c, 0x948, 0x91c, 0x93e, 0x928, 0x20, 0x909, 0x928, 0x94d, 0x939,
0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x905, 0x200d, 0x945, 0x91d, 0x94b,
0x930, 0x947, 0x938, 0x20, 0x935, 0x947, 0x933, 0x905, 0x200d, 0x945, 0x91d, 0x94b,
0x930, 0x947, 0x938, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935,
0x947, 0x933, 0x905, 0x200d, 0x945, 0x91d, 0x94b, 0x930, 0x947, 0x938, 0x20, 0x909,
0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92c, 0x93e,
0x902, 0x917, 0x932, 0x93e, 0x926, 0x947, 0x936, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92c, 0x93e, 0x902, 0x917, 0x932, 0x93e,
0x926, 0x947, 0x936, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20,
0x935, 0x947, 0x933, 0x92d, 0x942, 0x924, 0x93e, 0x928, 0x20, 0x935, 0x947, 0x933,
0x92c, 0x94d, 0x930, 0x93e, 0x91d, 0x93f, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92c, 0x94d, 0x930,
0x93e, 0x91d, 0x93f, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x909, 0x928, 0x94d, 0x939,
0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92c, 0x94d, 0x930, 0x941, 0x928,
0x947, 0x908, 0x20, 0x926, 0x93e, 0x930, 0x942, 0x938, 0x932, 0x93e, 0x92e, 0x20,
0x935, 0x947, 0x933, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x94d, 0x939, 0x930, 0x94d,
0x921, 0x947, 0x20, 0x935, 0x947, 0x933, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x94d,
0x939, 0x930, 0x94d, 0x921, 0x947, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x20, 0x935, 0x947, 0x933, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x94d, 0x939, 0x930,
0x94d, 0x921, 0x947, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20,
0x935, 0x947, 0x933, 0x91a, 0x93e, 0x92e, 0x94b, 0x930, 0x94b, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x945, 0x925, 0x92e,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x91a,
0x945, 0x925, 0x92e, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930,
0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x93f, 0x932, 0x940, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x93f,
0x932, 0x940, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935,
0x947, 0x933, 0x91a, 0x940, 0x928, 0x940, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x940,
0x928, 0x940, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947,
0x933, 0x91a, 0x940, 0x928, 0x940, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a,
0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x916, 0x94d, 0x930,
0x93f, 0x938, 0x92e, 0x938, 0x20, 0x92c, 0x947, 0x91f, 0x20, 0x935, 0x947, 0x933,
0x915, 0x949, 0x915, 0x94b, 0x938, 0x20, 0x92c, 0x947, 0x91f, 0x947, 0x20, 0x935,
0x947, 0x933, 0x915, 0x94b, 0x932, 0x902, 0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94b, 0x932,
0x902, 0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933,
0x940, 0x20, 0x935, 0x947, 0x933, 0x915, 0x941, 0x915, 0x20, 0x92c, 0x947, 0x91f,
0x947, 0x20, 0x935, 0x947, 0x933, 0x915, 0x941, 0x915, 0x20, 0x92c, 0x947, 0x91f,
0x947, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933,
0x915, 0x941, 0x915, 0x20, 0x92c, 0x947, 0x91f, 0x947, 0x20, 0x905, 0x930, 0x94d,
0x927, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947,
0x933, 0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x915,
0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x20, 0x935, 0x947, 0x933, 0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x20, 0x938,
0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935,
0x947, 0x933, 0x921, 0x947, 0x935, 0x94d, 0x939, 0x93f, 0x938, 0x20, 0x935, 0x947,
0x933, 0x921, 0x94d, 0x92f, 0x941, 0x92e, 0x949, 0x928, 0x94d, 0x91f, 0x2d, 0x921,
0x94d, 0x92f, 0x941, 0x930, 0x94d, 0x935, 0x93f, 0x932, 0x20, 0x935, 0x947, 0x933,
0x92a, 0x942, 0x930, 0x94d, 0x935, 0x20, 0x924, 0x93f, 0x92e, 0x94b, 0x930, 0x20,
0x935, 0x947, 0x933, 0x907, 0x938, 0x94d, 0x91f, 0x930, 0x20, 0x92c, 0x947, 0x91f,
0x20, 0x935, 0x947, 0x933, 0x907, 0x938, 0x94d, 0x91f, 0x930, 0x20, 0x92c, 0x947,
0x91f, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933,
0x907, 0x938, 0x94d, 0x91f, 0x930, 0x20, 0x92c, 0x947, 0x91f, 0x20, 0x909, 0x928,
0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x927, 0x94d,
0x200d, 0x92f, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20,
0x935, 0x947, 0x933, 0x92e, 0x927, 0x94d, 0x200d, 0x92f, 0x20, 0x92f, 0x941, 0x930,
0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x20, 0x935, 0x947, 0x933, 0x92e, 0x927, 0x94d, 0x200d, 0x92f, 0x20, 0x92f, 0x941,
0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e,
0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x20,
0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x935, 0x947, 0x933,
0x92a, 0x942, 0x930, 0x94d, 0x935, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x93f,
0x92f, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947,
0x933, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a,
0x93f, 0x92f, 0x928, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20,
0x935, 0x947, 0x933, 0x905, 0x917, 0x94d, 0x930, 0x2d, 0x92a, 0x94c, 0x930, 0x94d,
0x935, 0x93e, 0x924, 0x94d, 0x92f, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x940,
0x92f, 0x928, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e,
0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x935, 0x947,
0x933, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x92f, 0x941, 0x930, 0x94b,
0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20,
0x935, 0x947, 0x933, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x92f, 0x941,
0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e,
0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x949, 0x915, 0x932, 0x902, 0x921,
0x20, 0x92c, 0x947, 0x91f, 0x947, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x949, 0x915,
0x932, 0x902, 0x921, 0x20, 0x92c, 0x947, 0x91f, 0x947, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x949, 0x915, 0x932, 0x902,
0x921, 0x20, 0x92c, 0x947, 0x91f, 0x947, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e,
0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x93f, 0x91c, 0x940, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x93f, 0x91c,
0x940, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947,
0x933, 0x92b, 0x94d, 0x930, 0x947, 0x902, 0x91a, 0x20, 0x926, 0x915, 0x94d, 0x937,
0x93f, 0x923, 0x20, 0x906, 0x923, 0x93f, 0x20, 0x905, 0x902, 0x91f, 0x93e, 0x930,
0x94d, 0x915, 0x94d, 0x91f, 0x93f, 0x915, 0x20, 0x935, 0x947, 0x933, 0x917, 0x945,
0x932, 0x93e, 0x92a, 0x93e, 0x917, 0x94b, 0x938, 0x20, 0x935, 0x947, 0x933, 0x917,
0x945, 0x92e, 0x94d, 0x92c, 0x93f, 0x92f, 0x930, 0x20, 0x935, 0x947, 0x933, 0x91c,
0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f,
0x92f, 0x93e, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935,
0x947, 0x933, 0x917, 0x93f, 0x932, 0x94d, 0x92c, 0x930, 0x94d, 0x91f, 0x20, 0x92c,
0x947, 0x91f, 0x947, 0x20, 0x935, 0x947, 0x933, 0x917, 0x94d, 0x930, 0x940, 0x928,
0x93f, 0x91a, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947,
0x933, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928,
0x932, 0x901, 0x921, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x942, 0x930, 0x94d, 0x935,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x901, 0x921, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x942, 0x930, 0x94d,
0x935, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x901, 0x921, 0x20, 0x909,
0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x936,
0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x901,
0x921, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x901, 0x921, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x936, 0x94d, 0x91a, 0x93f,
0x92e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x901, 0x921, 0x20, 0x909,
0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x917, 0x941,
0x906, 0x92e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x935, 0x947, 0x933,
0x916, 0x93e, 0x921, 0x940, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20,
0x935, 0x947, 0x933, 0x939, 0x935, 0x93e, 0x908, 0x2d, 0x905, 0x932, 0x942, 0x936,
0x928, 0x20, 0x935, 0x947, 0x933, 0x939, 0x935, 0x93e, 0x908, 0x2d, 0x905, 0x932,
0x942, 0x936, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935,
0x947, 0x933, 0x939, 0x935, 0x93e, 0x908, 0x2d, 0x905, 0x932, 0x942, 0x936, 0x928,
0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936,
0x20, 0x935, 0x947, 0x933, 0x939, 0x93e, 0x901, 0x917, 0x20, 0x915, 0x93e, 0x901,
0x917, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933,
0x939, 0x93e, 0x901, 0x917, 0x20, 0x915, 0x93e, 0x901, 0x917, 0x20, 0x909, 0x928,
0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x939, 0x94b, 0x935,
0x94d, 0x939, 0x94d, 0x921, 0x20, 0x935, 0x947, 0x933, 0x939, 0x94b, 0x935, 0x94d,
0x939, 0x94d, 0x921, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935,
0x947, 0x933, 0x939, 0x94b, 0x935, 0x94d, 0x939, 0x94d, 0x921, 0x20, 0x909, 0x928,
0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92d, 0x93e, 0x930,
0x924, 0x940, 0x92f, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935,
0x947, 0x933, 0x939, 0x93f, 0x902, 0x926, 0x20, 0x92e, 0x939, 0x93e, 0x938, 0x93e,
0x917, 0x930, 0x20, 0x935, 0x947, 0x933, 0x907, 0x902, 0x921, 0x94b, 0x91a, 0x93e,
0x92f, 0x928, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x927, 0x94d, 0x200d, 0x92f,
0x20, 0x907, 0x902, 0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20,
0x935, 0x947, 0x933, 0x92a, 0x94c, 0x930, 0x94d, 0x935, 0x93e, 0x924, 0x94d, 0x92f,
0x20, 0x907, 0x902, 0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20,
0x935, 0x947, 0x933, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x907,
0x902, 0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x935, 0x947,
0x933, 0x907, 0x930, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x907, 0x930, 0x93e,
0x923, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933,
0x907, 0x930, 0x93e, 0x923, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d,
0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x907, 0x930, 0x94d, 0x915,
0x941, 0x924, 0x94d, 0x938, 0x915, 0x20, 0x935, 0x947, 0x933, 0x907, 0x930, 0x94d,
0x915, 0x941, 0x924, 0x94d, 0x938, 0x915, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x20, 0x935, 0x947, 0x933, 0x907, 0x930, 0x94d, 0x915, 0x941, 0x924, 0x94d,
0x938, 0x915, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935,
0x947, 0x933, 0x907, 0x938, 0x94d, 0x930, 0x93e, 0x92f, 0x932, 0x907, 0x938, 0x94d,
0x930, 0x93e, 0x92f, 0x932, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20,
0x935, 0x947, 0x933, 0x907, 0x938, 0x94d, 0x930, 0x93e, 0x92f, 0x932, 0x20, 0x938,
0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935,
0x947, 0x933, 0x91c, 0x92a, 0x93e, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x20, 0x935, 0x947, 0x933, 0x91c, 0x92a, 0x93e, 0x928, 0x20, 0x938, 0x942,
0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947,
0x933, 0x92a, 0x947, 0x91f, 0x94d, 0x930, 0x94b, 0x92a, 0x93e, 0x935, 0x94d, 0x939,
0x932, 0x94b, 0x938, 0x94d, 0x915, 0x2d, 0x20, 0x915, 0x93e, 0x92e, 0x91a, 0x93e,
0x91f, 0x94d, 0x938, 0x94d, 0x915, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x947,
0x91f, 0x94d, 0x930, 0x94b, 0x92a, 0x93e, 0x935, 0x94d, 0x939, 0x932, 0x94b, 0x938,
0x94d, 0x915, 0x2d, 0x20, 0x915, 0x93e, 0x92e, 0x91a, 0x93e, 0x91f, 0x94d, 0x938,
0x94d, 0x915, 0x940, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x935, 0x947,
0x933, 0x92a, 0x947, 0x91f, 0x94d, 0x930, 0x94b, 0x92a, 0x93e, 0x935, 0x94d, 0x939,
0x932, 0x94b, 0x938, 0x94d, 0x915, 0x2d, 0x20, 0x915, 0x93e, 0x92e, 0x91a, 0x93e,
0x91f, 0x94d, 0x938, 0x94d, 0x915, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x935, 0x947, 0x933, 0x92a,
0x942, 0x930, 0x94d, 0x935, 0x20, 0x915, 0x91d, 0x93e, 0x915, 0x938, 0x94d, 0x924,
0x93e, 0x928, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e,
0x20, 0x915, 0x91d, 0x93e, 0x915, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x935,
0x947, 0x933, 0x915, 0x94b, 0x930, 0x93f, 0x92f, 0x928, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94b, 0x930, 0x93f, 0x92f,
0x928, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e,
0x936, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94b, 0x938, 0x94d, 0x930, 0x93e, 0x908,
0x20, 0x935, 0x947, 0x933, 0x915, 0x94d, 0x930, 0x93e, 0x938, 0x94d, 0x928, 0x94b,
0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94d, 0x930, 0x93e, 0x938, 0x94d,
0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x915, 0x94d, 0x938, 0x20, 0x909, 0x928,
0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x915, 0x93f, 0x930,
0x917, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x935, 0x947, 0x933, 0x932,
0x902, 0x915, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x932, 0x93e, 0x907, 0x928, 0x20,
0x92c, 0x947, 0x91f, 0x947, 0x20, 0x935, 0x947, 0x933, 0x932, 0x949, 0x930, 0x94d,
0x921, 0x20, 0x939, 0x94b, 0x935, 0x947, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x20, 0x935, 0x947, 0x933, 0x932, 0x949, 0x930, 0x94d, 0x921, 0x20, 0x939,
0x94b, 0x935, 0x947, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930,
0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x915, 0x93e, 0x90a, 0x20,
0x935, 0x947, 0x933, 0x92e, 0x915, 0x93e, 0x90a, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x935, 0x947, 0x933, 0x92e, 0x915, 0x93e, 0x90a, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x935,
0x947, 0x933, 0x92e, 0x945, 0x917, 0x93e, 0x921, 0x928, 0x20, 0x935, 0x947, 0x933,
0x92e, 0x945, 0x917, 0x93e, 0x921, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x945, 0x917, 0x93e, 0x921, 0x928, 0x20,
0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92e,
0x93e, 0x932, 0x926, 0x93f, 0x935, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x93e, 0x930,
0x94d, 0x915, 0x94d, 0x935, 0x947, 0x938, 0x93e, 0x938, 0x20, 0x935, 0x947, 0x933,
0x92e, 0x93e, 0x930, 0x94d, 0x936, 0x932, 0x20, 0x92c, 0x947, 0x91f, 0x947, 0x20,
0x935, 0x947, 0x933, 0x92e, 0x949, 0x930, 0x940, 0x936, 0x938, 0x20, 0x935, 0x947,
0x933, 0x92e, 0x949, 0x930, 0x940, 0x936, 0x938, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x949, 0x930, 0x940, 0x936, 0x938,
0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933,
0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x94b, 0x20, 0x92a, 0x945, 0x938,
0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x947, 0x915, 0x94d,
0x938, 0x93f, 0x915, 0x94b, 0x20, 0x92a, 0x945, 0x938, 0x93f, 0x92b, 0x93f, 0x915,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92e,
0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x94b, 0x20, 0x92a, 0x945, 0x938, 0x93f,
0x92b, 0x93f, 0x915, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930,
0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x909, 0x932, 0x93e, 0x928, 0x20,
0x92c, 0x93e, 0x91f, 0x94b, 0x930, 0x20, 0x935, 0x947, 0x933, 0x909, 0x932, 0x93e,
0x928, 0x20, 0x92c, 0x93e, 0x91f, 0x94b, 0x930, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x909, 0x932, 0x93e, 0x928, 0x20, 0x92c,
0x93e, 0x91f, 0x94b, 0x930, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940,
0x20, 0x935, 0x947, 0x933, 0x92e, 0x949, 0x938, 0x94d, 0x915, 0x94b, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x949, 0x938,
0x94d, 0x915, 0x94b, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20,
0x935, 0x947, 0x933, 0x928, 0x909, 0x930, 0x941, 0x20, 0x935, 0x947, 0x933, 0x928,
0x94d, 0x92f, 0x942, 0x20, 0x915, 0x945, 0x932, 0x947, 0x921, 0x94b, 0x928, 0x93f,
0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x915,
0x945, 0x932, 0x947, 0x921, 0x94b, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x928, 0x94d, 0x92f, 0x942,
0x20, 0x915, 0x945, 0x932, 0x947, 0x921, 0x94b, 0x928, 0x93f, 0x92f, 0x93e, 0x20,
0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x928,
0x94d, 0x92f, 0x942, 0x91d, 0x940, 0x932, 0x902, 0x921, 0x20, 0x935, 0x947, 0x933,
0x928, 0x94d, 0x92f, 0x942, 0x91d, 0x940, 0x932, 0x902, 0x921, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x928, 0x94d, 0x92f, 0x942,
0x91d, 0x940, 0x932, 0x902, 0x921, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a,
0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x928, 0x94d, 0x92f,
0x942, 0x20, 0x92b, 0x93e, 0x909, 0x902, 0x921, 0x932, 0x902, 0x921, 0x20, 0x935,
0x947, 0x933, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x92b, 0x93e, 0x909, 0x902, 0x921,
0x932, 0x902, 0x921, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935,
0x947, 0x933, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x92b, 0x93e, 0x909, 0x902, 0x921,
0x932, 0x902, 0x921, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930,
0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x928, 0x949, 0x930, 0x92b, 0x94b,
0x915, 0x20, 0x92c, 0x947, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x928, 0x949, 0x930,
0x92b, 0x94b, 0x915, 0x20, 0x92c, 0x947, 0x91f, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x928, 0x949, 0x930, 0x92b, 0x94b, 0x915,
0x20, 0x92c, 0x947, 0x91f, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d,
0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x930, 0x94d, 0x928,
0x93e, 0x902, 0x921, 0x94b, 0x20, 0x926, 0x940, 0x20, 0x928, 0x94b, 0x930, 0x94b,
0x928, 0x94d, 0x939, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20,
0x935, 0x947, 0x933, 0x92b, 0x930, 0x94d, 0x928, 0x93e, 0x902, 0x921, 0x94b, 0x20,
0x926, 0x940, 0x20, 0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x20,
0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x909,
0x924, 0x94d, 0x924, 0x930, 0x20, 0x92e, 0x930, 0x93f, 0x928, 0x93e, 0x20, 0x92c,
0x947, 0x91f, 0x947, 0x20, 0x935, 0x947, 0x933, 0x928, 0x94b, 0x935, 0x94b, 0x938,
0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x935, 0x947, 0x933,
0x928, 0x94b, 0x935, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d,
0x915, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933,
0x928, 0x94b, 0x935, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d,
0x915, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947,
0x933, 0x913, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x913, 0x92e, 0x94d, 0x938, 0x94d, 0x915,
0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933,
0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x935, 0x947,
0x933, 0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x93e, 0x915,
0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e,
0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x93e, 0x92a, 0x941, 0x906, 0x20,
0x928, 0x94d, 0x92f, 0x942, 0x20, 0x917, 0x93f, 0x928, 0x940, 0x20, 0x935, 0x947,
0x933, 0x92a, 0x945, 0x930, 0x93e, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x935, 0x947,
0x933, 0x92a, 0x945, 0x930, 0x93e, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x945, 0x930, 0x93e,
0x917, 0x94d, 0x935, 0x947, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940,
0x20, 0x935, 0x947, 0x933, 0x92a, 0x947, 0x930, 0x941, 0x20, 0x935, 0x947, 0x933,
0x92a, 0x947, 0x930, 0x941, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20,
0x935, 0x947, 0x933, 0x92a, 0x947, 0x930, 0x941, 0x20, 0x909, 0x928, 0x94d, 0x939,
0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x93f, 0x932, 0x93f, 0x92a,
0x93e, 0x907, 0x928, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x93f, 0x932, 0x93f, 0x92a,
0x93e, 0x907, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935,
0x947, 0x933, 0x92b, 0x93f, 0x932, 0x93f, 0x92a, 0x93e, 0x907, 0x928, 0x20, 0x909,
0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x200d, 0x92b,
0x94b, 0x928, 0x93f, 0x915, 0x94d, 0x938, 0x20, 0x92c, 0x947, 0x91f, 0x947, 0x20,
0x935, 0x947, 0x933, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x92a, 0x93f, 0x92f, 0x930,
0x20, 0x906, 0x923, 0x93f, 0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932,
0x94b, 0x928, 0x20, 0x935, 0x947, 0x933, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x92a,
0x93f, 0x92f, 0x930, 0x947, 0x20, 0x906, 0x923, 0x93f, 0x20, 0x92e, 0x93f, 0x915,
0x94d, 0x935, 0x947, 0x932, 0x94b, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x20, 0x935, 0x947, 0x933, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x92a, 0x93f,
0x92f, 0x930, 0x947, 0x20, 0x906, 0x923, 0x93f, 0x20, 0x92e, 0x93f, 0x915, 0x94d,
0x935, 0x947, 0x932, 0x94b, 0x928, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a,
0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x93f, 0x91f,
0x915, 0x948, 0x930, 0x94d, 0x928, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x94b, 0x928,
0x945, 0x92a, 0x947, 0x20, 0x935, 0x947, 0x933, 0x915, 0x93c, 0x93f, 0x91d, 0x940,
0x932, 0x94b, 0x930, 0x94d, 0x921, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x915, 0x93c,
0x93f, 0x91d, 0x940, 0x932, 0x94b, 0x930, 0x94d, 0x921, 0x93e, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x935, 0x947, 0x933, 0x915, 0x93c, 0x93f, 0x91d, 0x940,
0x932, 0x94b, 0x930, 0x94d, 0x921, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x935, 0x947, 0x933, 0x938,
0x916, 0x932, 0x93f, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20,
0x935, 0x947, 0x933, 0x938, 0x916, 0x932, 0x93f, 0x928, 0x20, 0x909, 0x928, 0x94d,
0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x938, 0x92e, 0x93e, 0x930,
0x93e, 0x20, 0x935, 0x947, 0x933, 0x938, 0x93e, 0x92e, 0x930, 0x93e, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x935, 0x947, 0x933, 0x938, 0x93e, 0x92e, 0x930,
0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x935, 0x947, 0x933, 0x938, 0x93e, 0x92e, 0x94b, 0x906, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x938, 0x93e,
0x92e, 0x94b, 0x906, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930,
0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x938, 0x93f, 0x902, 0x917, 0x93e,
0x92a, 0x942, 0x930, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935,
0x947, 0x933, 0x938, 0x94b, 0x932, 0x94b, 0x92e, 0x949, 0x928, 0x20, 0x92c, 0x947,
0x91f, 0x947, 0x20, 0x935, 0x947, 0x933, 0x924, 0x948, 0x92a, 0x947, 0x908, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x924, 0x948,
0x92a, 0x947, 0x908, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930,
0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x924, 0x93e, 0x91c, 0x93f, 0x915,
0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x935, 0x947, 0x933, 0x91f, 0x94b,
0x915, 0x947, 0x932, 0x93e, 0x90a, 0x20, 0x935, 0x947, 0x933, 0x91f, 0x94b, 0x902,
0x917, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947,
0x933, 0x91f, 0x94b, 0x902, 0x917, 0x93e, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e,
0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x942, 0x915, 0x20, 0x935, 0x947,
0x933, 0x924, 0x941, 0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928, 0x93f, 0x938, 0x94d,
0x924, 0x93e, 0x928, 0x20, 0x935, 0x947, 0x933, 0x924, 0x941, 0x930, 0x94d, 0x915,
0x92e, 0x947, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x924, 0x941, 0x930, 0x94d,
0x915, 0x92e, 0x947, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x909,
0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x924, 0x941,
0x935, 0x93e, 0x932, 0x942, 0x20, 0x935, 0x947, 0x933, 0x909, 0x930, 0x941, 0x917,
0x94d, 0x935, 0x947, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935,
0x947, 0x933, 0x909, 0x930, 0x941, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x909, 0x928,
0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x909, 0x91d, 0x92c,
0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x935, 0x947, 0x933,
0x909, 0x91d, 0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x909, 0x91d,
0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x909, 0x928,
0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x935, 0x93e, 0x928,
0x941, 0x906, 0x924, 0x941, 0x20, 0x935, 0x947, 0x933, 0x935, 0x93e, 0x928, 0x941,
0x906, 0x924, 0x941, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935,
0x947, 0x933, 0x935, 0x93e, 0x928, 0x941, 0x906, 0x924, 0x941, 0x20, 0x909, 0x928,
0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x935, 0x94d, 0x939,
0x947, 0x928, 0x947, 0x91d, 0x941, 0x90f, 0x932, 0x93e, 0x20, 0x935, 0x947, 0x933,
0x935, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x935, 0x94b, 0x938, 0x94d, 0x924, 0x94b,
0x915, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933,
0x935, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x935, 0x94b, 0x938, 0x94d, 0x924, 0x94b,
0x915, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947,
0x933, 0x935, 0x94d, 0x939, 0x94b, 0x932, 0x94d, 0x917, 0x94b, 0x917, 0x94d, 0x930,
0x93e, 0x921, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947,
0x933, 0x935, 0x94d, 0x939, 0x94b, 0x932, 0x94d, 0x917, 0x94b, 0x917, 0x94d, 0x930,
0x93e, 0x921, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935,
0x947, 0x933, 0x935, 0x94d, 0x939, 0x94b, 0x938, 0x94d, 0x91f, 0x949, 0x915, 0x20,
0x935, 0x947, 0x933, 0x935, 0x947, 0x915, 0x20, 0x92c, 0x947, 0x91f, 0x20, 0x935,
0x947, 0x933, 0x935, 0x949, 0x932, 0x93f, 0x938, 0x20, 0x906, 0x923, 0x93f, 0x20,
0x92b, 0x941, 0x91f, 0x941, 0x928, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x92f, 0x93e,
0x915, 0x941, 0x924, 0x94d, 0x938, 0x915, 0x20, 0x935, 0x947, 0x933, 0x92f, 0x93e,
0x915, 0x941, 0x924, 0x94d, 0x938, 0x915, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x20, 0x935, 0x947, 0x933, 0x92f, 0x93e, 0x915, 0x941, 0x924, 0x94d, 0x938,
0x915, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947,
0x933, 0x92f, 0x947, 0x915, 0x924, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917,
0x20, 0x935, 0x947, 0x933, 0x92f, 0x947, 0x915, 0x924, 0x930, 0x93f, 0x928, 0x92c,
0x930, 0x94d, 0x917, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935,
0x947, 0x933, 0x92f, 0x947, 0x915, 0x924, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d,
0x917, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947,
0x933, 0x92f, 0x941, 0x915, 0x94b, 0x928, 0x20, 0x935, 0x947, 0x933, 0x410, 0x444,
0x433, 0x430, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x422, 0x4e9, 0x432, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x438, 0x439,
0x43d, 0x20, 0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20, 0x410, 0x444,
0x440, 0x438, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x4e8, 0x43c,
0x43d, 0x4e9, 0x434, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x438, 0x439, 0x43d,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430,
0x433, 0x411, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x410, 0x444, 0x440, 0x438,
0x43a, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x411, 0x430, 0x440, 0x443,
0x443, 0x43d, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x438, 0x439, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433,
0x411, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a,
0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433,
0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433,
0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x410, 0x43b, 0x44f,
0x441, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446,
0x430, 0x433, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x44b, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x410, 0x43c, 0x430,
0x437, 0x43e, 0x43d, 0x44b, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x422, 0x4e9, 0x432, 0x20, 0x446, 0x430, 0x433, 0x422, 0x4e9, 0x432, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433,
0x422, 0x4e9, 0x432, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433,
0x417, 0x4af, 0x4af, 0x43d, 0x20, 0x44d, 0x440, 0x433, 0x438, 0x439, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20, 0x44d, 0x440, 0x433, 0x438,
0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20, 0x44d, 0x440, 0x433, 0x438,
0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x423,
0x443, 0x43b, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x423, 0x443, 0x43b, 0x44b,
0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446,
0x430, 0x433, 0x423, 0x443, 0x43b, 0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b,
0x20, 0x446, 0x430, 0x433, 0x41d, 0x43e, 0x43c, 0x445, 0x43e, 0x43d, 0x20, 0x434,
0x430, 0x43b, 0x430, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41d, 0x43e, 0x43c,
0x445, 0x43e, 0x43d, 0x20, 0x434, 0x430, 0x43b, 0x430, 0x439, 0x43d, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x41d,
0x43e, 0x43c, 0x445, 0x43e, 0x43d, 0x20, 0x434, 0x430, 0x43b, 0x430, 0x439, 0x43d,
0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x410, 0x43f, 0x438,
0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x410, 0x43f, 0x438,
0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x410, 0x43f, 0x438, 0x430, 0x433, 0x438,
0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x410,
0x440, 0x430, 0x431, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x410, 0x440, 0x430,
0x431, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x446, 0x430, 0x433, 0x410, 0x440, 0x430, 0x431, 0x44b, 0x43d, 0x20, 0x437,
0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x410, 0x440, 0x433, 0x435, 0x43d,
0x442, 0x438, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x410, 0x440, 0x433, 0x435,
0x43d, 0x442, 0x438, 0x43d, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442,
0x438, 0x43d, 0x44b, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433,
0x411, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d,
0x442, 0x438, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x411, 0x430, 0x440, 0x443,
0x443, 0x43d, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x44b,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430,
0x433, 0x411, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x410, 0x440, 0x433, 0x435,
0x43d, 0x442, 0x438, 0x43d, 0x44b, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446,
0x430, 0x433, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x439, 0x43d, 0x20, 0x446,
0x430, 0x433, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x439, 0x43d, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x410,
0x440, 0x43c, 0x435, 0x43d, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b,
0x20, 0x446, 0x430, 0x433, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x44b, 0x43d,
0x20, 0x446, 0x430, 0x433, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x44b, 0x43d,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430,
0x433, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x44b, 0x43d, 0x20, 0x437, 0x443,
0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x422, 0x4e9, 0x432, 0x20, 0x410, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433,
0x422, 0x4e9, 0x432, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x446, 0x430, 0x433, 0x422, 0x4e9, 0x432, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446,
0x430, 0x433, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439, 0x43d,
0x20, 0x442, 0x4e9, 0x432, 0x20, 0x431, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20,
0x44d, 0x440, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x410, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x442, 0x4e9, 0x432,
0x20, 0x431, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x44d, 0x440, 0x433, 0x438,
0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x446, 0x430, 0x433, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439,
0x43d, 0x20, 0x442, 0x4e9, 0x432, 0x20, 0x431, 0x430, 0x440, 0x443, 0x443, 0x43d,
0x20, 0x44d, 0x440, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b,
0x20, 0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x410,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x4af,
0x4af, 0x43d, 0x20, 0x44d, 0x440, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x410, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x4af, 0x4af,
0x43d, 0x20, 0x44d, 0x440, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d,
0x44b, 0x20, 0x446, 0x430, 0x433, 0x411, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x446,
0x430, 0x433, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439, 0x43d,
0x20, 0x431, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x44d, 0x440, 0x433, 0x438,
0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x446, 0x430, 0x433, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439,
0x43d, 0x20, 0x431, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x44d, 0x440, 0x433,
0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433,
0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x436, 0x430, 0x43d, 0x44b, 0x20,
0x446, 0x430, 0x433, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x436, 0x430,
0x43d, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x446, 0x430, 0x433, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x436, 0x430,
0x43d, 0x44b, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x410,
0x437, 0x43e, 0x440, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x410, 0x437, 0x43e,
0x440, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x446, 0x430, 0x433, 0x410, 0x437, 0x43e, 0x440, 0x44b, 0x43d, 0x20, 0x437,
0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x411, 0x430, 0x43d, 0x433, 0x43b,
0x430, 0x434, 0x435, 0x448, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x411,
0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x438, 0x439, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433,
0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x438, 0x439, 0x43d,
0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x411, 0x443, 0x442,
0x430, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x411, 0x43e, 0x43b, 0x438, 0x432,
0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x411, 0x440, 0x430, 0x437, 0x438,
0x43b, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x411, 0x440, 0x430, 0x437,
0x438, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b,
0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433,
0x411, 0x440, 0x443, 0x43d, 0x435, 0x439, 0x20, 0x414, 0x430, 0x440, 0x443, 0x441,
0x441, 0x430, 0x43b, 0x430, 0x43c, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41a,
0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x438, 0x439, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434,
0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x446, 0x430, 0x433, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440,
0x434, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x427, 0x430, 0x43c, 0x43e, 0x440, 0x440, 0x43e, 0x433, 0x438, 0x439, 0x43d,
0x20, 0x446, 0x430, 0x433, 0x427, 0x430, 0x442, 0x435, 0x43c, 0x44b, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x427, 0x430, 0x442, 0x435, 0x43c, 0x44b, 0x43d, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x427,
0x430, 0x442, 0x435, 0x43c, 0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20,
0x446, 0x430, 0x433, 0x427, 0x438, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430,
0x433, 0x427, 0x438, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x427, 0x438, 0x43b, 0x438,
0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x425,
0x44f, 0x442, 0x430, 0x434, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x425, 0x44f,
0x442, 0x430, 0x434, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x425, 0x44f, 0x442, 0x430, 0x434, 0x44b,
0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41a, 0x440,
0x438, 0x441, 0x43c, 0x430, 0x441, 0x20, 0x430, 0x440, 0x43b, 0x44b, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x20, 0x430, 0x440, 0x43b,
0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431,
0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431,
0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x446, 0x430, 0x433, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x44b, 0x43d, 0x20,
0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41a, 0x4af, 0x4af, 0x43a,
0x438, 0x439, 0x43d, 0x20, 0x430, 0x440, 0x43b, 0x443, 0x443, 0x434, 0x44b, 0x43d,
0x20, 0x446, 0x430, 0x433, 0x41a, 0x4af, 0x4af, 0x43a, 0x438, 0x439, 0x43d, 0x20,
0x430, 0x440, 0x43b, 0x443, 0x443, 0x434, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x41a, 0x4af, 0x4af,
0x43a, 0x438, 0x439, 0x43d, 0x20, 0x430, 0x440, 0x43b, 0x443, 0x443, 0x434, 0x44b,
0x43d, 0x20, 0x445, 0x430, 0x433, 0x430, 0x441, 0x20, 0x437, 0x443, 0x43d, 0x44b,
0x20, 0x446, 0x430, 0x433, 0x41a, 0x443, 0x431, 0x44b, 0x43d, 0x20, 0x446, 0x430,
0x433, 0x41a, 0x443, 0x431, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x41a, 0x443, 0x431, 0x44b, 0x43d,
0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x414, 0x44d, 0x432,
0x438, 0x441, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x414, 0x44e, 0x43c,
0x43e, 0x43d, 0x20, 0x434, 0x2019, 0x42e, 0x440, 0x432, 0x438, 0x43b, 0x438, 0x439,
0x43d, 0x20, 0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20, 0x422, 0x438,
0x43c, 0x43e, 0x440, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af,
0x43d, 0x20, 0x418, 0x441, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x43d, 0x20, 0x446,
0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20, 0x418, 0x441, 0x43b, 0x430, 0x43d,
0x434, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20, 0x418, 0x441, 0x43b,
0x430, 0x43d, 0x434, 0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446,
0x430, 0x433, 0x42d, 0x43a, 0x432, 0x430, 0x434, 0x43e, 0x440, 0x44b, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x422, 0x4e9, 0x432, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f,
0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x422, 0x4e9, 0x432, 0x20, 0x415, 0x432,
0x440, 0x43e, 0x43f, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x422, 0x4e9, 0x432, 0x20, 0x415, 0x432,
0x440, 0x43e, 0x43f, 0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446,
0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f,
0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20, 0x415,
0x432, 0x440, 0x43e, 0x43f, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20,
0x415, 0x432, 0x440, 0x43e, 0x43f, 0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b,
0x20, 0x446, 0x430, 0x433, 0x410, 0x43b, 0x441, 0x20, 0x434, 0x43e, 0x440, 0x43d,
0x43e, 0x434, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x44b, 0x43d, 0x20, 0x446,
0x430, 0x433, 0x411, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x415, 0x432, 0x440,
0x43e, 0x43f, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x411, 0x430, 0x440, 0x443,
0x443, 0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x44b, 0x43d, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x411,
0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x44b,
0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x424, 0x43e,
0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x438, 0x439, 0x43d, 0x20, 0x430, 0x440,
0x43b, 0x443, 0x443, 0x434, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x424, 0x43e,
0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x438, 0x439, 0x43d, 0x20, 0x430, 0x440,
0x43b, 0x443, 0x443, 0x434, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x424, 0x43e, 0x43b, 0x43a, 0x43b,
0x435, 0x43d, 0x434, 0x438, 0x439, 0x43d, 0x20, 0x430, 0x440, 0x43b, 0x443, 0x443,
0x434, 0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433,
0x424, 0x438, 0x436, 0x438, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433,
0x424, 0x438, 0x436, 0x438, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x424, 0x438, 0x436,
0x438, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446,
0x430, 0x433, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x44b, 0x43d, 0x20, 0x413, 0x432,
0x438, 0x430, 0x43d, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433,
0x424, 0x440, 0x430, 0x43d, 0x446, 0x44b, 0x43d, 0x20, 0x4e8, 0x43c, 0x43d, 0x4e9,
0x434, 0x20, 0x431, 0x430, 0x20, 0x410, 0x43d, 0x442, 0x430, 0x440, 0x43a, 0x442,
0x438, 0x434, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x413, 0x430, 0x43b,
0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433,
0x413, 0x430, 0x43c, 0x431, 0x44c, 0x435, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446,
0x430, 0x433, 0x413, 0x4af, 0x440, 0x436, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430,
0x433, 0x413, 0x4af, 0x440, 0x436, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x413, 0x4af, 0x440,
0x436, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x413, 0x438, 0x43b, 0x44c, 0x431, 0x435, 0x440, 0x442, 0x438, 0x439, 0x43d,
0x20, 0x430, 0x440, 0x43b, 0x443, 0x443, 0x434, 0x44b, 0x43d, 0x20, 0x446, 0x430,
0x433, 0x413, 0x440, 0x438, 0x43d, 0x432, 0x438, 0x447, 0x438, 0x439, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20, 0x413, 0x440, 0x435, 0x43d,
0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x417, 0x4af,
0x4af, 0x43d, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x44b,
0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446,
0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b,
0x430, 0x43d, 0x434, 0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446,
0x430, 0x433, 0x411, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x413, 0x440, 0x435,
0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x411,
0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430,
0x43d, 0x434, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x446, 0x430, 0x433, 0x411, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20,
0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x43d, 0x20, 0x437,
0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41f, 0x435, 0x440, 0x441, 0x438,
0x439, 0x43d, 0x20, 0x431, 0x443, 0x43b, 0x430, 0x43d, 0x433, 0x438, 0x439, 0x43d,
0x20, 0x446, 0x430, 0x433, 0x413, 0x430, 0x439, 0x430, 0x43d, 0x430, 0x433, 0x438,
0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x425, 0x430, 0x432, 0x430, 0x439, 0x2d,
0x410, 0x43b, 0x435, 0x443, 0x442, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x425,
0x430, 0x432, 0x430, 0x439, 0x2d, 0x410, 0x43b, 0x435, 0x443, 0x442, 0x44b, 0x43d,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430,
0x433, 0x425, 0x430, 0x432, 0x430, 0x439, 0x2d, 0x410, 0x43b, 0x435, 0x443, 0x442,
0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x425,
0x43e, 0x43d, 0x433, 0x20, 0x41a, 0x43e, 0x43d, 0x433, 0x438, 0x439, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x425, 0x43e, 0x43d, 0x433, 0x20, 0x41a, 0x43e, 0x43d, 0x433,
0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x446, 0x430, 0x433, 0x425, 0x43e, 0x43d, 0x433, 0x20, 0x41a, 0x43e, 0x43d,
0x433, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x425, 0x43e, 0x432, 0x434, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x425,
0x43e, 0x432, 0x434, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x425, 0x43e, 0x432, 0x434, 0x44b, 0x43d,
0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x42d, 0x43d, 0x44d,
0x442, 0x445, 0x44d, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x42d,
0x43d, 0x44d, 0x442, 0x445, 0x44d, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x434, 0x430,
0x43b, 0x430, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x42d, 0x43d, 0x44d, 0x442,
0x445, 0x44d, 0x433, 0x2d, 0x425, 0x44f, 0x442, 0x430, 0x434, 0x44b, 0x43d, 0x20,
0x445, 0x43e, 0x439, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x422,
0x4e9, 0x432, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x439,
0x43d, 0x20, 0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20, 0x418, 0x43d,
0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433,
0x411, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d,
0x435, 0x437, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x418, 0x440, 0x430,
0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x418, 0x440, 0x430, 0x43d, 0x44b, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433,
0x418, 0x440, 0x430, 0x43d, 0x44b, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446,
0x430, 0x433, 0x42d, 0x440, 0x445, 0x4af, 0x4af, 0x433, 0x438, 0x439, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x42d, 0x440, 0x445, 0x4af, 0x4af, 0x433, 0x438, 0x439, 0x43d,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430,
0x433, 0x42d, 0x440, 0x445, 0x4af, 0x4af, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x437,
0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x418, 0x437, 0x440, 0x430, 0x438,
0x43b, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x418, 0x437, 0x440, 0x430,
0x438, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x418, 0x437, 0x440, 0x430, 0x438, 0x43b,
0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433,
0x42f, 0x43f, 0x43e, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x42f, 0x43f, 0x43e,
0x43d, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x446, 0x430, 0x433, 0x42f, 0x43f, 0x43e, 0x43d, 0x44b, 0x20, 0x437, 0x443, 0x43d,
0x44b, 0x20, 0x446, 0x430, 0x433, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442,
0x430, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20,
0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x44b, 0x20, 0x446,
0x430, 0x433, 0x411, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x41a, 0x430, 0x437,
0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x421,
0x43e, 0x43b, 0x43e, 0x43d, 0x433, 0x43e, 0x441, 0x44b, 0x43d, 0x20, 0x446, 0x430,
0x433, 0x421, 0x43e, 0x43b, 0x43e, 0x43d, 0x433, 0x43e, 0x441, 0x44b, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433,
0x421, 0x43e, 0x43b, 0x43e, 0x43d, 0x433, 0x43e, 0x441, 0x44b, 0x43d, 0x20, 0x437,
0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41a, 0x43e, 0x441, 0x440, 0x44d,
0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41a, 0x440, 0x430, 0x441,
0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430,
0x433, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x438,
0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x446, 0x430, 0x433, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441,
0x43a, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x41a, 0x438, 0x440, 0x433, 0x438, 0x437, 0x438, 0x439, 0x43d, 0x20, 0x446,
0x430, 0x433, 0x41b, 0x430, 0x439, 0x43d, 0x20, 0x430, 0x440, 0x43b, 0x443, 0x443,
0x434, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41b, 0x43e, 0x440, 0x434, 0x20,
0x425, 0x43e, 0x443, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41b,
0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x443, 0x433, 0x438, 0x439, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433,
0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x443, 0x433, 0x438, 0x439, 0x43d,
0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41c, 0x430, 0x433,
0x430, 0x434, 0x430, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41c, 0x430, 0x433,
0x430, 0x434, 0x430, 0x43d, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430,
0x43d, 0x44b, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41c,
0x430, 0x43b, 0x430, 0x439, 0x437, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41c,
0x430, 0x43b, 0x44c, 0x434, 0x438, 0x432, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430,
0x433, 0x41c, 0x430, 0x440, 0x43a, 0x435, 0x437, 0x44b, 0x43d, 0x20, 0x430, 0x440,
0x43b, 0x443, 0x443, 0x434, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41c, 0x430,
0x440, 0x448, 0x430, 0x43b, 0x43b, 0x44b, 0x43d, 0x20, 0x430, 0x440, 0x43b, 0x443,
0x443, 0x434, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41c, 0x430, 0x432, 0x440,
0x438, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41c, 0x430, 0x432,
0x440, 0x438, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x41c, 0x430, 0x432, 0x440, 0x438,
0x43a, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x41c, 0x43e, 0x443, 0x441, 0x43e, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433,
0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x2d, 0x41d, 0x43e, 0x43c, 0x445, 0x43e,
0x43d, 0x20, 0x434, 0x430, 0x43b, 0x430, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433,
0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x2d, 0x41d, 0x43e, 0x43c, 0x445, 0x43e,
0x43d, 0x20, 0x434, 0x430, 0x43b, 0x430, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x41c, 0x435, 0x43a,
0x441, 0x438, 0x43a, 0x2d, 0x41d, 0x43e, 0x43c, 0x445, 0x43e, 0x43d, 0x20, 0x434,
0x430, 0x43b, 0x430, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446,
0x430, 0x433, 0x423, 0x43b, 0x430, 0x430, 0x43d, 0x431, 0x430, 0x430, 0x442, 0x430,
0x440, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x423, 0x43b, 0x430, 0x430, 0x43d,
0x431, 0x430, 0x430, 0x442, 0x430, 0x440, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x423, 0x43b, 0x430,
0x430, 0x43d, 0x431, 0x430, 0x430, 0x442, 0x430, 0x440, 0x44b, 0x43d, 0x20, 0x437,
0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41c, 0x43e, 0x441, 0x43a, 0x432,
0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41c, 0x43e, 0x441,
0x43a, 0x432, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x41c, 0x43e, 0x441, 0x43a,
0x432, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20,
0x446, 0x430, 0x433, 0x41c, 0x44c, 0x44f, 0x43d, 0x43c, 0x430, 0x440, 0x44b, 0x43d,
0x20, 0x446, 0x430, 0x433, 0x41d, 0x430, 0x443, 0x440, 0x443, 0x433, 0x438, 0x439,
0x43d, 0x20, 0x446, 0x430, 0x433, 0x411, 0x430, 0x43b, 0x431, 0x44b, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x428, 0x438, 0x43d, 0x44d, 0x20, 0x41a, 0x430, 0x43b, 0x435,
0x434, 0x43e, 0x43d, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x428, 0x438,
0x43d, 0x44d, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x439,
0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446,
0x430, 0x433, 0x428, 0x438, 0x43d, 0x44d, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434,
0x43e, 0x43d, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446,
0x430, 0x433, 0x428, 0x438, 0x43d, 0x44d, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d,
0x434, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x428, 0x438, 0x43d, 0x44d, 0x20,
0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x428, 0x438, 0x43d,
0x44d, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x43d, 0x20, 0x437,
0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41d, 0x44c, 0x44e, 0x2d, 0x424,
0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x44b, 0x43d, 0x20, 0x446,
0x430, 0x433, 0x41d, 0x44c, 0x44e, 0x2d, 0x424, 0x430, 0x443, 0x43d, 0x434, 0x43b,
0x435, 0x43d, 0x434, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x41d, 0x44c, 0x44e, 0x2d, 0x424, 0x430,
0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x44b, 0x43d, 0x20, 0x437, 0x443,
0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41d, 0x438, 0x443, 0x44d, 0x433, 0x438,
0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b,
0x43a, 0x20, 0x430, 0x440, 0x43b, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41d,
0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x430, 0x440, 0x43b, 0x44b, 0x43d,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430,
0x433, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x430, 0x440, 0x43b,
0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x424,
0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d,
0x43e, 0x440, 0x43e, 0x43d, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430,
0x433, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435,
0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433,
0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20,
0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x437,
0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41d, 0x43e, 0x432, 0x43e, 0x441,
0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430,
0x433, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a,
0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x446, 0x430, 0x433, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438,
0x440, 0x441, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20,
0x446, 0x430, 0x433, 0x41e, 0x43c, 0x441, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x446,
0x430, 0x433, 0x41e, 0x43c, 0x441, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x41e, 0x43c,
0x441, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446,
0x430, 0x433, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x44b, 0x20,
0x446, 0x430, 0x433, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x44b,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430,
0x433, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x44b, 0x20, 0x437,
0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41f, 0x430, 0x43b, 0x430, 0x443,
0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41f, 0x430, 0x43f, 0x443,
0x430, 0x20, 0x428, 0x438, 0x43d, 0x44d, 0x20, 0x413, 0x432, 0x438, 0x43d, 0x435,
0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432,
0x430, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41f, 0x430, 0x440, 0x430, 0x433,
0x432, 0x430, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x446, 0x430, 0x433, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430,
0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41f,
0x435, 0x440, 0x443, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41f,
0x435, 0x440, 0x443, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x41f, 0x435, 0x440, 0x443,
0x433, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x438, 0x439, 0x20,
0x446, 0x430, 0x433, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x438,
0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446,
0x430, 0x433, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x438, 0x439,
0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x424, 0x435, 0x43d,
0x438, 0x43a, 0x441, 0x20, 0x430, 0x440, 0x43b, 0x44b, 0x43d, 0x20, 0x446, 0x430,
0x433, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20, 0x431,
0x430, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x44b, 0x20, 0x446,
0x430, 0x433, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20,
0x431, 0x430, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x44b, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433,
0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20, 0x431, 0x430,
0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x44b, 0x20, 0x437, 0x443,
0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41f, 0x438, 0x442, 0x43a, 0x435, 0x440,
0x43d, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41f, 0x43e, 0x43d, 0x430,
0x43f, 0x435, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41f, 0x451,
0x43d, 0x44c, 0x44f, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x420, 0x435, 0x44e,
0x43d, 0x438, 0x43e, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x420, 0x43e, 0x442,
0x435, 0x440, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x421,
0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x421,
0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x421, 0x430, 0x445, 0x430,
0x43b, 0x438, 0x43d, 0x44b, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446,
0x430, 0x433, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433,
0x421, 0x430, 0x43c, 0x43e, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443,
0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x421, 0x435, 0x439, 0x448, 0x435, 0x43b,
0x438, 0x439, 0x43d, 0x20, 0x430, 0x440, 0x43b, 0x443, 0x443, 0x434, 0x44b, 0x43d,
0x20, 0x446, 0x430, 0x433, 0x421, 0x438, 0x43d, 0x433, 0x430, 0x43f, 0x443, 0x440,
0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x421, 0x43e, 0x43b, 0x43e, 0x43c, 0x43e,
0x43d, 0x44b, 0x20, 0x430, 0x440, 0x43b, 0x443, 0x443, 0x434, 0x44b, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x4e8, 0x43c, 0x43d, 0x4e9, 0x434, 0x20, 0x416, 0x43e, 0x440,
0x436, 0x438, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x421,
0x443, 0x440, 0x438, 0x43d, 0x430, 0x43c, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433,
0x421, 0x451, 0x432, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433,
0x422, 0x430, 0x438, 0x442, 0x438, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430,
0x433, 0x422, 0x430, 0x439, 0x43f, 0x435, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433,
0x422, 0x430, 0x439, 0x43f, 0x435, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x422, 0x430, 0x439, 0x43f,
0x435, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433,
0x422, 0x430, 0x436, 0x438, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x44b, 0x20,
0x446, 0x430, 0x433, 0x422, 0x43e, 0x43a, 0x435, 0x43b, 0x430, 0x443, 0x433, 0x438,
0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x433,
0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x422, 0x43e, 0x43d, 0x433, 0x430,
0x433, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x446, 0x430, 0x433, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x433, 0x438,
0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x427,
0x4af, 0x4af, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x422, 0x443,
0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x44b, 0x20,
0x446, 0x430, 0x433, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x446, 0x430, 0x433, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d,
0x438, 0x441, 0x442, 0x430, 0x43d, 0x44b, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20,
0x446, 0x430, 0x433, 0x422, 0x443, 0x432, 0x430, 0x43b, 0x443, 0x433, 0x438, 0x439,
0x43d, 0x20, 0x446, 0x430, 0x433, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439,
0x43d, 0x20, 0x446, 0x430, 0x433, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439,
0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446,
0x430, 0x433, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x43d, 0x20, 0x437,
0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x423, 0x437, 0x431, 0x435, 0x43a,
0x438, 0x441, 0x442, 0x430, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x423, 0x437,
0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x44b, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x423, 0x437,
0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x44b, 0x20, 0x437, 0x443,
0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442,
0x443, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x412, 0x430, 0x43d,
0x443, 0x430, 0x442, 0x443, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x412, 0x430, 0x43d,
0x443, 0x430, 0x442, 0x443, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d,
0x44b, 0x20, 0x446, 0x430, 0x433, 0x412, 0x435, 0x43d, 0x435, 0x441, 0x443, 0x44d,
0x43b, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x412, 0x43b, 0x430, 0x434,
0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x446,
0x430, 0x433, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e,
0x43a, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x446, 0x430, 0x433, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e,
0x441, 0x442, 0x43e, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b,
0x20, 0x446, 0x430, 0x433, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430,
0x434, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x412, 0x43e, 0x43b, 0x433, 0x43e,
0x433, 0x440, 0x430, 0x434, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x412, 0x43e, 0x43b, 0x433, 0x43e,
0x433, 0x440, 0x430, 0x434, 0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20,
0x446, 0x430, 0x433, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x438, 0x439, 0x43d,
0x20, 0x446, 0x430, 0x433, 0x423, 0x44d, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x430,
0x440, 0x43b, 0x443, 0x443, 0x434, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x423,
0x43e, 0x43b, 0x43b, 0x438, 0x441, 0x20, 0x431, 0x430, 0x20, 0x424, 0x443, 0x442,
0x443, 0x43d, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x42f,
0x43a, 0x443, 0x442, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x42f, 0x43a, 0x443,
0x442, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x446, 0x430, 0x433, 0x42f, 0x43a, 0x443, 0x442, 0x44b, 0x43d, 0x20, 0x437,
0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x415, 0x43a, 0x430, 0x442, 0x435,
0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446,
0x430, 0x433, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443,
0x440, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440,
0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443,
0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x42e, 0x43a, 0x43e, 0x43d, 0x20, 0x446,
0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x431, 0x4af, 0x441, 0x1832, 0x1825, 0x182a,
0x20, 0x1834, 0x1820, 0x182d, 0x1833, 0x1825, 0x182a, 0x20, 0x1830, 0x1832, 0x180b, 0x1820,
0x1828, 0x180b, 0x1833, 0x1820, 0x1837, 0x1833, 0x180b, 0x20, 0x1834, 0x1820, 0x182d, 0x1832,
0x1825, 0x182a, 0x20, 0x1835, 0x1823, 0x1828, 0x20, 0x180e, 0x180e, 0x180e, 0x1824, 0x20,
0x1834, 0x1820, 0x182d, 0x1835, 0x1821, 0x182d, 0x1825, 0x1828, 0x20, 0x1821, 0x1837, 0x182d,
0x1821, 0x20, 0x180e, 0x1822, 0x1822, 0x1828, 0x20, 0x1834, 0x1820, 0x182d, 0x1835, 0x1821,
0x182d, 0x1825, 0x1828, 0x20, 0x1821, 0x1837, 0x182d, 0x1821, 0x20, 0x180e, 0x1822, 0x1822,
0x1828, 0x20, 0x1830, 0x1832, 0x180b, 0x1820, 0x1828, 0x180b, 0x1833, 0x1820, 0x1837, 0x1833,
0x180b, 0x20, 0x1834, 0x1820, 0x182d, 0x1835, 0x1821, 0x182d, 0x1825, 0x1828, 0x20, 0x1821,
0x1837, 0x182d, 0x1821, 0x20, 0x180e, 0x1822, 0x1822, 0x1828, 0x20, 0x1835, 0x1823, 0x1828,
0x20, 0x180e, 0x180e, 0x180e, 0x1824, 0x20, 0x1834, 0x1820, 0x182d, 0x1820, 0x182d, 0x1823,
0x182f, 0x1820, 0x20, 0x180e, 0x180e, 0x180e, 0x180e, 0x1822, 0x1822, 0x1828, 0x20, 0x1834,
0x1820, 0x182d, 0x1820, 0x182d, 0x1823, 0x182f, 0x1820, 0x20, 0x180e, 0x180e, 0x180e, 0x180e,
0x1822, 0x1822, 0x1828, 0x20, 0x1830, 0x1832, 0x180b, 0x1820, 0x1828, 0x180b, 0x1833, 0x1820,
0x1837, 0x1833, 0x180b, 0x20, 0x1834, 0x1820, 0x182d, 0x1820, 0x182d, 0x1823, 0x182f, 0x1820,
0x20, 0x180e, 0x1822, 0x1822, 0x1828, 0x20, 0x1835, 0x1823, 0x1828, 0x20, 0x180e, 0x180e,
0x1824, 0x20, 0x1834, 0x1820, 0x182d, 0x1828, 0x1823, 0x182e, 0x1823, 0x182c, 0x1820, 0x1828,
0x20, 0x1833, 0x1820, 0x182f, 0x1820, 0x1822, 0x20, 0x180e, 0x1822, 0x1822, 0x1828, 0x20,
0x1834, 0x1820, 0x182d, 0x1828, 0x1823, 0x182e, 0x1823, 0x182c, 0x1820, 0x1828, 0x20, 0x1833,
0x1820, 0x182f, 0x1820, 0x1822, 0x20, 0x180e, 0x180e, 0x1822, 0x1822, 0x1828, 0x20, 0x1830,
0x1832, 0x180b, 0x1820, 0x1828, 0x180b, 0x1833, 0x1820, 0x1837, 0x1833, 0x180b, 0x20, 0x1834,
0x1820, 0x182d, 0x1828, 0x1823, 0x182e, 0x1823, 0x182c, 0x1820, 0x1828, 0x20, 0x1833, 0x1820,
0x182f, 0x1820, 0x1822, 0x20, 0x180e, 0x1822, 0x1822, 0x1828, 0x20, 0x1835, 0x1823, 0x1828,
0x20, 0x180e, 0x180e, 0x180e, 0x182a, 0x20, 0x1834, 0x1820, 0x182d, 0x1820, 0x1832, 0x180b,
0x182f, 0x1820, 0x1828, 0x180b, 0x1832, 0x180b, 0x20, 0x180e, 0x1824, 0x1828, 0x20, 0x1834,
0x1820, 0x182d, 0x1820, 0x1832, 0x180b, 0x182f, 0x1820, 0x1828, 0x180b, 0x1832, 0x20, 0x180e,
0x1824, 0x1828, 0x20, 0x1830, 0x1832, 0x180b, 0x1820, 0x1828, 0x180b, 0x1833, 0x1820, 0x1837,
0x1833, 0x180b, 0x20, 0x1834, 0x1820, 0x182d, 0x1820, 0x1832, 0x180b, 0x182f, 0x1820, 0x1828,
0x180b, 0x1832, 0x20, 0x180e, 0x1824, 0x1828, 0x20, 0x1835, 0x1823, 0x1828, 0x20, 0x180e,
0x182a, 0x20, 0x1834, 0x1820, 0x182d, 0x1832, 0x1825, 0x182a, 0x20, 0x1821, 0x1838, 0x1823,
0x1822, 0x1837, 0x1824, 0x182b, 0x1820, 0x20, 0x1822, 0x1822, 0x1828, 0x20, 0x1834, 0x1820,
0x182d, 0x1832, 0x1825, 0x182a, 0x20, 0x1821, 0x1838, 0x1823, 0x1822, 0x1837, 0x1824, 0x182b,
0x1820, 0x20, 0x1822, 0x1822, 0x1828, 0x20, 0x1830, 0x1832, 0x180b, 0x1820, 0x1828, 0x180b,
0x1833, 0x1820, 0x1837, 0x1833, 0x180b, 0x20, 0x1834, 0x1820, 0x182d, 0x1832, 0x1825, 0x182a,
0x20, 0x1821, 0x1838, 0x1823, 0x1822, 0x1837, 0x1824, 0x182b, 0x1820, 0x20, 0x1822, 0x1822,
0x1828, 0x20, 0x1835, 0x1823, 0x1828, 0x20, 0x180e, 0x180e, 0x1824, 0x20, 0x1834, 0x1820,
0x182d, 0x1835, 0x1821, 0x182d, 0x1826, 0x1828, 0x20, 0x1821, 0x1838, 0x1823, 0x1822, 0x1837,
0x1824, 0x182b, 0x1820, 0x20, 0x1822, 0x1822, 0x1828, 0x20, 0x1834, 0x1820, 0x182d, 0x1835,
0x1821, 0x182d, 0x1826, 0x1828, 0x20, 0x1821, 0x1838, 0x1823, 0x1822, 0x1837, 0x1824, 0x182b,
0x1820, 0x20, 0x1822, 0x1822, 0x1828, 0x20, 0x1830, 0x1832, 0x180b, 0x1820, 0x1828, 0x180b,
0x1833, 0x1820, 0x1837, 0x1833, 0x180b, 0x20, 0x1834, 0x1820, 0x182d, 0x1835, 0x1821, 0x182d,
0x1826, 0x1828, 0x20, 0x1821, 0x1838, 0x1823, 0x1822, 0x1837, 0x1824, 0x182b, 0x1820, 0x20,
0x1822, 0x1822, 0x1828, 0x20, 0x1835, 0x1823, 0x1828, 0x20, 0x180e, 0x180e, 0x1824, 0x20,
0x1834, 0x1820, 0x182d, 0x182a, 0x1820, 0x1837, 0x1820, 0x182d, 0x1823, 0x1828, 0x20, 0x1821,
0x1838, 0x1823, 0x1822, 0x1837, 0x1824, 0x182b, 0x1820, 0x20, 0x1822, 0x1822, 0x1828, 0x20,
0x1834, 0x1820, 0x182d, 0x182a, 0x1820, 0x1837, 0x1820, 0x182d, 0x1823, 0x1828, 0x20, 0x1821,
0x1838, 0x1823, 0x1822, 0x1837, 0x1824, 0x182b, 0x1820, 0x20, 0x1822, 0x1822, 0x1828, 0x20,
0x1830, 0x1832, 0x180b, 0x1820, 0x1828, 0x180b, 0x1833, 0x1820, 0x1837, 0x1833, 0x180b, 0x20,
0x1834, 0x1820, 0x182d, 0x182a, 0x1820, 0x1837, 0x1820, 0x182d, 0x1823, 0x1828, 0x20, 0x1821,
0x1838, 0x1823, 0x1822, 0x1837, 0x1824, 0x182b, 0x1820, 0x20, 0x1822, 0x1822, 0x1828, 0x20,
0x1835, 0x1823, 0x1828, 0x20, 0x180e, 0x180e, 0x1824, 0x20, 0x1834, 0x1820, 0x182d, 0x182d,
0x1837, 0x1822, 0x1828, 0x180b, 0x1838, 0x1822, 0x1834, 0x1822, 0x20, 0x1822, 0x1822, 0x1828,
0x20, 0x1834, 0x1820, 0x182d, 0x905, 0x92b, 0x917, 0x93e, 0x928, 0x93f, 0x938, 0x94d,
0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x947, 0x928, 0x94d, 0x926,
0x94d, 0x930, 0x940, 0x92f, 0x20, 0x905, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x940,
0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x905,
0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x926, 0x915,
0x94d, 0x937, 0x93f, 0x923, 0x20, 0x905, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x940,
0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x905,
0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936,
0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x905, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x940,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d,
0x91a, 0x93f, 0x92e, 0x20, 0x905, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x940, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928,
0x20, 0x938, 0x92e, 0x92f, 0x905, 0x932, 0x938, 0x94d, 0x915, 0x93e, 0x915, 0x94b,
0x20, 0x938, 0x92e, 0x92f, 0x905, 0x932, 0x938, 0x94d, 0x915, 0x93e, 0x915, 0x94b,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x932, 0x938,
0x94d, 0x915, 0x93e, 0x915, 0x94b, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938,
0x92e, 0x92f, 0x90f, 0x92e, 0x93e, 0x91c, 0x94b, 0x928, 0x20, 0x938, 0x92e, 0x92f,
0x90f, 0x92e, 0x93e, 0x91c, 0x94b, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x90f, 0x92e, 0x93e, 0x91c, 0x94b, 0x928, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938,
0x92e, 0x92f, 0x915, 0x947, 0x928, 0x94d, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20,
0x938, 0x92e, 0x92f, 0x915, 0x947, 0x928, 0x94d, 0x926, 0x94d, 0x930, 0x940, 0x92f,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x947, 0x928,
0x94d, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20,
0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x926, 0x93f,
0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x93f, 0x92e, 0x93e, 0x932, 0x940,
0x20, 0x938, 0x92e, 0x92f, 0x939, 0x93f, 0x92e, 0x93e, 0x932, 0x940, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x93f, 0x92e, 0x93e, 0x932,
0x940, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x94d,
0x92f, 0x93e, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a,
0x94d, 0x92f, 0x93e, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x94d, 0x92f, 0x93e, 0x938, 0x93f, 0x92b,
0x93f, 0x915, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x906,
0x92a, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x906, 0x92a, 0x93f, 0x92f,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x906, 0x92a,
0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f,
0x905, 0x930, 0x92c, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x930, 0x92c, 0x940,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x930, 0x92c,
0x940, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x930,
0x94d, 0x91c, 0x947, 0x928, 0x91f, 0x93f, 0x928, 0x940, 0x20, 0x938, 0x92e, 0x92f,
0x905, 0x930, 0x94d, 0x91c, 0x947, 0x928, 0x91f, 0x93f, 0x928, 0x940, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x930, 0x94d, 0x91c, 0x947,
0x928, 0x91f, 0x93f, 0x928, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936,
0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x928,
0x91f, 0x93f, 0x928, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a,
0x93f, 0x92e, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x928, 0x91f, 0x93f,
0x928, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a,
0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947,
0x928, 0x91f, 0x93f, 0x928, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x930,
0x94d, 0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x905,
0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f,
0x92f, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e,
0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x90f, 0x91f, 0x94d, 0x932, 0x93e,
0x928, 0x94d, 0x91f, 0x93f, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x90f, 0x91f, 0x94d,
0x932, 0x93e, 0x928, 0x94d, 0x91f, 0x93f, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x90f, 0x91f, 0x94d, 0x932, 0x93e, 0x928, 0x94d, 0x91f,
0x93f, 0x915, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x915,
0x947, 0x928, 0x94d, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x905, 0x938, 0x94d,
0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f,
0x915, 0x947, 0x928, 0x94d, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x905, 0x938,
0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x947, 0x928, 0x94d, 0x926, 0x94d,
0x930, 0x940, 0x92f, 0x20, 0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932,
0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f,
0x915, 0x947, 0x928, 0x94d, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x92a, 0x936,
0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930,
0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x947, 0x928,
0x94d, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f,
0x92e, 0x940, 0x20, 0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f,
0x92f, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x915,
0x947, 0x928, 0x94d, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x92a, 0x936, 0x94d,
0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947,
0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e,
0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x905, 0x938, 0x94d, 0x91f,
0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92a,
0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930,
0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x905, 0x938, 0x94d,
0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93f, 0x935,
0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x91c, 0x930, 0x92c, 0x948, 0x91c, 0x93e,
0x928, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x91c, 0x930, 0x92c, 0x948, 0x91c, 0x93e,
0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x91c,
0x930, 0x92c, 0x948, 0x91c, 0x93e, 0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x90f,
0x91c, 0x94b, 0x930, 0x947, 0x938, 0x94d, 0x20, 0x938, 0x92e, 0x92f, 0x90f, 0x91c,
0x94b, 0x930, 0x947, 0x938, 0x94d, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x90f, 0x91c, 0x94b, 0x930, 0x947, 0x938, 0x94d, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938,
0x92e, 0x92f, 0x92c, 0x902, 0x917, 0x932, 0x93e, 0x926, 0x947, 0x936, 0x940, 0x20,
0x938, 0x92e, 0x92f, 0x92c, 0x902, 0x917, 0x932, 0x93e, 0x926, 0x947, 0x936, 0x940,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92c, 0x902, 0x917,
0x932, 0x93e, 0x926, 0x947, 0x936, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92d,
0x941, 0x91f, 0x93e, 0x928, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x92c, 0x94b, 0x932,
0x93f, 0x92d, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92c, 0x94d, 0x930,
0x93e, 0x91c, 0x93f, 0x932, 0x940, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92c,
0x94d, 0x930, 0x93e, 0x91c, 0x93f, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x93f,
0x932, 0x940, 0x92f, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92c, 0x94d, 0x930,
0x941, 0x928, 0x93e, 0x907, 0x20, 0x926, 0x93e, 0x930, 0x942, 0x938, 0x94d, 0x938,
0x932, 0x92e, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x947, 0x92a, 0x20, 0x92d, 0x930,
0x94d, 0x926, 0x947, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x947, 0x92a, 0x20, 0x92d,
0x930, 0x94d, 0x926, 0x947, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x915, 0x947, 0x92a, 0x20, 0x92d, 0x930, 0x94d, 0x926, 0x947, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x91a, 0x93e, 0x92e, 0x94b, 0x930, 0x94d, 0x930, 0x94b, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x91a, 0x93e, 0x925, 0x93e,
0x92e, 0x20, 0x938, 0x92e, 0x92f, 0x91a, 0x93e, 0x925, 0x93e, 0x92e, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x91a, 0x93e, 0x925, 0x93e, 0x92e,
0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x91a, 0x940, 0x928,
0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94d, 0x930,
0x93f, 0x938, 0x94d, 0x92e, 0x938, 0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x20, 0x938,
0x92e, 0x92f, 0x915, 0x94b, 0x915, 0x94b, 0x938, 0x20, 0x91f, 0x93e, 0x92a, 0x941,
0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x932, 0x92e, 0x94d, 0x92c, 0x93f, 0x92f,
0x93e, 0x932, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x932, 0x92e, 0x94d,
0x92c, 0x93f, 0x92f, 0x93e, 0x932, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x932, 0x92e, 0x94d, 0x92c, 0x93f, 0x92f, 0x93e,
0x932, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e,
0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x941, 0x915, 0x20, 0x91f,
0x93e, 0x92a, 0x941, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x941, 0x915, 0x20, 0x91f,
0x93e, 0x92a, 0x941, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x915, 0x941, 0x915, 0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x20, 0x906, 0x927, 0x93e,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940,
0x928, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x915,
0x94b, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x915,
0x94b, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94d,
0x92f, 0x942, 0x92c, 0x93e, 0x915, 0x94b, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20,
0x938, 0x92e, 0x92f, 0x921, 0x947, 0x92d, 0x93f, 0x938, 0x20, 0x938, 0x92e, 0x92f,
0x921, 0x941, 0x92e, 0x94b, 0x928, 0x94d, 0x91f, 0x2d, 0x921, 0x940, 0x2018, 0x20,
0x909, 0x930, 0x94d, 0x92d, 0x93f, 0x932, 0x947, 0x20, 0x938, 0x92e, 0x92f, 0x92a,
0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x91f, 0x93f, 0x92e, 0x94b, 0x930, 0x20,
0x938, 0x92e, 0x92f, 0x907, 0x938, 0x94d, 0x91f, 0x930, 0x20, 0x91f, 0x93e, 0x92a,
0x942, 0x20, 0x938, 0x92e, 0x92f, 0x907, 0x938, 0x94d, 0x91f, 0x930, 0x20, 0x91f,
0x93e, 0x92a, 0x942, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x907, 0x938, 0x94d, 0x91f, 0x930, 0x20, 0x91f, 0x93e, 0x92a, 0x942, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x938, 0x92e, 0x92f, 0x908, 0x915,
0x94d, 0x935, 0x94b, 0x921, 0x94b, 0x930, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x947,
0x928, 0x94d, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x92f, 0x941, 0x930, 0x94b,
0x92a, 0x947, 0x932, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x947, 0x928, 0x94d,
0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x947,
0x932, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x915,
0x947, 0x928, 0x94d, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x92f, 0x941, 0x930,
0x94b, 0x92a, 0x947, 0x932, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942,
0x930, 0x94d, 0x935, 0x940, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x947, 0x932,
0x940, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20,
0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x947, 0x932, 0x940, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20,
0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x947, 0x932, 0x940, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e,
0x92f, 0x925, 0x92a, 0x2d, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x92f,
0x941, 0x930, 0x94b, 0x92a, 0x947, 0x932, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x92a,
0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a,
0x947, 0x932, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f,
0x92e, 0x940, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x947, 0x932, 0x940, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x932, 0x94d, 0x915,
0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x92a, 0x942,
0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x932, 0x94d, 0x915, 0x932, 0x94d, 0x92f, 0x93e,
0x928, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x92a, 0x942, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x932, 0x94d, 0x915, 0x932, 0x94d, 0x92f,
0x93e, 0x928, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x92a, 0x942, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938,
0x92e, 0x92f, 0x92b, 0x93f, 0x91c, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93f,
0x91c, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92b,
0x93f, 0x91c, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915,
0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x94d, 0x930, 0x947,
0x928, 0x94d, 0x91a, 0x20, 0x917, 0x94d, 0x935, 0x93e, 0x928, 0x93e, 0x20, 0x938,
0x92e, 0x92f, 0x92b, 0x94d, 0x930, 0x947, 0x928, 0x94d, 0x91a, 0x20, 0x926, 0x915,
0x94d, 0x937, 0x93f, 0x923, 0x940, 0x20, 0x930, 0x20, 0x905, 0x928, 0x94d, 0x91f,
0x93e, 0x930, 0x94d, 0x91f, 0x93f, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x917, 0x93e,
0x932, 0x93e, 0x92a, 0x93e, 0x917, 0x94b, 0x938, 0x94d, 0x20, 0x938, 0x92e, 0x92f,
0x917, 0x94d, 0x92f, 0x93e, 0x92e, 0x94d, 0x92c, 0x93f, 0x92f, 0x930, 0x20, 0x938,
0x92e, 0x92f, 0x91c, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e,
0x92f, 0x91c, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x91c, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940,
0x928, 0x20, 0x938, 0x92e, 0x92f, 0x917, 0x93f, 0x932, 0x94d, 0x92c, 0x930, 0x94d,
0x91f, 0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x20, 0x938, 0x92e, 0x92f, 0x917, 0x94d,
0x930, 0x940, 0x928, 0x935, 0x93f, 0x91a, 0x20, 0x92e, 0x93f, 0x928, 0x20, 0x938,
0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x928, 0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x915, 0x94b, 0x20,
0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x928, 0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x915, 0x94b,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930,
0x94d, 0x935, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x94d, 0x92f,
0x93e, 0x928, 0x94d, 0x921, 0x915, 0x94b, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a,
0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928,
0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x915, 0x94b, 0x20, 0x938, 0x92e,
0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x928, 0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x915, 0x94b, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a,
0x93f, 0x92e, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x94d, 0x92f,
0x93e, 0x928, 0x94d, 0x921, 0x915, 0x94b, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x916,
0x93e, 0x921, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x939, 0x935, 0x93e, 0x908, 0x2d, 0x90f, 0x932, 0x941, 0x91f, 0x93f, 0x92f, 0x928,
0x20, 0x938, 0x92e, 0x92f, 0x939, 0x935, 0x93e, 0x908, 0x2d, 0x90f, 0x932, 0x941,
0x91f, 0x93f, 0x92f, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x939, 0x935, 0x93e, 0x908, 0x2d, 0x90f, 0x932, 0x941, 0x91f, 0x93f, 0x92f,
0x928, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x919,
0x915, 0x919, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x919, 0x915, 0x919, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x919, 0x915, 0x919, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928,
0x20, 0x938, 0x92e, 0x92f, 0x939, 0x94b, 0x92c, 0x94d, 0x921, 0x20, 0x938, 0x92e,
0x92f, 0x939, 0x94b, 0x92c, 0x94d, 0x921, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x939, 0x94b, 0x92c, 0x94d, 0x921, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e,
0x92f, 0x939, 0x93f, 0x928, 0x94d, 0x926, 0x20, 0x92e, 0x939, 0x93e, 0x938, 0x93e,
0x917, 0x930, 0x20, 0x938, 0x92e, 0x92f, 0x907, 0x928, 0x94d, 0x921, 0x94b, 0x91a,
0x93e, 0x907, 0x928, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x947, 0x928, 0x94d,
0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x907, 0x928, 0x94d, 0x921, 0x94b, 0x928,
0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930,
0x94d, 0x935, 0x940, 0x20, 0x907, 0x928, 0x94d, 0x921, 0x94b, 0x928, 0x947, 0x936,
0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f,
0x92e, 0x940, 0x20, 0x907, 0x928, 0x94d, 0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f,
0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x907, 0x930, 0x93e, 0x928, 0x940, 0x20,
0x938, 0x92e, 0x92f, 0x907, 0x930, 0x93e, 0x928, 0x940, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x907, 0x930, 0x93e, 0x928, 0x940, 0x20, 0x926,
0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x908, 0x930, 0x94d, 0x915, 0x941,
0x91f, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x908, 0x930, 0x94d,
0x915, 0x941, 0x91f, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x908, 0x930, 0x94d, 0x915, 0x941, 0x91f, 0x94d, 0x938,
0x94d, 0x915, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e,
0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x907, 0x91c, 0x930, 0x93e, 0x92f,
0x932, 0x20, 0x938, 0x92e, 0x92f, 0x907, 0x91c, 0x930, 0x93e, 0x92f, 0x932, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x907, 0x91c, 0x930, 0x93e,
0x92f, 0x932, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x91c,
0x93e, 0x92a, 0x93e, 0x928, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e,
0x92f, 0x915, 0x93e, 0x91c, 0x915, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x915, 0x94b,
0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x915,
0x93e, 0x91c, 0x915, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f,
0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x915, 0x93e, 0x91c, 0x915, 0x938,
0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x930, 0x93f,
0x92f, 0x93e, 0x932, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x930, 0x93f,
0x92f, 0x93e, 0x932, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x915, 0x94b, 0x930, 0x93f, 0x92f, 0x93e, 0x932, 0x940, 0x20, 0x926, 0x93f,
0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94d, 0x930, 0x93e, 0x938, 0x928,
0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94d,
0x930, 0x93e, 0x938, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x915, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94d, 0x930, 0x93e, 0x938,
0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x915, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f,
0x915, 0x93f, 0x930, 0x94d, 0x917, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x932, 0x93e, 0x907, 0x928, 0x20, 0x91f, 0x93e, 0x92a, 0x941,
0x20, 0x938, 0x92e, 0x92f, 0x932, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x93e, 0x935,
0x947, 0x20, 0x938, 0x92e, 0x92f, 0x932, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x93e,
0x935, 0x947, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x932,
0x930, 0x94d, 0x921, 0x20, 0x939, 0x93e, 0x935, 0x947, 0x20, 0x926, 0x93f, 0x935,
0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x932, 0x947, 0x938, 0x93f, 0x92f, 0x93e,
0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x93e, 0x932, 0x94d, 0x926, 0x93f, 0x92d, 0x94d,
0x938, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x93e, 0x930, 0x94d, 0x915, 0x93f, 0x938,
0x938, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x93e, 0x930, 0x94d, 0x936, 0x932, 0x20,
0x91f, 0x93e, 0x92a, 0x941, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x909, 0x930, 0x93f,
0x91f, 0x938, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x909, 0x930, 0x93f, 0x91f, 0x938,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x909, 0x930,
0x93f, 0x91f, 0x938, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915,
0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x94d, 0x935, 0x938,
0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915,
0x928, 0x20, 0x92a, 0x94d, 0x92f, 0x93e, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x928, 0x20,
0x92a, 0x94d, 0x92f, 0x93e, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f,
0x915, 0x928, 0x20, 0x92a, 0x94d, 0x92f, 0x93e, 0x938, 0x93f, 0x92b, 0x93f, 0x915,
0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x932, 0x93e,
0x928, 0x20, 0x92c, 0x93e, 0x91f, 0x94b, 0x930, 0x20, 0x938, 0x92e, 0x92f, 0x909,
0x932, 0x93e, 0x928, 0x20, 0x92c, 0x93e, 0x91f, 0x94b, 0x930, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x932, 0x93e, 0x928, 0x20, 0x92c,
0x93e, 0x91f, 0x94b, 0x930, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x938, 0x94d,
0x915, 0x94b, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x938, 0x94d, 0x915, 0x94b, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x938, 0x94d, 0x915,
0x94b, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x94d, 0x92f, 0x93e, 0x928, 0x92e,
0x93e, 0x930, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x93e, 0x909, 0x930, 0x941, 0x20,
0x938, 0x92e, 0x92f, 0x928, 0x947, 0x92a, 0x93e, 0x932, 0x940, 0x20, 0x938, 0x92e,
0x92f, 0x928, 0x92f, 0x93e, 0x901, 0x20, 0x915, 0x93e, 0x932, 0x947, 0x926, 0x94b,
0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x92f, 0x93e, 0x901,
0x20, 0x915, 0x93e, 0x932, 0x947, 0x926, 0x94b, 0x928, 0x93f, 0x92f, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x92f, 0x93e, 0x901,
0x20, 0x915, 0x93e, 0x932, 0x947, 0x926, 0x94b, 0x928, 0x93f, 0x92f, 0x93e, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928,
0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x91c, 0x93f, 0x932, 0x94d,
0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f,
0x942, 0x91c, 0x93f, 0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x91c,
0x93f, 0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x20, 0x926, 0x93f, 0x935,
0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x92b, 0x93e, 0x909,
0x928, 0x94d, 0x921, 0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x915, 0x94b,
0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x92b, 0x93e, 0x909, 0x928,
0x921, 0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x915, 0x94b, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x92b,
0x93e, 0x909, 0x928, 0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x915, 0x94b,
0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x93f, 0x909,
0x90f, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94b, 0x930, 0x94d, 0x92b, 0x94b, 0x915,
0x20, 0x91f, 0x93e, 0x92a, 0x942, 0x915, 0x94b, 0x20, 0x938, 0x92e, 0x92f, 0x928,
0x94b, 0x930, 0x94d, 0x92b, 0x94b, 0x915, 0x20, 0x91f, 0x93e, 0x92a, 0x942, 0x915,
0x94b, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94b,
0x930, 0x94d, 0x92b, 0x94b, 0x915, 0x20, 0x91f, 0x93e, 0x92a, 0x942, 0x915, 0x94b,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940,
0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x930, 0x94d, 0x928, 0x93e, 0x928, 0x94d,
0x921, 0x94b, 0x20, 0x921, 0x947, 0x20, 0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d,
0x939, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x930, 0x94d, 0x928, 0x93e, 0x928,
0x94d, 0x921, 0x94b, 0x20, 0x921, 0x947, 0x20, 0x928, 0x94b, 0x930, 0x94b, 0x928,
0x94d, 0x939, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x92b, 0x930, 0x94d, 0x928, 0x93e, 0x928, 0x94d, 0x921, 0x94b, 0x20, 0x921, 0x947,
0x20, 0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938,
0x92e, 0x92f, 0x928, 0x94b, 0x92d, 0x94b, 0x938, 0x93f, 0x935, 0x93f, 0x930, 0x94d,
0x938, 0x94d, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94b, 0x92d, 0x94b, 0x938,
0x93f, 0x935, 0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94b, 0x92d, 0x94b, 0x938, 0x93f, 0x935,
0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a,
0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x940, 0x20, 0x938, 0x92e,
0x92f, 0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x940, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x93e, 0x915, 0x93f,
0x938, 0x94d, 0x924, 0x93e, 0x928, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a,
0x93e, 0x932, 0x93e, 0x909, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x92a, 0x942, 0x935,
0x93e, 0x20, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x917, 0x93f, 0x928, 0x940, 0x20,
0x938, 0x92e, 0x92f, 0x92a, 0x93e, 0x930, 0x93e, 0x917, 0x94d, 0x935, 0x947, 0x20,
0x938, 0x92e, 0x92f, 0x92a, 0x93e, 0x930, 0x93e, 0x917, 0x94d, 0x935, 0x947, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x93e, 0x930, 0x93e,
0x917, 0x94d, 0x935, 0x947, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x947, 0x930,
0x941, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x947, 0x930, 0x941, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938,
0x92e, 0x92f, 0x92b, 0x93f, 0x932, 0x93f, 0x92a, 0x93f, 0x928, 0x940, 0x20, 0x938,
0x92e, 0x92f, 0x92b, 0x93f, 0x932, 0x93f, 0x92a, 0x93f, 0x928, 0x940, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93f, 0x932, 0x93f, 0x92a,
0x93f, 0x928, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915,
0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93f, 0x928, 0x93f,
0x915, 0x94d, 0x938, 0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x20, 0x938, 0x92e, 0x92f,
0x938, 0x947, 0x928, 0x94d, 0x91f, 0x20, 0x92a, 0x93f, 0x92f, 0x930, 0x94d, 0x930,
0x947, 0x20, 0x930, 0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x92f, 0x941, 0x932, 0x94b,
0x928, 0x915, 0x94b, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x947, 0x928, 0x94d, 0x91f,
0x20, 0x92a, 0x93f, 0x92f, 0x930, 0x94d, 0x930, 0x947, 0x20, 0x930, 0x20, 0x92e,
0x93f, 0x915, 0x94d, 0x92f, 0x941, 0x932, 0x94b, 0x928, 0x915, 0x94b, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x947, 0x928, 0x94d, 0x91f,
0x20, 0x92a, 0x93f, 0x92f, 0x930, 0x94d, 0x930, 0x947, 0x20, 0x930, 0x20, 0x92e,
0x93f, 0x915, 0x94d, 0x92f, 0x941, 0x932, 0x94b, 0x928, 0x915, 0x94b, 0x20, 0x926,
0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x93f, 0x91f, 0x915, 0x948,
0x930, 0x923, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x94d, 0x92f, 0x94b, 0x919, 0x92f,
0x93e, 0x919, 0x20, 0x938, 0x92e, 0x92f, 0x930, 0x93f, 0x92f, 0x941, 0x928, 0x93f,
0x92f, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x93e, 0x916, 0x93e, 0x932, 0x93f,
0x928, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x93e, 0x916, 0x93e, 0x932, 0x93f, 0x928,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x93e, 0x916,
0x93e, 0x932, 0x93f, 0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x93e, 0x92e,
0x94b, 0x905, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x93e, 0x92e, 0x94b, 0x905, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x93e, 0x92e, 0x94b,
0x905, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x947,
0x92f, 0x947, 0x91a, 0x947, 0x932, 0x93e, 0x938, 0x94d, 0x20, 0x938, 0x92e, 0x92f,
0x938, 0x93f, 0x902, 0x917, 0x93e, 0x92a, 0x941, 0x930, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x94b, 0x932, 0x94b, 0x92e, 0x94b, 0x928,
0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x20, 0x938, 0x92e, 0x92f, 0x926, 0x915, 0x94d,
0x937, 0x93f, 0x923, 0x20, 0x91c, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20,
0x938, 0x92e, 0x92f, 0x938, 0x941, 0x930, 0x93f, 0x928, 0x93e, 0x92e, 0x93e, 0x20,
0x938, 0x92e, 0x92f, 0x924, 0x93e, 0x907, 0x92a, 0x947, 0x907, 0x20, 0x938, 0x92e,
0x92f, 0x924, 0x93e, 0x907, 0x92a, 0x947, 0x907, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x924, 0x93e, 0x907, 0x92a, 0x947, 0x907, 0x20, 0x926,
0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x924, 0x93e, 0x91c, 0x93f, 0x915,
0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x924, 0x94b, 0x915,
0x947, 0x932, 0x93e, 0x909, 0x20, 0x938, 0x92e, 0x92f, 0x91a, 0x941, 0x909, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x924, 0x941, 0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928,
0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x91f, 0x941, 0x92d, 0x93e, 0x932, 0x941, 0x20, 0x938,
0x92e, 0x92f, 0x909, 0x930, 0x942, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x91c, 0x94d, 0x92c, 0x947, 0x915,
0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x91c,
0x94d, 0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x91c, 0x94d, 0x92c, 0x947,
0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f,
0x92d, 0x93e, 0x928, 0x941, 0x906, 0x924, 0x941, 0x20, 0x938, 0x92e, 0x92f, 0x92d,
0x93e, 0x928, 0x941, 0x906, 0x924, 0x941, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x92d, 0x93e, 0x928, 0x941, 0x906, 0x924, 0x941, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x92d, 0x947, 0x928, 0x947, 0x91c, 0x94d, 0x92f, 0x941, 0x90f,
0x932, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92d, 0x94d, 0x932, 0x93e, 0x926, 0x93f,
0x92d, 0x93e, 0x938, 0x94d, 0x91f, 0x94b, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92d,
0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x92d, 0x93e, 0x938, 0x94d, 0x91f, 0x94b, 0x915,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92d, 0x94d, 0x932,
0x93e, 0x926, 0x93f, 0x92d, 0x93e, 0x938, 0x94d, 0x91f, 0x94b, 0x915, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x92d, 0x94b, 0x932, 0x94d, 0x917, 0x93e, 0x917, 0x94d, 0x930,
0x93e, 0x921, 0x20, 0x938, 0x92e, 0x92f, 0x92d, 0x94b, 0x932, 0x94d, 0x917, 0x93e,
0x917, 0x94d, 0x930, 0x93e, 0x926, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x92d, 0x94b, 0x932, 0x94d, 0x917, 0x93e, 0x917, 0x94d, 0x930, 0x93e,
0x926, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x935, 0x947, 0x915, 0x20, 0x91f, 0x93e,
0x92a, 0x941, 0x20, 0x938, 0x92e, 0x92f, 0x935, 0x93e, 0x932, 0x93f, 0x938, 0x94d,
0x20, 0x930, 0x20, 0x92b, 0x941, 0x91f, 0x941, 0x928, 0x93e, 0x20, 0x938, 0x92e,
0x92f, 0x92f, 0x93e, 0x915, 0x941, 0x938, 0x94d, 0x91f, 0x20, 0x938, 0x92e, 0x92f,
0x92f, 0x93e, 0x915, 0x941, 0x938, 0x94d, 0x91f, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x92f, 0x93e, 0x915, 0x941, 0x938, 0x94d, 0x91f, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928,
0x20, 0x938, 0x92e, 0x92f, 0x92f, 0x947, 0x915, 0x93e, 0x91f, 0x947, 0x930, 0x93f,
0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20, 0x938, 0x92e, 0x92f, 0x92f, 0x947, 0x915,
0x93e, 0x91f, 0x947, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92f, 0x947, 0x915, 0x93e, 0x91f,
0x947, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e,
0x92f, 0x92f, 0x941, 0x915, 0x94b, 0x928, 0x915, 0x94b, 0x20, 0x938, 0x92e, 0x92f,
0x41, 0x66, 0x67, 0xe1, 0x6e, 0xed, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x4d, 0xed, 0x64, 0xfa, 0x6c, 0x20, 0xc1, 0x66, 0x72,
0xed, 0x6b, 0xe1, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xcd, 0x73, 0x74, 0x20,
0xc1, 0x66, 0x72, 0xed, 0x6b, 0xe1, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x53,
0x61, 0xfa, 0x74, 0x20, 0xc1, 0x66, 0x72, 0xed, 0x6b, 0xe1, 0x20, 0x46,
0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x57, 0x1eb9, 0x301,
0x73, 0x74, 0x20, 0xc1, 0x66, 0x72, 0xed, 0x6b, 0xe1, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0x20, 0xc1, 0x66, 0x72, 0xed,
0x6b, 0xe1, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0x20, 0xc1, 0x66, 0x72, 0xed, 0x6b,
0xe1, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x41, 0x6c, 0xe1, 0x73, 0x6b, 0xe1, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x41, 0x6c, 0xe1, 0x73, 0x6b, 0xe1, 0x20, 0x46,
0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x41, 0x6c, 0xe1,
0x73, 0x6b, 0xe1, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54,
0x61, 0x69, 0x6d, 0xc1, 0x6d, 0xe1, 0x7a, 0x1ecd, 0x6e, 0x20, 0x54, 0x61,
0x69, 0x6d, 0xc1, 0x6d, 0xe1, 0x7a, 0x1ecd, 0x6e, 0x20, 0x46, 0xed, 0x6b,
0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xc1, 0x6d, 0xe1, 0x7a, 0x1ecd,
0x6e, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0xed, 0x6e,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x1ecd, 0x301, 0x74, 0x20, 0x41, 0x6d,
0x1eb9, 0x301, 0x72, 0xed, 0x6b, 0xe1, 0x20, 0x4d, 0xed, 0x64, 0xfa, 0x6c,
0x20, 0xc9, 0x72, 0x69, 0xe1, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x1ecd,
0x301, 0x74, 0x20, 0x41, 0x6d, 0x1eb9, 0x301, 0x72, 0xed, 0x6b, 0xe1, 0x20,
0x4d, 0xed, 0x64, 0xfa, 0x6c, 0x20, 0xc9, 0x72, 0x69, 0xe1, 0x20, 0x46,
0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x1ecd, 0x301,
0x74, 0x20, 0x41, 0x6d, 0x1eb9, 0x301, 0x72, 0xed, 0x6b, 0xe1, 0x20, 0x4d,
0xed, 0x64, 0xfa, 0x6c, 0x20, 0xc9, 0x72, 0x69, 0xe1, 0x20, 0x44, 0xe9,
0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x1ecd, 0x301,
0x74, 0x20, 0x41, 0x6d, 0x1eb9, 0x301, 0x72, 0xed, 0x6b, 0xe1, 0x20, 0xcd,
0x73, 0x74, 0xe1, 0x6e, 0x20, 0xc9, 0x72, 0x69, 0xe1, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x4e, 0x1ecd, 0x301, 0x74, 0x20, 0x41, 0x6d, 0x1eb9, 0x301, 0x72,
0xed, 0x6b, 0xe1, 0x20, 0xcd, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0xc9, 0x72,
0x69, 0xe1, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x4e, 0x1ecd, 0x301, 0x74, 0x20, 0x41, 0x6d, 0x1eb9, 0x301, 0x72, 0xed,
0x6b, 0xe1, 0x20, 0xcd, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0xc9, 0x72, 0x69,
0xe1, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x4e, 0x1ecd, 0x301, 0x74, 0x20, 0x41, 0x6d, 0x1eb9, 0x301, 0x72, 0xed,
0x6b, 0xe1, 0x20, 0x4d, 0x61, 0xfa, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0xc9,
0x72, 0x69, 0xe1, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x1ecd, 0x301, 0x74,
0x20, 0x41, 0x6d, 0x1eb9, 0x301, 0x72, 0xed, 0x6b, 0xe1, 0x20, 0x4d, 0x61,
0xfa, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0xc9, 0x72, 0x69, 0xe1, 0x20, 0x46,
0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x1ecd, 0x301,
0x74, 0x20, 0x41, 0x6d, 0x1eb9, 0x301, 0x72, 0xed, 0x6b, 0xe1, 0x20, 0x4d,
0x61, 0xfa, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0xc9, 0x72, 0x69, 0xe1, 0x20,
0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e,
0x1ecd, 0x301, 0x74, 0x20, 0x41, 0x6d, 0x1eb9, 0x301, 0x72, 0xed, 0x6b, 0xe1,
0x20, 0x50, 0x61, 0x73, 0xed, 0x66, 0xed, 0x6b, 0x20, 0xc9, 0x72, 0x69,
0xe1, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x1ecd, 0x301, 0x74, 0x20, 0x41,
0x6d, 0x1eb9, 0x301, 0x72, 0xed, 0x6b, 0xe1, 0x20, 0x50, 0x61, 0x73, 0xed,
0x66, 0xed, 0x6b, 0x20, 0xc9, 0x72, 0x69, 0xe1, 0x20, 0x46, 0xed, 0x6b,
0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x1ecd, 0x301, 0x74, 0x20,
0x41, 0x6d, 0x1eb9, 0x301, 0x72, 0xed, 0x6b, 0xe1, 0x20, 0x50, 0x61, 0x73,
0xed, 0x66, 0xed, 0x6b, 0x20, 0xc9, 0x72, 0x69, 0xe1, 0x20, 0x44, 0xe9,
0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xc1, 0x70, 0x69,
0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xc1, 0x70, 0x69, 0x61, 0x20, 0x46,
0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xc1, 0x70, 0x69,
0x61, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x41, 0x72, 0xe9, 0x62, 0x69, 0xe1, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x41, 0x72, 0xe9, 0x62, 0x69, 0xe1, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x41, 0x72, 0xe9, 0x62, 0x69, 0xe1, 0x20,
0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x41,
0x6a, 0x1eb9, 0x6e, 0x74, 0xed, 0x6e, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x41, 0x6a, 0x1eb9, 0x6e, 0x74, 0xed, 0x6e, 0x61, 0x20, 0x46, 0xed, 0x6b,
0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x41, 0x6a, 0x1eb9, 0x6e, 0x74,
0xed, 0x6e, 0x61, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a,
0xed, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x57, 0x1eb9, 0x301, 0x73, 0x74,
0xe1, 0x6e, 0x20, 0x41, 0x6a, 0x1eb9, 0x6e, 0x74, 0xed, 0x6e, 0x61, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0xe1, 0x6e, 0x20,
0x41, 0x6a, 0x1eb9, 0x6e, 0x74, 0xed, 0x6e, 0x61, 0x20, 0x46, 0xed, 0x6b,
0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x57, 0x1eb9, 0x301, 0x73, 0x74,
0xe1, 0x6e, 0x20, 0x41, 0x6a, 0x1eb9, 0x6e, 0x74, 0xed, 0x6e, 0x61, 0x20,
0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0xed, 0x6e, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x41, 0x72, 0x6d, 0x1eb9, 0x301, 0x6e, 0x69, 0x61, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x41, 0x72, 0x6d, 0x1eb9, 0x301, 0x6e, 0x69, 0x61,
0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x41,
0x72, 0x6d, 0x1eb9, 0x301, 0x6e, 0x69, 0x61, 0x20, 0x48, 0x1ecd, 0x301, 0x74,
0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x41,
0x74, 0x6c, 0xe1, 0x6e, 0x74, 0xed, 0x6b, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x41, 0x74, 0x6c, 0xe1, 0x6e, 0x74, 0xed, 0x6b, 0x20, 0x46, 0xed, 0x6b,
0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x41, 0x74, 0x6c, 0xe1, 0x6e,
0x74, 0xed, 0x6b, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x4d, 0xed, 0x64, 0xfa, 0x6c, 0x20, 0x1ecc, 0x73, 0x74,
0x72, 0xe9, 0x6c, 0x69, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x1ecc, 0x73,
0x74, 0x72, 0xe9, 0x6c, 0x69, 0x61, 0x20, 0x4d, 0xed, 0x64, 0xfa, 0x6c,
0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x1ecc,
0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0x61, 0x20, 0x4d, 0xed, 0x64, 0xfa,
0x6c, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x1ecc, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0x61, 0x20, 0x4d, 0xed,
0x64, 0xfa, 0x6c, 0x20, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0xe1, 0x6e, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x1ecc, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0x61,
0x20, 0x4d, 0xed, 0x64, 0xfa, 0x6c, 0x20, 0x57, 0x1eb9, 0x301, 0x73, 0x74,
0xe1, 0x6e, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x1ecc, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0x61, 0x20, 0x4d, 0xed,
0x64, 0xfa, 0x6c, 0x20, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0xe1, 0x6e, 0x20,
0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xcd,
0x73, 0x74, 0xe1, 0x6e, 0x20, 0x1ecc, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69,
0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x1ecc, 0x73, 0x74, 0x72, 0xe9, 0x6c,
0x69, 0x61, 0x20, 0xcd, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0x46, 0xed, 0x6b,
0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x1ecc, 0x73, 0x74, 0x72, 0xe9,
0x6c, 0x69, 0x61, 0x20, 0xcd, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0x44, 0xe9,
0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x57, 0x1eb9, 0x301,
0x73, 0x74, 0xe1, 0x6e, 0x20, 0x1ecc, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69,
0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x1ecc, 0x73, 0x74, 0x72, 0xe9, 0x6c,
0x69, 0x61, 0x20, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0x46,
0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x1ecc, 0x73, 0x74,
0x72, 0xe9, 0x6c, 0x69, 0x61, 0x20, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0xe1,
0x6e, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x41, 0x7a, 0x1eb9, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x41, 0x7a, 0x1eb9, 0x72, 0x62, 0x61, 0x69, 0x6a,
0x61, 0x6e, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x41, 0x7a, 0x1eb9, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x20,
0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x41, 0x7a, 0x1ecd, 0x7a, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x41, 0x7a, 0x1ecd, 0x7a, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x41, 0x7a, 0x1ecd, 0x7a, 0x20, 0x48, 0x1ecd, 0x301, 0x74,
0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x1eb9, 0x73, 0x68, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x1eb9, 0x73, 0x68,
0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x1eb9, 0x73, 0x68, 0x20, 0x44, 0xe9,
0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x42, 0x75, 0x74,
0x61, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x42, 0x6f, 0x6c, 0xed, 0x76,
0x69, 0x61, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x20, 0x46,
0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x42, 0x72, 0x61,
0x73, 0xed, 0x6c, 0x69, 0x61, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53,
0xed, 0x7a, 0xed, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x42, 0x72, 0x75,
0x6e, 0x1eb9, 0x69, 0x20, 0x44, 0x61, 0x72, 0xfa, 0x73, 0x61, 0x6c, 0x61,
0x6d, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0x65, 0x70, 0x20, 0x56, 0x1eb9,
0x301, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0x65, 0x70, 0x20, 0x56,
0x1eb9, 0x301, 0x64, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x4b, 0x65, 0x70, 0x20, 0x56, 0x1eb9, 0x301, 0x64, 0x20, 0x48,
0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x43, 0x68, 0x61, 0x6d, 0xf3, 0x72, 0x6f, 0x20, 0x46, 0xed,
0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x43, 0x68, 0xe1, 0x74,
0x61, 0x6d, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x43, 0x68, 0xe1, 0x74, 0x61,
0x6d, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x43, 0x68, 0xe1, 0x74, 0x61, 0x6d, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed,
0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x43, 0x68, 0xed, 0x6c, 0x1eb9, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x43, 0x68, 0xed, 0x6c, 0x1eb9, 0x20, 0x46, 0xed,
0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x43, 0x68, 0xed, 0x6c,
0x1eb9, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0xed, 0x6e,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x43, 0x68, 0x61, 0xed, 0x6e, 0x61, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x43, 0x68, 0x61, 0xed, 0x6e, 0x61, 0x20, 0x46,
0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x43, 0x68, 0x61,
0xed, 0x6e, 0x61, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x4b, 0x72, 0xed, 0x73, 0x6d, 0xe1, 0x73, 0x20, 0x41,
0xed, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0xf3,
0x6b, 0xf3, 0x73, 0x20, 0x41, 0xed, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x4b, 0x6f, 0x6c, 0xf3, 0x6d, 0x62, 0x69, 0x61,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0x6f, 0x6c, 0xf3, 0x6d, 0x62, 0x69,
0x61, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x4b, 0x6f, 0x6c, 0xf3, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x48, 0x1ecd, 0x301,
0x74, 0x20, 0x53, 0xed, 0x7a, 0xed, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x4b, 0xfa, 0x6b, 0x20, 0x41, 0xed, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x4b, 0xfa, 0x6b, 0x20, 0x41, 0xed, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x4b, 0xfa, 0x6b, 0x20, 0x41, 0xed, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x20, 0x48, 0x61, 0x66, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53,
0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0xfa, 0x62,
0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0xfa, 0x62, 0x61, 0x20, 0x46,
0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0xfa, 0x62,
0x61, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x44, 0xe9, 0x76, 0x69, 0x73, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x44,
0x69, 0xfa, 0x6d, 0x1ecd, 0x301, 0x6e, 0x2d, 0x64, 0x2019, 0x55, 0x76, 0x69,
0x6c, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xcd, 0x73, 0x74, 0x20, 0x54, 0xed,
0x6d, 0x1ecd, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xcd, 0x73, 0x74, 0x61, 0x20,
0x54, 0x61, 0x69, 0x6d, 0xcd, 0x73, 0x74, 0x61, 0x20, 0x46, 0xed, 0x6b,
0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xcd, 0x73, 0x74, 0x61, 0x20,
0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0xed, 0x6e, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x1eb8, 0x301, 0x6b, 0x77, 0x75, 0xe1, 0x64, 0x1ecd, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x4d, 0xed, 0x64, 0xfa, 0x6c, 0x20, 0x59, 0xfa,
0x72, 0x6f, 0x70, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0xed, 0x64, 0xfa,
0x6c, 0x20, 0x59, 0xfa, 0x72, 0x6f, 0x70, 0x20, 0x46, 0xed, 0x6b, 0x73,
0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0xed, 0x64, 0xfa, 0x6c, 0x20,
0x59, 0xfa, 0x72, 0x6f, 0x70, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53,
0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xcd, 0x73, 0x74,
0xe1, 0x6e, 0x20, 0x59, 0xfa, 0x72, 0x6f, 0x70, 0x20, 0x54, 0x61, 0x69,
0x6d, 0xcd, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0x59, 0xfa, 0x72, 0x6f, 0x70,
0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xcd,
0x73, 0x74, 0xe1, 0x6e, 0x20, 0x59, 0xfa, 0x72, 0x6f, 0x70, 0x20, 0x48,
0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x46, 0x61, 0xe1, 0x2d, 0xcd, 0x73, 0x74, 0xe1, 0x6e, 0x20,
0x59, 0xfa, 0x72, 0x6f, 0x70, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x57, 0x1eb9,
0x301, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0x59, 0xfa, 0x72, 0x6f, 0x70, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0xe1, 0x6e, 0x20,
0x59, 0xfa, 0x72, 0x6f, 0x70, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0xe1, 0x6e, 0x20,
0x59, 0xfa, 0x72, 0x6f, 0x70, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53,
0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x46, 0x1ecd, 0x301,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x46,
0x1ecd, 0x301, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x46, 0xed, 0x6b,
0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x46, 0x1ecd, 0x301, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed,
0x7a, 0xed, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x46, 0xed, 0x6a, 0x69,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x46, 0xed, 0x6a, 0x69, 0x20, 0x46, 0xed,
0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x46, 0xed, 0x6a, 0x69,
0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0xed, 0x6e, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x46, 0x72, 0x1eb9, 0x301, 0x6e, 0x63, 0x68, 0x20,
0x47, 0x69, 0xe1, 0x6e, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x46, 0x72,
0x1eb9, 0x301, 0x6e, 0x63, 0x68, 0x20, 0x53, 0x61, 0xfa, 0x74, 0x61, 0x6e,
0x20, 0x61, 0x6e, 0x20, 0x41, 0x6e, 0x74, 0xe1, 0x74, 0xed, 0x6b, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x47, 0x61, 0x6c, 0xe1, 0x70, 0xe1, 0x67, 0x1ecd,
0x73, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x47, 0xe1, 0x6d, 0x62, 0x69, 0x1eb9,
0x72, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4a, 0x1ecd, 0x301, 0x6a, 0x69, 0x61,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x4a, 0x1ecd, 0x301, 0x6a, 0x69, 0x61, 0x20,
0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4a, 0x1ecd,
0x301, 0x6a, 0x69, 0x61, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed,
0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x47, 0xed, 0x6c, 0x62,
0x61, 0x74, 0x20, 0x41, 0xed, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x47, 0x72, 0xed, 0x6e, 0x77, 0xed, 0x63, 0x68, 0x20,
0x4d, 0xed, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xcd, 0x69, 0x73, 0x74,
0x20, 0x47, 0x72, 0xed, 0x6e, 0x6c, 0xe1, 0x6e, 0x64, 0x20, 0x54, 0x61,
0x69, 0x6d, 0xcd, 0x69, 0x73, 0x74, 0x20, 0x47, 0x72, 0xed, 0x6e, 0x6c,
0xe1, 0x6e, 0x64, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61,
0x69, 0x6d, 0xcd, 0x69, 0x73, 0x74, 0x20, 0x47, 0x72, 0xed, 0x6e, 0x6c,
0xe1, 0x6e, 0x64, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a,
0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x57, 0x1eb9, 0x301, 0x73, 0x74,
0x20, 0x47, 0x72, 0xed, 0x6e, 0x6c, 0xe1, 0x6e, 0x64, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0x20, 0x47, 0x72, 0xed, 0x6e,
0x6c, 0xe1, 0x6e, 0x64, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0x20, 0x47, 0x72, 0xed,
0x6e, 0x6c, 0xe1, 0x6e, 0x64, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53,
0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x47, 0x1ecd, 0x301,
0x6c, 0x66, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x47, 0x61, 0x79, 0xe1, 0x6e, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x48, 0x61, 0x77, 0x61, 0x69, 0xed, 0x2d, 0x45, 0x6c, 0xfa, 0x73, 0x68,
0x69, 0xe1, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x48, 0x61, 0x77, 0x61,
0x69, 0xed, 0x2d, 0x45, 0x6c, 0xfa, 0x73, 0x68, 0x69, 0xe1, 0x6e, 0x20,
0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x48, 0x61,
0x77, 0x61, 0x69, 0xed, 0x2d, 0x45, 0x6c, 0xfa, 0x73, 0x68, 0x69, 0xe1,
0x6e, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x48, 0x1ecd, 0x6e, 0x67, 0x20, 0x4b, 0x1ecd, 0x6e, 0x67, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x48, 0x1ecd, 0x6e, 0x67, 0x20, 0x4b, 0x1ecd, 0x6e, 0x67,
0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x48,
0x1ecd, 0x6e, 0x67, 0x20, 0x4b, 0x1ecd, 0x6e, 0x67, 0x20, 0x48, 0x1ecd, 0x301,
0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x48, 0x6f, 0x76, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x48, 0x6f, 0x76,
0x64, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x48, 0x6f, 0x76, 0x64, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed,
0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xcd, 0x6e, 0x64, 0x69,
0x61, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d,
0xcd, 0x6e, 0x64, 0x69, 0xe1, 0x6e, 0x20, 0xd3, 0x73, 0x68, 0x1eb9, 0x301,
0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x49, 0x6e, 0x64, 0x6f, 0x63, 0x68,
0x61, 0xed, 0x6e, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0xed, 0x64,
0xfa, 0x6c, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x1eb9, 0x301, 0x73, 0x68,
0x69, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xcd, 0x73, 0x74, 0xe1, 0x6e,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x1eb9, 0x301, 0x73, 0x68, 0x69, 0x61,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0xe1, 0x6e,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x1eb9, 0x301, 0x73, 0x68, 0x69, 0x61,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x44, 0xe9,
0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x49, 0x72, 0x6b,
0xfa, 0x74, 0x73, 0x6b, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x49, 0x72, 0x6b,
0xfa, 0x74, 0x73, 0x6b, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x49, 0x72, 0x6b, 0xfa, 0x74, 0x73, 0x6b, 0x20, 0x48,
0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61,
0x69, 0x6d, 0xcd, 0x7a, 0x72, 0x1eb9, 0x6c, 0x20, 0x54, 0x61, 0x69, 0x6d,
0xcd, 0x7a, 0x72, 0x1eb9, 0x6c, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20,
0x54, 0x61, 0x69, 0x6d, 0xcd, 0x7a, 0x72, 0x1eb9, 0x6c, 0x20, 0x44, 0xe9,
0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4a, 0x61, 0x70,
0x61, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4a, 0x61, 0x70, 0x61, 0x6e,
0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4a,
0x61, 0x70, 0x61, 0x6e, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x4b, 0x61, 0x7a, 0xe9, 0x6b, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xcd, 0x73, 0x74, 0x20, 0x4b, 0x61,
0x7a, 0xe9, 0x6b, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x57, 0x1eb9, 0x301, 0x73, 0x74, 0x20, 0x4b, 0x61, 0x7a, 0xe9, 0x6b, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0x6f, 0x72, 0x69,
0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0x6f, 0x72, 0x69, 0x61, 0x20,
0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0x6f,
0x72, 0x69, 0x61, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x4b, 0x1ecd, 0x301, 0x73, 0x72, 0x61, 0x1eb9, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x73,
0x6b, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x79, 0x61, 0x73, 0x6b, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x73,
0x6b, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0x1eb9, 0x67, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4c, 0x61, 0xed, 0x6e, 0x20, 0x41,
0xed, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4c,
0x1ecd, 0x64, 0x20, 0x48, 0x61, 0xfa, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4c,
0x1ecd, 0x64, 0x20, 0x48, 0x61, 0xfa, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x4c, 0x1ecd, 0x64, 0x20, 0x48, 0x61, 0xfa,
0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x4d, 0xe1, 0x67, 0xe1, 0x64, 0x61, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x4d, 0xe1, 0x67, 0xe1, 0x64, 0x61, 0x6e, 0x20, 0x46, 0xed, 0x6b, 0x73,
0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0xe1, 0x67, 0xe1, 0x64, 0x61,
0x6e, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0x61, 0x6c, 0xe9, 0x73, 0x68, 0x69,
0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0x1ecd, 0x301, 0x64, 0x69, 0x76,
0x73, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0x61, 0x6b, 0x77, 0x1eb9, 0x301,
0x73, 0x61, 0x73, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0xe1, 0x73, 0x68,
0x61, 0x6c, 0x20, 0x41, 0xed, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x4d, 0x1ecd, 0x72, 0xed, 0x73, 0x68, 0x1ecd, 0x73, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x4d, 0x1ecd, 0x72, 0xed, 0x73, 0x68, 0x1ecd, 0x73,
0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d,
0x1ecd, 0x72, 0xed, 0x73, 0x68, 0x1ecd, 0x73, 0x20, 0x48, 0x1ecd, 0x301, 0x74,
0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d,
0x1ecd, 0x301, 0x73, 0x1ecd, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0x1eb9,
0x301, 0x6b, 0x73, 0xed, 0x6b, 0xf3, 0x20, 0x50, 0x61, 0x73, 0xed, 0x66,
0xed, 0x6b, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0x1eb9, 0x301, 0x6b, 0x73,
0xed, 0x6b, 0xf3, 0x20, 0x50, 0x61, 0x73, 0xed, 0x66, 0xed, 0x6b, 0x20,
0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0x1eb9,
0x301, 0x6b, 0x73, 0xed, 0x6b, 0xf3, 0x20, 0x50, 0x61, 0x73, 0xed, 0x66,
0xed, 0x6b, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x4d, 0x1ecd, 0x6e, 0x67, 0xf3, 0x6c, 0x69, 0x61, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x4d, 0x1ecd, 0x6e, 0x67, 0xf3, 0x6c, 0x69, 0x61, 0x20,
0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0x1ecd,
0x6e, 0x67, 0xf3, 0x6c, 0x69, 0x61, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20,
0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0x1ecd,
0x301, 0x73, 0x6b, 0x6f, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0x1ecd, 0x301,
0x73, 0x6b, 0x6f, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x4d, 0x1ecd, 0x301, 0x73, 0x6b, 0x6f, 0x20, 0x48, 0x1ecd, 0x301,
0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x4d, 0x69, 0xe1, 0x6e, 0x6d, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e,
0x61, 0xfa, 0x72, 0x75, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x1eb9, 0x70,
0x1ecd, 0x6c, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x69, 0xfa, 0x20, 0x4b,
0x61, 0x6c, 0x1eb9, 0x64, 0xf3, 0x6e, 0x69, 0x61, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x4e, 0x69, 0xfa, 0x20, 0x4b, 0x61, 0x6c, 0x1eb9, 0x64, 0xf3, 0x6e,
0x69, 0x61, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x4e, 0x69, 0xfa, 0x20, 0x4b, 0x61, 0x6c, 0x1eb9, 0x64, 0xf3, 0x6e,
0x69, 0x61, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69,
0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x69, 0xfa, 0x20, 0x5a, 0x69,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x69, 0xfa,
0x20, 0x5a, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x46, 0xed, 0x6b, 0x73,
0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x69, 0xfa, 0x20, 0x5a, 0x69,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x4e, 0x69, 0xfa, 0x66, 0x61, 0xfa, 0x6e, 0x64,
0x6c, 0xe1, 0x6e, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x69, 0xfa,
0x66, 0x61, 0xfa, 0x6e, 0x64, 0x6c, 0xe1, 0x6e, 0x64, 0x20, 0x46, 0xed,
0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x69, 0xfa, 0x66,
0x61, 0xfa, 0x6e, 0x64, 0x6c, 0xe1, 0x6e, 0x64, 0x20, 0x44, 0xe9, 0x6c,
0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x69, 0xfa, 0x1eb9,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x1ecd, 0x301, 0x72, 0x66, 0x1ecd, 0x301,
0x6c, 0x6b, 0x20, 0x41, 0xed, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x4e, 0x1ecd, 0x301, 0x72, 0x66, 0x1ecd, 0x301, 0x6c, 0x6b, 0x20,
0x41, 0xed, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x1ecd, 0x301, 0x72, 0x66, 0x1ecd, 0x301,
0x6c, 0x6b, 0x20, 0x41, 0xed, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x44, 0xe9,
0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x46, 0x1eb9, 0x72,
0x6e, 0xe1, 0x6e, 0x64, 0xf3, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x1ecd, 0x72,
0xf3, 0x6e, 0x69, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x46, 0x1eb9, 0x72,
0x6e, 0xe1, 0x6e, 0x64, 0xf3, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x1ecd, 0x72,
0xf3, 0x6e, 0x69, 0x61, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x46, 0x1eb9, 0x72, 0x6e, 0xe1, 0x6e, 0x64, 0xf3, 0x20,
0x64, 0x69, 0x20, 0x4e, 0x1ecd, 0x72, 0xf3, 0x6e, 0x69, 0x61, 0x20, 0x48,
0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0xed, 0x6e, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x73, 0x6b,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x73, 0x6b, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x73, 0x6b,
0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x1ecc, 0x6d, 0x73, 0x6b, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x1ecc, 0x6d, 0x73, 0x6b, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x1ecc, 0x6d, 0x73, 0x6b, 0x20, 0x48, 0x1ecd, 0x301,
0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x50, 0xe1, 0x6b, 0xed, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x50, 0xe1, 0x6b, 0xed, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x46, 0xed,
0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x50, 0xe1, 0x6b, 0xed,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed,
0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x50, 0x61, 0x6c, 0xe1,
0x75, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x50, 0xe1, 0x70, 0x75, 0xe1, 0x20,
0x4e, 0x69, 0xfa, 0x20, 0x47, 0xed, 0x6e, 0x69, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x50, 0xe1, 0x72, 0xe1, 0x67, 0x77, 0x75, 0x65, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x50, 0xe1, 0x72, 0xe1, 0x67, 0x77, 0x75, 0x65, 0x20, 0x46,
0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x50, 0xe1, 0x72,
0xe1, 0x67, 0x77, 0x75, 0x65, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53,
0xed, 0x7a, 0xed, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x50, 0x1eb9, 0x72,
0x75, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x50, 0x1eb9, 0x72, 0x75, 0x20, 0x46,
0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x50, 0x1eb9, 0x72,
0x75, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0xed, 0x6e,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x46, 0xed, 0x6c, 0xed, 0x70, 0x69, 0x6e,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x46, 0xed, 0x6c, 0xed, 0x70, 0x69, 0x6e,
0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x46,
0xed, 0x6c, 0xed, 0x70, 0x69, 0x6e, 0x20, 0x48, 0x1ecd, 0x74, 0x20, 0x53,
0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x46, 0xed, 0x6e,
0x69, 0x6b, 0x73, 0x20, 0x41, 0xed, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x53, 0x65, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x1eb9,
0x72, 0x20, 0x26, 0x20, 0x4d, 0xed, 0x6b, 0x1eb9, 0x6c, 0x1ecd, 0x6e, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x53, 0x65, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x1eb9,
0x72, 0x20, 0x61, 0x6e, 0x20, 0x4d, 0xed, 0x6b, 0x1eb9, 0x6c, 0x1ecd, 0x6e,
0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x53,
0x65, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x1eb9, 0x72, 0x20, 0x61, 0x6e, 0x20,
0x4d, 0xed, 0x6b, 0x1eb9, 0x6c, 0x1ecd, 0x6e, 0x20, 0x44, 0xe9, 0x6c, 0x61,
0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x50, 0xed, 0x74, 0x6b, 0x61,
0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x50, 0xf3, 0x6e, 0xe1, 0x70, 0x1eb9,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x50, 0x69, 0x1ecd, 0x6e, 0x67, 0x79, 0x61,
0x6e, 0x67, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x52, 0x69, 0x79, 0xfa, 0x6e,
0x69, 0x1ecd, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x52, 0x6f, 0x74, 0x1eb9,
0x301, 0x72, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x53, 0xe1, 0x6b, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x53, 0xe1, 0x6b,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x53, 0xe1, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x53, 0xe1, 0x6d, 0x6f, 0xe1, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x53, 0xe1, 0x6d, 0x6f, 0xe1, 0x20, 0x46, 0xed, 0x6b, 0x73,
0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x53, 0xe1, 0x6d, 0x6f, 0xe1, 0x20,
0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x53,
0x1eb9, 0x301, 0x63, 0x68, 0x1eb9, 0x6c, 0x73, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x1ecd, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x53, 0xf3, 0x6c, 0xf3, 0x6d, 0x1ecd, 0x301, 0x6e, 0x20, 0x41, 0xed, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x53, 0x61, 0xfa,
0x74, 0x20, 0x4a, 0x1ecd, 0x301, 0x6a, 0x69, 0x61, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x53, 0xfa, 0x72, 0xed, 0x6e, 0x61, 0x6d, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x53, 0x69, 0xf3, 0x77, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x54,
0x61, 0x68, 0xed, 0x74, 0x69, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x54, 0x61,
0x69, 0x70, 0x1eb9, 0x69, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x54, 0x61, 0x69,
0x70, 0x1eb9, 0x69, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x54, 0x61, 0x69, 0x70, 0x1eb9, 0x69, 0x20, 0x44, 0xe9, 0x6c,
0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x54, 0x61, 0x6a, 0xed,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x54,
0x6f, 0x6b, 0x1eb9, 0x6c, 0xe1, 0x75, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x54,
0x1ecd, 0x301, 0x6e, 0x67, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x54, 0x1ecd,
0x301, 0x6e, 0x67, 0x61, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x54, 0x1ecd, 0x301, 0x6e, 0x67, 0x61, 0x20, 0x48, 0x1ecd,
0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x43, 0x68, 0x75, 0x6b, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x54, 0x1ecd,
0x6b, 0x6d, 0x1eb9, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x54, 0x1ecd, 0x6b, 0x6d, 0x1eb9, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x54, 0x1ecd, 0x6b, 0x6d, 0x1eb9, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x54, 0x75, 0x76, 0xe1, 0x6c, 0x75, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x59, 0xfa, 0x72, 0x75, 0x67, 0x77, 0x75, 0x65, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x59, 0xfa, 0x72, 0x75, 0x67, 0x77, 0x75, 0x65, 0x20,
0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x59, 0xfa,
0x72, 0x75, 0x67, 0x77, 0x75, 0x65, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20,
0x53, 0xed, 0x7a, 0xed, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x55, 0x7a,
0x62, 0x1eb9, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x55, 0x7a, 0x62, 0x1eb9, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x55, 0x7a,
0x62, 0x1eb9, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x48, 0x1ecd, 0x301,
0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x56, 0x61, 0x6e, 0x75, 0xe1, 0x74, 0x75, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x56, 0x61, 0x6e, 0x75, 0xe1, 0x74, 0x75, 0x20, 0x46, 0xed, 0x6b, 0x73,
0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x56, 0x61, 0x6e, 0x75, 0xe1, 0x74,
0x75, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x56, 0x1eb9, 0x6e, 0x1eb9, 0x7a, 0x75, 0x1eb9, 0x301, 0x6c, 0x61, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x1ecd, 0x73, 0x74,
0x1ecd, 0x6b, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x1ecd, 0x73, 0x74, 0x1ecd, 0x6b, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed,
0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x56, 0x6f, 0x6c, 0x76,
0xf3, 0x67, 0x72, 0x61, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x56, 0x6f,
0x6c, 0x76, 0xf3, 0x67, 0x72, 0x61, 0x64, 0x20, 0x46, 0xed, 0x6b, 0x73,
0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x56, 0x6f, 0x6c, 0x76, 0xf3, 0x67,
0x72, 0x61, 0x64, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a,
0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x56, 0x1ecd, 0x301, 0x73, 0x74,
0x1ecd, 0x6b, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x57, 0xe9, 0x6b, 0x20, 0x41,
0xed, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x57, 0xe1,
0x6c, 0x69, 0x73, 0x20, 0x61, 0x6e, 0x20, 0x46, 0xfa, 0x74, 0xfa, 0x6e,
0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x59, 0xe9, 0x6b, 0xfa, 0x74, 0x73,
0x6b, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x59, 0xe9, 0x6b, 0xfa, 0x74, 0x73,
0x6b, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x59, 0xe9, 0x6b, 0xfa, 0x74, 0x73, 0x6b, 0x20, 0x48, 0x1ecd, 0x301, 0x74,
0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x59,
0x1eb9, 0x6b, 0x65, 0x74, 0x1eb9, 0x72, 0xed, 0x6e, 0x62, 0x75, 0x67, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x59, 0x1eb9, 0x6b, 0x65, 0x74, 0x1eb9, 0x72, 0xed,
0x6e, 0x62, 0x75, 0x67, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x59, 0x1eb9, 0x6b, 0x65, 0x74, 0x1eb9, 0x72, 0xed, 0x6e,
0x62, 0x75, 0x67, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a,
0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x59, 0x75, 0x6b, 0xf3, 0x6e,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x7ca, 0x7dd, 0x7d1, 0x7dc, 0x7ed, 0x7ca, 0x7e3,
0x7cc, 0x7db, 0x7d1, 0x7d5, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7dd, 0x7d8, 0x7ca, 0x7ec, 0x7dd, 0x7cc, 0x7f2, 0x7ec,
0x7e0, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ca, 0x7f2, 0x7d3, 0x7ca, 0x7f2, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d9, 0x7cb, 0x7f2, 0x20, 0x7e2, 0x7ca, 0x7d3,
0x7d8, 0x7cd, 0x7dd, 0x7d8, 0x7ca, 0x7ec, 0x7cc, 0x7f2, 0x7ec, 0x7e0, 0x7ca, 0x7eb,
0x20, 0x7d3, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7d0, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x7d9, 0x7cb, 0x7f2, 0x20, 0x7e2, 0x7ca, 0x7d3, 0x7d8, 0x7cd, 0x7dd,
0x7d8, 0x7cc, 0x7ec, 0x7dd, 0x7cc, 0x7f2, 0x7ec, 0x7e0, 0x7ca, 0x7eb, 0x20, 0x7e5,
0x7d9, 0x7cf, 0x7ec, 0x7d8, 0x7ce, 0x7ee, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x7d9, 0x7cb, 0x7f2, 0x20, 0x7e2, 0x7ca, 0x7d3, 0x7d8, 0x7cd, 0x7dd, 0x7d8,
0x7ca, 0x7ec, 0x7dd, 0x7cc, 0x7f2, 0x7ec, 0x7e0, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7df,
0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d9,
0x7cb, 0x7f2, 0x7dd, 0x7d8, 0x7ca, 0x7ec, 0x7dd, 0x7cc, 0x7f2, 0x7ec, 0x7e0, 0x7ca,
0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x20, 0x7d5, 0x7ce, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x7d9, 0x7cb, 0x7f2, 0x20, 0x7e2, 0x7ca, 0x7d3, 0x7d8, 0x7cd,
0x7dd, 0x7d8, 0x7ca, 0x7ec, 0x7dd, 0x7cc, 0x7f2, 0x7ec, 0x7e0, 0x7ca, 0x7eb, 0x20,
0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7ca, 0x7d9, 0x7cb, 0x7f2, 0x7ca, 0x7df,
0x7ca, 0x7db, 0x7d1, 0x7de, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x7d8, 0x7ca, 0x7ca, 0x7df, 0x7ca, 0x7db, 0x7d1, 0x7de, 0x7ca, 0x7eb, 0x20,
0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca,
0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7ca, 0x7df, 0x7ca, 0x7db, 0x7d1, 0x7de,
0x7ca, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5,
0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7ca, 0x7e1, 0x7ca, 0x7d6, 0x7ed, 0x7cf, 0x7f2, 0x7eb,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7ca, 0x7e1, 0x7ca,
0x7d6, 0x7ed, 0x7cf, 0x7e3, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec,
0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7ca,
0x7e1, 0x7ca, 0x7d6, 0x7ed, 0x7cf, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7ca, 0x7e1, 0x7cb,
0x7d9, 0x7cc, 0x7de, 0x7cc, 0x7ec, 0x20, 0x7d5, 0x7ca, 0x7f2, 0x7d3, 0x7ca, 0x7f2,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7de, 0x7d0, 0x7f0,
0x7d8, 0x7ce, 0x7ee, 0x20, 0x7ca, 0x7e1, 0x7cb, 0x7d9, 0x7cc, 0x7de, 0x7cc, 0x7ec,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ca, 0x7f2,
0x7d3, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca,
0x7f2, 0x7d5, 0x7ca, 0x7de, 0x7d0, 0x7f0, 0x7d8, 0x7ce, 0x7ee, 0x20, 0x7ca, 0x7e1,
0x7cb, 0x7d9, 0x7cc, 0x7de, 0x7cc, 0x7ec, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7de, 0x7d0, 0x7f0, 0x7d8,
0x7ce, 0x7f0, 0x20, 0x7ca, 0x7e1, 0x7cb, 0x7d9, 0x7cc, 0x7de, 0x7cc, 0x7ec, 0x20,
0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7d0, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7de, 0x7d0, 0x7f0, 0x7d8, 0x7ce, 0x7f0, 0x20, 0x7ca,
0x7e1, 0x7cb, 0x7d9, 0x7cc, 0x7de, 0x7cc, 0x7ec, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec,
0x7d3, 0x7d0, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db,
0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7de, 0x7d0, 0x7f0,
0x7d8, 0x7ce, 0x7f0, 0x20, 0x7ca, 0x7e1, 0x7cb, 0x7d9, 0x7cc, 0x7de, 0x7cc, 0x7ec,
0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1,
0x7d8, 0x7ca, 0x7de, 0x7d9, 0x7ce, 0x7ec, 0x7de, 0x7ca, 0x7f2, 0x7ec, 0x7e0, 0x7ca,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7de, 0x7d9, 0x7ce,
0x7ec, 0x7de, 0x7ca, 0x7f2, 0x7ec, 0x7e0, 0x7ca, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2,
0x7d5, 0x7ca, 0x7de, 0x7d9, 0x7ce, 0x7ec, 0x7de, 0x7ca, 0x7f2, 0x7ec, 0x7e0, 0x7ca,
0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1,
0x7d8, 0x7ca, 0x7de, 0x7d0, 0x7f0, 0x7d8, 0x7ce, 0x7f0, 0x20, 0x7ca, 0x7e1, 0x7cb,
0x7d9, 0x7cc, 0x7de, 0x7cc, 0x7ec, 0x20, 0x7e3, 0x7cc, 0x7eb, 0x20, 0x7d6, 0x7d0,
0x7eb, 0x20, 0x7df, 0x7cc, 0x7f2, 0x7d3, 0x7ca, 0x7f2, 0x20, 0x7d5, 0x7ce, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7de, 0x7d0, 0x7f0, 0x7d8, 0x7ce, 0x7f0, 0x20,
0x7ca, 0x7e1, 0x7cb, 0x7d9, 0x7cc, 0x7de, 0x7cc, 0x7ec, 0x20, 0x7e3, 0x7cc, 0x7eb,
0x20, 0x7d6, 0x7d0, 0x7eb, 0x20, 0x7df, 0x7cc, 0x7f2, 0x7d3, 0x7ca, 0x7f2, 0x20,
0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca,
0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7de, 0x7d0, 0x7f0, 0x7d8, 0x7ce, 0x7f0,
0x20, 0x7ca, 0x7e1, 0x7cb, 0x7d9, 0x7cc, 0x7de, 0x7cc, 0x7ec, 0x20, 0x7e3, 0x7cc,
0x7eb, 0x20, 0x7d6, 0x7d0, 0x7eb, 0x20, 0x7df, 0x7cc, 0x7f2, 0x7d3, 0x7ca, 0x7f2,
0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1,
0x7d8, 0x7ca, 0x7ca, 0x7d4, 0x7cc, 0x7e6, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7ca, 0x7d4, 0x7cc, 0x7e6, 0x7ca, 0x7eb, 0x20,
0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca,
0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7ca, 0x7d4, 0x7cc, 0x7e6, 0x7ca, 0x7eb,
0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7ec,
0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca,
0x7f2, 0x7d5, 0x7ca, 0x7ca, 0x7d9, 0x7ca, 0x7d3, 0x7ce, 0x7df, 0x7ca, 0x7eb, 0x20,
0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7ca, 0x7d9, 0x7ca, 0x7d3,
0x7ce, 0x7df, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec,
0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7ca,
0x7d9, 0x7ca, 0x7d3, 0x7ce, 0x7df, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x20, 0x7db, 0x7ce,
0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7ca, 0x7d9, 0x7d1, 0x7d6,
0x7ed, 0x7ca, 0x7f2, 0x7d5, 0x7cc, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7ca, 0x7d9, 0x7d1, 0x7d6, 0x7ed, 0x7ca, 0x7f2, 0x7d5,
0x7cc, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20,
0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7ca, 0x7d9,
0x7d1, 0x7d6, 0x7ed, 0x7ca, 0x7f2, 0x7d5, 0x7cc, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7df,
0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7d5,
0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x20, 0x7ca, 0x7d9, 0x7d1, 0x7d6, 0x7ed, 0x7ca,
0x7f2, 0x7d5, 0x7cc, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x7d8, 0x7ca, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x20, 0x7d5, 0x7ce, 0x7ec,
0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca,
0x7f2, 0x7d5, 0x7ca, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x20, 0x7ca, 0x7d9,
0x7d1, 0x7d6, 0x7ed, 0x7ca, 0x7f2, 0x7d5, 0x7cc, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7df,
0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7ca,
0x7d9, 0x7d1, 0x7e1, 0x7cb, 0x7e3, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7ca, 0x7d9, 0x7d1, 0x7e1, 0x7cb, 0x7e3, 0x7cc, 0x7eb,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1,
0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7ca, 0x7d9, 0x7d1, 0x7e1, 0x7cb,
0x7e3, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20,
0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8,
0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7df, 0x7cc, 0x7f2, 0x7d3, 0x7ca, 0x7f2, 0x2d, 0x7e1,
0x7ca, 0x7f2, 0x7de, 0x7ca, 0x7f2, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x7d8, 0x7ca, 0x7df, 0x7cc, 0x7f2, 0x7d3, 0x7ca, 0x7f2, 0x2d, 0x7e1, 0x7ca, 0x7f2,
0x7de, 0x7ca, 0x7f2, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20,
0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7df, 0x7cc,
0x7f2, 0x7d3, 0x7ca, 0x7f2, 0x2d, 0x7e1, 0x7ca, 0x7f2, 0x7de, 0x7ca, 0x7f2, 0x20,
0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8,
0x7ca, 0x7d5, 0x7cd, 0x7e1, 0x7df, 0x7ca, 0x20, 0x7cf, 0x7db, 0x7d1, 0x7d5, 0x7d9,
0x7ca, 0x7df, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8,
0x7ca, 0x7d5, 0x7cd, 0x7e1, 0x7df, 0x7ca, 0x20, 0x7cf, 0x7db, 0x7d1, 0x7d5, 0x7d9,
0x7ca, 0x7df, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec,
0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d5,
0x7cd, 0x7e1, 0x7df, 0x7ca, 0x20, 0x7cf, 0x7db, 0x7d1, 0x7d5, 0x7d9, 0x7ca, 0x7df,
0x7cc, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5,
0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7cf, 0x7db, 0x7d1, 0x7d5, 0x7d9, 0x7ca, 0x7df, 0x7cc,
0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x7ec, 0x2d, 0x7d5, 0x7cd,
0x7e1, 0x7df, 0x7ca, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca,
0x7cf, 0x7db, 0x7d1, 0x7d5, 0x7d9, 0x7ca, 0x7df, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7df,
0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x7ec, 0x2d, 0x7d5, 0x7cd, 0x7e1, 0x7df, 0x7ca, 0x20,
0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca,
0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7cf, 0x7db, 0x7d1, 0x7d5, 0x7d9, 0x7ca,
0x7df, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x7ec, 0x2d,
0x7d5, 0x7cd, 0x7e1, 0x7df, 0x7ca, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3,
0x7d0, 0x20, 0x7cf, 0x7db, 0x7d1, 0x7d5, 0x7d9, 0x7ca, 0x7df, 0x7cc, 0x7eb, 0x20,
0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7df, 0x7cb, 0x7ec,
0x7d3, 0x7d0, 0x20, 0x7cf, 0x7db, 0x7d1, 0x7d5, 0x7d9, 0x7ca, 0x7df, 0x7cc, 0x7eb,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1,
0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3,
0x7d0, 0x20, 0x7cf, 0x7db, 0x7d1, 0x7d5, 0x7d9, 0x7ca, 0x7df, 0x7cc, 0x7eb, 0x20,
0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8,
0x7ca, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x20, 0x7cf, 0x7db, 0x7d1, 0x7d5,
0x7d9, 0x7ca, 0x7df, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x7d8, 0x7ca, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x20, 0x7cf, 0x7db, 0x7d1,
0x7d5, 0x7d9, 0x7ca, 0x7df, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8,
0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5,
0x7ca, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x20, 0x7cf, 0x7db, 0x7d1, 0x7d5,
0x7d9, 0x7ca, 0x7df, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7ca, 0x7d6, 0x7ed, 0x7cd, 0x7d9,
0x7d1, 0x7d3, 0x7ca, 0x7e6, 0x7d1, 0x7d6, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7ca, 0x7d6, 0x7ed, 0x7cd, 0x7d9, 0x7d1,
0x7d3, 0x7ca, 0x7e6, 0x7d1, 0x7d6, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec,
0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca,
0x7f2, 0x7d5, 0x7ca, 0x7ca, 0x7d6, 0x7ed, 0x7cd, 0x7d9, 0x7d1, 0x7d3, 0x7ca, 0x7e6,
0x7d1, 0x7d6, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca,
0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7ca, 0x7d7, 0x7ed, 0x7d0, 0x7d9, 0x7db,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7ca, 0x7d7, 0x7ed,
0x7d0, 0x7d9, 0x7db, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20,
0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7ca, 0x7d7,
0x7ed, 0x7d0, 0x7d9, 0x7db, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7d3, 0x7ca, 0x7f2, 0x7dc, 0x7ed, 0x7d1,
0x7df, 0x7ca, 0x7d8, 0x7cd, 0x7db, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x7d8, 0x7ca, 0x7d3, 0x7ca, 0x7f2, 0x7dc, 0x7ed, 0x7d1, 0x7df, 0x7ca, 0x7d8, 0x7cd,
0x7db, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce,
0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d3, 0x7ca, 0x7f2, 0x7dc,
0x7ed, 0x7d1, 0x7df, 0x7ca, 0x7d8, 0x7cd, 0x7db, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7d3, 0x7ce, 0x7d5,
0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca,
0x7d3, 0x7cf, 0x7df, 0x7dd, 0x7ed, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d3, 0x7d9, 0x7cb, 0x7d6, 0x7ed, 0x7df, 0x7cc, 0x7df,
0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d3,
0x7d9, 0x7cb, 0x7d6, 0x7ed, 0x7df, 0x7cc, 0x7df, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8,
0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d3, 0x7d9, 0x7cb, 0x7d6, 0x7ed, 0x7df, 0x7cc, 0x7df,
0x7ca, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5,
0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7d3, 0x7d9, 0x7ce, 0x7e3, 0x7cb, 0x7e6, 0x7cc, 0x7eb,
0x20, 0x7d8, 0x7ca, 0x7d9, 0x7ce, 0x7db, 0x7d1, 0x7db, 0x7df, 0x7ca, 0x7e1, 0x20,
0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7dc, 0x7d9, 0x7cb, 0x7de,
0x7ce, 0x7f2, 0x7eb, 0x2d, 0x7dd, 0x7d9, 0x7cc, 0x7db, 0x7cc, 0x7eb, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d9, 0x7cb, 0x7f2, 0x7dc, 0x7d9, 0x7cb, 0x7de,
0x7ce, 0x7f2, 0x7eb, 0x2d, 0x7dd, 0x7d9, 0x7cc, 0x7db, 0x7cc, 0x7eb, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d9, 0x7cb, 0x7f2, 0x20, 0x7e2, 0x7ca, 0x7d3,
0x7d8, 0x7cd, 0x7dc, 0x7d9, 0x7cb, 0x7de, 0x7ce, 0x7f2, 0x7eb, 0x2d, 0x7dd, 0x7d9,
0x7cc, 0x7db, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x20, 0x7d5, 0x7ce, 0x7e1, 0x7ca, 0x7d9, 0x7cb, 0x7f2, 0x7db, 0x7ed, 0x7ca, 0x7e1,
0x7cf, 0x7d9, 0x7cf, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec,
0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d7,
0x7d5, 0x7ca, 0x7e1, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca,
0x7d7, 0x7d5, 0x7ca, 0x7e1, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec,
0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d7,
0x7d5, 0x7ca, 0x7e1, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20,
0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7db, 0x7df, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7db, 0x7df, 0x7cc, 0x7eb, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2,
0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7db, 0x7df, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7df,
0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7db,
0x7cc, 0x7e3, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7db,
0x7cc, 0x7e3, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db,
0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7db, 0x7cc, 0x7e3,
0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7de, 0x7d1, 0x7d9, 0x7cc, 0x7db, 0x7d1, 0x7e1,
0x7ca, 0x7db, 0x20, 0x7d5, 0x7cc, 0x7f2, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x7d8, 0x7ca, 0x7de, 0x7cf, 0x7de, 0x7cf, 0x7db, 0x20, 0x7d5, 0x7cc, 0x7f2,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7de, 0x7d0, 0x7df,
0x7d0, 0x7f2, 0x7d3, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x7d8, 0x7ca, 0x7de, 0x7d0, 0x7df, 0x7d0, 0x7f2, 0x7d3, 0x7cc, 0x7eb, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2,
0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7de, 0x7d0, 0x7df, 0x7d0, 0x7f2, 0x7d3, 0x7cc,
0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce,
0x7e1, 0x7d8, 0x7ca, 0x7de, 0x7cf, 0x7de, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x7d8, 0x7ca, 0x7de, 0x7cf, 0x7de, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8,
0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5,
0x7ca, 0x7de, 0x7cf, 0x7de, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1,
0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7de, 0x7ce, 0x7f3, 0x7d3, 0x7ca,
0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7de, 0x7ce,
0x7f3, 0x7d3, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec,
0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7de,
0x7ce, 0x7f1, 0x7d3, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7d8, 0x7ca, 0x7dd, 0x7ed, 0x7cc,
0x7db, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d8, 0x7ce,
0x7e1, 0x7d0, 0x7f2, 0x2d, 0x7d8, 0x7ce, 0x7d9, 0x7d1, 0x7dd, 0x7ed, 0x7cc, 0x7df,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7df, 0x7cb,
0x7ec, 0x7d3, 0x7d0, 0x20, 0x7d5, 0x7cc, 0x7e1, 0x7d0, 0x7d9, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7cc, 0x7db, 0x7d1, 0x7df, 0x7ca, 0x7f2,
0x7d8, 0x20, 0x7d5, 0x7cc, 0x7f2, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x7d8, 0x7ca, 0x7cc, 0x7db, 0x7d1, 0x7df, 0x7ca, 0x7f2, 0x7d8, 0x20, 0x7d5, 0x7cc,
0x7f2, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce,
0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7cc, 0x7db, 0x7d1, 0x7df,
0x7ca, 0x7f2, 0x7d8, 0x20, 0x7d5, 0x7cc, 0x7f2, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7cc, 0x7de, 0x7d1,
0x7e5, 0x7ca, 0x7d8, 0x7d0, 0x7d9, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x7d8, 0x7ca, 0x7d5, 0x7cd, 0x7e1, 0x7ca, 0x7eb, 0x20, 0x7cb, 0x7d9, 0x7d0, 0x7d4,
0x7ce, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5,
0x7cd, 0x7e1, 0x7ca, 0x7eb, 0x20, 0x7cb, 0x7d9, 0x7d0, 0x7d4, 0x7ce, 0x7eb, 0x20,
0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca,
0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d5, 0x7cd, 0x7e1, 0x7ca, 0x7eb, 0x20,
0x7cb, 0x7d9, 0x7d0, 0x7d4, 0x7ce, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7d5, 0x7df, 0x7cb, 0x7ec,
0x7d3, 0x7d0, 0x20, 0x7cb, 0x7d9, 0x7d0, 0x7d4, 0x7ce, 0x7eb, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7d0,
0x20, 0x7cb, 0x7d9, 0x7d0, 0x7d4, 0x7ce, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2,
0x7d5, 0x7ca, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7d0, 0x20, 0x7cb, 0x7d9, 0x7d0,
0x7d4, 0x7ce, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20,
0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7de, 0x7ca, 0x7df, 0x7cc, 0x7e3, 0x7cc, 0x7f2,
0x7dc, 0x7ed, 0x7d9, 0x7ca, 0x7d8, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x7d8, 0x7ca, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x20, 0x7cb, 0x7d9, 0x7d0,
0x7d4, 0x7ce, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca,
0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x20, 0x7cb, 0x7d9, 0x7d0, 0x7d4, 0x7ce,
0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce,
0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d5, 0x7df, 0x7cb, 0x7ec,
0x7d3, 0x7cb, 0x20, 0x7cb, 0x7d9, 0x7d0, 0x7d4, 0x7ce, 0x7eb, 0x20, 0x7d5, 0x7df,
0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7dd,
0x7ca, 0x7df, 0x7de, 0x7d1, 0x7df, 0x7ca, 0x7f2, 0x7d8, 0x20, 0x7d5, 0x7cc, 0x7f2,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7dd, 0x7ca, 0x7df,
0x7de, 0x7d1, 0x7df, 0x7ca, 0x7f2, 0x7d8, 0x20, 0x7d5, 0x7cc, 0x7f2, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2,
0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7dd, 0x7ca, 0x7df, 0x7de, 0x7d1, 0x7df, 0x7ca,
0x7f2, 0x7d8, 0x20, 0x7d5, 0x7cc, 0x7f2, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7dd, 0x7d6, 0x7cc, 0x7eb,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7dd, 0x7d6, 0x7cc,
0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce,
0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7dd, 0x7d6, 0x7cc, 0x7eb,
0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7ec,
0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca,
0x7f2, 0x7d5, 0x7ca, 0x7dd, 0x7ca, 0x7ec, 0x7d9, 0x7ca, 0x7f2, 0x7ec, 0x7db, 0x7cc,
0x7ec, 0x20, 0x7dc, 0x7ed, 0x7ce, 0x7e6, 0x7e3, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d0, 0x7db, 0x7d1, 0x7d5, 0x7d9, 0x7ca,
0x7df, 0x7cc, 0x7eb, 0x20, 0x7d8, 0x7ce, 0x7f0, 0x7de, 0x7df, 0x7cf, 0x20, 0x7e3,
0x7cc, 0x7eb, 0x20, 0x7dd, 0x7ca, 0x7ec, 0x7d9, 0x7ca, 0x7f2, 0x7ec, 0x7db, 0x7cc,
0x7eb, 0x20, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d9, 0x7d1, 0x7d5, 0x7cc, 0x7de, 0x20,
0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d9, 0x7cb, 0x7dc, 0x7ed, 0x7df, 0x7ca,
0x7d4, 0x7ca, 0x7dc, 0x7ed, 0x7d0, 0x7db, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x7d8, 0x7ca, 0x7eb, 0x7dc, 0x7ed, 0x7ca, 0x7f2, 0x7d3, 0x7cc, 0x7e6, 0x7cd,
0x7d9, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d6, 0x7ed,
0x7d0, 0x7d9, 0x7d1, 0x7d6, 0x7ed, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d6, 0x7ed, 0x7d0, 0x7d9, 0x7d1, 0x7d6, 0x7ed, 0x7cc,
0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce,
0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d6, 0x7ed, 0x7d0, 0x7d9,
0x7d1, 0x7d6, 0x7ed, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca,
0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d6, 0x7ed, 0x7cc, 0x7df, 0x7d1, 0x7d3,
0x7cd, 0x7d9, 0x7d5, 0x20, 0x7d5, 0x7cc, 0x7f2, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7dc, 0x7ed, 0x7d9, 0x7cb, 0x7e3, 0x7cd, 0x7d5, 0x20,
0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7df, 0x7cb, 0x7ec,
0x7d3, 0x7d0, 0x20, 0x7dc, 0x7ed, 0x7cc, 0x7d9, 0x7cc, 0x7f2, 0x7df, 0x7ca, 0x7f2,
0x7d8, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7df,
0x7cb, 0x7ec, 0x7d3, 0x7d0, 0x20, 0x7dc, 0x7ed, 0x7cc, 0x7d9, 0x7cc, 0x7f2, 0x7df,
0x7ca, 0x7f2, 0x7d8, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20,
0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d5, 0x7df,
0x7cb, 0x7ec, 0x7d3, 0x7d0, 0x20, 0x7dc, 0x7ed, 0x7cc, 0x7d9, 0x7cc, 0x7f2, 0x7df,
0x7ca, 0x7f2, 0x7d8, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20,
0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x20,
0x7dc, 0x7ed, 0x7cc, 0x7d9, 0x7cc, 0x7f2, 0x7df, 0x7ca, 0x7f2, 0x7d8, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3,
0x7cb, 0x20, 0x7dc, 0x7ed, 0x7cc, 0x7d9, 0x7cc, 0x7f2, 0x7df, 0x7ca, 0x7f2, 0x7d8,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1,
0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3,
0x7cb, 0x20, 0x7dc, 0x7ed, 0x7cc, 0x7d9, 0x7cc, 0x7f2, 0x7df, 0x7ca, 0x7f2, 0x7d8,
0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1,
0x7d8, 0x7ca, 0x7d6, 0x7cc, 0x7d8, 0x7cf, 0x7ee, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7e4, 0x7e5, 0x7ca, 0x7e6, 0x20, 0x2d, 0x20, 0x7ca,
0x7df, 0x7cb, 0x7e6, 0x7ce, 0x7d5, 0x7cc, 0x7e6, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7e4, 0x7e5, 0x7ca, 0x7e6, 0x20,
0x2d, 0x20, 0x7ca, 0x7df, 0x7cb, 0x7e6, 0x7ce, 0x7d5, 0x7cc, 0x7e6, 0x7ca, 0x7f2,
0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce,
0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7e4, 0x7e5, 0x7ca, 0x7e6,
0x20, 0x2d, 0x20, 0x7ca, 0x7df, 0x7cb, 0x7e6, 0x7ce, 0x7d5, 0x7cc, 0x7e6, 0x7ca,
0x7f2, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5,
0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7e4, 0x7d0, 0x7f2, 0x7dc, 0x7ed, 0x20, 0x7de, 0x7d0,
0x7f2, 0x7dc, 0x7ed, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca,
0x7e4, 0x7d0, 0x7f2, 0x7dc, 0x7ed, 0x20, 0x7de, 0x7d0, 0x7f2, 0x7dc, 0x7ed, 0x20,
0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca,
0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7e4, 0x7d0, 0x7f2, 0x7dc, 0x7ed, 0x20,
0x7de, 0x7d0, 0x7f2, 0x7dc, 0x7ed, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7e4, 0x7cf,
0x7dd, 0x7ed, 0x7d8, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca,
0x7e4, 0x7cf, 0x7dd, 0x7ed, 0x7d8, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca,
0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca,
0x7e4, 0x7cf, 0x7dd, 0x7ed, 0x7d8, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7e4, 0x7cc,
0x7f2, 0x7d8, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec,
0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7cd,
0x7f2, 0x7d8, 0x7ce, 0x7eb, 0x20, 0x7df, 0x7cc, 0x7f2, 0x7d3, 0x7ca, 0x7f2, 0x7d8,
0x7ce, 0x7ef, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d9, 0x7cb, 0x7f2,
0x7e4, 0x7cc, 0x7f2, 0x7d8, 0x7cf, 0x7eb, 0x20, 0x7db, 0x7cc, 0x7e3, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7cd, 0x7e1, 0x7ca, 0x7eb,
0x20, 0x7cd, 0x7f2, 0x7d8, 0x7cb, 0x7e3, 0x7cf, 0x7db, 0x7cc, 0x7eb, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3,
0x7d0, 0x20, 0x7cd, 0x7f2, 0x7d8, 0x7cb, 0x7e3, 0x7cf, 0x7db, 0x7cc, 0x7eb, 0x20,
0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7df, 0x7cb, 0x7ec,
0x7d3, 0x7cb, 0x20, 0x7cd, 0x7f2, 0x7d8, 0x7cb, 0x7e3, 0x7cf, 0x7db, 0x7cc, 0x7eb,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7cc, 0x7d9, 0x7ca,
0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7cc,
0x7d9, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec,
0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7cc,
0x7d9, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7cc, 0x7d9, 0x7d1, 0x7de, 0x7ce, 0x7d5,
0x7d1, 0x7db, 0x7de, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca,
0x7cc, 0x7d9, 0x7d1, 0x7de, 0x7ce, 0x7d5, 0x7d1, 0x7db, 0x7de, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8,
0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7cc, 0x7d9, 0x7d1, 0x7de, 0x7ce, 0x7d5, 0x7d1, 0x7db,
0x7de, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce,
0x7e1, 0x7d8, 0x7ca, 0x7cc, 0x7db, 0x7cc, 0x7ec, 0x7d9, 0x7ca, 0x7ec, 0x7e6, 0x7cc,
0x7df, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca,
0x7cc, 0x7db, 0x7cc, 0x7ec, 0x7d9, 0x7ca, 0x7ec, 0x7e6, 0x7cc, 0x7df, 0x7ca, 0x7eb,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1,
0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7cc, 0x7db, 0x7cc, 0x7ec, 0x7d9,
0x7ca, 0x7ec, 0x7e6, 0x7cc, 0x7df, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x20, 0x7db, 0x7ce,
0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d6, 0x7ed, 0x7ca, 0x7d4,
0x7d0, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca,
0x7d6, 0x7ed, 0x7ca, 0x7d4, 0x7d0, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2,
0x7d5, 0x7ca, 0x7d6, 0x7ed, 0x7ca, 0x7d4, 0x7d0, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7df,
0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x7d8, 0x7ca, 0x7de, 0x7d6, 0x7ed, 0x7ca, 0x7de, 0x7cc, 0x7db, 0x7d1, 0x7d5, 0x7ca,
0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5,
0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7d0, 0x20, 0x7de, 0x7d6, 0x7ed, 0x7ca, 0x7de, 0x7cc,
0x7db, 0x7d1, 0x7d5, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x20, 0x7de, 0x7d6,
0x7ed, 0x7ca, 0x7de, 0x7cc, 0x7db, 0x7d1, 0x7d5, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7de, 0x7cf, 0x7d9, 0x7cb, 0x7eb,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7de, 0x7cf, 0x7d9,
0x7cb, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db,
0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7de, 0x7cf, 0x7d9,
0x7cb, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7de, 0x7cf, 0x7db, 0x7d1, 0x7d9,
0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7de,
0x7d9, 0x7ca, 0x7db, 0x7d1, 0x7e3, 0x7cf, 0x7e6, 0x7cc, 0x7e6, 0x7ca, 0x7d9, 0x7d1,
0x7db, 0x7de, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7de,
0x7d9, 0x7ca, 0x7db, 0x7d1, 0x7e3, 0x7cf, 0x7e6, 0x7cc, 0x7e6, 0x7ca, 0x7d9, 0x7d1,
0x7db, 0x7de, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db,
0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7de, 0x7d9, 0x7ca,
0x7db, 0x7d1, 0x7e3, 0x7cf, 0x7e6, 0x7cc, 0x7e6, 0x7ca, 0x7d9, 0x7d1, 0x7db, 0x7de,
0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1,
0x7d8, 0x7ca, 0x7dc, 0x7ed, 0x7cc, 0x7d9, 0x7d1, 0x7dc, 0x7ed, 0x7cc, 0x7db, 0x7d1,
0x7d5, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8,
0x7ca, 0x7cc, 0x7db, 0x7d1, 0x7df, 0x7ca, 0x7f2, 0x7d8, 0x20, 0x7de, 0x7cd, 0x7ec,
0x7d9, 0x7cd, 0x7f2, 0x7ec, 0x7d8, 0x7cd, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x7d8, 0x7ca, 0x7df, 0x7d0, 0x7d9, 0x7d8, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7df, 0x7d0, 0x7d9, 0x7d8, 0x20, 0x7d5, 0x7ce, 0x7ec,
0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca,
0x7f2, 0x7d5, 0x7ca, 0x7df, 0x7d0, 0x7d9, 0x7d8, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7e1, 0x7dc, 0x7ed,
0x7ca, 0x7d8, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x7d8, 0x7ca, 0x7e1, 0x7dc, 0x7ed, 0x7ca, 0x7d8, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2,
0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7e1, 0x7dc, 0x7ed, 0x7ca, 0x7d8, 0x7ca, 0x7f2,
0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce,
0x7e1, 0x7d8, 0x7ca, 0x7e1, 0x7ca, 0x7df, 0x7cd, 0x7d8, 0x7ce, 0x7ef, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7e1, 0x7ca, 0x7df, 0x7d1, 0x7d8,
0x7cc, 0x7dd, 0x7ed, 0x20, 0x7d5, 0x7cc, 0x7f2, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7e1, 0x7ca, 0x7d9, 0x7de, 0x7cc, 0x7db, 0x7ca, 0x7db,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7e1, 0x7ca, 0x7d9,
0x7d1, 0x7db, 0x7ca, 0x7df, 0x20, 0x7d5, 0x7cc, 0x7f2, 0x20, 0x7d5, 0x7ce, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7e1, 0x7cf, 0x7d9, 0x7cc, 0x7db, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d9, 0x7cb, 0x7f2, 0x7e1, 0x7cf, 0x7d9, 0x7cc,
0x7db, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d9, 0x7cb, 0x7f2, 0x20,
0x7e2, 0x7ca, 0x7d3, 0x7d8, 0x7cd, 0x7e1, 0x7cf, 0x7d9, 0x7cc, 0x7db, 0x20, 0x7d5,
0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7ca, 0x7d9,
0x7cb, 0x7f2, 0x7e1, 0x7ca, 0x7e5, 0x7d1, 0x7db, 0x7d0, 0x7f2, 0x7eb, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d6, 0x7d0, 0x7eb, 0x20, 0x7df,
0x7cc, 0x7f2, 0x7d3, 0x7ca, 0x7f2, 0x20, 0x7e1, 0x7cd, 0x7f2, 0x7de, 0x7db, 0x7cc,
0x7de, 0x7cc, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca,
0x7d6, 0x7d0, 0x7eb, 0x20, 0x7df, 0x7cc, 0x7f2, 0x7d3, 0x7ca, 0x7f2, 0x20, 0x7e1,
0x7cd, 0x7f2, 0x7de, 0x7db, 0x7cc, 0x7de, 0x7cc, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7ec,
0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca,
0x7f2, 0x7d5, 0x7ca, 0x7d6, 0x7d0, 0x7eb, 0x20, 0x7df, 0x7cc, 0x7f2, 0x7d3, 0x7ca,
0x7f2, 0x20, 0x7e1, 0x7cd, 0x7f2, 0x7de, 0x7db, 0x7cc, 0x7de, 0x7cc, 0x7ec, 0x20,
0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8,
0x7ca, 0x7ce, 0x7df, 0x7ca, 0x7f2, 0x7d3, 0x7d5, 0x7d0, 0x7d9, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7ce, 0x7df, 0x7ca, 0x7f2, 0x7d3, 0x7d5,
0x7d0, 0x7d9, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db,
0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7ce, 0x7df, 0x7ca,
0x7f2, 0x7d3, 0x7d5, 0x7d0, 0x7d9, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7e1, 0x7cf,
0x7db, 0x7d1, 0x7de, 0x7ce, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x7d8, 0x7ca, 0x7e1, 0x7cf, 0x7db, 0x7d1, 0x7de, 0x7ce, 0x7eb, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8,
0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7e1, 0x7cf, 0x7db, 0x7d1, 0x7de, 0x7ce, 0x7eb, 0x20,
0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8,
0x7ca, 0x7e1, 0x7cc, 0x7e6, 0x7ca, 0x7e3, 0x7d1, 0x7e1, 0x7ca, 0x7d9, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7e3, 0x7ca, 0x7e5, 0x7d9, 0x7ce,
0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7e3, 0x7cb,
0x7d4, 0x7ca, 0x7df, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca,
0x7de, 0x7ca, 0x7df, 0x7cc, 0x7d8, 0x7cf, 0x7e3, 0x7cc, 0x7eb, 0x20, 0x7de, 0x7ce,
0x7d8, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca,
0x7de, 0x7ca, 0x7df, 0x7cc, 0x7d8, 0x7cf, 0x7e3, 0x7cc, 0x7eb, 0x20, 0x7de, 0x7ce,
0x7d8, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20,
0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7de, 0x7ca,
0x7df, 0x7cc, 0x7d8, 0x7cf, 0x7e3, 0x7cc, 0x7eb, 0x20, 0x7de, 0x7ce, 0x7d8, 0x7ca,
0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce,
0x7e1, 0x7d8, 0x7ca, 0x7e3, 0x7cc, 0x7e6, 0x7ce, 0x2d, 0x7d6, 0x7ed, 0x7cb, 0x7df,
0x7ca, 0x7f2, 0x7d8, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca,
0x7e3, 0x7cc, 0x7e6, 0x7ce, 0x2d, 0x7d6, 0x7ed, 0x7cb, 0x7df, 0x7ca, 0x7f2, 0x7d8,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1,
0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7e3, 0x7cc, 0x7e6, 0x7ce, 0x2d,
0x7d6, 0x7ed, 0x7cb, 0x7df, 0x7ca, 0x7f2, 0x7d8, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7dd, 0x7ce, 0x7f2,
0x7d8, 0x7d1, 0x7df, 0x7ca, 0x7f2, 0x7d8, 0x20, 0x7de, 0x7ce, 0x7d8, 0x7ca, 0x7eb,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7dd, 0x7ce, 0x7f2,
0x7d8, 0x7d1, 0x7df, 0x7ca, 0x7f2, 0x7d8, 0x20, 0x7de, 0x7ce, 0x7d8, 0x7ca, 0x7eb,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1,
0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7dd, 0x7ce, 0x7f2, 0x7d8, 0x7d1,
0x7df, 0x7ca, 0x7f2, 0x7d8, 0x20, 0x7de, 0x7ce, 0x7d8, 0x7ca, 0x7eb, 0x20, 0x7d5,
0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca,
0x7e3, 0x7cc, 0x7e6, 0x7cb, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x7d8, 0x7ca, 0x7e3, 0x7d0, 0x7d9, 0x20, 0x7dd, 0x7d0, 0x7df, 0x7de, 0x20, 0x7d5,
0x7cc, 0x7f2, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7e3,
0x7d0, 0x7d9, 0x20, 0x7dd, 0x7d0, 0x7df, 0x7de, 0x20, 0x7d5, 0x7cc, 0x7f2, 0x20,
0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca,
0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7e3, 0x7d0, 0x7d9, 0x20, 0x7dd, 0x7d0,
0x7df, 0x7de, 0x20, 0x7d5, 0x7cc, 0x7f2, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7dd, 0x7cd, 0x7d9, 0x7d1,
0x7e3, 0x7ca, 0x7f2, 0x7d8, 0x7cf, 0x7eb, 0x20, 0x7d8, 0x7cb, 0x7ec, 0x20, 0x7e3,
0x7cf, 0x7d9, 0x7cf, 0x7f2, 0x7e4, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7dd, 0x7cd, 0x7d9, 0x7d1, 0x7e3, 0x7ca, 0x7f2, 0x7d8,
0x7cf, 0x7eb, 0x20, 0x7d8, 0x7cb, 0x7ec, 0x20, 0x7e3, 0x7cf, 0x7d9, 0x7cf, 0x7f2,
0x7e4, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20,
0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7dd, 0x7cd,
0x7d9, 0x7d1, 0x7e3, 0x7ca, 0x7f2, 0x7d8, 0x7cf, 0x7eb, 0x20, 0x7d8, 0x7cb, 0x7ec,
0x20, 0x7e3, 0x7cf, 0x7d9, 0x7cf, 0x7f2, 0x7e4, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7df,
0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7e3,
0x7dd, 0x7ed, 0x7cf, 0x7db, 0x7d3, 0x7cc, 0x7d9, 0x7d1, 0x7db, 0x7de, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7e3, 0x7dd, 0x7ed, 0x7cf, 0x7db,
0x7d3, 0x7cc, 0x7d9, 0x7d1, 0x7db, 0x7de, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8,
0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5,
0x7ca, 0x7e3, 0x7dd, 0x7ed, 0x7cf, 0x7db, 0x7d3, 0x7cc, 0x7d9, 0x7d1, 0x7db, 0x7de,
0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1,
0x7d8, 0x7ca, 0x7cf, 0x7e1, 0x7d1, 0x7db, 0x7de, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7cf, 0x7e1, 0x7d1, 0x7db, 0x7de, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8,
0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7cf, 0x7e1, 0x7d1, 0x7db, 0x7de, 0x20, 0x7d5, 0x7df,
0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7d4,
0x7ca, 0x7de, 0x7cc, 0x7db, 0x7d1, 0x7d5, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d4, 0x7ca, 0x7de, 0x7cc, 0x7db, 0x7d1,
0x7d5, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec,
0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d4,
0x7ca, 0x7de, 0x7cc, 0x7db, 0x7d1, 0x7d5, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7df,
0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7d4,
0x7df, 0x7ca, 0x7d0, 0x7db, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8,
0x7ca, 0x7d4, 0x7ca, 0x7d3, 0x7ce, 0x7e5, 0x7ca, 0x7eb, 0x20, 0x7d6, 0x7cc, 0x7ec,
0x7e3, 0x7cd, 0x7ec, 0x20, 0x7de, 0x7ce, 0x7d8, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d4, 0x7d9, 0x7ca, 0x7dc, 0x7ed, 0x7ce,
0x7e5, 0x7cb, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca,
0x7d4, 0x7d9, 0x7ca, 0x7dc, 0x7ed, 0x7ce, 0x7e5, 0x7cb, 0x7eb, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8,
0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d4, 0x7d9, 0x7ca, 0x7dc, 0x7ed, 0x7ce, 0x7e5, 0x7cb,
0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce,
0x7e1, 0x7d8, 0x7ca, 0x7d4, 0x7cb, 0x7d9, 0x7ce, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d4, 0x7cb, 0x7d9, 0x7ce, 0x7eb, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2,
0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d4, 0x7cb, 0x7d9, 0x7ce, 0x7eb, 0x20, 0x7d5,
0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca,
0x7dd, 0x7df, 0x7cc, 0x7d4, 0x7cc, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7dd, 0x7df, 0x7cc, 0x7d4, 0x7cc, 0x7f2, 0x7eb, 0x20,
0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca,
0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7dd, 0x7df, 0x7cc, 0x7d4, 0x7cc, 0x7f2,
0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce,
0x7e1, 0x7d8, 0x7ca, 0x7dd, 0x7cf, 0x7e3, 0x7cc, 0x7de, 0x7db, 0x20, 0x7d5, 0x7cc,
0x7f2, 0x20, 0x7e0, 0x7ce, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x7d8, 0x7ca, 0x7db, 0x7cd, 0x7f2, 0x2d, 0x7d4, 0x7cc, 0x7e6, 0x7cd, 0x7d9, 0x2d,
0x7cb, 0x2d, 0x7e1, 0x7cc, 0x7de, 0x7cb, 0x7df, 0x7d0, 0x7f2, 0x7eb, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7db, 0x7cd, 0x7f2, 0x2d, 0x7d4,
0x7cc, 0x7e6, 0x7cd, 0x7d9, 0x2d, 0x7cb, 0x2d, 0x7e1, 0x7cc, 0x7de, 0x7cb, 0x7df,
0x7d0, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20,
0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7db, 0x7cd,
0x7f2, 0x2d, 0x7d4, 0x7cc, 0x7e6, 0x7cd, 0x7d9, 0x2d, 0x7cb, 0x2d, 0x7e1, 0x7cc,
0x7de, 0x7cb, 0x7df, 0x7d0, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7d4, 0x7cc, 0x7d5, 0x7d1,
0x7de, 0x7cd, 0x7d9, 0x7e3, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8,
0x7ca, 0x7d4, 0x7cf, 0x7f2, 0x7d4, 0x7cd, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d4, 0x7cc, 0x7e6, 0x7d0, 0x7f2, 0x7dc, 0x7ed, 0x7e6,
0x7ca, 0x7f2, 0x7dc, 0x7ed, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8,
0x7ca, 0x7d9, 0x7cb, 0x7e3, 0x7cc, 0x7e6, 0x7d0, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d9, 0x7cb, 0x7f2, 0x7d9, 0x7cf, 0x7d5, 0x7cb, 0x7d9,
0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7db,
0x7ca, 0x7de, 0x7d1, 0x7e4, 0x7ca, 0x7df, 0x7cc, 0x7e3, 0x20, 0x7d5, 0x7ce, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7db, 0x7ca, 0x7de, 0x7d1, 0x7e4, 0x7ca, 0x7df,
0x7cc, 0x7e3, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db,
0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7db, 0x7ca, 0x7de,
0x7d1, 0x7e4, 0x7ca, 0x7df, 0x7cc, 0x7e3, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7db, 0x7ca, 0x7e1, 0x7cf,
0x7e5, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca,
0x7db, 0x7ca, 0x7e1, 0x7cf, 0x7e5, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2,
0x7d5, 0x7ca, 0x7db, 0x7ca, 0x7e1, 0x7cf, 0x7e5, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7df,
0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7db,
0x7cb, 0x7e6, 0x7cc, 0x7db, 0x7cc, 0x7df, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x7d9, 0x7cb, 0x7f2, 0x7db, 0x7cd, 0x7f2, 0x7dc, 0x7ed, 0x7ca, 0x7d4, 0x7ce,
0x7d9, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce,
0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7db, 0x7df, 0x7cf, 0x7ec,
0x7e1, 0x7e3, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7cc, 0x7f2, 0x20, 0x7d5, 0x7ce, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7e5, 0x7d9, 0x7cf, 0x7ec, 0x7d8, 0x7ce, 0x7ee,
0x20, 0x7d6, 0x7ed, 0x7d0, 0x7d9, 0x7d1, 0x7d6, 0x7ed, 0x7cc, 0x7df, 0x7ca, 0x7eb,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7db, 0x7ce, 0x7d9,
0x7cc, 0x7e3, 0x7ca, 0x7e1, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8,
0x7ca, 0x7db, 0x7e6, 0x7cf, 0x7e5, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7ca, 0x7e4, 0x7d5, 0x7cc, 0x7eb, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7ca, 0x7e6, 0x7cc, 0x7d4,
0x7cb, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5,
0x7ca, 0x7e6, 0x7cc, 0x7d4, 0x7cb, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8,
0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5,
0x7ca, 0x7d5, 0x7ca, 0x7e6, 0x7cc, 0x7d4, 0x7cb, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8,
0x7ca, 0x7d5, 0x7ca, 0x7d6, 0x7de, 0x7cc, 0x7db, 0x7d1, 0x7d5, 0x7ca, 0x7f2, 0x7eb,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7cf, 0x7de,
0x7cb, 0x7df, 0x7cf, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8,
0x7ca, 0x7d5, 0x7cf, 0x7f2, 0x7dc, 0x7ed, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7cf, 0x7f2, 0x7dc, 0x7ed, 0x7ca, 0x7eb,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1,
0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d5, 0x7cf, 0x7f2, 0x7dc, 0x7ed,
0x7ca, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5,
0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7d7, 0x7ce, 0x7de, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7ce, 0x7d9, 0x7de, 0x7cc, 0x7e1, 0x7cb, 0x7e3,
0x7cc, 0x7db, 0x7d1, 0x7d5, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7ce, 0x7d9, 0x7de, 0x7cc, 0x7e1, 0x7cb, 0x7e3,
0x7cc, 0x7db, 0x7d1, 0x7d5, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2,
0x7d5, 0x7ca, 0x7d5, 0x7ce, 0x7d9, 0x7de, 0x7d1, 0x7e1, 0x7cb, 0x7e3, 0x7cc, 0x7db,
0x7d1, 0x7d5, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7d5, 0x7ce, 0x7dd, 0x7ed, 0x7ca,
0x7df, 0x7ce, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca,
0x7ce, 0x7f3, 0x7d9, 0x7d1, 0x7dc, 0x7ed, 0x7cb, 0x7e6, 0x7cc, 0x7eb, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7ce, 0x7f3, 0x7d9, 0x7d1, 0x7dc,
0x7ed, 0x7cb, 0x7e6, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca,
0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca,
0x7ce, 0x7f3, 0x7d9, 0x7d1, 0x7dc, 0x7ed, 0x7cb, 0x7e6, 0x7cc, 0x7eb, 0x20, 0x7d5,
0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca,
0x7cf, 0x7d6, 0x7ed, 0x7d1, 0x7d3, 0x7ca, 0x7de, 0x7cc, 0x7db, 0x7d1, 0x7d5, 0x7ca,
0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7cf,
0x7d6, 0x7ed, 0x7d1, 0x7d3, 0x7ca, 0x7de, 0x7cc, 0x7db, 0x7d1, 0x7d5, 0x7ca, 0x7f2,
0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce,
0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7cf, 0x7d6, 0x7ed, 0x7d1,
0x7d3, 0x7ca, 0x7de, 0x7cc, 0x7db, 0x7d1, 0x7d5, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5,
0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca,
0x7dd, 0x7ed, 0x7ca, 0x7e3, 0x7ce, 0x7e5, 0x7ca, 0x7d5, 0x7ce, 0x7eb, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7dd, 0x7ed, 0x7ca, 0x7e3, 0x7ce,
0x7e5, 0x7ca, 0x7d5, 0x7ce, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca,
0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca,
0x7dd, 0x7ed, 0x7ca, 0x7e3, 0x7ce, 0x7e5, 0x7ca, 0x7d5, 0x7ce, 0x7eb, 0x20, 0x7d5,
0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca,
0x7dd, 0x7ed, 0x7e3, 0x7cb, 0x7d6, 0x7ed, 0x7ce, 0x7e6, 0x7cb, 0x7df, 0x7ca, 0x7eb,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7dd, 0x7ed, 0x7df,
0x7ca, 0x7d8, 0x7d1, 0x7dd, 0x7ed, 0x7cf, 0x7db, 0x7d1, 0x7d5, 0x7cf, 0x7de, 0x20,
0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7dd, 0x7ed, 0x7df, 0x7ca,
0x7d8, 0x7d1, 0x7dd, 0x7ed, 0x7cf, 0x7db, 0x7d1, 0x7d5, 0x7cf, 0x7de, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2,
0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7dd, 0x7ed, 0x7df, 0x7ca, 0x7d8, 0x7d1, 0x7dd,
0x7ed, 0x7cf, 0x7db, 0x7d1, 0x7d5, 0x7cf, 0x7de, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7dd, 0x7ed, 0x7cf,
0x7df, 0x7d1, 0x7dc, 0x7ed, 0x7cf, 0x7dc, 0x7ed, 0x7d9, 0x7ca, 0x7d8, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7dd, 0x7ed, 0x7cf, 0x7df, 0x7d1,
0x7dc, 0x7ed, 0x7cf, 0x7dc, 0x7ed, 0x7d9, 0x7ca, 0x7d8, 0x20, 0x7d5, 0x7ce, 0x7ec,
0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca,
0x7f2, 0x7d5, 0x7ca, 0x7dd, 0x7ed, 0x7cf, 0x7df, 0x7d1, 0x7dc, 0x7ed, 0x7cf, 0x7dc,
0x7ed, 0x7d9, 0x7ca, 0x7d8, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7dd, 0x7ed, 0x7d0, 0x7db, 0x7d1, 0x7d5,
0x7d0, 0x7de, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7e5,
0x7ca, 0x7de, 0x20, 0x7d5, 0x7cc, 0x7f2, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x7d8, 0x7ca, 0x7e5, 0x7ca, 0x7df, 0x7cc, 0x7db, 0x20, 0x7e3, 0x7cc, 0x7eb,
0x20, 0x7dd, 0x7d5, 0x7ce, 0x7e3, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7e6, 0x7ca, 0x7de, 0x7ce, 0x7d5, 0x7d1, 0x7db, 0x7de,
0x7cc, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7e6,
0x7ca, 0x7de, 0x7ce, 0x7d5, 0x7d1, 0x7db, 0x7de, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8,
0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7e6, 0x7ca, 0x7de, 0x7ce, 0x7d5, 0x7d1, 0x7db, 0x7de,
0x7cc, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5,
0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7e6, 0x7cb, 0x7de, 0x7ca, 0x7d5, 0x7cb, 0x7d9, 0x7cc,
0x7f2, 0x7d3, 0x7ce, 0x7d9, 0x7dc, 0x7ed, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x7d8, 0x7ca, 0x7e6, 0x7cb, 0x7de, 0x7ca, 0x7d5, 0x7cb, 0x7d9, 0x7cc, 0x7f2,
0x7d3, 0x7ce, 0x7d9, 0x7dc, 0x7ed, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca,
0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca,
0x7e6, 0x7cb, 0x7de, 0x7ca, 0x7d5, 0x7cb, 0x7d9, 0x7cc, 0x7f2, 0x7d3, 0x7ce, 0x7d9,
0x7dc, 0x7ed, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5,
0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7e6, 0x7ce, 0x7de, 0x7d0, 0x7f2, 0x7eb, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2,
0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x6af, 0x627, 0x65b, 0x62a, 0x20, 0x645, 0x6cc,
0x646, 0x62c, 0x627, 0x65b, 0x6cc, 0x6cc, 0x6af, 0x627, 0x65b, 0x62a, 0x20, 0x645,
0x6cc, 0x646, 0x62c, 0x627, 0x65b, 0x6cc, 0x6cc, 0x20, 0x626, 0x633, 0x62a, 0x627,
0x65b, 0x646, 0x62f, 0x627, 0x65b, 0x631, 0x62f, 0x631, 0x648, 0x659, 0x634, 0x646,
0x627, 0x6cc, 0x6cc, 0x20, 0x646, 0x626, 0x647, 0x627, 0x62f, 0x627, 0x631, 0x20,
0x631, 0x648, 0x659, 0x632, 0x67, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x45, 0x75,
0x72, 0x6f, 0x68, 0x70, 0xe1, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x67, 0x61,
0x73, 0x6b, 0x61, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x68, 0x70, 0xe1, 0x20,
0x64, 0xe1, 0x62, 0xe1, 0x6c, 0x61, 0x161, 0xe1, 0x69, 0x67, 0x69, 0x67,
0x61, 0x73, 0x6b, 0x61, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x68, 0x70, 0xe1,
0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x69, 0xe1, 0x69, 0x67, 0x69, 0x6e,
0x75, 0x6f, 0x72, 0x74, 0x69, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x68, 0x70,
0xe1, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x6e, 0x75, 0x6f, 0x72, 0x74, 0x69,
0x2d, 0x45, 0x75, 0x72, 0x6f, 0x68, 0x70, 0xe1, 0x20, 0x64, 0xe1, 0x62,
0xe1, 0x6c, 0x61, 0x161, 0xe1, 0x69, 0x67, 0x69, 0x6e, 0x75, 0x6f, 0x72,
0x74, 0x69, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x68, 0x70, 0xe1, 0x20, 0x67,
0x65, 0x61, 0x73, 0x73, 0x69, 0xe1, 0x69, 0x67, 0x69, 0x6f, 0x61, 0x72,
0x6a, 0x65, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x68, 0x70, 0xe1, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x6f, 0x61, 0x72, 0x6a, 0x65, 0x2d, 0x45, 0x75, 0x72,
0x6f, 0x68, 0x70, 0xe1, 0x20, 0x64, 0xe1, 0x62, 0xe1, 0x6c, 0x61, 0x161,
0xe1, 0x69, 0x67, 0x69, 0x6f, 0x61, 0x72, 0x6a, 0x65, 0x2d, 0x45, 0x75,
0x72, 0x6f, 0x68, 0x70, 0xe1, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x69,
0xe1, 0x69, 0x67, 0x69, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63,
0x68, 0x20, 0x67, 0x61, 0x73, 0x6b, 0x6b, 0x61, 0x20, 0xe1, 0x69, 0x67,
0x69, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x2d, 0xe1, 0x69, 0x67, 0x69,
0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x2d, 0x64, 0xe1, 0x62, 0xe1, 0x6c,
0x61, 0x161, 0xe1, 0x69, 0x67, 0x69, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61,
0x2d, 0x67, 0x65, 0x61, 0x73, 0x73, 0x69, 0xe1, 0x69, 0x67, 0x69, 0x41,
0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x68, 0x61, 0x6e, 0x61, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x47, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x41, 0x66,
0x72, 0x69, 0x68, 0x6b, 0xe1, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x75,
0x6f, 0x72, 0x74, 0x61, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x68, 0x6b, 0xe1,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x4c, 0x75, 0x6c, 0x6c, 0x69, 0x2d, 0x41,
0x66, 0x72, 0x69, 0x68, 0x6b, 0xe1, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x4f, 0x61, 0x72, 0x6a, 0x65, 0x2d, 0x41, 0x66,
0x72, 0x69, 0x68, 0x6b, 0xe1, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x61,
0x72, 0x6a, 0x65, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x68, 0x6b, 0xe1, 0x20,
0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x61, 0x72,
0x6a, 0x65, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x68, 0x6b, 0xe1, 0x20, 0x67,
0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61,
0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x64, 0xe1, 0x62, 0xe1, 0x6c, 0x61, 0x161, 0x20,
0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x64, 0xe1, 0x62,
0xe1, 0x6c, 0x61, 0x161, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1,
0x69, 0x67, 0x69, 0xe1, 0x69, 0x67, 0x69, 0x20, 0x6e, 0x75, 0x6f, 0x72,
0x74, 0x61, 0x6e, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x20, 0x6e, 0x75, 0x6f, 0x72, 0x74, 0x61, 0x6e, 0x67, 0x65, 0x61, 0x73,
0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x20, 0x6e, 0x75, 0x6f, 0x72, 0x74,
0x61, 0x6e, 0x64, 0x75, 0x6f, 0x74, 0x74, 0x61, 0x72, 0xe1, 0x69, 0x67,
0x69, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0x64, 0x75, 0x6f, 0x74, 0x74, 0x61,
0x72, 0xe1, 0x69, 0x67, 0x69, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0x64,
0x75, 0x6f, 0x74, 0x74, 0x61, 0x72, 0xe1, 0x69, 0x67, 0x69, 0x4a, 0x61,
0x73, 0x6b, 0x65, 0x73, 0xe1, 0x62, 0x69, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x4a, 0x61, 0x73, 0x6b, 0x65, 0x73, 0xe1, 0x62, 0x69, 0x20, 0x64, 0xe1,
0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4a, 0x61, 0x73, 0x6b, 0x65,
0x73, 0xe1, 0x62, 0x69, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x41, 0x70, 0x69, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x41, 0x70, 0x69, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x41, 0x70, 0x69, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x72, 0xe1, 0x62, 0x69, 0x61, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x41, 0x72, 0xe1, 0x62, 0x69, 0x61, 0x20, 0x64,
0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x72, 0xe1, 0x62,
0x69, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67,
0x69, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73,
0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x61, 0x72, 0x6a, 0x65, 0x2d,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x4f, 0x61, 0x72, 0x6a, 0x65, 0x2d, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x4f, 0x61, 0x72, 0x6a, 0x65, 0x2d, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61,
0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x72,
0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x61, 0x74, 0x6c, 0xe1, 0x6e, 0x74, 0x61, 0x6c,
0x61, 0x161, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x61, 0x74, 0x6c, 0xe1, 0x6e,
0x74, 0x61, 0x6c, 0x61, 0x161, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x61, 0x74, 0x6c, 0xe1, 0x6e, 0x74, 0x61, 0x6c, 0x61,
0x161, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x47, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1,
0x6c, 0x69, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x47, 0x61, 0x73, 0x6b,
0x61, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20,
0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x47, 0x61, 0x73,
0x6b, 0x61, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61,
0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x47,
0x61, 0x73, 0x6b, 0x61, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c,
0x69, 0x61, 0x20, 0x6f, 0x61, 0x72, 0x6a, 0x6a, 0x61, 0x62, 0x65, 0x61,
0x6c, 0x69, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x47, 0x61, 0x73, 0x6b, 0x61,
0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x6f,
0x61, 0x72, 0x6a, 0x6a, 0x61, 0x62, 0x65, 0x61, 0x6c, 0x69, 0x20, 0x64,
0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x47, 0x61, 0x73, 0x6b,
0x61, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20,
0x6f, 0x61, 0x72, 0x6a, 0x6a, 0x61, 0x62, 0x65, 0x61, 0x6c, 0x69, 0x20,
0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x75,
0x6f, 0x72, 0x74, 0x61, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c,
0x69, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x75, 0x6f, 0x72, 0x74,
0x61, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20,
0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x75, 0x6f,
0x72, 0x74, 0x61, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69,
0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x4f, 0x61, 0x72, 0x6a, 0x65, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1,
0x6c, 0x69, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x61, 0x72, 0x6a,
0x65, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20,
0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x61, 0x72,
0x6a, 0x65, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61,
0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x41,
0x73, 0x65, 0x72, 0x62, 0x61, 0x69, 0x17e, 0x61, 0x6e, 0x61, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x69, 0x17e, 0x61,
0x6e, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x69, 0x17e, 0x61, 0x6e, 0x61, 0x20,
0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x7a,
0x6f, 0x72, 0x61, 0x69, 0x64, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x7a,
0x6f, 0x72, 0x61, 0x69, 0x64, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x41, 0x7a, 0x6f, 0x72, 0x61, 0x69, 0x64, 0x20, 0x67,
0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x61, 0x20, 0xe1, 0x69, 0x67,
0x69, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x61,
0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x61, 0x20, 0x67, 0x65,
0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x42, 0x68, 0x75, 0x74,
0x61, 0x6e, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x42, 0x6f, 0x6c, 0x69,
0x76, 0x69, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x42, 0x72, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20,
0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x42, 0x72,
0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61,
0x6c, 0x61, 0x6d, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4b, 0x61, 0x70,
0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4b,
0x61, 0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x64, 0xe1, 0x6c,
0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4b, 0x61, 0x70, 0x20, 0x56, 0x65,
0x72, 0x64, 0x65, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x10c, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x20, 0x64, 0xe1,
0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x43, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x61, 0x20, 0x67,
0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x43, 0x68, 0x69,
0x6c, 0x65, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x43, 0x68, 0x69, 0x6c, 0x65,
0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x43, 0x68,
0x69, 0x6c, 0x65, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x4b, 0x69, 0x69, 0x6e, 0x6e, 0xe1, 0x20, 0xe1, 0x69, 0x67,
0x69, 0x4b, 0x69, 0x69, 0x6e, 0x6e, 0xe1, 0x20, 0x64, 0xe1, 0x6c, 0x76,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x4b, 0x69, 0x69, 0x6e, 0x6e, 0xe1, 0x20,
0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4a, 0x75,
0x6f, 0x76, 0x6c, 0x61, 0x73, 0x75, 0x6c, 0x6c, 0x6f, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x73, 0x75, 0x6c, 0x6c, 0x6f,
0x69, 0x64, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x43, 0x6f, 0x6c, 0x6f, 0x6d,
0x62, 0x69, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x43, 0x6f, 0x6c, 0x6f,
0x6d, 0x62, 0x69, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x67,
0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x43, 0x6f, 0x6f,
0x6b, 0x73, 0x75, 0x6c, 0x6c, 0x6f, 0x69, 0x64, 0x20, 0xe1, 0x69, 0x67,
0x69, 0x43, 0x6f, 0x6f, 0x6b, 0x73, 0x75, 0x6c, 0x6c, 0x6f, 0x69, 0x64,
0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x43, 0x6f,
0x6f, 0x6b, 0x73, 0x75, 0x6c, 0x6c, 0x6f, 0x69, 0x64, 0x20, 0x67, 0x65,
0x61, 0x73, 0x69, 0x20, 0x62, 0x65, 0x61, 0x6c, 0x6c, 0x65, 0x6d, 0x75,
0x74, 0x74, 0x75, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x43, 0x75, 0x62, 0x61,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x43, 0x75, 0x62, 0x61, 0x20, 0x64, 0xe1,
0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x43, 0x75, 0x62, 0x61, 0x20,
0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x44, 0x61,
0x76, 0x69, 0x73, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x44, 0x75, 0x6d,
0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c,
0x65, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x75, 0x6f, 0x72, 0x74, 0x61,
0x2d, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x42, 0x65, 0x61, 0x73, 0x73, 0xe1, 0x161, 0x73, 0x75, 0x6c, 0x6c, 0x6f,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x42, 0x65, 0x61, 0x73, 0x73, 0xe1, 0x161,
0x73, 0x75, 0x6c, 0x6c, 0x6f, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x42, 0x65, 0x61, 0x73, 0x73, 0xe1, 0x161, 0x73, 0x75,
0x6c, 0x6c, 0x6f, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x61, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x47, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x45, 0x75, 0x72,
0x6f, 0x68, 0x70, 0xe1, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x47, 0x61, 0x73,
0x6b, 0x61, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x68, 0x70, 0xe1, 0x20, 0x64,
0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x47, 0x61, 0x73, 0x6b,
0x61, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x68, 0x70, 0xe1, 0x20, 0x67, 0x65,
0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x75, 0x6f, 0x72,
0x74, 0x61, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x68, 0x70, 0x61, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x4e, 0x75, 0x6f, 0x72, 0x74, 0x61, 0x2d, 0x45, 0x75,
0x72, 0x6f, 0x68, 0x70, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x4e, 0x75, 0x6f, 0x72, 0x74, 0x61, 0x2d, 0x45, 0x75,
0x72, 0x6f, 0x68, 0x70, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x47, 0xe1, 0x69, 0x64, 0x64, 0x75, 0x73, 0x2d,
0x4e, 0x75, 0x6f, 0x72, 0x74, 0x74, 0x69, 0x20, 0x65, 0x75, 0x72, 0x6f,
0x68, 0x70, 0x61, 0x6c, 0x61, 0x161, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4f,
0x61, 0x72, 0x6a, 0x65, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x68, 0x70, 0xe1,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x61, 0x72, 0x6a, 0x65, 0x2d, 0x45,
0x75, 0x72, 0x6f, 0x68, 0x70, 0xe1, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x4f, 0x61, 0x72, 0x6a, 0x65, 0x2d, 0x45, 0x75,
0x72, 0x6f, 0x68, 0x70, 0xe1, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x75, 0x6c, 0x6c, 0x75, 0x69, 0x64, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x75, 0x6c, 0x6c,
0x75, 0x69, 0x64, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67,
0x69, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x75, 0x6c,
0x6c, 0x75, 0x69, 0x64, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x46, 0x69, 0x6a, 0x69, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x46, 0x69, 0x6a, 0x69, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x46, 0x72, 0x61, 0x6e, 0x6b, 0x72, 0x69,
0x69, 0x6b, 0x6b, 0x61, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x46, 0x72, 0x61, 0x6e, 0x6b, 0x72, 0x69, 0x69,
0x6b, 0x6b, 0x61, 0x20, 0x6c, 0x75, 0x6c, 0x6c, 0x69, 0x20, 0x26, 0x20,
0x61, 0x6e, 0x74, 0xe1, 0x72, 0x6b, 0x74, 0x69, 0x73, 0x61, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73,
0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65,
0x72, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x47, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x47, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73,
0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72,
0x74, 0x73, 0x75, 0x6c, 0x6c, 0x6f, 0x69, 0x64, 0x20, 0xe1, 0x69, 0x67,
0x69, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x61, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x4e, 0x75, 0x6f, 0x72, 0x74, 0x61, 0x2d, 0x52,
0x75, 0x6f, 0x6e, 0xe1, 0x65, 0x61, 0x74, 0x6e, 0x61, 0x6d, 0x61, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x4e, 0x75, 0x6f, 0x72, 0x74, 0x61, 0x2d, 0x52,
0x75, 0x6f, 0x6e, 0xe1, 0x65, 0x61, 0x74, 0x6e, 0x61, 0x6d, 0x61, 0x20,
0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x75, 0x6f,
0x72, 0x74, 0x61, 0x2d, 0x52, 0x75, 0x6f, 0x6e, 0xe1, 0x65, 0x61, 0x74,
0x6e, 0x61, 0x6d, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x4f, 0x61, 0x72, 0x6a, 0x65, 0x2d, 0x52, 0x75, 0x6f,
0x6e, 0xe1, 0x65, 0x61, 0x74, 0x6e, 0x61, 0x6d, 0x61, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x4f, 0x61, 0x72, 0x6a, 0x65, 0x2d, 0x52, 0x75, 0x6f, 0x6e,
0xe1, 0x65, 0x61, 0x74, 0x6e, 0x61, 0x6d, 0x61, 0x20, 0x64, 0xe1, 0x6c,
0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x61, 0x72, 0x6a, 0x65, 0x2d,
0x52, 0x75, 0x6f, 0x6e, 0xe1, 0x65, 0x61, 0x74, 0x6e, 0x61, 0x6d, 0x61,
0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x47,
0x6f, 0x6c, 0x66, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x61, 0x6c, 0x65,
0x75, 0x68, 0x74, 0x61, 0x6c, 0x61, 0x161, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x68,
0x74, 0x61, 0x6c, 0x61, 0x161, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x61, 0x6c,
0x65, 0x75, 0x68, 0x74, 0x61, 0x6c, 0x61, 0x161, 0x20, 0x67, 0x65, 0x61,
0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x48, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x48, 0x6f,
0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x64, 0xe1, 0x6c,
0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b,
0x6f, 0x6e, 0x67, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x48, 0x6f, 0x76, 0x64, 0x61, 0x20, 0xe1, 0x69, 0x67,
0x69, 0x48, 0x6f, 0x76, 0x64, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x48, 0x6f, 0x76, 0x64, 0x61, 0x20, 0x67, 0x65,
0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x49, 0x6e, 0x64, 0x69,
0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x49,
0x6e, 0x64, 0x69, 0x61, 0xe1, 0x62, 0x69, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x49, 0x6e, 0x64, 0x6f, 0x6b, 0x69, 0x69, 0x6e, 0x6e, 0xe1, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x47, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x49, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4e,
0x75, 0x6f, 0x72, 0x74, 0x61, 0x2d, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x73, 0x69, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x61, 0x72, 0x6a,
0x65, 0x2d, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x49, 0x72, 0x61, 0x6e, 0x61, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x49, 0x72, 0x61, 0x6e, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x49, 0x72, 0x61, 0x6e, 0x61, 0x20, 0x67,
0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x49, 0x72, 0x6b,
0x75, 0x63, 0x6b, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x49, 0x72, 0x6b,
0x75, 0x63, 0x6b, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x49, 0x72, 0x6b, 0x75, 0x63, 0x6b, 0x61, 0x20, 0x67, 0x65,
0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67,
0x69, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x61, 0x20, 0x67, 0x65, 0x61,
0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4a, 0x61, 0x70, 0xe1, 0x6e,
0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4a, 0x61, 0x70, 0xe1, 0x6e, 0x61,
0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4a, 0x61,
0x70, 0xe1, 0x6e, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x4e, 0x75, 0x6f, 0x72, 0x74, 0x61, 0x2d, 0x4b, 0x61,
0x73, 0x61, 0x6b, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x20, 0xe1, 0x69, 0x67,
0x69, 0x4f, 0x61, 0x72, 0x6a, 0x65, 0x2d, 0x4b, 0x61, 0x73, 0x61, 0x6b,
0x73, 0x74, 0x61, 0x6e, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4b, 0x6f,
0x72, 0x65, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4b, 0x6f, 0x72, 0x65,
0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4b,
0x6f, 0x72, 0x65, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x61, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72,
0x73, 0x6b, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x61, 0x20, 0x64, 0xe1, 0x6c,
0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x6a, 0x61, 0x72, 0x73, 0x6b, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x4b, 0x69, 0x72, 0x67, 0x69, 0x73, 0x69,
0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4c, 0x69, 0x6e, 0x65, 0x73, 0x75,
0x6c, 0x6c, 0x6f, 0x69, 0x64, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x64, 0xe1,
0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x61, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x61,
0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x4d, 0x61, 0x6c, 0x65, 0x73, 0x69, 0x61,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x4d, 0x61, 0x6c, 0x65, 0x64, 0x69, 0x69,
0x76, 0x76, 0x61, 0x69, 0x64, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4d, 0x61,
0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x69, 0x69, 0x64, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x73,
0x75, 0x6c, 0x6c, 0x6f, 0x69, 0x64, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x61, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x61,
0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x61, 0x20, 0x67, 0x65, 0x61,
0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4d, 0x61, 0x77, 0x73, 0x6f,
0x6e, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4d, 0x65, 0x6b, 0x73, 0x69,
0x6b, 0x6f, 0x20, 0x4a, 0xe1, 0x73, 0x6b, 0x65, 0x73, 0xe1, 0x62, 0x69,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f,
0x20, 0x4a, 0xe1, 0x73, 0x6b, 0x65, 0x73, 0xe1, 0x62, 0x69, 0x20, 0x64,
0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4d, 0x65, 0x6b, 0x73,
0x69, 0x6b, 0x6f, 0x20, 0x4a, 0xe1, 0x73, 0x6b, 0x65, 0x73, 0xe1, 0x62,
0x69, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x61, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74,
0x6f, 0x72, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67,
0x69, 0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x61,
0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4d,
0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4d, 0x6f,
0x73, 0x6b, 0x76, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20, 0x67, 0x65, 0x61,
0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4d, 0x79, 0x61, 0x6e, 0x6d,
0x61, 0x72, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x61, 0x75, 0x72,
0x75, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x61,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x111, 0x111, 0x61, 0x2d, 0x4b, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x4f, 0x111, 0x111, 0x61, 0x2d, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x69, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x4f, 0x111, 0x111, 0x61, 0x2d, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x69, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67,
0x69, 0x4f, 0x111, 0x111, 0x61, 0x2d, 0x53, 0x65, 0x6c, 0xe1, 0x6e, 0x64,
0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x111, 0x111, 0x61, 0x2d, 0x53,
0x65, 0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x4f, 0x111, 0x111, 0x61, 0x2d, 0x53, 0x65, 0x6c,
0xe1, 0x6e, 0x64, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c,
0x61, 0x6e, 0x64, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x65, 0x77,
0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x64,
0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x65, 0x77, 0x66,
0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x67, 0x65,
0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x69, 0x75, 0x65,
0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x73, 0x75, 0x6c, 0x6c, 0x6f, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20,
0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x61, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67,
0x69, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x4f, 0x6d, 0x73, 0x6b, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x6d,
0x73, 0x6b, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67,
0x69, 0x4f, 0x6d, 0x73, 0x6b, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x20,
0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x50, 0x61,
0x6c, 0x61, 0x75, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x50, 0x61, 0x70,
0x75, 0x61, 0x20, 0x4f, 0x111, 0x111, 0x61, 0x2d, 0x47, 0x75, 0x69, 0x6e,
0x65, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x79, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x50, 0x61, 0x72,
0x61, 0x67, 0x75, 0x61, 0x79, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79,
0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x50, 0x65, 0x72, 0x75, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x50, 0x65, 0x72,
0x75, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x50,
0x65, 0x72, 0x75, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x69, 0x6e, 0x6e,
0x61, 0x69, 0x64, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x70, 0x69, 0x69, 0x6e, 0x6e, 0x61, 0x69, 0x64, 0x20, 0x64, 0xe1,
0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x70, 0x69, 0x69, 0x6e, 0x6e, 0x61, 0x69, 0x64, 0x20, 0x67, 0x65, 0x61,
0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x50, 0x68, 0x6f, 0x65, 0x6e,
0x69, 0x78, 0x73, 0x75, 0x6c, 0x6c, 0x6f, 0x69, 0x64, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x20,
0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x53, 0x74, 0x2e,
0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e,
0x73, 0x75, 0x6c, 0x6c, 0x6f, 0x69, 0x64, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x50,
0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x61, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x61, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x61, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x61,
0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x53, 0x61,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x64, 0xe1, 0x6c,
0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20,
0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x53, 0x65,
0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x61, 0x69, 0x64, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65, 0x20,
0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x53, 0x61, 0x6c,
0x6f, 0x6d, 0x6f, 0x6e, 0x73, 0x75, 0x6c, 0x6c, 0x6f, 0x69, 0x64, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x4c, 0x75, 0x6c, 0x6c, 0x69, 0x2d, 0x47, 0x65,
0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x53, 0x75,
0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x53,
0x79, 0x6f, 0x77, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x54, 0x61, 0x68,
0x69, 0x74, 0x69, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67,
0x69, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x61, 0x20, 0x67, 0x65, 0x61,
0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x54, 0x61, 0x17e, 0x69, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x54,
0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67,
0x69, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x61, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x54, 0x75,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x20,
0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x54, 0x75, 0x72,
0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x20, 0x67,
0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x54, 0x75, 0x76,
0x61, 0x6c, 0x75, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x55, 0x72, 0x75, 0x67,
0x75, 0x61, 0x79, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x79, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x61, 0x20,
0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x55, 0x73,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x55,
0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x20, 0x67,
0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x67, 0x65,
0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x56, 0x65, 0x6e, 0x65,
0x7a, 0x75, 0x65, 0x6c, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x61, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67,
0x69, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x61, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x61, 0x20, 0x67, 0x65,
0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x56, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x57, 0x61, 0x6b, 0x65,
0x73, 0x75, 0x6c, 0x6c, 0x6f, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x57, 0x61,
0x6c, 0x6c, 0x69, 0x73, 0x2d, 0x20, 0x6a, 0x61, 0x20, 0x46, 0x75, 0x74,
0x75, 0x6e, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4a, 0x61, 0x6b, 0x75,
0x63, 0x6b, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4a, 0x61, 0x6b, 0x75,
0x63, 0x6b, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67,
0x69, 0x4a, 0x61, 0x6b, 0x75, 0x63, 0x6b, 0x61, 0x20, 0x67, 0x65, 0x61,
0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4a, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x61, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x4e, 0x61, 0x6b, 0x6f, 0x20, 0x50, 0x61, 0x6c, 0x6f,
0x67, 0x61, 0x72, 0x65, 0x20, 0x79, 0x61, 0x20, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x77, 0x69, 0x63, 0x68, 0x41, 0x63, 0x72, 0x65, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x63, 0x72, 0x65, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x61, 0x66, 0x67,
0x68, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x73, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0xf8, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x73, 0xf8, 0x72,
0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x76, 0x65, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x61, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x61, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x61, 0x6c, 0x61, 0x73, 0x6b, 0x69, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x61, 0x6c, 0x61, 0x73, 0x6b, 0x69, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x61, 0x6c,
0x61, 0x73, 0x6b, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x2c, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x64, 0x41,
0x6c, 0x6d, 0x61, 0x74, 0x79, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61,
0x73, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x73,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x74, 0x69, 0x64,
0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64, 0x65,
0x74, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x64, 0x2d, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x64, 0x65, 0x74, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x2d, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x74, 0x20, 0x73, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x2d, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e,
0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6e, 0x20, 0x6e, 0x6f,
0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b,
0x65, 0x20, 0xf8, 0x73, 0x74, 0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x64, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0xf8, 0x73, 0x74,
0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6e, 0x20,
0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0xf8, 0x73, 0x74, 0x6b, 0x79, 0x73, 0x74, 0x65,
0x6e, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x52, 0x6f, 0x63, 0x6b, 0x79, 0x20, 0x4d, 0x6f, 0x75, 0x6e,
0x74, 0x61, 0x69, 0x6e, 0x73, 0x20, 0x28, 0x55, 0x53, 0x41, 0x29, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x52, 0x6f, 0x63, 0x6b, 0x79, 0x20, 0x4d, 0x6f, 0x75, 0x6e, 0x74,
0x61, 0x69, 0x6e, 0x73, 0x20, 0x28, 0x55, 0x53, 0x41, 0x29, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x52, 0x6f, 0x63, 0x6b, 0x79, 0x20, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61,
0x69, 0x6e, 0x73, 0x20, 0x28, 0x55, 0x53, 0x41, 0x29, 0x74, 0x69, 0x64,
0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64, 0x65,
0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x53, 0x74, 0x69, 0x6c, 0x6c, 0x65,
0x68, 0x61, 0x76, 0x73, 0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x64, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x53, 0x74, 0x69, 0x6c,
0x6c, 0x65, 0x68, 0x61, 0x76, 0x73, 0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x64, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x53, 0x74,
0x69, 0x6c, 0x6c, 0x65, 0x68, 0x61, 0x76, 0x73, 0x6b, 0x79, 0x73, 0x74,
0x65, 0x6e, 0x52, 0x75, 0x73, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x28, 0x41,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x29, 0x20, 0x74, 0x69, 0x64, 0x52, 0x75,
0x73, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x28, 0x41, 0x6e, 0x61, 0x64, 0x79,
0x72, 0x29, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x52, 0x75, 0x73, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x28, 0x41, 0x6e, 0x61,
0x64, 0x79, 0x72, 0x29, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x41, 0x70, 0x69, 0x61, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x41, 0x70, 0x69,
0x61, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x41, 0x70, 0x69, 0x61, 0x41, 0x71, 0x74, 0x61, 0x75,
0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69,
0x64, 0x41, 0x71, 0x74, 0x61, 0x75, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x64,
0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x61, 0x72, 0x61, 0x62, 0x69, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x61, 0x72, 0x61, 0x62, 0x69, 0x73, 0x6b, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x64, 0x61,
0x72, 0x61, 0x62, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x61, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0x65,
0x73, 0x74, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x76, 0x65,
0x73, 0x74, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x61, 0x72,
0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x61, 0x72, 0x6d,
0x65, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x61, 0x72, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73,
0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6e, 0x20,
0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x65, 0x72,
0x68, 0x61, 0x76, 0x73, 0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x64, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x61, 0x74, 0x6c, 0x61,
0x6e, 0x74, 0x65, 0x72, 0x68, 0x61, 0x76, 0x73, 0x6b, 0x79, 0x73, 0x74,
0x65, 0x6e, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x64,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20,
0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x65, 0x72, 0x68, 0x61, 0x76, 0x73,
0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x2d, 0x73,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x2d,
0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x2d, 0x73, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0xf8, 0x73, 0x74,
0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0xf8, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0xf8,
0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73,
0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69,
0x64, 0x76, 0x65, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x61, 0x73, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x73, 0x6a, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x61, 0x73, 0x65, 0x72, 0x62, 0x61,
0x6a, 0x64, 0x73, 0x6a, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x61, 0x73, 0x65, 0x72, 0x62, 0x61,
0x6a, 0x64, 0x73, 0x6a, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x61, 0x73, 0x6f, 0x72, 0x69, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x61, 0x73, 0x6f, 0x72, 0x69, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x61, 0x73,
0x6f, 0x72, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73,
0x68, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x62, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x62, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x73, 0x68, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x62, 0x68, 0x75, 0x74, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x62, 0x6f, 0x6c, 0x69, 0x76, 0x69,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73,
0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x61, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x61, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69,
0x61, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72,
0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x6b, 0x61, 0x70, 0x70, 0x76,
0x65, 0x72, 0x64, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x6b, 0x61,
0x70, 0x70, 0x76, 0x65, 0x72, 0x64, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x6b, 0x61, 0x70, 0x70, 0x76,
0x65, 0x72, 0x64, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x43, 0x61, 0x73, 0x65, 0x79, 0x2d, 0x74, 0x69,
0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x74, 0x69,
0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x43,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x63, 0x68, 0x69, 0x6c, 0x65, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x63, 0x68, 0x69, 0x6c, 0x65, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x63, 0x68, 0x69, 0x6c, 0x65, 0x6e,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x6b, 0x69, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x6b, 0x69, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x6b, 0x69, 0x6e, 0x65, 0x73, 0x69,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0xf8, 0x79,
0x61, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0xf8, 0x79, 0x65, 0x6e, 0x65,
0x63, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x63, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x63, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64,
0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x43, 0x6f,
0x6f, 0x6b, 0xf8, 0x79, 0x65, 0x6e, 0x65, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x43, 0x6f, 0x6f,
0x6b, 0xf8, 0x79, 0x65, 0x6e, 0x65, 0x68, 0x61, 0x6c, 0x76, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x43, 0x6f, 0x6f, 0x6b, 0xf8, 0x79, 0x65, 0x6e, 0x65, 0x63, 0x75,
0x62, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x63, 0x75, 0x62,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x63, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73,
0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x44, 0x61, 0x76, 0x69,
0x73, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x20, 0x64, 0x2019, 0x55,
0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0xf8, 0x73, 0x74, 0x74, 0x69, 0x6d,
0x6f, 0x72, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x74,
0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x50, 0xe5, 0x73, 0x6b, 0x65, 0xf8, 0x79, 0x61, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x50, 0xe5,
0x73, 0x6b, 0x65, 0xf8, 0x79, 0x61, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x50, 0xe5, 0x73, 0x6b,
0x65, 0xf8, 0x79, 0x61, 0x65, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x69,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x73, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0xf8, 0x73, 0x74, 0x65,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0xf8, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0xf8, 0x73,
0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x66, 0x6a, 0x65, 0x72,
0x6e, 0x2d, 0xf8, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x65, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76,
0x65, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x76, 0x65,
0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64,
0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x46, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0xf8, 0x79, 0x65, 0x6e, 0x65,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0xf8,
0x79, 0x65, 0x6e, 0x65, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0xf8, 0x79, 0x65, 0x6e, 0x65, 0x66, 0x69, 0x6a, 0x69,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x66, 0x69, 0x6a, 0x69,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x66, 0x69, 0x6a, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73,
0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x46, 0x72, 0x61,
0x6e, 0x73, 0x6b, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x74, 0x69,
0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x44,
0x65, 0x20, 0x66, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0xf8,
0x72, 0x74, 0x65, 0x72, 0x72, 0x69, 0x74, 0x6f, 0x72, 0x69, 0x65, 0x72,
0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x47, 0x61, 0x6c, 0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73, 0xf8, 0x79,
0x65, 0x6e, 0x65, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x67,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x67,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x74,
0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0xf8, 0x79, 0x65, 0x6e, 0x65,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x20, 0x6d, 0x69,
0x64, 0x64, 0x65, 0x6c, 0x74, 0x69, 0x64, 0xf8, 0x73, 0x74, 0x67, 0x72,
0xf8, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0xf8, 0x73, 0x74, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0xf8,
0x73, 0x74, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x76, 0x65,
0x73, 0x74, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x67, 0x72, 0xf8, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x67, 0x72, 0xf8, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x47, 0x75, 0x61, 0x6d, 0x2d, 0x74, 0x69, 0x64,
0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x50, 0x65, 0x72, 0x73, 0x69, 0x61, 0x62, 0x75, 0x6b, 0x74, 0x61,
0x67, 0x75, 0x79, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x74,
0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x20, 0x6f, 0x67, 0x20, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x65, 0x6e, 0x65, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x48, 0x61, 0x77, 0x61,
0x69, 0x69, 0x20, 0x6f, 0x67, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x65,
0x6e, 0x65, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x20, 0x6f,
0x67, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x65, 0x6e, 0x65, 0x74, 0x69,
0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x48,
0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x48, 0x6f, 0x6e,
0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x6b,
0x6f, 0x6e, 0x67, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x4b, 0x68, 0x6f, 0x76, 0x64, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4b,
0x68, 0x6f, 0x76, 0x64, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4b, 0x68, 0x6f, 0x76, 0x64, 0x69,
0x6e, 0x64, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64,
0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x49, 0x6e,
0x64, 0x69, 0x61, 0x68, 0x61, 0x76, 0x65, 0x74, 0x69, 0x6e, 0x64, 0x6f,
0x6b, 0x69, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6e, 0x64, 0x6f, 0x6e,
0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0xf8, 0x73, 0x74,
0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x69, 0x72, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x69, 0x72, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x69, 0x72,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x69, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x69, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x69, 0x73,
0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x6a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x6a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x6a, 0x61, 0x70, 0x61,
0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x52, 0x75, 0x73, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x28, 0x50, 0x65,
0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d,
0x4b, 0x61, 0x6d, 0x74, 0x73, 0x6a, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x6a,
0x29, 0x20, 0x74, 0x69, 0x64, 0x52, 0x75, 0x73, 0x73, 0x69, 0x73, 0x6b,
0x20, 0x28, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x74, 0x73, 0x6a, 0x61, 0x74,
0x73, 0x6b, 0x69, 0x6a, 0x29, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x52, 0x75, 0x73, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x28,
0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73,
0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x74, 0x73, 0x6a, 0x61, 0x74, 0x73, 0x6b,
0x69, 0x6a, 0x29, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0xf8, 0x73, 0x74, 0x6b, 0x61, 0x73, 0x61, 0x6b, 0x68, 0x73, 0x74,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74,
0x6b, 0x61, 0x73, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x6b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x6b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x6b, 0x6f,
0x72, 0x65, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x74,
0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73,
0x6b, 0x6b, 0x69, 0x72, 0x67, 0x69, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x2d, 0x74, 0x69, 0x64, 0x74,
0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x4c, 0x69, 0x6e, 0x6a, 0x65, 0xf8, 0x79, 0x65, 0x6e, 0x65, 0x74, 0x69,
0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0xf8, 0x79, 0x61,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d,
0xf8, 0x79, 0x61, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x2d, 0xf8, 0x79, 0x61, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x2d,
0x74, 0x69, 0x64, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x2c, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x63,
0x61, 0x75, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x6d, 0x61, 0x6c, 0x61, 0x79, 0x73, 0x69, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x6d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x69, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e,
0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65,
0x73, 0x61, 0x73, 0xf8, 0x79, 0x65, 0x6e, 0x65, 0x6d, 0x61, 0x72, 0x73,
0x68, 0x61, 0x6c, 0x6c, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x6d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x6d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x73, 0x6b, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x6d, 0x61, 0x75,
0x72, 0x69, 0x74, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x74,
0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x64, 0x65, 0x6e, 0x20, 0x6d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x53, 0x74, 0x69, 0x6c, 0x6c, 0x65, 0x68, 0x61,
0x76, 0x73, 0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64, 0x65,
0x6e, 0x20, 0x6d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b,
0x65, 0x20, 0x53, 0x74, 0x69, 0x6c, 0x6c, 0x65, 0x68, 0x61, 0x76, 0x73,
0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6e, 0x20,
0x6d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20,
0x53, 0x74, 0x69, 0x6c, 0x6c, 0x65, 0x68, 0x61, 0x76, 0x73, 0x6b, 0x79,
0x73, 0x74, 0x65, 0x6e, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4d, 0x6f, 0x73, 0x6b,
0x76, 0x61, 0x6d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x6e, 0x61, 0x75, 0x72, 0x69, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x6e, 0x65, 0x70, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x73, 0x6b, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x6b, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x6e, 0x65, 0x77, 0x7a, 0x65, 0x61, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x6e, 0x65, 0x77, 0x7a,
0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x6e, 0x65, 0x77, 0x7a, 0x65, 0x61,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e,
0x64, 0x6c, 0x61, 0x6e, 0x64, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f,
0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4e, 0x65, 0x77,
0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x74, 0x69, 0x64,
0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4e, 0x69,
0x75, 0x65, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0xf8, 0x79,
0x61, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0xf8, 0x79,
0x61, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0xf8, 0x79,
0x61, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x4e, 0x6f, 0x72, 0x64, 0x2d, 0x4d, 0x61, 0x72, 0x69,
0x61, 0x6e, 0x65, 0x6e, 0x65, 0x2d, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64,
0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x74,
0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x4f, 0x6d, 0x73, 0x6b, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69,
0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x4f, 0x6d, 0x73, 0x6b, 0x70, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x70, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x70, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x70, 0x61,
0x6c, 0x61, 0x75, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x70, 0x61,
0x70, 0x75, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x70, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x70, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x70, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x70, 0x65,
0x72, 0x75, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x70, 0x65,
0x72, 0x75, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x70, 0x65, 0x72, 0x75, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x66, 0x69,
0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x66, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x66, 0x69, 0x6c, 0x69,
0x70, 0x70, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78,
0xf8, 0x79, 0x65, 0x6e, 0x65, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e,
0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x53, 0x61, 0x69, 0x6e,
0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x53, 0x61,
0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65,
0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x74, 0x69,
0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x50,
0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x74, 0x69, 0x64, 0x73, 0x73,
0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x50, 0x6f, 0x68, 0x6e,
0x70, 0x65, 0x69, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e,
0x67, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x2c, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x64, 0x51,
0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x2c, 0x20, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73,
0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x52, 0xe9, 0x75, 0x6e,
0x69, 0x6f, 0x6e, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x74,
0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x53, 0x61,
0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x52, 0x75, 0x73, 0x73, 0x69, 0x73, 0x6b, 0x20,
0x28, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x29, 0x20, 0x74, 0x69, 0x64,
0x52, 0x75, 0x73, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x28, 0x53, 0x61, 0x6d,
0x61, 0x72, 0x61, 0x29, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x52, 0x75, 0x73, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x28, 0x53,
0x61, 0x6d, 0x61, 0x72, 0x61, 0x29, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x73, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x73, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x73, 0x61,
0x6d, 0x6f, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x73, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c,
0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x73, 0x69, 0x6e, 0x67, 0x61,
0x70, 0x6f, 0x72, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x73, 0x61, 0x6c,
0x6f, 0x6d, 0x6f, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x74, 0x69,
0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x53,
0xf8, 0x72, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x73, 0x75,
0x72, 0x69, 0x6e, 0x61, 0x6d, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x74,
0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x53, 0x79, 0x6f, 0x77, 0x61, 0x74, 0x61, 0x68, 0x69, 0x74, 0x69, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e,
0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x54, 0x61,
0x69, 0x70, 0x65, 0x69, 0x74, 0x61, 0x64, 0x73, 0x6a, 0x69, 0x6b, 0x69,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f,
0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c,
0x61, 0x75, 0x74, 0x6f, 0x6e, 0x67, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x74, 0x6f, 0x6e, 0x67, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x74, 0x6f, 0x6e, 0x67,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0xf8, 0x79, 0x65, 0x6e,
0x65, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x74, 0x75, 0x72,
0x6b, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x74, 0x75, 0x76, 0x61, 0x6c, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x75, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x75, 0x72, 0x75, 0x67, 0x75, 0x61,
0x79, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x75, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x75, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x75, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x75, 0x73, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x69, 0x73, 0x6b, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x76, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x76, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61,
0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73,
0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x74, 0x69, 0x64,
0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e,
0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x57, 0x61, 0x6b, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64,
0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x2d, 0x20, 0x6f, 0x67, 0x20,
0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0xf8, 0x79, 0x65, 0x6e, 0x65, 0x74,
0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4a, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4a, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x74, 0x69, 0x64,
0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x59, 0x75,
0x6b, 0x6f, 0x6e, 0x61, 0x75, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74,
0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73,
0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x61, 0x6c, 0x61, 0x73,
0x6b, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73,
0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x64, 0x2d, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x64, 0x2d, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x73,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x64,
0x2d, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x74, 0x69, 0x64, 0x73,
0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6e,
0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x61, 0x75, 0x73, 0x74, 0x6b, 0x79, 0x73,
0x74, 0x65, 0x6e, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72,
0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x61, 0x75, 0x73, 0x74, 0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x64, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x61, 0x75, 0x73,
0x74, 0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x52, 0x6f, 0x63,
0x6b, 0x79, 0x20, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x73,
0x20, 0x28, 0x55, 0x53, 0x41, 0x29, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f,
0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6e, 0x20, 0x6e,
0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x73, 0x74, 0x69, 0x6c, 0x6c, 0x65, 0x68, 0x61, 0x76,
0x73, 0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6e,
0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x69, 0x6c, 0x6c, 0x65, 0x68,
0x61, 0x76, 0x73, 0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64,
0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x69, 0x6c, 0x6c,
0x65, 0x68, 0x61, 0x76, 0x73, 0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x41, 0x70, 0x69, 0x61, 0x61, 0x72, 0x61, 0x62, 0x69, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x61, 0x72,
0x61, 0x62, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0x69, 0x64, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x76,
0x65, 0x73, 0x74, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x61,
0x72, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6e, 0x20, 0x6e, 0x6f,
0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b,
0x65, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x65, 0x72, 0x68, 0x61,
0x76, 0x73, 0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x73, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x64,
0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x2d, 0x73, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x64, 0x76,
0x65, 0x73, 0x74, 0x2d, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x61, 0x75, 0x73, 0x74, 0x61, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x61,
0x75, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x64,
0x61, 0x75, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x76,
0x65, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x64,
0x76, 0x65, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x61,
0x73, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x73, 0x6a, 0x61, 0x6e, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x61,
0x73, 0x6f, 0x72, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x73, 0x68, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0x69, 0x64, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69,
0x61, 0x6b, 0x61, 0x70, 0x70, 0x76, 0x65, 0x72, 0x64, 0x69, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x73, 0x6f,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x63, 0x68, 0x69, 0x6c, 0x65,
0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x6b, 0x69, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73,
0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4b, 0x6f, 0x6b, 0x6f,
0x73, 0xf8, 0x79, 0x61, 0x6e, 0x65, 0x6b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x6b, 0x6f, 0x6c,
0x6f, 0x6d, 0x62, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x6b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0xf8, 0x79, 0x61, 0x6e,
0x65, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0xf8, 0x79, 0x61, 0x6e, 0x65,
0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0xf8, 0x79, 0x61, 0x6e, 0x65, 0x6b,
0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x6b, 0x75,
0x62, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x6b, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73,
0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x44, 0x75, 0x6d,
0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c,
0x65, 0x61, 0x75, 0x73, 0x74, 0x74, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x73,
0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x50, 0xe5, 0x73,
0x6b, 0x65, 0xf8, 0x79, 0x61, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x64, 0x73, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x61,
0x75, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x61, 0x75, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x74, 0x69, 0x64, 0x61, 0x75, 0x73, 0x74, 0x65, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x66, 0x6a, 0x65, 0x72, 0x6e, 0x2d, 0x61, 0x75,
0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0xf8, 0x79, 0x61, 0x6e, 0x65, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0xf8, 0x79, 0x61, 0x6e, 0x65, 0x73, 0x6f,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0xf8, 0x79, 0x61,
0x6e, 0x65, 0x66, 0x69, 0x6a, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73,
0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x44, 0x65, 0x69,
0x20, 0x66, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0xf8, 0x72,
0x74, 0x65, 0x72, 0x72, 0x69, 0x74, 0x6f, 0x72, 0x69, 0x61, 0x74, 0x69,
0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x47,
0x61, 0x6c, 0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73, 0xf8, 0x79, 0x61, 0x6e,
0x65, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73,
0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x47, 0x69, 0x6c, 0x62,
0x65, 0x72, 0x74, 0xf8, 0x79, 0x61, 0x6e, 0x65, 0x61, 0x75, 0x73, 0x74,
0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x61, 0x75, 0x73, 0x74, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x61, 0x75, 0x73, 0x74, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x20, 0x6f,
0x67, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x65, 0x6e, 0x65, 0x68, 0x6f,
0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x6b, 0x69, 0x6e, 0x65, 0x73, 0x69,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x68, 0x6f, 0x6e, 0x67, 0x6b, 0x6f,
0x6e, 0x67, 0x6b, 0x69, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x68, 0x6f, 0x6e, 0x67,
0x6b, 0x6f, 0x6e, 0x67, 0x6b, 0x69, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x73, 0x6f,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x4b, 0x68, 0x6f, 0x76, 0x64, 0x61, 0x75, 0x73, 0x74, 0x69, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x69,
0x72, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0x69, 0x64, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x69, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x61, 0x70, 0x61, 0x6e, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x61,
0x75, 0x73, 0x74, 0x6b, 0x61, 0x73, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61,
0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x6b, 0x6f, 0x72, 0x65, 0x61,
0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72,
0x73, 0x6b, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x4c, 0x69, 0x6e, 0x65, 0xf8, 0x79, 0x61, 0x6e, 0x65,
0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d,
0xf8, 0x79, 0x61, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0xf8, 0x79,
0x61, 0x6e, 0x65, 0x6d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69,
0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64,
0x65, 0x6e, 0x20, 0x6d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x73, 0x74, 0x69, 0x6c, 0x6c, 0x65, 0x68, 0x61, 0x76,
0x73, 0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6e,
0x20, 0x6d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x73, 0x74, 0x69, 0x6c, 0x6c, 0x65, 0x68, 0x61, 0x76, 0x73, 0x6b,
0x79, 0x73, 0x74, 0x65, 0x6e, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6e, 0x20, 0x6d,
0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73,
0x74, 0x69, 0x6c, 0x6c, 0x65, 0x68, 0x61, 0x76, 0x73, 0x6b, 0x79, 0x73,
0x74, 0x65, 0x6e, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x6b,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x6e, 0x79, 0x7a, 0x65, 0x61, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x6e, 0x79, 0x7a,
0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x6e, 0x79, 0x7a, 0x65, 0x61, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0x69, 0x64, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e,
0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0xf8, 0x79, 0x61, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x70, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x70, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0x69, 0x64, 0x70, 0x65, 0x72, 0x75, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x66, 0x69, 0x6c,
0x69, 0x70, 0x70, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e,
0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69,
0x78, 0xf8, 0x79, 0x61, 0x6e, 0x65, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x53, 0x61, 0x69, 0x6e,
0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x53, 0x61,
0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73, 0x61, 0x6d, 0x6f, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x74, 0x6f, 0x6e, 0x67,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0xf8, 0x79, 0x61, 0x6e,
0x65, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x75, 0x72, 0x75, 0x67,
0x75, 0x61, 0x79, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x75, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x76,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f,
0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69,
0x73, 0x2d, 0x20, 0x6f, 0x67, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61,
0xf8, 0x79, 0x61, 0x6e, 0x65, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4a, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41,
0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0xe8, 0x73, 0x74, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x75,
0x64, 0x61, 0x66, 0x72, 0x69, 0x63, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x2019,
0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41, 0x66, 0x72, 0x69,
0x63, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6d,
0x61, 0x7a, 0xf2, 0x6e, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x7a, 0xf2, 0x6e, 0x65, 0x73, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x6c, 0x2019,
0x41, 0x6d, 0x61, 0x7a, 0xf2, 0x6e, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0xc8, 0x73, 0x74, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xc8,
0x73, 0x74, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xc8, 0x73, 0x74, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x6d, 0x6f, 0x6e, 0x74,
0x61, 0x6e, 0x68, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61,
0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x68, 0x61, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x68, 0x61, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69,
0x63, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x2019, 0x41, 0x70, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41, 0x70, 0x69,
0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x2019, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75,
0x20, 0x64, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x6f, 0xe8, 0x73,
0x74, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x6f, 0xe8, 0x73,
0x74, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x6f, 0xe8, 0x73, 0x74, 0x20, 0x64,
0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61,
0x20, 0x64, 0x2019, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019,
0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0,
0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69,
0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019,
0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x65, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72,
0xe0, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x6f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x61, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x2019, 0x41,
0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x65, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0,
0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69,
0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61,
0x69, 0x6a, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x41, 0xe7, 0xf2, 0x72, 0x65, 0x73,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x41, 0xe7,
0xf2, 0x72, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x41,
0xe7, 0xf2, 0x72, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x42, 0x61, 0x6e, 0x67, 0x61, 0x6c, 0x64, 0x65, 0x73, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61,
0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x61, 0x6c, 0x64, 0x65,
0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x61, 0x6c, 0x64,
0x65, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x75,
0x74, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43,
0x61, 0x70, 0x2d, 0x56, 0x65, 0x72, 0x64, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65,
0x20, 0x43, 0x61, 0x70, 0x2d, 0x56, 0x65, 0x72, 0x64, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20,
0x43, 0x61, 0x70, 0x2d, 0x56, 0x65, 0x72, 0x64, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20,
0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x43,
0x68, 0x69, 0x6c, 0x65, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68,
0x69, 0x6e, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x69, 0x6c, 0x6c, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x61, 0x64, 0x61, 0x75, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x69, 0x6c, 0x6c, 0x61, 0x20, 0x64,
0x65, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65,
0x20, 0x43, 0x6f, 0x6c, 0xf2, 0x6d, 0x62, 0x69, 0x61, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x49, 0x6c, 0x6c, 0x65,
0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x73, 0x20, 0x49, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f,
0x6b, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75,
0x20, 0x6d, 0x69, 0x65, 0x6a, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65,
0x73, 0x20, 0x49, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x69, 0x6d, 0xf2, 0x72, 0x20, 0x6f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x20, 0x69, 0x6c, 0x6c, 0x61, 0x20, 0x64, 0x65,
0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x61, 0x20, 0x69, 0x6c, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20,
0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20,
0x69, 0x6c, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63,
0x75, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x71, 0x75, 0x61,
0x74, 0x6f, 0x72, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72,
0xf2, 0x70, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0xf2, 0x70, 0x61, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x45, 0x75,
0x72, 0xf2, 0x70, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0xf2, 0x70,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xe8, 0x73, 0x74, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61,
0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0xf2, 0x70, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0xe8, 0x73, 0x74, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0xf2,
0x70, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xe8, 0x73, 0x74, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x65, 0x78, 0x74, 0x72,
0x65, 0x6d, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0xf2, 0x70, 0x61, 0x20,
0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0xf2, 0x70, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x6f, 0xe8, 0x73, 0x74, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x2019,
0x45, 0x75, 0x72, 0xf2, 0x70, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x6f, 0xe8, 0x73, 0x74, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0xf2, 0x70, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x6f, 0xe8, 0x73, 0x74, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x69, 0x6c, 0x6c,
0x65, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x69, 0x6c, 0x6c,
0x65, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x69, 0x6c, 0x6c, 0x65, 0x73, 0x20,
0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x46, 0x69, 0x6a, 0x69, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x20, 0x47, 0x75, 0x61, 0x79, 0x61, 0x6e, 0x61, 0x20, 0x46,
0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x54, 0x65, 0x72, 0x72, 0x65, 0x73,
0x20, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x73, 0x20, 0x65,
0x20, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x74, 0x69, 0x71, 0x75, 0x65, 0x73,
0x20, 0x66, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x65, 0x73, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67,
0x6f, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x65, 0xf2, 0x72,
0x67, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x61, 0x6c, 0x20, 0x6d, 0x65,
0x72, 0x69, 0x64, 0x69, 0x61, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0xe8, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x47,
0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xe8, 0x73, 0x74, 0x20, 0x64, 0x65,
0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0xe8, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20,
0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x6f, 0xe8, 0x73, 0x74,
0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x6f,
0xe8, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x6f,
0xe8, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x74, 0x68,
0x20, 0x47, 0xf2, 0x6c, 0x66, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x48, 0x61, 0x77, 0x61, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69,
0x61, 0x6e, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61,
0x77, 0x61, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e,
0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x65, 0x73, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20,
0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65,
0x20, 0x48, 0x6f, 0x76, 0x64, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x4f, 0x63, 0x65, 0x61, 0x6e, 0x20, 0x49, 0x6e, 0x64,
0x69, 0x63, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x49, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x6f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x49, 0x72, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x2019,
0x49, 0x72, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x49, 0x72, 0x61, 0x6e, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61,
0x20, 0x64, 0x2019, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x49, 0x73, 0x72, 0x61, 0xe8, 0x6c, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61,
0x20, 0x64, 0x2019, 0x49, 0x73, 0x72, 0x61, 0xe8, 0x6c, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x49,
0x73, 0x72, 0x61, 0xe8, 0x6c, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4a,
0x61, 0x70, 0x6f, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6a,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61,
0x7a, 0x61, 0x6a, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x61, 0x74, 0x61, 0x6c, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x43, 0x6f, 0x72, 0xe8, 0x61, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64,
0x65, 0x20, 0x43, 0x6f, 0x72, 0xe8, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f,
0x72, 0xe8, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x69, 0x72, 0x67, 0x75, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65,
0x73, 0x20, 0x45, 0x73, 0x70, 0xf2, 0x72, 0x61, 0x64, 0x65, 0x73, 0x20,
0x45, 0x71, 0x75, 0x61, 0x74, 0x6f, 0x72, 0x69, 0x61, 0x6c, 0x73, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75,
0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x73, 0x20, 0x4d, 0x61,
0x6c, 0x76, 0x69, 0x6e, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x65, 0x73, 0x20, 0x49, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x4d,
0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x49, 0x6c, 0x6c, 0x65, 0x73,
0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69,
0x63, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20,
0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50,
0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63,
0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x6e,
0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0x61,
0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0xf2, 0x75, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0xf2, 0x75, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x6f, 0x73, 0x63, 0xf2, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4e, 0xf2, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0xf2, 0x6e,
0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4e, 0xf2, 0x76, 0x61, 0x20,
0x43, 0x61, 0x6c, 0x65, 0x64, 0xf2, 0x6e, 0x69, 0x61, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20,
0x4e, 0xf2, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0xf2, 0x6e,
0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0xf2, 0x76,
0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x4e, 0xf2, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e,
0x64, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x65, 0x20, 0x4e, 0xf2, 0x76, 0x61, 0x20, 0x5a, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x54, 0x65, 0x72, 0x72, 0x61, 0x2d, 0x4e, 0xf2, 0x76, 0x61, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61,
0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x2d, 0x4e, 0xf2,
0x76, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x2d, 0x4e,
0xf2, 0x76, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x69, 0x6c,
0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20,
0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20,
0x64, 0x2019, 0x4f, 0x6d, 0x73, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x6f, 0x73, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0xf2, 0x76,
0x61, 0x20, 0x47, 0x75, 0x69, 0x6e, 0xe8, 0x61, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64,
0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65,
0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x72, 0xfa, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64,
0x65, 0x20, 0x50, 0x65, 0x72, 0xfa, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x72,
0xfa, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x69, 0x6e, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x69, 0x6e, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x73, 0x20, 0x49, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x46, 0xe9, 0x6e,
0x69, 0x78, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x74, 0x2e,
0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20,
0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20,
0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x52, 0x65, 0xfc, 0x6e, 0x69,
0x6f, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6a,
0x61, 0x6c, 0x69, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61,
0x6a, 0x61, 0x6c, 0x69, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6a, 0x61,
0x6c, 0x69, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f,
0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x65, 0x79, 0x63,
0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x49, 0x6c, 0x6c, 0x65,
0x73, 0x20, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20,
0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65,
0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75,
0x20, 0x64, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x20, 0x64, 0x65, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x6c, 0x6c, 0x61, 0x20,
0x57, 0x61, 0x6b, 0x65, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20,
0x64, 0x2019, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62, 0x75,
0x72, 0x67, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x2019, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6d,
0x62, 0x75, 0x72, 0x67, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x66,
0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x63,
0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x75,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x64, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41,
0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x72, 0x20, 0x41, 0x6d,
0x61, 0x7a, 0xf2, 0x6e, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x72, 0x20,
0x41, 0x6d, 0x61, 0x7a, 0xf2, 0x6e, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x72, 0x20,
0x41, 0x6d, 0x61, 0x7a, 0xf2, 0x6e, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x75, 0x6f,
0x72, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x73,
0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x73, 0x20, 0x52,
0x6f, 0x63, 0x6f, 0x73, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x73, 0x20,
0x4d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x73, 0x20, 0x52, 0x6f,
0x63, 0x6f, 0x73, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x73, 0x20, 0x4d, 0x6f, 0x6e,
0x74, 0x61, 0x6e, 0x68, 0x65, 0x73, 0x20, 0x52, 0x6f, 0x63, 0x6f, 0x73,
0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x74, 0x68, 0x20, 0x50,
0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x74, 0x68,
0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x74, 0x68,
0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019,
0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x61, 0x72, 0x61, 0x62,
0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x67, 0x65,
0x72, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x72, 0x20, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x72, 0x20, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x72, 0x20, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x75,
0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c,
0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x75, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69,
0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x6f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x2019, 0x41,
0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x65, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75,
0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x6f, 0x63, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41,
0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73,
0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c,
0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c,
0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x75, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x73, 0x20, 0x41, 0xe7, 0xf2, 0x72, 0x65,
0x73, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x73, 0x20, 0x41, 0xe7, 0xf2, 0x72, 0x65,
0x73, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75,
0x20, 0x64, 0x65, 0x73, 0x20, 0x41, 0xe7, 0xf2, 0x72, 0x65, 0x73, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x70, 0x2d,
0x56, 0x65, 0x72, 0x64, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69,
0x6c, 0x65, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x72, 0x61, 0x20, 0x69, 0x73, 0x6c,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x61, 0x64, 0x61, 0x75, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x72, 0x61, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x20,
0x64, 0x65, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x43, 0x6f, 0x6c, 0xf2, 0x6d, 0x62, 0x69, 0x61, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x65, 0x73, 0x20, 0x43,
0x6f, 0x6f, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x73, 0x20, 0x49, 0x73, 0x6c,
0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x6d, 0x69, 0x65, 0x6a, 0x61,
0x20, 0x64, 0x65, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x65, 0x73, 0x20, 0x43,
0x6f, 0x6f, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x69, 0x6d, 0xf2, 0x72,
0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x72, 0x61, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x20, 0x64,
0x65, 0x20, 0x50, 0x61, 0x73, 0x71, 0x75, 0x61, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x72, 0x61, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50,
0x61, 0x73, 0x71, 0x75, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x72, 0x61, 0x20, 0x69, 0x73,
0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x71, 0x75, 0x61,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0xf2, 0x70, 0x61,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019,
0x45, 0x75, 0x72, 0xf2, 0x70, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0xf2, 0x70, 0x61, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x45, 0x75,
0x72, 0xf2, 0x70, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xe8, 0x73,
0x74, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x72, 0x20, 0x65, 0x78, 0x74,
0x72, 0x65, 0x6d, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0xf2, 0x70, 0x61,
0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0xf2, 0x70, 0x61, 0x20, 0x6f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019,
0x45, 0x75, 0x72, 0xf2, 0x70, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0xf2, 0x70,
0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x75,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x73, 0x20, 0x69, 0x73, 0x6c, 0x65,
0x73, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x73, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x73, 0x20, 0x69, 0x73, 0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61,
0x6c, 0x64, 0x69, 0x76, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x73, 0x20, 0x69, 0x73,
0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x73,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x72, 0x61, 0x20, 0x47, 0x75, 0x61,
0x79, 0x61, 0x6e, 0x61, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x6e, 0x74, 0x61,
0x72, 0x74, 0x69, 0x64, 0x61, 0x20, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72,
0x69, 0x74, 0x6f, 0x72, 0x69, 0x73, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x73, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x69,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x47, 0x65, 0xf2, 0x72, 0x67, 0x69, 0x61,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x65,
0x73, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x74, 0x68, 0x20, 0x6d, 0x65, 0x72, 0x69, 0x64, 0x69,
0x61, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77,
0x69, 0x63, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72,
0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x47,
0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72,
0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61,
0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61,
0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x65, 0x73,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f,
0x6e, 0x67, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x72, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x72, 0x20, 0x4f, 0x63, 0x65, 0x61, 0x6e,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x63, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x75, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x6f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20,
0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x2019, 0x49, 0x72, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x49, 0x73, 0x72,
0x61, 0xe8, 0x6c, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4a, 0x61, 0x70, 0x6f,
0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x7a, 0x61,
0x6a, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61,
0x7a, 0x61, 0x6a, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x43, 0x6f, 0x72, 0xe8, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x73, 0x20, 0x45, 0x73, 0x70, 0xf2, 0x72, 0x61, 0x64,
0x65, 0x73, 0x20, 0x45, 0x71, 0x75, 0x61, 0x74, 0x6f, 0x72, 0x69, 0x61,
0x6c, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x73, 0x20, 0x49, 0x73,
0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x65,
0x73, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x73, 0x20, 0x49, 0x73, 0x6c,
0x65, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x74, 0x68, 0x20, 0x50, 0x61, 0x63, 0xed,
0x66, 0x69, 0x63, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x74, 0x68, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66,
0x69, 0x63, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64,
0x65, 0x74, 0x68, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0xf2, 0x75, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x74, 0x68, 0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x2d, 0x4e,
0xf2, 0x76, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x72, 0x61, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x72, 0x61, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x72, 0x61, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x4f, 0x6d, 0x73, 0x6b,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x74, 0x68, 0x20, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x74, 0x68, 0x20,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x74, 0x68,
0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x50, 0x65, 0x72, 0xfa, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x75, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x72,
0xfa, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69,
0x6e, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x73, 0x20, 0x49,
0x73, 0x6c, 0x65, 0x73, 0x20, 0x46, 0xe9, 0x6e, 0x69, 0x78, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x52, 0x65, 0xfc, 0x6e, 0x69,
0x6f, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6a, 0x61, 0x6c,
0x69, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x73, 0x20, 0x49, 0x73,
0x6c, 0x65, 0x73, 0x20, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x65, 0xf2, 0x72, 0x67, 0x69,
0x61, 0x20, 0x64, 0x65, 0x74, 0x68, 0x20, 0x53, 0x75, 0x64, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x55, 0x72, 0x75, 0x67,
0x75, 0x61, 0x79, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x79, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x72, 0x61, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x57, 0x61,
0x6b, 0x65, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0xb06, 0xb15, 0xb3e, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb06, 0xb15, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb06, 0xb15, 0xb3e, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb2b, 0xb17, 0xb3e, 0xb28, 0xb3f, 0xb38, 0xb4d,
0xb24, 0xb3e, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb27, 0xb4d, 0xb5f, 0x20,
0xb06, 0xb2b, 0xb4d, 0xb30, 0xb3f, 0xb15, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a,
0xb42, 0xb30, 0xb4d, 0xb2c, 0x20, 0xb06, 0xb2b, 0xb4d, 0xb30, 0xb3f, 0xb15, 0xb3e,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb26, 0xb15, 0xb4d, 0xb37, 0xb3f, 0xb23, 0x20, 0xb06,
0xb2b, 0xb4d, 0xb30, 0xb3f, 0xb15, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19,
0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e,
0x20, 0xb06, 0xb2b, 0xb4d, 0xb30, 0xb3f, 0xb15, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20, 0xb06, 0xb2b, 0xb4d, 0xb30, 0xb3f,
0xb15, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20, 0xb06, 0xb2b, 0xb4d,
0xb30, 0xb3f, 0xb15, 0xb3e, 0x20, 0xb16, 0xb30, 0xb3e, 0xb26, 0xb3f, 0xb28, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb06, 0xb32, 0xb3e, 0xb38, 0xb4d, 0xb15, 0xb3e, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb06, 0xb32, 0xb3e, 0xb38, 0xb4d, 0xb15, 0xb3e, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb32, 0xb3e,
0xb38, 0xb4d, 0xb15, 0xb3e, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb32, 0xb2e, 0xb3e, 0xb1f, 0xb40, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb05, 0xb32, 0xb2e, 0xb3e, 0xb1f, 0xb40, 0x20, 0xb2e, 0xb3e, 0xb28,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb32, 0xb2e, 0xb3e, 0xb1f, 0xb40, 0x20,
0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06,
0xb2e, 0xb3e, 0xb1c, 0xb28, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb2e, 0xb3e,
0xb1c, 0xb28, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb06, 0xb2e, 0xb3e, 0xb1c, 0xb28, 0xb4d, 0x20, 0xb17, 0xb4d,
0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0xb40, 0xb28, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb15, 0xb47, 0xb28, 0xb4d, 0xb26, 0xb4d, 0xb30, 0xb40, 0xb5f, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb15, 0xb47, 0xb28, 0xb4d, 0xb26, 0xb4d, 0xb30, 0xb40, 0xb5f,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb15, 0xb47, 0xb28, 0xb4d, 0xb26, 0xb4d, 0xb30, 0xb40, 0xb5f, 0x20, 0xb26, 0xb3f,
0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb42, 0xb30,
0xb4d, 0xb2c, 0xb3e, 0xb1e, 0xb4d, 0xb1a, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a,
0xb42, 0xb30, 0xb4d, 0xb2c, 0xb3e, 0xb1e, 0xb4d, 0xb1a, 0xb33, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb42, 0xb30,
0xb4d, 0xb2c, 0xb3e, 0xb1e, 0xb4d, 0xb1a, 0xb33, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e,
0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb3e, 0xb30, 0xb4d, 0xb2c,
0xb24, 0xb4d, 0xb5f, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb3e, 0xb30, 0xb4d, 0xb2c,
0xb24, 0xb4d, 0xb5f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb3e, 0xb30, 0xb4d, 0xb2c, 0xb24, 0xb4d, 0xb5f, 0x20,
0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a,
0xb3e, 0xb38, 0xb3f, 0xb2b, 0xb3f, 0xb15, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a,
0xb3e, 0xb38, 0xb3f, 0xb2b, 0xb3f, 0xb15, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e,
0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb3e, 0xb38, 0xb3f, 0xb2b,
0xb3f, 0xb15, 0xb4d, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb05, 0xb28, 0xb3e, 0xb21, 0xb3f, 0xb30, 0xb4d, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb05, 0xb28, 0xb3e, 0xb21, 0xb3f, 0xb30, 0xb4d, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb28, 0xb3e, 0xb21, 0xb3f, 0xb30,
0xb4d, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb06, 0xb2a, 0xb3f, 0xb06, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb2a, 0xb3f,
0xb06, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb06, 0xb2a, 0xb3f, 0xb06, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb15, 0xb24, 0xb4c, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb05, 0xb15, 0xb24, 0xb4c, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb05, 0xb15, 0xb24, 0xb4c, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37,
0xb4d, 0xb2e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb15, 0xb4d, 0xb24, 0xb4b, 0xb2c,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb15, 0xb4d, 0xb24, 0xb4b, 0xb2c, 0x20, 0xb2e,
0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb15, 0xb4d, 0xb24, 0xb4b,
0xb2c, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb06, 0xb30, 0xb2c, 0xb40, 0xb5f, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb30,
0xb2c, 0xb40, 0xb5f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb06, 0xb30, 0xb2c, 0xb40, 0xb5f, 0x20, 0xb26, 0xb3f, 0xb2c,
0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb30, 0xb4d, 0xb1c,
0xb47, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb28, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06,
0xb30, 0xb4d, 0xb1c, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb28, 0xb3e, 0x20, 0xb2e,
0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb30,
0xb4d, 0xb1c, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb28, 0xb3e, 0x20, 0xb17, 0xb4d,
0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20, 0xb06, 0xb30, 0xb4d, 0xb1c, 0xb47,
0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb28, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb36,
0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20, 0xb06, 0xb30, 0xb4d, 0xb1c, 0xb47, 0xb23, 0xb4d,
0xb1f, 0xb3f, 0xb28, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20, 0xb06,
0xb30, 0xb4d, 0xb1c, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb28, 0xb3e, 0x20, 0xb17,
0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb06, 0xb30, 0xb4d, 0xb2e, 0xb47, 0xb28, 0xb3f, 0xb06, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb06, 0xb30, 0xb4d, 0xb2e, 0xb47, 0xb28, 0xb3f, 0xb06, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb30, 0xb4d,
0xb2e, 0xb47, 0xb28, 0xb3f, 0xb06, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d,
0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb1f, 0xb32, 0xb3e,
0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb15, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb1f,
0xb32, 0xb3e, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb15, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28,
0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb1f, 0xb32, 0xb3e,
0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb15, 0xb4d, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32,
0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb27, 0xb4d, 0xb5f, 0x20, 0xb05,
0xb37, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f, 0xb06, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb05, 0xb37, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f, 0xb5f, 0x20,
0xb2e, 0xb27, 0xb4d, 0xb5f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb37, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32,
0xb3f, 0xb5f, 0x20, 0xb2e, 0xb27, 0xb4d, 0xb5f, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e,
0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb37, 0xb4d, 0xb1f, 0xb4d,
0xb30, 0xb47, 0xb32, 0xb3f, 0xb5f, 0x20, 0xb2e, 0xb27, 0xb4d, 0xb5f, 0x20, 0xb2a,
0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb37, 0xb4d,
0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f, 0xb5f, 0x20, 0xb2e, 0xb27, 0xb4d, 0xb5f,
0x20, 0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e,
0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb37, 0xb4d, 0xb1f, 0xb4d,
0xb30, 0xb47, 0xb32, 0xb3f, 0xb5f, 0x20, 0xb2e, 0xb27, 0xb4d, 0xb5f, 0x20, 0xb2a,
0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb42, 0xb30, 0xb4d, 0xb2c, 0x20, 0xb05,
0xb37, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f, 0xb06, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb05, 0xb37, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f, 0xb5f, 0x20,
0xb2a, 0xb42, 0xb30, 0xb4d, 0xb2c, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb37, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47,
0xb32, 0xb3f, 0xb5f, 0x20, 0xb2a, 0xb42, 0xb30, 0xb4d, 0xb2c, 0x20, 0xb26, 0xb3f,
0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb36, 0xb4d,
0xb1a, 0xb3f, 0xb2e, 0x20, 0xb05, 0xb37, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32,
0xb3f, 0xb06, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb37, 0xb4d, 0xb1f, 0xb4d, 0xb30,
0xb47, 0xb32, 0xb3f, 0xb5f, 0x20, 0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb05,
0xb37, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f, 0xb5f, 0x20, 0xb2a, 0xb36,
0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb1c, 0xb47, 0xb30, 0xb2c, 0xb3e, 0xb07, 0xb1c,
0xb3e, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb1c, 0xb47, 0xb30, 0xb2c, 0xb3e,
0xb07, 0xb1c, 0xb3e, 0xb28, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb19, 0xb4d, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb06, 0xb1c, 0xb47, 0xb30, 0xb2c, 0xb3e, 0xb07, 0xb1c, 0xb3e,
0xb28, 0x20, 0xb16, 0xb30, 0xb3e, 0xb26, 0xb3f, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb06, 0xb1c, 0xb4b, 0xb30, 0xb47, 0xb38, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06,
0xb1c, 0xb4b, 0xb30, 0xb47, 0xb38, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb06, 0xb1c, 0xb4b, 0xb30, 0xb47, 0xb38, 0xb4d, 0x20, 0xb17,
0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2c, 0xb3e, 0xb02, 0xb32, 0xb3e, 0xb26, 0xb47, 0xb36, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2c, 0xb3e, 0xb02, 0xb32, 0xb3e, 0xb26, 0xb47, 0xb36, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2c, 0xb3e, 0xb02,
0xb32, 0xb3e, 0xb26, 0xb47, 0xb36, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d,
0xb2e, 0xb15, 0xb3e, 0xb33, 0xb40, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2d, 0xb41,
0xb1f, 0xb3e, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2c, 0xb32, 0xb3f, 0xb2d, 0xb3f,
0xb06, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2c, 0xb4d, 0xb30, 0xb3e, 0xb38, 0xb3f, 0xb32,
0xb3f, 0xb06, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2c, 0xb4d, 0xb30, 0xb3e, 0xb38, 0xb3f,
0xb32, 0xb3f, 0xb06, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb2c, 0xb4d, 0xb30, 0xb3e, 0xb38, 0xb3f, 0xb32, 0xb3f, 0xb06,
0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb2c, 0xb4d, 0xb30, 0xb41, 0xb28, 0xb47, 0xb07, 0x20, 0xb21,
0xb3e, 0xb30, 0xb41, 0xb38, 0xb3e, 0xb32, 0xb3e, 0xb2e, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb15, 0xb47, 0xb2a, 0xb4d, 0x200c, 0x20, 0xb2d, 0xb30, 0xb4d, 0xb26, 0xb47, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb15, 0xb47, 0xb2a, 0xb4d, 0x200c, 0x20, 0xb2d, 0xb30, 0xb4d,
0xb26, 0xb47, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb15, 0xb47, 0xb2a, 0xb4d, 0x200c, 0x20, 0xb2d, 0xb30, 0xb4d, 0xb26,
0xb47, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33,
0xb40, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1a, 0xb3e, 0xb2e, 0xb4b, 0xb30, 0xb4b,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1a, 0xb3e, 0xb25,
0xb3e, 0xb2e, 0xb4d, 0x200c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1a, 0xb3e, 0xb25, 0xb3e,
0xb2e, 0xb4d, 0x200c, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb1a, 0xb3e, 0xb25, 0xb3e, 0xb2e, 0xb4d, 0x200c, 0x20, 0xb26,
0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1a, 0xb3f,
0xb32, 0xb3f, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1a, 0xb3f, 0xb32, 0xb3f, 0x20, 0xb2e,
0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1a, 0xb3f,
0xb32, 0xb3f, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e,
0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1a, 0xb40, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb1a, 0xb40, 0xb28, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb1a, 0xb40, 0xb28, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb16, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb1f, 0xb2e, 0xb3e,
0xb38, 0x20, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15,
0xb4b, 0xb15, 0xb4b, 0xb38, 0xb4d, 0x200c, 0x20, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a,
0xb2a, 0xb41, 0xb1e, 0xb4d, 0xb1c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb32, 0xb2e,
0xb4d, 0xb2c, 0xb3f, 0xb06, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb32, 0xb2e, 0xb4d,
0xb2c, 0xb3f, 0xb06, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb15, 0xb32, 0xb2e, 0xb4d, 0xb2c, 0xb3f, 0xb06, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40,
0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb41,
0xb15, 0xb4d, 0x200c, 0x20, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a, 0xb2a, 0xb41, 0xb1e,
0xb4d, 0xb1c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb41, 0xb15, 0xb4d, 0x200c, 0x20,
0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a, 0xb2a, 0xb41, 0xb1e, 0xb4d, 0xb1c, 0x20, 0xb2e,
0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb41, 0xb15, 0xb4d, 0x200c,
0x20, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a, 0xb2a, 0xb41, 0xb1e, 0xb4d, 0xb1c, 0x20,
0xb05, 0xb30, 0xb4d, 0xb26, 0xb4d, 0xb27, 0x20, 0xb17, 0xb40, 0xb37, 0xb4d, 0xb2e,
0xb15, 0xb3e, 0xb33, 0xb40, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb4d, 0xb5f,
0xb41, 0xb2c, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb4d, 0xb5f, 0xb41, 0xb2c,
0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb15, 0xb4d, 0xb5f, 0xb41, 0xb2c, 0xb3e, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e,
0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb21, 0xb47, 0xb2d, 0xb3f, 0xb38,
0xb4d, 0x200c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb21, 0xb41, 0xb2e, 0xb4b, 0xb23, 0xb4d,
0xb1f, 0x2d, 0xb21, 0xb3f, 0x2018, 0xb09, 0xb30, 0xb2d, 0xb3f, 0xb32, 0xb4d, 0xb32,
0xb47, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb42, 0xb30, 0xb4d, 0xb2c, 0x20, 0xb24,
0xb3f, 0xb2e, 0xb4b, 0xb30, 0xb4d, 0x200c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb07, 0xb37,
0xb4d, 0xb1f, 0xb30, 0xb4d, 0x200c, 0x20, 0xb06, 0xb07, 0xb32, 0xb4d, 0xb5f, 0xb3e,
0xb23, 0xb4d, 0xb21, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb07, 0xb37, 0xb4d, 0xb1f, 0xb30,
0xb4d, 0x200c, 0x20, 0xb06, 0xb07, 0xb32, 0xb4d, 0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb07, 0xb37, 0xb4d, 0xb1f, 0xb30, 0xb4d, 0x200c, 0x20, 0xb06, 0xb07, 0xb32, 0xb4d,
0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d,
0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb07, 0xb15, 0xb4d, 0xb71,
0xb47, 0xb21, 0xb30, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb47, 0xb28, 0xb4d, 0xb26,
0xb4d, 0xb30, 0xb40, 0xb5f, 0x20, 0xb5f, 0xb41, 0xb30, 0xb4b, 0xb2a, 0xb40, 0xb5f,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb47, 0xb28, 0xb4d, 0xb26, 0xb4d, 0xb30, 0x20,
0xb5f, 0xb41, 0xb30, 0xb4b, 0xb2a, 0xb40, 0xb5f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb47, 0xb28, 0xb4d, 0xb26, 0xb4d, 0xb30, 0xb40,
0xb5f, 0x20, 0xb5f, 0xb41, 0xb30, 0xb4b, 0xb2a, 0xb40, 0xb5f, 0x20, 0xb17, 0xb4d,
0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb2a, 0xb42, 0xb30, 0xb4d, 0xb2c, 0xb3e, 0xb1e, 0xb4d, 0xb1a, 0xb33, 0x20, 0xb5f,
0xb41, 0xb30, 0xb4b, 0xb2a, 0xb40, 0xb5f, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb42,
0xb30, 0xb4d, 0xb2c, 0xb3e, 0xb1e, 0xb4d, 0xb1a, 0xb33, 0x20, 0xb5f, 0xb41, 0xb30,
0xb4b, 0xb2a, 0xb40, 0xb5f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb42, 0xb30, 0xb4d, 0xb2c, 0xb3e, 0xb1e, 0xb4d,
0xb1a, 0xb33, 0x20, 0xb5f, 0xb41, 0xb30, 0xb4b, 0xb2a, 0xb40, 0xb5f, 0x20, 0xb17,
0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2a, 0xb30, 0xb2c, 0xb30, 0xb4d, 0xb24, 0xb4d, 0xb24, 0xb40, 0x2d, 0xb2a,
0xb42, 0xb30, 0xb4d, 0xb2c, 0xb3e, 0xb1e, 0xb4d, 0xb1a, 0xb33, 0x20, 0xb5f, 0xb41,
0xb30, 0xb4b, 0xb2a, 0xb40, 0xb5f, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb36, 0xb4d,
0xb1a, 0xb3f, 0xb2e, 0xb3e, 0xb1e, 0xb4d, 0xb1a, 0xb33, 0x20, 0xb5f, 0xb41, 0xb30,
0xb4b, 0xb2a, 0xb40, 0xb5f, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb36, 0xb4d, 0xb1a,
0xb3f, 0xb2e, 0xb3e, 0xb1e, 0xb4d, 0xb1a, 0xb33, 0x20, 0xb5f, 0xb41, 0xb30, 0xb4b,
0xb2a, 0xb40, 0xb5f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0xb3e, 0xb1e, 0xb4d,
0xb1a, 0xb33, 0x20, 0xb5f, 0xb41, 0xb30, 0xb4b, 0xb2a, 0xb40, 0xb5f, 0x20, 0xb17,
0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2b, 0xb15, 0xb32, 0xb4d, 0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21, 0x20, 0xb26,
0xb4d, 0xb71, 0xb40, 0xb2a, 0xb2a, 0xb41, 0xb1e, 0xb4d, 0xb1c, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2b, 0xb15, 0xb32, 0xb4d, 0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21, 0x20, 0xb26,
0xb4d, 0xb71, 0xb40, 0xb2a, 0xb2a, 0xb41, 0xb1e, 0xb4d, 0xb1c, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2b, 0xb15, 0xb32, 0xb4d, 0xb5f, 0xb3e,
0xb23, 0xb4d, 0xb21, 0x20, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a, 0xb2a, 0xb41, 0xb1e,
0xb4d, 0xb1c, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e,
0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2b, 0xb3f, 0xb1c, 0xb3f, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2b, 0xb3f, 0xb1c, 0xb3f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb2b, 0xb3f, 0xb1c, 0xb3f, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37,
0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0xb40, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2b,
0xb4d, 0xb30, 0xb47, 0xb1e, 0xb4d, 0xb1a, 0x20, 0xb17, 0xb41, 0xb06, 0xb28, 0xb3e,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb2b, 0xb4d, 0xb30, 0xb47, 0xb1e, 0xb4d, 0xb1a, 0x20,
0xb26, 0xb15, 0xb4d, 0xb37, 0xb3f, 0xb23, 0x20, 0xb0f, 0xb2c, 0xb02, 0x20, 0xb06,
0xb23, 0xb4d, 0xb1f, 0xb3e, 0xb30, 0xb4d, 0xb15, 0xb3e, 0xb1f, 0xb3f, 0xb15, 0xb4d,
0x200c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb17, 0xb3e, 0xb32, 0xb3e, 0xb2a, 0xb3e, 0xb17,
0xb4b, 0xb38, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb17, 0xb3e, 0xb2e, 0xb4d, 0xb2c,
0xb3f, 0xb5f, 0xb30, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1c, 0xb30, 0xb4d, 0xb1c,
0xb3f, 0xb06, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1c, 0xb30, 0xb4d, 0xb1c, 0xb3f, 0xb06,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb1c, 0xb30, 0xb4d, 0xb1c, 0xb3f, 0xb06, 0x20, 0xb16, 0xb30, 0xb3e, 0xb26, 0xb3f,
0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb17, 0xb3f, 0xb32, 0xb2c, 0xb30, 0xb4d, 0xb1f,
0xb4d, 0x200c, 0x20, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a, 0xb2a, 0xb41, 0xb1e, 0xb4d,
0xb1c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb28, 0xb71, 0xb3f,
0xb1a, 0xb4d, 0x20, 0xb2e, 0xb3f, 0xb28, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a,
0xb42, 0xb30, 0xb4d, 0xb2c, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb28, 0xb32, 0xb4d,
0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb42,
0xb30, 0xb4d, 0xb2c, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb28, 0xb32, 0xb4d, 0xb5f,
0xb3e, 0xb23, 0xb4d, 0xb21, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb42, 0xb30, 0xb4d, 0xb2c, 0x20, 0xb17,
0xb4d, 0xb30, 0xb40, 0xb28, 0xb32, 0xb4d, 0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb4d,
0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20, 0xb17, 0xb4d,
0xb30, 0xb40, 0xb28, 0xb32, 0xb4d, 0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb4d, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20, 0xb17, 0xb4d,
0xb30, 0xb40, 0xb28, 0xb32, 0xb4d, 0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb4d, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a,
0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb28, 0xb32,
0xb4d, 0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb4d, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40,
0xb37, 0xb4d, 0xb2e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb17, 0xb32, 0xb4d, 0xb2b, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb17,
0xb41, 0xb0f, 0xb28, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb39, 0xb71, 0xb3e, 0xb07,
0x2d, 0xb06, 0xb32, 0xb47, 0xb09, 0xb1f, 0xb3f, 0xb5f, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb39, 0xb71, 0xb3e, 0xb07, 0x2d, 0xb06, 0xb32, 0xb47, 0xb09, 0xb1f, 0xb3f, 0xb5f,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb39, 0xb71, 0xb3e, 0xb07, 0x2d, 0xb06, 0xb32, 0xb47, 0xb09, 0xb1f, 0xb3f, 0xb5f,
0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb39, 0xb02, 0x20, 0xb15, 0xb02, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb39, 0xb02, 0x20,
0xb15, 0xb02, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb39,
0xb02, 0x20, 0xb15, 0xb02, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e,
0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb39, 0xb4b, 0xb2d, 0xb21, 0xb4d,
0x200c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb39, 0xb4b, 0xb2d, 0xb21, 0xb4d, 0x200c, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb39, 0xb4b, 0xb2d, 0xb21,
0xb4d, 0x200c, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e,
0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2d, 0xb3e, 0xb30, 0xb24, 0xb40, 0xb5f, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2d, 0xb3e, 0xb30, 0xb24,
0x20, 0xb2e, 0xb3e, 0xb39, 0xb3e, 0xb38, 0xb3e, 0xb17, 0xb30, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb07, 0xb23, 0xb4d, 0xb21, 0xb4b, 0xb1a, 0xb3e, 0xb07, 0xb28, 0xb3e, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb27, 0xb4d, 0xb5f, 0x20, 0xb07, 0xb23, 0xb4d, 0xb21,
0xb4b, 0xb28, 0xb47, 0xb38, 0xb3f, 0xb06, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb42,
0xb30, 0xb4d, 0xb2c, 0x20, 0xb07, 0xb23, 0xb4d, 0xb21, 0xb4b, 0xb28, 0xb47, 0xb38,
0xb3f, 0xb06, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e,
0x20, 0xb07, 0xb23, 0xb4d, 0xb21, 0xb4b, 0xb28, 0xb47, 0xb38, 0xb3f, 0xb06, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb07, 0xb30, 0xb3e, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb07,
0xb30, 0xb3e, 0xb28, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb07, 0xb30, 0xb3e, 0xb28, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb07, 0xb05, 0xb30, 0xb15, 0xb41, 0xb1f, 0xb38, 0xb4d,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb07, 0xb30, 0xb4d, 0xb15, 0xb41, 0xb1f, 0xb38,
0xb4d, 0xb15, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb07, 0xb30, 0xb4d, 0xb15, 0xb41, 0xb1f, 0xb38, 0xb4d, 0xb15, 0xb4d, 0x20, 0xb17,
0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0xb40, 0xb28, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb07, 0xb38, 0xb4d, 0xb30, 0xb3e, 0xb0f, 0xb32, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb07, 0xb38, 0xb4d, 0xb30, 0xb3e, 0xb0f, 0xb32, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb07, 0xb38, 0xb4d,
0xb30, 0xb3e, 0xb0f, 0xb32, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb1c, 0xb3e, 0xb2a, 0xb3e, 0xb28, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb1c, 0xb3e, 0xb2a, 0xb3e, 0xb28, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb1c, 0xb3e, 0xb2a, 0xb3e, 0xb28, 0x20, 0xb26, 0xb3f, 0xb2c,
0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb47, 0xb1f, 0xb4d,
0xb30, 0xb4b, 0xb2a, 0xb3e, 0xb2d, 0xb32, 0xb4b, 0xb2d, 0xb4d, 0xb38, 0xb15, 0x2d,
0xb15, 0xb2e, 0xb1a, 0xb1f, 0xb38, 0xb4d, 0xb15, 0xb3f, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb2a, 0xb47, 0xb1f, 0xb4d, 0xb30, 0xb4b, 0xb2a, 0xb3e, 0xb2d, 0xb32, 0xb4b, 0xb2d,
0xb4d, 0xb38, 0xb15, 0x2d, 0xb15, 0xb2e, 0xb1a, 0xb1f, 0xb38, 0xb4d, 0xb15, 0xb3f,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb47, 0xb1f,
0xb4d, 0xb30, 0xb4b, 0xb2a, 0xb3e, 0xb2d, 0xb32, 0xb4b, 0xb2d, 0xb4d, 0xb38, 0xb15,
0x2d, 0xb15, 0xb2e, 0xb1a, 0xb1f, 0xb38, 0xb4d, 0xb15, 0xb3f, 0x20, 0xb17, 0xb4d,
0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb3e, 0xb1c,
0xb3e, 0xb15, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb2a, 0xb42, 0xb30, 0xb4d, 0xb2c, 0x20, 0xb15, 0xb3e, 0xb1c, 0xb3e, 0xb15, 0xb38,
0xb4d, 0xb24, 0xb3e, 0xb28, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb36, 0xb4d,
0xb1a, 0xb3f, 0xb2e, 0x20, 0xb15, 0xb3e, 0xb1c, 0xb3e, 0xb15, 0xb38, 0xb4d, 0xb24,
0xb3e, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb4b, 0xb30, 0xb3f, 0xb5f, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb15, 0xb4b, 0xb30, 0xb3f, 0xb5f, 0x20, 0xb2e, 0xb3e, 0xb28,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb4b, 0xb30, 0xb3f, 0xb5f, 0x20, 0xb26,
0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb4b,
0xb38, 0xb30, 0xb47, 0xb07, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb4d, 0xb30, 0xb3e,
0xb38, 0xb28, 0xb4b, 0xb5f, 0xb3e, 0xb30, 0xb38, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb15, 0xb4d, 0xb30, 0xb3e, 0xb38, 0xb28, 0xb4b, 0xb5f, 0xb3e, 0xb30, 0xb38,
0xb4d, 0xb15, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb15, 0xb4d, 0xb30, 0xb3e, 0xb38, 0xb28, 0xb4b, 0xb5f, 0xb3e, 0xb30,
0xb38, 0xb4d, 0xb15, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15,
0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb3f, 0xb30, 0xb4d, 0xb17, 0xb3f,
0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb32, 0xb19, 0xb4d,
0xb15, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb32, 0xb3e, 0xb07, 0xb28, 0xb4d, 0x200c,
0x20, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a, 0xb2a, 0xb41, 0xb1e, 0xb4d, 0xb1c, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb32, 0xb30, 0xb4d, 0xb21, 0x20, 0xb39, 0xb4b, 0xb71, 0xb47,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb32, 0xb30, 0xb4d, 0xb21, 0x20, 0xb39, 0xb4b, 0xb71,
0xb47, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb32, 0xb30, 0xb4d, 0xb21, 0x20, 0xb39, 0xb4b, 0xb71, 0xb47, 0x20, 0xb26,
0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb3e,
0xb15, 0xb3e, 0xb13, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb15, 0xb3e, 0xb13, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb15, 0xb3e, 0xb13,
0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb2e, 0xb3e, 0xb17, 0xb3e, 0xb21, 0xb3e, 0xb28, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb2e, 0xb3e, 0xb17, 0xb3e, 0xb21, 0xb3e, 0xb28, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28,
0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb3e, 0xb17, 0xb3e,
0xb21, 0xb3e, 0xb28, 0xb4d, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e,
0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb3e, 0xb32, 0xb47, 0xb38,
0xb3f, 0xb06, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb3e, 0xb33, 0xb26, 0xb4d, 0xb71,
0xb40, 0xb2a, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb3e, 0xb30, 0xb4d, 0xb15, 0xb4d,
0xb71, 0xb47, 0xb38, 0xb3e, 0xb38, 0xb4d, 0x200c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e,
0xb3e, 0xb30, 0xb4d, 0xb36, 0xb3e, 0xb32, 0xb4d, 0x200c, 0x20, 0xb26, 0xb4d, 0xb71,
0xb40, 0xb2a, 0xb2a, 0xb41, 0xb1e, 0xb4d, 0xb1c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e,
0xb4c, 0xb30, 0xb3f, 0xb38, 0xb38, 0xb4d, 0x200c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e,
0xb4c, 0xb30, 0xb3f, 0xb38, 0xb38, 0xb4d, 0x200c, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e,
0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb4c, 0xb30, 0xb3f, 0xb38,
0xb38, 0xb4d, 0x200c, 0x20, 0xb16, 0xb30, 0xb3e, 0xb26, 0xb3f, 0xb28, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb2e, 0xb3e, 0xb71, 0xb38, 0xb28, 0xb4d, 0x200c, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2e, 0xb47, 0xb15, 0xb4d, 0xb38, 0xb3f, 0xb15, 0xb3e, 0xb28, 0xb4d, 0x20,
0xb2a, 0xb3e, 0xb38, 0xb3f, 0xb2b, 0xb3f, 0xb15, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb2e, 0xb47, 0xb15, 0xb4d, 0xb38, 0xb3f, 0xb15, 0xb3e, 0xb28, 0xb4d, 0x20, 0xb2a,
0xb3e, 0xb38, 0xb3f, 0xb2b, 0xb3f, 0xb15, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e,
0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb47, 0xb15, 0xb4d, 0xb38,
0xb3f, 0xb15, 0xb3e, 0xb28, 0xb4d, 0x20, 0xb2a, 0xb3e, 0xb38, 0xb3f, 0xb2b, 0xb3f,
0xb15, 0xb4d, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb09, 0xb32, 0xb3e, 0xb28, 0xb4d, 0x200c, 0xb2c, 0xb3e, 0xb1f, 0xb30,
0xb4d, 0x200c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb09, 0xb32, 0xb3e, 0xb28, 0xb4d, 0x200c,
0xb2c, 0xb3e, 0xb1f, 0xb30, 0xb4d, 0x200c, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb09, 0xb32, 0xb3e, 0xb28, 0xb4d, 0x200c, 0xb2c, 0xb3e, 0xb1f,
0xb30, 0xb4d, 0x200c, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15,
0xb3e, 0xb33, 0xb40, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb38, 0xb4d, 0xb15,
0xb4b, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb38, 0xb4d, 0xb15, 0xb4b, 0x20, 0xb2e,
0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb38,
0xb4d, 0xb15, 0xb4b, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15,
0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb3f, 0xb06, 0xb01, 0xb2e, 0xb3e,
0xb30, 0xb4d, 0x200c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb3e, 0xb09, 0xb30, 0xb41,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb47, 0xb2a, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb28, 0xb4d, 0xb5f, 0xb41, 0x20, 0xb15, 0xb3e, 0xb32, 0xb47, 0xb21, 0xb4b,
0xb28, 0xb3f, 0xb06, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb4d, 0xb5f, 0xb41, 0x20,
0xb15, 0xb3e, 0xb32, 0xb47, 0xb21, 0xb4b, 0xb28, 0xb3f, 0xb06, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb4d, 0xb5f,
0xb41, 0x20, 0xb15, 0xb3e, 0xb32, 0xb47, 0xb21, 0xb4b, 0xb28, 0xb3f, 0xb06, 0x20,
0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb28, 0xb4d, 0xb5f, 0xb41, 0xb1c, 0xb3f, 0xb32, 0xb3e, 0xb23, 0xb4d,
0xb21, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb4d, 0xb5f, 0xb41, 0xb1c, 0xb3f, 0xb32,
0xb3e, 0xb23, 0xb4d, 0xb21, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb4d, 0xb5f, 0xb41, 0xb1c, 0xb3f, 0xb32, 0xb3e,
0xb23, 0xb4d, 0xb21, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb28, 0xb4d, 0xb5f, 0xb41, 0xb2b, 0xb3e, 0xb09, 0xb23, 0xb4d,
0xb21, 0xb32, 0xb4d, 0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb4d, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb28, 0xb4d, 0xb5f, 0xb41, 0xb2b, 0xb3e, 0xb09, 0xb23, 0xb4d, 0xb21, 0xb32,
0xb4d, 0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e,
0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb4d, 0xb5f, 0xb41, 0xb2b,
0xb3e, 0xb09, 0xb23, 0xb4d, 0xb21, 0xb32, 0xb4d, 0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21,
0xb4d, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb28, 0xb3f, 0xb5f, 0xb41, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb30, 0xb2b,
0xb4b, 0xb15, 0xb4d, 0x200c, 0x20, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb28, 0xb30, 0xb2b, 0xb4b, 0xb15, 0xb4d, 0x200c, 0x20, 0xb26, 0xb4d,
0xb71, 0xb40, 0xb2a, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb28, 0xb30, 0xb2b, 0xb4b, 0xb15, 0xb4d, 0x200c, 0x20, 0xb26,
0xb4d, 0xb71, 0xb40, 0xb2a, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e,
0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2b, 0xb30, 0xb4d, 0xb23, 0xb4d,
0xb23, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb4b, 0x20, 0xb21, 0xb3f, 0x20, 0xb28, 0xb4b,
0xb30, 0xb4b, 0xb28, 0xb4d, 0xb28, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2b, 0xb30,
0xb4d, 0xb23, 0xb4d, 0xb23, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb4b, 0x20, 0xb21, 0xb3f,
0x20, 0xb28, 0xb4b, 0xb30, 0xb4b, 0xb28, 0xb4d, 0xb28, 0xb3e, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2b, 0xb30, 0xb4d,
0xb23, 0xb4d, 0xb23, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb4b, 0x20, 0xb21, 0xb3f, 0x20,
0xb28, 0xb4b, 0xb30, 0xb4b, 0xb28, 0xb4d, 0xb28, 0xb3e, 0x20, 0xb17, 0xb4d, 0xb30,
0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb09,
0xb24, 0xb4d, 0xb24, 0xb30, 0x20, 0xb2e, 0xb47, 0xb30, 0xb3f, 0xb06, 0xb28, 0xb3e,
0x20, 0xb06, 0xb07, 0xb32, 0xb4d, 0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb28, 0xb4b, 0xb2d, 0xb4b, 0xb38, 0xb3f, 0xb2c, 0xb3f, 0xb30, 0xb38,
0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb4b, 0xb2d, 0xb4b, 0xb38, 0xb3f,
0xb2c, 0xb3f, 0xb30, 0xb38, 0xb4d, 0xb15, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19,
0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb4b, 0xb2d, 0xb4b, 0xb38, 0xb3f,
0xb2c, 0xb3f, 0xb30, 0xb38, 0xb4d, 0xb15, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37,
0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb13, 0xb2e, 0xb38,
0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb13, 0xb2e, 0xb38, 0xb4d, 0xb15, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb13,
0xb2e, 0xb38, 0xb4d, 0xb15, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e,
0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb3e, 0xb15, 0xb3f, 0xb38,
0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb3e, 0xb15, 0xb3f,
0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb2a, 0xb3e, 0xb15, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20,
0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb2a, 0xb3e, 0xb32, 0xb3e, 0xb09, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a,
0xb2a, 0xb41, 0xb06, 0x20, 0xb28, 0xb4d, 0xb5f, 0xb41, 0x20, 0xb17, 0xb3f, 0xb28,
0xb3f, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb3e, 0xb30, 0xb3e, 0xb17, 0xb41, 0xb0f,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb3e, 0xb30, 0xb3e, 0xb17, 0xb41, 0xb0f, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a,
0xb3e, 0xb30, 0xb3e, 0xb17, 0xb41, 0xb0f, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37,
0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb47, 0xb30,
0xb41, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb47, 0xb30, 0xb41, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb47, 0xb30,
0xb41, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb2b, 0xb3f, 0xb32, 0xb3f, 0xb2a, 0xb3e, 0xb07, 0xb28,
0xb4d, 0x200c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2b, 0xb3f, 0xb32, 0xb3f, 0xb2a, 0xb3e,
0xb07, 0xb28, 0xb4d, 0x200c, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb2b, 0xb3f, 0xb32, 0xb3f, 0xb2a, 0xb3e, 0xb07, 0xb28,
0xb4d, 0x200c, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e,
0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2b, 0xb3f, 0xb28, 0xb3f, 0xb15, 0xb4d, 0xb38,
0x20, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a, 0xb2a, 0xb41, 0xb1e, 0xb4d, 0xb1c, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb38, 0xb47, 0xb23, 0xb4d, 0xb1f, 0x2e, 0x20, 0xb2a, 0xb3f,
0xb0f, 0xb30, 0xb47, 0x20, 0xb0f, 0xb2c, 0xb02, 0x20, 0xb2e, 0xb3f, 0xb15, 0xb4d,
0xb5f, 0xb41, 0xb32, 0xb4b, 0xb28, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb38, 0xb47,
0xb23, 0xb4d, 0xb1f, 0x2e, 0x20, 0xb2a, 0xb3f, 0xb0f, 0xb30, 0xb47, 0x20, 0xb0f,
0xb2c, 0xb02, 0x20, 0xb2e, 0xb3f, 0xb15, 0xb4d, 0xb5f, 0xb41, 0xb32, 0xb4b, 0xb28,
0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb38, 0xb47, 0xb23, 0xb4d, 0xb1f, 0x2e, 0x20, 0xb2a, 0xb3f, 0xb0f, 0xb30,
0xb47, 0x20, 0xb0f, 0xb2c, 0xb02, 0x20, 0xb2e, 0xb3f, 0xb15, 0xb4d, 0xb5f, 0xb41,
0xb32, 0xb4b, 0xb28, 0xb4d, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb3f, 0xb1f, 0xb15, 0xb47, 0xb30, 0xb4d, 0xb28,
0xb4d, 0x200c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb4b, 0xb28, 0xb3e, 0xb2a, 0xb47,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb4b, 0xb5f, 0xb19, 0xb4d, 0xb17, 0xb5f, 0xb3e,
0xb19, 0xb4d, 0xb17, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb3e, 0xb1c, 0xb3f, 0xb32,
0xb4b, 0xb30, 0xb4d, 0xb21, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb3e, 0xb1c,
0xb3f, 0xb32, 0xb4b, 0xb30, 0xb4d, 0xb21, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb3e, 0xb1c, 0xb3f, 0xb32, 0xb4b, 0xb30, 0xb4d,
0xb21, 0xb3e, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb30, 0xb3f, 0xb5f, 0xb41, 0xb28, 0xb3f, 0xb05, 0xb28, 0xb4d, 0x200c,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb30, 0xb4b, 0xb25, 0xb47, 0xb30, 0xb3e, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb38, 0xb16, 0xb3e, 0xb32, 0xb3f, 0xb28, 0xb4d, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb38, 0xb16, 0xb3e, 0xb32, 0xb3f, 0xb28, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28,
0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb38, 0xb16, 0xb3e, 0xb32,
0xb3f, 0xb28, 0xb4d, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15,
0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb38, 0xb2e, 0xb3e, 0xb30, 0xb3e, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb38, 0xb2e, 0xb3e, 0xb30, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb38, 0xb2e, 0xb3e, 0xb30, 0xb3e, 0x20, 0xb17,
0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb38, 0xb3e,
0xb2e, 0xb4b, 0xb06, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb38, 0xb3e, 0xb2e, 0xb4b, 0xb06,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb38, 0xb3e, 0xb2e,
0xb4b, 0xb06, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb38, 0xb47, 0xb1a, 0xb47, 0xb32, 0xb4d, 0xb32, 0xb47, 0xb38, 0xb4d,
0x200c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb38, 0xb3f, 0xb19, 0xb4d, 0xb17, 0xb3e, 0xb2a,
0xb41, 0xb30, 0xb4d, 0x200c, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb38, 0xb4b, 0xb32, 0xb4b, 0xb2e, 0xb28, 0x20, 0xb26, 0xb4d, 0xb71, 0xb40,
0xb2a, 0xb2a, 0xb41, 0xb1e, 0xb4d, 0xb1c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb26, 0xb15,
0xb4d, 0xb37, 0xb3f, 0xb23, 0x20, 0xb1c, 0xb30, 0xb4d, 0xb1c, 0xb3f, 0xb06, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb38, 0xb41, 0xb30, 0xb3f, 0xb28, 0xb47, 0xb2e, 0xb4d, 0x200c,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb38, 0xb4b, 0xb71, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb24, 0xb3e, 0xb39, 0xb3f, 0xb24, 0xb3f, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb24, 0xb3e,
0xb07, 0xb2a, 0xb47, 0xb07, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb24, 0xb3e, 0xb07, 0xb2a,
0xb47, 0xb07, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb24,
0xb3e, 0xb07, 0xb2a, 0xb47, 0xb07, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb24, 0xb3e, 0xb1c, 0xb3f, 0xb15, 0xb3f, 0xb38,
0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1f, 0xb4b, 0xb15, 0xb47,
0xb32, 0xb3e, 0xb09, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1f, 0xb4b, 0xb19, 0xb4d, 0xb17,
0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1f, 0xb4b, 0xb19, 0xb4d, 0xb17, 0xb3e, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1f, 0xb4b, 0xb19, 0xb4d,
0xb17, 0xb3e, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e,
0xb33, 0xb40, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1a, 0xb42, 0xb15, 0xb4d, 0x200c,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb24, 0xb41, 0xb30, 0xb4d, 0xb15, 0xb2e, 0xb47, 0xb28,
0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb24, 0xb41,
0xb30, 0xb4d, 0xb15, 0xb2e, 0xb47, 0xb28, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb24, 0xb41, 0xb30, 0xb4d, 0xb15, 0xb2e, 0xb47, 0xb28, 0xb3f, 0xb38, 0xb4d, 0xb24,
0xb3e, 0xb28, 0x20, 0xb16, 0xb30, 0xb3e, 0xb26, 0xb3f, 0xb28, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb24, 0xb41, 0xb2d, 0xb3e, 0xb32, 0xb41, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb09,
0xb30, 0xb41, 0xb17, 0xb41, 0xb0f, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb09, 0xb30, 0xb41,
0xb17, 0xb41, 0xb0f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb09, 0xb30, 0xb41, 0xb17, 0xb41, 0xb0f, 0x20, 0xb17, 0xb4d,
0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb09, 0xb1c, 0xb2c, 0xb47, 0xb15, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb09, 0xb1c, 0xb2c, 0xb47, 0xb15, 0xb3f, 0xb38, 0xb4d, 0xb24,
0xb3e, 0xb28, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb09,
0xb1c, 0xb2c, 0xb47, 0xb15, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb17,
0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0xb40, 0xb28, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb2d, 0xb3e, 0xb28, 0xb41, 0xb06, 0xb1f, 0xb41, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb2d, 0xb3e, 0xb28, 0xb41, 0xb06, 0xb1f, 0xb41, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2d, 0xb3e, 0xb28, 0xb41, 0xb06, 0xb1f,
0xb41, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33,
0xb40, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2d, 0xb47, 0xb28, 0xb3f, 0xb1c, 0xb41,
0xb0f, 0xb32, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2d, 0xb4d, 0xb32, 0xb3e, 0xb21,
0xb3f, 0xb2d, 0xb4b, 0xb37, 0xb4d, 0xb1f, 0xb4b, 0xb15, 0xb4d, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2d, 0xb4d, 0xb32, 0xb3e, 0xb21, 0xb3f, 0xb2d, 0xb4b, 0xb37, 0xb4d, 0xb1f,
0xb4b, 0xb15, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb2d, 0xb4d, 0xb32, 0xb3e, 0xb21, 0xb3f, 0xb2d, 0xb4b, 0xb37,
0xb4d, 0xb1f, 0xb4b, 0xb15, 0xb4d, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d,
0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2d, 0xb4b, 0xb32, 0xb17,
0xb4b, 0xb17, 0xb4d, 0xb30, 0xb3e, 0xb21, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2d,
0xb4b, 0xb32, 0xb17, 0xb4b, 0xb17, 0xb4d, 0xb30, 0xb3e, 0xb21, 0xb4d, 0x20, 0xb2e,
0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2d, 0xb4b,
0xb32, 0xb17, 0xb4b, 0xb17, 0xb4d, 0xb30, 0xb3e, 0xb21, 0xb4d, 0x20, 0xb17, 0xb4d,
0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb2d, 0xb4b, 0xb37, 0xb4d, 0xb1f, 0xb4b, 0xb15, 0xb4d, 0x200c, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb71, 0xb47, 0xb15, 0xb4d, 0x200c, 0x20, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb71, 0xb3e, 0xb32, 0xb3f, 0xb38, 0xb4d, 0x200c, 0x20,
0xb0f, 0xb2c, 0xb02, 0x20, 0xb2b, 0xb41, 0xb1f, 0xb41, 0xb28, 0xb3e, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb5f, 0xb3e, 0xb15, 0xb41, 0xb1f, 0xb38, 0xb4d, 0xb15, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb5f, 0xb3e, 0xb15, 0xb41, 0xb1f, 0xb38, 0xb4d, 0xb15, 0x20, 0xb2e,
0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb5f, 0xb3e,
0xb15, 0xb41, 0xb1f, 0xb38, 0xb4d, 0xb15, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37,
0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb5f, 0xb47, 0xb15,
0xb3e, 0xb1f, 0xb47, 0xb30, 0xb3f, 0xb28, 0xb2c, 0xb30, 0xb4d, 0xb17, 0xb4d, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb5f, 0xb47, 0xb15, 0xb3e, 0xb1f, 0xb47, 0xb30, 0xb3f, 0xb28,
0xb2c, 0xb30, 0xb4d, 0xb17, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb5f, 0xb47, 0xb15, 0xb3e, 0xb1f, 0xb47, 0xb30,
0xb3f, 0xb28, 0xb2c, 0xb30, 0xb4d, 0xb17, 0xb4d, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40,
0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb5f, 0xb41,
0xb15, 0xb4b, 0xb28, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x41, 0x66, 0x67, 0x61, 0x61, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x20, 0x47, 0x69,
0x64, 0x64, 0x75, 0x75, 0x67, 0x61, 0x6c, 0x65, 0x65, 0x73, 0x73, 0x61,
0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x61,
0x68, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x41, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x61, 0x20, 0x4b, 0x69, 0x62, 0x62, 0x61, 0x61, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x41, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x61, 0x20, 0x44, 0x68, 0x69, 0x68, 0x61, 0x61, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x41, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x61, 0x20, 0x44, 0x68, 0x69, 0x68, 0x61, 0x61, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61,
0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x20, 0x44, 0x68,
0x69, 0x68, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x41, 0x6c, 0x61, 0x61, 0x73, 0x6b, 0x61, 0x61, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x41, 0x6c, 0x61, 0x61,
0x73, 0x6b, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x47, 0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x41, 0x6c, 0x61, 0x61,
0x73, 0x6b, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6f, 0x6e, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6f, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42,
0x6f, 0x6e, 0x61, 0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6f, 0x6e,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47, 0x69, 0x64,
0x64, 0x75, 0x75, 0x67, 0x61, 0x6c, 0x65, 0x65, 0x73, 0x73, 0x61, 0x61,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74,
0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x47,
0x69, 0x64, 0x64, 0x75, 0x75, 0x67, 0x61, 0x6c, 0x65, 0x65, 0x73, 0x73,
0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47,
0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x47, 0x69, 0x64, 0x64, 0x75, 0x75,
0x67, 0x61, 0x6c, 0x65, 0x65, 0x73, 0x73, 0x61, 0x61, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x61, 0x68, 0x61, 0x61, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x42, 0x61,
0x68, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x47, 0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x42, 0x61, 0x68, 0x61, 0x61,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4d, 0x61, 0x61,
0x77, 0x6f, 0x6e, 0x74, 0x65, 0x65, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x4d, 0x61, 0x61, 0x77, 0x6f, 0x6e,
0x74, 0x65, 0x65, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x47, 0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x4d, 0x61, 0x61, 0x77,
0x6f, 0x6e, 0x74, 0x65, 0x65, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x50, 0x61, 0x61, 0x73, 0x66, 0x69, 0x69, 0x6b, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x50, 0x61,
0x61, 0x73, 0x66, 0x69, 0x69, 0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x47, 0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x50, 0x61,
0x61, 0x73, 0x66, 0x69, 0x69, 0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x41, 0x70, 0x69, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x41, 0x70, 0x69, 0x61, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47, 0x75, 0x79, 0x79, 0x61,
0x61, 0x20, 0x41, 0x70, 0x69, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x41, 0x72, 0x61, 0x62, 0x61, 0x61, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x41, 0x72, 0x61, 0x62,
0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47,
0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x41, 0x72, 0x61, 0x62, 0x61, 0x61,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x41, 0x72, 0x6a,
0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x41, 0x72, 0x6a, 0x65, 0x6e,
0x74, 0x69, 0x69, 0x6e, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x41, 0x72, 0x6a,
0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69,
0x69, 0x6e, 0x61, 0x61, 0x20, 0x44, 0x68, 0x69, 0x68, 0x61, 0x61, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x41, 0x72,
0x6a, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x61, 0x61, 0x20, 0x44, 0x68,
0x69, 0x68, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x41, 0x72, 0x6a, 0x65, 0x6e,
0x74, 0x69, 0x69, 0x6e, 0x61, 0x61, 0x20, 0x44, 0x68, 0x69, 0x68, 0x61,
0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x41, 0x72,
0x6d, 0x61, 0x61, 0x6e, 0x69, 0x79, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x41, 0x72, 0x6d, 0x61, 0x61,
0x6e, 0x69, 0x79, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x41, 0x72, 0x6d, 0x61,
0x61, 0x6e, 0x69, 0x79, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x41, 0x74, 0x69, 0x6c, 0x61, 0x61, 0x6e, 0x74, 0x69,
0x69, 0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49,
0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69,
0x20, 0x41, 0x74, 0x69, 0x6c, 0x61, 0x61, 0x6e, 0x74, 0x69, 0x69, 0x6b,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47, 0x75, 0x79,
0x79, 0x61, 0x61, 0x20, 0x41, 0x74, 0x69, 0x6c, 0x61, 0x61, 0x6e, 0x74,
0x69, 0x69, 0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x41, 0x77, 0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x79,
0x61, 0x61, 0x20, 0x47, 0x69, 0x64, 0x64, 0x75, 0x75, 0x67, 0x61, 0x6c,
0x65, 0x65, 0x73, 0x73, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x69, 0x69, 0x20, 0x41, 0x77, 0x75, 0x73, 0x74, 0x69, 0x72,
0x61, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x61, 0x20, 0x47, 0x69, 0x64, 0x64,
0x75, 0x75, 0x67, 0x61, 0x6c, 0x65, 0x65, 0x73, 0x73, 0x61, 0x61, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47, 0x75, 0x79, 0x79,
0x61, 0x61, 0x20, 0x41, 0x77, 0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x61,
0x6c, 0x69, 0x79, 0x61, 0x61, 0x20, 0x47, 0x69, 0x64, 0x64, 0x75, 0x75,
0x67, 0x61, 0x6c, 0x65, 0x65, 0x73, 0x73, 0x61, 0x61, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x44, 0x68, 0x69, 0x68, 0x61, 0x20,
0x41, 0x77, 0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x79,
0x61, 0x61, 0x20, 0x47, 0x69, 0x64, 0x64, 0x75, 0x75, 0x67, 0x61, 0x6c,
0x65, 0x65, 0x73, 0x73, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x69, 0x69, 0x20, 0x44, 0x68, 0x69, 0x68, 0x61, 0x20, 0x41,
0x77, 0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x79, 0x61,
0x61, 0x20, 0x47, 0x69, 0x64, 0x64, 0x75, 0x75, 0x67, 0x61, 0x6c, 0x65,
0x65, 0x73, 0x73, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x47, 0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x44, 0x68, 0x69,
0x68, 0x61, 0x20, 0x41, 0x77, 0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x61,
0x6c, 0x69, 0x79, 0x61, 0x61, 0x20, 0x47, 0x69, 0x64, 0x64, 0x75, 0x75,
0x67, 0x61, 0x6c, 0x65, 0x65, 0x73, 0x73, 0x61, 0x61, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x41, 0x77, 0x75, 0x73, 0x74, 0x69,
0x72, 0x61, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x61, 0x20, 0x42, 0x61, 0x68,
0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49,
0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69,
0x20, 0x41, 0x77, 0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x61, 0x6c, 0x69,
0x79, 0x61, 0x61, 0x20, 0x42, 0x61, 0x68, 0x61, 0x61, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47, 0x75, 0x79, 0x79, 0x61, 0x61,
0x20, 0x41, 0x77, 0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x61, 0x6c, 0x69,
0x79, 0x61, 0x61, 0x20, 0x42, 0x61, 0x68, 0x61, 0x61, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x41, 0x77, 0x75, 0x73, 0x74, 0x69,
0x72, 0x61, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x61, 0x20, 0x44, 0x68, 0x69,
0x68, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74,
0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x41,
0x77, 0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x79, 0x61,
0x61, 0x20, 0x44, 0x68, 0x69, 0x68, 0x61, 0x61, 0x44, 0x68, 0x69, 0x68,
0x61, 0x20, 0x41, 0x77, 0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x61, 0x6c,
0x69, 0x79, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x47, 0x75, 0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x41, 0x77, 0x75,
0x73, 0x74, 0x69, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x61, 0x20,
0x44, 0x68, 0x69, 0x68, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x61, 0x6a, 0x69,
0x79, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x69, 0x69, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x61, 0x6a, 0x69,
0x79, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62,
0x61, 0x61, 0x6a, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x41, 0x7a, 0x65, 0x65, 0x72, 0x6f, 0x6f,
0x73, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73,
0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20,
0x41, 0x7a, 0x65, 0x65, 0x72, 0x6f, 0x6f, 0x73, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x41,
0x7a, 0x65, 0x65, 0x72, 0x6f, 0x6f, 0x73, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x42, 0x61, 0x61, 0x6e, 0x67, 0x69, 0x6c, 0x61,
0x61, 0x64, 0x69, 0x73, 0x68, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x69, 0x69, 0x20, 0x42, 0x61, 0x61, 0x6e, 0x67, 0x69, 0x6c, 0x61,
0x61, 0x64, 0x69, 0x73, 0x68, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x42, 0x61, 0x61, 0x6e,
0x67, 0x69, 0x6c, 0x61, 0x61, 0x64, 0x69, 0x73, 0x68, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x69, 0x68, 0x75, 0x74, 0x61,
0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42,
0x6f, 0x6c, 0x69, 0x69, 0x76, 0x69, 0x79, 0x61, 0x61, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x69, 0x72, 0x61, 0x61, 0x7a,
0x69, 0x6c, 0x69, 0x79, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x69, 0x69, 0x20, 0x42, 0x69, 0x72, 0x61, 0x61, 0x7a, 0x69,
0x6c, 0x69, 0x79, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x42, 0x69, 0x72, 0x61,
0x61, 0x7a, 0x69, 0x6c, 0x69, 0x79, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x72, 0x75, 0x75, 0x6e, 0x65, 0x65,
0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x61, 0x6c, 0x61, 0x61, 0x6d, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4b, 0x65, 0x65, 0x70,
0x20, 0x56, 0x65, 0x65, 0x72, 0x64, 0x65, 0x65, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x4b, 0x65, 0x65, 0x70, 0x20,
0x56, 0x65, 0x65, 0x72, 0x64, 0x65, 0x65, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x4b, 0x65,
0x65, 0x70, 0x20, 0x56, 0x65, 0x65, 0x72, 0x64, 0x65, 0x65, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x4b, 0x61, 0x6d,
0x6f, 0x72, 0x6f, 0x6f, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47,
0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x43, 0x68,
0x69, 0x69, 0x6c, 0x69, 0x69, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x69, 0x69, 0x20, 0x43, 0x68, 0x69, 0x69, 0x6c, 0x69, 0x69, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61,
0x61, 0x20, 0x43, 0x68, 0x69, 0x69, 0x6c, 0x69, 0x69, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x43, 0x68, 0x61, 0x61, 0x79, 0x69,
0x6e, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69,
0x69, 0x20, 0x43, 0x68, 0x61, 0x61, 0x79, 0x69, 0x6e, 0x61, 0x61, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47, 0x75, 0x79, 0x79,
0x61, 0x61, 0x20, 0x43, 0x68, 0x61, 0x61, 0x79, 0x69, 0x6e, 0x61, 0x61,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4f, 0x64, 0x6f,
0x6c, 0x61, 0x20, 0x4b, 0x69, 0x72, 0x69, 0x73, 0x6d, 0x61, 0x61, 0x73,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4f, 0x64, 0x6f,
0x6c, 0x6f, 0x6f, 0x74, 0x61, 0x20, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4b, 0x6f, 0x6c, 0x6f,
0x6d, 0x62, 0x69, 0x79, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x69, 0x69, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69,
0x79, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x79, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x4f, 0x64, 0x6f, 0x6c, 0x6f, 0x6f, 0x74, 0x61, 0x20, 0x4b, 0x75,
0x75, 0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49,
0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69,
0x20, 0x4f, 0x64, 0x6f, 0x6c, 0x6f, 0x6f, 0x74, 0x61, 0x20, 0x4b, 0x75,
0x75, 0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42,
0x6f, 0x6e, 0x61, 0x20, 0x57, 0x61, 0x6c, 0x61, 0x6b, 0x6b, 0x61, 0x61,
0x20, 0x4f, 0x64, 0x6f, 0x6c, 0x6f, 0x6f, 0x74, 0x61, 0x20, 0x4b, 0x75,
0x75, 0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4b,
0x75, 0x75, 0x62, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x69, 0x69, 0x20, 0x4b, 0x75, 0x75, 0x62, 0x61, 0x61, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47, 0x75, 0x79, 0x79, 0x61,
0x61, 0x20, 0x4b, 0x75, 0x75, 0x62, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x44, 0x61, 0x61, 0x76, 0x69, 0x69, 0x73,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x44, 0x75, 0x6d,
0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c,
0x65, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x54, 0x69,
0x69, 0x6d, 0x6f, 0x6f, 0x72, 0x20, 0x42, 0x61, 0x68, 0x61, 0x61, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4f, 0x64, 0x6f, 0x6c,
0x61, 0x20, 0x42, 0x61, 0x68, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x4f, 0x64, 0x6f, 0x6c, 0x61, 0x20,
0x42, 0x61, 0x68, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x4f, 0x64, 0x6f, 0x6c,
0x61, 0x20, 0x42, 0x61, 0x68, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x49, 0x6b, 0x6b, 0x75, 0x77, 0x61, 0x61, 0x64,
0x6f, 0x6f, 0x72, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x41, 0x77, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x70, 0x61, 0x61, 0x20, 0x47,
0x69, 0x64, 0x64, 0x75, 0x75, 0x67, 0x61, 0x6c, 0x65, 0x65, 0x73, 0x73,
0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49,
0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69,
0x20, 0x41, 0x77, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x70, 0x61, 0x61, 0x20,
0x47, 0x69, 0x64, 0x64, 0x75, 0x75, 0x67, 0x61, 0x6c, 0x65, 0x65, 0x73,
0x73, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x41, 0x77, 0x75, 0x72, 0x6f, 0x6f,
0x70, 0x70, 0x61, 0x61, 0x20, 0x47, 0x69, 0x64, 0x64, 0x75, 0x75, 0x67,
0x61, 0x6c, 0x65, 0x65, 0x73, 0x73, 0x61, 0x61, 0x53, 0x61, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x41, 0x77, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x70,
0x61, 0x61, 0x20, 0x42, 0x61, 0x68, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x41, 0x77, 0x75, 0x72, 0x6f,
0x6f, 0x70, 0x70, 0x61, 0x61, 0x20, 0x42, 0x61, 0x68, 0x61, 0x61, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61,
0x61, 0x20, 0x41, 0x77, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x70, 0x61, 0x61,
0x20, 0x42, 0x61, 0x68, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x41, 0x77, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x70, 0x61,
0x61, 0x20, 0x42, 0x61, 0x68, 0x61, 0x61, 0x20, 0x44, 0x61, 0x62, 0x61,
0x6c, 0x61, 0x74, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x41, 0x77, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x70, 0x61, 0x61,
0x20, 0x44, 0x68, 0x69, 0x68, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x41, 0x77, 0x75, 0x72, 0x6f, 0x6f,
0x70, 0x70, 0x61, 0x61, 0x20, 0x44, 0x68, 0x69, 0x68, 0x61, 0x61, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61,
0x61, 0x20, 0x41, 0x77, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x70, 0x61, 0x61,
0x20, 0x44, 0x68, 0x69, 0x68, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x4f, 0x64, 0x6f, 0x6c, 0x6f, 0x6f, 0x74, 0x61,
0x20, 0x46, 0x61, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x61, 0x6e, 0x64, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x4f, 0x64,
0x6f, 0x6c, 0x6f, 0x6f, 0x74, 0x61, 0x20, 0x46, 0x61, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x61, 0x6e, 0x64, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x4f, 0x64, 0x6f, 0x6c,
0x6f, 0x6f, 0x74, 0x61, 0x20, 0x46, 0x61, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x61, 0x6e, 0x64, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x46, 0x69, 0x69, 0x6a, 0x69, 0x69, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x69, 0x69, 0x20, 0x46, 0x69, 0x69, 0x6a, 0x69, 0x69, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61,
0x61, 0x20, 0x46, 0x69, 0x69, 0x6a, 0x69, 0x69, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x46, 0x69, 0x72, 0x65, 0x65, 0x6e, 0x63,
0x68, 0x20, 0x47, 0x75, 0x79, 0x69, 0x6e, 0x61, 0x61, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x46, 0x69, 0x72, 0x61, 0x61, 0x6e,
0x73, 0x20, 0x4b, 0x69, 0x62, 0x62, 0x61, 0x61, 0x20, 0x66, 0x69, 0x20,
0x41, 0x6e, 0x74, 0x61, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x6b, 0x61, 0x61,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47, 0x61, 0x6c,
0x61, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x6f, 0x73, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x47, 0x61, 0x61, 0x6d, 0x62, 0x69, 0x79,
0x65, 0x65, 0x72, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x4a, 0x6f, 0x6f, 0x72, 0x6a, 0x69, 0x79, 0x61, 0x61, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x4a, 0x6f, 0x6f, 0x72,
0x6a, 0x69, 0x79, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x4a, 0x6f, 0x6f, 0x72,
0x6a, 0x69, 0x79, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x4f, 0x64, 0x6f, 0x6c, 0x6f, 0x6f, 0x74, 0x61, 0x20, 0x47,
0x69, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x47, 0x69, 0x72, 0x69, 0x69, 0x6e, 0x77, 0x69,
0x69, 0x63, 0x68, 0x20, 0x47, 0x69, 0x64, 0x64, 0x75, 0x75, 0x67, 0x61,
0x6c, 0x65, 0x65, 0x73, 0x73, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x47, 0x69, 0x72, 0x69, 0x69, 0x6e, 0x6c, 0x61,
0x61, 0x6e, 0x64, 0x20, 0x42, 0x61, 0x68, 0x61, 0x61, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x47, 0x69, 0x72, 0x69,
0x69, 0x6e, 0x6c, 0x61, 0x61, 0x6e, 0x64, 0x20, 0x42, 0x61, 0x68, 0x61,
0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f,
0x6e, 0x61, 0x61, 0x20, 0x47, 0x69, 0x72, 0x69, 0x69, 0x6e, 0x6c, 0x61,
0x61, 0x6e, 0x64, 0x20, 0x42, 0x61, 0x68, 0x61, 0x61, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47, 0x69, 0x72, 0x69, 0x69, 0x6e,
0x6c, 0x61, 0x61, 0x6e, 0x64, 0x20, 0x44, 0x68, 0x69, 0x68, 0x61, 0x61,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74,
0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x47,
0x69, 0x72, 0x69, 0x69, 0x6e, 0x6c, 0x61, 0x61, 0x6e, 0x64, 0x20, 0x44,
0x68, 0x69, 0x68, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x47, 0x69, 0x72, 0x69,
0x69, 0x6e, 0x6c, 0x61, 0x61, 0x6e, 0x64, 0x20, 0x44, 0x68, 0x69, 0x68,
0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49,
0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69,
0x20, 0x47, 0x75, 0x77, 0x61, 0x61, 0x6d, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x47, 0x61, 0x61, 0x6c, 0x66, 0x69,
0x69, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47, 0x75,
0x79, 0x61, 0x61, 0x6e, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x48, 0x61, 0x61, 0x77, 0x61, 0x79, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x77, 0x75, 0x74, 0x69, 0x79, 0x61, 0x6e, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x48, 0x61, 0x61, 0x77,
0x61, 0x79, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x77, 0x75, 0x74, 0x69, 0x79,
0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47,
0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x48, 0x61, 0x61, 0x77, 0x61, 0x79,
0x69, 0x2d, 0x41, 0x6c, 0x65, 0x77, 0x75, 0x74, 0x69, 0x79, 0x61, 0x6e,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x48, 0x6f, 0x6f,
0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6f, 0x6e, 0x67, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x48, 0x6f, 0x6f, 0x6e, 0x67,
0x20, 0x4b, 0x6f, 0x6f, 0x6e, 0x67, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x48, 0x6f, 0x6f,
0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6f, 0x6e, 0x67, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x48, 0x6f, 0x6f, 0x76, 0x64, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x48, 0x6f, 0x6f,
0x76, 0x64, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42,
0x6f, 0x6e, 0x61, 0x61, 0x20, 0x48, 0x6f, 0x6f, 0x76, 0x64, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x48, 0x69, 0x6e,
0x64, 0x69, 0x69, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x47, 0x61, 0x6c, 0x61, 0x61, 0x6e, 0x61, 0x20, 0x48, 0x69, 0x6e, 0x64,
0x69, 0x69, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49,
0x6e, 0x64, 0x6f, 0x6f, 0x43, 0x68, 0x61, 0x61, 0x79, 0x69, 0x6e, 0x61,
0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x6e,
0x64, 0x6f, 0x6f, 0x6e, 0x65, 0x65, 0x73, 0x68, 0x69, 0x79, 0x61, 0x61,
0x20, 0x47, 0x69, 0x64, 0x64, 0x75, 0x75, 0x67, 0x61, 0x6c, 0x65, 0x65,
0x73, 0x73, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6f, 0x6e, 0x65, 0x73, 0x68, 0x69, 0x79,
0x61, 0x61, 0x20, 0x42, 0x61, 0x68, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6f, 0x6e, 0x65,
0x65, 0x73, 0x68, 0x69, 0x79, 0x61, 0x61, 0x20, 0x44, 0x68, 0x69, 0x68,
0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49,
0x72, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x69, 0x69, 0x20, 0x49, 0x72, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x47, 0x75, 0x79, 0x79, 0x61, 0x61, 0x20,
0x49, 0x72, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73,
0x72, 0x61, 0x61, 0x2019, 0x65, 0x65, 0x6c, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x49, 0x73, 0x72, 0x61, 0x61, 0x2019,
0x65, 0x65, 0x6c, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x47, 0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x49, 0x73, 0x72, 0x61, 0x61,
0x2019, 0x65, 0x65, 0x6c, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x4a, 0x61, 0x61, 0x70, 0x70, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x4a, 0x61, 0x61, 0x70,
0x70, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x47, 0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x4a, 0x61, 0x61, 0x70,
0x70, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x4b, 0x61, 0x61, 0x7a, 0x61, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4b,
0x61, 0x61, 0x7a, 0x61, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e,
0x20, 0x42, 0x61, 0x68, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x4b, 0x61, 0x61, 0x7a, 0x61, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x61, 0x6e, 0x20, 0x44, 0x68, 0x69, 0x68, 0x61, 0x61, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4b, 0x6f, 0x6f, 0x72,
0x69, 0x79, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x69, 0x69, 0x20, 0x4b, 0x6f, 0x6f, 0x72, 0x69, 0x79, 0x61, 0x61, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47, 0x75, 0x79, 0x79,
0x61, 0x61, 0x20, 0x4b, 0x6f, 0x6f, 0x72, 0x69, 0x79, 0x61, 0x61, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4b, 0x6f, 0x6f, 0x73,
0x72, 0x65, 0x79, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73,
0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73,
0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61,
0x61, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73,
0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4b, 0x69,
0x79, 0x69, 0x72, 0x69, 0x67, 0x69, 0x79, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4c, 0x61, 0x61,
0x6e, 0x6b, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x4f, 0x64, 0x6f, 0x6c, 0x6f, 0x6f, 0x74, 0x61, 0x20, 0x4c, 0x69,
0x6e, 0x65, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x47, 0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4d, 0x61,
0x61, 0x6c, 0x65, 0x65, 0x73, 0x68, 0x69, 0x79, 0x61, 0x61, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4d, 0x61, 0x61, 0x6c, 0x64,
0x69, 0x69, 0x76, 0x73, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4f, 0x64, 0x6f, 0x6c, 0x6f,
0x6f, 0x74, 0x61, 0x20, 0x4d, 0x61, 0x61, 0x72, 0x73, 0x68, 0x61, 0x61,
0x6c, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4d, 0x6f,
0x6f, 0x72, 0x69, 0x69, 0x73, 0x68, 0x69, 0x79, 0x65, 0x65, 0x73, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x4d, 0x6f,
0x6f, 0x72, 0x69, 0x73, 0x68, 0x69, 0x79, 0x65, 0x65, 0x73, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61,
0x20, 0x4d, 0x6f, 0x6f, 0x72, 0x69, 0x73, 0x68, 0x69, 0x79, 0x65, 0x65,
0x73, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4d, 0x61,
0x77, 0x73, 0x6f, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x50, 0x61, 0x61, 0x73, 0x66, 0x69, 0x69, 0x6b, 0x20, 0x4d, 0x65,
0x6b, 0x73, 0x69, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x50, 0x61, 0x61, 0x73, 0x66,
0x69, 0x69, 0x6b, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x69, 0x6b, 0x61,
0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47,
0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x50, 0x61, 0x61, 0x73, 0x66, 0x69,
0x69, 0x6b, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x69, 0x6b, 0x61, 0x61,
0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x55, 0x6c,
0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x55, 0x6c, 0x61, 0x61,
0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x55, 0x6c,
0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4d, 0x6f, 0x6f, 0x73, 0x6b, 0x6f,
0x6f, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73,
0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20,
0x4d, 0x6f, 0x6f, 0x73, 0x6b, 0x6f, 0x6f, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x4d, 0x6f,
0x6f, 0x73, 0x6b, 0x6f, 0x6f, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x4d, 0x61, 0x61, 0x79, 0x69, 0x6e, 0x61, 0x61, 0x6d, 0x61,
0x61, 0x72, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4e,
0x61, 0x61, 0x77, 0x75, 0x72, 0x75, 0x75, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x4e, 0x65, 0x65, 0x70, 0x70, 0x61, 0x61, 0x6c,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4b, 0x61, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6f, 0x6e, 0x69, 0x79, 0x61, 0x61, 0x20, 0x48,
0x61, 0x61, 0x72, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x69, 0x69, 0x20, 0x4b, 0x61, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6f,
0x6e, 0x69, 0x79, 0x61, 0x61, 0x20, 0x48, 0x61, 0x61, 0x72, 0x61, 0x61,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e,
0x61, 0x61, 0x20, 0x4b, 0x61, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6f, 0x6e,
0x69, 0x79, 0x61, 0x61, 0x20, 0x48, 0x61, 0x61, 0x72, 0x61, 0x61, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4e, 0x65, 0x77, 0x20,
0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65,
0x61, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x47, 0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x4e, 0x65, 0x77,
0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x4e, 0x65, 0x77, 0x75, 0x66, 0x61, 0x61,
0x77, 0x6f, 0x6e, 0x64, 0x6c, 0x61, 0x61, 0x6e, 0x64, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x4e, 0x65, 0x77, 0x75,
0x66, 0x61, 0x61, 0x77, 0x6f, 0x6e, 0x64, 0x6c, 0x61, 0x61, 0x6e, 0x64,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47, 0x75, 0x79,
0x79, 0x61, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x75, 0x66, 0x61, 0x61, 0x77,
0x6f, 0x6e, 0x64, 0x6c, 0x61, 0x61, 0x6e, 0x64, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47, 0x75, 0x79, 0x79, 0x61, 0x61,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f,
0x6e, 0x61, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4f, 0x64, 0x6f, 0x6c,
0x6f, 0x6f, 0x74, 0x61, 0x20, 0x4d, 0x61, 0x61, 0x72, 0x69, 0x79, 0x61,
0x61, 0x6e, 0x61, 0x61, 0x20, 0x4b, 0x61, 0x61, 0x62, 0x61, 0x61, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x69, 0x73, 0x6b, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x69, 0x73, 0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x69, 0x73, 0x6b, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x4f, 0x6d,
0x73, 0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42,
0x6f, 0x6e, 0x61, 0x61, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x50, 0x61, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x69, 0x69, 0x20, 0x50, 0x61, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61,
0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f,
0x6e, 0x61, 0x61, 0x20, 0x50, 0x61, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x50,
0x61, 0x6c, 0x61, 0x77, 0x75, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x50, 0x61, 0x61, 0x70, 0x75, 0x77, 0x61, 0x61, 0x20, 0x47,
0x69, 0x69, 0x6e, 0x69, 0x69, 0x20, 0x48, 0x61, 0x61, 0x72, 0x61, 0x61,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x50, 0x61, 0x61,
0x72, 0x61, 0x61, 0x67, 0x75, 0x77, 0x61, 0x61, 0x79, 0x69, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x50, 0x61, 0x61,
0x72, 0x61, 0x61, 0x67, 0x75, 0x77, 0x61, 0x61, 0x79, 0x69, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61,
0x20, 0x50, 0x61, 0x61, 0x72, 0x61, 0x61, 0x67, 0x75, 0x77, 0x61, 0x61,
0x79, 0x69, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x50,
0x65, 0x65, 0x72, 0x75, 0x75, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x69, 0x69, 0x20, 0x50, 0x65, 0x65, 0x72, 0x75, 0x75, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61,
0x20, 0x50, 0x65, 0x65, 0x72, 0x75, 0x75, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x69,
0x6e, 0x73, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49,
0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69,
0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x69, 0x6e, 0x73, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61,
0x61, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x69, 0x6e, 0x73,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4f, 0x64, 0x6f,
0x6c, 0x6f, 0x6f, 0x74, 0x61, 0x20, 0x46, 0x6f, 0x6f, 0x6e, 0x65, 0x65,
0x6b, 0x73, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x51,
0x6c, 0x2e, 0x20, 0x50, 0x69, 0x79, 0x65, 0x65, 0x72, 0x65, 0x65, 0x20,
0x66, 0x69, 0x20, 0x4d, 0x69, 0x6b, 0x75, 0x79, 0x65, 0x6c, 0x6f, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x51, 0x6c,
0x2e, 0x20, 0x50, 0x69, 0x79, 0x65, 0x65, 0x72, 0x65, 0x65, 0x20, 0x66,
0x69, 0x20, 0x4d, 0x69, 0x6b, 0x75, 0x79, 0x65, 0x6c, 0x6f, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47, 0x75, 0x79, 0x79, 0x61,
0x61, 0x20, 0x51, 0x6c, 0x2e, 0x20, 0x50, 0x69, 0x79, 0x65, 0x65, 0x72,
0x65, 0x65, 0x20, 0x66, 0x69, 0x20, 0x4d, 0x69, 0x6b, 0x75, 0x79, 0x65,
0x6c, 0x6f, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x50,
0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x50, 0x69, 0x79, 0x6f, 0x6f,
0x6e, 0x67, 0x79, 0x61, 0x61, 0x6e, 0x67, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x52, 0x69, 0x69, 0x79, 0x75, 0x75, 0x6e, 0x69,
0x79, 0x65, 0x65, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73,
0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20,
0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x53,
0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x53, 0x61, 0x61, 0x6d, 0x6f, 0x77, 0x61, 0x61,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74,
0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x53,
0x61, 0x61, 0x6d, 0x6f, 0x77, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x47, 0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x53,
0x61, 0x61, 0x6d, 0x6f, 0x77, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x53, 0x69, 0x69, 0x73, 0x68, 0x65, 0x65, 0x6c,
0x73, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73,
0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20,
0x53, 0x69, 0x6e, 0x67, 0x61, 0x61, 0x70, 0x6f, 0x6f, 0x72, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4f, 0x64, 0x6f, 0x6c, 0x6f,
0x6f, 0x74, 0x61, 0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6f, 0x6e,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4a, 0x6f, 0x6f,
0x72, 0x6a, 0x69, 0x79, 0x61, 0x61, 0x20, 0x4b, 0x69, 0x62, 0x62, 0x61,
0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x53, 0x75,
0x72, 0x69, 0x6e, 0x61, 0x61, 0x6d, 0x65, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x54, 0x61, 0x79,
0x69, 0x70, 0x65, 0x79, 0x69, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x69, 0x69, 0x20, 0x54, 0x61, 0x79, 0x69, 0x70, 0x65, 0x79, 0x69,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47, 0x75, 0x79,
0x79, 0x61, 0x61, 0x20, 0x54, 0x61, 0x79, 0x69, 0x70, 0x65, 0x79, 0x69,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x54, 0x61, 0x6a,
0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x54, 0x61, 0x6b, 0x65, 0x6c, 0x61, 0x77,
0x75, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69,
0x69, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x43, 0x68, 0x75, 0x75, 0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x65, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x69, 0x69, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x65, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x54, 0x75, 0x72, 0x6b,
0x65, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x54, 0x75, 0x76, 0x61, 0x6c,
0x75, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x59, 0x75,
0x72, 0x61, 0x61, 0x67, 0x61, 0x61, 0x79, 0x69, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x59, 0x75, 0x72, 0x61, 0x61,
0x67, 0x61, 0x61, 0x79, 0x69, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x59, 0x75, 0x72, 0x61,
0x61, 0x67, 0x61, 0x61, 0x79, 0x69, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x65, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69,
0x69, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42,
0x6f, 0x6e, 0x61, 0x61, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x65, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x77, 0x61, 0x74, 0x75, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x56, 0x61, 0x6e,
0x75, 0x77, 0x61, 0x74, 0x75, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x56, 0x61, 0x6e, 0x75,
0x77, 0x61, 0x74, 0x75, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x56, 0x65, 0x65, 0x6e, 0x7a, 0x75, 0x77, 0x65, 0x65, 0x6c, 0x61,
0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69,
0x69, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61,
0x61, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x56, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4f,
0x64, 0x6f, 0x6c, 0x61, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73,
0x20, 0x66, 0x69, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49,
0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20,
0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69,
0x69, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x59, 0x75, 0x75, 0x6b, 0x6f, 0x6f, 0x6e, 0x410,
0x441, 0x442, 0x4d5, 0x443, 0x43a, 0x43a, 0x430, 0x433, 0x20, 0x415, 0x432, 0x440,
0x43e, 0x43f, 0x4d5, 0x439, 0x430, 0x433, 0x20, 0x440, 0x4d5, 0x441, 0x442, 0x4d5,
0x433, 0x410, 0x441, 0x442, 0x4d5, 0x443, 0x43a, 0x43a, 0x430, 0x433, 0x20, 0x415,
0x432, 0x440, 0x43e, 0x43f, 0x4d5, 0x439, 0x430, 0x433, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43e, 0x43d, 0x20, 0x440, 0x4d5, 0x441, 0x442,
0x4d5, 0x433, 0x410, 0x441, 0x442, 0x4d5, 0x443, 0x43a, 0x43a, 0x430, 0x433, 0x20,
0x415, 0x432, 0x440, 0x43e, 0x43f, 0x4d5, 0x439, 0x430, 0x433, 0x20, 0x441, 0x4d5,
0x440, 0x434, 0x44b, 0x433, 0x43e, 0x43d, 0x20, 0x440, 0x4d5, 0x441, 0x442, 0x4d5,
0x433, 0x421, 0x43a, 0x4d5, 0x441, 0x4d5, 0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e,
0x43f, 0x4d5, 0x439, 0x430, 0x433, 0x20, 0x440, 0x4d5, 0x441, 0x442, 0x4d5, 0x433,
0x421, 0x43a, 0x4d5, 0x441, 0x4d5, 0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f,
0x4d5, 0x439, 0x430, 0x433, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43e, 0x43d, 0x20, 0x440, 0x4d5, 0x441, 0x442, 0x4d5, 0x433, 0x421, 0x43a,
0x4d5, 0x441, 0x4d5, 0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x4d5, 0x439,
0x430, 0x433, 0x20, 0x441, 0x4d5, 0x440, 0x434, 0x44b, 0x433, 0x43e, 0x43d, 0x20,
0x440, 0x4d5, 0x441, 0x442, 0x4d5, 0x433, 0x41d, 0x44b, 0x433, 0x44a, 0x443, 0x44b,
0x43b, 0x4d5, 0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x4d5, 0x439, 0x430,
0x433, 0x20, 0x440, 0x4d5, 0x441, 0x442, 0x4d5, 0x433, 0x41d, 0x44b, 0x433, 0x44a,
0x443, 0x44b, 0x43b, 0x4d5, 0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x4d5,
0x439, 0x430, 0x433, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43e, 0x43d, 0x20, 0x440, 0x4d5, 0x441, 0x442, 0x4d5, 0x433, 0x41d, 0x44b, 0x433,
0x44a, 0x443, 0x44b, 0x43b, 0x4d5, 0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f,
0x4d5, 0x439, 0x430, 0x433, 0x20, 0x441, 0x4d5, 0x440, 0x434, 0x44b, 0x433, 0x43e,
0x43d, 0x20, 0x440, 0x4d5, 0x441, 0x442, 0x4d5, 0x433, 0x413, 0x443, 0x44b, 0x440,
0x434, 0x437, 0x44b, 0x441, 0x442, 0x43e, 0x43d, 0x44b, 0x20, 0x440, 0x4d5, 0x441,
0x442, 0x4d5, 0x433, 0x413, 0x443, 0x44b, 0x440, 0x434, 0x437, 0x44b, 0x441, 0x442,
0x43e, 0x43d, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43e, 0x43d, 0x20, 0x440, 0x4d5, 0x441, 0x442, 0x4d5, 0x433, 0x413, 0x443, 0x44b,
0x440, 0x434, 0x437, 0x44b, 0x441, 0x442, 0x43e, 0x43d, 0x44b, 0x20, 0x441, 0x4d5,
0x440, 0x434, 0x44b, 0x433, 0x43e, 0x43d, 0x20, 0x440, 0x4d5, 0x441, 0x442, 0x4d5,
0x433, 0x413, 0x440, 0x438, 0x43d, 0x432, 0x438, 0x447, 0x44b, 0x20, 0x440, 0x4d5,
0x441, 0x442, 0x4d5, 0x43c, 0x431, 0x438, 0x441, 0x20, 0x440, 0x4d5, 0x441, 0x442,
0x4d5, 0x433, 0x41c, 0x4d5, 0x441, 0x43a, 0x443, 0x44b, 0x439, 0x44b, 0x20, 0x440,
0x4d5, 0x441, 0x442, 0x4d5, 0x433, 0x41c, 0x4d5, 0x441, 0x43a, 0x443, 0x44b, 0x439,
0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43e, 0x43d,
0x20, 0x440, 0x4d5, 0x441, 0x442, 0x4d5, 0x433, 0x41c, 0x4d5, 0x441, 0x43a, 0x443,
0x44b, 0x439, 0x44b, 0x20, 0x441, 0x4d5, 0x440, 0x434, 0x44b, 0x433, 0x43e, 0x43d,
0x20, 0x440, 0x4d5, 0x441, 0x442, 0x4d5, 0x433, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x4f, 0x73, 0x74, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x41, 0x7a, 0xf3, 0x72, 0x65,
0x73, 0x4f, 0x72, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20,
0x64, 0x69, 0x20, 0x41, 0x7a, 0xf3, 0x72, 0x65, 0x73, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x69, 0x20, 0x41, 0x7a, 0xf3, 0x72, 0x65, 0x73, 0x20, 0x54,
0x65, 0x6d, 0x70, 0x6f, 0x72, 0x61, 0x64, 0x61, 0x20, 0x64, 0x69, 0x20,
0x5a, 0x6f, 0x6d, 0x65, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x20, 0x64, 0x69, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x69, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x53,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x54, 0x65, 0x6d, 0x70, 0x6f,
0x72, 0x61, 0x64, 0x61, 0x20, 0x64, 0x69, 0x20, 0x5a, 0x6f, 0x6d, 0x65,
0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4f, 0x73, 0x74, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x69, 0x20, 0x4f, 0x73, 0x74, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x4f, 0x73, 0x74, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x54, 0x65, 0x6d, 0x70, 0x6f, 0x72, 0x61, 0x64, 0x61, 0x20, 0x64, 0x69,
0x20, 0x5a, 0x6f, 0x6d, 0x65, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x57, 0xe8, 0x73, 0x74, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64,
0x69, 0x20, 0x57, 0xe8, 0x73, 0x74, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x57, 0xe8, 0x73, 0x74,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x54, 0x65, 0x6d, 0x70,
0x6f, 0x72, 0x61, 0x64, 0x61, 0x20, 0x64, 0x69, 0x20, 0x5a, 0x6f, 0x6d,
0x65, 0x72, 0x627, 0x641, 0x63a, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20,
0x648, 0x62e, 0x62a, 0x645, 0x646, 0x681, 0x646, 0x6cc, 0x20, 0x627, 0x641, 0x631,
0x64a, 0x642, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x62e, 0x62a, 0x64a, 0x681, 0x20,
0x627, 0x641, 0x631, 0x64a, 0x642, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x62c, 0x646,
0x648, 0x628, 0x64a, 0x20, 0x627, 0x641, 0x631, 0x64a, 0x642, 0x627, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x644, 0x648, 0x6d0,
0x62f, 0x64a, 0x681, 0x20, 0x627, 0x641, 0x631, 0x64a, 0x642, 0x627, 0x20, 0x648,
0x62e, 0x62a, 0x644, 0x648, 0x6cc, 0x62f, 0x6cc, 0x681, 0x20, 0x627, 0x641, 0x631,
0x6cc, 0x642, 0x627, 0x6cc, 0x64a, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x627, 0x641, 0x631, 0x6cc, 0x642, 0x627,
0x20, 0x627, 0x641, 0x631, 0x6cc, 0x642, 0x627, 0x20, 0x644, 0x648, 0x6cc, 0x62f,
0x6cc, 0x681, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x644, 0x627, 0x633, 0x6a9, 0x627,
0x20, 0x648, 0x62e, 0x62a, 0x627, 0x644, 0x627, 0x633, 0x6a9, 0x627, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x644, 0x627,
0x633, 0x6a9, 0x627, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x631,
0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x6cc, 0x645,
0x6cc, 0x632, 0x648, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x6cc, 0x645, 0x6cc,
0x632, 0x648, 0x646, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648,
0x62e, 0x62a, 0x627, 0x6cc, 0x645, 0x6cc, 0x632, 0x648, 0x646, 0x20, 0x627, 0x648,
0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x631, 0x6a9, 0x632, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x645, 0x631, 0x6a9, 0x632, 0x64a, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x631, 0x6a9, 0x632, 0x64a,
0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x631, 0x648, 0x69a, 0x627,
0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x62e, 0x62a, 0x6cc, 0x681, 0x20, 0x648,
0x62e, 0x62a, 0x62e, 0x62a, 0x64a, 0x681, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62e, 0x62a, 0x64a, 0x681, 0x20, 0x62f, 0x20,
0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x648, 0x62e, 0x62a,
0x62f, 0x20, 0x63a, 0x631, 0x647, 0x20, 0x62f, 0x20, 0x648, 0x62e, 0x62a, 0x62f,
0x20, 0x63a, 0x631, 0x647, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x63a, 0x631, 0x647, 0x20, 0x62f, 0x20, 0x631,
0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x648, 0x62e, 0x62a, 0x67e,
0x6cc, 0x633, 0x641, 0x6a9, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x67e, 0x6cc,
0x633, 0x641, 0x6a9, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x62e, 0x62a, 0x67e, 0x6cc, 0x633, 0x641, 0x6a9, 0x20, 0x62f, 0x20, 0x631, 0x6bc,
0x627, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x67e,
0x6cc, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x67e, 0x6cc, 0x627, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x67e, 0x64a,
0x627, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x631, 0x648, 0x69a,
0x627, 0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x639, 0x631, 0x628, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x639, 0x631, 0x628, 0x64a, 0x20, 0x645, 0x639, 0x6cc, 0x627,
0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x639, 0x631, 0x628, 0x64a, 0x20, 0x62f,
0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x631, 0x648, 0x69a, 0x627, 0x646, 0x647,
0x20, 0x648, 0x62e, 0x62a, 0x627, 0x631, 0x62c, 0x646, 0x67c, 0x627, 0x6cc, 0x646,
0x20, 0x648, 0x62e, 0x62a, 0x627, 0x631, 0x62c, 0x646, 0x67c, 0x627, 0x6cc, 0x646,
0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x627,
0x631, 0x62c, 0x646, 0x67c, 0x627, 0x6cc, 0x646, 0x20, 0x627, 0x648, 0x693, 0x64a,
0x20, 0x648, 0x62e, 0x62a, 0x644, 0x648, 0x6d0, 0x62f, 0x64a, 0x681, 0x20, 0x627,
0x631, 0x62c, 0x646, 0x67c, 0x627, 0x64a, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x644,
0x648, 0x6d0, 0x62f, 0x64a, 0x681, 0x20, 0x627, 0x631, 0x62c, 0x646, 0x67c, 0x627,
0x64a, 0x646, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e,
0x62a, 0x644, 0x648, 0x6d0, 0x62f, 0x64a, 0x681, 0x20, 0x627, 0x631, 0x62c, 0x646,
0x67c, 0x627, 0x64a, 0x646, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e,
0x62a, 0x627, 0x631, 0x645, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x648, 0x62e,
0x62a, 0x627, 0x631, 0x645, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x645, 0x639,
0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x631, 0x645, 0x646,
0x633, 0x62a, 0x627, 0x646, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e,
0x62a, 0x627, 0x62a, 0x644, 0x627, 0x646, 0x62a, 0x6cc, 0x6a9, 0x20, 0x648, 0x62e,
0x62a, 0x627, 0x62a, 0x644, 0x627, 0x646, 0x62a, 0x6cc, 0x6a9, 0x20, 0x645, 0x639,
0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x62a, 0x644, 0x627,
0x646, 0x62a, 0x6cc, 0x6a9, 0x20, 0x62f, 0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648,
0x631, 0x681, 0x6d0, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x631, 0x6a9, 0x632, 0x64a,
0x20, 0x622, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x20, 0x648, 0x62e,
0x62a, 0x622, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x648, 0x64a, 0x20, 0x645, 0x631,
0x6a9, 0x632, 0x64a, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x62e, 0x62a, 0x622, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x648, 0x64a, 0x20, 0x645,
0x631, 0x6a9, 0x632, 0x64a, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20,
0x631, 0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x622, 0x633,
0x62a, 0x631, 0x627, 0x644, 0x648, 0x64a, 0x20, 0x645, 0x631, 0x6a9, 0x632, 0x64a,
0x20, 0x644, 0x648, 0x6d0, 0x62f, 0x64a, 0x681, 0x20, 0x648, 0x62e, 0x62a, 0x622,
0x633, 0x62a, 0x631, 0x627, 0x644, 0x648, 0x64a, 0x20, 0x645, 0x631, 0x6a9, 0x632,
0x64a, 0x20, 0x644, 0x648, 0x6d0, 0x62f, 0x64a, 0x681, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x622, 0x633, 0x62a, 0x631, 0x627,
0x644, 0x648, 0x64a, 0x20, 0x645, 0x631, 0x6a9, 0x632, 0x64a, 0x20, 0x644, 0x648,
0x6d0, 0x62f, 0x64a, 0x681, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20,
0x631, 0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x62e, 0x62a,
0x64a, 0x681, 0x20, 0x622, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x20,
0x648, 0x62e, 0x62a, 0x622, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x648, 0x64a, 0x20,
0x62e, 0x62a, 0x64a, 0x681, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x622, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x648, 0x64a, 0x20,
0x62e, 0x62a, 0x64a, 0x681, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20,
0x631, 0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x644, 0x648,
0x6d0, 0x62f, 0x64a, 0x681, 0x20, 0x622, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a,
0x627, 0x20, 0x648, 0x62e, 0x62a, 0x622, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x648,
0x64a, 0x20, 0x644, 0x648, 0x6d0, 0x62f, 0x64a, 0x681, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x622, 0x633, 0x62a, 0x631, 0x627,
0x644, 0x648, 0x64a, 0x20, 0x644, 0x648, 0x6d0, 0x62f, 0x64a, 0x681, 0x20, 0x62f,
0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x631, 0x648, 0x69a, 0x627, 0x646, 0x647,
0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x622, 0x630, 0x631, 0x628, 0x627, 0x64a,
0x62c, 0x627, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x622, 0x630, 0x631, 0x628, 0x627,
0x64a, 0x62c, 0x627, 0x646, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x627, 0x630, 0x631, 0x628, 0x627, 0x6cc, 0x6cc, 0x62c, 0x627,
0x646, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x64a,
0x632, 0x648, 0x631, 0x633, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x64a, 0x632, 0x648,
0x631, 0x633, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e,
0x62a, 0x627, 0x64a, 0x632, 0x648, 0x631, 0x633, 0x20, 0x627, 0x648, 0x693, 0x64a,
0x20, 0x648, 0x62e, 0x62a, 0x628, 0x646, 0x6af, 0x644, 0x647, 0x20, 0x62f, 0x6d0,
0x634, 0x20, 0x648, 0x62e, 0x62a, 0x628, 0x646, 0x6ab, 0x644, 0x627, 0x62f, 0x6cc,
0x634, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a,
0x628, 0x646, 0x6ab, 0x644, 0x647, 0x20, 0x62f, 0x64a, 0x634, 0x20, 0x627, 0x648,
0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x628, 0x647, 0x648, 0x67c, 0x627, 0x646,
0x20, 0x648, 0x62e, 0x62a, 0x628, 0x648, 0x644, 0x6cc, 0x648, 0x6cc, 0x627, 0x20,
0x648, 0x62e, 0x62a, 0x628, 0x631, 0x633, 0x644, 0x6cc, 0x627, 0x20, 0x648, 0x62e,
0x62a, 0x628, 0x631, 0x633, 0x644, 0x6cc, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627,
0x631, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x628, 0x631, 0x633, 0x644, 0x6cc, 0x627,
0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x628, 0x631, 0x648,
0x646, 0x627, 0x64a, 0x64a, 0x20, 0x62f, 0x627, 0x631, 0x627, 0x644, 0x633, 0x644,
0x627, 0x645, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x6cc, 0x67e, 0x20, 0x648, 0x631,
0x62f, 0x689, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x6cc, 0x67e, 0x20, 0x648, 0x631,
0x62f, 0x689, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e,
0x62a, 0x6a9, 0x6cc, 0x67e, 0x20, 0x648, 0x631, 0x62f, 0x689, 0x20, 0x633, 0x645,
0x631, 0x20, 0x648, 0x62e, 0x62a, 0x686, 0x645, 0x627, 0x631, 0x648, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x686, 0x627, 0x62a,
0x627, 0x645, 0x20, 0x648, 0x62e, 0x62a, 0x686, 0x627, 0x62a, 0x627, 0x645, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x686, 0x627,
0x62a, 0x627, 0x645, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x631,
0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x686, 0x644, 0x6cc,
0x20, 0x648, 0x62e, 0x62a, 0x686, 0x644, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627,
0x631, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x686, 0x644, 0x6cc, 0x20, 0x627, 0x648,
0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x686, 0x64a, 0x646, 0x20, 0x648, 0x62e,
0x62a, 0x686, 0x6cc, 0x646, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x686, 0x64a, 0x646, 0x20, 0x62f, 0x20, 0x631,
0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x648, 0x62e, 0x62a, 0x6a9,
0x631, 0x64a, 0x633, 0x645, 0x633, 0x20, 0x67c, 0x627, 0x67e, 0x648, 0x20, 0x648,
0x62e, 0x62a, 0x6a9, 0x648, 0x6a9, 0x648, 0x632, 0x20, 0x67c, 0x627, 0x67e, 0x648,
0x6ab, 0x627, 0x646, 0x648, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x648, 0x644, 0x645,
0x628, 0x6cc, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x648, 0x644, 0x645, 0x628,
0x6cc, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x62e,
0x62a, 0x6a9, 0x648, 0x644, 0x645, 0x628, 0x6cc, 0x627, 0x20, 0x627, 0x648, 0x693,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x648, 0x6a9, 0x20, 0x67c, 0x627, 0x67e,
0x648, 0x6ab, 0x627, 0x646, 0x648, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x648, 0x6a9,
0x20, 0x67c, 0x627, 0x67e, 0x648, 0x6ab, 0x627, 0x646, 0x648, 0x20, 0x645, 0x639,
0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x648, 0x6a9, 0x20,
0x67c, 0x627, 0x67e, 0x648, 0x6ab, 0x627, 0x646, 0x648, 0x20, 0x646, 0x64a, 0x645,
0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x64a, 0x648,
0x628, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x6cc, 0x648, 0x628, 0x627, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x6cc,
0x648, 0x628, 0x627, 0x20, 0x62f, 0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631,
0x681, 0x6d0, 0x20, 0x648, 0x62e, 0x62a, 0x689, 0x64a, 0x648, 0x633, 0x20, 0x648,
0x62e, 0x62a, 0x689, 0x648, 0x645, 0x648, 0x646, 0x67c, 0x20, 0x689, 0x64a, 0x20,
0x627, 0x631, 0x648, 0x644, 0x62e, 0x62a, 0x64a, 0x681, 0x20, 0x62a, 0x64a, 0x645,
0x648, 0x631, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x64a, 0x633, 0x62a, 0x631, 0x20,
0x67c, 0x627, 0x67e, 0x648, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x64a, 0x633, 0x62a,
0x631, 0x20, 0x67c, 0x627, 0x67e, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x64a, 0x633, 0x62a, 0x631, 0x20, 0x67c,
0x627, 0x67e, 0x648, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a,
0x62f, 0x20, 0x627, 0x6a9, 0x648, 0x627, 0x62f, 0x648, 0x631, 0x20, 0x648, 0x62e,
0x62a, 0x645, 0x631, 0x6a9, 0x632, 0x64a, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627,
0x64a, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x645, 0x631, 0x6a9, 0x632,
0x64a, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627,
0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x631, 0x6a9, 0x632, 0x64a, 0x20,
0x627, 0x631, 0x648, 0x67e, 0x627, 0x64a, 0x64a, 0x627, 0x648, 0x693, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x62e, 0x62a, 0x64a, 0x681, 0x20, 0x627, 0x631, 0x648, 0x67e,
0x627, 0x64a, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62e, 0x62a, 0x64a, 0x681, 0x20,
0x627, 0x631, 0x648, 0x67e, 0x627, 0x64a, 0x64a, 0x20, 0x645, 0x639, 0x64a, 0x627,
0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62e, 0x62a, 0x64a, 0x681, 0x20, 0x627,
0x631, 0x648, 0x67e, 0x627, 0x64a, 0x64a, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x644, 0x631, 0x6d0, 0x20, 0x62e, 0x62a, 0x64a, 0x681, 0x20,
0x627, 0x631, 0x648, 0x67e, 0x627, 0x64a, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x644,
0x648, 0x6d0, 0x62f, 0x64a, 0x681, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627, 0x64a,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x644, 0x648, 0x6d0, 0x62f, 0x64a, 0x681, 0x20,
0x627, 0x631, 0x648, 0x67e, 0x627, 0x64a, 0x64a, 0x20, 0x645, 0x639, 0x64a, 0x627,
0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x644, 0x648, 0x6d0, 0x62f, 0x64a, 0x681,
0x20, 0x627, 0x631, 0x648, 0x67e, 0x627, 0x64a, 0x64a, 0x20, 0x627, 0x648, 0x693,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x641, 0x648, 0x6a9, 0x644, 0x646, 0x689, 0x20,
0x67c, 0x627, 0x67e, 0x648, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x641, 0x648,
0x6a9, 0x644, 0x646, 0x689, 0x20, 0x67c, 0x627, 0x67e, 0x648, 0x20, 0x645, 0x639,
0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x641, 0x648,
0x6a9, 0x644, 0x646, 0x689, 0x20, 0x67c, 0x627, 0x67e, 0x648, 0x20, 0x627, 0x648,
0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x641, 0x62c, 0x6cc, 0x20, 0x648, 0x62e,
0x62a, 0x641, 0x62c, 0x6cc, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x641, 0x62c, 0x64a, 0x20, 0x62f, 0x20, 0x627, 0x648, 0x693,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x641, 0x631, 0x627, 0x646, 0x633,
0x648, 0x64a, 0x20, 0x6ab, 0x627, 0x646, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x62f,
0x20, 0x641, 0x631, 0x627, 0x646, 0x633, 0x6d0, 0x20, 0x633, 0x648, 0x6cc, 0x644,
0x20, 0x627, 0x648, 0x20, 0x627, 0x646, 0x67c, 0x627, 0x631, 0x67c, 0x64a, 0x6a9,
0x20, 0x648, 0x62e, 0x62a, 0x6ab, 0x627, 0x644, 0x67e, 0x6ab, 0x648, 0x633, 0x20,
0x648, 0x62e, 0x62a, 0x6ab, 0x64a, 0x645, 0x628, 0x64a, 0x631, 0x20, 0x648, 0x62e,
0x62a, 0x62c, 0x648, 0x631, 0x62c, 0x6cc, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x62c,
0x648, 0x631, 0x62c, 0x6cc, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x62c, 0x648, 0x631, 0x62c, 0x64a, 0x627,
0x20, 0x62f, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62c,
0x644, 0x628, 0x631, 0x67c, 0x20, 0x67c, 0x627, 0x67e, 0x648, 0x6ab, 0x627, 0x646,
0x648, 0x20, 0x648, 0x62e, 0x62a, 0x6ab, 0x631, 0x64a, 0x646, 0x648, 0x64a, 0x686,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f,
0x20, 0x62e, 0x62a, 0x6cc, 0x681, 0x20, 0x6ab, 0x631, 0x6cc, 0x646, 0x644, 0x6cc,
0x646, 0x689, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x62e, 0x62a, 0x6cc, 0x681,
0x20, 0x6ab, 0x631, 0x6cc, 0x646, 0x644, 0x6cc, 0x646, 0x689, 0x20, 0x645, 0x639,
0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x62e, 0x62a,
0x6cc, 0x681, 0x20, 0x6ab, 0x631, 0x6cc, 0x646, 0x644, 0x6cc, 0x646, 0x689, 0x20,
0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x644, 0x648, 0x6cc, 0x62f,
0x6cc, 0x681, 0x20, 0x6ab, 0x631, 0x6cc, 0x646, 0x644, 0x6cc, 0x646, 0x689, 0x20,
0x648, 0x62e, 0x62a, 0x644, 0x648, 0x6cc, 0x62f, 0x6cc, 0x681, 0x20, 0x6ab, 0x631,
0x6cc, 0x646, 0x644, 0x6cc, 0x646, 0x689, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x644, 0x648, 0x6cc, 0x62f, 0x6cc, 0x681, 0x20,
0x6ab, 0x631, 0x6cc, 0x646, 0x644, 0x6cc, 0x646, 0x689, 0x20, 0x627, 0x648, 0x693,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62e, 0x644, 0x6cc, 0x62c, 0x20, 0x645, 0x639,
0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x6ab, 0x648,
0x627, 0x646, 0x627, 0x646, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x647, 0x648, 0x627,
0x6cc, 0x6cc, 0x20, 0x627, 0x644, 0x6cc, 0x648, 0x62a, 0x6cc, 0x646, 0x20, 0x648,
0x62e, 0x62a, 0x647, 0x648, 0x627, 0x6cc, 0x6cc, 0x20, 0x627, 0x644, 0x6cc, 0x648,
0x62a, 0x6cc, 0x646, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648,
0x62e, 0x62a, 0x647, 0x648, 0x627, 0x6cc, 0x6cc, 0x20, 0x627, 0x644, 0x6cc, 0x648,
0x62a, 0x6cc, 0x646, 0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d0,
0x20, 0x648, 0x62e, 0x62a, 0x647, 0x627, 0x646, 0x6ab, 0x20, 0x6a9, 0x627, 0x646,
0x6ab, 0x20, 0x648, 0x62e, 0x62a, 0x647, 0x627, 0x646, 0x6ab, 0x20, 0x6a9, 0x627,
0x646, 0x6ab, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e,
0x62a, 0x647, 0x627, 0x646, 0x6ab, 0x20, 0x6a9, 0x627, 0x646, 0x6ab, 0x20, 0x627,
0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x647, 0x627, 0x648, 0x689, 0x20,
0x648, 0x62e, 0x62a, 0x647, 0x627, 0x648, 0x689, 0x20, 0x645, 0x639, 0x6cc, 0x627,
0x631, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x647, 0x627, 0x648, 0x689, 0x20, 0x627,
0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x647, 0x646, 0x62f, 0x20, 0x645,
0x639, 0x6cc, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x647,
0x646, 0x62f, 0x20, 0x633, 0x645, 0x646, 0x62f, 0x631, 0x20, 0x648, 0x62e, 0x62a,
0x627, 0x646, 0x689, 0x648, 0x686, 0x627, 0x6cc, 0x646, 0x647, 0x20, 0x648, 0x62e,
0x62a, 0x645, 0x631, 0x6a9, 0x632, 0x64a, 0x20, 0x627, 0x62f, 0x648, 0x646, 0x64a,
0x632, 0x64a, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x646, 0x62f, 0x648, 0x646,
0x6cc, 0x632, 0x6cc, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x644, 0x648, 0x6cc, 0x62f,
0x6cc, 0x681, 0x20, 0x627, 0x646, 0x62f, 0x648, 0x646, 0x6cc, 0x632, 0x6cc, 0x627,
0x20, 0x648, 0x62e, 0x62a, 0x627, 0x6cc, 0x631, 0x627, 0x646, 0x20, 0x648, 0x62e,
0x62a, 0x627, 0x6cc, 0x631, 0x627, 0x646, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x64a, 0x631, 0x627, 0x646, 0x20, 0x62f,
0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x631, 0x648, 0x69a, 0x627, 0x646, 0x647,
0x20, 0x648, 0x62e, 0x62a, 0x627, 0x631, 0x6a9, 0x648, 0x67c, 0x633, 0x6a9, 0x20,
0x648, 0x62e, 0x62a, 0x627, 0x631, 0x6a9, 0x648, 0x67c, 0x633, 0x6a9, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x631, 0x6a9,
0x648, 0x67c, 0x633, 0x6a9, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e,
0x62a, 0x627, 0x633, 0x631, 0x627, 0x6cc, 0x6cc, 0x644, 0x20, 0x648, 0x62e, 0x62a,
0x627, 0x633, 0x631, 0x627, 0x6cc, 0x6cc, 0x644, 0x20, 0x645, 0x639, 0x64a, 0x627,
0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x633, 0x631, 0x627, 0x64a, 0x64a,
0x644, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x631, 0x648, 0x69a,
0x627, 0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x62c, 0x627, 0x67e, 0x627, 0x646,
0x20, 0x648, 0x62e, 0x62a, 0x62c, 0x627, 0x67e, 0x627, 0x646, 0x20, 0x645, 0x639,
0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62c, 0x627, 0x67e, 0x627,
0x646, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x631, 0x648, 0x69a,
0x627, 0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x642, 0x632, 0x627, 0x642, 0x633,
0x62a, 0x627, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x62e, 0x62a, 0x64a, 0x681, 0x20,
0x642, 0x627, 0x632, 0x642, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x648, 0x62e, 0x62a,
0x644, 0x648, 0x6cc, 0x62f, 0x6cc, 0x681, 0x20, 0x642, 0x632, 0x627, 0x642, 0x633,
0x62a, 0x627, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x648, 0x631, 0x64a, 0x627,
0x64a, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x648, 0x631, 0x64a, 0x627, 0x64a,
0x64a, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a,
0x6a9, 0x648, 0x631, 0x64a, 0x627, 0x64a, 0x64a, 0x20, 0x62f, 0x20, 0x648, 0x631,
0x681, 0x6d0, 0x20, 0x631, 0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x648, 0x62e,
0x62a, 0x6a9, 0x648, 0x633, 0x631, 0x627, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x6a9,
0x631, 0x64a, 0x633, 0x646, 0x648, 0x64a, 0x627, 0x631, 0x633, 0x6a9, 0x20, 0x648,
0x62e, 0x62a, 0x6a9, 0x631, 0x64a, 0x633, 0x646, 0x648, 0x64a, 0x627, 0x631, 0x633,
0x6a9, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a,
0x6a9, 0x631, 0x64a, 0x633, 0x646, 0x648, 0x64a, 0x627, 0x631, 0x633, 0x6a9, 0x20,
0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x631, 0x63a, 0x6cc,
0x632, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x644, 0x627, 0x64a,
0x646, 0x20, 0x67c, 0x627, 0x67e, 0x648, 0x6ab, 0x627, 0x646, 0x648, 0x20, 0x648,
0x62e, 0x62a, 0x644, 0x627, 0x631, 0x689, 0x20, 0x647, 0x648, 0x64a, 0x20, 0x648,
0x62e, 0x62a, 0x644, 0x627, 0x631, 0x689, 0x20, 0x647, 0x648, 0x64a, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x644, 0x627, 0x631,
0x689, 0x20, 0x647, 0x648, 0x64a, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d0,
0x20, 0x631, 0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x645,
0x64a, 0x6ab, 0x689, 0x627, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x64a, 0x6ab,
0x689, 0x627, 0x646, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x62e, 0x62a, 0x645, 0x64a, 0x6ab, 0x689, 0x627, 0x646, 0x20, 0x627, 0x648, 0x693,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x644, 0x627, 0x626, 0x6cc, 0x634, 0x6cc,
0x627, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x627, 0x644, 0x62f, 0x64a, 0x67e, 0x20,
0x648, 0x62e, 0x62a, 0x645, 0x627, 0x631, 0x6a9, 0x633, 0x633, 0x20, 0x648, 0x62e,
0x62a, 0x645, 0x627, 0x631, 0x634, 0x644, 0x20, 0x67c, 0x627, 0x67e, 0x648, 0x6ab,
0x627, 0x646, 0x648, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x627, 0x631, 0x64a, 0x634,
0x64a, 0x633, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x627, 0x631, 0x6cc, 0x634, 0x6cc,
0x633, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a,
0x645, 0x627, 0x631, 0x64a, 0x634, 0x64a, 0x633, 0x20, 0x62f, 0x20, 0x627, 0x648,
0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x627, 0x648, 0x633, 0x646, 0x20,
0x648, 0x62e, 0x62a, 0x645, 0x6a9, 0x633, 0x6cc, 0x6a9, 0x646, 0x20, 0x67e, 0x6cc,
0x633, 0x641, 0x6a9, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x6a9, 0x633, 0x6cc, 0x6a9,
0x646, 0x20, 0x67e, 0x6cc, 0x633, 0x641, 0x6a9, 0x20, 0x645, 0x639, 0x6cc, 0x627,
0x631, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x6a9, 0x633, 0x6cc, 0x6a9, 0x646,
0x20, 0x67e, 0x6cc, 0x633, 0x641, 0x6a9, 0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648,
0x631, 0x681, 0x6d0, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x648, 0x644, 0x627, 0x646,
0x20, 0x628, 0x627, 0x67c, 0x631, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x648, 0x644,
0x627, 0x646, 0x20, 0x628, 0x627, 0x67c, 0x631, 0x20, 0x645, 0x639, 0x6cc, 0x627,
0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x648, 0x644, 0x627, 0x646, 0x20,
0x628, 0x627, 0x67c, 0x631, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e,
0x62a, 0x645, 0x627, 0x633, 0x6a9, 0x648, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x627,
0x633, 0x6a9, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x62e, 0x62a, 0x645, 0x627, 0x633, 0x6a9, 0x648, 0x20, 0x633, 0x645, 0x631, 0x20,
0x648, 0x62e, 0x62a, 0x645, 0x6cc, 0x627, 0x646, 0x645, 0x627, 0x631, 0x20, 0x648,
0x62e, 0x62a, 0x646, 0x627, 0x648, 0x631, 0x648, 0x20, 0x648, 0x62e, 0x62a, 0x646,
0x6cc, 0x67e, 0x627, 0x644, 0x20, 0x648, 0x62e, 0x62a, 0x646, 0x6cc, 0x648, 0x20,
0x6a9, 0x627, 0x644, 0x6cc, 0x689, 0x648, 0x646, 0x6cc, 0x627, 0x20, 0x648, 0x62e,
0x62a, 0x646, 0x64a, 0x648, 0x20, 0x6a9, 0x627, 0x644, 0x6cc, 0x689, 0x648, 0x646,
0x6cc, 0x627, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e,
0x62a, 0x646, 0x64a, 0x648, 0x20, 0x6a9, 0x627, 0x64a, 0x689, 0x648, 0x646, 0x64a,
0x627, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x646, 0x64a,
0x648, 0x632, 0x64a, 0x20, 0x644, 0x6d0, 0x646, 0x689, 0x20, 0x648, 0x62e, 0x62a,
0x646, 0x64a, 0x648, 0x632, 0x64a, 0x20, 0x644, 0x6d0, 0x646, 0x689, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x646, 0x64a, 0x648,
0x632, 0x64a, 0x20, 0x644, 0x6d0, 0x646, 0x689, 0x20, 0x62f, 0x20, 0x648, 0x631,
0x681, 0x6d0, 0x20, 0x631, 0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x648, 0x62e,
0x62a, 0x646, 0x64a, 0x648, 0x20, 0x641, 0x627, 0x648, 0x646, 0x689, 0x644, 0x6d0,
0x646, 0x689, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x646, 0x648, 0x64a, 0x20,
0x641, 0x6cc, 0x644, 0x689, 0x644, 0x6cc, 0x646, 0x689, 0x20, 0x645, 0x639, 0x6cc,
0x627, 0x631, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x646, 0x64a, 0x648, 0x20, 0x641,
0x627, 0x648, 0x646, 0x689, 0x644, 0x6d0, 0x646, 0x689, 0x20, 0x62f, 0x20, 0x648,
0x631, 0x681, 0x6d0, 0x20, 0x631, 0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x648,
0x62e, 0x62a, 0x646, 0x6cc, 0x6cc, 0x648, 0x20, 0x648, 0x62e, 0x62a, 0x646, 0x648,
0x631, 0x641, 0x648, 0x6a9, 0x20, 0x67c, 0x627, 0x67e, 0x648, 0x20, 0x648, 0x62e,
0x62a, 0x62f, 0x20, 0x646, 0x648, 0x631, 0x641, 0x6a9, 0x627, 0x633, 0x20, 0x67c,
0x627, 0x67e, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x62e, 0x62a, 0x62f, 0x20, 0x646, 0x648, 0x631, 0x641, 0x6a9, 0x627, 0x633, 0x20,
0x67c, 0x627, 0x67e, 0x648, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e,
0x62a, 0x641, 0x631, 0x646, 0x627, 0x646, 0x688, 0x648, 0x20, 0x62f, 0x64a, 0x20,
0x646, 0x648, 0x631, 0x648, 0x646, 0x647, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x641,
0x631, 0x646, 0x627, 0x646, 0x688, 0x648, 0x20, 0x62f, 0x64a, 0x20, 0x646, 0x648,
0x631, 0x648, 0x646, 0x647, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc,
0x20, 0x648, 0x62e, 0x62a, 0x641, 0x631, 0x646, 0x627, 0x646, 0x688, 0x648, 0x20,
0x62f, 0x64a, 0x20, 0x646, 0x648, 0x631, 0x648, 0x646, 0x6be, 0x627, 0x20, 0x627,
0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x646, 0x648, 0x648, 0x633, 0x64a,
0x628, 0x631, 0x633, 0x6a9, 0x20, 0x648, 0x62e, 0x62a, 0x646, 0x648, 0x648, 0x633,
0x64a, 0x628, 0x631, 0x633, 0x6a9, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x62e, 0x62a, 0x646, 0x648, 0x648, 0x633, 0x64a, 0x628, 0x631, 0x633,
0x6a9, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x648,
0x645, 0x633, 0x6a9, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x648, 0x645, 0x633, 0x6a9,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627,
0x648, 0x645, 0x633, 0x6a9, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e,
0x62a, 0x67e, 0x627, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x648, 0x62e, 0x62a,
0x67e, 0x627, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x645, 0x639, 0x6cc, 0x627,
0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x67e, 0x627, 0x6a9, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x67e, 0x627,
0x644, 0x627, 0x648, 0x20, 0x648, 0x62e, 0x62a, 0x67e, 0x627, 0x67e, 0x648, 0x627,
0x20, 0x646, 0x6cc, 0x648, 0x20, 0x6ab, 0x646, 0x6cc, 0x20, 0x648, 0x62e, 0x62a,
0x67e, 0x64a, 0x631, 0x627, 0x6ab, 0x648, 0x627, 0x6cc, 0x20, 0x648, 0x62e, 0x62a,
0x67e, 0x6cc, 0x631, 0x627, 0x6ab, 0x648, 0x627, 0x6cc, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x67e, 0x627, 0x631, 0x627, 0x6ab,
0x648, 0x627, 0x6cc, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a,
0x67e, 0x6cc, 0x631, 0x648, 0x20, 0x648, 0x62e, 0x62a, 0x67e, 0x6cc, 0x631, 0x648,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x67e,
0x6cc, 0x631, 0x648, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a,
0x641, 0x644, 0x67e, 0x627, 0x64a, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x641, 0x644,
0x67e, 0x627, 0x64a, 0x646, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x641, 0x644, 0x67e, 0x627, 0x64a, 0x646, 0x20, 0x627, 0x648,
0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x641, 0x6cc, 0x646, 0x6a9,
0x633, 0x20, 0x67c, 0x627, 0x67e, 0x648, 0x20, 0x648, 0x62e, 0x62a, 0x633, 0x6cc,
0x646, 0x67c, 0x20, 0x67e, 0x6cc, 0x6cc, 0x631, 0x627, 0x20, 0x648, 0x20, 0x645,
0x64a, 0x6a9, 0x644, 0x64a, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x633, 0x6cc, 0x646,
0x67c, 0x20, 0x67e, 0x6cc, 0x6cc, 0x631, 0x627, 0x20, 0x648, 0x20, 0x645, 0x64a,
0x6a9, 0x644, 0x64a, 0x646, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20,
0x648, 0x62e, 0x62a, 0x633, 0x6cc, 0x646, 0x67c, 0x20, 0x67e, 0x6cc, 0x6cc, 0x631,
0x627, 0x20, 0x648, 0x20, 0x645, 0x64a, 0x6a9, 0x644, 0x64a, 0x646, 0x20, 0x631,
0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x648, 0x62e, 0x62a, 0x67e,
0x6cc, 0x67c, 0x20, 0x6a9, 0x627, 0x631, 0x6cc, 0x646, 0x20, 0x648, 0x62e, 0x62a,
0x67e, 0x648, 0x646, 0x64a, 0x67e, 0x20, 0x648, 0x62e, 0x62a, 0x67e, 0x64a, 0x627,
0x646, 0x6ab, 0x20, 0x64a, 0x627, 0x646, 0x6ab, 0x20, 0x648, 0x62e, 0x62a, 0x631,
0x64a, 0x20, 0x64a, 0x648, 0x646, 0x64a, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x631,
0x648, 0x62f, 0x631, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x633, 0x62e, 0x644, 0x64a,
0x646, 0x20, 0x648, 0x62e, 0x62a, 0x633, 0x62e, 0x644, 0x6cc, 0x646, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x633, 0x62e, 0x644,
0x64a, 0x646, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x633,
0x645, 0x648, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x633, 0x645, 0x648, 0x627, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x633, 0x645,
0x648, 0x627, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x631, 0x648,
0x69a, 0x627, 0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x633, 0x6cc, 0x686, 0x6cc,
0x644, 0x633, 0x20, 0x648, 0x62e, 0x62a, 0x633, 0x646, 0x6ab, 0x627, 0x67e, 0x648,
0x631, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a,
0x633, 0x644, 0x6cc, 0x645, 0x627, 0x646, 0x20, 0x67c, 0x627, 0x67e, 0x648, 0x6ab,
0x627, 0x646, 0x648, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x633, 0x648, 0x6cc,
0x644, 0x20, 0x62c, 0x648, 0x631, 0x62c, 0x6cc, 0x627, 0x20, 0x648, 0x62e, 0x62a,
0x633, 0x648, 0x631, 0x6cc, 0x646, 0x627, 0x645, 0x20, 0x648, 0x62e, 0x62a, 0x633,
0x6cc, 0x648, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x67c, 0x647, 0x64a, 0x67c, 0x64a,
0x20, 0x648, 0x62e, 0x62a, 0x62a, 0x627, 0x64a, 0x67e, 0x64a, 0x20, 0x648, 0x62e,
0x62a, 0x62a, 0x627, 0x64a, 0x67e, 0x64a, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62a, 0x627, 0x64a, 0x67e, 0x64a, 0x20, 0x62f,
0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x631, 0x648, 0x69a, 0x627, 0x646, 0x647,
0x20, 0x648, 0x62e, 0x62a, 0x62a, 0x627, 0x62c, 0x6a9, 0x633, 0x62a, 0x627, 0x646,
0x20, 0x648, 0x62e, 0x62a, 0x62a, 0x648, 0x6a9, 0x6cc, 0x644, 0x627, 0x648, 0x20,
0x648, 0x62e, 0x62a, 0x67c, 0x648, 0x646, 0x6ab, 0x627, 0x20, 0x648, 0x62e, 0x62a,
0x62f, 0x20, 0x67c, 0x648, 0x646, 0x6ab, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x67c, 0x648, 0x646, 0x6ab, 0x627, 0x20, 0x627,
0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x686, 0x648, 0x6a9, 0x20, 0x648,
0x62e, 0x62a, 0x62a, 0x631, 0x6a9, 0x645, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646,
0x20, 0x648, 0x62e, 0x62a, 0x62a, 0x631, 0x6a9, 0x645, 0x646, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a,
0x62a, 0x631, 0x6a9, 0x645, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x627, 0x648,
0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62a, 0x648, 0x648, 0x627, 0x644, 0x648,
0x20, 0x648, 0x62e, 0x62a, 0x6cc, 0x648, 0x631, 0x648, 0x6ab, 0x648, 0x627, 0x6cc,
0x20, 0x648, 0x62e, 0x62a, 0x6cc, 0x648, 0x631, 0x648, 0x6ab, 0x648, 0x627, 0x6cc,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x6cc,
0x648, 0x631, 0x648, 0x6ab, 0x648, 0x627, 0x6cc, 0x20, 0x627, 0x648, 0x693, 0x64a,
0x20, 0x648, 0x62e, 0x62a, 0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a, 0x627, 0x646,
0x20, 0x648, 0x62e, 0x62a, 0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a, 0x627, 0x646,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627,
0x632, 0x628, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x627, 0x648, 0x693, 0x64a,
0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x648, 0x646, 0x627, 0x62a, 0x648, 0x20,
0x648, 0x62e, 0x62a, 0x648, 0x646, 0x648, 0x627, 0x62a, 0x648, 0x20, 0x645, 0x639,
0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x648, 0x646, 0x648, 0x627,
0x62a, 0x648, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x648,
0x6cc, 0x646, 0x632, 0x648, 0x6cc, 0x644, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x648,
0x644, 0x627, 0x62f, 0x6cc, 0x648, 0x633, 0x62a, 0x627, 0x6a9, 0x20, 0x648, 0x62e,
0x62a, 0x648, 0x644, 0x627, 0x62f, 0x64a, 0x648, 0x633, 0x62a, 0x627, 0x6a9, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x648, 0x644,
0x627, 0x62f, 0x64a, 0x648, 0x633, 0x62a, 0x627, 0x6a9, 0x20, 0x627, 0x648, 0x693,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x648, 0x627, 0x644, 0x6ab, 0x648, 0x6ab, 0x631,
0x627, 0x62f, 0x20, 0x648, 0x62e, 0x62a, 0x648, 0x627, 0x644, 0x6ab, 0x648, 0x6ab,
0x631, 0x627, 0x62f, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x62e, 0x62a, 0x648, 0x627, 0x644, 0x6ab, 0x648, 0x6ab, 0x631, 0x627, 0x62f, 0x20,
0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x648, 0x627, 0x633, 0x62a,
0x648, 0x6a9, 0x20, 0x648, 0x62e, 0x62a, 0x648, 0x64a, 0x6a9, 0x20, 0x62a, 0x627,
0x67e, 0x648, 0x20, 0x648, 0x62e, 0x62a, 0x648, 0x627, 0x644, 0x633, 0x20, 0x627,
0x648, 0x20, 0x641, 0x648, 0x62a, 0x648, 0x646, 0x627, 0x20, 0x648, 0x62e, 0x62a,
0x64a, 0x627, 0x6a9, 0x648, 0x67c, 0x633, 0x6a9, 0x20, 0x648, 0x62e, 0x62a, 0x64a,
0x627, 0x6a9, 0x648, 0x67c, 0x633, 0x6a9, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x64a, 0x627, 0x6a9, 0x648, 0x67c, 0x633, 0x6a9,
0x20, 0x62f, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x6cc,
0x6a9, 0x627, 0x67c, 0x6cc, 0x631, 0x6cc, 0x646, 0x628, 0x631, 0x6ab, 0x20, 0x648,
0x62e, 0x62a, 0x62f, 0x20, 0x64a, 0x627, 0x6a9, 0x64a, 0x67c, 0x631, 0x646, 0x628,
0x631, 0x6ab, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e,
0x62a, 0x62f, 0x20, 0x64a, 0x627, 0x6a9, 0x67c, 0x631, 0x646, 0x628, 0x631, 0x6ab,
0x20, 0x62f, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f,
0x20, 0x6cc, 0x648, 0x6a9, 0x648, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20,
0x627, 0x644, 0x627, 0x633, 0x6a9, 0x627, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681,
0x6d2, 0x20, 0x631, 0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x6a9, 0x648, 0x644,
0x645, 0x631, 0x6a9, 0x632, 0x64a, 0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631,
0x681, 0x6d2, 0x20, 0x648, 0x62e, 0x62a, 0x62e, 0x62a, 0x64a, 0x681, 0x20, 0x62f,
0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20, 0x648, 0x62e,
0x62a, 0x62f, 0x20, 0x63a, 0x631, 0x647, 0x20, 0x62f, 0x20, 0x631, 0x6bc, 0x627,
0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20, 0x648, 0x62e, 0x62a, 0x67e, 0x6cc, 0x633,
0x641, 0x6a9, 0x20, 0x62f, 0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681,
0x6d2, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x627, 0x67e, 0x6cc, 0x627, 0x20,
0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20,
0x639, 0x631, 0x628, 0x64a, 0x20, 0x648, 0x631, 0x681, 0x67e, 0x627, 0x6bc, 0x6d2,
0x20, 0x648, 0x62e, 0x62a, 0x627, 0x62a, 0x644, 0x627, 0x646, 0x62a, 0x6cc, 0x6a9,
0x20, 0x62f, 0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20,
0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x627, 0x633, 0x67c, 0x631, 0x627, 0x644, 0x6cc,
0x627, 0x20, 0x644, 0x648, 0x6cc, 0x62f, 0x6cc, 0x681, 0x20, 0x62f, 0x20, 0x648,
0x631, 0x681, 0x6d2, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x686, 0x64a, 0x646,
0x20, 0x62f, 0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20,
0x648, 0x62e, 0x62a, 0x6a9, 0x6cc, 0x648, 0x628, 0x627, 0x20, 0x62f, 0x20, 0x631,
0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20, 0x648, 0x62e, 0x62a, 0x644,
0x648, 0x6d0, 0x62f, 0x64a, 0x632, 0x6d2, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627,
0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x644, 0x648, 0x62f, 0x6cc, 0x681, 0x6d2,
0x20, 0x627, 0x631, 0x648, 0x67e, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x644, 0x648, 0x62f, 0x6cc, 0x681,
0x6d2, 0x20, 0x627, 0x648, 0x631, 0x67e, 0x627, 0x20, 0x62f, 0x20, 0x627, 0x648,
0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x641, 0x631, 0x627, 0x646,
0x633, 0x6d2, 0x20, 0x633, 0x648, 0x6cc, 0x644, 0x20, 0x627, 0x648, 0x20, 0x627,
0x646, 0x67c, 0x627, 0x631, 0x67c, 0x64a, 0x6a9, 0x20, 0x648, 0x62e, 0x62a, 0x647,
0x648, 0x627, 0x6cc, 0x6cc, 0x20, 0x627, 0x644, 0x6cc, 0x648, 0x62a, 0x6cc, 0x646,
0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20, 0x648, 0x62e,
0x62a, 0x62f, 0x20, 0x627, 0x6cc, 0x631, 0x627, 0x646, 0x20, 0x62f, 0x20, 0x648,
0x631, 0x681, 0x6d2, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x627, 0x633, 0x631,
0x627, 0x6cc, 0x6cc, 0x644, 0x648, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d2,
0x20, 0x648, 0x62e, 0x62a, 0x62c, 0x627, 0x67e, 0x627, 0x646, 0x20, 0x62f, 0x20,
0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20, 0x648, 0x62e, 0x62a,
0x62f, 0x20, 0x6a9, 0x648, 0x631, 0x6cc, 0x627, 0x20, 0x62f, 0x20, 0x648, 0x631,
0x681, 0x6d2, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20, 0x648, 0x62e,
0x62a, 0x62f, 0x20, 0x6a9, 0x631, 0x69a, 0x6d2, 0x20, 0x67c, 0x627, 0x67c, 0x648,
0x628, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x631, 0x628, 0x20, 0x647, 0x627, 0x648,
0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20, 0x62f, 0x20, 0x631, 0x6bc,
0x627, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x6a9, 0x633, 0x6cc, 0x6a9, 0x646, 0x20,
0x67e, 0x6cc, 0x633, 0x641, 0x6a9, 0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631,
0x681, 0x6d2, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x646, 0x6cc, 0x648, 0x632,
0x6cc, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681,
0x6d2, 0x20, 0x62f, 0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x62f,
0x20, 0x646, 0x648, 0x64a, 0x20, 0x641, 0x6cc, 0x644, 0x689, 0x644, 0x6cc, 0x646,
0x689, 0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20, 0x648,
0x62e, 0x62a, 0x633, 0x6cc, 0x646, 0x67c, 0x20, 0x67e, 0x6cc, 0x6cc, 0x631, 0x627,
0x20, 0x648, 0x20, 0x645, 0x64a, 0x6a9, 0x644, 0x64a, 0x646, 0x20, 0x631, 0x6bc,
0x627, 0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20,
0x633, 0x645, 0x648, 0x627, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20,
0x631, 0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x6a9, 0x648, 0x644, 0x648, 0x642,
0x62a, 0x20, 0x627, 0x641, 0x63a, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x648,
0x642, 0x62a, 0x20, 0x645, 0x631, 0x6a9, 0x632, 0x20, 0x622, 0x641, 0x631, 0x6cc,
0x642, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x634, 0x631, 0x642, 0x20, 0x627, 0x641,
0x631, 0x6cc, 0x642, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc,
0x20, 0x62c, 0x646, 0x648, 0x628, 0x20, 0x627, 0x641, 0x631, 0x6cc, 0x642, 0x627,
0x648, 0x642, 0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x627, 0x641, 0x631, 0x6cc,
0x642, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x63a,
0x631, 0x628, 0x20, 0x627, 0x641, 0x631, 0x6cc, 0x642, 0x627, 0x648, 0x642, 0x62a,
0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x63a, 0x631,
0x628, 0x20, 0x627, 0x641, 0x631, 0x6cc, 0x642, 0x627, 0x648, 0x642, 0x62a, 0x20,
0x622, 0x644, 0x627, 0x633, 0x6a9, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627,
0x62f, 0x6cc, 0x20, 0x622, 0x644, 0x627, 0x633, 0x6a9, 0x627, 0x648, 0x642, 0x62a,
0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x622, 0x644,
0x627, 0x633, 0x6a9, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x622, 0x644, 0x645, 0x627,
0x622, 0x62a, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20,
0x622, 0x644, 0x645, 0x627, 0x622, 0x62a, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a,
0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x622, 0x644, 0x645, 0x627,
0x622, 0x62a, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x622, 0x645, 0x627, 0x632, 0x648,
0x646, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x622, 0x645,
0x627, 0x632, 0x648, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633,
0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x622, 0x645, 0x627, 0x632, 0x648, 0x646, 0x648,
0x642, 0x62a, 0x20, 0x645, 0x631, 0x6a9, 0x632, 0x20, 0x627, 0x645, 0x631, 0x6cc,
0x6a9, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x645,
0x631, 0x6a9, 0x632, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x648, 0x642,
0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x645,
0x631, 0x6a9, 0x632, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x648, 0x642,
0x62a, 0x20, 0x634, 0x631, 0x642, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627,
0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x634, 0x631, 0x642,
0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a,
0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x634, 0x631, 0x642, 0x20,
0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x6a9, 0x648,
0x647, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9,
0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x6a9, 0x648,
0x647, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9,
0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646,
0x6cc, 0x20, 0x6a9, 0x648, 0x647, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x627,
0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x63a, 0x631, 0x628,
0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639,
0x627, 0x62f, 0x6cc, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x627, 0x645, 0x631, 0x6cc,
0x6a9, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627,
0x646, 0x6cc, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9,
0x627, 0x648, 0x642, 0x62a, 0x20, 0x622, 0x646, 0x627, 0x62f, 0x6cc, 0x631, 0x648,
0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x622, 0x646, 0x627, 0x62f,
0x6cc, 0x631, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627,
0x646, 0x6cc, 0x20, 0x622, 0x646, 0x627, 0x62f, 0x6cc, 0x631, 0x648, 0x642, 0x62a,
0x20, 0x622, 0x67e, 0x6cc, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f,
0x6cc, 0x20, 0x622, 0x67e, 0x6cc, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627,
0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x622, 0x67e, 0x6cc, 0x627, 0x648,
0x642, 0x62a, 0x20, 0x639, 0x631, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642,
0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x639, 0x631, 0x628, 0x633, 0x62a,
0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627,
0x646, 0x6cc, 0x20, 0x639, 0x631, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642,
0x62a, 0x20, 0x622, 0x631, 0x698, 0x627, 0x646, 0x62a, 0x6cc, 0x646, 0x648, 0x642,
0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x622, 0x631, 0x698, 0x627, 0x646,
0x62a, 0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a,
0x627, 0x646, 0x6cc, 0x20, 0x622, 0x631, 0x698, 0x627, 0x646, 0x62a, 0x6cc, 0x646,
0x648, 0x642, 0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x622, 0x631, 0x698, 0x627,
0x646, 0x62a, 0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc,
0x20, 0x63a, 0x631, 0x628, 0x20, 0x622, 0x631, 0x698, 0x627, 0x646, 0x62a, 0x6cc,
0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646,
0x6cc, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x622, 0x631, 0x698, 0x627, 0x646, 0x62a,
0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x627, 0x631, 0x645, 0x646, 0x633, 0x62a,
0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x627,
0x631, 0x645, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a,
0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x627, 0x631, 0x645, 0x646,
0x633, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x622, 0x62a, 0x644, 0x627,
0x646, 0x62a, 0x6cc, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc,
0x20, 0x622, 0x62a, 0x644, 0x627, 0x646, 0x62a, 0x6cc, 0x6a9, 0x648, 0x642, 0x62a,
0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x622, 0x62a,
0x644, 0x627, 0x646, 0x62a, 0x6cc, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x645, 0x631,
0x6a9, 0x632, 0x20, 0x627, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x648,
0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x645, 0x631, 0x6a9, 0x632,
0x20, 0x627, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x648, 0x642, 0x62a,
0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x645, 0x631,
0x6a9, 0x632, 0x20, 0x627, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x648,
0x642, 0x62a, 0x20, 0x645, 0x631, 0x6a9, 0x632, 0x20, 0x627, 0x633, 0x62a, 0x631,
0x627, 0x644, 0x6cc, 0x627, 0x6cc, 0x20, 0x63a, 0x631, 0x628, 0x6cc, 0x648, 0x642,
0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x645, 0x631, 0x6a9, 0x632, 0x20,
0x627, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x6cc, 0x20, 0x63a, 0x631,
0x628, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627,
0x646, 0x6cc, 0x20, 0x645, 0x631, 0x6a9, 0x632, 0x20, 0x627, 0x633, 0x62a, 0x631,
0x627, 0x644, 0x6cc, 0x627, 0x6cc, 0x20, 0x63a, 0x631, 0x628, 0x6cc, 0x648, 0x642,
0x62a, 0x20, 0x627, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x6cc, 0x20,
0x634, 0x631, 0x642, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc,
0x20, 0x627, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x6cc, 0x20, 0x634,
0x631, 0x642, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a,
0x627, 0x646, 0x6cc, 0x20, 0x627, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627,
0x6cc, 0x20, 0x634, 0x631, 0x642, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x627, 0x633,
0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x6cc, 0x20, 0x63a, 0x631, 0x628, 0x6cc,
0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x627, 0x633, 0x62a,
0x631, 0x627, 0x644, 0x6cc, 0x627, 0x6cc, 0x20, 0x63a, 0x631, 0x628, 0x6cc, 0x648,
0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20,
0x627, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x6cc, 0x20, 0x63a, 0x631,
0x628, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x62c, 0x645, 0x647, 0x648, 0x631, 0x6cc,
0x20, 0x622, 0x630, 0x631, 0x628, 0x627, 0x6cc, 0x62c, 0x627, 0x646, 0x648, 0x642,
0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x62c, 0x645, 0x647, 0x648, 0x631,
0x6cc, 0x20, 0x622, 0x630, 0x631, 0x628, 0x627, 0x6cc, 0x62c, 0x627, 0x646, 0x648,
0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20,
0x62c, 0x645, 0x647, 0x648, 0x631, 0x6cc, 0x20, 0x622, 0x630, 0x631, 0x628, 0x627,
0x6cc, 0x62c, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x622, 0x632, 0x648, 0x631,
0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x622, 0x632, 0x648,
0x631, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646,
0x6cc, 0x20, 0x622, 0x632, 0x648, 0x631, 0x648, 0x642, 0x62a, 0x20, 0x628, 0x646,
0x6af, 0x644, 0x627, 0x62f, 0x634, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f,
0x6cc, 0x20, 0x628, 0x646, 0x6af, 0x644, 0x627, 0x62f, 0x634, 0x648, 0x642, 0x62a,
0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x628, 0x646,
0x6af, 0x644, 0x627, 0x62f, 0x634, 0x648, 0x642, 0x62a, 0x20, 0x628, 0x648, 0x62a,
0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x628, 0x648, 0x644, 0x6cc, 0x648, 0x6cc,
0x648, 0x642, 0x62a, 0x20, 0x628, 0x631, 0x627, 0x632, 0x6cc, 0x644, 0x6cc, 0x627,
0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x628, 0x631, 0x627,
0x632, 0x6cc, 0x644, 0x6cc, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628,
0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x628, 0x631, 0x627, 0x632, 0x6cc, 0x644,
0x6cc, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x628, 0x631, 0x648, 0x646, 0x626, 0x6cc,
0x20, 0x62f, 0x627, 0x631, 0x627, 0x644, 0x633, 0x644, 0x627, 0x645, 0x648, 0x642,
0x62a, 0x20, 0x6a9, 0x6cc, 0x67e, 0x200c, 0x648, 0x631, 0x62f, 0x648, 0x642, 0x62a,
0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x6a9, 0x6cc, 0x67e, 0x200c, 0x648, 0x631,
0x62f, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646,
0x6cc, 0x20, 0x6a9, 0x6cc, 0x67e, 0x200c, 0x648, 0x631, 0x62f, 0x648, 0x642, 0x62a,
0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x686, 0x627, 0x645, 0x648, 0x631, 0x648,
0x648, 0x642, 0x62a, 0x20, 0x686, 0x62a, 0x200c, 0x647, 0x627, 0x645, 0x648, 0x642,
0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x686, 0x62a, 0x200c, 0x647, 0x627,
0x645, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646,
0x6cc, 0x20, 0x686, 0x62a, 0x200c, 0x647, 0x627, 0x645, 0x648, 0x642, 0x62a, 0x20,
0x634, 0x6cc, 0x644, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc,
0x20, 0x634, 0x6cc, 0x644, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628,
0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x634, 0x6cc, 0x644, 0x6cc, 0x648, 0x642,
0x62a, 0x20, 0x686, 0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f,
0x6cc, 0x20, 0x686, 0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628,
0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x686, 0x6cc, 0x646, 0x648, 0x642, 0x62a,
0x20, 0x62c, 0x632, 0x6cc, 0x631, 0x647, 0x654, 0x20, 0x6a9, 0x631, 0x6cc, 0x633,
0x645, 0x633, 0x648, 0x642, 0x62a, 0x20, 0x62c, 0x632, 0x627, 0x6cc, 0x631, 0x20,
0x6a9, 0x648, 0x6a9, 0x648, 0x633, 0x648, 0x642, 0x62a, 0x20, 0x6a9, 0x644, 0x645,
0x628, 0x6cc, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20,
0x6a9, 0x644, 0x645, 0x628, 0x6cc, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627,
0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x6a9, 0x644, 0x645, 0x628, 0x6cc,
0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x62c, 0x632,
0x627, 0x6cc, 0x631, 0x20, 0x6a9, 0x648, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x62a,
0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x62c, 0x632, 0x627, 0x6cc,
0x631, 0x20, 0x6a9, 0x648, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x6a9, 0x648, 0x628,
0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x6a9, 0x648,
0x628, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627,
0x646, 0x6cc, 0x20, 0x6a9, 0x648, 0x628, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62f,
0x6cc, 0x648, 0x6cc, 0x633, 0x648, 0x642, 0x62a, 0x20, 0x62f, 0x648, 0x645, 0x648,
0x646, 0x20, 0x62f, 0x648, 0x631, 0x648, 0x6cc, 0x644, 0x648, 0x642, 0x62a, 0x20,
0x62a, 0x6cc, 0x645, 0x648, 0x631, 0x20, 0x634, 0x631, 0x642, 0x6cc, 0x648, 0x642,
0x62a, 0x20, 0x62c, 0x632, 0x6cc, 0x631, 0x647, 0x654, 0x20, 0x627, 0x6cc, 0x633,
0x62a, 0x631, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x62c,
0x632, 0x6cc, 0x631, 0x647, 0x654, 0x20, 0x627, 0x6cc, 0x633, 0x62a, 0x631, 0x648,
0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20,
0x62c, 0x632, 0x6cc, 0x631, 0x647, 0x654, 0x20, 0x627, 0x6cc, 0x633, 0x62a, 0x631,
0x648, 0x642, 0x62a, 0x20, 0x627, 0x6a9, 0x648, 0x627, 0x62f, 0x648, 0x631, 0x648,
0x642, 0x62a, 0x20, 0x645, 0x631, 0x6a9, 0x632, 0x20, 0x627, 0x631, 0x648, 0x67e,
0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x645, 0x631,
0x6a9, 0x632, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627, 0x648, 0x642, 0x62a, 0x20,
0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x645, 0x631, 0x6a9,
0x632, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x634,
0x631, 0x642, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627, 0x648, 0x642, 0x62a, 0x20,
0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x634, 0x631, 0x642, 0x20, 0x627, 0x631, 0x648,
0x67e, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627,
0x646, 0x6cc, 0x20, 0x634, 0x631, 0x642, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627,
0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc,
0x20, 0x645, 0x6a9, 0x627, 0x646, 0x200c, 0x647, 0x627, 0x6cc, 0x20, 0x62f, 0x6cc,
0x6af, 0x631, 0x20, 0x634, 0x631, 0x642, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627,
0x648, 0x642, 0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x627, 0x631, 0x648, 0x67e,
0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x63a, 0x631,
0x628, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a,
0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x63a, 0x631, 0x628, 0x20,
0x627, 0x631, 0x648, 0x67e, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62c, 0x632, 0x627,
0x6cc, 0x631, 0x20, 0x641, 0x627, 0x644, 0x6a9, 0x644, 0x646, 0x62f, 0x648, 0x642,
0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x62c, 0x632, 0x627, 0x6cc, 0x631,
0x20, 0x641, 0x627, 0x644, 0x6a9, 0x644, 0x646, 0x62f, 0x648, 0x642, 0x62a, 0x20,
0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x62c, 0x632, 0x627,
0x6cc, 0x631, 0x20, 0x641, 0x627, 0x644, 0x6a9, 0x644, 0x646, 0x62f, 0x648, 0x642,
0x62a, 0x20, 0x641, 0x6cc, 0x62c, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627,
0x62f, 0x6cc, 0x20, 0x641, 0x6cc, 0x62c, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x62a,
0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x641, 0x6cc, 0x62c, 0x6cc,
0x648, 0x642, 0x62a, 0x20, 0x6af, 0x648, 0x6cc, 0x627, 0x646, 0x20, 0x641, 0x631,
0x627, 0x646, 0x633, 0x647, 0x648, 0x642, 0x62a, 0x20, 0x633, 0x631, 0x632, 0x645,
0x6cc, 0x646, 0x200c, 0x647, 0x627, 0x6cc, 0x20, 0x62c, 0x646, 0x648, 0x628, 0x6cc,
0x20, 0x648, 0x20, 0x62c, 0x646, 0x648, 0x628, 0x6af, 0x627, 0x646, 0x20, 0x641,
0x631, 0x627, 0x646, 0x633, 0x647, 0x648, 0x642, 0x62a, 0x20, 0x6af, 0x627, 0x644,
0x627, 0x67e, 0x627, 0x6af, 0x648, 0x633, 0x648, 0x642, 0x62a, 0x20, 0x6af, 0x627,
0x645, 0x628, 0x6cc, 0x647, 0x648, 0x642, 0x62a, 0x20, 0x6af, 0x631, 0x62c, 0x633,
0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20,
0x6af, 0x631, 0x62c, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a,
0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x6af, 0x631, 0x62c, 0x633,
0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62c, 0x632, 0x627, 0x6cc, 0x631,
0x20, 0x6af, 0x6cc, 0x644, 0x628, 0x631, 0x62a, 0x648, 0x642, 0x62a, 0x20, 0x6af,
0x631, 0x6cc, 0x646, 0x648, 0x6cc, 0x686, 0x648, 0x642, 0x62a, 0x20, 0x634, 0x631,
0x642, 0x20, 0x6af, 0x631, 0x6cc, 0x646, 0x644, 0x646, 0x62f, 0x648, 0x642, 0x62a,
0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x634, 0x631, 0x642, 0x20, 0x6af, 0x631,
0x6cc, 0x646, 0x644, 0x646, 0x62f, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628,
0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x634, 0x631, 0x642, 0x20, 0x6af, 0x631,
0x6cc, 0x646, 0x644, 0x646, 0x62f, 0x648, 0x642, 0x62a, 0x20, 0x63a, 0x631, 0x628,
0x20, 0x6af, 0x631, 0x6cc, 0x646, 0x644, 0x646, 0x62f, 0x648, 0x642, 0x62a, 0x20,
0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x6af, 0x631, 0x6cc,
0x646, 0x644, 0x646, 0x62f, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633,
0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x6af, 0x631, 0x6cc,
0x646, 0x644, 0x646, 0x62f, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc,
0x20, 0x6af, 0x648, 0x627, 0x645, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f,
0x6cc, 0x20, 0x62e, 0x644, 0x6cc, 0x62c, 0x20, 0x641, 0x627, 0x631, 0x633, 0x648,
0x642, 0x62a, 0x20, 0x647, 0x627, 0x648, 0x627, 0x6cc, 0x6cc, 0x2010, 0x627, 0x644,
0x648, 0x634, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20,
0x647, 0x627, 0x648, 0x627, 0x6cc, 0x6cc, 0x2010, 0x627, 0x644, 0x648, 0x634, 0x646,
0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc,
0x20, 0x647, 0x627, 0x648, 0x627, 0x6cc, 0x6cc, 0x2010, 0x627, 0x644, 0x648, 0x634,
0x646, 0x648, 0x642, 0x62a, 0x20, 0x647, 0x646, 0x6af, 0x200c, 0x6a9, 0x646, 0x6af,
0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x647, 0x646, 0x6af,
0x200c, 0x6a9, 0x646, 0x6af, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633,
0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x647, 0x646, 0x6af, 0x200c, 0x6a9, 0x646, 0x6af,
0x648, 0x642, 0x62a, 0x20, 0x62e, 0x648, 0x648, 0x62f, 0x648, 0x642, 0x62a, 0x20,
0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x62e, 0x648, 0x648, 0x62f, 0x648, 0x642, 0x62a,
0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x62e, 0x648,
0x648, 0x62f, 0x648, 0x642, 0x62a, 0x20, 0x647, 0x646, 0x62f, 0x648, 0x642, 0x62a,
0x20, 0x627, 0x642, 0x6cc, 0x627, 0x646, 0x648, 0x633, 0x20, 0x647, 0x646, 0x62f,
0x648, 0x642, 0x62a, 0x20, 0x647, 0x646, 0x62f, 0x648, 0x686, 0x6cc, 0x646, 0x648,
0x642, 0x62a, 0x20, 0x645, 0x631, 0x6a9, 0x632, 0x20, 0x627, 0x646, 0x62f, 0x648,
0x646, 0x632, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x634, 0x631, 0x642, 0x20, 0x627,
0x646, 0x62f, 0x648, 0x646, 0x632, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x63a, 0x631,
0x628, 0x20, 0x627, 0x646, 0x62f, 0x648, 0x646, 0x632, 0x6cc, 0x648, 0x642, 0x62a,
0x20, 0x627, 0x6cc, 0x631, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627,
0x62f, 0x6cc, 0x20, 0x627, 0x6cc, 0x631, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20,
0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x627, 0x6cc, 0x631,
0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x627, 0x6cc, 0x631, 0x6a9, 0x648, 0x62a,
0x633, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x627,
0x6cc, 0x631, 0x6a9, 0x648, 0x62a, 0x633, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x62a,
0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x627, 0x6cc, 0x631, 0x6a9,
0x648, 0x62a, 0x633, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x627, 0x633, 0x631, 0x627,
0x626, 0x6cc, 0x644, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20,
0x627, 0x633, 0x631, 0x627, 0x626, 0x6cc, 0x644, 0x648, 0x642, 0x62a, 0x20, 0x62a,
0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x627, 0x633, 0x631, 0x627,
0x626, 0x6cc, 0x644, 0x648, 0x642, 0x62a, 0x20, 0x698, 0x627, 0x67e, 0x646, 0x648,
0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x698, 0x627, 0x67e, 0x646,
0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc,
0x20, 0x698, 0x627, 0x67e, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x67e, 0x62a, 0x631,
0x648, 0x67e, 0x627, 0x648, 0x644, 0x648, 0x633, 0x6a9, 0x2010, 0x6a9, 0x627, 0x645,
0x686, 0x627, 0x62a, 0x633, 0x6a9, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627,
0x62f, 0x6cc, 0x20, 0x67e, 0x62a, 0x631, 0x648, 0x67e, 0x627, 0x648, 0x644, 0x648,
0x633, 0x6a9, 0x2010, 0x6a9, 0x627, 0x645, 0x686, 0x627, 0x62a, 0x633, 0x6a9, 0x6cc,
0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc,
0x20, 0x67e, 0x62a, 0x631, 0x648, 0x67e, 0x627, 0x648, 0x644, 0x648, 0x633, 0x6a9,
0x2010, 0x6a9, 0x627, 0x645, 0x686, 0x627, 0x62a, 0x633, 0x6a9, 0x6cc, 0x648, 0x642,
0x62a, 0x20, 0x642, 0x632, 0x627, 0x642, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642,
0x62a, 0x20, 0x634, 0x631, 0x642, 0x20, 0x642, 0x632, 0x627, 0x642, 0x633, 0x62a,
0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x642, 0x632,
0x627, 0x642, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x6a9, 0x631,
0x647, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x6a9, 0x631,
0x647, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646,
0x6cc, 0x20, 0x6a9, 0x631, 0x647, 0x648, 0x642, 0x62a, 0x20, 0x6a9, 0x648, 0x633,
0x631, 0x627, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x6a9, 0x631, 0x627, 0x633, 0x646,
0x648, 0x6cc, 0x627, 0x631, 0x633, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627,
0x62f, 0x6cc, 0x20, 0x6a9, 0x631, 0x627, 0x633, 0x646, 0x648, 0x6cc, 0x627, 0x631,
0x633, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627,
0x646, 0x6cc, 0x20, 0x6a9, 0x631, 0x627, 0x633, 0x646, 0x648, 0x6cc, 0x627, 0x631,
0x633, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x642, 0x631, 0x642, 0x6cc, 0x632, 0x633,
0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x644, 0x627, 0x646, 0x6a9, 0x627,
0x648, 0x642, 0x62a, 0x20, 0x62c, 0x632, 0x627, 0x6cc, 0x631, 0x20, 0x644, 0x627,
0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x644, 0x631, 0x62f, 0x647, 0x648, 0x648,
0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x644, 0x631, 0x62f, 0x647,
0x648, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646,
0x6cc, 0x20, 0x644, 0x631, 0x62f, 0x647, 0x648, 0x648, 0x642, 0x62a, 0x20, 0x645,
0x627, 0x6a9, 0x627, 0x626, 0x648, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f,
0x6cc, 0x20, 0x645, 0x627, 0x6a9, 0x627, 0x626, 0x648, 0x648, 0x642, 0x62a, 0x20,
0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x645, 0x627, 0x6a9,
0x627, 0x626, 0x648, 0x648, 0x642, 0x62a, 0x20, 0x645, 0x627, 0x6af, 0x627, 0x62f,
0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x645,
0x627, 0x6af, 0x627, 0x62f, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627,
0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x645, 0x627, 0x6af, 0x627, 0x62f,
0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x645, 0x627, 0x644, 0x632, 0x6cc, 0x648,
0x642, 0x62a, 0x20, 0x645, 0x627, 0x644, 0x62f, 0x6cc, 0x648, 0x648, 0x642, 0x62a,
0x20, 0x645, 0x627, 0x631, 0x6a9, 0x648, 0x626, 0x632, 0x648, 0x642, 0x62a, 0x20,
0x62c, 0x632, 0x627, 0x6cc, 0x631, 0x20, 0x645, 0x627, 0x631, 0x634, 0x627, 0x644,
0x648, 0x642, 0x62a, 0x20, 0x645, 0x648, 0x631, 0x6cc, 0x633, 0x648, 0x642, 0x62a,
0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x645, 0x648, 0x631, 0x6cc, 0x633, 0x648,
0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20,
0x645, 0x648, 0x631, 0x6cc, 0x633, 0x648, 0x642, 0x62a, 0x20, 0x645, 0x627, 0x648,
0x633, 0x648, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x634, 0x631, 0x642, 0x20, 0x645,
0x6a9, 0x632, 0x6cc, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc,
0x20, 0x634, 0x631, 0x642, 0x20, 0x645, 0x6a9, 0x632, 0x6cc, 0x6a9, 0x648, 0x642,
0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x634,
0x631, 0x642, 0x20, 0x645, 0x6a9, 0x632, 0x6cc, 0x6a9, 0x648, 0x642, 0x62a, 0x20,
0x627, 0x648, 0x644, 0x627, 0x646, 0x200c, 0x628, 0x627, 0x62a, 0x648, 0x631, 0x648,
0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x627, 0x648, 0x644, 0x627,
0x646, 0x200c, 0x628, 0x627, 0x62a, 0x648, 0x631, 0x648, 0x642, 0x62a, 0x20, 0x62a,
0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x627, 0x648, 0x644, 0x627,
0x646, 0x200c, 0x628, 0x627, 0x62a, 0x648, 0x631, 0x648, 0x642, 0x62a, 0x20, 0x645,
0x633, 0x6a9, 0x648, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20,
0x645, 0x633, 0x6a9, 0x648, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633,
0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x645, 0x633, 0x6a9, 0x648, 0x648, 0x642, 0x62a,
0x20, 0x645, 0x6cc, 0x627, 0x646, 0x645, 0x627, 0x631, 0x648, 0x642, 0x62a, 0x20,
0x646, 0x627, 0x626, 0x648, 0x631, 0x648, 0x648, 0x642, 0x62a, 0x20, 0x646, 0x67e,
0x627, 0x644, 0x648, 0x642, 0x62a, 0x20, 0x6a9, 0x627, 0x644, 0x62f, 0x648, 0x646,
0x6cc, 0x627, 0x6cc, 0x20, 0x62c, 0x62f, 0x6cc, 0x62f, 0x648, 0x642, 0x62a, 0x20,
0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x6a9, 0x627, 0x644, 0x62f, 0x648, 0x646, 0x6cc,
0x627, 0x6cc, 0x20, 0x62c, 0x62f, 0x6cc, 0x62f, 0x648, 0x642, 0x62a, 0x20, 0x62a,
0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x6a9, 0x627, 0x644, 0x62f,
0x648, 0x646, 0x6cc, 0x627, 0x6cc, 0x20, 0x62c, 0x62f, 0x6cc, 0x62f, 0x648, 0x642,
0x62a, 0x20, 0x646, 0x6cc, 0x648, 0x632, 0x6cc, 0x644, 0x646, 0x62f, 0x648, 0x642,
0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x646, 0x6cc, 0x648, 0x632, 0x6cc,
0x644, 0x646, 0x62f, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a,
0x627, 0x646, 0x6cc, 0x20, 0x646, 0x6cc, 0x648, 0x632, 0x6cc, 0x644, 0x646, 0x62f,
0x648, 0x642, 0x62a, 0x20, 0x646, 0x6cc, 0x648, 0x641, 0x627, 0x646, 0x62f, 0x644,
0x646, 0x62f, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x646,
0x6cc, 0x648, 0x641, 0x627, 0x646, 0x62f, 0x644, 0x646, 0x62f, 0x648, 0x642, 0x62a,
0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x646, 0x6cc,
0x648, 0x641, 0x627, 0x646, 0x62f, 0x644, 0x646, 0x62f, 0x648, 0x642, 0x62a, 0x20,
0x646, 0x6cc, 0x648, 0x626, 0x647, 0x648, 0x642, 0x62a, 0x20, 0x62c, 0x632, 0x6cc,
0x631, 0x647, 0x654, 0x20, 0x646, 0x648, 0x631, 0x641, 0x648, 0x644, 0x6a9, 0x648,
0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x62c, 0x632, 0x6cc, 0x631,
0x647, 0x654, 0x20, 0x646, 0x648, 0x631, 0x641, 0x648, 0x644, 0x6a9, 0x648, 0x642,
0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x62c,
0x632, 0x6cc, 0x631, 0x647, 0x654, 0x20, 0x646, 0x648, 0x631, 0x641, 0x648, 0x644,
0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x641, 0x631, 0x646, 0x627, 0x646, 0x62f, 0x648,
0x20, 0x62f, 0x6cc, 0x20, 0x646, 0x648, 0x631, 0x648, 0x646, 0x6cc, 0x627, 0x648,
0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x641, 0x631, 0x646, 0x627,
0x646, 0x62f, 0x648, 0x20, 0x62f, 0x6cc, 0x20, 0x646, 0x648, 0x631, 0x648, 0x646,
0x6cc, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627,
0x646, 0x6cc, 0x20, 0x641, 0x631, 0x646, 0x627, 0x646, 0x62f, 0x648, 0x20, 0x62f,
0x6cc, 0x20, 0x646, 0x648, 0x631, 0x648, 0x646, 0x6cc, 0x627, 0x648, 0x642, 0x62a,
0x20, 0x62c, 0x632, 0x627, 0x6cc, 0x631, 0x20, 0x645, 0x627, 0x631, 0x6cc, 0x627,
0x646, 0x627, 0x6cc, 0x20, 0x634, 0x645, 0x627, 0x644, 0x6cc, 0x648, 0x642, 0x62a,
0x20, 0x646, 0x648, 0x648, 0x633, 0x6cc, 0x628, 0x6cc, 0x631, 0x633, 0x6a9, 0x648,
0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x646, 0x648, 0x648, 0x633,
0x6cc, 0x628, 0x6cc, 0x631, 0x633, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627,
0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x646, 0x648, 0x648, 0x633, 0x6cc,
0x628, 0x6cc, 0x631, 0x633, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x627, 0x648, 0x645,
0x633, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x627,
0x648, 0x645, 0x633, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633,
0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x627, 0x648, 0x645, 0x633, 0x6a9, 0x648, 0x642,
0x62a, 0x20, 0x67e, 0x627, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a,
0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x67e, 0x627, 0x6a9, 0x633, 0x62a, 0x627,
0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646,
0x6cc, 0x20, 0x67e, 0x627, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a,
0x20, 0x67e, 0x627, 0x644, 0x627, 0x626, 0x648, 0x648, 0x642, 0x62a, 0x20, 0x67e,
0x627, 0x67e, 0x648, 0x627, 0x20, 0x6af, 0x6cc, 0x646, 0x647, 0x654, 0x20, 0x646,
0x648, 0x648, 0x642, 0x62a, 0x20, 0x67e, 0x627, 0x631, 0x627, 0x6af, 0x648, 0x626,
0x647, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x67e, 0x627,
0x631, 0x627, 0x6af, 0x648, 0x626, 0x647, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627,
0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x67e, 0x627, 0x631, 0x627, 0x6af,
0x648, 0x626, 0x647, 0x648, 0x642, 0x62a, 0x20, 0x67e, 0x631, 0x648, 0x648, 0x642,
0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x67e, 0x631, 0x648, 0x648, 0x642,
0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x67e,
0x631, 0x648, 0x648, 0x642, 0x62a, 0x20, 0x641, 0x6cc, 0x644, 0x6cc, 0x67e, 0x6cc,
0x646, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x641, 0x6cc,
0x644, 0x6cc, 0x67e, 0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628,
0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x641, 0x6cc, 0x644, 0x6cc, 0x67e, 0x6cc,
0x646, 0x648, 0x642, 0x62a, 0x20, 0x62c, 0x632, 0x627, 0x6cc, 0x631, 0x20, 0x641,
0x648, 0x646, 0x6cc, 0x6a9, 0x633, 0x648, 0x642, 0x62a, 0x20, 0x633, 0x646, 0x62a,
0x200c, 0x67e, 0x6cc, 0x631, 0x20, 0x648, 0x20, 0x645, 0x6cc, 0x6a9, 0x644, 0x648,
0x646, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x633, 0x646,
0x62a, 0x200c, 0x67e, 0x6cc, 0x631, 0x20, 0x648, 0x20, 0x645, 0x6cc, 0x6a9, 0x644,
0x648, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627,
0x646, 0x6cc, 0x20, 0x633, 0x646, 0x62a, 0x200c, 0x67e, 0x6cc, 0x631, 0x20, 0x648,
0x20, 0x645, 0x6cc, 0x6a9, 0x644, 0x648, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x67e,
0x6cc, 0x62a, 0x6a9, 0x627, 0x6cc, 0x631, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x67e,
0x648, 0x646, 0x627, 0x67e, 0x647, 0x648, 0x642, 0x62a, 0x20, 0x67e, 0x6cc, 0x648,
0x646, 0x6af, 0x200c, 0x6cc, 0x627, 0x646, 0x6af, 0x648, 0x642, 0x62a, 0x20, 0x642,
0x6cc, 0x632, 0x6cc, 0x644, 0x200c, 0x627, 0x648, 0x631, 0x62f, 0x627, 0x648, 0x642,
0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x642, 0x6cc, 0x632, 0x6cc, 0x644,
0x200c, 0x627, 0x648, 0x631, 0x62f, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627,
0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x642, 0x6cc, 0x632, 0x6cc, 0x644,
0x200c, 0x627, 0x648, 0x631, 0x62f, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x631, 0x626,
0x648, 0x646, 0x6cc, 0x648, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x631, 0x648, 0x62a,
0x631, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x633, 0x627, 0x62e, 0x627, 0x644, 0x6cc,
0x646, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x633, 0x627,
0x62e, 0x627, 0x644, 0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628,
0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x633, 0x627, 0x62e, 0x627, 0x644, 0x6cc,
0x646, 0x648, 0x642, 0x62a, 0x20, 0x633, 0x627, 0x645, 0x627, 0x631, 0x627, 0x648,
0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x633, 0x627, 0x645, 0x627,
0x631, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627,
0x646, 0x6cc, 0x20, 0x633, 0x627, 0x645, 0x627, 0x631, 0x627, 0x648, 0x642, 0x62a,
0x20, 0x633, 0x627, 0x645, 0x648, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627,
0x62f, 0x6cc, 0x20, 0x633, 0x627, 0x645, 0x648, 0x627, 0x648, 0x642, 0x62a, 0x20,
0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x633, 0x627, 0x645,
0x648, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x633, 0x6cc, 0x634, 0x644, 0x648, 0x642,
0x62a, 0x20, 0x633, 0x646, 0x6af, 0x627, 0x67e, 0x648, 0x631, 0x648, 0x642, 0x62a,
0x20, 0x62c, 0x632, 0x627, 0x6cc, 0x631, 0x20, 0x633, 0x644, 0x6cc, 0x645, 0x627,
0x646, 0x648, 0x642, 0x62a, 0x20, 0x62c, 0x648, 0x631, 0x62c, 0x6cc, 0x627, 0x6cc,
0x20, 0x62c, 0x646, 0x648, 0x628, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x633, 0x648,
0x631, 0x6cc, 0x646, 0x627, 0x645, 0x648, 0x642, 0x62a, 0x20, 0x634, 0x648, 0x648,
0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x647, 0x6cc, 0x62a, 0x6cc, 0x648,
0x642, 0x62a, 0x20, 0x62a, 0x627, 0x6cc, 0x67e, 0x647, 0x648, 0x642, 0x62a, 0x20,
0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x62a, 0x627, 0x6cc, 0x67e, 0x647, 0x648, 0x642,
0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x62a,
0x627, 0x6cc, 0x67e, 0x647, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x62c, 0x6cc,
0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x648, 0x6a9,
0x644, 0x627, 0x626, 0x648, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x648, 0x646, 0x6af,
0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x62a, 0x648,
0x646, 0x6af, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a,
0x627, 0x646, 0x6cc, 0x20, 0x62a, 0x648, 0x646, 0x6af, 0x627, 0x648, 0x642, 0x62a,
0x20, 0x686, 0x648, 0x626, 0x648, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x631,
0x6a9, 0x645, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x639,
0x627, 0x62f, 0x6cc, 0x20, 0x62a, 0x631, 0x6a9, 0x645, 0x646, 0x633, 0x62a, 0x627,
0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646,
0x6cc, 0x20, 0x62a, 0x631, 0x6a9, 0x645, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x648,
0x642, 0x62a, 0x20, 0x62a, 0x648, 0x648, 0x627, 0x644, 0x648, 0x648, 0x642, 0x62a,
0x20, 0x627, 0x631, 0x648, 0x6af, 0x648, 0x626, 0x647, 0x648, 0x642, 0x62a, 0x20,
0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x627, 0x631, 0x648, 0x6af, 0x648, 0x626, 0x647,
0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc,
0x20, 0x627, 0x631, 0x648, 0x6af, 0x648, 0x626, 0x647, 0x648, 0x642, 0x62a, 0x20,
0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20,
0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a, 0x627,
0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646,
0x6cc, 0x20, 0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642,
0x62a, 0x20, 0x648, 0x627, 0x646, 0x627, 0x62a, 0x648, 0x648, 0x642, 0x62a, 0x20,
0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x648, 0x627, 0x646, 0x627, 0x62a, 0x648, 0x648,
0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20,
0x648, 0x627, 0x646, 0x627, 0x62a, 0x648, 0x648, 0x642, 0x62a, 0x20, 0x648, 0x646,
0x632, 0x648, 0x626, 0x644, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x648, 0x644, 0x627,
0x62f, 0x6cc, 0x200c, 0x648, 0x633, 0x62a, 0x648, 0x6a9, 0x648, 0x642, 0x62a, 0x20,
0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x648, 0x644, 0x627, 0x62f, 0x6cc, 0x200c, 0x648,
0x633, 0x62a, 0x648, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633,
0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x648, 0x644, 0x627, 0x62f, 0x6cc, 0x200c, 0x648,
0x633, 0x62a, 0x648, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x648, 0x644, 0x6af, 0x627,
0x6af, 0x631, 0x627, 0x62f, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc,
0x20, 0x648, 0x644, 0x6af, 0x627, 0x6af, 0x631, 0x627, 0x62f, 0x648, 0x642, 0x62a,
0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x648, 0x644,
0x6af, 0x627, 0x6af, 0x631, 0x627, 0x62f, 0x648, 0x642, 0x62a, 0x20, 0x648, 0x633,
0x62a, 0x648, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x62c, 0x632, 0x6cc, 0x631, 0x647,
0x654, 0x20, 0x648, 0x6cc, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x648, 0x627, 0x644,
0x6cc, 0x633, 0x20, 0x648, 0x20, 0x641, 0x648, 0x62a, 0x648, 0x646, 0x627, 0x648,
0x642, 0x62a, 0x20, 0x6cc, 0x627, 0x6a9, 0x648, 0x62a, 0x633, 0x6a9, 0x648, 0x642,
0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x6cc, 0x627, 0x6a9, 0x648, 0x62a,
0x633, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627,
0x646, 0x6cc, 0x20, 0x6cc, 0x627, 0x6a9, 0x648, 0x62a, 0x633, 0x6a9, 0x648, 0x642,
0x62a, 0x20, 0x6cc, 0x6a9, 0x627, 0x62a, 0x631, 0x6cc, 0x646, 0x628, 0x648, 0x631,
0x6af, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x6cc, 0x6a9,
0x627, 0x62a, 0x631, 0x6cc, 0x646, 0x628, 0x648, 0x631, 0x6af, 0x648, 0x642, 0x62a,
0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x6cc, 0x6a9,
0x627, 0x62a, 0x631, 0x6cc, 0x646, 0x628, 0x648, 0x631, 0x6af, 0x648, 0x642, 0x62a,
0x20, 0x6cc, 0x648, 0x6a9, 0x627, 0x646, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x41,
0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x15b, 0x72, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x61, 0x66,
0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61, 0x66,
0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x70, 0x6f, 0x142, 0x75, 0x64, 0x6e, 0x69, 0x6f, 0x77, 0x6f, 0x61,
0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61,
0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61,
0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61,
0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x29, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x61, 0x142, 0x6d, 0x61, 0x63, 0x6b, 0x69, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x61, 0x142, 0x6d, 0x61, 0x63, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x61, 0x142, 0x6d, 0x61, 0x63, 0x6b, 0x69, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x6d, 0x61,
0x7a, 0x6f, 0x144, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x61,
0x6d, 0x61, 0x7a, 0x6f, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x61, 0x6d, 0x61, 0x7a, 0x6f, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x15b, 0x72, 0x6f,
0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x79, 0x6b, 0x61,
0x144, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x15b, 0x72, 0x6f,
0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x79, 0x6b, 0x61,
0x144, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x15b, 0x72, 0x6f,
0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x79, 0x6b, 0x61,
0x144, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f,
0x61, 0x6d, 0x65, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69,
0x6f, 0x61, 0x6d, 0x65, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e,
0x69, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x67, 0xf3, 0x72, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x67,
0xf3, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x67, 0xf3,
0x72, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x70, 0x61, 0x63, 0x79, 0x66, 0x69, 0x63, 0x7a, 0x6e,
0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x70, 0x61, 0x63, 0x79, 0x66, 0x69,
0x63, 0x7a, 0x6e, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x70, 0x61, 0x63,
0x79, 0x66, 0x69, 0x63, 0x7a, 0x6e, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x41, 0x70, 0x69,
0x61, 0x41, 0x70, 0x69, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29,
0x41, 0x70, 0x69, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x75,
0x6b, 0x74, 0x61, 0x75, 0x63, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x61, 0x75, 0x6b, 0x74, 0x61, 0x75, 0x63, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x61, 0x75, 0x6b, 0x74, 0x61, 0x75, 0x63, 0x6b, 0x69, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x6b,
0x74, 0x69, 0x75, 0x62, 0x69, 0x144, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x61, 0x6b, 0x74, 0x69, 0x75, 0x62, 0x69, 0x144, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x6b, 0x74, 0x69, 0x75, 0x62,
0x69, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x50, 0xf3, 0x142, 0x77, 0x79, 0x73, 0x65, 0x70,
0x20, 0x41, 0x72, 0x61, 0x62, 0x73, 0x6b, 0x69, 0x50, 0xf3, 0x142, 0x77,
0x79, 0x73, 0x65, 0x70, 0x20, 0x41, 0x72, 0x61, 0x62, 0x73, 0x6b, 0x69,
0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x50, 0xf3, 0x142, 0x77, 0x79,
0x73, 0x65, 0x70, 0x20, 0x41, 0x72, 0x61, 0x62, 0x73, 0x6b, 0x69, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x79,
0x6e, 0x61, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x79, 0x6e, 0x61, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x79, 0x6e, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x79, 0x6e, 0x61, 0x20, 0x5a, 0x61, 0x63, 0x68, 0x6f,
0x64, 0x6e, 0x69, 0x61, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x79, 0x6e,
0x61, 0x20, 0x5a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x61, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x79, 0x6e, 0x61, 0x20, 0x5a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69,
0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e,
0x69, 0x61, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x29, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x79,
0x63, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x74, 0x6c, 0x61,
0x6e, 0x74, 0x79, 0x63, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x61,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x79, 0x63, 0x6b, 0x69, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x15b, 0x72, 0x6f, 0x64,
0x6b, 0x6f, 0x77, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x6a, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x15b, 0x72, 0x6f,
0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x15b, 0x72,
0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x15b, 0x72, 0x6f, 0x64, 0x6b, 0x6f, 0x77,
0x6f, 0x2d, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x15b, 0x72, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f,
0x2d, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x15b, 0x72, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x2d,
0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x77, 0x73, 0x63, 0x68,
0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x6a, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x77, 0x73,
0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x7a, 0x61, 0x63, 0x68,
0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x6a, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x7a, 0x61,
0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x65, 0x6a, 0x64, 0x17c, 0x61, 0x6e, 0x41, 0x7a, 0x65, 0x72, 0x62,
0x65, 0x6a, 0x64, 0x17c, 0x61, 0x6e, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x29, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65, 0x6a, 0x64, 0x17c, 0x61, 0x6e,
0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x79, 0x41,
0x7a, 0x6f, 0x72, 0x79, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x41,
0x7a, 0x6f, 0x72, 0x79, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x7a, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x7a, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x29, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x7a, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x42, 0x6f, 0x6c, 0x69, 0x77, 0x69, 0x61, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61,
0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x29, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0x5a, 0x69,
0x65, 0x6c, 0x6f, 0x6e, 0x65, 0x67, 0x6f, 0x20, 0x50, 0x72, 0x7a, 0x79,
0x6c, 0x105, 0x64, 0x6b, 0x61, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0x5a,
0x69, 0x65, 0x6c, 0x6f, 0x6e, 0x65, 0x67, 0x6f, 0x20, 0x50, 0x72, 0x7a,
0x79, 0x6c, 0x105, 0x64, 0x6b, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x29, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0x5a, 0x69, 0x65, 0x6c, 0x6f,
0x6e, 0x65, 0x67, 0x6f, 0x20, 0x50, 0x72, 0x7a, 0x79, 0x6c, 0x105, 0x64,
0x6b, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x43, 0x7a, 0x61, 0x6d,
0x6f, 0x72, 0x72, 0x6f, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x43, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65,
0x43, 0x68, 0x69, 0x6c, 0x65, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29,
0x43, 0x68, 0x69, 0x6c, 0x65, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x43,
0x68, 0x69, 0x6e, 0x79, 0x43, 0x68, 0x69, 0x6e, 0x79, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x29, 0x43, 0x68, 0x69, 0x6e, 0x79, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x57, 0x79, 0x73, 0x70, 0x61, 0x20, 0x42, 0x6f, 0x17c,
0x65, 0x67, 0x6f, 0x20, 0x4e, 0x61, 0x72, 0x6f, 0x64, 0x7a, 0x65, 0x6e,
0x69, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x57, 0x79, 0x73, 0x70, 0x79,
0x20, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x6f, 0x77, 0x65, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61, 0x4b, 0x6f,
0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x29, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0x43, 0x6f, 0x6f,
0x6b, 0x61, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0x43, 0x6f, 0x6f, 0x6b,
0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x57, 0x79, 0x73, 0x70,
0x79, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x4b, 0x75, 0x62, 0x61, 0x4b, 0x75, 0x62, 0x61, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x29, 0x4b, 0x75, 0x62, 0x61, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76,
0x69, 0x6c, 0x6c, 0x65, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x54, 0x69, 0x6d,
0x6f, 0x72, 0x20, 0x57, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x57, 0x79, 0x73, 0x70, 0x61, 0x20, 0x57, 0x69,
0x65, 0x6c, 0x6b, 0x61, 0x6e, 0x6f, 0x63, 0x6e, 0x61, 0x57, 0x79, 0x73,
0x70, 0x61, 0x20, 0x57, 0x69, 0x65, 0x6c, 0x6b, 0x61, 0x6e, 0x6f, 0x63,
0x6e, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x57, 0x79, 0x73,
0x70, 0x61, 0x20, 0x57, 0x69, 0x65, 0x6c, 0x6b, 0x61, 0x6e, 0x6f, 0x63,
0x6e, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x45, 0x6b, 0x77, 0x61,
0x64, 0x6f, 0x72, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x15b, 0x72, 0x6f, 0x64,
0x6b, 0x6f, 0x77, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73,
0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x15b, 0x72, 0x6f, 0x64, 0x6b,
0x6f, 0x77, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x15b, 0x72, 0x6f, 0x64, 0x6b, 0x6f,
0x77, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x77,
0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x65, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x77,
0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x65, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x77,
0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x65, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e,
0x69, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x64, 0x61, 0x6c, 0x73, 0x7a, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x65, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x65, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x65, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x79, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x79,
0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x79, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x46, 0x69,
0x64, 0x17c, 0x69, 0x46, 0x69, 0x64, 0x17c, 0x69, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x29, 0x46, 0x69, 0x64, 0x17c, 0x69, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x47, 0x75, 0x6a, 0x61, 0x6e, 0x61, 0x20, 0x46, 0x72, 0x61,
0x6e, 0x63, 0x75, 0x73, 0x6b, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x46,
0x72, 0x61, 0x6e, 0x63, 0x75, 0x73, 0x6b, 0x69, 0x65, 0x20, 0x54, 0x65,
0x72, 0x79, 0x74, 0x6f, 0x72, 0x69, 0x61, 0x20, 0x50, 0x6f, 0x142, 0x75,
0x64, 0x6e, 0x69, 0x6f, 0x77, 0x65, 0x20, 0x69, 0x20, 0x41, 0x6e, 0x74,
0x61, 0x72, 0x6b, 0x74, 0x79, 0x63, 0x7a, 0x6e, 0x65, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0x47, 0x61,
0x6d, 0x62, 0x69, 0x65, 0x72, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x47,
0x72, 0x75, 0x7a, 0x6a, 0x61, 0x47, 0x72, 0x75, 0x7a, 0x6a, 0x61, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x47, 0x72, 0x75, 0x7a, 0x6a, 0x61,
0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20,
0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x61, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x75, 0x6e, 0x69, 0x77, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x6e, 0x79,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x61, 0x20, 0x57, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69,
0x61, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20,
0x57, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x61, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x29, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x61, 0x20, 0x57, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x61,
0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x5a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e,
0x69, 0x61, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61,
0x20, 0x5a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x61, 0x20, 0x28,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x29, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x61, 0x20, 0x5a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69,
0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x5a, 0x61, 0x74, 0x6f, 0x6b,
0x61, 0x20, 0x50, 0x65, 0x72, 0x73, 0x6b, 0x61, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x48, 0x61, 0x77, 0x61, 0x6a, 0x65, 0x2d, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x79, 0x48, 0x61, 0x77, 0x61, 0x6a, 0x65, 0x2d, 0x41, 0x6c, 0x65,
0x75, 0x74, 0x79, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x48, 0x61,
0x77, 0x61, 0x6a, 0x65, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x79, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e,
0x67, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x29, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67,
0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4b, 0x6f, 0x62, 0x64, 0x6f, 0x4b,
0x6f, 0x62, 0x64, 0x6f, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x4b,
0x6f, 0x62, 0x64, 0x6f, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x69, 0x6e,
0x64, 0x79, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4f,
0x63, 0x65, 0x61, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x79, 0x6a, 0x73, 0x6b,
0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x69, 0x6e, 0x64, 0x6f, 0x63, 0x68,
0x69, 0x144, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x6a, 0x61, 0x20, 0x15a, 0x72, 0x6f, 0x64,
0x6b, 0x6f, 0x77, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x49, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x7a, 0x6a, 0x61, 0x20, 0x57, 0x73, 0x63, 0x68, 0x6f,
0x64, 0x6e, 0x69, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x49, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x7a, 0x6a, 0x61, 0x20, 0x5a, 0x61, 0x63, 0x68, 0x6f,
0x64, 0x6e, 0x69, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x49, 0x72, 0x61,
0x6e, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29,
0x49, 0x72, 0x61, 0x6e, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x49, 0x72,
0x6b, 0x75, 0x63, 0x6b, 0x49, 0x72, 0x6b, 0x75, 0x63, 0x6b, 0x20, 0x28,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x29, 0x49, 0x72, 0x6b, 0x75, 0x63, 0x6b, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x49,
0x7a, 0x72, 0x61, 0x65, 0x6c, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29,
0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x69, 0x61, 0x4a, 0x61, 0x70, 0x6f, 0x6e,
0x69, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x4a, 0x61, 0x70,
0x6f, 0x6e, 0x69, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x50, 0x69,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x77, 0x142, 0x6f, 0x77, 0x73, 0x6b,
0x20, 0x4b, 0x61, 0x6d, 0x63, 0x7a, 0x61, 0x63, 0x6b, 0x69, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x20, 0x50, 0x69, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x77,
0x142, 0x6f, 0x77, 0x73, 0x6b, 0x20, 0x4b, 0x61, 0x6d, 0x63, 0x7a, 0x61,
0x63, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x50, 0x69, 0x65, 0x74,
0x72, 0x6f, 0x70, 0x61, 0x77, 0x142, 0x6f, 0x77, 0x73, 0x6b, 0x20, 0x4b,
0x61, 0x6d, 0x63, 0x7a, 0x61, 0x63, 0x6b, 0x69, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x63,
0x68, 0x73, 0x74, 0x61, 0x6e, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4b, 0x61,
0x7a, 0x61, 0x63, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x57, 0x73, 0x63,
0x68, 0x6f, 0x64, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4b, 0x61,
0x7a, 0x61, 0x63, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x5a, 0x61, 0x63,
0x68, 0x6f, 0x64, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4b, 0x6f,
0x72, 0x65, 0x61, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x29, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x6a, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4b,
0x69, 0x72, 0x67, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x4c, 0x69, 0x6e, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x65,
0x7a, 0x6a, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x65,
0x64, 0x69, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72,
0x6b, 0x69, 0x7a, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x57, 0x79, 0x73,
0x70, 0x79, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x61,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69,
0x75, 0x73, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74,
0x69, 0x75, 0x73, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x77,
0x73, 0x6f, 0x6e, 0x4d, 0x65, 0x6b, 0x73, 0x79, 0x6b, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x70, 0x61, 0x63, 0x79, 0x66, 0x69, 0x63, 0x7a,
0x6e, 0x79, 0x29, 0x4d, 0x65, 0x6b, 0x73, 0x79, 0x6b, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x70, 0x61, 0x63, 0x79, 0x66, 0x69, 0x63, 0x7a,
0x6e, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x29, 0x4d, 0x65, 0x6b, 0x73, 0x79, 0x6b, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x70, 0x61, 0x63, 0x79, 0x66, 0x69, 0x63, 0x7a,
0x6e, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x55, 0x142, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72,
0x55, 0x142, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x28,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x29, 0x55, 0x142, 0x61, 0x6e, 0x20, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4d, 0x6f, 0x73,
0x6b, 0x77, 0x61, 0x4d, 0x6f, 0x73, 0x6b, 0x77, 0x61, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x29, 0x4d, 0x6f, 0x73, 0x6b, 0x77, 0x61, 0x20, 0x28,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x4d, 0x6a, 0x61, 0x6e, 0x6d, 0x61, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4e,
0x6f, 0x77, 0x61, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69,
0x61, 0x4e, 0x6f, 0x77, 0x61, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x69, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x4e, 0x6f,
0x77, 0x61, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61,
0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4e, 0x6f, 0x77, 0x61, 0x20, 0x5a,
0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x4e, 0x6f, 0x77, 0x61, 0x20,
0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x29, 0x4e, 0x6f, 0x77, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4e, 0x6f,
0x77, 0x61, 0x20, 0x46, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x61, 0x4e, 0x6f, 0x77, 0x61, 0x20, 0x46, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x4e,
0x6f, 0x77, 0x61, 0x20, 0x46, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4e, 0x69, 0x75, 0x65,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x28, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x29, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x79, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x79, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x79,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4f,
0x6d, 0x73, 0x6b, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x28, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x29, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x50, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x28, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x50,
0x61, 0x70, 0x75, 0x61, 0x2d, 0x4e, 0x6f, 0x77, 0x61, 0x20, 0x47, 0x77,
0x69, 0x6e, 0x65, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x50, 0x61, 0x72,
0x61, 0x67, 0x77, 0x61, 0x6a, 0x50, 0x61, 0x72, 0x61, 0x67, 0x77, 0x61,
0x6a, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x50, 0x61, 0x72, 0x61,
0x67, 0x77, 0x61, 0x6a, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x50, 0x65,
0x72, 0x75, 0x50, 0x65, 0x72, 0x75, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x29, 0x50, 0x65, 0x72, 0x75, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x46,
0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x79, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x69, 0x6e, 0x79, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x6e, 0x79, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x46, 0x65, 0x6e, 0x69, 0x6b, 0x73, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20,
0x69, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x53, 0x61,
0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x69,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x29, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x20, 0x69, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x50, 0x69, 0x74,
0x63, 0x61, 0x69, 0x72, 0x6e, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x50, 0x6f,
0x68, 0x6e, 0x70, 0x65, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x50, 0x6a,
0x6f, 0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x67, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x6b, 0x79, 0x7a, 0x79, 0x142, 0x6f, 0x72, 0x64, 0x7a, 0x6b, 0x69, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x6b, 0x79, 0x7a, 0x79, 0x142, 0x6f, 0x72, 0x64,
0x7a, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6b, 0x79, 0x7a, 0x79,
0x142, 0x6f, 0x72, 0x64, 0x7a, 0x6b, 0x69, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f,
0x6e, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72,
0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x28,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x29, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x29, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x53, 0x65, 0x73, 0x7a, 0x65, 0x6c, 0x65, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0x53, 0x61, 0x6c,
0x6f, 0x6d, 0x6f, 0x6e, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x47, 0x65,
0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x50, 0x6f, 0x142, 0x75, 0x64, 0x6e,
0x69, 0x6f, 0x77, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x53, 0x75, 0x72,
0x69, 0x6e, 0x61, 0x6d, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x53, 0x79, 0x6f,
0x77, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74,
0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a,
0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x29, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x20, 0x28, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x54, 0x61, 0x64, 0x17c, 0x79, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x43, 0x68, 0x75,
0x75, 0x6b, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x54, 0x75, 0x72, 0x6b, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x28, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x29, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x54, 0x75, 0x76, 0x61,
0x6c, 0x75, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x55, 0x72, 0x75, 0x67, 0x77,
0x61, 0x6a, 0x55, 0x72, 0x75, 0x67, 0x77, 0x61, 0x6a, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x29, 0x55, 0x72, 0x75, 0x67, 0x77, 0x61, 0x6a, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x55, 0x7a, 0x62, 0x65,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x57, 0x65,
0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x57, 0x142, 0x61, 0x64, 0x79, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x57,
0x142, 0x61, 0x64, 0x79, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x28,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x29, 0x57, 0x142, 0x61, 0x64, 0x79, 0x77, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x57, 0x6f,
0x142, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x57, 0x6f, 0x142, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x57,
0x6f, 0x142, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x57, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x57, 0x61, 0x6b, 0x65, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x57, 0x61,
0x6c, 0x6c, 0x69, 0x73, 0x20, 0x69, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e,
0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4a, 0x61, 0x6b, 0x75, 0x63, 0x6b,
0x4a, 0x61, 0x6b, 0x75, 0x63, 0x6b, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x29, 0x4a, 0x61, 0x6b, 0x75, 0x63, 0x6b, 0x20, 0x28, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x79, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x79, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x4a, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x79, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x4a, 0x75, 0x6b, 0x6f, 0x6e, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x63, 0x72,
0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x63, 0x72, 0x65, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x63, 0x72, 0x65, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x66,
0x65, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0xc1, 0x66, 0x72, 0x69,
0x63, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0xc1, 0x66, 0x72,
0x69, 0x63, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0xc1,
0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x53, 0x75, 0x6c,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0xc1,
0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0xc1, 0x66, 0x72,
0x69, 0x63, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0xc1, 0x66, 0x72,
0x69, 0x63, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x41, 0x6c, 0x61, 0x73, 0x63, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x41, 0x6c, 0x61, 0x73, 0x63, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x6f, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x63, 0x61, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74,
0x79, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74,
0x79, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x6c, 0x6d,
0x61, 0x74, 0x79, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x6f, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x6f, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x6f, 0x20, 0x4c, 0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x4c, 0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x6f, 0x20, 0x4c, 0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x4d, 0x6f, 0x6e,
0x74, 0x61, 0x6e, 0x68, 0x61, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x73,
0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x68, 0x61, 0x73, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61,
0x6e, 0x68, 0x61, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63,
0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x63,
0xed, 0x66, 0x69, 0x63, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x6f, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41,
0x70, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69,
0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69,
0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x41, 0x71, 0x74, 0x61, 0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41,
0x71, 0x74, 0x61, 0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x41, 0x71, 0x74, 0x61, 0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x6f, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x6f, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41,
0x72, 0xe1, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41,
0x72, 0xe1, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61,
0x20, 0x41, 0x72, 0xe1, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x4f, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20,
0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0x6d, 0xea,
0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0x6d,
0xea, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20,
0x41, 0x72, 0x6d, 0xea, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x74, 0x6c, 0xe2, 0x6e, 0x74,
0x69, 0x63, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x74, 0x6c,
0xe2, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x6f, 0x20, 0x41, 0x74, 0x6c, 0xe2, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x2d, 0x4f, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x6f, 0x2d, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x6f,
0x2d, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69,
0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c,
0x69, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x6f, 0x20, 0x41, 0x72, 0x7a, 0x65, 0x69, 0x62, 0x61, 0x69, 0x6a, 0xe3,
0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x72, 0x7a, 0x65, 0x69,
0x62, 0x61, 0x69, 0x6a, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x6f, 0x20, 0x41, 0x72, 0x7a, 0x65, 0x69, 0x62, 0x61, 0x69, 0x6a, 0xe3,
0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x73,
0x20, 0x41, 0xe7, 0x6f, 0x72, 0x65, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f,
0x73, 0x20, 0x41, 0xe7, 0x6f, 0x72, 0x65, 0x73, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x6f, 0x73, 0x20, 0x41, 0xe7, 0x6f, 0x72, 0x65, 0x73, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x6f, 0x20, 0x42, 0x75, 0x74, 0xe3, 0x6f, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x42, 0x6f, 0x6c,
0xed, 0x76, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69,
0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61,
0x73, 0xed, 0x6c, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44,
0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x62, 0x6f,
0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x43, 0x61, 0x62, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x62, 0x6f, 0x20, 0x56,
0x65, 0x72, 0x64, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x6c,
0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x6c,
0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20,
0x43, 0x68, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x43,
0x68, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20,
0x43, 0x68, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x61, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x20, 0x43, 0x68, 0x72,
0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20,
0x43, 0x6f, 0x63, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0xf4, 0x6d, 0x62, 0x69, 0x61, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0xf4, 0x6d, 0x62, 0x69,
0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x6c,
0xf4, 0x6d, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x43,
0x6f, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c,
0x68, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x4d, 0x65, 0x69, 0x6f,
0x20, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c,
0x68, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x44, 0x61, 0x76, 0x69,
0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76,
0x69, 0x6c, 0x6c, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x2d, 0x4c, 0x65, 0x73,
0x74, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61,
0x20, 0x49, 0x6c, 0x68, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0xe1, 0x73,
0x63, 0x6f, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6c, 0x68,
0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0xe1, 0x73, 0x63, 0x6f, 0x61, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x20,
0x64, 0x65, 0x20, 0x50, 0xe1, 0x73, 0x63, 0x6f, 0x61, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x45, 0x71, 0x75, 0x61,
0x64, 0x6f, 0x72, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x45, 0x78, 0x74, 0x72, 0x65,
0x6d, 0x6f, 0x20, 0x4c, 0x65, 0x73, 0x74, 0x65, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x4d,
0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61,
0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76,
0x69, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x73,
0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69,
0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x46, 0x69, 0x6a, 0x69, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x46, 0x69, 0x6a, 0x69, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x61, 0x20, 0x47, 0x75, 0x69, 0x61, 0x6e, 0x61, 0x20, 0x46, 0x72,
0x61, 0x6e, 0x63, 0x65, 0x73, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x6f, 0x73, 0x20, 0x54, 0x65, 0x72, 0x72, 0x69, 0x74,
0xf3, 0x72, 0x69, 0x6f, 0x73, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65,
0x73, 0x65, 0x73, 0x20, 0x64, 0x6f, 0x20, 0x53, 0x75, 0x6c, 0x20, 0x65,
0x20, 0x41, 0x6e, 0x74, 0xe1, 0x72, 0x74, 0x69, 0x64, 0x61, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x47, 0x61, 0x6c,
0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x47,
0x65, 0xf3, 0x72, 0x67, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20,
0x47, 0x65, 0xf3, 0x72, 0x67, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x61, 0x20, 0x47, 0x65, 0xf3, 0x72, 0x67, 0x69, 0x61, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c,
0x68, 0x61, 0x73, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x6f,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x4d,
0x65, 0x72, 0x69, 0x64, 0x69, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65,
0x6c, 0xe2, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x47, 0x72, 0x6f,
0x65, 0x6c, 0xe2, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20,
0x47, 0x72, 0x6f, 0x65, 0x6c, 0xe2, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c,
0xe2, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x47, 0x72, 0x6f,
0x65, 0x6e, 0x6c, 0xe2, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe2, 0x6e, 0x64, 0x69,
0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x47, 0x75, 0x61, 0x6d, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x47, 0x6f, 0x6c, 0x66,
0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x48, 0x61, 0x76, 0x61, 0xed, 0x20, 0x65, 0x20, 0x49, 0x6c, 0x68, 0x61,
0x73, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x61, 0x73, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x48, 0x61, 0x76, 0x61, 0xed, 0x20, 0x65, 0x20, 0x49,
0x6c, 0x68, 0x61, 0x73, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x61, 0x73,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x48, 0x61, 0x76, 0x61,
0xed, 0x20, 0x65, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x61, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e,
0x67, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6e, 0x67, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x61, 0x20, 0xcd, 0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x4f, 0x63, 0x65, 0x61,
0x6e, 0x6f, 0x20, 0xcd, 0x6e, 0x64, 0x69, 0x63, 0x6f, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f,
0x63, 0x68, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0xe9, 0x73, 0x69,
0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f,
0x6e, 0xe9, 0x73, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0xe9, 0x73, 0x69, 0x61, 0x20, 0x4f,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x49, 0x72, 0xe3, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x6f, 0x20, 0x49, 0x72, 0xe3, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x49, 0x72, 0xe3, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x4a, 0x61,
0x70, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x4a, 0x61, 0x70,
0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x4a, 0x61,
0x70, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73,
0x6b, 0x69, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72,
0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61,
0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76,
0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61,
0x74, 0x73, 0x6b, 0x69, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x43, 0x61, 0x7a, 0x61, 0x71, 0x75, 0x69, 0x73, 0x74,
0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f,
0x20, 0x43, 0x61, 0x7a, 0x61, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f,
0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x43, 0x61, 0x7a, 0x61,
0x71, 0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x20, 0x4f, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x69, 0x61, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x69, 0x61, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x69, 0x61,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4b,
0x6f, 0x73, 0x72, 0x61, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x6f, 0x20, 0x51, 0x75, 0x69, 0x72, 0x67, 0x75, 0x69, 0x73, 0x74, 0xe3,
0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x64,
0x61, 0x20, 0x4c, 0x69, 0x6e, 0x68, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61,
0x63, 0x61, 0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x63,
0x61, 0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61,
0x63, 0x61, 0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20,
0x4d, 0x61, 0x6c, 0xe1, 0x73, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73,
0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x61, 0x73, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72,
0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20,
0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0xed,
0x63, 0x69, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75,
0x72, 0xed, 0x63, 0x69, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x61, 0x75, 0x72, 0xed, 0x63, 0x69, 0x6f, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x77, 0x73,
0x6f, 0x6e, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f,
0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x4d, 0x65,
0x78, 0x69, 0x63, 0x61, 0x6e, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x4d, 0x65, 0x78,
0x69, 0x63, 0x61, 0x6e, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f,
0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x4d, 0x65,
0x78, 0x69, 0x63, 0x61, 0x6e, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0x61,
0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f,
0x72, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x75, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x69, 0x61, 0x6e, 0x6d,
0x61, 0x72, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x6f, 0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x4e, 0x6f, 0x76,
0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0xf4, 0x6e, 0x69, 0x61, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x43, 0x61,
0x6c, 0x65, 0x64, 0xf4, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65,
0x64, 0xf4, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c,
0xe2, 0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x4e,
0x6f, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0xe2, 0x6e, 0x64, 0x69, 0x61,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61,
0x20, 0x5a, 0x65, 0x6c, 0xe2, 0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x54, 0x65, 0x72, 0x72,
0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20,
0x54, 0x65, 0x72, 0x72, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x20,
0x4e, 0x6f, 0x76, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x20, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20,
0x49, 0x6c, 0x68, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6c, 0x68, 0x61,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20,
0x4d, 0x61, 0x72, 0x69, 0x61, 0x6e, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4e,
0x6f, 0x72, 0x74, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4f,
0x6d, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4f,
0x6d, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x6f, 0x20, 0x50, 0x61, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x71, 0x75, 0x69, 0x73, 0x74,
0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61,
0x71, 0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61,
0x70, 0x75, 0x61, 0x2d, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x47, 0x75, 0x69,
0x6e, 0xe9, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f,
0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x69, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x69, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x69, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x6f, 0x20, 0x50, 0x65, 0x72, 0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x50, 0x65, 0x72, 0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x50, 0x65, 0x72, 0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x61, 0x73, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61,
0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61,
0x73, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49,
0x6c, 0x68, 0x61, 0x73, 0x20, 0x46, 0xea, 0x6e, 0x69, 0x78, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0xe3, 0x6f,
0x20, 0x50, 0x65, 0x64, 0x72, 0x6f, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53,
0xe3, 0x6f, 0x20, 0x50, 0x65, 0x64, 0x72, 0x6f, 0x20, 0x65, 0x20, 0x4d,
0x69, 0x71, 0x75, 0x65, 0x6c, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x53, 0xe3, 0x6f, 0x20, 0x50, 0x65, 0x64, 0x72, 0x6f, 0x20, 0x65, 0x20,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61,
0x69, 0x72, 0x6e, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67,
0x79, 0x61, 0x6e, 0x67, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f,
0x72, 0x64, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x51,
0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x52, 0x65, 0x75, 0x6e, 0x69,
0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x63, 0x61, 0x6c,
0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x63,
0x61, 0x6c, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x53, 0x61, 0x63, 0x61, 0x6c, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61,
0x6d, 0x61, 0x72, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x65, 0x69, 0x63, 0x68,
0x65, 0x6c, 0x65, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x69,
0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73,
0x20, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x47, 0x65, 0xf3, 0x72, 0x67,
0x69, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x53, 0x75, 0x6c, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x53, 0x75, 0x72, 0x69,
0x6e, 0x61, 0x6d, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x54, 0x61, 0x69, 0x74, 0x69,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x54, 0x61, 0x6a, 0x69, 0x71, 0x75,
0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x54, 0x75, 0x72, 0x63, 0x6f, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f,
0x20, 0x54, 0x75, 0x72, 0x63, 0x6f, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x54, 0x75,
0x72, 0x63, 0x6f, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x54, 0x75,
0x76, 0x61, 0x6c, 0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x69, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x6f, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x69, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x69, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f,
0x20, 0x55, 0x7a, 0x62, 0x65, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x71, 0x75,
0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f,
0x20, 0x55, 0x7a, 0x62, 0x65, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x56, 0x65, 0x6e,
0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x57, 0x61,
0x6b, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x65, 0x20, 0x46, 0x75,
0x74, 0x75, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x45, 0x63, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72,
0x67, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x45, 0x63, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x45, 0x63, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x6f, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x63, 0x72, 0x65, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f,
0x20, 0x41, 0x63, 0x72, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x63,
0x72, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x66,
0x65, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x61, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x61, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x61, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x53, 0x75, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0xc1,
0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61,
0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x61, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4f, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x6f, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x63, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x41, 0x6c, 0x61, 0x73, 0x63, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41,
0x6c, 0x61, 0x73, 0x63, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6c,
0x6d, 0x61, 0x74, 0x79, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6c, 0x6d,
0x61, 0x74, 0x79, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x41,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x74,
0x65, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65,
0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x74,
0x65, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x2d, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x61, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x68, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x74, 0x65, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61,
0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x68,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x2d, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x63, 0x61, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x6d, 0x6f,
0x6e, 0x74, 0x61, 0x6e, 0x68, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65,
0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69,
0x63, 0x6f, 0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x2d, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x61, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x63,
0xed, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x2d,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x6f, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x6e,
0x6f, 0x72, 0x74, 0x65, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61,
0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6e,
0x61, 0x64, 0x79, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79,
0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x41, 0x70, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x41, 0x71, 0x74, 0x61, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x71, 0x74, 0x61,
0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62,
0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0xe1, 0x62, 0x69, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x61, 0x20, 0x41, 0x72, 0xe1, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61,
0x20, 0x41, 0x72, 0xe1, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x41,
0x72, 0x6d, 0xe9, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0x6d,
0xe9, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0x6d,
0xe9, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x41, 0x74, 0x6c, 0xe2, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x41, 0x74, 0x6c, 0xe2, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x6f, 0x20, 0x41, 0x74, 0x6c, 0xe2, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0xe1, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61,
0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x4f, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69,
0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x4f, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61,
0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x4f,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x6f, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0xe3, 0x6f,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0xe3,
0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61,
0x69, 0x6a, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x73,
0x20, 0x41, 0xe7, 0x6f, 0x72, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x73, 0x20, 0x41,
0xe7, 0x6f, 0x72, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x73, 0x20, 0x41,
0xe7, 0x6f, 0x72, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f,
0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x63, 0x68, 0x65,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x63,
0x68, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x63, 0x68, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x6f, 0x20, 0x42, 0x75, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x61, 0x20, 0x42, 0x6f, 0x6c, 0xed, 0x76, 0x69, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69,
0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x62, 0x6f, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x62, 0x6f,
0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43,
0x61, 0x62, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x6f, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x43, 0x68,
0x69, 0x6c, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x6c,
0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x43, 0x68, 0x69,
0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x61, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x4e, 0x61, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61,
0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x63, 0x6f,
0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x6c,
0xf4, 0x6d, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0xf4,
0x6d, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x6c,
0xf4, 0x6d, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61,
0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x43, 0x6f,
0x6f, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61,
0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20,
0x4c, 0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61,
0x20, 0x49, 0x6c, 0x68, 0x61, 0x20, 0x64, 0x61, 0x20, 0x50, 0xe1, 0x73,
0x63, 0x6f, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x20, 0x64,
0x61, 0x20, 0x50, 0xe1, 0x73, 0x63, 0x6f, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61,
0x20, 0x49, 0x6c, 0x68, 0x61, 0x20, 0x64, 0x61, 0x20, 0x50, 0xe1, 0x73,
0x63, 0x6f, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x45,
0x71, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f,
0x20, 0x45, 0x78, 0x74, 0x72, 0x65, 0x6d, 0x6f, 0x20, 0x4c, 0x65, 0x73,
0x74, 0x65, 0x20, 0x64, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73,
0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73,
0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x46, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x61, 0x20, 0x47, 0x75, 0x69, 0x61, 0x6e, 0x61, 0x20,
0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x61, 0x73, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x73, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x69, 0x73, 0x20, 0x65, 0x20, 0x41,
0x6e, 0x74, 0xe1, 0x72, 0x63, 0x74, 0x69, 0x63, 0x61, 0x73, 0x20, 0x46,
0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x61, 0x73, 0x20, 0x47, 0x61, 0x6c, 0xe1, 0x70, 0x61, 0x67,
0x6f, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x47, 0x65,
0xf3, 0x72, 0x67, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x47, 0x65, 0xf3, 0x72,
0x67, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x47, 0x65, 0xf3, 0x72,
0x67, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73, 0x20,
0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72,
0x74, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x61, 0x20, 0x47, 0x72, 0x6f, 0x6e, 0x65, 0x6c, 0xe2, 0x6e, 0x64, 0x69,
0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61,
0x20, 0x47, 0x72, 0x6f, 0x6e, 0x65, 0x6c, 0xe2, 0x6e, 0x64, 0x69, 0x61,
0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x61, 0x20, 0x47, 0x72, 0x6f, 0x6e, 0x65, 0x6c, 0xe2, 0x6e, 0x64, 0x69,
0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x6e, 0x65, 0x6c,
0xe2, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x6e, 0x65, 0x6c,
0xe2, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x6e,
0x65, 0x6c, 0xe2, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x47, 0x75, 0x61, 0x6d,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x47, 0x6f, 0x6c, 0x66, 0x6f, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x6f, 0x20, 0x48, 0x61, 0x76, 0x61, 0x69, 0x20, 0x65, 0x20,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x48, 0x61,
0x76, 0x61, 0x69, 0x20, 0x65, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x61,
0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x48, 0x61, 0x76, 0x61, 0x69, 0x20,
0x65, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x61, 0x73, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x48,
0x6f, 0x76, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0xcd, 0x6e, 0x64, 0x69, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4f, 0x63, 0x65, 0x61, 0x6e,
0x6f, 0x20, 0xcd, 0x6e, 0x64, 0x69, 0x63, 0x6f, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6e, 0x64,
0x6f, 0x6e, 0xe9, 0x73, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0xe9, 0x73, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20,
0x49, 0x6e, 0x64, 0x6f, 0x6e, 0xe9, 0x73, 0x69, 0x61, 0x20, 0x4f, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x6f, 0x20, 0x49, 0x72, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x49, 0x72,
0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x49, 0x72, 0xe3, 0x6f, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4a, 0x61, 0x70, 0xe3, 0x6f, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x6f, 0x20, 0x4a, 0x61, 0x70, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x4a, 0x61, 0x70, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76,
0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b,
0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76,
0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61,
0x74, 0x73, 0x6b, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x74,
0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b,
0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x6f, 0x20, 0x43, 0x61, 0x7a, 0x61, 0x71, 0x75, 0x69,
0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x43, 0x61, 0x7a, 0x61, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x20,
0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x6f, 0x20, 0x43, 0x61, 0x7a, 0x61, 0x71, 0x75, 0x69, 0x73,
0x74, 0xe3, 0x6f, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x72,
0x65, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x69, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x69, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73,
0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x79, 0x61, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f,
0x20, 0x51, 0x75, 0x69, 0x72, 0x67, 0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x53, 0x72, 0x69, 0x20,
0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61,
0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x4c, 0x69, 0x6e, 0x65,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x61, 0x20, 0x4d, 0x61, 0x6c, 0xe1, 0x73, 0x69, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76,
0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49,
0x6c, 0x68, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73,
0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49,
0x6c, 0x68, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c,
0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x4d, 0x61, 0x75,
0x72, 0xed, 0x63, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x4d, 0x61, 0x75, 0x72,
0xed, 0x63, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x4d, 0x61, 0x75,
0x72, 0xed, 0x63, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f,
0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x4d, 0x65,
0x78, 0x69, 0x63, 0x61, 0x6e, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x63,
0xed, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61,
0x6e, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66,
0x69, 0x63, 0x6f, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x6f,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x6e,
0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0x61,
0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x76,
0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x76, 0x6f, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x76, 0x6f, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x69, 0x61, 0x6e, 0x6d,
0x61, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4e, 0x65,
0x70, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x4e,
0x6f, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0xf3, 0x6e, 0x69,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c,
0x65, 0x64, 0xf3, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x4e,
0x6f, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0xf3, 0x6e, 0x69,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x4e, 0x6f, 0x76,
0x61, 0x20, 0x5a, 0x65, 0x6c, 0xe2, 0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61,
0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0xe2, 0x6e, 0x64,
0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20,
0x5a, 0x65, 0x6c, 0xe2, 0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x61, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x20, 0x4e, 0x6f,
0x76, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x20, 0x4e,
0x6f, 0x76, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x54, 0x65, 0x72, 0x72,
0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x69, 0x75, 0xea, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x61, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6c,
0x68, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x4d,
0x61, 0x72, 0x69, 0x61, 0x6e, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4e, 0x6f,
0x72, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4f, 0x6d, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73,
0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x71,
0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x71,
0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50,
0x61, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x6f,
0x76, 0x61, 0x20, 0x47, 0x75, 0x69, 0x6e, 0xe9, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x69,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x69, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x69,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x65, 0x72, 0x75,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x50, 0x65, 0x72, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x50, 0x65, 0x72, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73,
0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61,
0x73, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x61, 0x73, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e,
0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49,
0x6c, 0x68, 0x61, 0x73, 0x20, 0x46, 0xe9, 0x6e, 0x69, 0x78, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0xe3, 0x6f, 0x20, 0x50, 0x65,
0x64, 0x72, 0x6f, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0xe3, 0x6f, 0x20, 0x50, 0x65, 0x64,
0x72, 0x6f, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0xe3,
0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0xe3, 0x6f, 0x20, 0x50, 0x65,
0x64, 0x72, 0x6f, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72,
0x64, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c,
0x6f, 0x72, 0x64, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x52, 0x65, 0x75, 0x6e, 0x69, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x63, 0x61, 0x6c, 0x69, 0x6e, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x53, 0x61, 0x63, 0x61, 0x6c, 0x69, 0x6e, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x63, 0x61, 0x6c, 0x69, 0x6e, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x53, 0x65,
0x69, 0x63, 0x68, 0x65, 0x6c, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x69,
0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x53, 0x61,
0x6c, 0x6f, 0x6d, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61,
0x20, 0x47, 0x65, 0xf3, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x53, 0x75, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x53,
0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x6f, 0x20, 0x54, 0x61, 0x69, 0x74, 0x69, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x54,
0x61, 0x69, 0x70, 0xe9, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69,
0x70, 0xe9, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x54, 0x61,
0x6a, 0x69, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x54,
0x75, 0x72, 0x71, 0x75, 0x65, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0xe3,
0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x6f, 0x20, 0x54, 0x75, 0x72, 0x71, 0x75, 0x65, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x54,
0x75, 0x72, 0x71, 0x75, 0x65, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0xe3,
0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x55, 0x7a,
0x62, 0x65, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x55, 0x7a, 0x62, 0x65, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x6f, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x71, 0x75, 0x69, 0x73,
0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20,
0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6c, 0x68,
0x61, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x65, 0x20, 0x46,
0x75, 0x74, 0x75, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45,
0x63, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x45, 0x63, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62,
0x75, 0x72, 0x67, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x45, 0x63, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c, 0x61, 0x73, 0x20, 0x41, 0x6d, 0x113,
0x72, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c, 0x61, 0x73, 0x20, 0x41, 0x6d, 0x113,
0x72, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x7a, 0x113, 0x69, 0x6d, 0x61, 0x73,
0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x101,
0x6c, 0x61, 0x73, 0x20, 0x41, 0x6d, 0x113, 0x72, 0x69, 0x6b, 0x61, 0x73,
0x20, 0x64, 0x61, 0x67, 0x67, 0x61, 0x73, 0x20, 0x6b, 0x65, 0x72, 0x64,
0x101, 0x44, 0x113, 0x69, 0x6e, 0x69, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x41,
0x6d, 0x113, 0x72, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x6b, 0x65, 0x72, 0x64,
0x101, 0x44, 0x113, 0x69, 0x6e, 0x69, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x41,
0x6d, 0x113, 0x72, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x7a, 0x113, 0x69, 0x6d,
0x61, 0x73, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0x44, 0x113, 0x69, 0x6e,
0x69, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x41, 0x6d, 0x113, 0x72, 0x69, 0x6b,
0x61, 0x73, 0x20, 0x64, 0x61, 0x67, 0x67, 0x61, 0x73, 0x20, 0x6b, 0x65,
0x72, 0x64, 0x101, 0x41, 0x6d, 0x113, 0x72, 0x69, 0x6b, 0x61, 0x73, 0x20,
0x67, 0x101, 0x72, 0x62, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101,
0x41, 0x6d, 0x113, 0x72, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x67, 0x101, 0x72,
0x62, 0x61, 0x6e, 0x20, 0x7a, 0x113, 0x69, 0x6d, 0x61, 0x73, 0x20, 0x6b,
0x65, 0x72, 0x64, 0x101, 0x41, 0x6d, 0x113, 0x72, 0x69, 0x6b, 0x61, 0x73,
0x20, 0x67, 0x101, 0x72, 0x62, 0x61, 0x6e, 0x20, 0x64, 0x61, 0x67, 0x67,
0x61, 0x73, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0x50, 0x61, 0x63, 0x12b,
0x66, 0x69, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x41, 0x6d, 0x113, 0x72, 0x69,
0x6b, 0x61, 0x73, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0x50, 0x61, 0x63,
0x12b, 0x66, 0x69, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x41, 0x6d, 0x113, 0x72,
0x69, 0x6b, 0x61, 0x73, 0x20, 0x7a, 0x113, 0x69, 0x6d, 0x61, 0x73, 0x20,
0x6b, 0x65, 0x72, 0x64, 0x101, 0x50, 0x61, 0x63, 0x12b, 0x66, 0x69, 0x73,
0x6b, 0x61, 0x73, 0x20, 0x41, 0x6d, 0x113, 0x72, 0x69, 0x6b, 0x61, 0x73,
0x20, 0x64, 0x61, 0x67, 0x67, 0x61, 0x73, 0x20, 0x6b, 0x65, 0x72, 0x64,
0x101, 0x41, 0x74, 0x6c, 0x101, 0x6e, 0x74, 0x69, 0x73, 0x6b, 0x61, 0x20,
0x6b, 0x65, 0x72, 0x64, 0x101, 0x41, 0x74, 0x6c, 0x101, 0x6e, 0x74, 0x69,
0x73, 0x6b, 0x61, 0x20, 0x7a, 0x113, 0x69, 0x6d, 0x61, 0x73, 0x20, 0x6b,
0x65, 0x72, 0x64, 0x101, 0x41, 0x74, 0x6c, 0x101, 0x6e, 0x74, 0x69, 0x73,
0x6b, 0x61, 0x20, 0x64, 0x61, 0x67, 0x67, 0x61, 0x73, 0x20, 0x6b, 0x65,
0x72, 0x64, 0x101, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c, 0x61, 0x73,
0x20, 0x45, 0x75, 0x72, 0x14d, 0x70, 0x61, 0x73, 0x20, 0x6b, 0x65, 0x72,
0x64, 0x101, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c, 0x61, 0x73, 0x20,
0x45, 0x75, 0x72, 0x14d, 0x70, 0x61, 0x73, 0x20, 0x7a, 0x113, 0x69, 0x6d,
0x61, 0x73, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x101, 0x6c, 0x61, 0x73, 0x20, 0x45, 0x75, 0x72, 0x14d, 0x70, 0x61,
0x73, 0x20, 0x64, 0x61, 0x67, 0x67, 0x61, 0x73, 0x20, 0x6b, 0x65, 0x72,
0x64, 0x101, 0x44, 0x113, 0x69, 0x6e, 0x69, 0x73, 0x6b, 0x61, 0x73, 0x20,
0x45, 0x75, 0x72, 0x14d, 0x70, 0x61, 0x73, 0x20, 0x6b, 0x65, 0x72, 0x64,
0x101, 0x44, 0x113, 0x69, 0x6e, 0x69, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x45,
0x75, 0x72, 0x14d, 0x70, 0x61, 0x73, 0x20, 0x7a, 0x113, 0x69, 0x6d, 0x61,
0x73, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0x44, 0x113, 0x69, 0x6e, 0x69,
0x73, 0x6b, 0x61, 0x73, 0x20, 0x45, 0x75, 0x72, 0x14d, 0x70, 0x61, 0x73,
0x20, 0x64, 0x61, 0x67, 0x67, 0x61, 0x73, 0x20, 0x6b, 0x65, 0x72, 0x64,
0x101, 0x57, 0x61, 0x6b, 0x6b, 0x61, 0x72, 0x69, 0x73, 0x6b, 0x61, 0x73,
0x20, 0x45, 0x75, 0x72, 0x14d, 0x70, 0x61, 0x73, 0x20, 0x6b, 0x65, 0x72,
0x64, 0x101, 0x57, 0x61, 0x6b, 0x6b, 0x61, 0x72, 0x69, 0x73, 0x6b, 0x61,
0x73, 0x20, 0x45, 0x75, 0x72, 0x14d, 0x70, 0x61, 0x73, 0x20, 0x7a, 0x113,
0x69, 0x6d, 0x61, 0x73, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0x57, 0x61,
0x6b, 0x6b, 0x61, 0x72, 0x69, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x45, 0x75,
0x72, 0x14d, 0x70, 0x61, 0x73, 0x20, 0x64, 0x61, 0x67, 0x67, 0x61, 0x73,
0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77,
0x69, 0x63, 0x68, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0xa05, 0xa2b, 0xa3c,
0xa17, 0xa3e, 0xa28, 0xa3f, 0xa38, 0xa24, 0xa3e, 0xa28, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa15, 0xa47, 0xa02, 0xa26, 0xa30, 0xa40, 0x20, 0xa05, 0xa2b, 0xa30, 0xa40,
0xa15, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40,
0x20, 0xa05, 0xa2b, 0xa30, 0xa40, 0xa15, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa26, 0xa71, 0xa16, 0xa23, 0xa40, 0x20, 0xa05, 0xa2b, 0xa3c, 0xa30, 0xa40, 0xa15,
0xa3e, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa05, 0xa2b, 0xa30, 0xa40, 0xa15, 0xa3e,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa05,
0xa2b, 0xa30, 0xa40, 0xa15, 0xa3e, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa05, 0xa2b,
0xa30, 0xa40, 0xa15, 0xa3e, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20,
0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa32, 0xa3e, 0xa38, 0xa15,
0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa32, 0xa3e, 0xa38, 0xa15, 0xa3e,
0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05,
0xa32, 0xa3e, 0xa38, 0xa15, 0xa3e, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38,
0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa32, 0xa2e, 0xa3e, 0xa1f, 0xa40,
0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0xa05, 0xa32, 0xa2e, 0xa3e, 0xa1f, 0xa40, 0x20,
0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0xa05, 0xa32,
0xa2e, 0xa3e, 0xa1f, 0xa40, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0x2d, 0xa30, 0xa41,
0xa71, 0xa24, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0xa05, 0xa2e, 0xa47, 0xa1c, 0xa3c,
0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa2e, 0xa47, 0xa1c, 0xa3c, 0xa28,
0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05,
0xa2e, 0xa47, 0xa1c, 0xa3c, 0xa28, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02,
0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa71, 0xa24, 0xa30,
0xa40, 0x20, 0xa05, 0xa2e, 0xa30, 0xa40, 0xa15, 0xa40, 0x20, 0xa15, 0xa47, 0xa02,
0xa26, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa71, 0xa24, 0xa30,
0xa40, 0x20, 0xa05, 0xa2e, 0xa30, 0xa40, 0xa15, 0xa40, 0x20, 0xa15, 0xa47, 0xa02,
0xa26, 0xa30, 0xa40, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa09, 0xa71, 0xa24, 0xa30, 0xa40, 0x20, 0xa05, 0xa2e, 0xa30, 0xa40,
0xa15, 0xa40, 0x20, 0xa15, 0xa47, 0xa02, 0xa26, 0xa30, 0xa40, 0x20, 0xa2a, 0xa4d,
0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa71,
0xa24, 0xa30, 0xa40, 0x20, 0xa05, 0xa2e, 0xa30, 0xa40, 0xa15, 0xa40, 0x20, 0xa2a,
0xa42, 0xa30, 0xa2c, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa71, 0xa24,
0xa30, 0xa40, 0x20, 0xa05, 0xa2e, 0xa30, 0xa40, 0xa15, 0xa40, 0x20, 0xa2a, 0xa42,
0xa30, 0xa2c, 0xa40, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa09, 0xa71, 0xa24, 0xa30, 0xa40, 0x20, 0xa05, 0xa2e, 0xa30, 0xa40,
0xa15, 0xa40, 0x20, 0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40, 0x20, 0xa2a, 0xa4d, 0xa30,
0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa71, 0xa24,
0xa30, 0xa40, 0x20, 0xa05, 0xa2e, 0xa30, 0xa40, 0xa15, 0xa40, 0x20, 0xa2e, 0xa3e,
0xa09, 0xa02, 0xa1f, 0xa47, 0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa71,
0xa24, 0xa30, 0xa40, 0x20, 0xa05, 0xa2e, 0xa30, 0xa40, 0xa15, 0xa40, 0x20, 0xa2e,
0xa3e, 0xa09, 0xa02, 0xa1f, 0xa47, 0xa28, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa71, 0xa24, 0xa30, 0xa40, 0x20, 0xa05,
0xa2e, 0xa30, 0xa40, 0xa15, 0xa40, 0x20, 0xa2e, 0xa3e, 0xa09, 0xa02, 0xa1f, 0xa47,
0xa28, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa09, 0xa71, 0xa24, 0xa30, 0xa40, 0x20, 0xa05, 0xa2e, 0xa30, 0xa40,
0xa15, 0xa40, 0x20, 0xa2a, 0xa48, 0xa38, 0xa3f, 0xa2b, 0xa3f, 0xa15, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa09, 0xa71, 0xa24, 0xa30, 0xa40, 0x20, 0xa05, 0xa2e, 0xa30,
0xa40, 0xa15, 0xa40, 0x20, 0xa2a, 0xa48, 0xa38, 0xa3f, 0xa2b, 0xa3f, 0xa15, 0x20,
0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa71,
0xa24, 0xa30, 0xa40, 0x20, 0xa05, 0xa2e, 0xa30, 0xa40, 0xa15, 0xa40, 0x20, 0xa2a,
0xa48, 0xa38, 0xa3f, 0xa2b, 0xa3f, 0xa15, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e,
0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa10, 0xa2a, 0xa40, 0xa06, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa10, 0xa2a, 0xa40, 0xa06, 0x20, 0xa2e, 0xa3f, 0xa06,
0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa10, 0xa2a, 0xa40, 0xa06, 0x20,
0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa05, 0xa15, 0xa24, 0xa3e, 0xa09, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0xa05, 0xa15,
0xa24, 0xa3e, 0xa09, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa38, 0xa2e,
0xa3e, 0xa02, 0xa05, 0xa15, 0xa24, 0xa3e, 0xa09, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40,
0x2d, 0xa30, 0xa41, 0xa71, 0xa24, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0xa05, 0xa15,
0xa24, 0xa4b, 0xa2c, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0xa05, 0xa15, 0xa24, 0xa4b,
0xa2c, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02,
0xa05, 0xa15, 0xa24, 0xa4b, 0xa2c, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0x2d, 0xa30,
0xa41, 0xa71, 0xa24, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0xa05, 0xa30, 0xa2c, 0xa40,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa30, 0xa2c, 0xa40, 0x20, 0xa2e, 0xa3f,
0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa30, 0xa2c, 0xa40,
0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa05, 0xa30, 0xa1c, 0xa28, 0xa1f, 0xa40, 0xa28, 0xa3e, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa05, 0xa30, 0xa1c, 0xa28, 0xa1f, 0xa40, 0xa28, 0xa3e, 0x20, 0xa2e,
0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa30, 0xa1c,
0xa28, 0xa1f, 0xa40, 0xa28, 0xa3e, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02,
0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa71, 0xa1b, 0xa2e,
0xa40, 0x20, 0xa05, 0xa30, 0xa1c, 0xa28, 0xa1f, 0xa40, 0xa28, 0xa3e, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa05, 0xa30, 0xa1c,
0xa28, 0xa1f, 0xa40, 0xa28, 0xa3e, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa05, 0xa30,
0xa1c, 0xa28, 0xa1f, 0xa40, 0xa28, 0xa3e, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06,
0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa30, 0xa2e,
0xa40, 0xa28, 0xa40, 0xa06, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa30, 0xa2e,
0xa40, 0xa28, 0xa40, 0xa06, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa05, 0xa30, 0xa2e, 0xa40, 0xa28, 0xa40, 0xa06, 0x20, 0xa17,
0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa05, 0xa1f, 0xa32, 0xa3e, 0xa02, 0xa1f, 0xa3f, 0xa15, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa05, 0xa1f, 0xa32, 0xa3e, 0xa02, 0xa1f, 0xa3f, 0xa15, 0x20, 0xa2e,
0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa1f, 0xa32,
0xa3e, 0xa02, 0xa1f, 0xa3f, 0xa15, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38,
0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa47, 0xa02, 0xa26, 0xa30, 0xa40,
0x20, 0xa06, 0xa38, 0xa1f, 0xa4d, 0xa30, 0xa47, 0xa32, 0xa40, 0xa06, 0xa08, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa06, 0xa38, 0xa1f, 0xa4d, 0xa30, 0xa47, 0xa32, 0xa40,
0xa06, 0xa08, 0x20, 0xa15, 0xa47, 0xa02, 0xa26, 0xa30, 0xa40, 0x20, 0xa2e, 0xa3f,
0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa06, 0xa38, 0xa1f, 0xa4d,
0xa30, 0xa47, 0xa32, 0xa40, 0xa06, 0xa08, 0x20, 0xa15, 0xa47, 0xa02, 0xa26, 0xa30,
0xa40, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa06, 0xa38, 0xa1f, 0xa4d, 0xa30, 0xa47, 0xa32, 0xa40, 0xa06, 0xa08,
0x20, 0xa15, 0xa47, 0xa02, 0xa26, 0xa30, 0xa40, 0x20, 0xa2a, 0xa71, 0xa1b, 0xa2e,
0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa06, 0xa38, 0xa1f, 0xa4d, 0xa30, 0xa47,
0xa32, 0xa40, 0xa06, 0xa08, 0x20, 0xa15, 0xa47, 0xa02, 0xa26, 0xa30, 0xa40, 0x20,
0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa06, 0xa38, 0xa1f, 0xa4d, 0xa30, 0xa47, 0xa32, 0xa40,
0xa06, 0xa08, 0x20, 0xa15, 0xa47, 0xa02, 0xa26, 0xa30, 0xa40, 0x20, 0xa2a, 0xa71,
0xa1b, 0xa2e, 0xa40, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40, 0x20, 0xa06, 0xa38,
0xa1f, 0xa4d, 0xa30, 0xa47, 0xa32, 0xa40, 0xa06, 0xa08, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa06, 0xa38, 0xa1f, 0xa4d, 0xa30, 0xa47, 0xa32, 0xa40, 0xa06, 0xa08, 0x20,
0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa06, 0xa38, 0xa1f, 0xa4d, 0xa30, 0xa47, 0xa32, 0xa40,
0xa06, 0xa08, 0x20, 0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40, 0x20, 0xa2a, 0xa4d, 0xa30,
0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa71, 0xa1b,
0xa2e, 0xa40, 0x20, 0xa06, 0xa38, 0xa1f, 0xa4d, 0xa30, 0xa47, 0xa32, 0xa40, 0xa06,
0xa08, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa06, 0xa38, 0xa1f, 0xa4d, 0xa30, 0xa47,
0xa32, 0xa40, 0xa06, 0xa08, 0x20, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa2e,
0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa06, 0xa38, 0xa1f,
0xa4d, 0xa30, 0xa47, 0xa32, 0xa40, 0xa06, 0xa08, 0x20, 0xa2a, 0xa71, 0xa1b, 0xa2e,
0xa40, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa05, 0xa1c, 0xa3c, 0xa30, 0xa2c, 0xa3e, 0xa08, 0xa1c, 0xa3e, 0xa28,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa1c, 0xa3c, 0xa30, 0xa2c, 0xa3e, 0xa08,
0xa1c, 0xa3e, 0xa28, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa05, 0xa1c, 0xa3c, 0xa30, 0xa2c, 0xa3e, 0xa08, 0xa1c, 0xa3e, 0xa28,
0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa05, 0xa1c, 0xa4b, 0xa30, 0xa47, 0xa38, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa05, 0xa1c, 0xa4b, 0xa30, 0xa47, 0xa38, 0x20, 0xa2e, 0xa3f, 0xa06,
0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa1c, 0xa4b, 0xa30, 0xa47,
0xa38, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa2c, 0xa70, 0xa17, 0xa32, 0xa3e, 0xa26, 0xa47, 0xa38,
0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2c, 0xa70, 0xa17, 0xa32, 0xa3e, 0xa26,
0xa47, 0xa38, 0xa3c, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa2c, 0xa70, 0xa17, 0xa32, 0xa3e, 0xa26, 0xa47, 0xa38, 0xa3c, 0x20,
0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa2d, 0xa42, 0xa1f, 0xa3e, 0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa2c, 0xa4b, 0xa32, 0xa40, 0xa35, 0xa40, 0xa06, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa2c, 0xa4d, 0xa30, 0xa3e, 0xa1c, 0xa3c, 0xa40, 0xa32, 0xa40, 0xa06, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa2c, 0xa4d, 0xa30, 0xa3e, 0xa1c, 0xa3c, 0xa40, 0xa32, 0xa40,
0xa06, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa2c, 0xa4d, 0xa30, 0xa3e, 0xa1c, 0xa3c, 0xa40, 0xa32, 0xa40, 0xa06, 0x20, 0xa17,
0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa2c, 0xa30, 0xa42, 0xa28, 0xa47, 0xa08, 0x20, 0xa26, 0xa3e, 0xa30, 0xa42,
0xa38, 0xa32, 0xa3e, 0xa2e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa47, 0xa2a,
0x20, 0xa35, 0xa30, 0xa21, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa47, 0xa2a,
0x20, 0xa35, 0xa30, 0xa21, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa15, 0xa47, 0xa2a, 0x20, 0xa35, 0xa30, 0xa21, 0x20, 0xa17,
0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa15, 0xa47, 0xa38, 0xa40, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0xa1a, 0xa3e,
0xa2e, 0xa4b, 0xa30, 0xa4b, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa1a, 0xa48, 0xa25, 0xa2e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa1a, 0xa48, 0xa25, 0xa2e, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa1a, 0xa48, 0xa25, 0xa2e, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15,
0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1a, 0xa3f, 0xa32, 0xa40,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1a, 0xa3f, 0xa32, 0xa40, 0x20, 0xa2e, 0xa3f,
0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1a, 0xa3f, 0xa32, 0xa40,
0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa1a, 0xa40, 0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1a,
0xa40, 0xa28, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa1a, 0xa40, 0xa28, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa4d, 0xa30, 0xa3f, 0xa38, 0xa2e, 0xa38,
0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa15, 0xa4b, 0xa15, 0xa38, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa4b, 0xa32, 0xa70, 0xa2c, 0xa40, 0xa06, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa4b, 0xa32, 0xa70, 0xa2c, 0xa40, 0xa06, 0x20,
0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa4b,
0xa32, 0xa70, 0xa2c, 0xa40, 0xa06, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02,
0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa41, 0xa71, 0xa15,
0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa15, 0xa41, 0xa71, 0xa15, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0x20,
0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa41,
0xa71, 0xa15, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa05, 0xa71,
0xa27, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa3f, 0xa0a, 0xa2c, 0xa3e, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa15, 0xa3f, 0xa0a, 0xa2c, 0xa3e, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30,
0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa3f, 0xa0a, 0xa2c, 0xa3e, 0x20,
0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa21, 0xa47, 0xa35, 0xa3f, 0xa38, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa21, 0xa3f,
0xa09, 0xa2e, 0xa4b, 0xa02, 0xa1f, 0x20, 0xa21, 0xa3f, 0xa09, 0xa30, 0xa35, 0xa3f,
0xa32, 0xa47, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40,
0x20, 0xa24, 0xa3f, 0xa2e, 0xa42, 0xa30, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa08,
0xa38, 0xa1f, 0xa30, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa08, 0xa38, 0xa1f, 0xa30, 0x20, 0xa06, 0xa08, 0xa32, 0xa48,
0xa02, 0xa21, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa08, 0xa38, 0xa1f, 0xa30, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21,
0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa07, 0xa15, 0xa35, 0xa47, 0xa21, 0xa4b, 0xa30, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa2e, 0xa71, 0xa27, 0x20, 0xa2f, 0xa42, 0xa30, 0xa2a, 0xa40,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa71, 0xa27, 0x20, 0xa2f, 0xa42, 0xa30,
0xa2a, 0xa40, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa2e, 0xa71, 0xa27, 0x20, 0xa2f, 0xa42, 0xa30, 0xa2a, 0xa40, 0x20, 0xa17,
0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40, 0x20, 0xa2f, 0xa42, 0xa30, 0xa2a, 0xa40,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40, 0x20, 0xa2f,
0xa42, 0xa30, 0xa2a, 0xa40, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40, 0x20, 0xa2f, 0xa42, 0xa30,
0xa2a, 0xa40, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa39, 0xa4b, 0xa30, 0x2d, 0xa2a, 0xa42, 0xa30,
0xa2c, 0xa40, 0x20, 0xa2f, 0xa42, 0xa30, 0xa2a, 0xa40, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa2f, 0xa42, 0xa30, 0xa2a, 0xa40,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa2f,
0xa42, 0xa30, 0xa2a, 0xa40, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa2f, 0xa42, 0xa30,
0xa2a, 0xa40, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2b, 0xa3c, 0xa3e, 0xa15, 0xa32, 0xa48, 0xa02,
0xa21, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0xa38, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa2b, 0xa3c, 0xa3e, 0xa15, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa06,
0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0xa38, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2b, 0xa3c, 0xa3e, 0xa15, 0xa32, 0xa48, 0xa02,
0xa21, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0xa38, 0x20, 0xa17, 0xa30,
0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa2b, 0xa3c, 0xa3f, 0xa1c, 0xa3c, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2b,
0xa3c, 0xa3f, 0xa1c, 0xa3c, 0xa40, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa2b, 0xa3c, 0xa3f, 0xa1c, 0xa3c, 0xa40, 0x20, 0xa17,
0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa2b, 0xa4d, 0xa30, 0xa48, 0xa02, 0xa1a, 0x20, 0xa17, 0xa41, 0xa0f, 0xa28,
0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2b, 0xa4d, 0xa30, 0xa48, 0xa02, 0xa1a,
0x20, 0xa26, 0xa71, 0xa16, 0xa23, 0xa40, 0x20, 0xa05, 0xa24, 0xa47, 0x20, 0xa10,
0xa02, 0xa1f, 0xa3e, 0xa30, 0xa1f, 0xa3f, 0xa15, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa17, 0xa32, 0xa3e, 0xa2a, 0xa3e, 0xa17, 0xa4b, 0xa38, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa17, 0xa48, 0xa02, 0xa2c, 0xa40, 0xa05, 0xa30, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa1c, 0xa3e, 0xa30, 0xa1c, 0xa40, 0xa06, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa1c, 0xa3e, 0xa30, 0xa1c, 0xa40, 0xa06, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1c, 0xa3e, 0xa30, 0xa1c, 0xa40, 0xa06, 0x20,
0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa17, 0xa3f, 0xa32, 0xa2c, 0xa30, 0xa1f, 0x20, 0xa06, 0xa08, 0xa32,
0xa48, 0xa02, 0xa21, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa17, 0xa4d, 0xa30, 0xa40,
0xa28, 0xa35, 0xa3f, 0xa1a, 0x20, 0xa2e, 0xa40, 0xa28, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40, 0x20, 0xa17, 0xa4d, 0xa30, 0xa40, 0xa28,
0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa42, 0xa30,
0xa2c, 0xa40, 0x20, 0xa17, 0xa4d, 0xa30, 0xa40, 0xa28, 0xa32, 0xa48, 0xa02, 0xa21,
0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a,
0xa42, 0xa30, 0xa2c, 0xa40, 0x20, 0xa17, 0xa4d, 0xa30, 0xa40, 0xa28, 0xa32, 0xa48,
0xa02, 0xa21, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa17,
0xa4d, 0xa30, 0xa40, 0xa28, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa17, 0xa4d, 0xa30, 0xa40, 0xa28,
0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa17, 0xa4d, 0xa30,
0xa40, 0xa28, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06,
0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa17, 0xa41, 0xa06,
0xa2e, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0xa16, 0xa3e, 0xa5c, 0xa40, 0x20, 0xa2e,
0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa17, 0xa41, 0xa2f,
0xa3e, 0xa28, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa39, 0xa35, 0xa3e, 0xa08,
0x2d, 0xa05, 0xa32, 0xa47, 0xa2f, 0xa42, 0xa38, 0xa3c, 0xa3f, 0xa05, 0xa28, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa39, 0xa35, 0xa3e, 0xa08, 0x2d, 0xa05, 0xa32, 0xa47,
0xa2f, 0xa42, 0xa38, 0xa3c, 0xa3f, 0xa05, 0xa28, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30,
0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa39, 0xa35, 0xa3e, 0xa08, 0x2d, 0xa05,
0xa32, 0xa47, 0xa2f, 0xa42, 0xa38, 0xa3c, 0xa3f, 0xa05, 0xa28, 0x20, 0xa2a, 0xa4d,
0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa39, 0xa3e,
0xa02, 0xa17, 0x20, 0xa15, 0xa3e, 0xa02, 0xa17, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa39, 0xa3e, 0xa02, 0xa17, 0x20, 0xa15, 0xa3e, 0xa02, 0xa17, 0x20, 0xa2e, 0xa3f,
0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa39, 0xa3e, 0xa02, 0xa17,
0x20, 0xa15, 0xa3e, 0xa02, 0xa17, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02,
0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa39, 0xa4b, 0xa35, 0xa21,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa39, 0xa4b, 0xa35, 0xa21, 0x20, 0xa2e, 0xa3f,
0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa39, 0xa4b, 0xa35, 0xa21,
0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa2d, 0xa3e, 0xa30, 0xa24, 0xa40, 0x20, 0xa2e, 0xa3f, 0xa06,
0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa39, 0xa3f, 0xa70, 0xa26, 0x20,
0xa2e, 0xa39, 0xa3e, 0xa02, 0xa38, 0xa3e, 0xa17, 0xa30, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa07, 0xa70, 0xa21, 0xa4b, 0xa1a, 0xa3e, 0xa08, 0xa28, 0xa3e, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa2e, 0xa71, 0xa27, 0x20, 0xa07, 0xa70, 0xa21, 0xa4b, 0xa28,
0xa47, 0xa38, 0xa3c, 0xa40, 0xa06, 0xa08, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a,
0xa42, 0xa30, 0xa2c, 0xa40, 0x20, 0xa07, 0xa70, 0xa21, 0xa4b, 0xa28, 0xa47, 0xa38,
0xa3c, 0xa40, 0xa06, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa71, 0xa1b, 0xa2e,
0xa40, 0x20, 0xa07, 0xa70, 0xa21, 0xa4b, 0xa28, 0xa47, 0xa38, 0xa3c, 0xa40, 0xa06,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa08, 0xa30, 0xa3e, 0xa28, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa08, 0xa30, 0xa3e, 0xa28, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa08, 0xa30, 0xa3e, 0xa28, 0x20, 0xa2a, 0xa4d,
0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa07, 0xa30,
0xa15, 0xa41, 0xa24, 0xa38, 0xa15, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa07, 0xa30,
0xa15, 0xa41, 0xa24, 0xa38, 0xa15, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa07, 0xa30, 0xa15, 0xa41, 0xa24, 0xa38, 0xa15, 0x20,
0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa07, 0xa1c, 0xa3c, 0xa30, 0xa3e, 0xa08, 0xa32, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa07, 0xa1c, 0xa3c, 0xa30, 0xa3e, 0xa08, 0xa32, 0x20, 0xa2e, 0xa3f,
0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa07, 0xa1c, 0xa3c, 0xa30,
0xa3e, 0xa08, 0xa32, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa1c, 0xa2a, 0xa3e, 0xa28, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa1c, 0xa2a, 0xa3e, 0xa28, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa1c, 0xa2a, 0xa3e, 0xa28, 0x20, 0xa2a, 0xa4d, 0xa30,
0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa1c, 0xa3c,
0xa3e, 0xa16, 0xa3c, 0xa38, 0xa24, 0xa3e, 0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40, 0x20, 0xa15, 0xa1c, 0xa3c, 0xa3e, 0xa16, 0xa3c,
0xa38, 0xa24, 0xa3e, 0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa71, 0xa1b,
0xa2e, 0xa40, 0x20, 0xa15, 0xa1c, 0xa3c, 0xa3e, 0xa16, 0xa3c, 0xa38, 0xa24, 0xa3e,
0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa4b, 0xa30, 0xa40, 0xa06, 0xa08,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa4b, 0xa30, 0xa40, 0xa06, 0xa08, 0x20,
0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa4b,
0xa30, 0xa40, 0xa06, 0xa08, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa4b, 0xa38, 0xa30, 0xa47, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa15, 0xa4d, 0xa30, 0xa3e, 0xa38, 0xa28, 0xa4b, 0xa2f, 0xa3e,
0xa30, 0xa38, 0xa15, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa4d, 0xa30, 0xa3e,
0xa38, 0xa28, 0xa4b, 0xa2f, 0xa3e, 0xa30, 0xa38, 0xa15, 0x20, 0xa2e, 0xa3f, 0xa06,
0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa4d, 0xa30, 0xa3e, 0xa38,
0xa28, 0xa4b, 0xa2f, 0xa3e, 0xa30, 0xa38, 0xa15, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40,
0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa3f,
0xa30, 0xa17, 0xa3f, 0xa38, 0xa24, 0xa3e, 0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa32, 0xa70, 0xa15, 0xa3e, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0xa32, 0xa3e, 0xa08,
0xa28, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa32, 0xa4c, 0xa30, 0xa21, 0x20, 0xa39, 0xa4b, 0xa35, 0xa47, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa32, 0xa4c, 0xa30, 0xa21, 0x20, 0xa39, 0xa4b, 0xa35, 0xa47,
0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa32,
0xa4c, 0xa30, 0xa21, 0x20, 0xa39, 0xa4b, 0xa35, 0xa47, 0x20, 0xa2a, 0xa4d, 0xa30,
0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa15, 0xa3e,
0xa09, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0xa2e, 0xa15, 0xa3e, 0xa09, 0x20, 0xa2e,
0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0xa2e, 0xa15, 0xa3e,
0xa09, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0x2d, 0xa30, 0xa41, 0xa71, 0xa24, 0x20,
0xa38, 0xa2e, 0xa3e, 0xa02, 0xa2e, 0xa48, 0xa17, 0xa47, 0xa21, 0xa28, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa2e, 0xa48, 0xa17, 0xa47, 0xa21, 0xa28, 0x20, 0xa2e, 0xa3f,
0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa48, 0xa17, 0xa47,
0xa21, 0xa28, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa32, 0xa47, 0xa38, 0xa3c, 0xa40, 0xa06,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa3e, 0xa32, 0xa26, 0xa40, 0xa35, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa3e, 0xa30, 0xa15, 0xa3f, 0xa38, 0xa3e, 0xa38,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa3e, 0xa30, 0xa38, 0xa3c, 0xa32, 0x20,
0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e,
0xa4c, 0xa30, 0xa3f, 0xa38, 0xa3c, 0xa38, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e,
0xa4c, 0xa30, 0xa3f, 0xa38, 0xa3c, 0xa38, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa4c, 0xa30, 0xa3f, 0xa38, 0xa3c, 0xa38,
0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa2e, 0xa4c, 0xa38, 0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa2e, 0xa48, 0xa15, 0xa38, 0xa40, 0xa15, 0xa28, 0x20, 0xa2a, 0xa48, 0xa38, 0xa3f,
0xa2b, 0xa3f, 0xa15, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa48, 0xa15, 0xa38,
0xa40, 0xa15, 0xa28, 0x20, 0xa2a, 0xa48, 0xa38, 0xa3f, 0xa2b, 0xa3f, 0xa15, 0x20,
0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa48,
0xa15, 0xa38, 0xa40, 0xa15, 0xa28, 0x20, 0xa2a, 0xa48, 0xa38, 0xa3f, 0xa2b, 0xa3f,
0xa15, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa09, 0xa32, 0xa28, 0x20, 0xa2c, 0xa1f, 0xa4b, 0xa30, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa09, 0xa32, 0xa28, 0x20, 0xa2c, 0xa1f, 0xa4b, 0xa30, 0x20,
0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa32,
0xa28, 0x20, 0xa2c, 0xa1f, 0xa4b, 0xa30, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06,
0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa3e, 0xa38,
0xa15, 0xa4b, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa3e, 0xa38, 0xa15, 0xa4b,
0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e,
0xa3e, 0xa38, 0xa15, 0xa4b, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20,
0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa3f, 0xa06, 0xa02, 0xa2e,
0xa3e, 0xa30, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa28, 0xa3e, 0xa09, 0xa30, 0xa42,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa28, 0xa47, 0xa2a, 0xa3e, 0xa32, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa28, 0xa3f, 0xa0a, 0x20, 0xa15, 0xa48, 0xa32, 0xa47, 0xa21,
0xa4b, 0xa28, 0xa40, 0xa06, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa28, 0xa3f, 0xa0a,
0x20, 0xa15, 0xa48, 0xa32, 0xa47, 0xa21, 0xa4b, 0xa28, 0xa40, 0xa06, 0x20, 0xa2e,
0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa28, 0xa3f, 0xa0a,
0x20, 0xa15, 0xa48, 0xa32, 0xa47, 0xa21, 0xa4b, 0xa28, 0xa40, 0xa06, 0x20, 0xa17,
0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa28, 0xa3f, 0xa0a, 0xa1c, 0xa3c, 0xa40, 0xa32, 0xa48, 0xa02, 0xa21, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa28, 0xa3f, 0xa0a, 0xa1c, 0xa3c, 0xa40, 0xa32, 0xa48,
0xa02, 0xa21, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa28, 0xa3f, 0xa0a, 0xa1c, 0xa3c, 0xa40, 0xa32, 0xa48, 0xa02, 0xa21, 0x20,
0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa28, 0xa3f, 0xa0a, 0xa2b, 0xa3e, 0xa09, 0xa02, 0xa21, 0xa32, 0xa48, 0xa02, 0xa21,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa28, 0xa3f, 0xa0a, 0xa2b, 0xa3e, 0xa09, 0xa02,
0xa21, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa28, 0xa3f, 0xa0a, 0xa2b, 0xa3e, 0xa09, 0xa02, 0xa21,
0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa28, 0xa40, 0xa0a, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa28, 0xa4b, 0xa30, 0xa2b, 0xa4c, 0xa15, 0x20, 0xa06, 0xa08, 0xa32, 0xa48,
0xa02, 0xa21, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa28, 0xa4b, 0xa30, 0xa2b, 0xa4c,
0xa15, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa2e, 0xa3f, 0xa06,
0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa28, 0xa4b, 0xa30, 0xa2b, 0xa4c,
0xa15, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa2a, 0xa4d, 0xa30,
0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2b, 0xa30, 0xa28,
0xa3e, 0xa02, 0xa21, 0xa4b, 0x20, 0xa21, 0xa47, 0x20, 0xa28, 0xa4b, 0xa30, 0xa4b,
0xa28, 0xa39, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2b, 0xa30, 0xa28, 0xa3e,
0xa02, 0xa21, 0xa4b, 0x20, 0xa21, 0xa47, 0x20, 0xa28, 0xa4b, 0xa30, 0xa4b, 0xa28,
0xa39, 0xa3e, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa2b, 0xa30, 0xa28, 0xa3e, 0xa02, 0xa21, 0xa4b, 0x20, 0xa21, 0xa47, 0x20,
0xa28, 0xa4b, 0xa30, 0xa4b, 0xa28, 0xa39, 0xa3e, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40,
0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa71,
0xa24, 0xa30, 0xa40, 0x20, 0xa2e, 0xa30, 0xa3f, 0xa06, 0xa28, 0xa3e, 0x20, 0xa06,
0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0xa28, 0xa4c,
0xa35, 0xa4b, 0xa38, 0xa3f, 0xa2c, 0xa40, 0xa30, 0xa38, 0xa15, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa28, 0xa4c, 0xa35, 0xa4b, 0xa38, 0xa3f, 0xa2c, 0xa40, 0xa30, 0xa38,
0xa15, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa28, 0xa4c, 0xa35, 0xa4b, 0xa38, 0xa3f, 0xa2c, 0xa40, 0xa30, 0xa38, 0xa15, 0x20,
0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa13, 0xa2e, 0xa38, 0xa15, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa13,
0xa2e, 0xa38, 0xa15, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa13, 0xa2e, 0xa38, 0xa15, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06,
0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa3e, 0xa15,
0xa3f, 0xa38, 0xa24, 0xa3e, 0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa3e,
0xa15, 0xa3f, 0xa38, 0xa24, 0xa3e, 0xa28, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa3e, 0xa15, 0xa3f, 0xa38, 0xa24, 0xa3e,
0xa28, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa32, 0xa3e, 0xa09, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa2a, 0xa3e, 0xa2a, 0xa42, 0xa06, 0x20, 0xa28, 0xa3f, 0xa0a, 0x20, 0xa17,
0xa3f, 0xa28, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa48, 0xa30, 0xa3e,
0xa17, 0xa35, 0xa47, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa48, 0xa30, 0xa3e,
0xa17, 0xa35, 0xa47, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa2a, 0xa48, 0xa30, 0xa3e, 0xa17, 0xa35, 0xa47, 0x20, 0xa17, 0xa30,
0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa2a, 0xa47, 0xa30, 0xa42, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa47, 0xa30,
0xa42, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa2a, 0xa47, 0xa30, 0xa42, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20,
0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2b, 0xa3f, 0xa32, 0xa3f, 0xa2a,
0xa40, 0xa28, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2b, 0xa3f, 0xa32, 0xa3f,
0xa2a, 0xa40, 0xa28, 0xa40, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa2b, 0xa3f, 0xa32, 0xa3f, 0xa2a, 0xa40, 0xa28, 0xa40, 0x20,
0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa2b, 0xa3f, 0xa28, 0xa3f, 0xa15, 0xa38, 0x20, 0xa06, 0xa08, 0xa32,
0xa48, 0xa02, 0xa21, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa38, 0xa48, 0xa02, 0xa1f,
0x20, 0xa2a, 0xa40, 0xa05, 0xa30, 0xa47, 0x20, 0xa05, 0xa24, 0xa47, 0x20, 0xa2e,
0xa3f, 0xa15, 0xa47, 0xa32, 0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa38, 0xa48,
0xa02, 0xa1f, 0x20, 0xa2a, 0xa40, 0xa05, 0xa30, 0xa47, 0x20, 0xa05, 0xa24, 0xa47,
0x20, 0xa2e, 0xa3f, 0xa15, 0xa47, 0xa32, 0xa28, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30,
0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa38, 0xa48, 0xa02, 0xa1f, 0x20, 0xa2a,
0xa40, 0xa05, 0xa30, 0xa47, 0x20, 0xa05, 0xa24, 0xa47, 0x20, 0xa2e, 0xa3f, 0xa15,
0xa47, 0xa32, 0xa28, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa3f, 0xa1f, 0xa15, 0xa48, 0xa30, 0xa28, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa4b, 0xa28, 0xa3e, 0xa2a, 0xa47, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa2a, 0xa2f, 0xa4b, 0xa02, 0xa17, 0xa2f, 0xa3e, 0xa02, 0xa17,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa3f, 0xa1c, 0xa3c, 0xa32, 0xa4b, 0xa30,
0xa21, 0xa3e, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0xa15, 0xa3f, 0xa1c, 0xa3c, 0xa32,
0xa4b, 0xa30, 0xa21, 0xa3e, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa38,
0xa2e, 0xa3e, 0xa02, 0xa15, 0xa3f, 0xa1c, 0xa3c, 0xa32, 0xa4b, 0xa30, 0xa21, 0xa3e,
0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0x2d, 0xa30, 0xa41, 0xa71, 0xa24, 0x20, 0xa38,
0xa2e, 0xa3e, 0xa02, 0xa30, 0xa3f, 0xa2f, 0xa42, 0xa28, 0xa40, 0xa05, 0xa28, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa30, 0xa4b, 0xa25, 0xa47, 0xa30, 0xa3e, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa38, 0xa16, 0xa32, 0xa40, 0xa28, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa38, 0xa16, 0xa32, 0xa40, 0xa28, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa38, 0xa16, 0xa32, 0xa40, 0xa28, 0x20, 0xa17,
0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa38, 0xa3e, 0xa2e, 0xa4b, 0xa06, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa38,
0xa3e, 0xa2e, 0xa4b, 0xa06, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa38, 0xa3e, 0xa2e, 0xa4b, 0xa06, 0x20, 0xa2a, 0xa4d, 0xa30,
0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa38, 0xa47, 0xa38,
0xa3c, 0xa32, 0xa38, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa38, 0xa3f, 0xa70, 0xa17,
0xa3e, 0xa2a, 0xa41, 0xa30, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa38, 0xa4b, 0xa32, 0xa4b, 0xa2e, 0xa28, 0x20, 0xa06, 0xa08,
0xa32, 0xa48, 0xa02, 0xa21, 0xa38, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa26, 0xa71,
0xa16, 0xa23, 0xa40, 0x20, 0xa1c, 0xa3e, 0xa30, 0xa1c, 0xa40, 0xa06, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa38, 0xa42, 0xa30, 0xa40, 0xa28, 0xa3e, 0xa2e, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa38, 0xa3f, 0xa13, 0xa35, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa24, 0xa3e, 0xa39, 0xa40, 0xa24, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa24, 0xa48, 0xa2a, 0xa08, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa24, 0xa48, 0xa2a,
0xa08, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa24, 0xa48, 0xa2a, 0xa08, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa24, 0xa3e, 0xa1c, 0xa3f, 0xa15, 0xa3f, 0xa38,
0xa24, 0xa3e, 0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1f, 0xa4b, 0xa15, 0xa47,
0xa32, 0xa3e, 0xa09, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1f, 0xa4b, 0xa02, 0xa17,
0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1f, 0xa4b, 0xa02, 0xa17, 0xa3e, 0x20,
0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1f, 0xa4b,
0xa02, 0xa17, 0xa3e, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26,
0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1a, 0xa42, 0xa15, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa24, 0xa41, 0xa30, 0xa15, 0xa2e, 0xa47, 0xa28, 0xa3f, 0xa38, 0xa24,
0xa3e, 0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa24, 0xa41, 0xa30, 0xa15, 0xa2e,
0xa47, 0xa28, 0xa3f, 0xa38, 0xa24, 0xa3e, 0xa28, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30,
0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa24, 0xa41, 0xa30, 0xa15, 0xa2e, 0xa47,
0xa28, 0xa3f, 0xa38, 0xa24, 0xa3e, 0xa28, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06,
0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1f, 0xa41, 0xa35,
0xa3e, 0xa32, 0xa42, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa30, 0xa42, 0xa17,
0xa35, 0xa47, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa30, 0xa42, 0xa17, 0xa35,
0xa47, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa09, 0xa30, 0xa42, 0xa17, 0xa35, 0xa47, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06,
0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa1c, 0xa3c,
0xa2c, 0xa47, 0xa15, 0xa3f, 0xa38, 0xa24, 0xa3e, 0xa28, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa09, 0xa1c, 0xa3c, 0xa2c, 0xa47, 0xa15, 0xa3f, 0xa38, 0xa24, 0xa3e, 0xa28,
0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09,
0xa1c, 0xa3c, 0xa2c, 0xa47, 0xa15, 0xa3f, 0xa38, 0xa24, 0xa3e, 0xa28, 0x20, 0xa17,
0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa35, 0xa3e, 0xa28, 0xa42, 0xa06, 0xa1f, 0xa42, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa35, 0xa3e, 0xa28, 0xa42, 0xa06, 0xa1f, 0xa42, 0x20, 0xa2e, 0xa3f, 0xa06,
0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa35, 0xa3e, 0xa28, 0xa42, 0xa06,
0xa1f, 0xa42, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa35, 0xa48, 0xa28, 0xa47, 0xa1c, 0xa3c, 0xa42,
0xa0f, 0xa32, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa35, 0xa32, 0xa3e, 0xa26,
0xa40, 0xa35, 0xa4b, 0xa38, 0xa24, 0xa15, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa35,
0xa32, 0xa3e, 0xa26, 0xa40, 0xa35, 0xa4b, 0xa38, 0xa24, 0xa15, 0x20, 0xa2e, 0xa3f,
0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa35, 0xa32, 0xa3e, 0xa26,
0xa40, 0xa35, 0xa4b, 0xa38, 0xa24, 0xa15, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06,
0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa35, 0xa4b, 0xa32,
0xa17, 0xa4b, 0xa17, 0xa4d, 0xa30, 0xa47, 0xa21, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa35, 0xa4b, 0xa32, 0xa17, 0xa4b, 0xa17, 0xa4d, 0xa30, 0xa47, 0xa21, 0x20, 0xa2e,
0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa35, 0xa4b, 0xa32,
0xa17, 0xa4b, 0xa17, 0xa4d, 0xa30, 0xa47, 0xa21, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40,
0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa35, 0xa4b,
0xa38, 0xa1f, 0xa4b, 0xa15, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa35, 0xa47, 0xa15,
0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa35, 0xa3e, 0xa32, 0xa3f, 0xa38, 0x20, 0xa05, 0xa24, 0xa47, 0x20, 0xa2b, 0xa41,
0xa1f, 0xa42, 0xa28, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2f, 0xa15, 0xa41,
0xa24, 0xa38, 0xa15, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2f, 0xa15, 0xa41, 0xa24,
0xa38, 0xa15, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa2f, 0xa15, 0xa41, 0xa24, 0xa38, 0xa15, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40,
0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2f, 0xa15,
0xa47, 0xa24, 0xa30, 0xa3f, 0xa28, 0xa2c, 0xa30, 0xa17, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa2f, 0xa15, 0xa47, 0xa24, 0xa30, 0xa3f, 0xa28, 0xa2c, 0xa30, 0xa17, 0x20,
0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2f, 0xa15,
0xa47, 0xa24, 0xa30, 0xa3f, 0xa28, 0xa2c, 0xa30, 0xa17, 0x20, 0xa17, 0xa30, 0xa2e,
0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2f,
0xa42, 0xa15, 0xa4b, 0xa28, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73,
0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41,
0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x66, 0x72,
0x69, 0x63, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x75, 0x64, 0x61,
0x66, 0x72, 0x69, 0x63, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41,
0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63,
0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x45, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x45, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x45,
0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x45, 0x73,
0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61,
0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61, 0xf1, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x61, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61, 0xf1, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x4d, 0x6f, 0x6e, 0x74,
0x61, 0xf1, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63,
0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x4f, 0x65, 0x73, 0x74, 0x65, 0x20, 0x64, 0x65,
0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x4f, 0x65, 0x73, 0x74, 0x65, 0x20, 0x64, 0x65,
0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x4f, 0x65, 0x73, 0x74, 0x65, 0x20, 0x64,
0x65, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e,
0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e,
0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x41, 0x74, 0x6c, 0xe1, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x41, 0x74, 0x6c, 0xe1, 0x6e, 0x74, 0x69, 0x63, 0x6f,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x44, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x41, 0x74, 0x6c, 0xe1, 0x6e,
0x74, 0x69, 0x63, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x4f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x79, 0xe1, 0x6e, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x79,
0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x61, 0x69, 0x79, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73,
0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x73, 0x68, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61,
0x62, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x43, 0x61, 0x62, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72,
0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x43, 0x68, 0x72,
0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x63, 0x6f,
0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0x6f,
0x6d, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x43, 0x6f, 0x63, 0x6f,
0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20,
0x43, 0x6f, 0x63, 0x6f, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75,
0x62, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50,
0x61, 0x73, 0x63, 0x75, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x75,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45, 0x63, 0x75,
0x61, 0x64, 0x6f, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x6d, 0xe1, 0x73, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61,
0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61,
0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73,
0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x46, 0x69, 0x6a, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x47, 0x75, 0x61, 0x79, 0x61, 0x6e, 0x61, 0x20, 0x46, 0x72, 0x61, 0x6e,
0x63, 0x65, 0x73, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x46, 0x72, 0x61,
0x6e, 0x63, 0xe9, 0x73, 0x20, 0x6d, 0x65, 0x72, 0x69, 0x64, 0x69, 0x6f,
0x6e, 0x61, 0x6c, 0x20, 0x79, 0x20, 0x61, 0x6e, 0x74, 0xe1, 0x72, 0x74,
0x69, 0x63, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x73, 0x20, 0x47, 0x61, 0x6c, 0xe1, 0x70, 0x61, 0x67,
0x6f, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x65,
0x6f, 0x72, 0x67, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x47, 0x65,
0x6f, 0x72, 0x67, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x47, 0x69, 0x6c, 0x62,
0x65, 0x72, 0x74, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x4d, 0x65, 0x72, 0x69, 0x64, 0x69, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72,
0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x47, 0x6f, 0x6c, 0x66, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x2d, 0x41, 0x6c, 0x65,
0x75, 0x74, 0x69, 0x61, 0x6e, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x48,
0x61, 0x77, 0x61, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61,
0x6e, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61,
0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x6f, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e,
0x67, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6e, 0x67, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76,
0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4f, 0x63, 0x65, 0x61, 0x6e, 0x6f,
0x20, 0xcd, 0x6e, 0x64, 0x69, 0x63, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x49, 0x72, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x49, 0x72, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72,
0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4a, 0x61,
0x70, 0xf3, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4a, 0x61, 0x70, 0xf3,
0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4a, 0x61, 0x70, 0xf3, 0x6e,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x7a, 0x61,
0x6a, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6a, 0x69, 0x73, 0x74, 0xe1, 0x6e,
0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6a, 0x69, 0x73,
0x74, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4f, 0x65, 0x73, 0x74,
0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x4b, 0x69, 0x72, 0x67, 0x75, 0x69, 0x73, 0x74, 0xe1,
0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45, 0x73, 0x70,
0xf3, 0x72, 0x61, 0x64, 0x61, 0x73, 0x20, 0x45, 0x63, 0x75, 0x61, 0x74,
0x6f, 0x72, 0x69, 0x61, 0x6c, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x6c,
0x64, 0x69, 0x76, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20,
0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6f,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69,
0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x63, 0x69, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x4d, 0x65, 0x78,
0x69, 0x63, 0x61, 0x6e, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50,
0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x4d, 0x65, 0x78, 0x69,
0x63, 0x61, 0x6e, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50,
0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x4d, 0x65, 0x78, 0x69,
0x63, 0x61, 0x6e, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x55, 0x6c, 0xe1, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x55, 0x6c, 0xe1, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f,
0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0xe1, 0x6e, 0x20,
0x42, 0x61, 0x74, 0x6f, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f,
0x73, 0x63, 0xfa, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73,
0x63, 0xfa, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x65,
0x70, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x75, 0x65, 0x76,
0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x75, 0x65, 0x76, 0x61, 0x20, 0x43,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x75, 0x65, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e,
0x64, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x75, 0x65, 0x76,
0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x6e, 0x6f,
0x76, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73,
0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x6b, 0x69,
0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x75, 0x65, 0x76,
0x61, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x79, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50,
0x65, 0x72, 0xfa, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x72, 0xfa,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x72, 0xfa, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69,
0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x46,
0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x50, 0x68,
0x6f, 0x65, 0x6e, 0x69, 0x78, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x53, 0x61, 0x6e, 0x20, 0x50, 0x65, 0x64, 0x72, 0x6f, 0x20, 0x79,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0xf3, 0x6e, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x53, 0x61, 0x6e, 0x20, 0x50, 0x65, 0x64, 0x72, 0x6f, 0x20,
0x79, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0xf3, 0x6e, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6e, 0x20, 0x50, 0x65, 0x64, 0x72,
0x6f, 0x20, 0x79, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0xf3, 0x6e,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x6f, 0x68, 0x6e,
0x70, 0x65, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50,
0x69, 0x6f, 0x6e, 0x79, 0x61, 0x6e, 0x67, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x65,
0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x53,
0x61, 0x6c, 0x6f, 0x6d, 0xf3, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x53, 0x75, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61,
0x69, 0x70, 0xe9, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61,
0x69, 0x70, 0xe9, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x54, 0x61, 0x79, 0x69, 0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x79, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x79, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x55, 0x7a, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x20, 0x57, 0x61, 0x6b, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x59,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x45, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x6f, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x20, 0x55, 0x72,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x41, 0x63, 0x72, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x41, 0x63,
0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72,
0x103, 0x20, 0x41, 0x63, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x41, 0x66,
0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x75, 0x6c, 0x75, 0x69,
0x4f, 0x72, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x69, 0x69, 0x20,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x41, 0x66, 0x72, 0x69, 0x63, 0x69, 0x69, 0x20, 0x4f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x41, 0x66, 0x72,
0x69, 0x63, 0x69, 0x69, 0x20, 0x4d, 0x65, 0x72, 0x69, 0x64, 0x69, 0x6f,
0x6e, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69,
0x63, 0x69, 0x69, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x69,
0x69, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103,
0x20, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x69, 0x69, 0x20, 0x4f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x41, 0x6c, 0x6d, 0x61,
0x74, 0x79, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72,
0x103, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x4f, 0x72, 0x61, 0x20,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61,
0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x75, 0x6c, 0x75, 0x69, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61,
0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x75, 0x6c, 0x75, 0x69, 0x4f,
0x72, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x103, 0x20,
0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61,
0x6e, 0x103, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x103, 0x20,
0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61,
0x6e, 0x103, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72,
0x103, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x103, 0x20, 0x6e,
0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e,
0x103, 0x4f, 0x72, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x103, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x63, 0x61, 0x6e, 0x103, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x103, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x61, 0x6e, 0x103, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x103, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x61, 0x6e, 0x103, 0x4f, 0x72, 0x61, 0x20, 0x7a, 0x6f,
0x6e, 0x65, 0x69, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x65, 0x20,
0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61,
0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0xee, 0x6e, 0x20, 0x7a, 0x6f, 0x6e, 0x61, 0x20, 0x6d,
0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x103, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x103, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0xee, 0x6e, 0x20,
0x7a, 0x6f, 0x6e, 0x61, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x103,
0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0x6e, 0x103, 0x4f, 0x72, 0x61, 0x20, 0x7a, 0x6f, 0x6e, 0x65, 0x69,
0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x6e, 0x6f, 0x72,
0x64, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0xee, 0x6e, 0x20, 0x7a, 0x6f, 0x6e, 0x61, 0x20, 0x50, 0x61, 0x63, 0x69,
0x66, 0x69, 0x63, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x61, 0x6e, 0x103, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0xee, 0x6e, 0x20, 0x7a, 0x6f, 0x6e,
0x61, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x6e, 0x6f,
0x72, 0x64, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x103,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x41, 0x6e, 0x61, 0x64,
0x79, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79,
0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x41, 0x70, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x41, 0x70, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x41, 0x70, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x41, 0x71, 0x74, 0x61,
0x75, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x7a, 0x6f, 0x6e,
0x65, 0x69, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x4f, 0x72, 0x61, 0x20,
0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103,
0x20, 0x61, 0x20, 0x7a, 0x6f, 0x6e, 0x65, 0x69, 0x20, 0x41, 0x71, 0x74,
0x6f, 0x62, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x61, 0x72, 0x61, 0x62, 0x103,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x61, 0x72, 0x61, 0x62, 0x103, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x72, 0x61, 0x62, 0x103, 0x4f,
0x72, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x65,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72,
0x103, 0x20, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x65, 0x69, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x65, 0x69, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x65, 0x69, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x41, 0x72, 0x6d,
0x65, 0x6e, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x7a, 0x6f, 0x6e, 0x65, 0x69, 0x20,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x20, 0x6e, 0x6f, 0x72,
0x64, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0xee, 0x6e, 0x20, 0x7a, 0x6f, 0x6e, 0x61, 0x20, 0x41, 0x74, 0x6c, 0x61,
0x6e, 0x74, 0x69, 0x63, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x103, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0xee, 0x6e, 0x20, 0x7a, 0x6f,
0x6e, 0x61, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x20,
0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61,
0x6e, 0x103, 0x4f, 0x72, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x65, 0x69, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x65, 0x69, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x69, 0x20,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x69, 0x20, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x69, 0x20, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61,
0x72, 0x103, 0x20, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x65, 0x69, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20,
0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65,
0x69, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x69,
0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x69, 0x20, 0x4f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x69, 0x20, 0x4f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x69, 0x20,
0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x69, 0x20,
0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x6a,
0x61, 0x6e, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x69, 0x64, 0x6a, 0x61, 0x6e, 0x75, 0x6c, 0x75, 0x69,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20,
0x61, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x6a, 0x61,
0x6e, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x41, 0x7a, 0x6f, 0x72,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x4f, 0x72, 0x61, 0x20, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x75, 0x6c,
0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69,
0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c,
0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c,
0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61,
0x72, 0x103, 0x20, 0x61, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69,
0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x42, 0x72,
0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61,
0x6c, 0x61, 0x6d, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x43,
0x61, 0x70, 0x75, 0x6c, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x43, 0x61, 0x70, 0x75, 0x6c, 0x20, 0x56, 0x65, 0x72,
0x64, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72,
0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x43, 0x61, 0x70, 0x75, 0x6c, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x43,
0x68, 0x69, 0x6e, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6e,
0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72,
0x103, 0x20, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x65, 0x69, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61,
0x20, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x4f, 0x72,
0x61, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x6c, 0x6f, 0x72, 0x20,
0x43, 0x6f, 0x63, 0x6f, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x43, 0x6f, 0x6c,
0x75, 0x6d, 0x62, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x43, 0x6f, 0x6c,
0x75, 0x6d, 0x62, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0x75,
0x6d, 0x62, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x49, 0x6e, 0x73,
0x75, 0x6c, 0x65, 0x6c, 0x6f, 0x72, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x6c, 0x6f, 0x72, 0x20,
0x43, 0x6f, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65,
0x6c, 0x6f, 0x72, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20,
0x43, 0x75, 0x62, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x43, 0x75, 0x62, 0x65,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103,
0x20, 0x61, 0x20, 0x43, 0x75, 0x62, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d,
0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x75, 0x6c, 0x75, 0x69, 0x20, 0x64,
0x65, 0x20, 0x45, 0x73, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x50, 0x61, 0x219, 0x74,
0x65, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x49, 0x6e, 0x73,
0x75, 0x6c, 0x61, 0x20, 0x50, 0x61, 0x219, 0x74, 0x65, 0x6c, 0x75, 0x69,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x50,
0x61, 0x219, 0x74, 0x65, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x45,
0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72,
0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x20, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x69, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103,
0x20, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x20, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x20, 0x64, 0x65, 0x20, 0x45, 0x73,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x20,
0x64, 0x65, 0x20, 0x45, 0x73, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x69, 0x20, 0x64, 0x65, 0x20, 0x45, 0x73, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x20, 0x64, 0x65,
0x20, 0x45, 0x73, 0x74, 0x20, 0xee, 0x6e, 0x64, 0x65, 0x70, 0x103, 0x72,
0x74, 0x61, 0x74, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x69, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x73, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x73, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x69, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x73, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65,
0x6c, 0x65, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x6c, 0x65,
0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x6c, 0x65, 0x20, 0x46, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x46,
0x69, 0x6a, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61,
0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e,
0x61, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x7a, 0x103, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x54, 0x65, 0x72, 0x69, 0x74, 0x6f,
0x72, 0x69, 0x69, 0x6c, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x65, 0x20, 0x219, 0x69, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x63,
0x74, 0x69, 0x63, 0x65, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x7a,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x47, 0x61, 0x6c,
0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x4f, 0x72, 0x61,
0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x47, 0x65,
0x6f, 0x72, 0x67, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x49, 0x6e,
0x73, 0x75, 0x6c, 0x65, 0x6c, 0x6f, 0x72, 0x20, 0x47, 0x69, 0x6c, 0x62,
0x65, 0x72, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x77, 0x68, 0x69, 0x63, 0x68, 0x4f, 0x72, 0x61, 0x20,
0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x69, 0x20,
0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20,
0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x69, 0x20,
0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x47,
0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x69, 0x20, 0x6f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x69, 0x20,
0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x65,
0x69, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103,
0x20, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x65, 0x69, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x61, 0x20, 0x47, 0x6f, 0x6c, 0x66, 0x75, 0x6c, 0x75,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x6e, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x48,
0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x48, 0x6f, 0x76, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x69,
0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x4f, 0x63, 0x65, 0x61, 0x6e, 0x75,
0x6c, 0x75, 0x69, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x4f, 0x72,
0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x65, 0x69,
0x4f, 0x72, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69,
0x65, 0x69, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x65,
0x69, 0x20, 0x64, 0x65, 0x20, 0x45, 0x73, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x65, 0x69, 0x20, 0x64,
0x65, 0x20, 0x56, 0x65, 0x73, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x49, 0x72,
0x61, 0x6e, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x49, 0x72, 0x61,
0x6e, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x75,
0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x49,
0x72, 0x6b, 0x75, 0x21b, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x49, 0x72,
0x6b, 0x75, 0x21b, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x49, 0x72, 0x6b, 0x75,
0x21b, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x4a, 0x61, 0x70,
0x6f, 0x6e, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x4a, 0x61, 0x70, 0x6f,
0x6e, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x69,
0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x50, 0x65,
0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d,
0x4b, 0x61, 0x6d, 0x63, 0x69, 0x61, 0x21b, 0x6b, 0x69, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x69, 0x61, 0x21b, 0x6b,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61,
0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x69,
0x61, 0x21b, 0x6b, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x4b, 0x61, 0x7a, 0x61, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x68, 0x73, 0x74,
0x61, 0x6e, 0x75, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x45, 0x73, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x68,
0x73, 0x74, 0x61, 0x6e, 0x75, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x73, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x65, 0x69,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x61, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x65, 0x69, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x43,
0x6f, 0x72, 0x65, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x69, 0x61, 0x72,
0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x69, 0x61, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x69, 0x61, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x4b, 0xe2, 0x72, 0x67, 0xe2, 0x7a, 0x73,
0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x49,
0x6e, 0x73, 0x75, 0x6c, 0x65, 0x6c, 0x65, 0x20, 0x4c, 0x69, 0x6e, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4d, 0x61, 0x6c, 0x61, 0x79, 0x73,
0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4d, 0x61,
0x6c, 0x64, 0x69, 0x76, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x49, 0x6e, 0x73,
0x75, 0x6c, 0x65, 0x6c, 0x6f, 0x72, 0x20, 0x4d, 0x61, 0x72, 0x63, 0x68,
0x69, 0x7a, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c,
0x65, 0x6c, 0x6f, 0x72, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c,
0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x4f,
0x72, 0x61, 0x20, 0x7a, 0x6f, 0x6e, 0x65, 0x69, 0x20, 0x50, 0x61, 0x63,
0x69, 0x66, 0x69, 0x63, 0x20, 0x6d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x61, 0x20, 0x7a, 0x6f, 0x6e, 0x65, 0x69, 0x20, 0x50, 0x61,
0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x6d, 0x65, 0x78, 0x69, 0x63, 0x61,
0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72,
0x103, 0x20, 0x61, 0x20, 0x7a, 0x6f, 0x6e, 0x65, 0x69, 0x20, 0x50, 0x61,
0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x6d, 0x65, 0x78, 0x69, 0x63, 0x61,
0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x55, 0x6c,
0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f,
0x72, 0x4f, 0x72, 0x61, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x76, 0x65,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x61, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x76, 0x65, 0x69,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20,
0x61, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x76, 0x65, 0x69, 0x4f, 0x72,
0x61, 0x20, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x75, 0x6c, 0x75,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4e, 0x61, 0x75,
0x72, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x75,
0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x4e, 0x6f, 0x69, 0x69, 0x20,
0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x69, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20,
0x4e, 0x6f, 0x69, 0x69, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x69, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72,
0x103, 0x20, 0x61, 0x20, 0x4e, 0x6f, 0x69, 0x69, 0x20, 0x43, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x4e, 0x6f,
0x69, 0x69, 0x20, 0x5a, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x4e, 0x6f, 0x69, 0x69, 0x20, 0x5a, 0x65, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61,
0x72, 0x103, 0x20, 0x61, 0x20, 0x4e, 0x6f, 0x69, 0x69, 0x20, 0x5a, 0x65,
0x65, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f,
0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4e,
0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x69, 0x20, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x49, 0x6e, 0x73,
0x75, 0x6c, 0x65, 0x69, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20,
0x61, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x69, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4f, 0x6d,
0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x75, 0x6c, 0x75, 0x69, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x50, 0x61, 0x70, 0x75,
0x61, 0x20, 0x4e, 0x6f, 0x75, 0x61, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x50, 0x61, 0x72,
0x61, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x50, 0x65, 0x72, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x50,
0x65, 0x72, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61,
0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x50, 0x65, 0x72, 0x75, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x49, 0x6e, 0x73, 0x75,
0x6c, 0x65, 0x6c, 0x6f, 0x72, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69,
0x78, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x69,
0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x219, 0x69,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x20, 0x219, 0x69, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72,
0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x219, 0x69, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x50, 0x79, 0x6f, 0x6e,
0x67, 0x79, 0x61, 0x6e, 0x67, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72,
0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x6d,
0x61, 0x72, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72,
0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x6d, 0x6f,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c,
0x65, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x53, 0x69,
0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x49,
0x6e, 0x73, 0x75, 0x6c, 0x65, 0x6c, 0x6f, 0x72, 0x20, 0x53, 0x6f, 0x6c,
0x6f, 0x6d, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72,
0x67, 0x69, 0x65, 0x69, 0x20, 0x64, 0x65, 0x20, 0x53, 0x75, 0x64, 0x4f,
0x72, 0x61, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x75, 0x6c,
0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x53, 0x79,
0x6f, 0x77, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x54,
0x61, 0x68, 0x69, 0x74, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x54, 0x61,
0x69, 0x70, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x54, 0x61, 0x64, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c,
0x61, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x54, 0x6f, 0x6e, 0x67,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x79, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20,
0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x75, 0x6c, 0x75, 0x69, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61,
0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x75, 0x6c, 0x75, 0x69,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x55, 0x7a, 0x62, 0x65,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x55,
0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72,
0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x4f, 0x72, 0x61, 0x20, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65,
0x6c, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72,
0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x56, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c,
0x65, 0x69, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x219, 0x69,
0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x49, 0x61, 0x6b, 0x75, 0x21b, 0x6b, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x49, 0x61, 0x6b, 0x75, 0x21b, 0x6b, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x49, 0x61, 0x6b, 0x75, 0x21b, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x45,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e,
0x54, 0x65, 0x6d, 0x70, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x54, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x54, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73, 0x74, 0x61, 0x64,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x54, 0x65, 0x6d, 0x70,
0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x54, 0x65, 0x6d,
0x70, 0x20, 0x64, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x54, 0x65,
0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73, 0x74, 0x61, 0x64, 0x20, 0x6f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x54, 0x65, 0x6d, 0x70, 0x20,
0x64, 0x61, 0x20, 0x6d, 0x75, 0x6e, 0x74, 0x6f, 0x67, 0x6e, 0x61, 0x54,
0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x61, 0x20, 0x6d, 0x75, 0x6e, 0x74, 0x6f,
0x67, 0x6e, 0x61, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73,
0x74, 0x61, 0x64, 0x20, 0x64, 0x61, 0x20, 0x6d, 0x75, 0x6e, 0x74, 0x6f,
0x67, 0x6e, 0x61, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x70, 0x61, 0x63, 0x69,
0x66, 0x69, 0x63, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x70, 0x61, 0x63, 0x69,
0x66, 0x69, 0x63, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73,
0x74, 0x61, 0x64, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x54,
0x65, 0x6d, 0x70, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63,
0x54, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x63, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73, 0x74, 0x61,
0x64, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x54, 0x65,
0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x54,
0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x61, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61,
0x54, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73, 0x74, 0x61, 0x64,
0x20, 0x64, 0x61, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x54, 0x65, 0x6d,
0x70, 0x20, 0x64, 0x61, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x61, 0x54,
0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x61, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x61, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73, 0x74, 0x61,
0x64, 0x20, 0x64, 0x61, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x61, 0x54,
0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x64, 0x61, 0x6c, 0x20, 0x56, 0x65, 0x73, 0x74,
0x54, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x61, 0x20, 0x6c, 0x2019, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x64, 0x61, 0x6c, 0x20, 0x56, 0x65, 0x73,
0x74, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73, 0x74, 0x61,
0x64, 0x20, 0x64, 0x61, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x64, 0x61, 0x6c, 0x20, 0x56, 0x65, 0x73, 0x74, 0x54, 0x65,
0x6d, 0x70, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68,
0x410, 0x43a, 0x440, 0x438, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x43a,
0x440, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x43a, 0x440, 0x438,
0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x44f,
0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x412, 0x43e, 0x441, 0x442, 0x43e,
0x447, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x42e,
0x436, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x417,
0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x444, 0x440, 0x438,
0x43a, 0x430, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x44f, 0x20, 0x410,
0x444, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x444, 0x440,
0x438, 0x43a, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430,
0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x410, 0x43b, 0x43c, 0x430, 0x2d, 0x410, 0x442, 0x430, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x410, 0x43b, 0x43c, 0x430, 0x2d, 0x410, 0x442, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x43b, 0x43c, 0x430, 0x2d, 0x410,
0x442, 0x430, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a, 0x430, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d,
0x43a, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c,
0x43d, 0x430, 0x44f, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x426,
0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x44f, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x44f,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x412, 0x43e,
0x441, 0x442, 0x43e, 0x447, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x430, 0x44f,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x430, 0x44f,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x413, 0x43e,
0x440, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x20, 0x28,
0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x43c, 0x435,
0x440, 0x438, 0x43a, 0x430, 0x29, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x435, 0x20, 0x433, 0x43e, 0x440, 0x43d, 0x43e, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x20, 0x28, 0x421, 0x435, 0x432, 0x435, 0x440,
0x43d, 0x430, 0x44f, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x29,
0x41b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x433, 0x43e, 0x440, 0x43d, 0x43e,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x20, 0x28, 0x421, 0x435, 0x432,
0x435, 0x440, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a,
0x430, 0x29, 0x422, 0x438, 0x445, 0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441,
0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x422, 0x438, 0x445,
0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x422, 0x438, 0x445, 0x43e, 0x43e, 0x43a, 0x435, 0x430,
0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x412, 0x440, 0x435, 0x43c, 0x44f, 0x20,
0x43f, 0x43e, 0x20, 0x410, 0x43d, 0x430, 0x434, 0x44b, 0x440, 0x44e, 0x410, 0x43d,
0x430, 0x434, 0x44b, 0x440, 0x44c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410,
0x43d, 0x430, 0x434, 0x44b, 0x440, 0x44c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x43f, 0x438, 0x430, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x43f, 0x438, 0x430, 0x2c, 0x20,
0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x410, 0x43a, 0x442, 0x430, 0x443, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410,
0x43a, 0x442, 0x430, 0x443, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410,
0x43a, 0x442, 0x430, 0x443, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x43a, 0x442, 0x43e, 0x431, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x43a, 0x442, 0x43e, 0x431, 0x435, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x43a, 0x442, 0x43e, 0x431, 0x435, 0x20,
0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x421, 0x430, 0x443, 0x434, 0x43e, 0x432, 0x441, 0x43a, 0x430, 0x44f, 0x20, 0x410,
0x440, 0x430, 0x432, 0x438, 0x44f, 0x421, 0x430, 0x443, 0x434, 0x43e, 0x432, 0x441,
0x43a, 0x430, 0x44f, 0x20, 0x410, 0x440, 0x430, 0x432, 0x438, 0x44f, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x421, 0x430, 0x443, 0x434, 0x43e, 0x432, 0x441,
0x43a, 0x430, 0x44f, 0x20, 0x410, 0x440, 0x430, 0x432, 0x438, 0x44f, 0x2c, 0x20,
0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d,
0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x44f, 0x20,
0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x417, 0x430, 0x43f,
0x430, 0x434, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442,
0x438, 0x43d, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x417, 0x430,
0x43f, 0x430, 0x434, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d,
0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x438,
0x44f, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x440, 0x43c, 0x435,
0x43d, 0x438, 0x44f, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438,
0x447, 0x435, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x447, 0x435, 0x441, 0x43a, 0x43e,
0x435, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x442, 0x43b, 0x430, 0x43d,
0x442, 0x438, 0x447, 0x435, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x426, 0x435, 0x43d,
0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430,
0x43b, 0x44c, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x44f, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x426, 0x435,
0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x2c, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x426, 0x435, 0x43d,
0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x2c, 0x20, 0x437, 0x430, 0x43f, 0x430,
0x434, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x426, 0x435,
0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x2c, 0x20, 0x437, 0x430, 0x43f,
0x430, 0x434, 0x43d, 0x43e, 0x435, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x426,
0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x44f, 0x20, 0x410,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x2c, 0x20, 0x437, 0x430,
0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x435, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x412, 0x43e, 0x441, 0x442, 0x43e,
0x447, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x44f, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x430, 0x44f, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x430,
0x44f, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x2c,
0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x417, 0x430, 0x43f, 0x430, 0x434,
0x43d, 0x430, 0x44f, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x44f, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x417, 0x430, 0x43f, 0x430,
0x434, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x44f, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x434,
0x436, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x437,
0x435, 0x440, 0x431, 0x430, 0x439, 0x434, 0x436, 0x430, 0x43d, 0x2c, 0x20, 0x43b,
0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410,
0x437, 0x43e, 0x440, 0x441, 0x43a, 0x438, 0x435, 0x20, 0x43e, 0x2d, 0x432, 0x430,
0x410, 0x437, 0x43e, 0x440, 0x441, 0x43a, 0x438, 0x435, 0x20, 0x43e, 0x2d, 0x432,
0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x437, 0x43e, 0x440,
0x441, 0x43a, 0x438, 0x435, 0x20, 0x43e, 0x2d, 0x432, 0x430, 0x2c, 0x20, 0x43b,
0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x411,
0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448,
0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x44f, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438,
0x44f, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434,
0x435, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41a, 0x430, 0x431, 0x43e,
0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d,
0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x427, 0x430, 0x442, 0x435,
0x43c, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x427, 0x430, 0x442, 0x435,
0x43c, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x427, 0x438, 0x43b, 0x438, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x427, 0x438, 0x43b, 0x438, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d,
0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41a, 0x438, 0x442, 0x430,
0x439, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41a, 0x438, 0x442, 0x430,
0x439, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x43e, 0x2d, 0x432, 0x20, 0x420, 0x43e, 0x436, 0x434, 0x435,
0x441, 0x442, 0x432, 0x430, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x43e, 0x432, 0x44b,
0x435, 0x20, 0x43e, 0x2d, 0x432, 0x430, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431,
0x438, 0x44f, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41a, 0x43e, 0x43b,
0x443, 0x43c, 0x431, 0x438, 0x44f, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41e, 0x441, 0x442, 0x440, 0x43e,
0x432, 0x430, 0x20, 0x41a, 0x443, 0x43a, 0x430, 0x41e, 0x441, 0x442, 0x440, 0x43e,
0x432, 0x430, 0x20, 0x41a, 0x443, 0x43a, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x20, 0x41a, 0x443,
0x43a, 0x430, 0x2c, 0x20, 0x43f, 0x43e, 0x43b, 0x443, 0x43b, 0x435, 0x442, 0x43d,
0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41a, 0x443, 0x431, 0x430,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41a, 0x443, 0x431, 0x430, 0x2c,
0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x44b, 0x439, 0x20, 0x422,
0x438, 0x43c, 0x43e, 0x440, 0x41e, 0x2d, 0x432, 0x20, 0x41f, 0x430, 0x441, 0x445,
0x438, 0x41e, 0x2d, 0x432, 0x20, 0x41f, 0x430, 0x441, 0x445, 0x438, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41e, 0x2d, 0x432, 0x20, 0x41f, 0x430, 0x441,
0x445, 0x438, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c,
0x43d, 0x430, 0x44f, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x426, 0x435,
0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x44f, 0x20, 0x415, 0x432,
0x440, 0x43e, 0x43f, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x426,
0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x44f, 0x20, 0x415,
0x432, 0x440, 0x43e, 0x43f, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x412, 0x43e, 0x441, 0x442, 0x43e,
0x447, 0x43d, 0x430, 0x44f, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x412,
0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x430, 0x44f, 0x20, 0x415, 0x432, 0x440,
0x43e, 0x43f, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x412, 0x43e,
0x441, 0x442, 0x43e, 0x447, 0x43d, 0x430, 0x44f, 0x20, 0x415, 0x432, 0x440, 0x43e,
0x43f, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x41c, 0x43e, 0x441, 0x43a, 0x43e, 0x432, 0x441, 0x43a,
0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x417, 0x430, 0x43f, 0x430,
0x434, 0x43d, 0x430, 0x44f, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x417,
0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x44f, 0x20, 0x415, 0x432, 0x440, 0x43e,
0x43f, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x417, 0x430, 0x43f,
0x430, 0x434, 0x43d, 0x430, 0x44f, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430,
0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x441, 0x43a,
0x438, 0x435, 0x20, 0x43e, 0x2d, 0x432, 0x430, 0x424, 0x43e, 0x43b, 0x43a, 0x43b,
0x435, 0x43d, 0x434, 0x441, 0x43a, 0x438, 0x435, 0x20, 0x43e, 0x2d, 0x432, 0x430,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x424, 0x43e, 0x43b, 0x43a, 0x43b,
0x435, 0x43d, 0x434, 0x441, 0x43a, 0x438, 0x435, 0x20, 0x43e, 0x2d, 0x432, 0x430,
0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x424, 0x438, 0x434, 0x436, 0x438, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x424, 0x438, 0x434, 0x436, 0x438, 0x2c, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x424, 0x440, 0x430,
0x43d, 0x446, 0x443, 0x437, 0x441, 0x43a, 0x430, 0x44f, 0x20, 0x413, 0x432, 0x438,
0x430, 0x43d, 0x430, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x443, 0x437, 0x441, 0x43a,
0x438, 0x435, 0x20, 0x42e, 0x436, 0x43d, 0x44b, 0x435, 0x20, 0x438, 0x20, 0x410,
0x43d, 0x442, 0x430, 0x440, 0x43a, 0x442, 0x438, 0x447, 0x435, 0x441, 0x43a, 0x438,
0x435, 0x20, 0x442, 0x435, 0x440, 0x440, 0x438, 0x442, 0x43e, 0x440, 0x438, 0x438,
0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441, 0x441, 0x43a, 0x438,
0x435, 0x20, 0x43e, 0x2d, 0x432, 0x430, 0x413, 0x440, 0x443, 0x437, 0x438, 0x44f,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x413, 0x440, 0x443, 0x437, 0x438,
0x44f, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x43e, 0x2d, 0x432, 0x430, 0x20, 0x413, 0x438, 0x43b, 0x431,
0x435, 0x440, 0x442, 0x430, 0x421, 0x440, 0x435, 0x434, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x20, 0x43f, 0x43e, 0x20, 0x413, 0x440, 0x438,
0x43d, 0x432, 0x438, 0x447, 0x443, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d,
0x430, 0x44f, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438,
0x44f, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x430, 0x44f, 0x20, 0x413,
0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x430, 0x44f,
0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x2c,
0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x44f, 0x20, 0x413, 0x440,
0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x417, 0x430, 0x43f, 0x430,
0x434, 0x43d, 0x430, 0x44f, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d,
0x434, 0x438, 0x44f, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x417, 0x430,
0x43f, 0x430, 0x434, 0x43d, 0x430, 0x44f, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b,
0x430, 0x43d, 0x434, 0x438, 0x44f, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41f, 0x435, 0x440, 0x441, 0x438,
0x434, 0x441, 0x43a, 0x438, 0x439, 0x20, 0x437, 0x430, 0x43b, 0x438, 0x432, 0x413,
0x430, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x43e, 0x2d, 0x430, 0x43b, 0x435, 0x443,
0x442, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x413,
0x430, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x43e, 0x2d, 0x430, 0x43b, 0x435, 0x443,
0x442, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x413,
0x430, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x43e, 0x2d, 0x430, 0x43b, 0x435, 0x443,
0x442, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d,
0x433, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x413, 0x43e, 0x43d, 0x43a,
0x43e, 0x43d, 0x433, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x425, 0x43e, 0x432, 0x434, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x425, 0x43e, 0x432, 0x434, 0x2c, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x418, 0x43d,
0x434, 0x438, 0x439, 0x441, 0x43a, 0x438, 0x439, 0x20, 0x43e, 0x43a, 0x435, 0x430,
0x43d, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x44f,
0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x44f, 0x412, 0x43e,
0x441, 0x442, 0x43e, 0x447, 0x43d, 0x430, 0x44f, 0x20, 0x418, 0x43d, 0x434, 0x43e,
0x43d, 0x435, 0x437, 0x438, 0x44f, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430,
0x44f, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x44f, 0x418,
0x440, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x418, 0x440,
0x430, 0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441,
0x43a, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x418, 0x437, 0x440, 0x430, 0x438, 0x43b, 0x44c, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x418, 0x437, 0x440, 0x430, 0x438, 0x43b, 0x44c,
0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x42f, 0x43f, 0x43e, 0x43d, 0x438, 0x44f, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x42f, 0x43f, 0x43e, 0x43d, 0x438, 0x44f, 0x2c, 0x20, 0x43b,
0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41f,
0x435, 0x442, 0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a,
0x2d, 0x41a, 0x430, 0x43c, 0x447, 0x430, 0x442, 0x441, 0x43a, 0x438, 0x439, 0x41f,
0x435, 0x442, 0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a,
0x2d, 0x41a, 0x430, 0x43c, 0x447, 0x430, 0x442, 0x441, 0x43a, 0x438, 0x439, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41f, 0x435, 0x442, 0x440, 0x43e, 0x43f,
0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x2d, 0x41a, 0x430, 0x43c, 0x447,
0x430, 0x442, 0x441, 0x43a, 0x438, 0x439, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d,
0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x412, 0x43e, 0x441, 0x442,
0x43e, 0x447, 0x43d, 0x44b, 0x439, 0x20, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441,
0x442, 0x430, 0x43d, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x44b, 0x439, 0x20,
0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x41a, 0x43e, 0x440,
0x435, 0x44f, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41a, 0x43e, 0x440,
0x435, 0x44f, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440,
0x441, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41a, 0x440, 0x430,
0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x2c, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41a, 0x438, 0x440,
0x433, 0x438, 0x437, 0x438, 0x44f, 0x43e, 0x2d, 0x432, 0x430, 0x20, 0x41b, 0x430,
0x439, 0x43d, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x425, 0x430, 0x443, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x425, 0x430,
0x443, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x41c, 0x430, 0x43a, 0x430, 0x43e, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x41c, 0x430, 0x43a, 0x430, 0x43e, 0x2c, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41c, 0x430,
0x433, 0x430, 0x434, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41c, 0x430, 0x43b,
0x44c, 0x434, 0x438, 0x432, 0x44b, 0x41c, 0x430, 0x440, 0x43a, 0x438, 0x437, 0x441,
0x43a, 0x438, 0x435, 0x20, 0x43e, 0x2d, 0x432, 0x430, 0x41c, 0x430, 0x440, 0x448,
0x430, 0x43b, 0x43b, 0x43e, 0x432, 0x44b, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e,
0x432, 0x430, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438, 0x439, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438,
0x439, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x422, 0x438, 0x445, 0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d,
0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430,
0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x422,
0x438, 0x445, 0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x435,
0x20, 0x43c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e,
0x435, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x422, 0x438, 0x445, 0x43e, 0x43e,
0x43a, 0x435, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43c, 0x435, 0x43a,
0x441, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x423, 0x43b,
0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e,
0x440, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x2c, 0x20,
0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x41d, 0x43e, 0x432, 0x430, 0x44f, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e,
0x43d, 0x438, 0x44f, 0x41d, 0x43e, 0x432, 0x430, 0x44f, 0x20, 0x41a, 0x430, 0x43b,
0x435, 0x434, 0x43e, 0x43d, 0x438, 0x44f, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x41d, 0x43e, 0x432, 0x430, 0x44f, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434,
0x43e, 0x43d, 0x438, 0x44f, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41d, 0x43e, 0x432, 0x430, 0x44f, 0x20,
0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x41d, 0x43e, 0x432, 0x430,
0x44f, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41d, 0x43e, 0x432, 0x430, 0x44f, 0x20, 0x417,
0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x2c, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41d, 0x44c, 0x44e,
0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434,
0x43b, 0x435, 0x43d, 0x434, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b,
0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41d, 0x43e, 0x440, 0x444,
0x43e, 0x43b, 0x43a, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434,
0x443, 0x2d, 0x434, 0x438, 0x2d, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x424, 0x435, 0x440, 0x43d, 0x430,
0x43d, 0x434, 0x443, 0x2d, 0x434, 0x438, 0x2d, 0x41d, 0x43e, 0x440, 0x43e, 0x43d,
0x44c, 0x44f, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x44b, 0x435,
0x20, 0x41c, 0x430, 0x440, 0x438, 0x430, 0x43d, 0x441, 0x43a, 0x438, 0x435, 0x20,
0x43e, 0x2d, 0x432, 0x430, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438,
0x440, 0x441, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41d, 0x43e,
0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x2c, 0x20, 0x43b,
0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41e,
0x43c, 0x441, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41e, 0x43c,
0x441, 0x43a, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41f, 0x430, 0x43a, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41f, 0x430, 0x43f, 0x443, 0x430, 0x20, 0x2013,
0x20, 0x41d, 0x43e, 0x432, 0x430, 0x44f, 0x20, 0x413, 0x432, 0x438, 0x43d, 0x435,
0x44f, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439,
0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x41f, 0x435, 0x440, 0x443, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x41f, 0x435, 0x440, 0x443, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x424, 0x438, 0x43b, 0x438, 0x43f,
0x43f, 0x438, 0x43d, 0x44b, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d,
0x44b, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x424, 0x438, 0x43b, 0x438,
0x43f, 0x43f, 0x438, 0x43d, 0x44b, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x43e, 0x2d, 0x432, 0x430, 0x20,
0x424, 0x435, 0x43d, 0x438, 0x43a, 0x441, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x44c,
0x435, 0x440, 0x20, 0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d,
0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20, 0x438, 0x20, 0x41c,
0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20, 0x438, 0x20,
0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41a, 0x44b, 0x437,
0x44b, 0x43b, 0x43e, 0x440, 0x434, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x2a, 0x41a, 0x44b, 0x437, 0x44b, 0x43b, 0x43e, 0x440, 0x434, 0x430, 0x2c,
0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x2a, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x2c,
0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x412, 0x440, 0x435, 0x43c, 0x44f, 0x20, 0x432, 0x20, 0x421, 0x430, 0x43c,
0x430, 0x440, 0x435, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x441, 0x43a, 0x43e, 0x435,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x441,
0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x2c, 0x20, 0x43b,
0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x421,
0x435, 0x439, 0x448, 0x435, 0x43b, 0x44c, 0x441, 0x43a, 0x438, 0x435, 0x20, 0x41e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x421, 0x43e, 0x43b, 0x43e, 0x43c, 0x43e,
0x43d, 0x43e, 0x432, 0x44b, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430,
0x42e, 0x436, 0x43d, 0x430, 0x44f, 0x20, 0x413, 0x435, 0x43e, 0x440, 0x433, 0x438,
0x44f, 0x422, 0x430, 0x439, 0x432, 0x430, 0x43d, 0x44c, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x422, 0x430, 0x439, 0x432, 0x430, 0x43d, 0x44c, 0x2c, 0x20,
0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x422, 0x43e, 0x43d, 0x433, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x422, 0x43e, 0x43d, 0x433, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x422, 0x443, 0x440, 0x43a, 0x43c,
0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442,
0x430, 0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430,
0x439, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430,
0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x423, 0x437, 0x431, 0x435,
0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d,
0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x412, 0x430, 0x43d, 0x443,
0x430, 0x442, 0x443, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x412, 0x430,
0x43d, 0x443, 0x430, 0x442, 0x443, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x412, 0x43b, 0x430, 0x434, 0x438,
0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a,
0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433,
0x440, 0x430, 0x434, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x423, 0x43e, 0x43b, 0x43b, 0x438, 0x441, 0x20,
0x438, 0x20, 0x424, 0x443, 0x442, 0x443, 0x43d, 0x430, 0x42f, 0x43a, 0x443, 0x442,
0x441, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x42f, 0x43a, 0x443,
0x442, 0x441, 0x43a, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438,
0x43d, 0x431, 0x443, 0x440, 0x433, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433,
0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x410, 0x440, 0x430, 0x430, 0x431, 0x20, 0x43a, 0x44d, 0x43c, 0x44d,
0x410, 0x440, 0x430, 0x430, 0x431, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20,
0x43a, 0x44d, 0x43c, 0x44d, 0x410, 0x440, 0x430, 0x430, 0x431, 0x20, 0x441, 0x430,
0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x42d, 0x440,
0x43c, 0x44d, 0x44d, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x42d, 0x440, 0x43c,
0x44d, 0x44d, 0x43d, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d,
0x43c, 0x44d, 0x42d, 0x440, 0x43c, 0x44d, 0x44d, 0x43d, 0x20, 0x441, 0x430, 0x439,
0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41a, 0x438, 0x438,
0x43d, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20,
0x43a, 0x44d, 0x43c, 0x44d, 0x41a, 0x438, 0x438, 0x43d, 0x20, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d,
0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41a, 0x438, 0x438, 0x43d, 0x20, 0x410, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x441, 0x430, 0x439, 0x44b,
0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x418, 0x43b, 0x438, 0x43d,
0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x43a,
0x44d, 0x43c, 0x44d, 0x418, 0x43b, 0x438, 0x43d, 0x20, 0x410, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20,
0x43a, 0x44d, 0x43c, 0x44d, 0x418, 0x43b, 0x438, 0x43d, 0x20, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x441, 0x430, 0x439, 0x44b, 0x4a5,
0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x410, 0x440, 0x495, 0x430, 0x430,
0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x43a,
0x44d, 0x43c, 0x44d, 0x410, 0x440, 0x495, 0x430, 0x430, 0x20, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d,
0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x410, 0x440, 0x495, 0x430, 0x430, 0x20, 0x410,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x441, 0x430, 0x439,
0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41a, 0x44b, 0x442,
0x430, 0x439, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41a, 0x44b, 0x442, 0x430, 0x439,
0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41a,
0x44b, 0x442, 0x430, 0x439, 0x20, 0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b,
0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41a, 0x443, 0x440, 0x443, 0x441, 0x443, 0x443,
0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41a, 0x443, 0x440, 0x443, 0x441, 0x443,
0x443, 0x43d, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c,
0x44d, 0x41a, 0x443, 0x440, 0x443, 0x441, 0x443, 0x443, 0x43d, 0x20, 0x441, 0x430,
0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x418, 0x438,
0x43d, 0x434, 0x438, 0x439, 0x44d, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20,
0x43a, 0x44d, 0x43c, 0x44d, 0x418, 0x440, 0x430, 0x430, 0x43d, 0x20, 0x43a, 0x44d,
0x43c, 0x44d, 0x418, 0x440, 0x430, 0x430, 0x43d, 0x20, 0x441, 0x4af, 0x440, 0x4af,
0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x42b, 0x440, 0x430, 0x430, 0x43d, 0x20,
0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d,
0x414, 0x44c, 0x43e, 0x43f, 0x43f, 0x443, 0x43e, 0x43d, 0x20, 0x43a, 0x44d, 0x43c,
0x44d, 0x414, 0x44c, 0x43e, 0x43f, 0x43f, 0x443, 0x43e, 0x43d, 0x20, 0x441, 0x4af,
0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x414, 0x44c, 0x43e, 0x43f,
0x43f, 0x443, 0x43e, 0x43d, 0x20, 0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b,
0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x418, 0x43b, 0x438, 0x43d, 0x20, 0x41a, 0x430,
0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x430, 0x43d, 0x20, 0x43a, 0x44d, 0x43c,
0x44d, 0x410, 0x440, 0x495, 0x430, 0x430, 0x20, 0x41a, 0x430, 0x437, 0x430, 0x445,
0x441, 0x442, 0x430, 0x430, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41a, 0x44d,
0x440, 0x438, 0x44d, 0x439, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41a, 0x44d, 0x440,
0x438, 0x44d, 0x439, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d,
0x43c, 0x44d, 0x41a, 0x44d, 0x440, 0x438, 0x44d, 0x439, 0x20, 0x441, 0x430, 0x439,
0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41a, 0x440, 0x430,
0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x430, 0x439, 0x20, 0x43a, 0x44d,
0x43c, 0x44d, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a,
0x430, 0x439, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c,
0x44d, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x430,
0x439, 0x20, 0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d,
0x43c, 0x44d, 0x41a, 0x44b, 0x440, 0x433, 0x44b, 0x441, 0x442, 0x430, 0x430, 0x43d,
0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x430,
0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430,
0x430, 0x43d, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c,
0x44d, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x430, 0x43d, 0x20, 0x441, 0x430,
0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x423, 0x43b,
0x430, 0x43d, 0x20, 0x411, 0x430, 0x430, 0x442, 0x430, 0x440, 0x20, 0x43a, 0x44d,
0x43c, 0x44d, 0x423, 0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430, 0x430, 0x442, 0x430,
0x440, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d,
0x423, 0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430, 0x430, 0x442, 0x430, 0x440, 0x20,
0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d,
0x41c, 0x43e, 0x441, 0x43a, 0x443, 0x431, 0x430, 0x20, 0x43a, 0x44d, 0x43c, 0x44d,
0x41c, 0x43e, 0x441, 0x43a, 0x443, 0x431, 0x430, 0x20, 0x441, 0x4af, 0x440, 0x4af,
0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41c, 0x43e, 0x441, 0x43a, 0x443, 0x431,
0x430, 0x20, 0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d,
0x43c, 0x44d, 0x421, 0x430, 0x4a5, 0x430, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d,
0x434, 0x438, 0x44f, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x421, 0x430, 0x4a5, 0x430,
0x20, 0x421, 0x44d, 0x439, 0x43b, 0x44d, 0x43d, 0x434, 0x20, 0x441, 0x4af, 0x440,
0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x421, 0x430, 0x4a5, 0x430, 0x20,
0x421, 0x44d, 0x439, 0x43b, 0x44d, 0x43d, 0x434, 0x20, 0x441, 0x430, 0x439, 0x44b,
0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41d, 0x43e, 0x432, 0x43e,
0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x430, 0x439, 0x20, 0x43a, 0x44d,
0x43c, 0x44d, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441,
0x43a, 0x430, 0x439, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d,
0x43c, 0x44d, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441,
0x43a, 0x430, 0x439, 0x20, 0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20,
0x43a, 0x44d, 0x43c, 0x44d, 0x41e, 0x43c, 0x441, 0x43a, 0x430, 0x439, 0x20, 0x43a,
0x44d, 0x43c, 0x44d, 0x41e, 0x43c, 0x441, 0x43a, 0x430, 0x439, 0x20, 0x441, 0x4af,
0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41e, 0x43c, 0x441, 0x43a,
0x430, 0x439, 0x20, 0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a,
0x44d, 0x43c, 0x44d, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x430, 0x43d,
0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430,
0x430, 0x43d, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c,
0x44d, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x430, 0x43d, 0x20, 0x441,
0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x421,
0x430, 0x445, 0x430, 0x43b, 0x438, 0x438, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d,
0x421, 0x430, 0x445, 0x430, 0x43b, 0x44b, 0x44b, 0x43d, 0x20, 0x441, 0x4af, 0x440,
0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x421, 0x430, 0x445, 0x430, 0x43b,
0x44b, 0x44b, 0x43d, 0x20, 0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20,
0x43a, 0x44d, 0x43c, 0x44d, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441,
0x442, 0x43e, 0x43a, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x411, 0x44b, 0x43b, 0x430,
0x434, 0x44c, 0x44b, 0x431, 0x430, 0x441, 0x442, 0x44b, 0x430, 0x43a, 0x20, 0x441,
0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x411, 0x44b, 0x43b,
0x430, 0x434, 0x44c, 0x44b, 0x431, 0x430, 0x441, 0x442, 0x44b, 0x430, 0x43a, 0x20,
0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d,
0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x20, 0x43a, 0x44d,
0x43c, 0x44d, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x20,
0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x412, 0x43e,
0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x20, 0x441, 0x430, 0x439, 0x44b,
0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x414, 0x44c, 0x43e, 0x43a,
0x443, 0x443, 0x441, 0x43a, 0x430, 0x439, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x414,
0x44c, 0x43e, 0x43a, 0x443, 0x443, 0x441, 0x43a, 0x430, 0x439, 0x20, 0x441, 0x4af,
0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x414, 0x44c, 0x43e, 0x43a,
0x443, 0x443, 0x441, 0x43a, 0x430, 0x439, 0x20, 0x441, 0x430, 0x439, 0x44b, 0x4a5,
0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x415, 0x43a, 0x430, 0x442, 0x435,
0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x20, 0x43a, 0x44d, 0x43c, 0x44d,
0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x443, 0x440,
0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x415,
0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x443, 0x440, 0x20,
0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d,
0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915,
0x93e, 0x3a, 0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x938, 0x92e, 0x92f, 0x903,
0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915,
0x93e, 0x3a, 0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x906, 0x926, 0x930, 0x94d,
0x936, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20,
0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x3a, 0x20, 0x92e, 0x927, 0x94d,
0x92f, 0x20, 0x905, 0x92f, 0x93e, 0x92e, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x909,
0x924, 0x94d, 0x924, 0x930, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e,
0x3a, 0x20, 0x92a, 0x94c, 0x930, 0x94d, 0x935, 0x20, 0x938, 0x92e, 0x92f, 0x903,
0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915,
0x93e, 0x3a, 0x20, 0x92a, 0x94c, 0x930, 0x94d, 0x935, 0x20, 0x906, 0x926, 0x930,
0x94d, 0x936, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x909, 0x924, 0x94d, 0x924, 0x930,
0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x3a, 0x20, 0x92a, 0x94c,
0x930, 0x94d, 0x935, 0x20, 0x905, 0x92f, 0x93e, 0x92e, 0x20, 0x938, 0x92e, 0x92f,
0x3a, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f,
0x915, 0x93e, 0x3a, 0x20, 0x936, 0x948, 0x932, 0x20, 0x938, 0x92e, 0x92f, 0x903,
0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915,
0x93e, 0x3a, 0x20, 0x936, 0x948, 0x932, 0x20, 0x906, 0x926, 0x930, 0x94d, 0x936,
0x20, 0x938, 0x92e, 0x92f, 0x903, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x905,
0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x3a, 0x20, 0x936, 0x948, 0x932, 0x20,
0x905, 0x92f, 0x93e, 0x92e, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x909, 0x924, 0x94d,
0x924, 0x930, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x3a, 0x20,
0x938, 0x928, 0x94d, 0x927, 0x93f, 0x92a, 0x94d, 0x930, 0x93f, 0x92f, 0x20, 0x938,
0x92e, 0x92f, 0x903, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x905, 0x92e, 0x947,
0x930, 0x93f, 0x915, 0x93e, 0x3a, 0x20, 0x938, 0x928, 0x94d, 0x927, 0x93f, 0x92a,
0x94d, 0x930, 0x93f, 0x92f, 0x20, 0x906, 0x926, 0x930, 0x94d, 0x936, 0x20, 0x938,
0x92e, 0x92f, 0x903, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x905, 0x92e, 0x947,
0x930, 0x93f, 0x915, 0x93e, 0x3a, 0x20, 0x938, 0x928, 0x94d, 0x927, 0x93f, 0x92a,
0x94d, 0x930, 0x93f, 0x92f, 0x20, 0x905, 0x92f, 0x93e, 0x92e, 0x20, 0x938, 0x92e,
0x92f, 0x903, 0x905, 0x91f, 0x932, 0x93e, 0x923, 0x94d, 0x91f, 0x93f, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x903, 0x905, 0x91f, 0x932, 0x93e, 0x923, 0x94d, 0x91f, 0x93f,
0x915, 0x20, 0x906, 0x926, 0x930, 0x94d, 0x936, 0x20, 0x938, 0x92e, 0x92f, 0x903,
0x905, 0x91f, 0x932, 0x93e, 0x923, 0x94d, 0x91f, 0x93f, 0x915, 0x20, 0x905, 0x92f,
0x93e, 0x92e, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x92e, 0x927, 0x94d, 0x92f, 0x20,
0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x92f, 0x20, 0x938, 0x92e, 0x92f, 0x903,
0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x92f,
0x20, 0x906, 0x926, 0x930, 0x94d, 0x936, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x92e,
0x927, 0x94d, 0x92f, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x92f, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x938, 0x92e, 0x92f, 0x903,
0x92a, 0x94c, 0x930, 0x94d, 0x935, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940,
0x92f, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x92a, 0x94c, 0x930, 0x94d, 0x935, 0x20,
0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x92f, 0x20, 0x906, 0x926, 0x930, 0x94d,
0x936, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x92a, 0x94c, 0x930, 0x94d, 0x935, 0x20,
0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x92f, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x937, 0x94d, 0x92e, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x92a, 0x93e, 0x936, 0x94d,
0x91a, 0x93e, 0x924, 0x94d, 0x92f, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940,
0x92f, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x92a, 0x93e, 0x936, 0x94d, 0x91a, 0x93e,
0x924, 0x94d, 0x92f, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x92f, 0x20,
0x906, 0x926, 0x930, 0x94d, 0x936, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x92a, 0x93e,
0x936, 0x94d, 0x91a, 0x93e, 0x924, 0x94d, 0x92f, 0x20, 0x92f, 0x942, 0x930, 0x94b,
0x92a, 0x940, 0x92f, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20,
0x938, 0x92e, 0x92f, 0x903, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x935, 0x93f, 0x91a,
0x20, 0x92e, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x1c5f, 0x1c79, 0x1c5e,
0x1c5f, 0x1c65, 0x1c60, 0x1c5f, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c5f, 0x1c79, 0x1c5e,
0x1c5f, 0x1c65, 0x1c60, 0x1c5f, 0x20, 0x1c6e, 0x1c65, 0x1c74, 0x1c6e, 0x1c71, 0x1c70, 0x1c5f,
0x1c68, 0x1c70, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c5f, 0x1c79, 0x1c5e, 0x1c5f, 0x1c65,
0x1c60, 0x1c5f, 0x20, 0x1c65, 0x1c64, 0x1c67, 0x1c5f, 0x1c5c, 0x20, 0x1c5a, 0x1c60, 0x1c5b,
0x1c5a, 0x1c5f, 0x1c62, 0x1c5f, 0x1c61, 0x1c5a, 0x1c71, 0x20, 0x1c5a, 0x1c60, 0x1c74, 0x1c5a,
0x1c5f, 0x1c62, 0x1c5f, 0x1c61, 0x1c5a, 0x1c71, 0x20, 0x1c6e, 0x1c74, 0x1c6e, 0x1c71, 0x1c70,
0x1c5f, 0x1c68, 0x1c70, 0x20, 0x1c5a, 0x1c60, 0x1c74, 0x1c5a, 0x1c5f, 0x1c62, 0x1c5f, 0x1c61,
0x1c5a, 0x1c71, 0x20, 0x1c65, 0x1c64, 0x1c5b, 0x1c69, 0x1c5d, 0x20, 0x1c5a, 0x1c60, 0x1c74,
0x1c5a, 0x1c5b, 0x1c5f, 0x1c71, 0x1c5f, 0x1c5e, 0x1c5f, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a,
0x1c5b, 0x1c5f, 0x1c71, 0x1c5f, 0x1c5e, 0x1c5f, 0x20, 0x1c6e, 0x1c65, 0x1c74, 0x1c6e, 0x1c71,
0x1c70, 0x1c5f, 0x1c68, 0x1c70, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c5b, 0x1c5f, 0x1c71,
0x1c5f, 0x1c5e, 0x1c5f, 0x20, 0x1c65, 0x1c64, 0x1c67, 0x1c5f, 0x1c5c, 0x20, 0x1c5a, 0x1c60,
0x1c5b, 0x1c5a, 0x1c64, 0x1c65, 0x1c74, 0x1c5f, 0x1c68, 0x1c71, 0x20, 0x1c5a, 0x1c60, 0x1c5b,
0x1c5a, 0x1c64, 0x1c65, 0x1c74, 0x1c5f, 0x1c68, 0x1c71, 0x20, 0x1c6e, 0x1c65, 0x1c74, 0x1c6e,
0x1c71, 0x1c70, 0x1c5f, 0x1c68, 0x1c70, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c64, 0x1c65,
0x1c74, 0x1c5f, 0x1c68, 0x1c71, 0x20, 0x1c65, 0x1c64, 0x1c67, 0x1c5f, 0x1c5c, 0x20, 0x1c75,
0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c62, 0x1c5f, 0x1c69, 0x1c71, 0x1c74, 0x1c6e, 0x1c71, 0x20,
0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c62, 0x1c5f, 0x1c69, 0x1c71, 0x1c74, 0x1c6e, 0x1c71, 0x20,
0x1c6e, 0x1c65, 0x1c74, 0x1c6e, 0x1c71, 0x1c70, 0x1c5f, 0x1c68, 0x1c70, 0x20, 0x1c5a, 0x1c60,
0x1c5b, 0x1c5a, 0x1c62, 0x1c5f, 0x1c69, 0x1c71, 0x1c74, 0x1c6e, 0x1c71, 0x20, 0x1c65, 0x1c64,
0x1c67, 0x1c5f, 0x1c5c, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c6f, 0x1c6e, 0x1c65, 0x1c64,
0x1c6f, 0x1c77, 0x1c64, 0x1c60, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c6f, 0x1c6e, 0x1c65,
0x1c64, 0x1c6f, 0x1c77, 0x1c64, 0x1c60, 0x20, 0x1c6e, 0x1c65, 0x1c74, 0x1c6e, 0x1c71, 0x1c70,
0x1c5f, 0x1c68, 0x1c70, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c6f, 0x1c6e, 0x1c65, 0x1c64,
0x1c6f, 0x1c77, 0x1c64, 0x1c60, 0x20, 0x1c65, 0x1c64, 0x1c67, 0x1c5f, 0x1c5c, 0x20, 0x1c5a,
0x1c60, 0x1c5b, 0x1c5a, 0x1c5f, 0x1c79, 0x1c68, 0x1c61, 0x1c6e, 0x1c71, 0x1c74, 0x1c64, 0x1c71,
0x1c5f, 0x20, 0x1c5a, 0x1c60, 0x1c74, 0x1c5a, 0x1c5f, 0x1c79, 0x1c68, 0x1c61, 0x1c6e, 0x1c71,
0x1c74, 0x1c64, 0x1c71, 0x1c5f, 0x20, 0x1c6e, 0x1c74, 0x1c6e, 0x1c71, 0x1c70, 0x1c5f, 0x1c68,
0x1c70, 0x20, 0x1c5a, 0x1c60, 0x1c74, 0x1c5a, 0x1c5f, 0x1c79, 0x1c68, 0x1c61, 0x1c6e, 0x1c71,
0x1c74, 0x1c64, 0x1c71, 0x1c5f, 0x20, 0x1c65, 0x1c64, 0x1c5b, 0x1c69, 0x1c5d, 0x20, 0x1c5a,
0x1c60, 0x1c74, 0x1c5a, 0x1c6f, 0x1c5f, 0x1c6a, 0x1c6e, 0x20, 0x1c5f, 0x1c79, 0x1c68, 0x1c61,
0x1c6e, 0x1c71, 0x1c74, 0x1c64, 0x1c71, 0x1c5f, 0x20, 0x1c5a, 0x1c60, 0x1c74, 0x1c5a, 0x1c6f,
0x1c5f, 0x1c6a, 0x1c6e, 0x20, 0x1c5f, 0x1c79, 0x1c68, 0x1c61, 0x1c6e, 0x1c71, 0x1c74, 0x1c64,
0x1c71, 0x1c5f, 0x20, 0x1c6e, 0x1c65, 0x1c74, 0x1c6e, 0x1c71, 0x1c70, 0x1c5f, 0x1c68, 0x1c70,
0x20, 0x1c5a, 0x1c60, 0x1c74, 0x1c5a, 0x1c6f, 0x1c5f, 0x1c6a, 0x1c6e, 0x20, 0x1c5f, 0x1c79,
0x1c68, 0x1c61, 0x1c6e, 0x1c71, 0x1c74, 0x1c64, 0x1c71, 0x1c5f, 0x20, 0x1c65, 0x1c64, 0x1c5b,
0x1c69, 0x1c5d, 0x20, 0x1c5a, 0x1c60, 0x1c74, 0x1c5a, 0x1c6e, 0x1c74, 0x1c5e, 0x1c5f, 0x1c71,
0x1c74, 0x1c64, 0x1c60, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c6e, 0x1c74, 0x1c5e, 0x1c5f,
0x1c71, 0x1c74, 0x1c64, 0x1c60, 0x20, 0x1c6e, 0x1c65, 0x1c74, 0x1c6e, 0x1c71, 0x1c70, 0x1c5f,
0x1c68, 0x1c70, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c6e, 0x1c74, 0x1c5e, 0x1c5f, 0x1c71,
0x1c74, 0x1c64, 0x1c60, 0x20, 0x1c65, 0x1c64, 0x1c67, 0x1c5f, 0x1c5c, 0x20, 0x1c5a, 0x1c60,
0x1c5b, 0x1c5a, 0x1c60, 0x1c6d, 0x1c69, 0x1c75, 0x1c5f, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a,
0x1c60, 0x1c6d, 0x1c69, 0x1c75, 0x1c5f, 0x20, 0x1c62, 0x1c5f, 0x1c71, 0x1c5a, 0x1c60, 0x20,
0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c60, 0x1c6d, 0x1c69, 0x1c75, 0x1c5f, 0x20, 0x1c65, 0x1c64,
0x1c67, 0x1c5f, 0x1c5c, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c65, 0x1c6e, 0x1c71, 0x1c74,
0x1c68, 0x1c5f, 0x1c5e, 0x20, 0x1c69, 0x1c68, 0x1c5a, 0x1c6f, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71,
0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c65, 0x1c6e, 0x1c71, 0x1c74, 0x1c68, 0x1c5f, 0x1c5e,
0x20, 0x1c69, 0x1c68, 0x1c5a, 0x1c6f, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x20, 0x1c6e, 0x1c65,
0x1c74, 0x1c6e, 0x1c71, 0x1c70, 0x1c5f, 0x1c68, 0x1c70, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a,
0x1c65, 0x1c6e, 0x1c71, 0x1c74, 0x1c68, 0x1c5f, 0x1c5e, 0x20, 0x1c69, 0x1c68, 0x1c5a, 0x1c6f,
0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x20, 0x1c65, 0x1c5f, 0x1c62, 0x1c5f, 0x1c68, 0x20, 0x1c5a,
0x1c60, 0x1c5b, 0x1c5a, 0x1c64, 0x1c65, 0x1c74, 0x1c5f, 0x1c68, 0x1c71, 0x20, 0x1c69, 0x1c68,
0x1c5a, 0x1c6f, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c64,
0x1c65, 0x1c74, 0x1c5f, 0x1c68, 0x1c71, 0x20, 0x1c69, 0x1c68, 0x1c5a, 0x1c6f, 0x1c64, 0x1c6d,
0x1c5f, 0x1c71, 0x20, 0x1c6e, 0x1c65, 0x1c74, 0x1c6e, 0x1c71, 0x1c70, 0x1c5f, 0x1c68, 0x1c70,
0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c64, 0x1c65, 0x1c74, 0x1c5f, 0x1c68, 0x1c71, 0x20,
0x1c69, 0x1c68, 0x1c5a, 0x1c6f, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x20, 0x1c65, 0x1c5f, 0x1c62,
0x1c5f, 0x1c68, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c63, 0x1c6e, 0x1c65, 0x1c74, 0x1c5f,
0x1c68, 0x1c71, 0x20, 0x1c69, 0x1c68, 0x1c5a, 0x1c6f, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x20,
0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c63, 0x1c6e, 0x1c65, 0x1c74, 0x1c5f, 0x1c68, 0x1c71, 0x20,
0x1c69, 0x1c68, 0x1c5a, 0x1c6f, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x20, 0x1c6e, 0x1c65, 0x1c74,
0x1c6e, 0x1c71, 0x1c70, 0x1c5f, 0x1c68, 0x1c70, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c63,
0x1c6e, 0x1c65, 0x1c74, 0x1c5f, 0x1c68, 0x1c71, 0x20, 0x1c69, 0x1c68, 0x1c5a, 0x1c6f, 0x1c64,
0x1c6d, 0x1c5f, 0x1c71, 0x20, 0x1c65, 0x1c5f, 0x1c62, 0x1c5f, 0x1c68, 0x20, 0x1c5a, 0x1c60,
0x1c5b, 0x1c5a, 0x1c5c, 0x1c68, 0x1c64, 0x1c71, 0x1c63, 0x1c64, 0x1c6a, 0x20, 0x1c62, 0x1c64,
0x1c71, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c65, 0x1c5f, 0x1c62, 0x1c5f, 0x1c5d, 0x20,
0x1c5c, 0x1c7d, 0x1c68, 0x1c64, 0x1c71, 0x1c5e, 0x1c6e, 0x1c71, 0x1c70, 0x20, 0x1c5a, 0x1c60,
0x1c5b, 0x1c5a, 0x1c65, 0x1c5f, 0x1c62, 0x1c5f, 0x1c5d, 0x20, 0x1c5c, 0x1c7d, 0x1c68, 0x1c64,
0x1c71, 0x1c5e, 0x1c6e, 0x1c71, 0x1c70, 0x20, 0x1c62, 0x1c5f, 0x1c71, 0x1c5a, 0x1c60, 0x20,
0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c65, 0x1c5f, 0x1c62, 0x1c5f, 0x1c5d, 0x20, 0x1c5c, 0x1c7d,
0x1c68, 0x1c64, 0x1c71, 0x1c5e, 0x1c6e, 0x1c71, 0x1c70, 0x20, 0x1c65, 0x1c64, 0x1c5b, 0x1c69,
0x1c5d, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c6f, 0x1c5f, 0x1c6a, 0x1c6e, 0x20, 0x1c5c,
0x1c7d, 0x1c68, 0x1c64, 0x1c71, 0x1c5e, 0x1c6e, 0x1c71, 0x1c70, 0x20, 0x1c5a, 0x1c60, 0x1c5b,
0x1c5a, 0x1c6f, 0x1c5f, 0x1c6a, 0x1c6e, 0x20, 0x1c5c, 0x1c7d, 0x1c68, 0x1c64, 0x1c71, 0x1c5e,
0x1c6e, 0x1c71, 0x1c70, 0x20, 0x1c62, 0x1c5f, 0x1c71, 0x1c5a, 0x1c60, 0x20, 0x1c5a, 0x1c60,
0x1c5b, 0x1c5a, 0x1c6f, 0x1c5f, 0x1c6a, 0x1c6e, 0x20, 0x1c5c, 0x1c7d, 0x1c68, 0x1c64, 0x1c71,
0x1c5e, 0x1c6e, 0x1c71, 0x1c70, 0x20, 0x1c65, 0x1c64, 0x1c5b, 0x1c69, 0x1c5d, 0x20, 0x1c5a,
0x1c60, 0x1c5b, 0x1c5a, 0x1c66, 0x1c5f, 0x1c63, 0x1c5f, 0x1c6d, 0x1c64, 0x1c7c, 0x1c5f, 0x1c5e,
0x1c6e, 0x1c63, 0x1c74, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a,
0x1c66, 0x1c5f, 0x1c63, 0x1c5f, 0x1c6d, 0x1c64, 0x1c7c, 0x1c5f, 0x1c5e, 0x1c6e, 0x1c63, 0x1c74,
0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x20, 0x1c62, 0x1c5f, 0x1c71, 0x1c5a, 0x1c60, 0x20, 0x1c5a,
0x1c60, 0x1c5b, 0x1c5a, 0x1c66, 0x1c5f, 0x1c63, 0x1c5f, 0x1c6d, 0x1c64, 0x1c7c, 0x1c5f, 0x1c5e,
0x1c6e, 0x1c63, 0x1c74, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x20, 0x1c65, 0x1c64, 0x1c67, 0x1c5f,
0x1c5c, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c62, 0x1c6e, 0x1c60, 0x1c65, 0x1c64, 0x1c60,
0x1c5f, 0x1c71, 0x20, 0x1c6f, 0x1c68, 0x1c5a, 0x1c65, 0x1c5f, 0x1c71, 0x1c5b, 0x20, 0x1c5a,
0x1c60, 0x1c5b, 0x1c5a, 0x1c62, 0x1c6e, 0x1c60, 0x1c65, 0x1c64, 0x1c60, 0x1c5f, 0x1c71, 0x20,
0x1c6f, 0x1c68, 0x1c5a, 0x1c65, 0x1c5f, 0x1c71, 0x1c5b, 0x20, 0x1c62, 0x1c5f, 0x1c71, 0x1c5a,
0x1c60, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c62, 0x1c6e, 0x1c60, 0x1c65, 0x1c64, 0x1c60,
0x1c5f, 0x1c71, 0x20, 0x1c6f, 0x1c68, 0x1c5a, 0x1c65, 0x1c5f, 0x1c71, 0x1c5b, 0x20, 0x1c65,
0x1c64, 0x1c67, 0x1c5f, 0x1c5c, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c71, 0x1c64, 0x1c6d,
0x1c69, 0x1c6f, 0x1c77, 0x1c5f, 0x1c69, 0x1c71, 0x1c70, 0x1c5e, 0x1c6e, 0x1c78, 0x1c70, 0x20,
0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c71, 0x1c64, 0x1c6d, 0x1c69, 0x1c6f, 0x1c77, 0x1c5f, 0x1c69,
0x1c71, 0x1c70, 0x1c5e, 0x1c6e, 0x1c78, 0x1c70, 0x20, 0x1c62, 0x1c5f, 0x1c71, 0x1c5a, 0x1c60,
0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c71, 0x1c64, 0x1c6d, 0x1c69, 0x1c6f, 0x1c77, 0x1c5f,
0x1c69, 0x1c71, 0x1c70, 0x1c5e, 0x1c6e, 0x1c78, 0x1c70, 0x20, 0x1c65, 0x1c64, 0x1c67, 0x1c5f,
0x1c5c, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c6e, 0x1c65, 0x20, 0x1c74, 0x1c64, 0x20,
0x1c6f, 0x1c64, 0x1c6d, 0x1c6e, 0x1c68, 0x1c6e, 0x20, 0x1c5f, 0x1c68, 0x20, 0x1c62, 0x1c64,
0x1c60, 0x1c6e, 0x1c5e, 0x1c5a, 0x1c71, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c6e, 0x1c65,
0x20, 0x1c74, 0x1c64, 0x20, 0x1c6f, 0x1c64, 0x1c6d, 0x1c6e, 0x1c68, 0x1c6e, 0x20, 0x1c5f,
0x1c68, 0x20, 0x1c62, 0x1c64, 0x1c60, 0x1c6e, 0x1c5e, 0x1c5a, 0x1c71, 0x20, 0x1c62, 0x1c5f,
0x1c71, 0x1c5a, 0x1c60, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c6e, 0x1c65, 0x20, 0x1c74,
0x1c64, 0x20, 0x1c6f, 0x1c64, 0x1c6d, 0x1c6e, 0x1c68, 0x1c6e, 0x20, 0x1c5f, 0x1c68, 0x20,
0x1c62, 0x1c64, 0x1c60, 0x1c6e, 0x1c5e, 0x1c5a, 0x1c71, 0x20, 0x1c65, 0x1c64, 0x1c67, 0x1c5f,
0x1c5c, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c6d, 0x1c69, 0x1c60, 0x1c5a, 0x1c71, 0x20,
0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41,
0x63, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x63, 0x72, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x41, 0x63, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x2019, 0x41, 0x66, 0x67, 0x68, 0xe0, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0xc0,
0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x74, 0x7a, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019,
0xc0, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0xc0,
0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6d, 0x65, 0x72, 0x69, 0x64, 0x69,
0x6f, 0x6e, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x2019, 0xc0, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x74, 0x7a,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x73, 0x2019, 0xc0, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x74,
0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73,
0x2019, 0xc0, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x74, 0x7a, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x73, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x4f, 0x72,
0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41,
0x6c, 0x6d, 0x61, 0x74, 0x79, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x2019, 0x41, 0x6d, 0x61, 0x74, 0x7a, 0xf2, 0x6e, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x6d, 0x61, 0x74, 0x7a, 0xf2,
0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x6d, 0x61, 0x74, 0x7a,
0xf2, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x74, 0x7a, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x65, 0x20, 0x55, 0x53, 0x41, 0x4f, 0x72, 0x61,
0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x74,
0x7a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x20, 0x55, 0x53, 0x41,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x74,
0x7a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x20, 0x55, 0x53, 0x41,
0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x20,
0x55, 0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x65,
0x73, 0x20, 0x50, 0x65, 0x64, 0x72, 0x6f, 0x73, 0x6f, 0x73, 0x20, 0x55,
0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x65, 0x73, 0x20, 0x50,
0x65, 0x64, 0x72, 0x6f, 0x73, 0x6f, 0x73, 0x20, 0x55, 0x53, 0x41, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x4d, 0x6f,
0x6e, 0x74, 0x65, 0x73, 0x20, 0x50, 0x65, 0x64, 0x72, 0x6f, 0x73, 0x6f,
0x73, 0x20, 0x55, 0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x75, 0x20, 0x50, 0x61, 0x74, 0x7a, 0xec, 0x66, 0x69, 0x63, 0x75,
0x20, 0x55, 0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20,
0x50, 0x61, 0x74, 0x7a, 0xec, 0x66, 0x69, 0x63, 0x75, 0x20, 0x55, 0x53,
0x41, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x50, 0x61, 0x74, 0x7a, 0xec, 0x66,
0x69, 0x63, 0x75, 0x20, 0x55, 0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x4f, 0x72, 0x61, 0x20,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x6c,
0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6e, 0x61,
0x64, 0x79, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70,
0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x41, 0x70, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x41, 0x6b, 0x74, 0x61, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6b,
0x74, 0x61, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x41, 0x6b, 0x74, 0x6f, 0x62, 0x65, 0x4f, 0x72, 0x61, 0x20, 0xe0, 0x72,
0x61, 0x62, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0xe0, 0x72, 0x61, 0x62, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0xe0, 0x72, 0x61,
0x62, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x73, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x73, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f, 0x74, 0x7a, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x73, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20,
0x6f, 0x74, 0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x73, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x20, 0x6f, 0x74, 0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x72, 0x6d,
0xe8, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41,
0x72, 0x6d, 0xe8, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x72,
0x6d, 0xe8, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x2019, 0x41, 0x74, 0x6c, 0xe0, 0x6e, 0x74, 0x69, 0x63, 0x75, 0x4f,
0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x74, 0x6c, 0xe0, 0x6e, 0x74,
0x69, 0x63, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x74, 0x6c, 0xe0, 0x6e,
0x74, 0x69, 0x63, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x74,
0x7a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x73, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61,
0x20, 0x74, 0x7a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x73, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20,
0x74, 0x7a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0,
0x6c, 0x69, 0x61, 0x20, 0x74, 0x7a, 0x65, 0x6e, 0x74, 0x72, 0x75, 0x2d,
0x6f, 0x74, 0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0,
0x6c, 0x69, 0x61, 0x20, 0x74, 0x7a, 0x65, 0x6e, 0x74, 0x72, 0x75, 0x2d,
0x6f, 0x74, 0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x73, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61,
0x20, 0x74, 0x7a, 0x65, 0x6e, 0x74, 0x72, 0x75, 0x2d, 0x6f, 0x74, 0x7a,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c,
0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72,
0xe0, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0,
0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x75,
0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x74, 0x7a, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x73, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20,
0x6f, 0x74, 0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x73, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61,
0x20, 0x6f, 0x74, 0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x69, 0x67, 0x69, 0xe0, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x73, 0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x67, 0x69,
0xe0, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61,
0x69, 0x67, 0x69, 0xe0, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x61, 0x73, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x72, 0x61, 0x73, 0x4f,
0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x41, 0x7a, 0x6f, 0x72,
0x72, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x41, 0x7a, 0x6f,
0x72, 0x72, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x75, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x42, 0x68, 0x75, 0x74, 0xe0,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x42,
0x6f, 0x6c, 0xec, 0x76, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x42, 0x72, 0x61, 0x73, 0xec, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0xec, 0x6c, 0x69, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x42, 0x72, 0x61, 0x73, 0xec, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x43, 0x61,
0x62, 0x75, 0x20, 0x42, 0x69, 0x72, 0x64, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x73, 0x75, 0x20, 0x43, 0x61, 0x62, 0x75, 0x20, 0x42, 0x69, 0x72,
0x64, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x43, 0x61, 0x62, 0x75, 0x20,
0x42, 0x69, 0x72, 0x64, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x43, 0x61, 0x73, 0x65, 0x79, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68,
0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x61, 0x73, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x43, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x75, 0x20, 0x54, 0x7a, 0x69, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x73, 0x75, 0x20, 0x54, 0x7a, 0x69, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73,
0x75, 0x20, 0x54, 0x7a, 0x69, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x61, 0x20, 0x54, 0x7a, 0x69, 0x6e, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x54, 0x7a, 0x69, 0x6e, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x73, 0x61, 0x20, 0x54, 0x7a, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x61, 0x74, 0x69, 0x76, 0x69,
0x64, 0x61, 0x64, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x61, 0x73, 0x20, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f,
0x63, 0x6f, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61,
0x20, 0x43, 0x6f, 0x6c, 0xf2, 0x6d, 0x62, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x73, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0xf2, 0x6d, 0x62, 0x69,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0xf2, 0x6d, 0x62,
0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73,
0x20, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b,
0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0xcc, 0x73, 0x75,
0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61,
0x73, 0x20, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f,
0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x43,
0x75, 0x62, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x44, 0x61,
0x76, 0x69, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x44, 0x75,
0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20,
0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x45, 0x73, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x20,
0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x73, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20,
0x50, 0x61, 0x73, 0x63, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0xcc, 0x73, 0x75,
0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x45, 0x63, 0x75, 0x61,
0x64, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x74, 0x7a, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x74, 0x7a, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x74, 0x7a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x73, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x2019, 0x65, 0x73, 0x74, 0x72, 0x65, 0x6d, 0x75, 0x20,
0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x65, 0x20, 0x65, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x75, 0x20, 0x28, 0x4b, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x6e,
0x67, 0x72, 0x61, 0x64, 0x29, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x74, 0x7a,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x73, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x74,
0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73,
0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x74, 0x7a, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x73,
0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61,
0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x73,
0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73,
0x61, 0x73, 0x20, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x46, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x61, 0x73, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x4f, 0x72, 0x61,
0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x73, 0x61, 0x73, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x47, 0x75, 0x69, 0x61, 0x6e, 0x61,
0x20, 0x46, 0x72, 0x61, 0x6e, 0x74, 0x7a, 0x65, 0x73, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x54, 0x65, 0x72, 0x72,
0x61, 0x73, 0x20, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x73,
0x20, 0x65, 0x20, 0x61, 0x6e, 0x74, 0xe0, 0x72, 0x74, 0x69, 0x63, 0x61,
0x73, 0x20, 0x66, 0x72, 0x61, 0x6e, 0x74, 0x7a, 0x65, 0x73, 0x61, 0x73,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x47,
0x61, 0x6c, 0xe0, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x47, 0x65, 0xf2, 0x72,
0x67, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x47,
0x65, 0xf2, 0x72, 0x67, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x47,
0x65, 0xf2, 0x72, 0x67, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x61, 0x73, 0x20, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20,
0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x75, 0x20, 0x6d, 0x65, 0x72, 0x69, 0x64, 0x69, 0x61,
0x6e, 0x75, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77,
0x69, 0x63, 0x68, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61,
0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe0, 0x6e, 0x64, 0x69, 0x61,
0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c,
0xe0, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65,
0x6e, 0x6c, 0xe0, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe0, 0x6e, 0x64,
0x69, 0x61, 0x20, 0x6f, 0x74, 0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x47, 0x72,
0x6f, 0x65, 0x6e, 0x6c, 0xe0, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x74,
0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73,
0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe0, 0x6e, 0x64, 0x69,
0x61, 0x20, 0x6f, 0x74, 0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x47, 0x75, 0xe0, 0x6d, 0x4f, 0x72,
0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x47, 0x75, 0x6c, 0x66, 0x75, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x47, 0x75, 0x79,
0x61, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61,
0x73, 0x20, 0xec, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x6e, 0x61,
0x73, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0xec, 0x73,
0x75, 0x6c, 0x61, 0x73, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x6e, 0x61, 0x73, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73,
0x61, 0x73, 0x20, 0xec, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x48, 0x61,
0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x6e,
0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e,
0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x48,
0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f,
0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x48,
0x6f, 0x76, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x4f, 0x72, 0x61,
0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x73, 0x2019, 0xcc, 0x6e, 0x64, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x4f, 0x74, 0x7a, 0xe8, 0x61, 0x6e,
0x75, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x75, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x74, 0x7a,
0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019,
0x49, 0x6e, 0x64, 0x6f, 0x6e, 0xe8, 0x73, 0x69, 0x61, 0x20, 0x74, 0x7a,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0xe8, 0x73, 0x69,
0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x49, 0x6e, 0x64, 0x6f,
0x6e, 0xe8, 0x73, 0x69, 0x61, 0x20, 0x6f, 0x74, 0x7a, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x2019, 0x49, 0x72, 0xe0, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73,
0x2019, 0x49, 0x72, 0xe0, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x49, 0x72, 0xe0,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x47, 0x69, 0x61,
0x70, 0x6f, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20,
0x47, 0x69, 0x61, 0x70, 0x6f, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c,
0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20,
0x47, 0x69, 0x61, 0x70, 0x6f, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73,
0x6b, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x6c,
0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x74,
0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b,
0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x4b, 0x61, 0x7a, 0xe0, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x75, 0x20, 0x4b, 0x61, 0x7a, 0xe0, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x4b, 0x61, 0x7a,
0xe0, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6f, 0x74, 0x7a, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x63, 0x6f, 0x72,
0x65, 0x61, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b,
0x6f, 0x73, 0x72, 0x61, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x4f,
0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79,
0x61, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x75, 0x20, 0x4b, 0x69, 0x72, 0x67, 0x68, 0xec, 0x7a, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x61, 0x6e,
0x6b, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73,
0x20, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x64, 0x65, 0x20, 0x73,
0x61, 0x20, 0x4c, 0xec, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x6f,
0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x61, 0x63, 0x61, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6c,
0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x61, 0x20, 0x4d, 0x61, 0x6c, 0xe8, 0x73, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x64,
0x69, 0x76, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x61, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x63, 0x68, 0x65, 0x73, 0x61, 0x73,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0xcc,
0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61,
0x6c, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73,
0x20, 0x4d, 0x61, 0x75, 0x72, 0xec, 0x74, 0x7a, 0x69, 0x75, 0x73, 0x4f,
0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x75, 0x72,
0xec, 0x74, 0x7a, 0x69, 0x75, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20,
0x4d, 0x61, 0x75, 0x72, 0xec, 0x74, 0x7a, 0x69, 0x75, 0x73, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x50, 0x61, 0x74,
0x7a, 0xec, 0x66, 0x69, 0x63, 0x75, 0x20, 0x28, 0x4d, 0xe8, 0x73, 0x73,
0x69, 0x63, 0x75, 0x29, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20,
0x50, 0x61, 0x74, 0x7a, 0xec, 0x66, 0x69, 0x63, 0x75, 0x20, 0x28, 0x4d,
0xe8, 0x73, 0x73, 0x69, 0x63, 0x75, 0x29, 0x4f, 0x72, 0x61, 0x20, 0x6c,
0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20,
0x50, 0x61, 0x74, 0x7a, 0xec, 0x66, 0x69, 0x63, 0x75, 0x20, 0x28, 0x4d,
0xe8, 0x73, 0x73, 0x69, 0x63, 0x75, 0x29, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x55, 0x6c, 0xe0, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72,
0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0xe0, 0x6e, 0x20, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0xe0, 0x6e, 0x20, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f,
0x73, 0x63, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0xe0,
0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x61, 0x75, 0x72,
0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x4e,
0x65, 0x70, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0xf2, 0x6e, 0x69, 0x61, 0x20,
0x4e, 0x6f, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x43,
0x61, 0x6c, 0x65, 0x64, 0xf2, 0x6e, 0x69, 0x61, 0x20, 0x4e, 0x6f, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x73, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0xf2, 0x6e,
0x69, 0x61, 0x20, 0x4e, 0x6f, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20,
0x4e, 0x6f, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x5a,
0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x4e, 0x6f, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x73, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x4e,
0x6f, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72,
0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x54,
0x65, 0x72, 0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65,
0x72, 0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0xcc,
0x73, 0x75, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x73, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x75, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20,
0xcc, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x61,
0x6e, 0x6e, 0x61, 0x73, 0x20, 0x53, 0x65, 0x74, 0x65, 0x6e, 0x74, 0x72,
0x69, 0x6f, 0x6e, 0x61, 0x6c, 0x65, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d,
0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x50, 0xe0, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x50, 0xe0,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x50,
0xe0, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x61, 0x20, 0x50, 0xe0, 0x70, 0x75, 0x61, 0x20, 0x47,
0x75, 0x69, 0x6e, 0x65, 0x61, 0x20, 0x4e, 0x6f, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x79, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x50,
0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72, 0x61, 0x20, 0x6c,
0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x50, 0x65, 0x72, 0xf9, 0x4f, 0x72,
0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x50, 0x65, 0x72, 0xf9, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x73, 0x75, 0x20, 0x50, 0x65, 0x72, 0xf9, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69,
0x6e, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20,
0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73,
0x61, 0x73, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x73,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0xcc,
0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20,
0x46, 0x65, 0x6e, 0xec, 0x74, 0x7a, 0x69, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74,
0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x69, 0x6e,
0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4d,
0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69,
0x72, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x6f, 0x68,
0x6e, 0x70, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50,
0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72,
0x64, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x52,
0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x53, 0x61, 0x6d, 0x6f,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x53,
0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20,
0xcc, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x53, 0x61, 0x6c, 0x6f, 0x6d,
0x6f, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61,
0x20, 0x47, 0x65, 0xf2, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x75, 0x20, 0x53, 0x75, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x75, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x54, 0x61, 0x67, 0xec, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x75, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x54, 0x6f, 0x6e, 0x67,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x43, 0x68,
0x75, 0x75, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75,
0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0xe8, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x54, 0x75, 0x72,
0x6b, 0x6d, 0xe8, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73,
0x75, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0xe8, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20,
0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72,
0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x73, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x73, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x55, 0x7a, 0x62, 0xe8,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73,
0x2019, 0x55, 0x7a, 0x62, 0xe8, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x73, 0x2019, 0x55, 0x7a, 0x62, 0xe8, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73,
0x75, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73,
0x75, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x56, 0x65, 0x6e, 0x65, 0x74,
0x7a, 0x75, 0x65, 0x6c, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f,
0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x4f, 0x72, 0x61, 0x20,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0xcc, 0x73, 0x75,
0x6c, 0x61, 0x73, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x65, 0x20,
0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x4f, 0x72, 0x61, 0x20, 0x6c,
0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x59, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x59, 0x75, 0x6b, 0x6f,
0x6e, 0x410, 0x432, 0x433, 0x430, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b,
0x43d, 0x43e, 0x2d, 0x430, 0x444, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x2d,
0x430, 0x444, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x408, 0x443, 0x436, 0x43d, 0x43e, 0x2d, 0x430, 0x444, 0x440, 0x438, 0x447,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430,
0x434, 0x43d, 0x43e, 0x2d, 0x430, 0x444, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e,
0x2d, 0x430, 0x444, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x2d, 0x430, 0x444, 0x440,
0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x410, 0x459, 0x430, 0x441, 0x43a, 0x430, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x410, 0x459, 0x430, 0x441, 0x43a, 0x430, 0x2c, 0x20,
0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410,
0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410,
0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410,
0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d,
0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x446, 0x435,
0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440,
0x438, 0x447, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b,
0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440,
0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x446,
0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x43b, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435,
0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20,
0x438, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440,
0x438, 0x447, 0x43a, 0x43e, 0x20, 0x438, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e,
0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x438, 0x441, 0x442,
0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430,
0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43f, 0x43b, 0x430, 0x43d,
0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421,
0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447,
0x43a, 0x43e, 0x20, 0x43f, 0x43b, 0x430, 0x43d, 0x438, 0x43d, 0x441, 0x43a, 0x43e,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e,
0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43f, 0x43b, 0x430,
0x43d, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d,
0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43f, 0x430,
0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440,
0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x447,
0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440,
0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43f,
0x430, 0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43f, 0x438, 0x458,
0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43f, 0x438, 0x458, 0x430,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43f, 0x438, 0x458, 0x430, 0x2c,
0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x410, 0x440, 0x430, 0x431, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x410, 0x440, 0x430, 0x431, 0x438, 0x458, 0x441, 0x43a, 0x43e,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x430, 0x431, 0x438, 0x458, 0x441,
0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438,
0x43d, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x433, 0x435,
0x43d, 0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d,
0x430, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430,
0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x20,
0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x43b,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x408, 0x435,
0x440, 0x43c, 0x435, 0x43d, 0x438, 0x458, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x408, 0x435, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x458, 0x430, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x408, 0x435, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x458,
0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x441, 0x43a, 0x43e, 0x20,
0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410,
0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20,
0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458,
0x441, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d,
0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440,
0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x458, 0x441, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b,
0x43d, 0x43e, 0x20, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x458, 0x441, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b,
0x43d, 0x43e, 0x20, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458,
0x441, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d,
0x43e, 0x20, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20, 0x43b, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x443, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x438, 0x441,
0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410,
0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20,
0x438, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e,
0x20, 0x438, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x43b, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x443, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x437, 0x430, 0x43f,
0x430, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x443,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x437,
0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410,
0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20,
0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x437, 0x435, 0x440, 0x431,
0x435, 0x458, 0x45f, 0x430, 0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410,
0x437, 0x435, 0x440, 0x431, 0x435, 0x458, 0x45f, 0x430, 0x43d, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x410, 0x437, 0x435, 0x440, 0x431, 0x435, 0x458, 0x45f, 0x430,
0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x410, 0x437, 0x43e, 0x440, 0x438, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x410, 0x437, 0x43e, 0x440, 0x438, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x410, 0x437, 0x43e, 0x440, 0x438, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430,
0x434, 0x435, 0x448, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411, 0x430, 0x43d,
0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x2c, 0x20, 0x43b,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411, 0x443,
0x442, 0x430, 0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411, 0x43e, 0x43b,
0x438, 0x432, 0x438, 0x458, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411,
0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x458, 0x430, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x458, 0x430, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438,
0x458, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x411, 0x440, 0x443, 0x43d, 0x435, 0x458, 0x20, 0x414, 0x430,
0x440, 0x443, 0x441, 0x430, 0x43b, 0x443, 0x43c, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x417, 0x435, 0x43b, 0x435, 0x43d, 0x43e, 0x440, 0x442, 0x441, 0x43a, 0x430,
0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x417, 0x435, 0x43b, 0x435, 0x43d, 0x43e, 0x440, 0x442, 0x441, 0x43a, 0x430,
0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x417, 0x435, 0x43b, 0x435, 0x43d, 0x43e, 0x440, 0x442, 0x441, 0x43a, 0x430,
0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x430, 0x43c, 0x43e,
0x440, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x430, 0x442, 0x430,
0x43c, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x430, 0x442, 0x430, 0x43c,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x430, 0x442, 0x430, 0x43c, 0x2c,
0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x427, 0x438, 0x43b, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x438,
0x43b, 0x435, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x438, 0x43b, 0x435,
0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x41a, 0x438, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a,
0x438, 0x43d, 0x435, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a,
0x438, 0x43d, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x411, 0x43e, 0x436, 0x438, 0x45b, 0x43d, 0x43e, 0x20,
0x43e, 0x441, 0x442, 0x440, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x20, 0x28, 0x41a, 0x435, 0x43b, 0x438, 0x43d,
0x433, 0x29, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x458, 0x430,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431,
0x438, 0x458, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x43b,
0x443, 0x43c, 0x431, 0x438, 0x458, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x443, 0x43a, 0x43e, 0x432,
0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41a, 0x443, 0x43a, 0x43e, 0x432, 0x430, 0x20, 0x43e, 0x441, 0x442,
0x440, 0x432, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x443, 0x43a,
0x43e, 0x432, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x2c, 0x20,
0x43f, 0x43e, 0x43b, 0x443, 0x2d, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x41a, 0x443, 0x431, 0x430, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41a, 0x443, 0x431, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x414, 0x435, 0x458, 0x432, 0x438,
0x441, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x414, 0x438, 0x43c, 0x43e, 0x43d,
0x20, 0x434, 0x2019, 0x423, 0x440, 0x432, 0x438, 0x43b, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x438, 0x20, 0x442, 0x438,
0x43c, 0x43e, 0x440, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x423, 0x441, 0x43a,
0x440, 0x448, 0x45a, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x423, 0x441, 0x43a, 0x440, 0x448, 0x45a, 0x430,
0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x423, 0x441, 0x43a, 0x440, 0x448, 0x45a, 0x430, 0x20, 0x43e, 0x441, 0x442,
0x440, 0x432, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x415, 0x43a, 0x432, 0x430, 0x434, 0x43e, 0x440, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x440, 0x435, 0x434, 0x45a, 0x435, 0x435,
0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x421, 0x440, 0x435, 0x434, 0x45a, 0x435, 0x435, 0x432, 0x440, 0x43e, 0x43f,
0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x440, 0x435, 0x434,
0x45a, 0x435, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x43b,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x441,
0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a,
0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x434, 0x430, 0x459, 0x435, 0x433,
0x20, 0x438, 0x441, 0x442, 0x43e, 0x43a, 0x430, 0x20, 0x415, 0x432, 0x440, 0x43e,
0x43f, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x435, 0x432, 0x440,
0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x430, 0x43d,
0x434, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x430, 0x43d,
0x434, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x430, 0x43d,
0x434, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x2c,
0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x424, 0x438, 0x45f, 0x438, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x424, 0x438,
0x45f, 0x438, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x424, 0x438, 0x45f, 0x438,
0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x443, 0x441, 0x43a, 0x430, 0x20, 0x413,
0x432, 0x430, 0x458, 0x430, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x413, 0x430, 0x43c, 0x431, 0x438, 0x458, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x413, 0x440, 0x443, 0x437, 0x438, 0x458, 0x430, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x413, 0x440, 0x443, 0x437, 0x438, 0x458, 0x430,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x413, 0x440, 0x443, 0x437, 0x438, 0x458,
0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x413, 0x438, 0x43b, 0x431, 0x435, 0x440, 0x442, 0x20, 0x43e, 0x441,
0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x440,
0x435, 0x434, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x43f,
0x43e, 0x20, 0x413, 0x440, 0x438, 0x43d, 0x438, 0x447, 0x443, 0x418, 0x441, 0x442,
0x43e, 0x447, 0x43d, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d,
0x434, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x441, 0x442, 0x43e, 0x447,
0x43d, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x2c,
0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d,
0x43b, 0x430, 0x43d, 0x434, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430,
0x43f, 0x430, 0x434, 0x43d, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430,
0x43d, 0x434, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x417, 0x430, 0x43b, 0x438, 0x432, 0x441, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x425, 0x430, 0x432, 0x430, 0x458, 0x441, 0x43a,
0x43e, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x425, 0x430, 0x432, 0x430, 0x458, 0x441, 0x43a, 0x43e,
0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x425, 0x430, 0x432, 0x430, 0x458, 0x441, 0x43a, 0x43e, 0x2d, 0x430,
0x43b, 0x435, 0x443, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x425, 0x43e, 0x43d, 0x433, 0x20,
0x41a, 0x43e, 0x43d, 0x433, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x425, 0x43e,
0x43d, 0x433, 0x20, 0x41a, 0x43e, 0x43d, 0x433, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x425, 0x43e, 0x43d, 0x433, 0x20, 0x41a, 0x43e, 0x43d, 0x433, 0x2c, 0x20,
0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x425,
0x43e, 0x432, 0x434, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x425, 0x43e, 0x432,
0x434, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x425, 0x43e, 0x432, 0x434, 0x2c,
0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x418, 0x43d, 0x434, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x418, 0x43d, 0x434, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x43e, 0x43a,
0x435, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x418, 0x43d, 0x434, 0x43e, 0x43a, 0x438, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x2d,
0x438, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x458, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d,
0x43e, 0x2d, 0x438, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x458, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430,
0x434, 0x43d, 0x43e, 0x2d, 0x438, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438,
0x458, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x440,
0x430, 0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x430, 0x43d,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x430, 0x43d, 0x2c, 0x20,
0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418,
0x440, 0x43a, 0x443, 0x446, 0x43a, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418,
0x440, 0x43a, 0x443, 0x446, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418,
0x440, 0x43a, 0x443, 0x446, 0x43a, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x437, 0x440, 0x430, 0x435, 0x43b,
0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x437, 0x440, 0x430,
0x435, 0x43b, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x408, 0x430, 0x43f, 0x430, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x408, 0x430, 0x43f, 0x430, 0x43d,
0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x408, 0x430, 0x43f, 0x430,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x41f, 0x435, 0x442, 0x440, 0x43e, 0x43f, 0x430, 0x432,
0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x43e, 0x2d, 0x43a, 0x430, 0x43c, 0x447, 0x430,
0x442, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440,
0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x43d, 0x430, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x2d, 0x43a,
0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e,
0x2d, 0x43a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x440, 0x435, 0x458,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x440,
0x435, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x43e,
0x440, 0x435, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x448, 0x440, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x458,
0x430, 0x440, 0x441, 0x43a, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x440,
0x430, 0x441, 0x43d, 0x43e, 0x458, 0x430, 0x440, 0x441, 0x43a, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x458, 0x430, 0x440,
0x441, 0x43a, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41a, 0x438, 0x440, 0x433, 0x438, 0x441, 0x442, 0x430, 0x43d,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430,
0x20, 0x41b, 0x430, 0x458, 0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41b,
0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x432, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x432, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x432, 0x2c,
0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41c, 0x430, 0x43a, 0x430, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x43d, 0x430, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430,
0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41c, 0x430, 0x43b, 0x435, 0x437, 0x438, 0x458, 0x430, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x43b, 0x434, 0x438, 0x432, 0x438, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x440, 0x43a, 0x438, 0x437, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x440, 0x448, 0x430, 0x43b, 0x441,
0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41c, 0x430, 0x443, 0x440, 0x438, 0x446, 0x438, 0x458, 0x443,
0x441, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x443, 0x440, 0x438,
0x446, 0x438, 0x458, 0x443, 0x441, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c,
0x430, 0x443, 0x440, 0x438, 0x446, 0x438, 0x458, 0x443, 0x441, 0x2c, 0x20, 0x43b,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x43e,
0x441, 0x43e, 0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x435, 0x43a,
0x441, 0x438, 0x447, 0x43a, 0x438, 0x20, 0x41f, 0x430, 0x446, 0x438, 0x444, 0x438,
0x43a, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x447, 0x43a, 0x438, 0x20, 0x41f, 0x430,
0x446, 0x438, 0x444, 0x438, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c,
0x435, 0x43a, 0x441, 0x438, 0x447, 0x43a, 0x438, 0x20, 0x41f, 0x430, 0x446, 0x438,
0x444, 0x438, 0x43a, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x423, 0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430, 0x442,
0x43e, 0x440, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x423, 0x43b, 0x430, 0x43d,
0x20, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x423, 0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x2c, 0x20,
0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c,
0x43e, 0x441, 0x43a, 0x432, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c,
0x43e, 0x441, 0x43a, 0x432, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c,
0x43e, 0x441, 0x43a, 0x432, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x438, 0x458, 0x430, 0x43d, 0x43c,
0x430, 0x440, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x430, 0x443, 0x440,
0x443, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x435, 0x43f, 0x430, 0x43b,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x430, 0x20, 0x41a,
0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x458, 0x430, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x430, 0x20, 0x41a, 0x430, 0x43b, 0x435,
0x434, 0x43e, 0x43d, 0x438, 0x458, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41d, 0x43e, 0x432, 0x430, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d,
0x438, 0x458, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x438, 0x20, 0x417, 0x435, 0x43b,
0x430, 0x43d, 0x434, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432,
0x438, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x438, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d,
0x434, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x40a, 0x443, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d,
0x434, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x40a, 0x443, 0x444, 0x430, 0x443,
0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x438, 0x443, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a,
0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x41e, 0x441, 0x442,
0x440, 0x432, 0x43e, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x440,
0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x43e, 0x2c,
0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20,
0x41d, 0x43e, 0x440, 0x43e, 0x45a, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20,
0x41d, 0x43e, 0x440, 0x43e, 0x45a, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20,
0x41d, 0x43e, 0x440, 0x43e, 0x45a, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x43e, 0x441,
0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431,
0x438, 0x440, 0x441, 0x43a, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41e, 0x43c, 0x441, 0x43a, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41e, 0x43c, 0x441, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x41e, 0x43c, 0x441, 0x43a, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442,
0x430, 0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x43a, 0x438,
0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x430,
0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x43b, 0x430, 0x443,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x43f, 0x443, 0x430, 0x20,
0x41d, 0x43e, 0x432, 0x430, 0x20, 0x413, 0x432, 0x438, 0x43d, 0x435, 0x458, 0x430,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432,
0x430, 0x458, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x440, 0x430,
0x433, 0x432, 0x430, 0x458, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x430,
0x440, 0x430, 0x433, 0x432, 0x430, 0x458, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x435, 0x440, 0x443, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x435, 0x440, 0x443, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41f, 0x435, 0x440, 0x443, 0x2c, 0x20, 0x43b, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x424, 0x438, 0x43b, 0x438,
0x43f, 0x438, 0x43d, 0x438, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x424, 0x438,
0x43b, 0x438, 0x43f, 0x438, 0x43d, 0x438, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x424, 0x438, 0x43b, 0x438, 0x43f, 0x438, 0x43d, 0x438, 0x2c, 0x20, 0x43b, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x424, 0x435, 0x43d,
0x438, 0x43a, 0x441, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x43d, 0x20, 0x41f, 0x458, 0x435, 0x440,
0x20, 0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x43d, 0x20, 0x41f, 0x458, 0x435, 0x440,
0x20, 0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x2c, 0x20,
0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f,
0x438, 0x442, 0x43a, 0x435, 0x440, 0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41f, 0x43e, 0x43d, 0x43f, 0x435, 0x458, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41f, 0x458, 0x43e, 0x43d, 0x433, 0x458, 0x430, 0x43d, 0x448, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x420, 0x435, 0x438, 0x43d, 0x438, 0x43e, 0x43d,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x420, 0x43e, 0x442, 0x435, 0x440, 0x430,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438,
0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x445, 0x430, 0x43b,
0x438, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x445, 0x430,
0x43b, 0x438, 0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x458, 0x448,
0x435, 0x43b, 0x438, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x438, 0x43d,
0x433, 0x430, 0x43f, 0x443, 0x440, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421,
0x43e, 0x43b, 0x43e, 0x43c, 0x43e, 0x43d, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441,
0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x408, 0x443,
0x436, 0x43d, 0x430, 0x20, 0x40f, 0x43e, 0x440, 0x45f, 0x438, 0x458, 0x430, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x443, 0x440, 0x438, 0x43d, 0x430, 0x43c,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x428, 0x43e, 0x432, 0x430, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x422, 0x430, 0x445, 0x438, 0x442, 0x438, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x422, 0x430, 0x458, 0x43f, 0x435, 0x458, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x422, 0x430, 0x458, 0x43f, 0x435, 0x458, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x422, 0x430, 0x458, 0x43f, 0x435, 0x458, 0x2c, 0x20,
0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422,
0x430, 0x45f, 0x438, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x422, 0x43e, 0x43a, 0x435, 0x43b, 0x430, 0x443, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x443, 0x443, 0x43a,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435,
0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435,
0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x443, 0x432, 0x430, 0x43b,
0x443, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x423, 0x440, 0x443, 0x433, 0x432,
0x430, 0x458, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x423, 0x440, 0x443, 0x433,
0x432, 0x430, 0x458, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x423, 0x440, 0x443,
0x433, 0x432, 0x430, 0x458, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x423, 0x437, 0x431,
0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c,
0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x2c, 0x20, 0x43b,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x412, 0x435,
0x43d, 0x435, 0x446, 0x443, 0x435, 0x43b, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432,
0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x412,
0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x2c, 0x20,
0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x412,
0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440,
0x430, 0x434, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x412, 0x435, 0x458, 0x43a, 0x20, 0x43e, 0x441, 0x442, 0x440,
0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x412, 0x430, 0x43b, 0x438,
0x441, 0x20, 0x438, 0x20, 0x424, 0x443, 0x442, 0x443, 0x43d, 0x430, 0x20, 0x41e,
0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x408,
0x430, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x408, 0x430, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x408, 0x430, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x2c, 0x20, 0x43b, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x408, 0x435, 0x43a,
0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x408, 0x435, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438,
0x43d, 0x431, 0x443, 0x440, 0x433, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x408,
0x435, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433,
0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x2d, 0x430, 0x444, 0x440,
0x438, 0x447, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x459, 0x430, 0x441, 0x43a, 0x430,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x459, 0x430, 0x441,
0x43a, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x43c, 0x430, 0x437, 0x43e,
0x43d, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x421, 0x458, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e,
0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d,
0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x421, 0x458, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c,
0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440,
0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421,
0x458, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438,
0x447, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d,
0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x421, 0x458, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430,
0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x438, 0x441, 0x442, 0x43e,
0x447, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421,
0x458, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438,
0x447, 0x43a, 0x43e, 0x20, 0x438, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x458, 0x435, 0x432, 0x435, 0x440,
0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x438,
0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x458, 0x435, 0x432,
0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e,
0x20, 0x43f, 0x43b, 0x430, 0x43d, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x458, 0x435, 0x432, 0x435, 0x440,
0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43f,
0x43b, 0x430, 0x43d, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x421, 0x458, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430,
0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43f, 0x43b, 0x430, 0x43d,
0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x458, 0x435, 0x432, 0x435,
0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20,
0x43f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x458, 0x435, 0x432, 0x435, 0x440, 0x43d,
0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43f, 0x430,
0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x421, 0x458, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c,
0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43f, 0x430, 0x446, 0x438, 0x444,
0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x43f, 0x438, 0x458, 0x430, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x43f, 0x438, 0x458, 0x430,
0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x410, 0x440, 0x430, 0x431, 0x438, 0x458, 0x441, 0x43a, 0x43e,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x433, 0x435, 0x43d,
0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434,
0x43d, 0x430, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430,
0x434, 0x43d, 0x430, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d,
0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x408, 0x435, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x458,
0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x408, 0x435, 0x440,
0x43c, 0x435, 0x43d, 0x438, 0x458, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x408, 0x435, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x458, 0x430, 0x2c,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x441, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x442, 0x43b, 0x430, 0x43d,
0x442, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410,
0x442, 0x43b, 0x430, 0x43d, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x443,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x446,
0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x443,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x446,
0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x437, 0x430, 0x43f,
0x430, 0x434, 0x43d, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x438, 0x441, 0x442, 0x43e, 0x447,
0x43d, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x458, 0x441, 0x43a, 0x43e, 0x20, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x410, 0x437, 0x435, 0x440, 0x431, 0x435, 0x458, 0x45f, 0x430, 0x43d,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x437, 0x435, 0x440,
0x431, 0x435, 0x458, 0x45f, 0x430, 0x43d, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x437, 0x43e,
0x440, 0x438, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x437,
0x43e, 0x440, 0x438, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430,
0x434, 0x435, 0x448, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x411,
0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x2c, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x411,
0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x458, 0x430, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x458,
0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x443, 0x43d, 0x435, 0x458, 0x20, 0x414,
0x430, 0x440, 0x443, 0x441, 0x430, 0x43b, 0x443, 0x43c, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x417, 0x435, 0x43b, 0x435, 0x43d, 0x43e, 0x440, 0x442,
0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x435, 0x43b, 0x435, 0x43d, 0x43e,
0x440, 0x442, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x435, 0x43b, 0x435,
0x43d, 0x43e, 0x440, 0x442, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440,
0x432, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x427, 0x430, 0x442, 0x430, 0x43c, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x427, 0x430, 0x442, 0x430, 0x43c, 0x2c,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x427, 0x438, 0x43b, 0x435, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x427, 0x438, 0x43b, 0x435, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x438, 0x43d,
0x435, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a,
0x438, 0x43d, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x411, 0x43e, 0x436, 0x438, 0x45b, 0x43d,
0x43e, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x43e, 0x432, 0x430,
0x20, 0x28, 0x41a, 0x438, 0x43b, 0x438, 0x43d, 0x433, 0x29, 0x20, 0x43e, 0x441,
0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x458, 0x430, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438,
0x458, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x443, 0x43a, 0x43e, 0x432, 0x430, 0x20,
0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x41a, 0x443, 0x43a, 0x43e, 0x432, 0x430, 0x20, 0x41e, 0x441, 0x442,
0x440, 0x432, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a,
0x443, 0x43a, 0x43e, 0x432, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430,
0x2c, 0x20, 0x43f, 0x43e, 0x43b, 0x443, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x443, 0x431, 0x430, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x443, 0x431, 0x430, 0x2c,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x414, 0x438, 0x43c, 0x43e, 0x43d, 0x20, 0x434, 0x2019, 0x418, 0x440,
0x432, 0x438, 0x43b, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418,
0x441, 0x442, 0x43e, 0x447, 0x43d, 0x438, 0x20, 0x422, 0x438, 0x43c, 0x43e, 0x440,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x423, 0x441, 0x43a, 0x440,
0x448, 0x45a, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x423, 0x441, 0x43a, 0x440, 0x448, 0x45a,
0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x2c, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421,
0x440, 0x435, 0x434, 0x45a, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x440, 0x435,
0x434, 0x45a, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x440, 0x435, 0x434, 0x45a, 0x43e,
0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x441,
0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a,
0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x435, 0x432,
0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x424, 0x43e, 0x43b, 0x43a,
0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440,
0x432, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x424, 0x43e,
0x43b, 0x43a, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441,
0x442, 0x440, 0x432, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x424, 0x438, 0x45f, 0x438, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x424, 0x438, 0x45f, 0x438, 0x2c,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x413, 0x430, 0x43c, 0x431, 0x438, 0x458, 0x435, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x413, 0x440, 0x443, 0x437, 0x438, 0x458, 0x430,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x413, 0x440, 0x443, 0x437,
0x438, 0x458, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x413, 0x438, 0x43b, 0x431, 0x435, 0x440,
0x442, 0x43e, 0x432, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x440, 0x435, 0x434, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x20, 0x43f, 0x43e,
0x20, 0x413, 0x440, 0x438, 0x43d, 0x438, 0x447, 0x443, 0x418, 0x441, 0x442, 0x43e,
0x447, 0x43d, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x441, 0x442, 0x43e,
0x447, 0x43d, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434,
0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x438, 0x20, 0x413,
0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x438, 0x20, 0x413,
0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x2c, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x425, 0x430,
0x432, 0x430, 0x458, 0x441, 0x43a, 0x43e, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442,
0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x425, 0x43e, 0x43d, 0x433, 0x20, 0x41a, 0x43e,
0x43d, 0x433, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x425, 0x43e,
0x43d, 0x433, 0x20, 0x41a, 0x43e, 0x43d, 0x433, 0x2c, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x425, 0x43e,
0x432, 0x434, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x425, 0x43e,
0x432, 0x434, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x430, 0x43d, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x430, 0x43d, 0x2c, 0x20, 0x459,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x418, 0x440, 0x43a, 0x443, 0x446, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x418, 0x440, 0x43a, 0x443, 0x446, 0x43a, 0x2c, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418,
0x437, 0x440, 0x430, 0x435, 0x43b, 0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x408, 0x430,
0x43f, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x430, 0x437, 0x430,
0x445, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x440, 0x435, 0x458, 0x441, 0x43a, 0x43e,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x458, 0x430, 0x440, 0x441,
0x43a, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x440, 0x430,
0x441, 0x43d, 0x43e, 0x458, 0x430, 0x440, 0x441, 0x43a, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x458, 0x430,
0x440, 0x441, 0x43a, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41b, 0x438, 0x43d, 0x438, 0x458, 0x441,
0x43a, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e,
0x432, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41b, 0x43e, 0x440,
0x434, 0x20, 0x425, 0x43e, 0x432, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x433, 0x430,
0x434, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c,
0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x443,
0x440, 0x438, 0x446, 0x438, 0x458, 0x443, 0x441, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x41c, 0x430, 0x443, 0x440, 0x438, 0x446, 0x438, 0x458, 0x443,
0x441, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x447, 0x43a, 0x438,
0x20, 0x41f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x43a, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x447, 0x43a, 0x438,
0x20, 0x41f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x43a, 0x2c, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x423,
0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x423, 0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430,
0x442, 0x43e, 0x440, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x435, 0x43c, 0x435, 0x41c, 0x43e, 0x441, 0x43a, 0x432,
0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c, 0x43e, 0x441,
0x43a, 0x432, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c, 0x458, 0x430, 0x43d, 0x43c, 0x430,
0x440, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432,
0x430, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x458, 0x430,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x430,
0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x458, 0x430, 0x2c,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x438, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d,
0x434, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432,
0x438, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x2c, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x40a,
0x443, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x40a, 0x443, 0x444, 0x430, 0x443, 0x43d,
0x434, 0x43b, 0x435, 0x43d, 0x434, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d, 0x438, 0x458, 0x443,
0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x43e, 0x441, 0x442,
0x440, 0x432, 0x43e, 0x20, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x43e, 0x441, 0x442, 0x440, 0x432,
0x43e, 0x20, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x43e, 0x20,
0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x2c, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x424, 0x435,
0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d, 0x43e,
0x440, 0x43e, 0x45a, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20,
0x41d, 0x43e, 0x440, 0x43e, 0x45a, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432,
0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438,
0x440, 0x441, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d,
0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x2c, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x41e, 0x43c, 0x441, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x41e, 0x43c, 0x441, 0x43a, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x43a, 0x438,
0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41f,
0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x458, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x458, 0x2c,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x41f, 0x435, 0x440, 0x443, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x41f, 0x435, 0x440, 0x443, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x424, 0x438, 0x43b,
0x438, 0x43f, 0x438, 0x43d, 0x438, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x438, 0x43d, 0x438, 0x2c, 0x20, 0x459,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x421, 0x435, 0x43d, 0x20, 0x41f, 0x458, 0x435, 0x440, 0x20, 0x438, 0x20, 0x41c,
0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x421, 0x435, 0x43d, 0x20, 0x41f, 0x458, 0x435, 0x440, 0x20, 0x438,
0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x2c, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41f,
0x43e, 0x43d, 0x43f, 0x435, 0x458, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x41f, 0x458, 0x43e, 0x43d, 0x433, 0x458, 0x430, 0x43d, 0x448, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x420, 0x435, 0x443, 0x43d,
0x438, 0x43e, 0x43d, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421,
0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x2c, 0x20, 0x459,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x421, 0x430, 0x43c, 0x43e, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x438, 0x43d,
0x433, 0x430, 0x43f, 0x443, 0x440, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x422, 0x430, 0x458, 0x43f, 0x435, 0x458, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x422, 0x430, 0x458, 0x43f, 0x435, 0x458, 0x2c, 0x20, 0x459,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x422, 0x43e, 0x43d, 0x433, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x427, 0x443, 0x43a,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x422, 0x443, 0x440, 0x43a,
0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438,
0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x423, 0x440, 0x443, 0x433, 0x432,
0x430, 0x458, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x423, 0x440,
0x443, 0x433, 0x432, 0x430, 0x458, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x423, 0x437, 0x431, 0x435,
0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d,
0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442,
0x443, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441,
0x442, 0x43e, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x412,
0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x2c, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433,
0x440, 0x430, 0x434, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x43e,
0x20, 0x412, 0x435, 0x458, 0x43a, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x412, 0x430, 0x43b, 0x438,
0x441, 0x20, 0x438, 0x20, 0x424, 0x443, 0x442, 0x443, 0x43d, 0x430, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x408, 0x430, 0x43a, 0x443, 0x442, 0x441,
0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x408, 0x430, 0x43a,
0x443, 0x442, 0x441, 0x43a, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x408, 0x435, 0x43a, 0x430, 0x442,
0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x408, 0x435, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438,
0x43d, 0x431, 0x443, 0x440, 0x433, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41, 0x6b, 0x72, 0x65,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6b, 0x72, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x41, 0x6b, 0x72, 0x65, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x72, 0x61, 0x10d, 0x75, 0x6e, 0x61, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x6e, 0x61, 0x41, 0x76, 0x67, 0x61,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x2d,
0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x2d, 0x61,
0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4a, 0x75, 0x17e, 0x6e, 0x6f, 0x2d, 0x61, 0x66, 0x72, 0x69,
0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a,
0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x2d, 0x61, 0x66, 0x72, 0x69, 0x10d,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61,
0x70, 0x61, 0x64, 0x6e, 0x6f, 0x2d, 0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61,
0x64, 0x6e, 0x6f, 0x2d, 0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x41, 0x6c, 0x6a, 0x61, 0x73, 0x6b, 0x61, 0x2c, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x6c, 0x6a, 0x61, 0x73, 0x6b,
0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x69,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x69, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x72, 0x61, 0x10d, 0x75, 0x6e,
0x61, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x6e, 0x61,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x2c, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x2c, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65,
0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b,
0x6f, 0x20, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x6c, 0x61, 0x6e, 0x69, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76, 0x65,
0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20,
0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x41, 0x70, 0x69, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x41, 0x70, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x41, 0x70, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x41, 0x6b, 0x76, 0x61, 0x74, 0x61, 0x75, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x41, 0x6b, 0x76, 0x61, 0x74, 0x61, 0x75, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x41, 0x6b, 0x76, 0x61, 0x74, 0x61, 0x75, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x72, 0x61, 0x10d, 0x75, 0x6e, 0x61, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x6e, 0x61, 0x41, 0x6b, 0x75,
0x74, 0x6f, 0x62, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6b,
0x75, 0x74, 0x6f, 0x62, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6b,
0x75, 0x74, 0x6f, 0x62, 0x65, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x72, 0x61, 0x10d, 0x75, 0x6e, 0x61, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x6e, 0x61, 0x41, 0x72, 0x61, 0x62, 0x69, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64,
0x6e, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61,
0x64, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70,
0x61, 0x64, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x65, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x4a, 0x65, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x65, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x6e, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x69, 0x73,
0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f,
0x20, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f,
0x20, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x7a, 0x61, 0x70, 0x61,
0x64, 0x6e, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x7a, 0x65, 0x72, 0x62,
0x65, 0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65, 0x6a, 0x64, 0x17e, 0x61,
0x6e, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x65, 0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x2c, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x41, 0x7a, 0x6f, 0x72, 0x69, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x41, 0x7a, 0x6f, 0x72, 0x69, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x41, 0x7a, 0x6f, 0x72, 0x69, 0x2c, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x161, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x2c, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x42, 0x75, 0x74, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x6a, 0x61, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c,
0x69, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42,
0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x6a,
0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x6a,
0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x61, 0x6c, 0x75, 0x6d, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x65, 0x6c, 0x65, 0x6e, 0x6f,
0x72, 0x74, 0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x65, 0x6c, 0x65,
0x6e, 0x6f, 0x72, 0x74, 0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72,
0x76, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x65,
0x6c, 0x65, 0x6e, 0x6f, 0x72, 0x74, 0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73,
0x74, 0x72, 0x76, 0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10c, 0x61, 0x6d,
0x6f, 0x72, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10c,
0x61, 0x74, 0x61, 0x6d, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x10c, 0x61, 0x74, 0x61, 0x6d, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x10c, 0x61, 0x74, 0x61, 0x6d, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10c,
0x69, 0x6c, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10c,
0x69, 0x6c, 0x65, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10c,
0x69, 0x6c, 0x65, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x69, 0x6e, 0x61,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x69, 0x6e, 0x61,
0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42, 0x6f, 0x17e, 0x69, 0x107, 0x6e, 0x6f,
0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x6f, 0x76, 0x61, 0x20,
0x28, 0x4b, 0x69, 0x6c, 0x69, 0x6e, 0x67, 0x29, 0x20, 0x6f, 0x73, 0x74,
0x72, 0x76, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b,
0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a,
0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x6f, 0x6c,
0x75, 0x6d, 0x62, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b,
0x75, 0x6b, 0x6f, 0x76, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x75, 0x6b, 0x6f,
0x76, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x75, 0x6b, 0x6f, 0x76, 0x61, 0x20,
0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x2c, 0x20, 0x70, 0x6f, 0x6c, 0x75,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x4b, 0x75, 0x62, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x4b, 0x75, 0x62, 0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x44, 0x65, 0x6a, 0x76, 0x69, 0x73, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x44, 0x69, 0x6d, 0x6f, 0x6e, 0x20, 0x64, 0x2019, 0x49, 0x72,
0x76, 0x69, 0x6c, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49,
0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x69, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x73, 0x6b, 0x72,
0x161, 0x6e, 0x6a, 0x61, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x73, 0x6b, 0x72, 0x161,
0x6e, 0x6a, 0x61, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x2c, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x73, 0x6b, 0x72, 0x161, 0x6e,
0x6a, 0x61, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x2c, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x45, 0x6b, 0x76, 0x61, 0x64, 0x6f, 0x72, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x6f,
0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x53, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x6f, 0x65,
0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x53, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x6f, 0x65, 0x76, 0x72,
0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74,
0x6f, 0x10d, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x56, 0x72, 0x65, 0x6d, 0x65, 0x20, 0x64, 0x61,
0x6c, 0x6a, 0x65, 0x67, 0x20, 0x69, 0x73, 0x74, 0x6f, 0x6b, 0x61, 0x20,
0x45, 0x76, 0x72, 0x6f, 0x70, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e,
0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x61,
0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x2c,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x2c, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x2c,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x75, 0x73, 0x6b,
0x61, 0x20, 0x47, 0x76, 0x61, 0x6a, 0x61, 0x6e, 0x61, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x75, 0x73,
0x6b, 0x6f, 0x20, 0x6a, 0x75, 0x17e, 0x6e, 0x6f, 0x20, 0x69, 0x20, 0x61,
0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61,
0x67, 0x6f, 0x73, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x47,
0x61, 0x6d, 0x62, 0x69, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x6a, 0x61, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x6a, 0x61,
0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x47, 0x72, 0x75, 0x7a,
0x69, 0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x47, 0x69, 0x6c, 0x62,
0x65, 0x72, 0x74, 0x6f, 0x76, 0x61, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76,
0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x72, 0x65,
0x64, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x70, 0x6f, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x69, 0x10d, 0x75, 0x49,
0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x69, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x69, 0x20, 0x47,
0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x69, 0x20, 0x47,
0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2c, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x69, 0x20, 0x47, 0x72, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x69, 0x20,
0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2c, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x69, 0x20,
0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2c, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x5a, 0x61, 0x6c, 0x69, 0x76, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b,
0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x73,
0x6b, 0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x73,
0x6b, 0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67,
0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x2c, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x48, 0x6f, 0x76, 0x64, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x48, 0x6f, 0x76, 0x64, 0x2c, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x49, 0x6e, 0x64, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6f, 0x6b, 0x65,
0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x49, 0x6e, 0x64, 0x6f, 0x6b, 0x69, 0x6e, 0x61, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x6e, 0x6f, 0x2d, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49,
0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x2d, 0x69, 0x6e, 0x64, 0x6f, 0x6e,
0x65, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x2d, 0x69,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x61, 0x6e, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x61, 0x6e, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x61, 0x6e, 0x2c,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x6b, 0x75, 0x63, 0x6b, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x6b, 0x75, 0x63, 0x6b,
0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x6b, 0x75,
0x63, 0x6b, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x7a, 0x72, 0x61, 0x65,
0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x61, 0x70, 0x61,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x65, 0x74, 0x72,
0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x6f, 0x2d, 0x6b,
0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x6f, 0x2d, 0x6b, 0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x6f, 0x2d, 0x6b, 0x61,
0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x72, 0x61, 0x10d, 0x75, 0x6e, 0x61, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x6e, 0x61, 0x4b, 0x61, 0x7a, 0x61,
0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x2d,
0x6b, 0x61, 0x7a, 0x61, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61,
0x64, 0x6e, 0x6f, 0x2d, 0x6b, 0x61, 0x7a, 0x61, 0x68, 0x73, 0x74, 0x61,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x4b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x4b, 0x6f, 0x161, 0x72, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72,
0x73, 0x6b, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x69, 0x72, 0x67, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x160, 0x72, 0x69, 0x20, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x4c, 0x69, 0x6e, 0x69, 0x6a, 0x73, 0x6b, 0x61, 0x20,
0x6f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x76, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x76, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x76, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x6c, 0x65, 0x7a, 0x69, 0x6a,
0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x6c,
0x64, 0x69, 0x76, 0x69, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x4d, 0x61, 0x72, 0x6b, 0x69, 0x7a, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4d, 0x61, 0x72, 0x161, 0x61, 0x6c, 0x73, 0x6b, 0x61, 0x20,
0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6a, 0x75, 0x73,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x63, 0x69, 0x6a, 0x75, 0x73, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6a, 0x75, 0x73,
0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x6f, 0x73, 0x6f, 0x6e, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x10d,
0x6b, 0x69, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x6b, 0x4d, 0x65,
0x6b, 0x73, 0x69, 0x10d, 0x6b, 0x69, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66,
0x69, 0x6b, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x65,
0x6b, 0x73, 0x69, 0x10d, 0x6b, 0x69, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66,
0x69, 0x6b, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x6c, 0x61, 0x6e, 0x20,
0x42, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x6c, 0x61, 0x6e, 0x20,
0x42, 0x61, 0x74, 0x6f, 0x72, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x65, 0x6d, 0x65, 0x4d,
0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x2c, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4d, 0x6a, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x61, 0x20,
0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x61, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x4b,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x4b, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x4e, 0x6f, 0x76, 0x69, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x69,
0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x69, 0x20, 0x5a, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6a, 0x75, 0x66, 0x61,
0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x4e, 0x6a, 0x75, 0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c,
0x65, 0x6e, 0x64, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x69, 0x6a, 0x75,
0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6f, 0x73, 0x74,
0x72, 0x76, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6f, 0x73, 0x74, 0x72, 0x76,
0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x6f, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x2c, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x6a, 0x61, 0x2c, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x6a,
0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2c, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x4f, 0x6d, 0x73, 0x6b, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x4f, 0x6d, 0x73, 0x6b, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x61, 0x70, 0x75, 0x61,
0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x47, 0x76, 0x69, 0x6e, 0x65, 0x6a,
0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x61, 0x72,
0x61, 0x67, 0x76, 0x61, 0x6a, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61,
0x6a, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x65, 0x72, 0x75, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x65, 0x72, 0x75, 0x2c, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x65, 0x72, 0x75, 0x2c, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x69, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x69, 0x6e, 0x69, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46,
0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x69, 0x2c, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x46, 0x65, 0x6e, 0x69, 0x6b, 0x73, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76,
0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x65, 0x6e,
0x20, 0x50, 0x6a, 0x65, 0x72, 0x20, 0x69, 0x20, 0x4d, 0x69, 0x6b, 0x65,
0x6c, 0x6f, 0x6e, 0x53, 0x65, 0x6e, 0x20, 0x50, 0x6a, 0x65, 0x72, 0x20,
0x69, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x65, 0x6e, 0x20, 0x50, 0x6a, 0x65,
0x72, 0x20, 0x69, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x2c,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x50, 0x69, 0x74, 0x6b, 0x65, 0x72, 0x6e, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x6f, 0x6e, 0x70, 0x65,
0x6a, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x52, 0x65, 0x75,
0x6e, 0x69, 0x6f, 0x6e, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x52, 0x6f, 0x74, 0x65, 0x72, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x61, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x6f,
0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x65, 0x6a, 0x161,
0x65, 0x6c, 0x69, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53,
0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x73, 0x6b,
0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x75, 0x17e, 0x6e, 0x61, 0x20, 0x44, 0x17e,
0x6f, 0x72, 0x64, 0x17e, 0x69, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x160, 0x6f, 0x76, 0x61, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x54, 0x61, 0x6a, 0x70,
0x65, 0x6a, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x54, 0x61,
0x6a, 0x70, 0x65, 0x6a, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x54,
0x61, 0x64, 0x17e, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61,
0x75, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10c, 0x75, 0x6b,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x72, 0x75, 0x67, 0x76,
0x61, 0x6a, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x72,
0x75, 0x67, 0x76, 0x61, 0x6a, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x2c, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x7a, 0x62, 0x65, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2c,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x65,
0x6e, 0x65, 0x63, 0x75, 0x65, 0x6c, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6f,
0x73, 0x74, 0x72, 0x76, 0x6f, 0x20, 0x56, 0x65, 0x6a, 0x6b, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x61,
0x20, 0x56, 0x61, 0x6c, 0x69, 0x73, 0x20, 0x69, 0x20, 0x46, 0x75, 0x74,
0x75, 0x6e, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x76, 0x67, 0x61, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x2d, 0x61, 0x66, 0x72, 0x69, 0x10d,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f,
0x10d, 0x6e, 0x6f, 0x2d, 0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x4a, 0x75, 0x17e, 0x6e, 0x6f, 0x2d, 0x61,
0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x2d, 0x61, 0x66, 0x72, 0x69,
0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70,
0x61, 0x64, 0x6e, 0x6f, 0x2d, 0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f,
0x2d, 0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6c, 0x6a,
0x61, 0x73, 0x6b, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6c,
0x6a, 0x61, 0x73, 0x6b, 0x61, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x65,
0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b,
0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e,
0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x69, 0x73,
0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53,
0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d,
0x6b, 0x6f, 0x20, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x69, 0x73, 0x74, 0x6f, 0x10d,
0x6e, 0x6f, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x6c, 0x61, 0x6e, 0x69,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x65,
0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b,
0x6f, 0x20, 0x70, 0x6c, 0x61, 0x6e, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x6c, 0x61, 0x6e,
0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e,
0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x61,
0x63, 0x69, 0x66, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x10d,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x65, 0x76, 0x65, 0x72,
0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70,
0x61, 0x63, 0x69, 0x66, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x70, 0x69,
0x6a, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x70, 0x69, 0x6a,
0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x70, 0x69, 0x6a, 0x61,
0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x41, 0x72, 0x61, 0x62, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x72, 0x61, 0x62, 0x69, 0x6a, 0x73,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x72, 0x61, 0x62, 0x69,
0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x2c, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x5a, 0x61,
0x70, 0x61, 0x64, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70,
0x61, 0x64, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61,
0x64, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x4a, 0x65, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x6a, 0x61,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4a, 0x65, 0x72, 0x6d, 0x65, 0x6e,
0x69, 0x6a, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4a, 0x65, 0x72,
0x6d, 0x65, 0x6e, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x74, 0x6c, 0x61,
0x6e, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x6e, 0x6f, 0x20, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x6e, 0x6f, 0x20, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x6e, 0x6f, 0x20, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20,
0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b,
0x6f, 0x20, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73,
0x6b, 0x6f, 0x20, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20,
0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b,
0x6f, 0x20, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73,
0x6b, 0x6f, 0x20, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41,
0x7a, 0x65, 0x72, 0x62, 0x65, 0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65, 0x6a, 0x64,
0x17e, 0x61, 0x6e, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x65, 0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x2c, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x7a,
0x6f, 0x72, 0x69, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x7a, 0x6f,
0x72, 0x69, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x7a, 0x6f, 0x72,
0x69, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x161, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x42, 0x75, 0x74, 0x61,
0x6e, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x42, 0x6f, 0x6c, 0x69, 0x76,
0x69, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x42, 0x72, 0x61,
0x7a, 0x69, 0x6c, 0x69, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x6a, 0x61,
0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x6a, 0x20, 0x44, 0x61, 0x72,
0x75, 0x73, 0x61, 0x6c, 0x75, 0x6d, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x5a, 0x65, 0x6c, 0x65, 0x6e, 0x6f, 0x72, 0x74, 0x73, 0x6b, 0x61, 0x20,
0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x5a, 0x65, 0x6c, 0x65, 0x6e, 0x6f, 0x72, 0x74, 0x73, 0x6b, 0x61, 0x20,
0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x5a, 0x65, 0x6c, 0x65, 0x6e, 0x6f, 0x72, 0x74, 0x73, 0x6b, 0x61, 0x20,
0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x10c, 0x61, 0x6d, 0x6f,
0x72, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x10c, 0x61, 0x74, 0x61,
0x6d, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x10c, 0x61, 0x74, 0x61, 0x6d,
0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x10c, 0x61, 0x74, 0x61, 0x6d, 0x2c,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x10c, 0x69, 0x6c, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x10c,
0x69, 0x6c, 0x65, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x10c, 0x69, 0x6c,
0x65, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x4b, 0x69, 0x6e, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x4b, 0x69, 0x6e, 0x65, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x4b, 0x69, 0x6e, 0x61, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x42, 0x6f, 0x17e, 0x69, 0x107,
0x6e, 0x6f, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x20, 0x28, 0x4b, 0x65,
0x6c, 0x69, 0x6e, 0x67, 0x29, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62,
0x69, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b, 0x6f, 0x6c,
0x75, 0x6d, 0x62, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b,
0x75, 0x6b, 0x6f, 0x76, 0x61, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x61,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b, 0x75, 0x6b, 0x6f, 0x76, 0x61,
0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x4b, 0x75, 0x6b, 0x6f, 0x76, 0x61, 0x20, 0x6f, 0x73, 0x74, 0x72,
0x76, 0x61, 0x2c, 0x20, 0x70, 0x6f, 0x6c, 0x75, 0x2d, 0x6c, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b, 0x75, 0x62,
0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b, 0x75, 0x62, 0x61, 0x2c,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x44, 0x65, 0x6a, 0x76, 0x69, 0x73, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x44, 0x69, 0x6d, 0x6f, 0x6e, 0x20, 0x64, 0x2019, 0x55, 0x72, 0x76,
0x69, 0x6c, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f,
0x10d, 0x6e, 0x69, 0x20, 0x74, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x55, 0x73, 0x6b, 0x72, 0x161, 0x6e, 0x6a, 0x61, 0x20,
0x6f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x55, 0x73, 0x6b, 0x72, 0x161, 0x6e, 0x6a, 0x61, 0x20, 0x6f, 0x73, 0x74,
0x72, 0x76, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x55, 0x73, 0x6b,
0x72, 0x161, 0x6e, 0x6a, 0x61, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x61,
0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x45, 0x6b, 0x76, 0x61, 0x64, 0x6f, 0x72, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x53, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x65, 0x65, 0x76,
0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x53, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x65, 0x65, 0x76, 0x72, 0x6f, 0x70,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x72, 0x65, 0x64,
0x6e, 0x6a, 0x65, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74,
0x6f, 0x10d, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f,
0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70,
0x61, 0x64, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e,
0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x65, 0x76, 0x72,
0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61,
0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61,
0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x46, 0x69, 0x64, 0x17e, 0x69, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x46, 0x72, 0x61, 0x6e, 0x63,
0x75, 0x73, 0x6b, 0x61, 0x20, 0x47, 0x76, 0x61, 0x6a, 0x61, 0x6e, 0x61,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x75,
0x73, 0x6b, 0x6f, 0x20, 0x6a, 0x75, 0x17e, 0x6e, 0x6f, 0x20, 0x69, 0x20,
0x61, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x10d, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67,
0x6f, 0x73, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x47, 0x61, 0x6d, 0x62,
0x69, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x47, 0x72, 0x75,
0x7a, 0x69, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x47, 0x72,
0x75, 0x7a, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x47,
0x72, 0x75, 0x7a, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x47, 0x69, 0x6c, 0x62,
0x65, 0x72, 0x74, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x53, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x20, 0x70, 0x6f, 0x20, 0x47, 0x72, 0x69,
0x6e, 0x69, 0x10d, 0x75, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x69, 0x20,
0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2c, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x69, 0x20, 0x47, 0x72,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61,
0x64, 0x6e, 0x69, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e,
0x69, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2c, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x5a, 0x61, 0x6c, 0x69, 0x76, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x2d, 0x61,
0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x2d, 0x61, 0x6c,
0x65, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x48,
0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75,
0x74, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f,
0x6e, 0x67, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67,
0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x48,
0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x2c, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x48, 0x6f,
0x76, 0x64, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x48, 0x6f, 0x76, 0x64,
0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x48, 0x6f, 0x76, 0x64, 0x2c, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x49, 0x6e, 0x64, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x49, 0x6e, 0x64, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6f, 0x6b,
0x65, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x49, 0x6e, 0x64, 0x6f, 0x6b, 0x69, 0x6e, 0x61, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x2d,
0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e,
0x6f, 0x2d, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61,
0x64, 0x6e, 0x6f, 0x2d, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69,
0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x49, 0x72,
0x61, 0x6e, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x61, 0x6e,
0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x61, 0x6e, 0x2c, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x49, 0x72, 0x6b, 0x75, 0x63, 0x6b, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x49, 0x72, 0x6b, 0x75, 0x63, 0x6b, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x49, 0x72, 0x6b, 0x75, 0x63, 0x6b, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x49, 0x7a, 0x72, 0x61,
0x65, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x49,
0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4a,
0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b,
0x61, 0x7a, 0x61, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f,
0x2d, 0x6b, 0x61, 0x7a, 0x61, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64,
0x6e, 0x6f, 0x2d, 0x6b, 0x61, 0x7a, 0x61, 0x68, 0x73, 0x74, 0x61, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b, 0x6f, 0x72,
0x65, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b,
0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x4b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b, 0x6f, 0x161,
0x72, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b,
0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b, 0x69, 0x72, 0x67, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4f, 0x73,
0x74, 0x72, 0x76, 0x61, 0x20, 0x4c, 0x61, 0x6a, 0x6e, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x76, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x76, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x76, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x6c, 0x65, 0x7a, 0x69, 0x6a, 0x61,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76,
0x69, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x72, 0x6b, 0x69,
0x7a, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x72, 0x161, 0x61,
0x6c, 0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69,
0x6a, 0x75, 0x73, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x63, 0x69, 0x6a, 0x75, 0x73, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6a, 0x75, 0x73, 0x2c,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x4d, 0x6f, 0x73, 0x6f, 0x6e, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x4d, 0x65, 0x6b, 0x73, 0x69, 0x10d, 0x6b, 0x69, 0x20, 0x50, 0x61, 0x63,
0x69, 0x66, 0x69, 0x6b, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4d, 0x65,
0x6b, 0x73, 0x69, 0x10d, 0x6b, 0x69, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66,
0x69, 0x6b, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74,
0x6f, 0x72, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x55, 0x6c, 0x61, 0x6e,
0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x2c, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4d, 0x69, 0x6a, 0x61,
0x6e, 0x6d, 0x61, 0x72, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4e, 0x61,
0x75, 0x72, 0x75, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4e, 0x65, 0x70,
0x61, 0x6c, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x61,
0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x61, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x4b, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x69, 0x20,
0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x4e, 0x6f, 0x76, 0x69, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x69, 0x20, 0x5a, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4e, 0x6a, 0x75, 0x66, 0x61, 0x75,
0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x4e, 0x6a, 0x75, 0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64,
0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x4e, 0x69, 0x75, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x4f, 0x73, 0x74, 0x72,
0x76, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x6f, 0x2c, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20,
0x4f, 0x73, 0x74, 0x72, 0x76, 0x6f, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x6a, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b, 0x2c, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x50, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2c, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x50, 0x61,
0x6c, 0x61, 0x75, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x50, 0x61, 0x70,
0x75, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x47, 0x76, 0x69, 0x6e,
0x65, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x50, 0x61, 0x72,
0x61, 0x67, 0x76, 0x61, 0x6a, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x50,
0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x2c, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x50,
0x65, 0x72, 0x75, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x50, 0x65, 0x72,
0x75, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x50, 0x65, 0x72, 0x75, 0x2c,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x69, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x69, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e,
0x69, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x46, 0x65, 0x6e, 0x69, 0x6b, 0x73, 0x20, 0x6f, 0x73,
0x74, 0x72, 0x76, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x65,
0x6e, 0x20, 0x50, 0x6a, 0x65, 0x72, 0x20, 0x69, 0x20, 0x4d, 0x69, 0x6b,
0x65, 0x6c, 0x6f, 0x6e, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x65,
0x6e, 0x20, 0x50, 0x6a, 0x65, 0x72, 0x20, 0x69, 0x20, 0x4d, 0x69, 0x6b,
0x65, 0x6c, 0x6f, 0x6e, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x50, 0x69, 0x74, 0x6b, 0x65, 0x72,
0x6e, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x50, 0x6f, 0x6e, 0x70, 0x65,
0x6a, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x50, 0x6a, 0x6f, 0x6e, 0x67,
0x6a, 0x61, 0x6e, 0x161, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x52, 0x65, 0x69, 0x6e, 0x69, 0x6f, 0x6e, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x52, 0x6f, 0x74, 0x65, 0x72, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2c, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x65, 0x6a, 0x161, 0x65, 0x6c, 0x69,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70,
0x75, 0x72, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x6f, 0x6c, 0x6f,
0x6d, 0x6f, 0x6e, 0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76,
0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4a, 0x75, 0x17e, 0x6e, 0x61,
0x20, 0x44, 0x17e, 0x6f, 0x72, 0x64, 0x17e, 0x69, 0x6a, 0x61, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x160, 0x6f, 0x76, 0x61, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x2c, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x54,
0x61, 0x64, 0x17e, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x2c, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x10c, 0x75,
0x75, 0x6b, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2c, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x54, 0x75,
0x76, 0x61, 0x6c, 0x75, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x55, 0x72,
0x75, 0x67, 0x76, 0x61, 0x6a, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x55,
0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x2c, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x55, 0x7a, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x56, 0x65, 0x6e, 0x65, 0x63, 0x75, 0x65, 0x6c,
0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2c, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x56,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x56,
0x65, 0x6a, 0x6b, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x6f, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x56, 0x61, 0x6c, 0x69, 0x73, 0x20, 0x69, 0x20,
0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76,
0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4a, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4a, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4a, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4a, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x2c, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0xa2e7,
0xa0c5, 0xa395, 0xa3e6, 0xa12e, 0xa209, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x6c, 0x2019, 0x41, 0x66, 0x67, 0x68, 0xe0, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0xc0, 0x66,
0x72, 0x69, 0x63, 0x61, 0x20, 0x43, 0x69, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0xc0, 0x66,
0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4c, 0x69, 0x76, 0x61,
0x6e, 0x74, 0x69, 0x55, 0x72, 0x61, 0x20, 0x4e, 0x75, 0x72, 0x6d, 0x61,
0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0xc0, 0x66, 0x72, 0x69,
0x63, 0x61, 0x20, 0x64, 0x69, 0x20, 0x53, 0x63, 0x69, 0x72, 0x6f, 0x63,
0x63, 0x75, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0xc0,
0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x75, 0x6e,
0x65, 0x6e, 0x74, 0x69, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61,
0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0xc0, 0x66, 0x72, 0x69,
0x63, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x75, 0x6e, 0x65, 0x6e, 0x74,
0x69, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20,
0x64, 0x69, 0x20, 0x6c, 0x2019, 0xc0, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20,
0x64, 0x69, 0x20, 0x50, 0x75, 0x6e, 0x65, 0x6e, 0x74, 0x69, 0x55, 0x72,
0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20,
0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x55,
0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69,
0x20, 0x6c, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x55, 0x72, 0x61,
0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x7a, 0x7a, 0x6f,
0x6e, 0x69, 0x61, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72,
0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x7a, 0x7a,
0x6f, 0x6e, 0x69, 0x61, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61,
0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x7a,
0x7a, 0x6f, 0x6e, 0x69, 0x61, 0x55, 0x72, 0x61, 0x20, 0x63, 0x69, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c,
0x61, 0x72, 0x69, 0x20, 0x63, 0x69, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x63,
0x69, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x55, 0x72, 0x61, 0x20, 0x6c,
0x69, 0x76, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x55, 0x72, 0x61, 0x20,
0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x6c, 0x69, 0x76, 0x61, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61,
0x6c, 0x69, 0x20, 0x6c, 0x69, 0x76, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x55, 0x72, 0x61, 0x20, 0x64, 0xee, 0x20, 0x6d, 0x75, 0x6e, 0x74, 0x61,
0x67, 0x6e, 0x69, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72,
0x69, 0x20, 0x64, 0xee, 0x20, 0x6d, 0x75, 0x6e, 0x74, 0x61, 0x67, 0x6e,
0x69, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20,
0x64, 0xee, 0x20, 0x6d, 0x75, 0x6e, 0x74, 0x61, 0x67, 0x6e, 0x69, 0x55,
0x72, 0x61, 0x20, 0x64, 0xfb, 0x20, 0x50, 0x61, 0x63, 0xec, 0x66, 0x69,
0x63, 0x75, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69,
0x20, 0x64, 0xfb, 0x20, 0x50, 0x61, 0x63, 0xec, 0x66, 0x69, 0x63, 0x75,
0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64,
0xfb, 0x20, 0x50, 0x61, 0x63, 0xec, 0x66, 0x69, 0x63, 0x75, 0x55, 0x72,
0x61, 0x20, 0x64, 0x69, 0x20, 0x41, 0x70, 0x69, 0x61, 0x55, 0x72, 0x61,
0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x41,
0x70, 0x69, 0x61, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c,
0x69, 0x20, 0x64, 0x69, 0x20, 0x41, 0x70, 0x69, 0x61, 0x55, 0x72, 0x61,
0x20, 0xc0, 0x72, 0x61, 0x62, 0x62, 0x61, 0x55, 0x72, 0x61, 0x20, 0x73,
0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0xc0, 0x72, 0x61, 0x62, 0x62, 0x61,
0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0xc0,
0x72, 0x61, 0x62, 0x62, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x6c, 0x2019, 0x41, 0x72, 0x67, 0x69, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x55,
0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69,
0x20, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x69, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64,
0x69, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x69, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x72,
0x67, 0x69, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x64, 0x69, 0x20, 0x70,
0x75, 0x6e, 0x65, 0x6e, 0x74, 0x69, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75,
0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x72,
0x67, 0x69, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x64, 0x69, 0x20, 0x70,
0x75, 0x6e, 0x65, 0x6e, 0x74, 0x69, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69,
0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x72,
0x67, 0x69, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x64, 0x69, 0x20, 0x70,
0x75, 0x6e, 0x65, 0x6e, 0x74, 0x69, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x6c, 0x2019, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x55, 0x72,
0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20,
0x6c, 0x2019, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x55, 0x72, 0x61,
0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c,
0x2019, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x55, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c, 0xe0, 0x6e, 0x74, 0x69,
0x63, 0x75, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69,
0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c, 0xe0, 0x6e, 0x74,
0x69, 0x63, 0x75, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c,
0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c, 0xe0, 0x6e,
0x74, 0x69, 0x63, 0x75, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c,
0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63,
0x69, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x55, 0x72, 0x61, 0x20, 0x73,
0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x69, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67,
0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x69, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x69,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x70, 0x75,
0x6e, 0x65, 0x6e, 0x74, 0x69, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c,
0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x69, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x70, 0x75, 0x6e, 0x65, 0x6e,
0x74, 0x69, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69,
0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x63, 0x69, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x20, 0x64, 0x69, 0x20, 0x70, 0x75, 0x6e, 0x65, 0x6e, 0x74, 0x69, 0x55,
0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x69, 0x76,
0x61, 0x6e, 0x74, 0x69, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61,
0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x69, 0x76,
0x61, 0x6e, 0x74, 0x69, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61,
0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x69, 0x76,
0x61, 0x6e, 0x74, 0x69, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c,
0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64,
0x69, 0x20, 0x70, 0x75, 0x6e, 0x65, 0x6e, 0x74, 0x69, 0x55, 0x72, 0x61,
0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c,
0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64,
0x69, 0x20, 0x70, 0x75, 0x6e, 0x65, 0x6e, 0x74, 0x69, 0x55, 0x72, 0x61,
0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c,
0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64,
0x69, 0x20, 0x70, 0x75, 0x6e, 0x65, 0x6e, 0x74, 0x69, 0x55, 0x72, 0x61,
0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61,
0x69, 0x6a, 0x61, 0x6e, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61,
0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69,
0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x55, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x7a, 0x7a, 0x6f, 0x72, 0x72, 0x69,
0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64,
0x69, 0x20, 0x6c, 0x2019, 0x41, 0x7a, 0x7a, 0x6f, 0x72, 0x72, 0x69, 0x55,
0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69,
0x20, 0x6c, 0x2019, 0x41, 0x7a, 0x7a, 0x6f, 0x72, 0x72, 0x69, 0x55, 0x72,
0x61, 0x20, 0x64, 0xfb, 0x20, 0x42, 0xe0, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x73, 0x68, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72,
0x69, 0x20, 0x64, 0xfb, 0x20, 0x42, 0xe0, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x73, 0x68, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c,
0x69, 0x20, 0x64, 0xfb, 0x20, 0x42, 0xe0, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x73, 0x68, 0x55, 0x72, 0x61, 0x20, 0x64, 0xfb, 0x20, 0x42, 0x68,
0x75, 0x74, 0x61, 0x6e, 0x55, 0x72, 0x61, 0x20, 0x64, 0xe2, 0x20, 0x42,
0x75, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x55, 0x72, 0x61,
0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x42,
0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x55, 0x72, 0x61, 0x20, 0x6c,
0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64, 0xfb, 0x20,
0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x43, 0x61, 0x70, 0x75, 0x20, 0x56, 0x69, 0x72, 0x64, 0x69, 0x55,
0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69,
0x20, 0x43, 0x61, 0x70, 0x75, 0x20, 0x56, 0x69, 0x72, 0x64, 0x69, 0x55,
0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69,
0x20, 0x43, 0x61, 0x70, 0x75, 0x20, 0x56, 0x69, 0x72, 0x64, 0x69, 0x55,
0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72,
0x72, 0x6f, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x43, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61,
0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20,
0x64, 0x69, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x55, 0x72,
0x61, 0x20, 0x64, 0xfb, 0x20, 0x43, 0x69, 0x6c, 0x69, 0x55, 0x72, 0x61,
0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0xfb, 0x20, 0x43,
0x69, 0x6c, 0x69, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c,
0x69, 0x20, 0x64, 0xfb, 0x20, 0x43, 0x69, 0x6c, 0x69, 0x55, 0x72, 0x61,
0x20, 0x64, 0xe2, 0x20, 0x43, 0x69, 0x6e, 0x61, 0x55, 0x72, 0x61, 0x20,
0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0xe2, 0x20, 0x43, 0x69,
0x6e, 0x61, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69,
0x20, 0x64, 0xe2, 0x20, 0x43, 0x69, 0x6e, 0x61, 0x55, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x6c, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x64,
0x69, 0x20, 0x4e, 0x61, 0x74, 0x61, 0x6c, 0x69, 0x55, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x6c, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x69, 0x20, 0x43,
0x6f, 0x63, 0x6f, 0x73, 0x55, 0x72, 0x61, 0x20, 0x64, 0xe2, 0x20, 0x43,
0x75, 0x6c, 0x75, 0x6d, 0x6d, 0x69, 0x61, 0x55, 0x72, 0x61, 0x20, 0x73,
0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0xe2, 0x20, 0x43, 0x75, 0x6c,
0x75, 0x6d, 0x6d, 0x69, 0x61, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67,
0x61, 0x6c, 0x69, 0x20, 0x64, 0xe2, 0x20, 0x43, 0x75, 0x6c, 0x75, 0x6d,
0x6d, 0x69, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019,
0xcc, 0x73, 0x75, 0x6c, 0x69, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x55, 0x72,
0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20,
0x6c, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x69, 0x20, 0x43, 0x6f, 0x6f, 0x6b,
0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64,
0x69, 0x20, 0x6c, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x69, 0x20, 0x43, 0x6f,
0x6f, 0x6b, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x43, 0x75, 0x62,
0x62, 0x61, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69,
0x20, 0x64, 0x69, 0x20, 0x43, 0x75, 0x62, 0x62, 0x61, 0x55, 0x72, 0x61,
0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x43,
0x75, 0x62, 0x62, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x44,
0x61, 0x76, 0x69, 0x73, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x44,
0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69,
0x6c, 0x6c, 0x65, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x54, 0x69,
0x6d, 0x6f, 0x72, 0x20, 0x64, 0x69, 0x20, 0x4c, 0x69, 0x76, 0x61, 0x6e,
0x74, 0x69, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0xcc,
0x73, 0x75, 0x6c, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x61, 0x73, 0x63,
0x61, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20,
0x64, 0x69, 0x20, 0x6c, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x64,
0x69, 0x20, 0x50, 0x61, 0x73, 0x63, 0x61, 0x55, 0x72, 0x61, 0x20, 0x6c,
0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0xcc,
0x73, 0x75, 0x6c, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x61, 0x73, 0x63,
0x61, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0xc8, 0x63,
0x75, 0x61, 0x64, 0x6f, 0x72, 0x55, 0x72, 0x61, 0x20, 0x43, 0x69, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x20, 0x45, 0x75, 0x72, 0x75, 0x70, 0x65,
0x61, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20,
0x43, 0x69, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x20, 0x45, 0x75, 0x72,
0x75, 0x70, 0x65, 0x61, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61,
0x6c, 0x69, 0x20, 0x43, 0x69, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x20,
0x45, 0x75, 0x72, 0x75, 0x70, 0x65, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x64,
0x69, 0x20, 0x4c, 0x69, 0x76, 0x61, 0x6e, 0x74, 0x69, 0x55, 0x72, 0x61,
0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c,
0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4c,
0x69, 0x76, 0x61, 0x6e, 0x74, 0x69, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69,
0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4c, 0x69, 0x76, 0x61,
0x6e, 0x74, 0x69, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x63, 0x63, 0x68, 0x69, 0xf9,
0x20, 0x61, 0x20, 0x4c, 0x69, 0x76, 0x61, 0x6e, 0x74, 0x69, 0x55, 0x72,
0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x75, 0x6e, 0x65, 0x6e, 0x74, 0x69,
0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64,
0x69, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x64,
0x69, 0x20, 0x50, 0x75, 0x6e, 0x65, 0x6e, 0x74, 0x69, 0x55, 0x72, 0x61,
0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c,
0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50,
0x75, 0x6e, 0x65, 0x6e, 0x74, 0x69, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x6c, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x69, 0x20, 0x46, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c,
0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0xcc, 0x73, 0x75,
0x6c, 0x69, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x55,
0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69,
0x20, 0x6c, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x69, 0x20, 0x46, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x55, 0x72, 0x61, 0x20, 0x64, 0xee, 0x20,
0x46, 0x69, 0x67, 0x69, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61,
0x72, 0x69, 0x20, 0x64, 0xee, 0x20, 0x46, 0x69, 0x67, 0x69, 0x55, 0x72,
0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0xee, 0x20,
0x46, 0x69, 0x67, 0x69, 0x55, 0x72, 0x61, 0x20, 0x64, 0xe2, 0x20, 0x47,
0x75, 0x69, 0x61, 0x6e, 0x61, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x69,
0x73, 0x69, 0x55, 0x72, 0x61, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x69,
0x73, 0x69, 0x20, 0x64, 0x69, 0x20, 0x53, 0x63, 0x69, 0x72, 0x6f, 0x63,
0x63, 0x75, 0x20, 0x65, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x6e,
0x74, 0xe0, 0x72, 0x74, 0x69, 0x64, 0x69, 0x55, 0x72, 0x61, 0x20, 0x64,
0xee, 0x20, 0x47, 0x61, 0x6c, 0xe0, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x55,
0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65,
0x72, 0x55, 0x72, 0x61, 0x20, 0x64, 0xe2, 0x20, 0x47, 0x69, 0x6f, 0x72,
0x67, 0x69, 0x61, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72,
0x69, 0x20, 0x64, 0xe2, 0x20, 0x47, 0x69, 0x6f, 0x72, 0x67, 0x69, 0x61,
0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64,
0xe2, 0x20, 0x47, 0x69, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x55, 0x72, 0x61,
0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x69, 0x20,
0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x55, 0x72, 0x61, 0x20, 0x4d,
0x69, 0x6e, 0x7a, 0x61, 0x6e, 0x61, 0x20, 0x64, 0x69, 0x20, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x55, 0x72, 0x61, 0x20, 0x64,
0xe2, 0x20, 0x47, 0x72, 0x75, 0x69, 0x6c, 0x6c, 0x61, 0x6e, 0x6e, 0x69,
0x61, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20,
0x64, 0xe2, 0x20, 0x47, 0x72, 0x75, 0x69, 0x6c, 0x6c, 0x61, 0x6e, 0x6e,
0x69, 0x61, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69,
0x20, 0x64, 0xe2, 0x20, 0x47, 0x72, 0x75, 0x69, 0x6c, 0x6c, 0x61, 0x6e,
0x6e, 0x69, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64, 0xe2, 0x20, 0x47, 0x72,
0x75, 0x69, 0x6c, 0x6c, 0x61, 0x6e, 0x6e, 0x69, 0x61, 0x20, 0x6c, 0x69,
0x76, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x55, 0x72, 0x61, 0x20, 0x73,
0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0xe2, 0x20, 0x47, 0x72, 0x75,
0x69, 0x6c, 0x6c, 0x61, 0x6e, 0x6e, 0x69, 0x61, 0x20, 0x6c, 0x69, 0x76,
0x61, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69,
0x76, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x64, 0xe2, 0x20, 0x47,
0x72, 0x75, 0x69, 0x6c, 0x6c, 0x61, 0x6e, 0x6e, 0x69, 0x61, 0x20, 0x6c,
0x69, 0x76, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x55, 0x72, 0x61, 0x20,
0x64, 0xe2, 0x20, 0x47, 0x72, 0x75, 0x69, 0x6c, 0x6c, 0x61, 0x6e, 0x6e,
0x69, 0x61, 0x20, 0x70, 0x75, 0x6e, 0x69, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64,
0xe2, 0x20, 0x47, 0x72, 0x75, 0x69, 0x6c, 0x6c, 0x61, 0x6e, 0x6e, 0x69,
0x61, 0x20, 0x70, 0x75, 0x6e, 0x69, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x55,
0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0xe2,
0x20, 0x47, 0x72, 0x75, 0x69, 0x6c, 0x6c, 0x61, 0x6e, 0x6e, 0x69, 0x61,
0x20, 0x70, 0x75, 0x6e, 0x69, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x55, 0x72,
0x61, 0x20, 0x4e, 0x75, 0x72, 0x6d, 0x61, 0x6c, 0x69, 0x20, 0x64, 0xfb,
0x20, 0x47, 0x75, 0x72, 0x66, 0x75, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x6c, 0x2019, 0x48, 0x61, 0x77, 0x61, 0x61, 0x69, 0x55, 0x72, 0x61,
0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c,
0x2019, 0x48, 0x61, 0x77, 0x61, 0x61, 0x69, 0x55, 0x72, 0x61, 0x20, 0x6c,
0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x48,
0x61, 0x77, 0x61, 0x61, 0x69, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x55, 0x72, 0x61,
0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x48,
0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x55, 0x72, 0x61, 0x20,
0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x48, 0x6f,
0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x55, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75,
0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x48, 0x6f, 0x76, 0x64,
0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64,
0x69, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75,
0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0xcc, 0x6e,
0x6e, 0x69, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019,
0x55, 0x63, 0x69, 0x61, 0x6e, 0x75, 0x20, 0x49, 0x6e, 0x6e, 0x69, 0x61,
0x6e, 0x75, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x49,
0x6e, 0x6e, 0x75, 0x63, 0x69, 0x6e, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x6c, 0x2019, 0x49, 0x6e, 0x6e, 0x75, 0x6e, 0x65, 0x73, 0x69,
0x61, 0x20, 0x63, 0x69, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x55, 0x72,
0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x49, 0x6e, 0x6e, 0x75, 0x6e,
0x65, 0x73, 0x69, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x69, 0x76, 0x61,
0x6e, 0x74, 0x69, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019,
0x49, 0x6e, 0x6e, 0x75, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x64, 0x69,
0x20, 0x70, 0x75, 0x6e, 0x65, 0x6e, 0x74, 0x69, 0x55, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x6c, 0x2019, 0x49, 0x72, 0x61, 0x6e, 0x55, 0x72, 0x61,
0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c,
0x2019, 0x49, 0x72, 0x61, 0x6e, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67,
0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x49, 0x72, 0x61,
0x6e, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x49, 0x72, 0x74, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61,
0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x49, 0x72, 0x74, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69,
0x20, 0x64, 0x69, 0x20, 0x49, 0x72, 0x74, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x49, 0x73, 0x64, 0x72, 0x61,
0x65, 0x6c, 0x69, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72,
0x69, 0x20, 0x64, 0x69, 0x20, 0x49, 0x73, 0x64, 0x72, 0x61, 0x65, 0x6c,
0x69, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20,
0x64, 0x69, 0x20, 0x49, 0x73, 0x64, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x55,
0x72, 0x61, 0x20, 0x64, 0xfb, 0x20, 0x47, 0x69, 0x61, 0x70, 0x70, 0x75,
0x6e, 0x69, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69,
0x20, 0x64, 0xfb, 0x20, 0x47, 0x69, 0x61, 0x70, 0x70, 0x75, 0x6e, 0x69,
0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64,
0xfb, 0x20, 0x47, 0x69, 0x61, 0x70, 0x70, 0x75, 0x6e, 0x69, 0x55, 0x72,
0x61, 0x20, 0x64, 0xfb, 0x20, 0x4b, 0x61, 0x7a, 0x7a, 0xe0, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x55, 0x72, 0x61, 0x20, 0x64, 0xfb, 0x20, 0x4b,
0x61, 0x7a, 0x7a, 0xe0, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x64,
0x69, 0x20, 0x4c, 0x69, 0x76, 0x61, 0x6e, 0x74, 0x69, 0x55, 0x72, 0x61,
0x20, 0x64, 0xfb, 0x20, 0x4b, 0x61, 0x7a, 0x7a, 0xe0, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x50, 0x75, 0x6e, 0x65, 0x6e,
0x74, 0x69, 0x55, 0x72, 0x61, 0x20, 0x64, 0xe2, 0x20, 0x43, 0x75, 0x72,
0xec, 0x61, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69,
0x20, 0x64, 0xe2, 0x20, 0x43, 0x75, 0x72, 0xec, 0x61, 0x55, 0x72, 0x61,
0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0xe2, 0x20, 0x43,
0x75, 0x72, 0xec, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4b,
0x6f, 0x73, 0x72, 0x61, 0x65, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x55,
0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69,
0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b,
0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64,
0x69, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73,
0x6b, 0x55, 0x72, 0x61, 0x20, 0x64, 0xfb, 0x20, 0x4b, 0x69, 0x72, 0x67,
0x68, 0xec, 0x7a, 0x7a, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x55, 0x72, 0x61,
0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x69, 0x20,
0x4c, 0x69, 0x6e, 0x65, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x55, 0x72, 0x61, 0x20,
0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x55, 0x72, 0x61, 0x20, 0x6c,
0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x4d, 0x61, 0x67, 0x64, 0x61, 0x6e, 0x55, 0x72, 0x61, 0x20, 0x73,
0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x61, 0x67,
0x64, 0x61, 0x6e, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c,
0x69, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x61, 0x67, 0x64, 0x61, 0x6e, 0x55,
0x72, 0x61, 0x20, 0x64, 0xe2, 0x20, 0x4d, 0x61, 0x6c, 0x69, 0x73, 0x69,
0x61, 0x55, 0x72, 0x61, 0x20, 0x64, 0xee, 0x20, 0x4d, 0x61, 0x72, 0x64,
0x69, 0x76, 0x69, 0x55, 0x72, 0x61, 0x20, 0x64, 0xee, 0x20, 0x4d, 0x61,
0x72, 0x63, 0x68, 0x65, 0x73, 0x69, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x6c, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x69, 0x20, 0x4d, 0x61, 0x72,
0x73, 0x68, 0x61, 0x6c, 0x6c, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x55, 0x72, 0x61,
0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x55, 0x72, 0x61, 0x20,
0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x55, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x55, 0x72, 0x61, 0x20,
0x64, 0xfb, 0x20, 0x4d, 0xe8, 0x73, 0x73, 0x69, 0x63, 0x75, 0x20, 0x50,
0x61, 0x63, 0xec, 0x66, 0x69, 0x63, 0x75, 0x55, 0x72, 0x61, 0x20, 0x73,
0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0xfb, 0x20, 0x4d, 0xe8, 0x73,
0x73, 0x69, 0x63, 0x75, 0x20, 0x50, 0x61, 0x63, 0xec, 0x66, 0x69, 0x63,
0x75, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20,
0x64, 0xfb, 0x20, 0x4d, 0xe8, 0x73, 0x73, 0x69, 0x63, 0x75, 0x20, 0x50,
0x61, 0x63, 0xec, 0x66, 0x69, 0x63, 0x75, 0x55, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61,
0x72, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20,
0x64, 0x69, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74,
0x61, 0x72, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69,
0x20, 0x64, 0x69, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61,
0x74, 0x61, 0x72, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x6f,
0x73, 0x63, 0x61, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72,
0x69, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x61, 0x55, 0x72,
0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20,
0x4d, 0x6f, 0x73, 0x63, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x55, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x55, 0x72, 0x61, 0x20, 0x64,
0xfb, 0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x55, 0x72, 0x61, 0x20, 0x64,
0xe2, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x69, 0x64,
0x6f, 0x6e, 0x69, 0x61, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61,
0x72, 0x69, 0x20, 0x64, 0xe2, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x43,
0x61, 0x6c, 0x69, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x55, 0x72, 0x61, 0x20,
0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0xe2, 0x20, 0x4e, 0x6f,
0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x69, 0x64, 0x6f, 0x6e, 0x69, 0x61,
0x55, 0x72, 0x61, 0x20, 0x64, 0xe2, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20,
0x5a, 0x69, 0x6c, 0x61, 0x6e, 0x6e, 0x69, 0x61, 0x55, 0x72, 0x61, 0x20,
0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0xe2, 0x20, 0x4e, 0x6f,
0x76, 0x61, 0x20, 0x5a, 0x69, 0x6c, 0x61, 0x6e, 0x6e, 0x69, 0x61, 0x55,
0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0xe2,
0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x5a, 0x69, 0x6c, 0x61, 0x6e, 0x6e,
0x69, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x54, 0x69, 0x72,
0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75,
0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x54, 0x69, 0x72, 0x72,
0x61, 0x6e, 0x6f, 0x76, 0x61, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67,
0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x54, 0x69, 0x72, 0x72, 0x61,
0x6e, 0x6f, 0x76, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4e,
0x69, 0x75, 0x65, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019,
0xcc, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20,
0x64, 0x69, 0x20, 0x6c, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69,
0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0xcc, 0x73,
0x75, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x55,
0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20,
0x64, 0x69, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x69, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x55, 0x72,
0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x69, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20,
0x64, 0x69, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69,
0x20, 0x64, 0x69, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4f, 0x6d,
0x73, 0x6b, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69,
0x20, 0x64, 0x69, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x55, 0x72, 0x61, 0x20,
0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x4f, 0x6d,
0x73, 0x6b, 0x55, 0x72, 0x61, 0x20, 0x64, 0xfb, 0x20, 0x50, 0xe0, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c,
0x61, 0x72, 0x69, 0x20, 0x64, 0xfb, 0x20, 0x50, 0xe0, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c,
0x69, 0x20, 0x64, 0xfb, 0x20, 0x50, 0xe0, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x61, 0x6c, 0x61,
0x75, 0x55, 0x72, 0x61, 0x20, 0x64, 0xe2, 0x20, 0x50, 0x61, 0x70, 0x75,
0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x47, 0x75, 0x69, 0x6e, 0xec,
0x61, 0x55, 0x72, 0x61, 0x20, 0x64, 0xfb, 0x20, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x79, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61,
0x72, 0x69, 0x20, 0x64, 0xfb, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69,
0x20, 0x64, 0xfb, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79,
0x55, 0x72, 0x61, 0x20, 0x64, 0xfb, 0x20, 0x50, 0x69, 0x72, 0xf9, 0x55,
0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0xfb,
0x20, 0x50, 0x69, 0x72, 0xf9, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67,
0x61, 0x6c, 0x69, 0x20, 0x64, 0xfb, 0x20, 0x50, 0x69, 0x72, 0xf9, 0x55,
0x72, 0x61, 0x20, 0x64, 0xee, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70,
0x69, 0x6e, 0x69, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72,
0x69, 0x20, 0x64, 0xee, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69,
0x6e, 0x69, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69,
0x20, 0x64, 0xee, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e,
0x69, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0xcc, 0x73,
0x75, 0x6c, 0x69, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x55,
0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x53, 0x2e, 0x20, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69,
0x20, 0x64, 0x69, 0x20, 0x53, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64,
0x69, 0x20, 0x53, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20,
0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x55, 0x72,
0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72,
0x6e, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x6f, 0x6e, 0x61,
0x70, 0x65, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x79, 0x6f,
0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x55, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x55, 0x72,
0x61, 0x20, 0x64, 0x69, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20,
0x64, 0x69, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x55,
0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69,
0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x55, 0x72, 0x61,
0x20, 0x64, 0xee, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x55, 0x72, 0x61,
0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0xee, 0x20, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61,
0x6c, 0x69, 0x20, 0x64, 0xee, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x55,
0x72, 0x61, 0x20, 0x64, 0xee, 0x20, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65,
0x6c, 0x6c, 0x65, 0x73, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x53,
0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x69, 0x55, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x6c, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x69, 0x20, 0x53,
0x61, 0x6c, 0x75, 0x6d, 0x75, 0x6e, 0x69, 0x55, 0x72, 0x61, 0x20, 0x64,
0xe2, 0x20, 0x47, 0x69, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64, 0x69,
0x20, 0x73, 0x63, 0x69, 0x72, 0x6f, 0x63, 0x63, 0x75, 0x55, 0x72, 0x61,
0x20, 0x64, 0xfb, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x69,
0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61,
0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74,
0x69, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69,
0x20, 0x64, 0x69, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x55, 0x72,
0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20,
0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x55, 0x72, 0x61, 0x20, 0x64, 0xfb,
0x20, 0x54, 0x61, 0x67, 0x67, 0xec, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x54, 0x75, 0x6b, 0x69, 0x6c,
0x61, 0x75, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69,
0x20, 0x64, 0x69, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x55, 0x72, 0x61,
0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x43,
0x68, 0x75, 0x75, 0x6b, 0x55, 0x72, 0x61, 0x20, 0x64, 0xfb, 0x20, 0x54,
0x75, 0x72, 0x6b, 0x6d, 0xe8, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x55,
0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0xfb,
0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0xe8, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20,
0x64, 0xfb, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0xe8, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x54, 0x75,
0x76, 0x61, 0x6c, 0x75, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c,
0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x55, 0x72, 0x61, 0x20,
0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019,
0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x55, 0x72, 0x61, 0x20, 0x6c,
0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x6c, 0x2019, 0x55, 0x7a, 0x62, 0xe8, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20,
0x64, 0x69, 0x20, 0x6c, 0x2019, 0x55, 0x7a, 0x62, 0xe8, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c,
0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x55, 0x7a, 0x62, 0xe8, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x55, 0x72, 0x61, 0x20, 0x73,
0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61,
0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x55, 0x72, 0x61, 0x20, 0x64, 0xfb, 0x20, 0x56, 0x69, 0x6e, 0x69,
0x7a, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x56, 0x6c, 0x61, 0x64, 0xec, 0x76, 0x75, 0x73, 0x74, 0x6f, 0x6b,
0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64,
0x69, 0x20, 0x56, 0x6c, 0x61, 0x64, 0xec, 0x76, 0x75, 0x73, 0x74, 0x6f,
0x6b, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20,
0x64, 0x69, 0x20, 0x56, 0x6c, 0x61, 0x64, 0xec, 0x76, 0x75, 0x73, 0x74,
0x6f, 0x6b, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0xf2, 0x6c,
0x67, 0x75, 0x67, 0x72, 0x61, 0x64, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75,
0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x56, 0xf2, 0x6c, 0x67,
0x75, 0x67, 0x72, 0x61, 0x64, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67,
0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x56, 0xf2, 0x6c, 0x67, 0x75,
0x67, 0x72, 0x61, 0x64, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x6c, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x57, 0x61, 0x6b, 0x65,
0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69,
0x73, 0x20, 0x65, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x55, 0x72,
0x61, 0x20, 0x64, 0x69, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64,
0x69, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x55, 0x72, 0x61,
0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x59,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x69, 0x72, 0x69, 0x6d, 0x6d, 0x75,
0x72, 0x67, 0x75, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72,
0x69, 0x20, 0x64, 0x69, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x69, 0x72,
0x69, 0x6d, 0x6d, 0x75, 0x72, 0x67, 0x75, 0x55, 0x72, 0x61, 0x20, 0x6c,
0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x59, 0x65, 0x6b,
0x61, 0x74, 0x69, 0x72, 0x69, 0x6d, 0x6d, 0x75, 0x72, 0x67, 0x75, 0x55,
0x72, 0x61, 0x20, 0x64, 0xfb, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x70,
0x6f, 0x73, 0x74, 0x72, 0x7a, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x61,
0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x7a,
0x61, 0x73, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61,
0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x7a,
0x61, 0x73, 0x70, 0x6f, 0x142, 0x65, 0x64, 0x6e, 0x69, 0x6f, 0x77, 0x6f,
0x61, 0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f,
0x61, 0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f,
0x61, 0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20,
0x63, 0x7a, 0x61, 0x73, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69,
0x6f, 0x61, 0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x61, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x144, 0x73,
0x6b, 0x69, 0x61, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x61, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x144, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x74, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x142, 0x6d,
0x61, 0x63, 0x6b, 0x69, 0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x142,
0x6d, 0x61, 0x63, 0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79,
0x61, 0x6d, 0x61, 0x7a, 0x14d, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x7a,
0x61, 0x73, 0x61, 0x6d, 0x61, 0x7a, 0x14d, 0x144, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x61, 0x6d, 0x61, 0x7a, 0x14d, 0x144, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a,
0x61, 0x73, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x70, 0x6f, 0x73, 0x74, 0x72,
0x7a, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x79,
0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x70,
0x6f, 0x73, 0x74, 0x72, 0x7a, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x61,
0x6d, 0x65, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x70, 0x6f, 0x73, 0x74, 0x72, 0x7a, 0x6f, 0x64,
0x6b, 0x6f, 0x77, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x79, 0x6b, 0x61, 0x144,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f,
0x61, 0x6d, 0x65, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e,
0x69, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x67, 0x14d, 0x72, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x67, 0x14d, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x67, 0x14d, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77,
0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x70, 0x61, 0x63, 0x79, 0x66, 0x69,
0x63, 0x7a, 0x6e, 0x79, 0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x70, 0x61,
0x63, 0x79, 0x66, 0x69, 0x63, 0x7a, 0x6e, 0x79, 0x20, 0x6c, 0x61, 0x74,
0x6f, 0x77, 0x79, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x41, 0x6e, 0x61,
0x64, 0x79, 0x72, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x61, 0x75, 0x6b, 0x74, 0x61, 0x75, 0x63, 0x6b, 0x69, 0x20,
0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x75, 0x6b, 0x74, 0x61, 0x75, 0x63,
0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x61, 0x6b, 0x74, 0x69, 0x75, 0x62, 0x69, 0x144, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x6b, 0x74, 0x69, 0x75,
0x62, 0x69, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77,
0x79, 0x50, 0x14d, 0x142, 0x77, 0x79, 0x73, 0x65, 0x70, 0x20, 0x41, 0x72,
0x61, 0x62, 0x73, 0x6b, 0x69, 0x50, 0x14d, 0x142, 0x77, 0x79, 0x73, 0x65,
0x70, 0x20, 0x41, 0x72, 0x61, 0x62, 0x73, 0x6b, 0x69, 0x20, 0x28, 0x73,
0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20,
0x63, 0x7a, 0x61, 0x73, 0x29, 0x50, 0x14d, 0x142, 0x77, 0x79, 0x73, 0x65,
0x70, 0x20, 0x41, 0x72, 0x61, 0x62, 0x73, 0x6b, 0x69, 0x20, 0x28, 0x6c,
0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x41,
0x72, 0x67, 0x79, 0x6e, 0x74, 0x79, 0x6e, 0x61, 0x41, 0x72, 0x67, 0x79,
0x6e, 0x74, 0x79, 0x6e, 0x61, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x29, 0x41, 0x72, 0x67, 0x79, 0x6e, 0x74, 0x79, 0x6e, 0x61, 0x20, 0x28,
0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29,
0x41, 0x72, 0x67, 0x79, 0x6e, 0x74, 0x79, 0x6e, 0x61, 0x20, 0x5a, 0x61,
0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x41, 0x72, 0x67, 0x79, 0x6e,
0x74, 0x79, 0x6e, 0x61, 0x20, 0x5a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e,
0x69, 0x6f, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x41, 0x72,
0x67, 0x79, 0x6e, 0x74, 0x79, 0x6e, 0x61, 0x20, 0x5a, 0x61, 0x63, 0x68,
0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77,
0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x41, 0x72, 0x6d, 0x79, 0x6e,
0x69, 0x6a, 0x6f, 0x41, 0x72, 0x6d, 0x79, 0x6e, 0x69, 0x6a, 0x6f, 0x20,
0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x41, 0x72, 0x6d, 0x79, 0x6e,
0x69, 0x6a, 0x6f, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20,
0x63, 0x7a, 0x61, 0x73, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x79, 0x63, 0x6b, 0x69, 0x20, 0x73, 0x7a, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x79, 0x63, 0x6b, 0x69,
0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x70, 0x6f, 0x73, 0x74, 0x72,
0x7a, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x70, 0x6f, 0x73, 0x74, 0x72, 0x7a, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f,
0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x70, 0x6f, 0x73, 0x74, 0x72, 0x7a,
0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77,
0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x70, 0x6f, 0x73, 0x74, 0x72, 0x7a,
0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x2d, 0x7a, 0x61, 0x63, 0x68, 0x6f,
0x64, 0x6e, 0x69, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x70, 0x6f, 0x73,
0x74, 0x72, 0x7a, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x2d, 0x7a, 0x61,
0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x7a, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x70, 0x6f, 0x73, 0x74, 0x72, 0x7a, 0x6f, 0x64, 0x6b, 0x6f, 0x77,
0x6f, 0x2d, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x77,
0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20,
0x63, 0x7a, 0x61, 0x73, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69,
0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65, 0x6a, 0x64, 0x17c, 0x61, 0x6e,
0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x65, 0x6a, 0x64, 0x17c, 0x61, 0x6e, 0x20, 0x28, 0x6c, 0x61, 0x74,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x41, 0x7a, 0x6f,
0x72, 0x79, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x41, 0x7a,
0x6f, 0x72, 0x79, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20,
0x63, 0x7a, 0x61, 0x73, 0x29, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x73, 0x7a, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x7a, 0x20, 0x28, 0x6c,
0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x42,
0x6f, 0x6c, 0x69, 0x77, 0x69, 0x6a, 0x6f, 0x42, 0x72, 0x61, 0x73, 0xed,
0x6c, 0x69, 0x61, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x42,
0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x20, 0x28, 0x6c, 0x61, 0x74,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x57, 0x79, 0x73,
0x70, 0x79, 0x20, 0x5a, 0x69, 0x65, 0x6c, 0x14d, 0x6e, 0x65, 0x67, 0x6f,
0x20, 0x50, 0x72, 0x7a, 0x69, 0x6c, 0x14d, 0x6e, 0x64, 0x6b, 0x61, 0x57,
0x79, 0x73, 0x70, 0x79, 0x20, 0x5a, 0x69, 0x65, 0x6c, 0x14d, 0x6e, 0x65,
0x67, 0x6f, 0x20, 0x50, 0x72, 0x7a, 0x69, 0x6c, 0x14d, 0x6e, 0x64, 0x6b,
0x61, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x57, 0x79, 0x73,
0x70, 0x79, 0x20, 0x5a, 0x69, 0x65, 0x6c, 0x14d, 0x6e, 0x65, 0x67, 0x6f,
0x20, 0x50, 0x72, 0x7a, 0x69, 0x6c, 0x14d, 0x6e, 0x64, 0x6b, 0x61, 0x20,
0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x29, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x29, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x20, 0x28, 0x6c, 0x61, 0x74,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x43, 0x68, 0x69,
0x6e, 0x79, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x43, 0x68,
0x69, 0x6e, 0x79, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20,
0x63, 0x7a, 0x61, 0x73, 0x29, 0x47, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x20,
0x57, 0x79, 0x73, 0x70, 0x61, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69,
0x6a, 0x6f, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4b, 0x6f,
0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x6f, 0x20, 0x28, 0x6c, 0x61, 0x74,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x57, 0x79, 0x73,
0x70, 0x79, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x61, 0x20, 0x28, 0x73, 0x7a,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x29, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0x43, 0x6f,
0x6f, 0x6b, 0x61, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20,
0x63, 0x7a, 0x61, 0x73, 0x29, 0x4b, 0x75, 0x62, 0x61, 0x20, 0x28, 0x73,
0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20,
0x63, 0x7a, 0x61, 0x73, 0x29, 0x4b, 0x75, 0x62, 0x61, 0x20, 0x28, 0x6c,
0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x57,
0x79, 0x73, 0x70, 0x61, 0x20, 0x57, 0x69, 0x65, 0x6c, 0x6b, 0x61, 0x6e,
0x6f, 0x63, 0x6e, 0x6f, 0x57, 0x79, 0x73, 0x70, 0x61, 0x20, 0x57, 0x69,
0x65, 0x6c, 0x6b, 0x61, 0x6e, 0x6f, 0x63, 0x6e, 0x6f, 0x20, 0x28, 0x73,
0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20,
0x63, 0x7a, 0x61, 0x73, 0x29, 0x57, 0x79, 0x73, 0x70, 0x61, 0x20, 0x57,
0x69, 0x65, 0x6c, 0x6b, 0x61, 0x6e, 0x6f, 0x63, 0x6e, 0x6f, 0x20, 0x28,
0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29,
0x70, 0x6f, 0x73, 0x74, 0x72, 0x7a, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f,
0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x70, 0x6f, 0x73, 0x74, 0x72, 0x7a, 0x6f, 0x64, 0x6b,
0x6f, 0x77, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x70, 0x6f, 0x73, 0x74, 0x72,
0x7a, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e,
0x69, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e,
0x69, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64,
0x6e, 0x69, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x65, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x64, 0x61, 0x6c,
0x73, 0x7a, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x7a, 0x61, 0x63, 0x68,
0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x7a, 0x61, 0x63, 0x68,
0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x7a, 0x61, 0x63,
0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20,
0x63, 0x7a, 0x61, 0x73, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x79, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x46, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x79, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f,
0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x46, 0x69, 0x64, 0x17c,
0x69, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x46, 0x69, 0x64,
0x17c, 0x69, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x29, 0x47, 0x75, 0x6a, 0x61, 0x6e, 0x61, 0x20, 0x46,
0x72, 0x61, 0x6e, 0x63, 0x75, 0x73, 0x6b, 0x6f, 0x46, 0x72, 0x61, 0x6e,
0x63, 0x75, 0x73, 0x6b, 0x65, 0x20, 0x54, 0x65, 0x72, 0x79, 0x74, 0x6f,
0x72, 0x69, 0x61, 0x20, 0x50, 0x6f, 0x142, 0x65, 0x64, 0x6e, 0x69, 0x6f,
0x77, 0x65, 0x20, 0x69, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74,
0x79, 0x63, 0x7a, 0x6e, 0x65, 0x47, 0x72, 0x75, 0x7a, 0x79, 0x6a, 0x6f,
0x47, 0x72, 0x75, 0x7a, 0x79, 0x6a, 0x6f, 0x20, 0x28, 0x73, 0x7a, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a,
0x61, 0x73, 0x29, 0x47, 0x72, 0x75, 0x7a, 0x79, 0x6a, 0x6f, 0x20, 0x28,
0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29,
0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x6f, 0x77, 0x65, 0x20, 0x57,
0x79, 0x73, 0x70, 0x79, 0x75, 0x6e, 0x69, 0x77, 0x65, 0x72, 0x73, 0x61,
0x6c, 0x6e, 0x65, 0x67, 0x6f, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x75, 0x47,
0x72, 0x79, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x79, 0x6a, 0x6f, 0x20, 0x57,
0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x61, 0x47, 0x72, 0x79, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x79, 0x6a, 0x6f, 0x20, 0x57, 0x73, 0x63, 0x68,
0x6f, 0x64, 0x6e, 0x69, 0x61, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x29, 0x47, 0x72, 0x79, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x79, 0x6a, 0x6f,
0x20, 0x57, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x61, 0x20, 0x28,
0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29,
0x47, 0x72, 0x79, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x79, 0x6a, 0x6f, 0x20,
0x5a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x47, 0x72, 0x79,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x79, 0x6a, 0x6f, 0x20, 0x5a, 0x61, 0x63,
0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x29, 0x47, 0x72, 0x79, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x79, 0x6a,
0x6f, 0x20, 0x5a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x20,
0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x29, 0x48, 0x61, 0x77, 0x61, 0x6a, 0x65, 0x2d, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x79, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x48, 0x61,
0x77, 0x61, 0x6a, 0x65, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x79, 0x20,
0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x29, 0x48, 0x14d, 0x6e, 0x67, 0x6b, 0x14d, 0x6e, 0x67, 0x48, 0x14d, 0x6e,
0x67, 0x6b, 0x14d, 0x6e, 0x67, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x29, 0x48, 0x14d, 0x6e, 0x67, 0x6b, 0x14d, 0x6e, 0x67, 0x20, 0x28, 0x6c,
0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4b,
0x6f, 0x62, 0x64, 0x6f, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29,
0x4b, 0x6f, 0x62, 0x64, 0x6f, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77,
0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x69, 0x6e, 0x64, 0x79, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0xd4, 0x63, 0x65,
0x61, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x79, 0x6a, 0x73, 0x6b, 0x69, 0x69,
0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x49, 0x6e, 0x64, 0x14d, 0x6e, 0x65, 0x7a, 0x79, 0x6a,
0x6f, 0x20, 0x50, 0x6f, 0x73, 0x74, 0x72, 0x7a, 0x6f, 0x64, 0x6b, 0x6f,
0x77, 0x6f, 0x49, 0x6e, 0x64, 0x14d, 0x6e, 0x65, 0x7a, 0x79, 0x6a, 0x6f,
0x20, 0x57, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x49, 0x6e,
0x64, 0x14d, 0x6e, 0x65, 0x7a, 0x79, 0x6a, 0x6f, 0x20, 0x5a, 0x61, 0x63,
0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x49, 0x72, 0x6b, 0x75, 0x63, 0x6b,
0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x49, 0x72, 0x6b, 0x75,
0x63, 0x6b, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x29, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x20, 0x28,
0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c,
0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x29, 0x4a, 0x61, 0x70, 0x14d, 0x6e, 0x69, 0x6a, 0x6f, 0x4a, 0x61,
0x70, 0x14d, 0x6e, 0x69, 0x6a, 0x6f, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x29, 0x4a, 0x61, 0x70, 0x14d, 0x6e, 0x69, 0x6a, 0x6f, 0x20, 0x28,
0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29,
0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x50, 0x69, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x77, 0x142, 0x6f, 0x77, 0x73, 0x6b, 0x20, 0x4b, 0x61, 0x6d,
0x63, 0x7a, 0x61, 0x63, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x50,
0x69, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x77, 0x142, 0x6f, 0x77, 0x73,
0x6b, 0x20, 0x4b, 0x61, 0x6d, 0x63, 0x7a, 0x61, 0x63, 0x6b, 0x69, 0x20,
0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x6a, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x29, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b,
0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x29, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20,
0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4d, 0x61, 0x6c, 0x65, 0x7a, 0x79,
0x6a, 0x6f, 0x4d, 0x65, 0x6b, 0x73, 0x79, 0x6b, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x70, 0x61, 0x63, 0x79, 0x66, 0x69, 0x63, 0x7a, 0x6e,
0x79, 0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x29, 0x4d, 0x65, 0x6b, 0x73, 0x79, 0x6b, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x70, 0x61, 0x63, 0x79, 0x66, 0x69, 0x63, 0x7a,
0x6e, 0x79, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x29, 0x55, 0x142,
0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x28, 0x73, 0x7a,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x29, 0x55, 0x142, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74,
0x6f, 0x72, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x29, 0x4d, 0x6f, 0x73, 0x6b, 0x77, 0x61, 0x20, 0x28,
0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x29, 0x4d, 0x6f, 0x73, 0x6b, 0x77, 0x61, 0x20, 0x28, 0x6c, 0x61, 0x74,
0x6f, 0x77, 0x79, 0x29, 0x4e, 0x6f, 0x77, 0x6f, 0x20, 0x4b, 0x61, 0x6c,
0x65, 0x64, 0x14d, 0x6e, 0x69, 0x6a, 0x6f, 0x4e, 0x6f, 0x77, 0x6f, 0x20,
0x4b, 0x61, 0x6c, 0x65, 0x64, 0x14d, 0x6e, 0x69, 0x6a, 0x6f, 0x20, 0x28,
0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4e, 0x6f, 0x77, 0x6f, 0x20, 0x4b,
0x61, 0x6c, 0x65, 0x64, 0x14d, 0x6e, 0x69, 0x6a, 0x6f, 0x20, 0x28, 0x6c,
0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4e,
0x6f, 0x77, 0x6f, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x79, 0x6a,
0x6f, 0x4e, 0x6f, 0x77, 0x6f, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64,
0x79, 0x6a, 0x6f, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4e,
0x6f, 0x77, 0x6f, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x79, 0x6a,
0x6f, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a,
0x61, 0x73, 0x29, 0x4e, 0x6f, 0x77, 0x6f, 0x20, 0x46, 0x75, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0x79, 0x6a, 0x6f, 0x4e, 0x6f, 0x77, 0x6f, 0x20,
0x46, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x79, 0x6a, 0x6f, 0x20,
0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4e, 0x6f, 0x77, 0x6f, 0x20,
0x46, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x79, 0x6a, 0x6f, 0x20,
0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x29, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x28, 0x73, 0x7a,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x29, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20,
0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x29, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x79, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4e, 0x6f, 0x77, 0x6f,
0x73, 0x79, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x28, 0x6c, 0x61, 0x74,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0xd4, 0x6d, 0x73,
0x6b, 0xd4, 0x6d, 0x73, 0x6b, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x29, 0xd4, 0x6d, 0x73, 0x6b, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77,
0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x50, 0x61, 0x70, 0x75, 0x61,
0x2d, 0x4e, 0x6f, 0x77, 0x6f, 0x20, 0x47, 0x77, 0x69, 0x6e, 0x65, 0x61,
0x50, 0x61, 0x72, 0x61, 0x67, 0x77, 0x61, 0x6a, 0x20, 0x28, 0x73, 0x7a,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x29, 0x50, 0x61, 0x72, 0x61, 0x67, 0x77, 0x61, 0x6a,
0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x29, 0x50, 0x65, 0x72, 0x75, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x29, 0x50, 0x65, 0x72, 0x75, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f,
0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x79, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29,
0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x79, 0x20, 0x28, 0x6c, 0x61,
0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x46, 0x79,
0x6e, 0x69, 0x6b, 0x73, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x20, 0x69, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20,
0x69, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x28,
0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29,
0x50, 0x6a, 0x14d, 0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x67, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x6b, 0x79, 0x7a, 0x79, 0x142, 0x6f, 0x72, 0x64, 0x7a, 0x6b,
0x69, 0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6b, 0x79, 0x7a, 0x79, 0x142,
0x6f, 0x72, 0x64, 0x7a, 0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77,
0x79, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x28, 0x73,
0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20,
0x63, 0x7a, 0x61, 0x73, 0x29, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a,
0x61, 0x73, 0x29, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x53, 0x61, 0x6d,
0x61, 0x72, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x61, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x57, 0x79, 0x73,
0x70, 0x79, 0x20, 0x53, 0x61, 0x6c, 0x14d, 0x6d, 0x14d, 0x6e, 0x61, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x50, 0x6f, 0x142, 0x65, 0x64,
0x6e, 0x69, 0x6f, 0x77, 0x6f, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x20,
0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x54, 0x61, 0x6a, 0x70, 0x65,
0x6a, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a,
0x61, 0x73, 0x29, 0x54, 0x14d, 0x6e, 0x67, 0x61, 0x54, 0x14d, 0x6e, 0x67,
0x61, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x54, 0x14d, 0x6e,
0x67, 0x61, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x29, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x79, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x79, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x79, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x29, 0x55, 0x72, 0x75, 0x67, 0x77, 0x61, 0x6a, 0x20, 0x28, 0x73,
0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20,
0x63, 0x7a, 0x61, 0x73, 0x29, 0x55, 0x72, 0x75, 0x67, 0x77, 0x61, 0x6a,
0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x29, 0x57, 0x79, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x57,
0x142, 0x61, 0x64, 0x79, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x28,
0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x57, 0x142, 0x61, 0x64, 0x79, 0x77,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77,
0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x57, 0x6f, 0x142, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29,
0x57, 0x6f, 0x142, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x28, 0x6c,
0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4a,
0x61, 0x6b, 0x75, 0x63, 0x6b, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x29, 0x4a, 0x61, 0x6b, 0x75, 0x63, 0x6b, 0x20, 0x28, 0x6c, 0x61, 0x74,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4a, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x79, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x28,
0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x79, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x28, 0x6c, 0x61, 0x74,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x6a, 0x75, 0x6b, 0x14d, 0x144, 0x73, 0x6b, 0x69, 0x627, 0x641,
0x63a, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x645, 0x631, 0x6aa, 0x632, 0x64a, 0x20, 0x622, 0x641, 0x631, 0x64a,
0x642, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648, 0x680,
0x631, 0x20, 0x622, 0x641, 0x631, 0x64a, 0x6aa, 0x627, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x68f, 0x6a9, 0x6bb, 0x20, 0x622, 0x641, 0x631, 0x64a, 0x6aa,
0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x642, 0x62a, 0x627, 0x648, 0x644, 0x647, 0x647, 0x20, 0x622, 0x641, 0x631,
0x64a, 0x642, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648,
0x644, 0x647, 0x647, 0x20, 0x622, 0x641, 0x631, 0x64a, 0x642, 0x627, 0x20, 0x62c,
0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a,
0x627, 0x648, 0x644, 0x647, 0x647, 0x20, 0x622, 0x641, 0x631, 0x64a, 0x642, 0x627,
0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x644, 0x627, 0x633, 0x6aa, 0x627,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x644, 0x627, 0x633, 0x6aa,
0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x642, 0x62a, 0x627, 0x644, 0x627, 0x633, 0x6aa, 0x627, 0x20, 0x62c, 0x64a,
0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x627, 0x64a, 0x645, 0x64a, 0x632, 0x648, 0x646, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x627, 0x64a, 0x645, 0x64a, 0x632, 0x648, 0x646, 0x20, 0x62c,
0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a,
0x627, 0x64a, 0x645, 0x64a, 0x632, 0x648, 0x646, 0x20, 0x62c, 0x64a, 0x20, 0x627,
0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x645, 0x631, 0x6aa, 0x632, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x631,
0x6aa, 0x632, 0x64a, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x642, 0x62a, 0x645, 0x631, 0x6aa, 0x632, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647,
0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x634, 0x631, 0x642,
0x64a, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x634, 0x631, 0x642, 0x64a, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x634, 0x631,
0x642, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x67e, 0x647, 0x627, 0x699, 0x64a, 0x20, 0x648, 0x642, 0x62a,
0x67e, 0x647, 0x627, 0x699, 0x64a, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x642, 0x62a, 0x67e, 0x647, 0x627, 0x699, 0x64a, 0x20, 0x68f, 0x64a,
0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x64a,
0x633, 0x64a, 0x641, 0x6aa, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x64a, 0x633, 0x64a,
0x641, 0x6aa, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642,
0x62a, 0x67e, 0x64a, 0x633, 0x64a, 0x641, 0x6aa, 0x20, 0x68f, 0x64a, 0x646, 0x647,
0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x67e, 0x64a, 0x627,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x67e, 0x64a, 0x627, 0x20,
0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642,
0x62a, 0x627, 0x67e, 0x64a, 0x627, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646,
0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x639, 0x631, 0x628,
0x64a, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x639, 0x631, 0x628,
0x64a, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x642, 0x62a, 0x639, 0x631, 0x628, 0x64a, 0x646, 0x20, 0x62c, 0x64a,
0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x627, 0x631, 0x62c, 0x646, 0x67d, 0x64a, 0x646, 0x627, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x627, 0x631, 0x62c, 0x646, 0x67d, 0x627, 0x626, 0x646,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x627,
0x631, 0x62c, 0x646, 0x67d, 0x627, 0x626, 0x646, 0x20, 0x62c, 0x64a, 0x20, 0x627,
0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x627, 0x648, 0x644, 0x647, 0x647, 0x20, 0x627, 0x631, 0x62c, 0x646, 0x67d,
0x64a, 0x646, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648,
0x644, 0x647, 0x647, 0x20, 0x627, 0x631, 0x62c, 0x646, 0x67d, 0x64a, 0x646, 0x627,
0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x642, 0x62a, 0x627, 0x648, 0x644, 0x647, 0x647, 0x20, 0x627, 0x631, 0x62c, 0x646,
0x67d, 0x64a, 0x646, 0x627, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647,
0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x631,
0x645, 0x64a, 0x646, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x622, 0x631, 0x645, 0x64a, 0x646, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x631, 0x645,
0x64a, 0x646, 0x64a, 0x627, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647,
0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x64a,
0x67d, 0x644, 0x627, 0x646, 0x67d, 0x6aa, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x627, 0x64a, 0x67d, 0x644, 0x627, 0x646, 0x67d, 0x6aa, 0x20, 0x62c, 0x648,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x627,
0x64a, 0x67d, 0x644, 0x627, 0x646, 0x67d, 0x6aa, 0x20, 0x62c, 0x64a, 0x20, 0x68f,
0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x645,
0x631, 0x6aa, 0x632, 0x64a, 0x20, 0x622, 0x633, 0x67d, 0x631, 0x64a, 0x644, 0x64a,
0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x633, 0x67d, 0x631,
0x64a, 0x644, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x631, 0x6aa, 0x632,
0x64a, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a,
0x622, 0x633, 0x67d, 0x631, 0x64a, 0x644, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20,
0x645, 0x631, 0x6aa, 0x632, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x633, 0x67d, 0x631, 0x64a, 0x644,
0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x631, 0x6aa, 0x632, 0x64a, 0x20,
0x645, 0x63a, 0x631, 0x628, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x633, 0x67d,
0x631, 0x64a, 0x644, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x631, 0x6aa,
0x632, 0x64a, 0x20, 0x645, 0x63a, 0x631, 0x628, 0x64a, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x633, 0x67d, 0x631, 0x64a,
0x644, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x631, 0x6aa, 0x632, 0x64a,
0x20, 0x645, 0x63a, 0x631, 0x628, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648, 0x680, 0x631, 0x20,
0x622, 0x633, 0x67d, 0x631, 0x64a, 0x644, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x622, 0x633, 0x67d, 0x631, 0x64a, 0x644, 0x64a, 0x627, 0x20,
0x62c, 0x648, 0x20, 0x645, 0x634, 0x631, 0x642, 0x64a, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x633, 0x67d, 0x631, 0x64a,
0x644, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x634, 0x631, 0x642, 0x64a,
0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x645, 0x63a, 0x631, 0x628, 0x64a, 0x20, 0x622, 0x633, 0x67d, 0x631, 0x64a,
0x644, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x633,
0x67d, 0x631, 0x64a, 0x644, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x63a,
0x631, 0x628, 0x64a, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x642, 0x62a, 0x622, 0x633, 0x67d, 0x631, 0x64a, 0x644, 0x64a, 0x627, 0x20, 0x62c,
0x648, 0x20, 0x645, 0x63a, 0x631, 0x628, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647,
0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x630, 0x631, 0x628,
0x627, 0x626, 0x64a, 0x62c, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x622, 0x630, 0x631, 0x628, 0x627, 0x626, 0x64a, 0x62c, 0x627, 0x646, 0x20,
0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642,
0x62a, 0x622, 0x630, 0x631, 0x628, 0x627, 0x626, 0x64a, 0x62c, 0x627, 0x646, 0x20,
0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x632, 0x648, 0x631, 0x632, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x632, 0x648, 0x631, 0x632, 0x20, 0x62c,
0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a,
0x627, 0x632, 0x648, 0x631, 0x632, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646,
0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x628,
0x646, 0x6af, 0x644, 0x627, 0x62f, 0x64a, 0x634, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x628, 0x646, 0x6af, 0x644, 0x627, 0x62f, 0x64a, 0x634, 0x20, 0x62c,
0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a,
0x628, 0x646, 0x6af, 0x644, 0x627, 0x62f, 0x64a, 0x634, 0x20, 0x62c, 0x64a, 0x20,
0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x680, 0x648, 0x67d, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x628, 0x648, 0x644, 0x648, 0x64a, 0x627, 0x626, 0x64a, 0x20, 0x648,
0x642, 0x62a, 0x628, 0x631, 0x64a, 0x633, 0x64a, 0x644, 0x64a, 0x627, 0x626, 0x64a,
0x20, 0x648, 0x642, 0x62a, 0x628, 0x631, 0x627, 0x633, 0x64a, 0x644, 0x64a, 0x627,
0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x642, 0x62a, 0x628, 0x631, 0x627, 0x633, 0x64a, 0x644, 0x64a, 0x627, 0x20, 0x62c,
0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x628, 0x631, 0x648, 0x646, 0x627, 0x626, 0x64a, 0x20,
0x62f, 0x627, 0x631, 0x627, 0x644, 0x633, 0x644, 0x627, 0x645, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x64a, 0x67e, 0x20, 0x648, 0x631, 0x68a, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x64a, 0x67e, 0x20, 0x648, 0x631,
0x68a, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x642, 0x62a, 0x6aa, 0x64a, 0x67e, 0x20, 0x648, 0x631, 0x68a, 0x20, 0x62c,
0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x686, 0x645, 0x648, 0x631, 0x648, 0x20, 0x62c, 0x648,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x686,
0x626, 0x67f, 0x645, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x686, 0x626,
0x67f, 0x645, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x642, 0x62a, 0x686, 0x626, 0x67f, 0x645, 0x20, 0x62c, 0x64a, 0x20,
0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x686, 0x644, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x686, 0x644,
0x64a, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x642, 0x62a, 0x686, 0x644, 0x64a, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648,
0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x686, 0x64a, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x686, 0x627,
0x626, 0x646, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x642, 0x62a, 0x686, 0x627, 0x626, 0x646, 0x627, 0x20, 0x62c,
0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x6aa, 0x631, 0x633, 0x645, 0x633, 0x20, 0x622, 0x626, 0x64a, 0x20,
0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x6aa,
0x648, 0x6aa, 0x648, 0x633, 0x20, 0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a, 0x646,
0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x648, 0x644, 0x645,
0x628, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x648,
0x644, 0x645, 0x628, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x648, 0x644, 0x645, 0x628,
0x64a, 0x627, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631,
0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x648, 0x6aa, 0x20,
0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x6aa, 0x648, 0x6aa, 0x20, 0x622, 0x626, 0x64a, 0x20, 0x644,
0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x648, 0x6aa, 0x20, 0x622, 0x626, 0x64a,
0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x68c, 0x20,
0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x6aa, 0x64a, 0x648, 0x628, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x6aa, 0x64a, 0x648, 0x628, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x64a, 0x648,
0x628, 0x627, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x68a, 0x64a, 0x648, 0x633, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x68a, 0x648, 0x645, 0x627, 0x646, 0x67d, 0x20,
0x62f, 0x631, 0x648, 0x64a, 0x626, 0x644, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x627, 0x648, 0x680, 0x631, 0x20, 0x62a, 0x64a, 0x645, 0x648, 0x631, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x64a, 0x633, 0x67d, 0x631, 0x20,
0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x627, 0x64a, 0x633, 0x67d, 0x631, 0x20, 0x622, 0x626, 0x64a,
0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x64a, 0x633, 0x67d, 0x631,
0x20, 0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x64a,
0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x627, 0x64a, 0x6aa, 0x648, 0x627, 0x68a, 0x648, 0x631, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x631, 0x6aa, 0x632, 0x64a, 0x20,
0x64a, 0x648, 0x631, 0x67e, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x631, 0x6aa,
0x632, 0x64a, 0x20, 0x64a, 0x648, 0x631, 0x67e, 0x64a, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x631, 0x6aa, 0x632, 0x64a,
0x20, 0x64a, 0x648, 0x631, 0x67e, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627,
0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x634, 0x631,
0x642, 0x64a, 0x20, 0x64a, 0x648, 0x631, 0x67e, 0x64a, 0x20, 0x648, 0x642, 0x62a,
0x645, 0x634, 0x631, 0x642, 0x64a, 0x20, 0x64a, 0x648, 0x631, 0x67e, 0x64a, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x634,
0x631, 0x642, 0x64a, 0x20, 0x64a, 0x648, 0x631, 0x67e, 0x64a, 0x20, 0x627, 0x648,
0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x648, 0x68c, 0x64a, 0x6aa, 0x20, 0x645, 0x634, 0x631, 0x642, 0x64a, 0x20, 0x64a,
0x648, 0x631, 0x67e, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648, 0x644, 0x647,
0x647, 0x20, 0x64a, 0x648, 0x631, 0x67e, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x627,
0x648, 0x644, 0x647, 0x647, 0x20, 0x64a, 0x648, 0x631, 0x67e, 0x64a, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648, 0x644,
0x647, 0x647, 0x20, 0x64a, 0x648, 0x631, 0x67e, 0x64a, 0x20, 0x68f, 0x64a, 0x646,
0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x627, 0x6aa,
0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a,
0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x627, 0x6aa,
0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a,
0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x642, 0x62a, 0x641, 0x627, 0x6aa, 0x20, 0x644, 0x64a, 0x646, 0x68a,
0x20, 0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x64a,
0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x641, 0x62c, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x641, 0x62c, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627,
0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x62c, 0x64a, 0x20, 0x62c, 0x64a,
0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x641, 0x631, 0x627, 0x646, 0x633, 0x64a, 0x633, 0x64a, 0x20,
0x6af, 0x64a, 0x627, 0x646, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x641, 0x631, 0x627, 0x646, 0x633, 0x64a, 0x633, 0x64a, 0x20, 0x68f, 0x627, 0x6a9,
0x6bb, 0x64a, 0x648, 0x20, 0x6fd, 0x20, 0x627, 0x646, 0x67d, 0x627, 0x631, 0x6aa,
0x67d, 0x6aa, 0x20, 0x648, 0x642, 0x62a, 0x6af, 0x627, 0x644, 0x627, 0x67e, 0x627,
0x6af, 0x648, 0x632, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x6af, 0x64a,
0x645, 0x628, 0x64a, 0x626, 0x631, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x62c, 0x627, 0x631, 0x62c, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x62c, 0x627, 0x631, 0x62c, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x62c, 0x627, 0x631,
0x62c, 0x64a, 0x627, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627,
0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x6af, 0x644, 0x628,
0x631, 0x67d, 0x20, 0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x6af, 0x631, 0x64a, 0x646, 0x20, 0x648,
0x686, 0x20, 0x645, 0x64a, 0x646, 0x20, 0x67d, 0x627, 0x626, 0x64a, 0x645, 0x645,
0x634, 0x631, 0x642, 0x64a, 0x20, 0x6af, 0x631, 0x64a, 0x646, 0x20, 0x644, 0x64a,
0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x634, 0x631,
0x642, 0x64a, 0x20, 0x6af, 0x631, 0x64a, 0x646, 0x20, 0x644, 0x64a, 0x646, 0x68a,
0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x642, 0x62a, 0x645, 0x634, 0x631, 0x642, 0x64a, 0x20, 0x6af, 0x631, 0x64a, 0x646,
0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646,
0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x645,
0x63a, 0x631, 0x628, 0x64a, 0x20, 0x6af, 0x631, 0x64a, 0x646, 0x20, 0x644, 0x64a,
0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x63a, 0x631,
0x628, 0x64a, 0x20, 0x6af, 0x631, 0x64a, 0x646, 0x20, 0x644, 0x64a, 0x646, 0x68a,
0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x642, 0x62a, 0x645, 0x63a, 0x631, 0x628, 0x64a, 0x20, 0x6af, 0x631, 0x64a, 0x646,
0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646,
0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x62e,
0x644, 0x64a, 0x62c, 0x64a, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x642, 0x62a, 0x6af, 0x64a, 0x627, 0x646, 0x627, 0x626, 0x64a, 0x20, 0x648,
0x642, 0x62a, 0x647, 0x648, 0x627, 0x626, 0x64a, 0x20, 0x627, 0x644, 0x64a, 0x648,
0x67d, 0x64a, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x647, 0x648,
0x627, 0x626, 0x64a, 0x20, 0x627, 0x644, 0x64a, 0x648, 0x67d, 0x64a, 0x646, 0x20,
0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642,
0x62a, 0x647, 0x648, 0x627, 0x626, 0x64a, 0x20, 0x627, 0x644, 0x64a, 0x648, 0x67d,
0x64a, 0x646, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x647, 0x627, 0x646, 0x6af, 0x20, 0x6aa,
0x627, 0x646, 0x6af, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x647, 0x627,
0x646, 0x6af, 0x20, 0x6aa, 0x627, 0x646, 0x6af, 0x20, 0x62c, 0x648, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x647, 0x627, 0x646,
0x6af, 0x20, 0x6aa, 0x627, 0x646, 0x6af, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648,
0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x647, 0x648, 0x648, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x647,
0x648, 0x648, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x642, 0x62a, 0x647, 0x648, 0x648, 0x68a, 0x20, 0x62c, 0x64a,
0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x680, 0x627, 0x631, 0x62a, 0x20, 0x62c, 0x648, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x647, 0x646, 0x62f,
0x20, 0x633, 0x645, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x627, 0x646, 0x68a, 0x648, 0x20, 0x686, 0x627, 0x626, 0x646, 0x627, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x631, 0x6aa, 0x632, 0x64a, 0x20, 0x627,
0x646, 0x68a, 0x648, 0x646, 0x64a, 0x634, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x627, 0x648, 0x680, 0x631, 0x20, 0x627, 0x646, 0x68a, 0x648,
0x646, 0x64a, 0x634, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x627, 0x648, 0x644, 0x647, 0x647, 0x20, 0x627, 0x646, 0x68a, 0x648, 0x646, 0x64a,
0x634, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x64a,
0x631, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x64a,
0x631, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x64a, 0x631, 0x627, 0x646, 0x20, 0x62c,
0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x627, 0x631, 0x6aa, 0x62a, 0x633, 0x6aa, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x627, 0x631, 0x6aa, 0x62a, 0x633, 0x6aa, 0x20, 0x62c, 0x648,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x627,
0x631, 0x6aa, 0x62a, 0x633, 0x6aa, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646,
0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x633, 0x631,
0x627, 0x626, 0x64a, 0x644, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627,
0x633, 0x631, 0x627, 0x626, 0x64a, 0x644, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639,
0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x633, 0x631, 0x627,
0x626, 0x64a, 0x644, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x62c, 0x627, 0x67e, 0x627, 0x646,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x62c, 0x627, 0x67e, 0x627, 0x646,
0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x642, 0x62a, 0x62c, 0x627, 0x67e, 0x627, 0x646, 0x20, 0x62c, 0x64a, 0x20, 0x68f,
0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x642,
0x632, 0x627, 0x642, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x648, 0x642, 0x62a, 0x627,
0x648, 0x680, 0x631, 0x20, 0x642, 0x632, 0x627, 0x642, 0x633, 0x62a, 0x627, 0x646,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648, 0x644, 0x647, 0x647,
0x20, 0x642, 0x627, 0x632, 0x642, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x648, 0x631, 0x64a, 0x627, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x648, 0x631, 0x64a, 0x627, 0x20, 0x62c, 0x648,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x6aa,
0x648, 0x631, 0x64a, 0x627, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647,
0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x648, 0x633, 0x631,
0x627, 0x626, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x631,
0x6cc, 0x633, 0x646, 0x648, 0x6cc, 0x627, 0x631, 0x633, 0x6aa, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x631, 0x6cc, 0x633, 0x646, 0x648, 0x6cc, 0x627,
0x631, 0x633, 0x6aa, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x631, 0x6cc, 0x633, 0x646, 0x648, 0x6cc,
0x627, 0x631, 0x633, 0x6aa, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647,
0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x631, 0x63a, 0x632,
0x633, 0x62a, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x644,
0x627, 0x626, 0x646, 0x20, 0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a, 0x646, 0x68a,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x644, 0x648, 0x631, 0x68a, 0x20,
0x647, 0x648, 0x648, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x644,
0x648, 0x631, 0x68a, 0x20, 0x647, 0x648, 0x648, 0x64a, 0x20, 0x62c, 0x648, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x644, 0x648,
0x631, 0x68a, 0x20, 0x647, 0x648, 0x648, 0x64a, 0x20, 0x62c, 0x64a, 0x20, 0x68f,
0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x645,
0x6af, 0x627, 0x62f, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x645, 0x6af, 0x627, 0x62f, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639,
0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x6af, 0x627, 0x62f,
0x627, 0x646, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20,
0x62c, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x644, 0x627, 0x626, 0x64a, 0x634,
0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x627, 0x644,
0x62f, 0x64a, 0x67e, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x631,
0x6aa, 0x64a, 0x648, 0x633, 0x633, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x645, 0x627, 0x631, 0x634, 0x644, 0x20, 0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a,
0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x648, 0x631,
0x64a, 0x634, 0x64a, 0x633, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x645,
0x648, 0x631, 0x64a, 0x634, 0x64a, 0x633, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639,
0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x648, 0x631, 0x64a,
0x634, 0x64a, 0x633, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627,
0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x627, 0x626,
0x648, 0x633, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x64a,
0x6aa, 0x633, 0x64a, 0x6aa, 0x646, 0x20, 0x67e, 0x64a, 0x633, 0x64a, 0x641, 0x6aa,
0x20, 0x648, 0x642, 0x62a, 0x645, 0x64a, 0x6aa, 0x633, 0x64a, 0x6aa, 0x646, 0x20,
0x67e, 0x64a, 0x633, 0x64a, 0x641, 0x6aa, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639,
0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x64a, 0x6aa, 0x633,
0x64a, 0x6aa, 0x646, 0x20, 0x67e, 0x64a, 0x633, 0x64a, 0x641, 0x6aa, 0x20, 0x62c,
0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x627, 0x648, 0x644, 0x627, 0x646, 0x20, 0x628, 0x627, 0x62a, 0x631,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648, 0x644, 0x627, 0x646,
0x20, 0x628, 0x627, 0x62a, 0x631, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648, 0x644, 0x627, 0x646,
0x20, 0x628, 0x627, 0x62a, 0x631, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646,
0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x645,
0x627, 0x633, 0x6aa, 0x648, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x645,
0x627, 0x633, 0x6aa, 0x648, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627,
0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x627, 0x633, 0x6aa, 0x648, 0x20,
0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x64a, 0x20,
0x648, 0x642, 0x62a, 0x645, 0x64a, 0x627, 0x646, 0x645, 0x627, 0x631, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x627, 0x626, 0x648, 0x631, 0x648, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x64a, 0x67e, 0x627, 0x644, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x64a, 0x648, 0x20, 0x6aa, 0x64a,
0x644, 0x64a, 0x68a, 0x648, 0x646, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x646, 0x64a, 0x648, 0x20, 0x6aa, 0x64a, 0x644, 0x64a, 0x68a, 0x648,
0x646, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x64a, 0x648, 0x20, 0x6aa, 0x64a, 0x644,
0x64a, 0x68a, 0x648, 0x646, 0x64a, 0x627, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648,
0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x646, 0x64a, 0x648, 0x632, 0x64a, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x646, 0x64a, 0x648, 0x632, 0x64a, 0x644, 0x64a, 0x646,
0x68a, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x642, 0x62a, 0x646, 0x64a, 0x648, 0x632, 0x64a, 0x644, 0x64a, 0x646, 0x68a,
0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x646, 0x64a, 0x648, 0x20, 0x641, 0x627, 0x626, 0x648,
0x646, 0x68a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x646, 0x64a, 0x648, 0x20, 0x641, 0x627, 0x626, 0x648, 0x646, 0x68a,
0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x64a, 0x648, 0x20, 0x641,
0x627, 0x626, 0x648, 0x646, 0x68a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c,
0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x646, 0x64a, 0x648, 0x648, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x646, 0x627, 0x631, 0x20, 0x641, 0x648, 0x6aa, 0x20, 0x622, 0x626,
0x64a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x646, 0x627, 0x631, 0x20, 0x641, 0x648, 0x6aa, 0x20, 0x622, 0x626, 0x64a,
0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x627, 0x631, 0x20, 0x641,
0x648, 0x6aa, 0x20, 0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20,
0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x641, 0x631, 0x646, 0x627, 0x646, 0x68a, 0x648, 0x20, 0x62f,
0x64a, 0x20, 0x646, 0x648, 0x631, 0x648, 0x646, 0x647, 0x627, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x641, 0x631, 0x646, 0x627, 0x646, 0x68a, 0x648, 0x20,
0x62f, 0x64a, 0x20, 0x646, 0x648, 0x631, 0x648, 0x646, 0x647, 0x627, 0x20, 0x62c,
0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a,
0x641, 0x631, 0x646, 0x627, 0x646, 0x68a, 0x648, 0x20, 0x62f, 0x64a, 0x20, 0x646,
0x648, 0x631, 0x648, 0x646, 0x647, 0x627, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648,
0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x648, 0x648,
0x627, 0x633, 0x628, 0x626, 0x64a, 0x631, 0x633, 0x6aa, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x646, 0x648, 0x648, 0x627, 0x633, 0x628, 0x626, 0x64a, 0x631,
0x633, 0x6aa, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x642, 0x62a, 0x646, 0x648, 0x648, 0x627, 0x633, 0x628, 0x626, 0x64a,
0x631, 0x633, 0x6aa, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648, 0x645, 0x633, 0x6aa,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648, 0x645, 0x633, 0x6aa,
0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x642, 0x62a, 0x627, 0x648, 0x645, 0x633, 0x6aa, 0x20, 0x62c, 0x64a, 0x20, 0x68f,
0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x67e,
0x627, 0x6aa, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x67e, 0x627, 0x6aa, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x627,
0x6aa, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646,
0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x67e,
0x644, 0x627, 0x626, 0x648, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x67e,
0x627, 0x67e, 0x648, 0x627, 0x20, 0x646, 0x64a, 0x648, 0x20, 0x6af, 0x646, 0x64a,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x64a, 0x631, 0x627, 0x6af,
0x648, 0x626, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x64a,
0x631, 0x627, 0x6af, 0x648, 0x626, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639,
0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x64a, 0x631, 0x627,
0x6af, 0x648, 0x626, 0x64a, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647,
0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x64a,
0x631, 0x648, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x64a, 0x631,
0x648, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x642, 0x62a, 0x67e, 0x64a, 0x631, 0x648, 0x20, 0x62c, 0x64a, 0x20, 0x627,
0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x641, 0x644, 0x67e, 0x627, 0x626, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x641, 0x644, 0x67e, 0x627, 0x626, 0x646, 0x20, 0x62c, 0x648, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x644,
0x67e, 0x627, 0x626, 0x646, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647,
0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x648,
0x646, 0x64a, 0x6aa, 0x633, 0x20, 0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a, 0x646,
0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x633, 0x64a, 0x646, 0x67d,
0x20, 0x67e, 0x64a, 0x626, 0x631, 0x20, 0x6fd, 0x20, 0x645, 0x64a, 0x6aa, 0x644,
0x648, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x633, 0x64a, 0x646,
0x67d, 0x20, 0x67e, 0x64a, 0x626, 0x631, 0x20, 0x6fd, 0x20, 0x645, 0x64a, 0x6aa,
0x644, 0x648, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x642, 0x62a, 0x633, 0x64a, 0x646, 0x67d, 0x20, 0x67e, 0x64a,
0x626, 0x631, 0x20, 0x6fd, 0x20, 0x645, 0x64a, 0x6aa, 0x644, 0x648, 0x646, 0x20,
0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x67e, 0x67d, 0x6aa, 0x64a, 0x631, 0x646, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x67e, 0x648, 0x646, 0x627, 0x67e, 0x64a, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x64a, 0x627, 0x646, 0x6af, 0x20, 0x64a,
0x627, 0x646, 0x6af, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x631, 0x64a,
0x20, 0x64a, 0x648, 0x646, 0x64a, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x631, 0x648, 0x67f, 0x64a, 0x631, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x633, 0x62e, 0x627, 0x644, 0x64a, 0x646, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x633, 0x62e, 0x627, 0x644, 0x64a, 0x646, 0x20, 0x62c, 0x648,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x633,
0x62e, 0x627, 0x644, 0x64a, 0x646, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646,
0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x633, 0x627, 0x645,
0x648, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x633, 0x627, 0x645,
0x648, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x642, 0x62a, 0x633, 0x627, 0x645, 0x648, 0x627, 0x20, 0x62c, 0x64a,
0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x634, 0x64a, 0x20, 0x634, 0x644, 0x632, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x633, 0x646, 0x6af, 0x627, 0x67e, 0x648, 0x631, 0x20, 0x62c, 0x648,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x633,
0x648, 0x644, 0x648, 0x645, 0x646, 0x20, 0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a,
0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x68f, 0x6a9, 0x6bb,
0x20, 0x62c, 0x627, 0x631, 0x62c, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x633, 0x648, 0x631, 0x64a, 0x20, 0x646, 0x627, 0x645, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x633, 0x627, 0x626, 0x64a, 0x648, 0x627, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x62a, 0x627, 0x647, 0x64a, 0x67d, 0x64a,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x62a, 0x627, 0x626, 0x67e, 0x64a,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x62a, 0x627, 0x626, 0x67e, 0x64a,
0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x642, 0x62a, 0x62a, 0x627, 0x626, 0x67e, 0x64a, 0x20, 0x62c, 0x64a, 0x20, 0x68f,
0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x62a,
0x627, 0x62c, 0x6aa, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x67d, 0x648, 0x6aa, 0x64a, 0x644, 0x627, 0x626, 0x648, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x67d, 0x648, 0x646, 0x6af, 0x627, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x67d, 0x648, 0x646, 0x6af, 0x627, 0x20, 0x62c,
0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a,
0x67d, 0x648, 0x646, 0x6af, 0x627, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646,
0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x686,
0x64a, 0x648, 0x6aa, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x62a, 0x631,
0x6aa, 0x645, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x62a, 0x631, 0x6aa, 0x645, 0x627, 0x646, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x642, 0x62a, 0x62a, 0x631, 0x6aa, 0x645, 0x627, 0x646, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x62a, 0x648, 0x648, 0x627, 0x644,
0x648, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x64a, 0x648, 0x631, 0x648,
0x6af, 0x627, 0x626, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x64a,
0x648, 0x631, 0x648, 0x6af, 0x627, 0x626, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x64a, 0x648, 0x631,
0x648, 0x6af, 0x627, 0x626, 0x64a, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646,
0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627,
0x632, 0x628, 0x6aa, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x627, 0x632, 0x628, 0x6aa, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x62c,
0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a,
0x627, 0x632, 0x628, 0x6aa, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x62c, 0x64a, 0x20,
0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x648, 0x627, 0x646, 0x648, 0x627, 0x62a, 0x648, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x648, 0x627, 0x646, 0x648, 0x627, 0x62a, 0x648, 0x20,
0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642,
0x62a, 0x648, 0x627, 0x646, 0x648, 0x627, 0x62a, 0x648, 0x20, 0x62c, 0x64a, 0x20,
0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x648, 0x64a, 0x646, 0x632, 0x648, 0x64a, 0x644, 0x627, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x648, 0x644, 0x627, 0x62f, 0x648, 0x648, 0x633, 0x62a, 0x648,
0x6aa, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x648, 0x644, 0x627, 0x62f,
0x648, 0x648, 0x633, 0x62a, 0x648, 0x6aa, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639,
0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648, 0x644, 0x627,
0x62f, 0x648, 0x648, 0x633, 0x62a, 0x648, 0x6aa, 0x20, 0x62c, 0x64a, 0x20, 0x68f,
0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x648,
0x648, 0x644, 0x6af, 0x648, 0x6af, 0x631, 0x627, 0x62f, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x648, 0x648, 0x644, 0x6af, 0x648, 0x6af, 0x631, 0x627, 0x62f,
0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x642, 0x62a, 0x648, 0x648, 0x644, 0x6af, 0x648, 0x6af, 0x631, 0x627, 0x62f, 0x20,
0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x648, 0x648, 0x633, 0x67d, 0x648, 0x6aa, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x648, 0x64a, 0x6aa, 0x20, 0x622, 0x626, 0x64a, 0x20,
0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x648,
0x64a, 0x644, 0x633, 0x20, 0x6fd, 0x20, 0x641, 0x62a, 0x648, 0x646, 0x627, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x64a, 0x627, 0x6aa, 0x62a, 0x633, 0x6aa,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x64a, 0x627, 0x6aa, 0x62a, 0x633,
0x6aa, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x642, 0x62a, 0x64a, 0x627, 0x6aa, 0x62a, 0x633, 0x6aa, 0x20, 0x62c, 0x64a,
0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x64a, 0x6aa, 0x627, 0x67d, 0x64a, 0x631, 0x646, 0x628, 0x631, 0x6af, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x64a, 0x6aa, 0x627, 0x67d, 0x64a, 0x631,
0x646, 0x628, 0x631, 0x6af, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627,
0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x64a, 0x6aa, 0x627, 0x67d, 0x64a, 0x631,
0x646, 0x628, 0x631, 0x6af, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647,
0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x64a, 0x6aa, 0x648, 0x646,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x92e, 0x930, 0x915, 0x91c, 0x93c,
0x940, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x92e, 0x930, 0x915, 0x91c, 0x93c, 0x940,
0x20, 0x92e, 0x905, 0x92f, 0x93e, 0x930, 0x940, 0x20, 0x935, 0x915, 0x94d, 0x924,
0x92e, 0x930, 0x915, 0x91c, 0x93c, 0x940, 0x20, 0x926, 0x940, 0x902, 0x952, 0x939,
0x20, 0x91c, 0x94b, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x913, 0x92d, 0x930, 0x940,
0x20, 0x935, 0x915, 0x94d, 0x924, 0x913, 0x92d, 0x930, 0x940, 0x20, 0x92e, 0x905,
0x92f, 0x93e, 0x930, 0x940, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x913, 0x92d, 0x930,
0x940, 0x20, 0x926, 0x940, 0x902, 0x952, 0x939, 0x20, 0x91c, 0x94b, 0x20, 0x935,
0x915, 0x94d, 0x924, 0x92a, 0x939, 0x93e, 0x921, 0x93c, 0x940, 0x20, 0x935, 0x915,
0x94d, 0x924, 0x92a, 0x939, 0x93e, 0x921, 0x93c, 0x940, 0x20, 0x92e, 0x905, 0x92f,
0x93e, 0x930, 0x940, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x92a, 0x939, 0x93e, 0x921,
0x93c, 0x940, 0x20, 0x926, 0x940, 0x902, 0x952, 0x939, 0x20, 0x91c, 0x94b, 0x20,
0x935, 0x915, 0x94d, 0x924, 0x92a, 0x947, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20,
0x935, 0x915, 0x94d, 0x924, 0x92a, 0x947, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20,
0x92e, 0x905, 0x92f, 0x93e, 0x930, 0x940, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x92a,
0x947, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x926, 0x940, 0x902, 0x952, 0x939,
0x20, 0x91c, 0x94b, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x905, 0x91f, 0x932, 0x93e,
0x902, 0x91f, 0x93f, 0x915, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x905, 0x91f, 0x932,
0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20, 0x92e, 0x905, 0x92f, 0x93e, 0x930, 0x940,
0x20, 0x935, 0x915, 0x94d, 0x924, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f,
0x915, 0x20, 0x926, 0x940, 0x902, 0x952, 0x939, 0x20, 0x91c, 0x94b, 0x20, 0x935,
0x915, 0x94d, 0x924, 0x92e, 0x930, 0x915, 0x91c, 0x93c, 0x940, 0x20, 0x92f, 0x942,
0x930, 0x94b, 0x92a, 0x940, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x92e, 0x930, 0x915,
0x91c, 0x93c, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x20, 0x92e,
0x905, 0x92f, 0x93e, 0x930, 0x940, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x941, 0x92e,
0x930, 0x915, 0x91c, 0x93c, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940,
0x20, 0x909, 0x928, 0x939, 0x93e, 0x930, 0x947, 0x20, 0x91c, 0x94b, 0x20, 0x935,
0x915, 0x94d, 0x924, 0x941, 0x913, 0x92d, 0x930, 0x940, 0x20, 0x92f, 0x942, 0x930,
0x94b, 0x92a, 0x940, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x941, 0x913, 0x92d, 0x930,
0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x20, 0x92e, 0x905, 0x92f,
0x93e, 0x930, 0x940, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x941, 0x913, 0x92d, 0x930,
0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x20, 0x909, 0x928, 0x939,
0x93e, 0x930, 0x947, 0x20, 0x91c, 0x94b, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x941,
0x909, 0x932, 0x939, 0x902, 0x926, 0x94b, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a,
0x940, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x941, 0x909, 0x932, 0x939, 0x902, 0x926,
0x94b, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x20, 0x92e, 0x905, 0x92f,
0x93e, 0x930, 0x940, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x941, 0x909, 0x932, 0x939,
0x902, 0x926, 0x94b, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x20, 0x909,
0x928, 0x939, 0x93e, 0x930, 0x947, 0x20, 0x91c, 0x94b, 0x20, 0x935, 0x915, 0x94d,
0x924, 0x941, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x935, 0x93f, 0x91a, 0x20, 0x92e,
0x940, 0x928, 0x20, 0x935, 0x915, 0x93c, 0x94d, 0x924, 0x941, 0xd87, 0xdc6, 0xdca,
0xd9c, 0xdb1, 0xdd2, 0xdc3, 0xdca, 0xdae, 0xdcf, 0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdb8, 0xdb0, 0xdca, 0x200d, 0xdba, 0xdb8, 0x20, 0xd85, 0xdb4, 0xdca,
0x200d, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd85, 0xdb4,
0xdca, 0x200d, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdaf, 0xd9a, 0xdd4, 0xdab, 0xdd4, 0x20, 0xd85, 0xdb4, 0xdca, 0x200d,
0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdb6, 0xda7, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd85, 0xdb4, 0xdca, 0x200d, 0xdbb, 0xdd2,
0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xda7,
0xdc4, 0xdd2, 0xdbb, 0x20, 0xd85, 0xdb4, 0xdca, 0x200d, 0xdbb, 0xdd2, 0xd9a, 0xdcf,
0xdb1, 0xdd4, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdb6, 0xda7, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd85, 0xdb4, 0xdca, 0x200d,
0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3,
0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xd87, 0xdbd, 0xdc3, 0xdca,
0xd9a, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd87, 0xdbd, 0xdc3, 0xdca,
0xd9a, 0xdcf, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xd87, 0xdbd, 0xdc3, 0xdca, 0xd9a, 0xdcf, 0x20, 0xdaf, 0xdd2, 0xdc0,
0xdcf, 0xd86, 0xdbd, 0xddd, 0xd9a, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd87,
0xdb8, 0xdbb, 0xdca, 0xdc3, 0xdb1, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xd87, 0xdb8, 0xdbb, 0xdca, 0xdc3, 0xdb1, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8,
0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd87, 0xdb8, 0xdbb, 0xdca, 0xdc3,
0xdb1, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20,
0xd9a, 0xdcf, 0xdbd, 0xdba, 0xd8b, 0xdad, 0xdd4, 0xdbb, 0xdd4, 0x20, 0xd87, 0xdb8,
0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdb8, 0xdb0, 0xdca, 0x200d, 0xdba,
0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdad, 0xdd4, 0xdbb, 0xdd4,
0x20, 0xd87, 0xdb8, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdb8, 0xdb0,
0xdca, 0x200d, 0xdba, 0xdb8, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdad, 0xdd4, 0xdbb, 0xdd4, 0x20, 0xd87, 0xdb8,
0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdb8, 0xdb0, 0xdca, 0x200d, 0xdba,
0xdb8, 0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf, 0xd86, 0xdbd, 0xddd, 0xd9a, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdad, 0xdd4, 0xdbb, 0xdd4, 0x20, 0xd87, 0xdb8,
0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1,
0xdc4, 0xdd2, 0xdbb, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdad, 0xdd4,
0xdbb, 0xdd4, 0x20, 0xd87, 0xdb8, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20,
0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xdc3, 0xdb8, 0xdca,
0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdad, 0xdd4, 0xdbb,
0xdd4, 0x20, 0xd87, 0xdb8, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdb1,
0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf,
0xd86, 0xdbd, 0xddd, 0xd9a, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdad,
0xdd4, 0xdbb, 0xdd4, 0x20, 0xd87, 0xdb8, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4,
0x20, 0xd9a, 0xdb3, 0xdd4, 0xd9a, 0xdbb, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xd8b, 0xdad, 0xdd4, 0xdbb, 0xdd4, 0x20, 0xd87, 0xdb8, 0xdbb, 0xdd2, 0xd9a, 0xdcf,
0xdb1, 0xdd4, 0x20, 0xd9a, 0xdb3, 0xdd4, 0xd9a, 0xdbb, 0x20, 0xdc3, 0xdb8, 0xdca,
0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdad, 0xdd4, 0xdbb,
0xdd4, 0x20, 0xd87, 0xdb8, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xd9a,
0xdb3, 0xdd4, 0xd9a, 0xdbb, 0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf, 0xd86, 0xdbd, 0xddd,
0xd9a, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdad, 0xdd4, 0xdbb, 0xdd4,
0x20, 0xd87, 0xdb8, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdb4, 0xdd0,
0xdc3, 0xdd2, 0xdc6, 0xdd2, 0xd9a, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xd8b, 0xdad, 0xdd4, 0xdbb, 0xdd4, 0x20, 0xd87, 0xdb8, 0xdbb, 0xdd2, 0xd9a, 0xdcf,
0xdb1, 0xdd4, 0x20, 0xdb4, 0xdd0, 0xdc3, 0xdd2, 0xdc6, 0xdd2, 0xd9a, 0xdca, 0x20,
0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b,
0xdad, 0xdd4, 0xdbb, 0xdd4, 0x20, 0xd87, 0xdb8, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1,
0xdd4, 0x20, 0xdb4, 0xdd0, 0xdc3, 0xdd2, 0xdc6, 0xdd2, 0xd9a, 0xdca, 0x20, 0xdaf,
0xdd2, 0xdc0, 0xdcf, 0xd86, 0xdbd, 0xddd, 0xd9a, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xd85, 0xdb4, 0xdd2, 0xdba, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xd85, 0xdb4, 0xdd2, 0xdba, 0xdcf, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd85, 0xdb4, 0xdd2, 0xdba, 0xdcf, 0x20, 0xdaf,
0xdd2, 0xdc0, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd85, 0xdbb, 0xdcf,
0xdb6, 0xdd2, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd85, 0xdbb, 0xdcf, 0xdb6,
0xdd2, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xd85, 0xdbb, 0xdcf, 0xdb6, 0xdd2, 0x20, 0xdaf, 0xdc4, 0xdc0, 0xdbd, 0xdca,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd86, 0xdbb, 0xdca, 0xda2, 0xdb1, 0xdca,
0xda7, 0xdd2, 0xdb1, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd86, 0xdbb,
0xdca, 0xda2, 0xdb1, 0xdca, 0xda7, 0xdd2, 0xdb1, 0xdcf, 0x20, 0xdc3, 0xdb8, 0xdca,
0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd86, 0xdbb, 0xdca, 0xda2,
0xdb1, 0xdca, 0xda7, 0xdd2, 0xdb1, 0xdcf, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3,
0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xdb6, 0xda7, 0xdc4, 0xdd2,
0xdbb, 0x20, 0xd86, 0xdbb, 0xdca, 0xda2, 0xdb1, 0xdca, 0xda7, 0xdd2, 0xdb1, 0xdcf,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xda7, 0xdc4, 0xdd2, 0xdbb, 0x20,
0xd86, 0xdbb, 0xdca, 0xda2, 0xdb1, 0xdca, 0xda7, 0xdd2, 0xdb1, 0xdcf, 0x20, 0xdc3,
0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xda7,
0xdc4, 0xdd2, 0xdbb, 0x20, 0xd86, 0xdbb, 0xdca, 0xda2, 0xdb1, 0xdca, 0xda7, 0xdd2,
0xdb1, 0xdcf, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20,
0xd9a, 0xdcf, 0xdbd, 0xdba, 0xd86, 0xdb8, 0xdda, 0xdb1, 0xdd2, 0xdba, 0xdcf, 0xdb1,
0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd86, 0xdb8, 0xdda, 0xdb1, 0xdd2,
0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd86, 0xdb8, 0xdda, 0xdb1, 0xdd2, 0xdba, 0xdcf, 0xdb1,
0xdd4, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd85, 0xdad, 0xdca, 0xdbd, 0xdcf, 0xdb1, 0xdca, 0xdad,
0xdd2, 0xd9a, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd85, 0xdad, 0xdca,
0xdbd, 0xdcf, 0xdb1, 0xdca, 0xdad, 0xdd2, 0xd9a, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca,
0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd85, 0xdad, 0xdca, 0xdbd,
0xdcf, 0xdb1, 0xdca, 0xdad, 0xdd2, 0xd9a, 0xdca, 0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf,
0xd86, 0xdbd, 0xddd, 0xd9a, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdb0,
0xdca, 0x200d, 0xdba, 0xdb8, 0x20, 0xd95, 0xdc3, 0xdca, 0xda7, 0xdca, 0x200d, 0xdbb,
0xdda, 0xdbd, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xd95, 0xdc3, 0xdca, 0xda7, 0xdca, 0x200d, 0xdbb, 0xdda, 0xdbd, 0xdd2, 0xdba,
0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdb0, 0xdca, 0x200d, 0xdba, 0xdb8, 0x20, 0xd94, 0xdc3,
0xdca, 0xda7, 0xdca, 0x200d, 0xdbb, 0xdda, 0xdbd, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4,
0x20, 0xdaf, 0xdc4, 0xdc0, 0xdbd, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdb8, 0xdb0, 0xdca, 0x200d, 0xdba, 0xdb8, 0x20, 0xdb6, 0xda7, 0xdc4, 0xdd2, 0xdbb,
0x20, 0xd94, 0xdc3, 0xdca, 0xda7, 0xdca, 0x200d, 0xdbb, 0xdda, 0xdbd, 0xdd2, 0xdba,
0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdb0, 0xdca,
0x200d, 0xdba, 0xdb8, 0x20, 0xdb6, 0xda7, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd94, 0xdc3,
0xdca, 0xda7, 0xdca, 0x200d, 0xdbb, 0xdda, 0xdbd, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4,
0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdb8, 0xdb0, 0xdca, 0x200d, 0xdba, 0xdb8, 0x20, 0xdb6, 0xda7, 0xdc4, 0xdd2, 0xdbb,
0x20, 0xd94, 0xdc3, 0xdca, 0xda7, 0xdca, 0x200d, 0xdbb, 0xdda, 0xdbd, 0xdd2, 0xdba,
0xdcf, 0xdb1, 0xdd4, 0x20, 0xdaf, 0xdc4, 0xdc0, 0xdbd, 0xdca, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb, 0x20,
0xd95, 0xdc3, 0xdca, 0xda7, 0xdca, 0x200d, 0xdbb, 0xdda, 0xdbd, 0xdd2, 0xdba, 0xdcf,
0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd0, 0xd9c, 0xdd9,
0xdb1, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd95, 0xdc3, 0xdca, 0xda7, 0xdca, 0x200d, 0xdbb,
0xdda, 0xdbd, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8,
0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd0, 0xd9f, 0xdd9, 0xdb1,
0xdc4, 0xdd2, 0xdbb, 0x20, 0xd95, 0xdc3, 0xdca, 0xda7, 0xdca, 0x200d, 0xdbb, 0xdda,
0xdbd, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdaf, 0xdc4, 0xdc0, 0xdbd, 0xdca,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xda7, 0xdc4, 0xdd2, 0xdbb, 0x20,
0xd95, 0xdc3, 0xdca, 0xda7, 0xdca, 0x200d, 0xdbb, 0xdda, 0xdbd, 0xdd2, 0xdba, 0xdcf,
0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xda7, 0xdc4, 0xdd2,
0xdbb, 0x20, 0xd95, 0xdc3, 0xdca, 0xda7, 0xdca, 0x200d, 0xdbb, 0xdda, 0xdbd, 0xdd2,
0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd85, 0xdc3, 0xdbb, 0xdca, 0xdb6, 0xdba, 0xdd2, 0xda2,
0xdcf, 0xdb1, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd85, 0xdc3, 0xdbb,
0xdca, 0xdb6, 0xdba, 0xdd2, 0xda2, 0xdcf, 0xdb1, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca,
0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd85, 0xdc3, 0xdbb, 0xdca,
0xdb6, 0xdba, 0xdd2, 0xda2, 0xdcf, 0xdb1, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb,
0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd87, 0xdc3,
0xddc, 0xdbb, 0xdca, 0xdc3, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd87,
0xdc3, 0xddc, 0xdbb, 0xdca, 0xdc3, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd87, 0xdc3, 0xddc, 0xdbb, 0xdca, 0xdc3,
0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xd82, 0xd9c, 0xdbd, 0xdcf, 0xdaf, 0xdda, 0xdc1,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xd82, 0xd9c, 0xdbd, 0xdcf, 0xdaf,
0xdda, 0xdc1, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdb6, 0xd82, 0xd9c, 0xdbd, 0xdcf, 0xdaf, 0xdda, 0xdc1, 0x20, 0xd9c,
0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba,
0xdb7, 0xdd4, 0xdad, 0xdcf, 0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6,
0xddc, 0xdbd, 0xdd2, 0xdc0, 0xdd2, 0xdba, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdb6, 0xdca, 0x200d, 0xdbb, 0xdc3, 0xdd3, 0xdbd, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdb6, 0xdca, 0x200d, 0xdbb, 0xdc3, 0xdd3, 0xdbd, 0x20, 0xdc3, 0xdb8,
0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xdca, 0x200d,
0xdbb, 0xdc3, 0xdd3, 0xdbd, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca,
0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xdb6, 0xdd8, 0xdb1, 0xdcf, 0xdba, 0xdd2,
0x20, 0xdaf, 0xdbb, 0xdd4, 0xdc3, 0xdca, 0xdc3, 0xdbd, 0xdcf, 0xdb8, 0xdca, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdda, 0xdb4, 0xdca, 0xdc0, 0xdda, 0xda9,
0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdda, 0xdb4, 0xdca, 0xdc0,
0xdda, 0xda9, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdda, 0xdb4, 0xdca, 0xdc0, 0xdda, 0xda9, 0xdca, 0x20,
0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd,
0xdba, 0xda0, 0xdb8, 0xddc, 0xdbb, 0xddd, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xda0, 0xdd0, 0xdad, 0xdb8, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda0,
0xdd0, 0xdad, 0xdb8, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda0, 0xdd0, 0xdad, 0xdb8, 0xdca, 0x20, 0xdaf, 0xdd2,
0xdc0, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda0, 0xdd2, 0xdbd, 0xdd3,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda0, 0xdd2, 0xdbd, 0xdd3, 0x20, 0xdc3,
0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda0, 0xdd2,
0xdbd, 0xdd3, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20,
0xd9a, 0xdcf, 0xdbd, 0xdba, 0xda0, 0xdd3, 0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xda0, 0xdd3, 0xdb1, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda0, 0xdd3, 0xdb1, 0x20, 0xdaf, 0xdc4, 0xdc0, 0xdbd,
0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdca, 0x200d, 0xdbb, 0xdd2,
0xdc3, 0xdca, 0xdb8, 0xdc3, 0xdca, 0x20, 0xdaf, 0xdd4, 0xdb4, 0xdad, 0xdca, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xddc, 0xd9a, 0xddd, 0xdc3, 0xdca, 0x20,
0xdaf, 0xdd4, 0xdb4, 0xdad, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a,
0xddc, 0xdbd, 0xddc, 0xdb8, 0xdca, 0xdb6, 0xdd2, 0xdba, 0xdcf, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xddc, 0xdbd, 0xddc, 0xdb8, 0xdca, 0xdb6, 0xdd2, 0xdba,
0xdcf, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xd9a, 0xddc, 0xdbd, 0xddc, 0xdb8, 0xdca, 0xdb6, 0xdd2, 0xdba, 0xdcf, 0x20,
0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd,
0xdba, 0xd9a, 0xdd4, 0xd9a, 0xdca, 0x20, 0xdaf, 0xdd4, 0xdb4, 0xdad, 0xdca, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdd4, 0xd9a, 0xdca, 0x20, 0xdaf, 0xdd4,
0xdb4, 0xdad, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdd4, 0xd9a, 0xdca, 0x20, 0xdaf, 0xdd4, 0xdb4, 0xdad,
0xdca, 0x20, 0xdb7, 0xdcf, 0xd9c, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2,
0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdd2, 0xdba, 0xdd4,
0xdb6, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdd2,
0xdba, 0xdd4, 0xdb6, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdd2, 0xdba, 0xdd4, 0xdb6, 0xdcf,
0xdb1, 0xdd4, 0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf, 0xd86, 0xdbd, 0xddd, 0xd9a, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda9, 0xdcf, 0xdc0, 0xdd2, 0xdc3, 0xdca, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdaf, 0xdd4, 0xdb8, 0xddc, 0xdb1, 0xdca, 0xdad,
0xdca, 0x2d, 0xda9, 0xdca, 0xd8b, 0xdbb, 0xdca, 0xdc0, 0xdd2, 0xdbd, 0xdca, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2,
0xdbb, 0x20, 0xda7, 0xdd2, 0xdb8, 0xddd, 0xdbb, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xd8a, 0xdc3, 0xdca, 0xda7, 0xdbb, 0xdca, 0x20, 0xdaf, 0xdd6, 0xdb4,
0xdad, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8a, 0xdc3, 0xdca, 0xda7,
0xdbb, 0xdca, 0x20, 0xdaf, 0xdd6, 0xdb4, 0xdad, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca,
0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8a, 0xdc3, 0xdca, 0xda7,
0xdbb, 0xdca, 0x20, 0xdaf, 0xdd6, 0xdb4, 0xdad, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d,
0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xd89, 0xd9a,
0xdca, 0xdc0, 0xdaf, 0xddd, 0xdbb, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdb8, 0xdb0, 0xdca, 0x200d, 0xdba, 0xdb8, 0x20, 0xdba, 0xdd4, 0xdbb, 0xddd, 0xdb4,
0xdd3, 0xdba, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdb0, 0xdca, 0x200d,
0xdba, 0xdb8, 0x20, 0xdba, 0xdd4, 0xdbb, 0xddd, 0xdb4, 0xdd3, 0xdba, 0x20, 0xdc3,
0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdb0,
0xdca, 0x200d, 0xdba, 0xdb8, 0x20, 0xdba, 0xdd4, 0xdbb, 0xddd, 0xdb4, 0xdd3, 0xdba,
0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb, 0x20,
0xdba, 0xdd4, 0xdbb, 0xddd, 0xdb4, 0xdd3, 0xdba, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xdba, 0xdd4,
0xdbb, 0xddd, 0xdb4, 0xdd3, 0xdba, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2,
0xdbb, 0x20, 0xdba, 0xdd4, 0xdbb, 0xddd, 0xdb4, 0xdd3, 0xdba, 0x20, 0xd9c, 0xdca,
0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdad, 0xdc0, 0xdad, 0xdca, 0x2d, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2,
0xdbb, 0x20, 0xdba, 0xdd4, 0xdbb, 0xddd, 0xdb4, 0xdd3, 0xdba, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xda7, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xdba, 0xdd4, 0xdbb,
0xddd, 0xdb4, 0xdd3, 0xdba, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xda7,
0xdc4, 0xdd2, 0xdbb, 0x20, 0xdba, 0xdd4, 0xdbb, 0xddd, 0xdb4, 0xdd3, 0xdba, 0x20,
0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6,
0xda7, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xdba, 0xdd4, 0xdbb, 0xddd, 0xdb4, 0xdd3, 0xdba,
0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdc6, 0xddd, 0xd9a, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xda9, 0xdca,
0x20, 0xdaf, 0xdd6, 0xdb4, 0xdad, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdc6, 0xddd, 0xd9a, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xda9, 0xdca, 0x20, 0xdaf, 0xdd6,
0xdb4, 0xdad, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdc6, 0xddd, 0xd9a, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xda9, 0xdca,
0x20, 0xdaf, 0xdd6, 0xdb4, 0xdad, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3,
0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xdc6, 0xdd2, 0xda2, 0xdd2,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc6, 0xdd2, 0xda2, 0xdd2, 0x20, 0xdc3,
0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc6, 0xdd2,
0xda2, 0xdd2, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb4, 0xdca, 0x200d, 0xdbb, 0xd82, 0xdc1, 0x20,
0xd9c, 0xdba, 0xdb1, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb4, 0xdca,
0x200d, 0xdbb, 0xd82, 0xdc1, 0x20, 0xdaf, 0xd9a, 0xdd4, 0xdab, 0xdd4, 0xdaf, 0xdd2,
0xd9c, 0x20, 0xdc3, 0xdc4, 0x20, 0xd87, 0xdb1, 0xdca, 0xda7, 0xdcf, 0xdbb, 0xdca,
0xd9a, 0xdca, 0xda7, 0xdd2, 0xd9a, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xd9c, 0xdbd, 0xdb4, 0xd9c, 0xddc, 0xdc3, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xd9c, 0xdd0, 0xdb8, 0xdca, 0xdb6, 0xdd2, 0xdba, 0xdbb, 0xdca, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda2, 0xddd, 0xdbb, 0xdca, 0xda2, 0xdd2, 0xdba, 0xdcf,
0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda2, 0xddd, 0xdbb, 0xdca,
0xda2, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda2, 0xddd, 0xdbb, 0xdca, 0xda2, 0xdd2,
0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca,
0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9c, 0xdd2, 0xdbd, 0xdca, 0xdb6,
0xdbb, 0xdca, 0xda7, 0xdca, 0x20, 0xdaf, 0xdd4, 0xdb4, 0xdad, 0xdca, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd2, 0xdb1, 0xdd2, 0xda0,
0xdca, 0x20, 0xdb8, 0xdb0, 0xdca, 0x200d, 0xdba, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd9c,
0xdca, 0x200d, 0xdbb, 0xdd3, 0xdb1, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xdad, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb,
0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdb1, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xdad,
0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd9c, 0xdca, 0x200d,
0xdbb, 0xdd3, 0xdb1, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xdad, 0x20, 0xd9c, 0xdca, 0x200d,
0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xdb6, 0xda7,
0xdc4, 0xdd2, 0xdbb, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdb1, 0xdca, 0xdbd,
0xdb1, 0xdca, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xda7, 0xdc4,
0xdd2, 0xdbb, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdb1, 0xdca, 0xdbd, 0xdb1,
0xdca, 0xdad, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdb6, 0xda7, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb,
0xdd3, 0xdb1, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xdad, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb,
0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xd9c, 0xdbd, 0xdca,
0xdc6, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc4, 0xdc0, 0xdcf, 0xdba,
0xdd2, 0x2d, 0xd85, 0xdbd, 0xdd9, 0xdba, 0xdd4, 0xdad, 0xdd2, 0xdba, 0xdcf, 0xdb1,
0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc4, 0xdc0, 0xdcf, 0xdba, 0xdd2,
0x2d, 0xd85, 0xdbd, 0xdd9, 0xdba, 0xdd4, 0xdad, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdca,
0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdc4, 0xdc0, 0xdcf, 0xdba, 0xdd2, 0x2d, 0xd85, 0xdbd, 0xdd9, 0xdba, 0xdd4, 0xdad,
0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdca, 0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf, 0xd86, 0xdbd,
0xddd, 0xd9a, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc4, 0xddc, 0xd82, 0xd9a,
0xddc, 0xd82, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc4, 0xddc, 0xd82, 0xd9a,
0xddc, 0xd82, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdc4, 0xddc, 0xd82, 0xd9a, 0xddc, 0xd82, 0x20, 0xd9c, 0xdca, 0x200d,
0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc4,
0xddc, 0xdc0, 0xdca, 0xda9, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc4,
0xddc, 0xdc0, 0xdca, 0xda9, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc4, 0xddc, 0xdc0, 0xdca, 0xda9, 0xdca, 0x20,
0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xd89, 0xdb1, 0xdca, 0xdaf, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd89, 0xdb1, 0xdca, 0xdaf, 0xdd2, 0xdba, 0xdb1,
0xdca, 0x20, 0xdc3, 0xdcf, 0xd9c, 0xdbb, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xd89, 0xdb1, 0xdca, 0xdaf, 0xdd4, 0xda0, 0xdd3, 0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdb8, 0xdb0, 0xdca, 0x200d, 0xdba, 0xdb8, 0x20, 0xd89, 0xdb1, 0xdca,
0xdaf, 0xdd4, 0xdb1, 0xdd3, 0xdc3, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb,
0x20, 0xd89, 0xdb1, 0xdca, 0xdaf, 0xdd4, 0xdb1, 0xdd3, 0xdc3, 0xdd2, 0xdba, 0xdcf,
0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xda7, 0xdc4, 0xdd2,
0xdbb, 0x20, 0xd89, 0xdb1, 0xdca, 0xdaf, 0xdd4, 0xdb1, 0xdd3, 0xdc3, 0xdd2, 0xdba,
0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd89, 0xdbb, 0xdcf,
0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd89, 0xdbb, 0xdcf, 0xdb1, 0x20,
0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd89,
0xdbb, 0xdcf, 0xdb1, 0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf, 0x20, 0xd9a, 0xdcf, 0xdbd,
0xdba, 0xd89, 0xdbb, 0xdca, 0xd9a, 0xdd4, 0xda7, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd89, 0xdbb, 0xdca, 0xd9a, 0xdd4, 0xda7,
0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd89, 0xdbb, 0xdca, 0xd9a, 0xdd4, 0xda7, 0xdca,
0xdc3, 0xdca, 0xd9a, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca,
0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8a, 0xdc1, 0xdca, 0x200d, 0xdbb,
0xdcf, 0xdba, 0xdbd, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8a, 0xdc1, 0xdca,
0x200d, 0xdbb, 0xdcf, 0xdba, 0xdbd, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8a, 0xdc1, 0xdca, 0x200d, 0xdbb, 0xdcf, 0xdba,
0xdbd, 0x20, 0xdaf, 0xdc4, 0xdc0, 0xdbd, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xda2, 0xdb4, 0xdcf, 0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda2,
0xdb4, 0xdcf, 0xdb1, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xda2, 0xdb4, 0xdcf, 0xdb1, 0x20, 0xdaf, 0xdc4, 0xdc0, 0xdbd,
0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdc3, 0xd9a, 0xdc3, 0xdca,
0xdad, 0xdcf, 0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd0, 0xd9c,
0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd9a, 0xdc3, 0xd9a, 0xdc3, 0xdca, 0xdad,
0xdcf, 0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xda7, 0xdc4, 0xdd2,
0xdbb, 0x20, 0xd9a, 0xdc3, 0xd9a, 0xdc3, 0xdca, 0xdad, 0xdcf, 0xdb1, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xddc, 0xdbb, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xddc, 0xdbb, 0xdd2, 0xdba, 0xdcf,
0xdb1, 0xdd4, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xd9a, 0xddc, 0xdbb, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdaf,
0xdc4, 0xdc0, 0xdbd, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xddc,
0xdc3, 0xdca, 0xdbb, 0xdda, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdca,
0x200d, 0xdbb, 0xdc3, 0xdca, 0xdb1, 0xddc, 0xdba, 0xdcf, 0xdbb, 0xdca, 0xdc3, 0xdca,
0xd9a, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdca, 0x200d, 0xdbb,
0xdc3, 0xdca, 0xdb1, 0xddc, 0xdba, 0xdcf, 0xdbb, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca,
0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xd9a, 0xdca, 0x200d, 0xdbb, 0xdc3, 0xdca, 0xdb1, 0xddc, 0xdba, 0xdcf, 0xdbb, 0xdca,
0xdc3, 0xdca, 0xd9a, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca,
0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdd2, 0xdbb, 0xdca, 0xd9c,
0xdd2, 0xdc3, 0xdca, 0xdad, 0xdcf, 0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdc1, 0xdca, 0x200d, 0xdbb, 0xdd3, 0x20, 0xdbd, 0xd82, 0xd9a, 0xdcf, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdbd, 0xdba, 0xdd2, 0xdb1, 0xdca, 0x20, 0xdaf, 0xdd4,
0xdb4, 0xdad, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdbd, 0xddd, 0xdbb,
0xdca, 0xda9, 0xdca, 0x20, 0xdc4, 0xddd, 0xdc0, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdbd, 0xddd, 0xdbb, 0xdca, 0xda9, 0xdca, 0x20, 0xdc4, 0xddd, 0xdc0,
0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdbd, 0xddd, 0xdbb, 0xdca, 0xda9, 0xdca, 0x20, 0xdc4, 0xddd, 0xdc0, 0xdca,
0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8,
0xdd9, 0xd9c, 0xda9, 0xdb1, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8,
0xdd9, 0xd9c, 0xda9, 0xdb1, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdd9, 0xd9c, 0xda9, 0xdb1, 0xdca, 0x20,
0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdb8, 0xdd0, 0xdbd, 0xdda, 0xdc3, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdcf, 0xdbd, 0xdaf, 0xdd2, 0xdc0,
0xdba, 0xdd2, 0xdb1, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdcf,
0xdbb, 0xdca, 0xd9a, 0xdd4, 0xd91, 0xdc3, 0xdcf, 0xdc3, 0xdca, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdcf, 0xdbb, 0xdca, 0xdc2, 0xdbd, 0xdca, 0x20, 0xdaf,
0xdd4, 0xdb4, 0xdad, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdd4,
0xdbb, 0xdd4, 0xdc3, 0xdd2, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdd4,
0xdbb, 0xdd4, 0xdc3, 0xdd2, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdd4, 0xdbb, 0xdd4, 0xdc3, 0xdd2, 0x20, 0xd9c,
0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba,
0xdb8, 0xddc, 0xdc0, 0xdca, 0xdc3, 0xdb1, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdb8, 0xdd9, 0xd9a, 0xdca, 0xdc3, 0xdd2, 0xd9a, 0xddd, 0x20, 0xdb4, 0xdd0,
0xdc3, 0xdd2, 0xdc6, 0xdd2, 0xd9a, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdb8, 0xdd9, 0xd9a, 0xdca, 0xdc3, 0xdd2, 0xd9a, 0xddd, 0x20, 0xdb4, 0xdd0, 0xdc3,
0xdd2, 0xdc6, 0xdd2, 0xd9a, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdd9, 0xd9a, 0xdca, 0xdc3, 0xdd2, 0xd9a,
0xddd, 0x20, 0xdb4, 0xdd0, 0xdc3, 0xdd2, 0xdc6, 0xdd2, 0xd9a, 0xdca, 0x20, 0xdaf,
0xdd2, 0xdc0, 0xdcf, 0xd86, 0xdbd, 0xddd, 0xd9a, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xd8b, 0xdbd, 0xdcf, 0xdb1, 0xdca, 0x20, 0xdb6, 0xdcf, 0xda7, 0xdbb, 0xdca,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdbd, 0xdcf, 0xdb1, 0xdca, 0x20,
0xdb6, 0xdcf, 0xda7, 0xdbb, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdbd, 0xdcf, 0xdb1, 0xdca, 0x20, 0xdb6,
0xdcf, 0xda7, 0xdbb, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca,
0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xddc, 0xdc3, 0xdca, 0xd9a,
0xdc0, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xddc, 0xdc3, 0xdca,
0xd9a, 0xdc0, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xddc, 0xdc3, 0xdca, 0xd9a, 0xdc0, 0xdca, 0x20, 0xd9c,
0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdb8, 0xdd2, 0xdba, 0xdb1, 0xdca, 0xdb8, 0xdcf, 0xdbb, 0xdca, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdcf, 0xdc0, 0xdd4, 0xdbb, 0xdd4, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdda, 0xdb4, 0xdcf, 0xdbd, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdc0, 0x20, 0xdc3, 0xdd9, 0xdbd, 0xda9, 0xddc, 0xdb1,
0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1,
0xdc0, 0x20, 0xdc3, 0xdd9, 0xdbd, 0xda9, 0xddc, 0xdb1, 0xdd2, 0xdba, 0xdcf, 0xdb1,
0xdd4, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdb1, 0xdc0, 0x20, 0xdc3, 0xdd9, 0xdbd, 0xda9, 0xddc, 0xdb1, 0xdd2, 0xdba,
0xdcf, 0xdb1, 0xdd4, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdc0, 0xdc3, 0xdd3, 0xdbd, 0xdb1,
0xdca, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdc0, 0xdc3, 0xdd3,
0xdbd, 0xdb1, 0xdca, 0xdad, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdc0, 0xdc3, 0xdd3, 0xdbd, 0xdb1, 0xdca, 0xdad,
0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1,
0xdd2, 0xdc0, 0xdca, 0xdc6, 0xdc0, 0xdd4, 0xdb1, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xdad,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd2, 0xdc0, 0xdca, 0xdc6, 0xdc0,
0xdd4, 0xdb1, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xdad, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8,
0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd2, 0xdc0, 0xdca, 0xdc6,
0xdc0, 0xdd4, 0xdb1, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xdad, 0x20, 0xdaf, 0xdd2, 0xdc0,
0xdcf, 0xd86, 0xdbd, 0xddd, 0xd9a, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1,
0xdd2, 0xdba, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xddc, 0xdc6,
0xddc, 0xdbd, 0xdca, 0xd9a, 0xdca, 0x20, 0xdaf, 0xdd4, 0xdb4, 0xdad, 0xdca, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xddc, 0xdc6, 0xddc, 0xdbd, 0xdca, 0xd9a,
0xdca, 0x20, 0xdaf, 0xdd4, 0xdb4, 0xdad, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8,
0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xddc, 0xdc6, 0xddc, 0xdbd,
0xdca, 0xd9a, 0xdca, 0x20, 0xdaf, 0xdd4, 0xdb4, 0xdad, 0xdca, 0x20, 0xd9c, 0xdca,
0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdc6, 0xdbb, 0xdca, 0xdb1, 0xdd0, 0xdb1, 0xdca, 0xda9, 0xddd, 0x20, 0xda9, 0xdd2,
0x20, 0xdb1, 0xddc, 0xdbb, 0xddc, 0xdb1, 0xdca, 0xdc4, 0xdcf, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdc6, 0xdbb, 0xdca, 0xdb1, 0xdd0, 0xdb1, 0xdca, 0xda9, 0xddd,
0x20, 0xda9, 0xdd2, 0x20, 0xdb1, 0xddc, 0xdbb, 0xddc, 0xdb1, 0xdca, 0xdc4, 0xdcf,
0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdc6, 0xdbb, 0xdca, 0xdb1, 0xdd0, 0xdb1, 0xdca, 0xda9, 0xddd, 0x20, 0xda9, 0xdd2,
0x20, 0xdb1, 0xddc, 0xdbb, 0xddc, 0xdb1, 0xdca, 0xdc4, 0xdcf, 0x20, 0xd9c, 0xdca,
0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xdb1,
0xddc, 0xdc0, 0xdc3, 0xdd2, 0xdb6, 0xdd2, 0xdbb, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xddc, 0xdc0, 0xdc3, 0xdd2, 0xdb6,
0xdd2, 0xdbb, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8,
0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xddc, 0xdc0, 0xdc3, 0xdd2,
0xdb6, 0xdd2, 0xdbb, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d,
0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd94,
0xdb8, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xd94, 0xdb8, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8,
0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd94, 0xdb8, 0xdca, 0xdc3, 0xdca,
0xd9a, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb4, 0xdcf, 0xd9a, 0xdd2, 0xdc3, 0xdca, 0xdae,
0xdcf, 0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb4, 0xdcf, 0xd9a, 0xdd2,
0xdc3, 0xdca, 0xdae, 0xdcf, 0xdb1, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb4, 0xdcf, 0xd9a, 0xdd2, 0xdc3, 0xdca, 0xdae,
0xdcf, 0xdb1, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20,
0xd9a, 0xdcf, 0xdbd, 0xdba, 0xdb4, 0xdbd, 0xdcf, 0xdc0, 0xdd4, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdb4, 0xdd0, 0xdb4, 0xdd4, 0xdc0, 0xdcf, 0x20, 0xdb1, 0xdd2,
0xdc0, 0xdca, 0x20, 0xd9c, 0xdd2, 0xdb1, 0xdd3, 0xdba, 0xdcf, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdb4, 0xdd0, 0xdbb, 0xd9c, 0xdd4, 0xdc0, 0xdda, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb4, 0xdd0, 0xdbb, 0xd9c, 0xdd4, 0xdc0, 0xdda, 0x20,
0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb4,
0xdd0, 0xdbb, 0xd9c, 0xdd4, 0xdc0, 0xdda, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3,
0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xdb4, 0xdda, 0xdbb, 0xdd4,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb4, 0xdda, 0xdbb, 0xdd4, 0x20, 0xdc3,
0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb4, 0xdda,
0xdbb, 0xdd4, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20,
0xd9a, 0xdcf, 0xdbd, 0xdba, 0xdb4, 0xdd2, 0xdbd, 0xdd2, 0xdb4, 0xdd3, 0xdb1, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb4, 0xdd2, 0xdbd, 0xdd2, 0xdb4, 0xdd3, 0xdb1,
0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdb4, 0xdd2, 0xdbd, 0xdd2, 0xdb4, 0xdd3, 0xdb1, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb,
0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc6, 0xdd3,
0xdb1, 0xdd2, 0xd9a, 0xdca, 0xdc3, 0xdca, 0x20, 0xdaf, 0xdd4, 0xdb4, 0xdad, 0xdca,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc1, 0xdcf, 0xdb1, 0xdca, 0xdad, 0x20,
0xdb4, 0xdd2, 0xdba, 0xdbb, 0xdda, 0x20, 0xdc3, 0xdc4, 0x20, 0xdb8, 0xdd0, 0xd9a,
0xddd, 0xdbd, 0xdb1, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc1, 0xdcf,
0xdb1, 0xdca, 0xdad, 0x20, 0xdb4, 0xdd2, 0xdba, 0xdbb, 0xdda, 0x20, 0xdc3, 0xdc4,
0x20, 0xdb8, 0xdd0, 0xd9a, 0xddd, 0xdbd, 0xdb1, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca,
0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc1, 0xdcf, 0xdb1, 0xdca,
0xdad, 0x20, 0xdb4, 0xdd2, 0xdba, 0xdbb, 0xdda, 0x20, 0xdc3, 0xdc4, 0x20, 0xdb8,
0xdd0, 0xd9a, 0xddd, 0xdbd, 0xdb1, 0xdca, 0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf, 0xd86,
0xdbd, 0xddd, 0xd9a, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb4, 0xdd2, 0xda7,
0xdca, 0xd9a, 0xdd9, 0xdba, 0xdcf, 0xdb1, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdb4, 0xddc, 0xdb1, 0xdb4, 0xdda, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdb4, 0xdca, 0xdba, 0xddc, 0xdb1, 0xdca, 0xd9c, 0xdca, 0xdba, 0xdb1, 0xdca, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdbb, 0xdd2, 0xdba, 0xdd4, 0xdb1, 0xdd2, 0xdba,
0xdb1, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdbb, 0xddc, 0xdad, 0xdd9,
0xdbb, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc3, 0xd9b, 0xdbd, 0xdd2,
0xdb1, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc3, 0xd9b, 0xdbd, 0xdd2,
0xdb1, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdc3, 0xd9b, 0xdbd, 0xdd2, 0xdb1, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d,
0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc3,
0xdd0, 0xdb8, 0xddd, 0xdc0, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc3,
0xdd0, 0xdb8, 0xddd, 0xdc0, 0xdcf, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc3, 0xdd0, 0xdb8, 0xddd, 0xdc0, 0xdcf, 0x20,
0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdc3, 0xdd3, 0xdc2, 0xdd9, 0xdbd, 0xdca, 0xdc3, 0xdca, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc3, 0xdd2, 0xd82, 0xd9c, 0xdb4, 0xdca, 0xdb4, 0xdd4,
0xdbb, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc3, 0xddc, 0xdbd, 0xddc,
0xdb8, 0xdb1, 0xdca, 0x20, 0xdaf, 0xdd6, 0xdb4, 0xdad, 0xdca, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdaf, 0xd9a, 0xdd4, 0xdab, 0xdd4, 0x20, 0xda2, 0xddd, 0xda2,
0xdd2, 0xdba, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc3, 0xdd4, 0xdbb,
0xdd2, 0xdb1, 0xdcf, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc3, 0xdca,
0xdba, 0xddd, 0xdc0, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda7, 0xdcf,
0xdc4, 0xdd2, 0xda7, 0xdd2, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdad, 0xdcf,
0xdba, 0xdd2, 0xdb4, 0xdda, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdad, 0xdcf,
0xdba, 0xdd2, 0xdb4, 0xdda, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdad, 0xdcf, 0xdba, 0xdd2, 0xdb4, 0xdda, 0x20, 0xdaf,
0xdc4, 0xdc0, 0xdbd, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda7, 0xda2,
0xdd2, 0xd9a, 0xdd2, 0xdc3, 0xdca, 0xdad, 0xdcf, 0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xda7, 0xddc, 0xd9a, 0xdd9, 0xdbd, 0xdcf, 0xdc0, 0xdd4, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda7, 0xddc, 0xd82, 0xd9c, 0xdcf, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xda7, 0xddc, 0xd82, 0xd9c, 0xdcf, 0x20, 0xdc3, 0xdb8, 0xdca,
0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda7, 0xddc, 0xd82, 0xd9c,
0xdcf, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda0, 0xdd4, 0xd9a, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xda7, 0xdbb, 0xdca, 0xd9a, 0xdca, 0xdb8, 0xdd9, 0xdb1, 0xdd2, 0xdc3,
0xdca, 0xdad, 0xdcf, 0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda7, 0xdbb,
0xdca, 0xd9a, 0xdca, 0xdb8, 0xdd9, 0xdb1, 0xdd2, 0xdc3, 0xdca, 0xdad, 0xdcf, 0xdb1,
0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xda7, 0xdbb, 0xdca, 0xd9a, 0xdca, 0xdb8, 0xdd9, 0xdb1, 0xdd2, 0xdc3, 0xdca, 0xdad,
0xdcf, 0xdb1, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda7, 0xdd4, 0xdc0, 0xdcf, 0xdbd, 0xdd4, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdbb, 0xdd4, 0xd9c, 0xdd4, 0xdc0, 0xdda,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdbb, 0xdd4, 0xd9c, 0xdd4, 0xdc0,
0xdda, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xd8b, 0xdbb, 0xdd4, 0xd9c, 0xdd4, 0xdc0, 0xdda, 0x20, 0xd9c, 0xdca, 0x200d,
0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xd8b, 0xdc3,
0xdca, 0xdb6, 0xdd9, 0xd9a, 0xdd2, 0xdc3, 0xdca, 0xdad, 0xdcf, 0xdb1, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdc3, 0xdca, 0xdb6, 0xdd9, 0xd9a, 0xdd2, 0xdc3,
0xdca, 0xdad, 0xdcf, 0xdb1, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdc3, 0xdca, 0xdb6, 0xdd9, 0xd9a, 0xdd2, 0xdc3,
0xdca, 0xdad, 0xdcf, 0xdb1, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca,
0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc0, 0xdb1, 0xdd4, 0xd86, 0xda7,
0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc0, 0xdb1, 0xdd4, 0xd86, 0xda7,
0xdd4, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdc0, 0xdb1, 0xdd4, 0xd86, 0xda7, 0xdd4, 0x20, 0xd9c, 0xdd2, 0xdb8, 0xdca,
0xdc4, 0xdcf, 0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc0, 0xdd9, 0xdb1,
0xdd2, 0xdc3, 0xdd2, 0xdba, 0xdd4, 0xdbd, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdc0, 0xdca, 0xdbd, 0xdaf, 0xdd2, 0xdc0, 0xddd, 0xdc3, 0xdca, 0xda7, 0xddd,
0xd9a, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc0, 0xdca, 0xdbd, 0xdaf,
0xdd2, 0xdc0, 0xddd, 0xdc3, 0xdca, 0xda7, 0xddd, 0xd9a, 0xdca, 0x20, 0xdc3, 0xdb8,
0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc0, 0xdca, 0xdbd,
0xdaf, 0xdd2, 0xdc0, 0xddd, 0xdc3, 0xdca, 0xda7, 0xddd, 0xd9a, 0xdca, 0x20, 0xd9c,
0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdc0, 0xddc, 0xdbd, 0xdca, 0xd9c, 0xddc, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdcf,
0xda9, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc0, 0xddc, 0xdbd, 0xdca,
0xd9c, 0xddc, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdcf, 0xda9, 0xdca, 0x20, 0xdc3, 0xdb8,
0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc0, 0xddc, 0xdbd,
0xdca, 0xd9c, 0xddc, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdcf, 0xda9, 0xdca, 0x20, 0xd9c,
0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdc0, 0xddc, 0xdc3, 0xdca, 0xda7, 0xddc, 0xd9a, 0xdca, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdc0, 0xdda, 0xd9a, 0xdca, 0x20, 0xdaf, 0xdd6, 0xdb4, 0xdad,
0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc0, 0xdd0, 0xdbd, 0xdd2, 0xdc3,
0xdca, 0x20, 0xdc3, 0xdc4, 0x20, 0xdc6, 0xdd4, 0xda7, 0xdd4, 0xdb1, 0xdcf, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdba, 0xd9a, 0xdd4, 0xda7, 0xdca, 0xdc3, 0xdca,
0xd9a, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdba, 0xd9a, 0xdd4, 0xda7,
0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdba, 0xd9a, 0xdd4, 0xda7, 0xdca, 0xdc3, 0xdca,
0xd9a, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdba, 0xdd9, 0xd9a, 0xda7, 0xdd9, 0xdbb, 0xdd2,
0xdb1, 0xdca, 0xdb6, 0xdbb, 0xdca, 0xd9c, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdba, 0xdd9, 0xd9a, 0xda7, 0xdd9, 0xdbb, 0xdd2, 0xdb1, 0xdca, 0xdb6, 0xdbb,
0xdca, 0xd9c, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdba, 0xdd9, 0xd9a, 0xda7, 0xdd9, 0xdbb, 0xdd2, 0xdb1, 0xdca,
0xdb6, 0xdbb, 0xdca, 0xd9c, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2,
0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdba, 0xdd4, 0xd9a, 0xddc,
0xdb1, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0x61, 0x63, 0x72, 0x65,
0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x63, 0x72, 0x65, 0x6a,
0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x61, 0x66, 0x67, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x73, 0x74, 0x72, 0x65, 0x64, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x63,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6a, 0x75, 0x68, 0x6f, 0x61, 0x66,
0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70,
0x61, 0x64, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x161,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x63,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x61, 0x6c, 0x6a, 0x61, 0x161, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61,
0x6c, 0x6a, 0x61, 0x161, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6c, 0x6d, 0x61, 0x61, 0x74, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6c, 0x6d, 0x61, 0x61, 0x74,
0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6c, 0x6d, 0x61, 0x61, 0x74,
0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x61, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x161,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x61, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x76, 0x65,
0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0xe1, 0x6c, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73,
0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x6b, 0xfd, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0xe1, 0x6c, 0x6e, 0x79,
0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0xe1,
0x6c, 0x6e, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x6b, 0xfd, 0x20, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x76, 0xfd, 0x63, 0x68, 0x6f,
0x64, 0x6e, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x76, 0xfd, 0x63,
0x68, 0x6f, 0x64, 0x6e, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x68, 0x6f, 0x72, 0x73, 0x6b, 0xfd,
0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x68, 0x6f, 0x72, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65,
0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b, 0xfd,
0x20, 0x74, 0x69, 0x63, 0x68, 0x6f, 0x6d, 0x6f, 0x72, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x74, 0x69, 0x63, 0x68, 0x6f,
0x6d, 0x6f, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x76,
0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20,
0x74, 0x69, 0x63, 0x68, 0x6f, 0x6d, 0x6f, 0x72, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x6e, 0x61,
0x64, 0x79, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x6e,
0x61, 0x64, 0x79, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x6e,
0x61, 0x64, 0x79, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x70, 0x69, 0x6a, 0x73, 0x6b, 0xfd,
0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x61, 0x70, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6b, 0x74, 0x61,
0x75, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6b, 0x74, 0x61,
0x75, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6b, 0x74, 0x61, 0x75,
0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x61, 0x6b, 0x74, 0x6f, 0x62, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x61, 0x6b, 0x74, 0x6f, 0x62, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x61, 0x6b, 0x74, 0x6f, 0x62, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x61, 0x62, 0x73, 0x6b,
0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x61, 0x62, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x67,
0x65, 0x6e, 0x74, 0xed, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73,
0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e, 0x73, 0x6b, 0x79, 0x20,
0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e, 0x73, 0x6b,
0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x7a,
0xe1, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0xed,
0x6e, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61,
0x64, 0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e, 0x73, 0x6b,
0x79, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x72,
0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x6d, 0xe9, 0x6e,
0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x6d, 0xe9, 0x6e,
0x73, 0x6b, 0x79, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x6d, 0xe9, 0x6e, 0x73,
0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x161,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x74, 0x72,
0x65, 0x64, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x73, 0x6b,
0x79, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x74, 0x72, 0x65, 0x64, 0x6f, 0x61,
0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x73, 0x6b, 0x79, 0x20, 0x161, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x73, 0x74, 0x72, 0x65, 0x64, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0xe1,
0x6c, 0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x73, 0x74, 0x72, 0x65, 0x64, 0x6f, 0x7a, 0xe1, 0x70, 0x61,
0x64, 0x6e, 0xfd, 0x20, 0x61, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x73,
0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x74, 0x72, 0x65, 0x64, 0x6f,
0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6e, 0xfd, 0x20, 0x61, 0x75, 0x73, 0x74,
0x72, 0xe1, 0x6c, 0x73, 0x6b, 0x79, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x74, 0x72,
0x65, 0x64, 0x6f, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6e, 0xfd, 0x20, 0x61,
0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f,
0x64, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x73, 0x6b, 0x79,
0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x61,
0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x73, 0x6b, 0x79, 0x20, 0x161, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72,
0xe1, 0x6c, 0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x75, 0x73,
0x74, 0x72, 0xe1, 0x6c, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x7a,
0xe1, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c,
0x73, 0x6b, 0x79, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f,
0x61, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x73, 0x6b, 0x79, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x7a, 0x65, 0x72,
0x62, 0x61, 0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x61, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x17e, 0x61,
0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x7a, 0x65, 0x72, 0x62,
0x61, 0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x7a, 0x6f, 0x72, 0x73,
0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x7a, 0x6f, 0x72, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0xe9, 0x161, 0x73, 0x6b, 0x79, 0x20, 0x10d,
0x61, 0x73, 0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0xe9, 0x161, 0x73,
0x6b, 0x79, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0xe9, 0x161, 0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x62, 0x68, 0x75, 0x74, 0xe1, 0x6e, 0x73, 0x6b, 0x79,
0x20, 0x10d, 0x61, 0x73, 0x62, 0x6f, 0x6c, 0xed, 0x76, 0x69, 0x6a, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x72, 0x61, 0x7a, 0xed, 0x6c,
0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x72, 0x61, 0x7a, 0xed,
0x6c, 0x73, 0x6b, 0x79, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x72, 0x61, 0x7a, 0xed,
0x6c, 0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x6b, 0x61, 0x70, 0x76, 0x65, 0x72, 0x64, 0x73, 0x6b, 0xfd,
0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x6b, 0x61, 0x70, 0x76, 0x65, 0x72, 0x64, 0x73, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x63,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x63, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x69, 0x6c, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x69, 0x6c, 0x73, 0x6b, 0xfd,
0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x10d, 0x69, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0xed, 0x6e, 0x73, 0x6b,
0x79, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0xed, 0x6e, 0x73, 0x6b, 0x79, 0x20,
0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x10d, 0xed, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x56, 0x69,
0x61, 0x6e, 0x6f, 0x10d, 0x6e, 0xe9, 0x68, 0x6f, 0x20, 0x6f, 0x73, 0x74,
0x72, 0x6f, 0x76, 0x61, 0x10d, 0x61, 0x73, 0x20, 0x4b, 0x6f, 0x6b, 0x6f,
0x73, 0x6f, 0x76, 0xfd, 0x63, 0x68, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f,
0x76, 0x6f, 0x76, 0x6b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x73,
0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69,
0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x76,
0xfd, 0x63, 0x68, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x6f, 0x76,
0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x76, 0xfd, 0x63, 0x68,
0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x6f, 0x76, 0x6c, 0x65, 0x74,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f,
0x76, 0xfd, 0x63, 0x68, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x6f,
0x76, 0x6b, 0x75, 0x62, 0xe1, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61,
0x73, 0x6b, 0x75, 0x62, 0xe1, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x161, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x6b, 0x75, 0x62, 0xe1, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x44, 0x61,
0x76, 0x69, 0x73, 0x6f, 0x76, 0x65, 0x6a, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x63, 0x65, 0x10d, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x63, 0x65, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x20, 0x64,
0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x61, 0x10d, 0x61, 0x73,
0x20, 0x56, 0x65, 0x13e, 0x6b, 0x6f, 0x6e, 0x6f, 0x10d, 0x6e, 0xe9, 0x68,
0x6f, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x61, 0x161, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x20,
0x56, 0x65, 0x13e, 0x6b, 0x6f, 0x6e, 0x6f, 0x10d, 0x6e, 0xe9, 0x68, 0x6f,
0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x61, 0x6c, 0x65, 0x74, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x56, 0x65, 0x13e, 0x6b, 0x6f, 0x6e,
0x6f, 0x10d, 0x6e, 0xe9, 0x68, 0x6f, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f,
0x76, 0x61, 0x73, 0x74, 0x72, 0x65, 0x64, 0x6f, 0x65, 0x75, 0x72, 0xf3,
0x70, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x74, 0x72, 0x65,
0x64, 0x6f, 0x65, 0x75, 0x72, 0xf3, 0x70, 0x73, 0x6b, 0x79, 0x20, 0x161,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x73, 0x74, 0x72, 0x65, 0x64, 0x6f, 0x65, 0x75, 0x72, 0xf3, 0x70,
0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x65, 0x75, 0x72, 0xf3,
0x70, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68,
0x6f, 0x64, 0x6f, 0x65, 0x75, 0x72, 0xf3, 0x70, 0x73, 0x6b, 0x79, 0x20,
0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x65, 0x75, 0x72,
0xf3, 0x70, 0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x6d, 0x69, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x65, 0x75, 0x72, 0xf3, 0x70,
0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64,
0x6f, 0x65, 0x75, 0x72, 0xf3, 0x70, 0x73, 0x6b, 0x79, 0x20, 0x161, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x65, 0x75, 0x72, 0xf3, 0x70, 0x73,
0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x66, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0xfd, 0x20,
0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x66, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x66,
0x69, 0x64, 0x17e, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x66,
0x69, 0x64, 0x17e, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x72, 0x61, 0x6e, 0x63, 0xfa,
0x7a, 0x73, 0x6b, 0x6f, 0x67, 0x75, 0x79, 0x61, 0x6e, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x46, 0x72, 0x61, 0x6e,
0x63, 0xfa, 0x7a, 0x73, 0x6b, 0x79, 0x63, 0x68, 0x20, 0x6a, 0x75, 0x17e,
0x6e, 0xfd, 0x63, 0x68, 0x20, 0x61, 0x20, 0x61, 0x6e, 0x74, 0x61, 0x72,
0x6b, 0x74, 0x69, 0x63, 0x6b, 0xfd, 0x63, 0x68, 0x20, 0xfa, 0x7a, 0x65,
0x6d, 0xed, 0x67, 0x61, 0x6c, 0x61, 0x70, 0xe1, 0x67, 0x73, 0x6b, 0x79,
0x20, 0x10d, 0x61, 0x73, 0x67, 0x72, 0x75, 0x7a, 0xed, 0x6e, 0x73, 0x6b,
0x79, 0x20, 0x10d, 0x61, 0x73, 0x67, 0x72, 0x75, 0x7a, 0xed, 0x6e, 0x73,
0x6b, 0x79, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x67, 0x72, 0x75, 0x7a, 0xed, 0x6e, 0x73,
0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x10d, 0x61, 0x73, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x6f,
0x76, 0xfd, 0x63, 0x68, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x6f,
0x76, 0x67, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f,
0x67, 0x72, 0xf3, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x76,
0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x67, 0x72, 0xf3, 0x6e, 0x73, 0x6b,
0x79, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x67,
0x72, 0xf3, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x67, 0x72,
0xf3, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70,
0x61, 0x64, 0x6f, 0x67, 0x72, 0xf3, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x161,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x67, 0x72, 0xf3, 0x6e, 0x73,
0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x67, 0x75, 0x61, 0x6d, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x161,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x20, 0x50, 0x65, 0x72, 0x7a, 0x73, 0x6b, 0xe9, 0x68, 0x6f, 0x20,
0x7a, 0xe1, 0x6c, 0x69, 0x76, 0x75, 0x68, 0x61, 0x76, 0x61, 0x6a, 0x73,
0x6b, 0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0xfd, 0x20,
0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x68, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x2d, 0x61,
0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x68, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x68, 0x6f, 0x6e, 0x67, 0x6b,
0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x68, 0x6f, 0x6e, 0x67,
0x6b, 0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x6f, 0x76, 0x64, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x6f, 0x76, 0x64, 0x73, 0x6b, 0xfd,
0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x63, 0x68, 0x6f, 0x76, 0x64, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x6e, 0x64,
0x69, 0x63, 0x6b, 0x6f, 0x6f, 0x63, 0x65, 0xe1, 0x6e, 0x73, 0x6b, 0x79,
0x20, 0x10d, 0x61, 0x73, 0x69, 0x6e, 0x64, 0x6f, 0x10d, 0xed, 0x6e, 0x73,
0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x74, 0x72, 0x65, 0x64, 0x6f,
0x69, 0x6e, 0x64, 0x6f, 0x6e, 0xe9, 0x7a, 0x73, 0x6b, 0x79, 0x20, 0x10d,
0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x69, 0x6e, 0x64,
0x6f, 0x6e, 0xe9, 0x7a, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x7a,
0xe1, 0x70, 0x61, 0x64, 0x6f, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0xe9, 0x7a,
0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x72, 0xe1, 0x6e, 0x73,
0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x72, 0xe1, 0x6e, 0x73, 0x6b,
0x79, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x69, 0x72, 0xe1, 0x6e, 0x73, 0x6b, 0x79, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x69, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0xfd, 0x20,
0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x69, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6a, 0x61, 0x70,
0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6a, 0x61, 0x70, 0x6f,
0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61,
0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x10d, 0x61,
0x74, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0x10d, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x61, 0x7a, 0x61,
0x63, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x6b, 0x61, 0x7a, 0x61,
0x63, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x6b, 0x61, 0x7a, 0x61, 0x63,
0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x6b, 0xf3, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x6b, 0xf3, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b,
0xf3, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a,
0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x69, 0x72, 0x67, 0x69, 0x7a,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x72, 0xed, 0x6c, 0x61,
0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20,
0x52, 0x6f, 0x76, 0x6e, 0xed, 0x6b, 0x6f, 0x76, 0xfd, 0x63, 0x68, 0x20,
0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x6f, 0x76, 0x161, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x6f,
0x73, 0x74, 0x72, 0x6f, 0x76, 0x61, 0x20, 0x6c, 0x6f, 0x72, 0x64, 0x61,
0x20, 0x48, 0x6f, 0x77, 0x61, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x61, 0x20, 0x6c,
0x6f, 0x72, 0x64, 0x61, 0x20, 0x48, 0x6f, 0x77, 0x61, 0x6d, 0x61, 0x63,
0x61, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61, 0x63,
0x61, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61, 0x63, 0x61,
0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x6d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x73, 0x6b, 0xfd,
0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x6d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x73, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d,
0x61, 0x6c, 0x61, 0x6a, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x6d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x6d, 0x61, 0x72, 0x6b, 0xe9, 0x7a, 0x73, 0x6b, 0x79,
0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x73,
0x68, 0x61, 0x6c, 0x6c, 0x6f, 0x76, 0xfd, 0x63, 0x68, 0x20, 0x6f, 0x73,
0x74, 0x72, 0x6f, 0x76, 0x6f, 0x76, 0x6d, 0x61, 0x75, 0x72, 0xed, 0x63,
0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61, 0x75,
0x72, 0xed, 0x63, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d,
0x61, 0x75, 0x72, 0xed, 0x63, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20,
0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x6f, 0x76, 0x65, 0x6a, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x63, 0x65, 0x6d, 0x65, 0x78, 0x69, 0x63, 0x6b,
0xfd, 0x20, 0x74, 0x69, 0x63, 0x68, 0x6f, 0x6d, 0x6f, 0x72, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x65, 0x78, 0x69, 0x63, 0x6b, 0xfd,
0x20, 0x74, 0x69, 0x63, 0x68, 0x6f, 0x6d, 0x6f, 0x72, 0x73, 0x6b, 0xfd,
0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x6d, 0x65, 0x78, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x74,
0x69, 0x63, 0x68, 0x6f, 0x6d, 0x6f, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x75, 0x6c, 0x61, 0x6e,
0x62, 0xe1, 0x74, 0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x75, 0x6c, 0x61, 0x6e, 0x62, 0xe1, 0x74, 0x61, 0x72, 0x73, 0x6b, 0xfd,
0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x75, 0x6c, 0x61, 0x6e, 0x62, 0xe1, 0x74, 0x61, 0x72,
0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x6d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x6d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x73, 0x6b, 0xfd, 0x20,
0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x6d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x6a, 0x61,
0x6e, 0x6d, 0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6e,
0x61, 0x75, 0x72, 0x75, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6e,
0x65, 0x70, 0xe1, 0x6c, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x6e,
0x6f, 0x76, 0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0xf3, 0x6e, 0x73, 0x6b,
0x79, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x76, 0x6f, 0x6b, 0x61, 0x6c,
0x65, 0x64, 0xf3, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x161, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f,
0x76, 0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0xf3, 0x6e, 0x73, 0x6b, 0x79,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f,
0x76, 0x6f, 0x7a, 0xe9, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0xfd, 0x20,
0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x6e, 0x6f, 0x76, 0x6f, 0x7a, 0xe9, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x6e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x65, 0x77, 0x66, 0x6f,
0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x73, 0xfa, 0x6f, 0x73,
0x74, 0x72, 0x6f, 0x76, 0x69, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x20, 0x73, 0xfa, 0x6f, 0x73, 0x74, 0x72, 0x6f,
0x76, 0x69, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x6c,
0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x73, 0xfa, 0x6f,
0x73, 0x74, 0x72, 0x6f, 0x76, 0x69, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x6d, 0x61, 0x72,
0x69, 0xe1, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x161,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x6e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6f,
0x6d, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6f, 0x6d, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x70, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0xfd,
0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x70, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x10d, 0x61, 0x73, 0x20, 0x50, 0x61, 0x70, 0x75, 0x79, 0x2d, 0x4e, 0x6f,
0x76, 0x65, 0x6a, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x79, 0x70, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x70, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x6a, 0x73, 0x6b, 0xfd,
0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x70, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x6a, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x70, 0x65, 0x72, 0x75, 0xe1, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61,
0x73, 0x70, 0x65, 0x72, 0x75, 0xe1, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x161,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x70, 0x65, 0x72, 0x75, 0xe1, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x69, 0x6c, 0x69,
0x70, 0xed, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x69,
0x6c, 0x69, 0x70, 0xed, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x161, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x66,
0x69, 0x6c, 0x69, 0x70, 0xed, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x46,
0xe9, 0x6e, 0x69, 0x78, 0x6f, 0x76, 0xfd, 0x63, 0x68, 0x20, 0x6f, 0x73,
0x74, 0x72, 0x6f, 0x76, 0x6f, 0x76, 0x70, 0x69, 0x65, 0x72, 0x72, 0x65,
0x2d, 0x6d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x73, 0x6b, 0xfd,
0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x70, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x6d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x50,
0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x6f, 0x76, 0xfd, 0x63, 0x68,
0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x6f, 0x76, 0x70, 0x63, 0x68,
0x6a, 0x6f, 0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x6b, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72,
0x64, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x79, 0x7a, 0x79, 0x6c,
0x6f, 0x72, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x52, 0x6f, 0x74, 0x68,
0x65, 0x72, 0x6f, 0x76, 0x65, 0x6a, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x63, 0x65, 0x73, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73, 0x6b,
0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x53, 0x61, 0x6d, 0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x6d, 0x6f, 0x6a, 0x73,
0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x6d, 0x6f, 0x6a, 0x73, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73,
0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x161,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x10d, 0x61, 0x73, 0x20, 0x160, 0x61, 0x6c, 0x61, 0x6d, 0xfa, 0x6e,
0x6f, 0x76, 0xfd, 0x63, 0x68, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76,
0x6f, 0x76, 0x10d, 0x61, 0x73, 0x20, 0x4a, 0x75, 0x17e, 0x6e, 0x65, 0x6a,
0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x74, 0x63, 0x68, 0x61,
0x6a, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x63,
0x68, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x61, 0x64, 0x17e, 0x69,
0x63, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x6f, 0x6e, 0x17e, 0x73,
0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x6f, 0x6e, 0x17e, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x75,
0x72, 0x6b, 0x6d, 0xe9, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73,
0x74, 0x75, 0x72, 0x6b, 0x6d, 0xe9, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x161,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0xe9, 0x6e, 0x73, 0x6b, 0x79, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x75, 0x72, 0x75,
0x67, 0x75, 0x61, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x75,
0x72, 0x75, 0x67, 0x75, 0x61, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x75, 0x72, 0x75, 0x67, 0x75, 0x61, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x75, 0x7a, 0x62, 0x65,
0x63, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x75, 0x7a, 0x62, 0x65, 0x63, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x76,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x76,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x76, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x63, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x76, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x63, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x76, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x76, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61,
0x73, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x6f, 0x76, 0x20, 0x57,
0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x61, 0x20, 0x46, 0x75, 0x74, 0x75,
0x6e, 0x61, 0x6a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x161,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x6a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6a, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x6a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6a, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x66, 0x67,
0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x61,
0x66, 0x72, 0x69, 0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56, 0x7a,
0x68, 0x6f, 0x64, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x161, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x4a, 0x75, 0x17e, 0x6e, 0x6f, 0x61, 0x66, 0x72,
0x69, 0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x5a, 0x61, 0x68, 0x6f,
0x64, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x161, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x5a, 0x61, 0x68, 0x6f, 0x64, 0x6e, 0x6f, 0x61, 0x66, 0x72,
0x69, 0x161, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x5a, 0x61, 0x68, 0x6f, 0x64,
0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x161, 0x6b, 0x69, 0x20, 0x70, 0x6f,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x6c, 0x6a,
0x61, 0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x6c, 0x6a, 0x61,
0x161, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x6c, 0x6a, 0x61, 0x161, 0x6b,
0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x73, 0x6b, 0x69, 0x20,
0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x69, 0x20, 0x70, 0x6f,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56, 0x7a, 0x68,
0x6f, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56, 0x7a, 0x68, 0x6f,
0x64, 0x6e, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56, 0x7a, 0x68, 0x6f, 0x64, 0x6e,
0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x47, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x47,
0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x47, 0x6f, 0x72, 0x73,
0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x161, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x161, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x161, 0x6b, 0x69,
0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x70,
0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c, 0x61,
0x73, 0x3a, 0x20, 0x41, 0x70, 0x69, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x3a, 0x20, 0x41,
0x70, 0x69, 0x61, 0x50, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x3a, 0x20, 0x41, 0x70, 0x69, 0x61, 0x41, 0x72, 0x61, 0x62,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x72, 0x61, 0x62, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x72, 0x61, 0x62, 0x73, 0x6b, 0x69,
0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x7a, 0x61, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73,
0x6b, 0x69, 0x20, 0x7a, 0x61, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x7a, 0x61, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20, 0x70, 0x6f, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x72, 0x6d, 0x65, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x73, 0x6b,
0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x69, 0x20, 0x70, 0x6f, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x76, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x68, 0x6f, 0x64, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x69,
0x20, 0x7a, 0x61, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41,
0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x68, 0x6f,
0x64, 0x6e, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b,
0x69, 0x20, 0x76, 0x7a, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20,
0x76, 0x7a, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x76,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x76, 0x7a, 0x68,
0x6f, 0x64, 0x6e, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x6b, 0x69, 0x20, 0x7a, 0x61, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69,
0x20, 0x7a, 0x61, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41,
0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x7a, 0x61,
0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a,
0x64, 0x17e, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41,
0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64,
0x17e, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x7a, 0x6f, 0x72, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x7a, 0x6f, 0x72, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x41, 0x7a, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x70,
0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161,
0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x42, 0x75, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x73,
0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x4b, 0x61, 0x70, 0x76, 0x65, 0x72, 0x64, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x61, 0x70, 0x76, 0x65, 0x72, 0x64,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x61, 0x70, 0x76, 0x65, 0x72,
0x64, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x10c, 0x61, 0x6d, 0x6f, 0x72, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x10c, 0x61, 0x74, 0x61, 0x6d, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x10c, 0x61, 0x74, 0x61, 0x6d, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x10c, 0x61, 0x74, 0x61, 0x6d, 0x73, 0x6b, 0x69, 0x20, 0x70,
0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c, 0x69,
0x6c, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c, 0x69, 0x6c, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c, 0x69, 0x6c, 0x73, 0x6b, 0x69, 0x20,
0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b,
0x69, 0x74, 0x61, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b,
0x69, 0x74, 0x61, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x69,
0x74, 0x61, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x42, 0x6f, 0x17e, 0x69, 0x10d, 0x6e,
0x6f, 0x6f, 0x74, 0x6f, 0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c,
0x61, 0x73, 0x3a, 0x20, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x6f, 0x76, 0x69,
0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x69, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62,
0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x6f, 0x6c,
0x75, 0x6d, 0x62, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b,
0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x70,
0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x43, 0x6f,
0x6f, 0x6b, 0x6f, 0x76, 0x6f, 0x6f, 0x74, 0x6f, 0x161, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x76, 0x6f, 0x6f, 0x74,
0x6f, 0x161, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x43, 0x6f, 0x6f, 0x6b, 0x6f,
0x76, 0x6f, 0x6f, 0x74, 0x6f, 0x161, 0x6b, 0x69, 0x20, 0x73, 0x72, 0x65,
0x64, 0x6e, 0x6a, 0x65, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x4b, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x4b, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x4b, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x70,
0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c, 0x61,
0x73, 0x3a, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x10c, 0x61, 0x73, 0x3a,
0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72,
0x76, 0x69, 0x6c, 0x6c, 0x65, 0x56, 0x7a, 0x68, 0x6f, 0x64, 0x6e, 0x6f,
0x74, 0x69, 0x6d, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x10c, 0x61, 0x73, 0x3a, 0x20, 0x56, 0x65, 0x6c, 0x69, 0x6b, 0x6f, 0x6e,
0x6f, 0x10d, 0x6e, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x3a,
0x20, 0x56, 0x65, 0x6c, 0x69, 0x6b, 0x6f, 0x6e, 0x6f, 0x10d, 0x6e, 0x69,
0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x50, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x3a, 0x20, 0x56, 0x65, 0x6c, 0x69, 0x6b, 0x6f,
0x6e, 0x6f, 0x10d, 0x6e, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x45, 0x6b,
0x76, 0x61, 0x64, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x53, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x65, 0x65, 0x76, 0x72, 0x6f, 0x70,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x53, 0x72, 0x65, 0x64, 0x6e,
0x6a, 0x65, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x53, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x65, 0x65, 0x76, 0x72, 0x6f,
0x70, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x56, 0x7a, 0x68, 0x6f, 0x64, 0x6e, 0x6f, 0x65,
0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56,
0x7a, 0x68, 0x6f, 0x64, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x56, 0x7a, 0x68, 0x6f, 0x64, 0x6e, 0x6f,
0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x44, 0x6f, 0x64, 0x61,
0x74, 0x6e, 0x69, 0x20, 0x76, 0x7a, 0x68, 0x6f, 0x64, 0x6e, 0x6f, 0x65,
0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x5a,
0x61, 0x68, 0x6f, 0x64, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x5a, 0x61, 0x68, 0x6f, 0x64, 0x6e,
0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x5a, 0x61, 0x68, 0x6f, 0x64, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70,
0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x10c, 0x61, 0x73, 0x3a, 0x20, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x6f, 0x74, 0x6f, 0x10d,
0x6a, 0x65, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x3a, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x6f, 0x74, 0x6f, 0x10d, 0x6a, 0x65,
0x50, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x3a,
0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f,
0x20, 0x6f, 0x74, 0x6f, 0x10d, 0x6a, 0x65, 0x46, 0x69, 0x64, 0x17e, 0x69,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x46, 0x69, 0x64, 0x17e,
0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x46, 0x69, 0x64, 0x17e,
0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c, 0x61, 0x73, 0x3a, 0x20, 0x46, 0x72,
0x61, 0x6e, 0x63, 0x6f, 0x73, 0x6b, 0x61, 0x20, 0x47, 0x76, 0x61, 0x6a,
0x61, 0x6e, 0x61, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x6f, 0x73, 0x6b, 0x69,
0x20, 0x6a, 0x75, 0x17e, 0x6e, 0x69, 0x20, 0x69, 0x6e, 0x20, 0x61, 0x6e,
0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x10d, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x161, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x10c, 0x61, 0x73, 0x3a, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65,
0x72, 0x74, 0x6f, 0x76, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x69, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x161, 0x6b, 0x69, 0x20, 0x73, 0x72,
0x65, 0x64, 0x6e, 0x6a, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56, 0x7a, 0x68,
0x6f, 0x64, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56, 0x7a, 0x68, 0x6f, 0x64,
0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x56, 0x7a, 0x68, 0x6f, 0x64, 0x6e, 0x6f, 0x67,
0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x70,
0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x5a, 0x61,
0x68, 0x6f, 0x64, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x5a, 0x61, 0x68, 0x6f,
0x64, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x5a, 0x61, 0x68, 0x6f, 0x64, 0x6e, 0x6f,
0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20,
0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x5a,
0x61, 0x6c, 0x69, 0x76, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x47, 0x76,
0x61, 0x6a, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x48,
0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x61, 0x6c, 0x65, 0x75,
0x74, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x48, 0x61, 0x76, 0x61,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x48, 0x6f, 0x6e,
0x67, 0x6b, 0x6f, 0x6e, 0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x48,
0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x161, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x161, 0x6b, 0x69, 0x20, 0x70,
0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x48, 0x6f,
0x76, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x48, 0x6f, 0x76,
0x64, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x48, 0x6f, 0x76, 0x64, 0x73,
0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x49, 0x6e, 0x64, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x49, 0x6e, 0x64, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x6f, 0x63, 0x65,
0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x49, 0x6e, 0x64,
0x6f, 0x6b, 0x69, 0x74, 0x61, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x6f, 0x73, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x76, 0x7a, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x7a, 0x61, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x49, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x49, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x49,
0x72, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x69, 0x20,
0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4a,
0x61, 0x70, 0x6f, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4a,
0x61, 0x70, 0x6f, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4a, 0x61,
0x70, 0x6f, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70,
0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x10d,
0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x65, 0x74,
0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b,
0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x2d, 0x4b, 0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x70,
0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x61,
0x7a, 0x61, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x56, 0x7a, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20, 0x6b, 0x61,
0x7a, 0x61, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x5a, 0x61, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20, 0x6b, 0x61,
0x7a, 0x61, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x4b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x4b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x4b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x6f, 0x73,
0x72, 0x61, 0x6a, 0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a,
0x61, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x69, 0x72, 0x67, 0x69, 0x7a, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c,
0x61, 0x73, 0x3a, 0x20, 0x45, 0x6b, 0x76, 0x61, 0x74, 0x6f, 0x72, 0x73,
0x6b, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x69, 0x10c, 0x61, 0x73, 0x20,
0x6f, 0x74, 0x6f, 0x6b, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x61, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x50, 0x6f, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x6f, 0x74, 0x6f,
0x6b, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x73, 0x6b,
0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x4d, 0x61, 0x6c, 0x65, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x10c, 0x61, 0x73, 0x3a, 0x20, 0x4d, 0x61, 0x72,
0x6b, 0x69, 0x7a, 0x6e, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x69, 0x10c,
0x61, 0x73, 0x3a, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c,
0x6f, 0x76, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x69, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x63, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x4d, 0x65, 0x68, 0x69, 0x161, 0x6b, 0x69, 0x20, 0x70, 0x61,
0x63, 0x69, 0x66, 0x69, 0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4d,
0x65, 0x68, 0x69, 0x161, 0x6b, 0x69, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66,
0x69, 0x161, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4d, 0x65, 0x68, 0x69, 0x161,
0x6b, 0x69, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x161, 0x6b, 0x69,
0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f,
0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x55, 0x6c, 0x61, 0x6e, 0x62,
0x61, 0x74, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4d, 0x6f, 0x73, 0x6b, 0x6f,
0x76, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4d, 0x6f, 0x73, 0x6b,
0x6f, 0x76, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4d, 0x6f, 0x73, 0x6b,
0x6f, 0x76, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x4d, 0x6a, 0x61, 0x6e, 0x6d, 0x61, 0x72,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4e, 0x61, 0x75, 0x72, 0x75,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4e, 0x65, 0x70, 0x61,
0x6c, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4e, 0x6f, 0x76, 0x6f,
0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x6b, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4e,
0x6f, 0x76, 0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x7a, 0x65, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4e, 0x6f, 0x76, 0x6f,
0x7a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x4e, 0x6f, 0x76, 0x6f, 0x7a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4e, 0x6f, 0x76, 0x6f,
0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x4e, 0x69, 0x75, 0x65, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c, 0x61, 0x73, 0x3a, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x161, 0x6b, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b,
0x69, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x3a, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x161,
0x6b, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x69, 0x50, 0x6f, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x3a, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x161, 0x6b, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x69,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x161, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x161, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x4f, 0x6d, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4f,
0x6d, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4f, 0x6d, 0x73, 0x6b, 0x69,
0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x50, 0x61, 0x6c, 0x61, 0x76, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x50, 0x61, 0x70, 0x75, 0x61, 0x6e, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x61, 0x72, 0x61,
0x67, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x61,
0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x65, 0x72,
0x75, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x65, 0x72,
0x75, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x65, 0x72, 0x75,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69,
0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c, 0x61, 0x73, 0x3a, 0x20, 0x4f, 0x74,
0x6f, 0x10d, 0x6a, 0x65, 0x20, 0x46, 0x65, 0x6e, 0x69, 0x6b, 0x73, 0x10c,
0x61, 0x73, 0x3a, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x20, 0x69, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x3a, 0x20, 0x53, 0x61, 0x69, 0x6e,
0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x69, 0x6e, 0x20,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x50, 0x6f, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x3a, 0x20, 0x53, 0x61, 0x69,
0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x69, 0x6e,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x50, 0x69, 0x74,
0x63, 0x61, 0x69, 0x72, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x50, 0x6f, 0x6e, 0x61, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x50, 0x6a, 0x6f, 0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x161, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x53, 0x61, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x53, 0x61, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x53, 0x61, 0x6d, 0x61, 0x72,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x53, 0x61, 0x6d, 0x61, 0x72,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x73,
0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x53, 0x65, 0x6a, 0x161, 0x65, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x6f, 0x76,
0x6f, 0x6f, 0x74, 0x6f, 0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4a,
0x75, 0x17e, 0x6e, 0x6f, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61,
0x6d, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c, 0x61, 0x73, 0x3a,
0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x54, 0x61, 0x6a, 0x70,
0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x54, 0x61, 0x6a,
0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x54, 0x61, 0x6a,
0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x54, 0x61, 0x64, 0x17e, 0x69, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x76, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x54, 0x6f, 0x6e, 0x67, 0x6f, 0x76, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x54, 0x6f, 0x6e, 0x67, 0x6f, 0x76, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x54, 0x6f, 0x6e, 0x67, 0x6f, 0x76, 0x73, 0x6b, 0x69,
0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x10c, 0x61, 0x73, 0x3a, 0x20, 0x4f, 0x74, 0x6f, 0x6b, 0x20, 0x43, 0x68,
0x75, 0x75, 0x6b, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x54, 0x75,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x54, 0x75, 0x76, 0x61, 0x6c,
0x75, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x55, 0x72, 0x75,
0x67, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x55,
0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x70,
0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x55, 0x7a,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x55, 0x7a, 0x62, 0x65, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x161, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x161, 0x6b,
0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56, 0x6f, 0x73, 0x74,
0x6f, 0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c, 0x61, 0x73, 0x3a,
0x20, 0x4f, 0x74, 0x6f, 0x6b, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x10c, 0x61,
0x73, 0x3a, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x69, 0x6e,
0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x4a, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4a, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4a, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x161, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x161, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4a, 0x75, 0x6b, 0x6f, 0x6e, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x57, 0x61, 0x6b, 0x68, 0x74, 0x69,
0x67, 0x61, 0x20, 0x41, 0x63, 0x72, 0x65, 0x57, 0x61, 0x6b, 0x68, 0x74,
0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20,
0x61, 0x68, 0x20, 0x65, 0x65, 0x20, 0x41, 0x63, 0x72, 0x65, 0x57, 0x61,
0x6b, 0x68, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x75, 0x6c, 0x6b, 0x61,
0x20, 0x65, 0x65, 0x20, 0x41, 0x63, 0x72, 0x65, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x41, 0x66, 0x67, 0x67, 0x61, 0x61, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x42, 0x61, 0x72, 0x74, 0x61, 0x6d, 0x61, 0x68, 0x61, 0x20, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x42, 0x61, 0x72, 0x69, 0x67, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x79, 0x61, 0x64, 0x61, 0x20,
0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65,
0x65, 0x20, 0x4b, 0x6f, 0x6f, 0x6e, 0x66, 0x75, 0x72, 0x20, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x47, 0x61, 0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65,
0x65, 0x20, 0x47, 0x61, 0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20,
0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65,
0x20, 0x47, 0x61, 0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41,
0x68, 0x20, 0x65, 0x65, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61,
0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x41, 0x6c,
0x6d, 0x61, 0x74, 0x79, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x61, 0x68, 0x20, 0x65,
0x65, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x53, 0x61, 0x61, 0x63,
0x61, 0x64, 0x61, 0x20, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x4b, 0x75, 0x6c, 0x61, 0x79, 0x6c, 0x61, 0x68, 0x61, 0x20, 0x65, 0x65,
0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x41, 0x6d, 0x61, 0x73, 0x6f, 0x6e, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61,
0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x41, 0x6d, 0x61, 0x73, 0x6f,
0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67,
0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x41, 0x6d, 0x61, 0x73,
0x6f, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61,
0x72, 0x74, 0x61, 0x6d, 0x61, 0x68, 0x61, 0x20, 0x57, 0x61, 0x71, 0x6f,
0x6f, 0x79, 0x69, 0x67, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61,
0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20,
0x42, 0x61, 0x72, 0x74, 0x61, 0x6d, 0x61, 0x68, 0x61, 0x20, 0x57, 0x61,
0x71, 0x6f, 0x6f, 0x79, 0x69, 0x67, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65,
0x20, 0x42, 0x61, 0x72, 0x74, 0x61, 0x6d, 0x61, 0x68, 0x61, 0x20, 0x57,
0x61, 0x71, 0x6f, 0x6f, 0x79, 0x69, 0x67, 0x61, 0x20, 0x41, 0x6d, 0x65,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x42, 0x61, 0x72, 0x69, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x57,
0x61, 0x71, 0x6f, 0x6f, 0x79, 0x69, 0x67, 0x61, 0x20, 0x41, 0x6d, 0x65,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20,
0x65, 0x65, 0x20, 0x42, 0x61, 0x72, 0x69, 0x67, 0x61, 0x20, 0x57, 0x61,
0x71, 0x6f, 0x6f, 0x79, 0x69, 0x67, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65,
0x20, 0x42, 0x61, 0x72, 0x69, 0x67, 0x61, 0x20, 0x57, 0x61, 0x71, 0x6f,
0x6f, 0x79, 0x69, 0x67, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x75,
0x75, 0x72, 0x61, 0x6c, 0x65, 0x79, 0x64, 0x61, 0x20, 0x65, 0x65, 0x20,
0x57, 0x61, 0x71, 0x6f, 0x6f, 0x79, 0x69, 0x67, 0x61, 0x20, 0x41, 0x6d,
0x65, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x61, 0x68,
0x20, 0x65, 0x65, 0x20, 0x42, 0x75, 0x75, 0x72, 0x61, 0x6c, 0x65, 0x79,
0x64, 0x61, 0x20, 0x57, 0x61, 0x71, 0x6f, 0x6f, 0x79, 0x69, 0x67, 0x61,
0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72,
0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x42, 0x75, 0x75, 0x72, 0x6c, 0x65,
0x79, 0x64, 0x61, 0x20, 0x57, 0x61, 0x71, 0x6f, 0x6f, 0x79, 0x69, 0x67,
0x61, 0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x73, 0x69, 0x66, 0x69,
0x6b, 0x61, 0x20, 0x65, 0x65, 0x20, 0x57, 0x61, 0x71, 0x6f, 0x6f, 0x79,
0x69, 0x67, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64,
0x69, 0x67, 0x61, 0x20, 0x61, 0x68, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61,
0x73, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x20, 0x57, 0x61, 0x71, 0x6f, 0x6f,
0x79, 0x69, 0x67, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61,
0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61,
0x73, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x20, 0x57, 0x61, 0x71, 0x6f, 0x6f,
0x79, 0x69, 0x67, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x57, 0x61, 0x6b, 0x68, 0x74, 0x69, 0x67, 0x61, 0x20, 0x41, 0x6e,
0x61, 0x64, 0x79, 0x72, 0x57, 0x61, 0x6b, 0x68, 0x74, 0x69, 0x67, 0x61,
0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x61, 0x68, 0x20,
0x65, 0x65, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x57, 0x61, 0x6b,
0x68, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x75, 0x6c, 0x6b, 0x61, 0x20,
0x65, 0x65, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x41, 0x62, 0x69, 0x79, 0x61, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67,
0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x41, 0x62, 0x69, 0x79,
0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61,
0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x41, 0x62,
0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x41,
0x71, 0x74, 0x61, 0x75, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x61, 0x68, 0x20, 0x65,
0x65, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x53, 0x61, 0x61, 0x63, 0x61,
0x64, 0x61, 0x20, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b,
0x75, 0x6c, 0x61, 0x79, 0x6c, 0x61, 0x68, 0x61, 0x20, 0x41, 0x71, 0x74,
0x61, 0x75, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x41, 0x71,
0x74, 0x6f, 0x62, 0x65, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x61, 0x68, 0x20, 0x65,
0x65, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x53, 0x61, 0x61, 0x63,
0x61, 0x64, 0x61, 0x20, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x6b, 0x75, 0x6c, 0x61, 0x79, 0x6c, 0x61, 0x68, 0x61, 0x20, 0x41, 0x71,
0x74, 0x6f, 0x62, 0x65, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x43, 0x61, 0x72, 0x61, 0x62, 0x74, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41,
0x68, 0x20, 0x65, 0x65, 0x20, 0x43, 0x61, 0x72, 0x61, 0x62, 0x74, 0x61,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72,
0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x43, 0x61, 0x72,
0x61, 0x62, 0x74, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x61, 0x20, 0x41,
0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x61, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61,
0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x41, 0x72, 0x6a, 0x65, 0x6e,
0x74, 0x69, 0x69, 0x6e, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20,
0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x61, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x47, 0x61, 0x6c, 0x62, 0x65, 0x65,
0x64, 0x6b, 0x61, 0x20, 0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x69,
0x6e, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61,
0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20,
0x47, 0x61, 0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20, 0x41, 0x72,
0x6a, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x61, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20,
0x65, 0x65, 0x20, 0x47, 0x61, 0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b, 0x61,
0x20, 0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x61, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x65,
0x6e, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65,
0x65, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x65, 0x6e, 0x69, 0x79, 0x61, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61,
0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x65, 0x6e,
0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x61, 0x20, 0x65, 0x65, 0x20,
0x57, 0x61, 0x71, 0x6f, 0x6f, 0x79, 0x69, 0x67, 0x61, 0x20, 0x41, 0x6d,
0x65, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68,
0x20, 0x65, 0x65, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b,
0x61, 0x20, 0x57, 0x61, 0x71, 0x6f, 0x6f, 0x79, 0x69, 0x67, 0x61, 0x20,
0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74,
0x61, 0x20, 0x65, 0x65, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x6b, 0x61, 0x20, 0x57, 0x61, 0x71, 0x6f, 0x6f, 0x79, 0x69, 0x67, 0x61,
0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x72, 0x74, 0x61, 0x6d, 0x61,
0x68, 0x61, 0x20, 0x41, 0x73, 0x74, 0x61, 0x72, 0x61, 0x61, 0x6c, 0x69,
0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61,
0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20,
0x42, 0x61, 0x72, 0x74, 0x61, 0x6d, 0x61, 0x68, 0x61, 0x20, 0x41, 0x73,
0x74, 0x61, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72,
0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x72, 0x74, 0x61, 0x6d,
0x61, 0x68, 0x61, 0x20, 0x41, 0x73, 0x74, 0x61, 0x72, 0x61, 0x61, 0x6c,
0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42,
0x61, 0x72, 0x74, 0x61, 0x6d, 0x61, 0x68, 0x61, 0x20, 0x47, 0x61, 0x6c,
0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20, 0x41, 0x73, 0x74, 0x61, 0x72,
0x61, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68,
0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x72, 0x74, 0x61, 0x6d, 0x61, 0x68,
0x61, 0x20, 0x47, 0x61, 0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20,
0x41, 0x73, 0x74, 0x61, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61,
0x61, 0x72, 0x74, 0x61, 0x20, 0x42, 0x61, 0x72, 0x74, 0x61, 0x6d, 0x61,
0x68, 0x61, 0x20, 0x47, 0x61, 0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b, 0x61,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x72, 0x69, 0x67,
0x61, 0x20, 0x41, 0x73, 0x74, 0x61, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x79,
0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x79, 0x61, 0x64, 0x61, 0x20, 0x43,
0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x61, 0x68, 0x20, 0x65, 0x65,
0x20, 0x42, 0x61, 0x72, 0x69, 0x67, 0x61, 0x20, 0x41, 0x73, 0x74, 0x61,
0x72, 0x61, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61,
0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x72, 0x69, 0x67, 0x61, 0x20, 0x41,
0x73, 0x74, 0x61, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x47, 0x61, 0x6c, 0x62, 0x65, 0x65,
0x64, 0x6b, 0x61, 0x20, 0x41, 0x73, 0x74, 0x61, 0x72, 0x61, 0x61, 0x6c,
0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43,
0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65,
0x20, 0x47, 0x61, 0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20, 0x41,
0x73, 0x74, 0x61, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61,
0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x47, 0x61, 0x6c, 0x62, 0x65,
0x65, 0x64, 0x6b, 0x61, 0x20, 0x41, 0x73, 0x74, 0x61, 0x72, 0x61, 0x61,
0x6c, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x41, 0x73, 0x61, 0x72, 0x62, 0x65, 0x79, 0x6a, 0x61, 0x6e, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67,
0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x41, 0x73, 0x61, 0x72,
0x62, 0x65, 0x79, 0x6a, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65,
0x20, 0x41, 0x73, 0x61, 0x72, 0x62, 0x65, 0x79, 0x6a, 0x61, 0x6e, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x41, 0x73, 0x6f, 0x72, 0x65,
0x73, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61,
0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x41,
0x73, 0x6f, 0x72, 0x65, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20,
0x41, 0x73, 0x6f, 0x72, 0x65, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x65, 0x64, 0x65, 0x65, 0x73,
0x68, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61,
0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x65, 0x64, 0x65, 0x65, 0x73, 0x68, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67,
0x61, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x65, 0x64,
0x65, 0x65, 0x73, 0x68, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x42, 0x75, 0x74, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x42, 0x6f, 0x6c, 0x69, 0x69, 0x66, 0x69, 0x79, 0x61, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x72, 0x61, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x61, 0x68,
0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x72, 0x61, 0x61, 0x73, 0x69, 0x6c,
0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58,
0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61,
0x72, 0x61, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x75, 0x72, 0x75, 0x6e, 0x65, 0x79,
0x20, 0x44, 0x61, 0x61, 0x72, 0x75, 0x73, 0x61, 0x6c, 0x61, 0x61, 0x6d,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x65, 0x79, 0x62,
0x20, 0x46, 0x61, 0x61, 0x72, 0x64, 0x65, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41,
0x68, 0x20, 0x65, 0x65, 0x20, 0x4b, 0x65, 0x79, 0x62, 0x20, 0x46, 0x61,
0x61, 0x72, 0x64, 0x65, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x4b,
0x65, 0x79, 0x62, 0x20, 0x46, 0x61, 0x61, 0x72, 0x64, 0x65, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4a, 0x61, 0x6d, 0x6f, 0x72, 0x6f,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4a, 0x61, 0x61, 0x74,
0x61, 0x6d, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61,
0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20,
0x4a, 0x61, 0x61, 0x74, 0x61, 0x6d, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20,
0x65, 0x65, 0x20, 0x4a, 0x61, 0x61, 0x74, 0x61, 0x6d, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x4a, 0x69, 0x6c, 0x69, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61,
0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4a, 0x69, 0x6c, 0x69, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61,
0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x4a, 0x69, 0x6c, 0x69, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x53, 0x68, 0x69, 0x69, 0x6e, 0x61,
0x68, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61,
0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20,
0x53, 0x68, 0x69, 0x69, 0x6e, 0x61, 0x68, 0x61, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74,
0x61, 0x20, 0x65, 0x65, 0x20, 0x53, 0x68, 0x69, 0x69, 0x6e, 0x61, 0x68,
0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x69, 0x72,
0x69, 0x73, 0x6d, 0x61, 0x73, 0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x6f, 0x6b, 0x6f,
0x73, 0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x4b, 0x6f, 0x6c, 0x61, 0x6d, 0x62, 0x69, 0x79,
0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61,
0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4b,
0x6f, 0x6c, 0x61, 0x6d, 0x62, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20,
0x65, 0x65, 0x20, 0x4b, 0x6f, 0x6c, 0x61, 0x6d, 0x62, 0x69, 0x79, 0x61,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x75, 0x75, 0x6b,
0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x69, 0x73, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61,
0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4b, 0x75, 0x75, 0x6b, 0x20,
0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x69, 0x73, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x4e, 0x75, 0x73, 0x20, 0x58, 0x61, 0x67, 0x61,
0x61, 0x67, 0x61, 0x20, 0x61, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4b, 0x75,
0x75, 0x6b, 0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x69, 0x73, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x75, 0x75, 0x62, 0x61,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64,
0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4b, 0x75,
0x75, 0x62, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44,
0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20,
0x4b, 0x75, 0x75, 0x62, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x44, 0x61, 0x66, 0x69, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x20, 0x2d, 0x20, 0x64,
0x2019, 0x75, 0x72, 0x66, 0x69, 0x6c, 0x6c, 0x65, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x49, 0x69, 0x73, 0x74, 0x20, 0x54, 0x69, 0x6d,
0x6f, 0x6f, 0x72, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x49,
0x69, 0x73, 0x74, 0x61, 0x72, 0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64,
0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x49, 0x69,
0x73, 0x74, 0x61, 0x72, 0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61,
0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x49, 0x69, 0x73, 0x74, 0x61, 0x72,
0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x45, 0x6b, 0x75, 0x77, 0x61, 0x64, 0x6f, 0x6f, 0x72,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x72, 0x74,
0x61, 0x6d, 0x61, 0x68, 0x61, 0x20, 0x59, 0x75, 0x72, 0x75, 0x62, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69,
0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x72,
0x74, 0x61, 0x6d, 0x61, 0x68, 0x61, 0x20, 0x59, 0x75, 0x72, 0x75, 0x62,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61,
0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x72, 0x74, 0x61,
0x6d, 0x61, 0x68, 0x61, 0x20, 0x59, 0x75, 0x72, 0x75, 0x62, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x72, 0x69, 0x67, 0x61,
0x20, 0x59, 0x75, 0x72, 0x75, 0x62, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68,
0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x72, 0x69, 0x67, 0x61, 0x20, 0x59,
0x75, 0x72, 0x75, 0x62, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x42,
0x61, 0x72, 0x69, 0x67, 0x61, 0x20, 0x59, 0x75, 0x72, 0x75, 0x62, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x72, 0x69, 0x67,
0x61, 0x20, 0x46, 0x6f, 0x67, 0x20, 0x65, 0x65, 0x20, 0x59, 0x75, 0x72,
0x75, 0x62, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x47, 0x61,
0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20, 0x59, 0x75, 0x72, 0x75,
0x62, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61,
0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x47,
0x61, 0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20, 0x59, 0x75, 0x72,
0x75, 0x62, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61,
0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x47, 0x61, 0x6c,
0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20, 0x59, 0x75, 0x72, 0x75, 0x62,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x46, 0x61, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x61, 0x6e, 0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e,
0x69, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61,
0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20,
0x46, 0x61, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x61, 0x6e, 0x20, 0x41, 0x79,
0x6c, 0x61, 0x61, 0x6e, 0x69, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65,
0x20, 0x46, 0x61, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x61, 0x6e, 0x20, 0x41,
0x79, 0x6c, 0x61, 0x61, 0x6e, 0x69, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41,
0x68, 0x20, 0x65, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61,
0x20, 0x65, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x46, 0x65, 0x72, 0x65, 0x6e, 0x6a, 0x20, 0x47,
0x75, 0x79, 0x61, 0x6e, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x4b, 0x6f, 0x6f, 0x6e, 0x66, 0x75, 0x72, 0x74, 0x61, 0x20, 0x46,
0x61, 0x72, 0x61, 0x6e, 0x73, 0x69, 0x69, 0x73, 0x6b, 0x61, 0x20, 0x26,
0x20, 0x41, 0x6e, 0x74, 0x61, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x6b, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x62,
0x61, 0x67, 0x6f, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x47, 0x61, 0x6d, 0x62, 0x69, 0x79, 0x61, 0x72, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x4a, 0x6f, 0x6f, 0x72, 0x6a, 0x69, 0x79, 0x61,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64,
0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4a, 0x6f,
0x6f, 0x72, 0x6a, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65,
0x20, 0x4a, 0x6f, 0x6f, 0x72, 0x6a, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x4a, 0x69, 0x6c, 0x62, 0x65, 0x65, 0x72,
0x74, 0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x68,
0x74, 0x69, 0x67, 0x61, 0x20, 0x47, 0x69, 0x72, 0x69, 0x69, 0x6e, 0x77,
0x69, 0x6a, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61,
0x72, 0x69, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x47, 0x69, 0x72, 0x69,
0x69, 0x6e, 0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x61, 0x68,
0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x72, 0x69, 0x67, 0x61, 0x20, 0x47,
0x69, 0x72, 0x69, 0x69, 0x6e, 0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61,
0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x72, 0x69, 0x67, 0x61, 0x20, 0x47,
0x69, 0x72, 0x69, 0x69, 0x6e, 0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x47, 0x61, 0x6c, 0x62, 0x65, 0x65, 0x64,
0x6b, 0x61, 0x20, 0x47, 0x69, 0x72, 0x69, 0x69, 0x6e, 0x6c, 0x61, 0x61,
0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61,
0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x47,
0x61, 0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20, 0x47, 0x69, 0x72,
0x69, 0x69, 0x6e, 0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65,
0x65, 0x20, 0x47, 0x61, 0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20,
0x47, 0x69, 0x72, 0x69, 0x69, 0x6e, 0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x47, 0x61, 0x63, 0x61, 0x6e, 0x6b,
0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x47, 0x75, 0x79,
0x61, 0x61, 0x6e, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x48, 0x61, 0x77, 0x61, 0x61, 0x79, 0x2d, 0x41, 0x6c, 0x75, 0x74, 0x69,
0x79, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65,
0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x61, 0x79, 0x2d, 0x41, 0x6c, 0x75,
0x74, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20,
0x65, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x61, 0x79, 0x2d, 0x41, 0x6c,
0x75, 0x74, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x48, 0x6f, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6f,
0x6e, 0x67, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61,
0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20,
0x48, 0x6f, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6f, 0x6e, 0x67, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61,
0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x48, 0x6f, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6f, 0x6e, 0x67, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x48, 0x6f, 0x66, 0x75, 0x64, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68,
0x20, 0x65, 0x65, 0x20, 0x48, 0x6f, 0x66, 0x75, 0x64, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61,
0x20, 0x65, 0x65, 0x20, 0x48, 0x6f, 0x66, 0x75, 0x64, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61,
0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x48, 0x69, 0x6e, 0x64, 0x69,
0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61,
0x64, 0x77, 0x65, 0x79, 0x6e, 0x74, 0x61, 0x20, 0x48, 0x69, 0x6e, 0x64,
0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x49,
0x6e, 0x64, 0x6f, 0x73, 0x68, 0x69, 0x69, 0x6e, 0x61, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x72, 0x74, 0x61, 0x6d, 0x61,
0x68, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x65, 0x79, 0x73,
0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x49,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x65, 0x79, 0x73, 0x69, 0x79, 0x61, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x47, 0x61, 0x6c, 0x62, 0x65,
0x65, 0x64, 0x6b, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x65,
0x79, 0x73, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x49, 0x69, 0x72, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41,
0x68, 0x20, 0x65, 0x65, 0x20, 0x49, 0x69, 0x72, 0x61, 0x61, 0x6e, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61,
0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x49, 0x69, 0x72, 0x61,
0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x69, 0x6b, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68,
0x20, 0x65, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x69, 0x6b,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61,
0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x69, 0x6b, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x49,
0x73, 0x72, 0x61, 0x61, 0x69, 0x69, 0x6c, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41,
0x68, 0x20, 0x65, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x61, 0x69, 0x69,
0x6c, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61,
0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x49, 0x73,
0x72, 0x61, 0x61, 0x69, 0x69, 0x6c, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x4a, 0x61, 0x62, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20,
0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4a, 0x61, 0x62, 0x61, 0x61, 0x6e,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72,
0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x4a, 0x61, 0x62,
0x61, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x68, 0x74, 0x69, 0x67, 0x61, 0x20,
0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73,
0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69,
0x57, 0x61, 0x6b, 0x68, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61,
0x64, 0x69, 0x67, 0x61, 0x20, 0x61, 0x68, 0x20, 0x65, 0x65, 0x20, 0x50,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x57,
0x61, 0x6b, 0x68, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x75, 0x6c, 0x6b,
0x61, 0x20, 0x65, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61,
0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68,
0x61, 0x74, 0x73, 0x6b, 0x69, 0x57, 0x61, 0x6b, 0x68, 0x74, 0x69, 0x67,
0x61, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x72,
0x69, 0x67, 0x61, 0x20, 0x4b, 0x61, 0x73, 0x61, 0x6b, 0x68, 0x69, 0x73,
0x74, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x4b, 0x6f, 0x6f, 0x6e, 0x66, 0x75, 0x72, 0x74, 0x61, 0x20, 0x4b, 0x61,
0x73, 0x61, 0x6b, 0x68, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x75, 0x75, 0x72, 0x69, 0x79, 0x61,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64,
0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4b, 0x75,
0x75, 0x72, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65,
0x65, 0x20, 0x4b, 0x75, 0x75, 0x72, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x69, 0x79, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x61, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x69, 0x6b, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20,
0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4b, 0x61, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x79, 0x61, 0x72, 0x73, 0x69, 0x6b, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65,
0x65, 0x20, 0x4b, 0x61, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x69, 0x6b, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b,
0x69, 0x79, 0x72, 0x69, 0x67, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4c, 0x65, 0x79, 0x6e, 0x20,
0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x4c, 0x6f, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61,
0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4c, 0x6f, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68,
0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x4c,
0x6f, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x57, 0x61, 0x74, 0x69, 0x67, 0x61,
0x20, 0x4d, 0x61, 0x67, 0x65, 0x64, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20,
0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x65, 0x64, 0x61,
0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67,
0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x65,
0x64, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4d,
0x61, 0x6c, 0x65, 0x79, 0x73, 0x68, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x66, 0x69,
0x73, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4d, 0x61, 0x72,
0x71, 0x75, 0x77, 0x65, 0x73, 0x61, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x4d, 0x61, 0x61, 0x72, 0x73, 0x68, 0x61, 0x61, 0x6c,
0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x69, 0x73, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x4d, 0x6f, 0x72, 0x69, 0x73, 0x68, 0x69,
0x79, 0x61, 0x61, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65,
0x65, 0x20, 0x4d, 0x6f, 0x72, 0x69, 0x73, 0x68, 0x69, 0x79, 0x61, 0x61,
0x73, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67,
0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x4d, 0x6f, 0x72, 0x69,
0x73, 0x68, 0x69, 0x79, 0x61, 0x61, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x61, 0x73, 0x69, 0x66, 0x69,
0x6b, 0x61, 0x64, 0x61, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64,
0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61,
0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x64, 0x61, 0x20, 0x4d, 0x65,
0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65,
0x65, 0x20, 0x42, 0x61, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x64,
0x61, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61,
0x61, 0x74, 0x61, 0x61, 0x72, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20,
0x65, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74,
0x61, 0x61, 0x72, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58,
0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x55, 0x6c,
0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x61, 0x72, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x77,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64,
0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4d, 0x6f,
0x73, 0x6b, 0x6f, 0x77, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x4d,
0x6f, 0x73, 0x6b, 0x6f, 0x77, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x4d, 0x61, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x61, 0x72, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4e, 0x61, 0x77, 0x72, 0x6f, 0x6f,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4e, 0x65, 0x65, 0x62,
0x61, 0x61, 0x6c, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4e,
0x69, 0x79, 0x75, 0x75, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61,
0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20,
0x4e, 0x69, 0x79, 0x75, 0x75, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x4e,
0x69, 0x79, 0x75, 0x75, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4e,
0x69, 0x79, 0x75, 0x75, 0x20, 0x53, 0x69, 0x2019, 0x6c, 0x61, 0x61, 0x6e,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64,
0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4e, 0x69,
0x79, 0x75, 0x75, 0x20, 0x53, 0x69, 0x2019, 0x6c, 0x61, 0x61, 0x6e, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61,
0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x4e, 0x69, 0x79, 0x75,
0x75, 0x20, 0x53, 0x69, 0x2019, 0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x4e, 0x69, 0x79, 0x75, 0x75, 0x66, 0x6f,
0x6f, 0x6e, 0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68,
0x20, 0x65, 0x65, 0x20, 0x4e, 0x69, 0x79, 0x75, 0x75, 0x66, 0x6f, 0x6f,
0x6e, 0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65,
0x65, 0x20, 0x4e, 0x69, 0x79, 0x75, 0x75, 0x66, 0x6f, 0x6f, 0x6e, 0x6c,
0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61,
0x64, 0x69, 0x67, 0x61, 0x20, 0x61, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4d, 0x61, 0x61,
0x6c, 0x69, 0x6e, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x46, 0x61, 0x72, 0x6e, 0x61,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43,
0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65,
0x20, 0x46, 0x61, 0x72, 0x6e, 0x61, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67,
0x61, 0x20, 0x65, 0x65, 0x20, 0x46, 0x61, 0x72, 0x6e, 0x61, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4e, 0x6f,
0x66, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x69, 0x6b, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67,
0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4e, 0x6f, 0x66, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x69, 0x6b, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20,
0x65, 0x65, 0x20, 0x4e, 0x6f, 0x66, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x69, 0x6b, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4f,
0x6d, 0x73, 0x6b, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43,
0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65,
0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20,
0x4f, 0x6d, 0x73, 0x6b, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x42, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61,
0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x42,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x6c, 0x61, 0x77, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x62, 0x75, 0x77, 0x20, 0x4e,
0x69, 0x79, 0x75, 0x75, 0x20, 0x47, 0x69, 0x6e, 0x69, 0x79, 0x61, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x72, 0x61, 0x67,
0x77, 0x61, 0x61, 0x79, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65,
0x65, 0x20, 0x42, 0x61, 0x72, 0x61, 0x67, 0x77, 0x61, 0x61, 0x79, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61,
0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x72, 0x61, 0x67, 0x77,
0x61, 0x61, 0x79, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42,
0x65, 0x65, 0x72, 0x75, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65,
0x65, 0x20, 0x42, 0x65, 0x65, 0x72, 0x75, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65,
0x65, 0x20, 0x42, 0x65, 0x65, 0x72, 0x75, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x62, 0x69, 0x69, 0x6e, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69,
0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x46, 0x69, 0x6c,
0x69, 0x62, 0x69, 0x69, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20,
0x46, 0x69, 0x6c, 0x69, 0x62, 0x69, 0x69, 0x6e, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x46, 0x6f, 0x6f, 0x6e, 0x69, 0x6b, 0x69, 0x73,
0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x69, 0x73, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x53, 0x74, 0x2e, 0x20, 0x42, 0x65, 0x65,
0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x6b, 0x69, 0x77, 0x65, 0x6c,
0x6f, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61,
0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x53, 0x74, 0x2e,
0x20, 0x42, 0x65, 0x65, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x6b,
0x69, 0x77, 0x65, 0x6c, 0x6f, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20,
0x65, 0x65, 0x20, 0x53, 0x74, 0x2e, 0x20, 0x42, 0x65, 0x65, 0x72, 0x65,
0x20, 0x26, 0x20, 0x4d, 0x69, 0x6b, 0x69, 0x77, 0x65, 0x6c, 0x6f, 0x6e,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x69, 0x74, 0x6b,
0x65, 0x65, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42,
0x6f, 0x6e, 0x61, 0x62, 0x65, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x42, 0x6f, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c,
0x6f, 0x72, 0x64, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x61, 0x68, 0x20, 0x65,
0x65, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x53,
0x61, 0x61, 0x63, 0x61, 0x64, 0x61, 0x20, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x4b, 0x75, 0x6c, 0x61, 0x79, 0x6c, 0x61, 0x68, 0x61,
0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x52, 0x69, 0x79, 0x75, 0x75, 0x6e,
0x69, 0x79, 0x6f, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x52, 0x6f, 0x74, 0x65, 0x72, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67,
0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x53,
0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x57, 0x61, 0x6b, 0x68, 0x74,
0x69, 0x67, 0x61, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x57, 0x61,
0x6b, 0x68, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69,
0x67, 0x61, 0x20, 0x61, 0x68, 0x20, 0x65, 0x65, 0x20, 0x53, 0x61, 0x6d,
0x61, 0x72, 0x61, 0x57, 0x61, 0x6b, 0x68, 0x74, 0x69, 0x67, 0x61, 0x20,
0x4b, 0x75, 0x6c, 0x6b, 0x61, 0x20, 0x65, 0x65, 0x20, 0x53, 0x61, 0x6d,
0x61, 0x72, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65,
0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61,
0x20, 0x65, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x53, 0x69, 0x69, 0x73, 0x68, 0x61, 0x6c,
0x69, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x53, 0x69,
0x6e, 0x67, 0x61, 0x62, 0x75, 0x75, 0x72, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x20, 0x41,
0x79, 0x6c, 0x61, 0x61, 0x6e, 0x69, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x53, 0x6f, 0x77, 0x74, 0x20, 0x4a, 0x6f, 0x6f, 0x72,
0x6a, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x53, 0x75, 0x72, 0x69, 0x6e, 0x65, 0x79, 0x6d, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x53, 0x69, 0x79, 0x6f, 0x77, 0x61, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x54, 0x65, 0x79, 0x62,
0x65, 0x79, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61,
0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20,
0x54, 0x65, 0x79, 0x62, 0x65, 0x79, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20,
0x65, 0x65, 0x20, 0x54, 0x65, 0x79, 0x62, 0x65, 0x79, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x54, 0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x54,
0x6f, 0x6b, 0x65, 0x6c, 0x75, 0x75, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41,
0x68, 0x20, 0x65, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67,
0x61, 0x20, 0x65, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4a, 0x75, 0x75, 0x6b, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41,
0x68, 0x20, 0x65, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x54, 0x75, 0x66, 0x61,
0x6c, 0x75, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x55, 0x72,
0x75, 0x67, 0x77, 0x61, 0x61, 0x79, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68,
0x20, 0x65, 0x65, 0x20, 0x55, 0x72, 0x75, 0x67, 0x77, 0x61, 0x61, 0x79,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61,
0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x55, 0x72, 0x75, 0x67, 0x77,
0x61, 0x61, 0x79, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x55,
0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61,
0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x55, 0x73, 0x62, 0x65, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20,
0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x46, 0x61, 0x6e, 0x75, 0x75, 0x74,
0x75, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61,
0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x46,
0x61, 0x6e, 0x75, 0x75, 0x74, 0x75, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65,
0x20, 0x46, 0x61, 0x6e, 0x75, 0x75, 0x74, 0x75, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x46, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x77, 0x65,
0x65, 0x6c, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x46,
0x61, 0x6c, 0x61, 0x64, 0x69, 0x66, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69,
0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x46, 0x61, 0x6c,
0x61, 0x64, 0x69, 0x66, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61,
0x20, 0x65, 0x65, 0x20, 0x46, 0x61, 0x6c, 0x61, 0x64, 0x69, 0x66, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x46, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61,
0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x46, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x46,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x46, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x57, 0x61, 0x79, 0x6b, 0x20, 0x49,
0x79, 0x6c, 0x61, 0x61, 0x6e, 0x69, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x57, 0x61, 0x6c, 0x69, 0x73, 0x20, 0x26, 0x20, 0x46,
0x75, 0x74, 0x75, 0x6e, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x79, 0x61,
0x64, 0x61, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20,
0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61,
0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x61, 0x61, 0x67, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20,
0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x61, 0x61, 0x67, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65,
0x65, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x61, 0x61, 0x67, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x59,
0x75, 0x6b, 0x6f, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x41, 0x63, 0x72, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x63, 0x72,
0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x63, 0x72, 0x65, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x63,
0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20,
0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x53, 0x75, 0x64, 0xe1, 0x66, 0x72, 0x69, 0x63,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0xc1, 0x66, 0x72,
0x69, 0x63, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x63,
0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61,
0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61, 0xf1,
0x61, 0x73, 0x20, 0x52, 0x6f, 0x63, 0x6f, 0x73, 0x61, 0x73, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61,
0xf1, 0x61, 0x73, 0x20, 0x52, 0x6f, 0x63, 0x6f, 0x73, 0x61, 0x73, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x6f, 0x6e,
0x74, 0x61, 0xf1, 0x61, 0x73, 0x20, 0x52, 0x6f, 0x63, 0x6f, 0x73, 0x61,
0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61,
0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x68, 0x6f,
0x72, 0x61, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x61, 0x75,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x65, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x65, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x65, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x41, 0x74, 0x6c, 0xe1, 0x6e, 0x74, 0x69,
0x63, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x41, 0x74, 0x6c, 0xe1,
0x6e, 0x74, 0x69, 0x63, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x41, 0x74, 0x6c, 0xe1, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x6f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x79, 0xe1, 0x6e,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69,
0x79, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x69, 0x79, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65,
0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x41, 0x7a,
0x6f, 0x72, 0x65, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61,
0x73, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0xe9,
0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0xe9, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0xe9, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x75, 0x6e, 0xe9, 0x69, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x62, 0x6f, 0x20, 0x56, 0x65, 0x72,
0x64, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f,
0x72, 0x72, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68,
0x69, 0x6c, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x61, 0x76, 0x69, 0x64,
0x61, 0x64, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61,
0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x63, 0x6f,
0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d,
0x62, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f,
0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x6d,
0x65, 0x64, 0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x4f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x65,
0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50,
0x61, 0x73, 0x63, 0x75, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61,
0x73, 0x63, 0x75, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x65, 0x78, 0x74,
0x72, 0x65, 0x6d, 0x6f, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x69, 0x73, 0x6c, 0x61,
0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x73,
0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x73,
0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x79, 0x69, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x46, 0x69, 0x79, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x46, 0x69, 0x79, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x20, 0x47, 0x75, 0x61, 0x79, 0x61, 0x6e, 0x61, 0x20, 0x46,
0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x41, 0x6e, 0x74, 0xe1, 0x72, 0x74, 0x69, 0x64, 0x61,
0x20, 0x79, 0x20, 0x54, 0x65, 0x72, 0x72, 0x69, 0x74, 0x6f, 0x72, 0x69,
0x6f, 0x73, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x73,
0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x65, 0x73, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x61, 0x6c, 0xe1, 0x70, 0x61,
0x67, 0x6f, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x47, 0x65, 0x6f, 0x72,
0x67, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x73, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x47, 0x69,
0x6c, 0x62, 0x65, 0x72, 0x74, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x6d, 0x65, 0x72, 0x69, 0x64, 0x69, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20,
0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x47, 0x6f, 0x6c, 0x66, 0x6f, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0xe1, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x48, 0x61, 0x77, 0xe1, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x69, 0x61, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x48, 0x61, 0x77, 0xe1, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69,
0x61, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f,
0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x48, 0x6f, 0x76, 0x64, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x6f, 0x63, 0xe9, 0x61, 0x6e, 0x6f, 0x20, 0xcd, 0x6e,
0x64, 0x69, 0x63, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x6f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61,
0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0xe1, 0x6e, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4a, 0x61, 0x70, 0xf3, 0x6e, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x4a, 0x61, 0x70, 0xf3, 0x6e, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4a, 0x61, 0x70, 0xf3, 0x6e, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x6b,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61,
0x74, 0x6b, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x6d,
0x63, 0x68, 0x61, 0x74, 0x6b, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6a, 0x69, 0x73, 0x74, 0xe1, 0x6e,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x7a, 0x61,
0x6a, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b,
0x61, 0x7a, 0x61, 0x6a, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0x6f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79,
0x61, 0x72, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4b, 0x69, 0x72, 0x67, 0x75, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x72, 0x69, 0x20, 0x4c, 0x61,
0x6e, 0x6b, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x73, 0x20, 0x45, 0x73, 0x70, 0xf3, 0x72, 0x61, 0x64, 0x61, 0x73,
0x20, 0x45, 0x63, 0x75, 0x61, 0x74, 0x6f, 0x72, 0x69, 0x61, 0x6c, 0x65,
0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x61, 0x63, 0x61, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61,
0x63, 0x61, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x63,
0x61, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61,
0x67, 0x61, 0x64, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61,
0x67, 0x61, 0x64, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x61, 0x67, 0x61, 0x64, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x61, 0x6c, 0x61, 0x73, 0x69, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x61,
0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68,
0x61, 0x6c, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x63, 0x69, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4d, 0xe9, 0x78, 0x69, 0x63, 0x6f, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4d, 0xe9, 0x78, 0x69, 0x63, 0x6f, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4d, 0xe9, 0x78, 0x69, 0x63, 0x6f, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0xe1, 0x6e, 0x20, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0xe1, 0x6e,
0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x55, 0x6c, 0xe1, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0xfa, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0xfa, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0xfa, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x75, 0x65, 0x76, 0x61, 0x20, 0x43, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x75, 0x65, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x75, 0x65,
0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x75, 0x65, 0x76,
0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x75, 0x65, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x75, 0x65,
0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x6e,
0x6f, 0x76, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x69, 0x73,
0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x73, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x6b, 0x69,
0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x6f, 0x73, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x70, 0xfa, 0x61, 0x20, 0x4e, 0x75,
0x65, 0x76, 0x61, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x72,
0xfa, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x72, 0xfa, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x72, 0xfa, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x73, 0x20, 0x46, 0xe9, 0x6e, 0x69, 0x78, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6e, 0x20, 0x50, 0x65,
0x64, 0x72, 0x6f, 0x20, 0x79, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0xf3, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6e, 0x20, 0x50,
0x65, 0x64, 0x72, 0x6f, 0x20, 0x79, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0xf3, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6e,
0x20, 0x50, 0x65, 0x64, 0x72, 0x6f, 0x20, 0x79, 0x20, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0xf3, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x51,
0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6a, 0x61, 0x6c, 0xed, 0x6e, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6a, 0x61, 0x6c, 0xed, 0x6e, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6a, 0x61, 0x6c, 0xed, 0x6e,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65,
0x6c, 0x6c, 0x65, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x53, 0x61,
0x6c, 0x6f, 0x6d, 0xf3, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x53, 0x75, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x54, 0x61, 0x69, 0x70, 0xe9, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x54,
0x61, 0x69, 0x70, 0xe9, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x54,
0x61, 0x69, 0x70, 0xe9, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x54, 0x61, 0x79, 0x69, 0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x79, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x79, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x55,
0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x20, 0x57,
0x61, 0x6b, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x57,
0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x79, 0x20, 0x46, 0x75, 0x74, 0x75,
0x6e, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x45, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x59, 0x75, 0x6b, 0xf3, 0x6e, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x6d, 0x6f,
0x6e, 0x74, 0x61, 0xf1, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61,
0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0xf1, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0xf1, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x73, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x73,
0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61,
0x73, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x6d, 0x65, 0x64, 0x69, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x73, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f,
0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73,
0x63, 0x75, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20,
0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x45, 0x73, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x45, 0x73, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x45, 0x73, 0x74,
0x65, 0x68, 0x6f, 0x72, 0x61, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x6c, 0x65, 0x6a, 0x61, 0x6e, 0x6f, 0x20, 0x65, 0x73, 0x74, 0x65,
0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x4f, 0x65, 0x73, 0x74, 0x65, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x4f, 0x65, 0x73, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x4f, 0x65, 0x73, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x61, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x4d,
0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61,
0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61,
0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x54, 0x69, 0x65, 0x72, 0x72, 0x61,
0x73, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x73, 0x20,
0x79, 0x20, 0x41, 0x6e, 0x74, 0xe1, 0x72, 0x74, 0x69, 0x63, 0x61, 0x73,
0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x61, 0x73, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20,
0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61,
0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x20, 0x28, 0x42,
0x69, 0x72, 0x6d, 0x61, 0x6e, 0x69, 0x61, 0x29, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x69, 0x6f, 0x6e,
0x79, 0x61, 0x6e, 0x67, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0xf3,
0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x41, 0x70, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x61, 0x76, 0x69, 0x64, 0x61, 0x64, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61,
0x73, 0x63, 0x75, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e,
0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73,
0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c,
0x76, 0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x61, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x47,
0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x57, 0x61,
0x6b, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68,
0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x4f, 0x63, 0xe9, 0x61, 0x6e, 0x6f, 0x20, 0xcd, 0x6e,
0x64, 0x69, 0x63, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x73, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61,
0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x73,
0x20, 0x46, 0xe9, 0x6e, 0x69, 0x78, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x55, 0x73, 0x75, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x54,
0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x57,
0xe9, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x57, 0xe9, 0x74, 0x61, 0x6e, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x55, 0x73, 0x75, 0x6d, 0x20, 0x50, 0x61,
0x6e, 0x61, 0x73, 0x20, 0x57, 0xe9, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x50, 0x61, 0x67, 0x75, 0x6e, 0x75, 0x6e, 0x67, 0x61,
0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x50, 0x61, 0x67, 0x75, 0x6e, 0x75, 0x6e, 0x67, 0x61,
0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x55, 0x73, 0x75, 0x6d, 0x20,
0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x50, 0x61, 0x67, 0x75, 0x6e, 0x75,
0x6e, 0x67, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x55, 0x73,
0x75, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x50, 0x61, 0x73,
0x69, 0x66, 0x69, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x55, 0x73,
0x75, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x55,
0x73, 0x75, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x4b, 0x6f,
0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0xc9, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0xc9, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x54, 0x65, 0x6e, 0x67,
0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x55, 0x73, 0x75, 0x6d,
0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0xc9, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0xc9, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0xc9, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x75,
0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x55, 0x73, 0x75, 0x6d, 0x20,
0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0xc9, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0xc9,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0xc9, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x55, 0x73, 0x75, 0x6d, 0x20, 0x50, 0x61,
0x6e, 0x61, 0x73, 0x20, 0xc9, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x42, 0x61,
0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x41, 0x66, 0x67, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x61, 0x74, 0x69, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x4d,
0x61, 0x73, 0x68, 0x61, 0x72, 0x69, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x4b, 0x75, 0x73,
0x69, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69,
0x62, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72,
0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61,
0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x68,
0x61, 0x72, 0x69, 0x62, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41,
0x6c, 0x61, 0x73, 0x6b, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a,
0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x61, 0x74, 0x69,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x61, 0x74, 0x69, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4b, 0x61, 0x74, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69, 0x6b, 0x69, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69,
0x6b, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68,
0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61,
0x72, 0x69, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66,
0x69, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63,
0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x61, 0x73, 0x69,
0x66, 0x69, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x41,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4b, 0x69, 0x61, 0x6e, 0x67, 0x61, 0x7a, 0x69, 0x20, 0x7a, 0x61, 0x20,
0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x41, 0x70, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x41,
0x70, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63,
0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x70, 0x69, 0x61,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x61, 0x72, 0x61, 0x62,
0x75, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x61, 0x72,
0x61, 0x62, 0x75, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x72,
0x61, 0x62, 0x69, 0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x20, 0x7a, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69,
0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a,
0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72,
0x69, 0x62, 0x69, 0x20, 0x6d, 0x77, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x20, 0x6d, 0x77, 0x61,
0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20,
0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x20, 0x6d, 0x77, 0x61,
0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69,
0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a,
0x61, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x79,
0x61, 0x20, 0x4b, 0x61, 0x74, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x79, 0x61,
0x20, 0x4b, 0x61, 0x74, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4b,
0x61, 0x74, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61,
0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x20, 0x79, 0x61, 0x20, 0x4b,
0x61, 0x74, 0x69, 0x20, 0x79, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57,
0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61,
0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x20, 0x79, 0x61, 0x20, 0x4b,
0x61, 0x74, 0x69, 0x20, 0x79, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x67,
0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x61,
0x74, 0x69, 0x20, 0x79, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x4d, 0x61, 0x73, 0x68,
0x61, 0x72, 0x69, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x73, 0x68, 0x61, 0x72, 0x69, 0x6b, 0x69, 0x20, 0x6d, 0x77, 0x61,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69, 0x6b, 0x69,
0x20, 0x6d, 0x77, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61,
0x72, 0x69, 0x62, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57,
0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x68,
0x61, 0x72, 0x69, 0x62, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x68,
0x61, 0x72, 0x69, 0x62, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x20, 0x7a, 0x61, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a,
0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a,
0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20,
0x7a, 0x61, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61,
0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x7a, 0x6f, 0x72,
0x65, 0x73, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x7a, 0x6f, 0x72,
0x65, 0x73, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a,
0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20,
0x7a, 0x61, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x73, 0x68, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a,
0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x73, 0x68, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x7a, 0x61, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69,
0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69,
0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a,
0x61, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20,
0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65,
0x72, 0x64, 0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x43, 0x61, 0x70,
0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20,
0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x43, 0x61, 0x70, 0x65,
0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20,
0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x20, 0x7a, 0x61, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x7a, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20,
0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x43,
0x68, 0x69, 0x6c, 0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x55,
0x63, 0x68, 0x69, 0x6e, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x55,
0x63, 0x68, 0x69, 0x6e, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x63,
0x68, 0x69, 0x6e, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b,
0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x63, 0x68, 0x61, 0x20, 0x4b, 0x72,
0x69, 0x73, 0x6d, 0x61, 0x73, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x56, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x76, 0x79, 0x61, 0x20,
0x43, 0x6f, 0x63, 0x6f, 0x73, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20,
0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4b,
0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x56, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x76, 0x79, 0x61,
0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x56,
0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x76, 0x79, 0x61, 0x20, 0x43, 0x6f,
0x6f, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a,
0x69, 0x72, 0x61, 0x20, 0x6e, 0x75, 0x73, 0x75, 0x20, 0x79, 0x61, 0x20,
0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x69, 0x73, 0x69,
0x77, 0x61, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x79, 0x61, 0x20,
0x4b, 0x75, 0x62, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x75, 0x62,
0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x44, 0x61, 0x76, 0x69,
0x73, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x44, 0x75, 0x6d, 0x6f,
0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72,
0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69, 0x6b, 0x69, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20,
0x63, 0x68, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x20, 0x7a, 0x61, 0x20, 0x4b, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x63,
0x68, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79,
0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x69,
0x73, 0x69, 0x77, 0x61, 0x20, 0x63, 0x68, 0x61, 0x20, 0x45, 0x61, 0x73,
0x74, 0x65, 0x72, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x45, 0x6b,
0x77, 0x61, 0x64, 0x6f, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x55,
0x6c, 0x61, 0x79, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x61, 0x74, 0x69,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x79, 0x61, 0x20,
0x79, 0x61, 0x20, 0x4b, 0x61, 0x74, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20,
0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x79,
0x61, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x61, 0x74, 0x69, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69, 0x6b,
0x69, 0x20, 0x6d, 0x77, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x79, 0x61, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69,
0x6b, 0x69, 0x20, 0x6d, 0x77, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x79, 0x61,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72,
0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69, 0x6b, 0x69, 0x20, 0x6d,
0x77, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x79, 0x61, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69, 0x6b, 0x69,
0x20, 0x7a, 0x61, 0x69, 0x64, 0x69, 0x20, 0x6d, 0x77, 0x61, 0x20, 0x55,
0x6c, 0x61, 0x79, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x20, 0x6d, 0x77, 0x61,
0x20, 0x55, 0x6c, 0x61, 0x79, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x20, 0x6d, 0x77,
0x61, 0x20, 0x55, 0x6c, 0x61, 0x79, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20,
0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x68,
0x61, 0x72, 0x69, 0x62, 0x69, 0x20, 0x6d, 0x77, 0x61, 0x20, 0x55, 0x6c,
0x61, 0x79, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x69,
0x73, 0x69, 0x77, 0x61, 0x20, 0x76, 0x79, 0x61, 0x20, 0x46, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x56,
0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x76, 0x79, 0x61, 0x20, 0x46, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a,
0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x69, 0x73, 0x69, 0x77,
0x61, 0x20, 0x76, 0x79, 0x61, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x46, 0x69, 0x6a,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61,
0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20,
0x46, 0x69, 0x6a, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x47,
0x75, 0x69, 0x61, 0x6e, 0x61, 0x20, 0x79, 0x61, 0x20, 0x55, 0x66, 0x61,
0x72, 0x61, 0x6e, 0x73, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4b, 0x75, 0x73, 0x69, 0x6e, 0x69, 0x20, 0x6d, 0x77, 0x61, 0x20, 0x55,
0x66, 0x61, 0x72, 0x61, 0x6e, 0x73, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x41,
0x6e, 0x74, 0x61, 0x6b, 0x74, 0x69, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69,
0x65, 0x72, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4a, 0x6f, 0x6a,
0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4a, 0x6f, 0x6a, 0x69,
0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69,
0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a,
0x61, 0x20, 0x4a, 0x6f, 0x6a, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x56, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x76, 0x79, 0x61,
0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69,
0x6b, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72,
0x69, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61,
0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f,
0x20, 0x7a, 0x61, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69, 0x6b, 0x69, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69,
0x62, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a,
0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20,
0x7a, 0x61, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x20, 0x7a, 0x61, 0x20, 0x47, 0x75, 0x6c, 0x66, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61,
0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x69, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6e, 0x67, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57,
0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x48, 0x6f,
0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20,
0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x48, 0x6f, 0x6e, 0x67,
0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x48, 0x6f, 0x76, 0x64, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57,
0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x48, 0x6f,
0x76, 0x64, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a,
0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20,
0x7a, 0x61, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x42, 0x61, 0x68, 0x61, 0x72, 0x69, 0x20, 0x48, 0x69, 0x6e, 0x64,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f,
0x63, 0x68, 0x69, 0x6e, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x79, 0x61,
0x20, 0x4b, 0x61, 0x74, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69, 0x6b, 0x69, 0x20, 0x6d, 0x77,
0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72,
0x69, 0x62, 0x69, 0x20, 0x6d, 0x77, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x73, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x49, 0x72, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57,
0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x49, 0x72,
0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68,
0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a,
0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20,
0x7a, 0x61, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x69,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61,
0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4a, 0x61, 0x70, 0x61,
0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61,
0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70,
0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63,
0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20,
0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73,
0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x69, 0x61, 0x6e, 0x67,
0x61, 0x7a, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b,
0x68, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72,
0x69, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x61,
0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x67,
0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20,
0x4b, 0x6f, 0x72, 0x65, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x6f,
0x72, 0x65, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x6f,
0x73, 0x72, 0x61, 0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x20, 0x7a, 0x61, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x73, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a,
0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20,
0x7a, 0x61, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x79, 0x72,
0x67, 0x79, 0x73, 0x74, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x56, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x76, 0x79, 0x61, 0x20,
0x4c, 0x69, 0x6e, 0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74,
0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6c, 0x61, 0x79,
0x73, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61,
0x6c, 0x64, 0x69, 0x76, 0x65, 0x73, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20,
0x76, 0x79, 0x61, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x6f, 0x72, 0x69, 0x73,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x6f, 0x72, 0x69, 0x73,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69,
0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x6f, 0x72, 0x69, 0x73, 0x69, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x69,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x77, 0x61, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x7a, 0x61, 0x20, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b,
0x69, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x6d, 0x63, 0x68, 0x61, 0x6e,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b,
0x69, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20,
0x42, 0x61, 0x74, 0x6f, 0x72, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x55,
0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79,
0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x6c,
0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79,
0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x6f,
0x73, 0x63, 0x6f, 0x77, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20,
0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79,
0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4e, 0x65,
0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65,
0x61, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4e,
0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61,
0x6e, 0x64, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4e, 0x65, 0x77,
0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20,
0x7a, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c,
0x61, 0x6e, 0x64, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63,
0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x66,
0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4b, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x63, 0x68, 0x61,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x4b, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x63, 0x68, 0x61,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61,
0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x69, 0x73,
0x69, 0x77, 0x61, 0x20, 0x63, 0x68, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x68, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a,
0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69,
0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a,
0x61, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4f, 0x6d, 0x73, 0x6b,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20,
0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4f,
0x6d, 0x73, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61,
0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x50, 0x61, 0x6c, 0x61, 0x75, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x47, 0x75,
0x69, 0x6e, 0x65, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x50,
0x61, 0x72, 0x61, 0x67, 0x77, 0x61, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61,
0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x77, 0x61, 0x69, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79,
0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x61,
0x72, 0x61, 0x67, 0x77, 0x61, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x50, 0x65, 0x72, 0x75, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x50,
0x65, 0x72, 0x75, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61,
0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f,
0x20, 0x7a, 0x61, 0x20, 0x50, 0x65, 0x72, 0x75, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x55, 0x66, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x6f,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x66, 0x69, 0x6c, 0x69, 0x70,
0x69, 0x6e, 0x6f, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61,
0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f,
0x20, 0x7a, 0x61, 0x20, 0x55, 0x66, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e,
0x6f, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x69, 0x73, 0x69,
0x77, 0x61, 0x20, 0x76, 0x79, 0x61, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e,
0x69, 0x78, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x61, 0x69,
0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x6e, 0x61,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20,
0x79, 0x61, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63,
0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x61, 0x69, 0x6e,
0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x6e, 0x61, 0x20,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67,
0x79, 0x61, 0x6e, 0x67, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x52,
0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69,
0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a,
0x61, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x69, 0x61, 0x6e, 0x67, 0x61,
0x7a, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61,
0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x55, 0x73, 0x68, 0x65, 0x6c, 0x69, 0x73, 0x68, 0x65, 0x6c, 0x69, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f,
0x72, 0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x69, 0x73,
0x69, 0x77, 0x61, 0x20, 0x76, 0x79, 0x61, 0x20, 0x53, 0x6f, 0x6c, 0x6f,
0x6d, 0x6f, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x47, 0x65,
0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x4b, 0x75, 0x73, 0x69, 0x6e, 0x69,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e,
0x61, 0x6d, 0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x79,
0x6f, 0x77, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x54, 0x61,
0x68, 0x69, 0x74, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x54, 0x61,
0x69, 0x70, 0x65, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x54,
0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x7a, 0x61, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20,
0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x43,
0x68, 0x75, 0x75, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x54,
0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x7a, 0x61, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f,
0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x55, 0x72, 0x75, 0x67, 0x77, 0x61, 0x69, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20,
0x7a, 0x61, 0x20, 0x55, 0x72, 0x75, 0x67, 0x77, 0x61, 0x69, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20,
0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x55,
0x72, 0x75, 0x67, 0x77, 0x61, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61,
0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f,
0x20, 0x7a, 0x61, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57,
0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f,
0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x65, 0x6e, 0x65,
0x7a, 0x75, 0x65, 0x6c, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74,
0x6f, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61,
0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20,
0x63, 0x68, 0x61, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x6e, 0x61,
0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20,
0x7a, 0x61, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20,
0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x59,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x59, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79,
0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x59, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x66, 0x67, 0x68, 0x61, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a,
0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61,
0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x41, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20,
0x7a, 0x61, 0x20, 0x41, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61,
0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x6a, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x20, 0x6d, 0x77,
0x61, 0x20, 0x41, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62,
0x69, 0x20, 0x6d, 0x77, 0x61, 0x20, 0x41, 0x6a, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a,
0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69,
0x20, 0x6d, 0x77, 0x61, 0x20, 0x41, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69,
0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a,
0x61, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62,
0x69, 0x20, 0x6d, 0x77, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x69,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x61, 0x74, 0x69,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72,
0x69, 0x62, 0x69, 0x20, 0x6d, 0x77, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x61, 0x74,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61,
0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72,
0x69, 0x62, 0x69, 0x20, 0x6d, 0x77, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x61, 0x74,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x7a, 0x61, 0x62,
0x61, 0x6a, 0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x41,
0x7a, 0x61, 0x62, 0x61, 0x6a, 0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61,
0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x7a, 0x61,
0x62, 0x61, 0x6a, 0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a,
0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65,
0x73, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61,
0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x69,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72,
0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61,
0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x69,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x42, 0x75, 0x74, 0x61, 0x6e,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x42, 0x72, 0x61, 0x7a,
0x69, 0x6c, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x42, 0x72, 0x61,
0x7a, 0x69, 0x6c, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74,
0x6f, 0x20, 0x7a, 0x61, 0x20, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x76,
0x65, 0x64, 0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x65, 0x70,
0x75, 0x76, 0x65, 0x64, 0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f,
0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x76, 0x65,
0x64, 0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a,
0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20,
0x7a, 0x61, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61,
0x20, 0x4e, 0x75, 0x73, 0x75, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74,
0x6f, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20,
0x76, 0x79, 0x61, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a,
0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x69, 0x73, 0x69, 0x77,
0x61, 0x20, 0x63, 0x68, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72,
0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61,
0x20, 0x55, 0x6c, 0x61, 0x79, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x61,
0x74, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a,
0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69, 0x6b, 0x69,
0x20, 0x6d, 0x77, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x79, 0x61, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20,
0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x20, 0x6d, 0x77, 0x61,
0x20, 0x55, 0x6c, 0x61, 0x79, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4a, 0x69, 0x6f, 0x6a, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61,
0x20, 0x4a, 0x69, 0x6f, 0x6a, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20,
0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4a, 0x69, 0x6f, 0x6a,
0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a,
0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20,
0x7a, 0x61, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69, 0x6b, 0x69, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x20, 0x7a, 0x61, 0x20, 0x47, 0x68, 0x75, 0x62, 0x61, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79,
0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x48, 0x6f,
0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20,
0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x48, 0x6f, 0x76, 0x64,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x69,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x69, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x69, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79,
0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4a, 0x61, 0x70, 0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4a,
0x61, 0x70, 0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20,
0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69, 0x6b, 0x69, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69,
0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a,
0x61, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73,
0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69,
0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6c, 0x65, 0x73, 0x69, 0x61, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69,
0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x6f, 0x72, 0x69, 0x73, 0x69, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x61,
0x61, 0x72, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x61,
0x6e, 0x62, 0x61, 0x74, 0x61, 0x61, 0x72, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20,
0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x61,
0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20,
0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x6f, 0x73, 0x63,
0x6f, 0x77, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4e, 0x65, 0x70,
0x61, 0x6c, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x4d, 0x70, 0x79, 0x61,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x69, 0x61, 0x20, 0x4d, 0x70, 0x79, 0x61, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61,
0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x4d, 0x70, 0x79, 0x61, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4e, 0x79, 0x75, 0x7a, 0x69, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57,
0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4e, 0x79,
0x75, 0x7a, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4e, 0x79, 0x75, 0x7a, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61,
0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20,
0x4b, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x63, 0x68, 0x61, 0x20, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a,
0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a,
0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a,
0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74,
0x6f, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x77, 0x61,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69,
0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a,
0x61, 0x20, 0x50, 0x65, 0x72, 0x75, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a,
0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x66, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x6f, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x56,
0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x76, 0x79, 0x61, 0x20, 0x46, 0x69,
0x6e, 0x69, 0x6b, 0x73, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f,
0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68,
0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70,
0x6f, 0x6f, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4a, 0x6f, 0x6a,
0x69, 0x61, 0x20, 0x4b, 0x75, 0x73, 0x69, 0x6e, 0x69, 0x53, 0x61, 0x61,
0x7a, 0x61, 0x20, 0x54, 0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a,
0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20,
0x7a, 0x61, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57,
0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x54, 0x75,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61,
0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69,
0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a,
0x61, 0x20, 0x55, 0x72, 0x75, 0x67, 0x77, 0x61, 0x69, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x77, 0x61,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x7a, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61,
0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x7a, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61,
0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74,
0x6f, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74,
0x6f, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a,
0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20,
0x7a, 0x61, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20,
0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x59,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x76, 0xe4, 0x73, 0x74, 0x62, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61,
0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x62,
0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74,
0x62, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0xf6, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x73, 0x79, 0x64,
0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x76, 0xe4, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x61, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x61, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x74, 0x69,
0x64, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2c, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x41,
0x6c, 0x6d, 0x61, 0x74, 0x79, 0x74, 0x69, 0x64, 0x41, 0x6c, 0x6d, 0x61,
0x74, 0x79, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41,
0x6c, 0x6d, 0x61, 0x74, 0x79, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x2c, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e,
0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x72, 0x64, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0xf6, 0x73, 0x74, 0x6e, 0x6f,
0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0xf6, 0x73, 0x74, 0x6e, 0x6f, 0x72, 0x64, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0xf6, 0x73, 0x74, 0x6e, 0x6f,
0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x4b, 0x6c,
0x69, 0x70, 0x70, 0x69, 0x67, 0x61, 0x20, 0x62, 0x65, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x64, 0x4b, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x67, 0x61,
0x20, 0x62, 0x65, 0x72, 0x67, 0x65, 0x6e, 0x2c, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4b, 0x6c, 0x69, 0x70, 0x70, 0x69,
0x67, 0x61, 0x20, 0x62, 0x65, 0x72, 0x67, 0x65, 0x6e, 0x2c, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74,
0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x6e, 0x6f,
0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x76, 0xe4,
0x73, 0x74, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x74, 0x69, 0x64, 0x41,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x41, 0x70, 0x69, 0x61, 0x74, 0x69, 0x64,
0x41, 0x70, 0x69, 0x61, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x41, 0x70, 0x69, 0x61, 0x2c, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x41, 0x71, 0x74, 0x61, 0x75, 0x74,
0x69, 0x64, 0x41, 0x71, 0x74, 0x61, 0x75, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x41, 0x71, 0x74, 0x61, 0x75, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x41, 0x71, 0x74, 0xf6, 0x62, 0x65,
0x74, 0x69, 0x64, 0x41, 0x71, 0x74, 0xf6, 0x62, 0x65, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x71, 0x74, 0xf6, 0x62, 0x65,
0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x73, 0x61, 0x75,
0x64, 0x69, 0x61, 0x72, 0x61, 0x62, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x73, 0x61, 0x75, 0x64, 0x69, 0x61, 0x72, 0x61, 0x62, 0x69, 0x73,
0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x73,
0x61, 0x75, 0x64, 0x69, 0x61, 0x72, 0x61, 0x62, 0x69, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0xf6, 0x73, 0x74,
0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0xf6, 0x73, 0x74, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69,
0x64, 0xf6, 0x73, 0x74, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x76, 0xe4, 0x73, 0x74, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x61, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x61, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x61, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x61, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x61, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x61, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x74, 0x69, 0x64, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x61, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73,
0x74, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0xe4,
0x73, 0x74, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0xf6, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0xf6, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0xf6, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73,
0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73,
0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x61, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x6a, 0x64, 0x7a, 0x6a, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x61, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x7a,
0x6a, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x61, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x7a,
0x6a, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0x69, 0x64, 0x61, 0x7a, 0x6f, 0x72, 0x69, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x61, 0x7a, 0x6f, 0x72, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x61, 0x7a, 0x6f, 0x72, 0x69,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x2c, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x42, 0x72, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x61, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x74, 0x69, 0x64, 0x4b,
0x61, 0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x74, 0x69, 0x64, 0x4b,
0x61, 0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x2c, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4b, 0x61, 0x70, 0x20, 0x56,
0x65, 0x72, 0x64, 0x65, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0x69, 0x64, 0x43, 0x61, 0x73, 0x65, 0x79, 0x74, 0x69, 0x64, 0x43,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x74, 0x69, 0x64, 0x43, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x2c, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x4a, 0x75, 0x6c,
0xf6, 0x6e, 0x73, 0x20, 0x74, 0x69, 0x64, 0x4b, 0x65, 0x65, 0x6c, 0x69,
0x6e, 0x67, 0xf6, 0x61, 0x72, 0x6e, 0x61, 0x73, 0x20, 0x74, 0x69, 0x64,
0x63, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x43, 0x6f, 0x6f,
0x6b, 0xf6, 0x61, 0x72, 0x6e, 0x61, 0x73, 0x20, 0x74, 0x69, 0x64, 0x43,
0x6f, 0x6f, 0x6b, 0xf6, 0x61, 0x72, 0x6e, 0x61, 0x73, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x43, 0x6f, 0x6f, 0x6b, 0xf6,
0x61, 0x72, 0x6e, 0x61, 0x73, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0x69, 0x64, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x20, 0x64, 0x2019,
0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x2d, 0x74, 0x69, 0x64, 0xf6,
0x73, 0x74, 0x74, 0x69, 0x6d, 0x6f, 0x72, 0x69, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x50, 0xe5, 0x73, 0x6b, 0xf6, 0x74, 0x69, 0x64, 0x50, 0xe5,
0x73, 0x6b, 0xf6, 0x6e, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x50, 0xe5, 0x73, 0x6b, 0xf6, 0x6e, 0x2c, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0xf6, 0x73, 0x74, 0x65,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0xf6, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0xf6, 0x73,
0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x4b, 0x61, 0x6c, 0x69,
0x6e, 0x69, 0x6e, 0x67, 0x72, 0x61, 0x64, 0x74, 0x69, 0x64, 0x76, 0xe4,
0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0xf6, 0x61,
0x72, 0x6e, 0x61, 0x73, 0x20, 0x74, 0x69, 0x64, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0xf6, 0x61, 0x72, 0x6e, 0x61, 0x2c, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x46, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0xf6, 0x61, 0x72, 0x6e, 0x61, 0x2c,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x46, 0x69,
0x6a, 0x69, 0x74, 0x69, 0x64, 0x46, 0x69, 0x6a, 0x69, 0x2c, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x46, 0x69, 0x6a, 0x69,
0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x46,
0x72, 0x61, 0x6e, 0x73, 0x6b, 0x61, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e,
0x61, 0x74, 0x69, 0x64, 0x46, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x61, 0x20,
0x53, 0x79, 0x64, 0x74, 0x65, 0x72, 0x72, 0x69, 0x74, 0x6f, 0x72, 0x69,
0x65, 0x72, 0x6e, 0x61, 0x73, 0x20, 0x74, 0x69, 0x64, 0x47, 0x61, 0x6c,
0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x74, 0x69, 0x64, 0x47, 0x61, 0x6d,
0x62, 0x69, 0x65, 0x72, 0x74, 0x69, 0x64, 0x4b, 0x69, 0x72, 0x69, 0x62,
0x61, 0x74, 0x69, 0x74, 0x69, 0x64, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77,
0x69, 0x63, 0x68, 0x74, 0x69, 0x64, 0xf6, 0x73, 0x74, 0x67, 0x72, 0xf6,
0x6e, 0x6c, 0xe4, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0xf6,
0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0xe4, 0x6e, 0x64, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0xf6, 0x73,
0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0xe4, 0x6e, 0x64, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73,
0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0xe4, 0x6e, 0x64, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c,
0xe4, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c,
0xe4, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0x69, 0x64, 0x47, 0x75, 0x61, 0x6d, 0x74, 0x69, 0x64, 0x50, 0x65,
0x72, 0x73, 0x69, 0x73, 0x6b, 0x61, 0x20, 0x76, 0x69, 0x6b, 0x65, 0x6e,
0x74, 0x69, 0x64, 0x48, 0x6f, 0x6e, 0x6f, 0x6c, 0x75, 0x6c, 0x75, 0x74,
0x69, 0x64, 0x48, 0x6f, 0x6e, 0x6f, 0x6c, 0x75, 0x6c, 0x75, 0x2c, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x48, 0x6f, 0x6e,
0x6f, 0x6c, 0x75, 0x6c, 0x75, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67,
0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x48,
0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x2c, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x43, 0x68, 0x6f, 0x76, 0x64, 0x74,
0x69, 0x64, 0x43, 0x68, 0x6f, 0x76, 0x64, 0x2c, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x43, 0x68, 0x6f, 0x76, 0x64, 0x2c,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x42, 0x72,
0x69, 0x74, 0x74, 0x69, 0x73, 0x6b, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x69,
0x73, 0x6b, 0x61, 0x20, 0x6f, 0x63, 0x65, 0x61, 0x6e, 0xf6, 0x61, 0x72,
0x6e, 0x61, 0x73, 0x20, 0x74, 0x69, 0x64, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0xf6, 0x73, 0x74, 0x69, 0x6e, 0x64, 0x6f, 0x6e,
0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73,
0x74, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x74, 0x69,
0x64, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2c, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x69, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x69, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x73, 0x6b, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x69, 0x73, 0x72,
0x61, 0x65, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x4b, 0x61, 0x6d, 0x74, 0x6a, 0x61, 0x74, 0x6b,
0x61, 0x74, 0x69, 0x64, 0x4b, 0x61, 0x6d, 0x74, 0x6a, 0x61, 0x74, 0x6b,
0x61, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4b, 0x61,
0x6d, 0x74, 0x6a, 0x61, 0x74, 0x6b, 0x61, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x6b, 0x61, 0x7a, 0x61, 0x6b, 0x73, 0x74, 0x61,
0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0xf6, 0x73, 0x74, 0x6b, 0x61,
0x7a, 0x61, 0x6b, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x76, 0xe4, 0x73, 0x74, 0x6b, 0x61, 0x7a, 0x61, 0x6b, 0x73, 0x74,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2c, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x6b, 0x69, 0x72, 0x67, 0x69, 0x7a, 0x69, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x53, 0x72, 0x69, 0x20, 0x4c, 0x61, 0x6e,
0x6b, 0x61, 0x74, 0x69, 0x64, 0x4c, 0x69, 0x6e, 0x65, 0xf6, 0x61, 0x72,
0x6e, 0x61, 0x73, 0x20, 0x74, 0x69, 0x64, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x74, 0x69, 0x64, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x4d,
0x61, 0x63, 0x61, 0x6f, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x63, 0x61, 0x6f,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x63,
0x61, 0x6f, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x4d,
0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x72, 0x6e, 0x61, 0x74, 0x69, 0x64,
0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x74, 0x69, 0x64,
0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0xf6, 0x61, 0x72, 0x6e,
0x61, 0x73, 0x20, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74,
0x69, 0x75, 0x73, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74,
0x69, 0x75, 0x73, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x2c,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x6d, 0x65,
0x78, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x69, 0x6c,
0x6c, 0x61, 0x68, 0x61, 0x76, 0x73, 0x74, 0x69, 0x64, 0x6d, 0x65, 0x78,
0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x69, 0x6c, 0x6c,
0x61, 0x68, 0x61, 0x76, 0x73, 0x74, 0x69, 0x64, 0x2c, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x6d, 0x65, 0x78, 0x69, 0x6b,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x69, 0x6c, 0x6c, 0x61, 0x68,
0x61, 0x76, 0x73, 0x74, 0x69, 0x64, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61,
0x61, 0x74, 0x61, 0x72, 0x74, 0x69, 0x64, 0x55, 0x6c, 0x61, 0x61, 0x6e,
0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61,
0x61, 0x74, 0x61, 0x72, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0x69, 0x64, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x74, 0x69, 0x64,
0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x2c,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x62, 0x75,
0x72, 0x6d, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x4e,
0x61, 0x75, 0x72, 0x75, 0x74, 0x69, 0x64, 0x6e, 0x65, 0x70, 0x61, 0x6c,
0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x4e, 0x79, 0x61,
0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x65, 0x6e, 0x74,
0x69, 0x64, 0x4e, 0x79, 0x61, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x69, 0x65, 0x6e, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x4e, 0x79, 0x61, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x69, 0x65, 0x6e, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x6e, 0x79, 0x7a, 0x65, 0x65, 0x6c, 0xe4, 0x6e,
0x64, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x6e, 0x79, 0x7a, 0x65, 0x65,
0x6c, 0xe4, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x6e, 0x79, 0x7a, 0x65, 0x65, 0x6c, 0xe4, 0x6e,
0x64, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e,
0x64, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64,
0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0xf6, 0x6e, 0x73, 0x20, 0x74, 0x69,
0x64, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0xf6, 0x6e, 0x73, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0xf6, 0x6e, 0x73, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x74, 0x69, 0x64, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2c, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0x69, 0x64, 0x4e, 0x6f, 0x72, 0x64, 0x6d, 0x61, 0x72, 0x69, 0x61,
0x6e, 0x65, 0x72, 0x6e, 0x61, 0x73, 0x20, 0x74, 0x69, 0x64, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2c, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2c, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x4f, 0x6d, 0x73, 0x6b, 0x74, 0x69,
0x64, 0x4f, 0x6d, 0x73, 0x6b, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x4f, 0x6d, 0x73, 0x6b, 0x2c, 0x20, 0x73, 0x6f,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x50, 0x61, 0x6c, 0x61, 0x75,
0x74, 0x69, 0x64, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x79, 0x61,
0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x73, 0x20, 0x74, 0x69, 0x64,
0x45, 0x6e, 0x64, 0x65, 0x72, 0x62, 0x75, 0x72, 0x79, 0x74, 0x69, 0x64,
0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x2d, 0x74, 0x69, 0x64, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x74, 0x69, 0x64, 0x50,
0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x74, 0x69, 0x64, 0x51,
0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x74, 0x69, 0x64, 0x51,
0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72,
0x64, 0x61, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x53,
0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x74, 0x69, 0x64, 0x53, 0x61,
0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x74, 0x69, 0x64, 0x53, 0x61, 0x6d,
0x61, 0x72, 0x61, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0x69, 0x64, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65,
0x72, 0x6e, 0x61, 0x74, 0x69, 0x64, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70,
0x6f, 0x72, 0x65, 0x74, 0x69, 0x64, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f,
0x6e, 0xf6, 0x61, 0x72, 0x6e, 0x61, 0x73, 0x20, 0x74, 0x69, 0x64, 0x73,
0x79, 0x64, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x74, 0x69, 0x64,
0x53, 0x79, 0x6f, 0x77, 0x61, 0x74, 0x69, 0x64, 0x54, 0x61, 0x68, 0x69,
0x74, 0x69, 0x74, 0x69, 0x64, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x74,
0x69, 0x64, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x2c, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x54, 0x61, 0x64, 0x7a, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x74, 0x69, 0x64, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x74, 0x69,
0x64, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x74, 0x69, 0x64, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69,
0x64, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x74, 0x69,
0x64, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x74, 0x69, 0x64, 0x75, 0x7a,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x75, 0x7a,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x75, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x74, 0x69, 0x64, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x2c, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2c, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2c, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x57, 0x61, 0x6b, 0x65,
0xf6, 0x61, 0x72, 0x6e, 0x61, 0x73, 0x20, 0x74, 0x69, 0x64, 0x57, 0x61,
0x6c, 0x6c, 0x69, 0x73, 0x2d, 0x20, 0x6f, 0x63, 0x68, 0x20, 0x46, 0x75,
0x74, 0x75, 0x6e, 0x61, 0xf6, 0x61, 0x72, 0x6e, 0x61, 0x73, 0x20, 0x74,
0x69, 0x64, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2c, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4a, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x74, 0x69, 0x64, 0x41, 0x63,
0x72, 0x65, 0x2d, 0x5a, 0x69, 0x69, 0x74, 0x41, 0x63, 0x72, 0x65, 0x2d,
0x53, 0x63, 0x68, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x7a, 0x69,
0x69, 0x74, 0x41, 0x63, 0x72, 0x65, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0x69, 0x69, 0x74, 0x41, 0x66, 0x67, 0x68, 0x61, 0x6e, 0x69,
0x73, 0x63, 0x68, 0x74, 0x61, 0x6e, 0x2d, 0x5a, 0x69, 0x69, 0x74, 0x5a,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x6e, 0x69, 0x73, 0x63, 0x68, 0x69, 0x20, 0x5a, 0x69, 0x69, 0x74, 0x4f,
0x73, 0x63, 0x68, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69,
0x73, 0x63, 0x68, 0x69, 0x20, 0x5a, 0x69, 0x69, 0x74, 0x53, 0xfc, 0xfc,
0x64, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68,
0x69, 0x20, 0x7a, 0x69, 0x69, 0x74, 0x57, 0x65, 0x73, 0x63, 0x68, 0x74,
0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x69,
0x20, 0x5a, 0x69, 0x69, 0x74, 0x57, 0x65, 0x73, 0x63, 0x68, 0x74, 0x61,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x69, 0x20,
0x53, 0x63, 0x68, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x7a, 0x69,
0x69, 0x74, 0x57, 0x65, 0x73, 0x63, 0x68, 0x74, 0x61, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x69, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0x69, 0x69, 0x74, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x2d, 0x5a, 0x69, 0x69, 0x74, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x2d, 0x53, 0x63, 0x68, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x7a,
0x69, 0x69, 0x74, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x69, 0x74, 0x41, 0x6c, 0x6d, 0x61,
0x74, 0x79, 0x2d, 0x5a, 0x69, 0x69, 0x74, 0x41, 0x6c, 0x6d, 0x61, 0x74,
0x79, 0x2d, 0x53, 0x63, 0x68, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x7a, 0x69, 0x69, 0x74, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x2d, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x69, 0x74, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x2d, 0x5a, 0x69, 0x69, 0x74, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x2d, 0x53, 0x63, 0x68, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x7a, 0x69, 0x69, 0x74, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0x69, 0x69, 0x74, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x2d,
0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x20, 0x5a, 0x69, 0x69,
0x74, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x2d, 0x5a, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x61, 0x6c, 0x20, 0x53, 0x63, 0x68, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x7a, 0x69, 0x69, 0x74, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x2d, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c,
0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x69, 0x74, 0x4d,
0x69, 0x74, 0x74, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69,
0x73, 0x63, 0x68, 0x69, 0x20, 0x5a, 0x69, 0x69, 0x74, 0x4d, 0x69, 0x74,
0x74, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73, 0x63,
0x68, 0x69, 0x20, 0x53, 0x63, 0x68, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x7a, 0x69, 0x69, 0x74, 0x4d, 0x69, 0x74, 0x74, 0x65, 0x6c, 0x65,
0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73, 0x63, 0x68, 0x69, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x69, 0x74, 0x4f, 0x73, 0x63,
0x68, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73, 0x63, 0x68,
0x69, 0x20, 0x5a, 0x69, 0x69, 0x74, 0x4f, 0x73, 0x63, 0x68, 0x74, 0x65,
0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73, 0x63, 0x68, 0x69, 0x20, 0x53,
0x63, 0x68, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x7a, 0x69, 0x69,
0x74, 0x4f, 0x73, 0x63, 0x68, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4,
0x69, 0x73, 0x63, 0x68, 0x69, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0x69, 0x69, 0x74, 0x57, 0x65, 0x73, 0x63, 0x68, 0x74, 0x65, 0x75,
0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73, 0x63, 0x68, 0x69, 0x20, 0x5a, 0x69,
0x69, 0x74, 0x57, 0x65, 0x73, 0x63, 0x68, 0x74, 0x65, 0x75, 0x72, 0x6f,
0x70, 0xe4, 0x69, 0x73, 0x63, 0x68, 0x69, 0x20, 0x53, 0x63, 0x68, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x7a, 0x69, 0x69, 0x74, 0x57, 0x65,
0x73, 0x63, 0x68, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73,
0x63, 0x68, 0x69, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69,
0x69, 0x74, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75, 0x65, 0x72, 0x20, 0x5a,
0x69, 0x69, 0x74, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75, 0x65, 0x72, 0x20,
0x53, 0x63, 0x68, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x7a, 0x69,
0x69, 0x74, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75, 0x65, 0x72, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x69, 0x74, 0x725, 0x715, 0x722,
0x718, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x71d, 0x71f, 0x71d, 0x72a, 0x725, 0x715,
0x722, 0x718, 0x72c, 0x710, 0x20, 0x72b, 0x72a, 0x72b, 0x71d, 0x72c, 0x710, 0x20,
0x725, 0x715, 0x722, 0x718, 0x72c, 0x710, 0x20, 0x72b, 0x72a, 0x72b, 0x71d, 0x72c,
0x710, 0x20, 0x715, 0x710, 0x71d, 0x71f, 0x71d, 0x72a, 0x725, 0x715, 0x722, 0x718,
0x72c, 0x710, 0x20, 0x729, 0x71d, 0x72c, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710,
0x71d, 0x71f, 0x71d, 0x72a, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x726,
0x713, 0x710, 0x722, 0x723, 0x72c, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x721, 0x728, 0x725, 0x71d, 0x72c, 0x710, 0x20, 0x710, 0x726, 0x72a, 0x71d,
0x729, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x721, 0x715, 0x722, 0x71a,
0x20, 0x710, 0x726, 0x72a, 0x71d, 0x729, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20,
0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x72c,
0x71d, 0x721, 0x722, 0x20, 0x710, 0x726, 0x72a, 0x71d, 0x729, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x721, 0x725, 0x72a, 0x712, 0x20, 0x710, 0x726, 0x72a,
0x71d, 0x729, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a,
0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x721, 0x725, 0x72a, 0x712, 0x20,
0x710, 0x726, 0x72a, 0x71d, 0x729, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x729,
0x71d, 0x72c, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x721, 0x725, 0x72a, 0x712, 0x20,
0x710, 0x726, 0x72a, 0x71d, 0x729, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x710, 0x720, 0x710, 0x723, 0x71f, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721,
0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x720,
0x710, 0x723, 0x71f, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x720, 0x710, 0x723,
0x71f, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x720, 0x721, 0x710,
0x72c, 0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c,
0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x720, 0x721, 0x710, 0x72c, 0x71d,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718,
0x721, 0x710, 0x20, 0x715, 0x710, 0x720, 0x721, 0x710, 0x72c, 0x71d, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x710, 0x721, 0x719, 0x722, 0x725, 0x715, 0x722, 0x710,
0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715,
0x710, 0x721, 0x719, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x721, 0x719, 0x722,
0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x728, 0x725, 0x71d, 0x72c, 0x710, 0x20,
0x715, 0x710, 0x721, 0x72a, 0x71d, 0x71f, 0x710, 0x20, 0x713, 0x72a, 0x712, 0x71d,
0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a,
0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x721, 0x728, 0x725, 0x71d, 0x72c, 0x710,
0x20, 0x715, 0x710, 0x721, 0x72a, 0x71d, 0x71f, 0x710, 0x20, 0x713, 0x72a, 0x712,
0x71d, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x721, 0x728, 0x725, 0x71d, 0x72c,
0x710, 0x20, 0x715, 0x710, 0x721, 0x72a, 0x71d, 0x71f, 0x710, 0x20, 0x713, 0x72a,
0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x715,
0x722, 0x71a, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x721, 0x72a, 0x71d, 0x71f,
0x710, 0x20, 0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722,
0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20,
0x721, 0x715, 0x722, 0x71a, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x721, 0x72a,
0x71d, 0x71f, 0x710, 0x20, 0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721,
0x710, 0x20, 0x721, 0x715, 0x722, 0x71a, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710,
0x721, 0x72a, 0x71d, 0x71f, 0x710, 0x20, 0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x71b, 0x718, 0x72a, 0x71d, 0x72c, 0x710,
0x20, 0x715, 0x710, 0x721, 0x72a, 0x71d, 0x71f, 0x710, 0x20, 0x713, 0x72a, 0x712,
0x71d, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718,
0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x71b, 0x718, 0x72a, 0x71d, 0x72c,
0x710, 0x20, 0x715, 0x710, 0x721, 0x72a, 0x71d, 0x71f, 0x710, 0x20, 0x713, 0x72a,
0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712,
0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x71b, 0x718, 0x72a, 0x71d,
0x72c, 0x710, 0x20, 0x715, 0x710, 0x721, 0x72a, 0x71d, 0x71f, 0x710, 0x20, 0x713,
0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x72b,
0x71d, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x721, 0x72a, 0x71d, 0x71f,
0x710, 0x20, 0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722,
0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20,
0x72b, 0x71d, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x721, 0x72a, 0x71d,
0x71f, 0x710, 0x20, 0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710,
0x20, 0x72b, 0x71d, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x721, 0x72a,
0x71d, 0x71f, 0x710, 0x20, 0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x722, 0x710, 0x715, 0x71d, 0x72a, 0x725,
0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c,
0x710, 0x20, 0x715, 0x710, 0x722, 0x710, 0x715, 0x71d, 0x72a, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20,
0x715, 0x710, 0x722, 0x710, 0x715, 0x71d, 0x72a, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x710, 0x726, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b,
0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x726, 0x71d,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d,
0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x726, 0x71d, 0x710, 0x725, 0x715, 0x722,
0x710, 0x20, 0x710, 0x71f, 0x72c, 0x710, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20,
0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710,
0x71f, 0x72c, 0x710, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x71f, 0x72c, 0x710,
0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x71f, 0x72c, 0x718, 0x712,
0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d,
0x72c, 0x710, 0x20, 0x715, 0x710, 0x71f, 0x72c, 0x718, 0x712, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20,
0x715, 0x710, 0x71f, 0x72c, 0x718, 0x712, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x710, 0x72a, 0x712, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b,
0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x72a, 0x712,
0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20,
0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x72a, 0x712, 0x71d, 0x710, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x72a, 0x713, 0x722, 0x72c, 0x71d, 0x722,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722,
0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x72a, 0x713, 0x722, 0x72c, 0x71d, 0x722,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d,
0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x72a, 0x713, 0x722, 0x72c, 0x71d, 0x722,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x72a, 0x713, 0x722, 0x72c,
0x71d, 0x722, 0x710, 0x20, 0x721, 0x725, 0x72a, 0x712, 0x71d, 0x72c, 0x710, 0x725,
0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c,
0x710, 0x20, 0x715, 0x710, 0x72a, 0x713, 0x722, 0x72c, 0x71d, 0x722, 0x710, 0x20,
0x721, 0x725, 0x72a, 0x712, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710,
0x72a, 0x713, 0x722, 0x72c, 0x71d, 0x722, 0x710, 0x20, 0x721, 0x725, 0x72a, 0x712,
0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x72a, 0x721,
0x722, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a,
0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x72a, 0x721, 0x722, 0x71d,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d,
0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x72a, 0x721, 0x722, 0x71d, 0x710, 0x725,
0x715, 0x722, 0x710, 0x20, 0x710, 0x71b, 0x720, 0x722, 0x71b, 0x71d, 0x72c, 0x710,
0x20, 0x715, 0x710, 0x721, 0x72a, 0x71d, 0x71f, 0x710, 0x20, 0x713, 0x72a, 0x712,
0x71d, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718,
0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x710, 0x71b, 0x720, 0x722, 0x71b,
0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x721, 0x72a, 0x71d, 0x71f, 0x710, 0x20,
0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x710, 0x71b,
0x720, 0x722, 0x71b, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x721, 0x72a, 0x71d,
0x71f, 0x710, 0x20, 0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x710, 0x718, 0x723, 0x72c, 0x72a, 0x720, 0x71d, 0x710,
0x20, 0x721, 0x728, 0x725, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20,
0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710,
0x718, 0x723, 0x72c, 0x72a, 0x720, 0x71d, 0x710, 0x20, 0x721, 0x728, 0x725, 0x71d,
0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20,
0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x718, 0x723, 0x72c, 0x72a, 0x720,
0x71d, 0x710, 0x20, 0x721, 0x728, 0x725, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722,
0x710, 0x20, 0x721, 0x725, 0x72a, 0x712, 0x71d, 0x72c, 0x710, 0x20, 0x721, 0x728,
0x725, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x718, 0x723, 0x72c, 0x72a, 0x720,
0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c,
0x722, 0x71d, 0x72c, 0x710, 0x20, 0x721, 0x725, 0x72a, 0x712, 0x71d, 0x72c, 0x710,
0x20, 0x721, 0x728, 0x725, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x718, 0x723,
0x72c, 0x72a, 0x720, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712,
0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x721, 0x725, 0x72a, 0x712,
0x71d, 0x72c, 0x710, 0x20, 0x721, 0x728, 0x725, 0x71d, 0x72c, 0x710, 0x20, 0x715,
0x710, 0x718, 0x723, 0x72c, 0x72a, 0x720, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x710, 0x718, 0x723, 0x72c, 0x72a, 0x720, 0x71d, 0x710, 0x20, 0x721,
0x715, 0x722, 0x71a, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721,
0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x718,
0x723, 0x72c, 0x72a, 0x720, 0x71d, 0x710, 0x20, 0x721, 0x715, 0x722, 0x71a, 0x71d,
0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20,
0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x718, 0x723, 0x72c, 0x72a, 0x720,
0x71d, 0x710, 0x20, 0x721, 0x715, 0x722, 0x71a, 0x71d, 0x72c, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x710, 0x718, 0x723, 0x72c, 0x72a, 0x720, 0x71d, 0x710,
0x20, 0x721, 0x725, 0x72a, 0x712, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710,
0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715,
0x710, 0x718, 0x723, 0x72c, 0x72a, 0x720, 0x71d, 0x710, 0x20, 0x721, 0x725, 0x72a,
0x712, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x718, 0x723, 0x72c,
0x72a, 0x720, 0x71d, 0x710, 0x20, 0x721, 0x725, 0x72a, 0x712, 0x71d, 0x72c, 0x710,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x719, 0x72a, 0x712, 0x71d, 0x713,
0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c,
0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x719, 0x72a, 0x712, 0x71d, 0x713,
0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20,
0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x719, 0x72a, 0x712, 0x71d, 0x713,
0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x719, 0x718, 0x72a,
0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d,
0x72c, 0x710, 0x20, 0x715, 0x710, 0x719, 0x718, 0x72a, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715,
0x710, 0x719, 0x718, 0x72a, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x722,
0x713, 0x720, 0x710, 0x715, 0x71d, 0x72b, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721,
0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x712, 0x722,
0x713, 0x720, 0x710, 0x715, 0x71d, 0x72b, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x712, 0x722,
0x713, 0x720, 0x710, 0x715, 0x71d, 0x72b, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x718, 0x72c, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712,
0x718, 0x720, 0x71d, 0x712, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x72a, 0x710, 0x723, 0x71d, 0x720, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710,
0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715,
0x712, 0x72a, 0x710, 0x723, 0x71d, 0x720, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715,
0x712, 0x72a, 0x710, 0x723, 0x71d, 0x720, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x712, 0x72a, 0x718, 0x722, 0x710, 0x71d, 0x20, 0x715, 0x710, 0x72a,
0x718, 0x723, 0x710, 0x720, 0x721, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x71f,
0x710, 0x726, 0x20, 0x712, 0x71d, 0x72a, 0x715, 0x71d, 0x725, 0x715, 0x722, 0x710,
0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715,
0x71f, 0x710, 0x726, 0x20, 0x712, 0x71d, 0x72a, 0x715, 0x71d, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20,
0x715, 0x71f, 0x710, 0x726, 0x20, 0x712, 0x71d, 0x72a, 0x715, 0x71d, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x71f, 0x710, 0x71d, 0x723, 0x71d, 0x725, 0x715, 0x722,
0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20,
0x715, 0x72c, 0x72b, 0x710, 0x721, 0x718, 0x72a, 0x718, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x72c, 0x72b, 0x710, 0x72c, 0x721, 0x725, 0x715, 0x722, 0x710, 0x20,
0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x72c,
0x72b, 0x710, 0x72c, 0x721, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x72c, 0x72b, 0x710, 0x72c,
0x721, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x72c, 0x72b, 0x71d, 0x720, 0x71d,
0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d,
0x72c, 0x710, 0x20, 0x715, 0x72c, 0x72b, 0x71d, 0x720, 0x71d, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20,
0x715, 0x72c, 0x72b, 0x71d, 0x720, 0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x728, 0x71d, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a,
0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x728, 0x71d, 0x722, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710,
0x20, 0x715, 0x728, 0x71d, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x713,
0x719, 0x72a, 0x72c, 0x710, 0x20, 0x715, 0x71f, 0x72a, 0x71d, 0x723, 0x72c, 0x721,
0x723, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x713, 0x719, 0x71d, 0x716, 0x308,
0x710, 0x20, 0x715, 0x71f, 0x718, 0x71f, 0x718, 0x723, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x71f, 0x718, 0x720, 0x718, 0x721, 0x712, 0x71d, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710,
0x20, 0x715, 0x71f, 0x718, 0x720, 0x718, 0x721, 0x712, 0x71d, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710,
0x20, 0x715, 0x71f, 0x718, 0x720, 0x718, 0x721, 0x712, 0x71d, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x713, 0x719, 0x71d, 0x72a, 0x308, 0x710, 0x20, 0x715, 0x71f,
0x718, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c,
0x722, 0x71d, 0x72c, 0x710, 0x20, 0x713, 0x719, 0x71d, 0x72a, 0x308, 0x710, 0x20,
0x715, 0x71f, 0x718, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x713, 0x719, 0x71d, 0x72a, 0x308,
0x710, 0x20, 0x715, 0x71f, 0x718, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x71f, 0x718, 0x712, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718,
0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x71f, 0x718, 0x712, 0x710,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718,
0x721, 0x710, 0x20, 0x715, 0x71f, 0x718, 0x712, 0x710, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x715, 0x712, 0x71d, 0x723, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x715, 0x718, 0x721, 0x718, 0x722, 0x72c, 0x20, 0x715, 0x710, 0x718, 0x72a, 0x712,
0x71d, 0x720, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x721, 0x715, 0x722, 0x71a,
0x20, 0x72c, 0x71d, 0x721, 0x718, 0x72a, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x713, 0x719, 0x72a, 0x72c, 0x710, 0x20, 0x726, 0x728, 0x71a, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710,
0x20, 0x715, 0x713, 0x719, 0x72a, 0x72c, 0x710, 0x20, 0x726, 0x728, 0x71a, 0x710,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718,
0x721, 0x710, 0x20, 0x715, 0x713, 0x719, 0x72a, 0x72c, 0x710, 0x20, 0x726, 0x728,
0x71a, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x729, 0x718, 0x710,
0x715, 0x718, 0x72a, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x718, 0x72a,
0x718, 0x726, 0x710, 0x20, 0x721, 0x728, 0x725, 0x71d, 0x72c, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710,
0x20, 0x715, 0x710, 0x718, 0x72a, 0x718, 0x726, 0x710, 0x20, 0x721, 0x728, 0x725,
0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a,
0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x718, 0x72a, 0x718, 0x726,
0x710, 0x20, 0x721, 0x728, 0x725, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x710, 0x718, 0x72a, 0x718, 0x726, 0x710, 0x20, 0x721, 0x715, 0x722,
0x71a, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718,
0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x718, 0x72a, 0x718,
0x726, 0x710, 0x20, 0x721, 0x715, 0x722, 0x71a, 0x71d, 0x72c, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710,
0x20, 0x715, 0x710, 0x718, 0x72a, 0x718, 0x726, 0x710, 0x20, 0x721, 0x715, 0x722,
0x71a, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x718,
0x72a, 0x718, 0x726, 0x710, 0x20, 0x28, 0x717, 0x721, 0x20, 0x721, 0x715, 0x722,
0x71a, 0x710, 0x29, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x718, 0x72a,
0x718, 0x726, 0x710, 0x20, 0x721, 0x725, 0x72a, 0x712, 0x71d, 0x72c, 0x710, 0x725,
0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c,
0x710, 0x20, 0x715, 0x710, 0x718, 0x72a, 0x718, 0x726, 0x710, 0x20, 0x721, 0x725,
0x72a, 0x712, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712,
0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x718, 0x72a,
0x718, 0x726, 0x710, 0x20, 0x721, 0x725, 0x72a, 0x712, 0x71d, 0x72c, 0x710, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x713, 0x719, 0x71d, 0x72a, 0x308, 0x710, 0x20,
0x715, 0x726, 0x720, 0x71f, 0x720, 0x722, 0x715, 0x725, 0x715, 0x722, 0x710, 0x20,
0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x713,
0x719, 0x71d, 0x72a, 0x308, 0x710, 0x20, 0x715, 0x726, 0x720, 0x71f, 0x720, 0x722,
0x715, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d,
0x718, 0x721, 0x710, 0x20, 0x715, 0x713, 0x719, 0x71d, 0x72a, 0x308, 0x710, 0x20,
0x715, 0x726, 0x720, 0x71f, 0x720, 0x722, 0x715, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x726, 0x71d, 0x713, 0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b,
0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x726, 0x71d, 0x713,
0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d,
0x718, 0x721, 0x710, 0x20, 0x715, 0x726, 0x71d, 0x713, 0x71d, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x713, 0x718, 0x71d, 0x710, 0x722, 0x710, 0x20, 0x726, 0x72a,
0x722, 0x723, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x726,
0x72a, 0x722, 0x723, 0x710, 0x20, 0x72c, 0x71d, 0x721, 0x722, 0x71d, 0x72c, 0x710,
0x20, 0x718, 0x710, 0x722, 0x72c, 0x72a, 0x72c, 0x71d, 0x71f, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x713, 0x710, 0x720, 0x726, 0x710, 0x713, 0x718, 0x723,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x713, 0x721, 0x712, 0x71d, 0x72a, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x713, 0x718, 0x72a, 0x713, 0x71d, 0x710, 0x725,
0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c,
0x710, 0x20, 0x715, 0x713, 0x718, 0x72a, 0x713, 0x71d, 0x710, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20,
0x715, 0x713, 0x718, 0x72a, 0x713, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x713, 0x719, 0x71d, 0x72a, 0x308, 0x710, 0x20, 0x715, 0x713, 0x71d, 0x720,
0x712, 0x72a, 0x72c, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x713, 0x72a, 0x71d,
0x722, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x713, 0x72a, 0x71d, 0x722, 0x720,
0x722, 0x715, 0x20, 0x721, 0x715, 0x722, 0x71a, 0x71d, 0x72c, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710,
0x20, 0x713, 0x72a, 0x71d, 0x722, 0x720, 0x722, 0x715, 0x20, 0x721, 0x715, 0x722,
0x71a, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x713, 0x72a, 0x71d, 0x722, 0x720,
0x722, 0x715, 0x20, 0x721, 0x715, 0x722, 0x71a, 0x71d, 0x72c, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x713, 0x72a, 0x71d, 0x722, 0x720, 0x722, 0x715, 0x20,
0x715, 0x721, 0x725, 0x72a, 0x712, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710,
0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715,
0x713, 0x72a, 0x71d, 0x722, 0x720, 0x722, 0x715, 0x20, 0x715, 0x721, 0x725, 0x72a,
0x712, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x713, 0x72a, 0x71d, 0x722,
0x720, 0x722, 0x715, 0x20, 0x715, 0x721, 0x725, 0x72a, 0x712, 0x71d, 0x72c, 0x710,
0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d,
0x72c, 0x710, 0x20, 0x715, 0x713, 0x718, 0x710, 0x721, 0x725, 0x715, 0x722, 0x710,
0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715,
0x721, 0x725, 0x720, 0x722, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x717,
0x710, 0x718, 0x710, 0x71d, 0x71d, 0x20, 0x710, 0x720, 0x718, 0x72b, 0x71d, 0x722,
0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d,
0x72c, 0x710, 0x20, 0x715, 0x717, 0x710, 0x718, 0x710, 0x71d, 0x71d, 0x20, 0x710,
0x720, 0x718, 0x72b, 0x71d, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712,
0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x717, 0x710, 0x718,
0x710, 0x71d, 0x71d, 0x20, 0x710, 0x720, 0x718, 0x72b, 0x71d, 0x722, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x717, 0x718, 0x722, 0x713, 0x20, 0x71f, 0x718, 0x722,
0x713, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722,
0x71d, 0x72c, 0x710, 0x20, 0x715, 0x717, 0x718, 0x722, 0x713, 0x20, 0x71f, 0x718,
0x722, 0x713, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20,
0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x717, 0x718, 0x722, 0x713, 0x20, 0x71f,
0x718, 0x722, 0x713, 0x715, 0x717, 0x718, 0x712, 0x715, 0x725, 0x715, 0x722, 0x710,
0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715,
0x717, 0x718, 0x712, 0x715, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x717, 0x718, 0x712, 0x715,
0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d,
0x72c, 0x710, 0x20, 0x715, 0x717, 0x722, 0x715, 0x718, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x710, 0x718, 0x729, 0x71d, 0x722, 0x718, 0x723, 0x20, 0x717, 0x722,
0x715, 0x718, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x717, 0x722,
0x715, 0x718, 0x728, 0x71d, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x717,
0x722, 0x715, 0x718, 0x728, 0x71d, 0x722, 0x20, 0x721, 0x728, 0x725, 0x71d, 0x72c,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x717, 0x722, 0x715, 0x718, 0x728,
0x71d, 0x722, 0x20, 0x721, 0x715, 0x722, 0x71a, 0x71d, 0x72c, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x717, 0x722, 0x715, 0x718, 0x728, 0x71d, 0x722, 0x20,
0x721, 0x725, 0x72a, 0x712, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x710, 0x71d, 0x72a, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721,
0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x71d,
0x72a, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a,
0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x71d, 0x72a, 0x710, 0x722,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x71d, 0x72a, 0x71f, 0x718, 0x72c,
0x723, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c,
0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x71d, 0x72a, 0x71f, 0x718, 0x72c,
0x723, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20,
0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x71d, 0x72a, 0x71f, 0x718, 0x72c,
0x723, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x71d, 0x723, 0x72a,
0x710, 0x71d, 0x720, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a,
0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x71d, 0x723, 0x72a, 0x710,
0x71d, 0x720, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20,
0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x71d, 0x723, 0x72a, 0x710, 0x71d,
0x720, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x71d, 0x726, 0x722, 0x725, 0x715,
0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710,
0x20, 0x715, 0x71d, 0x726, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712,
0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x71d, 0x726, 0x722,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x726, 0x71d, 0x72c, 0x72a, 0x718, 0x726,
0x712, 0x720, 0x712, 0x723, 0x71f, 0x71d, 0x2d, 0x71f, 0x710, 0x721, 0x71f, 0x72c,
0x723, 0x71f, 0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a,
0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x726, 0x71d, 0x72c, 0x72a, 0x718,
0x726, 0x712, 0x720, 0x712, 0x723, 0x71f, 0x71d, 0x2d, 0x71f, 0x710, 0x721, 0x71f,
0x72c, 0x723, 0x71f, 0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x726, 0x71d, 0x72c, 0x72a,
0x718, 0x726, 0x712, 0x720, 0x712, 0x723, 0x71f, 0x71d, 0x2d, 0x71f, 0x710, 0x721,
0x71f, 0x72c, 0x723, 0x71f, 0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x721,
0x715, 0x722, 0x71a, 0x20, 0x71f, 0x719, 0x729, 0x723, 0x72c, 0x710, 0x722, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x721, 0x725, 0x72a, 0x712, 0x20, 0x71f, 0x719,
0x729, 0x723, 0x72c, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x71f,
0x718, 0x72a, 0x71d, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b,
0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x71f, 0x718, 0x72a,
0x71d, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a,
0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x71f, 0x718, 0x72a, 0x71d, 0x71d,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x71f, 0x718, 0x723, 0x72a, 0x710, 0x71d,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x71f, 0x72a, 0x710, 0x723, 0x722, 0x718,
0x71d, 0x72a, 0x723, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718,
0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x71f, 0x72a, 0x710, 0x723,
0x722, 0x718, 0x71d, 0x72a, 0x723, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x71f, 0x72a,
0x710, 0x723, 0x722, 0x718, 0x71d, 0x72a, 0x723, 0x71f, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x729, 0x71d, 0x72a, 0x713, 0x71d, 0x719, 0x723, 0x72c, 0x710, 0x722,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x720, 0x710, 0x722, 0x71f, 0x710, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x713, 0x719, 0x71d, 0x72a, 0x308, 0x710, 0x20,
0x715, 0x720, 0x710, 0x71d, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x720,
0x718, 0x72a, 0x715, 0x20, 0x717, 0x710, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20,
0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x720,
0x718, 0x72a, 0x715, 0x20, 0x717, 0x710, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x720,
0x718, 0x72a, 0x715, 0x20, 0x717, 0x710, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x721, 0x710, 0x71f, 0x710, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721,
0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x721, 0x710,
0x71f, 0x710, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a,
0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x721, 0x710, 0x71f, 0x710, 0x718,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x721, 0x710, 0x713, 0x710, 0x715, 0x710,
0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722,
0x71d, 0x72c, 0x710, 0x20, 0x715, 0x721, 0x710, 0x713, 0x715, 0x710, 0x722, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721,
0x710, 0x20, 0x715, 0x721, 0x710, 0x713, 0x715, 0x710, 0x722, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x721, 0x720, 0x71d, 0x719, 0x71d, 0x710, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x713, 0x719, 0x72a, 0x72c, 0x710, 0x20, 0x721, 0x710, 0x720,
0x715, 0x71d, 0x712, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x721, 0x710, 0x72a, 0x71f, 0x710, 0x718, 0x723, 0x710, 0x723, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x713, 0x719, 0x71d, 0x72a, 0x308, 0x710, 0x20, 0x715, 0x721,
0x710, 0x72a, 0x72b, 0x710, 0x720, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x721,
0x718, 0x72a, 0x71d, 0x71b, 0x71d, 0x718, 0x723, 0x725, 0x715, 0x722, 0x710, 0x20,
0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x721,
0x718, 0x72a, 0x71d, 0x71b, 0x71d, 0x718, 0x723, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x721,
0x718, 0x72a, 0x71d, 0x71b, 0x71d, 0x718, 0x723, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x721, 0x710, 0x718, 0x723, 0x718, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x721, 0x71f, 0x723, 0x71d, 0x729, 0x718, 0x20, 0x72b, 0x71d, 0x722, 0x71d,
0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c,
0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x721, 0x71f, 0x723, 0x71d, 0x729, 0x718,
0x20, 0x72b, 0x71d, 0x722, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x721,
0x71f, 0x723, 0x71d, 0x729, 0x718, 0x20, 0x72b, 0x71d, 0x722, 0x71d, 0x72c, 0x710,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x718, 0x720, 0x710, 0x722, 0x712,
0x710, 0x72c, 0x718, 0x72a, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718,
0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x718, 0x720, 0x710,
0x722, 0x712, 0x710, 0x72c, 0x718, 0x72a, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x718,
0x720, 0x710, 0x722, 0x712, 0x710, 0x72c, 0x718, 0x72a, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x721, 0x718, 0x723, 0x71f, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20,
0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x721,
0x718, 0x723, 0x71f, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x721, 0x718, 0x723, 0x71f,
0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x721, 0x71d, 0x710, 0x722, 0x721,
0x710, 0x72a, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x722, 0x710, 0x718, 0x72a,
0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x722, 0x71d, 0x726, 0x710, 0x720,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x722, 0x71d, 0x718, 0x20, 0x71f, 0x720,
0x71d, 0x715, 0x718, 0x722, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721,
0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x722, 0x71d,
0x718, 0x20, 0x71f, 0x720, 0x71d, 0x715, 0x718, 0x722, 0x71d, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710,
0x20, 0x715, 0x722, 0x71d, 0x718, 0x20, 0x71f, 0x720, 0x71d, 0x715, 0x718, 0x722,
0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x722, 0x71d, 0x718, 0x20,
0x719, 0x71d, 0x720, 0x722, 0x715, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b,
0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x722, 0x71d, 0x718,
0x20, 0x719, 0x71d, 0x720, 0x722, 0x715, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x722, 0x71d,
0x718, 0x20, 0x719, 0x71d, 0x720, 0x722, 0x715, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x722, 0x71d, 0x718, 0x726, 0x710, 0x718, 0x722, 0x720, 0x722, 0x715, 0x725,
0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c,
0x710, 0x20, 0x715, 0x722, 0x71d, 0x718, 0x726, 0x710, 0x718, 0x722, 0x720, 0x722,
0x715, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d,
0x718, 0x721, 0x710, 0x20, 0x715, 0x722, 0x71d, 0x718, 0x726, 0x710, 0x718, 0x722,
0x720, 0x722, 0x715, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x722, 0x71d, 0x718,
0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x713, 0x719, 0x72a, 0x72c, 0x710,
0x20, 0x715, 0x722, 0x718, 0x72a, 0x726, 0x720, 0x71f, 0x725, 0x715, 0x722, 0x710,
0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715,
0x713, 0x719, 0x72a, 0x72c, 0x710, 0x20, 0x715, 0x722, 0x718, 0x72a, 0x726, 0x720,
0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d,
0x718, 0x721, 0x710, 0x20, 0x715, 0x713, 0x719, 0x72a, 0x72c, 0x710, 0x20, 0x715,
0x722, 0x718, 0x72a, 0x726, 0x720, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x726, 0x72a, 0x722, 0x722, 0x715, 0x718, 0x20, 0x715, 0x722, 0x718, 0x72a, 0x718,
0x722, 0x717, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a,
0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x726, 0x72a, 0x722, 0x722, 0x715,
0x718, 0x20, 0x715, 0x722, 0x718, 0x72a, 0x718, 0x722, 0x717, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710,
0x20, 0x715, 0x726, 0x72a, 0x722, 0x722, 0x715, 0x718, 0x20, 0x715, 0x722, 0x718,
0x72a, 0x718, 0x722, 0x717, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x713,
0x719, 0x71d, 0x716, 0x308, 0x710, 0x20, 0x715, 0x721, 0x72a, 0x71d, 0x710, 0x722,
0x710, 0x20, 0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x722, 0x718, 0x712, 0x718, 0x723, 0x71d, 0x712, 0x72a, 0x723,
0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722,
0x71d, 0x72c, 0x710, 0x20, 0x715, 0x722, 0x718, 0x712, 0x718, 0x723, 0x71d, 0x712,
0x72a, 0x723, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a,
0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x722, 0x718, 0x712, 0x718, 0x723,
0x71d, 0x712, 0x72a, 0x723, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x718,
0x721, 0x723, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a,
0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x718, 0x721, 0x723, 0x71f, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721,
0x710, 0x20, 0x715, 0x718, 0x721, 0x723, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x726, 0x710, 0x71f, 0x723, 0x72c, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710,
0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715,
0x726, 0x710, 0x71f, 0x723, 0x72c, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x726,
0x710, 0x71f, 0x723, 0x72c, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x726, 0x720, 0x710, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x726, 0x710,
0x726, 0x718, 0x710, 0x20, 0x713, 0x71d, 0x722, 0x71d, 0x710, 0x20, 0x71a, 0x715,
0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x726, 0x72a, 0x713, 0x718,
0x710, 0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c,
0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x726, 0x72a, 0x713, 0x718, 0x710, 0x71d,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718,
0x721, 0x710, 0x20, 0x715, 0x726, 0x72a, 0x713, 0x718, 0x710, 0x71d, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x726, 0x71d, 0x72a, 0x718, 0x725, 0x715, 0x722, 0x710,
0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715,
0x726, 0x71d, 0x72a, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x726, 0x71d, 0x72a, 0x718,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x726, 0x71d, 0x720, 0x71d, 0x726, 0x71d,
0x722, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a,
0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x726, 0x71d, 0x720, 0x71d, 0x726,
0x71d, 0x722, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x726, 0x71d, 0x720, 0x71d,
0x726, 0x71d, 0x722, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x713,
0x719, 0x71d, 0x72a, 0x308, 0x710, 0x20, 0x715, 0x726, 0x71d, 0x722, 0x71d, 0x71f,
0x723, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x723, 0x710, 0x722, 0x72c, 0x20,
0x726, 0x71d, 0x725, 0x72a, 0x20, 0x718, 0x721, 0x729, 0x718, 0x720, 0x718, 0x722,
0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d,
0x72c, 0x710, 0x20, 0x715, 0x723, 0x710, 0x722, 0x72c, 0x20, 0x726, 0x71d, 0x725,
0x72a, 0x20, 0x718, 0x721, 0x729, 0x718, 0x720, 0x718, 0x722, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20,
0x715, 0x723, 0x710, 0x722, 0x72c, 0x20, 0x726, 0x71d, 0x725, 0x72a, 0x20, 0x718,
0x721, 0x729, 0x718, 0x720, 0x718, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x726, 0x71d, 0x72c, 0x71f, 0x710, 0x71d, 0x72a, 0x722, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x726, 0x718, 0x722, 0x710, 0x726, 0x71d, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x726, 0x71d, 0x718, 0x722, 0x713, 0x71d, 0x722, 0x713, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x71f, 0x71d, 0x719, 0x71d, 0x720, 0x718, 0x72a, 0x715,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722,
0x71d, 0x72c, 0x710, 0x20, 0x715, 0x71f, 0x71d, 0x719, 0x71d, 0x720, 0x718, 0x72a,
0x715, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20,
0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x71f, 0x71d, 0x719, 0x71d, 0x720, 0x718,
0x72a, 0x715, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x72a, 0x71d, 0x718,
0x722, 0x71d, 0x718, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x72a, 0x718,
0x72c, 0x71d, 0x72a, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x723, 0x710,
0x71a, 0x710, 0x720, 0x71d, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b,
0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x723, 0x710, 0x71a,
0x710, 0x720, 0x71d, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x723, 0x710, 0x71a, 0x710,
0x720, 0x71d, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x723, 0x721, 0x710,
0x72a, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c,
0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x723, 0x721, 0x710, 0x72a, 0x710, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721,
0x710, 0x20, 0x715, 0x723, 0x721, 0x710, 0x72a, 0x710, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x723, 0x721, 0x718, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721,
0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x723, 0x721,
0x718, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20,
0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x723, 0x721, 0x718, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x723, 0x710, 0x71d, 0x72b, 0x71d, 0x720, 0x725, 0x715,
0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710,
0x20, 0x715, 0x723, 0x71d, 0x722, 0x713, 0x710, 0x726, 0x718, 0x72a, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x713, 0x719, 0x71d, 0x72a, 0x308, 0x710, 0x20, 0x715,
0x72b, 0x720, 0x71d, 0x721, 0x718, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x713, 0x71d, 0x718, 0x72a, 0x713, 0x71d, 0x710, 0x20, 0x72c, 0x71d, 0x721, 0x722,
0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x723, 0x718, 0x72a,
0x71d, 0x722, 0x710, 0x721, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x723, 0x71d,
0x718, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x72c, 0x717, 0x71d, 0x72c,
0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x72c, 0x710, 0x71d, 0x726, 0x710,
0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722,
0x71d, 0x72c, 0x710, 0x20, 0x715, 0x72c, 0x710, 0x71d, 0x726, 0x710, 0x71d, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721,
0x710, 0x20, 0x715, 0x72c, 0x710, 0x71d, 0x726, 0x710, 0x71d, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x72c, 0x710, 0x713, 0x71d, 0x71f, 0x723, 0x72c, 0x710, 0x722,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x72c, 0x718, 0x71f, 0x71d, 0x720, 0x710,
0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x72c, 0x718, 0x722, 0x713, 0x710, 0x725,
0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c,
0x710, 0x20, 0x72c, 0x718, 0x722, 0x713, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x72c, 0x718,
0x722, 0x713, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x72c, 0x72b, 0x718,
0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x72c, 0x718, 0x72a, 0x71f, 0x721,
0x722, 0x723, 0x72c, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b,
0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x72c, 0x718, 0x72a,
0x71f, 0x721, 0x722, 0x723, 0x72c, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x72c,
0x718, 0x72a, 0x71f, 0x721, 0x722, 0x723, 0x72c, 0x710, 0x722, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x72c, 0x718, 0x712, 0x710, 0x720, 0x718, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x710, 0x718, 0x72a, 0x718, 0x713, 0x718, 0x710, 0x71d, 0x725,
0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c,
0x710, 0x20, 0x715, 0x710, 0x718, 0x72a, 0x718, 0x713, 0x718, 0x710, 0x71d, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721,
0x710, 0x20, 0x715, 0x710, 0x718, 0x72a, 0x718, 0x713, 0x718, 0x710, 0x71d, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x718, 0x719, 0x712, 0x71f, 0x723, 0x72c,
0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c,
0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x718, 0x719, 0x712, 0x71f, 0x723,
0x72c, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a,
0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x718, 0x719, 0x712, 0x71f,
0x723, 0x72c, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x710,
0x722, 0x718, 0x710, 0x71b, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b,
0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x712, 0x710, 0x722,
0x718, 0x710, 0x71b, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x712, 0x710, 0x722, 0x718,
0x710, 0x71b, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x722, 0x719,
0x718, 0x71d, 0x720, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x720,
0x710, 0x715, 0x71d, 0x712, 0x718, 0x723, 0x72c, 0x718, 0x71f, 0x725, 0x715, 0x722,
0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20,
0x715, 0x712, 0x720, 0x710, 0x715, 0x71d, 0x712, 0x718, 0x723, 0x72c, 0x718, 0x71f,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718,
0x721, 0x710, 0x20, 0x715, 0x712, 0x720, 0x710, 0x715, 0x71d, 0x712, 0x718, 0x723,
0x72c, 0x718, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x718, 0x720,
0x713, 0x718, 0x713, 0x72a, 0x710, 0x715, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721,
0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x712, 0x718,
0x720, 0x713, 0x718, 0x713, 0x72a, 0x710, 0x715, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x712,
0x718, 0x720, 0x713, 0x718, 0x713, 0x72a, 0x710, 0x715, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x712, 0x718, 0x723, 0x72c, 0x718, 0x71f, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x713, 0x719, 0x71d, 0x72a, 0x308, 0x710, 0x20, 0x715, 0x718, 0x710,
0x71d, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x718, 0x71d, 0x720, 0x71d,
0x723, 0x20, 0x718, 0x726, 0x718, 0x72c, 0x718, 0x722, 0x710, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x71d, 0x710, 0x71f, 0x718, 0x72c, 0x723, 0x71f, 0x725, 0x715,
0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710,
0x20, 0x715, 0x71d, 0x710, 0x71f, 0x718, 0x72c, 0x723, 0x71f, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20,
0x715, 0x71d, 0x710, 0x71f, 0x718, 0x72c, 0x723, 0x71f, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x71d, 0x71f, 0x710, 0x72c, 0x71d, 0x72a, 0x722, 0x712, 0x71d, 0x72a,
0x713, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722,
0x71d, 0x72c, 0x710, 0x20, 0x715, 0x71d, 0x71f, 0x710, 0x72c, 0x71d, 0x72a, 0x722,
0x712, 0x71d, 0x72a, 0x713, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x71d, 0x71f, 0x710, 0x72c,
0x71d, 0x72a, 0x722, 0x712, 0x71d, 0x72a, 0x713, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x71d, 0x718, 0x729, 0x718, 0x722, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x410, 0x444, 0x493, 0x43e, 0x43d, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x410, 0x444, 0x440, 0x438, 0x49b, 0x43e, 0x438, 0x20,
0x41c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x410, 0x444, 0x440, 0x438, 0x49b, 0x43e, 0x438, 0x20, 0x428, 0x430, 0x440,
0x49b, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x410, 0x444, 0x440, 0x438, 0x49b,
0x43e, 0x438, 0x20, 0x4b6, 0x430, 0x43d, 0x443, 0x431, 0x4e3, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x410, 0x444, 0x440, 0x438, 0x49b, 0x43e, 0x438, 0x20, 0x492,
0x430, 0x440, 0x431, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x410, 0x444, 0x440,
0x438, 0x49b, 0x43e, 0x438, 0x20, 0x492, 0x430, 0x440, 0x431, 0x4e3, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d,
0x430, 0x438, 0x20, 0x410, 0x444, 0x440, 0x438, 0x49b, 0x43e, 0x438, 0x20, 0x492,
0x430, 0x440, 0x431, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x410, 0x43b,
0x44f, 0x441, 0x43a, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x410, 0x43b, 0x44f,
0x441, 0x43a, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437,
0x43e, 0x43d, 0x430, 0x438, 0x20, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x438, 0x438, 0x20, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d,
0x430, 0x438, 0x20, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x43c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x4e3, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x438, 0x438, 0x20, 0x43c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x4e3, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20,
0x43c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x448, 0x430, 0x440, 0x49b, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x448,
0x430, 0x440, 0x49b, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef,
0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x448, 0x430, 0x440, 0x49b, 0x4e3, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x43a, 0x4ef, 0x4b3, 0x4e3, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438,
0x438, 0x20, 0x43a, 0x4ef, 0x4b3, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x43a, 0x4ef, 0x4b3, 0x4e3,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x423, 0x49b, 0x451, 0x43d, 0x443, 0x441,
0x438, 0x20, 0x41e, 0x440, 0x43e, 0x43c, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x423,
0x49b, 0x451, 0x43d, 0x443, 0x441, 0x438, 0x20, 0x41e, 0x440, 0x43e, 0x43c, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438,
0x20, 0x423, 0x49b, 0x451, 0x43d, 0x443, 0x441, 0x438, 0x20, 0x41e, 0x440, 0x43e,
0x43c, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x410, 0x43f, 0x438, 0x430, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x438, 0x438, 0x20, 0x410, 0x43f, 0x438, 0x430, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x410, 0x43f,
0x438, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x410, 0x440, 0x430, 0x431,
0x438, 0x441, 0x442, 0x43e, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x410, 0x440,
0x430, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x410, 0x440, 0x430,
0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438,
0x438, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e,
0x43d, 0x430, 0x438, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d,
0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d,
0x442, 0x438, 0x43d, 0x430, 0x438, 0x20, 0x492, 0x430, 0x440, 0x431, 0x4e3, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x438, 0x438, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d,
0x430, 0x438, 0x20, 0x492, 0x430, 0x440, 0x431, 0x4e3, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438,
0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x438, 0x20,
0x492, 0x430, 0x440, 0x431, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x410,
0x440, 0x43c, 0x430, 0x43d, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438,
0x438, 0x20, 0x410, 0x440, 0x43c, 0x430, 0x43d, 0x438, 0x441, 0x442, 0x43e, 0x43d,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430,
0x438, 0x20, 0x410, 0x440, 0x43c, 0x430, 0x43d, 0x438, 0x441, 0x442, 0x43e, 0x43d,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x430, 0x442, 0x43b, 0x430, 0x43d, 0x442,
0x438, 0x43a, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x430, 0x442, 0x43b, 0x430,
0x43d, 0x442, 0x438, 0x43a, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x440,
0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x430, 0x442, 0x43b, 0x430, 0x43d,
0x442, 0x438, 0x43a, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x410, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x438, 0x20, 0x41c, 0x430, 0x440,
0x43a, 0x430, 0x437, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x438, 0x20, 0x41c, 0x430, 0x440, 0x43a,
0x430, 0x437, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef, 0x43e,
0x43d, 0x430, 0x438, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x44f, 0x438, 0x20, 0x41c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x4e3, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x492, 0x430, 0x440, 0x431, 0x438, 0x438, 0x20, 0x41c,
0x430, 0x440, 0x43a, 0x430, 0x437, 0x438, 0x438, 0x20, 0x410, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x44f, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x492, 0x430,
0x440, 0x431, 0x438, 0x438, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x44f, 0x20, 0x41c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x4e3, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20,
0x492, 0x430, 0x440, 0x431, 0x438, 0x438, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x44f, 0x20, 0x41c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x4e3,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x44f, 0x438, 0x20, 0x428, 0x430, 0x440, 0x49b, 0x4e3, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x438, 0x438, 0x20, 0x448, 0x430, 0x440, 0x49b, 0x438, 0x438, 0x20, 0x410, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x448, 0x430, 0x440,
0x49b, 0x438, 0x438, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x44f, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x44f, 0x438, 0x20, 0x492, 0x430, 0x440, 0x431, 0x4e3, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x438, 0x438, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x44f, 0x438, 0x20, 0x492, 0x430, 0x440, 0x431, 0x4e3, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x410, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x438, 0x20, 0x492, 0x430, 0x440,
0x431, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x41e, 0x437, 0x430, 0x440,
0x431, 0x43e, 0x439, 0x4b7, 0x43e, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x41e,
0x437, 0x430, 0x440, 0x431, 0x43e, 0x439, 0x4b7, 0x43e, 0x43d, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430,
0x438, 0x20, 0x41e, 0x437, 0x430, 0x440, 0x431, 0x43e, 0x439, 0x4b7, 0x43e, 0x43d,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x410, 0x437, 0x43e, 0x440, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x438, 0x438, 0x20, 0x410, 0x437, 0x43e, 0x440, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20,
0x410, 0x437, 0x43e, 0x440, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x411, 0x430,
0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20,
0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430,
0x438, 0x20, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x411, 0x443, 0x442, 0x430, 0x43d, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x411, 0x43e, 0x43b, 0x438, 0x432, 0x438, 0x44f, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438,
0x44f, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b,
0x438, 0x44f, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438,
0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x411, 0x440, 0x430, 0x437, 0x438,
0x43b, 0x438, 0x44f, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x411, 0x440, 0x443,
0x43d, 0x435, 0x439, 0x20, 0x414, 0x43e, 0x440, 0x443, 0x441, 0x441, 0x430, 0x43b,
0x43e, 0x43c, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x41a, 0x430, 0x431, 0x43e,
0x20, 0x412, 0x435, 0x440, 0x434, 0x435, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x41a,
0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430,
0x438, 0x20, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x438, 0x438, 0x20, 0x427, 0x430, 0x43c, 0x43e, 0x440, 0x440, 0x43e,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x427, 0x430, 0x442, 0x430, 0x43c, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x438, 0x438, 0x20, 0x427, 0x430, 0x442, 0x430, 0x43c, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x427,
0x430, 0x442, 0x430, 0x43c, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x427, 0x438,
0x43b, 0x438, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x427, 0x438, 0x43b, 0x438, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e,
0x43d, 0x430, 0x438, 0x20, 0x427, 0x438, 0x43b, 0x438, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x427, 0x438, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x427, 0x438,
0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d,
0x430, 0x438, 0x20, 0x427, 0x438, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x4b7, 0x430, 0x437, 0x438, 0x440, 0x430, 0x438, 0x20, 0x41c, 0x430, 0x432, 0x43b,
0x443, 0x434, 0x438, 0x20, 0x418, 0x441, 0x43e, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x4b6, 0x430, 0x437, 0x438, 0x440, 0x430, 0x4b3, 0x43e, 0x438, 0x20, 0x41a,
0x43e, 0x43a, 0x43e, 0x441, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x41a, 0x43e,
0x43b, 0x443, 0x43c, 0x431, 0x438, 0x44f, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x41a,
0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x44f, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20,
0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x44f, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x4b7, 0x430, 0x437, 0x438, 0x440, 0x430, 0x4b3, 0x43e, 0x438, 0x20,
0x41a, 0x443, 0x43a, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x4b6, 0x430, 0x437, 0x438,
0x440, 0x430, 0x4b3, 0x43e, 0x438, 0x20, 0x41a, 0x443, 0x43a, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x43d, 0x438, 0x43c, 0x430, 0x438, 0x20, 0x442, 0x43e, 0x431,
0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x4b6, 0x430, 0x437, 0x438,
0x440, 0x430, 0x4b3, 0x43e, 0x438, 0x20, 0x41a, 0x443, 0x43a, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x41a, 0x443, 0x431, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20,
0x41a, 0x443, 0x431, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef,
0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x41a, 0x443, 0x431, 0x430, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x414, 0x430, 0x432, 0x438, 0x441, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x414, 0x44e, 0x43c, 0x43e, 0x43d, 0x2d, 0x434, 0x2019, 0x423,
0x440, 0x432, 0x438, 0x43b, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x422, 0x438,
0x43c, 0x43e, 0x440, 0x438, 0x20, 0x428, 0x430, 0x440, 0x49b, 0x4e3, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x4b7, 0x430, 0x437, 0x438, 0x440, 0x430, 0x438, 0x20,
0x41f, 0x430, 0x441, 0x445, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x4b7, 0x430,
0x437, 0x438, 0x440, 0x430, 0x438, 0x20, 0x41f, 0x430, 0x441, 0x445, 0x430, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e,
0x43d, 0x430, 0x438, 0x20, 0x4b7, 0x430, 0x437, 0x438, 0x440, 0x430, 0x438, 0x20,
0x41f, 0x430, 0x441, 0x445, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x42d,
0x43a, 0x432, 0x430, 0x434, 0x43e, 0x440, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x410, 0x432, 0x440, 0x443, 0x43f, 0x43e, 0x438, 0x438, 0x20, 0x41c, 0x430, 0x440,
0x43a, 0x430, 0x437, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x430, 0x432, 0x440,
0x443, 0x43f, 0x43e, 0x438, 0x438, 0x20, 0x43c, 0x430, 0x440, 0x43a, 0x430, 0x437,
0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441,
0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x430, 0x432, 0x440, 0x443, 0x43f, 0x43e,
0x438, 0x438, 0x20, 0x43c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x4e3, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x430, 0x432, 0x440, 0x443, 0x43f, 0x43e, 0x438, 0x438,
0x20, 0x448, 0x430, 0x440, 0x49b, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x430,
0x432, 0x440, 0x443, 0x43f, 0x43e, 0x438, 0x438, 0x20, 0x448, 0x430, 0x440, 0x49b,
0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441,
0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x430, 0x432, 0x440, 0x443, 0x43f, 0x43e,
0x438, 0x438, 0x20, 0x448, 0x430, 0x440, 0x49b, 0x4e3, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x410, 0x432, 0x440, 0x443, 0x43f, 0x43e, 0x438, 0x20, 0x428, 0x430,
0x440, 0x49b, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x430, 0x432, 0x440,
0x443, 0x43f, 0x43e, 0x438, 0x438, 0x20, 0x493, 0x430, 0x440, 0x431, 0x4e3, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x438, 0x438, 0x20, 0x430, 0x432, 0x440, 0x443, 0x43f, 0x43e, 0x438, 0x438,
0x20, 0x493, 0x430, 0x440, 0x431, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x430,
0x432, 0x440, 0x443, 0x43f, 0x43e, 0x438, 0x438, 0x20, 0x493, 0x430, 0x440, 0x431,
0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x4b6, 0x430, 0x437, 0x438, 0x440,
0x430, 0x4b3, 0x43e, 0x438, 0x20, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d,
0x434, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x4b6, 0x430, 0x437, 0x438, 0x440, 0x430,
0x4b3, 0x43e, 0x438, 0x20, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442,
0x43e, 0x43d, 0x430, 0x438, 0x20, 0x4b6, 0x430, 0x437, 0x438, 0x440, 0x430, 0x4b3,
0x43e, 0x438, 0x20, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x424, 0x438, 0x4b7, 0x438, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438,
0x438, 0x20, 0x424, 0x438, 0x4b7, 0x438, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x424,
0x438, 0x4b7, 0x438, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x413, 0x432, 0x438,
0x430, 0x43d, 0x430, 0x438, 0x20, 0x424, 0x430, 0x440, 0x43e, 0x43d, 0x441, 0x430,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x4b7, 0x430, 0x43d, 0x443, 0x431, 0x4e3,
0x20, 0x432, 0x430, 0x20, 0x410, 0x43d, 0x442, 0x430, 0x440, 0x43a, 0x442, 0x438,
0x434, 0x430, 0x438, 0x20, 0x424, 0x430, 0x440, 0x43e, 0x43d, 0x441, 0x430, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433,
0x43e, 0x441, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x413, 0x430, 0x43c, 0x431,
0x438, 0x440, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x413, 0x443, 0x440, 0x4b7,
0x438, 0x441, 0x442, 0x43e, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x413, 0x443,
0x440, 0x4b7, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20,
0x413, 0x443, 0x440, 0x4b7, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x4b6, 0x430, 0x437, 0x438, 0x440, 0x430, 0x4b3, 0x43e, 0x438,
0x20, 0x413, 0x438, 0x43b, 0x431, 0x435, 0x440, 0x442, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x43c, 0x438, 0x451, 0x43d, 0x430, 0x438, 0x20, 0x413, 0x440, 0x438,
0x43d, 0x432, 0x438, 0x447, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x413, 0x440,
0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x438, 0x20, 0x428, 0x430,
0x440, 0x49b, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d,
0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x438, 0x20, 0x428, 0x430, 0x440, 0x49b,
0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441,
0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430,
0x43d, 0x434, 0x438, 0x44f, 0x438, 0x20, 0x428, 0x430, 0x440, 0x49b, 0x4e3, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d,
0x434, 0x438, 0x44f, 0x438, 0x20, 0x492, 0x430, 0x440, 0x431, 0x4e3, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x438, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438,
0x44f, 0x438, 0x20, 0x492, 0x430, 0x440, 0x431, 0x4e3, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438,
0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x438,
0x20, 0x492, 0x430, 0x440, 0x431, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x425,
0x430, 0x43b, 0x438, 0x4b7, 0x438, 0x20, 0x424, 0x43e, 0x440, 0x441, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x413, 0x430, 0x439, 0x430, 0x43d, 0x430, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x4b2, 0x430, 0x432, 0x430, 0x439, 0x4e3, 0x2d, 0x410,
0x43b, 0x435, 0x443, 0x442, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x4b2, 0x430,
0x432, 0x430, 0x439, 0x4e3, 0x2d, 0x410, 0x43b, 0x435, 0x443, 0x442, 0x4e3, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438,
0x20, 0x4b2, 0x430, 0x432, 0x430, 0x439, 0x4e3, 0x2d, 0x410, 0x43b, 0x435, 0x443,
0x442, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x4b2, 0x43e, 0x43d, 0x43a,
0x43e, 0x43d, 0x433, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x4b2, 0x43e, 0x43d, 0x43a,
0x43e, 0x43d, 0x433, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431,
0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x4b2, 0x43e, 0x43d, 0x43a,
0x43e, 0x43d, 0x433, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x425, 0x43e, 0x432,
0x434, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x4b2, 0x43e, 0x432, 0x434, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d,
0x430, 0x438, 0x20, 0x425, 0x43e, 0x432, 0x434, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20,
0x4b2, 0x438, 0x43d, 0x434, 0x443, 0x441, 0x442, 0x43e, 0x43d, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x443, 0x49b, 0x451, 0x43d, 0x443, 0x441, 0x438, 0x20, 0x4b2,
0x438, 0x43d, 0x434, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x4b2, 0x438, 0x43d,
0x434, 0x443, 0x447, 0x438, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x418,
0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x44f, 0x438, 0x20, 0x41c, 0x430,
0x440, 0x43a, 0x430, 0x437, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x448,
0x430, 0x440, 0x49b, 0x438, 0x438, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435,
0x437, 0x438, 0x44f, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x418, 0x43d, 0x434,
0x43e, 0x43d, 0x435, 0x437, 0x438, 0x44f, 0x438, 0x20, 0x492, 0x430, 0x440, 0x431,
0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x42d, 0x440, 0x43e, 0x43d, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x438, 0x438, 0x20, 0x42d, 0x440, 0x43e, 0x43d, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x42d, 0x440,
0x43e, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x418, 0x440, 0x43a, 0x443,
0x442, 0x441, 0x43a, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x418, 0x440, 0x43a, 0x443,
0x442, 0x441, 0x43a, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431,
0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x418, 0x440, 0x43a, 0x443,
0x442, 0x441, 0x43a, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x418, 0x441, 0x440,
0x43e, 0x438, 0x43b, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x418, 0x441, 0x440, 0x43e,
0x438, 0x43b, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e,
0x43d, 0x430, 0x438, 0x20, 0x418, 0x441, 0x440, 0x43e, 0x438, 0x43b, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x4b6, 0x43e, 0x43f, 0x43e, 0x43d, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438,
0x438, 0x20, 0x4b6, 0x43e, 0x43f, 0x43e, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x440, 0x430, 0x432, 0x448, 0x430, 0x43d, 0x438, 0x438, 0x20, 0x4b6, 0x43e,
0x43f, 0x43e, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x49a, 0x430, 0x437,
0x43e, 0x49b, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x49a, 0x430, 0x437, 0x43e, 0x49b, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x438,
0x20, 0x428, 0x430, 0x440, 0x49b, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x49a, 0x430, 0x437, 0x43e, 0x49b, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x438, 0x20,
0x492, 0x430, 0x440, 0x431, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x41a,
0x43e, 0x440, 0x435, 0x44f, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x41a, 0x43e, 0x440,
0x435, 0x44f, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e,
0x43d, 0x430, 0x438, 0x20, 0x41a, 0x43e, 0x440, 0x435, 0x44f, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x41a, 0x43e, 0x441, 0x440, 0x430, 0x435, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441,
0x43a, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e,
0x44f, 0x440, 0x441, 0x43a, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e,
0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x41a, 0x440, 0x430,
0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x49a, 0x438, 0x440, 0x493, 0x438, 0x437, 0x438, 0x441, 0x442, 0x43e, 0x43d,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x4b6, 0x430, 0x437, 0x438, 0x440, 0x430,
0x4b3, 0x43e, 0x438, 0x20, 0x41b, 0x438, 0x43d, 0x41b, 0x43e, 0x440, 0x434, 0x20,
0x425, 0x43e, 0x443, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438,
0x20, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x443, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x41b,
0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x443, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438,
0x20, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438,
0x20, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x41c, 0x430, 0x43b, 0x430, 0x439, 0x437, 0x438, 0x44f, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x41c, 0x430, 0x43b, 0x434, 0x438, 0x432, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x41c, 0x430, 0x440, 0x43a, 0x435, 0x441, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x4b6, 0x430, 0x437, 0x438, 0x440, 0x430, 0x4b3, 0x43e,
0x438, 0x20, 0x41c, 0x430, 0x440, 0x448, 0x430, 0x43b, 0x43b, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438, 0x439, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x438, 0x438, 0x20, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438, 0x439,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442,
0x43e, 0x43d, 0x430, 0x438, 0x20, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438,
0x439, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x41c, 0x430, 0x432, 0x441, 0x43e,
0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x423, 0x49b, 0x451, 0x43d, 0x443,
0x441, 0x438, 0x20, 0x41e, 0x440, 0x43e, 0x43c, 0x20, 0x41c, 0x435, 0x43a, 0x441,
0x438, 0x43a, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x423, 0x49b, 0x451, 0x43d,
0x443, 0x441, 0x438, 0x20, 0x41e, 0x440, 0x43e, 0x43c, 0x20, 0x41c, 0x435, 0x43a,
0x441, 0x438, 0x43a, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef,
0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x423, 0x49b, 0x451, 0x43d, 0x443, 0x441,
0x438, 0x20, 0x41e, 0x440, 0x43e, 0x43c, 0x20, 0x41c, 0x435, 0x43a, 0x441, 0x438,
0x43a, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x423, 0x43b, 0x430, 0x43d,
0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x423,
0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430,
0x438, 0x20, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x438, 0x438, 0x20, 0x41c, 0x430, 0x441, 0x43a, 0x430, 0x432, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e,
0x43d, 0x430, 0x438, 0x20, 0x41c, 0x430, 0x441, 0x43a, 0x430, 0x432, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x41c, 0x44f, 0x43d, 0x43c, 0x430, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x41d, 0x430, 0x443, 0x440, 0x443, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x41d, 0x435, 0x43f, 0x430, 0x43b, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x44f, 0x438, 0x20,
0x41d, 0x430, 0x432, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x41a, 0x430, 0x43b, 0x435,
0x434, 0x43e, 0x43d, 0x438, 0x44f, 0x438, 0x20, 0x41d, 0x430, 0x432, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d,
0x430, 0x438, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x44f,
0x438, 0x20, 0x41d, 0x430, 0x432, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x417,
0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x438, 0x20, 0x41d, 0x430, 0x432,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x438, 0x438, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438,
0x44f, 0x438, 0x20, 0x41d, 0x430, 0x432, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x417, 0x435, 0x43b, 0x430,
0x43d, 0x434, 0x438, 0x44f, 0x438, 0x20, 0x41d, 0x430, 0x432, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x41d, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435,
0x43d, 0x434, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x41d, 0x44e, 0x444, 0x430, 0x443,
0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x41d, 0x44e, 0x444, 0x430,
0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x41d, 0x438, 0x443, 0x44d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x4b7,
0x430, 0x437, 0x438, 0x440, 0x430, 0x438, 0x20, 0x41d, 0x43e, 0x440, 0x444, 0x43e,
0x43b, 0x43a, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x4b7, 0x430, 0x437, 0x438, 0x440,
0x430, 0x438, 0x20, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20,
0x4b7, 0x430, 0x437, 0x438, 0x440, 0x430, 0x438, 0x20, 0x41d, 0x43e, 0x440, 0x444,
0x43e, 0x43b, 0x43a, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x424, 0x435, 0x440,
0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440,
0x43e, 0x43d, 0x445, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x424, 0x435, 0x440,
0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440,
0x43e, 0x43d, 0x445, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e,
0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x424, 0x435, 0x440,
0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440,
0x43e, 0x43d, 0x445, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x41d, 0x43e,
0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438,
0x438, 0x20, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441,
0x43a, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441,
0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438,
0x431, 0x438, 0x440, 0x441, 0x43a, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x41e,
0x43c, 0x441, 0x43a, 0x41e, 0x43c, 0x441, 0x43a, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4e3, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e,
0x43d, 0x430, 0x438, 0x20, 0x41e, 0x43c, 0x441, 0x43a, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x41f, 0x43e, 0x43a, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x438, 0x438, 0x20, 0x41f, 0x43e, 0x43a, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e,
0x43d, 0x430, 0x438, 0x20, 0x41f, 0x43e, 0x43a, 0x438, 0x441, 0x442, 0x43e, 0x43d,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x41f, 0x430, 0x43b, 0x430, 0x443, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x41f, 0x430, 0x43f, 0x443, 0x430, 0x20, 0x413,
0x432, 0x438, 0x43d, 0x435, 0x44f, 0x438, 0x20, 0x41d, 0x430, 0x432, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x438, 0x438, 0x20, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430,
0x439, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441,
0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432,
0x430, 0x439, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x41f, 0x435, 0x440, 0x443,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x438, 0x438, 0x20, 0x41f, 0x435, 0x440, 0x443, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430,
0x438, 0x20, 0x41f, 0x435, 0x440, 0x443, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438,
0x20, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430,
0x438, 0x20, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x4b6, 0x430, 0x437, 0x438, 0x440, 0x430, 0x4b3, 0x43e,
0x438, 0x20, 0x424, 0x438, 0x43d, 0x438, 0x43a, 0x441, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41f, 0x438, 0x435, 0x440, 0x20,
0x432, 0x430, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x438, 0x438, 0x20, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41f, 0x438, 0x435, 0x440,
0x20, 0x432, 0x430, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438,
0x20, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41f, 0x438, 0x435, 0x440, 0x20, 0x432,
0x430, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x41f, 0x438, 0x442, 0x43a, 0x44d, 0x440, 0x43d, 0x50, 0x6f,
0x6e, 0x61, 0x70, 0x65, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x41f, 0x445, 0x435, 0x43d, 0x44f, 0x43d, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x420, 0x435, 0x44e, 0x43d, 0x438, 0x43e, 0x43d, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x420, 0x43e, 0x442, 0x435, 0x440, 0x430, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x438, 0x438, 0x20, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e,
0x43d, 0x430, 0x438, 0x20, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x438, 0x438, 0x20, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x421, 0x430,
0x43c, 0x43e, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x421, 0x435, 0x439,
0x448, 0x435, 0x43b, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x421, 0x438, 0x43d, 0x433,
0x430, 0x43f, 0x443, 0x440, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x4b6, 0x430,
0x437, 0x438, 0x440, 0x430, 0x4b3, 0x43e, 0x438, 0x20, 0x421, 0x43e, 0x43b, 0x43e,
0x43c, 0x43e, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x4b6, 0x43e, 0x440,
0x4b7, 0x438, 0x44f, 0x438, 0x20, 0x4b6, 0x430, 0x43d, 0x443, 0x431, 0x4e3, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x421, 0x443, 0x440, 0x438, 0x43d, 0x430, 0x43c,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x421, 0x451, 0x432, 0x430, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x422, 0x430, 0x438, 0x442, 0x438, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x422, 0x430, 0x439, 0x431, 0x44d, 0x439, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438,
0x438, 0x20, 0x422, 0x430, 0x439, 0x431, 0x44d, 0x439, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x422, 0x430,
0x439, 0x431, 0x44d, 0x439, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x422, 0x43e,
0x4b7, 0x438, 0x43a, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x422, 0x43e, 0x43a, 0x435, 0x43b, 0x430, 0x443, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438,
0x20, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x422,
0x43e, 0x43d, 0x433, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x427, 0x443,
0x43a, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x422, 0x443, 0x440, 0x43a, 0x43c,
0x430, 0x43d, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20,
0x422, 0x443, 0x440, 0x43a, 0x43c, 0x430, 0x43d, 0x438, 0x441, 0x442, 0x43e, 0x43d,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442,
0x43e, 0x43d, 0x430, 0x438, 0x20, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x430, 0x43d,
0x438, 0x441, 0x442, 0x43e, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x422,
0x443, 0x432, 0x430, 0x43b, 0x443, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x423,
0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x423,
0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x423,
0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x4ee, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x438, 0x438, 0x20, 0x4ee, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x43e,
0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441,
0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x4ee, 0x437, 0x431, 0x435, 0x43a, 0x438,
0x441, 0x442, 0x43e, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x412, 0x430,
0x43d, 0x443, 0x430, 0x442, 0x443, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x412, 0x430,
0x43d, 0x443, 0x430, 0x442, 0x443, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442,
0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x412, 0x430,
0x43d, 0x443, 0x430, 0x442, 0x443, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x412,
0x435, 0x43d, 0x435, 0x441, 0x443, 0x44d, 0x43b, 0x430, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e,
0x43a, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432,
0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442,
0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x412, 0x43b,
0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x438, 0x438, 0x20, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440,
0x430, 0x434, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438,
0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x412, 0x43e, 0x43b, 0x433, 0x43e,
0x433, 0x440, 0x430, 0x434, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x412, 0x43e,
0x441, 0x442, 0x43e, 0x43a, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x431, 0x435,
0x434, 0x43e, 0x440, 0x438, 0x438, 0x20, 0x4b7, 0x430, 0x437, 0x438, 0x440, 0x430,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x423, 0x43e, 0x43b, 0x43b, 0x438, 0x441,
0x20, 0x432, 0x430, 0x20, 0x424, 0x443, 0x442, 0x443, 0x43d, 0x430, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x42f, 0x43a,
0x443, 0x442, 0x441, 0x43a, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4e3, 0x42f, 0x43a, 0x443, 0x442,
0x441, 0x43a, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431,
0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x438, 0x438, 0x20, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438,
0x43d, 0x431, 0x443, 0x440, 0x433, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442,
0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x415, 0x43a,
0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x42e, 0x43a, 0x43e, 0x43d, 0xb85, 0xb95, 0xbcd, 0xbb0,
0xbc7, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xb95, 0xbcd, 0xbb0, 0xbc7,
0x20, 0xba4, 0xbb0, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xb95, 0xbcd,
0xbb0, 0xbc7, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb86, 0xb83, 0xbaa, 0xbcd, 0xb95, 0xbbe, 0xba9, 0xbbf, 0xbb8, 0xbcd, 0xba4,
0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xba4, 0xbcd,
0xba4, 0xbbf, 0xbaf, 0x20, 0xb86, 0xbaa, 0xbcd, 0xbaa, 0xbbf, 0xbb0, 0xbbf, 0xb95,
0xbcd, 0xb95, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb4, 0xb95,
0xbcd, 0xb95, 0xbc1, 0x20, 0xb86, 0xbaa, 0xbcd, 0xbaa, 0xbbf, 0xbb0, 0xbbf, 0xb95,
0xbcd, 0xb95, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba4, 0xbc6, 0xba9, 0xbcd,
0x20, 0xb86, 0xbaa, 0xbcd, 0xbaa, 0xbbf, 0xbb0, 0xbbf, 0xb95, 0xbcd, 0xb95, 0x20,
0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb86, 0xbaa, 0xbcd, 0xbaa,
0xbbf, 0xbb0, 0xbbf, 0xb95, 0xbcd, 0xb95, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb86, 0xbaa, 0xbcd, 0xbaa, 0xbbf,
0xbb0, 0xbbf, 0xb95, 0xbcd, 0xb95, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe,
0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95,
0xbc1, 0x20, 0xb86, 0xbaa, 0xbcd, 0xbaa, 0xbbf, 0xbb0, 0xbbf, 0xb95, 0xbcd, 0xb95,
0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85,
0xbb2, 0xbbe, 0xbb8, 0xbcd, 0xb95, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb85, 0xbb2, 0xbbe, 0xbb8, 0xbcd, 0xb95, 0xbbe, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8,
0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xbb2, 0xbbe,
0xbb8, 0xbcd, 0xb95, 0xbbe, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xbb2, 0xbcd, 0xbae, 0xbbe, 0xb9f, 0xbbf,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xbb2, 0xbcd, 0xbae, 0xbbe, 0xb9f,
0xbbf, 0x20, 0xba4, 0xbb0, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xbb2,
0xbcd, 0xbae, 0xbbe, 0xb9f, 0xbbf, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xbae, 0xbc7, 0xb9a, 0xbbe, 0xba9, 0xbcd, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xbae, 0xbc7, 0xb9a, 0xbbe, 0xba9, 0xbcd,
0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb85, 0xbae, 0xbc7, 0xb9a, 0xbbe, 0xba9, 0xbcd, 0x20, 0xb95, 0xbcb,
0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xba4, 0xbcd, 0xba4,
0xbbf, 0xbaf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xba4, 0xbcd, 0xba4,
0xbbf, 0xbaf, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xbaa,
0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95,
0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95, 0xba4, 0xbcd,
0xba4, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95, 0xba4,
0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbb5, 0xbc1, 0xba9, 0xbcd, 0xb9f, 0xbc8,
0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbb5, 0xbc1, 0xba9,
0xbcd, 0xb9f, 0xbc8, 0xba9, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe,
0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbb5, 0xbc1, 0xba9, 0xbcd,
0xb9f, 0xbc8, 0xba9, 0xbcd, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xb9a, 0xbbf, 0xbaa, 0xbbf, 0xb95, 0xbcd,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xb9a, 0xbbf, 0xbaa, 0xbbf, 0xb95,
0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xbaa, 0xb9a, 0xbbf, 0xbaa, 0xbbf, 0xb95, 0xbcd, 0x20, 0xbaa,
0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85,
0xba9, 0xb9f, 0xbc0, 0xbb0, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85,
0xba9, 0xbbe, 0xb9f, 0xbc8, 0xbaf, 0xbb0, 0xbcd, 0x20, 0xba4, 0xbb0, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xba9, 0xbbe, 0xb9f, 0xbc8, 0xbaf, 0xbb0, 0xbcd,
0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb8f,
0xbaa, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb8f, 0xbaa,
0xbbf, 0xbaf, 0xbbe, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb8f, 0xbaa, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xbaa,
0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85,
0xb9f, 0xbcd, 0xb9f, 0xbcc, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xb9f,
0xbcd, 0xb9f, 0xbcc, 0x20, 0xba4, 0xbb0, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb85, 0xb9f, 0xbcd, 0xb9f, 0xbcc, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xb9f, 0xbcd, 0xb9f, 0xbcb, 0xbaa, 0xbc7, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xb9f, 0xbcd, 0xb9f, 0xbcb, 0xbaa, 0xbc7,
0x20, 0xba4, 0xbb0, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xb9f, 0xbcd,
0xb9f, 0xbcb, 0xbaa, 0xbc7, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb85, 0xbb0, 0xbc7, 0xbaa, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb85, 0xbb0, 0xbc7, 0xbaa, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbbf,
0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85,
0xbb0, 0xbc7, 0xbaa, 0xbbf, 0xbaf, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xbb0, 0xbcd, 0xb9c, 0xbc6, 0xba9,
0xbcd, 0xb9f, 0xbbf, 0xba9, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85,
0xbb0, 0xbcd, 0xb9c, 0xbc6, 0xba9, 0xbcd, 0xb9f, 0xbbf, 0xba9, 0xbbe, 0x20, 0xba8,
0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb85, 0xbb0, 0xbcd, 0xb9c, 0xbc6, 0xba9, 0xbcd, 0xb9f, 0xbbf, 0xba9, 0xbbe, 0x20,
0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbc7,
0xbb1, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xb85, 0xbb0, 0xbcd,
0xb9c, 0xbc6, 0xba9, 0xbcd, 0xb9f, 0xbbf, 0xba9, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf,
0x20, 0xb85, 0xbb0, 0xbcd, 0xb9c, 0xbc6, 0xba9, 0xbcd, 0xb9f, 0xbbf, 0xba9, 0xbbe,
0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf,
0x20, 0xb85, 0xbb0, 0xbcd, 0xb9c, 0xbc6, 0xba9, 0xbcd, 0xb9f, 0xbbf, 0xba9, 0xbbe,
0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb86,
0xbb0, 0xbcd, 0xbae, 0xbc7, 0xba9, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb86, 0xbb0, 0xbcd, 0xbae, 0xbc7, 0xba9, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbbf,
0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb86,
0xbb0, 0xbcd, 0xbae, 0xbc7, 0xba9, 0xbbf, 0xbaf, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xb9f, 0xbcd, 0xbb2, 0xbbe, 0xba3,
0xbcd, 0xb9f, 0xbbf, 0xb95, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85,
0xb9f, 0xbcd, 0xbb2, 0xbbe, 0xba3, 0xbcd, 0xb9f, 0xbbf, 0xb95, 0xbcd, 0x20, 0xba8,
0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb85, 0xb9f, 0xbcd, 0xbb2, 0xbbe, 0xba3, 0xbcd, 0xb9f, 0xbbf, 0xb95, 0xbcd, 0x20,
0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xbae, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xb86, 0xbb8, 0xbcd, 0xba4, 0xbbf,
0xbb0, 0xbc7, 0xbb2, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb86,
0xbb8, 0xbcd, 0xba4, 0xbbf, 0xbb0, 0xbc7, 0xbb2, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0x20,
0xbae, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf,
0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb86, 0xbb8, 0xbcd, 0xba4,
0xbbf, 0xbb0, 0xbc7, 0xbb2, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0x20, 0xbae, 0xba4, 0xbcd,
0xba4, 0xbbf, 0xbaf, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb86, 0xbb8, 0xbcd, 0xba4, 0xbbf, 0xbb0, 0xbc7, 0xbb2,
0xbbf, 0xbaf, 0xba9, 0xbcd, 0x20, 0xbae, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20,
0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb86, 0xbb8, 0xbcd, 0xba4, 0xbbf, 0xbb0, 0xbc7, 0xbb2,
0xbbf, 0xbaf, 0xba9, 0xbcd, 0x20, 0xbae, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20,
0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xba8,
0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb86, 0xbb8, 0xbcd, 0xba4, 0xbbf, 0xbb0, 0xbc7, 0xbb2, 0xbbf, 0xbaf, 0xba9, 0xbcd,
0x20, 0xbae, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xbae, 0xbc7, 0xbb1, 0xbcd,
0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3,
0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd,
0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xb86, 0xbb8, 0xbcd, 0xba4, 0xbbf,
0xbb0, 0xbc7, 0xbb2, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb86,
0xbb8, 0xbcd, 0xba4, 0xbbf, 0xbb0, 0xbc7, 0xbb2, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0x20,
0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20,
0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb86, 0xbb8, 0xbcd, 0xba4, 0xbbf, 0xbb0, 0xbc7, 0xbb2, 0xbbf, 0xbaf, 0xba9,
0xbcd, 0x20, 0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf,
0xbaf, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf,
0x20, 0xb86, 0xbb8, 0xbcd, 0xba4, 0xbbf, 0xbb0, 0xbc7, 0xbb2, 0xbbf, 0xbaf, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb86, 0xbb8, 0xbcd, 0xba4, 0xbbf, 0xbb0, 0xbc7,
0xbb2, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0x20, 0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95, 0xba4,
0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb86, 0xbb8, 0xbcd, 0xba4, 0xbbf, 0xbb0,
0xbc7, 0xbb2, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0x20, 0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95,
0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xb9a, 0xbb0, 0xbcd, 0xbaa, 0xbc8,
0xb9c, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xb9a,
0xbb0, 0xbcd, 0xbaa, 0xbc8, 0xb9c, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2,
0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xb9a,
0xbb0, 0xbcd, 0xbaa, 0xbc8, 0xb9c, 0xbbe, 0xba9, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f,
0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xb9a, 0xbcb, 0xbb0, 0xbb8,
0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xb9a, 0xbcb, 0xbb0, 0xbb8,
0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb85, 0xb9a, 0xbcb, 0xbb0, 0xbcd, 0xbb8, 0xbcd, 0x20, 0xb95,
0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5, 0xb99, 0xbcd,
0xb95, 0xba4, 0xbc7, 0xb9a, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5, 0xb99,
0xbcd, 0xb95, 0xba4, 0xbc7, 0xb9a, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe,
0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5, 0xb99, 0xbcd, 0xb95, 0xba4,
0xbc7, 0xb9a, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xbaa, 0xbc2, 0xb9f, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xbaa, 0xbca, 0xbb2, 0xbbf, 0xbb5, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbbf, 0xbb0, 0xbc7, 0xb9a, 0xbbf, 0xbb2, 0xbbf, 0xbaf,
0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbbf, 0xbb0, 0xbc7, 0xb9a,
0xbbf, 0xbb2, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe,
0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbbf, 0xbb0, 0xbc7, 0xb9a,
0xbbf, 0xbb2, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbc1, 0xbb0, 0xbc1, 0xba9, 0xbc7, 0x20, 0xb9f,
0xbb0, 0xbc1, 0xbb8, 0xbcd, 0xbb8, 0xbb2, 0xbbe, 0xbae, 0xbcd, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb95, 0xbc7, 0xbaa, 0xbcd, 0x20, 0xbb5, 0xbc6, 0xbb0, 0xbcd,
0xb9f, 0xbc7, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbc7, 0xbaa, 0xbcd,
0x20, 0xbb5, 0xbc6, 0xbb0, 0xbcd, 0xb9f, 0xbc7, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8,
0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbc7, 0xbaa,
0xbcd, 0x20, 0xbb5, 0xbc6, 0xbb0, 0xbcd, 0xb9f, 0xbc7, 0x20, 0xb95, 0xbcb, 0xb9f,
0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbbe, 0xbae, 0xbcb, 0xbb0,
0xbcb, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb9a, 0xba4, 0xbcd, 0xba4, 0xbbe, 0xbae, 0xbcd, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xba4, 0xbcd, 0xba4, 0xbbe, 0xbae, 0xbcd, 0x20,
0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb9a, 0xba4, 0xbcd, 0xba4, 0xbbe, 0xbae, 0xbcd, 0x20, 0xbaa, 0xb95, 0xbb2,
0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbbf, 0xbb2,
0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbbf, 0xbb2, 0xbbf, 0x20,
0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb9a, 0xbbf, 0xbb2, 0xbbf, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbc0, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb9a, 0xbc0, 0xba9, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbc0, 0xba9, 0x20, 0xbaa, 0xb95,
0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf,
0xbb1, 0xbbf, 0xbb8, 0xbcd, 0xba4, 0xbc1, 0xbae, 0xbb8, 0xbcd, 0x20, 0xba4, 0xbc0,
0xbb5, 0xbc1, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbcb, 0xb95, 0xbcb,
0xbb8, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0xb95, 0xbb3, 0xbcd, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbca, 0xbb2, 0xbae, 0xbcd, 0xbaa, 0xbbf, 0xbaf,
0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbca, 0xbb2, 0xbae, 0xbcd,
0xbaa, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbca, 0xbb2, 0xbae, 0xbcd, 0xbaa,
0xbbf, 0xbaf, 0xbbe, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb95, 0xbc1, 0xb95, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0xb95,
0xbb3, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbc1, 0xb95, 0xbcd,
0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0xb95, 0xbb3, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2,
0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbc1,
0xb95, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0xb95, 0xbb3, 0xbcd, 0x20, 0xb85,
0xbb0, 0xbc8, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb95, 0xbbf, 0xbaf, 0xbc2, 0xbaa, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb95, 0xbbf, 0xbaf, 0xbc2, 0xbaa, 0xbbe, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8,
0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbaf,
0xbc2, 0xbaa, 0xbbe, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9f, 0xbc7, 0xbb5, 0xbbf, 0xbb8, 0xbcd, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9f, 0xbc1, 0xbae, 0xbcb, 0xba3, 0xbcd, 0xb9f, 0xbcd,
0x2d, 0xb9f, 0xbbf, 0x20, 0xb89, 0xbb0, 0xbcd, 0xbb5, 0xbbf, 0xbb2, 0xbcd, 0xbb2,
0xbc7, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd,
0xb95, 0xbc1, 0x20, 0xba4, 0xbbf, 0xbae, 0xbcb, 0xbb0, 0xbcd, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb88, 0xbb8, 0xbcd, 0xb9f, 0xbb0, 0xbcd, 0x20, 0xba4, 0xbc0,
0xbb5, 0xbc1, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb88, 0xbb8, 0xbcd, 0xb9f,
0xbb0, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8,
0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb88, 0xbb8, 0xbcd,
0xb9f, 0xbb0, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0x20, 0xb95, 0xbcb, 0xb9f,
0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb88, 0xb95, 0xbcd, 0xbb5, 0xb9f,
0xbbe, 0xbb0, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xba4, 0xbcd,
0xba4, 0xbbf, 0xbaf, 0x20, 0xb90, 0xbb0, 0xbcb, 0xbaa, 0xbcd, 0xbaa, 0xbbf, 0xbaf,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf,
0x20, 0xb90, 0xbb0, 0xbcb, 0xbaa, 0xbcd, 0xbaa, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbbf,
0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae,
0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xb90, 0xbb0, 0xbcb, 0xbaa, 0xbcd, 0xbaa,
0xbbf, 0xbaf, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf,
0x20, 0xb90, 0xbb0, 0xbcb, 0xbaa, 0xbcd, 0xbaa, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4,
0xbbf, 0xbaf, 0x20, 0xb90, 0xbb0, 0xbcb, 0xbaa, 0xbcd, 0xbaa, 0xbbf, 0xbaf, 0x20,
0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf,
0x20, 0xb90, 0xbb0, 0xbcb, 0xbaa, 0xbcd, 0xbaa, 0xbbf, 0xbaf, 0x20, 0xb95, 0xbcb,
0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba4, 0xbc2, 0xbb0, 0x2d,
0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb90, 0xbb0, 0xbcb, 0xbaa,
0xbcd, 0xbaa, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbc7,
0xbb1, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xb90, 0xbb0, 0xbcb,
0xbaa, 0xbcd, 0xbaa, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae,
0xbc7, 0xbb1, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xb90, 0xbb0,
0xbcb, 0xbaa, 0xbcd, 0xbaa, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf,
0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbc7, 0xbb1, 0xbcd,
0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xb90, 0xbb0, 0xbcb, 0xbaa, 0xbcd,
0xbaa, 0xbbf, 0xbaf, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb83, 0xbaa, 0xbbe, 0xb95, 0xbcd, 0xbb2, 0xbbe, 0xba8, 0xbcd, 0xba4,
0xbc1, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0xb95, 0xbb3, 0xbcd, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb83, 0xbaa, 0xbbe, 0xb95, 0xbcd, 0xbb2, 0xbbe, 0xba8, 0xbcd,
0xba4, 0xbc1, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0xb95, 0xbb3, 0xbcd, 0x20, 0xba8,
0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb83, 0xbaa, 0xbbe, 0xb95, 0xbcd, 0xbb2, 0xbbe, 0xba8, 0xbcd, 0xba4, 0xbc1, 0x20,
0xba4, 0xbc0, 0xbb5, 0xbc1, 0xb95, 0xbb3, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb83, 0xbaa, 0xbbf, 0xb9c, 0xbbf, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb83, 0xbaa, 0xbbf, 0xb9c, 0xbbf, 0x20, 0xba8,
0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb83, 0xbaa, 0xbbf, 0xb9c, 0xbbf, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb83, 0xbaa, 0xbbf, 0xbb0, 0xb9e, 0xbcd, 0xb9a, 0xbc1,
0x20, 0xb95, 0xbaf, 0xbbe, 0xba9, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xbaa, 0xbbf, 0xbb0, 0xbc6, 0xb9e, 0xbcd, 0xb9a, 0xbc1, 0x20, 0xba4, 0xbc6, 0xbb1,
0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0x26, 0x20, 0xb85, 0xba3,
0xbcd, 0xb9f, 0xbbe, 0xbb0, 0xbcd, 0xb9f, 0xbbf, 0xb95, 0xbcd, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb95, 0xbb2, 0xbaa, 0xb95, 0xbcb, 0xbb8, 0xbcd, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbc7, 0xbae, 0xbcd, 0xbaa, 0xbbf, 0xbaf, 0xbb0,
0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9c, 0xbbe, 0xbb0, 0xbcd, 0xb9c,
0xbbf, 0xbaf, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9c, 0xbbe, 0xbb0,
0xbcd, 0xb9c, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe,
0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9c, 0xbbe, 0xbb0, 0xbcd, 0xb9c,
0xbbf, 0xbaf, 0xbbe, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb2, 0xbcd, 0xbaa, 0xbb0, 0xbcd, 0xb9f, 0xbcd, 0x20,
0xba4, 0xbc0, 0xbb5, 0xbc1, 0xb95, 0xbb3, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb95, 0xbbf, 0xbb0, 0xbc0, 0xba9, 0xbcd, 0xbb5, 0xbbf, 0xb9a, 0xbcd, 0x20,
0xb9a, 0xbb0, 0xbbe, 0xb9a, 0xbb0, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb95, 0xbbf, 0xbb0, 0xbc0,
0xba9, 0xbcd, 0xbb2, 0xbbe, 0xba8, 0xbcd, 0xba4, 0xbc1, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb95, 0xbbf,
0xbb0, 0xbc0, 0xba9, 0xbcd, 0xbb2, 0xbbe, 0xba8, 0xbcd, 0xba4, 0xbc1, 0x20, 0xba8,
0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb95, 0xbbf, 0xbb0, 0xbc0,
0xba9, 0xbcd, 0xbb2, 0xbbe, 0xba8, 0xbcd, 0xba4, 0xbc1, 0x20, 0xb95, 0xbcb, 0xb9f,
0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95,
0xbc1, 0x20, 0xb95, 0xbbf, 0xbb0, 0xbc0, 0xba9, 0xbcd, 0xbb2, 0xbbe, 0xba8, 0xbcd,
0xba4, 0xbc1, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbc7, 0xbb1, 0xbcd,
0xb95, 0xbc1, 0x20, 0xb95, 0xbbf, 0xbb0, 0xbc0, 0xba9, 0xbcd, 0xbb2, 0xbbe, 0xba8,
0xbcd, 0xba4, 0xbc1, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95, 0xbc1, 0x20,
0xb95, 0xbbf, 0xbb0, 0xbc0, 0xba9, 0xbcd, 0xbb2, 0xbbe, 0xba8, 0xbcd, 0xba4, 0xbc1,
0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95,
0xbae, 0xbcd, 0x20, 0xba4, 0xbb0, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5,
0xbb3, 0xbc8, 0xb95, 0xbc1, 0xb9f, 0xbbe, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf,
0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb9, 0xbb5, 0xbbe, 0xbaf,
0xbcd, 0x2d, 0xb85, 0xbb2, 0xbc7, 0xbb7, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb9, 0xbb5, 0xbbe, 0xbaf, 0xbcd, 0x2d, 0xb85, 0xbb2,
0xbc7, 0xbb7, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf,
0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb9, 0xbb5, 0xbbe, 0xbaf,
0xbcd, 0x2d, 0xb85, 0xbb2, 0xbc7, 0xbb7, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0x20, 0xbaa,
0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb9,
0xbbe, 0xb99, 0xbcd, 0xb95, 0xbbe, 0xb99, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xbb9, 0xbbe, 0xb99, 0xbcd, 0xb95, 0xbbe, 0xb99, 0xbcd, 0x20, 0xba8, 0xbbf,
0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb9,
0xbbe, 0xb99, 0xbcd, 0xb95, 0xbbe, 0xb99, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb9, 0xbcb, 0xbb5, 0xbcd, 0xba4, 0xbcd,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb9, 0xbcb, 0xbb5, 0xbcd, 0xba4, 0xbcd,
0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xbb9, 0xbcb, 0xbb5, 0xbcd, 0xba4, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f,
0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb87, 0xba8, 0xbcd, 0xba4, 0xbbf,
0xbaf, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb87, 0xba8, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0xbaa, 0xbcd, 0x20,
0xbaa, 0xbc6, 0xbb0, 0xbc1, 0xb99, 0xbcd, 0xb95, 0xb9f, 0xbb2, 0xbcd, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb87, 0xba8, 0xbcd, 0xba4, 0xbcb, 0xb9a, 0xbc0, 0xba9,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf,
0x20, 0xb87, 0xba8, 0xbcd, 0xba4, 0xbcb, 0xba9, 0xbc7, 0xb9a, 0xbbf, 0xbaf, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95, 0xba4,
0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xb87, 0xba8, 0xbcd, 0xba4, 0xbcb, 0xba9, 0xbc7,
0xb9a, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbc7, 0xbb1,
0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xb87, 0xba8, 0xbcd, 0xba4,
0xbcb, 0xba9, 0xbc7, 0xb9a, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb88, 0xbb0, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb88,
0xbb0, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb88, 0xbb0, 0xbbe, 0xba9, 0xbcd, 0x20,
0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb87, 0xbb0, 0xbcd, 0xb95, 0xbc1, 0xb9f, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb87, 0xbb0, 0xbcd, 0xb95, 0xbc1, 0xb9f, 0xbcd,
0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb87, 0xbb0, 0xbcd, 0xb95, 0xbc1, 0xb9f,
0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb87, 0xbb8, 0xbcd, 0xbb0, 0xbc7, 0xbb2, 0xbcd, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb87, 0xbb8, 0xbcd, 0xbb0, 0xbc7, 0xbb2, 0xbcd,
0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb87, 0xbb8, 0xbcd, 0xbb0, 0xbc7, 0xbb2, 0xbcd, 0x20, 0xbaa, 0xb95,
0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9c, 0xbaa,
0xbcd, 0xbaa, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9c,
0xbaa, 0xbcd, 0xbaa, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf,
0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9c, 0xbaa, 0xbcd, 0xbaa,
0xbbe, 0xba9, 0xbcd, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbc6, 0xb9f, 0xbcd, 0xbb0, 0xbcb, 0xbaa, 0xbb5,
0xbcd, 0xbb2, 0xbcb, 0xbb5, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20, 0xb95, 0xbae,
0xbcd, 0xb9a, 0xb9f, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xbaa, 0xbc6, 0xb9f, 0xbcd, 0xbb0, 0xbcb, 0xbaa, 0xbb5, 0xbcd, 0xbb2,
0xbcb, 0xbb5, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20, 0xb95, 0xbae, 0xbcd, 0xb9a,
0xb9f, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbbf, 0x20, 0xba4, 0xbb0, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbc6, 0xb9f, 0xbcd, 0xbb0, 0xbcb, 0xbaa, 0xbb5, 0xbcd,
0xbb2, 0xbcb, 0xbb5, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20, 0xb95, 0xbae, 0xbcd,
0xb9a, 0xb9f, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbbf, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xb9c, 0xb95, 0xbb8, 0xbcd, 0xba4,
0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb4,
0xb95, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb95, 0xb9c, 0xb95, 0xbb8, 0xbcd, 0xba4, 0xbbe,
0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbc7, 0xbb1, 0xbcd,
0xb95, 0xbc1, 0x20, 0xb95, 0xb9c, 0xb95, 0xbb8, 0xbcd, 0xba4, 0xbbe, 0xba9, 0xbcd,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbca, 0xbb0, 0xbbf, 0xbaf, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbca, 0xbb0, 0xbbf, 0xbaf, 0x20, 0xba8,
0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb95, 0xbca, 0xbb0, 0xbbf, 0xbaf, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbcb, 0xbb8, 0xbcd, 0xbb0, 0xbc7,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbcd, 0xbb0, 0xbb8, 0xbcd, 0xba9,
0xbcb, 0xbaf, 0xbbe, 0xbb0, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb95, 0xbcd, 0xbb0, 0xbb8, 0xbcd, 0xba9, 0xbcb, 0xbaf, 0xbbe,
0xbb0, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf,
0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbcd, 0xbb0, 0xbb8,
0xbcd, 0xba9, 0xbcb, 0xbaf, 0xbbe, 0xbb0, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20,
0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf,
0xbb0, 0xbcd, 0xb95, 0xbbf, 0xbb8, 0xbcd, 0xba4, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb2, 0xb99, 0xbcd, 0xb95, 0xbbe, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xbb2, 0xbc8, 0xba9, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1,
0xb95, 0xbb3, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb2, 0xbbe, 0xbb0,
0xbcd, 0xb9f, 0xbcd, 0x20, 0xbb9, 0xbcb, 0xbb5, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xbb2, 0xbbe, 0xbb0, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xbb9, 0xbcb, 0xbb5,
0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xbb2, 0xbbe, 0xbb0, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xbb9, 0xbcb,
0xbb5, 0xbcd, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xbae, 0xb95, 0xbcd, 0xb95, 0xbbe, 0xbb5, 0xbcd, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xb95, 0xbcd, 0xb95, 0xbbe, 0xbb5, 0xbcd, 0x20,
0xba4, 0xbb0, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xb95, 0xbcd, 0xb95,
0xbbe, 0xbb5, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xbae, 0xb95, 0xba4, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xbae, 0xb95, 0xba4, 0xba9, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf,
0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xb95, 0xba4, 0xba9,
0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xbae, 0xbb2, 0xbc7, 0xbb7, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xbae, 0xbbe, 0xbb2, 0xba4, 0xbcd, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0xb95, 0xbb3, 0xbcd,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbbe, 0xbb0, 0xbcd, 0xb95, 0xbbf,
0xbaf, 0xbc2, 0xb9a, 0xbbe, 0xbb8, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xbae, 0xbbe, 0xbb0, 0xbcd, 0xbb7, 0xbb2, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1,
0xb95, 0xbb3, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbca, 0xbb0,
0xbbf, 0xbb7, 0xbbf, 0xbaf, 0xbb8, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xbae, 0xbca, 0xbb0, 0xbbf, 0xbb7, 0xbbf, 0xbaf, 0xbb8, 0xbcd, 0x20, 0xba8, 0xbbf,
0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae,
0xbca, 0xbb0, 0xbbf, 0xbb7, 0xbbf, 0xbaf, 0xbb8, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f,
0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbbe, 0xb9a, 0xba9, 0xbcd,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbc6, 0xb95, 0xbcd, 0xbb8, 0xbbf,
0xb95, 0xba9, 0xbcd, 0x20, 0xbaa, 0xb9a, 0xbbf, 0xbaa, 0xbbf, 0xb95, 0xbcd, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbc6, 0xb95, 0xbcd, 0xbb8, 0xbbf, 0xb95,
0xba9, 0xbcd, 0x20, 0xbaa, 0xb9a, 0xbbf, 0xbaa, 0xbbf, 0xb95, 0xbcd, 0x20, 0xba8,
0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xbae, 0xbc6, 0xb95, 0xbcd, 0xbb8, 0xbbf, 0xb95, 0xba9, 0xbcd, 0x20, 0xbaa, 0xb9a,
0xbbf, 0xbaa, 0xbbf, 0xb95, 0xbcd, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb89, 0xbb2, 0xba9, 0xbcd, 0x20, 0xbaa,
0xbbe, 0xb9f, 0xbb0, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb89, 0xbb2,
0xba9, 0xbcd, 0x20, 0xbaa, 0xbbe, 0xb9f, 0xbb0, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2,
0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb89, 0xbb2,
0xba9, 0xbcd, 0x20, 0xbaa, 0xbbe, 0xb9f, 0xbb0, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f,
0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbbe, 0xbb8, 0xbcd, 0xb95,
0xbcb, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbbe, 0xbb8, 0xbcd, 0xb95,
0xbcb, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xbae, 0xbbe, 0xbb8, 0xbcd, 0xb95, 0xbcb, 0x20, 0xb95, 0xbcb,
0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbbf, 0xbaf, 0xbbe,
0xba9, 0xbcd, 0xbae, 0xbb0, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8,
0xbb5, 0xbcd, 0xbb0, 0xbc2, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8, 0xbc7,
0xbaa, 0xbbe, 0xbb3, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8, 0xbbf, 0xbaf,
0xbc2, 0x20, 0xb95, 0xbb2, 0xbbf, 0xb9f, 0xbcb, 0xba9, 0xbbf, 0xbaf, 0xbbe, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8, 0xbbf, 0xbaf, 0xbc2, 0x20, 0xb95, 0xbb2,
0xbbf, 0xb9f, 0xbcb, 0xba9, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8,
0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8, 0xbbf, 0xbaf,
0xbc2, 0x20, 0xb95, 0xbb2, 0xbbf, 0xb9f, 0xbcb, 0xba9, 0xbbf, 0xbaf, 0xbbe, 0x20,
0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8, 0xbbf,
0xbaf, 0xbc2, 0xb9a, 0xbbf, 0xbb2, 0xbbe, 0xba8, 0xbcd, 0xba4, 0xbc1, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8, 0xbbf, 0xbaf, 0xbc2, 0xb9a, 0xbbf, 0xbb2, 0xbbe,
0xba8, 0xbcd, 0xba4, 0xbc1, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8, 0xbbf, 0xbaf, 0xbc2, 0xb9a, 0xbbf,
0xbb2, 0xbbe, 0xba8, 0xbcd, 0xba4, 0xbc1, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3,
0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8, 0xbbf, 0xbaf, 0xbc2, 0xb83,
0xbaa, 0xbb5, 0xbc1, 0xba3, 0xbcd, 0xb9f, 0xbcd, 0xbb2, 0xbbe, 0xba8, 0xbcd, 0xba4,
0xbc1, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8, 0xbbf, 0xbaf, 0xbc2, 0xb83,
0xbaa, 0xbb5, 0xbc1, 0xba3, 0xbcd, 0xb9f, 0xbcd, 0xbb2, 0xbbe, 0xba8, 0xbcd, 0xba4,
0xbc1, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xba8, 0xbbf, 0xbaf, 0xbc2, 0xb83, 0xbaa, 0xbb5, 0xbc1, 0xba3,
0xbcd, 0xb9f, 0xbcd, 0xbb2, 0xbbe, 0xba8, 0xbcd, 0xba4, 0xbc1, 0x20, 0xbaa, 0xb95,
0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8, 0xbbf,
0xbaf, 0xbc1, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8, 0xbbe, 0xbb0, 0xbcd,
0xb83, 0xbaa, 0xbcb, 0xb95, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8, 0xbbe, 0xbb0, 0xbcd, 0xb83, 0xbaa, 0xbcb, 0xb95,
0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf,
0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8, 0xbbe, 0xbb0, 0xbcd,
0xb83, 0xbaa, 0xbcb, 0xb95, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0x20, 0xbaa,
0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa,
0xbc6, 0xbb0, 0xbcd, 0xba9, 0xbbe, 0xba3, 0xbcd, 0xb9f, 0xbcb, 0x20, 0xb9f, 0xbbf,
0x20, 0xba8, 0xbcb, 0xbb0, 0xba9, 0xbcd, 0xbb9, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xbaa, 0xbc6, 0xbb0, 0xbcd, 0xba9, 0xbbe, 0xba9, 0xbcd, 0xb9f, 0xbcb,
0x20, 0xb9f, 0xbbf, 0x20, 0xba8, 0xbcb, 0xbb0, 0xbcb, 0xba9, 0xbcd, 0xbb9, 0xbbe,
0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xbaa, 0xbc6, 0xbb0, 0xbcd, 0xba9, 0xbbe, 0xba9, 0xbcd, 0xb9f, 0xbcb,
0x20, 0xb9f, 0xbbf, 0x20, 0xba8, 0xbcb, 0xbb0, 0xbcb, 0xba9, 0xbcd, 0xbb9, 0xbbe,
0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5,
0xb9f, 0xb95, 0xbcd, 0xb95, 0xbc1, 0x20, 0xbae, 0xbb0, 0xbbf, 0xba9, 0xbbe, 0x20,
0xba4, 0xbc0, 0xbb5, 0xbc1, 0xb95, 0xbb3, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xba8, 0xbcb, 0xbb5, 0xbcb, 0xb9a, 0xbbf, 0xbaa, 0xbbf, 0xbb0, 0xbbf, 0xbb8,
0xbcd, 0xb95, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8, 0xbcb, 0xbb5,
0xbcb, 0xb9a, 0xbbf, 0xbaa, 0xbbf, 0xbb0, 0xbbf, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20,
0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xba8, 0xbcb, 0xbb5, 0xbcb, 0xb9a, 0xbbf, 0xbaa, 0xbbf, 0xbb0, 0xbbf, 0xbb8,
0xbcd, 0xb95, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb93, 0xbae, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb93, 0xbae, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20, 0xba8,
0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb93, 0xbae, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbbe, 0xb95, 0xbbf, 0xbb8, 0xbcd,
0xba4, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbbe,
0xb95, 0xbbf, 0xbb8, 0xbcd, 0xba4, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2,
0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbbe,
0xb95, 0xbbf, 0xbb8, 0xbcd, 0xba4, 0xbbe, 0xba9, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f,
0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbbe, 0xbb2, 0xbb5, 0xbcd,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbaa, 0xbc1, 0xbb5, 0xbbe, 0x20,
0xba8, 0xbbf, 0xbaf, 0xbc2, 0x20, 0xb95, 0xbbf, 0xba9, 0xbbf, 0xbaf, 0xbbe, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbb0, 0xbbe, 0xb95, 0xbc1, 0xbb5, 0xbc7,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbb0, 0xbbe, 0xb95, 0xbc1, 0xbb5,
0xbc7, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbb0, 0xbbe, 0xb95, 0xbc1, 0xbb5, 0xbc7, 0x20, 0xb95,
0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbc6, 0xbb0,
0xbc1, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbc6, 0xbb0, 0xbc1, 0x20,
0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xbaa, 0xbc6, 0xbb0, 0xbc1, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbbf, 0xbb2, 0xbbf, 0xbaa, 0xbcd, 0xbaa, 0xbc8,
0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbbf, 0xbb2, 0xbbf,
0xbaa, 0xbcd, 0xbaa, 0xbc8, 0xba9, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf,
0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbbf, 0xbb2, 0xbbf,
0xbaa, 0xbcd, 0xbaa, 0xbc8, 0xba9, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb83, 0xbaa, 0xbcb, 0xba9, 0xbbf, 0xb95, 0xbcd,
0xbb8, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0xb95, 0xbb3, 0xbcd, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbc6, 0xbaf, 0xbbf, 0xba9, 0xbcd, 0xb9f, 0xbcd,
0x20, 0xbaa, 0xbbf, 0xbaf, 0xbb0, 0xbbf, 0x20, 0x26, 0x20, 0xbae, 0xbbf, 0xb95,
0xbcd, 0xb95, 0xbc1, 0xbaf, 0xbbf, 0xbb2, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbc6, 0xbaf, 0xbbf, 0xba9, 0xbcd, 0xb9f, 0xbcd, 0x20,
0xbaa, 0xbbf, 0xbaf, 0xbb0, 0xbbf, 0x20, 0x26, 0x20, 0xbae, 0xbbf, 0xb95, 0xbcd,
0xb95, 0xbc1, 0xbaf, 0xbbf, 0xbb2, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2,
0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbc6,
0xbaf, 0xbbf, 0xba9, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xbaa, 0xbbf, 0xbaf, 0xbb0, 0xbbf,
0x20, 0x26, 0x20, 0xbae, 0xbbf, 0xb95, 0xbcd, 0xb95, 0xbc1, 0xbaf, 0xbbf, 0xbb2,
0xbbe, 0xba9, 0xbcd, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbbf, 0xb9f, 0xbcd, 0xb95, 0xbc6, 0xbaf, 0xbcd,
0xbb0, 0xbcd, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbcb,
0xba9, 0xbbe, 0xbaa, 0xbc7, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbbf,
0xbaf, 0xbbe, 0xb99, 0xbcd, 0xbaf, 0xbbe, 0xb99, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb95, 0xbc8, 0xb9c, 0xbc8, 0xbb2, 0xbcb, 0xbb0, 0xbcd, 0xb9f, 0xbbe,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbc8, 0xb9c, 0xbc8, 0xbb2, 0xbcb,
0xbb0, 0xbcd, 0xb9f, 0xbbe, 0x20, 0xba4, 0xbb0, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb95, 0xbc8, 0xb9c, 0xbc8, 0xbb2, 0xbcb, 0xbb0, 0xbcd, 0xb9f, 0xbbe, 0x20,
0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb0, 0xbc0,
0xbaf, 0xbc2, 0xba9, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xbb0, 0xbcb, 0xba4, 0xbc7, 0xbb0, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb9a, 0xb95, 0xbb2, 0xbbf, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb9a, 0xb95, 0xbb2, 0xbbf, 0xba9, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8,
0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xb95, 0xbb2,
0xbbf, 0xba9, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb9a, 0xbae, 0xbb0, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb9a, 0xbae, 0xbb0, 0xbbe, 0x20, 0xba4, 0xbb0, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb9a, 0xbae, 0xbb0, 0xbbe, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbae, 0xbcb, 0xbb5, 0xbbe, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbae, 0xbcb, 0xbb5, 0xbbe, 0x20, 0xba8, 0xbbf, 0xbb2,
0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbae,
0xbcb, 0xbb5, 0xbbe, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbc0, 0xb9a, 0xbc6, 0xbb2, 0xbcd, 0xbb8, 0xbcd,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbbf, 0xb99, 0xbcd, 0xb95, 0xbaa,
0xbcd, 0xbaa, 0xbc2, 0xbb0, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe,
0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbbe, 0xbb2, 0xbae, 0xba9,
0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0xb95, 0xbb3, 0xbcd, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xba4, 0xbc6, 0xbb1, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb9c, 0xbbe,
0xbb0, 0xbcd, 0xb9c, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb9a, 0xbc1, 0xbb0, 0xbbf, 0xba9, 0xbbe, 0xbae, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xbb8, 0xbcd, 0xbaf, 0xbcb, 0xbb5, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xba4, 0xbb9, 0xbbf, 0xba4, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xba4, 0xbbe, 0xbaf, 0xbcd, 0xbaa, 0xbc7, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xba4, 0xbbe, 0xbaf, 0xbcd, 0xbaa, 0xbc7, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8,
0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba4, 0xbbe, 0xbaf,
0xbcd, 0xbaa, 0xbc7, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba4, 0xb9c, 0xbbf, 0xb95, 0xbbf, 0xbb8, 0xbcd, 0xba4,
0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9f, 0xbcb, 0xb95,
0xbcd, 0xb95, 0xbc6, 0xbb2, 0xbbe, 0xbb5, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb9f, 0xbcb, 0xb99, 0xbcd, 0xb95, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb9f, 0xbcb, 0xb99, 0xbcd, 0xb95, 0xbbe, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8,
0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9f, 0xbcb, 0xb99,
0xbcd, 0xb95, 0xbbe, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb9a, 0xbc1, 0xb95, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xba4, 0xbc1, 0xbb0, 0xbcd, 0xb95, 0xbcd, 0xbae, 0xbc6, 0xba9, 0xbbf, 0xbb8, 0xbcd,
0xba4, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba4, 0xbc1,
0xbb0, 0xbcd, 0xb95, 0xbcd, 0xbae, 0xbc6, 0xba9, 0xbbf, 0xbb8, 0xbcd, 0xba4, 0xbbe,
0xba9, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba4, 0xbc1, 0xbb0, 0xbcd, 0xb95, 0xbcd, 0xbae, 0xbc6,
0xba9, 0xbbf, 0xbb8, 0xbcd, 0xba4, 0xbbe, 0xba9, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f,
0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba4, 0xbc1, 0xbb5, 0xbbe, 0xbb2,
0xbc1, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb89, 0xbb0, 0xbc1, 0xb95, 0xbc1,
0xbb5, 0xbc7, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb89, 0xbb0, 0xbc1, 0xb95,
0xbc1, 0xbb5, 0xbc7, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb89, 0xbb0, 0xbc1, 0xb95, 0xbc1, 0xbb5, 0xbc7,
0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb89,
0xbb8, 0xbcd, 0xbaa, 0xbc6, 0xb95, 0xbbf, 0xbb8, 0xbcd, 0xba4, 0xbbe, 0xba9, 0xbcd,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb89, 0xbb8, 0xbcd, 0xbaa, 0xbc6, 0xb95,
0xbbf, 0xbb8, 0xbcd, 0xba4, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8,
0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb89, 0xbb8, 0xbcd,
0xbaa, 0xbc6, 0xb95, 0xbbf, 0xbb8, 0xbcd, 0xba4, 0xbbe, 0xba9, 0xbcd, 0x20, 0xb95,
0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5, 0xba9, 0xbc1,
0xbb5, 0xbbe, 0xb9f, 0xbcd, 0xb9f, 0xbc1, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xbb5, 0xba9, 0xbc1, 0xbb5, 0xbbe, 0xb9f, 0xbcd, 0xb9f, 0xbc1, 0x20, 0xba8, 0xbbf,
0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5,
0xba9, 0xbc1, 0xbb5, 0xbbe, 0xb9f, 0xbcd, 0xb9f, 0xbc1, 0x20, 0xb95, 0xbcb, 0xb9f,
0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5, 0xbc6, 0xba9, 0xbbf, 0xb9a,
0xbc1, 0xbb2, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5, 0xbbf, 0xbb3,
0xbbe, 0xb9f, 0xbbf, 0xbb5, 0xbcb, 0xbb8, 0xbcd, 0xb9f, 0xbcb, 0xb95, 0xbcd, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5, 0xbbf, 0xbb3, 0xbbe, 0xb9f, 0xbbf, 0xbb5,
0xbcb, 0xbb8, 0xbcd, 0xb9f, 0xbcb, 0xb95, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8,
0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5, 0xbbf, 0xbb3,
0xbbe, 0xb9f, 0xbbf, 0xbb5, 0xbcb, 0xbb8, 0xbcd, 0xb9f, 0xbcb, 0xb95, 0xbcd, 0x20,
0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5, 0xbcb,
0xbb2, 0xbcd, 0xb95, 0xbcb, 0xb95, 0xbcd, 0xbb0, 0xbbe, 0xb9f, 0xbcd, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5, 0xbcb, 0xbb2, 0xbcd, 0xb95, 0xbcb, 0xb95, 0xbcd,
0xbb0, 0xbbe, 0xb9f, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5, 0xbcb, 0xbb2, 0xbcd, 0xb95, 0xbcb,
0xb95, 0xbcd, 0xbb0, 0xbbe, 0xb9f, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5, 0xbc7, 0xb95, 0xbcd, 0x20, 0xba4, 0xbc0,
0xbb5, 0xbc1, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5, 0xbbe, 0xbb2, 0xbbf,
0xbb8, 0xbcd, 0x20, 0xbae, 0xbb1, 0xbcd, 0xbb1, 0xbc1, 0xbae, 0xbcd, 0x20, 0xb83,
0xbaa, 0xbcd, 0xbaf, 0xbc2, 0xb9f, 0xbc1, 0xba9, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xbaf, 0xb95, 0xbc1, 0xb9f, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaf, 0xb95, 0xbc1, 0xb9f, 0xbcd, 0xbb8, 0xbcd,
0xb95, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaf, 0xb95, 0xbc1, 0xb9f, 0xbcd, 0xbb8, 0xbcd, 0xb95,
0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xbaf, 0xbc7, 0xb95, 0xbbe, 0xb9f, 0xbc6, 0xbb0, 0xbbf, 0xba9, 0xbcd, 0xbaa, 0xbb0,
0xbcd, 0xb95, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaf, 0xbc7, 0xb95,
0xbbe, 0xb9f, 0xbc6, 0xbb0, 0xbbf, 0xba9, 0xbcd, 0xbaa, 0xbb0, 0xbcd, 0xb95, 0xbcd,
0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xbaf, 0xbc7, 0xb95, 0xbbe, 0xb9f, 0xbc6, 0xbb0, 0xbbf, 0xba9, 0xbcd,
0xbaa, 0xbb0, 0xbcd, 0xb95, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaf, 0xbc2, 0xb95, 0xbcb, 0xba9, 0xbcd, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x43, 0x6f,
0x6e, 0x2d, 0x41, 0x6d, 0x61, 0x72, 0x69, 0x6b, 0x61, 0x53, 0x6e, 0x65,
0x67, 0x75, 0x6e, 0x20, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x43, 0x6f,
0x6e, 0x2d, 0x41, 0x6d, 0x61, 0x72, 0x69, 0x6b, 0x61, 0x4a, 0x69, 0x6b,
0x61, 0x6e, 0x20, 0x43, 0x6f, 0x6e, 0x2d, 0x41, 0x6d, 0x61, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x6f, 0x20, 0x4b, 0x61, 0x72, 0x61, 0x74, 0x20, 0x52,
0x62, 0x61, 0x67, 0x61, 0x6e, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x54,
0x6f, 0x6e, 0x2d, 0x41, 0x6d, 0x61, 0x72, 0x69, 0x6b, 0x61, 0x53, 0x6e,
0x65, 0x67, 0x75, 0x6e, 0x20, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x54,
0x6f, 0x6e, 0x2d, 0x41, 0x6d, 0x61, 0x72, 0x69, 0x6b, 0x61, 0x4a, 0x69,
0x6b, 0x61, 0x6e, 0x20, 0x54, 0x6f, 0x6e, 0x2d, 0x41, 0x6d, 0x61, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x6f, 0x20, 0x4b, 0x61, 0x72, 0x61, 0x74, 0x20,
0x52, 0x62, 0x61, 0x67, 0x61, 0x6e, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20,
0x59, 0x61, 0x6d, 0x61, 0x2d, 0x41, 0x6d, 0x61, 0x72, 0x69, 0x6b, 0x61,
0x53, 0x6e, 0x65, 0x67, 0x75, 0x6e, 0x20, 0x4a, 0x69, 0x6b, 0x61, 0x6e,
0x20, 0x59, 0x61, 0x6d, 0x61, 0x2d, 0x41, 0x6d, 0x61, 0x72, 0x69, 0x6b,
0x61, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x48, 0x69, 0x64, 0x61, 0x77,
0x20, 0x6e, 0x69, 0x71, 0x20, 0x59, 0x61, 0x6d, 0x61, 0x2d, 0x41, 0x6d,
0x61, 0x72, 0x69, 0x6b, 0x61, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x44,
0x61, 0x79, 0x62, 0x69, 0x6e, 0x79, 0x61, 0x6e, 0x53, 0x6e, 0x65, 0x67,
0x75, 0x6e, 0x20, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x41, 0x6d, 0x61,
0x72, 0x69, 0x6b, 0x61, 0x2d, 0x44, 0x61, 0x79, 0x62, 0x69, 0x6e, 0x79,
0x61, 0x6e, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x41, 0x6d, 0x61, 0x72,
0x69, 0x6b, 0x61, 0x2d, 0x44, 0x61, 0x79, 0x62, 0x69, 0x6e, 0x79, 0x61,
0x6e, 0x20, 0x6f, 0x20, 0x4b, 0x61, 0x72, 0x61, 0x74, 0x20, 0x52, 0x62,
0x61, 0x67, 0x61, 0x6e, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x59, 0x61,
0x79, 0x75, 0x6e, 0x20, 0x54, 0x61, 0x73, 0x69, 0x79, 0x61, 0x6e, 0x53,
0x6e, 0x65, 0x67, 0x75, 0x6e, 0x20, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20,
0x59, 0x61, 0x79, 0x75, 0x6e, 0x20, 0x54, 0x61, 0x73, 0x69, 0x79, 0x61,
0x6e, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x59, 0x61, 0x79, 0x75, 0x6e,
0x20, 0x54, 0x61, 0x73, 0x69, 0x79, 0x61, 0x6e, 0x20, 0x6f, 0x20, 0x4b,
0x61, 0x72, 0x61, 0x74, 0x20, 0x52, 0x62, 0x61, 0x67, 0x61, 0x6e, 0x4a,
0x69, 0x6b, 0x61, 0x6e, 0x20, 0x43, 0x6f, 0x6e, 0x6f, 0x77, 0x53, 0x6e,
0x65, 0x67, 0x75, 0x6e, 0x20, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x43,
0x6f, 0x6e, 0x6f, 0x77, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x43, 0x6f,
0x6e, 0x6f, 0x77, 0x20, 0x6f, 0x20, 0x4b, 0x61, 0x72, 0x61, 0x74, 0x20,
0x52, 0x62, 0x61, 0x67, 0x61, 0x6e, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20,
0x54, 0x6f, 0x6e, 0x6f, 0x77, 0x53, 0x6e, 0x65, 0x67, 0x75, 0x6e, 0x20,
0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x54, 0x6f, 0x6e, 0x6f, 0x77, 0x4a,
0x69, 0x6b, 0x61, 0x6e, 0x20, 0x54, 0x6f, 0x6e, 0x6f, 0x77, 0x20, 0x6f,
0x20, 0x4b, 0x61, 0x72, 0x61, 0x74, 0x20, 0x52, 0x62, 0x61, 0x67, 0x61,
0x6e, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x53, 0x69, 0x6f, 0x77, 0x53,
0x6e, 0x65, 0x67, 0x75, 0x6e, 0x20, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20,
0x53, 0x69, 0x6f, 0x77, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x53, 0x69,
0x6f, 0x77, 0x20, 0x6f, 0x20, 0x4b, 0x61, 0x72, 0x61, 0x74, 0x20, 0x52,
0x62, 0x61, 0x67, 0x61, 0x6e, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x51,
0x75, 0x72, 0x69, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x77, 0x65, 0x79, 0x6a,
0x69, 0x410, 0x43a, 0x440, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410,
0x43a, 0x440, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x43a, 0x440, 0x20, 0x497, 0x4d9,
0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x4d8, 0x444,
0x433, 0x430, 0x43d, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x4ae, 0x437, 0x4d9, 0x43a, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a,
0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x447,
0x44b, 0x433, 0x44b, 0x448, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x44c, 0x44f, 0x43a,
0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x431, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x410,
0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x41a, 0x4e9, 0x43d, 0x431, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x410, 0x444, 0x440,
0x438, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x431, 0x430,
0x442, 0x44b, 0x448, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x497,
0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410,
0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x43b,
0x44f, 0x441, 0x43a, 0x430, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a,
0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x43c, 0x430, 0x437,
0x43e, 0x43d, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x43c, 0x430, 0x437,
0x43e, 0x43d, 0x43a, 0x430, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x4e9, 0x43d, 0x44c, 0x44f, 0x43a, 0x20,
0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x4af, 0x437, 0x4d9, 0x43a,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x4e9, 0x43d, 0x44c, 0x44f,
0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x433, 0x430,
0x434, 0x4d9, 0x442, 0x438, 0x20, 0x4af, 0x437, 0x4d9, 0x43a, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x4e9, 0x43d, 0x44c, 0x44f, 0x43a, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435,
0x20, 0x4af, 0x437, 0x4d9, 0x43a, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x422, 0x4e9, 0x43d, 0x44c, 0x44f, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438,
0x43a, 0x430, 0x20, 0x43a, 0x4e9, 0x43d, 0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x4e9, 0x43d, 0x44c, 0x44f, 0x43a,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x433, 0x430, 0x434,
0x4d9, 0x442, 0x438, 0x20, 0x43a, 0x4e9, 0x43d, 0x447, 0x44b, 0x433, 0x44b, 0x448,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x4e9, 0x43d, 0x44c, 0x44f,
0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x497, 0x4d9,
0x439, 0x433, 0x435, 0x20, 0x43a, 0x4e9, 0x43d, 0x447, 0x44b, 0x433, 0x44b, 0x448,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x4e9, 0x43d, 0x44c, 0x44f,
0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x442, 0x430,
0x443, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x4e9, 0x43d, 0x44c,
0x44f, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x433,
0x430, 0x434, 0x4d9, 0x442, 0x438, 0x20, 0x442, 0x430, 0x443, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x4e9, 0x43d, 0x44c, 0x44f, 0x43a, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435,
0x20, 0x442, 0x430, 0x443, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422,
0x4e9, 0x43d, 0x44c, 0x44f, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a,
0x430, 0x20, 0x422, 0x44b, 0x43d, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x4e9, 0x43d, 0x44c, 0x44f, 0x43a,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x433, 0x430, 0x434,
0x4d9, 0x442, 0x438, 0x20, 0x422, 0x44b, 0x43d, 0x20, 0x43e, 0x43a, 0x435, 0x430,
0x43d, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x4e9, 0x43d, 0x44c,
0x44f, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x497,
0x4d9, 0x439, 0x433, 0x435, 0x20, 0x422, 0x44b, 0x43d, 0x20, 0x43e, 0x43a, 0x435,
0x430, 0x43d, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x43d, 0x430,
0x434, 0x44b, 0x440, 0x44c, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410,
0x43d, 0x430, 0x434, 0x44b, 0x440, 0x44c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x43d,
0x430, 0x434, 0x44b, 0x440, 0x44c, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x43f, 0x438, 0x430, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x43f, 0x438, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x410, 0x43f, 0x438, 0x430, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x413, 0x430, 0x440, 0x4d9, 0x43f, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x413, 0x430, 0x440, 0x4d9, 0x43f, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x413, 0x430, 0x440, 0x4d9, 0x43f, 0x20, 0x497, 0x4d9, 0x439,
0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x440, 0x433,
0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20,
0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x41a, 0x4e9, 0x43d, 0x431, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x410, 0x440, 0x433,
0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x41a, 0x4e9, 0x43d, 0x431, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x410, 0x440,
0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a,
0x4e9, 0x43d, 0x431, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x410, 0x440, 0x433, 0x435,
0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x438,
0x44f, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x440, 0x43c, 0x435,
0x43d, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x440, 0x43c, 0x435, 0x43d,
0x438, 0x44f, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x422, 0x4e9, 0x43d, 0x44c, 0x44f, 0x43a, 0x20, 0x410, 0x43c,
0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x430, 0x442, 0x43b, 0x430, 0x43d, 0x442,
0x438, 0x43a, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x4e9, 0x43d,
0x44c, 0x44f, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20,
0x433, 0x430, 0x434, 0x4d9, 0x442, 0x438, 0x20, 0x430, 0x442, 0x43b, 0x430, 0x43d,
0x442, 0x438, 0x43a, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x4e9,
0x43d, 0x44c, 0x44f, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430,
0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x430, 0x442, 0x43b, 0x430, 0x43d,
0x442, 0x438, 0x43a, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x4ae, 0x437,
0x4d9, 0x43a, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x432, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x44f, 0x20, 0x4af, 0x437, 0x4d9, 0x43a, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x4af,
0x437, 0x4d9, 0x43a, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x44f, 0x20, 0x4af, 0x437, 0x4d9, 0x43a, 0x20, 0x43a, 0x4e9, 0x43d, 0x431, 0x430,
0x442, 0x44b, 0x448, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x4af, 0x437, 0x4d9, 0x43a,
0x20, 0x43a, 0x4e9, 0x43d, 0x431, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x4af,
0x437, 0x4d9, 0x43a, 0x20, 0x43a, 0x4e9, 0x43d, 0x431, 0x430, 0x442, 0x44b, 0x448,
0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x41a, 0x4e9, 0x43d, 0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x410, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x410,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x41a, 0x4e9, 0x43d, 0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x431, 0x430,
0x442, 0x44b, 0x448, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x44f, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x431,
0x430, 0x442, 0x44b, 0x448, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x431, 0x430, 0x442,
0x44b, 0x448, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f,
0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x4d8, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x497, 0x430, 0x43d, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x4d8, 0x437, 0x435, 0x440, 0x431, 0x430,
0x439, 0x497, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x4d8, 0x437, 0x435, 0x440,
0x431, 0x430, 0x439, 0x497, 0x430, 0x43d, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x437, 0x43e, 0x440, 0x20,
0x443, 0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x437, 0x43e, 0x440, 0x20, 0x443, 0x442, 0x440,
0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x437,
0x43e, 0x443, 0x20, 0x443, 0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b,
0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434,
0x435, 0x448, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430,
0x434, 0x435, 0x448, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x411, 0x443, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x411, 0x43e, 0x43b, 0x438, 0x432, 0x438, 0x44f, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b,
0x438, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x411, 0x440, 0x430,
0x437, 0x438, 0x43b, 0x438, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x411, 0x440, 0x430,
0x437, 0x438, 0x43b, 0x438, 0x430, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x411, 0x440, 0x443, 0x43d, 0x435, 0x439,
0x2d, 0x414, 0x430, 0x440, 0x443, 0x441, 0x441, 0x430, 0x43b, 0x430, 0x43c, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412,
0x435, 0x440, 0x434, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a,
0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x20,
0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x427, 0x430, 0x43c, 0x43e, 0x440, 0x440, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x427,
0x430, 0x442, 0x435, 0x43c, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x427,
0x430, 0x442, 0x435, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x427, 0x430, 0x442, 0x435,
0x43c, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x427, 0x438, 0x43b, 0x438, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x427, 0x438, 0x43b, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x427, 0x438, 0x43b,
0x438, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x41a, 0x44b, 0x442, 0x430, 0x439, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x41a, 0x44b, 0x442, 0x430, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a,
0x44b, 0x442, 0x430, 0x439, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x420, 0x430, 0x448, 0x442, 0x443, 0x430, 0x20,
0x443, 0x442, 0x440, 0x430, 0x432, 0x44b, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x20, 0x443, 0x442, 0x440, 0x430, 0x443,
0x43b, 0x430, 0x440, 0x44b, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a,
0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x44f, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x44f, 0x20, 0x497,
0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a,
0x443, 0x43a, 0x20, 0x443, 0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x443, 0x43a, 0x20, 0x443,
0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x41a, 0x443, 0x43a, 0x20, 0x443, 0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440,
0x44b, 0x20, 0x443, 0x440, 0x442, 0x430, 0x447, 0x430, 0x20, 0x497, 0x4d9, 0x439,
0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x443, 0x431,
0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x443, 0x431, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x443, 0x431, 0x430, 0x20, 0x497, 0x4d9, 0x439,
0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x414, 0x44d, 0x432,
0x438, 0x441, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x414, 0x44e, 0x43c,
0x43e, 0x43d, 0x20, 0x434, 0x2019, 0x42e, 0x440, 0x432, 0x438, 0x43b, 0x44c, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x447, 0x44b, 0x433,
0x44b, 0x448, 0x20, 0x422, 0x438, 0x43c, 0x43e, 0x440, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x441, 0x445, 0x430, 0x20, 0x443, 0x442, 0x440,
0x430, 0x432, 0x44b, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41f, 0x430,
0x441, 0x445, 0x430, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x44b, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x41f, 0x430, 0x441, 0x445, 0x430, 0x20, 0x443, 0x442, 0x440, 0x430,
0x432, 0x44b, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x42d, 0x43a, 0x432, 0x430, 0x434, 0x43e, 0x440, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x4ae, 0x437, 0x4d9, 0x43a, 0x20, 0x415, 0x432,
0x440, 0x43e, 0x43f, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x433,
0x430, 0x434, 0x4d9, 0x442, 0x438, 0x20, 0x4ae, 0x437, 0x4d9, 0x43a, 0x20, 0x415,
0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x4ae, 0x437, 0x4d9, 0x43a, 0x20, 0x415,
0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x41a, 0x4e9, 0x43d, 0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x415, 0x432, 0x440,
0x43e, 0x43f, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x433, 0x430,
0x434, 0x4d9, 0x442, 0x438, 0x20, 0x41a, 0x4e9, 0x43d, 0x447, 0x44b, 0x433, 0x44b,
0x448, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x41a, 0x4e9, 0x43d,
0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x415, 0x440, 0x430, 0x43a, 0x20,
0x41a, 0x4e9, 0x43d, 0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x415, 0x432, 0x440,
0x43e, 0x43f, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9,
0x43d, 0x431, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f,
0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x433, 0x430, 0x434, 0x4d9,
0x442, 0x438, 0x20, 0x41a, 0x4e9, 0x43d, 0x431, 0x430, 0x442, 0x44b, 0x448, 0x20,
0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x41a, 0x4e9, 0x43d, 0x431, 0x430,
0x442, 0x44b, 0x448, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d,
0x434, 0x20, 0x443, 0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435,
0x43d, 0x434, 0x20, 0x443, 0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434,
0x20, 0x443, 0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b, 0x20, 0x497,
0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x424,
0x438, 0x434, 0x436, 0x438, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x424,
0x438, 0x434, 0x436, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x424, 0x438, 0x434, 0x436,
0x438, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x443, 0x437, 0x20, 0x413, 0x432,
0x438, 0x430, 0x43d, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x424,
0x440, 0x430, 0x43d, 0x446, 0x443, 0x437, 0x20, 0x43a, 0x4e9, 0x43d, 0x44c, 0x44f,
0x43a, 0x20, 0x4bb, 0x4d9, 0x43c, 0x20, 0x410, 0x43d, 0x442, 0x430, 0x440, 0x43a,
0x442, 0x438, 0x43a, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x413,
0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441, 0x20, 0x443, 0x442, 0x440,
0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x413, 0x430, 0x43c, 0x431, 0x44c, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x413, 0x440, 0x443, 0x437, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x413, 0x440, 0x443, 0x437, 0x438, 0x44f, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x413, 0x440, 0x443, 0x437, 0x438, 0x44f, 0x20, 0x497, 0x4d9, 0x439, 0x433,
0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x413, 0x438, 0x43b, 0x431,
0x435, 0x440, 0x442, 0x20, 0x443, 0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440,
0x44b, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x413, 0x440, 0x438, 0x43d,
0x432, 0x438, 0x447, 0x20, 0x443, 0x440, 0x442, 0x430, 0x447, 0x430, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x447, 0x44b, 0x433, 0x44b,
0x448, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x447, 0x44b,
0x433, 0x44b, 0x448, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434,
0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x447, 0x44b, 0x433,
0x44b, 0x448, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438,
0x44f, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x431, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x413,
0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x431, 0x430, 0x442, 0x44b, 0x448,
0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x431, 0x430, 0x442, 0x44b, 0x448, 0x20,
0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x497,
0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x424,
0x430, 0x440, 0x441, 0x44b, 0x20, 0x43a, 0x443, 0x43b, 0x442, 0x44b, 0x433, 0x44b,
0x43d, 0x44b, 0x4a3, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x413, 0x430, 0x439, 0x430, 0x43d,
0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x413, 0x430, 0x432, 0x430,
0x439, 0x2d, 0x410, 0x43b, 0x435, 0x443, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x413, 0x430, 0x432, 0x430, 0x439, 0x2d, 0x410, 0x43b, 0x435, 0x443,
0x442, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x413, 0x430, 0x432, 0x430, 0x439, 0x2d, 0x410,
0x43b, 0x435, 0x443, 0x442, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x413, 0x43e, 0x43d, 0x43a, 0x43e,
0x43d, 0x433, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d,
0x433, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x425, 0x43e, 0x432, 0x434, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x425, 0x43e, 0x432, 0x434, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x425, 0x43e, 0x432,
0x434, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x4ba, 0x438, 0x43d, 0x434, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x4ba, 0x438,
0x43d, 0x434, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x44b, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x4ba, 0x438, 0x43d, 0x434, 0x2d, 0x43a, 0x44b, 0x442,
0x430, 0x439, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x4ae, 0x437, 0x4d9,
0x43a, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x44f, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x447, 0x44b, 0x433,
0x44b, 0x448, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x44f,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x431, 0x430,
0x442, 0x44b, 0x448, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438,
0x44f, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x418, 0x440, 0x430, 0x43d,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x418, 0x440, 0x430, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x418, 0x440, 0x430, 0x43d, 0x20, 0x497, 0x4d9, 0x439, 0x433,
0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x418, 0x440, 0x43a, 0x443,
0x442, 0x441, 0x43a, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x418, 0x440,
0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x418, 0x440, 0x43a,
0x443, 0x442, 0x441, 0x43a, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x418, 0x441, 0x440, 0x430, 0x438, 0x43b, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x418, 0x441, 0x440, 0x430, 0x438, 0x43b,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x418, 0x441, 0x440, 0x430, 0x438, 0x43b, 0x20, 0x497,
0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x42f,
0x43f, 0x43e, 0x43d, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x42f, 0x43f,
0x43e, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x42f, 0x43f, 0x43e, 0x43d, 0x20, 0x497,
0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41f,
0x435, 0x442, 0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a,
0x2d, 0x41a, 0x430, 0x43c, 0x447, 0x430, 0x442, 0x441, 0x43a, 0x438, 0x439, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41f, 0x435, 0x442, 0x440, 0x43e, 0x43f,
0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x2d, 0x41a, 0x430, 0x43c, 0x447,
0x430, 0x442, 0x441, 0x43a, 0x438, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41f, 0x435,
0x442, 0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x2d,
0x41a, 0x430, 0x43c, 0x447, 0x430, 0x442, 0x441, 0x43a, 0x438, 0x439, 0x20, 0x497,
0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a,
0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20,
0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x431, 0x430, 0x442, 0x44b, 0x448,
0x20, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x43e, 0x440, 0x435, 0x44f, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x43e, 0x440, 0x435, 0x44f, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x41a, 0x43e, 0x440, 0x435, 0x44f, 0x20, 0x497, 0x4d9, 0x439, 0x433,
0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x43e, 0x441, 0x440,
0x430, 0x44d, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x440, 0x430,
0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440,
0x441, 0x43a, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x41a, 0x44b, 0x440, 0x433, 0x44b, 0x437, 0x441, 0x442, 0x430,
0x43d, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41b, 0x430, 0x439, 0x43d,
0x20, 0x443, 0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x430,
0x443, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41b, 0x43e, 0x440, 0x434,
0x20, 0x425, 0x430, 0x443, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41b, 0x43e, 0x440, 0x434,
0x20, 0x425, 0x430, 0x443, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x433, 0x430, 0x434,
0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430,
0x43d, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x41c, 0x430, 0x43b, 0x430, 0x439, 0x437, 0x438, 0x44f, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x43b, 0x44c, 0x434, 0x438, 0x432,
0x20, 0x443, 0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x440, 0x43a, 0x438, 0x437, 0x20,
0x443, 0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x440, 0x448, 0x430, 0x43b, 0x43b, 0x20,
0x443, 0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438, 0x439,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x432, 0x440, 0x438,
0x43a, 0x438, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x432, 0x440, 0x438,
0x43a, 0x438, 0x439, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x43e, 0x443, 0x441, 0x43e, 0x43d, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430,
0x20, 0x422, 0x44b, 0x43d, 0x44b, 0x447, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x435, 0x43a, 0x441, 0x438,
0x43a, 0x430, 0x20, 0x422, 0x44b, 0x43d, 0x44b, 0x447, 0x20, 0x43e, 0x43a, 0x435,
0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430,
0x20, 0x422, 0x44b, 0x43d, 0x44b, 0x447, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d,
0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411,
0x430, 0x442, 0x43e, 0x440, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x423, 0x43b, 0x430, 0x43d,
0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x4d9, 0x441, 0x43a, 0x4d9,
0x4af, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x4d9, 0x441, 0x43a,
0x4d9, 0x4af, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x4d9, 0x441, 0x43a, 0x4d9, 0x4af,
0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x41c, 0x44c, 0x44f, 0x43d, 0x43c, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x41d, 0x430, 0x443, 0x440, 0x443, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x41d, 0x435, 0x43f, 0x430, 0x43b, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x42f, 0x4a3, 0x430, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e,
0x43d, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x42f, 0x4a3,
0x430, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x44f, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x42f, 0x4a3, 0x430, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434,
0x43e, 0x43d, 0x438, 0x44f, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x42f, 0x4a3, 0x430, 0x20, 0x417, 0x435, 0x43b,
0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x42f, 0x4a3, 0x430, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x42f, 0x4a3, 0x430, 0x20, 0x417, 0x435, 0x43b, 0x430,
0x43d, 0x434, 0x438, 0x44f, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x443, 0x43d,
0x434, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434,
0x43b, 0x435, 0x43d, 0x434, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41d, 0x438, 0x443, 0x44d, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20,
0x443, 0x442, 0x440, 0x430, 0x432, 0x44b, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x443, 0x442, 0x440,
0x430, 0x432, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41d, 0x43e, 0x440, 0x444, 0x43e,
0x43b, 0x43a, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x44b, 0x20, 0x497, 0x4d9,
0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x424, 0x435,
0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x438, 0x2d, 0x41d, 0x43e,
0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x438, 0x2d,
0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x424,
0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x438, 0x2d, 0x41d,
0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41d, 0x43e, 0x432, 0x43e, 0x441,
0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438,
0x440, 0x441, 0x43a, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x41e, 0x43c, 0x441, 0x43a, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x41e, 0x43c, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41e,
0x43c, 0x441, 0x43a, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x43a, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x43a, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x43b, 0x430, 0x443, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x43f, 0x443, 0x430, 0x20, 0x42f, 0x4a3,
0x430, 0x20, 0x413, 0x432, 0x438, 0x43d, 0x435, 0x44f, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432,
0x430, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432,
0x430, 0x439, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x41f, 0x435, 0x440, 0x443, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x41f, 0x435, 0x440, 0x443, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41f, 0x435,
0x440, 0x443, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f,
0x438, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f,
0x438, 0x43d, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x424, 0x435, 0x43d, 0x438, 0x43a, 0x441, 0x20, 0x443, 0x442,
0x440, 0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20, 0x4bb,
0x4d9, 0x43c, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x44c, 0x435,
0x440, 0x20, 0x4bb, 0x4d9, 0x43c, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e,
0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x44c, 0x435,
0x440, 0x20, 0x4bb, 0x4d9, 0x43c, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e,
0x43d, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x41f, 0x438, 0x442, 0x43a, 0x44d, 0x440, 0x43d, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x41f, 0x43e, 0x43d, 0x430, 0x43f, 0x435, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41f, 0x445, 0x435, 0x43d, 0x44c, 0x44f, 0x43d,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x420, 0x435, 0x44e, 0x43d, 0x44c,
0x43e, 0x43d, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x420, 0x43e, 0x442,
0x435, 0x440, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x421, 0x430,
0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x421,
0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x421, 0x430, 0x43c, 0x430, 0x440,
0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x421, 0x430, 0x43c, 0x430,
0x440, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x430,
0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x421, 0x430,
0x43c, 0x43e, 0x430, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x421, 0x435, 0x439, 0x448, 0x435, 0x43b, 0x20, 0x443,
0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x421, 0x438, 0x43d, 0x433, 0x430, 0x43f, 0x443, 0x440, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x421, 0x43e, 0x43b, 0x43e, 0x43c, 0x43e, 0x43d, 0x20, 0x443,
0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x44c, 0x44f, 0x43a, 0x20, 0x414, 0x436,
0x43e, 0x440, 0x434, 0x436, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x421, 0x443, 0x440, 0x438, 0x43d, 0x430, 0x43c, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x421, 0x451, 0x432, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x422, 0x430, 0x438, 0x442, 0x438, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x422, 0x430, 0x439, 0x43f, 0x435, 0x439, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x422, 0x430, 0x439, 0x43f, 0x435, 0x439, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x422, 0x430, 0x439, 0x43f, 0x435, 0x439, 0x20, 0x497, 0x4d9, 0x439, 0x433,
0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x430, 0x497, 0x438,
0x43a, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x422, 0x43e, 0x43a, 0x435, 0x43b, 0x430, 0x443, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422,
0x43e, 0x43d, 0x433, 0x430, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x427, 0x443, 0x443, 0x43a, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x4e9, 0x440, 0x43a, 0x43c, 0x4d9, 0x43d, 0x441,
0x442, 0x430, 0x43d, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x4e9,
0x440, 0x43a, 0x43c, 0x4d9, 0x43d, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x422, 0x4e9, 0x440, 0x43a, 0x43c, 0x4d9, 0x43d, 0x441, 0x442, 0x430, 0x43d,
0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x422, 0x443, 0x432, 0x430, 0x43b, 0x443, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x20, 0x497,
0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x4ae,
0x437, 0x431, 0x4d9, 0x43a, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x4ae, 0x437, 0x431, 0x4d9, 0x43a, 0x441, 0x442, 0x430, 0x43d,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x4ae, 0x437, 0x431, 0x4d9, 0x43a, 0x441, 0x442, 0x430,
0x43d, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x497,
0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x412,
0x435, 0x43d, 0x435, 0x441, 0x443, 0x44d, 0x43b, 0x430, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442,
0x43e, 0x43a, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x412, 0x43b, 0x430,
0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20,
0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430,
0x434, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440,
0x430, 0x434, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x423, 0x44d, 0x439, 0x43a, 0x20, 0x443, 0x442, 0x440,
0x430, 0x432, 0x44b, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x423, 0x43e,
0x43b, 0x43b, 0x438, 0x441, 0x20, 0x4bb, 0x4d9, 0x43c, 0x20, 0x424, 0x443, 0x442,
0x443, 0x43d, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x42f, 0x43a,
0x443, 0x442, 0x441, 0x43a, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x42f,
0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x42f, 0x43a, 0x443,
0x442, 0x441, 0x43a, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d,
0x431, 0x443, 0x440, 0x433, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x415,
0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431,
0x443, 0x440, 0x433, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x42e, 0x43a, 0x43e, 0x43d, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0xc0f, 0xc15, 0xc30, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc0f, 0xc15, 0xc30, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23,
0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc0f, 0xc15, 0xc30, 0xc4d, 0x20,
0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc06, 0xc2b,
0xc4d, 0xc18, 0xc28, 0xc3f, 0xc38, 0xc4d, 0xc24, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc38, 0xc46, 0xc02, 0xc1f, 0xc4d, 0xc30, 0xc32, 0xc4d, 0x20,
0xc06, 0xc2b, 0xc4d, 0xc30, 0xc3f, 0xc15, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc06, 0xc2b, 0xc4d, 0xc30, 0xc3f,
0xc15, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc26, 0xc15, 0xc4d, 0xc37, 0xc3f,
0xc23, 0x20, 0xc06, 0xc2b, 0xc4d, 0xc30, 0xc3f, 0xc15, 0xc3e, 0x20, 0xc2a, 0xc4d,
0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc06, 0xc2b, 0xc4d, 0xc30, 0xc3f,
0xc15, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f,
0xc2e, 0x20, 0xc06, 0xc2b, 0xc4d, 0xc30, 0xc3f, 0xc15, 0xc3e, 0x20, 0xc2a, 0xc4d,
0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc06, 0xc2b, 0xc4d, 0xc30, 0xc3f,
0xc15, 0xc3e, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc05, 0xc32, 0xc3e, 0xc38, 0xc4d, 0xc15, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc05, 0xc32, 0xc3e, 0xc38, 0xc4d, 0xc15, 0xc3e, 0x20, 0xc2a, 0xc4d, 0xc30,
0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05,
0xc32, 0xc3e, 0xc38, 0xc4d, 0xc15, 0xc3e, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20,
0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc05, 0xc32, 0xc4d, 0xc2e, 0xc3e, 0xc1f, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc05, 0xc32, 0xc4d, 0xc2e, 0xc3e, 0xc1f, 0xc3f, 0x20, 0xc2a, 0xc4d, 0xc30,
0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05,
0xc32, 0xc4d, 0xc2e, 0xc3e, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc2e, 0xc46, 0xc1c, 0xc3e, 0xc28, 0xc4d,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc2e, 0xc46, 0xc1c, 0xc3e, 0xc28, 0xc4d,
0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc05, 0xc2e, 0xc46, 0xc1c, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc35,
0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc27, 0xc4d,
0xc2f, 0xc2e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc27, 0xc4d, 0xc2f, 0xc2e,
0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc27, 0xc4d, 0xc2f, 0xc2e, 0x20, 0xc2a, 0xc17, 0xc1f,
0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc2a, 0xc4d, 0xc30,
0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24,
0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35,
0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc2e, 0xc4c, 0xc02, 0xc1f, 0xc46, 0xc2f, 0xc3f, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc2e, 0xc4c, 0xc02, 0xc1f, 0xc46, 0xc2f, 0xc3f, 0xc28, 0xc4d, 0x20,
0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc2e, 0xc4c, 0xc02, 0xc1f, 0xc46, 0xc2f, 0xc3f, 0xc28, 0xc4d, 0x20,
0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30,
0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc38, 0xc3f, 0xc2b, 0xc3f, 0xc15,
0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc38, 0xc3f, 0xc2b, 0xc3f, 0xc15,
0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc38, 0xc3f, 0xc2b, 0xc3f, 0xc15, 0xc4d, 0x20,
0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30,
0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc28, 0xc21, 0xc48, 0xc30, 0xc4d,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc28, 0xc3e, 0xc28, 0xc4d, 0xc21, 0xc4d,
0xc30, 0xc3f, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc28, 0xc3e, 0xc28, 0xc4d, 0xc21, 0xc4d,
0xc30, 0xc3f, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc0f, 0xc2a, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc0f,
0xc2a, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23,
0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc0f, 0xc2a, 0xc3f, 0xc2f, 0xc3e,
0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc15,
0xc4d, 0xc35, 0xc3e, 0xc1f, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc15,
0xc4d, 0xc35, 0xc3e, 0xc1f, 0xc41, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e,
0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc15, 0xc4d, 0xc35,
0xc3e, 0xc1f, 0xc41, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc05, 0xc15, 0xc4d, 0xc1f, 0xc4b, 0xc2c, 0xc46, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc05, 0xc15, 0xc4d, 0xc1f, 0xc4b, 0xc2c, 0xc46, 0x20, 0xc2a, 0xc4d,
0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc05, 0xc15, 0xc4d, 0xc1f, 0xc4b, 0xc2c, 0xc46, 0x20, 0xc35, 0xc47, 0xc38, 0xc35,
0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc30, 0xc47, 0xc2c, 0xc3f, 0xc2f,
0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc30, 0xc47, 0xc2c, 0xc3f,
0xc2f, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f,
0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc30, 0xc47, 0xc2c, 0xc3f, 0xc2f,
0xc28, 0xc4d, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41,
0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc30, 0xc4d,
0xc1c, 0xc46, 0xc02, 0xc1f, 0xc40, 0xc28, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc05, 0xc30, 0xc4d, 0xc1c, 0xc46, 0xc02, 0xc1f, 0xc40, 0xc28, 0xc3e, 0x20, 0xc2a,
0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc06, 0xc30, 0xc4d, 0xc1c, 0xc46, 0xc02, 0xc1f, 0xc40, 0xc28, 0xc3e, 0x20,
0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc36,
0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc05, 0xc30, 0xc4d, 0xc1c, 0xc46, 0xc02, 0xc1f,
0xc40, 0xc28, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc36, 0xc4d, 0xc1a,
0xc3f, 0xc2e, 0x20, 0xc05, 0xc30, 0xc4d, 0xc1c, 0xc46, 0xc02, 0xc1f, 0xc40, 0xc28,
0xc3e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc05,
0xc30, 0xc4d, 0xc1c, 0xc46, 0xc02, 0xc1f, 0xc40, 0xc28, 0xc3e, 0x20, 0xc35, 0xc47,
0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc06, 0xc30, 0xc4d, 0xc2e,
0xc47, 0xc28, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc06, 0xc30,
0xc4d, 0xc2e, 0xc47, 0xc28, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e,
0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc06, 0xc30,
0xc4d, 0xc2e, 0xc47, 0xc28, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc35, 0xc47, 0xc38, 0xc35,
0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc1f, 0xc4d, 0xc32, 0xc3e, 0xc02,
0xc1f, 0xc3f, 0xc15, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc1f, 0xc4d,
0xc32, 0xc3e, 0xc02, 0xc1f, 0xc3f, 0xc15, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e,
0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc1f,
0xc4d, 0xc32, 0xc3e, 0xc02, 0xc1f, 0xc3f, 0xc15, 0xc4d, 0x20, 0xc2a, 0xc17, 0xc1f,
0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc06, 0xc38, 0xc4d, 0xc1f, 0xc4d, 0xc30, 0xc47, 0xc32, 0xc3f,
0xc2f, 0xc3e, 0x20, 0xc2e, 0xc27, 0xc4d, 0xc2f, 0xc2e, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc06, 0xc38, 0xc4d, 0xc1f, 0xc4d, 0xc30, 0xc47, 0xc32, 0xc3f, 0xc2f, 0xc3e,
0x20, 0xc2e, 0xc27, 0xc4d, 0xc2f, 0xc2e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e,
0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc06, 0xc38, 0xc4d,
0xc1f, 0xc4d, 0xc30, 0xc47, 0xc32, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc2e, 0xc27, 0xc4d,
0xc2f, 0xc2e, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41,
0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc06, 0xc38, 0xc4d,
0xc1f, 0xc4d, 0xc30, 0xc47, 0xc32, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc2e, 0xc27, 0xc4d,
0xc2f, 0xc2e, 0x20, 0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc2e, 0xc27, 0xc4d, 0xc2f, 0xc2e, 0x20, 0xc06, 0xc38, 0xc4d, 0xc1f,
0xc4d, 0xc30, 0xc47, 0xc32, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc36, 0xc4d,
0xc1a, 0xc3f, 0xc2e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f,
0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc06, 0xc38, 0xc4d, 0xc1f, 0xc4d, 0xc30,
0xc47, 0xc32, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc2e, 0xc27, 0xc4d, 0xc2f, 0xc2e, 0x20,
0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20,
0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc06, 0xc38, 0xc4d, 0xc1f,
0xc4d, 0xc30, 0xc47, 0xc32, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc06, 0xc38, 0xc4d, 0xc1f, 0xc4d, 0xc30, 0xc47, 0xc32, 0xc3f, 0xc2f, 0xc28, 0xc4d,
0x20, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e,
0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc06, 0xc38,
0xc4d, 0xc1f, 0xc4d, 0xc30, 0xc47, 0xc32, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20, 0xc24,
0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35,
0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc06, 0xc38, 0xc4d, 0xc1f, 0xc4d,
0xc30, 0xc47, 0xc32, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc06,
0xc38, 0xc4d, 0xc1f, 0xc4d, 0xc30, 0xc47, 0xc32, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20,
0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20,
0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc05, 0xc1c, 0xc30, 0xc4d, 0xc2c, 0xc48, 0xc1c, 0xc3e, 0xc28, 0xc4d, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc1c, 0xc30, 0xc4d, 0xc2c, 0xc48, 0xc1c, 0xc3e,
0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc1c, 0xc30, 0xc4d, 0xc2c, 0xc48, 0xc1c,
0xc3e, 0xc28, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc05, 0xc1c, 0xc4b, 0xc30, 0xc4d, 0xc38, 0xc4d, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc05, 0xc1c, 0xc4b, 0xc30, 0xc4d, 0xc38, 0xc4d, 0x20, 0xc2a, 0xc4d,
0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc05, 0xc1c, 0xc4b, 0xc30, 0xc4d, 0xc38, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35,
0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2c, 0xc02, 0xc17, 0xc4d, 0xc32, 0xc3e,
0xc26, 0xc47, 0xc36, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2c, 0xc02, 0xc17,
0xc4d, 0xc32, 0xc3e, 0xc26, 0xc47, 0xc36, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e,
0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2c, 0xc02,
0xc17, 0xc4d, 0xc32, 0xc3e, 0xc26, 0xc47, 0xc36, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38,
0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2d, 0xc42, 0xc1f, 0xc3e, 0xc28,
0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2c, 0xc4a, 0xc32, 0xc40, 0xc35, 0xc3f,
0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2c, 0xc4d, 0xc30, 0xc46, 0xc1c,
0xc3f, 0xc32, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2c, 0xc4d,
0xc30, 0xc46, 0xc1c, 0xc3f, 0xc32, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc2a, 0xc4d, 0xc30,
0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2c,
0xc4d, 0xc30, 0xc46, 0xc1c, 0xc3f, 0xc32, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc35, 0xc47,
0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2c, 0xc4d, 0xc30, 0xc42,
0xc28, 0xc47, 0x20, 0xc26, 0xc30, 0xc41, 0xc38, 0xc32, 0xc3e, 0xc2e, 0xc4d, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc47, 0xc2a, 0xc4d, 0x20, 0xc35, 0xc46, 0xc30,
0xc4d, 0xc21, 0xc46, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc47, 0xc2a, 0xc4d,
0x20, 0xc35, 0xc46, 0xc30, 0xc4d, 0xc21, 0xc46, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e,
0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc47,
0xc2a, 0xc4d, 0x20, 0xc35, 0xc46, 0xc30, 0xc4d, 0xc21, 0xc46, 0x20, 0xc35, 0xc47,
0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1a, 0xc3e, 0xc2e, 0xc30,
0xc4d, 0xc30, 0xc4b, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f,
0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1a, 0xc3e, 0xc25, 0xc2e, 0xc4d, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1a, 0xc3e, 0xc25, 0xc2e, 0xc4d, 0x20, 0xc2a, 0xc4d,
0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc1a, 0xc3e, 0xc25, 0xc2e, 0xc4d, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35,
0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc1a, 0xc3f, 0xc32, 0xc40, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1a, 0xc3f, 0xc32,
0xc40, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1a, 0xc3f, 0xc32, 0xc40, 0x20, 0xc35, 0xc47, 0xc38,
0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1a, 0xc48, 0xc28, 0xc3e, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1a, 0xc48, 0xc28, 0xc3e, 0x20, 0xc2a, 0xc4d, 0xc30,
0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1a,
0xc48, 0xc28, 0xc3e, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32,
0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4d,
0xc30, 0xc3f, 0xc38, 0xc4d, 0xc2e, 0xc38, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc3f,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4b, 0xc15, 0xc4b, 0xc38, 0xc4d, 0x20,
0xc26, 0xc40, 0xc35, 0xc41, 0xc32, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4a,
0xc32, 0xc02, 0xc2c, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15,
0xc4a, 0xc32, 0xc02, 0xc2c, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e,
0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4a,
0xc32, 0xc02, 0xc2c, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc41, 0xc15, 0xc4d, 0x20, 0xc26, 0xc40,
0xc35, 0xc41, 0xc32, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc41, 0xc15, 0xc4d,
0x20, 0xc26, 0xc40, 0xc35, 0xc41, 0xc32, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e,
0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc41, 0xc15,
0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc41, 0xc32, 0x20, 0xc05, 0xc30, 0xc4d, 0xc27,
0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15,
0xc4d, 0xc2f, 0xc42, 0xc2c, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4d,
0xc2f, 0xc42, 0xc2c, 0xc3e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23,
0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4d, 0xc2f, 0xc42, 0xc2c,
0xc3e, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24,
0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc21, 0xc47, 0xc35, 0xc3f,
0xc38, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc21, 0xc4d, 0xc2f, 0xc42, 0xc2e,
0xc3e, 0xc02, 0xc1f, 0xc4d, 0x2d, 0xc21, 0xc3f, 0x2019, 0xc09, 0xc30, 0xc4d, 0xc35,
0xc3f, 0xc32, 0xc4d, 0xc32, 0xc47, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc42,
0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc24, 0xc48, 0xc2e, 0xc42, 0xc30, 0xc4d, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc08, 0xc38, 0xc4d, 0xc1f, 0xc30, 0xc4d, 0x20, 0xc26,
0xc40, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc08, 0xc38, 0xc4d, 0xc1f,
0xc30, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc3f, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e,
0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc08, 0xc38,
0xc4d, 0xc1f, 0xc30, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc3f, 0x20, 0xc35, 0xc47,
0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc08, 0xc15, 0xc4d, 0xc35,
0xc21, 0xc3e, 0xc30, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc46, 0xc02,
0xc1f, 0xc4d, 0xc30, 0xc32, 0xc4d, 0x20, 0xc2f, 0xc42, 0xc30, 0xc4b, 0xc2a, 0xc3f,
0xc2f, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc46, 0xc02, 0xc1f,
0xc4d, 0xc30, 0xc32, 0xc4d, 0x20, 0xc2f, 0xc42, 0xc30, 0xc4b, 0xc2a, 0xc3f, 0xc2f,
0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc46, 0xc02, 0xc1f, 0xc4d, 0xc30, 0xc32,
0xc4d, 0x20, 0xc2f, 0xc42, 0xc30, 0xc4b, 0xc2a, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20,
0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc42,
0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc2f, 0xc42, 0xc30, 0xc4b, 0xc2a, 0xc3f, 0xc2f,
0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a,
0xc41, 0x20, 0xc2f, 0xc42, 0xc30, 0xc4b, 0xc2a, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20,
0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc2f, 0xc42, 0xc30,
0xc4b, 0xc2a, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc41, 0xc26, 0xc42, 0xc30, 0x2d, 0xc24,
0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc2f, 0xc42, 0xc30, 0xc4b, 0xc2a, 0xc3f,
0xc2f, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc36, 0xc4d, 0xc1a,
0xc3f, 0xc2e, 0x20, 0xc2f, 0xc42, 0xc30, 0xc4b, 0xc2a, 0xc3f, 0xc2f, 0xc28, 0xc4d,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20,
0xc2f, 0xc42, 0xc30, 0xc4b, 0xc2a, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d,
0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc2f, 0xc42, 0xc30, 0xc4b, 0xc2a,
0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc3e, 0xc15, 0xc4d, 0x200c, 0xc32, 0xc4d, 0xc2f, 0xc3e,
0xc02, 0xc21, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc41, 0xc32, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc2b, 0xc3e, 0xc15, 0xc4d, 0x200c, 0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc02,
0xc21, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc41, 0xc32, 0x20, 0xc2a, 0xc4d, 0xc30,
0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b,
0xc3e, 0xc15, 0xc4d, 0x200c, 0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc21, 0xc4d, 0x20,
0xc26, 0xc40, 0xc35, 0xc41, 0xc32, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc3f, 0xc1c, 0xc40, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc2b, 0xc3f, 0xc1c, 0xc40, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e,
0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc3f, 0xc1c, 0xc40,
0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b,
0xc4d, 0xc30, 0xc46, 0xc02, 0xc1a, 0xc4d, 0x20, 0xc17, 0xc2f, 0xc3e, 0xc28, 0xc3e,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc4d, 0xc30, 0xc46, 0xc02, 0xc1a, 0xc4d,
0x20, 0xc26, 0xc15, 0xc4d, 0xc37, 0xc3f, 0xc23, 0x20, 0xc2e, 0xc30, 0xc3f, 0xc2f,
0xc41, 0x20, 0xc05, 0xc02, 0xc1f, 0xc3e, 0xc30, 0xc4d, 0xc15, 0xc3f, 0xc1f, 0xc3f,
0xc15, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc17, 0xc3e, 0xc32, 0xc3e, 0xc2a,
0xc3e, 0xc17, 0xc4b, 0xc38, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc17, 0xc3e,
0xc02, 0xc2c, 0xc3f, 0xc2f, 0xc30, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1c,
0xc3e, 0xc30, 0xc4d, 0xc1c, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc1c, 0xc3e, 0xc30, 0xc4d, 0xc1c, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc2a, 0xc4d, 0xc30,
0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1c,
0xc3e, 0xc30, 0xc4d, 0xc1c, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc35, 0xc47, 0xc38, 0xc35,
0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc17, 0xc3f, 0xc32, 0xc4d, 0xc2c, 0xc30,
0xc4d, 0xc1f, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc41, 0xc32, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc17, 0xc4d, 0xc30, 0xc40, 0xc28, 0xc4d, 0x200c, 0xc35, 0xc3f, 0xc1a,
0xc4d, 0x20, 0xc38, 0xc17, 0xc1f, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24,
0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc17, 0xc4d, 0xc30, 0xc40, 0xc28, 0xc4d,
0x200c, 0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc21, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc17, 0xc4d, 0xc30, 0xc40,
0xc28, 0xc4d, 0x200c, 0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc21, 0xc4d, 0x20, 0xc2a,
0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc17, 0xc4d, 0xc30, 0xc40,
0xc28, 0xc4d, 0x200c, 0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc21, 0xc4d, 0x20, 0xc35,
0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc36, 0xc4d,
0xc1a, 0xc3f, 0xc2e, 0x20, 0xc17, 0xc4d, 0xc30, 0xc40, 0xc28, 0xc4d, 0x200c, 0xc32,
0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc21, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a,
0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc17, 0xc4d, 0xc30, 0xc40, 0xc28, 0xc4d,
0x200c, 0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc21, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30,
0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a,
0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc17, 0xc4d, 0xc30, 0xc40, 0xc28, 0xc4d,
0x200c, 0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc21, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38,
0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc17, 0xc4d, 0xc35, 0xc3e, 0xc2e,
0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc17, 0xc32, 0xc4d, 0xc2b, 0xc4d, 0x20, 0xc2a, 0xc4d,
0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc39, 0xc35, 0xc3e, 0xc2f, 0xc4d, 0x2d, 0xc05, 0xc32, 0xc4d, 0xc2f, 0xc42, 0xc37,
0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc39, 0xc35, 0xc3e,
0xc2f, 0xc4d, 0x2d, 0xc05, 0xc32, 0xc4d, 0xc2f, 0xc42, 0xc37, 0xc3f, 0xc2f, 0xc28,
0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc39, 0xc35, 0xc3e, 0xc2f, 0xc4d, 0x2d, 0xc05, 0xc32,
0xc4d, 0xc2f, 0xc42, 0xc37, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc17, 0xc1f,
0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc39, 0xc3e, 0xc02, 0xc15, 0xc3e, 0xc02, 0xc17, 0xc4d, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc39, 0xc3e, 0xc02, 0xc15, 0xc3e, 0xc02, 0xc17, 0xc4d,
0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc39, 0xc3e, 0xc02, 0xc15, 0xc3e, 0xc02, 0xc17, 0xc4d, 0x20,
0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc39, 0xc4b,
0xc35, 0xc4d, 0xc21, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc39, 0xc4b, 0xc35,
0xc4d, 0xc21, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f,
0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc39, 0xc4b, 0xc35, 0xc4d, 0xc21, 0xc4d,
0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2d,
0xc3e, 0xc30, 0xc24, 0xc26, 0xc47, 0xc36, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e,
0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc39, 0xc3f, 0xc02,
0xc26, 0xc42, 0x20, 0xc2e, 0xc39, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc41, 0xc26, 0xc4d,
0xc30, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc07, 0xc02, 0xc21, 0xc4b, 0xc1a, 0xc48,
0xc28, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc46, 0xc02, 0xc1f, 0xc4d,
0xc30, 0xc32, 0xc4d, 0x20, 0xc07, 0xc02, 0xc21, 0xc4b, 0xc28, 0xc47, 0xc37, 0xc3f,
0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a,
0xc41, 0x20, 0xc07, 0xc02, 0xc21, 0xc4b, 0xc28, 0xc47, 0xc37, 0xc3f, 0xc2f, 0xc3e,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20,
0xc07, 0xc02, 0xc21, 0xc4b, 0xc28, 0xc47, 0xc37, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc07, 0xc30, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc07, 0xc30, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e,
0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc07, 0xc30, 0xc3e,
0xc28, 0xc4d, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41,
0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc07, 0xc30, 0xc4d,
0xc15, 0xc41, 0xc1f, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc07, 0xc30, 0xc4d, 0xc15, 0xc41, 0xc1f, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d,
0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc07, 0xc30, 0xc4d, 0xc15, 0xc41, 0xc1f, 0xc4d, 0xc38, 0xc4d,
0xc15, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc07, 0xc1c, 0xc4d, 0xc30, 0xc3e, 0xc2f, 0xc3f, 0xc32, 0xc4d, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc07, 0xc1c, 0xc4d, 0xc30, 0xc3e, 0xc2f, 0xc3f, 0xc32, 0xc4d,
0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc07, 0xc1c, 0xc4d, 0xc30, 0xc3e, 0xc2f, 0xc3f, 0xc32, 0xc4d,
0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41,
0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1c, 0xc2a, 0xc3e, 0xc28, 0xc4d,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1c, 0xc2a, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc2a,
0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc1c, 0xc2a, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20,
0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc2a, 0xc46, 0xc1f, 0xc4d, 0xc30, 0xc4b, 0xc2a, 0xc3e, 0xc35, 0xc4d, 0xc32,
0xc4b, 0xc35, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0x2d, 0xc15, 0xc3e, 0xc2e, 0xc4d,
0xc1b, 0xc3e, 0xc1f, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc2a, 0xc46, 0xc1f, 0xc4d, 0xc30, 0xc4b, 0xc2a, 0xc3e, 0xc35, 0xc4d, 0xc32,
0xc4b, 0xc35, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0x2d, 0xc15, 0xc3e, 0xc2e, 0xc4d,
0xc1b, 0xc3e, 0xc1f, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc3f, 0x20, 0xc2a, 0xc4d, 0xc30,
0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a,
0xc46, 0xc1f, 0xc4d, 0xc30, 0xc4b, 0xc2a, 0xc3e, 0xc35, 0xc4d, 0xc32, 0xc4b, 0xc35,
0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0x2d, 0xc15, 0xc3e, 0xc2e, 0xc4d, 0xc1b, 0xc3e,
0xc1f, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc3f, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc1c, 0xc15, 0xc3f, 0xc38, 0xc4d, 0xc24,
0xc3e, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc42, 0xc30, 0xc4d,
0xc2a, 0xc41, 0x20, 0xc15, 0xc1c, 0xc15, 0xc3f, 0x200c, 0xc38, 0xc4d, 0xc24, 0xc3e,
0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f,
0xc2e, 0x20, 0xc15, 0xc1c, 0xc15, 0xc3f, 0xc38, 0xc4d, 0xc24, 0xc3e, 0xc28, 0xc4d,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4a, 0xc30, 0xc3f, 0xc2f, 0xc28, 0xc4d,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4a, 0xc30, 0xc3f, 0xc2f, 0xc28, 0xc4d,
0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4a, 0xc30, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20, 0xc2a,
0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4b, 0xc38, 0xc4d, 0xc30, 0xc3e, 0xc2f,
0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4d, 0xc30, 0xc3e, 0xc38, 0xc4d,
0xc28, 0xc4b, 0xc2f, 0xc3e, 0xc30, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4d, 0xc30, 0xc3e, 0xc38, 0xc4d, 0xc28, 0xc4b, 0xc2f,
0xc3e, 0xc30, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e,
0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4d,
0xc30, 0xc3e, 0xc38, 0xc4d, 0xc28, 0xc4b, 0xc2f, 0xc3e, 0xc30, 0xc4d, 0xc38, 0xc4d,
0xc15, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc15, 0xc3f, 0xc30, 0xc4d, 0xc17, 0xc3f, 0xc38, 0xc4d, 0xc24, 0xc3e, 0xc28,
0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc32, 0xc02, 0xc15, 0xc3e, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc32, 0xc48, 0xc28, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc41,
0xc32, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc32, 0xc3e, 0xc30, 0xc4d, 0xc21, 0xc4d,
0x20, 0xc39, 0xc4b, 0xc35, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc32, 0xc3e,
0xc30, 0xc4d, 0xc21, 0xc4d, 0x20, 0xc39, 0xc4b, 0xc35, 0xc4d, 0x20, 0xc2a, 0xc4d,
0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc32, 0xc3e, 0xc30, 0xc4d, 0xc21, 0xc4d, 0x20, 0xc39, 0xc4b, 0xc35, 0xc4d, 0x20,
0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc15, 0xc3e,
0xc35, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc15, 0xc3e, 0xc35, 0xc4d,
0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc15, 0xc3e, 0xc35, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38,
0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc17, 0xc21, 0xc3e, 0xc28,
0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc17, 0xc21, 0xc3e, 0xc28, 0xc4d,
0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc17, 0xc21, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc35, 0xc47,
0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc32, 0xc47, 0xc37,
0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc3e, 0xc32, 0xc4d,
0xc26, 0xc40, 0xc35, 0xc41, 0xc32, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc3e,
0xc30, 0xc4d, 0xc15, 0xc4d, 0xc35, 0xc47, 0xc38, 0xc3e, 0xc38, 0xc4d, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc3e, 0xc30, 0xc4d, 0xc37, 0xc32, 0xc4d, 0x20, 0xc26,
0xc40, 0xc35, 0xc41, 0xc32, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc3e, 0xc30,
0xc3f, 0xc37, 0xc38, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc3e, 0xc30,
0xc3f, 0xc37, 0xc38, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23,
0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc3e, 0xc30, 0xc3f, 0xc37,
0xc38, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc2e, 0xc3e, 0xc38, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e,
0xc46, 0xc15, 0xc4d, 0xc38, 0xc3f, 0xc15, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc38, 0xc3f,
0xc2b, 0xc3f, 0xc15, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc46, 0xc15,
0xc4d, 0xc38, 0xc3f, 0xc15, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc38, 0xc3f, 0xc2b, 0xc3f,
0xc15, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc46, 0xc15, 0xc4d, 0xc38, 0xc3f, 0xc15,
0xc28, 0xc4d, 0x20, 0xc2a, 0xc38, 0xc3f, 0xc2b, 0xc3f, 0xc15, 0xc4d, 0x20, 0xc2a,
0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc09, 0xc32, 0xc28, 0xc4d, 0x20, 0xc2c, 0xc24,
0xc4b, 0xc30, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc09, 0xc32, 0xc28, 0xc4d,
0x20, 0xc2c, 0xc24, 0xc4b, 0xc30, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e,
0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc09, 0xc32, 0xc28,
0xc4d, 0x20, 0xc2c, 0xc24, 0xc4b, 0xc30, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35,
0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc3e, 0xc38, 0xc4d, 0xc15, 0xc4b,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc3e, 0xc38, 0xc4d, 0xc15, 0xc4b, 0x20,
0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc2e, 0xc3e, 0xc38, 0xc4d, 0xc15, 0xc4b, 0x20, 0xc35, 0xc47, 0xc38,
0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc2f, 0xc28, 0xc4d, 0xc2e,
0xc3e, 0xc30, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28, 0xc4c, 0xc30, 0xc41,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28, 0xc47, 0xc2a, 0xc3e, 0xc32, 0xc4d, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28, 0xc4d, 0xc2f, 0xc42, 0x20, 0xc15, 0xc3e, 0xc32,
0xc46, 0xc21, 0xc4b, 0xc28, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc28, 0xc4d, 0xc2f, 0xc42, 0x20, 0xc15, 0xc3e, 0xc32, 0xc46, 0xc21, 0xc4b, 0xc28,
0xc3f, 0xc2f, 0xc3e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f,
0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28, 0xc4d, 0xc2f, 0xc42, 0x20, 0xc15,
0xc3e, 0xc32, 0xc46, 0xc21, 0xc4b, 0xc28, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc35, 0xc47,
0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28, 0xc4d, 0xc2f, 0xc42,
0xc1c, 0xc3f, 0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc21, 0xc4d, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc28, 0xc4d, 0xc2f, 0xc42, 0xc1c, 0xc3f, 0xc32, 0xc4d, 0xc2f, 0xc3e,
0xc02, 0xc21, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f,
0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28, 0xc4d, 0xc2f, 0xc42, 0xc1c, 0xc3f,
0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc21, 0xc4d, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f,
0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc28, 0xc4d, 0xc2f, 0xc42, 0xc2b, 0xc4c, 0xc02, 0xc21, 0xc4d, 0x200c,
0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc21, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc28, 0xc4d, 0xc2f, 0xc42, 0xc2b, 0xc4c, 0xc02, 0xc21, 0xc4d, 0x200c, 0xc32, 0xc4d,
0xc2f, 0xc3e, 0xc02, 0xc21, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e,
0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28, 0xc4d, 0xc2f, 0xc42,
0xc2b, 0xc4c, 0xc02, 0xc21, 0xc4d, 0x200c, 0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc21,
0xc4d, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24,
0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28, 0xc3f, 0xc2f, 0xc42,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28, 0xc3e, 0xc30, 0xc4d, 0xc2b, 0xc4b, 0xc15,
0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28,
0xc3e, 0xc30, 0xc4d, 0xc2b, 0xc4b, 0xc15, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc3f,
0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc28, 0xc3e, 0xc30, 0xc4d, 0xc2b, 0xc4b, 0xc15, 0xc4d, 0x20,
0xc26, 0xc40, 0xc35, 0xc3f, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc2b, 0xc46, 0xc30, 0xc4d, 0xc28, 0xc3e, 0xc02, 0xc21, 0xc4b, 0x20,
0xc21, 0xc3f, 0x20, 0xc28, 0xc4a, 0xc30, 0xc4b, 0xc28, 0xc4d, 0xc39, 0xc3e, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc46, 0xc30, 0xc4d, 0xc28, 0xc3e, 0xc02, 0xc21,
0xc4b, 0x20, 0xc21, 0xc3f, 0x20, 0xc28, 0xc4a, 0xc30, 0xc4b, 0xc28, 0xc4d, 0xc39,
0xc3e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc46, 0xc30, 0xc4d, 0xc28, 0xc3e, 0xc02, 0xc21,
0xc4b, 0x20, 0xc21, 0xc3f, 0x20, 0xc28, 0xc4a, 0xc30, 0xc4b, 0xc28, 0xc4d, 0xc39,
0xc3e, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc09, 0xc24, 0xc4d, 0xc24, 0xc30, 0x20, 0xc2e, 0xc3e, 0xc30, 0xc3f, 0xc2f, 0xc3e,
0xc28, 0xc3e, 0x20, 0xc26, 0xc40, 0xc35, 0xc41, 0xc32, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc28, 0xc4b, 0xc35, 0xc4b, 0xc38, 0xc3f, 0xc2c, 0xc3f, 0xc30, 0xc4d, 0xc38,
0xc4d, 0xc15, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28, 0xc4b, 0xc35, 0xc4b,
0xc38, 0xc3f, 0xc2c, 0xc3f, 0xc30, 0xc4d, 0xc15, 0xc4d, 0xc38, 0xc4d, 0x20, 0xc2a,
0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc28, 0xc4b, 0xc35, 0xc4b, 0xc38, 0xc3f, 0xc2c, 0xc3f, 0xc30, 0xc4d, 0xc38,
0xc4d, 0xc15, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc13, 0xc2e, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc13, 0xc2e, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0x20, 0xc2a, 0xc4d,
0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc13, 0xc2e, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35,
0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc3e, 0xc15, 0xc3f, 0xc38, 0xc4d,
0xc24, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc3e, 0xc15,
0xc3f, 0xc38, 0xc4d, 0xc24, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e,
0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc3e,
0xc15, 0xc3f, 0xc38, 0xc4d, 0xc24, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38,
0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc3e, 0xc32, 0xc3e, 0xc35,
0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc3e, 0xc2a, 0xc41, 0xc35, 0xc3e,
0x20, 0xc28, 0xc4d, 0xc2f, 0xc42, 0x20, 0xc17, 0xc3f, 0xc28, 0xc3f, 0xc2f, 0xc3e,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc30, 0xc3e, 0xc17, 0xc4d, 0xc35, 0xc47,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc30, 0xc3e, 0xc17, 0xc4d, 0xc35, 0xc47,
0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc30, 0xc3e, 0xc17, 0xc4d, 0xc35, 0xc47, 0x20, 0xc35,
0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc46, 0xc30,
0xc42, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc46, 0xc30, 0xc42, 0x20, 0xc2a,
0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc2a, 0xc46, 0xc30, 0xc42, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc3f, 0xc32, 0xc3f, 0xc2a, 0xc4d, 0xc2a, 0xc48,
0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc3f, 0xc32, 0xc3f, 0xc2a,
0xc4d, 0xc2a, 0xc48, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e,
0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc3f, 0xc32, 0xc3f,
0xc2a, 0xc4d, 0xc2a, 0xc48, 0xc28, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc3f, 0xc28, 0xc3f, 0xc15, 0xc4d, 0xc38,
0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc41, 0xc32, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc38, 0xc46, 0xc2f, 0xc3f, 0xc02, 0xc1f, 0xc4d, 0x20, 0xc2a, 0xc3f, 0xc2f, 0xc46,
0xc30, 0xc4d, 0x20, 0xc2e, 0xc30, 0xc3f, 0xc2f, 0xc41, 0x20, 0xc2e, 0xc3f, 0xc15,
0xc4d, 0xc35, 0xc46, 0xc32, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc38, 0xc46, 0xc2f, 0xc3f, 0xc02, 0xc1f, 0xc4d, 0x20, 0xc2a, 0xc3f, 0xc2f, 0xc46,
0xc30, 0xc4d, 0x20, 0xc2e, 0xc30, 0xc3f, 0xc2f, 0xc41, 0x20, 0xc2e, 0xc3f, 0xc15,
0xc4d, 0xc35, 0xc46, 0xc32, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e,
0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc46,
0xc2f, 0xc3f, 0xc02, 0xc1f, 0xc4d, 0x20, 0xc2a, 0xc3f, 0xc2f, 0xc30, 0xc4d, 0x20,
0xc2e, 0xc30, 0xc3f, 0xc2f, 0xc41, 0x20, 0xc2e, 0xc3f, 0xc15, 0xc4d, 0xc35, 0xc46,
0xc32, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46,
0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a,
0xc3f, 0xc1f, 0xc4d, 0x200c, 0xc15, 0xc48, 0xc30, 0xc4d, 0xc28, 0xc4d, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc4a, 0xc28, 0xc47, 0xc2a, 0xc4d, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc2a, 0xc4d, 0xc2f, 0xc4b, 0xc02, 0xc17, 0xc3e, 0xc02, 0xc17, 0xc4d,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc3f, 0xc1c, 0xc3f, 0xc32, 0xc4b, 0xc30,
0xc4d, 0xc21, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc3f, 0xc1c, 0xc3f,
0xc32, 0xc4b, 0xc30, 0xc4d, 0xc21, 0xc3e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e,
0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc3f, 0xc1c,
0xc3f, 0xc32, 0xc4b, 0xc30, 0xc4d, 0xc21, 0xc3e, 0x20, 0xc35, 0xc47, 0xc38, 0xc35,
0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc30, 0xc40, 0xc2f, 0xc42, 0xc28, 0xc3f,
0xc2f, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc30, 0xc4a, 0xc24, 0xc47,
0xc30, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc16, 0xc3e, 0xc32, 0xc3f,
0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc16, 0xc3e, 0xc32, 0xc3f,
0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc16, 0xc3e, 0xc32, 0xc3f, 0xc28, 0xc4d,
0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38,
0xc2e, 0xc3e, 0xc30, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc2e, 0xc3e,
0xc30, 0xc3e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc2e, 0xc3e, 0xc30, 0xc3e, 0x20, 0xc35,
0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc2e, 0xc4b,
0xc35, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc2e, 0xc4b, 0xc35, 0xc3e,
0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc38, 0xc2e, 0xc4b, 0xc35, 0xc3e, 0x20, 0xc2a, 0xc17, 0xc1f,
0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc38, 0xc40, 0xc37, 0xc46, 0xc32, 0xc4d, 0xc38, 0xc4d, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc3f, 0xc02, 0xc17, 0xc2a, 0xc42, 0xc30, 0xc4d,
0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc38, 0xc4b, 0xc32, 0xc2e, 0xc28, 0xc4d, 0x20, 0xc26, 0xc40,
0xc35, 0xc41, 0xc32, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc26, 0xc15, 0xc4d, 0xc37,
0xc3f, 0xc23, 0x20, 0xc1c, 0xc3e, 0xc30, 0xc4d, 0xc1c, 0xc3f, 0xc2f, 0xc3e, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc42, 0xc30, 0xc3f, 0xc28, 0xc3e, 0xc2e, 0xc4d,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc4d, 0xc2f, 0xc4b, 0xc35, 0xc3e, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc39, 0xc3f, 0xc24, 0xc3f, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc24, 0xc48, 0xc2a, 0xc40, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24,
0xc48, 0xc2a, 0xc40, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f,
0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc48, 0xc2a, 0xc40, 0x20, 0xc2a,
0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc1c, 0xc3f, 0xc15, 0xc3f, 0xc38, 0xc4d,
0xc24, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1f, 0xc4b, 0xc15,
0xc46, 0xc32, 0xc3e, 0xc35, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1f, 0xc3e,
0xc02, 0xc17, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1f, 0xc3e, 0xc02, 0xc17,
0xc3e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1f, 0xc3e, 0xc02, 0xc17, 0xc3e, 0x20, 0xc35, 0xc47,
0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1a, 0xc15, 0xc4d, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc41, 0xc30, 0xc4d, 0xc15, 0xc4d, 0x200c, 0xc2e,
0xc46, 0xc28, 0xc3f, 0xc38, 0xc4d, 0xc24, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc24, 0xc41, 0xc30, 0xc4d, 0xc15, 0xc4d, 0x200c, 0xc2e, 0xc46, 0xc28,
0xc3f, 0xc38, 0xc4d, 0xc24, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e,
0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc41,
0xc30, 0xc4d, 0xc15, 0xc4d, 0x200c, 0xc2e, 0xc46, 0xc28, 0xc3f, 0xc38, 0xc4d, 0xc24,
0xc3e, 0xc28, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc24, 0xc41, 0xc35, 0xc3e, 0xc32, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc09, 0xc30, 0xc41, 0xc17, 0xc4d, 0xc35, 0xc47, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc09, 0xc30, 0xc41, 0xc17, 0xc4d, 0xc35, 0xc47, 0x20, 0xc2a, 0xc4d, 0xc30,
0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc09,
0xc30, 0xc41, 0xc17, 0xc4d, 0xc35, 0xc47, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc09, 0xc1c, 0xc4d, 0xc2c, 0xc46, 0xc15, 0xc3f,
0xc38, 0xc4d, 0xc24, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc09,
0xc1c, 0xc4d, 0xc2c, 0xc46, 0xc15, 0xc3f, 0xc38, 0xc4d, 0xc24, 0xc3e, 0xc28, 0xc4d,
0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc09, 0xc1c, 0xc4d, 0xc2c, 0xc46, 0xc15, 0xc3f, 0xc38, 0xc4d,
0xc24, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc35, 0xc28, 0xc4c, 0xc1f, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc35, 0xc28, 0xc4c, 0xc1f, 0xc41, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e,
0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc35, 0xc28, 0xc4c,
0xc1f, 0xc41, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc35, 0xc46, 0xc28, 0xc3f, 0xc1c, 0xc41, 0xc32, 0xc3e, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc35, 0xc4d, 0xc32, 0xc3e, 0xc21, 0xc3f, 0xc35, 0xc4b, 0xc38, 0xc4d,
0xc1f, 0xc4b, 0xc15, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc35, 0xc4d, 0xc32,
0xc3e, 0xc21, 0xc3f, 0xc35, 0xc4b, 0xc38, 0xc4d, 0xc1f, 0xc4b, 0xc15, 0xc4d, 0x20,
0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc35, 0xc4d, 0xc32, 0xc3e, 0xc21, 0xc3f, 0xc35, 0xc4b, 0xc38, 0xc4d,
0xc1f, 0xc4b, 0xc15, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc35, 0xc4b, 0xc32, 0xc4d, 0xc17, 0xc4b, 0xc17, 0xc4d, 0xc30,
0xc3e, 0xc21, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc35, 0xc4b, 0xc32, 0xc4d,
0xc17, 0xc4b, 0xc17, 0xc4d, 0xc30, 0xc3e, 0xc21, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30,
0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc35,
0xc4b, 0xc32, 0xc4d, 0xc17, 0xc4b, 0xc17, 0xc4d, 0xc30, 0xc3e, 0xc21, 0xc4d, 0x20,
0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc35, 0xc47,
0xc15, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc35, 0xc3e, 0xc32, 0xc40, 0xc38, 0xc4d, 0x20, 0xc2e, 0xc30, 0xc3f, 0xc2f, 0xc41,
0x20, 0xc2b, 0xc41, 0xc1f, 0xc41, 0xc28, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc2f, 0xc3e, 0xc15, 0xc41, 0xc1f, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc2f, 0xc3e, 0xc15, 0xc41, 0xc1f, 0xc4d, 0xc38, 0xc4d, 0xc15,
0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2f, 0xc3e, 0xc15, 0xc41, 0xc1f, 0xc4d, 0xc38, 0xc4d,
0xc15, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc2f, 0xc46, 0xc15, 0xc1f, 0xc46, 0xc30, 0xc3f, 0xc28, 0xc4d, 0x200c, 0xc2c,
0xc30, 0xc4d, 0xc17, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2f, 0xc46, 0xc15,
0xc1f, 0xc46, 0xc30, 0xc3f, 0xc28, 0xc4d, 0x200c, 0xc2c, 0xc30, 0xc4d, 0xc17, 0xc4d,
0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc2f, 0xc46, 0xc15, 0xc1f, 0xc46, 0xc30, 0xc3f, 0xc28, 0xc4d,
0x200c, 0xc2c, 0xc30, 0xc4d, 0xc17, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2f, 0xc41, 0xc15, 0xc4b, 0xc28, 0xc4d, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe32, 0xe40, 0xe01,
0xe23, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19,
0xe2d, 0xe32, 0xe40, 0xe01, 0xe23, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39,
0xe23, 0xe49, 0xe2d, 0xe19, 0xe2d, 0xe32, 0xe40, 0xe01, 0xe23, 0xe40, 0xe27, 0xe25,
0xe32, 0xe2d, 0xe31, 0xe1f, 0xe01, 0xe32, 0xe19, 0xe34, 0xe2a, 0xe16, 0xe32, 0xe19,
0xe40, 0xe27, 0xe25, 0xe32, 0xe41, 0xe2d, 0xe1f, 0xe23, 0xe34, 0xe01, 0xe32, 0xe01,
0xe25, 0xe32, 0xe07, 0xe40, 0xe27, 0xe25, 0xe32, 0xe41, 0xe2d, 0xe1f, 0xe23, 0xe34,
0xe01, 0xe32, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe2d, 0xe2d, 0xe01, 0xe40, 0xe27,
0xe25, 0xe32, 0xe41, 0xe2d, 0xe1f, 0xe23, 0xe34, 0xe01, 0xe32, 0xe43, 0xe15, 0xe49,
0xe40, 0xe27, 0xe25, 0xe32, 0xe41, 0xe2d, 0xe1f, 0xe23, 0xe34, 0xe01, 0xe32, 0xe15,
0xe30, 0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32,
0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe41, 0xe2d, 0xe1f, 0xe23, 0xe34, 0xe01, 0xe32,
0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24,
0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe41, 0xe2d, 0xe1f, 0xe23, 0xe34, 0xe01,
0xe32, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32,
0xe2d, 0xe30, 0xe41, 0xe25, 0xe2a, 0xe01, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d, 0xe30, 0xe41, 0xe25, 0xe2a, 0xe01,
0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe02,
0xe2d, 0xe07, 0xe2d, 0xe30, 0xe41, 0xe25, 0xe2a, 0xe01, 0xe32, 0xe40, 0xe27, 0xe25,
0xe32, 0xe2d, 0xe31, 0xe25, 0xe21, 0xe32, 0xe15, 0xe35, 0xe40, 0xe27, 0xe25, 0xe32,
0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d, 0xe31, 0xe25, 0xe21, 0xe32,
0xe15, 0xe35, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d,
0xe19, 0xe2d, 0xe31, 0xe25, 0xe21, 0xe32, 0xe15, 0xe35, 0xe40, 0xe27, 0xe25, 0xe32,
0xe41, 0xe2d, 0xe21, 0xe30, 0xe0b, 0xe2d, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe41, 0xe2d, 0xe21, 0xe30, 0xe0b, 0xe2d,
0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19,
0xe41, 0xe2d, 0xe21, 0xe30, 0xe0b, 0xe2d, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe15,
0xe2d, 0xe19, 0xe01, 0xe25, 0xe32, 0xe07, 0xe43, 0xe19, 0xe2d, 0xe40, 0xe21, 0xe23,
0xe34, 0xe01, 0xe32, 0xe40, 0xe2b, 0xe19, 0xe37, 0xe2d, 0xe40, 0xe27, 0xe25, 0xe32,
0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe15, 0xe2d, 0xe19, 0xe01, 0xe25,
0xe32, 0xe07, 0xe43, 0xe19, 0xe2d, 0xe40, 0xe21, 0xe23, 0xe34, 0xe01, 0xe32, 0xe40,
0xe2b, 0xe19, 0xe37, 0xe2d, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41,
0xe2a, 0xe07, 0xe15, 0xe2d, 0xe19, 0xe01, 0xe25, 0xe32, 0xe07, 0xe43, 0xe19, 0xe2d,
0xe40, 0xe21, 0xe23, 0xe34, 0xe01, 0xe32, 0xe40, 0xe2b, 0xe19, 0xe37, 0xe2d, 0xe40,
0xe27, 0xe25, 0xe32, 0xe17, 0xe32, 0xe07, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe2d,
0xe2d, 0xe01, 0xe43, 0xe19, 0xe2d, 0xe40, 0xe21, 0xe23, 0xe34, 0xe01, 0xe32, 0xe40,
0xe2b, 0xe19, 0xe37, 0xe2d, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23,
0xe10, 0xe32, 0xe19, 0xe17, 0xe32, 0xe07, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe2d,
0xe2d, 0xe01, 0xe43, 0xe19, 0xe2d, 0xe40, 0xe21, 0xe23, 0xe34, 0xe01, 0xe32, 0xe40,
0xe2b, 0xe19, 0xe37, 0xe2d, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41,
0xe2a, 0xe07, 0xe17, 0xe32, 0xe07, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe2d, 0xe2d,
0xe01, 0xe43, 0xe19, 0xe2d, 0xe40, 0xe21, 0xe23, 0xe34, 0xe01, 0xe32, 0xe40, 0xe2b,
0xe19, 0xe37, 0xe2d, 0xe40, 0xe27, 0xe25, 0xe32, 0xe41, 0xe16, 0xe1a, 0xe20, 0xe39,
0xe40, 0xe02, 0xe32, 0xe43, 0xe19, 0xe2d, 0xe40, 0xe21, 0xe23, 0xe34, 0xe01, 0xe32,
0xe40, 0xe2b, 0xe19, 0xe37, 0xe2d, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15,
0xe23, 0xe10, 0xe32, 0xe19, 0xe41, 0xe16, 0xe1a, 0xe20, 0xe39, 0xe40, 0xe02, 0xe32,
0xe43, 0xe19, 0xe2d, 0xe40, 0xe21, 0xe23, 0xe34, 0xe01, 0xe32, 0xe40, 0xe2b, 0xe19,
0xe37, 0xe2d, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07,
0xe41, 0xe16, 0xe1a, 0xe20, 0xe39, 0xe40, 0xe02, 0xe32, 0xe43, 0xe19, 0xe2d, 0xe40,
0xe21, 0xe23, 0xe34, 0xe01, 0xe32, 0xe40, 0xe2b, 0xe19, 0xe37, 0xe2d, 0xe40, 0xe27,
0xe25, 0xe32, 0xe41, 0xe1b, 0xe0b, 0xe34, 0xe1f, 0xe34, 0xe01, 0xe43, 0xe19, 0xe2d,
0xe40, 0xe21, 0xe23, 0xe34, 0xe01, 0xe32, 0xe40, 0xe2b, 0xe19, 0xe37, 0xe2d, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe41, 0xe1b,
0xe0b, 0xe34, 0xe1f, 0xe34, 0xe01, 0xe43, 0xe19, 0xe2d, 0xe40, 0xe21, 0xe23, 0xe34,
0xe01, 0xe32, 0xe40, 0xe2b, 0xe19, 0xe37, 0xe2d, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d,
0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe41, 0xe1b, 0xe0b, 0xe34, 0xe1f, 0xe34, 0xe01,
0xe43, 0xe19, 0xe2d, 0xe40, 0xe21, 0xe23, 0xe34, 0xe01, 0xe32, 0xe40, 0xe2b, 0xe19,
0xe37, 0xe2d, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe30, 0xe19, 0xe32, 0xe14, 0xe35,
0xe23, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32,
0xe19, 0xe2d, 0xe30, 0xe19, 0xe32, 0xe14, 0xe35, 0xe23, 0xe4c, 0xe40, 0xe27, 0xe25,
0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe2d, 0xe30, 0xe19, 0xe32,
0xe14, 0xe35, 0xe23, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe32, 0xe1b, 0xe35,
0xe2d, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32,
0xe19, 0xe2d, 0xe32, 0xe1b, 0xe35, 0xe2d, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d,
0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe2d, 0xe32, 0xe1b, 0xe35, 0xe2d, 0xe32, 0xe40,
0xe27, 0xe25, 0xe32, 0xe2d, 0xe31, 0xe04, 0xe15, 0xe32, 0xe2d, 0xe39, 0xe40, 0xe27,
0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d, 0xe31, 0xe04,
0xe15, 0xe32, 0xe2d, 0xe39, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23,
0xe49, 0xe2d, 0xe19, 0xe2d, 0xe31, 0xe04, 0xe15, 0xe32, 0xe2d, 0xe39, 0xe40, 0xe27,
0xe25, 0xe32, 0xe2d, 0xe31, 0xe04, 0xe42, 0xe17, 0xe40, 0xe1a, 0xe40, 0xe27, 0xe25,
0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d, 0xe31, 0xe04, 0xe42,
0xe17, 0xe40, 0xe1a, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49,
0xe2d, 0xe19, 0xe2d, 0xe31, 0xe04, 0xe42, 0xe17, 0xe40, 0xe1a, 0xe40, 0xe27, 0xe25,
0xe32, 0xe2d, 0xe32, 0xe2b, 0xe23, 0xe31, 0xe1a, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d, 0xe32, 0xe2b, 0xe23, 0xe31, 0xe1a,
0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe2d, 0xe32,
0xe2b, 0xe23, 0xe31, 0xe1a, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe32, 0xe23, 0xe4c,
0xe40, 0xe08, 0xe19, 0xe15, 0xe34, 0xe19, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d, 0xe32, 0xe23, 0xe4c, 0xe40, 0xe08,
0xe19, 0xe15, 0xe34, 0xe19, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39,
0xe23, 0xe49, 0xe2d, 0xe19, 0xe2d, 0xe32, 0xe23, 0xe4c, 0xe40, 0xe08, 0xe19, 0xe15,
0xe34, 0xe19, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19,
0xe15, 0xe01, 0xe02, 0xe2d, 0xe07, 0xe2d, 0xe32, 0xe23, 0xe4c, 0xe40, 0xe08, 0xe19,
0xe15, 0xe34, 0xe19, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23,
0xe10, 0xe32, 0xe19, 0xe17, 0xe32, 0xe07, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15,
0xe01, 0xe02, 0xe2d, 0xe07, 0xe2d, 0xe32, 0xe23, 0xe4c, 0xe40, 0xe08, 0xe19, 0xe15,
0xe34, 0xe19, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49,
0xe2d, 0xe19, 0xe17, 0xe32, 0xe07, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15, 0xe01,
0xe02, 0xe2d, 0xe07, 0xe2d, 0xe32, 0xe23, 0xe4c, 0xe40, 0xe08, 0xe19, 0xe15, 0xe34,
0xe19, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe32, 0xe23, 0xe4c, 0xe40, 0xe21,
0xe40, 0xe19, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23,
0xe10, 0xe32, 0xe19, 0xe2d, 0xe32, 0xe23, 0xe4c, 0xe40, 0xe21, 0xe40, 0xe19, 0xe35,
0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19,
0xe2d, 0xe32, 0xe23, 0xe4c, 0xe40, 0xe21, 0xe40, 0xe19, 0xe35, 0xe22, 0xe40, 0xe27,
0xe25, 0xe32, 0xe41, 0xe2d, 0xe15, 0xe41, 0xe25, 0xe19, 0xe15, 0xe34, 0xe01, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe41, 0xe2d,
0xe15, 0xe41, 0xe25, 0xe19, 0xe15, 0xe34, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d,
0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe02, 0xe2d, 0xe07, 0xe41, 0xe2d, 0xe15, 0xe41,
0xe25, 0xe19, 0xe15, 0xe34, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe2a,
0xe40, 0xe15, 0xe23, 0xe40, 0xe25, 0xe35, 0xe22, 0xe01, 0xe25, 0xe32, 0xe07, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe17, 0xe32,
0xe07, 0xe15, 0xe2d, 0xe19, 0xe01, 0xe25, 0xe32, 0xe07, 0xe02, 0xe2d, 0xe07, 0xe2d,
0xe2d, 0xe2a, 0xe40, 0xe15, 0xe23, 0xe40, 0xe25, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25,
0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe17, 0xe32, 0xe07, 0xe15, 0xe2d,
0xe19, 0xe01, 0xe25, 0xe32, 0xe07, 0xe02, 0xe2d, 0xe07, 0xe2d, 0xe2d, 0xe2a, 0xe40,
0xe15, 0xe23, 0xe40, 0xe25, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe17, 0xe32,
0xe07, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe15, 0xe2d, 0xe19, 0xe01,
0xe25, 0xe32, 0xe07, 0xe02, 0xe2d, 0xe07, 0xe2d, 0xe2d, 0xe2a, 0xe40, 0xe15, 0xe23,
0xe40, 0xe25, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23,
0xe10, 0xe32, 0xe19, 0xe17, 0xe32, 0xe07, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15,
0xe01, 0xe15, 0xe2d, 0xe19, 0xe01, 0xe25, 0xe32, 0xe07, 0xe02, 0xe2d, 0xe07, 0xe2d,
0xe2d, 0xe2a, 0xe40, 0xe15, 0xe23, 0xe40, 0xe25, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25,
0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe17, 0xe32, 0xe07, 0xe15, 0xe30,
0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe15, 0xe2d, 0xe19, 0xe01, 0xe25, 0xe32, 0xe07,
0xe02, 0xe2d, 0xe07, 0xe2d, 0xe2d, 0xe2a, 0xe40, 0xe15, 0xe23, 0xe40, 0xe25, 0xe35,
0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe2a, 0xe40, 0xe15, 0xe23, 0xe40,
0xe25, 0xe35, 0xe22, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe2d, 0xe2d, 0xe01, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe17, 0xe32,
0xe07, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe2d, 0xe2d, 0xe01, 0xe02, 0xe2d, 0xe07,
0xe2d, 0xe2d, 0xe2a, 0xe40, 0xe15, 0xe23, 0xe40, 0xe25, 0xe35, 0xe22, 0xe40, 0xe27,
0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe17, 0xe32, 0xe07, 0xe15,
0xe30, 0xe27, 0xe31, 0xe19, 0xe2d, 0xe2d, 0xe01, 0xe02, 0xe2d, 0xe07, 0xe2d, 0xe2d,
0xe2a, 0xe40, 0xe15, 0xe23, 0xe40, 0xe25, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32,
0xe2d, 0xe2d, 0xe2a, 0xe40, 0xe15, 0xe23, 0xe40, 0xe25, 0xe35, 0xe22, 0xe15, 0xe30,
0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15,
0xe23, 0xe10, 0xe32, 0xe19, 0xe17, 0xe32, 0xe07, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19,
0xe15, 0xe01, 0xe02, 0xe2d, 0xe07, 0xe2d, 0xe2d, 0xe2a, 0xe40, 0xe15, 0xe23, 0xe40,
0xe25, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a,
0xe07, 0xe17, 0xe32, 0xe07, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe02,
0xe2d, 0xe07, 0xe2d, 0xe2d, 0xe2a, 0xe40, 0xe15, 0xe23, 0xe40, 0xe25, 0xe35, 0xe22,
0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe32, 0xe40, 0xe0b, 0xe2d, 0xe23, 0xe4c, 0xe44,
0xe1a, 0xe08, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23,
0xe10, 0xe32, 0xe19, 0xe2d, 0xe32, 0xe40, 0xe0b, 0xe2d, 0xe23, 0xe4c, 0xe44, 0xe1a,
0xe08, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49,
0xe2d, 0xe19, 0xe2d, 0xe32, 0xe40, 0xe0b, 0xe2d, 0xe23, 0xe4c, 0xe44, 0xe1a, 0xe08,
0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe30, 0xe42, 0xe0b, 0xe23, 0xe4c,
0xe2a, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19,
0xe2d, 0xe30, 0xe42, 0xe0b, 0xe23, 0xe4c, 0xe2a, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24,
0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe2d, 0xe30, 0xe42, 0xe0b, 0xe23, 0xe4c,
0xe2a, 0xe40, 0xe27, 0xe25, 0xe32, 0xe1a, 0xe31, 0xe07, 0xe01, 0xe25, 0xe32, 0xe40,
0xe17, 0xe28, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32,
0xe19, 0xe1a, 0xe31, 0xe07, 0xe01, 0xe25, 0xe32, 0xe40, 0xe17, 0xe28, 0xe40, 0xe27,
0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe1a, 0xe31, 0xe07,
0xe01, 0xe25, 0xe32, 0xe40, 0xe17, 0xe28, 0xe40, 0xe27, 0xe25, 0xe32, 0xe20, 0xe39,
0xe0f, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe42, 0xe1a, 0xe25, 0xe34, 0xe40,
0xe27, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe1a, 0xe23, 0xe32, 0xe0b, 0xe34,
0xe40, 0xe25, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23,
0xe10, 0xe32, 0xe19, 0xe1a, 0xe23, 0xe32, 0xe0b, 0xe34, 0xe40, 0xe25, 0xe35, 0xe22,
0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe1a,
0xe23, 0xe32, 0xe0b, 0xe34, 0xe40, 0xe25, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32,
0xe1a, 0xe23, 0xe39, 0xe44, 0xe19, 0xe14, 0xe32, 0xe23, 0xe38, 0xe2a, 0xe0b, 0xe32,
0xe25, 0xe32, 0xe21, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe04, 0xe1b, 0xe40, 0xe27,
0xe34, 0xe23, 0xe4c, 0xe14, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23,
0xe10, 0xe32, 0xe19, 0xe40, 0xe04, 0xe1b, 0xe40, 0xe27, 0xe34, 0xe23, 0xe4c, 0xe14,
0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe40,
0xe04, 0xe1b, 0xe40, 0xe27, 0xe34, 0xe23, 0xe4c, 0xe14, 0xe40, 0xe27, 0xe25, 0xe32,
0xe40, 0xe04, 0xe0b, 0xe35, 0xe22, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe0a, 0xe32,
0xe21, 0xe2d, 0xe23, 0xe4c, 0xe42, 0xe23, 0xe40, 0xe27, 0xe25, 0xe32, 0xe41, 0xe0a,
0xe17, 0xe31, 0xe21, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10,
0xe32, 0xe19, 0xe41, 0xe0a, 0xe17, 0xe31, 0xe21, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d,
0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe41, 0xe0a, 0xe17, 0xe31, 0xe21, 0xe40, 0xe27,
0xe25, 0xe32, 0xe0a, 0xe34, 0xe25, 0xe35, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32,
0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe0a, 0xe34, 0xe25, 0xe35, 0xe40, 0xe27, 0xe25,
0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe0a, 0xe34, 0xe25, 0xe35,
0xe40, 0xe27, 0xe25, 0xe32, 0xe08, 0xe35, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe08, 0xe35, 0xe19, 0xe40, 0xe27, 0xe25,
0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe08, 0xe35, 0xe19, 0xe40, 0xe27,
0xe25, 0xe32, 0xe40, 0xe01, 0xe32, 0xe30, 0xe04, 0xe23, 0xe34, 0xe2a, 0xe15, 0xe4c,
0xe21, 0xe32, 0xe2a, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2b, 0xe21, 0xe39, 0xe48, 0xe40,
0xe01, 0xe32, 0xe30, 0xe42, 0xe04, 0xe42, 0xe04, 0xe2a, 0xe40, 0xe27, 0xe25, 0xe32,
0xe42, 0xe04, 0xe25, 0xe2d, 0xe21, 0xe40, 0xe1a, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25,
0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe42, 0xe04, 0xe25, 0xe2d,
0xe21, 0xe40, 0xe1a, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39,
0xe23, 0xe49, 0xe2d, 0xe19, 0xe42, 0xe04, 0xe25, 0xe2d, 0xe21, 0xe40, 0xe1a, 0xe35,
0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2b, 0xe21, 0xe39, 0xe48, 0xe40, 0xe01, 0xe32,
0xe30, 0xe04, 0xe38, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23,
0xe10, 0xe32, 0xe19, 0xe2b, 0xe21, 0xe39, 0xe48, 0xe40, 0xe01, 0xe32, 0xe30, 0xe04,
0xe38, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe04, 0xe23, 0xe36, 0xe48, 0xe07, 0xe24,
0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe2b, 0xe21, 0xe39, 0xe48, 0xe40, 0xe01,
0xe32, 0xe30, 0xe04, 0xe38, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe04, 0xe34, 0xe27,
0xe1a, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32,
0xe19, 0xe04, 0xe34, 0xe27, 0xe1a, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d,
0xe21, 0xe41, 0xe2a, 0xe07, 0xe02, 0xe2d, 0xe07, 0xe04, 0xe34, 0xe27, 0xe1a, 0xe32,
0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe14, 0xe27, 0xe34, 0xe2a, 0xe40, 0xe27, 0xe25,
0xe32, 0xe14, 0xe39, 0xe21, 0xe2d, 0xe07, 0xe15, 0xe4c, 0xe14, 0xe39, 0xe23, 0xe4c,
0xe27, 0xe34, 0xe25, 0xe25, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe15, 0xe34, 0xe21,
0xe2d, 0xe23, 0xe4c, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe2d, 0xe2d, 0xe01, 0xe40,
0xe27, 0xe25, 0xe32, 0xe40, 0xe01, 0xe32, 0xe30, 0xe2d, 0xe35, 0xe2a, 0xe40, 0xe15,
0xe2d, 0xe23, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10,
0xe32, 0xe19, 0xe40, 0xe01, 0xe32, 0xe30, 0xe2d, 0xe35, 0xe2a, 0xe40, 0xe15, 0xe2d,
0xe23, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d,
0xe19, 0xe40, 0xe01, 0xe32, 0xe30, 0xe2d, 0xe35, 0xe2a, 0xe40, 0xe15, 0xe2d, 0xe23,
0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe2d, 0xe01, 0xe27, 0xe32, 0xe14, 0xe2d,
0xe23, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe22, 0xe38, 0xe42, 0xe23, 0xe1b, 0xe01,
0xe25, 0xe32, 0xe07, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10,
0xe32, 0xe19, 0xe22, 0xe38, 0xe42, 0xe23, 0xe1b, 0xe01, 0xe25, 0xe32, 0xe07, 0xe40,
0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe22, 0xe38,
0xe42, 0xe23, 0xe1b, 0xe01, 0xe25, 0xe32, 0xe07, 0xe40, 0xe27, 0xe25, 0xe32, 0xe22,
0xe38, 0xe42, 0xe23, 0xe1b, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe2d, 0xe2d, 0xe01,
0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe22,
0xe38, 0xe42, 0xe23, 0xe1b, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe2d, 0xe2d, 0xe01,
0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe22,
0xe38, 0xe42, 0xe23, 0xe1b, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe2d, 0xe2d, 0xe01,
0xe40, 0xe27, 0xe25, 0xe32, 0xe22, 0xe38, 0xe42, 0xe23, 0xe1b, 0xe15, 0xe30, 0xe27,
0xe31, 0xe19, 0xe2d, 0xe2d, 0xe01, 0xe44, 0xe01, 0xe25, 0xe40, 0xe27, 0xe25, 0xe32,
0xe22, 0xe38, 0xe42, 0xe23, 0xe1b, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15, 0xe01,
0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe22,
0xe38, 0xe42, 0xe23, 0xe1b, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe40,
0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe22, 0xe38,
0xe42, 0xe23, 0xe1b, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe40, 0xe27,
0xe25, 0xe32, 0xe2b, 0xe21, 0xe39, 0xe48, 0xe40, 0xe01, 0xe32, 0xe30, 0xe1f, 0xe2d,
0xe25, 0xe4c, 0xe01, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32,
0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2b, 0xe21, 0xe39, 0xe48, 0xe40,
0xe01, 0xe32, 0xe30, 0xe1f, 0xe2d, 0xe25, 0xe4c, 0xe01, 0xe41, 0xe25, 0xe19, 0xe14,
0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19,
0xe2b, 0xe21, 0xe39, 0xe48, 0xe40, 0xe01, 0xe32, 0xe30, 0xe1f, 0xe2d, 0xe25, 0xe4c,
0xe01, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe1f, 0xe34,
0xe08, 0xe34, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32,
0xe19, 0xe1f, 0xe34, 0xe08, 0xe34, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39,
0xe23, 0xe49, 0xe2d, 0xe19, 0xe1f, 0xe34, 0xe08, 0xe34, 0xe40, 0xe27, 0xe25, 0xe32,
0xe40, 0xe1f, 0xe23, 0xe19, 0xe0a, 0xe4c, 0xe40, 0xe01, 0xe35, 0xe22, 0xe19, 0xe32,
0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe1f, 0xe23, 0xe19, 0xe0a, 0xe4c, 0xe40, 0xe0b,
0xe32, 0xe40, 0xe17, 0xe34, 0xe23, 0xe4c, 0xe19, 0xe41, 0xe25, 0xe30, 0xe41, 0xe2d,
0xe19, 0xe15, 0xe32, 0xe23, 0xe4c, 0xe01, 0xe15, 0xe34, 0xe01, 0xe40, 0xe27, 0xe25,
0xe32, 0xe01, 0xe32, 0xe25, 0xe32, 0xe1b, 0xe32, 0xe42, 0xe01, 0xe2a, 0xe40, 0xe27,
0xe25, 0xe32, 0xe41, 0xe01, 0xe21, 0xe40, 0xe1a, 0xe35, 0xe22, 0xe23, 0xe4c, 0xe40,
0xe27, 0xe25, 0xe32, 0xe08, 0xe2d, 0xe23, 0xe4c, 0xe40, 0xe08, 0xe35, 0xe22, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe08, 0xe2d,
0xe23, 0xe4c, 0xe40, 0xe08, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14,
0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe08, 0xe2d, 0xe23, 0xe4c, 0xe40, 0xe08, 0xe35,
0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2b, 0xe21, 0xe39, 0xe48, 0xe40, 0xe01, 0xe32,
0xe30, 0xe01, 0xe34, 0xe25, 0xe40, 0xe1a, 0xe34, 0xe23, 0xe4c, 0xe15, 0xe40, 0xe27,
0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe01, 0xe23, 0xe35,
0xe19, 0xe34, 0xe0a, 0xe40, 0xe27, 0xe25, 0xe32, 0xe01, 0xe23, 0xe35, 0xe19, 0xe41,
0xe25, 0xe19, 0xe14, 0xe4c, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe2d, 0xe2d, 0xe01,
0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe01,
0xe23, 0xe35, 0xe19, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c, 0xe15, 0xe30, 0xe27, 0xe31,
0xe19, 0xe2d, 0xe2d, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23,
0xe49, 0xe2d, 0xe19, 0xe01, 0xe23, 0xe35, 0xe19, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c,
0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe2d, 0xe2d, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32,
0xe01, 0xe23, 0xe35, 0xe19, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c, 0xe15, 0xe30, 0xe27,
0xe31, 0xe19, 0xe15, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23,
0xe10, 0xe32, 0xe19, 0xe01, 0xe23, 0xe35, 0xe19, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c,
0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24,
0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe01, 0xe23, 0xe35, 0xe19, 0xe41, 0xe25,
0xe19, 0xe14, 0xe4c, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe40, 0xe27,
0xe25, 0xe32, 0xe01, 0xe27, 0xe21, 0xe40, 0xe27, 0xe25, 0xe32, 0xe01, 0xe31, 0xe25,
0xe1f, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe01, 0xe32, 0xe22, 0xe2d, 0xe32, 0xe19,
0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2e, 0xe32, 0xe27, 0xe32, 0xe22, 0x2d, 0xe2d,
0xe30, 0xe25, 0xe39, 0xe40, 0xe0a, 0xe35, 0xe22, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32,
0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2e, 0xe32, 0xe27, 0xe32, 0xe22,
0x2d, 0xe2d, 0xe30, 0xe25, 0xe39, 0xe40, 0xe0a, 0xe35, 0xe22, 0xe19, 0xe40, 0xe27,
0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe2e, 0xe32, 0xe27, 0xe32,
0xe22, 0x2d, 0xe2d, 0xe30, 0xe25, 0xe39, 0xe40, 0xe0a, 0xe35, 0xe22, 0xe19, 0xe40,
0xe27, 0xe25, 0xe32, 0xe2e, 0xe48, 0xe2d, 0xe07, 0xe01, 0xe07, 0xe40, 0xe27, 0xe25,
0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2e, 0xe48, 0xe2d, 0xe07,
0xe01, 0xe07, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d,
0xe19, 0xe2e, 0xe48, 0xe2d, 0xe07, 0xe01, 0xe07, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2e,
0xe2d, 0xe1f, 0xe14, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23,
0xe10, 0xe32, 0xe19, 0xe2e, 0xe2d, 0xe1f, 0xe14, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32,
0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe2e, 0xe2d, 0xe1f, 0xe14, 0xe4c,
0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe34, 0xe19, 0xe40, 0xe14, 0xe35, 0xe22, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe2b, 0xe32, 0xe2a, 0xe21, 0xe38, 0xe17, 0xe23, 0xe2d,
0xe34, 0xe19, 0xe40, 0xe14, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe34,
0xe19, 0xe42, 0xe14, 0xe08, 0xe35, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe34,
0xe19, 0xe42, 0xe14, 0xe19, 0xe35, 0xe40, 0xe0b, 0xe35, 0xe22, 0xe15, 0xe2d, 0xe19,
0xe01, 0xe25, 0xe32, 0xe07, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe34, 0xe19, 0xe42,
0xe14, 0xe19, 0xe35, 0xe40, 0xe0b, 0xe35, 0xe22, 0xe1d, 0xe31, 0xe48, 0xe07, 0xe15,
0xe30, 0xe27, 0xe31, 0xe19, 0xe2d, 0xe2d, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d,
0xe34, 0xe19, 0xe42, 0xe14, 0xe19, 0xe35, 0xe40, 0xe0b, 0xe35, 0xe22, 0xe1d, 0xe31,
0xe48, 0xe07, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe40, 0xe27, 0xe25,
0xe32, 0xe2d, 0xe34, 0xe2b, 0xe23, 0xe48, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32,
0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d, 0xe34, 0xe2b, 0xe23, 0xe48,
0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07,
0xe2d, 0xe34, 0xe2b, 0xe23, 0xe48, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d,
0xe35, 0xe23, 0xe4c, 0xe04, 0xe38, 0xe15, 0xe2a, 0xe04, 0xe4c, 0xe40, 0xe27, 0xe25,
0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d, 0xe35, 0xe23, 0xe4c,
0xe04, 0xe38, 0xe15, 0xe2a, 0xe04, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14,
0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe2d, 0xe35, 0xe23, 0xe4c, 0xe04, 0xe38, 0xe15,
0xe2a, 0xe04, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe34, 0xe2a, 0xe23, 0xe32,
0xe40, 0xe2d, 0xe25, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10,
0xe32, 0xe19, 0xe2d, 0xe34, 0xe2a, 0xe23, 0xe32, 0xe40, 0xe2d, 0xe25, 0xe40, 0xe27,
0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe2d, 0xe34, 0xe2a, 0xe23,
0xe32, 0xe40, 0xe2d, 0xe25, 0xe40, 0xe27, 0xe25, 0xe32, 0xe0d, 0xe35, 0xe48, 0xe1b,
0xe38, 0xe48, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10,
0xe32, 0xe19, 0xe0d, 0xe35, 0xe48, 0xe1b, 0xe38, 0xe48, 0xe19, 0xe40, 0xe27, 0xe25,
0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe0d, 0xe35, 0xe48, 0xe1b, 0xe38,
0xe48, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe04, 0xe31, 0xe21, 0xe0a, 0xe31, 0xe15,
0xe04, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe1b, 0xe42, 0xe15, 0xe23, 0xe1b,
0xe31, 0xe1f, 0xe25, 0xe2d, 0xe1f, 0xe2a, 0xe04, 0xe4c, 0x2d, 0xe04, 0xe31, 0xe21,
0xe0a, 0xe31, 0xe15, 0xe2a, 0xe01, 0xe35, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14,
0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe40, 0xe1b, 0xe42, 0xe15, 0xe23, 0xe1b, 0xe31,
0xe1f, 0xe25, 0xe2d, 0xe1f, 0xe2a, 0xe04, 0xe4c, 0x2d, 0xe04, 0xe31, 0xe21, 0xe0a,
0xe31, 0xe15, 0xe2a, 0xe01, 0xe35, 0xe40, 0xe27, 0xe25, 0xe32, 0xe04, 0xe32, 0xe0b,
0xe31, 0xe04, 0xe2a, 0xe16, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe04, 0xe32,
0xe0b, 0xe31, 0xe04, 0xe2a, 0xe16, 0xe32, 0xe19, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19,
0xe2d, 0xe2d, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe04, 0xe32, 0xe0b, 0xe31, 0xe04,
0xe2a, 0xe16, 0xe32, 0xe19, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe40,
0xe27, 0xe25, 0xe32, 0xe40, 0xe01, 0xe32, 0xe2b, 0xe25, 0xe35, 0xe40, 0xe27, 0xe25,
0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe40, 0xe01, 0xe32, 0xe2b,
0xe25, 0xe35, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07,
0xe40, 0xe01, 0xe32, 0xe2b, 0xe25, 0xe35, 0xe40, 0xe27, 0xe25, 0xe32, 0xe04, 0xe2d,
0xe2a, 0xe44, 0xe23, 0xe40, 0xe27, 0xe25, 0xe32, 0xe04, 0xe23, 0xe31, 0xe2a, 0xe42,
0xe19, 0xe22, 0xe32, 0xe2a, 0xe04, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32,
0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe04, 0xe23, 0xe31, 0xe2a, 0xe42, 0xe19, 0xe22,
0xe32, 0xe2a, 0xe04, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23,
0xe49, 0xe2d, 0xe19, 0xe04, 0xe23, 0xe31, 0xe2a, 0xe42, 0xe19, 0xe22, 0xe32, 0xe2a,
0xe04, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe04, 0xe35, 0xe23, 0xe4c, 0xe01, 0xe35,
0xe0b, 0xe2a, 0xe16, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe25, 0xe31, 0xe07,
0xe01, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2b, 0xe21, 0xe39, 0xe48, 0xe40, 0xe01,
0xe32, 0xe30, 0xe44, 0xe25, 0xe19, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe25, 0xe2d,
0xe23, 0xe4c, 0xe14, 0xe42, 0xe2e, 0xe27, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe25, 0xe2d, 0xe23, 0xe4c, 0xe14, 0xe42,
0xe2e, 0xe27, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a,
0xe07, 0xe25, 0xe2d, 0xe23, 0xe4c, 0xe14, 0xe42, 0xe2e, 0xe27, 0xe4c, 0xe40, 0xe27,
0xe25, 0xe32, 0xe21, 0xe32, 0xe40, 0xe01, 0xe4a, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32,
0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe21, 0xe32, 0xe40, 0xe01, 0xe4a,
0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19,
0xe21, 0xe32, 0xe40, 0xe01, 0xe4a, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32,
0xe01, 0xe32, 0xe14, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15,
0xe23, 0xe10, 0xe32, 0xe19, 0xe21, 0xe32, 0xe01, 0xe32, 0xe14, 0xe32, 0xe19, 0xe40,
0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe21, 0xe32,
0xe01, 0xe32, 0xe14, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe40,
0xe25, 0xe40, 0xe0b, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe31, 0xe25,
0xe14, 0xe35, 0xe1f, 0xe2a, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe23,
0xe4c, 0xe40, 0xe04, 0xe0b, 0xe31, 0xe2a, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2b, 0xe21,
0xe39, 0xe48, 0xe40, 0xe01, 0xe32, 0xe30, 0xe21, 0xe32, 0xe23, 0xe4c, 0xe41, 0xe0a,
0xe25, 0xe25, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe2d, 0xe23, 0xe34, 0xe40,
0xe0a, 0xe35, 0xe22, 0xe2a, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23,
0xe10, 0xe32, 0xe19, 0xe21, 0xe2d, 0xe23, 0xe34, 0xe40, 0xe0a, 0xe35, 0xe22, 0xe2a,
0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe02,
0xe2d, 0xe07, 0xe21, 0xe2d, 0xe23, 0xe34, 0xe40, 0xe0a, 0xe35, 0xe22, 0xe2a, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe2d, 0xe27, 0xe4c, 0xe2a, 0xe31, 0xe19, 0xe40, 0xe27,
0xe25, 0xe32, 0xe41, 0xe1b, 0xe0b, 0xe34, 0xe1f, 0xe34, 0xe01, 0xe40, 0xe21, 0xe47,
0xe01, 0xe0b, 0xe34, 0xe42, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15,
0xe23, 0xe10, 0xe32, 0xe19, 0xe41, 0xe1b, 0xe0b, 0xe34, 0xe1f, 0xe34, 0xe01, 0xe40,
0xe21, 0xe47, 0xe01, 0xe0b, 0xe34, 0xe42, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d,
0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe41, 0xe1b, 0xe0b, 0xe34, 0xe1f, 0xe34, 0xe01,
0xe40, 0xe21, 0xe47, 0xe01, 0xe0b, 0xe34, 0xe42, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32,
0xe2d, 0xe39, 0xe25, 0xe32, 0xe19, 0xe1a, 0xe32, 0xe15, 0xe2d, 0xe23, 0xe4c, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d, 0xe39,
0xe25, 0xe32, 0xe19, 0xe1a, 0xe32, 0xe15, 0xe2d, 0xe23, 0xe4c, 0xe40, 0xe27, 0xe25,
0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe2d, 0xe39, 0xe25, 0xe32,
0xe19, 0xe1a, 0xe32, 0xe15, 0xe2d, 0xe23, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe2d, 0xe2a, 0xe42, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23,
0xe10, 0xe32, 0xe19, 0xe21, 0xe2d, 0xe2a, 0xe42, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32,
0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe21, 0xe2d, 0xe2a, 0xe42, 0xe01,
0xe40, 0xe27, 0xe25, 0xe32, 0xe1e, 0xe21, 0xe48, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32,
0xe19, 0xe32, 0xe2d, 0xe39, 0xe23, 0xe39, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe19,
0xe1b, 0xe32, 0xe25, 0xe40, 0xe27, 0xe25, 0xe32, 0xe19, 0xe34, 0xe27, 0xe41, 0xe04,
0xe25, 0xe34, 0xe42, 0xe14, 0xe40, 0xe19, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32,
0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe19, 0xe34, 0xe27, 0xe41, 0xe04,
0xe25, 0xe34, 0xe42, 0xe14, 0xe40, 0xe19, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32,
0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe19, 0xe34, 0xe27, 0xe41, 0xe04,
0xe25, 0xe34, 0xe42, 0xe14, 0xe40, 0xe19, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32,
0xe19, 0xe34, 0xe27, 0xe0b, 0xe35, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c, 0xe40, 0xe27,
0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe19, 0xe34, 0xe27,
0xe0b, 0xe35, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d,
0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe19, 0xe34, 0xe27, 0xe0b, 0xe35, 0xe41, 0xe25,
0xe19, 0xe14, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe19, 0xe34, 0xe27, 0xe1f, 0xe31,
0xe19, 0xe14, 0xe4c, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32,
0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe19, 0xe34, 0xe27, 0xe1f, 0xe31,
0xe19, 0xe14, 0xe4c, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32,
0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe19, 0xe34, 0xe27, 0xe1f, 0xe31, 0xe19,
0xe14, 0xe4c, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe19,
0xe35, 0xe2d, 0xe39, 0xe40, 0xe2d, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe01, 0xe32,
0xe30, 0xe19, 0xe2d, 0xe23, 0xe4c, 0xe1f, 0xe2d, 0xe25, 0xe4c, 0xe01, 0xe40, 0xe27,
0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe40, 0xe01, 0xe32,
0xe30, 0xe19, 0xe2d, 0xe23, 0xe4c, 0xe1f, 0xe2d, 0xe25, 0xe4c, 0xe01, 0xe40, 0xe27,
0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe40, 0xe01, 0xe32, 0xe30,
0xe19, 0xe2d, 0xe23, 0xe4c, 0xe1f, 0xe2d, 0xe25, 0xe4c, 0xe01, 0xe40, 0xe27, 0xe25,
0xe32, 0xe2b, 0xe21, 0xe39, 0xe48, 0xe40, 0xe01, 0xe32, 0xe30, 0xe40, 0xe1f, 0xe2d,
0xe23, 0xe4c, 0xe19, 0xe31, 0xe19, 0xe42, 0xe14, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2b, 0xe21, 0xe39, 0xe48, 0xe40, 0xe01,
0xe32, 0xe30, 0xe40, 0xe1f, 0xe2d, 0xe23, 0xe4c, 0xe19, 0xe31, 0xe19, 0xe42, 0xe14,
0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe02,
0xe2d, 0xe07, 0xe2b, 0xe21, 0xe39, 0xe48, 0xe40, 0xe01, 0xe32, 0xe30, 0xe40, 0xe1f,
0xe2d, 0xe23, 0xe4c, 0xe19, 0xe31, 0xe19, 0xe42, 0xe14, 0xe40, 0xe27, 0xe25, 0xe32,
0xe2b, 0xe21, 0xe39, 0xe48, 0xe40, 0xe01, 0xe32, 0xe30, 0xe21, 0xe32, 0xe40, 0xe23,
0xe35, 0xe22, 0xe19, 0xe32, 0xe40, 0xe2b, 0xe19, 0xe37, 0xe2d, 0xe40, 0xe27, 0xe25,
0xe32, 0xe42, 0xe19, 0xe42, 0xe27, 0xe0b, 0xe35, 0xe1a, 0xe35, 0xe2a, 0xe04, 0xe4c,
0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe42,
0xe19, 0xe42, 0xe27, 0xe0b, 0xe35, 0xe1a, 0xe35, 0xe2a, 0xe04, 0xe4c, 0xe40, 0xe27,
0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe42, 0xe19, 0xe42,
0xe27, 0xe0b, 0xe35, 0xe1a, 0xe35, 0xe2a, 0xe04, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32,
0xe2d, 0xe2d, 0xe21, 0xe2a, 0xe04, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32,
0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d, 0xe2d, 0xe21, 0xe2a, 0xe04, 0xe4c, 0xe40,
0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe2d, 0xe2d,
0xe21, 0xe2a, 0xe04, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe1b, 0xe32, 0xe01, 0xe35,
0xe2a, 0xe16, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23,
0xe10, 0xe32, 0xe19, 0xe1b, 0xe32, 0xe01, 0xe35, 0xe2a, 0xe16, 0xe32, 0xe19, 0xe40,
0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe1b, 0xe32,
0xe01, 0xe35, 0xe2a, 0xe16, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe1b, 0xe32,
0xe40, 0xe25, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe1b, 0xe32, 0xe1b, 0xe31, 0xe27,
0xe19, 0xe34, 0xe27, 0xe01, 0xe34, 0xe19, 0xe35, 0xe40, 0xe27, 0xe25, 0xe32, 0xe1b,
0xe32, 0xe23, 0xe32, 0xe01, 0xe27, 0xe31, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe1b, 0xe32, 0xe23, 0xe32, 0xe01, 0xe27,
0xe31, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d,
0xe19, 0xe1b, 0xe32, 0xe23, 0xe32, 0xe01, 0xe27, 0xe31, 0xe22, 0xe40, 0xe27, 0xe25,
0xe32, 0xe40, 0xe1b, 0xe23, 0xe39, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15,
0xe23, 0xe10, 0xe32, 0xe19, 0xe40, 0xe1b, 0xe23, 0xe39, 0xe40, 0xe27, 0xe25, 0xe32,
0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe40, 0xe1b, 0xe23, 0xe39, 0xe40,
0xe27, 0xe25, 0xe32, 0xe1f, 0xe34, 0xe25, 0xe34, 0xe1b, 0xe1b, 0xe34, 0xe19, 0xe2a,
0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19,
0xe1f, 0xe34, 0xe25, 0xe34, 0xe1b, 0xe1b, 0xe34, 0xe19, 0xe2a, 0xe4c, 0xe40, 0xe27,
0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe1f, 0xe34, 0xe25,
0xe34, 0xe1b, 0xe1b, 0xe34, 0xe19, 0xe2a, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2b,
0xe21, 0xe39, 0xe48, 0xe40, 0xe01, 0xe32, 0xe30, 0xe1f, 0xe34, 0xe19, 0xe34, 0xe01,
0xe0b, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe41, 0xe0b, 0xe07, 0xe1b, 0xe35, 0xe41,
0xe22, 0xe23, 0xe4c, 0xe41, 0xe25, 0xe30, 0xe21, 0xe35, 0xe40, 0xe01, 0xe2d, 0xe25,
0xe07, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19,
0xe41, 0xe0b, 0xe07, 0xe1b, 0xe35, 0xe41, 0xe22, 0xe23, 0xe4c, 0xe41, 0xe25, 0xe30,
0xe21, 0xe35, 0xe40, 0xe01, 0xe2d, 0xe25, 0xe07, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d,
0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe02, 0xe2d, 0xe07, 0xe41, 0xe0b, 0xe07, 0xe1b,
0xe35, 0xe41, 0xe22, 0xe23, 0xe4c, 0xe41, 0xe25, 0xe30, 0xe21, 0xe35, 0xe40, 0xe01,
0xe2d, 0xe25, 0xe07, 0xe40, 0xe27, 0xe25, 0xe32, 0xe1e, 0xe34, 0xe15, 0xe41, 0xe04,
0xe23, 0xe4c, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe42, 0xe1b, 0xe19, 0xe32, 0xe40,
0xe1b, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe1b, 0xe35, 0xe22, 0xe07, 0xe22, 0xe32,
0xe07, 0xe40, 0xe27, 0xe25, 0xe32, 0xe04, 0xe37, 0xe22, 0xe0b, 0xe34, 0xe25, 0xe2d,
0xe2d, 0xe23, 0xe4c, 0xe14, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15,
0xe23, 0xe10, 0xe32, 0xe19, 0xe04, 0xe37, 0xe22, 0xe0b, 0xe34, 0xe25, 0xe2d, 0xe2d,
0xe23, 0xe4c, 0xe14, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23,
0xe49, 0xe2d, 0xe19, 0xe04, 0xe37, 0xe22, 0xe0b, 0xe34, 0xe25, 0xe2d, 0xe2d, 0xe23,
0xe4c, 0xe14, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe23, 0xe2d, 0xe39, 0xe19,
0xe35, 0xe22, 0xe07, 0xe40, 0xe27, 0xe25, 0xe32, 0xe42, 0xe23, 0xe18, 0xe35, 0xe23,
0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe0b, 0xe32, 0xe04, 0xe32, 0xe25, 0xe34, 0xe19,
0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe0b,
0xe32, 0xe04, 0xe32, 0xe25, 0xe34, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14,
0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe0b, 0xe32, 0xe04, 0xe32, 0xe25, 0xe34, 0xe19,
0xe40, 0xe27, 0xe25, 0xe32, 0xe0b, 0xe32, 0xe21, 0xe32, 0xe23, 0xe32, 0xe40, 0xe27,
0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe0b, 0xe32, 0xe21,
0xe32, 0xe23, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49,
0xe2d, 0xe19, 0xe0b, 0xe32, 0xe21, 0xe32, 0xe23, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32,
0xe0b, 0xe32, 0xe21, 0xe31, 0xe27, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15,
0xe23, 0xe10, 0xe32, 0xe19, 0xe0b, 0xe32, 0xe21, 0xe31, 0xe27, 0xe40, 0xe27, 0xe25,
0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe0b, 0xe32, 0xe21, 0xe31,
0xe27, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe0b, 0xe40, 0xe0a, 0xe25, 0xe2a, 0xe4c,
0xe40, 0xe27, 0xe25, 0xe32, 0xe2a, 0xe34, 0xe07, 0xe04, 0xe42, 0xe1b, 0xe23, 0xe4c,
0xe40, 0xe27, 0xe25, 0xe32, 0xe2b, 0xe21, 0xe39, 0xe48, 0xe40, 0xe01, 0xe32, 0xe30,
0xe42, 0xe0b, 0xe42, 0xe25, 0xe21, 0xe2d, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40,
0xe0b, 0xe32, 0xe17, 0xe4c, 0xe08, 0xe2d, 0xe23, 0xe4c, 0xe40, 0xe08, 0xe35, 0xe22,
0xe40, 0xe27, 0xe25, 0xe32, 0xe0b, 0xe39, 0xe23, 0xe34, 0xe19, 0xe32, 0xe40, 0xe21,
0xe40, 0xe27, 0xe25, 0xe32, 0xe44, 0xe0b, 0xe42, 0xe22, 0xe27, 0xe32, 0xe40, 0xe27,
0xe25, 0xe32, 0xe15, 0xe32, 0xe2e, 0xe35, 0xe15, 0xe35, 0xe40, 0xe27, 0xe25, 0xe32,
0xe44, 0xe17, 0xe40, 0xe1b, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23,
0xe10, 0xe32, 0xe19, 0xe44, 0xe17, 0xe40, 0xe1b, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d,
0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe44, 0xe17, 0xe40, 0xe1b, 0xe40, 0xe27, 0xe25,
0xe32, 0xe17, 0xe32, 0xe08, 0xe34, 0xe01, 0xe34, 0xe2a, 0xe16, 0xe32, 0xe19, 0xe40,
0xe27, 0xe25, 0xe32, 0xe42, 0xe15, 0xe40, 0xe01, 0xe40, 0xe25, 0xe32, 0xe40, 0xe27,
0xe25, 0xe32, 0xe15, 0xe2d, 0xe07, 0xe01, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe15, 0xe2d, 0xe07, 0xe01, 0xe32, 0xe40,
0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe15, 0xe2d,
0xe07, 0xe01, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe0a, 0xe38, 0xe01, 0xe40, 0xe27,
0xe25, 0xe32, 0xe40, 0xe15, 0xe34, 0xe23, 0xe4c, 0xe01, 0xe40, 0xe21, 0xe19, 0xe34,
0xe2a, 0xe16, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23,
0xe10, 0xe32, 0xe19, 0xe40, 0xe15, 0xe34, 0xe23, 0xe4c, 0xe01, 0xe40, 0xe21, 0xe19,
0xe34, 0xe2a, 0xe16, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39,
0xe23, 0xe49, 0xe2d, 0xe19, 0xe40, 0xe15, 0xe34, 0xe23, 0xe4c, 0xe01, 0xe40, 0xe21,
0xe19, 0xe34, 0xe2a, 0xe16, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe15, 0xe39,
0xe27, 0xe32, 0xe25, 0xe39, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe38, 0xe23, 0xe38,
0xe01, 0xe27, 0xe31, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23,
0xe10, 0xe32, 0xe19, 0xe2d, 0xe38, 0xe23, 0xe38, 0xe01, 0xe27, 0xe31, 0xe22, 0xe40,
0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe2d, 0xe38,
0xe23, 0xe38, 0xe01, 0xe27, 0xe31, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe38,
0xe0b, 0xe40, 0xe1a, 0xe01, 0xe34, 0xe2a, 0xe16, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25,
0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d, 0xe38, 0xe0b, 0xe40,
0xe1a, 0xe01, 0xe34, 0xe2a, 0xe16, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24,
0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe2d, 0xe38, 0xe0b, 0xe40, 0xe1a, 0xe01,
0xe34, 0xe2a, 0xe16, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe27, 0xe32, 0xe19,
0xe39, 0xe2d, 0xe32, 0xe15, 0xe39, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15,
0xe23, 0xe10, 0xe32, 0xe19, 0xe27, 0xe32, 0xe19, 0xe39, 0xe2d, 0xe32, 0xe15, 0xe39,
0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe27,
0xe32, 0xe19, 0xe39, 0xe2d, 0xe32, 0xe15, 0xe39, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40,
0xe27, 0xe40, 0xe19, 0xe0b, 0xe38, 0xe40, 0xe2d, 0xe25, 0xe32, 0xe40, 0xe27, 0xe25,
0xe32, 0xe27, 0xe25, 0xe32, 0xe14, 0xe35, 0xe27, 0xe2d, 0xe2a, 0xe15, 0xe2d, 0xe04,
0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe27,
0xe25, 0xe32, 0xe14, 0xe35, 0xe27, 0xe2d, 0xe2a, 0xe15, 0xe2d, 0xe04, 0xe40, 0xe27,
0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe27, 0xe25, 0xe32,
0xe14, 0xe35, 0xe27, 0xe2d, 0xe2a, 0xe15, 0xe2d, 0xe04, 0xe40, 0xe27, 0xe25, 0xe32,
0xe27, 0xe2d, 0xe25, 0xe42, 0xe01, 0xe01, 0xe23, 0xe32, 0xe14, 0xe40, 0xe27, 0xe25,
0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe27, 0xe2d, 0xe25, 0xe42,
0xe01, 0xe01, 0xe23, 0xe32, 0xe14, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39,
0xe23, 0xe49, 0xe2d, 0xe19, 0xe27, 0xe2d, 0xe25, 0xe42, 0xe01, 0xe01, 0xe23, 0xe32,
0xe14, 0xe40, 0xe27, 0xe25, 0xe32, 0xe27, 0xe2d, 0xe2a, 0xe15, 0xe2d, 0xe04, 0xe40,
0xe27, 0xe25, 0xe32, 0xe40, 0xe01, 0xe32, 0xe30, 0xe40, 0xe27, 0xe01, 0xe40, 0xe27,
0xe25, 0xe32, 0xe27, 0xe32, 0xe25, 0xe25, 0xe34, 0xe2a, 0xe41, 0xe25, 0xe30, 0xe1f,
0xe38, 0xe15, 0xe39, 0xe19, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe22, 0xe32, 0xe04,
0xe38, 0xe15, 0xe2a, 0xe04, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15,
0xe23, 0xe10, 0xe32, 0xe19, 0xe22, 0xe32, 0xe04, 0xe38, 0xe15, 0xe2a, 0xe04, 0xe4c,
0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe22,
0xe32, 0xe04, 0xe38, 0xe15, 0xe2a, 0xe04, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40,
0xe22, 0xe04, 0xe32, 0xe40, 0xe15, 0xe23, 0xe34, 0xe19, 0xe1a, 0xe39, 0xe23, 0xe4c,
0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19,
0xe40, 0xe22, 0xe04, 0xe32, 0xe40, 0xe15, 0xe23, 0xe34, 0xe19, 0xe1a, 0xe39, 0xe23,
0xe4c, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d,
0xe19, 0xe40, 0xe22, 0xe04, 0xe32, 0xe40, 0xe15, 0xe23, 0xe34, 0xe19, 0xe1a, 0xe39,
0xe23, 0xe4c, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe22, 0xe39, 0xe04, 0xe2d, 0xe19,
0x130d, 0x12d8, 0x20, 0x12a0, 0x12ad, 0x122a, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x12a3, 0x12ad, 0x122a, 0x130d, 0x12dc, 0x20, 0x12ad,
0x1228, 0x121d, 0x1272, 0x20, 0x12a3, 0x12ad, 0x122a, 0x1293, 0x12ed, 0x20, 0x12a0, 0x134d,
0x130b, 0x1292, 0x1235, 0x1273, 0x1295, 0x20, 0x130d, 0x12d8, 0x130d, 0x12dc, 0x20, 0x121b,
0x12a5, 0x12a8, 0x120b, 0x12ed, 0x20, 0x12a3, 0x134d, 0x122a, 0x1243, 0x130d, 0x12dc, 0x20,
0x121d, 0x1265, 0x122b, 0x1255, 0x20, 0x12a3, 0x134d, 0x122a, 0x1243, 0x130d, 0x12dc, 0x20,
0x12f0, 0x1261, 0x1265, 0x20, 0x12a3, 0x134d, 0x122a, 0x1243, 0x130d, 0x12dc, 0x20, 0x121d,
0x12d5, 0x122b, 0x1265, 0x20, 0x12a3, 0x134d, 0x122a, 0x1243, 0x1293, 0x12ed, 0x20, 0x1218,
0x12f0, 0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x121d, 0x12d5, 0x122b, 0x1265, 0x20,
0x12a3, 0x134d, 0x122a, 0x1243, 0x130d, 0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20,
0x121d, 0x12d5, 0x122b, 0x1265, 0x20, 0x12a3, 0x134d, 0x122a, 0x1243, 0x130d, 0x12d8, 0x20,
0x12a0, 0x120b, 0x1235, 0x12ab, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x130d, 0x12d8, 0x20, 0x12a0, 0x120b, 0x1235, 0x12ab, 0x1293, 0x12ed, 0x20, 0x1218, 0x12d3,
0x120d, 0x1272, 0x20, 0x130d, 0x12d8, 0x20, 0x12a0, 0x120b, 0x1235, 0x12ab, 0x130d, 0x12dc,
0x20, 0x12a3, 0x121b, 0x12de, 0x1295, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x130d, 0x12d8, 0x20, 0x12a3, 0x121b, 0x12de, 0x1295, 0x130d, 0x12dc, 0x20, 0x12ad,
0x1228, 0x121d, 0x1272, 0x20, 0x12a3, 0x121b, 0x12de, 0x1295, 0x121b, 0x12a5, 0x12a8, 0x120b,
0x12ed, 0x20, 0x12a0, 0x1218, 0x122a, 0x12ab, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x121b, 0x12a5, 0x12a8, 0x120b,
0x12ed, 0x20, 0x12a0, 0x1218, 0x122a, 0x12ab, 0x1293, 0x12ed, 0x20, 0x1218, 0x12d3, 0x120d,
0x1272, 0x20, 0x130d, 0x12d8, 0x20, 0x121b, 0x12a5, 0x12a8, 0x120b, 0x12ed, 0x20, 0x12a0,
0x1218, 0x122a, 0x12ab, 0x1293, 0x12ed, 0x20, 0x121d, 0x1265, 0x122b, 0x1253, 0x12ca, 0x20,
0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x130d, 0x12d8,
0x20, 0x121d, 0x1265, 0x122b, 0x1253, 0x12ca, 0x20, 0x12a3, 0x1218, 0x122a, 0x12ab, 0x1293,
0x12ed, 0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x130d, 0x12d8, 0x20, 0x121d, 0x1265,
0x122b, 0x1253, 0x12ca, 0x20, 0x12a0, 0x1218, 0x122a, 0x12ab, 0x1293, 0x12ed, 0x20, 0x130e,
0x1266, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x130d, 0x12d8, 0x20, 0x130e, 0x1266, 0x20, 0x12a3, 0x1218, 0x122a, 0x12ab, 0x1293, 0x12ed,
0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x130d, 0x12d8, 0x20, 0x130e, 0x1266, 0x20,
0x12a0, 0x1218, 0x122a, 0x12ab, 0x1293, 0x12ed, 0x20, 0x1353, 0x1235, 0x134a, 0x12ad, 0x20,
0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x130d, 0x12d8,
0x20, 0x1353, 0x1235, 0x134a, 0x12ad, 0x1293, 0x12ed, 0x20, 0x1218, 0x12d3, 0x120d, 0x1272,
0x20, 0x130d, 0x12d8, 0x20, 0x1353, 0x1235, 0x134a, 0x12ad, 0x1293, 0x12ed, 0x20, 0x12a0,
0x1352, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x12a0, 0x1352, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12d3,
0x120d, 0x1272, 0x20, 0x12a0, 0x1352, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x12a0, 0x1228, 0x1265, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x12a0, 0x1228, 0x1265, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218,
0x12d3, 0x120d, 0x1272, 0x20, 0x12a0, 0x1228, 0x1265, 0x20, 0x130d, 0x12d8, 0x130d, 0x12dc,
0x20, 0x12a3, 0x122d, 0x1300, 0x1295, 0x1272, 0x1293, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x12a3, 0x122d, 0x1300, 0x1295, 0x1272, 0x1293,
0x130d, 0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x12a3, 0x122d, 0x1300, 0x1295,
0x1272, 0x1293, 0x130d, 0x12dc, 0x20, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20, 0x12a3,
0x122d, 0x1300, 0x1295, 0x1272, 0x1293, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x130d, 0x12d8, 0x20, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20, 0x12a3, 0x122d,
0x1300, 0x1295, 0x1272, 0x1293, 0x130d, 0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20,
0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20, 0x12a3, 0x122d, 0x1300, 0x1295, 0x1272, 0x1293,
0x1293, 0x12ed, 0x20, 0x12a0, 0x122d, 0x121c, 0x1292, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293,
0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12a0, 0x122d, 0x121c, 0x1292, 0x12eb,
0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x12a0,
0x122d, 0x121c, 0x1292, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12a0, 0x1275,
0x120b, 0x1295, 0x1272, 0x12ad, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x12a0, 0x1275, 0x120b, 0x1295, 0x1272, 0x12ad,
0x1293, 0x12ed, 0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x130d, 0x12d8, 0x20, 0x12a0,
0x1275, 0x120b, 0x1295, 0x1272, 0x12ad, 0x1293, 0x12ed, 0x20, 0x12a0, 0x12cd, 0x1235, 0x122b,
0x120a, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121b, 0x12d5, 0x12a8, 0x120b,
0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12a0, 0x12cd, 0x1235, 0x122b, 0x120a,
0x12eb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121b, 0x12d5, 0x12a8, 0x120b, 0x12ed,
0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x12a0, 0x12cd, 0x1235, 0x122b, 0x120a, 0x12eb,
0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20,
0x12a0, 0x12cd, 0x1235, 0x122b, 0x120a, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12a0,
0x12cd, 0x1235, 0x122b, 0x120a, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121d,
0x12d5, 0x122b, 0x1263, 0x12ca, 0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x12a0, 0x12cd,
0x1235, 0x122b, 0x120a, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121d, 0x1265,
0x122b, 0x1253, 0x12ca, 0x20, 0x12a3, 0x12cd, 0x1235, 0x1275, 0x122b, 0x120d, 0x12eb, 0x20,
0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121d, 0x1265, 0x122b, 0x1253, 0x12ca, 0x20, 0x1218,
0x12f0, 0x1260, 0x129b, 0x20, 0x12a3, 0x12cd, 0x1235, 0x1275, 0x122b, 0x120d, 0x12eb, 0x20,
0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121d, 0x1265, 0x122b, 0x1253, 0x12ca, 0x20, 0x1218,
0x12d3, 0x120d, 0x1272, 0x20, 0x12a3, 0x12cd, 0x1235, 0x1275, 0x122b, 0x120d, 0x12eb, 0x20,
0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20, 0x12a0,
0x12cd, 0x1235, 0x1275, 0x122b, 0x120a, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12a0,
0x12cd, 0x1235, 0x1275, 0x122b, 0x120a, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x12a0,
0x12cd, 0x1235, 0x1275, 0x122b, 0x120a, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x12a0, 0x12d8, 0x122d, 0x1263, 0x12e3, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12a0, 0x12d8, 0x122d, 0x1263, 0x12e3, 0x1295, 0x20,
0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x12a0, 0x12d8,
0x122d, 0x1263, 0x12e3, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12a0, 0x12de,
0x1228, 0x1235, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x130d, 0x12d8, 0x20, 0x12a3, 0x12de, 0x1228, 0x1235, 0x1293, 0x12ed, 0x20, 0x12ad,
0x1228, 0x121d, 0x1272, 0x20, 0x12a0, 0x12de, 0x1228, 0x1235, 0x20, 0x130d, 0x12d8, 0x1293,
0x12ed, 0x20, 0x1263, 0x1295, 0x130d, 0x120b, 0x12f2, 0x123d, 0x20, 0x130d, 0x12d8, 0x1293,
0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1263, 0x1295, 0x130d, 0x120b, 0x12f2,
0x123d, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20,
0x1263, 0x1295, 0x130d, 0x120b, 0x12f2, 0x123d, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x1261, 0x1205, 0x1273, 0x1295, 0x20, 0x130d, 0x12d8, 0x130d, 0x12dc, 0x20, 0x1266, 0x120a,
0x126d, 0x12eb, 0x130d, 0x12dc, 0x20, 0x1265, 0x122b, 0x12da, 0x120d, 0x12eb, 0x1293, 0x12ed,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x1265, 0x122b, 0x12da,
0x120d, 0x12eb, 0x130d, 0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x1265, 0x122b,
0x12da, 0x120d, 0x12eb, 0x1293, 0x12ed, 0x20, 0x1265, 0x1229, 0x1294, 0x20, 0x12f3, 0x1229,
0x1233, 0x120c, 0x121d, 0x20, 0x130d, 0x12d8, 0x130d, 0x12dc, 0x20, 0x12ac, 0x1355, 0x20,
0x1268, 0x122d, 0x12f0, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x130d,
0x12d8, 0x20, 0x12ac, 0x1355, 0x20, 0x1268, 0x122d, 0x12f0, 0x130d, 0x12dc, 0x20, 0x12ad,
0x1228, 0x121d, 0x1272, 0x20, 0x12ac, 0x1355, 0x20, 0x1268, 0x122d, 0x12f0, 0x1293, 0x12ed,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x127b, 0x121e, 0x122e, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x127b, 0x1275, 0x1203, 0x121d, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x127b, 0x1275, 0x1203, 0x121d, 0x20, 0x130d,
0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x127b, 0x1275, 0x1203,
0x121d, 0x20, 0x130d, 0x12d8, 0x130d, 0x12dc, 0x20, 0x127a, 0x120c, 0x1293, 0x12ed, 0x20,
0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x127a, 0x120c, 0x130d, 0x12dc,
0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x127a, 0x120c, 0x1293, 0x12ed, 0x20, 0x127b,
0x12ed, 0x1293, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x127b, 0x12ed, 0x1293, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12d3,
0x120d, 0x1272, 0x20, 0x127b, 0x12ed, 0x1293, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x120d, 0x12f0, 0x1275, 0x20, 0x12f0, 0x1234, 0x1275, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed,
0x20, 0x12ae, 0x12ae, 0x1235, 0x20, 0x12f0, 0x1234, 0x1275, 0x20, 0x130d, 0x12d8, 0x130d,
0x12dc, 0x20, 0x12ae, 0x120e, 0x121d, 0x1265, 0x12eb, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x12ae, 0x120e, 0x121d, 0x1265, 0x12eb, 0x130d,
0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x12ae, 0x120e, 0x121d, 0x1265, 0x12eb,
0x1293, 0x12ed, 0x20, 0x12a9, 0x12ad, 0x20, 0x12f0, 0x1234, 0x1275, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12a9, 0x12ad, 0x20, 0x12f0,
0x1234, 0x1275, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x134d, 0x122d, 0x1242, 0x20,
0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x12a9, 0x12ad, 0x20, 0x12f0, 0x1234, 0x1275, 0x20,
0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12a9, 0x1263, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x12a9, 0x1263, 0x1293,
0x12ed, 0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x130d, 0x12d8, 0x20, 0x12a9, 0x1263,
0x1293, 0x12ed, 0x20, 0x12f4, 0x126a, 0x1235, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x12f1, 0x121e, 0x2d, 0x12f1, 0x122d, 0x126a, 0x120d, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed,
0x20, 0x121d, 0x1265, 0x122b, 0x1245, 0x20, 0x1272, 0x121e, 0x122d, 0x20, 0x130d, 0x12d8,
0x130d, 0x12dc, 0x20, 0x12f0, 0x1234, 0x1275, 0x20, 0x134b, 0x1232, 0x12ab, 0x1293, 0x12ed,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x12f0, 0x1234, 0x1275,
0x20, 0x134b, 0x1232, 0x12ab, 0x130d, 0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20,
0x12f0, 0x1234, 0x1275, 0x20, 0x134b, 0x1232, 0x12ab, 0x130d, 0x12dc, 0x20, 0x12a4, 0x12b3,
0x12f6, 0x122d, 0x1293, 0x12ed, 0x20, 0x121b, 0x12a5, 0x12a8, 0x120b, 0x12ed, 0x20, 0x12a4,
0x12cd, 0x122e, 0x1333, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x121b, 0x12a5, 0x12a8, 0x120b, 0x12ed, 0x20, 0x12a4,
0x12cd, 0x122e, 0x1333, 0x130d, 0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x12a4,
0x12cd, 0x122e, 0x1333, 0x1293, 0x12ed, 0x20, 0x121d, 0x1265, 0x122b, 0x1255, 0x20, 0x12a4,
0x12cd, 0x122e, 0x1333, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x121d, 0x1265, 0x122b, 0x1255, 0x20, 0x12a4, 0x12cd,
0x122e, 0x1333, 0x130d, 0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x121d, 0x1265,
0x122b, 0x1255, 0x20, 0x12a4, 0x12cd, 0x122e, 0x1333, 0x1293, 0x12ed, 0x20, 0x122d, 0x1211,
0x1255, 0x2d, 0x121d, 0x1265, 0x122b, 0x1255, 0x20, 0x12a4, 0x12cd, 0x122e, 0x1333, 0x12ca,
0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20,
0x12a4, 0x12cd, 0x122e, 0x1333, 0x12ca, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218,
0x12f0, 0x1260, 0x129b, 0x20, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20, 0x12a4, 0x12cd,
0x122e, 0x1333, 0x12ca, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d,
0x1272, 0x20, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20, 0x12a4, 0x12cd, 0x122e, 0x1333,
0x12ca, 0x20, 0x130d, 0x12d8, 0x130d, 0x12dc, 0x20, 0x12f0, 0x1234, 0x1273, 0x1275, 0x20,
0x134e, 0x12ad, 0x120b, 0x1295, 0x12f5, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x130d, 0x12d8, 0x20, 0x12f0, 0x1234, 0x1273, 0x1275, 0x20, 0x134e, 0x12ad, 0x120b,
0x1295, 0x12f5, 0x130d, 0x12dc, 0x20, 0x12a8, 0x1228, 0x121d, 0x1272, 0x20, 0x12f0, 0x1234,
0x1273, 0x1275, 0x20, 0x134e, 0x12ad, 0x120b, 0x1295, 0x12f5, 0x1293, 0x12ed, 0x20, 0x134a,
0x1302, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x134a, 0x1302, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272,
0x20, 0x134a, 0x1302, 0x20, 0x130d, 0x12d8, 0x130d, 0x12dc, 0x20, 0x1348, 0x1228, 0x1295,
0x1233, 0x12ca, 0x1275, 0x20, 0x130a, 0x12eb, 0x1293, 0x1293, 0x12ed, 0x20, 0x12f0, 0x1261,
0x1263, 0x12ca, 0x1295, 0x20, 0x12a3, 0x1295, 0x1273, 0x122d, 0x1272, 0x12ad, 0x20, 0x130d,
0x12d8, 0x130d, 0x12dc, 0x20, 0x130b, 0x120b, 0x1353, 0x130e, 0x1235, 0x1293, 0x12ed, 0x20,
0x130b, 0x121d, 0x1262, 0x12e8, 0x122d, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1306,
0x122d, 0x1302, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x1306, 0x122d, 0x1302, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x1306, 0x122d, 0x1302, 0x12eb, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x130a, 0x120d, 0x1260, 0x122d, 0x1275, 0x20, 0x12f0, 0x1234, 0x1273,
0x1275, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121d, 0x1265, 0x122b, 0x1253, 0x12ca,
0x20, 0x130d, 0x12d8, 0x20, 0x130d, 0x122a, 0x1295, 0x120b, 0x1295, 0x12f5, 0x1293, 0x12ed,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x121d, 0x1265, 0x122b, 0x1253, 0x12ca, 0x20,
0x130d, 0x12d8, 0x20, 0x130d, 0x122a, 0x1295, 0x120b, 0x1295, 0x12f5, 0x1293, 0x12ed, 0x20,
0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x121d, 0x1265, 0x122b, 0x1253, 0x12ca, 0x20, 0x130d,
0x12d8, 0x20, 0x130d, 0x122a, 0x1295, 0x120b, 0x1295, 0x12f5, 0x1293, 0x12ed, 0x20, 0x121d,
0x12d5, 0x122b, 0x1265, 0x20, 0x130d, 0x122a, 0x1295, 0x120b, 0x1295, 0x12f5, 0x20, 0x130d,
0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x121d, 0x12d5, 0x122b,
0x1265, 0x20, 0x130d, 0x122a, 0x1295, 0x120b, 0x1295, 0x12f5, 0x20, 0x130d, 0x12d8, 0x1293,
0x12ed, 0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x121d, 0x12d5, 0x122b, 0x1265, 0x20,
0x130d, 0x122a, 0x1295, 0x120b, 0x1295, 0x12f5, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1308, 0x120d, 0x134d, 0x20, 0x130d, 0x12d8, 0x130d,
0x12dc, 0x20, 0x1309, 0x12eb, 0x1293, 0x1293, 0x12ed, 0x20, 0x1203, 0x12cb, 0x12ed, 0x2d,
0x12a3, 0x120c, 0x12cd, 0x1272, 0x12eb, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1203, 0x12cb, 0x12ed, 0x2d, 0x12a3, 0x120c, 0x12cd,
0x1272, 0x12eb, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12d3, 0x120d,
0x1272, 0x20, 0x1203, 0x12cb, 0x12ed, 0x2d, 0x12a3, 0x120c, 0x12cd, 0x1272, 0x12eb, 0x1295,
0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1206, 0x1295, 0x130d, 0x20, 0x12ae, 0x1295,
0x130d, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x1206, 0x1295, 0x130d, 0x20, 0x12ae, 0x1295, 0x130d, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed,
0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x1206, 0x1295, 0x130d, 0x20, 0x12ae, 0x1295,
0x130d, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1206, 0x126d, 0x12f5, 0x20, 0x130d,
0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1206, 0x126d, 0x12f5,
0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x1206,
0x126d, 0x12f5, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x1205, 0x1295, 0x12f5, 0x20, 0x130d, 0x12d8, 0x130d, 0x12dc, 0x20, 0x1205, 0x1295,
0x12f3, 0x12ca, 0x20, 0x12cd, 0x1245, 0x12eb, 0x1296, 0x1235, 0x1293, 0x12ed, 0x20, 0x12a2,
0x1295, 0x12f6, 0x127b, 0x12ed, 0x1293, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121b,
0x12a5, 0x12a8, 0x120b, 0x12ed, 0x20, 0x12a2, 0x1295, 0x12f6, 0x1290, 0x12e5, 0x12eb, 0x20,
0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121d, 0x1265, 0x122b, 0x1253, 0x12ca, 0x20, 0x12a2,
0x1295, 0x12f6, 0x1290, 0x12e5, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121d,
0x12d5, 0x122b, 0x1263, 0x12ca, 0x20, 0x12a2, 0x1295, 0x12f6, 0x1290, 0x12e5, 0x12eb, 0x20,
0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12a2, 0x122b, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293,
0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12a2, 0x122b, 0x1295, 0x20, 0x130d,
0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x12a2, 0x122b, 0x1295,
0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12a2, 0x122d, 0x12a9, 0x1275, 0x1235, 0x12ad,
0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12a2,
0x122d, 0x12a9, 0x1275, 0x1235, 0x12ad, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad,
0x1228, 0x121d, 0x1272, 0x20, 0x12a2, 0x122d, 0x12a9, 0x1275, 0x1235, 0x12ad, 0x20, 0x130d,
0x12d8, 0x1293, 0x12ed, 0x20, 0x12a5, 0x1235, 0x122b, 0x12a4, 0x120d, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12a5, 0x1235, 0x122b, 0x12a4,
0x120d, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20,
0x12a5, 0x1235, 0x122b, 0x12a4, 0x120d, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1303,
0x1353, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x1303, 0x1353, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12d3,
0x120d, 0x1272, 0x20, 0x1303, 0x1353, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x12ab, 0x12db, 0x12aa, 0x1235, 0x1273, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x121d, 0x1265, 0x122b, 0x1245, 0x20, 0x12ab, 0x12db, 0x12aa, 0x1235, 0x1273, 0x1295, 0x20,
0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121d, 0x12d5, 0x122b, 0x1265, 0x20, 0x12ab, 0x12db,
0x12aa, 0x1235, 0x1273, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ae, 0x122a,
0x12eb, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x12ae, 0x122a, 0x12eb, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218,
0x12d3, 0x120d, 0x1272, 0x20, 0x12ae, 0x122a, 0x12eb, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293,
0x12ed, 0x20, 0x12ae, 0x122d, 0x1233, 0x12ed, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x12ad, 0x122b, 0x1295, 0x1235, 0x1296, 0x12eb, 0x122d, 0x12ad, 0x20, 0x130d, 0x12d8, 0x1293,
0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12ad, 0x122b, 0x1295, 0x1235, 0x1296,
0x12eb, 0x122d, 0x12ad, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d,
0x1272, 0x20, 0x12ad, 0x122b, 0x1295, 0x1235, 0x1296, 0x12eb, 0x122d, 0x12ad, 0x20, 0x130d,
0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x122d, 0x1305, 0x1235, 0x1273, 0x1295, 0x20, 0x130d,
0x12d8, 0x1293, 0x12ed, 0x20, 0x120b, 0x12ed, 0x1295, 0x20, 0x12f0, 0x1234, 0x1273, 0x1275,
0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x120e, 0x122d, 0x12f5, 0x20, 0x1206, 0x12cd,
0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f5, 0x1260, 0x129b, 0x20, 0x120e,
0x122d, 0x12f5, 0x20, 0x1206, 0x12cd, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218,
0x12d3, 0x120d, 0x1272, 0x20, 0x120e, 0x122d, 0x12f5, 0x20, 0x1206, 0x12cd, 0x20, 0x130d,
0x12d8, 0x1293, 0x12ed, 0x20, 0x121c, 0x130b, 0x12f3, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293,
0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x121c, 0x130b, 0x12f3, 0x1295, 0x20,
0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x121c, 0x130b,
0x12f3, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121b, 0x120c, 0x12e2, 0x12eb,
0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121e, 0x120d, 0x12f2, 0x126d, 0x1235, 0x20,
0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121b, 0x122d, 0x12a9, 0x12cc, 0x1233, 0x1235, 0x20,
0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121b, 0x122d, 0x123b, 0x120d, 0x20, 0x12f0, 0x1234,
0x1273, 0x1275, 0x20, 0x130d, 0x12d8, 0x130d, 0x12dc, 0x20, 0x121b, 0x12cd, 0x122a, 0x1238,
0x1235, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20,
0x121b, 0x12cd, 0x122a, 0x1238, 0x1235, 0x130d, 0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272,
0x20, 0x121b, 0x12cd, 0x122a, 0x1238, 0x1235, 0x1293, 0x12ed, 0x20, 0x121b, 0x12cd, 0x1236,
0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121c, 0x12ad, 0x1232, 0x12ae, 0x20,
0x1353, 0x1235, 0x134a, 0x12ad, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x121c, 0x12ad, 0x1232, 0x12ae, 0x20, 0x1353, 0x1235, 0x134a, 0x12ad,
0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x121c,
0x12ad, 0x1232, 0x12ae, 0x20, 0x1353, 0x1235, 0x134a, 0x12ad, 0x20, 0x130d, 0x12d8, 0x1293,
0x12ed, 0x20, 0x12a1, 0x120b, 0x1295, 0x1263, 0x1273, 0x122d, 0x20, 0x130d, 0x12d8, 0x1293,
0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12a1, 0x120b, 0x1295, 0x1263, 0x1273,
0x122d, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20,
0x12a1, 0x120b, 0x1295, 0x1263, 0x1273, 0x122d, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x121e, 0x1235, 0x12ae, 0x12cd, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x121e, 0x1235, 0x12ae, 0x12cd, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed,
0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x121e, 0x1235, 0x12ae, 0x12cd, 0x20, 0x130d,
0x12d8, 0x1293, 0x12ed, 0x20, 0x121d, 0x12eb, 0x1295, 0x121b, 0x122d, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x1293, 0x12cd, 0x1229, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x1294, 0x1353, 0x120d, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1292, 0x12cd, 0x20,
0x12ab, 0x120c, 0x12f6, 0x1292, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218,
0x12f0, 0x1260, 0x129b, 0x20, 0x1292, 0x12cd, 0x20, 0x12ab, 0x120c, 0x12f6, 0x1292, 0x12eb,
0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x1292,
0x12cd, 0x20, 0x12ab, 0x120c, 0x12f6, 0x1292, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed,
0x20, 0x1292, 0x12cd, 0x20, 0x12da, 0x120b, 0x1295, 0x12f5, 0x20, 0x130d, 0x12d8, 0x1293,
0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1292, 0x12cd, 0x20, 0x12da, 0x120b,
0x1295, 0x12f5, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12d3, 0x120d, 0x1272,
0x20, 0x1292, 0x12cd, 0x20, 0x12da, 0x120b, 0x1295, 0x12f5, 0x20, 0x130d, 0x12d8, 0x1293,
0x12ed, 0x20, 0x1292, 0x12cd, 0x134b, 0x12cd, 0x1295, 0x12f5, 0x120b, 0x1295, 0x12f5, 0x20,
0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1292, 0x12cd,
0x134b, 0x12cd, 0x1295, 0x12f5, 0x120b, 0x1295, 0x12f5, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed,
0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x1292, 0x12cd, 0x134b, 0x12cd, 0x1295, 0x12f5,
0x120b, 0x1295, 0x12f5, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1292, 0x12cc, 0x20,
0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1296, 0x122d, 0x134e, 0x120d, 0x12ad, 0x20, 0x12f0,
0x1234, 0x1275, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x1296, 0x122d, 0x134e, 0x120d, 0x12ad, 0x20, 0x12f0, 0x1234, 0x1275, 0x20, 0x130d,
0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x1296, 0x122d, 0x134e,
0x120d, 0x12ad, 0x20, 0x12f0, 0x1234, 0x1275, 0x20, 0x130d, 0x12d8, 0x130d, 0x12dc, 0x20,
0x1348, 0x122d, 0x1293, 0x1295, 0x12f6, 0x20, 0x12f0, 0x20, 0x1296, 0x122e, 0x1295, 0x12eb,
0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x1348,
0x122d, 0x1293, 0x1295, 0x12f6, 0x20, 0x12f0, 0x20, 0x1296, 0x122e, 0x1295, 0x12eb, 0x130d,
0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x1348, 0x122d, 0x1293, 0x1295, 0x12f6,
0x20, 0x12f0, 0x20, 0x1296, 0x122e, 0x1295, 0x12eb, 0x1293, 0x12ed, 0x20, 0x1296, 0x126e,
0x1232, 0x122a, 0x1235, 0x12ad, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x1296, 0x126e, 0x1232, 0x122a, 0x1235, 0x12ad, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x1296, 0x126e, 0x1232, 0x122a,
0x1235, 0x12ad, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12a6, 0x121d, 0x1235, 0x12ad,
0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12a6,
0x121d, 0x1235, 0x12ad, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d,
0x1272, 0x20, 0x12a6, 0x121d, 0x1235, 0x12ad, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x1353, 0x12aa, 0x1235, 0x1273, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218,
0x12f0, 0x1260, 0x129b, 0x20, 0x1353, 0x12aa, 0x1235, 0x1273, 0x1295, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x1353, 0x12aa, 0x1235, 0x1273,
0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1353, 0x120b, 0x12cd, 0x20, 0x130d,
0x12d8, 0x1293, 0x12ed, 0x20, 0x1353, 0x1357, 0x20, 0x1292, 0x12cd, 0x20, 0x130a, 0x1292,
0x20, 0x130d, 0x12d8, 0x130d, 0x12dc, 0x20, 0x1353, 0x122b, 0x1313, 0x12ed, 0x1293, 0x12ed,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x1353, 0x122b, 0x1313,
0x12ed, 0x130d, 0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x1353, 0x122b, 0x1313,
0x12ed, 0x130d, 0x12dc, 0x20, 0x1354, 0x1229, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x1354, 0x1229, 0x130d, 0x12dc, 0x20, 0x12ad, 0x1228,
0x121d, 0x1272, 0x20, 0x1354, 0x1229, 0x1293, 0x12ed, 0x20, 0x134a, 0x120a, 0x1352, 0x1295,
0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x134a,
0x120a, 0x1352, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d,
0x1272, 0x20, 0x134a, 0x120a, 0x1352, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x134a, 0x1292, 0x12ad, 0x1235, 0x20, 0x12f0, 0x1234, 0x1273, 0x1275, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x1245, 0x12f1, 0x1235, 0x20, 0x1352, 0x12e8, 0x122d, 0x1295, 0x20,
0x121a, 0x12a8, 0x120e, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x1245, 0x12f1, 0x1235, 0x20, 0x1352, 0x12e8, 0x122d, 0x1295, 0x20,
0x121a, 0x12a8, 0x120e, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12d3,
0x120d, 0x1272, 0x20, 0x1245, 0x12f1, 0x1235, 0x20, 0x1352, 0x12e8, 0x122d, 0x1295, 0x20,
0x121a, 0x12a8, 0x120e, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1352, 0x1275,
0x127b, 0x12ed, 0x122d, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1356, 0x1293,
0x1354, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1355, 0x12ee, 0x1295, 0x130d, 0x12eb,
0x1295, 0x130d, 0x20, 0x130d, 0x12d8, 0x130d, 0x12dc, 0x20, 0x122d, 0x12e9, 0x1295, 0x12e8,
0x1295, 0x1293, 0x12ed, 0x20, 0x122e, 0x1274, 0x122b, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed,
0x20, 0x1233, 0x12ad, 0x1203, 0x120a, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1233, 0x12ad, 0x1203, 0x120a, 0x1295, 0x20, 0x130d,
0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x1233, 0x12ad, 0x1203,
0x120a, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1233, 0x121e, 0x12cb, 0x20,
0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1233, 0x121e,
0x12cb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20,
0x1233, 0x121e, 0x12cb, 0x20, 0x130d, 0x12d8, 0x130d, 0x12dc, 0x20, 0x1232, 0x1238, 0x120d,
0x1235, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1232, 0x130b, 0x1356,
0x122d, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1236, 0x120e, 0x121e, 0x1295, 0x20,
0x12f0, 0x1234, 0x1273, 0x1275, 0x20, 0x130d, 0x12d8, 0x130d, 0x12dc, 0x20, 0x12f0, 0x1261,
0x1265, 0x20, 0x1306, 0x122d, 0x1305, 0x12eb, 0x130d, 0x12dc, 0x20, 0x1231, 0x122a, 0x1293,
0x121d, 0x1293, 0x12ed, 0x20, 0x1235, 0x12ee, 0x12cb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed,
0x20, 0x1272, 0x1202, 0x1272, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1274, 0x1352,
0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1274,
0x1352, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20,
0x1274, 0x1352, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1273, 0x1303, 0x12ad, 0x1235,
0x1273, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1276, 0x12ac, 0x120b, 0x12cd,
0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1276, 0x1295, 0x130b, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1276, 0x1295, 0x130b, 0x20,
0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x1276, 0x1295,
0x130b, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1279, 0x12ad, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x1271, 0x122d, 0x12ad, 0x121c, 0x1292, 0x1235, 0x1273, 0x1295, 0x20,
0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1271, 0x122d,
0x12ad, 0x121c, 0x1292, 0x1235, 0x1273, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x1271, 0x122d, 0x12ad, 0x121c, 0x1292, 0x1235, 0x1273,
0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1271, 0x126b, 0x1209, 0x20, 0x130d,
0x12d8, 0x130d, 0x12dc, 0x20, 0x12a1, 0x122b, 0x1313, 0x12ed, 0x1293, 0x12ed, 0x20, 0x1218,
0x12f0, 0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x12a1, 0x122b, 0x1313, 0x12ed, 0x130d,
0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x12a1, 0x122b, 0x1313, 0x12ed, 0x1293,
0x12ed, 0x20, 0x12a1, 0x12dd, 0x1264, 0x12aa, 0x1235, 0x1273, 0x1295, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12a1, 0x12dd, 0x1264, 0x12aa,
0x1235, 0x1273, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d,
0x1272, 0x20, 0x12a1, 0x12dd, 0x1264, 0x12aa, 0x1235, 0x1273, 0x1295, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x126b, 0x1297, 0x1273, 0x12cd, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x126b, 0x1297, 0x1273, 0x12cd, 0x20, 0x130d,
0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x126b, 0x1297, 0x1273,
0x12cd, 0x20, 0x130d, 0x12d8, 0x130d, 0x12dc, 0x20, 0x126c, 0x1294, 0x12dd, 0x12cc, 0x120b,
0x1293, 0x12ed, 0x20, 0x126d, 0x120b, 0x12f2, 0x126e, 0x1235, 0x1276, 0x12ad, 0x20, 0x130d,
0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x126d, 0x120b, 0x12f2,
0x126e, 0x1235, 0x1276, 0x12ad, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228,
0x121d, 0x1272, 0x20, 0x126d, 0x120b, 0x12f2, 0x126e, 0x1235, 0x1276, 0x12ad, 0x20, 0x130d,
0x12d8, 0x1293, 0x12ed, 0x20, 0x126e, 0x120d, 0x130e, 0x130d, 0x122b, 0x12f5, 0x20, 0x130d,
0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x126e, 0x120d, 0x130e,
0x130d, 0x122b, 0x12f5, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d,
0x1272, 0x20, 0x126e, 0x120d, 0x130e, 0x130d, 0x122b, 0x12f5, 0x20, 0x130d, 0x12d8, 0x1293,
0x12ed, 0x20, 0x126e, 0x1235, 0x1276, 0x12ad, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x12cc, 0x12ad, 0x20, 0x12f0, 0x1234, 0x1275, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x12cc, 0x120d, 0x1235, 0x1295, 0x20, 0x1349, 0x1277, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed,
0x20, 0x12eb, 0x12a9, 0x1275, 0x1235, 0x12ad, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12eb, 0x12a9, 0x1275, 0x1235, 0x12ad, 0x20, 0x130d,
0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x12eb, 0x12a9, 0x1275,
0x1235, 0x12ad, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12eb, 0x12ad, 0x1270, 0x122d,
0x1292, 0x1260, 0x122d, 0x130d, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x12eb, 0x12ad, 0x1270, 0x122d, 0x1292, 0x1260, 0x122d, 0x130d, 0x20,
0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x12eb, 0x12ad,
0x1270, 0x122d, 0x1292, 0x1260, 0x122d, 0x130d, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x12e9, 0x12ae, 0x1295, 0x20, 0x130d, 0x12d8, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6b, 0x65, 0x6c, 0x12b, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6b, 0x65, 0x6c, 0x12b,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61,
0x6b, 0x65, 0x6c, 0x12b, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c,
0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x61, 0x66, 0x69, 0x6b, 0x101, 0x6e, 0x69, 0x73, 0x69, 0x74,
0x61, 0x6e, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x2bb, 0x61, 0x66, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x6c, 0x6f, 0x74,
0x6f, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61,
0x66, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x68, 0x61, 0x68, 0x61, 0x6b,
0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61,
0x66, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f, 0x6e, 0x67, 0x61,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x66,
0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x66,
0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61,
0x66, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66,
0x6f, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61,
0x6c, 0x61, 0x73, 0x69, 0x6b, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6c, 0x61, 0x73, 0x69, 0x6b, 0x61, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6c,
0x61, 0x73, 0x69, 0x6b, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x2bb, 0x61, 0x6c, 0x61, 0x6d, 0x61, 0x74, 0x69, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6c, 0x61, 0x6d,
0x61, 0x74, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f,
0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x61, 0x6c, 0x61, 0x6d, 0x61, 0x74, 0x69, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6d, 0x61, 0x73, 0x14d,
0x6e, 0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb,
0x61, 0x6d, 0x61, 0x73, 0x14d, 0x6e, 0x65, 0x20, 0x74, 0x61, 0x69, 0x6d,
0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6d, 0x61, 0x73, 0x14d, 0x6e,
0x65, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61,
0x6d, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f, 0x6b, 0x65, 0x6c,
0x61, 0x75, 0x20, 0x6c, 0x6f, 0x74, 0x6f, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6d, 0x65, 0x6c, 0x69, 0x6b, 0x61,
0x2d, 0x74, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x6c, 0x6f, 0x74,
0x6f, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f,
0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb,
0x61, 0x6d, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f, 0x6b, 0x65,
0x6c, 0x61, 0x75, 0x20, 0x6c, 0x6f, 0x74, 0x6f, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6d, 0x65, 0x6c, 0x69, 0x6b,
0x61, 0x2d, 0x74, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x68, 0x61,
0x68, 0x61, 0x6b, 0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x61, 0x6d, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f,
0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x68, 0x61, 0x68, 0x61, 0x6b, 0x65,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61,
0x6d, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f, 0x6b, 0x65, 0x6c,
0x61, 0x75, 0x20, 0x68, 0x61, 0x68, 0x61, 0x6b, 0x65, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6d, 0x65, 0x6c, 0x69,
0x6b, 0x61, 0x2d, 0x74, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x6d,
0x6f, 0x2bb, 0x75, 0x6e, 0x67, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6d, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d,
0x74, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x6d, 0x6f, 0x2bb, 0x75,
0x6e, 0x67, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f,
0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x61, 0x6d, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f,
0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x6d, 0x6f, 0x2bb, 0x75, 0x6e, 0x67,
0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61,
0x6d, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f, 0x6b, 0x65, 0x6c,
0x61, 0x75, 0x20, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6d, 0x65,
0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75,
0x20, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6d, 0x65, 0x6c,
0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20,
0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x2bb,
0x61, 0x6e, 0x61, 0x74, 0x69, 0x6c, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x2bb, 0x61,
0x6e, 0x61, 0x74, 0x69, 0x6c, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x2bb, 0x61,
0x6e, 0x61, 0x74, 0x69, 0x6c, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x61, 0x70, 0x69, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x61, 0x70, 0x69, 0x61, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x61, 0x70, 0x69, 0x61, 0x20, 0x74,
0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6b, 0x69, 0x74,
0x61, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb,
0x61, 0x6b, 0x69, 0x74, 0x61, 0x75, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6b, 0x69, 0x74, 0x61, 0x75, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6b, 0x69,
0x74, 0x14d, 0x70, 0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x61, 0x6b, 0x69, 0x74, 0x14d, 0x70, 0x65, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6b, 0x69, 0x74,
0x14d, 0x70, 0x65, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69,
0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x2bb, 0x61, 0x6c, 0x65, 0x70, 0x65, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6c, 0x65, 0x70, 0x65, 0x61, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6c,
0x65, 0x70, 0x65, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c,
0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x61, 0x73, 0x65, 0x6e, 0x69, 0x74, 0x69, 0x6e, 0x61, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x73, 0x65,
0x6e, 0x69, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x73, 0x65, 0x6e, 0x69, 0x74, 0x69,
0x6e, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c,
0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb,
0x61, 0x73, 0x65, 0x6e, 0x69, 0x74, 0x69, 0x6e, 0x61, 0x2d, 0x68, 0x69,
0x68, 0x69, 0x66, 0x6f, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x61, 0x73, 0x65, 0x6e, 0x69, 0x74, 0x69, 0x6e, 0x61, 0x2d,
0x68, 0x69, 0x68, 0x69, 0x66, 0x6f, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x73, 0x65, 0x6e, 0x69, 0x74, 0x69,
0x6e, 0x61, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x101, 0x6d, 0x65, 0x6e, 0x69,
0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x101,
0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x2bb, 0x101, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x74,
0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6d, 0x65, 0x6c,
0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20,
0x2bb, 0x61, 0x74, 0x61, 0x6c, 0x61, 0x6e, 0x69, 0x74, 0x69, 0x6b, 0x69,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6d,
0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f, 0x6b, 0x65, 0x6c, 0x61,
0x75, 0x20, 0x2bb, 0x61, 0x74, 0x61, 0x6c, 0x61, 0x6e, 0x69, 0x74, 0x69,
0x6b, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74,
0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x2bb, 0x61, 0x6d, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f, 0x6b,
0x65, 0x6c, 0x61, 0x75, 0x20, 0x2bb, 0x61, 0x74, 0x61, 0x6c, 0x61, 0x6e,
0x69, 0x74, 0x69, 0x6b, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x2bb, 0x61, 0x6f, 0x73, 0x69, 0x74, 0x65, 0x6c, 0x113, 0x6c,
0x69, 0x61, 0x2d, 0x6c, 0x6f, 0x74, 0x6f, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6f, 0x73, 0x69, 0x74, 0x65, 0x6c,
0x113, 0x6c, 0x69, 0x61, 0x2d, 0x6c, 0x6f, 0x74, 0x6f, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6f, 0x73, 0x69,
0x74, 0x65, 0x6c, 0x113, 0x6c, 0x69, 0x61, 0x2d, 0x6c, 0x6f, 0x74, 0x6f,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6f,
0x73, 0x69, 0x74, 0x65, 0x6c, 0x113, 0x6c, 0x69, 0x61, 0x2d, 0x6c, 0x6f,
0x74, 0x6f, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6f, 0x73, 0x69, 0x74,
0x65, 0x6c, 0x113, 0x6c, 0x69, 0x61, 0x2d, 0x6c, 0x6f, 0x74, 0x6f, 0x2d,
0x68, 0x69, 0x68, 0x69, 0x66, 0x6f, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6f, 0x73, 0x69, 0x74, 0x65, 0x6c,
0x113, 0x6c, 0x69, 0x61, 0x2d, 0x6c, 0x6f, 0x74, 0x6f, 0x2d, 0x68, 0x69,
0x68, 0x69, 0x66, 0x6f, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c,
0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x61, 0x6f, 0x73, 0x69, 0x74, 0x65, 0x6c, 0x113, 0x6c, 0x69,
0x61, 0x2d, 0x68, 0x61, 0x68, 0x61, 0x6b, 0x65, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6f, 0x73, 0x69, 0x74, 0x65,
0x6c, 0x113, 0x6c, 0x69, 0x61, 0x2d, 0x68, 0x61, 0x68, 0x61, 0x6b, 0x65,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61,
0x6f, 0x73, 0x69, 0x74, 0x65, 0x6c, 0x113, 0x6c, 0x69, 0x61, 0x2d, 0x68,
0x61, 0x68, 0x61, 0x6b, 0x65, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x2bb, 0x61, 0x6f, 0x73, 0x69, 0x74, 0x65, 0x6c, 0x113, 0x6c,
0x69, 0x61, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6f, 0x73, 0x69, 0x74,
0x65, 0x6c, 0x113, 0x6c, 0x69, 0x61, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66,
0x6f, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f,
0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb,
0x61, 0x6f, 0x73, 0x69, 0x74, 0x65, 0x6c, 0x113, 0x6c, 0x69, 0x61, 0x2d,
0x68, 0x69, 0x68, 0x69, 0x66, 0x6f, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x73, 0x61, 0x70, 0x61, 0x69, 0x73, 0x61,
0x6e, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb,
0x61, 0x73, 0x61, 0x70, 0x61, 0x69, 0x73, 0x61, 0x6e, 0x69, 0x20, 0x74,
0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x73, 0x61,
0x70, 0x61, 0x69, 0x73, 0x61, 0x6e, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d,
0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x101, 0x73, 0x6f, 0x6c, 0x65, 0x73, 0x69,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x101, 0x73,
0x6f, 0x6c, 0x65, 0x73, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x2bb, 0x101, 0x73, 0x6f, 0x6c, 0x65, 0x73, 0x69, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x101, 0x6e, 0x67,
0x69, 0x6c, 0x101, 0x74, 0x65, 0x73, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x70, 0x101, 0x6e, 0x67, 0x69, 0x6c, 0x101, 0x74,
0x65, 0x73, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f,
0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x70, 0x101, 0x6e, 0x67, 0x69, 0x6c, 0x101, 0x74, 0x65, 0x73, 0x69,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x16b, 0x74,
0x61, 0x6e, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x70, 0x6f, 0x6c, 0x12b, 0x76, 0x69, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x70, 0x61, 0x6c, 0x101, 0x73, 0x69, 0x6c, 0x61,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x61, 0x6c,
0x101, 0x73, 0x69, 0x6c, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x70, 0x61, 0x6c, 0x101, 0x73, 0x69, 0x6c, 0x61, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x75, 0x6c, 0x75,
0x6e, 0x65, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6d, 0x75, 0x69, 0x2bb, 0x69, 0x2d, 0x76, 0x113, 0x74, 0x65, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x75, 0x69, 0x2bb, 0x69,
0x2d, 0x76, 0x113, 0x74, 0x65, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x6d, 0x75, 0x69, 0x2bb, 0x69, 0x2d, 0x76, 0x113, 0x74,
0x65, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x65,
0x65, 0x73, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6b, 0x61, 0x6d, 0x6f, 0x6c, 0x6f, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x73, 0x61, 0x74, 0x69, 0x68, 0x61, 0x6d, 0x69, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x61, 0x74, 0x69,
0x68, 0x61, 0x6d, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74,
0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x73, 0x61, 0x74, 0x69, 0x68, 0x61, 0x6d, 0x69, 0x20, 0x74,
0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x69, 0x6c, 0x69,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c,
0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73,
0x69, 0x61, 0x69, 0x6e, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x73, 0x69, 0x61, 0x69, 0x6e, 0x61, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x69, 0x61, 0x69, 0x6e, 0x61,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x6f, 0x74,
0x75, 0x6b, 0x69, 0x6c, 0x69, 0x73, 0x69, 0x6d, 0x61, 0x73, 0x69, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x6f, 0x74, 0x75,
0x6b, 0x6f, 0x6b, 0x6f, 0x73, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x6b, 0x6f, 0x6c, 0x6f, 0x6d, 0x69, 0x70, 0x69, 0x61,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x6f, 0x6c,
0x6f, 0x6d, 0x69, 0x70, 0x69, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x6b, 0x6f, 0x6c, 0x6f, 0x6d, 0x69, 0x70, 0x69,
0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x6f,
0x74, 0x75, 0x6d, 0x6f, 0x74, 0x75, 0x6b, 0x75, 0x6b, 0x69, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x6f, 0x74, 0x75, 0x6d,
0x6f, 0x74, 0x75, 0x6b, 0x75, 0x6b, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d,
0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x6f, 0x74, 0x75, 0x6d, 0x6f, 0x74,
0x75, 0x6b, 0x75, 0x6b, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x6b, 0x69, 0x75, 0x70, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x6b, 0x69, 0x75, 0x70, 0x61, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x69, 0x75, 0x70, 0x61,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x74, 0x61, 0x76,
0x69, 0x73, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x74, 0x16b, 0x6d, 0x6f, 0x6e, 0x69, 0x2d, 0x74, 0x16b, 0x76, 0x69, 0x6c,
0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x74, 0x69,
0x6d, 0x6f, 0x61, 0x2d, 0x68, 0x61, 0x68, 0x61, 0x6b, 0x65, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x6e,
0x75, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c,
0x61, 0x70, 0x61, 0x6e, 0x75, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x6e, 0x75, 0x69, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x65, 0x6b, 0x75,
0x65, 0x74, 0x6f, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x65, 0x75, 0x6c, 0x6f, 0x70, 0x65, 0x2d, 0x6c, 0x6f, 0x74,
0x6f, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x65,
0x75, 0x6c, 0x6f, 0x70, 0x65, 0x2d, 0x6c, 0x6f, 0x74, 0x6f, 0x20, 0x74,
0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x65, 0x75, 0x6c,
0x6f, 0x70, 0x65, 0x2d, 0x6c, 0x6f, 0x74, 0x6f, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x65, 0x75, 0x6c, 0x6f, 0x70, 0x65,
0x2d, 0x68, 0x61, 0x68, 0x61, 0x6b, 0x65, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x65, 0x75, 0x6c, 0x6f, 0x70, 0x65, 0x2d,
0x68, 0x61, 0x68, 0x61, 0x6b, 0x65, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x65, 0x75, 0x6c, 0x6f, 0x70, 0x65, 0x2d,
0x68, 0x61, 0x68, 0x61, 0x6b, 0x65, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x2bb, 0x65, 0x75, 0x6c, 0x6f, 0x70, 0x65, 0x2d, 0x68,
0x61, 0x68, 0x61, 0x6b, 0x65, 0x2d, 0x61, 0x6e, 0x67, 0x65, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x65, 0x75, 0x6c, 0x6f,
0x70, 0x65, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x65, 0x75, 0x6c, 0x6f, 0x70,
0x65, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x65, 0x75, 0x6c, 0x6f, 0x70,
0x65, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x6f, 0x74, 0x75, 0x6d, 0x6f, 0x74,
0x75, 0x2d, 0x66, 0x6f, 0x6b, 0x75, 0x6c, 0x61, 0x6e, 0x69, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x6f, 0x74, 0x75, 0x6d,
0x6f, 0x74, 0x75, 0x2d, 0x66, 0x6f, 0x6b, 0x75, 0x6c, 0x61, 0x6e, 0x69,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x6f,
0x74, 0x75, 0x6d, 0x6f, 0x74, 0x75, 0x2d, 0x66, 0x6f, 0x6b, 0x75, 0x6c,
0x61, 0x6e, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69,
0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x66, 0x69, 0x73, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x66, 0x69, 0x73, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x66, 0x69, 0x73, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d,
0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x6b, 0x75, 0x69, 0x61, 0x6e, 0x61, 0x2d, 0x66,
0x61, 0x6b, 0x61, 0x66, 0x61, 0x6c, 0x61, 0x6e, 0x69, 0x73, 0x113, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6e, 0x65,
0x74, 0x101, 0x74, 0x69, 0x6b, 0x61, 0x66, 0x61, 0x6c, 0x61, 0x6e, 0x69,
0x73, 0x113, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b,
0x61, 0x6c, 0x61, 0x70, 0x61, 0x6b, 0x6f, 0x73, 0x69, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x61, 0x6d, 0x69, 0x70, 0x69,
0x113, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x65,
0x14d, 0x73, 0x69, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x73, 0x65, 0x14d, 0x73, 0x69, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d,
0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x65, 0x14d, 0x73, 0x69, 0x61, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x69, 0x6c, 0x69,
0x70, 0x61, 0x73, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x6b, 0x69, 0x6c, 0x69, 0x6e, 0x69, 0x75, 0x69, 0x73, 0x69, 0x20,
0x6d, 0x101, 0x6c, 0x69, 0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x66, 0x6f, 0x6e, 0x75, 0x61, 0x6d, 0x61, 0x74, 0x61, 0x2d,
0x68, 0x61, 0x68, 0x61, 0x6b, 0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x66, 0x6f, 0x6e, 0x75, 0x61, 0x6d, 0x61, 0x74, 0x61,
0x2d, 0x68, 0x61, 0x68, 0x61, 0x6b, 0x65, 0x20, 0x74, 0x61, 0x69, 0x6d,
0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x66, 0x6f, 0x6e, 0x75, 0x61, 0x6d, 0x61,
0x74, 0x61, 0x2d, 0x68, 0x61, 0x68, 0x61, 0x6b, 0x65, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x66, 0x6f, 0x6e, 0x75, 0x61, 0x6d,
0x61, 0x74, 0x61, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x66, 0x6f, 0x6e, 0x75, 0x61,
0x6d, 0x61, 0x74, 0x61, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x66, 0x6f, 0x6e,
0x75, 0x61, 0x6d, 0x61, 0x74, 0x61, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66,
0x6f, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x75,
0x61, 0x6d, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6b, 0x16b, 0x6c, 0x69, 0x66, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x68, 0x61, 0x75, 0x61, 0x69, 0x2bb, 0x69, 0x2d, 0x61,
0x6c, 0x65, 0x75, 0x74, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x68, 0x61, 0x75, 0x61, 0x69, 0x2bb, 0x69, 0x2d, 0x61, 0x6c,
0x65, 0x75, 0x74, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74,
0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x68, 0x61, 0x75, 0x61, 0x69, 0x2bb, 0x69, 0x2d, 0x61, 0x6c,
0x65, 0x75, 0x74, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c,
0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x68, 0x6f, 0x6e, 0x67, 0x69, 0x2d, 0x6b, 0x6f, 0x6e, 0x67, 0x69,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x68, 0x6f, 0x6e,
0x67, 0x69, 0x2d, 0x6b, 0x6f, 0x6e, 0x67, 0x69, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x68, 0x6f, 0x6e, 0x67, 0x69, 0x2d,
0x6b, 0x6f, 0x6e, 0x67, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x68, 0x6f, 0x76, 0x69, 0x74, 0x65, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x68, 0x6f, 0x76, 0x69, 0x74, 0x65, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x68, 0x6f, 0x76,
0x69, 0x74, 0x65, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69,
0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x2bb, 0x69, 0x6e, 0x69, 0x74, 0x69, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x61, 0x2bb, 0x69, 0x6e,
0x69, 0x74, 0x69, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x69, 0x6e, 0x69, 0x74, 0x6f, 0x73, 0x69, 0x61, 0x69, 0x6e,
0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x69,
0x6e, 0x69, 0x74, 0x6f, 0x6e, 0x69, 0x73, 0x69, 0x61, 0x2d, 0x6c, 0x6f,
0x74, 0x6f, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb,
0x69, 0x6e, 0x69, 0x74, 0x6f, 0x6e, 0x69, 0x73, 0x69, 0x61, 0x2d, 0x68,
0x61, 0x68, 0x61, 0x6b, 0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x2bb, 0x69, 0x6e, 0x69, 0x74, 0x6f, 0x6e, 0x69, 0x73, 0x69,
0x61, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x69, 0x6c, 0x61, 0x61, 0x6e, 0x69,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x69, 0x6c,
0x61, 0x61, 0x6e, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74,
0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x2bb, 0x69, 0x6c, 0x61, 0x61, 0x6e, 0x69, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d,
0x2bb, 0x12b, 0x6b, 0x75, 0x74, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61,
0x2d, 0x2bb, 0x12b, 0x6b, 0x75, 0x74, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73,
0x69, 0x61, 0x2d, 0x2bb, 0x12b, 0x6b, 0x75, 0x74, 0x69, 0x73, 0x69, 0x6b,
0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x69,
0x73, 0x69, 0x6c, 0x65, 0x6c, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x2bb, 0x69, 0x73, 0x69, 0x6c, 0x65, 0x6c, 0x69, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x69, 0x73,
0x69, 0x6c, 0x65, 0x6c, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x73, 0x69, 0x61, 0x70, 0x61, 0x6e, 0x69, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x69, 0x61, 0x70, 0x61, 0x6e,
0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f,
0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73,
0x69, 0x61, 0x70, 0x61, 0x6e, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x70, 0x65, 0x74,
0x65, 0x6c, 0x6f, 0x70, 0x61, 0x76, 0x69, 0x6c, 0x6f, 0x76, 0x69, 0x73,
0x69, 0x6b, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x70, 0x65, 0x74, 0x65, 0x6c, 0x6f,
0x70, 0x61, 0x76, 0x69, 0x6c, 0x6f, 0x76, 0x69, 0x73, 0x69, 0x6b, 0x69,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b,
0x73, 0x69, 0x61, 0x2d, 0x70, 0x65, 0x74, 0x65, 0x6c, 0x6f, 0x70, 0x61,
0x76, 0x69, 0x6c, 0x6f, 0x76, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x74,
0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x61, 0x73, 0x61, 0x6b,
0x69, 0x74, 0x61, 0x6e, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x6b, 0x61, 0x73, 0x61, 0x6b, 0x69, 0x74, 0x61, 0x6e, 0x69,
0x2d, 0x68, 0x61, 0x68, 0x61, 0x6b, 0x65, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x6b, 0x61, 0x73, 0x61, 0x6b, 0x69, 0x74, 0x61,
0x6e, 0x69, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x14d, 0x6c, 0x65, 0x61, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x14d, 0x6c, 0x65,
0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f,
0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b,
0x14d, 0x6c, 0x65, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c,
0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x6b, 0x6f, 0x73, 0x69, 0x6c, 0x61, 0x65, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x6b,
0x61, 0x6c, 0x61, 0x73, 0x69, 0x6e, 0x6f, 0x69, 0x101, 0x73, 0x69, 0x6b,
0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b,
0x73, 0x69, 0x61, 0x2d, 0x6b, 0x61, 0x6c, 0x61, 0x73, 0x69, 0x6e, 0x6f,
0x69, 0x101, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x6b, 0x61,
0x6c, 0x61, 0x73, 0x69, 0x6e, 0x6f, 0x69, 0x101, 0x73, 0x69, 0x6b, 0x69,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x12b, 0x6b,
0x69, 0x73, 0x69, 0x74, 0x61, 0x6e, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x6c, 0x61, 0x6e, 0x67, 0x69, 0x6b, 0x101, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x6f, 0x74, 0x75,
0x6d, 0x6f, 0x74, 0x75, 0x6c, 0x61, 0x69, 0x6e, 0x65, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x6f, 0x74, 0x75, 0x2bb, 0x65,
0x69, 0x6b, 0x69, 0x68, 0x6f, 0x75, 0x65, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x6d, 0x6f, 0x74, 0x75, 0x2bb, 0x65, 0x69, 0x6b,
0x69, 0x68, 0x6f, 0x75, 0x65, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x6d, 0x6f, 0x74, 0x75, 0x2bb, 0x65, 0x69, 0x6b, 0x69,
0x68, 0x6f, 0x75, 0x65, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c,
0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x6d, 0x61, 0x6b, 0x61, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x6d, 0x61, 0x6b, 0x61, 0x75, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x61, 0x6b, 0x61, 0x75, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69,
0x61, 0x2d, 0x6d, 0x61, 0x6b, 0x61, 0x74, 0x61, 0x6e, 0x69, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61,
0x2d, 0x6d, 0x61, 0x6b, 0x61, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61,
0x2d, 0x6d, 0x61, 0x6b, 0x61, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x61, 0x6c, 0x65, 0x69, 0x73,
0x69, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d,
0x61, 0x6c, 0x61, 0x74, 0x69, 0x76, 0x69, 0x73, 0x69, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x101, 0x6b, 0x65, 0x73, 0x61,
0x73, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb,
0x6f, 0x74, 0x75, 0x6d, 0x6f, 0x74, 0x75, 0x6d, 0x61, 0x73, 0x6f, 0x6c,
0x6f, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x61,
0x75, 0x6c, 0x69, 0x74, 0x69, 0x75, 0x73, 0x69, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x61, 0x75, 0x6c, 0x69, 0x74, 0x69,
0x75, 0x73, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f,
0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x6d, 0x61, 0x75, 0x6c, 0x69, 0x74, 0x69, 0x75, 0x73, 0x69, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x61, 0x75, 0x73,
0x6f, 0x6e, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6d, 0x65, 0x6b, 0x69, 0x73, 0x69, 0x6b, 0x6f, 0x75, 0x2d, 0x70, 0x61,
0x73, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x6d, 0x65, 0x6b, 0x69, 0x73, 0x69, 0x6b, 0x6f, 0x75,
0x2d, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x65, 0x6b, 0x69, 0x73,
0x69, 0x6b, 0x6f, 0x75, 0x2d, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b,
0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x75,
0x6c, 0x101, 0x6e, 0x69, 0x70, 0x101, 0x74, 0x101, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x75, 0x6c, 0x101, 0x6e, 0x69, 0x70,
0x101, 0x74, 0x101, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f,
0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x75, 0x6c, 0x101, 0x6e, 0x69, 0x70, 0x101, 0x74, 0x101, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69,
0x61, 0x2d, 0x6d, 0x6f, 0x73, 0x69, 0x6b, 0x6f, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d,
0x6d, 0x6f, 0x73, 0x69, 0x6b, 0x6f, 0x75, 0x20, 0x74, 0x61, 0x69, 0x6d,
0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x6d,
0x6f, 0x73, 0x69, 0x6b, 0x6f, 0x75, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x70, 0x65, 0x6d, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x6e, 0x61, 0x75, 0x6c, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6e, 0x65, 0x70, 0x61, 0x6c, 0x69,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x61, 0x6c,
0x65, 0x74, 0x14d, 0x6e, 0x69, 0x61, 0x66, 0x6f, 0x2bb, 0x6f, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x61, 0x6c, 0x65,
0x74, 0x14d, 0x6e, 0x69, 0x61, 0x66, 0x6f, 0x2bb, 0x6f, 0x75, 0x20, 0x74,
0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x61, 0x6c, 0x65,
0x74, 0x14d, 0x6e, 0x69, 0x61, 0x66, 0x6f, 0x2bb, 0x6f, 0x75, 0x20, 0x74,
0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6e, 0x75, 0x2bb, 0x75, 0x73,
0x69, 0x6c, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6e, 0x75, 0x2bb, 0x75, 0x73, 0x69, 0x6c, 0x61, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6e, 0x75, 0x2bb, 0x75, 0x73, 0x69,
0x6c, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c,
0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x66,
0x6f, 0x6e, 0x75, 0x61, 0x2bb, 0x69, 0x6c, 0x6f, 0x66, 0x6f, 0x2bb, 0x6f,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x66, 0x6f,
0x6e, 0x75, 0x61, 0x2bb, 0x69, 0x6c, 0x6f, 0x66, 0x6f, 0x2bb, 0x6f, 0x75,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x66, 0x6f,
0x6e, 0x75, 0x61, 0x2bb, 0x69, 0x6c, 0x6f, 0x66, 0x6f, 0x2bb, 0x6f, 0x75,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6e, 0x69, 0x75,
0x113, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6e, 0x6f,
0x61, 0x66, 0x14d, 0x6b, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x6e, 0x6f, 0x61, 0x66, 0x14d, 0x6b, 0x69, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6e, 0x6f, 0x61, 0x66, 0x14d,
0x6b, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c,
0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x66,
0x113, 0x6e, 0x61, 0x6e, 0x69, 0x74, 0x6f, 0x2d, 0x74, 0x65, 0x2d, 0x6e,
0x6f, 0x6c, 0x14d, 0x6e, 0x69, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x66, 0x113, 0x6e, 0x61, 0x6e, 0x69, 0x74, 0x6f, 0x2d,
0x74, 0x65, 0x2d, 0x6e, 0x6f, 0x6c, 0x14d, 0x6e, 0x69, 0x61, 0x20, 0x74,
0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x66, 0x113, 0x6e, 0x61,
0x6e, 0x69, 0x74, 0x6f, 0x2d, 0x74, 0x65, 0x2d, 0x6e, 0x6f, 0x6c, 0x14d,
0x6e, 0x69, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69,
0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6d, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x61, 0x74, 0x6f, 0x6b, 0x65, 0x6c,
0x61, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c,
0x16b, 0x73, 0x69, 0x61, 0x2d, 0x6e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x70,
0x12b, 0x73, 0x69, 0x6b, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x6e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x70, 0x12b, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d,
0x6e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x70, 0x12b, 0x73, 0x69, 0x6b, 0x69,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73,
0x69, 0x61, 0x2d, 0x2bb, 0x6f, 0x6d, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69,
0x61, 0x2d, 0x2bb, 0x6f, 0x6d, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x74,
0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69,
0x61, 0x2d, 0x2bb, 0x6f, 0x6d, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x74,
0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x101, 0x6b, 0x69, 0x73,
0x69, 0x74, 0x61, 0x6e, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x70, 0x101, 0x6b, 0x69, 0x73, 0x69, 0x74, 0x61, 0x6e, 0x69,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x101,
0x6b, 0x69, 0x73, 0x69, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x61, 0x6c, 0x61, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x61, 0x70, 0x75, 0x61,
0x6e, 0x69, 0x75, 0x6b, 0x69, 0x6e, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x70, 0x61, 0x6c, 0x61, 0x6b, 0x75, 0x61, 0x69,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x61, 0x6c,
0x61, 0x6b, 0x75, 0x61, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x70, 0x61, 0x6c, 0x61, 0x6b, 0x75, 0x61, 0x69, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x65, 0x6c, 0x16b,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x65, 0x6c,
0x16b, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f,
0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70,
0x65, 0x6c, 0x16b, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69,
0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x66, 0x69, 0x6c, 0x69, 0x70, 0x61, 0x69, 0x6e, 0x65, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x66, 0x69, 0x6c, 0x69, 0x70, 0x61,
0x69, 0x6e, 0x65, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f,
0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x66, 0x69, 0x6c, 0x69, 0x70, 0x61, 0x69, 0x6e, 0x65, 0x20, 0x74,
0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x6f, 0x74, 0x75, 0x6d,
0x6f, 0x74, 0x75, 0x66, 0x6f, 0x69, 0x6e, 0x69, 0x6b, 0x69, 0x73, 0x69,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x101, 0x2d,
0x70, 0x69, 0x65, 0x61, 0x2d, 0x6d, 0x6f, 0x2d, 0x6d, 0x69, 0x6b, 0x65,
0x6c, 0x6f, 0x6e, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x73, 0x101, 0x2d, 0x70, 0x69, 0x65, 0x61, 0x2d, 0x6d, 0x6f, 0x2d,
0x6d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x69, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x101, 0x2d, 0x70, 0x69, 0x65,
0x61, 0x2d, 0x6d, 0x6f, 0x2d, 0x6d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e,
0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x69,
0x74, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x70, 0x14d, 0x6e, 0x61, 0x70, 0x113, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x69, 0x6f, 0x6e, 0x67, 0x69,
0x61, 0x6e, 0x67, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x6b, 0x69, 0x73, 0x69, 0x6c, 0x14d, 0x74, 0x61, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x69, 0x73, 0x69, 0x6c, 0x14d,
0x74, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74,
0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6b, 0x69, 0x73, 0x69, 0x6c, 0x14d, 0x74, 0x61, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x113, 0x75, 0x6e, 0x69, 0x6f, 0x6e,
0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x6f,
0x74, 0x65, 0x6c, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x73, 0x61, 0x6b, 0x101, 0x6c,
0x69, 0x6e, 0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x73, 0x61, 0x6b, 0x101, 0x6c, 0x69,
0x6e, 0x65, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74,
0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x73, 0x61, 0x6b, 0x101, 0x6c, 0x69,
0x6e, 0x65, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c,
0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c,
0x16b, 0x73, 0x69, 0x61, 0x2d, 0x73, 0x61, 0x6d, 0x61, 0x6c, 0x61, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69,
0x61, 0x2d, 0x73, 0x61, 0x6d, 0x61, 0x6c, 0x61, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d,
0x73, 0x61, 0x6d, 0x61, 0x6c, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x68, 0x61, 0x2bb, 0x61, 0x6d, 0x6f, 0x61, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x68, 0x61, 0x2bb, 0x61, 0x6d,
0x6f, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74,
0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x68, 0x61, 0x2bb, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d,
0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x6f, 0x74, 0x75, 0x6d, 0x6f, 0x74, 0x75,
0x2d, 0x73, 0x65, 0x69, 0x73, 0x65, 0x6c, 0x69, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f,
0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x6f,
0x74, 0x75, 0x6d, 0x6f, 0x74, 0x75, 0x73, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f,
0x6e, 0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73,
0x69, 0x6f, 0x73, 0x69, 0x61, 0x74, 0x6f, 0x6e, 0x67, 0x61, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x75, 0x6c, 0x69, 0x6e,
0x61, 0x6d, 0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x73, 0x69, 0x6f, 0x75, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x74, 0x61, 0x68, 0x69, 0x73, 0x69, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x74, 0x61, 0x69, 0x70, 0x65, 0x69, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x74, 0x61, 0x69, 0x70,
0x65, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74,
0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x74, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x74, 0x61, 0x73, 0x69, 0x6b, 0x69, 0x74, 0x61, 0x6e,
0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x74, 0x6f,
0x6b, 0x65, 0x6c, 0x61, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x74, 0x6f, 0x6e, 0x67, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x74, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x74, 0x6f, 0x6e, 0x67, 0x61,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x74, 0x16b, 0x6b,
0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x74, 0x16b,
0x6b, 0x69, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x69, 0x74, 0x61, 0x6e, 0x69,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x74, 0x16b, 0x6b,
0x69, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x69, 0x74, 0x61, 0x6e, 0x69, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x74, 0x16b, 0x6b,
0x69, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x69, 0x74, 0x61, 0x6e, 0x69, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x74, 0x16b, 0x76, 0x61,
0x6c, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb,
0x75, 0x6c, 0x75, 0x6b, 0x75, 0x61, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x75, 0x6c, 0x75, 0x6b, 0x75, 0x61, 0x69,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x75,
0x6c, 0x75, 0x6b, 0x75, 0x61, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x2bb, 0x75, 0x73, 0x69, 0x70, 0x65, 0x6b, 0x69, 0x74,
0x61, 0x6e, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x2bb, 0x75, 0x73, 0x69, 0x70, 0x65, 0x6b, 0x69, 0x74, 0x61, 0x6e, 0x69,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x75,
0x73, 0x69, 0x70, 0x65, 0x6b, 0x69, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x74,
0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x76, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x76,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x76, 0x65, 0x6e, 0x65,
0x73, 0x75, 0x65, 0x6c, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x76, 0x61, 0x6c, 0x61,
0x74, 0x69, 0x76, 0x6f, 0x73, 0x69, 0x74, 0x6f, 0x6b, 0x69, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61,
0x2d, 0x76, 0x61, 0x6c, 0x61, 0x74, 0x69, 0x76, 0x6f, 0x73, 0x69, 0x74,
0x6f, 0x6b, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f,
0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x76, 0x61, 0x6c, 0x61, 0x74,
0x69, 0x76, 0x6f, 0x73, 0x69, 0x74, 0x6f, 0x6b, 0x69, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d,
0x76, 0x6f, 0x6c, 0x69, 0x6b, 0x6f, 0x6b, 0x61, 0x6c, 0x61, 0x74, 0x69,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73,
0x69, 0x61, 0x2d, 0x76, 0x6f, 0x6c, 0x69, 0x6b, 0x6f, 0x6b, 0x61, 0x6c,
0x61, 0x74, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f,
0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x76, 0x6f, 0x6c, 0x69, 0x6b,
0x6f, 0x6b, 0x61, 0x6c, 0x61, 0x74, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d,
0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x76, 0x6f, 0x73, 0x69, 0x74, 0x6f, 0x6b, 0x69,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x6f, 0x74,
0x75, 0x75, 0x65, 0x6b, 0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x2bb, 0x75, 0x76, 0x65, 0x61, 0x20, 0x6d, 0x6f, 0x20, 0x66,
0x75, 0x74, 0x75, 0x6e, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x2bb, 0x69, 0x101, 0x6b,
0x75, 0x74, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x2bb, 0x69,
0x101, 0x6b, 0x75, 0x74, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61,
0x2d, 0x2bb, 0x69, 0x101, 0x6b, 0x75, 0x74, 0x69, 0x73, 0x69, 0x6b, 0x69,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73,
0x69, 0x61, 0x2d, 0x2bb, 0x69, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x6c, 0x69,
0x6e, 0x65, 0x70, 0x16b, 0x6b, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x2bb, 0x69, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x6c, 0x69, 0x6e, 0x65, 0x70, 0x16b, 0x6b, 0x69,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b,
0x73, 0x69, 0x61, 0x2d, 0x2bb, 0x69, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x6c,
0x69, 0x6e, 0x65, 0x70, 0x16b, 0x6b, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d,
0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x69, 0x75, 0x6b, 0x6f, 0x6e, 0x69, 0x50, 0x61,
0x6c, 0x6f, 0x67, 0x61, 0x72, 0x65, 0x20, 0x79, 0x61, 0x20, 0x6e, 0x61,
0x6b, 0x6f, 0x20, 0x79, 0x61, 0x20, 0x6e, 0x67, 0x77, 0x61, 0x67, 0x61,
0x20, 0x6c, 0x65, 0x20, 0x6e, 0x67, 0x77, 0x61, 0x67, 0x61, 0x20, 0x79,
0x61, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x41,
0x63, 0x72, 0x65, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x63, 0x72,
0x65, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x41, 0x63, 0x72, 0x65, 0x20, 0x59, 0x61, 0x7a,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4f, 0x72,
0x74, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x44, 0x6f, 0x11f, 0x75, 0x20, 0x41, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x47, 0xfc, 0x6e, 0x65,
0x79, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42,
0x61, 0x74, 0x131, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x42, 0x61, 0x74, 0x131, 0x20, 0x41, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42, 0x61, 0x74, 0x131, 0x20, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x131, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x131, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x131, 0x20, 0x59, 0x61, 0x7a, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x20, 0x59, 0x61, 0x7a,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x75, 0x7a, 0x65, 0x79, 0x20,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x4d, 0x65, 0x72, 0x6b,
0x65, 0x7a, 0x69, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x75, 0x7a,
0x65, 0x79, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x4d,
0x65, 0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x75, 0x7a,
0x65, 0x79, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x4d,
0x65, 0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x4b, 0x75, 0x7a, 0x65, 0x79, 0x20, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x44, 0x6f, 0x11f, 0x75, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x4b, 0x75, 0x7a, 0x65, 0x79, 0x20, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x44, 0x6f, 0x11f, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x4b, 0x75, 0x7a, 0x65, 0x79, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x44, 0x6f, 0x11f, 0x75, 0x20, 0x59, 0x61, 0x7a, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x75, 0x7a, 0x65, 0x79, 0x20, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x44, 0x61, 0x11f, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x4b, 0x75, 0x7a, 0x65, 0x79, 0x20, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x44, 0x61, 0x11f, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x4b, 0x75, 0x7a, 0x65, 0x79, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x44, 0x61, 0x11f, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x4b, 0x75, 0x7a, 0x65, 0x79, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x75, 0x7a, 0x65, 0x79, 0x20,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x50, 0x61, 0x73, 0x69,
0x66, 0x69, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x75, 0x7a, 0x65, 0x79, 0x20,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x50, 0x61, 0x73, 0x69,
0x66, 0x69, 0x6b, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x41, 0x6e, 0x61, 0x64, 0x131, 0x72, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x6e,
0x61, 0x64, 0x131, 0x72, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x41, 0x70, 0x69, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x41, 0x70, 0x69, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x70, 0x69, 0x61, 0x20,
0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x6b, 0x74,
0x61, 0x76, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x6b, 0x74, 0x61,
0x76, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x41, 0x6b, 0x74, 0x61, 0x76, 0x20, 0x59, 0x61,
0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x6b, 0x74, 0xf6, 0x62,
0x65, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x6b, 0x74, 0xf6, 0x62,
0x65, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x41, 0x6b, 0x74, 0xf6, 0x62, 0x65, 0x20, 0x59,
0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x72, 0x61, 0x62,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41,
0x72, 0x61, 0x62, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41,
0x72, 0x61, 0x62, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x59, 0x61, 0x7a,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x72, 0x6a, 0x61, 0x6e, 0x74,
0x69, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x72, 0x6a, 0x61,
0x6e, 0x74, 0x69, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x72, 0x6a, 0x61, 0x6e,
0x74, 0x69, 0x6e, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x42, 0x61, 0x74, 0x131, 0x20, 0x41, 0x72, 0x6a, 0x61, 0x6e, 0x74,
0x69, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42, 0x61, 0x74, 0x131,
0x20, 0x41, 0x72, 0x6a, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x42, 0x61, 0x74, 0x131, 0x20, 0x41, 0x72, 0x6a, 0x61, 0x6e, 0x74, 0x69,
0x6e, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x45,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x45, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x45, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x20, 0x59, 0x61,
0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4f, 0x72, 0x74, 0x61, 0x20,
0x41, 0x76, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x4f, 0x72, 0x74, 0x61, 0x20, 0x41, 0x76, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4f, 0x72,
0x74, 0x61, 0x20, 0x41, 0x76, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79,
0x61, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x130,
0xe7, 0x20, 0x42, 0x61, 0x74, 0x131, 0x20, 0x41, 0x76, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x130,
0xe7, 0x20, 0x42, 0x61, 0x74, 0x131, 0x20, 0x41, 0x76, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x130, 0xe7, 0x20, 0x42,
0x61, 0x74, 0x131, 0x20, 0x41, 0x76, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x79, 0x61, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x44, 0x6f, 0x11f, 0x75, 0x20, 0x41, 0x76, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x79, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x44, 0x6f, 0x11f,
0x75, 0x20, 0x41, 0x76, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x44, 0x6f, 0x11f, 0x75, 0x20, 0x41, 0x76, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x79, 0x63,
0x61, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x61, 0x79, 0x63, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x79, 0x63, 0x61, 0x6e, 0x20, 0x59, 0x61, 0x7a, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x7a, 0x6f, 0x72, 0x6c, 0x61, 0x72,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x7a, 0x6f, 0x72, 0x6c, 0x61,
0x72, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x41, 0x7a, 0x6f, 0x72, 0x6c, 0x61, 0x72, 0x20,
0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x15f, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x15f, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x15f, 0x20, 0x59, 0x61,
0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42, 0x75, 0x74, 0x61, 0x6e,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x79,
0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42, 0x72, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42, 0x72, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x61, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72,
0xfc, 0x2019, 0x73, 0x2d, 0x53, 0x65, 0x6c, 0x61, 0x6d, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64,
0x65, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x43, 0x61, 0x70, 0x65, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x43, 0x61, 0x70, 0x65,
0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x43, 0x61, 0x73, 0x65, 0x79, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x20,
0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x15e, 0x69, 0x6c,
0x69, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x15e, 0x69, 0x6c, 0x69, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x15e, 0x69, 0x6c, 0x69, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0xc7, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0xc7, 0x69, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0xc7, 0x69, 0x6e, 0x20, 0x59,
0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x43, 0x68, 0x72, 0x69,
0x73, 0x74, 0x6d, 0x61, 0x73, 0x20, 0x41, 0x64, 0x61, 0x73, 0x131, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x20, 0x41,
0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x79, 0x61, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x79, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x79, 0x61,
0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x43, 0x6f,
0x6f, 0x6b, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x41, 0x64, 0x61,
0x6c, 0x61, 0x72, 0x131, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x43, 0x6f, 0x6f, 0x6b, 0x20,
0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x59, 0x61, 0x72, 0x131,
0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0xfc,
0x62, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0xfc, 0x62, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x4b, 0xfc, 0x62, 0x61, 0x20, 0x59, 0x61, 0x7a, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x44, 0x61, 0x76, 0x69, 0x73, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64,
0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x44, 0x6f, 0x11f, 0x75, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x61, 0x73, 0x6b, 0x61, 0x6c,
0x79, 0x61, 0x20, 0x41, 0x64, 0x61, 0x73, 0x131, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x50, 0x61, 0x73, 0x6b, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x41,
0x64, 0x61, 0x73, 0x131, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x61, 0x73, 0x6b, 0x61,
0x6c, 0x79, 0x61, 0x20, 0x41, 0x64, 0x61, 0x73, 0x131, 0x20, 0x59, 0x61,
0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x45, 0x6b, 0x76, 0x61, 0x64,
0x6f, 0x72, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4f, 0x72, 0x74, 0x61,
0x20, 0x41, 0x76, 0x72, 0x75, 0x70, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x4f, 0x72, 0x74, 0x61, 0x20, 0x41, 0x76, 0x72, 0x75, 0x70, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x4f, 0x72, 0x74, 0x61, 0x20, 0x41, 0x76, 0x72, 0x75,
0x70, 0x61, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x44, 0x6f, 0x11f, 0x75, 0x20, 0x41, 0x76, 0x72, 0x75, 0x70, 0x61, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x44, 0x6f, 0x11f, 0x75, 0x20, 0x41, 0x76,
0x72, 0x75, 0x70, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x44, 0x6f, 0x11f, 0x75, 0x20,
0x41, 0x76, 0x72, 0x75, 0x70, 0x61, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x130, 0x6c, 0x65, 0x72, 0x69, 0x20, 0x44, 0x6f,
0x11f, 0x75, 0x20, 0x41, 0x76, 0x72, 0x75, 0x70, 0x61, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x42, 0x61, 0x74, 0x131, 0x20, 0x41, 0x76, 0x72, 0x75,
0x70, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42, 0x61, 0x74, 0x131,
0x20, 0x41, 0x76, 0x72, 0x75, 0x70, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42, 0x61,
0x74, 0x131, 0x20, 0x41, 0x76, 0x72, 0x75, 0x70, 0x61, 0x20, 0x59, 0x61,
0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x41, 0x64, 0x61,
0x6c, 0x61, 0x72, 0x131, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x46, 0x69, 0x6a, 0x69, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x46, 0x69, 0x6a, 0x69, 0x20,
0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x46, 0x72, 0x61,
0x6e, 0x73, 0x131, 0x7a, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x73,
0x131, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x46, 0x72, 0x61, 0x6e, 0x73,
0x131, 0x7a, 0x20, 0x47, 0xfc, 0x6e, 0x65, 0x79, 0x20, 0x76, 0x65, 0x20,
0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x6b, 0x61, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f,
0x73, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x47, 0x61, 0x6d, 0x62, 0x69,
0x65, 0x72, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x47, 0xfc, 0x72, 0x63,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x47,
0xfc, 0x72, 0x63, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x47,
0xfc, 0x72, 0x63, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x59, 0x61, 0x7a,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72,
0x74, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68,
0x20, 0x4f, 0x72, 0x74, 0x61, 0x6c, 0x61, 0x6d, 0x61, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x44, 0x6f, 0x11f, 0x75, 0x20, 0x47, 0x72, 0xf6, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x44, 0x6f,
0x11f, 0x75, 0x20, 0x47, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x44, 0x6f, 0x11f, 0x75, 0x20, 0x47, 0x72, 0xf6, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x42, 0x61, 0x74, 0x131, 0x20, 0x47, 0x72, 0xf6, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42, 0x61, 0x74, 0x131,
0x20, 0x47, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x42, 0x61, 0x74, 0x131, 0x20, 0x47, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x47,
0x75, 0x61, 0x6d, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0xf6, 0x72, 0x66, 0x65, 0x7a,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e,
0x67, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e,
0x67, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x48,
0x6f, 0x76, 0x64, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x48, 0x6f, 0x76,
0x64, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x59, 0x61, 0x7a,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x48, 0x69, 0x6e, 0x64, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x48, 0x69, 0x6e, 0x74, 0x20, 0x4f,
0x6b, 0x79, 0x61, 0x6e, 0x75, 0x73, 0x75, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x48, 0x69, 0x6e, 0x64, 0x69, 0xe7, 0x69, 0x6e, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x4f, 0x72, 0x74, 0x61, 0x20, 0x45, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x7a, 0x79, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x44,
0x6f, 0x11f, 0x75, 0x20, 0x45, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x79,
0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42, 0x61, 0x74, 0x131, 0x20,
0x45, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x79, 0x61, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x130, 0x72, 0x61, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x130, 0x72, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x130, 0x72, 0x61, 0x6e,
0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x130, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x130,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x130, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x130, 0x73, 0x72, 0x61, 0x69, 0x6c, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x130, 0x73, 0x72, 0x61, 0x69, 0x6c, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x130,
0x73, 0x72, 0x61, 0x69, 0x6c, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x79, 0x61, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x79, 0x61, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x79, 0x61, 0x20, 0x59, 0x61,
0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0xe7, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73,
0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0xe7, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0xe7, 0x61, 0x74, 0x73, 0x6b,
0x69, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b,
0x61, 0x7a, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x44, 0x6f, 0x11f, 0x75, 0x20, 0x4b, 0x61, 0x7a, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x42, 0x61, 0x74, 0x131, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x6f, 0x72,
0x65, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x6f, 0x72, 0x65, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x4b, 0x6f, 0x72, 0x65, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x73, 0x6b, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x59,
0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x131, 0x72, 0x67,
0x131, 0x7a, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x4c, 0x69, 0x6e, 0x65, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x4d, 0x61, 0x6b, 0x61, 0x6f, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x4d, 0x61, 0x6b, 0x61, 0x6f, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4d,
0x61, 0x6b, 0x61, 0x6f, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x59, 0x61, 0x7a,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x61, 0x6c, 0x65, 0x7a, 0x79,
0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x61, 0x6c, 0x64, 0x69,
0x76, 0x6c, 0x65, 0x72, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x61,
0x72, 0x6b, 0x69, 0x7a, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61,
0x6c, 0x6c, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x74, 0x69, 0x75, 0x73, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x74, 0x69, 0x75, 0x73, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x20, 0x50, 0x61,
0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x4b, 0x131, 0x79, 0x131, 0x73, 0x131,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b,
0x61, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x4b, 0x131,
0x79, 0x131, 0x73, 0x131, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x65, 0x6b, 0x73, 0x69,
0x6b, 0x61, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x4b,
0x131, 0x79, 0x131, 0x73, 0x131, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x75,
0x72, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x55, 0x6c, 0x61, 0x6e, 0x20,
0x42, 0x61, 0x74, 0x75, 0x72, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x55, 0x6c, 0x61, 0x6e,
0x20, 0x42, 0x61, 0x74, 0x75, 0x72, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x61, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x61, 0x20, 0x59,
0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x79, 0x61, 0x6e,
0x6d, 0x61, 0x72, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4e, 0x61, 0x75,
0x72, 0x75, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4e, 0x65, 0x70, 0x61,
0x6c, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x59, 0x65, 0x6e, 0x69, 0x20,
0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x79, 0x61, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x59, 0x65, 0x6e, 0x69, 0x20, 0x4b, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x79, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x59, 0x65, 0x6e, 0x69,
0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x79, 0x61, 0x20, 0x59,
0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x59, 0x65, 0x6e, 0x69,
0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x59, 0x65, 0x6e, 0x69, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e,
0x64, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x59, 0x65, 0x6e, 0x69, 0x20, 0x5a, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4e, 0x65, 0x77,
0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4e, 0x69,
0x75, 0x65, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x20, 0x41, 0x64, 0x61, 0x73, 0x131, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x41,
0x64, 0x61, 0x73, 0x131, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x20, 0x41, 0x64, 0x61, 0x73, 0x131, 0x20, 0x59, 0x61, 0x7a,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x75,
0x7a, 0x65, 0x79, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x6e, 0x61, 0x20,
0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x59, 0x61, 0x7a, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4f, 0x6d,
0x73, 0x6b, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x59, 0x61,
0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x61, 0x6c, 0x61, 0x75,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20,
0x59, 0x65, 0x6e, 0x69, 0x20, 0x47, 0x69, 0x6e, 0x65, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79,
0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x65,
0x72, 0x75, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x65, 0x72, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x50, 0x65, 0x72, 0x75, 0x20, 0x59, 0x61, 0x7a, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e,
0x6c, 0x65, 0x72, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x46, 0x69, 0x6c,
0x69, 0x70, 0x69, 0x6e, 0x6c, 0x65, 0x72, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x6e, 0x6c, 0x65, 0x72, 0x20, 0x59, 0x61, 0x7a,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69,
0x78, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x76, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x53, 0x61, 0x69,
0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x76, 0x65,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x20, 0x76, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x69,
0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x4b, 0x131, 0x7a, 0x131, 0x6c, 0x6f, 0x72, 0x64, 0x61,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x131, 0x7a, 0x131, 0x6c, 0x6f,
0x72, 0x64, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x131, 0x7a, 0x131, 0x6c, 0x6f,
0x72, 0x64, 0x61, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x59, 0x61,
0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x53, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x53, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20, 0x59,
0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x53, 0x61, 0x6d, 0x6f,
0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x59, 0x61, 0x7a,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x53, 0x65, 0x79, 0x15f, 0x65, 0x6c,
0x6c, 0x65, 0x72, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x53, 0x69, 0x6e,
0x67, 0x61, 0x70, 0x75, 0x72, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x53, 0x6f, 0x6c, 0x6f,
0x6d, 0x6f, 0x6e, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x47, 0xfc, 0x6e, 0x65, 0x79, 0x20, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x53, 0x68, 0x6f, 0x77, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x54, 0x61, 0x69,
0x70, 0x65, 0x69, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x54, 0x61, 0x63, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x54, 0xfc, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x54, 0xfc, 0x72,
0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x54, 0xfc, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x54, 0x75,
0x76, 0x61, 0x6c, 0x75, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x79, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x79, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0xd6, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0xd6, 0x7a, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0xd6, 0x7a, 0x62, 0x65, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x59, 0x61,
0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x56, 0x65, 0x6e, 0x65, 0x7a,
0x75, 0x65, 0x6c, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x57, 0x61, 0x6b, 0x65, 0x20, 0x41, 0x64, 0x61, 0x73, 0x131, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x76,
0x65, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x59, 0x61, 0x7a,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x4f, 0x77, 0x67, 0x61, 0x6e, 0x79, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x65, 0x72, 0x6b, 0x65,
0x7a, 0x69, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20,
0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x47, 0xfc, 0x6e, 0x6f, 0x72, 0x74, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61,
0x72, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x47, 0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x62,
0x61, 0x74, 0x61, 0x72, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x41, 0x6c, 0xfd, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x41, 0x6c, 0xfd, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x41, 0x6c, 0xfd, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75,
0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x4d, 0x65, 0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x4d, 0x65, 0x72, 0x6b, 0x65, 0x7a, 0x69,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d,
0x65, 0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x44, 0x65, 0x6d, 0x69, 0x72, 0x67, 0x61, 0x7a,
0x79, 0x6b, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x67,
0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x44, 0x65, 0x6d, 0x69, 0x72, 0x67, 0x61, 0x7a, 0x79, 0x6b, 0x20,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x67, 0xfc, 0x6e, 0x64,
0x6f, 0x67, 0x61, 0x72, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x44, 0x65, 0x6d, 0x69, 0x72,
0x67, 0x61, 0x7a, 0x79, 0x6b, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x67, 0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20, 0x74,
0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x44, 0x65, 0x6d, 0x69, 0x72, 0x67, 0x61, 0x7a, 0x79, 0x6b, 0x20, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x64, 0x61, 0x67, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x44, 0x65, 0x6d, 0x69, 0x72, 0x67, 0x61, 0x7a,
0x79, 0x6b, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x64,
0x61, 0x67, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x44, 0x65, 0x6d, 0x69, 0x72, 0x67, 0x61,
0x7a, 0x79, 0x6b, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x64, 0x61, 0x67, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x44, 0x65, 0x6d, 0x69, 0x72, 0x67, 0x61,
0x7a, 0x79, 0x6b, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20,
0xdd, 0x75, 0x77, 0x61, 0x15f, 0x20, 0x75, 0x6d, 0x6d, 0x61, 0x6e, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x44, 0x65, 0x6d, 0x69, 0x72, 0x67, 0x61,
0x7a, 0x79, 0x6b, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20,
0xdd, 0x75, 0x77, 0x61, 0x15f, 0x20, 0x75, 0x6d, 0x6d, 0x61, 0x6e, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x44, 0x65, 0x6d, 0x69, 0x72, 0x67, 0x61, 0x7a, 0x79, 0x6b,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0xdd, 0x75, 0x77,
0x61, 0x15f, 0x20, 0x75, 0x6d, 0x6d, 0x61, 0x6e, 0x20, 0x74, 0x6f, 0x6d,
0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x6e,
0x61, 0x64, 0x79, 0x72, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x70,
0x69, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x70, 0x69, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x41, 0x70, 0x69, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75,
0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x72, 0x61,
0x70, 0x20, 0xfd, 0x75, 0x72, 0x74, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x41, 0x72, 0x61, 0x70, 0x20, 0xfd, 0x75, 0x72,
0x74, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x72, 0x61, 0x70,
0x20, 0xfd, 0x75, 0x72, 0x74, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x74, 0x6f,
0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20,
0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x47, 0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x47, 0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc,
0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x45, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x45, 0x72,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x45,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x74, 0x6f,
0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x20, 0x74, 0x6f, 0x6d,
0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x65,
0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x65,
0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0xfd, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x65, 0x72, 0x6b, 0x65,
0x7a, 0x69, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0xfd,
0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x4d, 0x65, 0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20, 0x41,
0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0xfd, 0x61, 0x20, 0x67, 0xfc,
0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x4d, 0x65, 0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20, 0x41, 0x77, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0xfd, 0x61, 0x20, 0x67, 0xfc, 0x6e, 0x62, 0x61,
0x74, 0x61, 0x72, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x65, 0x72, 0x6b, 0x65, 0x7a,
0x69, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0xfd, 0x61,
0x20, 0x67, 0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0x74, 0x6f,
0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47,
0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20, 0x41, 0x77, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x47, 0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20, 0x41, 0x77, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0xfd, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc,
0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0xfd, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b,
0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x62, 0x61,
0x74, 0x61, 0x72, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x62,
0x61, 0x74, 0x61, 0x72, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0xfd, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x62, 0x61, 0x74,
0x61, 0x72, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0xfd,
0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0xfd, 0x6a, 0x61,
0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x7a, 0x65, 0x72, 0x62,
0x61, 0xfd, 0x6a, 0x61, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x61, 0xfd, 0x6a, 0x61, 0x6e, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73,
0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x7a, 0x6f, 0x72,
0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x41, 0x7a, 0x6f, 0x72, 0x20, 0x61, 0x64, 0x61, 0x6c, 0x61,
0x72, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x7a, 0x6f, 0x72, 0x20, 0x61, 0x64,
0x61, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b,
0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x15f, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x15f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x15f, 0x20, 0x74, 0x6f, 0x6d, 0x75,
0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x42, 0x75, 0x74,
0x61, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x42, 0x6f, 0x6c, 0x69,
0x77, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x42, 0x72,
0x61, 0x7a, 0x69, 0x6c, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0xfd, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0xfd, 0x61, 0x20, 0x74,
0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x42, 0x72, 0x75, 0x6e, 0x65, 0xfd, 0x2d, 0x44, 0x61, 0x72, 0x75, 0x73,
0x73, 0x61, 0x6c, 0x61, 0x6d, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4b,
0x61, 0x62, 0x6f, 0x2d, 0x57, 0x65, 0x72, 0x64, 0x65, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x4b, 0x61, 0x62, 0x6f, 0x2d, 0x57, 0x65, 0x72, 0x64,
0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x4b, 0x61, 0x62, 0x6f, 0x2d, 0x57, 0x65, 0x72,
0x64, 0x65, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0xc7, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0xc7, 0x61, 0x74, 0x65, 0x6d, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0xc7, 0x61, 0x74, 0x65, 0x6d, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0xc7, 0x61, 0x74, 0x65, 0x6d, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b,
0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xc7, 0x69, 0x6c, 0x69, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0xc7, 0x69, 0x6c, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0xc7, 0x69, 0x6c, 0x69, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x48, 0x79, 0x74, 0x61, 0xfd, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x48, 0x79, 0x74, 0x61, 0xfd, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x48, 0x79, 0x74, 0x61, 0xfd, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73,
0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x52, 0x6f, 0x17e, 0x64,
0x65, 0x73, 0x74, 0x77, 0x6f, 0x20, 0x61, 0x64, 0x61, 0x73, 0x79, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x20, 0x61,
0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0xfd, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0xfd, 0x61,
0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x4b, 0x75, 0x6b, 0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72,
0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4b, 0x75, 0x6b, 0x20, 0x61,
0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4b, 0x75, 0x6b,
0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x74, 0x6f, 0x6d,
0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4b, 0x75,
0x62, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4b, 0x75, 0x62, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x4b, 0x75, 0x62, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75,
0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x44, 0x65, 0xfd,
0x77, 0x69, 0x73, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x44, 0xfd, 0x75,
0x6d, 0x6f, 0x6e, 0x2d, 0x64, 0x2d, 0xdd, 0x75, 0x72, 0x77, 0x69, 0x6c,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x64, 0x6f, 0x67,
0x61, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x50, 0x61, 0x73, 0x68, 0x61, 0x20, 0x61, 0x64, 0x61, 0x73,
0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x50, 0x61, 0x73, 0x68, 0x61,
0x20, 0x61, 0x64, 0x61, 0x73, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x50, 0x61, 0x73,
0x68, 0x61, 0x20, 0x61, 0x64, 0x61, 0x73, 0x79, 0x20, 0x74, 0x6f, 0x6d,
0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x45, 0x6b,
0x77, 0x61, 0x64, 0x6f, 0x72, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d,
0x65, 0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20, 0xdd, 0x65, 0x77, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x65, 0x72, 0x6b,
0x65, 0x7a, 0x69, 0x20, 0xdd, 0x65, 0x77, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x4d, 0x65, 0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20, 0xdd, 0x65,
0x77, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b,
0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x64, 0x6f,
0x67, 0x61, 0x72, 0x20, 0xdd, 0x65, 0x77, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61,
0x72, 0x20, 0xdd, 0x65, 0x77, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x47, 0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20, 0xdd, 0x65, 0x77,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x55, 0x7a, 0x61, 0x6b, 0x20, 0x47,
0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20, 0xdd, 0x65, 0x77, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e,
0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0xdd, 0x65, 0x77, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x62, 0x61,
0x74, 0x61, 0x72, 0x20, 0xdd, 0x65, 0x77, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x47, 0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0xdd,
0x65, 0x77, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73,
0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x46, 0x6f, 0x6c, 0x6b,
0x6c, 0x65, 0x6e, 0x64, 0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x65,
0x6e, 0x64, 0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x65, 0x6e, 0x64, 0x20, 0x61, 0x64,
0x61, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b,
0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x46, 0x69, 0x6a, 0x69, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x46, 0x69, 0x6a, 0x69, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x46, 0x72, 0x61, 0x6e, 0x73, 0x75,
0x7a, 0x20, 0x47, 0x77, 0x69, 0x61, 0x6e, 0x61, 0x73, 0x79, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x46, 0x72, 0x61, 0x6e, 0x73, 0x75, 0x7a, 0x20,
0x47, 0xfc, 0x6e, 0x6f, 0x72, 0x74, 0x61, 0x20, 0x77, 0x65, 0x20, 0x41,
0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x6b, 0x61, 0x20, 0xfd, 0x65,
0x72, 0x6c, 0x65, 0x72, 0x69, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47,
0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x20, 0x61, 0x64, 0x61,
0x6c, 0x61, 0x72, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0x61,
0x6d, 0x62, 0xfd, 0x65, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0x72,
0x75, 0x7a, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47,
0x72, 0x75, 0x7a, 0x69, 0xfd, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0x72, 0x75,
0x7a, 0x69, 0xfd, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72,
0x74, 0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x47, 0x72, 0x69, 0x6e, 0x77, 0x69, 0xe7, 0x20, 0x6f,
0x72, 0x74, 0x61, 0xe7, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47,
0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20, 0x47, 0x72, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x47, 0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20, 0x47, 0x72,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0xfd, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x47, 0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20, 0x47, 0x72, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0xfd, 0x61, 0x20, 0x74, 0x6f, 0x6d,
0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc,
0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x47, 0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0x47, 0x72, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0xfd, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47,
0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0x47, 0x72, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0xfd, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75,
0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x50, 0x61, 0x72,
0x73, 0x20, 0x61, 0xfd, 0x6c, 0x61, 0x67, 0x79, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47,
0x61, 0xfd, 0x61, 0x6e, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47,
0x61, 0x77, 0x61, 0xfd, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x47, 0x61, 0x77, 0x61, 0xfd, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0x61, 0x77, 0x61, 0xfd, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b,
0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0x6f, 0x6e, 0x6b, 0x6f,
0x6e, 0x67, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0x6f, 0x6e, 0x6b,
0x6f, 0x6e, 0x67, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e,
0x67, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x48, 0x6f, 0x77, 0x64, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x48, 0x6f, 0x77, 0x64, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x48, 0x6f, 0x77, 0x64,
0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x48, 0x69, 0x6e, 0x64, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x48, 0x69, 0x6e, 0x64, 0x69, 0x20, 0x75, 0x6d, 0x6d, 0x61,
0x6e, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x48, 0x69, 0x6e, 0x64,
0x69, 0x68, 0x79, 0x74, 0x61, 0xfd, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x4d, 0x65, 0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20, 0x49, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x7a, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x47, 0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20, 0x49, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x7a, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x47, 0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x45, 0xfd, 0x72, 0x61, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x45, 0xfd, 0x72, 0x61, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x45, 0xfd, 0x72,
0x61, 0x6e, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x74,
0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x59, 0x73, 0x72, 0x61, 0xfd, 0x79, 0x6c, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x59, 0x73, 0x72, 0x61, 0xfd, 0x79, 0x6c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x59,
0x73, 0x72, 0x61, 0xfd, 0x79, 0x6c, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73,
0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xdd, 0x61, 0x70, 0x6f,
0x6e, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xdd, 0x61,
0x70, 0x6f, 0x6e, 0x69, 0xfd, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xdd, 0x61, 0x70,
0x6f, 0x6e, 0x69, 0xfd, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b,
0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x77, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0xe7, 0x61, 0x74, 0x73, 0x6b, 0x69, 0xfd, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x47, 0x61, 0x7a, 0x61, 0x67, 0x79, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61,
0x72, 0x20, 0x47, 0x61, 0x7a, 0x61, 0x67, 0x79, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x62, 0x61, 0x74,
0x61, 0x72, 0x20, 0x47, 0x61, 0x7a, 0x61, 0x67, 0x79, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4b, 0x6f, 0x72, 0x65, 0xfd,
0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4b, 0x6f, 0x72, 0x65, 0xfd,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x4b, 0x6f, 0x72, 0x65, 0xfd, 0x61, 0x20, 0x74,
0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x4b, 0x6f, 0x73, 0x72, 0x61, 0xfd, 0x65, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0xfd, 0x61, 0x72, 0x73, 0x6b,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0xfd, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0xfd, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x74, 0x6f, 0x6d, 0x75,
0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0x79, 0x72,
0x67, 0x79, 0x7a, 0x79, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x4c, 0x61, 0xfd, 0x6e, 0x20, 0x61, 0x64, 0x61, 0x6c, 0x61,
0x72, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4c, 0x6f, 0x72, 0x64,
0x2d, 0x48, 0x61, 0x75, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4c, 0x6f,
0x72, 0x64, 0x2d, 0x48, 0x61, 0x75, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4c, 0x6f, 0x72,
0x64, 0x2d, 0x48, 0x61, 0x75, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b,
0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x4d, 0x61, 0x6c, 0x61, 0xfd, 0x7a, 0x69, 0xfd, 0x61,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x77,
0x6c, 0x65, 0x72, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x61, 0x72,
0x6b, 0x69, 0x7a, 0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x61, 0x72, 0x15f, 0x61, 0x6c, 0x6c,
0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x4d, 0x61, 0x77, 0x72, 0x69, 0x6b, 0x69, 0xfd, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x4d, 0x61, 0x77, 0x72, 0x69, 0x6b, 0x69, 0xfd,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x4d, 0x61, 0x77, 0x72, 0x69, 0x6b, 0x69, 0xfd, 0x20,
0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x4d, 0x6f, 0x75, 0x73, 0x6f, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0xdd, 0x75,
0x77, 0x61, 0x15f, 0x20, 0x75, 0x6d, 0x6d, 0x61, 0x6e, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x6e, 0x20,
0xdd, 0x75, 0x77, 0x61, 0x15f, 0x20, 0x75, 0x6d, 0x6d, 0x61, 0x6e, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0xdd,
0x75, 0x77, 0x61, 0x15f, 0x20, 0x75, 0x6d, 0x6d, 0x61, 0x6e, 0x20, 0x74,
0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74,
0x6f, 0x72, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x6f, 0x73, 0x6b, 0x77, 0x61, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x6f, 0x73, 0x6b, 0x77, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x4d, 0x6f, 0x73, 0x6b, 0x77, 0x61, 0x20, 0x74, 0x6f, 0x6d,
0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0xfd,
0x61, 0x6e, 0x6d, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4e, 0x61,
0x75, 0x72, 0x75, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4e, 0x65, 0x70,
0x61, 0x6c, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x54, 0xe4, 0x7a, 0x65,
0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0xfd, 0x61, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x54, 0xe4, 0x7a, 0x65, 0x20, 0x4b, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0xfd, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x54,
0xe4, 0x7a, 0x65, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69,
0xfd, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x54, 0xe4, 0x7a, 0x65, 0x20, 0x5a, 0x65, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x54, 0xe4, 0x7a, 0x65, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0xfd, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x54, 0xe4, 0x7a, 0x65, 0x20, 0x5a, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0xfd, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75,
0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4e, 0xfd, 0x75,
0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x4e, 0xfd, 0x75, 0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c,
0x65, 0x6e, 0x64, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4e, 0xfd, 0x75, 0x66, 0x61, 0x75,
0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73,
0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4e, 0x69, 0x75, 0x65,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x20, 0x61, 0x64, 0x61, 0x73, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x61, 0x64, 0x61,
0x73, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x20, 0x61, 0x64, 0x61, 0x73, 0x79, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73,
0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x75, 0x2d, 0x64, 0x69, 0x2d, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x75, 0x2d, 0x64, 0x69, 0x2d, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0xfd, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x75, 0x2d, 0x64, 0x69, 0x2d, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0xfd, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4e, 0x6f, 0x77,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4e,
0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x6f,
0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4f,
0x6d, 0x73, 0x6b, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4f, 0x6d, 0x73,
0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x74, 0x6f, 0x6d,
0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x74, 0x6f, 0x6d, 0x75,
0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x50, 0x61, 0x6c,
0x61, 0x75, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x50, 0x61, 0x70, 0x75,
0x61, 0x20, 0x2d, 0x20, 0x54, 0xe4, 0x7a, 0x65, 0x20, 0x47, 0x77, 0x69,
0x6e, 0x65, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x50, 0x61,
0x72, 0x61, 0x67, 0x77, 0x61, 0xfd, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x50, 0x61, 0x72, 0x61, 0x67, 0x77, 0x61, 0xfd, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x50,
0x61, 0x72, 0x61, 0x67, 0x77, 0x61, 0xfd, 0x20, 0x74, 0x6f, 0x6d, 0x75,
0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x50, 0x65, 0x72,
0x75, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x50, 0x65, 0x72, 0x75, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x50, 0x65, 0x72, 0x75, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73,
0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x70, 0x69, 0x6e, 0x6c, 0x65, 0x72, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x6c, 0x65, 0x72,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x6c,
0x65, 0x72, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x46, 0x65, 0x6e, 0x69, 0x6b, 0x73, 0x20, 0x61,
0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x53, 0x65, 0x6e, 0x2d, 0x50, 0xfd, 0x65, 0x72, 0x20, 0x77, 0x65, 0x20,
0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x53, 0x65, 0x6e, 0x2d, 0x50,
0xfd, 0x65, 0x72, 0x20, 0x77, 0x65, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c,
0x6f, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x53, 0x65, 0x6e, 0x2d, 0x50, 0xfd, 0x65,
0x72, 0x20, 0x77, 0x65, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e,
0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x50, 0x69, 0x74, 0x6b, 0x65, 0x72, 0x6e, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x50, 0x68, 0x65, 0x6e, 0xfd, 0x61, 0x6e, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x52, 0x65, 0xfd, 0x75, 0x6e, 0xfd, 0x6f, 0x6e,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x52, 0x6f, 0x74, 0x65, 0x72, 0x61,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x53, 0x61, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x53, 0x65, 0xfd, 0x15f, 0x65, 0x6c, 0x20, 0x61,
0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x20, 0x61, 0x64,
0x61, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47,
0xfc, 0x6e, 0x6f, 0x72, 0x74, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67,
0x69, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x53, 0x75, 0x72,
0x69, 0x6e, 0x61, 0x6d, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x53, 0xfd,
0x6f, 0x77, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x54, 0x61, 0x69,
0x74, 0x69, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x54, 0x61, 0xfd, 0x62,
0x65, 0xfd, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x54, 0x61, 0xfd, 0x62,
0x65, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x54, 0x61, 0xfd, 0x62, 0x65, 0xfd, 0x20,
0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x54, 0xe4, 0x6a, 0x69, 0x67, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73,
0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xc7, 0x75, 0x75, 0x6b,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x54, 0xfc, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x54, 0xfc, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x54, 0xfc, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x54, 0x75, 0x77, 0x61, 0x6c, 0x75, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x55, 0x72, 0x75, 0x67, 0x77, 0x61, 0xfd,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x55, 0x72, 0x75, 0x67, 0x77, 0x61,
0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x55, 0x72, 0x75, 0x67, 0x77, 0x61, 0xfd, 0x20,
0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0xd6, 0x7a, 0x62, 0x65, 0x67, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0xd6, 0x7a, 0x62, 0x65, 0x67, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xd6, 0x7a, 0x62, 0x65, 0x67, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x57, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x57, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x57, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75,
0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x57, 0x65, 0x6e, 0x65, 0x73, 0x75, 0x65, 0x6c, 0x61, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x57, 0x6c,
0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20,
0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x57, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x57, 0x65, 0xfd, 0x6b, 0x20, 0x61, 0x64, 0x61, 0x73,
0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x55, 0x6f, 0x6c, 0x6c, 0x69,
0x73, 0x20, 0x77, 0x65, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0xdd, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xdd, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0xdd, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20,
0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0xdd, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xdd, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0xdd, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0xdd, 0x75, 0x6b, 0x6f, 0x6e, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x447, 0x430, 0x441, 0x3a, 0x20, 0x410, 0x43a, 0x440, 0x456, 0x447,
0x430, 0x441, 0x3a, 0x20, 0x410, 0x43a, 0x440, 0x456, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x439, 0x447, 0x430, 0x441,
0x3a, 0x20, 0x410, 0x43a, 0x440, 0x456, 0x2c, 0x20, 0x43b, 0x456, 0x442, 0x43d,
0x456, 0x439, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x432,
0x20, 0x410, 0x444, 0x433, 0x430, 0x43d, 0x456, 0x441, 0x442, 0x430, 0x43d, 0x456,
0x437, 0x430, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d,
0x43e, 0x430, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x445,
0x456, 0x434, 0x43d, 0x43e, 0x430, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x43f, 0x456, 0x432, 0x434, 0x435, 0x43d, 0x43d, 0x43e, 0x430, 0x444, 0x440,
0x438, 0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x437, 0x430, 0x445, 0x456, 0x434, 0x43d,
0x43e, 0x430, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x437, 0x430,
0x445, 0x456, 0x434, 0x43d, 0x43e, 0x430, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x43d,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x437, 0x430, 0x445, 0x456, 0x434, 0x43d, 0x43e, 0x430, 0x444, 0x440,
0x438, 0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456,
0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x410, 0x43b,
0x44f, 0x441, 0x446, 0x456, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x20, 0x43d, 0x430, 0x20, 0x410, 0x43b, 0x44f, 0x441, 0x446, 0x456, 0x437, 0x430,
0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x43d, 0x430, 0x20, 0x410, 0x43b, 0x44f, 0x441, 0x446, 0x456, 0x437,
0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x410,
0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x446, 0x456, 0x437, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x410, 0x43c, 0x430, 0x437, 0x43e,
0x43d, 0x446, 0x456, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x410, 0x43c,
0x430, 0x437, 0x43e, 0x43d, 0x446, 0x456, 0x437, 0x430, 0x20, 0x43f, 0x456, 0x432,
0x43d, 0x456, 0x447, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430,
0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440,
0x430, 0x43b, 0x44c, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x437, 0x430, 0x20, 0x43f, 0x456, 0x432, 0x43d, 0x456, 0x447, 0x43d, 0x43e, 0x430,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c,
0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x438, 0x43c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43f, 0x456, 0x432,
0x43d, 0x456, 0x447, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430,
0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440,
0x430, 0x43b, 0x44c, 0x43d, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43f, 0x456,
0x432, 0x43d, 0x456, 0x447, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a,
0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x445, 0x456, 0x434,
0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20,
0x43f, 0x456, 0x432, 0x43d, 0x456, 0x447, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x445,
0x456, 0x434, 0x43d, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x43f, 0x456, 0x432, 0x43d, 0x456, 0x447, 0x43d, 0x43e, 0x430, 0x43c,
0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x441, 0x445, 0x456, 0x434, 0x43d, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d,
0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43f,
0x456, 0x432, 0x43d, 0x456, 0x447, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438,
0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x433, 0x456, 0x440,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x43f, 0x456, 0x432, 0x43d, 0x456, 0x447, 0x43d, 0x43e, 0x430, 0x43c,
0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x433, 0x456, 0x440, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43f, 0x456, 0x432, 0x43d, 0x456, 0x447, 0x43d,
0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x433, 0x456, 0x440, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x437, 0x430, 0x20, 0x43f, 0x456, 0x432, 0x43d, 0x456, 0x447, 0x43d, 0x43e, 0x430,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c,
0x20, 0x442, 0x438, 0x445, 0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441, 0x44c,
0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20,
0x43f, 0x456, 0x432, 0x43d, 0x456, 0x447, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x442, 0x438,
0x445, 0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43f, 0x456, 0x432,
0x43d, 0x456, 0x447, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430,
0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x442, 0x438, 0x445, 0x43e, 0x43e,
0x43a, 0x435, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456,
0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x447, 0x430,
0x441, 0x3a, 0x20, 0x410, 0x43d, 0x430, 0x434, 0x438, 0x440, 0x447, 0x430, 0x441,
0x3a, 0x20, 0x410, 0x43d, 0x430, 0x434, 0x438, 0x440, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x439, 0x447, 0x430, 0x441,
0x3a, 0x20, 0x410, 0x43d, 0x430, 0x434, 0x438, 0x440, 0x2c, 0x20, 0x43b, 0x456,
0x442, 0x43d, 0x456, 0x439, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x20, 0x432, 0x20, 0x410, 0x43f, 0x456, 0x430, 0x437, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x432, 0x20, 0x410, 0x43f, 0x456, 0x430, 0x437, 0x430,
0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x432, 0x20, 0x410, 0x43f, 0x456, 0x430, 0x437, 0x430, 0x20, 0x430,
0x440, 0x430, 0x431, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x430, 0x440, 0x430, 0x431, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x430,
0x440, 0x430, 0x431, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442,
0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20,
0x430, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x441, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x430, 0x440,
0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d,
0x456, 0x43c, 0x20, 0x430, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x437, 0x430, 0x445, 0x456, 0x434, 0x43d, 0x43e, 0x430, 0x440, 0x433, 0x435,
0x43d, 0x442, 0x438, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x437, 0x430, 0x445, 0x456, 0x434, 0x43d,
0x43e, 0x430, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43b,
0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x437, 0x430, 0x20, 0x437, 0x430, 0x445,
0x456, 0x434, 0x43d, 0x43e, 0x430, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x432, 0x456, 0x440, 0x43c, 0x435, 0x43d, 0x441, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x432, 0x456,
0x440, 0x43c, 0x435, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x432, 0x456, 0x440, 0x43c, 0x435, 0x43d,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x430, 0x442, 0x43b,
0x430, 0x43d, 0x442, 0x438, 0x447, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x430, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438,
0x447, 0x43d, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x430, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x447, 0x43d, 0x438, 0x43c,
0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x437, 0x430, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c,
0x43d, 0x43e, 0x430, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x439, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c,
0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x43e, 0x430,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43b, 0x456,
0x442, 0x43d, 0x456, 0x43c, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b,
0x44c, 0x43d, 0x43e, 0x430, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x439,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x43e,
0x437, 0x430, 0x445, 0x456, 0x434, 0x43d, 0x438, 0x43c, 0x20, 0x430, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x446, 0x435, 0x43d, 0x442,
0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x43e, 0x437, 0x430, 0x445, 0x456, 0x434, 0x43d,
0x438, 0x43c, 0x20, 0x430, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x439,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x446, 0x435, 0x43d,
0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x43e, 0x437, 0x430, 0x445, 0x456, 0x434,
0x43d, 0x438, 0x43c, 0x20, 0x430, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456,
0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x437, 0x430, 0x20, 0x441, 0x445, 0x456, 0x434, 0x43d, 0x43e, 0x430, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x441, 0x445, 0x456, 0x434,
0x43d, 0x43e, 0x430, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x439, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x441, 0x445, 0x456, 0x434,
0x43d, 0x43e, 0x430, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x439, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x437, 0x430, 0x445, 0x456, 0x434, 0x43d, 0x43e, 0x430, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x437, 0x430, 0x445, 0x456, 0x434,
0x43d, 0x43e, 0x430, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x439, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x437, 0x430, 0x445, 0x456,
0x434, 0x43d, 0x43e, 0x430, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x439,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x430, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x434, 0x436, 0x430,
0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x438, 0x43c, 0x20, 0x430, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x434, 0x436,
0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x430,
0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x434, 0x436, 0x430, 0x43d, 0x441, 0x44c,
0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x410, 0x437, 0x43e,
0x440, 0x441, 0x44c, 0x43a, 0x438, 0x445, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e,
0x432, 0x430, 0x445, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x43d, 0x430, 0x20, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x44c, 0x43a, 0x438, 0x445,
0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x437, 0x430, 0x20,
0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x20, 0x43d, 0x430, 0x20, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x44c, 0x43a, 0x438,
0x445, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x437, 0x430,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x411, 0x430, 0x43d,
0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x443, 0x20, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434,
0x435, 0x448, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x411, 0x430, 0x43d, 0x433,
0x43b, 0x430, 0x434, 0x435, 0x448, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x443, 0x20, 0x411, 0x443, 0x442, 0x430, 0x43d, 0x456, 0x437, 0x430,
0x20, 0x431, 0x43e, 0x43b, 0x456, 0x432, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x431, 0x440,
0x430, 0x437, 0x438, 0x43b, 0x44c, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x431, 0x440, 0x430, 0x437, 0x438,
0x43b, 0x44c, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x431,
0x440, 0x430, 0x437, 0x438, 0x43b, 0x44c, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x443, 0x20, 0x411, 0x440, 0x443, 0x43d, 0x435, 0x457, 0x437, 0x430,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x41a, 0x430, 0x431, 0x43e, 0x2d,
0x412, 0x435, 0x440, 0x434, 0x435, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430,
0x445, 0x20, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435,
0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e,
0x432, 0x430, 0x445, 0x20, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440,
0x434, 0x435, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d,
0x430, 0x20, 0x41f, 0x456, 0x432, 0x43d, 0x456, 0x447, 0x43d, 0x438, 0x445, 0x20,
0x41c, 0x430, 0x440, 0x456, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x445, 0x20,
0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x437, 0x430, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x430, 0x440, 0x445, 0x456,
0x43f, 0x435, 0x43b, 0x430, 0x437, 0x456, 0x20, 0x427, 0x430, 0x442, 0x435, 0x43c,
0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20,
0x430, 0x440, 0x445, 0x456, 0x43f, 0x435, 0x43b, 0x430, 0x437, 0x456, 0x20, 0x427,
0x430, 0x442, 0x435, 0x43c, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x430,
0x440, 0x445, 0x456, 0x43f, 0x435, 0x43b, 0x430, 0x437, 0x456, 0x20, 0x427, 0x430,
0x442, 0x435, 0x43c, 0x437, 0x430, 0x20, 0x447, 0x438, 0x43b, 0x456, 0x439, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c,
0x20, 0x447, 0x438, 0x43b, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d,
0x456, 0x43c, 0x20, 0x447, 0x438, 0x43b, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43a, 0x438,
0x442, 0x430, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43a, 0x438, 0x442, 0x430, 0x439, 0x441, 0x44c,
0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20,
0x43a, 0x438, 0x442, 0x430, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b,
0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x420, 0x456, 0x437, 0x434, 0x432,
0x430, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430,
0x20, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x43e, 0x432, 0x438, 0x445, 0x20, 0x43e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x437, 0x430, 0x20, 0x43a, 0x43e,
0x43b, 0x443, 0x43c, 0x431, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x43a, 0x43e, 0x43b, 0x443,
0x43c, 0x431, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c,
0x20, 0x43a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x456, 0x439, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x430, 0x445, 0x20, 0x41a, 0x443, 0x43a, 0x430, 0x437, 0x430, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x41e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x41a, 0x443, 0x43a, 0x430, 0x437, 0x430,
0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x43d, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430,
0x445, 0x20, 0x41a, 0x443, 0x43a, 0x430, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x41a, 0x443, 0x431, 0x456, 0x437, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x41a, 0x443,
0x431, 0x456, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x41a, 0x443, 0x431,
0x456, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x446, 0x456, 0x457, 0x20, 0x414, 0x435, 0x432,
0x456, 0x441, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d,
0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x446, 0x456, 0x457, 0x20, 0x414, 0x44e,
0x43c, 0x43e, 0x43d, 0x20, 0x434, 0x2bc, 0x42e, 0x440, 0x432, 0x456, 0x43b, 0x44c,
0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x421,
0x445, 0x456, 0x434, 0x43d, 0x43e, 0x43c, 0x443, 0x20, 0x422, 0x438, 0x43c, 0x43e,
0x440, 0x456, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d,
0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x41f, 0x430,
0x441, 0x445, 0x438, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x41f,
0x430, 0x441, 0x445, 0x438, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x41f, 0x430, 0x441, 0x445, 0x438,
0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x432, 0x20, 0x415,
0x43a, 0x432, 0x430, 0x434, 0x43e, 0x440, 0x456, 0x437, 0x430, 0x20, 0x446, 0x435,
0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x43e, 0x454, 0x432, 0x440, 0x43e,
0x43f, 0x435, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b,
0x44c, 0x43d, 0x43e, 0x454, 0x432, 0x440, 0x43e, 0x43f, 0x435, 0x439, 0x441, 0x44c,
0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20,
0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x43e, 0x454, 0x432,
0x440, 0x43e, 0x43f, 0x435, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b,
0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x441, 0x445, 0x456, 0x434, 0x43d, 0x43e, 0x454, 0x432, 0x440, 0x43e,
0x43f, 0x435, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x445, 0x456, 0x434, 0x43d, 0x43e, 0x454,
0x432, 0x440, 0x43e, 0x43f, 0x435, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x445, 0x456, 0x434,
0x43d, 0x43e, 0x454, 0x432, 0x440, 0x43e, 0x43f, 0x435, 0x439, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x434, 0x430, 0x43b, 0x435, 0x43a, 0x43e,
0x441, 0x445, 0x456, 0x434, 0x43d, 0x438, 0x43c, 0x20, 0x454, 0x432, 0x440, 0x43e,
0x43f, 0x435, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x437, 0x430, 0x445, 0x456, 0x434, 0x43d, 0x43e,
0x454, 0x432, 0x440, 0x43e, 0x43f, 0x435, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x437, 0x430, 0x445,
0x456, 0x434, 0x43d, 0x43e, 0x454, 0x432, 0x440, 0x43e, 0x43f, 0x435, 0x439, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x437, 0x430, 0x445, 0x456, 0x434, 0x43d, 0x43e, 0x454, 0x432, 0x440, 0x43e,
0x43f, 0x435, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442,
0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x424, 0x43e, 0x43b,
0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x441, 0x44c, 0x43a, 0x438, 0x445, 0x20, 0x41e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x437, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x424, 0x43e, 0x43b, 0x43a, 0x43b,
0x435, 0x43d, 0x434, 0x441, 0x44c, 0x43a, 0x438, 0x445, 0x20, 0x41e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x430, 0x445, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d,
0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20,
0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x441, 0x44c, 0x43a, 0x438,
0x445, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x437, 0x430,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x424, 0x456, 0x434,
0x436, 0x456, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443,
0x20, 0x424, 0x456, 0x434, 0x436, 0x456, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442,
0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20,
0x424, 0x456, 0x434, 0x436, 0x456, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x443, 0x437, 0x44c, 0x43a, 0x43e,
0x457, 0x20, 0x490, 0x432, 0x456, 0x430, 0x43d, 0x438, 0x437, 0x430, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x424, 0x440, 0x430, 0x43d,
0x446, 0x443, 0x437, 0x44c, 0x43a, 0x438, 0x445, 0x20, 0x41f, 0x456, 0x432, 0x434,
0x435, 0x43d, 0x43d, 0x438, 0x445, 0x20, 0x456, 0x20, 0x410, 0x43d, 0x442, 0x430,
0x440, 0x43a, 0x442, 0x438, 0x447, 0x43d, 0x438, 0x445, 0x20, 0x442, 0x435, 0x440,
0x438, 0x442, 0x43e, 0x440, 0x456, 0x44f, 0x445, 0x437, 0x430, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x490, 0x430, 0x43b, 0x430, 0x43f, 0x430, 0x491, 0x43e,
0x441, 0x443, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d,
0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x490, 0x430,
0x43c, 0x431, 0x2bc, 0x454, 0x437, 0x430, 0x20, 0x433, 0x440, 0x443, 0x437, 0x438,
0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x438, 0x43c, 0x20, 0x433, 0x440, 0x443, 0x437, 0x438, 0x43d, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43b,
0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x433, 0x440, 0x443, 0x437, 0x438, 0x43d,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x413, 0x456, 0x43b, 0x431,
0x435, 0x440, 0x442, 0x430, 0x437, 0x430, 0x20, 0x413, 0x440, 0x438, 0x43d, 0x432,
0x456, 0x447, 0x435, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x445, 0x456, 0x434, 0x43d,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x490,
0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x456, 0x457, 0x437, 0x430, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20,
0x441, 0x445, 0x456, 0x434, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x443, 0x20, 0x490, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434,
0x456, 0x457, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20,
0x441, 0x445, 0x456, 0x434, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x443, 0x20, 0x490, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434,
0x456, 0x457, 0x437, 0x430, 0x20, 0x437, 0x430, 0x445, 0x456, 0x434, 0x43d, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x490, 0x440,
0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x456, 0x457, 0x437, 0x430, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x437,
0x430, 0x445, 0x456, 0x434, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x443, 0x20, 0x490, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434,
0x456, 0x457, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20,
0x437, 0x430, 0x445, 0x456, 0x434, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x443, 0x20, 0x490, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d,
0x434, 0x456, 0x457, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x413,
0x443, 0x430, 0x43c, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x41f, 0x435, 0x440, 0x441, 0x44c, 0x43a, 0x43e, 0x457, 0x20, 0x437, 0x430, 0x442,
0x43e, 0x43a, 0x438, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x443, 0x20, 0x490, 0x430, 0x44f, 0x43d, 0x456, 0x437, 0x430, 0x20, 0x433, 0x430,
0x432, 0x430, 0x439, 0x441, 0x44c, 0x43a, 0x43e, 0x2d, 0x430, 0x43b, 0x435, 0x443,
0x442, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x438, 0x43c, 0x20, 0x433, 0x430, 0x432, 0x430, 0x439, 0x441, 0x44c, 0x43a, 0x43e,
0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d,
0x456, 0x43c, 0x20, 0x433, 0x430, 0x432, 0x430, 0x439, 0x441, 0x44c, 0x43a, 0x43e,
0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x443, 0x20, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x437, 0x456,
0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x413,
0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x437, 0x456, 0x437, 0x430, 0x20, 0x43b, 0x456,
0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443,
0x20, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x437, 0x456, 0x437, 0x430, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x425, 0x43e, 0x432, 0x434,
0x456, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20,
0x425, 0x43e, 0x432, 0x434, 0x456, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d,
0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x425,
0x43e, 0x432, 0x434, 0x456, 0x437, 0x430, 0x20, 0x456, 0x43d, 0x434, 0x456, 0x439,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x432, 0x20, 0x406, 0x43d,
0x434, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x43e, 0x43c, 0x443, 0x20, 0x41e, 0x43a,
0x435, 0x430, 0x43d, 0x456, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x20, 0x432, 0x20, 0x406, 0x43d, 0x434, 0x43e, 0x43a, 0x438, 0x442, 0x430, 0x457,
0x437, 0x430, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d,
0x43e, 0x456, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x456, 0x439, 0x441, 0x44c,
0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20,
0x441, 0x445, 0x456, 0x434, 0x43d, 0x43e, 0x456, 0x43d, 0x434, 0x43e, 0x43d, 0x435,
0x437, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x437, 0x430, 0x445, 0x456, 0x434, 0x43d, 0x43e,
0x456, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x456, 0x439, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x456,
0x440, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x456, 0x440, 0x430, 0x43d, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x456,
0x440, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442,
0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20,
0x456, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x456, 0x440, 0x43a, 0x443, 0x442,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x456, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x44c, 0x43a, 0x438, 0x43c,
0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x437, 0x430, 0x20, 0x456, 0x437, 0x440, 0x430, 0x457, 0x43b, 0x44c, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x456, 0x437, 0x440, 0x430, 0x457, 0x43b, 0x44c, 0x441, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x456, 0x437,
0x440, 0x430, 0x457, 0x43b, 0x44c, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b,
0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x44f, 0x43f, 0x43e, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x44f, 0x43f, 0x43e, 0x43d,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x44f, 0x43f, 0x43e, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x437, 0x430, 0x20, 0x43a, 0x430, 0x43c, 0x447, 0x430, 0x442, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43a,
0x430, 0x43c, 0x447, 0x430, 0x442, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43a, 0x430, 0x43c, 0x447, 0x430,
0x442, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441,
0x442, 0x430, 0x43d, 0x456, 0x437, 0x430, 0x20, 0x441, 0x445, 0x456, 0x434, 0x43d,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41a,
0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x456, 0x437, 0x430, 0x20,
0x437, 0x430, 0x445, 0x456, 0x434, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442,
0x430, 0x43d, 0x456, 0x437, 0x430, 0x20, 0x43a, 0x43e, 0x440, 0x435, 0x439, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x43a, 0x43e, 0x440, 0x435, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43a, 0x43e, 0x440, 0x435,
0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e,
0x432, 0x456, 0x20, 0x41a, 0x43e, 0x441, 0x440, 0x430, 0x435, 0x437, 0x430, 0x20,
0x43a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43a, 0x440,
0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43a, 0x440, 0x430, 0x441,
0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456,
0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41a, 0x438, 0x440,
0x433, 0x438, 0x437, 0x441, 0x442, 0x430, 0x43d, 0x456, 0x447, 0x430, 0x441, 0x3a,
0x20, 0x41b, 0x430, 0x43d, 0x43a, 0x430, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432,
0x456, 0x20, 0x41b, 0x430, 0x439, 0x43d, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432,
0x456, 0x20, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x425, 0x430, 0x443, 0x437, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x425,
0x430, 0x443, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x456, 0x20, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x425, 0x430,
0x443, 0x437, 0x430, 0x20, 0x43c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x43c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43c, 0x430,
0x433, 0x430, 0x434, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b,
0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41c, 0x430,
0x43b, 0x430, 0x439, 0x437, 0x456, 0x457, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x41c, 0x430, 0x43b, 0x44c, 0x434, 0x456,
0x432, 0x430, 0x445, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x43d, 0x430, 0x20, 0x41c, 0x430, 0x440, 0x43a, 0x456, 0x437, 0x44c, 0x43a, 0x438,
0x445, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x437, 0x430,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x41c, 0x430,
0x440, 0x448, 0x430, 0x43b, 0x43b, 0x43e, 0x432, 0x438, 0x445, 0x20, 0x41e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432,
0x456, 0x20, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x456, 0x439, 0x437, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a,
0x456, 0x439, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x456, 0x20, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x456,
0x439, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x446, 0x456, 0x457, 0x20, 0x41c, 0x43e, 0x443,
0x441, 0x43e, 0x43d, 0x437, 0x430, 0x20, 0x442, 0x438, 0x445, 0x43e, 0x43e, 0x43a,
0x435, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x446, 0x456,
0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x438, 0x43c, 0x20, 0x442, 0x438, 0x445, 0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x443, 0x20, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x446, 0x456, 0x437, 0x430, 0x20,
0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x442, 0x438, 0x445, 0x43e, 0x43e,
0x43a, 0x435, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x446,
0x456, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x432, 0x20,
0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x456, 0x437,
0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x432, 0x20, 0x423, 0x43b,
0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x456, 0x437, 0x430, 0x20,
0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x20, 0x432, 0x20, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e,
0x440, 0x456, 0x437, 0x430, 0x20, 0x43c, 0x43e, 0x441, 0x43a, 0x43e, 0x432, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x43c, 0x43e, 0x441, 0x43a, 0x43e, 0x432, 0x441, 0x44c, 0x43a, 0x438, 0x43c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43c, 0x43e, 0x441,
0x43a, 0x43e, 0x432, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442,
0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41c, 0x2bc, 0x44f, 0x43d,
0x43c, 0x456, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d,
0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x41d, 0x430,
0x443, 0x440, 0x443, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x443, 0x20, 0x41d, 0x435, 0x43f, 0x430, 0x43b, 0x456, 0x437, 0x430, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x430, 0x445, 0x20, 0x41d, 0x43e, 0x432, 0x43e, 0x457, 0x20, 0x41a,
0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x456, 0x457, 0x437, 0x430, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x430, 0x445, 0x20, 0x41d, 0x43e, 0x432, 0x43e, 0x457, 0x20, 0x41a,
0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x456, 0x457, 0x437, 0x430, 0x20, 0x43b,
0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20,
0x41d, 0x43e, 0x432, 0x43e, 0x457, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e,
0x43d, 0x456, 0x457, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x443, 0x20, 0x41d, 0x43e, 0x432, 0x456, 0x439, 0x20, 0x417, 0x435, 0x43b, 0x430,
0x43d, 0x434, 0x456, 0x457, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x20, 0x443, 0x20, 0x41d, 0x43e, 0x432, 0x456, 0x439, 0x20, 0x417, 0x435, 0x43b,
0x430, 0x43d, 0x434, 0x456, 0x457, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d,
0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41d,
0x43e, 0x432, 0x456, 0x439, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x456,
0x457, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430,
0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x41d, 0x44c, 0x44e,
0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x437, 0x430, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x456, 0x20, 0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x443, 0x43d,
0x434, 0x43b, 0x435, 0x43d, 0x434, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d,
0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41d,
0x44c, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x437,
0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x41d, 0x456, 0x443, 0x435, 0x437,
0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x41d, 0x43e, 0x440, 0x444, 0x43e,
0x43b, 0x43a, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d,
0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x41d, 0x43e,
0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d,
0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20,
0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x41d, 0x43e, 0x440, 0x444,
0x43e, 0x43b, 0x43a, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x43d, 0x430, 0x20, 0x430, 0x440, 0x445, 0x456, 0x43f, 0x435, 0x43b, 0x430, 0x437,
0x456, 0x20, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434,
0x456, 0x2d, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x437, 0x430, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x430, 0x440, 0x445,
0x456, 0x43f, 0x435, 0x43b, 0x430, 0x437, 0x456, 0x20, 0x424, 0x435, 0x440, 0x43d,
0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x456, 0x2d, 0x41d, 0x43e, 0x440, 0x43e,
0x43d, 0x44c, 0x44f, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x430, 0x440,
0x445, 0x456, 0x43f, 0x435, 0x43b, 0x430, 0x437, 0x456, 0x20, 0x424, 0x435, 0x440,
0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x456, 0x2d, 0x41d, 0x43e, 0x440,
0x43e, 0x43d, 0x44c, 0x44f, 0x437, 0x430, 0x20, 0x43d, 0x43e, 0x432, 0x43e, 0x441,
0x438, 0x431, 0x456, 0x440, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43d, 0x43e, 0x432, 0x43e, 0x441, 0x438,
0x431, 0x456, 0x440, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431,
0x456, 0x440, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d,
0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43e,
0x43c, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x437, 0x430, 0x20, 0x43e, 0x43c, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43e, 0x43c, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x443, 0x20, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x456, 0x437,
0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41f, 0x430,
0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x456, 0x437, 0x430, 0x20, 0x43b, 0x456,
0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443,
0x20, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x456, 0x437, 0x430,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x41f, 0x430, 0x43b, 0x430, 0x443, 0x437,
0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x41f, 0x430, 0x43f, 0x443,
0x430, 0x2d, 0x41d, 0x43e, 0x432, 0x430, 0x20, 0x490, 0x432, 0x456, 0x43d, 0x435,
0x44f, 0x437, 0x430, 0x20, 0x43f, 0x430, 0x440, 0x430, 0x491, 0x432, 0x430, 0x439,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438,
0x43c, 0x20, 0x43f, 0x430, 0x440, 0x430, 0x491, 0x432, 0x430, 0x439, 0x441, 0x44c,
0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20,
0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x43f, 0x430, 0x440, 0x430, 0x491,
0x432, 0x430, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443,
0x20, 0x41f, 0x435, 0x440, 0x443, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x443, 0x20, 0x41f, 0x435, 0x440, 0x443, 0x437, 0x430, 0x20, 0x43b,
0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x443, 0x20, 0x41f, 0x435, 0x440, 0x443, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x424, 0x456, 0x43b, 0x456, 0x43f, 0x43f,
0x456, 0x43d, 0x430, 0x445, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x20, 0x43d, 0x430, 0x20, 0x424, 0x456, 0x43b, 0x456, 0x43f, 0x43f, 0x456, 0x43d,
0x430, 0x445, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x424, 0x456, 0x43b,
0x456, 0x43f, 0x43f, 0x456, 0x43d, 0x430, 0x445, 0x437, 0x430, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e,
0x432, 0x430, 0x445, 0x20, 0x424, 0x435, 0x43d, 0x456, 0x43a, 0x441, 0x437, 0x430,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x421, 0x435, 0x43d, 0x2d, 0x41f,
0x2019, 0x454, 0x440, 0x20, 0x456, 0x20, 0x41c, 0x456, 0x43a, 0x435, 0x43b, 0x43e,
0x43d, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430,
0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x421, 0x435,
0x43d, 0x2d, 0x41f, 0x2019, 0x454, 0x440, 0x20, 0x456, 0x20, 0x41c, 0x456, 0x43a,
0x435, 0x43b, 0x43e, 0x43d, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x421, 0x435, 0x43d, 0x2d,
0x41f, 0x2019, 0x454, 0x440, 0x20, 0x456, 0x20, 0x41c, 0x456, 0x43a, 0x435, 0x43b,
0x43e, 0x43d, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d,
0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x41f,
0x456, 0x442, 0x43a, 0x435, 0x440, 0x43d, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432,
0x456, 0x20, 0x41f, 0x43e, 0x43d, 0x430, 0x43f, 0x435, 0x437, 0x430, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41f, 0x445, 0x435, 0x43d, 0x44c,
0x44f, 0x43d, 0x456, 0x447, 0x430, 0x441, 0x3a, 0x20, 0x41a, 0x438, 0x437, 0x438,
0x43b, 0x43e, 0x440, 0x434, 0x430, 0x447, 0x430, 0x441, 0x3a, 0x20, 0x41a, 0x438,
0x437, 0x438, 0x43b, 0x43e, 0x440, 0x434, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x439, 0x447, 0x430, 0x441, 0x3a,
0x20, 0x41a, 0x438, 0x437, 0x438, 0x43b, 0x43e, 0x440, 0x434, 0x430, 0x2c, 0x20,
0x43b, 0x456, 0x442, 0x43d, 0x456, 0x439, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432,
0x456, 0x20, 0x420, 0x435, 0x44e, 0x43d, 0x44c, 0x439, 0x43e, 0x43d, 0x437, 0x430,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x446, 0x456, 0x457, 0x20, 0x420, 0x43e, 0x442, 0x435, 0x440, 0x430,
0x437, 0x430, 0x20, 0x441, 0x430, 0x445, 0x430, 0x43b, 0x456, 0x43d, 0x441, 0x44c,
0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20,
0x441, 0x430, 0x445, 0x430, 0x43b, 0x456, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x430, 0x445,
0x430, 0x43b, 0x456, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456,
0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x441, 0x430, 0x43c, 0x430, 0x440, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x430, 0x43c, 0x430,
0x440, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x437, 0x430, 0x20, 0x441, 0x430, 0x43c, 0x430, 0x440, 0x441, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d,
0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x421, 0x430,
0x43c, 0x43e, 0x430, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x421,
0x430, 0x43c, 0x43e, 0x430, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x421, 0x430, 0x43c, 0x43e, 0x430,
0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20,
0x421, 0x435, 0x439, 0x448, 0x435, 0x43b, 0x44c, 0x441, 0x44c, 0x43a, 0x438, 0x445,
0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x437, 0x430, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x421, 0x456, 0x43d, 0x433,
0x430, 0x43f, 0x443, 0x440, 0x456, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x43d, 0x430, 0x20, 0x421, 0x43e, 0x43b, 0x43e, 0x43c, 0x43e, 0x43d,
0x43e, 0x432, 0x438, 0x445, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430,
0x445, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430,
0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x41f, 0x456, 0x432,
0x434, 0x435, 0x43d, 0x43d, 0x430, 0x20, 0x414, 0x436, 0x43e, 0x440, 0x434, 0x436,
0x456, 0x44f, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443,
0x20, 0x421, 0x443, 0x440, 0x438, 0x43d, 0x430, 0x43c, 0x456, 0x437, 0x430, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x446, 0x456, 0x457, 0x20, 0x421, 0x435, 0x432, 0x430, 0x437, 0x430, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x456, 0x20, 0x422, 0x430, 0x457, 0x442, 0x456, 0x437, 0x430,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x422, 0x430, 0x439,
0x431, 0x435, 0x457, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x443, 0x20, 0x422, 0x430, 0x439, 0x431, 0x435, 0x457, 0x437, 0x430, 0x20, 0x43b,
0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x443, 0x20, 0x422, 0x430, 0x439, 0x431, 0x435, 0x457, 0x437, 0x430, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x422, 0x430, 0x434, 0x436, 0x438,
0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x456, 0x437, 0x430, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e,
0x432, 0x430, 0x445, 0x20, 0x422, 0x43e, 0x43a, 0x435, 0x43b, 0x430, 0x443, 0x437,
0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x422, 0x43e, 0x43d, 0x433,
0x430, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430,
0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x422, 0x43e,
0x43d, 0x433, 0x430, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x422, 0x43e, 0x43d, 0x433, 0x430,
0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20,
0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x427, 0x443, 0x443,
0x43a, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20,
0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x456, 0x441, 0x442, 0x430, 0x43d,
0x456, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20,
0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x456, 0x441, 0x442, 0x430, 0x43d,
0x456, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x422, 0x443, 0x440, 0x43a, 0x43c,
0x435, 0x43d, 0x456, 0x441, 0x442, 0x430, 0x43d, 0x456, 0x437, 0x430, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x430, 0x445, 0x20, 0x422, 0x443, 0x432, 0x430, 0x43b, 0x443, 0x437,
0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x432, 0x20, 0x423, 0x440,
0x443, 0x491, 0x432, 0x430, 0x457, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x432, 0x20, 0x423, 0x440, 0x443, 0x491, 0x432, 0x430, 0x457, 0x437,
0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x432, 0x20, 0x423, 0x440, 0x443, 0x491, 0x432, 0x430, 0x457,
0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x432, 0x20, 0x423,
0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x456, 0x437, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x432, 0x20, 0x423, 0x437, 0x431,
0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x456, 0x437, 0x430, 0x20, 0x43b,
0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x432, 0x20, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d,
0x456, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430,
0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x412, 0x430,
0x43d, 0x443, 0x430, 0x442, 0x443, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430,
0x445, 0x20, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x437, 0x430, 0x20,
0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445,
0x20, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x437, 0x430, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x412, 0x435, 0x43d, 0x435, 0x441,
0x443, 0x435, 0x43b, 0x456, 0x437, 0x430, 0x20, 0x432, 0x43b, 0x430, 0x434, 0x438,
0x432, 0x43e, 0x441, 0x442, 0x43e, 0x446, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x432, 0x43b, 0x430, 0x434, 0x438,
0x432, 0x43e, 0x441, 0x442, 0x43e, 0x446, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x432, 0x43b, 0x430, 0x434, 0x438,
0x432, 0x43e, 0x441, 0x442, 0x43e, 0x446, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b,
0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x432, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x432, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x441, 0x44c,
0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20,
0x432, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x43d, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x446, 0x456, 0x457, 0x20, 0x412,
0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456,
0x20, 0x412, 0x435, 0x439, 0x43a, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430,
0x445, 0x20, 0x423, 0x43e, 0x43b, 0x43b, 0x456, 0x441, 0x20, 0x456, 0x20, 0x424,
0x443, 0x442, 0x443, 0x43d, 0x430, 0x437, 0x430, 0x20, 0x44f, 0x43a, 0x443, 0x442,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x44f, 0x43a, 0x443, 0x442, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x44f, 0x43a, 0x443, 0x442,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x454, 0x43a, 0x430,
0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x437, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x454, 0x43a,
0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x437, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x454,
0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x437, 0x44c,
0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x20, 0x43d, 0x430, 0x20, 0x42e, 0x43a, 0x43e, 0x43d, 0x456, 0x61, 0x66, 0x67,
0x68, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x77, 0x75, 0x63, 0x68, 0x6f, 0x64,
0x6f, 0x61, 0x66, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x6a, 0x75, 0x17e, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x66,
0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0x61, 0x70,
0x61, 0x64, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61,
0x73, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x61, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x61, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61,
0x73, 0x61, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x6d, 0x61, 0x63,
0x6f, 0x6e, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x6d,
0x61, 0x63, 0x6f, 0x6e, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x41,
0x6d, 0x61, 0x63, 0x6f, 0x6e, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x77, 0x6a, 0x65,
0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73,
0x73, 0x65, 0x77, 0x6a, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e,
0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79,
0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x77, 0x6a, 0x65, 0x72, 0x6f, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x6e, 0x79, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x73, 0x65, 0x77, 0x6a, 0x65, 0x72, 0x6f, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x77, 0x75, 0x63, 0x68, 0x6f,
0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x77, 0x6a, 0x65,
0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x77,
0x75, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65,
0x77, 0x6a, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b,
0x69, 0x20, 0x77, 0x75, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x79, 0x20, 0x6c,
0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x77, 0x6a,
0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20,
0x68, 0xf3, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65,
0x77, 0x6a, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b,
0x69, 0x20, 0x68, 0xf3, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x73,
0x65, 0x77, 0x6a, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73,
0x6b, 0x69, 0x20, 0x68, 0xf3, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x77, 0x6a, 0x65,
0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x70,
0x61, 0x63, 0x69, 0x66, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x73, 0x65, 0x77, 0x6a, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x73, 0x6b, 0x69, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79,
0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x77, 0x6a, 0x65, 0x72, 0x6f, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x61, 0x63, 0x69,
0x66, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x41, 0x70, 0x69, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x41, 0x70, 0x69, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73,
0x41, 0x70, 0x69, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x61, 0x62, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x61, 0x62, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d,
0x61, 0x73, 0x61, 0x72, 0x61, 0x62, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73,
0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0x61, 0x70,
0x61, 0x64, 0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6f,
0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d,
0x61, 0x73, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x6d, 0x65, 0x6e, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x6d, 0x65, 0x6e, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79,
0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61,
0x73, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x72, 0x6a,
0x65, 0x64, 0x17a, 0x6f, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x72, 0x6a, 0x65, 0x64, 0x17a,
0x6f, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d,
0x61, 0x73, 0x73, 0x72, 0x6a, 0x65, 0x64, 0x17a, 0x6f, 0x61, 0x77, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x72, 0x6a, 0x65, 0x64, 0x17a, 0x6f,
0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x79, 0x20, 0x61, 0x77, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x72,
0x6a, 0x65, 0x64, 0x17a, 0x6f, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x79,
0x20, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d,
0x61, 0x73, 0x73, 0x6a, 0x65, 0x64, 0x17a, 0x6f, 0x7a, 0x61, 0x70, 0x61,
0x64, 0x6e, 0x79, 0x20, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x77, 0x75, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x61, 0x77, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x77, 0x75, 0x63,
0x68, 0x6f, 0x64, 0x6f, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x10d, 0x61, 0x73, 0x77, 0x75, 0x63, 0x68, 0x6f, 0x64, 0x6f,
0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0x61, 0x70, 0x61,
0x64, 0x6f, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x77,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x7a,
0x61, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x61, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x17a, 0x61, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x7a, 0x65, 0x72, 0x62,
0x61, 0x6a, 0x64, 0x17a, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73,
0x61, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x17a, 0x61, 0x6e, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x61, 0x63, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x61,
0x63, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x63, 0x6f,
0x72, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x161, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x68, 0x75, 0x74, 0x61,
0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x6f, 0x6c, 0x69,
0x77, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x42,
0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73,
0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x72, 0x75, 0x6e,
0x65, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x61, 0x70,
0x76, 0x65, 0x72, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6b,
0x61, 0x70, 0x76, 0x65, 0x72, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73,
0x6b, 0x61, 0x70, 0x76, 0x65, 0x72, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x61, 0x6d,
0x6f, 0x72, 0x72, 0x6f, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x63,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x63, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d,
0x61, 0x73, 0x63, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68,
0x69, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x69,
0x6c, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x69, 0x6c, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x63, 0x68, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x63,
0x68, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x69,
0x6e, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x48, 0x6f, 0x64, 0x6f, 0x77, 0x6e,
0x65, 0x6a, 0x65, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x10d, 0x61, 0x73, 0x20,
0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x6f, 0x77, 0x79, 0x63, 0x68, 0x20, 0x6b,
0x75, 0x70, 0x6f, 0x77, 0x6b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x6f, 0x6c, 0x75, 0x6d, 0x62,
0x69, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x6f, 0x6c, 0x75, 0x6d,
0x62, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f,
0x77, 0x79, 0x63, 0x68, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73,
0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x77, 0x79, 0x63, 0x68, 0x20, 0x6b,
0x75, 0x70, 0x6f, 0x77, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x77, 0x79, 0x63, 0x68, 0x20,
0x6b, 0x75, 0x70, 0x6f, 0x77, 0x6b, 0x75, 0x62, 0x61, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x6b, 0x75, 0x62, 0x61, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d,
0x61, 0x73, 0x6b, 0x75, 0x62, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x44, 0x61, 0x76, 0x69, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74,
0x20, 0x64, 0xb4, 0x20, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x77, 0x75, 0x63, 0x68, 0x6f, 0x64,
0x6e, 0x6f, 0x74, 0x69, 0x6d, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x4a, 0x75, 0x74, 0x72, 0x6f, 0x77,
0x6e, 0x65, 0x6a, 0x65, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x20,
0x4a, 0x75, 0x74, 0x72, 0x6f, 0x77, 0x6e, 0x65, 0x6a, 0x65, 0x20, 0x6b,
0x75, 0x70, 0x79, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x20, 0x4a, 0x75, 0x74, 0x72, 0x6f, 0x77, 0x6e, 0x65, 0x6a, 0x65, 0x20,
0x6b, 0x75, 0x70, 0x79, 0x65, 0x6b, 0x77, 0x61, 0x64, 0x6f, 0x72, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x72, 0x6a, 0x65, 0x64, 0x17a,
0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x73, 0x72, 0x6a, 0x65, 0x64, 0x17a, 0x6f, 0x65, 0x75, 0x72, 0x6f,
0x70, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x72, 0x6a, 0x65, 0x64,
0x17a, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x77, 0x75, 0x63, 0x68,
0x6f, 0x64, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x77, 0x75, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x65, 0x75,
0x72, 0x6f, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x77, 0x75, 0x63,
0x68, 0x6f, 0x64, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x61,
0x6c, 0x69, 0x6e, 0x69, 0x6e, 0x67, 0x72, 0x61, 0x64, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6f, 0x65, 0x75,
0x72, 0x6f, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0x61,
0x70, 0x61, 0x64, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x10d, 0x61, 0x73, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6f, 0x65, 0x75, 0x72,
0x6f, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x66, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x69, 0x64, 0x17a, 0x69, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x69, 0x64, 0x17a, 0x69, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79,
0x20, 0x10d, 0x61, 0x73, 0x66, 0x69, 0x64, 0x17a, 0x69, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x72,
0x61, 0x6e, 0x63, 0x6f, 0x73, 0x6b, 0x6f, 0x67, 0x75, 0x79, 0x61, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x46,
0x72, 0x61, 0x6e, 0x63, 0x6f, 0x73, 0x6b, 0x65, 0x68, 0x6f, 0x20, 0x6a,
0x75, 0x17e, 0x6e, 0x65, 0x68, 0x6f, 0x20, 0x61, 0x20, 0x61, 0x6e, 0x74,
0x61, 0x72, 0x6b, 0x74, 0x69, 0x73, 0x6b, 0x65, 0x68, 0x6f, 0x20, 0x74,
0x65, 0x72, 0x69, 0x74, 0x6f, 0x72, 0x69, 0x6a, 0x61, 0x67, 0x61, 0x6c,
0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x67, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x10d, 0x61, 0x73, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61,
0x73, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x6f, 0x77, 0x79,
0x63, 0x68, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x77, 0x69, 0x63, 0x68, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x77, 0x75, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x67, 0x72, 0xf6, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x77, 0x75,
0x63, 0x68, 0x6f, 0x64, 0x6f, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x77, 0x75, 0x63, 0x68, 0x6f,
0x64, 0x6f, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x7a,
0x61, 0x70, 0x61, 0x64, 0x6f, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0x61, 0x70, 0x61,
0x64, 0x6f, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79,
0x20, 0x10d, 0x61, 0x73, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6f, 0x67, 0x72,
0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x50,
0x65, 0x72, 0x73, 0x69, 0x73, 0x6b, 0x65, 0x68, 0x6f, 0x20, 0x67, 0x6f,
0x6c, 0x66, 0x61, 0x68, 0x61, 0x77, 0x61, 0x69, 0x69, 0x73, 0x6b, 0x6f,
0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x68, 0x61, 0x77, 0x61, 0x69, 0x69, 0x73, 0x6b, 0x6f, 0x2d, 0x61,
0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x68, 0x61,
0x77, 0x61, 0x69, 0x69, 0x73, 0x6b, 0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75,
0x74, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e,
0x67, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x48, 0x6f, 0x6e, 0x67, 0x6b,
0x6f, 0x6e, 0x67, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x43, 0x68, 0x6f, 0x77, 0x64, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x43, 0x68, 0x6f, 0x77, 0x64, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x10d, 0x61, 0x73, 0x43, 0x68, 0x6f, 0x77, 0x64, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x6e, 0x64,
0x69, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x6e, 0x64, 0x69,
0x73, 0x6b, 0x6f, 0x6f, 0x63, 0x65, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x69, 0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x72, 0x6a, 0x65, 0x64, 0x17a,
0x6f, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x77, 0x75, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x69, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x73, 0x6b, 0x69, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6f,
0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x69, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x69, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x72,
0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d,
0x61, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x73, 0x72,
0x61, 0x65, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x73, 0x72,
0x61, 0x65, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x6a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x6a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x10d, 0x61, 0x73, 0x6a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x61, 0x7a,
0x61, 0x63, 0x68, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x77, 0x75,
0x63, 0x68, 0x6f, 0x64, 0x6e, 0x6f, 0x6b, 0x61, 0x7a, 0x61, 0x63, 0x68,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0x61, 0x70, 0x61, 0x64,
0x6e, 0x6f, 0x6b, 0x61, 0x7a, 0x61, 0x63, 0x68, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x6b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x6b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d,
0x61, 0x73, 0x6b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x6f, 0x73, 0x72,
0x61, 0x65, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x69,
0x72, 0x67, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61,
0x73, 0x20, 0x4c, 0x69, 0x6e, 0x69, 0x6a, 0x6f, 0x77, 0x79, 0x63, 0x68,
0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x10d, 0x61, 0x73, 0x20, 0x6b, 0x75,
0x70, 0x79, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48, 0x6f, 0x77, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d,
0x61, 0x73, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x2d, 0x48, 0x6f, 0x77, 0x65, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x2d, 0x48, 0x6f, 0x77, 0x65, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x6d, 0x61, 0x6c, 0x61, 0x6a, 0x7a, 0x69, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61, 0x6c, 0x65, 0x64, 0x69, 0x77, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61, 0x72, 0x71, 0x75, 0x65,
0x73, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73,
0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x6f, 0x77, 0x79,
0x63, 0x68, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x6d, 0x61, 0x75, 0x72,
0x69, 0x74, 0x69, 0x75, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6d,
0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61,
0x73, 0x6d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x65,
0x78, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x65, 0x78, 0x69, 0x73,
0x6b, 0x69, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x10d, 0x61, 0x73, 0x6d, 0x65, 0x78, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x70,
0x61, 0x63, 0x69, 0x66, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42,
0x61, 0x74, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x55,
0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x10d, 0x61, 0x73, 0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f,
0x72, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x77, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x77, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x10d, 0x61, 0x73, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x77, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x79,
0x61, 0x6e, 0x6d, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x6e, 0x61, 0x75, 0x72, 0x75, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x6e, 0x65, 0x70, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x6e, 0x6f, 0x77, 0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x77, 0x6f, 0x6b, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x6e,
0x6f, 0x77, 0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6e,
0x6f, 0x77, 0x6f, 0x73, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x77, 0x6f, 0x73, 0x65, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f,
0x77, 0x6f, 0x73, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f,
0x77, 0x6f, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x77, 0x6f, 0x66, 0x75, 0x6e,
0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x6e,
0x6f, 0x77, 0x6f, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x6e, 0x69, 0x75, 0x65, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10d,
0x61, 0x73, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x20, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x20, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x10d, 0x61, 0x73, 0x20, 0x6b, 0x75, 0x70, 0x6f,
0x77, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x20,
0x6b, 0x75, 0x70, 0x6f, 0x77, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x6b,
0x75, 0x70, 0x6f, 0x77, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x4e, 0x6f, 0x77, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x4f, 0x6d, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61,
0x73, 0x4f, 0x6d, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x70, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x61, 0x6c, 0x61, 0x75, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x61, 0x70, 0x75, 0x61, 0x2d,
0x6e, 0x6f, 0x77, 0x6f, 0x67, 0x69, 0x6e, 0x65, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x79, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x61, 0x72,
0x61, 0x67, 0x75, 0x61, 0x79, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x65, 0x72, 0x75, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x65, 0x72, 0x75, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x10d, 0x61, 0x73, 0x70, 0x65, 0x72, 0x75, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x66,
0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x50,
0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x6f, 0x77, 0x79, 0x63, 0x68, 0x20,
0x6b, 0x75, 0x70, 0x6f, 0x77, 0x10d, 0x61, 0x73, 0x20, 0x6b, 0x75, 0x70,
0x6f, 0x77, 0x20, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x20, 0x61, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d,
0x61, 0x73, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x20, 0x53, 0x74, 0x2e,
0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x61, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x20, 0x53, 0x74,
0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x61, 0x20, 0x4d,
0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x10d, 0x61, 0x73, 0x20, 0x50,
0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x6f, 0x77, 0x79, 0x63, 0x68,
0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x70, 0x6f, 0x6e, 0x61, 0x70, 0x65,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x6a, 0xf6, 0x6e, 0x67,
0x6a, 0x61, 0x6e, 0x67, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x72,
0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x73, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x63, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x63, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x6d, 0x6f, 0x61, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x6d, 0x6f, 0x61, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79,
0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x6d, 0x6f, 0x61, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65,
0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x53, 0x61, 0x6c, 0x6f,
0x6d, 0x6f, 0x6e, 0x73, 0x6b, 0x69, 0x63, 0x68, 0x20, 0x6b, 0x75, 0x70,
0x6f, 0x77, 0x6a, 0x75, 0x17e, 0x6e, 0x6f, 0x67, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x75, 0x72, 0x69,
0x6e, 0x61, 0x6d, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x53, 0x79,
0x6f, 0x77, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x61,
0x68, 0x69, 0x74, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x54,
0x61, 0x69, 0x70, 0x65, 0x68, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x54, 0x61, 0x69, 0x70, 0x65, 0x68, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73,
0x54, 0x61, 0x69, 0x70, 0x65, 0x68, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x61, 0x64, 0x17a, 0x69,
0x6b, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x6f, 0x6b, 0x65,
0x6c, 0x61, 0x75, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x6f,
0x6e, 0x67, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x6f,
0x6e, 0x67, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x6f, 0x6e,
0x67, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x63, 0x68, 0x75, 0x75, 0x6b, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x74, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x75, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x75, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x75, 0x72, 0x75, 0x67, 0x75, 0x61,
0x79, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x75, 0x7a, 0x62, 0x65, 0x6b, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x75, 0x7a, 0x62, 0x65, 0x6b, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61,
0x73, 0x75, 0x7a, 0x62, 0x65, 0x6b, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x76, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x76, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x76, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x76, 0x65, 0x6e, 0x65, 0x7a, 0x75,
0x65, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x57, 0x6c, 0x61,
0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x57, 0x6c, 0x61, 0x64,
0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x57, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x10d, 0x61, 0x73, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x57, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x20, 0x57,
0x61, 0x6b, 0x65, 0x10d, 0x61, 0x73, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77,
0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x61, 0x20, 0x46, 0x75,
0x74, 0x75, 0x6e, 0x61, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x10d, 0x61, 0x73, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4a, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73,
0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x6f, 0x77, 0x79, 0x20, 0x10d,
0x61, 0x73, 0x627, 0x641, 0x63a, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20,
0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x648, 0x633, 0x637, 0x6cc, 0x20, 0x627,
0x641, 0x631, 0x6cc, 0x642, 0x6c1, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x634,
0x631, 0x642, 0x6cc, 0x20, 0x627, 0x641, 0x631, 0x6cc, 0x642, 0x6c1, 0x20, 0x679,
0x627, 0x626, 0x645, 0x62c, 0x646, 0x648, 0x628, 0x6cc, 0x20, 0x627, 0x641, 0x631,
0x6cc, 0x642, 0x6c1, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20,
0x679, 0x627, 0x626, 0x645, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627, 0x641,
0x631, 0x6cc, 0x642, 0x6c1, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x63a, 0x631,
0x628, 0x6cc, 0x20, 0x627, 0x641, 0x631, 0x6cc, 0x642, 0x6c1, 0x20, 0x633, 0x679,
0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x63a,
0x631, 0x628, 0x6cc, 0x20, 0x627, 0x641, 0x631, 0x6cc, 0x642, 0x6c1, 0x20, 0x633,
0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x644, 0x627, 0x633, 0x6a9,
0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x644, 0x627, 0x633, 0x6a9, 0x627,
0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x626, 0x645, 0x627, 0x644, 0x627, 0x633, 0x6a9, 0x627, 0x20, 0x688, 0x6d2, 0x20,
0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x645, 0x6cc,
0x632, 0x648, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x645, 0x6cc,
0x632, 0x648, 0x646, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631,
0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x645, 0x6cc, 0x632, 0x648, 0x646, 0x20,
0x6a9, 0x627, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627,
0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x633, 0x646, 0x679, 0x631, 0x644,
0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x646, 0x679, 0x631, 0x644, 0x20, 0x627,
0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645,
0x633, 0x646, 0x679, 0x631, 0x644, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626,
0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x633, 0x679, 0x631, 0x646,
0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x633, 0x679, 0x631, 0x646, 0x20,
0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626,
0x645, 0x627, 0x6cc, 0x633, 0x679, 0x631, 0x646, 0x20, 0x688, 0x6d2, 0x20, 0x644,
0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x627, 0x624, 0x646,
0x679, 0x6cc, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x627, 0x624, 0x646,
0x679, 0x6cc, 0x646, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688,
0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x627, 0x624, 0x646, 0x679, 0x6cc, 0x646,
0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626,
0x645, 0x67e, 0x6cc, 0x633, 0x641, 0x6a9, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e,
0x6cc, 0x633, 0x641, 0x6a9, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631,
0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e, 0x6cc, 0x633, 0x641, 0x6a9, 0x20,
0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645,
0x627, 0x646, 0x6cc, 0x62f, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x646,
0x6cc, 0x62f, 0x631, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688,
0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x646, 0x6cc, 0x62f, 0x631, 0x20, 0x633,
0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x67e, 0x6cc, 0x627,
0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x67e, 0x6cc, 0x627, 0x20, 0x633,
0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627,
0x6cc, 0x67e, 0x6cc, 0x627, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679,
0x20, 0x679, 0x627, 0x626, 0x645, 0x639, 0x631, 0x628, 0x20, 0x6a9, 0x627, 0x20,
0x648, 0x642, 0x62a, 0x639, 0x631, 0x628, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639,
0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x639, 0x631, 0x628, 0x20,
0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645,
0x627, 0x631, 0x62c, 0x646, 0x679, 0x6cc, 0x646, 0x627, 0x20, 0x6a9, 0x627, 0x20,
0x648, 0x642, 0x62a, 0x627, 0x631, 0x62c, 0x646, 0x679, 0x6cc, 0x646, 0x627, 0x20,
0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642,
0x62a, 0x627, 0x631, 0x62c, 0x646, 0x679, 0x6cc, 0x646, 0x627, 0x20, 0x6a9, 0x627,
0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9,
0x627, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627,
0x631, 0x62c, 0x646, 0x679, 0x6cc, 0x646, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648,
0x642, 0x62a, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627, 0x631, 0x62c, 0x646,
0x679, 0x6cc, 0x646, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627,
0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20,
0x627, 0x631, 0x62c, 0x646, 0x679, 0x6cc, 0x646, 0x627, 0x20, 0x6a9, 0x627, 0x20,
0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627,
0x20, 0x648, 0x642, 0x62a, 0x622, 0x631, 0x645, 0x6cc, 0x646, 0x6cc, 0x627, 0x20,
0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x631, 0x645, 0x6cc, 0x646, 0x6cc,
0x627, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20,
0x648, 0x642, 0x62a, 0x622, 0x631, 0x645, 0x6cc, 0x646, 0x6cc, 0x627, 0x20, 0x6a9,
0x627, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20,
0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x679, 0x644, 0x627, 0x646, 0x679,
0x6a9, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x679, 0x644, 0x627, 0x646, 0x679,
0x6a9, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x626, 0x645, 0x627, 0x679, 0x644, 0x627, 0x646, 0x679, 0x6a9, 0x20, 0x688,
0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x633,
0x646, 0x679, 0x631, 0x644, 0x20, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x6cc,
0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644,
0x6cc, 0x646, 0x20, 0x633, 0x646, 0x679, 0x631, 0x644, 0x20, 0x627, 0x633, 0x679,
0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x622, 0x633,
0x679, 0x631, 0x6cc, 0x644, 0x6cc, 0x646, 0x20, 0x633, 0x646, 0x679, 0x631, 0x644,
0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626,
0x645, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x6cc, 0x646, 0x20, 0x633, 0x646,
0x679, 0x631, 0x644, 0x20, 0x648, 0x6cc, 0x633, 0x679, 0x631, 0x646, 0x20, 0x679,
0x627, 0x626, 0x645, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x6cc, 0x646, 0x20,
0x633, 0x646, 0x679, 0x631, 0x644, 0x20, 0x648, 0x6cc, 0x633, 0x679, 0x631, 0x646,
0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x626, 0x645, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x6cc, 0x646, 0x20, 0x633,
0x646, 0x679, 0x631, 0x644, 0x20, 0x648, 0x6cc, 0x633, 0x679, 0x631, 0x646, 0x20,
0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645,
0x627, 0x6cc, 0x633, 0x679, 0x631, 0x646, 0x20, 0x622, 0x633, 0x679, 0x631, 0x6cc,
0x644, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x622, 0x633, 0x679, 0x631,
0x6cc, 0x644, 0x6cc, 0x646, 0x20, 0x627, 0x6cc, 0x633, 0x679, 0x631, 0x646, 0x20,
0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626,
0x645, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x6cc, 0x646, 0x20, 0x627, 0x6cc,
0x633, 0x679, 0x631, 0x646, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679,
0x20, 0x679, 0x627, 0x626, 0x645, 0x648, 0x6cc, 0x633, 0x679, 0x631, 0x646, 0x20,
0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x626,
0x645, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x6cc, 0x627, 0x20, 0x648, 0x6cc,
0x633, 0x679, 0x631, 0x646, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631,
0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644,
0x6cc, 0x646, 0x20, 0x648, 0x6cc, 0x633, 0x679, 0x631, 0x646, 0x20, 0x688, 0x6d2,
0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x622, 0x630,
0x631, 0x628, 0x627, 0x626, 0x6cc, 0x62c, 0x627, 0x646, 0x20, 0x6a9, 0x627, 0x20,
0x648, 0x642, 0x62a, 0x622, 0x630, 0x631, 0x628, 0x627, 0x626, 0x6cc, 0x62c, 0x627,
0x646, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20,
0x648, 0x642, 0x62a, 0x622, 0x630, 0x631, 0x628, 0x627, 0x626, 0x6cc, 0x62c, 0x627,
0x646, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631,
0x645, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x632, 0x648,
0x631, 0x6cc, 0x633, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x632,
0x648, 0x631, 0x6cc, 0x633, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627,
0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x632, 0x648, 0x631, 0x6cc, 0x633,
0x20, 0x6a9, 0x627, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645,
0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x628, 0x646, 0x6af, 0x644,
0x6c1, 0x20, 0x62f, 0x6cc, 0x634, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a,
0x628, 0x646, 0x6af, 0x644, 0x6c1, 0x20, 0x62f, 0x6cc, 0x634, 0x20, 0x6a9, 0x627,
0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x628,
0x646, 0x6af, 0x644, 0x6c1, 0x20, 0x62f, 0x6cc, 0x634, 0x20, 0x6a9, 0x627, 0x20,
0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627,
0x20, 0x648, 0x642, 0x62a, 0x628, 0x6be, 0x648, 0x679, 0x627, 0x646, 0x20, 0x6a9,
0x627, 0x20, 0x648, 0x642, 0x62a, 0x628, 0x648, 0x644, 0x6cc, 0x648, 0x6cc, 0x627,
0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x628, 0x631, 0x627, 0x632, 0x6cc,
0x644, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x628, 0x631, 0x627, 0x632,
0x6cc, 0x644, 0x6cc, 0x627, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631,
0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x628, 0x631, 0x627, 0x632, 0x6cc, 0x644,
0x6cc, 0x627, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x628,
0x631, 0x648, 0x646, 0x626, 0x6cc, 0x20, 0x62f, 0x627, 0x631, 0x627, 0x644, 0x633,
0x644, 0x627, 0x645, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x6cc, 0x67e, 0x20,
0x648, 0x631, 0x688, 0x6cc, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x6cc, 0x67e,
0x20, 0x648, 0x631, 0x688, 0x6cc, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631,
0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x6cc, 0x67e, 0x20, 0x648, 0x631,
0x688, 0x6cc, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x686,
0x627, 0x645, 0x648, 0x631, 0x648, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631,
0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x686, 0x6cc, 0x62a, 0x6be, 0x645, 0x20,
0x679, 0x627, 0x626, 0x645, 0x686, 0x6cc, 0x62a, 0x6be, 0x645, 0x20, 0x627, 0x633,
0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x686,
0x6cc, 0x62a, 0x6be, 0x645, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679,
0x20, 0x679, 0x627, 0x626, 0x645, 0x686, 0x644, 0x6cc, 0x20, 0x6a9, 0x627, 0x20,
0x648, 0x642, 0x62a, 0x686, 0x644, 0x6cc, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639,
0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x686, 0x644, 0x6cc, 0x20,
0x6a9, 0x627, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627,
0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x686, 0x6cc, 0x646, 0x20, 0x6a9,
0x627, 0x20, 0x648, 0x642, 0x62a, 0x686, 0x6cc, 0x646, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x686, 0x6cc, 0x646,
0x6cc, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627,
0x626, 0x645, 0x6a9, 0x631, 0x633, 0x645, 0x633, 0x20, 0x622, 0x626, 0x644, 0x6cc,
0x646, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x648, 0x6a9, 0x648, 0x633,
0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x632, 0x20, 0x679, 0x627, 0x626,
0x645, 0x6a9, 0x648, 0x644, 0x645, 0x628, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x626,
0x645, 0x6a9, 0x648, 0x644, 0x645, 0x628, 0x6cc, 0x627, 0x20, 0x6a9, 0x627, 0x20,
0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x6a9, 0x648,
0x644, 0x645, 0x628, 0x6cc, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x648, 0x633,
0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642,
0x62a, 0x6a9, 0x6a9, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x632, 0x20,
0x679, 0x627, 0x626, 0x645, 0x6a9, 0x6a9, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646,
0x688, 0x632, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x626, 0x645, 0x6a9, 0x6a9, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688,
0x632, 0x20, 0x646, 0x635, 0x641, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627,
0x626, 0x645, 0x6a9, 0x6cc, 0x648, 0x628, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645,
0x6a9, 0x6cc, 0x648, 0x628, 0x627, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688,
0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x6cc, 0x648, 0x628, 0x627,
0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626,
0x645, 0x688, 0x6cc, 0x648, 0x633, 0x20, 0x679, 0x627, 0x626, 0x645, 0x688, 0x648,
0x645, 0x648, 0x646, 0x679, 0x2d, 0x688, 0x6cc, 0x2019, 0x627, 0x631, 0x648, 0x6cc,
0x644, 0x6d2, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x634, 0x631, 0x642, 0x6cc,
0x20, 0x62a, 0x6cc, 0x645, 0x648, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627,
0x6cc, 0x633, 0x679, 0x631, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x20,
0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x6cc, 0x633, 0x679, 0x631, 0x20,
0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639,
0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x6cc, 0x633, 0x679,
0x631, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x6a9, 0x627, 0x20,
0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627,
0x20, 0x648, 0x642, 0x62a, 0x627, 0x6cc, 0x6a9, 0x648, 0x627, 0x688, 0x648, 0x631,
0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x648, 0x633, 0x637, 0x20, 0x6cc,
0x648, 0x631, 0x67e, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x648, 0x633,
0x637, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x6a9, 0x627, 0x20, 0x645,
0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x648, 0x633, 0x637,
0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x648,
0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648,
0x642, 0x62a, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e,
0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x634, 0x631, 0x642, 0x6cc,
0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc,
0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x634, 0x631, 0x642, 0x6cc,
0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x648, 0x633,
0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642,
0x62a, 0x628, 0x639, 0x6cc, 0x62f, 0x20, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20,
0x6cc, 0x648, 0x631, 0x67e, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x63a, 0x631,
0x628, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x6a9, 0x627, 0x20, 0x648,
0x642, 0x62a, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e,
0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648,
0x642, 0x62a, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e,
0x20, 0x6a9, 0x627, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645,
0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x627, 0x6a9, 0x20,
0x644, 0x6cc, 0x646, 0x688, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x632,
0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x627, 0x6a9, 0x20, 0x644,
0x6cc, 0x646, 0x688, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x632, 0x20,
0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642,
0x62a, 0x641, 0x627, 0x6a9, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x622, 0x626,
0x644, 0x6cc, 0x646, 0x688, 0x632, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x648, 0x633,
0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642,
0x62a, 0x641, 0x62c, 0x6cc, 0x20, 0x679, 0x627, 0x626, 0x645, 0x641, 0x62c, 0x6cc,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626,
0x645, 0x641, 0x62c, 0x6cc, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626,
0x645, 0x641, 0x631, 0x6cc, 0x646, 0x686, 0x20, 0x6af, 0x6cc, 0x627, 0x646, 0x627,
0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x631, 0x6cc, 0x646, 0x686,
0x20, 0x62c, 0x646, 0x648, 0x628, 0x6cc, 0x20, 0x627, 0x648, 0x631, 0x20, 0x627,
0x646, 0x679, 0x627, 0x631, 0x6a9, 0x679, 0x6a9, 0x20, 0x679, 0x627, 0x626, 0x645,
0x6af, 0x627, 0x644, 0x627, 0x67e, 0x627, 0x6af, 0x648, 0x632, 0x20, 0x6a9, 0x627,
0x20, 0x648, 0x642, 0x62a, 0x6af, 0x6cc, 0x645, 0x628, 0x6cc, 0x626, 0x631, 0x20,
0x679, 0x627, 0x626, 0x645, 0x62c, 0x627, 0x631, 0x62c, 0x6cc, 0x627, 0x20, 0x6a9,
0x627, 0x20, 0x648, 0x642, 0x62a, 0x62c, 0x627, 0x631, 0x62c, 0x6cc, 0x627, 0x20,
0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642,
0x62a, 0x62c, 0x627, 0x631, 0x62c, 0x6cc, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x645,
0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627, 0x20,
0x648, 0x642, 0x62a, 0x62c, 0x644, 0x628, 0x631, 0x679, 0x20, 0x622, 0x626, 0x644,
0x6cc, 0x646, 0x688, 0x632, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6af, 0x631, 0x6cc,
0x646, 0x20, 0x648, 0x686, 0x20, 0x6a9, 0x627, 0x20, 0x627, 0x635, 0x644, 0x20,
0x648, 0x642, 0x62a, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x6af, 0x631, 0x6cc,
0x646, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645,
0x634, 0x631, 0x642, 0x6cc, 0x20, 0x6af, 0x631, 0x6cc, 0x646, 0x20, 0x644, 0x6cc,
0x646, 0x688, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20,
0x679, 0x627, 0x626, 0x645, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x6af, 0x631,
0x6cc, 0x646, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x6a9, 0x627, 0x20, 0x645,
0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627, 0x20,
0x648, 0x642, 0x62a, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x6af, 0x631, 0x6cc,
0x646, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645,
0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x6af, 0x631, 0x6cc, 0x646, 0x20, 0x644, 0x6cc,
0x646, 0x688, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20,
0x679, 0x627, 0x626, 0x645, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x6af, 0x631,
0x6cc, 0x646, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x6a9, 0x627, 0x20, 0x645,
0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627, 0x20,
0x648, 0x642, 0x62a, 0x62e, 0x644, 0x6cc, 0x62c, 0x20, 0x6a9, 0x627, 0x20, 0x645,
0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x6c1, 0x648, 0x627,
0x626, 0x6cc, 0x20, 0x627, 0x644, 0x6cc, 0x648, 0x679, 0x6cc, 0x626, 0x646, 0x20,
0x679, 0x627, 0x626, 0x645, 0x6c1, 0x648, 0x627, 0x626, 0x6cc, 0x20, 0x627, 0x644,
0x6cc, 0x648, 0x679, 0x6cc, 0x626, 0x646, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646,
0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6c1, 0x648, 0x627, 0x626,
0x6cc, 0x20, 0x627, 0x644, 0x6cc, 0x648, 0x679, 0x6cc, 0x626, 0x646, 0x20, 0x688,
0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6c1,
0x627, 0x646, 0x6af, 0x20, 0x6a9, 0x627, 0x646, 0x6af, 0x20, 0x633, 0x645, 0x631,
0x20, 0x679, 0x627, 0x626, 0x645, 0x6c1, 0x648, 0x648, 0x688, 0x20, 0x679, 0x627,
0x626, 0x645, 0x6c1, 0x648, 0x648, 0x688, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688,
0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6c1, 0x648, 0x648, 0x688, 0x20,
0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6c1, 0x646, 0x62f, 0x648,
0x633, 0x62a, 0x627, 0x646, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627,
0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x628, 0x62d, 0x631, 0x20, 0x6c1, 0x646,
0x62f, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6c1, 0x646, 0x62f, 0x20, 0x686, 0x6cc,
0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x648, 0x633, 0x637, 0x6cc, 0x20, 0x627,
0x646, 0x688, 0x648, 0x646, 0x6cc, 0x634, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x626,
0x645, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x627, 0x646, 0x688, 0x648, 0x646,
0x6cc, 0x634, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x63a, 0x631,
0x628, 0x6cc, 0x20, 0x627, 0x646, 0x688, 0x648, 0x646, 0x6cc, 0x634, 0x6cc, 0x627,
0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x631, 0x627, 0x646, 0x20, 0x6a9,
0x627, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x6cc, 0x631, 0x627, 0x646, 0x20, 0x6a9,
0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a,
0x627, 0x6cc, 0x631, 0x627, 0x646, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626,
0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x631, 0x6a9, 0x62a, 0x633, 0x6a9,
0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x631, 0x6a9, 0x62a, 0x633, 0x6a9, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645,
0x627, 0x631, 0x6a9, 0x62a, 0x633, 0x6a9, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679,
0x627, 0x626, 0x645, 0x627, 0x633, 0x631, 0x627, 0x626, 0x6cc, 0x644, 0x20, 0x6a9,
0x627, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x633, 0x631, 0x627, 0x626, 0x6cc, 0x644,
0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648,
0x642, 0x62a, 0x627, 0x633, 0x631, 0x627, 0x626, 0x6cc, 0x644, 0x20, 0x688, 0x6d2,
0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x62c, 0x627,
0x67e, 0x627, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x62c, 0x627, 0x67e, 0x627,
0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x626, 0x645, 0x62c, 0x627, 0x67e, 0x627, 0x646, 0x20, 0x688, 0x6d2, 0x20, 0x644,
0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e, 0x6cc, 0x679, 0x631,
0x648, 0x67e, 0x627, 0x624, 0x644, 0x648, 0x648, 0x633, 0x6a9, 0x2d, 0x6a9, 0x6cc,
0x645, 0x686, 0x633, 0x6a9, 0x6cc, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e, 0x6cc,
0x679, 0x631, 0x648, 0x67e, 0x627, 0x624, 0x644, 0x648, 0x648, 0x633, 0x6a9, 0x2d,
0x6a9, 0x6cc, 0x645, 0x686, 0x633, 0x6a9, 0x6cc, 0x20, 0x627, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e, 0x6cc, 0x679,
0x631, 0x648, 0x67e, 0x627, 0x624, 0x644, 0x648, 0x648, 0x633, 0x6a9, 0x2d, 0x6a9,
0x6cc, 0x645, 0x686, 0x633, 0x6a9, 0x6cc, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679,
0x627, 0x626, 0x645, 0x642, 0x627, 0x632, 0x642, 0x633, 0x62a, 0x627, 0x646, 0x20,
0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20,
0x642, 0x632, 0x627, 0x62e, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6a9, 0x627, 0x20,
0x648, 0x642, 0x62a, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x642, 0x632, 0x627,
0x62e, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a,
0x6a9, 0x648, 0x631, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x648,
0x631, 0x6cc, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20,
0x679, 0x627, 0x626, 0x645, 0x6a9, 0x648, 0x631, 0x6cc, 0x627, 0x20, 0x688, 0x6d2,
0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x648,
0x633, 0x631, 0x6d2, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x631, 0x6cc, 0x633,
0x646, 0x648, 0x6cc, 0x627, 0x631, 0x633, 0x6a9, 0x20, 0x679, 0x627, 0x626, 0x645,
0x6a9, 0x631, 0x633, 0x646, 0x648, 0x6cc, 0x627, 0x631, 0x633, 0x6a9, 0x20, 0x633,
0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9,
0x631, 0x6cc, 0x633, 0x646, 0x648, 0x6cc, 0x627, 0x631, 0x633, 0x6a9, 0x20, 0x633,
0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x631, 0x63a, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x644, 0x627, 0x626,
0x646, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x632, 0x20, 0x679, 0x627,
0x626, 0x645, 0x644, 0x627, 0x631, 0x688, 0x20, 0x6c1, 0x648, 0x648, 0x6d2, 0x20,
0x679, 0x627, 0x626, 0x645, 0x644, 0x627, 0x631, 0x688, 0x20, 0x6c1, 0x648, 0x648,
0x6d2, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x626, 0x645, 0x644, 0x627, 0x631, 0x688, 0x20, 0x6c1, 0x648, 0x648, 0x6d2,
0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626,
0x645, 0x645, 0x6cc, 0x6af, 0x6cc, 0x62f, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645,
0x645, 0x6af, 0x627, 0x62f, 0x627, 0x646, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646,
0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x6cc, 0x6af, 0x6cc,
0x62f, 0x646, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645,
0x644, 0x6cc, 0x634, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x627,
0x644, 0x62f, 0x6cc, 0x67e, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x645,
0x627, 0x631, 0x6a9, 0x6cc, 0x633, 0x633, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645,
0x627, 0x631, 0x634, 0x644, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x632,
0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x627, 0x631, 0x6cc, 0x634, 0x633, 0x20,
0x679, 0x627, 0x626, 0x645, 0x645, 0x627, 0x631, 0x6cc, 0x634, 0x633, 0x20, 0x633,
0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645,
0x627, 0x631, 0x6cc, 0x634, 0x633, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627,
0x626, 0x645, 0x645, 0x627, 0x624, 0x633, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645,
0x645, 0x6cc, 0x6a9, 0x633, 0x6cc, 0x6a9, 0x646, 0x20, 0x67e, 0x6cc, 0x633, 0x641,
0x6a9, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x6cc, 0x6a9, 0x633, 0x6cc, 0x6a9,
0x646, 0x20, 0x67e, 0x6cc, 0x633, 0x641, 0x6a9, 0x20, 0x627, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x6cc, 0x6a9,
0x633, 0x6cc, 0x6a9, 0x646, 0x20, 0x67e, 0x6cc, 0x633, 0x641, 0x6a9, 0x20, 0x688,
0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6cc,
0x648, 0x644, 0x627, 0x646, 0x20, 0x628, 0x6cc, 0x62a, 0x648, 0x631, 0x20, 0x679,
0x627, 0x626, 0x645, 0x6cc, 0x648, 0x644, 0x627, 0x646, 0x20, 0x628, 0x6cc, 0x62a,
0x648, 0x631, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x626, 0x645, 0x6cc, 0x648, 0x644, 0x627, 0x646, 0x20, 0x628, 0x6cc, 0x62a,
0x648, 0x631, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645,
0x627, 0x633, 0x6a9, 0x648, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x627, 0x633,
0x6a9, 0x648, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20,
0x679, 0x627, 0x626, 0x645, 0x645, 0x627, 0x633, 0x6a9, 0x648, 0x20, 0x633, 0x645,
0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x6cc, 0x627, 0x646, 0x645, 0x627,
0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x646, 0x627, 0x624, 0x631, 0x648, 0x20,
0x679, 0x627, 0x626, 0x645, 0x646, 0x6cc, 0x67e, 0x627, 0x644, 0x20, 0x6a9, 0x627,
0x20, 0x648, 0x642, 0x62a, 0x646, 0x6cc, 0x648, 0x20, 0x6a9, 0x6cc, 0x644, 0x6cc,
0x688, 0x648, 0x646, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x646, 0x6cc,
0x648, 0x20, 0x6a9, 0x6cc, 0x644, 0x6cc, 0x688, 0x648, 0x646, 0x6cc, 0x627, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645,
0x646, 0x6cc, 0x648, 0x20, 0x6a9, 0x6cc, 0x644, 0x6cc, 0x688, 0x648, 0x646, 0x6cc,
0x627, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x646, 0x6cc,
0x648, 0x632, 0x6cc, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x6a9, 0x627, 0x20,
0x648, 0x642, 0x62a, 0x646, 0x6cc, 0x648, 0x632, 0x6cc, 0x20, 0x644, 0x6cc, 0x646,
0x688, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20,
0x648, 0x642, 0x62a, 0x646, 0x6cc, 0x648, 0x632, 0x6cc, 0x20, 0x644, 0x6cc, 0x646,
0x688, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627,
0x626, 0x645, 0x646, 0x6cc, 0x648, 0x20, 0x641, 0x627, 0x624, 0x646, 0x688, 0x20,
0x644, 0x6cc, 0x646, 0x688, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631,
0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x646, 0x6cc, 0x626, 0x648, 0x20, 0x679,
0x627, 0x626, 0x645, 0x646, 0x627, 0x631, 0x641, 0x648, 0x6a9, 0x20, 0x622, 0x626,
0x644, 0x6cc, 0x646, 0x688, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x646,
0x627, 0x631, 0x641, 0x648, 0x6a9, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688,
0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648,
0x642, 0x62a, 0x646, 0x627, 0x631, 0x641, 0x648, 0x6a9, 0x20, 0x622, 0x626, 0x644,
0x6cc, 0x646, 0x688, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20,
0x648, 0x642, 0x62a, 0x641, 0x631, 0x646, 0x627, 0x646, 0x688, 0x648, 0x20, 0x688,
0x6cc, 0x20, 0x646, 0x648, 0x631, 0x646, 0x6c1, 0x627, 0x20, 0x6a9, 0x627, 0x20,
0x648, 0x642, 0x62a, 0x641, 0x631, 0x646, 0x627, 0x646, 0x688, 0x648, 0x20, 0x688,
0x6cc, 0x20, 0x646, 0x648, 0x631, 0x646, 0x6c1, 0x627, 0x20, 0x6a9, 0x627, 0x20,
0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x631,
0x646, 0x627, 0x646, 0x688, 0x648, 0x20, 0x688, 0x6cc, 0x20, 0x646, 0x648, 0x631,
0x648, 0x646, 0x6c1, 0x627, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626,
0x645, 0x646, 0x648, 0x648, 0x648, 0x633, 0x6cc, 0x628, 0x631, 0x633, 0x6a9, 0x20,
0x679, 0x627, 0x626, 0x645, 0x646, 0x648, 0x648, 0x648, 0x633, 0x6cc, 0x628, 0x631,
0x633, 0x6a9, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x626, 0x645, 0x646, 0x648, 0x648, 0x648, 0x633, 0x6cc, 0x628, 0x631, 0x633,
0x6a9, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x648,
0x645, 0x633, 0x6a9, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x648, 0x645, 0x633,
0x6a9, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x626, 0x645, 0x627, 0x648, 0x645, 0x633, 0x6a9, 0x20, 0x633, 0x645, 0x631, 0x20,
0x679, 0x627, 0x626, 0x645, 0x67e, 0x627, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20,
0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x627, 0x6a9, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20,
0x648, 0x642, 0x62a, 0x67e, 0x627, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6a9,
0x627, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20,
0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x644, 0x627, 0x624, 0x20, 0x679,
0x627, 0x626, 0x645, 0x67e, 0x627, 0x67e, 0x648, 0x622, 0x20, 0x646, 0x6cc, 0x648,
0x20, 0x6af, 0x646, 0x6cc, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e, 0x6cc, 0x631,
0x627, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a,
0x67e, 0x6cc, 0x631, 0x627, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x6a9, 0x627, 0x20,
0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x6cc,
0x631, 0x627, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x648,
0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648,
0x642, 0x62a, 0x67e, 0x6cc, 0x631, 0x648, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642,
0x62a, 0x67e, 0x6cc, 0x631, 0x648, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc,
0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x6cc, 0x631, 0x648, 0x20,
0x6a9, 0x627, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627,
0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x644, 0x67e, 0x627, 0x626,
0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x641, 0x644, 0x67e, 0x627, 0x626, 0x646,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626,
0x645, 0x641, 0x644, 0x67e, 0x627, 0x626, 0x646, 0x20, 0x633, 0x645, 0x631, 0x20,
0x679, 0x627, 0x626, 0x645, 0x641, 0x6cc, 0x646, 0x6a9, 0x633, 0x20, 0x622, 0x626,
0x644, 0x6cc, 0x646, 0x688, 0x632, 0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x6cc,
0x646, 0x679, 0x20, 0x67e, 0x6cc, 0x626, 0x631, 0x20, 0x627, 0x648, 0x631, 0x20,
0x645, 0x6a9, 0x644, 0x6cc, 0x626, 0x648, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645,
0x633, 0x6cc, 0x646, 0x679, 0x20, 0x67e, 0x6cc, 0x626, 0x631, 0x20, 0x627, 0x648,
0x631, 0x20, 0x645, 0x6a9, 0x644, 0x6cc, 0x626, 0x648, 0x646, 0x20, 0x627, 0x633,
0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x633,
0x6cc, 0x646, 0x679, 0x20, 0x67e, 0x6cc, 0x626, 0x631, 0x20, 0x627, 0x648, 0x631,
0x20, 0x645, 0x6a9, 0x644, 0x6cc, 0x626, 0x648, 0x646, 0x20, 0x688, 0x6d2, 0x20,
0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e, 0x679, 0x6a9,
0x627, 0x626, 0x631, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e, 0x648, 0x646,
0x627, 0x67e, 0x6d2, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e, 0x6cc, 0x627, 0x646,
0x6af, 0x20, 0x6cc, 0x627, 0x646, 0x6af, 0x20, 0x648, 0x642, 0x62a, 0x631, 0x6cc,
0x20, 0x6cc, 0x648, 0x646, 0x6cc, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x631,
0x648, 0x62a, 0x6be, 0x6cc, 0x631, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642,
0x62a, 0x633, 0x62e, 0x627, 0x644, 0x6cc, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645,
0x633, 0x62e, 0x627, 0x644, 0x6cc, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688,
0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x62e, 0x627, 0x644, 0x6cc,
0x646, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x645,
0x627, 0x631, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x645, 0x627, 0x631,
0x627, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x626, 0x645, 0x633, 0x645, 0x627, 0x631, 0x627, 0x20, 0x633, 0x645, 0x631,
0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x627, 0x645, 0x648, 0x622, 0x20, 0x679,
0x627, 0x626, 0x645, 0x633, 0x627, 0x645, 0x648, 0x622, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x627, 0x645,
0x648, 0x622, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679,
0x627, 0x626, 0x645, 0x633, 0x6cc, 0x634, 0x644, 0x6cc, 0x632, 0x20, 0x679, 0x627,
0x626, 0x645, 0x633, 0x646, 0x6af, 0x627, 0x67e, 0x648, 0x631, 0x20, 0x633, 0x679,
0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x648,
0x644, 0x645, 0x646, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x632, 0x20,
0x679, 0x627, 0x626, 0x645, 0x62c, 0x646, 0x648, 0x628, 0x6cc, 0x20, 0x62c, 0x627,
0x631, 0x62c, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x648, 0x631,
0x6cc, 0x646, 0x627, 0x645, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x633,
0x6cc, 0x648, 0x648, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x62a, 0x627, 0x6c1,
0x6cc, 0x62a, 0x6cc, 0x20, 0x679, 0x627, 0x626, 0x645, 0x62a, 0x627, 0x626, 0x6cc,
0x20, 0x67e, 0x6cc, 0x626, 0x6cc, 0x20, 0x679, 0x627, 0x626, 0x645, 0x62a, 0x627,
0x626, 0x6cc, 0x20, 0x67e, 0x6cc, 0x626, 0x6cc, 0x20, 0x627, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x62a, 0x626, 0x6cc,
0x20, 0x67e, 0x6cc, 0x626, 0x6cc, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626,
0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x62a, 0x627, 0x62c, 0x6a9, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x679, 0x648, 0x6a9,
0x6cc, 0x644, 0x627, 0x624, 0x20, 0x679, 0x627, 0x626, 0x645, 0x679, 0x648, 0x646,
0x6af, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x679, 0x648, 0x646, 0x6af, 0x627,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626,
0x645, 0x679, 0x648, 0x646, 0x6af, 0x627, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679,
0x627, 0x626, 0x645, 0x686, 0x648, 0x6a9, 0x20, 0x679, 0x627, 0x626, 0x645, 0x62a,
0x631, 0x6a9, 0x645, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6a9, 0x627,
0x20, 0x648, 0x642, 0x62a, 0x62a, 0x631, 0x6a9, 0x645, 0x627, 0x646, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc,
0x20, 0x648, 0x642, 0x62a, 0x62a, 0x631, 0x6a9, 0x645, 0x627, 0x646, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af,
0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x679, 0x648,
0x627, 0x644, 0x648, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6cc, 0x648, 0x631, 0x648,
0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x6cc,
0x648, 0x631, 0x648, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x6a9, 0x627, 0x20, 0x645,
0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x6cc, 0x648, 0x631,
0x648, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x648, 0x633,
0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642,
0x62a, 0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6a9, 0x627,
0x20, 0x648, 0x642, 0x62a, 0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a, 0x627, 0x646,
0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648,
0x642, 0x62a, 0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6a9,
0x627, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20,
0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x648, 0x627, 0x646, 0x648, 0x622, 0x679,
0x648, 0x20, 0x679, 0x627, 0x626, 0x645, 0x648, 0x627, 0x646, 0x648, 0x622, 0x679,
0x648, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x626, 0x645, 0x648, 0x627, 0x646, 0x648, 0x622, 0x679, 0x648, 0x20, 0x633, 0x645,
0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x648, 0x6cc, 0x646, 0x632, 0x648, 0x626,
0x6cc, 0x644, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x648, 0x644,
0x627, 0x62f, 0x6cc, 0x20, 0x648, 0x648, 0x633, 0x62a, 0x6a9, 0x20, 0x679, 0x627,
0x626, 0x645, 0x648, 0x644, 0x627, 0x62f, 0x6cc, 0x20, 0x648, 0x648, 0x633, 0x62a,
0x6a9, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x626, 0x645, 0x648, 0x644, 0x627, 0x62f, 0x6cc, 0x20, 0x648, 0x648, 0x633, 0x62a,
0x6a9, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x648, 0x648,
0x644, 0x6af, 0x648, 0x6af, 0x631, 0x627, 0x62f, 0x20, 0x679, 0x627, 0x626, 0x645,
0x648, 0x648, 0x644, 0x6af, 0x648, 0x6af, 0x631, 0x627, 0x62f, 0x20, 0x627, 0x633,
0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x648,
0x648, 0x644, 0x6af, 0x648, 0x6af, 0x631, 0x627, 0x62f, 0x20, 0x633, 0x645, 0x631,
0x20, 0x679, 0x627, 0x626, 0x645, 0x648, 0x648, 0x633, 0x679, 0x627, 0x6a9, 0x20,
0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x648, 0x6cc, 0x6a9, 0x20, 0x622, 0x626,
0x644, 0x6cc, 0x646, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x648, 0x627, 0x644,
0x6cc, 0x632, 0x20, 0x627, 0x648, 0x631, 0x20, 0x641, 0x679, 0x648, 0x646, 0x627,
0x20, 0x679, 0x627, 0x626, 0x645, 0x6cc, 0x6a9, 0x648, 0x62a, 0x633, 0x6a9, 0x20,
0x679, 0x627, 0x626, 0x645, 0x6cc, 0x6a9, 0x648, 0x62a, 0x633, 0x6a9, 0x20, 0x627,
0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645,
0x6cc, 0x6a9, 0x648, 0x62a, 0x633, 0x6a9, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679,
0x627, 0x626, 0x645, 0x6cc, 0x6a9, 0x627, 0x679, 0x6cc, 0x631, 0x650, 0x646, 0x628,
0x631, 0x6af, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6cc, 0x6a9, 0x627, 0x679, 0x6cc,
0x631, 0x650, 0x646, 0x628, 0x631, 0x6af, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646,
0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6cc, 0x6a9, 0x627, 0x679,
0x6cc, 0x631, 0x650, 0x646, 0x628, 0x631, 0x6af, 0x20, 0x633, 0x645, 0x631, 0x20,
0x679, 0x627, 0x626, 0x645, 0x627, 0x641, 0x63a, 0x627, 0x646, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x645, 0x6cc, 0x632, 0x648,
0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x645, 0x6cc, 0x632, 0x648,
0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x626, 0x645, 0x627, 0x6cc, 0x645, 0x6cc, 0x632, 0x648, 0x646, 0x20, 0x633, 0x645,
0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x639, 0x631, 0x628, 0x20, 0x679, 0x627,
0x626, 0x645, 0x639, 0x631, 0x628, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631,
0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20,
0x627, 0x631, 0x62c, 0x646, 0x679, 0x6cc, 0x646, 0x627, 0x20, 0x679, 0x627, 0x626,
0x645, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627, 0x631, 0x62c, 0x646, 0x679,
0x6cc, 0x646, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20,
0x679, 0x627, 0x626, 0x645, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627, 0x631,
0x62c, 0x646, 0x679, 0x6cc, 0x646, 0x627, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679,
0x627, 0x626, 0x645, 0x622, 0x631, 0x645, 0x6cc, 0x646, 0x6cc, 0x627, 0x20, 0x679,
0x627, 0x626, 0x645, 0x622, 0x631, 0x645, 0x6cc, 0x646, 0x6cc, 0x627, 0x20, 0x633,
0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x622,
0x631, 0x645, 0x6cc, 0x646, 0x6cc, 0x627, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679,
0x627, 0x626, 0x645, 0x622, 0x630, 0x631, 0x628, 0x627, 0x626, 0x6cc, 0x62c, 0x627,
0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x622, 0x630, 0x631, 0x628, 0x627, 0x626,
0x6cc, 0x62c, 0x627, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688,
0x20, 0x679, 0x627, 0x626, 0x645, 0x622, 0x630, 0x631, 0x628, 0x627, 0x626, 0x6cc,
0x62c, 0x627, 0x646, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645,
0x628, 0x646, 0x6af, 0x644, 0x6c1, 0x20, 0x62f, 0x6cc, 0x634, 0x20, 0x679, 0x627,
0x626, 0x645, 0x628, 0x646, 0x6af, 0x644, 0x6c1, 0x20, 0x62f, 0x6cc, 0x634, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645,
0x628, 0x646, 0x6af, 0x644, 0x6c1, 0x20, 0x62f, 0x6cc, 0x634, 0x20, 0x633, 0x645,
0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x628, 0x6be, 0x648, 0x679, 0x627, 0x646,
0x20, 0x679, 0x627, 0x626, 0x645, 0x628, 0x648, 0x644, 0x6cc, 0x648, 0x6cc, 0x627,
0x20, 0x679, 0x627, 0x626, 0x645, 0x628, 0x631, 0x627, 0x632, 0x6cc, 0x644, 0x6cc,
0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x626, 0x645, 0x686, 0x644, 0x6cc, 0x20, 0x679, 0x627, 0x626, 0x645, 0x686, 0x644,
0x6cc, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x626, 0x645, 0x686, 0x644, 0x6cc, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627,
0x626, 0x645, 0x6a9, 0x648, 0x644, 0x645, 0x628, 0x6cc, 0x627, 0x20, 0x633, 0x679,
0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x648,
0x644, 0x645, 0x628, 0x6cc, 0x627, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627,
0x626, 0x645, 0x627, 0x6cc, 0x633, 0x679, 0x631, 0x20, 0x622, 0x626, 0x644, 0x6cc,
0x646, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x633, 0x679, 0x631,
0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x633, 0x679, 0x6cc, 0x646,
0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x633, 0x679,
0x631, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x633, 0x645, 0x631,
0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x6a9, 0x648, 0x627, 0x688, 0x648,
0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x648, 0x633, 0x637, 0x6cc, 0x20, 0x6cc,
0x648, 0x631, 0x67e, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x627,
0x6a9, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646,
0x688, 0x632, 0x20, 0x679, 0x627, 0x626, 0x645, 0x641, 0x627, 0x6a9, 0x20, 0x644,
0x6cc, 0x646, 0x688, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x632, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645,
0x641, 0x627, 0x6a9, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x622, 0x626, 0x644,
0x6cc, 0x646, 0x688, 0x632, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626,
0x645, 0x641, 0x631, 0x6cc, 0x646, 0x686, 0x20, 0x6af, 0x6cc, 0x627, 0x646, 0x627,
0x20, 0x679, 0x627, 0x626, 0x645, 0x6af, 0x627, 0x644, 0x627, 0x67e, 0x627, 0x6af,
0x648, 0x632, 0x20, 0x679, 0x627, 0x626, 0x645, 0x62c, 0x627, 0x631, 0x62c, 0x6cc,
0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x626, 0x645, 0x62c, 0x627, 0x631, 0x62c, 0x6cc, 0x627, 0x20, 0x633, 0x645, 0x631,
0x20, 0x679, 0x627, 0x626, 0x645, 0x6af, 0x631, 0x6cc, 0x646, 0x20, 0x648, 0x686,
0x20, 0x645, 0x6cc, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x62e, 0x644, 0x6cc,
0x62c, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x626, 0x645, 0x627, 0x646, 0x688, 0x6cc, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646,
0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x631, 0x627,
0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x631, 0x627, 0x646, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645,
0x627, 0x633, 0x631, 0x627, 0x626, 0x6cc, 0x644, 0x20, 0x679, 0x627, 0x626, 0x645,
0x627, 0x633, 0x631, 0x627, 0x626, 0x6cc, 0x644, 0x20, 0x633, 0x679, 0x6cc, 0x646,
0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x634, 0x631, 0x642,
0x6cc, 0x20, 0x642, 0x632, 0x627, 0x62e, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x679,
0x627, 0x626, 0x645, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x642, 0x632, 0x627,
0x62e, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x631,
0x63a, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x627,
0x644, 0x62f, 0x6cc, 0x67e, 0x20, 0x679, 0x627, 0x626, 0x645, 0x646, 0x6cc, 0x67e,
0x627, 0x644, 0x20, 0x679, 0x627, 0x626, 0x645, 0x646, 0x6cc, 0x648, 0x632, 0x6cc,
0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x646, 0x6cc,
0x648, 0x632, 0x6cc, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x641, 0x631, 0x646,
0x627, 0x646, 0x688, 0x648, 0x20, 0x688, 0x6cc, 0x20, 0x646, 0x648, 0x631, 0x646,
0x6c1, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x641, 0x631, 0x646, 0x627, 0x646,
0x688, 0x648, 0x20, 0x688, 0x6cc, 0x20, 0x646, 0x648, 0x631, 0x646, 0x6c1, 0x627,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626,
0x645, 0x67e, 0x627, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x679, 0x627, 0x626,
0x645, 0x67e, 0x627, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e, 0x627, 0x6a9,
0x633, 0x62a, 0x627, 0x646, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626,
0x645, 0x67e, 0x6cc, 0x631, 0x627, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x679, 0x627,
0x626, 0x645, 0x67e, 0x6cc, 0x631, 0x627, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x633,
0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e,
0x6cc, 0x631, 0x627, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x633, 0x645, 0x631, 0x20,
0x679, 0x627, 0x626, 0x645, 0x67e, 0x6cc, 0x631, 0x648, 0x20, 0x679, 0x627, 0x626,
0x645, 0x67e, 0x6cc, 0x631, 0x648, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631,
0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e, 0x6cc, 0x631, 0x648, 0x20, 0x633,
0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x631, 0x648, 0x62a, 0x6be, 0x6cc,
0x631, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x648, 0x631, 0x6cc, 0x646,
0x627, 0x645, 0x20, 0x679, 0x627, 0x626, 0x645, 0x62a, 0x631, 0x6a9, 0x645, 0x627,
0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627,
0x626, 0x645, 0x6cc, 0x648, 0x631, 0x648, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x679,
0x627, 0x626, 0x645, 0x6cc, 0x648, 0x631, 0x648, 0x6af, 0x648, 0x626, 0x6d2, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645,
0x6cc, 0x648, 0x631, 0x648, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x633, 0x645, 0x631,
0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x626, 0x645, 0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20,
0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x648, 0x6cc, 0x646, 0x632,
0x648, 0x626, 0x6cc, 0x644, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x648, 0x648,
0x633, 0x679, 0x627, 0x6a9, 0x20, 0x679, 0x627, 0x626, 0x645, 0x626, 0x627, 0x643,
0x631, 0x6d0, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x643, 0x631,
0x6d0, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x643, 0x631, 0x649, 0x20, 0x64a,
0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626,
0x627, 0x641, 0x63a, 0x627, 0x646, 0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x626, 0x648, 0x62a, 0x62a, 0x6c7, 0x631, 0x627, 0x20,
0x626, 0x627, 0x641, 0x631, 0x649, 0x642, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x634, 0x6d5, 0x631, 0x642, 0x649, 0x64a, 0x20, 0x626, 0x627, 0x641, 0x631,
0x649, 0x642, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62c, 0x6d5, 0x646,
0x6c7, 0x628, 0x649, 0x64a, 0x20, 0x626, 0x627, 0x641, 0x631, 0x649, 0x642, 0x627,
0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x63a, 0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20, 0x626,
0x627, 0x641, 0x631, 0x649, 0x642, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x63a, 0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20, 0x626, 0x627, 0x641, 0x631, 0x649,
0x642, 0x627, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x63a, 0x6d5, 0x631, 0x628, 0x649, 0x64a,
0x20, 0x626, 0x627, 0x641, 0x631, 0x649, 0x642, 0x627, 0x20, 0x64a, 0x627, 0x632,
0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x644,
0x64a, 0x627, 0x633, 0x643, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626,
0x627, 0x644, 0x64a, 0x627, 0x633, 0x643, 0x627, 0x20, 0x626, 0x6c6, 0x644, 0x686,
0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626,
0x627, 0x644, 0x64a, 0x627, 0x633, 0x643, 0x627, 0x20, 0x64a, 0x627, 0x632, 0x644,
0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x644, 0x645,
0x6c7, 0x62a, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x644,
0x645, 0x6c7, 0x62a, 0x627, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644,
0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x644, 0x645,
0x6c7, 0x62a, 0x627, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x645, 0x627, 0x632, 0x648, 0x646, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x645, 0x627, 0x632, 0x648, 0x646,
0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x645, 0x627, 0x632, 0x648, 0x646, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x626, 0x648, 0x62a, 0x62a, 0x6c7, 0x631, 0x627, 0x20, 0x642, 0x649, 0x633, 0x649,
0x645, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x648, 0x62a, 0x62a, 0x6c7,
0x631, 0x627, 0x20, 0x642, 0x649, 0x633, 0x649, 0x645, 0x20, 0x626, 0x6c6, 0x644,
0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x626, 0x648, 0x62a, 0x62a, 0x6c7, 0x631, 0x627, 0x20, 0x642, 0x649, 0x633, 0x649,
0x645, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x634, 0x6d5, 0x631, 0x642, 0x649, 0x64a, 0x20, 0x642, 0x649, 0x633,
0x649, 0x645, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634, 0x6d5, 0x631, 0x642,
0x649, 0x64a, 0x20, 0x642, 0x649, 0x633, 0x649, 0x645, 0x20, 0x626, 0x6c6, 0x644,
0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x634, 0x6d5, 0x631, 0x642, 0x649, 0x64a, 0x20, 0x642, 0x649, 0x633, 0x649, 0x645,
0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x62a, 0x627, 0x63a, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62a, 0x627,
0x63a, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62a, 0x627, 0x63a, 0x20, 0x64a, 0x627, 0x632,
0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62a, 0x649, 0x646,
0x686, 0x20, 0x626, 0x648, 0x643, 0x64a, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x62a, 0x649, 0x646, 0x686, 0x20, 0x626, 0x648, 0x643, 0x64a, 0x627,
0x646, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62a, 0x649, 0x646, 0x686, 0x20, 0x626, 0x648,
0x643, 0x64a, 0x627, 0x646, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x646, 0x627, 0x62f, 0x649, 0x631,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x646, 0x627, 0x62f, 0x649,
0x631, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x646, 0x627, 0x62f, 0x649, 0x631,
0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x626, 0x627, 0x642, 0x62a, 0x627, 0x64a, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x626, 0x627, 0x642, 0x62a, 0x627, 0x64a, 0x20, 0x626, 0x6c6, 0x644, 0x686,
0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626,
0x627, 0x642, 0x62a, 0x627, 0x64a, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x642, 0x62a, 0x6c6, 0x628,
0x6d5, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x642, 0x62a, 0x6c6,
0x628, 0x6d5, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x642, 0x62a, 0x6c6, 0x628,
0x6d5, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x626, 0x6d5, 0x631, 0x6d5, 0x628, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x626, 0x6d5, 0x631, 0x6d5, 0x628, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5,
0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x6d5,
0x631, 0x6d5, 0x628, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x631, 0x6af, 0x6d0, 0x646, 0x62a, 0x649,
0x646, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x631, 0x6af,
0x6d0, 0x646, 0x62a, 0x649, 0x646, 0x627, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5,
0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627,
0x631, 0x6af, 0x6d0, 0x646, 0x62a, 0x649, 0x646, 0x627, 0x20, 0x64a, 0x627, 0x632,
0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x63a, 0x6d5, 0x631,
0x628, 0x649, 0x64a, 0x20, 0x626, 0x627, 0x631, 0x6af, 0x6d0, 0x646, 0x62a, 0x649,
0x646, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x63a, 0x6d5, 0x631, 0x628,
0x649, 0x64a, 0x20, 0x626, 0x627, 0x631, 0x6af, 0x6d0, 0x646, 0x62a, 0x649, 0x646,
0x627, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x63a, 0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20,
0x626, 0x627, 0x631, 0x6af, 0x6d0, 0x646, 0x62a, 0x649, 0x646, 0x627, 0x20, 0x64a,
0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626,
0x6d5, 0x631, 0x645, 0x6d0, 0x646, 0x649, 0x64a, 0x6d5, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x626, 0x6d5, 0x631, 0x645, 0x6d0, 0x646, 0x649, 0x64a, 0x6d5, 0x20,
0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x626, 0x6d5, 0x631, 0x645, 0x6d0, 0x646, 0x649, 0x64a, 0x6d5,
0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x626, 0x627, 0x62a, 0x644, 0x627, 0x646, 0x62a, 0x649, 0x643, 0x20, 0x626,
0x648, 0x643, 0x64a, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626,
0x627, 0x62a, 0x644, 0x627, 0x646, 0x62a, 0x649, 0x643, 0x20, 0x626, 0x648, 0x643,
0x64a, 0x627, 0x646, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649,
0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x62a, 0x644, 0x627,
0x646, 0x62a, 0x649, 0x643, 0x20, 0x626, 0x648, 0x643, 0x64a, 0x627, 0x646, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x626, 0x627, 0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x649, 0x64a, 0x6d5, 0x20,
0x626, 0x648, 0x62a, 0x62a, 0x6c7, 0x631, 0x627, 0x20, 0x642, 0x649, 0x633, 0x649,
0x645, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x6cb, 0x633, 0x62a,
0x631, 0x627, 0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x626, 0x648, 0x62a, 0x62a, 0x6c7,
0x631, 0x627, 0x20, 0x642, 0x649, 0x633, 0x649, 0x645, 0x20, 0x626, 0x6c6, 0x644,
0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x626, 0x627, 0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x649, 0x64a, 0x6d5, 0x20,
0x626, 0x648, 0x62a, 0x62a, 0x6c7, 0x631, 0x627, 0x20, 0x642, 0x649, 0x633, 0x649,
0x645, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x626, 0x627, 0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x649, 0x64a,
0x6d5, 0x20, 0x626, 0x648, 0x62a, 0x62a, 0x6c7, 0x631, 0x627, 0x20, 0x63a, 0x6d5,
0x631, 0x628, 0x649, 0x64a, 0x20, 0x642, 0x649, 0x633, 0x649, 0x645, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x6cb, 0x633, 0x62a, 0x649, 0x631, 0x627,
0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x626, 0x648, 0x62a, 0x62a, 0x6c7, 0x631, 0x627,
0x20, 0x63a, 0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20, 0x642, 0x649, 0x633, 0x649,
0x645, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x6cb, 0x633, 0x62a, 0x631, 0x627,
0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x626, 0x648, 0x62a, 0x62a, 0x6c7, 0x631, 0x627,
0x20, 0x63a, 0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20, 0x642, 0x649, 0x633, 0x649,
0x645, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x626, 0x627, 0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x649, 0x64a,
0x6d5, 0x20, 0x634, 0x6d5, 0x631, 0x642, 0x649, 0x64a, 0x20, 0x642, 0x649, 0x633,
0x649, 0x645, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x6cb, 0x633,
0x62a, 0x631, 0x627, 0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x634, 0x6d5, 0x631, 0x642,
0x649, 0x64a, 0x20, 0x642, 0x649, 0x633, 0x649, 0x645, 0x20, 0x626, 0x6c6, 0x644,
0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x626, 0x627, 0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x649, 0x64a, 0x6d5, 0x20,
0x634, 0x6d5, 0x631, 0x642, 0x649, 0x64a, 0x20, 0x642, 0x649, 0x633, 0x649, 0x645,
0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x626, 0x627, 0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x649, 0x64a, 0x6d5,
0x20, 0x63a, 0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20, 0x642, 0x649, 0x633, 0x649,
0x645, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x6cb, 0x633, 0x62a,
0x631, 0x627, 0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x63a, 0x6d5, 0x631, 0x628, 0x649,
0x64a, 0x20, 0x642, 0x649, 0x633, 0x649, 0x645, 0x20, 0x626, 0x6c6, 0x644, 0x686,
0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626,
0x627, 0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x63a,
0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20, 0x642, 0x649, 0x633, 0x649, 0x645, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x626, 0x6d5, 0x632, 0x6d5, 0x631, 0x628, 0x6d5, 0x64a, 0x62c, 0x627, 0x646, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x6d5, 0x632, 0x6d5, 0x631, 0x628, 0x6d5,
0x64a, 0x62c, 0x627, 0x646, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644,
0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x6d5, 0x632, 0x6d5,
0x631, 0x628, 0x6d5, 0x64a, 0x62c, 0x627, 0x646, 0x20, 0x64a, 0x627, 0x632, 0x644,
0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x632, 0x648,
0x631, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x632, 0x648, 0x631,
0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x632, 0x648, 0x631, 0x20, 0x64a, 0x627,
0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x628, 0x627,
0x6ad, 0x644, 0x627, 0x62f, 0x649, 0x634, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x628, 0x627, 0x6ad, 0x644, 0x627, 0x62f, 0x649, 0x634, 0x20, 0x626, 0x6c6, 0x644,
0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x628, 0x627, 0x6ad, 0x644, 0x627, 0x62f, 0x649, 0x634, 0x20, 0x64a, 0x627, 0x632,
0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x628, 0x6c7, 0x62a,
0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x628, 0x648, 0x644, 0x649,
0x6cb, 0x649, 0x64a, 0x6d5, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x628, 0x649,
0x631, 0x627, 0x632, 0x649, 0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x628, 0x649, 0x631, 0x627, 0x632, 0x649, 0x644, 0x649, 0x64a, 0x6d5,
0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x628, 0x649, 0x631, 0x627, 0x632, 0x649, 0x644, 0x649,
0x64a, 0x6d5, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x628, 0x649, 0x631, 0x6c7, 0x646, 0x649, 0x64a, 0x20, 0x62f,
0x627, 0x631, 0x6c7, 0x633, 0x633, 0x627, 0x644, 0x627, 0x645, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x64a, 0x6d0, 0x634, 0x649, 0x644, 0x20, 0x62a, 0x6c7, 0x645,
0x634, 0x6c7, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x64a, 0x6d0, 0x634,
0x649, 0x644, 0x20, 0x62a, 0x6c7, 0x645, 0x634, 0x6c7, 0x642, 0x20, 0x626, 0x6c6,
0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x64a, 0x6d0, 0x634, 0x649, 0x644, 0x20, 0x62a, 0x6c7, 0x645, 0x634, 0x6c7,
0x642, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x643, 0x627, 0x633, 0x6d0, 0x64a, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x686, 0x627, 0x645, 0x648, 0x631, 0x631, 0x648, 0x20, 0x626, 0x6c6, 0x644,
0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x686, 0x627, 0x62a, 0x627, 0x645, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x686,
0x627, 0x62a, 0x627, 0x645, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644,
0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x686, 0x627, 0x62a, 0x627,
0x645, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x686, 0x649, 0x644, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x686, 0x649, 0x644, 0x649, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644,
0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x686, 0x649, 0x644, 0x649,
0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x62c, 0x6c7, 0x6ad, 0x6af, 0x648, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x62c, 0x6c7, 0x6ad, 0x6af, 0x648, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645,
0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62c, 0x6c7, 0x6ad,
0x6af, 0x648, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x631, 0x648, 0x698, 0x62f, 0x6d0, 0x633, 0x62a, 0x6cb, 0x648,
0x20, 0x626, 0x627, 0x631, 0x649, 0x644, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x643, 0x648, 0x643, 0x6c7, 0x633, 0x20, 0x626, 0x627, 0x631, 0x649, 0x644,
0x649, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x643, 0x648, 0x644, 0x648, 0x645,
0x628, 0x649, 0x64a, 0x6d5, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x643, 0x648,
0x644, 0x648, 0x645, 0x628, 0x649, 0x64a, 0x6d5, 0x20, 0x626, 0x6c6, 0x644, 0x686,
0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x643,
0x648, 0x644, 0x648, 0x645, 0x628, 0x649, 0x64a, 0x6d5, 0x20, 0x64a, 0x627, 0x632,
0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x643, 0x6c7, 0x643,
0x20, 0x626, 0x627, 0x631, 0x627, 0x644, 0x644, 0x649, 0x631, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x643, 0x6c7, 0x643, 0x20, 0x626, 0x627, 0x631, 0x627,
0x644, 0x644, 0x649, 0x631, 0x649, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645,
0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x643, 0x6c7, 0x643,
0x20, 0x626, 0x627, 0x631, 0x627, 0x644, 0x644, 0x649, 0x631, 0x649, 0x20, 0x64a,
0x6d0, 0x631, 0x649, 0x645, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x643, 0x6c7, 0x628, 0x627, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x643, 0x6c7, 0x628, 0x627, 0x20, 0x626, 0x6c6, 0x644, 0x686,
0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x643,
0x6c7, 0x628, 0x627, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x62f, 0x627, 0x6cb, 0x649, 0x633, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x62f, 0x6c7, 0x645, 0x648, 0x646, 0x62a, 0x2d, 0x62f, 0x6c7,
0x631, 0x6cb, 0x649, 0x644, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634, 0x6d5,
0x631, 0x642, 0x649, 0x64a, 0x20, 0x62a, 0x649, 0x645, 0x648, 0x631, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x626, 0x6d0, 0x633, 0x62a, 0x6d0, 0x631, 0x20, 0x626,
0x627, 0x631, 0x649, 0x644, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x67e,
0x627, 0x633, 0x643, 0x627, 0x644, 0x64a, 0x627, 0x20, 0x626, 0x627, 0x631, 0x649,
0x644, 0x649, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x6d0, 0x633, 0x62a, 0x6d0, 0x631,
0x20, 0x626, 0x627, 0x631, 0x649, 0x644, 0x649, 0x20, 0x64a, 0x627, 0x632, 0x644,
0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x6d0, 0x643, 0x6cb,
0x627, 0x62f, 0x648, 0x631, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x648,
0x62a, 0x62a, 0x6c7, 0x631, 0x627, 0x20, 0x64a, 0x627, 0x6cb, 0x631, 0x648, 0x67e,
0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x648, 0x62a, 0x62a, 0x6c7,
0x631, 0x627, 0x20, 0x64a, 0x627, 0x6cb, 0x631, 0x648, 0x67e, 0x627, 0x20, 0x626,
0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x626, 0x648, 0x62a, 0x62a, 0x6c7, 0x631, 0x627, 0x20, 0x64a, 0x627,
0x6cb, 0x631, 0x648, 0x67e, 0x627, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634, 0x6d5, 0x631, 0x642, 0x649, 0x64a,
0x20, 0x64a, 0x627, 0x6cb, 0x631, 0x648, 0x67e, 0x627, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x634, 0x6d5, 0x631, 0x642, 0x649, 0x64a, 0x20, 0x64a, 0x627, 0x6cb,
0x631, 0x648, 0x67e, 0x627, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644,
0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634, 0x6d5, 0x631, 0x642,
0x649, 0x64a, 0x20, 0x64a, 0x627, 0x6cb, 0x631, 0x648, 0x67e, 0x627, 0x20, 0x64a,
0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x63a,
0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20, 0x64a, 0x627, 0x6cb, 0x631, 0x648, 0x67e,
0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x63a, 0x6d5, 0x631, 0x628, 0x649,
0x64a, 0x20, 0x64a, 0x627, 0x6cb, 0x631, 0x648, 0x67e, 0x627, 0x20, 0x626, 0x6c6,
0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x63a, 0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20, 0x64a, 0x627, 0x6cb, 0x631,
0x648, 0x67e, 0x627, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x641, 0x627, 0x644, 0x643, 0x644, 0x627, 0x646, 0x62f,
0x20, 0x626, 0x627, 0x631, 0x627, 0x644, 0x644, 0x649, 0x631, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x641, 0x627, 0x644, 0x643, 0x644, 0x627, 0x646, 0x62f,
0x20, 0x626, 0x627, 0x631, 0x627, 0x644, 0x644, 0x649, 0x631, 0x649, 0x20, 0x626,
0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x641, 0x627, 0x644, 0x643, 0x644, 0x627, 0x646, 0x62f, 0x20, 0x626,
0x627, 0x631, 0x627, 0x644, 0x644, 0x649, 0x631, 0x649, 0x20, 0x64a, 0x627, 0x632,
0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x641, 0x649, 0x62c,
0x649, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x641, 0x649, 0x62c, 0x649, 0x20,
0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x641, 0x649, 0x62c, 0x649, 0x20, 0x64a, 0x627, 0x632, 0x644,
0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x641, 0x649, 0x631, 0x627,
0x646, 0x633, 0x649, 0x64a, 0x6d5, 0x6af, 0x6d5, 0x20, 0x642, 0x627, 0x631, 0x627,
0x634, 0x644, 0x649, 0x642, 0x20, 0x6af, 0x649, 0x6cb, 0x649, 0x64a, 0x627, 0x646,
0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x641, 0x649, 0x631, 0x627, 0x646,
0x633, 0x649, 0x64a, 0x6d5, 0x6af, 0x6d5, 0x20, 0x642, 0x627, 0x631, 0x627, 0x634,
0x644, 0x649, 0x642, 0x20, 0x62c, 0x6d5, 0x646, 0x6c7, 0x628, 0x649, 0x64a, 0x20,
0x6cb, 0x6d5, 0x20, 0x626, 0x627, 0x646, 0x62a, 0x627, 0x631, 0x643, 0x62a, 0x649,
0x643, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6af, 0x627, 0x644, 0x627,
0x67e, 0x627, 0x6af, 0x648, 0x633, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6af,
0x627, 0x645, 0x628, 0x649, 0x64a, 0x6d0, 0x631, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x6af, 0x649, 0x631, 0x6c7, 0x632, 0x649, 0x64a, 0x6d5, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x6af, 0x649, 0x631, 0x6c7, 0x632, 0x649, 0x64a, 0x6d5, 0x20,
0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x6af, 0x649, 0x631, 0x6c7, 0x632, 0x649, 0x64a, 0x6d5, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x6af, 0x649, 0x644, 0x628, 0x6d0, 0x631, 0x62a, 0x20, 0x626, 0x627, 0x631, 0x627,
0x644, 0x644, 0x649, 0x631, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6af,
0x649, 0x631, 0x649, 0x646, 0x6cb, 0x649, 0x686, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x634, 0x6d5, 0x631, 0x642, 0x649, 0x64a, 0x20, 0x6af, 0x649, 0x631, 0x6d0,
0x646, 0x644, 0x627, 0x646, 0x62f, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634,
0x6d5, 0x631, 0x642, 0x649, 0x64a, 0x20, 0x6af, 0x649, 0x631, 0x6d0, 0x646, 0x644,
0x627, 0x646, 0x62f, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649,
0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634, 0x6d5, 0x631, 0x642, 0x649,
0x64a, 0x20, 0x6af, 0x649, 0x631, 0x6d0, 0x646, 0x644, 0x627, 0x646, 0x62f, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x63a, 0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20, 0x6af, 0x649, 0x631, 0x6d0, 0x646,
0x644, 0x627, 0x646, 0x62f, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x63a, 0x6d5,
0x631, 0x628, 0x649, 0x64a, 0x20, 0x6af, 0x649, 0x631, 0x6d0, 0x646, 0x644, 0x627,
0x646, 0x62f, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x63a, 0x6d5, 0x631, 0x628, 0x649, 0x64a,
0x20, 0x6af, 0x649, 0x631, 0x6d0, 0x646, 0x644, 0x627, 0x646, 0x62f, 0x20, 0x64a,
0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6af,
0x6c7, 0x626, 0x627, 0x645, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644,
0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6af, 0x6c7, 0x644, 0x641,
0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x6be, 0x627, 0x6cb, 0x627, 0x64a, 0x2d, 0x626, 0x627,
0x644, 0x6d0, 0x64a, 0x6c7, 0x62a, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6be,
0x627, 0x6cb, 0x627, 0x64a, 0x2d, 0x626, 0x627, 0x644, 0x6d0, 0x64a, 0x6c7, 0x62a,
0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x6be, 0x627, 0x6cb, 0x627, 0x64a, 0x2d, 0x626, 0x627,
0x644, 0x6d0, 0x64a, 0x6c7, 0x62a, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634, 0x64a, 0x627, 0x6ad, 0x6af, 0x627,
0x6ad, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634, 0x64a, 0x627, 0x6ad, 0x6af,
0x627, 0x6ad, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634, 0x64a, 0x627, 0x6ad, 0x6af, 0x627,
0x6ad, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x62e, 0x648, 0x6cb, 0x62f, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x62e, 0x648, 0x6cb, 0x62f, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644,
0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62e, 0x648, 0x6cb, 0x62f,
0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x6be, 0x649, 0x646, 0x62f, 0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x626,
0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x6be, 0x649, 0x646, 0x62f, 0x649, 0x20, 0x626, 0x648, 0x643, 0x64a,
0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6be, 0x649, 0x646, 0x62f,
0x649, 0x20, 0x686, 0x649, 0x646, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x626, 0x648, 0x62a, 0x62a, 0x6c7, 0x631, 0x627, 0x20, 0x6be, 0x649, 0x646, 0x62f,
0x648, 0x646, 0x6d0, 0x632, 0x649, 0x64a, 0x6d5, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x634, 0x6d5, 0x631, 0x642, 0x649, 0x64a, 0x20, 0x6be, 0x649, 0x646, 0x62f,
0x648, 0x646, 0x6d0, 0x632, 0x649, 0x64a, 0x6d5, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x63a, 0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20, 0x6be, 0x649, 0x646, 0x62f,
0x648, 0x646, 0x6d0, 0x632, 0x649, 0x64a, 0x6d5, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x626, 0x649, 0x631, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x626, 0x649, 0x631, 0x627, 0x646, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645,
0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x649, 0x631,
0x627, 0x646, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x626, 0x649, 0x631, 0x643, 0x6c7, 0x62a, 0x633, 0x643, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x649, 0x631, 0x643, 0x6c7, 0x62a, 0x633,
0x643, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x649, 0x631, 0x643, 0x6c7, 0x62a, 0x633,
0x643, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x626, 0x649, 0x633, 0x631, 0x627, 0x626, 0x649, 0x644, 0x649, 0x64a,
0x6d5, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x649, 0x633, 0x631, 0x627,
0x626, 0x649, 0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5,
0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x649,
0x633, 0x631, 0x627, 0x626, 0x649, 0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x64a, 0x627,
0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x64a, 0x627,
0x67e, 0x648, 0x646, 0x649, 0x64a, 0x6d5, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x64a, 0x627, 0x67e, 0x648, 0x646, 0x649, 0x64a, 0x6d5, 0x20, 0x626, 0x6c6, 0x644,
0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x64a, 0x627, 0x67e, 0x648, 0x646, 0x649, 0x64a, 0x6d5, 0x20, 0x64a, 0x627, 0x632,
0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x67e, 0x6d0, 0x62a,
0x631, 0x648, 0x67e, 0x627, 0x6cb, 0x644, 0x648, 0x6cb, 0x633, 0x643, 0x2d, 0x643,
0x627, 0x645, 0x686, 0x627, 0x62a, 0x643, 0x633, 0x643, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x67e, 0x6d0, 0x62a, 0x631, 0x648, 0x67e, 0x627, 0x6cb, 0x644,
0x648, 0x6cb, 0x633, 0x643, 0x2d, 0x643, 0x627, 0x645, 0x686, 0x627, 0x62a, 0x643,
0x633, 0x643, 0x649, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649,
0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x67e, 0x6d0, 0x62a, 0x631, 0x648,
0x67e, 0x627, 0x6cb, 0x644, 0x648, 0x6cb, 0x633, 0x643, 0x2d, 0x643, 0x627, 0x645,
0x686, 0x627, 0x62a, 0x643, 0x633, 0x643, 0x649, 0x20, 0x64a, 0x627, 0x632, 0x644,
0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634, 0x6d5, 0x631, 0x642,
0x649, 0x64a, 0x20, 0x642, 0x627, 0x632, 0x627, 0x642, 0x649, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x63a, 0x6d5, 0x631, 0x628, 0x649,
0x64a, 0x20, 0x642, 0x627, 0x632, 0x627, 0x642, 0x649, 0x633, 0x62a, 0x627, 0x646,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x643, 0x648, 0x631, 0x649, 0x64a, 0x6d5,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x643, 0x648, 0x631, 0x649, 0x64a, 0x6d5,
0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x643, 0x648, 0x631, 0x649, 0x64a, 0x6d5, 0x20, 0x64a,
0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x643,
0x648, 0x633, 0x631, 0x627, 0x626, 0x6d0, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x643, 0x649, 0x631, 0x627, 0x633, 0x646, 0x648, 0x64a, 0x627, 0x631, 0x633, 0x643,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x643, 0x649, 0x631, 0x627, 0x633, 0x646,
0x648, 0x64a, 0x627, 0x631, 0x633, 0x643, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5,
0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x643, 0x649,
0x631, 0x627, 0x633, 0x646, 0x648, 0x64a, 0x627, 0x631, 0x633, 0x643, 0x20, 0x64a,
0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x642,
0x649, 0x631, 0x63a, 0x649, 0x632, 0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x633, 0x649, 0x631, 0x649, 0x20, 0x644, 0x627, 0x646,
0x643, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x644, 0x627, 0x64a, 0x646,
0x20, 0x626, 0x627, 0x631, 0x627, 0x644, 0x644, 0x649, 0x631, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x644, 0x648, 0x631, 0x62f, 0x2d, 0x62e, 0x627, 0x64a,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x644, 0x648, 0x631, 0x62f, 0x2d, 0x62e,
0x627, 0x64a, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x644, 0x648, 0x631, 0x62f, 0x2d, 0x62e,
0x627, 0x64a, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x626, 0x627, 0x6cb, 0x645, 0x6d0, 0x646, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x626, 0x627, 0x6cb, 0x645, 0x6d0, 0x646, 0x20, 0x626, 0x6c6,
0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x626, 0x627, 0x6cb, 0x645, 0x6d0, 0x646, 0x20, 0x64a, 0x627, 0x632, 0x644,
0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x645, 0x627, 0x6af, 0x627,
0x62f, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x645, 0x627, 0x6af,
0x627, 0x62f, 0x627, 0x646, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644,
0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x645, 0x627, 0x6af, 0x627,
0x62f, 0x627, 0x646, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x645, 0x627, 0x644, 0x627, 0x64a, 0x634, 0x649, 0x64a,
0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x645, 0x627, 0x644, 0x62f, 0x649,
0x6cb, 0x6d0, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x645, 0x627, 0x631, 0x643,
0x649, 0x632, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x645, 0x627, 0x631, 0x634,
0x627, 0x644, 0x20, 0x626, 0x627, 0x631, 0x627, 0x644, 0x644, 0x649, 0x631, 0x649,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x645, 0x627, 0x6cb, 0x631, 0x649, 0x62a,
0x649, 0x626, 0x6c7, 0x633, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x645, 0x627,
0x6cb, 0x631, 0x649, 0x62a, 0x649, 0x626, 0x6c7, 0x633, 0x20, 0x626, 0x6c6, 0x644,
0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x645, 0x627, 0x6cb, 0x631, 0x649, 0x62a, 0x649, 0x626, 0x6c7, 0x633, 0x20, 0x64a,
0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x645,
0x627, 0x6cb, 0x633, 0x648, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x645,
0x6d0, 0x643, 0x633, 0x649, 0x643, 0x627, 0x20, 0x62a, 0x649, 0x646, 0x686, 0x20,
0x626, 0x648, 0x643, 0x64a, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x645, 0x6d0, 0x643, 0x633, 0x649, 0x643, 0x627, 0x20, 0x62a, 0x649, 0x646, 0x686,
0x20, 0x626, 0x648, 0x643, 0x64a, 0x627, 0x646, 0x20, 0x626, 0x6c6, 0x644, 0x686,
0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x645,
0x6d0, 0x643, 0x633, 0x649, 0x643, 0x627, 0x20, 0x62a, 0x649, 0x646, 0x686, 0x20,
0x626, 0x648, 0x643, 0x64a, 0x627, 0x646, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649,
0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x6c7, 0x644, 0x627, 0x646,
0x628, 0x627, 0x62a, 0x648, 0x631, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626,
0x6c7, 0x644, 0x627, 0x646, 0x628, 0x627, 0x62a, 0x648, 0x631, 0x20, 0x626, 0x6c6,
0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x626, 0x6c7, 0x644, 0x627, 0x646, 0x628, 0x627, 0x62a, 0x648, 0x631, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x645, 0x648, 0x633, 0x643, 0x6cb, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x645, 0x648, 0x633, 0x643, 0x6cb, 0x627, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5,
0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x645, 0x648,
0x633, 0x643, 0x6cb, 0x627, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x628, 0x649, 0x631, 0x645, 0x627, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x646, 0x627, 0x6cb, 0x631, 0x6c7, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x646, 0x6d0, 0x67e, 0x627, 0x644, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x64a, 0x6d0, 0x6ad, 0x649, 0x20, 0x643, 0x627, 0x644, 0x6d0, 0x62f,
0x648, 0x646, 0x649, 0x64a, 0x6d5, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x64a,
0x6d0, 0x6ad, 0x649, 0x20, 0x643, 0x627, 0x644, 0x6d0, 0x62f, 0x648, 0x646, 0x649,
0x64a, 0x6d5, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x64a, 0x6d0, 0x6ad, 0x649, 0x20, 0x643,
0x627, 0x644, 0x6d0, 0x62f, 0x648, 0x646, 0x649, 0x64a, 0x6d5, 0x20, 0x64a, 0x627,
0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x64a, 0x6d0,
0x6ad, 0x649, 0x20, 0x632, 0x6d0, 0x644, 0x627, 0x646, 0x62f, 0x649, 0x64a, 0x6d5,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x64a, 0x6d0, 0x6ad, 0x649, 0x20, 0x632,
0x6d0, 0x644, 0x627, 0x646, 0x62f, 0x649, 0x64a, 0x6d5, 0x20, 0x626, 0x6c6, 0x644,
0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x64a, 0x6d0, 0x6ad, 0x649, 0x20, 0x632, 0x6d0, 0x644, 0x627, 0x646, 0x62f, 0x649,
0x64a, 0x6d5, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x646, 0x649, 0x6cb, 0x641, 0x648, 0x626, 0x6c7, 0x646, 0x644,
0x627, 0x646, 0x62f, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x646, 0x649, 0x6cb,
0x641, 0x648, 0x626, 0x6c7, 0x646, 0x644, 0x627, 0x646, 0x62f, 0x20, 0x626, 0x6c6,
0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x646, 0x649, 0x6cb, 0x641, 0x648, 0x626, 0x6c7, 0x646, 0x644, 0x627, 0x646,
0x62f, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x646, 0x649, 0x64a, 0x6c7, 0x626, 0x6d0, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x646, 0x648, 0x631, 0x641, 0x648, 0x644, 0x643, 0x20, 0x626, 0x627,
0x631, 0x627, 0x644, 0x644, 0x649, 0x631, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x646, 0x648, 0x631, 0x641, 0x648, 0x644, 0x643, 0x20, 0x626, 0x627, 0x631,
0x627, 0x644, 0x644, 0x649, 0x631, 0x649, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5,
0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x646, 0x648,
0x631, 0x641, 0x648, 0x644, 0x643, 0x20, 0x626, 0x627, 0x631, 0x627, 0x644, 0x644,
0x649, 0x631, 0x649, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x641, 0x6d0, 0x631, 0x646, 0x627, 0x646, 0x62f, 0x648,
0x2d, 0x646, 0x648, 0x631, 0x648, 0x646, 0x62e, 0x627, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x641, 0x6d0, 0x631, 0x646, 0x627, 0x646, 0x62f, 0x648, 0x2d, 0x646,
0x648, 0x631, 0x648, 0x646, 0x62e, 0x627, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5,
0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x641, 0x6d0,
0x631, 0x646, 0x627, 0x646, 0x62f, 0x648, 0x2d, 0x646, 0x648, 0x631, 0x648, 0x646,
0x62e, 0x627, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x634, 0x649, 0x645, 0x627, 0x644, 0x649, 0x64a, 0x20, 0x645,
0x627, 0x631, 0x649, 0x64a, 0x627, 0x646, 0x627, 0x20, 0x626, 0x627, 0x631, 0x627,
0x644, 0x644, 0x649, 0x631, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x646,
0x648, 0x6cb, 0x648, 0x633, 0x649, 0x628, 0x649, 0x631, 0x633, 0x643, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x646, 0x648, 0x6cb, 0x648, 0x633, 0x649, 0x628, 0x649,
0x631, 0x633, 0x643, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649,
0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x646, 0x648, 0x6cb, 0x648, 0x633,
0x649, 0x628, 0x649, 0x631, 0x633, 0x643, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649,
0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x648, 0x645, 0x633, 0x643,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x648, 0x645, 0x633, 0x643, 0x20,
0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x626, 0x648, 0x645, 0x633, 0x643, 0x20, 0x64a, 0x627, 0x632,
0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x67e, 0x627, 0x643,
0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x67e,
0x627, 0x643, 0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x626, 0x6c6, 0x644, 0x686,
0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x67e,
0x627, 0x643, 0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x64a, 0x627, 0x632, 0x644,
0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x67e, 0x627, 0x644, 0x627,
0x6cb, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x67e, 0x627, 0x67e, 0x6c7, 0x626,
0x627, 0x20, 0x64a, 0x6d0, 0x6ad, 0x649, 0x20, 0x6af, 0x649, 0x6cb, 0x649, 0x646,
0x6d0, 0x64a, 0x6d5, 0x633, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x67e,
0x627, 0x631, 0x627, 0x6af, 0x6cb, 0x627, 0x64a, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x67e, 0x627, 0x631, 0x627, 0x6af, 0x6cb, 0x627, 0x64a, 0x20, 0x626, 0x6c6,
0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x67e, 0x627, 0x631, 0x627, 0x6af, 0x6cb, 0x627, 0x64a, 0x20, 0x64a, 0x627,
0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x67e, 0x6d0,
0x631, 0x6c7, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x67e, 0x6d0, 0x631, 0x6c7,
0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x67e, 0x6d0, 0x631, 0x6c7, 0x20, 0x64a, 0x627, 0x632,
0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x641, 0x649, 0x644,
0x649, 0x67e, 0x67e, 0x649, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x641,
0x649, 0x644, 0x649, 0x67e, 0x67e, 0x649, 0x646, 0x20, 0x626, 0x6c6, 0x644, 0x686,
0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x641,
0x649, 0x644, 0x649, 0x67e, 0x67e, 0x649, 0x646, 0x20, 0x64a, 0x627, 0x632, 0x644,
0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x641, 0x6d0, 0x646, 0x649,
0x643, 0x633, 0x20, 0x626, 0x627, 0x631, 0x627, 0x644, 0x644, 0x649, 0x631, 0x649,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x633, 0x627, 0x64a, 0x646, 0x649, 0x62a,
0x20, 0x67e, 0x649, 0x626, 0x6d0, 0x631, 0x20, 0x6cb, 0x6d5, 0x20, 0x645, 0x649,
0x643, 0x6d0, 0x644, 0x648, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x633,
0x627, 0x64a, 0x646, 0x649, 0x62a, 0x20, 0x67e, 0x649, 0x626, 0x6d0, 0x631, 0x20,
0x6cb, 0x6d5, 0x20, 0x645, 0x649, 0x643, 0x6d0, 0x644, 0x648, 0x646, 0x20, 0x626,
0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x633, 0x627, 0x64a, 0x646, 0x649, 0x62a, 0x20, 0x67e, 0x649, 0x626,
0x6d0, 0x631, 0x20, 0x6cb, 0x6d5, 0x20, 0x645, 0x649, 0x643, 0x6d0, 0x644, 0x648,
0x646, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x67e, 0x649, 0x62a, 0x643, 0x627, 0x64a, 0x631, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x67e, 0x648, 0x646, 0x67e, 0x6d0, 0x64a, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x642, 0x649, 0x632, 0x649, 0x644, 0x626, 0x648, 0x631, 0x62f,
0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x642, 0x649, 0x632, 0x649, 0x644,
0x626, 0x648, 0x631, 0x62f, 0x627, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645,
0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x642, 0x649, 0x632,
0x649, 0x644, 0x626, 0x648, 0x631, 0x62f, 0x627, 0x20, 0x64a, 0x627, 0x632, 0x644,
0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x631, 0x6d0, 0x626, 0x648,
0x646, 0x649, 0x64a, 0x648, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x631,
0x648, 0x62a, 0x6d0, 0x631, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x633,
0x627, 0x62e, 0x627, 0x631, 0x649, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x633, 0x627, 0x62e, 0x627, 0x631, 0x649, 0x646, 0x20, 0x626, 0x6c6, 0x644, 0x686,
0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x633,
0x627, 0x62e, 0x627, 0x631, 0x649, 0x646, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649,
0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x633, 0x627, 0x645, 0x627, 0x631,
0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x633, 0x627, 0x645, 0x627, 0x631,
0x627, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x633, 0x627, 0x645, 0x627, 0x631, 0x627, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x633, 0x627, 0x645, 0x648, 0x626, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x633, 0x627, 0x645, 0x648, 0x626, 0x627, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5,
0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x633, 0x6d5,
0x645, 0x6d5, 0x631, 0x642, 0x6d5, 0x646, 0x62a, 0x20, 0x64a, 0x627, 0x632, 0x644,
0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x633, 0x6d0, 0x64a, 0x634,
0x6d0, 0x644, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x633, 0x649, 0x646, 0x6af,
0x627, 0x67e, 0x648, 0x631, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x633, 0x648,
0x644, 0x648, 0x645, 0x648, 0x646, 0x20, 0x626, 0x627, 0x631, 0x627, 0x644, 0x644,
0x649, 0x631, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62c, 0x6d5, 0x646,
0x6c7, 0x628, 0x649, 0x64a, 0x20, 0x62c, 0x648, 0x631, 0x62c, 0x649, 0x64a, 0x6d5,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x633, 0x6c7, 0x631, 0x649, 0x646, 0x627,
0x645, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634, 0x648, 0x6cb, 0x627, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62a, 0x627, 0x64a, 0x62a, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x62a, 0x6d5, 0x64a, 0x628, 0x6d0, 0x64a, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x62a, 0x6d5, 0x64a, 0x628, 0x6d0, 0x64a, 0x20, 0x626,
0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x62a, 0x6d5, 0x64a, 0x628, 0x6d0, 0x64a, 0x20, 0x64a, 0x627, 0x632,
0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62a, 0x627, 0x62c,
0x649, 0x643, 0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x62a, 0x648, 0x643, 0x6d0, 0x644, 0x627, 0x6cb, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x62a, 0x648, 0x646, 0x6af, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x62a, 0x648, 0x646, 0x6af, 0x627, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5,
0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62a, 0x648,
0x646, 0x6af, 0x627, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x686, 0x6c7, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x62a, 0x6c8, 0x631, 0x643, 0x645, 0x6d5, 0x646, 0x649, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62a, 0x6c8, 0x631, 0x643, 0x645,
0x6d5, 0x646, 0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x626, 0x6c6, 0x644, 0x686,
0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62a,
0x6c8, 0x631, 0x643, 0x645, 0x6d5, 0x646, 0x649, 0x633, 0x62a, 0x627, 0x646, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x62a, 0x6c7, 0x6cb, 0x627, 0x644, 0x6c7, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x626, 0x6c7, 0x631, 0x6c7, 0x6af, 0x6cb, 0x627, 0x64a, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x626, 0x6c7, 0x631, 0x6c7, 0x6af, 0x6cb, 0x627, 0x64a, 0x20, 0x626,
0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x626, 0x6c7, 0x631, 0x6c7, 0x6af, 0x6cb, 0x627, 0x64a, 0x20, 0x64a,
0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626,
0x6c6, 0x632, 0x628, 0x6d0, 0x643, 0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x626, 0x6c6, 0x632, 0x628, 0x6d0, 0x643, 0x649, 0x633,
0x62a, 0x627, 0x646, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649,
0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x6c6, 0x632, 0x628, 0x6d0,
0x643, 0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649,
0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6cb, 0x627, 0x646, 0x6c7, 0x626,
0x627, 0x62a, 0x6c7, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6cb, 0x627, 0x646,
0x6c7, 0x626, 0x627, 0x62a, 0x6c7, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645,
0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6cb, 0x627, 0x646,
0x6c7, 0x626, 0x627, 0x62a, 0x6c7, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6cb, 0x6d0, 0x646, 0x6d0, 0x632, 0x6c7,
0x626, 0x6d0, 0x644, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6cb, 0x649,
0x644, 0x627, 0x62f, 0x649, 0x6cb, 0x648, 0x633, 0x62a, 0x648, 0x643, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x6cb, 0x649, 0x644, 0x627, 0x62f, 0x649, 0x6cb, 0x648,
0x633, 0x62a, 0x648, 0x643, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644,
0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6cb, 0x649, 0x644, 0x627,
0x62f, 0x649, 0x6cb, 0x648, 0x633, 0x62a, 0x648, 0x643, 0x20, 0x64a, 0x627, 0x632,
0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6cb, 0x648, 0x644,
0x6af, 0x627, 0x6af, 0x631, 0x627, 0x62f, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x6cb, 0x648, 0x644, 0x6af, 0x627, 0x6af, 0x631, 0x627, 0x62f, 0x20, 0x626, 0x6c6,
0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x6cb, 0x648, 0x644, 0x6af, 0x627, 0x6af, 0x631, 0x627, 0x62f, 0x20, 0x64a,
0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6cb,
0x6d0, 0x64a, 0x643, 0x20, 0x626, 0x627, 0x631, 0x649, 0x644, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x6cb, 0x627, 0x644, 0x644, 0x649, 0x633, 0x20, 0x6cb,
0x6d5, 0x20, 0x641, 0x6c7, 0x62a, 0x6c7, 0x646, 0x627, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x64a, 0x627, 0x643, 0x6c7, 0x62a, 0x633, 0x643, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x64a, 0x627, 0x643, 0x6c7, 0x62a, 0x633, 0x643, 0x20, 0x626,
0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x64a, 0x627, 0x643, 0x6c7, 0x62a, 0x633, 0x643, 0x20, 0x64a, 0x627,
0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x64a, 0x6d0,
0x643, 0x627, 0x62a, 0x6d0, 0x631, 0x649, 0x646, 0x628, 0x6c7, 0x631, 0x6af, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x64a, 0x6d0, 0x643, 0x627, 0x62a, 0x6d0, 0x631,
0x649, 0x646, 0x628, 0x6c7, 0x631, 0x6af, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5,
0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x64a, 0x6d0,
0x643, 0x627, 0x62a, 0x6d0, 0x631, 0x649, 0x646, 0x628, 0x6c7, 0x631, 0x6af, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x41, 0x66, 0x67, 0x2bb, 0x6f, 0x6e, 0x69, 0x73, 0x74, 0x6f, 0x6e, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x72, 0x6b, 0x61, 0x7a, 0x69,
0x79, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69, 0x79, 0x20, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4a, 0x61,
0x6e, 0x75, 0x62, 0x69, 0x79, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x47, 0x2bb, 0x61, 0x72, 0x62, 0x69, 0x79, 0x20, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47,
0x2bb, 0x61, 0x72, 0x62, 0x69, 0x79, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x47, 0x2bb, 0x61, 0x72, 0x62, 0x69, 0x79, 0x20,
0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x6c, 0x79, 0x61, 0x73, 0x6b,
0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x6c, 0x79, 0x61, 0x73,
0x6b, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x6c, 0x79, 0x61, 0x73, 0x6b, 0x61,
0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x6b, 0x61, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x6b, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x6b, 0x61, 0x20, 0x79, 0x6f,
0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x72,
0x6b, 0x61, 0x7a, 0x69, 0x79, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x72, 0x6b, 0x61,
0x7a, 0x69, 0x79, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x4d, 0x61, 0x72, 0x6b, 0x61, 0x7a, 0x69, 0x79, 0x20, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69,
0x79, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69, 0x79, 0x20, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x68, 0x61,
0x72, 0x71, 0x69, 0x79, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x54, 0x6f, 0x67, 0x2018, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x20, 0x28,
0x41, 0x51, 0x53, 0x48, 0x29, 0x54, 0x6f, 0x67, 0x2018, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x20, 0x28, 0x41, 0x51, 0x53, 0x48, 0x29, 0x54, 0x6f, 0x67, 0x2018, 0x20,
0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x20,
0x28, 0x41, 0x51, 0x53, 0x48, 0x29, 0x54, 0x69, 0x6e, 0x63, 0x68, 0x20,
0x6f, 0x6b, 0x65, 0x61, 0x6e, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x54, 0x69, 0x6e, 0x63, 0x68, 0x20, 0x6f, 0x6b, 0x65, 0x61, 0x6e, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x54, 0x69, 0x6e, 0x63, 0x68, 0x20, 0x6f, 0x6b, 0x65,
0x61, 0x6e, 0x69, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x41, 0x70, 0x69, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x41, 0x70, 0x69, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x70, 0x69, 0x61,
0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x53, 0x61, 0x75, 0x64, 0x69, 0x79, 0x61, 0x20, 0x41, 0x72, 0x61, 0x62,
0x69, 0x73, 0x74, 0x6f, 0x6e, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x53, 0x61, 0x75, 0x64, 0x69, 0x79, 0x61, 0x20, 0x41, 0x72, 0x61, 0x62,
0x69, 0x73, 0x74, 0x6f, 0x6e, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x61, 0x75,
0x64, 0x69, 0x79, 0x61, 0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x73, 0x74,
0x6f, 0x6e, 0x69, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x47, 0x2bb, 0x61, 0x72, 0x62, 0x69, 0x79, 0x20, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x47, 0x2bb, 0x61, 0x72, 0x62, 0x69, 0x79, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x2bb,
0x61, 0x72, 0x62, 0x69, 0x79, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x41, 0x72, 0x6d, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x6f,
0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x72, 0x6d, 0x61, 0x6e,
0x69, 0x73, 0x74, 0x6f, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x72, 0x6d, 0x61,
0x6e, 0x69, 0x73, 0x74, 0x6f, 0x6e, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x6b, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x6b, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x72, 0x6b, 0x61, 0x7a,
0x69, 0x79, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79,
0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x72, 0x6b, 0x61,
0x7a, 0x69, 0x79, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x79, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x72, 0x6b, 0x61, 0x7a, 0x69,
0x79, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61,
0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x4d, 0x61, 0x72, 0x6b, 0x61, 0x7a, 0x69, 0x79, 0x20, 0x41, 0x76, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x67, 0x2018, 0x61, 0x72,
0x62, 0x69, 0x79, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x72,
0x6b, 0x61, 0x7a, 0x69, 0x79, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x79, 0x61, 0x20, 0x67, 0x2018, 0x61, 0x72, 0x62, 0x69, 0x79,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x4d, 0x61, 0x72, 0x6b, 0x61, 0x7a, 0x69, 0x79, 0x20,
0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x67,
0x2018, 0x61, 0x72, 0x62, 0x69, 0x79, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69,
0x79, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69,
0x79, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69, 0x79, 0x20, 0x41,
0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x79, 0x6f,
0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x2018, 0x61,
0x72, 0x62, 0x69, 0x79, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x79, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x2018, 0x61,
0x72, 0x62, 0x69, 0x79, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x79, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x2018, 0x61, 0x72, 0x62, 0x69,
0x79, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61,
0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x4f, 0x7a, 0x61, 0x72, 0x62, 0x61, 0x79, 0x6a, 0x6f, 0x6e, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x4f, 0x7a, 0x61, 0x72, 0x62, 0x61, 0x79, 0x6a,
0x6f, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x4f, 0x7a, 0x61, 0x72, 0x62, 0x61, 0x79,
0x6a, 0x6f, 0x6e, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x41, 0x7a, 0x6f, 0x72, 0x20, 0x6f, 0x72, 0x6f, 0x6c,
0x6c, 0x61, 0x72, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x7a,
0x6f, 0x72, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x41, 0x7a, 0x6f, 0x72, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c,
0x61, 0x72, 0x69, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73,
0x68, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x73, 0x68, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x42, 0x75, 0x74, 0x61, 0x6e, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x79,
0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x42, 0x72, 0x61, 0x7a, 0x69,
0x6c, 0x69, 0x79, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x42, 0x72,
0x61, 0x7a, 0x69, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x42, 0x72,
0x61, 0x7a, 0x69, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67,
0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x42, 0x72, 0x75, 0x6e, 0x65,
0x79, 0x2d, 0x44, 0x6f, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x6f, 0x6d,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x61, 0x62, 0x6f, 0x2d, 0x56,
0x65, 0x72, 0x64, 0x65, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x61,
0x62, 0x6f, 0x2d, 0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b,
0x61, 0x62, 0x6f, 0x2d, 0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x79, 0x6f,
0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x43, 0x68, 0x61,
0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x43, 0x68, 0x61, 0x74,
0x65, 0x6d, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x43, 0x68, 0x61, 0x74,
0x65, 0x6d, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x43, 0x68, 0x61, 0x74, 0x65, 0x6d, 0x20,
0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x43,
0x68, 0x69, 0x6c, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x43, 0x68,
0x69, 0x6c, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x43, 0x68, 0x69, 0x6c, 0x69, 0x20,
0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x58,
0x69, 0x74, 0x6f, 0x79, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x58, 0x69,
0x74, 0x6f, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x58, 0x69, 0x74, 0x6f, 0x79, 0x20,
0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x52,
0x6f, 0x6a, 0x64, 0x65, 0x73, 0x74, 0x76, 0x6f, 0x20, 0x6f, 0x72, 0x6f,
0x6c, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x6f, 0x6b, 0x6f,
0x73, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x79,
0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x6f, 0x6c, 0x75, 0x6d,
0x62, 0x69, 0x79, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x6f, 0x6c, 0x75, 0x6d,
0x62, 0x69, 0x79, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x4b, 0x75, 0x6b, 0x20, 0x6f, 0x72, 0x6f, 0x6c,
0x6c, 0x61, 0x72, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x75,
0x6b, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x4b, 0x75, 0x6b, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72,
0x69, 0x20, 0x79, 0x61, 0x72, 0x69, 0x6d, 0x20, 0x79, 0x6f, 0x7a, 0x67,
0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x75, 0x62, 0x61, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x75, 0x62, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x4b, 0x75, 0x62, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x44, 0x65, 0x79, 0x76, 0x69, 0x73, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x44, 0x79, 0x75, 0x6d, 0x6f, 0x6e, 0x2d, 0x64,
0x2019, 0x59, 0x75, 0x72, 0x76, 0x69, 0x6c, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69, 0x79, 0x20, 0x54, 0x69, 0x6d,
0x6f, 0x72, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x50, 0x61, 0x73, 0x78,
0x61, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x50, 0x61, 0x73, 0x78, 0x61, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x50, 0x61, 0x73, 0x78, 0x61, 0x20, 0x6f, 0x72, 0x6f,
0x6c, 0x69, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x45, 0x6b, 0x76, 0x61, 0x64, 0x6f, 0x72, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x4d, 0x61, 0x72, 0x6b, 0x61, 0x7a, 0x69, 0x79, 0x20,
0x59, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x4d, 0x61, 0x72, 0x6b, 0x61, 0x7a, 0x69, 0x79, 0x20, 0x59, 0x65,
0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x72, 0x6b,
0x61, 0x7a, 0x69, 0x79, 0x20, 0x59, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x53, 0x68, 0x61, 0x72, 0x71, 0x69, 0x79, 0x20, 0x59, 0x65, 0x76, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x68, 0x61,
0x72, 0x71, 0x69, 0x79, 0x20, 0x59, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69, 0x79, 0x20, 0x59,
0x65, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x61, 0x6c, 0x69, 0x6e, 0x69,
0x6e, 0x67, 0x72, 0x61, 0x64, 0x20, 0x76, 0x61, 0x20, 0x4d, 0x69, 0x6e,
0x73, 0x6b, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x2018, 0x61, 0x72,
0x62, 0x69, 0x79, 0x20, 0x59, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x2018, 0x61, 0x72, 0x62, 0x69, 0x79,
0x20, 0x59, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47,
0x2018, 0x61, 0x72, 0x62, 0x69, 0x79, 0x20, 0x59, 0x65, 0x76, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x65, 0x6e, 0x64, 0x20, 0x6f,
0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x65, 0x6e, 0x64, 0x20, 0x6f, 0x72,
0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x46, 0x6f, 0x6c,
0x6b, 0x6c, 0x65, 0x6e, 0x64, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61,
0x72, 0x69, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x46, 0x69, 0x6a, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x46, 0x69, 0x6a, 0x69, 0x20,
0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x46,
0x72, 0x61, 0x6e, 0x73, 0x75, 0x7a, 0x20, 0x47, 0x76, 0x69, 0x61, 0x6e,
0x61, 0x73, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x46, 0x72, 0x61,
0x6e, 0x73, 0x75, 0x7a, 0x20, 0x4a, 0x61, 0x6e, 0x75, 0x62, 0x69, 0x79,
0x20, 0x68, 0x75, 0x64, 0x75, 0x64, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x76,
0x61, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x6b, 0x61,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61,
0x67, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x61, 0x6d,
0x62, 0x79, 0x65, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x72, 0x75,
0x7a, 0x69, 0x79, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x72,
0x75, 0x7a, 0x69, 0x79, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x72, 0x75, 0x7a,
0x69, 0x79, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x20, 0x6f,
0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x47, 0x72, 0x69, 0x6e, 0x76, 0x69, 0x63, 0x68, 0x20, 0x6f, 0x2018,
0x72, 0x74, 0x61, 0x63, 0x68, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x53, 0x68, 0x61, 0x72, 0x71, 0x69, 0x79, 0x20, 0x47, 0x72, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69, 0x79, 0x20, 0x47, 0x72, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53,
0x68, 0x61, 0x72, 0x71, 0x69, 0x79, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x2018, 0x61, 0x72, 0x62, 0x69,
0x79, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79,
0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x2018, 0x61, 0x72, 0x62,
0x69, 0x79, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x79, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x2018, 0x61, 0x72, 0x62, 0x69, 0x79,
0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61,
0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x46, 0x6f, 0x72, 0x73, 0x20, 0x6b, 0x6f, 0x2018, 0x72, 0x66, 0x61, 0x7a,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x47, 0x61, 0x79, 0x61, 0x6e, 0x61, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x47, 0x61, 0x76, 0x61, 0x79, 0x69, 0x2d, 0x61,
0x6c, 0x65, 0x75, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x61,
0x76, 0x61, 0x79, 0x69, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x47, 0x61, 0x76, 0x61, 0x79, 0x69, 0x2d, 0x61, 0x6c, 0x65, 0x75,
0x74, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x47, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e, 0x67, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x47, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e, 0x67, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x47, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e, 0x67, 0x20, 0x79, 0x6f, 0x7a, 0x67,
0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x58, 0x6f, 0x76, 0x64, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x58, 0x6f, 0x76, 0x64, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x58, 0x6f, 0x76, 0x64, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x48, 0x69, 0x6e, 0x64, 0x69, 0x73, 0x74, 0x6f,
0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x48, 0x69, 0x6e, 0x64, 0x20, 0x6f, 0x6b, 0x65,
0x61, 0x6e, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x48, 0x69, 0x6e,
0x64, 0x69, 0x78, 0x69, 0x74, 0x6f, 0x79, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x4d, 0x61, 0x72, 0x6b, 0x61, 0x7a, 0x69, 0x79, 0x20, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x79, 0x61, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69, 0x79, 0x20, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x79, 0x61, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x47, 0x2bb, 0x61, 0x72, 0x62, 0x69, 0x79, 0x20, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x79, 0x61, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x45, 0x72, 0x6f, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x45, 0x72, 0x6f, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x45, 0x72, 0x6f, 0x6e, 0x20,
0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x49,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x49, 0x73, 0x72, 0x6f, 0x69, 0x6c, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x49, 0x73, 0x72, 0x6f, 0x69, 0x6c, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x49, 0x73, 0x72, 0x6f, 0x69, 0x6c, 0x20, 0x79, 0x6f, 0x7a,
0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x59, 0x61, 0x70, 0x6f,
0x6e, 0x69, 0x79, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x59, 0x61,
0x70, 0x6f, 0x6e, 0x69, 0x79, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x59, 0x61, 0x70,
0x6f, 0x6e, 0x69, 0x79, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x51, 0x6f, 0x7a, 0x6f, 0x67, 0x2bb, 0x69,
0x73, 0x74, 0x6f, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x68,
0x61, 0x72, 0x71, 0x69, 0x79, 0x20, 0x51, 0x6f, 0x7a, 0x6f, 0x67, 0x2bb,
0x69, 0x73, 0x74, 0x6f, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47,
0x2bb, 0x61, 0x72, 0x62, 0x69, 0x79, 0x20, 0x51, 0x6f, 0x7a, 0x6f, 0x67,
0x2bb, 0x69, 0x73, 0x74, 0x6f, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x4b, 0x6f, 0x72, 0x65, 0x79, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x4b, 0x6f, 0x72, 0x65, 0x79, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x6f, 0x72,
0x65, 0x79, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x79, 0x6f,
0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x51, 0x69, 0x72,
0x67, 0x2bb, 0x69, 0x7a, 0x69, 0x73, 0x74, 0x6f, 0x6e, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x4c, 0x61, 0x79, 0x6e, 0x20, 0x6f, 0x72, 0x6f, 0x6c,
0x6c, 0x61, 0x72, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4c, 0x6f,
0x72, 0x64, 0x2d, 0x58, 0x61, 0x75, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x58, 0x61, 0x75, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4c,
0x6f, 0x72, 0x64, 0x2d, 0x58, 0x61, 0x75, 0x20, 0x79, 0x6f, 0x7a, 0x67,
0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x4d, 0x61, 0x6c, 0x61, 0x79, 0x7a, 0x69, 0x79, 0x61, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x20, 0x6f,
0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x7a, 0x20, 0x6f, 0x72, 0x6f, 0x6c,
0x6c, 0x61, 0x72, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61,
0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c,
0x61, 0x72, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x76,
0x72, 0x69, 0x6b, 0x69, 0x79, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d,
0x61, 0x76, 0x72, 0x69, 0x6b, 0x69, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61,
0x76, 0x72, 0x69, 0x6b, 0x69, 0x79, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x6f, 0x75, 0x73, 0x6f, 0x6e,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b,
0x61, 0x20, 0x54, 0x69, 0x6e, 0x63, 0x68, 0x20, 0x6f, 0x6b, 0x65, 0x61,
0x6e, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x65, 0x6b, 0x73,
0x69, 0x6b, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x63, 0x68, 0x20, 0x6f, 0x6b,
0x65, 0x61, 0x6e, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x65, 0x6b, 0x73, 0x69,
0x6b, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x63, 0x68, 0x20, 0x6f, 0x6b, 0x65,
0x61, 0x6e, 0x69, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f,
0x72, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x55, 0x6c, 0x61, 0x6e, 0x2d,
0x42, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x55, 0x6c, 0x61, 0x6e,
0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20, 0x79, 0x6f,
0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x79, 0x61,
0x6e, 0x6d, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4e, 0x61, 0x75,
0x72, 0x75, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4e, 0x65, 0x70, 0x61,
0x6c, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x59, 0x61, 0x6e, 0x67, 0x69,
0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x79, 0x61, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x59, 0x61, 0x6e, 0x67, 0x69, 0x20, 0x4b,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x79, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x59, 0x61, 0x6e, 0x67, 0x69, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x69, 0x79, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x59, 0x61, 0x6e, 0x67, 0x69, 0x20, 0x5a, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x59, 0x61, 0x6e, 0x67, 0x69, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x79, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x59, 0x61, 0x6e, 0x67, 0x69,
0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x79,
0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4e, 0x79,
0x75, 0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x4e, 0x79, 0x75, 0x66, 0x61, 0x75, 0x6e, 0x64,
0x6c, 0x65, 0x6e, 0x64, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4e, 0x79, 0x75, 0x66, 0x61,
0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x20, 0x79, 0x6f, 0x7a, 0x67,
0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4e, 0x69, 0x75, 0x79, 0x65,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x6f, 0x72, 0x6f,
0x6c, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x69, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x75, 0x2d, 0x64, 0x69, 0x2d, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x79,
0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x75, 0x2d, 0x64, 0x69, 0x2d, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x79, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x75, 0x2d, 0x64, 0x69, 0x2d, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x79, 0x61,
0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4f,
0x6d, 0x73, 0x6b, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x50, 0x6f, 0x6b, 0x69, 0x73, 0x74, 0x6f, 0x6e, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x50, 0x6f, 0x6b, 0x69, 0x73, 0x74, 0x6f,
0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x50, 0x6f, 0x6b, 0x69, 0x73, 0x74, 0x6f, 0x6e,
0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x50, 0x61, 0x6c, 0x61, 0x75, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x50,
0x61, 0x70, 0x75, 0x61, 0x2d, 0x59, 0x61, 0x6e, 0x67, 0x69, 0x20, 0x47,
0x76, 0x69, 0x6e, 0x65, 0x79, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x79, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x79, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x79, 0x20, 0x79, 0x6f,
0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x50, 0x65, 0x72,
0x75, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x50, 0x65, 0x72, 0x75, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x50, 0x65, 0x72, 0x75, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70,
0x69, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x70, 0x69, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x70, 0x69, 0x6e, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x46, 0x65, 0x6e, 0x69, 0x6b, 0x73, 0x20, 0x6f, 0x72,
0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x53, 0x65, 0x6e, 0x2d, 0x50, 0x79, 0x65, 0x72, 0x20, 0x76, 0x61, 0x20,
0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x53, 0x65, 0x6e, 0x2d, 0x50, 0x79, 0x65, 0x72, 0x20, 0x76, 0x61,
0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53,
0x65, 0x6e, 0x2d, 0x50, 0x79, 0x65, 0x72, 0x20, 0x76, 0x61, 0x20, 0x4d,
0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x50, 0x69, 0x74, 0x6b, 0x65, 0x72,
0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x50, 0x6f, 0x6e, 0x61, 0x70,
0x65, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x50, 0x78, 0x65, 0x6e, 0x79,
0x61, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x52, 0x65, 0x79, 0x75,
0x6e, 0x69, 0x6f, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x52, 0x6f,
0x74, 0x65, 0x72, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x61,
0x78, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53,
0x61, 0x78, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x61, 0x78,
0x61, 0x6c, 0x69, 0x6e, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x53, 0x65, 0x79, 0x73, 0x68, 0x65, 0x6c, 0x20,
0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x20,
0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x4a, 0x61, 0x6e, 0x75, 0x62, 0x69, 0x79, 0x20, 0x47, 0x65,
0x6f, 0x72, 0x67, 0x69, 0x79, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x53, 0x79, 0x6f, 0x76, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x54, 0x61, 0x69, 0x74, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x54,
0x61, 0x79, 0x76, 0x61, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x54,
0x61, 0x79, 0x76, 0x61, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x54, 0x61, 0x79, 0x76,
0x61, 0x6e, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x54, 0x6f, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x6f, 0x6e,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61,
0x75, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67,
0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x43, 0x68, 0x75, 0x75, 0x6b,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x61,
0x6e, 0x69, 0x73, 0x74, 0x6f, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x6f, 0x6e,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x61, 0x6e, 0x69, 0x73,
0x74, 0x6f, 0x6e, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x79, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x79, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x79, 0x20, 0x79, 0x6f,
0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4f, 0x2018, 0x7a,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x6f, 0x6e, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x4f, 0x2018, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x6f,
0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x4f, 0x2018, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x6f, 0x6e, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x79, 0x6f,
0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x56, 0x65, 0x6e,
0x65, 0x73, 0x75, 0x65, 0x6c, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x55, 0x65, 0x79, 0x6b, 0x20, 0x6f,
0x72, 0x6f, 0x6c, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x55, 0x6f,
0x6c, 0x6c, 0x69, 0x73, 0x20, 0x76, 0x61, 0x20, 0x46, 0x75, 0x74, 0x75,
0x6e, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x59, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x59, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x59, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x59, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x627, 0x641, 0x63a, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x648,
0x642, 0x62a, 0x6cc, 0x410, 0x444, 0x493, 0x43e, 0x43d, 0x438, 0x441, 0x442, 0x43e,
0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x430, 0x440, 0x43a, 0x430,
0x437, 0x438, 0x439, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x428, 0x430, 0x440, 0x49b, 0x438, 0x439, 0x20, 0x410,
0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x416,
0x430, 0x43d, 0x443, 0x431, 0x438, 0x439, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a,
0x430, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x492, 0x430, 0x440, 0x431, 0x438,
0x439, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x492, 0x430, 0x440, 0x431, 0x438, 0x439, 0x20, 0x410, 0x444, 0x440,
0x438, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x492, 0x430, 0x440, 0x431, 0x438, 0x439,
0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x451, 0x437, 0x433, 0x438,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x20, 0x43a, 0x443,
0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a, 0x430, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a, 0x430, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a, 0x430, 0x20, 0x451, 0x437,
0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x438, 0x43c, 0x43e,
0x43b, 0x438, 0x439, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x428,
0x438, 0x43c, 0x43e, 0x43b, 0x438, 0x439, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438,
0x43a, 0x430, 0x20, 0x43c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x438, 0x439, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x428, 0x438, 0x43c, 0x43e, 0x43b, 0x438, 0x439, 0x20, 0x410, 0x43c,
0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x43c, 0x430, 0x440, 0x43a, 0x430, 0x437,
0x438, 0x439, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x438, 0x43c, 0x43e, 0x43b, 0x438, 0x439,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x448, 0x430, 0x440,
0x49b, 0x438, 0x439, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x438, 0x43c,
0x43e, 0x43b, 0x438, 0x439, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430,
0x20, 0x448, 0x430, 0x440, 0x49b, 0x438, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x438,
0x43c, 0x43e, 0x43b, 0x438, 0x439, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a,
0x430, 0x20, 0x448, 0x430, 0x440, 0x49b, 0x438, 0x439, 0x20, 0x43a, 0x443, 0x43d,
0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428,
0x438, 0x43c, 0x43e, 0x43b, 0x438, 0x439, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438,
0x43a, 0x430, 0x20, 0x442, 0x43e, 0x493, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x428, 0x438, 0x43c, 0x43e, 0x43b, 0x438, 0x439, 0x20, 0x410, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x20, 0x442, 0x43e, 0x493, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x438,
0x43c, 0x43e, 0x43b, 0x438, 0x439, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a,
0x430, 0x20, 0x442, 0x43e, 0x493, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437,
0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x438, 0x43c, 0x43e,
0x43b, 0x438, 0x439, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20,
0x442, 0x438, 0x43d, 0x447, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x438, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x438, 0x43c, 0x43e, 0x43b, 0x438, 0x439,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x442, 0x438, 0x43d,
0x447, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x438, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428,
0x438, 0x43c, 0x43e, 0x43b, 0x438, 0x439, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438,
0x43a, 0x430, 0x20, 0x442, 0x438, 0x43d, 0x447, 0x20, 0x43e, 0x43a, 0x435, 0x430,
0x43d, 0x438, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x440, 0x430, 0x431, 0x438, 0x441, 0x442,
0x43e, 0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x440, 0x430, 0x431,
0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x440, 0x430, 0x431,
0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437,
0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x440, 0x433, 0x435,
0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410,
0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410,
0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x451, 0x437, 0x433,
0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x492, 0x430, 0x440, 0x431, 0x438,
0x439, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x492, 0x430, 0x440, 0x431, 0x438, 0x439, 0x20,
0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x492, 0x430, 0x440, 0x431, 0x438, 0x439, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d,
0x442, 0x438, 0x43d, 0x430, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x410, 0x440, 0x430, 0x43c, 0x43d, 0x438, 0x441, 0x442, 0x43e,
0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x440, 0x43c, 0x430, 0x43d,
0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x440, 0x43c, 0x430,
0x43d, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438,
0x43a, 0x430, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x442, 0x43b, 0x430,
0x43d, 0x442, 0x438, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x442, 0x43b, 0x430,
0x43d, 0x442, 0x438, 0x43a, 0x430, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437,
0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x430, 0x440, 0x43a,
0x430, 0x437, 0x438, 0x439, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x44f, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x430, 0x440, 0x43a,
0x430, 0x437, 0x438, 0x439, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x438,
0x439, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20,
0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x41c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x438, 0x439, 0x20, 0x410,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x492, 0x430, 0x440,
0x431, 0x438, 0x439, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x430, 0x440,
0x43a, 0x430, 0x437, 0x438, 0x439, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x44f, 0x20, 0x492, 0x430, 0x440, 0x431, 0x438, 0x439, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x41c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x438, 0x439, 0x20, 0x410, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x492, 0x430, 0x440, 0x431,
0x438, 0x439, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x430, 0x440, 0x49b, 0x438, 0x439, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x428, 0x430, 0x440, 0x49b, 0x438, 0x439, 0x20, 0x410, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x430,
0x440, 0x49b, 0x438, 0x439, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x44f, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x492, 0x430, 0x440, 0x431, 0x438, 0x439, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x492, 0x430, 0x440, 0x431, 0x438, 0x439, 0x20, 0x410, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x492, 0x430,
0x440, 0x431, 0x438, 0x439, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x44f, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x41e, 0x437, 0x430, 0x440, 0x431, 0x430, 0x439,
0x436, 0x43e, 0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41e, 0x437, 0x430,
0x440, 0x431, 0x430, 0x439, 0x436, 0x43e, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41e, 0x437,
0x430, 0x440, 0x431, 0x430, 0x439, 0x436, 0x43e, 0x43d, 0x20, 0x451, 0x437, 0x433,
0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x437, 0x43e, 0x440, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x437, 0x43e, 0x440, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x410, 0x437, 0x43e, 0x440, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430,
0x434, 0x435, 0x448, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430,
0x434, 0x435, 0x448, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x411, 0x443, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x411, 0x43e, 0x43b, 0x438, 0x432, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x44f, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x44f,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x44f, 0x20,
0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x411, 0x440,
0x443, 0x43d, 0x435, 0x439, 0x20, 0x414, 0x430, 0x440, 0x443, 0x441, 0x441, 0x430,
0x43b, 0x43e, 0x43c, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a, 0x430, 0x431,
0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435,
0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a,
0x430, 0x43c, 0x43e, 0x440, 0x440, 0x43e, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x427, 0x430, 0x442, 0x445, 0x430, 0x43c, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x427, 0x430, 0x442, 0x445, 0x430, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x427, 0x430, 0x442,
0x445, 0x430, 0x43c, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x427, 0x438, 0x43b, 0x438, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x427, 0x438, 0x43b, 0x438, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x427,
0x438, 0x43b, 0x438, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x425, 0x438, 0x442, 0x43e, 0x439, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x425, 0x438, 0x442, 0x43e, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x425, 0x438, 0x442,
0x43e, 0x439, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x420, 0x43e, 0x436, 0x434, 0x435, 0x441, 0x442,
0x432, 0x43e, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x438, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x20, 0x43e, 0x440, 0x43e, 0x43b,
0x43b, 0x430, 0x440, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a, 0x43e,
0x43b, 0x443, 0x43c, 0x431, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a,
0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x44f, 0x20, 0x451, 0x437, 0x433, 0x438,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a, 0x443, 0x43a, 0x20, 0x43e, 0x440,
0x43e, 0x43b, 0x43b, 0x430, 0x440, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x41a, 0x443, 0x43a, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x43b, 0x430, 0x440, 0x438,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x41a, 0x443, 0x43a, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x43b,
0x430, 0x440, 0x438, 0x20, 0x44f, 0x440, 0x438, 0x43c, 0x20, 0x451, 0x437, 0x433,
0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a, 0x443, 0x431, 0x430, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a, 0x443, 0x431, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x41a, 0x443, 0x431, 0x430, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433,
0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x414, 0x44d, 0x432, 0x438, 0x441,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x414, 0x443, 0x43c, 0x43e, 0x43d, 0x442,
0x2d, 0x434, 0x2d, 0x423, 0x440, 0x432, 0x438, 0x43b, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x428, 0x430, 0x440, 0x49b, 0x438, 0x439, 0x20, 0x422, 0x438, 0x43c,
0x43e, 0x440, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41f, 0x430, 0x441, 0x445,
0x438, 0x20, 0x41e, 0x440, 0x43e, 0x43b, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x41f, 0x430, 0x441, 0x445, 0x438, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x438,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x41f, 0x430, 0x441, 0x445, 0x438, 0x20, 0x43e, 0x440, 0x43e,
0x43b, 0x438, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x42d, 0x43a, 0x432, 0x430, 0x434, 0x43e, 0x440, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x41c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x438, 0x439, 0x20, 0x415,
0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c,
0x430, 0x440, 0x43a, 0x430, 0x437, 0x438, 0x439, 0x20, 0x415, 0x432, 0x440, 0x43e,
0x43f, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x438,
0x439, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x451, 0x437, 0x433,
0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x430, 0x440, 0x49b, 0x438,
0x439, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x428, 0x430, 0x440, 0x49b, 0x438, 0x439, 0x20, 0x415, 0x432, 0x440,
0x43e, 0x43f, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x430, 0x440, 0x49b, 0x438, 0x439,
0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x451, 0x437, 0x433, 0x438,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x492, 0x430, 0x440, 0x431, 0x438, 0x439,
0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x492, 0x430, 0x440, 0x431, 0x438, 0x439, 0x20, 0x415, 0x432, 0x440, 0x43e,
0x43f, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x492, 0x430, 0x440, 0x431, 0x438, 0x439, 0x20,
0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x424, 0x43e, 0x43b, 0x44c, 0x43a, 0x43b, 0x435,
0x43d, 0x434, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x43b, 0x430, 0x440, 0x438, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x424, 0x43e, 0x43b, 0x44c, 0x43a, 0x43b, 0x435,
0x43d, 0x434, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x43b, 0x430, 0x440, 0x438, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x424, 0x43e, 0x43b, 0x44c, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x20,
0x43e, 0x440, 0x43e, 0x43b, 0x43b, 0x430, 0x440, 0x438, 0x20, 0x451, 0x437, 0x433,
0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x424, 0x438, 0x436, 0x438, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x424, 0x438, 0x436, 0x438, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x424, 0x438, 0x436, 0x438, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x443, 0x437, 0x20, 0x413,
0x432, 0x438, 0x430, 0x43d, 0x430, 0x441, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x443, 0x437, 0x20, 0x436, 0x430, 0x43d,
0x443, 0x431, 0x438, 0x439, 0x20, 0x432, 0x430, 0x20, 0x410, 0x43d, 0x442, 0x430,
0x440, 0x43a, 0x442, 0x438, 0x43a, 0x430, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x413, 0x430, 0x43c, 0x431, 0x438, 0x435, 0x440, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x413, 0x440, 0x443, 0x437, 0x438, 0x44f, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x413, 0x440, 0x443, 0x437, 0x438, 0x44f, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x413, 0x440, 0x443, 0x437, 0x438, 0x44f, 0x20, 0x451, 0x437, 0x433, 0x438,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x413, 0x438, 0x43b, 0x431, 0x435, 0x440,
0x442, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x43b, 0x430, 0x440, 0x438, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x413, 0x440, 0x438, 0x43d, 0x432, 0x438, 0x447, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x430, 0x440, 0x49b, 0x438, 0x439, 0x20,
0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x428, 0x430, 0x440, 0x49b, 0x438, 0x439, 0x20, 0x413,
0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x428, 0x430, 0x440, 0x49b, 0x438, 0x439, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b,
0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x492, 0x430, 0x440, 0x431, 0x438, 0x439, 0x20, 0x413,
0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x492, 0x430, 0x440, 0x431, 0x438, 0x439, 0x20, 0x413, 0x440,
0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x492,
0x430, 0x440, 0x431, 0x438, 0x439, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430,
0x43d, 0x434, 0x438, 0x44f, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x41a, 0x45e, 0x440, 0x444, 0x430, 0x437, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x413, 0x430, 0x439, 0x430, 0x43d, 0x430, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x413, 0x430, 0x432, 0x430, 0x439, 0x438, 0x2d, 0x430, 0x43b,
0x435, 0x443, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x413, 0x430, 0x432,
0x430, 0x439, 0x438, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x413, 0x430, 0x432, 0x430, 0x439, 0x438, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442,
0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x20, 0x451, 0x437,
0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x425, 0x43e, 0x432, 0x434,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x425, 0x43e, 0x432, 0x434, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x425, 0x43e, 0x432, 0x434, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x4b2, 0x438, 0x43d, 0x434, 0x438, 0x441, 0x442, 0x43e,
0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x4b2, 0x438, 0x43d, 0x434, 0x20,
0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x4b2, 0x438, 0x43d, 0x434, 0x2d, 0x425, 0x438, 0x442, 0x43e, 0x439, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x41c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x438, 0x439,
0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x44f, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x428, 0x430, 0x440, 0x49b, 0x438, 0x439, 0x20, 0x418,
0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x492, 0x430, 0x440, 0x431, 0x438, 0x439, 0x20, 0x418, 0x43d, 0x434,
0x43e, 0x43d, 0x435, 0x437, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x42d, 0x440, 0x43e, 0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x42d, 0x440,
0x43e, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x42d, 0x440, 0x43e, 0x43d, 0x20, 0x43a, 0x443,
0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x418,
0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x418, 0x441, 0x440, 0x43e, 0x438, 0x43b, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x418, 0x441, 0x440, 0x43e, 0x438, 0x43b, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x418, 0x441, 0x440, 0x43e, 0x438, 0x43b, 0x20, 0x43a, 0x443, 0x43d,
0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x42f,
0x43f, 0x43e, 0x43d, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x42f,
0x43f, 0x43e, 0x43d, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x42f, 0x43f, 0x43e, 0x43d,
0x438, 0x44f, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x430, 0x440, 0x49b, 0x438, 0x439, 0x20,
0x49a, 0x43e, 0x437, 0x43e, 0x493, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x492, 0x430, 0x440, 0x431, 0x438, 0x439, 0x20, 0x49a,
0x43e, 0x437, 0x43e, 0x493, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x41a, 0x43e, 0x440, 0x435, 0x44f, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x41a, 0x43e, 0x440, 0x435, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a, 0x43e,
0x440, 0x435, 0x44f, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a, 0x43e, 0x441, 0x440, 0x430, 0x435,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e,
0x44f, 0x440, 0x441, 0x43a, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a, 0x440,
0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a,
0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x20, 0x451, 0x437,
0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x49a, 0x438, 0x440, 0x493,
0x438, 0x437, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x41b, 0x430, 0x439, 0x43d, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x43b, 0x430,
0x440, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41b, 0x43e, 0x440, 0x434,
0x20, 0x425, 0x43e, 0x432, 0x435, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41b,
0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x432, 0x435, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41b,
0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x432, 0x435, 0x20, 0x43a, 0x443, 0x43d,
0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c,
0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x430,
0x433, 0x430, 0x434, 0x430, 0x43d, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x41c, 0x430, 0x43b, 0x430, 0x439, 0x437, 0x438, 0x44f,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x430, 0x43b, 0x44c, 0x434, 0x438,
0x432, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x43b, 0x430, 0x440, 0x41c, 0x430, 0x440,
0x43a, 0x435, 0x437, 0x430, 0x441, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c,
0x430, 0x440, 0x448, 0x430, 0x43b, 0x43b, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x43b,
0x430, 0x440, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x430, 0x432,
0x440, 0x438, 0x43a, 0x438, 0x439, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c,
0x430, 0x432, 0x440, 0x438, 0x43a, 0x438, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x430,
0x432, 0x440, 0x438, 0x43a, 0x438, 0x439, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x43e, 0x443, 0x432, 0x441, 0x43e, 0x43d,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411,
0x430, 0x442, 0x43e, 0x440, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x423, 0x43b,
0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x423,
0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x20, 0x451, 0x437,
0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x43e, 0x441, 0x43a,
0x432, 0x430, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x43e, 0x441, 0x43a,
0x432, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x20,
0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x44c,
0x44f, 0x43d, 0x43c, 0x430, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41d, 0x430,
0x443, 0x440, 0x443, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41d, 0x435, 0x43f,
0x430, 0x43b, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x42f, 0x43d, 0x433, 0x438,
0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x44f, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x42f, 0x43d, 0x433, 0x438, 0x20, 0x41a, 0x430, 0x43b,
0x435, 0x434, 0x43e, 0x43d, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x42f, 0x43d, 0x433,
0x438, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x44f, 0x20,
0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x42f, 0x43d,
0x433, 0x438, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x42f, 0x43d, 0x433, 0x438, 0x20, 0x417, 0x435,
0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x42f, 0x43d, 0x433,
0x438, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x43a,
0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d,
0x434, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41d, 0x44c, 0x44e, 0x444, 0x430,
0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41d, 0x44c,
0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x43a,
0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x41d, 0x438, 0x443, 0x435, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41d,
0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x438,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b,
0x43a, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41d, 0x43e,
0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x438, 0x20,
0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x424, 0x435,
0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d, 0x43e,
0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x424,
0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d,
0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x424, 0x435, 0x440,
0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440,
0x43e, 0x43d, 0x44c, 0x44f, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440,
0x441, 0x43a, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41d, 0x43e, 0x432, 0x43e,
0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41d, 0x43e,
0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x20, 0x451, 0x437,
0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41e, 0x43c, 0x441, 0x43a,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x41e, 0x43c, 0x441, 0x43a, 0x20, 0x451, 0x437, 0x433, 0x438,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41f, 0x43e, 0x43a, 0x438, 0x441, 0x442,
0x43e, 0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41f, 0x43e, 0x43a, 0x438,
0x441, 0x442, 0x43e, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41f, 0x43e, 0x43a, 0x438, 0x441,
0x442, 0x43e, 0x43d, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x41f, 0x430, 0x43b, 0x430, 0x443, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x41f, 0x430, 0x43f, 0x443, 0x430, 0x2d, 0x42f, 0x43d, 0x433, 0x438, 0x20,
0x413, 0x432, 0x438, 0x43d, 0x435, 0x44f, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439, 0x20, 0x451, 0x437,
0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41f, 0x435, 0x440, 0x443,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41f, 0x435, 0x440, 0x443, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x41f, 0x435, 0x440, 0x443, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f,
0x438, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438,
0x43d, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x424, 0x435, 0x43d, 0x438, 0x43a, 0x441, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x43b,
0x430, 0x440, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x421, 0x435, 0x43d,
0x442, 0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20, 0x432, 0x430, 0x20, 0x41c, 0x438,
0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x421,
0x435, 0x43d, 0x442, 0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20, 0x432, 0x430, 0x20,
0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x421, 0x435,
0x43d, 0x442, 0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20, 0x432, 0x430, 0x20, 0x41c,
0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443,
0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41f, 0x438, 0x442,
0x43a, 0x435, 0x440, 0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41f, 0x43e,
0x43d, 0x430, 0x43f, 0x435, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x420, 0x435,
0x44e, 0x43d, 0x44c, 0x43e, 0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x420,
0x43e, 0x442, 0x435, 0x440, 0x430, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x421,
0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x421, 0x430,
0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x421,
0x430, 0x43c, 0x43e, 0x430, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433,
0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x421, 0x435, 0x439, 0x448, 0x435,
0x43b, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x43b, 0x430, 0x440, 0x438, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x421, 0x438, 0x43d, 0x433, 0x430, 0x43f, 0x443, 0x440,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x421, 0x43e, 0x43b, 0x43e, 0x43c, 0x43e,
0x43d, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x43b, 0x430, 0x440, 0x438, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x416, 0x430, 0x43d, 0x443, 0x431, 0x438, 0x439, 0x20,
0x414, 0x436, 0x43e, 0x440, 0x434, 0x436, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x421, 0x443, 0x440, 0x438, 0x43d, 0x430, 0x43c, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x421, 0x44c, 0x43e, 0x432, 0x430, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x422, 0x430, 0x438, 0x442, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x422, 0x430, 0x439, 0x43f, 0x435, 0x439, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x422, 0x430, 0x439, 0x43f, 0x435, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x422, 0x430,
0x439, 0x43f, 0x435, 0x439, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433,
0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x422, 0x43e, 0x436, 0x438, 0x43a,
0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x422,
0x43e, 0x43a, 0x435, 0x43b, 0x430, 0x443, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x422,
0x43e, 0x43d, 0x433, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x422, 0x43e, 0x43d, 0x433, 0x430,
0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x427,
0x443, 0x443, 0x43a, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x422, 0x443, 0x440,
0x43a, 0x43c, 0x430, 0x43d, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x430, 0x43d, 0x438, 0x441,
0x442, 0x43e, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x430,
0x43d, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x422, 0x443, 0x432, 0x430, 0x43b, 0x443, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430,
0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x20,
0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x40e, 0x437,
0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x40e, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x43e, 0x43d,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x40e, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x43e,
0x43d, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x412,
0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x412, 0x435, 0x43d, 0x435, 0x441, 0x443, 0x44d,
0x43b, 0x430, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x412, 0x43b, 0x430, 0x434,
0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442,
0x43e, 0x43a, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430,
0x434, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430,
0x434, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x412, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x423, 0x44d, 0x439, 0x43a, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x438, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x423, 0x44d, 0x43b, 0x43b, 0x438, 0x441, 0x20, 0x432,
0x430, 0x20, 0x424, 0x443, 0x442, 0x443, 0x43d, 0x430, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x42f,
0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d,
0x431, 0x443, 0x440, 0x433, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x415, 0x43a,
0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440,
0x433, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x47, 0x69, 0x1edd, 0x20, 0x41, 0x63, 0x72, 0x65, 0x47, 0x69, 0x1edd, 0x20,
0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x41, 0x63, 0x72, 0x65, 0x47, 0x69,
0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x41, 0x63, 0x72,
0x65, 0x47, 0x69, 0x1edd, 0x20, 0x41, 0x66, 0x67, 0x68, 0x61, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x54, 0x72, 0x75, 0x6e,
0x67, 0x20, 0x50, 0x68, 0x69, 0x47, 0x69, 0x1edd, 0x20, 0x110, 0xf4, 0x6e,
0x67, 0x20, 0x50, 0x68, 0x69, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75,
0x1ea9, 0x6e, 0x20, 0x4e, 0x61, 0x6d, 0x20, 0x50, 0x68, 0x69, 0x47, 0x69,
0x1edd, 0x20, 0x54, 0xe2, 0x79, 0x20, 0x50, 0x68, 0x69, 0x47, 0x69, 0x1edd,
0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x54, 0xe2, 0x79, 0x20, 0x50,
0x68, 0x69, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8,
0x20, 0x54, 0xe2, 0x79, 0x20, 0x50, 0x68, 0x69, 0x47, 0x69, 0x1edd, 0x20,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68,
0x75, 0x1ea9, 0x6e, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x47, 0x69,
0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74,
0x79, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x41,
0x6c, 0x6d, 0x61, 0x74, 0x79, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61,
0x20, 0x48, 0xe8, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x47, 0x69,
0x1edd, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x47, 0x69, 0x1edd, 0x20,
0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x41,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0x69, 0x1ec1,
0x6e, 0x20, 0x54, 0x72, 0x75, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x63,
0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x6d, 0x69, 0x1ec1, 0x6e, 0x20, 0x54, 0x72,
0x75, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68,
0xe8, 0x20, 0x6d, 0x69, 0x1ec1, 0x6e, 0x20, 0x54, 0x72, 0x75, 0x6e, 0x67,
0x47, 0x69, 0x1edd, 0x20, 0x6d, 0x69, 0x1ec1, 0x6e, 0x20, 0x110, 0xf4, 0x6e,
0x67, 0x47, 0x69, 0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x6d,
0x69, 0x1ec1, 0x6e, 0x20, 0x110, 0xf4, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20,
0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x6d, 0x69, 0x1ec1, 0x6e, 0x20,
0x110, 0xf4, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0x69, 0x1ec1, 0x6e,
0x20, 0x6e, 0xfa, 0x69, 0x47, 0x69, 0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9,
0x6e, 0x20, 0x6d, 0x69, 0x1ec1, 0x6e, 0x20, 0x6e, 0xfa, 0x69, 0x47, 0x69,
0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x6d, 0x69, 0x1ec1,
0x6e, 0x20, 0x6e, 0xfa, 0x69, 0x47, 0x69, 0x1edd, 0x20, 0x54, 0x68, 0xe1,
0x69, 0x20, 0x42, 0xec, 0x6e, 0x68, 0x20, 0x44, 0x1b0, 0x1a1, 0x6e, 0x67,
0x47, 0x69, 0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x54, 0x68,
0xe1, 0x69, 0x20, 0x42, 0xec, 0x6e, 0x68, 0x20, 0x44, 0x1b0, 0x1a1, 0x6e,
0x67, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20,
0x54, 0x68, 0xe1, 0x69, 0x20, 0x42, 0xec, 0x6e, 0x68, 0x20, 0x44, 0x1b0,
0x1a1, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79,
0x72, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x41,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61,
0x20, 0x68, 0xe8, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x47, 0x69,
0x1edd, 0x20, 0x41, 0x70, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68,
0x75, 0x1ea9, 0x6e, 0x20, 0x41, 0x70, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20,
0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x41, 0x70, 0x69, 0x61, 0x47,
0x69, 0x1edd, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x47, 0x69, 0x1edd, 0x20,
0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x47,
0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x41, 0x71,
0x74, 0x61, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62,
0x65, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x41,
0x71, 0x74, 0x6f, 0x62, 0x65, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61,
0x20, 0x48, 0xe8, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x47, 0x69,
0x1edd, 0x20, 0x1ea2, 0x20, 0x52, 0x1ead, 0x70, 0x47, 0x69, 0x1edd, 0x20, 0x63,
0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x1ea2, 0x20, 0x52, 0x1ead, 0x70, 0x47, 0x69,
0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x1ea2, 0x20, 0x52,
0x1ead, 0x70, 0x47, 0x69, 0x1edd, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e,
0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x47, 0x69,
0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0x69,
0x1ec1, 0x6e, 0x20, 0x74, 0xe2, 0x79, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9,
0x6e, 0x20, 0x6d, 0x69, 0x1ec1, 0x6e, 0x20, 0x74, 0xe2, 0x79, 0x20, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x47, 0x69, 0x1edd, 0x20,
0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x6d, 0x69, 0x1ec1, 0x6e, 0x20,
0x74, 0xe2, 0x79, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x47, 0x69, 0x1edd, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61,
0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x41, 0x72,
0x6d, 0x65, 0x6e, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61,
0x20, 0x48, 0xe8, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x47,
0x69, 0x1edd, 0x20, 0x110, 0x1ea1, 0x69, 0x20, 0x54, 0xe2, 0x79, 0x20, 0x44,
0x1b0, 0x1a1, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9,
0x6e, 0x20, 0x110, 0x1ea1, 0x69, 0x20, 0x54, 0xe2, 0x79, 0x20, 0x44, 0x1b0,
0x1a1, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68,
0xe8, 0x20, 0x110, 0x1ea1, 0x69, 0x20, 0x54, 0xe2, 0x79, 0x20, 0x44, 0x1b0,
0x1a1, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20,
0x54, 0x72, 0x75, 0x6e, 0x67, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e,
0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x54, 0x72, 0x75, 0x6e, 0x67, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x47, 0x69, 0x1edd,
0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e,
0x20, 0x54, 0x72, 0x75, 0x6e, 0x67, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e,
0x20, 0x54, 0x72, 0x75, 0x6e, 0x67, 0x20, 0x54, 0xe2, 0x79, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20,
0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x54,
0x72, 0x75, 0x6e, 0x67, 0x20, 0x54, 0xe2, 0x79, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9,
0x61, 0x20, 0x48, 0xe8, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x54, 0x72,
0x75, 0x6e, 0x67, 0x20, 0x54, 0xe2, 0x79, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x69, 0x1ec1,
0x6e, 0x20, 0x110, 0xf4, 0x6e, 0x67, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9,
0x6e, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x110, 0xf4, 0x6e, 0x67, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x47, 0x69, 0x1edd,
0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e,
0x20, 0x110, 0xf4, 0x6e, 0x67, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20,
0x54, 0xe2, 0x79, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4d,
0x69, 0x1ec1, 0x6e, 0x20, 0x54, 0xe2, 0x79, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61,
0x20, 0x48, 0xe8, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x54, 0xe2, 0x79,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x47, 0x69,
0x1edd, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e,
0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20,
0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62,
0x61, 0x69, 0x6a, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x73, 0x47, 0x69, 0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9, 0x6e,
0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x47, 0x69, 0x1edd, 0x20, 0x6d,
0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73,
0x47, 0x69, 0x1edd, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x73, 0x68, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x47, 0x69,
0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x47, 0x69, 0x1edd, 0x20, 0x42,
0x68, 0x75, 0x74, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x42, 0x6f, 0x6c,
0x69, 0x76, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x42, 0x72, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9,
0x6e, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x47, 0x69,
0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x42, 0x72, 0x75,
0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c,
0x61, 0x6d, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56,
0x65, 0x72, 0x64, 0x65, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9,
0x6e, 0x20, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65,
0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x43,
0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x47, 0x69, 0x1edd,
0x20, 0x43, 0x61, 0x73, 0x65, 0x79, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68,
0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75,
0x1ea9, 0x6e, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x47, 0x69,
0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x43, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x69, 0x6c,
0x65, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x43,
0x68, 0x69, 0x6c, 0x65, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20,
0x48, 0xe8, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x47, 0x69, 0x1edd, 0x20,
0x54, 0x72, 0x75, 0x6e, 0x67, 0x20, 0x51, 0x75, 0x1ed1, 0x63, 0x47, 0x69,
0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x54, 0x72, 0x75, 0x6e,
0x67, 0x20, 0x51, 0x75, 0x1ed1, 0x63, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9,
0x61, 0x20, 0x48, 0xe8, 0x20, 0x54, 0x72, 0x75, 0x6e, 0x67, 0x20, 0x51,
0x75, 0x1ed1, 0x63, 0x47, 0x69, 0x1edd, 0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x43,
0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x47, 0x69, 0x1edd, 0x20,
0x51, 0x75, 0x1ea7, 0x6e, 0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x43, 0x6f, 0x63,
0x6f, 0x73, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20,
0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20,
0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d,
0x62, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x51, 0x75, 0x1ea7, 0x6e, 0x20,
0x110, 0x1ea3, 0x6f, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x47, 0x69, 0x1edd, 0x20,
0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x51, 0x75, 0x1ea7, 0x6e, 0x20, 0x110,
0x1ea3, 0x6f, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x4e,
0x1eed, 0x61, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x51, 0x75,
0x1ea7, 0x6e, 0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x47,
0x69, 0x1edd, 0x20, 0x43, 0x75, 0x62, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43,
0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x43, 0x75, 0x62, 0x61, 0x47, 0x69, 0x1edd,
0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x43, 0x75, 0x62, 0x61,
0x47, 0x69, 0x1edd, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x47, 0x69, 0x1edd,
0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72,
0x76, 0x69, 0x6c, 0x6c, 0x65, 0x47, 0x69, 0x1edd, 0x20, 0x110, 0xf4, 0x6e,
0x67, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x47, 0x69, 0x1edd, 0x20, 0x110,
0x1ea3, 0x6f, 0x20, 0x50, 0x68, 0x1ee5, 0x63, 0x20, 0x53, 0x69, 0x6e, 0x68,
0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x110, 0x1ea3,
0x6f, 0x20, 0x50, 0x68, 0x1ee5, 0x63, 0x20, 0x53, 0x69, 0x6e, 0x68, 0x47,
0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x110, 0x1ea3,
0x6f, 0x20, 0x50, 0x68, 0x1ee5, 0x63, 0x20, 0x53, 0x69, 0x6e, 0x68, 0x47,
0x69, 0x1edd, 0x20, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x47, 0x69,
0x1edd, 0x20, 0x54, 0x72, 0x75, 0x6e, 0x67, 0x20, 0xc2, 0x75, 0x47, 0x69,
0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x54, 0x72, 0x75, 0x6e,
0x67, 0x20, 0xc2, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20,
0x68, 0xe8, 0x20, 0x54, 0x72, 0x75, 0x6e, 0x67, 0x20, 0xc2, 0x75, 0x47,
0x69, 0x1edd, 0x20, 0x110, 0xf4, 0x6e, 0x67, 0x20, 0xc2, 0x75, 0x47, 0x69,
0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x110, 0xf4, 0x6e, 0x67,
0x20, 0xc2, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68,
0xe8, 0x20, 0x110, 0xf4, 0x6e, 0x67, 0x20, 0xc2, 0x75, 0x47, 0x69, 0x1edd,
0x20, 0x56, 0x69, 0x1ec5, 0x6e, 0x20, 0x111, 0xf4, 0x6e, 0x67, 0x20, 0x43,
0x68, 0xe2, 0x75, 0x20, 0xc2, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x54, 0xe2,
0x79, 0x20, 0xc2, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9,
0x6e, 0x20, 0x54, 0xe2, 0x79, 0x20, 0xc2, 0x75, 0x47, 0x69, 0x1edd, 0x20,
0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x54, 0xe2, 0x79, 0x20, 0xc2,
0x75, 0x47, 0x69, 0x1edd, 0x20, 0x51, 0x75, 0x1ea7, 0x6e, 0x20, 0x110, 0x1ea3,
0x6f, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x47, 0x69,
0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x51, 0x75, 0x1ea7, 0x6e,
0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20,
0x51, 0x75, 0x1ea7, 0x6e, 0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x46, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x47, 0x69, 0x1edd, 0x20, 0x46, 0x69, 0x6a,
0x69, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x46,
0x69, 0x6a, 0x69, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48,
0xe8, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x47, 0x69, 0x1edd, 0x20, 0x47, 0x75,
0x69, 0x61, 0x6e, 0x61, 0x20, 0x74, 0x68, 0x75, 0x1ed9, 0x63, 0x20, 0x50,
0x68, 0xe1, 0x70, 0x47, 0x69, 0x1edd, 0x20, 0x4e, 0x61, 0x6d, 0x20, 0x43,
0x1ef1, 0x63, 0x20, 0x76, 0xe0, 0x20, 0x4e, 0x61, 0x6d, 0x20, 0x4e, 0x1b0,
0x1edb, 0x63, 0x20, 0x50, 0x68, 0xe1, 0x70, 0x47, 0x69, 0x1edd, 0x20, 0x47,
0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x47, 0x69, 0x1edd, 0x20,
0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x47, 0x69, 0x1edd, 0x20, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68,
0x75, 0x1ea9, 0x6e, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x47,
0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x47, 0x65,
0x6f, 0x72, 0x67, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x51, 0x75, 0x1ea7,
0x6e, 0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72,
0x74, 0x47, 0x69, 0x1edd, 0x20, 0x54, 0x72, 0x75, 0x6e, 0x67, 0x20, 0x62,
0xec, 0x6e, 0x68, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63,
0x68, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x110, 0xf4,
0x6e, 0x67, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4d, 0x69,
0x1ec1, 0x6e, 0x20, 0x110, 0xf4, 0x6e, 0x67, 0x20, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61,
0x20, 0x48, 0xe8, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x110, 0xf4, 0x6e,
0x67, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x47,
0x69, 0x1edd, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x54, 0xe2, 0x79, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x47, 0x69, 0x1edd,
0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20,
0x54, 0xe2, 0x79, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20,
0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x54, 0xe2, 0x79, 0x20, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68,
0x75, 0x1ea9, 0x6e, 0x20, 0x47, 0x75, 0x61, 0x6d, 0x47, 0x69, 0x1edd, 0x20,
0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x56, 0xf9, 0x6e, 0x67, 0x20, 0x56,
0x1ecb, 0x6e, 0x68, 0x47, 0x69, 0x1edd, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e,
0x61, 0x47, 0x69, 0x1edd, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75,
0x1ea9, 0x6e, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48,
0xe8, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65,
0x75, 0x74, 0x47, 0x69, 0x1edd, 0x20, 0x48, 0x1ed3, 0x6e, 0x67, 0x20, 0x4b,
0xf4, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e,
0x20, 0x48, 0x1ed3, 0x6e, 0x67, 0x20, 0x4b, 0xf4, 0x6e, 0x67, 0x47, 0x69,
0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x48, 0x1ed3, 0x6e,
0x67, 0x20, 0x4b, 0xf4, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x48, 0x6f,
0x76, 0x64, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20,
0x48, 0x6f, 0x76, 0x64, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20,
0x48, 0xe8, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x47, 0x69, 0x1edd, 0x20, 0x43,
0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x1ea4, 0x6e, 0x20, 0x110, 0x1ed9, 0x47, 0x69,
0x1edd, 0x20, 0x1ea4, 0x6e, 0x20, 0x110, 0x1ed9, 0x20, 0x44, 0x1b0, 0x1a1, 0x6e,
0x67, 0x47, 0x69, 0x1edd, 0x20, 0x110, 0xf4, 0x6e, 0x67, 0x20, 0x44, 0x1b0,
0x1a1, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20,
0x54, 0x72, 0x75, 0x6e, 0x67, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x73, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20,
0x110, 0xf4, 0x6e, 0x67, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73,
0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x54,
0xe2, 0x79, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61,
0x47, 0x69, 0x1edd, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20,
0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x47, 0x69,
0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x49, 0x72, 0x61,
0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61,
0x20, 0x48, 0xe8, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x47,
0x69, 0x1edd, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x47, 0x69, 0x1edd,
0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20,
0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x47, 0x69, 0x1edd, 0x20, 0x4e, 0x68,
0x1ead, 0x74, 0x20, 0x42, 0x1ea3, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68,
0x75, 0x1ea9, 0x6e, 0x20, 0x4e, 0x68, 0x1ead, 0x74, 0x20, 0x42, 0x1ea3, 0x6e,
0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x4e,
0x68, 0x1ead, 0x74, 0x20, 0x42, 0x1ea3, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x50,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x47,
0x69, 0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x50, 0x65, 0x74,
0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b,
0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x47, 0x69, 0x1edd,
0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x50, 0x65, 0x74, 0x72,
0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61,
0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x47, 0x69, 0x1edd, 0x20,
0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x47, 0x69,
0x1edd, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x110, 0xf4, 0x6e, 0x67, 0x20,
0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x47, 0x69,
0x1edd, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x54, 0xe2, 0x79, 0x20, 0x4b,
0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x47, 0x69, 0x1edd,
0x20, 0x48, 0xe0, 0x6e, 0x20, 0x51, 0x75, 0x1ed1, 0x63, 0x47, 0x69, 0x1edd,
0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x48, 0xe0, 0x6e, 0x20, 0x51,
0x75, 0x1ed1, 0x63, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48,
0xe8, 0x20, 0x48, 0xe0, 0x6e, 0x20, 0x51, 0x75, 0x1ed1, 0x63, 0x47, 0x69,
0x1edd, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x47, 0x69, 0x1edd, 0x20,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x47,
0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x47, 0x69, 0x1edd, 0x20,
0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x4b, 0x79,
0x72, 0x67, 0x79, 0x73, 0x74, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x4c,
0x61, 0x6e, 0x6b, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x51, 0x75, 0x1ea7, 0x6e,
0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x4c, 0x69, 0x6e, 0x65, 0x47, 0x69, 0x1edd,
0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x47, 0x69,
0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61,
0x20, 0x48, 0xe8, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x61, 0x20, 0x43, 0x61, 0x6f, 0x47,
0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4d, 0x61, 0x20,
0x43, 0x61, 0x6f, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48,
0xe8, 0x20, 0x4d, 0x61, 0x20, 0x43, 0x61, 0x6f, 0x47, 0x69, 0x1edd, 0x20,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x43,
0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x61,
0x6c, 0x61, 0x79, 0x73, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x61,
0x6c, 0x64, 0x69, 0x76, 0x65, 0x73, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x61,
0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x47, 0x69, 0x1edd, 0x20, 0x51,
0x75, 0x1ea7, 0x6e, 0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x4d, 0x61, 0x72, 0x73,
0x68, 0x61, 0x6c, 0x6c, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x74, 0x69, 0x75, 0x73, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75,
0x1ea9, 0x6e, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73,
0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x47, 0x69, 0x1edd, 0x20,
0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x54, 0x68,
0xe1, 0x69, 0x20, 0x42, 0xec, 0x6e, 0x68, 0x20, 0x44, 0x1b0, 0x1a1, 0x6e,
0x67, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f, 0x47, 0x69, 0x1edd, 0x20,
0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x54, 0x68, 0xe1, 0x69, 0x20, 0x42,
0xec, 0x6e, 0x68, 0x20, 0x44, 0x1b0, 0x1a1, 0x6e, 0x67, 0x20, 0x4d, 0x65,
0x78, 0x69, 0x63, 0x6f, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20,
0x48, 0xe8, 0x20, 0x54, 0x68, 0xe1, 0x69, 0x20, 0x42, 0xec, 0x6e, 0x68,
0x20, 0x44, 0x1b0, 0x1a1, 0x6e, 0x67, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63,
0x6f, 0x47, 0x69, 0x1edd, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x47, 0x69, 0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9, 0x6e,
0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x47,
0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x55, 0x6c,
0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x47, 0x69, 0x1edd, 0x20,
0x4d, 0x61, 0x74, 0x78, 0x63, 0x1a1, 0x76, 0x61, 0x47, 0x69, 0x1edd, 0x20,
0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4d, 0x61, 0x74, 0x78, 0x63, 0x1a1,
0x76, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8,
0x20, 0x4d, 0x61, 0x74, 0x78, 0x63, 0x1a1, 0x76, 0x61, 0x47, 0x69, 0x1edd,
0x20, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x47, 0x69, 0x1edd, 0x20,
0x4e, 0x61, 0x75, 0x72, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x4e, 0x65, 0x70,
0x61, 0x6c, 0x47, 0x69, 0x1edd, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43,
0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9,
0x61, 0x20, 0x48, 0xe8, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4e, 0x65,
0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x47, 0x69, 0x1edd,
0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x5a,
0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9,
0x61, 0x20, 0x48, 0xe8, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61,
0x6c, 0x61, 0x6e, 0x64, 0x47, 0x69, 0x1edd, 0x20, 0x4e, 0x65, 0x77, 0x66,
0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x47, 0x69, 0x1edd, 0x20,
0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75,
0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9,
0x61, 0x20, 0x48, 0xe8, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e,
0x64, 0x6c, 0x61, 0x6e, 0x64, 0x47, 0x69, 0x1edd, 0x20, 0x4e, 0x69, 0x75,
0x65, 0x47, 0x69, 0x1edd, 0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9,
0x6e, 0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20,
0x110, 0x1ea3, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x47,
0x69, 0x1edd, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x47, 0x69,
0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48,
0xe8, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x47, 0x69, 0x1edd,
0x20, 0x51, 0x75, 0x1ea7, 0x6e, 0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x42, 0x1eaf,
0x63, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x6e, 0x61, 0x47, 0x69, 0x1edd,
0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x47, 0x69, 0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x47, 0x69, 0x1edd,
0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x4f,
0x6d, 0x73, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9, 0x6e,
0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61,
0x20, 0x68, 0xe8, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x47, 0x69, 0x1edd, 0x20,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20,
0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8,
0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x47, 0x69, 0x1edd,
0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x50, 0x61,
0x70, 0x75, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x47, 0x75, 0x69, 0x6e,
0x65, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x47, 0x69, 0x1edd, 0x20,
0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x79, 0x47, 0x69, 0x1edd, 0x20, 0x50, 0x65, 0x72, 0x75, 0x47,
0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x50, 0x65, 0x72,
0x75, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20,
0x50, 0x65, 0x72, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x50, 0x68, 0x69, 0x6c,
0x69, 0x70, 0x70, 0x69, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75,
0x1ea9, 0x6e, 0x20, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e,
0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x50,
0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x47, 0x69, 0x1edd, 0x20,
0x51, 0x75, 0x1ea7, 0x6e, 0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x50, 0x68, 0x6f,
0x65, 0x6e, 0x69, 0x78, 0x47, 0x69, 0x1edd, 0x20, 0x53, 0x74, 0x2e, 0x20,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x76, 0xe0, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68,
0x75, 0x1ea9, 0x6e, 0x20, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x20, 0x76, 0xe0, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8,
0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x20, 0x76, 0xe0, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f,
0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72,
0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x47,
0x69, 0x1edd, 0x20, 0x42, 0xec, 0x6e, 0x68, 0x20, 0x4e, 0x68, 0x1b0, 0x1ee1,
0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f,
0x72, 0x64, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e,
0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x47, 0x69,
0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x51, 0x79, 0x7a,
0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x52, 0x65,
0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x52, 0x6f, 0x74,
0x68, 0x65, 0x72, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9,
0x6e, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x47, 0x69,
0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x53, 0x61, 0x6b,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x53, 0x61, 0x6d,
0x61, 0x72, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e,
0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x6d,
0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x47, 0x69, 0x1edd, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x47, 0x69, 0x1edd,
0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x47, 0x69, 0x1edd, 0x20, 0x62, 0x61, 0x6e, 0x20, 0x6e, 0x67, 0xe0, 0x79,
0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x53, 0x65,
0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x47, 0x69, 0x1edd, 0x20,
0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65, 0x47, 0x69, 0x1edd,
0x20, 0x51, 0x75, 0x1ea7, 0x6e, 0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x53, 0x6f,
0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x4e, 0x61, 0x6d,
0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20,
0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x47, 0x69, 0x1edd, 0x20,
0x53, 0x79, 0x6f, 0x77, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x54, 0x61, 0x68,
0x69, 0x74, 0x69, 0x47, 0x69, 0x1edd, 0x20, 0x110, 0xe0, 0x69, 0x20, 0x42,
0x1eaf, 0x63, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20,
0x110, 0xe0, 0x69, 0x20, 0x42, 0x1eaf, 0x63, 0x47, 0x69, 0x1edd, 0x20, 0x4d,
0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x110, 0xe0, 0x69, 0x20, 0x42, 0x1eaf,
0x63, 0x47, 0x69, 0x1edd, 0x20, 0x54, 0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c,
0x61, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x47,
0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8,
0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68,
0x75, 0x75, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x43,
0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61,
0x20, 0x48, 0xe8, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x54, 0x75, 0x76, 0x61,
0x6c, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61,
0x79, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9,
0x61, 0x20, 0x48, 0xe8, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79,
0x47, 0x69, 0x1edd, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20,
0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x47, 0x69,
0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x55, 0x7a, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68,
0x75, 0x1ea9, 0x6e, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x47,
0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x56, 0x65, 0x6e,
0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x47, 0x69, 0x1edd,
0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9,
0x61, 0x20, 0x68, 0xe8, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75,
0x1ea9, 0x6e, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x47, 0x69, 0x1edd, 0x20,
0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x110, 0x1ea3,
0x6f, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x47, 0x69, 0x1edd, 0x20, 0x57, 0x61,
0x6c, 0x6c, 0x69, 0x73, 0x20, 0x76, 0xe0, 0x20, 0x46, 0x75, 0x74, 0x75,
0x6e, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x59,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9,
0x61, 0x20, 0x68, 0xe8, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x47, 0x69, 0x1edd, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75,
0x1ea9, 0x6e, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20,
0x68, 0xe8, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x59, 0x75, 0x6b, 0x6f,
0x6e, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x161, 0x69, 0x20, 0x5a,
0x69, 0x74, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x161, 0x69, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x7a, 0x69, 0x74, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x161, 0x69, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0x69, 0x74, 0x4d, 0x69, 0x74, 0x74, 0x65, 0x6c,
0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x161, 0x69, 0x20, 0x5a, 0x69,
0x74, 0x4d, 0x69, 0x74, 0x74, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70,
0xe4, 0x69, 0x161, 0x69, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x7a, 0x69, 0x74, 0x4d, 0x69, 0x74, 0x74, 0x65, 0x6c, 0x65, 0x75,
0x72, 0x6f, 0x70, 0xe4, 0x69, 0x161, 0x69, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0x69, 0x74, 0x4f, 0x161, 0x74, 0x65, 0x75, 0x72, 0x6f,
0x70, 0xe4, 0x69, 0x161, 0x69, 0x20, 0x5a, 0x69, 0x74, 0x4f, 0x161, 0x74,
0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x161, 0x69, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x7a, 0x69, 0x74, 0x4f, 0x161, 0x74,
0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x161, 0x69, 0x20, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x74, 0x57, 0x65, 0x161, 0x74, 0x65,
0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x161, 0x69, 0x20, 0x5a, 0x69, 0x74,
0x57, 0x65, 0x161, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x161,
0x69, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x7a, 0x69,
0x74, 0x57, 0x65, 0x161, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69,
0x161, 0x69, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x74,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x41, 0x66, 0x67, 0x68, 0x61, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x43,
0x61, 0x6e, 0x6f, 0x6c, 0x62, 0x61, 0x72, 0x74, 0x68, 0x20, 0x41, 0x66,
0x66, 0x72, 0x69, 0x63, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x44,
0x77, 0x79, 0x72, 0x61, 0x69, 0x6e, 0x20, 0x41, 0x66, 0x66, 0x72, 0x69,
0x63, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f,
0x6e, 0x6f, 0x6c, 0x20, 0x44, 0x65, 0x20, 0x41, 0x66, 0x66, 0x72, 0x69,
0x63, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x47, 0x6f, 0x72, 0x6c,
0x6c, 0x65, 0x77, 0x69, 0x6e, 0x20, 0x41, 0x66, 0x66, 0x72, 0x69, 0x63,
0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e,
0x6f, 0x6c, 0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69, 0x6e,
0x20, 0x41, 0x66, 0x66, 0x72, 0x69, 0x63, 0x61, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c, 0x65,
0x77, 0x69, 0x6e, 0x20, 0x41, 0x66, 0x66, 0x72, 0x69, 0x63, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c,
0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x48, 0x61, 0x66, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61,
0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e,
0x6f, 0x6c, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x43,
0x61, 0x6e, 0x6f, 0x6c, 0x62, 0x61, 0x72, 0x74, 0x68, 0x20, 0x47, 0x6f,
0x67, 0x6c, 0x65, 0x64, 0x64, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e,
0x6f, 0x6c, 0x20, 0x43, 0x61, 0x6e, 0x6f, 0x6c, 0x62, 0x61, 0x72, 0x74,
0x68, 0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65, 0x64, 0x64, 0x20, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48,
0x61, 0x66, 0x20, 0x43, 0x61, 0x6e, 0x6f, 0x6c, 0x62, 0x61, 0x72, 0x74,
0x68, 0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65, 0x64, 0x64, 0x20, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x44,
0x77, 0x79, 0x72, 0x61, 0x69, 0x6e, 0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65,
0x64, 0x64, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20,
0x44, 0x77, 0x79, 0x72, 0x61, 0x69, 0x6e, 0x20, 0x47, 0x6f, 0x67, 0x6c,
0x65, 0x64, 0x64, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x44, 0x77, 0x79,
0x72, 0x61, 0x69, 0x6e, 0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65, 0x64, 0x64,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x4d, 0x79, 0x6e, 0x79, 0x64, 0x64, 0x6f, 0x65, 0x64, 0x64,
0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65, 0x64, 0x64, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61,
0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x4d, 0x79, 0x6e, 0x79, 0x64, 0x64,
0x6f, 0x65, 0x64, 0x64, 0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65, 0x64, 0x64,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x4d, 0x79, 0x6e, 0x79, 0x64, 0x64,
0x6f, 0x65, 0x64, 0x64, 0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65, 0x64, 0x64,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x43, 0x65, 0x66, 0x6e, 0x66, 0x6f, 0x72, 0x20, 0x54, 0x61,
0x77, 0x65, 0x6c, 0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65, 0x64, 0x64, 0x20,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x43, 0x65, 0x66,
0x6e, 0x66, 0x6f, 0x72, 0x20, 0x54, 0x61, 0x77, 0x65, 0x6c, 0x20, 0x47,
0x6f, 0x67, 0x6c, 0x65, 0x64, 0x64, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20,
0x43, 0x65, 0x66, 0x6e, 0x66, 0x6f, 0x72, 0x20, 0x54, 0x61, 0x77, 0x65,
0x6c, 0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65, 0x64, 0x64, 0x20, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x41,
0x70, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66,
0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x41, 0x70, 0x69, 0x61, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x41, 0x70, 0x69, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x41, 0x72, 0x61, 0x62, 0x61, 0x69, 0x64,
0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e,
0x6f, 0x6c, 0x20, 0x41, 0x72, 0x61, 0x62, 0x61, 0x69, 0x64, 0x64, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x41, 0x72, 0x61,
0x62, 0x61, 0x69, 0x64, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x79,
0x72, 0x20, 0x41, 0x72, 0x69, 0x61, 0x6e, 0x6e, 0x69, 0x6e, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20,
0x41, 0x72, 0x69, 0x61, 0x6e, 0x6e, 0x69, 0x6e, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x41, 0x72, 0x69, 0x61, 0x6e, 0x6e,
0x69, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x47, 0x6f, 0x72, 0x6c,
0x6c, 0x65, 0x77, 0x69, 0x6e, 0x20, 0x41, 0x72, 0x69, 0x61, 0x6e, 0x6e,
0x69, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f,
0x6e, 0x6f, 0x6c, 0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69,
0x6e, 0x20, 0x41, 0x72, 0x69, 0x61, 0x6e, 0x6e, 0x69, 0x6e, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x47, 0x6f, 0x72, 0x6c,
0x6c, 0x65, 0x77, 0x69, 0x6e, 0x20, 0x41, 0x72, 0x69, 0x61, 0x6e, 0x6e,
0x69, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66,
0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x41, 0x72,
0x6d, 0x65, 0x6e, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x43,
0x65, 0x66, 0x6e, 0x66, 0x6f, 0x72, 0x20, 0x79, 0x72, 0x20, 0x49, 0x77,
0x65, 0x72, 0x79, 0x64, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53,
0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x43, 0x65, 0x66, 0x6e, 0x66,
0x6f, 0x72, 0x20, 0x79, 0x72, 0x20, 0x49, 0x77, 0x65, 0x72, 0x79, 0x64,
0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x43,
0x65, 0x66, 0x6e, 0x66, 0x6f, 0x72, 0x20, 0x79, 0x72, 0x20, 0x49, 0x77,
0x65, 0x72, 0x79, 0x64, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x43,
0x61, 0x6e, 0x6f, 0x6c, 0x62, 0x61, 0x72, 0x74, 0x68, 0x20, 0x41, 0x77,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x43, 0x61, 0x6e,
0x6f, 0x6c, 0x62, 0x61, 0x72, 0x74, 0x68, 0x20, 0x41, 0x77, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48,
0x61, 0x66, 0x20, 0x43, 0x61, 0x6e, 0x6f, 0x6c, 0x62, 0x61, 0x72, 0x74,
0x68, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x43, 0x61, 0x6e, 0x6f, 0x6c, 0x62, 0x61,
0x72, 0x74, 0x68, 0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69,
0x6e, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c,
0x20, 0x43, 0x61, 0x6e, 0x6f, 0x6c, 0x62, 0x61, 0x72, 0x74, 0x68, 0x20,
0x47, 0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69, 0x6e, 0x20, 0x41, 0x77,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x48, 0x61, 0x66, 0x20, 0x43, 0x61, 0x6e, 0x6f, 0x6c, 0x62, 0x61,
0x72, 0x74, 0x68, 0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69,
0x6e, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x44, 0x77, 0x79, 0x72, 0x61, 0x69, 0x6e,
0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20,
0x44, 0x77, 0x79, 0x72, 0x61, 0x69, 0x6e, 0x20, 0x41, 0x77, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48,
0x61, 0x66, 0x20, 0x44, 0x77, 0x79, 0x72, 0x61, 0x69, 0x6e, 0x20, 0x41,
0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69, 0x6e, 0x20,
0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x47,
0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69, 0x6e, 0x20, 0x41, 0x77, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x48, 0x61, 0x66, 0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69,
0x6e, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x69,
0x6a, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66,
0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x69,
0x6a, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66,
0x20, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x79, 0x72, 0x20, 0x41, 0x7a, 0x6f, 0x72,
0x65, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f,
0x6e, 0x6f, 0x6c, 0x20, 0x79, 0x72, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65,
0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x79,
0x72, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f,
0x6c, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x42, 0x6f, 0x6c, 0x69, 0x66, 0x69, 0x61, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c,
0x20, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x42, 0x72, 0x61, 0x73, 0xed,
0x6c, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x42, 0x72, 0x75,
0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c,
0x61, 0x6d, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x43, 0x61, 0x62, 0x6f,
0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x43, 0x61, 0x62, 0x6f,
0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x48, 0x61, 0x66, 0x20, 0x43, 0x61, 0x62, 0x6f, 0x20, 0x56, 0x65, 0x72,
0x64, 0x65, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x43, 0x68, 0x61, 0x6d,
0x6f, 0x72, 0x72, 0x6f, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53,
0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x43, 0x68, 0x69, 0x6c,
0x65, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x43,
0x68, 0x69, 0x6c, 0x65, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x54, 0x73,
0x69, 0x65, 0x69, 0x6e, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53,
0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x54, 0x73, 0x69, 0x65, 0x69,
0x6e, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20,
0x54, 0x73, 0x69, 0x65, 0x69, 0x6e, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x59, 0x20, 0x4e, 0x61, 0x64, 0x6f,
0x6c, 0x69, 0x67, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x59, 0x6e, 0x79,
0x73, 0x6f, 0x65, 0x64, 0x64, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69,
0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e,
0x6f, 0x6c, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x43, 0x6f, 0x6c,
0x6f, 0x6d, 0x62, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x59,
0x6e, 0x79, 0x73, 0x6f, 0x65, 0x64, 0x64, 0x20, 0x43, 0x6f, 0x6f, 0x6b,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f,
0x6c, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x6f, 0x65, 0x64, 0x64, 0x20, 0x43,
0x6f, 0x6f, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x6e,
0x6e, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x59, 0x6e, 0x79, 0x73,
0x6f, 0x65, 0x64, 0x64, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x43, 0x69, 0x77, 0x62, 0x61, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x43, 0x69,
0x77, 0x62, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66,
0x20, 0x43, 0x69, 0x77, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x44,
0x61, 0x76, 0x69, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x44, 0x75,
0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c,
0x6c, 0x65, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x44, 0x77, 0x79, 0x72,
0x61, 0x69, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x79, 0x20, 0x50, 0x61,
0x73, 0x67, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f,
0x6e, 0x6f, 0x6c, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x79, 0x20, 0x50,
0x61, 0x73, 0x67, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66,
0x20, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x79, 0x20, 0x50, 0x61, 0x73, 0x67,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f,
0x72, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x43, 0x61, 0x6e, 0x6f, 0x6c,
0x62, 0x61, 0x72, 0x74, 0x68, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c,
0x20, 0x43, 0x61, 0x6e, 0x6f, 0x6c, 0x62, 0x61, 0x72, 0x74, 0x68, 0x20,
0x45, 0x77, 0x72, 0x6f, 0x70, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48,
0x61, 0x66, 0x20, 0x43, 0x61, 0x6e, 0x6f, 0x6c, 0x62, 0x61, 0x72, 0x74,
0x68, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x44, 0x77, 0x79, 0x72, 0x61, 0x69, 0x6e, 0x20, 0x45, 0x77, 0x72,
0x6f, 0x70, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f,
0x6e, 0x6f, 0x6c, 0x20, 0x44, 0x77, 0x79, 0x72, 0x61, 0x69, 0x6e, 0x20,
0x45, 0x77, 0x72, 0x6f, 0x70, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48,
0x61, 0x66, 0x20, 0x44, 0x77, 0x79, 0x72, 0x61, 0x69, 0x6e, 0x20, 0x45,
0x77, 0x72, 0x6f, 0x70, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x44, 0x77,
0x79, 0x72, 0x61, 0x69, 0x6e, 0x20, 0x50, 0x65, 0x6c, 0x6c, 0x20, 0x45,
0x77, 0x72, 0x6f, 0x70, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x47, 0x6f,
0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69, 0x6e, 0x20, 0x45, 0x77, 0x72, 0x6f,
0x70, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e,
0x6f, 0x6c, 0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69, 0x6e,
0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x48, 0x61, 0x66, 0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69,
0x6e, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x59, 0x6e, 0x79, 0x73, 0x6f, 0x65, 0x64, 0x64, 0x20, 0x46, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x2f, 0x4d, 0x61, 0x6c, 0x76,
0x69, 0x6e, 0x61, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61,
0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x6f, 0x65,
0x64, 0x64, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x2f, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x6f,
0x65, 0x64, 0x64, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x2f, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x46, 0x69,
0x6a, 0x69, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20,
0x46, 0x69, 0x6a, 0x69, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x47, 0x75,
0x79, 0x61, 0x6e, 0x65, 0x20, 0x46, 0x66, 0x72, 0x65, 0x6e, 0x67, 0x69,
0x67, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x54, 0x69, 0x72, 0x6f, 0x65,
0x64, 0x64, 0x20, 0x46, 0x66, 0x72, 0x61, 0x69, 0x6e, 0x63, 0x20, 0x79,
0x6e, 0x20, 0x79, 0x20, 0x44, 0x65, 0x20, 0x61, 0x2019, 0x72, 0x20, 0x41,
0x6e, 0x74, 0x61, 0x72, 0x63, 0x74, 0x69, 0x67, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e,
0x6f, 0x6c, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x47, 0x65, 0x6f, 0x72,
0x67, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x59, 0x6e, 0x79,
0x73, 0x6f, 0x65, 0x64, 0x64, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72,
0x74, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e,
0x6f, 0x6c, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x44, 0x77, 0x79, 0x72, 0x61, 0x69,
0x6e, 0x20, 0x79, 0x72, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x4c, 0x61,
0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e,
0x6f, 0x6c, 0x20, 0x44, 0x77, 0x79, 0x72, 0x61, 0x69, 0x6e, 0x20, 0x79,
0x72, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x4c, 0x61, 0x73, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x44, 0x77, 0x79, 0x72,
0x61, 0x69, 0x6e, 0x20, 0x79, 0x72, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x20,
0x4c, 0x61, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x47, 0x6f, 0x72,
0x6c, 0x6c, 0x65, 0x77, 0x69, 0x6e, 0x20, 0x79, 0x72, 0x20, 0x59, 0x6e,
0x79, 0x73, 0x20, 0x4c, 0x61, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x47, 0x6f, 0x72, 0x6c,
0x6c, 0x65, 0x77, 0x69, 0x6e, 0x20, 0x79, 0x72, 0x20, 0x59, 0x6e, 0x79,
0x73, 0x20, 0x4c, 0x61, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48,
0x61, 0x66, 0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69, 0x6e,
0x20, 0x79, 0x72, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x4c, 0x61, 0x73,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f,
0x6c, 0x20, 0x79, 0x20, 0x47, 0x77, 0x6c, 0x66, 0x66, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x48, 0x61, 0x77, 0x61,
0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b,
0x6f, 0x6e, 0x67, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66,
0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f,
0x6e, 0x67, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20,
0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x48, 0x6f, 0x76,
0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x48,
0x6f, 0x76, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x49, 0x6e, 0x64,
0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x43, 0x65, 0x66, 0x6e,
0x66, 0x6f, 0x72, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x2d, 0x54, 0x73, 0x69, 0x65,
0x69, 0x6e, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x43, 0x61, 0x6e,
0x6f, 0x6c, 0x62, 0x61, 0x72, 0x74, 0x68, 0x20, 0x49, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x73, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x44,
0x77, 0x79, 0x72, 0x61, 0x69, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e,
0x65, 0x73, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x47, 0x6f,
0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x73, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x49,
0x72, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66,
0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f,
0x6c, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f,
0x6e, 0x6f, 0x6c, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x4a, 0x61, 0x70, 0x61,
0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e,
0x6f, 0x6c, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73,
0x74, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x44, 0x77, 0x79,
0x72, 0x61, 0x69, 0x6e, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73,
0x74, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x47, 0x6f, 0x72,
0x6c, 0x6c, 0x65, 0x77, 0x69, 0x6e, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b,
0x68, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x43,
0x6f, 0x72, 0x65, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61,
0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x43, 0x6f, 0x72,
0x65, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x4b, 0x6f, 0x73, 0x72,
0x61, 0x65, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x79, 0x61, 0x72, 0x73, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x4b,
0x79, 0x72, 0x67, 0x79, 0x7a, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x6f, 0x65, 0x64, 0x64, 0x20,
0x4c, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x79, 0x72,
0x20, 0x41, 0x72, 0x67, 0x6c, 0x77, 0x79, 0x64, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f,
0x6e, 0x6f, 0x6c, 0x20, 0x79, 0x72, 0x20, 0x41, 0x72, 0x67, 0x6c, 0x77,
0x79, 0x64, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x79, 0x72, 0x20, 0x41, 0x72, 0x67,
0x6c, 0x77, 0x79, 0x64, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c,
0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x4d, 0x61, 0x6c, 0x61, 0x79,
0x73, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x59, 0x20, 0x4d,
0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x69, 0x73, 0x65, 0x73, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x6f, 0x65, 0x64, 0x64,
0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f,
0x6c, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x50, 0x61, 0x73, 0x69, 0x66, 0x66, 0x69, 0x67, 0x20, 0x4d, 0x65, 0x63,
0x73, 0x69, 0x63, 0x6f, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61,
0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x66,
0x69, 0x67, 0x20, 0x4d, 0x65, 0x63, 0x73, 0x69, 0x63, 0x6f, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x50, 0x61, 0x73, 0x69,
0x66, 0x66, 0x69, 0x67, 0x20, 0x4d, 0x65, 0x63, 0x73, 0x69, 0x63, 0x6f,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42,
0x61, 0x74, 0x6f, 0x72, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61,
0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42,
0x61, 0x74, 0x6f, 0x72, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61,
0x66, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x66, 0x61,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f,
0x6c, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x66, 0x61, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x66, 0x61,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61,
0x72, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x69, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x79, 0x64, 0x64, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x43,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x4e, 0x65, 0x77,
0x79, 0x64, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66,
0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x4e,
0x65, 0x77, 0x79, 0x64, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53,
0x65, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x4e, 0x65, 0x77, 0x79, 0x64, 0x64,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f,
0x6c, 0x20, 0x53, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x4e, 0x65, 0x77,
0x79, 0x64, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66,
0x20, 0x53, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x4e, 0x65, 0x77, 0x79,
0x64, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x4e, 0x65, 0x77, 0x66,
0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x4e, 0x65,
0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x4e, 0x65, 0x77, 0x66,
0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x59, 0x6e, 0x79, 0x73, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f,
0x6c, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20,
0x59, 0x6e, 0x79, 0x73, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e,
0x6f, 0x6c, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x4f, 0x6d, 0x73, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61,
0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x4f, 0x6d, 0x73, 0x6b,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f,
0x6e, 0x6f, 0x6c, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x50, 0x61, 0x6c, 0x61, 0x75, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x50,
0x61, 0x70, 0x75, 0x61, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x20,
0x4e, 0x65, 0x77, 0x79, 0x64, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x48, 0x61, 0x66, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x50, 0x65, 0x72, 0x69, 0x77, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c,
0x20, 0x50, 0x65, 0x72, 0x69, 0x77, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x48, 0x61, 0x66, 0x20, 0x50, 0x65, 0x72, 0x69, 0x77, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x50, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x73,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f,
0x6c, 0x20, 0x50, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x50, 0x69, 0x6c,
0x69, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x59, 0x6e, 0x79, 0x73, 0x6f, 0x65, 0x64, 0x64, 0x20, 0x50, 0x68, 0x6f,
0x65, 0x6e, 0x69, 0x78, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61,
0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65,
0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20,
0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x53, 0x61,
0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65,
0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x50, 0x6f, 0x68, 0x6e, 0x70, 0x65,
0x69, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67,
0x79, 0x61, 0x6e, 0x67, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x52, 0xe9,
0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x52,
0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x53, 0x61,
0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x48, 0x61, 0x66, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f,
0x6c, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c,
0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x48, 0x61, 0x66, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65,
0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61,
0x70, 0x6f, 0x72, 0x65, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x59, 0x6e,
0x79, 0x73, 0x6f, 0x65, 0x64, 0x64, 0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d,
0x6f, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x44, 0x65, 0x20, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x54, 0x61, 0x69,
0x70, 0x65, 0x69, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66,
0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x54, 0x61, 0x6a, 0x69, 0x63, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c,
0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x48, 0x61, 0x66, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x54, 0x79, 0x72, 0x63, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f,
0x6e, 0x6f, 0x6c, 0x20, 0x54, 0x79, 0x72, 0x63, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61,
0x66, 0x20, 0x54, 0x79, 0x72, 0x63, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x54, 0x75, 0x76, 0x61,
0x6c, 0x75, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x55, 0x72, 0x75, 0x67,
0x75, 0x61, 0x79, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66,
0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x79, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x55,
0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x55,
0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66,
0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61,
0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48,
0x61, 0x66, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53,
0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61,
0x66, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x57, 0x61,
0x6b, 0x65, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x57, 0x61, 0x6c, 0x6c,
0x69, 0x73, 0x20, 0x61, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f,
0x6c, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x59, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x59,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x59, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x627, 0x648,
0x6af, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x648, 0x62e, 0x62a,
0x631, 0x648, 0x686, 0x200c, 0x62f, 0x631, 0x627, 0x62a, 0x6cc, 0x646, 0x20, 0x642,
0x632, 0x627, 0x642, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x648, 0x62e, 0x62a,
0x631, 0x648, 0x686, 0x200c, 0x6a9, 0x67e, 0x62a, 0x6cc, 0x646, 0x20, 0x642, 0x632,
0x627, 0x642, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x642,
0x6cc, 0x631, 0x63a, 0x6cc, 0x632, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x648,
0x62e, 0x62a, 0x62a, 0x627, 0x62c, 0x6cc, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x6cc,
0x20, 0x648, 0x62e, 0x62a, 0x41, 0x63, 0x72, 0x65, 0x2d, 0x74, 0x69, 0x69,
0x64, 0x41, 0x63, 0x72, 0x65, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65,
0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x41, 0x63, 0x72, 0x65, 0x2d, 0x73,
0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x41, 0x66, 0x67,
0x68, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64,
0x53, 0x69, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x2d, 0x41, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69,
0x64, 0x45, 0x61, 0x73, 0x74, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x53, 0xfb,
0x64, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x6b,
0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74,
0x69, 0x69, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x41, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x57, 0x65, 0x73, 0x74,
0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x41,
0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x41, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72,
0x74, 0x74, 0x69, 0x69, 0x64, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d,
0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x41, 0x6c,
0x6d, 0x61, 0x2d, 0x41, 0x74, 0x61, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x41,
0x6c, 0x6d, 0x61, 0x2d, 0x41, 0x74, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x41, 0x6c, 0x6d, 0x61,
0x2d, 0x41, 0x74, 0x61, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x69, 0x64, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x65, 0x2d, 0x74,
0x69, 0x69, 0x64, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x65, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x41,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x65, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x69, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x2d, 0x74, 0x69, 0x69, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69,
0x64, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x2d, 0x73, 0x69, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x45, 0x61, 0x73, 0x74, 0x65,
0x72, 0x6e, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x45, 0x61, 0x73, 0x74, 0x65,
0x72, 0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74,
0x69, 0x69, 0x64, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x2d, 0x73,
0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x6f, 0x75,
0x6e, 0x74, 0x61, 0x69, 0x6e, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x6f,
0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x6f, 0x75, 0x6e, 0x74,
0x61, 0x69, 0x6e, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x69, 0x64, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x2d, 0x74, 0x69,
0x69, 0x64, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x50, 0x61,
0x73, 0x69, 0x66, 0x69, 0x6b, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x69, 0x64, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x2d, 0x74,
0x69, 0x69, 0x64, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x41, 0x6e,
0x61, 0x64, 0x79, 0x72, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x69, 0x64, 0x41, 0x70, 0x69, 0x61, 0x2d, 0x74, 0x69, 0x69, 0x64,
0x41, 0x70, 0x69, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72,
0x74, 0x74, 0x69, 0x69, 0x64, 0x41, 0x70, 0x69, 0x61, 0x2d, 0x73, 0x69,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x41, 0x71, 0x74, 0x61,
0x75, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x41, 0x71, 0x74, 0x61, 0x75, 0x2d,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64,
0x41, 0x71, 0x74, 0x61, 0x75, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x69, 0x64, 0x41, 0x71, 0x74, 0xf6, 0x62, 0x65, 0x2d, 0x74,
0x69, 0x69, 0x64, 0x41, 0x71, 0x74, 0xf6, 0x62, 0x65, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x41, 0x71,
0x74, 0xf6, 0x62, 0x65, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x69, 0x64, 0x41, 0x72, 0x61, 0x62, 0x79, 0x73, 0x6b, 0x65, 0x20,
0x74, 0x69, 0x69, 0x64, 0x41, 0x72, 0x61, 0x62, 0x79, 0x73, 0x6b, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69,
0x64, 0x41, 0x72, 0x61, 0x62, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x79, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x79, 0x6e, 0x73, 0x6b, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x79, 0x6e, 0x73, 0x6b, 0x65, 0x20,
0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x57, 0x65,
0x73, 0x74, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x79, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x79, 0x6e, 0x73, 0x6b, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64,
0x57, 0x65, 0x73, 0x74, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x79,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x69, 0x64, 0x41, 0x72, 0x6d, 0x65, 0x65, 0x6e, 0x73, 0x65, 0x20,
0x74, 0x69, 0x69, 0x64, 0x41, 0x72, 0x6d, 0x65, 0x65, 0x6e, 0x73, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69,
0x64, 0x41, 0x72, 0x6d, 0x65, 0x65, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x69,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x41, 0x74, 0x6c, 0x61,
0x6e, 0x74, 0x69, 0x63, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x63, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65,
0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x63, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69,
0x64, 0x4d, 0x69, 0x64, 0x64, 0x65, 0x6e, 0x2d, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64,
0x4d, 0x69, 0x64, 0x64, 0x65, 0x6e, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x69, 0x64, 0x64, 0x65,
0x6e, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x73, 0x6b,
0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64,
0x4d, 0x69, 0x64, 0x64, 0x65, 0x6e, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x77, 0x65, 0x73, 0x74, 0x65,
0x6c, 0x69, 0x6a, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x69,
0x64, 0x64, 0x65, 0x6e, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x79, 0x73, 0x6b, 0x65, 0x20, 0x77, 0x65, 0x73, 0x74, 0x65, 0x6c, 0x69,
0x6a, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74,
0x74, 0x69, 0x69, 0x64, 0x4d, 0x69, 0x64, 0x64, 0x65, 0x6e, 0x2d, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x77,
0x65, 0x73, 0x74, 0x65, 0x6c, 0x69, 0x6a, 0x6b, 0x65, 0x20, 0x73, 0x69,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x45, 0x61, 0x73, 0x74,
0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x73, 0x6b, 0x65,
0x20, 0x74, 0x69, 0x69, 0x64, 0x45, 0x61, 0x73, 0x74, 0x2d, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x45, 0x61,
0x73, 0x74, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x73,
0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69,
0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x57, 0x65,
0x73, 0x74, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x73,
0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74,
0x69, 0x69, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65,
0x69, 0x64, 0x7a, 0x6a, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74,
0x69, 0x69, 0x64, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65, 0x69, 0x64, 0x7a,
0x6a, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x65, 0x69, 0x64, 0x7a, 0x6a, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x6e, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x41, 0x7a,
0x6f, 0x72, 0x65, 0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72,
0x74, 0x74, 0x69, 0x69, 0x64, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x6e, 0x2d,
0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x42, 0x65,
0x6e, 0x67, 0x61, 0x6c, 0x65, 0x73, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64,
0x42, 0x65, 0x6e, 0x67, 0x61, 0x6c, 0x65, 0x73, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x42, 0x65,
0x6e, 0x67, 0x61, 0x6c, 0x65, 0x73, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x42, 0x68, 0x75, 0x74, 0x61, 0x61,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x42, 0x6f, 0x6c,
0x69, 0x76, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69,
0x69, 0x64, 0x42, 0x72, 0x61, 0x7a, 0x79, 0x6c, 0x6a, 0x61, 0x61, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x42, 0x72, 0x61, 0x7a,
0x79, 0x6c, 0x6a, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x42, 0x72,
0x61, 0x7a, 0x79, 0x6c, 0x6a, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20,
0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x42, 0x72,
0x75, 0x6e, 0x65, 0x69, 0x73, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4b,
0x61, 0x61, 0x70, 0x76, 0x65, 0x72, 0x64, 0x79, 0x73, 0x6b, 0x65, 0x20,
0x74, 0x69, 0x69, 0x64, 0x4b, 0x61, 0x61, 0x70, 0x76, 0x65, 0x72, 0x64,
0x79, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72,
0x74, 0x74, 0x69, 0x69, 0x64, 0x4b, 0x61, 0x61, 0x70, 0x76, 0x65, 0x72,
0x64, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x69, 0x64, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f,
0x2d, 0x74, 0x69, 0x69, 0x64, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x20, 0x74, 0x69, 0x69, 0x64, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69,
0x64, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x20, 0x73, 0x69, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x53, 0x69, 0x6c, 0x65, 0x65,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x53, 0x69, 0x6c,
0x65, 0x65, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x53, 0x69, 0x6c, 0x65, 0x65,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x69, 0x64, 0x53, 0x69, 0x6e, 0x65, 0x65, 0x73, 0x6b, 0x65, 0x20,
0x74, 0x69, 0x69, 0x64, 0x53, 0x69, 0x6e, 0x65, 0x65, 0x73, 0x6b, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69,
0x64, 0x53, 0x69, 0x6e, 0x65, 0x65, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x4b, 0x72, 0x79, 0x73,
0x74, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69,
0x69, 0x64, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x65, 0x69, 0x6c, 0xe2, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4b, 0x6f, 0x6c, 0x6f,
0x6d, 0x62, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69,
0x69, 0x64, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x61, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74,
0x74, 0x69, 0x69, 0x64, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61,
0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x69, 0x64, 0x43, 0x6f, 0x6f, 0x6b, 0x65, 0x69, 0x6c, 0xe2,
0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x43, 0x6f, 0x6f, 0x6b,
0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x43, 0x6f, 0x6f, 0x6b,
0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x65, 0x20, 0x68, 0x61, 0x6c, 0x76,
0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64,
0x4b, 0x75, 0x62, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69,
0x69, 0x64, 0x4b, 0x75, 0x62, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64,
0x4b, 0x75, 0x62, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x44, 0x61, 0x76, 0x69,
0x73, 0x20, 0x74, 0x69, 0x69, 0x64, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74,
0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x20, 0x74,
0x69, 0x69, 0x64, 0x45, 0x61, 0x73, 0x74, 0x2d, 0x54, 0x69, 0x6d, 0x6f,
0x72, 0x65, 0x73, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x50, 0x65, 0x61,
0x73, 0x6b, 0x65, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20,
0x74, 0x69, 0x69, 0x64, 0x50, 0x65, 0x61, 0x73, 0x6b, 0x65, 0x65, 0x69,
0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x50, 0x65, 0x61, 0x73, 0x6b,
0x65, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x45, 0x63, 0x75, 0x61,
0x64, 0x6f, 0x72, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69,
0x69, 0x64, 0x4d, 0x69, 0x64, 0x64, 0x65, 0x6e, 0x2d, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x65, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64,
0x4d, 0x69, 0x64, 0x64, 0x65, 0x6e, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x65, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65,
0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x69, 0x64, 0x64, 0x65, 0x6e,
0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x65, 0x73, 0x6b, 0x65, 0x20,
0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x45, 0x61,
0x73, 0x74, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x65, 0x73, 0x6b,
0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x45, 0x61, 0x73, 0x74, 0x2d, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x65, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x45, 0x61,
0x73, 0x74, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x65, 0x73, 0x6b,
0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64,
0x46, 0x69, 0x65, 0x72, 0x64, 0x65, 0x72, 0x2d, 0x65, 0x61, 0x73, 0x74,
0x6c, 0x69, 0x6b, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x65, 0x73,
0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x65, 0x73, 0x6b, 0x65, 0x20, 0x74,
0x69, 0x69, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x65, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x65, 0x73, 0x6b, 0x65, 0x20, 0x73,
0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x46, 0x61, 0x6c,
0x6b, 0x6c, 0xe2, 0x6e, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x74, 0x69, 0x69, 0x64, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0xe2, 0x6e,
0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x46, 0x61, 0x6c,
0x6b, 0x6c, 0xe2, 0x6e, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x46,
0x69, 0x6a, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x46,
0x69, 0x6a, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x46, 0x69, 0x6a, 0x79, 0x73,
0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69,
0x64, 0x46, 0x72, 0xe2, 0x6e, 0x73, 0x2d, 0x47, 0x75, 0x79, 0x61, 0x61,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x46, 0x72, 0xe2,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x53, 0xfa, 0x64, 0x6c, 0x69, 0x6b, 0x65,
0x20, 0x65, 0x6e, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x63, 0x74, 0x79,
0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x47, 0x61, 0x6c, 0x61,
0x70, 0x61, 0x67, 0x6f, 0x73, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b,
0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65,
0x72, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69,
0x69, 0x64, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x79, 0x73, 0x6b, 0x65, 0x20,
0x74, 0x69, 0x69, 0x64, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x79, 0x73, 0x6b,
0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69,
0x69, 0x64, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x79, 0x73, 0x6b, 0x65, 0x20,
0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x47, 0x69,
0x6c, 0x62, 0x65, 0x72, 0x74, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b,
0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x45, 0x61, 0x73, 0x74, 0x2d, 0x47,
0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74,
0x69, 0x69, 0x64, 0x45, 0x61, 0x73, 0x74, 0x2d, 0x47, 0x72, 0x6f, 0x65,
0x6e, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x45, 0x61, 0x73, 0x74,
0x2d, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x57,
0x65, 0x73, 0x74, 0x2d, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe2, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x57, 0x65, 0x73, 0x74,
0x2d, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69,
0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c,
0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x69, 0x64, 0x47, 0x75, 0x61, 0x6d, 0x65, 0x73, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64,
0x47, 0x6f, 0x6c, 0x66, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72,
0x74, 0x74, 0x69, 0x69, 0x64, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x6f, 0x65, 0x74, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x74,
0x69, 0x69, 0x64, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c,
0x65, 0x6f, 0x65, 0x74, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x6f, 0x65, 0x74, 0x79, 0x73,
0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69,
0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x73, 0x65, 0x20,
0x74, 0x69, 0x69, 0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67,
0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74,
0x69, 0x69, 0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x73,
0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64,
0x48, 0x6f, 0x76, 0x64, 0x20, 0x74, 0x69, 0x69, 0x64, 0x48, 0x6f, 0x76,
0x64, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69,
0x69, 0x64, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x69, 0x64, 0x59, 0x6e, 0x64, 0x69, 0x61, 0x61, 0x73,
0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x59, 0x6e, 0x64, 0x79, 0x73,
0x6b, 0x65, 0x20, 0x4f, 0x63, 0x65, 0x61, 0x61, 0x6e, 0x2d, 0x74, 0x69,
0x69, 0x64, 0x59, 0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x65, 0x73,
0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x53, 0x69, 0x6e, 0x74, 0x72, 0x61,
0x61, 0x6c, 0x2d, 0x59, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x79, 0x73,
0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x45, 0x61, 0x73, 0x74, 0x2d,
0x59, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x79, 0x73, 0x6b, 0x65, 0x20,
0x74, 0x69, 0x69, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x59, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x7a, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69,
0x64, 0x49, 0x72, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69,
0x69, 0x64, 0x49, 0x72, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x49,
0x72, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x49, 0x72, 0x6b, 0x6f, 0x65, 0x74,
0x73, 0x6b, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x49, 0x72, 0x6b, 0x6f, 0x65,
0x74, 0x73, 0x6b, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74,
0x74, 0x69, 0x69, 0x64, 0x49, 0x72, 0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b,
0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x49,
0x73, 0x72, 0x61, 0xeb, 0x6c, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69,
0x69, 0x64, 0x49, 0x73, 0x72, 0x61, 0xeb, 0x6c, 0x79, 0x73, 0x6b, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69,
0x64, 0x49, 0x73, 0x72, 0x61, 0xeb, 0x6c, 0x79, 0x73, 0x6b, 0x65, 0x20,
0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x4a, 0x61,
0x70, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4a,
0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4a, 0x61, 0x70, 0x61,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x69, 0x64, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c,
0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x74, 0x73, 0x6a, 0x61,
0x74, 0x73, 0x6b, 0x69, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x50, 0x65, 0x74,
0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b,
0x61, 0x6d, 0x74, 0x73, 0x6a, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x50,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x2d, 0x4b, 0x61, 0x6d, 0x74, 0x73, 0x6a, 0x61, 0x74, 0x73, 0x6b, 0x69,
0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x4b,
0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x74, 0x69,
0x69, 0x64, 0x45, 0x61, 0x73, 0x74, 0x2d, 0x4b, 0x61, 0x7a, 0x61, 0x63,
0x68, 0x73, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x57, 0x65, 0x73, 0x74,
0x2d, 0x4b, 0x61, 0x7a, 0x61, 0x63, 0x68, 0x73, 0x65, 0x20, 0x74, 0x69,
0x69, 0x64, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x74, 0x69, 0x69, 0x64, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x61, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74,
0x74, 0x69, 0x69, 0x64, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x61, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69,
0x64, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x73, 0x65, 0x20, 0x74, 0x69,
0x69, 0x64, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73,
0x6b, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65,
0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x69, 0x64, 0x4b, 0x69, 0x72, 0x67, 0x69, 0x7a, 0x79, 0x73,
0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4c, 0x61, 0x6e, 0x6b, 0x61,
0x2d, 0x74, 0x69, 0x69, 0x64, 0x4c, 0x69, 0x6e, 0x65, 0x2d, 0x65, 0x69,
0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0x65, 0x69, 0x6c,
0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0x65, 0x69, 0x6c, 0xe2,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72,
0x74, 0x74, 0x69, 0x69, 0x64, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x2d, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20,
0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x61,
0x63, 0x61, 0x75, 0x73, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x61,
0x63, 0x61, 0x75, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65,
0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x73,
0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x2d, 0x74, 0x69, 0x69, 0x64,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x69, 0x64, 0x4d, 0x61, 0x6c, 0x65, 0x69, 0x73, 0x79, 0x73, 0x6b, 0x65,
0x20, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x79,
0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x61, 0x72, 0x71,
0x75, 0x65, 0x73, 0x61, 0x73, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b,
0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61,
0x6c, 0x6c, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74,
0x69, 0x69, 0x64, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x61, 0x61,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x74, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x4d,
0x61, 0x77, 0x73, 0x6f, 0x6e, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x65,
0x6b, 0x73, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x50,
0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x4d,
0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20,
0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x65, 0x6b, 0x73,
0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x50, 0x61, 0x63,
0x69, 0x66, 0x69, 0x63, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x74, 0x69, 0x69, 0x64, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61,
0x74, 0x61, 0x72, 0x20, 0x74, 0x69, 0x69, 0x64, 0x55, 0x6c, 0x61, 0x61,
0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x55, 0x6c, 0x61, 0x61,
0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x20, 0x73, 0x69, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x75,
0x2d, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x75, 0x2d,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64,
0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x75, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72,
0x65, 0x73, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4e, 0x61, 0x75, 0x72,
0x75, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64,
0x4e, 0x65, 0x70, 0x61, 0x6c, 0x65, 0x73, 0x65, 0x20, 0x74, 0x69, 0x69,
0x64, 0x4e, 0x69, 0x6a, 0x2d, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x79, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4e, 0x69, 0x6a,
0x2d, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x79, 0x73, 0x6b, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69,
0x64, 0x4e, 0x69, 0x6a, 0x2d, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x79, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x69, 0x64, 0x4e, 0x69, 0x6a, 0x2d, 0x53, 0x65, 0x65, 0x6c, 0xe2,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4e, 0x69, 0x6a,
0x2d, 0x53, 0x65, 0x65, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4e,
0x69, 0x6a, 0x2d, 0x53, 0x65, 0x65, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x4e,
0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0xe2, 0x6e, 0x73, 0x6b,
0x65, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75,
0x6e, 0x64, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x2d, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4e, 0x65, 0x77,
0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x2d,
0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x4e, 0x69,
0x75, 0x65, 0x73, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x74, 0x69, 0x69, 0x64, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69,
0x64, 0x4e, 0x6f, 0x61, 0x72, 0x64, 0x6c, 0x69, 0x6b, 0x65, 0x20, 0x4d,
0x61, 0x72, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69,
0x69, 0x64, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65,
0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x69, 0x64, 0x4f, 0x6d, 0x73, 0x6b, 0x2d, 0x74, 0x69, 0x69,
0x64, 0x4f, 0x6d, 0x73, 0x6b, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65,
0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4f, 0x6d, 0x73, 0x6b, 0x2d, 0x73,
0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69,
0x69, 0x64, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74,
0x69, 0x69, 0x64, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x69, 0x64, 0x42, 0x65, 0x6c, 0x61, 0x75, 0x73, 0x65, 0x20, 0x74, 0x69,
0x69, 0x64, 0x50, 0x61, 0x70, 0x6f, 0x65, 0x61, 0x2d, 0x4e, 0x69, 0x6a,
0x2d, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x65, 0x73, 0x6b, 0x65, 0x20, 0x74,
0x69, 0x69, 0x64, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x61,
0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69,
0x64, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x61, 0x61, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x69, 0x64, 0x50, 0x65, 0x72, 0x75, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x74, 0x69, 0x69, 0x64, 0x50, 0x65, 0x72, 0x75, 0x61, 0x61, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74,
0x74, 0x69, 0x69, 0x64, 0x50, 0x65, 0x72, 0x75, 0x61, 0x61, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69,
0x64, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6a, 0x6e, 0x73, 0x65, 0x20,
0x74, 0x69, 0x69, 0x64, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6a, 0x6e,
0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74,
0x69, 0x69, 0x64, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6a, 0x6e, 0x73,
0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64,
0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x65, 0x69, 0x6c, 0xe2, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x53, 0x61, 0x69, 0x6e,
0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x6e, 0x20,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x74, 0x69, 0x69,
0x64, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x20, 0x65, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f,
0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69,
0x69, 0x64, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x20, 0x65, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69,
0x64, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x65, 0x69, 0x6c,
0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x50,
0x6f, 0x68, 0x6e, 0x70, 0x65, 0x69, 0x20, 0x74, 0x69, 0x69, 0x64, 0x50,
0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x2d, 0x74, 0x69, 0x69,
0x64, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x2d, 0x74,
0x69, 0x69, 0x64, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69,
0x64, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x2d, 0x73,
0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x52, 0xe9, 0x75,
0x6e, 0x69, 0x6f, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x52,
0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x20, 0x74, 0x69, 0x69, 0x64, 0x53,
0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x74, 0x69, 0x69, 0x64,
0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x53, 0x61, 0x63,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x69, 0x64, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d, 0x74,
0x69, 0x69, 0x64, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x53, 0x61,
0x6d, 0x61, 0x72, 0x61, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x69, 0x64, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x61, 0x6e, 0x73, 0x6b,
0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x61,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72,
0x74, 0x74, 0x69, 0x69, 0x64, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x61, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x69, 0x64, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x73, 0x65, 0x20,
0x74, 0x69, 0x69, 0x64, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72,
0x65, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74,
0x74, 0x69, 0x69, 0x64, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x73,
0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69,
0x64, 0x53, 0xfb, 0x64, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x79, 0x73,
0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x53, 0x75, 0x72, 0x69, 0x6e,
0x61, 0x61, 0x6d, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x53,
0x79, 0x6f, 0x77, 0x61, 0x20, 0x74, 0x69, 0x69, 0x64, 0x54, 0x61, 0x68,
0x69, 0x74, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69,
0x69, 0x64, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x74, 0x69, 0x69,
0x64, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69,
0x64, 0x54, 0x61, 0x64, 0x7a, 0x6a, 0x69, 0x65, 0x6b, 0x73, 0x65, 0x20,
0x74, 0x69, 0x69, 0x64, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x2d,
0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69,
0x64, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20,
0x74, 0x69, 0x69, 0x64, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x61, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74,
0x69, 0x69, 0x64, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x61, 0x6e, 0x73, 0x6b,
0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64,
0x43, 0x68, 0x75, 0x75, 0x6b, 0x73, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x65, 0x6e, 0x73, 0x65, 0x20, 0x74,
0x69, 0x69, 0x64, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x65, 0x6e, 0x73,
0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69,
0x69, 0x64, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x65, 0x6e, 0x73, 0x65,
0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x54,
0x75, 0x76, 0x61, 0x6c, 0x75, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20,
0x74, 0x69, 0x69, 0x64, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x61,
0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x79, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64,
0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x61, 0x61, 0x6e, 0x73, 0x6b,
0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64,
0x4f, 0x65, 0x7a, 0x62, 0x65, 0x65, 0x6b, 0x73, 0x65, 0x20, 0x74, 0x69,
0x69, 0x64, 0x4f, 0x65, 0x7a, 0x62, 0x65, 0x65, 0x6b, 0x73, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64,
0x4f, 0x65, 0x7a, 0x62, 0x65, 0x65, 0x6b, 0x73, 0x65, 0x20, 0x73, 0x69,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69,
0x69, 0x64, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x61, 0x61, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74,
0x74, 0x69, 0x69, 0x64, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x61,
0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x69, 0x64, 0x46, 0x65, 0x6e, 0x65, 0x7a, 0x6f, 0x6c, 0x61,
0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x74, 0x69,
0x69, 0x64, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69,
0x69, 0x64, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64,
0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2d, 0x74, 0x69,
0x69, 0x64, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2d,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64,
0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2d, 0x73, 0x69,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x56, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x20, 0x74, 0x69, 0x69, 0x64, 0x57, 0x61, 0x6b, 0x65, 0x2d,
0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69,
0x64, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x65, 0x6e, 0x20, 0x46,
0x75, 0x74, 0x75, 0x6e, 0x61, 0x73, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64,
0x4a, 0x61, 0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b, 0x2d, 0x74, 0x69, 0x69,
0x64, 0x4a, 0x61, 0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4a, 0x61,
0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x69, 0x64, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x65, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x2d, 0x74, 0x69, 0x69,
0x64, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x65, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72,
0x74, 0x74, 0x69, 0x69, 0x64, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x65, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x2d, 0x73, 0x69, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x59, 0x75, 0x6b, 0x6f, 0x6e,
0x2d, 0x74, 0x69, 0x69, 0x64, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x41,
0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x78,
0x74, 0x75, 0x20, 0x41, 0x66, 0x72, 0x69, 0x71, 0x75, 0x65, 0x20, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75,
0x20, 0x41, 0x66, 0x72, 0x69, 0x71, 0x75, 0x65, 0x20, 0x73, 0x6f, 0x77,
0x77, 0x75, 0x20, 0x6a, 0x61, 0x6e, 0x74, 0x57, 0x61, 0x78, 0x74, 0x75,
0x20, 0x73, 0x6f, 0x77, 0x77, 0x75, 0x20, 0x41, 0x66, 0x72, 0x69, 0x71,
0x75, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20,
0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x73, 0x6f, 0x77, 0x77,
0x75, 0x20, 0x41, 0x66, 0x72, 0x69, 0x71, 0x75, 0x65, 0x41, 0x66, 0x72,
0x69, 0x71, 0x75, 0x65, 0x20, 0x64, 0x75, 0x20, 0x73, 0x6f, 0x77, 0x77,
0x75, 0x20, 0x6a, 0x61, 0x6e, 0x74, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20,
0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20,
0x62, 0xeb, 0x63, 0x63, 0xeb, 0x67, 0x20, 0x63, 0x69, 0x20, 0x41, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75,
0xf1, 0x20, 0x6a, 0x61, 0x67, 0x6c, 0x65, 0x65, 0x6c, 0x20, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x74,
0x65, 0x20, 0x62, 0x75, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x43,
0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x73, 0xe0, 0x6e,
0x74, 0x61, 0x72, 0x61, 0x6c, 0x29, 0x43, 0x53, 0x54, 0x20, 0x28, 0x77,
0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x75, 0x20, 0x73, 0xe0, 0x6e, 0x74, 0x61, 0x72, 0x61, 0x61,
0x6c, 0x29, 0x43, 0x44, 0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74, 0x75,
0x20, 0x62, 0xeb, 0x63, 0x63, 0xeb, 0x67, 0x75, 0x20, 0x73, 0xe0, 0x6e,
0x74, 0x61, 0x72, 0x61, 0x61, 0x6c, 0x45, 0x54, 0x20, 0x77, 0x61, 0x78,
0x74, 0x75, 0x20, 0x70, 0x65, 0x6e, 0x6b, 0x75, 0x45, 0x53, 0x54, 0x20,
0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x75, 0x20, 0x70, 0x65, 0x6e, 0x6b, 0x75, 0x29,
0x45, 0x44, 0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62,
0xeb, 0x63, 0x63, 0xeb, 0x67, 0x75, 0x20, 0x70, 0x65, 0x6e, 0x6b, 0x75,
0x29, 0x4d, 0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x74,
0x75, 0x6e, 0x64, 0x75, 0x29, 0x4d, 0x53, 0x54, 0x20, 0x28, 0x77, 0x61,
0x78, 0x74, 0x75, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x75, 0x20, 0x74, 0x75, 0x6e, 0x64, 0x75, 0x29, 0x4d, 0x44, 0x54,
0x20, 0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0xeb, 0x63, 0x63,
0xeb, 0x67, 0x75, 0x20, 0x74, 0x75, 0x6e, 0x64, 0x75, 0x29, 0x50, 0x54,
0x20, 0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x70, 0x61, 0x73, 0x69,
0x66, 0x69, 0x6b, 0x29, 0x50, 0x53, 0x54, 0x20, 0x28, 0x77, 0x61, 0x78,
0x74, 0x75, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x75, 0x20, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x29, 0x50, 0x44,
0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0xeb, 0x63,
0x63, 0xeb, 0x67, 0x75, 0x20, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b,
0x29, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x41, 0x70, 0x69, 0x61, 0x57,
0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69,
0x6e, 0x20, 0x63, 0x69, 0x20, 0x41, 0x70, 0x69, 0x61, 0x41, 0x70, 0x69,
0x61, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0xeb, 0x63, 0x63,
0xeb, 0x67, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x61, 0x72, 0x61, 0x61,
0x62, 0x20, 0x79, 0x69, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75,
0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x41, 0x72,
0x61, 0x61, 0x62, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0xeb, 0x63,
0x63, 0xeb, 0x67, 0x20, 0x63, 0x69, 0x20, 0x41, 0x72, 0x61, 0x61, 0x62,
0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x41, 0x72, 0x73, 0x61, 0x6e, 0x74,
0x69, 0x69, 0x6e, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1,
0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x41, 0x72, 0x73,
0x61, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20,
0x65, 0x74, 0x65, 0x20, 0x62, 0x75, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x65, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x73, 0x6f,
0x77, 0x77, 0x75, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d,
0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x73, 0x6f, 0x77, 0x77, 0x75,
0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x65, 0x77, 0x61,
0x78, 0x74, 0x75, 0x20, 0x65, 0x74, 0x65, 0x20, 0x62, 0x75, 0x20, 0x73,
0x6f, 0x77, 0x77, 0x75, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20,
0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x74, 0x65, 0x20,
0x62, 0x75, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x41, 0x54, 0x20,
0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x61, 0x74, 0x6c, 0xe0, 0x6e,
0x74, 0x69, 0x6b, 0x29, 0x41, 0x53, 0x54, 0x20, 0x28, 0x77, 0x61, 0x78,
0x74, 0x75, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x75, 0x20, 0x70, 0x65, 0x6e, 0x6b, 0x75, 0x29, 0x41, 0x44, 0x54, 0x20,
0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0xeb, 0x63, 0x63, 0xeb,
0x67, 0x75, 0x20, 0x61, 0x74, 0x6c, 0xe0, 0x6e, 0x74, 0x69, 0x6b, 0x29,
0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x65, 0x20, 0x62, 0x75, 0x20, 0x64, 0x69, 0x67, 0x67, 0x69,
0x20, 0x62, 0x69, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1,
0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20,
0x62, 0xeb, 0x63, 0x63, 0xeb, 0x67, 0x20, 0x63, 0x69, 0x20, 0x64, 0x69,
0x67, 0x67, 0x75, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x73, 0x6f, 0x77, 0x77, 0x75,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x57, 0x61,
0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e,
0x20, 0x63, 0x69, 0x20, 0x64, 0x69, 0x67, 0x67, 0x75, 0x20, 0x73, 0x6f,
0x77, 0x77, 0x75, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0xeb, 0x63, 0x63, 0xeb,
0x67, 0x20, 0x63, 0x69, 0x20, 0x64, 0x69, 0x67, 0x67, 0x75, 0x20, 0x73,
0x6f, 0x77, 0x77, 0x75, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x65, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x70, 0x65, 0x6e, 0x6b,
0x75, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x57,
0x61, 0x78, 0x74, 0x75, 0x20, 0x70, 0x65, 0x6e, 0x6b, 0x75, 0x20, 0x62,
0x75, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x57,
0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69,
0x6e, 0x20, 0x63, 0x69, 0x20, 0x70, 0x65, 0x6e, 0x6b, 0x75, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x77, 0x61, 0x78, 0x74,
0x75, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x20,
0x62, 0x75, 0x20, 0x62, 0xeb, 0x74, 0x75, 0x20, 0x53, 0x6f, 0x6f, 0x77,
0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69,
0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x73, 0x6f, 0x77, 0x77, 0x75, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x57, 0x61, 0x78,
0x74, 0x75, 0x20, 0x62, 0xeb, 0x63, 0x63, 0xeb, 0x67, 0x20, 0x62, 0x75,
0x20, 0x73, 0x6f, 0x77, 0x77, 0x75, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x65, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0xef, 0x64,
0x6a, 0x61, 0x6e, 0x20, 0x57, 0x61, 0x78, 0x74, 0x75, 0x57, 0x61, 0x78,
0x74, 0x75, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0xef, 0x64, 0x6a,
0x61, 0x6e, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x74, 0x65, 0x20,
0x62, 0x75, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0xef, 0x64, 0x6a,
0x61, 0x6e, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x41, 0x7a, 0x6f, 0x72,
0x65, 0x73, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20,
0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x41, 0x7a, 0x6f, 0x72,
0x65, 0x73, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x20, 0x77, 0x61, 0x78,
0x74, 0x75, 0x20, 0x65, 0x74, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x57, 0x61,
0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e,
0x20, 0x63, 0x69, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x73, 0x68, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x74, 0x65, 0x20,
0x62, 0x75, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73,
0x68, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x42, 0x68, 0x6f, 0x75, 0x74,
0x61, 0x6e, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x42, 0x6f, 0x6c, 0x69,
0x76, 0x69, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x42, 0x72, 0x65,
0x73, 0x69, 0x6c, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20,
0x74, 0x69, 0x6d, 0x65, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61,
0x20, 0x73, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x74, 0x69, 0x6d, 0x65,
0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x77,
0x61, 0x78, 0x74, 0x75, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72,
0x64, 0x65, 0x20, 0x63, 0x69, 0x20, 0x6a, 0x61, 0x6d, 0x6f, 0x6e, 0x6f,
0x79, 0x20, 0x65, 0x74, 0x65, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x43,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20,
0x53, 0x69, 0x6c, 0x69, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75,
0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x53, 0x69,
0x6c, 0x69, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x74, 0x65, 0x20,
0x62, 0x75, 0x20, 0x53, 0x69, 0x6c, 0x69, 0x57, 0x61, 0x78, 0x74, 0x75,
0x20, 0x43, 0x68, 0x69, 0x6e, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20,
0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20,
0x43, 0x68, 0x69, 0x6e, 0x65, 0x43, 0x68, 0x69, 0x6e, 0x65, 0x20, 0x77,
0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0xeb, 0x63, 0x63, 0xeb, 0x67, 0x77,
0x61, 0x78, 0x74, 0x75, 0x20, 0x69, 0x6c, 0x65, 0x20, 0x62, 0x75, 0x20,
0x6e, 0x6f, 0x65, 0x6c, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x69, 0x6c,
0x65, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x57, 0x61, 0x78, 0x74, 0x75,
0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x57, 0x61, 0x78, 0x74,
0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63,
0x69, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x4a, 0x61, 0x6d,
0x6f, 0x6e, 0x6f, 0x79, 0x20, 0x65, 0x74, 0x65, 0x20, 0x63, 0x69, 0x20,
0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x57, 0x61, 0x78, 0x74, 0x75,
0x20, 0x49, 0x6c, 0x65, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x57, 0x61, 0x78,
0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20,
0x63, 0x69, 0x20, 0x49, 0x6c, 0x65, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x49,
0x6c, 0x65, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x78, 0x61, 0x61, 0x6a,
0x75, 0x20, 0x65, 0x74, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x43,
0x75, 0x62, 0x61, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1,
0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x43, 0x75, 0x62,
0x61, 0x43, 0x75, 0x62, 0x61, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20,
0x62, 0xeb, 0x63, 0x63, 0xeb, 0x67, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20,
0x44, 0x61, 0x76, 0x69, 0x73, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x69,
0x6c, 0x65, 0x20, 0x62, 0x75, 0x20, 0x50, 0xe2, 0x71, 0x75, 0x65, 0x73,
0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69,
0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x49, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x50, 0xe2, 0x71, 0x75, 0x65, 0x73, 0x4a, 0x61, 0x6d, 0x6f, 0x6e,
0x6f, 0x79, 0x20, 0x65, 0x74, 0x65, 0x20, 0x63, 0x69, 0x20, 0x49, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x50, 0xe2, 0x71, 0x75, 0x65, 0x73, 0x77,
0x61, 0x78, 0x74, 0x75, 0x20, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72,
0x43, 0x54, 0x45, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0xeb,
0x72, 0x6f, 0x6f, 0x70, 0x20, 0x73, 0xe0, 0x6e, 0x74, 0x61, 0x72, 0x61,
0x61, 0x6c, 0x29, 0x43, 0x45, 0x53, 0x54, 0x20, 0x28, 0x77, 0x61, 0x78,
0x74, 0x75, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x75, 0x20, 0xeb, 0x72, 0x6f, 0x6f, 0x70, 0x20, 0x73, 0xe0, 0x6e, 0x74,
0x61, 0x72, 0x61, 0x61, 0x6c, 0x29, 0x43, 0x45, 0x53, 0x54, 0x20, 0x28,
0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x74, 0x65, 0x20, 0x77, 0x75,
0x20, 0xeb, 0x72, 0x6f, 0x6f, 0x70, 0x20, 0x73, 0xe0, 0x6e, 0x74, 0x61,
0x72, 0x61, 0x61, 0x6c, 0x29, 0x45, 0x45, 0x54, 0x20, 0x28, 0x77, 0x61,
0x78, 0x74, 0x75, 0x20, 0xeb, 0x72, 0x6f, 0x6f, 0x70, 0x20, 0x75, 0x20,
0x70, 0x65, 0x6e, 0x6b, 0x75, 0x29, 0x45, 0x45, 0x53, 0x54, 0x20, 0x28,
0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x75, 0x20, 0xeb, 0x72, 0x6f, 0x6f, 0x70, 0x20, 0x75,
0x20, 0x70, 0x65, 0x6e, 0x6b, 0x75, 0x29, 0x45, 0x45, 0x53, 0x54, 0x20,
0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x74, 0x65, 0x20, 0x77,
0x75, 0x20, 0xeb, 0x72, 0x6f, 0x6f, 0x70, 0x20, 0x75, 0x20, 0x70, 0x65,
0x6e, 0x6b, 0x75, 0x29, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x70, 0x65,
0x6e, 0x6b, 0x75, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x57, 0x45,
0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0xeb, 0x72, 0x6f,
0x6f, 0x70, 0x20, 0x75, 0x20, 0x73, 0x6f, 0x77, 0x77, 0x75, 0x2d, 0x6a,
0x61, 0x6e, 0x74, 0x57, 0x45, 0x53, 0x54, 0x20, 0x28, 0x77, 0x61, 0x78,
0x74, 0x75, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x75, 0x20, 0xeb, 0x72, 0x6f, 0x6f, 0x70, 0x20, 0x75, 0x20, 0x73, 0x6f,
0x77, 0x77, 0x75, 0x2d, 0x6a, 0x61, 0x6e, 0x74, 0x29, 0x57, 0x45, 0x53,
0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x74, 0x65,
0x20, 0x77, 0x75, 0x20, 0xeb, 0x72, 0x6f, 0x6f, 0x70, 0x20, 0x75, 0x20,
0x73, 0x6f, 0x77, 0x77, 0x75, 0x2d, 0x6a, 0x61, 0x6e, 0x74, 0x29, 0x46,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x74, 0x69, 0x6d, 0x65,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x77, 0x61, 0x78,
0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x4a,
0x61, 0x6d, 0x6f, 0x6e, 0x6f, 0x79, 0x20, 0x65, 0x74, 0x65, 0x20, 0x63,
0x69, 0x20, 0x69, 0x6c, 0x65, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x46, 0x69, 0x64, 0x6a,
0x69, 0x4a, 0x61, 0x6d, 0x6f, 0x6e, 0x6f, 0x79, 0x20, 0x65, 0x74, 0x65,
0x20, 0x63, 0x69, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x57, 0x61, 0x78, 0x74,
0x75, 0x20, 0x53, 0x75, 0x64, 0x20, 0x61, 0x6b, 0x20, 0x41, 0x6e, 0x74,
0x61, 0x72, 0x63, 0x74, 0x69, 0x71, 0x75, 0x65, 0x20, 0x62, 0x75, 0x20,
0x46, 0x72, 0x61, 0x6e, 0xe7, 0x61, 0x69, 0x73, 0x77, 0x61, 0x78, 0x74,
0x75, 0x20, 0x67, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x57,
0x61, 0x78, 0x74, 0x75, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72,
0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x65, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x20, 0x77, 0x61, 0x78,
0x74, 0x75, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x20, 0x77, 0x61,
0x78, 0x74, 0x75, 0x20, 0x65, 0x74, 0x65, 0x77, 0x61, 0x78, 0x74, 0x75,
0x20, 0x69, 0x6c, 0x65, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74,
0x47, 0x4d, 0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x29, 0x77, 0x61, 0x78,
0x74, 0x75, 0x20, 0x70, 0x65, 0x6e, 0x6b, 0x75, 0x20, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20,
0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20,
0x70, 0x65, 0x6e, 0x6b, 0x75, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x74, 0x65,
0x20, 0x62, 0x75, 0x20, 0x70, 0x65, 0x6e, 0x6b, 0x75, 0x20, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x77, 0x61, 0x78, 0x74, 0x75,
0x20, 0x73, 0x6f, 0x77, 0x77, 0x75, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75,
0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x73, 0x6f,
0x77, 0x77, 0x75, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x74, 0x65, 0x20, 0x62,
0x75, 0x20, 0x73, 0x6f, 0x77, 0x77, 0x75, 0x20, 0x47, 0x72, 0x6f, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x47,
0x6f, 0x6c, 0x66, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x47, 0x75, 0x79,
0x61, 0x6e, 0x61, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e,
0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6a, 0x61,
0x67, 0x6c, 0x65, 0x65, 0x6c, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x57, 0x61, 0x78,
0x74, 0x75, 0x20, 0x62, 0xeb, 0x63, 0x63, 0xeb, 0x67, 0x20, 0x62, 0x75,
0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x69, 0x61, 0x6e, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x48, 0x6f,
0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x77, 0x61, 0x78, 0x74, 0x75,
0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69,
0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x4a, 0x61,
0x6d, 0x6f, 0x6e, 0x6f, 0x79, 0x20, 0x65, 0x74, 0x65, 0x20, 0x63, 0x69,
0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x48, 0x6f,
0x76, 0x64, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x48, 0x6f, 0x76, 0x64,
0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x77, 0x61, 0x78, 0x74,
0x75, 0x20, 0x65, 0x74, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x49,
0x6e, 0x64, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x67, 0xe9, 0x65,
0x6a, 0x75, 0x20, 0x49, 0x6e, 0x64, 0x65, 0x77, 0x61, 0x78, 0x74, 0x75,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x65, 0x57, 0x61,
0x78, 0x74, 0x75, 0x20, 0x45, 0x6e, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73,
0x69, 0x20, 0x62, 0x75, 0x20, 0x64, 0x69, 0x67, 0x67, 0x69, 0x20, 0x62,
0x69, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x70, 0x65, 0x6e, 0x6b, 0x75,
0x20, 0x45, 0x6e, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x77, 0x61,
0x78, 0x74, 0x75, 0x20, 0x73, 0x6f, 0x77, 0x77, 0x75, 0x20, 0x45, 0x6e,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x57, 0x61, 0x78, 0x74, 0x75,
0x20, 0x49, 0x72, 0x61, 0x6e, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x77, 0x61,
0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e,
0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0xeb, 0x63, 0x63, 0xeb, 0x67,
0x20, 0x63, 0x69, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x57, 0x61, 0x78, 0x74,
0x75, 0x20, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x77, 0x61, 0x78, 0x74,
0x75, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x6d, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x74, 0x65, 0x20,
0x62, 0x75, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57, 0x61,
0x78, 0x74, 0x75, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x57, 0x61,
0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e,
0x20, 0x63, 0x69, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x49, 0x73,
0x72, 0x61, 0x65, 0x6c, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62,
0xeb, 0x63, 0x63, 0xeb, 0x67, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x6a,
0x61, 0x70, 0x6f, 0x6e, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x20, 0x77, 0x61,
0x78, 0x74, 0x75, 0x20, 0x62, 0xeb, 0x63, 0x63, 0xeb, 0x67, 0x57, 0x61,
0x78, 0x74, 0x75, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74,
0x61, 0x61, 0x6e, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x70, 0x65, 0x6e, 0x6b, 0x75, 0x4b, 0x61, 0x7a, 0x61, 0x6b,
0x68, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x62, 0x75, 0x20, 0x73, 0x6f, 0x77,
0x77, 0x75, 0x20, 0x6a, 0x61, 0x6e, 0x74, 0x77, 0x61, 0x78, 0x74, 0x75,
0x20, 0x4b, 0x6f, 0x72, 0x65, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62,
0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x4b,
0x6f, 0x72, 0x65, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0xeb, 0x63,
0x63, 0xeb, 0x67, 0x20, 0x63, 0x69, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x57,
0x61, 0x78, 0x74, 0x75, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x57,
0x61, 0x78, 0x74, 0x75, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79,
0x61, 0x72, 0x73, 0x6b, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x73, 0x6b, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x63, 0x69, 0x20,
0x6a, 0x61, 0x6d, 0x6f, 0x6e, 0x6f, 0x79, 0x20, 0x65, 0x74, 0x65, 0x57,
0x61, 0x78, 0x74, 0x75, 0x20, 0x4b, 0x69, 0x72, 0x67, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x49, 0x6c, 0x65, 0x20,
0x4c, 0x69, 0x6e, 0x65, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20,
0x6d, 0x69, 0x69, 0x6e, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x61, 0x6e, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x2c, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x57, 0x61, 0x78, 0x74,
0x75, 0x20, 0x65, 0x74, 0x65, 0x20, 0x62, 0x75, 0x20, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x4d, 0x61, 0x6c, 0x61, 0x79, 0x73, 0x69, 0x65,
0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76,
0x65, 0x73, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x72, 0x71,
0x75, 0x65, 0x73, 0x61, 0x73, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x49,
0x6c, 0x65, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x77,
0x61, 0x78, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x65,
0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69,
0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63,
0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x74, 0x65, 0x20, 0x62,
0x75, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x65, 0x77, 0x61, 0x78,
0x74, 0x75, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x77, 0x61, 0x78,
0x74, 0x75, 0x20, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x62,
0x75, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x57, 0x61, 0x78, 0x74, 0x75,
0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69,
0x20, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x62, 0x75, 0x20,
0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20,
0x62, 0xeb, 0x63, 0x63, 0xeb, 0x67, 0x20, 0x63, 0x69, 0x20, 0x50, 0x61,
0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x71, 0x75,
0x65, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x61, 0x74, 0x61,
0x72, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x55,
0x6c, 0x61, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x61, 0x74, 0x61, 0x72, 0x20,
0x74, 0x69, 0x6d, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x4d, 0x6f,
0x73, 0x63, 0x6f, 0x77, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x20, 0x57,
0x61, 0x78, 0x74, 0x75, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x74,
0x65, 0x20, 0x62, 0x75, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x75, 0x77,
0x61, 0x78, 0x74, 0x75, 0x20, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72,
0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x77,
0x61, 0x78, 0x74, 0x75, 0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x57, 0x61,
0x78, 0x74, 0x75, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x69, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62,
0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x43,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x65, 0x20, 0x62, 0x75, 0x20,
0x62, 0x65, 0x65, 0x73, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x74,
0x65, 0x20, 0x62, 0x75, 0x20, 0x4e, 0x6f, 0x75, 0x76, 0x65, 0x6c, 0x6c,
0x65, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x65, 0x57,
0x61, 0x78, 0x74, 0x75, 0x20, 0x4e, 0x6f, 0x75, 0x76, 0x65, 0x6c, 0x6c,
0x65, 0x2d, 0x5a, 0xe9, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x57, 0x61, 0x78,
0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20,
0x63, 0x69, 0x20, 0x4e, 0x6f, 0x75, 0x76, 0x65, 0x6c, 0x6c, 0x65, 0x2d,
0x5a, 0xe9, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x77, 0x61, 0x78, 0x74, 0x75,
0x20, 0x54, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x4e, 0x65, 0x75, 0x76, 0x65,
0x54, 0x65, 0x72, 0x72, 0x65, 0x20, 0x4e, 0x65, 0x75, 0x76, 0x65, 0x57,
0x61, 0x78, 0x74, 0x75, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x77, 0x61, 0x78,
0x74, 0x75, 0x20, 0x69, 0x6c, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20,
0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x49, 0x6c, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x77, 0x61, 0x78, 0x74, 0x75,
0x20, 0x62, 0xeb, 0x63, 0x63, 0xeb, 0x67, 0x20, 0x63, 0x69, 0x20, 0x69,
0x6c, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x6e, 0x6f,
0x72, 0x6f, 0x6e, 0x68, 0x61, 0x56, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x6e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x73, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x57, 0x61, 0x78, 0x74, 0x75,
0x20, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20,
0x77, 0x61, 0x78, 0x74, 0x75, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x20, 0x63, 0x69, 0x20, 0x6a, 0x61, 0x6d, 0x6f,
0x6e, 0x6f, 0x79, 0x20, 0x65, 0x74, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75,
0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62,
0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x4f,
0x6d, 0x73, 0x6b, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x77, 0x61, 0x78, 0x74,
0x75, 0x20, 0x65, 0x74, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x78, 0x74, 0x75,
0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69,
0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x78,
0x74, 0x75, 0x20, 0x65, 0x74, 0x65, 0x20, 0x62, 0x75, 0x20, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20,
0x50, 0x61, 0x6c, 0x61, 0x75, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x50,
0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x70, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x79, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x50, 0x61, 0x72,
0x61, 0x67, 0x75, 0x61, 0x79, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20,
0x65, 0x74, 0x65, 0x50, 0x65, 0x72, 0x75, 0x20, 0x77, 0x61, 0x78, 0x74,
0x75, 0x50, 0x65, 0x72, 0x75, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20,
0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x50, 0x65, 0x72, 0x75,
0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x74, 0x65, 0x66, 0x69,
0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x65, 0x73, 0x20, 0x77, 0x61, 0x78,
0x74, 0x75, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20,
0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x66, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x69, 0x6e, 0x4a, 0x61, 0x6d, 0x6f, 0x6e, 0x6f, 0x79, 0x20,
0x65, 0x74, 0x65, 0x20, 0x63, 0x69, 0x20, 0x50, 0x68, 0x69, 0x6c, 0x69,
0x70, 0x70, 0x69, 0x6e, 0x65, 0x73, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20,
0x69, 0x6c, 0x65, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20,
0x61, 0x6b, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20,
0x61, 0x6e, 0x64, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69,
0x72, 0x6e, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x50, 0x6f, 0x6e, 0x61,
0x70, 0x65, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x50, 0x79, 0x6f, 0x6e,
0x67, 0x79, 0x61, 0x6e, 0x67, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x6e,
0x64, 0x61, 0x6a, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x52, 0x6f,
0x74, 0x65, 0x72, 0x61, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x53, 0x61,
0x78, 0x61, 0x6c, 0x69, 0x6e, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x77, 0x61,
0x78, 0x74, 0x75, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0xeb, 0x63,
0x63, 0xeb, 0x67, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x53, 0x65, 0x79,
0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x77, 0x61, 0x78, 0x74, 0x75,
0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69,
0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x75, 0x72, 0x57, 0x61,
0x78, 0x74, 0x75, 0x20, 0x49, 0x6c, 0x65, 0x20, 0x53, 0x6f, 0x6c, 0x6f,
0x6d, 0x6f, 0x6e, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x20, 0x64,
0x75, 0x20, 0x53, 0x75, 0x64, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x53,
0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20,
0x73, 0x79, 0x6f, 0x77, 0x61, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x54,
0x61, 0x68, 0x69, 0x74, 0x69, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62,
0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20,
0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x6c, 0x65, 0x65, 0x72, 0x61, 0x6c,
0x75, 0x20, 0x62, 0x69, 0x73, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x54,
0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x54, 0x6f,
0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x74, 0x69, 0x6d, 0x65, 0x57, 0x61,
0x78, 0x74, 0x75, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1,
0x20, 0x6d, 0x69, 0x69, 0x6e, 0x4a, 0x61, 0x6d, 0x6f, 0x6e, 0x6f, 0x79,
0x20, 0x65, 0x74, 0x65, 0x20, 0x63, 0x69, 0x20, 0x54, 0x6f, 0x6e, 0x67,
0x61, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b,
0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20,
0x65, 0x74, 0x65, 0x20, 0x62, 0x75, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x78, 0x74, 0x75,
0x20, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x57, 0x61, 0x78, 0x74, 0x75,
0x20, 0x55, 0x72, 0x75, 0x67, 0x77, 0x61, 0x79, 0x55, 0x72, 0x75, 0x67,
0x75, 0x61, 0x79, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75,
0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61,
0x79, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x74, 0x65, 0x57,
0x61, 0x78, 0x74, 0x75, 0x20, 0x4f, 0x75, 0x7a, 0x62, 0xe9, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75,
0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x4f, 0x75,
0x7a, 0x62, 0xe9, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x78,
0x74, 0x75, 0x20, 0x65, 0x74, 0x65, 0x20, 0x62, 0x75, 0x20, 0x4f, 0x75,
0x7a, 0x62, 0xe9, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x78,
0x74, 0x75, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x57, 0x61,
0x78, 0x74, 0x75, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x57, 0x61, 0x78, 0x74,
0x75, 0x20, 0x65, 0x74, 0x65, 0x20, 0x62, 0x75, 0x20, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x56, 0x65,
0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x57, 0x61, 0x78, 0x74, 0x75,
0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20,
0x63, 0x69, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x63, 0x69, 0x20, 0x6a,
0x61, 0x6d, 0x6f, 0x6e, 0x6f, 0x79, 0x20, 0x65, 0x74, 0x65, 0x57, 0x61,
0x78, 0x74, 0x75, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x77,
0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69,
0x6e, 0x4a, 0x61, 0x6d, 0x6f, 0x6e, 0x6f, 0x79, 0x20, 0x65, 0x74, 0x65,
0x20, 0x62, 0x75, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x49, 0x6c, 0x65, 0x20, 0x57,
0x61, 0x6b, 0x65, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x57,
0x61, 0x78, 0x74, 0x75, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65,
0x74, 0x65, 0x20, 0x62, 0x75, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x59, 0x65, 0x6b, 0x61,
0x74, 0xe9, 0x72, 0x69, 0x6e, 0x62, 0x6f, 0x75, 0x72, 0x67, 0x57, 0x61,
0x78, 0x74, 0x75, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x41, 0x70, 0x69,
0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x70, 0x69, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x41, 0x70, 0x69, 0x61, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68,
0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73,
0x73, 0x61, 0x6c, 0x61, 0x6d, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x6f,
0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x48,
0x61, 0x6c, 0x66, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019,
0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x45, 0x61, 0x73, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69,
0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x41, 0x66, 0x67, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0xe1, 0x72, 0xed, 0x6e,
0x20, 0x41, 0x66, 0xed, 0x72, 0x69, 0x6b, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xcc, 0x6c, 0xe0, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20,
0x41, 0x66, 0xed, 0x72, 0x69, 0x6b, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xcc, 0x77, 0x1ecd, 0x300, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20,
0x41, 0x66, 0xed, 0x72, 0x69, 0x6b, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xcc, 0x77, 0x1ecd, 0x300, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20,
0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x41, 0x66, 0xed, 0x72,
0x69, 0x6b, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x1ecd,
0x300, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x4f, 0x6f, 0x72, 0x75,
0x20, 0x41, 0x66, 0xed, 0x72, 0x69, 0x6b, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x41, 0x6c, 0xe1, 0x73, 0xed, 0x6b, 0xe0, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x41,
0x6c, 0xe1, 0x73, 0xed, 0x6b, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x4f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x301, 0x20, 0x41, 0x6c, 0xe1, 0x73, 0xed,
0x6b, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x41, 0x6d, 0xe1, 0x73,
0x1ecd, 0x301, 0x6e, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x41, 0x66,
0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x41, 0x6d, 0xe1, 0x73, 0x1ecd, 0x301,
0x6e, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x72, 0x75, 0x20,
0x41, 0x6d, 0xe1, 0x73, 0x1ecd, 0x301, 0x6e, 0xec, 0xe0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xe0, 0xe1, 0x72, 0xed, 0x6e, 0x20, 0x67, 0x62, 0xf9, 0x6e,
0x67, 0x62, 0xf9, 0x6e, 0xe0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x61, 0x73,
0x69, 0x6b, 0x6f, 0x20, 0xe0, 0xe1, 0x72, 0xed, 0x6e, 0x20, 0x67, 0x62,
0xf9, 0x6e, 0x67, 0x62, 0xf9, 0x6e, 0x41, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0xe0, 0xe1, 0x72, 0xed, 0x6e, 0x20, 0x67, 0x62, 0xf9, 0x6e, 0x67, 0x62,
0xf9, 0x6e, 0x20, 0x6f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xec, 0x68, 0xe0, 0x20, 0xec, 0x6c, 0xe0, 0x20,
0x6f, 0xf2, 0x72, 0xf9, 0x6e, 0x41, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0,
0x73, 0x69, 0x6b, 0xf2, 0x20, 0xcc, 0x68, 0x61, 0x20, 0xcc, 0x6c, 0x61,
0x20, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x6f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20, 0xcc, 0x68, 0xe0, 0x20,
0xcc, 0x6c, 0xe0, 0x20, 0x4f, 0xf2, 0x72, 0x75, 0x6e, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0xf2, 0x6b, 0xe8, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x61, 0x73, 0x69, 0x6b, 0x6f, 0x20, 0xf2, 0x6b, 0xe8, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x6f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20, 0x4f,
0x72, 0x69, 0x2d, 0xf2, 0x6b, 0xe8, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x50, 0xe0, 0x73, 0xed, 0x66, 0xed, 0xec, 0x6b, 0xec, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0xe0, 0x73, 0x69, 0x6b, 0xf2, 0x20, 0x50, 0xe0, 0x73,
0xed, 0x66, 0xed, 0xec, 0x6b, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0xcc, 0x79, 0xe1, 0x6c, 0x1eb9, 0x74, 0x61, 0x20, 0x50, 0xe0, 0x73, 0xed,
0x66, 0xed, 0xec, 0x6b, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x41,
0x70, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9,
0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x41, 0x70, 0x69, 0x61, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20, 0x41,
0x70, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x41, 0x72, 0x61,
0x62, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9,
0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd,
0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x41, 0x61, 0x67, 0x6f, 0x20,
0x41, 0x6a, 0x1eb9, 0x6e, 0x74, 0xec, 0x6e, 0xe0, 0x41, 0x61, 0x67, 0x6f,
0x20, 0xe0, 0x73, 0xec, 0x6b, 0xf2, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x41, 0x61, 0x67, 0x6f, 0x20, 0x53, 0x6f, 0x6d,
0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x49, 0x77, 0x1ecd, 0x301, 0x2d, 0x6f, 0xf2,
0x72, 0xf9, 0x6e, 0x20, 0x41, 0x6a, 0x1eb9, 0x301, 0x6e, 0x74, 0xed, 0x6e,
0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x49, 0x77, 0x1ecd, 0x301, 0x2d,
0x6f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b,
0xf2, 0x20, 0x41, 0x6a, 0x1eb9, 0x301, 0x6e, 0x74, 0xed, 0x6e, 0xe0, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x72, 0x75, 0x20, 0x49, 0x77, 0x1ecd,
0x301, 0x2d, 0x6f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x41, 0x6a, 0x1eb9, 0x301,
0x6e, 0x74, 0xed, 0x6e, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x41,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d,
0xe0, 0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xc0, 0x74, 0xec, 0x6c, 0xe1, 0x144, 0x74, 0xed, 0xec, 0x6b,
0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xe0, 0x73, 0x69, 0x6b, 0xf2,
0x20, 0xc0, 0x74, 0xec, 0x6c, 0xe1, 0x144, 0x74, 0xed, 0xec, 0x6b, 0xec,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x79, 0xe1, 0x6c, 0x1eb9, 0x74,
0x61, 0x20, 0xc0, 0x74, 0xec, 0x6c, 0xe1, 0x144, 0x74, 0xed, 0xec, 0x6b,
0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0xe0, 0x72, 0x69, 0x6e,
0x20, 0x47, 0xf9, 0x6e, 0x67, 0xf9, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0,
0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0xc0, 0xe0, 0x72, 0x69, 0x6e,
0x20, 0x47, 0xf9, 0x6e, 0x67, 0xf9, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f,
0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20, 0xc0, 0xe0, 0x72, 0x69, 0x6e,
0x20, 0x47, 0xf9, 0x6e, 0x67, 0xf9, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0,
0xe0, 0x72, 0x69, 0x6e, 0x20, 0x47, 0xf9, 0x6e, 0x67, 0xf9, 0x6e, 0x20,
0xcc, 0x77, 0x1ecd, 0x300, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0xc0, 0xe0,
0x72, 0x69, 0x6e, 0x20, 0x47, 0xf9, 0x6e, 0x67, 0xf9, 0x6e, 0x20, 0xcc,
0x77, 0x1ecd, 0x300, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20, 0xc0, 0xe0, 0x72,
0x69, 0x6e, 0x20, 0x47, 0xf9, 0x6e, 0x67, 0xf9, 0x6e, 0x20, 0xcc, 0x77,
0x1ecd, 0x300, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0xcc, 0x6c, 0xe0, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0xcc, 0x6c, 0xe0,
0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a,
0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20, 0xcc, 0x6c, 0xe0, 0x2d, 0x4f, 0xf2,
0x72, 0xf9, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x1ecd, 0x300, 0x2d,
0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9,
0x6e, 0x75, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x1ecd, 0x300, 0x2d, 0x4f, 0xf2,
0x72, 0xf9, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x1ecd,
0x6d, 0x1ecd, 0x20, 0xcc, 0x77, 0x1ecd, 0x300, 0x2d, 0x4f, 0xf2, 0x72, 0xf9,
0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69,
0x6a, 0x61, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9,
0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69,
0x6a, 0x61, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d,
0xe0, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc1, 0x73, 0x1ecd, 0x301, 0x73, 0xec,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b,
0xf2, 0x20, 0xc1, 0x73, 0x1ecd, 0x301, 0x73, 0xec, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x4f, 0x6f, 0x72, 0x75, 0x20, 0xc1, 0x73, 0x1ecd, 0x301, 0x73,
0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x73, 0x68, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0,
0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x73, 0x68, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53,
0x1ecd, 0x6d, 0xe0, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x73, 0x68, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x42, 0x68, 0x75, 0x74,
0x61, 0x6e, 0x41, 0x61, 0x67, 0x6f, 0x20, 0x42, 0x6f, 0x6c, 0x69, 0x76,
0x69, 0x61, 0x41, 0x61, 0x67, 0x6f, 0x20, 0x42, 0xf9, 0x72, 0xe0, 0x73,
0xed, 0x6c, 0xed, 0xe0, 0x41, 0x61, 0x67, 0x6f, 0x20, 0xe0, 0x73, 0xec,
0x6b, 0xf2, 0x20, 0x42, 0xf9, 0x72, 0xe0, 0x73, 0xed, 0x6c, 0xed, 0xe0,
0x41, 0x61, 0x67, 0x6f, 0x20, 0x53, 0x6f, 0x6d, 0x61, 0x20, 0x42, 0x72,
0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x4b, 0xe9, 0x70, 0xfa, 0x20, 0x46, 0xe1, 0xe0, 0x64, 0xec, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20,
0x4b, 0xe9, 0x70, 0xfa, 0x20, 0x46, 0xe1, 0xe0, 0x64, 0xec, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x1eb8, 0x300, 0x72, 0xfa, 0x6e, 0x20, 0x4b, 0xe9,
0x70, 0xfa, 0x20, 0x46, 0xe1, 0xe0, 0x64, 0xec, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x43, 0x68,
0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x43, 0x68, 0x61, 0x74, 0x61, 0x6d, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x43, 0x68, 0x61, 0x74,
0x61, 0x6d, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d,
0x1ecd, 0x6d, 0x1ecd, 0x20, 0x43, 0x68, 0x61, 0x74, 0x61, 0x6d, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x1e62, 0xed, 0x6c, 0xe8, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x1e62, 0xed,
0x6c, 0xe8, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x72, 0x75, 0x20,
0x1e62, 0xed, 0x6c, 0xe8, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x1e62, 0xe1,
0xed, 0x6e, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x66, 0x1eb9,
0x6e, 0x75, 0x6b, 0xf2, 0x73, 0xed, 0x20, 0x1e62, 0xe1, 0xed, 0x6e, 0xe0,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d,
0x1ecd, 0x20, 0x1e62, 0xe1, 0xed, 0x6e, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x45, 0x72, 0xe9, 0x6b, 0xf9, 0x73, 0xf9, 0x20, 0x43, 0x68, 0x72,
0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0xc0, 0x77, 0x1ecd, 0x6e, 0x20, 0x45, 0x72, 0xe9, 0x6b, 0xf9, 0x73, 0xf9,
0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x41, 0x61, 0x67, 0x6f, 0x20, 0x4b,
0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x41, 0x61, 0x67, 0x6f, 0x20,
0xe0, 0x73, 0xec, 0x6b, 0xf2, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x61, 0x41, 0x61, 0x67, 0x6f, 0x20, 0x53, 0x6f, 0x6d, 0x61, 0x20,
0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xc0, 0x77, 0x1ecd, 0x6e, 0x20, 0x45, 0x72, 0x65, 0x6b, 0x75,
0x73, 0x75, 0x20, 0x4b, 0x75, 0x75, 0x6b, 0x75, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0xc0, 0x77,
0x1ecd, 0x6e, 0x20, 0x45, 0x72, 0x65, 0x6b, 0x75, 0x73, 0x75, 0x20, 0x4b,
0x75, 0x75, 0x6b, 0x75, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x49, 0x6c,
0x61, 0x6a, 0x69, 0x20, 0x53, 0x1ecd, 0x6d, 0x61, 0x20, 0xc0, 0x77, 0x1ecd,
0x6e, 0x20, 0x45, 0x72, 0x65, 0x6b, 0x75, 0x73, 0x75, 0x20, 0x4b, 0x75,
0x75, 0x6b, 0x75, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4b, 0xfa, 0x62,
0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75,
0x6b, 0xf2, 0x20, 0x4b, 0xfa, 0x62, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20, 0x4b, 0xfa, 0x62,
0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74,
0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x6c, 0xe0, 0x20, 0x6f, 0x6f, 0x72, 0xf9,
0x6e, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x41, 0x61, 0x67, 0x6f, 0x20,
0x41, 0x6a, 0x6e, 0x64, 0x65, 0x20, 0x49, 0x62, 0xf9, 0x67, 0x62, 0xe9,
0x20, 0x4f, 0x6d, 0x69, 0x41, 0x61, 0x67, 0x6f, 0x20, 0xe0, 0x73, 0xec,
0x6b, 0xf2, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x41, 0x61, 0x67, 0x6f, 0x20, 0x53, 0x6f, 0x6d,
0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x41, 0x61, 0x67, 0x6f, 0x20, 0x45, 0x63, 0x75, 0x61,
0x64, 0x6f, 0x72, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0xe1, 0x72,
0x69, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0xc0, 0xe1, 0x72, 0x69, 0x6e, 0x20, 0xe0, 0x73, 0x69,
0x6b, 0xf2, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0xc0, 0xe1, 0x72, 0x69, 0x6e, 0x20, 0x53, 0x1ecd, 0x6d,
0xe0, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xcc, 0x68, 0xe0, 0x20, 0xcc, 0x6c, 0xe0, 0x20, 0x4f, 0xf2,
0x72, 0xf9, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xe0, 0x73, 0x69, 0x6b, 0xf2, 0x20, 0xcc, 0x68,
0xe0, 0x20, 0xcc, 0x6c, 0xe0, 0x20, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0xcc, 0x68, 0x61, 0x20, 0xcc, 0x6c, 0xe0,
0x20, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x49, 0x77, 0xe1, 0x6a, 0xfa,
0x20, 0xcc, 0x6c, 0xe0, 0x20, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x59,
0xfa, 0x72, 0xf3, 0xf2, 0x70, 0xf9, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0xcc, 0x77, 0x1ecd, 0x20, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xe0, 0x73,
0x69, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x1ecd, 0x20, 0x4f, 0xf2, 0x72, 0xf9,
0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0xcc, 0x68, 0xe0, 0x20, 0xcc,
0x77, 0x1ecd, 0x20, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x46, 0xf3, 0x6b,
0xed, 0x6c, 0xe1, 0x6e, 0x64, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x45, 0x74, 0xed, 0x6b,
0x75, 0x6e, 0x20, 0x46, 0xf3, 0x6b, 0xed, 0x6c, 0xe1, 0x6e, 0x64, 0xec,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6f, 0x72, 0x75, 0x20, 0x45,
0x74, 0xed, 0x6b, 0x75, 0x6e, 0x20, 0x46, 0xf3, 0x6b, 0xed, 0x6c, 0xe1,
0x6e, 0x64, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x46, 0x69, 0x6a,
0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75,
0x6b, 0xf2, 0x20, 0x46, 0x69, 0x6a, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x53, 0x1ecd, 0x6d, 0x61, 0x20, 0x46, 0x69, 0x6a, 0x69, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x47, 0xfa, 0x79, 0xe1, 0x6e, 0xe0, 0x20, 0x46,
0xe1, 0x72, 0xe1, 0x6e, 0x73, 0xe8, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x47, 0xfa, 0x73, 0xf9, 0x20, 0x46, 0xe1, 0x72, 0x61, 0x6e, 0x73, 0xe9,
0x20, 0xe0, 0x74, 0x69, 0x20, 0xc0, 0x6e, 0x74, 0xe1, 0x74, 0xed, 0xec,
0x6b, 0xec, 0x41, 0x61, 0x67, 0x6f, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x70,
0x61, 0x67, 0x6f, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x47, 0x61, 0x6d,
0x62, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66,
0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xc0, 0x77, 0x1ecd, 0x6e, 0x20, 0x45, 0x72, 0x65, 0x6b, 0x75, 0x73,
0x75, 0x20, 0x47, 0x69, 0x6c, 0x69, 0x62, 0x61, 0x74, 0x69, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x6c, 0xe0, 0x20, 0x6f, 0x6f, 0x72, 0xf9,
0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x1ecd, 0x300, 0x20, 0xcc, 0x66,
0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x6f, 0x6f, 0x72, 0xf9, 0x6e, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0xec, 0x67, 0x62, 0xe0, 0x20, 0x4f, 0x6f, 0x72, 0x75,
0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x1ecd, 0x300, 0x20, 0x6f, 0x6f, 0x72,
0xf9, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b,
0xf2, 0x20, 0xcc, 0x77, 0x1ecd, 0x300, 0x20, 0x4f, 0xf2, 0x72, 0xf9, 0x6e,
0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20,
0xcc, 0x67, 0x62, 0x61, 0x20, 0x4f, 0xf2, 0x72, 0x75, 0x20, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x47, 0x75, 0x6c,
0x66, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x47, 0xfa, 0x79, 0xe0, 0x6e,
0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69,
0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20,
0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x69, 0x61, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa,
0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2,
0x73, 0xed, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0x48,
0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x48, 0x6f, 0x66, 0x69, 0x64, 0x69, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xcc, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x73, 0xed, 0x20,
0x48, 0x6f, 0x66, 0x69, 0x64, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0x48, 0x6f, 0x66, 0x69, 0x64, 0x69, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x45, 0x74, 0xed, 0x6b, 0x75, 0x6e, 0x20, 0xcd, 0x6e, 0x64, 0xed, 0xe0,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x63, 0x68,
0x69, 0x6e, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0xe0, 0x72,
0x69, 0x6e, 0x20, 0x47, 0x62, 0xf9, 0x6e, 0x67, 0x62, 0xf9, 0x6e, 0x20,
0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0xcc, 0x6c, 0xe0, 0x20, 0x6f, 0x6f, 0x72, 0xf9, 0x6e,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x1ecd, 0x300, 0x20, 0x6f, 0x6f, 0x72,
0xf9, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x69, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2,
0x20, 0x49, 0x72, 0x61, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x4f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20, 0x49, 0x72, 0x61, 0x6e,
0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcd, 0x6b, 0xf3, 0x73, 0xed,
0x6b, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e,
0x75, 0x6b, 0xf2, 0x20, 0xcd, 0x6b, 0xf3, 0x73, 0xed, 0x6b, 0xec, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0xcd, 0x6b,
0xfa, 0x73, 0xed, 0x6b, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x49,
0x73, 0x72, 0x61, 0x65, 0x6c, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0,
0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65,
0x6c, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x1ecd,
0x6d, 0x1ecd, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xcc, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x73, 0xed, 0x20,
0x4a, 0x61, 0x70, 0x61, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53,
0x1ecd, 0x6d, 0xe0, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x41, 0x61, 0x67,
0x6f, 0x20, 0x4b, 0x61, 0x73, 0x61, 0x73, 0x69, 0x74, 0xe1, 0xe0, 0x6e,
0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x6c, 0xe0, 0x2d, 0x4f,
0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x4b, 0x61, 0x73, 0x61, 0x73, 0x69, 0x74,
0xe1, 0xe0, 0x6e, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x77,
0x1ecd, 0x300, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x4b, 0x61, 0x73,
0x61, 0x73, 0x69, 0x74, 0xe1, 0xe0, 0x6e, 0xec, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x4b, 0x6f, 0x72, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xcc, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x73, 0xed, 0x20, 0x4b,
0x6f, 0x72, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a,
0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20, 0x4b, 0x6f, 0x72, 0x69, 0x61, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4b, 0x6f, 0x73, 0x69, 0x72, 0x61, 0x69,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x69, 0x6e,
0x6f, 0x79, 0x61, 0x73, 0x69, 0x6b, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4b, 0x72, 0x61,
0x73, 0x69, 0x6e, 0x6f, 0x79, 0x61, 0x73, 0x69, 0x6b, 0x69, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0x4b, 0x72, 0x61,
0x73, 0x69, 0x6e, 0x6f, 0x79, 0x61, 0x73, 0x69, 0x6b, 0x69, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x4b, 0x69, 0x72, 0x69, 0x67, 0x69, 0x73, 0x69,
0x74, 0x61, 0x61, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0,
0x77, 0x1ecd, 0x6e, 0x20, 0x45, 0x72, 0x65, 0x6b, 0x75, 0x73, 0x75, 0x20,
0x4c, 0x61, 0x69, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4d, 0x61, 0x6c, 0x61, 0x79,
0x73, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4d, 0x61, 0x6c,
0x64, 0x69, 0x76, 0x65, 0x73, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4d,
0x61, 0x6b, 0x75, 0x65, 0x73, 0x61, 0x73, 0x69, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xc0, 0x77, 0x1ecd, 0x6e, 0x20, 0x45, 0x72, 0x65, 0x6b, 0x75,
0x73, 0x75, 0x20, 0x4d, 0x61, 0x73, 0x61, 0x61, 0x6c, 0x69, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x4d, 0xe1, 0x72, 0xed, 0x1e63, 0xfa, 0x1e63, 0xec,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b,
0xf2, 0x20, 0x4d, 0xe1, 0x72, 0xed, 0x1e63, 0xfa, 0x1e63, 0xec, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6f, 0x72, 0x75, 0x20, 0x4d, 0xe1, 0x72,
0xed, 0x1e63, 0xfa, 0x1e63, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4d,
0x61, 0x77, 0x73, 0x6f, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x50,
0xe1, 0x73, 0xed, 0x66, 0xed, 0xec, 0x6b, 0xec, 0x20, 0x4d, 0x1eb9, 0x301,
0x1e63, 0xed, 0x6b, 0xf2, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66,
0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x50, 0xe1, 0x73, 0xed, 0x66, 0xed,
0xec, 0x6b, 0xec, 0x20, 0x4d, 0x1eb9, 0x301, 0x73, 0xed, 0x6b, 0xf2, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd,
0x20, 0x50, 0xe1, 0x73, 0xed, 0x66, 0xed, 0xec, 0x6b, 0xec, 0x20, 0x4d,
0x1eb9, 0x301, 0x73, 0xed, 0x6b, 0xf2, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2,
0x73, 0xed, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74,
0x61, 0x72, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0,
0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4d, 0x6f, 0x73, 0x69, 0x6b, 0x6f,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b,
0xf2, 0x20, 0x4d, 0x6f, 0x73, 0x69, 0x6b, 0x6f, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0x4d, 0x6f, 0x73, 0x69, 0x6b,
0x6f, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x6c, 0xe0, 0x20, 0x4d,
0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x4e, 0x61, 0x75, 0x72, 0x75, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4e,
0x65, 0x70, 0x61, 0x6c, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4b,
0x61, 0x6c, 0x69, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x54, 0x75, 0x6e,
0x74, 0x75, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9,
0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4b, 0x61, 0x6c, 0x69, 0x64, 0x6f, 0x6e,
0x69, 0x61, 0x20, 0x54, 0x75, 0x6e, 0x74, 0x75, 0x6e, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0x61, 0x20, 0x4b, 0x61, 0x6c, 0x69,
0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x54, 0x75, 0x6e, 0x74, 0x75, 0x6e,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65,
0x61, 0x6c, 0x61, 0x6e, 0x64, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0,
0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x7a,
0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x4f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20, 0x4e, 0x65, 0x77, 0x20,
0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e,
0x64, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75,
0x6b, 0xf2, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c,
0x61, 0x6e, 0x64, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa,
0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e,
0x64, 0x6c, 0x61, 0x6e, 0x64, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4e,
0x69, 0x75, 0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x45, 0x72, 0xe9,
0x6b, 0xf9, 0x73, 0xf9, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b,
0xf2, 0x20, 0x45, 0x72, 0xe9, 0x6b, 0xf9, 0x73, 0xf9, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f,
0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20, 0x45, 0x72, 0xe9, 0x6b, 0xf9,
0x73, 0xf9, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x41, 0x61,
0x67, 0x6f, 0x20, 0x46, 0x65, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x61, 0x41, 0x61, 0x67, 0x6f,
0x20, 0xe0, 0x73, 0xec, 0x6b, 0xf2, 0x20, 0x46, 0x65, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x61,
0x41, 0x61, 0x67, 0x6f, 0x20, 0x53, 0x6f, 0x6d, 0x61, 0x20, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x68, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4e,
0x6f, 0x66, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x69, 0x73, 0x6b, 0x69,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b,
0xf2, 0x20, 0x4e, 0x6f, 0x66, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x69,
0x6b, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0,
0x20, 0x4e, 0x6f, 0x66, 0x6f, 0x72, 0x69, 0x62, 0x69, 0x73, 0x69, 0x6b,
0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6d, 0x69, 0x73, 0x69,
0x6b, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e,
0x75, 0x6b, 0xf2, 0x20, 0x4f, 0x6d, 0x69, 0x73, 0x69, 0x6b, 0x69, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0x4f, 0x6d,
0x69, 0x73, 0x69, 0x6b, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x50, 0x61, 0x6c,
0x61, 0x75, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x50, 0x61, 0x70, 0x75,
0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x50, 0xe1, 0x72, 0xe1, 0x67, 0xfa,
0x77, 0xe8, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e,
0x75, 0x6b, 0xf2, 0x20, 0x50, 0xe1, 0x72, 0xe1, 0x67, 0xfa, 0x77, 0xe8,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6f, 0x72, 0x75, 0x20, 0x50,
0xe1, 0x72, 0xe1, 0x67, 0xfa, 0x77, 0xe8, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x50, 0xe9, 0x72, 0xf9, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0,
0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x50, 0xe9, 0x72, 0xf9, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6f, 0x72, 0x75, 0x20, 0x50, 0xe9,
0x72, 0xf9, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x69, 0x6e, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x46, 0x69, 0x6c,
0x69, 0x70, 0x69, 0x6e, 0x69, 0x6e, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x69, 0x6e, 0x69, 0x6e, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0xc0, 0x77, 0x1ecd, 0x6e, 0x20, 0x45, 0x72, 0xe9, 0x6b, 0xf9, 0x73, 0xf9,
0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d,
0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d,
0x1ecd, 0x6d, 0x1ecd, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61,
0x6e, 0x67, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x52, 0x1eb9, 0x301,
0x79, 0xfa, 0x6e, 0xed, 0x1ecd, 0x301, 0x6e, 0xec, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2,
0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9,
0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53,
0xe8, 0x1e63, 0x1eb9, 0x301, 0x1eb9, 0x300, 0x6c, 0xec, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x53, 0x69,
0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xc0, 0x77, 0x1ecd, 0x6e, 0x20, 0x45, 0x72, 0x65, 0x6b, 0x75, 0x73,
0x75, 0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x47, 0xfa, 0x73, 0xf9, 0x20, 0x4a, 0x1ecd, 0x301, 0x6a,
0xed, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0xfa, 0x72, 0xed,
0x6e, 0xe1, 0x6d, 0xf9, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x79,
0x6f, 0x77, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x54, 0x61, 0x68,
0x69, 0x74, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x54, 0x61, 0x69,
0x70, 0x65, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x66, 0x1eb9,
0x6e, 0x75, 0x6b, 0xf2, 0x73, 0xed, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x1ecd,
0x6d, 0x1ecd, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x54, 0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x69, 0x74,
0x61, 0x61, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x54, 0x6f,
0x6b, 0x65, 0x6c, 0x61, 0x75, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66,
0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x43, 0x68, 0x75,
0x75, 0x6b, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x54, 0x75, 0x72, 0x6b,
0x69, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x69, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0,
0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x54, 0x75, 0x66, 0x61,
0x6c, 0x75, 0x41, 0x61, 0x67, 0x6f, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x79, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e,
0x75, 0x6b, 0xf2, 0x20, 0xda, 0x72, 0xfa, 0x67, 0xfa, 0x77, 0xe8, 0x41,
0x61, 0x67, 0x6f, 0x20, 0x53, 0x6f, 0x6d, 0x61, 0x20, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x79, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x55, 0x73,
0x69, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20,
0x55, 0x73, 0x69, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0x55,
0x73, 0x69, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x46, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b,
0xf2, 0x20, 0x46, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0x46, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x41, 0x61, 0x67, 0x6f, 0x20, 0x56, 0x65, 0x6e, 0x65,
0x7a, 0x75, 0x65, 0x6c, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x46,
0x69, 0x6c, 0x61, 0x64, 0x69, 0x66, 0x6f, 0x73, 0x69, 0x74, 0x6f, 0x6b,
0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75,
0x6b, 0xf2, 0x20, 0x46, 0x69, 0x6c, 0x61, 0x64, 0x69, 0x66, 0x6f, 0x73,
0x69, 0x74, 0x6f, 0x6b, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53,
0x1ecd, 0x6d, 0xe0, 0x20, 0x46, 0x69, 0x6c, 0x61, 0x64, 0x69, 0x66, 0x6f,
0x73, 0x69, 0x74, 0x6f, 0x6b, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x46, 0x6f, 0x6c, 0x69, 0x67, 0x6f, 0x67, 0x69, 0x72, 0x61, 0x64, 0x69,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b,
0xf2, 0x20, 0x46, 0x6f, 0x6c, 0x69, 0x67, 0x6f, 0x67, 0x69, 0x72, 0x61,
0x64, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0,
0x20, 0x46, 0x6f, 0x6c, 0x69, 0x67, 0x6f, 0x67, 0x69, 0x72, 0x61, 0x64,
0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x45, 0x72, 0xe9, 0x6b, 0xf9,
0x73, 0xf9, 0x20, 0x57, 0x61, 0x6b, 0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x26, 0x20, 0x46, 0x75,
0x74, 0x75, 0x69, 0x6e, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x59,
0x61, 0x74, 0x75, 0x74, 0x73, 0x6b, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x59, 0x61, 0x74, 0x75,
0x74, 0x73, 0x6b, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d,
0xe0, 0x20, 0x59, 0x61, 0x74, 0x75, 0x74, 0x73, 0x6b, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66,
0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x59, 0xfa, 0x6b, 0x1ecd, 0x301, 0x6e, 0xec, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xcc, 0x77, 0x254, 0x300, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e,
0x20, 0x41, 0x66, 0xed, 0x72, 0x69, 0x6b, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xcc, 0x77, 0x254, 0x300, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e,
0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x41, 0x66, 0xed,
0x72, 0x69, 0x6b, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x77,
0x254, 0x300, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x4f, 0x6f, 0x72,
0x75, 0x20, 0x41, 0x66, 0xed, 0x72, 0x69, 0x6b, 0xe0, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x41,
0x6c, 0xe1, 0x73, 0xed, 0x6b, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x4f, 0x6a, 0xfa, 0x6d, 0x254, 0x301, 0x20, 0x41, 0x6c, 0xe1, 0x73, 0xed,
0x6b, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x41, 0x6d, 0xe1, 0x73,
0x254, 0x301, 0x6e, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x41, 0x66,
0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x41, 0x6d, 0xe1, 0x73, 0x254, 0x301,
0x6e, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x72, 0x75, 0x20,
0x41, 0x6d, 0xe1, 0x73, 0x254, 0x301, 0x6e, 0xec, 0x41, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xe0, 0xe1, 0x72, 0xed, 0x6e, 0x20, 0x67, 0x62, 0xf9, 0x6e,
0x67, 0x62, 0xf9, 0x6e, 0x20, 0x6f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x6f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d,
0x254, 0x20, 0xcc, 0x68, 0xe0, 0x20, 0xcc, 0x6c, 0xe0, 0x20, 0x4f, 0xf2,
0x72, 0x75, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x6f, 0x6a, 0xfa,
0x6d, 0x254, 0x6d, 0x254, 0x20, 0x4f, 0x72, 0x69, 0x2d, 0xf2, 0x6b, 0xe8,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x79, 0xe1, 0x6c, 0x25b, 0x74,
0x61, 0x20, 0x50, 0xe0, 0x73, 0xed, 0x66, 0xed, 0xec, 0x6b, 0xec, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2,
0x20, 0x41, 0x70, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f,
0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20, 0x41, 0x70, 0x69, 0x61, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2,
0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20, 0x41, 0x72, 0x61,
0x62, 0x69, 0x61, 0x41, 0x61, 0x67, 0x6f, 0x20, 0x41, 0x6a, 0x25b, 0x6e,
0x74, 0xec, 0x6e, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x49, 0x77,
0x254, 0x301, 0x2d, 0x6f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x41, 0x6a, 0x25b,
0x301, 0x6e, 0x74, 0xed, 0x6e, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x49, 0x77, 0x254, 0x301, 0x2d, 0x6f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0xc0,
0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x41, 0x6a, 0x25b, 0x301, 0x6e,
0x74, 0xed, 0x6e, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x72,
0x75, 0x20, 0x49, 0x77, 0x254, 0x301, 0x2d, 0x6f, 0xf2, 0x72, 0xf9, 0x6e,
0x20, 0x41, 0x6a, 0x25b, 0x301, 0x6e, 0x74, 0xed, 0x6e, 0xe0, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20,
0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x79, 0xe1, 0x6c, 0x25b, 0x74,
0x61, 0x20, 0xc0, 0x74, 0xec, 0x6c, 0xe1, 0x144, 0x74, 0xed, 0xec, 0x6b,
0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75,
0x6b, 0xf2, 0x20, 0xc0, 0xe0, 0x72, 0x69, 0x6e, 0x20, 0x47, 0xf9, 0x6e,
0x67, 0xf9, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x254,
0x6d, 0x254, 0x20, 0xc0, 0xe0, 0x72, 0x69, 0x6e, 0x20, 0x47, 0xf9, 0x6e,
0x67, 0xf9, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0xe0, 0x72, 0x69, 0x6e,
0x20, 0x47, 0xf9, 0x6e, 0x67, 0xf9, 0x6e, 0x20, 0xcc, 0x77, 0x254, 0x300,
0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66,
0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0xc0, 0xe0, 0x72, 0x69, 0x6e, 0x20,
0x47, 0xf9, 0x6e, 0x67, 0xf9, 0x6e, 0x20, 0xcc, 0x77, 0x254, 0x300, 0x2d,
0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa,
0x6d, 0x254, 0x6d, 0x254, 0x20, 0xc0, 0xe0, 0x72, 0x69, 0x6e, 0x20, 0x47,
0xf9, 0x6e, 0x67, 0xf9, 0x6e, 0x20, 0xcc, 0x77, 0x254, 0x300, 0x2d, 0x4f,
0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e,
0x75, 0x6b, 0xf2, 0x20, 0xcc, 0x6c, 0xe0, 0x2d, 0x4f, 0xf2, 0x72, 0xf9,
0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254,
0x20, 0xcc, 0x6c, 0xe0, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xcc, 0x77, 0x254, 0x300, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20,
0xcc, 0x77, 0x254, 0x300, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20, 0xcc, 0x77,
0x254, 0x300, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x53, 0x254, 0x6d, 0xe0, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69,
0x6a, 0x61, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc1, 0x73, 0x254,
0x301, 0x73, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b,
0x6e, 0x75, 0x6b, 0xf2, 0x20, 0xc1, 0x73, 0x254, 0x301, 0x73, 0xec, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6f, 0x72, 0x75, 0x20, 0xc1, 0x73,
0x254, 0x301, 0x73, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66,
0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x73, 0x68, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254,
0x6d, 0xe0, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73,
0x68, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75,
0x6b, 0xf2, 0x20, 0x4b, 0xe9, 0x70, 0xfa, 0x20, 0x46, 0xe1, 0xe0, 0x64,
0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x190, 0x300, 0x72, 0xfa, 0x6e,
0x20, 0x4b, 0xe9, 0x70, 0xfa, 0x20, 0x46, 0xe1, 0xe0, 0x64, 0xec, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2,
0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x43,
0x68, 0x61, 0x74, 0x61, 0x6d, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f,
0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20, 0x43, 0x68, 0x61, 0x74, 0x61,
0x6d, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x68, 0xed, 0x6c, 0xe8,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b,
0xf2, 0x20, 0x53, 0x68, 0xed, 0x6c, 0xe8, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x4f, 0x72, 0x75, 0x20, 0x53, 0x68, 0xed, 0x6c, 0xe8, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x68, 0xe1, 0xed, 0x6e, 0xe0, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x73,
0xed, 0x20, 0x53, 0x68, 0xe1, 0xed, 0x6e, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20, 0x53, 0x68,
0xe1, 0xed, 0x6e, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x77,
0x254, 0x6e, 0x20, 0x45, 0x72, 0xe9, 0x6b, 0xf9, 0x73, 0xf9, 0x20, 0x43,
0x6f, 0x63, 0x6f, 0x73, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x77,
0x254, 0x6e, 0x20, 0x45, 0x72, 0x65, 0x6b, 0x75, 0x73, 0x75, 0x20, 0x4b,
0x75, 0x75, 0x6b, 0x75, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66,
0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0xc0, 0x77, 0x254, 0x6e, 0x20, 0x45,
0x72, 0x65, 0x6b, 0x75, 0x73, 0x75, 0x20, 0x4b, 0x75, 0x75, 0x6b, 0x75,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x49, 0x6c, 0x61, 0x6a, 0x69, 0x20,
0x53, 0x254, 0x6d, 0x61, 0x20, 0xc0, 0x77, 0x254, 0x6e, 0x20, 0x45, 0x72,
0x65, 0x6b, 0x75, 0x73, 0x75, 0x20, 0x4b, 0x75, 0x75, 0x6b, 0x75, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2,
0x20, 0x4b, 0xfa, 0x62, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f,
0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20, 0x4b, 0xfa, 0x62, 0xe0, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0xe1, 0x72, 0x69, 0x6e, 0x20, 0x53,
0x254, 0x6d, 0xe0, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0xcc, 0x68, 0x61,
0x20, 0xcc, 0x6c, 0xe0, 0x20, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc,
0x77, 0x254, 0x20, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xe0, 0x73, 0x69,
0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x254, 0x20, 0x4f, 0xf2, 0x72, 0xf9, 0x6e,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0xcc, 0x68, 0xe0, 0x20, 0xcc, 0x77,
0x254, 0x20, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e,
0x75, 0x6b, 0xf2, 0x20, 0x45, 0x74, 0xed, 0x6b, 0x75, 0x6e, 0x20, 0x46,
0xf3, 0x6b, 0xed, 0x6c, 0xe1, 0x6e, 0x64, 0xec, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x46, 0x69,
0x6a, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d, 0x61,
0x20, 0x46, 0x69, 0x6a, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0,
0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d, 0xe0,
0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xc0, 0x77, 0x254, 0x6e, 0x20, 0x45, 0x72, 0x65, 0x6b, 0x75,
0x73, 0x75, 0x20, 0x47, 0x69, 0x6c, 0x69, 0x62, 0x61, 0x74, 0x69, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x254, 0x300, 0x20, 0xcc, 0x66,
0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x6f, 0x6f, 0x72, 0xf9, 0x6e, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x254, 0x300, 0x20, 0x6f, 0x6f, 0x72, 0xf9,
0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2,
0x20, 0xcc, 0x77, 0x254, 0x300, 0x20, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0xcc,
0x67, 0x62, 0x61, 0x20, 0x4f, 0xf2, 0x72, 0x75, 0x20, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x47, 0x75, 0x6c, 0x66,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b,
0xf2, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65,
0x75, 0x74, 0x69, 0x61, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f,
0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69,
0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x73,
0xed, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0x48, 0x6f,
0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xcc, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x73, 0xed, 0x20, 0x48,
0x6f, 0x66, 0x69, 0x64, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53,
0x254, 0x6d, 0xe0, 0x20, 0x48, 0x6f, 0x66, 0x69, 0x64, 0x69, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc,
0x77, 0x254, 0x300, 0x20, 0x6f, 0x6f, 0x72, 0xf9, 0x6e, 0x20, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x49, 0x72, 0x61,
0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d,
0x254, 0x6d, 0x254, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x69, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0xcd,
0x6b, 0xf3, 0x73, 0xed, 0x6b, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x53, 0x254, 0x6d, 0xe0, 0x20, 0xcd, 0x6b, 0xfa, 0x73, 0xed, 0x6b, 0xec,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b,
0xf2, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20, 0x49, 0x73,
0x72, 0x61, 0x65, 0x6c, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x66,
0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x73, 0xed, 0x20, 0x4a, 0x61, 0x70, 0x61,
0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20,
0x4a, 0x61, 0x70, 0x61, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc,
0x77, 0x254, 0x300, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x4b, 0x61,
0x73, 0x61, 0x73, 0x69, 0x74, 0xe1, 0xe0, 0x6e, 0xec, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0xcc, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x73, 0xed,
0x20, 0x4b, 0x6f, 0x72, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x4f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20, 0x4b, 0x6f, 0x72, 0x69,
0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75,
0x6b, 0xf2, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x69, 0x6e, 0x6f, 0x79, 0x61,
0x73, 0x69, 0x6b, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254,
0x6d, 0xe0, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x69, 0x6e, 0x6f, 0x79, 0x61,
0x73, 0x69, 0x6b, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x77,
0x254, 0x6e, 0x20, 0x45, 0x72, 0x65, 0x6b, 0x75, 0x73, 0x75, 0x20, 0x4c,
0x61, 0x69, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66,
0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa,
0x6d, 0x254, 0x6d, 0x254, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e,
0x75, 0x6b, 0xf2, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x69,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xc0, 0x77, 0x254, 0x6e, 0x20, 0x45, 0x72, 0x65, 0x6b, 0x75, 0x73,
0x75, 0x20, 0x4d, 0x61, 0x73, 0x61, 0x61, 0x6c, 0x69, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x4d, 0xe1, 0x72, 0xed, 0x73, 0x68, 0xfa, 0x73, 0x68,
0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75,
0x6b, 0xf2, 0x20, 0x4d, 0xe1, 0x72, 0xed, 0x73, 0x68, 0xfa, 0x73, 0x68,
0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6f, 0x72, 0x75, 0x20,
0x4d, 0xe1, 0x72, 0xed, 0x73, 0x68, 0xfa, 0x73, 0x68, 0xec, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x50, 0xe1, 0x73, 0xed, 0x66, 0xed, 0xec, 0x6b,
0xec, 0x20, 0x4d, 0x25b, 0x301, 0x73, 0x68, 0xed, 0x6b, 0xf2, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20,
0x50, 0xe1, 0x73, 0xed, 0x66, 0xed, 0xec, 0x6b, 0xec, 0x20, 0x4d, 0x25b,
0x301, 0x73, 0xed, 0x6b, 0xf2, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f,
0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20, 0x50, 0xe1, 0x73, 0xed, 0x66,
0xed, 0xec, 0x6b, 0xec, 0x20, 0x4d, 0x25b, 0x301, 0x73, 0xed, 0x6b, 0xf2,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x66, 0x25b, 0x6e, 0x75, 0x6b,
0xf2, 0x73, 0xed, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61,
0x74, 0x61, 0x72, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d,
0xe0, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61,
0x72, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75,
0x6b, 0xf2, 0x20, 0x4d, 0x6f, 0x73, 0x69, 0x6b, 0x6f, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0x4d, 0x6f, 0x73, 0x69,
0x6b, 0x6f, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e,
0x75, 0x6b, 0xf2, 0x20, 0x4b, 0x61, 0x6c, 0x69, 0x64, 0x6f, 0x6e, 0x69,
0x61, 0x20, 0x54, 0x75, 0x6e, 0x74, 0x75, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x53, 0x254, 0x6d, 0x61, 0x20, 0x4b, 0x61, 0x6c, 0x69, 0x64,
0x6f, 0x6e, 0x69, 0x61, 0x20, 0x54, 0x75, 0x6e, 0x74, 0x75, 0x6e, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2,
0x20, 0x4e, 0x65, 0x77, 0x20, 0x7a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d,
0x254, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e,
0x64, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75,
0x6b, 0xf2, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c,
0x61, 0x6e, 0x64, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa,
0x6d, 0x254, 0x6d, 0x254, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e,
0x64, 0x6c, 0x61, 0x6e, 0x64, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0,
0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x45, 0x72, 0xe9, 0x6b, 0xf9,
0x73, 0xf9, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20,
0x45, 0x72, 0xe9, 0x6b, 0xf9, 0x73, 0xf9, 0x20, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b,
0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4e, 0x6f, 0x66, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x69, 0x6b, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53,
0x254, 0x6d, 0xe0, 0x20, 0x4e, 0x6f, 0x66, 0x6f, 0x72, 0x69, 0x62, 0x69,
0x73, 0x69, 0x6b, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66,
0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4f, 0x6d, 0x69, 0x73, 0x69, 0x6b,
0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20,
0x4f, 0x6d, 0x69, 0x73, 0x69, 0x6b, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x53, 0x254, 0x6d, 0xe0, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e,
0x75, 0x6b, 0xf2, 0x20, 0x50, 0xe1, 0x72, 0xe1, 0x67, 0xfa, 0x77, 0xe8,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b,
0xf2, 0x20, 0x50, 0xe9, 0x72, 0xf9, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x69, 0x6e, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69,
0x6e, 0x69, 0x6e, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0,
0x77, 0x254, 0x6e, 0x20, 0x45, 0x72, 0xe9, 0x6b, 0xf9, 0x73, 0xf9, 0x20,
0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d,
0x254, 0x6d, 0x254, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x52, 0x25b, 0x301, 0x79, 0xfa, 0x6e, 0xed, 0x254, 0x301,
0x6e, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e,
0x75, 0x6b, 0xf2, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0x53,
0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d,
0x254, 0x6d, 0x254, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x53, 0xe8, 0x73, 0x68, 0x25b, 0x301, 0x25b, 0x300, 0x6c,
0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75,
0x6b, 0xf2, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x77, 0x254, 0x6e, 0x20, 0x45,
0x72, 0x65, 0x6b, 0x75, 0x73, 0x75, 0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d,
0x6f, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x47, 0xfa, 0x73, 0xf9,
0x20, 0x4a, 0x254, 0x301, 0x6a, 0xed, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xcc, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x73, 0xed, 0x20, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f,
0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75,
0x6b, 0xf2, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b,
0xf2, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x69, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254,
0x6d, 0xe0, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66,
0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0xda, 0x72, 0xfa, 0x67, 0xfa, 0x77,
0xe8, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75,
0x6b, 0xf2, 0x20, 0x55, 0x73, 0x69, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d,
0xe0, 0x20, 0x55, 0x73, 0x69, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e,
0x75, 0x6b, 0xf2, 0x20, 0x46, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0x46, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0,
0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x46, 0x69, 0x6c, 0x61, 0x64,
0x69, 0x66, 0x6f, 0x73, 0x69, 0x74, 0x6f, 0x6b, 0x69, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0x46, 0x69, 0x6c, 0x61,
0x64, 0x69, 0x66, 0x6f, 0x73, 0x69, 0x74, 0x6f, 0x6b, 0x69, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20,
0x46, 0x6f, 0x6c, 0x69, 0x67, 0x6f, 0x67, 0x69, 0x72, 0x61, 0x64, 0x69,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0x46,
0x6f, 0x6c, 0x69, 0x67, 0x6f, 0x67, 0x69, 0x72, 0x61, 0x64, 0x69, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2,
0x20, 0x59, 0x61, 0x74, 0x75, 0x74, 0x73, 0x6b, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0x59, 0x61, 0x74, 0x75, 0x74,
0x73, 0x6b, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e,
0x75, 0x6b, 0xf2, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53,
0x254, 0x6d, 0xe0, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x59,
0xfa, 0x6b, 0x254, 0x301, 0x6e, 0xec, 0x47, 0x77, 0x7a, 0x6c, 0x69, 0x6e,
0x7a, 0x76, 0x65, 0x69, 0x68, 0x63, 0x69, 0x20, 0x42, 0x79, 0x61, 0x75,
0x68, 0x63, 0x75, 0x6e, 0x6a, 0x20, 0x53, 0x69, 0x7a, 0x67, 0x65, 0x6e,
0x68, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x41, 0x66, 0x67, 0x68, 0x61, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x4d, 0x70,
0x75, 0x6d, 0x61, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x20, 0x41, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73,
0x61, 0x73, 0x65, 0x4e, 0x69, 0x6e, 0x67, 0x69, 0x7a, 0x69, 0x6d, 0x75,
0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x4e, 0x74, 0x73,
0x68, 0x6f, 0x6e, 0x61, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x20, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20,
0x73, 0x61, 0x73, 0x65, 0x4e, 0x74, 0x73, 0x68, 0x6f, 0x6e, 0x61, 0x6c,
0x61, 0x6e, 0x67, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61, 0x73, 0x65, 0x4e,
0x74, 0x73, 0x68, 0x6f, 0x6e, 0x61, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x20,
0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b,
0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65,
0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62,
0x6f, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65,
0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x72,
0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x73, 0x61, 0x73, 0x65,
0x6d, 0x69, 0x6e, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x74, 0x68,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x45, 0x61,
0x73, 0x74, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x73, 0x74,
0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b,
0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x73,
0x74, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x6e, 0x20, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61,
0x69, 0x6e, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c,
0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x74,
0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x4d,
0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x20, 0x73, 0x61, 0x73, 0x65,
0x6d, 0x69, 0x6e, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x74, 0x68,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61,
0x63, 0x69, 0x66, 0x69, 0x63, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x74,
0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50,
0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77,
0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x73,
0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x65, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x65, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x65, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x6e, 0x61, 0x64,
0x79, 0x72, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x70, 0x69, 0x61, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x41, 0x70, 0x69, 0x61, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69,
0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x70, 0x69, 0x61, 0x20, 0x73, 0x61,
0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x72, 0x61,
0x62, 0x69, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x6e,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x65,
0x6d, 0x69, 0x6e, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x72,
0x61, 0x62, 0x69, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61,
0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x73, 0x61, 0x73,
0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x4e, 0x79, 0x61, 0x6b,
0x61, 0x74, 0x68, 0x6f, 0x20, 0x6e, 0x65, 0x2d, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x4e, 0x79, 0x61, 0x6b, 0x61,
0x74, 0x68, 0x6f, 0x20, 0x6e, 0x65, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79,
0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x4e, 0x79, 0x61,
0x6b, 0x61, 0x74, 0x68, 0x6f, 0x20, 0x6e, 0x65, 0x2d, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68,
0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65,
0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x65, 0x68, 0x6c, 0x6f,
0x62, 0x6f, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x63, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x63, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65,
0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x63, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d,
0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x6e, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x57, 0x65, 0x73, 0x74, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e,
0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x57, 0x65, 0x73,
0x74, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x6d, 0x69, 0x6e, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x57, 0x65, 0x73, 0x74, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c,
0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x73, 0x74, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x6d, 0x69, 0x6e, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x73,
0x74, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x57,
0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61,
0x6d, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72,
0x6e, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69,
0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x65, 0x68, 0x6c,
0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x7a,
0x6f, 0x72, 0x65, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65,
0x6b, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x7a,
0x6f, 0x72, 0x65, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73,
0x68, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x42, 0x6f,
0x6c, 0x69, 0x76, 0x69, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x42, 0x72, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x42, 0x72, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x61, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65,
0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c,
0x6f, 0x62, 0x6f, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69,
0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73,
0x65, 0x7a, 0x69, 0x6e, 0x67, 0x65, 0x6e, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x65,
0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43,
0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61,
0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x68,
0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x68,
0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x20,
0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69,
0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x20, 0x73,
0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43,
0x68, 0x69, 0x6e, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x65, 0x6d, 0x69,
0x6e, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x68, 0x69, 0x6e,
0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61,
0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43,
0x6f, 0x63, 0x6f, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x65,
0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c,
0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61,
0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65,
0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x65, 0x73, 0x69, 0x79, 0x69, 0x6e, 0x67, 0x78, 0x65, 0x6e, 0x79, 0x65,
0x20, 0x79, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x75, 0x62, 0x61, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x43, 0x75, 0x62, 0x61, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77,
0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x43, 0x75, 0x62, 0x61, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e,
0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x44, 0x61, 0x76, 0x69, 0x73, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76,
0x69, 0x6c, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x45, 0x61, 0x73, 0x74, 0x20,
0x54, 0x69, 0x6d, 0x6f, 0x72, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x45, 0x61, 0x73, 0x74,
0x65, 0x72, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65,
0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x45, 0x61, 0x73, 0x74, 0x65,
0x72, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x73, 0x61, 0x73,
0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x45, 0x63, 0x75,
0x61, 0x64, 0x6f, 0x72, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77,
0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x45,
0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65,
0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c,
0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x45, 0x61, 0x73, 0x74, 0x65,
0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68,
0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x45, 0x61,
0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x46, 0x75, 0x72, 0x74, 0x68, 0x65, 0x72, 0x2d, 0x65,
0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65,
0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x65, 0x73, 0x69, 0x6a,
0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c,
0x6f, 0x62, 0x6f, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x46, 0x69, 0x6a, 0x69, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76,
0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x46,
0x69, 0x6a, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x46, 0x69, 0x6a, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x46, 0x72, 0x65,
0x6e, 0x63, 0x68, 0x20, 0x47, 0x75, 0x69, 0x61, 0x6e, 0x61, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x46, 0x72, 0x65, 0x6e, 0x63, 0x68, 0x20, 0x53, 0x6f, 0x75, 0x74,
0x68, 0x65, 0x72, 0x6e, 0x20, 0x6e, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x6e,
0x74, 0x61, 0x72, 0x63, 0x74, 0x69, 0x63, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x47, 0x61,
0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x47, 0x61,
0x6d, 0x62, 0x69, 0x65, 0x72, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x47, 0x65, 0x6f, 0x72,
0x67, 0x69, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x65, 0x68,
0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x47, 0x65,
0x6f, 0x72, 0x67, 0x69, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x47, 0x69, 0x6c, 0x62,
0x65, 0x72, 0x74, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x20,
0x4d, 0x65, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x45, 0x61, 0x73, 0x74, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x45, 0x61, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c,
0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x45, 0x61, 0x73, 0x74,
0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x73,
0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x57, 0x65,
0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x57, 0x65, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61,
0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x57,
0x65, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69,
0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x47, 0x75, 0x6c, 0x66, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x47, 0x75, 0x79, 0x61, 0x6e,
0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x20,
0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69,
0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x20, 0x73, 0x61, 0x73, 0x65,
0x6d, 0x69, 0x6e, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x48, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6e, 0x67, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f,
0x6e, 0x67, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x48, 0x6f, 0x76, 0x64, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x45, 0x73, 0x69, 0x6d, 0x69, 0x73, 0x69, 0x77, 0x65,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x48, 0x6f, 0x76, 0x64, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x65, 0x68, 0x6c,
0x6f, 0x62, 0x6f, 0x20, 0x65, 0x2d, 0x48, 0x6f, 0x76, 0x64, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61,
0x6d, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x49, 0x6e, 0x64, 0x69, 0x61,
0x6e, 0x20, 0x4f, 0x63, 0x65, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x49, 0x6e,
0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x73, 0x69, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72,
0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x49, 0x72,
0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x65, 0x73,
0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x49, 0x72,
0x61, 0x6e, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x45, 0x73, 0x69, 0x76, 0x61,
0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f,
0x20, 0x65, 0x2d, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69,
0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x20, 0x73, 0x61, 0x6b,
0x77, 0x61, 0x2d, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x4a, 0x61, 0x70, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x65, 0x6d,
0x69, 0x6e, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4a, 0x61, 0x70,
0x61, 0x6e, 0x65, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50, 0x65, 0x74, 0x72,
0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61,
0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x65, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x65, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c,
0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74,
0x73, 0x6b, 0x69, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73,
0x74, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4d, 0x70, 0x75, 0x6d, 0x61, 0x6c,
0x61, 0x6e, 0x67, 0x61, 0x20, 0x6e, 0x65, 0x2d, 0x4b, 0x61, 0x7a, 0x61,
0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x4e, 0x74, 0x73, 0x68,
0x6f, 0x6e, 0x61, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x20, 0x6e, 0x65, 0x2d,
0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x45, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c,
0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4b, 0x6f, 0x72, 0x65, 0x61,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x65,
0x6d, 0x69, 0x6e, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4b, 0x6f,
0x72, 0x65, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x45, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x65, 0x2d,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x4b, 0x79, 0x72, 0x67, 0x79, 0x73, 0x74, 0x61, 0x6e, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x4c, 0x69, 0x6e, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x45, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69,
0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x65,
0x2d, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4d,
0x61, 0x6c, 0x61, 0x79, 0x73, 0x69, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4d, 0x61,
0x6c, 0x64, 0x69, 0x76, 0x65, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4d, 0x61, 0x72,
0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4d, 0x61, 0x72,
0x73, 0x68, 0x61, 0x6c, 0x6c, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65,
0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x65, 0x68,
0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4d, 0x61,
0x77, 0x73, 0x6f, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4d, 0x65, 0x78, 0x69, 0x63,
0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63,
0x69, 0x66, 0x69, 0x63, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79,
0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4d, 0x65,
0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69,
0x63, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x45, 0x73, 0x69,
0x6d, 0x69, 0x73, 0x69, 0x77, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x65, 0x68, 0x6c,
0x6f, 0x62, 0x6f, 0x20, 0x65, 0x2d, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42,
0x61, 0x74, 0x6f, 0x72, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4d, 0x6f, 0x73, 0x63, 0x6f,
0x77, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x20, 0x65,
0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c,
0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x65, 0x2d, 0x4d,
0x6f, 0x73, 0x63, 0x6f, 0x77, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4d, 0x79, 0x61, 0x6e,
0x6d, 0x61, 0x72, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x65,
0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x69, 0x61, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c,
0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c,
0x6f, 0x62, 0x6f, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65,
0x61, 0x6c, 0x61, 0x6e, 0x64, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65,
0x61, 0x6c, 0x61, 0x6e, 0x64, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61,
0x6c, 0x61, 0x6e, 0x64, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x65, 0x77, 0x66, 0x6f,
0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x65,
0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x65,
0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c,
0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e,
0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x4e, 0x69, 0x75, 0x65, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x65, 0x73, 0x69,
0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61,
0x6d, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x45, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x4f, 0x6d, 0x73, 0x6b, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x45, 0x73, 0x69, 0x76, 0x61,
0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4f, 0x6d,
0x73, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x4f, 0x6d, 0x73, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d,
0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50,
0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x47, 0x75, 0x69,
0x6e, 0x65, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x20, 0x73,
0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50,
0x65, 0x72, 0x75, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50, 0x65, 0x72, 0x75, 0x20, 0x65,
0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50,
0x65, 0x72, 0x75, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62,
0x6f, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69,
0x6e, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e,
0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x65, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x53, 0x61, 0x69, 0x6e, 0x74,
0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x6e, 0x61, 0x73, 0x65,
0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x49, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20,
0x6e, 0x61, 0x73, 0x65, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f,
0x6e, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65,
0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x53, 0x61, 0x69, 0x6e, 0x74,
0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x6e, 0x61, 0x73, 0x65,
0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x73, 0x61,
0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50, 0x69, 0x74,
0x63, 0x61, 0x69, 0x72, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50, 0x6f, 0x6e, 0x61,
0x70, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61,
0x6e, 0x67, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x45, 0x73, 0x69, 0x76, 0x61,
0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x53, 0x61,
0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62,
0x6f, 0x20, 0x65, 0x2d, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x65, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x65, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x53,
0x61, 0x6d, 0x61, 0x72, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x65, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20,
0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69,
0x6e, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c,
0x6c, 0x65, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72,
0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x53, 0x6f,
0x75, 0x74, 0x68, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x54, 0x61, 0x68,
0x69, 0x74, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73,
0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x54, 0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x73, 0x61, 0x73,
0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x68, 0x75,
0x75, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x54, 0x75, 0x76, 0x61,
0x6c, 0x75, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x20, 0x65,
0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c,
0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x20,
0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76,
0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x55,
0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x65, 0x68, 0x6c, 0x6f,
0x62, 0x6f, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x55, 0x7a, 0x62, 0x65,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c,
0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62,
0x6f, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c,
0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x45, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x65,
0x2d, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x45, 0x73,
0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x73,
0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x56,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x57, 0x61, 0x6b, 0x65,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x57, 0x61,
0x6c, 0x6c, 0x69, 0x73, 0x20, 0x6e, 0x61, 0x73, 0x65, 0x2d, 0x46, 0x75,
0x74, 0x75, 0x6e, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x59, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x45, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x65, 0x2d, 0x59, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x45, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69,
0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x59, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68,
0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x65, 0x2d, 0x59, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x41, 0x6b, 0x72, 0x65,
0x20, 0x74, 0xe1, 0x20, 0xf3, 0x72, 0x61, 0xd3, 0x72, 0x61, 0x20, 0x50,
0xe3, 0x20, 0x41, 0x6b, 0x72, 0x65, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20,
0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x41, 0x6b, 0x72, 0x65, 0x20,
0x74, 0xe1, 0x41, 0x66, 0x65, 0x67, 0x61, 0x6e, 0x129, 0x74, 0xe3, 0x20,
0x74, 0xe1, 0x20, 0xf3, 0x72, 0x61, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x2d, 0x4b, 0x75, 0x6a, 0x75, 0x20, 0x74, 0xe1, 0x20, 0xf3, 0x72, 0x61,
0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x52, 0xe3, 0x6a, 0x75, 0x72,
0x20, 0x74, 0xe1, 0x20, 0xf3, 0x72, 0x61, 0x53, 0x75, 0x72, 0x2d, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x74, 0xe1, 0x20, 0xf3, 0x72, 0x61,
0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x52, 0xe3, 0x70, 0x75, 0x72,
0x20, 0x74, 0xe1, 0x20, 0xf3, 0x72, 0x61, 0xd3, 0x72, 0x61, 0x20, 0x50,
0xe3, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x52, 0xe3, 0x70,
0x75, 0x72, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3,
0x72, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x52, 0xe3,
0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0x41, 0x72, 0x61, 0x6e, 0x68, 0x6b,
0x61, 0x20, 0x74, 0xe1, 0x20, 0xf3, 0x72, 0x61, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x41, 0x72, 0x61, 0x6e, 0x68, 0x6b, 0x61, 0x20, 0x74,
0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x41,
0x72, 0x61, 0x6e, 0x68, 0x6b, 0x61, 0x20, 0x74, 0xe1, 0x41, 0x72, 0x6d,
0x61, 0x74, 0x79, 0x20, 0x74, 0xe1, 0x20, 0xf3, 0x72, 0x61, 0xd3, 0x72,
0x61, 0x20, 0x50, 0xe3, 0x20, 0x41, 0x72, 0x6d, 0x61, 0x74, 0x79, 0x20,
0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20,
0x41, 0x72, 0x6d, 0x61, 0x74, 0x79, 0x20, 0x74, 0xe1, 0x41, 0x6d, 0x1ef9,
0x6a, 0x6f, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0x20, 0xf3, 0x72, 0x61, 0xd3,
0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x41, 0x6d, 0x1ef9, 0x6a, 0x6f, 0x6e,
0x1ef9, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72,
0x61, 0x20, 0x41, 0x6d, 0x1ef9, 0x6a, 0x6f, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4b, 0x75, 0x6a, 0x75, 0x20,
0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20,
0x4b, 0x75, 0x6a, 0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x52,
0xe3, 0x6a, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50,
0xe3, 0x20, 0x52, 0xe3, 0x6a, 0x75, 0x72, 0x20, 0x74, 0xe1, 0x52, 0x1ef9,
0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x52, 0xe3, 0x6a, 0x75,
0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4b, 0x72, 0x129, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4b, 0x72, 0x129,
0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61,
0x20, 0x4b, 0x72, 0x129, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x52,
0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50,
0xe3, 0x20, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0x52, 0x1ef9,
0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x52, 0xe3, 0x70, 0x75,
0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x41, 0x6e, 0x1ef9, 0x6e,
0x6e, 0x79, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3,
0x20, 0x41, 0x6e, 0x1ef9, 0x6e, 0x68, 0x79, 0x72, 0x20, 0x74, 0xe1, 0x52,
0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x41, 0x6e, 0x1ef9,
0x6e, 0x68, 0x79, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x41,
0x70, 0x69, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50,
0xe3, 0x20, 0x41, 0x70, 0x69, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0x52, 0x1ef9,
0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x41, 0x70, 0x69, 0x6a,
0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x41, 0x67, 0x74, 0x61,
0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x41,
0x67, 0x74, 0x61, 0x76, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3,
0x20, 0xf3, 0x72, 0x61, 0x20, 0x41, 0x67, 0x74, 0x61, 0x76, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x41, 0x67, 0x74, 0xf3, 0x6d, 0x65, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x41, 0x67, 0x74,
0xf3, 0x6d, 0x65, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20,
0xf3, 0x72, 0x61, 0x20, 0x41, 0x67, 0x74, 0xf3, 0x6d, 0x65, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x41, 0x72, 0x61, 0x6d, 0x69, 0x6a, 0x61,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x41, 0x72,
0x61, 0x6d, 0x69, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b,
0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x41, 0x72, 0x61, 0x6d, 0x69, 0x6a,
0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x41, 0x72, 0x6a, 0x1ebd,
0x74, 0x129, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4f,
0x70, 0xe3, 0x20, 0x41, 0x72, 0x6a, 0x1ebd, 0x74, 0x129, 0x6e, 0x1ef9, 0x20,
0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20,
0x41, 0x72, 0x6a, 0x1ebd, 0x74, 0x129, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x41, 0x72, 0x6a, 0x1ebd, 0x74, 0x129, 0x6e, 0x1ef9, 0x20,
0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x41, 0x72, 0x6a, 0x1ebd, 0x74, 0x129, 0x6e, 0x1ef9, 0x20,
0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b,
0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x41, 0x72, 0x6a, 0x1ebd, 0x74, 0x129,
0x6e, 0x1ef9, 0x20, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x41, 0x72, 0x6d, 0x1ebd, 0x6e, 0x129, 0x6a, 0x61, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x41, 0x72, 0x6d, 0x1ebd,
0x6e, 0x129, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3,
0x20, 0xf3, 0x72, 0x61, 0x20, 0x41, 0x72, 0x6d, 0x1ebd, 0x6e, 0x129, 0x6a,
0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x41, 0x74, 0x72, 0x1ef9,
0x74, 0x69, 0x6b, 0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50,
0xe3, 0x20, 0x41, 0x74, 0x72, 0x1ef9, 0x74, 0x69, 0x6b, 0x75, 0x20, 0x74,
0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x41,
0x74, 0x72, 0x1ef9, 0x74, 0x69, 0x6b, 0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x41, 0x76, 0x6f, 0x74, 0x79, 0x72, 0x61, 0x72, 0x69, 0x61,
0x20, 0x4b, 0x75, 0x6a, 0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x41, 0x76, 0x6f, 0x74, 0x79, 0x72, 0x61, 0x72, 0x69,
0x61, 0x20, 0x4b, 0x75, 0x6a, 0x75, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20,
0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x41, 0x76, 0x6f, 0x74, 0x79,
0x72, 0x61, 0x72, 0x69, 0x61, 0x20, 0x4b, 0x75, 0x6a, 0x75, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x41, 0x76, 0x6f, 0x74, 0x79, 0x61, 0x72,
0x69, 0x61, 0x20, 0x4b, 0x75, 0x6a, 0x75, 0x2d, 0x52, 0xe3, 0x70, 0x75,
0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x41,
0x76, 0x6f, 0x74, 0x79, 0x72, 0x61, 0x72, 0x69, 0x61, 0x20, 0x4b, 0x75,
0x6a, 0x75, 0x2d, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0x52,
0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x41, 0x76, 0x6f,
0x74, 0x79, 0x72, 0x61, 0x72, 0x69, 0x61, 0x20, 0x4b, 0x75, 0x6a, 0x75,
0x2d, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x41, 0x76, 0x6f, 0x74, 0x79, 0x72, 0x61, 0x72, 0x69, 0x61, 0x20,
0x52, 0xe3, 0x6a, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x41, 0x76, 0x6f, 0x74, 0x79, 0x72, 0x61, 0x72, 0x69,
0x61, 0x20, 0x52, 0xe3, 0x6a, 0x75, 0x72, 0x20, 0x74, 0xe1, 0x52, 0x1ef9,
0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x41, 0x76, 0x6f, 0x74,
0x79, 0x72, 0x61, 0x72, 0x69, 0x61, 0x20, 0x52, 0xe3, 0x6a, 0x75, 0x72,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x41, 0x76, 0x6f, 0x74, 0x79,
0x72, 0x61, 0x72, 0x69, 0x61, 0x20, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x41, 0x76, 0x6f,
0x74, 0x79, 0x72, 0x61, 0x72, 0x69, 0x61, 0x20, 0x52, 0xe3, 0x70, 0x75,
0x72, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72,
0x61, 0x20, 0x41, 0x76, 0x6f, 0x74, 0x79, 0x72, 0x61, 0x72, 0x69, 0x61,
0x20, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x41, 0x6a, 0x65, 0x72, 0x6d, 0x61, 0x6a, 0x6a, 0xe1, 0x76, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x41, 0x6a, 0x65,
0x72, 0x6d, 0x61, 0x6a, 0x6a, 0xe1, 0x76, 0x20, 0x74, 0xe1, 0x52, 0x1ef9,
0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x41, 0x6a, 0x65, 0x72,
0x6d, 0x61, 0x6a, 0x6a, 0xe1, 0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x41, 0x73, 0x6f, 0x72, 0x2d, 0x61, 0x67, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x41, 0x73, 0x6f, 0x72, 0x2d, 0x61,
0x67, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72,
0x61, 0x20, 0x41, 0x73, 0x6f, 0x2d, 0x61, 0x67, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x4d, 0x1ef9, 0x6e, 0x67, 0x72, 0x61, 0x6e, 0xe9, 0x73,
0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4d,
0x1ef9, 0x6e, 0x67, 0x72, 0x61, 0x6e, 0xe9, 0x73, 0x69, 0x20, 0x74, 0xe1,
0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4d, 0x1ef9,
0x6e, 0x67, 0x72, 0x61, 0x6e, 0xe9, 0x73, 0x69, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x4d, 0x75, 0x74, 0x1ef9, 0x76, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x4d, 0x6f, 0x72, 0x69, 0x76, 0x69, 0x61, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4d, 0x72, 0x61, 0x73, 0x69, 0x72, 0x6a,
0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4d,
0x72, 0x61, 0x73, 0x69, 0x72, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0x52, 0x1ef9,
0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4d, 0x72, 0x61, 0x73,
0x69, 0x72, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4d,
0x72, 0x75, 0x6e, 0x1ebd, 0x6a, 0x20, 0x4e, 0x61, 0x72, 0x75, 0x73, 0x61,
0x72, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0x75, 0x20,
0x54, 0xe1, 0x6e, 0x68, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50,
0xe3, 0x20, 0x50, 0x75, 0x20, 0x54, 0xe1, 0x6e, 0x68, 0x20, 0x74, 0xe1,
0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x50, 0x75,
0x20, 0x74, 0xe1, 0x6e, 0x68, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x53, 0x61, 0x6d, 0xe3, 0x68, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x53, 0x61, 0x6e, 0x2d, 0x68, 0x1ef9, 0x6d, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x53, 0x61, 0x6e, 0x2d, 0x68, 0x1ef9,
0x6d, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72,
0x61, 0x20, 0x53, 0x61, 0x6e, 0x2d, 0x68, 0x1ef9, 0x6d, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x53, 0x69, 0x72, 0x65, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x53, 0x69, 0x72, 0x65, 0x20, 0x74,
0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x53,
0x69, 0x72, 0x65, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x53, 0x129,
0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20,
0x53, 0x129, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3,
0x20, 0xf3, 0x72, 0x61, 0x20, 0x53, 0x129, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x4b, 0x72, 0x129, 0x74, 0x69, 0x6d, 0x1ef9, 0x6e,
0x68, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4b, 0x6f, 0x6b, 0x6f, 0x6e, 0x68, 0x20,
0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x4b, 0x6f, 0x72, 0xe3, 0x6d, 0x69, 0x6a, 0x61, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4b, 0x6f, 0x72,
0xe3, 0x6d, 0x69, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b,
0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4b, 0x6f, 0x72, 0xe3, 0x6d, 0x69,
0x6a, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4b, 0x75, 0x6b,
0x69, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4b, 0x75, 0x6b, 0x69,
0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1,
0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4b, 0x75,
0x6b, 0x69, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4b, 0x75, 0x6d, 0x61, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4b, 0x75, 0x6d, 0x61,
0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61,
0x20, 0x4b, 0x75, 0x6d, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x4e, 0x61, 0x76, 0x69, 0x6e, 0x68, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x4e, 0x75, 0x6d, 0xe3, 0x2d, 0x4e, 0x75, 0x72, 0x76, 0x69, 0x72,
0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x54, 0x129, 0x6d, 0xe3,
0x72, 0x2d, 0x52, 0xe3, 0x6a, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x50, 0x61, 0x6e, 0x68, 0x6b, 0x75, 0x76, 0x61, 0x20, 0x47,
0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x50, 0xe3, 0x20, 0x50, 0x61, 0x6e, 0x68, 0x6b, 0x75, 0x76,
0x61, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74,
0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x50,
0x61, 0x6e, 0x68, 0x6b, 0x75, 0x76, 0x61, 0x20, 0x47, 0x6f, 0x6a, 0x2d,
0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x45,
0x6b, 0x75, 0x76, 0x61, 0x6e, 0x6f, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x4f, 0x72, 0xf3, 0x70, 0x61, 0x20, 0x4b, 0x75, 0x6a, 0x75,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4f, 0x72,
0xf3, 0x70, 0x61, 0x20, 0x4b, 0x75, 0x6a, 0x75, 0x20, 0x74, 0xe1, 0x52,
0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4f, 0x72, 0xf3,
0x70, 0x61, 0x20, 0x4b, 0x75, 0x6a, 0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x4f, 0x72, 0xf3, 0x70, 0x61, 0x20, 0x52, 0xe3, 0x6a, 0x75,
0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4f,
0x72, 0xf3, 0x70, 0x61, 0x20, 0x52, 0xe3, 0x6a, 0x75, 0x72, 0x20, 0x74,
0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4f,
0x72, 0xf3, 0x70, 0x61, 0x20, 0x52, 0xe3, 0x6a, 0x75, 0x72, 0x20, 0x74,
0xe1, 0x52, 0xe3, 0x6a, 0x75, 0x72, 0x20, 0x74, 0x1ef9, 0x20, 0x4f, 0x72,
0xf3, 0x70, 0x61, 0x20, 0x6a, 0xe3, 0x20, 0x68, 0xe1, 0x20, 0x74, 0xe1,
0x20, 0xf3, 0x72, 0x61, 0xd3, 0x72, 0x61, 0x20, 0x4f, 0x72, 0xf3, 0x70,
0x61, 0x20, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x50, 0xe3, 0x20, 0x4f, 0x72, 0xf3, 0x70, 0x61, 0x20, 0x52,
0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3,
0x20, 0xf3, 0x72, 0x61, 0x20, 0x4f, 0x72, 0xf3, 0x70, 0x61, 0x20, 0x52,
0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4d,
0x1ef9, 0x72, 0x76, 0x129, 0x6e, 0x1ef9, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76,
0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x20, 0x50, 0xe3, 0x20,
0x4d, 0x1ef9, 0x72, 0x76, 0x129, 0x6e, 0x1ef9, 0x20, 0x47, 0x6f, 0x6a, 0x2d,
0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3,
0x20, 0xf3, 0x72, 0x61, 0x20, 0x4d, 0x1ef9, 0x72, 0x76, 0x129, 0x6e, 0x1ef9,
0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x74,
0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x46,
0x69, 0x6a, 0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x46, 0x72,
0x1ef9, 0x73, 0x61, 0x20, 0x47, 0x69, 0x6a, 0x61, 0x6e, 0x1ef9, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x46, 0x72, 0x1ef9, 0x73, 0x61, 0x20, 0x47,
0x61, 0x20, 0x53, 0x75, 0x72, 0x20, 0x6b, 0x61, 0x72, 0x20, 0x1ef8, 0x74,
0x61, 0x72, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x47, 0x61, 0x72, 0x61, 0x20, 0x50, 0x61, 0x67, 0x6f, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x47, 0x1ef9, 0x6d, 0x6d, 0x69, 0x6a, 0x65,
0x72, 0xd3, 0x72, 0x61, 0x20, 0x4a, 0x65, 0xf3, 0x72, 0x6a, 0x61, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4a, 0x65, 0xf3,
0x72, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20,
0xf3, 0x72, 0x61, 0x20, 0x4a, 0x65, 0xf3, 0x72, 0x6a, 0x61, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4a, 0x69, 0x72, 0x6d, 0xe9, 0x72, 0x74,
0x75, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4d, 0x69, 0x72, 0x69, 0x6e, 0x6a, 0x1ef9,
0x6e, 0x169, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x169, 0x76, 0x69, 0x73, 0x69,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x1ebd, 0x72,
0x1ef9, 0x6e, 0x69, 0x6a, 0x61, 0x20, 0x52, 0xe3, 0x6a, 0x75, 0x72, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x47, 0x72, 0x6f,
0x1ebd, 0x72, 0x1ef9, 0x6e, 0x69, 0x6a, 0x61, 0x20, 0x52, 0xe3, 0x6a, 0x75,
0x72, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72,
0x61, 0x20, 0x47, 0x72, 0x6f, 0x1ebd, 0x72, 0x1ef9, 0x6e, 0x69, 0x6a, 0x61,
0x20, 0x52, 0xe3, 0x6a, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x47, 0x72, 0x6f, 0x1ebd, 0x72, 0x1ef9, 0x6e, 0x69, 0x6a, 0x61, 0x20,
0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x47, 0x72, 0x6f, 0x1ebd, 0x72, 0x1ef9, 0x6e, 0x69, 0x6a,
0x61, 0x20, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0x52, 0x1ef9,
0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x1ebd,
0x72, 0x1ef9, 0x6e, 0x69, 0x6a, 0x61, 0x20, 0x52, 0xe3, 0x70, 0x75, 0x72,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x47, 0x75,
0x76, 0x1ef9, 0x6d, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x47, 0x6f,
0x72, 0x66, 0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x47, 0x69,
0x6a, 0x1ef9, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x48,
0x61, 0x76, 0x61, 0x2019, 0x69, 0x20, 0x6b, 0x61, 0x72, 0x20, 0x41, 0x72,
0x65, 0x76, 0x74, 0x61, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73,
0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x48,
0x61, 0x76, 0x61, 0x2019, 0x69, 0x20, 0x6b, 0x61, 0x72, 0x20, 0x41, 0x72,
0x65, 0x76, 0x74, 0x61, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73,
0x6f, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x6b, 0xe3, 0x20, 0xf3, 0x72,
0x61, 0x20, 0x48, 0x61, 0x76, 0x61, 0x2019, 0x69, 0x20, 0x6b, 0x61, 0x72,
0x20, 0x41, 0x72, 0x65, 0x76, 0x74, 0x61, 0x20, 0x47, 0x6f, 0x6a, 0x2d,
0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x48,
0xe3, 0x67, 0x20, 0x4b, 0xe3, 0x67, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x50, 0xe3, 0x20, 0x48, 0xe3, 0x67, 0x20, 0x4b, 0xe3, 0x67, 0x20,
0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20,
0x48, 0xe3, 0x67, 0x20, 0x4b, 0xe3, 0x67, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x48, 0xf3, 0x76, 0x69, 0x6e, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x50, 0xe3, 0x20, 0x48, 0xf3, 0x76, 0x69, 0x6e, 0x20, 0x74,
0xe1, 0x52, 0x1ef9, 0x20, 0x6b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x48,
0xf3, 0x76, 0x69, 0x6e, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50,
0xe3, 0x20, 0x128, 0x6e, 0x69, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x4f, 0x73, 0x69, 0x1ef9, 0x6e, 0x6f, 0x20, 0x128, 0x6e, 0x69,
0x6b, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x128, 0x6e, 0x6f,
0x73, 0x129, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x128,
0x6e, 0x6f, 0x6e, 0x1ebd, 0x6a, 0x61, 0x20, 0x4b, 0x75, 0x6a, 0x75, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x128, 0x6e, 0x6f, 0x6e, 0x1ebd, 0x6a,
0x61, 0x20, 0x52, 0xe3, 0x6a, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x128, 0x6e, 0x6f, 0x6e, 0x1ebd, 0x6a, 0x61, 0x20, 0x52, 0xe3,
0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x49, 0x72,
0x1ef9, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x49,
0x72, 0x1ef9, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3,
0x72, 0x61, 0x20, 0x49, 0x72, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x69, 0x6e, 0x68, 0x6b, 0x69, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x69, 0x6e, 0x68, 0x6b, 0x69, 0x20, 0x74, 0xe1, 0x52, 0x1ef9,
0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x69, 0x6e, 0x68, 0x6b, 0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x49, 0x73, 0x69, 0x68, 0x61, 0xe9, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x50, 0xe3, 0x20, 0x49, 0x73, 0x69, 0x68, 0x61, 0xe9, 0x20,
0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20,
0x49, 0x73, 0x69, 0x68, 0x61, 0xe9, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x4a, 0x61, 0x70, 0xe3, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x4a, 0x61, 0x70, 0xe3, 0x20, 0x74, 0xe1, 0x52, 0x1ef9,
0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4a, 0x61, 0x70, 0xe3,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0x65, 0x74, 0x72, 0x75,
0x70, 0x61, 0x76, 0x69, 0x72, 0xf3, 0x76, 0x69, 0x6e, 0x68, 0x6b, 0x69,
0x2d, 0x4b, 0x1ef9, 0x6d, 0x73, 0x61, 0x74, 0x69, 0x6e, 0x68, 0x6b, 0x69,
0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x50, 0x65, 0x74, 0x72, 0x75,
0x70, 0x61, 0x76, 0x69, 0x72, 0xf3, 0x76, 0x69, 0x6e, 0x68, 0x6b, 0x69,
0x2d, 0x4b, 0x1ef9, 0x6d, 0x73, 0x61, 0x74, 0x69, 0x6e, 0x68, 0x6b, 0x69,
0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x50, 0x65,
0x74, 0x72, 0x75, 0x70, 0x61, 0x76, 0x69, 0x72, 0xf3, 0x76, 0x69, 0x6e,
0x68, 0x6b, 0x69, 0x2d, 0x4b, 0x1ef9, 0x6d, 0x73, 0x61, 0x74, 0x69, 0x6e,
0x68, 0x6b, 0x69, 0xd3, 0x72, 0x61, 0x20, 0x4b, 0x61, 0x6a, 0x61, 0x6b,
0x69, 0x6e, 0x68, 0x74, 0x1ef9, 0x76, 0x20, 0x52, 0xe3, 0x6a, 0x75, 0x72,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4b, 0x61, 0x6a, 0x61, 0x6b,
0x69, 0x6e, 0x68, 0x74, 0x1ef9, 0x76, 0x20, 0x52, 0xe3, 0x70, 0x75, 0x72,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4b, 0x6f, 0x72, 0xe9, 0x6a,
0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4b,
0x6f, 0x72, 0xe9, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b,
0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4b, 0x6f, 0x72, 0xe9, 0x6a, 0x61,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x6f,
0x73, 0x69, 0x72, 0x61, 0x6a, 0x65, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x4b, 0x72, 0x61, 0x6e, 0x68, 0x6e, 0xe3, 0x6a, 0x61, 0x72, 0x6b,
0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4b,
0x72, 0x61, 0x6e, 0x68, 0x6e, 0xe3, 0x6a, 0x61, 0x72, 0x6b, 0x69, 0x20,
0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20,
0x4b, 0x72, 0x61, 0x6e, 0x68, 0x6e, 0xe3, 0x6a, 0x61, 0x72, 0x6b, 0x69,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4b, 0x69, 0x72, 0x67, 0x69,
0x6e, 0x68, 0x74, 0x1ef9, 0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x52, 0x1ef9, 0x6e, 0x6b, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x56, 0xe3, 0x66, 0x65, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73,
0x6f, 0x20, 0x74, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x52,
0xf3, 0x72, 0x6e, 0x69, 0x20, 0x48, 0x6f, 0x76, 0x65, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x52, 0xf3, 0x72, 0x6e, 0x69,
0x20, 0x48, 0x6f, 0x76, 0x65, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b,
0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x52, 0xf3, 0x72, 0x6e, 0x69, 0x20,
0x48, 0x6f, 0x76, 0x65, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4d,
0x1ef9, 0x6b, 0x61, 0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50,
0xe3, 0x20, 0x4d, 0x1ef9, 0x6b, 0x61, 0x76, 0x20, 0x74, 0xe1, 0x52, 0x1ef9,
0x20, 0x6b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4d, 0x1ef9, 0x6b, 0x61,
0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4d, 0x1ef9, 0x67, 0x61,
0x6e, 0x61, 0x6e, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3,
0x20, 0x4d, 0x1ef9, 0x67, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x74, 0xe1, 0x52,
0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4d, 0x1ef9, 0x67,
0x61, 0x6e, 0x61, 0x6e, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4d,
0x1ef9, 0x72, 0x61, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x47, 0x6f, 0x6a, 0x20, 0x56, 0x1ebd, 0x73, 0x6f, 0x20, 0x4d, 0x1ef9, 0x72,
0x6e, 0x69, 0x76, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4d,
0x1ef9, 0x72, 0x6b, 0x65, 0x6a, 0x61, 0x20, 0x46, 0x61, 0x67, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4d, 0x1ef9, 0x72, 0x53, 0x61, 0x72, 0x20,
0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x4d, 0xe3, 0x72, 0x69, 0x73, 0x6f, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4d, 0xe3, 0x72, 0x69, 0x73,
0x6f, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72,
0x61, 0x20, 0x4d, 0xe3, 0x72, 0x69, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x4d, 0x1ef9, 0x75, 0x73, 0xe3, 0x6e, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x4d, 0x1ebd, 0x73, 0x69, 0x6b, 0x75, 0x20, 0x50,
0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x50, 0xe3, 0x20, 0x4d, 0x1ebd, 0x73, 0x69, 0x6b, 0x75, 0x20,
0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x75, 0x20, 0x74, 0xe1, 0x52,
0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4d, 0x1ebd, 0x73,
0x69, 0x6b, 0x75, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x75,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x55, 0x72, 0x61, 0x6e, 0x20,
0x4d, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x55, 0x72, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x74, 0x6f,
0x72, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72,
0x61, 0x20, 0x55, 0x72, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x74, 0x6f, 0x72,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4d, 0xe3, 0x6e, 0x68, 0x6b,
0x6f, 0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20,
0x4d, 0xe3, 0x6e, 0x68, 0x6b, 0x6f, 0x76, 0x20, 0x74, 0xe1, 0x52, 0x1ef9,
0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4d, 0xe3, 0x6e, 0x68,
0x6b, 0x6f, 0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4d, 0x129,
0x6a, 0x1ef9, 0x6d, 0x1ef9, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x4e, 0x1ef9, 0x76, 0x75, 0x72, 0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x4e, 0x1ebd, 0x70, 0x61, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x4b, 0x61, 0x72, 0x65, 0x6e, 0x6f, 0x6e, 0x129, 0x6a, 0x61, 0x20,
0x54, 0xe3, 0x67, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3,
0x20, 0x4b, 0x61, 0x72, 0x65, 0x6e, 0x6f, 0x6e, 0x129, 0x6a, 0x61, 0x20,
0x54, 0xe3, 0x67, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20,
0xf3, 0x72, 0x61, 0x20, 0x4b, 0x61, 0x72, 0x65, 0x6e, 0x6f, 0x6e, 0x129,
0x6a, 0x61, 0x20, 0x54, 0xe3, 0x67, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x4a, 0x65, 0x72, 0x1ef9, 0x6e, 0x69, 0x6a, 0x61, 0x20, 0x54, 0xe3,
0x67, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4a,
0x65, 0x72, 0x1ef9, 0x6e, 0x69, 0x6a, 0x61, 0x20, 0x54, 0xe3, 0x67, 0x20,
0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20,
0x4a, 0x65, 0x72, 0x1ef9, 0x6e, 0x69, 0x6a, 0x61, 0x20, 0x54, 0xe3, 0x67,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x47, 0x61, 0x20, 0x74, 0xe3,
0x67, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x47,
0x61, 0x20, 0x54, 0xe3, 0x67, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b,
0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x47, 0x61, 0x20, 0x54, 0xe3, 0x67,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4e, 0x129, 0x76, 0x65, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4e, 0xe3, 0x72, 0x66, 0x6f, 0x72,
0x6b, 0x69, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4e, 0xe3, 0x72,
0x66, 0x6f, 0x72, 0x6b, 0x69, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd,
0x73, 0x6f, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3,
0x72, 0x61, 0x20, 0x4e, 0xe3, 0x72, 0x66, 0x6f, 0x72, 0x6b, 0x69, 0x20,
0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x1ef9, 0x6e, 0x75, 0x20, 0x4e,
0xe3, 0x72, 0xe3, 0x6a, 0x61, 0x2d, 0x74, 0xe1, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x1ef9, 0x6e,
0x75, 0x20, 0x4e, 0xe3, 0x72, 0xe3, 0x6a, 0x61, 0x2d, 0x74, 0xe1, 0x20,
0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20,
0x46, 0x65, 0x72, 0x6e, 0x1ef9, 0x6e, 0x75, 0x20, 0x4e, 0xe3, 0x72, 0xe3,
0x6a, 0x61, 0x2d, 0x74, 0xe1, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x4e, 0xe3, 0x72, 0x74, 0x69, 0x2d, 0x4d, 0x1ef9, 0x72, 0x69, 0x6a, 0x1ef9,
0x6e, 0x1ef9, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x53, 0x69, 0x6d, 0x69, 0x72, 0x73,
0x69, 0x6b, 0x69, 0x20, 0x54, 0xe3, 0x67, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x50, 0xe3, 0x20, 0x53, 0x69, 0x6d, 0x69, 0x72, 0x73, 0x69,
0x6b, 0x69, 0x20, 0x54, 0xe3, 0x67, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20,
0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x53, 0x69,
0x6d, 0x69, 0x72, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x54, 0xe3, 0x67, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4f, 0x6d, 0x129, 0x6e, 0x68, 0x6b,
0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4f,
0x6d, 0x129, 0x6e, 0x68, 0x6b, 0x69, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20,
0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4f, 0x6d, 0x129, 0x6e, 0x68,
0x6b, 0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0x61, 0x6b,
0x69, 0x6e, 0x68, 0x74, 0x1ef9, 0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x50, 0xe3, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x6e, 0x68, 0x74, 0x1ef9,
0x76, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72,
0x61, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x6e, 0x68, 0x74, 0x1ef9, 0x76, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0x61, 0x72, 0x61, 0x76, 0x75,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0x61, 0x70, 0x75, 0x76,
0x61, 0x2d, 0x47, 0x69, 0x6e, 0x1ebd, 0x20, 0x54, 0xe3, 0x67, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x76,
0x61, 0x6a, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x76, 0x61, 0x6a, 0x20, 0x74, 0xe1,
0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x76, 0x61, 0x6a, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x50, 0x69, 0x72, 0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x50, 0xe3, 0x20, 0x50, 0x69, 0x72, 0x75, 0x20, 0x74, 0xe1, 0x52,
0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x50, 0x69, 0x72,
0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x46, 0x69, 0x72, 0x69,
0x70, 0x69, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50,
0xe3, 0x20, 0x46, 0x69, 0x72, 0x69, 0x70, 0x69, 0x6e, 0x1ef9, 0x20, 0x74,
0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0x46, 0x69, 0x72, 0x69, 0x70,
0x69, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x46, 0x1ebd,
0x6e, 0x129, 0x67, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x53, 0x1ef9, 0x20, 0x50, 0x65,
0x6e, 0x72, 0x75, 0x20, 0x6b, 0x61, 0x72, 0x20, 0x4d, 0x129, 0x6b, 0x65,
0x72, 0x1ef9, 0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3,
0x20, 0x53, 0x1ef9, 0x20, 0x50, 0x65, 0x64, 0x72, 0x6f, 0x20, 0x6b, 0x61,
0x72, 0x20, 0x4d, 0x129, 0x6b, 0x65, 0x72, 0x1ef9, 0x76, 0x20, 0x74, 0xe1,
0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x53, 0x1ef9,
0x20, 0x50, 0x65, 0x64, 0x72, 0x6f, 0x20, 0x6b, 0x61, 0x72, 0x20, 0x4d,
0x129, 0x6b, 0x65, 0x72, 0x1ef9, 0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x50, 0x69, 0x6e, 0x6b, 0x61, 0x6a, 0x69, 0x72, 0x20, 0x47, 0x6f,
0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x50, 0x6f, 0x6e, 0x1ef9, 0x70, 0x65, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x50, 0x69, 0xe3, 0x67, 0x69, 0x1ef9, 0x6e, 0x67, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4b, 0x79, 0x73, 0x79, 0x72, 0x6f, 0x72,
0x6e, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20,
0x4b, 0x79, 0x73, 0x79, 0x72, 0x6f, 0x72, 0x6e, 0x61, 0x20, 0x74, 0xe1,
0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4b, 0x79,
0x73, 0x79, 0x72, 0x6f, 0x72, 0x6e, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x48, 0x75, 0x6a, 0xe1, 0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x52, 0x6f, 0x74, 0x65, 0x72, 0x61, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x53, 0x61, 0x6b, 0x61, 0x72, 0x69, 0x6e, 0x1ef9, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x53, 0x61, 0x6b,
0x61, 0x72, 0x69, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b,
0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x53, 0x61, 0x6b, 0x61, 0x72, 0x69,
0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x53, 0x61, 0x6d,
0x1ef9, 0x72, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3,
0x20, 0x53, 0x61, 0x6d, 0x1ef9, 0x72, 0x61, 0x20, 0x74, 0xe1, 0x52, 0x1ef9,
0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x53, 0x61, 0x6d, 0x1ef9,
0x72, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x53, 0x61, 0x6d,
0xe3, 0x76, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3,
0x20, 0x53, 0x61, 0x6d, 0xe3, 0x76, 0x61, 0x20, 0x74, 0xe1, 0x52, 0x1ef9,
0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x53, 0x61, 0x6d, 0xe3,
0x76, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x53, 0x65, 0x6a,
0x73, 0x65, 0x72, 0x65, 0x6e, 0x68, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x50, 0xe3, 0x20, 0x53, 0x129, 0x67, 0x61, 0x70, 0x75, 0x72, 0x61,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x53, 0x61, 0x72, 0x6f, 0x6d,
0x1ef9, 0x76, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4a, 0x69, 0xf3, 0x72, 0x6a, 0x61,
0x20, 0x74, 0x1ef9, 0x20, 0x53, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x1ef9, 0x6d, 0x129, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x53, 0x69, 0x6f, 0x76, 0x61, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x54, 0x61, 0x6a, 0x74, 0x69, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x54, 0x61, 0x6a,
0x70, 0x65, 0x6a, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20,
0xf3, 0x72, 0x61, 0x20, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x54, 0x61, 0x6a, 0x69, 0x6b, 0x69, 0x6e,
0x68, 0x74, 0x1ef9, 0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x54,
0x6f, 0x6b, 0x65, 0x72, 0x61, 0x76, 0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x54, 0xe3, 0x67, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x50, 0xe3, 0x20, 0x54, 0xe3, 0x67, 0x61, 0x20, 0x74, 0xe1, 0x52,
0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x54, 0xe3, 0x67,
0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x53, 0x75, 0x75, 0x6b,
0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x54, 0x75, 0x72, 0x6b,
0x6f, 0x6d, 0x1ebd, 0x6e, 0x129, 0x6e, 0x68, 0x74, 0x1ef9, 0x76, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x54, 0x75, 0x72, 0x6b,
0x6f, 0x6d, 0x1ebd, 0x6e, 0x129, 0x6e, 0x68, 0x74, 0x1ef9, 0x76, 0x20, 0x74,
0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x54,
0x75, 0x72, 0x6b, 0x6f, 0x6d, 0x1ebd, 0x6e, 0x129, 0x6e, 0x68, 0x74, 0x1ef9,
0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x54, 0x75, 0x76, 0x61,
0x72, 0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x55, 0x72, 0x75,
0x67, 0x75, 0x76, 0x61, 0x6a, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x76, 0x61, 0x6a, 0x20,
0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20,
0x55, 0x72, 0x75, 0x67, 0x75, 0x76, 0x61, 0x6a, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x55, 0x6e, 0x68, 0x6d, 0x65, 0x6b, 0x69, 0x6e, 0x68,
0x74, 0x1ef9, 0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3,
0x20, 0x55, 0x6e, 0x6e, 0x6d, 0x65, 0x6b, 0x69, 0x6e, 0x68, 0x74, 0x1ef9,
0x76, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72,
0x61, 0x20, 0x55, 0x6e, 0x68, 0x6d, 0x65, 0x6b, 0x69, 0x6e, 0x68, 0x74,
0x1ef9, 0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x56, 0x61, 0x6e,
0x169, 0x76, 0x61, 0x74, 0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x56, 0x61, 0x6e, 0x169, 0x76, 0x61, 0x74, 0x75, 0x20,
0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20,
0x56, 0x61, 0x6e, 0x169, 0x76, 0x61, 0x74, 0x75, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x56, 0x65, 0x6e, 0x1ebd, 0x6a, 0x75, 0x76, 0xe9, 0x72,
0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x55, 0x72, 0x61, 0x6e,
0x69, 0x76, 0xf3, 0x6e, 0x68, 0x74, 0xf3, 0x6b, 0x69, 0x69, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x55, 0x72, 0x61, 0x6e,
0x69, 0x76, 0xf3, 0x6e, 0x68, 0x74, 0xf3, 0x6b, 0x69, 0x20, 0x74, 0xe1,
0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x55, 0x72,
0x61, 0x6e, 0x69, 0x76, 0xf3, 0x6e, 0x68, 0x74, 0xf3, 0x6b, 0x69, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x56, 0x6f, 0x72, 0x67, 0x75, 0x67,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50,
0xe3, 0x20, 0x56, 0x6f, 0x72, 0x67, 0x75, 0x67, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61,
0x20, 0x56, 0x6f, 0x72, 0x67, 0x75, 0x67, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x56, 0x6f, 0x6e, 0x68, 0x74, 0xf3,
0x6b, 0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x56, 0x65, 0x6a,
0x6b, 0x69, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x56, 0x61, 0x72, 0x69, 0x6e, 0x68,
0x20, 0x6b, 0x61, 0x72, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x1ef9, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x59, 0x6a, 0x61, 0x6b, 0x75, 0x74,
0x69, 0x6e, 0x68, 0x6b, 0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x59, 0x6a, 0x61, 0x6b, 0x75, 0x74, 0x69, 0x6e, 0x68,
0x6b, 0x69, 0x69, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20,
0xf3, 0x72, 0x61, 0x20, 0x59, 0x6a, 0x61, 0x6b, 0x75, 0x74, 0x69, 0x6e,
0x68, 0x6b, 0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x45, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x129, 0x6e, 0x6d, 0x75, 0x72, 0x67, 0x75, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x45, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x129, 0x6e, 0x6d, 0x75, 0x72, 0x67, 0x75, 0x20, 0x74,
0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x45,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x129, 0x6e, 0x6d, 0x75, 0x72, 0x67, 0x75,
0x20, 0x74, 0xe1, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x41, 0x63, 0x72, 0x65, 0x20, 0x79, 0x61, 0x72, 0x61, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61,
0x20, 0x41, 0x63, 0x72, 0x65, 0x20, 0x79, 0x61, 0x72, 0x61, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73,
0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x41, 0x63, 0x72, 0x65, 0x20, 0x79,
0x61, 0x72, 0x61, 0x41, 0x66, 0x65, 0x67, 0xe3, 0x6e, 0x69, 0x72, 0x65,
0x74, 0xe3, 0x6d, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x41, 0x66, 0x69, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x50, 0x69, 0x74,
0x65, 0x72, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x41, 0x66, 0x69, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x55, 0x72, 0x69, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x66, 0x69, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x53, 0x75, 0x20, 0x73, 0x75, 0xed, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x66, 0x69, 0x72, 0x69, 0x6b,
0x69, 0x61, 0x20, 0x55, 0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61,
0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41,
0x66, 0x69, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x55, 0x73, 0x69, 0x64, 0x1ebd,
0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x41, 0x66,
0x69, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x55, 0x73, 0x69, 0x64, 0x1ebd, 0x74,
0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed,
0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x41, 0x6c, 0x61, 0x73, 0x69, 0x6b, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x6c, 0x61, 0x73, 0x69, 0x6b, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x45, 0x74,
0xe9, 0x41, 0x6c, 0x61, 0x73, 0x69, 0x6b, 0x61, 0x20, 0x4b, 0x75, 0x72,
0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x41, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x75, 0x6d, 0x61, 0x74,
0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x45,
0x74, 0xe9, 0x41, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x20, 0x4b, 0x75, 0x72,
0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x41, 0x6d, 0x61, 0x73, 0x75, 0x6e, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x6d, 0x61, 0x73,
0x75, 0x6e, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x20, 0x45, 0x74, 0xe9, 0x41, 0x6d, 0x61, 0x73, 0x75, 0x6e, 0x61, 0x20,
0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0xd3, 0x72, 0x61, 0x20, 0x4b,
0x75, 0x6a, 0x75, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x20, 0x50, 0x69, 0x74, 0x65, 0x72,
0x61, 0x77, 0x61, 0x72, 0x61, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61,
0x20, 0x50, 0x69, 0x74, 0x65, 0x72, 0x61, 0x77, 0x61, 0x72, 0x61, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x4c, 0x65, 0x73, 0x69,
0x74, 0x69, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52,
0x65, 0x74, 0x65, 0x77, 0x61, 0x20, 0x4c, 0x65, 0x73, 0x69, 0x74, 0x69,
0x20, 0x79, 0x61, 0x72, 0x61, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61,
0x20, 0x4c, 0x65, 0x73, 0x69, 0x74, 0x69, 0xd3, 0x72, 0x61, 0x20, 0x4b,
0x72, 0x129, 0x2d, 0x61, 0x67, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x4b, 0x72, 0x129, 0x2d, 0x61, 0x67, 0x20, 0x74, 0xe1,
0x52, 0x1ef9, 0x20, 0x6b, 0xe3, 0x20, 0xd3, 0x72, 0x61, 0x20, 0x4b, 0x72,
0x129, 0x2d, 0x61, 0x67, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50,
0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x50, 0xe3, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b,
0x75, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xd3, 0x72,
0x61, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x75, 0x20, 0x74,
0xe1, 0x41, 0x6e, 0x61, 0x64, 0xed, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x41, 0x6e, 0x61, 0x64, 0xed, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61,
0x41, 0x6e, 0x61, 0x64, 0xed, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed,
0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x41, 0x70, 0x69, 0x79, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x41, 0x70, 0x69, 0x79, 0x61, 0x20, 0x55, 0x72, 0x61,
0x72, 0x75, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61,
0x41, 0x70, 0x69, 0x79, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed,
0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x41, 0x6b, 0x69, 0x74, 0x61, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x41, 0x6b, 0x69, 0x74, 0x61, 0x75, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65,
0x77, 0x61, 0x41, 0x6b, 0x69, 0x74, 0x61, 0x75, 0x20, 0x4b, 0x75, 0x72,
0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x41, 0x6b, 0x69, 0x74, 0x75, 0x62, 0x69, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x6b, 0x69, 0x74,
0x75, 0x62, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x41, 0x6b, 0x69, 0x74, 0x75,
0x62, 0x69, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x72,
0xe1, 0x62, 0x69, 0x79, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x41, 0x72, 0xe1, 0x62, 0x69, 0x79, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77,
0x61, 0x41, 0x72, 0x61, 0x62, 0x69, 0x79, 0x61, 0x20, 0x4b, 0x75, 0x72,
0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x41, 0x72, 0x67, 0x1ebd, 0x74, 0x69, 0x6e, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x72, 0x67,
0x1ebd, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x41, 0x72, 0x67,
0x1ebd, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed,
0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x41, 0x72, 0x67, 0x1ebd, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x55, 0x73,
0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x72, 0x67, 0x1ebd, 0x74, 0x69,
0x6e, 0x61, 0x20, 0x55, 0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61,
0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x41, 0x72, 0x67, 0x1ebd, 0x74, 0x69,
0x6e, 0x61, 0x20, 0x55, 0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61,
0x72, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x72,
0x69, 0x6d, 0x1ebd, 0x6e, 0x69, 0x79, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x41, 0x72, 0x69, 0x6d, 0x1ebd, 0x6e, 0x69, 0x79,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52,
0x65, 0x74, 0x65, 0x77, 0x61, 0x41, 0x72, 0x69, 0x6d, 0x1ebd, 0x6e, 0x69,
0x79, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x74,
0x61, 0x6c, 0xe3, 0x74, 0x69, 0x6b, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x41, 0x74, 0x61, 0x6c, 0xe3, 0x74, 0x69, 0x6b,
0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52,
0x65, 0x74, 0x65, 0x77, 0x61, 0x41, 0x74, 0x61, 0x6c, 0xe3, 0x74, 0x6b,
0x75, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x75, 0x73,
0x69, 0x74, 0x61, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x50, 0x69,
0x74, 0x65, 0x72, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x75, 0x73, 0x69, 0x74, 0x61, 0x72,
0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x50, 0x69, 0x74, 0x65, 0x72, 0x61,
0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x41, 0x75, 0x73, 0x69,
0x74, 0x61, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x50, 0x69, 0x74,
0x65, 0x72, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61,
0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x41, 0x75, 0x73, 0x69, 0x74, 0x61, 0x72, 0x61, 0x6c,
0x69, 0x79, 0x61, 0x20, 0x50, 0x69, 0x74, 0x65, 0x72, 0x61, 0x77, 0x61,
0x72, 0x61, 0x2d, 0x55, 0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61,
0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41,
0x75, 0x73, 0x69, 0x74, 0x61, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20,
0x50, 0x69, 0x74, 0x65, 0x72, 0x61, 0x77, 0x61, 0x72, 0x61, 0x2d, 0x55,
0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65,
0x77, 0x61, 0x41, 0x75, 0x73, 0x69, 0x74, 0x61, 0x72, 0x61, 0x6c, 0x69,
0x79, 0x61, 0x20, 0x50, 0x69, 0x74, 0x65, 0x72, 0x61, 0x77, 0x61, 0x72,
0x61, 0x2d, 0x55, 0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72,
0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x75, 0x73,
0x69, 0x74, 0x61, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x55, 0x72,
0x69, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x75, 0x73, 0x69, 0x74, 0x61, 0x72,
0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x55, 0x72, 0x69, 0x1ebd, 0x74, 0x61,
0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x41, 0x75, 0x73, 0x69,
0x74, 0x61, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x55, 0x72, 0x69,
0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61,
0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x41, 0x75, 0x73, 0x69, 0x74, 0x61, 0x72, 0x61, 0x6c,
0x69, 0x79, 0x61, 0x20, 0x55, 0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77,
0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x41, 0x75, 0x73, 0x69, 0x74, 0x61, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61,
0x20, 0x55, 0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65,
0x74, 0x65, 0x77, 0x61, 0x41, 0x75, 0x73, 0x69, 0x74, 0x61, 0x72, 0x61,
0x6c, 0x69, 0x79, 0x61, 0x20, 0x55, 0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61,
0x77, 0x61, 0x72, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20,
0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x41, 0x73, 0x65, 0x72, 0x69, 0x72, 0x65, 0x74, 0xe3, 0x6d, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x73, 0x65, 0x72,
0x69, 0x72, 0x65, 0x74, 0xe3, 0x6d, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x41,
0x73, 0x65, 0x72, 0x69, 0x72, 0x65, 0x74, 0xe3, 0x6d, 0x61, 0x20, 0x4b,
0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x73, 0x6f, 0x72, 0x69, 0x2d,
0x69, 0x74, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x41, 0x73, 0x6f, 0x72, 0x69, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77,
0x61, 0x41, 0x73, 0x6f, 0x72, 0x69, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x4b,
0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x42, 0xe3, 0x6b, 0x61, 0x72, 0x61,
0x64, 0x65, 0x78, 0x69, 0x20, 0x48, 0x6f, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x42, 0xe3, 0x6b, 0x61, 0x72, 0x61, 0x64, 0x65, 0x78, 0x69, 0x20,
0x48, 0x6f, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74,
0x65, 0x77, 0x61, 0x42, 0xe3, 0x6b, 0x61, 0x72, 0x61, 0x64, 0x65, 0x78,
0x69, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61,
0x20, 0x48, 0x6f, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x42, 0x75, 0x74,
0xe3, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x42,
0x75, 0x72, 0xed, 0x77, 0x69, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0xd3, 0x72, 0x61, 0x20, 0x42, 0x72, 0x61, 0x73, 0xed,
0x6c, 0x69, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3,
0x20, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x20, 0x74, 0xe1,
0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xd3, 0x72, 0x61, 0x20, 0x42, 0x72,
0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x20, 0x74, 0xe1, 0x42, 0x75, 0x72,
0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x61, 0x72,
0x61, 0x6d, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b,
0x61, 0x62, 0x75, 0x20, 0x53, 0x75, 0x69, 0x6b, 0x69, 0x72, 0x69, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x62, 0x75,
0x20, 0x53, 0x75, 0x69, 0x6b, 0x69, 0x72, 0x69, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61,
0x4b, 0x61, 0x62, 0x75, 0x20, 0x53, 0x75, 0x69, 0x6b, 0x69, 0x72, 0x69,
0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x58, 0x61, 0x6d, 0x6f,
0x72, 0x6f, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x58,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x58, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61,
0x58, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73,
0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x58, 0x69, 0x72, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x58, 0x69, 0x72, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x58,
0x69, 0x72, 0x69, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41,
0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x58,
0x69, 0x6e, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x58, 0x69, 0x6e, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x58, 0x69, 0x6e, 0x61,
0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x70, 0x75,
0xe3, 0x6d, 0x61, 0x4b, 0x69, 0x72, 0x69, 0x74, 0x69, 0x6d, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x70, 0x75,
0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x4b, 0x75, 0x6b, 0x75, 0x2d, 0x69,
0x74, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b,
0x75, 0x72, 0x169, 0x62, 0x69, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x4b, 0x75, 0x72, 0x169, 0x62, 0x69, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65,
0x77, 0x61, 0x4b, 0x75, 0x72, 0x169, 0x62, 0x69, 0x61, 0x20, 0x4b, 0x75,
0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x2d, 0x69,
0x74, 0x61, 0x20, 0x4b, 0x6f, 0x6f, 0x6b, 0x69, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x2d, 0x69,
0x74, 0x61, 0x20, 0x4b, 0x6f, 0x6f, 0x6b, 0x69, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61,
0x4b, 0x61, 0x70, 0x75, 0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x4b, 0x6f,
0x6f, 0x6b, 0x69, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41,
0x72, 0x61, 0x20, 0x50, 0x69, 0x74, 0x65, 0x72, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x75, 0x62, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x75, 0x62, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74,
0x65, 0x77, 0x61, 0x4b, 0x75, 0x62, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61,
0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x44, 0x61, 0x77, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64,
0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x54, 0x69, 0x6d, 0x75, 0x2d, 0x53, 0x65,
0x6d, 0x75, 0x73, 0x61, 0x77, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x50, 0x61, 0x73, 0x69, 0x6b, 0x75, 0x77, 0x61, 0x20,
0x4b, 0x61, 0x70, 0x75, 0xe3, 0x6d, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x50, 0x61, 0x73, 0x69, 0x6b, 0x75, 0x77, 0x61,
0x20, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x6d, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61,
0x50, 0x61, 0x73, 0x69, 0x6b, 0x75, 0x77, 0x61, 0x20, 0x4b, 0x61, 0x70,
0x75, 0xe3, 0x6d, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20,
0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x45, 0x6b, 0x75, 0x61, 0x64, 0xfa, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x45, 0x75, 0x72, 0x75, 0x70, 0x61, 0x20, 0x50, 0x69,
0x74, 0x65, 0x72, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x45, 0x75, 0x72, 0x75, 0x70, 0x61, 0x20,
0x50, 0x69, 0x74, 0x65, 0x72, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65,
0x77, 0x61, 0x45, 0x75, 0x72, 0x75, 0x70, 0x61, 0x20, 0x50, 0x69, 0x74,
0x65, 0x72, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61,
0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x45, 0x75, 0x72, 0x75, 0x70, 0x61, 0x20, 0x55, 0x72,
0x69, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x45, 0x75, 0x72, 0x75, 0x70, 0x61, 0x20,
0x55, 0x72, 0x69, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65,
0x77, 0x61, 0x45, 0x75, 0x72, 0x75, 0x70, 0x61, 0x20, 0x55, 0x72, 0x69,
0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61,
0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x45, 0x75, 0x72, 0x75, 0x70, 0x61, 0x20, 0x4c, 0x65,
0x73, 0x69, 0x74, 0x69, 0x2d, 0x65, 0x74, 0xe9, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x45, 0x75, 0x72, 0x75, 0x70, 0x61, 0x20,
0x55, 0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x45, 0x75, 0x72, 0x75,
0x70, 0x61, 0x20, 0x55, 0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61,
0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x45, 0x75, 0x72, 0x75, 0x70, 0x61,
0x20, 0x55, 0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61,
0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x70, 0x75,
0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x4d, 0x61, 0x77, 0x69, 0x6e, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x70,
0x75, 0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x4d, 0x61, 0x77, 0x69, 0x6e,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52,
0x65, 0x74, 0x65, 0x77, 0x61, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x2d, 0x69,
0x74, 0x61, 0x20, 0x4d, 0x61, 0x77, 0x69, 0x6e, 0x61, 0x20, 0x4b, 0x75,
0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x46, 0x69, 0x79, 0x69, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x46, 0x69, 0x79, 0x69, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65,
0x77, 0x61, 0x46, 0x69, 0x79, 0x69, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73,
0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x47, 0x69, 0x79, 0x61, 0x6e, 0x61, 0x20, 0x46, 0x72, 0xe3,
0x73, 0x61, 0x20, 0x79, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x54, 0x65, 0x74, 0xe3, 0x6d, 0x61, 0x20, 0x53,
0x75, 0x20, 0x73, 0x75, 0xed, 0x20, 0x46, 0x72, 0xe3, 0x73, 0x61, 0x20,
0x79, 0x61, 0x72, 0x61, 0x20, 0x61, 0x73, 0x75, 0xed, 0x20, 0xc3, 0x74,
0x61, 0x72, 0x74, 0x69, 0x64, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x47, 0x61, 0x72, 0x61, 0x70, 0x61, 0x67, 0x75, 0x2d,
0x69, 0x74, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x47, 0xe3, 0x62, 0x69, 0x65, 0x72, 0x65, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x47, 0x65, 0x75, 0x67, 0x69, 0x79, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x47, 0x65, 0x75, 0x67,
0x69, 0x79, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x47, 0x65, 0x75, 0x67, 0x69,
0x79, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61,
0x70, 0x75, 0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x59, 0x75, 0x62, 0x65,
0x74, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x20, 0x4d, 0x69, 0x72,
0x69, 0x64, 0x69, 0x79, 0x61, 0x6e, 0x75, 0x20, 0x79, 0x61, 0x72, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x47, 0x75, 0x72,
0x75, 0x1ebd, 0x72, 0xe3, 0x64, 0x69, 0x79, 0x61, 0x20, 0x55, 0x72, 0x69,
0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x47, 0x75, 0x72, 0x75, 0x1ebd, 0x72, 0xe3, 0x64,
0x69, 0x79, 0x61, 0x20, 0x55, 0x72, 0x69, 0x1ebd, 0x74, 0x61, 0x77, 0x61,
0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x47, 0x75, 0x72, 0x75, 0x1ebd, 0x72,
0xe3, 0x64, 0x69, 0x79, 0x61, 0x20, 0x55, 0x72, 0x69, 0x1ebd, 0x74, 0x61,
0x77, 0x61, 0x72, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20,
0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x47, 0x75, 0x72, 0x75, 0x1ebd, 0x72, 0xe3, 0x64, 0x69, 0x79, 0x61, 0x20,
0x55, 0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x47, 0x75, 0x72, 0x75,
0x1ebd, 0x72, 0xe3, 0x64, 0x69, 0x79, 0x61, 0x20, 0x55, 0x73, 0x69, 0x64,
0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x47,
0x75, 0x72, 0x75, 0x1ebd, 0x72, 0xe3, 0x64, 0x69, 0x79, 0x61, 0x20, 0x55,
0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x4b,
0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x47, 0x75, 0x77, 0xe3, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65,
0x77, 0x61, 0x47, 0x6f, 0x6c, 0x66, 0x6f, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x47, 0x69, 0x79, 0x61, 0x6e, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x48, 0x61, 0x77, 0x61, 0xed,
0x20, 0x61, 0x73, 0x75, 0xed, 0x20, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x2d,
0x69, 0x74, 0x61, 0x20, 0x41, 0x72, 0x65, 0x75, 0x74, 0x61, 0x2d, 0x69,
0x74, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x48,
0x61, 0x77, 0x61, 0xed, 0x20, 0x61, 0x73, 0x75, 0xed, 0x20, 0x4b, 0x61,
0x70, 0x75, 0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x41, 0x72, 0x65, 0x75,
0x74, 0x61, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x48, 0x61,
0x77, 0x61, 0xed, 0x20, 0x61, 0x73, 0x75, 0xed, 0x20, 0x4b, 0x61, 0x70,
0x75, 0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x41, 0x72, 0x65, 0x75, 0x74,
0x61, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed,
0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x48, 0x169, 0x67, 0x20, 0x4b, 0x169, 0x67, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x48, 0x169, 0x67, 0x20, 0x4b, 0x169, 0x67,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65,
0x74, 0x65, 0x77, 0x61, 0x48, 0x169, 0x67, 0x20, 0x4b, 0x169, 0x67, 0x20,
0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x48, 0x6f, 0x77, 0x69, 0x64,
0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x48, 0x6f,
0x77, 0x69, 0x64, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x48, 0x6f, 0x77, 0x69,
0x64, 0x69, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x128, 0x64,
0x69, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x55, 0x73, 0x65, 0x79, 0x61, 0x6e,
0x75, 0x20, 0x128, 0x64, 0x69, 0x6b, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x128, 0x64, 0x6f, 0x78, 0x69, 0x6e, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x128, 0x64, 0x6f, 0x6e,
0x65, 0x73, 0x69, 0x79, 0x61, 0x20, 0x50, 0x69, 0x74, 0x65, 0x72, 0x61,
0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x128, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x79, 0x61, 0x20, 0x55,
0x72, 0x69, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x128, 0x64, 0x6f, 0x6e, 0x65, 0x73,
0x69, 0x79, 0x61, 0x20, 0x55, 0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77,
0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x49, 0x72, 0xe3, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x49, 0x72, 0xe3, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x49, 0x72, 0xe3, 0x20, 0x4b,
0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x69,
0x73, 0x69, 0x6b, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74,
0x65, 0x77, 0x61, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x69, 0x73, 0x69, 0x6b,
0x69, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x49, 0x73, 0x69,
0x72, 0x61, 0x79, 0x65, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x49, 0x73, 0x69, 0x72, 0x61, 0x79, 0x65, 0x75, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65,
0x77, 0x61, 0x49, 0x73, 0x69, 0x72, 0x61, 0x79, 0x65, 0x75, 0x20, 0x4b,
0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4e, 0x69, 0x70, 0xf5, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4e, 0x69, 0x70, 0xf5, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74,
0x65, 0x77, 0x61, 0x4e, 0x69, 0x70, 0xf5, 0x20, 0x4b, 0x75, 0x72, 0x61,
0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c,
0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74,
0x73, 0x6b, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73,
0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65,
0x74, 0x65, 0x77, 0x61, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76,
0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61,
0x74, 0x73, 0x6b, 0x69, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20,
0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x4b, 0x61, 0x73, 0x61, 0x6b, 0x69, 0x72, 0x65, 0x74, 0xe3, 0x6d, 0x61,
0x20, 0x55, 0x72, 0x69, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x73, 0x61,
0x6b, 0x69, 0x72, 0x65, 0x74, 0xe3, 0x6d, 0x61, 0x20, 0x55, 0x73, 0x69,
0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x75, 0x72, 0x65, 0x79, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x75, 0x72, 0x65,
0x79, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x4b, 0x75, 0x72, 0x65, 0x79, 0x61,
0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x75, 0x73, 0x69,
0x72, 0x61, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x4b, 0x61, 0x72, 0x61, 0x73, 0x69, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x69,
0x73, 0x69, 0x6b, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x4b, 0x61, 0x72, 0x61, 0x73, 0x69, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x69, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x4b, 0x61, 0x72,
0x61, 0x73, 0x69, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x69, 0x73, 0x69, 0x6b,
0x69, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x69, 0x72,
0x69, 0x67, 0x69, 0x72, 0x65, 0x74, 0xe3, 0x6d, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x52, 0xe3, 0x6b, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x70, 0x75, 0xe3,
0x2d, 0x69, 0x74, 0x61, 0x20, 0x49, 0x6e, 0x129, 0x62, 0x75, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4c, 0x75, 0x72, 0x75, 0x64,
0x69, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x4c, 0x75, 0x72, 0x75, 0x64, 0x69, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x4c, 0x75, 0x72, 0x75, 0x64, 0x69,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed,
0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x4d, 0x61, 0x6b, 0x61, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x4d, 0x61, 0x6b, 0x61, 0x75, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61,
0x4d, 0x61, 0x6b, 0x61, 0x75, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed,
0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x4d, 0x61, 0x67, 0x61, 0x64, 0xe3, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x4d, 0x61, 0x67, 0x61, 0x64, 0xe3, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65,
0x77, 0x61, 0x4d, 0x61, 0x67, 0x61, 0x64, 0xe3, 0x20, 0x4b, 0x75, 0x72,
0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x4d, 0x61, 0x6c, 0x61, 0x73, 0x69, 0x79, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x70,
0x75, 0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x4d, 0x61, 0x75, 0x64, 0x69,
0x77, 0x61, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x4d, 0x61, 0x6b, 0x65, 0x73, 0x61, 0x2d, 0x69, 0x74,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61,
0x70, 0x75, 0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x4d, 0x61, 0x72, 0x73,
0x68, 0x61, 0x6c, 0x6c, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x73, 0x69, 0x79, 0x75, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x73, 0x69, 0x79, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x73, 0x69, 0x79, 0x75, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed,
0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x4d, 0x61, 0x75, 0x73, 0x169, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x75, 0x20,
0x4d, 0x65, 0x78, 0x69, 0x6b, 0x61, 0x6e, 0x75, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b,
0x75, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x61, 0x6e, 0x75, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65,
0x77, 0x61, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x75, 0x20, 0x4d,
0x65, 0x78, 0x69, 0x6b, 0x61, 0x6e, 0x75, 0x20, 0x4b, 0x75, 0x72, 0x61,
0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x55, 0x72, 0xe3, 0x20, 0x42, 0x61, 0x74, 0x75, 0x72,
0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x55, 0x72,
0xe3, 0x20, 0x42, 0x61, 0x74, 0x75, 0x72, 0x75, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61,
0x55, 0x72, 0xe3, 0x20, 0x42, 0x61, 0x74, 0x75, 0x72, 0x75, 0x20, 0x4b,
0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x75,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4d, 0x6f, 0x73,
0x6b, 0x6f, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x4d, 0x6f, 0x73, 0x6b, 0x6f,
0x75, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4d, 0x69, 0x79,
0x61, 0x6d, 0xe1, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x4e, 0x61, 0x75, 0x72, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x4e, 0x65, 0x70, 0x61, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x72, 0x65, 0x64, 0x169, 0x6e, 0x69,
0x79, 0x61, 0x20, 0x50, 0x69, 0x73, 0x61, 0x73, 0xfa, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x72, 0x65, 0x64, 0x169,
0x6e, 0x69, 0x79, 0x61, 0x20, 0x50, 0x69, 0x73, 0x61, 0x73, 0xfa, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74,
0x65, 0x77, 0x61, 0x4b, 0x61, 0x72, 0x65, 0x64, 0x169, 0x6e, 0x69, 0x79,
0x61, 0x20, 0x50, 0x69, 0x73, 0x61, 0x73, 0xfa, 0x20, 0x4b, 0x75, 0x72,
0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x53, 0x65, 0x72, 0xe3, 0x64, 0x69, 0x79, 0x61,
0x20, 0x50, 0x69, 0x73, 0x61, 0x73, 0xfa, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x53, 0x65, 0x72, 0xe3, 0x64, 0x69, 0x79, 0x61,
0x20, 0x50, 0x69, 0x73, 0x61, 0x73, 0xfa, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x53,
0x65, 0x72, 0xe3, 0x64, 0x69, 0x79, 0x61, 0x20, 0x50, 0x69, 0x73, 0x61,
0x73, 0xfa, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x49, 0x77,
0xed, 0x20, 0x50, 0x69, 0x73, 0x61, 0x73, 0xfa, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x49, 0x77, 0xed, 0x20, 0x50, 0x69, 0x73,
0x61, 0x73, 0xfa, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x49, 0x77, 0xed, 0x20, 0x50,
0x69, 0x73, 0x61, 0x73, 0xfa, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed,
0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x4e, 0x69, 0x77, 0x65, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x6d, 0x61, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x6d, 0x61, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x4b, 0x61, 0x70,
0x75, 0xe3, 0x6d, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x46, 0x65, 0x6e, 0xe3,
0x64, 0x75, 0x20, 0x4e, 0x75, 0x72, 0x75, 0x79, 0xe3, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x46, 0x65, 0x6e, 0xe3, 0x64, 0x75,
0x20, 0x4e, 0x75, 0x72, 0x75, 0x79, 0xe3, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x46,
0x65, 0x6e, 0xe3, 0x64, 0x75, 0x20, 0x4e, 0x75, 0x72, 0x75, 0x79, 0xe3,
0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x70, 0x75,
0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x79, 0xe3,
0x6e, 0x61, 0x20, 0x4e, 0x75, 0x74, 0x69, 0x20, 0x73, 0x75, 0xed, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x53, 0x69, 0x62, 0x69,
0x72, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x50, 0x69, 0x73, 0x61, 0x73,
0xfa, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x53, 0x69,
0x62, 0x69, 0x72, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x50, 0x69, 0x73,
0x61, 0x73, 0xfa, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x53, 0x69, 0x62, 0x69, 0x72,
0x69, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x50, 0x69, 0x73, 0x61, 0x73, 0xfa,
0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4f, 0x6d, 0x69, 0x73,
0x69, 0x6b, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x4f, 0x6d, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x4f,
0x6d, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73,
0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x50, 0x61, 0x6b, 0x69, 0x72, 0x65, 0x74, 0xe3, 0x6d, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x50, 0x61, 0x6b,
0x69, 0x72, 0x65, 0x74, 0xe3, 0x6d, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x50,
0x61, 0x6b, 0x69, 0x72, 0x65, 0x74, 0xe3, 0x6d, 0x61, 0x20, 0x4b, 0x75,
0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x50, 0x61, 0x72, 0x61, 0x75, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x50, 0x61, 0x70, 0x75, 0x77,
0x61, 0x2d, 0x47, 0x69, 0x6e, 0xe9, 0x20, 0x50, 0x69, 0x73, 0x61, 0x73,
0xfa, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x77, 0x61, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x77, 0x61,
0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52,
0x65, 0x74, 0x65, 0x77, 0x61, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x77,
0x61, 0x69, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x50, 0x65, 0x72,
0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x50, 0x65,
0x72, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x50, 0x65, 0x72, 0x75, 0x20, 0x4b,
0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x46, 0x69, 0x72, 0x69, 0x70, 0x69,
0x6e, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x46,
0x69, 0x72, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x46,
0x69, 0x72, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61,
0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x2d, 0x69, 0x74, 0x61,
0x20, 0x46, 0x1ebd, 0x6e, 0x69, 0x78, 0x53, 0xe3, 0x20, 0x50, 0x65, 0x64,
0x75, 0x72, 0x75, 0x20, 0x61, 0x73, 0x75, 0xed, 0x20, 0x4d, 0x69, 0x6b,
0x69, 0x72, 0xe3, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x53, 0xe3, 0x20, 0x50, 0x65, 0x64, 0x75, 0x72, 0x75, 0x20, 0x61,
0x73, 0x75, 0xed, 0x20, 0x4d, 0x69, 0x6b, 0x69, 0x72, 0xe3, 0x75, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74,
0x65, 0x77, 0x61, 0x53, 0xe3, 0x20, 0x50, 0x65, 0x64, 0x75, 0x72, 0x75,
0x20, 0x61, 0x73, 0x75, 0xed, 0x20, 0x4d, 0x69, 0x6b, 0x69, 0x72, 0xe3,
0x75, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x50, 0x69, 0x74,
0x63, 0x61, 0x69, 0x72, 0x6e, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x50, 0x69, 0xfa, 0x67, 0x69, 0xe3, 0x67,
0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x69,
0x73, 0x69, 0x72, 0x6f, 0x64, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x4b, 0x69, 0x73, 0x69, 0x72, 0x6f, 0x64, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74,
0x65, 0x77, 0x61, 0x4b, 0x69, 0x73, 0x69, 0x72, 0x6f, 0x64, 0x61, 0x20,
0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x59, 0x75, 0x6d, 0x75, 0x61,
0x74, 0x69, 0x72, 0x69, 0x73, 0x61, 0x77, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x52, 0x6f, 0x74, 0x65, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x53, 0x61, 0x6b, 0x61,
0x72, 0x69, 0x6e, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x53, 0x61, 0x6b, 0x61, 0x72, 0x69, 0x6e, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77,
0x61, 0x53, 0x61, 0x6b, 0x61, 0x72, 0x69, 0x6e, 0x61, 0x20, 0x4b, 0x75,
0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x53, 0x61, 0x6d, 0x6f,
0x77, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x53,
0x61, 0x6d, 0x6f, 0x77, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x53, 0x61, 0x6d,
0x6f, 0x77, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41,
0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x53,
0x65, 0x69, 0x78, 0x65, 0x72, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x53, 0x129, 0x67, 0x61, 0x70, 0x75, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74,
0x65, 0x77, 0x61, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x6d, 0x61, 0x2d, 0x69,
0x74, 0x61, 0x20, 0x53, 0x61, 0x72, 0x75, 0x6d, 0x169, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x47, 0x65, 0x75, 0x67, 0x69, 0x79,
0x61, 0x20, 0x53, 0x75, 0x20, 0x73, 0x75, 0xed, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d,
0x65, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x53, 0x69,
0x79, 0x6f, 0x77, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x54, 0x61, 0x69, 0x74, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65,
0x74, 0x65, 0x77, 0x61, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x4b,
0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x54, 0x61, 0x79, 0x69, 0x6b, 0x69,
0x72, 0x65, 0x74, 0xe3, 0x6d, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x54, 0x6f, 0x6b, 0x65, 0x72, 0x61, 0x75, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x54, 0xf5, 0x67, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x54, 0xf5, 0x67, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65,
0x74, 0x65, 0x77, 0x61, 0x54, 0xf5, 0x67, 0x61, 0x20, 0x4b, 0x75, 0x72,
0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x54, 0x75, 0x72, 0x6b, 0x75, 0x72,
0x61, 0x6e, 0x61, 0x72, 0x65, 0x74, 0xe3, 0x6d, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x54, 0x75, 0x72, 0x6b, 0x75, 0x72,
0x61, 0x6e, 0x61, 0x72, 0x65, 0x74, 0xe3, 0x6d, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77,
0x61, 0x54, 0x75, 0x72, 0x6b, 0x75, 0x72, 0x61, 0x6e, 0x61, 0x72, 0x65,
0x74, 0xe3, 0x6d, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20,
0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x54, 0x75, 0x76, 0x61, 0x72, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x55, 0x72, 0x75, 0x67, 0x75, 0x77, 0x61, 0x69, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x55, 0x72, 0x75, 0x67,
0x75, 0x77, 0x61, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x55, 0x72, 0x75, 0x67,
0x75, 0x77, 0x61, 0x69, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20,
0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x59, 0x169, 0x62, 0x75, 0x65, 0x73, 0x61, 0x72, 0x61, 0x2d, 0x72, 0x65,
0x74, 0xe3, 0x6d, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x59, 0x169, 0x62, 0x75, 0x65, 0x73, 0x61, 0x72, 0x61, 0x2d, 0x72,
0x65, 0x74, 0xe3, 0x6d, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x59, 0x169, 0x62,
0x75, 0x65, 0x73, 0x61, 0x72, 0x61, 0x2d, 0x72, 0x65, 0x74, 0xe3, 0x6d,
0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x57, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x57, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61,
0x57, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x4b, 0x75, 0x72, 0x61,
0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x57, 0x65, 0x6e, 0x1ebd, 0x73, 0x75, 0x65, 0x72, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x57, 0x61, 0x72,
0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x69, 0x74, 0x75, 0x6b, 0x69, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x57, 0x61, 0x72, 0x61,
0x64, 0x69, 0x77, 0x6f, 0x73, 0x69, 0x74, 0x75, 0x6b, 0x69, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65,
0x77, 0x61, 0x57, 0x61, 0x72, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x69,
0x74, 0x75, 0x6b, 0x69, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20,
0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x57, 0x6f, 0x72, 0x77, 0x6f, 0x67, 0x61, 0x72, 0x61, 0x64, 0x75, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x57, 0x6f, 0x72, 0x77,
0x6f, 0x67, 0x61, 0x72, 0x61, 0x64, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x57,
0x6f, 0x72, 0x77, 0x6f, 0x67, 0x61, 0x72, 0x61, 0x64, 0x75, 0x20, 0x4b,
0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x57, 0x6f, 0x73, 0x69, 0x74, 0x75,
0x6b, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b,
0x61, 0x70, 0x75, 0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x57, 0x61, 0x6b,
0x65, 0x57, 0x61, 0x72, 0x69, 0x20, 0x61, 0x73, 0x75, 0xed, 0x20, 0x46,
0x75, 0x74, 0x75, 0x6e, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x69, 0x73, 0x69, 0x6b, 0x69,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x59, 0x61, 0x6b,
0x75, 0x74, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x59,
0x61, 0x6b, 0x75, 0x74, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x4b, 0x75,
0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x129,
0x62, 0x75, 0x67, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x129, 0x62, 0x75, 0x67, 0x75,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65,
0x74, 0x65, 0x77, 0x61, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x129, 0x62,
0x75, 0x67, 0x75, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41,
0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x42,
0x75, 0x74, 0x61, 0x6e, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x53, 0xe3, 0x20, 0x50, 0x65, 0x64, 0x75, 0x72, 0x75, 0x20, 0x61,
0x73, 0x75, 0xed, 0x20, 0x4d, 0x69, 0x6b, 0x69, 0x72, 0x61, 0x6e, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x53, 0xe3, 0x20, 0x50,
0x65, 0x64, 0x75, 0x72, 0x75, 0x20, 0x61, 0x73, 0x75, 0xed, 0x20, 0x4d,
0x69, 0x6b, 0x69, 0x72, 0x61, 0x6e, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x53, 0xe3,
0x20, 0x50, 0x65, 0x64, 0x75, 0x72, 0x75, 0x20, 0x61, 0x73, 0x75, 0xed,
0x20, 0x4d, 0x69, 0x6b, 0x69, 0x72, 0x61, 0x6e, 0x20, 0x4b, 0x75, 0x72,
0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x41, 0x66, 0x67, 0x68, 0x61, 0x61, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x6f, 0x6f, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x55, 0x61, 0x73, 0x74, 0x20,
0x41, 0x66, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x53, 0xfc, 0xfc, 0x64, 0x65, 0x6c, 0x6b, 0x20, 0x41, 0x66, 0x72, 0x69,
0x6b, 0x6f, 0x6f, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x20, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x57,
0x61, 0x61, 0x73, 0x74, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x6f, 0x6f,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x20, 0x41, 0x66, 0x72, 0x69,
0x6b, 0x6f, 0x6f, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x6a, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x41,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x20, 0x53, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x6a, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61,
0x6c, 0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c,
0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x20, 0x41, 0x6d, 0x65,
0x65, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x6a, 0x55, 0x61, 0x73, 0x74, 0x20, 0x41, 0x6d, 0x65,
0x65, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x55,
0x61, 0x73, 0x74, 0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x6f,
0x6f, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x55, 0x61, 0x73, 0x74, 0x20, 0x41, 0x6d, 0x65, 0x65,
0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x20, 0x41, 0x6d, 0x65,
0x65, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x57,
0x61, 0x61, 0x73, 0x74, 0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b,
0x6f, 0x6f, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x20, 0x41, 0x6d,
0x65, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x53, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x6a, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b,
0x6f, 0x20, 0x50, 0x61, 0x73, 0x69, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x6f,
0x20, 0x50, 0x61, 0x73, 0x69, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x41,
0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x50, 0x61, 0x73,
0x69, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x6a, 0x41, 0x70, 0x69, 0x61, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x41, 0x70, 0x69, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x41, 0x70, 0x69, 0x61, 0x20, 0x53,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x41, 0x72, 0x61, 0x61,
0x62, 0x69, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x41, 0x72, 0x61,
0x61, 0x62, 0x69, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x41, 0x72, 0x61, 0x61, 0x62,
0x69, 0x73, 0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x6a, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x69, 0x65,
0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x69, 0x6e, 0x69, 0x65, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x69, 0x6e, 0x69, 0x65, 0x6e, 0x20, 0x53, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x20,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x69, 0x65, 0x6e,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x20, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x69, 0x65, 0x6e, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x69, 0x6e, 0x69, 0x65, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x6a, 0x41, 0x72, 0x6d, 0x65, 0x65, 0x6e, 0x69,
0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x41, 0x72, 0x6d, 0x65, 0x65,
0x6e, 0x69, 0x65, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x41, 0x72, 0x6d, 0x65, 0x65, 0x6e,
0x69, 0x65, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x6a, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x41, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x41, 0x6d, 0x65, 0x65,
0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a,
0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x6a, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61,
0x6c, 0x20, 0x57, 0x61, 0x61, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x20, 0x57, 0x61, 0x61, 0x73,
0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x73, 0x6b,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x20, 0x57,
0x61, 0x61, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c,
0x69, 0x73, 0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x6a, 0x55, 0x61, 0x73, 0x74, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x55, 0x61,
0x73, 0x74, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x69,
0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x55, 0x61, 0x73, 0x74, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x53, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x57,
0x61, 0x61, 0x73, 0x74, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61,
0x6c, 0x69, 0x73, 0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x6a, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x73, 0x63,
0x68, 0x61, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x41, 0x73, 0x65,
0x72, 0x62, 0x61, 0x69, 0x64, 0x73, 0x63, 0x68, 0x61, 0x61, 0x6e, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x73, 0x63, 0x68,
0x61, 0x61, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x6a, 0x41, 0x74, 0x73, 0x6f, 0x6f, 0x72, 0x65, 0x6e, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x41, 0x74, 0x73, 0x6f, 0x6f, 0x72, 0x65, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x41, 0x74, 0x73, 0x6f, 0x6f, 0x72, 0x65, 0x6e, 0x20, 0x53, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x73, 0x63, 0x68, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x42, 0x68, 0x75, 0x75,
0x74, 0x61, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x42, 0x6f, 0x6c,
0x69, 0x69, 0x77, 0x69, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x42,
0x72, 0x61, 0x73, 0x69, 0x69, 0x6c, 0x69, 0x65, 0x6e, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x42, 0x72, 0x61, 0x73, 0x69, 0x69, 0x6c, 0x69, 0x65, 0x6e,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x42, 0x72, 0x61, 0x73, 0x69, 0x69, 0x6c, 0x69, 0x65, 0x6e,
0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x42, 0x72,
0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61,
0x6c, 0x61, 0x6d, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4b, 0x61, 0x70, 0x77,
0x65, 0x72, 0x64, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4b, 0x61,
0x70, 0x77, 0x65, 0x72, 0x64, 0x65, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4b, 0x61, 0x70,
0x77, 0x65, 0x72, 0x64, 0x65, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x6a, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72,
0x6f, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x20, 0x53,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x43, 0x68, 0x69, 0x69,
0x6c, 0x65, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x43, 0x68, 0x69, 0x69, 0x6c,
0x65, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x43, 0x68, 0x69, 0x69, 0x6c, 0x65, 0x20, 0x53, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x53, 0x63, 0x68, 0x69, 0x69,
0x6e, 0x61, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53, 0x63, 0x68, 0x69, 0x69,
0x6e, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x53, 0x63, 0x68, 0x69, 0x69, 0x6e, 0x61, 0x20,
0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x4a, 0x75, 0x6c,
0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x43,
0x6f, 0x63, 0x6f, 0x73, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x65, 0x6e,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69,
0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4b, 0x6f, 0x6c, 0x75, 0x6d,
0x62, 0x69, 0x65, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62,
0x69, 0x65, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x6a, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x65,
0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x45,
0x69, 0x6c, 0x75, 0x6e, 0x65, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x43, 0x6f, 0x6f, 0x6b,
0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x65, 0x6e, 0x20, 0x48, 0x75, 0x61,
0x6c, 0x65, 0x77, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x6a, 0x4b, 0x75, 0x75, 0x62, 0x61, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4b,
0x75, 0x75, 0x62, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4b, 0x75, 0x75, 0x62, 0x61, 0x20,
0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x44, 0x61, 0x76,
0x69, 0x73, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x44, 0x75, 0x6d, 0x6f, 0x6e,
0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x55, 0x61, 0x73, 0x74, 0x20, 0x54, 0x69, 0x69,
0x6d, 0x6f, 0x72, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x50, 0x75, 0x61, 0x73,
0x6b, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x50, 0x75, 0x61, 0x73, 0x6b, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x50, 0x75, 0x61, 0x73, 0x6b, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e,
0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x45, 0x6b,
0x77, 0x61, 0x64, 0x6f, 0x6f, 0x72, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4d,
0x61, 0x64, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x65, 0x73,
0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4d, 0x61, 0x64, 0x65, 0x6c, 0x65,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x65, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4d, 0x61,
0x64, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x65, 0x73, 0x6b,
0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x55, 0x61,
0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x65, 0x73, 0x6b, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x55, 0x61, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x65, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x55, 0x61, 0x73, 0x74, 0x65,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x65, 0x73, 0x6b, 0x20, 0x53, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x46, 0x61, 0x61, 0x72, 0x64, 0x65,
0x72, 0x20, 0x55, 0x61, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x65, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73,
0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x65, 0x73, 0x6b, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x65, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73, 0x74,
0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x65, 0x73, 0x6b, 0x20, 0x53, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x75, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x65, 0x6e, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x75, 0x6e, 0x20, 0x45,
0x69, 0x6c, 0x75, 0x6e, 0x65, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x75, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x65, 0x6e, 0x20,
0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x46, 0x69, 0x64,
0x73, 0x63, 0x68, 0x69, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x46, 0x69, 0x64,
0x73, 0x63, 0x68, 0x69, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x46, 0x69, 0x64, 0x73, 0x63, 0x68,
0x69, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x46,
0x72, 0x61, 0x6e, 0x73, 0xf6, 0xf6, 0x73, 0x6b, 0x20, 0x47, 0x75, 0x61,
0x79, 0x61, 0x61, 0x6e, 0x61, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x46, 0x72,
0x61, 0x6e, 0x73, 0xf6, 0xf6, 0x73, 0x6b, 0x20, 0x53, 0xfc, 0xfc, 0x64,
0x20, 0x61, 0x6e, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69,
0x73, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61,
0x67, 0x6f, 0x73, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x47, 0x61, 0x6d, 0x62,
0x69, 0x65, 0x72, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x47, 0x65, 0x6f, 0x72,
0x67, 0x69, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x65, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x47, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x65, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x6a, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x20, 0x45, 0x69, 0x6c,
0x75, 0x6e, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4d, 0x61, 0x64,
0x65, 0x6c, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x55, 0x61, 0x73, 0x74, 0x20, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x6c, 0x75, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x55,
0x61, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x75, 0x6e,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x55, 0x61, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x6c, 0x75, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x6c, 0x75, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73,
0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x75, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x57, 0x61, 0x61, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c,
0x75, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a,
0x47, 0x6f, 0x6c, 0x66, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x47, 0x75, 0x79,
0x61, 0x61, 0x6e, 0x61, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x65, 0x6e, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x65, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x48, 0x61, 0x77, 0x61,
0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x65, 0x6e, 0x20, 0x53,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x48, 0x6f, 0x6e, 0x67,
0x6b, 0x6f, 0x6e, 0x67, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x48, 0x6f, 0x6e,
0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x48, 0x6f, 0x6e, 0x67, 0x6b,
0x6f, 0x6e, 0x67, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x6a, 0x43, 0x68, 0x6f, 0x77, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x43,
0x68, 0x6f, 0x77, 0x64, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x43, 0x68, 0x6f, 0x77, 0x64, 0x20,
0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x49, 0x6e, 0x64,
0x69, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x49, 0x6e, 0x64, 0x69,
0x73, 0x6b, 0x20, 0x4f, 0x6f, 0x73, 0x65, 0x61, 0x61, 0x6e, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x49, 0x6e, 0x64, 0x6f, 0x73, 0x63, 0x68, 0x69, 0x69,
0x6e, 0x61, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x61, 0x6c, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x65, 0x73,
0x69, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x55, 0x61, 0x73, 0x74,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x65, 0x73, 0x69, 0x65, 0x6e,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x20, 0x49,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x65, 0x73, 0x69, 0x65, 0x6e, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x49, 0x72, 0x61, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x49, 0x72, 0x61, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x49, 0x72, 0x61, 0x61,
0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x49,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x49,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x6a, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a,
0x4a, 0x61, 0x61, 0x70, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4a,
0x61, 0x61, 0x70, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4a, 0x61, 0x61, 0x70, 0x61,
0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x4b,
0x61, 0x73, 0x61, 0x63, 0x68, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x55, 0x61, 0x73, 0x74, 0x20, 0x4b, 0x61, 0x73, 0x61,
0x63, 0x68, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x57, 0x61, 0x61, 0x73, 0x74, 0x20, 0x4b, 0x61, 0x73, 0x61, 0x63, 0x68,
0x73, 0x74, 0x61, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4b, 0x6f,
0x72, 0x65, 0x61, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4b, 0x6f, 0x72, 0x65,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x53, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a,
0x61, 0x72, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x53, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x4b, 0x69, 0x72, 0x67, 0x69,
0x73, 0x74, 0x61, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4c, 0x69,
0x6e, 0x65, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x65, 0x6e, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x6a, 0x4d, 0x61, 0x6c, 0x61, 0x79, 0x73, 0x69, 0x61, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x4d, 0x61, 0x6c, 0x65, 0x64, 0x69, 0x69, 0x77, 0x65, 0x6e,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73,
0x61, 0x73, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4d, 0x61, 0x72, 0x73, 0x68,
0x61, 0x6c, 0x6c, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x65, 0x6e, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74,
0x69, 0x75, 0x73, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69,
0x75, 0x73, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a,
0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4d,
0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x20, 0x50, 0x61, 0x73, 0x69, 0x69,
0x66, 0x69, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4d, 0x65, 0x6b, 0x73,
0x69, 0x6b, 0x6f, 0x20, 0x50, 0x61, 0x73, 0x69, 0x69, 0x66, 0x69, 0x6b,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x20, 0x50, 0x61,
0x73, 0x69, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x6a, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61,
0x74, 0x61, 0x72, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x55, 0x6c, 0x61, 0x61,
0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x55, 0x6c, 0x61,
0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x20, 0x53, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75, 0x20, 0x53, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x6a, 0x4d, 0x6a, 0x61, 0x6e, 0x6d, 0x61, 0x61,
0x72, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x4e, 0x65, 0x65, 0x70, 0x61, 0x61, 0x6c, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x4e, 0x65, 0x69, 0x6b, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6f, 0x6e, 0x69, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4e,
0x65, 0x69, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6f, 0x6e, 0x69, 0x65,
0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x4e, 0x65, 0x69, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6f, 0x6e, 0x69, 0x65, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x6a, 0x4e, 0x65, 0x69, 0x73, 0x69, 0x61, 0x6c, 0x75, 0x6e,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x4e, 0x65, 0x69, 0x73, 0x69, 0x61, 0x6c,
0x75, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x4e, 0x65, 0x69, 0x73, 0x69, 0x61, 0x6c, 0x75,
0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x4e,
0x65, 0x69, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x75, 0x6e, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x4e, 0x65, 0x69, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x75, 0x6e,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x4e, 0x65, 0x69, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x75, 0x6e,
0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x4e, 0x69,
0x75, 0x65, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x45, 0x69, 0x6c,
0x75, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20,
0x45, 0x69, 0x6c, 0x75, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x6a, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x53, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x6a, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4e, 0x6f, 0x77,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4e, 0x6f,
0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x53, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x4f, 0x6d, 0x73, 0x6b, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4f, 0x6d,
0x73, 0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x20, 0x53,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x50, 0x61, 0x6c, 0x61,
0x75, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20,
0x4e, 0x65, 0x69, 0x67, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x20, 0x53, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x50, 0x65, 0x72, 0x75, 0x75,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x50, 0x65, 0x72, 0x75, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x50, 0x65, 0x72, 0x75, 0x75, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x6a, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x69, 0x6e, 0x65,
0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69,
0x69, 0x6e, 0x65, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69,
0x69, 0x6e, 0x65, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x6a, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x20, 0x45, 0x69,
0x6c, 0x75, 0x6e, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53, 0x74,
0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x61, 0x6e, 0x20,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20,
0x61, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20,
0x61, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20,
0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x50, 0x69, 0x74,
0x63, 0x61, 0x69, 0x72, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x65,
0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x50, 0x6a, 0xf6, 0x6e, 0x67, 0x6a, 0x61,
0x6e, 0x67, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x52, 0xe9, 0x75, 0x6e, 0x69,
0x6f, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x52, 0x6f, 0x74, 0x68, 0x65,
0x72, 0x61, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53, 0x61, 0x63, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53, 0x61, 0x63, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x53,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x53, 0x65, 0x73, 0x63,
0x68, 0x65, 0x6c, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53, 0x69,
0x6e, 0x67, 0x61, 0x70, 0x75, 0x75, 0x72, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6f, 0x6e, 0x65, 0x6e, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x53, 0xfc, 0xfc, 0x64, 0x67, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53, 0x75, 0x75, 0x72,
0x69, 0x6e, 0x61, 0x6d, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53, 0x79, 0x6f,
0x77, 0x61, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x54, 0x61, 0x68, 0x69, 0x74,
0x69, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x54, 0x61, 0x69, 0x70, 0x65, 0x65,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x54, 0x61, 0x69, 0x70, 0x65, 0x65, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x54, 0x61, 0x69, 0x70, 0x65, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x6a, 0x54, 0x61, 0x64, 0x6a, 0x69, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x54, 0x6f,
0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x54, 0x6f, 0x6e, 0x67,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x53, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x54,
0x75, 0x72, 0x6b, 0x6d, 0x65, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x61,
0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x61, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x6a, 0x54, 0x75, 0x77, 0x61, 0x61, 0x6c, 0x75, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x20, 0x53, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x6a, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x55, 0x73, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x55, 0x73,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x20, 0x53, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20,
0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x57, 0x65, 0x65,
0x6e, 0x65, 0x73, 0x75, 0x65, 0x6c, 0x61, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x6a, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x57, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x57, 0x61, 0x6b,
0x65, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x61, 0x6e, 0x20, 0x46, 0x75,
0x74, 0x75, 0x6e, 0x61, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4a, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4a, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x4a,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4a, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x53, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x59, 0x75, 0x6b, 0x6f, 0x6e,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x20, 0x935,
0x93f, 0x91a, 0x20, 0x92e, 0x940, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x74,
0x65, 0x6e, 0x70, 0x6f, 0x20, 0x70, 0x69, 0x20, 0x6d, 0x61, 0x20, 0x4b,
0x65, 0x6e, 0x69, 0x47, 0x72, 0x69, 0x6e, 0x69, 0x73, 0x20, 0x4d, 0x69,
0x6e, 0x20, 0x54, 0x61, 0x65, 0x6d, 0x627, 0x64e, 0x644, 0x627, 0x633, 0x6a9,
0x627, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x627, 0x64e, 0x644, 0x627,
0x633, 0x6a9, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af,
0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x627, 0x64e, 0x644, 0x627, 0x633,
0x6a9, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20,
0x633, 0x627, 0x647, 0x62a, 0x627, 0x645, 0x627, 0x632, 0x6c6, 0x646, 0x626, 0x6d2,
0x20, 0x633, 0x627, 0x647, 0x62a, 0x627, 0x645, 0x627, 0x632, 0x6c6, 0x646, 0x626,
0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633,
0x627, 0x647, 0x62a, 0x627, 0x645, 0x627, 0x632, 0x6c6, 0x646, 0x626, 0x6d2, 0x20,
0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646,
0x6cc, 0x627, 0x645, 0x6cc, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x626,
0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x6cc, 0x627, 0x645, 0x6cc, 0x20,
0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634,
0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x6cc,
0x627, 0x645, 0x6cc, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x626, 0x6d2,
0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a,
0x631, 0x6c6, 0x62f, 0x631, 0x627, 0x62a, 0x6a9, 0x6cc, 0x20, 0x627, 0x645, 0x631,
0x6cc, 0x6a9, 0x627, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6,
0x62f, 0x631, 0x627, 0x62a, 0x6a9, 0x6cc, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9,
0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646,
0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x62f, 0x631, 0x627, 0x62a, 0x6a9,
0x6cc, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x626, 0x6d2, 0x20, 0x6af,
0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x6a9, 0x6c6,
0x647, 0x633, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9,
0x627, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x6a9, 0x6c6, 0x647, 0x633,
0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x626,
0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633,
0x627, 0x647, 0x62a, 0x6a9, 0x6c6, 0x647, 0x633, 0x62a, 0x6af, 0x6ce, 0x646, 0x20,
0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645,
0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x622, 0x631, 0x627, 0x645,
0x632, 0x650, 0x631, 0x6cc, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x626,
0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x622, 0x631, 0x627, 0x645, 0x632, 0x650,
0x631, 0x6cc, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x626, 0x6d2, 0x20,
0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647,
0x62a, 0x622, 0x631, 0x627, 0x645, 0x632, 0x650, 0x631, 0x6cc, 0x20, 0x627, 0x645,
0x631, 0x6cc, 0x6a9, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af,
0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x627, 0x631, 0x62c, 0x646, 0x62a, 0x6cc,
0x646, 0x627, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x627, 0x631, 0x62c,
0x646, 0x62a, 0x6cc, 0x646, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651,
0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x627, 0x631, 0x62c,
0x646, 0x62a, 0x6cc, 0x646, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627,
0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x646, 0x646, 0x62f,
0x6cc, 0x20, 0x627, 0x631, 0x62c, 0x646, 0x62a, 0x6cc, 0x646, 0x627, 0x626, 0x6d2,
0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x646, 0x646, 0x62f, 0x6cc, 0x20,
0x627, 0x631, 0x62c, 0x646, 0x62a, 0x6cc, 0x646, 0x627, 0x626, 0x6d2, 0x20, 0x6af,
0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a,
0x631, 0x6c6, 0x646, 0x646, 0x62f, 0x6cc, 0x20, 0x627, 0x631, 0x62c, 0x646, 0x62a,
0x6cc, 0x646, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc,
0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x6cc, 0x627, 0x645, 0x6cc, 0x20, 0x627,
0x64f, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x626, 0x6d2, 0x20, 0x633,
0x627, 0x647, 0x62a, 0x646, 0x6cc, 0x627, 0x645, 0x6cc, 0x20, 0x627, 0x64f, 0x633,
0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634,
0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x6cc,
0x627, 0x645, 0x6cc, 0x20, 0x627, 0x64f, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc,
0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633,
0x627, 0x647, 0x62a, 0x646, 0x6cc, 0x627, 0x645, 0x200c, 0x631, 0x6c6, 0x646, 0x646,
0x62f, 0x6cc, 0x20, 0x627, 0x64f, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627,
0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x6cc, 0x627, 0x645, 0x200c,
0x631, 0x6c6, 0x646, 0x646, 0x62f, 0x6cc, 0x20, 0x627, 0x64f, 0x633, 0x62a, 0x631,
0x627, 0x644, 0x6cc, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a,
0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x6cc, 0x627, 0x645,
0x200c, 0x631, 0x6c6, 0x646, 0x646, 0x62f, 0x6cc, 0x20, 0x627, 0x64f, 0x633, 0x62a,
0x631, 0x627, 0x644, 0x6cc, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627,
0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x62f, 0x631, 0x627,
0x62a, 0x6a9, 0x6cc, 0x20, 0x627, 0x64f, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc,
0x627, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x62f, 0x631,
0x627, 0x62a, 0x6a9, 0x6cc, 0x20, 0x627, 0x64f, 0x633, 0x62a, 0x631, 0x627, 0x644,
0x6cc, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce,
0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x62f, 0x631, 0x627, 0x62a,
0x6a9, 0x6cc, 0x20, 0x627, 0x64f, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627,
0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627,
0x647, 0x62a, 0x628, 0x631, 0x627, 0x632, 0x6cc, 0x644, 0x626, 0x6d2, 0x20, 0x633,
0x627, 0x647, 0x62a, 0x628, 0x631, 0x627, 0x632, 0x6cc, 0x644, 0x626, 0x6d2, 0x20,
0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647,
0x62a, 0x628, 0x631, 0x627, 0x632, 0x6cc, 0x644, 0x626, 0x6d2, 0x20, 0x6af, 0x631,
0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x6cc, 0x627,
0x645, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x626, 0x6d2, 0x20, 0x633, 0x627,
0x647, 0x62a, 0x646, 0x6cc, 0x627, 0x645, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e,
0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20,
0x633, 0x627, 0x647, 0x62a, 0x646, 0x6cc, 0x627, 0x645, 0x6cc, 0x20, 0x6cc, 0x648,
0x631, 0x67e, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20,
0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x62f, 0x631, 0x627, 0x62a, 0x6a9, 0x6cc,
0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a,
0x631, 0x6c6, 0x62f, 0x631, 0x627, 0x62a, 0x6a9, 0x6cc, 0x20, 0x6cc, 0x648, 0x631,
0x67e, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646,
0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x62f, 0x631, 0x627, 0x62a, 0x6a9,
0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645,
0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x62f, 0x6ce, 0x645, 0x62a,
0x631, 0x6cc, 0x20, 0x631, 0x6c6, 0x62f, 0x631, 0x627, 0x62a, 0x6a9, 0x6cc, 0x20,
0x6cc, 0x648, 0x631, 0x67e, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a,
0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x646, 0x646,
0x62f, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x626, 0x6d2, 0x20, 0x633, 0x627,
0x647, 0x62a, 0x631, 0x6c6, 0x646, 0x646, 0x62f, 0x6cc, 0x20, 0x6cc, 0x648, 0x631,
0x67e, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646,
0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x646, 0x646, 0x62f, 0x6cc, 0x20,
0x6cc, 0x648, 0x631, 0x67e, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af,
0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x6af, 0x631, 0x6cc, 0x646, 0x200c, 0x648,
0x650, 0x686, 0x20, 0x645, 0x6cc, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x647,
0x626, 0x648, 0x627, 0x6cc, 0x6cc, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a,
0x647, 0x626, 0x648, 0x627, 0x6cc, 0x6cc, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634,
0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x647, 0x626,
0x648, 0x627, 0x6cc, 0x6cc, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af,
0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x6cc, 0x627, 0x645, 0x6cc, 0x20,
0x627, 0x646, 0x62f, 0x6c6, 0x646, 0x6cc, 0x632, 0x6cc, 0x627, 0x626, 0x6d2, 0x20,
0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647,
0x62a, 0x631, 0x6c6, 0x62f, 0x631, 0x627, 0x62a, 0x6a9, 0x6cc, 0x20, 0x627, 0x646,
0x62f, 0x6c6, 0x646, 0x6cc, 0x632, 0x6cc, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc,
0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x631,
0x6c6, 0x646, 0x646, 0x62f, 0x6cc, 0x20, 0x627, 0x646, 0x62f, 0x6c6, 0x646, 0x6cc,
0x632, 0x6cc, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af,
0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x627, 0x6cc, 0x631, 0x6a9, 0x648,
0x62a, 0x633, 0x6a9, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x627, 0x6cc,
0x631, 0x6a9, 0x648, 0x62a, 0x633, 0x6a9, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634,
0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x627, 0x6cc,
0x631, 0x6a9, 0x648, 0x62a, 0x633, 0x6a9, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645,
0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x62f, 0x631,
0x627, 0x62a, 0x6a9, 0x6cc, 0x20, 0x6a9, 0x627, 0x632, 0x6a9, 0x633, 0x62a, 0x627,
0x646, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646,
0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x646, 0x646, 0x62f, 0x6cc, 0x20,
0x6a9, 0x627, 0x632, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x626, 0x6d2, 0x20, 0x6af,
0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a,
0x6a9, 0x631, 0x627, 0x646, 0x633, 0x646, 0x64f, 0x6cc, 0x627, 0x631, 0x633, 0x6a9,
0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x6a9, 0x631, 0x627, 0x646, 0x633,
0x646, 0x64f, 0x6cc, 0x627, 0x631, 0x633, 0x6a9, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc,
0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x6a9,
0x631, 0x627, 0x646, 0x633, 0x646, 0x64f, 0x6cc, 0x627, 0x631, 0x633, 0x6a9, 0x626,
0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647,
0x62a, 0x644, 0x6c6, 0x631, 0x688, 0x20, 0x647, 0x626, 0x648, 0x20, 0x627, 0x64f,
0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x626, 0x6d2, 0x20, 0x633, 0x627,
0x647, 0x62a, 0x644, 0x6c6, 0x631, 0x688, 0x20, 0x647, 0x626, 0x648, 0x20, 0x627,
0x64f, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x626, 0x6d2, 0x20, 0x6af,
0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a,
0x644, 0x6c6, 0x631, 0x688, 0x20, 0x647, 0x626, 0x648, 0x20, 0x627, 0x64f, 0x633,
0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645,
0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x645, 0x64e, 0x6af, 0x64e,
0x62f, 0x64e, 0x646, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x645, 0x64e,
0x6af, 0x64e, 0x62f, 0x64e, 0x646, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651,
0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x645, 0x64e, 0x6af,
0x64e, 0x62f, 0x64e, 0x646, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af,
0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x622, 0x631, 0x627, 0x645, 0x632, 0x650,
0x631, 0x6cc, 0x20, 0x645, 0x650, 0x6a9, 0x633, 0x6cc, 0x6a9, 0x6c6, 0x626, 0x6d2,
0x20, 0x633, 0x627, 0x647, 0x62a, 0x622, 0x631, 0x627, 0x645, 0x632, 0x650, 0x631,
0x6cc, 0x20, 0x645, 0x650, 0x6a9, 0x633, 0x6cc, 0x6a9, 0x6c6, 0x626, 0x6d2, 0x20,
0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647,
0x62a, 0x622, 0x631, 0x627, 0x645, 0x632, 0x650, 0x631, 0x6cc, 0x20, 0x645, 0x650,
0x6a9, 0x633, 0x6cc, 0x6a9, 0x6c6, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627,
0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x645, 0x627, 0x633, 0x6a9, 0x6c6,
0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x645, 0x627, 0x633, 0x6a9, 0x6c6,
0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20,
0x633, 0x627, 0x647, 0x62a, 0x645, 0x627, 0x633, 0x6a9, 0x6c6, 0x626, 0x6d2, 0x20,
0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646,
0x6cc, 0x648, 0x641, 0x627, 0x648, 0x648, 0x646, 0x688, 0x644, 0x626, 0x6cc, 0x646,
0x688, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x6cc, 0x648, 0x641,
0x627, 0x648, 0x648, 0x646, 0x688, 0x644, 0x626, 0x6cc, 0x646, 0x688, 0x626, 0x6d2,
0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627,
0x647, 0x62a, 0x646, 0x6cc, 0x648, 0x641, 0x627, 0x648, 0x648, 0x646, 0x688, 0x644,
0x626, 0x6cc, 0x646, 0x688, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af,
0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x64f, 0x631, 0x64f, 0x646, 0x647,
0x627, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x64f, 0x631, 0x64f,
0x646, 0x647, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af,
0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x64f, 0x631, 0x64f, 0x646,
0x647, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20,
0x633, 0x627, 0x647, 0x62a, 0x646, 0x6c6, 0x648, 0x6c6, 0x633, 0x6cc, 0x628, 0x6cc,
0x631, 0x633, 0x6a9, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x6c6,
0x648, 0x6c6, 0x633, 0x6cc, 0x628, 0x6cc, 0x631, 0x633, 0x6a9, 0x626, 0x6d2, 0x20,
0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647,
0x62a, 0x646, 0x6c6, 0x648, 0x6c6, 0x633, 0x6cc, 0x628, 0x6cc, 0x631, 0x633, 0x6a9,
0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627,
0x647, 0x62a, 0x627, 0x6c6, 0x645, 0x633, 0x6a9, 0x626, 0x6d2, 0x20, 0x633, 0x627,
0x647, 0x62a, 0x627, 0x6c6, 0x645, 0x633, 0x6a9, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc,
0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x627,
0x6c6, 0x645, 0x633, 0x6a9, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af,
0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x648, 0x644, 0x627, 0x62f, 0x6cc, 0x648,
0x64f, 0x633, 0x62a, 0x64f, 0x6a9, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a,
0x648, 0x644, 0x627, 0x62f, 0x6cc, 0x648, 0x64f, 0x633, 0x62a, 0x64f, 0x6a9, 0x626,
0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633,
0x627, 0x647, 0x62a, 0x648, 0x644, 0x627, 0x62f, 0x6cc, 0x648, 0x64f, 0x633, 0x62a,
0x64f, 0x6a9, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20,
0x633, 0x627, 0x647, 0x62a, 0x6cc, 0x627, 0x6a9, 0x648, 0x62a, 0x633, 0x6a9, 0x626,
0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x6cc, 0x627, 0x6a9, 0x648, 0x62a, 0x633,
0x6a9, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646,
0x20, 0x633, 0x627, 0x647, 0x62a, 0x6cc, 0x627, 0x6a9, 0x648, 0x62a, 0x633, 0x6a9,
0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627,
0x647, 0x62a, 0x6cc, 0x6ce, 0x6a9, 0x627, 0x62a, 0x631, 0x6cc, 0x646, 0x628, 0x648,
0x631, 0x6af, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x6cc, 0x6ce, 0x6a9,
0x627, 0x62a, 0x631, 0x6cc, 0x646, 0x628, 0x648, 0x631, 0x6af, 0x626, 0x6d2, 0x20,
0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647,
0x62a, 0x6cc, 0x6ce, 0x6a9, 0x627, 0x62a, 0x631, 0x6cc, 0x646, 0x628, 0x648, 0x631,
0x6af, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633,
0x627, 0x647, 0x62a, 0x41, 0x77, 0x67, 0xe1, 0x6e, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x44, 0x65, 0x6c, 0x67,
0xe1, 0x68, 0x69, 0x20, 0x41, 0x70, 0x72, 0x69, 0x6b, 0xe1, 0x61, 0x79,
0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x64, 0x61, 0x72, 0xe1, 0x74,
0x6b, 0x69, 0x20, 0x41, 0x70, 0x72, 0x69, 0x6b, 0xe1, 0x61, 0x79, 0x20,
0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x64, 0x61, 0x72, 0xe1, 0x74, 0x6b,
0x69, 0x20, 0x41, 0x70, 0x72, 0x69, 0x6b, 0xe1, 0x61, 0x79, 0x20, 0x61,
0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3,
0x6e, 0x65, 0x6e, 0x64, 0x69, 0x20, 0x41, 0x70, 0x72, 0x69, 0x6b, 0xe1,
0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x6e, 0x65, 0x6e,
0x64, 0x69, 0x20, 0x41, 0x70, 0x72, 0x69, 0x6b, 0xe1, 0x61, 0x79, 0x20,
0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52,
0xf3, 0x6e, 0x65, 0x6e, 0x64, 0x69, 0x20, 0x41, 0x70, 0x72, 0x69, 0x6b,
0xe1, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20,
0x77, 0x61, 0x68, 0x64, 0x41, 0x6c, 0xe1, 0x73, 0x6b, 0xe1, 0x61, 0x79,
0x20, 0x77, 0x61, 0x68, 0x64, 0x41, 0x6c, 0xe1, 0x73, 0x6b, 0xe1, 0x61,
0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0x41, 0x6c, 0xe1, 0x73, 0x6b, 0xe1, 0x61, 0x79, 0x20, 0x67, 0x61,
0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x41, 0x6d,
0xe1, 0x7a, 0xf3, 0x6e, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x41,
0x6d, 0xe1, 0x7a, 0xf3, 0x6e, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1,
0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x41, 0x6d, 0xe1, 0x7a, 0xf3,
0x6e, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20,
0x77, 0x61, 0x68, 0x64, 0x44, 0x65, 0x6c, 0x67, 0xe1, 0x68, 0x69, 0x20,
0x41, 0x6d, 0x72, 0x69, 0x6b, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68,
0x64, 0x44, 0x65, 0x6c, 0x67, 0xe1, 0x68, 0x69, 0x20, 0x41, 0x6d, 0x72,
0x69, 0x6b, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69,
0x20, 0x77, 0x61, 0x68, 0x64, 0x44, 0x65, 0x6c, 0x67, 0xe1, 0x68, 0x69,
0x20, 0x41, 0x6d, 0x72, 0x69, 0x6b, 0xe1, 0x61, 0x79, 0x20, 0x67, 0x61,
0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3,
0x64, 0x61, 0x72, 0xe1, 0x74, 0x6b, 0x69, 0x20, 0x41, 0x6d, 0x72, 0x69,
0x6b, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x64,
0x61, 0x72, 0xe1, 0x74, 0x6b, 0x69, 0x20, 0x41, 0x6d, 0x72, 0x69, 0x6b,
0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77,
0x61, 0x68, 0x64, 0x52, 0xf3, 0x64, 0x61, 0x72, 0xe1, 0x74, 0x6b, 0x69,
0x20, 0x41, 0x6d, 0x72, 0x69, 0x6b, 0xe1, 0x61, 0x79, 0x20, 0x67, 0x61,
0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4b, 0xf3,
0x68, 0x65, 0x73, 0x74, 0x61, 0x67, 0x69, 0x20, 0x41, 0x6d, 0x72, 0x69,
0x6b, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4b, 0xf3, 0x68,
0x65, 0x73, 0x74, 0x61, 0x67, 0x69, 0x20, 0x41, 0x6d, 0x72, 0x69, 0x6b,
0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77,
0x61, 0x68, 0x64, 0x4b, 0xf3, 0x68, 0x65, 0x73, 0x74, 0x61, 0x67, 0x69,
0x20, 0x41, 0x6d, 0x72, 0x69, 0x6b, 0xe1, 0x61, 0x79, 0x20, 0x67, 0x61,
0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0xc1, 0x72,
0xe1, 0x6d, 0x7a, 0x65, 0x72, 0x69, 0x20, 0x41, 0x6d, 0x72, 0x69, 0x6b,
0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0xc1, 0x72, 0xe1, 0x6d,
0x7a, 0x65, 0x72, 0x69, 0x20, 0x41, 0x6d, 0x72, 0x69, 0x6b, 0xe1, 0x61,
0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0xc1, 0x72, 0xe1, 0x6d, 0x7a, 0x65, 0x72, 0x69, 0x20, 0x41, 0x6d,
0x72, 0x69, 0x6b, 0xe1, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1,
0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x41, 0x70, 0x69, 0xe1, 0x61,
0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x41, 0x70, 0x69, 0xe1, 0x61, 0x79,
0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0x41, 0x70, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x72, 0xf3, 0x63, 0x68, 0x69,
0x20, 0x77, 0x61, 0x68, 0x64, 0x41, 0x72, 0x61, 0x62, 0x69, 0x20, 0x77,
0x61, 0x68, 0x64, 0x41, 0x72, 0x61, 0x62, 0x69, 0x20, 0x61, 0x6e, 0x6a,
0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x41, 0x72, 0x61, 0x62,
0x69, 0x20, 0x72, 0xf3, 0x63, 0x68, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0xe1, 0x61, 0x79, 0x20,
0x77, 0x61, 0x68, 0x64, 0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77,
0x61, 0x68, 0x64, 0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0xe1,
0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77,
0x61, 0x68, 0x64, 0x52, 0xf3, 0x6e, 0x65, 0x6e, 0x64, 0x69, 0x20, 0x41,
0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0xe1, 0x61, 0x79, 0x20, 0x77,
0x61, 0x68, 0x64, 0x52, 0xf3, 0x6e, 0x65, 0x6e, 0x64, 0x69, 0x20, 0x41,
0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0xe1, 0x61, 0x79, 0x20, 0x61,
0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3,
0x6e, 0x65, 0x6e, 0x64, 0x69, 0x20, 0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0xe1, 0x61, 0x79, 0x20, 0x67, 0x72, 0x61, 0x6d, 0xe1, 0x67,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0xc1, 0x72, 0x6d, 0x69, 0x6e, 0x69,
0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0xc1, 0x72, 0x6d, 0x69,
0x6e, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69,
0x20, 0x77, 0x61, 0x68, 0x64, 0xc1, 0x72, 0x6d, 0x69, 0x6e, 0x69, 0xe1,
0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77,
0x61, 0x68, 0x64, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0xe1, 0x61, 0x79,
0x20, 0x77, 0x61, 0x68, 0x64, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0xe1,
0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61,
0x68, 0x64, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0xe1, 0x61, 0x79, 0x20,
0x72, 0xf3, 0x63, 0x68, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x44, 0x65,
0x6c, 0x67, 0xe1, 0x68, 0x69, 0x20, 0xc1, 0x73, 0x74, 0x72, 0xe9, 0x6c,
0x69, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x44, 0x65, 0x6c,
0x67, 0xe1, 0x68, 0x69, 0x20, 0xc1, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69,
0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77,
0x61, 0x68, 0x64, 0x44, 0x65, 0x6c, 0x67, 0xe1, 0x68, 0x69, 0x20, 0xc1,
0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x67, 0x61,
0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x44, 0x65,
0x6c, 0x67, 0xe1, 0x68, 0x69, 0x72, 0xf3, 0x6e, 0x65, 0x6e, 0x64, 0x69,
0x20, 0xc1, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0xe1, 0x61, 0x79, 0x20,
0x77, 0x61, 0x68, 0x64, 0x44, 0x65, 0x6c, 0x67, 0xe1, 0x68, 0x69, 0x72,
0xf3, 0x6e, 0x65, 0x6e, 0x64, 0x69, 0x20, 0xc1, 0x73, 0x74, 0x72, 0xe9,
0x6c, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69,
0x20, 0x77, 0x61, 0x68, 0x64, 0x44, 0x65, 0x6c, 0x67, 0xe1, 0x68, 0x69,
0x72, 0xf3, 0x6e, 0x65, 0x6e, 0x64, 0x69, 0x20, 0xc1, 0x73, 0x74, 0x72,
0xe9, 0x6c, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1,
0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x64, 0x61, 0x72,
0xe1, 0x74, 0x6b, 0x69, 0x20, 0xc1, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69,
0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x64, 0x61,
0x72, 0xe1, 0x74, 0x6b, 0x69, 0x20, 0xc1, 0x73, 0x74, 0x72, 0xe9, 0x6c,
0x69, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20,
0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x64, 0x61, 0x72, 0xe1, 0x74, 0x6b,
0x69, 0x20, 0xc1, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0xe1, 0x61, 0x79,
0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0x52, 0xf3, 0x6e, 0x65, 0x6e, 0x64, 0x69, 0x20, 0xc1, 0x73, 0x74,
0x72, 0xe9, 0x6c, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64,
0x52, 0xf3, 0x6e, 0x65, 0x6e, 0x64, 0x69, 0x20, 0xc1, 0x73, 0x74, 0x72,
0xe9, 0x6c, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x6e, 0x65, 0x6e, 0x64,
0x69, 0x20, 0xc1, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0xe1, 0x61, 0x79,
0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0xc1, 0x7a, 0x65, 0x72, 0x62, 0xe1, 0x69, 0x6a, 0xe1, 0x6e, 0x61,
0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0xc1, 0x7a, 0x65, 0x72, 0x62, 0xe1,
0x69, 0x6a, 0xe1, 0x6e, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0xc1, 0x7a, 0x65, 0x72, 0x62, 0xe1,
0x69, 0x6a, 0xe1, 0x6e, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1,
0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x41, 0x7a, 0xf3, 0x72, 0x65,
0x73, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x41, 0x7a, 0xf3, 0x72,
0x65, 0x73, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20,
0x77, 0x61, 0x68, 0x64, 0x41, 0x7a, 0xf3, 0x72, 0x65, 0x73, 0x61, 0x79,
0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0x42, 0x61, 0x6e, 0x67, 0x61, 0x6c, 0x61, 0x64, 0xe9, 0x73, 0x68,
0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x42, 0x61, 0x6e, 0x67, 0x61,
0x6c, 0x61, 0x64, 0xe9, 0x73, 0x68, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a,
0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x42, 0x61, 0x6e, 0x67,
0x61, 0x6c, 0x61, 0x64, 0xe9, 0x73, 0x68, 0x61, 0x79, 0x20, 0x67, 0x61,
0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x42, 0x75,
0x74, 0x68, 0xe1, 0x6e, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x42,
0x6f, 0x6c, 0x69, 0x77, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68,
0x64, 0x42, 0x72, 0xe1, 0x7a, 0x69, 0x6c, 0x61, 0x79, 0x20, 0x77, 0x61,
0x68, 0x64, 0x42, 0x72, 0xe1, 0x7a, 0x69, 0x6c, 0x61, 0x79, 0x20, 0x61,
0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x42, 0x72,
0xe1, 0x7a, 0x69, 0x6c, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1,
0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x42, 0x72, 0x75, 0x6e, 0xe1,
0x69, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4b, 0x61, 0x70, 0x20,
0x57, 0x61, 0x72, 0x64, 0xe9, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64,
0x4b, 0x61, 0x70, 0x20, 0x57, 0x61, 0x72, 0x64, 0xe9, 0x61, 0x79, 0x20,
0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4b,
0x61, 0x70, 0x20, 0x57, 0x61, 0x72, 0x64, 0xe9, 0x61, 0x79, 0x20, 0x67,
0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x43,
0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0xf3, 0x61, 0x79, 0x20, 0x77, 0x61,
0x68, 0x64, 0x43, 0x68, 0x61, 0x74, 0x61, 0x6d, 0x20, 0x77, 0x61, 0x68,
0x64, 0x43, 0x68, 0x61, 0x74, 0x61, 0x6d, 0x20, 0x61, 0x6e, 0x6a, 0xe1,
0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x43, 0x68, 0x61, 0x74, 0x61,
0x6d, 0x20, 0x72, 0xf3, 0x63, 0x68, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0x43, 0x68, 0x69, 0x6c, 0x61, 0x79, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68,
0x64, 0x43, 0x68, 0x69, 0x6c, 0x61, 0x79, 0x61, 0x79, 0x20, 0x61, 0x6e,
0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x43, 0x68, 0x69,
0x6c, 0x61, 0x79, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x79,
0x20, 0x77, 0x61, 0x68, 0x64, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x79, 0x20,
0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x43,
0x68, 0x69, 0x6e, 0x61, 0x79, 0x20, 0x72, 0xf3, 0x63, 0x68, 0x69, 0x20,
0x77, 0x61, 0x68, 0x64, 0x4b, 0x72, 0x65, 0x73, 0x6d, 0x65, 0x73, 0x20,
0x49, 0x73, 0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68,
0x64, 0x4b, 0x75, 0x6b, 0x75, 0x73, 0x20, 0x49, 0x73, 0x6c, 0xe1, 0x6e,
0x64, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4b, 0x6f, 0x6c, 0x61,
0x6d, 0x62, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4b,
0x6f, 0x6c, 0x61, 0x6d, 0x62, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e,
0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4b, 0x6f, 0x6c,
0x61, 0x6d, 0x62, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d,
0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4b, 0x75, 0x6b, 0x20,
0x49, 0x73, 0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68,
0x64, 0x4b, 0x75, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0xe1, 0x6e, 0x64, 0x61,
0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0x4b, 0x75, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0xe1, 0x6e, 0x64, 0x61,
0x79, 0x20, 0x6e, 0xe9, 0x6d, 0x2d, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4b, 0x79, 0x75, 0x62, 0xe1, 0x61,
0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4b, 0x79, 0x75, 0x62, 0xe1, 0x61,
0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0x4b, 0x79, 0x75, 0x62, 0xe1, 0x61, 0x79, 0x20, 0x72, 0xf3, 0x63,
0x68, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x44, 0x61, 0x77, 0x69,
0x73, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x44, 0x75, 0x6d, 0x6f,
0x6e, 0x74, 0x20, 0x55, 0x72, 0x77, 0x69, 0x6c, 0x61, 0x79, 0x20, 0x77,
0x61, 0x68, 0x64, 0x52, 0xf3, 0x64, 0x61, 0x72, 0xe1, 0x74, 0x6b, 0x69,
0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68,
0x64, 0x49, 0x73, 0x74, 0x68, 0x61, 0x72, 0x20, 0x49, 0x73, 0x6c, 0xe1,
0x6e, 0x64, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x49, 0x73, 0x74,
0x68, 0x61, 0x72, 0x20, 0x49, 0x73, 0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x79,
0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0x49, 0x73, 0x74, 0x68, 0x61, 0x72, 0x20, 0x49, 0x73, 0x6c, 0xe1, 0x6e,
0x64, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20,
0x77, 0x61, 0x68, 0x64, 0x45, 0x6b, 0x77, 0xe1, 0x64, 0xf3, 0x72, 0x61,
0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x44, 0x65, 0x6c, 0x67, 0xe1, 0x68,
0x69, 0x20, 0x59, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x79, 0x20, 0x77, 0x61,
0x68, 0x64, 0x44, 0x65, 0x6c, 0x67, 0xe1, 0x68, 0x69, 0x20, 0x59, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69,
0x20, 0x77, 0x61, 0x68, 0x64, 0x44, 0x65, 0x6c, 0x67, 0xe1, 0x68, 0x69,
0x20, 0x59, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72,
0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x64,
0x61, 0x72, 0xe1, 0x74, 0x6b, 0x69, 0x20, 0x59, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x64, 0x61, 0x72,
0xe1, 0x74, 0x6b, 0x69, 0x20, 0x59, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x79,
0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0x52, 0xf3, 0x64, 0x61, 0x72, 0xe1, 0x74, 0x6b, 0x69, 0x20, 0x59, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x44, 0xe9, 0x6d, 0x74, 0x65, 0x72,
0xe9, 0x6e, 0x20, 0x52, 0xf3, 0x64, 0x61, 0x72, 0xe1, 0x74, 0x6b, 0x69,
0x20, 0x59, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a,
0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x6e, 0x65,
0x6e, 0x64, 0x69, 0x20, 0x59, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x79, 0x20,
0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x6e, 0x65, 0x6e, 0x64, 0x69, 0x20,
0x59, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1,
0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x6e, 0x65, 0x6e,
0x64, 0x69, 0x20, 0x59, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x79, 0x20, 0x67,
0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x50,
0x61, 0x6c, 0x6b, 0x6c, 0xe1, 0x6e, 0x64, 0x20, 0x49, 0x73, 0x6c, 0xe1,
0x6e, 0x64, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x50, 0x61, 0x6c,
0x6b, 0x6c, 0xe1, 0x6e, 0x64, 0x20, 0x49, 0x73, 0x6c, 0xe1, 0x6e, 0x64,
0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61,
0x68, 0x64, 0x50, 0x61, 0x6c, 0x6b, 0x6c, 0xe1, 0x6e, 0x64, 0x20, 0x49,
0x73, 0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d,
0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x46, 0x69, 0x6a, 0x69,
0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x46, 0x69, 0x6a, 0x69, 0x61,
0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0x46, 0x69, 0x6a, 0x69, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d,
0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x50, 0x61, 0x72, 0xe1,
0x6e, 0x73, 0x69, 0x20, 0x47, 0x77, 0x69, 0x6e, 0xe1, 0x61, 0x79, 0x20,
0x77, 0x61, 0x68, 0x64, 0x5a, 0x65, 0x72, 0x62, 0xe1, 0x72, 0x69, 0x20,
0x50, 0x61, 0x72, 0x61, 0x6e, 0x73, 0x69, 0x20, 0x6f, 0x20, 0x41, 0x6e,
0x74, 0xe1, 0x72, 0x6b, 0x74, 0x69, 0x6b, 0xe1, 0x61, 0x79, 0x20, 0x77,
0x61, 0x68, 0x64, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73,
0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x47, 0x61, 0x6d, 0x62, 0x69,
0x72, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4a, 0xe1, 0x72, 0x6a,
0x69, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4a, 0xe1, 0x72,
0x6a, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69,
0x20, 0x77, 0x61, 0x68, 0x64, 0x4a, 0xe1, 0x72, 0x6a, 0x69, 0xe1, 0x61,
0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61,
0x68, 0x64, 0x47, 0x65, 0x6c, 0x62, 0x61, 0x72, 0x74, 0x20, 0x49, 0x73,
0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x47,
0x72, 0x69, 0x6e, 0x77, 0x65, 0x63, 0x68, 0x20, 0x4d, 0x69, 0x6e, 0x20,
0x57, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x64, 0x61, 0x72, 0xe1, 0x74, 0x6b,
0x69, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x79,
0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x64, 0x61, 0x72, 0xe1, 0x74,
0x6b, 0x69, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0xe1, 0x6e, 0x64, 0x61,
0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0x52, 0xf3, 0x64, 0x61, 0x72, 0xe1, 0x74, 0x6b, 0x69, 0x20, 0x47,
0x72, 0x69, 0x6e, 0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x67, 0x61,
0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3,
0x6e, 0x65, 0x6e, 0x64, 0x69, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0xe1,
0x6e, 0x64, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x6e,
0x65, 0x6e, 0x64, 0x69, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0xe1, 0x6e,
0x64, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77,
0x61, 0x68, 0x64, 0x52, 0xf3, 0x6e, 0x65, 0x6e, 0x64, 0x69, 0x20, 0x47,
0x72, 0x69, 0x6e, 0x6c, 0xe1, 0x6e, 0x64, 0x20, 0x47, 0x61, 0x72, 0x6d,
0xe1, 0x67, 0x69, 0x20, 0x57, 0x61, 0x68, 0x64, 0x4b, 0x68, 0x61, 0x6c,
0x69, 0x6a, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20,
0x77, 0x61, 0x68, 0x64, 0x47, 0x75, 0x79, 0xe1, 0x6e, 0xe1, 0x61, 0x79,
0x20, 0x77, 0x61, 0x68, 0x64, 0x48, 0x61, 0x77, 0xe1, 0x69, 0x2f, 0x41,
0x6c, 0x75, 0x73, 0x68, 0x69, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64,
0x48, 0x61, 0x77, 0xe1, 0x69, 0x2f, 0x41, 0x6c, 0x75, 0x73, 0x68, 0x69,
0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61,
0x68, 0x64, 0x48, 0x61, 0x77, 0xe1, 0x69, 0x2f, 0x41, 0x6c, 0x75, 0x73,
0x68, 0x69, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69,
0x20, 0x77, 0x61, 0x68, 0x64, 0x48, 0xe1, 0x6e, 0x67, 0x20, 0x4b, 0xe1,
0x6e, 0x67, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x48, 0xe1, 0x6e,
0x67, 0x20, 0x4b, 0xe1, 0x6e, 0x67, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a,
0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x48, 0xe1, 0x6e, 0x67,
0x20, 0x4b, 0xe1, 0x6e, 0x67, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d,
0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x48, 0xf3, 0x77, 0x64,
0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x48, 0xf3, 0x77, 0x64, 0x61,
0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0x48, 0xf3, 0x77, 0x64, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d,
0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x48, 0x65, 0x6e, 0x64,
0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61,
0x68, 0x64, 0x48, 0x65, 0x6e, 0x64, 0x69, 0x20, 0x7a, 0x65, 0x72, 0x61,
0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x48, 0x65, 0x6e, 0x64, 0xf3, 0x63,
0x68, 0x69, 0x6e, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x44, 0x65,
0x6c, 0x67, 0xe1, 0x68, 0x69, 0x20, 0x45, 0x6e, 0x64, 0x68, 0x6f, 0x6e,
0x69, 0x73, 0x68, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1,
0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x64, 0x61, 0x72,
0xe1, 0x74, 0x6b, 0x69, 0x20, 0x45, 0x6e, 0x64, 0x68, 0x6f, 0x6e, 0x69,
0x73, 0x68, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x6e, 0x65, 0x6e, 0x64,
0x69, 0x20, 0x45, 0x6e, 0x64, 0x68, 0x6f, 0x6e, 0x69, 0x73, 0x68, 0x69,
0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77,
0x61, 0x68, 0x64, 0xc9, 0x72, 0xe1, 0x6e, 0x61, 0x79, 0x20, 0x77, 0x61,
0x68, 0x64, 0xc9, 0x72, 0xe1, 0x6e, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a,
0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0xc9, 0x72, 0xe1, 0x6e,
0x61, 0x79, 0x20, 0x72, 0xf3, 0x63, 0x68, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0x45, 0x72, 0x6b, 0x75, 0x73, 0x6b, 0x61, 0x79, 0x20, 0x77, 0x61,
0x68, 0x64, 0x45, 0x72, 0x6b, 0x75, 0x73, 0x6b, 0x61, 0x79, 0x20, 0x61,
0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x45, 0x72,
0x6b, 0x75, 0x73, 0x6b, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1,
0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x45, 0x73, 0x72, 0xe1, 0x69,
0x6c, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x45, 0x73, 0x72, 0xe1,
0x69, 0x6c, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20,
0x77, 0x61, 0x68, 0x64, 0x45, 0x73, 0x72, 0xe1, 0x69, 0x6c, 0x61, 0x79,
0x20, 0x72, 0xf3, 0x63, 0x68, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4a,
0xe1, 0x70, 0xe1, 0x6e, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4a,
0xe1, 0x70, 0xe1, 0x6e, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4a, 0xe1, 0x70, 0xe1, 0x6e, 0x61,
0x79, 0x20, 0x72, 0xf3, 0x63, 0x68, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0x4b, 0xe1, 0x7a, 0x61, 0x6b, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x61, 0x79,
0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x64, 0x61, 0x72, 0xe1, 0x74,
0x6b, 0x69, 0x20, 0x4b, 0xe1, 0x7a, 0x65, 0x6b, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77,
0x61, 0x68, 0x64, 0x52, 0xf3, 0x6e, 0x65, 0x6e, 0x64, 0x69, 0x20, 0x4b,
0xe1, 0x7a, 0x65, 0x6b, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x61, 0x79, 0x20,
0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4b,
0xf3, 0x72, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4b,
0xf3, 0x72, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4b, 0xf3, 0x72, 0x69, 0xe1, 0x61,
0x79, 0x20, 0x72, 0xf3, 0x63, 0x68, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0x4b, 0xf3, 0x73, 0x72, 0x61, 0x69, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68,
0x64, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0xf3, 0x79, 0xe1, 0x73, 0x6b, 0x61,
0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0xf3,
0x79, 0xe1, 0x73, 0x6b, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0xf3,
0x79, 0xe1, 0x73, 0x6b, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1,
0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4b, 0x61, 0x72, 0x67, 0x65,
0x7a, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68,
0x64, 0x4c, 0x69, 0x6e, 0xe9, 0x20, 0x49, 0x73, 0x6c, 0xe1, 0x6e, 0x64,
0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0xc1, 0x73, 0x74, 0x72, 0xe9,
0x6c, 0x69, 0xe1, 0x61, 0x79, 0x2c, 0x20, 0x4c, 0xe1, 0x64, 0x68, 0x61,
0x77, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0xc1, 0x73, 0x74, 0x72,
0xe9, 0x6c, 0x69, 0xe1, 0x61, 0x79, 0x2c, 0x20, 0x4c, 0xe1, 0x64, 0x68,
0x61, 0x77, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20,
0x77, 0x61, 0x68, 0x64, 0xc1, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0xe1,
0x61, 0x79, 0x2c, 0x20, 0x4c, 0xe1, 0x64, 0x68, 0x61, 0x77, 0x61, 0x79,
0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0x4d, 0xe1, 0x67, 0x61, 0x64, 0xe1, 0x6e, 0x61, 0x79, 0x20, 0x77,
0x61, 0x68, 0x64, 0x4d, 0xe1, 0x67, 0x61, 0x64, 0xe1, 0x6e, 0x61, 0x79,
0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0x4d, 0xe1, 0x67, 0x61, 0x64, 0xe1, 0x6e, 0x61, 0x79, 0x20, 0x67, 0x61,
0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4d, 0x61,
0x6c, 0x69, 0x73, 0x68, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68,
0x64, 0x4d, 0xe1, 0x6c, 0x64, 0x69, 0x70, 0x61, 0x79, 0x20, 0x77, 0x61,
0x68, 0x64, 0x4d, 0x61, 0x72, 0x6b, 0xe9, 0x73, 0xe1, 0x73, 0x61, 0x79,
0x20, 0x77, 0x61, 0x68, 0x64, 0x4d, 0xe1, 0x72, 0x73, 0x68, 0x61, 0x6c,
0x20, 0x49, 0x73, 0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x77, 0x61,
0x68, 0x64, 0x4d, 0x75, 0x72, 0x69, 0x74, 0xe1, 0x6e, 0x69, 0xe1, 0x61,
0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4d, 0x75, 0x72, 0x69, 0x74, 0xe1,
0x6e, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69,
0x20, 0x77, 0x61, 0x68, 0x64, 0x4d, 0x75, 0x72, 0x69, 0x74, 0xe1, 0x6e,
0x69, 0xe1, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69,
0x20, 0x77, 0x61, 0x68, 0x64, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x61,
0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0xc1, 0x72, 0xe1, 0x6d, 0x7a, 0x65,
0x72, 0x69, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0xf3, 0x61, 0x79,
0x20, 0x77, 0x61, 0x68, 0x64, 0xc1, 0x72, 0xe1, 0x6d, 0x7a, 0x65, 0x72,
0x69, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0xf3, 0x61, 0x79, 0x20,
0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0xc1,
0x72, 0xe1, 0x6d, 0x7a, 0x65, 0x72, 0x69, 0x20, 0x4d, 0x65, 0x6b, 0x73,
0x69, 0x6b, 0xf3, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x55, 0x6c, 0xe1, 0x6e, 0x62, 0xe1,
0x74, 0x61, 0x72, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x55, 0x6c,
0xe1, 0x6e, 0x62, 0xe1, 0x74, 0x61, 0x72, 0x61, 0x79, 0x20, 0x61, 0x6e,
0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x55, 0x6c, 0xe1,
0x6e, 0x62, 0xe1, 0x74, 0x61, 0x72, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72,
0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4d, 0xe1, 0x73,
0x6b, 0xf3, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4d, 0xe1, 0x73,
0x6b, 0xf3, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20,
0x77, 0x61, 0x68, 0x64, 0x4d, 0xe1, 0x73, 0x6b, 0xf3, 0x61, 0x79, 0x20,
0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0x4d, 0x79, 0x61, 0x6e, 0x6d, 0xe1, 0x72, 0x61, 0x79, 0x20, 0x77, 0x61,
0x68, 0x64, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x61, 0x79, 0x20, 0x77, 0x61,
0x68, 0x64, 0x4e, 0xe9, 0x70, 0xe1, 0x6c, 0x61, 0x79, 0x20, 0x77, 0x61,
0x68, 0x64, 0x4e, 0x79, 0x75, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0xf3,
0x6e, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4e, 0x79,
0x75, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0xf3, 0x6e, 0x69, 0xe1, 0x61,
0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0x4e, 0x79, 0x75, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0xf3, 0x6e,
0x69, 0xe1, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69,
0x20, 0x77, 0x61, 0x68, 0x64, 0x4e, 0x69, 0x75, 0x20, 0x5a, 0x69, 0x6c,
0xe1, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4e, 0x69,
0x75, 0x20, 0x5a, 0x69, 0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x61,
0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4e, 0x69,
0x75, 0x20, 0x5a, 0x69, 0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x72,
0xf3, 0x63, 0x68, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4e, 0x69, 0x70,
0xe1, 0x6e, 0x64, 0x6c, 0x61, 0x79, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x77,
0x61, 0x68, 0x64, 0x4e, 0x69, 0x70, 0xe1, 0x6e, 0x64, 0x6c, 0x61, 0x79,
0x6e, 0x64, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20,
0x77, 0x61, 0x68, 0x64, 0x4e, 0x69, 0x70, 0xe1, 0x6e, 0x64, 0x6c, 0x61,
0x79, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4e, 0x69, 0x75, 0x61, 0x79, 0x20,
0x77, 0x61, 0x68, 0x64, 0x4e, 0x75, 0x72, 0x70, 0x6f, 0x6c, 0x6b, 0x20,
0x49, 0x73, 0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68,
0x64, 0x4e, 0x75, 0x72, 0x70, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c,
0xe1, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69,
0x20, 0x77, 0x61, 0x68, 0x64, 0x4e, 0x75, 0x72, 0x70, 0x6f, 0x6c, 0x6b,
0x20, 0x49, 0x73, 0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x72, 0xf3,
0x63, 0x68, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69,
0x20, 0x77, 0x61, 0x68, 0x64, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0xe1, 0x61,
0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61,
0x68, 0x64, 0x4e, 0x61, 0x77, 0xe1, 0x73, 0x69, 0x62, 0x69, 0x73, 0x6b,
0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4e, 0x61, 0x77, 0xe1, 0x73,
0x69, 0x62, 0x69, 0x73, 0x6b, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1,
0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4e, 0x61, 0x77, 0xe1, 0x73,
0x69, 0x62, 0x69, 0x73, 0x6b, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d,
0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0xd3, 0x6d, 0x73, 0x6b,
0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0xd3, 0x6d, 0x73, 0x6b, 0x61,
0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0xd3, 0x6d, 0x73, 0x6b, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d,
0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x50, 0xe1, 0x6b, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x50,
0xe1, 0x6b, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x61, 0x79, 0x20, 0x61, 0x6e,
0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x50, 0xe1, 0x6b,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d,
0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x50, 0x61, 0x6c, 0xe1,
0x75, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x50, 0xe1, 0x70, 0x75,
0xe1, 0x20, 0x4e, 0x69, 0x75, 0x20, 0x47, 0x69, 0x6e, 0x69, 0xe1, 0x61,
0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x50, 0x61, 0x72, 0x61, 0x67, 0xf3,
0x61, 0x79, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x50, 0x61, 0x72,
0x61, 0x67, 0xf3, 0x61, 0x79, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1,
0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x50, 0x61, 0x72, 0x61, 0x67,
0xf3, 0x61, 0x79, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x50, 0xe9, 0x72, 0xf3, 0x61, 0x79,
0x20, 0x77, 0x61, 0x68, 0x64, 0x50, 0xe9, 0x72, 0xf3, 0x61, 0x79, 0x20,
0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x50,
0xe9, 0x72, 0xf3, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x50, 0x65, 0x6c, 0x70, 0x69, 0x6e,
0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x50, 0x65, 0x6c, 0x70, 0x69,
0x6e, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77,
0x61, 0x68, 0x64, 0x50, 0x65, 0x6c, 0x70, 0x69, 0x6e, 0x61, 0x79, 0x20,
0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0x50, 0x68, 0x6f, 0x65, 0x6e, 0x65, 0x6b, 0x73, 0x20, 0x49, 0x73, 0x6c,
0xe1, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x53, 0x74,
0x2e, 0x20, 0x50, 0xe9, 0x72, 0x69, 0x20, 0x6f, 0x20, 0x4d, 0x69, 0x6b,
0x77, 0xe9, 0x6c, 0x69, 0x6e, 0x20, 0x77, 0x61, 0x68, 0x64, 0x53, 0x74,
0x2e, 0x20, 0x50, 0xe9, 0x72, 0x69, 0x20, 0x6f, 0x20, 0x4d, 0x69, 0x6b,
0x77, 0xe9, 0x6c, 0x69, 0x6e, 0x20, 0x61, 0x6a, 0xe1, 0x72, 0x69, 0x20,
0x77, 0x61, 0x68, 0x64, 0x53, 0x74, 0x2e, 0x20, 0x50, 0xe9, 0x72, 0x69,
0x20, 0x6f, 0x20, 0x4d, 0x69, 0x6b, 0x77, 0xe9, 0x6c, 0x69, 0x6e, 0x20,
0x72, 0xf3, 0x63, 0x68, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x50, 0x69,
0x74, 0x6b, 0x61, 0x72, 0xe9, 0x6e, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68,
0x64, 0x50, 0xf3, 0x6e, 0x70, 0x69, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68,
0x64, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0xe1, 0x6e, 0x67, 0x61, 0x79,
0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xe9, 0x79, 0x75, 0x6e, 0x69, 0x61,
0x6e, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0x6f, 0x74, 0x68,
0xe9, 0x72, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x53, 0x61,
0x6b, 0x68, 0xe1, 0x6c, 0x69, 0x6e, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68,
0x64, 0x53, 0x61, 0x6b, 0x68, 0xe1, 0x6c, 0x69, 0x6e, 0x61, 0x79, 0x20,
0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x53,
0x61, 0x6b, 0x68, 0xe1, 0x6c, 0x69, 0x6e, 0x61, 0x79, 0x20, 0x67, 0x61,
0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x53, 0x61,
0x6d, 0xf3, 0xe1, 0x77, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x53,
0x61, 0x6d, 0xf3, 0xe1, 0x77, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1,
0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x53, 0x61, 0x6d, 0xf3, 0xe1,
0x77, 0x61, 0x79, 0x20, 0x72, 0xf3, 0x63, 0x68, 0x69, 0x20, 0x77, 0x61,
0x68, 0x64, 0x53, 0xe9, 0x63, 0x68, 0x65, 0x6c, 0x65, 0x73, 0x61, 0x79,
0x20, 0x77, 0x61, 0x68, 0x64, 0x53, 0x65, 0x6e, 0x67, 0xe1, 0x70, 0x75,
0x72, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77,
0x61, 0x68, 0x64, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0xe1, 0x6e, 0x20, 0x49,
0x73, 0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64,
0x5a, 0x65, 0x72, 0x62, 0xe1, 0x72, 0x69, 0x20, 0x4a, 0xe1, 0x72, 0x6a,
0x69, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x53, 0x75, 0x72,
0x69, 0x6e, 0x61, 0x79, 0x6d, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64,
0x53, 0x79, 0xf3, 0x77, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64,
0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68,
0x64, 0x54, 0xe1, 0x69, 0x70, 0x69, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68,
0x64, 0x54, 0xe1, 0x69, 0x70, 0x69, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a,
0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x54, 0xe1, 0x69, 0x70,
0x69, 0x61, 0x79, 0x20, 0x72, 0xf3, 0x63, 0x68, 0x69, 0x20, 0x77, 0x61,
0x68, 0x64, 0x54, 0xe1, 0x6a, 0x65, 0x6b, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x54, 0x6f, 0x6b, 0x65, 0x6c,
0xe1, 0x75, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x54, 0x6f, 0x6e,
0x67, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x54, 0x6f, 0x6e,
0x67, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20,
0x77, 0x61, 0x68, 0x64, 0x54, 0x6f, 0x6e, 0x67, 0xe1, 0x61, 0x79, 0x20,
0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0x43, 0x68, 0x75, 0x6b, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x54,
0x6f, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x61,
0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x54, 0x6f, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a,
0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x54, 0x6f, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x61, 0x79, 0x20, 0x67,
0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x54,
0x75, 0x77, 0xe1, 0x6c, 0x75, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64,
0x59, 0x75, 0x72, 0xf3, 0x67, 0xf3, 0x61, 0x79, 0x61, 0x79, 0x20, 0x77,
0x61, 0x68, 0x64, 0x59, 0x75, 0x72, 0xf3, 0x67, 0xf3, 0x61, 0x79, 0x61,
0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0x59, 0x75, 0x72, 0xf3, 0x67, 0xf3, 0x61, 0x79, 0x61, 0x79, 0x20,
0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0x4f, 0x7a, 0x62, 0x65, 0x6b, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x61, 0x79,
0x20, 0x77, 0x61, 0x68, 0x64, 0x4f, 0x7a, 0x62, 0x65, 0x6b, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69,
0x20, 0x77, 0x61, 0x68, 0x64, 0x4f, 0x7a, 0x62, 0x65, 0x6b, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x57, 0xe1, 0x6e, 0x75, 0xe1, 0x74,
0x75, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x57, 0xe1, 0x6e, 0x75,
0xe1, 0x74, 0x75, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69,
0x20, 0x77, 0x61, 0x68, 0x64, 0x57, 0xe1, 0x6e, 0x75, 0xe1, 0x74, 0x75,
0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77,
0x61, 0x68, 0x64, 0x57, 0x65, 0x6e, 0x65, 0x7a, 0x77, 0xe9, 0x6c, 0xe1,
0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x57, 0x61, 0x6c, 0xe1, 0x64,
0x69, 0x77, 0xe1, 0x73, 0x74, 0xf3, 0x6b, 0x61, 0x79, 0x20, 0x77, 0x61,
0x68, 0x64, 0x57, 0x61, 0x6c, 0xe1, 0x64, 0x69, 0x77, 0xe1, 0x73, 0x74,
0xf3, 0x6b, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20,
0x77, 0x61, 0x68, 0x64, 0x57, 0x61, 0x6c, 0xe1, 0x64, 0x69, 0x77, 0xe1,
0x73, 0x74, 0xf3, 0x6b, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1,
0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x57, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x57,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x61, 0x79, 0x20, 0x61,
0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x57, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x61, 0x79, 0x20, 0x67, 0x61,
0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x57, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x57,
0x61, 0x79, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x79,
0x20, 0x77, 0x61, 0x68, 0x64, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20,
0x6f, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0xe1, 0x61, 0x79, 0x20, 0x77,
0x61, 0x68, 0x64, 0x59, 0xe1, 0x6b, 0x75, 0x73, 0x6b, 0x61, 0x79, 0x20,
0x77, 0x61, 0x68, 0x64, 0x59, 0xe1, 0x6b, 0x75, 0x73, 0x6b, 0x61, 0x79,
0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0x59, 0xe1, 0x6b, 0x75, 0x73, 0x6b, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72,
0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x59, 0x61, 0x6b,
0xe1, 0x74, 0x72, 0x69, 0x6e, 0x62, 0x6f, 0x72, 0x67, 0x61, 0x79, 0x20,
0x77, 0x61, 0x68, 0x64, 0x59, 0x61, 0x6b, 0xe1, 0x74, 0x72, 0x69, 0x6e,
0x62, 0x6f, 0x72, 0x67, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x59, 0x61, 0x6b, 0xe1, 0x74, 0x72,
0x69, 0x6e, 0x62, 0x6f, 0x72, 0x67, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72,
0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x59, 0x75, 0x6b,
0xf3, 0x6e, 0x20, 0x77, 0x61, 0x68, 0x64, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x41, 0x66, 0x67, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x66,
0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x6d, 0x65, 0x7a, 0x6f,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x66, 0x72, 0x69,
0x63, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x76, 0x61, 0x6e, 0x74,
0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x66, 0x72,
0x69, 0x63, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x6d, 0x65, 0x72, 0x69, 0x64,
0x69, 0x6f, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41,
0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x20, 0x70, 0x6f, 0x6e,
0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x66, 0x72,
0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e,
0x74, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20,
0x64, 0x65, 0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x73, 0x73, 0xf2, 0x6e, 0x69,
0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x73, 0x73, 0xf2,
0x6e, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x73, 0x73, 0xf2,
0x6e, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x6d, 0x65, 0x7a,
0x6f, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x6f, 0x20, 0x6d, 0x65, 0x7a, 0x6f, 0x20, 0x6e,
0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0xf1, 0x61,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x6f,
0x20, 0x6d, 0x65, 0x7a, 0x6f, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x6f,
0x20, 0x6c, 0x65, 0x76, 0x61, 0x6e, 0x74, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0xf1, 0x61, 0x6f, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x6f,
0x20, 0x6c, 0x65, 0x76, 0x61, 0x6e, 0x74, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0xf1, 0x61, 0x6f, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x6f, 0x20, 0x6c,
0x65, 0x76, 0x61, 0x6e, 0x74, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61, 0x67, 0x6e, 0x65, 0x20, 0x41,
0x6c, 0x6c, 0x69, 0x67, 0x67, 0x69, 0xe6, 0x6f, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f,
0x6e, 0x74, 0x61, 0x67, 0x6e, 0x65, 0x20, 0x41, 0x6c, 0x6c, 0x69, 0x67,
0x67, 0x69, 0xe6, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61, 0x67, 0x6e, 0x65,
0x20, 0x41, 0x6c, 0x6c, 0x69, 0x67, 0x67, 0x69, 0xe6, 0x6f, 0x61, 0x20,
0x64, 0x6f, 0x20, 0x50, 0x61, 0xe7, 0x69, 0x66, 0x69, 0x63, 0x6f, 0x20,
0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0xf1,
0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0xe7, 0x69, 0x66, 0x69, 0x63, 0x6f,
0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61,
0xf1, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20,
0x64, 0x6f, 0x20, 0x50, 0x61, 0xe7, 0x69, 0x66, 0x69, 0x63, 0x6f, 0x20,
0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0xf1,
0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x6f,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x61, 0x62, 0x69,
0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0xf1, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74,
0xe6, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0xf1, 0x61, 0x20, 0x64, 0x65,
0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0xf1, 0x61, 0x20, 0x64,
0x65, 0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0xf1, 0x61, 0x20, 0x64, 0x65,
0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x45, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x6f,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x45, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x45, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x63, 0x6f, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x20, 0x6e, 0x6f, 0x72,
0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0xf1, 0x61, 0x6f, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6d, 0x65, 0x7a, 0x6f, 0x6f,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6d, 0x65, 0x7a, 0x6f, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6d, 0x65, 0x7a, 0x6f, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64,
0x6f, 0x20, 0x6d, 0x65, 0x7a, 0x6f, 0x2d, 0x70, 0x6f, 0x6e, 0x65, 0x6e,
0x74, 0x65, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x6d, 0x65, 0x7a, 0x6f,
0x2d, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x6d, 0x65, 0x7a, 0x6f, 0x2d, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x76,
0x61, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65,
0x76, 0x61, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x76,
0x61, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x65,
0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64,
0x65, 0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x65,
0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x67,
0x69, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x61, 0x69, 0x67, 0x69, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x61, 0x69, 0x67, 0x69, 0x61, 0x6e, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x6f, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74,
0xe6, 0x20, 0x64, 0x65, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x6f, 0x61,
0x20, 0x64, 0x6f, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x73, 0x68, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x6f, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x73, 0x68, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x6f, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x73, 0x68, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x42, 0x68, 0x75, 0x74,
0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x61, 0x20, 0x42, 0x6f, 0x6c, 0x69,
0x76, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74,
0xe6, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69,
0x61, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x42, 0x72, 0x75, 0x6e, 0x65,
0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x70, 0x70, 0x6f, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x70, 0x70,
0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x70, 0x70,
0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x6f, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x43, 0x69,
0x6c, 0x65, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x6f, 0x20, 0x43, 0x69, 0x6c, 0x65, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x6f, 0x20, 0x43, 0x69,
0x6c, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x61, 0x20, 0x43, 0x69, 0xf1, 0x61,
0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x61, 0x20, 0x43, 0x69, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x61, 0x20, 0x43, 0x69, 0xf1, 0x61,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x69, 0x73, 0x6f, 0x61,
0x20, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x6f, 0x61,
0x20, 0x64, 0x65, 0x20, 0x69, 0x73, 0x6f, 0x65, 0x20, 0x43, 0x6f, 0x63,
0x6f, 0x73, 0x6f, 0x61, 0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0x6f,
0x6d, 0x62, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d,
0x62, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x69, 0x73, 0x6f, 0x65, 0x20, 0x43,
0x6f, 0x6f, 0x6b, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x69, 0x73, 0x6f, 0x65, 0x20, 0x43,
0x6f, 0x6f, 0x6b, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x65, 0x20, 0x69, 0x73, 0x6f, 0x65, 0x20, 0x43, 0x6f, 0x6f,
0x6b, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x62, 0x61,
0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x62, 0x61, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62,
0x62, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x44, 0x61, 0x76, 0x69,
0x73, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e,
0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x45,
0x73, 0x74, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x69, 0x73,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x71, 0x75, 0x61,
0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x69, 0x73, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x50, 0x61, 0x73, 0x71, 0x75, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x69, 0x73,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x71, 0x75, 0x61,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x63, 0x75, 0x61,
0x64, 0x6f, 0x72, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45,
0x75, 0x72, 0xf6, 0x70, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x6d, 0x65, 0x7a,
0x6f, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0xf6, 0x70, 0x61,
0x20, 0x64, 0x6f, 0x20, 0x6d, 0x65, 0x7a, 0x6f, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45,
0x75, 0x72, 0xf6, 0x70, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x6d, 0x65, 0x7a,
0x6f, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72,
0xf6, 0x70, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x76, 0x61, 0x6e,
0x74, 0x65, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0xf6, 0x70,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x76, 0x61, 0x6e, 0x74, 0x65,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0xf6, 0x70, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x65, 0x76, 0x61, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4b, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x6e, 0x67, 0x72, 0x61,
0x64, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72,
0xf6, 0x70, 0x61, 0x20, 0x64, 0x65, 0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e,
0x74, 0x65, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0xf6, 0x70,
0x61, 0x20, 0x64, 0x65, 0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0xf6, 0x70, 0x61, 0x20, 0x64, 0x65,
0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x69, 0x73, 0x6f, 0x65, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69,
0xf1, 0x65, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x69, 0x73, 0x6f, 0x65, 0x20, 0x4d, 0x61,
0x6c, 0x76, 0x69, 0xf1, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x69, 0x73, 0x6f, 0x65, 0x20, 0x4d,
0x61, 0x6c, 0x76, 0x69, 0xf1, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x46, 0x69, 0x67, 0x69, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x67, 0x69, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20,
0x46, 0x69, 0x67, 0x69, 0x6f, 0x61, 0x20, 0x64, 0x61, 0x20, 0x47, 0x75,
0x79, 0x61, 0x6e, 0x61, 0x20, 0x66, 0x72, 0x61, 0x6e, 0x73, 0x65, 0x69,
0x73, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0xe6, 0x72, 0x65,
0x20, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x20, 0x65, 0x20,
0x61, 0x6e, 0x74, 0x61, 0x72, 0x74, 0x69, 0x63, 0x68, 0x65, 0x20, 0x66,
0x72, 0x61, 0x6e, 0x73, 0x65, 0x69, 0x78, 0x69, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72,
0x6f, 0x61, 0x20, 0x64, 0x61, 0x20, 0x47, 0x65, 0xf2, 0x72, 0x67, 0x69,
0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x61, 0x20, 0x47, 0x65, 0xf2, 0x72, 0x67, 0x69, 0x61, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x61, 0x20,
0x47, 0x65, 0xf2, 0x72, 0x67, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x69, 0x73, 0x6f, 0x65, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72,
0x74, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x6d, 0x65, 0x72, 0x69, 0x64,
0x69, 0x61, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x77, 0x69, 0x63, 0x68, 0x6f, 0x61, 0x20, 0x64, 0x61, 0x20, 0x47, 0x72,
0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x65, 0x76, 0x61, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x61, 0x20, 0x47,
0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x65, 0x76, 0x61, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x61, 0x20, 0x47, 0x72,
0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x65, 0x76, 0x61, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x64,
0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x61, 0x20, 0x64, 0x65, 0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65,
0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74,
0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64,
0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x61, 0x20, 0x64, 0x65, 0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65,
0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x6f, 0x20, 0x47, 0x6f, 0x72, 0x66, 0x6f, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65,
0x75, 0x74, 0x69, 0xf1, 0x65, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61,
0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0xf1, 0x65, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20,
0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x69, 0xf1, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e,
0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e,
0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6e, 0x67, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f,
0x76, 0x64, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f,
0x76, 0x64, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x6e,
0x64, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f,
0xe7, 0x65, 0x61, 0x6e, 0x6f, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x6e, 0x64, 0x6f,
0x63, 0x69, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6d, 0x65, 0x7a, 0x6f, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x65, 0x76, 0x61, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73,
0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74,
0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x72, 0x61,
0x6e, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x72, 0x61, 0x6e, 0x6f, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x49, 0x72, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74,
0xe6, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x6f, 0x61, 0x20, 0x64, 0x2019, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x65,
0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x2019, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x65, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x2019, 0x49, 0x73, 0x72,
0x61, 0x65, 0x6c, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x47, 0x69,
0x61, 0x70, 0x70, 0x6f, 0x6e, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x6f, 0x20, 0x47, 0x69, 0x61, 0x70,
0x70, 0x6f, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x6f, 0x20, 0x47, 0x69, 0x61, 0x70, 0x70, 0x6f, 0x6e, 0x6f,
0x61, 0x20, 0x64, 0x6f, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x76, 0x61, 0x6e,
0x74, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4b, 0x61, 0x7a, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x70, 0x6f,
0x6e, 0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x61, 0x20, 0x43,
0x6f, 0x72, 0x65, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x61,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x61,
0x20, 0x43, 0x6f, 0x72, 0x65, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x6f,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73,
0x6b, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64,
0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73,
0x6b, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4b, 0x69, 0x72, 0x67, 0x68,
0x69, 0x7a, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x69, 0x73, 0x6f, 0x65, 0x20, 0x64, 0x61, 0x20, 0x4c, 0x69, 0x6e,
0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x6f, 0x61,
0x20, 0x64, 0x61, 0x20, 0x4d, 0x61, 0x6c, 0x65, 0x73, 0x69, 0x61, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x72, 0x63, 0x68, 0x65,
0x69, 0x78, 0x69, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x69, 0x73, 0x6f,
0x65, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x6f, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x6f,
0x61, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0xe7, 0x69, 0x66, 0x69, 0x63,
0x6f, 0x20, 0x6d, 0x65, 0x73, 0x63, 0x69, 0x63, 0x61, 0xf1, 0x61, 0x6f,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x6f, 0x20, 0x50, 0x61, 0xe7, 0x69, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x6d,
0x65, 0x73, 0x63, 0x69, 0x63, 0x61, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0xe7,
0x69, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x6d, 0x65, 0x73, 0x63, 0x69, 0x63,
0x61, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x2019, 0x55, 0x6c, 0x61, 0x6e,
0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x55, 0x6c, 0x61, 0x6e,
0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x74, 0xe6, 0x20, 0x64, 0x2019, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42,
0x61, 0x74, 0x6f, 0x72, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f,
0x73, 0x63, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x61, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x6f, 0x73, 0x63, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x61, 0x20, 0x42,
0x69, 0x72, 0x6d, 0x61, 0x6e, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x4e, 0x65, 0x70, 0x61, 0x6c, 0x6f, 0x61, 0x20, 0x64, 0x61, 0x20, 0x4e,
0x65, 0x75, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0xf2, 0x6e,
0x69, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x61, 0x20, 0x4e, 0x65, 0x75, 0x76, 0x61, 0x20, 0x43,
0x61, 0x6c, 0x65, 0x64, 0xf2, 0x6e, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x61, 0x20, 0x4e, 0x65, 0x75,
0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0xf2, 0x6e, 0x69, 0x61,
0x6f, 0x61, 0x20, 0x64, 0x61, 0x20, 0x4e, 0x65, 0x75, 0x76, 0x61, 0x20,
0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x61, 0x20, 0x4e, 0x65,
0x75, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x61, 0x20,
0x4e, 0x65, 0x75, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64,
0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0xe6, 0x72, 0x61, 0x6e,
0x65, 0x75, 0x76, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x54, 0xe6, 0x72, 0x61, 0x6e,
0x65, 0x75, 0x76, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74,
0xe6, 0x20, 0x64, 0x65, 0x20, 0x54, 0xe6, 0x72, 0x61, 0x6e, 0x65, 0x75,
0x76, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x69, 0x75, 0x65,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x69, 0x73, 0x6f, 0x61,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x6f, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x69, 0x73, 0x6f, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x69, 0x73, 0x6f, 0x61, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x68, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0xd2, 0x6d, 0x73, 0x6b, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0xd2, 0x6d, 0x73, 0x6b, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20,
0xd2, 0x6d, 0x73, 0x6b, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x74, 0xe6, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x6c, 0x61,
0x75, 0x6f, 0x61, 0x20, 0x64, 0x61, 0x20, 0x50, 0x61, 0x70, 0x75, 0x61,
0x20, 0x4e, 0x65, 0x75, 0x76, 0x61, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65,
0x61, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x79, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20,
0x64, 0x6f, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x6f,
0x61, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x65, 0x72, 0xf9, 0x6f, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x6f, 0x20,
0x50, 0x65, 0x72, 0xf9, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74,
0xe6, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x65, 0x72, 0xf9, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0xf1, 0x65, 0x6f,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0xf1, 0x65, 0x6f, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x46,
0x69, 0x6c, 0x69, 0x70, 0x69, 0xf1, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x69, 0x73, 0x6f, 0x65, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69,
0x78, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6e, 0x20, 0x50,
0xea, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x6e, 0x20, 0x50, 0xea, 0x20, 0x65, 0x20,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6e,
0x20, 0x50, 0xea, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x69, 0x74, 0x63,
0x61, 0x69, 0x72, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x6f,
0x68, 0x6e, 0x70, 0x65, 0x69, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50,
0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x6f, 0x61, 0x20, 0x64,
0x61, 0x20, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x6f, 0x61,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20,
0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65,
0x73, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x63, 0x69, 0x6e, 0x67,
0x61, 0x70, 0xf4, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x69, 0x73, 0x6f,
0x65, 0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x6f, 0x61, 0x20,
0x64, 0x61, 0x20, 0x47, 0x65, 0xf2, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64,
0x6f, 0x20, 0x73, 0x75, 0x64, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x53,
0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69,
0x70, 0x65, 0x69, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x6f, 0x61,
0x20, 0x64, 0x6f, 0x20, 0x54, 0x61, 0x67, 0x69, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x6f, 0x6b, 0x65,
0x6c, 0x61, 0x75, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x6f, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x43, 0x68,
0x75, 0x75, 0x6b, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x54, 0x75, 0x72,
0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x6f, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x6f,
0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x75, 0x76, 0x61, 0x6c,
0x75, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x79, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x79, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74,
0xe6, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x79, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x7a,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x6f, 0x61, 0x20, 0x64, 0x6f,
0x20, 0x56, 0x65, 0x6e, 0x65, 0x73, 0x73, 0x75, 0x65, 0x6c, 0x61, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x69, 0x73, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x57,
0x61, 0x6b, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x57, 0x61, 0x6c,
0x6c, 0x69, 0x73, 0x20, 0x65, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20,
0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x61, 0x20, 0x64, 0x2019,
0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x2019, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20,
0x64, 0x2019, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x59, 0x75, 0x6b, 0x6f,
0x6e, 0x627, 0x641, 0x63a, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633,
0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x627, 0x6cc, 0x645, 0x6cc, 0x632, 0x648, 0x646,
0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648,
0x64e, 0x62e, 0x627, 0x645, 0x6cc, 0x632, 0x648, 0x646, 0x20, 0x633, 0x6cc, 0x20,
0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc,
0x20, 0x648, 0x64e, 0x62e, 0x639, 0x631, 0x628, 0x20, 0x633, 0x6cc, 0x20, 0x648,
0x64e, 0x62e, 0x639, 0x631, 0x628, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc,
0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x627, 0x631, 0x62c, 0x646, 0x679,
0x6cc, 0x646, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x627, 0x631,
0x62c, 0x646, 0x679, 0x6cc, 0x646, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639,
0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x627, 0x631, 0x62c, 0x646,
0x679, 0x6cc, 0x646, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645,
0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e,
0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627, 0x631, 0x62c, 0x646, 0x679, 0x6cc,
0x646, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x645, 0x63a, 0x631,
0x628, 0x6cc, 0x20, 0x627, 0x631, 0x62c, 0x646, 0x679, 0x6cc, 0x646, 0x627, 0x20,
0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e,
0x62e, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627, 0x631, 0x62c, 0x646, 0x679,
0x6cc, 0x646, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20,
0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x622,
0x631, 0x645, 0x6cc, 0x646, 0x6cc, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e,
0x62e, 0x622, 0x631, 0x645, 0x6cc, 0x646, 0x6cc, 0x627, 0x20, 0x633, 0x6cc, 0x20,
0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x622, 0x631,
0x645, 0x6cc, 0x646, 0x6cc, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633,
0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e,
0x62e, 0x622, 0x630, 0x631, 0x628, 0x627, 0x626, 0x6cc, 0x62c, 0x627, 0x646, 0x20,
0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x622, 0x630, 0x631, 0x628, 0x627, 0x626,
0x6cc, 0x62c, 0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627,
0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x622, 0x630, 0x631, 0x628, 0x627, 0x626,
0x6cc, 0x62c, 0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645,
0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e,
0x627, 0x632, 0x648, 0x631, 0x6cc, 0x633, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e,
0x62e, 0x627, 0x632, 0x648, 0x631, 0x6cc, 0x633, 0x20, 0x633, 0x6cc, 0x20, 0x645,
0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x627, 0x632, 0x648,
0x631, 0x6cc, 0x633, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20,
0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x628,
0x646, 0x6af, 0x644, 0x6c1, 0x20, 0x62f, 0x6cc, 0x634, 0x20, 0x633, 0x6cc, 0x20,
0x648, 0x64e, 0x62e, 0x628, 0x646, 0x6af, 0x644, 0x6c1, 0x20, 0x62f, 0x6cc, 0x634,
0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648,
0x64e, 0x62e, 0x628, 0x646, 0x6af, 0x644, 0x6c1, 0x20, 0x62f, 0x6cc, 0x634, 0x20,
0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627,
0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x628, 0x6be, 0x648, 0x679, 0x627,
0x646, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x628, 0x648, 0x644, 0x6cc,
0x648, 0x6cc, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x686, 0x644,
0x6cc, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x686, 0x644, 0x6cc, 0x20,
0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e,
0x62e, 0x686, 0x644, 0x6cc, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645,
0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e,
0x686, 0x6cc, 0x646, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x6a9, 0x648,
0x644, 0x645, 0x628, 0x6cc, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc,
0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x6a9, 0x648, 0x644, 0x645, 0x628,
0x6cc, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af,
0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x627, 0x6cc,
0x633, 0x679, 0x631, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x633,
0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x627, 0x6cc, 0x633, 0x679, 0x631, 0x20, 0x622,
0x626, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc,
0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x627, 0x6cc, 0x633, 0x679, 0x631,
0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x633, 0x6cc, 0x20, 0x645,
0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20,
0x648, 0x64e, 0x62e, 0x627, 0x6cc, 0x6a9, 0x648, 0x627, 0x688, 0x648, 0x631, 0x20,
0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x648, 0x633, 0x637, 0x20, 0x6cc, 0x648,
0x631, 0x67e, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x648, 0x633, 0x637,
0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639,
0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x648, 0x633, 0x637, 0x6cc,
0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633,
0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e,
0x62e, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20,
0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20,
0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627,
0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20,
0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645,
0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e,
0x628, 0x639, 0x6cc, 0x62f, 0x20, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x6cc,
0x648, 0x631, 0x67e, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x645, 0x63a, 0x631, 0x628,
0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e,
0x62e, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20,
0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e,
0x62e, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20,
0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627,
0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x641, 0x627, 0x6a9, 0x20, 0x644,
0x6cc, 0x646, 0x688, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x632, 0x20,
0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x641, 0x627, 0x6a9, 0x20, 0x644, 0x6cc,
0x646, 0x688, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x632, 0x20, 0x633,
0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e,
0x641, 0x627, 0x6a9, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x622, 0x626, 0x644,
0x6cc, 0x646, 0x688, 0x632, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645,
0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e,
0x641, 0x631, 0x6cc, 0x646, 0x686, 0x20, 0x6af, 0x6cc, 0x627, 0x646, 0x627, 0x20,
0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x641, 0x631, 0x6cc, 0x646, 0x686, 0x20,
0x62c, 0x646, 0x648, 0x628, 0x6cc, 0x20, 0x622, 0x6ba, 0x20, 0x627, 0x646, 0x679,
0x627, 0x631, 0x6a9, 0x679, 0x6a9, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6af, 0x627,
0x644, 0x627, 0x67e, 0x627, 0x6af, 0x648, 0x632, 0x20, 0x633, 0x6cc, 0x20, 0x648,
0x64e, 0x62e, 0x62c, 0x627, 0x631, 0x62c, 0x6cc, 0x627, 0x20, 0x633, 0x6cc, 0x20,
0x648, 0x64e, 0x62e, 0x62c, 0x627, 0x631, 0x62c, 0x6cc, 0x627, 0x20, 0x633, 0x6cc,
0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x62c,
0x627, 0x631, 0x62c, 0x6cc, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633,
0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e,
0x62e, 0x6af, 0x631, 0x6cc, 0x646, 0x20, 0x648, 0x686, 0x20, 0x633, 0x6cc, 0x20,
0x627, 0x635, 0x644, 0x20, 0x648, 0x64e, 0x62e, 0x645, 0x634, 0x631, 0x642, 0x6cc,
0x20, 0x6af, 0x631, 0x6cc, 0x646, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x633,
0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20,
0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20,
0x6af, 0x631, 0x6cc, 0x646, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x633, 0x6cc,
0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633,
0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x62e, 0x644, 0x6cc, 0x62c, 0x20, 0x633, 0x6cc,
0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x6c1,
0x627, 0x646, 0x6af, 0x20, 0x633, 0x6cc, 0x646, 0x6af, 0x20, 0x679, 0x627, 0x626,
0x645, 0x6c1, 0x627, 0x646, 0x6af, 0x20, 0x633, 0x6cc, 0x646, 0x6af, 0x20, 0x633,
0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6c1,
0x627, 0x646, 0x6af, 0x20, 0x633, 0x6cc, 0x646, 0x6af, 0x20, 0x633, 0x645, 0x631,
0x20, 0x679, 0x627, 0x626, 0x645, 0x6c1, 0x646, 0x62f, 0x648, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20,
0x648, 0x64e, 0x62e, 0x627, 0x6cc, 0x631, 0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20,
0x648, 0x64e, 0x62e, 0x627, 0x6cc, 0x631, 0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20,
0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x627, 0x633,
0x631, 0x627, 0x626, 0x6cc, 0x644, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e,
0x627, 0x633, 0x631, 0x627, 0x626, 0x6cc, 0x644, 0x20, 0x633, 0x6cc, 0x20, 0x645,
0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x645, 0x634, 0x631,
0x642, 0x6cc, 0x20, 0x642, 0x632, 0x627, 0x62e, 0x633, 0x62a, 0x627, 0x646, 0x20,
0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20,
0x642, 0x632, 0x627, 0x62e, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20,
0x648, 0x64e, 0x62e, 0x6a9, 0x631, 0x63a, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633,
0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x645, 0x627, 0x644, 0x62f, 0x6cc, 0x67e, 0x20,
0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x646, 0x6cc, 0x67e, 0x627, 0x644, 0x20,
0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x646, 0x6cc, 0x648, 0x632, 0x6cc, 0x20,
0x644, 0x6cc, 0x646, 0x688, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x646,
0x6cc, 0x648, 0x632, 0x6cc, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x633, 0x6cc,
0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x646,
0x627, 0x631, 0x641, 0x648, 0x6a9, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688,
0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x646, 0x627, 0x631, 0x641, 0x648,
0x6a9, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x633, 0x6cc, 0x20,
0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x646, 0x627,
0x631, 0x641, 0x648, 0x6a9, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x20,
0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627,
0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x641, 0x631, 0x646, 0x627, 0x646,
0x688, 0x648, 0x20, 0x688, 0x6cc, 0x20, 0x646, 0x648, 0x631, 0x646, 0x6c1, 0x627,
0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x641, 0x631, 0x646, 0x627, 0x646,
0x688, 0x648, 0x20, 0x688, 0x6cc, 0x20, 0x646, 0x648, 0x631, 0x646, 0x6c1, 0x627,
0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648,
0x64e, 0x62e, 0x67e, 0x627, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633, 0x6cc,
0x20, 0x648, 0x64e, 0x62e, 0x67e, 0x627, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20,
0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e,
0x62e, 0x67e, 0x627, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20,
0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc,
0x20, 0x648, 0x64e, 0x62e, 0x67e, 0x6cc, 0x631, 0x627, 0x6af, 0x648, 0x626, 0x6d2,
0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x67e, 0x6cc, 0x631, 0x627, 0x6af,
0x648, 0x626, 0x6d2, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631,
0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x67e, 0x6cc, 0x631, 0x627, 0x6af, 0x648, 0x626,
0x6d2, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631,
0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x67e, 0x6cc, 0x631,
0x648, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x67e, 0x6cc, 0x631, 0x648,
0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648,
0x64e, 0x62e, 0x67e, 0x6cc, 0x631, 0x648, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648,
0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648,
0x64e, 0x62e, 0x633, 0x6cc, 0x646, 0x679, 0x20, 0x67e, 0x6cc, 0x626, 0x631, 0x20,
0x622, 0x6ba, 0x20, 0x645, 0x6a9, 0x644, 0x6cc, 0x626, 0x648, 0x646, 0x20, 0x679,
0x627, 0x626, 0x645, 0x633, 0x6cc, 0x646, 0x679, 0x20, 0x67e, 0x6cc, 0x626, 0x631,
0x20, 0x622, 0x6ba, 0x20, 0x645, 0x6a9, 0x644, 0x6cc, 0x626, 0x648, 0x646, 0x20,
0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626,
0x645, 0x633, 0x6cc, 0x646, 0x679, 0x20, 0x67e, 0x6cc, 0x626, 0x631, 0x20, 0x622,
0x6ba, 0x20, 0x645, 0x6a9, 0x644, 0x6cc, 0x626, 0x648, 0x646, 0x20, 0x688, 0x6d2,
0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e, 0x6cc,
0x627, 0x646, 0x6af, 0x20, 0x6cc, 0x627, 0x646, 0x6af, 0x20, 0x648, 0x64e, 0x62e,
0x631, 0x648, 0x62a, 0x6be, 0x6cc, 0x631, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648,
0x64e, 0x62e, 0x633, 0x648, 0x631, 0x6cc, 0x646, 0x627, 0x645, 0x20, 0x633, 0x6cc,
0x20, 0x648, 0x64e, 0x62e, 0x62a, 0x627, 0x62c, 0x6a9, 0x633, 0x62a, 0x627, 0x646,
0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x62a, 0x631, 0x6a9, 0x645, 0x627,
0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e,
0x62a, 0x631, 0x6a9, 0x645, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633,
0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e,
0x62a, 0x631, 0x6a9, 0x645, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633,
0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20,
0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x6cc, 0x648, 0x631, 0x648, 0x6af, 0x648,
0x626, 0x6d2, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x6cc, 0x648, 0x631,
0x648, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc,
0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x6cc, 0x648, 0x631, 0x648, 0x6af,
0x648, 0x626, 0x6d2, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20,
0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x627,
0x632, 0x628, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20, 0x648,
0x64e, 0x62e, 0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633,
0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e,
0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20,
0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc,
0x20, 0x648, 0x64e, 0x62e, 0x648, 0x6cc, 0x646, 0x632, 0x648, 0x626, 0x6cc, 0x644,
0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x648, 0x648, 0x633, 0x679,
0x627, 0x6a9, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x41, 0x66, 0x67,
0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x47, 0x61, 0x72, 0x269, 0x256,
0x6f, 0x6e, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x67, 0x269, 0x63, 0x269, 0x269,
0x63, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x47, 0x61, 0x72, 0x269, 0x256, 0x6f, 0x6e, 0x74, 0x1dd, 0x6e,
0x61, 0x20, 0x67, 0x61, 0x6a, 0x61, 0x6b, 0x61, 0x6c, 0x61, 0x14b, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x47,
0x61, 0x72, 0x269, 0x256, 0x6f, 0x6e, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x67,
0x28a, 0x6e, 0x79, 0x269, 0x73, 0x6f, 0x6e, 0x6f, 0x6c, 0x61, 0x14b, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x47,
0x61, 0x72, 0x269, 0x256, 0x6f, 0x6e, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x67,
0x269, 0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x47, 0x61, 0x72, 0x269, 0x256, 0x6f, 0x6e, 0x74, 0x1dd, 0x6e, 0x61, 0x20,
0x67, 0x269, 0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61,
0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x47, 0x61, 0x72, 0x269,
0x256, 0x6f, 0x6e, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x67, 0x269, 0x74, 0x65,
0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61,
0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x61, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69,
0x256, 0x65, 0x69, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x61, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67,
0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x41, 0x6d, 0x61, 0x73, 0x254,
0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x41, 0x6d, 0x61, 0x73, 0x254, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256,
0x65, 0x69, 0x41, 0x6d, 0x61, 0x73, 0x254, 0x6e, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66,
0x28a, 0x62, 0x61, 0x6b, 0x61, 0x41, 0x6d, 0x61, 0x6c, 0x269, 0x6b, 0x61,
0x20, 0x67, 0x28a, 0x6e, 0x79, 0x269, 0x70, 0x25b, 0x6e, 0x25b, 0x6c, 0x61,
0x14b, 0x20, 0x6b, 0x61, 0x67, 0x269, 0x63, 0x269, 0x269, 0x63, 0x61, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x41,
0x6d, 0x61, 0x6c, 0x269, 0x6b, 0x61, 0x20, 0x67, 0x28a, 0x6e, 0x79, 0x269,
0x70, 0x25b, 0x6e, 0x25b, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x67, 0x269,
0x63, 0x269, 0x269, 0x63, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69,
0x41, 0x6d, 0x61, 0x6c, 0x269, 0x6b, 0x61, 0x20, 0x67, 0x28a, 0x6e, 0x79,
0x269, 0x70, 0x25b, 0x6e, 0x25b, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x67,
0x269, 0x63, 0x269, 0x269, 0x63, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62,
0x61, 0x6b, 0x61, 0x41, 0x6d, 0x61, 0x6c, 0x269, 0x6b, 0x61, 0x20, 0x67,
0x28a, 0x6e, 0x79, 0x269, 0x70, 0x25b, 0x6e, 0x25b, 0x6c, 0x61, 0x14b, 0x20,
0x6b, 0x61, 0x61, 0x6a, 0x61, 0x6b, 0x61, 0x6c, 0x61, 0x14b, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x41, 0x6d,
0x61, 0x6c, 0x269, 0x6b, 0x61, 0x20, 0x67, 0x28a, 0x6e, 0x79, 0x269, 0x70,
0x25b, 0x6e, 0x25b, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6a, 0x61,
0x6b, 0x61, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69,
0x41, 0x6d, 0x61, 0x6c, 0x269, 0x6b, 0x61, 0x20, 0x67, 0x28a, 0x6e, 0x79,
0x269, 0x70, 0x25b, 0x6e, 0x25b, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61,
0x6a, 0x61, 0x6b, 0x61, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a,
0x62, 0x61, 0x6b, 0x61, 0x41, 0x6d, 0x61, 0x6c, 0x269, 0x6b, 0x61, 0x20,
0x67, 0x28a, 0x6e, 0x79, 0x269, 0x70, 0x25b, 0x6e, 0x25b, 0x6c, 0x61, 0x14b,
0x20, 0x6b, 0x61, 0x62, 0x28a, 0x6e, 0x28a, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x41, 0x6d, 0x61, 0x6c, 0x269,
0x6b, 0x61, 0x20, 0x67, 0x28a, 0x6e, 0x79, 0x269, 0x70, 0x25b, 0x6e, 0x25b,
0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x62, 0x28a, 0x6e, 0x28a, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256,
0x65, 0x69, 0x256, 0x65, 0x69, 0x41, 0x6d, 0x61, 0x6c, 0x269, 0x6b, 0x61,
0x20, 0x67, 0x28a, 0x6e, 0x79, 0x269, 0x70, 0x25b, 0x6e, 0x25b, 0x6c, 0x61,
0x14b, 0x20, 0x6b, 0x61, 0x62, 0x28a, 0x6e, 0x28a, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66,
0x28a, 0x62, 0x61, 0x6b, 0x61, 0x41, 0x6d, 0x61, 0x6c, 0x269, 0x6b, 0x61,
0x20, 0x6b, 0x61, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x41,
0x6d, 0x61, 0x6c, 0x269, 0x6b, 0x61, 0x20, 0x6b, 0x61, 0x70, 0x61, 0x73,
0x69, 0x66, 0x69, 0x6b, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69,
0x41, 0x6d, 0x61, 0x6c, 0x269, 0x6b, 0x61, 0x20, 0x6b, 0x61, 0x70, 0x61,
0x73, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62,
0x61, 0x6b, 0x61, 0x41, 0x70, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x41, 0x70, 0x69, 0x79,
0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x41, 0x70, 0x69, 0x79,
0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x47, 0x61,
0x6c, 0x61, 0x61, 0x72, 0x69, 0x62, 0x75, 0x74, 0x1dd, 0x6e, 0x61, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x47,
0x61, 0x6c, 0x61, 0x61, 0x72, 0x69, 0x62, 0x75, 0x74, 0x1dd, 0x6e, 0x61,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x47, 0x61, 0x6c, 0x61, 0x61,
0x72, 0x69, 0x62, 0x75, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66,
0x28a, 0x62, 0x61, 0x6b, 0x61, 0x41, 0x72, 0x6a, 0x61, 0x6e, 0x74, 0x69,
0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x41, 0x72, 0x6a, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65,
0x69, 0x256, 0x65, 0x69, 0x41, 0x72, 0x6a, 0x61, 0x6e, 0x74, 0x69, 0x6e,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x41, 0x72, 0x6a,
0x61, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0x67, 0x269, 0x74, 0x65, 0x14b, 0x73,
0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x41, 0x72, 0x6a, 0x61, 0x6e,
0x74, 0x69, 0x6e, 0x20, 0x67, 0x269, 0x74, 0x65, 0x14b, 0x73, 0x68, 0x69,
0x6c, 0x65, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69,
0x41, 0x72, 0x6a, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0x67, 0x269, 0x74,
0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67,
0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x41, 0x72, 0x6d, 0x65, 0x6e,
0x69, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x69, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65,
0x69, 0x256, 0x65, 0x69, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x69, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x41, 0x6d, 0x61, 0x6c,
0x269, 0x6b, 0x61, 0x20, 0x6b, 0x61, 0x74, 0x1dd, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x6b, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x41, 0x6d, 0x61, 0x6c, 0x269, 0x6b, 0x61, 0x20, 0x6b,
0x61, 0x74, 0x1dd, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x61, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256,
0x65, 0x69, 0x256, 0x65, 0x69, 0x41, 0x6d, 0x61, 0x6c, 0x269, 0x6b, 0x61,
0x20, 0x6b, 0x61, 0x74, 0x1dd, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x61,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x186, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x67, 0x269, 0x63,
0x269, 0x269, 0x63, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x186, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79,
0x61, 0x20, 0x6b, 0x61, 0x67, 0x269, 0x63, 0x269, 0x269, 0x63, 0x61, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x186, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x67, 0x269, 0x63, 0x269, 0x269, 0x63,
0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x186, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x67, 0x269,
0x63, 0x269, 0x269, 0x63, 0x61, 0x20, 0x67, 0x269, 0x74, 0x65, 0x14b, 0x73,
0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x186, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x67, 0x269, 0x63, 0x269, 0x269,
0x63, 0x61, 0x20, 0x67, 0x269, 0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c,
0x65, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x186,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x67,
0x269, 0x63, 0x269, 0x269, 0x63, 0x61, 0x20, 0x67, 0x269, 0x74, 0x65, 0x14b,
0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66,
0x28a, 0x62, 0x61, 0x6b, 0x61, 0x186, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x79, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6a, 0x61, 0x6b, 0x61, 0x6c, 0x61,
0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x186, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x6b,
0x61, 0x61, 0x6a, 0x61, 0x6b, 0x61, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65,
0x69, 0x256, 0x65, 0x69, 0x186, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79,
0x61, 0x20, 0x6b, 0x61, 0x61, 0x6a, 0x61, 0x6b, 0x61, 0x6c, 0x61, 0x14b,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x186, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x67, 0x269, 0x74,
0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x186, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x67, 0x269,
0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x186, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x67, 0x269, 0x74, 0x65, 0x14b, 0x73,
0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a,
0x62, 0x61, 0x6b, 0x61, 0x41, 0x73, 0x25b, 0x72, 0x62, 0x61, 0x269, 0x6a,
0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x41, 0x73, 0x25b, 0x72, 0x62, 0x61, 0x269, 0x6a, 0x61, 0x14b,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x41, 0x73, 0x25b, 0x72, 0x62,
0x61, 0x269, 0x6a, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61,
0x6b, 0x61, 0x41, 0x73, 0x254, 0x254, 0x72, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x41, 0x73, 0x254, 0x254, 0x72,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x41, 0x73, 0x254, 0x254, 0x72,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x42, 0x61, 0x14b,
0x67, 0x6c, 0x61, 0x256, 0x25b, 0x25b, 0x73, 0x68, 0x269, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x42, 0x61, 0x14b,
0x67, 0x6c, 0x61, 0x256, 0x25b, 0x25b, 0x73, 0x68, 0x269, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65,
0x69, 0x256, 0x65, 0x69, 0x42, 0x61, 0x14b, 0x67, 0x6c, 0x61, 0x256, 0x25b,
0x25b, 0x73, 0x68, 0x269, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b,
0x61, 0x42, 0x75, 0x74, 0x61, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x42, 0x6f, 0x6c, 0x69, 0x66, 0x69,
0x79, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x79, 0x61, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x42,
0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69,
0x256, 0x65, 0x69, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x79, 0x61,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x42, 0x72, 0x75,
0x6e, 0x25b, 0x269, 0x20, 0x189, 0x61, 0x72, 0x75, 0x73, 0x61, 0x6c, 0x61,
0x61, 0x6d, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x4b, 0x61, 0x70, 0x66, 0x25b, 0x25b, 0x72, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4b, 0x61, 0x70,
0x66, 0x25b, 0x25b, 0x72, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x4b,
0x61, 0x70, 0x66, 0x25b, 0x25b, 0x72, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62,
0x61, 0x6b, 0x61, 0x53, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x6f, 0x6f, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x53,
0x68, 0x61, 0x74, 0x61, 0x6d, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x53, 0x68, 0x61, 0x74, 0x61, 0x6d, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x53, 0x68, 0x61, 0x74, 0x61, 0x6d,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x53, 0x68, 0x69,
0x6c, 0x69, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x53, 0x68, 0x69, 0x6c, 0x69, 0x69, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65,
0x69, 0x256, 0x65, 0x69, 0x53, 0x68, 0x69, 0x6c, 0x69, 0x69, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67,
0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x43, 0x61, 0x269, 0x6e, 0x61,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x43, 0x61, 0x269, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69,
0x43, 0x61, 0x269, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61,
0x6b, 0x61, 0x4e, 0x6f, 0x77, 0x25b, 0x6c, 0x20, 0x6b, 0x61, 0x41, 0x74,
0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4b, 0x6f, 0x6b, 0x6f,
0x6f, 0x73, 0x20, 0x6b, 0x61, 0x42, 0x28a, 0x74, 0x75, 0x6b, 0x1dd, 0x6c,
0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x4b, 0x6f, 0x6c, 0x254, 0x6d, 0x62, 0x69, 0x69,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x4b, 0x6f, 0x6c, 0x254, 0x6d, 0x62, 0x69, 0x69, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69,
0x256, 0x65, 0x69, 0x4b, 0x6f, 0x6c, 0x254, 0x6d, 0x62, 0x69, 0x69, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4b, 0x28a, 0x6b, 0x28a,
0x20, 0x6b, 0x61, 0x42, 0x28a, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd,
0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x4b, 0x28a, 0x6b, 0x28a, 0x20, 0x6b, 0x61, 0x42, 0x28a, 0x74,
0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69,
0x256, 0x65, 0x69, 0x4b, 0x28a, 0x6b, 0x28a, 0x20, 0x6b, 0x61, 0x42, 0x28a,
0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61,
0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4b, 0x75, 0x62, 0x61, 0x61, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4b,
0x75, 0x62, 0x61, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x4b,
0x75, 0x62, 0x61, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b,
0x61, 0x189, 0x65, 0x66, 0x69, 0x73, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x189, 0x69, 0x6d, 0x254, 0x6e, 0x20,
0x189, 0x69, 0x72, 0x66, 0x69, 0x6c, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x54, 0x69, 0x6d, 0x254, 0x254, 0x20,
0x67, 0x61, 0x6a, 0x61, 0x6b, 0x61, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x50, 0x61, 0x61,
0x6b, 0x269, 0x20, 0x6b, 0x61, 0x41, 0x61, 0x74, 0x75, 0x6b, 0x1dd, 0x6c,
0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x50, 0x61, 0x61, 0x6b, 0x269, 0x20, 0x6b, 0x61,
0x41, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256,
0x65, 0x69, 0x256, 0x65, 0x69, 0x50, 0x61, 0x61, 0x6b, 0x269, 0x20, 0x6b,
0x61, 0x41, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x45, 0x6b, 0x75, 0x77,
0x61, 0x256, 0x254, 0x254, 0x72, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x47, 0x61, 0x72, 0x269, 0x66, 0x254, 0x6e,
0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x67, 0x269, 0x63, 0x269, 0x269, 0x63, 0x61,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x47, 0x61, 0x72, 0x269, 0x66, 0x254, 0x6e, 0x74, 0x1dd, 0x6e, 0x61, 0x20,
0x67, 0x269, 0x63, 0x269, 0x269, 0x63, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256,
0x65, 0x69, 0x47, 0x61, 0x72, 0x269, 0x66, 0x254, 0x6e, 0x74, 0x1dd, 0x6e,
0x61, 0x20, 0x67, 0x269, 0x63, 0x269, 0x269, 0x63, 0x61, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61,
0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x47, 0x61, 0x72, 0x269, 0x66, 0x254,
0x6e, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x67, 0x61, 0x6a, 0x61, 0x6b, 0x61,
0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x47, 0x61, 0x72, 0x269, 0x66, 0x254, 0x6e, 0x74, 0x1dd,
0x6e, 0x61, 0x20, 0x67, 0x61, 0x6a, 0x61, 0x6b, 0x61, 0x6c, 0x61, 0x14b,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x47, 0x61, 0x72, 0x269, 0x66,
0x254, 0x6e, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x67, 0x61, 0x6a, 0x61, 0x6b,
0x61, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b,
0x61, 0x47, 0x61, 0x72, 0x269, 0x66, 0x254, 0x6e, 0x74, 0x1dd, 0x6e, 0x61,
0x20, 0x67, 0x61, 0x6a, 0x61, 0x6b, 0x61, 0x6c, 0x61, 0x14b, 0x20, 0x6b,
0x61, 0x61, 0x6a, 0x61, 0x6b, 0x61, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x47, 0x61, 0x72,
0x269, 0x66, 0x254, 0x6e, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x67, 0x269, 0x74,
0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x47, 0x61,
0x72, 0x269, 0x66, 0x254, 0x6e, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x67, 0x269,
0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x47, 0x61, 0x72, 0x269, 0x66, 0x254,
0x6e, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x67, 0x269, 0x74, 0x65, 0x14b, 0x73,
0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a,
0x62, 0x61, 0x6b, 0x61, 0x46, 0x254, 0x6b, 0x6c, 0x61, 0x6e, 0x256, 0x20,
0x6b, 0x61, 0x42, 0x28a, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e,
0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x46, 0x254, 0x6b, 0x6c, 0x61, 0x6e, 0x256, 0x20, 0x6b, 0x61, 0x42,
0x28a, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256,
0x65, 0x69, 0x256, 0x65, 0x69, 0x46, 0x254, 0x6b, 0x6c, 0x61, 0x6e, 0x256,
0x20, 0x6b, 0x61, 0x42, 0x28a, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd,
0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x46,
0x69, 0x6a, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256,
0x65, 0x69, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62,
0x61, 0x6b, 0x61, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x61, 0x20, 0x47,
0x61, 0x66, 0x254, 0x6e, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x6a,
0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x47, 0x61, 0x66, 0x254, 0x6e, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x67,
0x28a, 0x6e, 0x79, 0x269, 0x73, 0x6f, 0x6e, 0x6f, 0x6c, 0x61, 0x14b, 0x20,
0x6e, 0x61, 0x20, 0x47, 0x61, 0x74, 0x75, 0x74, 0x61, 0x6c, 0x74, 0x1dd,
0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x254, 0x73, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x47,
0x61, 0x6d, 0x62, 0x69, 0x79, 0x65, 0x65, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4a, 0x254, 0x72, 0x6a, 0x69,
0x79, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x4a, 0x254, 0x72, 0x6a, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65,
0x69, 0x256, 0x65, 0x69, 0x4a, 0x254, 0x72, 0x6a, 0x69, 0x79, 0x61, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4a, 0x69, 0x6c, 0x62,
0x25b, 0x25b, 0x72, 0x20, 0x6b, 0x61, 0x42, 0x28a, 0x74, 0x75, 0x6b, 0x1dd,
0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x47, 0x72, 0x69, 0x69, 0x6e, 0x77, 0x69,
0x73, 0x68, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x256, 0x20, 0x67,
0x61, 0x6a, 0x61, 0x6b, 0x61, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x47, 0x72, 0x69, 0x6e,
0x6c, 0x61, 0x6e, 0x256, 0x20, 0x67, 0x61, 0x6a, 0x61, 0x6b, 0x61, 0x6c,
0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x47, 0x72, 0x69,
0x6e, 0x6c, 0x61, 0x6e, 0x256, 0x20, 0x67, 0x61, 0x6a, 0x61, 0x6b, 0x61,
0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61,
0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x256, 0x20, 0x67, 0x269, 0x74,
0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x47, 0x72,
0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x256, 0x20, 0x67, 0x269, 0x74, 0x65, 0x14b,
0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69,
0x256, 0x65, 0x69, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x256, 0x20,
0x67, 0x269, 0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61,
0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x47, 0x75,
0x77, 0x61, 0x6d, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x47, 0x254, 0x6c, 0x66, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x47, 0x75, 0x79, 0x61, 0x6e,
0x61, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x41, 0x77, 0x61, 0x79, 0x269, 0x269, 0x20, 0x6e, 0x2019, 0x41,
0x6c, 0x65, 0x77, 0x75, 0x74, 0x69, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x41, 0x77, 0x61, 0x79, 0x269,
0x269, 0x20, 0x6e, 0x2019, 0x41, 0x6c, 0x65, 0x77, 0x75, 0x74, 0x69, 0x69,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x41, 0x77, 0x61, 0x79, 0x269,
0x269, 0x20, 0x6e, 0x2019, 0x41, 0x6c, 0x65, 0x77, 0x75, 0x74, 0x69, 0x69,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x48, 0x254, 0x14b,
0x20, 0x4b, 0x254, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x48, 0x254, 0x14b, 0x20, 0x4b, 0x254, 0x14b, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x48, 0x254, 0x14b, 0x20, 0x4b, 0x254,
0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x48, 0x254,
0x66, 0x256, 0x1dd, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x48, 0x254, 0x66, 0x256, 0x1dd, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69,
0x256, 0x65, 0x69, 0x48, 0x254, 0x66, 0x256, 0x1dd, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66,
0x28a, 0x62, 0x61, 0x6b, 0x61, 0x49, 0x6e, 0x256, 0x69, 0x79, 0x61, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x49,
0x6e, 0x256, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x74, 0x65, 0x14b, 0x6b,
0x75, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x49, 0x6e, 0x256, 0x69, 0x63, 0x61, 0x269, 0x6e, 0x61, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x190, 0x6e,
0x256, 0x6f, 0x6e, 0x6f, 0x73, 0x69, 0x69, 0x20, 0x6b, 0x61, 0x67, 0x269,
0x63, 0x269, 0x269, 0x63, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x190, 0x6e, 0x256, 0x6f, 0x6e, 0x6f, 0x73,
0x69, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6a, 0x61, 0x6b, 0x61, 0x6c, 0x61,
0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x190, 0x6e, 0x256, 0x6f, 0x6e, 0x6f, 0x73, 0x69, 0x69, 0x20, 0x6b,
0x61, 0x67, 0x269, 0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c,
0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x49, 0x72, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x49, 0x72, 0x61, 0x14b, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256,
0x65, 0x69, 0x256, 0x65, 0x69, 0x49, 0x72, 0x61, 0x14b, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61,
0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x49,
0x72, 0x6b, 0x75, 0x74, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x49,
0x72, 0x6b, 0x75, 0x74, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b,
0x61, 0x59, 0x69, 0x73, 0x68, 0x72, 0x61, 0x25b, 0x6c, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x59, 0x69, 0x73,
0x68, 0x72, 0x61, 0x25b, 0x6c, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69,
0x59, 0x69, 0x73, 0x68, 0x72, 0x61, 0x25b, 0x6c, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66,
0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4a, 0x61, 0x70, 0x61, 0x14b, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4a, 0x61,
0x70, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x4a, 0x61,
0x70, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61,
0x4b, 0x61, 0x73, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4b, 0x61, 0x73, 0x61,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x67, 0x61, 0x6a, 0x61, 0x6b, 0x61, 0x6c,
0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x4b, 0x61, 0x73, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x67,
0x269, 0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x4b, 0x6f, 0x72, 0x65, 0x65, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4b, 0x6f, 0x72, 0x65, 0x65, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256,
0x65, 0x69, 0x256, 0x65, 0x69, 0x4b, 0x6f, 0x72, 0x65, 0x65, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67,
0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4b, 0x254, 0x73, 0x72, 0x25b,
0x25b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x6b, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x6b, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65,
0x69, 0x256, 0x65, 0x69, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x6b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4b,
0x69, 0x72, 0x67, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x53, 0x69, 0x72, 0x69,
0x20, 0x4c, 0x61, 0x14b, 0x6b, 0x61, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4c, 0x61, 0x269, 0x6e, 0x20,
0x6b, 0x61, 0x42, 0x28a, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e,
0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x4c, 0x254, 0x72, 0x256, 0x20, 0x48, 0x6f, 0x6f, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4c, 0x254, 0x72,
0x256, 0x20, 0x48, 0x6f, 0x6f, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69,
0x4c, 0x254, 0x72, 0x256, 0x20, 0x48, 0x6f, 0x6f, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66,
0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4d, 0x61, 0x6b, 0x61, 0x77, 0x6f, 0x6f,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x4d, 0x61, 0x6b, 0x61, 0x77, 0x6f, 0x6f, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256,
0x65, 0x69, 0x61, 0x6b, 0x61, 0x77, 0x6f, 0x6f, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66,
0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4d, 0x61, 0x67, 0x61, 0x256, 0x61, 0x6e,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x4d, 0x61, 0x67, 0x61, 0x256, 0x61, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256,
0x65, 0x69, 0x4d, 0x61, 0x67, 0x61, 0x256, 0x61, 0x6e, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61,
0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4d, 0x61, 0x6c, 0x25b, 0x73, 0x69,
0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x4d, 0x61, 0x6c, 0x256, 0x69, 0x69, 0x66, 0x75, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4d, 0x61, 0x72,
0x6b, 0x65, 0x73, 0x61, 0x73, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c,
0x20, 0x6b, 0x61, 0x42, 0x28a, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd,
0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x49, 0x6d, 0x6f, 0x72, 0x69, 0x73, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x49, 0x6d, 0x6f, 0x72,
0x69, 0x73, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x49, 0x6d, 0x6f,
0x72, 0x69, 0x73, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61,
0x4d, 0x254, 0x73, 0x254, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4d, 0x25b, 0x73, 0x69, 0x6b, 0x20, 0x6b,
0x61, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4d, 0x25b, 0x73,
0x69, 0x6b, 0x20, 0x6b, 0x61, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b,
0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x4d, 0x25b, 0x73, 0x69,
0x6b, 0x20, 0x6b, 0x61, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x61,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x55, 0x6c, 0x61,
0x6e, 0x62, 0x61, 0x74, 0x254, 0x254, 0x72, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x55, 0x6c, 0x61, 0x6e, 0x62,
0x61, 0x74, 0x254, 0x254, 0x72, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69,
0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x254, 0x254, 0x72, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67,
0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4d, 0x6f, 0x73, 0x6b, 0x75,
0x75, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x4d, 0x6f, 0x73, 0x6b, 0x75, 0x75, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256,
0x65, 0x69, 0x4d, 0x6f, 0x73, 0x6b, 0x75, 0x75, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66,
0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4d, 0x69, 0x79, 0x61, 0x6e, 0x6d, 0x61,
0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x4e, 0x61, 0x77, 0x75, 0x72, 0x75, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4e, 0x65, 0x65, 0x70, 0x61,
0x6c, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x4b, 0x61, 0x6c, 0x65, 0x256, 0x6f, 0x6e, 0x69, 0x69, 0x20, 0x61,
0x66, 0x254, 0x6c, 0x269, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x4b, 0x61, 0x6c, 0x65, 0x256, 0x6f, 0x6e, 0x69,
0x69, 0x20, 0x61, 0x66, 0x254, 0x6c, 0x269, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256,
0x65, 0x69, 0x4b, 0x61, 0x6c, 0x65, 0x256, 0x6f, 0x6e, 0x69, 0x69, 0x20,
0x61, 0x66, 0x254, 0x6c, 0x269, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61,
0x6b, 0x61, 0x53, 0x65, 0x6c, 0x61, 0x6e, 0x256, 0x20, 0x61, 0x66, 0x254,
0x6c, 0x269, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x53, 0x65, 0x6c, 0x61, 0x6e, 0x256, 0x20, 0x61, 0x66, 0x254,
0x6c, 0x269, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x53, 0x65, 0x6c,
0x61, 0x6e, 0x256, 0x20, 0x61, 0x66, 0x254, 0x6c, 0x269, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61,
0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x46, 0x61, 0x28a, 0x6e, 0x256, 0x6c,
0x61, 0x6e, 0x256, 0x20, 0x61, 0x66, 0x254, 0x6c, 0x269, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x46, 0x61, 0x28a,
0x6e, 0x256, 0x6c, 0x61, 0x6e, 0x256, 0x20, 0x61, 0x66, 0x254, 0x6c, 0x269,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x46, 0x61, 0x28a, 0x6e, 0x256,
0x6c, 0x61, 0x6e, 0x256, 0x20, 0x61, 0x66, 0x254, 0x6c, 0x269, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67,
0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4e, 0x69, 0x77, 0x75, 0x79,
0x65, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x4e, 0x254, 0x72, 0x66, 0x6f, 0x6f, 0x6b, 0x20, 0x6b, 0x61, 0x41,
0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4e, 0x254, 0x72,
0x66, 0x6f, 0x6f, 0x6b, 0x20, 0x6b, 0x61, 0x41, 0x74, 0x75, 0x6b, 0x1dd,
0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69,
0x4e, 0x254, 0x72, 0x66, 0x6f, 0x6f, 0x6b, 0x20, 0x6b, 0x61, 0x41, 0x74,
0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66,
0x28a, 0x62, 0x61, 0x6b, 0x61, 0x46, 0x25b, 0x72, 0x6e, 0x61, 0x6e, 0x256,
0x6f, 0x20, 0x256, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x254, 0x6e, 0x79, 0x61,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x46, 0x25b, 0x72, 0x6e, 0x61, 0x6e, 0x256, 0x6f, 0x20, 0x256, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x254, 0x6e, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256,
0x65, 0x69, 0x46, 0x25b, 0x72, 0x6e, 0x61, 0x6e, 0x256, 0x6f, 0x20, 0x256,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x254, 0x6e, 0x79, 0x61, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61,
0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4d, 0x61, 0x72, 0x69, 0x79, 0x61,
0x6e, 0x20, 0x6b, 0x1dd, 0x62, 0x28a, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74,
0x1dd, 0x6e, 0x61, 0x20, 0x67, 0x28a, 0x6e, 0x79, 0x269, 0x70, 0x25b, 0x6e,
0x25b, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x4e, 0x6f, 0x66, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x6b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x4e, 0x6f, 0x66, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x6b,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x4e, 0x6f, 0x66, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x6b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61,
0x6b, 0x61, 0x186, 0x6d, 0x73, 0x1dd, 0x6b, 0x1dd, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x186, 0x6d, 0x73, 0x1dd,
0x6b, 0x1dd, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x186, 0x6d, 0x73,
0x1dd, 0x6b, 0x1dd, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x50, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a,
0x62, 0x61, 0x6b, 0x61, 0x50, 0x61, 0x6c, 0x61, 0x77, 0x6f, 0x6f, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x50,
0x61, 0x70, 0x75, 0x61, 0x73, 0x69, 0x69, 0x20, 0x47, 0x69, 0x6e, 0x65,
0x65, 0x20, 0x61, 0x66, 0x254, 0x6c, 0x269, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x77, 0x65, 0x65, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x77, 0x65,
0x65, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x77, 0x65, 0x65, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61,
0x6b, 0x61, 0x50, 0x65, 0x72, 0x75, 0x75, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x50, 0x65, 0x72, 0x75, 0x75,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x50, 0x65, 0x72, 0x75, 0x75,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x46, 0x69, 0x6c,
0x69, 0x70, 0x69, 0x69, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x69,
0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x69, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b,
0x61, 0x46, 0x6f, 0x65, 0x6e, 0x69, 0x69, 0x73, 0x20, 0x6b, 0x61, 0x42,
0x28a, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x53, 0x25b,
0x14b, 0x2d, 0x50, 0x65, 0x74, 0x72, 0x254, 0x73, 0x20, 0x6e, 0x61, 0x20,
0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x254, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x53, 0x25b, 0x14b, 0x2d, 0x50,
0x65, 0x74, 0x72, 0x254, 0x73, 0x20, 0x6e, 0x61, 0x20, 0x4d, 0x69, 0x6b,
0x65, 0x6c, 0x254, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x53,
0x25b, 0x14b, 0x2d, 0x50, 0x65, 0x74, 0x72, 0x254, 0x73, 0x20, 0x6e, 0x61,
0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x254, 0x14b, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66,
0x28a, 0x62, 0x61, 0x6b, 0x61, 0x50, 0x269, 0x74, 0x6b, 0x25b, 0x25b, 0x6e,
0x20, 0x6b, 0x61, 0x42, 0x28a, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd,
0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x65, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x50, 0x269, 0x79,
0x254, 0x14b, 0x79, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x52, 0x65, 0x65, 0x6e, 0x69, 0x79, 0x254,
0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x52, 0x6f, 0x74, 0x65, 0x72, 0x61, 0x61, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x53, 0x61, 0x6b, 0x61,
0x6c, 0x69, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x53, 0x61, 0x6b, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256,
0x65, 0x69, 0x256, 0x65, 0x69, 0x53, 0x61, 0x6b, 0x61, 0x6c, 0x69, 0x6e,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x53, 0x61, 0x6d,
0x6f, 0x77, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x53, 0x61, 0x6d, 0x6f, 0x77, 0x61, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65,
0x69, 0x256, 0x65, 0x69, 0x53, 0x61, 0x6d, 0x6f, 0x77, 0x61, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67,
0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x53, 0x65, 0x73, 0x68, 0x25b,
0x6c, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x53, 0x69, 0x14b, 0x67, 0x61, 0x70, 0x75, 0x75, 0x72, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x53, 0x61,
0x6c, 0x6f, 0x6d, 0x254, 0x254, 0x6e, 0x20, 0x6b, 0x61, 0x42, 0x28a, 0x74,
0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4a, 0x254, 0x72, 0x6a,
0x69, 0x79, 0x61, 0x20, 0x67, 0x28a, 0x6e, 0x79, 0x269, 0x73, 0x6f, 0x6e,
0x6f, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x53, 0x69, 0x79, 0x6f,
0x77, 0x61, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x69, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x54, 0x61,
0x269, 0x70, 0x65, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x54, 0x61, 0x269, 0x70, 0x65, 0x69, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256,
0x65, 0x69, 0x256, 0x65, 0x69, 0x54, 0x61, 0x269, 0x70, 0x65, 0x69, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x54, 0x61, 0x6a, 0x69,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61,
0x28a, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x54, 0x254, 0x14b, 0x67, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x54, 0x254, 0x14b, 0x67, 0x61, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x54, 0x254, 0x14b, 0x67, 0x61, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x43, 0x75, 0x75, 0x6b,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x54, 0x75, 0x72, 0x6b, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a,
0x62, 0x61, 0x6b, 0x61, 0x54, 0x75, 0x66, 0x61, 0x6c, 0x75, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x55, 0x72,
0x75, 0x67, 0x75, 0x77, 0x65, 0x65, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x55, 0x72, 0x75, 0x67, 0x75, 0x77,
0x65, 0x65, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x55, 0x72, 0x75,
0x67, 0x75, 0x77, 0x65, 0x65, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61,
0x6b, 0x61, 0x55, 0x73, 0x62, 0x65, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x55, 0x73, 0x62, 0x65, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x55, 0x73, 0x62, 0x65, 0x65,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62,
0x61, 0x6b, 0x61, 0x46, 0x61, 0x6e, 0x75, 0x77, 0x61, 0x74, 0x75, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x46,
0x61, 0x6e, 0x75, 0x77, 0x61, 0x74, 0x75, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256,
0x65, 0x69, 0x46, 0x61, 0x6e, 0x75, 0x77, 0x61, 0x74, 0x75, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67,
0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x46, 0x65, 0x6e, 0x65, 0x73,
0x75, 0x77, 0x65, 0x6c, 0x61, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x46, 0x6c, 0x61, 0x256, 0x69, 0x66,
0x254, 0x73, 0x74, 0x254, 0x6b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x46, 0x6c, 0x61, 0x256, 0x69, 0x66, 0x254,
0x73, 0x74, 0x254, 0x6b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x46,
0x6c, 0x61, 0x256, 0x69, 0x66, 0x254, 0x73, 0x74, 0x254, 0x6b, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67,
0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x46, 0x254, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x61, 0x256, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x46, 0x254, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x61, 0x256, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x46, 0x254,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x61, 0x256, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66,
0x28a, 0x62, 0x61, 0x6b, 0x61, 0x46, 0x254, 0x73, 0x74, 0x254, 0x6b, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x57,
0x65, 0x65, 0x6b, 0x20, 0x6b, 0x61, 0x42, 0x28a, 0x74, 0x75, 0x6b, 0x1dd,
0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x57, 0x61, 0x6c, 0x69, 0x73, 0x20, 0x6e,
0x61, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x59, 0x61, 0x6b, 0x75,
0x74, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65,
0x69, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62,
0x61, 0x6b, 0x61, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x75, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x75, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x59, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x75, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67,
0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x59, 0x75, 0x6b, 0x254, 0x6e,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x905, 0x92b, 0x93c, 0x917, 0x93e, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928,
0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92e, 0x927, 0x94d, 0x92f,
0x20, 0x905, 0x92b, 0x93c, 0x94d, 0x930, 0x940, 0x915, 0x93e, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x948, 0x932, 0x947, 0x940, 0x20, 0x905, 0x92b,
0x93c, 0x94d, 0x930, 0x940, 0x915, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x926, 0x916, 0x923, 0x20, 0x905, 0x92b, 0x93c, 0x94d, 0x930, 0x940, 0x915,
0x93e, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948,
0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x905, 0x92b, 0x93c, 0x94d,
0x930, 0x940, 0x915, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a,
0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x905, 0x92b, 0x93c, 0x94d, 0x930, 0x940,
0x915, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x948, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x905, 0x92b, 0x93c,
0x94d, 0x930, 0x940, 0x915, 0x93e, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902,
0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x905,
0x932, 0x93e, 0x938, 0x94d, 0x915, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x905, 0x932, 0x93e, 0x938, 0x94d, 0x200d, 0x915, 0x93e, 0x20, 0x926, 0x93e,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x932, 0x93e,
0x938, 0x94d, 0x200d, 0x915, 0x93e, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f,
0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932,
0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x90f, 0x92e, 0x947, 0x91c,
0x93c, 0x94b, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x90f, 0x92e,
0x947, 0x91c, 0x93c, 0x94b, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x91f, 0x948, 0x92e, 0x90f, 0x92e, 0x947, 0x91c, 0x93c, 0x94b, 0x928,
0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940,
0x20, 0x905, 0x92e, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x915, 0x947, 0x902, 0x926,
0x94d, 0x930, 0x940, 0x92f, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x909,
0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x930, 0x93f, 0x915, 0x940,
0x20, 0x915, 0x947, 0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x926, 0x93e,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x909, 0x924, 0x94d,
0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x915,
0x947, 0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x926, 0x947, 0x20, 0x927,
0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c,
0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x909, 0x924,
0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x930, 0x93f, 0x915, 0x940, 0x20,
0x92a, 0x948, 0x932, 0x947, 0x940, 0x20, 0x91f, 0x948, 0x92e, 0x909, 0x924, 0x94d,
0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x92a,
0x948, 0x932, 0x947, 0x940, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x948, 0x92e, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905,
0x92e, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x92a, 0x948, 0x932, 0x947, 0x940, 0x20,
0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926,
0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x930,
0x93f, 0x915, 0x940, 0x20, 0x92a, 0x939, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20,
0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x92a, 0x939, 0x93e, 0x921,
0x93c, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x948, 0x92e, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x947,
0x930, 0x93f, 0x915, 0x940, 0x20, 0x92a, 0x939, 0x93e, 0x921, 0x93c, 0x947, 0x20,
0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926,
0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x947,
0x930, 0x93f, 0x915, 0x940, 0x20, 0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924,
0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x909, 0x924, 0x94d, 0x924,
0x930, 0x940, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x92a,
0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x909, 0x924, 0x94d, 0x924, 0x930,
0x940, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x92a, 0x94d,
0x930, 0x936, 0x93e, 0x902, 0x924, 0x947, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d,
0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e,
0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x90f, 0x92a, 0x93f,
0x906, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x90f, 0x92a, 0x93f, 0x906,
0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e,
0x90f, 0x92a, 0x93f, 0x906, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e,
0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x930, 0x92c, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x930, 0x92c, 0x20, 0x926, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x930, 0x92c, 0x20,
0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926,
0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x930, 0x94d, 0x91c, 0x947,
0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f,
0x940, 0x928, 0x93e, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f,
0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x936, 0x94d,
0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f,
0x940, 0x928, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x936,
0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902,
0x91f, 0x940, 0x928, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20,
0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20, 0x926,
0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x906, 0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f, 0x92f,
0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x906, 0x930, 0x94d, 0x92e,
0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x91f, 0x948, 0x92e, 0x906, 0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f,
0x92f, 0x93e, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f,
0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x91f, 0x932, 0x93e,
0x902, 0x91f, 0x93f, 0x915, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x905,
0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20, 0x926, 0x93e, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x91f, 0x932, 0x93e, 0x902,
0x91f, 0x93f, 0x915, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921,
0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92c, 0x93f, 0x91a, 0x932, 0x947, 0x20,
0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x938, 0x94d, 0x200d, 0x91f, 0x94d,
0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x947, 0x20, 0x915, 0x947,
0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x938, 0x94d, 0x200d, 0x91f, 0x94d,
0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x947, 0x20, 0x915, 0x947,
0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d,
0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e,
0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x938, 0x94d,
0x200d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x915,
0x947, 0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x92a, 0x936, 0x94d, 0x91a,
0x93f, 0x92e, 0x940, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x938,
0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x915,
0x947, 0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x92a, 0x936, 0x94d, 0x91a,
0x93f, 0x92e, 0x940, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x948, 0x92e, 0x905, 0x938, 0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947, 0x932,
0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x915, 0x947, 0x902, 0x926, 0x94d, 0x930, 0x940,
0x92f, 0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x927, 0x94d,
0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e,
0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x948, 0x932,
0x947, 0x940, 0x20, 0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f,
0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x938, 0x94d,
0x200d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x92a,
0x948, 0x932, 0x947, 0x940, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x948, 0x92e, 0x905, 0x938, 0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947,
0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x92a, 0x948, 0x932, 0x947, 0x940, 0x20,
0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926,
0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905, 0x938,
0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932,
0x93f, 0x92f, 0x93e, 0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20,
0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x905,
0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x92a,
0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d,
0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e,
0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x91c, 0x93c,
0x930, 0x92c, 0x948, 0x91c, 0x93e, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x905, 0x91c, 0x93c, 0x930, 0x92c, 0x948, 0x91c, 0x93e, 0x928, 0x20, 0x926,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x91c,
0x93c, 0x930, 0x92c, 0x948, 0x91c, 0x93e, 0x928, 0x20, 0x926, 0x940, 0x20, 0x924,
0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x905, 0x91c, 0x93c, 0x94b, 0x930, 0x947, 0x938, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x905, 0x91c, 0x93c, 0x94b, 0x930, 0x947, 0x938, 0x20, 0x926,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x91c,
0x93c, 0x94b, 0x930, 0x947, 0x938, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902,
0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92c,
0x93e, 0x902, 0x917, 0x94d, 0x932, 0x93e, 0x926, 0x947, 0x936, 0x93e, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92c, 0x93e, 0x902, 0x917, 0x94d, 0x932, 0x93e,
0x926, 0x947, 0x936, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x948, 0x92e, 0x92c, 0x93e, 0x902, 0x917, 0x94d, 0x932, 0x93e, 0x926,
0x947, 0x936, 0x947, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f,
0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92d, 0x942, 0x91f,
0x93e, 0x928, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92c, 0x94b,
0x932, 0x940, 0x935, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x93c, 0x940, 0x932, 0x93f, 0x92f, 0x93e,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92c, 0x94d, 0x930, 0x93e, 0x91c,
0x93c, 0x940, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x93c,
0x940, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902,
0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92c,
0x94d, 0x930, 0x942, 0x928, 0x947, 0x908, 0x20, 0x926, 0x93e, 0x930, 0x942, 0x938,
0x94d, 0x938, 0x932, 0x92e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x915,
0x947, 0x92a, 0x20, 0x935, 0x930, 0x94d, 0x921, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x930, 0x94d, 0x921, 0x20, 0x926,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x947,
0x92a, 0x20, 0x935, 0x930, 0x94d, 0x921, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b,
0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x91a, 0x93e, 0x92e, 0x94b, 0x930, 0x94b, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x91a, 0x948, 0x925, 0x92e, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x91a, 0x948, 0x925, 0x92e, 0x20, 0x926, 0x93e,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x91a, 0x948, 0x925,
0x92e, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947,
0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x91a, 0x93f, 0x932, 0x940, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x91a, 0x93f, 0x932, 0x940, 0x20, 0x926, 0x93e, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x91a, 0x93f, 0x932, 0x940, 0x20,
0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x91a, 0x940, 0x928, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x91a, 0x940, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x91a, 0x940, 0x928, 0x20, 0x926, 0x947,
0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20,
0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x915, 0x94d, 0x930, 0x93f, 0x938, 0x92e, 0x938, 0x20, 0x91f, 0x93e, 0x92a, 0x942,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x94b, 0x915, 0x94b, 0x938,
0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x906, 0x902, 0x20, 0x926, 0x947, 0x20, 0x91d,
0x941, 0x902, 0x921, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x915,
0x94b, 0x932, 0x902, 0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x915, 0x94b, 0x932, 0x902, 0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x926,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x94b,
0x932, 0x902, 0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b,
0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x915, 0x941, 0x915, 0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x906, 0x902, 0x20, 0x926,
0x947, 0x20, 0x91d, 0x941, 0x902, 0x921, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x915, 0x941, 0x915, 0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x906, 0x902,
0x20, 0x926, 0x947, 0x20, 0x91d, 0x941, 0x902, 0x921, 0x947, 0x20, 0x926, 0x93e,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x941, 0x915,
0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x906, 0x902, 0x20, 0x926, 0x947, 0x20, 0x91d,
0x941, 0x902, 0x921, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x905, 0x926, 0x94d, 0x927,
0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x20,
0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x915,
0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f,
0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932,
0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x921, 0x947, 0x935, 0x93f,
0x938, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x921, 0x94d, 0x92f, 0x942,
0x92e, 0x94b, 0x902, 0x91f, 0x20, 0x921, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x935,
0x93f, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x948,
0x932, 0x947, 0x940, 0x20, 0x924, 0x93f, 0x92e, 0x94b, 0x930, 0x947, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x908, 0x938, 0x94d, 0x91f, 0x930, 0x20, 0x91f,
0x93e, 0x92a, 0x942, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x908, 0x938,
0x94d, 0x91f, 0x930, 0x20, 0x91f, 0x93e, 0x92a, 0x942, 0x20, 0x926, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x908, 0x938, 0x94d, 0x91f,
0x930, 0x20, 0x91f, 0x93e, 0x92a, 0x942, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b,
0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x907, 0x915, 0x94d, 0x935, 0x93e, 0x921, 0x94b, 0x930, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x92c, 0x93f, 0x91a, 0x932, 0x947, 0x20, 0x92f, 0x942, 0x930,
0x94b, 0x92a, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92c, 0x93f,
0x91a, 0x932, 0x947, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x947, 0x20, 0x926,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92c, 0x93f,
0x91a, 0x932, 0x947, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x947, 0x20, 0x926,
0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x948, 0x932, 0x947, 0x940, 0x20, 0x92f, 0x942,
0x930, 0x94b, 0x92a, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a,
0x948, 0x932, 0x947, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x947, 0x20,
0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92a,
0x948, 0x932, 0x947, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x947, 0x20,
0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x917, 0x94d, 0x930, 0x20, 0x92a, 0x948,
0x932, 0x947, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x947, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940,
0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x92f, 0x942,
0x930, 0x94b, 0x92a, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20,
0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x947, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b,
0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x92b, 0x93c, 0x949, 0x915, 0x932, 0x948, 0x902, 0x921, 0x20, 0x91f, 0x93e, 0x92a,
0x941, 0x906, 0x902, 0x20, 0x926, 0x947, 0x20, 0x91d, 0x941, 0x902, 0x921, 0x947,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92b, 0x93c, 0x949, 0x915, 0x932,
0x948, 0x902, 0x921, 0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x906, 0x902, 0x20, 0x926,
0x947, 0x20, 0x91d, 0x941, 0x902, 0x921, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92b, 0x93c, 0x949, 0x915, 0x932,
0x948, 0x902, 0x921, 0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x906, 0x902, 0x20, 0x926,
0x947, 0x20, 0x91d, 0x941, 0x902, 0x921, 0x947, 0x20, 0x926, 0x940, 0x20, 0x924,
0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x92b, 0x93c, 0x93f, 0x91c, 0x940, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x92b, 0x93c, 0x93f, 0x91c, 0x940, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92b, 0x93c, 0x93f, 0x91c, 0x940, 0x20,
0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92b, 0x93c, 0x94d, 0x930, 0x947, 0x902, 0x91a,
0x20, 0x917, 0x941, 0x92f, 0x93e, 0x928, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x926, 0x916, 0x923, 0x947, 0x20, 0x92c, 0x916, 0x947, 0x902, 0x20,
0x92b, 0x93c, 0x94d, 0x930, 0x93e, 0x902, 0x938, 0x20, 0x915, 0x928, 0x947, 0x20,
0x905, 0x902, 0x91f, 0x93e, 0x930, 0x94d, 0x915, 0x91f, 0x93f, 0x915, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x917, 0x948, 0x932, 0x93e, 0x92a, 0x947, 0x917,
0x94b, 0x938, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x917, 0x948, 0x902,
0x92c, 0x93f, 0x92f, 0x930, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x91c,
0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x926,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x91c, 0x949,
0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b,
0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x917, 0x93f, 0x932, 0x94d, 0x92c, 0x930, 0x94d, 0x91f, 0x20, 0x91f, 0x93e, 0x92a,
0x941, 0x906, 0x902, 0x20, 0x926, 0x947, 0x20, 0x91d, 0x941, 0x902, 0x921, 0x947,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x917, 0x94d, 0x930, 0x940, 0x928,
0x935, 0x93f, 0x91a, 0x20, 0x92e, 0x940, 0x928, 0x20, 0x91f, 0x948, 0x92e, 0x92a,
0x948, 0x932, 0x947, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x948,
0x902, 0x921, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x948, 0x932, 0x947, 0x940, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x948, 0x902, 0x921, 0x20, 0x926, 0x93e,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x948, 0x932,
0x947, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x948, 0x902, 0x921,
0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e,
0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x948, 0x902, 0x921, 0x20,
0x91f, 0x948, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x928, 0x932, 0x948, 0x902, 0x921, 0x20, 0x926, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x936, 0x94d, 0x91a,
0x93f, 0x92e, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x948, 0x902,
0x921, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x916, 0x93e, 0x921, 0x93c, 0x940,
0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e,
0x939, 0x935, 0x93e, 0x908, 0x2013, 0x906, 0x932, 0x94d, 0x92f, 0x942, 0x936, 0x928,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x939, 0x935, 0x93e, 0x908, 0x2013,
0x906, 0x932, 0x94d, 0x92f, 0x942, 0x936, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x939, 0x935, 0x93e, 0x908, 0x2013,
0x906, 0x932, 0x94d, 0x92f, 0x942, 0x936, 0x928, 0x20, 0x926, 0x947, 0x20, 0x927,
0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c,
0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x939, 0x93e,
0x901, 0x917, 0x20, 0x915, 0x93e, 0x901, 0x917, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x939, 0x93e, 0x901, 0x917, 0x20, 0x915, 0x93e, 0x901, 0x917, 0x20,
0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x939,
0x93e, 0x901, 0x917, 0x20, 0x915, 0x93e, 0x901, 0x917, 0x20, 0x926, 0x940, 0x20,
0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x939, 0x94b, 0x935, 0x94d, 0x921, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x939, 0x94b, 0x935, 0x94d, 0x921, 0x20, 0x926, 0x93e, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x939, 0x94b, 0x935, 0x94d, 0x921,
0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92d, 0x93e, 0x930, 0x924, 0x947, 0x20,
0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x939,
0x93f, 0x902, 0x926, 0x20, 0x92e, 0x939, 0x93e, 0x938, 0x93e, 0x917, 0x930, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x907, 0x902, 0x921, 0x94b, 0x91a, 0x93e,
0x907, 0x928, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92c, 0x93f,
0x91a, 0x932, 0x947, 0x20, 0x907, 0x902, 0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f,
0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x948, 0x932,
0x947, 0x940, 0x20, 0x907, 0x902, 0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f, 0x92f,
0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x936, 0x94d, 0x91a,
0x93f, 0x92e, 0x940, 0x20, 0x907, 0x902, 0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f,
0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x908, 0x930, 0x93e,
0x928, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x908, 0x930, 0x93e, 0x928,
0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e,
0x908, 0x930, 0x93e, 0x928, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e,
0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x907, 0x930, 0x94d, 0x915, 0x941,
0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x907, 0x930, 0x94d, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x926,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x907, 0x930,
0x94d, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x926, 0x940, 0x20,
0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x907, 0x91c, 0x93c, 0x930, 0x93e, 0x907, 0x932, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x907, 0x91c, 0x93c, 0x930, 0x93e, 0x907, 0x932, 0x20,
0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x907,
0x91c, 0x93c, 0x930, 0x93e, 0x907, 0x932, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d,
0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e,
0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x91c, 0x93e, 0x92a,
0x93e, 0x928, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x91c, 0x93e,
0x92a, 0x93e, 0x928, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x948, 0x92e, 0x91c, 0x93e, 0x92a, 0x93e, 0x928, 0x947, 0x20, 0x926,
0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947,
0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x92a, 0x948, 0x932, 0x947, 0x20, 0x915, 0x91c, 0x93c, 0x93e, 0x916, 0x938,
0x94d, 0x924, 0x93e, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a,
0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x915, 0x91c, 0x93c, 0x93e, 0x916, 0x938,
0x94d, 0x924, 0x93e, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x915,
0x94b, 0x930, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x915, 0x94b, 0x930, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x926, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x94b, 0x930, 0x93f,
0x92f, 0x93e, 0x908, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921,
0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x94b, 0x938, 0x930, 0x93e, 0x90f,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x94d, 0x930, 0x93e, 0x938,
0x94d, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x94d, 0x930, 0x93e, 0x938, 0x94d, 0x928,
0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x926, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x94d, 0x930, 0x93e,
0x938, 0x94d, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20,
0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x93f, 0x930, 0x94d, 0x917, 0x93f, 0x938,
0x94d, 0x200d, 0x924, 0x93e, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x932, 0x93e, 0x907, 0x928, 0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x906, 0x902, 0x20,
0x926, 0x947, 0x20, 0x91d, 0x941, 0x902, 0x921, 0x947, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x932, 0x949, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x94b, 0x935,
0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x932, 0x949, 0x930, 0x94d,
0x921, 0x20, 0x939, 0x94b, 0x935, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x932, 0x949, 0x930, 0x94d, 0x921, 0x20,
0x939, 0x94b, 0x935, 0x947, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e,
0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92e, 0x93e, 0x917, 0x93e, 0x926,
0x93e, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92e, 0x93e, 0x917,
0x93e, 0x926, 0x93e, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x948, 0x92e, 0x92e, 0x93e, 0x917, 0x93e, 0x926, 0x93e, 0x928, 0x20,
0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92e, 0x932, 0x947, 0x936, 0x93f, 0x92f, 0x93e,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92e, 0x93e, 0x932, 0x926, 0x940,
0x935, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92e, 0x93e, 0x930, 0x94d,
0x915, 0x947, 0x938, 0x938, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92e,
0x93e, 0x930, 0x94d, 0x936, 0x932, 0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x906, 0x902,
0x20, 0x926, 0x947, 0x20, 0x91d, 0x941, 0x902, 0x921, 0x947, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x92e, 0x949, 0x930, 0x940, 0x936, 0x938, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92e, 0x949, 0x930, 0x940, 0x936, 0x938, 0x20,
0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92e,
0x949, 0x930, 0x940, 0x936, 0x938, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902,
0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92e,
0x93e, 0x935, 0x94d, 0x938, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x928, 0x20, 0x92a, 0x94d, 0x930,
0x936, 0x93e, 0x902, 0x924, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92e,
0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x936,
0x93e, 0x902, 0x924, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x948, 0x92e, 0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x928, 0x20,
0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924, 0x20, 0x926, 0x947, 0x20, 0x927,
0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c,
0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x909, 0x932,
0x93e, 0x928, 0x20, 0x92c, 0x91f, 0x94b, 0x930, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x909, 0x932, 0x93e, 0x928, 0x20, 0x92c, 0x91f, 0x94b, 0x930, 0x20,
0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x909,
0x932, 0x93e, 0x928, 0x20, 0x92c, 0x91f, 0x94b, 0x930, 0x20, 0x926, 0x940, 0x20,
0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x92e, 0x949, 0x938, 0x94d, 0x915, 0x94b, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x92e, 0x949, 0x938, 0x94d, 0x915, 0x94b, 0x20, 0x926, 0x93e,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92e, 0x949, 0x938,
0x94d, 0x915, 0x94b, 0x20, 0x926, 0x93f, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f,
0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92e, 0x94d, 0x92f,
0x93e, 0x902, 0x92e, 0x93e, 0x930, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x928, 0x94c, 0x930, 0x942, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x928,
0x947, 0x92a, 0x93e, 0x932, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x928,
0x94d, 0x92f, 0x942, 0x20, 0x915, 0x948, 0x932, 0x947, 0x921, 0x94b, 0x928, 0x93f,
0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x928, 0x94d, 0x92f,
0x942, 0x20, 0x915, 0x948, 0x932, 0x947, 0x921, 0x94b, 0x928, 0x93f, 0x92f, 0x93e,
0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e,
0x928, 0x94d, 0x92f, 0x942, 0x20, 0x915, 0x948, 0x932, 0x947, 0x921, 0x94b, 0x928,
0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f,
0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x928, 0x94d, 0x92f,
0x942, 0x91c, 0x93c, 0x940, 0x932, 0x948, 0x902, 0x921, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x928, 0x94d, 0x92f, 0x942, 0x91c, 0x93c, 0x940, 0x932, 0x948,
0x902, 0x921, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x948, 0x92e, 0x928, 0x94d, 0x92f, 0x942, 0x91c, 0x93c, 0x940, 0x932, 0x948, 0x902,
0x921, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947,
0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x928, 0x94d, 0x92f, 0x942, 0x92b, 0x93c, 0x93e, 0x909,
0x902, 0x921, 0x932, 0x948, 0x902, 0x921, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x928, 0x94d, 0x92f, 0x942, 0x92b, 0x93c, 0x93e, 0x909, 0x902, 0x921, 0x932,
0x948, 0x902, 0x921, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x948, 0x92e, 0x928, 0x94d, 0x92f, 0x942, 0x92b, 0x93c, 0x93e, 0x909, 0x902,
0x921, 0x932, 0x948, 0x902, 0x921, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f,
0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932,
0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x928, 0x940, 0x92f, 0x942,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x928, 0x949, 0x930, 0x92b, 0x93c,
0x949, 0x915, 0x20, 0x91f, 0x93e, 0x92a, 0x942, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x928, 0x949, 0x930, 0x92b, 0x93c, 0x949, 0x915, 0x20, 0x91f, 0x93e,
0x92a, 0x942, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x948, 0x92e, 0x928, 0x949, 0x930, 0x92b, 0x93c, 0x949, 0x915, 0x20, 0x91f, 0x93e,
0x92a, 0x942, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c,
0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92b, 0x93c, 0x930, 0x94d, 0x928, 0x93e, 0x902,
0x930, 0x94d, 0x921, 0x94b, 0x20, 0x921, 0x947, 0x20, 0x928, 0x94b, 0x930, 0x94b,
0x928, 0x94d, 0x939, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92b,
0x93c, 0x930, 0x94d, 0x928, 0x93e, 0x902, 0x930, 0x94d, 0x921, 0x94b, 0x20, 0x921,
0x947, 0x20, 0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92b, 0x93c,
0x930, 0x94d, 0x928, 0x93e, 0x902, 0x930, 0x94d, 0x921, 0x94b, 0x20, 0x921, 0x947,
0x20, 0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x20, 0x926, 0x947,
0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x928, 0x94b, 0x935, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930,
0x94d, 0x938, 0x94d, 0x915, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x928,
0x94b, 0x935, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915,
0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e,
0x928, 0x94b, 0x935, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d,
0x915, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x913, 0x92e, 0x94d, 0x938, 0x94d,
0x915, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x913, 0x92e, 0x94d, 0x938,
0x94d, 0x915, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x948, 0x92e, 0x913, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x926, 0x940, 0x20,
0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x947,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x93e, 0x915, 0x93f, 0x938,
0x94d, 0x924, 0x93e, 0x928, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924,
0x93e, 0x928, 0x947, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f,
0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x932, 0x93e,
0x909, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x93e, 0x92a, 0x941,
0x906, 0x20, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x917, 0x93f, 0x928, 0x940, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x948, 0x930, 0x93e, 0x917, 0x94d,
0x935, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x948, 0x930,
0x93e, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x948, 0x930, 0x93e, 0x917, 0x94d, 0x935,
0x947, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x947, 0x930, 0x942, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x947, 0x930, 0x942, 0x20, 0x926,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x947,
0x930, 0x942, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f,
0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92b, 0x93c, 0x93f, 0x932,
0x93f, 0x92a, 0x940, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92b,
0x93c, 0x93f, 0x932, 0x93f, 0x92a, 0x940, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92b, 0x93c, 0x93f, 0x932, 0x93f,
0x92a, 0x940, 0x928, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f,
0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92b, 0x93c, 0x940,
0x928, 0x93f, 0x915, 0x94d, 0x938, 0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x906, 0x902,
0x20, 0x926, 0x947, 0x20, 0x91d, 0x941, 0x902, 0x921, 0x947, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x92a, 0x93f, 0x90f,
0x930, 0x947, 0x20, 0x915, 0x928, 0x947, 0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x935,
0x947, 0x932, 0x93e, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x938,
0x947, 0x902, 0x91f, 0x20, 0x92a, 0x93f, 0x90f, 0x930, 0x947, 0x20, 0x915, 0x928,
0x947, 0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932, 0x93e, 0x928, 0x20,
0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x938,
0x947, 0x902, 0x91f, 0x20, 0x92a, 0x93f, 0x90f, 0x930, 0x947, 0x20, 0x915, 0x928,
0x947, 0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932, 0x93e, 0x928, 0x20,
0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926,
0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x92a, 0x93f, 0x91f, 0x915, 0x948, 0x930, 0x94d, 0x928, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x94b, 0x928, 0x93e, 0x92a, 0x947, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x94d, 0x92f, 0x94b, 0x902, 0x917,
0x92f, 0x93e, 0x902, 0x917, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x930,
0x940, 0x92f, 0x942, 0x928, 0x93f, 0x92f, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x930, 0x94b, 0x925, 0x947, 0x930, 0x93e, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x938, 0x916, 0x93e, 0x932, 0x93f, 0x928, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x938, 0x916, 0x93e, 0x932, 0x93f, 0x928, 0x20, 0x926,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x938, 0x916,
0x93e, 0x932, 0x93f, 0x928, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926,
0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x938, 0x92e,
0x94b, 0x906, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x938, 0x92e, 0x94b,
0x906, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948,
0x92e, 0x938, 0x92e, 0x94b, 0x906, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f,
0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932,
0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x938, 0x947, 0x936, 0x947,
0x932, 0x94d, 0x938, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x938, 0x93f,
0x902, 0x917, 0x93e, 0x92a, 0x941, 0x930, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x938, 0x94b, 0x932, 0x94b, 0x92e, 0x928, 0x20, 0x91f, 0x93e, 0x92a,
0x941, 0x906, 0x902, 0x20, 0x926, 0x947, 0x20, 0x91d, 0x941, 0x902, 0x921, 0x947,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x926, 0x916, 0x923, 0x20, 0x91c,
0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x938, 0x942, 0x930, 0x940, 0x928, 0x93e, 0x92e, 0x947, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x938, 0x94d, 0x92f, 0x94b, 0x935, 0x93e, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x924, 0x93e, 0x939, 0x93f, 0x924, 0x940,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x924, 0x93e, 0x907, 0x92a, 0x947,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x924, 0x93e, 0x907, 0x92a, 0x947,
0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e,
0x924, 0x93e, 0x907, 0x92a, 0x947, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f,
0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932,
0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x924, 0x93e, 0x91c, 0x93f,
0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x947, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x91f, 0x94b, 0x915, 0x947, 0x932, 0x93e, 0x90a, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x91f, 0x94b, 0x902, 0x917, 0x93e, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x91f, 0x94b, 0x902, 0x917, 0x93e, 0x20, 0x926,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x91f, 0x94b,
0x902, 0x917, 0x93e, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f,
0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x91a, 0x941, 0x915,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x924, 0x941, 0x930, 0x94d, 0x915,
0x92e, 0x947, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x924, 0x941, 0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928,
0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x924, 0x941, 0x930, 0x94d, 0x915,
0x92e, 0x947, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x947, 0x20, 0x926,
0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x924, 0x941, 0x935, 0x93e, 0x932, 0x942, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x909, 0x930, 0x941, 0x917, 0x94d, 0x935, 0x947,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x909, 0x930, 0x941, 0x917, 0x94d,
0x935, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x948, 0x92e, 0x909, 0x930, 0x941, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x926, 0x940,
0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x909, 0x91c, 0x93c, 0x94d, 0x92c, 0x947, 0x915, 0x93f, 0x938,
0x94d, 0x924, 0x93e, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x909,
0x91c, 0x93c, 0x94d, 0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928,
0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e,
0x909, 0x91c, 0x93c, 0x94d, 0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e,
0x928, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x935, 0x928, 0x941, 0x906, 0x924,
0x942, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x935, 0x928, 0x941, 0x906,
0x924, 0x942, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x948, 0x92e, 0x935, 0x928, 0x941, 0x906, 0x924, 0x942, 0x20, 0x926, 0x940, 0x20,
0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x935, 0x947, 0x928, 0x947, 0x91c, 0x93c, 0x941, 0x90f, 0x932, 0x93e,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x935, 0x94d, 0x932, 0x93e, 0x926,
0x93f, 0x935, 0x94b, 0x938, 0x94d, 0x924, 0x94b, 0x915, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x935, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x935, 0x94b, 0x938,
0x94d, 0x924, 0x94b, 0x915, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x948, 0x92e, 0x935, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x935, 0x94b,
0x938, 0x94d, 0x924, 0x94b, 0x915, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902,
0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x935,
0x94b, 0x932, 0x94d, 0x917, 0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x921, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x935, 0x94b, 0x932, 0x94d, 0x917, 0x94b, 0x917,
0x94d, 0x930, 0x93e, 0x921, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x948, 0x92e, 0x935, 0x94b, 0x932, 0x94d, 0x917, 0x94b, 0x917, 0x94d,
0x930, 0x93e, 0x921, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f,
0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x935, 0x947, 0x915,
0x20, 0x91f, 0x93e, 0x92a, 0x942, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x935, 0x93e, 0x932, 0x93f, 0x938, 0x20, 0x915, 0x928, 0x947, 0x20, 0x92b, 0x93c,
0x94d, 0x92f, 0x942, 0x91a, 0x942, 0x928, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x92f, 0x93e, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92f, 0x93e, 0x915, 0x941, 0x924, 0x94d,
0x938, 0x94d, 0x915, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x948, 0x92e, 0x92f, 0x93e, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915,
0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92f, 0x947, 0x915, 0x93e, 0x924, 0x947,
0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x92f, 0x947, 0x915, 0x93e, 0x924, 0x947, 0x930, 0x93f, 0x928, 0x92c,
0x930, 0x94d, 0x917, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x948, 0x92e, 0x92f, 0x947, 0x915, 0x93e, 0x924, 0x947, 0x930, 0x93f, 0x928,
0x92c, 0x930, 0x94d, 0x917, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926,
0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92f, 0x941,
0x915, 0x949, 0x928, 0x20, 0x91f, 0x948, 0x92e, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x66, 0x67, 0x68, 0x61, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0xc0, 0x66, 0x72, 0x65, 0x67, 0x61, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0xc0, 0x66, 0x72, 0x65, 0x67, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0xc0, 0x66, 0x72, 0x65, 0x67, 0x61, 0x20, 0x6d, 0x65, 0x72, 0x69,
0x64, 0x69, 0x6f, 0x6e, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0xc0, 0x66, 0x72, 0x65, 0x67, 0x61, 0x20, 0x6f,
0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0xc0, 0x66, 0x72, 0x65, 0x67, 0x61, 0x20, 0x6f, 0x73, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xc0,
0x66, 0x72, 0x65, 0x67, 0x61, 0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x41, 0x6d, 0x61, 0x73, 0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x73, 0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x73, 0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x20,
0x41, 0x6d, 0xe8, 0x72, 0x65, 0x67, 0x61, 0x20, 0x73, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x6e, 0x6f, 0x72,
0x64, 0x20, 0x41, 0x6d, 0xe8, 0x72, 0x65, 0x67, 0x61, 0x20, 0x73, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x6e, 0x6f,
0x72, 0x64, 0x20, 0x41, 0x6d, 0xe8, 0x72, 0x65, 0x67, 0x61, 0x20, 0x73,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x2019, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x20, 0x41, 0x6d, 0xe8,
0x72, 0x65, 0x67, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x20,
0x41, 0x6d, 0xe8, 0x72, 0x65, 0x67, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69,
0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x6e, 0x6f, 0x72,
0x64, 0x20, 0x41, 0x6d, 0xe8, 0x72, 0x65, 0x67, 0x61, 0x20, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x2019, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x20, 0x41, 0x6d, 0xe8,
0x72, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0x6d,
0x6f, 0x6e, 0x74, 0x61, 0x67, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20,
0x6e, 0x6f, 0x72, 0x64, 0x20, 0x41, 0x6d, 0xe8, 0x72, 0x65, 0x67, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61,
0x67, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74,
0xe0, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x20,
0x41, 0x6d, 0xe8, 0x72, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x67, 0x6e, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x50, 0x61, 0x73, 0xec, 0x66,
0x65, 0x67, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x50, 0x61, 0x73, 0xec,
0x66, 0x65, 0x67, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73,
0x74, 0xe0, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x50, 0x61, 0x73, 0xec,
0x66, 0x65, 0x67, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65,
0x20, 0x41, 0x70, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41,
0x72, 0x61, 0x62, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41,
0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f, 0x73, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f, 0x73,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f, 0x73,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c, 0xe0,
0x6e, 0x74, 0x65, 0x67, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74,
0x6c, 0xe0, 0x6e, 0x74, 0x65, 0x67, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41,
0x74, 0x6c, 0xe0, 0x6e, 0x74, 0x65, 0x67, 0x6f, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20,
0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x20, 0x6f, 0x73,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73,
0x65, 0x6e, 0x74, 0x72, 0x6f, 0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73,
0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x6f,
0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74,
0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x73, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x73,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x73,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69,
0x6a, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x65, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x65, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x2019, 0x6c, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x73, 0x68, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x2019, 0x6c, 0x20, 0x42, 0x75, 0x74, 0x61, 0x6e, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x42, 0x6f, 0x6c, 0x69,
0x76, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72,
0x61, 0x7a, 0x69, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61,
0x7a, 0x69, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69,
0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x7a, 0x69,
0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20,
0x42, 0x72, 0x75, 0x6e, 0xe9, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x43, 0x61, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x6f, 0x4f, 0x72,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x43, 0x61, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x6f, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20,
0x43, 0x61, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x6f, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x6f, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f,
0x6c, 0x65, 0x20, 0x43, 0x69, 0x61, 0x74, 0x65, 0x6d, 0x4f, 0x72, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x43, 0x69, 0x61,
0x74, 0x65, 0x6d, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74,
0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c,
0x65, 0x20, 0x43, 0x69, 0x61, 0x74, 0x65, 0x6d, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x2019, 0x6c, 0x20, 0x43, 0x69, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x2019,
0x6c, 0x20, 0x43, 0x69, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x43, 0x69,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20,
0x53, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x53, 0x69,
0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x53, 0x69, 0x6e, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xcc, 0x7a, 0x6f, 0x6c,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x61, 0x64, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f,
0x6c, 0x65, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6e, 0x62,
0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0x6f,
0x6e, 0x62, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73,
0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x43, 0x6f, 0x6c,
0x6f, 0x6e, 0x62, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x43, 0x6f, 0x6f,
0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65,
0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69,
0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a,
0x6f, 0x6c, 0x65, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x43,
0x75, 0x62, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74,
0xe0, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x20, 0x64,
0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x2019, 0x6c, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x45,
0x73, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xcc,
0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63,
0x75, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xcc, 0x7a, 0x6f, 0x6c, 0x61,
0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50,
0x61, 0x73, 0x63, 0x75, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0xc8, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69,
0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x74,
0x72, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f,
0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65,
0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x4d, 0x61, 0x6c, 0x76,
0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74,
0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c,
0x65, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65,
0x20, 0x46, 0x69, 0x6a, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc,
0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x47, 0x75,
0x79, 0x61, 0x6e, 0x61, 0x20, 0x66, 0x72, 0x61, 0x6e, 0x73, 0x65, 0x7a,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0x54,
0x65, 0x72, 0x65, 0x20, 0x66, 0x72, 0x61, 0x6e, 0x73, 0x65, 0x7a, 0x69,
0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x73, 0x75, 0x64, 0x20, 0x65, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6e, 0x74, 0xe0, 0x72, 0x74, 0x65,
0x67, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20,
0x47, 0x61, 0x6c, 0xe0, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65,
0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x4a, 0x65, 0x6f, 0x72, 0x6a, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x4a, 0x65, 0x6f, 0x72, 0x6a, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x20, 0x4a, 0x65, 0x6f, 0x72, 0x6a, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f,
0x6c, 0x65, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x6d, 0x65, 0x72, 0x69, 0x64,
0x69, 0x61, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x77, 0x69, 0x63, 0x68, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x61,
0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x47,
0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x6f, 0x73,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x61,
0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x61, 0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x47, 0x6f,
0x6c, 0x66, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61,
0x77, 0x61, 0x69, 0x20, 0x65, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69,
0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x20, 0x65,
0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x48,
0x61, 0x77, 0x61, 0x69, 0x20, 0x65, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b,
0x6f, 0x6e, 0x67, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74,
0xe0, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f,
0x6e, 0x67, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x48,
0x6f, 0x76, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x4f, 0x73, 0xe8, 0x61, 0x6e, 0x6f, 0x20, 0x49, 0x6e, 0x64,
0x69, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x49, 0x6e, 0x64, 0x6f, 0x73, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a,
0x69, 0x61, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x7a, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x61, 0x20, 0x6f, 0x73,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x72, 0x61, 0x6e, 0x4f, 0x72, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x49, 0x72, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x72,
0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64,
0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x49,
0x7a, 0x72, 0x61, 0x65, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x2019, 0x6c, 0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x2019, 0x6c,
0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x4a, 0x61,
0x70, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20,
0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20,
0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6f,
0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x43, 0x6f, 0x72, 0xe8, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x43, 0x6f, 0x72, 0xe8, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x61, 0x20, 0x43, 0x6f, 0x72, 0xe8, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20,
0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x4b, 0x69, 0x72, 0x67, 0x68, 0x69,
0x7a, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x45, 0x63,
0x75, 0x61, 0x74, 0x6f, 0x72, 0x69, 0x61, 0x6c, 0x69, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69,
0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xcc, 0x7a, 0x6f,
0x6c, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x20, 0x4d, 0x61, 0x6c, 0x65, 0x7a, 0x69, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a,
0x6f, 0x6c, 0x65, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f,
0x6c, 0x65, 0x20, 0x4d, 0x61, 0x72, 0x63, 0x68, 0x65, 0x7a, 0x69, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f,
0x6c, 0x65, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f,
0x6c, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x73, 0x69, 0x6f, 0x4f,
0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x73, 0x69, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20,
0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x73,
0x69, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x4d,
0xe8, 0x73, 0x65, 0x67, 0x6f, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x50,
0x61, 0x73, 0xec, 0x66, 0x65, 0x67, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20,
0x4d, 0xe8, 0x73, 0x65, 0x67, 0x6f, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20,
0x50, 0x61, 0x73, 0xec, 0x66, 0x65, 0x67, 0x6f, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20,
0x4d, 0xe8, 0x73, 0x65, 0x67, 0x6f, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20,
0x50, 0x61, 0x73, 0xec, 0x66, 0x65, 0x67, 0x6f, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0xe0, 0x74, 0x6f,
0x72, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0xe0, 0x74,
0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0,
0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0xe0, 0x74,
0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x6f, 0x73, 0x63, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x20, 0x28, 0x42, 0x69, 0x72,
0x6d, 0x61, 0x6e, 0x69, 0x61, 0x29, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x4e, 0x61, 0x75,
0x72, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x4e,
0x65, 0x70, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x4e, 0x6f,
0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x43, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x5a, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x4e,
0x6f, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c,
0x61, 0x6e, 0x64, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x76, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xcc, 0x7a, 0x6f, 0x6c,
0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x4f, 0x72, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74,
0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xcc, 0x7a, 0x6f, 0x6c, 0x61,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4f,
0x6d, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74,
0xe0, 0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74,
0xe0, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65,
0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x50, 0x61, 0x6c, 0xe0, 0x75,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x50, 0x61,
0x70, 0x75, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x47, 0x75, 0x69,
0x6e, 0xe8, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0xe0, 0x69, 0x4f, 0x72, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x2019, 0x6c,
0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0xe0, 0x69, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x2019, 0x6c,
0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0xe0, 0x69, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x50, 0x65, 0x72, 0xf9, 0x4f, 0x72,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x2019, 0x6c, 0x20, 0x50, 0x65, 0x72, 0xf9, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x50,
0x65, 0x72, 0xf9, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65,
0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f,
0x6c, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x46, 0x65,
0x6e, 0x69, 0x7a, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53,
0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x6f, 0x20, 0x65, 0x20, 0x4d, 0x69,
0x63, 0x68, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x53, 0x2e, 0x20,
0x50, 0x69, 0x65, 0x72, 0x6f, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x63, 0x68,
0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73,
0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x53, 0x2e, 0x20, 0x50, 0x69, 0x65,
0x72, 0x6f, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x63, 0x68, 0x65, 0x6c, 0x6f,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc,
0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xcc, 0x7a,
0x6f, 0x6c, 0x61, 0x20, 0x50, 0x6f, 0x6e, 0x70, 0xe8, 0x69, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xcc, 0x7a, 0x6f, 0x6c, 0x61,
0x20, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x61, 0x6c, 0x69, 0x6e, 0x4f, 0x72,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x53, 0x61, 0x6b, 0x61, 0x6c, 0x69, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61,
0x6b, 0x61, 0x6c, 0x69, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x53, 0x61, 0x6d,
0xf2, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c,
0x65, 0x20, 0x53, 0x61, 0x6d, 0xf2, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20,
0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x53, 0x61, 0x6d, 0xf2, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x53, 0x65,
0x69, 0x73, 0x65, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x53, 0x61, 0x6c, 0x6f,
0x6d, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61,
0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64, 0x65, 0x2019,
0x6c, 0x20, 0x73, 0x75, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019,
0x6c, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xcc, 0x7a, 0x6f, 0x6c, 0x61,
0x20, 0x54, 0x61, 0x69, 0x74, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x54, 0x61, 0x69, 0x70, 0xe9, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61,
0x69, 0x70, 0xe9, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73,
0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0xe9, 0x69,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x54, 0x61, 0x6a,
0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x54,
0x6f, 0x6b, 0x65, 0x6c, 0xe0, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f,
0x6c, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65,
0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x43, 0x68, 0x75,
0x75, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x54,
0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f,
0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x2019, 0x6c, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73,
0x74, 0xe0, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20,
0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0xe0, 0x69, 0x4f, 0x72,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0xe0, 0x69, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0xe0, 0x69, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x7a, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55,
0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65,
0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f, 0x72, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74,
0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c,
0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x56, 0x65, 0x6e, 0x65, 0x73, 0x75,
0x65, 0x6c, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69,
0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x4f,
0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64,
0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6f,
0x6c, 0x6f, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x57,
0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x65, 0x20, 0x46, 0x75, 0x74, 0x75,
0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64,
0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x45, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x45,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x6f,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x59, 0x75, 0x6b,
0x6f, 0x6e, 0x61, 0x61, 0x70, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x61, 0x64, 0x69, 0x6e, 0x69,
0x20, 0x61, 0x61, 0x70, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x76, 0x65, 0x1e0d, 0x61, 0x20, 0x68, 0x70, 0x75, 0x20, 0x61, 0x61,
0x70, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x64, 0x6b,
0x69, 0x1e47, 0x20, 0x61, 0x61, 0x70, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x76, 0x65, 0x1e0d, 0x61, 0x6b, 0x75, 0x1e47, 0x70, 0x75,
0x20, 0x61, 0x61, 0x70, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x76, 0x65, 0x1e0d, 0x61, 0x20, 0x6b, 0x75, 0x1e47, 0x70, 0x75, 0x20,
0x61, 0x61, 0x70, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x6d, 0x61, 0x6e, 0x61,
0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x76, 0x65, 0x1e0d, 0x61,
0x20, 0x6b, 0x75, 0x1e47, 0x70, 0x75, 0x20, 0x61, 0x61, 0x70, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x6b, 0x61, 0x72, 0xe3, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x61, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x61,
0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x6d, 0x61, 0x6e, 0x61, 0x6e, 0x6b,
0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x61, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x20, 0x1e0d, 0x65, 0x6c, 0x61, 0x61, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x101, 0x6d, 0x61, 0x6a, 0x6f, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x101, 0x6d, 0x61, 0x6a, 0x6f, 0x6e, 0x20, 0x6d, 0x61, 0x6e, 0x61, 0x6e,
0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x6d, 0x61, 0x6a, 0x6f,
0x6e, 0x20, 0x6b, 0x61, 0x72, 0xe3, 0x20, 0x6d, 0x61, 0x73, 0x61, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x6d, 0x61, 0x64, 0x69, 0x6e, 0x12b, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x6d, 0x61, 0x64, 0x69, 0x6e, 0x12b, 0x20, 0x6d, 0x61,
0x6e, 0x61, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x61,
0x64, 0x69, 0x6e, 0x12b, 0x20, 0x1e0d, 0x65, 0x6c, 0x61, 0x61, 0x69, 0x1e6d,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x76, 0x65, 0x1e0d, 0x61, 0x68, 0x61, 0x70,
0x75, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x76, 0x65, 0x1e0d, 0x61, 0x68, 0x61,
0x70, 0x75, 0x20, 0x6d, 0x61, 0x6e, 0x61, 0x6e, 0x6b, 0x61, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x76, 0x65, 0x1e0d, 0x61, 0x68, 0x61, 0x70, 0x75, 0x20,
0x1e0d, 0x65, 0x6c, 0x61, 0x61, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x68, 0x61, 0x72, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x68, 0x61,
0x72, 0x6b, 0x61, 0x20, 0x6d, 0x61, 0x6e, 0x61, 0x6e, 0x6b, 0x61, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x68, 0x61, 0x72, 0x6b, 0x61, 0x20, 0x1e0d, 0x65,
0x6c, 0x61, 0x61, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x65,
0x73, 0x69, 0x70, 0x69, 0x6b, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x65,
0x73, 0x69, 0x70, 0x69, 0x6b, 0x20, 0x6d, 0x61, 0x6e, 0x61, 0x6e, 0x6b,
0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x65, 0x73, 0x69, 0x70, 0x69,
0x6b, 0x20, 0x1e0d, 0x65, 0x6c, 0x61, 0x61, 0x69, 0x1e6d, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x65, 0x70, 0x69, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x65,
0x70, 0x69, 0x101, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x65, 0x70, 0x69, 0x101, 0x20, 0x64, 0x65, 0x6c,
0x101, 0x69, 0x74, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x72, 0x62, 0x69,
0x101, 0x74, 0x69, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x72, 0x62, 0x69,
0x101, 0x74, 0x69, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x101, 0x72, 0x62, 0x69, 0x101, 0x74, 0x69, 0x20,
0x64, 0x65, 0x6c, 0x101, 0x69, 0x74, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101,
0x72, 0x6a, 0x65, 0x6e, 0x1e6d, 0x69, 0x6e, 0x101, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x101, 0x72, 0x6a, 0x65, 0x6e, 0x1e6d, 0x69, 0x6e, 0x101, 0x20, 0x6d,
0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101,
0x72, 0x6a, 0x65, 0x6e, 0x1e6d, 0x69, 0x6e, 0x101, 0x20, 0x6b, 0x101, 0x72,
0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x77, 0x65, 0x1e0d, 0x101, 0x6b, 0x75, 0x1e47, 0x75, 0x70, 0x75, 0x20, 0x101,
0x72, 0x6a, 0x65, 0x6e, 0x1e6d, 0x69, 0x6e, 0x101, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x77, 0x65, 0x1e0d, 0x101, 0x6b, 0x75, 0x1e47, 0x75, 0x70, 0x75, 0x20,
0x101, 0x72, 0x6a, 0x65, 0x6e, 0x1e6d, 0x69, 0x6e, 0x101, 0x20, 0x6d, 0x101,
0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x65,
0x1e0d, 0x101, 0x6b, 0x75, 0x1e47, 0x75, 0x70, 0x75, 0x20, 0x101, 0x72, 0x6a,
0x65, 0x6e, 0x1e6d, 0x69, 0x6e, 0x101, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x101,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x101,
0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x101, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x101, 0x20, 0x6b, 0x101, 0x72,
0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x1e6d, 0x6c, 0x101, 0x6e,
0x1e6d, 0x69, 0x6b, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x1e6d, 0x6c, 0x101,
0x6e, 0x1e6d, 0x69, 0x6b, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x1e6d, 0x6c, 0x101, 0x6e, 0x1e6d, 0x69,
0x6b, 0x20, 0x64, 0x65, 0x6c, 0x101, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x6d, 0x101, 0x64, 0x69, 0x6e, 0x101, 0x20, 0x101, 0x73, 0x74, 0x72,
0x65, 0x6c, 0x69, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x73, 0x74,
0x72, 0x65, 0x6c, 0x69, 0x101, 0x74, 0x69, 0x20, 0x6d, 0x101, 0x64, 0x69,
0x6e, 0x101, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x101, 0x73, 0x74, 0x72, 0x65, 0x6c, 0x69, 0x101, 0x74,
0x69, 0x20, 0x6d, 0x101, 0x64, 0x69, 0x6e, 0x101, 0x20, 0x64, 0x65, 0x6c,
0x101, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x73, 0x74, 0x72,
0x65, 0x6c, 0x69, 0x101, 0x74, 0x69, 0x20, 0x6d, 0x101, 0x64, 0x69, 0x6e,
0x101, 0x20, 0x77, 0x65, 0x1e0d, 0x101, 0x6b, 0x75, 0x1e47, 0x75, 0x70, 0x75,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x73, 0x74, 0x72, 0x65, 0x6c, 0x69,
0x101, 0x74, 0x69, 0x20, 0x6d, 0x101, 0x64, 0x69, 0x6e, 0x101, 0x20, 0x77,
0x65, 0x1e0d, 0x101, 0x6b, 0x75, 0x1e47, 0x75, 0x70, 0x75, 0x20, 0x6d, 0x101,
0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x73,
0x74, 0x72, 0x65, 0x6c, 0x69, 0x101, 0x74, 0x69, 0x20, 0x6d, 0x101, 0x64,
0x69, 0x6e, 0x101, 0x20, 0x77, 0x65, 0x1e0d, 0x101, 0x6b, 0x75, 0x1e47, 0x75,
0x70, 0x75, 0x20, 0x64, 0x65, 0x6c, 0x101, 0x69, 0x1e6d, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x101, 0x68, 0x61, 0x70, 0x75, 0x20, 0x101,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x101, 0x73, 0x74, 0x72, 0x65, 0x6c, 0x69, 0x101, 0x74, 0x69, 0x20, 0x77,
0x65, 0x1e0d, 0x101, 0x68, 0x61, 0x70, 0x16b, 0x20, 0x6d, 0x101, 0x6e, 0x101,
0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x73, 0x74, 0x72,
0x65, 0x6c, 0x69, 0x101, 0x74, 0x69, 0x20, 0x77, 0x65, 0x1e0d, 0x101, 0x68,
0x61, 0x70, 0x16b, 0x20, 0x64, 0x65, 0x6c, 0x101, 0x69, 0x74, 0x69, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x101, 0x6b, 0x75, 0x1e47, 0x75,
0x70, 0x16b, 0x20, 0x101, 0x73, 0x74, 0x72, 0x65, 0x6c, 0x69, 0x101, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x101, 0x73, 0x74, 0x72, 0x65, 0x6c, 0x69, 0x101,
0x74, 0x69, 0x20, 0x77, 0x65, 0x1e0d, 0x101, 0x6b, 0x75, 0x1e47, 0x75, 0x70,
0x16b, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x101, 0x73, 0x74, 0x72, 0x65, 0x6c, 0x69, 0x101, 0x74, 0x69,
0x20, 0x77, 0x65, 0x1e0d, 0x101, 0x6b, 0x75, 0x1e47, 0x75, 0x70, 0x16b, 0x20,
0x64, 0x65, 0x6c, 0x101, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101,
0x6a, 0x65, 0x72, 0x62, 0x101, 0x69, 0x6a, 0x101, 0x6e, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x101, 0x6a, 0x65, 0x72, 0x62, 0x101, 0x69, 0x6a, 0x101, 0x6e,
0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x101, 0x6a, 0x65, 0x72, 0x62, 0x101, 0x69, 0x6a, 0x101, 0x6e, 0x20,
0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x61, 0x6a,
0x6f, 0x72, 0x65, 0x73, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x61, 0x6a, 0x6f,
0x72, 0x65, 0x73, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x61, 0x6a, 0x6f, 0x72, 0x65, 0x73, 0x20, 0x6b,
0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x62, 0x101, 0x6e,
0x67, 0x6c, 0x101, 0x64, 0x65, 0x73, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x62,
0x101, 0x6e, 0x67, 0x6c, 0x101, 0x64, 0x65, 0x73, 0x20, 0x6d, 0x101, 0x6e,
0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x62, 0x101, 0x6e,
0x67, 0x6c, 0x101, 0x64, 0x65, 0x73, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x62, 0x75, 0x74, 0x101, 0x6e, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x62, 0x6f, 0x6c, 0x77, 0x69, 0x101, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x62, 0x72, 0x101, 0x6a, 0x69, 0x6c, 0x69, 0x101, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x62, 0x72, 0x101, 0x6a, 0x69, 0x6c, 0x69, 0x101, 0x20,
0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x62, 0x72, 0x101, 0x6a, 0x69, 0x6c, 0x69, 0x101, 0x20, 0x6b, 0x101, 0x72,
0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x62, 0x72, 0x75, 0x6e, 0x65,
0x74, 0x69, 0x20, 0x64, 0x101, 0x72, 0x75, 0x73, 0x101, 0x6c, 0x101, 0x6d,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x65, 0x70, 0x20, 0x62, 0x1e0d, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x6b, 0x65, 0x70, 0x20, 0x62, 0x1e0d, 0x20, 0x6d,
0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b,
0x65, 0x70, 0x20, 0x62, 0x1e0d, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x63, 0x101, 0x6d, 0x6f, 0x72, 0x20, 0x6d, 0x101,
0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x63, 0x79,
0x101, 0x74, 0x101, 0x6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x63, 0x79, 0x101,
0x74, 0x101, 0x6d, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x63, 0x79, 0x101, 0x74, 0x101, 0x6d, 0x20, 0x64,
0x65, 0x6c, 0x101, 0x69, 0x74, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x63, 0x69,
0x6e, 0x69, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x63, 0x69, 0x6c, 0x69, 0x20,
0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x63, 0x69, 0x6c, 0x69, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x63, 0x69, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x63,
0x69, 0x6e, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x63, 0x69, 0x6e, 0x20, 0x64, 0x65, 0x6c, 0x101, 0x69,
0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x72, 0x69, 0x73, 0x6d, 0x101,
0x73, 0x20, 0x64, 0x12b, 0x70, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x6f,
0x6b, 0x6f, 0x73, 0x20, 0x64, 0x12b, 0x70, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x6b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x101, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x6b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x101, 0x20, 0x6d, 0x101,
0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x6f,
0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x101, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x75, 0x6b, 0x20, 0x64, 0x12b, 0x70,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x75, 0x6b, 0x20, 0x64, 0x12b, 0x70,
0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x6b, 0x75, 0x6b, 0x20, 0x64, 0x12b, 0x70, 0x20, 0x101, 0x64, 0x101,
0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b,
0x75, 0x62, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x75, 0x62, 0x101,
0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x6b, 0x75, 0x62, 0x101, 0x20, 0x64, 0x65, 0x6c, 0x101, 0x69, 0x1e6d,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x64, 0x65, 0x62, 0x69, 0x73, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x1e0d, 0x75, 0x6d, 0x6f, 0x6e, 0x1e6d, 0x20, 0x1e0d, 0x69,
0x20, 0x61, 0x72, 0x77, 0x69, 0x6c, 0x113, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x77, 0x65, 0x1e0d, 0x101, 0x68, 0x61, 0x70, 0x75, 0x20, 0x74, 0x69, 0x6d,
0x6f, 0x72, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x69, 0x73, 0x1e6d, 0x72, 0x20,
0x64, 0x12b, 0x70, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x69, 0x73, 0x1e6d, 0x72,
0x20, 0x64, 0x12b, 0x70, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x69, 0x73, 0x1e6d, 0x72, 0x20, 0x64, 0x12b,
0x70, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x69, 0x71, 0x75, 0x101, 0x1e0d, 0x6f, 0x72, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x6d, 0x101, 0x64, 0x69, 0x6e, 0x101, 0x20, 0x79, 0x75, 0x72, 0x6f, 0x70,
0x69, 0x101, 0x74, 0x69, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x101, 0x64,
0x69, 0x6e, 0x101, 0x20, 0x79, 0x75, 0x72, 0x6f, 0x70, 0x69, 0x101, 0x74,
0x69, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x6d, 0x101, 0x64, 0x69, 0x6e, 0x101, 0x20, 0x79, 0x75, 0x72,
0x6f, 0x70, 0x69, 0x101, 0x74, 0x69, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x101, 0x68, 0x70, 0x75,
0x20, 0x79, 0x75, 0x72, 0x6f, 0x70, 0x69, 0x101, 0x74, 0x69, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x101, 0x68, 0x70, 0x75, 0x20, 0x79,
0x75, 0x72, 0x6f, 0x70, 0x69, 0x101, 0x74, 0x69, 0x20, 0x6d, 0x101, 0x6e,
0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d,
0x101, 0x68, 0x70, 0x75, 0x20, 0x79, 0x75, 0x72, 0x6f, 0x70, 0x69, 0x101,
0x74, 0x69, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x61, 0x72, 0x20, 0x77, 0x65, 0x1e0d, 0x101, 0x68, 0x70, 0x75, 0x20,
0x79, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x74, 0x69, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x77, 0x65, 0x1e0d, 0x101, 0x6b, 0x16b, 0x1e47, 0x16b, 0x70, 0x75, 0x20,
0x79, 0x75, 0x72, 0x6f, 0x70, 0x69, 0x101, 0x74, 0x69, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x101, 0x6b, 0x16b, 0x1e47, 0x16b, 0x70, 0x75,
0x20, 0x79, 0x75, 0x72, 0x6f, 0x70, 0x69, 0x101, 0x74, 0x69, 0x20, 0x6d,
0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77,
0x65, 0x1e0d, 0x101, 0x6b, 0x16b, 0x1e47, 0x16b, 0x70, 0x75, 0x20, 0x79, 0x75,
0x72, 0x6f, 0x70, 0x69, 0x101, 0x74, 0x69, 0x20, 0x6b, 0x101, 0x72, 0x7e,
0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x61, 0x6b, 0x6c, 0x79, 0x101,
0x6e, 0x1e0d, 0x20, 0x64, 0x12b, 0x70, 0x61, 0x74, 0x69, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x70, 0x61, 0x6b, 0x6c, 0x79, 0x101, 0x6e, 0x1e0d, 0x20, 0x64,
0x12b, 0x70, 0x61, 0x74, 0x69, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b,
0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x61, 0x6b, 0x6c, 0x79, 0x101,
0x6e, 0x1e0d, 0x20, 0x64, 0x12b, 0x70, 0x61, 0x74, 0x69, 0x20, 0x6b, 0x101,
0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x69, 0x6a, 0x69,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x69, 0x6a, 0x69, 0x20, 0x6d, 0x101,
0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x69,
0x6a, 0x69, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x70, 0x72, 0x65, 0x6e, 0x63, 0x68, 0x20, 0x67, 0x75, 0x79, 0x101,
0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x72, 0x65, 0x6e, 0x63, 0x20,
0x64, 0x61, 0x6b, 0x69, 0x1e47, 0x20, 0x61, 0x1e0d, 0x65, 0x20, 0x61, 0x61,
0x6e, 0x1e6d, 0x101, 0x72, 0x6b, 0x1e6d, 0x69, 0x6b, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x67, 0x101, 0x6c, 0x101, 0x70, 0x6f, 0x67, 0x73, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x67, 0x61, 0x6d, 0x62, 0x69, 0x79, 0x72, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x6a, 0x61, 0x72, 0x6a, 0x12b, 0x101, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x6a, 0x61, 0x72, 0x6a, 0x69, 0x101, 0x20, 0x6d, 0x101, 0x6e, 0x101,
0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6a, 0x61, 0x72, 0x6a,
0x69, 0x101, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x67, 0x12b, 0x6c, 0x62, 0x72, 0x1e6d, 0x20, 0x64, 0x12b, 0x70, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x67, 0x72, 0x69, 0x6e, 0x77, 0x69, 0x63, 0x20,
0x6d, 0x12b, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x101,
0x68, 0x70, 0x75, 0x20, 0x67, 0x72, 0x69, 0x6e, 0x6c, 0x101, 0x6e, 0x64,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x101, 0x68, 0x70, 0x75,
0x20, 0x67, 0x72, 0x69, 0x6e, 0x6c, 0x101, 0x6e, 0x64, 0x20, 0x6d, 0x101,
0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x65,
0x1e0d, 0x101, 0x68, 0x70, 0x75, 0x20, 0x67, 0x72, 0x69, 0x6e, 0x6c, 0x101,
0x6e, 0x64, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x77, 0x65, 0x1e0d, 0x61, 0x6b, 0x16b, 0x1e47, 0x16b, 0x70, 0x20, 0x67,
0x72, 0x69, 0x6e, 0x6c, 0x101, 0x6e, 0x1e0d, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x77, 0x65, 0x1e0d, 0x61, 0x6b, 0x16b, 0x1e47, 0x16b, 0x70, 0x20, 0x67, 0x72,
0x69, 0x6e, 0x6c, 0x101, 0x6e, 0x1e0d, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e,
0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x61, 0x6b,
0x16b, 0x1e47, 0x16b, 0x70, 0x20, 0x67, 0x72, 0x69, 0x6e, 0x6c, 0x101, 0x6e,
0x1e0d, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x67, 0x61, 0x6c, 0x70, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x68, 0x101, 0x77, 0x101, 0x65, 0x2d, 0x20,
0x61, 0x61, 0x6c, 0x16b, 0x73, 0x101, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x68, 0x101, 0x77, 0x101, 0x65, 0x2d, 0x20, 0x61, 0x61, 0x6c, 0x16b, 0x73,
0x101, 0x6e, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x68, 0x101, 0x77, 0x101, 0x65, 0x2d, 0x20, 0x61, 0x61,
0x6c, 0x16b, 0x73, 0x101, 0x6e, 0x20, 0x1e0d, 0x65, 0x6c, 0x101, 0x69, 0x1e6d,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x68, 0x61, 0x6e, 0x67, 0x20, 0x6b, 0x61,
0x6e, 0x67, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x68, 0x61, 0x6e, 0x67, 0x20,
0x6b, 0x61, 0x6e, 0x67, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x68, 0x61, 0x6e, 0x67, 0x20, 0x6b, 0x61,
0x6e, 0x67, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x68, 0x77, 0x6f, 0x1e0d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x68, 0x77,
0x6f, 0x1e0d, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x68, 0x77, 0x6f, 0x1e0d, 0x20, 0x6b, 0x101, 0x72, 0x7e,
0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x62, 0x101, 0x72, 0x61, 0x74, 0x20,
0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x62, 0x101, 0x72, 0x61, 0x74, 0x20, 0x6b, 0x101, 0x6a, 0x101, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x69, 0x6e, 0x1e0d, 0x6f, 0x63, 0x69, 0x6e, 0x101, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x6d, 0x101, 0x64, 0x69, 0x6e, 0x69, 0x20, 0x69,
0x6e, 0x1e0d, 0x6e, 0x65, 0x73, 0x69, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x77, 0x65, 0x1e0d, 0x101, 0x68, 0x70, 0x75, 0x20, 0x69, 0x6e, 0x1e0d, 0x6e,
0x65, 0x73, 0x69, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d,
0x101, 0x6b, 0x16b, 0x1e47, 0x70, 0x16b, 0x20, 0x69, 0x6e, 0x1e0d, 0x6e, 0x65,
0x73, 0x69, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x69, 0x72, 0x101, 0x6e,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x69, 0x72, 0x101, 0x6e, 0x20, 0x6d, 0x101,
0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x69, 0x72,
0x101, 0x6e, 0x20, 0x1e0d, 0x65, 0x6c, 0x101, 0x69, 0x1e6d, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x12b, 0x72, 0x6b, 0x75, 0x73, 0x74, 0x61, 0x6b, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x12b, 0x72, 0x6b, 0x75, 0x73, 0x74, 0x61, 0x6b, 0x20,
0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x12b, 0x72, 0x6b, 0x75, 0x73, 0x74, 0x61, 0x6b, 0x20, 0x6b, 0x101, 0x72,
0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x69, 0x6a, 0x61, 0x72, 0x101,
0x12b, 0x6c, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x69, 0x6a, 0x61, 0x72, 0x101,
0x12b, 0x6c, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x69, 0x6a, 0x61, 0x72, 0x101, 0x12b, 0x6c, 0x20, 0x1e0d,
0x65, 0x6c, 0x101, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6a, 0x101,
0x70, 0x101, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6a, 0x101, 0x70, 0x101,
0x6e, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x6a, 0x101, 0x70, 0x101, 0x6e, 0x20, 0x1e0d, 0x65, 0x6c, 0x101,
0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x101, 0x6a, 0x101, 0x6b,
0x73, 0x74, 0x101, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d,
0x101, 0x68, 0x61, 0x70, 0x75, 0x20, 0x6b, 0x101, 0x6a, 0x101, 0x6b, 0x73,
0x74, 0x101, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x101,
0x6b, 0x16b, 0x1e47, 0x70, 0x16b, 0x20, 0x6b, 0x101, 0x6a, 0x101, 0x6b, 0x73,
0x74, 0x101, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x6f, 0x72, 0x69,
0x101, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x6f, 0x72, 0x69, 0x101,
0x6e, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x6b, 0x6f, 0x72, 0x69, 0x101, 0x6e, 0x20, 0x1e0d, 0x65, 0x6c,
0x101, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x61, 0x73, 0x72,
0x101, 0x113, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x72, 0x101, 0x73, 0x6e,
0x14d, 0x72, 0x73, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x72,
0x101, 0x73, 0x6e, 0x14d, 0x72, 0x73, 0x6b, 0x61, 0x20, 0x6d, 0x101, 0x6e,
0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x72, 0x101,
0x73, 0x6e, 0x14d, 0x72, 0x73, 0x6b, 0x61, 0x20, 0x6b, 0x101, 0x72, 0x7e,
0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x69, 0x72, 0x67, 0x69, 0x6a,
0x73, 0x74, 0x101, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6c, 0x101, 0x69,
0x6e, 0x20, 0x64, 0x12b, 0x70, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6c, 0x61,
0x1e5b, 0x20, 0x68, 0x61, 0x77, 0x65, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6c,
0x61, 0x1e5b, 0x20, 0x68, 0x61, 0x77, 0x65, 0x20, 0x6d, 0x101, 0x6e, 0x101,
0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6c, 0x61, 0x1e5b, 0x20,
0x68, 0x61, 0x77, 0x65, 0x20, 0x1e0d, 0x65, 0x6c, 0x101, 0x69, 0x1e6d, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x6d, 0x101, 0x67, 0x101, 0x64, 0x61, 0x6e, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x6d, 0x101, 0x67, 0x101, 0x64, 0x61, 0x6e, 0x20,
0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x6d, 0x101, 0x67, 0x101, 0x64, 0x61, 0x6e, 0x20, 0x1e0d, 0x65, 0x6c, 0x101,
0x69, 0x74, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x61, 0x6c, 0x65, 0x73,
0x69, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x101, 0x6c, 0x64, 0x12b,
0x70, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x101, 0x72, 0x6b, 0x73, 0x61,
0x73, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x101, 0x72, 0x73, 0x101, 0x6c,
0x20, 0x64, 0x12b, 0x70, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x61,
0x72, 0x12b, 0x73, 0x61, 0x73, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x61,
0x72, 0x12b, 0x73, 0x61, 0x73, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x61, 0x72, 0x12b, 0x73, 0x61, 0x73,
0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d,
0x101, 0x77, 0x6f, 0x73, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x65,
0x6b, 0x73, 0x69, 0x6b, 0x101, 0x6e, 0x20, 0x70, 0x65, 0x73, 0x69, 0x70,
0x69, 0x63, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x65, 0x6b, 0x73, 0x69,
0x6b, 0x101, 0x6e, 0x20, 0x70, 0x65, 0x73, 0x69, 0x70, 0x69, 0x63, 0x20,
0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x6d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x101, 0x6e, 0x20, 0x70, 0x65, 0x73,
0x69, 0x70, 0x69, 0x63, 0x20, 0x1e0d, 0x65, 0x6c, 0x101, 0x69, 0x1e6d, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x16b, 0x6c, 0x101, 0x6e, 0x62, 0x61, 0x74, 0x6f,
0x72, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x16b, 0x6c, 0x101, 0x6e, 0x62, 0x61,
0x74, 0x6f, 0x72, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x16b, 0x6c, 0x101, 0x6e, 0x62, 0x61, 0x74, 0x6f,
0x72, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x6d, 0x6f, 0x73, 0x6b, 0x6f, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x6f,
0x73, 0x6b, 0x6f, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x6d, 0x6f, 0x73, 0x6b, 0x6f, 0x20, 0x6b, 0x101,
0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x69, 0xf1, 0x79,
0x101, 0x6d, 0x101, 0x72, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e, 0x101, 0x75,
0x72, 0x16b, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e, 0x65, 0x70, 0x101, 0x6c,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e, 0x69, 0x16b, 0x20, 0x6b, 0x65, 0x6c,
0x65, 0x1e0d, 0x6f, 0x6e, 0x69, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e,
0x69, 0x16b, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x1e0d, 0x6f, 0x6e, 0x69, 0x101,
0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x6e, 0x69, 0x16b, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x1e0d, 0x6f, 0x6e,
0x69, 0x101, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x6e, 0x69, 0x16b, 0x6a, 0x69, 0x6c, 0x101, 0x6e, 0x1e0d, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x6e, 0x69, 0x16b, 0x6a, 0x69, 0x6c, 0x101, 0x6e, 0x1e0d,
0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x6e, 0x69, 0x16b, 0x6a, 0x69, 0x6c, 0x101, 0x6e, 0x1e0d, 0x20, 0x1e0d,
0x65, 0x6c, 0x101, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e, 0x69,
0x16b, 0x70, 0x61, 0x75, 0x6e, 0x1e0d, 0x6c, 0x79, 0x101, 0x6e, 0x1e0d, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x6e, 0x69, 0x16b, 0x70, 0x61, 0x75, 0x6e, 0x1e0d,
0x6c, 0x79, 0x101, 0x6e, 0x1e0d, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b,
0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e, 0x69, 0x16b, 0x70, 0x61, 0x75,
0x6e, 0x1e0d, 0x6c, 0x79, 0x101, 0x6e, 0x1e0d, 0x20, 0x1e0d, 0x65, 0x6c, 0x101,
0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e, 0x69, 0x16b, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x6e, 0x6f, 0x72, 0x70, 0x6f, 0x6b, 0x20, 0x64, 0x12b,
0x70, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e, 0x6f, 0x72, 0x70, 0x6f, 0x6b,
0x20, 0x64, 0x12b, 0x70, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e, 0x6f, 0x72, 0x70, 0x6f, 0x6b, 0x20,
0x64, 0x12b, 0x70, 0x20, 0x1e0d, 0x65, 0x6c, 0x101, 0x69, 0x1e6d, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x70, 0x61, 0x72, 0x6e, 0x7e, 0x1e0d, 0x6f, 0x20, 0x1e0d,
0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6f, 0x68, 0x101, 0x7e, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x70, 0x61, 0x72, 0x6e, 0x7e, 0x1e0d, 0x6f, 0x20, 0x1e0d, 0x65,
0x20, 0x6e, 0x6f, 0x72, 0x6f, 0x68, 0x101, 0x7e, 0x20, 0x6d, 0x101, 0x6e,
0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x61, 0x72,
0x6e, 0x7e, 0x1e0d, 0x6f, 0x20, 0x1e0d, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6f,
0x68, 0x101, 0x7e, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x6e, 0x61, 0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x73, 0x72,
0x6b, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e, 0x61, 0x77, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x73, 0x72, 0x6b, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b,
0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e, 0x61, 0x77, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x73, 0x72, 0x6b, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x61, 0x6d, 0x73, 0x6b, 0x61, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x61, 0x6d, 0x73, 0x6b, 0x61, 0x20, 0x6d, 0x101, 0x6e, 0x101,
0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x61, 0x6d, 0x73, 0x6b,
0x61, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x70, 0x101, 0x6b, 0x69, 0x73, 0x74, 0x101, 0x6e, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x70, 0x101, 0x6b, 0x69, 0x73, 0x74, 0x101, 0x6e, 0x20, 0x6d, 0x101,
0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x101,
0x6b, 0x69, 0x73, 0x74, 0x101, 0x6e, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x101, 0x6c, 0x101, 0x75, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x70, 0x101, 0x70, 0x75, 0x101, 0x20, 0x6e, 0x69, 0x75,
0x20, 0x67, 0x75, 0x6e, 0x12b, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70,
0x101, 0x72, 0x101, 0x67, 0x75, 0x79, 0x113, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x70, 0x101, 0x72, 0x101, 0x67, 0x75, 0x79, 0x113, 0x20, 0x6d, 0x101, 0x6e,
0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x101, 0x72,
0x101, 0x67, 0x75, 0x79, 0x113, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x70, 0x65, 0x72, 0x75, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x70, 0x65, 0x72, 0x75, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b,
0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x65, 0x72, 0x75, 0x20, 0x6b,
0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x69, 0x6c,
0x69, 0x70, 0x69, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x69, 0x6c,
0x69, 0x70, 0x69, 0x6e, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e,
0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70,
0x69, 0x6e, 0x69, 0x6b, 0x73, 0x61, 0x20, 0x64, 0x12b, 0x70, 0x61, 0x74,
0x69, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x73, 0x113, 0x6e, 0x1e6d, 0x20, 0x70,
0x69, 0x65, 0x72, 0x113, 0x20, 0x61, 0x1e0d, 0x113, 0x20, 0x6d, 0x65, 0x6b,
0x77, 0x113, 0x6c, 0x101, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x73, 0x113,
0x6e, 0x1e6d, 0x20, 0x70, 0x69, 0x65, 0x72, 0x113, 0x20, 0x61, 0x1e0d, 0x113,
0x20, 0x6d, 0x65, 0x6b, 0x77, 0x113, 0x6c, 0x101, 0x6e, 0x20, 0x6d, 0x101,
0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x73, 0x113,
0x6e, 0x1e6d, 0x20, 0x70, 0x69, 0x65, 0x72, 0x113, 0x20, 0x61, 0x1e0d, 0x113,
0x20, 0x6d, 0x65, 0x6b, 0x77, 0x113, 0x6c, 0x101, 0x6e, 0x20, 0x1e0d, 0x65,
0x6c, 0x101, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x69, 0x1e6d,
0x6b, 0x113, 0x72, 0x61, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x6f,
0x6e, 0x101, 0x70, 0x65, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x79, 0x6f,
0x6e, 0x67, 0x61, 0x79, 0x101, 0x6e, 0x67, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x72, 0x12b, 0x16b, 0x6e, 0x12b, 0x61, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x72, 0x6f, 0x74, 0x113, 0x72, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x73,
0x61, 0x6b, 0x101, 0x6c, 0x69, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x73,
0x61, 0x6b, 0x101, 0x6c, 0x69, 0x6e, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e,
0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x73, 0x61, 0x6b, 0x101, 0x6c,
0x69, 0x6e, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x73, 0x61, 0x101, 0x6d, 0x6f, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x73, 0x61, 0x101, 0x6d, 0x6f, 0x61, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e,
0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x73, 0x61, 0x101, 0x6d, 0x6f,
0x61, 0x20, 0x1e0d, 0x65, 0x6c, 0x101, 0x69, 0x1e6d, 0x73, 0x113, 0x73, 0x65,
0x6c, 0x73, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x73, 0x69, 0x6e, 0x67, 0x101,
0x70, 0x75, 0x72, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x73, 0x6f, 0x6c, 0x6f, 0x6d, 0x61, 0x6e, 0x20,
0x64, 0x12b, 0x70, 0x61, 0x74, 0x69, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x64,
0x6b, 0x12b, 0x1e47, 0x61, 0x20, 0x6a, 0x61, 0x72, 0x6a, 0x12b, 0x101, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x73, 0x75, 0x72, 0x12b, 0x6e, 0x101, 0x6d, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x73, 0x61, 0x77, 0x101, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x74, 0x101, 0x68, 0x69, 0x74, 0x69, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x74, 0x101, 0x69, 0x70, 0x113, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x74, 0x101,
0x69, 0x70, 0x113, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x74, 0x101, 0x69, 0x70, 0x113, 0x20, 0x1e0d, 0x65,
0x6c, 0x101, 0x69, 0x1e6d, 0x74, 0x101, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74,
0x101, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x1e6d, 0x6f, 0x6b, 0x65, 0x6c,
0x101, 0x75, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x1e6d, 0x61, 0x6e, 0x67, 0x101,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x1e6d, 0x61, 0x6e, 0x67, 0x101, 0x20, 0x6d,
0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x1e6d,
0x61, 0x6e, 0x67, 0x101, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x63, 0x75, 0x6b, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x74,
0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x101, 0x6e, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x101, 0x6e, 0x20, 0x6d, 0x61, 0x6e, 0x61, 0x6b, 0x61, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x101, 0x6e, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x74, 0x75, 0x77, 0x61, 0x6c, 0x16b, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x75, 0x72, 0x67, 0x75, 0x77, 0x113, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x75, 0x72, 0x67, 0x75, 0x77, 0x113, 0x20, 0x6d, 0x61, 0x6e, 0x61,
0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x75, 0x72, 0x67, 0x75, 0x77,
0x113, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x75, 0x7a, 0x77, 0x113, 0x6b, 0x69, 0x73, 0x74, 0x101, 0x6e, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x75, 0x7a, 0x77, 0x113, 0x6b, 0x69, 0x73, 0x74, 0x101,
0x6e, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x75, 0x7a, 0x77, 0x113, 0x6b, 0x69, 0x73, 0x74, 0x101, 0x6e,
0x20, 0x6b, 0x101, 0x72, 0xe3, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x61,
0x6e, 0x75, 0x101, 0x74, 0x16b, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x61,
0x6e, 0x75, 0x101, 0x74, 0x16b, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b,
0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x61, 0x6e, 0x75, 0x101, 0x74,
0x16b, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x77, 0x65, 0x6e, 0x113, 0x7a, 0x75, 0x65, 0x6c, 0x101, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x77, 0x6c, 0x101, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x61,
0x6b, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x6c, 0x101, 0x64, 0x69, 0x77,
0x6f, 0x73, 0x74, 0x61, 0x6b, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b,
0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x6c, 0x101, 0x64, 0x69, 0x77,
0x6f, 0x73, 0x74, 0x61, 0x6b, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x77, 0x101, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x101,
0x1e0d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x101, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x101, 0x1e0d, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x77, 0x101, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x101,
0x1e0d, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x77, 0x61, 0x6b, 0x61, 0x20, 0x64, 0x12b, 0x70, 0x61, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x77, 0x61, 0x6c, 0x69, 0x73, 0x20, 0x61, 0x1e0d, 0x113, 0x20,
0x70, 0x75, 0x1e6d, 0x75, 0x6e, 0x101, 0x79, 0x61, 0x6b, 0x75, 0x73, 0x74,
0x75, 0x6b, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x79, 0x61, 0x6b, 0x75, 0x73,
0x74, 0x75, 0x6b, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x79, 0x61, 0x6b, 0x75, 0x73, 0x74, 0x75, 0x6b,
0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x79,
0x61, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x61, 0x72, 0x67,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x79, 0x61, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x61, 0x72, 0x67, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e,
0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x79, 0x61, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x61, 0x72, 0x67, 0x20, 0x6b, 0x101, 0x72,
0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x906, 0x92a, 0x917, 0x93e, 0x928,
0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e,
0x93e, 0x926, 0x93f, 0x928, 0x940, 0x20, 0x906, 0x92a, 0x94d, 0x930, 0x93f, 0x915,
0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20,
0x939, 0x949, 0x92a, 0x941, 0x20, 0x906, 0x92a, 0x94d, 0x930, 0x93f, 0x915, 0x93e,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x926, 0x915, 0x94d, 0x937, 0x93f, 0x923, 0x20,
0x906, 0x92a, 0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x93e,
0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x938, 0x94d, 0x91a,
0x93f, 0x92e, 0x20, 0x906, 0x92a, 0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x92a, 0x938, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x906, 0x92a,
0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915,
0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x938, 0x94d, 0x91a, 0x93f, 0x92e,
0x20, 0x906, 0x92a, 0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x20, 0x915, 0x93e, 0x930,
0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x905, 0x932, 0x93e, 0x938, 0x94d, 0x915, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x905, 0x932, 0x93e, 0x938, 0x94d, 0x200d, 0x915, 0x93e, 0x20, 0x92e, 0x93e, 0x928,
0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x905, 0x932, 0x93e,
0x938, 0x94d, 0x200d, 0x915, 0x93e, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x905, 0x92e, 0x947, 0x91c, 0x93c, 0x928, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x905, 0x92e, 0x947, 0x91c, 0x93c, 0x928, 0x20, 0x92e,
0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x905,
0x92e, 0x947, 0x91c, 0x93c, 0x928, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20,
0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e, 0x93e, 0x926,
0x93f, 0x928, 0x940, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e, 0x93e, 0x926, 0x93f,
0x928, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x92e, 0x93e, 0x926, 0x93f, 0x928, 0x940, 0x20, 0x921, 0x947,
0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x947, 0x921,
0x93c, 0x93e, 0x20, 0x939, 0x949, 0x92a, 0x941, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x939, 0x949, 0x92a, 0x941, 0x20, 0x92e,
0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935,
0x947, 0x921, 0x93c, 0x93e, 0x20, 0x939, 0x949, 0x92a, 0x941, 0x20, 0x921, 0x947,
0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x939, 0x949, 0x930,
0x94d, 0x915, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x939, 0x949, 0x930, 0x94d,
0x915, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x939, 0x949, 0x930, 0x94d, 0x915, 0x93e, 0x20, 0x921, 0x947,
0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x947, 0x938,
0x93f, 0x92a, 0x93f, 0x915, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x947, 0x938,
0x93f, 0x92a, 0x93f, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x947, 0x938, 0x93f, 0x92a, 0x93f, 0x915,
0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x90f, 0x92a, 0x93f, 0x906, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x90f, 0x92a, 0x93f,
0x906, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x90f, 0x92a, 0x93f, 0x906, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907,
0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x905, 0x930, 0x92c, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x905, 0x930, 0x92c, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915,
0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x905, 0x930, 0x92c, 0x20, 0x921, 0x947,
0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x905, 0x930, 0x94d,
0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20, 0x92e,
0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x905,
0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20, 0x915, 0x93e,
0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x915, 0x941, 0x923, 0x94d, 0x92a,
0x941, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x915,
0x941, 0x923, 0x94d, 0x92a, 0x941, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902,
0x91f, 0x940, 0x928, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x915,
0x941, 0x923, 0x94d, 0x92a, 0x941, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902,
0x91f, 0x940, 0x928, 0x93e, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e,
0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x906, 0x930, 0x94d, 0x92e,
0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x906, 0x930,
0x94d, 0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x93e,
0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x906, 0x930, 0x94d, 0x92e,
0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20,
0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x905, 0x91f, 0x932,
0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x905, 0x91f,
0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902,
0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x905, 0x91f, 0x932, 0x93e, 0x902,
0x91f, 0x93f, 0x915, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x92e, 0x93e, 0x926, 0x93f, 0x928, 0x940, 0x20, 0x911, 0x938,
0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x924, 0x93f, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x911, 0x938, 0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947,
0x932, 0x93f, 0x92f, 0x93e, 0x924, 0x93f, 0x20, 0x92e, 0x93e, 0x926, 0x93f, 0x928,
0x940, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x911, 0x938, 0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f,
0x92f, 0x93e, 0x924, 0x93f, 0x20, 0x92e, 0x93e, 0x926, 0x93f, 0x928, 0x940, 0x20,
0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x911,
0x938, 0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x924,
0x93f, 0x20, 0x92e, 0x93e, 0x926, 0x93f, 0x928, 0x940, 0x20, 0x935, 0x947, 0x921,
0x93c, 0x93e, 0x20, 0x915, 0x941, 0x923, 0x94d, 0x92a, 0x941, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x911, 0x938, 0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f,
0x92f, 0x93e, 0x924, 0x93f, 0x20, 0x92e, 0x93e, 0x926, 0x93f, 0x928, 0x940, 0x20,
0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x915, 0x941, 0x923, 0x94d, 0x92a, 0x941,
0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x911, 0x938, 0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f,
0x93e, 0x924, 0x93f, 0x20, 0x92e, 0x93e, 0x926, 0x93f, 0x928, 0x940, 0x20, 0x935,
0x947, 0x921, 0x93c, 0x93e, 0x20, 0x915, 0x941, 0x923, 0x94d, 0x92a, 0x941, 0x20,
0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935,
0x947, 0x921, 0x93c, 0x93e, 0x20, 0x939, 0x949, 0x92a, 0x941, 0x20, 0x911, 0x938,
0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x911, 0x938, 0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f,
0x92f, 0x93e, 0x924, 0x93f, 0x20, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x939,
0x949, 0x92a, 0x941, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x911, 0x938, 0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947,
0x932, 0x93f, 0x92f, 0x93e, 0x924, 0x93f, 0x20, 0x935, 0x947, 0x921, 0x93c, 0x93e,
0x20, 0x939, 0x949, 0x92a, 0x941, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x915,
0x941, 0x923, 0x94d, 0x92a, 0x941, 0x20, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930,
0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x911, 0x938,
0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x924, 0x93f, 0x20,
0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x915, 0x941, 0x923, 0x94d, 0x92a, 0x941,
0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e,
0x924, 0x93f, 0x20, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x915, 0x941, 0x923,
0x94d, 0x92a, 0x941, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x905, 0x91c, 0x930, 0x92c, 0x93e, 0x907, 0x91c, 0x93e, 0x928,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x905, 0x91c, 0x930, 0x92c, 0x93e, 0x907, 0x91c,
0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x905, 0x91c, 0x930, 0x92c, 0x93e, 0x907, 0x91c, 0x93e, 0x928,
0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x905, 0x91c, 0x93c, 0x94b, 0x930, 0x947, 0x938, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x905, 0x91c, 0x93c, 0x94b, 0x930, 0x947, 0x938, 0x20,
0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x905, 0x91c, 0x93c, 0x94b, 0x930, 0x947, 0x938, 0x20, 0x915, 0x93e, 0x930, 0x93e,
0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92c,
0x93e, 0x902, 0x917, 0x94d, 0x932, 0x93e, 0x926, 0x947, 0x938, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x92c, 0x93e, 0x902, 0x917, 0x94d, 0x932, 0x93e, 0x926, 0x947, 0x938,
0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x92c, 0x93e, 0x902, 0x917, 0x94d, 0x932, 0x93e, 0x926, 0x947, 0x938, 0x20,
0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x92c, 0x941, 0x91f, 0x93e, 0x928, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x92c, 0x94b, 0x932, 0x940, 0x935, 0x93f, 0x92f, 0x93e, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x93c, 0x940, 0x932, 0x93f, 0x92f,
0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x93c,
0x940, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915,
0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x93c,
0x940, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20,
0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92c, 0x94d, 0x930,
0x942, 0x928, 0x947, 0x924, 0x93f, 0x20, 0x926, 0x93e, 0x930, 0x942, 0x938, 0x94d,
0x938, 0x932, 0x92e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x915, 0x947, 0x92a, 0x20,
0x935, 0x930, 0x94d, 0x921, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x915, 0x947, 0x92a,
0x20, 0x935, 0x930, 0x94d, 0x921, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915,
0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x930,
0x94d, 0x921, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938,
0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x91a, 0x93e, 0x92e, 0x94b, 0x930, 0x94b,
0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x91a, 0x94d, 0x92f, 0x93e, 0x924, 0x93e, 0x92e, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x91a, 0x94d, 0x92f, 0x93e, 0x924, 0x93e, 0x92e, 0x20, 0x92e, 0x93e, 0x928,
0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x91a, 0x94d, 0x92f,
0x93e, 0x924, 0x93e, 0x92e, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x91a, 0x93f, 0x932, 0x940, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x91a, 0x93f, 0x932, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915,
0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x91a, 0x93f, 0x932, 0x940, 0x20, 0x915,
0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x91a, 0x940, 0x928, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x91a, 0x940,
0x928, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x91a, 0x940, 0x928, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x915, 0x94d, 0x930, 0x93f, 0x938, 0x92e, 0x938,
0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x915,
0x94b, 0x915, 0x94b, 0x938, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x915, 0x94b, 0x932, 0x902, 0x92c, 0x93f, 0x92f, 0x93e, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x915, 0x94b, 0x932, 0x902, 0x92c, 0x93f, 0x92f, 0x93e,
0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x915, 0x94b, 0x932, 0x902, 0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x915, 0x93e,
0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x915, 0x941, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x915, 0x941, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a,
0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x915, 0x941, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x906,
0x927, 0x93e, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938,
0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907,
0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x921, 0x947, 0x935, 0x93f, 0x938, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x921, 0x94d, 0x92f, 0x942, 0x92e, 0x94b, 0x902, 0x91f,
0x20, 0x921, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x935, 0x93f, 0x932, 0x947, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x939, 0x949,
0x92a, 0x941, 0x20, 0x924, 0x93f, 0x92e, 0x94b, 0x930, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x908, 0x938, 0x94d, 0x91f, 0x930, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x908, 0x938, 0x94d, 0x91f, 0x930, 0x20, 0x926,
0x94d, 0x935, 0x940, 0x92a, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x908, 0x938, 0x94d, 0x91f, 0x930, 0x20, 0x926,
0x94d, 0x935, 0x940, 0x92a, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e,
0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x907, 0x915, 0x94d, 0x935,
0x93e, 0x921, 0x94b, 0x930, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e, 0x93e, 0x926,
0x93f, 0x928, 0x940, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x2d, 0x924, 0x93f,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e, 0x93e, 0x926, 0x93f, 0x928, 0x940, 0x20,
0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x2d, 0x924, 0x93f, 0x20, 0x92e, 0x93e, 0x928,
0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e, 0x93e, 0x926,
0x93f, 0x928, 0x940, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x2d, 0x924, 0x93f,
0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x939, 0x949,
0x92a, 0x941, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x2d, 0x924, 0x93f, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x939, 0x949,
0x92a, 0x941, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x2d, 0x924, 0x93f, 0x20,
0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x939, 0x949, 0x92a, 0x941, 0x20, 0x92f,
0x941, 0x930, 0x94b, 0x92a, 0x2d, 0x924, 0x93f, 0x20, 0x915, 0x93e, 0x930, 0x93e,
0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x911,
0x930, 0x949, 0x20, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x939, 0x949, 0x92a,
0x941, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x2d, 0x924, 0x93f, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x915, 0x941, 0x923,
0x94d, 0x92a, 0x941, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x2d, 0x924, 0x93f,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x915,
0x941, 0x923, 0x94d, 0x92a, 0x941, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x2d,
0x924, 0x93f, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x915, 0x941, 0x923,
0x94d, 0x92a, 0x941, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x2d, 0x924, 0x93f,
0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x93e, 0x915, 0x20, 0x2d, 0x932, 0x947, 0x902,
0x921, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x92a, 0x93e, 0x915, 0x20, 0x2d, 0x932, 0x947, 0x902, 0x921, 0x20, 0x926, 0x94d,
0x935, 0x940, 0x92a, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x93e, 0x915, 0x20, 0x2d, 0x932, 0x947, 0x902,
0x921, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x915, 0x93e, 0x930, 0x93e,
0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92a,
0x93c, 0x93f, 0x91c, 0x940, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x93c, 0x93f,
0x91c, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x92a, 0x93c, 0x93f, 0x91c, 0x940, 0x20, 0x915, 0x93e, 0x930,
0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x92a, 0x93c, 0x94d, 0x930, 0x947, 0x902, 0x91a, 0x20, 0x917, 0x941, 0x92f, 0x93e,
0x928, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x926, 0x949, 0x915, 0x93f, 0x923,
0x949, 0x20, 0x92a, 0x93c, 0x94d, 0x930, 0x93e, 0x902, 0x938, 0x20, 0x911, 0x921,
0x93c, 0x947, 0x20, 0x905, 0x902, 0x91f, 0x93e, 0x930, 0x94d, 0x915, 0x91f, 0x93f,
0x915, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x917, 0x94d, 0x92f, 0x93e, 0x932, 0x93e,
0x92a, 0x93e, 0x917, 0x94b, 0x938, 0x20, 0x924, 0x93f, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x917, 0x94d, 0x92f, 0x93e, 0x92e, 0x94d, 0x92c, 0x940, 0x92f, 0x930, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x915,
0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x917, 0x93f, 0x932, 0x94d, 0x92c, 0x930, 0x94d, 0x91f, 0x20, 0x926,
0x94d, 0x935, 0x940, 0x92a, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x917, 0x94d, 0x930,
0x940, 0x928, 0x935, 0x93f, 0x91a, 0x20, 0x92e, 0x940, 0x928, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x939, 0x949, 0x92a, 0x941,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x947, 0x902, 0x921, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x939, 0x949, 0x92a,
0x941, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x947, 0x902, 0x921, 0x20,
0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x939, 0x949, 0x92a, 0x941, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x928, 0x932, 0x947, 0x902, 0x921, 0x20, 0x915, 0x93e, 0x930,
0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x915, 0x941, 0x923, 0x94d, 0x92a, 0x941,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x947, 0x902, 0x921, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x915, 0x941, 0x923,
0x94d, 0x92a, 0x941, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x947, 0x902,
0x921, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x915, 0x941, 0x923, 0x94d,
0x92a, 0x941, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x947, 0x902, 0x921,
0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x917, 0x932, 0x94d, 0x92a, 0x20, 0x92e, 0x93e, 0x928,
0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x939, 0x935, 0x93e,
0x924, 0x93f, 0x2013, 0x906, 0x932, 0x94d, 0x92f, 0x942, 0x938, 0x928, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x939, 0x935, 0x93e, 0x924, 0x93f, 0x2013, 0x906, 0x932, 0x94d,
0x92f, 0x942, 0x938, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x939, 0x935, 0x93e, 0x924, 0x93f, 0x2013, 0x906,
0x932, 0x94d, 0x92f, 0x942, 0x938, 0x928, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907,
0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x939, 0x93e, 0x901, 0x917, 0x20, 0x915,
0x93e, 0x901, 0x917, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x939, 0x93e, 0x901, 0x917,
0x20, 0x915, 0x93e, 0x901, 0x917, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915,
0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x939, 0x93e, 0x901, 0x917, 0x20, 0x915,
0x93e, 0x901, 0x917, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e,
0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x939, 0x94b, 0x935, 0x94d, 0x921,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x939, 0x94b, 0x935, 0x94d, 0x921, 0x20, 0x92e,
0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x939,
0x94b, 0x935, 0x94d, 0x921, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e,
0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92c, 0x93e, 0x930, 0x924,
0x940, 0x92f, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x92c, 0x93e, 0x930, 0x949, 0x924, 0x20, 0x915, 0x93e, 0x91c,
0x93e, 0x20, 0x938, 0x93e, 0x92e, 0x941, 0x926, 0x94d, 0x930, 0x940, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x907, 0x902, 0x921, 0x94b, 0x91a, 0x93e, 0x907, 0x928, 0x93e,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x907, 0x902,
0x921, 0x94b, 0x928, 0x947, 0x938, 0x93f, 0x92f, 0x93e, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x939, 0x949, 0x92a, 0x941, 0x20,
0x907, 0x902, 0x921, 0x94b, 0x928, 0x947, 0x938, 0x93f, 0x92f, 0x93e, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x915, 0x941, 0x923,
0x94d, 0x92a, 0x941, 0x20, 0x907, 0x902, 0x921, 0x94b, 0x928, 0x947, 0x938, 0x93f,
0x92f, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x907, 0x930, 0x93e, 0x928, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x907, 0x930, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928,
0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x924, 0x93f, 0x930,
0x93e, 0x928, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x907, 0x930, 0x94d, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x907, 0x930, 0x94d, 0x915, 0x941, 0x924, 0x94d,
0x938, 0x94d, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x907, 0x930, 0x94d, 0x915, 0x941, 0x924, 0x94d, 0x938,
0x94d, 0x915, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938,
0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x907, 0x91c, 0x93c, 0x930, 0x93e, 0x907,
0x932, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x907, 0x91c, 0x93c, 0x930, 0x93e, 0x907,
0x932, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x907, 0x91c, 0x93c, 0x930, 0x93e, 0x907, 0x932, 0x20, 0x921, 0x947,
0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x91c, 0x93e, 0x92a,
0x93e, 0x928, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x91c, 0x93e, 0x92a, 0x93e, 0x928,
0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x91c, 0x93e, 0x92a, 0x93e, 0x928, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907,
0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x915, 0x91c, 0x93c, 0x93e, 0x915, 0x938,
0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x947, 0x921,
0x93c, 0x93e, 0x20, 0x939, 0x949, 0x92a, 0x941, 0x20, 0x915, 0x91c, 0x93c, 0x93e,
0x915, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935,
0x947, 0x921, 0x93c, 0x93e, 0x20, 0x915, 0x941, 0x923, 0x94d, 0x92a, 0x941, 0x20,
0x915, 0x91c, 0x93c, 0x93e, 0x915, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x915, 0x94b, 0x930, 0x93f, 0x92f, 0x93e, 0x924, 0x93f, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x915, 0x94b, 0x930, 0x93f, 0x92f, 0x93e, 0x924, 0x93f,
0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x915, 0x94b, 0x930, 0x93f, 0x92f, 0x93e, 0x924, 0x93f, 0x20, 0x921, 0x947,
0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x915, 0x94b, 0x938,
0x930, 0x93e, 0x90f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x915, 0x94d, 0x930, 0x93e,
0x938, 0x94d, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x915, 0x94d, 0x930, 0x93e, 0x938, 0x94d, 0x928, 0x94b,
0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x93e,
0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x915, 0x94d, 0x930, 0x93e,
0x938, 0x94d, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20,
0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x915, 0x93f, 0x930, 0x94d, 0x917, 0x93f, 0x938, 0x94d, 0x200d,
0x924, 0x93e, 0x928, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x932, 0x93e, 0x907, 0x928,
0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x932,
0x949, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x94b, 0x935, 0x947, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x932, 0x949, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x94b, 0x935, 0x947,
0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x932, 0x949, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x94b, 0x935, 0x947, 0x20,
0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e,
0x93e, 0x917, 0x93e, 0x926, 0x93e, 0x928, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e,
0x93e, 0x917, 0x93e, 0x926, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902,
0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e, 0x93e, 0x917, 0x93e, 0x926,
0x93e, 0x928, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938,
0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e, 0x932, 0x947, 0x938, 0x93f, 0x92f,
0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e, 0x93e, 0x932, 0x926, 0x940, 0x935,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e, 0x93e, 0x930, 0x94d, 0x915, 0x947, 0x938,
0x938, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e, 0x93e, 0x930, 0x94d, 0x938, 0x932,
0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e,
0x949, 0x930, 0x940, 0x938, 0x938, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e, 0x949,
0x930, 0x940, 0x938, 0x938, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e, 0x949, 0x930, 0x940, 0x938, 0x938, 0x20,
0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x92e, 0x93e, 0x935, 0x94d, 0x938, 0x928, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x94b, 0x924, 0x93f,
0x20, 0x92a, 0x947, 0x938, 0x93f, 0x92a, 0x93f, 0x915, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x94b, 0x924, 0x93f, 0x20,
0x92a, 0x947, 0x938, 0x93f, 0x92a, 0x93f, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x93e,
0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e, 0x947, 0x915, 0x94d,
0x938, 0x93f, 0x915, 0x94b, 0x924, 0x93f, 0x20, 0x92a, 0x947, 0x938, 0x93f, 0x92a,
0x93f, 0x915, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x909, 0x932, 0x93e, 0x928, 0x20, 0x92c, 0x91f, 0x94b, 0x930, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x909, 0x932, 0x93e, 0x928, 0x20, 0x92c, 0x91f, 0x94b,
0x930, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x909, 0x932, 0x93e, 0x928, 0x20, 0x92c, 0x91f, 0x94b, 0x930, 0x20,
0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x92e, 0x949, 0x938, 0x94d, 0x915, 0x94b, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x92e, 0x949, 0x938, 0x94d, 0x915, 0x94b, 0x20, 0x92e, 0x93e, 0x928,
0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e, 0x949, 0x938,
0x94d, 0x915, 0x94b, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e,
0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e, 0x94d, 0x92f, 0x93e, 0x902,
0x92e, 0x93e, 0x930, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x928, 0x949, 0x909, 0x930,
0x941, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x928, 0x947, 0x92a, 0x93e, 0x932, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x915, 0x947, 0x932,
0x947, 0x921, 0x94b, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x928, 0x94d, 0x92f, 0x942, 0x20, 0x915, 0x947, 0x932, 0x947, 0x921, 0x94b, 0x928,
0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x915, 0x947, 0x932,
0x947, 0x921, 0x94b, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x915, 0x93e, 0x930, 0x93e,
0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x928,
0x94d, 0x92f, 0x942, 0x91c, 0x93c, 0x940, 0x932, 0x947, 0x902, 0x921, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x928, 0x94d, 0x92f, 0x942, 0x91c, 0x93c, 0x940, 0x932, 0x947,
0x902, 0x921, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x928, 0x94d, 0x92f, 0x942, 0x91c, 0x93c, 0x940, 0x932, 0x947,
0x902, 0x921, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x928, 0x94d, 0x92f, 0x942, 0x92a, 0x93c, 0x93e, 0x909, 0x902, 0x921,
0x932, 0x947, 0x902, 0x921, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x928, 0x94d, 0x92f,
0x942, 0x92a, 0x93c, 0x93e, 0x909, 0x902, 0x921, 0x932, 0x947, 0x902, 0x921, 0x20,
0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x928, 0x94d, 0x92f, 0x942, 0x92a, 0x93c, 0x93e, 0x909, 0x902, 0x921, 0x932, 0x947,
0x902, 0x921, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x928, 0x940, 0x92f, 0x942, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x928,
0x949, 0x930, 0x92a, 0x93c, 0x949, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x928, 0x949, 0x930, 0x92a, 0x93c, 0x949, 0x915,
0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902,
0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x928, 0x949, 0x930, 0x92a, 0x93c,
0x949, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x921, 0x947, 0x932,
0x93e, 0x907, 0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x93c, 0x930, 0x94d,
0x928, 0x93e, 0x902, 0x930, 0x94d, 0x921, 0x94b, 0x20, 0x921, 0x947, 0x20, 0x928,
0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x92a, 0x93c, 0x930, 0x94d, 0x928, 0x93e, 0x902, 0x930, 0x94d, 0x921, 0x94b, 0x20,
0x921, 0x947, 0x20, 0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x92a, 0x93c, 0x930, 0x94d, 0x928, 0x93e, 0x902, 0x930, 0x94d, 0x921, 0x94b, 0x20,
0x921, 0x947, 0x20, 0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x20,
0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x928, 0x94b, 0x935, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930,
0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x928, 0x94b, 0x935,
0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92e,
0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x928,
0x94b, 0x935, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915,
0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x913, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x913, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92e, 0x93e,
0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x913, 0x92e,
0x94d, 0x938, 0x94d, 0x915, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e,
0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x93e, 0x915, 0x93f,
0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x93e,
0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x93e,
0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x93e, 0x915, 0x93f,
0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20,
0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x932, 0x93e,
0x909, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x93e, 0x92a, 0x941, 0x906, 0x20,
0x928, 0x94d, 0x92f, 0x942, 0x20, 0x917, 0x93f, 0x928, 0x940, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x92a, 0x948, 0x930, 0x93e, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x92a, 0x948, 0x930, 0x93e, 0x917, 0x94d, 0x935, 0x947, 0x20,
0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x92a, 0x948, 0x930, 0x93e, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x915, 0x93e, 0x930,
0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x92a, 0x947, 0x930, 0x942, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x947, 0x930,
0x942, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x92a, 0x947, 0x930, 0x942, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901,
0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x93c,
0x93f, 0x932, 0x93f, 0x92a, 0x940, 0x928, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92a,
0x93c, 0x93f, 0x932, 0x93f, 0x92a, 0x940, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x93e,
0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x93c, 0x93f, 0x932,
0x93f, 0x92a, 0x940, 0x928, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e,
0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x93c, 0x940, 0x928,
0x93f, 0x915, 0x94d, 0x938, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x92a, 0x93f, 0x90f, 0x930,
0x947, 0x20, 0x911, 0x921, 0x93c, 0x947, 0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x935,
0x947, 0x932, 0x93e, 0x928, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x938, 0x947, 0x902,
0x91f, 0x20, 0x92a, 0x93f, 0x90f, 0x930, 0x947, 0x20, 0x911, 0x921, 0x93c, 0x947,
0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932, 0x93e, 0x928, 0x20, 0x92e,
0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x938,
0x947, 0x902, 0x91f, 0x20, 0x92a, 0x93f, 0x90f, 0x930, 0x947, 0x20, 0x911, 0x921,
0x93c, 0x947, 0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932, 0x93e, 0x928,
0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x92a, 0x93f, 0x91f, 0x915, 0x947, 0x930, 0x94d, 0x928, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x92a, 0x94b, 0x928, 0x93e, 0x92a, 0x947, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x92a, 0x94d, 0x92f, 0x94b, 0x902, 0x917, 0x92f, 0x93e, 0x902, 0x917, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x930, 0x940, 0x92f, 0x942, 0x928, 0x93f, 0x92f, 0x928, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x930, 0x94b, 0x925, 0x947, 0x930, 0x93e, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x938, 0x915, 0x93e, 0x932, 0x93f, 0x928, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x938, 0x915, 0x93e, 0x932, 0x93f, 0x928, 0x20, 0x92e, 0x93e, 0x928,
0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x938, 0x915, 0x93e,
0x932, 0x93f, 0x928, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e,
0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x938, 0x92e, 0x94b, 0x906, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x938, 0x92e, 0x94b, 0x906, 0x20, 0x92e, 0x93e, 0x928,
0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x938, 0x92e, 0x94b,
0x906, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x938, 0x947, 0x938, 0x947, 0x932, 0x94d, 0x938, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x938, 0x93f, 0x902, 0x917, 0x93e, 0x92a, 0x941, 0x930, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x938, 0x94b, 0x932, 0x94b, 0x92e, 0x928, 0x20, 0x926, 0x94d, 0x935,
0x940, 0x92a, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x926, 0x915, 0x94d, 0x937, 0x93f,
0x923, 0x940, 0x20, 0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x938, 0x942, 0x930, 0x940, 0x928, 0x93e, 0x92e, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x938, 0x94d, 0x92f, 0x94b, 0x935, 0x93e, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x924, 0x93e, 0x939, 0x93f, 0x924, 0x940, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x924, 0x93e, 0x907, 0x92a, 0x947, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x924, 0x93e, 0x907, 0x92a, 0x947, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915,
0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x924, 0x93e, 0x907, 0x92a, 0x947, 0x20,
0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x924,
0x93e, 0x91c, 0x93f, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x91f, 0x94b, 0x915, 0x947, 0x932, 0x93e, 0x90a, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x91f, 0x94b, 0x902, 0x917, 0x93e, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x91f, 0x94b, 0x902, 0x917, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902,
0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x91f, 0x94b, 0x902, 0x917, 0x93e,
0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x91a, 0x941, 0x915, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x924, 0x941, 0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928, 0x93f, 0x938, 0x94d, 0x924,
0x93e, 0x928, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x924, 0x941, 0x930, 0x94d, 0x915,
0x92e, 0x947, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92e, 0x93e,
0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x924, 0x941,
0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928,
0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x924, 0x941, 0x935, 0x93e, 0x932, 0x942, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x909, 0x930, 0x941, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x909, 0x930, 0x941, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x92e,
0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x909,
0x930, 0x941, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901,
0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x909, 0x91c,
0x93c, 0x94d, 0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x909, 0x91c, 0x93c, 0x94d, 0x92c, 0x947, 0x915, 0x93f,
0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915,
0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x909, 0x91c, 0x93c, 0x94d, 0x92c, 0x947,
0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x915, 0x93e, 0x930, 0x93e,
0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935,
0x928, 0x941, 0x906, 0x924, 0x942, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x928,
0x941, 0x906, 0x924, 0x942, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x928, 0x941, 0x906, 0x924, 0x942, 0x20,
0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x935, 0x947, 0x928, 0x947, 0x91c, 0x93c, 0x941, 0x90f, 0x932,
0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x94d, 0x932, 0x93e, 0x926, 0x93f,
0x935, 0x94b, 0x938, 0x94d, 0x924, 0x94b, 0x915, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x935, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x935, 0x94b, 0x938, 0x94d, 0x924, 0x94b,
0x915, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x935, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x935, 0x94b, 0x938, 0x94d,
0x924, 0x94b, 0x915, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e,
0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x94b, 0x932, 0x94d, 0x917,
0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x921, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935,
0x94b, 0x932, 0x94d, 0x917, 0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x921, 0x20, 0x92e,
0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935,
0x94b, 0x932, 0x94d, 0x917, 0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x921, 0x20, 0x915,
0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x935, 0x947, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x935, 0x93e, 0x932, 0x93f, 0x938, 0x20, 0x911, 0x921,
0x93c, 0x947, 0x20, 0x92a, 0x93c, 0x94d, 0x92f, 0x942, 0x91a, 0x942, 0x928, 0x93e,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92f, 0x93e, 0x915, 0x941, 0x924, 0x94d, 0x938,
0x94d, 0x915, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92f, 0x93e, 0x915, 0x941, 0x924,
0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92f, 0x93e, 0x915, 0x941, 0x924, 0x94d, 0x938,
0x94d, 0x915, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938,
0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92f, 0x947, 0x915, 0x93e, 0x924, 0x947,
0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x92f, 0x947, 0x915, 0x93e, 0x924, 0x947, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d,
0x917, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x92f, 0x947, 0x915, 0x93e, 0x924, 0x947, 0x930, 0x93f, 0x928, 0x92c,
0x930, 0x94d, 0x917, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e,
0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0xb06, 0xb2a, 0xb17, 0xb3e, 0xb28,
0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e,
0xb3e, 0xb26, 0xb3f, 0xb28, 0xb3f, 0x20, 0xb06, 0xb2a, 0xb4d, 0xb30, 0xb3f, 0xb15,
0xb3e, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0x20,
0xb39, 0xb2a, 0xb41, 0x20, 0xb06, 0xb2a, 0xb4d, 0xb30, 0xb3f, 0xb15, 0xb3e, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb26, 0xb15, 0xb3f, 0xb23, 0x20, 0xb06, 0xb2a, 0xb4d,
0xb30, 0xb3f, 0xb15, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0x20, 0xb15,
0xb41, 0xb23, 0xb4d, 0x200d, 0xb2a, 0xb41, 0x20, 0xb06, 0xb2a, 0xb4d, 0xb30, 0xb3f,
0xb15, 0xb3e, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e,
0x20, 0xb15, 0xb41, 0xb23, 0xb4d, 0x200d, 0xb2a, 0xb41, 0x20, 0xb06, 0xb2a, 0xb4d,
0xb30, 0xb3f, 0xb15, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0x20, 0xb15,
0xb41, 0xb23, 0xb4d, 0x200d, 0xb2a, 0xb41, 0x20, 0xb06, 0xb2a, 0xb4d, 0xb30, 0xb3f,
0xb15, 0xb3e, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb06, 0xb32, 0xb3e, 0xb38, 0xb4d, 0xb15, 0xb3e, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb06, 0xb32, 0xb3e, 0xb38, 0xb4d, 0xb15, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28,
0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb06, 0xb32, 0xb3e,
0xb38, 0xb4d, 0xb15, 0xb3e, 0x20, 0xb21, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0xb4d,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb06, 0xb2e, 0xb47, 0xb1c, 0xb28, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb06, 0xb2e, 0xb47, 0xb1c, 0xb28, 0x20, 0xb2e, 0xb3e, 0xb28,
0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb06, 0xb2e, 0xb47,
0xb1c, 0xb28, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb2e, 0xb3e, 0xb26, 0xb3f, 0xb28, 0xb3f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb2e, 0xb3e, 0xb26, 0xb3f, 0xb28, 0xb3f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19,
0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e, 0xb3e, 0xb26, 0xb3f, 0xb28,
0xb3f, 0x20, 0xb21, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0xb4d, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0xb39, 0xb2a, 0xb41, 0xb24, 0xb3f,
0x20, 0xb1c, 0xb3e, 0xb17, 0xb3e, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47,
0xb21, 0xb3c, 0xb3e, 0xb39, 0xb2a, 0xb41, 0x20, 0xb1c, 0xb3e, 0xb17, 0xb3e, 0xb24,
0xb3f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0xb39, 0xb2a, 0xb41, 0x20, 0xb1c,
0xb3e, 0xb17, 0xb3e, 0xb24, 0xb3f, 0x20, 0xb21, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f,
0xb4d, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb39, 0xb30, 0xb4d, 0xb15, 0xb3e, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb39, 0xb30, 0xb4d, 0xb15, 0xb3e, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb39, 0xb30,
0xb4d, 0xb15, 0xb3e, 0x20, 0xb21, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0xb4d, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb47, 0xb38, 0xb3f, 0xb2a, 0xb3f, 0xb15, 0xb4d,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb47, 0xb38, 0xb3f, 0xb2a, 0xb3f, 0xb15,
0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb2a, 0xb47, 0xb38, 0xb3f, 0xb2a, 0xb3f, 0xb15, 0xb4d, 0x20, 0xb21,
0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0xb4d, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb0f,
0xb2a, 0xb3f, 0xb06, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb0f, 0xb2a, 0xb3f, 0xb06,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb0f, 0xb2a, 0xb3f, 0xb06, 0x20, 0xb21, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f,
0xb4d, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb06, 0xb30, 0xb2c, 0xb3f, 0xb06, 0xb24,
0xb3f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb06, 0xb30, 0xb2c, 0xb3f, 0xb06, 0xb24,
0xb3f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb06, 0xb30, 0xb2c, 0xb3f, 0xb06, 0xb24, 0xb3f, 0x20, 0xb2e, 0xb47,
0xb26, 0xb3e, 0xb23, 0xb3e, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb06, 0xb30, 0xb1c,
0xb47, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb28, 0xb3e, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb06, 0xb30, 0xb1c, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb28, 0xb3e, 0x20, 0xb2e,
0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb06,
0xb30, 0xb4d, 0xb1c, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb28, 0xb3e, 0x20, 0xb15,
0xb3e, 0xb30, 0xb3e, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c,
0xb3e, 0x20, 0xb15, 0xb41, 0xb23, 0xb4d, 0x200d, 0xb2a, 0xb41, 0x20, 0xb06, 0xb30,
0xb4d, 0xb1c, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb28, 0xb3e, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0x20, 0xb15, 0xb41, 0xb23, 0xb4d,
0x200d, 0xb2a, 0xb41, 0x20, 0xb06, 0xb30, 0xb4d, 0xb1c, 0xb47, 0xb23, 0xb4d, 0xb1f,
0xb3f, 0xb28, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0x20, 0xb15, 0xb41,
0xb23, 0xb4d, 0x200d, 0xb2a, 0xb41, 0x20, 0xb06, 0xb30, 0xb4d, 0xb1c, 0xb47, 0xb23,
0xb4d, 0xb1f, 0xb3f, 0xb28, 0xb3e, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb06, 0xb30, 0xb2e, 0xb47, 0xb28, 0xb3f, 0xb5f, 0xb3e,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb06, 0xb30, 0xb2e, 0xb47, 0xb28, 0xb3f, 0xb5f,
0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb06, 0xb30, 0xb2e, 0xb47, 0xb28, 0xb3f, 0xb5f, 0xb3e, 0x20, 0xb15,
0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb06, 0xb1f, 0xb4d,
0xb32, 0xb3e, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb06, 0xb1f, 0xb4d, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb15, 0x20, 0xb2e,
0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb06,
0xb1f, 0xb4d, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb15, 0x20, 0xb21, 0xb47,
0xb32, 0xb3e, 0xb07, 0xb1f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e, 0xb3e, 0xb26,
0xb3f, 0xb28, 0xb3f, 0x20, 0xb05, 0xb38, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32,
0xb3f, 0xb06, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb05, 0xb38, 0xb4d, 0xb1f, 0xb4d,
0xb30, 0xb47, 0xb32, 0xb3f, 0xb06, 0xb24, 0xb3f, 0x20, 0xb2e, 0xb3e, 0xb26, 0xb3f,
0xb28, 0xb3f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb05, 0xb38, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f,
0xb06, 0xb24, 0xb3f, 0x20, 0xb2e, 0xb3e, 0xb26, 0xb3f, 0xb28, 0xb3f, 0x20, 0xb21,
0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb05, 0xb38,
0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f, 0xb06, 0xb24, 0xb3f, 0x20, 0xb2e,
0xb3e, 0xb26, 0xb3f, 0xb28, 0xb3f, 0x20, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0xb15,
0xb41, 0xb23, 0xb4d, 0x200d, 0xb2a, 0xb41, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb05,
0xb38, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f, 0xb06, 0xb24, 0xb3f, 0x20,
0xb2e, 0xb3e, 0xb26, 0xb3f, 0xb28, 0xb3f, 0x20, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e,
0xb15, 0xb41, 0xb23, 0xb4d, 0x200d, 0xb2a, 0xb41, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e,
0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb05, 0xb38, 0xb4d, 0xb1f,
0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f, 0xb06, 0xb24, 0xb3f, 0x20, 0xb2e, 0xb3e, 0xb26,
0xb3f, 0xb28, 0xb3f, 0x20, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0xb15, 0xb41, 0xb23,
0xb4d, 0x200d, 0xb2a, 0xb41, 0x20, 0xb21, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0xb39, 0xb2a, 0xb41,
0x20, 0xb05, 0xb38, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f, 0xb06, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb05, 0xb38, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32,
0xb3f, 0xb06, 0xb24, 0xb3f, 0x20, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0xb39, 0xb2a,
0xb41, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb05, 0xb38, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f, 0xb06,
0xb24, 0xb3f, 0x20, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0xb39, 0xb2a, 0xb41, 0x20,
0xb21, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71,
0xb47, 0xb21, 0xb3c, 0xb3e, 0xb15, 0xb41, 0xb23, 0xb4d, 0x200d, 0xb2a, 0xb41, 0x20,
0xb05, 0xb38, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f, 0xb06, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb05, 0xb38, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f,
0xb06, 0xb24, 0xb3f, 0x20, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0xb15, 0xb41, 0xb23,
0xb4d, 0x200d, 0xb2a, 0xb41, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb05, 0xb38, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47,
0xb32, 0xb3f, 0xb06, 0xb24, 0xb3f, 0x20, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0xb15,
0xb41, 0xb23, 0xb4d, 0x200d, 0xb2a, 0xb41, 0x20, 0xb21, 0xb47, 0xb32, 0xb3e, 0xb07,
0xb1f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb05, 0xb1c, 0xb47, 0xb30, 0xb2c, 0xb3e,
0xb07, 0xb1c, 0xb3e, 0xb28, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb05, 0xb1c, 0xb47,
0xb30, 0xb2c, 0xb3e, 0xb07, 0xb1c, 0xb3e, 0xb28, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e,
0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb05, 0xb1c, 0xb47, 0xb30,
0xb2c, 0xb3e, 0xb07, 0xb1c, 0xb3e, 0xb28, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb05, 0xb1c, 0xb30, 0xb47, 0xb38, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb05, 0xb1c, 0xb30, 0xb47, 0xb38, 0x20, 0xb2e, 0xb3e, 0xb28,
0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb05, 0xb1c, 0xb30,
0xb47, 0xb38, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb2c, 0xb3e, 0xb19, 0xb4d, 0xb17, 0xb32, 0xb3e, 0xb26, 0xb47, 0xb38, 0xb4d,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2c, 0xb3e, 0xb19, 0xb4d, 0xb17, 0xb32, 0xb3e,
0xb26, 0xb47, 0xb38, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2c, 0xb3e, 0xb19, 0xb4d, 0xb17, 0xb32, 0xb3e,
0xb26, 0xb47, 0xb38, 0xb4d, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb2c, 0xb41, 0xb1f, 0xb3e, 0xb28, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb2c, 0xb32, 0xb71, 0xb3f, 0xb06, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2c,
0xb4d, 0xb30, 0xb3e, 0xb1c, 0xb3f, 0xb32, 0xb3f, 0xb5f, 0xb3e, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb2c, 0xb4d, 0xb30, 0xb3e, 0xb1c, 0xb3f, 0xb32, 0xb3f, 0xb5f, 0xb3e,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2c, 0xb4d,
0xb30, 0xb3e, 0xb1c, 0xb3f, 0xb32, 0xb3f, 0xb5f, 0xb3e, 0x20, 0xb15, 0xb3e, 0xb30,
0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2c, 0xb4d, 0xb30, 0xb41, 0xb28,
0xb47, 0xb07, 0x20, 0xb26, 0xb3e, 0xb30, 0xb41, 0xb38, 0xb3e, 0xb32, 0xb3e, 0xb2e,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb15, 0xb47, 0xb2a, 0x20, 0xb2c, 0xb30, 0xb4d,
0xb21, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb15, 0xb47, 0xb2a, 0x20, 0xb2c, 0xb30,
0xb4d, 0xb21, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb15, 0xb47, 0xb2a, 0x20, 0xb2c, 0xb30, 0xb4d, 0xb21, 0x20,
0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb1a, 0xb2e,
0xb30, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb1a, 0xb3e, 0xb24, 0xb3e, 0xb2e, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb1a, 0xb3e, 0xb24, 0xb3e, 0xb2e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb1a, 0xb3e, 0xb24, 0xb3e, 0xb2e, 0x20,
0xb21, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0xb4d, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb1a, 0xb3f, 0xb32, 0xb3f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb1a, 0xb3f, 0xb32,
0xb3f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb1a, 0xb3f, 0xb32, 0xb3f, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb1a, 0xb3f, 0xb28, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb1a, 0xb3f, 0xb28, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb1a, 0xb3f, 0xb28, 0x20, 0xb21, 0xb47, 0xb32,
0xb3e, 0xb07, 0xb1f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb15, 0xb4d, 0xb30, 0xb3f,
0xb38, 0xb2e, 0xb3e, 0xb38, 0x20, 0xb26, 0xb3f, 0xb2a, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb15, 0xb15, 0xb38, 0x20, 0xb26, 0xb3f, 0xb2a, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb15, 0xb32, 0xb2e, 0xb4d, 0xb2c, 0xb3f, 0xb5f, 0xb3e, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb15, 0xb32, 0xb2e, 0xb4d, 0xb2c, 0xb3f, 0xb5f, 0xb3e, 0x20, 0xb2e,
0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb15,
0xb32, 0xb2e, 0xb4d, 0xb2c, 0xb3f, 0xb5f, 0xb3e, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e,
0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb15, 0xb41, 0xb15, 0x20, 0xb26, 0xb3f,
0xb2a, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb15, 0xb41, 0xb15, 0x20, 0xb26, 0xb3f,
0xb2a, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb15, 0xb41, 0xb15, 0x20, 0xb26, 0xb3f, 0xb2a, 0x20, 0xb06, 0xb26,
0xb4d, 0xb26, 0xb3e, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb15, 0xb4d, 0xb71, 0xb41, 0xb2c, 0xb3e, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb15, 0xb4d, 0xb71, 0xb41, 0xb2c, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e,
0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb15, 0xb4d, 0xb71, 0xb41,
0xb2c, 0xb3e, 0x20, 0xb21, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0xb4d, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb21, 0xb47, 0xb2c, 0xb3f, 0xb38, 0xb4d, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb21, 0xb4d, 0xb5f, 0xb41, 0xb2e, 0xb3e, 0xb23, 0xb4d, 0xb1f, 0x20,
0xb21, 0xb3f, 0x20, 0xb05, 0xb30, 0xb71, 0xb3f, 0xb32, 0xb47, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0xb39, 0xb2a, 0xb41, 0x20, 0xb24,
0xb3f, 0xb2e, 0xb30, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb38, 0xb4d, 0xb1f,
0xb30, 0x20, 0xb26, 0xb3f, 0xb2a, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb38,
0xb4d, 0xb1f, 0xb30, 0x20, 0xb26, 0xb3f, 0xb2a, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e,
0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb38, 0xb4d, 0xb1f,
0xb30, 0x20, 0xb26, 0xb3f, 0xb2a, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb15, 0xb4d, 0xb71, 0xb3e, 0xb21, 0xb30, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e, 0xb3e, 0xb26, 0xb3f, 0xb28, 0xb3f, 0x20, 0xb5f,
0xb41, 0xb30, 0xb2a, 0xb3f, 0xb06, 0x20, 0xb24, 0xb3f, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb2e, 0xb3e, 0xb26, 0xb3f, 0xb28, 0xb3f, 0x20, 0xb5f, 0xb41, 0xb30, 0xb2a,
0xb3f, 0xb06, 0x20, 0xb24, 0xb3f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e, 0xb3e, 0xb26, 0xb3f, 0xb28, 0xb3e,
0x20, 0xb5f, 0xb41, 0xb30, 0xb2a, 0xb3f, 0xb06, 0x20, 0xb24, 0xb3f, 0x20, 0xb15,
0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21,
0xb3c, 0xb3e, 0x20, 0xb39, 0xb2a, 0xb41, 0x20, 0xb5f, 0xb41, 0xb30, 0xb2a, 0xb24,
0xb3f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0xb39,
0xb2a, 0xb41, 0x20, 0xb5f, 0xb41, 0xb30, 0xb2a, 0xb3f, 0xb06, 0xb24, 0xb3f, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0xb39, 0xb2a, 0xb41, 0x20, 0xb5f, 0xb41, 0xb30,
0xb2a, 0xb3f, 0xb06, 0xb24, 0xb3f, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb05, 0xb30, 0x20, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e,
0xb39, 0xb2a, 0xb41, 0x20, 0xb5f, 0xb41, 0xb30, 0xb2a, 0xb24, 0xb3f, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0xb15, 0xb41, 0xb23, 0xb4d,
0x200d, 0xb2a, 0xb41, 0x20, 0xb5f, 0xb41, 0xb30, 0xb2a, 0xb24, 0xb3f, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0xb15, 0xb41, 0xb23, 0xb4d,
0x200d, 0xb2a, 0xb41, 0x20, 0xb5f, 0xb41, 0xb30, 0xb2a, 0xb3f, 0xb06, 0xb24, 0xb3f,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0xb15, 0xb41, 0xb23, 0xb4d, 0x200d, 0xb2a,
0xb41, 0x20, 0xb5f, 0xb41, 0xb30, 0xb2a, 0xb24, 0xb3f, 0x20, 0xb15, 0xb3e, 0xb30,
0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb3e, 0xb15, 0xb32, 0xb3e,
0xb23, 0xb4d, 0xb21, 0x20, 0xb26, 0xb3f, 0xb2a, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb2a, 0xb3e, 0xb15, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb21, 0x20, 0xb26, 0xb3f, 0xb2a,
0xb24, 0xb3f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb2a, 0xb3e, 0xb15, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb21, 0x20,
0xb26, 0xb3f, 0xb2a, 0xb24, 0xb3f, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb3f, 0xb1c, 0xb3f, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb2a, 0xb3f, 0xb1c, 0xb3f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb3f, 0xb1c, 0xb3f, 0x20, 0xb15,
0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb4d, 0xb30,
0xb47, 0xb1e, 0xb4d, 0xb1a, 0xb4d, 0x20, 0xb17, 0xb41, 0xb5f, 0xb3e, 0xb28, 0xb3e,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb4d, 0xb30, 0xb47, 0xb1e, 0xb4d, 0xb1a,
0x20, 0xb26, 0xb15, 0xb3f, 0xb23, 0x20, 0xb05, 0xb21, 0xb3c, 0xb47, 0x20, 0xb06,
0xb23, 0xb4d, 0xb1f, 0xb3e, 0xb30, 0xb1f, 0xb3f, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb17, 0xb32, 0xb3e, 0xb2a, 0xb17, 0xb38, 0x20, 0xb24, 0xb3f, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb17, 0xb2e, 0xb4d, 0xb2c, 0xb3f, 0xb5f, 0xb30, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb1c, 0xb30, 0xb4d, 0xb1c, 0xb3f, 0xb5f, 0xb3e, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb1c, 0xb30, 0xb4d, 0xb1c, 0xb3f, 0xb5f, 0xb3e, 0x20, 0xb2e,
0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb1c,
0xb30, 0xb4d, 0xb1c, 0xb3f, 0xb5f, 0xb3e, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb17, 0xb3f, 0xb32, 0xb4d, 0x200d, 0xb2c, 0xb47, 0xb30,
0xb4d, 0xb1f, 0x20, 0xb26, 0xb3f, 0xb2a, 0xb24, 0xb3f, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb17, 0xb4d, 0xb30, 0xb3f, 0xb28, 0xb71, 0xb3f, 0xb1a, 0x20, 0xb2e, 0xb3f,
0xb28, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0x20,
0xb39, 0xb2a, 0xb41, 0x20, 0xb17, 0xb4d, 0xb30, 0xb3f, 0xb28, 0xb32, 0xb3e, 0xb23,
0xb4d, 0xb21, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e,
0x20, 0xb39, 0xb2a, 0xb41, 0x20, 0xb17, 0xb4d, 0xb30, 0xb3f, 0xb28, 0xb32, 0xb3e,
0xb23, 0xb4d, 0xb21, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0x20, 0xb39, 0xb2a,
0xb41, 0x20, 0xb17, 0xb4d, 0xb30, 0xb3f, 0xb28, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb21,
0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71,
0xb47, 0xb21, 0xb3c, 0xb3e, 0x20, 0xb15, 0xb41, 0xb23, 0xb4d, 0x200d, 0xb2a, 0xb41,
0x20, 0xb17, 0xb4d, 0xb30, 0xb3f, 0xb28, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb21, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0x20, 0xb15, 0xb41,
0xb23, 0xb4d, 0x200d, 0xb2a, 0xb41, 0x20, 0xb17, 0xb4d, 0xb30, 0xb3f, 0xb28, 0xb32,
0xb3e, 0xb23, 0xb4d, 0xb21, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0x20, 0xb15,
0xb41, 0xb23, 0xb4d, 0x200d, 0xb2a, 0xb41, 0x20, 0xb17, 0xb4d, 0xb30, 0xb3f, 0xb28,
0xb32, 0xb3e, 0xb23, 0xb4d, 0xb21, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb17, 0xb32, 0xb4d, 0xb2a, 0x20, 0xb2e, 0xb3e, 0xb28,
0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb39, 0xb3e, 0xb71,
0xb3e, 0xb07, 0x2d, 0xb06, 0xb32, 0xb4d, 0xb5f, 0xb41, 0xb38, 0xb3e, 0xb28, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb39, 0xb3e, 0xb71, 0xb3e, 0xb07, 0x2d, 0xb06, 0xb32,
0xb4d, 0xb5f, 0xb41, 0xb38, 0xb3e, 0xb28, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19,
0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb39, 0xb3e, 0xb71, 0xb3e, 0xb07,
0x2d, 0xb06, 0xb32, 0xb4d, 0xb5f, 0xb41, 0xb38, 0xb3e, 0xb28, 0x20, 0xb21, 0xb47,
0x20, 0xb32, 0xb3e, 0xb07, 0xb1f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb39, 0xb19,
0xb4d, 0xb17, 0x20, 0xb15, 0xb19, 0xb4d, 0xb17, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb39, 0xb19, 0xb4d, 0xb17, 0x20, 0xb15, 0xb19, 0xb4d, 0xb17, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb39, 0xb19,
0xb4d, 0xb17, 0x20, 0xb15, 0xb19, 0xb4d, 0xb15, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e,
0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb39, 0xb71, 0xb21, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb39, 0xb71, 0xb21, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb39, 0xb71, 0xb21, 0x20, 0xb15, 0xb3e,
0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2c, 0xb3e, 0xb30, 0xb24,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb2c, 0xb3e, 0xb30, 0xb24, 0x20, 0xb15, 0xb3e, 0xb1c, 0xb3e, 0x20, 0xb38,
0xb3e, 0xb2e, 0xb41, 0xb26, 0xb4d, 0xb30, 0xb3f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb07, 0xb23, 0xb4d, 0xb21, 0xb1a, 0xb3f, 0xb28, 0xb3e, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb2e, 0xb3e, 0xb26, 0xb3f, 0xb28, 0xb3f, 0x20, 0xb07, 0xb23, 0xb4d, 0xb21,
0xb28, 0xb47, 0xb38, 0xb3f, 0xb06, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47,
0xb21, 0xb3c, 0xb3e, 0xb39, 0xb2a, 0xb41, 0x20, 0xb07, 0xb23, 0xb4d, 0xb21, 0xb28,
0xb47, 0xb38, 0xb3f, 0xb06, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21,
0xb3c, 0xb3e, 0xb15, 0xb41, 0xb23, 0xb4d, 0xb2a, 0xb41, 0x20, 0xb07, 0xb23, 0xb4d,
0xb21, 0xb28, 0xb47, 0xb38, 0xb3f, 0xb06, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb07,
0xb30, 0xb3e, 0xb28, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb30, 0xb3e, 0xb28,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb07, 0xb30, 0xb3e, 0xb28, 0x20, 0xb21, 0xb47, 0x20, 0xb32, 0xb3e, 0xb07,
0xb1f, 0xb4d, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb30, 0xb15, 0xb41, 0xb38,
0xb4d, 0xb24, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb30, 0xb15, 0xb41,
0xb38, 0xb4d, 0xb24, 0xb15, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb30, 0xb15, 0xb41, 0xb38, 0xb4d, 0xb24,
0xb15, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb07, 0xb1c, 0xb30, 0xb3e, 0xb07, 0xb32, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb07,
0xb1c, 0xb30, 0xb3e, 0xb07, 0xb32, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1c, 0xb30, 0xb3e, 0xb07, 0xb32,
0x20, 0xb21, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb1c, 0xb3e, 0xb2a, 0xb3e, 0xb28, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb1c, 0xb3e,
0xb2a, 0xb3e, 0xb28, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb1c, 0xb3e, 0xb2a, 0xb3e, 0xb28, 0x20, 0xb21, 0xb47,
0xb32, 0xb3e, 0xb07, 0xb1f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb15, 0xb1c, 0xb3e,
0xb15, 0xb3e, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0xb39, 0xb2a, 0xb41, 0x20, 0xb15, 0xb1c, 0xb3e,
0xb15, 0xb3e, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0xb15, 0xb41, 0xb23, 0xb4d, 0xb2a, 0xb41, 0x20,
0xb15, 0xb1c, 0xb3e, 0xb15, 0xb3e, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb15, 0xb30, 0xb3f, 0xb5f, 0xb3e, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb15, 0xb30, 0xb3f, 0xb5f, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19,
0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb15, 0xb30, 0xb3f, 0xb5f, 0xb3e,
0x20, 0xb21, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb15, 0xb38, 0xb30, 0xb3e, 0xb0f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb15, 0xb4d,
0xb30, 0xb3e, 0xb38, 0xb4d, 0xb28, 0xb5f, 0xb3e, 0xb30, 0xb4d, 0xb38, 0xb4d, 0xb15,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb15, 0xb4d, 0xb30, 0xb3e, 0xb38, 0xb4d, 0xb28,
0xb5f, 0xb3e, 0xb30, 0xb4d, 0xb38, 0xb4d, 0xb15, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e,
0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb15, 0xb4d, 0xb30, 0xb3e,
0xb38, 0xb4d, 0xb28, 0xb3e, 0xb30, 0xb4d, 0xb38, 0xb4d, 0xb15, 0x20, 0xb15, 0xb3e,
0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb15, 0xb3f, 0xb30, 0xb4d,
0xb17, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb32, 0xb3e, 0xb07, 0xb28, 0x20, 0xb26, 0xb3f, 0xb2a, 0xb24, 0xb3f, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb32, 0xb30, 0xb4d, 0xb21, 0x20, 0xb39, 0xb71, 0xb47, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb32, 0xb30, 0xb4d, 0xb21, 0x20, 0xb39, 0xb71, 0xb47,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb32, 0xb30, 0xb4d, 0xb21, 0x20, 0xb39, 0xb71, 0xb47, 0x20, 0xb21, 0xb47,
0xb32, 0xb3e, 0xb07, 0xb1f, 0xb4d, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e, 0xb3e,
0xb17, 0xb3e, 0xb26, 0xb3e, 0xb28, 0x20, 0xb26, 0xb3f, 0xb2a, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb2e, 0xb3e, 0xb17, 0xb3e, 0xb26, 0xb3e, 0xb28, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e, 0xb3e, 0xb17,
0xb3e, 0xb26, 0xb3e, 0xb28, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb2e, 0xb3e, 0xb32, 0xb47, 0xb38, 0xb3f, 0xb5f, 0xb3e, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e, 0xb3e, 0xb32, 0xb21, 0xb4d, 0xb71, 0xb3f, 0xb38,
0xb4d, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e, 0xb3e, 0xb30, 0xb4d, 0xb15, 0xb38,
0xb38, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e, 0xb3e, 0xb30, 0xb4d, 0xb38, 0xb3e,
0xb32, 0x20, 0xb26, 0xb3f, 0xb2a, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e, 0xb30,
0xb3f, 0xb38, 0xb38, 0xb4d, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e, 0xb30, 0xb3f,
0xb38, 0xb38, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e, 0xb30, 0xb3f, 0xb38, 0xb38, 0xb4d, 0x20, 0xb15,
0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e, 0xb3e, 0xb71,
0xb38, 0xb28, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e, 0xb47, 0xb15, 0xb4d, 0xb38,
0xb3f, 0xb15, 0xb24, 0xb3f, 0x20, 0xb2a, 0xb47, 0xb38, 0xb3f, 0xb2a, 0xb3f, 0xb15,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e, 0xb47, 0xb15, 0xb4d, 0xb38, 0xb3f, 0xb15,
0xb24, 0xb3f, 0x20, 0xb2a, 0xb47, 0xb38, 0xb3f, 0xb2a, 0xb3f, 0xb15, 0x20, 0xb2e,
0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e,
0xb47, 0xb15, 0xb4d, 0xb38, 0xb3f, 0xb15, 0xb24, 0xb3f, 0x20, 0xb2a, 0xb47, 0xb38,
0xb3f, 0xb2a, 0xb3f, 0xb15, 0x20, 0xb21, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb09, 0xb32, 0xb3e, 0xb28, 0x20, 0xb2c, 0xb1f, 0xb30,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb09, 0xb32, 0xb3e, 0xb28, 0x20, 0xb2c, 0xb1f,
0xb30, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb09, 0xb32, 0xb3e, 0xb28, 0x20, 0xb2c, 0xb1f, 0xb30, 0x20, 0xb15,
0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e, 0xb38, 0xb4d,
0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e, 0xb38, 0xb4d, 0xb15, 0x20, 0xb2e,
0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e,
0xb38, 0xb4d, 0xb15, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb2e, 0xb4d, 0xb5f, 0xb3e, 0xb01, 0xb2e, 0xb3e, 0xb30, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb28, 0xb09, 0xb30, 0xb41, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb28, 0xb47, 0xb2a, 0xb3e, 0xb32, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb28, 0xb4d,
0xb5f, 0xb41, 0x20, 0xb15, 0xb47, 0xb32, 0xb47, 0xb21, 0xb28, 0xb3f, 0xb5f, 0xb3e,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb28, 0xb4d, 0xb5f, 0xb41, 0x20, 0xb15, 0xb47,
0xb32, 0xb47, 0xb21, 0xb28, 0xb3f, 0xb5f, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e,
0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb28, 0xb4d, 0xb5f, 0xb41,
0x20, 0xb15, 0xb47, 0xb32, 0xb47, 0xb21, 0xb28, 0xb3f, 0xb5f, 0xb3e, 0x20, 0xb15,
0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb28, 0xb4d, 0xb5f,
0xb41, 0xb1c, 0xb3f, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb21, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb28, 0xb4d, 0xb5f, 0xb41, 0xb1c, 0xb3f, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb21,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb28, 0xb4d, 0xb5f, 0xb41, 0xb1c, 0xb3f, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb21,
0x20, 0xb21, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb28, 0xb4d, 0xb5f, 0xb41, 0xb2a, 0xb3e, 0xb09, 0xb23, 0xb4d, 0xb21, 0xb32, 0xb3e,
0xb23, 0xb4d, 0xb21, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb28, 0xb4d, 0xb5f, 0xb41,
0xb2a, 0xb3e, 0xb09, 0xb23, 0xb4d, 0xb21, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb21, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb28, 0xb4d, 0xb5f, 0xb41, 0xb2a, 0xb3e, 0xb09, 0xb23, 0xb4d, 0xb21, 0xb32, 0xb3e,
0xb23, 0xb4d, 0xb21, 0x20, 0xb21, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb28, 0xb3f, 0xb5f, 0xb41, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb28, 0xb30, 0xb2a, 0xb3e, 0xb01, 0xb15, 0x20, 0xb26, 0xb3f, 0xb2a, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb28, 0xb30, 0xb2a, 0xb3e, 0xb01, 0xb15, 0x20, 0xb26, 0xb3f,
0xb2a, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb28, 0xb30, 0xb2a, 0xb3e, 0xb01, 0xb15, 0x20, 0xb26, 0xb3f, 0xb2a,
0x20, 0xb21, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb2a, 0xb47, 0xb30, 0xb4d, 0xb28, 0xb3e, 0xb21, 0x20, 0xb21, 0xb47, 0x20, 0xb28,
0xb30, 0xb39, 0xb4d, 0xb28, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb30, 0xb4d,
0xb28, 0xb3e, 0xb21, 0x20, 0xb21, 0xb47, 0x20, 0xb28, 0xb30, 0xb39, 0xb4d, 0xb28,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0xb3e, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb2a, 0xb30, 0xb4d, 0xb28, 0xb3e, 0xb21, 0x20, 0xb21, 0xb47, 0x20,
0xb28, 0xb30, 0xb39, 0xb4d, 0xb28, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb28, 0xb71, 0xb38, 0xb3f, 0xb2c, 0xb3f, 0xb30, 0xb4d,
0xb38, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb28, 0xb71, 0xb38, 0xb3f,
0xb2c, 0xb3f, 0xb30, 0xb4d, 0xb38, 0xb4d, 0xb15, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e,
0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb28, 0xb71, 0xb38, 0xb3f,
0xb2c, 0xb30, 0xb4d, 0xb38, 0xb4d, 0xb15, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01,
0x20, 0xb2c, 0xb32, 0xb3e, 0xb05, 0xb2e, 0xb38, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb05, 0xb2e, 0xb38, 0xb4d, 0xb15, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e,
0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb05, 0xb2e, 0xb38, 0xb4d,
0xb15, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb2a, 0xb3e, 0xb15, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb2a, 0xb3e, 0xb15, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb2a, 0xb3e, 0xb15, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb15, 0xb3e,
0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb32, 0xb3e, 0xb09,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb3e, 0xb2a, 0xb41, 0xb5f, 0xb3e, 0x20,
0xb2a, 0xb41, 0xb28, 0xb3f, 0x20, 0xb17, 0xb3f, 0xb28, 0xb3f, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb2a, 0xb47, 0xb30, 0xb3e, 0xb17, 0xb4d, 0xb71, 0xb47, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb2a, 0xb47, 0xb30, 0xb3e, 0xb17, 0xb4d, 0xb71, 0xb47, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb2a, 0xb47, 0xb30, 0xb3e, 0xb17, 0xb4d, 0xb71, 0xb47, 0x20, 0xb15, 0xb3e, 0xb30,
0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb47, 0xb30, 0xb41, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb47, 0xb30, 0xb41, 0x20, 0xb2e, 0xb3e, 0xb28,
0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb47, 0xb30,
0xb41, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb2a, 0xb3f, 0xb32, 0xb3f, 0xb2a, 0xb3f, 0xb28, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb2a, 0xb3f, 0xb32, 0xb3f, 0xb2a, 0xb3f, 0xb28, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e,
0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb3f, 0xb32, 0xb3f,
0xb2a, 0xb3f, 0xb28, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb2a, 0xb3f, 0xb28, 0xb3f, 0xb15, 0xb4d, 0xb38, 0x20, 0xb26, 0xb3f,
0xb2a, 0xb24, 0xb3f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb38, 0xb47, 0xb23, 0xb4d,
0xb1f, 0x20, 0xb2a, 0xb3f, 0xb0f, 0xb30, 0xb47, 0x20, 0xb05, 0xb21, 0xb3c, 0xb47,
0x20, 0xb2e, 0xb3f, 0xb15, 0xb4d, 0xb71, 0xb47, 0xb32, 0xb3e, 0xb28, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb38, 0xb47, 0xb23, 0xb4d, 0xb1f, 0x20, 0xb2a, 0xb3f, 0xb0f,
0xb30, 0xb47, 0x20, 0xb05, 0xb21, 0xb3c, 0xb47, 0x20, 0xb2e, 0xb3f, 0xb15, 0xb4d,
0xb71, 0xb47, 0xb32, 0xb3e, 0xb28, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb38, 0xb47, 0xb23, 0xb4d, 0xb1f, 0x20,
0xb2a, 0xb3f, 0xb0f, 0xb30, 0xb47, 0x20, 0xb05, 0xb21, 0xb3c, 0xb47, 0x20, 0xb2e,
0xb3f, 0xb15, 0xb4d, 0xb71, 0xb47, 0xb32, 0xb3e, 0xb28, 0x20, 0xb21, 0xb47, 0xb32,
0xb3e, 0xb07, 0xb1f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb3f, 0xb1f, 0xb15,
0xb07, 0xb30, 0xb28, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb28, 0xb3e, 0xb2a,
0xb47, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb4d, 0xb5f, 0xb19, 0xb4d, 0xb17,
0xb5f, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb30, 0xb3f,
0xb5f, 0xb41, 0xb28, 0xb3f, 0xb5f, 0xb28, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb30,
0xb24, 0xb47, 0xb30, 0xb3e, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb38, 0xb15, 0xb3e,
0xb32, 0xb3f, 0xb28, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb38, 0xb15, 0xb3e, 0xb32,
0xb3f, 0xb28, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb38, 0xb15, 0xb3e, 0xb32, 0xb3f, 0xb28, 0x20, 0xb15, 0xb3e,
0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb38, 0xb3e, 0xb2e, 0xb06,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb38, 0xb3e, 0xb2e, 0xb06, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb38, 0xb3e,
0xb2e, 0xb06, 0x20, 0xb21, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb38, 0xb47, 0xb38, 0xb47, 0xb32, 0xb4d, 0xb38, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb38, 0xb3e, 0xb19, 0xb4d, 0xb17, 0xb3e, 0xb2a, 0xb41, 0xb30, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb38, 0xb32, 0xb2e, 0xb28, 0x20, 0xb26, 0xb3f, 0xb2a, 0xb24, 0xb3f, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb26, 0xb15, 0xb3f, 0xb23, 0x20, 0xb1c, 0xb30, 0xb4d, 0xb1c,
0xb3f, 0xb5f, 0xb3e, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb38, 0xb41, 0xb30, 0xb3f,
0xb28, 0xb3e, 0xb2e, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb38, 0xb71, 0xb3e, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb24, 0xb3e, 0xb39, 0xb3f, 0xb24, 0xb3f, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb24, 0xb3e, 0xb07, 0xb2a, 0xb47, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb24, 0xb3e, 0xb07, 0xb2a, 0xb47, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb19, 0xb4d,
0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb24, 0xb3e, 0xb07, 0xb2a, 0xb47, 0x20,
0xb21, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb24,
0xb3e, 0xb1c, 0xb3f, 0xb15, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb1f, 0xb15, 0xb47, 0xb32, 0xb3e, 0xb09, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb1f, 0xb19, 0xb4d, 0xb17, 0xb3e, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb1f, 0xb19, 0xb4d, 0xb17, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb1f, 0xb19, 0xb4d, 0xb17, 0xb3e, 0x20,
0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb1a, 0xb41,
0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb24, 0xb41, 0xb30, 0xb4d, 0xb15, 0xb2e,
0xb47, 0xb28, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb24, 0xb41, 0xb30, 0xb4d, 0xb15, 0xb2e, 0xb47, 0xb28, 0xb3f, 0xb38, 0xb4d,
0xb24, 0xb3e, 0xb28, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb24, 0xb41, 0xb30, 0xb4d, 0xb15, 0xb2e, 0xb47, 0xb28,
0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb24, 0xb41, 0xb71, 0xb3e, 0xb32, 0xb41, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb09, 0xb30, 0xb41, 0xb17, 0xb4d, 0xb71, 0xb47, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb09, 0xb30, 0xb41, 0xb17, 0xb4d, 0xb71, 0xb47, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb09, 0xb30, 0xb41, 0xb17, 0xb4d, 0xb71, 0xb47, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e,
0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb09, 0xb1c, 0xb4d, 0xb71, 0xb47, 0xb15,
0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb09,
0xb1c, 0xb4d, 0xb71, 0xb47, 0xb15, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb09, 0xb1c, 0xb4d, 0xb71, 0xb47, 0xb15, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28,
0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71,
0xb28, 0xb41, 0xb06, 0xb24, 0xb41, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb28,
0xb41, 0xb06, 0xb24, 0xb41, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb28, 0xb41, 0xb06, 0xb24, 0xb41, 0x20,
0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47,
0xb28, 0xb47, 0xb1c, 0xb41, 0xb0f, 0xb32, 0xb3e, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb71, 0xb4d, 0xb32, 0xb3e, 0xb26, 0xb3f, 0xb71, 0xb38, 0xb4d, 0xb24, 0xb15, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb4d, 0xb32, 0xb3e, 0xb26, 0xb3f, 0xb71, 0xb38,
0xb4d, 0xb24, 0xb15, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb4d, 0xb32, 0xb3e, 0xb26, 0xb3f, 0xb71, 0xb38,
0xb4d, 0xb24, 0xb15, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb71, 0xb3e, 0xb32, 0xb17, 0xb17, 0xb4d, 0xb30, 0xb3e, 0xb21, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb3e, 0xb32, 0xb17, 0xb17, 0xb4d, 0xb30, 0xb3e,
0xb21, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb71, 0xb3e, 0xb32, 0xb17, 0xb17, 0xb4d, 0xb30, 0xb3e, 0xb21, 0x20,
0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47,
0xb15, 0x20, 0xb26, 0xb3f, 0xb2a, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb3e,
0xb32, 0xb3f, 0xb38, 0x20, 0xb05, 0xb21, 0xb3c, 0xb47, 0x20, 0xb2a, 0xb41, 0xb1f,
0xb41, 0xb28, 0xb3e, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb5f, 0xb3e, 0xb15, 0xb41,
0xb24, 0xb4d, 0xb38, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb5f, 0xb3e, 0xb15,
0xb41, 0xb24, 0xb4d, 0xb38, 0xb15, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb5f, 0xb3e, 0xb15, 0xb41, 0xb24, 0xb4d,
0xb38, 0xb15, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb5f, 0xb47, 0xb15, 0xb3e, 0xb24, 0xb47, 0xb30, 0xb3f, 0xb28, 0xb2c, 0xb30,
0xb4d, 0xb17, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb5f, 0xb47, 0xb15, 0xb3e, 0xb24,
0xb47, 0xb30, 0xb3f, 0xb28, 0xb2c, 0xb30, 0xb4d, 0xb17, 0x20, 0xb2e, 0xb3e, 0xb28,
0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb5f, 0xb47, 0xb15,
0xb3e, 0xb24, 0xb47, 0xb30, 0xb3f, 0xb28, 0xb2c, 0xb30, 0xb4d, 0xb17, 0x20, 0xb15,
0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0x1420, 0x1551, 0x1401,
0x14c2, 0x1428, 0x20, 0x142f, 0x152d, 0x146f, 0x1426, 0x1456, 0x140f, 0x1423, 0x20, 0x1401,
0x1403, 0x1422, 0x1438, 0x14c2, 0x1420
};

// GENERATED PART ENDS HERE

} // QtTimeZoneLocale

#undef LOCALE_TAGS

QT_END_NAMESPACE

#endif // QTIMEZONELOCALE_DATA_P_H
